import "./monaco.css";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function _format(message, args) {
    var result;
    if (args.length === 0) {
        result = message;
    }
    else {
        result = message.replace(/\{(\d+)\}/g, function (match, rest) {
            var index = rest[0];
            return typeof args[index] !== 'undefined' ? args[index] : match;
        });
    }
    return result;
}
function localize(data, message) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    return _format(message, args);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var _typeof = {
    number: 'number',
    string: 'string',
    undefined: 'undefined',
    object: 'object',
    function: 'function'
};
/**
 * @returns whether the provided parameter is a JavaScript Array or not.
 */
function isArray(array) {
    if (Array.isArray) {
        return Array.isArray(array);
    }
    if (array && typeof (array.length) === _typeof.number && array.constructor === Array) {
        return true;
    }
    return false;
}
/**
 * @returns whether the provided parameter is a JavaScript String or not.
 */
function isString(str) {
    if (typeof (str) === _typeof.string || str instanceof String) {
        return true;
    }
    return false;
}
/**
 *
 * @returns whether the provided parameter is of type `object` but **not**
 *	`null`, an `array`, a `regexp`, nor a `date`.
 */
function isObject(obj) {
    // The method can't do a type cast since there are type (like strings) which
    // are subclasses of any put not positvely matched by the function. Hence type
    // narrowing results in wrong results.
    return typeof obj === _typeof.object
        && obj !== null
        && !Array.isArray(obj)
        && !(obj instanceof RegExp)
        && !(obj instanceof Date);
}
/**
 * In **contrast** to just checking `typeof` this will return `false` for `NaN`.
 * @returns whether the provided parameter is a JavaScript Number or not.
 */
function isNumber(obj) {
    if ((typeof (obj) === _typeof.number || obj instanceof Number) && !isNaN(obj)) {
        return true;
    }
    return false;
}
/**
 * @returns whether the provided parameter is a JavaScript Boolean or not.
 */
function isBoolean(obj) {
    return obj === true || obj === false;
}
/**
 * @returns whether the provided parameter is undefined.
 */
function isUndefined(obj) {
    return typeof (obj) === _typeof.undefined;
}
/**
 * @returns whether the provided parameter is undefined or null.
 */
function isUndefinedOrNull(obj) {
    return isUndefined(obj) || obj === null;
}
var hasOwnProperty = Object.prototype.hasOwnProperty;
/**
 * @returns whether the provided parameter is an empty JavaScript Object or not.
 */
function isEmptyObject(obj) {
    if (!isObject(obj)) {
        return false;
    }
    for (var key in obj) {
        if (hasOwnProperty.call(obj, key)) {
            return false;
        }
    }
    return true;
}
/**
 * @returns whether the provided parameter is a JavaScript Function or not.
 */
function isFunction(obj) {
    return typeof obj === _typeof.function;
}
function validateConstraints(args, constraints) {
    var len = Math.min(args.length, constraints.length);
    for (var i = 0; i < len; i++) {
        validateConstraint(args[i], constraints[i]);
    }
}
function validateConstraint(arg, constraint) {
    if (isString(constraint)) {
        if (typeof arg !== constraint) {
            throw new Error("argument does not match constraint: typeof " + constraint);
        }
    }
    else if (isFunction(constraint)) {
        if (arg instanceof constraint) {
            return;
        }
        if (!isUndefinedOrNull(arg) && arg.constructor === constraint) {
            return;
        }
        if (constraint.length === 1 && constraint.call(undefined, arg) === true) {
            return;
        }
        throw new Error("argument does not match one of these constraints: arg instanceof constraint, arg.constructor === constraint, nor constraint(arg) === true");
    }
}
/**
 * Creates a new object of the provided class and will call the constructor with
 * any additional argument supplied.
 */
function create(ctor) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    var obj = Object.create(ctor.prototype);
    ctor.apply(obj, args);
    return obj;
}

/**
 * Extracted from https://github.com/winjs/winjs
 * Version: 4.4.0(ec3258a9f3a36805a187848984e3bb938044178d)
 * Copyright (c) Microsoft Corporation.
 * All Rights Reserved.
 * Licensed under the MIT License.
 */
var __winjs_exports;

(function() {

var _modules = Object.create(null);//{};
_modules["WinJS/Core/_WinJS"] = {};

var _winjs = function(moduleId, deps, factory) {
    var exports = {};
    var exportsPassedIn = false;

    var depsValues = deps.map(function(dep) {
        if (dep === 'exports') {
            exportsPassedIn = true;
            return exports;
        }
        return _modules[dep];
    });

    var result = factory.apply({}, depsValues);

    _modules[moduleId] = exportsPassedIn ? exports : result;
};


_winjs("WinJS/Core/_Global", [], function () {

    // Appease jshint
    /* global window, self, global */

    var globalObject =
        typeof window !== 'undefined' ? window :
        typeof self !== 'undefined' ? self :
        typeof global !== 'undefined' ? global :
        {};
    return globalObject;
});

_winjs("WinJS/Core/_BaseCoreUtils", ["WinJS/Core/_Global"], function baseCoreUtilsInit(_Global) {

    var hasWinRT = !!_Global.Windows;

    function markSupportedForProcessing(func) {
        /// <signature helpKeyword="WinJS.Utilities.markSupportedForProcessing">
        /// <summary locid="WinJS.Utilities.markSupportedForProcessing">
        /// Marks a function as being compatible with declarative processing, such as WinJS.UI.processAll
        /// or WinJS.Binding.processAll.
        /// </summary>
        /// <param name="func" type="Function" locid="WinJS.Utilities.markSupportedForProcessing_p:func">
        /// The function to be marked as compatible with declarative processing.
        /// </param>
        /// <returns type="Function" locid="WinJS.Utilities.markSupportedForProcessing_returnValue">
        /// The input function.
        /// </returns>
        /// </signature>
        func.supportedForProcessing = true;
        return func;
    }

    var actualSetImmediate = null;

    return {
        hasWinRT: hasWinRT,
        markSupportedForProcessing: markSupportedForProcessing,
        _setImmediate: function (callback) {
            // BEGIN monaco change
            if (actualSetImmediate === null) {
                if (_Global.setImmediate) {
                    actualSetImmediate = _Global.setImmediate.bind(_Global);
                } else if (typeof process !== 'undefined' && typeof process.nextTick === 'function') {
                    actualSetImmediate = process.nextTick.bind(process);
                } else {
                    actualSetImmediate = _Global.setTimeout.bind(_Global);
                }
            }
            actualSetImmediate(callback);
            // END monaco change
        }
    };
});
_winjs("WinJS/Core/_WriteProfilerMark", ["WinJS/Core/_Global"], function profilerInit(_Global) {

    return _Global.msWriteProfilerMark || function () { };
});
_winjs("WinJS/Core/_Base", ["WinJS/Core/_WinJS","WinJS/Core/_Global","WinJS/Core/_BaseCoreUtils","WinJS/Core/_WriteProfilerMark"], function baseInit(_WinJS, _Global, _BaseCoreUtils, _WriteProfilerMark) {

    function initializeProperties(target, members, prefix) {
        var keys = Object.keys(members);
        var isArray = Array.isArray(target);
        var properties;
        var i, len;
        for (i = 0, len = keys.length; i < len; i++) {
            var key = keys[i];
            var enumerable = key.charCodeAt(0) !== /*_*/95;
            var member = members[key];
            if (member && typeof member === 'object') {
                if (member.value !== undefined || typeof member.get === 'function' || typeof member.set === 'function') {
                    if (member.enumerable === undefined) {
                        member.enumerable = enumerable;
                    }
                    if (prefix && member.setName && typeof member.setName === 'function') {
                        member.setName(prefix + "." + key);
                    }
                    properties = properties || {};
                    properties[key] = member;
                    continue;
                }
            }
            if (!enumerable) {
                properties = properties || {};
                properties[key] = { value: member, enumerable: enumerable, configurable: true, writable: true };
                continue;
            }
            if (isArray) {
                target.forEach(function (target) {
                    target[key] = member;
                });
            } else {
                target[key] = member;
            }
        }
        if (properties) {
            if (isArray) {
                target.forEach(function (target) {
                    Object.defineProperties(target, properties);
                });
            } else {
                Object.defineProperties(target, properties);
            }
        }
    }

    (function () {

        var _rootNamespace = _WinJS;
        if (!_rootNamespace.Namespace) {
            _rootNamespace.Namespace = Object.create(Object.prototype);
        }

        function createNamespace(parentNamespace, name) {
            var currentNamespace = parentNamespace || {};
            if (name) {
                var namespaceFragments = name.split(".");
                if (currentNamespace === _Global && namespaceFragments[0] === "WinJS") {
                    currentNamespace = _WinJS;
                    namespaceFragments.splice(0, 1);
                }
                for (var i = 0, len = namespaceFragments.length; i < len; i++) {
                    var namespaceName = namespaceFragments[i];
                    if (!currentNamespace[namespaceName]) {
                        Object.defineProperty(currentNamespace, namespaceName,
                            { value: {}, writable: false, enumerable: true, configurable: true }
                        );
                    }
                    currentNamespace = currentNamespace[namespaceName];
                }
            }
            return currentNamespace;
        }

        function defineWithParent(parentNamespace, name, members) {
            /// <signature helpKeyword="WinJS.Namespace.defineWithParent">
            /// <summary locid="WinJS.Namespace.defineWithParent">
            /// Defines a new namespace with the specified name under the specified parent namespace.
            /// </summary>
            /// <param name="parentNamespace" type="Object" locid="WinJS.Namespace.defineWithParent_p:parentNamespace">
            /// The parent namespace.
            /// </param>
            /// <param name="name" type="String" locid="WinJS.Namespace.defineWithParent_p:name">
            /// The name of the new namespace.
            /// </param>
            /// <param name="members" type="Object" locid="WinJS.Namespace.defineWithParent_p:members">
            /// The members of the new namespace.
            /// </param>
            /// <returns type="Object" locid="WinJS.Namespace.defineWithParent_returnValue">
            /// The newly-defined namespace.
            /// </returns>
            /// </signature>
            var currentNamespace = createNamespace(parentNamespace, name);

            if (members) {
                initializeProperties(currentNamespace, members, name || "<ANONYMOUS>");
            }

            return currentNamespace;
        }

        function define(name, members) {
            /// <signature helpKeyword="WinJS.Namespace.define">
            /// <summary locid="WinJS.Namespace.define">
            /// Defines a new namespace with the specified name.
            /// </summary>
            /// <param name="name" type="String" locid="WinJS.Namespace.define_p:name">
            /// The name of the namespace. This could be a dot-separated name for nested namespaces.
            /// </param>
            /// <param name="members" type="Object" locid="WinJS.Namespace.define_p:members">
            /// The members of the new namespace.
            /// </param>
            /// <returns type="Object" locid="WinJS.Namespace.define_returnValue">
            /// The newly-defined namespace.
            /// </returns>
            /// </signature>
            return defineWithParent(_Global, name, members);
        }

        var LazyStates = {
            uninitialized: 1,
            working: 2,
            initialized: 3,
        };

        function lazy(f) {
            var name;
            var state = LazyStates.uninitialized;
            var result;
            return {
                setName: function (value) {
                    name = value;
                },
                get: function () {
                    switch (state) {
                        case LazyStates.initialized:
                            return result;

                        case LazyStates.uninitialized:
                            state = LazyStates.working;
                            try {
                                _WriteProfilerMark("WinJS.Namespace._lazy:" + name + ",StartTM");
                                result = f();
                            } finally {
                                _WriteProfilerMark("WinJS.Namespace._lazy:" + name + ",StopTM");
                                state = LazyStates.uninitialized;
                            }
                            f = null;
                            state = LazyStates.initialized;
                            return result;

                        case LazyStates.working:
                            throw "Illegal: reentrancy on initialization";

                        default:
                            throw "Illegal";
                    }
                },
                set: function (value) {
                    switch (state) {
                        case LazyStates.working:
                            throw "Illegal: reentrancy on initialization";

                        default:
                            state = LazyStates.initialized;
                            result = value;
                            break;
                    }
                },
                enumerable: true,
                configurable: true,
            };
        }

        // helper for defining AMD module members
        function moduleDefine(exports, name, members) {
            var target = [exports];
            var publicNS = null;
            if (name) {
                publicNS = createNamespace(_Global, name);
                target.push(publicNS);
            }
            initializeProperties(target, members, name || "<ANONYMOUS>");
            return publicNS;
        }

        // Establish members of the "WinJS.Namespace" namespace
        Object.defineProperties(_rootNamespace.Namespace, {

            defineWithParent: { value: defineWithParent, writable: true, enumerable: true, configurable: true },

            define: { value: define, writable: true, enumerable: true, configurable: true },

            _lazy: { value: lazy, writable: true, enumerable: true, configurable: true },

            _moduleDefine: { value: moduleDefine, writable: true, enumerable: true, configurable: true }

        });

    })();

    (function () {

        function define(constructor, instanceMembers, staticMembers) {
            /// <signature helpKeyword="WinJS.Class.define">
            /// <summary locid="WinJS.Class.define">
            /// Defines a class using the given constructor and the specified instance members.
            /// </summary>
            /// <param name="constructor" type="Function" locid="WinJS.Class.define_p:constructor">
            /// A constructor function that is used to instantiate this class.
            /// </param>
            /// <param name="instanceMembers" type="Object" locid="WinJS.Class.define_p:instanceMembers">
            /// The set of instance fields, properties, and methods made available on the class.
            /// </param>
            /// <param name="staticMembers" type="Object" locid="WinJS.Class.define_p:staticMembers">
            /// The set of static fields, properties, and methods made available on the class.
            /// </param>
            /// <returns type="Function" locid="WinJS.Class.define_returnValue">
            /// The newly-defined class.
            /// </returns>
            /// </signature>
            constructor = constructor || function () { };
            _BaseCoreUtils.markSupportedForProcessing(constructor);
            if (instanceMembers) {
                initializeProperties(constructor.prototype, instanceMembers);
            }
            if (staticMembers) {
                initializeProperties(constructor, staticMembers);
            }
            return constructor;
        }

        function derive(baseClass, constructor, instanceMembers, staticMembers) {
            /// <signature helpKeyword="WinJS.Class.derive">
            /// <summary locid="WinJS.Class.derive">
            /// Creates a sub-class based on the supplied baseClass parameter, using prototypal inheritance.
            /// </summary>
            /// <param name="baseClass" type="Function" locid="WinJS.Class.derive_p:baseClass">
            /// The class to inherit from.
            /// </param>
            /// <param name="constructor" type="Function" locid="WinJS.Class.derive_p:constructor">
            /// A constructor function that is used to instantiate this class.
            /// </param>
            /// <param name="instanceMembers" type="Object" locid="WinJS.Class.derive_p:instanceMembers">
            /// The set of instance fields, properties, and methods to be made available on the class.
            /// </param>
            /// <param name="staticMembers" type="Object" locid="WinJS.Class.derive_p:staticMembers">
            /// The set of static fields, properties, and methods to be made available on the class.
            /// </param>
            /// <returns type="Function" locid="WinJS.Class.derive_returnValue">
            /// The newly-defined class.
            /// </returns>
            /// </signature>
            if (baseClass) {
                constructor = constructor || function () { };
                var basePrototype = baseClass.prototype;
                constructor.prototype = Object.create(basePrototype);
                _BaseCoreUtils.markSupportedForProcessing(constructor);
                Object.defineProperty(constructor.prototype, "constructor", { value: constructor, writable: true, configurable: true, enumerable: true });
                if (instanceMembers) {
                    initializeProperties(constructor.prototype, instanceMembers);
                }
                if (staticMembers) {
                    initializeProperties(constructor, staticMembers);
                }
                return constructor;
            } else {
                return define(constructor, instanceMembers, staticMembers);
            }
        }

        function mix(constructor) {
            /// <signature helpKeyword="WinJS.Class.mix">
            /// <summary locid="WinJS.Class.mix">
            /// Defines a class using the given constructor and the union of the set of instance members
            /// specified by all the mixin objects. The mixin parameter list is of variable length.
            /// </summary>
            /// <param name="constructor" locid="WinJS.Class.mix_p:constructor">
            /// A constructor function that is used to instantiate this class.
            /// </param>
            /// <returns type="Function" locid="WinJS.Class.mix_returnValue">
            /// The newly-defined class.
            /// </returns>
            /// </signature>
            constructor = constructor || function () { };
            var i, len;
            for (i = 1, len = arguments.length; i < len; i++) {
                initializeProperties(constructor.prototype, arguments[i]);
            }
            return constructor;
        }

        // Establish members of "WinJS.Class" namespace
        _WinJS.Namespace.define("WinJS.Class", {
            define: define,
            derive: derive,
            mix: mix
        });

    })();

    return {
        Namespace: _WinJS.Namespace,
        Class: _WinJS.Class
    };

});
_winjs("WinJS/Core/_ErrorFromName", ["WinJS/Core/_Base"], function errorsInit(_Base) {

    var ErrorFromName = _Base.Class.derive(Error, function (name, message) {
        /// <signature helpKeyword="WinJS.ErrorFromName">
        /// <summary locid="WinJS.ErrorFromName">
        /// Creates an Error object with the specified name and message properties.
        /// </summary>
        /// <param name="name" type="String" locid="WinJS.ErrorFromName_p:name">The name of this error. The name is meant to be consumed programmatically and should not be localized.</param>
        /// <param name="message" type="String" optional="true" locid="WinJS.ErrorFromName_p:message">The message for this error. The message is meant to be consumed by humans and should be localized.</param>
        /// <returns type="Error" locid="WinJS.ErrorFromName_returnValue">Error instance with .name and .message properties populated</returns>
        /// </signature>
        this.name = name;
        this.message = message || name;
    }, {
        /* empty */
    }, {
        supportedForProcessing: false,
    });

    _Base.Namespace.define("WinJS", {
        // ErrorFromName establishes a simple pattern for returning error codes.
        //
        ErrorFromName: ErrorFromName
    });

    return ErrorFromName;

});


_winjs("WinJS/Core/_Events", ["exports","WinJS/Core/_Base"], function eventsInit(exports, _Base) {


    function createEventProperty(name) {
        var eventPropStateName = "_on" + name + "state";

        return {
            get: function () {
                var state = this[eventPropStateName];
                return state && state.userHandler;
            },
            set: function (handler) {
                var state = this[eventPropStateName];
                if (handler) {
                    if (!state) {
                        state = { wrapper: function (evt) { return state.userHandler(evt); }, userHandler: handler };
                        Object.defineProperty(this, eventPropStateName, { value: state, enumerable: false, writable:true, configurable: true });
                        this.addEventListener(name, state.wrapper, false);
                    }
                    state.userHandler = handler;
                } else if (state) {
                    this.removeEventListener(name, state.wrapper, false);
                    this[eventPropStateName] = null;
                }
            },
            enumerable: true
        };
    }

    function createEventProperties() {
        /// <signature helpKeyword="WinJS.Utilities.createEventProperties">
        /// <summary locid="WinJS.Utilities.createEventProperties">
        /// Creates an object that has one property for each name passed to the function.
        /// </summary>
        /// <param name="events" locid="WinJS.Utilities.createEventProperties_p:events">
        /// A variable list of property names.
        /// </param>
        /// <returns type="Object" locid="WinJS.Utilities.createEventProperties_returnValue">
        /// The object with the specified properties. The names of the properties are prefixed with 'on'.
        /// </returns>
        /// </signature>
        var props = {};
        for (var i = 0, len = arguments.length; i < len; i++) {
            var name = arguments[i];
            props["on" + name] = createEventProperty(name);
        }
        return props;
    }

    var EventMixinEvent = _Base.Class.define(
        function EventMixinEvent_ctor(type, detail, target) {
            this.detail = detail;
            this.target = target;
            this.timeStamp = Date.now();
            this.type = type;
        },
        {
            bubbles: { value: false, writable: false },
            cancelable: { value: false, writable: false },
            currentTarget: {
                get: function () { return this.target; }
            },
            defaultPrevented: {
                get: function () { return this._preventDefaultCalled; }
            },
            trusted: { value: false, writable: false },
            eventPhase: { value: 0, writable: false },
            target: null,
            timeStamp: null,
            type: null,

            preventDefault: function () {
                this._preventDefaultCalled = true;
            },
            stopImmediatePropagation: function () {
                this._stopImmediatePropagationCalled = true;
            },
            stopPropagation: function () {
            }
        }, {
            supportedForProcessing: false,
        }
    );

    var eventMixin = {
        _listeners: null,

        addEventListener: function (type, listener, useCapture) {
            /// <signature helpKeyword="WinJS.Utilities.eventMixin.addEventListener">
            /// <summary locid="WinJS.Utilities.eventMixin.addEventListener">
            /// Adds an event listener to the control.
            /// </summary>
            /// <param name="type" locid="WinJS.Utilities.eventMixin.addEventListener_p:type">
            /// The type (name) of the event.
            /// </param>
            /// <param name="listener" locid="WinJS.Utilities.eventMixin.addEventListener_p:listener">
            /// The listener to invoke when the event is raised.
            /// </param>
            /// <param name="useCapture" locid="WinJS.Utilities.eventMixin.addEventListener_p:useCapture">
            /// if true initiates capture, otherwise false.
            /// </param>
            /// </signature>
            useCapture = useCapture || false;
            this._listeners = this._listeners || {};
            var eventListeners = (this._listeners[type] = this._listeners[type] || []);
            for (var i = 0, len = eventListeners.length; i < len; i++) {
                var l = eventListeners[i];
                if (l.useCapture === useCapture && l.listener === listener) {
                    return;
                }
            }
            eventListeners.push({ listener: listener, useCapture: useCapture });
        },
        dispatchEvent: function (type, details) {
            /// <signature helpKeyword="WinJS.Utilities.eventMixin.dispatchEvent">
            /// <summary locid="WinJS.Utilities.eventMixin.dispatchEvent">
            /// Raises an event of the specified type and with the specified additional properties.
            /// </summary>
            /// <param name="type" locid="WinJS.Utilities.eventMixin.dispatchEvent_p:type">
            /// The type (name) of the event.
            /// </param>
            /// <param name="details" locid="WinJS.Utilities.eventMixin.dispatchEvent_p:details">
            /// The set of additional properties to be attached to the event object when the event is raised.
            /// </param>
            /// <returns type="Boolean" locid="WinJS.Utilities.eventMixin.dispatchEvent_returnValue">
            /// true if preventDefault was called on the event.
            /// </returns>
            /// </signature>
            var listeners = this._listeners && this._listeners[type];
            if (listeners) {
                var eventValue = new EventMixinEvent(type, details, this);
                // Need to copy the array to protect against people unregistering while we are dispatching
                listeners = listeners.slice(0, listeners.length);
                for (var i = 0, len = listeners.length; i < len && !eventValue._stopImmediatePropagationCalled; i++) {
                    listeners[i].listener(eventValue);
                }
                return eventValue.defaultPrevented || false;
            }
            return false;
        },
        removeEventListener: function (type, listener, useCapture) {
            /// <signature helpKeyword="WinJS.Utilities.eventMixin.removeEventListener">
            /// <summary locid="WinJS.Utilities.eventMixin.removeEventListener">
            /// Removes an event listener from the control.
            /// </summary>
            /// <param name="type" locid="WinJS.Utilities.eventMixin.removeEventListener_p:type">
            /// The type (name) of the event.
            /// </param>
            /// <param name="listener" locid="WinJS.Utilities.eventMixin.removeEventListener_p:listener">
            /// The listener to remove.
            /// </param>
            /// <param name="useCapture" locid="WinJS.Utilities.eventMixin.removeEventListener_p:useCapture">
            /// Specifies whether to initiate capture.
            /// </param>
            /// </signature>
            useCapture = useCapture || false;
            var listeners = this._listeners && this._listeners[type];
            if (listeners) {
                for (var i = 0, len = listeners.length; i < len; i++) {
                    var l = listeners[i];
                    if (l.listener === listener && l.useCapture === useCapture) {
                        listeners.splice(i, 1);
                        if (listeners.length === 0) {
                            delete this._listeners[type];
                        }
                        // Only want to remove one element for each call to removeEventListener
                        break;
                    }
                }
            }
        }
    };

    _Base.Namespace._moduleDefine(exports, "WinJS.Utilities", {
        _createEventProperty: createEventProperty,
        createEventProperties: createEventProperties,
        eventMixin: eventMixin
    });

});


_winjs("WinJS/Core/_Trace", ["WinJS/Core/_Global"], function traceInit(_Global) {

    function nop(v) {
        return v;
    }

    return {
        _traceAsyncOperationStarting: (_Global.Debug && _Global.Debug.msTraceAsyncOperationStarting && _Global.Debug.msTraceAsyncOperationStarting.bind(_Global.Debug)) || nop,
        _traceAsyncOperationCompleted: (_Global.Debug && _Global.Debug.msTraceAsyncOperationCompleted && _Global.Debug.msTraceAsyncOperationCompleted.bind(_Global.Debug)) || nop,
        _traceAsyncCallbackStarting: (_Global.Debug && _Global.Debug.msTraceAsyncCallbackStarting && _Global.Debug.msTraceAsyncCallbackStarting.bind(_Global.Debug)) || nop,
        _traceAsyncCallbackCompleted: (_Global.Debug && _Global.Debug.msTraceAsyncCallbackCompleted && _Global.Debug.msTraceAsyncCallbackCompleted.bind(_Global.Debug)) || nop
    };
});
_winjs("WinJS/Promise/_StateMachine", ["WinJS/Core/_Global","WinJS/Core/_BaseCoreUtils","WinJS/Core/_Base","WinJS/Core/_ErrorFromName","WinJS/Core/_Events","WinJS/Core/_Trace"], function promiseStateMachineInit(_Global, _BaseCoreUtils, _Base, _ErrorFromName, _Events, _Trace) {

    _Global.Debug && (_Global.Debug.setNonUserCodeExceptions = true);

    var ListenerType = _Base.Class.mix(_Base.Class.define(null, { /*empty*/ }, { supportedForProcessing: false }), _Events.eventMixin);
    var promiseEventListeners = new ListenerType();
    // make sure there is a listeners collection so that we can do a more trivial check below
    promiseEventListeners._listeners = {};
    var errorET = "error";
    var canceledName = "Canceled";
    var tagWithStack = false;
    var tag = {
        promise: 0x01,
        thenPromise: 0x02,
        errorPromise: 0x04,
        exceptionPromise: 0x08,
        completePromise: 0x10,
    };
    tag.all = tag.promise | tag.thenPromise | tag.errorPromise | tag.exceptionPromise | tag.completePromise;

    //
    // Global error counter, for each error which enters the system we increment this once and then
    // the error number travels with the error as it traverses the tree of potential handlers.
    //
    // When someone has registered to be told about errors (WinJS.Promise.callonerror) promises
    // which are in error will get tagged with a ._errorId field. This tagged field is the
    // contract by which nested promises with errors will be identified as chaining for the
    // purposes of the callonerror semantics. If a nested promise in error is encountered without
    // a ._errorId it will be assumed to be foreign and treated as an interop boundary and
    // a new error id will be minted.
    //
    var error_number = 1;

    //
    // The state machine has a interesting hiccup in it with regards to notification, in order
    // to flatten out notification and avoid recursion for synchronous completion we have an
    // explicit set of *_notify states which are responsible for notifying their entire tree
    // of children. They can do this because they know that immediate children are always
    // ThenPromise instances and we can therefore reach into their state to access the
    // _listeners collection.
    //
    // So, what happens is that a Promise will be fulfilled through the _completed or _error
    // messages at which point it will enter a *_notify state and be responsible for to move
    // its children into an (as appropriate) success or error state and also notify that child's
    // listeners of the state transition, until leaf notes are reached.
    //

    var state_created,              // -> working
        state_working,              // -> error | error_notify | success | success_notify | canceled | waiting
        state_waiting,              // -> error | error_notify | success | success_notify | waiting_canceled
        state_waiting_canceled,     // -> error | error_notify | success | success_notify | canceling
        state_canceled,             // -> error | error_notify | success | success_notify | canceling
        state_canceling,            // -> error_notify
        state_success_notify,       // -> success
        state_success,              // -> .
        state_error_notify,         // -> error
        state_error;                // -> .

    // Noop function, used in the various states to indicate that they don't support a given
    // message. Named with the somewhat cute name '_' because it reads really well in the states.

    function _() { }

    // Initial state
    //
    state_created = {
        name: "created",
        enter: function (promise) {
            promise._setState(state_working);
        },
        cancel: _,
        done: _,
        then: _,
        _completed: _,
        _error: _,
        _notify: _,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Ready state, waiting for a message (completed/error/progress), able to be canceled
    //
    state_working = {
        name: "working",
        enter: _,
        cancel: function (promise) {
            promise._setState(state_canceled);
        },
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Waiting state, if a promise is completed with a value which is itself a promise
    // (has a then() method) it signs up to be informed when that child promise is
    // fulfilled at which point it will be fulfilled with that value.
    //
    state_waiting = {
        name: "waiting",
        enter: function (promise) {
            var waitedUpon = promise._value;
            // We can special case our own intermediate promises which are not in a
            //  terminal state by just pushing this promise as a listener without
            //  having to create new indirection functions
            if (waitedUpon instanceof ThenPromise &&
                waitedUpon._state !== state_error &&
                waitedUpon._state !== state_success) {
                pushListener(waitedUpon, { promise: promise });
            } else {
                var error = function (value) {
                    if (waitedUpon._errorId) {
                        promise._chainedError(value, waitedUpon);
                    } else {
                        // Because this is an interop boundary we want to indicate that this
                        //  error has been handled by the promise infrastructure before we
                        //  begin a new handling chain.
                        //
                        callonerror(promise, value, detailsForHandledError, waitedUpon, error);
                        promise._error(value);
                    }
                };
                error.handlesOnError = true;
                waitedUpon.then(
                    promise._completed.bind(promise),
                    error,
                    promise._progress.bind(promise)
                );
            }
        },
        cancel: function (promise) {
            promise._setState(state_waiting_canceled);
        },
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Waiting canceled state, when a promise has been in a waiting state and receives a
    // request to cancel its pending work it will forward that request to the child promise
    // and then waits to be informed of the result. This promise moves itself into the
    // canceling state but understands that the child promise may instead push it to a
    // different state.
    //
    state_waiting_canceled = {
        name: "waiting_canceled",
        enter: function (promise) {
            // Initiate a transition to canceling. Triggering a cancel on the promise
            // that we are waiting upon may result in a different state transition
            // before the state machine pump runs again.
            promise._setState(state_canceling);
            var waitedUpon = promise._value;
            if (waitedUpon.cancel) {
                waitedUpon.cancel();
            }
        },
        cancel: _,
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Canceled state, moves to the canceling state and then tells the promise to do
    // whatever it might need to do on cancelation.
    //
    state_canceled = {
        name: "canceled",
        enter: function (promise) {
            // Initiate a transition to canceling. The _cancelAction may change the state
            // before the state machine pump runs again.
            promise._setState(state_canceling);
            promise._cancelAction();
        },
        cancel: _,
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Canceling state, commits to the promise moving to an error state with an error
    // object whose 'name' and 'message' properties contain the string "Canceled"
    //
    state_canceling = {
        name: "canceling",
        enter: function (promise) {
            var error = new Error(canceledName);
            error.name = error.message;
            promise._value = error;
            promise._setState(state_error_notify);
        },
        cancel: _,
        done: _,
        then: _,
        _completed: _,
        _error: _,
        _notify: _,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Success notify state, moves a promise to the success state and notifies all children
    //
    state_success_notify = {
        name: "complete_notify",
        enter: function (promise) {
            promise.done = CompletePromise.prototype.done;
            promise.then = CompletePromise.prototype.then;
            if (promise._listeners) {
                var queue = [promise];
                var p;
                while (queue.length) {
                    p = queue.shift();
                    p._state._notify(p, queue);
                }
            }
            promise._setState(state_success);
        },
        cancel: _,
        done: null, /*error to get here */
        then: null, /*error to get here */
        _completed: _,
        _error: _,
        _notify: notifySuccess,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Success state, moves a promise to the success state and does NOT notify any children.
    // Some upstream promise is owning the notification pass.
    //
    state_success = {
        name: "success",
        enter: function (promise) {
            promise.done = CompletePromise.prototype.done;
            promise.then = CompletePromise.prototype.then;
            promise._cleanupAction();
        },
        cancel: _,
        done: null, /*error to get here */
        then: null, /*error to get here */
        _completed: _,
        _error: _,
        _notify: notifySuccess,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Error notify state, moves a promise to the error state and notifies all children
    //
    state_error_notify = {
        name: "error_notify",
        enter: function (promise) {
            promise.done = ErrorPromise.prototype.done;
            promise.then = ErrorPromise.prototype.then;
            if (promise._listeners) {
                var queue = [promise];
                var p;
                while (queue.length) {
                    p = queue.shift();
                    p._state._notify(p, queue);
                }
            }
            promise._setState(state_error);
        },
        cancel: _,
        done: null, /*error to get here*/
        then: null, /*error to get here*/
        _completed: _,
        _error: _,
        _notify: notifyError,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Error state, moves a promise to the error state and does NOT notify any children.
    // Some upstream promise is owning the notification pass.
    //
    state_error = {
        name: "error",
        enter: function (promise) {
            promise.done = ErrorPromise.prototype.done;
            promise.then = ErrorPromise.prototype.then;
            promise._cleanupAction();
        },
        cancel: _,
        done: null, /*error to get here*/
        then: null, /*error to get here*/
        _completed: _,
        _error: _,
        _notify: notifyError,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    //
    // The statemachine implementation follows a very particular pattern, the states are specified
    // as static stateless bags of functions which are then indirected through the state machine
    // instance (a Promise). As such all of the functions on each state have the promise instance
    // passed to them explicitly as a parameter and the Promise instance members do a little
    // dance where they indirect through the state and insert themselves in the argument list.
    //
    // We could instead call directly through the promise states however then every caller
    // would have to remember to do things like pumping the state machine to catch state transitions.
    //

    var PromiseStateMachine = _Base.Class.define(null, {
        _listeners: null,
        _nextState: null,
        _state: null,
        _value: null,

        cancel: function () {
            /// <signature helpKeyword="WinJS.PromiseStateMachine.cancel">
            /// <summary locid="WinJS.PromiseStateMachine.cancel">
            /// Attempts to cancel the fulfillment of a promised value. If the promise hasn't
            /// already been fulfilled and cancellation is supported, the promise enters
            /// the error state with a value of Error("Canceled").
            /// </summary>
            /// </signature>
            this._state.cancel(this);
            this._run();
        },
        done: function Promise_done(onComplete, onError, onProgress) {
            /// <signature helpKeyword="WinJS.PromiseStateMachine.done">
            /// <summary locid="WinJS.PromiseStateMachine.done">
            /// Allows you to specify the work to be done on the fulfillment of the promised value,
            /// the error handling to be performed if the promise fails to fulfill
            /// a value, and the handling of progress notifications along the way.
            ///
            /// After the handlers have finished executing, this function throws any error that would have been returned
            /// from then() as a promise in the error state.
            /// </summary>
            /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.done_p:onComplete">
            /// The function to be called if the promise is fulfilled successfully with a value.
            /// The fulfilled value is passed as the single argument. If the value is null,
            /// the fulfilled value is returned. The value returned
            /// from the function becomes the fulfilled value of the promise returned by
            /// then(). If an exception is thrown while executing the function, the promise returned
            /// by then() moves into the error state.
            /// </param>
            /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onError">
            /// The function to be called if the promise is fulfilled with an error. The error
            /// is passed as the single argument. If it is null, the error is forwarded.
            /// The value returned from the function is the fulfilled value of the promise returned by then().
            /// </param>
            /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onProgress">
            /// the function to be called if the promise reports progress. Data about the progress
            /// is passed as the single argument. Promises are not required to support
            /// progress.
            /// </param>
            /// </signature>
            this._state.done(this, onComplete, onError, onProgress);
        },
        then: function Promise_then(onComplete, onError, onProgress) {
            /// <signature helpKeyword="WinJS.PromiseStateMachine.then">
            /// <summary locid="WinJS.PromiseStateMachine.then">
            /// Allows you to specify the work to be done on the fulfillment of the promised value,
            /// the error handling to be performed if the promise fails to fulfill
            /// a value, and the handling of progress notifications along the way.
            /// </summary>
            /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.then_p:onComplete">
            /// The function to be called if the promise is fulfilled successfully with a value.
            /// The value is passed as the single argument. If the value is null, the value is returned.
            /// The value returned from the function becomes the fulfilled value of the promise returned by
            /// then(). If an exception is thrown while this function is being executed, the promise returned
            /// by then() moves into the error state.
            /// </param>
            /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onError">
            /// The function to be called if the promise is fulfilled with an error. The error
            /// is passed as the single argument. If it is null, the error is forwarded.
            /// The value returned from the function becomes the fulfilled value of the promise returned by then().
            /// </param>
            /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onProgress">
            /// The function to be called if the promise reports progress. Data about the progress
            /// is passed as the single argument. Promises are not required to support
            /// progress.
            /// </param>
            /// <returns type="WinJS.Promise" locid="WinJS.PromiseStateMachine.then_returnValue">
            /// The promise whose value is the result of executing the complete or
            /// error function.
            /// </returns>
            /// </signature>
            // BEGIN monaco change
            if (this.then !== Promise_then) {
                this.then(onComplete, onError, onProgress);
                return;
            }
            // END monaco change
            return this._state.then(this, onComplete, onError, onProgress);
        },

        _chainedError: function (value, context) {
            var result = this._state._error(this, value, detailsForChainedError, context);
            this._run();
            return result;
        },
        _completed: function (value) {
            var result = this._state._completed(this, value);
            this._run();
            return result;
        },
        _error: function (value) {
            var result = this._state._error(this, value, detailsForError);
            this._run();
            return result;
        },
        _progress: function (value) {
            this._state._progress(this, value);
        },
        _setState: function (state) {
            this._nextState = state;
        },
        _setCompleteValue: function (value) {
            this._state._setCompleteValue(this, value);
            this._run();
        },
        _setChainedErrorValue: function (value, context) {
            var result = this._state._setErrorValue(this, value, detailsForChainedError, context);
            this._run();
            return result;
        },
        _setExceptionValue: function (value) {
            var result = this._state._setErrorValue(this, value, detailsForException);
            this._run();
            return result;
        },
        _run: function () {
            while (this._nextState) {
                this._state = this._nextState;
                this._nextState = null;
                this._state.enter(this);
            }
        }
    }, {
        supportedForProcessing: false
    });

    //
    // Implementations of shared state machine code.
    //

    function completed(promise, value) {
        var targetState;
        if (value && typeof value === "object" && typeof value.then === "function") {
            targetState = state_waiting;
        } else {
            targetState = state_success_notify;
        }
        promise._value = value;
        promise._setState(targetState);
    }
    function createErrorDetails(exception, error, promise, id, parent, handler) {
        return {
            exception: exception,
            error: error,
            promise: promise,
            handler: handler,
            id: id,
            parent: parent
        };
    }
    function detailsForHandledError(promise, errorValue, context, handler) {
        var exception = context._isException;
        var errorId = context._errorId;
        return createErrorDetails(
            exception ? errorValue : null,
            exception ? null : errorValue,
            promise,
            errorId,
            context,
            handler
        );
    }
    function detailsForChainedError(promise, errorValue, context) {
        var exception = context._isException;
        var errorId = context._errorId;
        setErrorInfo(promise, errorId, exception);
        return createErrorDetails(
            exception ? errorValue : null,
            exception ? null : errorValue,
            promise,
            errorId,
            context
        );
    }
    function detailsForError(promise, errorValue) {
        var errorId = ++error_number;
        setErrorInfo(promise, errorId);
        return createErrorDetails(
            null,
            errorValue,
            promise,
            errorId
        );
    }
    function detailsForException(promise, exceptionValue) {
        var errorId = ++error_number;
        setErrorInfo(promise, errorId, true);
        return createErrorDetails(
            exceptionValue,
            null,
            promise,
            errorId
        );
    }
    function done(promise, onComplete, onError, onProgress) {
        var asyncOpID = _Trace._traceAsyncOperationStarting("WinJS.Promise.done");
        pushListener(promise, { c: onComplete, e: onError, p: onProgress, asyncOpID: asyncOpID });
    }
    function error(promise, value, onerrorDetails, context) {
        promise._value = value;
        callonerror(promise, value, onerrorDetails, context);
        promise._setState(state_error_notify);
    }
    function notifySuccess(promise, queue) {
        var value = promise._value;
        var listeners = promise._listeners;
        if (!listeners) {
            return;
        }
        promise._listeners = null;
        var i, len;
        for (i = 0, len = Array.isArray(listeners) ? listeners.length : 1; i < len; i++) {
            var listener = len === 1 ? listeners : listeners[i];
            var onComplete = listener.c;
            var target = listener.promise;

            _Trace._traceAsyncOperationCompleted(listener.asyncOpID, _Global.Debug && _Global.Debug.MS_ASYNC_OP_STATUS_SUCCESS);

            if (target) {
                _Trace._traceAsyncCallbackStarting(listener.asyncOpID);
                try {
                    target._setCompleteValue(onComplete ? onComplete(value) : value);
                } catch (ex) {
                    target._setExceptionValue(ex);
                } finally {
                    _Trace._traceAsyncCallbackCompleted();
                }
                if (target._state !== state_waiting && target._listeners) {
                    queue.push(target);
                }
            } else {
                CompletePromise.prototype.done.call(promise, onComplete);
            }
        }
    }
    function notifyError(promise, queue) {
        var value = promise._value;
        var listeners = promise._listeners;
        if (!listeners) {
            return;
        }
        promise._listeners = null;
        var i, len;
        for (i = 0, len = Array.isArray(listeners) ? listeners.length : 1; i < len; i++) {
            var listener = len === 1 ? listeners : listeners[i];
            var onError = listener.e;
            var target = listener.promise;

            var errorID = _Global.Debug && (value && value.name === canceledName ? _Global.Debug.MS_ASYNC_OP_STATUS_CANCELED : _Global.Debug.MS_ASYNC_OP_STATUS_ERROR);
            _Trace._traceAsyncOperationCompleted(listener.asyncOpID, errorID);

            if (target) {
                var asyncCallbackStarted = false;
                try {
                    if (onError) {
                        _Trace._traceAsyncCallbackStarting(listener.asyncOpID);
                        asyncCallbackStarted = true;
                        if (!onError.handlesOnError) {
                            callonerror(target, value, detailsForHandledError, promise, onError);
                        }
                        target._setCompleteValue(onError(value));
                    } else {
                        target._setChainedErrorValue(value, promise);
                    }
                } catch (ex) {
                    target._setExceptionValue(ex);
                } finally {
                    if (asyncCallbackStarted) {
                        _Trace._traceAsyncCallbackCompleted();
                    }
                }
                if (target._state !== state_waiting && target._listeners) {
                    queue.push(target);
                }
            } else {
                ErrorPromise.prototype.done.call(promise, null, onError);
            }
        }
    }
    function callonerror(promise, value, onerrorDetailsGenerator, context, handler) {
        if (promiseEventListeners._listeners[errorET]) {
            if (value instanceof Error && value.message === canceledName) {
                return;
            }
            promiseEventListeners.dispatchEvent(errorET, onerrorDetailsGenerator(promise, value, context, handler));
        }
    }
    function progress(promise, value) {
        var listeners = promise._listeners;
        if (listeners) {
            var i, len;
            for (i = 0, len = Array.isArray(listeners) ? listeners.length : 1; i < len; i++) {
                var listener = len === 1 ? listeners : listeners[i];
                var onProgress = listener.p;
                if (onProgress) {
                    try { onProgress(value); } catch (ex) { }
                }
                if (!(listener.c || listener.e) && listener.promise) {
                    listener.promise._progress(value);
                }
            }
        }
    }
    function pushListener(promise, listener) {
        var listeners = promise._listeners;
        if (listeners) {
            // We may have either a single listener (which will never be wrapped in an array)
            // or 2+ listeners (which will be wrapped). Since we are now adding one more listener
            // we may have to wrap the single listener before adding the second.
            listeners = Array.isArray(listeners) ? listeners : [listeners];
            listeners.push(listener);
        } else {
            listeners = listener;
        }
        promise._listeners = listeners;
    }
    // The difference beween setCompleteValue()/setErrorValue() and complete()/error() is that setXXXValue() moves
    // a promise directly to the success/error state without starting another notification pass (because one
    // is already ongoing).
    function setErrorInfo(promise, errorId, isException) {
        promise._isException = isException || false;
        promise._errorId = errorId;
    }
    function setErrorValue(promise, value, onerrorDetails, context) {
        promise._value = value;
        callonerror(promise, value, onerrorDetails, context);
        promise._setState(state_error);
    }
    function setCompleteValue(promise, value) {
        var targetState;
        if (value && typeof value === "object" && typeof value.then === "function") {
            targetState = state_waiting;
        } else {
            targetState = state_success;
        }
        promise._value = value;
        promise._setState(targetState);
    }
    function then(promise, onComplete, onError, onProgress) {
        var result = new ThenPromise(promise);
        var asyncOpID = _Trace._traceAsyncOperationStarting("WinJS.Promise.then");
        pushListener(promise, { promise: result, c: onComplete, e: onError, p: onProgress, asyncOpID: asyncOpID });
        return result;
    }

    //
    // Internal implementation detail promise, ThenPromise is created when a promise needs
    // to be returned from a then() method.
    //
    var ThenPromise = _Base.Class.derive(PromiseStateMachine,
        function (creator) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.thenPromise))) {
                this._stack = Promise._getStack();
            }

            this._creator = creator;
            this._setState(state_created);
            this._run();
        }, {
            _creator: null,

            _cancelAction: function () { if (this._creator) { this._creator.cancel(); } },
            _cleanupAction: function () { this._creator = null; }
        }, {
            supportedForProcessing: false
        }
    );

    //
    // Slim promise implementations for already completed promises, these are created
    // under the hood on synchronous completion paths as well as by WinJS.Promise.wrap
    // and WinJS.Promise.wrapError.
    //

    var ErrorPromise = _Base.Class.define(
        function ErrorPromise_ctor(value) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.errorPromise))) {
                this._stack = Promise._getStack();
            }

            this._value = value;
            callonerror(this, value, detailsForError);
        }, {
            cancel: function () {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.cancel">
                /// <summary locid="WinJS.PromiseStateMachine.cancel">
                /// Attempts to cancel the fulfillment of a promised value. If the promise hasn't
                /// already been fulfilled and cancellation is supported, the promise enters
                /// the error state with a value of Error("Canceled").
                /// </summary>
                /// </signature>
            },
            done: function ErrorPromise_done(unused, onError) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.done">
                /// <summary locid="WinJS.PromiseStateMachine.done">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                ///
                /// After the handlers have finished executing, this function throws any error that would have been returned
                /// from then() as a promise in the error state.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.done_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The fulfilled value is passed as the single argument. If the value is null,
                /// the fulfilled value is returned. The value returned
                /// from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while executing the function, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function is the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onProgress">
                /// the function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// </signature>
                var value = this._value;
                if (onError) {
                    try {
                        if (!onError.handlesOnError) {
                            callonerror(null, value, detailsForHandledError, this, onError);
                        }
                        var result = onError(value);
                        if (result && typeof result === "object" && typeof result.done === "function") {
                            // If a promise is returned we need to wait on it.
                            result.done();
                        }
                        return;
                    } catch (ex) {
                        value = ex;
                    }
                }
                if (value instanceof Error && value.message === canceledName) {
                    // suppress cancel
                    return;
                }
                // force the exception to be thrown asyncronously to avoid any try/catch blocks
                //
                Promise._doneHandler(value);
            },
            then: function ErrorPromise_then(unused, onError) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.then">
                /// <summary locid="WinJS.PromiseStateMachine.then">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.then_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The value is passed as the single argument. If the value is null, the value is returned.
                /// The value returned from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while this function is being executed, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function becomes the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onProgress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.PromiseStateMachine.then_returnValue">
                /// The promise whose value is the result of executing the complete or
                /// error function.
                /// </returns>
                /// </signature>

                // If the promise is already in a error state and no error handler is provided
                // we optimize by simply returning the promise instead of creating a new one.
                //
                if (!onError) { return this; }
                var result;
                var value = this._value;
                try {
                    if (!onError.handlesOnError) {
                        callonerror(null, value, detailsForHandledError, this, onError);
                    }
                    result = new CompletePromise(onError(value));
                } catch (ex) {
                    // If the value throw from the error handler is the same as the value
                    // provided to the error handler then there is no need for a new promise.
                    //
                    if (ex === value) {
                        result = this;
                    } else {
                        result = new ExceptionPromise(ex);
                    }
                }
                return result;
            }
        }, {
            supportedForProcessing: false
        }
    );

    var ExceptionPromise = _Base.Class.derive(ErrorPromise,
        function ExceptionPromise_ctor(value) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.exceptionPromise))) {
                this._stack = Promise._getStack();
            }

            this._value = value;
            callonerror(this, value, detailsForException);
        }, {
            /* empty */
        }, {
            supportedForProcessing: false
        }
    );

    var CompletePromise = _Base.Class.define(
        function CompletePromise_ctor(value) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.completePromise))) {
                this._stack = Promise._getStack();
            }

            if (value && typeof value === "object" && typeof value.then === "function") {
                var result = new ThenPromise(null);
                result._setCompleteValue(value);
                return result;
            }
            this._value = value;
        }, {
            cancel: function () {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.cancel">
                /// <summary locid="WinJS.PromiseStateMachine.cancel">
                /// Attempts to cancel the fulfillment of a promised value. If the promise hasn't
                /// already been fulfilled and cancellation is supported, the promise enters
                /// the error state with a value of Error("Canceled").
                /// </summary>
                /// </signature>
            },
            done: function CompletePromise_done(onComplete) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.done">
                /// <summary locid="WinJS.PromiseStateMachine.done">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                ///
                /// After the handlers have finished executing, this function throws any error that would have been returned
                /// from then() as a promise in the error state.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.done_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The fulfilled value is passed as the single argument. If the value is null,
                /// the fulfilled value is returned. The value returned
                /// from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while executing the function, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function is the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onProgress">
                /// the function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// </signature>
                if (!onComplete) { return; }
                try {
                    var result = onComplete(this._value);
                    if (result && typeof result === "object" && typeof result.done === "function") {
                        result.done();
                    }
                } catch (ex) {
                    // force the exception to be thrown asynchronously to avoid any try/catch blocks
                    Promise._doneHandler(ex);
                }
            },
            then: function CompletePromise_then(onComplete) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.then">
                /// <summary locid="WinJS.PromiseStateMachine.then">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.then_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The value is passed as the single argument. If the value is null, the value is returned.
                /// The value returned from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while this function is being executed, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function becomes the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onProgress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.PromiseStateMachine.then_returnValue">
                /// The promise whose value is the result of executing the complete or
                /// error function.
                /// </returns>
                /// </signature>
                try {
                    // If the value returned from the completion handler is the same as the value
                    // provided to the completion handler then there is no need for a new promise.
                    //
                    var newValue = onComplete ? onComplete(this._value) : this._value;
                    return newValue === this._value ? this : new CompletePromise(newValue);
                } catch (ex) {
                    return new ExceptionPromise(ex);
                }
            }
        }, {
            supportedForProcessing: false
        }
    );

    //
    // Promise is the user-creatable WinJS.Promise object.
    //

    function timeout(timeoutMS) {
        var id;
        return new Promise(
            function (c) {
                if (timeoutMS) {
                    id = _Global.setTimeout(c, timeoutMS);
                } else {
                    _BaseCoreUtils._setImmediate(c);
                }
            },
            function () {
                if (id) {
                    _Global.clearTimeout(id);
                }
            }
        );
    }

    function timeoutWithPromise(timeout, promise) {
        var cancelPromise = function () { promise.cancel(); };
        var cancelTimeout = function () { timeout.cancel(); };
        timeout.then(cancelPromise);
        promise.then(cancelTimeout, cancelTimeout);
        return promise;
    }

    var staticCanceledPromise;

    var Promise = _Base.Class.derive(PromiseStateMachine,
        function Promise_ctor(init, oncancel) {
            /// <signature helpKeyword="WinJS.Promise">
            /// <summary locid="WinJS.Promise">
            /// A promise provides a mechanism to schedule work to be done on a value that
            /// has not yet been computed. It is a convenient abstraction for managing
            /// interactions with asynchronous APIs.
            /// </summary>
            /// <param name="init" type="Function" locid="WinJS.Promise_p:init">
            /// The function that is called during construction of the  promise. The function
            /// is given three arguments (complete, error, progress). Inside this function
            /// you should add event listeners for the notifications supported by this value.
            /// </param>
            /// <param name="oncancel" optional="true" locid="WinJS.Promise_p:oncancel">
            /// The function to call if a consumer of this promise wants
            /// to cancel its undone work. Promises are not required to
            /// support cancellation.
            /// </param>
            /// </signature>

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.promise))) {
                this._stack = Promise._getStack();
            }

            this._oncancel = oncancel;
            this._setState(state_created);
            this._run();

            try {
                var complete = this._completed.bind(this);
                var error = this._error.bind(this);
                var progress = this._progress.bind(this);
                init(complete, error, progress);
            } catch (ex) {
                this._setExceptionValue(ex);
            }
        }, {
            _oncancel: null,

            _cancelAction: function () {
                // BEGIN monaco change
                try {
                    if (this._oncancel) {
                        this._oncancel();
                    } else {
                        throw new Error('Promise did not implement oncancel');
                    }
                } catch (ex) {
                    // Access fields to get them created
                    var msg = ex.message;
                    var stack = ex.stack;
                    promiseEventListeners.dispatchEvent('error', ex);
                }
                // END monaco change
            },
            _cleanupAction: function () { this._oncancel = null; }
        }, {

            addEventListener: function Promise_addEventListener(eventType, listener, capture) {
                /// <signature helpKeyword="WinJS.Promise.addEventListener">
                /// <summary locid="WinJS.Promise.addEventListener">
                /// Adds an event listener to the control.
                /// </summary>
                /// <param name="eventType" locid="WinJS.Promise.addEventListener_p:eventType">
                /// The type (name) of the event.
                /// </param>
                /// <param name="listener" locid="WinJS.Promise.addEventListener_p:listener">
                /// The listener to invoke when the event is raised.
                /// </param>
                /// <param name="capture" locid="WinJS.Promise.addEventListener_p:capture">
                /// Specifies whether or not to initiate capture.
                /// </param>
                /// </signature>
                promiseEventListeners.addEventListener(eventType, listener, capture);
            },
            any: function Promise_any(values) {
                /// <signature helpKeyword="WinJS.Promise.any">
                /// <summary locid="WinJS.Promise.any">
                /// Returns a promise that is fulfilled when one of the input promises
                /// has been fulfilled.
                /// </summary>
                /// <param name="values" type="Array" locid="WinJS.Promise.any_p:values">
                /// An array that contains promise objects or objects whose property
                /// values include promise objects.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.any_returnValue">
                /// A promise that on fulfillment yields the value of the input (complete or error).
                /// </returns>
                /// </signature>
                return new Promise(
                    function (complete, error) {
                        var keys = Object.keys(values);
                        if (keys.length === 0) {
                            complete();
                        }
                        var canceled = 0;
                        keys.forEach(function (key) {
                            Promise.as(values[key]).then(
                                function () { complete({ key: key, value: values[key] }); },
                                function (e) {
                                    if (e instanceof Error && e.name === canceledName) {
                                        if ((++canceled) === keys.length) {
                                            complete(Promise.cancel);
                                        }
                                        return;
                                    }
                                    error({ key: key, value: values[key] });
                                }
                            );
                        });
                    },
                    function () {
                        var keys = Object.keys(values);
                        keys.forEach(function (key) {
                            var promise = Promise.as(values[key]);
                            if (typeof promise.cancel === "function") {
                                promise.cancel();
                            }
                        });
                    }
                );
            },
            as: function Promise_as(value) {
                /// <signature helpKeyword="WinJS.Promise.as">
                /// <summary locid="WinJS.Promise.as">
                /// Returns a promise. If the object is already a promise it is returned;
                /// otherwise the object is wrapped in a promise.
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.as_p:value">
                /// The value to be treated as a promise.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.as_returnValue">
                /// A promise.
                /// </returns>
                /// </signature>
                if (value && typeof value === "object" && typeof value.then === "function") {
                    return value;
                }
                return new CompletePromise(value);
            },
            /// <field type="WinJS.Promise" helpKeyword="WinJS.Promise.cancel" locid="WinJS.Promise.cancel">
            /// Canceled promise value, can be returned from a promise completion handler
            /// to indicate cancelation of the promise chain.
            /// </field>
            cancel: {
                get: function () {
                    return (staticCanceledPromise = staticCanceledPromise || new ErrorPromise(new _ErrorFromName(canceledName)));
                }
            },
            dispatchEvent: function Promise_dispatchEvent(eventType, details) {
                /// <signature helpKeyword="WinJS.Promise.dispatchEvent">
                /// <summary locid="WinJS.Promise.dispatchEvent">
                /// Raises an event of the specified type and properties.
                /// </summary>
                /// <param name="eventType" locid="WinJS.Promise.dispatchEvent_p:eventType">
                /// The type (name) of the event.
                /// </param>
                /// <param name="details" locid="WinJS.Promise.dispatchEvent_p:details">
                /// The set of additional properties to be attached to the event object.
                /// </param>
                /// <returns type="Boolean" locid="WinJS.Promise.dispatchEvent_returnValue">
                /// Specifies whether preventDefault was called on the event.
                /// </returns>
                /// </signature>
                return promiseEventListeners.dispatchEvent(eventType, details);
            },
            is: function Promise_is(value) {
                /// <signature helpKeyword="WinJS.Promise.is">
                /// <summary locid="WinJS.Promise.is">
                /// Determines whether a value fulfills the promise contract.
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.is_p:value">
                /// A value that may be a promise.
                /// </param>
                /// <returns type="Boolean" locid="WinJS.Promise.is_returnValue">
                /// true if the specified value is a promise, otherwise false.
                /// </returns>
                /// </signature>
                return value && typeof value === "object" && typeof value.then === "function";
            },
            join: function Promise_join(values) {
                /// <signature helpKeyword="WinJS.Promise.join">
                /// <summary locid="WinJS.Promise.join">
                /// Creates a promise that is fulfilled when all the values are fulfilled.
                /// </summary>
                /// <param name="values" type="Object" locid="WinJS.Promise.join_p:values">
                /// An object whose fields contain values, some of which may be promises.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.join_returnValue">
                /// A promise whose value is an object with the same field names as those of the object in the values parameter, where
                /// each field value is the fulfilled value of a promise.
                /// </returns>
                /// </signature>
                return new Promise(
                    function (complete, error, progress) {
                        var keys = Object.keys(values);
                        var errors = Array.isArray(values) ? [] : {};
                        var results = Array.isArray(values) ? [] : {};
                        var undefineds = 0;
                        var pending = keys.length;
                        var argDone = function (key) {
                            if ((--pending) === 0) {
                                var errorCount = Object.keys(errors).length;
                                if (errorCount === 0) {
                                    complete(results);
                                } else {
                                    var canceledCount = 0;
                                    keys.forEach(function (key) {
                                        var e = errors[key];
                                        if (e instanceof Error && e.name === canceledName) {
                                            canceledCount++;
                                        }
                                    });
                                    if (canceledCount === errorCount) {
                                        complete(Promise.cancel);
                                    } else {
                                        error(errors);
                                    }
                                }
                            } else {
                                progress({ Key: key, Done: true });
                            }
                        };
                        keys.forEach(function (key) {
                            var value = values[key];
                            if (value === undefined) {
                                undefineds++;
                            } else {
                                Promise.then(value,
                                    function (value) { results[key] = value; argDone(key); },
                                    function (value) { errors[key] = value; argDone(key); }
                                );
                            }
                        });
                        pending -= undefineds;
                        if (pending === 0) {
                            complete(results);
                            return;
                        }
                    },
                    function () {
                        Object.keys(values).forEach(function (key) {
                            var promise = Promise.as(values[key]);
                            if (typeof promise.cancel === "function") {
                                promise.cancel();
                            }
                        });
                    }
                );
            },
            removeEventListener: function Promise_removeEventListener(eventType, listener, capture) {
                /// <signature helpKeyword="WinJS.Promise.removeEventListener">
                /// <summary locid="WinJS.Promise.removeEventListener">
                /// Removes an event listener from the control.
                /// </summary>
                /// <param name='eventType' locid="WinJS.Promise.removeEventListener_eventType">
                /// The type (name) of the event.
                /// </param>
                /// <param name='listener' locid="WinJS.Promise.removeEventListener_listener">
                /// The listener to remove.
                /// </param>
                /// <param name='capture' locid="WinJS.Promise.removeEventListener_capture">
                /// Specifies whether or not to initiate capture.
                /// </param>
                /// </signature>
                promiseEventListeners.removeEventListener(eventType, listener, capture);
            },
            supportedForProcessing: false,
            then: function Promise_then(value, onComplete, onError, onProgress) {
                /// <signature helpKeyword="WinJS.Promise.then">
                /// <summary locid="WinJS.Promise.then">
                /// A static version of the promise instance method then().
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.then_p:value">
                /// the value to be treated as a promise.
                /// </param>
                /// <param name="onComplete" type="Function" locid="WinJS.Promise.then_p:complete">
                /// The function to be called if the promise is fulfilled with a value.
                /// If it is null, the promise simply
                /// returns the value. The value is passed as the single argument.
                /// </param>
                /// <param name="onError" type="Function" optional="true" locid="WinJS.Promise.then_p:error">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument.
                /// </param>
                /// <param name="onProgress" type="Function" optional="true" locid="WinJS.Promise.then_p:progress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.then_returnValue">
                /// A promise whose value is the result of executing the provided complete function.
                /// </returns>
                /// </signature>
                return Promise.as(value).then(onComplete, onError, onProgress);
            },
            thenEach: function Promise_thenEach(values, onComplete, onError, onProgress) {
                /// <signature helpKeyword="WinJS.Promise.thenEach">
                /// <summary locid="WinJS.Promise.thenEach">
                /// Performs an operation on all the input promises and returns a promise
                /// that has the shape of the input and contains the result of the operation
                /// that has been performed on each input.
                /// </summary>
                /// <param name="values" locid="WinJS.Promise.thenEach_p:values">
                /// A set of values (which could be either an array or an object) of which some or all are promises.
                /// </param>
                /// <param name="onComplete" type="Function" locid="WinJS.Promise.thenEach_p:complete">
                /// The function to be called if the promise is fulfilled with a value.
                /// If the value is null, the promise returns the value.
                /// The value is passed as the single argument.
                /// </param>
                /// <param name="onError" type="Function" optional="true" locid="WinJS.Promise.thenEach_p:error">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument.
                /// </param>
                /// <param name="onProgress" type="Function" optional="true" locid="WinJS.Promise.thenEach_p:progress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.thenEach_returnValue">
                /// A promise that is the result of calling Promise.join on the values parameter.
                /// </returns>
                /// </signature>
                var result = Array.isArray(values) ? [] : {};
                Object.keys(values).forEach(function (key) {
                    result[key] = Promise.as(values[key]).then(onComplete, onError, onProgress);
                });
                return Promise.join(result);
            },
            timeout: function Promise_timeout(time, promise) {
                /// <signature helpKeyword="WinJS.Promise.timeout">
                /// <summary locid="WinJS.Promise.timeout">
                /// Creates a promise that is fulfilled after a timeout.
                /// </summary>
                /// <param name="timeout" type="Number" optional="true" locid="WinJS.Promise.timeout_p:timeout">
                /// The timeout period in milliseconds. If this value is zero or not specified
                /// setImmediate is called, otherwise setTimeout is called.
                /// </param>
                /// <param name="promise" type="Promise" optional="true" locid="WinJS.Promise.timeout_p:promise">
                /// A promise that will be canceled if it doesn't complete before the
                /// timeout has expired.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.timeout_returnValue">
                /// A promise that is completed asynchronously after the specified timeout.
                /// </returns>
                /// </signature>
                var to = timeout(time);
                return promise ? timeoutWithPromise(to, promise) : to;
            },
            wrap: function Promise_wrap(value) {
                /// <signature helpKeyword="WinJS.Promise.wrap">
                /// <summary locid="WinJS.Promise.wrap">
                /// Wraps a non-promise value in a promise. You can use this function if you need
                /// to pass a value to a function that requires a promise.
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.wrap_p:value">
                /// Some non-promise value to be wrapped in a promise.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.wrap_returnValue">
                /// A promise that is successfully fulfilled with the specified value
                /// </returns>
                /// </signature>
                return new CompletePromise(value);
            },
            wrapError: function Promise_wrapError(error) {
                /// <signature helpKeyword="WinJS.Promise.wrapError">
                /// <summary locid="WinJS.Promise.wrapError">
                /// Wraps a non-promise error value in a promise. You can use this function if you need
                /// to pass an error to a function that requires a promise.
                /// </summary>
                /// <param name="error" locid="WinJS.Promise.wrapError_p:error">
                /// A non-promise error value to be wrapped in a promise.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.wrapError_returnValue">
                /// A promise that is in an error state with the specified value.
                /// </returns>
                /// </signature>
                return new ErrorPromise(error);
            },

            _veryExpensiveTagWithStack: {
                get: function () { return tagWithStack; },
                set: function (value) { tagWithStack = value; }
            },
            _veryExpensiveTagWithStack_tag: tag,
            _getStack: function () {
                if (_Global.Debug && _Global.Debug.debuggerEnabled) {
                    try { throw new Error(); } catch (e) { return e.stack; }
                }
            },

            _cancelBlocker: function Promise__cancelBlocker(input, oncancel) {
                //
                // Returns a promise which on cancelation will still result in downstream cancelation while
                //  protecting the promise 'input' from being  canceled which has the effect of allowing
                //  'input' to be shared amoung various consumers.
                //
                if (!Promise.is(input)) {
                    return Promise.wrap(input);
                }
                var complete;
                var error;
                var output = new Promise(
                    function (c, e) {
                        complete = c;
                        error = e;
                    },
                    function () {
                        complete = null;
                        error = null;
                        oncancel && oncancel();
                    }
                );
                input.then(
                    function (v) { complete && complete(v); },
                    function (e) { error && error(e); }
                );
                return output;
            },

        }
    );
    Object.defineProperties(Promise, _Events.createEventProperties(errorET));

    Promise._doneHandler = function (value) {
        _BaseCoreUtils._setImmediate(function Promise_done_rethrow() {
            throw value;
        });
    };

    return {
        PromiseStateMachine: PromiseStateMachine,
        Promise: Promise,
        state_created: state_created
    };
});

_winjs("WinJS/Promise", ["WinJS/Core/_Base","WinJS/Promise/_StateMachine"], function promiseInit( _Base, _StateMachine) {

    _Base.Namespace.define("WinJS", {
        Promise: _StateMachine.Promise
    });

    return _StateMachine.Promise;
});

__winjs_exports = _modules["WinJS/Core/_WinJS"];
__winjs_exports.TPromise = __winjs_exports.Promise;
__winjs_exports.PPromise = __winjs_exports.Promise;

// ESM-comment-begin
// if (typeof exports === 'undefined' && typeof define === 'function' && define.amd) {
//     define([], __winjs_exports);
// } else {
//     module.exports = __winjs_exports;
// }
// ESM-comment-end

})();

// ESM-uncomment-begin
var Promise$1 = __winjs_exports.Promise;
var TPromise = __winjs_exports.TPromise;
var PPromise = __winjs_exports.PPromise;
// ESM-uncomment-end

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// ------ BEGIN Hook up error listeners to winjs promises
var outstandingPromiseErrors = {};
function promiseErrorHandler(e) {
    //
    // e.detail looks like: { exception, error, promise, handler, id, parent }
    //
    var details = e.detail;
    var id = details.id;
    // If the error has a parent promise then this is not the origination of the
    //  error so we check if it has a handler, and if so we mark that the error
    //  was handled by removing it from outstandingPromiseErrors
    //
    if (details.parent) {
        if (details.handler && outstandingPromiseErrors) {
            delete outstandingPromiseErrors[id];
        }
        return;
    }
    // Indicate that this error was originated and needs to be handled
    outstandingPromiseErrors[id] = details;
    // The first time the queue fills up this iteration, schedule a timeout to
    // check if any errors are still unhandled.
    if (Object.keys(outstandingPromiseErrors).length === 1) {
        setTimeout(function () {
            var errors = outstandingPromiseErrors;
            outstandingPromiseErrors = {};
            Object.keys(errors).forEach(function (errorId) {
                var error = errors[errorId];
                if (error.exception) {
                    onUnexpectedError(error.exception);
                }
                else if (error.error) {
                    onUnexpectedError(error.error);
                }
                console.log('WARNING: Promise with no error callback:' + error.id);
                console.log(error);
                if (error.exception) {
                    console.log(error.exception.stack);
                }
            });
        }, 0);
    }
}
TPromise.addEventListener('error', promiseErrorHandler);
// Avoid circular dependency on EventEmitter by implementing a subset of the interface.
var ErrorHandler = /** @class */ (function () {
    function ErrorHandler() {
        this.listeners = [];
        this.unexpectedErrorHandler = function (e) {
            setTimeout(function () {
                if (e.stack) {
                    throw new Error(e.message + '\n\n' + e.stack);
                }
                throw e;
            }, 0);
        };
    }
    ErrorHandler.prototype.emit = function (e) {
        this.listeners.forEach(function (listener) {
            listener(e);
        });
    };
    ErrorHandler.prototype.onUnexpectedError = function (e) {
        this.unexpectedErrorHandler(e);
        this.emit(e);
    };
    // For external errors, we don't want the listeners to be called
    ErrorHandler.prototype.onUnexpectedExternalError = function (e) {
        this.unexpectedErrorHandler(e);
    };
    return ErrorHandler;
}());
var errorHandler = new ErrorHandler();
function onUnexpectedError(e) {
    // ignore errors from cancelled promises
    if (!isPromiseCanceledError(e)) {
        errorHandler.onUnexpectedError(e);
    }
    return undefined;
}
function onUnexpectedExternalError(e) {
    // ignore errors from cancelled promises
    if (!isPromiseCanceledError(e)) {
        errorHandler.onUnexpectedExternalError(e);
    }
    return undefined;
}
function transformErrorForSerialization(error) {
    if (error instanceof Error) {
        var name_1 = error.name, message = error.message;
        var stack = error.stacktrace || error.stack;
        return {
            $isError: true,
            name: name_1,
            message: message,
            stack: stack
        };
    }
    // return as is
    return error;
}
var canceledName = 'Canceled';
/**
 * Checks if the given error is a promise in canceled state
 */
function isPromiseCanceledError(error) {
    return error instanceof Error && error.name === canceledName && error.message === canceledName;
}
/**
 * Returns an error that signals cancellation.
 */
function canceled() {
    var error = new Error(canceledName);
    error.name = error.message;
    return error;
}
function illegalArgument(name) {
    if (name) {
        return new Error("Illegal argument: " + name);
    }
    else {
        return new Error('Illegal argument');
    }
}
function illegalState(name) {
    if (name) {
        return new Error("Illegal state: " + name);
    }
    else {
        return new Error('Illegal state');
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function once(fn) {
    var _this = this;
    var didCall = false;
    var result;
    return function () {
        if (didCall) {
            return result;
        }
        didCall = true;
        result = fn.apply(_this, arguments);
        return result;
    };
}

function isDisposable(thing) {
    return typeof thing.dispose === 'function'
        && thing.dispose.length === 0;
}
function dispose(first) {
    var rest = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        rest[_i - 1] = arguments[_i];
    }
    if (Array.isArray(first)) {
        first.forEach(function (d) { return d && d.dispose(); });
        return [];
    }
    else if (rest.length === 0) {
        if (first) {
            first.dispose();
            return first;
        }
        return undefined;
    }
    else {
        dispose(first);
        dispose(rest);
        return [];
    }
}
function combinedDisposable(disposables) {
    return { dispose: function () { return dispose(disposables); } };
}
function toDisposable(fn) {
    return { dispose: function () { fn(); } };
}
var Disposable = /** @class */ (function () {
    function Disposable() {
        this._toDispose = [];
    }
    Disposable.prototype.dispose = function () {
        this._toDispose = dispose(this._toDispose);
    };
    Disposable.prototype._register = function (t) {
        this._toDispose.push(t);
        return t;
    };
    Disposable.None = Object.freeze({ dispose: function () { } });
    return Disposable;
}());
var ImmortalReference = /** @class */ (function () {
    function ImmortalReference(object) {
        this.object = object;
    }
    ImmortalReference.prototype.dispose = function () { };
    return ImmortalReference;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var FIN = { done: true, value: undefined };
var Iterator;
(function (Iterator) {
    var _empty = {
        next: function () {
            return FIN;
        }
    };
    function empty() {
        return _empty;
    }
    Iterator.empty = empty;
    function fromArray(array, index, length) {
        if (index === void 0) { index = 0; }
        if (length === void 0) { length = array.length; }
        return {
            next: function () {
                if (index >= length) {
                    return FIN;
                }
                return { done: false, value: array[index++] };
            }
        };
    }
    Iterator.fromArray = fromArray;
    function from(elements) {
        if (!elements) {
            return Iterator.empty();
        }
        else if (Array.isArray(elements)) {
            return Iterator.fromArray(elements);
        }
        else {
            return elements;
        }
    }
    Iterator.from = from;
    function map(iterator, fn) {
        return {
            next: function () {
                var element = iterator.next();
                if (element.done) {
                    return FIN;
                }
                else {
                    return { done: false, value: fn(element.value) };
                }
            }
        };
    }
    Iterator.map = map;
    function filter(iterator, fn) {
        return {
            next: function () {
                while (true) {
                    var element = iterator.next();
                    if (element.done) {
                        return FIN;
                    }
                    if (fn(element.value)) {
                        return { done: false, value: element.value };
                    }
                }
            }
        };
    }
    Iterator.filter = filter;
    function forEach(iterator, fn) {
        for (var next = iterator.next(); !next.done; next = iterator.next()) {
            fn(next.value);
        }
    }
    Iterator.forEach = forEach;
    function collect(iterator) {
        var result = [];
        forEach(iterator, function (value) { return result.push(value); });
        return result;
    }
    Iterator.collect = collect;
})(Iterator || (Iterator = {}));
var ArrayIterator = /** @class */ (function () {
    function ArrayIterator(items, start, end, index) {
        if (start === void 0) { start = 0; }
        if (end === void 0) { end = items.length; }
        if (index === void 0) { index = start - 1; }
        this.items = items;
        this.start = start;
        this.end = end;
        this.index = index;
    }
    ArrayIterator.prototype.next = function () {
        this.index = Math.min(this.index + 1, this.end);
        return this.current();
    };
    ArrayIterator.prototype.current = function () {
        if (this.index === this.start - 1 || this.index === this.end) {
            return null;
        }
        return this.items[this.index];
    };
    return ArrayIterator;
}());
var ArrayNavigator = /** @class */ (function (_super) {
    __extends(ArrayNavigator, _super);
    function ArrayNavigator(items, start, end, index) {
        if (start === void 0) { start = 0; }
        if (end === void 0) { end = items.length; }
        if (index === void 0) { index = start - 1; }
        return _super.call(this, items, start, end, index) || this;
    }
    ArrayNavigator.prototype.current = function () {
        return _super.prototype.current.call(this);
    };
    ArrayNavigator.prototype.previous = function () {
        this.index = Math.max(this.index - 1, this.start - 1);
        return this.current();
    };
    ArrayNavigator.prototype.first = function () {
        this.index = this.start;
        return this.current();
    };
    ArrayNavigator.prototype.last = function () {
        this.index = this.end - 1;
        return this.current();
    };
    ArrayNavigator.prototype.parent = function () {
        return null;
    };
    return ArrayNavigator;
}(ArrayIterator));
var MappedIterator = /** @class */ (function () {
    function MappedIterator(iterator, fn) {
        this.iterator = iterator;
        this.fn = fn;
        // noop
    }
    MappedIterator.prototype.next = function () { return this.fn(this.iterator.next()); };
    return MappedIterator;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Node$1 = /** @class */ (function () {
    function Node(element) {
        this.element = element;
    }
    return Node;
}());
var LinkedList = /** @class */ (function () {
    function LinkedList() {
    }
    LinkedList.prototype.isEmpty = function () {
        return !this._first;
    };
    LinkedList.prototype.unshift = function (element) {
        return this.insert(element, false);
    };
    LinkedList.prototype.push = function (element) {
        return this.insert(element, true);
    };
    LinkedList.prototype.insert = function (element, atTheEnd) {
        var _this = this;
        var newNode = new Node$1(element);
        if (!this._first) {
            this._first = newNode;
            this._last = newNode;
        }
        else if (atTheEnd) {
            // push
            var oldLast = this._last;
            this._last = newNode;
            newNode.prev = oldLast;
            oldLast.next = newNode;
        }
        else {
            // unshift
            var oldFirst = this._first;
            this._first = newNode;
            newNode.next = oldFirst;
            oldFirst.prev = newNode;
        }
        return function () {
            var candidate = _this._first;
            while (candidate instanceof Node$1) {
                if (candidate !== newNode) {
                    candidate = candidate.next;
                    continue;
                }
                if (candidate.prev && candidate.next) {
                    // middle
                    var anchor = candidate.prev;
                    anchor.next = candidate.next;
                    candidate.next.prev = anchor;
                }
                else if (!candidate.prev && !candidate.next) {
                    // only node
                    _this._first = undefined;
                    _this._last = undefined;
                }
                else if (!candidate.next) {
                    // last
                    _this._last = _this._last.prev;
                    _this._last.next = undefined;
                }
                else if (!candidate.prev) {
                    // first
                    _this._first = _this._first.next;
                    _this._first.prev = undefined;
                }
                // done
                break;
            }
        };
    };
    LinkedList.prototype.iterator = function () {
        var element;
        var node = this._first;
        return {
            next: function () {
                if (!node) {
                    return FIN;
                }
                if (!element) {
                    element = { done: false, value: node.element };
                }
                else {
                    element.value = node.element;
                }
                node = node.next;
                return element;
            }
        };
    };
    return LinkedList;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Event;
(function (Event) {
    var _disposable = { dispose: function () { } };
    Event.None = function () { return _disposable; };
})(Event || (Event = {}));
/**
 * The Emitter can be used to expose an Event to the public
 * to fire it from the insides.
 * Sample:
    class Document {

        private _onDidChange = new Emitter<(value:string)=>any>();

        public onDidChange = this._onDidChange.event;

        // getter-style
        // get onDidChange(): Event<(value:string)=>any> {
        // 	return this._onDidChange.event;
        // }

        private _doIt() {
            //...
            this._onDidChange.fire(value);
        }
    }
 */
var Emitter = /** @class */ (function () {
    function Emitter(_options) {
        if (_options === void 0) { _options = null; }
        this._options = _options;
        this._event = null;
        this._disposed = false;
        this._deliveryQueue = null;
        this._listeners = null;
    }
    Object.defineProperty(Emitter.prototype, "event", {
        /**
         * For the public to allow to subscribe
         * to events from this Emitter
         */
        get: function () {
            var _this = this;
            if (!this._event) {
                this._event = function (listener, thisArgs, disposables) {
                    if (!_this._listeners) {
                        _this._listeners = new LinkedList();
                    }
                    var firstListener = _this._listeners.isEmpty();
                    if (firstListener && _this._options && _this._options.onFirstListenerAdd) {
                        _this._options.onFirstListenerAdd(_this);
                    }
                    var remove = _this._listeners.push(!thisArgs ? listener : [listener, thisArgs]);
                    if (firstListener && _this._options && _this._options.onFirstListenerDidAdd) {
                        _this._options.onFirstListenerDidAdd(_this);
                    }
                    if (_this._options && _this._options.onListenerDidAdd) {
                        _this._options.onListenerDidAdd(_this, listener, thisArgs);
                    }
                    var result;
                    result = {
                        dispose: function () {
                            result.dispose = Emitter._noop;
                            if (!_this._disposed) {
                                remove();
                                if (_this._options && _this._options.onLastListenerRemove) {
                                    var hasListeners = (_this._listeners && !_this._listeners.isEmpty());
                                    if (!hasListeners) {
                                        _this._options.onLastListenerRemove(_this);
                                    }
                                }
                            }
                        }
                    };
                    if (Array.isArray(disposables)) {
                        disposables.push(result);
                    }
                    return result;
                };
            }
            return this._event;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * To be kept private to fire an event to
     * subscribers
     */
    Emitter.prototype.fire = function (event) {
        if (this._listeners) {
            // put all [listener,event]-pairs into delivery queue
            // then emit all event. an inner/nested event might be
            // the driver of this
            if (!this._deliveryQueue) {
                this._deliveryQueue = [];
            }
            for (var iter = this._listeners.iterator(), e = iter.next(); !e.done; e = iter.next()) {
                this._deliveryQueue.push([e.value, event]);
            }
            while (this._deliveryQueue.length > 0) {
                var _a = this._deliveryQueue.shift(), listener = _a[0], event_1 = _a[1];
                try {
                    if (typeof listener === 'function') {
                        listener.call(undefined, event_1);
                    }
                    else {
                        listener[0].call(listener[1], event_1);
                    }
                }
                catch (e) {
                    onUnexpectedError(e);
                }
            }
        }
    };
    Emitter.prototype.dispose = function () {
        if (this._listeners) {
            this._listeners = null;
        }
        if (this._deliveryQueue) {
            this._deliveryQueue.length = 0;
        }
        this._disposed = true;
    };
    Emitter._noop = function () { };
    return Emitter;
}());
var EventMultiplexer = /** @class */ (function () {
    function EventMultiplexer() {
        var _this = this;
        this.hasListeners = false;
        this.events = [];
        this.emitter = new Emitter({
            onFirstListenerAdd: function () { return _this.onFirstListenerAdd(); },
            onLastListenerRemove: function () { return _this.onLastListenerRemove(); }
        });
    }
    Object.defineProperty(EventMultiplexer.prototype, "event", {
        get: function () {
            return this.emitter.event;
        },
        enumerable: true,
        configurable: true
    });
    EventMultiplexer.prototype.add = function (event) {
        var _this = this;
        var e = { event: event, listener: null };
        this.events.push(e);
        if (this.hasListeners) {
            this.hook(e);
        }
        var dispose$$1 = function () {
            if (_this.hasListeners) {
                _this.unhook(e);
            }
            var idx = _this.events.indexOf(e);
            _this.events.splice(idx, 1);
        };
        return toDisposable(once(dispose$$1));
    };
    EventMultiplexer.prototype.onFirstListenerAdd = function () {
        var _this = this;
        this.hasListeners = true;
        this.events.forEach(function (e) { return _this.hook(e); });
    };
    EventMultiplexer.prototype.onLastListenerRemove = function () {
        var _this = this;
        this.hasListeners = false;
        this.events.forEach(function (e) { return _this.unhook(e); });
    };
    EventMultiplexer.prototype.hook = function (e) {
        var _this = this;
        e.listener = e.event(function (r) { return _this.emitter.fire(r); });
    };
    EventMultiplexer.prototype.unhook = function (e) {
        if (e.listener) {
            e.listener.dispose();
        }
        e.listener = null;
    };
    EventMultiplexer.prototype.dispose = function () {
        this.emitter.dispose();
    };
    return EventMultiplexer;
}());
function once$1(event) {
    return function (listener, thisArgs, disposables) {
        if (thisArgs === void 0) { thisArgs = null; }
        // we need this, in case the event fires during the listener call
        var didFire = false;
        var result = event(function (e) {
            if (didFire) {
                return;
            }
            else if (result) {
                result.dispose();
            }
            else {
                didFire = true;
            }
            return listener.call(thisArgs, e);
        }, null, disposables);
        if (didFire) {
            result.dispose();
        }
        return result;
    };
}
function anyEvent() {
    var events = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        events[_i] = arguments[_i];
    }
    return function (listener, thisArgs, disposables) {
        if (thisArgs === void 0) { thisArgs = null; }
        return combinedDisposable(events.map(function (event) { return event(function (e) { return listener.call(thisArgs, e); }, null, disposables); }));
    };
}
function debounceEvent(event, merger, delay, leading) {
    if (delay === void 0) { delay = 100; }
    if (leading === void 0) { leading = false; }
    var subscription;
    var output = undefined;
    var handle = undefined;
    var numDebouncedCalls = 0;
    var emitter = new Emitter({
        onFirstListenerAdd: function () {
            subscription = event(function (cur) {
                numDebouncedCalls++;
                output = merger(output, cur);
                if (leading && !handle) {
                    emitter.fire(output);
                }
                clearTimeout(handle);
                handle = setTimeout(function () {
                    var _output = output;
                    output = undefined;
                    handle = undefined;
                    if (!leading || numDebouncedCalls > 1) {
                        emitter.fire(_output);
                    }
                    numDebouncedCalls = 0;
                }, delay);
            });
        },
        onLastListenerRemove: function () {
            subscription.dispose();
        }
    });
    return emitter.event;
}
/**
 * The EventDelayer is useful in situations in which you want
 * to delay firing your events during some code.
 * You can wrap that code and be sure that the event will not
 * be fired during that wrap.
 *
 * ```
 * const emitter: Emitter;
 * const delayer = new EventDelayer();
 * const delayedEvent = delayer.wrapEvent(emitter.event);
 *
 * delayedEvent(console.log);
 *
 * delayer.bufferEvents(() => {
 *   emitter.fire(); // event will not be fired yet
 * });
 *
 * // event will only be fired at this point
 * ```
 */
var EventBufferer = /** @class */ (function () {
    function EventBufferer() {
        this.buffers = [];
    }
    EventBufferer.prototype.wrapEvent = function (event) {
        var _this = this;
        return function (listener, thisArgs, disposables) {
            return event(function (i) {
                var buffer = _this.buffers[_this.buffers.length - 1];
                if (buffer) {
                    buffer.push(function () { return listener.call(thisArgs, i); });
                }
                else {
                    listener.call(thisArgs, i);
                }
            }, void 0, disposables);
        };
    };
    EventBufferer.prototype.bufferEvents = function (fn) {
        var buffer = [];
        this.buffers.push(buffer);
        var r = fn();
        this.buffers.pop();
        buffer.forEach(function (flush) { return flush(); });
        return r;
    };
    return EventBufferer;
}());
function mapEvent(event, map) {
    return function (listener, thisArgs, disposables) {
        if (thisArgs === void 0) { thisArgs = null; }
        return event(function (i) { return listener.call(thisArgs, map(i)); }, null, disposables);
    };
}
function filterEvent(event, filter) {
    return function (listener, thisArgs, disposables) {
        if (thisArgs === void 0) { thisArgs = null; }
        return event(function (e) { return filter(e) && listener.call(thisArgs, e); }, null, disposables);
    };
}
var ChainableEvent = /** @class */ (function () {
    function ChainableEvent(_event) {
        this._event = _event;
    }
    Object.defineProperty(ChainableEvent.prototype, "event", {
        get: function () { return this._event; },
        enumerable: true,
        configurable: true
    });
    ChainableEvent.prototype.map = function (fn) {
        return new ChainableEvent(mapEvent(this._event, fn));
    };
    ChainableEvent.prototype.filter = function (fn) {
        return new ChainableEvent(filterEvent(this._event, fn));
    };
    ChainableEvent.prototype.on = function (listener, thisArgs, disposables) {
        return this._event(listener, thisArgs, disposables);
    };
    return ChainableEvent;
}());
function chain(event) {
    return new ChainableEvent(event);
}
var Relay = /** @class */ (function () {
    function Relay() {
        var _this = this;
        this.listening = false;
        this.inputEvent = Event.None;
        this.inputEventListener = Disposable.None;
        this.emitter = new Emitter({
            onFirstListenerDidAdd: function () {
                _this.listening = true;
                _this.inputEventListener = _this.inputEvent(_this.emitter.fire, _this.emitter);
            },
            onLastListenerRemove: function () {
                _this.listening = false;
                _this.inputEventListener.dispose();
            }
        });
        this.event = this.emitter.event;
    }
    Object.defineProperty(Relay.prototype, "input", {
        set: function (event) {
            this.inputEvent = event;
            if (this.listening) {
                this.inputEventListener.dispose();
                this.inputEventListener = event(this.emitter.fire, this.emitter);
            }
        },
        enumerable: true,
        configurable: true
    });
    Relay.prototype.dispose = function () {
        this.inputEventListener.dispose();
        this.emitter.dispose();
    };
    return Relay;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var shortcutEvent = Object.freeze(function (callback, context) {
    var handle = setTimeout(callback.bind(context), 0);
    return { dispose: function () { clearTimeout(handle); } };
});
var CancellationToken;
(function (CancellationToken) {
    function isCancellationToken(thing) {
        if (thing === CancellationToken.None || thing === CancellationToken.Cancelled) {
            return true;
        }
        if (thing instanceof MutableToken) {
            return true;
        }
        if (!thing || typeof thing !== 'object') {
            return false;
        }
        return typeof thing.isCancellationRequested === 'boolean'
            && typeof thing.onCancellationRequested === 'function';
    }
    CancellationToken.isCancellationToken = isCancellationToken;
    CancellationToken.None = Object.freeze({
        isCancellationRequested: false,
        onCancellationRequested: Event.None
    });
    CancellationToken.Cancelled = Object.freeze({
        isCancellationRequested: true,
        onCancellationRequested: shortcutEvent
    });
})(CancellationToken || (CancellationToken = {}));
var MutableToken = /** @class */ (function () {
    function MutableToken() {
        this._isCancelled = false;
        this._emitter = null;
    }
    MutableToken.prototype.cancel = function () {
        if (!this._isCancelled) {
            this._isCancelled = true;
            if (this._emitter) {
                this._emitter.fire(undefined);
                this.dispose();
            }
        }
    };
    Object.defineProperty(MutableToken.prototype, "isCancellationRequested", {
        get: function () {
            return this._isCancelled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MutableToken.prototype, "onCancellationRequested", {
        get: function () {
            if (this._isCancelled) {
                return shortcutEvent;
            }
            if (!this._emitter) {
                this._emitter = new Emitter();
            }
            return this._emitter.event;
        },
        enumerable: true,
        configurable: true
    });
    MutableToken.prototype.dispose = function () {
        if (this._emitter) {
            this._emitter.dispose();
            this._emitter = null;
        }
    };
    return MutableToken;
}());
var CancellationTokenSource = /** @class */ (function () {
    function CancellationTokenSource() {
    }
    Object.defineProperty(CancellationTokenSource.prototype, "token", {
        get: function () {
            if (!this._token) {
                // be lazy and create the token only when
                // actually needed
                this._token = new MutableToken();
            }
            return this._token;
        },
        enumerable: true,
        configurable: true
    });
    CancellationTokenSource.prototype.cancel = function () {
        if (!this._token) {
            // save an object by returning the default
            // cancelled token when cancellation happens
            // before someone asks for the token
            this._token = CancellationToken.Cancelled;
        }
        else if (this._token instanceof MutableToken) {
            // actually cancel
            this._token.cancel();
        }
    };
    CancellationTokenSource.prototype.dispose = function () {
        if (!this._token) {
            // ensure to initialize with an empty token if we had none
            this._token = CancellationToken.None;
        }
        else if (this._token instanceof MutableToken) {
            // actually dispose
            this._token.dispose();
        }
    };
    return CancellationTokenSource;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function isThenable(obj) {
    return obj && typeof obj.then === 'function';
}
function createCancelablePromise(callback) {
    var source = new CancellationTokenSource();
    var thenable = callback(source.token);
    var promise = new Promise(function (resolve, reject) {
        source.token.onCancellationRequested(function () {
            reject(canceled());
        });
        Promise.resolve(thenable).then(function (value) {
            source.dispose();
            resolve(value);
        }, function (err) {
            source.dispose();
            reject(err);
        });
    });
    return new /** @class */ (function () {
        function class_1() {
        }
        class_1.prototype.cancel = function () {
            source.cancel();
        };
        class_1.prototype.then = function (resolve, reject) {
            return promise.then(resolve, reject);
        };
        class_1.prototype.catch = function (reject) {
            return this.then(undefined, reject);
        };
        return class_1;
    }());
}
/**
 * A helper to delay execution of a task that is being requested often.
 *
 * Following the throttler, now imagine the mail man wants to optimize the number of
 * trips proactively. The trip itself can be long, so he decides not to make the trip
 * as soon as a letter is submitted. Instead he waits a while, in case more
 * letters are submitted. After said waiting period, if no letters were submitted, he
 * decides to make the trip. Imagine that N more letters were submitted after the first
 * one, all within a short period of time between each other. Even though N+1
 * submissions occurred, only 1 delivery was made.
 *
 * The delayer offers this behavior via the trigger() method, into which both the task
 * to be executed and the waiting period (delay) must be passed in as arguments. Following
 * the example:
 *
 * 		const delayer = new Delayer(WAITING_PERIOD);
 * 		const letters = [];
 *
 * 		function letterReceived(l) {
 * 			letters.push(l);
 * 			delayer.trigger(() => { return makeTheTrip(); });
 * 		}
 */
var Delayer = /** @class */ (function () {
    function Delayer(defaultDelay) {
        this.defaultDelay = defaultDelay;
        this.timeout = null;
        this.completionPromise = null;
        this.doResolve = null;
        this.task = null;
    }
    Delayer.prototype.trigger = function (task, delay) {
        var _this = this;
        if (delay === void 0) { delay = this.defaultDelay; }
        this.task = task;
        this.cancelTimeout();
        if (!this.completionPromise) {
            this.completionPromise = new TPromise(function (c, e) {
                _this.doResolve = c;
                _this.doReject = e;
            }).then(function () {
                _this.completionPromise = null;
                _this.doResolve = null;
                var task = _this.task;
                _this.task = null;
                return task();
            });
        }
        this.timeout = setTimeout(function () {
            _this.timeout = null;
            _this.doResolve(null);
        }, delay);
        return this.completionPromise;
    };
    Delayer.prototype.cancel = function () {
        this.cancelTimeout();
        if (this.completionPromise) {
            this.doReject(canceled());
            this.completionPromise = null;
        }
    };
    Delayer.prototype.cancelTimeout = function () {
        if (this.timeout !== null) {
            clearTimeout(this.timeout);
            this.timeout = null;
        }
    };
    Delayer.prototype.dispose = function () {
        this.cancelTimeout();
    };
    return Delayer;
}());
function timeout(millis, token) {
    if (!token) {
        return createCancelablePromise(function (token) { return timeout(millis, token); });
    }
    return new Promise(function (resolve, reject) {
        var handle = setTimeout(resolve, millis);
        token.onCancellationRequested(function () {
            clearTimeout(handle);
            reject(canceled());
        });
    });
}
/**
 * Returns a new promise that joins the provided promise. Upon completion of
 * the provided promise the provided function will always be called. This
 * method is comparable to a try-finally code block.
 * @param promise a promise
 * @param callback a function that will be call in the success and error case.
 */
function always(promise, callback) {
    function safeCallback() {
        try {
            callback();
        }
        catch (err) {
            onUnexpectedError(err);
        }
    }
    promise.then(function (_) { return safeCallback(); }, function (_) { return safeCallback(); });
    return Promise.resolve(promise);
}
function first(promiseFactories, shouldStop, defaultValue) {
    if (shouldStop === void 0) { shouldStop = function (t) { return !!t; }; }
    if (defaultValue === void 0) { defaultValue = null; }
    var index = 0;
    var len = promiseFactories.length;
    var loop = function () {
        if (index >= len) {
            return Promise.resolve(defaultValue);
        }
        var factory = promiseFactories[index++];
        var promise = Promise.resolve(factory());
        return promise.then(function (result) {
            if (shouldStop(result)) {
                return Promise.resolve(result);
            }
            return loop();
        });
    };
    return loop();
}
var TimeoutTimer = /** @class */ (function (_super) {
    __extends$1(TimeoutTimer, _super);
    function TimeoutTimer(runner, timeout) {
        var _this = _super.call(this) || this;
        _this._token = -1;
        if (typeof runner === 'function' && typeof timeout === 'number') {
            _this.setIfNotSet(runner, timeout);
        }
        return _this;
    }
    TimeoutTimer.prototype.dispose = function () {
        this.cancel();
        _super.prototype.dispose.call(this);
    };
    TimeoutTimer.prototype.cancel = function () {
        if (this._token !== -1) {
            clearTimeout(this._token);
            this._token = -1;
        }
    };
    TimeoutTimer.prototype.cancelAndSet = function (runner, timeout) {
        var _this = this;
        this.cancel();
        this._token = setTimeout(function () {
            _this._token = -1;
            runner();
        }, timeout);
    };
    TimeoutTimer.prototype.setIfNotSet = function (runner, timeout) {
        var _this = this;
        if (this._token !== -1) {
            // timer is already set
            return;
        }
        this._token = setTimeout(function () {
            _this._token = -1;
            runner();
        }, timeout);
    };
    return TimeoutTimer;
}(Disposable));
var IntervalTimer = /** @class */ (function (_super) {
    __extends$1(IntervalTimer, _super);
    function IntervalTimer() {
        var _this = _super.call(this) || this;
        _this._token = -1;
        return _this;
    }
    IntervalTimer.prototype.dispose = function () {
        this.cancel();
        _super.prototype.dispose.call(this);
    };
    IntervalTimer.prototype.cancel = function () {
        if (this._token !== -1) {
            clearInterval(this._token);
            this._token = -1;
        }
    };
    IntervalTimer.prototype.cancelAndSet = function (runner, interval) {
        this.cancel();
        this._token = setInterval(function () {
            runner();
        }, interval);
    };
    return IntervalTimer;
}(Disposable));
var RunOnceScheduler = /** @class */ (function () {
    function RunOnceScheduler(runner, timeout) {
        this.timeoutToken = -1;
        this.runner = runner;
        this.timeout = timeout;
        this.timeoutHandler = this.onTimeout.bind(this);
    }
    /**
     * Dispose RunOnceScheduler
     */
    RunOnceScheduler.prototype.dispose = function () {
        this.cancel();
        this.runner = null;
    };
    /**
     * Cancel current scheduled runner (if any).
     */
    RunOnceScheduler.prototype.cancel = function () {
        if (this.isScheduled()) {
            clearTimeout(this.timeoutToken);
            this.timeoutToken = -1;
        }
    };
    /**
     * Cancel previous runner (if any) & schedule a new runner.
     */
    RunOnceScheduler.prototype.schedule = function (delay) {
        if (delay === void 0) { delay = this.timeout; }
        this.cancel();
        this.timeoutToken = setTimeout(this.timeoutHandler, delay);
    };
    /**
     * Returns true if scheduled.
     */
    RunOnceScheduler.prototype.isScheduled = function () {
        return this.timeoutToken !== -1;
    };
    RunOnceScheduler.prototype.onTimeout = function () {
        this.timeoutToken = -1;
        if (this.runner) {
            this.doRun();
        }
    };
    RunOnceScheduler.prototype.doRun = function () {
        if (this.runner) {
            this.runner();
        }
    };
    return RunOnceScheduler;
}());
/**
 * Execute the callback the next time the browser is idle
 */
var runWhenIdle;
(function () {
    if (typeof requestIdleCallback !== 'function' || typeof cancelIdleCallback !== 'function') {
        var dummyIdle_1 = Object.freeze({
            didTimeout: true,
            timeRemaining: function () { return 15; }
        });
        runWhenIdle = function (runner, timeout) {
            if (timeout === void 0) { timeout = 0; }
            var handle = setTimeout(function () { return runner(dummyIdle_1); }, timeout);
            var disposed = false;
            return {
                dispose: function () {
                    if (disposed) {
                        return;
                    }
                    disposed = true;
                    clearTimeout(handle);
                }
            };
        };
    }
    else {
        runWhenIdle = function (runner, timeout) {
            var handle = requestIdleCallback(runner, typeof timeout === 'number' ? { timeout: timeout } : undefined);
            var disposed = false;
            return {
                dispose: function () {
                    if (disposed) {
                        return;
                    }
                    disposed = true;
                    cancelIdleCallback(handle);
                }
            };
        };
    }
})();
/**
 * An implementation of the "idle-until-urgent"-strategy as introduced
 * here: https://philipwalton.com/articles/idle-until-urgent/
 */
var IdleValue = /** @class */ (function () {
    function IdleValue(executor) {
        var _this = this;
        this._executor = function () {
            try {
                _this._value = executor();
            }
            catch (err) {
                _this._error = err;
            }
            finally {
                _this._didRun = true;
            }
        };
        this._handle = runWhenIdle(function () { return _this._executor(); });
    }
    IdleValue.prototype.dispose = function () {
        this._handle.dispose();
    };
    IdleValue.prototype.getValue = function () {
        if (!this._didRun) {
            this._handle.dispose();
            this._executor();
        }
        if (this._error) {
            throw this._error;
        }
        return this._value;
    };
    return IdleValue;
}());
//#endregion

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var _isWindows = false;
var _isMacintosh = false;
var _isLinux = false;
var _isNative = false;
var _isWeb = false;
var _locale = undefined;
var _translationsConfigFile = undefined;
var LANGUAGE_DEFAULT = 'en';
var isElectronRenderer = (typeof process !== 'undefined' && typeof process.versions !== 'undefined' && typeof process.versions.electron !== 'undefined' && process.type === 'renderer');
// OS detection
if (typeof navigator === 'object' && !isElectronRenderer) {
    var userAgent = navigator.userAgent;
    _isWindows = userAgent.indexOf('Windows') >= 0;
    _isMacintosh = userAgent.indexOf('Macintosh') >= 0;
    _isLinux = userAgent.indexOf('Linux') >= 0;
    _isWeb = true;
    _locale = navigator.language;
}
else if (typeof process === 'object') {
    _isWindows = (process.platform === 'win32');
    _isMacintosh = (process.platform === 'darwin');
    _isLinux = (process.platform === 'linux');
    _locale = LANGUAGE_DEFAULT;
    var rawNlsConfig = process.env['VSCODE_NLS_CONFIG'];
    if (rawNlsConfig) {
        try {
            var nlsConfig = JSON.parse(rawNlsConfig);
            var resolved = nlsConfig.availableLanguages['*'];
            _locale = nlsConfig.locale;
            _translationsConfigFile = nlsConfig._translationsConfigFile;
        }
        catch (e) {
        }
    }
    _isNative = true;
}
var isWindows = _isWindows;
var isMacintosh = _isMacintosh;
var isLinux = _isLinux;
var isNative = _isNative;
var isWeb = _isWeb;
var _globals = (typeof self === 'object' ? self : typeof global === 'object' ? global : {});
var globals = _globals;
var _setImmediate = null;
function setImmediate(callback) {
    if (_setImmediate === null) {
        if (globals.setImmediate) {
            _setImmediate = globals.setImmediate.bind(globals);
        }
        else if (typeof process !== 'undefined' && typeof process.nextTick === 'function') {
            _setImmediate = process.nextTick.bind(process);
        }
        else {
            _setImmediate = globals.setTimeout.bind(globals);
        }
    }
    return _setImmediate(callback);
}
var OS = (_isMacintosh ? 2 /* Macintosh */ : (_isWindows ? 1 /* Windows */ : 3 /* Linux */));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var _a;
var _schemePattern = /^\w[\w\d+.-]*$/;
var _singleSlashStart = /^\//;
var _doubleSlashStart = /^\/\//;
function _validateUri(ret) {
    // scheme, must be set
    if (!ret.scheme) {
        {
            throw new Error("[UriError]: Scheme is missing: {scheme: \"\", authority: \"" + ret.authority + "\", path: \"" + ret.path + "\", query: \"" + ret.query + "\", fragment: \"" + ret.fragment + "\"}");
        }
    }
    // scheme, https://tools.ietf.org/html/rfc3986#section-3.1
    // ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )
    if (ret.scheme && !_schemePattern.test(ret.scheme)) {
        throw new Error('[UriError]: Scheme contains illegal characters.');
    }
    // path, http://tools.ietf.org/html/rfc3986#section-3.3
    // If a URI contains an authority component, then the path component
    // must either be empty or begin with a slash ("/") character.  If a URI
    // does not contain an authority component, then the path cannot begin
    // with two slash characters ("//").
    if (ret.path) {
        if (ret.authority) {
            if (!_singleSlashStart.test(ret.path)) {
                throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash ("/") character');
            }
        }
        else {
            if (_doubleSlashStart.test(ret.path)) {
                throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters ("//")');
            }
        }
    }
}
// implements a bit of https://tools.ietf.org/html/rfc3986#section-5
function _referenceResolution(scheme, path) {
    // the slash-character is our 'default base' as we don't
    // support constructing URIs relative to other URIs. This
    // also means that we alter and potentially break paths.
    // see https://tools.ietf.org/html/rfc3986#section-5.1.4
    switch (scheme) {
        case 'https':
        case 'http':
        case 'file':
            if (!path) {
                path = _slash;
            }
            else if (path[0] !== _slash) {
                path = _slash + path;
            }
            break;
    }
    return path;
}
var _empty = '';
var _slash = '/';
var _regexp = /^(([^:/?#]+?):)?(\/\/([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/;
/**
 * Uniform Resource Identifier (URI) http://tools.ietf.org/html/rfc3986.
 * This class is a simple parser which creates the basic component parts
 * (http://tools.ietf.org/html/rfc3986#section-3) with minimal validation
 * and encoding.
 *
 *       foo://example.com:8042/over/there?name=ferret#nose
 *       \_/   \______________/\_________/ \_________/ \__/
 *        |           |            |            |        |
 *     scheme     authority       path        query   fragment
 *        |   _____________________|__
 *       / \ /                        \
 *       urn:example:animal:ferret:nose
 */
var URI = /** @class */ (function () {
    /**
     * @internal
     */
    function URI(schemeOrData, authority, path, query, fragment) {
        if (typeof schemeOrData === 'object') {
            this.scheme = schemeOrData.scheme || _empty;
            this.authority = schemeOrData.authority || _empty;
            this.path = schemeOrData.path || _empty;
            this.query = schemeOrData.query || _empty;
            this.fragment = schemeOrData.fragment || _empty;
            // no validation because it's this URI
            // that creates uri components.
            // _validateUri(this);
        }
        else {
            this.scheme = schemeOrData || _empty;
            this.authority = authority || _empty;
            this.path = _referenceResolution(this.scheme, path || _empty);
            this.query = query || _empty;
            this.fragment = fragment || _empty;
            _validateUri(this);
        }
    }
    URI.isUri = function (thing) {
        if (thing instanceof URI) {
            return true;
        }
        if (!thing) {
            return false;
        }
        return typeof thing.authority === 'string'
            && typeof thing.fragment === 'string'
            && typeof thing.path === 'string'
            && typeof thing.query === 'string'
            && typeof thing.scheme === 'string';
    };
    Object.defineProperty(URI.prototype, "fsPath", {
        // ---- filesystem path -----------------------
        /**
         * Returns a string representing the corresponding file system path of this URI.
         * Will handle UNC paths, normalizes windows drive letters to lower-case, and uses the
         * platform specific path separator.
         *
         * * Will *not* validate the path for invalid characters and semantics.
         * * Will *not* look at the scheme of this URI.
         * * The result shall *not* be used for display purposes but for accessing a file on disk.
         *
         *
         * The *difference* to `URI#path` is the use of the platform specific separator and the handling
         * of UNC paths. See the below sample of a file-uri with an authority (UNC path).
         *
         * ```ts
            const u = URI.parse('file://server/c$/folder/file.txt')
            u.authority === 'server'
            u.path === '/shares/c$/file.txt'
            u.fsPath === '\\server\c$\folder\file.txt'
        ```
         *
         * Using `URI#path` to read a file (using fs-apis) would not be enough because parts of the path,
         * namely the server name, would be missing. Therefore `URI#fsPath` exists - it's sugar to ease working
         * with URIs that represent files on disk (`file` scheme).
         */
        get: function () {
            // if (this.scheme !== 'file') {
            // 	console.warn(`[UriError] calling fsPath with scheme ${this.scheme}`);
            // }
            return _makeFsPath(this);
        },
        enumerable: true,
        configurable: true
    });
    // ---- modify to new -------------------------
    URI.prototype.with = function (change) {
        if (!change) {
            return this;
        }
        var scheme = change.scheme, authority = change.authority, path = change.path, query = change.query, fragment = change.fragment;
        if (scheme === void 0) {
            scheme = this.scheme;
        }
        else if (scheme === null) {
            scheme = _empty;
        }
        if (authority === void 0) {
            authority = this.authority;
        }
        else if (authority === null) {
            authority = _empty;
        }
        if (path === void 0) {
            path = this.path;
        }
        else if (path === null) {
            path = _empty;
        }
        if (query === void 0) {
            query = this.query;
        }
        else if (query === null) {
            query = _empty;
        }
        if (fragment === void 0) {
            fragment = this.fragment;
        }
        else if (fragment === null) {
            fragment = _empty;
        }
        if (scheme === this.scheme
            && authority === this.authority
            && path === this.path
            && query === this.query
            && fragment === this.fragment) {
            return this;
        }
        return new _URI(scheme, authority, path, query, fragment);
    };
    // ---- parse & validate ------------------------
    /**
     * Creates a new URI from a string, e.g. `http://www.msft.com/some/path`,
     * `file:///usr/home`, or `scheme:with/path`.
     *
     * @param value A string which represents an URI (see `URI#toString`).
     */
    URI.parse = function (value) {
        var match = _regexp.exec(value);
        if (!match) {
            return new _URI(_empty, _empty, _empty, _empty, _empty);
        }
        return new _URI(match[2] || _empty, decodeURIComponent(match[4] || _empty), decodeURIComponent(match[5] || _empty), decodeURIComponent(match[7] || _empty), decodeURIComponent(match[9] || _empty));
    };
    /**
     * Creates a new URI from a file system path, e.g. `c:\my\files`,
     * `/usr/home`, or `\\server\share\some\path`.
     *
     * The *difference* between `URI#parse` and `URI#file` is that the latter treats the argument
     * as path, not as stringified-uri. E.g. `URI.file(path)` is **not the same as**
     * `URI.parse('file://' + path)` because the path might contain characters that are
     * interpreted (# and ?). See the following sample:
     * ```ts
    const good = URI.file('/coding/c#/project1');
    good.scheme === 'file';
    good.path === '/coding/c#/project1';
    good.fragment === '';
    const bad = URI.parse('file://' + '/coding/c#/project1');
    bad.scheme === 'file';
    bad.path === '/coding/c'; // path is now broken
    bad.fragment === '/project1';
    ```
     *
     * @param path A file system path (see `URI#fsPath`)
     */
    URI.file = function (path) {
        var authority = _empty;
        // normalize to fwd-slashes on windows,
        // on other systems bwd-slashes are valid
        // filename character, eg /f\oo/ba\r.txt
        if (isWindows) {
            path = path.replace(/\\/g, _slash);
        }
        // check for authority as used in UNC shares
        // or use the path as given
        if (path[0] === _slash && path[1] === _slash) {
            var idx = path.indexOf(_slash, 2);
            if (idx === -1) {
                authority = path.substring(2);
                path = _slash;
            }
            else {
                authority = path.substring(2, idx);
                path = path.substring(idx) || _slash;
            }
        }
        return new _URI('file', authority, path, _empty, _empty);
    };
    URI.from = function (components) {
        return new _URI(components.scheme, components.authority, components.path, components.query, components.fragment);
    };
    // ---- printing/externalize ---------------------------
    /**
     * Creates a string presentation for this URI. It's guaranteed that calling
     * `URI.parse` with the result of this function creates an URI which is equal
     * to this URI.
     *
     * * The result shall *not* be used for display purposes but for externalization or transport.
     * * The result will be encoded using the percentage encoding and encoding happens mostly
     * ignore the scheme-specific encoding rules.
     *
     * @param skipEncoding Do not encode the result, default is `false`
     */
    URI.prototype.toString = function (skipEncoding) {
        if (skipEncoding === void 0) { skipEncoding = false; }
        return _asFormatted(this, skipEncoding);
    };
    URI.prototype.toJSON = function () {
        return this;
    };
    URI.revive = function (data) {
        if (!data) {
            return data;
        }
        else if (data instanceof URI) {
            return data;
        }
        else {
            var result = new _URI(data);
            result._fsPath = data.fsPath;
            result._formatted = data.external;
            return result;
        }
    };
    return URI;
}());
// tslint:disable-next-line:class-name
var _URI = /** @class */ (function (_super) {
    __extends$2(_URI, _super);
    function _URI() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._formatted = null;
        _this._fsPath = null;
        return _this;
    }
    Object.defineProperty(_URI.prototype, "fsPath", {
        get: function () {
            if (!this._fsPath) {
                this._fsPath = _makeFsPath(this);
            }
            return this._fsPath;
        },
        enumerable: true,
        configurable: true
    });
    _URI.prototype.toString = function (skipEncoding) {
        if (skipEncoding === void 0) { skipEncoding = false; }
        if (!skipEncoding) {
            if (!this._formatted) {
                this._formatted = _asFormatted(this, false);
            }
            return this._formatted;
        }
        else {
            // we don't cache that
            return _asFormatted(this, true);
        }
    };
    _URI.prototype.toJSON = function () {
        var res = {
            $mid: 1
        };
        // cached state
        if (this._fsPath) {
            res.fsPath = this._fsPath;
        }
        if (this._formatted) {
            res.external = this._formatted;
        }
        // uri components
        if (this.path) {
            res.path = this.path;
        }
        if (this.scheme) {
            res.scheme = this.scheme;
        }
        if (this.authority) {
            res.authority = this.authority;
        }
        if (this.query) {
            res.query = this.query;
        }
        if (this.fragment) {
            res.fragment = this.fragment;
        }
        return res;
    };
    return _URI;
}(URI));
// reserved characters: https://tools.ietf.org/html/rfc3986#section-2.2
var encodeTable = (_a = {},
    _a[58 /* Colon */] = '%3A',
    _a[47 /* Slash */] = '%2F',
    _a[63 /* QuestionMark */] = '%3F',
    _a[35 /* Hash */] = '%23',
    _a[91 /* OpenSquareBracket */] = '%5B',
    _a[93 /* CloseSquareBracket */] = '%5D',
    _a[64 /* AtSign */] = '%40',
    _a[33 /* ExclamationMark */] = '%21',
    _a[36 /* DollarSign */] = '%24',
    _a[38 /* Ampersand */] = '%26',
    _a[39 /* SingleQuote */] = '%27',
    _a[40 /* OpenParen */] = '%28',
    _a[41 /* CloseParen */] = '%29',
    _a[42 /* Asterisk */] = '%2A',
    _a[43 /* Plus */] = '%2B',
    _a[44 /* Comma */] = '%2C',
    _a[59 /* Semicolon */] = '%3B',
    _a[61 /* Equals */] = '%3D',
    _a[32 /* Space */] = '%20',
    _a);
function encodeURIComponentFast(uriComponent, allowSlash) {
    var res = undefined;
    var nativeEncodePos = -1;
    for (var pos = 0; pos < uriComponent.length; pos++) {
        var code = uriComponent.charCodeAt(pos);
        // unreserved characters: https://tools.ietf.org/html/rfc3986#section-2.3
        if ((code >= 97 /* a */ && code <= 122 /* z */)
            || (code >= 65 /* A */ && code <= 90 /* Z */)
            || (code >= 48 /* Digit0 */ && code <= 57 /* Digit9 */)
            || code === 45 /* Dash */
            || code === 46 /* Period */
            || code === 95 /* Underline */
            || code === 126 /* Tilde */
            || (allowSlash && code === 47 /* Slash */)) {
            // check if we are delaying native encode
            if (nativeEncodePos !== -1) {
                res += encodeURIComponent(uriComponent.substring(nativeEncodePos, pos));
                nativeEncodePos = -1;
            }
            // check if we write into a new string (by default we try to return the param)
            if (res !== undefined) {
                res += uriComponent.charAt(pos);
            }
        }
        else {
            // encoding needed, we need to allocate a new string
            if (res === undefined) {
                res = uriComponent.substr(0, pos);
            }
            // check with default table first
            var escaped = encodeTable[code];
            if (escaped !== undefined) {
                // check if we are delaying native encode
                if (nativeEncodePos !== -1) {
                    res += encodeURIComponent(uriComponent.substring(nativeEncodePos, pos));
                    nativeEncodePos = -1;
                }
                // append escaped variant to result
                res += escaped;
            }
            else if (nativeEncodePos === -1) {
                // use native encode only when needed
                nativeEncodePos = pos;
            }
        }
    }
    if (nativeEncodePos !== -1) {
        res += encodeURIComponent(uriComponent.substring(nativeEncodePos));
    }
    return res !== undefined ? res : uriComponent;
}
function encodeURIComponentMinimal(path) {
    var res = undefined;
    for (var pos = 0; pos < path.length; pos++) {
        var code = path.charCodeAt(pos);
        if (code === 35 /* Hash */ || code === 63 /* QuestionMark */) {
            if (res === undefined) {
                res = path.substr(0, pos);
            }
            res += encodeTable[code];
        }
        else {
            if (res !== undefined) {
                res += path[pos];
            }
        }
    }
    return res !== undefined ? res : path;
}
/**
 * Compute `fsPath` for the given uri
 * @param uri
 */
function _makeFsPath(uri) {
    var value;
    if (uri.authority && uri.path.length > 1 && uri.scheme === 'file') {
        // unc path: file://shares/c$/far/boo
        value = "//" + uri.authority + uri.path;
    }
    else if (uri.path.charCodeAt(0) === 47 /* Slash */
        && (uri.path.charCodeAt(1) >= 65 /* A */ && uri.path.charCodeAt(1) <= 90 /* Z */ || uri.path.charCodeAt(1) >= 97 /* a */ && uri.path.charCodeAt(1) <= 122 /* z */)
        && uri.path.charCodeAt(2) === 58 /* Colon */) {
        // windows drive letter: file:///c:/far/boo
        value = uri.path[1].toLowerCase() + uri.path.substr(2);
    }
    else {
        // other path
        value = uri.path;
    }
    if (isWindows) {
        value = value.replace(/\//g, '\\');
    }
    return value;
}
/**
 * Create the external version of a uri
 */
function _asFormatted(uri, skipEncoding) {
    var encoder = !skipEncoding
        ? encodeURIComponentFast
        : encodeURIComponentMinimal;
    var res = '';
    var scheme = uri.scheme, authority = uri.authority, path = uri.path, query = uri.query, fragment = uri.fragment;
    if (scheme) {
        res += scheme;
        res += ':';
    }
    if (authority || scheme === 'file') {
        res += _slash;
        res += _slash;
    }
    if (authority) {
        var idx = authority.indexOf('@');
        if (idx !== -1) {
            // <user>@<auth>
            var userinfo = authority.substr(0, idx);
            authority = authority.substr(idx + 1);
            idx = userinfo.indexOf(':');
            if (idx === -1) {
                res += encoder(userinfo, false);
            }
            else {
                // <user>:<pass>@<auth>
                res += encoder(userinfo.substr(0, idx), false);
                res += ':';
                res += encoder(userinfo.substr(idx + 1), false);
            }
            res += '@';
        }
        authority = authority.toLowerCase();
        idx = authority.indexOf(':');
        if (idx === -1) {
            res += encoder(authority, false);
        }
        else {
            // <auth>:<port>
            res += encoder(authority.substr(0, idx), false);
            res += authority.substr(idx);
        }
    }
    if (path) {
        // lower-case windows drive letters in /C:/fff or C:/fff
        if (path.length >= 3 && path.charCodeAt(0) === 47 /* Slash */ && path.charCodeAt(2) === 58 /* Colon */) {
            var code = path.charCodeAt(1);
            if (code >= 65 /* A */ && code <= 90 /* Z */) {
                path = "/" + String.fromCharCode(code + 32) + ":" + path.substr(3); // "/c:".length === 3
            }
        }
        else if (path.length >= 2 && path.charCodeAt(1) === 58 /* Colon */) {
            var code = path.charCodeAt(0);
            if (code >= 65 /* A */ && code <= 90 /* Z */) {
                path = String.fromCharCode(code + 32) + ":" + path.substr(2); // "/c:".length === 3
            }
        }
        // encode the rest of the path
        res += encoder(path, true);
    }
    if (query) {
        res += '?';
        res += encoder(query, false);
    }
    if (fragment) {
        res += '#';
        res += !skipEncoding ? encodeURIComponentFast(fragment, false) : fragment;
    }
    return res;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// ------ internal util
var _util;
(function (_util) {
    _util.serviceIds = new Map();
    _util.DI_TARGET = '$di$target';
    _util.DI_DEPENDENCIES = '$di$dependencies';
    function getServiceDependencies(ctor) {
        return ctor[_util.DI_DEPENDENCIES] || [];
    }
    _util.getServiceDependencies = getServiceDependencies;
})(_util || (_util = {}));
var IInstantiationService = createDecorator('instantiationService');
function storeServiceDependency(id, target, index, optional) {
    if (target[_util.DI_TARGET] === target) {
        target[_util.DI_DEPENDENCIES].push({ id: id, index: index, optional: optional });
    }
    else {
        target[_util.DI_DEPENDENCIES] = [{ id: id, index: index, optional: optional }];
        target[_util.DI_TARGET] = target;
    }
}
/**
 * A *only* valid way to create a {{ServiceIdentifier}}.
 */
function createDecorator(serviceId) {
    if (_util.serviceIds.has(serviceId)) {
        return _util.serviceIds.get(serviceId);
    }
    var id = function (target, key, index) {
        if (arguments.length !== 3) {
            throw new Error('@IServiceName-decorator can only be used to decorate a parameter');
        }
        storeServiceDependency(id, target, index, false);
    };
    id.toString = function () { return serviceId; };
    _util.serviceIds.set(serviceId, id);
    return id;
}
/**
 * Mark a service dependency as optional.
 */
function optional(serviceIdentifier) {
    return function (target, key, index) {
        if (arguments.length !== 3) {
            throw new Error('@optional-decorator can only be used to decorate a parameter');
        }
        storeServiceDependency(serviceIdentifier, target, index, true);
    };
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ICodeEditorService = createDecorator('codeEditorService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * A position in the editor.
 */
var Position = /** @class */ (function () {
    function Position(lineNumber, column) {
        this.lineNumber = lineNumber;
        this.column = column;
    }
    /**
     * Create a new postion from this position.
     *
     * @param newLineNumber new line number
     * @param newColumn new column
     */
    Position.prototype.with = function (newLineNumber, newColumn) {
        if (newLineNumber === void 0) { newLineNumber = this.lineNumber; }
        if (newColumn === void 0) { newColumn = this.column; }
        if (newLineNumber === this.lineNumber && newColumn === this.column) {
            return this;
        }
        else {
            return new Position(newLineNumber, newColumn);
        }
    };
    /**
     * Derive a new position from this position.
     *
     * @param deltaLineNumber line number delta
     * @param deltaColumn column delta
     */
    Position.prototype.delta = function (deltaLineNumber, deltaColumn) {
        if (deltaLineNumber === void 0) { deltaLineNumber = 0; }
        if (deltaColumn === void 0) { deltaColumn = 0; }
        return this.with(this.lineNumber + deltaLineNumber, this.column + deltaColumn);
    };
    /**
     * Test if this position equals other position
     */
    Position.prototype.equals = function (other) {
        return Position.equals(this, other);
    };
    /**
     * Test if position `a` equals position `b`
     */
    Position.equals = function (a, b) {
        if (!a && !b) {
            return true;
        }
        return (!!a &&
            !!b &&
            a.lineNumber === b.lineNumber &&
            a.column === b.column);
    };
    /**
     * Test if this position is before other position.
     * If the two positions are equal, the result will be false.
     */
    Position.prototype.isBefore = function (other) {
        return Position.isBefore(this, other);
    };
    /**
     * Test if position `a` is before position `b`.
     * If the two positions are equal, the result will be false.
     */
    Position.isBefore = function (a, b) {
        if (a.lineNumber < b.lineNumber) {
            return true;
        }
        if (b.lineNumber < a.lineNumber) {
            return false;
        }
        return a.column < b.column;
    };
    /**
     * Test if this position is before other position.
     * If the two positions are equal, the result will be true.
     */
    Position.prototype.isBeforeOrEqual = function (other) {
        return Position.isBeforeOrEqual(this, other);
    };
    /**
     * Test if position `a` is before position `b`.
     * If the two positions are equal, the result will be true.
     */
    Position.isBeforeOrEqual = function (a, b) {
        if (a.lineNumber < b.lineNumber) {
            return true;
        }
        if (b.lineNumber < a.lineNumber) {
            return false;
        }
        return a.column <= b.column;
    };
    /**
     * A function that compares positions, useful for sorting
     */
    Position.compare = function (a, b) {
        var aLineNumber = a.lineNumber | 0;
        var bLineNumber = b.lineNumber | 0;
        if (aLineNumber === bLineNumber) {
            var aColumn = a.column | 0;
            var bColumn = b.column | 0;
            return aColumn - bColumn;
        }
        return aLineNumber - bLineNumber;
    };
    /**
     * Clone this position.
     */
    Position.prototype.clone = function () {
        return new Position(this.lineNumber, this.column);
    };
    /**
     * Convert to a human-readable representation.
     */
    Position.prototype.toString = function () {
        return '(' + this.lineNumber + ',' + this.column + ')';
    };
    // ---
    /**
     * Create a `Position` from an `IPosition`.
     */
    Position.lift = function (pos) {
        return new Position(pos.lineNumber, pos.column);
    };
    /**
     * Test if `obj` is an `IPosition`.
     */
    Position.isIPosition = function (obj) {
        return (obj
            && (typeof obj.lineNumber === 'number')
            && (typeof obj.column === 'number'));
    };
    return Position;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IModelService = createDecorator('modelService');
function shouldSynchronizeModel(model) {
    return (!model.isTooLargeForSyncing() && !model.isForSimpleWidget);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ITextModelService = createDecorator('textModelService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$3 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Action = /** @class */ (function () {
    function Action(id, label, cssClass, enabled, actionCallback) {
        if (label === void 0) { label = ''; }
        if (cssClass === void 0) { cssClass = ''; }
        if (enabled === void 0) { enabled = true; }
        this._onDidChange = new Emitter();
        this.onDidChange = this._onDidChange.event;
        this._id = id;
        this._label = label;
        this._cssClass = cssClass;
        this._enabled = enabled;
        this._actionCallback = actionCallback;
    }
    Object.defineProperty(Action.prototype, "id", {
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Action.prototype, "label", {
        get: function () {
            return this._label;
        },
        set: function (value) {
            this._setLabel(value);
        },
        enumerable: true,
        configurable: true
    });
    Action.prototype._setLabel = function (value) {
        if (this._label !== value) {
            this._label = value;
            this._onDidChange.fire({ label: value });
        }
    };
    Object.defineProperty(Action.prototype, "tooltip", {
        get: function () {
            return this._tooltip;
        },
        set: function (value) {
            this._setTooltip(value);
        },
        enumerable: true,
        configurable: true
    });
    Action.prototype._setTooltip = function (value) {
        if (this._tooltip !== value) {
            this._tooltip = value;
            this._onDidChange.fire({ tooltip: value });
        }
    };
    Object.defineProperty(Action.prototype, "class", {
        get: function () {
            return this._cssClass;
        },
        set: function (value) {
            this._setClass(value);
        },
        enumerable: true,
        configurable: true
    });
    Action.prototype._setClass = function (value) {
        if (this._cssClass !== value) {
            this._cssClass = value;
            this._onDidChange.fire({ class: value });
        }
    };
    Object.defineProperty(Action.prototype, "enabled", {
        get: function () {
            return this._enabled;
        },
        set: function (value) {
            this._setEnabled(value);
        },
        enumerable: true,
        configurable: true
    });
    Action.prototype._setEnabled = function (value) {
        if (this._enabled !== value) {
            this._enabled = value;
            this._onDidChange.fire({ enabled: value });
        }
    };
    Object.defineProperty(Action.prototype, "checked", {
        get: function () {
            return this._checked;
        },
        set: function (value) {
            this._setChecked(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Action.prototype, "radio", {
        get: function () {
            return this._radio;
        },
        set: function (value) {
            this._setRadio(value);
        },
        enumerable: true,
        configurable: true
    });
    Action.prototype._setChecked = function (value) {
        if (this._checked !== value) {
            this._checked = value;
            this._onDidChange.fire({ checked: value });
        }
    };
    Action.prototype._setRadio = function (value) {
        if (this._radio !== value) {
            this._radio = value;
            this._onDidChange.fire({ radio: value });
        }
    };
    Action.prototype.run = function (event, _data) {
        if (this._actionCallback) {
            return this._actionCallback(event);
        }
        return Promise.resolve(true);
    };
    Action.prototype.dispose = function () {
        this._onDidChange.dispose();
    };
    return Action;
}());
var ActionRunner = /** @class */ (function (_super) {
    __extends$3(ActionRunner, _super);
    function ActionRunner() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onDidBeforeRun = _this._register(new Emitter());
        _this.onDidBeforeRun = _this._onDidBeforeRun.event;
        _this._onDidRun = _this._register(new Emitter());
        _this.onDidRun = _this._onDidRun.event;
        return _this;
    }
    ActionRunner.prototype.run = function (action, context) {
        var _this = this;
        if (!action.enabled) {
            return Promise.resolve(null);
        }
        this._onDidBeforeRun.fire({ action: action });
        return this.runAction(action, context).then(function (result) {
            _this._onDidRun.fire({ action: action, result: result });
        }, function (error) {
            _this._onDidRun.fire({ action: action, error: error });
        });
    };
    ActionRunner.prototype.runAction = function (action, context) {
        var res = context ? action.run(context) : action.run();
        return Promise.resolve(res);
    };
    return ActionRunner;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * The empty string.
 */
var empty = '';
function isFalsyOrWhitespace(str) {
    if (!str || typeof str !== 'string') {
        return true;
    }
    return str.trim().length === 0;
}
/**
 * @returns the provided number with the given number of preceding zeros.
 */
function pad(n, l, char) {
    if (char === void 0) { char = '0'; }
    var str = '' + n;
    var r = [str];
    for (var i = str.length; i < l; i++) {
        r.push(char);
    }
    return r.reverse().join('');
}
var _formatRegexp = /{(\d+)}/g;
/**
 * Helper to produce a string with a variable number of arguments. Insert variable segments
 * into the string using the {n} notation where N is the index of the argument following the string.
 * @param value string to which formatting is applied
 * @param args replacements for {n}-entries
 */
function format(value) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    if (args.length === 0) {
        return value;
    }
    return value.replace(_formatRegexp, function (match, group) {
        var idx = parseInt(group, 10);
        return isNaN(idx) || idx < 0 || idx >= args.length ?
            match :
            args[idx];
    });
}
/**
 * Converts HTML characters inside the string to use entities instead. Makes the string safe from
 * being used e.g. in HTMLElement.innerHTML.
 */
function escape(html) {
    return html.replace(/[<|>|&]/g, function (match) {
        switch (match) {
            case '<': return '&lt;';
            case '>': return '&gt;';
            case '&': return '&amp;';
            default: return match;
        }
    });
}
/**
 * Escapes regular expression characters in a given string
 */
function escapeRegExpCharacters(value) {
    return value.replace(/[\-\\\{\}\*\+\?\|\^\$\.\[\]\(\)\#]/g, '\\$&');
}
/**
 * Removes all occurrences of needle from the beginning and end of haystack.
 * @param haystack string to trim
 * @param needle the thing to trim (default is a blank)
 */
function trim(haystack, needle) {
    if (needle === void 0) { needle = ' '; }
    var trimmed = ltrim(haystack, needle);
    return rtrim(trimmed, needle);
}
/**
 * Removes all occurrences of needle from the beginning of haystack.
 * @param haystack string to trim
 * @param needle the thing to trim
 */
function ltrim(haystack, needle) {
    if (!haystack || !needle) {
        return haystack;
    }
    var needleLen = needle.length;
    if (needleLen === 0 || haystack.length === 0) {
        return haystack;
    }
    var offset = 0;
    while (haystack.indexOf(needle, offset) === offset) {
        offset = offset + needleLen;
    }
    return haystack.substring(offset);
}
/**
 * Removes all occurrences of needle from the end of haystack.
 * @param haystack string to trim
 * @param needle the thing to trim
 */
function rtrim(haystack, needle) {
    if (!haystack || !needle) {
        return haystack;
    }
    var needleLen = needle.length, haystackLen = haystack.length;
    if (needleLen === 0 || haystackLen === 0) {
        return haystack;
    }
    var offset = haystackLen, idx = -1;
    while (true) {
        idx = haystack.lastIndexOf(needle, offset - 1);
        if (idx === -1 || idx + needleLen !== offset) {
            break;
        }
        if (idx === 0) {
            return '';
        }
        offset = idx;
    }
    return haystack.substring(0, offset);
}
function convertSimple2RegExpPattern(pattern) {
    return pattern.replace(/[\-\\\{\}\+\?\|\^\$\.\,\[\]\(\)\#\s]/g, '\\$&').replace(/[\*]/g, '.*');
}
/**
 * Determines if haystack starts with needle.
 */
function startsWith(haystack, needle) {
    if (haystack.length < needle.length) {
        return false;
    }
    if (haystack === needle) {
        return true;
    }
    for (var i = 0; i < needle.length; i++) {
        if (haystack[i] !== needle[i]) {
            return false;
        }
    }
    return true;
}
/**
 * Determines if haystack ends with needle.
 */
function endsWith(haystack, needle) {
    var diff = haystack.length - needle.length;
    if (diff > 0) {
        return haystack.indexOf(needle, diff) === diff;
    }
    else if (diff === 0) {
        return haystack === needle;
    }
    else {
        return false;
    }
}
function createRegExp(searchString, isRegex, options) {
    if (options === void 0) { options = {}; }
    if (!searchString) {
        throw new Error('Cannot create regex from empty string');
    }
    if (!isRegex) {
        searchString = escapeRegExpCharacters(searchString);
    }
    if (options.wholeWord) {
        if (!/\B/.test(searchString.charAt(0))) {
            searchString = '\\b' + searchString;
        }
        if (!/\B/.test(searchString.charAt(searchString.length - 1))) {
            searchString = searchString + '\\b';
        }
    }
    var modifiers = '';
    if (options.global) {
        modifiers += 'g';
    }
    if (!options.matchCase) {
        modifiers += 'i';
    }
    if (options.multiline) {
        modifiers += 'm';
    }
    return new RegExp(searchString, modifiers);
}
function regExpLeadsToEndlessLoop(regexp) {
    // Exit early if it's one of these special cases which are meant to match
    // against an empty string
    if (regexp.source === '^' || regexp.source === '^$' || regexp.source === '$' || regexp.source === '^\\s*$') {
        return false;
    }
    // We check against an empty string. If the regular expression doesn't advance
    // (e.g. ends in an endless loop) it will match an empty string.
    var match = regexp.exec('');
    return !!(match && regexp.lastIndex === 0);
}
/**
 * Returns first index of the string that is not whitespace.
 * If string is empty or contains only whitespaces, returns -1
 */
function firstNonWhitespaceIndex(str) {
    for (var i = 0, len = str.length; i < len; i++) {
        var chCode = str.charCodeAt(i);
        if (chCode !== 32 /* Space */ && chCode !== 9 /* Tab */) {
            return i;
        }
    }
    return -1;
}
/**
 * Returns the leading whitespace of the string.
 * If the string contains only whitespaces, returns entire string
 */
function getLeadingWhitespace(str, start, end) {
    if (start === void 0) { start = 0; }
    if (end === void 0) { end = str.length; }
    for (var i = start; i < end; i++) {
        var chCode = str.charCodeAt(i);
        if (chCode !== 32 /* Space */ && chCode !== 9 /* Tab */) {
            return str.substring(start, i);
        }
    }
    return str.substring(start, end);
}
/**
 * Returns last index of the string that is not whitespace.
 * If string is empty or contains only whitespaces, returns -1
 */
function lastNonWhitespaceIndex(str, startIndex) {
    if (startIndex === void 0) { startIndex = str.length - 1; }
    for (var i = startIndex; i >= 0; i--) {
        var chCode = str.charCodeAt(i);
        if (chCode !== 32 /* Space */ && chCode !== 9 /* Tab */) {
            return i;
        }
    }
    return -1;
}
function compare(a, b) {
    if (a < b) {
        return -1;
    }
    else if (a > b) {
        return 1;
    }
    else {
        return 0;
    }
}
function isLowerAsciiLetter(code) {
    return code >= 97 /* a */ && code <= 122 /* z */;
}
function isUpperAsciiLetter(code) {
    return code >= 65 /* A */ && code <= 90 /* Z */;
}
function isAsciiLetter(code) {
    return isLowerAsciiLetter(code) || isUpperAsciiLetter(code);
}
function equalsIgnoreCase(a, b) {
    var len1 = a ? a.length : 0;
    var len2 = b ? b.length : 0;
    if (len1 !== len2) {
        return false;
    }
    return doEqualsIgnoreCase(a, b);
}
function doEqualsIgnoreCase(a, b, stopAt) {
    if (stopAt === void 0) { stopAt = a.length; }
    if (typeof a !== 'string' || typeof b !== 'string') {
        return false;
    }
    for (var i = 0; i < stopAt; i++) {
        var codeA = a.charCodeAt(i);
        var codeB = b.charCodeAt(i);
        if (codeA === codeB) {
            continue;
        }
        // a-z A-Z
        if (isAsciiLetter(codeA) && isAsciiLetter(codeB)) {
            var diff = Math.abs(codeA - codeB);
            if (diff !== 0 && diff !== 32) {
                return false;
            }
        }
        // Any other charcode
        else {
            if (String.fromCharCode(codeA).toLowerCase() !== String.fromCharCode(codeB).toLowerCase()) {
                return false;
            }
        }
    }
    return true;
}
function startsWithIgnoreCase(str, candidate) {
    var candidateLength = candidate.length;
    if (candidate.length > str.length) {
        return false;
    }
    return doEqualsIgnoreCase(str, candidate, candidateLength);
}
/**
 * @returns the length of the common prefix of the two strings.
 */
function commonPrefixLength(a, b) {
    var i, len = Math.min(a.length, b.length);
    for (i = 0; i < len; i++) {
        if (a.charCodeAt(i) !== b.charCodeAt(i)) {
            return i;
        }
    }
    return len;
}
/**
 * @returns the length of the common suffix of the two strings.
 */
function commonSuffixLength(a, b) {
    var i, len = Math.min(a.length, b.length);
    var aLastIndex = a.length - 1;
    var bLastIndex = b.length - 1;
    for (i = 0; i < len; i++) {
        if (a.charCodeAt(aLastIndex - i) !== b.charCodeAt(bLastIndex - i)) {
            return i;
        }
    }
    return len;
}
// --- unicode
// http://en.wikipedia.org/wiki/Surrogate_pair
// Returns the code point starting at a specified index in a string
// Code points U+0000 to U+D7FF and U+E000 to U+FFFF are represented on a single character
// Code points U+10000 to U+10FFFF are represented on two consecutive characters
//export function getUnicodePoint(str:string, index:number, len:number):number {
//	let chrCode = str.charCodeAt(index);
//	if (0xD800 <= chrCode && chrCode <= 0xDBFF && index + 1 < len) {
//		let nextChrCode = str.charCodeAt(index + 1);
//		if (0xDC00 <= nextChrCode && nextChrCode <= 0xDFFF) {
//			return (chrCode - 0xD800) << 10 + (nextChrCode - 0xDC00) + 0x10000;
//		}
//	}
//	return chrCode;
//}
function isHighSurrogate(charCode) {
    return (0xD800 <= charCode && charCode <= 0xDBFF);
}
function isLowSurrogate(charCode) {
    return (0xDC00 <= charCode && charCode <= 0xDFFF);
}
/**
 * Generated using https://github.com/alexandrudima/unicode-utils/blob/master/generate-rtl-test.js
 */
var CONTAINS_RTL = /(?:[\u05BE\u05C0\u05C3\u05C6\u05D0-\u05F4\u0608\u060B\u060D\u061B-\u064A\u066D-\u066F\u0671-\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u0710\u0712-\u072F\u074D-\u07A5\u07B1-\u07EA\u07F4\u07F5\u07FA-\u0815\u081A\u0824\u0828\u0830-\u0858\u085E-\u08BD\u200F\uFB1D\uFB1F-\uFB28\uFB2A-\uFD3D\uFD50-\uFDFC\uFE70-\uFEFC]|\uD802[\uDC00-\uDD1B\uDD20-\uDE00\uDE10-\uDE33\uDE40-\uDEE4\uDEEB-\uDF35\uDF40-\uDFFF]|\uD803[\uDC00-\uDCFF]|\uD83A[\uDC00-\uDCCF\uDD00-\uDD43\uDD50-\uDFFF]|\uD83B[\uDC00-\uDEBB])/;
/**
 * Returns true if `str` contains any Unicode character that is classified as "R" or "AL".
 */
function containsRTL(str) {
    return CONTAINS_RTL.test(str);
}
/**
 * Generated using https://github.com/alexandrudima/unicode-utils/blob/master/generate-emoji-test.js
 */
var CONTAINS_EMOJI = /(?:[\u231A\u231B\u23F0\u23F3\u2600-\u27BF\u2B50\u2B55]|\uD83C[\uDDE6-\uDDFF\uDF00-\uDFFF]|\uD83D[\uDC00-\uDE4F\uDE80-\uDEF8]|\uD83E[\uDD00-\uDDE6])/;
function containsEmoji(str) {
    return CONTAINS_EMOJI.test(str);
}
var IS_BASIC_ASCII = /^[\t\n\r\x20-\x7E]*$/;
/**
 * Returns true if `str` contains only basic ASCII characters in the range 32 - 126 (including 32 and 126) or \n, \r, \t
 */
function isBasicASCII(str) {
    return IS_BASIC_ASCII.test(str);
}
function containsFullWidthCharacter(str) {
    for (var i = 0, len = str.length; i < len; i++) {
        if (isFullWidthCharacter(str.charCodeAt(i))) {
            return true;
        }
    }
    return false;
}
function isFullWidthCharacter(charCode) {
    // Do a cheap trick to better support wrapping of wide characters, treat them as 2 columns
    // http://jrgraphix.net/research/unicode_blocks.php
    //          2E80 — 2EFF   CJK Radicals Supplement
    //          2F00 — 2FDF   Kangxi Radicals
    //          2FF0 — 2FFF   Ideographic Description Characters
    //          3000 — 303F   CJK Symbols and Punctuation
    //          3040 — 309F   Hiragana
    //          30A0 — 30FF   Katakana
    //          3100 — 312F   Bopomofo
    //          3130 — 318F   Hangul Compatibility Jamo
    //          3190 — 319F   Kanbun
    //          31A0 — 31BF   Bopomofo Extended
    //          31F0 — 31FF   Katakana Phonetic Extensions
    //          3200 — 32FF   Enclosed CJK Letters and Months
    //          3300 — 33FF   CJK Compatibility
    //          3400 — 4DBF   CJK Unified Ideographs Extension A
    //          4DC0 — 4DFF   Yijing Hexagram Symbols
    //          4E00 — 9FFF   CJK Unified Ideographs
    //          A000 — A48F   Yi Syllables
    //          A490 — A4CF   Yi Radicals
    //          AC00 — D7AF   Hangul Syllables
    // [IGNORE] D800 — DB7F   High Surrogates
    // [IGNORE] DB80 — DBFF   High Private Use Surrogates
    // [IGNORE] DC00 — DFFF   Low Surrogates
    // [IGNORE] E000 — F8FF   Private Use Area
    //          F900 — FAFF   CJK Compatibility Ideographs
    // [IGNORE] FB00 — FB4F   Alphabetic Presentation Forms
    // [IGNORE] FB50 — FDFF   Arabic Presentation Forms-A
    // [IGNORE] FE00 — FE0F   Variation Selectors
    // [IGNORE] FE20 — FE2F   Combining Half Marks
    // [IGNORE] FE30 — FE4F   CJK Compatibility Forms
    // [IGNORE] FE50 — FE6F   Small Form Variants
    // [IGNORE] FE70 — FEFF   Arabic Presentation Forms-B
    //          FF00 — FFEF   Halfwidth and Fullwidth Forms
    //               [https://en.wikipedia.org/wiki/Halfwidth_and_fullwidth_forms]
    //               of which FF01 - FF5E fullwidth ASCII of 21 to 7E
    // [IGNORE]    and FF65 - FFDC halfwidth of Katakana and Hangul
    // [IGNORE] FFF0 — FFFF   Specials
    charCode = +charCode; // @perf
    return ((charCode >= 0x2E80 && charCode <= 0xD7AF)
        || (charCode >= 0xF900 && charCode <= 0xFAFF)
        || (charCode >= 0xFF01 && charCode <= 0xFF5E));
}
// -- UTF-8 BOM
var UTF8_BOM_CHARACTER = String.fromCharCode(65279 /* UTF8_BOM */);
function startsWithUTF8BOM(str) {
    return !!(str && str.length > 0 && str.charCodeAt(0) === 65279 /* UTF8_BOM */);
}
function safeBtoa(str) {
    return btoa(encodeURIComponent(str)); // we use encodeURIComponent because btoa fails for non Latin 1 values
}
function repeat(s, count) {
    var result = '';
    for (var i = 0; i < count; i++) {
        result += s;
    }
    return result;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$4 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ContextKeyExpr = /** @class */ (function () {
    function ContextKeyExpr() {
    }
    ContextKeyExpr.has = function (key) {
        return new ContextKeyDefinedExpr(key);
    };
    ContextKeyExpr.equals = function (key, value) {
        return new ContextKeyEqualsExpr(key, value);
    };
    ContextKeyExpr.regex = function (key, value) {
        return new ContextKeyRegexExpr(key, value);
    };
    ContextKeyExpr.not = function (key) {
        return new ContextKeyNotExpr(key);
    };
    ContextKeyExpr.and = function () {
        var expr = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            expr[_i] = arguments[_i];
        }
        return new ContextKeyAndExpr(expr);
    };
    ContextKeyExpr.deserialize = function (serialized) {
        var _this = this;
        if (!serialized) {
            return null;
        }
        var pieces = serialized.split('&&');
        var result = new ContextKeyAndExpr(pieces.map(function (p) { return _this._deserializeOne(p); }));
        return result.normalize();
    };
    ContextKeyExpr._deserializeOne = function (serializedOne) {
        serializedOne = serializedOne.trim();
        if (serializedOne.indexOf('!=') >= 0) {
            var pieces = serializedOne.split('!=');
            return new ContextKeyNotEqualsExpr(pieces[0].trim(), this._deserializeValue(pieces[1]));
        }
        if (serializedOne.indexOf('==') >= 0) {
            var pieces = serializedOne.split('==');
            return new ContextKeyEqualsExpr(pieces[0].trim(), this._deserializeValue(pieces[1]));
        }
        if (serializedOne.indexOf('=~') >= 0) {
            var pieces = serializedOne.split('=~');
            return new ContextKeyRegexExpr(pieces[0].trim(), this._deserializeRegexValue(pieces[1]));
        }
        if (/^\!\s*/.test(serializedOne)) {
            return new ContextKeyNotExpr(serializedOne.substr(1).trim());
        }
        return new ContextKeyDefinedExpr(serializedOne);
    };
    ContextKeyExpr._deserializeValue = function (serializedValue) {
        serializedValue = serializedValue.trim();
        if (serializedValue === 'true') {
            return true;
        }
        if (serializedValue === 'false') {
            return false;
        }
        var m = /^'([^']*)'$/.exec(serializedValue);
        if (m) {
            return m[1].trim();
        }
        return serializedValue;
    };
    ContextKeyExpr._deserializeRegexValue = function (serializedValue) {
        if (isFalsyOrWhitespace(serializedValue)) {
            console.warn('missing regexp-value for =~-expression');
            return null;
        }
        var start = serializedValue.indexOf('/');
        var end = serializedValue.lastIndexOf('/');
        if (start === end || start < 0 /* || to < 0 */) {
            console.warn("bad regexp-value '" + serializedValue + "', missing /-enclosure");
            return null;
        }
        var value = serializedValue.slice(start + 1, end);
        var caseIgnoreFlag = serializedValue[end + 1] === 'i' ? 'i' : '';
        try {
            return new RegExp(value, caseIgnoreFlag);
        }
        catch (e) {
            console.warn("bad regexp-value '" + serializedValue + "', parse error: " + e);
            return null;
        }
    };
    return ContextKeyExpr;
}());
function cmp(a, b) {
    var aType = a.getType();
    var bType = b.getType();
    if (aType !== bType) {
        return aType - bType;
    }
    switch (aType) {
        case 1 /* Defined */:
            return a.cmp(b);
        case 2 /* Not */:
            return a.cmp(b);
        case 3 /* Equals */:
            return a.cmp(b);
        case 4 /* NotEquals */:
            return a.cmp(b);
        case 6 /* Regex */:
            return a.cmp(b);
        default:
            throw new Error('Unknown ContextKeyExpr!');
    }
}
var ContextKeyDefinedExpr = /** @class */ (function () {
    function ContextKeyDefinedExpr(key) {
        this.key = key;
    }
    ContextKeyDefinedExpr.prototype.getType = function () {
        return 1 /* Defined */;
    };
    ContextKeyDefinedExpr.prototype.cmp = function (other) {
        if (this.key < other.key) {
            return -1;
        }
        if (this.key > other.key) {
            return 1;
        }
        return 0;
    };
    ContextKeyDefinedExpr.prototype.equals = function (other) {
        if (other instanceof ContextKeyDefinedExpr) {
            return (this.key === other.key);
        }
        return false;
    };
    ContextKeyDefinedExpr.prototype.evaluate = function (context) {
        return (!!context.getValue(this.key));
    };
    ContextKeyDefinedExpr.prototype.normalize = function () {
        return this;
    };
    ContextKeyDefinedExpr.prototype.keys = function () {
        return [this.key];
    };
    return ContextKeyDefinedExpr;
}());
var ContextKeyEqualsExpr = /** @class */ (function () {
    function ContextKeyEqualsExpr(key, value) {
        this.key = key;
        this.value = value;
    }
    ContextKeyEqualsExpr.prototype.getType = function () {
        return 3 /* Equals */;
    };
    ContextKeyEqualsExpr.prototype.cmp = function (other) {
        if (this.key < other.key) {
            return -1;
        }
        if (this.key > other.key) {
            return 1;
        }
        if (this.value < other.value) {
            return -1;
        }
        if (this.value > other.value) {
            return 1;
        }
        return 0;
    };
    ContextKeyEqualsExpr.prototype.equals = function (other) {
        if (other instanceof ContextKeyEqualsExpr) {
            return (this.key === other.key && this.value === other.value);
        }
        return false;
    };
    ContextKeyEqualsExpr.prototype.evaluate = function (context) {
        /* tslint:disable:triple-equals */
        // Intentional ==
        return (context.getValue(this.key) == this.value);
        /* tslint:enable:triple-equals */
    };
    ContextKeyEqualsExpr.prototype.normalize = function () {
        if (typeof this.value === 'boolean') {
            if (this.value) {
                return new ContextKeyDefinedExpr(this.key);
            }
            return new ContextKeyNotExpr(this.key);
        }
        return this;
    };
    ContextKeyEqualsExpr.prototype.keys = function () {
        return [this.key];
    };
    return ContextKeyEqualsExpr;
}());
var ContextKeyNotEqualsExpr = /** @class */ (function () {
    function ContextKeyNotEqualsExpr(key, value) {
        this.key = key;
        this.value = value;
    }
    ContextKeyNotEqualsExpr.prototype.getType = function () {
        return 4 /* NotEquals */;
    };
    ContextKeyNotEqualsExpr.prototype.cmp = function (other) {
        if (this.key < other.key) {
            return -1;
        }
        if (this.key > other.key) {
            return 1;
        }
        if (this.value < other.value) {
            return -1;
        }
        if (this.value > other.value) {
            return 1;
        }
        return 0;
    };
    ContextKeyNotEqualsExpr.prototype.equals = function (other) {
        if (other instanceof ContextKeyNotEqualsExpr) {
            return (this.key === other.key && this.value === other.value);
        }
        return false;
    };
    ContextKeyNotEqualsExpr.prototype.evaluate = function (context) {
        /* tslint:disable:triple-equals */
        // Intentional !=
        return (context.getValue(this.key) != this.value);
        /* tslint:enable:triple-equals */
    };
    ContextKeyNotEqualsExpr.prototype.normalize = function () {
        if (typeof this.value === 'boolean') {
            if (this.value) {
                return new ContextKeyNotExpr(this.key);
            }
            return new ContextKeyDefinedExpr(this.key);
        }
        return this;
    };
    ContextKeyNotEqualsExpr.prototype.keys = function () {
        return [this.key];
    };
    return ContextKeyNotEqualsExpr;
}());
var ContextKeyNotExpr = /** @class */ (function () {
    function ContextKeyNotExpr(key) {
        this.key = key;
    }
    ContextKeyNotExpr.prototype.getType = function () {
        return 2 /* Not */;
    };
    ContextKeyNotExpr.prototype.cmp = function (other) {
        if (this.key < other.key) {
            return -1;
        }
        if (this.key > other.key) {
            return 1;
        }
        return 0;
    };
    ContextKeyNotExpr.prototype.equals = function (other) {
        if (other instanceof ContextKeyNotExpr) {
            return (this.key === other.key);
        }
        return false;
    };
    ContextKeyNotExpr.prototype.evaluate = function (context) {
        return (!context.getValue(this.key));
    };
    ContextKeyNotExpr.prototype.normalize = function () {
        return this;
    };
    ContextKeyNotExpr.prototype.keys = function () {
        return [this.key];
    };
    return ContextKeyNotExpr;
}());
var ContextKeyRegexExpr = /** @class */ (function () {
    function ContextKeyRegexExpr(key, regexp) {
        this.key = key;
        this.regexp = regexp;
        //
    }
    ContextKeyRegexExpr.prototype.getType = function () {
        return 6 /* Regex */;
    };
    ContextKeyRegexExpr.prototype.cmp = function (other) {
        if (this.key < other.key) {
            return -1;
        }
        if (this.key > other.key) {
            return 1;
        }
        var thisSource = this.regexp ? this.regexp.source : '';
        var otherSource = other.regexp ? other.regexp.source : '';
        if (thisSource < otherSource) {
            return -1;
        }
        if (thisSource > otherSource) {
            return 1;
        }
        return 0;
    };
    ContextKeyRegexExpr.prototype.equals = function (other) {
        if (other instanceof ContextKeyRegexExpr) {
            var thisSource = this.regexp ? this.regexp.source : '';
            var otherSource = other.regexp ? other.regexp.source : '';
            return (this.key === other.key && thisSource === otherSource);
        }
        return false;
    };
    ContextKeyRegexExpr.prototype.evaluate = function (context) {
        var value = context.getValue(this.key);
        return this.regexp ? this.regexp.test(value) : false;
    };
    ContextKeyRegexExpr.prototype.normalize = function () {
        return this;
    };
    ContextKeyRegexExpr.prototype.keys = function () {
        return [this.key];
    };
    return ContextKeyRegexExpr;
}());
var ContextKeyAndExpr = /** @class */ (function () {
    function ContextKeyAndExpr(expr) {
        this.expr = ContextKeyAndExpr._normalizeArr(expr);
    }
    ContextKeyAndExpr.prototype.getType = function () {
        return 5 /* And */;
    };
    ContextKeyAndExpr.prototype.equals = function (other) {
        if (other instanceof ContextKeyAndExpr) {
            if (this.expr.length !== other.expr.length) {
                return false;
            }
            for (var i = 0, len = this.expr.length; i < len; i++) {
                if (!this.expr[i].equals(other.expr[i])) {
                    return false;
                }
            }
            return true;
        }
        return false;
    };
    ContextKeyAndExpr.prototype.evaluate = function (context) {
        for (var i = 0, len = this.expr.length; i < len; i++) {
            if (!this.expr[i].evaluate(context)) {
                return false;
            }
        }
        return true;
    };
    ContextKeyAndExpr._normalizeArr = function (arr) {
        var expr = [];
        if (arr) {
            for (var i = 0, len = arr.length; i < len; i++) {
                var e = arr[i];
                if (!e) {
                    continue;
                }
                e = e.normalize();
                if (!e) {
                    continue;
                }
                if (e instanceof ContextKeyAndExpr) {
                    expr = expr.concat(e.expr);
                    continue;
                }
                expr.push(e);
            }
            expr.sort(cmp);
        }
        return expr;
    };
    ContextKeyAndExpr.prototype.normalize = function () {
        if (this.expr.length === 0) {
            return null;
        }
        if (this.expr.length === 1) {
            return this.expr[0];
        }
        return this;
    };
    ContextKeyAndExpr.prototype.keys = function () {
        var result = [];
        for (var _i = 0, _a = this.expr; _i < _a.length; _i++) {
            var expr = _a[_i];
            result.push.apply(result, expr.keys());
        }
        return result;
    };
    return ContextKeyAndExpr;
}());
var RawContextKey = /** @class */ (function (_super) {
    __extends$4(RawContextKey, _super);
    function RawContextKey(key, defaultValue) {
        var _this = _super.call(this, key) || this;
        _this._defaultValue = defaultValue;
        return _this;
    }
    RawContextKey.prototype.bindTo = function (target) {
        return target.createKey(this.key, this._defaultValue);
    };
    RawContextKey.prototype.getValue = function (target) {
        return target.getContextKeyValue(this.key);
    };
    RawContextKey.prototype.toNegated = function () {
        return ContextKeyExpr.not(this.key);
    };
    return RawContextKey;
}(ContextKeyDefinedExpr));
var IContextKeyService = createDecorator('contextKeyService');
var SET_CONTEXT_COMMAND_ID = 'setContext';

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ICommandService = createDecorator('commandService');
var CommandsRegistry = new /** @class */ (function () {
    function class_1() {
        this._commands = new Map();
    }
    class_1.prototype.registerCommand = function (idOrCommand, handler) {
        var _this = this;
        if (!idOrCommand) {
            throw new Error("invalid command");
        }
        if (typeof idOrCommand === 'string') {
            if (!handler) {
                throw new Error("invalid command");
            }
            return this.registerCommand({ id: idOrCommand, handler: handler });
        }
        // add argument validation if rich command metadata is provided
        if (idOrCommand.description) {
            var constraints_1 = [];
            for (var _i = 0, _a = idOrCommand.description.args; _i < _a.length; _i++) {
                var arg = _a[_i];
                constraints_1.push(arg.constraint);
            }
            var actualHandler_1 = idOrCommand.handler;
            idOrCommand.handler = function (accessor) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                validateConstraints(args, constraints_1);
                return actualHandler_1.apply(void 0, [accessor].concat(args));
            };
        }
        // find a place to store the command
        var id = idOrCommand.id;
        var commands = this._commands.get(id);
        if (!commands) {
            commands = new LinkedList();
            this._commands.set(id, commands);
        }
        var removeFn = commands.unshift(idOrCommand);
        return toDisposable(function () {
            removeFn();
            if (_this._commands.get(id).isEmpty()) {
                _this._commands.delete(id);
            }
        });
    };
    class_1.prototype.registerCommandAlias = function (oldId, newId) {
        return CommandsRegistry.registerCommand(oldId, function (accessor) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var _a;
            (_a = accessor.get(ICommandService)).executeCommand.apply(_a, [newId].concat(args));
        });
    };
    class_1.prototype.getCommand = function (id) {
        var list = this._commands.get(id);
        if (!list || list.isEmpty()) {
            return undefined;
        }
        return list.iterator().next().value;
    };
    class_1.prototype.getCommands = function () {
        var _this = this;
        var result = Object.create(null);
        this._commands.forEach(function (value, key) {
            result[key] = _this.getCommand(key);
        });
        return result;
    };
    return class_1;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$5 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
function isIMenuItem(item) {
    return item.command !== undefined;
}
var MenuId = /** @class */ (function () {
    function MenuId() {
        this.id = String(MenuId.ID++);
    }
    MenuId.ID = 1;
    MenuId.EditorContext = new MenuId();
    MenuId.CommandPalette = new MenuId();
    MenuId.MenubarEditMenu = new MenuId();
    MenuId.MenubarSelectionMenu = new MenuId();
    MenuId.MenubarGoMenu = new MenuId();
    return MenuId;
}());
var IMenuService = createDecorator('menuService');
var MenuRegistry = new /** @class */ (function () {
    function class_1() {
        this._commands = Object.create(null);
        this._menuItems = Object.create(null);
    }
    class_1.prototype.addCommand = function (command) {
        var old = this._commands[command.id];
        this._commands[command.id] = command;
        return old !== void 0;
    };
    class_1.prototype.getCommand = function (id) {
        return this._commands[id];
    };
    class_1.prototype.getCommands = function () {
        var result = Object.create(null);
        for (var key in this._commands) {
            result[key] = this.getCommand(key);
        }
        return result;
    };
    class_1.prototype.appendMenuItem = function (_a, item) {
        var id = _a.id;
        var array = this._menuItems[id];
        if (!array) {
            this._menuItems[id] = array = [item];
        }
        else {
            array.push(item);
        }
        return {
            dispose: function () {
                var idx = array.indexOf(item);
                if (idx >= 0) {
                    array.splice(idx, 1);
                }
            }
        };
    };
    class_1.prototype.getMenuItems = function (_a) {
        var id = _a.id;
        var result = this._menuItems[id] || [];
        if (id === MenuId.CommandPalette.id) {
            // CommandPalette is special because it shows
            // all commands by default
            this._appendImplicitItems(result);
        }
        return result;
    };
    class_1.prototype._appendImplicitItems = function (result) {
        var set = new Set();
        var temp = result.filter(function (item) { return isIMenuItem(item); });
        for (var _i = 0, temp_1 = temp; _i < temp_1.length; _i++) {
            var _a = temp_1[_i], command = _a.command, alt = _a.alt;
            set.add(command.id);
            if (alt) {
                set.add(alt.id);
            }
        }
        for (var id in this._commands) {
            if (!set.has(id)) {
                result.push({ command: this._commands[id] });
            }
        }
    };
    return class_1;
}());
var ExecuteCommandAction = /** @class */ (function (_super) {
    __extends$5(ExecuteCommandAction, _super);
    function ExecuteCommandAction(id, label, _commandService) {
        var _this = _super.call(this, id, label) || this;
        _this._commandService = _commandService;
        return _this;
    }
    ExecuteCommandAction.prototype.run = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _a;
        return (_a = this._commandService).executeCommand.apply(_a, [this.id].concat(args));
    };
    ExecuteCommandAction = __decorate([
        __param(2, ICommandService)
    ], ExecuteCommandAction);
    return ExecuteCommandAction;
}(Action));
var SubmenuItemAction = /** @class */ (function (_super) {
    __extends$5(SubmenuItemAction, _super);
    function SubmenuItemAction(item) {
        var _this = this;
        typeof item.title === 'string' ? _this = _super.call(this, '', item.title, 'submenu') || this : _this = _super.call(this, '', item.title.value, 'submenu') || this;
        _this.item = item;
        return _this;
    }
    return SubmenuItemAction;
}(Action));
var MenuItemAction = /** @class */ (function (_super) {
    __extends$5(MenuItemAction, _super);
    function MenuItemAction(item, alt, options, contextKeyService, commandService) {
        var _this = this;
        typeof item.title === 'string' ? _this = _super.call(this, item.id, item.title, commandService) || this : _this = _super.call(this, item.id, item.title.value, commandService) || this;
        _this._cssClass = undefined;
        _this._enabled = !item.precondition || contextKeyService.contextMatchesRules(item.precondition);
        _this._checked = Boolean(item.toggled && contextKeyService.contextMatchesRules(item.toggled));
        _this._options = options || {};
        _this.item = item;
        _this.alt = alt ? new MenuItemAction(alt, undefined, _this._options, contextKeyService, commandService) : undefined;
        return _this;
    }
    MenuItemAction.prototype.run = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var runArgs = [];
        if (this._options.arg) {
            runArgs = runArgs.concat([this._options.arg]);
        }
        if (this._options.shouldForwardArgs) {
            runArgs = runArgs.concat(args);
        }
        return _super.prototype.run.apply(this, runArgs);
    };
    MenuItemAction = __decorate([
        __param(3, IContextKeyService),
        __param(4, ICommandService)
    ], MenuItemAction);
    return MenuItemAction;
}(ExecuteCommandAction));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var KeyCodeStrMap = /** @class */ (function () {
    function KeyCodeStrMap() {
        this._keyCodeToStr = [];
        this._strToKeyCode = Object.create(null);
    }
    KeyCodeStrMap.prototype.define = function (keyCode, str) {
        this._keyCodeToStr[keyCode] = str;
        this._strToKeyCode[str.toLowerCase()] = keyCode;
    };
    KeyCodeStrMap.prototype.keyCodeToStr = function (keyCode) {
        return this._keyCodeToStr[keyCode];
    };
    KeyCodeStrMap.prototype.strToKeyCode = function (str) {
        return this._strToKeyCode[str.toLowerCase()] || 0 /* Unknown */;
    };
    return KeyCodeStrMap;
}());
var uiMap = new KeyCodeStrMap();
var userSettingsUSMap = new KeyCodeStrMap();
var userSettingsGeneralMap = new KeyCodeStrMap();
(function () {
    function define(keyCode, uiLabel, usUserSettingsLabel, generalUserSettingsLabel) {
        if (usUserSettingsLabel === void 0) { usUserSettingsLabel = uiLabel; }
        if (generalUserSettingsLabel === void 0) { generalUserSettingsLabel = usUserSettingsLabel; }
        uiMap.define(keyCode, uiLabel);
        userSettingsUSMap.define(keyCode, usUserSettingsLabel);
        userSettingsGeneralMap.define(keyCode, generalUserSettingsLabel);
    }
    define(0 /* Unknown */, 'unknown');
    define(1 /* Backspace */, 'Backspace');
    define(2 /* Tab */, 'Tab');
    define(3 /* Enter */, 'Enter');
    define(4 /* Shift */, 'Shift');
    define(5 /* Ctrl */, 'Ctrl');
    define(6 /* Alt */, 'Alt');
    define(7 /* PauseBreak */, 'PauseBreak');
    define(8 /* CapsLock */, 'CapsLock');
    define(9 /* Escape */, 'Escape');
    define(10 /* Space */, 'Space');
    define(11 /* PageUp */, 'PageUp');
    define(12 /* PageDown */, 'PageDown');
    define(13 /* End */, 'End');
    define(14 /* Home */, 'Home');
    define(15 /* LeftArrow */, 'LeftArrow', 'Left');
    define(16 /* UpArrow */, 'UpArrow', 'Up');
    define(17 /* RightArrow */, 'RightArrow', 'Right');
    define(18 /* DownArrow */, 'DownArrow', 'Down');
    define(19 /* Insert */, 'Insert');
    define(20 /* Delete */, 'Delete');
    define(21 /* KEY_0 */, '0');
    define(22 /* KEY_1 */, '1');
    define(23 /* KEY_2 */, '2');
    define(24 /* KEY_3 */, '3');
    define(25 /* KEY_4 */, '4');
    define(26 /* KEY_5 */, '5');
    define(27 /* KEY_6 */, '6');
    define(28 /* KEY_7 */, '7');
    define(29 /* KEY_8 */, '8');
    define(30 /* KEY_9 */, '9');
    define(31 /* KEY_A */, 'A');
    define(32 /* KEY_B */, 'B');
    define(33 /* KEY_C */, 'C');
    define(34 /* KEY_D */, 'D');
    define(35 /* KEY_E */, 'E');
    define(36 /* KEY_F */, 'F');
    define(37 /* KEY_G */, 'G');
    define(38 /* KEY_H */, 'H');
    define(39 /* KEY_I */, 'I');
    define(40 /* KEY_J */, 'J');
    define(41 /* KEY_K */, 'K');
    define(42 /* KEY_L */, 'L');
    define(43 /* KEY_M */, 'M');
    define(44 /* KEY_N */, 'N');
    define(45 /* KEY_O */, 'O');
    define(46 /* KEY_P */, 'P');
    define(47 /* KEY_Q */, 'Q');
    define(48 /* KEY_R */, 'R');
    define(49 /* KEY_S */, 'S');
    define(50 /* KEY_T */, 'T');
    define(51 /* KEY_U */, 'U');
    define(52 /* KEY_V */, 'V');
    define(53 /* KEY_W */, 'W');
    define(54 /* KEY_X */, 'X');
    define(55 /* KEY_Y */, 'Y');
    define(56 /* KEY_Z */, 'Z');
    define(57 /* Meta */, 'Meta');
    define(58 /* ContextMenu */, 'ContextMenu');
    define(59 /* F1 */, 'F1');
    define(60 /* F2 */, 'F2');
    define(61 /* F3 */, 'F3');
    define(62 /* F4 */, 'F4');
    define(63 /* F5 */, 'F5');
    define(64 /* F6 */, 'F6');
    define(65 /* F7 */, 'F7');
    define(66 /* F8 */, 'F8');
    define(67 /* F9 */, 'F9');
    define(68 /* F10 */, 'F10');
    define(69 /* F11 */, 'F11');
    define(70 /* F12 */, 'F12');
    define(71 /* F13 */, 'F13');
    define(72 /* F14 */, 'F14');
    define(73 /* F15 */, 'F15');
    define(74 /* F16 */, 'F16');
    define(75 /* F17 */, 'F17');
    define(76 /* F18 */, 'F18');
    define(77 /* F19 */, 'F19');
    define(78 /* NumLock */, 'NumLock');
    define(79 /* ScrollLock */, 'ScrollLock');
    define(80 /* US_SEMICOLON */, ';', ';', 'OEM_1');
    define(81 /* US_EQUAL */, '=', '=', 'OEM_PLUS');
    define(82 /* US_COMMA */, ',', ',', 'OEM_COMMA');
    define(83 /* US_MINUS */, '-', '-', 'OEM_MINUS');
    define(84 /* US_DOT */, '.', '.', 'OEM_PERIOD');
    define(85 /* US_SLASH */, '/', '/', 'OEM_2');
    define(86 /* US_BACKTICK */, '`', '`', 'OEM_3');
    define(110 /* ABNT_C1 */, 'ABNT_C1');
    define(111 /* ABNT_C2 */, 'ABNT_C2');
    define(87 /* US_OPEN_SQUARE_BRACKET */, '[', '[', 'OEM_4');
    define(88 /* US_BACKSLASH */, '\\', '\\', 'OEM_5');
    define(89 /* US_CLOSE_SQUARE_BRACKET */, ']', ']', 'OEM_6');
    define(90 /* US_QUOTE */, '\'', '\'', 'OEM_7');
    define(91 /* OEM_8 */, 'OEM_8');
    define(92 /* OEM_102 */, 'OEM_102');
    define(93 /* NUMPAD_0 */, 'NumPad0');
    define(94 /* NUMPAD_1 */, 'NumPad1');
    define(95 /* NUMPAD_2 */, 'NumPad2');
    define(96 /* NUMPAD_3 */, 'NumPad3');
    define(97 /* NUMPAD_4 */, 'NumPad4');
    define(98 /* NUMPAD_5 */, 'NumPad5');
    define(99 /* NUMPAD_6 */, 'NumPad6');
    define(100 /* NUMPAD_7 */, 'NumPad7');
    define(101 /* NUMPAD_8 */, 'NumPad8');
    define(102 /* NUMPAD_9 */, 'NumPad9');
    define(103 /* NUMPAD_MULTIPLY */, 'NumPad_Multiply');
    define(104 /* NUMPAD_ADD */, 'NumPad_Add');
    define(105 /* NUMPAD_SEPARATOR */, 'NumPad_Separator');
    define(106 /* NUMPAD_SUBTRACT */, 'NumPad_Subtract');
    define(107 /* NUMPAD_DECIMAL */, 'NumPad_Decimal');
    define(108 /* NUMPAD_DIVIDE */, 'NumPad_Divide');
})();
var KeyCodeUtils;
(function (KeyCodeUtils) {
    function toString(keyCode) {
        return uiMap.keyCodeToStr(keyCode);
    }
    KeyCodeUtils.toString = toString;
    function fromString(key) {
        return uiMap.strToKeyCode(key);
    }
    KeyCodeUtils.fromString = fromString;
    function toUserSettingsUS(keyCode) {
        return userSettingsUSMap.keyCodeToStr(keyCode);
    }
    KeyCodeUtils.toUserSettingsUS = toUserSettingsUS;
    function toUserSettingsGeneral(keyCode) {
        return userSettingsGeneralMap.keyCodeToStr(keyCode);
    }
    KeyCodeUtils.toUserSettingsGeneral = toUserSettingsGeneral;
    function fromUserSettings(key) {
        return userSettingsUSMap.strToKeyCode(key) || userSettingsGeneralMap.strToKeyCode(key);
    }
    KeyCodeUtils.fromUserSettings = fromUserSettings;
})(KeyCodeUtils || (KeyCodeUtils = {}));
function KeyChord(firstPart, secondPart) {
    var chordPart = ((secondPart & 0x0000ffff) << 16) >>> 0;
    return (firstPart | chordPart) >>> 0;
}
function createKeybinding(keybinding, OS) {
    if (keybinding === 0) {
        return null;
    }
    var firstPart = (keybinding & 0x0000ffff) >>> 0;
    var chordPart = (keybinding & 0xffff0000) >>> 16;
    if (chordPart !== 0) {
        return new ChordKeybinding(createSimpleKeybinding(firstPart, OS), createSimpleKeybinding(chordPart, OS));
    }
    return createSimpleKeybinding(firstPart, OS);
}
function createSimpleKeybinding(keybinding, OS) {
    var ctrlCmd = (keybinding & 2048 /* CtrlCmd */ ? true : false);
    var winCtrl = (keybinding & 256 /* WinCtrl */ ? true : false);
    var ctrlKey = (OS === 2 /* Macintosh */ ? winCtrl : ctrlCmd);
    var shiftKey = (keybinding & 1024 /* Shift */ ? true : false);
    var altKey = (keybinding & 512 /* Alt */ ? true : false);
    var metaKey = (OS === 2 /* Macintosh */ ? ctrlCmd : winCtrl);
    var keyCode = (keybinding & 255 /* KeyCode */);
    return new SimpleKeybinding(ctrlKey, shiftKey, altKey, metaKey, keyCode);
}
var SimpleKeybinding = /** @class */ (function () {
    function SimpleKeybinding(ctrlKey, shiftKey, altKey, metaKey, keyCode) {
        this.type = 1 /* Simple */;
        this.ctrlKey = ctrlKey;
        this.shiftKey = shiftKey;
        this.altKey = altKey;
        this.metaKey = metaKey;
        this.keyCode = keyCode;
    }
    SimpleKeybinding.prototype.equals = function (other) {
        if (other.type !== 1 /* Simple */) {
            return false;
        }
        return (this.ctrlKey === other.ctrlKey
            && this.shiftKey === other.shiftKey
            && this.altKey === other.altKey
            && this.metaKey === other.metaKey
            && this.keyCode === other.keyCode);
    };
    SimpleKeybinding.prototype.isModifierKey = function () {
        return (this.keyCode === 0 /* Unknown */
            || this.keyCode === 5 /* Ctrl */
            || this.keyCode === 57 /* Meta */
            || this.keyCode === 6 /* Alt */
            || this.keyCode === 4 /* Shift */);
    };
    /**
     * Does this keybinding refer to the key code of a modifier and it also has the modifier flag?
     */
    SimpleKeybinding.prototype.isDuplicateModifierCase = function () {
        return ((this.ctrlKey && this.keyCode === 5 /* Ctrl */)
            || (this.shiftKey && this.keyCode === 4 /* Shift */)
            || (this.altKey && this.keyCode === 6 /* Alt */)
            || (this.metaKey && this.keyCode === 57 /* Meta */));
    };
    return SimpleKeybinding;
}());
var ChordKeybinding = /** @class */ (function () {
    function ChordKeybinding(firstPart, chordPart) {
        this.type = 2 /* Chord */;
        this.firstPart = firstPart;
        this.chordPart = chordPart;
    }
    return ChordKeybinding;
}());
var ResolvedKeybindingPart = /** @class */ (function () {
    function ResolvedKeybindingPart(ctrlKey, shiftKey, altKey, metaKey, kbLabel, kbAriaLabel) {
        this.ctrlKey = ctrlKey;
        this.shiftKey = shiftKey;
        this.altKey = altKey;
        this.metaKey = metaKey;
        this.keyLabel = kbLabel;
        this.keyAriaLabel = kbAriaLabel;
    }
    return ResolvedKeybindingPart;
}());
/**
 * A resolved keybinding. Can be a simple keybinding or a chord keybinding.
 */
var ResolvedKeybinding = /** @class */ (function () {
    function ResolvedKeybinding() {
    }
    return ResolvedKeybinding;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Throws an error with the provided message if the provided value does not evaluate to a true Javascript value.
 */
function ok(value, message) {
    if (!value || value === null) {
        throw new Error(message ? 'Assertion failed (' + message + ')' : 'Assertion Failed');
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var RegistryImpl = /** @class */ (function () {
    function RegistryImpl() {
        this.data = {};
    }
    RegistryImpl.prototype.add = function (id, data) {
        ok(isString(id));
        ok(isObject(data));
        ok(!this.data.hasOwnProperty(id), 'There is already an extension with this id');
        this.data[id] = data;
    };
    RegistryImpl.prototype.as = function (id) {
        return this.data[id] || null;
    };
    return RegistryImpl;
}());
var Registry = new RegistryImpl();

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var KeybindingsRegistryImpl = /** @class */ (function () {
    function KeybindingsRegistryImpl() {
        this._keybindings = [];
        this._keybindingsSorted = true;
    }
    /**
     * Take current platform into account and reduce to primary & secondary.
     */
    KeybindingsRegistryImpl.bindToCurrentPlatform = function (kb) {
        if (OS === 1 /* Windows */) {
            if (kb && kb.win) {
                return kb.win;
            }
        }
        else if (OS === 2 /* Macintosh */) {
            if (kb && kb.mac) {
                return kb.mac;
            }
        }
        else {
            if (kb && kb.linux) {
                return kb.linux;
            }
        }
        return kb;
    };
    KeybindingsRegistryImpl.prototype.registerKeybindingRule = function (rule, source) {
        if (source === void 0) { source = 0 /* Core */; }
        var actualKb = KeybindingsRegistryImpl.bindToCurrentPlatform(rule);
        if (actualKb && actualKb.primary) {
            var kk = createKeybinding(actualKb.primary, OS);
            if (kk) {
                this._registerDefaultKeybinding(kk, rule.id, rule.weight, 0, rule.when, source);
            }
        }
        if (actualKb && Array.isArray(actualKb.secondary)) {
            for (var i = 0, len = actualKb.secondary.length; i < len; i++) {
                var k = actualKb.secondary[i];
                var kk = createKeybinding(k, OS);
                if (kk) {
                    this._registerDefaultKeybinding(kk, rule.id, rule.weight, -i - 1, rule.when, source);
                }
            }
        }
    };
    KeybindingsRegistryImpl.prototype.registerCommandAndKeybindingRule = function (desc, source) {
        if (source === void 0) { source = 0 /* Core */; }
        this.registerKeybindingRule(desc, source);
        CommandsRegistry.registerCommand(desc);
    };
    KeybindingsRegistryImpl._mightProduceChar = function (keyCode) {
        if (keyCode >= 21 /* KEY_0 */ && keyCode <= 30 /* KEY_9 */) {
            return true;
        }
        if (keyCode >= 31 /* KEY_A */ && keyCode <= 56 /* KEY_Z */) {
            return true;
        }
        return (keyCode === 80 /* US_SEMICOLON */
            || keyCode === 81 /* US_EQUAL */
            || keyCode === 82 /* US_COMMA */
            || keyCode === 83 /* US_MINUS */
            || keyCode === 84 /* US_DOT */
            || keyCode === 85 /* US_SLASH */
            || keyCode === 86 /* US_BACKTICK */
            || keyCode === 110 /* ABNT_C1 */
            || keyCode === 111 /* ABNT_C2 */
            || keyCode === 87 /* US_OPEN_SQUARE_BRACKET */
            || keyCode === 88 /* US_BACKSLASH */
            || keyCode === 89 /* US_CLOSE_SQUARE_BRACKET */
            || keyCode === 90 /* US_QUOTE */
            || keyCode === 91 /* OEM_8 */
            || keyCode === 92 /* OEM_102 */);
    };
    KeybindingsRegistryImpl.prototype._assertNoCtrlAlt = function (keybinding, commandId) {
        if (keybinding.ctrlKey && keybinding.altKey && !keybinding.metaKey) {
            if (KeybindingsRegistryImpl._mightProduceChar(keybinding.keyCode)) {
                console.warn('Ctrl+Alt+ keybindings should not be used by default under Windows. Offender: ', keybinding, ' for ', commandId);
            }
        }
    };
    KeybindingsRegistryImpl.prototype._registerDefaultKeybinding = function (keybinding, commandId, weight1, weight2, when, source) {
        if (source === 0 /* Core */ && OS === 1 /* Windows */) {
            if (keybinding.type === 2 /* Chord */) {
                this._assertNoCtrlAlt(keybinding.firstPart, commandId);
            }
            else {
                this._assertNoCtrlAlt(keybinding, commandId);
            }
        }
        this._keybindings.push({
            keybinding: keybinding,
            command: commandId,
            commandArgs: undefined,
            when: when,
            weight1: weight1,
            weight2: weight2
        });
        this._keybindingsSorted = false;
    };
    KeybindingsRegistryImpl.prototype.getDefaultKeybindings = function () {
        if (!this._keybindingsSorted) {
            this._keybindings.sort(sorter);
            this._keybindingsSorted = true;
        }
        return this._keybindings.slice(0);
    };
    return KeybindingsRegistryImpl;
}());
var KeybindingsRegistry = new KeybindingsRegistryImpl();
// Define extension point ids
var Extensions = {
    EditorModes: 'platform.keybindingsRegistry'
};
Registry.add(Extensions.EditorModes, KeybindingsRegistry);
function sorter(a, b) {
    if (a.weight1 !== b.weight1) {
        return a.weight1 - b.weight1;
    }
    if (a.command < b.command) {
        return -1;
    }
    if (a.command > b.command) {
        return 1;
    }
    return a.weight2 - b.weight2;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ITelemetryService = createDecorator('telemetryService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$6 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (undefined && undefined.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var Command = /** @class */ (function () {
    function Command(opts) {
        this.id = opts.id;
        this.precondition = opts.precondition;
        this._kbOpts = opts.kbOpts;
        this._menubarOpts = opts.menubarOpts;
        this._description = opts.description;
    }
    Command.prototype.register = function () {
        var _this = this;
        if (this._menubarOpts) {
            MenuRegistry.appendMenuItem(this._menubarOpts.menuId, {
                group: this._menubarOpts.group,
                command: {
                    id: this.id,
                    title: this._menubarOpts.title,
                },
                when: this._menubarOpts.when,
                order: this._menubarOpts.order
            });
        }
        if (this._kbOpts) {
            var kbWhen = this._kbOpts.kbExpr;
            if (this.precondition) {
                if (kbWhen) {
                    kbWhen = ContextKeyExpr.and(kbWhen, this.precondition);
                }
                else {
                    kbWhen = this.precondition;
                }
            }
            KeybindingsRegistry.registerCommandAndKeybindingRule({
                id: this.id,
                handler: function (accessor, args) { return _this.runCommand(accessor, args); },
                weight: this._kbOpts.weight,
                when: kbWhen || null,
                primary: this._kbOpts.primary,
                secondary: this._kbOpts.secondary,
                win: this._kbOpts.win,
                linux: this._kbOpts.linux,
                mac: this._kbOpts.mac,
                description: this._description
            });
        }
        else {
            CommandsRegistry.registerCommand({
                id: this.id,
                handler: function (accessor, args) { return _this.runCommand(accessor, args); },
                description: this._description
            });
        }
    };
    return Command;
}());
var EditorCommand = /** @class */ (function (_super) {
    __extends$6(EditorCommand, _super);
    function EditorCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Create a command class that is bound to a certain editor contribution.
     */
    EditorCommand.bindToContribution = function (controllerGetter) {
        return /** @class */ (function (_super) {
            __extends$6(EditorControllerCommandImpl, _super);
            function EditorControllerCommandImpl(opts) {
                var _this = _super.call(this, opts) || this;
                _this._callback = opts.handler;
                return _this;
            }
            EditorControllerCommandImpl.prototype.runEditorCommand = function (accessor, editor, args) {
                var controller = controllerGetter(editor);
                if (controller) {
                    this._callback(controllerGetter(editor));
                }
            };
            return EditorControllerCommandImpl;
        }(EditorCommand));
    };
    EditorCommand.prototype.runCommand = function (accessor, args) {
        var _this = this;
        var codeEditorService = accessor.get(ICodeEditorService);
        // Find the editor with text focus or active
        var editor = codeEditorService.getFocusedCodeEditor() || codeEditorService.getActiveCodeEditor();
        if (!editor) {
            // well, at least we tried...
            return;
        }
        return editor.invokeWithinContext(function (editorAccessor) {
            var kbService = editorAccessor.get(IContextKeyService);
            if (!kbService.contextMatchesRules(_this.precondition)) {
                // precondition does not hold
                return;
            }
            return _this.runEditorCommand(editorAccessor, editor, args);
        });
    };
    return EditorCommand;
}(Command));
var EditorAction = /** @class */ (function (_super) {
    __extends$6(EditorAction, _super);
    function EditorAction(opts) {
        var _this = _super.call(this, opts) || this;
        _this.label = opts.label;
        _this.alias = opts.alias;
        _this.menuOpts = opts.menuOpts;
        return _this;
    }
    EditorAction.prototype.register = function () {
        if (this.menuOpts) {
            MenuRegistry.appendMenuItem(MenuId.EditorContext, {
                command: {
                    id: this.id,
                    title: this.label
                },
                when: ContextKeyExpr.and(this.precondition, this.menuOpts.when),
                group: this.menuOpts.group,
                order: this.menuOpts.order
            });
        }
        _super.prototype.register.call(this);
    };
    EditorAction.prototype.runEditorCommand = function (accessor, editor, args) {
        this.reportTelemetry(accessor, editor);
        return this.run(accessor, editor, args || {});
    };
    EditorAction.prototype.reportTelemetry = function (accessor, editor) {
        /* __GDPR__
            "editorActionInvoked" : {
                "name" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" },
                "id": { "classification": "SystemMetaData", "purpose": "FeatureInsight" },
                "${include}": [
                    "${EditorTelemetryData}"
                ]
            }
        */
        accessor.get(ITelemetryService).publicLog('editorActionInvoked', __assign({ name: this.label, id: this.id }, editor.getTelemetryData()));
    };
    return EditorAction;
}(EditorCommand));
//#endregion EditorAction
// --- Registration of commands and actions
function registerLanguageCommand(id, handler) {
    CommandsRegistry.registerCommand(id, function (accessor, args) { return handler(accessor, args || {}); });
}
function registerDefaultLanguageCommand(id, handler) {
    registerLanguageCommand(id, function (accessor, args) {
        var resource = args.resource, position = args.position;
        if (!(resource instanceof URI)) {
            throw illegalArgument('resource');
        }
        if (!Position.isIPosition(position)) {
            throw illegalArgument('position');
        }
        var model = accessor.get(IModelService).getModel(resource);
        if (model) {
            var editorPosition = Position.lift(position);
            return handler(model, editorPosition, args);
        }
        return accessor.get(ITextModelService).createModelReference(resource).then(function (reference) {
            return always(new Promise(function (resolve, reject) {
                try {
                    var result = handler(reference.object.textEditorModel, Position.lift(position), args);
                    resolve(result);
                }
                catch (err) {
                    reject(err);
                }
            }), function () {
                reference.dispose();
            });
        });
    });
}
function registerEditorCommand(editorCommand) {
    EditorContributionRegistry.INSTANCE.registerEditorCommand(editorCommand);
    return editorCommand;
}
function registerEditorAction(ctor) {
    EditorContributionRegistry.INSTANCE.registerEditorAction(new ctor());
}
function registerInstantiatedEditorAction(editorAction) {
    EditorContributionRegistry.INSTANCE.registerEditorAction(editorAction);
}
function registerEditorContribution(ctor) {
    EditorContributionRegistry.INSTANCE.registerEditorContribution(ctor);
}
var EditorExtensionsRegistry;
(function (EditorExtensionsRegistry) {
    function getEditorCommand(commandId) {
        return EditorContributionRegistry.INSTANCE.getEditorCommand(commandId);
    }
    EditorExtensionsRegistry.getEditorCommand = getEditorCommand;
    function getEditorActions() {
        return EditorContributionRegistry.INSTANCE.getEditorActions();
    }
    EditorExtensionsRegistry.getEditorActions = getEditorActions;
    function getEditorContributions() {
        return EditorContributionRegistry.INSTANCE.getEditorContributions();
    }
    EditorExtensionsRegistry.getEditorContributions = getEditorContributions;
})(EditorExtensionsRegistry || (EditorExtensionsRegistry = {}));
// Editor extension points
var Extensions$1 = {
    EditorCommonContributions: 'editor.contributions'
};
var EditorContributionRegistry = /** @class */ (function () {
    function EditorContributionRegistry() {
        this.editorContributions = [];
        this.editorActions = [];
        this.editorCommands = Object.create(null);
    }
    EditorContributionRegistry.prototype.registerEditorContribution = function (ctor) {
        this.editorContributions.push(ctor);
    };
    EditorContributionRegistry.prototype.registerEditorAction = function (action) {
        action.register();
        this.editorActions.push(action);
    };
    EditorContributionRegistry.prototype.getEditorContributions = function () {
        return this.editorContributions.slice(0);
    };
    EditorContributionRegistry.prototype.getEditorActions = function () {
        return this.editorActions.slice(0);
    };
    EditorContributionRegistry.prototype.registerEditorCommand = function (editorCommand) {
        editorCommand.register();
        this.editorCommands[editorCommand.id] = editorCommand;
    };
    EditorContributionRegistry.prototype.getEditorCommand = function (commandId) {
        return (this.editorCommands[commandId] || null);
    };
    EditorContributionRegistry.INSTANCE = new EditorContributionRegistry();
    return EditorContributionRegistry;
}());
Registry.add(Extensions$1.EditorCommonContributions, EditorContributionRegistry.INSTANCE);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * A range in the editor. (startLineNumber,startColumn) is <= (endLineNumber,endColumn)
 */
var Range = /** @class */ (function () {
    function Range(startLineNumber, startColumn, endLineNumber, endColumn) {
        if ((startLineNumber > endLineNumber) || (startLineNumber === endLineNumber && startColumn > endColumn)) {
            this.startLineNumber = endLineNumber;
            this.startColumn = endColumn;
            this.endLineNumber = startLineNumber;
            this.endColumn = startColumn;
        }
        else {
            this.startLineNumber = startLineNumber;
            this.startColumn = startColumn;
            this.endLineNumber = endLineNumber;
            this.endColumn = endColumn;
        }
    }
    /**
     * Test if this range is empty.
     */
    Range.prototype.isEmpty = function () {
        return Range.isEmpty(this);
    };
    /**
     * Test if `range` is empty.
     */
    Range.isEmpty = function (range) {
        return (range.startLineNumber === range.endLineNumber && range.startColumn === range.endColumn);
    };
    /**
     * Test if position is in this range. If the position is at the edges, will return true.
     */
    Range.prototype.containsPosition = function (position) {
        return Range.containsPosition(this, position);
    };
    /**
     * Test if `position` is in `range`. If the position is at the edges, will return true.
     */
    Range.containsPosition = function (range, position) {
        if (position.lineNumber < range.startLineNumber || position.lineNumber > range.endLineNumber) {
            return false;
        }
        if (position.lineNumber === range.startLineNumber && position.column < range.startColumn) {
            return false;
        }
        if (position.lineNumber === range.endLineNumber && position.column > range.endColumn) {
            return false;
        }
        return true;
    };
    /**
     * Test if range is in this range. If the range is equal to this range, will return true.
     */
    Range.prototype.containsRange = function (range) {
        return Range.containsRange(this, range);
    };
    /**
     * Test if `otherRange` is in `range`. If the ranges are equal, will return true.
     */
    Range.containsRange = function (range, otherRange) {
        if (otherRange.startLineNumber < range.startLineNumber || otherRange.endLineNumber < range.startLineNumber) {
            return false;
        }
        if (otherRange.startLineNumber > range.endLineNumber || otherRange.endLineNumber > range.endLineNumber) {
            return false;
        }
        if (otherRange.startLineNumber === range.startLineNumber && otherRange.startColumn < range.startColumn) {
            return false;
        }
        if (otherRange.endLineNumber === range.endLineNumber && otherRange.endColumn > range.endColumn) {
            return false;
        }
        return true;
    };
    /**
     * A reunion of the two ranges.
     * The smallest position will be used as the start point, and the largest one as the end point.
     */
    Range.prototype.plusRange = function (range) {
        return Range.plusRange(this, range);
    };
    /**
     * A reunion of the two ranges.
     * The smallest position will be used as the start point, and the largest one as the end point.
     */
    Range.plusRange = function (a, b) {
        var startLineNumber;
        var startColumn;
        var endLineNumber;
        var endColumn;
        if (b.startLineNumber < a.startLineNumber) {
            startLineNumber = b.startLineNumber;
            startColumn = b.startColumn;
        }
        else if (b.startLineNumber === a.startLineNumber) {
            startLineNumber = b.startLineNumber;
            startColumn = Math.min(b.startColumn, a.startColumn);
        }
        else {
            startLineNumber = a.startLineNumber;
            startColumn = a.startColumn;
        }
        if (b.endLineNumber > a.endLineNumber) {
            endLineNumber = b.endLineNumber;
            endColumn = b.endColumn;
        }
        else if (b.endLineNumber === a.endLineNumber) {
            endLineNumber = b.endLineNumber;
            endColumn = Math.max(b.endColumn, a.endColumn);
        }
        else {
            endLineNumber = a.endLineNumber;
            endColumn = a.endColumn;
        }
        return new Range(startLineNumber, startColumn, endLineNumber, endColumn);
    };
    /**
     * A intersection of the two ranges.
     */
    Range.prototype.intersectRanges = function (range) {
        return Range.intersectRanges(this, range);
    };
    /**
     * A intersection of the two ranges.
     */
    Range.intersectRanges = function (a, b) {
        var resultStartLineNumber = a.startLineNumber;
        var resultStartColumn = a.startColumn;
        var resultEndLineNumber = a.endLineNumber;
        var resultEndColumn = a.endColumn;
        var otherStartLineNumber = b.startLineNumber;
        var otherStartColumn = b.startColumn;
        var otherEndLineNumber = b.endLineNumber;
        var otherEndColumn = b.endColumn;
        if (resultStartLineNumber < otherStartLineNumber) {
            resultStartLineNumber = otherStartLineNumber;
            resultStartColumn = otherStartColumn;
        }
        else if (resultStartLineNumber === otherStartLineNumber) {
            resultStartColumn = Math.max(resultStartColumn, otherStartColumn);
        }
        if (resultEndLineNumber > otherEndLineNumber) {
            resultEndLineNumber = otherEndLineNumber;
            resultEndColumn = otherEndColumn;
        }
        else if (resultEndLineNumber === otherEndLineNumber) {
            resultEndColumn = Math.min(resultEndColumn, otherEndColumn);
        }
        // Check if selection is now empty
        if (resultStartLineNumber > resultEndLineNumber) {
            return null;
        }
        if (resultStartLineNumber === resultEndLineNumber && resultStartColumn > resultEndColumn) {
            return null;
        }
        return new Range(resultStartLineNumber, resultStartColumn, resultEndLineNumber, resultEndColumn);
    };
    /**
     * Test if this range equals other.
     */
    Range.prototype.equalsRange = function (other) {
        return Range.equalsRange(this, other);
    };
    /**
     * Test if range `a` equals `b`.
     */
    Range.equalsRange = function (a, b) {
        return (!!a &&
            !!b &&
            a.startLineNumber === b.startLineNumber &&
            a.startColumn === b.startColumn &&
            a.endLineNumber === b.endLineNumber &&
            a.endColumn === b.endColumn);
    };
    /**
     * Return the end position (which will be after or equal to the start position)
     */
    Range.prototype.getEndPosition = function () {
        return new Position(this.endLineNumber, this.endColumn);
    };
    /**
     * Return the start position (which will be before or equal to the end position)
     */
    Range.prototype.getStartPosition = function () {
        return new Position(this.startLineNumber, this.startColumn);
    };
    /**
     * Transform to a user presentable string representation.
     */
    Range.prototype.toString = function () {
        return '[' + this.startLineNumber + ',' + this.startColumn + ' -> ' + this.endLineNumber + ',' + this.endColumn + ']';
    };
    /**
     * Create a new range using this range's start position, and using endLineNumber and endColumn as the end position.
     */
    Range.prototype.setEndPosition = function (endLineNumber, endColumn) {
        return new Range(this.startLineNumber, this.startColumn, endLineNumber, endColumn);
    };
    /**
     * Create a new range using this range's end position, and using startLineNumber and startColumn as the start position.
     */
    Range.prototype.setStartPosition = function (startLineNumber, startColumn) {
        return new Range(startLineNumber, startColumn, this.endLineNumber, this.endColumn);
    };
    /**
     * Create a new empty range using this range's start position.
     */
    Range.prototype.collapseToStart = function () {
        return Range.collapseToStart(this);
    };
    /**
     * Create a new empty range using this range's start position.
     */
    Range.collapseToStart = function (range) {
        return new Range(range.startLineNumber, range.startColumn, range.startLineNumber, range.startColumn);
    };
    // ---
    Range.fromPositions = function (start, end) {
        if (end === void 0) { end = start; }
        return new Range(start.lineNumber, start.column, end.lineNumber, end.column);
    };
    Range.lift = function (range) {
        if (!range) {
            return null;
        }
        return new Range(range.startLineNumber, range.startColumn, range.endLineNumber, range.endColumn);
    };
    /**
     * Test if `obj` is an `IRange`.
     */
    Range.isIRange = function (obj) {
        return (obj
            && (typeof obj.startLineNumber === 'number')
            && (typeof obj.startColumn === 'number')
            && (typeof obj.endLineNumber === 'number')
            && (typeof obj.endColumn === 'number'));
    };
    /**
     * Test if the two ranges are touching in any way.
     */
    Range.areIntersectingOrTouching = function (a, b) {
        // Check if `a` is before `b`
        if (a.endLineNumber < b.startLineNumber || (a.endLineNumber === b.startLineNumber && a.endColumn < b.startColumn)) {
            return false;
        }
        // Check if `b` is before `a`
        if (b.endLineNumber < a.startLineNumber || (b.endLineNumber === a.startLineNumber && b.endColumn < a.startColumn)) {
            return false;
        }
        // These ranges must intersect
        return true;
    };
    /**
     * Test if the two ranges are intersecting. If the ranges are touching it returns true.
     */
    Range.areIntersecting = function (a, b) {
        // Check if `a` is before `b`
        if (a.endLineNumber < b.startLineNumber || (a.endLineNumber === b.startLineNumber && a.endColumn <= b.startColumn)) {
            return false;
        }
        // Check if `b` is before `a`
        if (b.endLineNumber < a.startLineNumber || (b.endLineNumber === a.startLineNumber && b.endColumn <= a.startColumn)) {
            return false;
        }
        // These ranges must intersect
        return true;
    };
    /**
     * A function that compares ranges, useful for sorting ranges
     * It will first compare ranges on the startPosition and then on the endPosition
     */
    Range.compareRangesUsingStarts = function (a, b) {
        if (a && b) {
            var aStartLineNumber = a.startLineNumber | 0;
            var bStartLineNumber = b.startLineNumber | 0;
            if (aStartLineNumber === bStartLineNumber) {
                var aStartColumn = a.startColumn | 0;
                var bStartColumn = b.startColumn | 0;
                if (aStartColumn === bStartColumn) {
                    var aEndLineNumber = a.endLineNumber | 0;
                    var bEndLineNumber = b.endLineNumber | 0;
                    if (aEndLineNumber === bEndLineNumber) {
                        var aEndColumn = a.endColumn | 0;
                        var bEndColumn = b.endColumn | 0;
                        return aEndColumn - bEndColumn;
                    }
                    return aEndLineNumber - bEndLineNumber;
                }
                return aStartColumn - bStartColumn;
            }
            return aStartLineNumber - bStartLineNumber;
        }
        var aExists = (a ? 1 : 0);
        var bExists = (b ? 1 : 0);
        return aExists - bExists;
    };
    /**
     * A function that compares ranges, useful for sorting ranges
     * It will first compare ranges on the endPosition and then on the startPosition
     */
    Range.compareRangesUsingEnds = function (a, b) {
        if (a.endLineNumber === b.endLineNumber) {
            if (a.endColumn === b.endColumn) {
                if (a.startLineNumber === b.startLineNumber) {
                    return a.startColumn - b.startColumn;
                }
                return a.startLineNumber - b.startLineNumber;
            }
            return a.endColumn - b.endColumn;
        }
        return a.endLineNumber - b.endLineNumber;
    };
    /**
     * Test if the range spans multiple lines.
     */
    Range.spansMultipleLines = function (range) {
        return range.endLineNumber > range.startLineNumber;
    };
    return Range;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$7 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * A selection in the editor.
 * The selection is a range that has an orientation.
 */
var Selection = /** @class */ (function (_super) {
    __extends$7(Selection, _super);
    function Selection(selectionStartLineNumber, selectionStartColumn, positionLineNumber, positionColumn) {
        var _this = _super.call(this, selectionStartLineNumber, selectionStartColumn, positionLineNumber, positionColumn) || this;
        _this.selectionStartLineNumber = selectionStartLineNumber;
        _this.selectionStartColumn = selectionStartColumn;
        _this.positionLineNumber = positionLineNumber;
        _this.positionColumn = positionColumn;
        return _this;
    }
    /**
     * Clone this selection.
     */
    Selection.prototype.clone = function () {
        return new Selection(this.selectionStartLineNumber, this.selectionStartColumn, this.positionLineNumber, this.positionColumn);
    };
    /**
     * Transform to a human-readable representation.
     */
    Selection.prototype.toString = function () {
        return '[' + this.selectionStartLineNumber + ',' + this.selectionStartColumn + ' -> ' + this.positionLineNumber + ',' + this.positionColumn + ']';
    };
    /**
     * Test if equals other selection.
     */
    Selection.prototype.equalsSelection = function (other) {
        return (Selection.selectionsEqual(this, other));
    };
    /**
     * Test if the two selections are equal.
     */
    Selection.selectionsEqual = function (a, b) {
        return (a.selectionStartLineNumber === b.selectionStartLineNumber &&
            a.selectionStartColumn === b.selectionStartColumn &&
            a.positionLineNumber === b.positionLineNumber &&
            a.positionColumn === b.positionColumn);
    };
    /**
     * Get directions (LTR or RTL).
     */
    Selection.prototype.getDirection = function () {
        if (this.selectionStartLineNumber === this.startLineNumber && this.selectionStartColumn === this.startColumn) {
            return 0 /* LTR */;
        }
        return 1 /* RTL */;
    };
    /**
     * Create a new selection with a different `positionLineNumber` and `positionColumn`.
     */
    Selection.prototype.setEndPosition = function (endLineNumber, endColumn) {
        if (this.getDirection() === 0 /* LTR */) {
            return new Selection(this.startLineNumber, this.startColumn, endLineNumber, endColumn);
        }
        return new Selection(endLineNumber, endColumn, this.startLineNumber, this.startColumn);
    };
    /**
     * Get the position at `positionLineNumber` and `positionColumn`.
     */
    Selection.prototype.getPosition = function () {
        return new Position(this.positionLineNumber, this.positionColumn);
    };
    /**
     * Create a new selection with a different `selectionStartLineNumber` and `selectionStartColumn`.
     */
    Selection.prototype.setStartPosition = function (startLineNumber, startColumn) {
        if (this.getDirection() === 0 /* LTR */) {
            return new Selection(startLineNumber, startColumn, this.endLineNumber, this.endColumn);
        }
        return new Selection(this.endLineNumber, this.endColumn, startLineNumber, startColumn);
    };
    // ----
    /**
     * Create a `Selection` from one or two positions
     */
    Selection.fromPositions = function (start, end) {
        if (end === void 0) { end = start; }
        return new Selection(start.lineNumber, start.column, end.lineNumber, end.column);
    };
    /**
     * Create a `Selection` from an `ISelection`.
     */
    Selection.liftSelection = function (sel) {
        return new Selection(sel.selectionStartLineNumber, sel.selectionStartColumn, sel.positionLineNumber, sel.positionColumn);
    };
    /**
     * `a` equals `b`.
     */
    Selection.selectionsArrEqual = function (a, b) {
        if (a && !b || !a && b) {
            return false;
        }
        if (!a && !b) {
            return true;
        }
        if (a.length !== b.length) {
            return false;
        }
        for (var i = 0, len = a.length; i < len; i++) {
            if (!this.selectionsEqual(a[i], b[i])) {
                return false;
            }
        }
        return true;
    };
    /**
     * Test if `obj` is an `ISelection`.
     */
    Selection.isISelection = function (obj) {
        return (obj
            && (typeof obj.selectionStartLineNumber === 'number')
            && (typeof obj.selectionStartColumn === 'number')
            && (typeof obj.positionLineNumber === 'number')
            && (typeof obj.positionColumn === 'number'));
    };
    /**
     * Create with a direction.
     */
    Selection.createWithDirection = function (startLineNumber, startColumn, endLineNumber, endColumn, direction) {
        if (direction === 0 /* LTR */) {
            return new Selection(startLineNumber, startColumn, endLineNumber, endColumn);
        }
        return new Selection(endLineNumber, endColumn, startLineNumber, startColumn);
    };
    return Selection;
}(Range));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var hasPerformanceNow = (globals.performance && typeof globals.performance.now === 'function');
var StopWatch = /** @class */ (function () {
    function StopWatch(highResolution) {
        this._highResolution = hasPerformanceNow && highResolution;
        this._startTime = this._now();
        this._stopTime = -1;
    }
    StopWatch.create = function (highResolution) {
        if (highResolution === void 0) { highResolution = true; }
        return new StopWatch(highResolution);
    };
    StopWatch.prototype.elapsed = function () {
        if (this._stopTime !== -1) {
            return this._stopTime - this._startTime;
        }
        return this._now() - this._startTime;
    };
    StopWatch.prototype._now = function () {
        return this._highResolution ? globals.performance.now() : new Date().getTime();
    };
    return StopWatch;
}());

/**
 * Returns the last element of an array.
 * @param array The array.
 * @param n Which element from the end (default is zero).
 */
function tail(array, n) {
    if (n === void 0) { n = 0; }
    return array[array.length - (1 + n)];
}
function equals(one, other, itemEquals) {
    if (itemEquals === void 0) { itemEquals = function (a, b) { return a === b; }; }
    if (one === other) {
        return true;
    }
    if (!one || !other) {
        return false;
    }
    if (one.length !== other.length) {
        return false;
    }
    for (var i = 0, len = one.length; i < len; i++) {
        if (!itemEquals(one[i], other[i])) {
            return false;
        }
    }
    return true;
}
function binarySearch(array, key, comparator) {
    var low = 0, high = array.length - 1;
    while (low <= high) {
        var mid = ((low + high) / 2) | 0;
        var comp = comparator(array[mid], key);
        if (comp < 0) {
            low = mid + 1;
        }
        else if (comp > 0) {
            high = mid - 1;
        }
        else {
            return mid;
        }
    }
    return -(low + 1);
}
/**
 * Takes a sorted array and a function p. The array is sorted in such a way that all elements where p(x) is false
 * are located before all elements where p(x) is true.
 * @returns the least x for which p(x) is true or array.length if no element fullfills the given function.
 */
function findFirstInSorted(array, p) {
    var low = 0, high = array.length;
    if (high === 0) {
        return 0; // no children
    }
    while (low < high) {
        var mid = Math.floor((low + high) / 2);
        if (p(array[mid])) {
            high = mid;
        }
        else {
            low = mid + 1;
        }
    }
    return low;
}
/**
 * Like `Array#sort` but always stable. Usually runs a little slower `than Array#sort`
 * so only use this when actually needing stable sort.
 */
function mergeSort(data, compare) {
    _sort(data, compare, 0, data.length - 1, []);
    return data;
}
function _merge(a, compare, lo, mid, hi, aux) {
    var leftIdx = lo, rightIdx = mid + 1;
    for (var i = lo; i <= hi; i++) {
        aux[i] = a[i];
    }
    for (var i = lo; i <= hi; i++) {
        if (leftIdx > mid) {
            // left side consumed
            a[i] = aux[rightIdx++];
        }
        else if (rightIdx > hi) {
            // right side consumed
            a[i] = aux[leftIdx++];
        }
        else if (compare(aux[rightIdx], aux[leftIdx]) < 0) {
            // right element is less -> comes first
            a[i] = aux[rightIdx++];
        }
        else {
            // left element comes first (less or equal)
            a[i] = aux[leftIdx++];
        }
    }
}
function _sort(a, compare, lo, hi, aux) {
    if (hi <= lo) {
        return;
    }
    var mid = lo + ((hi - lo) / 2) | 0;
    _sort(a, compare, lo, mid, aux);
    _sort(a, compare, mid + 1, hi, aux);
    if (compare(a[mid], a[mid + 1]) <= 0) {
        // left and right are sorted and if the last-left element is less
        // or equals than the first-right element there is nothing else
        // to do
        return;
    }
    _merge(a, compare, lo, mid, hi, aux);
}
function groupBy(data, compare) {
    var result = [];
    var currentGroup = undefined;
    for (var _i = 0, _a = mergeSort(data.slice(0), compare); _i < _a.length; _i++) {
        var element = _a[_i];
        if (!currentGroup || compare(currentGroup[0], element) !== 0) {
            currentGroup = [element];
            result.push(currentGroup);
        }
        else {
            currentGroup.push(element);
        }
    }
    return result;
}
/**
 * @returns a new array with all falsy values removed. The original array IS NOT modified.
 */
function coalesce(array) {
    if (!array) {
        return array;
    }
    return array.filter(function (e) { return !!e; });
}
/**
 * @returns {{false}} if the provided object is an array
 * 	and not empty.
 */
function isFalsyOrEmpty(obj) {
    return !Array.isArray(obj) || obj.length === 0;
}
/**
 * Removes duplicates from the given array. The optional keyFn allows to specify
 * how elements are checked for equalness by returning a unique string for each.
 */
function distinct(array, keyFn) {
    if (!keyFn) {
        return array.filter(function (element, position) {
            return array.indexOf(element) === position;
        });
    }
    var seen = Object.create(null);
    return array.filter(function (elem) {
        var key = keyFn(elem);
        if (seen[key]) {
            return false;
        }
        seen[key] = true;
        return true;
    });
}
function firstIndex(array, fn) {
    for (var i = 0; i < array.length; i++) {
        var element = array[i];
        if (fn(element)) {
            return i;
        }
    }
    return -1;
}
function first$1(array, fn, notFoundValue) {
    if (notFoundValue === void 0) { notFoundValue = null; }
    var index = firstIndex(array, fn);
    return index < 0 ? notFoundValue : array[index];
}
function range(arg, to) {
    var from = typeof to === 'number' ? arg : 0;
    if (typeof to === 'number') {
        from = arg;
    }
    else {
        from = 0;
        to = arg;
    }
    var result = [];
    if (from <= to) {
        for (var i = from; i < to; i++) {
            result.push(i);
        }
    }
    else {
        for (var i = from; i > to; i--) {
            result.push(i);
        }
    }
    return result;
}
/**
 * Insert `insertArr` inside `target` at `insertIndex`.
 * Please don't touch unless you understand https://jsperf.com/inserting-an-array-within-an-array
 */
function arrayInsert(target, insertIndex, insertArr) {
    var before = target.slice(0, insertIndex);
    var after = target.slice(insertIndex);
    return before.concat(insertArr, after);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function deepClone(obj) {
    if (!obj || typeof obj !== 'object') {
        return obj;
    }
    if (obj instanceof RegExp) {
        // See https://github.com/Microsoft/TypeScript/issues/10990
        return obj;
    }
    var result = Array.isArray(obj) ? [] : {};
    Object.keys(obj).forEach(function (key) {
        if (obj[key] && typeof obj[key] === 'object') {
            result[key] = deepClone(obj[key]);
        }
        else {
            result[key] = obj[key];
        }
    });
    return result;
}
function deepFreeze(obj) {
    if (!obj || typeof obj !== 'object') {
        return obj;
    }
    var stack = [obj];
    while (stack.length > 0) {
        var obj_1 = stack.shift();
        Object.freeze(obj_1);
        for (var key in obj_1) {
            if (_hasOwnProperty.call(obj_1, key)) {
                var prop = obj_1[key];
                if (typeof prop === 'object' && !Object.isFrozen(prop)) {
                    stack.push(prop);
                }
            }
        }
    }
    return obj;
}
var _hasOwnProperty = Object.prototype.hasOwnProperty;
/**
 * Copies all properties of source into destination. The optional parameter "overwrite" allows to control
 * if existing properties on the destination should be overwritten or not. Defaults to true (overwrite).
 */
function mixin(destination, source, overwrite) {
    if (overwrite === void 0) { overwrite = true; }
    if (!isObject(destination)) {
        return source;
    }
    if (isObject(source)) {
        Object.keys(source).forEach(function (key) {
            if (key in destination) {
                if (overwrite) {
                    if (isObject(destination[key]) && isObject(source[key])) {
                        mixin(destination[key], source[key], overwrite);
                    }
                    else {
                        destination[key] = source[key];
                    }
                }
            }
            else {
                destination[key] = source[key];
            }
        });
    }
    return destination;
}
function assign(destination) {
    var sources = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        sources[_i - 1] = arguments[_i];
    }
    sources.forEach(function (source) { return Object.keys(source).forEach(function (key) { return destination[key] = source[key]; }); });
    return destination;
}
function equals$1(one, other) {
    if (one === other) {
        return true;
    }
    if (one === null || one === undefined || other === null || other === undefined) {
        return false;
    }
    if (typeof one !== typeof other) {
        return false;
    }
    if (typeof one !== 'object') {
        return false;
    }
    if ((Array.isArray(one)) !== (Array.isArray(other))) {
        return false;
    }
    var i;
    var key;
    if (Array.isArray(one)) {
        if (one.length !== other.length) {
            return false;
        }
        for (i = 0; i < one.length; i++) {
            if (!equals$1(one[i], other[i])) {
                return false;
            }
        }
    }
    else {
        var oneKeys = [];
        for (key in one) {
            oneKeys.push(key);
        }
        oneKeys.sort();
        var otherKeys = [];
        for (key in other) {
            otherKeys.push(key);
        }
        otherKeys.sort();
        if (!equals$1(oneKeys, otherKeys)) {
            return false;
        }
        for (i = 0; i < oneKeys.length; i++) {
            if (!equals$1(one[oneKeys[i]], other[oneKeys[i]])) {
                return false;
            }
        }
    }
    return true;
}
function arrayToHash(array) {
    var result = {};
    for (var i = 0; i < array.length; ++i) {
        result[array[i]] = true;
    }
    return result;
}
/**
 * Given an array of strings, returns a function which, given a string
 * returns true or false whether the string is in that array.
 */
function createKeywordMatcher(arr, caseInsensitive) {
    if (caseInsensitive === void 0) { caseInsensitive = false; }
    if (caseInsensitive) {
        arr = arr.map(function (x) { return x.toLowerCase(); });
    }
    var hash = arrayToHash(arr);
    if (caseInsensitive) {
        return function (word) {
            return hash[word.toLowerCase()] !== undefined && hash.hasOwnProperty(word.toLowerCase());
        };
    }
    else {
        return function (word) {
            return hash[word] !== undefined && hash.hasOwnProperty(word);
        };
    }
}
function getOrDefault(obj, fn, defaultValue) {
    if (defaultValue === void 0) { defaultValue = null; }
    var result = fn(obj);
    return typeof result === 'undefined' ? defaultValue : result;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var USUAL_WORD_SEPARATORS = '`~!@#$%^&*()-=+[{]}\\|;:\'",.<>/?';
/**
 * Create a word definition regular expression based on default word separators.
 * Optionally provide allowed separators that should be included in words.
 *
 * The default would look like this:
 * /(-?\d*\.\d\w*)|([^\`\~\!\@\#\$\%\^\&\*\(\)\-\=\+\[\{\]\}\\\|\;\:\'\"\,\.\<\>\/\?\s]+)/g
 */
function createWordRegExp(allowInWords) {
    if (allowInWords === void 0) { allowInWords = ''; }
    var source = '(-?\\d*\\.\\d\\w*)|([^';
    for (var i = 0; i < USUAL_WORD_SEPARATORS.length; i++) {
        if (allowInWords.indexOf(USUAL_WORD_SEPARATORS[i]) >= 0) {
            continue;
        }
        source += '\\' + USUAL_WORD_SEPARATORS[i];
    }
    source += '\\s]+)';
    return new RegExp(source, 'g');
}
// catches numbers (including floating numbers) in the first group, and alphanum in the second
var DEFAULT_WORD_REGEXP = createWordRegExp();
function ensureValidWordDefinition(wordDefinition) {
    var result = DEFAULT_WORD_REGEXP;
    if (wordDefinition && (wordDefinition instanceof RegExp)) {
        if (!wordDefinition.global) {
            var flags = 'g';
            if (wordDefinition.ignoreCase) {
                flags += 'i';
            }
            if (wordDefinition.multiline) {
                flags += 'm';
            }
            result = new RegExp(wordDefinition.source, flags);
        }
        else {
            result = wordDefinition;
        }
    }
    result.lastIndex = 0;
    return result;
}
function getWordAtPosFast(column, wordDefinition, text, textOffset) {
    // find whitespace enclosed text around column and match from there
    var pos = column - 1 - textOffset;
    var start = text.lastIndexOf(' ', pos - 1) + 1;
    var end = text.indexOf(' ', pos);
    if (end === -1) {
        end = text.length;
    }
    wordDefinition.lastIndex = start;
    var match;
    while (match = wordDefinition.exec(text)) {
        var matchIndex = match.index || 0;
        if (matchIndex <= pos && wordDefinition.lastIndex >= pos) {
            return {
                word: match[0],
                startColumn: textOffset + 1 + matchIndex,
                endColumn: textOffset + 1 + wordDefinition.lastIndex
            };
        }
    }
    return null;
}
function getWordAtPosSlow(column, wordDefinition, text, textOffset) {
    // matches all words starting at the beginning
    // of the input until it finds a match that encloses
    // the desired column. slow but correct
    var pos = column - 1 - textOffset;
    wordDefinition.lastIndex = 0;
    var match;
    while (match = wordDefinition.exec(text)) {
        var matchIndex = match.index || 0;
        if (matchIndex > pos) {
            // |nW -> matched only after the pos
            return null;
        }
        else if (wordDefinition.lastIndex >= pos) {
            // W|W -> match encloses pos
            return {
                word: match[0],
                startColumn: textOffset + 1 + matchIndex,
                endColumn: textOffset + 1 + wordDefinition.lastIndex
            };
        }
    }
    return null;
}
function getWordAtText(column, wordDefinition, text, textOffset) {
    // if `words` can contain whitespace character we have to use the slow variant
    // otherwise we use the fast variant of finding a word
    wordDefinition.lastIndex = 0;
    var match = wordDefinition.exec(text);
    if (!match) {
        return null;
    }
    // todo@joh the `match` could already be the (first) word
    var ret = match[0].indexOf(' ') >= 0
        // did match a word which contains a space character -> use slow word find
        ? getWordAtPosSlow(column, wordDefinition, text, textOffset)
        // sane word definition -> use fast word find
        : getWordAtPosFast(column, wordDefinition, text, textOffset);
    // both (getWordAtPosFast and getWordAtPosSlow) leave the wordDefinition-RegExp
    // in an undefined state and to not confuse other users of the wordDefinition
    // we reset the lastIndex
    wordDefinition.lastIndex = 0;
    return ret;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __assign$1 = (undefined && undefined.__assign) || function () {
    __assign$1 = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign$1.apply(this, arguments);
};
/**
 * @internal
 */
function blinkingStyleToString(blinkingStyle) {
    if (blinkingStyle === 1 /* Blink */) {
        return 'blink';
    }
    else if (blinkingStyle === 4 /* Expand */) {
        return 'expand';
    }
    else if (blinkingStyle === 3 /* Phase */) {
        return 'phase';
    }
    else if (blinkingStyle === 2 /* Smooth */) {
        return 'smooth';
    }
    else if (blinkingStyle === 5 /* Solid */) {
        return 'solid';
    }
    else {
        throw new Error('blinkingStyleToString: Unknown blinkingStyle');
    }
}
/**
 * The style in which the editor's cursor should be rendered.
 */
var TextEditorCursorStyle;
(function (TextEditorCursorStyle) {
    /**
     * As a vertical line (sitting between two characters).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["Line"] = 1] = "Line";
    /**
     * As a block (sitting on top of a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["Block"] = 2] = "Block";
    /**
     * As a horizontal line (sitting under a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["Underline"] = 3] = "Underline";
    /**
     * As a thin vertical line (sitting between two characters).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["LineThin"] = 4] = "LineThin";
    /**
     * As an outlined block (sitting on top of a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["BlockOutline"] = 5] = "BlockOutline";
    /**
     * As a thin horizontal line (sitting under a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["UnderlineThin"] = 6] = "UnderlineThin";
})(TextEditorCursorStyle || (TextEditorCursorStyle = {}));
/**
 * @internal
 */
function cursorStyleToString(cursorStyle) {
    if (cursorStyle === TextEditorCursorStyle.Line) {
        return 'line';
    }
    else if (cursorStyle === TextEditorCursorStyle.Block) {
        return 'block';
    }
    else if (cursorStyle === TextEditorCursorStyle.Underline) {
        return 'underline';
    }
    else if (cursorStyle === TextEditorCursorStyle.LineThin) {
        return 'line-thin';
    }
    else if (cursorStyle === TextEditorCursorStyle.BlockOutline) {
        return 'block-outline';
    }
    else if (cursorStyle === TextEditorCursorStyle.UnderlineThin) {
        return 'underline-thin';
    }
    else {
        throw new Error('cursorStyleToString: Unknown cursorStyle');
    }
}
function _cursorStyleFromString(cursorStyle, defaultValue) {
    if (typeof cursorStyle !== 'string') {
        return defaultValue;
    }
    if (cursorStyle === 'line') {
        return TextEditorCursorStyle.Line;
    }
    else if (cursorStyle === 'block') {
        return TextEditorCursorStyle.Block;
    }
    else if (cursorStyle === 'underline') {
        return TextEditorCursorStyle.Underline;
    }
    else if (cursorStyle === 'line-thin') {
        return TextEditorCursorStyle.LineThin;
    }
    else if (cursorStyle === 'block-outline') {
        return TextEditorCursorStyle.BlockOutline;
    }
    else if (cursorStyle === 'underline-thin') {
        return TextEditorCursorStyle.UnderlineThin;
    }
    return TextEditorCursorStyle.Line;
}
/**
 * Internal configuration options (transformed or computed) for the editor.
 */
var InternalEditorOptions = /** @class */ (function () {
    /**
     * @internal
     */
    function InternalEditorOptions(source) {
        this.canUseLayerHinting = source.canUseLayerHinting;
        this.pixelRatio = source.pixelRatio;
        this.editorClassName = source.editorClassName;
        this.lineHeight = source.lineHeight | 0;
        this.readOnly = source.readOnly;
        this.accessibilitySupport = source.accessibilitySupport;
        this.multiCursorModifier = source.multiCursorModifier;
        this.multiCursorMergeOverlapping = source.multiCursorMergeOverlapping;
        this.wordSeparators = source.wordSeparators;
        this.autoClosingBrackets = source.autoClosingBrackets;
        this.autoClosingQuotes = source.autoClosingQuotes;
        this.autoSurround = source.autoSurround;
        this.autoIndent = source.autoIndent;
        this.useTabStops = source.useTabStops;
        this.tabFocusMode = source.tabFocusMode;
        this.dragAndDrop = source.dragAndDrop;
        this.emptySelectionClipboard = source.emptySelectionClipboard;
        this.copyWithSyntaxHighlighting = source.copyWithSyntaxHighlighting;
        this.layoutInfo = source.layoutInfo;
        this.fontInfo = source.fontInfo;
        this.viewInfo = source.viewInfo;
        this.wrappingInfo = source.wrappingInfo;
        this.contribInfo = source.contribInfo;
        this.showUnused = source.showUnused;
    }
    /**
     * @internal
     */
    InternalEditorOptions.prototype.equals = function (other) {
        return (this.canUseLayerHinting === other.canUseLayerHinting
            && this.pixelRatio === other.pixelRatio
            && this.editorClassName === other.editorClassName
            && this.lineHeight === other.lineHeight
            && this.readOnly === other.readOnly
            && this.accessibilitySupport === other.accessibilitySupport
            && this.multiCursorModifier === other.multiCursorModifier
            && this.multiCursorMergeOverlapping === other.multiCursorMergeOverlapping
            && this.wordSeparators === other.wordSeparators
            && this.autoClosingBrackets === other.autoClosingBrackets
            && this.autoClosingQuotes === other.autoClosingQuotes
            && this.autoSurround === other.autoSurround
            && this.autoIndent === other.autoIndent
            && this.useTabStops === other.useTabStops
            && this.tabFocusMode === other.tabFocusMode
            && this.dragAndDrop === other.dragAndDrop
            && this.showUnused === other.showUnused
            && this.emptySelectionClipboard === other.emptySelectionClipboard
            && this.copyWithSyntaxHighlighting === other.copyWithSyntaxHighlighting
            && InternalEditorOptions._equalsLayoutInfo(this.layoutInfo, other.layoutInfo)
            && this.fontInfo.equals(other.fontInfo)
            && InternalEditorOptions._equalsViewOptions(this.viewInfo, other.viewInfo)
            && InternalEditorOptions._equalsWrappingInfo(this.wrappingInfo, other.wrappingInfo)
            && InternalEditorOptions._equalsContribOptions(this.contribInfo, other.contribInfo));
    };
    /**
     * @internal
     */
    InternalEditorOptions.prototype.createChangeEvent = function (newOpts) {
        return {
            canUseLayerHinting: (this.canUseLayerHinting !== newOpts.canUseLayerHinting),
            pixelRatio: (this.pixelRatio !== newOpts.pixelRatio),
            editorClassName: (this.editorClassName !== newOpts.editorClassName),
            lineHeight: (this.lineHeight !== newOpts.lineHeight),
            readOnly: (this.readOnly !== newOpts.readOnly),
            accessibilitySupport: (this.accessibilitySupport !== newOpts.accessibilitySupport),
            multiCursorModifier: (this.multiCursorModifier !== newOpts.multiCursorModifier),
            multiCursorMergeOverlapping: (this.multiCursorMergeOverlapping !== newOpts.multiCursorMergeOverlapping),
            wordSeparators: (this.wordSeparators !== newOpts.wordSeparators),
            autoClosingBrackets: (this.autoClosingBrackets !== newOpts.autoClosingBrackets),
            autoClosingQuotes: (this.autoClosingQuotes !== newOpts.autoClosingQuotes),
            autoSurround: (this.autoSurround !== newOpts.autoSurround),
            autoIndent: (this.autoIndent !== newOpts.autoIndent),
            useTabStops: (this.useTabStops !== newOpts.useTabStops),
            tabFocusMode: (this.tabFocusMode !== newOpts.tabFocusMode),
            dragAndDrop: (this.dragAndDrop !== newOpts.dragAndDrop),
            emptySelectionClipboard: (this.emptySelectionClipboard !== newOpts.emptySelectionClipboard),
            copyWithSyntaxHighlighting: (this.copyWithSyntaxHighlighting !== newOpts.copyWithSyntaxHighlighting),
            layoutInfo: (!InternalEditorOptions._equalsLayoutInfo(this.layoutInfo, newOpts.layoutInfo)),
            fontInfo: (!this.fontInfo.equals(newOpts.fontInfo)),
            viewInfo: (!InternalEditorOptions._equalsViewOptions(this.viewInfo, newOpts.viewInfo)),
            wrappingInfo: (!InternalEditorOptions._equalsWrappingInfo(this.wrappingInfo, newOpts.wrappingInfo)),
            contribInfo: (!InternalEditorOptions._equalsContribOptions(this.contribInfo, newOpts.contribInfo))
        };
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsLayoutInfo = function (a, b) {
        return (a.width === b.width
            && a.height === b.height
            && a.glyphMarginLeft === b.glyphMarginLeft
            && a.glyphMarginWidth === b.glyphMarginWidth
            && a.glyphMarginHeight === b.glyphMarginHeight
            && a.lineNumbersLeft === b.lineNumbersLeft
            && a.lineNumbersWidth === b.lineNumbersWidth
            && a.lineNumbersHeight === b.lineNumbersHeight
            && a.decorationsLeft === b.decorationsLeft
            && a.decorationsWidth === b.decorationsWidth
            && a.decorationsHeight === b.decorationsHeight
            && a.contentLeft === b.contentLeft
            && a.contentWidth === b.contentWidth
            && a.contentHeight === b.contentHeight
            && a.renderMinimap === b.renderMinimap
            && a.minimapLeft === b.minimapLeft
            && a.minimapWidth === b.minimapWidth
            && a.viewportColumn === b.viewportColumn
            && a.verticalScrollbarWidth === b.verticalScrollbarWidth
            && a.horizontalScrollbarHeight === b.horizontalScrollbarHeight
            && this._equalsOverviewRuler(a.overviewRuler, b.overviewRuler));
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsOverviewRuler = function (a, b) {
        return (a.width === b.width
            && a.height === b.height
            && a.top === b.top
            && a.right === b.right);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsViewOptions = function (a, b) {
        return (a.extraEditorClassName === b.extraEditorClassName
            && a.disableMonospaceOptimizations === b.disableMonospaceOptimizations
            && equals(a.rulers, b.rulers)
            && a.ariaLabel === b.ariaLabel
            && a.renderLineNumbers === b.renderLineNumbers
            && a.renderCustomLineNumbers === b.renderCustomLineNumbers
            && a.selectOnLineNumbers === b.selectOnLineNumbers
            && a.glyphMargin === b.glyphMargin
            && a.revealHorizontalRightPadding === b.revealHorizontalRightPadding
            && a.roundedSelection === b.roundedSelection
            && a.overviewRulerLanes === b.overviewRulerLanes
            && a.overviewRulerBorder === b.overviewRulerBorder
            && a.cursorBlinking === b.cursorBlinking
            && a.mouseWheelZoom === b.mouseWheelZoom
            && a.cursorStyle === b.cursorStyle
            && a.cursorWidth === b.cursorWidth
            && a.hideCursorInOverviewRuler === b.hideCursorInOverviewRuler
            && a.scrollBeyondLastLine === b.scrollBeyondLastLine
            && a.scrollBeyondLastColumn === b.scrollBeyondLastColumn
            && a.smoothScrolling === b.smoothScrolling
            && a.stopRenderingLineAfter === b.stopRenderingLineAfter
            && a.renderWhitespace === b.renderWhitespace
            && a.renderControlCharacters === b.renderControlCharacters
            && a.fontLigatures === b.fontLigatures
            && a.renderIndentGuides === b.renderIndentGuides
            && a.highlightActiveIndentGuide === b.highlightActiveIndentGuide
            && a.renderLineHighlight === b.renderLineHighlight
            && this._equalsScrollbarOptions(a.scrollbar, b.scrollbar)
            && this._equalsMinimapOptions(a.minimap, b.minimap)
            && a.fixedOverflowWidgets === b.fixedOverflowWidgets);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsScrollbarOptions = function (a, b) {
        return (a.arrowSize === b.arrowSize
            && a.vertical === b.vertical
            && a.horizontal === b.horizontal
            && a.useShadows === b.useShadows
            && a.verticalHasArrows === b.verticalHasArrows
            && a.horizontalHasArrows === b.horizontalHasArrows
            && a.handleMouseWheel === b.handleMouseWheel
            && a.horizontalScrollbarSize === b.horizontalScrollbarSize
            && a.horizontalSliderSize === b.horizontalSliderSize
            && a.verticalScrollbarSize === b.verticalScrollbarSize
            && a.verticalSliderSize === b.verticalSliderSize
            && a.mouseWheelScrollSensitivity === b.mouseWheelScrollSensitivity);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsMinimapOptions = function (a, b) {
        return (a.enabled === b.enabled
            && a.side === b.side
            && a.showSlider === b.showSlider
            && a.renderCharacters === b.renderCharacters
            && a.maxColumn === b.maxColumn);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalFindOptions = function (a, b) {
        return (a.seedSearchStringFromSelection === b.seedSearchStringFromSelection
            && a.autoFindInSelection === b.autoFindInSelection
            && a.globalFindClipboard === b.globalFindClipboard);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsParameterHintOptions = function (a, b) {
        return (a.enabled === b.enabled
            && a.cycle === b.cycle);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsHoverOptions = function (a, b) {
        return (a.enabled === b.enabled
            && a.delay === b.delay
            && a.sticky === b.sticky);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsSuggestOptions = function (a, b) {
        if (a === b) {
            return true;
        }
        else if (!a || !b) {
            return false;
        }
        else {
            return a.filterGraceful === b.filterGraceful
                && a.snippets === b.snippets
                && a.snippetsPreventQuickSuggestions === b.snippetsPreventQuickSuggestions
                && a.localityBonus === b.localityBonus;
        }
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsWrappingInfo = function (a, b) {
        return (a.inDiffEditor === b.inDiffEditor
            && a.isDominatedByLongLines === b.isDominatedByLongLines
            && a.isWordWrapMinified === b.isWordWrapMinified
            && a.isViewportWrapping === b.isViewportWrapping
            && a.wrappingColumn === b.wrappingColumn
            && a.wrappingIndent === b.wrappingIndent
            && a.wordWrapBreakBeforeCharacters === b.wordWrapBreakBeforeCharacters
            && a.wordWrapBreakAfterCharacters === b.wordWrapBreakAfterCharacters
            && a.wordWrapBreakObtrusiveCharacters === b.wordWrapBreakObtrusiveCharacters);
    };
    /**
     * @internal
     */
    InternalEditorOptions._equalsContribOptions = function (a, b) {
        return (a.selectionClipboard === b.selectionClipboard
            && this._equalsHoverOptions(a.hover, b.hover)
            && a.links === b.links
            && a.contextmenu === b.contextmenu
            && InternalEditorOptions._equalsQuickSuggestions(a.quickSuggestions, b.quickSuggestions)
            && a.quickSuggestionsDelay === b.quickSuggestionsDelay
            && this._equalsParameterHintOptions(a.parameterHints, b.parameterHints)
            && a.iconsInSuggestions === b.iconsInSuggestions
            && a.formatOnType === b.formatOnType
            && a.formatOnPaste === b.formatOnPaste
            && a.suggestOnTriggerCharacters === b.suggestOnTriggerCharacters
            && a.acceptSuggestionOnEnter === b.acceptSuggestionOnEnter
            && a.acceptSuggestionOnCommitCharacter === b.acceptSuggestionOnCommitCharacter
            && a.wordBasedSuggestions === b.wordBasedSuggestions
            && a.suggestSelection === b.suggestSelection
            && a.suggestFontSize === b.suggestFontSize
            && a.suggestLineHeight === b.suggestLineHeight
            && a.tabCompletion === b.tabCompletion
            && this._equalsSuggestOptions(a.suggest, b.suggest)
            && a.selectionHighlight === b.selectionHighlight
            && a.occurrencesHighlight === b.occurrencesHighlight
            && a.codeLens === b.codeLens
            && a.folding === b.folding
            && a.foldingStrategy === b.foldingStrategy
            && a.showFoldingControls === b.showFoldingControls
            && a.matchBrackets === b.matchBrackets
            && this._equalFindOptions(a.find, b.find)
            && a.colorDecorators === b.colorDecorators
            && equals$1(a.codeActionsOnSave, b.codeActionsOnSave)
            && a.codeActionsOnSaveTimeout === b.codeActionsOnSaveTimeout
            && a.lightbulbEnabled === b.lightbulbEnabled);
    };
    InternalEditorOptions._equalsQuickSuggestions = function (a, b) {
        if (typeof a === 'boolean') {
            if (typeof b !== 'boolean') {
                return false;
            }
            return a === b;
        }
        if (typeof b === 'boolean') {
            return false;
        }
        return (a.comments === b.comments
            && a.other === b.other
            && a.strings === b.strings);
    };
    return InternalEditorOptions;
}());
function _boolean(value, defaultValue) {
    if (typeof value === 'undefined') {
        return defaultValue;
    }
    if (value === 'false') {
        // treat the string 'false' as false
        return false;
    }
    return Boolean(value);
}
function _booleanMap(value, defaultValue) {
    if (!value) {
        return defaultValue;
    }
    var out = Object.create(null);
    for (var _i = 0, _a = Object.keys(value); _i < _a.length; _i++) {
        var k = _a[_i];
        var v = value[k];
        if (typeof v === 'boolean') {
            out[k] = v;
        }
    }
    return out;
}
function _string(value, defaultValue) {
    if (typeof value !== 'string') {
        return defaultValue;
    }
    return value;
}
function _stringSet(value, defaultValue, allowedValues) {
    if (typeof value !== 'string') {
        return defaultValue;
    }
    if (allowedValues.indexOf(value) === -1) {
        return defaultValue;
    }
    return value;
}
function _clampedInt(value, defaultValue, minimum, maximum) {
    var r;
    if (typeof value === 'undefined') {
        r = defaultValue;
    }
    else {
        r = parseInt(value, 10);
        if (isNaN(r)) {
            r = defaultValue;
        }
    }
    r = Math.max(minimum, r);
    r = Math.min(maximum, r);
    return r | 0;
}
function _float(value, defaultValue) {
    var r = parseFloat(value);
    if (isNaN(r)) {
        r = defaultValue;
    }
    return r;
}
function _wrappingIndentFromString(wrappingIndent, defaultValue) {
    if (typeof wrappingIndent !== 'string') {
        return defaultValue;
    }
    if (wrappingIndent === 'same') {
        return 1 /* Same */;
    }
    else if (wrappingIndent === 'indent') {
        return 2 /* Indent */;
    }
    else if (wrappingIndent === 'deepIndent') {
        return 3 /* DeepIndent */;
    }
    else {
        return 0 /* None */;
    }
}
function _cursorBlinkingStyleFromString(cursorBlinkingStyle, defaultValue) {
    if (typeof cursorBlinkingStyle !== 'string') {
        return defaultValue;
    }
    switch (cursorBlinkingStyle) {
        case 'blink':
            return 1 /* Blink */;
        case 'smooth':
            return 2 /* Smooth */;
        case 'phase':
            return 3 /* Phase */;
        case 'expand':
            return 4 /* Expand */;
        case 'visible': // maintain compatibility
        case 'solid':
            return 5 /* Solid */;
    }
    return 1 /* Blink */;
}
function _scrollbarVisibilityFromString(visibility, defaultValue) {
    if (typeof visibility !== 'string') {
        return defaultValue;
    }
    switch (visibility) {
        case 'hidden':
            return 2 /* Hidden */;
        case 'visible':
            return 3 /* Visible */;
        default:
            return 1 /* Auto */;
    }
}
/**
 * @internal
 */
var EditorOptionsValidator = /** @class */ (function () {
    function EditorOptionsValidator() {
    }
    /**
     * Validate raw editor options.
     * i.e. since they can be defined by the user, they might be invalid.
     */
    EditorOptionsValidator.validate = function (opts, defaults) {
        var wordWrap = opts.wordWrap;
        {
            // Compatibility with old true or false values
            if (wordWrap === true) {
                wordWrap = 'on';
            }
            else if (wordWrap === false) {
                wordWrap = 'off';
            }
            wordWrap = _stringSet(wordWrap, defaults.wordWrap, ['off', 'on', 'wordWrapColumn', 'bounded']);
        }
        var viewInfo = this._sanitizeViewInfo(opts, defaults.viewInfo);
        var contribInfo = this._sanitizeContribInfo(opts, defaults.contribInfo);
        var configuredMulticursorModifier = undefined;
        if (typeof opts.multiCursorModifier === 'string') {
            if (opts.multiCursorModifier === 'ctrlCmd') {
                configuredMulticursorModifier = isMacintosh ? 'metaKey' : 'ctrlKey';
            }
            else {
                configuredMulticursorModifier = 'altKey';
            }
        }
        var multiCursorModifier = _stringSet(configuredMulticursorModifier, defaults.multiCursorModifier, ['altKey', 'metaKey', 'ctrlKey']);
        var autoClosingBrackets;
        var autoClosingQuotes;
        var autoSurround;
        if (typeof opts.autoClosingBrackets === 'boolean' && opts.autoClosingBrackets === false) {
            // backwards compatibility: disable all on boolean false
            autoClosingBrackets = 'never';
            autoClosingQuotes = 'never';
            autoSurround = 'never';
        }
        else {
            autoClosingBrackets = _stringSet(opts.autoClosingBrackets, defaults.autoClosingBrackets, ['always', 'languageDefined', 'beforeWhitespace', 'never']);
            autoClosingQuotes = _stringSet(opts.autoClosingQuotes, defaults.autoClosingQuotes, ['always', 'languageDefined', 'beforeWhitespace', 'never']);
            autoSurround = _stringSet(opts.autoSurround, defaults.autoSurround, ['languageDefined', 'brackets', 'quotes', 'never']);
        }
        return {
            inDiffEditor: _boolean(opts.inDiffEditor, defaults.inDiffEditor),
            wordSeparators: _string(opts.wordSeparators, defaults.wordSeparators),
            lineNumbersMinChars: _clampedInt(opts.lineNumbersMinChars, defaults.lineNumbersMinChars, 1, 10),
            lineDecorationsWidth: (typeof opts.lineDecorationsWidth === 'undefined' ? defaults.lineDecorationsWidth : opts.lineDecorationsWidth),
            readOnly: _boolean(opts.readOnly, defaults.readOnly),
            mouseStyle: _stringSet(opts.mouseStyle, defaults.mouseStyle, ['text', 'default', 'copy']),
            disableLayerHinting: _boolean(opts.disableLayerHinting, defaults.disableLayerHinting),
            automaticLayout: _boolean(opts.automaticLayout, defaults.automaticLayout),
            wordWrap: wordWrap,
            wordWrapColumn: _clampedInt(opts.wordWrapColumn, defaults.wordWrapColumn, 1, 1073741824 /* MAX_SAFE_SMALL_INTEGER */),
            wordWrapMinified: _boolean(opts.wordWrapMinified, defaults.wordWrapMinified),
            wrappingIndent: _wrappingIndentFromString(opts.wrappingIndent, defaults.wrappingIndent),
            wordWrapBreakBeforeCharacters: _string(opts.wordWrapBreakBeforeCharacters, defaults.wordWrapBreakBeforeCharacters),
            wordWrapBreakAfterCharacters: _string(opts.wordWrapBreakAfterCharacters, defaults.wordWrapBreakAfterCharacters),
            wordWrapBreakObtrusiveCharacters: _string(opts.wordWrapBreakObtrusiveCharacters, defaults.wordWrapBreakObtrusiveCharacters),
            autoClosingBrackets: autoClosingBrackets,
            autoClosingQuotes: autoClosingQuotes,
            autoSurround: autoSurround,
            autoIndent: _boolean(opts.autoIndent, defaults.autoIndent),
            dragAndDrop: _boolean(opts.dragAndDrop, defaults.dragAndDrop),
            emptySelectionClipboard: _boolean(opts.emptySelectionClipboard, defaults.emptySelectionClipboard),
            copyWithSyntaxHighlighting: _boolean(opts.copyWithSyntaxHighlighting, defaults.copyWithSyntaxHighlighting),
            useTabStops: _boolean(opts.useTabStops, defaults.useTabStops),
            multiCursorModifier: multiCursorModifier,
            multiCursorMergeOverlapping: _boolean(opts.multiCursorMergeOverlapping, defaults.multiCursorMergeOverlapping),
            accessibilitySupport: _stringSet(opts.accessibilitySupport, defaults.accessibilitySupport, ['auto', 'on', 'off']),
            showUnused: _boolean(opts.showUnused, defaults.showUnused),
            viewInfo: viewInfo,
            contribInfo: contribInfo,
        };
    };
    EditorOptionsValidator._sanitizeScrollbarOpts = function (opts, defaults, mouseWheelScrollSensitivity) {
        if (typeof opts !== 'object') {
            return defaults;
        }
        var horizontalScrollbarSize = _clampedInt(opts.horizontalScrollbarSize, defaults.horizontalScrollbarSize, 0, 1000);
        var verticalScrollbarSize = _clampedInt(opts.verticalScrollbarSize, defaults.verticalScrollbarSize, 0, 1000);
        return {
            vertical: _scrollbarVisibilityFromString(opts.vertical, defaults.vertical),
            horizontal: _scrollbarVisibilityFromString(opts.horizontal, defaults.horizontal),
            arrowSize: _clampedInt(opts.arrowSize, defaults.arrowSize, 0, 1000),
            useShadows: _boolean(opts.useShadows, defaults.useShadows),
            verticalHasArrows: _boolean(opts.verticalHasArrows, defaults.verticalHasArrows),
            horizontalHasArrows: _boolean(opts.horizontalHasArrows, defaults.horizontalHasArrows),
            horizontalScrollbarSize: horizontalScrollbarSize,
            horizontalSliderSize: _clampedInt(opts.horizontalSliderSize, horizontalScrollbarSize, 0, 1000),
            verticalScrollbarSize: verticalScrollbarSize,
            verticalSliderSize: _clampedInt(opts.verticalSliderSize, verticalScrollbarSize, 0, 1000),
            handleMouseWheel: _boolean(opts.handleMouseWheel, defaults.handleMouseWheel),
            mouseWheelScrollSensitivity: mouseWheelScrollSensitivity
        };
    };
    EditorOptionsValidator._sanitizeMinimapOpts = function (opts, defaults) {
        if (typeof opts !== 'object') {
            return defaults;
        }
        return {
            enabled: _boolean(opts.enabled, defaults.enabled),
            side: _stringSet(opts.side, defaults.side, ['right', 'left']),
            showSlider: _stringSet(opts.showSlider, defaults.showSlider, ['always', 'mouseover']),
            renderCharacters: _boolean(opts.renderCharacters, defaults.renderCharacters),
            maxColumn: _clampedInt(opts.maxColumn, defaults.maxColumn, 1, 10000),
        };
    };
    EditorOptionsValidator._santizeFindOpts = function (opts, defaults) {
        if (typeof opts !== 'object') {
            return defaults;
        }
        return {
            seedSearchStringFromSelection: _boolean(opts.seedSearchStringFromSelection, defaults.seedSearchStringFromSelection),
            autoFindInSelection: _boolean(opts.autoFindInSelection, defaults.autoFindInSelection),
            globalFindClipboard: _boolean(opts.globalFindClipboard, defaults.globalFindClipboard)
        };
    };
    EditorOptionsValidator._sanitizeParameterHintOpts = function (opts, defaults) {
        if (typeof opts !== 'object') {
            return defaults;
        }
        return {
            enabled: _boolean(opts.enabled, defaults.enabled),
            cycle: _boolean(opts.cycle, defaults.cycle)
        };
    };
    EditorOptionsValidator._santizeHoverOpts = function (_opts, defaults) {
        var opts;
        if (typeof _opts === 'boolean') {
            opts = {
                enabled: _opts
            };
        }
        else if (typeof _opts === 'object') {
            opts = _opts;
        }
        else {
            return defaults;
        }
        return {
            enabled: _boolean(opts.enabled, defaults.enabled),
            delay: _clampedInt(opts.delay, defaults.delay, 0, 10000),
            sticky: _boolean(opts.sticky, defaults.sticky)
        };
    };
    EditorOptionsValidator._sanitizeSuggestOpts = function (opts, defaults) {
        var suggestOpts = opts.suggest || {};
        return {
            filterGraceful: _boolean(suggestOpts.filterGraceful, defaults.filterGraceful),
            snippets: _stringSet(opts.snippetSuggestions, defaults.snippets, ['top', 'bottom', 'inline', 'none']),
            snippetsPreventQuickSuggestions: _boolean(suggestOpts.snippetsPreventQuickSuggestions, defaults.filterGraceful),
            localityBonus: _boolean(suggestOpts.localityBonus, defaults.localityBonus),
        };
    };
    EditorOptionsValidator._sanitizeTabCompletionOpts = function (opts, defaults) {
        if (opts === false) {
            return 'off';
        }
        else if (opts === true) {
            return 'onlySnippets';
        }
        else {
            return _stringSet(opts, defaults, ['on', 'off', 'onlySnippets']);
        }
    };
    EditorOptionsValidator._sanitizeViewInfo = function (opts, defaults) {
        var rulers = [];
        if (Array.isArray(opts.rulers)) {
            for (var i = 0, len = opts.rulers.length; i < len; i++) {
                rulers.push(_clampedInt(opts.rulers[i], 0, 0, 10000));
            }
            rulers.sort();
        }
        var renderLineNumbers = defaults.renderLineNumbers;
        var renderCustomLineNumbers = defaults.renderCustomLineNumbers;
        if (typeof opts.lineNumbers !== 'undefined') {
            var lineNumbers = opts.lineNumbers;
            // Compatibility with old true or false values
            if (lineNumbers === true) {
                lineNumbers = 'on';
            }
            else if (lineNumbers === false) {
                lineNumbers = 'off';
            }
            if (typeof lineNumbers === 'function') {
                renderLineNumbers = 4 /* Custom */;
                renderCustomLineNumbers = lineNumbers;
            }
            else if (lineNumbers === 'interval') {
                renderLineNumbers = 3 /* Interval */;
            }
            else if (lineNumbers === 'relative') {
                renderLineNumbers = 2 /* Relative */;
            }
            else if (lineNumbers === 'on') {
                renderLineNumbers = 1 /* On */;
            }
            else {
                renderLineNumbers = 0 /* Off */;
            }
        }
        var fontLigatures = _boolean(opts.fontLigatures, defaults.fontLigatures);
        var disableMonospaceOptimizations = _boolean(opts.disableMonospaceOptimizations, defaults.disableMonospaceOptimizations) || fontLigatures;
        var renderWhitespace = opts.renderWhitespace;
        {
            // Compatibility with old true or false values
            if (renderWhitespace === true) {
                renderWhitespace = 'boundary';
            }
            else if (renderWhitespace === false) {
                renderWhitespace = 'none';
            }
            renderWhitespace = _stringSet(opts.renderWhitespace, defaults.renderWhitespace, ['none', 'boundary', 'all']);
        }
        var renderLineHighlight = opts.renderLineHighlight;
        {
            // Compatibility with old true or false values
            if (renderLineHighlight === true) {
                renderLineHighlight = 'line';
            }
            else if (renderLineHighlight === false) {
                renderLineHighlight = 'none';
            }
            renderLineHighlight = _stringSet(opts.renderLineHighlight, defaults.renderLineHighlight, ['none', 'gutter', 'line', 'all']);
        }
        var mouseWheelScrollSensitivity = _float(opts.mouseWheelScrollSensitivity, defaults.scrollbar.mouseWheelScrollSensitivity);
        if (mouseWheelScrollSensitivity === 0) {
            // Disallow 0, as it would prevent/block scrolling
            mouseWheelScrollSensitivity = 1;
        }
        var scrollbar = this._sanitizeScrollbarOpts(opts.scrollbar, defaults.scrollbar, mouseWheelScrollSensitivity);
        var minimap = this._sanitizeMinimapOpts(opts.minimap, defaults.minimap);
        return {
            extraEditorClassName: _string(opts.extraEditorClassName, defaults.extraEditorClassName),
            disableMonospaceOptimizations: disableMonospaceOptimizations,
            rulers: rulers,
            ariaLabel: _string(opts.ariaLabel, defaults.ariaLabel),
            renderLineNumbers: renderLineNumbers,
            renderCustomLineNumbers: renderCustomLineNumbers,
            selectOnLineNumbers: _boolean(opts.selectOnLineNumbers, defaults.selectOnLineNumbers),
            glyphMargin: _boolean(opts.glyphMargin, defaults.glyphMargin),
            revealHorizontalRightPadding: _clampedInt(opts.revealHorizontalRightPadding, defaults.revealHorizontalRightPadding, 0, 1000),
            roundedSelection: _boolean(opts.roundedSelection, defaults.roundedSelection),
            overviewRulerLanes: _clampedInt(opts.overviewRulerLanes, defaults.overviewRulerLanes, 0, 3),
            overviewRulerBorder: _boolean(opts.overviewRulerBorder, defaults.overviewRulerBorder),
            cursorBlinking: _cursorBlinkingStyleFromString(opts.cursorBlinking, defaults.cursorBlinking),
            mouseWheelZoom: _boolean(opts.mouseWheelZoom, defaults.mouseWheelZoom),
            cursorStyle: _cursorStyleFromString(opts.cursorStyle, defaults.cursorStyle),
            cursorWidth: _clampedInt(opts.cursorWidth, defaults.cursorWidth, 0, Number.MAX_VALUE),
            hideCursorInOverviewRuler: _boolean(opts.hideCursorInOverviewRuler, defaults.hideCursorInOverviewRuler),
            scrollBeyondLastLine: _boolean(opts.scrollBeyondLastLine, defaults.scrollBeyondLastLine),
            scrollBeyondLastColumn: _clampedInt(opts.scrollBeyondLastColumn, defaults.scrollBeyondLastColumn, 0, 1073741824 /* MAX_SAFE_SMALL_INTEGER */),
            smoothScrolling: _boolean(opts.smoothScrolling, defaults.smoothScrolling),
            stopRenderingLineAfter: _clampedInt(opts.stopRenderingLineAfter, defaults.stopRenderingLineAfter, -1, 1073741824 /* MAX_SAFE_SMALL_INTEGER */),
            renderWhitespace: renderWhitespace,
            renderControlCharacters: _boolean(opts.renderControlCharacters, defaults.renderControlCharacters),
            fontLigatures: fontLigatures,
            renderIndentGuides: _boolean(opts.renderIndentGuides, defaults.renderIndentGuides),
            highlightActiveIndentGuide: _boolean(opts.highlightActiveIndentGuide, defaults.highlightActiveIndentGuide),
            renderLineHighlight: renderLineHighlight,
            scrollbar: scrollbar,
            minimap: minimap,
            fixedOverflowWidgets: _boolean(opts.fixedOverflowWidgets, defaults.fixedOverflowWidgets),
        };
    };
    EditorOptionsValidator._sanitizeContribInfo = function (opts, defaults) {
        var quickSuggestions;
        if (typeof opts.quickSuggestions === 'object') {
            quickSuggestions = __assign$1({ other: true }, opts.quickSuggestions);
        }
        else {
            quickSuggestions = _boolean(opts.quickSuggestions, defaults.quickSuggestions);
        }
        // Compatibility support for acceptSuggestionOnEnter
        if (typeof opts.acceptSuggestionOnEnter === 'boolean') {
            opts.acceptSuggestionOnEnter = opts.acceptSuggestionOnEnter ? 'on' : 'off';
        }
        var find = this._santizeFindOpts(opts.find, defaults.find);
        return {
            selectionClipboard: _boolean(opts.selectionClipboard, defaults.selectionClipboard),
            hover: this._santizeHoverOpts(opts.hover, defaults.hover),
            links: _boolean(opts.links, defaults.links),
            contextmenu: _boolean(opts.contextmenu, defaults.contextmenu),
            quickSuggestions: quickSuggestions,
            quickSuggestionsDelay: _clampedInt(opts.quickSuggestionsDelay, defaults.quickSuggestionsDelay, -1073741824 /* MIN_SAFE_SMALL_INTEGER */, 1073741824 /* MAX_SAFE_SMALL_INTEGER */),
            parameterHints: this._sanitizeParameterHintOpts(opts.parameterHints, defaults.parameterHints),
            iconsInSuggestions: _boolean(opts.iconsInSuggestions, defaults.iconsInSuggestions),
            formatOnType: _boolean(opts.formatOnType, defaults.formatOnType),
            formatOnPaste: _boolean(opts.formatOnPaste, defaults.formatOnPaste),
            suggestOnTriggerCharacters: _boolean(opts.suggestOnTriggerCharacters, defaults.suggestOnTriggerCharacters),
            acceptSuggestionOnEnter: _stringSet(opts.acceptSuggestionOnEnter, defaults.acceptSuggestionOnEnter, ['on', 'smart', 'off']),
            acceptSuggestionOnCommitCharacter: _boolean(opts.acceptSuggestionOnCommitCharacter, defaults.acceptSuggestionOnCommitCharacter),
            wordBasedSuggestions: _boolean(opts.wordBasedSuggestions, defaults.wordBasedSuggestions),
            suggestSelection: _stringSet(opts.suggestSelection, defaults.suggestSelection, ['first', 'recentlyUsed', 'recentlyUsedByPrefix']),
            suggestFontSize: _clampedInt(opts.suggestFontSize, defaults.suggestFontSize, 0, 1000),
            suggestLineHeight: _clampedInt(opts.suggestLineHeight, defaults.suggestLineHeight, 0, 1000),
            tabCompletion: this._sanitizeTabCompletionOpts(opts.tabCompletion, defaults.tabCompletion),
            suggest: this._sanitizeSuggestOpts(opts, defaults.suggest),
            selectionHighlight: _boolean(opts.selectionHighlight, defaults.selectionHighlight),
            occurrencesHighlight: _boolean(opts.occurrencesHighlight, defaults.occurrencesHighlight),
            codeLens: _boolean(opts.codeLens, defaults.codeLens),
            folding: _boolean(opts.folding, defaults.folding),
            foldingStrategy: _stringSet(opts.foldingStrategy, defaults.foldingStrategy, ['auto', 'indentation']),
            showFoldingControls: _stringSet(opts.showFoldingControls, defaults.showFoldingControls, ['always', 'mouseover']),
            matchBrackets: _boolean(opts.matchBrackets, defaults.matchBrackets),
            find: find,
            colorDecorators: _boolean(opts.colorDecorators, defaults.colorDecorators),
            lightbulbEnabled: _boolean(opts.lightbulb ? opts.lightbulb.enabled : false, defaults.lightbulbEnabled),
            codeActionsOnSave: _booleanMap(opts.codeActionsOnSave, {}),
            codeActionsOnSaveTimeout: _clampedInt(opts.codeActionsOnSaveTimeout, defaults.codeActionsOnSaveTimeout, 1, 10000)
        };
    };
    return EditorOptionsValidator;
}());
/**
 * @internal
 */
var InternalEditorOptionsFactory = /** @class */ (function () {
    function InternalEditorOptionsFactory() {
    }
    InternalEditorOptionsFactory._tweakValidatedOptions = function (opts, accessibilitySupport) {
        var accessibilityIsOn = (accessibilitySupport === 2 /* Enabled */);
        var accessibilityIsOff = (accessibilitySupport === 1 /* Disabled */);
        return {
            inDiffEditor: opts.inDiffEditor,
            wordSeparators: opts.wordSeparators,
            lineNumbersMinChars: opts.lineNumbersMinChars,
            lineDecorationsWidth: opts.lineDecorationsWidth,
            readOnly: opts.readOnly,
            mouseStyle: opts.mouseStyle,
            disableLayerHinting: opts.disableLayerHinting,
            automaticLayout: opts.automaticLayout,
            wordWrap: opts.wordWrap,
            wordWrapColumn: opts.wordWrapColumn,
            wordWrapMinified: opts.wordWrapMinified,
            wrappingIndent: opts.wrappingIndent,
            wordWrapBreakBeforeCharacters: opts.wordWrapBreakBeforeCharacters,
            wordWrapBreakAfterCharacters: opts.wordWrapBreakAfterCharacters,
            wordWrapBreakObtrusiveCharacters: opts.wordWrapBreakObtrusiveCharacters,
            autoClosingBrackets: opts.autoClosingBrackets,
            autoClosingQuotes: opts.autoClosingQuotes,
            autoSurround: opts.autoSurround,
            autoIndent: opts.autoIndent,
            dragAndDrop: opts.dragAndDrop,
            emptySelectionClipboard: opts.emptySelectionClipboard,
            copyWithSyntaxHighlighting: opts.copyWithSyntaxHighlighting,
            useTabStops: opts.useTabStops,
            multiCursorModifier: opts.multiCursorModifier,
            multiCursorMergeOverlapping: opts.multiCursorMergeOverlapping,
            accessibilitySupport: opts.accessibilitySupport,
            showUnused: opts.showUnused,
            viewInfo: {
                extraEditorClassName: opts.viewInfo.extraEditorClassName,
                disableMonospaceOptimizations: opts.viewInfo.disableMonospaceOptimizations,
                rulers: opts.viewInfo.rulers,
                ariaLabel: (accessibilityIsOff ? localize('accessibilityOffAriaLabel', "The editor is not accessible at this time. Press Alt+F1 for options.") : opts.viewInfo.ariaLabel),
                renderLineNumbers: opts.viewInfo.renderLineNumbers,
                renderCustomLineNumbers: opts.viewInfo.renderCustomLineNumbers,
                selectOnLineNumbers: opts.viewInfo.selectOnLineNumbers,
                glyphMargin: opts.viewInfo.glyphMargin,
                revealHorizontalRightPadding: opts.viewInfo.revealHorizontalRightPadding,
                roundedSelection: (accessibilityIsOn ? false : opts.viewInfo.roundedSelection),
                overviewRulerLanes: opts.viewInfo.overviewRulerLanes,
                overviewRulerBorder: opts.viewInfo.overviewRulerBorder,
                cursorBlinking: opts.viewInfo.cursorBlinking,
                mouseWheelZoom: opts.viewInfo.mouseWheelZoom,
                cursorStyle: opts.viewInfo.cursorStyle,
                cursorWidth: opts.viewInfo.cursorWidth,
                hideCursorInOverviewRuler: opts.viewInfo.hideCursorInOverviewRuler,
                scrollBeyondLastLine: opts.viewInfo.scrollBeyondLastLine,
                scrollBeyondLastColumn: opts.viewInfo.scrollBeyondLastColumn,
                smoothScrolling: opts.viewInfo.smoothScrolling,
                stopRenderingLineAfter: opts.viewInfo.stopRenderingLineAfter,
                renderWhitespace: (accessibilityIsOn ? 'none' : opts.viewInfo.renderWhitespace),
                renderControlCharacters: (accessibilityIsOn ? false : opts.viewInfo.renderControlCharacters),
                fontLigatures: (accessibilityIsOn ? false : opts.viewInfo.fontLigatures),
                renderIndentGuides: (accessibilityIsOn ? false : opts.viewInfo.renderIndentGuides),
                highlightActiveIndentGuide: opts.viewInfo.highlightActiveIndentGuide,
                renderLineHighlight: opts.viewInfo.renderLineHighlight,
                scrollbar: opts.viewInfo.scrollbar,
                minimap: {
                    enabled: (accessibilityIsOn ? false : opts.viewInfo.minimap.enabled),
                    side: opts.viewInfo.minimap.side,
                    renderCharacters: opts.viewInfo.minimap.renderCharacters,
                    showSlider: opts.viewInfo.minimap.showSlider,
                    maxColumn: opts.viewInfo.minimap.maxColumn
                },
                fixedOverflowWidgets: opts.viewInfo.fixedOverflowWidgets
            },
            contribInfo: {
                selectionClipboard: opts.contribInfo.selectionClipboard,
                hover: opts.contribInfo.hover,
                links: (accessibilityIsOn ? false : opts.contribInfo.links),
                contextmenu: opts.contribInfo.contextmenu,
                quickSuggestions: opts.contribInfo.quickSuggestions,
                quickSuggestionsDelay: opts.contribInfo.quickSuggestionsDelay,
                parameterHints: opts.contribInfo.parameterHints,
                iconsInSuggestions: opts.contribInfo.iconsInSuggestions,
                formatOnType: opts.contribInfo.formatOnType,
                formatOnPaste: opts.contribInfo.formatOnPaste,
                suggestOnTriggerCharacters: opts.contribInfo.suggestOnTriggerCharacters,
                acceptSuggestionOnEnter: opts.contribInfo.acceptSuggestionOnEnter,
                acceptSuggestionOnCommitCharacter: opts.contribInfo.acceptSuggestionOnCommitCharacter,
                wordBasedSuggestions: opts.contribInfo.wordBasedSuggestions,
                suggestSelection: opts.contribInfo.suggestSelection,
                suggestFontSize: opts.contribInfo.suggestFontSize,
                suggestLineHeight: opts.contribInfo.suggestLineHeight,
                tabCompletion: opts.contribInfo.tabCompletion,
                suggest: opts.contribInfo.suggest,
                selectionHighlight: (accessibilityIsOn ? false : opts.contribInfo.selectionHighlight),
                occurrencesHighlight: (accessibilityIsOn ? false : opts.contribInfo.occurrencesHighlight),
                codeLens: (accessibilityIsOn ? false : opts.contribInfo.codeLens),
                folding: (accessibilityIsOn ? false : opts.contribInfo.folding),
                foldingStrategy: opts.contribInfo.foldingStrategy,
                showFoldingControls: opts.contribInfo.showFoldingControls,
                matchBrackets: (accessibilityIsOn ? false : opts.contribInfo.matchBrackets),
                find: opts.contribInfo.find,
                colorDecorators: opts.contribInfo.colorDecorators,
                lightbulbEnabled: opts.contribInfo.lightbulbEnabled,
                codeActionsOnSave: opts.contribInfo.codeActionsOnSave,
                codeActionsOnSaveTimeout: opts.contribInfo.codeActionsOnSaveTimeout
            }
        };
    };
    InternalEditorOptionsFactory.createInternalEditorOptions = function (env, _opts) {
        var accessibilitySupport;
        if (_opts.accessibilitySupport === 'auto') {
            // The editor reads the `accessibilitySupport` from the environment
            accessibilitySupport = env.accessibilitySupport;
        }
        else if (_opts.accessibilitySupport === 'on') {
            accessibilitySupport = 2 /* Enabled */;
        }
        else {
            accessibilitySupport = 1 /* Disabled */;
        }
        // Disable some non critical features to get as best performance as possible
        // See https://github.com/Microsoft/vscode/issues/26730
        var opts = this._tweakValidatedOptions(_opts, accessibilitySupport);
        var lineDecorationsWidth;
        if (typeof opts.lineDecorationsWidth === 'string' && /^\d+(\.\d+)?ch$/.test(opts.lineDecorationsWidth)) {
            var multiple = parseFloat(opts.lineDecorationsWidth.substr(0, opts.lineDecorationsWidth.length - 2));
            lineDecorationsWidth = multiple * env.fontInfo.typicalHalfwidthCharacterWidth;
        }
        else {
            lineDecorationsWidth = _clampedInt(opts.lineDecorationsWidth, 0, 0, 1000);
        }
        if (opts.contribInfo.folding) {
            lineDecorationsWidth += 16;
        }
        var layoutInfo = EditorLayoutProvider.compute({
            outerWidth: env.outerWidth,
            outerHeight: env.outerHeight,
            showGlyphMargin: opts.viewInfo.glyphMargin,
            lineHeight: env.fontInfo.lineHeight,
            showLineNumbers: (opts.viewInfo.renderLineNumbers !== 0 /* Off */),
            lineNumbersMinChars: opts.lineNumbersMinChars,
            lineNumbersDigitCount: env.lineNumbersDigitCount,
            lineDecorationsWidth: lineDecorationsWidth,
            typicalHalfwidthCharacterWidth: env.fontInfo.typicalHalfwidthCharacterWidth,
            maxDigitWidth: env.fontInfo.maxDigitWidth,
            verticalScrollbarWidth: opts.viewInfo.scrollbar.verticalScrollbarSize,
            horizontalScrollbarHeight: opts.viewInfo.scrollbar.horizontalScrollbarSize,
            scrollbarArrowSize: opts.viewInfo.scrollbar.arrowSize,
            verticalScrollbarHasArrows: opts.viewInfo.scrollbar.verticalHasArrows,
            minimap: opts.viewInfo.minimap.enabled,
            minimapSide: opts.viewInfo.minimap.side,
            minimapRenderCharacters: opts.viewInfo.minimap.renderCharacters,
            minimapMaxColumn: opts.viewInfo.minimap.maxColumn,
            pixelRatio: env.pixelRatio
        });
        var bareWrappingInfo = null;
        {
            var wordWrap = opts.wordWrap;
            var wordWrapColumn = opts.wordWrapColumn;
            var wordWrapMinified = opts.wordWrapMinified;
            if (accessibilitySupport === 2 /* Enabled */) {
                // See https://github.com/Microsoft/vscode/issues/27766
                // Never enable wrapping when a screen reader is attached
                // because arrow down etc. will not move the cursor in the way
                // a screen reader expects.
                bareWrappingInfo = {
                    isWordWrapMinified: false,
                    isViewportWrapping: false,
                    wrappingColumn: -1
                };
            }
            else if (wordWrapMinified && env.isDominatedByLongLines) {
                // Force viewport width wrapping if model is dominated by long lines
                bareWrappingInfo = {
                    isWordWrapMinified: true,
                    isViewportWrapping: true,
                    wrappingColumn: Math.max(1, layoutInfo.viewportColumn)
                };
            }
            else if (wordWrap === 'on') {
                bareWrappingInfo = {
                    isWordWrapMinified: false,
                    isViewportWrapping: true,
                    wrappingColumn: Math.max(1, layoutInfo.viewportColumn)
                };
            }
            else if (wordWrap === 'bounded') {
                bareWrappingInfo = {
                    isWordWrapMinified: false,
                    isViewportWrapping: true,
                    wrappingColumn: Math.min(Math.max(1, layoutInfo.viewportColumn), wordWrapColumn)
                };
            }
            else if (wordWrap === 'wordWrapColumn') {
                bareWrappingInfo = {
                    isWordWrapMinified: false,
                    isViewportWrapping: false,
                    wrappingColumn: wordWrapColumn
                };
            }
            else {
                bareWrappingInfo = {
                    isWordWrapMinified: false,
                    isViewportWrapping: false,
                    wrappingColumn: -1
                };
            }
        }
        var wrappingInfo = {
            inDiffEditor: opts.inDiffEditor,
            isDominatedByLongLines: env.isDominatedByLongLines,
            isWordWrapMinified: bareWrappingInfo.isWordWrapMinified,
            isViewportWrapping: bareWrappingInfo.isViewportWrapping,
            wrappingColumn: bareWrappingInfo.wrappingColumn,
            wrappingIndent: opts.wrappingIndent,
            wordWrapBreakBeforeCharacters: opts.wordWrapBreakBeforeCharacters,
            wordWrapBreakAfterCharacters: opts.wordWrapBreakAfterCharacters,
            wordWrapBreakObtrusiveCharacters: opts.wordWrapBreakObtrusiveCharacters,
        };
        var className = 'monaco-editor';
        if (opts.viewInfo.extraEditorClassName) {
            className += ' ' + opts.viewInfo.extraEditorClassName;
        }
        if (env.extraEditorClassName) {
            className += ' ' + env.extraEditorClassName;
        }
        if (opts.viewInfo.fontLigatures) {
            className += ' enable-ligatures';
        }
        if (opts.mouseStyle === 'default') {
            className += ' mouse-default';
        }
        else if (opts.mouseStyle === 'copy') {
            className += ' mouse-copy';
        }
        return new InternalEditorOptions({
            canUseLayerHinting: opts.disableLayerHinting ? false : true,
            pixelRatio: env.pixelRatio,
            editorClassName: className,
            lineHeight: env.fontInfo.lineHeight,
            readOnly: opts.readOnly,
            accessibilitySupport: accessibilitySupport,
            multiCursorModifier: opts.multiCursorModifier,
            multiCursorMergeOverlapping: opts.multiCursorMergeOverlapping,
            wordSeparators: opts.wordSeparators,
            autoClosingBrackets: opts.autoClosingBrackets,
            autoClosingQuotes: opts.autoClosingQuotes,
            autoSurround: opts.autoSurround,
            autoIndent: opts.autoIndent,
            useTabStops: opts.useTabStops,
            tabFocusMode: opts.readOnly ? true : env.tabFocusMode,
            dragAndDrop: opts.dragAndDrop,
            emptySelectionClipboard: opts.emptySelectionClipboard && env.emptySelectionClipboard,
            copyWithSyntaxHighlighting: opts.copyWithSyntaxHighlighting,
            layoutInfo: layoutInfo,
            fontInfo: env.fontInfo,
            viewInfo: opts.viewInfo,
            wrappingInfo: wrappingInfo,
            contribInfo: opts.contribInfo,
            showUnused: opts.showUnused,
        });
    };
    return InternalEditorOptionsFactory;
}());
/**
 * @internal
 */
var EditorLayoutProvider = /** @class */ (function () {
    function EditorLayoutProvider() {
    }
    EditorLayoutProvider.compute = function (_opts) {
        var outerWidth = _opts.outerWidth | 0;
        var outerHeight = _opts.outerHeight | 0;
        var showGlyphMargin = _opts.showGlyphMargin;
        var lineHeight = _opts.lineHeight | 0;
        var showLineNumbers = _opts.showLineNumbers;
        var lineNumbersMinChars = _opts.lineNumbersMinChars | 0;
        var lineNumbersDigitCount = _opts.lineNumbersDigitCount | 0;
        var lineDecorationsWidth = _opts.lineDecorationsWidth | 0;
        var typicalHalfwidthCharacterWidth = _opts.typicalHalfwidthCharacterWidth;
        var maxDigitWidth = _opts.maxDigitWidth;
        var verticalScrollbarWidth = _opts.verticalScrollbarWidth | 0;
        var verticalScrollbarHasArrows = _opts.verticalScrollbarHasArrows;
        var scrollbarArrowSize = _opts.scrollbarArrowSize | 0;
        var horizontalScrollbarHeight = _opts.horizontalScrollbarHeight | 0;
        var minimap = _opts.minimap;
        var minimapSide = _opts.minimapSide;
        var minimapRenderCharacters = _opts.minimapRenderCharacters;
        var minimapMaxColumn = _opts.minimapMaxColumn | 0;
        var pixelRatio = _opts.pixelRatio;
        var lineNumbersWidth = 0;
        if (showLineNumbers) {
            var digitCount = Math.max(lineNumbersDigitCount, lineNumbersMinChars);
            lineNumbersWidth = Math.round(digitCount * maxDigitWidth);
        }
        var glyphMarginWidth = 0;
        if (showGlyphMargin) {
            glyphMarginWidth = lineHeight;
        }
        var glyphMarginLeft = 0;
        var lineNumbersLeft = glyphMarginLeft + glyphMarginWidth;
        var decorationsLeft = lineNumbersLeft + lineNumbersWidth;
        var contentLeft = decorationsLeft + lineDecorationsWidth;
        var remainingWidth = outerWidth - glyphMarginWidth - lineNumbersWidth - lineDecorationsWidth;
        var renderMinimap;
        var minimapLeft;
        var minimapWidth;
        var contentWidth;
        if (!minimap) {
            minimapLeft = 0;
            minimapWidth = 0;
            renderMinimap = 0 /* None */;
            contentWidth = remainingWidth;
        }
        else {
            var minimapCharWidth = void 0;
            if (pixelRatio >= 2) {
                renderMinimap = minimapRenderCharacters ? 2 /* Large */ : 4 /* LargeBlocks */;
                minimapCharWidth = 2 / pixelRatio;
            }
            else {
                renderMinimap = minimapRenderCharacters ? 1 /* Small */ : 3 /* SmallBlocks */;
                minimapCharWidth = 1 / pixelRatio;
            }
            // Given:
            // (leaving 2px for the cursor to have space after the last character)
            // viewportColumn = (contentWidth - verticalScrollbarWidth - 2) / typicalHalfwidthCharacterWidth
            // minimapWidth = viewportColumn * minimapCharWidth
            // contentWidth = remainingWidth - minimapWidth
            // What are good values for contentWidth and minimapWidth ?
            // minimapWidth = ((contentWidth - verticalScrollbarWidth - 2) / typicalHalfwidthCharacterWidth) * minimapCharWidth
            // typicalHalfwidthCharacterWidth * minimapWidth = (contentWidth - verticalScrollbarWidth - 2) * minimapCharWidth
            // typicalHalfwidthCharacterWidth * minimapWidth = (remainingWidth - minimapWidth - verticalScrollbarWidth - 2) * minimapCharWidth
            // (typicalHalfwidthCharacterWidth + minimapCharWidth) * minimapWidth = (remainingWidth - verticalScrollbarWidth - 2) * minimapCharWidth
            // minimapWidth = ((remainingWidth - verticalScrollbarWidth - 2) * minimapCharWidth) / (typicalHalfwidthCharacterWidth + minimapCharWidth)
            minimapWidth = Math.max(0, Math.floor(((remainingWidth - verticalScrollbarWidth - 2) * minimapCharWidth) / (typicalHalfwidthCharacterWidth + minimapCharWidth)));
            var minimapColumns = minimapWidth / minimapCharWidth;
            if (minimapColumns > minimapMaxColumn) {
                minimapWidth = Math.floor(minimapMaxColumn * minimapCharWidth);
            }
            contentWidth = remainingWidth - minimapWidth;
            if (minimapSide === 'left') {
                minimapLeft = 0;
                glyphMarginLeft += minimapWidth;
                lineNumbersLeft += minimapWidth;
                decorationsLeft += minimapWidth;
                contentLeft += minimapWidth;
            }
            else {
                minimapLeft = outerWidth - minimapWidth - verticalScrollbarWidth;
            }
        }
        // (leaving 2px for the cursor to have space after the last character)
        var viewportColumn = Math.max(1, Math.floor((contentWidth - verticalScrollbarWidth - 2) / typicalHalfwidthCharacterWidth));
        var verticalArrowSize = (verticalScrollbarHasArrows ? scrollbarArrowSize : 0);
        return {
            width: outerWidth,
            height: outerHeight,
            glyphMarginLeft: glyphMarginLeft,
            glyphMarginWidth: glyphMarginWidth,
            glyphMarginHeight: outerHeight,
            lineNumbersLeft: lineNumbersLeft,
            lineNumbersWidth: lineNumbersWidth,
            lineNumbersHeight: outerHeight,
            decorationsLeft: decorationsLeft,
            decorationsWidth: lineDecorationsWidth,
            decorationsHeight: outerHeight,
            contentLeft: contentLeft,
            contentWidth: contentWidth,
            contentHeight: outerHeight,
            renderMinimap: renderMinimap,
            minimapLeft: minimapLeft,
            minimapWidth: minimapWidth,
            viewportColumn: viewportColumn,
            verticalScrollbarWidth: verticalScrollbarWidth,
            horizontalScrollbarHeight: horizontalScrollbarHeight,
            overviewRuler: {
                top: verticalArrowSize,
                width: verticalScrollbarWidth,
                height: (outerHeight - 2 * verticalArrowSize),
                right: 0
            }
        };
    };
    return EditorLayoutProvider;
}());
var DEFAULT_WINDOWS_FONT_FAMILY = 'Consolas, \'Courier New\', monospace';
var DEFAULT_MAC_FONT_FAMILY = 'Menlo, Monaco, \'Courier New\', monospace';
var DEFAULT_LINUX_FONT_FAMILY = '\'Droid Sans Mono\', \'monospace\', monospace, \'Droid Sans Fallback\'';
/**
 * @internal
 */
var EDITOR_FONT_DEFAULTS = {
    fontFamily: (isMacintosh ? DEFAULT_MAC_FONT_FAMILY : (isLinux ? DEFAULT_LINUX_FONT_FAMILY : DEFAULT_WINDOWS_FONT_FAMILY)),
    fontWeight: 'normal',
    fontSize: (isMacintosh ? 12 : 14),
    lineHeight: 0,
    letterSpacing: 0,
};
/**
 * @internal
 */
var EDITOR_MODEL_DEFAULTS = {
    tabSize: 4,
    insertSpaces: true,
    detectIndentation: true,
    trimAutoWhitespace: true,
    largeFileOptimizations: true
};
/**
 * @internal
 */
var EDITOR_DEFAULTS = {
    inDiffEditor: false,
    wordSeparators: USUAL_WORD_SEPARATORS,
    lineNumbersMinChars: 5,
    lineDecorationsWidth: 10,
    readOnly: false,
    mouseStyle: 'text',
    disableLayerHinting: false,
    automaticLayout: false,
    wordWrap: 'off',
    wordWrapColumn: 80,
    wordWrapMinified: true,
    wrappingIndent: 1 /* Same */,
    wordWrapBreakBeforeCharacters: '([{‘“〈《「『【〔（［｛｢£¥＄￡￥+＋',
    wordWrapBreakAfterCharacters: ' \t})]?|&,;¢°′″‰℃、。｡､￠，．：；？！％・･ゝゞヽヾーァィゥェォッャュョヮヵヶぁぃぅぇぉっゃゅょゎゕゖㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ々〻ｧｨｩｪｫｬｭｮｯｰ”〉》」』】〕）］｝｣',
    wordWrapBreakObtrusiveCharacters: '.',
    autoClosingBrackets: 'languageDefined',
    autoClosingQuotes: 'languageDefined',
    autoSurround: 'languageDefined',
    autoIndent: true,
    dragAndDrop: true,
    emptySelectionClipboard: true,
    copyWithSyntaxHighlighting: true,
    useTabStops: true,
    multiCursorModifier: 'altKey',
    multiCursorMergeOverlapping: true,
    accessibilitySupport: 'auto',
    showUnused: true,
    viewInfo: {
        extraEditorClassName: '',
        disableMonospaceOptimizations: false,
        rulers: [],
        ariaLabel: localize('editorViewAccessibleLabel', "Editor content"),
        renderLineNumbers: 1 /* On */,
        renderCustomLineNumbers: null,
        selectOnLineNumbers: true,
        glyphMargin: true,
        revealHorizontalRightPadding: 30,
        roundedSelection: true,
        overviewRulerLanes: 2,
        overviewRulerBorder: true,
        cursorBlinking: 1 /* Blink */,
        mouseWheelZoom: false,
        cursorStyle: TextEditorCursorStyle.Line,
        cursorWidth: 0,
        hideCursorInOverviewRuler: false,
        scrollBeyondLastLine: true,
        scrollBeyondLastColumn: 5,
        smoothScrolling: false,
        stopRenderingLineAfter: 10000,
        renderWhitespace: 'none',
        renderControlCharacters: false,
        fontLigatures: false,
        renderIndentGuides: true,
        highlightActiveIndentGuide: true,
        renderLineHighlight: 'line',
        scrollbar: {
            vertical: 1 /* Auto */,
            horizontal: 1 /* Auto */,
            arrowSize: 11,
            useShadows: true,
            verticalHasArrows: false,
            horizontalHasArrows: false,
            horizontalScrollbarSize: 10,
            horizontalSliderSize: 10,
            verticalScrollbarSize: 14,
            verticalSliderSize: 14,
            handleMouseWheel: true,
            mouseWheelScrollSensitivity: 1,
        },
        minimap: {
            enabled: true,
            side: 'right',
            showSlider: 'mouseover',
            renderCharacters: true,
            maxColumn: 120
        },
        fixedOverflowWidgets: false,
    },
    contribInfo: {
        selectionClipboard: true,
        hover: {
            enabled: true,
            delay: 300,
            sticky: true
        },
        links: true,
        contextmenu: true,
        quickSuggestions: { other: true, comments: false, strings: false },
        quickSuggestionsDelay: 10,
        parameterHints: {
            enabled: true,
            cycle: false
        },
        iconsInSuggestions: true,
        formatOnType: false,
        formatOnPaste: false,
        suggestOnTriggerCharacters: true,
        acceptSuggestionOnEnter: 'on',
        acceptSuggestionOnCommitCharacter: true,
        wordBasedSuggestions: true,
        suggestSelection: 'recentlyUsed',
        suggestFontSize: 0,
        suggestLineHeight: 0,
        tabCompletion: 'off',
        suggest: {
            filterGraceful: true,
            snippets: 'inline',
            snippetsPreventQuickSuggestions: true,
            localityBonus: false
        },
        selectionHighlight: true,
        occurrencesHighlight: true,
        codeLens: true,
        folding: true,
        foldingStrategy: 'auto',
        showFoldingControls: 'mouseover',
        matchBrackets: true,
        find: {
            seedSearchStringFromSelection: true,
            autoFindInSelection: false,
            globalFindClipboard: false
        },
        colorDecorators: true,
        lightbulbEnabled: true,
        codeActionsOnSave: {},
        codeActionsOnSaveTimeout: 750
    },
};

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Vertical Lane in the overview ruler of the editor.
 */
var OverviewRulerLane;
(function (OverviewRulerLane) {
    OverviewRulerLane[OverviewRulerLane["Left"] = 1] = "Left";
    OverviewRulerLane[OverviewRulerLane["Center"] = 2] = "Center";
    OverviewRulerLane[OverviewRulerLane["Right"] = 4] = "Right";
    OverviewRulerLane[OverviewRulerLane["Full"] = 7] = "Full";
})(OverviewRulerLane || (OverviewRulerLane = {}));
var TextModelResolvedOptions = /** @class */ (function () {
    /**
     * @internal
     */
    function TextModelResolvedOptions(src) {
        this.tabSize = src.tabSize | 0;
        this.insertSpaces = Boolean(src.insertSpaces);
        this.defaultEOL = src.defaultEOL | 0;
        this.trimAutoWhitespace = Boolean(src.trimAutoWhitespace);
    }
    /**
     * @internal
     */
    TextModelResolvedOptions.prototype.equals = function (other) {
        return (this.tabSize === other.tabSize
            && this.insertSpaces === other.insertSpaces
            && this.defaultEOL === other.defaultEOL
            && this.trimAutoWhitespace === other.trimAutoWhitespace);
    };
    /**
     * @internal
     */
    TextModelResolvedOptions.prototype.createChangeEvent = function (newOpts) {
        return {
            tabSize: this.tabSize !== newOpts.tabSize,
            insertSpaces: this.insertSpaces !== newOpts.insertSpaces,
            trimAutoWhitespace: this.trimAutoWhitespace !== newOpts.trimAutoWhitespace,
        };
    };
    return TextModelResolvedOptions;
}());
var FindMatch = /** @class */ (function () {
    /**
     * @internal
     */
    function FindMatch(range, matches) {
        this.range = range;
        this.matches = matches;
    }
    return FindMatch;
}());
/**
 * @internal
 */
var ApplyEditsResult = /** @class */ (function () {
    function ApplyEditsResult(reverseEdits, changes, trimAutoWhitespaceLineNumbers) {
        this.reverseEdits = reverseEdits;
        this.changes = changes;
        this.trimAutoWhitespaceLineNumbers = trimAutoWhitespaceLineNumbers;
    }
    return ApplyEditsResult;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var EditStackElement = /** @class */ (function () {
    function EditStackElement(beforeVersionId, beforeCursorState) {
        this.beforeVersionId = beforeVersionId;
        this.beforeCursorState = beforeCursorState;
        this.afterCursorState = null;
        this.afterVersionId = -1;
        this.editOperations = [];
    }
    EditStackElement.prototype.undo = function (model) {
        // Apply all operations in reverse order
        for (var i = this.editOperations.length - 1; i >= 0; i--) {
            this.editOperations[i] = {
                operations: model.applyEdits(this.editOperations[i].operations)
            };
        }
    };
    EditStackElement.prototype.redo = function (model) {
        // Apply all operations
        for (var i = 0; i < this.editOperations.length; i++) {
            this.editOperations[i] = {
                operations: model.applyEdits(this.editOperations[i].operations)
            };
        }
    };
    return EditStackElement;
}());
function getModelEOL(model) {
    var eol = model.getEOL();
    if (eol === '\n') {
        return 0 /* LF */;
    }
    else {
        return 1 /* CRLF */;
    }
}
var EOLStackElement = /** @class */ (function () {
    function EOLStackElement(beforeVersionId, setEOL) {
        this.beforeVersionId = beforeVersionId;
        this.beforeCursorState = null;
        this.afterCursorState = null;
        this.afterVersionId = -1;
        this.eol = setEOL;
    }
    EOLStackElement.prototype.undo = function (model) {
        var redoEOL = getModelEOL(model);
        model.setEOL(this.eol);
        this.eol = redoEOL;
    };
    EOLStackElement.prototype.redo = function (model) {
        var undoEOL = getModelEOL(model);
        model.setEOL(this.eol);
        this.eol = undoEOL;
    };
    return EOLStackElement;
}());
var EditStack = /** @class */ (function () {
    function EditStack(model) {
        this.model = model;
        this.currentOpenStackElement = null;
        this.past = [];
        this.future = [];
    }
    EditStack.prototype.pushStackElement = function () {
        if (this.currentOpenStackElement !== null) {
            this.past.push(this.currentOpenStackElement);
            this.currentOpenStackElement = null;
        }
    };
    EditStack.prototype.clear = function () {
        this.currentOpenStackElement = null;
        this.past = [];
        this.future = [];
    };
    EditStack.prototype.pushEOL = function (eol) {
        // No support for parallel universes :(
        this.future = [];
        if (this.currentOpenStackElement) {
            this.pushStackElement();
        }
        var prevEOL = getModelEOL(this.model);
        var stackElement = new EOLStackElement(this.model.getAlternativeVersionId(), prevEOL);
        this.model.setEOL(eol);
        stackElement.afterVersionId = this.model.getVersionId();
        this.currentOpenStackElement = stackElement;
        this.pushStackElement();
    };
    EditStack.prototype.pushEditOperation = function (beforeCursorState, editOperations, cursorStateComputer) {
        // No support for parallel universes :(
        this.future = [];
        var stackElement = null;
        if (this.currentOpenStackElement) {
            if (this.currentOpenStackElement instanceof EditStackElement) {
                stackElement = this.currentOpenStackElement;
            }
            else {
                this.pushStackElement();
            }
        }
        if (!this.currentOpenStackElement) {
            stackElement = new EditStackElement(this.model.getAlternativeVersionId(), beforeCursorState);
            this.currentOpenStackElement = stackElement;
        }
        var inverseEditOperation = {
            operations: this.model.applyEdits(editOperations)
        };
        stackElement.editOperations.push(inverseEditOperation);
        stackElement.afterCursorState = EditStack._computeCursorState(cursorStateComputer, inverseEditOperation.operations);
        stackElement.afterVersionId = this.model.getVersionId();
        return stackElement.afterCursorState;
    };
    EditStack._computeCursorState = function (cursorStateComputer, inverseEditOperations) {
        try {
            return cursorStateComputer ? cursorStateComputer(inverseEditOperations) : null;
        }
        catch (e) {
            onUnexpectedError(e);
            return null;
        }
    };
    EditStack.prototype.undo = function () {
        this.pushStackElement();
        if (this.past.length > 0) {
            var pastStackElement = this.past.pop();
            try {
                pastStackElement.undo(this.model);
            }
            catch (e) {
                onUnexpectedError(e);
                this.clear();
                return null;
            }
            this.future.push(pastStackElement);
            return {
                selections: pastStackElement.beforeCursorState,
                recordedVersionId: pastStackElement.beforeVersionId
            };
        }
        return null;
    };
    EditStack.prototype.canUndo = function () {
        return (this.past.length > 0);
    };
    EditStack.prototype.redo = function () {
        if (this.future.length > 0) {
            var futureStackElement = this.future.pop();
            try {
                futureStackElement.redo(this.model);
            }
            catch (e) {
                onUnexpectedError(e);
                this.clear();
                return null;
            }
            this.past.push(futureStackElement);
            return {
                selections: futureStackElement.afterCursorState,
                recordedVersionId: futureStackElement.afterVersionId
            };
        }
        return null;
    };
    EditStack.prototype.canRedo = function () {
        return (this.future.length > 0);
    };
    return EditStack;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var SpacesDiffResult = /** @class */ (function () {
    function SpacesDiffResult() {
    }
    return SpacesDiffResult;
}());
/**
 * Compute the diff in spaces between two line's indentation.
 */
function spacesDiff(a, aLength, b, bLength, result) {
    result.spacesDiff = 0;
    result.looksLikeAlignment = false;
    // This can go both ways (e.g.):
    //  - a: "\t"
    //  - b: "\t    "
    //  => This should count 1 tab and 4 spaces
    var i;
    for (i = 0; i < aLength && i < bLength; i++) {
        var aCharCode = a.charCodeAt(i);
        var bCharCode = b.charCodeAt(i);
        if (aCharCode !== bCharCode) {
            break;
        }
    }
    var aSpacesCnt = 0, aTabsCount = 0;
    for (var j = i; j < aLength; j++) {
        var aCharCode = a.charCodeAt(j);
        if (aCharCode === 32 /* Space */) {
            aSpacesCnt++;
        }
        else {
            aTabsCount++;
        }
    }
    var bSpacesCnt = 0, bTabsCount = 0;
    for (var j = i; j < bLength; j++) {
        var bCharCode = b.charCodeAt(j);
        if (bCharCode === 32 /* Space */) {
            bSpacesCnt++;
        }
        else {
            bTabsCount++;
        }
    }
    if (aSpacesCnt > 0 && aTabsCount > 0) {
        return;
    }
    if (bSpacesCnt > 0 && bTabsCount > 0) {
        return;
    }
    var tabsDiff = Math.abs(aTabsCount - bTabsCount);
    var spacesDiff = Math.abs(aSpacesCnt - bSpacesCnt);
    if (tabsDiff === 0) {
        // check if the indentation difference might be caused by alignment reasons
        // sometime folks like to align their code, but this should not be used as a hint
        result.spacesDiff = spacesDiff;
        if (spacesDiff > 0 && 0 <= bSpacesCnt - 1 && bSpacesCnt - 1 < a.length && bSpacesCnt < b.length) {
            if (b.charCodeAt(bSpacesCnt) !== 32 /* Space */ && a.charCodeAt(bSpacesCnt - 1) === 32 /* Space */) {
                // This looks like an alignment desire: e.g.
                // const a = b + c,
                //       d = b - c;
                result.looksLikeAlignment = true;
            }
        }
        return;
    }
    if (spacesDiff % tabsDiff === 0) {
        result.spacesDiff = spacesDiff / tabsDiff;
        return;
    }
}
function guessIndentation(source, defaultTabSize, defaultInsertSpaces) {
    // Look at most at the first 10k lines
    var linesCount = Math.min(source.getLineCount(), 10000);
    var linesIndentedWithTabsCount = 0; // number of lines that contain at least one tab in indentation
    var linesIndentedWithSpacesCount = 0; // number of lines that contain only spaces in indentation
    var previousLineText = ''; // content of latest line that contained non-whitespace chars
    var previousLineIndentation = 0; // index at which latest line contained the first non-whitespace char
    var ALLOWED_TAB_SIZE_GUESSES = [2, 4, 6, 8, 3, 5, 7]; // prefer even guesses for `tabSize`, limit to [2, 8].
    var MAX_ALLOWED_TAB_SIZE_GUESS = 8; // max(ALLOWED_TAB_SIZE_GUESSES) = 8
    var spacesDiffCount = [0, 0, 0, 0, 0, 0, 0, 0, 0]; // `tabSize` scores
    var tmp = new SpacesDiffResult();
    for (var lineNumber = 1; lineNumber <= linesCount; lineNumber++) {
        var currentLineLength = source.getLineLength(lineNumber);
        var currentLineText = source.getLineContent(lineNumber);
        // if the text buffer is chunk based, so long lines are cons-string, v8 will flattern the string when we check charCode.
        // checking charCode on chunks directly is cheaper.
        var useCurrentLineText = (currentLineLength <= 65536);
        var currentLineHasContent = false; // does `currentLineText` contain non-whitespace chars
        var currentLineIndentation = 0; // index at which `currentLineText` contains the first non-whitespace char
        var currentLineSpacesCount = 0; // count of spaces found in `currentLineText` indentation
        var currentLineTabsCount = 0; // count of tabs found in `currentLineText` indentation
        for (var j = 0, lenJ = currentLineLength; j < lenJ; j++) {
            var charCode = (useCurrentLineText ? currentLineText.charCodeAt(j) : source.getLineCharCode(lineNumber, j));
            if (charCode === 9 /* Tab */) {
                currentLineTabsCount++;
            }
            else if (charCode === 32 /* Space */) {
                currentLineSpacesCount++;
            }
            else {
                // Hit non whitespace character on this line
                currentLineHasContent = true;
                currentLineIndentation = j;
                break;
            }
        }
        // Ignore empty or only whitespace lines
        if (!currentLineHasContent) {
            continue;
        }
        if (currentLineTabsCount > 0) {
            linesIndentedWithTabsCount++;
        }
        else if (currentLineSpacesCount > 1) {
            linesIndentedWithSpacesCount++;
        }
        spacesDiff(previousLineText, previousLineIndentation, currentLineText, currentLineIndentation, tmp);
        if (tmp.looksLikeAlignment) {
            // skip this line entirely
            continue;
        }
        var currentSpacesDiff = tmp.spacesDiff;
        if (currentSpacesDiff <= MAX_ALLOWED_TAB_SIZE_GUESS) {
            spacesDiffCount[currentSpacesDiff]++;
        }
        previousLineText = currentLineText;
        previousLineIndentation = currentLineIndentation;
    }
    var insertSpaces = defaultInsertSpaces;
    if (linesIndentedWithTabsCount !== linesIndentedWithSpacesCount) {
        insertSpaces = (linesIndentedWithTabsCount < linesIndentedWithSpacesCount);
    }
    var tabSize = defaultTabSize;
    var tabSizeScore = (insertSpaces ? 0 : 0.1 * linesCount);
    // console.log("score threshold: " + tabSizeScore);
    ALLOWED_TAB_SIZE_GUESSES.forEach(function (possibleTabSize) {
        var possibleTabSizeScore = spacesDiffCount[possibleTabSize];
        if (possibleTabSizeScore > tabSizeScore) {
            tabSizeScore = possibleTabSizeScore;
            tabSize = possibleTabSize;
        }
    });
    // console.log('--------------------------');
    // console.log('linesIndentedWithTabsCount: ' + linesIndentedWithTabsCount + ', linesIndentedWithSpacesCount: ' + linesIndentedWithSpacesCount);
    // console.log('spacesDiffCount: ' + spacesDiffCount);
    // console.log('tabSize: ' + tabSize + ', tabSizeScore: ' + tabSizeScore);
    return {
        insertSpaces: insertSpaces,
        tabSize: tabSize
    };
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getNodeColor(node) {
    return ((node.metadata & 1 /* ColorMask */) >>> 0 /* ColorOffset */);
}
function setNodeColor(node, color) {
    node.metadata = ((node.metadata & 254 /* ColorMaskInverse */) | (color << 0 /* ColorOffset */));
}
function getNodeIsVisited(node) {
    return ((node.metadata & 2 /* IsVisitedMask */) >>> 1 /* IsVisitedOffset */) === 1;
}
function setNodeIsVisited(node, value) {
    node.metadata = ((node.metadata & 253 /* IsVisitedMaskInverse */) | ((value ? 1 : 0) << 1 /* IsVisitedOffset */));
}
function getNodeIsForValidation(node) {
    return ((node.metadata & 4 /* IsForValidationMask */) >>> 2 /* IsForValidationOffset */) === 1;
}
function setNodeIsForValidation(node, value) {
    node.metadata = ((node.metadata & 251 /* IsForValidationMaskInverse */) | ((value ? 1 : 0) << 2 /* IsForValidationOffset */));
}
function getNodeIsInOverviewRuler(node) {
    return ((node.metadata & 8 /* IsInOverviewRulerMask */) >>> 3 /* IsInOverviewRulerOffset */) === 1;
}
function setNodeIsInOverviewRuler(node, value) {
    node.metadata = ((node.metadata & 247 /* IsInOverviewRulerMaskInverse */) | ((value ? 1 : 0) << 3 /* IsInOverviewRulerOffset */));
}
function getNodeStickiness(node) {
    return ((node.metadata & 48 /* StickinessMask */) >>> 4 /* StickinessOffset */);
}
function _setNodeStickiness(node, stickiness) {
    node.metadata = ((node.metadata & 207 /* StickinessMaskInverse */) | (stickiness << 4 /* StickinessOffset */));
}
function getCollapseOnReplaceEdit(node) {
    return ((node.metadata & 64 /* CollapseOnReplaceEditMask */) >>> 6 /* CollapseOnReplaceEditOffset */) === 1;
}
function setCollapseOnReplaceEdit(node, value) {
    node.metadata = ((node.metadata & 191 /* CollapseOnReplaceEditMaskInverse */) | ((value ? 1 : 0) << 6 /* CollapseOnReplaceEditOffset */));
}
var IntervalNode = /** @class */ (function () {
    function IntervalNode(id, start, end) {
        this.metadata = 0;
        this.parent = this;
        this.left = this;
        this.right = this;
        setNodeColor(this, 1 /* Red */);
        this.start = start;
        this.end = end;
        // FORCE_OVERFLOWING_TEST: this.delta = start;
        this.delta = 0;
        this.maxEnd = end;
        this.id = id;
        this.ownerId = 0;
        this.options = null;
        setNodeIsForValidation(this, false);
        _setNodeStickiness(this, 1 /* NeverGrowsWhenTypingAtEdges */);
        setNodeIsInOverviewRuler(this, false);
        setCollapseOnReplaceEdit(this, false);
        this.cachedVersionId = 0;
        this.cachedAbsoluteStart = start;
        this.cachedAbsoluteEnd = end;
        this.range = null;
        setNodeIsVisited(this, false);
    }
    IntervalNode.prototype.reset = function (versionId, start, end, range) {
        this.start = start;
        this.end = end;
        this.maxEnd = end;
        this.cachedVersionId = versionId;
        this.cachedAbsoluteStart = start;
        this.cachedAbsoluteEnd = end;
        this.range = range;
    };
    IntervalNode.prototype.setOptions = function (options) {
        this.options = options;
        var className = this.options.className;
        setNodeIsForValidation(this, (className === "squiggly-error" /* EditorErrorDecoration */
            || className === "squiggly-warning" /* EditorWarningDecoration */
            || className === "squiggly-info" /* EditorInfoDecoration */));
        _setNodeStickiness(this, this.options.stickiness);
        setNodeIsInOverviewRuler(this, (this.options.overviewRuler && this.options.overviewRuler.color) ? true : false);
        setCollapseOnReplaceEdit(this, this.options.collapseOnReplaceEdit);
    };
    IntervalNode.prototype.setCachedOffsets = function (absoluteStart, absoluteEnd, cachedVersionId) {
        if (this.cachedVersionId !== cachedVersionId) {
            this.range = null;
        }
        this.cachedVersionId = cachedVersionId;
        this.cachedAbsoluteStart = absoluteStart;
        this.cachedAbsoluteEnd = absoluteEnd;
    };
    IntervalNode.prototype.detach = function () {
        this.parent = null;
        this.left = null;
        this.right = null;
    };
    return IntervalNode;
}());
var SENTINEL = new IntervalNode(null, 0, 0);
SENTINEL.parent = SENTINEL;
SENTINEL.left = SENTINEL;
SENTINEL.right = SENTINEL;
setNodeColor(SENTINEL, 0 /* Black */);
var IntervalTree = /** @class */ (function () {
    function IntervalTree() {
        this.root = SENTINEL;
        this.requestNormalizeDelta = false;
    }
    IntervalTree.prototype.intervalSearch = function (start, end, filterOwnerId, filterOutValidation, cachedVersionId) {
        if (this.root === SENTINEL) {
            return [];
        }
        return intervalSearch(this, start, end, filterOwnerId, filterOutValidation, cachedVersionId);
    };
    IntervalTree.prototype.search = function (filterOwnerId, filterOutValidation, cachedVersionId) {
        if (this.root === SENTINEL) {
            return [];
        }
        return search(this, filterOwnerId, filterOutValidation, cachedVersionId);
    };
    /**
     * Will not set `cachedAbsoluteStart` nor `cachedAbsoluteEnd` on the returned nodes!
     */
    IntervalTree.prototype.collectNodesFromOwner = function (ownerId) {
        return collectNodesFromOwner(this, ownerId);
    };
    /**
     * Will not set `cachedAbsoluteStart` nor `cachedAbsoluteEnd` on the returned nodes!
     */
    IntervalTree.prototype.collectNodesPostOrder = function () {
        return collectNodesPostOrder(this);
    };
    IntervalTree.prototype.insert = function (node) {
        rbTreeInsert(this, node);
        this._normalizeDeltaIfNecessary();
    };
    IntervalTree.prototype.delete = function (node) {
        rbTreeDelete(this, node);
        this._normalizeDeltaIfNecessary();
    };
    IntervalTree.prototype.resolveNode = function (node, cachedVersionId) {
        var initialNode = node;
        var delta = 0;
        while (node !== this.root) {
            if (node === node.parent.right) {
                delta += node.parent.delta;
            }
            node = node.parent;
        }
        var nodeStart = initialNode.start + delta;
        var nodeEnd = initialNode.end + delta;
        initialNode.setCachedOffsets(nodeStart, nodeEnd, cachedVersionId);
    };
    IntervalTree.prototype.acceptReplace = function (offset, length, textLength, forceMoveMarkers) {
        // Our strategy is to remove all directly impacted nodes, and then add them back to the tree.
        // (1) collect all nodes that are intersecting this edit as nodes of interest
        var nodesOfInterest = searchForEditing(this, offset, offset + length);
        // (2) remove all nodes that are intersecting this edit
        for (var i = 0, len = nodesOfInterest.length; i < len; i++) {
            var node = nodesOfInterest[i];
            rbTreeDelete(this, node);
        }
        this._normalizeDeltaIfNecessary();
        // (3) edit all tree nodes except the nodes of interest
        noOverlapReplace(this, offset, offset + length, textLength);
        this._normalizeDeltaIfNecessary();
        // (4) edit the nodes of interest and insert them back in the tree
        for (var i = 0, len = nodesOfInterest.length; i < len; i++) {
            var node = nodesOfInterest[i];
            node.start = node.cachedAbsoluteStart;
            node.end = node.cachedAbsoluteEnd;
            nodeAcceptEdit(node, offset, (offset + length), textLength, forceMoveMarkers);
            node.maxEnd = node.end;
            rbTreeInsert(this, node);
        }
        this._normalizeDeltaIfNecessary();
    };
    IntervalTree.prototype._normalizeDeltaIfNecessary = function () {
        if (!this.requestNormalizeDelta) {
            return;
        }
        this.requestNormalizeDelta = false;
        normalizeDelta(this);
    };
    return IntervalTree;
}());
//#region Delta Normalization
function normalizeDelta(T) {
    var node = T.root;
    var delta = 0;
    while (node !== SENTINEL) {
        if (node.left !== SENTINEL && !getNodeIsVisited(node.left)) {
            // go left
            node = node.left;
            continue;
        }
        if (node.right !== SENTINEL && !getNodeIsVisited(node.right)) {
            // go right
            delta += node.delta;
            node = node.right;
            continue;
        }
        // handle current node
        node.start = delta + node.start;
        node.end = delta + node.end;
        node.delta = 0;
        recomputeMaxEnd(node);
        setNodeIsVisited(node, true);
        // going up from this node
        setNodeIsVisited(node.left, false);
        setNodeIsVisited(node.right, false);
        if (node === node.parent.right) {
            delta -= node.parent.delta;
        }
        node = node.parent;
    }
    setNodeIsVisited(T.root, false);
}
function adjustMarkerBeforeColumn(markerOffset, markerStickToPreviousCharacter, checkOffset, moveSemantics) {
    if (markerOffset < checkOffset) {
        return true;
    }
    if (markerOffset > checkOffset) {
        return false;
    }
    if (moveSemantics === 1 /* ForceMove */) {
        return false;
    }
    if (moveSemantics === 2 /* ForceStay */) {
        return true;
    }
    return markerStickToPreviousCharacter;
}
/**
 * This is a lot more complicated than strictly necessary to maintain the same behaviour
 * as when decorations were implemented using two markers.
 */
function nodeAcceptEdit(node, start, end, textLength, forceMoveMarkers) {
    var nodeStickiness = getNodeStickiness(node);
    var startStickToPreviousCharacter = (nodeStickiness === 0 /* AlwaysGrowsWhenTypingAtEdges */
        || nodeStickiness === 2 /* GrowsOnlyWhenTypingBefore */);
    var endStickToPreviousCharacter = (nodeStickiness === 1 /* NeverGrowsWhenTypingAtEdges */
        || nodeStickiness === 2 /* GrowsOnlyWhenTypingBefore */);
    var deletingCnt = (end - start);
    var insertingCnt = textLength;
    var commonLength = Math.min(deletingCnt, insertingCnt);
    var nodeStart = node.start;
    var startDone = false;
    var nodeEnd = node.end;
    var endDone = false;
    if (start <= nodeStart && nodeEnd <= end && getCollapseOnReplaceEdit(node)) {
        // This edit encompasses the entire decoration range
        // and the decoration has asked to become collapsed
        node.start = start;
        startDone = true;
        node.end = start;
        endDone = true;
    }
    {
        var moveSemantics = forceMoveMarkers ? 1 /* ForceMove */ : (deletingCnt > 0 ? 2 /* ForceStay */ : 0 /* MarkerDefined */);
        if (!startDone && adjustMarkerBeforeColumn(nodeStart, startStickToPreviousCharacter, start, moveSemantics)) {
            startDone = true;
        }
        if (!endDone && adjustMarkerBeforeColumn(nodeEnd, endStickToPreviousCharacter, start, moveSemantics)) {
            endDone = true;
        }
    }
    if (commonLength > 0 && !forceMoveMarkers) {
        var moveSemantics = (deletingCnt > insertingCnt ? 2 /* ForceStay */ : 0 /* MarkerDefined */);
        if (!startDone && adjustMarkerBeforeColumn(nodeStart, startStickToPreviousCharacter, start + commonLength, moveSemantics)) {
            startDone = true;
        }
        if (!endDone && adjustMarkerBeforeColumn(nodeEnd, endStickToPreviousCharacter, start + commonLength, moveSemantics)) {
            endDone = true;
        }
    }
    {
        var moveSemantics = forceMoveMarkers ? 1 /* ForceMove */ : 0 /* MarkerDefined */;
        if (!startDone && adjustMarkerBeforeColumn(nodeStart, startStickToPreviousCharacter, end, moveSemantics)) {
            node.start = start + insertingCnt;
            startDone = true;
        }
        if (!endDone && adjustMarkerBeforeColumn(nodeEnd, endStickToPreviousCharacter, end, moveSemantics)) {
            node.end = start + insertingCnt;
            endDone = true;
        }
    }
    // Finish
    var deltaColumn = (insertingCnt - deletingCnt);
    if (!startDone) {
        node.start = Math.max(0, nodeStart + deltaColumn);
        startDone = true;
    }
    if (!endDone) {
        node.end = Math.max(0, nodeEnd + deltaColumn);
        endDone = true;
    }
    if (node.start > node.end) {
        node.end = node.start;
    }
}
function searchForEditing(T, start, end) {
    // https://en.wikipedia.org/wiki/Interval_tree#Augmented_tree
    // Now, it is known that two intervals A and B overlap only when both
    // A.low <= B.high and A.high >= B.low. When searching the trees for
    // nodes overlapping with a given interval, you can immediately skip:
    //  a) all nodes to the right of nodes whose low value is past the end of the given interval.
    //  b) all nodes that have their maximum 'high' value below the start of the given interval.
    var node = T.root;
    var delta = 0;
    var nodeMaxEnd = 0;
    var nodeStart = 0;
    var nodeEnd = 0;
    var result = [];
    var resultLen = 0;
    while (node !== SENTINEL) {
        if (getNodeIsVisited(node)) {
            // going up from this node
            setNodeIsVisited(node.left, false);
            setNodeIsVisited(node.right, false);
            if (node === node.parent.right) {
                delta -= node.parent.delta;
            }
            node = node.parent;
            continue;
        }
        if (!getNodeIsVisited(node.left)) {
            // first time seeing this node
            nodeMaxEnd = delta + node.maxEnd;
            if (nodeMaxEnd < start) {
                // cover case b) from above
                // there is no need to search this node or its children
                setNodeIsVisited(node, true);
                continue;
            }
            if (node.left !== SENTINEL) {
                // go left
                node = node.left;
                continue;
            }
        }
        // handle current node
        nodeStart = delta + node.start;
        if (nodeStart > end) {
            // cover case a) from above
            // there is no need to search this node or its right subtree
            setNodeIsVisited(node, true);
            continue;
        }
        nodeEnd = delta + node.end;
        if (nodeEnd >= start) {
            node.setCachedOffsets(nodeStart, nodeEnd, 0);
            result[resultLen++] = node;
        }
        setNodeIsVisited(node, true);
        if (node.right !== SENTINEL && !getNodeIsVisited(node.right)) {
            // go right
            delta += node.delta;
            node = node.right;
            continue;
        }
    }
    setNodeIsVisited(T.root, false);
    return result;
}
function noOverlapReplace(T, start, end, textLength) {
    // https://en.wikipedia.org/wiki/Interval_tree#Augmented_tree
    // Now, it is known that two intervals A and B overlap only when both
    // A.low <= B.high and A.high >= B.low. When searching the trees for
    // nodes overlapping with a given interval, you can immediately skip:
    //  a) all nodes to the right of nodes whose low value is past the end of the given interval.
    //  b) all nodes that have their maximum 'high' value below the start of the given interval.
    var node = T.root;
    var delta = 0;
    var nodeMaxEnd = 0;
    var nodeStart = 0;
    var editDelta = (textLength - (end - start));
    while (node !== SENTINEL) {
        if (getNodeIsVisited(node)) {
            // going up from this node
            setNodeIsVisited(node.left, false);
            setNodeIsVisited(node.right, false);
            if (node === node.parent.right) {
                delta -= node.parent.delta;
            }
            recomputeMaxEnd(node);
            node = node.parent;
            continue;
        }
        if (!getNodeIsVisited(node.left)) {
            // first time seeing this node
            nodeMaxEnd = delta + node.maxEnd;
            if (nodeMaxEnd < start) {
                // cover case b) from above
                // there is no need to search this node or its children
                setNodeIsVisited(node, true);
                continue;
            }
            if (node.left !== SENTINEL) {
                // go left
                node = node.left;
                continue;
            }
        }
        // handle current node
        nodeStart = delta + node.start;
        if (nodeStart > end) {
            node.start += editDelta;
            node.end += editDelta;
            node.delta += editDelta;
            if (node.delta < -1073741824 /* MIN_SAFE_DELTA */ || node.delta > 1073741824 /* MAX_SAFE_DELTA */) {
                T.requestNormalizeDelta = true;
            }
            // cover case a) from above
            // there is no need to search this node or its right subtree
            setNodeIsVisited(node, true);
            continue;
        }
        setNodeIsVisited(node, true);
        if (node.right !== SENTINEL && !getNodeIsVisited(node.right)) {
            // go right
            delta += node.delta;
            node = node.right;
            continue;
        }
    }
    setNodeIsVisited(T.root, false);
}
//#endregion
//#region Searching
function collectNodesFromOwner(T, ownerId) {
    var node = T.root;
    var result = [];
    var resultLen = 0;
    while (node !== SENTINEL) {
        if (getNodeIsVisited(node)) {
            // going up from this node
            setNodeIsVisited(node.left, false);
            setNodeIsVisited(node.right, false);
            node = node.parent;
            continue;
        }
        if (node.left !== SENTINEL && !getNodeIsVisited(node.left)) {
            // go left
            node = node.left;
            continue;
        }
        // handle current node
        if (node.ownerId === ownerId) {
            result[resultLen++] = node;
        }
        setNodeIsVisited(node, true);
        if (node.right !== SENTINEL && !getNodeIsVisited(node.right)) {
            // go right
            node = node.right;
            continue;
        }
    }
    setNodeIsVisited(T.root, false);
    return result;
}
function collectNodesPostOrder(T) {
    var node = T.root;
    var result = [];
    var resultLen = 0;
    while (node !== SENTINEL) {
        if (getNodeIsVisited(node)) {
            // going up from this node
            setNodeIsVisited(node.left, false);
            setNodeIsVisited(node.right, false);
            node = node.parent;
            continue;
        }
        if (node.left !== SENTINEL && !getNodeIsVisited(node.left)) {
            // go left
            node = node.left;
            continue;
        }
        if (node.right !== SENTINEL && !getNodeIsVisited(node.right)) {
            // go right
            node = node.right;
            continue;
        }
        // handle current node
        result[resultLen++] = node;
        setNodeIsVisited(node, true);
    }
    setNodeIsVisited(T.root, false);
    return result;
}
function search(T, filterOwnerId, filterOutValidation, cachedVersionId) {
    var node = T.root;
    var delta = 0;
    var nodeStart = 0;
    var nodeEnd = 0;
    var result = [];
    var resultLen = 0;
    while (node !== SENTINEL) {
        if (getNodeIsVisited(node)) {
            // going up from this node
            setNodeIsVisited(node.left, false);
            setNodeIsVisited(node.right, false);
            if (node === node.parent.right) {
                delta -= node.parent.delta;
            }
            node = node.parent;
            continue;
        }
        if (node.left !== SENTINEL && !getNodeIsVisited(node.left)) {
            // go left
            node = node.left;
            continue;
        }
        // handle current node
        nodeStart = delta + node.start;
        nodeEnd = delta + node.end;
        node.setCachedOffsets(nodeStart, nodeEnd, cachedVersionId);
        var include = true;
        if (filterOwnerId && node.ownerId && node.ownerId !== filterOwnerId) {
            include = false;
        }
        if (filterOutValidation && getNodeIsForValidation(node)) {
            include = false;
        }
        if (include) {
            result[resultLen++] = node;
        }
        setNodeIsVisited(node, true);
        if (node.right !== SENTINEL && !getNodeIsVisited(node.right)) {
            // go right
            delta += node.delta;
            node = node.right;
            continue;
        }
    }
    setNodeIsVisited(T.root, false);
    return result;
}
function intervalSearch(T, intervalStart, intervalEnd, filterOwnerId, filterOutValidation, cachedVersionId) {
    // https://en.wikipedia.org/wiki/Interval_tree#Augmented_tree
    // Now, it is known that two intervals A and B overlap only when both
    // A.low <= B.high and A.high >= B.low. When searching the trees for
    // nodes overlapping with a given interval, you can immediately skip:
    //  a) all nodes to the right of nodes whose low value is past the end of the given interval.
    //  b) all nodes that have their maximum 'high' value below the start of the given interval.
    var node = T.root;
    var delta = 0;
    var nodeMaxEnd = 0;
    var nodeStart = 0;
    var nodeEnd = 0;
    var result = [];
    var resultLen = 0;
    while (node !== SENTINEL) {
        if (getNodeIsVisited(node)) {
            // going up from this node
            setNodeIsVisited(node.left, false);
            setNodeIsVisited(node.right, false);
            if (node === node.parent.right) {
                delta -= node.parent.delta;
            }
            node = node.parent;
            continue;
        }
        if (!getNodeIsVisited(node.left)) {
            // first time seeing this node
            nodeMaxEnd = delta + node.maxEnd;
            if (nodeMaxEnd < intervalStart) {
                // cover case b) from above
                // there is no need to search this node or its children
                setNodeIsVisited(node, true);
                continue;
            }
            if (node.left !== SENTINEL) {
                // go left
                node = node.left;
                continue;
            }
        }
        // handle current node
        nodeStart = delta + node.start;
        if (nodeStart > intervalEnd) {
            // cover case a) from above
            // there is no need to search this node or its right subtree
            setNodeIsVisited(node, true);
            continue;
        }
        nodeEnd = delta + node.end;
        if (nodeEnd >= intervalStart) {
            // There is overlap
            node.setCachedOffsets(nodeStart, nodeEnd, cachedVersionId);
            var include = true;
            if (filterOwnerId && node.ownerId && node.ownerId !== filterOwnerId) {
                include = false;
            }
            if (filterOutValidation && getNodeIsForValidation(node)) {
                include = false;
            }
            if (include) {
                result[resultLen++] = node;
            }
        }
        setNodeIsVisited(node, true);
        if (node.right !== SENTINEL && !getNodeIsVisited(node.right)) {
            // go right
            delta += node.delta;
            node = node.right;
            continue;
        }
    }
    setNodeIsVisited(T.root, false);
    return result;
}
//#endregion
//#region Insertion
function rbTreeInsert(T, newNode) {
    if (T.root === SENTINEL) {
        newNode.parent = SENTINEL;
        newNode.left = SENTINEL;
        newNode.right = SENTINEL;
        setNodeColor(newNode, 0 /* Black */);
        T.root = newNode;
        return T.root;
    }
    treeInsert(T, newNode);
    recomputeMaxEndWalkToRoot(newNode.parent);
    // repair tree
    var x = newNode;
    while (x !== T.root && getNodeColor(x.parent) === 1 /* Red */) {
        if (x.parent === x.parent.parent.left) {
            var y = x.parent.parent.right;
            if (getNodeColor(y) === 1 /* Red */) {
                setNodeColor(x.parent, 0 /* Black */);
                setNodeColor(y, 0 /* Black */);
                setNodeColor(x.parent.parent, 1 /* Red */);
                x = x.parent.parent;
            }
            else {
                if (x === x.parent.right) {
                    x = x.parent;
                    leftRotate(T, x);
                }
                setNodeColor(x.parent, 0 /* Black */);
                setNodeColor(x.parent.parent, 1 /* Red */);
                rightRotate(T, x.parent.parent);
            }
        }
        else {
            var y = x.parent.parent.left;
            if (getNodeColor(y) === 1 /* Red */) {
                setNodeColor(x.parent, 0 /* Black */);
                setNodeColor(y, 0 /* Black */);
                setNodeColor(x.parent.parent, 1 /* Red */);
                x = x.parent.parent;
            }
            else {
                if (x === x.parent.left) {
                    x = x.parent;
                    rightRotate(T, x);
                }
                setNodeColor(x.parent, 0 /* Black */);
                setNodeColor(x.parent.parent, 1 /* Red */);
                leftRotate(T, x.parent.parent);
            }
        }
    }
    setNodeColor(T.root, 0 /* Black */);
    return newNode;
}
function treeInsert(T, z) {
    var delta = 0;
    var x = T.root;
    var zAbsoluteStart = z.start;
    var zAbsoluteEnd = z.end;
    while (true) {
        var cmp = intervalCompare(zAbsoluteStart, zAbsoluteEnd, x.start + delta, x.end + delta);
        if (cmp < 0) {
            // this node should be inserted to the left
            // => it is not affected by the node's delta
            if (x.left === SENTINEL) {
                z.start -= delta;
                z.end -= delta;
                z.maxEnd -= delta;
                x.left = z;
                break;
            }
            else {
                x = x.left;
            }
        }
        else {
            // this node should be inserted to the right
            // => it is not affected by the node's delta
            if (x.right === SENTINEL) {
                z.start -= (delta + x.delta);
                z.end -= (delta + x.delta);
                z.maxEnd -= (delta + x.delta);
                x.right = z;
                break;
            }
            else {
                delta += x.delta;
                x = x.right;
            }
        }
    }
    z.parent = x;
    z.left = SENTINEL;
    z.right = SENTINEL;
    setNodeColor(z, 1 /* Red */);
}
//#endregion
//#region Deletion
function rbTreeDelete(T, z) {
    var x;
    var y;
    // RB-DELETE except we don't swap z and y in case c)
    // i.e. we always delete what's pointed at by z.
    if (z.left === SENTINEL) {
        x = z.right;
        y = z;
        // x's delta is no longer influenced by z's delta
        x.delta += z.delta;
        if (x.delta < -1073741824 /* MIN_SAFE_DELTA */ || x.delta > 1073741824 /* MAX_SAFE_DELTA */) {
            T.requestNormalizeDelta = true;
        }
        x.start += z.delta;
        x.end += z.delta;
    }
    else if (z.right === SENTINEL) {
        x = z.left;
        y = z;
    }
    else {
        y = leftest(z.right);
        x = y.right;
        // y's delta is no longer influenced by z's delta,
        // but we don't want to walk the entire right-hand-side subtree of x.
        // we therefore maintain z's delta in y, and adjust only x
        x.start += y.delta;
        x.end += y.delta;
        x.delta += y.delta;
        if (x.delta < -1073741824 /* MIN_SAFE_DELTA */ || x.delta > 1073741824 /* MAX_SAFE_DELTA */) {
            T.requestNormalizeDelta = true;
        }
        y.start += z.delta;
        y.end += z.delta;
        y.delta = z.delta;
        if (y.delta < -1073741824 /* MIN_SAFE_DELTA */ || y.delta > 1073741824 /* MAX_SAFE_DELTA */) {
            T.requestNormalizeDelta = true;
        }
    }
    if (y === T.root) {
        T.root = x;
        setNodeColor(x, 0 /* Black */);
        z.detach();
        resetSentinel();
        recomputeMaxEnd(x);
        T.root.parent = SENTINEL;
        return;
    }
    var yWasRed = (getNodeColor(y) === 1 /* Red */);
    if (y === y.parent.left) {
        y.parent.left = x;
    }
    else {
        y.parent.right = x;
    }
    if (y === z) {
        x.parent = y.parent;
    }
    else {
        if (y.parent === z) {
            x.parent = y;
        }
        else {
            x.parent = y.parent;
        }
        y.left = z.left;
        y.right = z.right;
        y.parent = z.parent;
        setNodeColor(y, getNodeColor(z));
        if (z === T.root) {
            T.root = y;
        }
        else {
            if (z === z.parent.left) {
                z.parent.left = y;
            }
            else {
                z.parent.right = y;
            }
        }
        if (y.left !== SENTINEL) {
            y.left.parent = y;
        }
        if (y.right !== SENTINEL) {
            y.right.parent = y;
        }
    }
    z.detach();
    if (yWasRed) {
        recomputeMaxEndWalkToRoot(x.parent);
        if (y !== z) {
            recomputeMaxEndWalkToRoot(y);
            recomputeMaxEndWalkToRoot(y.parent);
        }
        resetSentinel();
        return;
    }
    recomputeMaxEndWalkToRoot(x);
    recomputeMaxEndWalkToRoot(x.parent);
    if (y !== z) {
        recomputeMaxEndWalkToRoot(y);
        recomputeMaxEndWalkToRoot(y.parent);
    }
    // RB-DELETE-FIXUP
    var w;
    while (x !== T.root && getNodeColor(x) === 0 /* Black */) {
        if (x === x.parent.left) {
            w = x.parent.right;
            if (getNodeColor(w) === 1 /* Red */) {
                setNodeColor(w, 0 /* Black */);
                setNodeColor(x.parent, 1 /* Red */);
                leftRotate(T, x.parent);
                w = x.parent.right;
            }
            if (getNodeColor(w.left) === 0 /* Black */ && getNodeColor(w.right) === 0 /* Black */) {
                setNodeColor(w, 1 /* Red */);
                x = x.parent;
            }
            else {
                if (getNodeColor(w.right) === 0 /* Black */) {
                    setNodeColor(w.left, 0 /* Black */);
                    setNodeColor(w, 1 /* Red */);
                    rightRotate(T, w);
                    w = x.parent.right;
                }
                setNodeColor(w, getNodeColor(x.parent));
                setNodeColor(x.parent, 0 /* Black */);
                setNodeColor(w.right, 0 /* Black */);
                leftRotate(T, x.parent);
                x = T.root;
            }
        }
        else {
            w = x.parent.left;
            if (getNodeColor(w) === 1 /* Red */) {
                setNodeColor(w, 0 /* Black */);
                setNodeColor(x.parent, 1 /* Red */);
                rightRotate(T, x.parent);
                w = x.parent.left;
            }
            if (getNodeColor(w.left) === 0 /* Black */ && getNodeColor(w.right) === 0 /* Black */) {
                setNodeColor(w, 1 /* Red */);
                x = x.parent;
            }
            else {
                if (getNodeColor(w.left) === 0 /* Black */) {
                    setNodeColor(w.right, 0 /* Black */);
                    setNodeColor(w, 1 /* Red */);
                    leftRotate(T, w);
                    w = x.parent.left;
                }
                setNodeColor(w, getNodeColor(x.parent));
                setNodeColor(x.parent, 0 /* Black */);
                setNodeColor(w.left, 0 /* Black */);
                rightRotate(T, x.parent);
                x = T.root;
            }
        }
    }
    setNodeColor(x, 0 /* Black */);
    resetSentinel();
}
function leftest(node) {
    while (node.left !== SENTINEL) {
        node = node.left;
    }
    return node;
}
function resetSentinel() {
    SENTINEL.parent = SENTINEL;
    SENTINEL.delta = 0; // optional
    SENTINEL.start = 0; // optional
    SENTINEL.end = 0; // optional
}
//#endregion
//#region Rotations
function leftRotate(T, x) {
    var y = x.right; // set y.
    y.delta += x.delta; // y's delta is no longer influenced by x's delta
    if (y.delta < -1073741824 /* MIN_SAFE_DELTA */ || y.delta > 1073741824 /* MAX_SAFE_DELTA */) {
        T.requestNormalizeDelta = true;
    }
    y.start += x.delta;
    y.end += x.delta;
    x.right = y.left; // turn y's left subtree into x's right subtree.
    if (y.left !== SENTINEL) {
        y.left.parent = x;
    }
    y.parent = x.parent; // link x's parent to y.
    if (x.parent === SENTINEL) {
        T.root = y;
    }
    else if (x === x.parent.left) {
        x.parent.left = y;
    }
    else {
        x.parent.right = y;
    }
    y.left = x; // put x on y's left.
    x.parent = y;
    recomputeMaxEnd(x);
    recomputeMaxEnd(y);
}
function rightRotate(T, y) {
    var x = y.left;
    y.delta -= x.delta;
    if (y.delta < -1073741824 /* MIN_SAFE_DELTA */ || y.delta > 1073741824 /* MAX_SAFE_DELTA */) {
        T.requestNormalizeDelta = true;
    }
    y.start -= x.delta;
    y.end -= x.delta;
    y.left = x.right;
    if (x.right !== SENTINEL) {
        x.right.parent = y;
    }
    x.parent = y.parent;
    if (y.parent === SENTINEL) {
        T.root = x;
    }
    else if (y === y.parent.right) {
        y.parent.right = x;
    }
    else {
        y.parent.left = x;
    }
    x.right = y;
    y.parent = x;
    recomputeMaxEnd(y);
    recomputeMaxEnd(x);
}
//#endregion
//#region max end computation
function computeMaxEnd(node) {
    var maxEnd = node.end;
    if (node.left !== SENTINEL) {
        var leftMaxEnd = node.left.maxEnd;
        if (leftMaxEnd > maxEnd) {
            maxEnd = leftMaxEnd;
        }
    }
    if (node.right !== SENTINEL) {
        var rightMaxEnd = node.right.maxEnd + node.delta;
        if (rightMaxEnd > maxEnd) {
            maxEnd = rightMaxEnd;
        }
    }
    return maxEnd;
}
function recomputeMaxEnd(node) {
    node.maxEnd = computeMaxEnd(node);
}
function recomputeMaxEndWalkToRoot(node) {
    while (node !== SENTINEL) {
        var maxEnd = computeMaxEnd(node);
        if (node.maxEnd === maxEnd) {
            // no need to go further
            return;
        }
        node.maxEnd = maxEnd;
        node = node.parent;
    }
}
//#endregion
//#region utils
function intervalCompare(aStart, aEnd, bStart, bEnd) {
    if (aStart === bStart) {
        return aEnd - bEnd;
    }
    return aStart - bStart;
}
//#endregion

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var TreeNode = /** @class */ (function () {
    function TreeNode(piece, color) {
        this.piece = piece;
        this.color = color;
        this.size_left = 0;
        this.lf_left = 0;
        this.parent = this;
        this.left = this;
        this.right = this;
    }
    TreeNode.prototype.next = function () {
        if (this.right !== SENTINEL$1) {
            return leftest$1(this.right);
        }
        var node = this;
        while (node.parent !== SENTINEL$1) {
            if (node.parent.left === node) {
                break;
            }
            node = node.parent;
        }
        if (node.parent === SENTINEL$1) {
            return SENTINEL$1;
        }
        else {
            return node.parent;
        }
    };
    TreeNode.prototype.prev = function () {
        if (this.left !== SENTINEL$1) {
            return righttest(this.left);
        }
        var node = this;
        while (node.parent !== SENTINEL$1) {
            if (node.parent.right === node) {
                break;
            }
            node = node.parent;
        }
        if (node.parent === SENTINEL$1) {
            return SENTINEL$1;
        }
        else {
            return node.parent;
        }
    };
    TreeNode.prototype.detach = function () {
        this.parent = null;
        this.left = null;
        this.right = null;
    };
    return TreeNode;
}());
var SENTINEL$1 = new TreeNode(null, 0 /* Black */);
SENTINEL$1.parent = SENTINEL$1;
SENTINEL$1.left = SENTINEL$1;
SENTINEL$1.right = SENTINEL$1;
SENTINEL$1.color = 0 /* Black */;
function leftest$1(node) {
    while (node.left !== SENTINEL$1) {
        node = node.left;
    }
    return node;
}
function righttest(node) {
    while (node.right !== SENTINEL$1) {
        node = node.right;
    }
    return node;
}
function calculateSize(node) {
    if (node === SENTINEL$1) {
        return 0;
    }
    return node.size_left + node.piece.length + calculateSize(node.right);
}
function calculateLF(node) {
    if (node === SENTINEL$1) {
        return 0;
    }
    return node.lf_left + node.piece.lineFeedCnt + calculateLF(node.right);
}
function resetSentinel$1() {
    SENTINEL$1.parent = SENTINEL$1;
}
function leftRotate$1(tree, x) {
    var y = x.right;
    // fix size_left
    y.size_left += x.size_left + (x.piece ? x.piece.length : 0);
    y.lf_left += x.lf_left + (x.piece ? x.piece.lineFeedCnt : 0);
    x.right = y.left;
    if (y.left !== SENTINEL$1) {
        y.left.parent = x;
    }
    y.parent = x.parent;
    if (x.parent === SENTINEL$1) {
        tree.root = y;
    }
    else if (x.parent.left === x) {
        x.parent.left = y;
    }
    else {
        x.parent.right = y;
    }
    y.left = x;
    x.parent = y;
}
function rightRotate$1(tree, y) {
    var x = y.left;
    y.left = x.right;
    if (x.right !== SENTINEL$1) {
        x.right.parent = y;
    }
    x.parent = y.parent;
    // fix size_left
    y.size_left -= x.size_left + (x.piece ? x.piece.length : 0);
    y.lf_left -= x.lf_left + (x.piece ? x.piece.lineFeedCnt : 0);
    if (y.parent === SENTINEL$1) {
        tree.root = x;
    }
    else if (y === y.parent.right) {
        y.parent.right = x;
    }
    else {
        y.parent.left = x;
    }
    x.right = y;
    y.parent = x;
}
function rbDelete(tree, z) {
    var x;
    var y;
    if (z.left === SENTINEL$1) {
        y = z;
        x = y.right;
    }
    else if (z.right === SENTINEL$1) {
        y = z;
        x = y.left;
    }
    else {
        y = leftest$1(z.right);
        x = y.right;
    }
    if (y === tree.root) {
        tree.root = x;
        // if x is null, we are removing the only node
        x.color = 0 /* Black */;
        z.detach();
        resetSentinel$1();
        tree.root.parent = SENTINEL$1;
        return;
    }
    var yWasRed = (y.color === 1 /* Red */);
    if (y === y.parent.left) {
        y.parent.left = x;
    }
    else {
        y.parent.right = x;
    }
    if (y === z) {
        x.parent = y.parent;
        recomputeTreeMetadata(tree, x);
    }
    else {
        if (y.parent === z) {
            x.parent = y;
        }
        else {
            x.parent = y.parent;
        }
        // as we make changes to x's hierarchy, update size_left of subtree first
        recomputeTreeMetadata(tree, x);
        y.left = z.left;
        y.right = z.right;
        y.parent = z.parent;
        y.color = z.color;
        if (z === tree.root) {
            tree.root = y;
        }
        else {
            if (z === z.parent.left) {
                z.parent.left = y;
            }
            else {
                z.parent.right = y;
            }
        }
        if (y.left !== SENTINEL$1) {
            y.left.parent = y;
        }
        if (y.right !== SENTINEL$1) {
            y.right.parent = y;
        }
        // update metadata
        // we replace z with y, so in this sub tree, the length change is z.item.length
        y.size_left = z.size_left;
        y.lf_left = z.lf_left;
        recomputeTreeMetadata(tree, y);
    }
    z.detach();
    if (x.parent.left === x) {
        var newSizeLeft = calculateSize(x);
        var newLFLeft = calculateLF(x);
        if (newSizeLeft !== x.parent.size_left || newLFLeft !== x.parent.lf_left) {
            var delta = newSizeLeft - x.parent.size_left;
            var lf_delta = newLFLeft - x.parent.lf_left;
            x.parent.size_left = newSizeLeft;
            x.parent.lf_left = newLFLeft;
            updateTreeMetadata(tree, x.parent, delta, lf_delta);
        }
    }
    recomputeTreeMetadata(tree, x.parent);
    if (yWasRed) {
        resetSentinel$1();
        return;
    }
    // RB-DELETE-FIXUP
    var w;
    while (x !== tree.root && x.color === 0 /* Black */) {
        if (x === x.parent.left) {
            w = x.parent.right;
            if (w.color === 1 /* Red */) {
                w.color = 0 /* Black */;
                x.parent.color = 1 /* Red */;
                leftRotate$1(tree, x.parent);
                w = x.parent.right;
            }
            if (w.left.color === 0 /* Black */ && w.right.color === 0 /* Black */) {
                w.color = 1 /* Red */;
                x = x.parent;
            }
            else {
                if (w.right.color === 0 /* Black */) {
                    w.left.color = 0 /* Black */;
                    w.color = 1 /* Red */;
                    rightRotate$1(tree, w);
                    w = x.parent.right;
                }
                w.color = x.parent.color;
                x.parent.color = 0 /* Black */;
                w.right.color = 0 /* Black */;
                leftRotate$1(tree, x.parent);
                x = tree.root;
            }
        }
        else {
            w = x.parent.left;
            if (w.color === 1 /* Red */) {
                w.color = 0 /* Black */;
                x.parent.color = 1 /* Red */;
                rightRotate$1(tree, x.parent);
                w = x.parent.left;
            }
            if (w.left.color === 0 /* Black */ && w.right.color === 0 /* Black */) {
                w.color = 1 /* Red */;
                x = x.parent;
            }
            else {
                if (w.left.color === 0 /* Black */) {
                    w.right.color = 0 /* Black */;
                    w.color = 1 /* Red */;
                    leftRotate$1(tree, w);
                    w = x.parent.left;
                }
                w.color = x.parent.color;
                x.parent.color = 0 /* Black */;
                w.left.color = 0 /* Black */;
                rightRotate$1(tree, x.parent);
                x = tree.root;
            }
        }
    }
    x.color = 0 /* Black */;
    resetSentinel$1();
}
function fixInsert(tree, x) {
    recomputeTreeMetadata(tree, x);
    while (x !== tree.root && x.parent.color === 1 /* Red */) {
        if (x.parent === x.parent.parent.left) {
            var y = x.parent.parent.right;
            if (y.color === 1 /* Red */) {
                x.parent.color = 0 /* Black */;
                y.color = 0 /* Black */;
                x.parent.parent.color = 1 /* Red */;
                x = x.parent.parent;
            }
            else {
                if (x === x.parent.right) {
                    x = x.parent;
                    leftRotate$1(tree, x);
                }
                x.parent.color = 0 /* Black */;
                x.parent.parent.color = 1 /* Red */;
                rightRotate$1(tree, x.parent.parent);
            }
        }
        else {
            var y = x.parent.parent.left;
            if (y.color === 1 /* Red */) {
                x.parent.color = 0 /* Black */;
                y.color = 0 /* Black */;
                x.parent.parent.color = 1 /* Red */;
                x = x.parent.parent;
            }
            else {
                if (x === x.parent.left) {
                    x = x.parent;
                    rightRotate$1(tree, x);
                }
                x.parent.color = 0 /* Black */;
                x.parent.parent.color = 1 /* Red */;
                leftRotate$1(tree, x.parent.parent);
            }
        }
    }
    tree.root.color = 0 /* Black */;
}
function updateTreeMetadata(tree, x, delta, lineFeedCntDelta) {
    // node length change or line feed count change
    while (x !== tree.root && x !== SENTINEL$1) {
        if (x.parent.left === x) {
            x.parent.size_left += delta;
            x.parent.lf_left += lineFeedCntDelta;
        }
        x = x.parent;
    }
}
function recomputeTreeMetadata(tree, x) {
    var delta = 0;
    var lf_delta = 0;
    if (x === tree.root) {
        return;
    }
    if (delta === 0) {
        // go upwards till the node whose left subtree is changed.
        while (x !== tree.root && x === x.parent.right) {
            x = x.parent;
        }
        if (x === tree.root) {
            // well, it means we add a node to the end (inorder)
            return;
        }
        // x is the node whose right subtree is changed.
        x = x.parent;
        delta = calculateSize(x.left) - x.size_left;
        lf_delta = calculateLF(x.left) - x.lf_left;
        x.size_left += delta;
        x.lf_left += lf_delta;
    }
    // go upwards till root. O(logN)
    while (x !== tree.root && (delta !== 0 || lf_delta !== 0)) {
        if (x.parent.left === x) {
            x.parent.size_left += delta;
            x.parent.lf_left += lf_delta;
        }
        x = x.parent;
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Uint8Matrix = /** @class */ (function () {
    function Uint8Matrix(rows, cols, defaultValue) {
        var data = new Uint8Array(rows * cols);
        for (var i = 0, len = rows * cols; i < len; i++) {
            data[i] = defaultValue;
        }
        this._data = data;
        this.rows = rows;
        this.cols = cols;
    }
    Uint8Matrix.prototype.get = function (row, col) {
        return this._data[row * this.cols + col];
    };
    Uint8Matrix.prototype.set = function (row, col, value) {
        this._data[row * this.cols + col] = value;
    };
    return Uint8Matrix;
}());
function toUint8(v) {
    if (v < 0) {
        return 0;
    }
    if (v > 255 /* MAX_UINT_8 */) {
        return 255 /* MAX_UINT_8 */;
    }
    return v | 0;
}
function toUint32(v) {
    if (v < 0) {
        return 0;
    }
    if (v > 4294967295 /* MAX_UINT_32 */) {
        return 4294967295 /* MAX_UINT_32 */;
    }
    return v | 0;
}
function toUint32Array(arr) {
    var len = arr.length;
    var r = new Uint32Array(len);
    for (var i = 0; i < len; i++) {
        r[i] = toUint32(arr[i]);
    }
    return r;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * A fast character classifier that uses a compact array for ASCII values.
 */
var CharacterClassifier = /** @class */ (function () {
    function CharacterClassifier(_defaultValue) {
        var defaultValue = toUint8(_defaultValue);
        this._defaultValue = defaultValue;
        this._asciiMap = CharacterClassifier._createAsciiMap(defaultValue);
        this._map = new Map();
    }
    CharacterClassifier._createAsciiMap = function (defaultValue) {
        var asciiMap = new Uint8Array(256);
        for (var i = 0; i < 256; i++) {
            asciiMap[i] = defaultValue;
        }
        return asciiMap;
    };
    CharacterClassifier.prototype.set = function (charCode, _value) {
        var value = toUint8(_value);
        if (charCode >= 0 && charCode < 256) {
            this._asciiMap[charCode] = value;
        }
        else {
            this._map.set(charCode, value);
        }
    };
    CharacterClassifier.prototype.get = function (charCode) {
        if (charCode >= 0 && charCode < 256) {
            return this._asciiMap[charCode];
        }
        else {
            return (this._map.get(charCode) || this._defaultValue);
        }
    };
    return CharacterClassifier;
}());
var CharacterSet = /** @class */ (function () {
    function CharacterSet() {
        this._actual = new CharacterClassifier(0 /* False */);
    }
    CharacterSet.prototype.add = function (charCode) {
        this._actual.set(charCode, 1 /* True */);
    };
    CharacterSet.prototype.has = function (charCode) {
        return (this._actual.get(charCode) === 1 /* True */);
    };
    return CharacterSet;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$8 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var WordCharacterClassifier = /** @class */ (function (_super) {
    __extends$8(WordCharacterClassifier, _super);
    function WordCharacterClassifier(wordSeparators) {
        var _this = _super.call(this, 0 /* Regular */) || this;
        for (var i = 0, len = wordSeparators.length; i < len; i++) {
            _this.set(wordSeparators.charCodeAt(i), 2 /* WordSeparator */);
        }
        _this.set(32 /* Space */, 1 /* Whitespace */);
        _this.set(9 /* Tab */, 1 /* Whitespace */);
        return _this;
    }
    return WordCharacterClassifier;
}(CharacterClassifier));
function once$2(computeFn) {
    var cache = {}; // TODO@Alex unbounded cache
    return function (input) {
        if (!cache.hasOwnProperty(input)) {
            cache[input] = computeFn(input);
        }
        return cache[input];
    };
}
var getMapForWordSeparators = once$2(function (input) { return new WordCharacterClassifier(input); });

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var LIMIT_FIND_COUNT = 999;
var SearchParams = /** @class */ (function () {
    function SearchParams(searchString, isRegex, matchCase, wordSeparators) {
        this.searchString = searchString;
        this.isRegex = isRegex;
        this.matchCase = matchCase;
        this.wordSeparators = wordSeparators;
    }
    SearchParams.prototype.parseSearchRequest = function () {
        if (this.searchString === '') {
            return null;
        }
        // Try to create a RegExp out of the params
        var multiline;
        if (this.isRegex) {
            multiline = isMultilineRegexSource(this.searchString);
        }
        else {
            multiline = (this.searchString.indexOf('\n') >= 0);
        }
        var regex = null;
        try {
            regex = createRegExp(this.searchString, this.isRegex, {
                matchCase: this.matchCase,
                wholeWord: false,
                multiline: multiline,
                global: true
            });
        }
        catch (err) {
            return null;
        }
        if (!regex) {
            return null;
        }
        var canUseSimpleSearch = (!this.isRegex && !multiline);
        if (canUseSimpleSearch && this.searchString.toLowerCase() !== this.searchString.toUpperCase()) {
            // casing might make a difference
            canUseSimpleSearch = this.matchCase;
        }
        return new SearchData(regex, this.wordSeparators ? getMapForWordSeparators(this.wordSeparators) : null, canUseSimpleSearch ? this.searchString : null);
    };
    return SearchParams;
}());
function isMultilineRegexSource(searchString) {
    if (!searchString || searchString.length === 0) {
        return false;
    }
    for (var i = 0, len = searchString.length; i < len; i++) {
        var chCode = searchString.charCodeAt(i);
        if (chCode === 92 /* Backslash */) {
            // move to next char
            i++;
            if (i >= len) {
                // string ends with a \
                break;
            }
            var nextChCode = searchString.charCodeAt(i);
            if (nextChCode === 110 /* n */ || nextChCode === 114 /* r */ || nextChCode === 87 /* W */) {
                return true;
            }
        }
    }
    return false;
}
var SearchData = /** @class */ (function () {
    function SearchData(regex, wordSeparators, simpleSearch) {
        this.regex = regex;
        this.wordSeparators = wordSeparators;
        this.simpleSearch = simpleSearch;
    }
    return SearchData;
}());
function createFindMatch(range, rawMatches, captureMatches) {
    if (!captureMatches) {
        return new FindMatch(range, null);
    }
    var matches = [];
    for (var i = 0, len = rawMatches.length; i < len; i++) {
        matches[i] = rawMatches[i];
    }
    return new FindMatch(range, matches);
}
var LineFeedCounter = /** @class */ (function () {
    function LineFeedCounter(text) {
        var lineFeedsOffsets = [];
        var lineFeedsOffsetsLen = 0;
        for (var i = 0, textLen = text.length; i < textLen; i++) {
            if (text.charCodeAt(i) === 10 /* LineFeed */) {
                lineFeedsOffsets[lineFeedsOffsetsLen++] = i;
            }
        }
        this._lineFeedsOffsets = lineFeedsOffsets;
    }
    LineFeedCounter.prototype.findLineFeedCountBeforeOffset = function (offset) {
        var lineFeedsOffsets = this._lineFeedsOffsets;
        var min = 0;
        var max = lineFeedsOffsets.length - 1;
        if (max === -1) {
            // no line feeds
            return 0;
        }
        if (offset <= lineFeedsOffsets[0]) {
            // before first line feed
            return 0;
        }
        while (min < max) {
            var mid = min + ((max - min) / 2 >> 0);
            if (lineFeedsOffsets[mid] >= offset) {
                max = mid - 1;
            }
            else {
                if (lineFeedsOffsets[mid + 1] >= offset) {
                    // bingo!
                    min = mid;
                    max = mid;
                }
                else {
                    min = mid + 1;
                }
            }
        }
        return min + 1;
    };
    return LineFeedCounter;
}());
var TextModelSearch = /** @class */ (function () {
    function TextModelSearch() {
    }
    TextModelSearch.findMatches = function (model, searchParams, searchRange, captureMatches, limitResultCount) {
        var searchData = searchParams.parseSearchRequest();
        if (!searchData) {
            return [];
        }
        if (searchData.regex.multiline) {
            return this._doFindMatchesMultiline(model, searchRange, new Searcher(searchData.wordSeparators, searchData.regex), captureMatches, limitResultCount);
        }
        return this._doFindMatchesLineByLine(model, searchRange, searchData, captureMatches, limitResultCount);
    };
    /**
     * Multiline search always executes on the lines concatenated with \n.
     * We must therefore compensate for the count of \n in case the model is CRLF
     */
    TextModelSearch._getMultilineMatchRange = function (model, deltaOffset, text, lfCounter, matchIndex, match0) {
        var startOffset;
        var lineFeedCountBeforeMatch = 0;
        if (lfCounter) {
            lineFeedCountBeforeMatch = lfCounter.findLineFeedCountBeforeOffset(matchIndex);
            startOffset = deltaOffset + matchIndex + lineFeedCountBeforeMatch /* add as many \r as there were \n */;
        }
        else {
            startOffset = deltaOffset + matchIndex;
        }
        var endOffset;
        if (lfCounter) {
            var lineFeedCountBeforeEndOfMatch = lfCounter.findLineFeedCountBeforeOffset(matchIndex + match0.length);
            var lineFeedCountInMatch = lineFeedCountBeforeEndOfMatch - lineFeedCountBeforeMatch;
            endOffset = startOffset + match0.length + lineFeedCountInMatch /* add as many \r as there were \n */;
        }
        else {
            endOffset = startOffset + match0.length;
        }
        var startPosition = model.getPositionAt(startOffset);
        var endPosition = model.getPositionAt(endOffset);
        return new Range(startPosition.lineNumber, startPosition.column, endPosition.lineNumber, endPosition.column);
    };
    TextModelSearch._doFindMatchesMultiline = function (model, searchRange, searcher, captureMatches, limitResultCount) {
        var deltaOffset = model.getOffsetAt(searchRange.getStartPosition());
        // We always execute multiline search over the lines joined with \n
        // This makes it that \n will match the EOL for both CRLF and LF models
        // We compensate for offset errors in `_getMultilineMatchRange`
        var text = model.getValueInRange(searchRange, 1 /* LF */);
        var lfCounter = (model.getEOL() === '\r\n' ? new LineFeedCounter(text) : null);
        var result = [];
        var counter = 0;
        var m;
        searcher.reset(0);
        while ((m = searcher.next(text))) {
            result[counter++] = createFindMatch(this._getMultilineMatchRange(model, deltaOffset, text, lfCounter, m.index, m[0]), m, captureMatches);
            if (counter >= limitResultCount) {
                return result;
            }
        }
        return result;
    };
    TextModelSearch._doFindMatchesLineByLine = function (model, searchRange, searchData, captureMatches, limitResultCount) {
        var result = [];
        var resultLen = 0;
        // Early case for a search range that starts & stops on the same line number
        if (searchRange.startLineNumber === searchRange.endLineNumber) {
            var text_1 = model.getLineContent(searchRange.startLineNumber).substring(searchRange.startColumn - 1, searchRange.endColumn - 1);
            resultLen = this._findMatchesInLine(searchData, text_1, searchRange.startLineNumber, searchRange.startColumn - 1, resultLen, result, captureMatches, limitResultCount);
            return result;
        }
        // Collect results from first line
        var text = model.getLineContent(searchRange.startLineNumber).substring(searchRange.startColumn - 1);
        resultLen = this._findMatchesInLine(searchData, text, searchRange.startLineNumber, searchRange.startColumn - 1, resultLen, result, captureMatches, limitResultCount);
        // Collect results from middle lines
        for (var lineNumber = searchRange.startLineNumber + 1; lineNumber < searchRange.endLineNumber && resultLen < limitResultCount; lineNumber++) {
            resultLen = this._findMatchesInLine(searchData, model.getLineContent(lineNumber), lineNumber, 0, resultLen, result, captureMatches, limitResultCount);
        }
        // Collect results from last line
        if (resultLen < limitResultCount) {
            var text_2 = model.getLineContent(searchRange.endLineNumber).substring(0, searchRange.endColumn - 1);
            resultLen = this._findMatchesInLine(searchData, text_2, searchRange.endLineNumber, 0, resultLen, result, captureMatches, limitResultCount);
        }
        return result;
    };
    TextModelSearch._findMatchesInLine = function (searchData, text, lineNumber, deltaOffset, resultLen, result, captureMatches, limitResultCount) {
        var wordSeparators = searchData.wordSeparators;
        if (!captureMatches && searchData.simpleSearch) {
            var searchString = searchData.simpleSearch;
            var searchStringLen = searchString.length;
            var textLength = text.length;
            var lastMatchIndex = -searchStringLen;
            while ((lastMatchIndex = text.indexOf(searchString, lastMatchIndex + searchStringLen)) !== -1) {
                if (!wordSeparators || isValidMatch(wordSeparators, text, textLength, lastMatchIndex, searchStringLen)) {
                    result[resultLen++] = new FindMatch(new Range(lineNumber, lastMatchIndex + 1 + deltaOffset, lineNumber, lastMatchIndex + 1 + searchStringLen + deltaOffset), null);
                    if (resultLen >= limitResultCount) {
                        return resultLen;
                    }
                }
            }
            return resultLen;
        }
        var searcher = new Searcher(searchData.wordSeparators, searchData.regex);
        var m;
        // Reset regex to search from the beginning
        searcher.reset(0);
        do {
            m = searcher.next(text);
            if (m) {
                result[resultLen++] = createFindMatch(new Range(lineNumber, m.index + 1 + deltaOffset, lineNumber, m.index + 1 + m[0].length + deltaOffset), m, captureMatches);
                if (resultLen >= limitResultCount) {
                    return resultLen;
                }
            }
        } while (m);
        return resultLen;
    };
    TextModelSearch.findNextMatch = function (model, searchParams, searchStart, captureMatches) {
        var searchData = searchParams.parseSearchRequest();
        if (!searchData) {
            return null;
        }
        var searcher = new Searcher(searchData.wordSeparators, searchData.regex);
        if (searchData.regex.multiline) {
            return this._doFindNextMatchMultiline(model, searchStart, searcher, captureMatches);
        }
        return this._doFindNextMatchLineByLine(model, searchStart, searcher, captureMatches);
    };
    TextModelSearch._doFindNextMatchMultiline = function (model, searchStart, searcher, captureMatches) {
        var searchTextStart = new Position(searchStart.lineNumber, 1);
        var deltaOffset = model.getOffsetAt(searchTextStart);
        var lineCount = model.getLineCount();
        // We always execute multiline search over the lines joined with \n
        // This makes it that \n will match the EOL for both CRLF and LF models
        // We compensate for offset errors in `_getMultilineMatchRange`
        var text = model.getValueInRange(new Range(searchTextStart.lineNumber, searchTextStart.column, lineCount, model.getLineMaxColumn(lineCount)), 1 /* LF */);
        var lfCounter = (model.getEOL() === '\r\n' ? new LineFeedCounter(text) : null);
        searcher.reset(searchStart.column - 1);
        var m = searcher.next(text);
        if (m) {
            return createFindMatch(this._getMultilineMatchRange(model, deltaOffset, text, lfCounter, m.index, m[0]), m, captureMatches);
        }
        if (searchStart.lineNumber !== 1 || searchStart.column !== 1) {
            // Try again from the top
            return this._doFindNextMatchMultiline(model, new Position(1, 1), searcher, captureMatches);
        }
        return null;
    };
    TextModelSearch._doFindNextMatchLineByLine = function (model, searchStart, searcher, captureMatches) {
        var lineCount = model.getLineCount();
        var startLineNumber = searchStart.lineNumber;
        // Look in first line
        var text = model.getLineContent(startLineNumber);
        var r = this._findFirstMatchInLine(searcher, text, startLineNumber, searchStart.column, captureMatches);
        if (r) {
            return r;
        }
        for (var i = 1; i <= lineCount; i++) {
            var lineIndex = (startLineNumber + i - 1) % lineCount;
            var text_3 = model.getLineContent(lineIndex + 1);
            var r_1 = this._findFirstMatchInLine(searcher, text_3, lineIndex + 1, 1, captureMatches);
            if (r_1) {
                return r_1;
            }
        }
        return null;
    };
    TextModelSearch._findFirstMatchInLine = function (searcher, text, lineNumber, fromColumn, captureMatches) {
        // Set regex to search from column
        searcher.reset(fromColumn - 1);
        var m = searcher.next(text);
        if (m) {
            return createFindMatch(new Range(lineNumber, m.index + 1, lineNumber, m.index + 1 + m[0].length), m, captureMatches);
        }
        return null;
    };
    TextModelSearch.findPreviousMatch = function (model, searchParams, searchStart, captureMatches) {
        var searchData = searchParams.parseSearchRequest();
        if (!searchData) {
            return null;
        }
        var searcher = new Searcher(searchData.wordSeparators, searchData.regex);
        if (searchData.regex.multiline) {
            return this._doFindPreviousMatchMultiline(model, searchStart, searcher, captureMatches);
        }
        return this._doFindPreviousMatchLineByLine(model, searchStart, searcher, captureMatches);
    };
    TextModelSearch._doFindPreviousMatchMultiline = function (model, searchStart, searcher, captureMatches) {
        var matches = this._doFindMatchesMultiline(model, new Range(1, 1, searchStart.lineNumber, searchStart.column), searcher, captureMatches, 10 * LIMIT_FIND_COUNT);
        if (matches.length > 0) {
            return matches[matches.length - 1];
        }
        var lineCount = model.getLineCount();
        if (searchStart.lineNumber !== lineCount || searchStart.column !== model.getLineMaxColumn(lineCount)) {
            // Try again with all content
            return this._doFindPreviousMatchMultiline(model, new Position(lineCount, model.getLineMaxColumn(lineCount)), searcher, captureMatches);
        }
        return null;
    };
    TextModelSearch._doFindPreviousMatchLineByLine = function (model, searchStart, searcher, captureMatches) {
        var lineCount = model.getLineCount();
        var startLineNumber = searchStart.lineNumber;
        // Look in first line
        var text = model.getLineContent(startLineNumber).substring(0, searchStart.column - 1);
        var r = this._findLastMatchInLine(searcher, text, startLineNumber, captureMatches);
        if (r) {
            return r;
        }
        for (var i = 1; i <= lineCount; i++) {
            var lineIndex = (lineCount + startLineNumber - i - 1) % lineCount;
            var text_4 = model.getLineContent(lineIndex + 1);
            var r_2 = this._findLastMatchInLine(searcher, text_4, lineIndex + 1, captureMatches);
            if (r_2) {
                return r_2;
            }
        }
        return null;
    };
    TextModelSearch._findLastMatchInLine = function (searcher, text, lineNumber, captureMatches) {
        var bestResult = null;
        var m;
        searcher.reset(0);
        while ((m = searcher.next(text))) {
            bestResult = createFindMatch(new Range(lineNumber, m.index + 1, lineNumber, m.index + 1 + m[0].length), m, captureMatches);
        }
        return bestResult;
    };
    return TextModelSearch;
}());
function leftIsWordBounday(wordSeparators, text, textLength, matchStartIndex, matchLength) {
    if (matchStartIndex === 0) {
        // Match starts at start of string
        return true;
    }
    var charBefore = text.charCodeAt(matchStartIndex - 1);
    if (wordSeparators.get(charBefore) !== 0 /* Regular */) {
        // The character before the match is a word separator
        return true;
    }
    if (charBefore === 13 /* CarriageReturn */ || charBefore === 10 /* LineFeed */) {
        // The character before the match is line break or carriage return.
        return true;
    }
    if (matchLength > 0) {
        var firstCharInMatch = text.charCodeAt(matchStartIndex);
        if (wordSeparators.get(firstCharInMatch) !== 0 /* Regular */) {
            // The first character inside the match is a word separator
            return true;
        }
    }
    return false;
}
function rightIsWordBounday(wordSeparators, text, textLength, matchStartIndex, matchLength) {
    if (matchStartIndex + matchLength === textLength) {
        // Match ends at end of string
        return true;
    }
    var charAfter = text.charCodeAt(matchStartIndex + matchLength);
    if (wordSeparators.get(charAfter) !== 0 /* Regular */) {
        // The character after the match is a word separator
        return true;
    }
    if (charAfter === 13 /* CarriageReturn */ || charAfter === 10 /* LineFeed */) {
        // The character after the match is line break or carriage return.
        return true;
    }
    if (matchLength > 0) {
        var lastCharInMatch = text.charCodeAt(matchStartIndex + matchLength - 1);
        if (wordSeparators.get(lastCharInMatch) !== 0 /* Regular */) {
            // The last character in the match is a word separator
            return true;
        }
    }
    return false;
}
function isValidMatch(wordSeparators, text, textLength, matchStartIndex, matchLength) {
    return (leftIsWordBounday(wordSeparators, text, textLength, matchStartIndex, matchLength)
        && rightIsWordBounday(wordSeparators, text, textLength, matchStartIndex, matchLength));
}
var Searcher = /** @class */ (function () {
    function Searcher(wordSeparators, searchRegex) {
        this._wordSeparators = wordSeparators;
        this._searchRegex = searchRegex;
        this._prevMatchStartIndex = -1;
        this._prevMatchLength = 0;
    }
    Searcher.prototype.reset = function (lastIndex) {
        this._searchRegex.lastIndex = lastIndex;
        this._prevMatchStartIndex = -1;
        this._prevMatchLength = 0;
    };
    Searcher.prototype.next = function (text) {
        var textLength = text.length;
        var m;
        do {
            if (this._prevMatchStartIndex + this._prevMatchLength === textLength) {
                // Reached the end of the line
                return null;
            }
            m = this._searchRegex.exec(text);
            if (!m) {
                return null;
            }
            var matchStartIndex = m.index;
            var matchLength = m[0].length;
            if (matchStartIndex === this._prevMatchStartIndex && matchLength === this._prevMatchLength) {
                // Exit early if the regex matches the same range twice
                return null;
            }
            this._prevMatchStartIndex = matchStartIndex;
            this._prevMatchLength = matchLength;
            if (!this._wordSeparators || isValidMatch(this._wordSeparators, text, textLength, matchStartIndex, matchLength)) {
                return m;
            }
        } while (m);
        return null;
    };
    return Searcher;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// const lfRegex = new RegExp(/\r\n|\r|\n/g);
var AverageBufferSize = 65535;
function createUintArray(arr) {
    var r;
    if (arr[arr.length - 1] < 65536) {
        r = new Uint16Array(arr.length);
    }
    else {
        r = new Uint32Array(arr.length);
    }
    r.set(arr, 0);
    return r;
}
var LineStarts = /** @class */ (function () {
    function LineStarts(lineStarts, cr, lf, crlf, isBasicASCII) {
        this.lineStarts = lineStarts;
        this.cr = cr;
        this.lf = lf;
        this.crlf = crlf;
        this.isBasicASCII = isBasicASCII;
    }
    return LineStarts;
}());
function createLineStartsFast(str, readonly) {
    if (readonly === void 0) { readonly = true; }
    var r = [0], rLength = 1;
    for (var i = 0, len = str.length; i < len; i++) {
        var chr = str.charCodeAt(i);
        if (chr === 13 /* CarriageReturn */) {
            if (i + 1 < len && str.charCodeAt(i + 1) === 10 /* LineFeed */) {
                // \r\n... case
                r[rLength++] = i + 2;
                i++; // skip \n
            }
            else {
                // \r... case
                r[rLength++] = i + 1;
            }
        }
        else if (chr === 10 /* LineFeed */) {
            r[rLength++] = i + 1;
        }
    }
    if (readonly) {
        return createUintArray(r);
    }
    else {
        return r;
    }
}
function createLineStarts(r, str) {
    r.length = 0;
    r[0] = 0;
    var rLength = 1;
    var cr = 0, lf = 0, crlf = 0;
    var isBasicASCII = true;
    for (var i = 0, len = str.length; i < len; i++) {
        var chr = str.charCodeAt(i);
        if (chr === 13 /* CarriageReturn */) {
            if (i + 1 < len && str.charCodeAt(i + 1) === 10 /* LineFeed */) {
                // \r\n... case
                crlf++;
                r[rLength++] = i + 2;
                i++; // skip \n
            }
            else {
                cr++;
                // \r... case
                r[rLength++] = i + 1;
            }
        }
        else if (chr === 10 /* LineFeed */) {
            lf++;
            r[rLength++] = i + 1;
        }
        else {
            if (isBasicASCII) {
                if (chr !== 9 /* Tab */ && (chr < 32 || chr > 126)) {
                    isBasicASCII = false;
                }
            }
        }
    }
    var result = new LineStarts(createUintArray(r), cr, lf, crlf, isBasicASCII);
    r.length = 0;
    return result;
}
var Piece = /** @class */ (function () {
    function Piece(bufferIndex, start, end, lineFeedCnt, length) {
        this.bufferIndex = bufferIndex;
        this.start = start;
        this.end = end;
        this.lineFeedCnt = lineFeedCnt;
        this.length = length;
    }
    return Piece;
}());
var StringBuffer = /** @class */ (function () {
    function StringBuffer(buffer, lineStarts) {
        this.buffer = buffer;
        this.lineStarts = lineStarts;
    }
    return StringBuffer;
}());
var PieceTreeSearchCache = /** @class */ (function () {
    function PieceTreeSearchCache(limit) {
        this._limit = limit;
        this._cache = [];
    }
    PieceTreeSearchCache.prototype.get = function (offset) {
        for (var i = this._cache.length - 1; i >= 0; i--) {
            var nodePos = this._cache[i];
            if (nodePos.nodeStartOffset <= offset && nodePos.nodeStartOffset + nodePos.node.piece.length >= offset) {
                return nodePos;
            }
        }
        return null;
    };
    PieceTreeSearchCache.prototype.get2 = function (lineNumber) {
        for (var i = this._cache.length - 1; i >= 0; i--) {
            var nodePos = this._cache[i];
            if (nodePos.nodeStartLineNumber && nodePos.nodeStartLineNumber < lineNumber && nodePos.nodeStartLineNumber + nodePos.node.piece.lineFeedCnt >= lineNumber) {
                return nodePos;
            }
        }
        return null;
    };
    PieceTreeSearchCache.prototype.set = function (nodePosition) {
        if (this._cache.length >= this._limit) {
            this._cache.shift();
        }
        this._cache.push(nodePosition);
    };
    PieceTreeSearchCache.prototype.valdiate = function (offset) {
        var hasInvalidVal = false;
        var tmp = this._cache;
        for (var i = 0; i < tmp.length; i++) {
            var nodePos = tmp[i];
            if (nodePos.node.parent === null || nodePos.nodeStartOffset >= offset) {
                tmp[i] = null;
                hasInvalidVal = true;
                continue;
            }
        }
        if (hasInvalidVal) {
            var newArr = [];
            for (var i = 0; i < tmp.length; i++) {
                var entry = tmp[i];
                if (entry !== null) {
                    newArr.push(entry);
                }
            }
            this._cache = newArr;
        }
    };
    return PieceTreeSearchCache;
}());
var PieceTreeBase = /** @class */ (function () {
    function PieceTreeBase(chunks, eol, eolNormalized) {
        this.create(chunks, eol, eolNormalized);
    }
    PieceTreeBase.prototype.create = function (chunks, eol, eolNormalized) {
        this._buffers = [
            new StringBuffer('', [0])
        ];
        this._lastChangeBufferPos = { line: 0, column: 0 };
        this.root = SENTINEL$1;
        this._lineCnt = 1;
        this._length = 0;
        this._EOL = eol;
        this._EOLLength = eol.length;
        this._EOLNormalized = eolNormalized;
        var lastNode = null;
        for (var i = 0, len = chunks.length; i < len; i++) {
            if (chunks[i].buffer.length > 0) {
                if (!chunks[i].lineStarts) {
                    chunks[i].lineStarts = createLineStartsFast(chunks[i].buffer);
                }
                var piece = new Piece(i + 1, { line: 0, column: 0 }, { line: chunks[i].lineStarts.length - 1, column: chunks[i].buffer.length - chunks[i].lineStarts[chunks[i].lineStarts.length - 1] }, chunks[i].lineStarts.length - 1, chunks[i].buffer.length);
                this._buffers.push(chunks[i]);
                lastNode = this.rbInsertRight(lastNode, piece);
            }
        }
        this._searchCache = new PieceTreeSearchCache(1);
        this._lastVisitedLine = { lineNumber: 0, value: '' };
        this.computeBufferMetadata();
    };
    PieceTreeBase.prototype.normalizeEOL = function (eol) {
        var _this = this;
        var averageBufferSize = AverageBufferSize;
        var min = averageBufferSize - Math.floor(averageBufferSize / 3);
        var max = min * 2;
        var tempChunk = '';
        var tempChunkLen = 0;
        var chunks = [];
        this.iterate(this.root, function (node) {
            var str = _this.getNodeContent(node);
            var len = str.length;
            if (tempChunkLen <= min || tempChunkLen + len < max) {
                tempChunk += str;
                tempChunkLen += len;
                return true;
            }
            // flush anyways
            var text = tempChunk.replace(/\r\n|\r|\n/g, eol);
            chunks.push(new StringBuffer(text, createLineStartsFast(text)));
            tempChunk = str;
            tempChunkLen = len;
            return true;
        });
        if (tempChunkLen > 0) {
            var text = tempChunk.replace(/\r\n|\r|\n/g, eol);
            chunks.push(new StringBuffer(text, createLineStartsFast(text)));
        }
        this.create(chunks, eol, true);
    };
    // #region Buffer API
    PieceTreeBase.prototype.getEOL = function () {
        return this._EOL;
    };
    PieceTreeBase.prototype.setEOL = function (newEOL) {
        this._EOL = newEOL;
        this._EOLLength = this._EOL.length;
        this.normalizeEOL(newEOL);
    };
    PieceTreeBase.prototype.getOffsetAt = function (lineNumber, column) {
        var leftLen = 0; // inorder
        var x = this.root;
        while (x !== SENTINEL$1) {
            if (x.left !== SENTINEL$1 && x.lf_left + 1 >= lineNumber) {
                x = x.left;
            }
            else if (x.lf_left + x.piece.lineFeedCnt + 1 >= lineNumber) {
                leftLen += x.size_left;
                // lineNumber >= 2
                var accumualtedValInCurrentIndex = this.getAccumulatedValue(x, lineNumber - x.lf_left - 2);
                return leftLen += accumualtedValInCurrentIndex + column - 1;
            }
            else {
                lineNumber -= x.lf_left + x.piece.lineFeedCnt;
                leftLen += x.size_left + x.piece.length;
                x = x.right;
            }
        }
        return leftLen;
    };
    PieceTreeBase.prototype.getPositionAt = function (offset) {
        offset = Math.floor(offset);
        offset = Math.max(0, offset);
        var x = this.root;
        var lfCnt = 0;
        var originalOffset = offset;
        while (x !== SENTINEL$1) {
            if (x.size_left !== 0 && x.size_left >= offset) {
                x = x.left;
            }
            else if (x.size_left + x.piece.length >= offset) {
                var out = this.getIndexOf(x, offset - x.size_left);
                lfCnt += x.lf_left + out.index;
                if (out.index === 0) {
                    var lineStartOffset = this.getOffsetAt(lfCnt + 1, 1);
                    var column = originalOffset - lineStartOffset;
                    return new Position(lfCnt + 1, column + 1);
                }
                return new Position(lfCnt + 1, out.remainder + 1);
            }
            else {
                offset -= x.size_left + x.piece.length;
                lfCnt += x.lf_left + x.piece.lineFeedCnt;
                if (x.right === SENTINEL$1) {
                    // last node
                    var lineStartOffset = this.getOffsetAt(lfCnt + 1, 1);
                    var column = originalOffset - offset - lineStartOffset;
                    return new Position(lfCnt + 1, column + 1);
                }
                else {
                    x = x.right;
                }
            }
        }
        return new Position(1, 1);
    };
    PieceTreeBase.prototype.getValueInRange = function (range, eol) {
        if (range.startLineNumber === range.endLineNumber && range.startColumn === range.endColumn) {
            return '';
        }
        var startPosition = this.nodeAt2(range.startLineNumber, range.startColumn);
        var endPosition = this.nodeAt2(range.endLineNumber, range.endColumn);
        var value = this.getValueInRange2(startPosition, endPosition);
        if (eol) {
            if (eol !== this._EOL || !this._EOLNormalized) {
                return value.replace(/\r\n|\r|\n/g, eol);
            }
            if (eol === this.getEOL() && this._EOLNormalized) {
                return value;
            }
            return value.replace(/\r\n|\r|\n/g, eol);
        }
        return value;
    };
    PieceTreeBase.prototype.getValueInRange2 = function (startPosition, endPosition) {
        if (startPosition.node === endPosition.node) {
            var node = startPosition.node;
            var buffer_1 = this._buffers[node.piece.bufferIndex].buffer;
            var startOffset_1 = this.offsetInBuffer(node.piece.bufferIndex, node.piece.start);
            return buffer_1.substring(startOffset_1 + startPosition.remainder, startOffset_1 + endPosition.remainder);
        }
        var x = startPosition.node;
        var buffer = this._buffers[x.piece.bufferIndex].buffer;
        var startOffset = this.offsetInBuffer(x.piece.bufferIndex, x.piece.start);
        var ret = buffer.substring(startOffset + startPosition.remainder, startOffset + x.piece.length);
        x = x.next();
        while (x !== SENTINEL$1) {
            var buffer_2 = this._buffers[x.piece.bufferIndex].buffer;
            var startOffset_2 = this.offsetInBuffer(x.piece.bufferIndex, x.piece.start);
            if (x === endPosition.node) {
                ret += buffer_2.substring(startOffset_2, startOffset_2 + endPosition.remainder);
                break;
            }
            else {
                ret += buffer_2.substr(startOffset_2, x.piece.length);
            }
            x = x.next();
        }
        return ret;
    };
    PieceTreeBase.prototype.getLinesContent = function () {
        return this.getContentOfSubTree(this.root).split(/\r\n|\r|\n/);
    };
    PieceTreeBase.prototype.getLength = function () {
        return this._length;
    };
    PieceTreeBase.prototype.getLineCount = function () {
        return this._lineCnt;
    };
    PieceTreeBase.prototype.getLineContent = function (lineNumber) {
        if (this._lastVisitedLine.lineNumber === lineNumber) {
            return this._lastVisitedLine.value;
        }
        this._lastVisitedLine.lineNumber = lineNumber;
        if (lineNumber === this._lineCnt) {
            this._lastVisitedLine.value = this.getLineRawContent(lineNumber);
        }
        else if (this._EOLNormalized) {
            this._lastVisitedLine.value = this.getLineRawContent(lineNumber, this._EOLLength);
        }
        else {
            this._lastVisitedLine.value = this.getLineRawContent(lineNumber).replace(/(\r\n|\r|\n)$/, '');
        }
        return this._lastVisitedLine.value;
    };
    PieceTreeBase.prototype.getLineCharCode = function (lineNumber, index) {
        var nodePos = this.nodeAt2(lineNumber, index + 1);
        if (nodePos.remainder === nodePos.node.piece.length) {
            // the char we want to fetch is at the head of next node.
            var matchingNode = nodePos.node.next();
            if (!matchingNode) {
                return 0;
            }
            var buffer = this._buffers[matchingNode.piece.bufferIndex];
            var startOffset = this.offsetInBuffer(matchingNode.piece.bufferIndex, matchingNode.piece.start);
            return buffer.buffer.charCodeAt(startOffset);
        }
        else {
            var buffer = this._buffers[nodePos.node.piece.bufferIndex];
            var startOffset = this.offsetInBuffer(nodePos.node.piece.bufferIndex, nodePos.node.piece.start);
            var targetOffset = startOffset + nodePos.remainder;
            return buffer.buffer.charCodeAt(targetOffset);
        }
    };
    PieceTreeBase.prototype.getLineLength = function (lineNumber) {
        if (lineNumber === this.getLineCount()) {
            var startOffset = this.getOffsetAt(lineNumber, 1);
            return this.getLength() - startOffset;
        }
        return this.getOffsetAt(lineNumber + 1, 1) - this.getOffsetAt(lineNumber, 1) - this._EOLLength;
    };
    PieceTreeBase.prototype.findMatchesInNode = function (node, searcher, startLineNumber, startColumn, startCursor, endCursor, searchData, captureMatches, limitResultCount, resultLen, result) {
        var buffer = this._buffers[node.piece.bufferIndex];
        var startOffsetInBuffer = this.offsetInBuffer(node.piece.bufferIndex, node.piece.start);
        var start = this.offsetInBuffer(node.piece.bufferIndex, startCursor);
        var end = this.offsetInBuffer(node.piece.bufferIndex, endCursor);
        var m;
        // Reset regex to search from the beginning
        searcher.reset(start);
        var ret = { line: 0, column: 0 };
        do {
            m = searcher.next(buffer.buffer);
            if (m) {
                if (m.index >= end) {
                    return resultLen;
                }
                this.positionInBuffer(node, m.index - startOffsetInBuffer, ret);
                var lineFeedCnt = this.getLineFeedCnt(node.piece.bufferIndex, startCursor, ret);
                var retStartColumn = ret.line === startCursor.line ? ret.column - startCursor.column + startColumn : ret.column + 1;
                var retEndColumn = retStartColumn + m[0].length;
                result[resultLen++] = createFindMatch(new Range(startLineNumber + lineFeedCnt, retStartColumn, startLineNumber + lineFeedCnt, retEndColumn), m, captureMatches);
                if (m.index + m[0].length >= end) {
                    return resultLen;
                }
                if (resultLen >= limitResultCount) {
                    return resultLen;
                }
            }
        } while (m);
        return resultLen;
    };
    PieceTreeBase.prototype.findMatchesLineByLine = function (searchRange, searchData, captureMatches, limitResultCount) {
        var result = [];
        var resultLen = 0;
        var searcher = new Searcher(searchData.wordSeparators, searchData.regex);
        var startPostion = this.nodeAt2(searchRange.startLineNumber, searchRange.startColumn);
        if (startPostion === null) {
            return [];
        }
        var endPosition = this.nodeAt2(searchRange.endLineNumber, searchRange.endColumn);
        if (endPosition === null) {
            return [];
        }
        var start = this.positionInBuffer(startPostion.node, startPostion.remainder);
        var end = this.positionInBuffer(endPosition.node, endPosition.remainder);
        if (startPostion.node === endPosition.node) {
            this.findMatchesInNode(startPostion.node, searcher, searchRange.startLineNumber, searchRange.startColumn, start, end, searchData, captureMatches, limitResultCount, resultLen, result);
            return result;
        }
        var startLineNumber = searchRange.startLineNumber;
        var currentNode = startPostion.node;
        while (currentNode !== endPosition.node) {
            var lineBreakCnt = this.getLineFeedCnt(currentNode.piece.bufferIndex, start, currentNode.piece.end);
            if (lineBreakCnt >= 1) {
                // last line break position
                var lineStarts = this._buffers[currentNode.piece.bufferIndex].lineStarts;
                var startOffsetInBuffer = this.offsetInBuffer(currentNode.piece.bufferIndex, currentNode.piece.start);
                var nextLineStartOffset = lineStarts[start.line + lineBreakCnt];
                var startColumn_1 = startLineNumber === searchRange.startLineNumber ? searchRange.startColumn : 1;
                resultLen = this.findMatchesInNode(currentNode, searcher, startLineNumber, startColumn_1, start, this.positionInBuffer(currentNode, nextLineStartOffset - startOffsetInBuffer), searchData, captureMatches, limitResultCount, resultLen, result);
                if (resultLen >= limitResultCount) {
                    return result;
                }
                startLineNumber += lineBreakCnt;
            }
            var startColumn_2 = startLineNumber === searchRange.startLineNumber ? searchRange.startColumn - 1 : 0;
            // search for the remaining content
            if (startLineNumber === searchRange.endLineNumber) {
                var text = this.getLineContent(startLineNumber).substring(startColumn_2, searchRange.endColumn - 1);
                resultLen = this._findMatchesInLine(searchData, searcher, text, searchRange.endLineNumber, startColumn_2, resultLen, result, captureMatches, limitResultCount);
                return result;
            }
            resultLen = this._findMatchesInLine(searchData, searcher, this.getLineContent(startLineNumber).substr(startColumn_2), startLineNumber, startColumn_2, resultLen, result, captureMatches, limitResultCount);
            if (resultLen >= limitResultCount) {
                return result;
            }
            startLineNumber++;
            startPostion = this.nodeAt2(startLineNumber, 1);
            currentNode = startPostion.node;
            start = this.positionInBuffer(startPostion.node, startPostion.remainder);
        }
        if (startLineNumber === searchRange.endLineNumber) {
            var startColumn_3 = startLineNumber === searchRange.startLineNumber ? searchRange.startColumn - 1 : 0;
            var text = this.getLineContent(startLineNumber).substring(startColumn_3, searchRange.endColumn - 1);
            resultLen = this._findMatchesInLine(searchData, searcher, text, searchRange.endLineNumber, startColumn_3, resultLen, result, captureMatches, limitResultCount);
            return result;
        }
        var startColumn = startLineNumber === searchRange.startLineNumber ? searchRange.startColumn : 1;
        resultLen = this.findMatchesInNode(endPosition.node, searcher, startLineNumber, startColumn, start, end, searchData, captureMatches, limitResultCount, resultLen, result);
        return result;
    };
    PieceTreeBase.prototype._findMatchesInLine = function (searchData, searcher, text, lineNumber, deltaOffset, resultLen, result, captureMatches, limitResultCount) {
        var wordSeparators = searchData.wordSeparators;
        if (!captureMatches && searchData.simpleSearch) {
            var searchString = searchData.simpleSearch;
            var searchStringLen = searchString.length;
            var textLength = text.length;
            var lastMatchIndex = -searchStringLen;
            while ((lastMatchIndex = text.indexOf(searchString, lastMatchIndex + searchStringLen)) !== -1) {
                if (!wordSeparators || isValidMatch(wordSeparators, text, textLength, lastMatchIndex, searchStringLen)) {
                    result[resultLen++] = new FindMatch(new Range(lineNumber, lastMatchIndex + 1 + deltaOffset, lineNumber, lastMatchIndex + 1 + searchStringLen + deltaOffset), null);
                    if (resultLen >= limitResultCount) {
                        return resultLen;
                    }
                }
            }
            return resultLen;
        }
        var m;
        // Reset regex to search from the beginning
        searcher.reset(0);
        do {
            m = searcher.next(text);
            if (m) {
                result[resultLen++] = createFindMatch(new Range(lineNumber, m.index + 1 + deltaOffset, lineNumber, m.index + 1 + m[0].length + deltaOffset), m, captureMatches);
                if (resultLen >= limitResultCount) {
                    return resultLen;
                }
            }
        } while (m);
        return resultLen;
    };
    // #endregion
    // #region Piece Table
    PieceTreeBase.prototype.insert = function (offset, value, eolNormalized) {
        if (eolNormalized === void 0) { eolNormalized = false; }
        this._EOLNormalized = this._EOLNormalized && eolNormalized;
        this._lastVisitedLine.lineNumber = 0;
        this._lastVisitedLine.value = '';
        if (this.root !== SENTINEL$1) {
            var _a = this.nodeAt(offset), node = _a.node, remainder = _a.remainder, nodeStartOffset = _a.nodeStartOffset;
            var piece = node.piece;
            var bufferIndex = piece.bufferIndex;
            var insertPosInBuffer = this.positionInBuffer(node, remainder);
            if (node.piece.bufferIndex === 0 &&
                piece.end.line === this._lastChangeBufferPos.line &&
                piece.end.column === this._lastChangeBufferPos.column &&
                (nodeStartOffset + piece.length === offset) &&
                value.length < AverageBufferSize) {
                // changed buffer
                this.appendToNode(node, value);
                this.computeBufferMetadata();
                return;
            }
            if (nodeStartOffset === offset) {
                this.insertContentToNodeLeft(value, node);
                this._searchCache.valdiate(offset);
            }
            else if (nodeStartOffset + node.piece.length > offset) {
                // we are inserting into the middle of a node.
                var nodesToDel = [];
                var newRightPiece = new Piece(piece.bufferIndex, insertPosInBuffer, piece.end, this.getLineFeedCnt(piece.bufferIndex, insertPosInBuffer, piece.end), this.offsetInBuffer(bufferIndex, piece.end) - this.offsetInBuffer(bufferIndex, insertPosInBuffer));
                if (this.shouldCheckCRLF() && this.endWithCR(value)) {
                    var headOfRight = this.nodeCharCodeAt(node, remainder);
                    if (headOfRight === 10 /** \n */) {
                        var newStart = { line: newRightPiece.start.line + 1, column: 0 };
                        newRightPiece = new Piece(newRightPiece.bufferIndex, newStart, newRightPiece.end, this.getLineFeedCnt(newRightPiece.bufferIndex, newStart, newRightPiece.end), newRightPiece.length - 1);
                        value += '\n';
                    }
                }
                // reuse node for content before insertion point.
                if (this.shouldCheckCRLF() && this.startWithLF(value)) {
                    var tailOfLeft = this.nodeCharCodeAt(node, remainder - 1);
                    if (tailOfLeft === 13 /** \r */) {
                        var previousPos = this.positionInBuffer(node, remainder - 1);
                        this.deleteNodeTail(node, previousPos);
                        value = '\r' + value;
                        if (node.piece.length === 0) {
                            nodesToDel.push(node);
                        }
                    }
                    else {
                        this.deleteNodeTail(node, insertPosInBuffer);
                    }
                }
                else {
                    this.deleteNodeTail(node, insertPosInBuffer);
                }
                var newPieces = this.createNewPieces(value);
                if (newRightPiece.length > 0) {
                    this.rbInsertRight(node, newRightPiece);
                }
                var tmpNode = node;
                for (var k = 0; k < newPieces.length; k++) {
                    tmpNode = this.rbInsertRight(tmpNode, newPieces[k]);
                }
                this.deleteNodes(nodesToDel);
            }
            else {
                this.insertContentToNodeRight(value, node);
            }
        }
        else {
            // insert new node
            var pieces = this.createNewPieces(value);
            var node = this.rbInsertLeft(null, pieces[0]);
            for (var k = 1; k < pieces.length; k++) {
                node = this.rbInsertRight(node, pieces[k]);
            }
        }
        // todo, this is too brutal. Total line feed count should be updated the same way as lf_left.
        this.computeBufferMetadata();
    };
    PieceTreeBase.prototype.delete = function (offset, cnt) {
        this._lastVisitedLine.lineNumber = 0;
        this._lastVisitedLine.value = '';
        if (cnt <= 0 || this.root === SENTINEL$1) {
            return;
        }
        var startPosition = this.nodeAt(offset);
        var endPosition = this.nodeAt(offset + cnt);
        var startNode = startPosition.node;
        var endNode = endPosition.node;
        if (startNode === endNode) {
            var startSplitPosInBuffer_1 = this.positionInBuffer(startNode, startPosition.remainder);
            var endSplitPosInBuffer_1 = this.positionInBuffer(startNode, endPosition.remainder);
            if (startPosition.nodeStartOffset === offset) {
                if (cnt === startNode.piece.length) { // delete node
                    var next = startNode.next();
                    rbDelete(this, startNode);
                    this.validateCRLFWithPrevNode(next);
                    this.computeBufferMetadata();
                    return;
                }
                this.deleteNodeHead(startNode, endSplitPosInBuffer_1);
                this._searchCache.valdiate(offset);
                this.validateCRLFWithPrevNode(startNode);
                this.computeBufferMetadata();
                return;
            }
            if (startPosition.nodeStartOffset + startNode.piece.length === offset + cnt) {
                this.deleteNodeTail(startNode, startSplitPosInBuffer_1);
                this.validateCRLFWithNextNode(startNode);
                this.computeBufferMetadata();
                return;
            }
            // delete content in the middle, this node will be splitted to nodes
            this.shrinkNode(startNode, startSplitPosInBuffer_1, endSplitPosInBuffer_1);
            this.computeBufferMetadata();
            return;
        }
        var nodesToDel = [];
        var startSplitPosInBuffer = this.positionInBuffer(startNode, startPosition.remainder);
        this.deleteNodeTail(startNode, startSplitPosInBuffer);
        this._searchCache.valdiate(offset);
        if (startNode.piece.length === 0) {
            nodesToDel.push(startNode);
        }
        // update last touched node
        var endSplitPosInBuffer = this.positionInBuffer(endNode, endPosition.remainder);
        this.deleteNodeHead(endNode, endSplitPosInBuffer);
        if (endNode.piece.length === 0) {
            nodesToDel.push(endNode);
        }
        // delete nodes in between
        var secondNode = startNode.next();
        for (var node = secondNode; node !== SENTINEL$1 && node !== endNode; node = node.next()) {
            nodesToDel.push(node);
        }
        var prev = startNode.piece.length === 0 ? startNode.prev() : startNode;
        this.deleteNodes(nodesToDel);
        this.validateCRLFWithNextNode(prev);
        this.computeBufferMetadata();
    };
    PieceTreeBase.prototype.insertContentToNodeLeft = function (value, node) {
        // we are inserting content to the beginning of node
        var nodesToDel = [];
        if (this.shouldCheckCRLF() && this.endWithCR(value) && this.startWithLF(node)) {
            // move `\n` to new node.
            var piece = node.piece;
            var newStart = { line: piece.start.line + 1, column: 0 };
            var nPiece = new Piece(piece.bufferIndex, newStart, piece.end, this.getLineFeedCnt(piece.bufferIndex, newStart, piece.end), piece.length - 1);
            node.piece = nPiece;
            value += '\n';
            updateTreeMetadata(this, node, -1, -1);
            if (node.piece.length === 0) {
                nodesToDel.push(node);
            }
        }
        var newPieces = this.createNewPieces(value);
        var newNode = this.rbInsertLeft(node, newPieces[newPieces.length - 1]);
        for (var k = newPieces.length - 2; k >= 0; k--) {
            newNode = this.rbInsertLeft(newNode, newPieces[k]);
        }
        this.validateCRLFWithPrevNode(newNode);
        this.deleteNodes(nodesToDel);
    };
    PieceTreeBase.prototype.insertContentToNodeRight = function (value, node) {
        // we are inserting to the right of this node.
        if (this.adjustCarriageReturnFromNext(value, node)) {
            // move \n to the new node.
            value += '\n';
        }
        var newPieces = this.createNewPieces(value);
        var newNode = this.rbInsertRight(node, newPieces[0]);
        var tmpNode = newNode;
        for (var k = 1; k < newPieces.length; k++) {
            tmpNode = this.rbInsertRight(tmpNode, newPieces[k]);
        }
        this.validateCRLFWithPrevNode(newNode);
    };
    PieceTreeBase.prototype.positionInBuffer = function (node, remainder, ret) {
        var piece = node.piece;
        var bufferIndex = node.piece.bufferIndex;
        var lineStarts = this._buffers[bufferIndex].lineStarts;
        var startOffset = lineStarts[piece.start.line] + piece.start.column;
        var offset = startOffset + remainder;
        // binary search offset between startOffset and endOffset
        var low = piece.start.line;
        var high = piece.end.line;
        var mid = 0;
        var midStop = 0;
        var midStart = 0;
        while (low <= high) {
            mid = low + ((high - low) / 2) | 0;
            midStart = lineStarts[mid];
            if (mid === high) {
                break;
            }
            midStop = lineStarts[mid + 1];
            if (offset < midStart) {
                high = mid - 1;
            }
            else if (offset >= midStop) {
                low = mid + 1;
            }
            else {
                break;
            }
        }
        if (ret) {
            ret.line = mid;
            ret.column = offset - midStart;
            return null;
        }
        return {
            line: mid,
            column: offset - midStart
        };
    };
    PieceTreeBase.prototype.getLineFeedCnt = function (bufferIndex, start, end) {
        // we don't need to worry about start: abc\r|\n, or abc|\r, or abc|\n, or abc|\r\n doesn't change the fact that, there is one line break after start.
        // now let's take care of end: abc\r|\n, if end is in between \r and \n, we need to add line feed count by 1
        if (end.column === 0) {
            return end.line - start.line;
        }
        var lineStarts = this._buffers[bufferIndex].lineStarts;
        if (end.line === lineStarts.length - 1) { // it means, there is no \n after end, otherwise, there will be one more lineStart.
            return end.line - start.line;
        }
        var nextLineStartOffset = lineStarts[end.line + 1];
        var endOffset = lineStarts[end.line] + end.column;
        if (nextLineStartOffset > endOffset + 1) { // there are more than 1 character after end, which means it can't be \n
            return end.line - start.line;
        }
        // endOffset + 1 === nextLineStartOffset
        // character at endOffset is \n, so we check the character before first
        // if character at endOffset is \r, end.column is 0 and we can't get here.
        var previousCharOffset = endOffset - 1; // end.column > 0 so it's okay.
        var buffer = this._buffers[bufferIndex].buffer;
        if (buffer.charCodeAt(previousCharOffset) === 13) {
            return end.line - start.line + 1;
        }
        else {
            return end.line - start.line;
        }
    };
    PieceTreeBase.prototype.offsetInBuffer = function (bufferIndex, cursor) {
        var lineStarts = this._buffers[bufferIndex].lineStarts;
        return lineStarts[cursor.line] + cursor.column;
    };
    PieceTreeBase.prototype.deleteNodes = function (nodes) {
        for (var i = 0; i < nodes.length; i++) {
            rbDelete(this, nodes[i]);
        }
    };
    PieceTreeBase.prototype.createNewPieces = function (text) {
        if (text.length > AverageBufferSize) {
            // the content is large, operations like substring, charCode becomes slow
            // so here we split it into smaller chunks, just like what we did for CR/LF normalization
            var newPieces = [];
            while (text.length > AverageBufferSize) {
                var lastChar = text.charCodeAt(AverageBufferSize - 1);
                var splitText = void 0;
                if (lastChar === 13 /* CarriageReturn */ || (lastChar >= 0xd800 && lastChar <= 0xdbff)) {
                    // last character is \r or a high surrogate => keep it back
                    splitText = text.substring(0, AverageBufferSize - 1);
                    text = text.substring(AverageBufferSize - 1);
                }
                else {
                    splitText = text.substring(0, AverageBufferSize);
                    text = text.substring(AverageBufferSize);
                }
                var lineStarts_1 = createLineStartsFast(splitText);
                newPieces.push(new Piece(this._buffers.length, /* buffer index */ { line: 0, column: 0 }, { line: lineStarts_1.length - 1, column: splitText.length - lineStarts_1[lineStarts_1.length - 1] }, lineStarts_1.length - 1, splitText.length));
                this._buffers.push(new StringBuffer(splitText, lineStarts_1));
            }
            var lineStarts_2 = createLineStartsFast(text);
            newPieces.push(new Piece(this._buffers.length, /* buffer index */ { line: 0, column: 0 }, { line: lineStarts_2.length - 1, column: text.length - lineStarts_2[lineStarts_2.length - 1] }, lineStarts_2.length - 1, text.length));
            this._buffers.push(new StringBuffer(text, lineStarts_2));
            return newPieces;
        }
        var startOffset = this._buffers[0].buffer.length;
        var lineStarts = createLineStartsFast(text, false);
        var start = this._lastChangeBufferPos;
        if (this._buffers[0].lineStarts[this._buffers[0].lineStarts.length - 1] === startOffset
            && startOffset !== 0
            && this.startWithLF(text)
            && this.endWithCR(this._buffers[0].buffer) // todo, we can check this._lastChangeBufferPos's column as it's the last one
        ) {
            this._lastChangeBufferPos = { line: this._lastChangeBufferPos.line, column: this._lastChangeBufferPos.column + 1 };
            start = this._lastChangeBufferPos;
            for (var i = 0; i < lineStarts.length; i++) {
                lineStarts[i] += startOffset + 1;
            }
            this._buffers[0].lineStarts = this._buffers[0].lineStarts.concat(lineStarts.slice(1));
            this._buffers[0].buffer += '_' + text;
            startOffset += 1;
        }
        else {
            if (startOffset !== 0) {
                for (var i = 0; i < lineStarts.length; i++) {
                    lineStarts[i] += startOffset;
                }
            }
            this._buffers[0].lineStarts = this._buffers[0].lineStarts.concat(lineStarts.slice(1));
            this._buffers[0].buffer += text;
        }
        var endOffset = this._buffers[0].buffer.length;
        var endIndex = this._buffers[0].lineStarts.length - 1;
        var endColumn = endOffset - this._buffers[0].lineStarts[endIndex];
        var endPos = { line: endIndex, column: endColumn };
        var newPiece = new Piece(0, /** todo@peng */ start, endPos, this.getLineFeedCnt(0, start, endPos), endOffset - startOffset);
        this._lastChangeBufferPos = endPos;
        return [newPiece];
    };
    PieceTreeBase.prototype.getLineRawContent = function (lineNumber, endOffset) {
        if (endOffset === void 0) { endOffset = 0; }
        var x = this.root;
        var ret = '';
        var cache = this._searchCache.get2(lineNumber);
        if (cache) {
            x = cache.node;
            var prevAccumualtedValue = this.getAccumulatedValue(x, lineNumber - cache.nodeStartLineNumber - 1);
            var buffer = this._buffers[x.piece.bufferIndex].buffer;
            var startOffset = this.offsetInBuffer(x.piece.bufferIndex, x.piece.start);
            if (cache.nodeStartLineNumber + x.piece.lineFeedCnt === lineNumber) {
                ret = buffer.substring(startOffset + prevAccumualtedValue, startOffset + x.piece.length);
            }
            else {
                var accumualtedValue = this.getAccumulatedValue(x, lineNumber - cache.nodeStartLineNumber);
                return buffer.substring(startOffset + prevAccumualtedValue, startOffset + accumualtedValue - endOffset);
            }
        }
        else {
            var nodeStartOffset = 0;
            var originalLineNumber = lineNumber;
            while (x !== SENTINEL$1) {
                if (x.left !== SENTINEL$1 && x.lf_left >= lineNumber - 1) {
                    x = x.left;
                }
                else if (x.lf_left + x.piece.lineFeedCnt > lineNumber - 1) {
                    var prevAccumualtedValue = this.getAccumulatedValue(x, lineNumber - x.lf_left - 2);
                    var accumualtedValue = this.getAccumulatedValue(x, lineNumber - x.lf_left - 1);
                    var buffer = this._buffers[x.piece.bufferIndex].buffer;
                    var startOffset = this.offsetInBuffer(x.piece.bufferIndex, x.piece.start);
                    nodeStartOffset += x.size_left;
                    this._searchCache.set({
                        node: x,
                        nodeStartOffset: nodeStartOffset,
                        nodeStartLineNumber: originalLineNumber - (lineNumber - 1 - x.lf_left)
                    });
                    return buffer.substring(startOffset + prevAccumualtedValue, startOffset + accumualtedValue - endOffset);
                }
                else if (x.lf_left + x.piece.lineFeedCnt === lineNumber - 1) {
                    var prevAccumualtedValue = this.getAccumulatedValue(x, lineNumber - x.lf_left - 2);
                    var buffer = this._buffers[x.piece.bufferIndex].buffer;
                    var startOffset = this.offsetInBuffer(x.piece.bufferIndex, x.piece.start);
                    ret = buffer.substring(startOffset + prevAccumualtedValue, startOffset + x.piece.length);
                    break;
                }
                else {
                    lineNumber -= x.lf_left + x.piece.lineFeedCnt;
                    nodeStartOffset += x.size_left + x.piece.length;
                    x = x.right;
                }
            }
        }
        // search in order, to find the node contains end column
        x = x.next();
        while (x !== SENTINEL$1) {
            var buffer = this._buffers[x.piece.bufferIndex].buffer;
            if (x.piece.lineFeedCnt > 0) {
                var accumualtedValue = this.getAccumulatedValue(x, 0);
                var startOffset = this.offsetInBuffer(x.piece.bufferIndex, x.piece.start);
                ret += buffer.substring(startOffset, startOffset + accumualtedValue - endOffset);
                return ret;
            }
            else {
                var startOffset = this.offsetInBuffer(x.piece.bufferIndex, x.piece.start);
                ret += buffer.substr(startOffset, x.piece.length);
            }
            x = x.next();
        }
        return ret;
    };
    PieceTreeBase.prototype.computeBufferMetadata = function () {
        var x = this.root;
        var lfCnt = 1;
        var len = 0;
        while (x !== SENTINEL$1) {
            lfCnt += x.lf_left + x.piece.lineFeedCnt;
            len += x.size_left + x.piece.length;
            x = x.right;
        }
        this._lineCnt = lfCnt;
        this._length = len;
        this._searchCache.valdiate(this._length);
    };
    // #region node operations
    PieceTreeBase.prototype.getIndexOf = function (node, accumulatedValue) {
        var piece = node.piece;
        var pos = this.positionInBuffer(node, accumulatedValue);
        var lineCnt = pos.line - piece.start.line;
        if (this.offsetInBuffer(piece.bufferIndex, piece.end) - this.offsetInBuffer(piece.bufferIndex, piece.start) === accumulatedValue) {
            // we are checking the end of this node, so a CRLF check is necessary.
            var realLineCnt = this.getLineFeedCnt(node.piece.bufferIndex, piece.start, pos);
            if (realLineCnt !== lineCnt) {
                // aha yes, CRLF
                return { index: realLineCnt, remainder: 0 };
            }
        }
        return { index: lineCnt, remainder: pos.column };
    };
    PieceTreeBase.prototype.getAccumulatedValue = function (node, index) {
        if (index < 0) {
            return 0;
        }
        var piece = node.piece;
        var lineStarts = this._buffers[piece.bufferIndex].lineStarts;
        var expectedLineStartIndex = piece.start.line + index + 1;
        if (expectedLineStartIndex > piece.end.line) {
            return lineStarts[piece.end.line] + piece.end.column - lineStarts[piece.start.line] - piece.start.column;
        }
        else {
            return lineStarts[expectedLineStartIndex] - lineStarts[piece.start.line] - piece.start.column;
        }
    };
    PieceTreeBase.prototype.deleteNodeTail = function (node, pos) {
        var piece = node.piece;
        var originalLFCnt = piece.lineFeedCnt;
        var originalEndOffset = this.offsetInBuffer(piece.bufferIndex, piece.end);
        var newEnd = pos;
        var newEndOffset = this.offsetInBuffer(piece.bufferIndex, newEnd);
        var newLineFeedCnt = this.getLineFeedCnt(piece.bufferIndex, piece.start, newEnd);
        var lf_delta = newLineFeedCnt - originalLFCnt;
        var size_delta = newEndOffset - originalEndOffset;
        var newLength = piece.length + size_delta;
        node.piece = new Piece(piece.bufferIndex, piece.start, newEnd, newLineFeedCnt, newLength);
        updateTreeMetadata(this, node, size_delta, lf_delta);
    };
    PieceTreeBase.prototype.deleteNodeHead = function (node, pos) {
        var piece = node.piece;
        var originalLFCnt = piece.lineFeedCnt;
        var originalStartOffset = this.offsetInBuffer(piece.bufferIndex, piece.start);
        var newStart = pos;
        var newLineFeedCnt = this.getLineFeedCnt(piece.bufferIndex, newStart, piece.end);
        var newStartOffset = this.offsetInBuffer(piece.bufferIndex, newStart);
        var lf_delta = newLineFeedCnt - originalLFCnt;
        var size_delta = originalStartOffset - newStartOffset;
        var newLength = piece.length + size_delta;
        node.piece = new Piece(piece.bufferIndex, newStart, piece.end, newLineFeedCnt, newLength);
        updateTreeMetadata(this, node, size_delta, lf_delta);
    };
    PieceTreeBase.prototype.shrinkNode = function (node, start, end) {
        var piece = node.piece;
        var originalStartPos = piece.start;
        var originalEndPos = piece.end;
        // old piece, originalStartPos, start
        var oldLength = piece.length;
        var oldLFCnt = piece.lineFeedCnt;
        var newEnd = start;
        var newLineFeedCnt = this.getLineFeedCnt(piece.bufferIndex, piece.start, newEnd);
        var newLength = this.offsetInBuffer(piece.bufferIndex, start) - this.offsetInBuffer(piece.bufferIndex, originalStartPos);
        node.piece = new Piece(piece.bufferIndex, piece.start, newEnd, newLineFeedCnt, newLength);
        updateTreeMetadata(this, node, newLength - oldLength, newLineFeedCnt - oldLFCnt);
        // new right piece, end, originalEndPos
        var newPiece = new Piece(piece.bufferIndex, end, originalEndPos, this.getLineFeedCnt(piece.bufferIndex, end, originalEndPos), this.offsetInBuffer(piece.bufferIndex, originalEndPos) - this.offsetInBuffer(piece.bufferIndex, end));
        var newNode = this.rbInsertRight(node, newPiece);
        this.validateCRLFWithPrevNode(newNode);
    };
    PieceTreeBase.prototype.appendToNode = function (node, value) {
        if (this.adjustCarriageReturnFromNext(value, node)) {
            value += '\n';
        }
        var hitCRLF = this.shouldCheckCRLF() && this.startWithLF(value) && this.endWithCR(node);
        var startOffset = this._buffers[0].buffer.length;
        this._buffers[0].buffer += value;
        var lineStarts = createLineStartsFast(value, false);
        for (var i = 0; i < lineStarts.length; i++) {
            lineStarts[i] += startOffset;
        }
        if (hitCRLF) {
            var prevStartOffset = this._buffers[0].lineStarts[this._buffers[0].lineStarts.length - 2];
            this._buffers[0].lineStarts.pop();
            // _lastChangeBufferPos is already wrong
            this._lastChangeBufferPos = { line: this._lastChangeBufferPos.line - 1, column: startOffset - prevStartOffset };
        }
        this._buffers[0].lineStarts = this._buffers[0].lineStarts.concat(lineStarts.slice(1));
        var endIndex = this._buffers[0].lineStarts.length - 1;
        var endColumn = this._buffers[0].buffer.length - this._buffers[0].lineStarts[endIndex];
        var newEnd = { line: endIndex, column: endColumn };
        var newLength = node.piece.length + value.length;
        var oldLineFeedCnt = node.piece.lineFeedCnt;
        var newLineFeedCnt = this.getLineFeedCnt(0, node.piece.start, newEnd);
        var lf_delta = newLineFeedCnt - oldLineFeedCnt;
        node.piece = new Piece(node.piece.bufferIndex, node.piece.start, newEnd, newLineFeedCnt, newLength);
        this._lastChangeBufferPos = newEnd;
        updateTreeMetadata(this, node, value.length, lf_delta);
    };
    PieceTreeBase.prototype.nodeAt = function (offset) {
        var x = this.root;
        var cache = this._searchCache.get(offset);
        if (cache) {
            return {
                node: cache.node,
                nodeStartOffset: cache.nodeStartOffset,
                remainder: offset - cache.nodeStartOffset
            };
        }
        var nodeStartOffset = 0;
        while (x !== SENTINEL$1) {
            if (x.size_left > offset) {
                x = x.left;
            }
            else if (x.size_left + x.piece.length >= offset) {
                nodeStartOffset += x.size_left;
                var ret = {
                    node: x,
                    remainder: offset - x.size_left,
                    nodeStartOffset: nodeStartOffset
                };
                this._searchCache.set(ret);
                return ret;
            }
            else {
                offset -= x.size_left + x.piece.length;
                nodeStartOffset += x.size_left + x.piece.length;
                x = x.right;
            }
        }
        return null;
    };
    PieceTreeBase.prototype.nodeAt2 = function (lineNumber, column) {
        var x = this.root;
        var nodeStartOffset = 0;
        while (x !== SENTINEL$1) {
            if (x.left !== SENTINEL$1 && x.lf_left >= lineNumber - 1) {
                x = x.left;
            }
            else if (x.lf_left + x.piece.lineFeedCnt > lineNumber - 1) {
                var prevAccumualtedValue = this.getAccumulatedValue(x, lineNumber - x.lf_left - 2);
                var accumualtedValue = this.getAccumulatedValue(x, lineNumber - x.lf_left - 1);
                nodeStartOffset += x.size_left;
                return {
                    node: x,
                    remainder: Math.min(prevAccumualtedValue + column - 1, accumualtedValue),
                    nodeStartOffset: nodeStartOffset
                };
            }
            else if (x.lf_left + x.piece.lineFeedCnt === lineNumber - 1) {
                var prevAccumualtedValue = this.getAccumulatedValue(x, lineNumber - x.lf_left - 2);
                if (prevAccumualtedValue + column - 1 <= x.piece.length) {
                    return {
                        node: x,
                        remainder: prevAccumualtedValue + column - 1,
                        nodeStartOffset: nodeStartOffset
                    };
                }
                else {
                    column -= x.piece.length - prevAccumualtedValue;
                    break;
                }
            }
            else {
                lineNumber -= x.lf_left + x.piece.lineFeedCnt;
                nodeStartOffset += x.size_left + x.piece.length;
                x = x.right;
            }
        }
        // search in order, to find the node contains position.column
        x = x.next();
        while (x !== SENTINEL$1) {
            if (x.piece.lineFeedCnt > 0) {
                var accumualtedValue = this.getAccumulatedValue(x, 0);
                var nodeStartOffset_1 = this.offsetOfNode(x);
                return {
                    node: x,
                    remainder: Math.min(column - 1, accumualtedValue),
                    nodeStartOffset: nodeStartOffset_1
                };
            }
            else {
                if (x.piece.length >= column - 1) {
                    var nodeStartOffset_2 = this.offsetOfNode(x);
                    return {
                        node: x,
                        remainder: column - 1,
                        nodeStartOffset: nodeStartOffset_2
                    };
                }
                else {
                    column -= x.piece.length;
                }
            }
            x = x.next();
        }
        return null;
    };
    PieceTreeBase.prototype.nodeCharCodeAt = function (node, offset) {
        if (node.piece.lineFeedCnt < 1) {
            return -1;
        }
        var buffer = this._buffers[node.piece.bufferIndex];
        var newOffset = this.offsetInBuffer(node.piece.bufferIndex, node.piece.start) + offset;
        return buffer.buffer.charCodeAt(newOffset);
    };
    PieceTreeBase.prototype.offsetOfNode = function (node) {
        if (!node) {
            return 0;
        }
        var pos = node.size_left;
        while (node !== this.root) {
            if (node.parent.right === node) {
                pos += node.parent.size_left + node.parent.piece.length;
            }
            node = node.parent;
        }
        return pos;
    };
    // #endregion
    // #region CRLF
    PieceTreeBase.prototype.shouldCheckCRLF = function () {
        return !(this._EOLNormalized && this._EOL === '\n');
    };
    PieceTreeBase.prototype.startWithLF = function (val) {
        if (typeof val === 'string') {
            return val.charCodeAt(0) === 10;
        }
        if (val === SENTINEL$1 || val.piece.lineFeedCnt === 0) {
            return false;
        }
        var piece = val.piece;
        var lineStarts = this._buffers[piece.bufferIndex].lineStarts;
        var line = piece.start.line;
        var startOffset = lineStarts[line] + piece.start.column;
        if (line === lineStarts.length - 1) {
            // last line, so there is no line feed at the end of this line
            return false;
        }
        var nextLineOffset = lineStarts[line + 1];
        if (nextLineOffset > startOffset + 1) {
            return false;
        }
        return this._buffers[piece.bufferIndex].buffer.charCodeAt(startOffset) === 10;
    };
    PieceTreeBase.prototype.endWithCR = function (val) {
        if (typeof val === 'string') {
            return val.charCodeAt(val.length - 1) === 13;
        }
        if (val === SENTINEL$1 || val.piece.lineFeedCnt === 0) {
            return false;
        }
        return this.nodeCharCodeAt(val, val.piece.length - 1) === 13;
    };
    PieceTreeBase.prototype.validateCRLFWithPrevNode = function (nextNode) {
        if (this.shouldCheckCRLF() && this.startWithLF(nextNode)) {
            var node = nextNode.prev();
            if (this.endWithCR(node)) {
                this.fixCRLF(node, nextNode);
            }
        }
    };
    PieceTreeBase.prototype.validateCRLFWithNextNode = function (node) {
        if (this.shouldCheckCRLF() && this.endWithCR(node)) {
            var nextNode = node.next();
            if (this.startWithLF(nextNode)) {
                this.fixCRLF(node, nextNode);
            }
        }
    };
    PieceTreeBase.prototype.fixCRLF = function (prev, next) {
        var nodesToDel = [];
        // update node
        var lineStarts = this._buffers[prev.piece.bufferIndex].lineStarts;
        var newEnd;
        if (prev.piece.end.column === 0) {
            // it means, last line ends with \r, not \r\n
            newEnd = { line: prev.piece.end.line - 1, column: lineStarts[prev.piece.end.line] - lineStarts[prev.piece.end.line - 1] - 1 };
        }
        else {
            // \r\n
            newEnd = { line: prev.piece.end.line, column: prev.piece.end.column - 1 };
        }
        var prevNewLength = prev.piece.length - 1;
        var prevNewLFCnt = prev.piece.lineFeedCnt - 1;
        prev.piece = new Piece(prev.piece.bufferIndex, prev.piece.start, newEnd, prevNewLFCnt, prevNewLength);
        updateTreeMetadata(this, prev, -1, -1);
        if (prev.piece.length === 0) {
            nodesToDel.push(prev);
        }
        // update nextNode
        var newStart = { line: next.piece.start.line + 1, column: 0 };
        var newLength = next.piece.length - 1;
        var newLineFeedCnt = this.getLineFeedCnt(next.piece.bufferIndex, newStart, next.piece.end);
        next.piece = new Piece(next.piece.bufferIndex, newStart, next.piece.end, newLineFeedCnt, newLength);
        updateTreeMetadata(this, next, -1, -1);
        if (next.piece.length === 0) {
            nodesToDel.push(next);
        }
        // create new piece which contains \r\n
        var pieces = this.createNewPieces('\r\n');
        this.rbInsertRight(prev, pieces[0]);
        // delete empty nodes
        for (var i = 0; i < nodesToDel.length; i++) {
            rbDelete(this, nodesToDel[i]);
        }
    };
    PieceTreeBase.prototype.adjustCarriageReturnFromNext = function (value, node) {
        if (this.shouldCheckCRLF() && this.endWithCR(value)) {
            var nextNode = node.next();
            if (this.startWithLF(nextNode)) {
                // move `\n` forward
                value += '\n';
                if (nextNode.piece.length === 1) {
                    rbDelete(this, nextNode);
                }
                else {
                    var piece = nextNode.piece;
                    var newStart = { line: piece.start.line + 1, column: 0 };
                    var newLength = piece.length - 1;
                    var newLineFeedCnt = this.getLineFeedCnt(piece.bufferIndex, newStart, piece.end);
                    nextNode.piece = new Piece(piece.bufferIndex, newStart, piece.end, newLineFeedCnt, newLength);
                    updateTreeMetadata(this, nextNode, -1, -1);
                }
                return true;
            }
        }
        return false;
    };
    // #endregion
    // #endregion
    // #region Tree operations
    PieceTreeBase.prototype.iterate = function (node, callback) {
        if (node === SENTINEL$1) {
            return callback(SENTINEL$1);
        }
        var leftRet = this.iterate(node.left, callback);
        if (!leftRet) {
            return leftRet;
        }
        return callback(node) && this.iterate(node.right, callback);
    };
    PieceTreeBase.prototype.getNodeContent = function (node) {
        if (node === SENTINEL$1) {
            return '';
        }
        var buffer = this._buffers[node.piece.bufferIndex];
        var currentContent;
        var piece = node.piece;
        var startOffset = this.offsetInBuffer(piece.bufferIndex, piece.start);
        var endOffset = this.offsetInBuffer(piece.bufferIndex, piece.end);
        currentContent = buffer.buffer.substring(startOffset, endOffset);
        return currentContent;
    };
    /**
     *      node              node
     *     /  \              /  \
     *    a   b    <----   a    b
     *                         /
     *                        z
     */
    PieceTreeBase.prototype.rbInsertRight = function (node, p) {
        var z = new TreeNode(p, 1 /* Red */);
        z.left = SENTINEL$1;
        z.right = SENTINEL$1;
        z.parent = SENTINEL$1;
        z.size_left = 0;
        z.lf_left = 0;
        var x = this.root;
        if (x === SENTINEL$1) {
            this.root = z;
            z.color = 0 /* Black */;
        }
        else if (node.right === SENTINEL$1) {
            node.right = z;
            z.parent = node;
        }
        else {
            var nextNode = leftest$1(node.right);
            nextNode.left = z;
            z.parent = nextNode;
        }
        fixInsert(this, z);
        return z;
    };
    /**
     *      node              node
     *     /  \              /  \
     *    a   b     ---->   a    b
     *                       \
     *                        z
     */
    PieceTreeBase.prototype.rbInsertLeft = function (node, p) {
        var z = new TreeNode(p, 1 /* Red */);
        z.left = SENTINEL$1;
        z.right = SENTINEL$1;
        z.parent = SENTINEL$1;
        z.size_left = 0;
        z.lf_left = 0;
        if (this.root === SENTINEL$1) {
            this.root = z;
            z.color = 0 /* Black */;
        }
        else if (node.left === SENTINEL$1) {
            node.left = z;
            z.parent = node;
        }
        else {
            var prevNode = righttest(node.left); // a
            prevNode.right = z;
            z.parent = prevNode;
        }
        fixInsert(this, z);
        return z;
    };
    PieceTreeBase.prototype.getContentOfSubTree = function (node) {
        var _this = this;
        var str = '';
        this.iterate(node, function (node) {
            str += _this.getNodeContent(node);
            return true;
        });
        return str;
    };
    return PieceTreeBase;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var PieceTreeTextBuffer = /** @class */ (function () {
    function PieceTreeTextBuffer(chunks, BOM, eol, containsRTL$$1, isBasicASCII$$1, eolNormalized) {
        this._BOM = BOM;
        this._mightContainNonBasicASCII = !isBasicASCII$$1;
        this._mightContainRTL = containsRTL$$1;
        this._pieceTree = new PieceTreeBase(chunks, eol, eolNormalized);
    }
    PieceTreeTextBuffer.prototype.mightContainRTL = function () {
        return this._mightContainRTL;
    };
    PieceTreeTextBuffer.prototype.mightContainNonBasicASCII = function () {
        return this._mightContainNonBasicASCII;
    };
    PieceTreeTextBuffer.prototype.getBOM = function () {
        return this._BOM;
    };
    PieceTreeTextBuffer.prototype.getEOL = function () {
        return this._pieceTree.getEOL();
    };
    PieceTreeTextBuffer.prototype.getOffsetAt = function (lineNumber, column) {
        return this._pieceTree.getOffsetAt(lineNumber, column);
    };
    PieceTreeTextBuffer.prototype.getPositionAt = function (offset) {
        return this._pieceTree.getPositionAt(offset);
    };
    PieceTreeTextBuffer.prototype.getRangeAt = function (start, length) {
        var end = start + length;
        var startPosition = this.getPositionAt(start);
        var endPosition = this.getPositionAt(end);
        return new Range(startPosition.lineNumber, startPosition.column, endPosition.lineNumber, endPosition.column);
    };
    PieceTreeTextBuffer.prototype.getValueInRange = function (range, eol) {
        if (eol === void 0) { eol = 0 /* TextDefined */; }
        if (range.isEmpty()) {
            return '';
        }
        var lineEnding = this._getEndOfLine(eol);
        return this._pieceTree.getValueInRange(range, lineEnding);
    };
    PieceTreeTextBuffer.prototype.getValueLengthInRange = function (range, eol) {
        if (eol === void 0) { eol = 0 /* TextDefined */; }
        if (range.isEmpty()) {
            return 0;
        }
        if (range.startLineNumber === range.endLineNumber) {
            return (range.endColumn - range.startColumn);
        }
        var startOffset = this.getOffsetAt(range.startLineNumber, range.startColumn);
        var endOffset = this.getOffsetAt(range.endLineNumber, range.endColumn);
        return endOffset - startOffset;
    };
    PieceTreeTextBuffer.prototype.getLength = function () {
        return this._pieceTree.getLength();
    };
    PieceTreeTextBuffer.prototype.getLineCount = function () {
        return this._pieceTree.getLineCount();
    };
    PieceTreeTextBuffer.prototype.getLinesContent = function () {
        return this._pieceTree.getLinesContent();
    };
    PieceTreeTextBuffer.prototype.getLineContent = function (lineNumber) {
        return this._pieceTree.getLineContent(lineNumber);
    };
    PieceTreeTextBuffer.prototype.getLineCharCode = function (lineNumber, index) {
        return this._pieceTree.getLineCharCode(lineNumber, index);
    };
    PieceTreeTextBuffer.prototype.getLineLength = function (lineNumber) {
        return this._pieceTree.getLineLength(lineNumber);
    };
    PieceTreeTextBuffer.prototype.getLineFirstNonWhitespaceColumn = function (lineNumber) {
        var result = firstNonWhitespaceIndex(this.getLineContent(lineNumber));
        if (result === -1) {
            return 0;
        }
        return result + 1;
    };
    PieceTreeTextBuffer.prototype.getLineLastNonWhitespaceColumn = function (lineNumber) {
        var result = lastNonWhitespaceIndex(this.getLineContent(lineNumber));
        if (result === -1) {
            return 0;
        }
        return result + 2;
    };
    PieceTreeTextBuffer.prototype._getEndOfLine = function (eol) {
        switch (eol) {
            case 1 /* LF */:
                return '\n';
            case 2 /* CRLF */:
                return '\r\n';
            case 0 /* TextDefined */:
                return this.getEOL();
        }
        throw new Error('Unknown EOL preference');
    };
    PieceTreeTextBuffer.prototype.setEOL = function (newEOL) {
        this._pieceTree.setEOL(newEOL);
    };
    PieceTreeTextBuffer.prototype.applyEdits = function (rawOperations, recordTrimAutoWhitespace) {
        var mightContainRTL = this._mightContainRTL;
        var mightContainNonBasicASCII = this._mightContainNonBasicASCII;
        var canReduceOperations = true;
        var operations = [];
        for (var i = 0; i < rawOperations.length; i++) {
            var op = rawOperations[i];
            if (canReduceOperations && op._isTracked) {
                canReduceOperations = false;
            }
            var validatedRange = op.range;
            if (!mightContainRTL && op.text) {
                // check if the new inserted text contains RTL
                mightContainRTL = containsRTL(op.text);
            }
            if (!mightContainNonBasicASCII && op.text) {
                mightContainNonBasicASCII = !isBasicASCII(op.text);
            }
            operations[i] = {
                sortIndex: i,
                identifier: op.identifier || null,
                range: validatedRange,
                rangeOffset: this.getOffsetAt(validatedRange.startLineNumber, validatedRange.startColumn),
                rangeLength: this.getValueLengthInRange(validatedRange),
                lines: op.text ? op.text.split(/\r\n|\r|\n/) : null,
                forceMoveMarkers: Boolean(op.forceMoveMarkers),
                isAutoWhitespaceEdit: op.isAutoWhitespaceEdit || false
            };
        }
        // Sort operations ascending
        operations.sort(PieceTreeTextBuffer._sortOpsAscending);
        var hasTouchingRanges = false;
        for (var i = 0, count = operations.length - 1; i < count; i++) {
            var rangeEnd = operations[i].range.getEndPosition();
            var nextRangeStart = operations[i + 1].range.getStartPosition();
            if (nextRangeStart.isBeforeOrEqual(rangeEnd)) {
                if (nextRangeStart.isBefore(rangeEnd)) {
                    // overlapping ranges
                    throw new Error('Overlapping ranges are not allowed!');
                }
                hasTouchingRanges = true;
            }
        }
        if (canReduceOperations) {
            operations = this._reduceOperations(operations);
        }
        // Delta encode operations
        var reverseRanges = PieceTreeTextBuffer._getInverseEditRanges(operations);
        var newTrimAutoWhitespaceCandidates = [];
        for (var i = 0; i < operations.length; i++) {
            var op = operations[i];
            var reverseRange = reverseRanges[i];
            if (recordTrimAutoWhitespace && op.isAutoWhitespaceEdit && op.range.isEmpty()) {
                // Record already the future line numbers that might be auto whitespace removal candidates on next edit
                for (var lineNumber = reverseRange.startLineNumber; lineNumber <= reverseRange.endLineNumber; lineNumber++) {
                    var currentLineContent = '';
                    if (lineNumber === reverseRange.startLineNumber) {
                        currentLineContent = this.getLineContent(op.range.startLineNumber);
                        if (firstNonWhitespaceIndex(currentLineContent) !== -1) {
                            continue;
                        }
                    }
                    newTrimAutoWhitespaceCandidates.push({ lineNumber: lineNumber, oldContent: currentLineContent });
                }
            }
        }
        var reverseOperations = [];
        for (var i = 0; i < operations.length; i++) {
            var op = operations[i];
            var reverseRange = reverseRanges[i];
            reverseOperations[i] = {
                sortIndex: op.sortIndex,
                identifier: op.identifier,
                range: reverseRange,
                text: this.getValueInRange(op.range),
                forceMoveMarkers: op.forceMoveMarkers
            };
        }
        // Can only sort reverse operations when the order is not significant
        if (!hasTouchingRanges) {
            reverseOperations.sort(function (a, b) { return a.sortIndex - b.sortIndex; });
        }
        this._mightContainRTL = mightContainRTL;
        this._mightContainNonBasicASCII = mightContainNonBasicASCII;
        var contentChanges = this._doApplyEdits(operations);
        var trimAutoWhitespaceLineNumbers = null;
        if (recordTrimAutoWhitespace && newTrimAutoWhitespaceCandidates.length > 0) {
            // sort line numbers auto whitespace removal candidates for next edit descending
            newTrimAutoWhitespaceCandidates.sort(function (a, b) { return b.lineNumber - a.lineNumber; });
            trimAutoWhitespaceLineNumbers = [];
            for (var i = 0, len = newTrimAutoWhitespaceCandidates.length; i < len; i++) {
                var lineNumber = newTrimAutoWhitespaceCandidates[i].lineNumber;
                if (i > 0 && newTrimAutoWhitespaceCandidates[i - 1].lineNumber === lineNumber) {
                    // Do not have the same line number twice
                    continue;
                }
                var prevContent = newTrimAutoWhitespaceCandidates[i].oldContent;
                var lineContent = this.getLineContent(lineNumber);
                if (lineContent.length === 0 || lineContent === prevContent || firstNonWhitespaceIndex(lineContent) !== -1) {
                    continue;
                }
                trimAutoWhitespaceLineNumbers.push(lineNumber);
            }
        }
        return new ApplyEditsResult(reverseOperations, contentChanges, trimAutoWhitespaceLineNumbers);
    };
    /**
     * Transform operations such that they represent the same logic edit,
     * but that they also do not cause OOM crashes.
     */
    PieceTreeTextBuffer.prototype._reduceOperations = function (operations) {
        if (operations.length < 1000) {
            // We know from empirical testing that a thousand edits work fine regardless of their shape.
            return operations;
        }
        // At one point, due to how events are emitted and how each operation is handled,
        // some operations can trigger a high amount of temporary string allocations,
        // that will immediately get edited again.
        // e.g. a formatter inserting ridiculous ammounts of \n on a model with a single line
        // Therefore, the strategy is to collapse all the operations into a huge single edit operation
        return [this._toSingleEditOperation(operations)];
    };
    PieceTreeTextBuffer.prototype._toSingleEditOperation = function (operations) {
        var forceMoveMarkers = false, firstEditRange = operations[0].range, lastEditRange = operations[operations.length - 1].range, entireEditRange = new Range(firstEditRange.startLineNumber, firstEditRange.startColumn, lastEditRange.endLineNumber, lastEditRange.endColumn), lastEndLineNumber = firstEditRange.startLineNumber, lastEndColumn = firstEditRange.startColumn, result = [];
        for (var i = 0, len = operations.length; i < len; i++) {
            var operation = operations[i], range = operation.range;
            forceMoveMarkers = forceMoveMarkers || operation.forceMoveMarkers;
            // (1) -- Push old text
            for (var lineNumber = lastEndLineNumber; lineNumber < range.startLineNumber; lineNumber++) {
                if (lineNumber === lastEndLineNumber) {
                    result.push(this.getLineContent(lineNumber).substring(lastEndColumn - 1));
                }
                else {
                    result.push('\n');
                    result.push(this.getLineContent(lineNumber));
                }
            }
            if (range.startLineNumber === lastEndLineNumber) {
                result.push(this.getLineContent(range.startLineNumber).substring(lastEndColumn - 1, range.startColumn - 1));
            }
            else {
                result.push('\n');
                result.push(this.getLineContent(range.startLineNumber).substring(0, range.startColumn - 1));
            }
            // (2) -- Push new text
            if (operation.lines) {
                for (var j = 0, lenJ = operation.lines.length; j < lenJ; j++) {
                    if (j !== 0) {
                        result.push('\n');
                    }
                    result.push(operation.lines[j]);
                }
            }
            lastEndLineNumber = operation.range.endLineNumber;
            lastEndColumn = operation.range.endColumn;
        }
        return {
            sortIndex: 0,
            identifier: operations[0].identifier,
            range: entireEditRange,
            rangeOffset: this.getOffsetAt(entireEditRange.startLineNumber, entireEditRange.startColumn),
            rangeLength: this.getValueLengthInRange(entireEditRange, 0 /* TextDefined */),
            lines: result.join('').split('\n'),
            forceMoveMarkers: forceMoveMarkers,
            isAutoWhitespaceEdit: false
        };
    };
    PieceTreeTextBuffer.prototype._doApplyEdits = function (operations) {
        operations.sort(PieceTreeTextBuffer._sortOpsDescending);
        var contentChanges = [];
        // operations are from bottom to top
        for (var i = 0; i < operations.length; i++) {
            var op = operations[i];
            var startLineNumber = op.range.startLineNumber;
            var startColumn = op.range.startColumn;
            var endLineNumber = op.range.endLineNumber;
            var endColumn = op.range.endColumn;
            if (startLineNumber === endLineNumber && startColumn === endColumn && (!op.lines || op.lines.length === 0)) {
                // no-op
                continue;
            }
            var deletingLinesCnt = endLineNumber - startLineNumber;
            var insertingLinesCnt = (op.lines ? op.lines.length - 1 : 0);
            var editingLinesCnt = Math.min(deletingLinesCnt, insertingLinesCnt);
            var text = (op.lines ? op.lines.join(this.getEOL()) : '');
            if (text) {
                // replacement
                this._pieceTree.delete(op.rangeOffset, op.rangeLength);
                this._pieceTree.insert(op.rangeOffset, text, true);
            }
            else {
                // deletion
                this._pieceTree.delete(op.rangeOffset, op.rangeLength);
            }
            if (editingLinesCnt < insertingLinesCnt) {
                var newLinesContent = [];
                for (var j = editingLinesCnt + 1; j <= insertingLinesCnt; j++) {
                    newLinesContent.push(op.lines[j]);
                }
                newLinesContent[newLinesContent.length - 1] = this.getLineContent(startLineNumber + insertingLinesCnt - 1);
            }
            var contentChangeRange = new Range(startLineNumber, startColumn, endLineNumber, endColumn);
            contentChanges.push({
                range: contentChangeRange,
                rangeLength: op.rangeLength,
                text: text,
                rangeOffset: op.rangeOffset,
                forceMoveMarkers: op.forceMoveMarkers
            });
        }
        return contentChanges;
    };
    PieceTreeTextBuffer.prototype.findMatchesLineByLine = function (searchRange, searchData, captureMatches, limitResultCount) {
        return this._pieceTree.findMatchesLineByLine(searchRange, searchData, captureMatches, limitResultCount);
    };
    /**
     * Assumes `operations` are validated and sorted ascending
     */
    PieceTreeTextBuffer._getInverseEditRanges = function (operations) {
        var result = [];
        var prevOpEndLineNumber = 0;
        var prevOpEndColumn = 0;
        var prevOp = null;
        for (var i = 0, len = operations.length; i < len; i++) {
            var op = operations[i];
            var startLineNumber = void 0;
            var startColumn = void 0;
            if (prevOp) {
                if (prevOp.range.endLineNumber === op.range.startLineNumber) {
                    startLineNumber = prevOpEndLineNumber;
                    startColumn = prevOpEndColumn + (op.range.startColumn - prevOp.range.endColumn);
                }
                else {
                    startLineNumber = prevOpEndLineNumber + (op.range.startLineNumber - prevOp.range.endLineNumber);
                    startColumn = op.range.startColumn;
                }
            }
            else {
                startLineNumber = op.range.startLineNumber;
                startColumn = op.range.startColumn;
            }
            var resultRange = void 0;
            if (op.lines && op.lines.length > 0) {
                // the operation inserts something
                var lineCount = op.lines.length;
                var firstLine = op.lines[0];
                var lastLine = op.lines[lineCount - 1];
                if (lineCount === 1) {
                    // single line insert
                    resultRange = new Range(startLineNumber, startColumn, startLineNumber, startColumn + firstLine.length);
                }
                else {
                    // multi line insert
                    resultRange = new Range(startLineNumber, startColumn, startLineNumber + lineCount - 1, lastLine.length + 1);
                }
            }
            else {
                // There is nothing to insert
                resultRange = new Range(startLineNumber, startColumn, startLineNumber, startColumn);
            }
            prevOpEndLineNumber = resultRange.endLineNumber;
            prevOpEndColumn = resultRange.endColumn;
            result.push(resultRange);
            prevOp = op;
        }
        return result;
    };
    PieceTreeTextBuffer._sortOpsAscending = function (a, b) {
        var r = Range.compareRangesUsingEnds(a.range, b.range);
        if (r === 0) {
            return a.sortIndex - b.sortIndex;
        }
        return r;
    };
    PieceTreeTextBuffer._sortOpsDescending = function (a, b) {
        var r = Range.compareRangesUsingEnds(a.range, b.range);
        if (r === 0) {
            return b.sortIndex - a.sortIndex;
        }
        return -r;
    };
    return PieceTreeTextBuffer;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var PieceTreeTextBufferFactory = /** @class */ (function () {
    function PieceTreeTextBufferFactory(_chunks, _bom, _cr, _lf, _crlf, _containsRTL, _isBasicASCII, _normalizeEOL) {
        this._chunks = _chunks;
        this._bom = _bom;
        this._cr = _cr;
        this._lf = _lf;
        this._crlf = _crlf;
        this._containsRTL = _containsRTL;
        this._isBasicASCII = _isBasicASCII;
        this._normalizeEOL = _normalizeEOL;
    }
    PieceTreeTextBufferFactory.prototype._getEOL = function (defaultEOL) {
        var totalEOLCount = this._cr + this._lf + this._crlf;
        var totalCRCount = this._cr + this._crlf;
        if (totalEOLCount === 0) {
            // This is an empty file or a file with precisely one line
            return (defaultEOL === 1 /* LF */ ? '\n' : '\r\n');
        }
        if (totalCRCount > totalEOLCount / 2) {
            // More than half of the file contains \r\n ending lines
            return '\r\n';
        }
        // At least one line more ends in \n
        return '\n';
    };
    PieceTreeTextBufferFactory.prototype.create = function (defaultEOL) {
        var eol = this._getEOL(defaultEOL);
        var chunks = this._chunks;
        if (this._normalizeEOL &&
            ((eol === '\r\n' && (this._cr > 0 || this._lf > 0))
                || (eol === '\n' && (this._cr > 0 || this._crlf > 0)))) {
            // Normalize pieces
            for (var i = 0, len = chunks.length; i < len; i++) {
                var str = chunks[i].buffer.replace(/\r\n|\r|\n/g, eol);
                var newLineStart = createLineStartsFast(str);
                chunks[i] = new StringBuffer(str, newLineStart);
            }
        }
        return new PieceTreeTextBuffer(chunks, this._bom, eol, this._containsRTL, this._isBasicASCII, this._normalizeEOL);
    };
    return PieceTreeTextBufferFactory;
}());
var PieceTreeTextBufferBuilder = /** @class */ (function () {
    function PieceTreeTextBufferBuilder() {
        this.chunks = [];
        this.BOM = '';
        this._hasPreviousChar = false;
        this._previousChar = 0;
        this._tmpLineStarts = [];
        this.cr = 0;
        this.lf = 0;
        this.crlf = 0;
        this.containsRTL = false;
        this.isBasicASCII = true;
    }
    PieceTreeTextBufferBuilder.prototype.acceptChunk = function (chunk) {
        if (chunk.length === 0) {
            return;
        }
        if (this.chunks.length === 0) {
            if (startsWithUTF8BOM(chunk)) {
                this.BOM = UTF8_BOM_CHARACTER;
                chunk = chunk.substr(1);
            }
        }
        var lastChar = chunk.charCodeAt(chunk.length - 1);
        if (lastChar === 13 /* CarriageReturn */ || (lastChar >= 0xd800 && lastChar <= 0xdbff)) {
            // last character is \r or a high surrogate => keep it back
            this._acceptChunk1(chunk.substr(0, chunk.length - 1), false);
            this._hasPreviousChar = true;
            this._previousChar = lastChar;
        }
        else {
            this._acceptChunk1(chunk, false);
            this._hasPreviousChar = false;
            this._previousChar = lastChar;
        }
    };
    PieceTreeTextBufferBuilder.prototype._acceptChunk1 = function (chunk, allowEmptyStrings) {
        if (!allowEmptyStrings && chunk.length === 0) {
            // Nothing to do
            return;
        }
        if (this._hasPreviousChar) {
            this._acceptChunk2(String.fromCharCode(this._previousChar) + chunk);
        }
        else {
            this._acceptChunk2(chunk);
        }
    };
    PieceTreeTextBufferBuilder.prototype._acceptChunk2 = function (chunk) {
        var lineStarts = createLineStarts(this._tmpLineStarts, chunk);
        this.chunks.push(new StringBuffer(chunk, lineStarts.lineStarts));
        this.cr += lineStarts.cr;
        this.lf += lineStarts.lf;
        this.crlf += lineStarts.crlf;
        if (this.isBasicASCII) {
            this.isBasicASCII = lineStarts.isBasicASCII;
        }
        if (!this.isBasicASCII && !this.containsRTL) {
            // No need to check if is basic ASCII
            this.containsRTL = containsRTL(chunk);
        }
    };
    PieceTreeTextBufferBuilder.prototype.finish = function (normalizeEOL) {
        if (normalizeEOL === void 0) { normalizeEOL = true; }
        this._finish();
        return new PieceTreeTextBufferFactory(this.chunks, this.BOM, this.cr, this.lf, this.crlf, this.containsRTL, this.isBasicASCII, normalizeEOL);
    };
    PieceTreeTextBufferBuilder.prototype._finish = function () {
        if (this.chunks.length === 0) {
            this._acceptChunk1('', true);
        }
        if (this._hasPreviousChar) {
            this._hasPreviousChar = false;
            // recreate last chunk
            var lastChunk = this.chunks[this.chunks.length - 1];
            lastChunk.buffer += String.fromCharCode(this._previousChar);
            var newLineStarts = createLineStartsFast(lastChunk.buffer);
            lastChunk.lineStarts = newLineStarts;
            if (this._previousChar === 13 /* CarriageReturn */) {
                this.cr++;
            }
        }
    };
    return PieceTreeTextBufferBuilder;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * An event describing that a model has been reset to a new value.
 * @internal
 */
var ModelRawFlush = /** @class */ (function () {
    function ModelRawFlush() {
        this.changeType = 1 /* Flush */;
    }
    return ModelRawFlush;
}());
/**
 * An event describing that a line has changed in a model.
 * @internal
 */
var ModelRawLineChanged = /** @class */ (function () {
    function ModelRawLineChanged(lineNumber, detail) {
        this.changeType = 2 /* LineChanged */;
        this.lineNumber = lineNumber;
        this.detail = detail;
    }
    return ModelRawLineChanged;
}());
/**
 * An event describing that line(s) have been deleted in a model.
 * @internal
 */
var ModelRawLinesDeleted = /** @class */ (function () {
    function ModelRawLinesDeleted(fromLineNumber, toLineNumber) {
        this.changeType = 3 /* LinesDeleted */;
        this.fromLineNumber = fromLineNumber;
        this.toLineNumber = toLineNumber;
    }
    return ModelRawLinesDeleted;
}());
/**
 * An event describing that line(s) have been inserted in a model.
 * @internal
 */
var ModelRawLinesInserted = /** @class */ (function () {
    function ModelRawLinesInserted(fromLineNumber, toLineNumber, detail) {
        this.changeType = 4 /* LinesInserted */;
        this.fromLineNumber = fromLineNumber;
        this.toLineNumber = toLineNumber;
        this.detail = detail;
    }
    return ModelRawLinesInserted;
}());
/**
 * An event describing that a model has had its EOL changed.
 * @internal
 */
var ModelRawEOLChanged = /** @class */ (function () {
    function ModelRawEOLChanged() {
        this.changeType = 5 /* EOLChanged */;
    }
    return ModelRawEOLChanged;
}());
/**
 * An event describing a change in the text of a model.
 * @internal
 */
var ModelRawContentChangedEvent = /** @class */ (function () {
    function ModelRawContentChangedEvent(changes, versionId, isUndoing, isRedoing) {
        this.changes = changes;
        this.versionId = versionId;
        this.isUndoing = isUndoing;
        this.isRedoing = isRedoing;
    }
    ModelRawContentChangedEvent.prototype.containsEvent = function (type) {
        for (var i = 0, len = this.changes.length; i < len; i++) {
            var change = this.changes[i];
            if (change.changeType === type) {
                return true;
            }
        }
        return false;
    };
    ModelRawContentChangedEvent.merge = function (a, b) {
        var changes = [].concat(a.changes).concat(b.changes);
        var versionId = b.versionId;
        var isUndoing = (a.isUndoing || b.isUndoing);
        var isRedoing = (a.isRedoing || b.isRedoing);
        return new ModelRawContentChangedEvent(changes, versionId, isUndoing, isRedoing);
    };
    return ModelRawContentChangedEvent;
}());
/**
 * @internal
 */
var InternalModelContentChangeEvent = /** @class */ (function () {
    function InternalModelContentChangeEvent(rawContentChangedEvent, contentChangedEvent) {
        this.rawContentChangedEvent = rawContentChangedEvent;
        this.contentChangedEvent = contentChangedEvent;
    }
    InternalModelContentChangeEvent.prototype.merge = function (other) {
        var rawContentChangedEvent = ModelRawContentChangedEvent.merge(this.rawContentChangedEvent, other.rawContentChangedEvent);
        var contentChangedEvent = InternalModelContentChangeEvent._mergeChangeEvents(this.contentChangedEvent, other.contentChangedEvent);
        return new InternalModelContentChangeEvent(rawContentChangedEvent, contentChangedEvent);
    };
    InternalModelContentChangeEvent._mergeChangeEvents = function (a, b) {
        var changes = [].concat(a.changes).concat(b.changes);
        var eol = b.eol;
        var versionId = b.versionId;
        var isUndoing = (a.isUndoing || b.isUndoing);
        var isRedoing = (a.isRedoing || b.isRedoing);
        var isFlush = (a.isFlush || b.isFlush);
        return {
            changes: changes,
            eol: eol,
            versionId: versionId,
            isUndoing: isUndoing,
            isRedoing: isRedoing,
            isFlush: isFlush
        };
    };
    return InternalModelContentChangeEvent;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * The forward slash path separator.
 */
var sep = '/';
/**
 * The native path separator depending on the OS.
 */
var nativeSep = isWindows ? '\\' : '/';
/**
 * @param path the path to get the dirname from
 * @param separator the separator to use
 * @returns the directory name of a path.
 *
 */
function dirname(path, separator) {
    if (separator === void 0) { separator = nativeSep; }
    var idx = ~path.lastIndexOf('/') || ~path.lastIndexOf('\\');
    if (idx === 0) {
        return '.';
    }
    else if (~idx === 0) {
        return path[0];
    }
    else if (~idx === path.length - 1) {
        return dirname(path.substring(0, path.length - 1));
    }
    else {
        var res = path.substring(0, ~idx);
        if (isWindows && res[res.length - 1] === ':') {
            res += separator; // make sure drive letters end with backslash
        }
        return res;
    }
}
/**
 * @returns the base name of a path.
 */
function basename(path) {
    var idx = ~path.lastIndexOf('/') || ~path.lastIndexOf('\\');
    if (idx === 0) {
        return path;
    }
    else if (~idx === path.length - 1) {
        return basename(path.substring(0, path.length - 1));
    }
    else {
        return path.substr(~idx + 1);
    }
}
/**
 * @returns `.far` from `boo.far` or the empty string.
 */
function extname(path) {
    path = basename(path);
    var idx = ~path.lastIndexOf('.');
    return idx ? path.substring(~idx) : '';
}
var _posixBadPath = /(\/\.\.?\/)|(\/\.\.?)$|^(\.\.?\/)|(\/\/+)|(\\)/;
var _winBadPath = /(\\\.\.?\\)|(\\\.\.?)$|^(\.\.?\\)|(\\\\+)|(\/)/;
function _isNormal(path, win) {
    return win
        ? !_winBadPath.test(path)
        : !_posixBadPath.test(path);
}
function normalize(path, toOSPath) {
    if (path === null || path === void 0) {
        return path;
    }
    var len = path.length;
    if (len === 0) {
        return '.';
    }
    var wantsBackslash = !!(isWindows && toOSPath);
    if (_isNormal(path, wantsBackslash)) {
        return path;
    }
    var sep = wantsBackslash ? '\\' : '/';
    var root = getRoot(path, sep);
    // skip the root-portion of the path
    var start = root.length;
    var skip = false;
    var res = '';
    for (var end = root.length; end <= len; end++) {
        // either at the end or at a path-separator character
        if (end === len || path.charCodeAt(end) === 47 /* Slash */ || path.charCodeAt(end) === 92 /* Backslash */) {
            if (streql(path, start, end, '..')) {
                // skip current and remove parent (if there is already something)
                var prev_start = res.lastIndexOf(sep);
                var prev_part = res.slice(prev_start + 1);
                if ((root || prev_part.length > 0) && prev_part !== '..') {
                    res = prev_start === -1 ? '' : res.slice(0, prev_start);
                    skip = true;
                }
            }
            else if (streql(path, start, end, '.') && (root || res || end < len - 1)) {
                // skip current (if there is already something or if there is more to come)
                skip = true;
            }
            if (!skip) {
                var part = path.slice(start, end);
                if (res !== '' && res[res.length - 1] !== sep) {
                    res += sep;
                }
                res += part;
            }
            start = end + 1;
            skip = false;
        }
    }
    return root + res;
}
function streql(value, start, end, other) {
    return start + other.length === end && value.indexOf(other, start) === start;
}
/**
 * Computes the _root_ this path, like `getRoot('c:\files') === c:\`,
 * `getRoot('files:///files/path') === files:///`,
 * or `getRoot('\\server\shares\path') === \\server\shares\`
 */
function getRoot(path, sep) {
    if (sep === void 0) { sep = '/'; }
    if (!path) {
        return '';
    }
    var len = path.length;
    var code = path.charCodeAt(0);
    if (code === 47 /* Slash */ || code === 92 /* Backslash */) {
        code = path.charCodeAt(1);
        if (code === 47 /* Slash */ || code === 92 /* Backslash */) {
            // UNC candidate \\localhost\shares\ddd
            //               ^^^^^^^^^^^^^^^^^^^
            code = path.charCodeAt(2);
            if (code !== 47 /* Slash */ && code !== 92 /* Backslash */) {
                var pos_1 = 3;
                var start = pos_1;
                for (; pos_1 < len; pos_1++) {
                    code = path.charCodeAt(pos_1);
                    if (code === 47 /* Slash */ || code === 92 /* Backslash */) {
                        break;
                    }
                }
                code = path.charCodeAt(pos_1 + 1);
                if (start !== pos_1 && code !== 47 /* Slash */ && code !== 92 /* Backslash */) {
                    pos_1 += 1;
                    for (; pos_1 < len; pos_1++) {
                        code = path.charCodeAt(pos_1);
                        if (code === 47 /* Slash */ || code === 92 /* Backslash */) {
                            return path.slice(0, pos_1 + 1) // consume this separator
                                .replace(/[\\/]/g, sep);
                        }
                    }
                }
            }
        }
        // /user/far
        // ^
        return sep;
    }
    else if ((code >= 65 /* A */ && code <= 90 /* Z */) || (code >= 97 /* a */ && code <= 122 /* z */)) {
        // check for windows drive letter c:\ or c:
        if (path.charCodeAt(1) === 58 /* Colon */) {
            code = path.charCodeAt(2);
            if (code === 47 /* Slash */ || code === 92 /* Backslash */) {
                // C:\fff
                // ^^^
                return path.slice(0, 2) + sep;
            }
            else {
                // C:
                // ^^
                return path.slice(0, 2);
            }
        }
    }
    // check for URI
    // scheme://authority/path
    // ^^^^^^^^^^^^^^^^^^^
    var pos = path.indexOf('://');
    if (pos !== -1) {
        pos += 3; // 3 -> "://".length
        for (; pos < len; pos++) {
            code = path.charCodeAt(pos);
            if (code === 47 /* Slash */ || code === 92 /* Backslash */) {
                return path.slice(0, pos + 1); // consume this separator
            }
        }
    }
    return '';
}
var join = function () {
    // Not using a function with var-args because of how TS compiles
    // them to JS - it would result in 2*n runtime cost instead
    // of 1*n, where n is parts.length.
    var value = '';
    for (var i = 0; i < arguments.length; i++) {
        var part = arguments[i];
        if (i > 0) {
            // add the separater between two parts unless
            // there already is one
            var last = value.charCodeAt(value.length - 1);
            if (last !== 47 /* Slash */ && last !== 92 /* Backslash */) {
                var next = part.charCodeAt(0);
                if (next !== 47 /* Slash */ && next !== 92 /* Backslash */) {
                    value += sep;
                }
            }
        }
        value += part;
    }
    return normalize(value);
};
function isEqualOrParent(path, candidate, ignoreCase, separator) {
    if (separator === void 0) { separator = nativeSep; }
    if (path === candidate) {
        return true;
    }
    if (!path || !candidate) {
        return false;
    }
    if (candidate.length > path.length) {
        return false;
    }
    if (ignoreCase) {
        var beginsWith = startsWithIgnoreCase(path, candidate);
        if (!beginsWith) {
            return false;
        }
        if (candidate.length === path.length) {
            return true; // same path, different casing
        }
        var sepOffset = candidate.length;
        if (candidate.charAt(candidate.length - 1) === separator) {
            sepOffset--; // adjust the expected sep offset in case our candidate already ends in separator character
        }
        return path.charAt(sepOffset) === separator;
    }
    if (candidate.charAt(candidate.length - 1) !== separator) {
        candidate += separator;
    }
    return path.indexOf(candidate) === 0;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$9 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function values(forEachable) {
    var result = [];
    forEachable.forEach(function (value) { return result.push(value); });
    return result;
}
function keys(map) {
    var result = [];
    map.forEach(function (value, key) { return result.push(key); });
    return result;
}
var StringIterator = /** @class */ (function () {
    function StringIterator() {
        this._value = '';
        this._pos = 0;
    }
    StringIterator.prototype.reset = function (key) {
        this._value = key;
        this._pos = 0;
        return this;
    };
    StringIterator.prototype.next = function () {
        this._pos += 1;
        return this;
    };
    StringIterator.prototype.hasNext = function () {
        return this._pos < this._value.length - 1;
    };
    StringIterator.prototype.cmp = function (a) {
        var aCode = a.charCodeAt(0);
        var thisCode = this._value.charCodeAt(this._pos);
        return aCode - thisCode;
    };
    StringIterator.prototype.value = function () {
        return this._value[this._pos];
    };
    return StringIterator;
}());
var PathIterator = /** @class */ (function () {
    function PathIterator() {
    }
    PathIterator.prototype.reset = function (key) {
        this._value = key.replace(/\\$|\/$/, '');
        this._from = 0;
        this._to = 0;
        return this.next();
    };
    PathIterator.prototype.hasNext = function () {
        return this._to < this._value.length;
    };
    PathIterator.prototype.next = function () {
        // this._data = key.split(/[\\/]/).filter(s => !!s);
        this._from = this._to;
        var justSeps = true;
        for (; this._to < this._value.length; this._to++) {
            var ch = this._value.charCodeAt(this._to);
            if (ch === 47 /* Slash */ || ch === 92 /* Backslash */) {
                if (justSeps) {
                    this._from++;
                }
                else {
                    break;
                }
            }
            else {
                justSeps = false;
            }
        }
        return this;
    };
    PathIterator.prototype.cmp = function (a) {
        var aPos = 0;
        var aLen = a.length;
        var thisPos = this._from;
        while (aPos < aLen && thisPos < this._to) {
            var cmp = a.charCodeAt(aPos) - this._value.charCodeAt(thisPos);
            if (cmp !== 0) {
                return cmp;
            }
            aPos += 1;
            thisPos += 1;
        }
        if (aLen === this._to - this._from) {
            return 0;
        }
        else if (aPos < aLen) {
            return -1;
        }
        else {
            return 1;
        }
    };
    PathIterator.prototype.value = function () {
        return this._value.substring(this._from, this._to);
    };
    return PathIterator;
}());
var TernarySearchTreeNode = /** @class */ (function () {
    function TernarySearchTreeNode() {
    }
    return TernarySearchTreeNode;
}());
var TernarySearchTree = /** @class */ (function () {
    function TernarySearchTree(segments) {
        this._iter = segments;
    }
    TernarySearchTree.forPaths = function () {
        return new TernarySearchTree(new PathIterator());
    };
    TernarySearchTree.forStrings = function () {
        return new TernarySearchTree(new StringIterator());
    };
    TernarySearchTree.prototype.clear = function () {
        this._root = undefined;
    };
    TernarySearchTree.prototype.set = function (key, element) {
        var iter = this._iter.reset(key);
        var node;
        if (!this._root) {
            this._root = new TernarySearchTreeNode();
            this._root.segment = iter.value();
        }
        node = this._root;
        while (true) {
            var val = iter.cmp(node.segment);
            if (val > 0) {
                // left
                if (!node.left) {
                    node.left = new TernarySearchTreeNode();
                    node.left.segment = iter.value();
                }
                node = node.left;
            }
            else if (val < 0) {
                // right
                if (!node.right) {
                    node.right = new TernarySearchTreeNode();
                    node.right.segment = iter.value();
                }
                node = node.right;
            }
            else if (iter.hasNext()) {
                // mid
                iter.next();
                if (!node.mid) {
                    node.mid = new TernarySearchTreeNode();
                    node.mid.segment = iter.value();
                }
                node = node.mid;
            }
            else {
                break;
            }
        }
        var oldElement = node.value;
        node.value = element;
        node.key = key;
        return oldElement;
    };
    TernarySearchTree.prototype.get = function (key) {
        var iter = this._iter.reset(key);
        var node = this._root;
        while (node) {
            var val = iter.cmp(node.segment);
            if (val > 0) {
                // left
                node = node.left;
            }
            else if (val < 0) {
                // right
                node = node.right;
            }
            else if (iter.hasNext()) {
                // mid
                iter.next();
                node = node.mid;
            }
            else {
                break;
            }
        }
        return node ? node.value : undefined;
    };
    TernarySearchTree.prototype.findSubstr = function (key) {
        var iter = this._iter.reset(key);
        var node = this._root;
        var candidate = undefined;
        while (node) {
            var val = iter.cmp(node.segment);
            if (val > 0) {
                // left
                node = node.left;
            }
            else if (val < 0) {
                // right
                node = node.right;
            }
            else if (iter.hasNext()) {
                // mid
                iter.next();
                candidate = node.value || candidate;
                node = node.mid;
            }
            else {
                break;
            }
        }
        return node && node.value || candidate;
    };
    TernarySearchTree.prototype.forEach = function (callback) {
        this._forEach(this._root, callback);
    };
    TernarySearchTree.prototype._forEach = function (node, callback) {
        if (node) {
            // left
            this._forEach(node.left, callback);
            // node
            if (node.value) {
                // callback(node.value, this._iter.join(parts));
                callback(node.value, node.key);
            }
            // mid
            this._forEach(node.mid, callback);
            // right
            this._forEach(node.right, callback);
        }
    };
    return TernarySearchTree;
}());
var ResourceMap = /** @class */ (function () {
    function ResourceMap() {
        this.map = new Map();
        this.ignoreCase = false; // in the future this should be an uri-comparator
    }
    ResourceMap.prototype.set = function (resource, value) {
        this.map.set(this.toKey(resource), value);
    };
    ResourceMap.prototype.get = function (resource) {
        return this.map.get(this.toKey(resource));
    };
    ResourceMap.prototype.toKey = function (resource) {
        var key = resource.toString();
        if (this.ignoreCase) {
            key = key.toLowerCase();
        }
        return key;
    };
    return ResourceMap;
}());
var LinkedMap = /** @class */ (function () {
    function LinkedMap() {
        this._map = new Map();
        this._head = undefined;
        this._tail = undefined;
        this._size = 0;
    }
    LinkedMap.prototype.clear = function () {
        this._map.clear();
        this._head = undefined;
        this._tail = undefined;
        this._size = 0;
    };
    Object.defineProperty(LinkedMap.prototype, "size", {
        get: function () {
            return this._size;
        },
        enumerable: true,
        configurable: true
    });
    LinkedMap.prototype.get = function (key, touch) {
        if (touch === void 0) { touch = 0 /* None */; }
        var item = this._map.get(key);
        if (!item) {
            return undefined;
        }
        if (touch !== 0 /* None */) {
            this.touch(item, touch);
        }
        return item.value;
    };
    LinkedMap.prototype.set = function (key, value, touch) {
        if (touch === void 0) { touch = 0 /* None */; }
        var item = this._map.get(key);
        if (item) {
            item.value = value;
            if (touch !== 0 /* None */) {
                this.touch(item, touch);
            }
        }
        else {
            item = { key: key, value: value, next: undefined, previous: undefined };
            switch (touch) {
                case 0 /* None */:
                    this.addItemLast(item);
                    break;
                case 1 /* AsOld */:
                    this.addItemFirst(item);
                    break;
                case 2 /* AsNew */:
                    this.addItemLast(item);
                    break;
                default:
                    this.addItemLast(item);
                    break;
            }
            this._map.set(key, item);
            this._size++;
        }
    };
    LinkedMap.prototype.forEach = function (callbackfn, thisArg) {
        var current = this._head;
        while (current) {
            if (thisArg) {
                callbackfn.bind(thisArg)(current.value, current.key, this);
            }
            else {
                callbackfn(current.value, current.key, this);
            }
            current = current.next;
        }
    };
    /* VS Code / Monaco editor runs on es5 which has no Symbol.iterator
    public keys(): IterableIterator<K> {
        let current = this._head;
        let iterator: IterableIterator<K> = {
            [Symbol.iterator]() {
                return iterator;
            },
            next():IteratorResult<K> {
                if (current) {
                    let result = { value: current.key, done: false };
                    current = current.next;
                    return result;
                } else {
                    return { value: undefined, done: true };
                }
            }
        };
        return iterator;
    }

    public values(): IterableIterator<V> {
        let current = this._head;
        let iterator: IterableIterator<V> = {
            [Symbol.iterator]() {
                return iterator;
            },
            next():IteratorResult<V> {
                if (current) {
                    let result = { value: current.value, done: false };
                    current = current.next;
                    return result;
                } else {
                    return { value: undefined, done: true };
                }
            }
        };
        return iterator;
    }
    */
    LinkedMap.prototype.trimOld = function (newSize) {
        if (newSize >= this.size) {
            return;
        }
        if (newSize === 0) {
            this.clear();
            return;
        }
        var current = this._head;
        var currentSize = this.size;
        while (current && currentSize > newSize) {
            this._map.delete(current.key);
            current = current.next;
            currentSize--;
        }
        this._head = current;
        this._size = currentSize;
        if (current) {
            current.previous = void 0;
        }
    };
    LinkedMap.prototype.addItemFirst = function (item) {
        // First time Insert
        if (!this._head && !this._tail) {
            this._tail = item;
        }
        else if (!this._head) {
            throw new Error('Invalid list');
        }
        else {
            item.next = this._head;
            this._head.previous = item;
        }
        this._head = item;
    };
    LinkedMap.prototype.addItemLast = function (item) {
        // First time Insert
        if (!this._head && !this._tail) {
            this._head = item;
        }
        else if (!this._tail) {
            throw new Error('Invalid list');
        }
        else {
            item.previous = this._tail;
            this._tail.next = item;
        }
        this._tail = item;
    };
    LinkedMap.prototype.touch = function (item, touch) {
        if (!this._head || !this._tail) {
            throw new Error('Invalid list');
        }
        if ((touch !== 1 /* AsOld */ && touch !== 2 /* AsNew */)) {
            return;
        }
        if (touch === 1 /* AsOld */) {
            if (item === this._head) {
                return;
            }
            var next = item.next;
            var previous = item.previous;
            // Unlink the item
            if (item === this._tail) {
                // previous must be defined since item was not head but is tail
                // So there are more than on item in the map
                previous.next = void 0;
                this._tail = previous;
            }
            else {
                // Both next and previous are not undefined since item was neither head nor tail.
                next.previous = previous;
                previous.next = next;
            }
            // Insert the node at head
            item.previous = void 0;
            item.next = this._head;
            this._head.previous = item;
            this._head = item;
        }
        else if (touch === 2 /* AsNew */) {
            if (item === this._tail) {
                return;
            }
            var next = item.next;
            var previous = item.previous;
            // Unlink the item.
            if (item === this._head) {
                // next must be defined since item was not tail but is head
                // So there are more than on item in the map
                next.previous = void 0;
                this._head = next;
            }
            else {
                // Both next and previous are not undefined since item was neither head nor tail.
                next.previous = previous;
                previous.next = next;
            }
            item.next = void 0;
            item.previous = this._tail;
            this._tail.next = item;
            this._tail = item;
        }
    };
    LinkedMap.prototype.toJSON = function () {
        var data = [];
        this.forEach(function (value, key) {
            data.push([key, value]);
        });
        return data;
    };
    return LinkedMap;
}());
var LRUCache = /** @class */ (function (_super) {
    __extends$9(LRUCache, _super);
    function LRUCache(limit, ratio) {
        if (ratio === void 0) { ratio = 1; }
        var _this = _super.call(this) || this;
        _this._limit = limit;
        _this._ratio = Math.min(Math.max(0, ratio), 1);
        return _this;
    }
    LRUCache.prototype.get = function (key) {
        return _super.prototype.get.call(this, key, 2 /* AsNew */);
    };
    LRUCache.prototype.set = function (key, value) {
        _super.prototype.set.call(this, key, value, 2 /* AsNew */);
        this.checkTrim();
    };
    LRUCache.prototype.checkTrim = function () {
        if (this.size > this._limit) {
            this.trimOld(Math.round(this._limit * this._ratio));
        }
    };
    return LRUCache;
}(LinkedMap));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var GLOBSTAR = '**';
var GLOB_SPLIT = '/';
var PATH_REGEX = '[/\\\\]'; // any slash or backslash
var NO_PATH_REGEX = '[^/\\\\]'; // any non-slash and non-backslash
var ALL_FORWARD_SLASHES = /\//g;
function starsToRegExp(starCount) {
    switch (starCount) {
        case 0:
            return '';
        case 1:
            return NO_PATH_REGEX + "*?"; // 1 star matches any number of characters except path separator (/ and \) - non greedy (?)
        default:
            // Matches:  (Path Sep OR Path Val followed by Path Sep OR Path Sep followed by Path Val) 0-many times
            // Group is non capturing because we don't need to capture at all (?:...)
            // Overall we use non-greedy matching because it could be that we match too much
            return "(?:" + PATH_REGEX + "|" + NO_PATH_REGEX + "+" + PATH_REGEX + "|" + PATH_REGEX + NO_PATH_REGEX + "+)*?";
    }
}
function splitGlobAware(pattern, splitChar) {
    if (!pattern) {
        return [];
    }
    var segments = [];
    var inBraces = false;
    var inBrackets = false;
    var char;
    var curVal = '';
    for (var i = 0; i < pattern.length; i++) {
        char = pattern[i];
        switch (char) {
            case splitChar:
                if (!inBraces && !inBrackets) {
                    segments.push(curVal);
                    curVal = '';
                    continue;
                }
                break;
            case '{':
                inBraces = true;
                break;
            case '}':
                inBraces = false;
                break;
            case '[':
                inBrackets = true;
                break;
            case ']':
                inBrackets = false;
                break;
        }
        curVal += char;
    }
    // Tail
    if (curVal) {
        segments.push(curVal);
    }
    return segments;
}
function parseRegExp(pattern) {
    if (!pattern) {
        return '';
    }
    var regEx = '';
    // Split up into segments for each slash found
    var segments = splitGlobAware(pattern, GLOB_SPLIT);
    // Special case where we only have globstars
    if (segments.every(function (s) { return s === GLOBSTAR; })) {
        regEx = '.*';
    }
    // Build regex over segments
    else {
        var previousSegmentWasGlobStar_1 = false;
        segments.forEach(function (segment, index) {
            // Globstar is special
            if (segment === GLOBSTAR) {
                // if we have more than one globstar after another, just ignore it
                if (!previousSegmentWasGlobStar_1) {
                    regEx += starsToRegExp(2);
                    previousSegmentWasGlobStar_1 = true;
                }
                return;
            }
            // States
            var inBraces = false;
            var braceVal = '';
            var inBrackets = false;
            var bracketVal = '';
            var char;
            for (var i = 0; i < segment.length; i++) {
                char = segment[i];
                // Support brace expansion
                if (char !== '}' && inBraces) {
                    braceVal += char;
                    continue;
                }
                // Support brackets
                if (inBrackets && (char !== ']' || !bracketVal) /* ] is literally only allowed as first character in brackets to match it */) {
                    var res = void 0;
                    // range operator
                    if (char === '-') {
                        res = char;
                    }
                    // negation operator (only valid on first index in bracket)
                    else if ((char === '^' || char === '!') && !bracketVal) {
                        res = '^';
                    }
                    // glob split matching is not allowed within character ranges
                    // see http://man7.org/linux/man-pages/man7/glob.7.html
                    else if (char === GLOB_SPLIT) {
                        res = '';
                    }
                    // anything else gets escaped
                    else {
                        res = escapeRegExpCharacters(char);
                    }
                    bracketVal += res;
                    continue;
                }
                switch (char) {
                    case '{':
                        inBraces = true;
                        continue;
                    case '[':
                        inBrackets = true;
                        continue;
                    case '}':
                        var choices = splitGlobAware(braceVal, ',');
                        // Converts {foo,bar} => [foo|bar]
                        var braceRegExp = "(?:" + choices.map(function (c) { return parseRegExp(c); }).join('|') + ")";
                        regEx += braceRegExp;
                        inBraces = false;
                        braceVal = '';
                        break;
                    case ']':
                        regEx += ('[' + bracketVal + ']');
                        inBrackets = false;
                        bracketVal = '';
                        break;
                    case '?':
                        regEx += NO_PATH_REGEX; // 1 ? matches any single character except path separator (/ and \)
                        continue;
                    case '*':
                        regEx += starsToRegExp(1);
                        continue;
                    default:
                        regEx += escapeRegExpCharacters(char);
                }
            }
            // Tail: Add the slash we had split on if there is more to come and the remaining pattern is not a globstar
            // For example if pattern: some/**/*.js we want the "/" after some to be included in the RegEx to prevent
            // a folder called "something" to match as well.
            // However, if pattern: some/**, we tolerate that we also match on "something" because our globstar behaviour
            // is to match 0-N segments.
            if (index < segments.length - 1 && (segments[index + 1] !== GLOBSTAR || index + 2 < segments.length)) {
                regEx += PATH_REGEX;
            }
            // reset state
            previousSegmentWasGlobStar_1 = false;
        });
    }
    return regEx;
}
// regexes to check for trival glob patterns that just check for String#endsWith
var T1 = /^\*\*\/\*\.[\w\.-]+$/; // **/*.something
var T2 = /^\*\*\/([\w\.-]+)\/?$/; // **/something
var T3 = /^{\*\*\/[\*\.]?[\w\.-]+\/?(,\*\*\/[\*\.]?[\w\.-]+\/?)*}$/; // {**/*.something,**/*.else} or {**/package.json,**/project.json}
var T3_2 = /^{\*\*\/[\*\.]?[\w\.-]+(\/(\*\*)?)?(,\*\*\/[\*\.]?[\w\.-]+(\/(\*\*)?)?)*}$/; // Like T3, with optional trailing /**
var T4 = /^\*\*((\/[\w\.-]+)+)\/?$/; // **/something/else
var T5 = /^([\w\.-]+(\/[\w\.-]+)*)\/?$/; // something/else
var CACHE = new LRUCache(10000); // bounded to 10000 elements
var FALSE = function () {
    return false;
};
var NULL = function () {
    return null;
};
function parsePattern(arg1, options) {
    if (!arg1) {
        return NULL;
    }
    // Handle IRelativePattern
    var pattern;
    if (typeof arg1 !== 'string') {
        pattern = arg1.pattern;
    }
    else {
        pattern = arg1;
    }
    // Whitespace trimming
    pattern = pattern.trim();
    // Check cache
    var patternKey = pattern + "_" + !!options.trimForExclusions;
    var parsedPattern = CACHE.get(patternKey);
    if (parsedPattern) {
        return wrapRelativePattern(parsedPattern, arg1);
    }
    // Check for Trivias
    var match;
    if (T1.test(pattern)) { // common pattern: **/*.txt just need endsWith check
        var base_1 = pattern.substr(4); // '**/*'.length === 4
        parsedPattern = function (path, basename$$1) {
            return path && endsWith(path, base_1) ? pattern : null;
        };
    }
    else if (match = T2.exec(trimForExclusions(pattern, options))) { // common pattern: **/some.txt just need basename check
        parsedPattern = trivia2(match[1], pattern);
    }
    else if ((options.trimForExclusions ? T3_2 : T3).test(pattern)) { // repetition of common patterns (see above) {**/*.txt,**/*.png}
        parsedPattern = trivia3(pattern, options);
    }
    else if (match = T4.exec(trimForExclusions(pattern, options))) { // common pattern: **/something/else just need endsWith check
        parsedPattern = trivia4and5(match[1].substr(1), pattern, true);
    }
    else if (match = T5.exec(trimForExclusions(pattern, options))) { // common pattern: something/else just need equals check
        parsedPattern = trivia4and5(match[1], pattern, false);
    }
    // Otherwise convert to pattern
    else {
        parsedPattern = toRegExp(pattern);
    }
    // Cache
    CACHE.set(patternKey, parsedPattern);
    return wrapRelativePattern(parsedPattern, arg1);
}
function wrapRelativePattern(parsedPattern, arg2) {
    if (typeof arg2 === 'string') {
        return parsedPattern;
    }
    return function (path, basename$$1) {
        if (!isEqualOrParent(path, arg2.base)) {
            return null;
        }
        return parsedPattern(arg2.pathToRelative(arg2.base, path), basename$$1);
    };
}
function trimForExclusions(pattern, options) {
    return options.trimForExclusions && endsWith(pattern, '/**') ? pattern.substr(0, pattern.length - 2) : pattern; // dropping **, tailing / is dropped later
}
// common pattern: **/some.txt just need basename check
function trivia2(base, originalPattern) {
    var slashBase = "/" + base;
    var backslashBase = "\\" + base;
    var parsedPattern = function (path, basename$$1) {
        if (!path) {
            return null;
        }
        if (basename$$1) {
            return basename$$1 === base ? originalPattern : null;
        }
        return path === base || endsWith(path, slashBase) || endsWith(path, backslashBase) ? originalPattern : null;
    };
    var basenames = [base];
    parsedPattern.basenames = basenames;
    parsedPattern.patterns = [originalPattern];
    parsedPattern.allBasenames = basenames;
    return parsedPattern;
}
// repetition of common patterns (see above) {**/*.txt,**/*.png}
function trivia3(pattern, options) {
    var parsedPatterns = aggregateBasenameMatches(pattern.slice(1, -1).split(',')
        .map(function (pattern) { return parsePattern(pattern, options); })
        .filter(function (pattern) { return pattern !== NULL; }), pattern);
    var n = parsedPatterns.length;
    if (!n) {
        return NULL;
    }
    if (n === 1) {
        return parsedPatterns[0];
    }
    var parsedPattern = function (path, basename$$1) {
        for (var i = 0, n_1 = parsedPatterns.length; i < n_1; i++) {
            if (parsedPatterns[i](path, basename$$1)) {
                return pattern;
            }
        }
        return null;
    };
    var withBasenames = first$1(parsedPatterns, function (pattern) { return !!pattern.allBasenames; });
    if (withBasenames) {
        parsedPattern.allBasenames = withBasenames.allBasenames;
    }
    var allPaths = parsedPatterns.reduce(function (all, current) { return current.allPaths ? all.concat(current.allPaths) : all; }, []);
    if (allPaths.length) {
        parsedPattern.allPaths = allPaths;
    }
    return parsedPattern;
}
// common patterns: **/something/else just need endsWith check, something/else just needs and equals check
function trivia4and5(path, pattern, matchPathEnds) {
    var nativePath = nativeSep !== sep ? path.replace(ALL_FORWARD_SLASHES, nativeSep) : path;
    var nativePathEnd = nativeSep + nativePath;
    var parsedPattern = matchPathEnds ? function (path, basename$$1) {
        return path && (path === nativePath || endsWith(path, nativePathEnd)) ? pattern : null;
    } : function (path, basename$$1) {
        return path && path === nativePath ? pattern : null;
    };
    parsedPattern.allPaths = [(matchPathEnds ? '*/' : './') + path];
    return parsedPattern;
}
function toRegExp(pattern) {
    try {
        var regExp_1 = new RegExp("^" + parseRegExp(pattern) + "$");
        return function (path, basename$$1) {
            regExp_1.lastIndex = 0; // reset RegExp to its initial state to reuse it!
            return path && regExp_1.test(path) ? pattern : null;
        };
    }
    catch (error) {
        return NULL;
    }
}
function match(arg1, path, hasSibling) {
    if (!arg1 || !path) {
        return false;
    }
    return parse(arg1)(path, undefined, hasSibling);
}
function parse(arg1, options) {
    if (options === void 0) { options = {}; }
    if (!arg1) {
        return FALSE;
    }
    // Glob with String
    if (typeof arg1 === 'string' || isRelativePattern(arg1)) {
        var parsedPattern_1 = parsePattern(arg1, options);
        if (parsedPattern_1 === NULL) {
            return FALSE;
        }
        var resultPattern = function (path, basename$$1) {
            return !!parsedPattern_1(path, basename$$1);
        };
        if (parsedPattern_1.allBasenames) {
            resultPattern.allBasenames = parsedPattern_1.allBasenames;
        }
        if (parsedPattern_1.allPaths) {
            resultPattern.allPaths = parsedPattern_1.allPaths;
        }
        return resultPattern;
    }
    // Glob with Expression
    return parsedExpression(arg1, options);
}
function isRelativePattern(obj) {
    var rp = obj;
    return rp && typeof rp.base === 'string' && typeof rp.pattern === 'string' && typeof rp.pathToRelative === 'function';
}
function parsedExpression(expression, options) {
    var parsedPatterns = aggregateBasenameMatches(Object.getOwnPropertyNames(expression)
        .map(function (pattern) { return parseExpressionPattern(pattern, expression[pattern], options); })
        .filter(function (pattern) { return pattern !== NULL; }));
    var n = parsedPatterns.length;
    if (!n) {
        return NULL;
    }
    if (!parsedPatterns.some(function (parsedPattern) { return !!parsedPattern.requiresSiblings; })) {
        if (n === 1) {
            return parsedPatterns[0];
        }
        var resultExpression_1 = function (path, basename$$1) {
            for (var i = 0, n_2 = parsedPatterns.length; i < n_2; i++) {
                // Pattern matches path
                var result = parsedPatterns[i](path, basename$$1);
                if (result) {
                    return result;
                }
            }
            return null;
        };
        var withBasenames_1 = first$1(parsedPatterns, function (pattern) { return !!pattern.allBasenames; });
        if (withBasenames_1) {
            resultExpression_1.allBasenames = withBasenames_1.allBasenames;
        }
        var allPaths_1 = parsedPatterns.reduce(function (all, current) { return current.allPaths ? all.concat(current.allPaths) : all; }, []);
        if (allPaths_1.length) {
            resultExpression_1.allPaths = allPaths_1;
        }
        return resultExpression_1;
    }
    var resultExpression = function (path, basename$$1, hasSibling) {
        var name = undefined;
        for (var i = 0, n_3 = parsedPatterns.length; i < n_3; i++) {
            // Pattern matches path
            var parsedPattern = parsedPatterns[i];
            if (parsedPattern.requiresSiblings && hasSibling) {
                if (!basename$$1) {
                    basename$$1 = basename(path);
                }
                if (!name) {
                    name = basename$$1.substr(0, basename$$1.length - extname(path).length);
                }
            }
            var result = parsedPattern(path, basename$$1, name, hasSibling);
            if (result) {
                return result;
            }
        }
        return null;
    };
    var withBasenames = first$1(parsedPatterns, function (pattern) { return !!pattern.allBasenames; });
    if (withBasenames) {
        resultExpression.allBasenames = withBasenames.allBasenames;
    }
    var allPaths = parsedPatterns.reduce(function (all, current) { return current.allPaths ? all.concat(current.allPaths) : all; }, []);
    if (allPaths.length) {
        resultExpression.allPaths = allPaths;
    }
    return resultExpression;
}
function parseExpressionPattern(pattern, value, options) {
    if (value === false) {
        return NULL; // pattern is disabled
    }
    var parsedPattern = parsePattern(pattern, options);
    if (parsedPattern === NULL) {
        return NULL;
    }
    // Expression Pattern is <boolean>
    if (typeof value === 'boolean') {
        return parsedPattern;
    }
    // Expression Pattern is <SiblingClause>
    if (value) {
        var when_1 = value.when;
        if (typeof when_1 === 'string') {
            var result = function (path, basename$$1, name, hasSibling) {
                if (!hasSibling || !parsedPattern(path, basename$$1)) {
                    return null;
                }
                var clausePattern = when_1.replace('$(basename)', name);
                var matched = hasSibling(clausePattern);
                return isThenable(matched) ?
                    matched.then(function (m) { return m ? pattern : null; }) :
                    matched ? pattern : null;
            };
            result.requiresSiblings = true;
            return result;
        }
    }
    // Expression is Anything
    return parsedPattern;
}
function aggregateBasenameMatches(parsedPatterns, result) {
    var basenamePatterns = parsedPatterns.filter(function (parsedPattern) { return !!parsedPattern.basenames; });
    if (basenamePatterns.length < 2) {
        return parsedPatterns;
    }
    var basenames = basenamePatterns.reduce(function (all, current) {
        var basenames = current.basenames;
        return basenames ? all.concat(basenames) : all;
    }, []);
    var patterns;
    if (result) {
        patterns = [];
        for (var i = 0, n = basenames.length; i < n; i++) {
            patterns.push(result);
        }
    }
    else {
        patterns = basenamePatterns.reduce(function (all, current) {
            var patterns = current.patterns;
            return patterns ? all.concat(patterns) : all;
        }, []);
    }
    var aggregate = function (path, basename$$1) {
        if (!path) {
            return null;
        }
        if (!basename$$1) {
            var i = void 0;
            for (i = path.length; i > 0; i--) {
                var ch = path.charCodeAt(i - 1);
                if (ch === 47 /* Slash */ || ch === 92 /* Backslash */) {
                    break;
                }
            }
            basename$$1 = path.substr(i);
        }
        var index = basenames.indexOf(basename$$1);
        return index !== -1 ? patterns[index] : null;
    };
    aggregate.basenames = basenames;
    aggregate.patterns = patterns;
    aggregate.allBasenames = basenames;
    var aggregatedPatterns = parsedPatterns.filter(function (parsedPattern) { return !parsedPattern.basenames; });
    aggregatedPatterns.push(aggregate);
    return aggregatedPatterns;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function score(selector, candidateUri, candidateLanguage, candidateIsSynchronized) {
    if (Array.isArray(selector)) {
        // array -> take max individual value
        var ret = 0;
        for (var _i = 0, selector_1 = selector; _i < selector_1.length; _i++) {
            var filter = selector_1[_i];
            var value = score(filter, candidateUri, candidateLanguage, candidateIsSynchronized);
            if (value === 10) {
                return value; // already at the highest
            }
            if (value > ret) {
                ret = value;
            }
        }
        return ret;
    }
    else if (typeof selector === 'string') {
        if (!candidateIsSynchronized) {
            return 0;
        }
        // short-hand notion, desugars to
        // 'fooLang' -> { language: 'fooLang'}
        // '*' -> { language: '*' }
        if (selector === '*') {
            return 5;
        }
        else if (selector === candidateLanguage) {
            return 10;
        }
        else {
            return 0;
        }
    }
    else if (selector) {
        // filter -> select accordingly, use defaults for scheme
        var language = selector.language, pattern = selector.pattern, scheme = selector.scheme, hasAccessToAllModels = selector.hasAccessToAllModels;
        if (!candidateIsSynchronized && !hasAccessToAllModels) {
            return 0;
        }
        var ret = 0;
        if (scheme) {
            if (scheme === candidateUri.scheme) {
                ret = 10;
            }
            else if (scheme === '*') {
                ret = 5;
            }
            else {
                return 0;
            }
        }
        if (language) {
            if (language === candidateLanguage) {
                ret = 10;
            }
            else if (language === '*') {
                ret = Math.max(ret, 5);
            }
            else {
                return 0;
            }
        }
        if (pattern) {
            if (pattern === candidateUri.fsPath || match(pattern, candidateUri.fsPath)) {
                ret = 10;
            }
            else {
                return 0;
            }
        }
        return ret;
    }
    else {
        return 0;
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function isExclusive(selector) {
    if (typeof selector === 'string') {
        return false;
    }
    else if (Array.isArray(selector)) {
        return selector.every(isExclusive);
    }
    else {
        return !!selector.exclusive;
    }
}
var LanguageFeatureRegistry = /** @class */ (function () {
    function LanguageFeatureRegistry() {
        this._clock = 0;
        this._entries = [];
        this._onDidChange = new Emitter();
    }
    Object.defineProperty(LanguageFeatureRegistry.prototype, "onDidChange", {
        get: function () {
            return this._onDidChange.event;
        },
        enumerable: true,
        configurable: true
    });
    LanguageFeatureRegistry.prototype.register = function (selector, provider) {
        var _this = this;
        var entry = {
            selector: selector,
            provider: provider,
            _score: -1,
            _time: this._clock++
        };
        this._entries.push(entry);
        this._lastCandidate = undefined;
        this._onDidChange.fire(this._entries.length);
        return toDisposable(function () {
            if (entry) {
                var idx = _this._entries.indexOf(entry);
                if (idx >= 0) {
                    _this._entries.splice(idx, 1);
                    _this._lastCandidate = undefined;
                    _this._onDidChange.fire(_this._entries.length);
                    entry = undefined;
                }
            }
        });
    };
    LanguageFeatureRegistry.prototype.has = function (model) {
        return this.all(model).length > 0;
    };
    LanguageFeatureRegistry.prototype.all = function (model) {
        if (!model) {
            return [];
        }
        this._updateScores(model);
        var result = [];
        // from registry
        for (var _i = 0, _a = this._entries; _i < _a.length; _i++) {
            var entry = _a[_i];
            if (entry._score > 0) {
                result.push(entry.provider);
            }
        }
        return result;
    };
    LanguageFeatureRegistry.prototype.ordered = function (model) {
        var result = [];
        this._orderedForEach(model, function (entry) { return result.push(entry.provider); });
        return result;
    };
    LanguageFeatureRegistry.prototype.orderedGroups = function (model) {
        var result = [];
        var lastBucket;
        var lastBucketScore;
        this._orderedForEach(model, function (entry) {
            if (lastBucket && lastBucketScore === entry._score) {
                lastBucket.push(entry.provider);
            }
            else {
                lastBucketScore = entry._score;
                lastBucket = [entry.provider];
                result.push(lastBucket);
            }
        });
        return result;
    };
    LanguageFeatureRegistry.prototype._orderedForEach = function (model, callback) {
        if (!model) {
            return;
        }
        this._updateScores(model);
        for (var from = 0; from < this._entries.length; from++) {
            var entry = this._entries[from];
            if (entry._score > 0) {
                callback(entry);
            }
        }
    };
    LanguageFeatureRegistry.prototype._updateScores = function (model) {
        var candidate = {
            uri: model.uri.toString(),
            language: model.getLanguageIdentifier().language
        };
        if (this._lastCandidate
            && this._lastCandidate.language === candidate.language
            && this._lastCandidate.uri === candidate.uri) {
            // nothing has changed
            return;
        }
        this._lastCandidate = candidate;
        for (var _i = 0, _a = this._entries; _i < _a.length; _i++) {
            var entry = _a[_i];
            entry._score = score(entry.selector, model.uri, model.getLanguageIdentifier().language, shouldSynchronizeModel(model));
            if (isExclusive(entry.selector) && entry._score > 0) {
                // support for one exclusive selector that overwrites
                // any other selector
                for (var _b = 0, _c = this._entries; _b < _c.length; _b++) {
                    var entry_1 = _c[_b];
                    entry_1._score = 0;
                }
                entry._score = 1000;
                break;
            }
        }
        // needs sorting
        this._entries.sort(LanguageFeatureRegistry._compareByScoreAndTime);
    };
    LanguageFeatureRegistry._compareByScoreAndTime = function (a, b) {
        if (a._score < b._score) {
            return 1;
        }
        else if (a._score > b._score) {
            return -1;
        }
        else if (a._time < b._time) {
            return 1;
        }
        else if (a._time > b._time) {
            return -1;
        }
        else {
            return 0;
        }
    };
    return LanguageFeatureRegistry;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var TokenizationRegistryImpl = /** @class */ (function () {
    function TokenizationRegistryImpl() {
        this._onDidChange = new Emitter();
        this.onDidChange = this._onDidChange.event;
        this._map = Object.create(null);
        this._promises = Object.create(null);
        this._colorMap = null;
    }
    TokenizationRegistryImpl.prototype.fire = function (languages) {
        this._onDidChange.fire({
            changedLanguages: languages,
            changedColorMap: false
        });
    };
    TokenizationRegistryImpl.prototype.register = function (language, support) {
        var _this = this;
        this._map[language] = support;
        this.fire([language]);
        return toDisposable(function () {
            if (_this._map[language] !== support) {
                return;
            }
            delete _this._map[language];
            _this.fire([language]);
        });
    };
    TokenizationRegistryImpl.prototype.getPromise = function (language) {
        var _this = this;
        var support = this.get(language);
        if (support) {
            return Promise.resolve(support);
        }
        var promise = this._promises[language];
        if (promise) {
            return promise.then(function (_) { return _this.get(language); });
        }
        return null;
    };
    TokenizationRegistryImpl.prototype.get = function (language) {
        return (this._map[language] || null);
    };
    TokenizationRegistryImpl.prototype.setColorMap = function (colorMap) {
        this._colorMap = colorMap;
        this._onDidChange.fire({
            changedLanguages: Object.keys(this._map),
            changedColorMap: true
        });
    };
    TokenizationRegistryImpl.prototype.getColorMap = function () {
        return this._colorMap;
    };
    TokenizationRegistryImpl.prototype.getDefaultBackground = function () {
        if (this._colorMap && this._colorMap.length > 2 /* DefaultBackground */) {
            return this._colorMap[2 /* DefaultBackground */];
        }
        return null;
    };
    return TokenizationRegistryImpl;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * @internal
 */
var LanguageIdentifier = /** @class */ (function () {
    function LanguageIdentifier(language, id) {
        this.language = language;
        this.id = id;
    }
    return LanguageIdentifier;
}());
/**
 * @internal
 */
var TokenMetadata = /** @class */ (function () {
    function TokenMetadata() {
    }
    TokenMetadata.getLanguageId = function (metadata) {
        return (metadata & 255 /* LANGUAGEID_MASK */) >>> 0 /* LANGUAGEID_OFFSET */;
    };
    TokenMetadata.getTokenType = function (metadata) {
        return (metadata & 1792 /* TOKEN_TYPE_MASK */) >>> 8 /* TOKEN_TYPE_OFFSET */;
    };
    TokenMetadata.getFontStyle = function (metadata) {
        return (metadata & 14336 /* FONT_STYLE_MASK */) >>> 11 /* FONT_STYLE_OFFSET */;
    };
    TokenMetadata.getForeground = function (metadata) {
        return (metadata & 8372224 /* FOREGROUND_MASK */) >>> 14 /* FOREGROUND_OFFSET */;
    };
    TokenMetadata.getBackground = function (metadata) {
        return (metadata & 4286578688 /* BACKGROUND_MASK */) >>> 23 /* BACKGROUND_OFFSET */;
    };
    TokenMetadata.getClassNameFromMetadata = function (metadata) {
        var foreground = this.getForeground(metadata);
        var className = 'mtk' + foreground;
        var fontStyle = this.getFontStyle(metadata);
        if (fontStyle & 1 /* Italic */) {
            className += ' mtki';
        }
        if (fontStyle & 2 /* Bold */) {
            className += ' mtkb';
        }
        if (fontStyle & 4 /* Underline */) {
            className += ' mtku';
        }
        return className;
    };
    TokenMetadata.getInlineStyleFromMetadata = function (metadata, colorMap) {
        var foreground = this.getForeground(metadata);
        var fontStyle = this.getFontStyle(metadata);
        var result = "color: " + colorMap[foreground] + ";";
        if (fontStyle & 1 /* Italic */) {
            result += 'font-style: italic;';
        }
        if (fontStyle & 2 /* Bold */) {
            result += 'font-weight: bold;';
        }
        if (fontStyle & 4 /* Underline */) {
            result += 'text-decoration: underline;';
        }
        return result;
    };
    return TokenMetadata;
}());
/**
 * @internal
 */
var completionKindToCssClass = (function () {
    var data = Object.create(null);
    data[0 /* Method */] = 'method';
    data[1 /* Function */] = 'function';
    data[2 /* Constructor */] = 'constructor';
    data[3 /* Field */] = 'field';
    data[4 /* Variable */] = 'variable';
    data[5 /* Class */] = 'class';
    data[6 /* Struct */] = 'struct';
    data[7 /* Interface */] = 'interface';
    data[8 /* Module */] = 'module';
    data[9 /* Property */] = 'property';
    data[10 /* Event */] = 'event';
    data[11 /* Operator */] = 'operator';
    data[12 /* Unit */] = 'unit';
    data[13 /* Value */] = 'value';
    data[14 /* Constant */] = 'constant';
    data[15 /* Enum */] = 'enum';
    data[16 /* EnumMember */] = 'enum-member';
    data[17 /* Keyword */] = 'keyword';
    data[25 /* Snippet */] = 'snippet';
    data[18 /* Text */] = 'text';
    data[19 /* Color */] = 'color';
    data[20 /* File */] = 'file';
    data[21 /* Reference */] = 'reference';
    data[22 /* Customcolor */] = 'customcolor';
    data[23 /* Folder */] = 'folder';
    data[24 /* TypeParameter */] = 'type-parameter';
    return function (kind) {
        return data[kind] || 'property';
    };
})();
/**
 * @internal
 */
var completionKindFromLegacyString = (function () {
    var data = Object.create(null);
    data['method'] = 0 /* Method */;
    data['function'] = 1 /* Function */;
    data['constructor'] = 2 /* Constructor */;
    data['field'] = 3 /* Field */;
    data['variable'] = 4 /* Variable */;
    data['class'] = 5 /* Class */;
    data['struct'] = 6 /* Struct */;
    data['interface'] = 7 /* Interface */;
    data['module'] = 8 /* Module */;
    data['property'] = 9 /* Property */;
    data['event'] = 10 /* Event */;
    data['operator'] = 11 /* Operator */;
    data['unit'] = 12 /* Unit */;
    data['value'] = 13 /* Value */;
    data['constant'] = 14 /* Constant */;
    data['enum'] = 15 /* Enum */;
    data['enum-member'] = 16 /* EnumMember */;
    data['keyword'] = 17 /* Keyword */;
    data['snippet'] = 25 /* Snippet */;
    data['text'] = 18 /* Text */;
    data['color'] = 19 /* Color */;
    data['file'] = 20 /* File */;
    data['reference'] = 21 /* Reference */;
    data['customcolor'] = 22 /* Customcolor */;
    data['folder'] = 23 /* Folder */;
    data['type-parameter'] = 24 /* TypeParameter */;
    return function (value) {
        return data[value] || 'property';
    };
})();
var SignatureHelpTriggerReason;
(function (SignatureHelpTriggerReason) {
    SignatureHelpTriggerReason[SignatureHelpTriggerReason["Invoke"] = 1] = "Invoke";
    SignatureHelpTriggerReason[SignatureHelpTriggerReason["TriggerCharacter"] = 2] = "TriggerCharacter";
    SignatureHelpTriggerReason[SignatureHelpTriggerReason["ContentChange"] = 3] = "ContentChange";
})(SignatureHelpTriggerReason || (SignatureHelpTriggerReason = {}));
/**
 * A document highlight kind.
 */
var DocumentHighlightKind;
(function (DocumentHighlightKind) {
    /**
     * A textual occurrence.
     */
    DocumentHighlightKind[DocumentHighlightKind["Text"] = 0] = "Text";
    /**
     * Read-access of a symbol, like reading a variable.
     */
    DocumentHighlightKind[DocumentHighlightKind["Read"] = 1] = "Read";
    /**
     * Write-access of a symbol, like writing to a variable.
     */
    DocumentHighlightKind[DocumentHighlightKind["Write"] = 2] = "Write";
})(DocumentHighlightKind || (DocumentHighlightKind = {}));
/**
 * @internal
 */
var symbolKindToCssClass = (function () {
    var _fromMapping = Object.create(null);
    _fromMapping[0 /* File */] = 'file';
    _fromMapping[1 /* Module */] = 'module';
    _fromMapping[2 /* Namespace */] = 'namespace';
    _fromMapping[3 /* Package */] = 'package';
    _fromMapping[4 /* Class */] = 'class';
    _fromMapping[5 /* Method */] = 'method';
    _fromMapping[6 /* Property */] = 'property';
    _fromMapping[7 /* Field */] = 'field';
    _fromMapping[8 /* Constructor */] = 'constructor';
    _fromMapping[9 /* Enum */] = 'enum';
    _fromMapping[10 /* Interface */] = 'interface';
    _fromMapping[11 /* Function */] = 'function';
    _fromMapping[12 /* Variable */] = 'variable';
    _fromMapping[13 /* Constant */] = 'constant';
    _fromMapping[14 /* String */] = 'string';
    _fromMapping[15 /* Number */] = 'number';
    _fromMapping[16 /* Boolean */] = 'boolean';
    _fromMapping[17 /* Array */] = 'array';
    _fromMapping[18 /* Object */] = 'object';
    _fromMapping[19 /* Key */] = 'key';
    _fromMapping[20 /* Null */] = 'null';
    _fromMapping[21 /* EnumMember */] = 'enum-member';
    _fromMapping[22 /* Struct */] = 'struct';
    _fromMapping[23 /* Event */] = 'event';
    _fromMapping[24 /* Operator */] = 'operator';
    _fromMapping[25 /* TypeParameter */] = 'type-parameter';
    return function toCssClassName(kind) {
        return "symbol-icon " + (_fromMapping[kind] || 'property');
    };
})();
var FoldingRangeKind = /** @class */ (function () {
    /**
     * Creates a new [FoldingRangeKind](#FoldingRangeKind).
     *
     * @param value of the kind.
     */
    function FoldingRangeKind(value) {
        this.value = value;
    }
    /**
     * Kind for folding range representing a comment. The value of the kind is 'comment'.
     */
    FoldingRangeKind.Comment = new FoldingRangeKind('comment');
    /**
     * Kind for folding range representing a import. The value of the kind is 'imports'.
     */
    FoldingRangeKind.Imports = new FoldingRangeKind('imports');
    /**
     * Kind for folding range representing regions (for example marked by `#region`, `#endregion`).
     * The value of the kind is 'region'.
     */
    FoldingRangeKind.Region = new FoldingRangeKind('region');
    return FoldingRangeKind;
}());
/**
 * @internal
 */
function isResourceTextEdit(thing) {
    return isObject(thing) && thing.resource && Array.isArray(thing.edits);
}
// --- feature registries ------
/**
 * @internal
 */
var ReferenceProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var RenameProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var CompletionProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var SignatureHelpProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var HoverProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var DocumentSymbolProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var DocumentHighlightProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var DefinitionProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var ImplementationProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var TypeDefinitionProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var CodeLensProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var CodeActionProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var DocumentFormattingEditProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var DocumentRangeFormattingEditProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var OnTypeFormattingEditProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var LinkProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var ColorProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var FoldingRangeProviderRegistry = new LanguageFeatureRegistry();
/**
 * @internal
 */
var TokenizationRegistry = new TokenizationRegistryImpl();

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var LineTokens = /** @class */ (function () {
    function LineTokens(tokens, text) {
        this._tokens = tokens;
        this._tokensCount = (this._tokens.length >>> 1);
        this._text = text;
    }
    LineTokens.prototype.equals = function (other) {
        if (other instanceof LineTokens) {
            return this.slicedEquals(other, 0, this._tokensCount);
        }
        return false;
    };
    LineTokens.prototype.slicedEquals = function (other, sliceFromTokenIndex, sliceTokenCount) {
        if (this._text !== other._text) {
            return false;
        }
        if (this._tokensCount !== other._tokensCount) {
            return false;
        }
        var from = (sliceFromTokenIndex << 1);
        var to = from + (sliceTokenCount << 1);
        for (var i = from; i < to; i++) {
            if (this._tokens[i] !== other._tokens[i]) {
                return false;
            }
        }
        return true;
    };
    LineTokens.prototype.getLineContent = function () {
        return this._text;
    };
    LineTokens.prototype.getCount = function () {
        return this._tokensCount;
    };
    LineTokens.prototype.getStartOffset = function (tokenIndex) {
        if (tokenIndex > 0) {
            return this._tokens[(tokenIndex - 1) << 1];
        }
        return 0;
    };
    LineTokens.prototype.getLanguageId = function (tokenIndex) {
        var metadata = this._tokens[(tokenIndex << 1) + 1];
        return TokenMetadata.getLanguageId(metadata);
    };
    LineTokens.prototype.getStandardTokenType = function (tokenIndex) {
        var metadata = this._tokens[(tokenIndex << 1) + 1];
        return TokenMetadata.getTokenType(metadata);
    };
    LineTokens.prototype.getForeground = function (tokenIndex) {
        var metadata = this._tokens[(tokenIndex << 1) + 1];
        return TokenMetadata.getForeground(metadata);
    };
    LineTokens.prototype.getClassName = function (tokenIndex) {
        var metadata = this._tokens[(tokenIndex << 1) + 1];
        return TokenMetadata.getClassNameFromMetadata(metadata);
    };
    LineTokens.prototype.getInlineStyle = function (tokenIndex, colorMap) {
        var metadata = this._tokens[(tokenIndex << 1) + 1];
        return TokenMetadata.getInlineStyleFromMetadata(metadata, colorMap);
    };
    LineTokens.prototype.getEndOffset = function (tokenIndex) {
        return this._tokens[tokenIndex << 1];
    };
    /**
     * Find the token containing offset `offset`.
     * @param offset The search offset
     * @return The index of the token containing the offset.
     */
    LineTokens.prototype.findTokenIndexAtOffset = function (offset) {
        return LineTokens.findIndexInTokensArray(this._tokens, offset);
    };
    LineTokens.prototype.inflate = function () {
        return this;
    };
    LineTokens.prototype.sliceAndInflate = function (startOffset, endOffset, deltaOffset) {
        return new SlicedLineTokens(this, startOffset, endOffset, deltaOffset);
    };
    LineTokens.convertToEndOffset = function (tokens, lineTextLength) {
        var tokenCount = (tokens.length >>> 1);
        var lastTokenIndex = tokenCount - 1;
        for (var tokenIndex = 0; tokenIndex < lastTokenIndex; tokenIndex++) {
            tokens[tokenIndex << 1] = tokens[(tokenIndex + 1) << 1];
        }
        tokens[lastTokenIndex << 1] = lineTextLength;
    };
    LineTokens.findIndexInTokensArray = function (tokens, desiredIndex) {
        if (tokens.length <= 2) {
            return 0;
        }
        var low = 0;
        var high = (tokens.length >>> 1) - 1;
        while (low < high) {
            var mid = low + Math.floor((high - low) / 2);
            var endOffset = tokens[(mid << 1)];
            if (endOffset === desiredIndex) {
                return mid + 1;
            }
            else if (endOffset < desiredIndex) {
                low = mid + 1;
            }
            else if (endOffset > desiredIndex) {
                high = mid;
            }
        }
        return low;
    };
    return LineTokens;
}());
var SlicedLineTokens = /** @class */ (function () {
    function SlicedLineTokens(source, startOffset, endOffset, deltaOffset) {
        this._source = source;
        this._startOffset = startOffset;
        this._endOffset = endOffset;
        this._deltaOffset = deltaOffset;
        this._firstTokenIndex = source.findTokenIndexAtOffset(startOffset);
        this._tokensCount = 0;
        for (var i = this._firstTokenIndex, len = source.getCount(); i < len; i++) {
            var tokenStartOffset = source.getStartOffset(i);
            if (tokenStartOffset >= endOffset) {
                break;
            }
            this._tokensCount++;
        }
    }
    SlicedLineTokens.prototype.equals = function (other) {
        if (other instanceof SlicedLineTokens) {
            return (this._startOffset === other._startOffset
                && this._endOffset === other._endOffset
                && this._deltaOffset === other._deltaOffset
                && this._source.slicedEquals(other._source, this._firstTokenIndex, this._tokensCount));
        }
        return false;
    };
    SlicedLineTokens.prototype.getCount = function () {
        return this._tokensCount;
    };
    SlicedLineTokens.prototype.getForeground = function (tokenIndex) {
        return this._source.getForeground(this._firstTokenIndex + tokenIndex);
    };
    SlicedLineTokens.prototype.getEndOffset = function (tokenIndex) {
        var tokenEndOffset = this._source.getEndOffset(this._firstTokenIndex + tokenIndex);
        return Math.min(this._endOffset, tokenEndOffset) - this._startOffset + this._deltaOffset;
    };
    SlicedLineTokens.prototype.getClassName = function (tokenIndex) {
        return this._source.getClassName(this._firstTokenIndex + tokenIndex);
    };
    SlicedLineTokens.prototype.getInlineStyle = function (tokenIndex, colorMap) {
        return this._source.getInlineStyle(this._firstTokenIndex + tokenIndex, colorMap);
    };
    SlicedLineTokens.prototype.findTokenIndexAtOffset = function (offset) {
        return this._source.findTokenIndexAtOffset(offset + this._startOffset - this._deltaOffset) - this._firstTokenIndex;
    };
    return SlicedLineTokens;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Token = /** @class */ (function () {
    function Token(offset, type, language) {
        this.offset = offset | 0; // @perf
        this.type = type;
        this.language = language;
    }
    Token.prototype.toString = function () {
        return '(' + this.offset + ', ' + this.type + ')';
    };
    return Token;
}());
var TokenizationResult = /** @class */ (function () {
    function TokenizationResult(tokens, endState) {
        this.tokens = tokens;
        this.endState = endState;
    }
    return TokenizationResult;
}());
var TokenizationResult2 = /** @class */ (function () {
    function TokenizationResult2(tokens, endState) {
        this.tokens = tokens;
        this.endState = endState;
    }
    return TokenizationResult2;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var NullStateImpl = /** @class */ (function () {
    function NullStateImpl() {
    }
    NullStateImpl.prototype.clone = function () {
        return this;
    };
    NullStateImpl.prototype.equals = function (other) {
        return (this === other);
    };
    return NullStateImpl;
}());
var NULL_STATE = new NullStateImpl();
var NULL_MODE_ID = 'vs.editor.nullMode';
var NULL_LANGUAGE_IDENTIFIER = new LanguageIdentifier(NULL_MODE_ID, 0 /* Null */);
function nullTokenize(modeId, buffer, state, deltaOffset) {
    return new TokenizationResult([new Token(deltaOffset, '', modeId)], state);
}
function nullTokenize2(languageId, buffer, state, deltaOffset) {
    var tokens = new Uint32Array(2);
    tokens[0] = deltaOffset;
    tokens[1] = ((languageId << 0 /* LANGUAGEID_OFFSET */)
        | (0 /* Other */ << 8 /* TOKEN_TYPE_OFFSET */)
        | (0 /* None */ << 11 /* FONT_STYLE_OFFSET */)
        | (1 /* DefaultForeground */ << 14 /* FOREGROUND_OFFSET */)
        | (2 /* DefaultBackground */ << 23 /* BACKGROUND_OFFSET */)) >>> 0;
    return new TokenizationResult2(tokens, state === null ? NULL_STATE : state);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getDefaultMetadata(topLevelLanguageId) {
    return ((topLevelLanguageId << 0 /* LANGUAGEID_OFFSET */)
        | (0 /* Other */ << 8 /* TOKEN_TYPE_OFFSET */)
        | (0 /* None */ << 11 /* FONT_STYLE_OFFSET */)
        | (1 /* DefaultForeground */ << 14 /* FOREGROUND_OFFSET */)
        | (2 /* DefaultBackground */ << 23 /* BACKGROUND_OFFSET */)) >>> 0;
}
var EMPTY_LINE_TOKENS = (new Uint32Array(0)).buffer;
var ModelLineTokens = /** @class */ (function () {
    function ModelLineTokens(state) {
        this._state = state;
        this._lineTokens = null;
        this._invalid = true;
    }
    ModelLineTokens.prototype.deleteBeginning = function (toChIndex) {
        if (this._lineTokens === null || this._lineTokens === EMPTY_LINE_TOKENS) {
            return;
        }
        this.delete(0, toChIndex);
    };
    ModelLineTokens.prototype.deleteEnding = function (fromChIndex) {
        if (this._lineTokens === null || this._lineTokens === EMPTY_LINE_TOKENS) {
            return;
        }
        var tokens = new Uint32Array(this._lineTokens);
        var lineTextLength = tokens[tokens.length - 2];
        this.delete(fromChIndex, lineTextLength);
    };
    ModelLineTokens.prototype.delete = function (fromChIndex, toChIndex) {
        if (this._lineTokens === null || this._lineTokens === EMPTY_LINE_TOKENS || fromChIndex === toChIndex) {
            return;
        }
        var tokens = new Uint32Array(this._lineTokens);
        var tokensCount = (tokens.length >>> 1);
        // special case: deleting everything
        if (fromChIndex === 0 && tokens[tokens.length - 2] === toChIndex) {
            this._lineTokens = EMPTY_LINE_TOKENS;
            return;
        }
        var fromTokenIndex = LineTokens.findIndexInTokensArray(tokens, fromChIndex);
        var fromTokenStartOffset = (fromTokenIndex > 0 ? tokens[(fromTokenIndex - 1) << 1] : 0);
        var fromTokenEndOffset = tokens[fromTokenIndex << 1];
        if (toChIndex < fromTokenEndOffset) {
            // the delete range is inside a single token
            var delta_1 = (toChIndex - fromChIndex);
            for (var i = fromTokenIndex; i < tokensCount; i++) {
                tokens[i << 1] -= delta_1;
            }
            return;
        }
        var dest;
        var lastEnd;
        if (fromTokenStartOffset !== fromChIndex) {
            tokens[fromTokenIndex << 1] = fromChIndex;
            dest = ((fromTokenIndex + 1) << 1);
            lastEnd = fromChIndex;
        }
        else {
            dest = (fromTokenIndex << 1);
            lastEnd = fromTokenStartOffset;
        }
        var delta = (toChIndex - fromChIndex);
        for (var tokenIndex = fromTokenIndex + 1; tokenIndex < tokensCount; tokenIndex++) {
            var tokenEndOffset = tokens[tokenIndex << 1] - delta;
            if (tokenEndOffset > lastEnd) {
                tokens[dest++] = tokenEndOffset;
                tokens[dest++] = tokens[(tokenIndex << 1) + 1];
                lastEnd = tokenEndOffset;
            }
        }
        if (dest === tokens.length) {
            // nothing to trim
            return;
        }
        var tmp = new Uint32Array(dest);
        tmp.set(tokens.subarray(0, dest), 0);
        this._lineTokens = tmp.buffer;
    };
    ModelLineTokens.prototype.append = function (_otherTokens) {
        if (_otherTokens === EMPTY_LINE_TOKENS) {
            return;
        }
        if (this._lineTokens === EMPTY_LINE_TOKENS) {
            this._lineTokens = _otherTokens;
            return;
        }
        if (this._lineTokens === null) {
            return;
        }
        if (_otherTokens === null) {
            // cannot determine combined line length...
            this._lineTokens = null;
            return;
        }
        var myTokens = new Uint32Array(this._lineTokens);
        var otherTokens = new Uint32Array(_otherTokens);
        var otherTokensCount = (otherTokens.length >>> 1);
        var result = new Uint32Array(myTokens.length + otherTokens.length);
        result.set(myTokens, 0);
        var dest = myTokens.length;
        var delta = myTokens[myTokens.length - 2];
        for (var i = 0; i < otherTokensCount; i++) {
            result[dest++] = otherTokens[(i << 1)] + delta;
            result[dest++] = otherTokens[(i << 1) + 1];
        }
        this._lineTokens = result.buffer;
    };
    ModelLineTokens.prototype.insert = function (chIndex, textLength) {
        if (!this._lineTokens) {
            // nothing to do
            return;
        }
        var tokens = new Uint32Array(this._lineTokens);
        var tokensCount = (tokens.length >>> 1);
        var fromTokenIndex = LineTokens.findIndexInTokensArray(tokens, chIndex);
        if (fromTokenIndex > 0) {
            var fromTokenStartOffset = (fromTokenIndex > 0 ? tokens[(fromTokenIndex - 1) << 1] : 0);
            if (fromTokenStartOffset === chIndex) {
                fromTokenIndex--;
            }
        }
        for (var tokenIndex = fromTokenIndex; tokenIndex < tokensCount; tokenIndex++) {
            tokens[tokenIndex << 1] += textLength;
        }
    };
    return ModelLineTokens;
}());
var ModelLinesTokens = /** @class */ (function () {
    function ModelLinesTokens(languageIdentifier, tokenizationSupport) {
        this.languageIdentifier = languageIdentifier;
        this.tokenizationSupport = tokenizationSupport;
        this._tokens = [];
        if (this.tokenizationSupport) {
            var initialState = null;
            try {
                initialState = this.tokenizationSupport.getInitialState();
            }
            catch (e) {
                onUnexpectedError(e);
                this.tokenizationSupport = null;
            }
            if (initialState) {
                this._tokens[0] = new ModelLineTokens(initialState);
            }
        }
        this._invalidLineStartIndex = 0;
        this._lastState = null;
    }
    Object.defineProperty(ModelLinesTokens.prototype, "inValidLineStartIndex", {
        get: function () {
            return this._invalidLineStartIndex;
        },
        enumerable: true,
        configurable: true
    });
    ModelLinesTokens.prototype.getTokens = function (topLevelLanguageId, lineIndex, lineText) {
        var rawLineTokens = null;
        if (lineIndex < this._tokens.length && this._tokens[lineIndex]) {
            rawLineTokens = this._tokens[lineIndex]._lineTokens;
        }
        if (rawLineTokens !== null && rawLineTokens !== EMPTY_LINE_TOKENS) {
            return new LineTokens(new Uint32Array(rawLineTokens), lineText);
        }
        var lineTokens = new Uint32Array(2);
        lineTokens[0] = lineText.length;
        lineTokens[1] = getDefaultMetadata(topLevelLanguageId);
        return new LineTokens(lineTokens, lineText);
    };
    ModelLinesTokens.prototype.isCheapToTokenize = function (lineNumber) {
        var firstInvalidLineNumber = this._invalidLineStartIndex + 1;
        return (firstInvalidLineNumber >= lineNumber);
    };
    ModelLinesTokens.prototype.hasLinesToTokenize = function (buffer) {
        return (this._invalidLineStartIndex < buffer.getLineCount());
    };
    ModelLinesTokens.prototype.invalidateLine = function (lineIndex) {
        this._setIsInvalid(lineIndex, true);
        if (lineIndex < this._invalidLineStartIndex) {
            this._setIsInvalid(this._invalidLineStartIndex, true);
            this._invalidLineStartIndex = lineIndex;
        }
    };
    ModelLinesTokens.prototype._setIsInvalid = function (lineIndex, invalid) {
        if (lineIndex < this._tokens.length && this._tokens[lineIndex]) {
            this._tokens[lineIndex]._invalid = invalid;
        }
    };
    ModelLinesTokens.prototype._isInvalid = function (lineIndex) {
        if (lineIndex < this._tokens.length && this._tokens[lineIndex]) {
            return this._tokens[lineIndex]._invalid;
        }
        return true;
    };
    ModelLinesTokens.prototype._getState = function (lineIndex) {
        if (lineIndex < this._tokens.length && this._tokens[lineIndex]) {
            return this._tokens[lineIndex]._state;
        }
        return null;
    };
    ModelLinesTokens.prototype._setTokens = function (topLevelLanguageId, lineIndex, lineTextLength, tokens) {
        var target;
        if (lineIndex < this._tokens.length && this._tokens[lineIndex]) {
            target = this._tokens[lineIndex];
        }
        else {
            target = new ModelLineTokens(null);
            this._tokens[lineIndex] = target;
        }
        if (lineTextLength === 0) {
            target._lineTokens = EMPTY_LINE_TOKENS;
            return;
        }
        if (!tokens || tokens.length === 0) {
            tokens = new Uint32Array(2);
            tokens[0] = 0;
            tokens[1] = getDefaultMetadata(topLevelLanguageId);
        }
        LineTokens.convertToEndOffset(tokens, lineTextLength);
        target._lineTokens = tokens.buffer;
    };
    ModelLinesTokens.prototype._setState = function (lineIndex, state) {
        if (lineIndex < this._tokens.length && this._tokens[lineIndex]) {
            this._tokens[lineIndex]._state = state;
        }
        else {
            var tmp = new ModelLineTokens(state);
            this._tokens[lineIndex] = tmp;
        }
    };
    //#region Editing
    ModelLinesTokens.prototype.applyEdits = function (range$$1, eolCount, firstLineLength) {
        var deletingLinesCnt = range$$1.endLineNumber - range$$1.startLineNumber;
        var insertingLinesCnt = eolCount;
        var editingLinesCnt = Math.min(deletingLinesCnt, insertingLinesCnt);
        for (var j = editingLinesCnt; j >= 0; j--) {
            this.invalidateLine(range$$1.startLineNumber + j - 1);
        }
        this._acceptDeleteRange(range$$1);
        this._acceptInsertText(new Position(range$$1.startLineNumber, range$$1.startColumn), eolCount, firstLineLength);
    };
    ModelLinesTokens.prototype._acceptDeleteRange = function (range$$1) {
        var firstLineIndex = range$$1.startLineNumber - 1;
        if (firstLineIndex >= this._tokens.length) {
            return;
        }
        if (range$$1.startLineNumber === range$$1.endLineNumber) {
            if (range$$1.startColumn === range$$1.endColumn) {
                // Nothing to delete
                return;
            }
            this._tokens[firstLineIndex].delete(range$$1.startColumn - 1, range$$1.endColumn - 1);
            return;
        }
        var firstLine = this._tokens[firstLineIndex];
        firstLine.deleteEnding(range$$1.startColumn - 1);
        var lastLineIndex = range$$1.endLineNumber - 1;
        var lastLineTokens = null;
        if (lastLineIndex < this._tokens.length) {
            var lastLine = this._tokens[lastLineIndex];
            lastLine.deleteBeginning(range$$1.endColumn - 1);
            lastLineTokens = lastLine._lineTokens;
        }
        // Take remaining text on last line and append it to remaining text on first line
        firstLine.append(lastLineTokens);
        // Delete middle lines
        this._tokens.splice(range$$1.startLineNumber, range$$1.endLineNumber - range$$1.startLineNumber);
    };
    ModelLinesTokens.prototype._acceptInsertText = function (position, eolCount, firstLineLength) {
        if (eolCount === 0 && firstLineLength === 0) {
            // Nothing to insert
            return;
        }
        var lineIndex = position.lineNumber - 1;
        if (lineIndex >= this._tokens.length) {
            return;
        }
        if (eolCount === 0) {
            // Inserting text on one line
            this._tokens[lineIndex].insert(position.column - 1, firstLineLength);
            return;
        }
        var line = this._tokens[lineIndex];
        line.deleteEnding(position.column - 1);
        line.insert(position.column - 1, firstLineLength);
        var insert = new Array(eolCount);
        for (var i = eolCount - 1; i >= 0; i--) {
            insert[i] = new ModelLineTokens(null);
        }
        this._tokens = arrayInsert(this._tokens, position.lineNumber, insert);
    };
    //#endregion
    //#region Tokenization
    ModelLinesTokens.prototype._tokenizeOneLine = function (buffer, eventBuilder) {
        if (!this.hasLinesToTokenize(buffer)) {
            return buffer.getLineCount() + 1;
        }
        var lineNumber = this._invalidLineStartIndex + 1;
        this._updateTokensUntilLine(buffer, eventBuilder, lineNumber);
        return lineNumber;
    };
    ModelLinesTokens.prototype._tokenizeText = function (buffer, text, state) {
        var r = null;
        if (this.tokenizationSupport) {
            try {
                r = this.tokenizationSupport.tokenize2(text, state, 0);
            }
            catch (e) {
                onUnexpectedError(e);
            }
        }
        if (!r) {
            r = nullTokenize2(this.languageIdentifier.id, text, state, 0);
        }
        return r;
    };
    ModelLinesTokens.prototype._updateTokensUntilLine = function (buffer, eventBuilder, lineNumber) {
        if (!this.tokenizationSupport) {
            this._invalidLineStartIndex = buffer.getLineCount();
            return;
        }
        var linesLength = buffer.getLineCount();
        var endLineIndex = lineNumber - 1;
        // Validate all states up to and including endLineIndex
        for (var lineIndex = this._invalidLineStartIndex; lineIndex <= endLineIndex; lineIndex++) {
            var endStateIndex = lineIndex + 1;
            var text = buffer.getLineContent(lineIndex + 1);
            var lineStartState = this._getState(lineIndex);
            var r = null;
            try {
                // Tokenize only the first X characters
                var freshState = lineStartState.clone();
                r = this.tokenizationSupport.tokenize2(text, freshState, 0);
            }
            catch (e) {
                onUnexpectedError(e);
            }
            if (!r) {
                r = nullTokenize2(this.languageIdentifier.id, text, lineStartState, 0);
            }
            this._setTokens(this.languageIdentifier.id, lineIndex, text.length, r.tokens);
            eventBuilder.registerChangedTokens(lineIndex + 1);
            this._setIsInvalid(lineIndex, false);
            if (endStateIndex < linesLength) {
                var previousEndState = this._getState(endStateIndex);
                if (previousEndState !== null && r.endState.equals(previousEndState)) {
                    // The end state of this line remains the same
                    var nextInvalidLineIndex = lineIndex + 1;
                    while (nextInvalidLineIndex < linesLength) {
                        if (this._isInvalid(nextInvalidLineIndex)) {
                            break;
                        }
                        if (nextInvalidLineIndex + 1 < linesLength) {
                            if (this._getState(nextInvalidLineIndex + 1) === null) {
                                break;
                            }
                        }
                        else {
                            if (this._lastState === null) {
                                break;
                            }
                        }
                        nextInvalidLineIndex++;
                    }
                    this._invalidLineStartIndex = Math.max(this._invalidLineStartIndex, nextInvalidLineIndex);
                    lineIndex = nextInvalidLineIndex - 1; // -1 because the outer loop increments it
                }
                else {
                    this._setState(endStateIndex, r.endState);
                }
            }
            else {
                this._lastState = r.endState;
            }
        }
        this._invalidLineStartIndex = Math.max(this._invalidLineStartIndex, endLineIndex + 1);
    };
    return ModelLinesTokens;
}());
var ModelTokensChangedEventBuilder = /** @class */ (function () {
    function ModelTokensChangedEventBuilder() {
        this._ranges = [];
    }
    ModelTokensChangedEventBuilder.prototype.registerChangedTokens = function (lineNumber) {
        var ranges = this._ranges;
        var rangesLength = ranges.length;
        var previousRange = rangesLength > 0 ? ranges[rangesLength - 1] : null;
        if (previousRange && previousRange.toLineNumber === lineNumber - 1) {
            // extend previous range
            previousRange.toLineNumber++;
        }
        else {
            // insert new range
            ranges[rangesLength] = {
                fromLineNumber: lineNumber,
                toLineNumber: lineNumber
            };
        }
    };
    ModelTokensChangedEventBuilder.prototype.build = function () {
        if (this._ranges.length === 0) {
            return null;
        }
        return {
            ranges: this._ranges
        };
    };
    return ModelTokensChangedEventBuilder;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Describes what to do with the indentation when pressing Enter.
 */
var IndentAction;
(function (IndentAction) {
    /**
     * Insert new line and copy the previous line's indentation.
     */
    IndentAction[IndentAction["None"] = 0] = "None";
    /**
     * Insert new line and indent once (relative to the previous line's indentation).
     */
    IndentAction[IndentAction["Indent"] = 1] = "Indent";
    /**
     * Insert two new lines:
     *  - the first one indented which will hold the cursor
     *  - the second one at the same indentation level
     */
    IndentAction[IndentAction["IndentOutdent"] = 2] = "IndentOutdent";
    /**
     * Insert new line and outdent once (relative to the previous line's indentation).
     */
    IndentAction[IndentAction["Outdent"] = 3] = "Outdent";
})(IndentAction || (IndentAction = {}));
/**
 * @internal
 */
var StandardAutoClosingPairConditional = /** @class */ (function () {
    function StandardAutoClosingPairConditional(source) {
        this.open = source.open;
        this.close = source.close;
        // initially allowed in all tokens
        this._standardTokenMask = 0;
        if (Array.isArray(source.notIn)) {
            for (var i = 0, len = source.notIn.length; i < len; i++) {
                var notIn = source.notIn[i];
                switch (notIn) {
                    case 'string':
                        this._standardTokenMask |= 2 /* String */;
                        break;
                    case 'comment':
                        this._standardTokenMask |= 1 /* Comment */;
                        break;
                    case 'regex':
                        this._standardTokenMask |= 4 /* RegEx */;
                        break;
                }
            }
        }
    }
    StandardAutoClosingPairConditional.prototype.isOK = function (standardToken) {
        return (this._standardTokenMask & standardToken) === 0;
    };
    return StandardAutoClosingPairConditional;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function createScopedLineTokens(context, offset) {
    var tokenCount = context.getCount();
    var tokenIndex = context.findTokenIndexAtOffset(offset);
    var desiredLanguageId = context.getLanguageId(tokenIndex);
    var lastTokenIndex = tokenIndex;
    while (lastTokenIndex + 1 < tokenCount && context.getLanguageId(lastTokenIndex + 1) === desiredLanguageId) {
        lastTokenIndex++;
    }
    var firstTokenIndex = tokenIndex;
    while (firstTokenIndex > 0 && context.getLanguageId(firstTokenIndex - 1) === desiredLanguageId) {
        firstTokenIndex--;
    }
    return new ScopedLineTokens(context, desiredLanguageId, firstTokenIndex, lastTokenIndex + 1, context.getStartOffset(firstTokenIndex), context.getEndOffset(lastTokenIndex));
}
var ScopedLineTokens = /** @class */ (function () {
    function ScopedLineTokens(actual, languageId, firstTokenIndex, lastTokenIndex, firstCharOffset, lastCharOffset) {
        this._actual = actual;
        this.languageId = languageId;
        this._firstTokenIndex = firstTokenIndex;
        this._lastTokenIndex = lastTokenIndex;
        this.firstCharOffset = firstCharOffset;
        this._lastCharOffset = lastCharOffset;
    }
    ScopedLineTokens.prototype.getLineContent = function () {
        var actualLineContent = this._actual.getLineContent();
        return actualLineContent.substring(this.firstCharOffset, this._lastCharOffset);
    };
    ScopedLineTokens.prototype.getTokenCount = function () {
        return this._lastTokenIndex - this._firstTokenIndex;
    };
    ScopedLineTokens.prototype.findTokenIndexAtOffset = function (offset) {
        return this._actual.findTokenIndexAtOffset(offset + this.firstCharOffset) - this._firstTokenIndex;
    };
    ScopedLineTokens.prototype.getStandardTokenType = function (tokenIndex) {
        return this._actual.getStandardTokenType(tokenIndex + this._firstTokenIndex);
    };
    return ScopedLineTokens;
}());
function ignoreBracketsInToken(standardTokenType) {
    return (standardTokenType & 7 /* value */) !== 0;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CharacterPairSupport = /** @class */ (function () {
    function CharacterPairSupport(config) {
        if (config.autoClosingPairs) {
            this._autoClosingPairs = config.autoClosingPairs.map(function (el) { return new StandardAutoClosingPairConditional(el); });
        }
        else if (config.brackets) {
            this._autoClosingPairs = config.brackets.map(function (b) { return new StandardAutoClosingPairConditional({ open: b[0], close: b[1] }); });
        }
        else {
            this._autoClosingPairs = [];
        }
        this._autoCloseBefore = typeof config.autoCloseBefore === 'string' ? config.autoCloseBefore : CharacterPairSupport.DEFAULT_AUTOCLOSE_BEFORE_LANGUAGE_DEFINED;
        this._surroundingPairs = config.surroundingPairs || this._autoClosingPairs;
    }
    CharacterPairSupport.prototype.getAutoClosingPairs = function () {
        return this._autoClosingPairs;
    };
    CharacterPairSupport.prototype.getAutoCloseBeforeSet = function () {
        return this._autoCloseBefore;
    };
    CharacterPairSupport.prototype.shouldAutoClosePair = function (character, context, column) {
        // Always complete on empty line
        if (context.getTokenCount() === 0) {
            return true;
        }
        var tokenIndex = context.findTokenIndexAtOffset(column - 2);
        var standardTokenType = context.getStandardTokenType(tokenIndex);
        for (var i = 0; i < this._autoClosingPairs.length; ++i) {
            var autoClosingPair = this._autoClosingPairs[i];
            if (autoClosingPair.open === character) {
                return autoClosingPair.isOK(standardTokenType);
            }
        }
        return false;
    };
    CharacterPairSupport.prototype.getSurroundingPairs = function () {
        return this._surroundingPairs;
    };
    CharacterPairSupport.DEFAULT_AUTOCLOSE_BEFORE_LANGUAGE_DEFINED = ';:.,=}])> \n\t';
    return CharacterPairSupport;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var RichEditBracket = /** @class */ (function () {
    function RichEditBracket(languageIdentifier, open, close, forwardRegex, reversedRegex) {
        this.languageIdentifier = languageIdentifier;
        this.open = open;
        this.close = close;
        this.forwardRegex = forwardRegex;
        this.reversedRegex = reversedRegex;
    }
    return RichEditBracket;
}());
var RichEditBrackets = /** @class */ (function () {
    function RichEditBrackets(languageIdentifier, brackets) {
        var _this = this;
        this.brackets = brackets.map(function (b) {
            return new RichEditBracket(languageIdentifier, b[0], b[1], getRegexForBracketPair({ open: b[0], close: b[1] }), getReversedRegexForBracketPair({ open: b[0], close: b[1] }));
        });
        this.forwardRegex = getRegexForBrackets(this.brackets);
        this.reversedRegex = getReversedRegexForBrackets(this.brackets);
        this.textIsBracket = {};
        this.textIsOpenBracket = {};
        var maxBracketLength = 0;
        this.brackets.forEach(function (b) {
            _this.textIsBracket[b.open.toLowerCase()] = b;
            _this.textIsBracket[b.close.toLowerCase()] = b;
            _this.textIsOpenBracket[b.open.toLowerCase()] = true;
            _this.textIsOpenBracket[b.close.toLowerCase()] = false;
            maxBracketLength = Math.max(maxBracketLength, b.open.length);
            maxBracketLength = Math.max(maxBracketLength, b.close.length);
        });
        this.maxBracketLength = maxBracketLength;
    }
    return RichEditBrackets;
}());
function once$3(keyFn, computeFn) {
    var cache = {};
    return function (input) {
        var key = keyFn(input);
        if (!cache.hasOwnProperty(key)) {
            cache[key] = computeFn(input);
        }
        return cache[key];
    };
}
var getRegexForBracketPair = once$3(function (input) { return input.open + ";" + input.close; }, function (input) {
    return createBracketOrRegExp([input.open, input.close]);
});
var getReversedRegexForBracketPair = once$3(function (input) { return input.open + ";" + input.close; }, function (input) {
    return createBracketOrRegExp([toReversedString(input.open), toReversedString(input.close)]);
});
var getRegexForBrackets = once$3(function (input) { return input.map(function (b) { return b.open + ";" + b.close; }).join(';'); }, function (input) {
    var pieces = [];
    input.forEach(function (b) {
        pieces.push(b.open);
        pieces.push(b.close);
    });
    return createBracketOrRegExp(pieces);
});
var getReversedRegexForBrackets = once$3(function (input) { return input.map(function (b) { return b.open + ";" + b.close; }).join(';'); }, function (input) {
    var pieces = [];
    input.forEach(function (b) {
        pieces.push(toReversedString(b.open));
        pieces.push(toReversedString(b.close));
    });
    return createBracketOrRegExp(pieces);
});
function prepareBracketForRegExp(str) {
    // This bracket pair uses letters like e.g. "begin" - "end"
    var insertWordBoundaries = (/^[\w]+$/.test(str));
    str = escapeRegExpCharacters(str);
    return (insertWordBoundaries ? "\\b" + str + "\\b" : str);
}
function createBracketOrRegExp(pieces) {
    var regexStr = "(" + pieces.map(prepareBracketForRegExp).join(')|(') + ")";
    return createRegExp(regexStr, true);
}
var toReversedString = (function () {
    function reverse(str) {
        var reversedStr = '';
        for (var i = str.length - 1; i >= 0; i--) {
            reversedStr += str.charAt(i);
        }
        return reversedStr;
    }
    var lastInput = null;
    var lastOutput = null;
    return function toReversedString(str) {
        if (lastInput !== str) {
            lastInput = str;
            lastOutput = reverse(lastInput);
        }
        return lastOutput;
    };
})();
var BracketsUtils = /** @class */ (function () {
    function BracketsUtils() {
    }
    BracketsUtils._findPrevBracketInText = function (reversedBracketRegex, lineNumber, reversedText, offset) {
        var m = reversedText.match(reversedBracketRegex);
        if (!m) {
            return null;
        }
        var matchOffset = reversedText.length - (m.index || 0);
        var matchLength = m[0].length;
        var absoluteMatchOffset = offset + matchOffset;
        return new Range(lineNumber, absoluteMatchOffset - matchLength + 1, lineNumber, absoluteMatchOffset + 1);
    };
    BracketsUtils.findPrevBracketInToken = function (reversedBracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd) {
        // Because JS does not support backwards regex search, we search forwards in a reversed string with a reversed regex ;)
        var reversedLineText = toReversedString(lineText);
        var reversedTokenText = reversedLineText.substring(lineText.length - currentTokenEnd, lineText.length - currentTokenStart);
        return this._findPrevBracketInText(reversedBracketRegex, lineNumber, reversedTokenText, currentTokenStart);
    };
    BracketsUtils.findNextBracketInText = function (bracketRegex, lineNumber, text, offset) {
        var m = text.match(bracketRegex);
        if (!m) {
            return null;
        }
        var matchOffset = m.index || 0;
        var matchLength = m[0].length;
        if (matchLength === 0) {
            return null;
        }
        var absoluteMatchOffset = offset + matchOffset;
        return new Range(lineNumber, absoluteMatchOffset + 1, lineNumber, absoluteMatchOffset + 1 + matchLength);
    };
    BracketsUtils.findNextBracketInToken = function (bracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd) {
        var currentTokenText = lineText.substring(currentTokenStart, currentTokenEnd);
        return this.findNextBracketInText(bracketRegex, lineNumber, currentTokenText, currentTokenStart);
    };
    return BracketsUtils;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var BracketElectricCharacterSupport = /** @class */ (function () {
    function BracketElectricCharacterSupport(richEditBrackets, autoClosePairs, contribution) {
        contribution = contribution || {};
        this._richEditBrackets = richEditBrackets;
        this._complexAutoClosePairs = autoClosePairs.filter(function (pair) { return pair.open.length > 1 && !!pair.close; }).map(function (el) { return new StandardAutoClosingPairConditional(el); });
        if (contribution.docComment) {
            // IDocComment is legacy, only partially supported
            this._complexAutoClosePairs.push(new StandardAutoClosingPairConditional({ open: contribution.docComment.open, close: contribution.docComment.close }));
        }
    }
    BracketElectricCharacterSupport.prototype.getElectricCharacters = function () {
        var result = [];
        if (this._richEditBrackets) {
            for (var i = 0, len = this._richEditBrackets.brackets.length; i < len; i++) {
                var bracketPair = this._richEditBrackets.brackets[i];
                var lastChar = bracketPair.close.charAt(bracketPair.close.length - 1);
                result.push(lastChar);
            }
        }
        // auto close
        for (var _i = 0, _a = this._complexAutoClosePairs; _i < _a.length; _i++) {
            var pair = _a[_i];
            result.push(pair.open.charAt(pair.open.length - 1));
        }
        // Filter duplicate entries
        result = result.filter(function (item, pos, array) {
            return array.indexOf(item) === pos;
        });
        return result;
    };
    BracketElectricCharacterSupport.prototype.onElectricCharacter = function (character, context, column) {
        return (this._onElectricAutoClose(character, context, column) ||
            this._onElectricAutoIndent(character, context, column));
    };
    BracketElectricCharacterSupport.prototype._onElectricAutoIndent = function (character, context, column) {
        if (!this._richEditBrackets || this._richEditBrackets.brackets.length === 0) {
            return null;
        }
        var tokenIndex = context.findTokenIndexAtOffset(column - 1);
        if (ignoreBracketsInToken(context.getStandardTokenType(tokenIndex))) {
            return null;
        }
        var reversedBracketRegex = this._richEditBrackets.reversedRegex;
        var text = context.getLineContent().substring(0, column - 1) + character;
        var r = BracketsUtils.findPrevBracketInToken(reversedBracketRegex, 1, text, 0, text.length);
        if (!r) {
            return null;
        }
        var bracketText = text.substring(r.startColumn - 1, r.endColumn - 1);
        bracketText = bracketText.toLowerCase();
        var isOpen = this._richEditBrackets.textIsOpenBracket[bracketText];
        if (isOpen) {
            return null;
        }
        var textBeforeBracket = text.substring(0, r.startColumn - 1);
        if (!/^\s*$/.test(textBeforeBracket)) {
            // There is other text on the line before the bracket
            return null;
        }
        return {
            matchOpenBracket: bracketText
        };
    };
    BracketElectricCharacterSupport.prototype._onElectricAutoClose = function (character, context, column) {
        if (!this._complexAutoClosePairs.length) {
            return null;
        }
        var line = context.getLineContent();
        for (var i = 0, len = this._complexAutoClosePairs.length; i < len; i++) {
            var pair = this._complexAutoClosePairs[i];
            // See if the right electric character was pressed
            if (character !== pair.open.charAt(pair.open.length - 1)) {
                continue;
            }
            // check if the full open bracket matches
            var start = column - pair.open.length + 1;
            var actual = line.substring(start - 1, column - 1) + character;
            if (actual !== pair.open) {
                continue;
            }
            var lastTokenIndex = context.findTokenIndexAtOffset(column - 1);
            var lastTokenStandardType = context.getStandardTokenType(lastTokenIndex);
            // If we're in a scope listed in 'notIn', do nothing
            if (!pair.isOK(lastTokenStandardType)) {
                continue;
            }
            // If this line already contains the closing tag, do nothing.
            if (line.indexOf(pair.close, column - 1) >= 0) {
                continue;
            }
            return { appendText: pair.close };
        }
        return null;
    };
    return BracketElectricCharacterSupport;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IndentRulesSupport = /** @class */ (function () {
    function IndentRulesSupport(indentationRules) {
        this._indentationRules = indentationRules;
    }
    IndentRulesSupport.prototype.shouldIncrease = function (text) {
        if (this._indentationRules) {
            if (this._indentationRules.increaseIndentPattern && this._indentationRules.increaseIndentPattern.test(text)) {
                return true;
            }
            // if (this._indentationRules.indentNextLinePattern && this._indentationRules.indentNextLinePattern.test(text)) {
            // 	return true;
            // }
        }
        return false;
    };
    IndentRulesSupport.prototype.shouldDecrease = function (text) {
        if (this._indentationRules && this._indentationRules.decreaseIndentPattern && this._indentationRules.decreaseIndentPattern.test(text)) {
            return true;
        }
        return false;
    };
    IndentRulesSupport.prototype.shouldIndentNextLine = function (text) {
        if (this._indentationRules && this._indentationRules.indentNextLinePattern && this._indentationRules.indentNextLinePattern.test(text)) {
            return true;
        }
        return false;
    };
    IndentRulesSupport.prototype.shouldIgnore = function (text) {
        // the text matches `unIndentedLinePattern`
        if (this._indentationRules && this._indentationRules.unIndentedLinePattern && this._indentationRules.unIndentedLinePattern.test(text)) {
            return true;
        }
        return false;
    };
    IndentRulesSupport.prototype.getIndentMetadata = function (text) {
        var ret = 0;
        if (this.shouldIncrease(text)) {
            ret += 1 /* INCREASE_MASK */;
        }
        if (this.shouldDecrease(text)) {
            ret += 2 /* DECREASE_MASK */;
        }
        if (this.shouldIndentNextLine(text)) {
            ret += 4 /* INDENT_NEXTLINE_MASK */;
        }
        if (this.shouldIgnore(text)) {
            ret += 8 /* UNINDENT_MASK */;
        }
        return ret;
    };
    return IndentRulesSupport;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var OnEnterSupport = /** @class */ (function () {
    function OnEnterSupport(opts) {
        var _this = this;
        opts = opts || {};
        opts.brackets = opts.brackets || [
            ['(', ')'],
            ['{', '}'],
            ['[', ']']
        ];
        this._brackets = [];
        opts.brackets.forEach(function (bracket) {
            var openRegExp = OnEnterSupport._createOpenBracketRegExp(bracket[0]);
            var closeRegExp = OnEnterSupport._createCloseBracketRegExp(bracket[1]);
            if (openRegExp && closeRegExp) {
                _this._brackets.push({
                    open: bracket[0],
                    openRegExp: openRegExp,
                    close: bracket[1],
                    closeRegExp: closeRegExp,
                });
            }
        });
        this._regExpRules = opts.regExpRules || [];
    }
    OnEnterSupport.prototype.onEnter = function (oneLineAboveText, beforeEnterText, afterEnterText) {
        // (1): `regExpRules`
        for (var i = 0, len = this._regExpRules.length; i < len; i++) {
            var rule = this._regExpRules[i];
            var regResult = [{
                    reg: rule.beforeText,
                    text: beforeEnterText
                }, {
                    reg: rule.afterText,
                    text: afterEnterText
                }, {
                    reg: rule.oneLineAboveText,
                    text: oneLineAboveText
                }].every(function (obj) {
                return obj.reg ? obj.reg.test(obj.text) : true;
            });
            if (regResult) {
                return rule.action;
            }
        }
        // (2): Special indent-outdent
        if (beforeEnterText.length > 0 && afterEnterText.length > 0) {
            for (var i = 0, len = this._brackets.length; i < len; i++) {
                var bracket = this._brackets[i];
                if (bracket.openRegExp.test(beforeEnterText) && bracket.closeRegExp.test(afterEnterText)) {
                    return { indentAction: IndentAction.IndentOutdent };
                }
            }
        }
        // (4): Open bracket based logic
        if (beforeEnterText.length > 0) {
            for (var i = 0, len = this._brackets.length; i < len; i++) {
                var bracket = this._brackets[i];
                if (bracket.openRegExp.test(beforeEnterText)) {
                    return { indentAction: IndentAction.Indent };
                }
            }
        }
        return null;
    };
    OnEnterSupport._createOpenBracketRegExp = function (bracket) {
        var str = escapeRegExpCharacters(bracket);
        if (!/\B/.test(str.charAt(0))) {
            str = '\\b' + str;
        }
        str += '\\s*$';
        return OnEnterSupport._safeRegExp(str);
    };
    OnEnterSupport._createCloseBracketRegExp = function (bracket) {
        var str = escapeRegExpCharacters(bracket);
        if (!/\B/.test(str.charAt(str.length - 1))) {
            str = str + '\\b';
        }
        str = '^\\s*' + str;
        return OnEnterSupport._safeRegExp(str);
    };
    OnEnterSupport._safeRegExp = function (def) {
        try {
            return new RegExp(def);
        }
        catch (err) {
            onUnexpectedError(err);
            return null;
        }
    };
    return OnEnterSupport;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var RichEditSupport = /** @class */ (function () {
    function RichEditSupport(languageIdentifier, previous, rawConf) {
        this._languageIdentifier = languageIdentifier;
        this._brackets = null;
        this._electricCharacter = null;
        var prev = null;
        if (previous) {
            prev = previous._conf;
        }
        this._conf = RichEditSupport._mergeConf(prev, rawConf);
        this.onEnter = RichEditSupport._handleOnEnter(this._conf);
        this.comments = RichEditSupport._handleComments(this._conf);
        this.characterPair = new CharacterPairSupport(this._conf);
        this.wordDefinition = this._conf.wordPattern || DEFAULT_WORD_REGEXP;
        this.indentationRules = this._conf.indentationRules;
        if (this._conf.indentationRules) {
            this.indentRulesSupport = new IndentRulesSupport(this._conf.indentationRules);
        }
        this.foldingRules = this._conf.folding || {};
    }
    Object.defineProperty(RichEditSupport.prototype, "brackets", {
        get: function () {
            if (!this._brackets && this._conf.brackets) {
                this._brackets = new RichEditBrackets(this._languageIdentifier, this._conf.brackets);
            }
            return this._brackets;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RichEditSupport.prototype, "electricCharacter", {
        get: function () {
            if (!this._electricCharacter) {
                var autoClosingPairs = [];
                if (this._conf.autoClosingPairs) {
                    autoClosingPairs = this._conf.autoClosingPairs;
                }
                else if (this._conf.brackets) {
                    autoClosingPairs = this._conf.brackets.map(function (b) {
                        return { open: b[0], close: b[1] };
                    });
                }
                this._electricCharacter = new BracketElectricCharacterSupport(this.brackets, autoClosingPairs, this._conf.__electricCharacterSupport);
            }
            return this._electricCharacter;
        },
        enumerable: true,
        configurable: true
    });
    RichEditSupport._mergeConf = function (prev, current) {
        return {
            comments: (prev ? current.comments || prev.comments : current.comments),
            brackets: (prev ? current.brackets || prev.brackets : current.brackets),
            wordPattern: (prev ? current.wordPattern || prev.wordPattern : current.wordPattern),
            indentationRules: (prev ? current.indentationRules || prev.indentationRules : current.indentationRules),
            onEnterRules: (prev ? current.onEnterRules || prev.onEnterRules : current.onEnterRules),
            autoClosingPairs: (prev ? current.autoClosingPairs || prev.autoClosingPairs : current.autoClosingPairs),
            surroundingPairs: (prev ? current.surroundingPairs || prev.surroundingPairs : current.surroundingPairs),
            autoCloseBefore: (prev ? current.autoCloseBefore || prev.autoCloseBefore : current.autoCloseBefore),
            folding: (prev ? current.folding || prev.folding : current.folding),
            __electricCharacterSupport: (prev ? current.__electricCharacterSupport || prev.__electricCharacterSupport : current.__electricCharacterSupport),
        };
    };
    RichEditSupport._handleOnEnter = function (conf) {
        // on enter
        var onEnter = {};
        var empty$$1 = true;
        if (conf.brackets) {
            empty$$1 = false;
            onEnter.brackets = conf.brackets;
        }
        if (conf.indentationRules) {
            empty$$1 = false;
        }
        if (conf.onEnterRules) {
            empty$$1 = false;
            onEnter.regExpRules = conf.onEnterRules;
        }
        if (!empty$$1) {
            return new OnEnterSupport(onEnter);
        }
        return null;
    };
    RichEditSupport._handleComments = function (conf) {
        var commentRule = conf.comments;
        if (!commentRule) {
            return null;
        }
        // comment configuration
        var comments = {};
        if (commentRule.lineComment) {
            comments.lineCommentToken = commentRule.lineComment;
        }
        if (commentRule.blockComment) {
            var _a = commentRule.blockComment, blockStart = _a[0], blockEnd = _a[1];
            comments.blockCommentStartToken = blockStart;
            comments.blockCommentEndToken = blockEnd;
        }
        return comments;
    };
    return RichEditSupport;
}());
var LanguageConfigurationRegistryImpl = /** @class */ (function () {
    function LanguageConfigurationRegistryImpl() {
        this._onDidChange = new Emitter();
        this.onDidChange = this._onDidChange.event;
        this._entries = [];
    }
    LanguageConfigurationRegistryImpl.prototype.register = function (languageIdentifier, configuration) {
        var _this = this;
        var previous = this._getRichEditSupport(languageIdentifier.id);
        var current = new RichEditSupport(languageIdentifier, previous, configuration);
        this._entries[languageIdentifier.id] = current;
        this._onDidChange.fire({ languageIdentifier: languageIdentifier });
        return toDisposable(function () {
            if (_this._entries[languageIdentifier.id] === current) {
                _this._entries[languageIdentifier.id] = previous;
                _this._onDidChange.fire({ languageIdentifier: languageIdentifier });
            }
        });
    };
    LanguageConfigurationRegistryImpl.prototype._getRichEditSupport = function (languageId) {
        return this._entries[languageId] || null;
    };
    // begin electricCharacter
    LanguageConfigurationRegistryImpl.prototype._getElectricCharacterSupport = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return null;
        }
        return value.electricCharacter || null;
    };
    LanguageConfigurationRegistryImpl.prototype.getElectricCharacters = function (languageId) {
        var electricCharacterSupport = this._getElectricCharacterSupport(languageId);
        if (!electricCharacterSupport) {
            return [];
        }
        return electricCharacterSupport.getElectricCharacters();
    };
    /**
     * Should return opening bracket type to match indentation with
     */
    LanguageConfigurationRegistryImpl.prototype.onElectricCharacter = function (character, context, column) {
        var scopedLineTokens = createScopedLineTokens(context, column - 1);
        var electricCharacterSupport = this._getElectricCharacterSupport(scopedLineTokens.languageId);
        if (!electricCharacterSupport) {
            return null;
        }
        return electricCharacterSupport.onElectricCharacter(character, scopedLineTokens, column - scopedLineTokens.firstCharOffset);
    };
    // end electricCharacter
    LanguageConfigurationRegistryImpl.prototype.getComments = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return null;
        }
        return value.comments || null;
    };
    // begin characterPair
    LanguageConfigurationRegistryImpl.prototype._getCharacterPairSupport = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return null;
        }
        return value.characterPair || null;
    };
    LanguageConfigurationRegistryImpl.prototype.getAutoClosingPairs = function (languageId) {
        var characterPairSupport = this._getCharacterPairSupport(languageId);
        if (!characterPairSupport) {
            return [];
        }
        return characterPairSupport.getAutoClosingPairs();
    };
    LanguageConfigurationRegistryImpl.prototype.getAutoCloseBeforeSet = function (languageId) {
        var characterPairSupport = this._getCharacterPairSupport(languageId);
        if (!characterPairSupport) {
            return CharacterPairSupport.DEFAULT_AUTOCLOSE_BEFORE_LANGUAGE_DEFINED;
        }
        return characterPairSupport.getAutoCloseBeforeSet();
    };
    LanguageConfigurationRegistryImpl.prototype.getSurroundingPairs = function (languageId) {
        var characterPairSupport = this._getCharacterPairSupport(languageId);
        if (!characterPairSupport) {
            return [];
        }
        return characterPairSupport.getSurroundingPairs();
    };
    LanguageConfigurationRegistryImpl.prototype.shouldAutoClosePair = function (character, context, column) {
        var scopedLineTokens = createScopedLineTokens(context, column - 1);
        var characterPairSupport = this._getCharacterPairSupport(scopedLineTokens.languageId);
        if (!characterPairSupport) {
            return false;
        }
        return characterPairSupport.shouldAutoClosePair(character, scopedLineTokens, column - scopedLineTokens.firstCharOffset);
    };
    // end characterPair
    LanguageConfigurationRegistryImpl.prototype.getWordDefinition = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return ensureValidWordDefinition(null);
        }
        return ensureValidWordDefinition(value.wordDefinition || null);
    };
    LanguageConfigurationRegistryImpl.prototype.getFoldingRules = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return {};
        }
        return value.foldingRules;
    };
    // begin Indent Rules
    LanguageConfigurationRegistryImpl.prototype.getIndentRulesSupport = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return null;
        }
        return value.indentRulesSupport || null;
    };
    /**
     * Get nearest preceiding line which doesn't match unIndentPattern or contains all whitespace.
     * Result:
     * -1: run into the boundary of embedded languages
     * 0: every line above are invalid
     * else: nearest preceding line of the same language
     */
    LanguageConfigurationRegistryImpl.prototype.getPrecedingValidLine = function (model, lineNumber, indentRulesSupport) {
        var languageID = model.getLanguageIdAtPosition(lineNumber, 0);
        if (lineNumber > 1) {
            var lastLineNumber = lineNumber - 1;
            var resultLineNumber = -1;
            for (lastLineNumber = lineNumber - 1; lastLineNumber >= 1; lastLineNumber--) {
                if (model.getLanguageIdAtPosition(lastLineNumber, 0) !== languageID) {
                    return resultLineNumber;
                }
                var text = model.getLineContent(lastLineNumber);
                if (indentRulesSupport.shouldIgnore(text) || /^\s+$/.test(text) || text === '') {
                    resultLineNumber = lastLineNumber;
                    continue;
                }
                return lastLineNumber;
            }
        }
        return -1;
    };
    /**
     * Get inherited indentation from above lines.
     * 1. Find the nearest preceding line which doesn't match unIndentedLinePattern.
     * 2. If this line matches indentNextLinePattern or increaseIndentPattern, it means that the indent level of `lineNumber` should be 1 greater than this line.
     * 3. If this line doesn't match any indent rules
     *   a. check whether the line above it matches indentNextLinePattern
     *   b. If not, the indent level of this line is the result
     *   c. If so, it means the indent of this line is *temporary*, go upward utill we find a line whose indent is not temporary (the same workflow a -> b -> c).
     * 4. Otherwise, we fail to get an inherited indent from aboves. Return null and we should not touch the indent of `lineNumber`
     *
     * This function only return the inherited indent based on above lines, it doesn't check whether current line should decrease or not.
     */
    LanguageConfigurationRegistryImpl.prototype.getInheritIndentForLine = function (model, lineNumber, honorIntentialIndent) {
        if (honorIntentialIndent === void 0) { honorIntentialIndent = true; }
        var indentRulesSupport = this.getIndentRulesSupport(model.getLanguageIdentifier().id);
        if (!indentRulesSupport) {
            return null;
        }
        if (lineNumber <= 1) {
            return {
                indentation: '',
                action: null
            };
        }
        var precedingUnIgnoredLine = this.getPrecedingValidLine(model, lineNumber, indentRulesSupport);
        if (precedingUnIgnoredLine < 0) {
            return null;
        }
        else if (precedingUnIgnoredLine < 1) {
            return {
                indentation: '',
                action: null
            };
        }
        var precedingUnIgnoredLineContent = model.getLineContent(precedingUnIgnoredLine);
        if (indentRulesSupport.shouldIncrease(precedingUnIgnoredLineContent) || indentRulesSupport.shouldIndentNextLine(precedingUnIgnoredLineContent)) {
            return {
                indentation: getLeadingWhitespace(precedingUnIgnoredLineContent),
                action: IndentAction.Indent,
                line: precedingUnIgnoredLine
            };
        }
        else if (indentRulesSupport.shouldDecrease(precedingUnIgnoredLineContent)) {
            return {
                indentation: getLeadingWhitespace(precedingUnIgnoredLineContent),
                action: null,
                line: precedingUnIgnoredLine
            };
        }
        else {
            // precedingUnIgnoredLine can not be ignored.
            // it doesn't increase indent of following lines
            // it doesn't increase just next line
            // so current line is not affect by precedingUnIgnoredLine
            // and then we should get a correct inheritted indentation from above lines
            if (precedingUnIgnoredLine === 1) {
                return {
                    indentation: getLeadingWhitespace(model.getLineContent(precedingUnIgnoredLine)),
                    action: null,
                    line: precedingUnIgnoredLine
                };
            }
            var previousLine = precedingUnIgnoredLine - 1;
            var previousLineIndentMetadata = indentRulesSupport.getIndentMetadata(model.getLineContent(previousLine));
            if (!(previousLineIndentMetadata & (1 /* INCREASE_MASK */ | 2 /* DECREASE_MASK */)) &&
                (previousLineIndentMetadata & 4 /* INDENT_NEXTLINE_MASK */)) {
                var stopLine = 0;
                for (var i = previousLine - 1; i > 0; i--) {
                    if (indentRulesSupport.shouldIndentNextLine(model.getLineContent(i))) {
                        continue;
                    }
                    stopLine = i;
                    break;
                }
                return {
                    indentation: getLeadingWhitespace(model.getLineContent(stopLine + 1)),
                    action: null,
                    line: stopLine + 1
                };
            }
            if (honorIntentialIndent) {
                return {
                    indentation: getLeadingWhitespace(model.getLineContent(precedingUnIgnoredLine)),
                    action: null,
                    line: precedingUnIgnoredLine
                };
            }
            else {
                // search from precedingUnIgnoredLine until we find one whose indent is not temporary
                for (var i = precedingUnIgnoredLine; i > 0; i--) {
                    var lineContent = model.getLineContent(i);
                    if (indentRulesSupport.shouldIncrease(lineContent)) {
                        return {
                            indentation: getLeadingWhitespace(lineContent),
                            action: IndentAction.Indent,
                            line: i
                        };
                    }
                    else if (indentRulesSupport.shouldIndentNextLine(lineContent)) {
                        var stopLine = 0;
                        for (var j = i - 1; j > 0; j--) {
                            if (indentRulesSupport.shouldIndentNextLine(model.getLineContent(i))) {
                                continue;
                            }
                            stopLine = j;
                            break;
                        }
                        return {
                            indentation: getLeadingWhitespace(model.getLineContent(stopLine + 1)),
                            action: null,
                            line: stopLine + 1
                        };
                    }
                    else if (indentRulesSupport.shouldDecrease(lineContent)) {
                        return {
                            indentation: getLeadingWhitespace(lineContent),
                            action: null,
                            line: i
                        };
                    }
                }
                return {
                    indentation: getLeadingWhitespace(model.getLineContent(1)),
                    action: null,
                    line: 1
                };
            }
        }
    };
    LanguageConfigurationRegistryImpl.prototype.getGoodIndentForLine = function (virtualModel, languageId, lineNumber, indentConverter) {
        var indentRulesSupport = this.getIndentRulesSupport(languageId);
        if (!indentRulesSupport) {
            return null;
        }
        var indent = this.getInheritIndentForLine(virtualModel, lineNumber);
        var lineContent = virtualModel.getLineContent(lineNumber);
        if (indent) {
            var inheritLine = indent.line;
            if (inheritLine !== undefined) {
                var onEnterSupport = this._getOnEnterSupport(languageId);
                var enterResult = null;
                try {
                    if (onEnterSupport) {
                        enterResult = onEnterSupport.onEnter('', virtualModel.getLineContent(inheritLine), '');
                    }
                }
                catch (e) {
                    onUnexpectedError(e);
                }
                if (enterResult) {
                    var indentation = getLeadingWhitespace(virtualModel.getLineContent(inheritLine));
                    if (enterResult.removeText) {
                        indentation = indentation.substring(0, indentation.length - enterResult.removeText);
                    }
                    if ((enterResult.indentAction === IndentAction.Indent) ||
                        (enterResult.indentAction === IndentAction.IndentOutdent)) {
                        indentation = indentConverter.shiftIndent(indentation);
                    }
                    else if (enterResult.indentAction === IndentAction.Outdent) {
                        indentation = indentConverter.unshiftIndent(indentation);
                    }
                    if (indentRulesSupport.shouldDecrease(lineContent)) {
                        indentation = indentConverter.unshiftIndent(indentation);
                    }
                    if (enterResult.appendText) {
                        indentation += enterResult.appendText;
                    }
                    return getLeadingWhitespace(indentation);
                }
            }
            if (indentRulesSupport.shouldDecrease(lineContent)) {
                if (indent.action === IndentAction.Indent) {
                    return indent.indentation;
                }
                else {
                    return indentConverter.unshiftIndent(indent.indentation);
                }
            }
            else {
                if (indent.action === IndentAction.Indent) {
                    return indentConverter.shiftIndent(indent.indentation);
                }
                else {
                    return indent.indentation;
                }
            }
        }
        return null;
    };
    LanguageConfigurationRegistryImpl.prototype.getIndentForEnter = function (model, range, indentConverter, autoIndent) {
        model.forceTokenization(range.startLineNumber);
        var lineTokens = model.getLineTokens(range.startLineNumber);
        var beforeEnterText;
        var afterEnterText;
        var scopedLineTokens = createScopedLineTokens(lineTokens, range.startColumn - 1);
        var scopedLineText = scopedLineTokens.getLineContent();
        var embeddedLanguage = false;
        if (scopedLineTokens.firstCharOffset > 0 && lineTokens.getLanguageId(0) !== scopedLineTokens.languageId) {
            // we are in the embeded language content
            embeddedLanguage = true; // if embeddedLanguage is true, then we don't touch the indentation of current line
            beforeEnterText = scopedLineText.substr(0, range.startColumn - 1 - scopedLineTokens.firstCharOffset);
        }
        else {
            beforeEnterText = lineTokens.getLineContent().substring(0, range.startColumn - 1);
        }
        if (range.isEmpty()) {
            afterEnterText = scopedLineText.substr(range.startColumn - 1 - scopedLineTokens.firstCharOffset);
        }
        else {
            var endScopedLineTokens = this.getScopedLineTokens(model, range.endLineNumber, range.endColumn);
            afterEnterText = endScopedLineTokens.getLineContent().substr(range.endColumn - 1 - scopedLineTokens.firstCharOffset);
        }
        var indentRulesSupport = this.getIndentRulesSupport(scopedLineTokens.languageId);
        if (!indentRulesSupport) {
            return null;
        }
        var beforeEnterResult = beforeEnterText;
        var beforeEnterIndent = getLeadingWhitespace(beforeEnterText);
        if (!autoIndent && !embeddedLanguage) {
            var beforeEnterIndentAction = this.getInheritIndentForLine(model, range.startLineNumber);
            if (indentRulesSupport.shouldDecrease(beforeEnterText)) {
                if (beforeEnterIndentAction) {
                    beforeEnterIndent = beforeEnterIndentAction.indentation;
                    if (beforeEnterIndentAction.action !== IndentAction.Indent) {
                        beforeEnterIndent = indentConverter.unshiftIndent(beforeEnterIndent);
                    }
                }
            }
            beforeEnterResult = beforeEnterIndent + ltrim(ltrim(beforeEnterText, ' '), '\t');
        }
        var virtualModel = {
            getLineTokens: function (lineNumber) {
                return model.getLineTokens(lineNumber);
            },
            getLanguageIdentifier: function () {
                return model.getLanguageIdentifier();
            },
            getLanguageIdAtPosition: function (lineNumber, column) {
                return model.getLanguageIdAtPosition(lineNumber, column);
            },
            getLineContent: function (lineNumber) {
                if (lineNumber === range.startLineNumber) {
                    return beforeEnterResult;
                }
                else {
                    return model.getLineContent(lineNumber);
                }
            }
        };
        var currentLineIndent = getLeadingWhitespace(lineTokens.getLineContent());
        var afterEnterAction = this.getInheritIndentForLine(virtualModel, range.startLineNumber + 1);
        if (!afterEnterAction) {
            var beforeEnter = embeddedLanguage ? currentLineIndent : beforeEnterIndent;
            return {
                beforeEnter: beforeEnter,
                afterEnter: beforeEnter
            };
        }
        var afterEnterIndent = embeddedLanguage ? currentLineIndent : afterEnterAction.indentation;
        if (afterEnterAction.action === IndentAction.Indent) {
            afterEnterIndent = indentConverter.shiftIndent(afterEnterIndent);
        }
        if (indentRulesSupport.shouldDecrease(afterEnterText)) {
            afterEnterIndent = indentConverter.unshiftIndent(afterEnterIndent);
        }
        return {
            beforeEnter: embeddedLanguage ? currentLineIndent : beforeEnterIndent,
            afterEnter: afterEnterIndent
        };
    };
    /**
     * We should always allow intentional indentation. It means, if users change the indentation of `lineNumber` and the content of
     * this line doesn't match decreaseIndentPattern, we should not adjust the indentation.
     */
    LanguageConfigurationRegistryImpl.prototype.getIndentActionForType = function (model, range, ch, indentConverter) {
        var scopedLineTokens = this.getScopedLineTokens(model, range.startLineNumber, range.startColumn);
        var indentRulesSupport = this.getIndentRulesSupport(scopedLineTokens.languageId);
        if (!indentRulesSupport) {
            return null;
        }
        var scopedLineText = scopedLineTokens.getLineContent();
        var beforeTypeText = scopedLineText.substr(0, range.startColumn - 1 - scopedLineTokens.firstCharOffset);
        var afterTypeText;
        // selection support
        if (range.isEmpty()) {
            afterTypeText = scopedLineText.substr(range.startColumn - 1 - scopedLineTokens.firstCharOffset);
        }
        else {
            var endScopedLineTokens = this.getScopedLineTokens(model, range.endLineNumber, range.endColumn);
            afterTypeText = endScopedLineTokens.getLineContent().substr(range.endColumn - 1 - scopedLineTokens.firstCharOffset);
        }
        // If previous content already matches decreaseIndentPattern, it means indentation of this line should already be adjusted
        // Users might change the indentation by purpose and we should honor that instead of readjusting.
        if (!indentRulesSupport.shouldDecrease(beforeTypeText + afterTypeText) && indentRulesSupport.shouldDecrease(beforeTypeText + ch + afterTypeText)) {
            // after typing `ch`, the content matches decreaseIndentPattern, we should adjust the indent to a good manner.
            // 1. Get inherited indent action
            var r = this.getInheritIndentForLine(model, range.startLineNumber, false);
            if (!r) {
                return null;
            }
            var indentation = r.indentation;
            if (r.action !== IndentAction.Indent) {
                indentation = indentConverter.unshiftIndent(indentation);
            }
            return indentation;
        }
        return null;
    };
    LanguageConfigurationRegistryImpl.prototype.getIndentMetadata = function (model, lineNumber) {
        var indentRulesSupport = this.getIndentRulesSupport(model.getLanguageIdentifier().id);
        if (!indentRulesSupport) {
            return null;
        }
        if (lineNumber < 1 || lineNumber > model.getLineCount()) {
            return null;
        }
        return indentRulesSupport.getIndentMetadata(model.getLineContent(lineNumber));
    };
    // end Indent Rules
    // begin onEnter
    LanguageConfigurationRegistryImpl.prototype._getOnEnterSupport = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return null;
        }
        return value.onEnter || null;
    };
    LanguageConfigurationRegistryImpl.prototype.getRawEnterActionAtPosition = function (model, lineNumber, column) {
        var r = this.getEnterAction(model, new Range(lineNumber, column, lineNumber, column));
        return r ? r.enterAction : null;
    };
    LanguageConfigurationRegistryImpl.prototype.getEnterAction = function (model, range) {
        var indentation = this.getIndentationAtPosition(model, range.startLineNumber, range.startColumn);
        var scopedLineTokens = this.getScopedLineTokens(model, range.startLineNumber, range.startColumn);
        var onEnterSupport = this._getOnEnterSupport(scopedLineTokens.languageId);
        if (!onEnterSupport) {
            return null;
        }
        var scopedLineText = scopedLineTokens.getLineContent();
        var beforeEnterText = scopedLineText.substr(0, range.startColumn - 1 - scopedLineTokens.firstCharOffset);
        var afterEnterText;
        // selection support
        if (range.isEmpty()) {
            afterEnterText = scopedLineText.substr(range.startColumn - 1 - scopedLineTokens.firstCharOffset);
        }
        else {
            var endScopedLineTokens = this.getScopedLineTokens(model, range.endLineNumber, range.endColumn);
            afterEnterText = endScopedLineTokens.getLineContent().substr(range.endColumn - 1 - scopedLineTokens.firstCharOffset);
        }
        var lineNumber = range.startLineNumber;
        var oneLineAboveText = '';
        if (lineNumber > 1 && scopedLineTokens.firstCharOffset === 0) {
            // This is not the first line and the entire line belongs to this mode
            var oneLineAboveScopedLineTokens = this.getScopedLineTokens(model, lineNumber - 1);
            if (oneLineAboveScopedLineTokens.languageId === scopedLineTokens.languageId) {
                // The line above ends with text belonging to the same mode
                oneLineAboveText = oneLineAboveScopedLineTokens.getLineContent();
            }
        }
        var enterResult = null;
        try {
            enterResult = onEnterSupport.onEnter(oneLineAboveText, beforeEnterText, afterEnterText);
        }
        catch (e) {
            onUnexpectedError(e);
        }
        if (!enterResult) {
            return null;
        }
        else {
            // Here we add `\t` to appendText first because enterAction is leveraging appendText and removeText to change indentation.
            if (!enterResult.appendText) {
                if ((enterResult.indentAction === IndentAction.Indent) ||
                    (enterResult.indentAction === IndentAction.IndentOutdent)) {
                    enterResult.appendText = '\t';
                }
                else {
                    enterResult.appendText = '';
                }
            }
        }
        if (enterResult.removeText) {
            indentation = indentation.substring(0, indentation.length - enterResult.removeText);
        }
        return {
            enterAction: enterResult,
            indentation: indentation,
        };
    };
    LanguageConfigurationRegistryImpl.prototype.getIndentationAtPosition = function (model, lineNumber, column) {
        var lineText = model.getLineContent(lineNumber);
        var indentation = getLeadingWhitespace(lineText);
        if (indentation.length > column - 1) {
            indentation = indentation.substring(0, column - 1);
        }
        return indentation;
    };
    LanguageConfigurationRegistryImpl.prototype.getScopedLineTokens = function (model, lineNumber, columnNumber) {
        model.forceTokenization(lineNumber);
        var lineTokens = model.getLineTokens(lineNumber);
        var column = (typeof columnNumber === 'undefined' ? model.getLineMaxColumn(lineNumber) - 1 : columnNumber - 1);
        var scopedLineTokens = createScopedLineTokens(lineTokens, column);
        return scopedLineTokens;
    };
    // end onEnter
    LanguageConfigurationRegistryImpl.prototype.getBracketsSupport = function (languageId) {
        var value = this._getRichEditSupport(languageId);
        if (!value) {
            return null;
        }
        return value.brackets || null;
    };
    return LanguageConfigurationRegistryImpl;
}());
var LanguageConfigurationRegistry = new LanguageConfigurationRegistryImpl();

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$a = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CHEAP_TOKENIZATION_LENGTH_LIMIT = 2048;
function createTextBufferBuilder() {
    return new PieceTreeTextBufferBuilder();
}
function createTextBufferFactory(text) {
    var builder = createTextBufferBuilder();
    builder.acceptChunk(text);
    return builder.finish();
}
function createTextBuffer(value, defaultEOL) {
    var factory = (typeof value === 'string' ? createTextBufferFactory(value) : value);
    return factory.create(defaultEOL);
}
var MODEL_ID = 0;
/**
 * Produces 'a'-'z', followed by 'A'-'Z'... followed by 'a'-'z', etc.
 */
function singleLetter(result) {
    var LETTERS_CNT = (90 /* Z */ - 65 /* A */ + 1);
    result = result % (2 * LETTERS_CNT);
    if (result < LETTERS_CNT) {
        return String.fromCharCode(97 /* a */ + result);
    }
    return String.fromCharCode(65 /* A */ + result - LETTERS_CNT);
}
var LIMIT_FIND_COUNT$1 = 999;
var LONG_LINE_BOUNDARY = 10000;
var invalidFunc = function () { throw new Error("Invalid change accessor"); };
var TextModel = /** @class */ (function (_super) {
    __extends$a(TextModel, _super);
    //#endregion
    function TextModel(source, creationOptions, languageIdentifier, associatedResource) {
        if (associatedResource === void 0) { associatedResource = null; }
        var _this = _super.call(this) || this;
        //#region Events
        _this._onWillDispose = _this._register(new Emitter());
        _this.onWillDispose = _this._onWillDispose.event;
        _this._onDidChangeDecorations = _this._register(new DidChangeDecorationsEmitter());
        _this.onDidChangeDecorations = _this._onDidChangeDecorations.event;
        _this._onDidChangeLanguage = _this._register(new Emitter());
        _this.onDidChangeLanguage = _this._onDidChangeLanguage.event;
        _this._onDidChangeLanguageConfiguration = _this._register(new Emitter());
        _this.onDidChangeLanguageConfiguration = _this._onDidChangeLanguageConfiguration.event;
        _this._onDidChangeTokens = _this._register(new Emitter());
        _this.onDidChangeTokens = _this._onDidChangeTokens.event;
        _this._onDidChangeOptions = _this._register(new Emitter());
        _this.onDidChangeOptions = _this._onDidChangeOptions.event;
        _this._eventEmitter = _this._register(new DidChangeContentEmitter());
        // Generate a new unique model id
        MODEL_ID++;
        _this.id = '$model' + MODEL_ID;
        _this.isForSimpleWidget = creationOptions.isForSimpleWidget;
        if (typeof associatedResource === 'undefined' || associatedResource === null) {
            _this._associatedResource = URI.parse('inmemory://model/' + MODEL_ID);
        }
        else {
            _this._associatedResource = associatedResource;
        }
        _this._attachedEditorCount = 0;
        _this._buffer = createTextBuffer(source, creationOptions.defaultEOL);
        _this._options = TextModel.resolveOptions(_this._buffer, creationOptions);
        var bufferLineCount = _this._buffer.getLineCount();
        var bufferTextLength = _this._buffer.getValueLengthInRange(new Range(1, 1, bufferLineCount, _this._buffer.getLineLength(bufferLineCount) + 1), 0 /* TextDefined */);
        // !!! Make a decision in the ctor and permanently respect this decision !!!
        // If a model is too large at construction time, it will never get tokenized,
        // under no circumstances.
        if (creationOptions.largeFileOptimizations) {
            _this._isTooLargeForTokenization = ((bufferTextLength > TextModel.LARGE_FILE_SIZE_THRESHOLD)
                || (bufferLineCount > TextModel.LARGE_FILE_LINE_COUNT_THRESHOLD));
        }
        else {
            _this._isTooLargeForTokenization = false;
        }
        _this._isTooLargeForSyncing = (bufferTextLength > TextModel.MODEL_SYNC_LIMIT);
        _this._setVersionId(1);
        _this._isDisposed = false;
        _this._isDisposing = false;
        _this._languageIdentifier = languageIdentifier || NULL_LANGUAGE_IDENTIFIER;
        _this._tokenizationListener = TokenizationRegistry.onDidChange(function (e) {
            if (e.changedLanguages.indexOf(_this._languageIdentifier.language) === -1) {
                return;
            }
            _this._resetTokenizationState();
            _this.emitModelTokensChangedEvent({
                ranges: [{
                        fromLineNumber: 1,
                        toLineNumber: _this.getLineCount()
                    }]
            });
            if (_this._shouldAutoTokenize()) {
                _this._warmUpTokens();
            }
        });
        _this._revalidateTokensTimeout = -1;
        _this._languageRegistryListener = LanguageConfigurationRegistry.onDidChange(function (e) {
            if (e.languageIdentifier.id === _this._languageIdentifier.id) {
                _this._onDidChangeLanguageConfiguration.fire({});
            }
        });
        _this._resetTokenizationState();
        _this._instanceId = singleLetter(MODEL_ID);
        _this._lastDecorationId = 0;
        _this._decorations = Object.create(null);
        _this._decorationsTree = new DecorationsTrees();
        _this._commandManager = new EditStack(_this);
        _this._isUndoing = false;
        _this._isRedoing = false;
        _this._trimAutoWhitespaceLines = null;
        return _this;
    }
    TextModel.createFromString = function (text, options, languageIdentifier, uri) {
        if (options === void 0) { options = TextModel.DEFAULT_CREATION_OPTIONS; }
        if (languageIdentifier === void 0) { languageIdentifier = null; }
        if (uri === void 0) { uri = null; }
        return new TextModel(text, options, languageIdentifier, uri);
    };
    TextModel.resolveOptions = function (textBuffer, options) {
        if (options.detectIndentation) {
            var guessedIndentation = guessIndentation(textBuffer, options.tabSize, options.insertSpaces);
            return new TextModelResolvedOptions({
                tabSize: guessedIndentation.tabSize,
                insertSpaces: guessedIndentation.insertSpaces,
                trimAutoWhitespace: options.trimAutoWhitespace,
                defaultEOL: options.defaultEOL
            });
        }
        return new TextModelResolvedOptions({
            tabSize: options.tabSize,
            insertSpaces: options.insertSpaces,
            trimAutoWhitespace: options.trimAutoWhitespace,
            defaultEOL: options.defaultEOL
        });
    };
    TextModel.prototype.onDidChangeRawContentFast = function (listener) {
        return this._eventEmitter.fastEvent(function (e) { return listener(e.rawContentChangedEvent); });
    };
    TextModel.prototype.onDidChangeRawContent = function (listener) {
        return this._eventEmitter.slowEvent(function (e) { return listener(e.rawContentChangedEvent); });
    };
    TextModel.prototype.onDidChangeContent = function (listener) {
        return this._eventEmitter.slowEvent(function (e) { return listener(e.contentChangedEvent); });
    };
    TextModel.prototype.dispose = function () {
        this._isDisposing = true;
        this._onWillDispose.fire();
        this._tokenizationListener.dispose();
        this._languageRegistryListener.dispose();
        this._clearTimers();
        this._isDisposed = true;
        // Null out members, such that any use of a disposed model will throw exceptions sooner rather than later
        _super.prototype.dispose.call(this);
        this._isDisposing = false;
    };
    TextModel.prototype._assertNotDisposed = function () {
        if (this._isDisposed) {
            throw new Error('Model is disposed!');
        }
    };
    TextModel.prototype._emitContentChangedEvent = function (rawChange, change) {
        if (this._isDisposing) {
            // Do not confuse listeners by emitting any event after disposing
            return;
        }
        this._eventEmitter.fire(new InternalModelContentChangeEvent(rawChange, change));
    };
    TextModel.prototype.setValue = function (value) {
        this._assertNotDisposed();
        if (value === null) {
            // There's nothing to do
            return;
        }
        var textBuffer = createTextBuffer(value, this._options.defaultEOL);
        this.setValueFromTextBuffer(textBuffer);
    };
    TextModel.prototype._createContentChanged2 = function (range, rangeOffset, rangeLength, text, isUndoing, isRedoing, isFlush) {
        return {
            changes: [{
                    range: range,
                    rangeOffset: rangeOffset,
                    rangeLength: rangeLength,
                    text: text,
                }],
            eol: this._buffer.getEOL(),
            versionId: this.getVersionId(),
            isUndoing: isUndoing,
            isRedoing: isRedoing,
            isFlush: isFlush
        };
    };
    TextModel.prototype.setValueFromTextBuffer = function (textBuffer) {
        this._assertNotDisposed();
        if (textBuffer === null) {
            // There's nothing to do
            return;
        }
        var oldFullModelRange = this.getFullModelRange();
        var oldModelValueLength = this.getValueLengthInRange(oldFullModelRange);
        var endLineNumber = this.getLineCount();
        var endColumn = this.getLineMaxColumn(endLineNumber);
        this._buffer = textBuffer;
        this._increaseVersionId();
        // Cancel tokenization, clear all tokens and begin tokenizing
        this._resetTokenizationState();
        // Destroy all my decorations
        this._decorations = Object.create(null);
        this._decorationsTree = new DecorationsTrees();
        // Destroy my edit history and settings
        this._commandManager = new EditStack(this);
        this._trimAutoWhitespaceLines = null;
        this._emitContentChangedEvent(new ModelRawContentChangedEvent([
            new ModelRawFlush()
        ], this._versionId, false, false), this._createContentChanged2(new Range(1, 1, endLineNumber, endColumn), 0, oldModelValueLength, this.getValue(), false, false, true));
    };
    TextModel.prototype.setEOL = function (eol) {
        this._assertNotDisposed();
        var newEOL = (eol === 1 /* CRLF */ ? '\r\n' : '\n');
        if (this._buffer.getEOL() === newEOL) {
            // Nothing to do
            return;
        }
        var oldFullModelRange = this.getFullModelRange();
        var oldModelValueLength = this.getValueLengthInRange(oldFullModelRange);
        var endLineNumber = this.getLineCount();
        var endColumn = this.getLineMaxColumn(endLineNumber);
        this._onBeforeEOLChange();
        this._buffer.setEOL(newEOL);
        this._increaseVersionId();
        this._onAfterEOLChange();
        this._emitContentChangedEvent(new ModelRawContentChangedEvent([
            new ModelRawEOLChanged()
        ], this._versionId, false, false), this._createContentChanged2(new Range(1, 1, endLineNumber, endColumn), 0, oldModelValueLength, this.getValue(), false, false, false));
    };
    TextModel.prototype._onBeforeEOLChange = function () {
        // Ensure all decorations get their `range` set.
        var versionId = this.getVersionId();
        var allDecorations = this._decorationsTree.search(0, false, false, versionId);
        this._ensureNodesHaveRanges(allDecorations);
    };
    TextModel.prototype._onAfterEOLChange = function () {
        // Transform back `range` to offsets
        var versionId = this.getVersionId();
        var allDecorations = this._decorationsTree.collectNodesPostOrder();
        for (var i = 0, len = allDecorations.length; i < len; i++) {
            var node = allDecorations[i];
            var delta = node.cachedAbsoluteStart - node.start;
            var startOffset = this._buffer.getOffsetAt(node.range.startLineNumber, node.range.startColumn);
            var endOffset = this._buffer.getOffsetAt(node.range.endLineNumber, node.range.endColumn);
            node.cachedAbsoluteStart = startOffset;
            node.cachedAbsoluteEnd = endOffset;
            node.cachedVersionId = versionId;
            node.start = startOffset - delta;
            node.end = endOffset - delta;
            recomputeMaxEnd(node);
        }
    };
    TextModel.prototype._resetTokenizationState = function () {
        this._clearTimers();
        var tokenizationSupport = (this._isTooLargeForTokenization
            ? null
            : TokenizationRegistry.get(this._languageIdentifier.language));
        this._tokens = new ModelLinesTokens(this._languageIdentifier, tokenizationSupport);
        this._beginBackgroundTokenization();
    };
    TextModel.prototype._clearTimers = function () {
        if (this._revalidateTokensTimeout !== -1) {
            clearTimeout(this._revalidateTokensTimeout);
            this._revalidateTokensTimeout = -1;
        }
    };
    TextModel.prototype.onBeforeAttached = function () {
        this._attachedEditorCount++;
        // Warm up tokens for the editor
        this._warmUpTokens();
    };
    TextModel.prototype.onBeforeDetached = function () {
        this._attachedEditorCount--;
    };
    TextModel.prototype._shouldAutoTokenize = function () {
        return this.isAttachedToEditor();
    };
    TextModel.prototype.isAttachedToEditor = function () {
        return this._attachedEditorCount > 0;
    };
    TextModel.prototype.getAttachedEditorCount = function () {
        return this._attachedEditorCount;
    };
    TextModel.prototype.isTooLargeForSyncing = function () {
        return this._isTooLargeForSyncing;
    };
    TextModel.prototype.isTooLargeForTokenization = function () {
        return this._isTooLargeForTokenization;
    };
    TextModel.prototype.isDisposed = function () {
        return this._isDisposed;
    };
    TextModel.prototype.isDominatedByLongLines = function () {
        this._assertNotDisposed();
        if (this.isTooLargeForTokenization()) {
            // Cannot word wrap huge files anyways, so it doesn't really matter
            return false;
        }
        var smallLineCharCount = 0;
        var longLineCharCount = 0;
        var lineCount = this._buffer.getLineCount();
        for (var lineNumber = 1; lineNumber <= lineCount; lineNumber++) {
            var lineLength = this._buffer.getLineLength(lineNumber);
            if (lineLength >= LONG_LINE_BOUNDARY) {
                longLineCharCount += lineLength;
            }
            else {
                smallLineCharCount += lineLength;
            }
        }
        return (longLineCharCount > smallLineCharCount);
    };
    Object.defineProperty(TextModel.prototype, "uri", {
        get: function () {
            return this._associatedResource;
        },
        enumerable: true,
        configurable: true
    });
    //#region Options
    TextModel.prototype.getOptions = function () {
        this._assertNotDisposed();
        return this._options;
    };
    TextModel.prototype.updateOptions = function (_newOpts) {
        this._assertNotDisposed();
        var tabSize = (typeof _newOpts.tabSize !== 'undefined') ? _newOpts.tabSize : this._options.tabSize;
        var insertSpaces = (typeof _newOpts.insertSpaces !== 'undefined') ? _newOpts.insertSpaces : this._options.insertSpaces;
        var trimAutoWhitespace = (typeof _newOpts.trimAutoWhitespace !== 'undefined') ? _newOpts.trimAutoWhitespace : this._options.trimAutoWhitespace;
        var newOpts = new TextModelResolvedOptions({
            tabSize: tabSize,
            insertSpaces: insertSpaces,
            defaultEOL: this._options.defaultEOL,
            trimAutoWhitespace: trimAutoWhitespace
        });
        if (this._options.equals(newOpts)) {
            return;
        }
        var e = this._options.createChangeEvent(newOpts);
        this._options = newOpts;
        this._onDidChangeOptions.fire(e);
    };
    TextModel.prototype.detectIndentation = function (defaultInsertSpaces, defaultTabSize) {
        this._assertNotDisposed();
        var guessedIndentation = guessIndentation(this._buffer, defaultTabSize, defaultInsertSpaces);
        this.updateOptions({
            insertSpaces: guessedIndentation.insertSpaces,
            tabSize: guessedIndentation.tabSize
        });
    };
    TextModel._normalizeIndentationFromWhitespace = function (str, tabSize, insertSpaces) {
        var spacesCnt = 0;
        for (var i = 0; i < str.length; i++) {
            if (str.charAt(i) === '\t') {
                spacesCnt += tabSize;
            }
            else {
                spacesCnt++;
            }
        }
        var result = '';
        if (!insertSpaces) {
            var tabsCnt = Math.floor(spacesCnt / tabSize);
            spacesCnt = spacesCnt % tabSize;
            for (var i = 0; i < tabsCnt; i++) {
                result += '\t';
            }
        }
        for (var i = 0; i < spacesCnt; i++) {
            result += ' ';
        }
        return result;
    };
    TextModel.normalizeIndentation = function (str, tabSize, insertSpaces) {
        var firstNonWhitespaceIndex$$1 = firstNonWhitespaceIndex(str);
        if (firstNonWhitespaceIndex$$1 === -1) {
            firstNonWhitespaceIndex$$1 = str.length;
        }
        return TextModel._normalizeIndentationFromWhitespace(str.substring(0, firstNonWhitespaceIndex$$1), tabSize, insertSpaces) + str.substring(firstNonWhitespaceIndex$$1);
    };
    TextModel.prototype.normalizeIndentation = function (str) {
        this._assertNotDisposed();
        return TextModel.normalizeIndentation(str, this._options.tabSize, this._options.insertSpaces);
    };
    TextModel.prototype.getOneIndent = function () {
        this._assertNotDisposed();
        var tabSize = this._options.tabSize;
        var insertSpaces = this._options.insertSpaces;
        if (insertSpaces) {
            var result = '';
            for (var i = 0; i < tabSize; i++) {
                result += ' ';
            }
            return result;
        }
        else {
            return '\t';
        }
    };
    //#endregion
    //#region Reading
    TextModel.prototype.getVersionId = function () {
        this._assertNotDisposed();
        return this._versionId;
    };
    TextModel.prototype.mightContainRTL = function () {
        return this._buffer.mightContainRTL();
    };
    TextModel.prototype.mightContainNonBasicASCII = function () {
        return this._buffer.mightContainNonBasicASCII();
    };
    TextModel.prototype.getAlternativeVersionId = function () {
        this._assertNotDisposed();
        return this._alternativeVersionId;
    };
    TextModel.prototype.getOffsetAt = function (rawPosition) {
        this._assertNotDisposed();
        var position = this._validatePosition(rawPosition.lineNumber, rawPosition.column, false);
        return this._buffer.getOffsetAt(position.lineNumber, position.column);
    };
    TextModel.prototype.getPositionAt = function (rawOffset) {
        this._assertNotDisposed();
        var offset = (Math.min(this._buffer.getLength(), Math.max(0, rawOffset)));
        return this._buffer.getPositionAt(offset);
    };
    TextModel.prototype._increaseVersionId = function () {
        this._setVersionId(this._versionId + 1);
    };
    TextModel.prototype._setVersionId = function (newVersionId) {
        this._versionId = newVersionId;
        this._alternativeVersionId = this._versionId;
    };
    TextModel.prototype._overwriteAlternativeVersionId = function (newAlternativeVersionId) {
        this._alternativeVersionId = newAlternativeVersionId;
    };
    TextModel.prototype.getValue = function (eol, preserveBOM) {
        if (preserveBOM === void 0) { preserveBOM = false; }
        this._assertNotDisposed();
        var fullModelRange = this.getFullModelRange();
        var fullModelValue = this.getValueInRange(fullModelRange, eol);
        if (preserveBOM) {
            return this._buffer.getBOM() + fullModelValue;
        }
        return fullModelValue;
    };
    TextModel.prototype.getValueLength = function (eol, preserveBOM) {
        if (preserveBOM === void 0) { preserveBOM = false; }
        this._assertNotDisposed();
        var fullModelRange = this.getFullModelRange();
        var fullModelValue = this.getValueLengthInRange(fullModelRange, eol);
        if (preserveBOM) {
            return this._buffer.getBOM().length + fullModelValue;
        }
        return fullModelValue;
    };
    TextModel.prototype.getValueInRange = function (rawRange, eol) {
        if (eol === void 0) { eol = 0 /* TextDefined */; }
        this._assertNotDisposed();
        return this._buffer.getValueInRange(this.validateRange(rawRange), eol);
    };
    TextModel.prototype.getValueLengthInRange = function (rawRange, eol) {
        if (eol === void 0) { eol = 0 /* TextDefined */; }
        this._assertNotDisposed();
        return this._buffer.getValueLengthInRange(this.validateRange(rawRange), eol);
    };
    TextModel.prototype.getLineCount = function () {
        this._assertNotDisposed();
        return this._buffer.getLineCount();
    };
    TextModel.prototype.getLineContent = function (lineNumber) {
        this._assertNotDisposed();
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            throw new Error('Illegal value for lineNumber');
        }
        return this._buffer.getLineContent(lineNumber);
    };
    TextModel.prototype.getLineLength = function (lineNumber) {
        this._assertNotDisposed();
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            throw new Error('Illegal value for lineNumber');
        }
        return this._buffer.getLineLength(lineNumber);
    };
    TextModel.prototype.getLinesContent = function () {
        this._assertNotDisposed();
        return this._buffer.getLinesContent();
    };
    TextModel.prototype.getEOL = function () {
        this._assertNotDisposed();
        return this._buffer.getEOL();
    };
    TextModel.prototype.getLineMinColumn = function (lineNumber) {
        this._assertNotDisposed();
        return 1;
    };
    TextModel.prototype.getLineMaxColumn = function (lineNumber) {
        this._assertNotDisposed();
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            throw new Error('Illegal value for lineNumber');
        }
        return this._buffer.getLineLength(lineNumber) + 1;
    };
    TextModel.prototype.getLineFirstNonWhitespaceColumn = function (lineNumber) {
        this._assertNotDisposed();
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            throw new Error('Illegal value for lineNumber');
        }
        return this._buffer.getLineFirstNonWhitespaceColumn(lineNumber);
    };
    TextModel.prototype.getLineLastNonWhitespaceColumn = function (lineNumber) {
        this._assertNotDisposed();
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            throw new Error('Illegal value for lineNumber');
        }
        return this._buffer.getLineLastNonWhitespaceColumn(lineNumber);
    };
    /**
     * Validates `range` is within buffer bounds, but allows it to sit in between surrogate pairs, etc.
     * Will try to not allocate if possible.
     */
    TextModel.prototype._validateRangeRelaxedNoAllocations = function (range) {
        var linesCount = this._buffer.getLineCount();
        var initialStartLineNumber = range.startLineNumber;
        var initialStartColumn = range.startColumn;
        var startLineNumber;
        var startColumn;
        if (initialStartLineNumber < 1) {
            startLineNumber = 1;
            startColumn = 1;
        }
        else if (initialStartLineNumber > linesCount) {
            startLineNumber = linesCount;
            startColumn = this.getLineMaxColumn(startLineNumber);
        }
        else {
            startLineNumber = initialStartLineNumber | 0;
            if (initialStartColumn <= 1) {
                startColumn = 1;
            }
            else {
                var maxColumn = this.getLineMaxColumn(startLineNumber);
                if (initialStartColumn >= maxColumn) {
                    startColumn = maxColumn;
                }
                else {
                    startColumn = initialStartColumn | 0;
                }
            }
        }
        var initialEndLineNumber = range.endLineNumber;
        var initialEndColumn = range.endColumn;
        var endLineNumber;
        var endColumn;
        if (initialEndLineNumber < 1) {
            endLineNumber = 1;
            endColumn = 1;
        }
        else if (initialEndLineNumber > linesCount) {
            endLineNumber = linesCount;
            endColumn = this.getLineMaxColumn(endLineNumber);
        }
        else {
            endLineNumber = initialEndLineNumber | 0;
            if (initialEndColumn <= 1) {
                endColumn = 1;
            }
            else {
                var maxColumn = this.getLineMaxColumn(endLineNumber);
                if (initialEndColumn >= maxColumn) {
                    endColumn = maxColumn;
                }
                else {
                    endColumn = initialEndColumn | 0;
                }
            }
        }
        if (initialStartLineNumber === startLineNumber
            && initialStartColumn === startColumn
            && initialEndLineNumber === endLineNumber
            && initialEndColumn === endColumn
            && range instanceof Range
            && !(range instanceof Selection)) {
            return range;
        }
        return new Range(startLineNumber, startColumn, endLineNumber, endColumn);
    };
    /**
     * @param strict Do NOT allow a position inside a high-low surrogate pair
     */
    TextModel.prototype._isValidPosition = function (lineNumber, column, strict) {
        if (isNaN(lineNumber)) {
            return false;
        }
        if (lineNumber < 1) {
            return false;
        }
        var lineCount = this._buffer.getLineCount();
        if (lineNumber > lineCount) {
            return false;
        }
        if (isNaN(column)) {
            return false;
        }
        if (column < 1) {
            return false;
        }
        var maxColumn = this.getLineMaxColumn(lineNumber);
        if (column > maxColumn) {
            return false;
        }
        if (strict) {
            if (column > 1) {
                var charCodeBefore = this._buffer.getLineCharCode(lineNumber, column - 2);
                if (isHighSurrogate(charCodeBefore)) {
                    return false;
                }
            }
        }
        return true;
    };
    /**
     * @param strict Do NOT allow a position inside a high-low surrogate pair
     */
    TextModel.prototype._validatePosition = function (_lineNumber, _column, strict) {
        var lineNumber = Math.floor((typeof _lineNumber === 'number' && !isNaN(_lineNumber)) ? _lineNumber : 1);
        var column = Math.floor((typeof _column === 'number' && !isNaN(_column)) ? _column : 1);
        var lineCount = this._buffer.getLineCount();
        if (lineNumber < 1) {
            return new Position(1, 1);
        }
        if (lineNumber > lineCount) {
            return new Position(lineCount, this.getLineMaxColumn(lineCount));
        }
        if (column <= 1) {
            return new Position(lineNumber, 1);
        }
        var maxColumn = this.getLineMaxColumn(lineNumber);
        if (column >= maxColumn) {
            return new Position(lineNumber, maxColumn);
        }
        if (strict) {
            // If the position would end up in the middle of a high-low surrogate pair,
            // we move it to before the pair
            // !!At this point, column > 1
            var charCodeBefore = this._buffer.getLineCharCode(lineNumber, column - 2);
            if (isHighSurrogate(charCodeBefore)) {
                return new Position(lineNumber, column - 1);
            }
        }
        return new Position(lineNumber, column);
    };
    TextModel.prototype.validatePosition = function (position) {
        this._assertNotDisposed();
        // Avoid object allocation and cover most likely case
        if (position instanceof Position) {
            if (this._isValidPosition(position.lineNumber, position.column, true)) {
                return position;
            }
        }
        return this._validatePosition(position.lineNumber, position.column, true);
    };
    /**
     * @param strict Do NOT allow a range to have its boundaries inside a high-low surrogate pair
     */
    TextModel.prototype._isValidRange = function (range, strict) {
        var startLineNumber = range.startLineNumber;
        var startColumn = range.startColumn;
        var endLineNumber = range.endLineNumber;
        var endColumn = range.endColumn;
        if (!this._isValidPosition(startLineNumber, startColumn, false)) {
            return false;
        }
        if (!this._isValidPosition(endLineNumber, endColumn, false)) {
            return false;
        }
        if (strict) {
            var charCodeBeforeStart = (startColumn > 1 ? this._buffer.getLineCharCode(startLineNumber, startColumn - 2) : 0);
            var charCodeBeforeEnd = (endColumn > 1 && endColumn <= this._buffer.getLineLength(endLineNumber) ? this._buffer.getLineCharCode(endLineNumber, endColumn - 2) : 0);
            var startInsideSurrogatePair = isHighSurrogate(charCodeBeforeStart);
            var endInsideSurrogatePair = isHighSurrogate(charCodeBeforeEnd);
            if (!startInsideSurrogatePair && !endInsideSurrogatePair) {
                return true;
            }
            return false;
        }
        return true;
    };
    TextModel.prototype.validateRange = function (_range) {
        this._assertNotDisposed();
        // Avoid object allocation and cover most likely case
        if ((_range instanceof Range) && !(_range instanceof Selection)) {
            if (this._isValidRange(_range, true)) {
                return _range;
            }
        }
        var start = this._validatePosition(_range.startLineNumber, _range.startColumn, false);
        var end = this._validatePosition(_range.endLineNumber, _range.endColumn, false);
        var startLineNumber = start.lineNumber;
        var startColumn = start.column;
        var endLineNumber = end.lineNumber;
        var endColumn = end.column;
        var charCodeBeforeStart = (startColumn > 1 ? this._buffer.getLineCharCode(startLineNumber, startColumn - 2) : 0);
        var charCodeBeforeEnd = (endColumn > 1 && endColumn <= this._buffer.getLineLength(endLineNumber) ? this._buffer.getLineCharCode(endLineNumber, endColumn - 2) : 0);
        var startInsideSurrogatePair = isHighSurrogate(charCodeBeforeStart);
        var endInsideSurrogatePair = isHighSurrogate(charCodeBeforeEnd);
        if (!startInsideSurrogatePair && !endInsideSurrogatePair) {
            return new Range(startLineNumber, startColumn, endLineNumber, endColumn);
        }
        if (startLineNumber === endLineNumber && startColumn === endColumn) {
            // do not expand a collapsed range, simply move it to a valid location
            return new Range(startLineNumber, startColumn - 1, endLineNumber, endColumn - 1);
        }
        if (startInsideSurrogatePair && endInsideSurrogatePair) {
            // expand range at both ends
            return new Range(startLineNumber, startColumn - 1, endLineNumber, endColumn + 1);
        }
        if (startInsideSurrogatePair) {
            // only expand range at the start
            return new Range(startLineNumber, startColumn - 1, endLineNumber, endColumn);
        }
        // only expand range at the end
        return new Range(startLineNumber, startColumn, endLineNumber, endColumn + 1);
    };
    TextModel.prototype.modifyPosition = function (rawPosition, offset) {
        this._assertNotDisposed();
        var candidate = this.getOffsetAt(rawPosition) + offset;
        return this.getPositionAt(Math.min(this._buffer.getLength(), Math.max(0, candidate)));
    };
    TextModel.prototype.getFullModelRange = function () {
        this._assertNotDisposed();
        var lineCount = this.getLineCount();
        return new Range(1, 1, lineCount, this.getLineMaxColumn(lineCount));
    };
    TextModel.prototype.findMatchesLineByLine = function (searchRange, searchData, captureMatches, limitResultCount) {
        return this._buffer.findMatchesLineByLine(searchRange, searchData, captureMatches, limitResultCount);
    };
    TextModel.prototype.findMatches = function (searchString, rawSearchScope, isRegex, matchCase, wordSeparators, captureMatches, limitResultCount) {
        if (limitResultCount === void 0) { limitResultCount = LIMIT_FIND_COUNT$1; }
        this._assertNotDisposed();
        var searchRange;
        if (Range.isIRange(rawSearchScope)) {
            searchRange = this.validateRange(rawSearchScope);
        }
        else {
            searchRange = this.getFullModelRange();
        }
        if (!isRegex && searchString.indexOf('\n') < 0) {
            // not regex, not multi line
            var searchParams = new SearchParams(searchString, isRegex, matchCase, wordSeparators);
            var searchData = searchParams.parseSearchRequest();
            if (!searchData) {
                return [];
            }
            return this.findMatchesLineByLine(searchRange, searchData, captureMatches, limitResultCount);
        }
        return TextModelSearch.findMatches(this, new SearchParams(searchString, isRegex, matchCase, wordSeparators), searchRange, captureMatches, limitResultCount);
    };
    TextModel.prototype.findNextMatch = function (searchString, rawSearchStart, isRegex, matchCase, wordSeparators, captureMatches) {
        this._assertNotDisposed();
        var searchStart = this.validatePosition(rawSearchStart);
        if (!isRegex && searchString.indexOf('\n') < 0) {
            var searchParams = new SearchParams(searchString, isRegex, matchCase, wordSeparators);
            var searchData = searchParams.parseSearchRequest();
            if (!searchData) {
                return null;
            }
            var lineCount = this.getLineCount();
            var searchRange = new Range(searchStart.lineNumber, searchStart.column, lineCount, this.getLineMaxColumn(lineCount));
            var ret = this.findMatchesLineByLine(searchRange, searchData, captureMatches, 1);
            TextModelSearch.findNextMatch(this, new SearchParams(searchString, isRegex, matchCase, wordSeparators), searchStart, captureMatches);
            if (ret.length > 0) {
                return ret[0];
            }
            searchRange = new Range(1, 1, searchStart.lineNumber, this.getLineMaxColumn(searchStart.lineNumber));
            ret = this.findMatchesLineByLine(searchRange, searchData, captureMatches, 1);
            if (ret.length > 0) {
                return ret[0];
            }
            return null;
        }
        return TextModelSearch.findNextMatch(this, new SearchParams(searchString, isRegex, matchCase, wordSeparators), searchStart, captureMatches);
    };
    TextModel.prototype.findPreviousMatch = function (searchString, rawSearchStart, isRegex, matchCase, wordSeparators, captureMatches) {
        this._assertNotDisposed();
        var searchStart = this.validatePosition(rawSearchStart);
        return TextModelSearch.findPreviousMatch(this, new SearchParams(searchString, isRegex, matchCase, wordSeparators), searchStart, captureMatches);
    };
    //#endregion
    //#region Editing
    TextModel.prototype.pushStackElement = function () {
        this._commandManager.pushStackElement();
    };
    TextModel.prototype.pushEOL = function (eol) {
        var currentEOL = (this.getEOL() === '\n' ? 0 /* LF */ : 1 /* CRLF */);
        if (currentEOL === eol) {
            return;
        }
        try {
            this._onDidChangeDecorations.beginDeferredEmit();
            this._eventEmitter.beginDeferredEmit();
            this._commandManager.pushEOL(eol);
        }
        finally {
            this._eventEmitter.endDeferredEmit();
            this._onDidChangeDecorations.endDeferredEmit();
        }
    };
    TextModel.prototype.pushEditOperations = function (beforeCursorState, editOperations, cursorStateComputer) {
        try {
            this._onDidChangeDecorations.beginDeferredEmit();
            this._eventEmitter.beginDeferredEmit();
            return this._pushEditOperations(beforeCursorState, editOperations, cursorStateComputer);
        }
        finally {
            this._eventEmitter.endDeferredEmit();
            this._onDidChangeDecorations.endDeferredEmit();
        }
    };
    TextModel.prototype._pushEditOperations = function (beforeCursorState, editOperations, cursorStateComputer) {
        var _this = this;
        if (this._options.trimAutoWhitespace && this._trimAutoWhitespaceLines) {
            // Go through each saved line number and insert a trim whitespace edit
            // if it is safe to do so (no conflicts with other edits).
            var incomingEdits = editOperations.map(function (op) {
                return {
                    range: _this.validateRange(op.range),
                    text: op.text
                };
            });
            // Sometimes, auto-formatters change ranges automatically which can cause undesired auto whitespace trimming near the cursor
            // We'll use the following heuristic: if the edits occur near the cursor, then it's ok to trim auto whitespace
            var editsAreNearCursors = true;
            for (var i = 0, len = beforeCursorState.length; i < len; i++) {
                var sel = beforeCursorState[i];
                var foundEditNearSel = false;
                for (var j = 0, lenJ = incomingEdits.length; j < lenJ; j++) {
                    var editRange = incomingEdits[j].range;
                    var selIsAbove = editRange.startLineNumber > sel.endLineNumber;
                    var selIsBelow = sel.startLineNumber > editRange.endLineNumber;
                    if (!selIsAbove && !selIsBelow) {
                        foundEditNearSel = true;
                        break;
                    }
                }
                if (!foundEditNearSel) {
                    editsAreNearCursors = false;
                    break;
                }
            }
            if (editsAreNearCursors) {
                for (var i = 0, len = this._trimAutoWhitespaceLines.length; i < len; i++) {
                    var trimLineNumber = this._trimAutoWhitespaceLines[i];
                    var maxLineColumn = this.getLineMaxColumn(trimLineNumber);
                    var allowTrimLine = true;
                    for (var j = 0, lenJ = incomingEdits.length; j < lenJ; j++) {
                        var editRange = incomingEdits[j].range;
                        var editText = incomingEdits[j].text;
                        if (trimLineNumber < editRange.startLineNumber || trimLineNumber > editRange.endLineNumber) {
                            // `trimLine` is completely outside this edit
                            continue;
                        }
                        // At this point:
                        //   editRange.startLineNumber <= trimLine <= editRange.endLineNumber
                        if (trimLineNumber === editRange.startLineNumber && editRange.startColumn === maxLineColumn
                            && editRange.isEmpty() && editText && editText.length > 0 && editText.charAt(0) === '\n') {
                            // This edit inserts a new line (and maybe other text) after `trimLine`
                            continue;
                        }
                        if (trimLineNumber === editRange.startLineNumber && editRange.startColumn === 1
                            && editRange.isEmpty() && editText && editText.length > 0 && editText.charAt(editText.length - 1) === '\n') {
                            // This edit inserts a new line (and maybe other text) before `trimLine`
                            continue;
                        }
                        // Looks like we can't trim this line as it would interfere with an incoming edit
                        allowTrimLine = false;
                        break;
                    }
                    if (allowTrimLine) {
                        editOperations.push({
                            range: new Range(trimLineNumber, 1, trimLineNumber, maxLineColumn),
                            text: null
                        });
                    }
                }
            }
            this._trimAutoWhitespaceLines = null;
        }
        return this._commandManager.pushEditOperation(beforeCursorState, editOperations, cursorStateComputer);
    };
    TextModel.prototype.applyEdits = function (rawOperations) {
        try {
            this._onDidChangeDecorations.beginDeferredEmit();
            this._eventEmitter.beginDeferredEmit();
            return this._applyEdits(rawOperations);
        }
        finally {
            this._eventEmitter.endDeferredEmit();
            this._onDidChangeDecorations.endDeferredEmit();
        }
    };
    TextModel._eolCount = function (text) {
        var eolCount = 0;
        var firstLineLength = 0;
        for (var i = 0, len = text.length; i < len; i++) {
            var chr = text.charCodeAt(i);
            if (chr === 13 /* CarriageReturn */) {
                if (eolCount === 0) {
                    firstLineLength = i;
                }
                eolCount++;
                if (i + 1 < len && text.charCodeAt(i + 1) === 10 /* LineFeed */) {
                    // \r\n... case
                    i++; // skip \n
                }
            }
            else if (chr === 10 /* LineFeed */) {
                if (eolCount === 0) {
                    firstLineLength = i;
                }
                eolCount++;
            }
        }
        if (eolCount === 0) {
            firstLineLength = text.length;
        }
        return [eolCount, firstLineLength];
    };
    TextModel.prototype._applyEdits = function (rawOperations) {
        for (var i = 0, len = rawOperations.length; i < len; i++) {
            rawOperations[i].range = this.validateRange(rawOperations[i].range);
        }
        var oldLineCount = this._buffer.getLineCount();
        var result = this._buffer.applyEdits(rawOperations, this._options.trimAutoWhitespace);
        var newLineCount = this._buffer.getLineCount();
        var contentChanges = result.changes;
        this._trimAutoWhitespaceLines = result.trimAutoWhitespaceLineNumbers;
        if (contentChanges.length !== 0) {
            var rawContentChanges = [];
            var lineCount = oldLineCount;
            for (var i = 0, len = contentChanges.length; i < len; i++) {
                var change = contentChanges[i];
                var _a = TextModel._eolCount(change.text), eolCount = _a[0], firstLineLength = _a[1];
                try {
                    this._tokens.applyEdits(change.range, eolCount, firstLineLength);
                }
                catch (err) {
                    // emergency recovery => reset tokens
                    this._tokens = new ModelLinesTokens(this._tokens.languageIdentifier, this._tokens.tokenizationSupport);
                }
                this._onDidChangeDecorations.fire();
                this._decorationsTree.acceptReplace(change.rangeOffset, change.rangeLength, change.text.length, change.forceMoveMarkers);
                var startLineNumber = change.range.startLineNumber;
                var endLineNumber = change.range.endLineNumber;
                var deletingLinesCnt = endLineNumber - startLineNumber;
                var insertingLinesCnt = eolCount;
                var editingLinesCnt = Math.min(deletingLinesCnt, insertingLinesCnt);
                var changeLineCountDelta = (insertingLinesCnt - deletingLinesCnt);
                for (var j = editingLinesCnt; j >= 0; j--) {
                    var editLineNumber = startLineNumber + j;
                    var currentEditLineNumber = newLineCount - lineCount - changeLineCountDelta + editLineNumber;
                    rawContentChanges.push(new ModelRawLineChanged(editLineNumber, this.getLineContent(currentEditLineNumber)));
                }
                if (editingLinesCnt < deletingLinesCnt) {
                    // Must delete some lines
                    var spliceStartLineNumber = startLineNumber + editingLinesCnt;
                    rawContentChanges.push(new ModelRawLinesDeleted(spliceStartLineNumber + 1, endLineNumber));
                }
                if (editingLinesCnt < insertingLinesCnt) {
                    // Must insert some lines
                    var spliceLineNumber = startLineNumber + editingLinesCnt;
                    var cnt = insertingLinesCnt - editingLinesCnt;
                    var fromLineNumber = newLineCount - lineCount - cnt + spliceLineNumber + 1;
                    var newLines = [];
                    for (var i_1 = 0; i_1 < cnt; i_1++) {
                        var lineNumber = fromLineNumber + i_1;
                        newLines[lineNumber - fromLineNumber] = this.getLineContent(lineNumber);
                    }
                    rawContentChanges.push(new ModelRawLinesInserted(spliceLineNumber + 1, startLineNumber + insertingLinesCnt, newLines));
                }
                lineCount += changeLineCountDelta;
            }
            this._increaseVersionId();
            this._emitContentChangedEvent(new ModelRawContentChangedEvent(rawContentChanges, this.getVersionId(), this._isUndoing, this._isRedoing), {
                changes: contentChanges,
                eol: this._buffer.getEOL(),
                versionId: this.getVersionId(),
                isUndoing: this._isUndoing,
                isRedoing: this._isRedoing,
                isFlush: false
            });
        }
        if (this._tokens.hasLinesToTokenize(this._buffer)) {
            this._beginBackgroundTokenization();
        }
        return result.reverseEdits;
    };
    TextModel.prototype._undo = function () {
        this._isUndoing = true;
        var r = this._commandManager.undo();
        this._isUndoing = false;
        if (!r) {
            return null;
        }
        this._overwriteAlternativeVersionId(r.recordedVersionId);
        return r.selections;
    };
    TextModel.prototype.undo = function () {
        try {
            this._onDidChangeDecorations.beginDeferredEmit();
            this._eventEmitter.beginDeferredEmit();
            return this._undo();
        }
        finally {
            this._eventEmitter.endDeferredEmit();
            this._onDidChangeDecorations.endDeferredEmit();
        }
    };
    TextModel.prototype.canUndo = function () {
        return this._commandManager.canUndo();
    };
    TextModel.prototype._redo = function () {
        this._isRedoing = true;
        var r = this._commandManager.redo();
        this._isRedoing = false;
        if (!r) {
            return null;
        }
        this._overwriteAlternativeVersionId(r.recordedVersionId);
        return r.selections;
    };
    TextModel.prototype.redo = function () {
        try {
            this._onDidChangeDecorations.beginDeferredEmit();
            this._eventEmitter.beginDeferredEmit();
            return this._redo();
        }
        finally {
            this._eventEmitter.endDeferredEmit();
            this._onDidChangeDecorations.endDeferredEmit();
        }
    };
    TextModel.prototype.canRedo = function () {
        return this._commandManager.canRedo();
    };
    //#endregion
    //#region Decorations
    TextModel.prototype.changeDecorations = function (callback, ownerId) {
        if (ownerId === void 0) { ownerId = 0; }
        this._assertNotDisposed();
        try {
            this._onDidChangeDecorations.beginDeferredEmit();
            return this._changeDecorations(ownerId, callback);
        }
        finally {
            this._onDidChangeDecorations.endDeferredEmit();
        }
    };
    TextModel.prototype._changeDecorations = function (ownerId, callback) {
        var _this = this;
        var changeAccessor = {
            addDecoration: function (range, options) {
                _this._onDidChangeDecorations.fire();
                return _this._deltaDecorationsImpl(ownerId, [], [{ range: range, options: options }])[0];
            },
            changeDecoration: function (id, newRange) {
                _this._onDidChangeDecorations.fire();
                _this._changeDecorationImpl(id, newRange);
            },
            changeDecorationOptions: function (id, options) {
                _this._onDidChangeDecorations.fire();
                _this._changeDecorationOptionsImpl(id, _normalizeOptions(options));
            },
            removeDecoration: function (id) {
                _this._onDidChangeDecorations.fire();
                _this._deltaDecorationsImpl(ownerId, [id], []);
            },
            deltaDecorations: function (oldDecorations, newDecorations) {
                if (oldDecorations.length === 0 && newDecorations.length === 0) {
                    // nothing to do
                    return [];
                }
                _this._onDidChangeDecorations.fire();
                return _this._deltaDecorationsImpl(ownerId, oldDecorations, newDecorations);
            }
        };
        var result = null;
        try {
            result = callback(changeAccessor);
        }
        catch (e) {
            onUnexpectedError(e);
        }
        // Invalidate change accessor
        changeAccessor.addDecoration = invalidFunc;
        changeAccessor.changeDecoration = invalidFunc;
        changeAccessor.changeDecorationOptions = invalidFunc;
        changeAccessor.removeDecoration = invalidFunc;
        changeAccessor.deltaDecorations = invalidFunc;
        return result;
    };
    TextModel.prototype.deltaDecorations = function (oldDecorations, newDecorations, ownerId) {
        if (ownerId === void 0) { ownerId = 0; }
        this._assertNotDisposed();
        if (!oldDecorations) {
            oldDecorations = [];
        }
        if (oldDecorations.length === 0 && newDecorations.length === 0) {
            // nothing to do
            return [];
        }
        try {
            this._onDidChangeDecorations.beginDeferredEmit();
            this._onDidChangeDecorations.fire();
            return this._deltaDecorationsImpl(ownerId, oldDecorations, newDecorations);
        }
        finally {
            this._onDidChangeDecorations.endDeferredEmit();
        }
    };
    TextModel.prototype._getTrackedRange = function (id) {
        return this.getDecorationRange(id);
    };
    TextModel.prototype._setTrackedRange = function (id, newRange, newStickiness) {
        var node = (id ? this._decorations[id] : null);
        if (!node) {
            if (!newRange) {
                // node doesn't exist, the request is to delete => nothing to do
                return null;
            }
            // node doesn't exist, the request is to set => add the tracked range
            return this._deltaDecorationsImpl(0, [], [{ range: newRange, options: TRACKED_RANGE_OPTIONS[newStickiness] }])[0];
        }
        if (!newRange) {
            // node exists, the request is to delete => delete node
            this._decorationsTree.delete(node);
            delete this._decorations[node.id];
            return null;
        }
        // node exists, the request is to set => change the tracked range and its options
        var range = this._validateRangeRelaxedNoAllocations(newRange);
        var startOffset = this._buffer.getOffsetAt(range.startLineNumber, range.startColumn);
        var endOffset = this._buffer.getOffsetAt(range.endLineNumber, range.endColumn);
        this._decorationsTree.delete(node);
        node.reset(this.getVersionId(), startOffset, endOffset, range);
        node.setOptions(TRACKED_RANGE_OPTIONS[newStickiness]);
        this._decorationsTree.insert(node);
        return node.id;
    };
    TextModel.prototype.removeAllDecorationsWithOwnerId = function (ownerId) {
        if (this._isDisposed) {
            return;
        }
        var nodes = this._decorationsTree.collectNodesFromOwner(ownerId);
        for (var i = 0, len = nodes.length; i < len; i++) {
            var node = nodes[i];
            this._decorationsTree.delete(node);
            delete this._decorations[node.id];
        }
    };
    TextModel.prototype.getDecorationOptions = function (decorationId) {
        var node = this._decorations[decorationId];
        if (!node) {
            return null;
        }
        return node.options;
    };
    TextModel.prototype.getDecorationRange = function (decorationId) {
        var node = this._decorations[decorationId];
        if (!node) {
            return null;
        }
        var versionId = this.getVersionId();
        if (node.cachedVersionId !== versionId) {
            this._decorationsTree.resolveNode(node, versionId);
        }
        if (node.range === null) {
            node.range = this._getRangeAt(node.cachedAbsoluteStart, node.cachedAbsoluteEnd);
        }
        return node.range;
    };
    TextModel.prototype.getLineDecorations = function (lineNumber, ownerId, filterOutValidation) {
        if (ownerId === void 0) { ownerId = 0; }
        if (filterOutValidation === void 0) { filterOutValidation = false; }
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            return [];
        }
        return this.getLinesDecorations(lineNumber, lineNumber, ownerId, filterOutValidation);
    };
    TextModel.prototype.getLinesDecorations = function (_startLineNumber, _endLineNumber, ownerId, filterOutValidation) {
        if (ownerId === void 0) { ownerId = 0; }
        if (filterOutValidation === void 0) { filterOutValidation = false; }
        var lineCount = this.getLineCount();
        var startLineNumber = Math.min(lineCount, Math.max(1, _startLineNumber));
        var endLineNumber = Math.min(lineCount, Math.max(1, _endLineNumber));
        var endColumn = this.getLineMaxColumn(endLineNumber);
        return this._getDecorationsInRange(new Range(startLineNumber, 1, endLineNumber, endColumn), ownerId, filterOutValidation);
    };
    TextModel.prototype.getDecorationsInRange = function (range, ownerId, filterOutValidation) {
        if (ownerId === void 0) { ownerId = 0; }
        if (filterOutValidation === void 0) { filterOutValidation = false; }
        var validatedRange = this.validateRange(range);
        return this._getDecorationsInRange(validatedRange, ownerId, filterOutValidation);
    };
    TextModel.prototype.getOverviewRulerDecorations = function (ownerId, filterOutValidation) {
        if (ownerId === void 0) { ownerId = 0; }
        if (filterOutValidation === void 0) { filterOutValidation = false; }
        var versionId = this.getVersionId();
        var result = this._decorationsTree.search(ownerId, filterOutValidation, true, versionId);
        return this._ensureNodesHaveRanges(result);
    };
    TextModel.prototype.getAllDecorations = function (ownerId, filterOutValidation) {
        if (ownerId === void 0) { ownerId = 0; }
        if (filterOutValidation === void 0) { filterOutValidation = false; }
        var versionId = this.getVersionId();
        var result = this._decorationsTree.search(ownerId, filterOutValidation, false, versionId);
        return this._ensureNodesHaveRanges(result);
    };
    TextModel.prototype._getDecorationsInRange = function (filterRange, filterOwnerId, filterOutValidation) {
        var startOffset = this._buffer.getOffsetAt(filterRange.startLineNumber, filterRange.startColumn);
        var endOffset = this._buffer.getOffsetAt(filterRange.endLineNumber, filterRange.endColumn);
        var versionId = this.getVersionId();
        var result = this._decorationsTree.intervalSearch(startOffset, endOffset, filterOwnerId, filterOutValidation, versionId);
        return this._ensureNodesHaveRanges(result);
    };
    TextModel.prototype._ensureNodesHaveRanges = function (nodes) {
        for (var i = 0, len = nodes.length; i < len; i++) {
            var node = nodes[i];
            if (node.range === null) {
                node.range = this._getRangeAt(node.cachedAbsoluteStart, node.cachedAbsoluteEnd);
            }
        }
        return nodes;
    };
    TextModel.prototype._getRangeAt = function (start, end) {
        return this._buffer.getRangeAt(start, end - start);
    };
    TextModel.prototype._changeDecorationImpl = function (decorationId, _range) {
        var node = this._decorations[decorationId];
        if (!node) {
            return;
        }
        var range = this._validateRangeRelaxedNoAllocations(_range);
        var startOffset = this._buffer.getOffsetAt(range.startLineNumber, range.startColumn);
        var endOffset = this._buffer.getOffsetAt(range.endLineNumber, range.endColumn);
        this._decorationsTree.delete(node);
        node.reset(this.getVersionId(), startOffset, endOffset, range);
        this._decorationsTree.insert(node);
    };
    TextModel.prototype._changeDecorationOptionsImpl = function (decorationId, options) {
        var node = this._decorations[decorationId];
        if (!node) {
            return;
        }
        var nodeWasInOverviewRuler = (node.options.overviewRuler && node.options.overviewRuler.color ? true : false);
        var nodeIsInOverviewRuler = (options.overviewRuler && options.overviewRuler.color ? true : false);
        if (nodeWasInOverviewRuler !== nodeIsInOverviewRuler) {
            // Delete + Insert due to an overview ruler status change
            this._decorationsTree.delete(node);
            node.setOptions(options);
            this._decorationsTree.insert(node);
        }
        else {
            node.setOptions(options);
        }
    };
    TextModel.prototype._deltaDecorationsImpl = function (ownerId, oldDecorationsIds, newDecorations) {
        var versionId = this.getVersionId();
        var oldDecorationsLen = oldDecorationsIds.length;
        var oldDecorationIndex = 0;
        var newDecorationsLen = newDecorations.length;
        var newDecorationIndex = 0;
        var result = new Array(newDecorationsLen);
        while (oldDecorationIndex < oldDecorationsLen || newDecorationIndex < newDecorationsLen) {
            var node = null;
            if (oldDecorationIndex < oldDecorationsLen) {
                // (1) get ourselves an old node
                do {
                    node = this._decorations[oldDecorationsIds[oldDecorationIndex++]];
                } while (!node && oldDecorationIndex < oldDecorationsLen);
                // (2) remove the node from the tree (if it exists)
                if (node) {
                    this._decorationsTree.delete(node);
                }
            }
            if (newDecorationIndex < newDecorationsLen) {
                // (3) create a new node if necessary
                if (!node) {
                    var internalDecorationId = (++this._lastDecorationId);
                    var decorationId = this._instanceId + ";" + internalDecorationId;
                    node = new IntervalNode(decorationId, 0, 0);
                    this._decorations[decorationId] = node;
                }
                // (4) initialize node
                var newDecoration = newDecorations[newDecorationIndex];
                var range = this._validateRangeRelaxedNoAllocations(newDecoration.range);
                var options = _normalizeOptions(newDecoration.options);
                var startOffset = this._buffer.getOffsetAt(range.startLineNumber, range.startColumn);
                var endOffset = this._buffer.getOffsetAt(range.endLineNumber, range.endColumn);
                node.ownerId = ownerId;
                node.reset(versionId, startOffset, endOffset, range);
                node.setOptions(options);
                this._decorationsTree.insert(node);
                result[newDecorationIndex] = node.id;
                newDecorationIndex++;
            }
            else {
                if (node) {
                    delete this._decorations[node.id];
                }
            }
        }
        return result;
    };
    //#endregion
    //#region Tokenization
    TextModel.prototype.tokenizeViewport = function (startLineNumber, endLineNumber) {
        if (!this._tokens.tokenizationSupport) {
            return;
        }
        // we tokenize `this._tokens.inValidLineStartIndex` lines in around 20ms so it's a good baseline.
        var contextBefore = Math.floor(this._tokens.inValidLineStartIndex * 0.3);
        startLineNumber = Math.max(1, startLineNumber - contextBefore);
        if (startLineNumber <= this._tokens.inValidLineStartIndex) {
            this.forceTokenization(endLineNumber);
            return;
        }
        var eventBuilder = new ModelTokensChangedEventBuilder();
        var nonWhitespaceColumn = this.getLineFirstNonWhitespaceColumn(startLineNumber);
        var fakeLines = [];
        var i = startLineNumber - 1;
        var initialState = null;
        if (nonWhitespaceColumn > 0) {
            while (nonWhitespaceColumn > 0 && i >= 1) {
                var newNonWhitespaceIndex = this.getLineFirstNonWhitespaceColumn(i);
                if (newNonWhitespaceIndex === 0) {
                    i--;
                    continue;
                }
                if (newNonWhitespaceIndex < nonWhitespaceColumn) {
                    initialState = this._tokens._getState(i - 1);
                    if (initialState) {
                        break;
                    }
                    fakeLines.push(this.getLineContent(i));
                    nonWhitespaceColumn = newNonWhitespaceIndex;
                }
                i--;
            }
        }
        if (!initialState) {
            initialState = this._tokens.tokenizationSupport.getInitialState();
        }
        var state = initialState.clone();
        for (var i_2 = fakeLines.length - 1; i_2 >= 0; i_2--) {
            var r = this._tokens._tokenizeText(this._buffer, fakeLines[i_2], state);
            if (r) {
                state = r.endState.clone();
            }
            else {
                state = initialState.clone();
            }
        }
        var contextAfter = Math.floor(this._tokens.inValidLineStartIndex * 0.4);
        endLineNumber = Math.min(this.getLineCount(), endLineNumber + contextAfter);
        for (var i_3 = startLineNumber; i_3 <= endLineNumber; i_3++) {
            var text = this.getLineContent(i_3);
            var r = this._tokens._tokenizeText(this._buffer, text, state);
            if (r) {
                this._tokens._setTokens(this._tokens.languageIdentifier.id, i_3 - 1, text.length, r.tokens);
                /*
                 * we think it's valid and give it a state but we don't update `_invalidLineStartIndex` then the top-to-bottom tokenization
                 * goes through the viewport, it can skip them if they already have correct tokens and state, and the lines after the viewport
                 * can still be tokenized.
                 */
                this._tokens._setIsInvalid(i_3 - 1, false);
                this._tokens._setState(i_3 - 1, state);
                state = r.endState.clone();
                eventBuilder.registerChangedTokens(i_3);
            }
            else {
                state = initialState.clone();
            }
        }
        var e = eventBuilder.build();
        if (e) {
            this._onDidChangeTokens.fire(e);
        }
    };
    TextModel.prototype.forceTokenization = function (lineNumber) {
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            throw new Error('Illegal value for lineNumber');
        }
        var eventBuilder = new ModelTokensChangedEventBuilder();
        this._tokens._updateTokensUntilLine(this._buffer, eventBuilder, lineNumber);
        var e = eventBuilder.build();
        if (e) {
            this._onDidChangeTokens.fire(e);
        }
    };
    TextModel.prototype.isCheapToTokenize = function (lineNumber) {
        if (!this._tokens.isCheapToTokenize(lineNumber)) {
            return false;
        }
        if (lineNumber < this._tokens.inValidLineStartIndex + 1) {
            return true;
        }
        if (this.getLineLength(lineNumber) < CHEAP_TOKENIZATION_LENGTH_LIMIT) {
            return true;
        }
        return false;
    };
    TextModel.prototype.tokenizeIfCheap = function (lineNumber) {
        if (this.isCheapToTokenize(lineNumber)) {
            this.forceTokenization(lineNumber);
        }
    };
    TextModel.prototype.getLineTokens = function (lineNumber) {
        if (lineNumber < 1 || lineNumber > this.getLineCount()) {
            throw new Error('Illegal value for lineNumber');
        }
        return this._getLineTokens(lineNumber);
    };
    TextModel.prototype._getLineTokens = function (lineNumber) {
        var lineText = this._buffer.getLineContent(lineNumber);
        return this._tokens.getTokens(this._languageIdentifier.id, lineNumber - 1, lineText);
    };
    TextModel.prototype.getLanguageIdentifier = function () {
        return this._languageIdentifier;
    };
    TextModel.prototype.getModeId = function () {
        return this._languageIdentifier.language;
    };
    TextModel.prototype.setMode = function (languageIdentifier) {
        if (this._languageIdentifier.id === languageIdentifier.id) {
            // There's nothing to do
            return;
        }
        var e = {
            oldLanguage: this._languageIdentifier.language,
            newLanguage: languageIdentifier.language
        };
        this._languageIdentifier = languageIdentifier;
        // Cancel tokenization, clear all tokens and begin tokenizing
        this._resetTokenizationState();
        this.emitModelTokensChangedEvent({
            ranges: [{
                    fromLineNumber: 1,
                    toLineNumber: this.getLineCount()
                }]
        });
        this._onDidChangeLanguage.fire(e);
        this._onDidChangeLanguageConfiguration.fire({});
    };
    TextModel.prototype.getLanguageIdAtPosition = function (_lineNumber, _column) {
        if (!this._tokens.tokenizationSupport) {
            return this._languageIdentifier.id;
        }
        var _a = this.validatePosition({ lineNumber: _lineNumber, column: _column }), lineNumber = _a.lineNumber, column = _a.column;
        var lineTokens = this._getLineTokens(lineNumber);
        return lineTokens.getLanguageId(lineTokens.findTokenIndexAtOffset(column - 1));
    };
    TextModel.prototype._beginBackgroundTokenization = function () {
        var _this = this;
        if (this._shouldAutoTokenize() && this._revalidateTokensTimeout === -1) {
            this._revalidateTokensTimeout = setTimeout(function () {
                _this._revalidateTokensTimeout = -1;
                _this._revalidateTokensNow();
            }, 0);
        }
    };
    TextModel.prototype._warmUpTokens = function () {
        // Warm up first 100 lines (if it takes less than 50ms)
        var maxLineNumber = Math.min(100, this.getLineCount());
        this._revalidateTokensNow(maxLineNumber);
        if (this._tokens.hasLinesToTokenize(this._buffer)) {
            this._beginBackgroundTokenization();
        }
    };
    TextModel.prototype._revalidateTokensNow = function (toLineNumber) {
        if (toLineNumber === void 0) { toLineNumber = this._buffer.getLineCount(); }
        var MAX_ALLOWED_TIME = 20;
        var eventBuilder = new ModelTokensChangedEventBuilder();
        var sw = StopWatch.create(false);
        while (this._tokens.hasLinesToTokenize(this._buffer)) {
            if (sw.elapsed() > MAX_ALLOWED_TIME) {
                // Stop if MAX_ALLOWED_TIME is reached
                break;
            }
            var tokenizedLineNumber = this._tokens._tokenizeOneLine(this._buffer, eventBuilder);
            if (tokenizedLineNumber >= toLineNumber) {
                break;
            }
        }
        if (this._tokens.hasLinesToTokenize(this._buffer)) {
            this._beginBackgroundTokenization();
        }
        var e = eventBuilder.build();
        if (e) {
            this._onDidChangeTokens.fire(e);
        }
    };
    TextModel.prototype.emitModelTokensChangedEvent = function (e) {
        if (!this._isDisposing) {
            this._onDidChangeTokens.fire(e);
        }
    };
    // Having tokens allows implementing additional helper methods
    TextModel.prototype.getWordAtPosition = function (_position) {
        this._assertNotDisposed();
        var position = this.validatePosition(_position);
        var lineContent = this.getLineContent(position.lineNumber);
        var lineTokens = this._getLineTokens(position.lineNumber);
        var tokenIndex = lineTokens.findTokenIndexAtOffset(position.column - 1);
        // (1). First try checking right biased word
        var _a = TextModel._findLanguageBoundaries(lineTokens, tokenIndex), rbStartOffset = _a[0], rbEndOffset = _a[1];
        var rightBiasedWord = getWordAtText(position.column, LanguageConfigurationRegistry.getWordDefinition(lineTokens.getLanguageId(tokenIndex)), lineContent.substring(rbStartOffset, rbEndOffset), rbStartOffset);
        // Make sure the result touches the original passed in position
        if (rightBiasedWord && rightBiasedWord.startColumn <= _position.column && _position.column <= rightBiasedWord.endColumn) {
            return rightBiasedWord;
        }
        // (2). Else, if we were at a language boundary, check the left biased word
        if (tokenIndex > 0 && rbStartOffset === position.column - 1) {
            // edge case, where `position` sits between two tokens belonging to two different languages
            var _b = TextModel._findLanguageBoundaries(lineTokens, tokenIndex - 1), lbStartOffset = _b[0], lbEndOffset = _b[1];
            var leftBiasedWord = getWordAtText(position.column, LanguageConfigurationRegistry.getWordDefinition(lineTokens.getLanguageId(tokenIndex - 1)), lineContent.substring(lbStartOffset, lbEndOffset), lbStartOffset);
            // Make sure the result touches the original passed in position
            if (leftBiasedWord && leftBiasedWord.startColumn <= _position.column && _position.column <= leftBiasedWord.endColumn) {
                return leftBiasedWord;
            }
        }
        return null;
    };
    TextModel._findLanguageBoundaries = function (lineTokens, tokenIndex) {
        var languageId = lineTokens.getLanguageId(tokenIndex);
        // go left until a different language is hit
        var startOffset = 0;
        for (var i = tokenIndex; i >= 0 && lineTokens.getLanguageId(i) === languageId; i--) {
            startOffset = lineTokens.getStartOffset(i);
        }
        // go right until a different language is hit
        var endOffset = lineTokens.getLineContent().length;
        for (var i = tokenIndex, tokenCount = lineTokens.getCount(); i < tokenCount && lineTokens.getLanguageId(i) === languageId; i++) {
            endOffset = lineTokens.getEndOffset(i);
        }
        return [startOffset, endOffset];
    };
    TextModel.prototype.getWordUntilPosition = function (position) {
        var wordAtPosition = this.getWordAtPosition(position);
        if (!wordAtPosition) {
            return {
                word: '',
                startColumn: position.column,
                endColumn: position.column
            };
        }
        return {
            word: wordAtPosition.word.substr(0, position.column - wordAtPosition.startColumn),
            startColumn: wordAtPosition.startColumn,
            endColumn: position.column
        };
    };
    TextModel.prototype.findMatchingBracketUp = function (_bracket, _position) {
        var bracket = _bracket.toLowerCase();
        var position = this.validatePosition(_position);
        var lineTokens = this._getLineTokens(position.lineNumber);
        var languageId = lineTokens.getLanguageId(lineTokens.findTokenIndexAtOffset(position.column - 1));
        var bracketsSupport = LanguageConfigurationRegistry.getBracketsSupport(languageId);
        if (!bracketsSupport) {
            return null;
        }
        var data = bracketsSupport.textIsBracket[bracket];
        if (!data) {
            return null;
        }
        return this._findMatchingBracketUp(data, position);
    };
    TextModel.prototype.matchBracket = function (position) {
        return this._matchBracket(this.validatePosition(position));
    };
    TextModel.prototype._matchBracket = function (position) {
        var lineNumber = position.lineNumber;
        var lineTokens = this._getLineTokens(lineNumber);
        var lineText = this._buffer.getLineContent(lineNumber);
        var tokenIndex = lineTokens.findTokenIndexAtOffset(position.column - 1);
        if (tokenIndex < 0) {
            return null;
        }
        var currentModeBrackets = LanguageConfigurationRegistry.getBracketsSupport(lineTokens.getLanguageId(tokenIndex));
        // check that the token is not to be ignored
        if (currentModeBrackets && !ignoreBracketsInToken(lineTokens.getStandardTokenType(tokenIndex))) {
            // limit search to not go before `maxBracketLength`
            var searchStartOffset = Math.max(lineTokens.getStartOffset(tokenIndex), position.column - 1 - currentModeBrackets.maxBracketLength);
            // limit search to not go after `maxBracketLength`
            var searchEndOffset = Math.min(lineTokens.getEndOffset(tokenIndex), position.column - 1 + currentModeBrackets.maxBracketLength);
            // it might be the case that [currentTokenStart -> currentTokenEnd] contains multiple brackets
            // `bestResult` will contain the most right-side result
            var bestResult = null;
            while (true) {
                var foundBracket = BracketsUtils.findNextBracketInToken(currentModeBrackets.forwardRegex, lineNumber, lineText, searchStartOffset, searchEndOffset);
                if (!foundBracket) {
                    // there are no more brackets in this text
                    break;
                }
                // check that we didn't hit a bracket too far away from position
                if (foundBracket.startColumn <= position.column && position.column <= foundBracket.endColumn) {
                    var foundBracketText = lineText.substring(foundBracket.startColumn - 1, foundBracket.endColumn - 1);
                    foundBracketText = foundBracketText.toLowerCase();
                    var r = this._matchFoundBracket(foundBracket, currentModeBrackets.textIsBracket[foundBracketText], currentModeBrackets.textIsOpenBracket[foundBracketText]);
                    // check that we can actually match this bracket
                    if (r) {
                        bestResult = r;
                    }
                }
                searchStartOffset = foundBracket.endColumn - 1;
            }
            if (bestResult) {
                return bestResult;
            }
        }
        // If position is in between two tokens, try also looking in the previous token
        if (tokenIndex > 0 && lineTokens.getStartOffset(tokenIndex) === position.column - 1) {
            var searchEndOffset = lineTokens.getStartOffset(tokenIndex);
            tokenIndex--;
            var prevModeBrackets = LanguageConfigurationRegistry.getBracketsSupport(lineTokens.getLanguageId(tokenIndex));
            // check that previous token is not to be ignored
            if (prevModeBrackets && !ignoreBracketsInToken(lineTokens.getStandardTokenType(tokenIndex))) {
                // limit search in case previous token is very large, there's no need to go beyond `maxBracketLength`
                var searchStartOffset = Math.max(lineTokens.getStartOffset(tokenIndex), position.column - 1 - prevModeBrackets.maxBracketLength);
                var foundBracket = BracketsUtils.findPrevBracketInToken(prevModeBrackets.reversedRegex, lineNumber, lineText, searchStartOffset, searchEndOffset);
                // check that we didn't hit a bracket too far away from position
                if (foundBracket && foundBracket.startColumn <= position.column && position.column <= foundBracket.endColumn) {
                    var foundBracketText = lineText.substring(foundBracket.startColumn - 1, foundBracket.endColumn - 1);
                    foundBracketText = foundBracketText.toLowerCase();
                    var r = this._matchFoundBracket(foundBracket, prevModeBrackets.textIsBracket[foundBracketText], prevModeBrackets.textIsOpenBracket[foundBracketText]);
                    // check that we can actually match this bracket
                    if (r) {
                        return r;
                    }
                }
            }
        }
        return null;
    };
    TextModel.prototype._matchFoundBracket = function (foundBracket, data, isOpen) {
        if (!data) {
            return null;
        }
        if (isOpen) {
            var matched = this._findMatchingBracketDown(data, foundBracket.getEndPosition());
            if (matched) {
                return [foundBracket, matched];
            }
        }
        else {
            var matched = this._findMatchingBracketUp(data, foundBracket.getStartPosition());
            if (matched) {
                return [foundBracket, matched];
            }
        }
        return null;
    };
    TextModel.prototype._findMatchingBracketUp = function (bracket, position) {
        // console.log('_findMatchingBracketUp: ', 'bracket: ', JSON.stringify(bracket), 'startPosition: ', String(position));
        var languageId = bracket.languageIdentifier.id;
        var reversedBracketRegex = bracket.reversedRegex;
        var count = -1;
        for (var lineNumber = position.lineNumber; lineNumber >= 1; lineNumber--) {
            var lineTokens = this._getLineTokens(lineNumber);
            var tokenCount = lineTokens.getCount();
            var lineText = this._buffer.getLineContent(lineNumber);
            var tokenIndex = tokenCount - 1;
            var searchStopOffset = -1;
            if (lineNumber === position.lineNumber) {
                tokenIndex = lineTokens.findTokenIndexAtOffset(position.column - 1);
                searchStopOffset = position.column - 1;
            }
            for (; tokenIndex >= 0; tokenIndex--) {
                var tokenLanguageId = lineTokens.getLanguageId(tokenIndex);
                var tokenType = lineTokens.getStandardTokenType(tokenIndex);
                var tokenStartOffset = lineTokens.getStartOffset(tokenIndex);
                var tokenEndOffset = lineTokens.getEndOffset(tokenIndex);
                if (searchStopOffset === -1) {
                    searchStopOffset = tokenEndOffset;
                }
                if (tokenLanguageId === languageId && !ignoreBracketsInToken(tokenType)) {
                    while (true) {
                        var r = BracketsUtils.findPrevBracketInToken(reversedBracketRegex, lineNumber, lineText, tokenStartOffset, searchStopOffset);
                        if (!r) {
                            break;
                        }
                        var hitText = lineText.substring(r.startColumn - 1, r.endColumn - 1);
                        hitText = hitText.toLowerCase();
                        if (hitText === bracket.open) {
                            count++;
                        }
                        else if (hitText === bracket.close) {
                            count--;
                        }
                        if (count === 0) {
                            return r;
                        }
                        searchStopOffset = r.startColumn - 1;
                    }
                }
                searchStopOffset = -1;
            }
        }
        return null;
    };
    TextModel.prototype._findMatchingBracketDown = function (bracket, position) {
        // console.log('_findMatchingBracketDown: ', 'bracket: ', JSON.stringify(bracket), 'startPosition: ', String(position));
        var languageId = bracket.languageIdentifier.id;
        var bracketRegex = bracket.forwardRegex;
        var count = 1;
        for (var lineNumber = position.lineNumber, lineCount = this.getLineCount(); lineNumber <= lineCount; lineNumber++) {
            var lineTokens = this._getLineTokens(lineNumber);
            var tokenCount = lineTokens.getCount();
            var lineText = this._buffer.getLineContent(lineNumber);
            var tokenIndex = 0;
            var searchStartOffset = 0;
            if (lineNumber === position.lineNumber) {
                tokenIndex = lineTokens.findTokenIndexAtOffset(position.column - 1);
                searchStartOffset = position.column - 1;
            }
            for (; tokenIndex < tokenCount; tokenIndex++) {
                var tokenLanguageId = lineTokens.getLanguageId(tokenIndex);
                var tokenType = lineTokens.getStandardTokenType(tokenIndex);
                var tokenStartOffset = lineTokens.getStartOffset(tokenIndex);
                var tokenEndOffset = lineTokens.getEndOffset(tokenIndex);
                if (searchStartOffset === 0) {
                    searchStartOffset = tokenStartOffset;
                }
                if (tokenLanguageId === languageId && !ignoreBracketsInToken(tokenType)) {
                    while (true) {
                        var r = BracketsUtils.findNextBracketInToken(bracketRegex, lineNumber, lineText, searchStartOffset, tokenEndOffset);
                        if (!r) {
                            break;
                        }
                        var hitText = lineText.substring(r.startColumn - 1, r.endColumn - 1);
                        hitText = hitText.toLowerCase();
                        if (hitText === bracket.open) {
                            count++;
                        }
                        else if (hitText === bracket.close) {
                            count--;
                        }
                        if (count === 0) {
                            return r;
                        }
                        searchStartOffset = r.endColumn - 1;
                    }
                }
                searchStartOffset = 0;
            }
        }
        return null;
    };
    TextModel.prototype.findNextBracket = function (_position) {
        var position = this.validatePosition(_position);
        var languageId = -1;
        var modeBrackets = null;
        for (var lineNumber = position.lineNumber, lineCount = this.getLineCount(); lineNumber <= lineCount; lineNumber++) {
            var lineTokens = this._getLineTokens(lineNumber);
            var tokenCount = lineTokens.getCount();
            var lineText = this._buffer.getLineContent(lineNumber);
            var tokenIndex = 0;
            var searchStartOffset = 0;
            if (lineNumber === position.lineNumber) {
                tokenIndex = lineTokens.findTokenIndexAtOffset(position.column - 1);
                searchStartOffset = position.column - 1;
            }
            for (; tokenIndex < tokenCount; tokenIndex++) {
                var tokenLanguageId = lineTokens.getLanguageId(tokenIndex);
                var tokenType = lineTokens.getStandardTokenType(tokenIndex);
                var tokenStartOffset = lineTokens.getStartOffset(tokenIndex);
                var tokenEndOffset = lineTokens.getEndOffset(tokenIndex);
                if (searchStartOffset === 0) {
                    searchStartOffset = tokenStartOffset;
                }
                if (languageId !== tokenLanguageId) {
                    languageId = tokenLanguageId;
                    modeBrackets = LanguageConfigurationRegistry.getBracketsSupport(languageId);
                }
                if (modeBrackets && !ignoreBracketsInToken(tokenType)) {
                    var r = BracketsUtils.findNextBracketInToken(modeBrackets.forwardRegex, lineNumber, lineText, searchStartOffset, tokenEndOffset);
                    if (r) {
                        return this._toFoundBracket(modeBrackets, r);
                    }
                }
                searchStartOffset = 0;
            }
        }
        return null;
    };
    TextModel.prototype._toFoundBracket = function (modeBrackets, r) {
        if (!r) {
            return null;
        }
        var text = this.getValueInRange(r);
        text = text.toLowerCase();
        var data = modeBrackets.textIsBracket[text];
        if (!data) {
            return null;
        }
        return {
            range: r,
            open: data.open,
            close: data.close,
            isOpen: modeBrackets.textIsOpenBracket[text]
        };
    };
    /**
     * Returns:
     *  - -1 => the line consists of whitespace
     *  - otherwise => the indent level is returned value
     */
    TextModel.computeIndentLevel = function (line, tabSize) {
        var indent = 0;
        var i = 0;
        var len = line.length;
        while (i < len) {
            var chCode = line.charCodeAt(i);
            if (chCode === 32 /* Space */) {
                indent++;
            }
            else if (chCode === 9 /* Tab */) {
                indent = indent - indent % tabSize + tabSize;
            }
            else {
                break;
            }
            i++;
        }
        if (i === len) {
            return -1; // line only consists of whitespace
        }
        return indent;
    };
    TextModel.prototype._computeIndentLevel = function (lineIndex) {
        return TextModel.computeIndentLevel(this._buffer.getLineContent(lineIndex + 1), this._options.tabSize);
    };
    TextModel.prototype.getActiveIndentGuide = function (lineNumber, minLineNumber, maxLineNumber) {
        var _this = this;
        this._assertNotDisposed();
        var lineCount = this.getLineCount();
        if (lineNumber < 1 || lineNumber > lineCount) {
            throw new Error('Illegal value for lineNumber');
        }
        var foldingRules = LanguageConfigurationRegistry.getFoldingRules(this._languageIdentifier.id);
        var offSide = Boolean(foldingRules && foldingRules.offSide);
        var up_aboveContentLineIndex = -2; /* -2 is a marker for not having computed it */
        var up_aboveContentLineIndent = -1;
        var up_belowContentLineIndex = -2; /* -2 is a marker for not having computed it */
        var up_belowContentLineIndent = -1;
        var up_resolveIndents = function (lineNumber) {
            if (up_aboveContentLineIndex !== -1 && (up_aboveContentLineIndex === -2 || up_aboveContentLineIndex > lineNumber - 1)) {
                up_aboveContentLineIndex = -1;
                up_aboveContentLineIndent = -1;
                // must find previous line with content
                for (var lineIndex = lineNumber - 2; lineIndex >= 0; lineIndex--) {
                    var indent_1 = _this._computeIndentLevel(lineIndex);
                    if (indent_1 >= 0) {
                        up_aboveContentLineIndex = lineIndex;
                        up_aboveContentLineIndent = indent_1;
                        break;
                    }
                }
            }
            if (up_belowContentLineIndex === -2) {
                up_belowContentLineIndex = -1;
                up_belowContentLineIndent = -1;
                // must find next line with content
                for (var lineIndex = lineNumber; lineIndex < lineCount; lineIndex++) {
                    var indent_2 = _this._computeIndentLevel(lineIndex);
                    if (indent_2 >= 0) {
                        up_belowContentLineIndex = lineIndex;
                        up_belowContentLineIndent = indent_2;
                        break;
                    }
                }
            }
        };
        var down_aboveContentLineIndex = -2; /* -2 is a marker for not having computed it */
        var down_aboveContentLineIndent = -1;
        var down_belowContentLineIndex = -2; /* -2 is a marker for not having computed it */
        var down_belowContentLineIndent = -1;
        var down_resolveIndents = function (lineNumber) {
            if (down_aboveContentLineIndex === -2) {
                down_aboveContentLineIndex = -1;
                down_aboveContentLineIndent = -1;
                // must find previous line with content
                for (var lineIndex = lineNumber - 2; lineIndex >= 0; lineIndex--) {
                    var indent_3 = _this._computeIndentLevel(lineIndex);
                    if (indent_3 >= 0) {
                        down_aboveContentLineIndex = lineIndex;
                        down_aboveContentLineIndent = indent_3;
                        break;
                    }
                }
            }
            if (down_belowContentLineIndex !== -1 && (down_belowContentLineIndex === -2 || down_belowContentLineIndex < lineNumber - 1)) {
                down_belowContentLineIndex = -1;
                down_belowContentLineIndent = -1;
                // must find next line with content
                for (var lineIndex = lineNumber; lineIndex < lineCount; lineIndex++) {
                    var indent_4 = _this._computeIndentLevel(lineIndex);
                    if (indent_4 >= 0) {
                        down_belowContentLineIndex = lineIndex;
                        down_belowContentLineIndent = indent_4;
                        break;
                    }
                }
            }
        };
        var startLineNumber = 0;
        var goUp = true;
        var endLineNumber = 0;
        var goDown = true;
        var indent = 0;
        for (var distance = 0; goUp || goDown; distance++) {
            var upLineNumber = lineNumber - distance;
            var downLineNumber = lineNumber + distance;
            if (distance !== 0 && (upLineNumber < 1 || upLineNumber < minLineNumber)) {
                goUp = false;
            }
            if (distance !== 0 && (downLineNumber > lineCount || downLineNumber > maxLineNumber)) {
                goDown = false;
            }
            if (distance > 50000) {
                // stop processing
                goUp = false;
                goDown = false;
            }
            if (goUp) {
                // compute indent level going up
                var upLineIndentLevel = void 0;
                var currentIndent = this._computeIndentLevel(upLineNumber - 1);
                if (currentIndent >= 0) {
                    // This line has content (besides whitespace)
                    // Use the line's indent
                    up_belowContentLineIndex = upLineNumber - 1;
                    up_belowContentLineIndent = currentIndent;
                    upLineIndentLevel = Math.ceil(currentIndent / this._options.tabSize);
                }
                else {
                    up_resolveIndents(upLineNumber);
                    upLineIndentLevel = this._getIndentLevelForWhitespaceLine(offSide, up_aboveContentLineIndent, up_belowContentLineIndent);
                }
                if (distance === 0) {
                    // This is the initial line number
                    startLineNumber = upLineNumber;
                    endLineNumber = downLineNumber;
                    indent = upLineIndentLevel;
                    if (indent === 0) {
                        // No need to continue
                        return { startLineNumber: startLineNumber, endLineNumber: endLineNumber, indent: indent };
                    }
                    continue;
                }
                if (upLineIndentLevel >= indent) {
                    startLineNumber = upLineNumber;
                }
                else {
                    goUp = false;
                }
            }
            if (goDown) {
                // compute indent level going down
                var downLineIndentLevel = void 0;
                var currentIndent = this._computeIndentLevel(downLineNumber - 1);
                if (currentIndent >= 0) {
                    // This line has content (besides whitespace)
                    // Use the line's indent
                    down_aboveContentLineIndex = downLineNumber - 1;
                    down_aboveContentLineIndent = currentIndent;
                    downLineIndentLevel = Math.ceil(currentIndent / this._options.tabSize);
                }
                else {
                    down_resolveIndents(downLineNumber);
                    downLineIndentLevel = this._getIndentLevelForWhitespaceLine(offSide, down_aboveContentLineIndent, down_belowContentLineIndent);
                }
                if (downLineIndentLevel >= indent) {
                    endLineNumber = downLineNumber;
                }
                else {
                    goDown = false;
                }
            }
        }
        return { startLineNumber: startLineNumber, endLineNumber: endLineNumber, indent: indent };
    };
    TextModel.prototype.getLinesIndentGuides = function (startLineNumber, endLineNumber) {
        this._assertNotDisposed();
        var lineCount = this.getLineCount();
        if (startLineNumber < 1 || startLineNumber > lineCount) {
            throw new Error('Illegal value for startLineNumber');
        }
        if (endLineNumber < 1 || endLineNumber > lineCount) {
            throw new Error('Illegal value for endLineNumber');
        }
        var foldingRules = LanguageConfigurationRegistry.getFoldingRules(this._languageIdentifier.id);
        var offSide = Boolean(foldingRules && foldingRules.offSide);
        var result = new Array(endLineNumber - startLineNumber + 1);
        var aboveContentLineIndex = -2; /* -2 is a marker for not having computed it */
        var aboveContentLineIndent = -1;
        var belowContentLineIndex = -2; /* -2 is a marker for not having computed it */
        var belowContentLineIndent = -1;
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            var resultIndex = lineNumber - startLineNumber;
            var currentIndent = this._computeIndentLevel(lineNumber - 1);
            if (currentIndent >= 0) {
                // This line has content (besides whitespace)
                // Use the line's indent
                aboveContentLineIndex = lineNumber - 1;
                aboveContentLineIndent = currentIndent;
                result[resultIndex] = Math.ceil(currentIndent / this._options.tabSize);
                continue;
            }
            if (aboveContentLineIndex === -2) {
                aboveContentLineIndex = -1;
                aboveContentLineIndent = -1;
                // must find previous line with content
                for (var lineIndex = lineNumber - 2; lineIndex >= 0; lineIndex--) {
                    var indent = this._computeIndentLevel(lineIndex);
                    if (indent >= 0) {
                        aboveContentLineIndex = lineIndex;
                        aboveContentLineIndent = indent;
                        break;
                    }
                }
            }
            if (belowContentLineIndex !== -1 && (belowContentLineIndex === -2 || belowContentLineIndex < lineNumber - 1)) {
                belowContentLineIndex = -1;
                belowContentLineIndent = -1;
                // must find next line with content
                for (var lineIndex = lineNumber; lineIndex < lineCount; lineIndex++) {
                    var indent = this._computeIndentLevel(lineIndex);
                    if (indent >= 0) {
                        belowContentLineIndex = lineIndex;
                        belowContentLineIndent = indent;
                        break;
                    }
                }
            }
            result[resultIndex] = this._getIndentLevelForWhitespaceLine(offSide, aboveContentLineIndent, belowContentLineIndent);
        }
        return result;
    };
    TextModel.prototype._getIndentLevelForWhitespaceLine = function (offSide, aboveContentLineIndent, belowContentLineIndent) {
        if (aboveContentLineIndent === -1 || belowContentLineIndent === -1) {
            // At the top or bottom of the file
            return 0;
        }
        else if (aboveContentLineIndent < belowContentLineIndent) {
            // we are inside the region above
            return (1 + Math.floor(aboveContentLineIndent / this._options.tabSize));
        }
        else if (aboveContentLineIndent === belowContentLineIndent) {
            // we are in between two regions
            return Math.ceil(belowContentLineIndent / this._options.tabSize);
        }
        else {
            if (offSide) {
                // same level as region below
                return Math.ceil(belowContentLineIndent / this._options.tabSize);
            }
            else {
                // we are inside the region that ends below
                return (1 + Math.floor(belowContentLineIndent / this._options.tabSize));
            }
        }
    };
    TextModel.MODEL_SYNC_LIMIT = 50 * 1024 * 1024; // 50 MB
    TextModel.LARGE_FILE_SIZE_THRESHOLD = 20 * 1024 * 1024; // 20 MB;
    TextModel.LARGE_FILE_LINE_COUNT_THRESHOLD = 300 * 1000; // 300K lines
    TextModel.DEFAULT_CREATION_OPTIONS = {
        isForSimpleWidget: false,
        tabSize: EDITOR_MODEL_DEFAULTS.tabSize,
        insertSpaces: EDITOR_MODEL_DEFAULTS.insertSpaces,
        detectIndentation: false,
        defaultEOL: 1 /* LF */,
        trimAutoWhitespace: EDITOR_MODEL_DEFAULTS.trimAutoWhitespace,
        largeFileOptimizations: EDITOR_MODEL_DEFAULTS.largeFileOptimizations,
    };
    return TextModel;
}(Disposable));
//#region Decorations
var DecorationsTrees = /** @class */ (function () {
    function DecorationsTrees() {
        this._decorationsTree0 = new IntervalTree();
        this._decorationsTree1 = new IntervalTree();
    }
    DecorationsTrees.prototype.intervalSearch = function (start, end, filterOwnerId, filterOutValidation, cachedVersionId) {
        var r0 = this._decorationsTree0.intervalSearch(start, end, filterOwnerId, filterOutValidation, cachedVersionId);
        var r1 = this._decorationsTree1.intervalSearch(start, end, filterOwnerId, filterOutValidation, cachedVersionId);
        return r0.concat(r1);
    };
    DecorationsTrees.prototype.search = function (filterOwnerId, filterOutValidation, overviewRulerOnly, cachedVersionId) {
        if (overviewRulerOnly) {
            return this._decorationsTree1.search(filterOwnerId, filterOutValidation, cachedVersionId);
        }
        else {
            var r0 = this._decorationsTree0.search(filterOwnerId, filterOutValidation, cachedVersionId);
            var r1 = this._decorationsTree1.search(filterOwnerId, filterOutValidation, cachedVersionId);
            return r0.concat(r1);
        }
    };
    DecorationsTrees.prototype.collectNodesFromOwner = function (ownerId) {
        var r0 = this._decorationsTree0.collectNodesFromOwner(ownerId);
        var r1 = this._decorationsTree1.collectNodesFromOwner(ownerId);
        return r0.concat(r1);
    };
    DecorationsTrees.prototype.collectNodesPostOrder = function () {
        var r0 = this._decorationsTree0.collectNodesPostOrder();
        var r1 = this._decorationsTree1.collectNodesPostOrder();
        return r0.concat(r1);
    };
    DecorationsTrees.prototype.insert = function (node) {
        if (getNodeIsInOverviewRuler(node)) {
            this._decorationsTree1.insert(node);
        }
        else {
            this._decorationsTree0.insert(node);
        }
    };
    DecorationsTrees.prototype.delete = function (node) {
        if (getNodeIsInOverviewRuler(node)) {
            this._decorationsTree1.delete(node);
        }
        else {
            this._decorationsTree0.delete(node);
        }
    };
    DecorationsTrees.prototype.resolveNode = function (node, cachedVersionId) {
        if (getNodeIsInOverviewRuler(node)) {
            this._decorationsTree1.resolveNode(node, cachedVersionId);
        }
        else {
            this._decorationsTree0.resolveNode(node, cachedVersionId);
        }
    };
    DecorationsTrees.prototype.acceptReplace = function (offset, length, textLength, forceMoveMarkers) {
        this._decorationsTree0.acceptReplace(offset, length, textLength, forceMoveMarkers);
        this._decorationsTree1.acceptReplace(offset, length, textLength, forceMoveMarkers);
    };
    return DecorationsTrees;
}());
function cleanClassName(className) {
    return className.replace(/[^a-z0-9\-_]/gi, ' ');
}
var ModelDecorationOverviewRulerOptions = /** @class */ (function () {
    function ModelDecorationOverviewRulerOptions(options) {
        this.color = options.color || empty;
        this.darkColor = options.darkColor || empty;
        this.position = (typeof options.position === 'number' ? options.position : OverviewRulerLane.Center);
        this._resolvedColor = null;
    }
    ModelDecorationOverviewRulerOptions.prototype.getColor = function (theme) {
        if (!this._resolvedColor) {
            if (theme.type !== 'light' && this.darkColor) {
                this._resolvedColor = this._resolveColor(this.darkColor, theme);
            }
            else {
                this._resolvedColor = this._resolveColor(this.color, theme);
            }
        }
        return this._resolvedColor;
    };
    ModelDecorationOverviewRulerOptions.prototype.invalidateCachedColor = function () {
        this._resolvedColor = null;
    };
    ModelDecorationOverviewRulerOptions.prototype._resolveColor = function (color, theme) {
        if (typeof color === 'string') {
            return color;
        }
        var c = color ? theme.getColor(color.id) : null;
        if (!c) {
            return empty;
        }
        return c.toString();
    };
    return ModelDecorationOverviewRulerOptions;
}());
var ModelDecorationOptions = /** @class */ (function () {
    function ModelDecorationOptions(options) {
        this.stickiness = options.stickiness || 0 /* AlwaysGrowsWhenTypingAtEdges */;
        this.zIndex = options.zIndex || 0;
        this.className = options.className ? cleanClassName(options.className) : null;
        this.hoverMessage = options.hoverMessage || null;
        this.glyphMarginHoverMessage = options.glyphMarginHoverMessage || null;
        this.isWholeLine = options.isWholeLine || false;
        this.showIfCollapsed = options.showIfCollapsed || false;
        this.collapseOnReplaceEdit = options.collapseOnReplaceEdit || false;
        this.overviewRuler = options.overviewRuler ? new ModelDecorationOverviewRulerOptions(options.overviewRuler) : null;
        this.glyphMarginClassName = options.glyphMarginClassName ? cleanClassName(options.glyphMarginClassName) : null;
        this.linesDecorationsClassName = options.linesDecorationsClassName ? cleanClassName(options.linesDecorationsClassName) : null;
        this.marginClassName = options.marginClassName ? cleanClassName(options.marginClassName) : null;
        this.inlineClassName = options.inlineClassName ? cleanClassName(options.inlineClassName) : null;
        this.inlineClassNameAffectsLetterSpacing = options.inlineClassNameAffectsLetterSpacing || false;
        this.beforeContentClassName = options.beforeContentClassName ? cleanClassName(options.beforeContentClassName) : null;
        this.afterContentClassName = options.afterContentClassName ? cleanClassName(options.afterContentClassName) : null;
    }
    ModelDecorationOptions.register = function (options) {
        return new ModelDecorationOptions(options);
    };
    ModelDecorationOptions.createDynamic = function (options) {
        return new ModelDecorationOptions(options);
    };
    return ModelDecorationOptions;
}());
ModelDecorationOptions.EMPTY = ModelDecorationOptions.register({});
/**
 * The order carefully matches the values of the enum.
 */
var TRACKED_RANGE_OPTIONS = [
    ModelDecorationOptions.register({ stickiness: 0 /* AlwaysGrowsWhenTypingAtEdges */ }),
    ModelDecorationOptions.register({ stickiness: 1 /* NeverGrowsWhenTypingAtEdges */ }),
    ModelDecorationOptions.register({ stickiness: 2 /* GrowsOnlyWhenTypingBefore */ }),
    ModelDecorationOptions.register({ stickiness: 3 /* GrowsOnlyWhenTypingAfter */ }),
];
function _normalizeOptions(options) {
    if (options instanceof ModelDecorationOptions) {
        return options;
    }
    return ModelDecorationOptions.createDynamic(options);
}
var DidChangeDecorationsEmitter = /** @class */ (function (_super) {
    __extends$a(DidChangeDecorationsEmitter, _super);
    function DidChangeDecorationsEmitter() {
        var _this = _super.call(this) || this;
        _this._actual = _this._register(new Emitter());
        _this.event = _this._actual.event;
        _this._deferredCnt = 0;
        _this._shouldFire = false;
        return _this;
    }
    DidChangeDecorationsEmitter.prototype.beginDeferredEmit = function () {
        this._deferredCnt++;
    };
    DidChangeDecorationsEmitter.prototype.endDeferredEmit = function () {
        this._deferredCnt--;
        if (this._deferredCnt === 0) {
            if (this._shouldFire) {
                this._shouldFire = false;
                this._actual.fire({});
            }
        }
    };
    DidChangeDecorationsEmitter.prototype.fire = function () {
        this._shouldFire = true;
    };
    return DidChangeDecorationsEmitter;
}(Disposable));
//#endregion
var DidChangeContentEmitter = /** @class */ (function (_super) {
    __extends$a(DidChangeContentEmitter, _super);
    function DidChangeContentEmitter() {
        var _this = _super.call(this) || this;
        /**
         * Both `fastEvent` and `slowEvent` work the same way and contain the same events, but first we invoke `fastEvent` and then `slowEvent`.
         */
        _this._fastEmitter = _this._register(new Emitter());
        _this.fastEvent = _this._fastEmitter.event;
        _this._slowEmitter = _this._register(new Emitter());
        _this.slowEvent = _this._slowEmitter.event;
        _this._deferredCnt = 0;
        _this._deferredEvent = null;
        return _this;
    }
    DidChangeContentEmitter.prototype.beginDeferredEmit = function () {
        this._deferredCnt++;
    };
    DidChangeContentEmitter.prototype.endDeferredEmit = function () {
        this._deferredCnt--;
        if (this._deferredCnt === 0) {
            if (this._deferredEvent !== null) {
                var e = this._deferredEvent;
                this._deferredEvent = null;
                this._fastEmitter.fire(e);
                this._slowEmitter.fire(e);
            }
        }
    };
    DidChangeContentEmitter.prototype.fire = function (e) {
        if (this._deferredCnt > 0) {
            if (this._deferredEvent) {
                this._deferredEvent = this._deferredEvent.merge(e);
            }
            else {
                this._deferredEvent = e;
            }
            return;
        }
        this._fastEmitter.fire(e);
        this._slowEmitter.fire(e);
    };
    return DidChangeContentEmitter;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var autoCloseAlways = function (_) { return true; };
var autoCloseNever = function (_) { return false; };
var autoCloseBeforeWhitespace = function (chr) { return (chr === ' ' || chr === '\t'); };
var CursorConfiguration = /** @class */ (function () {
    function CursorConfiguration(languageIdentifier, oneIndent, modelOptions, configuration) {
        this._languageIdentifier = languageIdentifier;
        var c = configuration.editor;
        this.readOnly = c.readOnly;
        this.tabSize = modelOptions.tabSize;
        this.insertSpaces = modelOptions.insertSpaces;
        this.oneIndent = oneIndent;
        this.pageSize = Math.max(1, Math.floor(c.layoutInfo.height / c.fontInfo.lineHeight) - 2);
        this.lineHeight = c.lineHeight;
        this.useTabStops = c.useTabStops;
        this.wordSeparators = c.wordSeparators;
        this.emptySelectionClipboard = c.emptySelectionClipboard;
        this.copyWithSyntaxHighlighting = c.copyWithSyntaxHighlighting;
        this.multiCursorMergeOverlapping = c.multiCursorMergeOverlapping;
        this.autoClosingBrackets = c.autoClosingBrackets;
        this.autoClosingQuotes = c.autoClosingQuotes;
        this.autoSurround = c.autoSurround;
        this.autoIndent = c.autoIndent;
        this.autoClosingPairsOpen = {};
        this.autoClosingPairsClose = {};
        this.surroundingPairs = {};
        this._electricChars = null;
        this.shouldAutoCloseBefore = {
            quote: CursorConfiguration._getShouldAutoClose(languageIdentifier, this.autoClosingQuotes),
            bracket: CursorConfiguration._getShouldAutoClose(languageIdentifier, this.autoClosingBrackets)
        };
        var autoClosingPairs = CursorConfiguration._getAutoClosingPairs(languageIdentifier);
        if (autoClosingPairs) {
            for (var i = 0; i < autoClosingPairs.length; i++) {
                this.autoClosingPairsOpen[autoClosingPairs[i].open] = autoClosingPairs[i].close;
                this.autoClosingPairsClose[autoClosingPairs[i].close] = autoClosingPairs[i].open;
            }
        }
        var surroundingPairs = CursorConfiguration._getSurroundingPairs(languageIdentifier);
        if (surroundingPairs) {
            for (var i = 0; i < surroundingPairs.length; i++) {
                this.surroundingPairs[surroundingPairs[i].open] = surroundingPairs[i].close;
            }
        }
    }
    CursorConfiguration.shouldRecreate = function (e) {
        return (e.layoutInfo
            || e.wordSeparators
            || e.emptySelectionClipboard
            || e.multiCursorMergeOverlapping
            || e.autoClosingBrackets
            || e.autoClosingQuotes
            || e.autoSurround
            || e.useTabStops
            || e.lineHeight
            || e.readOnly);
    };
    Object.defineProperty(CursorConfiguration.prototype, "electricChars", {
        get: function () {
            if (!this._electricChars) {
                this._electricChars = {};
                var electricChars = CursorConfiguration._getElectricCharacters(this._languageIdentifier);
                if (electricChars) {
                    for (var i = 0; i < electricChars.length; i++) {
                        this._electricChars[electricChars[i]] = true;
                    }
                }
            }
            return this._electricChars;
        },
        enumerable: true,
        configurable: true
    });
    CursorConfiguration.prototype.normalizeIndentation = function (str) {
        return TextModel.normalizeIndentation(str, this.tabSize, this.insertSpaces);
    };
    CursorConfiguration._getElectricCharacters = function (languageIdentifier) {
        try {
            return LanguageConfigurationRegistry.getElectricCharacters(languageIdentifier.id);
        }
        catch (e) {
            onUnexpectedError(e);
            return null;
        }
    };
    CursorConfiguration._getAutoClosingPairs = function (languageIdentifier) {
        try {
            return LanguageConfigurationRegistry.getAutoClosingPairs(languageIdentifier.id);
        }
        catch (e) {
            onUnexpectedError(e);
            return null;
        }
    };
    CursorConfiguration._getShouldAutoClose = function (languageIdentifier, autoCloseConfig) {
        switch (autoCloseConfig) {
            case 'beforeWhitespace':
                return autoCloseBeforeWhitespace;
            case 'languageDefined':
                return CursorConfiguration._getLanguageDefinedShouldAutoClose(languageIdentifier);
            case 'always':
                return autoCloseAlways;
            case 'never':
                return autoCloseNever;
        }
    };
    CursorConfiguration._getLanguageDefinedShouldAutoClose = function (languageIdentifier) {
        try {
            var autoCloseBeforeSet_1 = LanguageConfigurationRegistry.getAutoCloseBeforeSet(languageIdentifier.id);
            return function (c) { return autoCloseBeforeSet_1.indexOf(c) !== -1; };
        }
        catch (e) {
            onUnexpectedError(e);
            return autoCloseNever;
        }
    };
    CursorConfiguration._getSurroundingPairs = function (languageIdentifier) {
        try {
            return LanguageConfigurationRegistry.getSurroundingPairs(languageIdentifier.id);
        }
        catch (e) {
            onUnexpectedError(e);
            return null;
        }
    };
    return CursorConfiguration;
}());
/**
 * Represents the cursor state on either the model or on the view model.
 */
var SingleCursorState = /** @class */ (function () {
    function SingleCursorState(selectionStart, selectionStartLeftoverVisibleColumns, position, leftoverVisibleColumns) {
        this.selectionStart = selectionStart;
        this.selectionStartLeftoverVisibleColumns = selectionStartLeftoverVisibleColumns;
        this.position = position;
        this.leftoverVisibleColumns = leftoverVisibleColumns;
        this.selection = SingleCursorState._computeSelection(this.selectionStart, this.position);
    }
    SingleCursorState.prototype.equals = function (other) {
        return (this.selectionStartLeftoverVisibleColumns === other.selectionStartLeftoverVisibleColumns
            && this.leftoverVisibleColumns === other.leftoverVisibleColumns
            && this.position.equals(other.position)
            && this.selectionStart.equalsRange(other.selectionStart));
    };
    SingleCursorState.prototype.hasSelection = function () {
        return (!this.selection.isEmpty() || !this.selectionStart.isEmpty());
    };
    SingleCursorState.prototype.move = function (inSelectionMode, lineNumber, column, leftoverVisibleColumns) {
        if (inSelectionMode) {
            // move just position
            return new SingleCursorState(this.selectionStart, this.selectionStartLeftoverVisibleColumns, new Position(lineNumber, column), leftoverVisibleColumns);
        }
        else {
            // move everything
            return new SingleCursorState(new Range(lineNumber, column, lineNumber, column), leftoverVisibleColumns, new Position(lineNumber, column), leftoverVisibleColumns);
        }
    };
    SingleCursorState._computeSelection = function (selectionStart, position) {
        var startLineNumber, startColumn, endLineNumber, endColumn;
        if (selectionStart.isEmpty()) {
            startLineNumber = selectionStart.startLineNumber;
            startColumn = selectionStart.startColumn;
            endLineNumber = position.lineNumber;
            endColumn = position.column;
        }
        else {
            if (position.isBeforeOrEqual(selectionStart.getStartPosition())) {
                startLineNumber = selectionStart.endLineNumber;
                startColumn = selectionStart.endColumn;
                endLineNumber = position.lineNumber;
                endColumn = position.column;
            }
            else {
                startLineNumber = selectionStart.startLineNumber;
                startColumn = selectionStart.startColumn;
                endLineNumber = position.lineNumber;
                endColumn = position.column;
            }
        }
        return new Selection(startLineNumber, startColumn, endLineNumber, endColumn);
    };
    return SingleCursorState;
}());
var CursorContext = /** @class */ (function () {
    function CursorContext(configuration, model, viewModel) {
        this.model = model;
        this.viewModel = viewModel;
        this.config = new CursorConfiguration(this.model.getLanguageIdentifier(), this.model.getOneIndent(), this.model.getOptions(), configuration);
    }
    CursorContext.prototype.validateViewPosition = function (viewPosition, modelPosition) {
        return this.viewModel.coordinatesConverter.validateViewPosition(viewPosition, modelPosition);
    };
    CursorContext.prototype.validateViewRange = function (viewRange, expectedModelRange) {
        return this.viewModel.coordinatesConverter.validateViewRange(viewRange, expectedModelRange);
    };
    CursorContext.prototype.convertViewRangeToModelRange = function (viewRange) {
        return this.viewModel.coordinatesConverter.convertViewRangeToModelRange(viewRange);
    };
    CursorContext.prototype.convertViewPositionToModelPosition = function (lineNumber, column) {
        return this.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new Position(lineNumber, column));
    };
    CursorContext.prototype.convertModelPositionToViewPosition = function (modelPosition) {
        return this.viewModel.coordinatesConverter.convertModelPositionToViewPosition(modelPosition);
    };
    CursorContext.prototype.convertModelRangeToViewRange = function (modelRange) {
        return this.viewModel.coordinatesConverter.convertModelRangeToViewRange(modelRange);
    };
    CursorContext.prototype.getCurrentScrollTop = function () {
        return this.viewModel.viewLayout.getCurrentScrollTop();
    };
    CursorContext.prototype.getCompletelyVisibleViewRange = function () {
        return this.viewModel.getCompletelyVisibleViewRange();
    };
    CursorContext.prototype.getCompletelyVisibleModelRange = function () {
        var viewRange = this.viewModel.getCompletelyVisibleViewRange();
        return this.viewModel.coordinatesConverter.convertViewRangeToModelRange(viewRange);
    };
    CursorContext.prototype.getCompletelyVisibleViewRangeAtScrollTop = function (scrollTop) {
        return this.viewModel.getCompletelyVisibleViewRangeAtScrollTop(scrollTop);
    };
    CursorContext.prototype.getVerticalOffsetForViewLine = function (viewLineNumber) {
        return this.viewModel.viewLayout.getVerticalOffsetForLineNumber(viewLineNumber);
    };
    return CursorContext;
}());
var PartialModelCursorState = /** @class */ (function () {
    function PartialModelCursorState(modelState) {
        this.modelState = modelState;
        this.viewState = null;
    }
    return PartialModelCursorState;
}());
var PartialViewCursorState = /** @class */ (function () {
    function PartialViewCursorState(viewState) {
        this.modelState = null;
        this.viewState = viewState;
    }
    return PartialViewCursorState;
}());
var CursorState = /** @class */ (function () {
    function CursorState(modelState, viewState) {
        this.modelState = modelState;
        this.viewState = viewState;
    }
    CursorState.fromModelState = function (modelState) {
        return new PartialModelCursorState(modelState);
    };
    CursorState.fromViewState = function (viewState) {
        return new PartialViewCursorState(viewState);
    };
    CursorState.fromModelSelection = function (modelSelection) {
        var selectionStartLineNumber = modelSelection.selectionStartLineNumber;
        var selectionStartColumn = modelSelection.selectionStartColumn;
        var positionLineNumber = modelSelection.positionLineNumber;
        var positionColumn = modelSelection.positionColumn;
        var modelState = new SingleCursorState(new Range(selectionStartLineNumber, selectionStartColumn, selectionStartLineNumber, selectionStartColumn), 0, new Position(positionLineNumber, positionColumn), 0);
        return CursorState.fromModelState(modelState);
    };
    CursorState.fromModelSelections = function (modelSelections) {
        var states = [];
        for (var i = 0, len = modelSelections.length; i < len; i++) {
            states[i] = this.fromModelSelection(modelSelections[i]);
        }
        return states;
    };
    CursorState.prototype.equals = function (other) {
        return (this.viewState.equals(other.viewState) && this.modelState.equals(other.modelState));
    };
    return CursorState;
}());
var EditOperationResult = /** @class */ (function () {
    function EditOperationResult(type, commands, opts) {
        this.type = type;
        this.commands = commands;
        this.shouldPushStackElementBefore = opts.shouldPushStackElementBefore;
        this.shouldPushStackElementAfter = opts.shouldPushStackElementAfter;
    }
    return EditOperationResult;
}());
/**
 * Common operations that work and make sense both on the model and on the view model.
 */
var CursorColumns = /** @class */ (function () {
    function CursorColumns() {
    }
    CursorColumns.isLowSurrogate = function (model, lineNumber, charOffset) {
        var lineContent = model.getLineContent(lineNumber);
        if (charOffset < 0 || charOffset >= lineContent.length) {
            return false;
        }
        return isLowSurrogate(lineContent.charCodeAt(charOffset));
    };
    CursorColumns.isHighSurrogate = function (model, lineNumber, charOffset) {
        var lineContent = model.getLineContent(lineNumber);
        if (charOffset < 0 || charOffset >= lineContent.length) {
            return false;
        }
        return isHighSurrogate(lineContent.charCodeAt(charOffset));
    };
    CursorColumns.isInsideSurrogatePair = function (model, lineNumber, column) {
        return this.isHighSurrogate(model, lineNumber, column - 2);
    };
    CursorColumns.visibleColumnFromColumn = function (lineContent, column, tabSize) {
        var endOffset = lineContent.length;
        if (endOffset > column - 1) {
            endOffset = column - 1;
        }
        var result = 0;
        for (var i = 0; i < endOffset; i++) {
            var charCode = lineContent.charCodeAt(i);
            if (charCode === 9 /* Tab */) {
                result = this.nextTabStop(result, tabSize);
            }
            else if (isFullWidthCharacter(charCode)) {
                result = result + 2;
            }
            else {
                result = result + 1;
            }
        }
        return result;
    };
    CursorColumns.visibleColumnFromColumn2 = function (config, model, position) {
        return this.visibleColumnFromColumn(model.getLineContent(position.lineNumber), position.column, config.tabSize);
    };
    CursorColumns.columnFromVisibleColumn = function (lineContent, visibleColumn, tabSize) {
        if (visibleColumn <= 0) {
            return 1;
        }
        var lineLength = lineContent.length;
        var beforeVisibleColumn = 0;
        for (var i = 0; i < lineLength; i++) {
            var charCode = lineContent.charCodeAt(i);
            var afterVisibleColumn = void 0;
            if (charCode === 9 /* Tab */) {
                afterVisibleColumn = this.nextTabStop(beforeVisibleColumn, tabSize);
            }
            else if (isFullWidthCharacter(charCode)) {
                afterVisibleColumn = beforeVisibleColumn + 2;
            }
            else {
                afterVisibleColumn = beforeVisibleColumn + 1;
            }
            if (afterVisibleColumn >= visibleColumn) {
                var prevDelta = visibleColumn - beforeVisibleColumn;
                var afterDelta = afterVisibleColumn - visibleColumn;
                if (afterDelta < prevDelta) {
                    return i + 2;
                }
                else {
                    return i + 1;
                }
            }
            beforeVisibleColumn = afterVisibleColumn;
        }
        // walked the entire string
        return lineLength + 1;
    };
    CursorColumns.columnFromVisibleColumn2 = function (config, model, lineNumber, visibleColumn) {
        var result = this.columnFromVisibleColumn(model.getLineContent(lineNumber), visibleColumn, config.tabSize);
        var minColumn = model.getLineMinColumn(lineNumber);
        if (result < minColumn) {
            return minColumn;
        }
        var maxColumn = model.getLineMaxColumn(lineNumber);
        if (result > maxColumn) {
            return maxColumn;
        }
        return result;
    };
    /**
     * ATTENTION: This works with 0-based columns (as oposed to the regular 1-based columns)
     */
    CursorColumns.nextTabStop = function (visibleColumn, tabSize) {
        return visibleColumn + tabSize - visibleColumn % tabSize;
    };
    /**
     * ATTENTION: This works with 0-based columns (as oposed to the regular 1-based columns)
     */
    CursorColumns.prevTabStop = function (column, tabSize) {
        return column - 1 - (column - 1) % tabSize;
    };
    return CursorColumns;
}());
function isQuote(ch) {
    return (ch === '\'' || ch === '"' || ch === '`');
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ColumnSelection = /** @class */ (function () {
    function ColumnSelection() {
    }
    ColumnSelection._columnSelect = function (config, model, fromLineNumber, fromVisibleColumn, toLineNumber, toVisibleColumn) {
        var lineCount = Math.abs(toLineNumber - fromLineNumber) + 1;
        var reversed = (fromLineNumber > toLineNumber);
        var isRTL = (fromVisibleColumn > toVisibleColumn);
        var isLTR = (fromVisibleColumn < toVisibleColumn);
        var result = [];
        // console.log(`fromVisibleColumn: ${fromVisibleColumn}, toVisibleColumn: ${toVisibleColumn}`);
        for (var i = 0; i < lineCount; i++) {
            var lineNumber = fromLineNumber + (reversed ? -i : i);
            var startColumn = CursorColumns.columnFromVisibleColumn2(config, model, lineNumber, fromVisibleColumn);
            var endColumn = CursorColumns.columnFromVisibleColumn2(config, model, lineNumber, toVisibleColumn);
            var visibleStartColumn = CursorColumns.visibleColumnFromColumn2(config, model, new Position(lineNumber, startColumn));
            var visibleEndColumn = CursorColumns.visibleColumnFromColumn2(config, model, new Position(lineNumber, endColumn));
            // console.log(`lineNumber: ${lineNumber}: visibleStartColumn: ${visibleStartColumn}, visibleEndColumn: ${visibleEndColumn}`);
            if (isLTR) {
                if (visibleStartColumn > toVisibleColumn) {
                    continue;
                }
                if (visibleEndColumn < fromVisibleColumn) {
                    continue;
                }
            }
            if (isRTL) {
                if (visibleEndColumn > fromVisibleColumn) {
                    continue;
                }
                if (visibleStartColumn < toVisibleColumn) {
                    continue;
                }
            }
            result.push(new SingleCursorState(new Range(lineNumber, startColumn, lineNumber, startColumn), 0, new Position(lineNumber, endColumn), 0));
        }
        return {
            viewStates: result,
            reversed: reversed,
            toLineNumber: toLineNumber,
            toVisualColumn: toVisibleColumn
        };
    };
    ColumnSelection.columnSelect = function (config, model, fromViewSelection, toViewLineNumber, toViewVisualColumn) {
        var fromViewPosition = new Position(fromViewSelection.selectionStartLineNumber, fromViewSelection.selectionStartColumn);
        var fromViewVisibleColumn = CursorColumns.visibleColumnFromColumn2(config, model, fromViewPosition);
        return ColumnSelection._columnSelect(config, model, fromViewPosition.lineNumber, fromViewVisibleColumn, toViewLineNumber, toViewVisualColumn);
    };
    ColumnSelection.columnSelectLeft = function (config, model, cursor, toViewLineNumber, toViewVisualColumn) {
        if (toViewVisualColumn > 1) {
            toViewVisualColumn--;
        }
        return this.columnSelect(config, model, cursor.selection, toViewLineNumber, toViewVisualColumn);
    };
    ColumnSelection.columnSelectRight = function (config, model, cursor, toViewLineNumber, toViewVisualColumn) {
        var maxVisualViewColumn = 0;
        var minViewLineNumber = Math.min(cursor.position.lineNumber, toViewLineNumber);
        var maxViewLineNumber = Math.max(cursor.position.lineNumber, toViewLineNumber);
        for (var lineNumber = minViewLineNumber; lineNumber <= maxViewLineNumber; lineNumber++) {
            var lineMaxViewColumn = model.getLineMaxColumn(lineNumber);
            var lineMaxVisualViewColumn = CursorColumns.visibleColumnFromColumn2(config, model, new Position(lineNumber, lineMaxViewColumn));
            maxVisualViewColumn = Math.max(maxVisualViewColumn, lineMaxVisualViewColumn);
        }
        if (toViewVisualColumn < maxVisualViewColumn) {
            toViewVisualColumn++;
        }
        return this.columnSelect(config, model, cursor.selection, toViewLineNumber, toViewVisualColumn);
    };
    ColumnSelection.columnSelectUp = function (config, model, cursor, isPaged, toViewLineNumber, toViewVisualColumn) {
        var linesCount = isPaged ? config.pageSize : 1;
        toViewLineNumber -= linesCount;
        if (toViewLineNumber < 1) {
            toViewLineNumber = 1;
        }
        return this.columnSelect(config, model, cursor.selection, toViewLineNumber, toViewVisualColumn);
    };
    ColumnSelection.columnSelectDown = function (config, model, cursor, isPaged, toViewLineNumber, toViewVisualColumn) {
        var linesCount = isPaged ? config.pageSize : 1;
        toViewLineNumber += linesCount;
        if (toViewLineNumber > model.getLineCount()) {
            toViewLineNumber = model.getLineCount();
        }
        return this.columnSelect(config, model, cursor.selection, toViewLineNumber, toViewVisualColumn);
    };
    return ColumnSelection;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ReplaceCommand = /** @class */ (function () {
    function ReplaceCommand(range, text, insertsAutoWhitespace) {
        if (insertsAutoWhitespace === void 0) { insertsAutoWhitespace = false; }
        this._range = range;
        this._text = text;
        this.insertsAutoWhitespace = insertsAutoWhitespace;
    }
    ReplaceCommand.prototype.getEditOperations = function (model, builder) {
        builder.addTrackedEditOperation(this._range, this._text);
    };
    ReplaceCommand.prototype.computeCursorState = function (model, helper) {
        var inverseEditOperations = helper.getInverseEditOperations();
        var srcRange = inverseEditOperations[0].range;
        return new Selection(srcRange.endLineNumber, srcRange.endColumn, srcRange.endLineNumber, srcRange.endColumn);
    };
    return ReplaceCommand;
}());
var ReplaceCommandWithoutChangingPosition = /** @class */ (function () {
    function ReplaceCommandWithoutChangingPosition(range, text, insertsAutoWhitespace) {
        if (insertsAutoWhitespace === void 0) { insertsAutoWhitespace = false; }
        this._range = range;
        this._text = text;
        this.insertsAutoWhitespace = insertsAutoWhitespace;
    }
    ReplaceCommandWithoutChangingPosition.prototype.getEditOperations = function (model, builder) {
        builder.addTrackedEditOperation(this._range, this._text);
    };
    ReplaceCommandWithoutChangingPosition.prototype.computeCursorState = function (model, helper) {
        var inverseEditOperations = helper.getInverseEditOperations();
        var srcRange = inverseEditOperations[0].range;
        return new Selection(srcRange.startLineNumber, srcRange.startColumn, srcRange.startLineNumber, srcRange.startColumn);
    };
    return ReplaceCommandWithoutChangingPosition;
}());
var ReplaceCommandWithOffsetCursorState = /** @class */ (function () {
    function ReplaceCommandWithOffsetCursorState(range, text, lineNumberDeltaOffset, columnDeltaOffset, insertsAutoWhitespace) {
        if (insertsAutoWhitespace === void 0) { insertsAutoWhitespace = false; }
        this._range = range;
        this._text = text;
        this._columnDeltaOffset = columnDeltaOffset;
        this._lineNumberDeltaOffset = lineNumberDeltaOffset;
        this.insertsAutoWhitespace = insertsAutoWhitespace;
    }
    ReplaceCommandWithOffsetCursorState.prototype.getEditOperations = function (model, builder) {
        builder.addTrackedEditOperation(this._range, this._text);
    };
    ReplaceCommandWithOffsetCursorState.prototype.computeCursorState = function (model, helper) {
        var inverseEditOperations = helper.getInverseEditOperations();
        var srcRange = inverseEditOperations[0].range;
        return new Selection(srcRange.endLineNumber + this._lineNumberDeltaOffset, srcRange.endColumn + this._columnDeltaOffset, srcRange.endLineNumber + this._lineNumberDeltaOffset, srcRange.endColumn + this._columnDeltaOffset);
    };
    return ReplaceCommandWithOffsetCursorState;
}());
var ReplaceCommandThatPreservesSelection = /** @class */ (function () {
    function ReplaceCommandThatPreservesSelection(editRange, text, initialSelection) {
        this._range = editRange;
        this._text = text;
        this._initialSelection = initialSelection;
    }
    ReplaceCommandThatPreservesSelection.prototype.getEditOperations = function (model, builder) {
        builder.addEditOperation(this._range, this._text);
        this._selectionId = builder.trackSelection(this._initialSelection);
    };
    ReplaceCommandThatPreservesSelection.prototype.computeCursorState = function (model, helper) {
        return helper.getTrackedSelection(this._selectionId);
    };
    return ReplaceCommandThatPreservesSelection;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CursorPosition = /** @class */ (function () {
    function CursorPosition(lineNumber, column, leftoverVisibleColumns) {
        this.lineNumber = lineNumber;
        this.column = column;
        this.leftoverVisibleColumns = leftoverVisibleColumns;
    }
    return CursorPosition;
}());
var MoveOperations = /** @class */ (function () {
    function MoveOperations() {
    }
    MoveOperations.left = function (config, model, lineNumber, column) {
        if (column > model.getLineMinColumn(lineNumber)) {
            if (CursorColumns.isLowSurrogate(model, lineNumber, column - 2)) {
                // character before column is a low surrogate
                column = column - 2;
            }
            else {
                column = column - 1;
            }
        }
        else if (lineNumber > 1) {
            lineNumber = lineNumber - 1;
            column = model.getLineMaxColumn(lineNumber);
        }
        return new CursorPosition(lineNumber, column, 0);
    };
    MoveOperations.moveLeft = function (config, model, cursor, inSelectionMode, noOfColumns) {
        var lineNumber, column;
        if (cursor.hasSelection() && !inSelectionMode) {
            // If we are in selection mode, move left without selection cancels selection and puts cursor at the beginning of the selection
            lineNumber = cursor.selection.startLineNumber;
            column = cursor.selection.startColumn;
        }
        else {
            var r = MoveOperations.left(config, model, cursor.position.lineNumber, cursor.position.column - (noOfColumns - 1));
            lineNumber = r.lineNumber;
            column = r.column;
        }
        return cursor.move(inSelectionMode, lineNumber, column, 0);
    };
    MoveOperations.right = function (config, model, lineNumber, column) {
        if (column < model.getLineMaxColumn(lineNumber)) {
            if (CursorColumns.isHighSurrogate(model, lineNumber, column - 1)) {
                // character after column is a high surrogate
                column = column + 2;
            }
            else {
                column = column + 1;
            }
        }
        else if (lineNumber < model.getLineCount()) {
            lineNumber = lineNumber + 1;
            column = model.getLineMinColumn(lineNumber);
        }
        return new CursorPosition(lineNumber, column, 0);
    };
    MoveOperations.moveRight = function (config, model, cursor, inSelectionMode, noOfColumns) {
        var lineNumber, column;
        if (cursor.hasSelection() && !inSelectionMode) {
            // If we are in selection mode, move right without selection cancels selection and puts cursor at the end of the selection
            lineNumber = cursor.selection.endLineNumber;
            column = cursor.selection.endColumn;
        }
        else {
            var r = MoveOperations.right(config, model, cursor.position.lineNumber, cursor.position.column + (noOfColumns - 1));
            lineNumber = r.lineNumber;
            column = r.column;
        }
        return cursor.move(inSelectionMode, lineNumber, column, 0);
    };
    MoveOperations.down = function (config, model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine) {
        var currentVisibleColumn = CursorColumns.visibleColumnFromColumn(model.getLineContent(lineNumber), column, config.tabSize) + leftoverVisibleColumns;
        lineNumber = lineNumber + count;
        var lineCount = model.getLineCount();
        if (lineNumber > lineCount) {
            lineNumber = lineCount;
            if (allowMoveOnLastLine) {
                column = model.getLineMaxColumn(lineNumber);
            }
            else {
                column = Math.min(model.getLineMaxColumn(lineNumber), column);
                if (CursorColumns.isInsideSurrogatePair(model, lineNumber, column)) {
                    column = column - 1;
                }
            }
        }
        else {
            column = CursorColumns.columnFromVisibleColumn2(config, model, lineNumber, currentVisibleColumn);
            if (CursorColumns.isInsideSurrogatePair(model, lineNumber, column)) {
                column = column - 1;
            }
        }
        leftoverVisibleColumns = currentVisibleColumn - CursorColumns.visibleColumnFromColumn(model.getLineContent(lineNumber), column, config.tabSize);
        return new CursorPosition(lineNumber, column, leftoverVisibleColumns);
    };
    MoveOperations.moveDown = function (config, model, cursor, inSelectionMode, linesCount) {
        var lineNumber, column;
        if (cursor.hasSelection() && !inSelectionMode) {
            // If we are in selection mode, move down acts relative to the end of selection
            lineNumber = cursor.selection.endLineNumber;
            column = cursor.selection.endColumn;
        }
        else {
            lineNumber = cursor.position.lineNumber;
            column = cursor.position.column;
        }
        var r = MoveOperations.down(config, model, lineNumber, column, cursor.leftoverVisibleColumns, linesCount, true);
        return cursor.move(inSelectionMode, r.lineNumber, r.column, r.leftoverVisibleColumns);
    };
    MoveOperations.translateDown = function (config, model, cursor) {
        var selection = cursor.selection;
        var selectionStart = MoveOperations.down(config, model, selection.selectionStartLineNumber, selection.selectionStartColumn, cursor.selectionStartLeftoverVisibleColumns, 1, false);
        var position = MoveOperations.down(config, model, selection.positionLineNumber, selection.positionColumn, cursor.leftoverVisibleColumns, 1, false);
        return new SingleCursorState(new Range(selectionStart.lineNumber, selectionStart.column, selectionStart.lineNumber, selectionStart.column), selectionStart.leftoverVisibleColumns, new Position(position.lineNumber, position.column), position.leftoverVisibleColumns);
    };
    MoveOperations.up = function (config, model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine) {
        var currentVisibleColumn = CursorColumns.visibleColumnFromColumn(model.getLineContent(lineNumber), column, config.tabSize) + leftoverVisibleColumns;
        lineNumber = lineNumber - count;
        if (lineNumber < 1) {
            lineNumber = 1;
            if (allowMoveOnFirstLine) {
                column = model.getLineMinColumn(lineNumber);
            }
            else {
                column = Math.min(model.getLineMaxColumn(lineNumber), column);
                if (CursorColumns.isInsideSurrogatePair(model, lineNumber, column)) {
                    column = column - 1;
                }
            }
        }
        else {
            column = CursorColumns.columnFromVisibleColumn2(config, model, lineNumber, currentVisibleColumn);
            if (CursorColumns.isInsideSurrogatePair(model, lineNumber, column)) {
                column = column - 1;
            }
        }
        leftoverVisibleColumns = currentVisibleColumn - CursorColumns.visibleColumnFromColumn(model.getLineContent(lineNumber), column, config.tabSize);
        return new CursorPosition(lineNumber, column, leftoverVisibleColumns);
    };
    MoveOperations.moveUp = function (config, model, cursor, inSelectionMode, linesCount) {
        var lineNumber, column;
        if (cursor.hasSelection() && !inSelectionMode) {
            // If we are in selection mode, move up acts relative to the beginning of selection
            lineNumber = cursor.selection.startLineNumber;
            column = cursor.selection.startColumn;
        }
        else {
            lineNumber = cursor.position.lineNumber;
            column = cursor.position.column;
        }
        var r = MoveOperations.up(config, model, lineNumber, column, cursor.leftoverVisibleColumns, linesCount, true);
        return cursor.move(inSelectionMode, r.lineNumber, r.column, r.leftoverVisibleColumns);
    };
    MoveOperations.translateUp = function (config, model, cursor) {
        var selection = cursor.selection;
        var selectionStart = MoveOperations.up(config, model, selection.selectionStartLineNumber, selection.selectionStartColumn, cursor.selectionStartLeftoverVisibleColumns, 1, false);
        var position = MoveOperations.up(config, model, selection.positionLineNumber, selection.positionColumn, cursor.leftoverVisibleColumns, 1, false);
        return new SingleCursorState(new Range(selectionStart.lineNumber, selectionStart.column, selectionStart.lineNumber, selectionStart.column), selectionStart.leftoverVisibleColumns, new Position(position.lineNumber, position.column), position.leftoverVisibleColumns);
    };
    MoveOperations.moveToBeginningOfLine = function (config, model, cursor, inSelectionMode) {
        var lineNumber = cursor.position.lineNumber;
        var minColumn = model.getLineMinColumn(lineNumber);
        var firstNonBlankColumn = model.getLineFirstNonWhitespaceColumn(lineNumber) || minColumn;
        var column;
        var relevantColumnNumber = cursor.position.column;
        if (relevantColumnNumber === firstNonBlankColumn) {
            column = minColumn;
        }
        else {
            column = firstNonBlankColumn;
        }
        return cursor.move(inSelectionMode, lineNumber, column, 0);
    };
    MoveOperations.moveToEndOfLine = function (config, model, cursor, inSelectionMode) {
        var lineNumber = cursor.position.lineNumber;
        var maxColumn = model.getLineMaxColumn(lineNumber);
        return cursor.move(inSelectionMode, lineNumber, maxColumn, 0);
    };
    MoveOperations.moveToBeginningOfBuffer = function (config, model, cursor, inSelectionMode) {
        return cursor.move(inSelectionMode, 1, 1, 0);
    };
    MoveOperations.moveToEndOfBuffer = function (config, model, cursor, inSelectionMode) {
        var lastLineNumber = model.getLineCount();
        var lastColumn = model.getLineMaxColumn(lastLineNumber);
        return cursor.move(inSelectionMode, lastLineNumber, lastColumn, 0);
    };
    return MoveOperations;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var DeleteOperations = /** @class */ (function () {
    function DeleteOperations() {
    }
    DeleteOperations.deleteRight = function (prevEditOperationType, config, model, selections) {
        var commands = [];
        var shouldPushStackElementBefore = (prevEditOperationType !== 3 /* DeletingRight */);
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var deleteSelection = selection;
            if (deleteSelection.isEmpty()) {
                var position = selection.getPosition();
                var rightOfPosition = MoveOperations.right(config, model, position.lineNumber, position.column);
                deleteSelection = new Range(rightOfPosition.lineNumber, rightOfPosition.column, position.lineNumber, position.column);
            }
            if (deleteSelection.isEmpty()) {
                // Probably at end of file => ignore
                commands[i] = null;
                continue;
            }
            if (deleteSelection.startLineNumber !== deleteSelection.endLineNumber) {
                shouldPushStackElementBefore = true;
            }
            commands[i] = new ReplaceCommand(deleteSelection, '');
        }
        return [shouldPushStackElementBefore, commands];
    };
    DeleteOperations._isAutoClosingPairDelete = function (config, model, selections) {
        if (config.autoClosingBrackets === 'never' && config.autoClosingQuotes === 'never') {
            return false;
        }
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var position = selection.getPosition();
            if (!selection.isEmpty()) {
                return false;
            }
            var lineText = model.getLineContent(position.lineNumber);
            var character = lineText[position.column - 2];
            if (!config.autoClosingPairsOpen.hasOwnProperty(character)) {
                return false;
            }
            if (isQuote(character)) {
                if (config.autoClosingQuotes === 'never') {
                    return false;
                }
            }
            else {
                if (config.autoClosingBrackets === 'never') {
                    return false;
                }
            }
            var afterCharacter = lineText[position.column - 1];
            var closeCharacter = config.autoClosingPairsOpen[character];
            if (afterCharacter !== closeCharacter) {
                return false;
            }
        }
        return true;
    };
    DeleteOperations._runAutoClosingPairDelete = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var position = selections[i].getPosition();
            var deleteSelection = new Range(position.lineNumber, position.column - 1, position.lineNumber, position.column + 1);
            commands[i] = new ReplaceCommand(deleteSelection, '');
        }
        return [true, commands];
    };
    DeleteOperations.deleteLeft = function (prevEditOperationType, config, model, selections) {
        if (this._isAutoClosingPairDelete(config, model, selections)) {
            return this._runAutoClosingPairDelete(config, model, selections);
        }
        var commands = [];
        var shouldPushStackElementBefore = (prevEditOperationType !== 2 /* DeletingLeft */);
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var deleteSelection = selection;
            if (deleteSelection.isEmpty()) {
                var position = selection.getPosition();
                if (config.useTabStops && position.column > 1) {
                    var lineContent = model.getLineContent(position.lineNumber);
                    var firstNonWhitespaceIndex$$1 = firstNonWhitespaceIndex(lineContent);
                    var lastIndentationColumn = (firstNonWhitespaceIndex$$1 === -1
                        ? /* entire string is whitespace */ lineContent.length + 1
                        : firstNonWhitespaceIndex$$1 + 1);
                    if (position.column <= lastIndentationColumn) {
                        var fromVisibleColumn = CursorColumns.visibleColumnFromColumn2(config, model, position);
                        var toVisibleColumn = CursorColumns.prevTabStop(fromVisibleColumn, config.tabSize);
                        var toColumn = CursorColumns.columnFromVisibleColumn2(config, model, position.lineNumber, toVisibleColumn);
                        deleteSelection = new Range(position.lineNumber, toColumn, position.lineNumber, position.column);
                    }
                    else {
                        deleteSelection = new Range(position.lineNumber, position.column - 1, position.lineNumber, position.column);
                    }
                }
                else {
                    var leftOfPosition = MoveOperations.left(config, model, position.lineNumber, position.column);
                    deleteSelection = new Range(leftOfPosition.lineNumber, leftOfPosition.column, position.lineNumber, position.column);
                }
            }
            if (deleteSelection.isEmpty()) {
                // Probably at beginning of file => ignore
                commands[i] = null;
                continue;
            }
            if (deleteSelection.startLineNumber !== deleteSelection.endLineNumber) {
                shouldPushStackElementBefore = true;
            }
            commands[i] = new ReplaceCommand(deleteSelection, '');
        }
        return [shouldPushStackElementBefore, commands];
    };
    DeleteOperations.cut = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (selection.isEmpty()) {
                if (config.emptySelectionClipboard) {
                    // This is a full line cut
                    var position = selection.getPosition();
                    var startLineNumber = void 0, startColumn = void 0, endLineNumber = void 0, endColumn = void 0;
                    if (position.lineNumber < model.getLineCount()) {
                        // Cutting a line in the middle of the model
                        startLineNumber = position.lineNumber;
                        startColumn = 1;
                        endLineNumber = position.lineNumber + 1;
                        endColumn = 1;
                    }
                    else if (position.lineNumber > 1) {
                        // Cutting the last line & there are more than 1 lines in the model
                        startLineNumber = position.lineNumber - 1;
                        startColumn = model.getLineMaxColumn(position.lineNumber - 1);
                        endLineNumber = position.lineNumber;
                        endColumn = model.getLineMaxColumn(position.lineNumber);
                    }
                    else {
                        // Cutting the single line that the model contains
                        startLineNumber = position.lineNumber;
                        startColumn = 1;
                        endLineNumber = position.lineNumber;
                        endColumn = model.getLineMaxColumn(position.lineNumber);
                    }
                    var deleteSelection = new Range(startLineNumber, startColumn, endLineNumber, endColumn);
                    if (!deleteSelection.isEmpty()) {
                        commands[i] = new ReplaceCommand(deleteSelection, '');
                    }
                    else {
                        commands[i] = null;
                    }
                }
                else {
                    // Cannot cut empty selection
                    commands[i] = null;
                }
            }
            else {
                commands[i] = new ReplaceCommand(selection, '');
            }
        }
        return new EditOperationResult(0 /* Other */, commands, {
            shouldPushStackElementBefore: true,
            shouldPushStackElementAfter: true
        });
    };
    return DeleteOperations;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$b = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var WordOperations = /** @class */ (function () {
    function WordOperations() {
    }
    WordOperations._createWord = function (lineContent, wordType, nextCharClass, start, end) {
        // console.log('WORD ==> ' + start + ' => ' + end + ':::: <<<' + lineContent.substring(start, end) + '>>>');
        return { start: start, end: end, wordType: wordType, nextCharClass: nextCharClass };
    };
    WordOperations._findPreviousWordOnLine = function (wordSeparators, model, position) {
        var lineContent = model.getLineContent(position.lineNumber);
        return this._doFindPreviousWordOnLine(lineContent, wordSeparators, position);
    };
    WordOperations._doFindPreviousWordOnLine = function (lineContent, wordSeparators, position) {
        var wordType = 0 /* None */;
        for (var chIndex = position.column - 2; chIndex >= 0; chIndex--) {
            var chCode = lineContent.charCodeAt(chIndex);
            var chClass = wordSeparators.get(chCode);
            if (chClass === 0 /* Regular */) {
                if (wordType === 2 /* Separator */) {
                    return this._createWord(lineContent, wordType, chClass, chIndex + 1, this._findEndOfWord(lineContent, wordSeparators, wordType, chIndex + 1));
                }
                wordType = 1 /* Regular */;
            }
            else if (chClass === 2 /* WordSeparator */) {
                if (wordType === 1 /* Regular */) {
                    return this._createWord(lineContent, wordType, chClass, chIndex + 1, this._findEndOfWord(lineContent, wordSeparators, wordType, chIndex + 1));
                }
                wordType = 2 /* Separator */;
            }
            else if (chClass === 1 /* Whitespace */) {
                if (wordType !== 0 /* None */) {
                    return this._createWord(lineContent, wordType, chClass, chIndex + 1, this._findEndOfWord(lineContent, wordSeparators, wordType, chIndex + 1));
                }
            }
        }
        if (wordType !== 0 /* None */) {
            return this._createWord(lineContent, wordType, 1 /* Whitespace */, 0, this._findEndOfWord(lineContent, wordSeparators, wordType, 0));
        }
        return null;
    };
    WordOperations._findEndOfWord = function (lineContent, wordSeparators, wordType, startIndex) {
        var len = lineContent.length;
        for (var chIndex = startIndex; chIndex < len; chIndex++) {
            var chCode = lineContent.charCodeAt(chIndex);
            var chClass = wordSeparators.get(chCode);
            if (chClass === 1 /* Whitespace */) {
                return chIndex;
            }
            if (wordType === 1 /* Regular */ && chClass === 2 /* WordSeparator */) {
                return chIndex;
            }
            if (wordType === 2 /* Separator */ && chClass === 0 /* Regular */) {
                return chIndex;
            }
        }
        return len;
    };
    WordOperations._findNextWordOnLine = function (wordSeparators, model, position) {
        var lineContent = model.getLineContent(position.lineNumber);
        return this._doFindNextWordOnLine(lineContent, wordSeparators, position);
    };
    WordOperations._doFindNextWordOnLine = function (lineContent, wordSeparators, position) {
        var wordType = 0 /* None */;
        var len = lineContent.length;
        for (var chIndex = position.column - 1; chIndex < len; chIndex++) {
            var chCode = lineContent.charCodeAt(chIndex);
            var chClass = wordSeparators.get(chCode);
            if (chClass === 0 /* Regular */) {
                if (wordType === 2 /* Separator */) {
                    return this._createWord(lineContent, wordType, chClass, this._findStartOfWord(lineContent, wordSeparators, wordType, chIndex - 1), chIndex);
                }
                wordType = 1 /* Regular */;
            }
            else if (chClass === 2 /* WordSeparator */) {
                if (wordType === 1 /* Regular */) {
                    return this._createWord(lineContent, wordType, chClass, this._findStartOfWord(lineContent, wordSeparators, wordType, chIndex - 1), chIndex);
                }
                wordType = 2 /* Separator */;
            }
            else if (chClass === 1 /* Whitespace */) {
                if (wordType !== 0 /* None */) {
                    return this._createWord(lineContent, wordType, chClass, this._findStartOfWord(lineContent, wordSeparators, wordType, chIndex - 1), chIndex);
                }
            }
        }
        if (wordType !== 0 /* None */) {
            return this._createWord(lineContent, wordType, 1 /* Whitespace */, this._findStartOfWord(lineContent, wordSeparators, wordType, len - 1), len);
        }
        return null;
    };
    WordOperations._findStartOfWord = function (lineContent, wordSeparators, wordType, startIndex) {
        for (var chIndex = startIndex; chIndex >= 0; chIndex--) {
            var chCode = lineContent.charCodeAt(chIndex);
            var chClass = wordSeparators.get(chCode);
            if (chClass === 1 /* Whitespace */) {
                return chIndex + 1;
            }
            if (wordType === 1 /* Regular */ && chClass === 2 /* WordSeparator */) {
                return chIndex + 1;
            }
            if (wordType === 2 /* Separator */ && chClass === 0 /* Regular */) {
                return chIndex + 1;
            }
        }
        return 0;
    };
    WordOperations.moveWordLeft = function (wordSeparators, model, position, wordNavigationType) {
        var lineNumber = position.lineNumber;
        var column = position.column;
        var movedToPreviousLine = false;
        if (column === 1) {
            if (lineNumber > 1) {
                movedToPreviousLine = true;
                lineNumber = lineNumber - 1;
                column = model.getLineMaxColumn(lineNumber);
            }
        }
        var prevWordOnLine = WordOperations._findPreviousWordOnLine(wordSeparators, model, new Position(lineNumber, column));
        if (wordNavigationType === 0 /* WordStart */) {
            if (prevWordOnLine && !movedToPreviousLine) {
                // Special case for Visual Studio compatibility:
                // when starting in the trim whitespace at the end of a line,
                // go to the end of the last word
                var lastWhitespaceColumn = model.getLineLastNonWhitespaceColumn(lineNumber);
                if (lastWhitespaceColumn < column) {
                    return new Position(lineNumber, prevWordOnLine.end + 1);
                }
            }
            return new Position(lineNumber, prevWordOnLine ? prevWordOnLine.start + 1 : 1);
        }
        if (wordNavigationType === 1 /* WordStartFast */) {
            if (prevWordOnLine
                && prevWordOnLine.wordType === 2 /* Separator */
                && prevWordOnLine.end - prevWordOnLine.start === 1
                && prevWordOnLine.nextCharClass === 0 /* Regular */) {
                // Skip over a word made up of one single separator and followed by a regular character
                prevWordOnLine = WordOperations._findPreviousWordOnLine(wordSeparators, model, new Position(lineNumber, prevWordOnLine.start + 1));
            }
            return new Position(lineNumber, prevWordOnLine ? prevWordOnLine.start + 1 : 1);
        }
        // We are stopping at the ending of words
        if (prevWordOnLine && column <= prevWordOnLine.end + 1) {
            prevWordOnLine = WordOperations._findPreviousWordOnLine(wordSeparators, model, new Position(lineNumber, prevWordOnLine.start + 1));
        }
        return new Position(lineNumber, prevWordOnLine ? prevWordOnLine.end + 1 : 1);
    };
    WordOperations._moveWordPartLeft = function (model, position) {
        var lineNumber = position.lineNumber;
        var maxColumn = model.getLineMaxColumn(lineNumber);
        if (position.column === 1) {
            return (lineNumber > 1 ? new Position(lineNumber - 1, model.getLineMaxColumn(lineNumber - 1)) : position);
        }
        var lineContent = model.getLineContent(lineNumber);
        for (var column = position.column - 1; column > 1; column--) {
            var left = lineContent.charCodeAt(column - 2);
            var right = lineContent.charCodeAt(column - 1);
            if (left !== 95 /* Underline */ && right === 95 /* Underline */) {
                // snake_case_variables
                return new Position(lineNumber, column);
            }
            if (isLowerAsciiLetter(left) && isUpperAsciiLetter(right)) {
                // camelCaseVariables
                return new Position(lineNumber, column);
            }
            if (isUpperAsciiLetter(left) && isUpperAsciiLetter(right)) {
                // thisIsACamelCaseWithOneLetterWords
                if (column + 1 < maxColumn) {
                    var rightRight = lineContent.charCodeAt(column);
                    if (isLowerAsciiLetter(rightRight)) {
                        return new Position(lineNumber, column);
                    }
                }
            }
        }
        return new Position(lineNumber, 1);
    };
    WordOperations.moveWordRight = function (wordSeparators, model, position, wordNavigationType) {
        var lineNumber = position.lineNumber;
        var column = position.column;
        if (column === model.getLineMaxColumn(lineNumber)) {
            if (lineNumber < model.getLineCount()) {
                lineNumber = lineNumber + 1;
                column = 1;
            }
        }
        var nextWordOnLine = WordOperations._findNextWordOnLine(wordSeparators, model, new Position(lineNumber, column));
        if (wordNavigationType === 2 /* WordEnd */) {
            if (nextWordOnLine && nextWordOnLine.wordType === 2 /* Separator */) {
                if (nextWordOnLine.end - nextWordOnLine.start === 1 && nextWordOnLine.nextCharClass === 0 /* Regular */) {
                    // Skip over a word made up of one single separator and followed by a regular character
                    nextWordOnLine = WordOperations._findNextWordOnLine(wordSeparators, model, new Position(lineNumber, nextWordOnLine.end + 1));
                }
            }
            if (nextWordOnLine) {
                column = nextWordOnLine.end + 1;
            }
            else {
                column = model.getLineMaxColumn(lineNumber);
            }
        }
        else {
            if (nextWordOnLine && column >= nextWordOnLine.start + 1) {
                nextWordOnLine = WordOperations._findNextWordOnLine(wordSeparators, model, new Position(lineNumber, nextWordOnLine.end + 1));
            }
            if (nextWordOnLine) {
                column = nextWordOnLine.start + 1;
            }
            else {
                column = model.getLineMaxColumn(lineNumber);
            }
        }
        return new Position(lineNumber, column);
    };
    WordOperations._moveWordPartRight = function (model, position) {
        var lineNumber = position.lineNumber;
        var maxColumn = model.getLineMaxColumn(lineNumber);
        if (position.column === maxColumn) {
            return (lineNumber < model.getLineCount() ? new Position(lineNumber + 1, 1) : position);
        }
        var lineContent = model.getLineContent(lineNumber);
        for (var column = position.column + 1; column < maxColumn; column++) {
            var left = lineContent.charCodeAt(column - 2);
            var right = lineContent.charCodeAt(column - 1);
            if (left === 95 /* Underline */ && right !== 95 /* Underline */) {
                // snake_case_variables
                return new Position(lineNumber, column);
            }
            if (isLowerAsciiLetter(left) && isUpperAsciiLetter(right)) {
                // camelCaseVariables
                return new Position(lineNumber, column);
            }
            if (isUpperAsciiLetter(left) && isUpperAsciiLetter(right)) {
                // thisIsACamelCaseWithOneLetterWords
                if (column + 1 < maxColumn) {
                    var rightRight = lineContent.charCodeAt(column);
                    if (isLowerAsciiLetter(rightRight)) {
                        return new Position(lineNumber, column);
                    }
                }
            }
        }
        return new Position(lineNumber, maxColumn);
    };
    WordOperations._deleteWordLeftWhitespace = function (model, position) {
        var lineContent = model.getLineContent(position.lineNumber);
        var startIndex = position.column - 2;
        var lastNonWhitespace = lastNonWhitespaceIndex(lineContent, startIndex);
        if (lastNonWhitespace + 1 < startIndex) {
            return new Range(position.lineNumber, lastNonWhitespace + 2, position.lineNumber, position.column);
        }
        return null;
    };
    WordOperations.deleteWordLeft = function (wordSeparators, model, selection, whitespaceHeuristics, wordNavigationType) {
        if (!selection.isEmpty()) {
            return selection;
        }
        var position = new Position(selection.positionLineNumber, selection.positionColumn);
        var lineNumber = position.lineNumber;
        var column = position.column;
        if (lineNumber === 1 && column === 1) {
            // Ignore deleting at beginning of file
            return null;
        }
        if (whitespaceHeuristics) {
            var r = this._deleteWordLeftWhitespace(model, position);
            if (r) {
                return r;
            }
        }
        var prevWordOnLine = WordOperations._findPreviousWordOnLine(wordSeparators, model, position);
        if (wordNavigationType === 0 /* WordStart */) {
            if (prevWordOnLine) {
                column = prevWordOnLine.start + 1;
            }
            else {
                if (column > 1) {
                    column = 1;
                }
                else {
                    lineNumber--;
                    column = model.getLineMaxColumn(lineNumber);
                }
            }
        }
        else {
            if (prevWordOnLine && column <= prevWordOnLine.end + 1) {
                prevWordOnLine = WordOperations._findPreviousWordOnLine(wordSeparators, model, new Position(lineNumber, prevWordOnLine.start + 1));
            }
            if (prevWordOnLine) {
                column = prevWordOnLine.end + 1;
            }
            else {
                if (column > 1) {
                    column = 1;
                }
                else {
                    lineNumber--;
                    column = model.getLineMaxColumn(lineNumber);
                }
            }
        }
        return new Range(lineNumber, column, position.lineNumber, position.column);
    };
    WordOperations._deleteWordPartLeft = function (model, selection) {
        if (!selection.isEmpty()) {
            return selection;
        }
        var pos = selection.getPosition();
        var toPosition = WordOperations._moveWordPartLeft(model, pos);
        return new Range(pos.lineNumber, pos.column, toPosition.lineNumber, toPosition.column);
    };
    WordOperations._findFirstNonWhitespaceChar = function (str, startIndex) {
        var len = str.length;
        for (var chIndex = startIndex; chIndex < len; chIndex++) {
            var ch = str.charAt(chIndex);
            if (ch !== ' ' && ch !== '\t') {
                return chIndex;
            }
        }
        return len;
    };
    WordOperations._deleteWordRightWhitespace = function (model, position) {
        var lineContent = model.getLineContent(position.lineNumber);
        var startIndex = position.column - 1;
        var firstNonWhitespace = this._findFirstNonWhitespaceChar(lineContent, startIndex);
        if (startIndex + 1 < firstNonWhitespace) {
            // bingo
            return new Range(position.lineNumber, position.column, position.lineNumber, firstNonWhitespace + 1);
        }
        return null;
    };
    WordOperations.deleteWordRight = function (wordSeparators, model, selection, whitespaceHeuristics, wordNavigationType) {
        if (!selection.isEmpty()) {
            return selection;
        }
        var position = new Position(selection.positionLineNumber, selection.positionColumn);
        var lineNumber = position.lineNumber;
        var column = position.column;
        var lineCount = model.getLineCount();
        var maxColumn = model.getLineMaxColumn(lineNumber);
        if (lineNumber === lineCount && column === maxColumn) {
            // Ignore deleting at end of file
            return null;
        }
        if (whitespaceHeuristics) {
            var r = this._deleteWordRightWhitespace(model, position);
            if (r) {
                return r;
            }
        }
        var nextWordOnLine = WordOperations._findNextWordOnLine(wordSeparators, model, position);
        if (wordNavigationType === 2 /* WordEnd */) {
            if (nextWordOnLine) {
                column = nextWordOnLine.end + 1;
            }
            else {
                if (column < maxColumn || lineNumber === lineCount) {
                    column = maxColumn;
                }
                else {
                    lineNumber++;
                    nextWordOnLine = WordOperations._findNextWordOnLine(wordSeparators, model, new Position(lineNumber, 1));
                    if (nextWordOnLine) {
                        column = nextWordOnLine.start + 1;
                    }
                    else {
                        column = model.getLineMaxColumn(lineNumber);
                    }
                }
            }
        }
        else {
            if (nextWordOnLine && column >= nextWordOnLine.start + 1) {
                nextWordOnLine = WordOperations._findNextWordOnLine(wordSeparators, model, new Position(lineNumber, nextWordOnLine.end + 1));
            }
            if (nextWordOnLine) {
                column = nextWordOnLine.start + 1;
            }
            else {
                if (column < maxColumn || lineNumber === lineCount) {
                    column = maxColumn;
                }
                else {
                    lineNumber++;
                    nextWordOnLine = WordOperations._findNextWordOnLine(wordSeparators, model, new Position(lineNumber, 1));
                    if (nextWordOnLine) {
                        column = nextWordOnLine.start + 1;
                    }
                    else {
                        column = model.getLineMaxColumn(lineNumber);
                    }
                }
            }
        }
        return new Range(lineNumber, column, position.lineNumber, position.column);
    };
    WordOperations._deleteWordPartRight = function (model, selection) {
        if (!selection.isEmpty()) {
            return selection;
        }
        var pos = selection.getPosition();
        var toPosition = WordOperations._moveWordPartRight(model, pos);
        return new Range(pos.lineNumber, pos.column, toPosition.lineNumber, toPosition.column);
    };
    WordOperations.word = function (config, model, cursor, inSelectionMode, position) {
        var wordSeparators = getMapForWordSeparators(config.wordSeparators);
        var prevWord = WordOperations._findPreviousWordOnLine(wordSeparators, model, position);
        var nextWord = WordOperations._findNextWordOnLine(wordSeparators, model, position);
        if (!inSelectionMode) {
            // Entering word selection for the first time
            var startColumn_1;
            var endColumn_1;
            if (prevWord && prevWord.wordType === 1 /* Regular */ && prevWord.start <= position.column - 1 && position.column - 1 <= prevWord.end) {
                // isTouchingPrevWord
                startColumn_1 = prevWord.start + 1;
                endColumn_1 = prevWord.end + 1;
            }
            else if (nextWord && nextWord.wordType === 1 /* Regular */ && nextWord.start <= position.column - 1 && position.column - 1 <= nextWord.end) {
                // isTouchingNextWord
                startColumn_1 = nextWord.start + 1;
                endColumn_1 = nextWord.end + 1;
            }
            else {
                if (prevWord) {
                    startColumn_1 = prevWord.end + 1;
                }
                else {
                    startColumn_1 = 1;
                }
                if (nextWord) {
                    endColumn_1 = nextWord.start + 1;
                }
                else {
                    endColumn_1 = model.getLineMaxColumn(position.lineNumber);
                }
            }
            return new SingleCursorState(new Range(position.lineNumber, startColumn_1, position.lineNumber, endColumn_1), 0, new Position(position.lineNumber, endColumn_1), 0);
        }
        var startColumn;
        var endColumn;
        if (prevWord && prevWord.wordType === 1 /* Regular */ && prevWord.start < position.column - 1 && position.column - 1 < prevWord.end) {
            // isInsidePrevWord
            startColumn = prevWord.start + 1;
            endColumn = prevWord.end + 1;
        }
        else if (nextWord && nextWord.wordType === 1 /* Regular */ && nextWord.start < position.column - 1 && position.column - 1 < nextWord.end) {
            // isInsideNextWord
            startColumn = nextWord.start + 1;
            endColumn = nextWord.end + 1;
        }
        else {
            startColumn = position.column;
            endColumn = position.column;
        }
        var lineNumber = position.lineNumber;
        var column;
        if (cursor.selectionStart.containsPosition(position)) {
            column = cursor.selectionStart.endColumn;
        }
        else if (position.isBeforeOrEqual(cursor.selectionStart.getStartPosition())) {
            column = startColumn;
            var possiblePosition = new Position(lineNumber, column);
            if (cursor.selectionStart.containsPosition(possiblePosition)) {
                column = cursor.selectionStart.endColumn;
            }
        }
        else {
            column = endColumn;
            var possiblePosition = new Position(lineNumber, column);
            if (cursor.selectionStart.containsPosition(possiblePosition)) {
                column = cursor.selectionStart.startColumn;
            }
        }
        return cursor.move(true, lineNumber, column, 0);
    };
    return WordOperations;
}());
var WordPartOperations = /** @class */ (function (_super) {
    __extends$b(WordPartOperations, _super);
    function WordPartOperations() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WordPartOperations.deleteWordPartLeft = function (wordSeparators, model, selection, whitespaceHeuristics) {
        var candidates = enforceDefined([
            WordOperations.deleteWordLeft(wordSeparators, model, selection, whitespaceHeuristics, 0 /* WordStart */),
            WordOperations.deleteWordLeft(wordSeparators, model, selection, whitespaceHeuristics, 2 /* WordEnd */),
            WordOperations._deleteWordPartLeft(model, selection)
        ]);
        candidates.sort(Range.compareRangesUsingEnds);
        return candidates[2];
    };
    WordPartOperations.deleteWordPartRight = function (wordSeparators, model, selection, whitespaceHeuristics) {
        var candidates = enforceDefined([
            WordOperations.deleteWordRight(wordSeparators, model, selection, whitespaceHeuristics, 0 /* WordStart */),
            WordOperations.deleteWordRight(wordSeparators, model, selection, whitespaceHeuristics, 2 /* WordEnd */),
            WordOperations._deleteWordPartRight(model, selection)
        ]);
        candidates.sort(Range.compareRangesUsingStarts);
        return candidates[0];
    };
    WordPartOperations.moveWordPartLeft = function (wordSeparators, model, position) {
        var candidates = enforceDefined([
            WordOperations.moveWordLeft(wordSeparators, model, position, 0 /* WordStart */),
            WordOperations.moveWordLeft(wordSeparators, model, position, 2 /* WordEnd */),
            WordOperations._moveWordPartLeft(model, position)
        ]);
        candidates.sort(Position.compare);
        return candidates[2];
    };
    WordPartOperations.moveWordPartRight = function (wordSeparators, model, position) {
        var candidates = enforceDefined([
            WordOperations.moveWordRight(wordSeparators, model, position, 0 /* WordStart */),
            WordOperations.moveWordRight(wordSeparators, model, position, 2 /* WordEnd */),
            WordOperations._moveWordPartRight(model, position)
        ]);
        candidates.sort(Position.compare);
        return candidates[0];
    };
    return WordPartOperations;
}(WordOperations));
function enforceDefined(arr) {
    return arr.filter(function (el) { return Boolean(el); });
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CursorMoveCommands = /** @class */ (function () {
    function CursorMoveCommands() {
    }
    CursorMoveCommands.addCursorDown = function (context, cursors, useLogicalLine) {
        var result = [], resultLen = 0;
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[resultLen++] = new CursorState(cursor.modelState, cursor.viewState);
            if (useLogicalLine) {
                result[resultLen++] = CursorState.fromModelState(MoveOperations.translateDown(context.config, context.model, cursor.modelState));
            }
            else {
                result[resultLen++] = CursorState.fromViewState(MoveOperations.translateDown(context.config, context.viewModel, cursor.viewState));
            }
        }
        return result;
    };
    CursorMoveCommands.addCursorUp = function (context, cursors, useLogicalLine) {
        var result = [], resultLen = 0;
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[resultLen++] = new CursorState(cursor.modelState, cursor.viewState);
            if (useLogicalLine) {
                result[resultLen++] = CursorState.fromModelState(MoveOperations.translateUp(context.config, context.model, cursor.modelState));
            }
            else {
                result[resultLen++] = CursorState.fromViewState(MoveOperations.translateUp(context.config, context.viewModel, cursor.viewState));
            }
        }
        return result;
    };
    CursorMoveCommands.moveToBeginningOfLine = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = this._moveToLineStart(context, cursor, inSelectionMode);
        }
        return result;
    };
    CursorMoveCommands._moveToLineStart = function (context, cursor, inSelectionMode) {
        var currentViewStateColumn = cursor.viewState.position.column;
        var currentModelStateColumn = cursor.modelState.position.column;
        var isFirstLineOfWrappedLine = currentViewStateColumn === currentModelStateColumn;
        var currentViewStatelineNumber = cursor.viewState.position.lineNumber;
        var firstNonBlankColumn = context.viewModel.getLineFirstNonWhitespaceColumn(currentViewStatelineNumber);
        var isBeginningOfViewLine = currentViewStateColumn === firstNonBlankColumn;
        if (!isFirstLineOfWrappedLine && !isBeginningOfViewLine) {
            return this._moveToLineStartByView(context, cursor, inSelectionMode);
        }
        else {
            return this._moveToLineStartByModel(context, cursor, inSelectionMode);
        }
    };
    CursorMoveCommands._moveToLineStartByView = function (context, cursor, inSelectionMode) {
        return CursorState.fromViewState(MoveOperations.moveToBeginningOfLine(context.config, context.viewModel, cursor.viewState, inSelectionMode));
    };
    CursorMoveCommands._moveToLineStartByModel = function (context, cursor, inSelectionMode) {
        return CursorState.fromModelState(MoveOperations.moveToBeginningOfLine(context.config, context.model, cursor.modelState, inSelectionMode));
    };
    CursorMoveCommands.moveToEndOfLine = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = this._moveToLineEnd(context, cursor, inSelectionMode);
        }
        return result;
    };
    CursorMoveCommands._moveToLineEnd = function (context, cursor, inSelectionMode) {
        var viewStatePosition = cursor.viewState.position;
        var viewModelMaxColumn = context.viewModel.getLineMaxColumn(viewStatePosition.lineNumber);
        var isEndOfViewLine = viewStatePosition.column === viewModelMaxColumn;
        var modelStatePosition = cursor.modelState.position;
        var modelMaxColumn = context.model.getLineMaxColumn(modelStatePosition.lineNumber);
        var isEndLineOfWrappedLine = viewModelMaxColumn - viewStatePosition.column === modelMaxColumn - modelStatePosition.column;
        if (isEndOfViewLine || isEndLineOfWrappedLine) {
            return this._moveToLineEndByModel(context, cursor, inSelectionMode);
        }
        else {
            return this._moveToLineEndByView(context, cursor, inSelectionMode);
        }
    };
    CursorMoveCommands._moveToLineEndByView = function (context, cursor, inSelectionMode) {
        return CursorState.fromViewState(MoveOperations.moveToEndOfLine(context.config, context.viewModel, cursor.viewState, inSelectionMode));
    };
    CursorMoveCommands._moveToLineEndByModel = function (context, cursor, inSelectionMode) {
        return CursorState.fromModelState(MoveOperations.moveToEndOfLine(context.config, context.model, cursor.modelState, inSelectionMode));
    };
    CursorMoveCommands.expandLineSelection = function (context, cursors) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewSelection = cursor.viewState.selection;
            var startLineNumber = viewSelection.startLineNumber;
            var lineCount = context.viewModel.getLineCount();
            var endLineNumber = viewSelection.endLineNumber;
            var endColumn = void 0;
            if (endLineNumber === lineCount) {
                endColumn = context.viewModel.getLineMaxColumn(lineCount);
            }
            else {
                endLineNumber++;
                endColumn = 1;
            }
            result[i] = CursorState.fromViewState(new SingleCursorState(new Range(startLineNumber, 1, startLineNumber, 1), 0, new Position(endLineNumber, endColumn), 0));
        }
        return result;
    };
    CursorMoveCommands.moveToBeginningOfBuffer = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = CursorState.fromModelState(MoveOperations.moveToBeginningOfBuffer(context.config, context.model, cursor.modelState, inSelectionMode));
        }
        return result;
    };
    CursorMoveCommands.moveToEndOfBuffer = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = CursorState.fromModelState(MoveOperations.moveToEndOfBuffer(context.config, context.model, cursor.modelState, inSelectionMode));
        }
        return result;
    };
    CursorMoveCommands.selectAll = function (context, cursor) {
        var lineCount = context.model.getLineCount();
        var maxColumn = context.model.getLineMaxColumn(lineCount);
        return CursorState.fromModelState(new SingleCursorState(new Range(1, 1, 1, 1), 0, new Position(lineCount, maxColumn), 0));
    };
    CursorMoveCommands.line = function (context, cursor, inSelectionMode, _position, _viewPosition) {
        var position = context.model.validatePosition(_position);
        var viewPosition = (_viewPosition
            ? context.validateViewPosition(new Position(_viewPosition.lineNumber, _viewPosition.column), position)
            : context.convertModelPositionToViewPosition(position));
        if (!inSelectionMode || !cursor.modelState.hasSelection()) {
            // Entering line selection for the first time
            var lineCount = context.model.getLineCount();
            var selectToLineNumber = position.lineNumber + 1;
            var selectToColumn = 1;
            if (selectToLineNumber > lineCount) {
                selectToLineNumber = lineCount;
                selectToColumn = context.model.getLineMaxColumn(selectToLineNumber);
            }
            return CursorState.fromModelState(new SingleCursorState(new Range(position.lineNumber, 1, selectToLineNumber, selectToColumn), 0, new Position(selectToLineNumber, selectToColumn), 0));
        }
        // Continuing line selection
        var enteringLineNumber = cursor.modelState.selectionStart.getStartPosition().lineNumber;
        if (position.lineNumber < enteringLineNumber) {
            return CursorState.fromViewState(cursor.viewState.move(cursor.modelState.hasSelection(), viewPosition.lineNumber, 1, 0));
        }
        else if (position.lineNumber > enteringLineNumber) {
            var lineCount = context.viewModel.getLineCount();
            var selectToViewLineNumber = viewPosition.lineNumber + 1;
            var selectToViewColumn = 1;
            if (selectToViewLineNumber > lineCount) {
                selectToViewLineNumber = lineCount;
                selectToViewColumn = context.viewModel.getLineMaxColumn(selectToViewLineNumber);
            }
            return CursorState.fromViewState(cursor.viewState.move(cursor.modelState.hasSelection(), selectToViewLineNumber, selectToViewColumn, 0));
        }
        else {
            var endPositionOfSelectionStart = cursor.modelState.selectionStart.getEndPosition();
            return CursorState.fromModelState(cursor.modelState.move(cursor.modelState.hasSelection(), endPositionOfSelectionStart.lineNumber, endPositionOfSelectionStart.column, 0));
        }
    };
    CursorMoveCommands.word = function (context, cursor, inSelectionMode, _position) {
        var position = context.model.validatePosition(_position);
        return CursorState.fromModelState(WordOperations.word(context.config, context.model, cursor.modelState, inSelectionMode, position));
    };
    CursorMoveCommands.cancelSelection = function (context, cursor) {
        if (!cursor.modelState.hasSelection()) {
            return new CursorState(cursor.modelState, cursor.viewState);
        }
        var lineNumber = cursor.viewState.position.lineNumber;
        var column = cursor.viewState.position.column;
        return CursorState.fromViewState(new SingleCursorState(new Range(lineNumber, column, lineNumber, column), 0, new Position(lineNumber, column), 0));
    };
    CursorMoveCommands.moveTo = function (context, cursor, inSelectionMode, _position, _viewPosition) {
        var position = context.model.validatePosition(_position);
        var viewPosition = (_viewPosition
            ? context.validateViewPosition(new Position(_viewPosition.lineNumber, _viewPosition.column), position)
            : context.convertModelPositionToViewPosition(position));
        return CursorState.fromViewState(cursor.viewState.move(inSelectionMode, viewPosition.lineNumber, viewPosition.column, 0));
    };
    CursorMoveCommands.move = function (context, cursors, args) {
        var inSelectionMode = args.select;
        var value = args.value;
        switch (args.direction) {
            case 0 /* Left */: {
                if (args.unit === 4 /* HalfLine */) {
                    // Move left by half the current line length
                    return this._moveHalfLineLeft(context, cursors, inSelectionMode);
                }
                else {
                    // Move left by `moveParams.value` columns
                    return this._moveLeft(context, cursors, inSelectionMode, value);
                }
            }
            case 1 /* Right */: {
                if (args.unit === 4 /* HalfLine */) {
                    // Move right by half the current line length
                    return this._moveHalfLineRight(context, cursors, inSelectionMode);
                }
                else {
                    // Move right by `moveParams.value` columns
                    return this._moveRight(context, cursors, inSelectionMode, value);
                }
            }
            case 2 /* Up */: {
                if (args.unit === 2 /* WrappedLine */) {
                    // Move up by view lines
                    return this._moveUpByViewLines(context, cursors, inSelectionMode, value);
                }
                else {
                    // Move up by model lines
                    return this._moveUpByModelLines(context, cursors, inSelectionMode, value);
                }
            }
            case 3 /* Down */: {
                if (args.unit === 2 /* WrappedLine */) {
                    // Move down by view lines
                    return this._moveDownByViewLines(context, cursors, inSelectionMode, value);
                }
                else {
                    // Move down by model lines
                    return this._moveDownByModelLines(context, cursors, inSelectionMode, value);
                }
            }
            case 4 /* WrappedLineStart */: {
                // Move to the beginning of the current view line
                return this._moveToViewMinColumn(context, cursors, inSelectionMode);
            }
            case 5 /* WrappedLineFirstNonWhitespaceCharacter */: {
                // Move to the first non-whitespace column of the current view line
                return this._moveToViewFirstNonWhitespaceColumn(context, cursors, inSelectionMode);
            }
            case 6 /* WrappedLineColumnCenter */: {
                // Move to the "center" of the current view line
                return this._moveToViewCenterColumn(context, cursors, inSelectionMode);
            }
            case 7 /* WrappedLineEnd */: {
                // Move to the end of the current view line
                return this._moveToViewMaxColumn(context, cursors, inSelectionMode);
            }
            case 8 /* WrappedLineLastNonWhitespaceCharacter */: {
                // Move to the last non-whitespace column of the current view line
                return this._moveToViewLastNonWhitespaceColumn(context, cursors, inSelectionMode);
            }
            case 9 /* ViewPortTop */: {
                // Move to the nth line start in the viewport (from the top)
                var cursor = cursors[0];
                var visibleModelRange = context.getCompletelyVisibleModelRange();
                var modelLineNumber = this._firstLineNumberInRange(context.model, visibleModelRange, value);
                var modelColumn = context.model.getLineFirstNonWhitespaceColumn(modelLineNumber);
                return [this._moveToModelPosition(context, cursor, inSelectionMode, modelLineNumber, modelColumn)];
            }
            case 11 /* ViewPortBottom */: {
                // Move to the nth line start in the viewport (from the bottom)
                var cursor = cursors[0];
                var visibleModelRange = context.getCompletelyVisibleModelRange();
                var modelLineNumber = this._lastLineNumberInRange(context.model, visibleModelRange, value);
                var modelColumn = context.model.getLineFirstNonWhitespaceColumn(modelLineNumber);
                return [this._moveToModelPosition(context, cursor, inSelectionMode, modelLineNumber, modelColumn)];
            }
            case 10 /* ViewPortCenter */: {
                // Move to the line start in the viewport center
                var cursor = cursors[0];
                var visibleModelRange = context.getCompletelyVisibleModelRange();
                var modelLineNumber = Math.round((visibleModelRange.startLineNumber + visibleModelRange.endLineNumber) / 2);
                var modelColumn = context.model.getLineFirstNonWhitespaceColumn(modelLineNumber);
                return [this._moveToModelPosition(context, cursor, inSelectionMode, modelLineNumber, modelColumn)];
            }
            case 12 /* ViewPortIfOutside */: {
                // Move to a position inside the viewport
                var visibleViewRange = context.getCompletelyVisibleViewRange();
                var result = [];
                for (var i = 0, len = cursors.length; i < len; i++) {
                    var cursor = cursors[i];
                    result[i] = this.findPositionInViewportIfOutside(context, cursor, visibleViewRange, inSelectionMode);
                }
                return result;
            }
        }
        return null;
    };
    CursorMoveCommands.findPositionInViewportIfOutside = function (context, cursor, visibleViewRange, inSelectionMode) {
        var viewLineNumber = cursor.viewState.position.lineNumber;
        if (visibleViewRange.startLineNumber <= viewLineNumber && viewLineNumber <= visibleViewRange.endLineNumber - 1) {
            // Nothing to do, cursor is in viewport
            return new CursorState(cursor.modelState, cursor.viewState);
        }
        else {
            if (viewLineNumber > visibleViewRange.endLineNumber - 1) {
                viewLineNumber = visibleViewRange.endLineNumber - 1;
            }
            if (viewLineNumber < visibleViewRange.startLineNumber) {
                viewLineNumber = visibleViewRange.startLineNumber;
            }
            var viewColumn = context.viewModel.getLineFirstNonWhitespaceColumn(viewLineNumber);
            return this._moveToViewPosition(context, cursor, inSelectionMode, viewLineNumber, viewColumn);
        }
    };
    /**
     * Find the nth line start included in the range (from the start).
     */
    CursorMoveCommands._firstLineNumberInRange = function (model, range, count) {
        var startLineNumber = range.startLineNumber;
        if (range.startColumn !== model.getLineMinColumn(startLineNumber)) {
            // Move on to the second line if the first line start is not included in the range
            startLineNumber++;
        }
        return Math.min(range.endLineNumber, startLineNumber + count - 1);
    };
    /**
     * Find the nth line start included in the range (from the end).
     */
    CursorMoveCommands._lastLineNumberInRange = function (model, range, count) {
        var startLineNumber = range.startLineNumber;
        if (range.startColumn !== model.getLineMinColumn(startLineNumber)) {
            // Move on to the second line if the first line start is not included in the range
            startLineNumber++;
        }
        return Math.max(startLineNumber, range.endLineNumber - count + 1);
    };
    CursorMoveCommands._moveLeft = function (context, cursors, inSelectionMode, noOfColumns) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var newViewState = MoveOperations.moveLeft(context.config, context.viewModel, cursor.viewState, inSelectionMode, noOfColumns);
            if (noOfColumns === 1 && newViewState.position.lineNumber !== cursor.viewState.position.lineNumber) {
                // moved over to the previous view line
                var newViewModelPosition = context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(newViewState.position);
                if (newViewModelPosition.lineNumber === cursor.modelState.position.lineNumber) {
                    // stayed on the same model line => pass wrapping point where 2 view positions map to a single model position
                    newViewState = MoveOperations.moveLeft(context.config, context.viewModel, newViewState, inSelectionMode, 1);
                }
            }
            result[i] = CursorState.fromViewState(newViewState);
        }
        return result;
    };
    CursorMoveCommands._moveHalfLineLeft = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewLineNumber = cursor.viewState.position.lineNumber;
            var halfLine = Math.round(context.viewModel.getLineContent(viewLineNumber).length / 2);
            result[i] = CursorState.fromViewState(MoveOperations.moveLeft(context.config, context.viewModel, cursor.viewState, inSelectionMode, halfLine));
        }
        return result;
    };
    CursorMoveCommands._moveRight = function (context, cursors, inSelectionMode, noOfColumns) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var newViewState = MoveOperations.moveRight(context.config, context.viewModel, cursor.viewState, inSelectionMode, noOfColumns);
            if (noOfColumns === 1 && newViewState.position.lineNumber !== cursor.viewState.position.lineNumber) {
                // moved over to the next view line
                var newViewModelPosition = context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(newViewState.position);
                if (newViewModelPosition.lineNumber === cursor.modelState.position.lineNumber) {
                    // stayed on the same model line => pass wrapping point where 2 view positions map to a single model position
                    newViewState = MoveOperations.moveRight(context.config, context.viewModel, newViewState, inSelectionMode, 1);
                }
            }
            result[i] = CursorState.fromViewState(newViewState);
        }
        return result;
    };
    CursorMoveCommands._moveHalfLineRight = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewLineNumber = cursor.viewState.position.lineNumber;
            var halfLine = Math.round(context.viewModel.getLineContent(viewLineNumber).length / 2);
            result[i] = CursorState.fromViewState(MoveOperations.moveRight(context.config, context.viewModel, cursor.viewState, inSelectionMode, halfLine));
        }
        return result;
    };
    CursorMoveCommands._moveDownByViewLines = function (context, cursors, inSelectionMode, linesCount) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = CursorState.fromViewState(MoveOperations.moveDown(context.config, context.viewModel, cursor.viewState, inSelectionMode, linesCount));
        }
        return result;
    };
    CursorMoveCommands._moveDownByModelLines = function (context, cursors, inSelectionMode, linesCount) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = CursorState.fromModelState(MoveOperations.moveDown(context.config, context.model, cursor.modelState, inSelectionMode, linesCount));
        }
        return result;
    };
    CursorMoveCommands._moveUpByViewLines = function (context, cursors, inSelectionMode, linesCount) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = CursorState.fromViewState(MoveOperations.moveUp(context.config, context.viewModel, cursor.viewState, inSelectionMode, linesCount));
        }
        return result;
    };
    CursorMoveCommands._moveUpByModelLines = function (context, cursors, inSelectionMode, linesCount) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            result[i] = CursorState.fromModelState(MoveOperations.moveUp(context.config, context.model, cursor.modelState, inSelectionMode, linesCount));
        }
        return result;
    };
    CursorMoveCommands._moveToViewPosition = function (context, cursor, inSelectionMode, toViewLineNumber, toViewColumn) {
        return CursorState.fromViewState(cursor.viewState.move(inSelectionMode, toViewLineNumber, toViewColumn, 0));
    };
    CursorMoveCommands._moveToModelPosition = function (context, cursor, inSelectionMode, toModelLineNumber, toModelColumn) {
        return CursorState.fromModelState(cursor.modelState.move(inSelectionMode, toModelLineNumber, toModelColumn, 0));
    };
    CursorMoveCommands._moveToViewMinColumn = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewLineNumber = cursor.viewState.position.lineNumber;
            var viewColumn = context.viewModel.getLineMinColumn(viewLineNumber);
            result[i] = this._moveToViewPosition(context, cursor, inSelectionMode, viewLineNumber, viewColumn);
        }
        return result;
    };
    CursorMoveCommands._moveToViewFirstNonWhitespaceColumn = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewLineNumber = cursor.viewState.position.lineNumber;
            var viewColumn = context.viewModel.getLineFirstNonWhitespaceColumn(viewLineNumber);
            result[i] = this._moveToViewPosition(context, cursor, inSelectionMode, viewLineNumber, viewColumn);
        }
        return result;
    };
    CursorMoveCommands._moveToViewCenterColumn = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewLineNumber = cursor.viewState.position.lineNumber;
            var viewColumn = Math.round((context.viewModel.getLineMaxColumn(viewLineNumber) + context.viewModel.getLineMinColumn(viewLineNumber)) / 2);
            result[i] = this._moveToViewPosition(context, cursor, inSelectionMode, viewLineNumber, viewColumn);
        }
        return result;
    };
    CursorMoveCommands._moveToViewMaxColumn = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewLineNumber = cursor.viewState.position.lineNumber;
            var viewColumn = context.viewModel.getLineMaxColumn(viewLineNumber);
            result[i] = this._moveToViewPosition(context, cursor, inSelectionMode, viewLineNumber, viewColumn);
        }
        return result;
    };
    CursorMoveCommands._moveToViewLastNonWhitespaceColumn = function (context, cursors, inSelectionMode) {
        var result = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            var cursor = cursors[i];
            var viewLineNumber = cursor.viewState.position.lineNumber;
            var viewColumn = context.viewModel.getLineLastNonWhitespaceColumn(viewLineNumber);
            result[i] = this._moveToViewPosition(context, cursor, inSelectionMode, viewLineNumber, viewColumn);
        }
        return result;
    };
    return CursorMoveCommands;
}());
var CursorMove;
(function (CursorMove) {
    var isCursorMoveArgs = function (arg) {
        if (!isObject(arg)) {
            return false;
        }
        var cursorMoveArg = arg;
        if (!isString(cursorMoveArg.to)) {
            return false;
        }
        if (!isUndefined(cursorMoveArg.select) && !isBoolean(cursorMoveArg.select)) {
            return false;
        }
        if (!isUndefined(cursorMoveArg.by) && !isString(cursorMoveArg.by)) {
            return false;
        }
        if (!isUndefined(cursorMoveArg.value) && !isNumber(cursorMoveArg.value)) {
            return false;
        }
        return true;
    };
    CursorMove.description = {
        description: 'Move cursor to a logical position in the view',
        args: [
            {
                name: 'Cursor move argument object',
                description: "Property-value pairs that can be passed through this argument:\n\t\t\t\t\t* 'to': A mandatory logical position value providing where to move the cursor.\n\t\t\t\t\t\t```\n\t\t\t\t\t\t'left', 'right', 'up', 'down'\n\t\t\t\t\t\t'wrappedLineStart', 'wrappedLineEnd', 'wrappedLineColumnCenter'\n\t\t\t\t\t\t'wrappedLineFirstNonWhitespaceCharacter', 'wrappedLineLastNonWhitespaceCharacter'\n\t\t\t\t\t\t'viewPortTop', 'viewPortCenter', 'viewPortBottom', 'viewPortIfOutside'\n\t\t\t\t\t\t```\n\t\t\t\t\t* 'by': Unit to move. Default is computed based on 'to' value.\n\t\t\t\t\t\t```\n\t\t\t\t\t\t'line', 'wrappedLine', 'character', 'halfLine'\n\t\t\t\t\t\t```\n\t\t\t\t\t* 'value': Number of units to move. Default is '1'.\n\t\t\t\t\t* 'select': If 'true' makes the selection. Default is 'false'.\n\t\t\t\t",
                constraint: isCursorMoveArgs
            }
        ]
    };
    /**
     * Positions in the view for cursor move command.
     */
    CursorMove.RawDirection = {
        Left: 'left',
        Right: 'right',
        Up: 'up',
        Down: 'down',
        WrappedLineStart: 'wrappedLineStart',
        WrappedLineFirstNonWhitespaceCharacter: 'wrappedLineFirstNonWhitespaceCharacter',
        WrappedLineColumnCenter: 'wrappedLineColumnCenter',
        WrappedLineEnd: 'wrappedLineEnd',
        WrappedLineLastNonWhitespaceCharacter: 'wrappedLineLastNonWhitespaceCharacter',
        ViewPortTop: 'viewPortTop',
        ViewPortCenter: 'viewPortCenter',
        ViewPortBottom: 'viewPortBottom',
        ViewPortIfOutside: 'viewPortIfOutside'
    };
    /**
     * Units for Cursor move 'by' argument
     */
    CursorMove.RawUnit = {
        Line: 'line',
        WrappedLine: 'wrappedLine',
        Character: 'character',
        HalfLine: 'halfLine'
    };
    function parse(args) {
        if (!args.to) {
            // illegal arguments
            return null;
        }
        var direction;
        switch (args.to) {
            case CursorMove.RawDirection.Left:
                direction = 0 /* Left */;
                break;
            case CursorMove.RawDirection.Right:
                direction = 1 /* Right */;
                break;
            case CursorMove.RawDirection.Up:
                direction = 2 /* Up */;
                break;
            case CursorMove.RawDirection.Down:
                direction = 3 /* Down */;
                break;
            case CursorMove.RawDirection.WrappedLineStart:
                direction = 4 /* WrappedLineStart */;
                break;
            case CursorMove.RawDirection.WrappedLineFirstNonWhitespaceCharacter:
                direction = 5 /* WrappedLineFirstNonWhitespaceCharacter */;
                break;
            case CursorMove.RawDirection.WrappedLineColumnCenter:
                direction = 6 /* WrappedLineColumnCenter */;
                break;
            case CursorMove.RawDirection.WrappedLineEnd:
                direction = 7 /* WrappedLineEnd */;
                break;
            case CursorMove.RawDirection.WrappedLineLastNonWhitespaceCharacter:
                direction = 8 /* WrappedLineLastNonWhitespaceCharacter */;
                break;
            case CursorMove.RawDirection.ViewPortTop:
                direction = 9 /* ViewPortTop */;
                break;
            case CursorMove.RawDirection.ViewPortBottom:
                direction = 11 /* ViewPortBottom */;
                break;
            case CursorMove.RawDirection.ViewPortCenter:
                direction = 10 /* ViewPortCenter */;
                break;
            case CursorMove.RawDirection.ViewPortIfOutside:
                direction = 12 /* ViewPortIfOutside */;
                break;
            default:
                // illegal arguments
                return null;
        }
        var unit = 0 /* None */;
        switch (args.by) {
            case CursorMove.RawUnit.Line:
                unit = 1 /* Line */;
                break;
            case CursorMove.RawUnit.WrappedLine:
                unit = 2 /* WrappedLine */;
                break;
            case CursorMove.RawUnit.Character:
                unit = 3 /* Character */;
                break;
            case CursorMove.RawUnit.HalfLine:
                unit = 4 /* HalfLine */;
                break;
        }
        return {
            direction: direction,
            unit: unit,
            select: (!!args.select),
            value: (args.value || 1)
        };
    }
    CursorMove.parse = parse;
})(CursorMove || (CursorMove = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ShiftCommand = /** @class */ (function () {
    function ShiftCommand(range, opts) {
        this._opts = opts;
        this._selection = range;
        this._useLastEditRangeForCursorEndPosition = false;
        this._selectionStartColumnStaysPut = false;
    }
    ShiftCommand.unshiftIndentCount = function (line, column, tabSize) {
        // Determine the visible column where the content starts
        var contentStartVisibleColumn = CursorColumns.visibleColumnFromColumn(line, column, tabSize);
        var desiredTabStop = CursorColumns.prevTabStop(contentStartVisibleColumn, tabSize);
        // The `desiredTabStop` is a multiple of `tabSize` => determine the number of indents
        return desiredTabStop / tabSize;
    };
    ShiftCommand.shiftIndentCount = function (line, column, tabSize) {
        // Determine the visible column where the content starts
        var contentStartVisibleColumn = CursorColumns.visibleColumnFromColumn(line, column, tabSize);
        var desiredTabStop = CursorColumns.nextTabStop(contentStartVisibleColumn, tabSize);
        // The `desiredTabStop` is a multiple of `tabSize` => determine the number of indents
        return desiredTabStop / tabSize;
    };
    ShiftCommand.prototype._addEditOperation = function (builder, range, text) {
        if (this._useLastEditRangeForCursorEndPosition) {
            builder.addTrackedEditOperation(range, text);
        }
        else {
            builder.addEditOperation(range, text);
        }
    };
    ShiftCommand.prototype.getEditOperations = function (model, builder) {
        var startLine = this._selection.startLineNumber;
        var endLine = this._selection.endLineNumber;
        if (this._selection.endColumn === 1 && startLine !== endLine) {
            endLine = endLine - 1;
        }
        var tabSize = this._opts.tabSize;
        var oneIndent = this._opts.oneIndent;
        var shouldIndentEmptyLines = (startLine === endLine);
        // if indenting or outdenting on a whitespace only line
        if (this._selection.isEmpty()) {
            if (/^\s*$/.test(model.getLineContent(startLine))) {
                this._useLastEditRangeForCursorEndPosition = true;
            }
        }
        if (this._opts.useTabStops) {
            // indents[i] represents i * oneIndent
            var indents = ['', oneIndent];
            // keep track of previous line's "miss-alignment"
            var previousLineExtraSpaces = 0, extraSpaces = 0;
            for (var lineNumber = startLine; lineNumber <= endLine; lineNumber++, previousLineExtraSpaces = extraSpaces) {
                extraSpaces = 0;
                var lineText = model.getLineContent(lineNumber);
                var indentationEndIndex = firstNonWhitespaceIndex(lineText);
                if (this._opts.isUnshift && (lineText.length === 0 || indentationEndIndex === 0)) {
                    // empty line or line with no leading whitespace => nothing to do
                    continue;
                }
                if (!shouldIndentEmptyLines && !this._opts.isUnshift && lineText.length === 0) {
                    // do not indent empty lines => nothing to do
                    continue;
                }
                if (indentationEndIndex === -1) {
                    // the entire line is whitespace
                    indentationEndIndex = lineText.length;
                }
                if (lineNumber > 1) {
                    var contentStartVisibleColumn = CursorColumns.visibleColumnFromColumn(lineText, indentationEndIndex + 1, tabSize);
                    if (contentStartVisibleColumn % tabSize !== 0) {
                        // The current line is "miss-aligned", so let's see if this is expected...
                        // This can only happen when it has trailing commas in the indent
                        if (model.isCheapToTokenize(lineNumber - 1)) {
                            var enterAction = LanguageConfigurationRegistry.getRawEnterActionAtPosition(model, lineNumber - 1, model.getLineMaxColumn(lineNumber - 1));
                            if (enterAction) {
                                extraSpaces = previousLineExtraSpaces;
                                if (enterAction.appendText) {
                                    for (var j = 0, lenJ = enterAction.appendText.length; j < lenJ && extraSpaces < tabSize; j++) {
                                        if (enterAction.appendText.charCodeAt(j) === 32 /* Space */) {
                                            extraSpaces++;
                                        }
                                        else {
                                            break;
                                        }
                                    }
                                }
                                if (enterAction.removeText) {
                                    extraSpaces = Math.max(0, extraSpaces - enterAction.removeText);
                                }
                                // Act as if `prefixSpaces` is not part of the indentation
                                for (var j = 0; j < extraSpaces; j++) {
                                    if (indentationEndIndex === 0 || lineText.charCodeAt(indentationEndIndex - 1) !== 32 /* Space */) {
                                        break;
                                    }
                                    indentationEndIndex--;
                                }
                            }
                        }
                    }
                }
                if (this._opts.isUnshift && indentationEndIndex === 0) {
                    // line with no leading whitespace => nothing to do
                    continue;
                }
                var desiredIndentCount = void 0;
                if (this._opts.isUnshift) {
                    desiredIndentCount = ShiftCommand.unshiftIndentCount(lineText, indentationEndIndex + 1, tabSize);
                }
                else {
                    desiredIndentCount = ShiftCommand.shiftIndentCount(lineText, indentationEndIndex + 1, tabSize);
                }
                // Fill `indents`, as needed
                for (var j = indents.length; j <= desiredIndentCount; j++) {
                    indents[j] = indents[j - 1] + oneIndent;
                }
                this._addEditOperation(builder, new Range(lineNumber, 1, lineNumber, indentationEndIndex + 1), indents[desiredIndentCount]);
                if (lineNumber === startLine) {
                    // Force the startColumn to stay put because we're inserting after it
                    this._selectionStartColumnStaysPut = (this._selection.startColumn <= indentationEndIndex + 1);
                }
            }
        }
        else {
            for (var lineNumber = startLine; lineNumber <= endLine; lineNumber++) {
                var lineText = model.getLineContent(lineNumber);
                var indentationEndIndex = firstNonWhitespaceIndex(lineText);
                if (this._opts.isUnshift && (lineText.length === 0 || indentationEndIndex === 0)) {
                    // empty line or line with no leading whitespace => nothing to do
                    continue;
                }
                if (!shouldIndentEmptyLines && !this._opts.isUnshift && lineText.length === 0) {
                    // do not indent empty lines => nothing to do
                    continue;
                }
                if (indentationEndIndex === -1) {
                    // the entire line is whitespace
                    indentationEndIndex = lineText.length;
                }
                if (this._opts.isUnshift && indentationEndIndex === 0) {
                    // line with no leading whitespace => nothing to do
                    continue;
                }
                if (this._opts.isUnshift) {
                    indentationEndIndex = Math.min(indentationEndIndex, tabSize);
                    for (var i = 0; i < indentationEndIndex; i++) {
                        var chr = lineText.charCodeAt(i);
                        if (chr === 9 /* Tab */) {
                            indentationEndIndex = i + 1;
                            break;
                        }
                    }
                    this._addEditOperation(builder, new Range(lineNumber, 1, lineNumber, indentationEndIndex + 1), '');
                }
                else {
                    this._addEditOperation(builder, new Range(lineNumber, 1, lineNumber, 1), oneIndent);
                    if (lineNumber === startLine) {
                        // Force the startColumn to stay put because we're inserting after it
                        this._selectionStartColumnStaysPut = (this._selection.startColumn === 1);
                    }
                }
            }
        }
        this._selectionId = builder.trackSelection(this._selection);
    };
    ShiftCommand.prototype.computeCursorState = function (model, helper) {
        if (this._useLastEditRangeForCursorEndPosition) {
            var lastOp = helper.getInverseEditOperations()[0];
            return new Selection(lastOp.range.endLineNumber, lastOp.range.endColumn, lastOp.range.endLineNumber, lastOp.range.endColumn);
        }
        var result = helper.getTrackedSelection(this._selectionId);
        if (this._selectionStartColumnStaysPut) {
            // The selection start should not move
            var initialStartColumn = this._selection.startColumn;
            var resultStartColumn = result.startColumn;
            if (resultStartColumn <= initialStartColumn) {
                return result;
            }
            if (result.getDirection() === 0 /* LTR */) {
                return new Selection(result.startLineNumber, initialStartColumn, result.endLineNumber, result.endColumn);
            }
            return new Selection(result.endLineNumber, result.endColumn, result.startLineNumber, initialStartColumn);
        }
        return result;
    };
    return ShiftCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var SurroundSelectionCommand = /** @class */ (function () {
    function SurroundSelectionCommand(range, charBeforeSelection, charAfterSelection) {
        this._range = range;
        this._charBeforeSelection = charBeforeSelection;
        this._charAfterSelection = charAfterSelection;
    }
    SurroundSelectionCommand.prototype.getEditOperations = function (model, builder) {
        builder.addTrackedEditOperation(new Range(this._range.startLineNumber, this._range.startColumn, this._range.startLineNumber, this._range.startColumn), this._charBeforeSelection);
        builder.addTrackedEditOperation(new Range(this._range.endLineNumber, this._range.endColumn, this._range.endLineNumber, this._range.endColumn), this._charAfterSelection);
    };
    SurroundSelectionCommand.prototype.computeCursorState = function (model, helper) {
        var inverseEditOperations = helper.getInverseEditOperations();
        var firstOperationRange = inverseEditOperations[0].range;
        var secondOperationRange = inverseEditOperations[1].range;
        return new Selection(firstOperationRange.endLineNumber, firstOperationRange.endColumn, secondOperationRange.endLineNumber, secondOperationRange.endColumn - this._charAfterSelection.length);
    };
    return SurroundSelectionCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var TypeOperations = /** @class */ (function () {
    function TypeOperations() {
    }
    TypeOperations.indent = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            commands[i] = new ShiftCommand(selections[i], {
                isUnshift: false,
                tabSize: config.tabSize,
                oneIndent: config.oneIndent,
                useTabStops: config.useTabStops
            });
        }
        return commands;
    };
    TypeOperations.outdent = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            commands[i] = new ShiftCommand(selections[i], {
                isUnshift: true,
                tabSize: config.tabSize,
                oneIndent: config.oneIndent,
                useTabStops: config.useTabStops
            });
        }
        return commands;
    };
    TypeOperations.shiftIndent = function (config, indentation, count) {
        count = count || 1;
        var desiredIndentCount = ShiftCommand.shiftIndentCount(indentation, indentation.length + count, config.tabSize);
        var newIndentation = '';
        for (var i = 0; i < desiredIndentCount; i++) {
            newIndentation += '\t';
        }
        return newIndentation;
    };
    TypeOperations.unshiftIndent = function (config, indentation, count) {
        count = count || 1;
        var desiredIndentCount = ShiftCommand.unshiftIndentCount(indentation, indentation.length + count, config.tabSize);
        var newIndentation = '';
        for (var i = 0; i < desiredIndentCount; i++) {
            newIndentation += '\t';
        }
        return newIndentation;
    };
    TypeOperations._distributedPaste = function (config, model, selections, text) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            commands[i] = new ReplaceCommand(selections[i], text[i]);
        }
        return new EditOperationResult(0 /* Other */, commands, {
            shouldPushStackElementBefore: true,
            shouldPushStackElementAfter: true
        });
    };
    TypeOperations._simplePaste = function (config, model, selections, text, pasteOnNewLine) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var position = selection.getPosition();
            if (pasteOnNewLine && text.indexOf('\n') !== text.length - 1) {
                pasteOnNewLine = false;
            }
            if (pasteOnNewLine && selection.startLineNumber !== selection.endLineNumber) {
                pasteOnNewLine = false;
            }
            if (pasteOnNewLine && selection.startColumn === model.getLineMinColumn(selection.startLineNumber) && selection.endColumn === model.getLineMaxColumn(selection.startLineNumber)) {
                pasteOnNewLine = false;
            }
            if (pasteOnNewLine) {
                // Paste entire line at the beginning of line
                var typeSelection = new Range(position.lineNumber, 1, position.lineNumber, 1);
                commands[i] = new ReplaceCommand(typeSelection, text);
            }
            else {
                commands[i] = new ReplaceCommand(selection, text);
            }
        }
        return new EditOperationResult(0 /* Other */, commands, {
            shouldPushStackElementBefore: true,
            shouldPushStackElementAfter: true
        });
    };
    TypeOperations._distributePasteToCursors = function (selections, text, pasteOnNewLine, multicursorText) {
        if (pasteOnNewLine) {
            return null;
        }
        if (selections.length === 1) {
            return null;
        }
        if (multicursorText && multicursorText.length === selections.length) {
            return multicursorText;
        }
        // Remove trailing \n if present
        if (text.charCodeAt(text.length - 1) === 10 /* LineFeed */) {
            text = text.substr(0, text.length - 1);
        }
        var lines = text.split(/\r\n|\r|\n/);
        if (lines.length === selections.length) {
            return lines;
        }
        return null;
    };
    TypeOperations.paste = function (config, model, selections, text, pasteOnNewLine, multicursorText) {
        var distributedPaste = this._distributePasteToCursors(selections, text, pasteOnNewLine, multicursorText);
        if (distributedPaste) {
            selections = selections.sort(Range.compareRangesUsingStarts);
            return this._distributedPaste(config, model, selections, distributedPaste);
        }
        else {
            return this._simplePaste(config, model, selections, text, pasteOnNewLine);
        }
    };
    TypeOperations._goodIndentForLine = function (config, model, lineNumber) {
        var action = null;
        var indentation = '';
        var expectedIndentAction = config.autoIndent ? LanguageConfigurationRegistry.getInheritIndentForLine(model, lineNumber, false) : null;
        if (expectedIndentAction) {
            action = expectedIndentAction.action;
            indentation = expectedIndentAction.indentation;
        }
        else if (lineNumber > 1) {
            var lastLineNumber = lineNumber - 1;
            for (lastLineNumber = lineNumber - 1; lastLineNumber >= 1; lastLineNumber--) {
                var lineText = model.getLineContent(lastLineNumber);
                var nonWhitespaceIdx = lastNonWhitespaceIndex(lineText);
                if (nonWhitespaceIdx >= 0) {
                    break;
                }
            }
            if (lastLineNumber < 1) {
                // No previous line with content found
                return null;
            }
            var maxColumn = model.getLineMaxColumn(lastLineNumber);
            var expectedEnterAction = LanguageConfigurationRegistry.getEnterAction(model, new Range(lastLineNumber, maxColumn, lastLineNumber, maxColumn));
            if (expectedEnterAction) {
                indentation = expectedEnterAction.indentation;
                action = expectedEnterAction.enterAction;
                if (action) {
                    indentation += action.appendText;
                }
            }
        }
        if (action) {
            if (action === IndentAction.Indent) {
                indentation = TypeOperations.shiftIndent(config, indentation);
            }
            if (action === IndentAction.Outdent) {
                indentation = TypeOperations.unshiftIndent(config, indentation);
            }
            indentation = config.normalizeIndentation(indentation);
        }
        if (!indentation) {
            return null;
        }
        return indentation;
    };
    TypeOperations._replaceJumpToNextIndent = function (config, model, selection, insertsAutoWhitespace) {
        var typeText = '';
        var position = selection.getStartPosition();
        if (config.insertSpaces) {
            var visibleColumnFromColumn = CursorColumns.visibleColumnFromColumn2(config, model, position);
            var tabSize = config.tabSize;
            var spacesCnt = tabSize - (visibleColumnFromColumn % tabSize);
            for (var i = 0; i < spacesCnt; i++) {
                typeText += ' ';
            }
        }
        else {
            typeText = '\t';
        }
        return new ReplaceCommand(selection, typeText, insertsAutoWhitespace);
    };
    TypeOperations.tab = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (selection.isEmpty()) {
                var lineText = model.getLineContent(selection.startLineNumber);
                if (/^\s*$/.test(lineText) && model.isCheapToTokenize(selection.startLineNumber)) {
                    var goodIndent = this._goodIndentForLine(config, model, selection.startLineNumber);
                    goodIndent = goodIndent || '\t';
                    var possibleTypeText = config.normalizeIndentation(goodIndent);
                    if (!startsWith(lineText, possibleTypeText)) {
                        commands[i] = new ReplaceCommand(new Range(selection.startLineNumber, 1, selection.startLineNumber, lineText.length + 1), possibleTypeText, true);
                        continue;
                    }
                }
                commands[i] = this._replaceJumpToNextIndent(config, model, selection, true);
            }
            else {
                if (selection.startLineNumber === selection.endLineNumber) {
                    var lineMaxColumn = model.getLineMaxColumn(selection.startLineNumber);
                    if (selection.startColumn !== 1 || selection.endColumn !== lineMaxColumn) {
                        // This is a single line selection that is not the entire line
                        commands[i] = this._replaceJumpToNextIndent(config, model, selection, false);
                        continue;
                    }
                }
                commands[i] = new ShiftCommand(selection, {
                    isUnshift: false,
                    tabSize: config.tabSize,
                    oneIndent: config.oneIndent,
                    useTabStops: config.useTabStops
                });
            }
        }
        return commands;
    };
    TypeOperations.replacePreviousChar = function (prevEditOperationType, config, model, selections, txt, replaceCharCnt) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (!selection.isEmpty()) {
                // looks like https://github.com/Microsoft/vscode/issues/2773
                // where a cursor operation occurred before a canceled composition
                // => ignore composition
                commands[i] = null;
                continue;
            }
            var pos = selection.getPosition();
            var startColumn = Math.max(1, pos.column - replaceCharCnt);
            var range = new Range(pos.lineNumber, startColumn, pos.lineNumber, pos.column);
            commands[i] = new ReplaceCommand(range, txt);
        }
        return new EditOperationResult(1 /* Typing */, commands, {
            shouldPushStackElementBefore: (prevEditOperationType !== 1 /* Typing */),
            shouldPushStackElementAfter: false
        });
    };
    TypeOperations._typeCommand = function (range, text, keepPosition) {
        if (keepPosition) {
            return new ReplaceCommandWithoutChangingPosition(range, text, true);
        }
        else {
            return new ReplaceCommand(range, text, true);
        }
    };
    TypeOperations._enter = function (config, model, keepPosition, range) {
        if (!model.isCheapToTokenize(range.getStartPosition().lineNumber)) {
            var lineText_1 = model.getLineContent(range.startLineNumber);
            var indentation_1 = getLeadingWhitespace(lineText_1).substring(0, range.startColumn - 1);
            return TypeOperations._typeCommand(range, '\n' + config.normalizeIndentation(indentation_1), keepPosition);
        }
        var r = LanguageConfigurationRegistry.getEnterAction(model, range);
        if (r) {
            var enterAction = r.enterAction;
            var indentation_2 = r.indentation;
            if (enterAction.indentAction === IndentAction.None) {
                // Nothing special
                return TypeOperations._typeCommand(range, '\n' + config.normalizeIndentation(indentation_2 + enterAction.appendText), keepPosition);
            }
            else if (enterAction.indentAction === IndentAction.Indent) {
                // Indent once
                return TypeOperations._typeCommand(range, '\n' + config.normalizeIndentation(indentation_2 + enterAction.appendText), keepPosition);
            }
            else if (enterAction.indentAction === IndentAction.IndentOutdent) {
                // Ultra special
                var normalIndent = config.normalizeIndentation(indentation_2);
                var increasedIndent = config.normalizeIndentation(indentation_2 + enterAction.appendText);
                var typeText = '\n' + increasedIndent + '\n' + normalIndent;
                if (keepPosition) {
                    return new ReplaceCommandWithoutChangingPosition(range, typeText, true);
                }
                else {
                    return new ReplaceCommandWithOffsetCursorState(range, typeText, -1, increasedIndent.length - normalIndent.length, true);
                }
            }
            else if (enterAction.indentAction === IndentAction.Outdent) {
                var actualIndentation = TypeOperations.unshiftIndent(config, indentation_2);
                return TypeOperations._typeCommand(range, '\n' + config.normalizeIndentation(actualIndentation + enterAction.appendText), keepPosition);
            }
        }
        // no enter rules applied, we should check indentation rules then.
        if (!config.autoIndent) {
            // Nothing special
            var lineText_2 = model.getLineContent(range.startLineNumber);
            var indentation_3 = getLeadingWhitespace(lineText_2).substring(0, range.startColumn - 1);
            return TypeOperations._typeCommand(range, '\n' + config.normalizeIndentation(indentation_3), keepPosition);
        }
        var ir = LanguageConfigurationRegistry.getIndentForEnter(model, range, {
            unshiftIndent: function (indent) {
                return TypeOperations.unshiftIndent(config, indent);
            },
            shiftIndent: function (indent) {
                return TypeOperations.shiftIndent(config, indent);
            },
            normalizeIndentation: function (indent) {
                return config.normalizeIndentation(indent);
            }
        }, config.autoIndent);
        var lineText = model.getLineContent(range.startLineNumber);
        var indentation = getLeadingWhitespace(lineText).substring(0, range.startColumn - 1);
        if (ir) {
            var oldEndViewColumn = CursorColumns.visibleColumnFromColumn2(config, model, range.getEndPosition());
            var oldEndColumn = range.endColumn;
            var beforeText = '\n';
            if (indentation !== config.normalizeIndentation(ir.beforeEnter)) {
                beforeText = config.normalizeIndentation(ir.beforeEnter) + lineText.substring(indentation.length, range.startColumn - 1) + '\n';
                range = new Range(range.startLineNumber, 1, range.endLineNumber, range.endColumn);
            }
            var newLineContent = model.getLineContent(range.endLineNumber);
            var firstNonWhitespace = firstNonWhitespaceIndex(newLineContent);
            if (firstNonWhitespace >= 0) {
                range = range.setEndPosition(range.endLineNumber, Math.max(range.endColumn, firstNonWhitespace + 1));
            }
            else {
                range = range.setEndPosition(range.endLineNumber, model.getLineMaxColumn(range.endLineNumber));
            }
            if (keepPosition) {
                return new ReplaceCommandWithoutChangingPosition(range, beforeText + config.normalizeIndentation(ir.afterEnter), true);
            }
            else {
                var offset = 0;
                if (oldEndColumn <= firstNonWhitespace + 1) {
                    if (!config.insertSpaces) {
                        oldEndViewColumn = Math.ceil(oldEndViewColumn / config.tabSize);
                    }
                    offset = Math.min(oldEndViewColumn + 1 - config.normalizeIndentation(ir.afterEnter).length - 1, 0);
                }
                return new ReplaceCommandWithOffsetCursorState(range, beforeText + config.normalizeIndentation(ir.afterEnter), 0, offset, true);
            }
        }
        else {
            return TypeOperations._typeCommand(range, '\n' + config.normalizeIndentation(indentation), keepPosition);
        }
    };
    TypeOperations._isAutoIndentType = function (config, model, selections) {
        if (!config.autoIndent) {
            return false;
        }
        for (var i = 0, len = selections.length; i < len; i++) {
            if (!model.isCheapToTokenize(selections[i].getEndPosition().lineNumber)) {
                return false;
            }
        }
        return true;
    };
    TypeOperations._runAutoIndentType = function (config, model, range, ch) {
        var currentIndentation = LanguageConfigurationRegistry.getIndentationAtPosition(model, range.startLineNumber, range.startColumn);
        var actualIndentation = LanguageConfigurationRegistry.getIndentActionForType(model, range, ch, {
            shiftIndent: function (indentation) {
                return TypeOperations.shiftIndent(config, indentation);
            },
            unshiftIndent: function (indentation) {
                return TypeOperations.unshiftIndent(config, indentation);
            },
        });
        if (actualIndentation === null) {
            return null;
        }
        if (actualIndentation !== config.normalizeIndentation(currentIndentation)) {
            var firstNonWhitespace = model.getLineFirstNonWhitespaceColumn(range.startLineNumber);
            if (firstNonWhitespace === 0) {
                return TypeOperations._typeCommand(new Range(range.startLineNumber, 0, range.endLineNumber, range.endColumn), config.normalizeIndentation(actualIndentation) + ch, false);
            }
            else {
                return TypeOperations._typeCommand(new Range(range.startLineNumber, 0, range.endLineNumber, range.endColumn), config.normalizeIndentation(actualIndentation) +
                    model.getLineContent(range.startLineNumber).substring(firstNonWhitespace - 1, range.startColumn - 1) + ch, false);
            }
        }
        return null;
    };
    TypeOperations._isAutoClosingCloseCharType = function (config, model, selections, ch) {
        var autoCloseConfig = isQuote(ch) ? config.autoClosingQuotes : config.autoClosingBrackets;
        if (autoCloseConfig === 'never' || !config.autoClosingPairsClose.hasOwnProperty(ch)) {
            return false;
        }
        var isEqualPair = (ch === config.autoClosingPairsClose[ch]);
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (!selection.isEmpty()) {
                return false;
            }
            var position = selection.getPosition();
            var lineText = model.getLineContent(position.lineNumber);
            var afterCharacter = lineText.charAt(position.column - 1);
            if (afterCharacter !== ch) {
                return false;
            }
            if (isEqualPair) {
                var lineTextBeforeCursor = lineText.substr(0, position.column - 1);
                var chCntBefore = this._countNeedlesInHaystack(lineTextBeforeCursor, ch);
                if (chCntBefore % 2 === 0) {
                    return false;
                }
            }
        }
        return true;
    };
    TypeOperations._countNeedlesInHaystack = function (haystack, needle) {
        var cnt = 0;
        var lastIndex = -1;
        while ((lastIndex = haystack.indexOf(needle, lastIndex + 1)) !== -1) {
            cnt++;
        }
        return cnt;
    };
    TypeOperations._runAutoClosingCloseCharType = function (prevEditOperationType, config, model, selections, ch) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var position = selection.getPosition();
            var typeSelection = new Range(position.lineNumber, position.column, position.lineNumber, position.column + 1);
            commands[i] = new ReplaceCommand(typeSelection, ch);
        }
        return new EditOperationResult(1 /* Typing */, commands, {
            shouldPushStackElementBefore: (prevEditOperationType !== 1 /* Typing */),
            shouldPushStackElementAfter: false
        });
    };
    TypeOperations._isBeforeClosingBrace = function (config, ch, characterAfter) {
        var thisBraceIsSymmetric = (config.autoClosingPairsOpen[ch] === ch);
        var isBeforeCloseBrace = false;
        for (var otherCloseBrace in config.autoClosingPairsClose) {
            var otherBraceIsSymmetric = (config.autoClosingPairsOpen[otherCloseBrace] === otherCloseBrace);
            if (!thisBraceIsSymmetric && otherBraceIsSymmetric) {
                continue;
            }
            if (characterAfter === otherCloseBrace) {
                isBeforeCloseBrace = true;
                break;
            }
        }
        return isBeforeCloseBrace;
    };
    TypeOperations._isAutoClosingOpenCharType = function (config, model, selections, ch) {
        var chIsQuote = isQuote(ch);
        var autoCloseConfig = chIsQuote ? config.autoClosingQuotes : config.autoClosingBrackets;
        if (autoCloseConfig === 'never' || !config.autoClosingPairsOpen.hasOwnProperty(ch)) {
            return false;
        }
        var shouldAutoCloseBefore = chIsQuote ? config.shouldAutoCloseBefore.quote : config.shouldAutoCloseBefore.bracket;
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (!selection.isEmpty()) {
                return false;
            }
            var position = selection.getPosition();
            var lineText = model.getLineContent(position.lineNumber);
            // Do not auto-close ' or " after a word character
            if (chIsQuote && position.column > 1) {
                var wordSeparators = getMapForWordSeparators(config.wordSeparators);
                var characterBeforeCode = lineText.charCodeAt(position.column - 2);
                var characterBeforeType = wordSeparators.get(characterBeforeCode);
                if (characterBeforeType === 0 /* Regular */) {
                    return false;
                }
            }
            // Only consider auto closing the pair if a space follows or if another autoclosed pair follows
            var characterAfter = lineText.charAt(position.column - 1);
            if (characterAfter) {
                var isBeforeCloseBrace = TypeOperations._isBeforeClosingBrace(config, ch, characterAfter);
                if (!isBeforeCloseBrace && !shouldAutoCloseBefore(characterAfter)) {
                    return false;
                }
            }
            if (!model.isCheapToTokenize(position.lineNumber)) {
                // Do not force tokenization
                return false;
            }
            model.forceTokenization(position.lineNumber);
            var lineTokens = model.getLineTokens(position.lineNumber);
            var shouldAutoClosePair = false;
            try {
                shouldAutoClosePair = LanguageConfigurationRegistry.shouldAutoClosePair(ch, lineTokens, position.column);
            }
            catch (e) {
                onUnexpectedError(e);
            }
            if (!shouldAutoClosePair) {
                return false;
            }
        }
        return true;
    };
    TypeOperations._runAutoClosingOpenCharType = function (prevEditOperationType, config, model, selections, ch) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var closeCharacter = config.autoClosingPairsOpen[ch];
            commands[i] = new ReplaceCommandWithOffsetCursorState(selection, ch + closeCharacter, 0, -closeCharacter.length);
        }
        return new EditOperationResult(1 /* Typing */, commands, {
            shouldPushStackElementBefore: true,
            shouldPushStackElementAfter: false
        });
    };
    TypeOperations._shouldSurroundChar = function (config, ch) {
        if (isQuote(ch)) {
            return (config.autoSurround === 'quotes' || config.autoSurround === 'languageDefined');
        }
        else {
            // Character is a bracket
            return (config.autoSurround === 'brackets' || config.autoSurround === 'languageDefined');
        }
    };
    TypeOperations._isSurroundSelectionType = function (config, model, selections, ch) {
        if (!TypeOperations._shouldSurroundChar(config, ch) || !config.surroundingPairs.hasOwnProperty(ch)) {
            return false;
        }
        var isTypingAQuoteCharacter = isQuote(ch);
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (selection.isEmpty()) {
                return false;
            }
            var selectionContainsOnlyWhitespace = true;
            for (var lineNumber = selection.startLineNumber; lineNumber <= selection.endLineNumber; lineNumber++) {
                var lineText = model.getLineContent(lineNumber);
                var startIndex = (lineNumber === selection.startLineNumber ? selection.startColumn - 1 : 0);
                var endIndex = (lineNumber === selection.endLineNumber ? selection.endColumn - 1 : lineText.length);
                var selectedText = lineText.substring(startIndex, endIndex);
                if (/[^ \t]/.test(selectedText)) {
                    // this selected text contains something other than whitespace
                    selectionContainsOnlyWhitespace = false;
                    break;
                }
            }
            if (selectionContainsOnlyWhitespace) {
                return false;
            }
            if (isTypingAQuoteCharacter && selection.startLineNumber === selection.endLineNumber && selection.startColumn + 1 === selection.endColumn) {
                var selectionText = model.getValueInRange(selection);
                if (isQuote(selectionText)) {
                    // Typing a quote character on top of another quote character
                    // => disable surround selection type
                    return false;
                }
            }
        }
        return true;
    };
    TypeOperations._runSurroundSelectionType = function (prevEditOperationType, config, model, selections, ch) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var closeCharacter = config.surroundingPairs[ch];
            commands[i] = new SurroundSelectionCommand(selection, ch, closeCharacter);
        }
        return new EditOperationResult(0 /* Other */, commands, {
            shouldPushStackElementBefore: true,
            shouldPushStackElementAfter: true
        });
    };
    TypeOperations._isTypeInterceptorElectricChar = function (config, model, selections) {
        if (selections.length === 1 && model.isCheapToTokenize(selections[0].getEndPosition().lineNumber)) {
            return true;
        }
        return false;
    };
    TypeOperations._typeInterceptorElectricChar = function (prevEditOperationType, config, model, selection, ch) {
        if (!config.electricChars.hasOwnProperty(ch) || !selection.isEmpty()) {
            return null;
        }
        var position = selection.getPosition();
        model.forceTokenization(position.lineNumber);
        var lineTokens = model.getLineTokens(position.lineNumber);
        var electricAction;
        try {
            electricAction = LanguageConfigurationRegistry.onElectricCharacter(ch, lineTokens, position.column);
        }
        catch (e) {
            onUnexpectedError(e);
            return null;
        }
        if (!electricAction) {
            return null;
        }
        if (electricAction.appendText) {
            var command = new ReplaceCommandWithOffsetCursorState(selection, ch + electricAction.appendText, 0, -electricAction.appendText.length);
            return new EditOperationResult(1 /* Typing */, [command], {
                shouldPushStackElementBefore: false,
                shouldPushStackElementAfter: true
            });
        }
        if (electricAction.matchOpenBracket) {
            var endColumn = (lineTokens.getLineContent() + ch).lastIndexOf(electricAction.matchOpenBracket) + 1;
            var match = model.findMatchingBracketUp(electricAction.matchOpenBracket, {
                lineNumber: position.lineNumber,
                column: endColumn
            });
            if (match) {
                if (match.startLineNumber === position.lineNumber) {
                    // matched something on the same line => no change in indentation
                    return null;
                }
                var matchLine = model.getLineContent(match.startLineNumber);
                var matchLineIndentation = getLeadingWhitespace(matchLine);
                var newIndentation = config.normalizeIndentation(matchLineIndentation);
                var lineText = model.getLineContent(position.lineNumber);
                var lineFirstNonBlankColumn = model.getLineFirstNonWhitespaceColumn(position.lineNumber) || position.column;
                var prefix = lineText.substring(lineFirstNonBlankColumn - 1, position.column - 1);
                var typeText = newIndentation + prefix + ch;
                var typeSelection = new Range(position.lineNumber, 1, position.lineNumber, position.column);
                var command = new ReplaceCommand(typeSelection, typeText);
                return new EditOperationResult(1 /* Typing */, [command], {
                    shouldPushStackElementBefore: false,
                    shouldPushStackElementAfter: true
                });
            }
        }
        return null;
    };
    TypeOperations.compositionEndWithInterceptors = function (prevEditOperationType, config, model, selections) {
        if (config.autoClosingQuotes === 'never') {
            return null;
        }
        var commands = [];
        for (var i = 0; i < selections.length; i++) {
            if (!selections[i].isEmpty()) {
                continue;
            }
            var position = selections[i].getPosition();
            var lineText = model.getLineContent(position.lineNumber);
            var ch = lineText.charAt(position.column - 2);
            if (config.autoClosingPairsClose.hasOwnProperty(ch)) { // first of all, it's a closing tag
                if (ch === config.autoClosingPairsClose[ch] /** isEqualPair */) {
                    var lineTextBeforeCursor = lineText.substr(0, position.column - 2);
                    var chCntBefore = this._countNeedlesInHaystack(lineTextBeforeCursor, ch);
                    if (chCntBefore % 2 === 1) {
                        continue; // it pairs with the opening tag.
                    }
                }
            }
            // As we are not typing in a new character, so we don't need to run `_runAutoClosingCloseCharType`
            // Next step, let's try to check if it's an open char.
            if (config.autoClosingPairsOpen.hasOwnProperty(ch)) {
                if (isQuote(ch) && position.column > 2) {
                    var wordSeparators = getMapForWordSeparators(config.wordSeparators);
                    var characterBeforeCode = lineText.charCodeAt(position.column - 3);
                    var characterBeforeType = wordSeparators.get(characterBeforeCode);
                    if (characterBeforeType === 0 /* Regular */) {
                        continue;
                    }
                }
                var characterAfter = lineText.charAt(position.column - 1);
                if (characterAfter) {
                    var isBeforeCloseBrace = TypeOperations._isBeforeClosingBrace(config, ch, characterAfter);
                    var shouldAutoCloseBefore = isQuote(ch) ? config.shouldAutoCloseBefore.quote : config.shouldAutoCloseBefore.bracket;
                    if (isBeforeCloseBrace) {
                        // In normal auto closing logic, we will auto close if the cursor is even before a closing brace intentionally.
                        // However for composition mode, we do nothing here as users might clear all the characters for composition and we don't want to do a unnecessary auto close.
                        // Related: microsoft/vscode#57250.
                        continue;
                    }
                    if (!shouldAutoCloseBefore(characterAfter)) {
                        continue;
                    }
                }
                if (!model.isCheapToTokenize(position.lineNumber)) {
                    // Do not force tokenization
                    continue;
                }
                model.forceTokenization(position.lineNumber);
                var lineTokens = model.getLineTokens(position.lineNumber);
                var shouldAutoClosePair = false;
                try {
                    shouldAutoClosePair = LanguageConfigurationRegistry.shouldAutoClosePair(ch, lineTokens, position.column - 1);
                }
                catch (e) {
                    onUnexpectedError(e);
                }
                if (shouldAutoClosePair) {
                    var closeCharacter = config.autoClosingPairsOpen[ch];
                    commands[i] = new ReplaceCommandWithOffsetCursorState(selections[i], closeCharacter, 0, -closeCharacter.length);
                }
            }
        }
        return new EditOperationResult(1 /* Typing */, commands, {
            shouldPushStackElementBefore: true,
            shouldPushStackElementAfter: false
        });
    };
    TypeOperations.typeWithInterceptors = function (prevEditOperationType, config, model, selections, ch) {
        if (ch === '\n') {
            var commands_1 = [];
            for (var i = 0, len = selections.length; i < len; i++) {
                commands_1[i] = TypeOperations._enter(config, model, false, selections[i]);
            }
            return new EditOperationResult(1 /* Typing */, commands_1, {
                shouldPushStackElementBefore: true,
                shouldPushStackElementAfter: false,
            });
        }
        if (this._isAutoIndentType(config, model, selections)) {
            var commands_2 = [];
            var autoIndentFails = false;
            for (var i = 0, len = selections.length; i < len; i++) {
                commands_2[i] = this._runAutoIndentType(config, model, selections[i], ch);
                if (!commands_2[i]) {
                    autoIndentFails = true;
                    break;
                }
            }
            if (!autoIndentFails) {
                return new EditOperationResult(1 /* Typing */, commands_2, {
                    shouldPushStackElementBefore: true,
                    shouldPushStackElementAfter: false,
                });
            }
        }
        if (this._isAutoClosingCloseCharType(config, model, selections, ch)) {
            return this._runAutoClosingCloseCharType(prevEditOperationType, config, model, selections, ch);
        }
        if (this._isAutoClosingOpenCharType(config, model, selections, ch)) {
            return this._runAutoClosingOpenCharType(prevEditOperationType, config, model, selections, ch);
        }
        if (this._isSurroundSelectionType(config, model, selections, ch)) {
            return this._runSurroundSelectionType(prevEditOperationType, config, model, selections, ch);
        }
        // Electric characters make sense only when dealing with a single cursor,
        // as multiple cursors typing brackets for example would interfer with bracket matching
        if (this._isTypeInterceptorElectricChar(config, model, selections)) {
            var r = this._typeInterceptorElectricChar(prevEditOperationType, config, model, selections[0], ch);
            if (r) {
                return r;
            }
        }
        // A simple character type
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            commands[i] = new ReplaceCommand(selections[i], ch);
        }
        var shouldPushStackElementBefore = (prevEditOperationType !== 1 /* Typing */);
        if (ch === ' ') {
            shouldPushStackElementBefore = true;
        }
        return new EditOperationResult(1 /* Typing */, commands, {
            shouldPushStackElementBefore: shouldPushStackElementBefore,
            shouldPushStackElementAfter: false
        });
    };
    TypeOperations.typeWithoutInterceptors = function (prevEditOperationType, config, model, selections, str) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            commands[i] = new ReplaceCommand(selections[i], str);
        }
        return new EditOperationResult(1 /* Typing */, commands, {
            shouldPushStackElementBefore: (prevEditOperationType !== 1 /* Typing */),
            shouldPushStackElementAfter: false
        });
    };
    TypeOperations.lineInsertBefore = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var lineNumber = selections[i].positionLineNumber;
            if (lineNumber === 1) {
                commands[i] = new ReplaceCommandWithoutChangingPosition(new Range(1, 1, 1, 1), '\n');
            }
            else {
                lineNumber--;
                var column = model.getLineMaxColumn(lineNumber);
                commands[i] = this._enter(config, model, false, new Range(lineNumber, column, lineNumber, column));
            }
        }
        return commands;
    };
    TypeOperations.lineInsertAfter = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var lineNumber = selections[i].positionLineNumber;
            var column = model.getLineMaxColumn(lineNumber);
            commands[i] = this._enter(config, model, false, new Range(lineNumber, column, lineNumber, column));
        }
        return commands;
    };
    TypeOperations.lineBreakInsert = function (config, model, selections) {
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            commands[i] = this._enter(config, model, true, selections[i]);
        }
        return commands;
    };
    return TypeOperations;
}());

/**
 * @internal
 */
function isThemeColor(o) {
    return o && typeof o.id === 'string';
}
/**
 * The type of the `IEditor`.
 */
var EditorType = {
    ICodeEditor: 'vs.editor.ICodeEditor',
    IDiffEditor: 'vs.editor.IDiffEditor'
};
/**
 * Built-in commands.
 * @internal
 */
var Handler = {
    ExecuteCommand: 'executeCommand',
    ExecuteCommands: 'executeCommands',
    Type: 'type',
    ReplacePreviousChar: 'replacePreviousChar',
    CompositionStart: 'compositionStart',
    CompositionEnd: 'compositionEnd',
    Paste: 'paste',
    Cut: 'cut',
    Undo: 'undo',
    Redo: 'redo',
};

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var EditorContextKeys;
(function (EditorContextKeys) {
    /**
     * A context key that is set when the editor's text has focus (cursor is blinking).
     */
    EditorContextKeys.editorTextFocus = new RawContextKey('editorTextFocus', false);
    /**
     * A context key that is set when the editor's text or an editor's widget has focus.
     */
    EditorContextKeys.focus = new RawContextKey('editorFocus', false);
    /**
     * A context key that is set when any editor input has focus (regular editor, repl input...).
     */
    EditorContextKeys.textInputFocus = new RawContextKey('textInputFocus', false);
    EditorContextKeys.readOnly = new RawContextKey('editorReadonly', false);
    EditorContextKeys.writable = EditorContextKeys.readOnly.toNegated();
    EditorContextKeys.hasNonEmptySelection = new RawContextKey('editorHasSelection', false);
    EditorContextKeys.hasOnlyEmptySelection = EditorContextKeys.hasNonEmptySelection.toNegated();
    EditorContextKeys.hasMultipleSelections = new RawContextKey('editorHasMultipleSelections', false);
    EditorContextKeys.hasSingleSelection = EditorContextKeys.hasMultipleSelections.toNegated();
    EditorContextKeys.tabMovesFocus = new RawContextKey('editorTabMovesFocus', false);
    EditorContextKeys.tabDoesNotMoveFocus = EditorContextKeys.tabMovesFocus.toNegated();
    EditorContextKeys.isInEmbeddedEditor = new RawContextKey('isInEmbeddedEditor', false);
    EditorContextKeys.canUndo = new RawContextKey('canUndo', false);
    EditorContextKeys.canRedo = new RawContextKey('canRedo', false);
    // -- mode context keys
    EditorContextKeys.languageId = new RawContextKey('editorLangId', '');
    EditorContextKeys.hasCompletionItemProvider = new RawContextKey('editorHasCompletionItemProvider', false);
    EditorContextKeys.hasCodeActionsProvider = new RawContextKey('editorHasCodeActionsProvider', false);
    EditorContextKeys.hasCodeLensProvider = new RawContextKey('editorHasCodeLensProvider', false);
    EditorContextKeys.hasDefinitionProvider = new RawContextKey('editorHasDefinitionProvider', false);
    EditorContextKeys.hasImplementationProvider = new RawContextKey('editorHasImplementationProvider', false);
    EditorContextKeys.hasTypeDefinitionProvider = new RawContextKey('editorHasTypeDefinitionProvider', false);
    EditorContextKeys.hasHoverProvider = new RawContextKey('editorHasHoverProvider', false);
    EditorContextKeys.hasDocumentHighlightProvider = new RawContextKey('editorHasDocumentHighlightProvider', false);
    EditorContextKeys.hasDocumentSymbolProvider = new RawContextKey('editorHasDocumentSymbolProvider', false);
    EditorContextKeys.hasReferenceProvider = new RawContextKey('editorHasReferenceProvider', false);
    EditorContextKeys.hasRenameProvider = new RawContextKey('editorHasRenameProvider', false);
    EditorContextKeys.hasDocumentFormattingProvider = new RawContextKey('editorHasDocumentFormattingProvider', false);
    EditorContextKeys.hasDocumentSelectionFormattingProvider = new RawContextKey('editorHasDocumentSelectionFormattingProvider', false);
    EditorContextKeys.hasSignatureHelpProvider = new RawContextKey('editorHasSignatureHelpProvider', false);
})(EditorContextKeys || (EditorContextKeys = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$c = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CORE_WEIGHT = 0 /* EditorCore */;
var CoreEditorCommand = /** @class */ (function (_super) {
    __extends$c(CoreEditorCommand, _super);
    function CoreEditorCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CoreEditorCommand.prototype.runEditorCommand = function (accessor, editor, args) {
        var cursors = editor._getCursors();
        if (!cursors) {
            // the editor has no view => has no cursors
            return;
        }
        this.runCoreEditorCommand(cursors, args || {});
    };
    return CoreEditorCommand;
}(EditorCommand));
var EditorScroll_;
(function (EditorScroll_) {
    var isEditorScrollArgs = function (arg) {
        if (!isObject(arg)) {
            return false;
        }
        var scrollArg = arg;
        if (!isString(scrollArg.to)) {
            return false;
        }
        if (!isUndefined(scrollArg.by) && !isString(scrollArg.by)) {
            return false;
        }
        if (!isUndefined(scrollArg.value) && !isNumber(scrollArg.value)) {
            return false;
        }
        if (!isUndefined(scrollArg.revealCursor) && !isBoolean(scrollArg.revealCursor)) {
            return false;
        }
        return true;
    };
    EditorScroll_.description = {
        description: 'Scroll editor in the given direction',
        args: [
            {
                name: 'Editor scroll argument object',
                description: "Property-value pairs that can be passed through this argument:\n\t\t\t\t\t* 'to': A mandatory direction value.\n\t\t\t\t\t\t```\n\t\t\t\t\t\t'up', 'down'\n\t\t\t\t\t\t```\n\t\t\t\t\t* 'by': Unit to move. Default is computed based on 'to' value.\n\t\t\t\t\t\t```\n\t\t\t\t\t\t'line', 'wrappedLine', 'page', 'halfPage'\n\t\t\t\t\t\t```\n\t\t\t\t\t* 'value': Number of units to move. Default is '1'.\n\t\t\t\t\t* 'revealCursor': If 'true' reveals the cursor if it is outside view port.\n\t\t\t\t",
                constraint: isEditorScrollArgs
            }
        ]
    };
    /**
     * Directions in the view for editor scroll command.
     */
    EditorScroll_.RawDirection = {
        Up: 'up',
        Down: 'down',
    };
    /**
     * Units for editor scroll 'by' argument
     */
    EditorScroll_.RawUnit = {
        Line: 'line',
        WrappedLine: 'wrappedLine',
        Page: 'page',
        HalfPage: 'halfPage'
    };
    function parse(args) {
        var direction;
        switch (args.to) {
            case EditorScroll_.RawDirection.Up:
                direction = 1 /* Up */;
                break;
            case EditorScroll_.RawDirection.Down:
                direction = 2 /* Down */;
                break;
            default:
                // Illegal arguments
                return null;
        }
        var unit;
        switch (args.by) {
            case EditorScroll_.RawUnit.Line:
                unit = 1 /* Line */;
                break;
            case EditorScroll_.RawUnit.WrappedLine:
                unit = 2 /* WrappedLine */;
                break;
            case EditorScroll_.RawUnit.Page:
                unit = 3 /* Page */;
                break;
            case EditorScroll_.RawUnit.HalfPage:
                unit = 4 /* HalfPage */;
                break;
            default:
                unit = 2 /* WrappedLine */;
        }
        var value = Math.floor(args.value || 1);
        var revealCursor = !!args.revealCursor;
        return {
            direction: direction,
            unit: unit,
            value: value,
            revealCursor: revealCursor,
            select: (!!args.select)
        };
    }
    EditorScroll_.parse = parse;
})(EditorScroll_ || (EditorScroll_ = {}));
var RevealLine_;
(function (RevealLine_) {
    var isRevealLineArgs = function (arg) {
        if (!isObject(arg)) {
            return false;
        }
        var reveaLineArg = arg;
        if (!isNumber(reveaLineArg.lineNumber)) {
            return false;
        }
        if (!isUndefined(reveaLineArg.at) && !isString(reveaLineArg.at)) {
            return false;
        }
        return true;
    };
    RevealLine_.description = {
        description: 'Reveal the given line at the given logical position',
        args: [
            {
                name: 'Reveal line argument object',
                description: "Property-value pairs that can be passed through this argument:\n\t\t\t\t\t* 'lineNumber': A mandatory line number value.\n\t\t\t\t\t* 'at': Logical position at which line has to be revealed .\n\t\t\t\t\t\t```\n\t\t\t\t\t\t'top', 'center', 'bottom'\n\t\t\t\t\t\t```\n\t\t\t\t",
                constraint: isRevealLineArgs
            }
        ]
    };
    /**
     * Values for reveal line 'at' argument
     */
    RevealLine_.RawAtArgument = {
        Top: 'top',
        Center: 'center',
        Bottom: 'bottom'
    };
})(RevealLine_ || (RevealLine_ = {}));
var CoreNavigationCommands;
(function (CoreNavigationCommands) {
    var BaseMoveToCommand = /** @class */ (function (_super) {
        __extends$c(BaseMoveToCommand, _super);
        function BaseMoveToCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        BaseMoveToCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, [
                CursorMoveCommands.moveTo(cursors.context, cursors.getPrimaryCursor(), this._inSelectionMode, args.position, args.viewPosition)
            ]);
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return BaseMoveToCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.MoveTo = registerEditorCommand(new BaseMoveToCommand({
        id: '_moveTo',
        inSelectionMode: false,
        precondition: null
    }));
    CoreNavigationCommands.MoveToSelect = registerEditorCommand(new BaseMoveToCommand({
        id: '_moveToSelect',
        inSelectionMode: true,
        precondition: null
    }));
    var ColumnSelectCommand = /** @class */ (function (_super) {
        __extends$c(ColumnSelectCommand, _super);
        function ColumnSelectCommand() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ColumnSelectCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            var result = this._getColumnSelectResult(cursors.context, cursors.getPrimaryCursor(), cursors.getColumnSelectData(), args);
            cursors.setStates(args.source, 3 /* Explicit */, result.viewStates.map(function (viewState) { return CursorState.fromViewState(viewState); }));
            cursors.setColumnSelectData({
                toViewLineNumber: result.toLineNumber,
                toViewVisualColumn: result.toVisualColumn
            });
            cursors.reveal(true, (result.reversed ? 1 /* TopMost */ : 2 /* BottomMost */), 0 /* Smooth */);
        };
        return ColumnSelectCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.ColumnSelect = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_1, _super);
        function class_1() {
            return _super.call(this, {
                id: 'columnSelect',
                precondition: null
            }) || this;
        }
        class_1.prototype._getColumnSelectResult = function (context, primary, prevColumnSelectData, args) {
            // validate `args`
            var validatedPosition = context.model.validatePosition(args.position);
            var validatedViewPosition;
            if (args.viewPosition) {
                validatedViewPosition = context.validateViewPosition(new Position(args.viewPosition.lineNumber, args.viewPosition.column), validatedPosition);
            }
            else {
                validatedViewPosition = context.convertModelPositionToViewPosition(validatedPosition);
            }
            return ColumnSelection.columnSelect(context.config, context.viewModel, primary.viewState.selection, validatedViewPosition.lineNumber, args.mouseColumn - 1);
        };
        return class_1;
    }(ColumnSelectCommand)));
    CoreNavigationCommands.CursorColumnSelectLeft = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_2, _super);
        function class_2() {
            return _super.call(this, {
                id: 'cursorColumnSelectLeft',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 512 /* Alt */ | 15 /* LeftArrow */,
                    linux: { primary: 0 }
                }
            }) || this;
        }
        class_2.prototype._getColumnSelectResult = function (context, primary, prevColumnSelectData, args) {
            return ColumnSelection.columnSelectLeft(context.config, context.viewModel, primary.viewState, prevColumnSelectData.toViewLineNumber, prevColumnSelectData.toViewVisualColumn);
        };
        return class_2;
    }(ColumnSelectCommand)));
    CoreNavigationCommands.CursorColumnSelectRight = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_3, _super);
        function class_3() {
            return _super.call(this, {
                id: 'cursorColumnSelectRight',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 512 /* Alt */ | 17 /* RightArrow */,
                    linux: { primary: 0 }
                }
            }) || this;
        }
        class_3.prototype._getColumnSelectResult = function (context, primary, prevColumnSelectData, args) {
            return ColumnSelection.columnSelectRight(context.config, context.viewModel, primary.viewState, prevColumnSelectData.toViewLineNumber, prevColumnSelectData.toViewVisualColumn);
        };
        return class_3;
    }(ColumnSelectCommand)));
    var ColumnSelectUpCommand = /** @class */ (function (_super) {
        __extends$c(ColumnSelectUpCommand, _super);
        function ColumnSelectUpCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._isPaged = opts.isPaged;
            return _this;
        }
        ColumnSelectUpCommand.prototype._getColumnSelectResult = function (context, primary, prevColumnSelectData, args) {
            return ColumnSelection.columnSelectUp(context.config, context.viewModel, primary.viewState, this._isPaged, prevColumnSelectData.toViewLineNumber, prevColumnSelectData.toViewVisualColumn);
        };
        return ColumnSelectUpCommand;
    }(ColumnSelectCommand));
    CoreNavigationCommands.CursorColumnSelectUp = registerEditorCommand(new ColumnSelectUpCommand({
        isPaged: false,
        id: 'cursorColumnSelectUp',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 512 /* Alt */ | 16 /* UpArrow */,
            linux: { primary: 0 }
        }
    }));
    CoreNavigationCommands.CursorColumnSelectPageUp = registerEditorCommand(new ColumnSelectUpCommand({
        isPaged: true,
        id: 'cursorColumnSelectPageUp',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 512 /* Alt */ | 11 /* PageUp */,
            linux: { primary: 0 }
        }
    }));
    var ColumnSelectDownCommand = /** @class */ (function (_super) {
        __extends$c(ColumnSelectDownCommand, _super);
        function ColumnSelectDownCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._isPaged = opts.isPaged;
            return _this;
        }
        ColumnSelectDownCommand.prototype._getColumnSelectResult = function (context, primary, prevColumnSelectData, args) {
            return ColumnSelection.columnSelectDown(context.config, context.viewModel, primary.viewState, this._isPaged, prevColumnSelectData.toViewLineNumber, prevColumnSelectData.toViewVisualColumn);
        };
        return ColumnSelectDownCommand;
    }(ColumnSelectCommand));
    CoreNavigationCommands.CursorColumnSelectDown = registerEditorCommand(new ColumnSelectDownCommand({
        isPaged: false,
        id: 'cursorColumnSelectDown',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 512 /* Alt */ | 18 /* DownArrow */,
            linux: { primary: 0 }
        }
    }));
    CoreNavigationCommands.CursorColumnSelectPageDown = registerEditorCommand(new ColumnSelectDownCommand({
        isPaged: true,
        id: 'cursorColumnSelectPageDown',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 512 /* Alt */ | 12 /* PageDown */,
            linux: { primary: 0 }
        }
    }));
    var CursorMoveImpl = /** @class */ (function (_super) {
        __extends$c(CursorMoveImpl, _super);
        function CursorMoveImpl() {
            return _super.call(this, {
                id: 'cursorMove',
                precondition: null,
                description: CursorMove.description
            }) || this;
        }
        CursorMoveImpl.prototype.runCoreEditorCommand = function (cursors, args) {
            var parsed = CursorMove.parse(args);
            if (!parsed) {
                // illegal arguments
                return;
            }
            this._runCursorMove(cursors, args.source, parsed);
        };
        CursorMoveImpl.prototype._runCursorMove = function (cursors, source, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(source, 3 /* Explicit */, CursorMoveCommands.move(cursors.context, cursors.getAll(), args));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return CursorMoveImpl;
    }(CoreEditorCommand));
    CoreNavigationCommands.CursorMoveImpl = CursorMoveImpl;
    CoreNavigationCommands.CursorMove = registerEditorCommand(new CursorMoveImpl());
    var CursorMoveBasedCommand = /** @class */ (function (_super) {
        __extends$c(CursorMoveBasedCommand, _super);
        function CursorMoveBasedCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._staticArgs = opts.args;
            return _this;
        }
        CursorMoveBasedCommand.prototype.runCoreEditorCommand = function (cursors, dynamicArgs) {
            var args = this._staticArgs;
            if (this._staticArgs.value === -1 /* PAGE_SIZE_MARKER */) {
                // -1 is a marker for page size
                args = {
                    direction: this._staticArgs.direction,
                    unit: this._staticArgs.unit,
                    select: this._staticArgs.select,
                    value: cursors.context.config.pageSize
                };
            }
            CoreNavigationCommands.CursorMove._runCursorMove(cursors, dynamicArgs.source, args);
        };
        return CursorMoveBasedCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.CursorLeft = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 0 /* Left */,
            unit: 0 /* None */,
            select: false,
            value: 1
        },
        id: 'cursorLeft',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 15 /* LeftArrow */,
            mac: { primary: 15 /* LeftArrow */, secondary: [256 /* WinCtrl */ | 32 /* KEY_B */] }
        }
    }));
    CoreNavigationCommands.CursorLeftSelect = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 0 /* Left */,
            unit: 0 /* None */,
            select: true,
            value: 1
        },
        id: 'cursorLeftSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 15 /* LeftArrow */
        }
    }));
    CoreNavigationCommands.CursorRight = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 1 /* Right */,
            unit: 0 /* None */,
            select: false,
            value: 1
        },
        id: 'cursorRight',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 17 /* RightArrow */,
            mac: { primary: 17 /* RightArrow */, secondary: [256 /* WinCtrl */ | 36 /* KEY_F */] }
        }
    }));
    CoreNavigationCommands.CursorRightSelect = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 1 /* Right */,
            unit: 0 /* None */,
            select: true,
            value: 1
        },
        id: 'cursorRightSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 17 /* RightArrow */
        }
    }));
    CoreNavigationCommands.CursorUp = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 2 /* Up */,
            unit: 2 /* WrappedLine */,
            select: false,
            value: 1
        },
        id: 'cursorUp',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 16 /* UpArrow */,
            mac: { primary: 16 /* UpArrow */, secondary: [256 /* WinCtrl */ | 46 /* KEY_P */] }
        }
    }));
    CoreNavigationCommands.CursorUpSelect = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 2 /* Up */,
            unit: 2 /* WrappedLine */,
            select: true,
            value: 1
        },
        id: 'cursorUpSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 16 /* UpArrow */,
            secondary: [2048 /* CtrlCmd */ | 1024 /* Shift */ | 16 /* UpArrow */],
            mac: { primary: 1024 /* Shift */ | 16 /* UpArrow */ },
            linux: { primary: 1024 /* Shift */ | 16 /* UpArrow */ }
        }
    }));
    CoreNavigationCommands.CursorPageUp = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 2 /* Up */,
            unit: 2 /* WrappedLine */,
            select: false,
            value: -1 /* PAGE_SIZE_MARKER */
        },
        id: 'cursorPageUp',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 11 /* PageUp */
        }
    }));
    CoreNavigationCommands.CursorPageUpSelect = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 2 /* Up */,
            unit: 2 /* WrappedLine */,
            select: true,
            value: -1 /* PAGE_SIZE_MARKER */
        },
        id: 'cursorPageUpSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 11 /* PageUp */
        }
    }));
    CoreNavigationCommands.CursorDown = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 3 /* Down */,
            unit: 2 /* WrappedLine */,
            select: false,
            value: 1
        },
        id: 'cursorDown',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 18 /* DownArrow */,
            mac: { primary: 18 /* DownArrow */, secondary: [256 /* WinCtrl */ | 44 /* KEY_N */] }
        }
    }));
    CoreNavigationCommands.CursorDownSelect = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 3 /* Down */,
            unit: 2 /* WrappedLine */,
            select: true,
            value: 1
        },
        id: 'cursorDownSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 18 /* DownArrow */,
            secondary: [2048 /* CtrlCmd */ | 1024 /* Shift */ | 18 /* DownArrow */],
            mac: { primary: 1024 /* Shift */ | 18 /* DownArrow */ },
            linux: { primary: 1024 /* Shift */ | 18 /* DownArrow */ }
        }
    }));
    CoreNavigationCommands.CursorPageDown = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 3 /* Down */,
            unit: 2 /* WrappedLine */,
            select: false,
            value: -1 /* PAGE_SIZE_MARKER */
        },
        id: 'cursorPageDown',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 12 /* PageDown */
        }
    }));
    CoreNavigationCommands.CursorPageDownSelect = registerEditorCommand(new CursorMoveBasedCommand({
        args: {
            direction: 3 /* Down */,
            unit: 2 /* WrappedLine */,
            select: true,
            value: -1 /* PAGE_SIZE_MARKER */
        },
        id: 'cursorPageDownSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 12 /* PageDown */
        }
    }));
    CoreNavigationCommands.CreateCursor = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_4, _super);
        function class_4() {
            return _super.call(this, {
                id: 'createCursor',
                precondition: null
            }) || this;
        }
        class_4.prototype.runCoreEditorCommand = function (cursors, args) {
            var context = cursors.context;
            var newState;
            if (args.wholeLine) {
                newState = CursorMoveCommands.line(context, cursors.getPrimaryCursor(), false, args.position, args.viewPosition);
            }
            else {
                newState = CursorMoveCommands.moveTo(context, cursors.getPrimaryCursor(), false, args.position, args.viewPosition);
            }
            var states = cursors.getAll();
            // Check if we should remove a cursor (sort of like a toggle)
            if (states.length > 1) {
                var newModelPosition = (newState.modelState ? newState.modelState.position : null);
                var newViewPosition = (newState.viewState ? newState.viewState.position : null);
                for (var i = 0, len = states.length; i < len; i++) {
                    var state = states[i];
                    if (newModelPosition && !state.modelState.selection.containsPosition(newModelPosition)) {
                        continue;
                    }
                    if (newViewPosition && !state.viewState.selection.containsPosition(newViewPosition)) {
                        continue;
                    }
                    // => Remove the cursor
                    states.splice(i, 1);
                    cursors.context.model.pushStackElement();
                    cursors.setStates(args.source, 3 /* Explicit */, states);
                    return;
                }
            }
            // => Add the new cursor
            states.push(newState);
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, states);
        };
        return class_4;
    }(CoreEditorCommand)));
    CoreNavigationCommands.LastCursorMoveToSelect = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_5, _super);
        function class_5() {
            return _super.call(this, {
                id: '_lastCursorMoveToSelect',
                precondition: null
            }) || this;
        }
        class_5.prototype.runCoreEditorCommand = function (cursors, args) {
            var context = cursors.context;
            var lastAddedCursorIndex = cursors.getLastAddedCursorIndex();
            var states = cursors.getAll();
            var newStates = states.slice(0);
            newStates[lastAddedCursorIndex] = CursorMoveCommands.moveTo(context, states[lastAddedCursorIndex], true, args.position, args.viewPosition);
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, newStates);
        };
        return class_5;
    }(CoreEditorCommand)));
    var HomeCommand = /** @class */ (function (_super) {
        __extends$c(HomeCommand, _super);
        function HomeCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        HomeCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, CursorMoveCommands.moveToBeginningOfLine(cursors.context, cursors.getAll(), this._inSelectionMode));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return HomeCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.CursorHome = registerEditorCommand(new HomeCommand({
        inSelectionMode: false,
        id: 'cursorHome',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 14 /* Home */,
            mac: { primary: 14 /* Home */, secondary: [2048 /* CtrlCmd */ | 15 /* LeftArrow */] }
        }
    }));
    CoreNavigationCommands.CursorHomeSelect = registerEditorCommand(new HomeCommand({
        inSelectionMode: true,
        id: 'cursorHomeSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 14 /* Home */,
            mac: { primary: 1024 /* Shift */ | 14 /* Home */, secondary: [2048 /* CtrlCmd */ | 1024 /* Shift */ | 15 /* LeftArrow */] }
        }
    }));
    CoreNavigationCommands.CursorLineStart = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_6, _super);
        function class_6() {
            return _super.call(this, {
                id: 'cursorLineStart',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 0,
                    mac: { primary: 256 /* WinCtrl */ | 31 /* KEY_A */ }
                }
            }) || this;
        }
        class_6.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, this._exec(cursors.context, cursors.getAll()));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        class_6.prototype._exec = function (context, cursors) {
            var result = [];
            for (var i = 0, len = cursors.length; i < len; i++) {
                var cursor = cursors[i];
                var lineNumber = cursor.modelState.position.lineNumber;
                result[i] = CursorState.fromModelState(cursor.modelState.move(false, lineNumber, 1, 0));
            }
            return result;
        };
        return class_6;
    }(CoreEditorCommand)));
    var EndCommand = /** @class */ (function (_super) {
        __extends$c(EndCommand, _super);
        function EndCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        EndCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, CursorMoveCommands.moveToEndOfLine(cursors.context, cursors.getAll(), this._inSelectionMode));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return EndCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.CursorEnd = registerEditorCommand(new EndCommand({
        inSelectionMode: false,
        id: 'cursorEnd',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 13 /* End */,
            mac: { primary: 13 /* End */, secondary: [2048 /* CtrlCmd */ | 17 /* RightArrow */] }
        }
    }));
    CoreNavigationCommands.CursorEndSelect = registerEditorCommand(new EndCommand({
        inSelectionMode: true,
        id: 'cursorEndSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 1024 /* Shift */ | 13 /* End */,
            mac: { primary: 1024 /* Shift */ | 13 /* End */, secondary: [2048 /* CtrlCmd */ | 1024 /* Shift */ | 17 /* RightArrow */] }
        }
    }));
    CoreNavigationCommands.CursorLineEnd = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_7, _super);
        function class_7() {
            return _super.call(this, {
                id: 'cursorLineEnd',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 0,
                    mac: { primary: 256 /* WinCtrl */ | 35 /* KEY_E */ }
                }
            }) || this;
        }
        class_7.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, this._exec(cursors.context, cursors.getAll()));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        class_7.prototype._exec = function (context, cursors) {
            var result = [];
            for (var i = 0, len = cursors.length; i < len; i++) {
                var cursor = cursors[i];
                var lineNumber = cursor.modelState.position.lineNumber;
                var maxColumn = context.model.getLineMaxColumn(lineNumber);
                result[i] = CursorState.fromModelState(cursor.modelState.move(false, lineNumber, maxColumn, 0));
            }
            return result;
        };
        return class_7;
    }(CoreEditorCommand)));
    var TopCommand = /** @class */ (function (_super) {
        __extends$c(TopCommand, _super);
        function TopCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        TopCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, CursorMoveCommands.moveToBeginningOfBuffer(cursors.context, cursors.getAll(), this._inSelectionMode));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return TopCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.CursorTop = registerEditorCommand(new TopCommand({
        inSelectionMode: false,
        id: 'cursorTop',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 14 /* Home */,
            mac: { primary: 2048 /* CtrlCmd */ | 16 /* UpArrow */ }
        }
    }));
    CoreNavigationCommands.CursorTopSelect = registerEditorCommand(new TopCommand({
        inSelectionMode: true,
        id: 'cursorTopSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 14 /* Home */,
            mac: { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 16 /* UpArrow */ }
        }
    }));
    var BottomCommand = /** @class */ (function (_super) {
        __extends$c(BottomCommand, _super);
        function BottomCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        BottomCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, CursorMoveCommands.moveToEndOfBuffer(cursors.context, cursors.getAll(), this._inSelectionMode));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return BottomCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.CursorBottom = registerEditorCommand(new BottomCommand({
        inSelectionMode: false,
        id: 'cursorBottom',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 13 /* End */,
            mac: { primary: 2048 /* CtrlCmd */ | 18 /* DownArrow */ }
        }
    }));
    CoreNavigationCommands.CursorBottomSelect = registerEditorCommand(new BottomCommand({
        inSelectionMode: true,
        id: 'cursorBottomSelect',
        precondition: null,
        kbOpts: {
            weight: CORE_WEIGHT,
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 13 /* End */,
            mac: { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 18 /* DownArrow */ }
        }
    }));
    var EditorScrollImpl = /** @class */ (function (_super) {
        __extends$c(EditorScrollImpl, _super);
        function EditorScrollImpl() {
            return _super.call(this, {
                id: 'editorScroll',
                precondition: null,
                description: EditorScroll_.description
            }) || this;
        }
        EditorScrollImpl.prototype.runCoreEditorCommand = function (cursors, args) {
            var parsed = EditorScroll_.parse(args);
            if (!parsed) {
                // illegal arguments
                return;
            }
            this._runEditorScroll(cursors, args.source, parsed);
        };
        EditorScrollImpl.prototype._runEditorScroll = function (cursors, source, args) {
            var desiredScrollTop = this._computeDesiredScrollTop(cursors.context, args);
            if (args.revealCursor) {
                // must ensure cursor is in new visible range
                var desiredVisibleViewRange = cursors.context.getCompletelyVisibleViewRangeAtScrollTop(desiredScrollTop);
                cursors.setStates(source, 3 /* Explicit */, [
                    CursorMoveCommands.findPositionInViewportIfOutside(cursors.context, cursors.getPrimaryCursor(), desiredVisibleViewRange, args.select)
                ]);
            }
            cursors.scrollTo(desiredScrollTop);
        };
        EditorScrollImpl.prototype._computeDesiredScrollTop = function (context, args) {
            if (args.unit === 1 /* Line */) {
                // scrolling by model lines
                var visibleModelRange = context.getCompletelyVisibleModelRange();
                var desiredTopModelLineNumber = void 0;
                if (args.direction === 1 /* Up */) {
                    // must go x model lines up
                    desiredTopModelLineNumber = Math.max(1, visibleModelRange.startLineNumber - args.value);
                }
                else {
                    // must go x model lines down
                    desiredTopModelLineNumber = Math.min(context.model.getLineCount(), visibleModelRange.startLineNumber + args.value);
                }
                var desiredTopViewPosition = context.convertModelPositionToViewPosition(new Position(desiredTopModelLineNumber, 1));
                return context.getVerticalOffsetForViewLine(desiredTopViewPosition.lineNumber);
            }
            var noOfLines;
            if (args.unit === 3 /* Page */) {
                noOfLines = context.config.pageSize * args.value;
            }
            else if (args.unit === 4 /* HalfPage */) {
                noOfLines = Math.round(context.config.pageSize / 2) * args.value;
            }
            else {
                noOfLines = args.value;
            }
            var deltaLines = (args.direction === 1 /* Up */ ? -1 : 1) * noOfLines;
            return context.getCurrentScrollTop() + deltaLines * context.config.lineHeight;
        };
        return EditorScrollImpl;
    }(CoreEditorCommand));
    CoreNavigationCommands.EditorScrollImpl = EditorScrollImpl;
    CoreNavigationCommands.EditorScroll = registerEditorCommand(new EditorScrollImpl());
    CoreNavigationCommands.ScrollLineUp = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_8, _super);
        function class_8() {
            return _super.call(this, {
                id: 'scrollLineUp',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 2048 /* CtrlCmd */ | 16 /* UpArrow */,
                    mac: { primary: 256 /* WinCtrl */ | 11 /* PageUp */ }
                }
            }) || this;
        }
        class_8.prototype.runCoreEditorCommand = function (cursors, args) {
            CoreNavigationCommands.EditorScroll._runEditorScroll(cursors, args.source, {
                direction: 1 /* Up */,
                unit: 2 /* WrappedLine */,
                value: 1,
                revealCursor: false,
                select: false
            });
        };
        return class_8;
    }(CoreEditorCommand)));
    CoreNavigationCommands.ScrollPageUp = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_9, _super);
        function class_9() {
            return _super.call(this, {
                id: 'scrollPageUp',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 2048 /* CtrlCmd */ | 11 /* PageUp */,
                    win: { primary: 512 /* Alt */ | 11 /* PageUp */ },
                    linux: { primary: 512 /* Alt */ | 11 /* PageUp */ }
                }
            }) || this;
        }
        class_9.prototype.runCoreEditorCommand = function (cursors, args) {
            CoreNavigationCommands.EditorScroll._runEditorScroll(cursors, args.source, {
                direction: 1 /* Up */,
                unit: 3 /* Page */,
                value: 1,
                revealCursor: false,
                select: false
            });
        };
        return class_9;
    }(CoreEditorCommand)));
    CoreNavigationCommands.ScrollLineDown = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_10, _super);
        function class_10() {
            return _super.call(this, {
                id: 'scrollLineDown',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 2048 /* CtrlCmd */ | 18 /* DownArrow */,
                    mac: { primary: 256 /* WinCtrl */ | 12 /* PageDown */ }
                }
            }) || this;
        }
        class_10.prototype.runCoreEditorCommand = function (cursors, args) {
            CoreNavigationCommands.EditorScroll._runEditorScroll(cursors, args.source, {
                direction: 2 /* Down */,
                unit: 2 /* WrappedLine */,
                value: 1,
                revealCursor: false,
                select: false
            });
        };
        return class_10;
    }(CoreEditorCommand)));
    CoreNavigationCommands.ScrollPageDown = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_11, _super);
        function class_11() {
            return _super.call(this, {
                id: 'scrollPageDown',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 2048 /* CtrlCmd */ | 12 /* PageDown */,
                    win: { primary: 512 /* Alt */ | 12 /* PageDown */ },
                    linux: { primary: 512 /* Alt */ | 12 /* PageDown */ }
                }
            }) || this;
        }
        class_11.prototype.runCoreEditorCommand = function (cursors, args) {
            CoreNavigationCommands.EditorScroll._runEditorScroll(cursors, args.source, {
                direction: 2 /* Down */,
                unit: 3 /* Page */,
                value: 1,
                revealCursor: false,
                select: false
            });
        };
        return class_11;
    }(CoreEditorCommand)));
    var WordCommand = /** @class */ (function (_super) {
        __extends$c(WordCommand, _super);
        function WordCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        WordCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, [
                CursorMoveCommands.word(cursors.context, cursors.getPrimaryCursor(), this._inSelectionMode, args.position)
            ]);
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return WordCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.WordSelect = registerEditorCommand(new WordCommand({
        inSelectionMode: false,
        id: '_wordSelect',
        precondition: null
    }));
    CoreNavigationCommands.WordSelectDrag = registerEditorCommand(new WordCommand({
        inSelectionMode: true,
        id: '_wordSelectDrag',
        precondition: null
    }));
    CoreNavigationCommands.LastCursorWordSelect = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_12, _super);
        function class_12() {
            return _super.call(this, {
                id: 'lastCursorWordSelect',
                precondition: null
            }) || this;
        }
        class_12.prototype.runCoreEditorCommand = function (cursors, args) {
            var context = cursors.context;
            var lastAddedCursorIndex = cursors.getLastAddedCursorIndex();
            var states = cursors.getAll();
            var newStates = states.slice(0);
            var lastAddedState = states[lastAddedCursorIndex];
            newStates[lastAddedCursorIndex] = CursorMoveCommands.word(context, lastAddedState, lastAddedState.modelState.hasSelection(), args.position);
            context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, newStates);
        };
        return class_12;
    }(CoreEditorCommand)));
    var LineCommand = /** @class */ (function (_super) {
        __extends$c(LineCommand, _super);
        function LineCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        LineCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, [
                CursorMoveCommands.line(cursors.context, cursors.getPrimaryCursor(), this._inSelectionMode, args.position, args.viewPosition)
            ]);
            cursors.reveal(false, 0 /* Primary */, 0 /* Smooth */);
        };
        return LineCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.LineSelect = registerEditorCommand(new LineCommand({
        inSelectionMode: false,
        id: '_lineSelect',
        precondition: null
    }));
    CoreNavigationCommands.LineSelectDrag = registerEditorCommand(new LineCommand({
        inSelectionMode: true,
        id: '_lineSelectDrag',
        precondition: null
    }));
    var LastCursorLineCommand = /** @class */ (function (_super) {
        __extends$c(LastCursorLineCommand, _super);
        function LastCursorLineCommand(opts) {
            var _this = _super.call(this, opts) || this;
            _this._inSelectionMode = opts.inSelectionMode;
            return _this;
        }
        LastCursorLineCommand.prototype.runCoreEditorCommand = function (cursors, args) {
            var lastAddedCursorIndex = cursors.getLastAddedCursorIndex();
            var states = cursors.getAll();
            var newStates = states.slice(0);
            newStates[lastAddedCursorIndex] = CursorMoveCommands.line(cursors.context, states[lastAddedCursorIndex], this._inSelectionMode, args.position, args.viewPosition);
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, newStates);
        };
        return LastCursorLineCommand;
    }(CoreEditorCommand));
    CoreNavigationCommands.LastCursorLineSelect = registerEditorCommand(new LastCursorLineCommand({
        inSelectionMode: false,
        id: 'lastCursorLineSelect',
        precondition: null
    }));
    CoreNavigationCommands.LastCursorLineSelectDrag = registerEditorCommand(new LastCursorLineCommand({
        inSelectionMode: true,
        id: 'lastCursorLineSelectDrag',
        precondition: null
    }));
    CoreNavigationCommands.ExpandLineSelection = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_13, _super);
        function class_13() {
            return _super.call(this, {
                id: 'expandLineSelection',
                precondition: null,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 2048 /* CtrlCmd */ | 39 /* KEY_I */
                }
            }) || this;
        }
        class_13.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, CursorMoveCommands.expandLineSelection(cursors.context, cursors.getAll()));
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return class_13;
    }(CoreEditorCommand)));
    CoreNavigationCommands.CancelSelection = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_14, _super);
        function class_14() {
            return _super.call(this, {
                id: 'cancelSelection',
                precondition: EditorContextKeys.hasNonEmptySelection,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 9 /* Escape */,
                    secondary: [1024 /* Shift */ | 9 /* Escape */]
                }
            }) || this;
        }
        class_14.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, [
                CursorMoveCommands.cancelSelection(cursors.context, cursors.getPrimaryCursor())
            ]);
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return class_14;
    }(CoreEditorCommand)));
    CoreNavigationCommands.RemoveSecondaryCursors = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_15, _super);
        function class_15() {
            return _super.call(this, {
                id: 'removeSecondaryCursors',
                precondition: EditorContextKeys.hasMultipleSelections,
                kbOpts: {
                    weight: CORE_WEIGHT + 1,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 9 /* Escape */,
                    secondary: [1024 /* Shift */ | 9 /* Escape */]
                }
            }) || this;
        }
        class_15.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, [
                cursors.getPrimaryCursor()
            ]);
            cursors.reveal(true, 0 /* Primary */, 0 /* Smooth */);
        };
        return class_15;
    }(CoreEditorCommand)));
    CoreNavigationCommands.RevealLine = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_16, _super);
        function class_16() {
            return _super.call(this, {
                id: 'revealLine',
                precondition: null,
                description: RevealLine_.description
            }) || this;
        }
        class_16.prototype.runCoreEditorCommand = function (cursors, args) {
            var revealLineArg = args;
            var lineNumber = (revealLineArg.lineNumber || 0) + 1;
            if (lineNumber < 1) {
                lineNumber = 1;
            }
            var lineCount = cursors.context.model.getLineCount();
            if (lineNumber > lineCount) {
                lineNumber = lineCount;
            }
            var range = new Range(lineNumber, 1, lineNumber, cursors.context.model.getLineMaxColumn(lineNumber));
            var revealAt = 0 /* Simple */;
            if (revealLineArg.at) {
                switch (revealLineArg.at) {
                    case RevealLine_.RawAtArgument.Top:
                        revealAt = 3 /* Top */;
                        break;
                    case RevealLine_.RawAtArgument.Center:
                        revealAt = 1 /* Center */;
                        break;
                    case RevealLine_.RawAtArgument.Bottom:
                        revealAt = 4 /* Bottom */;
                        break;
                    default:
                        break;
                }
            }
            var viewRange = cursors.context.convertModelRangeToViewRange(range);
            cursors.revealRange(false, viewRange, revealAt, 0 /* Smooth */);
        };
        return class_16;
    }(CoreEditorCommand)));
    CoreNavigationCommands.SelectAll = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_17, _super);
        function class_17() {
            return _super.call(this, {
                id: 'selectAll',
                precondition: null
            }) || this;
        }
        class_17.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, [
                CursorMoveCommands.selectAll(cursors.context, cursors.getPrimaryCursor())
            ]);
        };
        return class_17;
    }(CoreEditorCommand)));
    CoreNavigationCommands.SetSelection = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_18, _super);
        function class_18() {
            return _super.call(this, {
                id: 'setSelection',
                precondition: null
            }) || this;
        }
        class_18.prototype.runCoreEditorCommand = function (cursors, args) {
            cursors.context.model.pushStackElement();
            cursors.setStates(args.source, 3 /* Explicit */, [
                CursorState.fromModelSelection(args.selection)
            ]);
        };
        return class_18;
    }(CoreEditorCommand)));
})(CoreNavigationCommands || (CoreNavigationCommands = {}));
var CoreEditingCommands;
(function (CoreEditingCommands) {
    var CoreEditingCommand = /** @class */ (function (_super) {
        __extends$c(CoreEditingCommand, _super);
        function CoreEditingCommand() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        CoreEditingCommand.prototype.runEditorCommand = function (accessor, editor, args) {
            var cursors = editor._getCursors();
            if (!cursors) {
                // the editor has no view => has no cursors
                return;
            }
            this.runCoreEditingCommand(editor, cursors, args || {});
        };
        return CoreEditingCommand;
    }(EditorCommand));
    CoreEditingCommands.CoreEditingCommand = CoreEditingCommand;
    CoreEditingCommands.LineBreakInsert = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_19, _super);
        function class_19() {
            return _super.call(this, {
                id: 'lineBreakInsert',
                precondition: EditorContextKeys.writable,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 0,
                    mac: { primary: 256 /* WinCtrl */ | 45 /* KEY_O */ }
                }
            }) || this;
        }
        class_19.prototype.runCoreEditingCommand = function (editor, cursors, args) {
            editor.pushUndoStop();
            editor.executeCommands(this.id, TypeOperations.lineBreakInsert(cursors.context.config, cursors.context.model, cursors.getAll().map(function (s) { return s.modelState.selection; })));
        };
        return class_19;
    }(CoreEditingCommand)));
    CoreEditingCommands.Outdent = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_20, _super);
        function class_20() {
            return _super.call(this, {
                id: 'outdent',
                precondition: EditorContextKeys.writable,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: ContextKeyExpr.and(EditorContextKeys.editorTextFocus, EditorContextKeys.tabDoesNotMoveFocus),
                    primary: 1024 /* Shift */ | 2 /* Tab */
                }
            }) || this;
        }
        class_20.prototype.runCoreEditingCommand = function (editor, cursors, args) {
            editor.pushUndoStop();
            editor.executeCommands(this.id, TypeOperations.outdent(cursors.context.config, cursors.context.model, cursors.getAll().map(function (s) { return s.modelState.selection; })));
            editor.pushUndoStop();
        };
        return class_20;
    }(CoreEditingCommand)));
    CoreEditingCommands.Tab = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_21, _super);
        function class_21() {
            return _super.call(this, {
                id: 'tab',
                precondition: EditorContextKeys.writable,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: ContextKeyExpr.and(EditorContextKeys.editorTextFocus, EditorContextKeys.tabDoesNotMoveFocus),
                    primary: 2 /* Tab */
                }
            }) || this;
        }
        class_21.prototype.runCoreEditingCommand = function (editor, cursors, args) {
            editor.pushUndoStop();
            editor.executeCommands(this.id, TypeOperations.tab(cursors.context.config, cursors.context.model, cursors.getAll().map(function (s) { return s.modelState.selection; })));
            editor.pushUndoStop();
        };
        return class_21;
    }(CoreEditingCommand)));
    CoreEditingCommands.DeleteLeft = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_22, _super);
        function class_22() {
            return _super.call(this, {
                id: 'deleteLeft',
                precondition: EditorContextKeys.writable,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 1 /* Backspace */,
                    secondary: [1024 /* Shift */ | 1 /* Backspace */],
                    mac: { primary: 1 /* Backspace */, secondary: [1024 /* Shift */ | 1 /* Backspace */, 256 /* WinCtrl */ | 38 /* KEY_H */, 256 /* WinCtrl */ | 1 /* Backspace */] }
                }
            }) || this;
        }
        class_22.prototype.runCoreEditingCommand = function (editor, cursors, args) {
            var _a = DeleteOperations.deleteLeft(cursors.getPrevEditOperationType(), cursors.context.config, cursors.context.model, cursors.getAll().map(function (s) { return s.modelState.selection; })), shouldPushStackElementBefore = _a[0], commands = _a[1];
            if (shouldPushStackElementBefore) {
                editor.pushUndoStop();
            }
            editor.executeCommands(this.id, commands);
            cursors.setPrevEditOperationType(2 /* DeletingLeft */);
        };
        return class_22;
    }(CoreEditingCommand)));
    CoreEditingCommands.DeleteRight = registerEditorCommand(new /** @class */ (function (_super) {
        __extends$c(class_23, _super);
        function class_23() {
            return _super.call(this, {
                id: 'deleteRight',
                precondition: EditorContextKeys.writable,
                kbOpts: {
                    weight: CORE_WEIGHT,
                    kbExpr: EditorContextKeys.textInputFocus,
                    primary: 20 /* Delete */,
                    mac: { primary: 20 /* Delete */, secondary: [256 /* WinCtrl */ | 34 /* KEY_D */, 256 /* WinCtrl */ | 20 /* Delete */] }
                }
            }) || this;
        }
        class_23.prototype.runCoreEditingCommand = function (editor, cursors, args) {
            var _a = DeleteOperations.deleteRight(cursors.getPrevEditOperationType(), cursors.context.config, cursors.context.model, cursors.getAll().map(function (s) { return s.modelState.selection; })), shouldPushStackElementBefore = _a[0], commands = _a[1];
            if (shouldPushStackElementBefore) {
                editor.pushUndoStop();
            }
            editor.executeCommands(this.id, commands);
            cursors.setPrevEditOperationType(3 /* DeletingRight */);
        };
        return class_23;
    }(CoreEditingCommand)));
})(CoreEditingCommands || (CoreEditingCommands = {}));
function registerCommand(command) {
    command.register();
}
/**
 * A command that will:
 *  1. invoke a command on the focused editor.
 *  2. otherwise, invoke a browser built-in command on the `activeElement`.
 *  3. otherwise, invoke a command on the workbench active editor.
 */
var EditorOrNativeTextInputCommand = /** @class */ (function (_super) {
    __extends$c(EditorOrNativeTextInputCommand, _super);
    function EditorOrNativeTextInputCommand(opts) {
        var _this = _super.call(this, opts) || this;
        _this._editorHandler = opts.editorHandler;
        _this._inputHandler = opts.inputHandler;
        return _this;
    }
    EditorOrNativeTextInputCommand.prototype.runCommand = function (accessor, args) {
        var focusedEditor = accessor.get(ICodeEditorService).getFocusedCodeEditor();
        // Only if editor text focus (i.e. not if editor has widget focus).
        if (focusedEditor && focusedEditor.hasTextFocus()) {
            return this._runEditorHandler(accessor, focusedEditor, args);
        }
        // Ignore this action when user is focused on an element that allows for entering text
        var activeElement = document.activeElement;
        if (activeElement && ['input', 'textarea'].indexOf(activeElement.tagName.toLowerCase()) >= 0) {
            document.execCommand(this._inputHandler);
            return;
        }
        // Redirecting to active editor
        var activeEditor = accessor.get(ICodeEditorService).getActiveCodeEditor();
        if (activeEditor) {
            activeEditor.focus();
            return this._runEditorHandler(accessor, activeEditor, args);
        }
    };
    EditorOrNativeTextInputCommand.prototype._runEditorHandler = function (accessor, editor, args) {
        var HANDLER = this._editorHandler;
        if (typeof HANDLER === 'string') {
            editor.trigger('keyboard', HANDLER, args);
        }
        else {
            args = args || {};
            args.source = 'keyboard';
            HANDLER.runEditorCommand(accessor, editor, args);
        }
    };
    return EditorOrNativeTextInputCommand;
}(Command));
/**
 * A command that will invoke a command on the focused editor.
 */
var EditorHandlerCommand = /** @class */ (function (_super) {
    __extends$c(EditorHandlerCommand, _super);
    function EditorHandlerCommand(id, handlerId) {
        var _this = _super.call(this, {
            id: id,
            precondition: null
        }) || this;
        _this._handlerId = handlerId;
        return _this;
    }
    EditorHandlerCommand.prototype.runCommand = function (accessor, args) {
        var editor = accessor.get(ICodeEditorService).getFocusedCodeEditor();
        if (!editor) {
            return;
        }
        editor.trigger('keyboard', this._handlerId, args);
    };
    return EditorHandlerCommand;
}(Command));
registerCommand(new EditorOrNativeTextInputCommand({
    editorHandler: CoreNavigationCommands.SelectAll,
    inputHandler: 'selectAll',
    id: 'editor.action.selectAll',
    precondition: EditorContextKeys.textInputFocus,
    kbOpts: {
        weight: CORE_WEIGHT,
        kbExpr: null,
        primary: 2048 /* CtrlCmd */ | 31 /* KEY_A */
    },
    menubarOpts: {
        menuId: MenuId.MenubarSelectionMenu,
        group: '1_basic',
        title: localize({ key: 'miSelectAll', comment: ['&& denotes a mnemonic'] }, "&&Select All"),
        order: 1
    }
}));
registerCommand(new EditorOrNativeTextInputCommand({
    editorHandler: Handler.Undo,
    inputHandler: 'undo',
    id: Handler.Undo,
    precondition: EditorContextKeys.writable,
    kbOpts: {
        weight: CORE_WEIGHT,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 2048 /* CtrlCmd */ | 56 /* KEY_Z */
    },
    menubarOpts: {
        menuId: MenuId.MenubarEditMenu,
        group: '1_do',
        title: localize({ key: 'miUndo', comment: ['&& denotes a mnemonic'] }, "&&Undo"),
        order: 1
    }
}));
registerCommand(new EditorHandlerCommand('default:' + Handler.Undo, Handler.Undo));
registerCommand(new EditorOrNativeTextInputCommand({
    editorHandler: Handler.Redo,
    inputHandler: 'redo',
    id: Handler.Redo,
    precondition: EditorContextKeys.writable,
    kbOpts: {
        weight: CORE_WEIGHT,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 2048 /* CtrlCmd */ | 55 /* KEY_Y */,
        secondary: [2048 /* CtrlCmd */ | 1024 /* Shift */ | 56 /* KEY_Z */],
        mac: { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 56 /* KEY_Z */ }
    },
    menubarOpts: {
        menuId: MenuId.MenubarEditMenu,
        group: '1_do',
        title: localize({ key: 'miRedo', comment: ['&& denotes a mnemonic'] }, "&&Redo"),
        order: 2
    }
}));
registerCommand(new EditorHandlerCommand('default:' + Handler.Redo, Handler.Redo));
function registerOverwritableCommand(handlerId) {
    registerCommand(new EditorHandlerCommand('default:' + handlerId, handlerId));
    registerCommand(new EditorHandlerCommand(handlerId, handlerId));
}
registerOverwritableCommand(Handler.Type);
registerOverwritableCommand(Handler.ReplacePreviousChar);
registerOverwritableCommand(Handler.CompositionStart);
registerOverwritableCommand(Handler.CompositionEnd);
registerOverwritableCommand(Handler.Paste);
registerOverwritableCommand(Handler.Cut);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var WindowManager = /** @class */ (function () {
    function WindowManager() {
        // --- Zoom Level
        this._zoomLevel = 0;
        this._lastZoomLevelChangeTime = 0;
        this._onDidChangeZoomLevel = new Emitter();
        this.onDidChangeZoomLevel = this._onDidChangeZoomLevel.event;
        // --- Accessibility
        this._accessibilitySupport = 0 /* Unknown */;
        this._onDidChangeAccessibilitySupport = new Emitter();
        this.onDidChangeAccessibilitySupport = this._onDidChangeAccessibilitySupport.event;
    }
    WindowManager.prototype.getZoomLevel = function () {
        return this._zoomLevel;
    };
    WindowManager.prototype.getTimeSinceLastZoomLevelChanged = function () {
        return Date.now() - this._lastZoomLevelChangeTime;
    };
    // --- Pixel Ratio
    WindowManager.prototype.getPixelRatio = function () {
        var ctx = document.createElement('canvas').getContext('2d');
        var dpr = window.devicePixelRatio || 1;
        var bsr = ctx.webkitBackingStorePixelRatio ||
            ctx.mozBackingStorePixelRatio ||
            ctx.msBackingStorePixelRatio ||
            ctx.oBackingStorePixelRatio ||
            ctx.backingStorePixelRatio || 1;
        return dpr / bsr;
    };
    WindowManager.prototype.getAccessibilitySupport = function () {
        return this._accessibilitySupport;
    };
    WindowManager.INSTANCE = new WindowManager();
    return WindowManager;
}());
function getZoomLevel() {
    return WindowManager.INSTANCE.getZoomLevel();
}
/** Returns the time (in ms) since the zoom level was changed */
function getTimeSinceLastZoomLevelChanged() {
    return WindowManager.INSTANCE.getTimeSinceLastZoomLevelChanged();
}
function onDidChangeZoomLevel(callback) {
    return WindowManager.INSTANCE.onDidChangeZoomLevel(callback);
}
function getPixelRatio() {
    return WindowManager.INSTANCE.getPixelRatio();
}
function getAccessibilitySupport() {
    return WindowManager.INSTANCE.getAccessibilitySupport();
}
function onDidChangeAccessibilitySupport(callback) {
    return WindowManager.INSTANCE.onDidChangeAccessibilitySupport(callback);
}
var userAgent$1 = navigator.userAgent;
var isIE = (userAgent$1.indexOf('Trident') >= 0);
var isEdge = (userAgent$1.indexOf('Edge/') >= 0);
var isEdgeOrIE = isIE || isEdge;
var isFirefox = (userAgent$1.indexOf('Firefox') >= 0);
var isWebKit = (userAgent$1.indexOf('AppleWebKit') >= 0);
var isChrome = (userAgent$1.indexOf('Chrome') >= 0);
var isSafari = (userAgent$1.indexOf('Chrome') === -1) && (userAgent$1.indexOf('Safari') >= 0);
var isIPad = (userAgent$1.indexOf('iPad') >= 0);
var isEdgeWebView = isEdge && (userAgent$1.indexOf('WebView/') >= 0);
function hasClipboardSupport() {
    if (isIE) {
        return false;
    }
    if (isEdge) {
        var index = userAgent$1.indexOf('Edge/');
        var version = parseInt(userAgent$1.substring(index + 5, userAgent$1.indexOf('.', index)), 10);
        if (!version || (version >= 12 && version <= 16)) {
            return false;
        }
    }
    return true;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var domEvent = function (element, type, useCapture) {
    var fn = function (e) { return emitter.fire(e); };
    var emitter = new Emitter({
        onFirstListenerAdd: function () {
            element.addEventListener(type, fn, useCapture);
        },
        onLastListenerRemove: function () {
            element.removeEventListener(type, fn, useCapture);
        }
    });
    return emitter.event;
};

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var KEY_CODE_MAP = new Array(230);
var INVERSE_KEY_CODE_MAP = new Array(112 /* MAX_VALUE */);
(function () {
    for (var i = 0; i < INVERSE_KEY_CODE_MAP.length; i++) {
        INVERSE_KEY_CODE_MAP[i] = -1;
    }
    function define(code, keyCode) {
        KEY_CODE_MAP[code] = keyCode;
        INVERSE_KEY_CODE_MAP[keyCode] = code;
    }
    define(3, 7 /* PauseBreak */); // VK_CANCEL 0x03 Control-break processing
    define(8, 1 /* Backspace */);
    define(9, 2 /* Tab */);
    define(13, 3 /* Enter */);
    define(16, 4 /* Shift */);
    define(17, 5 /* Ctrl */);
    define(18, 6 /* Alt */);
    define(19, 7 /* PauseBreak */);
    define(20, 8 /* CapsLock */);
    define(27, 9 /* Escape */);
    define(32, 10 /* Space */);
    define(33, 11 /* PageUp */);
    define(34, 12 /* PageDown */);
    define(35, 13 /* End */);
    define(36, 14 /* Home */);
    define(37, 15 /* LeftArrow */);
    define(38, 16 /* UpArrow */);
    define(39, 17 /* RightArrow */);
    define(40, 18 /* DownArrow */);
    define(45, 19 /* Insert */);
    define(46, 20 /* Delete */);
    define(48, 21 /* KEY_0 */);
    define(49, 22 /* KEY_1 */);
    define(50, 23 /* KEY_2 */);
    define(51, 24 /* KEY_3 */);
    define(52, 25 /* KEY_4 */);
    define(53, 26 /* KEY_5 */);
    define(54, 27 /* KEY_6 */);
    define(55, 28 /* KEY_7 */);
    define(56, 29 /* KEY_8 */);
    define(57, 30 /* KEY_9 */);
    define(65, 31 /* KEY_A */);
    define(66, 32 /* KEY_B */);
    define(67, 33 /* KEY_C */);
    define(68, 34 /* KEY_D */);
    define(69, 35 /* KEY_E */);
    define(70, 36 /* KEY_F */);
    define(71, 37 /* KEY_G */);
    define(72, 38 /* KEY_H */);
    define(73, 39 /* KEY_I */);
    define(74, 40 /* KEY_J */);
    define(75, 41 /* KEY_K */);
    define(76, 42 /* KEY_L */);
    define(77, 43 /* KEY_M */);
    define(78, 44 /* KEY_N */);
    define(79, 45 /* KEY_O */);
    define(80, 46 /* KEY_P */);
    define(81, 47 /* KEY_Q */);
    define(82, 48 /* KEY_R */);
    define(83, 49 /* KEY_S */);
    define(84, 50 /* KEY_T */);
    define(85, 51 /* KEY_U */);
    define(86, 52 /* KEY_V */);
    define(87, 53 /* KEY_W */);
    define(88, 54 /* KEY_X */);
    define(89, 55 /* KEY_Y */);
    define(90, 56 /* KEY_Z */);
    define(93, 58 /* ContextMenu */);
    define(96, 93 /* NUMPAD_0 */);
    define(97, 94 /* NUMPAD_1 */);
    define(98, 95 /* NUMPAD_2 */);
    define(99, 96 /* NUMPAD_3 */);
    define(100, 97 /* NUMPAD_4 */);
    define(101, 98 /* NUMPAD_5 */);
    define(102, 99 /* NUMPAD_6 */);
    define(103, 100 /* NUMPAD_7 */);
    define(104, 101 /* NUMPAD_8 */);
    define(105, 102 /* NUMPAD_9 */);
    define(106, 103 /* NUMPAD_MULTIPLY */);
    define(107, 104 /* NUMPAD_ADD */);
    define(108, 105 /* NUMPAD_SEPARATOR */);
    define(109, 106 /* NUMPAD_SUBTRACT */);
    define(110, 107 /* NUMPAD_DECIMAL */);
    define(111, 108 /* NUMPAD_DIVIDE */);
    define(112, 59 /* F1 */);
    define(113, 60 /* F2 */);
    define(114, 61 /* F3 */);
    define(115, 62 /* F4 */);
    define(116, 63 /* F5 */);
    define(117, 64 /* F6 */);
    define(118, 65 /* F7 */);
    define(119, 66 /* F8 */);
    define(120, 67 /* F9 */);
    define(121, 68 /* F10 */);
    define(122, 69 /* F11 */);
    define(123, 70 /* F12 */);
    define(124, 71 /* F13 */);
    define(125, 72 /* F14 */);
    define(126, 73 /* F15 */);
    define(127, 74 /* F16 */);
    define(128, 75 /* F17 */);
    define(129, 76 /* F18 */);
    define(130, 77 /* F19 */);
    define(144, 78 /* NumLock */);
    define(145, 79 /* ScrollLock */);
    define(186, 80 /* US_SEMICOLON */);
    define(187, 81 /* US_EQUAL */);
    define(188, 82 /* US_COMMA */);
    define(189, 83 /* US_MINUS */);
    define(190, 84 /* US_DOT */);
    define(191, 85 /* US_SLASH */);
    define(192, 86 /* US_BACKTICK */);
    define(193, 110 /* ABNT_C1 */);
    define(194, 111 /* ABNT_C2 */);
    define(219, 87 /* US_OPEN_SQUARE_BRACKET */);
    define(220, 88 /* US_BACKSLASH */);
    define(221, 89 /* US_CLOSE_SQUARE_BRACKET */);
    define(222, 90 /* US_QUOTE */);
    define(223, 91 /* OEM_8 */);
    define(226, 92 /* OEM_102 */);
    /**
     * https://lists.w3.org/Archives/Public/www-dom/2010JulSep/att-0182/keyCode-spec.html
     * If an Input Method Editor is processing key input and the event is keydown, return 229.
     */
    define(229, 109 /* KEY_IN_COMPOSITION */);
    if (isIE) {
        define(91, 57 /* Meta */);
    }
    else if (isFirefox) {
        define(59, 80 /* US_SEMICOLON */);
        define(107, 81 /* US_EQUAL */);
        define(109, 83 /* US_MINUS */);
        if (isMacintosh) {
            define(224, 57 /* Meta */);
        }
    }
    else if (isWebKit) {
        define(91, 57 /* Meta */);
        if (isMacintosh) {
            // the two meta keys in the Mac have different key codes (91 and 93)
            define(93, 57 /* Meta */);
        }
        else {
            define(92, 57 /* Meta */);
        }
    }
})();
function extractKeyCode(e) {
    if (e.charCode) {
        // "keypress" events mostly
        var char = String.fromCharCode(e.charCode).toUpperCase();
        return KeyCodeUtils.fromString(char);
    }
    return KEY_CODE_MAP[e.keyCode] || 0 /* Unknown */;
}
var ctrlKeyMod = (isMacintosh ? 256 /* WinCtrl */ : 2048 /* CtrlCmd */);
var altKeyMod = 512 /* Alt */;
var shiftKeyMod = 1024 /* Shift */;
var metaKeyMod = (isMacintosh ? 2048 /* CtrlCmd */ : 256 /* WinCtrl */);
var StandardKeyboardEvent = /** @class */ (function () {
    function StandardKeyboardEvent(source) {
        var e = source;
        this.browserEvent = e;
        this.target = e.target;
        this.ctrlKey = e.ctrlKey;
        this.shiftKey = e.shiftKey;
        this.altKey = e.altKey;
        this.metaKey = e.metaKey;
        this.keyCode = extractKeyCode(e);
        this.code = e.code;
        // console.info(e.type + ": keyCode: " + e.keyCode + ", which: " + e.which + ", charCode: " + e.charCode + ", detail: " + e.detail + " ====> " + this.keyCode + ' -- ' + KeyCode[this.keyCode]);
        this.ctrlKey = this.ctrlKey || this.keyCode === 5 /* Ctrl */;
        this.altKey = this.altKey || this.keyCode === 6 /* Alt */;
        this.shiftKey = this.shiftKey || this.keyCode === 4 /* Shift */;
        this.metaKey = this.metaKey || this.keyCode === 57 /* Meta */;
        this._asKeybinding = this._computeKeybinding();
        this._asRuntimeKeybinding = this._computeRuntimeKeybinding();
        // console.log(`code: ${e.code}, keyCode: ${e.keyCode}, key: ${e.key}`);
    }
    StandardKeyboardEvent.prototype.preventDefault = function () {
        if (this.browserEvent && this.browserEvent.preventDefault) {
            this.browserEvent.preventDefault();
        }
    };
    StandardKeyboardEvent.prototype.stopPropagation = function () {
        if (this.browserEvent && this.browserEvent.stopPropagation) {
            this.browserEvent.stopPropagation();
        }
    };
    StandardKeyboardEvent.prototype.toKeybinding = function () {
        return this._asRuntimeKeybinding;
    };
    StandardKeyboardEvent.prototype.equals = function (other) {
        return this._asKeybinding === other;
    };
    StandardKeyboardEvent.prototype._computeKeybinding = function () {
        var key = 0 /* Unknown */;
        if (this.keyCode !== 5 /* Ctrl */ && this.keyCode !== 4 /* Shift */ && this.keyCode !== 6 /* Alt */ && this.keyCode !== 57 /* Meta */) {
            key = this.keyCode;
        }
        var result = 0;
        if (this.ctrlKey) {
            result |= ctrlKeyMod;
        }
        if (this.altKey) {
            result |= altKeyMod;
        }
        if (this.shiftKey) {
            result |= shiftKeyMod;
        }
        if (this.metaKey) {
            result |= metaKeyMod;
        }
        result |= key;
        return result;
    };
    StandardKeyboardEvent.prototype._computeRuntimeKeybinding = function () {
        var key = 0 /* Unknown */;
        if (this.keyCode !== 5 /* Ctrl */ && this.keyCode !== 4 /* Shift */ && this.keyCode !== 6 /* Alt */ && this.keyCode !== 57 /* Meta */) {
            key = this.keyCode;
        }
        return new SimpleKeybinding(this.ctrlKey, this.shiftKey, this.altKey, this.metaKey, key);
    };
    return StandardKeyboardEvent;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var hasDifferentOriginAncestorFlag = false;
var sameOriginWindowChainCache = null;
function getParentWindowIfSameOrigin(w) {
    if (!w.parent || w.parent === w) {
        return null;
    }
    // Cannot really tell if we have access to the parent window unless we try to access something in it
    try {
        var location_1 = w.location;
        var parentLocation = w.parent.location;
        if (location_1.protocol !== parentLocation.protocol || location_1.hostname !== parentLocation.hostname || location_1.port !== parentLocation.port) {
            hasDifferentOriginAncestorFlag = true;
            return null;
        }
    }
    catch (e) {
        hasDifferentOriginAncestorFlag = true;
        return null;
    }
    return w.parent;
}
function findIframeElementInParentWindow(parentWindow, childWindow) {
    var parentWindowIframes = parentWindow.document.getElementsByTagName('iframe');
    var iframe;
    for (var i = 0, len = parentWindowIframes.length; i < len; i++) {
        iframe = parentWindowIframes[i];
        if (iframe.contentWindow === childWindow) {
            return iframe;
        }
    }
    return null;
}
var IframeUtils = /** @class */ (function () {
    function IframeUtils() {
    }
    /**
     * Returns a chain of embedded windows with the same origin (which can be accessed programmatically).
     * Having a chain of length 1 might mean that the current execution environment is running outside of an iframe or inside an iframe embedded in a window with a different origin.
     * To distinguish if at one point the current execution environment is running inside a window with a different origin, see hasDifferentOriginAncestor()
     */
    IframeUtils.getSameOriginWindowChain = function () {
        if (!sameOriginWindowChainCache) {
            sameOriginWindowChainCache = [];
            var w = window;
            var parent_1;
            do {
                parent_1 = getParentWindowIfSameOrigin(w);
                if (parent_1) {
                    sameOriginWindowChainCache.push({
                        window: w,
                        iframeElement: findIframeElementInParentWindow(parent_1, w)
                    });
                }
                else {
                    sameOriginWindowChainCache.push({
                        window: w,
                        iframeElement: null
                    });
                }
                w = parent_1;
            } while (w);
        }
        return sameOriginWindowChainCache.slice(0);
    };
    /**
     * Returns true if the current execution environment is chained in a list of iframes which at one point ends in a window with a different origin.
     * Returns false if the current execution environment is not running inside an iframe or if the entire chain of iframes have the same origin.
     */
    IframeUtils.hasDifferentOriginAncestor = function () {
        if (!sameOriginWindowChainCache) {
            this.getSameOriginWindowChain();
        }
        return hasDifferentOriginAncestorFlag;
    };
    /**
     * Returns the position of `childWindow` relative to `ancestorWindow`
     */
    IframeUtils.getPositionOfChildWindowRelativeToAncestorWindow = function (childWindow, ancestorWindow) {
        if (!ancestorWindow || childWindow === ancestorWindow) {
            return {
                top: 0,
                left: 0
            };
        }
        var top = 0, left = 0;
        var windowChain = this.getSameOriginWindowChain();
        for (var i = 0; i < windowChain.length; i++) {
            var windowChainEl = windowChain[i];
            if (windowChainEl.window === ancestorWindow) {
                break;
            }
            if (!windowChainEl.iframeElement) {
                break;
            }
            var boundingRect = windowChainEl.iframeElement.getBoundingClientRect();
            top += boundingRect.top;
            left += boundingRect.left;
        }
        return {
            top: top,
            left: left
        };
    };
    return IframeUtils;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$d = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var StandardMouseEvent = /** @class */ (function () {
    function StandardMouseEvent(e) {
        this.timestamp = Date.now();
        this.browserEvent = e;
        this.leftButton = e.button === 0;
        this.middleButton = e.button === 1;
        this.rightButton = e.button === 2;
        this.target = e.target;
        this.detail = e.detail || 1;
        if (e.type === 'dblclick') {
            this.detail = 2;
        }
        this.ctrlKey = e.ctrlKey;
        this.shiftKey = e.shiftKey;
        this.altKey = e.altKey;
        this.metaKey = e.metaKey;
        if (typeof e.pageX === 'number') {
            this.posx = e.pageX;
            this.posy = e.pageY;
        }
        else {
            // Probably hit by MSGestureEvent
            this.posx = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
            this.posy = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
        }
        // Find the position of the iframe this code is executing in relative to the iframe where the event was captured.
        var iframeOffsets = IframeUtils.getPositionOfChildWindowRelativeToAncestorWindow(self, e.view);
        this.posx -= iframeOffsets.left;
        this.posy -= iframeOffsets.top;
    }
    StandardMouseEvent.prototype.preventDefault = function () {
        if (this.browserEvent.preventDefault) {
            this.browserEvent.preventDefault();
        }
    };
    StandardMouseEvent.prototype.stopPropagation = function () {
        if (this.browserEvent.stopPropagation) {
            this.browserEvent.stopPropagation();
        }
    };
    return StandardMouseEvent;
}());
var DragMouseEvent = /** @class */ (function (_super) {
    __extends$d(DragMouseEvent, _super);
    function DragMouseEvent(e) {
        var _this = _super.call(this, e) || this;
        _this.dataTransfer = e.dataTransfer;
        return _this;
    }
    return DragMouseEvent;
}(StandardMouseEvent));
var StandardMouseWheelEvent = /** @class */ (function () {
    function StandardMouseWheelEvent(e, deltaX, deltaY) {
        if (deltaX === void 0) { deltaX = 0; }
        if (deltaY === void 0) { deltaY = 0; }
        this.browserEvent = e || null;
        this.target = e ? (e.target || e.targetNode || e.srcElement) : null;
        this.deltaY = deltaY;
        this.deltaX = deltaX;
        if (e) {
            var e1 = e;
            var e2 = e;
            // vertical delta scroll
            if (typeof e1.wheelDeltaY !== 'undefined') {
                this.deltaY = e1.wheelDeltaY / 120;
            }
            else if (typeof e2.VERTICAL_AXIS !== 'undefined' && e2.axis === e2.VERTICAL_AXIS) {
                this.deltaY = -e2.detail / 3;
            }
            // horizontal delta scroll
            if (typeof e1.wheelDeltaX !== 'undefined') {
                if (isSafari && isWindows) {
                    this.deltaX = -(e1.wheelDeltaX / 120);
                }
                else {
                    this.deltaX = e1.wheelDeltaX / 120;
                }
            }
            else if (typeof e2.HORIZONTAL_AXIS !== 'undefined' && e2.axis === e2.HORIZONTAL_AXIS) {
                this.deltaX = -e.detail / 3;
            }
            // Assume a vertical scroll if nothing else worked
            if (this.deltaY === 0 && this.deltaX === 0 && e.wheelDelta) {
                this.deltaY = e.wheelDelta / 120;
            }
        }
    }
    StandardMouseWheelEvent.prototype.preventDefault = function () {
        if (this.browserEvent) {
            if (this.browserEvent.preventDefault) {
                this.browserEvent.preventDefault();
            }
        }
    };
    StandardMouseWheelEvent.prototype.stopPropagation = function () {
        if (this.browserEvent) {
            if (this.browserEvent.stopPropagation) {
                this.browserEvent.stopPropagation();
            }
        }
    };
    return StandardMouseWheelEvent;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$e = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function clearNode(node) {
    while (node.firstChild) {
        node.removeChild(node.firstChild);
    }
}
function removeNode(node) {
    if (node.parentNode) {
        node.parentNode.removeChild(node);
    }
}
function isInDOM(node) {
    while (node) {
        if (node === document.body) {
            return true;
        }
        node = node.parentNode;
    }
    return false;
}
var _manualClassList = new /** @class */ (function () {
    function class_1() {
    }
    class_1.prototype._findClassName = function (node, className) {
        var classes = node.className;
        if (!classes) {
            this._lastStart = -1;
            return;
        }
        className = className.trim();
        var classesLen = classes.length, classLen = className.length;
        if (classLen === 0) {
            this._lastStart = -1;
            return;
        }
        if (classesLen < classLen) {
            this._lastStart = -1;
            return;
        }
        if (classes === className) {
            this._lastStart = 0;
            this._lastEnd = classesLen;
            return;
        }
        var idx = -1, idxEnd;
        while ((idx = classes.indexOf(className, idx + 1)) >= 0) {
            idxEnd = idx + classLen;
            // a class that is followed by another class
            if ((idx === 0 || classes.charCodeAt(idx - 1) === 32 /* Space */) && classes.charCodeAt(idxEnd) === 32 /* Space */) {
                this._lastStart = idx;
                this._lastEnd = idxEnd + 1;
                return;
            }
            // last class
            if (idx > 0 && classes.charCodeAt(idx - 1) === 32 /* Space */ && idxEnd === classesLen) {
                this._lastStart = idx - 1;
                this._lastEnd = idxEnd;
                return;
            }
            // equal - duplicate of cmp above
            if (idx === 0 && idxEnd === classesLen) {
                this._lastStart = 0;
                this._lastEnd = idxEnd;
                return;
            }
        }
        this._lastStart = -1;
    };
    class_1.prototype.hasClass = function (node, className) {
        this._findClassName(node, className);
        return this._lastStart !== -1;
    };
    class_1.prototype.addClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) { return nameValue.split(' ').forEach(function (name) { return _this.addClass(node, name); }); });
    };
    class_1.prototype.addClass = function (node, className) {
        if (!node.className) { // doesn't have it for sure
            node.className = className;
        }
        else {
            this._findClassName(node, className); // see if it's already there
            if (this._lastStart === -1) {
                node.className = node.className + ' ' + className;
            }
        }
    };
    class_1.prototype.removeClass = function (node, className) {
        this._findClassName(node, className);
        if (this._lastStart === -1) {
            return; // Prevent styles invalidation if not necessary
        }
        else {
            node.className = node.className.substring(0, this._lastStart) + node.className.substring(this._lastEnd);
        }
    };
    class_1.prototype.removeClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) { return nameValue.split(' ').forEach(function (name) { return _this.removeClass(node, name); }); });
    };
    class_1.prototype.toggleClass = function (node, className, shouldHaveIt) {
        this._findClassName(node, className);
        if (this._lastStart !== -1 && (shouldHaveIt === void 0 || !shouldHaveIt)) {
            this.removeClass(node, className);
        }
        if (this._lastStart === -1 && (shouldHaveIt === void 0 || shouldHaveIt)) {
            this.addClass(node, className);
        }
    };
    return class_1;
}());
var _nativeClassList = new /** @class */ (function () {
    function class_2() {
    }
    class_2.prototype.hasClass = function (node, className) {
        return Boolean(className) && node.classList && node.classList.contains(className);
    };
    class_2.prototype.addClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) { return nameValue.split(' ').forEach(function (name) { return _this.addClass(node, name); }); });
    };
    class_2.prototype.addClass = function (node, className) {
        if (className && node.classList) {
            node.classList.add(className);
        }
    };
    class_2.prototype.removeClass = function (node, className) {
        if (className && node.classList) {
            node.classList.remove(className);
        }
    };
    class_2.prototype.removeClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) { return nameValue.split(' ').forEach(function (name) { return _this.removeClass(node, name); }); });
    };
    class_2.prototype.toggleClass = function (node, className, shouldHaveIt) {
        if (node.classList) {
            node.classList.toggle(className, shouldHaveIt);
        }
    };
    return class_2;
}());
// In IE11 there is only partial support for `classList` which makes us keep our
// custom implementation. Otherwise use the native implementation, see: http://caniuse.com/#search=classlist
var _classList = isIE ? _manualClassList : _nativeClassList;
var hasClass = _classList.hasClass.bind(_classList);
var addClass = _classList.addClass.bind(_classList);
var addClasses = _classList.addClasses.bind(_classList);
var removeClass = _classList.removeClass.bind(_classList);
var removeClasses = _classList.removeClasses.bind(_classList);
var toggleClass = _classList.toggleClass.bind(_classList);
var DomListener = /** @class */ (function () {
    function DomListener(node, type, handler, useCapture) {
        this._node = node;
        this._type = type;
        this._handler = handler;
        this._useCapture = (useCapture || false);
        this._node.addEventListener(this._type, this._handler, this._useCapture);
    }
    DomListener.prototype.dispose = function () {
        if (!this._handler) {
            // Already disposed
            return;
        }
        this._node.removeEventListener(this._type, this._handler, this._useCapture);
        // Prevent leakers from holding on to the dom or handler func
        this._node = null;
        this._handler = null;
    };
    return DomListener;
}());
function addDisposableListener(node, type, handler, useCapture) {
    return new DomListener(node, type, handler, useCapture);
}
function _wrapAsStandardMouseEvent(handler) {
    return function (e) {
        return handler(new StandardMouseEvent(e));
    };
}
function _wrapAsStandardKeyboardEvent(handler) {
    return function (e) {
        return handler(new StandardKeyboardEvent(e));
    };
}
var addStandardDisposableListener = function addStandardDisposableListener(node, type, handler, useCapture) {
    var wrapHandler = handler;
    if (type === 'click' || type === 'mousedown') {
        wrapHandler = _wrapAsStandardMouseEvent(handler);
    }
    else if (type === 'keydown' || type === 'keypress' || type === 'keyup') {
        wrapHandler = _wrapAsStandardKeyboardEvent(handler);
    }
    return addDisposableListener(node, type, wrapHandler, useCapture);
};
function addDisposableNonBubblingMouseOutListener(node, handler) {
    return addDisposableListener(node, 'mouseout', function (e) {
        // Mouse out bubbles, so this is an attempt to ignore faux mouse outs coming from children elements
        var toElement = (e.relatedTarget || e.toElement);
        while (toElement && toElement !== node) {
            toElement = toElement.parentNode;
        }
        if (toElement === node) {
            return;
        }
        handler(e);
    });
}
var _animationFrame = null;
function doRequestAnimationFrame(callback) {
    if (!_animationFrame) {
        var emulatedRequestAnimationFrame = function (callback) {
            return setTimeout(function () { return callback(new Date().getTime()); }, 0);
        };
        _animationFrame = (self.requestAnimationFrame
            || self.msRequestAnimationFrame
            || self.webkitRequestAnimationFrame
            || self.mozRequestAnimationFrame
            || self.oRequestAnimationFrame
            || emulatedRequestAnimationFrame);
    }
    return _animationFrame.call(self, callback);
}
/**
 * Schedule a callback to be run at the next animation frame.
 * This allows multiple parties to register callbacks that should run at the next animation frame.
 * If currently in an animation frame, `runner` will be executed immediately.
 * @return token that can be used to cancel the scheduled runner (only if `runner` was not executed immediately).
 */
var runAtThisOrScheduleAtNextAnimationFrame;
/**
 * Schedule a callback to be run at the next animation frame.
 * This allows multiple parties to register callbacks that should run at the next animation frame.
 * If currently in an animation frame, `runner` will be executed at the next animation frame.
 * @return token that can be used to cancel the scheduled runner.
 */
var scheduleAtNextAnimationFrame;
var AnimationFrameQueueItem = /** @class */ (function () {
    function AnimationFrameQueueItem(runner, priority) {
        if (priority === void 0) { priority = 0; }
        this._runner = runner;
        this.priority = priority;
        this._canceled = false;
    }
    AnimationFrameQueueItem.prototype.dispose = function () {
        this._canceled = true;
    };
    AnimationFrameQueueItem.prototype.execute = function () {
        if (this._canceled) {
            return;
        }
        try {
            this._runner();
        }
        catch (e) {
            onUnexpectedError(e);
        }
    };
    // Sort by priority (largest to lowest)
    AnimationFrameQueueItem.sort = function (a, b) {
        return b.priority - a.priority;
    };
    return AnimationFrameQueueItem;
}());
(function () {
    /**
     * The runners scheduled at the next animation frame
     */
    var NEXT_QUEUE = [];
    /**
     * The runners scheduled at the current animation frame
     */
    var CURRENT_QUEUE = null;
    /**
     * A flag to keep track if the native requestAnimationFrame was already called
     */
    var animFrameRequested = false;
    /**
     * A flag to indicate if currently handling a native requestAnimationFrame callback
     */
    var inAnimationFrameRunner = false;
    var animationFrameRunner = function () {
        animFrameRequested = false;
        CURRENT_QUEUE = NEXT_QUEUE;
        NEXT_QUEUE = [];
        inAnimationFrameRunner = true;
        while (CURRENT_QUEUE.length > 0) {
            CURRENT_QUEUE.sort(AnimationFrameQueueItem.sort);
            var top_1 = CURRENT_QUEUE.shift();
            top_1.execute();
        }
        inAnimationFrameRunner = false;
    };
    scheduleAtNextAnimationFrame = function (runner, priority) {
        if (priority === void 0) { priority = 0; }
        var item = new AnimationFrameQueueItem(runner, priority);
        NEXT_QUEUE.push(item);
        if (!animFrameRequested) {
            animFrameRequested = true;
            doRequestAnimationFrame(animationFrameRunner);
        }
        return item;
    };
    runAtThisOrScheduleAtNextAnimationFrame = function (runner, priority) {
        if (inAnimationFrameRunner) {
            var item = new AnimationFrameQueueItem(runner, priority);
            CURRENT_QUEUE.push(item);
            return item;
        }
        else {
            return scheduleAtNextAnimationFrame(runner, priority);
        }
    };
})();
var MINIMUM_TIME_MS = 16;
var DEFAULT_EVENT_MERGER = function (lastEvent, currentEvent) {
    return currentEvent;
};
var TimeoutThrottledDomListener = /** @class */ (function (_super) {
    __extends$e(TimeoutThrottledDomListener, _super);
    function TimeoutThrottledDomListener(node, type, handler, eventMerger, minimumTimeMs) {
        if (eventMerger === void 0) { eventMerger = DEFAULT_EVENT_MERGER; }
        if (minimumTimeMs === void 0) { minimumTimeMs = MINIMUM_TIME_MS; }
        var _this = _super.call(this) || this;
        var lastEvent = null;
        var lastHandlerTime = 0;
        var timeout$$1 = _this._register(new TimeoutTimer());
        var invokeHandler = function () {
            lastHandlerTime = (new Date()).getTime();
            handler(lastEvent);
            lastEvent = null;
        };
        _this._register(addDisposableListener(node, type, function (e) {
            lastEvent = eventMerger(lastEvent, e);
            var elapsedTime = (new Date()).getTime() - lastHandlerTime;
            if (elapsedTime >= minimumTimeMs) {
                timeout$$1.cancel();
                invokeHandler();
            }
            else {
                timeout$$1.setIfNotSet(invokeHandler, minimumTimeMs - elapsedTime);
            }
        }));
        return _this;
    }
    return TimeoutThrottledDomListener;
}(Disposable));
function addDisposableThrottledListener(node, type, handler, eventMerger, minimumTimeMs) {
    return new TimeoutThrottledDomListener(node, type, handler, eventMerger, minimumTimeMs);
}
function getComputedStyle(el) {
    return document.defaultView.getComputedStyle(el, null);
}
// Adapted from WinJS
// Converts a CSS positioning string for the specified element to pixels.
var convertToPixels = (function () {
    return function (element, value) {
        return parseFloat(value) || 0;
    };
})();
function getDimension(element, cssPropertyName, jsPropertyName) {
    var computedStyle = getComputedStyle(element);
    var value = '0';
    if (computedStyle) {
        if (computedStyle.getPropertyValue) {
            value = computedStyle.getPropertyValue(cssPropertyName);
        }
        else {
            // IE8
            value = computedStyle.getAttribute(jsPropertyName);
        }
    }
    return convertToPixels(element, value);
}
function getClientArea(element) {
    // Try with DOM clientWidth / clientHeight
    if (element !== document.body) {
        return new Dimension(element.clientWidth, element.clientHeight);
    }
    // Try innerWidth / innerHeight
    if (window.innerWidth && window.innerHeight) {
        return new Dimension(window.innerWidth, window.innerHeight);
    }
    // Try with document.body.clientWidth / document.body.clientHeight
    if (document.body && document.body.clientWidth && document.body.clientHeight) {
        return new Dimension(document.body.clientWidth, document.body.clientHeight);
    }
    // Try with document.documentElement.clientWidth / document.documentElement.clientHeight
    if (document.documentElement && document.documentElement.clientWidth && document.documentElement.clientHeight) {
        return new Dimension(document.documentElement.clientWidth, document.documentElement.clientHeight);
    }
    throw new Error('Unable to figure out browser width and height');
}
var sizeUtils = {
    getBorderLeftWidth: function (element) {
        return getDimension(element, 'border-left-width', 'borderLeftWidth');
    },
    getBorderRightWidth: function (element) {
        return getDimension(element, 'border-right-width', 'borderRightWidth');
    },
    getBorderTopWidth: function (element) {
        return getDimension(element, 'border-top-width', 'borderTopWidth');
    },
    getBorderBottomWidth: function (element) {
        return getDimension(element, 'border-bottom-width', 'borderBottomWidth');
    },
    getPaddingLeft: function (element) {
        return getDimension(element, 'padding-left', 'paddingLeft');
    },
    getPaddingRight: function (element) {
        return getDimension(element, 'padding-right', 'paddingRight');
    },
    getPaddingTop: function (element) {
        return getDimension(element, 'padding-top', 'paddingTop');
    },
    getPaddingBottom: function (element) {
        return getDimension(element, 'padding-bottom', 'paddingBottom');
    },
    getMarginLeft: function (element) {
        return getDimension(element, 'margin-left', 'marginLeft');
    },
    getMarginTop: function (element) {
        return getDimension(element, 'margin-top', 'marginTop');
    },
    getMarginRight: function (element) {
        return getDimension(element, 'margin-right', 'marginRight');
    },
    getMarginBottom: function (element) {
        return getDimension(element, 'margin-bottom', 'marginBottom');
    },
    __commaSentinel: false
};
// ----------------------------------------------------------------------------------------
// Position & Dimension
var Dimension = /** @class */ (function () {
    function Dimension(width, height) {
        this.width = width;
        this.height = height;
    }
    return Dimension;
}());
function getTopLeftOffset(element) {
    // Adapted from WinJS.Utilities.getPosition
    // and added borders to the mix
    var offsetParent = element.offsetParent, top = element.offsetTop, left = element.offsetLeft;
    while ((element = element.parentNode) !== null && element !== document.body && element !== document.documentElement) {
        top -= element.scrollTop;
        var c = getComputedStyle(element);
        if (c) {
            left -= c.direction !== 'rtl' ? element.scrollLeft : -element.scrollLeft;
        }
        if (element === offsetParent) {
            left += sizeUtils.getBorderLeftWidth(element);
            top += sizeUtils.getBorderTopWidth(element);
            top += element.offsetTop;
            left += element.offsetLeft;
            offsetParent = element.offsetParent;
        }
    }
    return {
        left: left,
        top: top
    };
}
/**
 * Returns the position of a dom node relative to the entire page.
 */
function getDomNodePagePosition(domNode) {
    var bb = domNode.getBoundingClientRect();
    return {
        left: bb.left + StandardWindow.scrollX,
        top: bb.top + StandardWindow.scrollY,
        width: bb.width,
        height: bb.height
    };
}
var StandardWindow = new /** @class */ (function () {
    function class_3() {
    }
    Object.defineProperty(class_3.prototype, "scrollX", {
        get: function () {
            if (typeof window.scrollX === 'number') {
                // modern browsers
                return window.scrollX;
            }
            else {
                return document.body.scrollLeft + document.documentElement.scrollLeft;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(class_3.prototype, "scrollY", {
        get: function () {
            if (typeof window.scrollY === 'number') {
                // modern browsers
                return window.scrollY;
            }
            else {
                return document.body.scrollTop + document.documentElement.scrollTop;
            }
        },
        enumerable: true,
        configurable: true
    });
    return class_3;
}());
// Adapted from WinJS
// Gets the width of the element, including margins.
function getTotalWidth(element) {
    var margin = sizeUtils.getMarginLeft(element) + sizeUtils.getMarginRight(element);
    return element.offsetWidth + margin;
}
function getContentWidth(element) {
    var border = sizeUtils.getBorderLeftWidth(element) + sizeUtils.getBorderRightWidth(element);
    var padding = sizeUtils.getPaddingLeft(element) + sizeUtils.getPaddingRight(element);
    return element.offsetWidth - border - padding;
}
// Adapted from WinJS
// Gets the height of the content of the specified element. The content height does not include borders or padding.
function getContentHeight(element) {
    var border = sizeUtils.getBorderTopWidth(element) + sizeUtils.getBorderBottomWidth(element);
    var padding = sizeUtils.getPaddingTop(element) + sizeUtils.getPaddingBottom(element);
    return element.offsetHeight - border - padding;
}
// Adapted from WinJS
// Gets the height of the element, including its margins.
function getTotalHeight(element) {
    var margin = sizeUtils.getMarginTop(element) + sizeUtils.getMarginBottom(element);
    return element.offsetHeight + margin;
}
// ----------------------------------------------------------------------------------------
function isAncestor(testChild, testAncestor) {
    while (testChild) {
        if (testChild === testAncestor) {
            return true;
        }
        testChild = testChild.parentNode;
    }
    return false;
}
function findParentWithClass(node, clazz, stopAtClazzOrNode) {
    while (node) {
        if (hasClass(node, clazz)) {
            return node;
        }
        if (stopAtClazzOrNode) {
            if (typeof stopAtClazzOrNode === 'string') {
                if (hasClass(node, stopAtClazzOrNode)) {
                    return null;
                }
            }
            else {
                if (node === stopAtClazzOrNode) {
                    return null;
                }
            }
        }
        node = node.parentNode;
    }
    return null;
}
function createStyleSheet(container) {
    if (container === void 0) { container = document.getElementsByTagName('head')[0]; }
    var style = document.createElement('style');
    style.type = 'text/css';
    style.media = 'screen';
    container.appendChild(style);
    return style;
}
var _sharedStyleSheet = null;
function getSharedStyleSheet() {
    if (!_sharedStyleSheet) {
        _sharedStyleSheet = createStyleSheet();
    }
    return _sharedStyleSheet;
}
function getDynamicStyleSheetRules(style) {
    if (style && style.sheet && style.sheet.rules) {
        // Chrome, IE
        return style.sheet.rules;
    }
    if (style && style.sheet && style.sheet.cssRules) {
        // FF
        return style.sheet.cssRules;
    }
    return [];
}
function createCSSRule(selector, cssText, style) {
    if (style === void 0) { style = getSharedStyleSheet(); }
    if (!style || !cssText) {
        return;
    }
    style.sheet.insertRule(selector + '{' + cssText + '}', 0);
}
function removeCSSRulesContainingSelector(ruleName, style) {
    if (style === void 0) { style = getSharedStyleSheet(); }
    if (!style) {
        return;
    }
    var rules = getDynamicStyleSheetRules(style);
    var toDelete = [];
    for (var i = 0; i < rules.length; i++) {
        var rule = rules[i];
        if (rule.selectorText.indexOf(ruleName) !== -1) {
            toDelete.push(i);
        }
    }
    for (var i = toDelete.length - 1; i >= 0; i--) {
        style.sheet.deleteRule(toDelete[i]);
    }
}
function isHTMLElement(o) {
    if (typeof HTMLElement === 'object') {
        return o instanceof HTMLElement;
    }
    return o && typeof o === 'object' && o.nodeType === 1 && typeof o.nodeName === 'string';
}
var EventType = {
    // Mouse
    CLICK: 'click',
    DBLCLICK: 'dblclick',
    MOUSE_UP: 'mouseup',
    MOUSE_DOWN: 'mousedown',
    MOUSE_OVER: 'mouseover',
    MOUSE_MOVE: 'mousemove',
    MOUSE_OUT: 'mouseout',
    MOUSE_ENTER: 'mouseenter',
    MOUSE_LEAVE: 'mouseleave',
    CONTEXT_MENU: 'contextmenu',
    WHEEL: 'wheel',
    // Keyboard
    KEY_DOWN: 'keydown',
    KEY_PRESS: 'keypress',
    KEY_UP: 'keyup',
    // HTML Document
    LOAD: 'load',
    UNLOAD: 'unload',
    ABORT: 'abort',
    ERROR: 'error',
    RESIZE: 'resize',
    SCROLL: 'scroll',
    // Form
    SELECT: 'select',
    CHANGE: 'change',
    SUBMIT: 'submit',
    RESET: 'reset',
    FOCUS: 'focus',
    FOCUS_IN: 'focusin',
    FOCUS_OUT: 'focusout',
    BLUR: 'blur',
    INPUT: 'input',
    // Local Storage
    STORAGE: 'storage',
    // Drag
    DRAG_START: 'dragstart',
    DRAG: 'drag',
    DRAG_ENTER: 'dragenter',
    DRAG_LEAVE: 'dragleave',
    DRAG_OVER: 'dragover',
    DROP: 'drop',
    DRAG_END: 'dragend',
    // Animation
    ANIMATION_START: isWebKit ? 'webkitAnimationStart' : 'animationstart',
    ANIMATION_END: isWebKit ? 'webkitAnimationEnd' : 'animationend',
    ANIMATION_ITERATION: isWebKit ? 'webkitAnimationIteration' : 'animationiteration'
};
var EventHelper = {
    stop: function (e, cancelBubble) {
        if (e.preventDefault) {
            e.preventDefault();
        }
        else {
            // IE8
            e.returnValue = false;
        }
        if (cancelBubble) {
            if (e.stopPropagation) {
                e.stopPropagation();
            }
            else {
                // IE8
                e.cancelBubble = true;
            }
        }
    }
};
function saveParentsScrollTop(node) {
    var r = [];
    for (var i = 0; node && node.nodeType === node.ELEMENT_NODE; i++) {
        r[i] = node.scrollTop;
        node = node.parentNode;
    }
    return r;
}
function restoreParentsScrollTop(node, state) {
    for (var i = 0; node && node.nodeType === node.ELEMENT_NODE; i++) {
        if (node.scrollTop !== state[i]) {
            node.scrollTop = state[i];
        }
        node = node.parentNode;
    }
}
var FocusTracker = /** @class */ (function () {
    function FocusTracker(element) {
        var _this = this;
        this._onDidFocus = new Emitter();
        this.onDidFocus = this._onDidFocus.event;
        this._onDidBlur = new Emitter();
        this.onDidBlur = this._onDidBlur.event;
        this.disposables = [];
        var hasFocus = isAncestor(document.activeElement, element);
        var loosingFocus = false;
        var onFocus = function () {
            loosingFocus = false;
            if (!hasFocus) {
                hasFocus = true;
                _this._onDidFocus.fire();
            }
        };
        var onBlur = function () {
            if (hasFocus) {
                loosingFocus = true;
                window.setTimeout(function () {
                    if (loosingFocus) {
                        loosingFocus = false;
                        hasFocus = false;
                        _this._onDidBlur.fire();
                    }
                }, 0);
            }
        };
        domEvent(element, EventType.FOCUS, true)(onFocus, null, this.disposables);
        domEvent(element, EventType.BLUR, true)(onBlur, null, this.disposables);
    }
    FocusTracker.prototype.dispose = function () {
        this.disposables = dispose(this.disposables);
        this._onDidFocus.dispose();
        this._onDidBlur.dispose();
    };
    return FocusTracker;
}());
function trackFocus(element) {
    return new FocusTracker(element);
}
function append(parent) {
    var children = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        children[_i - 1] = arguments[_i];
    }
    children.forEach(function (child) { return parent.appendChild(child); });
    return children[children.length - 1];
}
var SELECTOR_REGEX = /([\w\-]+)?(#([\w\-]+))?((.([\w\-]+))*)/;
function $(description, attrs) {
    var children = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        children[_i - 2] = arguments[_i];
    }
    var match = SELECTOR_REGEX.exec(description);
    if (!match) {
        throw new Error('Bad use of emmet');
    }
    var result = document.createElement(match[1] || 'div');
    if (match[3]) {
        result.id = match[3];
    }
    if (match[4]) {
        result.className = match[4].replace(/\./g, ' ').trim();
    }
    attrs = attrs || {};
    Object.keys(attrs).forEach(function (name) {
        var value = attrs[name];
        if (/^on\w+$/.test(name)) {
            result[name] = value;
        }
        else if (name === 'selected') {
            if (value) {
                result.setAttribute(name, 'true');
            }
        }
        else {
            result.setAttribute(name, value);
        }
    });
    children
        .filter(function (child) { return !!child; })
        .forEach(function (child) {
        if (child instanceof Node) {
            result.appendChild(child);
        }
        else {
            result.appendChild(document.createTextNode(child));
        }
    });
    return result;
}
function show() {
    var elements = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        elements[_i] = arguments[_i];
    }
    for (var _a = 0, elements_1 = elements; _a < elements_1.length; _a++) {
        var element = elements_1[_a];
        element.style.display = '';
        element.removeAttribute('aria-hidden');
    }
}
function hide() {
    var elements = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        elements[_i] = arguments[_i];
    }
    for (var _a = 0, elements_2 = elements; _a < elements_2.length; _a++) {
        var element = elements_2[_a];
        element.style.display = 'none';
        element.setAttribute('aria-hidden', 'true');
    }
}
function findParentWithAttribute(node, attribute) {
    while (node) {
        if (node instanceof HTMLElement && node.hasAttribute(attribute)) {
            return node;
        }
        node = node.parentNode;
    }
    return null;
}
function removeTabIndexAndUpdateFocus(node) {
    if (!node || !node.hasAttribute('tabIndex')) {
        return;
    }
    // If we are the currently focused element and tabIndex is removed,
    // standard DOM behavior is to move focus to the <body> element. We
    // typically never want that, rather put focus to the closest element
    // in the hierarchy of the parent DOM nodes.
    if (document.activeElement === node) {
        var parentFocusable = findParentWithAttribute(node.parentElement, 'tabIndex');
        if (parentFocusable) {
            parentFocusable.focus();
        }
    }
    node.removeAttribute('tabindex');
}
function getElementsByTagName(tag) {
    return Array.prototype.slice.call(document.getElementsByTagName(tag), 0);
}
/**
 * Find a value usable for a dom node size such that the likelihood that it would be
 * displayed with constant screen pixels size is as high as possible.
 *
 * e.g. We would desire for the cursors to be 2px (CSS px) wide. Under a devicePixelRatio
 * of 1.25, the cursor will be 2.5 screen pixels wide. Depending on how the dom node aligns/"snaps"
 * with the screen pixels, it will sometimes be rendered with 2 screen pixels, and sometimes with 3 screen pixels.
 */
function computeScreenAwareSize(cssPx) {
    var screenPx = window.devicePixelRatio * cssPx;
    return Math.max(1, Math.floor(screenPx)) / window.devicePixelRatio;
}
/**
 * See https://github.com/Microsoft/monaco-editor/issues/601
 * To protect against malicious code in the linked site, particularly phishing attempts,
 * the window.opener should be set to null to prevent the linked site from having access
 * to change the location of the current page.
 * See https://mathiasbynens.github.io/rel-noopener/
 */
function windowOpenNoOpener(url) {
    if (isNative || isEdgeWebView) {
        // In VSCode, window.open() always returns null...
        // The same is true for a WebView (see https://github.com/Microsoft/monaco-editor/issues/628)
        window.open(url);
    }
    else {
        var newTab = window.open();
        if (newTab) {
            newTab.opener = null;
            newTab.location.href = url;
        }
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Schemas;
(function (Schemas) {
    /**
     * A schema that is used for models that exist in memory
     * only and that have no correspondence on a server or such.
     */
    Schemas.inMemory = 'inmemory';
    /**
     * A schema that is used for setting files
     */
    Schemas.vscode = 'vscode';
    /**
     * A schema that is used for internal private files
     */
    Schemas.internal = 'private';
    /**
     * A walk-through document.
     */
    Schemas.walkThrough = 'walkThrough';
    /**
     * An embedded code snippet.
     */
    Schemas.walkThroughSnippet = 'walkThroughSnippet';
    Schemas.http = 'http';
    Schemas.https = 'https';
    Schemas.file = 'file';
    Schemas.mailto = 'mailto';
    Schemas.untitled = 'untitled';
    Schemas.data = 'data';
})(Schemas || (Schemas = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CharWidthRequest = /** @class */ (function () {
    function CharWidthRequest(chr, type) {
        this.chr = chr;
        this.type = type;
        this.width = 0;
    }
    CharWidthRequest.prototype.fulfill = function (width) {
        this.width = width;
    };
    return CharWidthRequest;
}());
var DomCharWidthReader = /** @class */ (function () {
    function DomCharWidthReader(bareFontInfo, requests) {
        this._bareFontInfo = bareFontInfo;
        this._requests = requests;
        this._container = null;
        this._testElements = null;
    }
    DomCharWidthReader.prototype.read = function () {
        // Create a test container with all these test elements
        this._createDomElements();
        // Add the container to the DOM
        document.body.appendChild(this._container);
        // Read character widths
        this._readFromDomElements();
        // Remove the container from the DOM
        document.body.removeChild(this._container);
        this._container = null;
        this._testElements = null;
    };
    DomCharWidthReader.prototype._createDomElements = function () {
        var container = document.createElement('div');
        container.style.position = 'absolute';
        container.style.top = '-50000px';
        container.style.width = '50000px';
        var regularDomNode = document.createElement('div');
        regularDomNode.style.fontFamily = this._bareFontInfo.getMassagedFontFamily();
        regularDomNode.style.fontWeight = this._bareFontInfo.fontWeight;
        regularDomNode.style.fontSize = this._bareFontInfo.fontSize + 'px';
        regularDomNode.style.lineHeight = this._bareFontInfo.lineHeight + 'px';
        regularDomNode.style.letterSpacing = this._bareFontInfo.letterSpacing + 'px';
        container.appendChild(regularDomNode);
        var boldDomNode = document.createElement('div');
        boldDomNode.style.fontFamily = this._bareFontInfo.getMassagedFontFamily();
        boldDomNode.style.fontWeight = 'bold';
        boldDomNode.style.fontSize = this._bareFontInfo.fontSize + 'px';
        boldDomNode.style.lineHeight = this._bareFontInfo.lineHeight + 'px';
        boldDomNode.style.letterSpacing = this._bareFontInfo.letterSpacing + 'px';
        container.appendChild(boldDomNode);
        var italicDomNode = document.createElement('div');
        italicDomNode.style.fontFamily = this._bareFontInfo.getMassagedFontFamily();
        italicDomNode.style.fontWeight = this._bareFontInfo.fontWeight;
        italicDomNode.style.fontSize = this._bareFontInfo.fontSize + 'px';
        italicDomNode.style.lineHeight = this._bareFontInfo.lineHeight + 'px';
        italicDomNode.style.letterSpacing = this._bareFontInfo.letterSpacing + 'px';
        italicDomNode.style.fontStyle = 'italic';
        container.appendChild(italicDomNode);
        var testElements = [];
        for (var i = 0, len = this._requests.length; i < len; i++) {
            var request = this._requests[i];
            var parent_1 = void 0;
            if (request.type === 0 /* Regular */) {
                parent_1 = regularDomNode;
            }
            if (request.type === 2 /* Bold */) {
                parent_1 = boldDomNode;
            }
            if (request.type === 1 /* Italic */) {
                parent_1 = italicDomNode;
            }
            parent_1.appendChild(document.createElement('br'));
            var testElement = document.createElement('span');
            DomCharWidthReader._render(testElement, request);
            parent_1.appendChild(testElement);
            testElements[i] = testElement;
        }
        this._container = container;
        this._testElements = testElements;
    };
    DomCharWidthReader._render = function (testElement, request) {
        if (request.chr === ' ') {
            var htmlString = '&nbsp;';
            // Repeat character 256 (2^8) times
            for (var i = 0; i < 8; i++) {
                htmlString += htmlString;
            }
            testElement.innerHTML = htmlString;
        }
        else {
            var testString = request.chr;
            // Repeat character 256 (2^8) times
            for (var i = 0; i < 8; i++) {
                testString += testString;
            }
            testElement.textContent = testString;
        }
    };
    DomCharWidthReader.prototype._readFromDomElements = function () {
        for (var i = 0, len = this._requests.length; i < len; i++) {
            var request = this._requests[i];
            var testElement = this._testElements[i];
            request.fulfill(testElement.offsetWidth / 256);
        }
    };
    return DomCharWidthReader;
}());
function readCharWidths(bareFontInfo, requests) {
    var reader = new DomCharWidthReader(bareFontInfo, requests);
    reader.read();
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$f = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ElementSizeObserver = /** @class */ (function (_super) {
    __extends$f(ElementSizeObserver, _super);
    function ElementSizeObserver(referenceDomElement, changeCallback) {
        var _this = _super.call(this) || this;
        _this.referenceDomElement = referenceDomElement;
        _this.changeCallback = changeCallback;
        _this.measureReferenceDomElementToken = -1;
        _this.width = -1;
        _this.height = -1;
        _this.measureReferenceDomElement(false);
        return _this;
    }
    ElementSizeObserver.prototype.dispose = function () {
        this.stopObserving();
        _super.prototype.dispose.call(this);
    };
    ElementSizeObserver.prototype.getWidth = function () {
        return this.width;
    };
    ElementSizeObserver.prototype.getHeight = function () {
        return this.height;
    };
    ElementSizeObserver.prototype.startObserving = function () {
        var _this = this;
        if (this.measureReferenceDomElementToken === -1) {
            this.measureReferenceDomElementToken = setInterval(function () { return _this.measureReferenceDomElement(true); }, 100);
        }
    };
    ElementSizeObserver.prototype.stopObserving = function () {
        if (this.measureReferenceDomElementToken !== -1) {
            clearInterval(this.measureReferenceDomElementToken);
            this.measureReferenceDomElementToken = -1;
        }
    };
    ElementSizeObserver.prototype.observe = function (dimension) {
        this.measureReferenceDomElement(true, dimension);
    };
    ElementSizeObserver.prototype.measureReferenceDomElement = function (callChangeCallback, dimension) {
        var observedWidth = 0;
        var observedHeight = 0;
        if (dimension) {
            observedWidth = dimension.width;
            observedHeight = dimension.height;
        }
        else if (this.referenceDomElement) {
            observedWidth = this.referenceDomElement.clientWidth;
            observedHeight = this.referenceDomElement.clientHeight;
        }
        observedWidth = Math.max(5, observedWidth);
        observedHeight = Math.max(5, observedHeight);
        if (this.width !== observedWidth || this.height !== observedHeight) {
            this.width = observedWidth;
            this.height = observedHeight;
            if (callChangeCallback) {
                this.changeCallback();
            }
        }
    };
    return ElementSizeObserver;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var EditorZoom = new /** @class */ (function () {
    function class_1() {
        this._zoomLevel = 0;
        this._onDidChangeZoomLevel = new Emitter();
        this.onDidChangeZoomLevel = this._onDidChangeZoomLevel.event;
    }
    class_1.prototype.getZoomLevel = function () {
        return this._zoomLevel;
    };
    class_1.prototype.setZoomLevel = function (zoomLevel) {
        zoomLevel = Math.min(Math.max(-5, zoomLevel), 20);
        if (this._zoomLevel === zoomLevel) {
            return;
        }
        this._zoomLevel = zoomLevel;
        this._onDidChangeZoomLevel.fire(this._zoomLevel);
    };
    return class_1;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$g = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * Determined from empirical observations.
 * @internal
 */
var GOLDEN_LINE_HEIGHT_RATIO = isMacintosh ? 1.5 : 1.35;
/**
 * Font settings maximum and minimum limits
 */
var MINIMUM_FONT_SIZE = 8;
var MAXIMUM_FONT_SIZE = 100;
var MINIMUM_LINE_HEIGHT = 8;
var MAXIMUM_LINE_HEIGHT = 150;
var MINIMUM_LETTER_SPACING = -5;
var MAXIMUM_LETTER_SPACING = 20;
function safeParseFloat(n, defaultValue) {
    if (typeof n === 'number') {
        return n;
    }
    if (typeof n === 'undefined') {
        return defaultValue;
    }
    var r = parseFloat(n);
    if (isNaN(r)) {
        return defaultValue;
    }
    return r;
}
function safeParseInt(n, defaultValue) {
    if (typeof n === 'number') {
        return Math.round(n);
    }
    if (typeof n === 'undefined') {
        return defaultValue;
    }
    var r = parseInt(n);
    if (isNaN(r)) {
        return defaultValue;
    }
    return r;
}
function clamp(n, min, max) {
    if (n < min) {
        return min;
    }
    if (n > max) {
        return max;
    }
    return n;
}
function _string$1(value, defaultValue) {
    if (typeof value !== 'string') {
        return defaultValue;
    }
    return value;
}
var BareFontInfo = /** @class */ (function () {
    /**
     * @internal
     */
    function BareFontInfo(opts) {
        this.zoomLevel = opts.zoomLevel;
        this.fontFamily = String(opts.fontFamily);
        this.fontWeight = String(opts.fontWeight);
        this.fontSize = opts.fontSize;
        this.lineHeight = opts.lineHeight | 0;
        this.letterSpacing = opts.letterSpacing;
    }
    /**
     * @internal
     */
    BareFontInfo.createFromRawSettings = function (opts, zoomLevel) {
        var fontFamily = _string$1(opts.fontFamily, EDITOR_FONT_DEFAULTS.fontFamily);
        var fontWeight = _string$1(opts.fontWeight, EDITOR_FONT_DEFAULTS.fontWeight);
        var fontSize = safeParseFloat(opts.fontSize, EDITOR_FONT_DEFAULTS.fontSize);
        fontSize = clamp(fontSize, 0, MAXIMUM_FONT_SIZE);
        if (fontSize === 0) {
            fontSize = EDITOR_FONT_DEFAULTS.fontSize;
        }
        else if (fontSize < MINIMUM_FONT_SIZE) {
            fontSize = MINIMUM_FONT_SIZE;
        }
        var lineHeight = safeParseInt(opts.lineHeight, 0);
        lineHeight = clamp(lineHeight, 0, MAXIMUM_LINE_HEIGHT);
        if (lineHeight === 0) {
            lineHeight = Math.round(GOLDEN_LINE_HEIGHT_RATIO * fontSize);
        }
        else if (lineHeight < MINIMUM_LINE_HEIGHT) {
            lineHeight = MINIMUM_LINE_HEIGHT;
        }
        var letterSpacing = safeParseFloat(opts.letterSpacing, 0);
        letterSpacing = clamp(letterSpacing, MINIMUM_LETTER_SPACING, MAXIMUM_LETTER_SPACING);
        var editorZoomLevelMultiplier = 1 + (EditorZoom.getZoomLevel() * 0.1);
        fontSize *= editorZoomLevelMultiplier;
        lineHeight *= editorZoomLevelMultiplier;
        return new BareFontInfo({
            zoomLevel: zoomLevel,
            fontFamily: fontFamily,
            fontWeight: fontWeight,
            fontSize: fontSize,
            lineHeight: lineHeight,
            letterSpacing: letterSpacing
        });
    };
    /**
     * @internal
     */
    BareFontInfo.prototype.getId = function () {
        return this.zoomLevel + '-' + this.fontFamily + '-' + this.fontWeight + '-' + this.fontSize + '-' + this.lineHeight + '-' + this.letterSpacing;
    };
    /**
     * @internal
     */
    BareFontInfo.prototype.getMassagedFontFamily = function () {
        if (/[,"']/.test(this.fontFamily)) {
            // Looks like the font family might be already escaped
            return this.fontFamily;
        }
        if (/[+ ]/.test(this.fontFamily)) {
            // Wrap a font family using + or <space> with quotes
            return "\"" + this.fontFamily + "\"";
        }
        return this.fontFamily;
    };
    return BareFontInfo;
}());
var FontInfo = /** @class */ (function (_super) {
    __extends$g(FontInfo, _super);
    /**
     * @internal
     */
    function FontInfo(opts, isTrusted) {
        var _this = _super.call(this, opts) || this;
        _this.isTrusted = isTrusted;
        _this.isMonospace = opts.isMonospace;
        _this.typicalHalfwidthCharacterWidth = opts.typicalHalfwidthCharacterWidth;
        _this.typicalFullwidthCharacterWidth = opts.typicalFullwidthCharacterWidth;
        _this.canUseHalfwidthRightwardsArrow = opts.canUseHalfwidthRightwardsArrow;
        _this.spaceWidth = opts.spaceWidth;
        _this.maxDigitWidth = opts.maxDigitWidth;
        return _this;
    }
    /**
     * @internal
     */
    FontInfo.prototype.equals = function (other) {
        return (this.fontFamily === other.fontFamily
            && this.fontWeight === other.fontWeight
            && this.fontSize === other.fontSize
            && this.lineHeight === other.lineHeight
            && this.letterSpacing === other.letterSpacing
            && this.typicalHalfwidthCharacterWidth === other.typicalHalfwidthCharacterWidth
            && this.typicalFullwidthCharacterWidth === other.typicalFullwidthCharacterWidth
            && this.canUseHalfwidthRightwardsArrow === other.canUseHalfwidthRightwardsArrow
            && this.spaceWidth === other.spaceWidth
            && this.maxDigitWidth === other.maxDigitWidth);
    };
    return FontInfo;
}(BareFontInfo));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Extensions$2 = {
    JSONContribution: 'base.contributions.json'
};
function normalizeId(id) {
    if (id.length > 0 && id.charAt(id.length - 1) === '#') {
        return id.substring(0, id.length - 1);
    }
    return id;
}
var JSONContributionRegistry = /** @class */ (function () {
    function JSONContributionRegistry() {
        this._onDidChangeSchema = new Emitter();
        this.schemasById = {};
    }
    JSONContributionRegistry.prototype.registerSchema = function (uri, unresolvedSchemaContent) {
        this.schemasById[normalizeId(uri)] = unresolvedSchemaContent;
        this._onDidChangeSchema.fire(uri);
    };
    return JSONContributionRegistry;
}());
var jsonContributionRegistry = new JSONContributionRegistry();
Registry.add(Extensions$2.JSONContribution, jsonContributionRegistry);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Extensions$3 = {
    Configuration: 'base.contributions.configuration'
};
var allSettings = { properties: {}, patternProperties: {} };
var applicationSettings = { properties: {}, patternProperties: {} };
var windowSettings = { properties: {}, patternProperties: {} };
var resourceSettings = { properties: {}, patternProperties: {} };
var editorConfigurationSchemaId = 'vscode://schemas/settings/editor';
var contributionRegistry = Registry.as(Extensions$2.JSONContribution);
var ConfigurationRegistry = /** @class */ (function () {
    function ConfigurationRegistry() {
        this.overrideIdentifiers = [];
        this._onDidSchemaChange = new Emitter();
        this._onDidRegisterConfiguration = new Emitter();
        this.configurationContributors = [];
        this.editorConfigurationSchema = { properties: {}, patternProperties: {}, additionalProperties: false, errorMessage: 'Unknown editor configuration setting' };
        this.configurationProperties = {};
        this.excludedConfigurationProperties = {};
        this.computeOverridePropertyPattern();
        contributionRegistry.registerSchema(editorConfigurationSchemaId, this.editorConfigurationSchema);
    }
    ConfigurationRegistry.prototype.registerConfiguration = function (configuration, validate) {
        if (validate === void 0) { validate = true; }
        this.registerConfigurations([configuration], [], validate);
    };
    ConfigurationRegistry.prototype.registerConfigurations = function (configurations, defaultConfigurations, validate) {
        var _this = this;
        if (validate === void 0) { validate = true; }
        var configurationNode = this.toConfiguration(defaultConfigurations);
        if (configurationNode) {
            configurations.push(configurationNode);
        }
        var properties = [];
        configurations.forEach(function (configuration) {
            properties.push.apply(properties, _this.validateAndRegisterProperties(configuration, validate)); // fills in defaults
            _this.configurationContributors.push(configuration);
            _this.registerJSONConfiguration(configuration);
            _this.updateSchemaForOverrideSettingsConfiguration(configuration);
        });
        this._onDidRegisterConfiguration.fire(properties);
    };
    ConfigurationRegistry.prototype.registerOverrideIdentifiers = function (overrideIdentifiers) {
        var _a;
        (_a = this.overrideIdentifiers).push.apply(_a, overrideIdentifiers);
        this.updateOverridePropertyPatternKey();
    };
    ConfigurationRegistry.prototype.toConfiguration = function (defaultConfigurations) {
        var configurationNode = {
            id: 'defaultOverrides',
            title: localize('defaultConfigurations.title', "Default Configuration Overrides"),
            properties: {}
        };
        for (var _i = 0, defaultConfigurations_1 = defaultConfigurations; _i < defaultConfigurations_1.length; _i++) {
            var defaultConfiguration = defaultConfigurations_1[_i];
            for (var key in defaultConfiguration.defaults) {
                var defaultValue = defaultConfiguration.defaults[key];
                if (OVERRIDE_PROPERTY_PATTERN.test(key) && typeof defaultValue === 'object') {
                    configurationNode.properties[key] = {
                        type: 'object',
                        default: defaultValue,
                        description: localize('overrideSettings.description', "Configure editor settings to be overridden for {0} language.", key),
                        $ref: editorConfigurationSchemaId
                    };
                }
            }
        }
        return Object.keys(configurationNode.properties).length ? configurationNode : null;
    };
    ConfigurationRegistry.prototype.validateAndRegisterProperties = function (configuration, validate, scope, overridable) {
        if (validate === void 0) { validate = true; }
        if (scope === void 0) { scope = 2 /* WINDOW */; }
        if (overridable === void 0) { overridable = false; }
        scope = isUndefinedOrNull(configuration.scope) ? scope : configuration.scope;
        overridable = configuration.overridable || overridable;
        var propertyKeys = [];
        var properties = configuration.properties;
        if (properties) {
            for (var key in properties) {
                var message = void 0;
                if (validate && (message = validateProperty(key))) {
                    console.warn(message);
                    delete properties[key];
                    continue;
                }
                // fill in default values
                var property = properties[key];
                var defaultValue = property.default;
                if (isUndefined(defaultValue)) {
                    property.default = getDefaultValue(property.type);
                }
                // Inherit overridable property from parent
                if (overridable) {
                    property.overridable = true;
                }
                if (OVERRIDE_PROPERTY_PATTERN.test(key)) {
                    property.scope = void 0; // No scope for overridable properties `[${identifier}]`
                }
                else {
                    property.scope = isUndefinedOrNull(property.scope) ? scope : property.scope;
                }
                // Add to properties maps
                // Property is included by default if 'included' is unspecified
                if (properties[key].hasOwnProperty('included') && !properties[key].included) {
                    this.excludedConfigurationProperties[key] = properties[key];
                    delete properties[key];
                    continue;
                }
                else {
                    this.configurationProperties[key] = properties[key];
                }
                propertyKeys.push(key);
            }
        }
        var subNodes = configuration.allOf;
        if (subNodes) {
            for (var _i = 0, subNodes_1 = subNodes; _i < subNodes_1.length; _i++) {
                var node = subNodes_1[_i];
                propertyKeys.push.apply(propertyKeys, this.validateAndRegisterProperties(node, validate, scope, overridable));
            }
        }
        return propertyKeys;
    };
    ConfigurationRegistry.prototype.getConfigurationProperties = function () {
        return this.configurationProperties;
    };
    ConfigurationRegistry.prototype.registerJSONConfiguration = function (configuration) {
        function register(configuration) {
            var properties = configuration.properties;
            if (properties) {
                for (var key in properties) {
                    allSettings.properties[key] = properties[key];
                    switch (properties[key].scope) {
                        case 1 /* APPLICATION */:
                            applicationSettings.properties[key] = properties[key];
                            break;
                        case 2 /* WINDOW */:
                            windowSettings.properties[key] = properties[key];
                            break;
                        case 3 /* RESOURCE */:
                            resourceSettings.properties[key] = properties[key];
                            break;
                    }
                }
            }
            var subNodes = configuration.allOf;
            if (subNodes) {
                subNodes.forEach(register);
            }
        }
        register(configuration);
        this._onDidSchemaChange.fire();
    };
    ConfigurationRegistry.prototype.updateSchemaForOverrideSettingsConfiguration = function (configuration) {
        if (configuration.id !== SETTINGS_OVERRRIDE_NODE_ID) {
            this.update(configuration);
            contributionRegistry.registerSchema(editorConfigurationSchemaId, this.editorConfigurationSchema);
        }
    };
    ConfigurationRegistry.prototype.updateOverridePropertyPatternKey = function () {
        var patternProperties = allSettings.patternProperties[this.overridePropertyPattern];
        if (!patternProperties) {
            patternProperties = {
                type: 'object',
                description: localize('overrideSettings.defaultDescription', "Configure editor settings to be overridden for a language."),
                errorMessage: 'Unknown Identifier. Use language identifiers',
                $ref: editorConfigurationSchemaId
            };
        }
        delete allSettings.patternProperties[this.overridePropertyPattern];
        delete applicationSettings.patternProperties[this.overridePropertyPattern];
        delete windowSettings.patternProperties[this.overridePropertyPattern];
        delete resourceSettings.patternProperties[this.overridePropertyPattern];
        this.computeOverridePropertyPattern();
        allSettings.patternProperties[this.overridePropertyPattern] = patternProperties;
        applicationSettings.patternProperties[this.overridePropertyPattern] = patternProperties;
        windowSettings.patternProperties[this.overridePropertyPattern] = patternProperties;
        resourceSettings.patternProperties[this.overridePropertyPattern] = patternProperties;
        this._onDidSchemaChange.fire();
    };
    ConfigurationRegistry.prototype.update = function (configuration) {
        var _this = this;
        var properties = configuration.properties;
        if (properties) {
            for (var key in properties) {
                if (properties[key].overridable) {
                    this.editorConfigurationSchema.properties[key] = this.getConfigurationProperties()[key];
                }
            }
        }
        var subNodes = configuration.allOf;
        if (subNodes) {
            subNodes.forEach(function (subNode) { return _this.update(subNode); });
        }
    };
    ConfigurationRegistry.prototype.computeOverridePropertyPattern = function () {
        this.overridePropertyPattern = this.overrideIdentifiers.length ? OVERRIDE_PATTERN_WITH_SUBSTITUTION.replace('${0}', this.overrideIdentifiers.map(function (identifier) { return createRegExp(identifier, false).source; }).join('|')) : OVERRIDE_PROPERTY;
    };
    return ConfigurationRegistry;
}());
var SETTINGS_OVERRRIDE_NODE_ID = 'override';
var OVERRIDE_PROPERTY = '\\[.*\\]$';
var OVERRIDE_PATTERN_WITH_SUBSTITUTION = '\\[(${0})\\]$';
var OVERRIDE_PROPERTY_PATTERN = new RegExp(OVERRIDE_PROPERTY);
function getDefaultValue(type) {
    var t = Array.isArray(type) ? type[0] : type;
    switch (t) {
        case 'boolean':
            return false;
        case 'integer':
        case 'number':
            return 0;
        case 'string':
            return '';
        case 'array':
            return [];
        case 'object':
            return {};
        default:
            return null;
    }
}
var configurationRegistry = new ConfigurationRegistry();
Registry.add(Extensions$3.Configuration, configurationRegistry);
function validateProperty(property) {
    if (OVERRIDE_PROPERTY_PATTERN.test(property)) {
        return localize('config.property.languageDefault', "Cannot register '{0}'. This matches property pattern '\\\\[.*\\\\]$' for describing language specific editor settings. Use 'configurationDefaults' contribution.", property);
    }
    if (configurationRegistry.getConfigurationProperties()[property] !== void 0) {
        return localize('config.property.duplicate', "Cannot register '{0}'. This property is already registered.", property);
    }
    return null;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$h = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var EDITOR_DEFAULTS$1 = EDITOR_DEFAULTS;
var EDITOR_FONT_DEFAULTS$1 = EDITOR_FONT_DEFAULTS;
var EDITOR_MODEL_DEFAULTS$1 = EDITOR_MODEL_DEFAULTS;
var TabFocus = new /** @class */ (function () {
    function class_1() {
        this._tabFocus = false;
        this._onDidChangeTabFocus = new Emitter();
        this.onDidChangeTabFocus = this._onDidChangeTabFocus.event;
    }
    class_1.prototype.getTabFocusMode = function () {
        return this._tabFocus;
    };
    class_1.prototype.setTabFocusMode = function (tabFocusMode) {
        if (this._tabFocus === tabFocusMode) {
            return;
        }
        this._tabFocus = tabFocusMode;
        this._onDidChangeTabFocus.fire(this._tabFocus);
    };
    return class_1;
}());
var hasOwnProperty$1 = Object.hasOwnProperty;
var CommonEditorConfiguration = /** @class */ (function (_super) {
    __extends$h(CommonEditorConfiguration, _super);
    function CommonEditorConfiguration(options) {
        var _this = _super.call(this) || this;
        _this._onDidChange = _this._register(new Emitter());
        _this.onDidChange = _this._onDidChange.event;
        // Do a "deep clone of sorts" on the incoming options
        _this._rawOptions = mixin({}, options || {});
        _this._rawOptions.scrollbar = mixin({}, _this._rawOptions.scrollbar || {});
        _this._rawOptions.minimap = mixin({}, _this._rawOptions.minimap || {});
        _this._rawOptions.find = mixin({}, _this._rawOptions.find || {});
        _this._rawOptions.hover = mixin({}, _this._rawOptions.hover || {});
        _this._rawOptions.parameterHints = mixin({}, _this._rawOptions.parameterHints || {});
        _this._validatedOptions = EditorOptionsValidator.validate(_this._rawOptions, EDITOR_DEFAULTS$1);
        _this._isDominatedByLongLines = false;
        _this._lineNumbersDigitCount = 1;
        _this._register(EditorZoom.onDidChangeZoomLevel(function (_) { return _this._recomputeOptions(); }));
        _this._register(TabFocus.onDidChangeTabFocus(function (_) { return _this._recomputeOptions(); }));
        return _this;
    }
    CommonEditorConfiguration.prototype.observeReferenceElement = function (dimension) {
    };
    CommonEditorConfiguration.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    CommonEditorConfiguration.prototype._recomputeOptions = function () {
        var oldOptions = this.editor;
        var newOptions = this._computeInternalOptions();
        if (oldOptions && oldOptions.equals(newOptions)) {
            return;
        }
        this.editor = newOptions;
        if (oldOptions) {
            this._onDidChange.fire(oldOptions.createChangeEvent(newOptions));
        }
    };
    CommonEditorConfiguration.prototype.getRawOptions = function () {
        return this._rawOptions;
    };
    CommonEditorConfiguration.prototype._computeInternalOptions = function () {
        var opts = this._validatedOptions;
        var partialEnv = this._getEnvConfiguration();
        var bareFontInfo = BareFontInfo.createFromRawSettings(this._rawOptions, partialEnv.zoomLevel);
        var env = {
            outerWidth: partialEnv.outerWidth,
            outerHeight: partialEnv.outerHeight,
            fontInfo: this.readConfiguration(bareFontInfo),
            extraEditorClassName: partialEnv.extraEditorClassName,
            isDominatedByLongLines: this._isDominatedByLongLines,
            lineNumbersDigitCount: this._lineNumbersDigitCount,
            emptySelectionClipboard: partialEnv.emptySelectionClipboard,
            pixelRatio: partialEnv.pixelRatio,
            tabFocusMode: TabFocus.getTabFocusMode(),
            accessibilitySupport: partialEnv.accessibilitySupport
        };
        return InternalEditorOptionsFactory.createInternalEditorOptions(env, opts);
    };
    CommonEditorConfiguration._primitiveArrayEquals = function (a, b) {
        if (a.length !== b.length) {
            return false;
        }
        for (var i = 0; i < a.length; i++) {
            if (a[i] !== b[i]) {
                return false;
            }
        }
        return true;
    };
    CommonEditorConfiguration._subsetEquals = function (base, subset) {
        for (var key in subset) {
            if (hasOwnProperty$1.call(subset, key)) {
                var subsetValue = subset[key];
                var baseValue = base[key];
                if (baseValue === subsetValue) {
                    continue;
                }
                if (Array.isArray(baseValue) && Array.isArray(subsetValue)) {
                    if (!this._primitiveArrayEquals(baseValue, subsetValue)) {
                        return false;
                    }
                    continue;
                }
                if (typeof baseValue === 'object' && typeof subsetValue === 'object') {
                    if (!this._subsetEquals(baseValue, subsetValue)) {
                        return false;
                    }
                    continue;
                }
                return false;
            }
        }
        return true;
    };
    CommonEditorConfiguration.prototype.updateOptions = function (newOptions) {
        if (typeof newOptions === 'undefined') {
            return;
        }
        if (CommonEditorConfiguration._subsetEquals(this._rawOptions, newOptions)) {
            return;
        }
        this._rawOptions = mixin(this._rawOptions, newOptions || {});
        this._validatedOptions = EditorOptionsValidator.validate(this._rawOptions, EDITOR_DEFAULTS$1);
        this._recomputeOptions();
    };
    CommonEditorConfiguration.prototype.setIsDominatedByLongLines = function (isDominatedByLongLines) {
        this._isDominatedByLongLines = isDominatedByLongLines;
        this._recomputeOptions();
    };
    CommonEditorConfiguration.prototype.setMaxLineNumber = function (maxLineNumber) {
        var digitCount = CommonEditorConfiguration._digitCount(maxLineNumber);
        if (this._lineNumbersDigitCount === digitCount) {
            return;
        }
        this._lineNumbersDigitCount = digitCount;
        this._recomputeOptions();
    };
    CommonEditorConfiguration._digitCount = function (n) {
        var r = 0;
        while (n) {
            n = Math.floor(n / 10);
            r++;
        }
        return r ? r : 1;
    };
    return CommonEditorConfiguration;
}(Disposable));
var configurationRegistry$1 = Registry.as(Extensions$3.Configuration);
var editorConfiguration = {
    'id': 'editor',
    'order': 5,
    'type': 'object',
    'title': localize('editorConfigurationTitle', "Editor"),
    'overridable': true,
    'scope': 3 /* RESOURCE */,
    'properties': {
        'editor.fontFamily': {
            'type': 'string',
            'default': EDITOR_FONT_DEFAULTS$1.fontFamily,
            'description': localize('fontFamily', "Controls the font family.")
        },
        'editor.fontWeight': {
            'type': 'string',
            'enum': ['normal', 'bold', '100', '200', '300', '400', '500', '600', '700', '800', '900'],
            'default': EDITOR_FONT_DEFAULTS$1.fontWeight,
            'description': localize('fontWeight', "Controls the font weight.")
        },
        'editor.fontSize': {
            'type': 'number',
            'default': EDITOR_FONT_DEFAULTS$1.fontSize,
            'description': localize('fontSize', "Controls the font size in pixels.")
        },
        'editor.lineHeight': {
            'type': 'number',
            'default': EDITOR_FONT_DEFAULTS$1.lineHeight,
            'description': localize('lineHeight', "Controls the line height. Use 0 to compute the line height from the font size.")
        },
        'editor.letterSpacing': {
            'type': 'number',
            'default': EDITOR_FONT_DEFAULTS$1.letterSpacing,
            'description': localize('letterSpacing', "Controls the letter spacing in pixels.")
        },
        'editor.lineNumbers': {
            'type': 'string',
            'enum': ['off', 'on', 'relative', 'interval'],
            'enumDescriptions': [
                localize('lineNumbers.off', "Line numbers are not rendered."),
                localize('lineNumbers.on', "Line numbers are rendered as absolute number."),
                localize('lineNumbers.relative', "Line numbers are rendered as distance in lines to cursor position."),
                localize('lineNumbers.interval', "Line numbers are rendered every 10 lines.")
            ],
            'default': 'on',
            'description': localize('lineNumbers', "Controls the display of line numbers.")
        },
        'editor.rulers': {
            'type': 'array',
            'items': {
                'type': 'number'
            },
            'default': EDITOR_DEFAULTS$1.viewInfo.rulers,
            'description': localize('rulers', "Render vertical rulers after a certain number of monospace characters. Use multiple values for multiple rulers. No rulers are drawn if array is empty.")
        },
        'editor.wordSeparators': {
            'type': 'string',
            'default': EDITOR_DEFAULTS$1.wordSeparators,
            'description': localize('wordSeparators', "Characters that will be used as word separators when doing word related navigations or operations.")
        },
        'editor.tabSize': {
            'type': 'number',
            'default': EDITOR_MODEL_DEFAULTS$1.tabSize,
            'minimum': 1,
            'markdownDescription': localize('tabSize', "The number of spaces a tab is equal to. This setting is overridden based on the file contents when `#editor.detectIndentation#` is on.")
        },
        'editor.insertSpaces': {
            'type': 'boolean',
            'default': EDITOR_MODEL_DEFAULTS$1.insertSpaces,
            'markdownDescription': localize('insertSpaces', "Insert spaces when pressing `Tab`. This setting is overridden based on the file contents when `#editor.detectIndentation#` is on.")
        },
        'editor.detectIndentation': {
            'type': 'boolean',
            'default': EDITOR_MODEL_DEFAULTS$1.detectIndentation,
            'markdownDescription': localize('detectIndentation', "Controls whether `#editor.tabSize#` and `#editor.insertSpaces#` will be automatically detected when a file is opened based on the file contents.")
        },
        'editor.roundedSelection': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.roundedSelection,
            'description': localize('roundedSelection', "Controls whether selections should have rounded corners.")
        },
        'editor.scrollBeyondLastLine': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.scrollBeyondLastLine,
            'description': localize('scrollBeyondLastLine', "Controls whether the editor will scroll beyond the last line.")
        },
        'editor.scrollBeyondLastColumn': {
            'type': 'number',
            'default': EDITOR_DEFAULTS$1.viewInfo.scrollBeyondLastColumn,
            'description': localize('scrollBeyondLastColumn', "Controls the number of extra characters beyond which the editor will scroll horizontally.")
        },
        'editor.smoothScrolling': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.smoothScrolling,
            'description': localize('smoothScrolling', "Controls whether the editor will scroll using an animation.")
        },
        'editor.minimap.enabled': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.minimap.enabled,
            'description': localize('minimap.enabled', "Controls whether the minimap is shown.")
        },
        'editor.minimap.side': {
            'type': 'string',
            'enum': ['left', 'right'],
            'default': EDITOR_DEFAULTS$1.viewInfo.minimap.side,
            'description': localize('minimap.side', "Controls the side where to render the minimap.")
        },
        'editor.minimap.showSlider': {
            'type': 'string',
            'enum': ['always', 'mouseover'],
            'default': EDITOR_DEFAULTS$1.viewInfo.minimap.showSlider,
            'description': localize('minimap.showSlider', "Controls whether the minimap slider is automatically hidden.")
        },
        'editor.minimap.renderCharacters': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.minimap.renderCharacters,
            'description': localize('minimap.renderCharacters', "Render the actual characters on a line as opposed to color blocks.")
        },
        'editor.minimap.maxColumn': {
            'type': 'number',
            'default': EDITOR_DEFAULTS$1.viewInfo.minimap.maxColumn,
            'description': localize('minimap.maxColumn', "Limit the width of the minimap to render at most a certain number of columns.")
        },
        'editor.hover.enabled': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.hover.enabled,
            'description': localize('hover.enabled', "Controls whether the hover is shown.")
        },
        'editor.hover.delay': {
            'type': 'number',
            'default': EDITOR_DEFAULTS$1.contribInfo.hover.delay,
            'description': localize('hover.delay', "Controls the delay in milliseconds after which the hover is shown.")
        },
        'editor.hover.sticky': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.hover.sticky,
            'description': localize('hover.sticky', "Controls whether the hover should remain visible when mouse is moved over it.")
        },
        'editor.find.seedSearchStringFromSelection': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.find.seedSearchStringFromSelection,
            'description': localize('find.seedSearchStringFromSelection', "Controls whether the search string in the Find Widget is seeded from the editor selection.")
        },
        'editor.find.autoFindInSelection': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.find.autoFindInSelection,
            'description': localize('find.autoFindInSelection', "Controls whether the find operation is carried out on selected text or the entire file in the editor.")
        },
        'editor.find.globalFindClipboard': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.find.globalFindClipboard,
            'description': localize('find.globalFindClipboard', "Controls whether the Find Widget should read or modify the shared find clipboard on macOS."),
            'included': isMacintosh
        },
        'editor.wordWrap': {
            'type': 'string',
            'enum': ['off', 'on', 'wordWrapColumn', 'bounded'],
            'markdownEnumDescriptions': [
                localize('wordWrap.off', "Lines will never wrap."),
                localize('wordWrap.on', "Lines will wrap at the viewport width."),
                localize({
                    key: 'wordWrap.wordWrapColumn',
                    comment: [
                        '- `editor.wordWrapColumn` refers to a different setting and should not be localized.'
                    ]
                }, "Lines will wrap at `#editor.wordWrapColumn#`."),
                localize({
                    key: 'wordWrap.bounded',
                    comment: [
                        '- viewport means the edge of the visible window size.',
                        '- `editor.wordWrapColumn` refers to a different setting and should not be localized.'
                    ]
                }, "Lines will wrap at the minimum of viewport and `#editor.wordWrapColumn#`."),
            ],
            'default': EDITOR_DEFAULTS$1.wordWrap,
            'description': localize({
                key: 'wordWrap',
                comment: [
                    '- \'off\', \'on\', \'wordWrapColumn\' and \'bounded\' refer to values the setting can take and should not be localized.',
                    '- `editor.wordWrapColumn` refers to a different setting and should not be localized.'
                ]
            }, "Controls how lines should wrap.")
        },
        'editor.wordWrapColumn': {
            'type': 'integer',
            'default': EDITOR_DEFAULTS$1.wordWrapColumn,
            'minimum': 1,
            'markdownDescription': localize({
                key: 'wordWrapColumn',
                comment: [
                    '- `editor.wordWrap` refers to a different setting and should not be localized.',
                    '- \'wordWrapColumn\' and \'bounded\' refer to values the different setting can take and should not be localized.'
                ]
            }, "Controls the wrapping column of the editor when `#editor.wordWrap#` is `wordWrapColumn` or `bounded`.")
        },
        'editor.wrappingIndent': {
            'type': 'string',
            'enum': ['none', 'same', 'indent', 'deepIndent'],
            enumDescriptions: [
                localize('wrappingIndent.none', "No indentation. Wrapped lines begin at column 1."),
                localize('wrappingIndent.same', "Wrapped lines get the same indentation as the parent."),
                localize('wrappingIndent.indent', "Wrapped lines get +1 indentation toward the parent."),
                localize('wrappingIndent.deepIndent', "Wrapped lines get +2 indentation toward the parent."),
            ],
            'default': 'same',
            'description': localize('wrappingIndent', "Controls the indentation of wrapped lines."),
        },
        'editor.mouseWheelScrollSensitivity': {
            'type': 'number',
            'default': EDITOR_DEFAULTS$1.viewInfo.scrollbar.mouseWheelScrollSensitivity,
            'markdownDescription': localize('mouseWheelScrollSensitivity', "A multiplier to be used on the `deltaX` and `deltaY` of mouse wheel scroll events.")
        },
        'editor.multiCursorModifier': {
            'type': 'string',
            'enum': ['ctrlCmd', 'alt'],
            'markdownEnumDescriptions': [
                localize('multiCursorModifier.ctrlCmd', "Maps to `Control` on Windows and Linux and to `Command` on macOS."),
                localize('multiCursorModifier.alt', "Maps to `Alt` on Windows and Linux and to `Option` on macOS.")
            ],
            'default': 'alt',
            'markdownDescription': localize({
                key: 'multiCursorModifier',
                comment: [
                    '- `ctrlCmd` refers to a value the setting can take and should not be localized.',
                    '- `Control` and `Command` refer to the modifier keys Ctrl or Cmd on the keyboard and can be localized.'
                ]
            }, "The modifier to be used to add multiple cursors with the mouse. The Go To Definition and Open Link mouse gestures will adapt such that they do not conflict with the multicursor modifier. [Read more](https://code.visualstudio.com/docs/editor/codebasics#_multicursor-modifier).")
        },
        'editor.multiCursorMergeOverlapping': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.multiCursorMergeOverlapping,
            'description': localize('multiCursorMergeOverlapping', "Merge multiple cursors when they are overlapping.")
        },
        'editor.quickSuggestions': {
            'anyOf': [
                {
                    type: 'boolean',
                },
                {
                    type: 'object',
                    properties: {
                        strings: {
                            type: 'boolean',
                            default: false,
                            description: localize('quickSuggestions.strings', "Enable quick suggestions inside strings.")
                        },
                        comments: {
                            type: 'boolean',
                            default: false,
                            description: localize('quickSuggestions.comments', "Enable quick suggestions inside comments.")
                        },
                        other: {
                            type: 'boolean',
                            default: true,
                            description: localize('quickSuggestions.other', "Enable quick suggestions outside of strings and comments.")
                        },
                    }
                }
            ],
            'default': EDITOR_DEFAULTS$1.contribInfo.quickSuggestions,
            'description': localize('quickSuggestions', "Controls whether suggestions should automatically show up while typing.")
        },
        'editor.quickSuggestionsDelay': {
            'type': 'integer',
            'default': EDITOR_DEFAULTS$1.contribInfo.quickSuggestionsDelay,
            'minimum': 0,
            'description': localize('quickSuggestionsDelay', "Controls the delay in milliseconds after which quick suggestions will show up.")
        },
        'editor.parameterHints.enabled': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.parameterHints.enabled,
            'description': localize('parameterHints.enabled', "Enables a pop-up that shows parameter documentation and type information as you type.")
        },
        'editor.parameterHints.cycle': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.parameterHints.cycle,
            'description': localize('parameterHints.cycle', "Controls whether the parameter hints menu cycles or closes when reaching the end of the list.")
        },
        'editor.autoClosingBrackets': {
            type: 'string',
            enum: ['always', 'languageDefined', 'beforeWhitespace', 'never'],
            enumDescriptions: [
                '',
                localize('editor.autoClosingBrackets.languageDefined', "Use language configurations to determine when to autoclose brackets."),
                localize('editor.autoClosingBrackets.beforeWhitespace', "Autoclose brackets only when the cursor is to the left of whitespace."),
                '',
            ],
            'default': EDITOR_DEFAULTS$1.autoClosingBrackets,
            'description': localize('autoClosingBrackets', "Controls whether the editor should automatically close brackets after the user adds an opening bracket.")
        },
        'editor.autoClosingQuotes': {
            type: 'string',
            enum: ['always', 'languageDefined', 'beforeWhitespace', 'never'],
            enumDescriptions: [
                '',
                localize('editor.autoClosingQuotes.languageDefined', "Use language configurations to determine when to autoclose quotes."),
                localize('editor.autoClosingQuotes.beforeWhitespace', "Autoclose quotes only when the cursor is to the left of whitespace."),
                '',
            ],
            'default': EDITOR_DEFAULTS$1.autoClosingQuotes,
            'description': localize('autoClosingQuotes', "Controls whether the editor should automatically close quotes after the user adds an opening quote.")
        },
        'editor.autoSurround': {
            type: 'string',
            enum: ['languageDefined', 'brackets', 'quotes', 'never'],
            enumDescriptions: [
                localize('editor.autoSurround.languageDefined', "Use language configurations to determine when to automatically surround selections."),
                localize('editor.autoSurround.brackets', "Surround with brackets but not quotes."),
                localize('editor.autoSurround.quotes', "Surround with quotes but not brackets."),
                ''
            ],
            'default': EDITOR_DEFAULTS$1.autoSurround,
            'description': localize('autoSurround', "Controls whether the editor should automatically surround selections.")
        },
        'editor.formatOnType': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.formatOnType,
            'description': localize('formatOnType', "Controls whether the editor should automatically format the line after typing.")
        },
        'editor.formatOnPaste': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.formatOnPaste,
            'description': localize('formatOnPaste', "Controls whether the editor should automatically format the pasted content. A formatter must be available and the formatter should be able to format a range in a document.")
        },
        'editor.autoIndent': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.autoIndent,
            'description': localize('autoIndent', "Controls whether the editor should automatically adjust the indentation when users type, paste or move lines. Extensions with indentation rules of the language must be available.")
        },
        'editor.suggestOnTriggerCharacters': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.suggestOnTriggerCharacters,
            'description': localize('suggestOnTriggerCharacters', "Controls whether suggestions should automatically show up when typing trigger characters.")
        },
        'editor.acceptSuggestionOnEnter': {
            'type': 'string',
            'enum': ['on', 'smart', 'off'],
            'default': EDITOR_DEFAULTS$1.contribInfo.acceptSuggestionOnEnter,
            'markdownEnumDescriptions': [
                '',
                localize('acceptSuggestionOnEnterSmart', "Only accept a suggestion with `Enter` when it makes a textual change."),
                ''
            ],
            'markdownDescription': localize('acceptSuggestionOnEnter', "Controls whether suggestions should be accepted on `Enter`, in addition to `Tab`. Helps to avoid ambiguity between inserting new lines or accepting suggestions.")
        },
        'editor.acceptSuggestionOnCommitCharacter': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.acceptSuggestionOnCommitCharacter,
            'markdownDescription': localize('acceptSuggestionOnCommitCharacter', "Controls whether suggestions should be accepted on commit characters. For example, in JavaScript, the semi-colon (`;`) can be a commit character that accepts a suggestion and types that character.")
        },
        'editor.snippetSuggestions': {
            'type': 'string',
            'enum': ['top', 'bottom', 'inline', 'none'],
            'enumDescriptions': [
                localize('snippetSuggestions.top', "Show snippet suggestions on top of other suggestions."),
                localize('snippetSuggestions.bottom', "Show snippet suggestions below other suggestions."),
                localize('snippetSuggestions.inline', "Show snippets suggestions with other suggestions."),
                localize('snippetSuggestions.none', "Do not show snippet suggestions."),
            ],
            'default': EDITOR_DEFAULTS$1.contribInfo.suggest.snippets,
            'description': localize('snippetSuggestions', "Controls whether snippets are shown with other suggestions and how they are sorted.")
        },
        'editor.emptySelectionClipboard': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.emptySelectionClipboard,
            'description': localize('emptySelectionClipboard', "Controls whether copying without a selection copies the current line.")
        },
        'editor.copyWithSyntaxHighlighting': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.copyWithSyntaxHighlighting,
            'description': localize('copyWithSyntaxHighlighting', "Controls whether syntax highlighting should be copied into the clipboard.")
        },
        'editor.wordBasedSuggestions': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.wordBasedSuggestions,
            'description': localize('wordBasedSuggestions', "Controls whether completions should be computed based on words in the document.")
        },
        'editor.suggestSelection': {
            'type': 'string',
            'enum': ['first', 'recentlyUsed', 'recentlyUsedByPrefix'],
            'markdownEnumDescriptions': [
                localize('suggestSelection.first', "Always select the first suggestion."),
                localize('suggestSelection.recentlyUsed', "Select recent suggestions unless further typing selects one, e.g. `console.| -> console.log` because `log` has been completed recently."),
                localize('suggestSelection.recentlyUsedByPrefix', "Select suggestions based on previous prefixes that have completed those suggestions, e.g. `co -> console` and `con -> const`."),
            ],
            'default': 'recentlyUsed',
            'description': localize('suggestSelection', "Controls how suggestions are pre-selected when showing the suggest list.")
        },
        'editor.suggestFontSize': {
            'type': 'integer',
            'default': 0,
            'minimum': 0,
            'markdownDescription': localize('suggestFontSize', "Font size for the suggest widget. When set to `0`, the value of `#editor.fontSize#` is used.")
        },
        'editor.suggestLineHeight': {
            'type': 'integer',
            'default': 0,
            'minimum': 0,
            'markdownDescription': localize('suggestLineHeight', "Line height for the suggest widget. When set to `0`, the value of `#editor.lineHeight#` is used.")
        },
        'editor.tabCompletion': {
            type: 'string',
            default: 'off',
            enum: ['on', 'off', 'onlySnippets'],
            enumDescriptions: [
                localize('tabCompletion.on', "Tab complete will insert the best matching suggestion when pressing tab."),
                localize('tabCompletion.off', "Disable tab completions."),
                localize('tabCompletion.onlySnippets', "Tab complete snippets when their prefix match. Works best when 'quickSuggestions' aren't enabled."),
            ],
            description: localize('tabCompletion', "Enables tab completions.")
        },
        'editor.suggest.filterGraceful': {
            type: 'boolean',
            default: true,
            description: localize('suggest.filterGraceful', "Controls whether filtering and sorting suggestions accounts for small typos.")
        },
        'editor.suggest.localityBonus': {
            type: 'boolean',
            default: false,
            description: localize('suggest.localityBonus', "Controls whether sorting favours words that appear close to the cursor.")
        },
        'editor.suggest.snippetsPreventQuickSuggestions': {
            type: 'boolean',
            default: true,
            description: localize('suggest.snippetsPreventQuickSuggestions', "Control whether an active snippet prevents quick suggestions.")
        },
        'editor.selectionHighlight': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.selectionHighlight,
            'description': localize('selectionHighlight', "Controls whether the editor should highlight matches similar to the selection")
        },
        'editor.occurrencesHighlight': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.occurrencesHighlight,
            'description': localize('occurrencesHighlight', "Controls whether the editor should highlight semantic symbol occurrences.")
        },
        'editor.overviewRulerLanes': {
            'type': 'integer',
            'default': 3,
            'description': localize('overviewRulerLanes', "Controls the number of decorations that can show up at the same position in the overview ruler.")
        },
        'editor.overviewRulerBorder': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.overviewRulerBorder,
            'description': localize('overviewRulerBorder', "Controls whether a border should be drawn around the overview ruler.")
        },
        'editor.cursorBlinking': {
            'type': 'string',
            'enum': ['blink', 'smooth', 'phase', 'expand', 'solid'],
            'default': blinkingStyleToString(EDITOR_DEFAULTS$1.viewInfo.cursorBlinking),
            'description': localize('cursorBlinking', "Control the cursor animation style.")
        },
        'editor.mouseWheelZoom': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.mouseWheelZoom,
            'markdownDescription': localize('mouseWheelZoom', "Zoom the font of the editor when using mouse wheel and holding `Ctrl`.")
        },
        'editor.cursorStyle': {
            'type': 'string',
            'enum': ['block', 'block-outline', 'line', 'line-thin', 'underline', 'underline-thin'],
            'default': cursorStyleToString(EDITOR_DEFAULTS$1.viewInfo.cursorStyle),
            'description': localize('cursorStyle', "Controls the cursor style.")
        },
        'editor.cursorWidth': {
            'type': 'integer',
            'default': EDITOR_DEFAULTS$1.viewInfo.cursorWidth,
            'markdownDescription': localize('cursorWidth', "Controls the width of the cursor when `#editor.cursorStyle#` is set to `line`.")
        },
        'editor.fontLigatures': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.fontLigatures,
            'description': localize('fontLigatures', "Enables/Disables font ligatures.")
        },
        'editor.hideCursorInOverviewRuler': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.hideCursorInOverviewRuler,
            'description': localize('hideCursorInOverviewRuler', "Controls whether the cursor should be hidden in the overview ruler.")
        },
        'editor.renderWhitespace': {
            'type': 'string',
            'enum': ['none', 'boundary', 'all'],
            'enumDescriptions': [
                '',
                localize('renderWhiteSpace.boundary', "Render whitespace characters except for single spaces between words."),
                ''
            ],
            default: EDITOR_DEFAULTS$1.viewInfo.renderWhitespace,
            description: localize('renderWhitespace', "Controls how the editor should render whitespace characters.")
        },
        'editor.renderControlCharacters': {
            'type': 'boolean',
            default: EDITOR_DEFAULTS$1.viewInfo.renderControlCharacters,
            description: localize('renderControlCharacters', "Controls whether the editor should render control characters.")
        },
        'editor.renderIndentGuides': {
            'type': 'boolean',
            default: EDITOR_DEFAULTS$1.viewInfo.renderIndentGuides,
            description: localize('renderIndentGuides', "Controls whether the editor should render indent guides.")
        },
        'editor.highlightActiveIndentGuide': {
            'type': 'boolean',
            default: EDITOR_DEFAULTS$1.viewInfo.highlightActiveIndentGuide,
            description: localize('highlightActiveIndentGuide', "Controls whether the editor should highlight the active indent guide.")
        },
        'editor.renderLineHighlight': {
            'type': 'string',
            'enum': ['none', 'gutter', 'line', 'all'],
            'enumDescriptions': [
                '',
                '',
                '',
                localize('renderLineHighlight.all', "Highlights both the gutter and the current line."),
            ],
            default: EDITOR_DEFAULTS$1.viewInfo.renderLineHighlight,
            description: localize('renderLineHighlight', "Controls how the editor should render the current line highlight.")
        },
        'editor.codeLens': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.codeLens,
            'description': localize('codeLens', "Controls whether the editor shows CodeLens")
        },
        'editor.folding': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.folding,
            'description': localize('folding', "Controls whether the editor has code folding enabled")
        },
        'editor.foldingStrategy': {
            'type': 'string',
            'enum': ['auto', 'indentation'],
            'default': EDITOR_DEFAULTS$1.contribInfo.foldingStrategy,
            'markdownDescription': localize('foldingStrategy', "Controls the strategy for computing folding ranges. `auto` uses a language specific folding strategy, if available. `indentation` uses the indentation based folding strategy.")
        },
        'editor.showFoldingControls': {
            'type': 'string',
            'enum': ['always', 'mouseover'],
            'default': EDITOR_DEFAULTS$1.contribInfo.showFoldingControls,
            'description': localize('showFoldingControls', "Controls whether the fold controls on the gutter are automatically hidden.")
        },
        'editor.matchBrackets': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.matchBrackets,
            'description': localize('matchBrackets', "Highlight matching brackets when one of them is selected.")
        },
        'editor.glyphMargin': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.viewInfo.glyphMargin,
            'description': localize('glyphMargin', "Controls whether the editor should render the vertical glyph margin. Glyph margin is mostly used for debugging.")
        },
        'editor.useTabStops': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.useTabStops,
            'description': localize('useTabStops', "Inserting and deleting whitespace follows tab stops.")
        },
        'editor.trimAutoWhitespace': {
            'type': 'boolean',
            'default': EDITOR_MODEL_DEFAULTS$1.trimAutoWhitespace,
            'description': localize('trimAutoWhitespace', "Remove trailing auto inserted whitespace.")
        },
        'editor.stablePeek': {
            'type': 'boolean',
            'default': false,
            'markdownDescription': localize('stablePeek', "Keep peek editors open even when double clicking their content or when hitting `Escape`.")
        },
        'editor.dragAndDrop': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.dragAndDrop,
            'description': localize('dragAndDrop', "Controls whether the editor should allow moving selections via drag and drop.")
        },
        'editor.accessibilitySupport': {
            'type': 'string',
            'enum': ['auto', 'on', 'off'],
            'enumDescriptions': [
                localize('accessibilitySupport.auto', "The editor will use platform APIs to detect when a Screen Reader is attached."),
                localize('accessibilitySupport.on', "The editor will be permanently optimized for usage with a Screen Reader."),
                localize('accessibilitySupport.off', "The editor will never be optimized for usage with a Screen Reader."),
            ],
            'default': EDITOR_DEFAULTS$1.accessibilitySupport,
            'description': localize('accessibilitySupport', "Controls whether the editor should run in a mode where it is optimized for screen readers.")
        },
        'editor.showUnused': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.showUnused,
            'description': localize('showUnused', "Controls fading out of unused code.")
        },
        'editor.links': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.links,
            'description': localize('links', "Controls whether the editor should detect links and make them clickable.")
        },
        'editor.colorDecorators': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.colorDecorators,
            'description': localize('colorDecorators', "Controls whether the editor should render the inline color decorators and color picker.")
        },
        'editor.lightbulb.enabled': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.lightbulbEnabled,
            'description': localize('codeActions', "Enables the code action lightbulb in the editor.")
        },
        'editor.codeActionsOnSave': {
            'type': 'object',
            'properties': {
                'source.organizeImports': {
                    'type': 'boolean',
                    'description': localize('codeActionsOnSave.organizeImports', "Controls whether organize imports action should be run on file save.")
                }
            },
            'additionalProperties': {
                'type': 'boolean'
            },
            'default': EDITOR_DEFAULTS$1.contribInfo.codeActionsOnSave,
            'description': localize('codeActionsOnSave', "Code action kinds to be run on save.")
        },
        'editor.codeActionsOnSaveTimeout': {
            'type': 'number',
            'default': EDITOR_DEFAULTS$1.contribInfo.codeActionsOnSaveTimeout,
            'description': localize('codeActionsOnSaveTimeout', "Timeout in milliseconds after which the code actions that are run on save are cancelled.")
        },
        'editor.selectionClipboard': {
            'type': 'boolean',
            'default': EDITOR_DEFAULTS$1.contribInfo.selectionClipboard,
            'description': localize('selectionClipboard', "Controls whether the Linux primary clipboard should be supported."),
            'included': isLinux
        },
        'diffEditor.renderSideBySide': {
            'type': 'boolean',
            'default': true,
            'description': localize('sideBySide', "Controls whether the diff editor shows the diff side by side or inline.")
        },
        'diffEditor.ignoreTrimWhitespace': {
            'type': 'boolean',
            'default': true,
            'description': localize('ignoreTrimWhitespace', "Controls whether the diff editor shows changes in leading or trailing whitespace as diffs.")
        },
        'editor.largeFileOptimizations': {
            'type': 'boolean',
            'default': EDITOR_MODEL_DEFAULTS$1.largeFileOptimizations,
            'description': localize('largeFileOptimizations', "Special handling for large files to disable certain memory intensive features.")
        },
        'diffEditor.renderIndicators': {
            'type': 'boolean',
            'default': true,
            'description': localize('renderIndicators', "Controls whether the diff editor shows +/- indicators for added/removed changes.")
        }
    }
};
var cachedEditorConfigurationKeys = null;
function getEditorConfigurationKeys() {
    if (cachedEditorConfigurationKeys === null) {
        cachedEditorConfigurationKeys = Object.create(null);
        Object.keys(editorConfiguration.properties).forEach(function (prop) {
            cachedEditorConfigurationKeys[prop] = true;
        });
    }
    return cachedEditorConfigurationKeys;
}
function isEditorConfigurationKey(key) {
    var editorConfigurationKeys = getEditorConfigurationKeys();
    return (editorConfigurationKeys["editor." + key] || false);
}
function isDiffEditorConfigurationKey(key) {
    var editorConfigurationKeys = getEditorConfigurationKeys();
    return (editorConfigurationKeys["diffEditor." + key] || false);
}
configurationRegistry$1.registerConfiguration(editorConfiguration);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$i = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CSSBasedConfigurationCache = /** @class */ (function () {
    function CSSBasedConfigurationCache() {
        this._keys = Object.create(null);
        this._values = Object.create(null);
    }
    CSSBasedConfigurationCache.prototype.has = function (item) {
        var itemId = item.getId();
        return !!this._values[itemId];
    };
    CSSBasedConfigurationCache.prototype.get = function (item) {
        var itemId = item.getId();
        return this._values[itemId];
    };
    CSSBasedConfigurationCache.prototype.put = function (item, value) {
        var itemId = item.getId();
        this._keys[itemId] = item;
        this._values[itemId] = value;
    };
    CSSBasedConfigurationCache.prototype.remove = function (item) {
        var itemId = item.getId();
        delete this._keys[itemId];
        delete this._values[itemId];
    };
    CSSBasedConfigurationCache.prototype.getValues = function () {
        var _this = this;
        return Object.keys(this._keys).map(function (id) { return _this._values[id]; });
    };
    return CSSBasedConfigurationCache;
}());
var CSSBasedConfiguration = /** @class */ (function (_super) {
    __extends$i(CSSBasedConfiguration, _super);
    function CSSBasedConfiguration() {
        var _this = _super.call(this) || this;
        _this._onDidChange = _this._register(new Emitter());
        _this.onDidChange = _this._onDidChange.event;
        _this._cache = new CSSBasedConfigurationCache();
        _this._evictUntrustedReadingsTimeout = -1;
        return _this;
    }
    CSSBasedConfiguration.prototype.dispose = function () {
        if (this._evictUntrustedReadingsTimeout !== -1) {
            clearTimeout(this._evictUntrustedReadingsTimeout);
            this._evictUntrustedReadingsTimeout = -1;
        }
        _super.prototype.dispose.call(this);
    };
    CSSBasedConfiguration.prototype._writeToCache = function (item, value) {
        var _this = this;
        this._cache.put(item, value);
        if (!value.isTrusted && this._evictUntrustedReadingsTimeout === -1) {
            // Try reading again after some time
            this._evictUntrustedReadingsTimeout = setTimeout(function () {
                _this._evictUntrustedReadingsTimeout = -1;
                _this._evictUntrustedReadings();
            }, 5000);
        }
    };
    CSSBasedConfiguration.prototype._evictUntrustedReadings = function () {
        var values = this._cache.getValues();
        var somethingRemoved = false;
        for (var i = 0, len = values.length; i < len; i++) {
            var item = values[i];
            if (!item.isTrusted) {
                somethingRemoved = true;
                this._cache.remove(item);
            }
        }
        if (somethingRemoved) {
            this._onDidChange.fire();
        }
    };
    CSSBasedConfiguration.prototype.readConfiguration = function (bareFontInfo) {
        if (!this._cache.has(bareFontInfo)) {
            var readConfig = CSSBasedConfiguration._actualReadConfiguration(bareFontInfo);
            if (readConfig.typicalHalfwidthCharacterWidth <= 2 || readConfig.typicalFullwidthCharacterWidth <= 2 || readConfig.spaceWidth <= 2 || readConfig.maxDigitWidth <= 2) {
                // Hey, it's Bug 14341 ... we couldn't read
                readConfig = new FontInfo({
                    zoomLevel: getZoomLevel(),
                    fontFamily: readConfig.fontFamily,
                    fontWeight: readConfig.fontWeight,
                    fontSize: readConfig.fontSize,
                    lineHeight: readConfig.lineHeight,
                    letterSpacing: readConfig.letterSpacing,
                    isMonospace: readConfig.isMonospace,
                    typicalHalfwidthCharacterWidth: Math.max(readConfig.typicalHalfwidthCharacterWidth, 5),
                    typicalFullwidthCharacterWidth: Math.max(readConfig.typicalFullwidthCharacterWidth, 5),
                    canUseHalfwidthRightwardsArrow: readConfig.canUseHalfwidthRightwardsArrow,
                    spaceWidth: Math.max(readConfig.spaceWidth, 5),
                    maxDigitWidth: Math.max(readConfig.maxDigitWidth, 5),
                }, false);
            }
            this._writeToCache(bareFontInfo, readConfig);
        }
        return this._cache.get(bareFontInfo);
    };
    CSSBasedConfiguration.createRequest = function (chr, type, all, monospace) {
        var result = new CharWidthRequest(chr, type);
        all.push(result);
        if (monospace) {
            monospace.push(result);
        }
        return result;
    };
    CSSBasedConfiguration._actualReadConfiguration = function (bareFontInfo) {
        var all = [];
        var monospace = [];
        var typicalHalfwidthCharacter = this.createRequest('n', 0 /* Regular */, all, monospace);
        var typicalFullwidthCharacter = this.createRequest('\uff4d', 0 /* Regular */, all, null);
        var space = this.createRequest(' ', 0 /* Regular */, all, monospace);
        var digit0 = this.createRequest('0', 0 /* Regular */, all, monospace);
        var digit1 = this.createRequest('1', 0 /* Regular */, all, monospace);
        var digit2 = this.createRequest('2', 0 /* Regular */, all, monospace);
        var digit3 = this.createRequest('3', 0 /* Regular */, all, monospace);
        var digit4 = this.createRequest('4', 0 /* Regular */, all, monospace);
        var digit5 = this.createRequest('5', 0 /* Regular */, all, monospace);
        var digit6 = this.createRequest('6', 0 /* Regular */, all, monospace);
        var digit7 = this.createRequest('7', 0 /* Regular */, all, monospace);
        var digit8 = this.createRequest('8', 0 /* Regular */, all, monospace);
        var digit9 = this.createRequest('9', 0 /* Regular */, all, monospace);
        // monospace test: used for whitespace rendering
        var rightwardsArrow = this.createRequest('→', 0 /* Regular */, all, monospace);
        var halfwidthRightwardsArrow = this.createRequest('￫', 0 /* Regular */, all, null);
        this.createRequest('·', 0 /* Regular */, all, monospace);
        // monospace test: some characters
        this.createRequest('|', 0 /* Regular */, all, monospace);
        this.createRequest('/', 0 /* Regular */, all, monospace);
        this.createRequest('-', 0 /* Regular */, all, monospace);
        this.createRequest('_', 0 /* Regular */, all, monospace);
        this.createRequest('i', 0 /* Regular */, all, monospace);
        this.createRequest('l', 0 /* Regular */, all, monospace);
        this.createRequest('m', 0 /* Regular */, all, monospace);
        // monospace italic test
        this.createRequest('|', 1 /* Italic */, all, monospace);
        this.createRequest('_', 1 /* Italic */, all, monospace);
        this.createRequest('i', 1 /* Italic */, all, monospace);
        this.createRequest('l', 1 /* Italic */, all, monospace);
        this.createRequest('m', 1 /* Italic */, all, monospace);
        this.createRequest('n', 1 /* Italic */, all, monospace);
        // monospace bold test
        this.createRequest('|', 2 /* Bold */, all, monospace);
        this.createRequest('_', 2 /* Bold */, all, monospace);
        this.createRequest('i', 2 /* Bold */, all, monospace);
        this.createRequest('l', 2 /* Bold */, all, monospace);
        this.createRequest('m', 2 /* Bold */, all, monospace);
        this.createRequest('n', 2 /* Bold */, all, monospace);
        readCharWidths(bareFontInfo, all);
        var maxDigitWidth = Math.max(digit0.width, digit1.width, digit2.width, digit3.width, digit4.width, digit5.width, digit6.width, digit7.width, digit8.width, digit9.width);
        var isMonospace = true;
        var referenceWidth = monospace[0].width;
        for (var i = 1, len = monospace.length; i < len; i++) {
            var diff = referenceWidth - monospace[i].width;
            if (diff < -0.001 || diff > 0.001) {
                isMonospace = false;
                break;
            }
        }
        var canUseHalfwidthRightwardsArrow = true;
        if (isMonospace && halfwidthRightwardsArrow.width !== referenceWidth) {
            // using a halfwidth rightwards arrow would break monospace...
            canUseHalfwidthRightwardsArrow = false;
        }
        if (halfwidthRightwardsArrow.width > rightwardsArrow.width) {
            // using a halfwidth rightwards arrow would paint a larger arrow than a regular rightwards arrow
            canUseHalfwidthRightwardsArrow = false;
        }
        // let's trust the zoom level only 2s after it was changed.
        var canTrustBrowserZoomLevel = (getTimeSinceLastZoomLevelChanged() > 2000);
        return new FontInfo({
            zoomLevel: getZoomLevel(),
            fontFamily: bareFontInfo.fontFamily,
            fontWeight: bareFontInfo.fontWeight,
            fontSize: bareFontInfo.fontSize,
            lineHeight: bareFontInfo.lineHeight,
            letterSpacing: bareFontInfo.letterSpacing,
            isMonospace: isMonospace,
            typicalHalfwidthCharacterWidth: typicalHalfwidthCharacter.width,
            typicalFullwidthCharacterWidth: typicalFullwidthCharacter.width,
            canUseHalfwidthRightwardsArrow: canUseHalfwidthRightwardsArrow,
            spaceWidth: space.width,
            maxDigitWidth: maxDigitWidth
        }, canTrustBrowserZoomLevel);
    };
    CSSBasedConfiguration.INSTANCE = new CSSBasedConfiguration();
    return CSSBasedConfiguration;
}(Disposable));
var Configuration = /** @class */ (function (_super) {
    __extends$i(Configuration, _super);
    function Configuration(options, referenceDomElement) {
        if (referenceDomElement === void 0) { referenceDomElement = null; }
        var _this = _super.call(this, options) || this;
        _this._elementSizeObserver = _this._register(new ElementSizeObserver(referenceDomElement, function () { return _this._onReferenceDomElementSizeChanged(); }));
        _this._register(CSSBasedConfiguration.INSTANCE.onDidChange(function () { return _this._onCSSBasedConfigurationChanged(); }));
        if (_this._validatedOptions.automaticLayout) {
            _this._elementSizeObserver.startObserving();
        }
        _this._register(onDidChangeZoomLevel(function (_) { return _this._recomputeOptions(); }));
        _this._register(onDidChangeAccessibilitySupport(function () { return _this._recomputeOptions(); }));
        _this._recomputeOptions();
        return _this;
    }
    Configuration.applyFontInfoSlow = function (domNode, fontInfo) {
        domNode.style.fontFamily = fontInfo.getMassagedFontFamily();
        domNode.style.fontWeight = fontInfo.fontWeight;
        domNode.style.fontSize = fontInfo.fontSize + 'px';
        domNode.style.lineHeight = fontInfo.lineHeight + 'px';
        domNode.style.letterSpacing = fontInfo.letterSpacing + 'px';
    };
    Configuration.applyFontInfo = function (domNode, fontInfo) {
        domNode.setFontFamily(fontInfo.getMassagedFontFamily());
        domNode.setFontWeight(fontInfo.fontWeight);
        domNode.setFontSize(fontInfo.fontSize);
        domNode.setLineHeight(fontInfo.lineHeight);
        domNode.setLetterSpacing(fontInfo.letterSpacing);
    };
    Configuration.prototype._onReferenceDomElementSizeChanged = function () {
        this._recomputeOptions();
    };
    Configuration.prototype._onCSSBasedConfigurationChanged = function () {
        this._recomputeOptions();
    };
    Configuration.prototype.observeReferenceElement = function (dimension) {
        this._elementSizeObserver.observe(dimension);
    };
    Configuration.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    Configuration.prototype._getExtraEditorClassName = function () {
        var extra = '';
        if (isIE) {
            extra += 'ie ';
        }
        else if (isFirefox) {
            extra += 'ff ';
        }
        else if (isEdge) {
            extra += 'edge ';
        }
        else if (isSafari) {
            extra += 'safari ';
        }
        if (isMacintosh) {
            extra += 'mac ';
        }
        return extra;
    };
    Configuration.prototype._getEnvConfiguration = function () {
        return {
            extraEditorClassName: this._getExtraEditorClassName(),
            outerWidth: this._elementSizeObserver.getWidth(),
            outerHeight: this._elementSizeObserver.getHeight(),
            emptySelectionClipboard: isWebKit || isFirefox,
            pixelRatio: getPixelRatio(),
            zoomLevel: getZoomLevel(),
            accessibilitySupport: getAccessibilitySupport()
        };
    };
    Configuration.prototype.readConfiguration = function (bareFontInfo) {
        return CSSBasedConfiguration.INSTANCE.readConfiguration(bareFontInfo);
    };
    return Configuration;
}(CommonEditorConfiguration));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var FastDomNode = /** @class */ (function () {
    function FastDomNode(domNode) {
        this.domNode = domNode;
        this._maxWidth = -1;
        this._width = -1;
        this._height = -1;
        this._top = -1;
        this._left = -1;
        this._bottom = -1;
        this._right = -1;
        this._fontFamily = '';
        this._fontWeight = '';
        this._fontSize = -1;
        this._lineHeight = -1;
        this._letterSpacing = -100;
        this._className = '';
        this._display = '';
        this._position = '';
        this._visibility = '';
        this._layerHint = false;
    }
    FastDomNode.prototype.setMaxWidth = function (maxWidth) {
        if (this._maxWidth === maxWidth) {
            return;
        }
        this._maxWidth = maxWidth;
        this.domNode.style.maxWidth = this._maxWidth + 'px';
    };
    FastDomNode.prototype.setWidth = function (width) {
        if (this._width === width) {
            return;
        }
        this._width = width;
        this.domNode.style.width = this._width + 'px';
    };
    FastDomNode.prototype.setHeight = function (height) {
        if (this._height === height) {
            return;
        }
        this._height = height;
        this.domNode.style.height = this._height + 'px';
    };
    FastDomNode.prototype.setTop = function (top) {
        if (this._top === top) {
            return;
        }
        this._top = top;
        this.domNode.style.top = this._top + 'px';
    };
    FastDomNode.prototype.unsetTop = function () {
        if (this._top === -1) {
            return;
        }
        this._top = -1;
        this.domNode.style.top = '';
    };
    FastDomNode.prototype.setLeft = function (left) {
        if (this._left === left) {
            return;
        }
        this._left = left;
        this.domNode.style.left = this._left + 'px';
    };
    FastDomNode.prototype.setBottom = function (bottom) {
        if (this._bottom === bottom) {
            return;
        }
        this._bottom = bottom;
        this.domNode.style.bottom = this._bottom + 'px';
    };
    FastDomNode.prototype.setRight = function (right) {
        if (this._right === right) {
            return;
        }
        this._right = right;
        this.domNode.style.right = this._right + 'px';
    };
    FastDomNode.prototype.setFontFamily = function (fontFamily) {
        if (this._fontFamily === fontFamily) {
            return;
        }
        this._fontFamily = fontFamily;
        this.domNode.style.fontFamily = this._fontFamily;
    };
    FastDomNode.prototype.setFontWeight = function (fontWeight) {
        if (this._fontWeight === fontWeight) {
            return;
        }
        this._fontWeight = fontWeight;
        this.domNode.style.fontWeight = this._fontWeight;
    };
    FastDomNode.prototype.setFontSize = function (fontSize) {
        if (this._fontSize === fontSize) {
            return;
        }
        this._fontSize = fontSize;
        this.domNode.style.fontSize = this._fontSize + 'px';
    };
    FastDomNode.prototype.setLineHeight = function (lineHeight) {
        if (this._lineHeight === lineHeight) {
            return;
        }
        this._lineHeight = lineHeight;
        this.domNode.style.lineHeight = this._lineHeight + 'px';
    };
    FastDomNode.prototype.setLetterSpacing = function (letterSpacing) {
        if (this._letterSpacing === letterSpacing) {
            return;
        }
        this._letterSpacing = letterSpacing;
        this.domNode.style.letterSpacing = this._letterSpacing + 'px';
    };
    FastDomNode.prototype.setClassName = function (className) {
        if (this._className === className) {
            return;
        }
        this._className = className;
        this.domNode.className = this._className;
    };
    FastDomNode.prototype.toggleClassName = function (className, shouldHaveIt) {
        toggleClass(this.domNode, className, shouldHaveIt);
        this._className = this.domNode.className;
    };
    FastDomNode.prototype.setDisplay = function (display) {
        if (this._display === display) {
            return;
        }
        this._display = display;
        this.domNode.style.display = this._display;
    };
    FastDomNode.prototype.setPosition = function (position) {
        if (this._position === position) {
            return;
        }
        this._position = position;
        this.domNode.style.position = this._position;
    };
    FastDomNode.prototype.setVisibility = function (visibility) {
        if (this._visibility === visibility) {
            return;
        }
        this._visibility = visibility;
        this.domNode.style.visibility = this._visibility;
    };
    FastDomNode.prototype.setLayerHinting = function (layerHint) {
        if (this._layerHint === layerHint) {
            return;
        }
        this._layerHint = layerHint;
        this.domNode.style.willChange = this._layerHint ? 'transform' : 'auto';
    };
    FastDomNode.prototype.setAttribute = function (name, value) {
        this.domNode.setAttribute(name, value);
    };
    FastDomNode.prototype.removeAttribute = function (name) {
        this.domNode.removeAttribute(name);
    };
    FastDomNode.prototype.appendChild = function (child) {
        this.domNode.appendChild(child.domNode);
    };
    FastDomNode.prototype.removeChild = function (child) {
        this.domNode.removeChild(child.domNode);
    };
    return FastDomNode;
}());
function createFastDomNode(domNode) {
    return new FastDomNode(domNode);
}

function memoize(target, key, descriptor) {
    var fnKey = null;
    var fn = null;
    if (typeof descriptor.value === 'function') {
        fnKey = 'value';
        fn = descriptor.value;
        if (fn.length !== 0) {
            console.warn('Memoize should only be used in functions with zero parameters');
        }
    }
    else if (typeof descriptor.get === 'function') {
        fnKey = 'get';
        fn = descriptor.get;
    }
    if (!fn) {
        throw new Error('not supported');
    }
    var memoizeKey = "$memoize$" + key;
    descriptor[fnKey] = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!this.hasOwnProperty(memoizeKey)) {
            Object.defineProperty(this, memoizeKey, {
                configurable: false,
                enumerable: false,
                writable: false,
                value: fn.apply(this, args)
            });
        }
        return this[memoizeKey];
    };
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$j = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var EventType$1;
(function (EventType$$1) {
    EventType$$1.Tap = '-monaco-gesturetap';
    EventType$$1.Change = '-monaco-gesturechange';
    EventType$$1.Start = '-monaco-gesturestart';
    EventType$$1.End = '-monaco-gesturesend';
    EventType$$1.Contextmenu = '-monaco-gesturecontextmenu';
})(EventType$1 || (EventType$1 = {}));
var Gesture = /** @class */ (function (_super) {
    __extends$j(Gesture, _super);
    function Gesture() {
        var _this = _super.call(this) || this;
        _this.activeTouches = {};
        _this.handle = null;
        _this.targets = [];
        _this._register(addDisposableListener(document, 'touchstart', function (e) { return _this.onTouchStart(e); }));
        _this._register(addDisposableListener(document, 'touchend', function (e) { return _this.onTouchEnd(e); }));
        _this._register(addDisposableListener(document, 'touchmove', function (e) { return _this.onTouchMove(e); }));
        return _this;
    }
    Gesture.addTarget = function (element) {
        if (!Gesture.isTouchDevice()) {
            return;
        }
        if (!Gesture.INSTANCE) {
            Gesture.INSTANCE = new Gesture();
        }
        Gesture.INSTANCE.targets.push(element);
    };
    Gesture.isTouchDevice = function () {
        return 'ontouchstart' in window || navigator.maxTouchPoints > 0 || window.navigator.msMaxTouchPoints > 0;
    };
    Gesture.prototype.dispose = function () {
        if (this.handle) {
            this.handle.dispose();
            this.handle = null;
        }
        _super.prototype.dispose.call(this);
    };
    Gesture.prototype.onTouchStart = function (e) {
        var timestamp = Date.now(); // use Date.now() because on FF e.timeStamp is not epoch based.
        if (this.handle) {
            this.handle.dispose();
            this.handle = null;
        }
        for (var i = 0, len = e.targetTouches.length; i < len; i++) {
            var touch = e.targetTouches.item(i);
            this.activeTouches[touch.identifier] = {
                id: touch.identifier,
                initialTarget: touch.target,
                initialTimeStamp: timestamp,
                initialPageX: touch.pageX,
                initialPageY: touch.pageY,
                rollingTimestamps: [timestamp],
                rollingPageX: [touch.pageX],
                rollingPageY: [touch.pageY]
            };
            var evt = this.newGestureEvent(EventType$1.Start, touch.target);
            evt.pageX = touch.pageX;
            evt.pageY = touch.pageY;
            this.dispatchEvent(evt);
        }
        if (this.dispatched) {
            e.preventDefault();
            e.stopPropagation();
            this.dispatched = false;
        }
    };
    Gesture.prototype.onTouchEnd = function (e) {
        var timestamp = Date.now(); // use Date.now() because on FF e.timeStamp is not epoch based.
        var activeTouchCount = Object.keys(this.activeTouches).length;
        var _loop_1 = function (i, len) {
            var touch = e.changedTouches.item(i);
            if (!this_1.activeTouches.hasOwnProperty(String(touch.identifier))) {
                console.warn('move of an UNKNOWN touch', touch);
                return "continue";
            }
            var data = this_1.activeTouches[touch.identifier], holdTime = Date.now() - data.initialTimeStamp;
            if (holdTime < Gesture.HOLD_DELAY
                && Math.abs(data.initialPageX - tail(data.rollingPageX)) < 30
                && Math.abs(data.initialPageY - tail(data.rollingPageY)) < 30) {
                var evt = this_1.newGestureEvent(EventType$1.Tap, data.initialTarget);
                evt.pageX = tail(data.rollingPageX);
                evt.pageY = tail(data.rollingPageY);
                this_1.dispatchEvent(evt);
            }
            else if (holdTime >= Gesture.HOLD_DELAY
                && Math.abs(data.initialPageX - tail(data.rollingPageX)) < 30
                && Math.abs(data.initialPageY - tail(data.rollingPageY)) < 30) {
                var evt = this_1.newGestureEvent(EventType$1.Contextmenu, data.initialTarget);
                evt.pageX = tail(data.rollingPageX);
                evt.pageY = tail(data.rollingPageY);
                this_1.dispatchEvent(evt);
            }
            else if (activeTouchCount === 1) {
                var finalX = tail(data.rollingPageX);
                var finalY = tail(data.rollingPageY);
                var deltaT = tail(data.rollingTimestamps) - data.rollingTimestamps[0];
                var deltaX = finalX - data.rollingPageX[0];
                var deltaY = finalY - data.rollingPageY[0];
                // We need to get all the dispatch targets on the start of the inertia event
                var dispatchTo = this_1.targets.filter(function (t) { return data.initialTarget instanceof Node && t.contains(data.initialTarget); });
                this_1.inertia(dispatchTo, timestamp, // time now
                Math.abs(deltaX) / deltaT, // speed
                deltaX > 0 ? 1 : -1, // x direction
                finalX, // x now
                Math.abs(deltaY) / deltaT, // y speed
                deltaY > 0 ? 1 : -1, // y direction
                finalY // y now
                );
            }
            this_1.dispatchEvent(this_1.newGestureEvent(EventType$1.End, data.initialTarget));
            // forget about this touch
            delete this_1.activeTouches[touch.identifier];
        };
        var this_1 = this;
        for (var i = 0, len = e.changedTouches.length; i < len; i++) {
            _loop_1(i, len);
        }
        if (this.dispatched) {
            e.preventDefault();
            e.stopPropagation();
            this.dispatched = false;
        }
    };
    Gesture.prototype.newGestureEvent = function (type, intialTarget) {
        var event = document.createEvent('CustomEvent');
        event.initEvent(type, false, true);
        event.initialTarget = intialTarget;
        return event;
    };
    Gesture.prototype.dispatchEvent = function (event) {
        var _this = this;
        this.targets.forEach(function (target) {
            if (event.initialTarget instanceof Node && target.contains(event.initialTarget)) {
                target.dispatchEvent(event);
                _this.dispatched = true;
            }
        });
    };
    Gesture.prototype.inertia = function (dispatchTo, t1, vX, dirX, x, vY, dirY, y) {
        var _this = this;
        this.handle = scheduleAtNextAnimationFrame(function () {
            var now = Date.now();
            // velocity: old speed + accel_over_time
            var deltaT = now - t1, delta_pos_x = 0, delta_pos_y = 0, stopped = true;
            vX += Gesture.SCROLL_FRICTION * deltaT;
            vY += Gesture.SCROLL_FRICTION * deltaT;
            if (vX > 0) {
                stopped = false;
                delta_pos_x = dirX * vX * deltaT;
            }
            if (vY > 0) {
                stopped = false;
                delta_pos_y = dirY * vY * deltaT;
            }
            // dispatch translation event
            var evt = _this.newGestureEvent(EventType$1.Change);
            evt.translationX = delta_pos_x;
            evt.translationY = delta_pos_y;
            dispatchTo.forEach(function (d) { return d.dispatchEvent(evt); });
            if (!stopped) {
                _this.inertia(dispatchTo, now, vX, dirX, x + delta_pos_x, vY, dirY, y + delta_pos_y);
            }
        });
    };
    Gesture.prototype.onTouchMove = function (e) {
        var timestamp = Date.now(); // use Date.now() because on FF e.timeStamp is not epoch based.
        for (var i = 0, len = e.changedTouches.length; i < len; i++) {
            var touch = e.changedTouches.item(i);
            if (!this.activeTouches.hasOwnProperty(String(touch.identifier))) {
                console.warn('end of an UNKNOWN touch', touch);
                continue;
            }
            var data = this.activeTouches[touch.identifier];
            var evt = this.newGestureEvent(EventType$1.Change, data.initialTarget);
            evt.translationX = touch.pageX - tail(data.rollingPageX);
            evt.translationY = touch.pageY - tail(data.rollingPageY);
            evt.pageX = touch.pageX;
            evt.pageY = touch.pageY;
            this.dispatchEvent(evt);
            // only keep a few data points, to average the final speed
            if (data.rollingPageX.length > 3) {
                data.rollingPageX.shift();
                data.rollingPageY.shift();
                data.rollingTimestamps.shift();
            }
            data.rollingPageX.push(touch.pageX);
            data.rollingPageY.push(touch.pageY);
            data.rollingTimestamps.push(timestamp);
        }
        if (this.dispatched) {
            e.preventDefault();
            e.stopPropagation();
            this.dispatched = false;
        }
    };
    Gesture.SCROLL_FRICTION = -0.005;
    Gesture.HOLD_DELAY = 700;
    __decorate$1([
        memoize
    ], Gesture, "isTouchDevice", null);
    return Gesture;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$k = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function standardMouseMoveMerger(lastEvent, currentEvent) {
    var ev = new StandardMouseEvent(currentEvent);
    ev.preventDefault();
    return {
        leftButton: ev.leftButton,
        posx: ev.posx,
        posy: ev.posy
    };
}
var GlobalMouseMoveMonitor = /** @class */ (function (_super) {
    __extends$k(GlobalMouseMoveMonitor, _super);
    function GlobalMouseMoveMonitor() {
        var _this = _super.call(this) || this;
        _this.hooks = [];
        _this.mouseMoveEventMerger = null;
        _this.mouseMoveCallback = null;
        _this.onStopCallback = null;
        return _this;
    }
    GlobalMouseMoveMonitor.prototype.dispose = function () {
        this.stopMonitoring(false);
        _super.prototype.dispose.call(this);
    };
    GlobalMouseMoveMonitor.prototype.stopMonitoring = function (invokeStopCallback) {
        if (!this.isMonitoring()) {
            // Not monitoring
            return;
        }
        // Unhook
        this.hooks = dispose(this.hooks);
        this.mouseMoveEventMerger = null;
        this.mouseMoveCallback = null;
        var onStopCallback = this.onStopCallback;
        this.onStopCallback = null;
        if (invokeStopCallback && onStopCallback) {
            onStopCallback();
        }
    };
    GlobalMouseMoveMonitor.prototype.isMonitoring = function () {
        return this.hooks.length > 0;
    };
    GlobalMouseMoveMonitor.prototype.startMonitoring = function (mouseMoveEventMerger, mouseMoveCallback, onStopCallback) {
        var _this = this;
        if (this.isMonitoring()) {
            // I am already hooked
            return;
        }
        this.mouseMoveEventMerger = mouseMoveEventMerger;
        this.mouseMoveCallback = mouseMoveCallback;
        this.onStopCallback = onStopCallback;
        var windowChain = IframeUtils.getSameOriginWindowChain();
        for (var i = 0; i < windowChain.length; i++) {
            this.hooks.push(addDisposableThrottledListener(windowChain[i].window.document, 'mousemove', function (data) { return _this.mouseMoveCallback(data); }, function (lastEvent, currentEvent) { return _this.mouseMoveEventMerger(lastEvent, currentEvent); }));
            this.hooks.push(addDisposableListener(windowChain[i].window.document, 'mouseup', function (e) { return _this.stopMonitoring(true); }));
        }
        if (IframeUtils.hasDifferentOriginAncestor()) {
            var lastSameOriginAncestor = windowChain[windowChain.length - 1];
            // We might miss a mouse up if it happens outside the iframe
            // This one is for Chrome
            this.hooks.push(addDisposableListener(lastSameOriginAncestor.window.document, 'mouseout', function (browserEvent) {
                var e = new StandardMouseEvent(browserEvent);
                if (e.target.tagName.toLowerCase() === 'html') {
                    _this.stopMonitoring(true);
                }
            }));
            // This one is for FF
            this.hooks.push(addDisposableListener(lastSameOriginAncestor.window.document, 'mouseover', function (browserEvent) {
                var e = new StandardMouseEvent(browserEvent);
                if (e.target.tagName.toLowerCase() === 'html') {
                    _this.stopMonitoring(true);
                }
            }));
            // This one is for IE
            this.hooks.push(addDisposableListener(lastSameOriginAncestor.window.document.body, 'mouseleave', function (browserEvent) {
                _this.stopMonitoring(true);
            }));
        }
    };
    return GlobalMouseMoveMonitor;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$l = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * Coordinates relative to the whole document (e.g. mouse event's pageX and pageY)
 */
var PageCoordinates = /** @class */ (function () {
    function PageCoordinates(x, y) {
        this.x = x;
        this.y = y;
    }
    PageCoordinates.prototype.toClientCoordinates = function () {
        return new ClientCoordinates(this.x - StandardWindow.scrollX, this.y - StandardWindow.scrollY);
    };
    return PageCoordinates;
}());
/**
 * Coordinates within the application's client area (i.e. origin is document's scroll position).
 *
 * For example, clicking in the top-left corner of the client area will
 * always result in a mouse event with a client.x value of 0, regardless
 * of whether the page is scrolled horizontally.
 */
var ClientCoordinates = /** @class */ (function () {
    function ClientCoordinates(clientX, clientY) {
        this.clientX = clientX;
        this.clientY = clientY;
    }
    ClientCoordinates.prototype.toPageCoordinates = function () {
        return new PageCoordinates(this.clientX + StandardWindow.scrollX, this.clientY + StandardWindow.scrollY);
    };
    return ClientCoordinates;
}());
/**
 * The position of the editor in the page.
 */
var EditorPagePosition = /** @class */ (function () {
    function EditorPagePosition(x, y, width, height) {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    }
    return EditorPagePosition;
}());
function createEditorPagePosition(editorViewDomNode) {
    var editorPos = getDomNodePagePosition(editorViewDomNode);
    return new EditorPagePosition(editorPos.left, editorPos.top, editorPos.width, editorPos.height);
}
var EditorMouseEvent = /** @class */ (function (_super) {
    __extends$l(EditorMouseEvent, _super);
    function EditorMouseEvent(e, editorViewDomNode) {
        var _this = _super.call(this, e) || this;
        _this.pos = new PageCoordinates(_this.posx, _this.posy);
        _this.editorPos = createEditorPagePosition(editorViewDomNode);
        return _this;
    }
    return EditorMouseEvent;
}(StandardMouseEvent));
var EditorMouseEventFactory = /** @class */ (function () {
    function EditorMouseEventFactory(editorViewDomNode) {
        this._editorViewDomNode = editorViewDomNode;
    }
    EditorMouseEventFactory.prototype._create = function (e) {
        return new EditorMouseEvent(e, this._editorViewDomNode);
    };
    EditorMouseEventFactory.prototype.onContextMenu = function (target, callback) {
        var _this = this;
        return addDisposableListener(target, 'contextmenu', function (e) {
            callback(_this._create(e));
        });
    };
    EditorMouseEventFactory.prototype.onMouseUp = function (target, callback) {
        var _this = this;
        return addDisposableListener(target, 'mouseup', function (e) {
            callback(_this._create(e));
        });
    };
    EditorMouseEventFactory.prototype.onMouseDown = function (target, callback) {
        var _this = this;
        return addDisposableListener(target, 'mousedown', function (e) {
            callback(_this._create(e));
        });
    };
    EditorMouseEventFactory.prototype.onMouseLeave = function (target, callback) {
        var _this = this;
        return addDisposableNonBubblingMouseOutListener(target, function (e) {
            callback(_this._create(e));
        });
    };
    EditorMouseEventFactory.prototype.onMouseMoveThrottled = function (target, callback, merger, minimumTimeMs) {
        var _this = this;
        var myMerger = function (lastEvent, currentEvent) {
            return merger(lastEvent, _this._create(currentEvent));
        };
        return addDisposableThrottledListener(target, 'mousemove', callback, myMerger, minimumTimeMs);
    };
    return EditorMouseEventFactory;
}());
var GlobalEditorMouseMoveMonitor = /** @class */ (function (_super) {
    __extends$l(GlobalEditorMouseMoveMonitor, _super);
    function GlobalEditorMouseMoveMonitor(editorViewDomNode) {
        var _this = _super.call(this) || this;
        _this._editorViewDomNode = editorViewDomNode;
        _this._globalMouseMoveMonitor = _this._register(new GlobalMouseMoveMonitor());
        _this._keydownListener = null;
        return _this;
    }
    GlobalEditorMouseMoveMonitor.prototype.startMonitoring = function (merger, mouseMoveCallback, onStopCallback) {
        var _this = this;
        // Add a <<capture>> keydown event listener that will cancel the monitoring
        // if something other than a modifier key is pressed
        this._keydownListener = addStandardDisposableListener(document, 'keydown', function (e) {
            var kb = e.toKeybinding();
            if (kb.isModifierKey()) {
                // Allow modifier keys
                return;
            }
            _this._globalMouseMoveMonitor.stopMonitoring(true);
        }, true);
        var myMerger = function (lastEvent, currentEvent) {
            return merger(lastEvent, new EditorMouseEvent(currentEvent, _this._editorViewDomNode));
        };
        this._globalMouseMoveMonitor.startMonitoring(myMerger, mouseMoveCallback, function () {
            _this._keydownListener.dispose();
            onStopCallback();
        });
    };
    return GlobalEditorMouseMoveMonitor;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$m = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewEventHandler = /** @class */ (function (_super) {
    __extends$m(ViewEventHandler, _super);
    function ViewEventHandler() {
        var _this = _super.call(this) || this;
        _this._shouldRender = true;
        return _this;
    }
    ViewEventHandler.prototype.shouldRender = function () {
        return this._shouldRender;
    };
    ViewEventHandler.prototype.forceShouldRender = function () {
        this._shouldRender = true;
    };
    ViewEventHandler.prototype.setShouldRender = function () {
        this._shouldRender = true;
    };
    ViewEventHandler.prototype.onDidRender = function () {
        this._shouldRender = false;
    };
    // --- begin event handlers
    ViewEventHandler.prototype.onConfigurationChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onCursorStateChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onDecorationsChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onFlushed = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onFocusChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onLanguageConfigurationChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onLineMappingChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onLinesChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onLinesDeleted = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onLinesInserted = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onRevealRangeRequest = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onScrollChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onTokensChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onTokensColorsChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onZonesChanged = function (e) {
        return false;
    };
    ViewEventHandler.prototype.onThemeChanged = function (e) {
        return false;
    };
    // --- end event handlers
    ViewEventHandler.prototype.handleEvents = function (events) {
        var shouldRender = false;
        for (var i = 0, len = events.length; i < len; i++) {
            var e = events[i];
            switch (e.type) {
                case 1 /* ViewConfigurationChanged */:
                    if (this.onConfigurationChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 2 /* ViewCursorStateChanged */:
                    if (this.onCursorStateChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 3 /* ViewDecorationsChanged */:
                    if (this.onDecorationsChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 4 /* ViewFlushed */:
                    if (this.onFlushed(e)) {
                        shouldRender = true;
                    }
                    break;
                case 5 /* ViewFocusChanged */:
                    if (this.onFocusChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 16 /* ViewLanguageConfigurationChanged */:
                    if (this.onLanguageConfigurationChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 6 /* ViewLineMappingChanged */:
                    if (this.onLineMappingChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 7 /* ViewLinesChanged */:
                    if (this.onLinesChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 8 /* ViewLinesDeleted */:
                    if (this.onLinesDeleted(e)) {
                        shouldRender = true;
                    }
                    break;
                case 9 /* ViewLinesInserted */:
                    if (this.onLinesInserted(e)) {
                        shouldRender = true;
                    }
                    break;
                case 10 /* ViewRevealRangeRequest */:
                    if (this.onRevealRangeRequest(e)) {
                        shouldRender = true;
                    }
                    break;
                case 11 /* ViewScrollChanged */:
                    if (this.onScrollChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 12 /* ViewTokensChanged */:
                    if (this.onTokensChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 13 /* ViewTokensColorsChanged */:
                    if (this.onTokensColorsChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 14 /* ViewZonesChanged */:
                    if (this.onZonesChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                case 15 /* ViewThemeChanged */:
                    if (this.onThemeChanged(e)) {
                        shouldRender = true;
                    }
                    break;
                default:
                    console.info('View received unknown event: ');
                    console.info(e);
            }
        }
        if (shouldRender) {
            this._shouldRender = true;
        }
    };
    return ViewEventHandler;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$n = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewPart = /** @class */ (function (_super) {
    __extends$n(ViewPart, _super);
    function ViewPart(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._context.addEventHandler(_this);
        return _this;
    }
    ViewPart.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        _super.prototype.dispose.call(this);
    };
    return ViewPart;
}(ViewEventHandler));
var PartFingerprints = /** @class */ (function () {
    function PartFingerprints() {
    }
    PartFingerprints.write = function (target, partId) {
        if (target instanceof FastDomNode) {
            target.setAttribute('data-mprt', String(partId));
        }
        else {
            target.setAttribute('data-mprt', String(partId));
        }
    };
    PartFingerprints.read = function (target) {
        var r = target.getAttribute('data-mprt');
        if (r === null) {
            return 0 /* None */;
        }
        return parseInt(r, 10);
    };
    PartFingerprints.collect = function (child, stopAt) {
        var result = [], resultLen = 0;
        while (child && child !== document.body) {
            if (child === stopAt) {
                break;
            }
            if (child.nodeType === child.ELEMENT_NODE) {
                result[resultLen++] = this.read(child);
            }
            child = child.parentElement;
        }
        var r = new Uint8Array(resultLen);
        for (var i = 0; i < resultLen; i++) {
            r[i] = result[resultLen - i - 1];
        }
        return r;
    };
    return PartFingerprints;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$o = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var RestrictedRenderingContext = /** @class */ (function () {
    function RestrictedRenderingContext(viewLayout, viewportData) {
        this._viewLayout = viewLayout;
        this.viewportData = viewportData;
        this.scrollWidth = this._viewLayout.getScrollWidth();
        this.scrollHeight = this._viewLayout.getScrollHeight();
        this.visibleRange = this.viewportData.visibleRange;
        this.bigNumbersDelta = this.viewportData.bigNumbersDelta;
        var vInfo = this._viewLayout.getCurrentViewport();
        this.scrollTop = vInfo.top;
        this.scrollLeft = vInfo.left;
        this.viewportWidth = vInfo.width;
        this.viewportHeight = vInfo.height;
    }
    RestrictedRenderingContext.prototype.getScrolledTopFromAbsoluteTop = function (absoluteTop) {
        return absoluteTop - this.scrollTop;
    };
    RestrictedRenderingContext.prototype.getVerticalOffsetForLineNumber = function (lineNumber) {
        return this._viewLayout.getVerticalOffsetForLineNumber(lineNumber);
    };
    RestrictedRenderingContext.prototype.getDecorationsInViewport = function () {
        return this.viewportData.getDecorationsInViewport();
    };
    return RestrictedRenderingContext;
}());
var RenderingContext = /** @class */ (function (_super) {
    __extends$o(RenderingContext, _super);
    function RenderingContext(viewLayout, viewportData, viewLines) {
        var _this = _super.call(this, viewLayout, viewportData) || this;
        _this._viewLines = viewLines;
        return _this;
    }
    RenderingContext.prototype.linesVisibleRangesForRange = function (range, includeNewLines) {
        return this._viewLines.linesVisibleRangesForRange(range, includeNewLines);
    };
    RenderingContext.prototype.visibleRangeForPosition = function (position) {
        return this._viewLines.visibleRangeForPosition(position);
    };
    return RenderingContext;
}(RestrictedRenderingContext));
var LineVisibleRanges = /** @class */ (function () {
    function LineVisibleRanges(lineNumber, ranges) {
        this.lineNumber = lineNumber;
        this.ranges = ranges;
    }
    return LineVisibleRanges;
}());
var HorizontalRange = /** @class */ (function () {
    function HorizontalRange(left, width) {
        this.left = Math.round(left);
        this.width = Math.round(width);
    }
    HorizontalRange.prototype.toString = function () {
        return "[" + this.left + "," + this.width + "]";
    };
    return HorizontalRange;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var FloatHorizontalRange = /** @class */ (function () {
    function FloatHorizontalRange(left, width) {
        this.left = left;
        this.width = width;
    }
    FloatHorizontalRange.prototype.toString = function () {
        return "[" + this.left + "," + this.width + "]";
    };
    FloatHorizontalRange.compare = function (a, b) {
        return a.left - b.left;
    };
    return FloatHorizontalRange;
}());
var RangeUtil = /** @class */ (function () {
    function RangeUtil() {
    }
    RangeUtil._createRange = function () {
        if (!this._handyReadyRange) {
            this._handyReadyRange = document.createRange();
        }
        return this._handyReadyRange;
    };
    RangeUtil._detachRange = function (range, endNode) {
        // Move range out of the span node, IE doesn't like having many ranges in
        // the same spot and will act badly for lines containing dashes ('-')
        range.selectNodeContents(endNode);
    };
    RangeUtil._readClientRects = function (startElement, startOffset, endElement, endOffset, endNode) {
        var range = this._createRange();
        try {
            range.setStart(startElement, startOffset);
            range.setEnd(endElement, endOffset);
            return range.getClientRects();
        }
        catch (e) {
            // This is life ...
            return null;
        }
        finally {
            this._detachRange(range, endNode);
        }
    };
    RangeUtil._mergeAdjacentRanges = function (ranges) {
        if (ranges.length === 1) {
            // There is nothing to merge
            return [new HorizontalRange(ranges[0].left, ranges[0].width)];
        }
        ranges.sort(FloatHorizontalRange.compare);
        var result = [], resultLen = 0;
        var prevLeft = ranges[0].left;
        var prevWidth = ranges[0].width;
        for (var i = 1, len = ranges.length; i < len; i++) {
            var range = ranges[i];
            var myLeft = range.left;
            var myWidth = range.width;
            if (prevLeft + prevWidth + 0.9 /* account for browser's rounding errors*/ >= myLeft) {
                prevWidth = Math.max(prevWidth, myLeft + myWidth - prevLeft);
            }
            else {
                result[resultLen++] = new HorizontalRange(prevLeft, prevWidth);
                prevLeft = myLeft;
                prevWidth = myWidth;
            }
        }
        result[resultLen++] = new HorizontalRange(prevLeft, prevWidth);
        return result;
    };
    RangeUtil._createHorizontalRangesFromClientRects = function (clientRects, clientRectDeltaLeft) {
        if (!clientRects || clientRects.length === 0) {
            return null;
        }
        // We go through FloatHorizontalRange because it has been observed in bi-di text
        // that the clientRects are not coming in sorted from the browser
        var result = [];
        for (var i = 0, len = clientRects.length; i < len; i++) {
            var clientRect = clientRects[i];
            result[i] = new FloatHorizontalRange(Math.max(0, clientRect.left - clientRectDeltaLeft), clientRect.width);
        }
        return this._mergeAdjacentRanges(result);
    };
    RangeUtil.readHorizontalRanges = function (domNode, startChildIndex, startOffset, endChildIndex, endOffset, clientRectDeltaLeft, endNode) {
        // Panic check
        var min = 0;
        var max = domNode.children.length - 1;
        if (min > max) {
            return null;
        }
        startChildIndex = Math.min(max, Math.max(min, startChildIndex));
        endChildIndex = Math.min(max, Math.max(min, endChildIndex));
        // If crossing over to a span only to select offset 0, then use the previous span's maximum offset
        // Chrome is buggy and doesn't handle 0 offsets well sometimes.
        if (startChildIndex !== endChildIndex) {
            if (endChildIndex > 0 && endOffset === 0) {
                endChildIndex--;
                endOffset = Number.MAX_VALUE;
            }
        }
        var startElement = domNode.children[startChildIndex].firstChild;
        var endElement = domNode.children[endChildIndex].firstChild;
        if (!startElement || !endElement) {
            // When having an empty <span> (without any text content), try to move to the previous <span>
            if (!startElement && startOffset === 0 && startChildIndex > 0) {
                startElement = domNode.children[startChildIndex - 1].firstChild;
                startOffset = 1073741824 /* MAX_SAFE_SMALL_INTEGER */;
            }
            if (!endElement && endOffset === 0 && endChildIndex > 0) {
                endElement = domNode.children[endChildIndex - 1].firstChild;
                endOffset = 1073741824 /* MAX_SAFE_SMALL_INTEGER */;
            }
        }
        if (!startElement || !endElement) {
            return null;
        }
        startOffset = Math.min(startElement.textContent.length, Math.max(0, startOffset));
        endOffset = Math.min(endElement.textContent.length, Math.max(0, endOffset));
        var clientRects = this._readClientRects(startElement, startOffset, endElement, endOffset, endNode);
        return this._createHorizontalRangesFromClientRects(clientRects, clientRectDeltaLeft);
    };
    return RangeUtil;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var LineDecoration = /** @class */ (function () {
    function LineDecoration(startColumn, endColumn, className, type) {
        this.startColumn = startColumn;
        this.endColumn = endColumn;
        this.className = className;
        this.type = type;
    }
    LineDecoration._equals = function (a, b) {
        return (a.startColumn === b.startColumn
            && a.endColumn === b.endColumn
            && a.className === b.className
            && a.type === b.type);
    };
    LineDecoration.equalsArr = function (a, b) {
        var aLen = a.length;
        var bLen = b.length;
        if (aLen !== bLen) {
            return false;
        }
        for (var i = 0; i < aLen; i++) {
            if (!LineDecoration._equals(a[i], b[i])) {
                return false;
            }
        }
        return true;
    };
    LineDecoration.filter = function (lineDecorations, lineNumber, minLineColumn, maxLineColumn) {
        if (lineDecorations.length === 0) {
            return [];
        }
        var result = [], resultLen = 0;
        for (var i = 0, len = lineDecorations.length; i < len; i++) {
            var d = lineDecorations[i];
            var range = d.range;
            if (range.endLineNumber < lineNumber || range.startLineNumber > lineNumber) {
                // Ignore decorations that sit outside this line
                continue;
            }
            if (range.isEmpty() && (d.type === 0 /* Regular */ || d.type === 3 /* RegularAffectingLetterSpacing */)) {
                // Ignore empty range decorations
                continue;
            }
            var startColumn = (range.startLineNumber === lineNumber ? range.startColumn : minLineColumn);
            var endColumn = (range.endLineNumber === lineNumber ? range.endColumn : maxLineColumn);
            result[resultLen++] = new LineDecoration(startColumn, endColumn, d.inlineClassName, d.type);
        }
        return result;
    };
    LineDecoration.compare = function (a, b) {
        if (a.startColumn === b.startColumn) {
            if (a.endColumn === b.endColumn) {
                if (a.className < b.className) {
                    return -1;
                }
                if (a.className > b.className) {
                    return 1;
                }
                return 0;
            }
            return a.endColumn - b.endColumn;
        }
        return a.startColumn - b.startColumn;
    };
    return LineDecoration;
}());
var DecorationSegment = /** @class */ (function () {
    function DecorationSegment(startOffset, endOffset, className) {
        this.startOffset = startOffset;
        this.endOffset = endOffset;
        this.className = className;
    }
    return DecorationSegment;
}());
var Stack = /** @class */ (function () {
    function Stack() {
        this.stopOffsets = [];
        this.classNames = [];
        this.count = 0;
    }
    Stack.prototype.consumeLowerThan = function (maxStopOffset, nextStartOffset, result) {
        while (this.count > 0 && this.stopOffsets[0] < maxStopOffset) {
            var i = 0;
            // Take all equal stopping offsets
            while (i + 1 < this.count && this.stopOffsets[i] === this.stopOffsets[i + 1]) {
                i++;
            }
            // Basically we are consuming the first i + 1 elements of the stack
            result.push(new DecorationSegment(nextStartOffset, this.stopOffsets[i], this.classNames.join(' ')));
            nextStartOffset = this.stopOffsets[i] + 1;
            // Consume them
            this.stopOffsets.splice(0, i + 1);
            this.classNames.splice(0, i + 1);
            this.count -= (i + 1);
        }
        if (this.count > 0 && nextStartOffset < maxStopOffset) {
            result.push(new DecorationSegment(nextStartOffset, maxStopOffset - 1, this.classNames.join(' ')));
            nextStartOffset = maxStopOffset;
        }
        return nextStartOffset;
    };
    Stack.prototype.insert = function (stopOffset, className) {
        if (this.count === 0 || this.stopOffsets[this.count - 1] <= stopOffset) {
            // Insert at the end
            this.stopOffsets.push(stopOffset);
            this.classNames.push(className);
        }
        else {
            // Find the insertion position for `stopOffset`
            for (var i = 0; i < this.count; i++) {
                if (this.stopOffsets[i] >= stopOffset) {
                    this.stopOffsets.splice(i, 0, stopOffset);
                    this.classNames.splice(i, 0, className);
                    break;
                }
            }
        }
        this.count++;
        return;
    };
    return Stack;
}());
var LineDecorationsNormalizer = /** @class */ (function () {
    function LineDecorationsNormalizer() {
    }
    /**
     * Normalize line decorations. Overlapping decorations will generate multiple segments
     */
    LineDecorationsNormalizer.normalize = function (lineContent, lineDecorations) {
        if (lineDecorations.length === 0) {
            return [];
        }
        var result = [];
        var stack = new Stack();
        var nextStartOffset = 0;
        for (var i = 0, len = lineDecorations.length; i < len; i++) {
            var d = lineDecorations[i];
            var startColumn = d.startColumn;
            var endColumn = d.endColumn;
            var className = d.className;
            // If the position would end up in the middle of a high-low surrogate pair, we move it to before the pair
            if (startColumn > 1) {
                var charCodeBefore = lineContent.charCodeAt(startColumn - 2);
                if (isHighSurrogate(charCodeBefore)) {
                    startColumn--;
                }
            }
            if (endColumn > 1) {
                var charCodeBefore = lineContent.charCodeAt(endColumn - 2);
                if (isHighSurrogate(charCodeBefore)) {
                    endColumn--;
                }
            }
            var currentStartOffset = startColumn - 1;
            var currentEndOffset = endColumn - 2;
            nextStartOffset = stack.consumeLowerThan(currentStartOffset, nextStartOffset, result);
            if (stack.count === 0) {
                nextStartOffset = currentStartOffset;
            }
            stack.insert(currentEndOffset, className);
        }
        stack.consumeLowerThan(1073741824 /* MAX_SAFE_SMALL_INTEGER */, nextStartOffset, result);
        return result;
    };
    return LineDecorationsNormalizer;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var createStringBuilder;
if (typeof TextDecoder !== 'undefined') {
    createStringBuilder = function (capacity) { return new StringBuilder(capacity); };
}
else {
    createStringBuilder = function (capacity) { return new CompatStringBuilder(); };
}
var StringBuilder = /** @class */ (function () {
    function StringBuilder(capacity) {
        this._decoder = new TextDecoder('UTF-16LE');
        this._capacity = capacity | 0;
        this._buffer = new Uint16Array(this._capacity);
        this._completedStrings = null;
        this._bufferLength = 0;
    }
    StringBuilder.prototype.reset = function () {
        this._completedStrings = null;
        this._bufferLength = 0;
    };
    StringBuilder.prototype.build = function () {
        if (this._completedStrings !== null) {
            this._flushBuffer();
            return this._completedStrings.join('');
        }
        return this._buildBuffer();
    };
    StringBuilder.prototype._buildBuffer = function () {
        if (this._bufferLength === 0) {
            return '';
        }
        var view = new Uint16Array(this._buffer.buffer, 0, this._bufferLength);
        return this._decoder.decode(view);
    };
    StringBuilder.prototype._flushBuffer = function () {
        var bufferString = this._buildBuffer();
        this._bufferLength = 0;
        if (this._completedStrings === null) {
            this._completedStrings = [bufferString];
        }
        else {
            this._completedStrings[this._completedStrings.length] = bufferString;
        }
    };
    StringBuilder.prototype.write1 = function (charCode) {
        var remainingSpace = this._capacity - this._bufferLength;
        if (remainingSpace <= 1) {
            if (remainingSpace === 0 || isHighSurrogate(charCode)) {
                this._flushBuffer();
            }
        }
        this._buffer[this._bufferLength++] = charCode;
    };
    StringBuilder.prototype.appendASCII = function (charCode) {
        if (this._bufferLength === this._capacity) {
            // buffer is full
            this._flushBuffer();
        }
        this._buffer[this._bufferLength++] = charCode;
    };
    StringBuilder.prototype.appendASCIIString = function (str) {
        var strLen = str.length;
        if (this._bufferLength + strLen >= this._capacity) {
            // This string does not fit in the remaining buffer space
            this._flushBuffer();
            this._completedStrings[this._completedStrings.length] = str;
            return;
        }
        for (var i = 0; i < strLen; i++) {
            this._buffer[this._bufferLength++] = str.charCodeAt(i);
        }
    };
    return StringBuilder;
}());
var CompatStringBuilder = /** @class */ (function () {
    function CompatStringBuilder() {
        this._pieces = [];
        this._piecesLen = 0;
    }
    CompatStringBuilder.prototype.reset = function () {
        this._pieces = [];
        this._piecesLen = 0;
    };
    CompatStringBuilder.prototype.build = function () {
        return this._pieces.join('');
    };
    CompatStringBuilder.prototype.write1 = function (charCode) {
        this._pieces[this._piecesLen++] = String.fromCharCode(charCode);
    };
    CompatStringBuilder.prototype.appendASCII = function (charCode) {
        this._pieces[this._piecesLen++] = String.fromCharCode(charCode);
    };
    CompatStringBuilder.prototype.appendASCIIString = function (str) {
        this._pieces[this._piecesLen++] = str;
    };
    return CompatStringBuilder;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var LinePart = /** @class */ (function () {
    function LinePart(endIndex, type) {
        this.endIndex = endIndex;
        this.type = type;
    }
    return LinePart;
}());
var RenderLineInput = /** @class */ (function () {
    function RenderLineInput(useMonospaceOptimizations, canUseHalfwidthRightwardsArrow, lineContent, continuesWithWrappedLine, isBasicASCII$$1, containsRTL$$1, fauxIndentLength, lineTokens, lineDecorations, tabSize, spaceWidth, stopRenderingLineAfter, renderWhitespace, renderControlCharacters, fontLigatures) {
        this.useMonospaceOptimizations = useMonospaceOptimizations;
        this.canUseHalfwidthRightwardsArrow = canUseHalfwidthRightwardsArrow;
        this.lineContent = lineContent;
        this.continuesWithWrappedLine = continuesWithWrappedLine;
        this.isBasicASCII = isBasicASCII$$1;
        this.containsRTL = containsRTL$$1;
        this.fauxIndentLength = fauxIndentLength;
        this.lineTokens = lineTokens;
        this.lineDecorations = lineDecorations;
        this.tabSize = tabSize;
        this.spaceWidth = spaceWidth;
        this.stopRenderingLineAfter = stopRenderingLineAfter;
        this.renderWhitespace = (renderWhitespace === 'all'
            ? 2 /* All */
            : renderWhitespace === 'boundary'
                ? 1 /* Boundary */
                : 0 /* None */);
        this.renderControlCharacters = renderControlCharacters;
        this.fontLigatures = fontLigatures;
    }
    RenderLineInput.prototype.equals = function (other) {
        return (this.useMonospaceOptimizations === other.useMonospaceOptimizations
            && this.canUseHalfwidthRightwardsArrow === other.canUseHalfwidthRightwardsArrow
            && this.lineContent === other.lineContent
            && this.continuesWithWrappedLine === other.continuesWithWrappedLine
            && this.isBasicASCII === other.isBasicASCII
            && this.containsRTL === other.containsRTL
            && this.fauxIndentLength === other.fauxIndentLength
            && this.tabSize === other.tabSize
            && this.spaceWidth === other.spaceWidth
            && this.stopRenderingLineAfter === other.stopRenderingLineAfter
            && this.renderWhitespace === other.renderWhitespace
            && this.renderControlCharacters === other.renderControlCharacters
            && this.fontLigatures === other.fontLigatures
            && LineDecoration.equalsArr(this.lineDecorations, other.lineDecorations)
            && this.lineTokens.equals(other.lineTokens));
    };
    return RenderLineInput;
}());
/**
 * Provides a both direction mapping between a line's character and its rendered position.
 */
var CharacterMapping = /** @class */ (function () {
    function CharacterMapping(length, partCount) {
        this.length = length;
        this._data = new Uint32Array(this.length);
        this._absoluteOffsets = new Uint32Array(this.length);
    }
    CharacterMapping.getPartIndex = function (partData) {
        return (partData & 4294901760 /* PART_INDEX_MASK */) >>> 16 /* PART_INDEX_OFFSET */;
    };
    CharacterMapping.getCharIndex = function (partData) {
        return (partData & 65535 /* CHAR_INDEX_MASK */) >>> 0 /* CHAR_INDEX_OFFSET */;
    };
    CharacterMapping.prototype.setPartData = function (charOffset, partIndex, charIndex, partAbsoluteOffset) {
        var partData = ((partIndex << 16 /* PART_INDEX_OFFSET */)
            | (charIndex << 0 /* CHAR_INDEX_OFFSET */)) >>> 0;
        this._data[charOffset] = partData;
        this._absoluteOffsets[charOffset] = partAbsoluteOffset + charIndex;
    };
    CharacterMapping.prototype.getAbsoluteOffsets = function () {
        return this._absoluteOffsets;
    };
    CharacterMapping.prototype.charOffsetToPartData = function (charOffset) {
        if (this.length === 0) {
            return 0;
        }
        if (charOffset < 0) {
            return this._data[0];
        }
        if (charOffset >= this.length) {
            return this._data[this.length - 1];
        }
        return this._data[charOffset];
    };
    CharacterMapping.prototype.partDataToCharOffset = function (partIndex, partLength, charIndex) {
        if (this.length === 0) {
            return 0;
        }
        var searchEntry = ((partIndex << 16 /* PART_INDEX_OFFSET */)
            | (charIndex << 0 /* CHAR_INDEX_OFFSET */)) >>> 0;
        var min = 0;
        var max = this.length - 1;
        while (min + 1 < max) {
            var mid = ((min + max) >>> 1);
            var midEntry = this._data[mid];
            if (midEntry === searchEntry) {
                return mid;
            }
            else if (midEntry > searchEntry) {
                max = mid;
            }
            else {
                min = mid;
            }
        }
        if (min === max) {
            return min;
        }
        var minEntry = this._data[min];
        var maxEntry = this._data[max];
        if (minEntry === searchEntry) {
            return min;
        }
        if (maxEntry === searchEntry) {
            return max;
        }
        var minPartIndex = CharacterMapping.getPartIndex(minEntry);
        var minCharIndex = CharacterMapping.getCharIndex(minEntry);
        var maxPartIndex = CharacterMapping.getPartIndex(maxEntry);
        var maxCharIndex;
        if (minPartIndex !== maxPartIndex) {
            // sitting between parts
            maxCharIndex = partLength;
        }
        else {
            maxCharIndex = CharacterMapping.getCharIndex(maxEntry);
        }
        var minEntryDistance = charIndex - minCharIndex;
        var maxEntryDistance = maxCharIndex - charIndex;
        if (minEntryDistance <= maxEntryDistance) {
            return min;
        }
        return max;
    };
    return CharacterMapping;
}());
var RenderLineOutput = /** @class */ (function () {
    function RenderLineOutput(characterMapping, containsRTL$$1, containsForeignElements) {
        this.characterMapping = characterMapping;
        this.containsRTL = containsRTL$$1;
        this.containsForeignElements = containsForeignElements;
    }
    return RenderLineOutput;
}());
function renderViewLine(input, sb) {
    if (input.lineContent.length === 0) {
        var containsForeignElements = 0 /* None */;
        // This is basically for IE's hit test to work
        var content = '<span><span>\u00a0</span></span>';
        if (input.lineDecorations.length > 0) {
            // This line is empty, but it contains inline decorations
            var classNames = [];
            for (var i = 0, len = input.lineDecorations.length; i < len; i++) {
                var lineDecoration = input.lineDecorations[i];
                if (lineDecoration.type === 1 /* Before */) {
                    classNames.push(input.lineDecorations[i].className);
                    containsForeignElements |= 1 /* Before */;
                }
                if (lineDecoration.type === 2 /* After */) {
                    classNames.push(input.lineDecorations[i].className);
                    containsForeignElements |= 2 /* After */;
                }
            }
            if (containsForeignElements !== 0 /* None */) {
                content = "<span><span class=\"" + classNames.join(' ') + "\"></span></span>";
            }
        }
        sb.appendASCIIString(content);
        return new RenderLineOutput(new CharacterMapping(0, 0), false, containsForeignElements);
    }
    return _renderLine(resolveRenderLineInput(input), sb);
}
var RenderLineOutput2 = /** @class */ (function () {
    function RenderLineOutput2(characterMapping, html, containsRTL$$1, containsForeignElements) {
        this.characterMapping = characterMapping;
        this.html = html;
        this.containsRTL = containsRTL$$1;
        this.containsForeignElements = containsForeignElements;
    }
    return RenderLineOutput2;
}());
function renderViewLine2(input) {
    var sb = createStringBuilder(10000);
    var out = renderViewLine(input, sb);
    return new RenderLineOutput2(out.characterMapping, sb.build(), out.containsRTL, out.containsForeignElements);
}
var ResolvedRenderLineInput = /** @class */ (function () {
    function ResolvedRenderLineInput(fontIsMonospace, canUseHalfwidthRightwardsArrow, lineContent, len, isOverflowing, parts, containsForeignElements, tabSize, containsRTL$$1, spaceWidth, renderWhitespace, renderControlCharacters) {
        this.fontIsMonospace = fontIsMonospace;
        this.canUseHalfwidthRightwardsArrow = canUseHalfwidthRightwardsArrow;
        this.lineContent = lineContent;
        this.len = len;
        this.isOverflowing = isOverflowing;
        this.parts = parts;
        this.containsForeignElements = containsForeignElements;
        this.tabSize = tabSize;
        this.containsRTL = containsRTL$$1;
        this.spaceWidth = spaceWidth;
        this.renderWhitespace = renderWhitespace;
        this.renderControlCharacters = renderControlCharacters;
        //
    }
    return ResolvedRenderLineInput;
}());
function resolveRenderLineInput(input) {
    var useMonospaceOptimizations = input.useMonospaceOptimizations;
    var lineContent = input.lineContent;
    var isOverflowing;
    var len;
    if (input.stopRenderingLineAfter !== -1 && input.stopRenderingLineAfter < lineContent.length) {
        isOverflowing = true;
        len = input.stopRenderingLineAfter;
    }
    else {
        isOverflowing = false;
        len = lineContent.length;
    }
    var tokens = transformAndRemoveOverflowing(input.lineTokens, input.fauxIndentLength, len);
    if (input.renderWhitespace === 2 /* All */ || input.renderWhitespace === 1 /* Boundary */) {
        tokens = _applyRenderWhitespace(lineContent, len, input.continuesWithWrappedLine, tokens, input.fauxIndentLength, input.tabSize, useMonospaceOptimizations, input.renderWhitespace === 1 /* Boundary */);
    }
    var containsForeignElements = 0 /* None */;
    if (input.lineDecorations.length > 0) {
        for (var i = 0, len_1 = input.lineDecorations.length; i < len_1; i++) {
            var lineDecoration = input.lineDecorations[i];
            if (lineDecoration.type === 3 /* RegularAffectingLetterSpacing */) {
                // Pretend there are foreign elements... although not 100% accurate.
                containsForeignElements |= 1 /* Before */;
            }
            else if (lineDecoration.type === 1 /* Before */) {
                containsForeignElements |= 1 /* Before */;
            }
            else if (lineDecoration.type === 2 /* After */) {
                containsForeignElements |= 2 /* After */;
            }
        }
        tokens = _applyInlineDecorations(lineContent, len, tokens, input.lineDecorations);
    }
    if (!input.containsRTL) {
        // We can never split RTL text, as it ruins the rendering
        tokens = splitLargeTokens(lineContent, tokens, !input.isBasicASCII || input.fontLigatures);
    }
    return new ResolvedRenderLineInput(useMonospaceOptimizations, input.canUseHalfwidthRightwardsArrow, lineContent, len, isOverflowing, tokens, containsForeignElements, input.tabSize, input.containsRTL, input.spaceWidth, input.renderWhitespace, input.renderControlCharacters);
}
/**
 * In the rendering phase, characters are always looped until token.endIndex.
 * Ensure that all tokens end before `len` and the last one ends precisely at `len`.
 */
function transformAndRemoveOverflowing(tokens, fauxIndentLength, len) {
    var result = [], resultLen = 0;
    // The faux indent part of the line should have no token type
    if (fauxIndentLength > 0) {
        result[resultLen++] = new LinePart(fauxIndentLength, '');
    }
    for (var tokenIndex = 0, tokensLen = tokens.getCount(); tokenIndex < tokensLen; tokenIndex++) {
        var endIndex = tokens.getEndOffset(tokenIndex);
        if (endIndex <= fauxIndentLength) {
            // The faux indent part of the line should have no token type
            continue;
        }
        var type = tokens.getClassName(tokenIndex);
        if (endIndex >= len) {
            result[resultLen++] = new LinePart(len, type);
            break;
        }
        result[resultLen++] = new LinePart(endIndex, type);
    }
    return result;
}
/**
 * See https://github.com/Microsoft/vscode/issues/6885.
 * It appears that having very large spans causes very slow reading of character positions.
 * So here we try to avoid that.
 */
function splitLargeTokens(lineContent, tokens, onlyAtSpaces) {
    var lastTokenEndIndex = 0;
    var result = [], resultLen = 0;
    if (onlyAtSpaces) {
        // Split only at spaces => we need to walk each character
        for (var i = 0, len = tokens.length; i < len; i++) {
            var token = tokens[i];
            var tokenEndIndex = token.endIndex;
            if (lastTokenEndIndex + 50 /* LongToken */ < tokenEndIndex) {
                var tokenType = token.type;
                var lastSpaceOffset = -1;
                var currTokenStart = lastTokenEndIndex;
                for (var j = lastTokenEndIndex; j < tokenEndIndex; j++) {
                    if (lineContent.charCodeAt(j) === 32 /* Space */) {
                        lastSpaceOffset = j;
                    }
                    if (lastSpaceOffset !== -1 && j - currTokenStart >= 50 /* LongToken */) {
                        // Split at `lastSpaceOffset` + 1
                        result[resultLen++] = new LinePart(lastSpaceOffset + 1, tokenType);
                        currTokenStart = lastSpaceOffset + 1;
                        lastSpaceOffset = -1;
                    }
                }
                if (currTokenStart !== tokenEndIndex) {
                    result[resultLen++] = new LinePart(tokenEndIndex, tokenType);
                }
            }
            else {
                result[resultLen++] = token;
            }
            lastTokenEndIndex = tokenEndIndex;
        }
    }
    else {
        // Split anywhere => we don't need to walk each character
        for (var i = 0, len = tokens.length; i < len; i++) {
            var token = tokens[i];
            var tokenEndIndex = token.endIndex;
            var diff = (tokenEndIndex - lastTokenEndIndex);
            if (diff > 50 /* LongToken */) {
                var tokenType = token.type;
                var piecesCount = Math.ceil(diff / 50 /* LongToken */);
                for (var j = 1; j < piecesCount; j++) {
                    var pieceEndIndex = lastTokenEndIndex + (j * 50 /* LongToken */);
                    result[resultLen++] = new LinePart(pieceEndIndex, tokenType);
                }
                result[resultLen++] = new LinePart(tokenEndIndex, tokenType);
            }
            else {
                result[resultLen++] = token;
            }
            lastTokenEndIndex = tokenEndIndex;
        }
    }
    return result;
}
/**
 * Whitespace is rendered by "replacing" tokens with a special-purpose `vs-whitespace` type that is later recognized in the rendering phase.
 * Moreover, a token is created for every visual indent because on some fonts the glyphs used for rendering whitespace (&rarr; or &middot;) do not have the same width as &nbsp;.
 * The rendering phase will generate `style="width:..."` for these tokens.
 */
function _applyRenderWhitespace(lineContent, len, continuesWithWrappedLine, tokens, fauxIndentLength, tabSize, useMonospaceOptimizations, onlyBoundary) {
    var result = [], resultLen = 0;
    var tokenIndex = 0;
    var tokenType = tokens[tokenIndex].type;
    var tokenEndIndex = tokens[tokenIndex].endIndex;
    var tokensLength = tokens.length;
    var firstNonWhitespaceIndex$$1 = firstNonWhitespaceIndex(lineContent);
    var lastNonWhitespaceIndex$$1;
    if (firstNonWhitespaceIndex$$1 === -1) {
        // The entire line is whitespace
        firstNonWhitespaceIndex$$1 = len;
        lastNonWhitespaceIndex$$1 = len;
    }
    else {
        lastNonWhitespaceIndex$$1 = lastNonWhitespaceIndex(lineContent);
    }
    var tmpIndent = 0;
    for (var charIndex = 0; charIndex < fauxIndentLength; charIndex++) {
        var chCode = lineContent.charCodeAt(charIndex);
        if (chCode === 9 /* Tab */) {
            tmpIndent = tabSize;
        }
        else if (isFullWidthCharacter(chCode)) {
            tmpIndent += 2;
        }
        else {
            tmpIndent++;
        }
    }
    tmpIndent = tmpIndent % tabSize;
    var wasInWhitespace = false;
    for (var charIndex = fauxIndentLength; charIndex < len; charIndex++) {
        var chCode = lineContent.charCodeAt(charIndex);
        var isInWhitespace = void 0;
        if (charIndex < firstNonWhitespaceIndex$$1 || charIndex > lastNonWhitespaceIndex$$1) {
            // in leading or trailing whitespace
            isInWhitespace = true;
        }
        else if (chCode === 9 /* Tab */) {
            // a tab character is rendered both in all and boundary cases
            isInWhitespace = true;
        }
        else if (chCode === 32 /* Space */) {
            // hit a space character
            if (onlyBoundary) {
                // rendering only boundary whitespace
                if (wasInWhitespace) {
                    isInWhitespace = true;
                }
                else {
                    var nextChCode = (charIndex + 1 < len ? lineContent.charCodeAt(charIndex + 1) : 0 /* Null */);
                    isInWhitespace = (nextChCode === 32 /* Space */ || nextChCode === 9 /* Tab */);
                }
            }
            else {
                isInWhitespace = true;
            }
        }
        else {
            isInWhitespace = false;
        }
        if (wasInWhitespace) {
            // was in whitespace token
            if (!isInWhitespace || (!useMonospaceOptimizations && tmpIndent >= tabSize)) {
                // leaving whitespace token or entering a new indent
                result[resultLen++] = new LinePart(charIndex, 'vs-whitespace');
                tmpIndent = tmpIndent % tabSize;
            }
        }
        else {
            // was in regular token
            if (charIndex === tokenEndIndex || (isInWhitespace && charIndex > fauxIndentLength)) {
                result[resultLen++] = new LinePart(charIndex, tokenType);
                tmpIndent = tmpIndent % tabSize;
            }
        }
        if (chCode === 9 /* Tab */) {
            tmpIndent = tabSize;
        }
        else if (isFullWidthCharacter(chCode)) {
            tmpIndent += 2;
        }
        else {
            tmpIndent++;
        }
        wasInWhitespace = isInWhitespace;
        if (charIndex === tokenEndIndex) {
            tokenIndex++;
            if (tokenIndex < tokensLength) {
                tokenType = tokens[tokenIndex].type;
                tokenEndIndex = tokens[tokenIndex].endIndex;
            }
        }
    }
    var generateWhitespace = false;
    if (wasInWhitespace) {
        // was in whitespace token
        if (continuesWithWrappedLine && onlyBoundary) {
            var lastCharCode = (len > 0 ? lineContent.charCodeAt(len - 1) : 0 /* Null */);
            var prevCharCode = (len > 1 ? lineContent.charCodeAt(len - 2) : 0 /* Null */);
            var isSingleTrailingSpace = (lastCharCode === 32 /* Space */ && (prevCharCode !== 32 /* Space */ && prevCharCode !== 9 /* Tab */));
            if (!isSingleTrailingSpace) {
                generateWhitespace = true;
            }
        }
        else {
            generateWhitespace = true;
        }
    }
    result[resultLen++] = new LinePart(len, generateWhitespace ? 'vs-whitespace' : tokenType);
    return result;
}
/**
 * Inline decorations are "merged" on top of tokens.
 * Special care must be taken when multiple inline decorations are at play and they overlap.
 */
function _applyInlineDecorations(lineContent, len, tokens, _lineDecorations) {
    _lineDecorations.sort(LineDecoration.compare);
    var lineDecorations = LineDecorationsNormalizer.normalize(lineContent, _lineDecorations);
    var lineDecorationsLen = lineDecorations.length;
    var lineDecorationIndex = 0;
    var result = [], resultLen = 0, lastResultEndIndex = 0;
    for (var tokenIndex = 0, len_2 = tokens.length; tokenIndex < len_2; tokenIndex++) {
        var token = tokens[tokenIndex];
        var tokenEndIndex = token.endIndex;
        var tokenType = token.type;
        while (lineDecorationIndex < lineDecorationsLen && lineDecorations[lineDecorationIndex].startOffset < tokenEndIndex) {
            var lineDecoration = lineDecorations[lineDecorationIndex];
            if (lineDecoration.startOffset > lastResultEndIndex) {
                lastResultEndIndex = lineDecoration.startOffset;
                result[resultLen++] = new LinePart(lastResultEndIndex, tokenType);
            }
            if (lineDecoration.endOffset + 1 <= tokenEndIndex) {
                // This line decoration ends before this token ends
                lastResultEndIndex = lineDecoration.endOffset + 1;
                result[resultLen++] = new LinePart(lastResultEndIndex, tokenType + ' ' + lineDecoration.className);
                lineDecorationIndex++;
            }
            else {
                // This line decoration continues on to the next token
                lastResultEndIndex = tokenEndIndex;
                result[resultLen++] = new LinePart(lastResultEndIndex, tokenType + ' ' + lineDecoration.className);
                break;
            }
        }
        if (tokenEndIndex > lastResultEndIndex) {
            lastResultEndIndex = tokenEndIndex;
            result[resultLen++] = new LinePart(lastResultEndIndex, tokenType);
        }
    }
    var lastTokenEndIndex = tokens[tokens.length - 1].endIndex;
    if (lineDecorationIndex < lineDecorationsLen && lineDecorations[lineDecorationIndex].startOffset === lastTokenEndIndex) {
        var classNames = [];
        while (lineDecorationIndex < lineDecorationsLen && lineDecorations[lineDecorationIndex].startOffset === lastTokenEndIndex) {
            classNames.push(lineDecorations[lineDecorationIndex].className);
            lineDecorationIndex++;
        }
        result[resultLen++] = new LinePart(lastResultEndIndex, classNames.join(' '));
    }
    return result;
}
/**
 * This function is on purpose not split up into multiple functions to allow runtime type inference (i.e. performance reasons).
 * Notice how all the needed data is fully resolved and passed in (i.e. no other calls).
 */
function _renderLine(input, sb) {
    var fontIsMonospace = input.fontIsMonospace;
    var canUseHalfwidthRightwardsArrow = input.canUseHalfwidthRightwardsArrow;
    var containsForeignElements = input.containsForeignElements;
    var lineContent = input.lineContent;
    var len = input.len;
    var isOverflowing = input.isOverflowing;
    var parts = input.parts;
    var tabSize = input.tabSize;
    var containsRTL$$1 = input.containsRTL;
    var spaceWidth = input.spaceWidth;
    var renderWhitespace = input.renderWhitespace;
    var renderControlCharacters = input.renderControlCharacters;
    var characterMapping = new CharacterMapping(len + 1, parts.length);
    var charIndex = 0;
    var tabsCharDelta = 0;
    var charOffsetInPart = 0;
    var prevPartContentCnt = 0;
    var partAbsoluteOffset = 0;
    sb.appendASCIIString('<span>');
    for (var partIndex = 0, tokensLen = parts.length; partIndex < tokensLen; partIndex++) {
        partAbsoluteOffset += prevPartContentCnt;
        var part = parts[partIndex];
        var partEndIndex = part.endIndex;
        var partType = part.type;
        var partRendersWhitespace = (renderWhitespace !== 0 /* None */ && (partType.indexOf('vs-whitespace') >= 0));
        charOffsetInPart = 0;
        sb.appendASCIIString('<span class="');
        sb.appendASCIIString(partType);
        sb.appendASCII(34 /* DoubleQuote */);
        if (partRendersWhitespace) {
            var partContentCnt = 0;
            {
                var _charIndex = charIndex;
                var _tabsCharDelta = tabsCharDelta;
                for (; _charIndex < partEndIndex; _charIndex++) {
                    var charCode = lineContent.charCodeAt(_charIndex);
                    if (charCode === 9 /* Tab */) {
                        var insertSpacesCount = tabSize - (_charIndex + _tabsCharDelta) % tabSize;
                        _tabsCharDelta += insertSpacesCount - 1;
                        partContentCnt += insertSpacesCount;
                    }
                    else {
                        // must be CharCode.Space
                        partContentCnt++;
                    }
                }
            }
            if (!fontIsMonospace) {
                var partIsOnlyWhitespace = (partType === 'vs-whitespace');
                if (partIsOnlyWhitespace || !containsForeignElements) {
                    sb.appendASCIIString(' style="width:');
                    sb.appendASCIIString(String(spaceWidth * partContentCnt));
                    sb.appendASCIIString('px"');
                }
            }
            sb.appendASCII(62 /* GreaterThan */);
            for (; charIndex < partEndIndex; charIndex++) {
                characterMapping.setPartData(charIndex, partIndex, charOffsetInPart, partAbsoluteOffset);
                var charCode = lineContent.charCodeAt(charIndex);
                if (charCode === 9 /* Tab */) {
                    var insertSpacesCount = tabSize - (charIndex + tabsCharDelta) % tabSize;
                    tabsCharDelta += insertSpacesCount - 1;
                    charOffsetInPart += insertSpacesCount - 1;
                    if (insertSpacesCount > 0) {
                        if (!canUseHalfwidthRightwardsArrow || insertSpacesCount > 1) {
                            sb.write1(0x2192); // RIGHTWARDS ARROW
                        }
                        else {
                            sb.write1(0xffeb); // HALFWIDTH RIGHTWARDS ARROW
                        }
                        insertSpacesCount--;
                    }
                    while (insertSpacesCount > 0) {
                        sb.write1(0xA0); // &nbsp;
                        insertSpacesCount--;
                    }
                }
                else {
                    // must be CharCode.Space
                    sb.write1(0xb7); // &middot;
                }
                charOffsetInPart++;
            }
            prevPartContentCnt = partContentCnt;
        }
        else {
            var partContentCnt = 0;
            if (containsRTL$$1) {
                sb.appendASCIIString(' dir="ltr"');
            }
            sb.appendASCII(62 /* GreaterThan */);
            for (; charIndex < partEndIndex; charIndex++) {
                characterMapping.setPartData(charIndex, partIndex, charOffsetInPart, partAbsoluteOffset);
                var charCode = lineContent.charCodeAt(charIndex);
                switch (charCode) {
                    case 9 /* Tab */:
                        var insertSpacesCount = tabSize - (charIndex + tabsCharDelta) % tabSize;
                        tabsCharDelta += insertSpacesCount - 1;
                        charOffsetInPart += insertSpacesCount - 1;
                        while (insertSpacesCount > 0) {
                            sb.write1(0xA0); // &nbsp;
                            partContentCnt++;
                            insertSpacesCount--;
                        }
                        break;
                    case 32 /* Space */:
                        sb.write1(0xA0); // &nbsp;
                        partContentCnt++;
                        break;
                    case 60 /* LessThan */:
                        sb.appendASCIIString('&lt;');
                        partContentCnt++;
                        break;
                    case 62 /* GreaterThan */:
                        sb.appendASCIIString('&gt;');
                        partContentCnt++;
                        break;
                    case 38 /* Ampersand */:
                        sb.appendASCIIString('&amp;');
                        partContentCnt++;
                        break;
                    case 0 /* Null */:
                        sb.appendASCIIString('&#00;');
                        partContentCnt++;
                        break;
                    case 65279 /* UTF8_BOM */:
                    case 8232 /* LINE_SEPARATOR_2028 */:
                        sb.write1(0xfffd);
                        partContentCnt++;
                        break;
                    default:
                        if (isFullWidthCharacter(charCode)) {
                            tabsCharDelta++;
                        }
                        if (renderControlCharacters && charCode < 32) {
                            sb.write1(9216 + charCode);
                            partContentCnt++;
                        }
                        else {
                            sb.write1(charCode);
                            partContentCnt++;
                        }
                }
                charOffsetInPart++;
            }
            prevPartContentCnt = partContentCnt;
        }
        sb.appendASCIIString('</span>');
    }
    // When getting client rects for the last character, we will position the
    // text range at the end of the span, insteaf of at the beginning of next span
    characterMapping.setPartData(len, parts.length - 1, charOffsetInPart, partAbsoluteOffset);
    if (isOverflowing) {
        sb.appendASCIIString('<span>&hellip;</span>');
    }
    sb.appendASCIIString('</span>');
    return new RenderLineOutput(characterMapping, containsRTL$$1, containsForeignElements);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IThemeService = createDecorator('themeService');
function themeColorFromId(id) {
    return { id: id };
}
// base themes
var DARK = 'dark';
var HIGH_CONTRAST = 'hc';
function getThemeTypeSelector(type) {
    switch (type) {
        case DARK: return 'vs-dark';
        case HIGH_CONTRAST: return 'hc-black';
        default: return 'vs';
    }
}
// static theming participant
var Extensions$4 = {
    ThemingContribution: 'base.contributions.theming'
};
var ThemingRegistry = /** @class */ (function () {
    function ThemingRegistry() {
        this.themingParticipants = [];
        this.themingParticipants = [];
        this.onThemingParticipantAddedEmitter = new Emitter();
    }
    ThemingRegistry.prototype.onThemeChange = function (participant) {
        var _this = this;
        this.themingParticipants.push(participant);
        this.onThemingParticipantAddedEmitter.fire(participant);
        return toDisposable(function () {
            var idx = _this.themingParticipants.indexOf(participant);
            _this.themingParticipants.splice(idx, 1);
        });
    };
    ThemingRegistry.prototype.getThemingParticipants = function () {
        return this.themingParticipants;
    };
    return ThemingRegistry;
}());
var themingRegistry = new ThemingRegistry();
Registry.add(Extensions$4.ThemingContribution, themingRegistry);
function registerThemingParticipant(participant) {
    return themingRegistry.onThemeChange(participant);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$p = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var canUseFastRenderedViewLine = (function () {
    if (isNative) {
        // In VSCode we know very well when the zoom level changes
        return true;
    }
    if (isLinux || isFirefox || isSafari) {
        // On Linux, it appears that zooming affects char widths (in pixels), which is unexpected.
        // --
        // Even though we read character widths correctly, having read them at a specific zoom level
        // does not mean they are the same at the current zoom level.
        // --
        // This could be improved if we ever figure out how to get an event when browsers zoom,
        // but until then we have to stick with reading client rects.
        // --
        // The same has been observed with Firefox on Windows7
        // --
        // The same has been oversved with Safari
        return false;
    }
    return true;
})();
var alwaysRenderInlineSelection = (isEdgeOrIE);
var DomReadingContext = /** @class */ (function () {
    function DomReadingContext(domNode, endNode) {
        this._domNode = domNode;
        this._clientRectDeltaLeft = 0;
        this._clientRectDeltaLeftRead = false;
        this.endNode = endNode;
    }
    Object.defineProperty(DomReadingContext.prototype, "clientRectDeltaLeft", {
        get: function () {
            if (!this._clientRectDeltaLeftRead) {
                this._clientRectDeltaLeftRead = true;
                this._clientRectDeltaLeft = this._domNode.getBoundingClientRect().left;
            }
            return this._clientRectDeltaLeft;
        },
        enumerable: true,
        configurable: true
    });
    return DomReadingContext;
}());
var ViewLineOptions = /** @class */ (function () {
    function ViewLineOptions(config, themeType) {
        this.themeType = themeType;
        this.renderWhitespace = config.editor.viewInfo.renderWhitespace;
        this.renderControlCharacters = config.editor.viewInfo.renderControlCharacters;
        this.spaceWidth = config.editor.fontInfo.spaceWidth;
        this.useMonospaceOptimizations = (config.editor.fontInfo.isMonospace
            && !config.editor.viewInfo.disableMonospaceOptimizations);
        this.canUseHalfwidthRightwardsArrow = config.editor.fontInfo.canUseHalfwidthRightwardsArrow;
        this.lineHeight = config.editor.lineHeight;
        this.stopRenderingLineAfter = config.editor.viewInfo.stopRenderingLineAfter;
        this.fontLigatures = config.editor.viewInfo.fontLigatures;
    }
    ViewLineOptions.prototype.equals = function (other) {
        return (this.themeType === other.themeType
            && this.renderWhitespace === other.renderWhitespace
            && this.renderControlCharacters === other.renderControlCharacters
            && this.spaceWidth === other.spaceWidth
            && this.useMonospaceOptimizations === other.useMonospaceOptimizations
            && this.canUseHalfwidthRightwardsArrow === other.canUseHalfwidthRightwardsArrow
            && this.lineHeight === other.lineHeight
            && this.stopRenderingLineAfter === other.stopRenderingLineAfter
            && this.fontLigatures === other.fontLigatures);
    };
    return ViewLineOptions;
}());
var ViewLine = /** @class */ (function () {
    function ViewLine(options) {
        this._options = options;
        this._isMaybeInvalid = true;
        this._renderedViewLine = null;
    }
    // --- begin IVisibleLineData
    ViewLine.prototype.getDomNode = function () {
        if (this._renderedViewLine && this._renderedViewLine.domNode) {
            return this._renderedViewLine.domNode.domNode;
        }
        return null;
    };
    ViewLine.prototype.setDomNode = function (domNode) {
        if (this._renderedViewLine) {
            this._renderedViewLine.domNode = createFastDomNode(domNode);
        }
        else {
            throw new Error('I have no rendered view line to set the dom node to...');
        }
    };
    ViewLine.prototype.onContentChanged = function () {
        this._isMaybeInvalid = true;
    };
    ViewLine.prototype.onTokensChanged = function () {
        this._isMaybeInvalid = true;
    };
    ViewLine.prototype.onDecorationsChanged = function () {
        this._isMaybeInvalid = true;
    };
    ViewLine.prototype.onOptionsChanged = function (newOptions) {
        this._isMaybeInvalid = true;
        this._options = newOptions;
    };
    ViewLine.prototype.onSelectionChanged = function () {
        if (alwaysRenderInlineSelection || this._options.themeType === HIGH_CONTRAST) {
            this._isMaybeInvalid = true;
            return true;
        }
        return false;
    };
    ViewLine.prototype.renderLine = function (lineNumber, deltaTop, viewportData, sb) {
        if (this._isMaybeInvalid === false) {
            // it appears that nothing relevant has changed
            return false;
        }
        this._isMaybeInvalid = false;
        var lineData = viewportData.getViewLineRenderingData(lineNumber);
        var options = this._options;
        var actualInlineDecorations = LineDecoration.filter(lineData.inlineDecorations, lineNumber, lineData.minColumn, lineData.maxColumn);
        if (alwaysRenderInlineSelection || options.themeType === HIGH_CONTRAST) {
            var selections = viewportData.selections;
            for (var i = 0, len = selections.length; i < len; i++) {
                var selection = selections[i];
                if (selection.endLineNumber < lineNumber || selection.startLineNumber > lineNumber) {
                    // Selection does not intersect line
                    continue;
                }
                var startColumn = (selection.startLineNumber === lineNumber ? selection.startColumn : lineData.minColumn);
                var endColumn = (selection.endLineNumber === lineNumber ? selection.endColumn : lineData.maxColumn);
                if (startColumn < endColumn) {
                    actualInlineDecorations.push(new LineDecoration(startColumn, endColumn, 'inline-selected-text', 0 /* Regular */));
                }
            }
        }
        var renderLineInput = new RenderLineInput(options.useMonospaceOptimizations, options.canUseHalfwidthRightwardsArrow, lineData.content, lineData.continuesWithWrappedLine, lineData.isBasicASCII, lineData.containsRTL, lineData.minColumn - 1, lineData.tokens, actualInlineDecorations, lineData.tabSize, options.spaceWidth, options.stopRenderingLineAfter, options.renderWhitespace, options.renderControlCharacters, options.fontLigatures);
        if (this._renderedViewLine && this._renderedViewLine.input.equals(renderLineInput)) {
            // no need to do anything, we have the same render input
            return false;
        }
        sb.appendASCIIString('<div style="top:');
        sb.appendASCIIString(String(deltaTop));
        sb.appendASCIIString('px;height:');
        sb.appendASCIIString(String(this._options.lineHeight));
        sb.appendASCIIString('px;" class="');
        sb.appendASCIIString(ViewLine.CLASS_NAME);
        sb.appendASCIIString('">');
        var output = renderViewLine(renderLineInput, sb);
        sb.appendASCIIString('</div>');
        var renderedViewLine = null;
        if (canUseFastRenderedViewLine && lineData.isBasicASCII && options.useMonospaceOptimizations && output.containsForeignElements === 0 /* None */) {
            if (lineData.content.length < 300 && renderLineInput.lineTokens.getCount() < 100) {
                // Browser rounding errors have been observed in Chrome and IE, so using the fast
                // view line only for short lines. Please test before removing the length check...
                // ---
                // Another rounding error has been observed on Linux in VSCode, where <span> width
                // rounding errors add up to an observable large number...
                // ---
                // Also see another example of rounding errors on Windows in
                // https://github.com/Microsoft/vscode/issues/33178
                renderedViewLine = new FastRenderedViewLine(this._renderedViewLine ? this._renderedViewLine.domNode : null, renderLineInput, output.characterMapping);
            }
        }
        if (!renderedViewLine) {
            renderedViewLine = createRenderedLine(this._renderedViewLine ? this._renderedViewLine.domNode : null, renderLineInput, output.characterMapping, output.containsRTL, output.containsForeignElements);
        }
        this._renderedViewLine = renderedViewLine;
        return true;
    };
    ViewLine.prototype.layoutLine = function (lineNumber, deltaTop) {
        if (this._renderedViewLine && this._renderedViewLine.domNode) {
            this._renderedViewLine.domNode.setTop(deltaTop);
            this._renderedViewLine.domNode.setHeight(this._options.lineHeight);
        }
    };
    // --- end IVisibleLineData
    ViewLine.prototype.getWidth = function () {
        if (!this._renderedViewLine) {
            return 0;
        }
        return this._renderedViewLine.getWidth();
    };
    ViewLine.prototype.getWidthIsFast = function () {
        if (!this._renderedViewLine) {
            return true;
        }
        return this._renderedViewLine.getWidthIsFast();
    };
    ViewLine.prototype.getVisibleRangesForRange = function (startColumn, endColumn, context) {
        if (!this._renderedViewLine) {
            return null;
        }
        startColumn = startColumn | 0; // @perf
        endColumn = endColumn | 0; // @perf
        startColumn = Math.min(this._renderedViewLine.input.lineContent.length + 1, Math.max(1, startColumn));
        endColumn = Math.min(this._renderedViewLine.input.lineContent.length + 1, Math.max(1, endColumn));
        var stopRenderingLineAfter = this._renderedViewLine.input.stopRenderingLineAfter | 0; // @perf
        if (stopRenderingLineAfter !== -1 && startColumn > stopRenderingLineAfter && endColumn > stopRenderingLineAfter) {
            // This range is obviously not visible
            return null;
        }
        if (stopRenderingLineAfter !== -1 && startColumn > stopRenderingLineAfter) {
            startColumn = stopRenderingLineAfter;
        }
        if (stopRenderingLineAfter !== -1 && endColumn > stopRenderingLineAfter) {
            endColumn = stopRenderingLineAfter;
        }
        return this._renderedViewLine.getVisibleRangesForRange(startColumn, endColumn, context);
    };
    ViewLine.prototype.getColumnOfNodeOffset = function (lineNumber, spanNode, offset) {
        if (!this._renderedViewLine) {
            return 1;
        }
        return this._renderedViewLine.getColumnOfNodeOffset(lineNumber, spanNode, offset);
    };
    ViewLine.CLASS_NAME = 'view-line';
    return ViewLine;
}());
/**
 * A rendered line which is guaranteed to contain only regular ASCII and is rendered with a monospace font.
 */
var FastRenderedViewLine = /** @class */ (function () {
    function FastRenderedViewLine(domNode, renderLineInput, characterMapping) {
        this.domNode = domNode;
        this.input = renderLineInput;
        this._characterMapping = characterMapping;
        this._charWidth = renderLineInput.spaceWidth;
    }
    FastRenderedViewLine.prototype.getWidth = function () {
        return this._getCharPosition(this._characterMapping.length);
    };
    FastRenderedViewLine.prototype.getWidthIsFast = function () {
        return true;
    };
    FastRenderedViewLine.prototype.getVisibleRangesForRange = function (startColumn, endColumn, context) {
        var startPosition = this._getCharPosition(startColumn);
        var endPosition = this._getCharPosition(endColumn);
        return [new HorizontalRange(startPosition, endPosition - startPosition)];
    };
    FastRenderedViewLine.prototype._getCharPosition = function (column) {
        var charOffset = this._characterMapping.getAbsoluteOffsets();
        if (charOffset.length === 0) {
            // No characters on this line
            return 0;
        }
        return Math.round(this._charWidth * charOffset[column - 1]);
    };
    FastRenderedViewLine.prototype.getColumnOfNodeOffset = function (lineNumber, spanNode, offset) {
        var spanNodeTextContentLength = spanNode.textContent.length;
        var spanIndex = -1;
        while (spanNode) {
            spanNode = spanNode.previousSibling;
            spanIndex++;
        }
        var charOffset = this._characterMapping.partDataToCharOffset(spanIndex, spanNodeTextContentLength, offset);
        return charOffset + 1;
    };
    return FastRenderedViewLine;
}());
/**
 * Every time we render a line, we save what we have rendered in an instance of this class.
 */
var RenderedViewLine = /** @class */ (function () {
    function RenderedViewLine(domNode, renderLineInput, characterMapping, containsRTL, containsForeignElements) {
        this.domNode = domNode;
        this.input = renderLineInput;
        this._characterMapping = characterMapping;
        this._isWhitespaceOnly = /^\s*$/.test(renderLineInput.lineContent);
        this._containsForeignElements = containsForeignElements;
        this._cachedWidth = -1;
        this._pixelOffsetCache = null;
        if (!containsRTL || this._characterMapping.length === 0 /* the line is empty */) {
            this._pixelOffsetCache = new Int32Array(Math.max(2, this._characterMapping.length + 1));
            for (var column = 0, len = this._characterMapping.length; column <= len; column++) {
                this._pixelOffsetCache[column] = -1;
            }
        }
    }
    // --- Reading from the DOM methods
    RenderedViewLine.prototype._getReadingTarget = function () {
        return this.domNode.domNode.firstChild;
    };
    /**
     * Width of the line in pixels
     */
    RenderedViewLine.prototype.getWidth = function () {
        if (this._cachedWidth === -1) {
            this._cachedWidth = this._getReadingTarget().offsetWidth;
        }
        return this._cachedWidth;
    };
    RenderedViewLine.prototype.getWidthIsFast = function () {
        if (this._cachedWidth === -1) {
            return false;
        }
        return true;
    };
    /**
     * Visible ranges for a model range
     */
    RenderedViewLine.prototype.getVisibleRangesForRange = function (startColumn, endColumn, context) {
        if (this._pixelOffsetCache !== null) {
            // the text is LTR
            var startOffset = this._readPixelOffset(startColumn, context);
            if (startOffset === -1) {
                return null;
            }
            var endOffset = this._readPixelOffset(endColumn, context);
            if (endOffset === -1) {
                return null;
            }
            return [new HorizontalRange(startOffset, endOffset - startOffset)];
        }
        return this._readVisibleRangesForRange(startColumn, endColumn, context);
    };
    RenderedViewLine.prototype._readVisibleRangesForRange = function (startColumn, endColumn, context) {
        if (startColumn === endColumn) {
            var pixelOffset = this._readPixelOffset(startColumn, context);
            if (pixelOffset === -1) {
                return null;
            }
            else {
                return [new HorizontalRange(pixelOffset, 0)];
            }
        }
        else {
            return this._readRawVisibleRangesForRange(startColumn, endColumn, context);
        }
    };
    RenderedViewLine.prototype._readPixelOffset = function (column, context) {
        if (this._characterMapping.length === 0) {
            // This line has no content
            if (this._containsForeignElements === 0 /* None */) {
                // We can assume the line is really empty
                return 0;
            }
            if (this._containsForeignElements === 2 /* After */) {
                // We have foreign elements after the (empty) line
                return 0;
            }
            if (this._containsForeignElements === 1 /* Before */) {
                // We have foreign element before the (empty) line
                return this.getWidth();
            }
        }
        if (this._pixelOffsetCache !== null) {
            // the text is LTR
            var cachedPixelOffset = this._pixelOffsetCache[column];
            if (cachedPixelOffset !== -1) {
                return cachedPixelOffset;
            }
            var result = this._actualReadPixelOffset(column, context);
            this._pixelOffsetCache[column] = result;
            return result;
        }
        return this._actualReadPixelOffset(column, context);
    };
    RenderedViewLine.prototype._actualReadPixelOffset = function (column, context) {
        if (this._characterMapping.length === 0) {
            // This line has no content
            var r_1 = RangeUtil.readHorizontalRanges(this._getReadingTarget(), 0, 0, 0, 0, context.clientRectDeltaLeft, context.endNode);
            if (!r_1 || r_1.length === 0) {
                return -1;
            }
            return r_1[0].left;
        }
        if (column === this._characterMapping.length && this._isWhitespaceOnly && this._containsForeignElements === 0 /* None */) {
            // This branch helps in the case of whitespace only lines which have a width set
            return this.getWidth();
        }
        var partData = this._characterMapping.charOffsetToPartData(column - 1);
        var partIndex = CharacterMapping.getPartIndex(partData);
        var charOffsetInPart = CharacterMapping.getCharIndex(partData);
        var r = RangeUtil.readHorizontalRanges(this._getReadingTarget(), partIndex, charOffsetInPart, partIndex, charOffsetInPart, context.clientRectDeltaLeft, context.endNode);
        if (!r || r.length === 0) {
            return -1;
        }
        return r[0].left;
    };
    RenderedViewLine.prototype._readRawVisibleRangesForRange = function (startColumn, endColumn, context) {
        if (startColumn === 1 && endColumn === this._characterMapping.length) {
            // This branch helps IE with bidi text & gives a performance boost to other browsers when reading visible ranges for an entire line
            return [new HorizontalRange(0, this.getWidth())];
        }
        var startPartData = this._characterMapping.charOffsetToPartData(startColumn - 1);
        var startPartIndex = CharacterMapping.getPartIndex(startPartData);
        var startCharOffsetInPart = CharacterMapping.getCharIndex(startPartData);
        var endPartData = this._characterMapping.charOffsetToPartData(endColumn - 1);
        var endPartIndex = CharacterMapping.getPartIndex(endPartData);
        var endCharOffsetInPart = CharacterMapping.getCharIndex(endPartData);
        return RangeUtil.readHorizontalRanges(this._getReadingTarget(), startPartIndex, startCharOffsetInPart, endPartIndex, endCharOffsetInPart, context.clientRectDeltaLeft, context.endNode);
    };
    /**
     * Returns the column for the text found at a specific offset inside a rendered dom node
     */
    RenderedViewLine.prototype.getColumnOfNodeOffset = function (lineNumber, spanNode, offset) {
        var spanNodeTextContentLength = spanNode.textContent.length;
        var spanIndex = -1;
        while (spanNode) {
            spanNode = spanNode.previousSibling;
            spanIndex++;
        }
        var charOffset = this._characterMapping.partDataToCharOffset(spanIndex, spanNodeTextContentLength, offset);
        return charOffset + 1;
    };
    return RenderedViewLine;
}());
var WebKitRenderedViewLine = /** @class */ (function (_super) {
    __extends$p(WebKitRenderedViewLine, _super);
    function WebKitRenderedViewLine() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebKitRenderedViewLine.prototype._readVisibleRangesForRange = function (startColumn, endColumn, context) {
        var output = _super.prototype._readVisibleRangesForRange.call(this, startColumn, endColumn, context);
        if (!output || output.length === 0 || startColumn === endColumn || (startColumn === 1 && endColumn === this._characterMapping.length)) {
            return output;
        }
        // WebKit is buggy and returns an expanded range (to contain words in some cases)
        // The last client rect is enlarged (I think)
        if (!this.input.containsRTL) {
            // This is an attempt to patch things up
            // Find position of last column
            var endPixelOffset = this._readPixelOffset(endColumn, context);
            if (endPixelOffset !== -1) {
                var lastRange = output[output.length - 1];
                if (lastRange.left < endPixelOffset) {
                    // Trim down the width of the last visible range to not go after the last column's position
                    lastRange.width = endPixelOffset - lastRange.left;
                }
            }
        }
        return output;
    };
    return WebKitRenderedViewLine;
}(RenderedViewLine));
var createRenderedLine = (function () {
    if (isWebKit) {
        return createWebKitRenderedLine;
    }
    return createNormalRenderedLine;
})();
function createWebKitRenderedLine(domNode, renderLineInput, characterMapping, containsRTL, containsForeignElements) {
    return new WebKitRenderedViewLine(domNode, renderLineInput, characterMapping, containsRTL, containsForeignElements);
}
function createNormalRenderedLine(domNode, renderLineInput, characterMapping, containsRTL, containsForeignElements) {
    return new RenderedViewLine(domNode, renderLineInput, characterMapping, containsRTL, containsForeignElements);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$q = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var MouseTarget = /** @class */ (function () {
    function MouseTarget(element, type, mouseColumn, position, range, detail) {
        if (mouseColumn === void 0) { mouseColumn = 0; }
        if (position === void 0) { position = null; }
        if (range === void 0) { range = null; }
        if (detail === void 0) { detail = null; }
        this.element = element;
        this.type = type;
        this.mouseColumn = mouseColumn;
        this.position = position;
        if (!range && position) {
            range = new Range(position.lineNumber, position.column, position.lineNumber, position.column);
        }
        this.range = range;
        this.detail = detail;
    }
    MouseTarget._typeToString = function (type) {
        if (type === 1 /* TEXTAREA */) {
            return 'TEXTAREA';
        }
        if (type === 2 /* GUTTER_GLYPH_MARGIN */) {
            return 'GUTTER_GLYPH_MARGIN';
        }
        if (type === 3 /* GUTTER_LINE_NUMBERS */) {
            return 'GUTTER_LINE_NUMBERS';
        }
        if (type === 4 /* GUTTER_LINE_DECORATIONS */) {
            return 'GUTTER_LINE_DECORATIONS';
        }
        if (type === 5 /* GUTTER_VIEW_ZONE */) {
            return 'GUTTER_VIEW_ZONE';
        }
        if (type === 6 /* CONTENT_TEXT */) {
            return 'CONTENT_TEXT';
        }
        if (type === 7 /* CONTENT_EMPTY */) {
            return 'CONTENT_EMPTY';
        }
        if (type === 8 /* CONTENT_VIEW_ZONE */) {
            return 'CONTENT_VIEW_ZONE';
        }
        if (type === 9 /* CONTENT_WIDGET */) {
            return 'CONTENT_WIDGET';
        }
        if (type === 10 /* OVERVIEW_RULER */) {
            return 'OVERVIEW_RULER';
        }
        if (type === 11 /* SCROLLBAR */) {
            return 'SCROLLBAR';
        }
        if (type === 12 /* OVERLAY_WIDGET */) {
            return 'OVERLAY_WIDGET';
        }
        return 'UNKNOWN';
    };
    MouseTarget.toString = function (target) {
        return this._typeToString(target.type) + ': ' + target.position + ' - ' + target.range + ' - ' + target.detail;
    };
    MouseTarget.prototype.toString = function () {
        return MouseTarget.toString(this);
    };
    return MouseTarget;
}());
var ElementPath = /** @class */ (function () {
    function ElementPath() {
    }
    ElementPath.isTextArea = function (path) {
        return (path.length === 2
            && path[0] === 3 /* OverflowGuard */
            && path[1] === 6 /* TextArea */);
    };
    ElementPath.isChildOfViewLines = function (path) {
        return (path.length >= 4
            && path[0] === 3 /* OverflowGuard */
            && path[3] === 7 /* ViewLines */);
    };
    ElementPath.isStrictChildOfViewLines = function (path) {
        return (path.length > 4
            && path[0] === 3 /* OverflowGuard */
            && path[3] === 7 /* ViewLines */);
    };
    ElementPath.isChildOfScrollableElement = function (path) {
        return (path.length >= 2
            && path[0] === 3 /* OverflowGuard */
            && path[1] === 5 /* ScrollableElement */);
    };
    ElementPath.isChildOfMinimap = function (path) {
        return (path.length >= 2
            && path[0] === 3 /* OverflowGuard */
            && path[1] === 8 /* Minimap */);
    };
    ElementPath.isChildOfContentWidgets = function (path) {
        return (path.length >= 4
            && path[0] === 3 /* OverflowGuard */
            && path[3] === 1 /* ContentWidgets */);
    };
    ElementPath.isChildOfOverflowingContentWidgets = function (path) {
        return (path.length >= 1
            && path[0] === 2 /* OverflowingContentWidgets */);
    };
    ElementPath.isChildOfOverlayWidgets = function (path) {
        return (path.length >= 2
            && path[0] === 3 /* OverflowGuard */
            && path[1] === 4 /* OverlayWidgets */);
    };
    return ElementPath;
}());
var HitTestContext = /** @class */ (function () {
    function HitTestContext(context, viewHelper, lastViewCursorsRenderData) {
        this.model = context.model;
        this.layoutInfo = context.configuration.editor.layoutInfo;
        this.viewDomNode = viewHelper.viewDomNode;
        this.lineHeight = context.configuration.editor.lineHeight;
        this.typicalHalfwidthCharacterWidth = context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        this.lastViewCursorsRenderData = lastViewCursorsRenderData;
        this._context = context;
        this._viewHelper = viewHelper;
    }
    HitTestContext.prototype.getZoneAtCoord = function (mouseVerticalOffset) {
        return HitTestContext.getZoneAtCoord(this._context, mouseVerticalOffset);
    };
    HitTestContext.getZoneAtCoord = function (context, mouseVerticalOffset) {
        // The target is either a view zone or the empty space after the last view-line
        var viewZoneWhitespace = context.viewLayout.getWhitespaceAtVerticalOffset(mouseVerticalOffset);
        if (viewZoneWhitespace) {
            var viewZoneMiddle = viewZoneWhitespace.verticalOffset + viewZoneWhitespace.height / 2, lineCount = context.model.getLineCount(), positionBefore = null, position = void 0, positionAfter = null;
            if (viewZoneWhitespace.afterLineNumber !== lineCount) {
                // There are more lines after this view zone
                positionAfter = new Position(viewZoneWhitespace.afterLineNumber + 1, 1);
            }
            if (viewZoneWhitespace.afterLineNumber > 0) {
                // There are more lines above this view zone
                positionBefore = new Position(viewZoneWhitespace.afterLineNumber, context.model.getLineMaxColumn(viewZoneWhitespace.afterLineNumber));
            }
            if (positionAfter === null) {
                position = positionBefore;
            }
            else if (positionBefore === null) {
                position = positionAfter;
            }
            else if (mouseVerticalOffset < viewZoneMiddle) {
                position = positionBefore;
            }
            else {
                position = positionAfter;
            }
            return {
                viewZoneId: viewZoneWhitespace.id,
                afterLineNumber: viewZoneWhitespace.afterLineNumber,
                positionBefore: positionBefore,
                positionAfter: positionAfter,
                position: position
            };
        }
        return null;
    };
    HitTestContext.prototype.getFullLineRangeAtCoord = function (mouseVerticalOffset) {
        if (this._context.viewLayout.isAfterLines(mouseVerticalOffset)) {
            // Below the last line
            var lineNumber_1 = this._context.model.getLineCount();
            var maxLineColumn_1 = this._context.model.getLineMaxColumn(lineNumber_1);
            return {
                range: new Range(lineNumber_1, maxLineColumn_1, lineNumber_1, maxLineColumn_1),
                isAfterLines: true
            };
        }
        var lineNumber = this._context.viewLayout.getLineNumberAtVerticalOffset(mouseVerticalOffset);
        var maxLineColumn = this._context.model.getLineMaxColumn(lineNumber);
        return {
            range: new Range(lineNumber, 1, lineNumber, maxLineColumn),
            isAfterLines: false
        };
    };
    HitTestContext.prototype.getLineNumberAtVerticalOffset = function (mouseVerticalOffset) {
        return this._context.viewLayout.getLineNumberAtVerticalOffset(mouseVerticalOffset);
    };
    HitTestContext.prototype.isAfterLines = function (mouseVerticalOffset) {
        return this._context.viewLayout.isAfterLines(mouseVerticalOffset);
    };
    HitTestContext.prototype.getVerticalOffsetForLineNumber = function (lineNumber) {
        return this._context.viewLayout.getVerticalOffsetForLineNumber(lineNumber);
    };
    HitTestContext.prototype.findAttribute = function (element, attr) {
        return HitTestContext._findAttribute(element, attr, this._viewHelper.viewDomNode);
    };
    HitTestContext._findAttribute = function (element, attr, stopAt) {
        while (element && element !== document.body) {
            if (element.hasAttribute && element.hasAttribute(attr)) {
                return element.getAttribute(attr);
            }
            if (element === stopAt) {
                return null;
            }
            element = element.parentNode;
        }
        return null;
    };
    HitTestContext.prototype.getLineWidth = function (lineNumber) {
        return this._viewHelper.getLineWidth(lineNumber);
    };
    HitTestContext.prototype.visibleRangeForPosition2 = function (lineNumber, column) {
        return this._viewHelper.visibleRangeForPosition2(lineNumber, column);
    };
    HitTestContext.prototype.getPositionFromDOMInfo = function (spanNode, offset) {
        return this._viewHelper.getPositionFromDOMInfo(spanNode, offset);
    };
    HitTestContext.prototype.getCurrentScrollTop = function () {
        return this._context.viewLayout.getCurrentScrollTop();
    };
    HitTestContext.prototype.getCurrentScrollLeft = function () {
        return this._context.viewLayout.getCurrentScrollLeft();
    };
    return HitTestContext;
}());
var BareHitTestRequest = /** @class */ (function () {
    function BareHitTestRequest(ctx, editorPos, pos) {
        this.editorPos = editorPos;
        this.pos = pos;
        this.mouseVerticalOffset = Math.max(0, ctx.getCurrentScrollTop() + pos.y - editorPos.y);
        this.mouseContentHorizontalOffset = ctx.getCurrentScrollLeft() + pos.x - editorPos.x - ctx.layoutInfo.contentLeft;
        this.isInMarginArea = (pos.x - editorPos.x < ctx.layoutInfo.contentLeft && pos.x - editorPos.x >= ctx.layoutInfo.glyphMarginLeft);
        this.isInContentArea = !this.isInMarginArea;
        this.mouseColumn = Math.max(0, MouseTargetFactory._getMouseColumn(this.mouseContentHorizontalOffset, ctx.typicalHalfwidthCharacterWidth));
    }
    return BareHitTestRequest;
}());
var HitTestRequest = /** @class */ (function (_super) {
    __extends$q(HitTestRequest, _super);
    function HitTestRequest(ctx, editorPos, pos, target) {
        var _this = _super.call(this, ctx, editorPos, pos) || this;
        _this._ctx = ctx;
        if (target) {
            _this.target = target;
            _this.targetPath = PartFingerprints.collect(target, ctx.viewDomNode);
        }
        else {
            _this.target = null;
            _this.targetPath = new Uint8Array(0);
        }
        return _this;
    }
    HitTestRequest.prototype.toString = function () {
        return "pos(" + this.pos.x + "," + this.pos.y + "), editorPos(" + this.editorPos.x + "," + this.editorPos.y + "), mouseVerticalOffset: " + this.mouseVerticalOffset + ", mouseContentHorizontalOffset: " + this.mouseContentHorizontalOffset + "\n\ttarget: " + (this.target ? this.target.outerHTML : null);
    };
    HitTestRequest.prototype.fulfill = function (type, position, range, detail) {
        if (position === void 0) { position = null; }
        if (range === void 0) { range = null; }
        if (detail === void 0) { detail = null; }
        return new MouseTarget(this.target, type, this.mouseColumn, position, range, detail);
    };
    HitTestRequest.prototype.withTarget = function (target) {
        return new HitTestRequest(this._ctx, this.editorPos, this.pos, target);
    };
    return HitTestRequest;
}(BareHitTestRequest));
var EMPTY_CONTENT_AFTER_LINES = { isAfterLines: true };
function createEmptyContentDataInLines(horizontalDistanceToText) {
    return {
        isAfterLines: false,
        horizontalDistanceToText: horizontalDistanceToText
    };
}
var MouseTargetFactory = /** @class */ (function () {
    function MouseTargetFactory(context, viewHelper) {
        this._context = context;
        this._viewHelper = viewHelper;
    }
    MouseTargetFactory.prototype.mouseTargetIsWidget = function (e) {
        var t = e.target;
        var path = PartFingerprints.collect(t, this._viewHelper.viewDomNode);
        // Is it a content widget?
        if (ElementPath.isChildOfContentWidgets(path) || ElementPath.isChildOfOverflowingContentWidgets(path)) {
            return true;
        }
        // Is it an overlay widget?
        if (ElementPath.isChildOfOverlayWidgets(path)) {
            return true;
        }
        return false;
    };
    MouseTargetFactory.prototype.createMouseTarget = function (lastViewCursorsRenderData, editorPos, pos, target) {
        var ctx = new HitTestContext(this._context, this._viewHelper, lastViewCursorsRenderData);
        var request = new HitTestRequest(ctx, editorPos, pos, target);
        try {
            var r = MouseTargetFactory._createMouseTarget(ctx, request, false);
            // console.log(r.toString());
            return r;
        }
        catch (err) {
            // console.log(err);
            return request.fulfill(0 /* UNKNOWN */);
        }
    };
    MouseTargetFactory._createMouseTarget = function (ctx, request, domHitTestExecuted) {
        // console.log(`${domHitTestExecuted ? '=>' : ''}CAME IN REQUEST: ${request}`);
        // First ensure the request has a target
        if (request.target === null) {
            if (domHitTestExecuted) {
                // Still no target... and we have already executed hit test...
                return request.fulfill(0 /* UNKNOWN */);
            }
            var hitTestResult = MouseTargetFactory._doHitTest(ctx, request);
            if (hitTestResult.position) {
                return MouseTargetFactory.createMouseTargetFromHitTestPosition(ctx, request, hitTestResult.position.lineNumber, hitTestResult.position.column);
            }
            return this._createMouseTarget(ctx, request.withTarget(hitTestResult.hitTarget), true);
        }
        // we know for a fact that request.target is not null
        var resolvedRequest = request;
        var result = null;
        result = result || MouseTargetFactory._hitTestContentWidget(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestOverlayWidget(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestMinimap(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestScrollbarSlider(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestViewZone(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestMargin(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestViewCursor(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestTextArea(ctx, resolvedRequest);
        result = result || MouseTargetFactory._hitTestViewLines(ctx, resolvedRequest, domHitTestExecuted);
        result = result || MouseTargetFactory._hitTestScrollbar(ctx, resolvedRequest);
        return (result || request.fulfill(0 /* UNKNOWN */));
    };
    MouseTargetFactory._hitTestContentWidget = function (ctx, request) {
        // Is it a content widget?
        if (ElementPath.isChildOfContentWidgets(request.targetPath) || ElementPath.isChildOfOverflowingContentWidgets(request.targetPath)) {
            var widgetId = ctx.findAttribute(request.target, 'widgetId');
            if (widgetId) {
                return request.fulfill(9 /* CONTENT_WIDGET */, null, null, widgetId);
            }
            else {
                return request.fulfill(0 /* UNKNOWN */);
            }
        }
        return null;
    };
    MouseTargetFactory._hitTestOverlayWidget = function (ctx, request) {
        // Is it an overlay widget?
        if (ElementPath.isChildOfOverlayWidgets(request.targetPath)) {
            var widgetId = ctx.findAttribute(request.target, 'widgetId');
            if (widgetId) {
                return request.fulfill(12 /* OVERLAY_WIDGET */, null, null, widgetId);
            }
            else {
                return request.fulfill(0 /* UNKNOWN */);
            }
        }
        return null;
    };
    MouseTargetFactory._hitTestViewCursor = function (ctx, request) {
        if (request.target) {
            // Check if we've hit a painted cursor
            var lastViewCursorsRenderData = ctx.lastViewCursorsRenderData;
            for (var i = 0, len = lastViewCursorsRenderData.length; i < len; i++) {
                var d = lastViewCursorsRenderData[i];
                if (request.target === d.domNode) {
                    return request.fulfill(6 /* CONTENT_TEXT */, d.position);
                }
            }
        }
        if (request.isInContentArea) {
            // Edge has a bug when hit-testing the exact position of a cursor,
            // instead of returning the correct dom node, it returns the
            // first or last rendered view line dom node, therefore help it out
            // and first check if we are on top of a cursor
            var lastViewCursorsRenderData = ctx.lastViewCursorsRenderData;
            var mouseContentHorizontalOffset = request.mouseContentHorizontalOffset;
            var mouseVerticalOffset = request.mouseVerticalOffset;
            for (var i = 0, len = lastViewCursorsRenderData.length; i < len; i++) {
                var d = lastViewCursorsRenderData[i];
                if (mouseContentHorizontalOffset < d.contentLeft) {
                    // mouse position is to the left of the cursor
                    continue;
                }
                if (mouseContentHorizontalOffset > d.contentLeft + d.width) {
                    // mouse position is to the right of the cursor
                    continue;
                }
                var cursorVerticalOffset = ctx.getVerticalOffsetForLineNumber(d.position.lineNumber);
                if (cursorVerticalOffset <= mouseVerticalOffset
                    && mouseVerticalOffset <= cursorVerticalOffset + d.height) {
                    return request.fulfill(6 /* CONTENT_TEXT */, d.position);
                }
            }
        }
        return null;
    };
    MouseTargetFactory._hitTestViewZone = function (ctx, request) {
        var viewZoneData = ctx.getZoneAtCoord(request.mouseVerticalOffset);
        if (viewZoneData) {
            var mouseTargetType = (request.isInContentArea ? 8 /* CONTENT_VIEW_ZONE */ : 5 /* GUTTER_VIEW_ZONE */);
            return request.fulfill(mouseTargetType, viewZoneData.position, null, viewZoneData);
        }
        return null;
    };
    MouseTargetFactory._hitTestTextArea = function (ctx, request) {
        // Is it the textarea?
        if (ElementPath.isTextArea(request.targetPath)) {
            return request.fulfill(1 /* TEXTAREA */);
        }
        return null;
    };
    MouseTargetFactory._hitTestMargin = function (ctx, request) {
        if (request.isInMarginArea) {
            var res = ctx.getFullLineRangeAtCoord(request.mouseVerticalOffset);
            var pos = res.range.getStartPosition();
            var offset = Math.abs(request.pos.x - request.editorPos.x);
            var detail = {
                isAfterLines: res.isAfterLines,
                glyphMarginLeft: ctx.layoutInfo.glyphMarginLeft,
                glyphMarginWidth: ctx.layoutInfo.glyphMarginWidth,
                lineNumbersWidth: ctx.layoutInfo.lineNumbersWidth,
                offsetX: offset
            };
            offset -= ctx.layoutInfo.glyphMarginLeft;
            if (offset <= ctx.layoutInfo.glyphMarginWidth) {
                // On the glyph margin
                return request.fulfill(2 /* GUTTER_GLYPH_MARGIN */, pos, res.range, detail);
            }
            offset -= ctx.layoutInfo.glyphMarginWidth;
            if (offset <= ctx.layoutInfo.lineNumbersWidth) {
                // On the line numbers
                return request.fulfill(3 /* GUTTER_LINE_NUMBERS */, pos, res.range, detail);
            }
            offset -= ctx.layoutInfo.lineNumbersWidth;
            // On the line decorations
            return request.fulfill(4 /* GUTTER_LINE_DECORATIONS */, pos, res.range, detail);
        }
        return null;
    };
    MouseTargetFactory._hitTestViewLines = function (ctx, request, domHitTestExecuted) {
        if (!ElementPath.isChildOfViewLines(request.targetPath)) {
            return null;
        }
        // Check if it is below any lines and any view zones
        if (ctx.isAfterLines(request.mouseVerticalOffset)) {
            // This most likely indicates it happened after the last view-line
            var lineCount = ctx.model.getLineCount();
            var maxLineColumn = ctx.model.getLineMaxColumn(lineCount);
            return request.fulfill(7 /* CONTENT_EMPTY */, new Position(lineCount, maxLineColumn), void 0, EMPTY_CONTENT_AFTER_LINES);
        }
        if (domHitTestExecuted) {
            // Check if we are hitting a view-line (can happen in the case of inline decorations on empty lines)
            // See https://github.com/Microsoft/vscode/issues/46942
            if (ElementPath.isStrictChildOfViewLines(request.targetPath)) {
                var lineNumber = ctx.getLineNumberAtVerticalOffset(request.mouseVerticalOffset);
                if (ctx.model.getLineLength(lineNumber) === 0) {
                    var lineWidth = ctx.getLineWidth(lineNumber);
                    var detail = createEmptyContentDataInLines(request.mouseContentHorizontalOffset - lineWidth);
                    return request.fulfill(7 /* CONTENT_EMPTY */, new Position(lineNumber, 1), void 0, detail);
                }
            }
            // We have already executed hit test...
            return request.fulfill(0 /* UNKNOWN */);
        }
        var hitTestResult = MouseTargetFactory._doHitTest(ctx, request);
        if (hitTestResult.position) {
            return MouseTargetFactory.createMouseTargetFromHitTestPosition(ctx, request, hitTestResult.position.lineNumber, hitTestResult.position.column);
        }
        return this._createMouseTarget(ctx, request.withTarget(hitTestResult.hitTarget), true);
    };
    MouseTargetFactory._hitTestMinimap = function (ctx, request) {
        if (ElementPath.isChildOfMinimap(request.targetPath)) {
            var possibleLineNumber = ctx.getLineNumberAtVerticalOffset(request.mouseVerticalOffset);
            var maxColumn = ctx.model.getLineMaxColumn(possibleLineNumber);
            return request.fulfill(11 /* SCROLLBAR */, new Position(possibleLineNumber, maxColumn));
        }
        return null;
    };
    MouseTargetFactory._hitTestScrollbarSlider = function (ctx, request) {
        if (ElementPath.isChildOfScrollableElement(request.targetPath)) {
            if (request.target && request.target.nodeType === 1) {
                var className = request.target.className;
                if (className && /\b(slider|scrollbar)\b/.test(className)) {
                    var possibleLineNumber = ctx.getLineNumberAtVerticalOffset(request.mouseVerticalOffset);
                    var maxColumn = ctx.model.getLineMaxColumn(possibleLineNumber);
                    return request.fulfill(11 /* SCROLLBAR */, new Position(possibleLineNumber, maxColumn));
                }
            }
        }
        return null;
    };
    MouseTargetFactory._hitTestScrollbar = function (ctx, request) {
        // Is it the overview ruler?
        // Is it a child of the scrollable element?
        if (ElementPath.isChildOfScrollableElement(request.targetPath)) {
            var possibleLineNumber = ctx.getLineNumberAtVerticalOffset(request.mouseVerticalOffset);
            var maxColumn = ctx.model.getLineMaxColumn(possibleLineNumber);
            return request.fulfill(11 /* SCROLLBAR */, new Position(possibleLineNumber, maxColumn));
        }
        return null;
    };
    MouseTargetFactory.prototype.getMouseColumn = function (editorPos, pos) {
        var layoutInfo = this._context.configuration.editor.layoutInfo;
        var mouseContentHorizontalOffset = this._context.viewLayout.getCurrentScrollLeft() + pos.x - editorPos.x - layoutInfo.contentLeft;
        return MouseTargetFactory._getMouseColumn(mouseContentHorizontalOffset, this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth);
    };
    MouseTargetFactory._getMouseColumn = function (mouseContentHorizontalOffset, typicalHalfwidthCharacterWidth) {
        if (mouseContentHorizontalOffset < 0) {
            return 1;
        }
        var chars = Math.round(mouseContentHorizontalOffset / typicalHalfwidthCharacterWidth);
        return (chars + 1);
    };
    MouseTargetFactory.createMouseTargetFromHitTestPosition = function (ctx, request, lineNumber, column) {
        var pos = new Position(lineNumber, column);
        var lineWidth = ctx.getLineWidth(lineNumber);
        if (request.mouseContentHorizontalOffset > lineWidth) {
            if (isEdge && pos.column === 1) {
                // See https://github.com/Microsoft/vscode/issues/10875
                var detail_1 = createEmptyContentDataInLines(request.mouseContentHorizontalOffset - lineWidth);
                return request.fulfill(7 /* CONTENT_EMPTY */, new Position(lineNumber, ctx.model.getLineMaxColumn(lineNumber)), void 0, detail_1);
            }
            var detail = createEmptyContentDataInLines(request.mouseContentHorizontalOffset - lineWidth);
            return request.fulfill(7 /* CONTENT_EMPTY */, pos, void 0, detail);
        }
        var visibleRange = ctx.visibleRangeForPosition2(lineNumber, column);
        if (!visibleRange) {
            return request.fulfill(0 /* UNKNOWN */, pos);
        }
        var columnHorizontalOffset = visibleRange.left;
        if (request.mouseContentHorizontalOffset === columnHorizontalOffset) {
            return request.fulfill(6 /* CONTENT_TEXT */, pos);
        }
        var points = [];
        points.push({ offset: visibleRange.left, column: column });
        if (column > 1) {
            var visibleRange_1 = ctx.visibleRangeForPosition2(lineNumber, column - 1);
            if (visibleRange_1) {
                points.push({ offset: visibleRange_1.left, column: column - 1 });
            }
        }
        var lineMaxColumn = ctx.model.getLineMaxColumn(lineNumber);
        if (column < lineMaxColumn) {
            var visibleRange_2 = ctx.visibleRangeForPosition2(lineNumber, column + 1);
            if (visibleRange_2) {
                points.push({ offset: visibleRange_2.left, column: column + 1 });
            }
        }
        points.sort(function (a, b) { return a.offset - b.offset; });
        for (var i = 1; i < points.length; i++) {
            var prev = points[i - 1];
            var curr = points[i];
            if (prev.offset <= request.mouseContentHorizontalOffset && request.mouseContentHorizontalOffset <= curr.offset) {
                var rng = new Range(lineNumber, prev.column, lineNumber, curr.column);
                return request.fulfill(6 /* CONTENT_TEXT */, pos, rng);
            }
        }
        return request.fulfill(6 /* CONTENT_TEXT */, pos);
    };
    /**
     * Most probably WebKit browsers and Edge
     */
    MouseTargetFactory._doHitTestWithCaretRangeFromPoint = function (ctx, request) {
        // In Chrome, especially on Linux it is possible to click between lines,
        // so try to adjust the `hity` below so that it lands in the center of a line
        var lineNumber = ctx.getLineNumberAtVerticalOffset(request.mouseVerticalOffset);
        var lineVerticalOffset = ctx.getVerticalOffsetForLineNumber(lineNumber);
        var lineCenteredVerticalOffset = lineVerticalOffset + Math.floor(ctx.lineHeight / 2);
        var adjustedPageY = request.pos.y + (lineCenteredVerticalOffset - request.mouseVerticalOffset);
        if (adjustedPageY <= request.editorPos.y) {
            adjustedPageY = request.editorPos.y + 1;
        }
        if (adjustedPageY >= request.editorPos.y + ctx.layoutInfo.height) {
            adjustedPageY = request.editorPos.y + ctx.layoutInfo.height - 1;
        }
        var adjustedPage = new PageCoordinates(request.pos.x, adjustedPageY);
        var r = this._actualDoHitTestWithCaretRangeFromPoint(ctx, adjustedPage.toClientCoordinates());
        if (r.position) {
            return r;
        }
        // Also try to hit test without the adjustment (for the edge cases that we are near the top or bottom)
        return this._actualDoHitTestWithCaretRangeFromPoint(ctx, request.pos.toClientCoordinates());
    };
    MouseTargetFactory._actualDoHitTestWithCaretRangeFromPoint = function (ctx, coords) {
        var range = document.caretRangeFromPoint(coords.clientX, coords.clientY);
        if (!range || !range.startContainer) {
            return {
                position: null,
                hitTarget: null
            };
        }
        // Chrome always hits a TEXT_NODE, while Edge sometimes hits a token span
        var startContainer = range.startContainer;
        var hitTarget = null;
        if (startContainer.nodeType === startContainer.TEXT_NODE) {
            // startContainer is expected to be the token text
            var parent1 = startContainer.parentNode; // expected to be the token span
            var parent2 = parent1 ? parent1.parentNode : null; // expected to be the view line container span
            var parent3 = parent2 ? parent2.parentNode : null; // expected to be the view line div
            var parent3ClassName = parent3 && parent3.nodeType === parent3.ELEMENT_NODE ? parent3.className : null;
            if (parent3ClassName === ViewLine.CLASS_NAME) {
                var p = ctx.getPositionFromDOMInfo(parent1, range.startOffset);
                return {
                    position: p,
                    hitTarget: null
                };
            }
            else {
                hitTarget = startContainer.parentNode;
            }
        }
        else if (startContainer.nodeType === startContainer.ELEMENT_NODE) {
            // startContainer is expected to be the token span
            var parent1 = startContainer.parentNode; // expected to be the view line container span
            var parent2 = parent1 ? parent1.parentNode : null; // expected to be the view line div
            var parent2ClassName = parent2 && parent2.nodeType === parent2.ELEMENT_NODE ? parent2.className : null;
            if (parent2ClassName === ViewLine.CLASS_NAME) {
                var p = ctx.getPositionFromDOMInfo(startContainer, startContainer.textContent.length);
                return {
                    position: p,
                    hitTarget: null
                };
            }
            else {
                hitTarget = startContainer;
            }
        }
        return {
            position: null,
            hitTarget: hitTarget
        };
    };
    /**
     * Most probably Gecko
     */
    MouseTargetFactory._doHitTestWithCaretPositionFromPoint = function (ctx, coords) {
        var hitResult = document.caretPositionFromPoint(coords.clientX, coords.clientY);
        if (hitResult.offsetNode.nodeType === hitResult.offsetNode.TEXT_NODE) {
            // offsetNode is expected to be the token text
            var parent1 = hitResult.offsetNode.parentNode; // expected to be the token span
            var parent2 = parent1 ? parent1.parentNode : null; // expected to be the view line container span
            var parent3 = parent2 ? parent2.parentNode : null; // expected to be the view line div
            var parent3ClassName = parent3 && parent3.nodeType === parent3.ELEMENT_NODE ? parent3.className : null;
            if (parent3ClassName === ViewLine.CLASS_NAME) {
                var p = ctx.getPositionFromDOMInfo(hitResult.offsetNode.parentNode, hitResult.offset);
                return {
                    position: p,
                    hitTarget: null
                };
            }
            else {
                return {
                    position: null,
                    hitTarget: hitResult.offsetNode.parentNode
                };
            }
        }
        return {
            position: null,
            hitTarget: hitResult.offsetNode
        };
    };
    /**
     * Most probably IE
     */
    MouseTargetFactory._doHitTestWithMoveToPoint = function (ctx, coords) {
        var resultPosition = null;
        var resultHitTarget = null;
        var textRange = document.body.createTextRange();
        try {
            textRange.moveToPoint(coords.clientX, coords.clientY);
        }
        catch (err) {
            return {
                position: null,
                hitTarget: null
            };
        }
        textRange.collapse(true);
        // Now, let's do our best to figure out what we hit :)
        var parentElement = textRange ? textRange.parentElement() : null;
        var parent1 = parentElement ? parentElement.parentNode : null;
        var parent2 = parent1 ? parent1.parentNode : null;
        var parent2ClassName = parent2 && parent2.nodeType === parent2.ELEMENT_NODE ? parent2.className : '';
        if (parent2ClassName === ViewLine.CLASS_NAME) {
            var rangeToContainEntireSpan = textRange.duplicate();
            rangeToContainEntireSpan.moveToElementText(parentElement);
            rangeToContainEntireSpan.setEndPoint('EndToStart', textRange);
            resultPosition = ctx.getPositionFromDOMInfo(parentElement, rangeToContainEntireSpan.text.length);
            // Move range out of the span node, IE doesn't like having many ranges in
            // the same spot and will act badly for lines containing dashes ('-')
            rangeToContainEntireSpan.moveToElementText(ctx.viewDomNode);
        }
        else {
            // Looks like we've hit the hover or something foreign
            resultHitTarget = parentElement;
        }
        // Move range out of the span node, IE doesn't like having many ranges in
        // the same spot and will act badly for lines containing dashes ('-')
        textRange.moveToElementText(ctx.viewDomNode);
        return {
            position: resultPosition,
            hitTarget: resultHitTarget
        };
    };
    MouseTargetFactory._doHitTest = function (ctx, request) {
        // State of the art (18.10.2012):
        // The spec says browsers should support document.caretPositionFromPoint, but nobody implemented it (http://dev.w3.org/csswg/cssom-view/)
        // Gecko:
        //    - they tried to implement it once, but failed: https://bugzilla.mozilla.org/show_bug.cgi?id=654352
        //    - however, they do give out rangeParent/rangeOffset properties on mouse events
        // Webkit:
        //    - they have implemented a previous version of the spec which was using document.caretRangeFromPoint
        // IE:
        //    - they have a proprietary method on ranges, moveToPoint: https://msdn.microsoft.com/en-us/library/ie/ms536632(v=vs.85).aspx
        // 24.08.2016: Edge has added WebKit's document.caretRangeFromPoint, but it is quite buggy
        //    - when hit testing the cursor it returns the first or the last line in the viewport
        //    - it inconsistently hits text nodes or span nodes, while WebKit only hits text nodes
        //    - when toggling render whitespace on, and hit testing in the empty content after a line, it always hits offset 0 of the first span of the line
        // Thank you browsers for making this so 'easy' :)
        if (document.caretRangeFromPoint) {
            return this._doHitTestWithCaretRangeFromPoint(ctx, request);
        }
        else if (document.caretPositionFromPoint) {
            return this._doHitTestWithCaretPositionFromPoint(ctx, request.pos.toClientCoordinates());
        }
        else if (document.body.createTextRange) {
            return this._doHitTestWithMoveToPoint(ctx, request.pos.toClientCoordinates());
        }
        return {
            position: null,
            hitTarget: null
        };
    };
    return MouseTargetFactory;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$r = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * Merges mouse events when mouse move events are throttled
 */
function createMouseMoveEventMerger(mouseTargetFactory) {
    return function (lastEvent, currentEvent) {
        var targetIsWidget = false;
        if (mouseTargetFactory) {
            targetIsWidget = mouseTargetFactory.mouseTargetIsWidget(currentEvent);
        }
        if (!targetIsWidget) {
            currentEvent.preventDefault();
        }
        return currentEvent;
    };
}
var MouseHandler = /** @class */ (function (_super) {
    __extends$r(MouseHandler, _super);
    function MouseHandler(context, viewController, viewHelper) {
        var _this = _super.call(this) || this;
        _this._isFocused = false;
        _this._context = context;
        _this.viewController = viewController;
        _this.viewHelper = viewHelper;
        _this.mouseTargetFactory = new MouseTargetFactory(_this._context, viewHelper);
        _this._mouseDownOperation = _this._register(new MouseDownOperation(_this._context, _this.viewController, _this.viewHelper, function (e, testEventTarget) { return _this._createMouseTarget(e, testEventTarget); }, function (e) { return _this._getMouseColumn(e); }));
        _this._asyncFocus = _this._register(new RunOnceScheduler(function () { return _this.viewHelper.focusTextArea(); }, 0));
        _this.lastMouseLeaveTime = -1;
        var mouseEvents = new EditorMouseEventFactory(_this.viewHelper.viewDomNode);
        _this._register(mouseEvents.onContextMenu(_this.viewHelper.viewDomNode, function (e) { return _this._onContextMenu(e, true); }));
        _this._register(mouseEvents.onMouseMoveThrottled(_this.viewHelper.viewDomNode, function (e) { return _this._onMouseMove(e); }, createMouseMoveEventMerger(_this.mouseTargetFactory), MouseHandler.MOUSE_MOVE_MINIMUM_TIME));
        _this._register(mouseEvents.onMouseUp(_this.viewHelper.viewDomNode, function (e) { return _this._onMouseUp(e); }));
        _this._register(mouseEvents.onMouseLeave(_this.viewHelper.viewDomNode, function (e) { return _this._onMouseLeave(e); }));
        _this._register(mouseEvents.onMouseDown(_this.viewHelper.viewDomNode, function (e) { return _this._onMouseDown(e); }));
        var onMouseWheel = function (browserEvent) {
            if (!_this._context.configuration.editor.viewInfo.mouseWheelZoom) {
                return;
            }
            var e = new StandardMouseWheelEvent(browserEvent);
            if (e.browserEvent.ctrlKey || e.browserEvent.metaKey) {
                var zoomLevel = EditorZoom.getZoomLevel();
                var delta = e.deltaY > 0 ? 1 : -1;
                EditorZoom.setZoomLevel(zoomLevel + delta);
                e.preventDefault();
                e.stopPropagation();
            }
        };
        _this._register(addDisposableListener(_this.viewHelper.viewDomNode, 'mousewheel', onMouseWheel, true));
        _this._register(addDisposableListener(_this.viewHelper.viewDomNode, 'DOMMouseScroll', onMouseWheel, true));
        _this._context.addEventHandler(_this);
        return _this;
    }
    MouseHandler.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    MouseHandler.prototype.onCursorStateChanged = function (e) {
        this._mouseDownOperation.onCursorStateChanged(e);
        return false;
    };
    MouseHandler.prototype.onFocusChanged = function (e) {
        this._isFocused = e.isFocused;
        return false;
    };
    MouseHandler.prototype.onScrollChanged = function (e) {
        this._mouseDownOperation.onScrollChanged();
        return false;
    };
    // --- end event handlers
    MouseHandler.prototype.getTargetAtClientPoint = function (clientX, clientY) {
        var clientPos = new ClientCoordinates(clientX, clientY);
        var pos = clientPos.toPageCoordinates();
        var editorPos = createEditorPagePosition(this.viewHelper.viewDomNode);
        if (pos.y < editorPos.y || pos.y > editorPos.y + editorPos.height || pos.x < editorPos.x || pos.x > editorPos.x + editorPos.width) {
            return null;
        }
        var lastViewCursorsRenderData = this.viewHelper.getLastViewCursorsRenderData();
        return this.mouseTargetFactory.createMouseTarget(lastViewCursorsRenderData, editorPos, pos, null);
    };
    MouseHandler.prototype._createMouseTarget = function (e, testEventTarget) {
        var lastViewCursorsRenderData = this.viewHelper.getLastViewCursorsRenderData();
        return this.mouseTargetFactory.createMouseTarget(lastViewCursorsRenderData, e.editorPos, e.pos, testEventTarget ? e.target : null);
    };
    MouseHandler.prototype._getMouseColumn = function (e) {
        return this.mouseTargetFactory.getMouseColumn(e.editorPos, e.pos);
    };
    MouseHandler.prototype._onContextMenu = function (e, testEventTarget) {
        this.viewController.emitContextMenu({
            event: e,
            target: this._createMouseTarget(e, testEventTarget)
        });
    };
    MouseHandler.prototype._onMouseMove = function (e) {
        if (this._mouseDownOperation.isActive()) {
            // In selection/drag operation
            return;
        }
        var actualMouseMoveTime = e.timestamp;
        if (actualMouseMoveTime < this.lastMouseLeaveTime) {
            // Due to throttling, this event occurred before the mouse left the editor, therefore ignore it.
            return;
        }
        this.viewController.emitMouseMove({
            event: e,
            target: this._createMouseTarget(e, true)
        });
    };
    MouseHandler.prototype._onMouseLeave = function (e) {
        this.lastMouseLeaveTime = (new Date()).getTime();
        this.viewController.emitMouseLeave({
            event: e,
            target: null
        });
    };
    MouseHandler.prototype._onMouseUp = function (e) {
        this.viewController.emitMouseUp({
            event: e,
            target: this._createMouseTarget(e, true)
        });
    };
    MouseHandler.prototype._onMouseDown = function (e) {
        var _this = this;
        var t = this._createMouseTarget(e, true);
        var targetIsContent = (t.type === 6 /* CONTENT_TEXT */ || t.type === 7 /* CONTENT_EMPTY */);
        var targetIsGutter = (t.type === 2 /* GUTTER_GLYPH_MARGIN */ || t.type === 3 /* GUTTER_LINE_NUMBERS */ || t.type === 4 /* GUTTER_LINE_DECORATIONS */);
        var targetIsLineNumbers = (t.type === 3 /* GUTTER_LINE_NUMBERS */);
        var selectOnLineNumbers = this._context.configuration.editor.viewInfo.selectOnLineNumbers;
        var targetIsViewZone = (t.type === 8 /* CONTENT_VIEW_ZONE */ || t.type === 5 /* GUTTER_VIEW_ZONE */);
        var targetIsWidget = (t.type === 9 /* CONTENT_WIDGET */);
        var shouldHandle = e.leftButton || e.middleButton;
        if (isMacintosh && e.leftButton && e.ctrlKey) {
            shouldHandle = false;
        }
        var focus = function () {
            // In IE11, if the focus is in the browser's address bar and
            // then you click in the editor, calling preventDefault()
            // will not move focus properly (focus remains the address bar)
            if (isIE && !_this._isFocused) {
                _this._asyncFocus.schedule();
            }
            else {
                e.preventDefault();
                _this.viewHelper.focusTextArea();
            }
        };
        if (shouldHandle && (targetIsContent || (targetIsLineNumbers && selectOnLineNumbers))) {
            focus();
            this._mouseDownOperation.start(t.type, e);
        }
        else if (targetIsGutter) {
            // Do not steal focus
            e.preventDefault();
        }
        else if (targetIsViewZone) {
            var viewZoneData = t.detail;
            if (this.viewHelper.shouldSuppressMouseDownOnViewZone(viewZoneData.viewZoneId)) {
                focus();
                this._mouseDownOperation.start(t.type, e);
                e.preventDefault();
            }
        }
        else if (targetIsWidget && this.viewHelper.shouldSuppressMouseDownOnWidget(t.detail)) {
            focus();
            e.preventDefault();
        }
        this.viewController.emitMouseDown({
            event: e,
            target: t
        });
    };
    MouseHandler.MOUSE_MOVE_MINIMUM_TIME = 100; // ms
    return MouseHandler;
}(ViewEventHandler));
var MouseDownOperation = /** @class */ (function (_super) {
    __extends$r(MouseDownOperation, _super);
    function MouseDownOperation(context, viewController, viewHelper, createMouseTarget, getMouseColumn) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._viewController = viewController;
        _this._viewHelper = viewHelper;
        _this._createMouseTarget = createMouseTarget;
        _this._getMouseColumn = getMouseColumn;
        _this._mouseMoveMonitor = _this._register(new GlobalEditorMouseMoveMonitor(_this._viewHelper.viewDomNode));
        _this._onScrollTimeout = _this._register(new TimeoutTimer());
        _this._mouseState = new MouseDownState();
        _this._currentSelection = new Selection(1, 1, 1, 1);
        _this._isActive = false;
        _this._lastMouseEvent = null;
        return _this;
    }
    MouseDownOperation.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    MouseDownOperation.prototype.isActive = function () {
        return this._isActive;
    };
    MouseDownOperation.prototype._onMouseDownThenMove = function (e) {
        this._lastMouseEvent = e;
        this._mouseState.setModifiers(e);
        var position = this._findMousePosition(e, true);
        if (!position) {
            // Ignoring because position is unknown
            return;
        }
        if (this._mouseState.isDragAndDrop) {
            this._viewController.emitMouseDrag({
                event: e,
                target: position
            });
        }
        else {
            this._dispatchMouse(position, true);
        }
    };
    MouseDownOperation.prototype.start = function (targetType, e) {
        var _this = this;
        this._lastMouseEvent = e;
        this._mouseState.setStartedOnLineNumbers(targetType === 3 /* GUTTER_LINE_NUMBERS */);
        this._mouseState.setStartButtons(e);
        this._mouseState.setModifiers(e);
        var position = this._findMousePosition(e, true);
        if (!position || !position.position) {
            // Ignoring because position is unknown
            return;
        }
        this._mouseState.trySetCount(e.detail, position.position);
        // Overwrite the detail of the MouseEvent, as it will be sent out in an event and contributions might rely on it.
        e.detail = this._mouseState.count;
        if (!this._context.configuration.editor.readOnly
            && this._context.configuration.editor.dragAndDrop
            && !this._mouseState.altKey // we don't support multiple mouse
            && e.detail < 2 // only single click on a selection can work
            && !this._isActive // the mouse is not down yet
            && !this._currentSelection.isEmpty() // we don't drag single cursor
            && (position.type === 6 /* CONTENT_TEXT */) // single click on text
            && position.position && this._currentSelection.containsPosition(position.position) // single click on a selection
        ) {
            this._mouseState.isDragAndDrop = true;
            this._isActive = true;
            this._mouseMoveMonitor.startMonitoring(createMouseMoveEventMerger(null), function (e) { return _this._onMouseDownThenMove(e); }, function () {
                var position = _this._findMousePosition(_this._lastMouseEvent, true);
                _this._viewController.emitMouseDrop({
                    event: _this._lastMouseEvent,
                    target: (position ? _this._createMouseTarget(_this._lastMouseEvent, true) : null) // Ignoring because position is unknown, e.g., Content View Zone
                });
                _this._stop();
            });
            return;
        }
        this._mouseState.isDragAndDrop = false;
        this._dispatchMouse(position, e.shiftKey);
        if (!this._isActive) {
            this._isActive = true;
            this._mouseMoveMonitor.startMonitoring(createMouseMoveEventMerger(null), function (e) { return _this._onMouseDownThenMove(e); }, function () { return _this._stop(); });
        }
    };
    MouseDownOperation.prototype._stop = function () {
        this._isActive = false;
        this._onScrollTimeout.cancel();
    };
    MouseDownOperation.prototype.onScrollChanged = function () {
        var _this = this;
        if (!this._isActive) {
            return;
        }
        this._onScrollTimeout.setIfNotSet(function () {
            if (!_this._lastMouseEvent) {
                return;
            }
            var position = _this._findMousePosition(_this._lastMouseEvent, false);
            if (!position) {
                // Ignoring because position is unknown
                return;
            }
            if (_this._mouseState.isDragAndDrop) {
                // Ignoring because users are dragging the text
                return;
            }
            _this._dispatchMouse(position, true);
        }, 10);
    };
    MouseDownOperation.prototype.onCursorStateChanged = function (e) {
        this._currentSelection = e.selections[0];
    };
    MouseDownOperation.prototype._getPositionOutsideEditor = function (e) {
        var editorContent = e.editorPos;
        var model = this._context.model;
        var viewLayout = this._context.viewLayout;
        var mouseColumn = this._getMouseColumn(e);
        if (e.posy < editorContent.y) {
            var verticalOffset = Math.max(viewLayout.getCurrentScrollTop() - (editorContent.y - e.posy), 0);
            var viewZoneData = HitTestContext.getZoneAtCoord(this._context, verticalOffset);
            if (viewZoneData) {
                var newPosition = this._helpPositionJumpOverViewZone(viewZoneData);
                if (newPosition) {
                    return new MouseTarget(null, 13 /* OUTSIDE_EDITOR */, mouseColumn, newPosition);
                }
            }
            var aboveLineNumber = viewLayout.getLineNumberAtVerticalOffset(verticalOffset);
            return new MouseTarget(null, 13 /* OUTSIDE_EDITOR */, mouseColumn, new Position(aboveLineNumber, 1));
        }
        if (e.posy > editorContent.y + editorContent.height) {
            var verticalOffset = viewLayout.getCurrentScrollTop() + (e.posy - editorContent.y);
            var viewZoneData = HitTestContext.getZoneAtCoord(this._context, verticalOffset);
            if (viewZoneData) {
                var newPosition = this._helpPositionJumpOverViewZone(viewZoneData);
                if (newPosition) {
                    return new MouseTarget(null, 13 /* OUTSIDE_EDITOR */, mouseColumn, newPosition);
                }
            }
            var belowLineNumber = viewLayout.getLineNumberAtVerticalOffset(verticalOffset);
            return new MouseTarget(null, 13 /* OUTSIDE_EDITOR */, mouseColumn, new Position(belowLineNumber, model.getLineMaxColumn(belowLineNumber)));
        }
        var possibleLineNumber = viewLayout.getLineNumberAtVerticalOffset(viewLayout.getCurrentScrollTop() + (e.posy - editorContent.y));
        if (e.posx < editorContent.x) {
            return new MouseTarget(null, 13 /* OUTSIDE_EDITOR */, mouseColumn, new Position(possibleLineNumber, 1));
        }
        if (e.posx > editorContent.x + editorContent.width) {
            return new MouseTarget(null, 13 /* OUTSIDE_EDITOR */, mouseColumn, new Position(possibleLineNumber, model.getLineMaxColumn(possibleLineNumber)));
        }
        return null;
    };
    MouseDownOperation.prototype._findMousePosition = function (e, testEventTarget) {
        var positionOutsideEditor = this._getPositionOutsideEditor(e);
        if (positionOutsideEditor) {
            return positionOutsideEditor;
        }
        var t = this._createMouseTarget(e, testEventTarget);
        var hintedPosition = t.position;
        if (!hintedPosition) {
            return null;
        }
        if (t.type === 8 /* CONTENT_VIEW_ZONE */ || t.type === 5 /* GUTTER_VIEW_ZONE */) {
            var newPosition = this._helpPositionJumpOverViewZone(t.detail);
            if (newPosition) {
                return new MouseTarget(t.element, t.type, t.mouseColumn, newPosition, null, t.detail);
            }
        }
        return t;
    };
    MouseDownOperation.prototype._helpPositionJumpOverViewZone = function (viewZoneData) {
        // Force position on view zones to go above or below depending on where selection started from
        var selectionStart = new Position(this._currentSelection.selectionStartLineNumber, this._currentSelection.selectionStartColumn);
        var positionBefore = viewZoneData.positionBefore;
        var positionAfter = viewZoneData.positionAfter;
        if (positionBefore && positionAfter) {
            if (positionBefore.isBefore(selectionStart)) {
                return positionBefore;
            }
            else {
                return positionAfter;
            }
        }
        return null;
    };
    MouseDownOperation.prototype._dispatchMouse = function (position, inSelectionMode) {
        if (!position.position) {
            return;
        }
        this._viewController.dispatchMouse({
            position: position.position,
            mouseColumn: position.mouseColumn,
            startedOnLineNumbers: this._mouseState.startedOnLineNumbers,
            inSelectionMode: inSelectionMode,
            mouseDownCount: this._mouseState.count,
            altKey: this._mouseState.altKey,
            ctrlKey: this._mouseState.ctrlKey,
            metaKey: this._mouseState.metaKey,
            shiftKey: this._mouseState.shiftKey,
            leftButton: this._mouseState.leftButton,
            middleButton: this._mouseState.middleButton,
        });
    };
    return MouseDownOperation;
}(Disposable));
var MouseDownState = /** @class */ (function () {
    function MouseDownState() {
        this._altKey = false;
        this._ctrlKey = false;
        this._metaKey = false;
        this._shiftKey = false;
        this._leftButton = false;
        this._middleButton = false;
        this._startedOnLineNumbers = false;
        this._lastMouseDownPosition = null;
        this._lastMouseDownPositionEqualCount = 0;
        this._lastMouseDownCount = 0;
        this._lastSetMouseDownCountTime = 0;
        this.isDragAndDrop = false;
    }
    Object.defineProperty(MouseDownState.prototype, "altKey", {
        get: function () { return this._altKey; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MouseDownState.prototype, "ctrlKey", {
        get: function () { return this._ctrlKey; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MouseDownState.prototype, "metaKey", {
        get: function () { return this._metaKey; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MouseDownState.prototype, "shiftKey", {
        get: function () { return this._shiftKey; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MouseDownState.prototype, "leftButton", {
        get: function () { return this._leftButton; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MouseDownState.prototype, "middleButton", {
        get: function () { return this._middleButton; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MouseDownState.prototype, "startedOnLineNumbers", {
        get: function () { return this._startedOnLineNumbers; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MouseDownState.prototype, "count", {
        get: function () {
            return this._lastMouseDownCount;
        },
        enumerable: true,
        configurable: true
    });
    MouseDownState.prototype.setModifiers = function (source) {
        this._altKey = source.altKey;
        this._ctrlKey = source.ctrlKey;
        this._metaKey = source.metaKey;
        this._shiftKey = source.shiftKey;
    };
    MouseDownState.prototype.setStartButtons = function (source) {
        this._leftButton = source.leftButton;
        this._middleButton = source.middleButton;
    };
    MouseDownState.prototype.setStartedOnLineNumbers = function (startedOnLineNumbers) {
        this._startedOnLineNumbers = startedOnLineNumbers;
    };
    MouseDownState.prototype.trySetCount = function (setMouseDownCount, newMouseDownPosition) {
        // a. Invalidate multiple clicking if too much time has passed (will be hit by IE because the detail field of mouse events contains garbage in IE10)
        var currentTime = (new Date()).getTime();
        if (currentTime - this._lastSetMouseDownCountTime > MouseDownState.CLEAR_MOUSE_DOWN_COUNT_TIME) {
            setMouseDownCount = 1;
        }
        this._lastSetMouseDownCountTime = currentTime;
        // b. Ensure that we don't jump from single click to triple click in one go (will be hit by IE because the detail field of mouse events contains garbage in IE10)
        if (setMouseDownCount > this._lastMouseDownCount + 1) {
            setMouseDownCount = this._lastMouseDownCount + 1;
        }
        // c. Invalidate multiple clicking if the logical position is different
        if (this._lastMouseDownPosition && this._lastMouseDownPosition.equals(newMouseDownPosition)) {
            this._lastMouseDownPositionEqualCount++;
        }
        else {
            this._lastMouseDownPositionEqualCount = 1;
        }
        this._lastMouseDownPosition = newMouseDownPosition;
        // Finally set the lastMouseDownCount
        this._lastMouseDownCount = Math.min(setMouseDownCount, this._lastMouseDownPositionEqualCount);
    };
    MouseDownState.CLEAR_MOUSE_DOWN_COUNT_TIME = 400; // ms
    return MouseDownState;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$s = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function gestureChangeEventMerger(lastEvent, currentEvent) {
    var r = {
        translationY: currentEvent.translationY,
        translationX: currentEvent.translationX
    };
    if (lastEvent) {
        r.translationY += lastEvent.translationY;
        r.translationX += lastEvent.translationX;
    }
    return r;
}
/**
 * Basically IE10 and IE11
 */
var MsPointerHandler = /** @class */ (function (_super) {
    __extends$s(MsPointerHandler, _super);
    function MsPointerHandler(context, viewController, viewHelper) {
        var _this = _super.call(this, context, viewController, viewHelper) || this;
        _this.viewHelper.linesContentDomNode.style.msTouchAction = 'none';
        _this.viewHelper.linesContentDomNode.style.msContentZooming = 'none';
        // TODO@Alex -> this expects that the view is added in 100 ms, might not be the case
        // This handler should be added when the dom node is in the dom tree
        _this._installGestureHandlerTimeout = window.setTimeout(function () {
            _this._installGestureHandlerTimeout = -1;
            if (window.MSGesture) {
                var touchGesture_1 = new MSGesture();
                var penGesture_1 = new MSGesture();
                touchGesture_1.target = _this.viewHelper.linesContentDomNode;
                penGesture_1.target = _this.viewHelper.linesContentDomNode;
                _this.viewHelper.linesContentDomNode.addEventListener('MSPointerDown', function (e) {
                    // Circumvent IE11 breaking change in e.pointerType & TypeScript's stale definitions
                    var pointerType = e.pointerType;
                    if (pointerType === (e.MSPOINTER_TYPE_MOUSE || 'mouse')) {
                        _this._lastPointerType = 'mouse';
                        return;
                    }
                    else if (pointerType === (e.MSPOINTER_TYPE_TOUCH || 'touch')) {
                        _this._lastPointerType = 'touch';
                        touchGesture_1.addPointer(e.pointerId);
                    }
                    else {
                        _this._lastPointerType = 'pen';
                        penGesture_1.addPointer(e.pointerId);
                    }
                });
                _this._register(addDisposableThrottledListener(_this.viewHelper.linesContentDomNode, 'MSGestureChange', function (e) { return _this._onGestureChange(e); }, gestureChangeEventMerger));
                _this._register(addDisposableListener(_this.viewHelper.linesContentDomNode, 'MSGestureTap', function (e) { return _this._onCaptureGestureTap(e); }, true));
            }
        }, 100);
        _this._lastPointerType = 'mouse';
        return _this;
    }
    MsPointerHandler.prototype._onMouseDown = function (e) {
        if (this._lastPointerType === 'mouse') {
            _super.prototype._onMouseDown.call(this, e);
        }
    };
    MsPointerHandler.prototype._onCaptureGestureTap = function (rawEvent) {
        var _this = this;
        var e = new EditorMouseEvent(rawEvent, this.viewHelper.viewDomNode);
        var t = this._createMouseTarget(e, false);
        if (t.position) {
            this.viewController.moveTo(t.position);
        }
        // IE does not want to focus when coming in from the browser's address bar
        if (e.browserEvent.fromElement) {
            e.preventDefault();
            this.viewHelper.focusTextArea();
        }
        else {
            // TODO@Alex -> cancel this is focus is lost
            setTimeout(function () {
                _this.viewHelper.focusTextArea();
            });
        }
    };
    MsPointerHandler.prototype._onGestureChange = function (e) {
        this._context.viewLayout.deltaScrollNow(-e.translationX, -e.translationY);
    };
    MsPointerHandler.prototype.dispose = function () {
        window.clearTimeout(this._installGestureHandlerTimeout);
        _super.prototype.dispose.call(this);
    };
    return MsPointerHandler;
}(MouseHandler));
/**
 * Basically Edge but should be modified to handle any pointerEnabled, even without support of MSGesture
 */
var StandardPointerHandler = /** @class */ (function (_super) {
    __extends$s(StandardPointerHandler, _super);
    function StandardPointerHandler(context, viewController, viewHelper) {
        var _this = _super.call(this, context, viewController, viewHelper) || this;
        _this.viewHelper.linesContentDomNode.style.touchAction = 'none';
        // TODO@Alex -> this expects that the view is added in 100 ms, might not be the case
        // This handler should be added when the dom node is in the dom tree
        _this._installGestureHandlerTimeout = window.setTimeout(function () {
            _this._installGestureHandlerTimeout = -1;
            // TODO@Alex: replace the usage of MSGesture here with something that works across all browsers
            if (window.MSGesture) {
                var touchGesture_2 = new MSGesture();
                var penGesture_2 = new MSGesture();
                touchGesture_2.target = _this.viewHelper.linesContentDomNode;
                penGesture_2.target = _this.viewHelper.linesContentDomNode;
                _this.viewHelper.linesContentDomNode.addEventListener('pointerdown', function (e) {
                    var pointerType = e.pointerType;
                    if (pointerType === 'mouse') {
                        _this._lastPointerType = 'mouse';
                        return;
                    }
                    else if (pointerType === 'touch') {
                        _this._lastPointerType = 'touch';
                        touchGesture_2.addPointer(e.pointerId);
                    }
                    else {
                        _this._lastPointerType = 'pen';
                        penGesture_2.addPointer(e.pointerId);
                    }
                });
                _this._register(addDisposableThrottledListener(_this.viewHelper.linesContentDomNode, 'MSGestureChange', function (e) { return _this._onGestureChange(e); }, gestureChangeEventMerger));
                _this._register(addDisposableListener(_this.viewHelper.linesContentDomNode, 'MSGestureTap', function (e) { return _this._onCaptureGestureTap(e); }, true));
            }
        }, 100);
        _this._lastPointerType = 'mouse';
        return _this;
    }
    StandardPointerHandler.prototype._onMouseDown = function (e) {
        if (this._lastPointerType === 'mouse') {
            _super.prototype._onMouseDown.call(this, e);
        }
    };
    StandardPointerHandler.prototype._onCaptureGestureTap = function (rawEvent) {
        var _this = this;
        var e = new EditorMouseEvent(rawEvent, this.viewHelper.viewDomNode);
        var t = this._createMouseTarget(e, false);
        if (t.position) {
            this.viewController.moveTo(t.position);
        }
        // IE does not want to focus when coming in from the browser's address bar
        if (e.browserEvent.fromElement) {
            e.preventDefault();
            this.viewHelper.focusTextArea();
        }
        else {
            // TODO@Alex -> cancel this is focus is lost
            setTimeout(function () {
                _this.viewHelper.focusTextArea();
            });
        }
    };
    StandardPointerHandler.prototype._onGestureChange = function (e) {
        this._context.viewLayout.deltaScrollNow(-e.translationX, -e.translationY);
    };
    StandardPointerHandler.prototype.dispose = function () {
        window.clearTimeout(this._installGestureHandlerTimeout);
        _super.prototype.dispose.call(this);
    };
    return StandardPointerHandler;
}(MouseHandler));
var TouchHandler = /** @class */ (function (_super) {
    __extends$s(TouchHandler, _super);
    function TouchHandler(context, viewController, viewHelper) {
        var _this = _super.call(this, context, viewController, viewHelper) || this;
        Gesture.addTarget(_this.viewHelper.linesContentDomNode);
        _this._register(addDisposableListener(_this.viewHelper.linesContentDomNode, EventType$1.Tap, function (e) { return _this.onTap(e); }));
        _this._register(addDisposableListener(_this.viewHelper.linesContentDomNode, EventType$1.Change, function (e) { return _this.onChange(e); }));
        _this._register(addDisposableListener(_this.viewHelper.linesContentDomNode, EventType$1.Contextmenu, function (e) { return _this._onContextMenu(new EditorMouseEvent(e, _this.viewHelper.viewDomNode), false); }));
        return _this;
    }
    TouchHandler.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    TouchHandler.prototype.onTap = function (event) {
        event.preventDefault();
        this.viewHelper.focusTextArea();
        var target = this._createMouseTarget(new EditorMouseEvent(event, this.viewHelper.viewDomNode), false);
        if (target.position) {
            this.viewController.moveTo(target.position);
        }
    };
    TouchHandler.prototype.onChange = function (e) {
        this._context.viewLayout.deltaScrollNow(-e.translationX, -e.translationY);
    };
    return TouchHandler;
}(MouseHandler));
var PointerHandler = /** @class */ (function () {
    function PointerHandler(context, viewController, viewHelper) {
        if (window.navigator.msPointerEnabled) {
            this.handler = new MsPointerHandler(context, viewController, viewHelper);
        }
        else if (window.TouchEvent) {
            this.handler = new TouchHandler(context, viewController, viewHelper);
        }
        else if (window.navigator.pointerEnabled || window.PointerEvent) {
            this.handler = new StandardPointerHandler(context, viewController, viewHelper);
        }
        else {
            this.handler = new MouseHandler(context, viewController, viewHelper);
        }
    }
    PointerHandler.prototype.getTargetAtClientPoint = function (clientX, clientY) {
        return this.handler.getTargetAtClientPoint(clientX, clientY);
    };
    PointerHandler.prototype.dispose = function () {
        this.handler.dispose();
    };
    return PointerHandler;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var TextAreaState = /** @class */ (function () {
    function TextAreaState(value, selectionStart, selectionEnd, selectionStartPosition, selectionEndPosition) {
        this.value = value;
        this.selectionStart = selectionStart;
        this.selectionEnd = selectionEnd;
        this.selectionStartPosition = selectionStartPosition;
        this.selectionEndPosition = selectionEndPosition;
    }
    TextAreaState.prototype.toString = function () {
        return '[ <' + this.value + '>, selectionStart: ' + this.selectionStart + ', selectionEnd: ' + this.selectionEnd + ']';
    };
    TextAreaState.readFromTextArea = function (textArea) {
        return new TextAreaState(textArea.getValue(), textArea.getSelectionStart(), textArea.getSelectionEnd(), null, null);
    };
    TextAreaState.prototype.collapseSelection = function () {
        return new TextAreaState(this.value, this.value.length, this.value.length, null, null);
    };
    TextAreaState.prototype.writeToTextArea = function (reason, textArea, select) {
        // console.log(Date.now() + ': writeToTextArea ' + reason + ': ' + this.toString());
        textArea.setValue(reason, this.value);
        if (select) {
            textArea.setSelectionRange(reason, this.selectionStart, this.selectionEnd);
        }
    };
    TextAreaState.prototype.deduceEditorPosition = function (offset) {
        if (offset <= this.selectionStart) {
            var str = this.value.substring(offset, this.selectionStart);
            return this._finishDeduceEditorPosition(this.selectionStartPosition, str, -1);
        }
        if (offset >= this.selectionEnd) {
            var str = this.value.substring(this.selectionEnd, offset);
            return this._finishDeduceEditorPosition(this.selectionEndPosition, str, 1);
        }
        var str1 = this.value.substring(this.selectionStart, offset);
        if (str1.indexOf(String.fromCharCode(8230)) === -1) {
            return this._finishDeduceEditorPosition(this.selectionStartPosition, str1, 1);
        }
        var str2 = this.value.substring(offset, this.selectionEnd);
        return this._finishDeduceEditorPosition(this.selectionEndPosition, str2, -1);
    };
    TextAreaState.prototype._finishDeduceEditorPosition = function (anchor, deltaText, signum) {
        var lineFeedCnt = 0;
        var lastLineFeedIndex = -1;
        while ((lastLineFeedIndex = deltaText.indexOf('\n', lastLineFeedIndex + 1)) !== -1) {
            lineFeedCnt++;
        }
        return [anchor, signum * deltaText.length, lineFeedCnt];
    };
    TextAreaState.selectedText = function (text) {
        return new TextAreaState(text, 0, text.length, null, null);
    };
    TextAreaState.deduceInput = function (previousState, currentState, couldBeEmojiInput, couldBeTypingAtOffset0) {
        if (!previousState) {
            // This is the EMPTY state
            return {
                text: '',
                replaceCharCnt: 0
            };
        }
        // console.log('------------------------deduceInput');
        // console.log('PREVIOUS STATE: ' + previousState.toString());
        // console.log('CURRENT STATE: ' + currentState.toString());
        var previousValue = previousState.value;
        var previousSelectionStart = previousState.selectionStart;
        var previousSelectionEnd = previousState.selectionEnd;
        var currentValue = currentState.value;
        var currentSelectionStart = currentState.selectionStart;
        var currentSelectionEnd = currentState.selectionEnd;
        if (couldBeTypingAtOffset0 && previousValue.length > 0 && previousSelectionStart === previousSelectionEnd && currentSelectionStart === currentSelectionEnd) {
            // See https://github.com/Microsoft/vscode/issues/42251
            // where typing always happens at offset 0 in the textarea
            // when using a custom title area in OSX and moving the window
            if (!startsWith(currentValue, previousValue) && endsWith(currentValue, previousValue)) {
                // Looks like something was typed at offset 0
                // ==> pretend we placed the cursor at offset 0 to begin with...
                previousSelectionStart = 0;
                previousSelectionEnd = 0;
            }
        }
        // Strip the previous suffix from the value (without interfering with the current selection)
        var previousSuffix = previousValue.substring(previousSelectionEnd);
        var currentSuffix = currentValue.substring(currentSelectionEnd);
        var suffixLength = commonSuffixLength(previousSuffix, currentSuffix);
        currentValue = currentValue.substring(0, currentValue.length - suffixLength);
        previousValue = previousValue.substring(0, previousValue.length - suffixLength);
        var previousPrefix = previousValue.substring(0, previousSelectionStart);
        var currentPrefix = currentValue.substring(0, currentSelectionStart);
        var prefixLength = commonPrefixLength(previousPrefix, currentPrefix);
        currentValue = currentValue.substring(prefixLength);
        previousValue = previousValue.substring(prefixLength);
        currentSelectionStart -= prefixLength;
        previousSelectionStart -= prefixLength;
        currentSelectionEnd -= prefixLength;
        previousSelectionEnd -= prefixLength;
        // console.log('AFTER DIFFING PREVIOUS STATE: <' + previousValue + '>, selectionStart: ' + previousSelectionStart + ', selectionEnd: ' + previousSelectionEnd);
        // console.log('AFTER DIFFING CURRENT STATE: <' + currentValue + '>, selectionStart: ' + currentSelectionStart + ', selectionEnd: ' + currentSelectionEnd);
        if (couldBeEmojiInput && currentSelectionStart === currentSelectionEnd && previousValue.length > 0) {
            // on OSX, emojis from the emoji picker are inserted at random locations
            // the only hints we can use is that the selection is immediately after the inserted emoji
            // and that none of the old text has been deleted
            var potentialEmojiInput = null;
            if (currentSelectionStart === currentValue.length) {
                // emoji potentially inserted "somewhere" after the previous selection => it should appear at the end of `currentValue`
                if (startsWith(currentValue, previousValue)) {
                    // only if all of the old text is accounted for
                    potentialEmojiInput = currentValue.substring(previousValue.length);
                }
            }
            else {
                // emoji potentially inserted "somewhere" before the previous selection => it should appear at the start of `currentValue`
                if (endsWith(currentValue, previousValue)) {
                    // only if all of the old text is accounted for
                    potentialEmojiInput = currentValue.substring(0, currentValue.length - previousValue.length);
                }
            }
            if (potentialEmojiInput !== null && potentialEmojiInput.length > 0) {
                // now we check that this is indeed an emoji
                // emojis can grow quite long, so a length check is of no help
                // e.g. 1F3F4 E0067 E0062 E0065 E006E E0067 E007F  ; fully-qualified     # 🏴󠁧󠁢󠁥󠁮󠁧󠁿 England
                // Oftentimes, emojis use Variation Selector-16 (U+FE0F), so that is a good hint
                // http://emojipedia.org/variation-selector-16/
                // > An invisible codepoint which specifies that the preceding character
                // > should be displayed with emoji presentation. Only required if the
                // > preceding character defaults to text presentation.
                if (/\uFE0F/.test(potentialEmojiInput) || containsEmoji(potentialEmojiInput)) {
                    return {
                        text: potentialEmojiInput,
                        replaceCharCnt: 0
                    };
                }
            }
        }
        if (currentSelectionStart === currentSelectionEnd) {
            // composition accept case (noticed in FF + Japanese)
            // [blahblah] => blahblah|
            if (previousValue === currentValue
                && previousSelectionStart === 0
                && previousSelectionEnd === previousValue.length
                && currentSelectionStart === currentValue.length
                && currentValue.indexOf('\n') === -1) {
                if (containsFullWidthCharacter(currentValue)) {
                    return {
                        text: '',
                        replaceCharCnt: 0
                    };
                }
            }
            // no current selection
            var replacePreviousCharacters_1 = (previousPrefix.length - prefixLength);
            // console.log('REMOVE PREVIOUS: ' + (previousPrefix.length - prefixLength) + ' chars');
            return {
                text: currentValue,
                replaceCharCnt: replacePreviousCharacters_1
            };
        }
        // there is a current selection => composition case
        var replacePreviousCharacters = previousSelectionEnd - previousSelectionStart;
        return {
            text: currentValue,
            replaceCharCnt: replacePreviousCharacters
        };
    };
    TextAreaState.EMPTY = new TextAreaState('', 0, 0, null, null);
    return TextAreaState;
}());
var PagedScreenReaderStrategy = /** @class */ (function () {
    function PagedScreenReaderStrategy() {
    }
    PagedScreenReaderStrategy._getPageOfLine = function (lineNumber) {
        return Math.floor((lineNumber - 1) / PagedScreenReaderStrategy._LINES_PER_PAGE);
    };
    PagedScreenReaderStrategy._getRangeForPage = function (page) {
        var offset = page * PagedScreenReaderStrategy._LINES_PER_PAGE;
        var startLineNumber = offset + 1;
        var endLineNumber = offset + PagedScreenReaderStrategy._LINES_PER_PAGE;
        return new Range(startLineNumber, 1, endLineNumber + 1, 1);
    };
    PagedScreenReaderStrategy.fromEditorSelection = function (previousState, model, selection, trimLongText) {
        var selectionStartPage = PagedScreenReaderStrategy._getPageOfLine(selection.startLineNumber);
        var selectionStartPageRange = PagedScreenReaderStrategy._getRangeForPage(selectionStartPage);
        var selectionEndPage = PagedScreenReaderStrategy._getPageOfLine(selection.endLineNumber);
        var selectionEndPageRange = PagedScreenReaderStrategy._getRangeForPage(selectionEndPage);
        var pretextRange = selectionStartPageRange.intersectRanges(new Range(1, 1, selection.startLineNumber, selection.startColumn));
        var pretext = model.getValueInRange(pretextRange, 1 /* LF */);
        var lastLine = model.getLineCount();
        var lastLineMaxColumn = model.getLineMaxColumn(lastLine);
        var posttextRange = selectionEndPageRange.intersectRanges(new Range(selection.endLineNumber, selection.endColumn, lastLine, lastLineMaxColumn));
        var posttext = model.getValueInRange(posttextRange, 1 /* LF */);
        var text;
        if (selectionStartPage === selectionEndPage || selectionStartPage + 1 === selectionEndPage) {
            // take full selection
            text = model.getValueInRange(selection, 1 /* LF */);
        }
        else {
            var selectionRange1 = selectionStartPageRange.intersectRanges(selection);
            var selectionRange2 = selectionEndPageRange.intersectRanges(selection);
            text = (model.getValueInRange(selectionRange1, 1 /* LF */)
                + String.fromCharCode(8230)
                + model.getValueInRange(selectionRange2, 1 /* LF */));
        }
        // Chromium handles very poorly text even of a few thousand chars
        // Cut text to avoid stalling the entire UI
        if (trimLongText) {
            var LIMIT_CHARS = 500;
            if (pretext.length > LIMIT_CHARS) {
                pretext = pretext.substring(pretext.length - LIMIT_CHARS, pretext.length);
            }
            if (posttext.length > LIMIT_CHARS) {
                posttext = posttext.substring(0, LIMIT_CHARS);
            }
            if (text.length > 2 * LIMIT_CHARS) {
                text = text.substring(0, LIMIT_CHARS) + String.fromCharCode(8230) + text.substring(text.length - LIMIT_CHARS, text.length);
            }
        }
        return new TextAreaState(pretext + text + posttext, pretext.length, pretext.length + text.length, new Position(selection.startLineNumber, selection.startColumn), new Position(selection.endLineNumber, selection.endColumn));
    };
    PagedScreenReaderStrategy._LINES_PER_PAGE = 10;
    return PagedScreenReaderStrategy;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$t = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CopyOptions = {
    forceCopyWithSyntaxHighlighting: false
};
/**
 * Writes screen reader content to the textarea and is able to analyze its input events to generate:
 *  - onCut
 *  - onPaste
 *  - onType
 *
 * Composition events are generated for presentation purposes (composition input is reflected in onType).
 */
var TextAreaInput = /** @class */ (function (_super) {
    __extends$t(TextAreaInput, _super);
    function TextAreaInput(host, textArea) {
        var _this = _super.call(this) || this;
        _this._onFocus = _this._register(new Emitter());
        _this.onFocus = _this._onFocus.event;
        _this._onBlur = _this._register(new Emitter());
        _this.onBlur = _this._onBlur.event;
        _this._onKeyDown = _this._register(new Emitter());
        _this.onKeyDown = _this._onKeyDown.event;
        _this._onKeyUp = _this._register(new Emitter());
        _this.onKeyUp = _this._onKeyUp.event;
        _this._onCut = _this._register(new Emitter());
        _this.onCut = _this._onCut.event;
        _this._onPaste = _this._register(new Emitter());
        _this.onPaste = _this._onPaste.event;
        _this._onType = _this._register(new Emitter());
        _this.onType = _this._onType.event;
        _this._onCompositionStart = _this._register(new Emitter());
        _this.onCompositionStart = _this._onCompositionStart.event;
        _this._onCompositionUpdate = _this._register(new Emitter());
        _this.onCompositionUpdate = _this._onCompositionUpdate.event;
        _this._onCompositionEnd = _this._register(new Emitter());
        _this.onCompositionEnd = _this._onCompositionEnd.event;
        _this._onSelectionChangeRequest = _this._register(new Emitter());
        _this.onSelectionChangeRequest = _this._onSelectionChangeRequest.event;
        _this._host = host;
        _this._textArea = _this._register(new TextAreaWrapper(textArea));
        _this._lastTextAreaEvent = 0 /* none */;
        _this._asyncTriggerCut = _this._register(new RunOnceScheduler(function () { return _this._onCut.fire(); }, 0));
        _this._textAreaState = TextAreaState.EMPTY;
        _this.writeScreenReaderContent('ctor');
        _this._hasFocus = false;
        _this._isDoingComposition = false;
        _this._nextCommand = 0 /* Type */;
        _this._register(addStandardDisposableListener(textArea.domNode, 'keydown', function (e) {
            if (_this._isDoingComposition &&
                (e.keyCode === 109 /* KEY_IN_COMPOSITION */ || e.keyCode === 1 /* Backspace */)) {
                // Stop propagation for keyDown events if the IME is processing key input
                e.stopPropagation();
            }
            if (e.equals(9 /* Escape */)) {
                // Prevent default always for `Esc`, otherwise it will generate a keypress
                // See https://msdn.microsoft.com/en-us/library/ie/ms536939(v=vs.85).aspx
                e.preventDefault();
            }
            _this._onKeyDown.fire(e);
        }));
        _this._register(addStandardDisposableListener(textArea.domNode, 'keyup', function (e) {
            _this._onKeyUp.fire(e);
        }));
        _this._register(addDisposableListener(textArea.domNode, 'compositionstart', function (e) {
            _this._lastTextAreaEvent = 1 /* compositionstart */;
            if (_this._isDoingComposition) {
                return;
            }
            _this._isDoingComposition = true;
            // In IE we cannot set .value when handling 'compositionstart' because the entire composition will get canceled.
            if (!isEdgeOrIE) {
                _this._setAndWriteTextAreaState('compositionstart', TextAreaState.EMPTY);
            }
            _this._onCompositionStart.fire();
        }));
        /**
         * Deduce the typed input from a text area's value and the last observed state.
         */
        var deduceInputFromTextAreaValue = function (couldBeEmojiInput, couldBeTypingAtOffset0) {
            var oldState = _this._textAreaState;
            var newState = TextAreaState.readFromTextArea(_this._textArea);
            return [newState, TextAreaState.deduceInput(oldState, newState, couldBeEmojiInput, couldBeTypingAtOffset0)];
        };
        /**
         * Deduce the composition input from a string.
         */
        var deduceComposition = function (text) {
            var oldState = _this._textAreaState;
            var newState = TextAreaState.selectedText(text);
            var typeInput = {
                text: newState.value,
                replaceCharCnt: oldState.selectionEnd - oldState.selectionStart
            };
            return [newState, typeInput];
        };
        var compositionDataInValid = function (locale) {
            // https://github.com/Microsoft/monaco-editor/issues/339
            // Multi-part Japanese compositions reset cursor in Edge/IE, Chinese and Korean IME don't have this issue.
            // The reason that we can't use this path for all CJK IME is IE and Edge behave differently when handling Korean IME,
            // which breaks this path of code.
            if (isEdgeOrIE && locale === 'ja') {
                return true;
            }
            // https://github.com/Microsoft/monaco-editor/issues/545
            // On IE11, we can't trust composition data when typing Chinese as IE11 doesn't emit correct
            // events when users type numbers in IME.
            // Chinese: zh-Hans-CN, zh-Hans-SG, zh-Hant-TW, zh-Hant-HK
            if (isIE && locale.indexOf('zh-Han') === 0) {
                return true;
            }
            return false;
        };
        _this._register(addDisposableListener(textArea.domNode, 'compositionupdate', function (e) {
            _this._lastTextAreaEvent = 2 /* compositionupdate */;
            if (compositionDataInValid(e.locale)) {
                var _a = deduceInputFromTextAreaValue(/*couldBeEmojiInput*/ false, /*couldBeTypingAtOffset0*/ false), newState_1 = _a[0], typeInput_1 = _a[1];
                _this._textAreaState = newState_1;
                _this._onType.fire(typeInput_1);
                _this._onCompositionUpdate.fire(e);
                return;
            }
            var _b = deduceComposition(e.data), newState = _b[0], typeInput = _b[1];
            _this._textAreaState = newState;
            _this._onType.fire(typeInput);
            _this._onCompositionUpdate.fire(e);
        }));
        _this._register(addDisposableListener(textArea.domNode, 'compositionend', function (e) {
            _this._lastTextAreaEvent = 3 /* compositionend */;
            if (compositionDataInValid(e.locale)) {
                // https://github.com/Microsoft/monaco-editor/issues/339
                var _a = deduceInputFromTextAreaValue(/*couldBeEmojiInput*/ false, /*couldBeTypingAtOffset0*/ false), newState = _a[0], typeInput = _a[1];
                _this._textAreaState = newState;
                _this._onType.fire(typeInput);
            }
            else {
                var _b = deduceComposition(e.data), newState = _b[0], typeInput = _b[1];
                _this._textAreaState = newState;
                _this._onType.fire(typeInput);
            }
            // Due to isEdgeOrIE (where the textarea was not cleared initially) and isChrome (the textarea is not updated correctly when composition ends)
            // we cannot assume the text at the end consists only of the composited text
            if (isEdgeOrIE || isChrome) {
                _this._textAreaState = TextAreaState.readFromTextArea(_this._textArea);
            }
            if (!_this._isDoingComposition) {
                return;
            }
            _this._isDoingComposition = false;
            _this._onCompositionEnd.fire();
        }));
        _this._register(addDisposableListener(textArea.domNode, 'input', function () {
            // We want to find out if this is the first `input` after a `focus`.
            var previousEventWasFocus = (_this._lastTextAreaEvent === 8 /* focus */);
            _this._lastTextAreaEvent = 4 /* input */;
            // Pretend here we touched the text area, as the `input` event will most likely
            // result in a `selectionchange` event which we want to ignore
            _this._textArea.setIgnoreSelectionChangeTime('received input event');
            if (_this._isDoingComposition) {
                return;
            }
            var _a = deduceInputFromTextAreaValue(/*couldBeEmojiInput*/ isMacintosh, /*couldBeTypingAtOffset0*/ previousEventWasFocus && isMacintosh), newState = _a[0], typeInput = _a[1];
            if (typeInput.replaceCharCnt === 0 && typeInput.text.length === 1 && isHighSurrogate(typeInput.text.charCodeAt(0))) {
                // Ignore invalid input but keep it around for next time
                return;
            }
            _this._textAreaState = newState;
            if (_this._nextCommand === 0 /* Type */) {
                if (typeInput.text !== '') {
                    _this._onType.fire(typeInput);
                }
            }
            else {
                if (typeInput.text !== '') {
                    _this._onPaste.fire({
                        text: typeInput.text
                    });
                }
                _this._nextCommand = 0 /* Type */;
            }
        }));
        // --- Clipboard operations
        _this._register(addDisposableListener(textArea.domNode, 'cut', function (e) {
            _this._lastTextAreaEvent = 5 /* cut */;
            // Pretend here we touched the text area, as the `cut` event will most likely
            // result in a `selectionchange` event which we want to ignore
            _this._textArea.setIgnoreSelectionChangeTime('received cut event');
            _this._ensureClipboardGetsEditorSelection(e);
            _this._asyncTriggerCut.schedule();
        }));
        _this._register(addDisposableListener(textArea.domNode, 'copy', function (e) {
            _this._lastTextAreaEvent = 6 /* copy */;
            _this._ensureClipboardGetsEditorSelection(e);
        }));
        _this._register(addDisposableListener(textArea.domNode, 'paste', function (e) {
            _this._lastTextAreaEvent = 7 /* paste */;
            // Pretend here we touched the text area, as the `paste` event will most likely
            // result in a `selectionchange` event which we want to ignore
            _this._textArea.setIgnoreSelectionChangeTime('received paste event');
            if (ClipboardEventUtils.canUseTextData(e)) {
                var pastePlainText = ClipboardEventUtils.getTextData(e);
                if (pastePlainText !== '') {
                    _this._onPaste.fire({
                        text: pastePlainText
                    });
                }
            }
            else {
                if (_this._textArea.getSelectionStart() !== _this._textArea.getSelectionEnd()) {
                    // Clean up the textarea, to get a clean paste
                    _this._setAndWriteTextAreaState('paste', TextAreaState.EMPTY);
                }
                _this._nextCommand = 1 /* Paste */;
            }
        }));
        _this._register(addDisposableListener(textArea.domNode, 'focus', function () {
            _this._lastTextAreaEvent = 8 /* focus */;
            _this._setHasFocus(true);
        }));
        _this._register(addDisposableListener(textArea.domNode, 'blur', function () {
            _this._lastTextAreaEvent = 9 /* blur */;
            _this._setHasFocus(false);
        }));
        // See https://github.com/Microsoft/vscode/issues/27216
        // When using a Braille display, it is possible for users to reposition the
        // system caret. This is reflected in Chrome as a `selectionchange` event.
        //
        // The `selectionchange` event appears to be emitted under numerous other circumstances,
        // so it is quite a challenge to distinguish a `selectionchange` coming in from a user
        // using a Braille display from all the other cases.
        //
        // The problems with the `selectionchange` event are:
        //  * the event is emitted when the textarea is focused programmatically -- textarea.focus()
        //  * the event is emitted when the selection is changed in the textarea programatically -- textarea.setSelectionRange(...)
        //  * the event is emitted when the value of the textarea is changed programmatically -- textarea.value = '...'
        //  * the event is emitted when tabbing into the textarea
        //  * the event is emitted asynchronously (sometimes with a delay as high as a few tens of ms)
        //  * the event sometimes comes in bursts for a single logical textarea operation
        // `selectionchange` events often come multiple times for a single logical change
        // so throttle multiple `selectionchange` events that burst in a short period of time.
        var previousSelectionChangeEventTime = 0;
        _this._register(addDisposableListener(document, 'selectionchange', function (e) {
            if (!_this._hasFocus) {
                return;
            }
            if (_this._isDoingComposition) {
                return;
            }
            if (!isChrome || !isWindows) {
                // Support only for Chrome on Windows until testing happens on other browsers + OS configurations
                return;
            }
            var now = Date.now();
            var delta1 = now - previousSelectionChangeEventTime;
            previousSelectionChangeEventTime = now;
            if (delta1 < 5) {
                // received another `selectionchange` event within 5ms of the previous `selectionchange` event
                // => ignore it
                return;
            }
            var delta2 = now - _this._textArea.getIgnoreSelectionChangeTime();
            _this._textArea.resetSelectionChangeTime();
            if (delta2 < 100) {
                // received a `selectionchange` event within 100ms since we touched the textarea
                // => ignore it, since we caused it
                return;
            }
            if (!_this._textAreaState.selectionStartPosition || !_this._textAreaState.selectionEndPosition) {
                // Cannot correlate a position in the textarea with a position in the editor...
                return;
            }
            var newValue = _this._textArea.getValue();
            if (_this._textAreaState.value !== newValue) {
                // Cannot correlate a position in the textarea with a position in the editor...
                return;
            }
            var newSelectionStart = _this._textArea.getSelectionStart();
            var newSelectionEnd = _this._textArea.getSelectionEnd();
            if (_this._textAreaState.selectionStart === newSelectionStart && _this._textAreaState.selectionEnd === newSelectionEnd) {
                // Nothing to do...
                return;
            }
            var _newSelectionStartPosition = _this._textAreaState.deduceEditorPosition(newSelectionStart);
            var newSelectionStartPosition = _this._host.deduceModelPosition(_newSelectionStartPosition[0], _newSelectionStartPosition[1], _newSelectionStartPosition[2]);
            var _newSelectionEndPosition = _this._textAreaState.deduceEditorPosition(newSelectionEnd);
            var newSelectionEndPosition = _this._host.deduceModelPosition(_newSelectionEndPosition[0], _newSelectionEndPosition[1], _newSelectionEndPosition[2]);
            var newSelection = new Selection(newSelectionStartPosition.lineNumber, newSelectionStartPosition.column, newSelectionEndPosition.lineNumber, newSelectionEndPosition.column);
            _this._onSelectionChangeRequest.fire(newSelection);
        }));
        return _this;
    }
    TextAreaInput.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    TextAreaInput.prototype.focusTextArea = function () {
        // Setting this._hasFocus and writing the screen reader content
        // will result in a focus() and setSelectionRange() in the textarea
        this._setHasFocus(true);
    };
    TextAreaInput.prototype.isFocused = function () {
        return this._hasFocus;
    };
    TextAreaInput.prototype._setHasFocus = function (newHasFocus) {
        if (this._hasFocus === newHasFocus) {
            // no change
            return;
        }
        this._hasFocus = newHasFocus;
        if (this._hasFocus) {
            if (isEdge) {
                // Edge has a bug where setting the selection range while the focus event
                // is dispatching doesn't work. To reproduce, "tab into" the editor.
                this._setAndWriteTextAreaState('focusgain', TextAreaState.EMPTY);
            }
            else {
                this.writeScreenReaderContent('focusgain');
            }
        }
        if (this._hasFocus) {
            this._onFocus.fire();
        }
        else {
            this._onBlur.fire();
        }
    };
    TextAreaInput.prototype._setAndWriteTextAreaState = function (reason, textAreaState) {
        if (!this._hasFocus) {
            textAreaState = textAreaState.collapseSelection();
        }
        textAreaState.writeToTextArea(reason, this._textArea, this._hasFocus);
        this._textAreaState = textAreaState;
    };
    TextAreaInput.prototype.writeScreenReaderContent = function (reason) {
        if (this._isDoingComposition) {
            // Do not write to the text area when doing composition
            return;
        }
        this._setAndWriteTextAreaState(reason, this._host.getScreenReaderContent(this._textAreaState));
    };
    TextAreaInput.prototype._ensureClipboardGetsEditorSelection = function (e) {
        var copyPlainText = this._host.getPlainTextToCopy();
        if (!ClipboardEventUtils.canUseTextData(e)) {
            // Looks like an old browser. The strategy is to place the text
            // we'd like to be copied to the clipboard in the textarea and select it.
            this._setAndWriteTextAreaState('copy or cut', TextAreaState.selectedText(copyPlainText));
            return;
        }
        var copyHTML = null;
        if (hasClipboardSupport() && (copyPlainText.length < 65536 || CopyOptions.forceCopyWithSyntaxHighlighting)) {
            copyHTML = this._host.getHTMLToCopy();
        }
        ClipboardEventUtils.setTextData(e, copyPlainText, copyHTML);
    };
    return TextAreaInput;
}(Disposable));
var ClipboardEventUtils = /** @class */ (function () {
    function ClipboardEventUtils() {
    }
    ClipboardEventUtils.canUseTextData = function (e) {
        if (e.clipboardData) {
            return true;
        }
        if (window.clipboardData) {
            return true;
        }
        return false;
    };
    ClipboardEventUtils.getTextData = function (e) {
        if (e.clipboardData) {
            e.preventDefault();
            return e.clipboardData.getData('text/plain');
        }
        if (window.clipboardData) {
            e.preventDefault();
            return window.clipboardData.getData('Text');
        }
        throw new Error('ClipboardEventUtils.getTextData: Cannot use text data!');
    };
    ClipboardEventUtils.setTextData = function (e, text, richText) {
        if (e.clipboardData) {
            e.clipboardData.setData('text/plain', text);
            if (richText !== null) {
                e.clipboardData.setData('text/html', richText);
            }
            e.preventDefault();
            return;
        }
        if (window.clipboardData) {
            window.clipboardData.setData('Text', text);
            e.preventDefault();
            return;
        }
        throw new Error('ClipboardEventUtils.setTextData: Cannot use text data!');
    };
    return ClipboardEventUtils;
}());
var TextAreaWrapper = /** @class */ (function (_super) {
    __extends$t(TextAreaWrapper, _super);
    function TextAreaWrapper(_textArea) {
        var _this = _super.call(this) || this;
        _this._actual = _textArea;
        _this._ignoreSelectionChangeTime = 0;
        return _this;
    }
    TextAreaWrapper.prototype.setIgnoreSelectionChangeTime = function (reason) {
        this._ignoreSelectionChangeTime = Date.now();
    };
    TextAreaWrapper.prototype.getIgnoreSelectionChangeTime = function () {
        return this._ignoreSelectionChangeTime;
    };
    TextAreaWrapper.prototype.resetSelectionChangeTime = function () {
        this._ignoreSelectionChangeTime = 0;
    };
    TextAreaWrapper.prototype.getValue = function () {
        // console.log('current value: ' + this._textArea.value);
        return this._actual.domNode.value;
    };
    TextAreaWrapper.prototype.setValue = function (reason, value) {
        var textArea = this._actual.domNode;
        if (textArea.value === value) {
            // No change
            return;
        }
        // console.log('reason: ' + reason + ', current value: ' + textArea.value + ' => new value: ' + value);
        this.setIgnoreSelectionChangeTime('setValue');
        textArea.value = value;
    };
    TextAreaWrapper.prototype.getSelectionStart = function () {
        return this._actual.domNode.selectionStart;
    };
    TextAreaWrapper.prototype.getSelectionEnd = function () {
        return this._actual.domNode.selectionEnd;
    };
    TextAreaWrapper.prototype.setSelectionRange = function (reason, selectionStart, selectionEnd) {
        var textArea = this._actual.domNode;
        var currentIsFocused = (document.activeElement === textArea);
        var currentSelectionStart = textArea.selectionStart;
        var currentSelectionEnd = textArea.selectionEnd;
        if (currentIsFocused && currentSelectionStart === selectionStart && currentSelectionEnd === selectionEnd) {
            // No change
            // Firefox iframe bug https://github.com/Microsoft/monaco-editor/issues/643#issuecomment-367871377
            if (isFirefox && window.parent !== window) {
                textArea.focus();
            }
            return;
        }
        // console.log('reason: ' + reason + ', setSelectionRange: ' + selectionStart + ' -> ' + selectionEnd);
        if (currentIsFocused) {
            // No need to focus, only need to change the selection range
            this.setIgnoreSelectionChangeTime('setSelectionRange');
            textArea.setSelectionRange(selectionStart, selectionEnd);
            if (isFirefox && window.parent !== window) {
                textArea.focus();
            }
            return;
        }
        // If the focus is outside the textarea, browsers will try really hard to reveal the textarea.
        // Here, we try to undo the browser's desperate reveal.
        try {
            var scrollState = saveParentsScrollTop(textArea);
            this.setIgnoreSelectionChangeTime('setSelectionRange');
            textArea.focus();
            textArea.setSelectionRange(selectionStart, selectionEnd);
            restoreParentsScrollTop(textArea, scrollState);
        }
        catch (e) {
            // Sometimes IE throws when setting selection (e.g. textarea is off-DOM)
        }
    };
    return TextAreaWrapper;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$u = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DynamicViewOverlay = /** @class */ (function (_super) {
    __extends$u(DynamicViewOverlay, _super);
    function DynamicViewOverlay() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return DynamicViewOverlay;
}(ViewEventHandler));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function roundFloat(number, decimalPoints) {
    var decimal = Math.pow(10, decimalPoints);
    return Math.round(number * decimal) / decimal;
}
var RGBA = /** @class */ (function () {
    function RGBA(r, g, b, a) {
        if (a === void 0) { a = 1; }
        this.r = Math.min(255, Math.max(0, r)) | 0;
        this.g = Math.min(255, Math.max(0, g)) | 0;
        this.b = Math.min(255, Math.max(0, b)) | 0;
        this.a = roundFloat(Math.max(Math.min(1, a), 0), 3);
    }
    RGBA.equals = function (a, b) {
        return a.r === b.r && a.g === b.g && a.b === b.b && a.a === b.a;
    };
    return RGBA;
}());
var HSLA = /** @class */ (function () {
    function HSLA(h, s, l, a) {
        this.h = Math.max(Math.min(360, h), 0) | 0;
        this.s = roundFloat(Math.max(Math.min(1, s), 0), 3);
        this.l = roundFloat(Math.max(Math.min(1, l), 0), 3);
        this.a = roundFloat(Math.max(Math.min(1, a), 0), 3);
    }
    HSLA.equals = function (a, b) {
        return a.h === b.h && a.s === b.s && a.l === b.l && a.a === b.a;
    };
    /**
     * Converts an RGB color value to HSL. Conversion formula
     * adapted from http://en.wikipedia.org/wiki/HSL_color_space.
     * Assumes r, g, and b are contained in the set [0, 255] and
     * returns h in the set [0, 360], s, and l in the set [0, 1].
     */
    HSLA.fromRGBA = function (rgba) {
        var r = rgba.r / 255;
        var g = rgba.g / 255;
        var b = rgba.b / 255;
        var a = rgba.a;
        var max = Math.max(r, g, b);
        var min = Math.min(r, g, b);
        var h = 0;
        var s = 0;
        var l = (min + max) / 2;
        var chroma = max - min;
        if (chroma > 0) {
            s = Math.min((l <= 0.5 ? chroma / (2 * l) : chroma / (2 - (2 * l))), 1);
            switch (max) {
                case r:
                    h = (g - b) / chroma + (g < b ? 6 : 0);
                    break;
                case g:
                    h = (b - r) / chroma + 2;
                    break;
                case b:
                    h = (r - g) / chroma + 4;
                    break;
            }
            h *= 60;
            h = Math.round(h);
        }
        return new HSLA(h, s, l, a);
    };
    HSLA._hue2rgb = function (p, q, t) {
        if (t < 0) {
            t += 1;
        }
        if (t > 1) {
            t -= 1;
        }
        if (t < 1 / 6) {
            return p + (q - p) * 6 * t;
        }
        if (t < 1 / 2) {
            return q;
        }
        if (t < 2 / 3) {
            return p + (q - p) * (2 / 3 - t) * 6;
        }
        return p;
    };
    /**
     * Converts an HSL color value to RGB. Conversion formula
     * adapted from http://en.wikipedia.org/wiki/HSL_color_space.
     * Assumes h in the set [0, 360] s, and l are contained in the set [0, 1] and
     * returns r, g, and b in the set [0, 255].
     */
    HSLA.toRGBA = function (hsla) {
        var h = hsla.h / 360;
        var s = hsla.s, l = hsla.l, a = hsla.a;
        var r, g, b;
        if (s === 0) {
            r = g = b = l; // achromatic
        }
        else {
            var q = l < 0.5 ? l * (1 + s) : l + s - l * s;
            var p = 2 * l - q;
            r = HSLA._hue2rgb(p, q, h + 1 / 3);
            g = HSLA._hue2rgb(p, q, h);
            b = HSLA._hue2rgb(p, q, h - 1 / 3);
        }
        return new RGBA(Math.round(r * 255), Math.round(g * 255), Math.round(b * 255), a);
    };
    return HSLA;
}());
var HSVA = /** @class */ (function () {
    function HSVA(h, s, v, a) {
        this.h = Math.max(Math.min(360, h), 0) | 0;
        this.s = roundFloat(Math.max(Math.min(1, s), 0), 3);
        this.v = roundFloat(Math.max(Math.min(1, v), 0), 3);
        this.a = roundFloat(Math.max(Math.min(1, a), 0), 3);
    }
    HSVA.equals = function (a, b) {
        return a.h === b.h && a.s === b.s && a.v === b.v && a.a === b.a;
    };
    // from http://www.rapidtables.com/convert/color/rgb-to-hsv.htm
    HSVA.fromRGBA = function (rgba) {
        var r = rgba.r / 255;
        var g = rgba.g / 255;
        var b = rgba.b / 255;
        var cmax = Math.max(r, g, b);
        var cmin = Math.min(r, g, b);
        var delta = cmax - cmin;
        var s = cmax === 0 ? 0 : (delta / cmax);
        var m;
        if (delta === 0) {
            m = 0;
        }
        else if (cmax === r) {
            m = ((((g - b) / delta) % 6) + 6) % 6;
        }
        else if (cmax === g) {
            m = ((b - r) / delta) + 2;
        }
        else {
            m = ((r - g) / delta) + 4;
        }
        return new HSVA(Math.round(m * 60), s, cmax, rgba.a);
    };
    // from http://www.rapidtables.com/convert/color/hsv-to-rgb.htm
    HSVA.toRGBA = function (hsva) {
        var h = hsva.h, s = hsva.s, v = hsva.v, a = hsva.a;
        var c = v * s;
        var x = c * (1 - Math.abs((h / 60) % 2 - 1));
        var m = v - c;
        var _a = [0, 0, 0], r = _a[0], g = _a[1], b = _a[2];
        if (h < 60) {
            r = c;
            g = x;
        }
        else if (h < 120) {
            r = x;
            g = c;
        }
        else if (h < 180) {
            g = c;
            b = x;
        }
        else if (h < 240) {
            g = x;
            b = c;
        }
        else if (h < 300) {
            r = x;
            b = c;
        }
        else if (h < 360) {
            r = c;
            b = x;
        }
        r = Math.round((r + m) * 255);
        g = Math.round((g + m) * 255);
        b = Math.round((b + m) * 255);
        return new RGBA(r, g, b, a);
    };
    return HSVA;
}());
var Color = /** @class */ (function () {
    function Color(arg) {
        if (!arg) {
            throw new Error('Color needs a value');
        }
        else if (arg instanceof RGBA) {
            this.rgba = arg;
        }
        else if (arg instanceof HSLA) {
            this._hsla = arg;
            this.rgba = HSLA.toRGBA(arg);
        }
        else if (arg instanceof HSVA) {
            this._hsva = arg;
            this.rgba = HSVA.toRGBA(arg);
        }
        else {
            throw new Error('Invalid color ctor argument');
        }
    }
    Color.fromHex = function (hex) {
        return Color.Format.CSS.parseHex(hex) || Color.red;
    };
    Object.defineProperty(Color.prototype, "hsla", {
        get: function () {
            if (this._hsla) {
                return this._hsla;
            }
            else {
                return HSLA.fromRGBA(this.rgba);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Color.prototype, "hsva", {
        get: function () {
            if (this._hsva) {
                return this._hsva;
            }
            return HSVA.fromRGBA(this.rgba);
        },
        enumerable: true,
        configurable: true
    });
    Color.prototype.equals = function (other) {
        return !!other && RGBA.equals(this.rgba, other.rgba) && HSLA.equals(this.hsla, other.hsla) && HSVA.equals(this.hsva, other.hsva);
    };
    /**
     * http://www.w3.org/TR/WCAG20/#relativeluminancedef
     * Returns the number in the set [0, 1]. O => Darkest Black. 1 => Lightest white.
     */
    Color.prototype.getRelativeLuminance = function () {
        var R = Color._relativeLuminanceForComponent(this.rgba.r);
        var G = Color._relativeLuminanceForComponent(this.rgba.g);
        var B = Color._relativeLuminanceForComponent(this.rgba.b);
        var luminance = 0.2126 * R + 0.7152 * G + 0.0722 * B;
        return roundFloat(luminance, 4);
    };
    Color._relativeLuminanceForComponent = function (color) {
        var c = color / 255;
        return (c <= 0.03928) ? c / 12.92 : Math.pow(((c + 0.055) / 1.055), 2.4);
    };
    /**
     *	http://24ways.org/2010/calculating-color-contrast
     *  Return 'true' if lighter color otherwise 'false'
     */
    Color.prototype.isLighter = function () {
        var yiq = (this.rgba.r * 299 + this.rgba.g * 587 + this.rgba.b * 114) / 1000;
        return yiq >= 128;
    };
    Color.prototype.isLighterThan = function (another) {
        var lum1 = this.getRelativeLuminance();
        var lum2 = another.getRelativeLuminance();
        return lum1 > lum2;
    };
    Color.prototype.isDarkerThan = function (another) {
        var lum1 = this.getRelativeLuminance();
        var lum2 = another.getRelativeLuminance();
        return lum1 < lum2;
    };
    Color.prototype.lighten = function (factor) {
        return new Color(new HSLA(this.hsla.h, this.hsla.s, this.hsla.l + this.hsla.l * factor, this.hsla.a));
    };
    Color.prototype.darken = function (factor) {
        return new Color(new HSLA(this.hsla.h, this.hsla.s, this.hsla.l - this.hsla.l * factor, this.hsla.a));
    };
    Color.prototype.transparent = function (factor) {
        var _a = this.rgba, r = _a.r, g = _a.g, b = _a.b, a = _a.a;
        return new Color(new RGBA(r, g, b, a * factor));
    };
    Color.prototype.isTransparent = function () {
        return this.rgba.a === 0;
    };
    Color.prototype.isOpaque = function () {
        return this.rgba.a === 1;
    };
    Color.prototype.opposite = function () {
        return new Color(new RGBA(255 - this.rgba.r, 255 - this.rgba.g, 255 - this.rgba.b, this.rgba.a));
    };
    Color.prototype.toString = function () {
        return '' + Color.Format.CSS.format(this);
    };
    Color.getLighterColor = function (of, relative, factor) {
        if (of.isLighterThan(relative)) {
            return of;
        }
        factor = factor ? factor : 0.5;
        var lum1 = of.getRelativeLuminance();
        var lum2 = relative.getRelativeLuminance();
        factor = factor * (lum2 - lum1) / lum2;
        return of.lighten(factor);
    };
    Color.getDarkerColor = function (of, relative, factor) {
        if (of.isDarkerThan(relative)) {
            return of;
        }
        factor = factor ? factor : 0.5;
        var lum1 = of.getRelativeLuminance();
        var lum2 = relative.getRelativeLuminance();
        factor = factor * (lum1 - lum2) / lum1;
        return of.darken(factor);
    };
    Color.white = new Color(new RGBA(255, 255, 255, 1));
    Color.black = new Color(new RGBA(0, 0, 0, 1));
    Color.red = new Color(new RGBA(255, 0, 0, 1));
    Color.blue = new Color(new RGBA(0, 0, 255, 1));
    Color.cyan = new Color(new RGBA(0, 255, 255, 1));
    Color.lightgrey = new Color(new RGBA(211, 211, 211, 1));
    Color.transparent = new Color(new RGBA(0, 0, 0, 0));
    return Color;
}());
(function (Color) {
    var Format;
    (function (Format) {
        var CSS;
        (function (CSS) {
            function formatRGB(color) {
                if (color.rgba.a === 1) {
                    return "rgb(" + color.rgba.r + ", " + color.rgba.g + ", " + color.rgba.b + ")";
                }
                return Color.Format.CSS.formatRGBA(color);
            }
            CSS.formatRGB = formatRGB;
            function formatRGBA(color) {
                return "rgba(" + color.rgba.r + ", " + color.rgba.g + ", " + color.rgba.b + ", " + +(color.rgba.a).toFixed(2) + ")";
            }
            CSS.formatRGBA = formatRGBA;
            function formatHSL(color) {
                if (color.hsla.a === 1) {
                    return "hsl(" + color.hsla.h + ", " + (color.hsla.s * 100).toFixed(2) + "%, " + (color.hsla.l * 100).toFixed(2) + "%)";
                }
                return Color.Format.CSS.formatHSLA(color);
            }
            CSS.formatHSL = formatHSL;
            function formatHSLA(color) {
                return "hsla(" + color.hsla.h + ", " + (color.hsla.s * 100).toFixed(2) + "%, " + (color.hsla.l * 100).toFixed(2) + "%, " + color.hsla.a.toFixed(2) + ")";
            }
            CSS.formatHSLA = formatHSLA;
            function _toTwoDigitHex(n) {
                var r = n.toString(16);
                return r.length !== 2 ? '0' + r : r;
            }
            /**
             * Formats the color as #RRGGBB
             */
            function formatHex(color) {
                return "#" + _toTwoDigitHex(color.rgba.r) + _toTwoDigitHex(color.rgba.g) + _toTwoDigitHex(color.rgba.b);
            }
            CSS.formatHex = formatHex;
            /**
             * Formats the color as #RRGGBBAA
             * If 'compact' is set, colors without transparancy will be printed as #RRGGBB
             */
            function formatHexA(color, compact) {
                if (compact === void 0) { compact = false; }
                if (compact && color.rgba.a === 1) {
                    return Color.Format.CSS.formatHex(color);
                }
                return "#" + _toTwoDigitHex(color.rgba.r) + _toTwoDigitHex(color.rgba.g) + _toTwoDigitHex(color.rgba.b) + _toTwoDigitHex(Math.round(color.rgba.a * 255));
            }
            CSS.formatHexA = formatHexA;
            /**
             * The default format will use HEX if opaque and RGBA otherwise.
             */
            function format(color) {
                if (!color) {
                    return null;
                }
                if (color.isOpaque()) {
                    return Color.Format.CSS.formatHex(color);
                }
                return Color.Format.CSS.formatRGBA(color);
            }
            CSS.format = format;
            /**
             * Converts an Hex color value to a Color.
             * returns r, g, and b are contained in the set [0, 255]
             * @param hex string (#RGB, #RGBA, #RRGGBB or #RRGGBBAA).
             */
            function parseHex(hex) {
                if (!hex) {
                    // Invalid color
                    return null;
                }
                var length = hex.length;
                if (length === 0) {
                    // Invalid color
                    return null;
                }
                if (hex.charCodeAt(0) !== 35 /* Hash */) {
                    // Does not begin with a #
                    return null;
                }
                if (length === 7) {
                    // #RRGGBB format
                    var r = 16 * _parseHexDigit(hex.charCodeAt(1)) + _parseHexDigit(hex.charCodeAt(2));
                    var g = 16 * _parseHexDigit(hex.charCodeAt(3)) + _parseHexDigit(hex.charCodeAt(4));
                    var b = 16 * _parseHexDigit(hex.charCodeAt(5)) + _parseHexDigit(hex.charCodeAt(6));
                    return new Color(new RGBA(r, g, b, 1));
                }
                if (length === 9) {
                    // #RRGGBBAA format
                    var r = 16 * _parseHexDigit(hex.charCodeAt(1)) + _parseHexDigit(hex.charCodeAt(2));
                    var g = 16 * _parseHexDigit(hex.charCodeAt(3)) + _parseHexDigit(hex.charCodeAt(4));
                    var b = 16 * _parseHexDigit(hex.charCodeAt(5)) + _parseHexDigit(hex.charCodeAt(6));
                    var a = 16 * _parseHexDigit(hex.charCodeAt(7)) + _parseHexDigit(hex.charCodeAt(8));
                    return new Color(new RGBA(r, g, b, a / 255));
                }
                if (length === 4) {
                    // #RGB format
                    var r = _parseHexDigit(hex.charCodeAt(1));
                    var g = _parseHexDigit(hex.charCodeAt(2));
                    var b = _parseHexDigit(hex.charCodeAt(3));
                    return new Color(new RGBA(16 * r + r, 16 * g + g, 16 * b + b));
                }
                if (length === 5) {
                    // #RGBA format
                    var r = _parseHexDigit(hex.charCodeAt(1));
                    var g = _parseHexDigit(hex.charCodeAt(2));
                    var b = _parseHexDigit(hex.charCodeAt(3));
                    var a = _parseHexDigit(hex.charCodeAt(4));
                    return new Color(new RGBA(16 * r + r, 16 * g + g, 16 * b + b, (16 * a + a) / 255));
                }
                // Invalid color
                return null;
            }
            CSS.parseHex = parseHex;
            function _parseHexDigit(charCode) {
                switch (charCode) {
                    case 48 /* Digit0 */: return 0;
                    case 49 /* Digit1 */: return 1;
                    case 50 /* Digit2 */: return 2;
                    case 51 /* Digit3 */: return 3;
                    case 52 /* Digit4 */: return 4;
                    case 53 /* Digit5 */: return 5;
                    case 54 /* Digit6 */: return 6;
                    case 55 /* Digit7 */: return 7;
                    case 56 /* Digit8 */: return 8;
                    case 57 /* Digit9 */: return 9;
                    case 97 /* a */: return 10;
                    case 65 /* A */: return 10;
                    case 98 /* b */: return 11;
                    case 66 /* B */: return 11;
                    case 99 /* c */: return 12;
                    case 67 /* C */: return 12;
                    case 100 /* d */: return 13;
                    case 68 /* D */: return 13;
                    case 101 /* e */: return 14;
                    case 69 /* E */: return 14;
                    case 102 /* f */: return 15;
                    case 70 /* F */: return 15;
                }
                return 0;
            }
        })(CSS = Format.CSS || (Format.CSS = {}));
    })(Format = Color.Format || (Color.Format = {}));
})(Color || (Color = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// color registry
var Extensions$5 = {
    ColorContribution: 'base.contributions.colors'
};
var ColorRegistry = /** @class */ (function () {
    function ColorRegistry() {
        this.colorSchema = { type: 'object', description: localize('schema.colors', "Colors used in the workbench."), properties: {}, additionalProperties: false };
        this.colorReferenceSchema = { type: 'string', enum: [], enumDescriptions: [] };
        this.colorsById = {};
    }
    ColorRegistry.prototype.registerColor = function (id, defaults, description, needsTransparency, deprecationMessage) {
        if (needsTransparency === void 0) { needsTransparency = false; }
        var colorContribution = { id: id, description: description, defaults: defaults, needsTransparency: needsTransparency, deprecationMessage: deprecationMessage };
        this.colorsById[id] = colorContribution;
        var propertySchema = { type: 'string', description: description, format: 'color-hex', default: '#ff0000' };
        if (deprecationMessage) {
            propertySchema.deprecationMessage = deprecationMessage;
        }
        this.colorSchema.properties[id] = propertySchema;
        this.colorReferenceSchema.enum.push(id);
        this.colorReferenceSchema.enumDescriptions.push(description);
        return id;
    };
    ColorRegistry.prototype.resolveDefaultColor = function (id, theme) {
        var colorDesc = this.colorsById[id];
        if (colorDesc && colorDesc.defaults) {
            var colorValue = colorDesc.defaults[theme.type];
            return resolveColorValue(colorValue, theme);
        }
        return null;
    };
    ColorRegistry.prototype.toString = function () {
        var _this = this;
        var sorter = function (a, b) {
            var cat1 = a.indexOf('.') === -1 ? 0 : 1;
            var cat2 = b.indexOf('.') === -1 ? 0 : 1;
            if (cat1 !== cat2) {
                return cat1 - cat2;
            }
            return a.localeCompare(b);
        };
        return Object.keys(this.colorsById).sort(sorter).map(function (k) { return "- `" + k + "`: " + _this.colorsById[k].description; }).join('\n');
    };
    return ColorRegistry;
}());
var colorRegistry = new ColorRegistry();
Registry.add(Extensions$5.ColorContribution, colorRegistry);
function registerColor(id, defaults, description, needsTransparency, deprecationMessage) {
    return colorRegistry.registerColor(id, defaults, description, needsTransparency, deprecationMessage);
}
// ----- base colors
var foreground = registerColor('foreground', { dark: '#CCCCCC', light: '#616161', hc: '#FFFFFF' }, localize('foreground', "Overall foreground color. This color is only used if not overridden by a component."));
var errorForeground = registerColor('errorForeground', { dark: '#F48771', light: '#A1260D', hc: '#F48771' }, localize('errorForeground', "Overall foreground color for error messages. This color is only used if not overridden by a component."));
var focusBorder = registerColor('focusBorder', { dark: Color.fromHex('#0E639C').transparent(0.8), light: Color.fromHex('#007ACC').transparent(0.4), hc: '#F38518' }, localize('focusBorder', "Overall border color for focused elements. This color is only used if not overridden by a component."));
var contrastBorder = registerColor('contrastBorder', { light: null, dark: null, hc: '#6FC3DF' }, localize('contrastBorder', "An extra border around elements to separate them from others for greater contrast."));
var activeContrastBorder = registerColor('contrastActiveBorder', { light: null, dark: null, hc: focusBorder }, localize('activeContrastBorder', "An extra border around active elements to separate them from others for greater contrast."));
var textLinkForeground = registerColor('textLink.foreground', { light: '#006AB1', dark: '#3794FF', hc: '#3794FF' }, localize('textLinkForeground', "Foreground color for links in text."));
var textCodeBlockBackground = registerColor('textCodeBlock.background', { light: '#dcdcdc66', dark: '#0a0a0a66', hc: Color.black }, localize('textCodeBlockBackground', "Background color for code blocks in text."));
// ----- widgets
var widgetShadow = registerColor('widget.shadow', { dark: '#000000', light: '#A8A8A8', hc: null }, localize('widgetShadow', 'Shadow color of widgets such as find/replace inside the editor.'));
var inputBackground = registerColor('input.background', { dark: '#3C3C3C', light: Color.white, hc: Color.black }, localize('inputBoxBackground', "Input box background."));
var inputForeground = registerColor('input.foreground', { dark: foreground, light: foreground, hc: foreground }, localize('inputBoxForeground', "Input box foreground."));
var inputBorder = registerColor('input.border', { dark: null, light: null, hc: contrastBorder }, localize('inputBoxBorder', "Input box border."));
var inputActiveOptionBorder = registerColor('inputOption.activeBorder', { dark: '#007ACC', light: '#007ACC', hc: activeContrastBorder }, localize('inputBoxActiveOptionBorder', "Border color of activated options in input fields."));
var inputValidationInfoBackground = registerColor('inputValidation.infoBackground', { dark: '#063B49', light: '#D6ECF2', hc: Color.black }, localize('inputValidationInfoBackground', "Input validation background color for information severity."));
var inputValidationInfoForeground = registerColor('inputValidation.infoForeground', { dark: null, light: null, hc: null }, localize('inputValidationInfoForeground', "Input validation foreground color for information severity."));
var inputValidationInfoBorder = registerColor('inputValidation.infoBorder', { dark: '#007acc', light: '#007acc', hc: contrastBorder }, localize('inputValidationInfoBorder', "Input validation border color for information severity."));
var inputValidationWarningBackground = registerColor('inputValidation.warningBackground', { dark: '#352A05', light: '#F6F5D2', hc: Color.black }, localize('inputValidationWarningBackground', "Input validation background color for warning severity."));
var inputValidationWarningForeground = registerColor('inputValidation.warningForeground', { dark: null, light: null, hc: null }, localize('inputValidationWarningForeground', "Input validation foreground color for warning severity."));
var inputValidationWarningBorder = registerColor('inputValidation.warningBorder', { dark: '#B89500', light: '#B89500', hc: contrastBorder }, localize('inputValidationWarningBorder', "Input validation border color for warning severity."));
var inputValidationErrorBackground = registerColor('inputValidation.errorBackground', { dark: '#5A1D1D', light: '#F2DEDE', hc: Color.black }, localize('inputValidationErrorBackground', "Input validation background color for error severity."));
var inputValidationErrorForeground = registerColor('inputValidation.errorForeground', { dark: null, light: null, hc: null }, localize('inputValidationErrorForeground', "Input validation foreground color for error severity."));
var inputValidationErrorBorder = registerColor('inputValidation.errorBorder', { dark: '#BE1100', light: '#BE1100', hc: contrastBorder }, localize('inputValidationErrorBorder', "Input validation border color for error severity."));
var selectBackground = registerColor('dropdown.background', { dark: '#3C3C3C', light: Color.white, hc: Color.black }, localize('dropdownBackground', "Dropdown background."));
var selectForeground = registerColor('dropdown.foreground', { dark: '#F0F0F0', light: null, hc: Color.white }, localize('dropdownForeground', "Dropdown foreground."));
var listFocusBackground = registerColor('list.focusBackground', { dark: '#062F4A', light: '#D6EBFF', hc: null }, localize('listFocusBackground', "List/Tree background color for the focused item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not."));
var listFocusForeground = registerColor('list.focusForeground', { dark: null, light: null, hc: null }, localize('listFocusForeground', "List/Tree foreground color for the focused item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not."));
var listActiveSelectionBackground = registerColor('list.activeSelectionBackground', { dark: '#094771', light: '#2477CE', hc: null }, localize('listActiveSelectionBackground', "List/Tree background color for the selected item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not."));
var listActiveSelectionForeground = registerColor('list.activeSelectionForeground', { dark: Color.white, light: Color.white, hc: null }, localize('listActiveSelectionForeground', "List/Tree foreground color for the selected item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not."));
var listInactiveSelectionBackground = registerColor('list.inactiveSelectionBackground', { dark: '#37373D', light: '#E4E6F1', hc: null }, localize('listInactiveSelectionBackground', "List/Tree background color for the selected item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not."));
var listInactiveSelectionForeground = registerColor('list.inactiveSelectionForeground', { dark: null, light: null, hc: null }, localize('listInactiveSelectionForeground', "List/Tree foreground color for the selected item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not."));
var listInactiveFocusBackground = registerColor('list.inactiveFocusBackground', { dark: '#313135', light: '#d8dae6', hc: null }, localize('listInactiveFocusBackground', "List/Tree background color for the focused item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not."));
var listHoverBackground = registerColor('list.hoverBackground', { dark: '#2A2D2E', light: '#F0F0F0', hc: null }, localize('listHoverBackground', "List/Tree background when hovering over items using the mouse."));
var listHoverForeground = registerColor('list.hoverForeground', { dark: null, light: null, hc: null }, localize('listHoverForeground', "List/Tree foreground when hovering over items using the mouse."));
var listDropBackground = registerColor('list.dropBackground', { dark: listFocusBackground, light: listFocusBackground, hc: null }, localize('listDropBackground', "List/Tree drag and drop background when moving items around using the mouse."));
var listHighlightForeground = registerColor('list.highlightForeground', { dark: '#0097fb', light: '#0066BF', hc: focusBorder }, localize('highlight', 'List/Tree foreground color of the match highlights when searching inside the list/tree.'));
var pickerGroupForeground = registerColor('pickerGroup.foreground', { dark: '#3794FF', light: '#0066BF', hc: Color.white }, localize('pickerGroupForeground', "Quick picker color for grouping labels."));
var pickerGroupBorder = registerColor('pickerGroup.border', { dark: '#3F3F46', light: '#CCCEDB', hc: Color.white }, localize('pickerGroupBorder', "Quick picker color for grouping borders."));
var badgeBackground = registerColor('badge.background', { dark: '#4D4D4D', light: '#C4C4C4', hc: Color.black }, localize('badgeBackground', "Badge background color. Badges are small information labels, e.g. for search results count."));
var badgeForeground = registerColor('badge.foreground', { dark: Color.white, light: '#333', hc: Color.white }, localize('badgeForeground', "Badge foreground color. Badges are small information labels, e.g. for search results count."));
var scrollbarShadow = registerColor('scrollbar.shadow', { dark: '#000000', light: '#DDDDDD', hc: null }, localize('scrollbarShadow', "Scrollbar shadow to indicate that the view is scrolled."));
var scrollbarSliderBackground = registerColor('scrollbarSlider.background', { dark: Color.fromHex('#797979').transparent(0.4), light: Color.fromHex('#646464').transparent(0.4), hc: transparent(contrastBorder, 0.6) }, localize('scrollbarSliderBackground', "Scrollbar slider background color."));
var scrollbarSliderHoverBackground = registerColor('scrollbarSlider.hoverBackground', { dark: Color.fromHex('#646464').transparent(0.7), light: Color.fromHex('#646464').transparent(0.7), hc: transparent(contrastBorder, 0.8) }, localize('scrollbarSliderHoverBackground', "Scrollbar slider background color when hovering."));
var scrollbarSliderActiveBackground = registerColor('scrollbarSlider.activeBackground', { dark: Color.fromHex('#BFBFBF').transparent(0.4), light: Color.fromHex('#000000').transparent(0.6), hc: contrastBorder }, localize('scrollbarSliderActiveBackground', "Scrollbar slider background color when clicked on."));
var progressBarBackground = registerColor('progressBar.background', { dark: Color.fromHex('#0E70C0'), light: Color.fromHex('#0E70C0'), hc: contrastBorder }, localize('progressBarBackground', "Background color of the progress bar that can show for long running operations."));
var menuBorder = registerColor('menu.border', { dark: null, light: null, hc: contrastBorder }, localize('menuBorder', "Border color of menus."));
var menuForeground = registerColor('menu.foreground', { dark: selectForeground, light: selectForeground, hc: selectForeground }, localize('menuForeground', "Foreground color of menu items."));
var menuBackground = registerColor('menu.background', { dark: selectBackground, light: selectBackground, hc: selectBackground }, localize('menuBackground', "Background color of menu items."));
var menuSelectionForeground = registerColor('menu.selectionForeground', { dark: listActiveSelectionForeground, light: listActiveSelectionForeground, hc: listActiveSelectionForeground }, localize('menuSelectionForeground', "Foreground color of the selected menu item in menus."));
var menuSelectionBackground = registerColor('menu.selectionBackground', { dark: listActiveSelectionBackground, light: listActiveSelectionBackground, hc: listActiveSelectionBackground }, localize('menuSelectionBackground', "Background color of the selected menu item in menus."));
var menuSelectionBorder = registerColor('menu.selectionBorder', { dark: null, light: null, hc: activeContrastBorder }, localize('menuSelectionBorder', "Border color of the selected menu item in menus."));
var menuSeparatorBackground = registerColor('menu.separatorBackground', { dark: '#BBBBBB', light: '#888888', hc: contrastBorder }, localize('menuSeparatorBackground', "Color of a separator menu item in menus."));
/**
 * Editor background color.
 * Because of bug https://monacotools.visualstudio.com/DefaultCollection/Monaco/_workitems/edit/13254
 * we are *not* using the color white (or #ffffff, rgba(255,255,255)) but something very close to white.
 */
var editorBackground = registerColor('editor.background', { light: '#fffffe', dark: '#1E1E1E', hc: Color.black }, localize('editorBackground', "Editor background color."));
/**
 * Editor foreground color.
 */
var editorForeground = registerColor('editor.foreground', { light: '#333333', dark: '#BBBBBB', hc: Color.white }, localize('editorForeground', "Editor default foreground color."));
/**
 * Editor widgets
 */
var editorWidgetBackground = registerColor('editorWidget.background', { dark: '#252526', light: '#F3F3F3', hc: '#0C141F' }, localize('editorWidgetBackground', 'Background color of editor widgets, such as find/replace.'));
var editorWidgetBorder = registerColor('editorWidget.border', { dark: '#454545', light: '#C8C8C8', hc: contrastBorder }, localize('editorWidgetBorder', 'Border color of editor widgets. The color is only used if the widget chooses to have a border and if the color is not overridden by a widget.'));
var editorWidgetResizeBorder = registerColor('editorWidget.resizeBorder', { light: null, dark: null, hc: null }, localize('editorWidgetResizeBorder', "Border color of the resize bar of editor widgets. The color is only used if the widget chooses to have a resize border and if the color is not overridden by a widget."));
/**
 * Editor selection colors.
 */
var editorSelectionBackground = registerColor('editor.selectionBackground', { light: '#ADD6FF', dark: '#264F78', hc: '#f3f518' }, localize('editorSelectionBackground', "Color of the editor selection."));
var editorSelectionForeground = registerColor('editor.selectionForeground', { light: null, dark: null, hc: '#000000' }, localize('editorSelectionForeground', "Color of the selected text for high contrast."));
var editorInactiveSelection = registerColor('editor.inactiveSelectionBackground', { light: transparent(editorSelectionBackground, 0.5), dark: transparent(editorSelectionBackground, 0.5), hc: transparent(editorSelectionBackground, 0.5) }, localize('editorInactiveSelection', "Color of the selection in an inactive editor. The color must not be opaque to not hide underlying decorations."), true);
var editorSelectionHighlight = registerColor('editor.selectionHighlightBackground', { light: lessProminent(editorSelectionBackground, editorBackground, 0.3, 0.6), dark: lessProminent(editorSelectionBackground, editorBackground, 0.3, 0.6), hc: null }, localize('editorSelectionHighlight', 'Color for regions with the same content as the selection. The color must not be opaque to not hide underlying decorations.'), true);
var editorSelectionHighlightBorder = registerColor('editor.selectionHighlightBorder', { light: null, dark: null, hc: activeContrastBorder }, localize('editorSelectionHighlightBorder', "Border color for regions with the same content as the selection."));
/**
 * Editor find match colors.
 */
var editorFindMatch = registerColor('editor.findMatchBackground', { light: '#A8AC94', dark: '#515C6A', hc: null }, localize('editorFindMatch', "Color of the current search match."));
var editorFindMatchHighlight = registerColor('editor.findMatchHighlightBackground', { light: '#EA5C0055', dark: '#EA5C0055', hc: null }, localize('findMatchHighlight', "Color of the other search matches. The color must not be opaque to not hide underlying decorations."), true);
var editorFindRangeHighlight = registerColor('editor.findRangeHighlightBackground', { dark: '#3a3d4166', light: '#b4b4b44d', hc: null }, localize('findRangeHighlight', "Color of the range limiting the search. The color must not be opaque to not hide underlying decorations."), true);
var editorFindMatchBorder = registerColor('editor.findMatchBorder', { light: null, dark: null, hc: activeContrastBorder }, localize('editorFindMatchBorder', "Border color of the current search match."));
var editorFindMatchHighlightBorder = registerColor('editor.findMatchHighlightBorder', { light: null, dark: null, hc: activeContrastBorder }, localize('findMatchHighlightBorder', "Border color of the other search matches."));
var editorFindRangeHighlightBorder = registerColor('editor.findRangeHighlightBorder', { dark: null, light: null, hc: transparent(activeContrastBorder, 0.4) }, localize('findRangeHighlightBorder', "Border color of the range limiting the search. The color must not be opaque to not hide underlying decorations."), true);
/**
 * Editor hover
 */
var editorHoverHighlight = registerColor('editor.hoverHighlightBackground', { light: '#ADD6FF26', dark: '#264f7840', hc: '#ADD6FF26' }, localize('hoverHighlight', 'Highlight below the word for which a hover is shown. The color must not be opaque to not hide underlying decorations.'), true);
var editorHoverBackground = registerColor('editorHoverWidget.background', { light: editorWidgetBackground, dark: editorWidgetBackground, hc: editorWidgetBackground }, localize('hoverBackground', 'Background color of the editor hover.'));
var editorHoverBorder = registerColor('editorHoverWidget.border', { light: editorWidgetBorder, dark: editorWidgetBorder, hc: editorWidgetBorder }, localize('hoverBorder', 'Border color of the editor hover.'));
/**
 * Editor link colors
 */
var editorActiveLinkForeground = registerColor('editorLink.activeForeground', { dark: '#4E94CE', light: Color.blue, hc: Color.cyan }, localize('activeLinkForeground', 'Color of active links.'));
/**
 * Diff Editor Colors
 */
var defaultInsertColor = new Color(new RGBA(155, 185, 85, 0.2));
var defaultRemoveColor = new Color(new RGBA(255, 0, 0, 0.2));
var diffInserted = registerColor('diffEditor.insertedTextBackground', { dark: defaultInsertColor, light: defaultInsertColor, hc: null }, localize('diffEditorInserted', 'Background color for text that got inserted. The color must not be opaque to not hide underlying decorations.'), true);
var diffRemoved = registerColor('diffEditor.removedTextBackground', { dark: defaultRemoveColor, light: defaultRemoveColor, hc: null }, localize('diffEditorRemoved', 'Background color for text that got removed. The color must not be opaque to not hide underlying decorations.'), true);
var diffInsertedOutline = registerColor('diffEditor.insertedTextBorder', { dark: null, light: null, hc: '#33ff2eff' }, localize('diffEditorInsertedOutline', 'Outline color for the text that got inserted.'));
var diffRemovedOutline = registerColor('diffEditor.removedTextBorder', { dark: null, light: null, hc: '#FF008F' }, localize('diffEditorRemovedOutline', 'Outline color for text that got removed.'));
var diffBorder = registerColor('diffEditor.border', { dark: null, light: null, hc: contrastBorder }, localize('diffEditorBorder', 'Border color between the two text editors.'));
/**
 * Snippet placeholder colors
 */
var snippetTabstopHighlightBackground = registerColor('editor.snippetTabstopHighlightBackground', { dark: new Color(new RGBA(124, 124, 124, 0.3)), light: new Color(new RGBA(10, 50, 100, 0.2)), hc: new Color(new RGBA(124, 124, 124, 0.3)) }, localize('snippetTabstopHighlightBackground', "Highlight background color of a snippet tabstop."));
var snippetTabstopHighlightBorder = registerColor('editor.snippetTabstopHighlightBorder', { dark: null, light: null, hc: null }, localize('snippetTabstopHighlightBorder', "Highlight border color of a snippet tabstop."));
var snippetFinalTabstopHighlightBackground = registerColor('editor.snippetFinalTabstopHighlightBackground', { dark: null, light: null, hc: null }, localize('snippetFinalTabstopHighlightBackground', "Highlight background color of the final tabstop of a snippet."));
var snippetFinalTabstopHighlightBorder = registerColor('editor.snippetFinalTabstopHighlightBorder', { dark: '#525252', light: new Color(new RGBA(10, 50, 100, 0.5)), hc: '#525252' }, localize('snippetFinalTabstopHighlightBorder', "Highlight border color of the final stabstop of a snippet."));
var findMatchColorDefault = new Color(new RGBA(246, 185, 77, 0.7));
var overviewRulerFindMatchForeground = registerColor('editorOverviewRuler.findMatchForeground', { dark: findMatchColorDefault, light: findMatchColorDefault, hc: findMatchColorDefault }, localize('overviewRulerFindMatchForeground', 'Overview ruler marker color for find matches. The color must not be opaque to not hide underlying decorations.'), true);
var overviewRulerSelectionHighlightForeground = registerColor('editorOverviewRuler.selectionHighlightForeground', { dark: '#A0A0A0CC', light: '#A0A0A0CC', hc: '#A0A0A0CC' }, localize('overviewRulerSelectionHighlightForeground', 'Overview ruler marker color for selection highlights. The color must not be opaque to not hide underlying decorations.'), true);
function transparent(colorValue, factor) {
    return function (theme) {
        var color = resolveColorValue(colorValue, theme);
        if (color) {
            return color.transparent(factor);
        }
        return null;
    };
}
function oneOf() {
    var colorValues = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        colorValues[_i] = arguments[_i];
    }
    return function (theme) {
        for (var _i = 0, colorValues_1 = colorValues; _i < colorValues_1.length; _i++) {
            var colorValue = colorValues_1[_i];
            var color = resolveColorValue(colorValue, theme);
            if (color) {
                return color;
            }
        }
        return null;
    };
}
function lessProminent(colorValue, backgroundColorValue, factor, transparency) {
    return function (theme) {
        var from = resolveColorValue(colorValue, theme);
        if (from) {
            var backgroundColor = resolveColorValue(backgroundColorValue, theme);
            if (backgroundColor) {
                if (from.isDarkerThan(backgroundColor)) {
                    return Color.getLighterColor(from, backgroundColor, factor).transparent(transparency);
                }
                return Color.getDarkerColor(from, backgroundColor, factor).transparent(transparency);
            }
            return from.transparent(factor * transparency);
        }
        return null;
    };
}
// ----- implementation
/**
 * @param colorValue Resolve a color value in the context of a theme
 */
function resolveColorValue(colorValue, theme) {
    if (colorValue === null) {
        return null;
    }
    else if (typeof colorValue === 'string') {
        if (colorValue[0] === '#') {
            return Color.fromHex(colorValue);
        }
        return theme.getColor(colorValue);
    }
    else if (colorValue instanceof Color) {
        return colorValue;
    }
    else if (typeof colorValue === 'function') {
        return colorValue(theme);
    }
    return null;
}
// setTimeout(_ => console.log(colorRegistry.toString()), 5000);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Definition of the editor colors
 */
var editorLineHighlight = registerColor('editor.lineHighlightBackground', { dark: null, light: null, hc: null }, localize('lineHighlight', 'Background color for the highlight of line at the cursor position.'));
var editorLineHighlightBorder = registerColor('editor.lineHighlightBorder', { dark: '#282828', light: '#eeeeee', hc: '#f38518' }, localize('lineHighlightBorderBox', 'Background color for the border around the line at the cursor position.'));
var editorRangeHighlight = registerColor('editor.rangeHighlightBackground', { dark: '#ffffff0b', light: '#fdff0033', hc: null }, localize('rangeHighlight', 'Background color of highlighted ranges, like by quick open and find features. The color must not be opaque to not hide underlying decorations.'), true);
var editorRangeHighlightBorder = registerColor('editor.rangeHighlightBorder', { dark: null, light: null, hc: activeContrastBorder }, localize('rangeHighlightBorder', 'Background color of the border around highlighted ranges.'), true);
var editorCursorForeground = registerColor('editorCursor.foreground', { dark: '#AEAFAD', light: Color.black, hc: Color.white }, localize('caret', 'Color of the editor cursor.'));
var editorCursorBackground = registerColor('editorCursor.background', null, localize('editorCursorBackground', 'The background color of the editor cursor. Allows customizing the color of a character overlapped by a block cursor.'));
var editorWhitespaces = registerColor('editorWhitespace.foreground', { dark: '#e3e4e229', light: '#33333333', hc: '#e3e4e229' }, localize('editorWhitespaces', 'Color of whitespace characters in the editor.'));
var editorIndentGuides = registerColor('editorIndentGuide.background', { dark: editorWhitespaces, light: editorWhitespaces, hc: editorWhitespaces }, localize('editorIndentGuides', 'Color of the editor indentation guides.'));
var editorActiveIndentGuides = registerColor('editorIndentGuide.activeBackground', { dark: editorWhitespaces, light: editorWhitespaces, hc: editorWhitespaces }, localize('editorActiveIndentGuide', 'Color of the active editor indentation guides.'));
var editorLineNumbers = registerColor('editorLineNumber.foreground', { dark: '#858585', light: '#237893', hc: Color.white }, localize('editorLineNumbers', 'Color of editor line numbers.'));
var deprecatedEditorActiveLineNumber = registerColor('editorActiveLineNumber.foreground', { dark: '#c6c6c6', light: '#0B216F', hc: activeContrastBorder }, localize('editorActiveLineNumber', 'Color of editor active line number'), false, localize('deprecatedEditorActiveLineNumber', 'Id is deprecated. Use \'editorLineNumber.activeForeground\' instead.'));
var editorActiveLineNumber = registerColor('editorLineNumber.activeForeground', { dark: deprecatedEditorActiveLineNumber, light: deprecatedEditorActiveLineNumber, hc: deprecatedEditorActiveLineNumber }, localize('editorActiveLineNumber', 'Color of editor active line number'));
var editorRuler = registerColor('editorRuler.foreground', { dark: '#5A5A5A', light: Color.lightgrey, hc: Color.white }, localize('editorRuler', 'Color of the editor rulers.'));
var editorCodeLensForeground = registerColor('editorCodeLens.foreground', { dark: '#999999', light: '#999999', hc: '#999999' }, localize('editorCodeLensForeground', 'Foreground color of editor code lenses'));
var editorBracketMatchBackground = registerColor('editorBracketMatch.background', { dark: '#0064001a', light: '#0064001a', hc: '#0064001a' }, localize('editorBracketMatchBackground', 'Background color behind matching brackets'));
var editorBracketMatchBorder = registerColor('editorBracketMatch.border', { dark: '#888', light: '#B9B9B9', hc: '#fff' }, localize('editorBracketMatchBorder', 'Color for matching brackets boxes'));
var editorOverviewRulerBorder = registerColor('editorOverviewRuler.border', { dark: '#7f7f7f4d', light: '#7f7f7f4d', hc: '#7f7f7f4d' }, localize('editorOverviewRulerBorder', 'Color of the overview ruler border.'));
var editorGutter = registerColor('editorGutter.background', { dark: editorBackground, light: editorBackground, hc: editorBackground }, localize('editorGutter', 'Background color of the editor gutter. The gutter contains the glyph margins and the line numbers.'));
var editorErrorForeground = registerColor('editorError.foreground', { dark: '#ea4646', light: '#d60a0a', hc: null }, localize('errorForeground', 'Foreground color of error squigglies in the editor.'));
var editorErrorBorder = registerColor('editorError.border', { dark: null, light: null, hc: Color.fromHex('#E47777').transparent(0.8) }, localize('errorBorder', 'Border color of error squigglies in the editor.'));
var editorWarningForeground = registerColor('editorWarning.foreground', { dark: '#4d9e4d', light: '#117711', hc: null }, localize('warningForeground', 'Foreground color of warning squigglies in the editor.'));
var editorWarningBorder = registerColor('editorWarning.border', { dark: null, light: null, hc: Color.fromHex('#71B771').transparent(0.8) }, localize('warningBorder', 'Border color of warning squigglies in the editor.'));
var editorInfoForeground = registerColor('editorInfo.foreground', { dark: '#008000', light: '#008000', hc: null }, localize('infoForeground', 'Foreground color of info squigglies in the editor.'));
var editorInfoBorder = registerColor('editorInfo.border', { dark: null, light: null, hc: Color.fromHex('#71B771').transparent(0.8) }, localize('infoBorder', 'Border color of info squigglies in the editor.'));
var editorHintForeground = registerColor('editorHint.foreground', { dark: Color.fromHex('#eeeeee').transparent(0.7), light: '#6c6c6c', hc: null }, localize('hintForeground', 'Foreground color of hint squigglies in the editor.'));
var editorHintBorder = registerColor('editorHint.border', { dark: null, light: null, hc: Color.fromHex('#eeeeee').transparent(0.8) }, localize('hintBorder', 'Border color of hint squigglies in the editor.'));
var editorUnnecessaryCodeBorder = registerColor('editorUnnecessaryCode.border', { dark: null, light: null, hc: Color.fromHex('#fff').transparent(0.8) }, localize('unnecessaryCodeBorder', 'Border of unnecessary code in the editor.'));
var editorUnnecessaryCodeOpacity = registerColor('editorUnnecessaryCode.opacity', { dark: Color.fromHex('#000a'), light: Color.fromHex('#0007'), hc: null }, localize('unnecessaryCodeOpacity', 'Opacity of unnecessary code in the editor.'));
var overviewRulerError = registerColor('editorOverviewRuler.errorForeground', { dark: new Color(new RGBA(255, 18, 18, 0.7)), light: new Color(new RGBA(255, 18, 18, 0.7)), hc: new Color(new RGBA(255, 50, 50, 1)) }, localize('overviewRuleError', 'Overview ruler marker color for errors.'));
var overviewRulerWarning = registerColor('editorOverviewRuler.warningForeground', { dark: new Color(new RGBA(18, 136, 18, 0.7)), light: new Color(new RGBA(18, 136, 18, 0.7)), hc: new Color(new RGBA(50, 255, 50, 1)) }, localize('overviewRuleWarning', 'Overview ruler marker color for warnings.'));
var overviewRulerInfo = registerColor('editorOverviewRuler.infoForeground', { dark: new Color(new RGBA(18, 18, 136, 0.7)), light: new Color(new RGBA(18, 18, 136, 0.7)), hc: new Color(new RGBA(50, 50, 255, 1)) }, localize('overviewRuleInfo', 'Overview ruler marker color for infos.'));
// contains all color rules that used to defined in editor/browser/widget/editor.css
registerThemingParticipant(function (theme, collector) {
    var background = theme.getColor(editorBackground);
    if (background) {
        collector.addRule(".monaco-editor, .monaco-editor-background, .monaco-editor .inputarea.ime-input { background-color: " + background + "; }");
    }
    var foreground$$1 = theme.getColor(editorForeground);
    if (foreground$$1) {
        collector.addRule(".monaco-editor, .monaco-editor .inputarea.ime-input { color: " + foreground$$1 + "; }");
    }
    var gutter = theme.getColor(editorGutter);
    if (gutter) {
        collector.addRule(".monaco-editor .margin { background-color: " + gutter + "; }");
    }
    var rangeHighlight = theme.getColor(editorRangeHighlight);
    if (rangeHighlight) {
        collector.addRule(".monaco-editor .rangeHighlight { background-color: " + rangeHighlight + "; }");
    }
    var rangeHighlightBorder = theme.getColor(editorRangeHighlightBorder);
    if (rangeHighlightBorder) {
        collector.addRule(".monaco-editor .rangeHighlight { border: 1px " + (theme.type === 'hc' ? 'dotted' : 'solid') + " " + rangeHighlightBorder + "; }");
    }
    var invisibles = theme.getColor(editorWhitespaces);
    if (invisibles) {
        collector.addRule(".vs-whitespace { color: " + invisibles + " !important; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$v = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var LineNumbersOverlay = /** @class */ (function (_super) {
    __extends$v(LineNumbersOverlay, _super);
    function LineNumbersOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._readConfig();
        _this._lastCursorModelPosition = new Position(1, 1);
        _this._renderResult = null;
        _this._context.addEventHandler(_this);
        return _this;
    }
    LineNumbersOverlay.prototype._readConfig = function () {
        var config = this._context.configuration.editor;
        this._lineHeight = config.lineHeight;
        this._renderLineNumbers = config.viewInfo.renderLineNumbers;
        this._renderCustomLineNumbers = config.viewInfo.renderCustomLineNumbers;
        this._lineNumbersLeft = config.layoutInfo.lineNumbersLeft;
        this._lineNumbersWidth = config.layoutInfo.lineNumbersWidth;
    };
    LineNumbersOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        this._renderResult = null;
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    LineNumbersOverlay.prototype.onConfigurationChanged = function (e) {
        this._readConfig();
        return true;
    };
    LineNumbersOverlay.prototype.onCursorStateChanged = function (e) {
        var primaryViewPosition = e.selections[0].getPosition();
        this._lastCursorModelPosition = this._context.model.coordinatesConverter.convertViewPositionToModelPosition(primaryViewPosition);
        if (this._renderLineNumbers === 2 /* Relative */ || this._renderLineNumbers === 3 /* Interval */) {
            return true;
        }
        return false;
    };
    LineNumbersOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    LineNumbersOverlay.prototype.onLinesChanged = function (e) {
        return true;
    };
    LineNumbersOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    LineNumbersOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    LineNumbersOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged;
    };
    LineNumbersOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    LineNumbersOverlay.prototype._getLineRenderLineNumber = function (viewLineNumber) {
        var modelPosition = this._context.model.coordinatesConverter.convertViewPositionToModelPosition(new Position(viewLineNumber, 1));
        if (modelPosition.column !== 1) {
            return '';
        }
        var modelLineNumber = modelPosition.lineNumber;
        if (this._renderCustomLineNumbers) {
            return this._renderCustomLineNumbers(modelLineNumber);
        }
        if (this._renderLineNumbers === 2 /* Relative */) {
            var diff = Math.abs(this._lastCursorModelPosition.lineNumber - modelLineNumber);
            if (diff === 0) {
                return '<span class="relative-current-line-number">' + modelLineNumber + '</span>';
            }
            return String(diff);
        }
        if (this._renderLineNumbers === 3 /* Interval */) {
            if (this._lastCursorModelPosition.lineNumber === modelLineNumber) {
                return String(modelLineNumber);
            }
            if (modelLineNumber % 10 === 0) {
                return String(modelLineNumber);
            }
            return '';
        }
        return String(modelLineNumber);
    };
    LineNumbersOverlay.prototype.prepareRender = function (ctx) {
        if (this._renderLineNumbers === 0 /* Off */) {
            this._renderResult = null;
            return;
        }
        var lineHeightClassName = (isLinux ? (this._lineHeight % 2 === 0 ? ' lh-even' : ' lh-odd') : '');
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        var common = '<div class="' + LineNumbersOverlay.CLASS_NAME + lineHeightClassName + '" style="left:' + this._lineNumbersLeft.toString() + 'px;width:' + this._lineNumbersWidth.toString() + 'px;">';
        var output = [];
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var lineIndex = lineNumber - visibleStartLineNumber;
            var renderLineNumber = this._getLineRenderLineNumber(lineNumber);
            if (renderLineNumber) {
                output[lineIndex] = (common
                    + renderLineNumber
                    + '</div>');
            }
            else {
                output[lineIndex] = '';
            }
        }
        this._renderResult = output;
    };
    LineNumbersOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (!this._renderResult) {
            return '';
        }
        var lineIndex = lineNumber - startLineNumber;
        if (lineIndex < 0 || lineIndex >= this._renderResult.length) {
            return '';
        }
        return this._renderResult[lineIndex];
    };
    LineNumbersOverlay.CLASS_NAME = 'line-numbers';
    return LineNumbersOverlay;
}(DynamicViewOverlay));
// theming
registerThemingParticipant(function (theme, collector) {
    var lineNumbers = theme.getColor(editorLineNumbers);
    if (lineNumbers) {
        collector.addRule(".monaco-editor .line-numbers { color: " + lineNumbers + "; }");
    }
    var activeLineNumber = theme.getColor(editorActiveLineNumber);
    if (activeLineNumber) {
        collector.addRule(".monaco-editor .current-line ~ .line-numbers { color: " + activeLineNumber + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$w = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Margin = /** @class */ (function (_super) {
    __extends$w(Margin, _super);
    function Margin(context) {
        var _this = _super.call(this, context) || this;
        _this._canUseLayerHinting = _this._context.configuration.editor.canUseLayerHinting;
        _this._contentLeft = _this._context.configuration.editor.layoutInfo.contentLeft;
        _this._glyphMarginLeft = _this._context.configuration.editor.layoutInfo.glyphMarginLeft;
        _this._glyphMarginWidth = _this._context.configuration.editor.layoutInfo.glyphMarginWidth;
        _this._domNode = _this._createDomNode();
        return _this;
    }
    Margin.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    Margin.prototype.getDomNode = function () {
        return this._domNode;
    };
    Margin.prototype._createDomNode = function () {
        var domNode = createFastDomNode(document.createElement('div'));
        domNode.setClassName(Margin.OUTER_CLASS_NAME);
        domNode.setPosition('absolute');
        domNode.setAttribute('role', 'presentation');
        domNode.setAttribute('aria-hidden', 'true');
        this._glyphMarginBackgroundDomNode = createFastDomNode(document.createElement('div'));
        this._glyphMarginBackgroundDomNode.setClassName(Margin.CLASS_NAME);
        domNode.appendChild(this._glyphMarginBackgroundDomNode);
        return domNode;
    };
    // --- begin event handlers
    Margin.prototype.onConfigurationChanged = function (e) {
        if (e.canUseLayerHinting) {
            this._canUseLayerHinting = this._context.configuration.editor.canUseLayerHinting;
        }
        if (e.layoutInfo) {
            this._contentLeft = this._context.configuration.editor.layoutInfo.contentLeft;
            this._glyphMarginLeft = this._context.configuration.editor.layoutInfo.glyphMarginLeft;
            this._glyphMarginWidth = this._context.configuration.editor.layoutInfo.glyphMarginWidth;
        }
        return true;
    };
    Margin.prototype.onScrollChanged = function (e) {
        return _super.prototype.onScrollChanged.call(this, e) || e.scrollTopChanged;
    };
    // --- end event handlers
    Margin.prototype.prepareRender = function (ctx) {
        // Nothing to read
    };
    Margin.prototype.render = function (ctx) {
        this._domNode.setLayerHinting(this._canUseLayerHinting);
        var adjustedScrollTop = ctx.scrollTop - ctx.bigNumbersDelta;
        this._domNode.setTop(-adjustedScrollTop);
        var height = Math.min(ctx.scrollHeight, 1000000);
        this._domNode.setHeight(height);
        this._domNode.setWidth(this._contentLeft);
        this._glyphMarginBackgroundDomNode.setLeft(this._glyphMarginLeft);
        this._glyphMarginBackgroundDomNode.setWidth(this._glyphMarginWidth);
        this._glyphMarginBackgroundDomNode.setHeight(height);
    };
    Margin.CLASS_NAME = 'glyph-margin';
    Margin.OUTER_CLASS_NAME = 'margin';
    return Margin;
}(ViewPart));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$x = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewConfigurationChangedEvent = /** @class */ (function () {
    function ViewConfigurationChangedEvent(source) {
        this.type = 1 /* ViewConfigurationChanged */;
        this.canUseLayerHinting = source.canUseLayerHinting;
        this.pixelRatio = source.pixelRatio;
        this.editorClassName = source.editorClassName;
        this.lineHeight = source.lineHeight;
        this.readOnly = source.readOnly;
        this.accessibilitySupport = source.accessibilitySupport;
        this.emptySelectionClipboard = source.emptySelectionClipboard;
        this.copyWithSyntaxHighlighting = source.copyWithSyntaxHighlighting;
        this.layoutInfo = source.layoutInfo;
        this.fontInfo = source.fontInfo;
        this.viewInfo = source.viewInfo;
        this.wrappingInfo = source.wrappingInfo;
    }
    return ViewConfigurationChangedEvent;
}());
var ViewCursorStateChangedEvent = /** @class */ (function () {
    function ViewCursorStateChangedEvent(selections) {
        this.type = 2 /* ViewCursorStateChanged */;
        this.selections = selections;
    }
    return ViewCursorStateChangedEvent;
}());
var ViewDecorationsChangedEvent = /** @class */ (function () {
    function ViewDecorationsChangedEvent() {
        this.type = 3 /* ViewDecorationsChanged */;
        // Nothing to do
    }
    return ViewDecorationsChangedEvent;
}());
var ViewFlushedEvent = /** @class */ (function () {
    function ViewFlushedEvent() {
        this.type = 4 /* ViewFlushed */;
        // Nothing to do
    }
    return ViewFlushedEvent;
}());
var ViewFocusChangedEvent = /** @class */ (function () {
    function ViewFocusChangedEvent(isFocused) {
        this.type = 5 /* ViewFocusChanged */;
        this.isFocused = isFocused;
    }
    return ViewFocusChangedEvent;
}());
var ViewLineMappingChangedEvent = /** @class */ (function () {
    function ViewLineMappingChangedEvent() {
        this.type = 6 /* ViewLineMappingChanged */;
        // Nothing to do
    }
    return ViewLineMappingChangedEvent;
}());
var ViewLinesChangedEvent = /** @class */ (function () {
    function ViewLinesChangedEvent(fromLineNumber, toLineNumber) {
        this.type = 7 /* ViewLinesChanged */;
        this.fromLineNumber = fromLineNumber;
        this.toLineNumber = toLineNumber;
    }
    return ViewLinesChangedEvent;
}());
var ViewLinesDeletedEvent = /** @class */ (function () {
    function ViewLinesDeletedEvent(fromLineNumber, toLineNumber) {
        this.type = 8 /* ViewLinesDeleted */;
        this.fromLineNumber = fromLineNumber;
        this.toLineNumber = toLineNumber;
    }
    return ViewLinesDeletedEvent;
}());
var ViewLinesInsertedEvent = /** @class */ (function () {
    function ViewLinesInsertedEvent(fromLineNumber, toLineNumber) {
        this.type = 9 /* ViewLinesInserted */;
        this.fromLineNumber = fromLineNumber;
        this.toLineNumber = toLineNumber;
    }
    return ViewLinesInsertedEvent;
}());
var ViewRevealRangeRequestEvent = /** @class */ (function () {
    function ViewRevealRangeRequestEvent(range, verticalType, revealHorizontal, scrollType) {
        this.type = 10 /* ViewRevealRangeRequest */;
        this.range = range;
        this.verticalType = verticalType;
        this.revealHorizontal = revealHorizontal;
        this.scrollType = scrollType;
    }
    return ViewRevealRangeRequestEvent;
}());
var ViewScrollChangedEvent = /** @class */ (function () {
    function ViewScrollChangedEvent(source) {
        this.type = 11 /* ViewScrollChanged */;
        this.scrollWidth = source.scrollWidth;
        this.scrollLeft = source.scrollLeft;
        this.scrollHeight = source.scrollHeight;
        this.scrollTop = source.scrollTop;
        this.scrollWidthChanged = source.scrollWidthChanged;
        this.scrollLeftChanged = source.scrollLeftChanged;
        this.scrollHeightChanged = source.scrollHeightChanged;
        this.scrollTopChanged = source.scrollTopChanged;
    }
    return ViewScrollChangedEvent;
}());
var ViewTokensChangedEvent = /** @class */ (function () {
    function ViewTokensChangedEvent(ranges) {
        this.type = 12 /* ViewTokensChanged */;
        this.ranges = ranges;
    }
    return ViewTokensChangedEvent;
}());
var ViewThemeChangedEvent = /** @class */ (function () {
    function ViewThemeChangedEvent() {
        this.type = 15 /* ViewThemeChanged */;
    }
    return ViewThemeChangedEvent;
}());
var ViewTokensColorsChangedEvent = /** @class */ (function () {
    function ViewTokensColorsChangedEvent() {
        this.type = 13 /* ViewTokensColorsChanged */;
        // Nothing to do
    }
    return ViewTokensColorsChangedEvent;
}());
var ViewZonesChangedEvent = /** @class */ (function () {
    function ViewZonesChangedEvent() {
        this.type = 14 /* ViewZonesChanged */;
        // Nothing to do
    }
    return ViewZonesChangedEvent;
}());
var ViewLanguageConfigurationEvent = /** @class */ (function () {
    function ViewLanguageConfigurationEvent() {
        this.type = 16 /* ViewLanguageConfigurationChanged */;
    }
    return ViewLanguageConfigurationEvent;
}());
var ViewEventEmitter = /** @class */ (function (_super) {
    __extends$x(ViewEventEmitter, _super);
    function ViewEventEmitter() {
        var _this = _super.call(this) || this;
        _this._listeners = [];
        _this._collector = null;
        _this._collectorCnt = 0;
        return _this;
    }
    ViewEventEmitter.prototype.dispose = function () {
        this._listeners = [];
        _super.prototype.dispose.call(this);
    };
    ViewEventEmitter.prototype._beginEmit = function () {
        this._collectorCnt++;
        if (this._collectorCnt === 1) {
            this._collector = new ViewEventsCollector();
        }
        return this._collector;
    };
    ViewEventEmitter.prototype._endEmit = function () {
        this._collectorCnt--;
        if (this._collectorCnt === 0) {
            var events = this._collector.finalize();
            this._collector = null;
            if (events.length > 0) {
                this._emit(events);
            }
        }
    };
    ViewEventEmitter.prototype._emit = function (events) {
        var listeners = this._listeners.slice(0);
        for (var i = 0, len = listeners.length; i < len; i++) {
            safeInvokeListener(listeners[i], events);
        }
    };
    ViewEventEmitter.prototype.addEventListener = function (listener) {
        var _this = this;
        this._listeners.push(listener);
        return toDisposable(function () {
            var listeners = _this._listeners;
            for (var i = 0, len = listeners.length; i < len; i++) {
                if (listeners[i] === listener) {
                    listeners.splice(i, 1);
                    break;
                }
            }
        });
    };
    return ViewEventEmitter;
}(Disposable));
var ViewEventsCollector = /** @class */ (function () {
    function ViewEventsCollector() {
        this._eventsLen = 0;
        this._events = [];
        this._eventsLen = 0;
    }
    ViewEventsCollector.prototype.emit = function (event) {
        this._events[this._eventsLen++] = event;
    };
    ViewEventsCollector.prototype.finalize = function () {
        var result = this._events;
        this._events = [];
        return result;
    };
    return ViewEventsCollector;
}());
function safeInvokeListener(listener, events) {
    try {
        listener(events);
    }
    catch (e) {
        onUnexpectedError(e);
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$y = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var VisibleTextAreaData = /** @class */ (function () {
    function VisibleTextAreaData(top, left, width) {
        this.top = top;
        this.left = left;
        this.width = width;
    }
    VisibleTextAreaData.prototype.setWidth = function (width) {
        return new VisibleTextAreaData(this.top, this.left, width);
    };
    return VisibleTextAreaData;
}());
var canUseZeroSizeTextarea = (isEdgeOrIE || isFirefox);
/**
 * Every time we write to the clipboard, we record a bit of extra metadata here.
 * Every time we read from the cipboard, if the text matches our last written text,
 * we can fetch the previous metadata.
 */
var LocalClipboardMetadataManager = /** @class */ (function () {
    function LocalClipboardMetadataManager() {
        this._lastState = null;
    }
    LocalClipboardMetadataManager.prototype.set = function (state) {
        this._lastState = state;
    };
    LocalClipboardMetadataManager.prototype.get = function (pastedText) {
        if (this._lastState && this._lastState.lastCopiedValue === pastedText) {
            // match!
            return this._lastState;
        }
        this._lastState = null;
        return null;
    };
    LocalClipboardMetadataManager.INSTANCE = new LocalClipboardMetadataManager();
    return LocalClipboardMetadataManager;
}());
var TextAreaHandler = /** @class */ (function (_super) {
    __extends$y(TextAreaHandler, _super);
    function TextAreaHandler(context, viewController, viewHelper) {
        var _this = _super.call(this, context) || this;
        // --- end view API
        _this._primaryCursorVisibleRange = null;
        _this._viewController = viewController;
        _this._viewHelper = viewHelper;
        var conf = _this._context.configuration.editor;
        _this._accessibilitySupport = conf.accessibilitySupport;
        _this._contentLeft = conf.layoutInfo.contentLeft;
        _this._contentWidth = conf.layoutInfo.contentWidth;
        _this._contentHeight = conf.layoutInfo.contentHeight;
        _this._scrollLeft = 0;
        _this._scrollTop = 0;
        _this._fontInfo = conf.fontInfo;
        _this._lineHeight = conf.lineHeight;
        _this._emptySelectionClipboard = conf.emptySelectionClipboard;
        _this._copyWithSyntaxHighlighting = conf.copyWithSyntaxHighlighting;
        _this._visibleTextArea = null;
        _this._selections = [new Selection(1, 1, 1, 1)];
        // Text Area (The focus will always be in the textarea when the cursor is blinking)
        _this.textArea = createFastDomNode(document.createElement('textarea'));
        PartFingerprints.write(_this.textArea, 6 /* TextArea */);
        _this.textArea.setClassName('inputarea');
        _this.textArea.setAttribute('wrap', 'off');
        _this.textArea.setAttribute('autocorrect', 'off');
        _this.textArea.setAttribute('autocapitalize', 'off');
        _this.textArea.setAttribute('autocomplete', 'off');
        _this.textArea.setAttribute('spellcheck', 'false');
        _this.textArea.setAttribute('aria-label', conf.viewInfo.ariaLabel);
        _this.textArea.setAttribute('role', 'textbox');
        _this.textArea.setAttribute('aria-multiline', 'true');
        _this.textArea.setAttribute('aria-haspopup', 'false');
        _this.textArea.setAttribute('aria-autocomplete', 'both');
        _this.textAreaCover = createFastDomNode(document.createElement('div'));
        _this.textAreaCover.setPosition('absolute');
        var simpleModel = {
            getLineCount: function () {
                return _this._context.model.getLineCount();
            },
            getLineMaxColumn: function (lineNumber) {
                return _this._context.model.getLineMaxColumn(lineNumber);
            },
            getValueInRange: function (range, eol) {
                return _this._context.model.getValueInRange(range, eol);
            }
        };
        var textAreaInputHost = {
            getPlainTextToCopy: function () {
                var rawWhatToCopy = _this._context.model.getPlainTextToCopy(_this._selections, _this._emptySelectionClipboard, isWindows);
                var newLineCharacter = _this._context.model.getEOL();
                var isFromEmptySelection = (_this._emptySelectionClipboard && _this._selections.length === 1 && _this._selections[0].isEmpty());
                var multicursorText = (Array.isArray(rawWhatToCopy) ? rawWhatToCopy : null);
                var whatToCopy = (Array.isArray(rawWhatToCopy) ? rawWhatToCopy.join(newLineCharacter) : rawWhatToCopy);
                var metadata = null;
                if (isFromEmptySelection || multicursorText) {
                    // Only store the non-default metadata
                    // When writing "LINE\r\n" to the clipboard and then pasting,
                    // Firefox pastes "LINE\n", so let's work around this quirk
                    var lastCopiedValue = (isFirefox ? whatToCopy.replace(/\r\n/g, '\n') : whatToCopy);
                    metadata = {
                        lastCopiedValue: lastCopiedValue,
                        isFromEmptySelection: (_this._emptySelectionClipboard && _this._selections.length === 1 && _this._selections[0].isEmpty()),
                        multicursorText: multicursorText
                    };
                }
                LocalClipboardMetadataManager.INSTANCE.set(metadata);
                return whatToCopy;
            },
            getHTMLToCopy: function () {
                if (!_this._copyWithSyntaxHighlighting && !CopyOptions.forceCopyWithSyntaxHighlighting) {
                    return null;
                }
                return _this._context.model.getHTMLToCopy(_this._selections, _this._emptySelectionClipboard);
            },
            getScreenReaderContent: function (currentState) {
                if (isIPad) {
                    // Do not place anything in the textarea for the iPad
                    return TextAreaState.EMPTY;
                }
                if (_this._accessibilitySupport === 1 /* Disabled */) {
                    // We know for a fact that a screen reader is not attached
                    // On OSX, we write the character before the cursor to allow for "long-press" composition
                    // Also on OSX, we write the word before the cursor to allow for the Accessibility Keyboard to give good hints
                    if (isMacintosh) {
                        var selection = _this._selections[0];
                        if (selection.isEmpty()) {
                            var position = selection.getStartPosition();
                            var textBefore = _this._getWordBeforePosition(position);
                            if (textBefore.length === 0) {
                                textBefore = _this._getCharacterBeforePosition(position);
                            }
                            if (textBefore.length > 0) {
                                return new TextAreaState(textBefore, textBefore.length, textBefore.length, position, position);
                            }
                        }
                    }
                    return TextAreaState.EMPTY;
                }
                return PagedScreenReaderStrategy.fromEditorSelection(currentState, simpleModel, _this._selections[0], _this._accessibilitySupport === 0 /* Unknown */);
            },
            deduceModelPosition: function (viewAnchorPosition, deltaOffset, lineFeedCnt) {
                return _this._context.model.deduceModelPositionRelativeToViewPosition(viewAnchorPosition, deltaOffset, lineFeedCnt);
            }
        };
        _this._textAreaInput = _this._register(new TextAreaInput(textAreaInputHost, _this.textArea));
        _this._register(_this._textAreaInput.onKeyDown(function (e) {
            _this._viewController.emitKeyDown(e);
        }));
        _this._register(_this._textAreaInput.onKeyUp(function (e) {
            _this._viewController.emitKeyUp(e);
        }));
        _this._register(_this._textAreaInput.onPaste(function (e) {
            var metadata = LocalClipboardMetadataManager.INSTANCE.get(e.text);
            var pasteOnNewLine = false;
            var multicursorText = null;
            if (metadata) {
                pasteOnNewLine = (_this._emptySelectionClipboard && metadata.isFromEmptySelection);
                multicursorText = metadata.multicursorText;
            }
            _this._viewController.paste('keyboard', e.text, pasteOnNewLine, multicursorText);
        }));
        _this._register(_this._textAreaInput.onCut(function () {
            _this._viewController.cut('keyboard');
        }));
        _this._register(_this._textAreaInput.onType(function (e) {
            if (e.replaceCharCnt) {
                _this._viewController.replacePreviousChar('keyboard', e.text, e.replaceCharCnt);
            }
            else {
                _this._viewController.type('keyboard', e.text);
            }
        }));
        _this._register(_this._textAreaInput.onSelectionChangeRequest(function (modelSelection) {
            _this._viewController.setSelection('keyboard', modelSelection);
        }));
        _this._register(_this._textAreaInput.onCompositionStart(function () {
            var lineNumber = _this._selections[0].startLineNumber;
            var column = _this._selections[0].startColumn;
            _this._context.privateViewEventBus.emit(new ViewRevealRangeRequestEvent(new Range(lineNumber, column, lineNumber, column), 0 /* Simple */, true, 1 /* Immediate */));
            // Find range pixel position
            var visibleRange = _this._viewHelper.visibleRangeForPositionRelativeToEditor(lineNumber, column);
            if (visibleRange) {
                _this._visibleTextArea = new VisibleTextAreaData(_this._context.viewLayout.getVerticalOffsetForLineNumber(lineNumber), visibleRange.left, canUseZeroSizeTextarea ? 0 : 1);
                _this._render();
            }
            // Show the textarea
            _this.textArea.setClassName('inputarea ime-input');
            _this._viewController.compositionStart('keyboard');
        }));
        _this._register(_this._textAreaInput.onCompositionUpdate(function (e) {
            if (isEdgeOrIE) {
                // Due to isEdgeOrIE (where the textarea was not cleared initially)
                // we cannot assume the text consists only of the composited text
                _this._visibleTextArea = _this._visibleTextArea.setWidth(0);
            }
            else {
                // adjust width by its size
                _this._visibleTextArea = _this._visibleTextArea.setWidth(measureText(e.data, _this._fontInfo));
            }
            _this._render();
        }));
        _this._register(_this._textAreaInput.onCompositionEnd(function () {
            _this._visibleTextArea = null;
            _this._render();
            _this.textArea.setClassName('inputarea');
            _this._viewController.compositionEnd('keyboard');
        }));
        _this._register(_this._textAreaInput.onFocus(function () {
            _this._context.privateViewEventBus.emit(new ViewFocusChangedEvent(true));
        }));
        _this._register(_this._textAreaInput.onBlur(function () {
            _this._context.privateViewEventBus.emit(new ViewFocusChangedEvent(false));
        }));
        return _this;
    }
    TextAreaHandler.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    TextAreaHandler.prototype._getWordBeforePosition = function (position) {
        var lineContent = this._context.model.getLineContent(position.lineNumber);
        var wordSeparators = getMapForWordSeparators(this._context.configuration.editor.wordSeparators);
        var column = position.column;
        var distance = 0;
        while (column > 1) {
            var charCode = lineContent.charCodeAt(column - 2);
            var charClass = wordSeparators.get(charCode);
            if (charClass !== 0 /* Regular */ || distance > 50) {
                return lineContent.substring(column - 1, position.column - 1);
            }
            distance++;
            column--;
        }
        return lineContent.substring(0, position.column - 1);
    };
    TextAreaHandler.prototype._getCharacterBeforePosition = function (position) {
        if (position.column > 1) {
            var lineContent = this._context.model.getLineContent(position.lineNumber);
            var charBefore = lineContent.charAt(position.column - 2);
            if (!isHighSurrogate(charBefore.charCodeAt(0))) {
                return charBefore;
            }
        }
        return '';
    };
    // --- begin event handlers
    TextAreaHandler.prototype.onConfigurationChanged = function (e) {
        var conf = this._context.configuration.editor;
        if (e.fontInfo) {
            this._fontInfo = conf.fontInfo;
        }
        if (e.viewInfo) {
            this.textArea.setAttribute('aria-label', conf.viewInfo.ariaLabel);
        }
        if (e.layoutInfo) {
            this._contentLeft = conf.layoutInfo.contentLeft;
            this._contentWidth = conf.layoutInfo.contentWidth;
            this._contentHeight = conf.layoutInfo.contentHeight;
        }
        if (e.lineHeight) {
            this._lineHeight = conf.lineHeight;
        }
        if (e.accessibilitySupport) {
            this._accessibilitySupport = conf.accessibilitySupport;
            this._textAreaInput.writeScreenReaderContent('strategy changed');
        }
        if (e.emptySelectionClipboard) {
            this._emptySelectionClipboard = conf.emptySelectionClipboard;
        }
        if (e.copyWithSyntaxHighlighting) {
            this._copyWithSyntaxHighlighting = conf.copyWithSyntaxHighlighting;
        }
        return true;
    };
    TextAreaHandler.prototype.onCursorStateChanged = function (e) {
        this._selections = e.selections.slice(0);
        this._textAreaInput.writeScreenReaderContent('selection changed');
        return true;
    };
    TextAreaHandler.prototype.onDecorationsChanged = function (e) {
        // true for inline decorations that can end up relayouting text
        return true;
    };
    TextAreaHandler.prototype.onFlushed = function (e) {
        return true;
    };
    TextAreaHandler.prototype.onLinesChanged = function (e) {
        return true;
    };
    TextAreaHandler.prototype.onLinesDeleted = function (e) {
        return true;
    };
    TextAreaHandler.prototype.onLinesInserted = function (e) {
        return true;
    };
    TextAreaHandler.prototype.onScrollChanged = function (e) {
        this._scrollLeft = e.scrollLeft;
        this._scrollTop = e.scrollTop;
        return true;
    };
    TextAreaHandler.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    // --- begin view API
    TextAreaHandler.prototype.isFocused = function () {
        return this._textAreaInput.isFocused();
    };
    TextAreaHandler.prototype.focusTextArea = function () {
        this._textAreaInput.focusTextArea();
    };
    TextAreaHandler.prototype.prepareRender = function (ctx) {
        if (this._accessibilitySupport === 2 /* Enabled */) {
            // Do not move the textarea with the cursor, as this generates accessibility events that might confuse screen readers
            // See https://github.com/Microsoft/vscode/issues/26730
            this._primaryCursorVisibleRange = null;
        }
        else {
            var primaryCursorPosition = new Position(this._selections[0].positionLineNumber, this._selections[0].positionColumn);
            this._primaryCursorVisibleRange = ctx.visibleRangeForPosition(primaryCursorPosition);
        }
    };
    TextAreaHandler.prototype.render = function (ctx) {
        this._textAreaInput.writeScreenReaderContent('render');
        this._render();
    };
    TextAreaHandler.prototype._render = function () {
        if (this._visibleTextArea) {
            // The text area is visible for composition reasons
            this._renderInsideEditor(this._visibleTextArea.top - this._scrollTop, this._contentLeft + this._visibleTextArea.left - this._scrollLeft, this._visibleTextArea.width, this._lineHeight, true);
            return;
        }
        if (!this._primaryCursorVisibleRange) {
            // The primary cursor is outside the viewport => place textarea to the top left
            this._renderAtTopLeft();
            return;
        }
        var left = this._contentLeft + this._primaryCursorVisibleRange.left - this._scrollLeft;
        if (left < this._contentLeft || left > this._contentLeft + this._contentWidth) {
            // cursor is outside the viewport
            this._renderAtTopLeft();
            return;
        }
        var top = this._context.viewLayout.getVerticalOffsetForLineNumber(this._selections[0].positionLineNumber) - this._scrollTop;
        if (top < 0 || top > this._contentHeight) {
            // cursor is outside the viewport
            this._renderAtTopLeft();
            return;
        }
        // The primary cursor is in the viewport (at least vertically) => place textarea on the cursor
        this._renderInsideEditor(top, left, canUseZeroSizeTextarea ? 0 : 1, canUseZeroSizeTextarea ? 0 : 1, false);
    };
    TextAreaHandler.prototype._renderInsideEditor = function (top, left, width, height, useEditorFont) {
        var ta = this.textArea;
        var tac = this.textAreaCover;
        if (useEditorFont) {
            Configuration.applyFontInfo(ta, this._fontInfo);
        }
        else {
            ta.setFontSize(1);
            ta.setLineHeight(this._fontInfo.lineHeight);
        }
        ta.setTop(top);
        ta.setLeft(left);
        ta.setWidth(width);
        ta.setHeight(height);
        tac.setTop(0);
        tac.setLeft(0);
        tac.setWidth(0);
        tac.setHeight(0);
    };
    TextAreaHandler.prototype._renderAtTopLeft = function () {
        var ta = this.textArea;
        var tac = this.textAreaCover;
        Configuration.applyFontInfo(ta, this._fontInfo);
        ta.setTop(0);
        ta.setLeft(0);
        tac.setTop(0);
        tac.setLeft(0);
        if (canUseZeroSizeTextarea) {
            ta.setWidth(0);
            ta.setHeight(0);
            tac.setWidth(0);
            tac.setHeight(0);
            return;
        }
        // (in WebKit the textarea is 1px by 1px because it cannot handle input to a 0x0 textarea)
        // specifically, when doing Korean IME, setting the textare to 0x0 breaks IME badly.
        ta.setWidth(1);
        ta.setHeight(1);
        tac.setWidth(1);
        tac.setHeight(1);
        if (this._context.configuration.editor.viewInfo.glyphMargin) {
            tac.setClassName('monaco-editor-background textAreaCover ' + Margin.OUTER_CLASS_NAME);
        }
        else {
            if (this._context.configuration.editor.viewInfo.renderLineNumbers !== 0 /* Off */) {
                tac.setClassName('monaco-editor-background textAreaCover ' + LineNumbersOverlay.CLASS_NAME);
            }
            else {
                tac.setClassName('monaco-editor-background textAreaCover');
            }
        }
    };
    return TextAreaHandler;
}(ViewPart));
function measureText(text, fontInfo) {
    // adjust width by its size
    var canvasElem = document.createElement('canvas');
    var context = canvasElem.getContext('2d');
    context.font = createFontString(fontInfo);
    var metrics = context.measureText(text);
    if (isFirefox) {
        return metrics.width + 2; // +2 for Japanese...
    }
    else {
        return metrics.width;
    }
}
function createFontString(bareFontInfo) {
    return doCreateFontString('normal', bareFontInfo.fontWeight, bareFontInfo.fontSize, bareFontInfo.lineHeight, bareFontInfo.fontFamily);
}
function doCreateFontString(fontStyle, fontWeight, fontSize, lineHeight, fontFamily) {
    // The full font syntax is:
    // style | variant | weight | stretch | size/line-height | fontFamily
    // (https://developer.mozilla.org/en-US/docs/Web/CSS/font)
    // But it appears Edge and IE11 cannot properly parse `stretch`.
    return fontStyle + " normal " + fontWeight + " " + fontSize + "px / " + lineHeight + "px " + fontFamily;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ViewController = /** @class */ (function () {
    function ViewController(configuration, viewModel, outgoingEvents, commandDelegate) {
        this.configuration = configuration;
        this.viewModel = viewModel;
        this.outgoingEvents = outgoingEvents;
        this.commandDelegate = commandDelegate;
    }
    ViewController.prototype._execMouseCommand = function (editorCommand, args) {
        args.source = 'mouse';
        this.commandDelegate.executeEditorCommand(editorCommand, args);
    };
    ViewController.prototype.paste = function (source, text, pasteOnNewLine, multicursorText) {
        this.commandDelegate.paste(source, text, pasteOnNewLine, multicursorText);
    };
    ViewController.prototype.type = function (source, text) {
        this.commandDelegate.type(source, text);
    };
    ViewController.prototype.replacePreviousChar = function (source, text, replaceCharCnt) {
        this.commandDelegate.replacePreviousChar(source, text, replaceCharCnt);
    };
    ViewController.prototype.compositionStart = function (source) {
        this.commandDelegate.compositionStart(source);
    };
    ViewController.prototype.compositionEnd = function (source) {
        this.commandDelegate.compositionEnd(source);
    };
    ViewController.prototype.cut = function (source) {
        this.commandDelegate.cut(source);
    };
    ViewController.prototype.setSelection = function (source, modelSelection) {
        this.commandDelegate.executeEditorCommand(CoreNavigationCommands.SetSelection, {
            source: source,
            selection: modelSelection
        });
    };
    ViewController.prototype._validateViewColumn = function (viewPosition) {
        var minColumn = this.viewModel.getLineMinColumn(viewPosition.lineNumber);
        if (viewPosition.column < minColumn) {
            return new Position(viewPosition.lineNumber, minColumn);
        }
        return viewPosition;
    };
    ViewController.prototype._hasMulticursorModifier = function (data) {
        switch (this.configuration.editor.multiCursorModifier) {
            case 'altKey':
                return data.altKey;
            case 'ctrlKey':
                return data.ctrlKey;
            case 'metaKey':
                return data.metaKey;
        }
        return false;
    };
    ViewController.prototype._hasNonMulticursorModifier = function (data) {
        switch (this.configuration.editor.multiCursorModifier) {
            case 'altKey':
                return data.ctrlKey || data.metaKey;
            case 'ctrlKey':
                return data.altKey || data.metaKey;
            case 'metaKey':
                return data.ctrlKey || data.altKey;
        }
        return false;
    };
    ViewController.prototype.dispatchMouse = function (data) {
        if (data.middleButton) {
            if (data.inSelectionMode) {
                this.columnSelect(data.position, data.mouseColumn);
            }
            else {
                this.moveTo(data.position);
            }
        }
        else if (data.startedOnLineNumbers) {
            // If the dragging started on the gutter, then have operations work on the entire line
            if (this._hasMulticursorModifier(data)) {
                if (data.inSelectionMode) {
                    this.lastCursorLineSelect(data.position);
                }
                else {
                    this.createCursor(data.position, true);
                }
            }
            else {
                if (data.inSelectionMode) {
                    this.lineSelectDrag(data.position);
                }
                else {
                    this.lineSelect(data.position);
                }
            }
        }
        else if (data.mouseDownCount >= 4) {
            this.selectAll();
        }
        else if (data.mouseDownCount === 3) {
            if (this._hasMulticursorModifier(data)) {
                if (data.inSelectionMode) {
                    this.lastCursorLineSelectDrag(data.position);
                }
                else {
                    this.lastCursorLineSelect(data.position);
                }
            }
            else {
                if (data.inSelectionMode) {
                    this.lineSelectDrag(data.position);
                }
                else {
                    this.lineSelect(data.position);
                }
            }
        }
        else if (data.mouseDownCount === 2) {
            if (this._hasMulticursorModifier(data)) {
                this.lastCursorWordSelect(data.position);
            }
            else {
                if (data.inSelectionMode) {
                    this.wordSelectDrag(data.position);
                }
                else {
                    this.wordSelect(data.position);
                }
            }
        }
        else {
            if (this._hasMulticursorModifier(data)) {
                if (!this._hasNonMulticursorModifier(data)) {
                    if (data.shiftKey) {
                        this.columnSelect(data.position, data.mouseColumn);
                    }
                    else {
                        // Do multi-cursor operations only when purely alt is pressed
                        if (data.inSelectionMode) {
                            this.lastCursorMoveToSelect(data.position);
                        }
                        else {
                            this.createCursor(data.position, false);
                        }
                    }
                }
            }
            else {
                if (data.inSelectionMode) {
                    this.moveToSelect(data.position);
                }
                else {
                    this.moveTo(data.position);
                }
            }
        }
    };
    ViewController.prototype._usualArgs = function (viewPosition) {
        viewPosition = this._validateViewColumn(viewPosition);
        return {
            position: this.convertViewToModelPosition(viewPosition),
            viewPosition: viewPosition
        };
    };
    ViewController.prototype.moveTo = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.MoveTo, this._usualArgs(viewPosition));
    };
    ViewController.prototype.moveToSelect = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.MoveToSelect, this._usualArgs(viewPosition));
    };
    ViewController.prototype.columnSelect = function (viewPosition, mouseColumn) {
        viewPosition = this._validateViewColumn(viewPosition);
        this._execMouseCommand(CoreNavigationCommands.ColumnSelect, {
            position: this.convertViewToModelPosition(viewPosition),
            viewPosition: viewPosition,
            mouseColumn: mouseColumn
        });
    };
    ViewController.prototype.createCursor = function (viewPosition, wholeLine) {
        viewPosition = this._validateViewColumn(viewPosition);
        this._execMouseCommand(CoreNavigationCommands.CreateCursor, {
            position: this.convertViewToModelPosition(viewPosition),
            viewPosition: viewPosition,
            wholeLine: wholeLine
        });
    };
    ViewController.prototype.lastCursorMoveToSelect = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.LastCursorMoveToSelect, this._usualArgs(viewPosition));
    };
    ViewController.prototype.wordSelect = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.WordSelect, this._usualArgs(viewPosition));
    };
    ViewController.prototype.wordSelectDrag = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.WordSelectDrag, this._usualArgs(viewPosition));
    };
    ViewController.prototype.lastCursorWordSelect = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.LastCursorWordSelect, this._usualArgs(viewPosition));
    };
    ViewController.prototype.lineSelect = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.LineSelect, this._usualArgs(viewPosition));
    };
    ViewController.prototype.lineSelectDrag = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.LineSelectDrag, this._usualArgs(viewPosition));
    };
    ViewController.prototype.lastCursorLineSelect = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.LastCursorLineSelect, this._usualArgs(viewPosition));
    };
    ViewController.prototype.lastCursorLineSelectDrag = function (viewPosition) {
        this._execMouseCommand(CoreNavigationCommands.LastCursorLineSelectDrag, this._usualArgs(viewPosition));
    };
    ViewController.prototype.selectAll = function () {
        this._execMouseCommand(CoreNavigationCommands.SelectAll, {});
    };
    // ----------------------
    ViewController.prototype.convertViewToModelPosition = function (viewPosition) {
        return this.viewModel.coordinatesConverter.convertViewPositionToModelPosition(viewPosition);
    };
    ViewController.prototype.emitKeyDown = function (e) {
        this.outgoingEvents.emitKeyDown(e);
    };
    ViewController.prototype.emitKeyUp = function (e) {
        this.outgoingEvents.emitKeyUp(e);
    };
    ViewController.prototype.emitContextMenu = function (e) {
        this.outgoingEvents.emitContextMenu(e);
    };
    ViewController.prototype.emitMouseMove = function (e) {
        this.outgoingEvents.emitMouseMove(e);
    };
    ViewController.prototype.emitMouseLeave = function (e) {
        this.outgoingEvents.emitMouseLeave(e);
    };
    ViewController.prototype.emitMouseUp = function (e) {
        this.outgoingEvents.emitMouseUp(e);
    };
    ViewController.prototype.emitMouseDown = function (e) {
        this.outgoingEvents.emitMouseDown(e);
    };
    ViewController.prototype.emitMouseDrag = function (e) {
        this.outgoingEvents.emitMouseDrag(e);
    };
    ViewController.prototype.emitMouseDrop = function (e) {
        this.outgoingEvents.emitMouseDrop(e);
    };
    return ViewController;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var RenderedLinesCollection = /** @class */ (function () {
    function RenderedLinesCollection(createLine) {
        this._createLine = createLine;
        this._set(1, []);
    }
    RenderedLinesCollection.prototype.flush = function () {
        this._set(1, []);
    };
    RenderedLinesCollection.prototype._set = function (rendLineNumberStart, lines) {
        this._lines = lines;
        this._rendLineNumberStart = rendLineNumberStart;
    };
    RenderedLinesCollection.prototype._get = function () {
        return {
            rendLineNumberStart: this._rendLineNumberStart,
            lines: this._lines
        };
    };
    /**
     * @returns Inclusive line number that is inside this collection
     */
    RenderedLinesCollection.prototype.getStartLineNumber = function () {
        return this._rendLineNumberStart;
    };
    /**
     * @returns Inclusive line number that is inside this collection
     */
    RenderedLinesCollection.prototype.getEndLineNumber = function () {
        return this._rendLineNumberStart + this._lines.length - 1;
    };
    RenderedLinesCollection.prototype.getCount = function () {
        return this._lines.length;
    };
    RenderedLinesCollection.prototype.getLine = function (lineNumber) {
        var lineIndex = lineNumber - this._rendLineNumberStart;
        if (lineIndex < 0 || lineIndex >= this._lines.length) {
            throw new Error('Illegal value for lineNumber');
        }
        return this._lines[lineIndex];
    };
    /**
     * @returns Lines that were removed from this collection
     */
    RenderedLinesCollection.prototype.onLinesDeleted = function (deleteFromLineNumber, deleteToLineNumber) {
        if (this.getCount() === 0) {
            // no lines
            return null;
        }
        var startLineNumber = this.getStartLineNumber();
        var endLineNumber = this.getEndLineNumber();
        if (deleteToLineNumber < startLineNumber) {
            // deleting above the viewport
            var deleteCnt = deleteToLineNumber - deleteFromLineNumber + 1;
            this._rendLineNumberStart -= deleteCnt;
            return null;
        }
        if (deleteFromLineNumber > endLineNumber) {
            // deleted below the viewport
            return null;
        }
        // Record what needs to be deleted
        var deleteStartIndex = 0;
        var deleteCount = 0;
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            var lineIndex = lineNumber - this._rendLineNumberStart;
            if (deleteFromLineNumber <= lineNumber && lineNumber <= deleteToLineNumber) {
                // this is a line to be deleted
                if (deleteCount === 0) {
                    // this is the first line to be deleted
                    deleteStartIndex = lineIndex;
                    deleteCount = 1;
                }
                else {
                    deleteCount++;
                }
            }
        }
        // Adjust this._rendLineNumberStart for lines deleted above
        if (deleteFromLineNumber < startLineNumber) {
            // Something was deleted above
            var deleteAboveCount = 0;
            if (deleteToLineNumber < startLineNumber) {
                // the entire deleted lines are above
                deleteAboveCount = deleteToLineNumber - deleteFromLineNumber + 1;
            }
            else {
                deleteAboveCount = startLineNumber - deleteFromLineNumber;
            }
            this._rendLineNumberStart -= deleteAboveCount;
        }
        var deleted = this._lines.splice(deleteStartIndex, deleteCount);
        return deleted;
    };
    RenderedLinesCollection.prototype.onLinesChanged = function (changeFromLineNumber, changeToLineNumber) {
        if (this.getCount() === 0) {
            // no lines
            return false;
        }
        var startLineNumber = this.getStartLineNumber();
        var endLineNumber = this.getEndLineNumber();
        var someoneNotified = false;
        for (var changedLineNumber = changeFromLineNumber; changedLineNumber <= changeToLineNumber; changedLineNumber++) {
            if (changedLineNumber >= startLineNumber && changedLineNumber <= endLineNumber) {
                // Notify the line
                this._lines[changedLineNumber - this._rendLineNumberStart].onContentChanged();
                someoneNotified = true;
            }
        }
        return someoneNotified;
    };
    RenderedLinesCollection.prototype.onLinesInserted = function (insertFromLineNumber, insertToLineNumber) {
        if (this.getCount() === 0) {
            // no lines
            return null;
        }
        var insertCnt = insertToLineNumber - insertFromLineNumber + 1;
        var startLineNumber = this.getStartLineNumber();
        var endLineNumber = this.getEndLineNumber();
        if (insertFromLineNumber <= startLineNumber) {
            // inserting above the viewport
            this._rendLineNumberStart += insertCnt;
            return null;
        }
        if (insertFromLineNumber > endLineNumber) {
            // inserting below the viewport
            return null;
        }
        if (insertCnt + insertFromLineNumber > endLineNumber) {
            // insert inside the viewport in such a way that all remaining lines are pushed outside
            var deleted = this._lines.splice(insertFromLineNumber - this._rendLineNumberStart, endLineNumber - insertFromLineNumber + 1);
            return deleted;
        }
        // insert inside the viewport, push out some lines, but not all remaining lines
        var newLines = [];
        for (var i = 0; i < insertCnt; i++) {
            newLines[i] = this._createLine();
        }
        var insertIndex = insertFromLineNumber - this._rendLineNumberStart;
        var beforeLines = this._lines.slice(0, insertIndex);
        var afterLines = this._lines.slice(insertIndex, this._lines.length - insertCnt);
        var deletedLines = this._lines.slice(this._lines.length - insertCnt, this._lines.length);
        this._lines = beforeLines.concat(newLines).concat(afterLines);
        return deletedLines;
    };
    RenderedLinesCollection.prototype.onTokensChanged = function (ranges) {
        if (this.getCount() === 0) {
            // no lines
            return false;
        }
        var startLineNumber = this.getStartLineNumber();
        var endLineNumber = this.getEndLineNumber();
        var notifiedSomeone = false;
        for (var i = 0, len = ranges.length; i < len; i++) {
            var rng = ranges[i];
            if (rng.toLineNumber < startLineNumber || rng.fromLineNumber > endLineNumber) {
                // range outside viewport
                continue;
            }
            var from = Math.max(startLineNumber, rng.fromLineNumber);
            var to = Math.min(endLineNumber, rng.toLineNumber);
            for (var lineNumber = from; lineNumber <= to; lineNumber++) {
                var lineIndex = lineNumber - this._rendLineNumberStart;
                this._lines[lineIndex].onTokensChanged();
                notifiedSomeone = true;
            }
        }
        return notifiedSomeone;
    };
    return RenderedLinesCollection;
}());
var VisibleLinesCollection = /** @class */ (function () {
    function VisibleLinesCollection(host) {
        var _this = this;
        this._host = host;
        this.domNode = this._createDomNode();
        this._linesCollection = new RenderedLinesCollection(function () { return _this._host.createVisibleLine(); });
    }
    VisibleLinesCollection.prototype._createDomNode = function () {
        var domNode = createFastDomNode(document.createElement('div'));
        domNode.setClassName('view-layer');
        domNode.setPosition('absolute');
        domNode.domNode.setAttribute('role', 'presentation');
        domNode.domNode.setAttribute('aria-hidden', 'true');
        return domNode;
    };
    // ---- begin view event handlers
    VisibleLinesCollection.prototype.onConfigurationChanged = function (e) {
        return e.layoutInfo;
    };
    VisibleLinesCollection.prototype.onFlushed = function (e) {
        this._linesCollection.flush();
        // No need to clear the dom node because a full .innerHTML will occur in ViewLayerRenderer._render
        return true;
    };
    VisibleLinesCollection.prototype.onLinesChanged = function (e) {
        return this._linesCollection.onLinesChanged(e.fromLineNumber, e.toLineNumber);
    };
    VisibleLinesCollection.prototype.onLinesDeleted = function (e) {
        var deleted = this._linesCollection.onLinesDeleted(e.fromLineNumber, e.toLineNumber);
        if (deleted) {
            // Remove from DOM
            for (var i = 0, len = deleted.length; i < len; i++) {
                var lineDomNode = deleted[i].getDomNode();
                if (lineDomNode) {
                    this.domNode.domNode.removeChild(lineDomNode);
                }
            }
        }
        return true;
    };
    VisibleLinesCollection.prototype.onLinesInserted = function (e) {
        var deleted = this._linesCollection.onLinesInserted(e.fromLineNumber, e.toLineNumber);
        if (deleted) {
            // Remove from DOM
            for (var i = 0, len = deleted.length; i < len; i++) {
                var lineDomNode = deleted[i].getDomNode();
                if (lineDomNode) {
                    this.domNode.domNode.removeChild(lineDomNode);
                }
            }
        }
        return true;
    };
    VisibleLinesCollection.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged;
    };
    VisibleLinesCollection.prototype.onTokensChanged = function (e) {
        return this._linesCollection.onTokensChanged(e.ranges);
    };
    VisibleLinesCollection.prototype.onZonesChanged = function (e) {
        return true;
    };
    // ---- end view event handlers
    VisibleLinesCollection.prototype.getStartLineNumber = function () {
        return this._linesCollection.getStartLineNumber();
    };
    VisibleLinesCollection.prototype.getEndLineNumber = function () {
        return this._linesCollection.getEndLineNumber();
    };
    VisibleLinesCollection.prototype.getVisibleLine = function (lineNumber) {
        return this._linesCollection.getLine(lineNumber);
    };
    VisibleLinesCollection.prototype.renderLines = function (viewportData) {
        var inp = this._linesCollection._get();
        var renderer = new ViewLayerRenderer(this.domNode.domNode, this._host, viewportData);
        var ctx = {
            rendLineNumberStart: inp.rendLineNumberStart,
            lines: inp.lines,
            linesLength: inp.lines.length
        };
        // Decide if this render will do a single update (single large .innerHTML) or many updates (inserting/removing dom nodes)
        var resCtx = renderer.render(ctx, viewportData.startLineNumber, viewportData.endLineNumber, viewportData.relativeVerticalOffset);
        this._linesCollection._set(resCtx.rendLineNumberStart, resCtx.lines);
    };
    return VisibleLinesCollection;
}());
var ViewLayerRenderer = /** @class */ (function () {
    function ViewLayerRenderer(domNode, host, viewportData) {
        this.domNode = domNode;
        this.host = host;
        this.viewportData = viewportData;
    }
    ViewLayerRenderer.prototype.render = function (inContext, startLineNumber, stopLineNumber, deltaTop) {
        var ctx = {
            rendLineNumberStart: inContext.rendLineNumberStart,
            lines: inContext.lines.slice(0),
            linesLength: inContext.linesLength
        };
        if ((ctx.rendLineNumberStart + ctx.linesLength - 1 < startLineNumber) || (stopLineNumber < ctx.rendLineNumberStart)) {
            // There is no overlap whatsoever
            ctx.rendLineNumberStart = startLineNumber;
            ctx.linesLength = stopLineNumber - startLineNumber + 1;
            ctx.lines = [];
            for (var x = startLineNumber; x <= stopLineNumber; x++) {
                ctx.lines[x - startLineNumber] = this.host.createVisibleLine();
            }
            this._finishRendering(ctx, true, deltaTop);
            return ctx;
        }
        // Update lines which will remain untouched
        this._renderUntouchedLines(ctx, Math.max(startLineNumber - ctx.rendLineNumberStart, 0), Math.min(stopLineNumber - ctx.rendLineNumberStart, ctx.linesLength - 1), deltaTop, startLineNumber);
        if (ctx.rendLineNumberStart > startLineNumber) {
            // Insert lines before
            var fromLineNumber = startLineNumber;
            var toLineNumber = Math.min(stopLineNumber, ctx.rendLineNumberStart - 1);
            if (fromLineNumber <= toLineNumber) {
                this._insertLinesBefore(ctx, fromLineNumber, toLineNumber, deltaTop, startLineNumber);
                ctx.linesLength += toLineNumber - fromLineNumber + 1;
            }
        }
        else if (ctx.rendLineNumberStart < startLineNumber) {
            // Remove lines before
            var removeCnt = Math.min(ctx.linesLength, startLineNumber - ctx.rendLineNumberStart);
            if (removeCnt > 0) {
                this._removeLinesBefore(ctx, removeCnt);
                ctx.linesLength -= removeCnt;
            }
        }
        ctx.rendLineNumberStart = startLineNumber;
        if (ctx.rendLineNumberStart + ctx.linesLength - 1 < stopLineNumber) {
            // Insert lines after
            var fromLineNumber = ctx.rendLineNumberStart + ctx.linesLength;
            var toLineNumber = stopLineNumber;
            if (fromLineNumber <= toLineNumber) {
                this._insertLinesAfter(ctx, fromLineNumber, toLineNumber, deltaTop, startLineNumber);
                ctx.linesLength += toLineNumber - fromLineNumber + 1;
            }
        }
        else if (ctx.rendLineNumberStart + ctx.linesLength - 1 > stopLineNumber) {
            // Remove lines after
            var fromLineNumber = Math.max(0, stopLineNumber - ctx.rendLineNumberStart + 1);
            var toLineNumber = ctx.linesLength - 1;
            var removeCnt = toLineNumber - fromLineNumber + 1;
            if (removeCnt > 0) {
                this._removeLinesAfter(ctx, removeCnt);
                ctx.linesLength -= removeCnt;
            }
        }
        this._finishRendering(ctx, false, deltaTop);
        return ctx;
    };
    ViewLayerRenderer.prototype._renderUntouchedLines = function (ctx, startIndex, endIndex, deltaTop, deltaLN) {
        var rendLineNumberStart = ctx.rendLineNumberStart;
        var lines = ctx.lines;
        for (var i = startIndex; i <= endIndex; i++) {
            var lineNumber = rendLineNumberStart + i;
            lines[i].layoutLine(lineNumber, deltaTop[lineNumber - deltaLN]);
        }
    };
    ViewLayerRenderer.prototype._insertLinesBefore = function (ctx, fromLineNumber, toLineNumber, deltaTop, deltaLN) {
        var newLines = [];
        var newLinesLen = 0;
        for (var lineNumber = fromLineNumber; lineNumber <= toLineNumber; lineNumber++) {
            newLines[newLinesLen++] = this.host.createVisibleLine();
        }
        ctx.lines = newLines.concat(ctx.lines);
    };
    ViewLayerRenderer.prototype._removeLinesBefore = function (ctx, removeCount) {
        for (var i = 0; i < removeCount; i++) {
            var lineDomNode = ctx.lines[i].getDomNode();
            if (lineDomNode) {
                this.domNode.removeChild(lineDomNode);
            }
        }
        ctx.lines.splice(0, removeCount);
    };
    ViewLayerRenderer.prototype._insertLinesAfter = function (ctx, fromLineNumber, toLineNumber, deltaTop, deltaLN) {
        var newLines = [];
        var newLinesLen = 0;
        for (var lineNumber = fromLineNumber; lineNumber <= toLineNumber; lineNumber++) {
            newLines[newLinesLen++] = this.host.createVisibleLine();
        }
        ctx.lines = ctx.lines.concat(newLines);
    };
    ViewLayerRenderer.prototype._removeLinesAfter = function (ctx, removeCount) {
        var removeIndex = ctx.linesLength - removeCount;
        for (var i = 0; i < removeCount; i++) {
            var lineDomNode = ctx.lines[removeIndex + i].getDomNode();
            if (lineDomNode) {
                this.domNode.removeChild(lineDomNode);
            }
        }
        ctx.lines.splice(removeIndex, removeCount);
    };
    ViewLayerRenderer.prototype._finishRenderingNewLines = function (ctx, domNodeIsEmpty, newLinesHTML, wasNew) {
        var lastChild = this.domNode.lastChild;
        if (domNodeIsEmpty || !lastChild) {
            this.domNode.innerHTML = newLinesHTML;
        }
        else {
            lastChild.insertAdjacentHTML('afterend', newLinesHTML);
        }
        var currChild = this.domNode.lastChild;
        for (var i = ctx.linesLength - 1; i >= 0; i--) {
            var line = ctx.lines[i];
            if (wasNew[i]) {
                line.setDomNode(currChild);
                currChild = currChild.previousSibling;
            }
        }
    };
    ViewLayerRenderer.prototype._finishRenderingInvalidLines = function (ctx, invalidLinesHTML, wasInvalid) {
        var hugeDomNode = document.createElement('div');
        hugeDomNode.innerHTML = invalidLinesHTML;
        for (var i = 0; i < ctx.linesLength; i++) {
            var line = ctx.lines[i];
            if (wasInvalid[i]) {
                var source = hugeDomNode.firstChild;
                var lineDomNode = line.getDomNode();
                lineDomNode.parentNode.replaceChild(source, lineDomNode);
                line.setDomNode(source);
            }
        }
    };
    ViewLayerRenderer.prototype._finishRendering = function (ctx, domNodeIsEmpty, deltaTop) {
        var sb = ViewLayerRenderer._sb;
        var linesLength = ctx.linesLength;
        var lines = ctx.lines;
        var rendLineNumberStart = ctx.rendLineNumberStart;
        var wasNew = [];
        {
            sb.reset();
            var hadNewLine = false;
            for (var i = 0; i < linesLength; i++) {
                var line = lines[i];
                wasNew[i] = false;
                var lineDomNode = line.getDomNode();
                if (lineDomNode) {
                    // line is not new
                    continue;
                }
                var renderResult = line.renderLine(i + rendLineNumberStart, deltaTop[i], this.viewportData, sb);
                if (!renderResult) {
                    // line does not need rendering
                    continue;
                }
                wasNew[i] = true;
                hadNewLine = true;
            }
            if (hadNewLine) {
                this._finishRenderingNewLines(ctx, domNodeIsEmpty, sb.build(), wasNew);
            }
        }
        {
            sb.reset();
            var hadInvalidLine = false;
            var wasInvalid = [];
            for (var i = 0; i < linesLength; i++) {
                var line = lines[i];
                wasInvalid[i] = false;
                if (wasNew[i]) {
                    // line was new
                    continue;
                }
                var renderResult = line.renderLine(i + rendLineNumberStart, deltaTop[i], this.viewportData, sb);
                if (!renderResult) {
                    // line does not need rendering
                    continue;
                }
                wasInvalid[i] = true;
                hadInvalidLine = true;
            }
            if (hadInvalidLine) {
                this._finishRenderingInvalidLines(ctx, sb.build(), wasInvalid);
            }
        }
    };
    ViewLayerRenderer._sb = createStringBuilder(100000);
    return ViewLayerRenderer;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$z = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewOverlays = /** @class */ (function (_super) {
    __extends$z(ViewOverlays, _super);
    function ViewOverlays(context) {
        var _this = _super.call(this, context) || this;
        _this._visibleLines = new VisibleLinesCollection(_this);
        _this.domNode = _this._visibleLines.domNode;
        _this._dynamicOverlays = [];
        _this._isFocused = false;
        _this.domNode.setClassName('view-overlays');
        return _this;
    }
    ViewOverlays.prototype.shouldRender = function () {
        if (_super.prototype.shouldRender.call(this)) {
            return true;
        }
        for (var i = 0, len = this._dynamicOverlays.length; i < len; i++) {
            var dynamicOverlay = this._dynamicOverlays[i];
            if (dynamicOverlay.shouldRender()) {
                return true;
            }
        }
        return false;
    };
    ViewOverlays.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        for (var i = 0, len = this._dynamicOverlays.length; i < len; i++) {
            var dynamicOverlay = this._dynamicOverlays[i];
            dynamicOverlay.dispose();
        }
        this._dynamicOverlays = [];
    };
    ViewOverlays.prototype.getDomNode = function () {
        return this.domNode;
    };
    // ---- begin IVisibleLinesHost
    ViewOverlays.prototype.createVisibleLine = function () {
        return new ViewOverlayLine(this._context.configuration, this._dynamicOverlays);
    };
    // ---- end IVisibleLinesHost
    ViewOverlays.prototype.addDynamicOverlay = function (overlay) {
        this._dynamicOverlays.push(overlay);
    };
    // ----- event handlers
    ViewOverlays.prototype.onConfigurationChanged = function (e) {
        this._visibleLines.onConfigurationChanged(e);
        var startLineNumber = this._visibleLines.getStartLineNumber();
        var endLineNumber = this._visibleLines.getEndLineNumber();
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            var line = this._visibleLines.getVisibleLine(lineNumber);
            line.onConfigurationChanged(e);
        }
        return true;
    };
    ViewOverlays.prototype.onFlushed = function (e) {
        return this._visibleLines.onFlushed(e);
    };
    ViewOverlays.prototype.onFocusChanged = function (e) {
        this._isFocused = e.isFocused;
        return true;
    };
    ViewOverlays.prototype.onLinesChanged = function (e) {
        return this._visibleLines.onLinesChanged(e);
    };
    ViewOverlays.prototype.onLinesDeleted = function (e) {
        return this._visibleLines.onLinesDeleted(e);
    };
    ViewOverlays.prototype.onLinesInserted = function (e) {
        return this._visibleLines.onLinesInserted(e);
    };
    ViewOverlays.prototype.onScrollChanged = function (e) {
        return this._visibleLines.onScrollChanged(e) || true;
    };
    ViewOverlays.prototype.onTokensChanged = function (e) {
        return this._visibleLines.onTokensChanged(e);
    };
    ViewOverlays.prototype.onZonesChanged = function (e) {
        return this._visibleLines.onZonesChanged(e);
    };
    // ----- end event handlers
    ViewOverlays.prototype.prepareRender = function (ctx) {
        var toRender = this._dynamicOverlays.filter(function (overlay) { return overlay.shouldRender(); });
        for (var i = 0, len = toRender.length; i < len; i++) {
            var dynamicOverlay = toRender[i];
            dynamicOverlay.prepareRender(ctx);
            dynamicOverlay.onDidRender();
        }
    };
    ViewOverlays.prototype.render = function (ctx) {
        // Overwriting to bypass `shouldRender` flag
        this._viewOverlaysRender(ctx);
        this.domNode.toggleClassName('focused', this._isFocused);
    };
    ViewOverlays.prototype._viewOverlaysRender = function (ctx) {
        this._visibleLines.renderLines(ctx.viewportData);
    };
    return ViewOverlays;
}(ViewPart));
var ViewOverlayLine = /** @class */ (function () {
    function ViewOverlayLine(configuration, dynamicOverlays) {
        this._configuration = configuration;
        this._lineHeight = this._configuration.editor.lineHeight;
        this._dynamicOverlays = dynamicOverlays;
        this._domNode = null;
        this._renderedContent = null;
    }
    ViewOverlayLine.prototype.getDomNode = function () {
        if (!this._domNode) {
            return null;
        }
        return this._domNode.domNode;
    };
    ViewOverlayLine.prototype.setDomNode = function (domNode) {
        this._domNode = createFastDomNode(domNode);
    };
    ViewOverlayLine.prototype.onContentChanged = function () {
        // Nothing
    };
    ViewOverlayLine.prototype.onTokensChanged = function () {
        // Nothing
    };
    ViewOverlayLine.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._configuration.editor.lineHeight;
        }
    };
    ViewOverlayLine.prototype.renderLine = function (lineNumber, deltaTop, viewportData, sb) {
        var result = '';
        for (var i = 0, len = this._dynamicOverlays.length; i < len; i++) {
            var dynamicOverlay = this._dynamicOverlays[i];
            result += dynamicOverlay.render(viewportData.startLineNumber, lineNumber);
        }
        if (this._renderedContent === result) {
            // No rendering needed
            return false;
        }
        this._renderedContent = result;
        sb.appendASCIIString('<div style="position:absolute;top:');
        sb.appendASCIIString(String(deltaTop));
        sb.appendASCIIString('px;width:100%;height:');
        sb.appendASCIIString(String(this._lineHeight));
        sb.appendASCIIString('px;">');
        sb.appendASCIIString(result);
        sb.appendASCIIString('</div>');
        return true;
    };
    ViewOverlayLine.prototype.layoutLine = function (lineNumber, deltaTop) {
        if (this._domNode) {
            this._domNode.setTop(deltaTop);
            this._domNode.setHeight(this._lineHeight);
        }
    };
    return ViewOverlayLine;
}());
var ContentViewOverlays = /** @class */ (function (_super) {
    __extends$z(ContentViewOverlays, _super);
    function ContentViewOverlays(context) {
        var _this = _super.call(this, context) || this;
        _this._contentWidth = _this._context.configuration.editor.layoutInfo.contentWidth;
        _this.domNode.setHeight(0);
        return _this;
    }
    // --- begin event handlers
    ContentViewOverlays.prototype.onConfigurationChanged = function (e) {
        if (e.layoutInfo) {
            this._contentWidth = this._context.configuration.editor.layoutInfo.contentWidth;
        }
        return _super.prototype.onConfigurationChanged.call(this, e);
    };
    ContentViewOverlays.prototype.onScrollChanged = function (e) {
        return _super.prototype.onScrollChanged.call(this, e) || e.scrollWidthChanged;
    };
    // --- end event handlers
    ContentViewOverlays.prototype._viewOverlaysRender = function (ctx) {
        _super.prototype._viewOverlaysRender.call(this, ctx);
        this.domNode.setWidth(Math.max(ctx.scrollWidth, this._contentWidth));
    };
    return ContentViewOverlays;
}(ViewOverlays));
var MarginViewOverlays = /** @class */ (function (_super) {
    __extends$z(MarginViewOverlays, _super);
    function MarginViewOverlays(context) {
        var _this = _super.call(this, context) || this;
        _this._contentLeft = _this._context.configuration.editor.layoutInfo.contentLeft;
        _this.domNode.setClassName('margin-view-overlays');
        _this.domNode.setWidth(1);
        Configuration.applyFontInfo(_this.domNode, _this._context.configuration.editor.fontInfo);
        return _this;
    }
    MarginViewOverlays.prototype.onConfigurationChanged = function (e) {
        var shouldRender = false;
        if (e.fontInfo) {
            Configuration.applyFontInfo(this.domNode, this._context.configuration.editor.fontInfo);
            shouldRender = true;
        }
        if (e.layoutInfo) {
            this._contentLeft = this._context.configuration.editor.layoutInfo.contentLeft;
            shouldRender = true;
        }
        return _super.prototype.onConfigurationChanged.call(this, e) || shouldRender;
    };
    MarginViewOverlays.prototype.onScrollChanged = function (e) {
        return _super.prototype.onScrollChanged.call(this, e) || e.scrollHeightChanged;
    };
    MarginViewOverlays.prototype._viewOverlaysRender = function (ctx) {
        _super.prototype._viewOverlaysRender.call(this, ctx);
        var height = Math.min(ctx.scrollHeight, 1000000);
        this.domNode.setHeight(height);
        this.domNode.setWidth(this._contentLeft);
    };
    return MarginViewOverlays;
}(ViewOverlays));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$A = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Coordinate = /** @class */ (function () {
    function Coordinate(top, left) {
        this.top = top;
        this.left = left;
    }
    return Coordinate;
}());
var ViewContentWidgets = /** @class */ (function (_super) {
    __extends$A(ViewContentWidgets, _super);
    function ViewContentWidgets(context, viewDomNode) {
        var _this = _super.call(this, context) || this;
        _this._viewDomNode = viewDomNode;
        _this._widgets = {};
        _this.domNode = createFastDomNode(document.createElement('div'));
        PartFingerprints.write(_this.domNode, 1 /* ContentWidgets */);
        _this.domNode.setClassName('contentWidgets');
        _this.domNode.setPosition('absolute');
        _this.domNode.setTop(0);
        _this.overflowingContentWidgetsDomNode = createFastDomNode(document.createElement('div'));
        PartFingerprints.write(_this.overflowingContentWidgetsDomNode, 2 /* OverflowingContentWidgets */);
        _this.overflowingContentWidgetsDomNode.setClassName('overflowingContentWidgets');
        return _this;
    }
    ViewContentWidgets.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this._widgets = {};
    };
    // --- begin event handlers
    ViewContentWidgets.prototype.onConfigurationChanged = function (e) {
        var keys = Object.keys(this._widgets);
        for (var i = 0, len = keys.length; i < len; i++) {
            var widgetId = keys[i];
            this._widgets[widgetId].onConfigurationChanged(e);
        }
        return true;
    };
    ViewContentWidgets.prototype.onDecorationsChanged = function (e) {
        // true for inline decorations that can end up relayouting text
        return true;
    };
    ViewContentWidgets.prototype.onFlushed = function (e) {
        return true;
    };
    ViewContentWidgets.prototype.onLineMappingChanged = function (e) {
        var keys = Object.keys(this._widgets);
        for (var i = 0, len = keys.length; i < len; i++) {
            var widgetId = keys[i];
            this._widgets[widgetId].onLineMappingChanged(e);
        }
        return true;
    };
    ViewContentWidgets.prototype.onLinesChanged = function (e) {
        return true;
    };
    ViewContentWidgets.prototype.onLinesDeleted = function (e) {
        return true;
    };
    ViewContentWidgets.prototype.onLinesInserted = function (e) {
        return true;
    };
    ViewContentWidgets.prototype.onScrollChanged = function (e) {
        return true;
    };
    ViewContentWidgets.prototype.onZonesChanged = function (e) {
        return true;
    };
    // ---- end view event handlers
    ViewContentWidgets.prototype.addWidget = function (_widget) {
        var myWidget = new Widget(this._context, this._viewDomNode, _widget);
        this._widgets[myWidget.id] = myWidget;
        if (myWidget.allowEditorOverflow) {
            this.overflowingContentWidgetsDomNode.appendChild(myWidget.domNode);
        }
        else {
            this.domNode.appendChild(myWidget.domNode);
        }
        this.setShouldRender();
    };
    ViewContentWidgets.prototype.setWidgetPosition = function (widget, position, range, preference) {
        var myWidget = this._widgets[widget.getId()];
        myWidget.setPosition(position, range, preference);
        this.setShouldRender();
    };
    ViewContentWidgets.prototype.removeWidget = function (widget) {
        var widgetId = widget.getId();
        if (this._widgets.hasOwnProperty(widgetId)) {
            var myWidget = this._widgets[widgetId];
            delete this._widgets[widgetId];
            var domNode = myWidget.domNode.domNode;
            domNode.parentNode.removeChild(domNode);
            domNode.removeAttribute('monaco-visible-content-widget');
            this.setShouldRender();
        }
    };
    ViewContentWidgets.prototype.shouldSuppressMouseDownOnWidget = function (widgetId) {
        if (this._widgets.hasOwnProperty(widgetId)) {
            return this._widgets[widgetId].suppressMouseDown;
        }
        return false;
    };
    ViewContentWidgets.prototype.onBeforeRender = function (viewportData) {
        var keys = Object.keys(this._widgets);
        for (var i = 0, len = keys.length; i < len; i++) {
            var widgetId = keys[i];
            this._widgets[widgetId].onBeforeRender(viewportData);
        }
    };
    ViewContentWidgets.prototype.prepareRender = function (ctx) {
        var keys = Object.keys(this._widgets);
        for (var i = 0, len = keys.length; i < len; i++) {
            var widgetId = keys[i];
            this._widgets[widgetId].prepareRender(ctx);
        }
    };
    ViewContentWidgets.prototype.render = function (ctx) {
        var keys = Object.keys(this._widgets);
        for (var i = 0, len = keys.length; i < len; i++) {
            var widgetId = keys[i];
            this._widgets[widgetId].render(ctx);
        }
    };
    return ViewContentWidgets;
}(ViewPart));
var Widget = /** @class */ (function () {
    function Widget(context, viewDomNode, actual) {
        this._context = context;
        this._viewDomNode = viewDomNode;
        this._actual = actual;
        this.domNode = createFastDomNode(this._actual.getDomNode());
        this.id = this._actual.getId();
        this.allowEditorOverflow = this._actual.allowEditorOverflow || false;
        this.suppressMouseDown = this._actual.suppressMouseDown || false;
        this._fixedOverflowWidgets = this._context.configuration.editor.viewInfo.fixedOverflowWidgets;
        this._contentWidth = this._context.configuration.editor.layoutInfo.contentWidth;
        this._contentLeft = this._context.configuration.editor.layoutInfo.contentLeft;
        this._lineHeight = this._context.configuration.editor.lineHeight;
        this._setPosition(null, null);
        this._preference = [];
        this._cachedDomNodeClientWidth = -1;
        this._cachedDomNodeClientHeight = -1;
        this._maxWidth = this._getMaxWidth();
        this._isVisible = false;
        this._renderData = null;
        this.domNode.setPosition((this._fixedOverflowWidgets && this.allowEditorOverflow) ? 'fixed' : 'absolute');
        this.domNode.setVisibility('hidden');
        this.domNode.setAttribute('widgetId', this.id);
        this.domNode.setMaxWidth(this._maxWidth);
    }
    Widget.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.layoutInfo) {
            this._contentLeft = this._context.configuration.editor.layoutInfo.contentLeft;
            this._contentWidth = this._context.configuration.editor.layoutInfo.contentWidth;
            this._maxWidth = this._getMaxWidth();
        }
    };
    Widget.prototype.onLineMappingChanged = function (e) {
        this._setPosition(this._position, this._range);
    };
    Widget.prototype._setPosition = function (position, range) {
        this._position = position || null;
        this._range = range || null;
        this._viewPosition = null;
        this._viewRange = null;
        if (this._position) {
            // Do not trust that widgets give a valid position
            var validModelPosition = this._context.model.validateModelPosition(this._position);
            if (this._context.model.coordinatesConverter.modelPositionIsVisible(validModelPosition)) {
                this._viewPosition = this._context.model.coordinatesConverter.convertModelPositionToViewPosition(validModelPosition);
            }
        }
        if (this._range) {
            // Do not trust that widgets give a valid position
            var validModelRange = this._context.model.validateModelRange(this._range);
            this._viewRange = this._context.model.coordinatesConverter.convertModelRangeToViewRange(validModelRange);
        }
    };
    Widget.prototype._getMaxWidth = function () {
        return (this.allowEditorOverflow
            ? window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth
            : this._contentWidth);
    };
    Widget.prototype.setPosition = function (position, range, preference) {
        this._setPosition(position, range);
        this._preference = preference || null;
        this._cachedDomNodeClientWidth = -1;
        this._cachedDomNodeClientHeight = -1;
    };
    Widget.prototype._layoutBoxInViewport = function (topLeft, bottomLeft, width, height, ctx) {
        // Our visible box is split horizontally by the current line => 2 boxes
        // a) the box above the line
        var aboveLineTop = topLeft.top;
        var heightAboveLine = aboveLineTop;
        // b) the box under the line
        var underLineTop = bottomLeft.top + this._lineHeight;
        var heightUnderLine = ctx.viewportHeight - underLineTop;
        var aboveTop = aboveLineTop - height;
        var fitsAbove = (heightAboveLine >= height);
        var belowTop = underLineTop;
        var fitsBelow = (heightUnderLine >= height);
        // And its left
        var actualAboveLeft = topLeft.left;
        var actualBelowLeft = bottomLeft.left;
        if (actualAboveLeft + width > ctx.scrollLeft + ctx.viewportWidth) {
            actualAboveLeft = ctx.scrollLeft + ctx.viewportWidth - width;
        }
        if (actualBelowLeft + width > ctx.scrollLeft + ctx.viewportWidth) {
            actualBelowLeft = ctx.scrollLeft + ctx.viewportWidth - width;
        }
        if (actualAboveLeft < ctx.scrollLeft) {
            actualAboveLeft = ctx.scrollLeft;
        }
        if (actualBelowLeft < ctx.scrollLeft) {
            actualBelowLeft = ctx.scrollLeft;
        }
        return {
            fitsAbove: fitsAbove,
            aboveTop: aboveTop,
            aboveLeft: actualAboveLeft,
            fitsBelow: fitsBelow,
            belowTop: belowTop,
            belowLeft: actualBelowLeft,
        };
    };
    Widget.prototype._layoutBoxInPage = function (topLeft, bottomLeft, width, height, ctx) {
        var aboveLeft0 = topLeft.left - ctx.scrollLeft;
        var belowLeft0 = bottomLeft.left - ctx.scrollLeft;
        if (aboveLeft0 < 0 || aboveLeft0 > this._contentWidth) {
            // Don't render if position is scrolled outside viewport
            return null;
        }
        var aboveTop = topLeft.top - height;
        var belowTop = bottomLeft.top + this._lineHeight;
        var aboveLeft = aboveLeft0 + this._contentLeft;
        var belowLeft = belowLeft0 + this._contentLeft;
        var domNodePosition = getDomNodePagePosition(this._viewDomNode.domNode);
        var absoluteAboveTop = domNodePosition.top + aboveTop - StandardWindow.scrollY;
        var absoluteBelowTop = domNodePosition.top + belowTop - StandardWindow.scrollY;
        var absoluteAboveLeft = domNodePosition.left + aboveLeft - StandardWindow.scrollX;
        var absoluteBelowLeft = domNodePosition.left + belowLeft - StandardWindow.scrollX;
        var INNER_WIDTH = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
        var INNER_HEIGHT = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
        // Leave some clearance to the bottom
        var TOP_PADDING = 22;
        var BOTTOM_PADDING = 22;
        var fitsAbove = (absoluteAboveTop >= TOP_PADDING), fitsBelow = (absoluteBelowTop + height <= INNER_HEIGHT - BOTTOM_PADDING);
        if (absoluteAboveLeft + width + 20 > INNER_WIDTH) {
            var delta = absoluteAboveLeft - (INNER_WIDTH - width - 20);
            absoluteAboveLeft -= delta;
            aboveLeft -= delta;
        }
        if (absoluteBelowLeft + width + 20 > INNER_WIDTH) {
            var delta = absoluteBelowLeft - (INNER_WIDTH - width - 20);
            absoluteBelowLeft -= delta;
            belowLeft -= delta;
        }
        if (absoluteAboveLeft < 0) {
            var delta = absoluteAboveLeft;
            absoluteAboveLeft -= delta;
            aboveLeft -= delta;
        }
        if (absoluteBelowLeft < 0) {
            var delta = absoluteBelowLeft;
            absoluteBelowLeft -= delta;
            belowLeft -= delta;
        }
        if (this._fixedOverflowWidgets) {
            aboveTop = absoluteAboveTop;
            belowTop = absoluteBelowTop;
            aboveLeft = absoluteAboveLeft;
            belowLeft = absoluteBelowLeft;
        }
        return { fitsAbove: fitsAbove, aboveTop: aboveTop, aboveLeft: aboveLeft, fitsBelow: fitsBelow, belowTop: belowTop, belowLeft: belowLeft };
    };
    Widget.prototype._prepareRenderWidgetAtExactPositionOverflowing = function (topLeft) {
        return new Coordinate(topLeft.top, topLeft.left + this._contentLeft);
    };
    /**
     * Compute `this._topLeft`
     */
    Widget.prototype._getTopAndBottomLeft = function (ctx) {
        if (!this._viewPosition) {
            return [null, null];
        }
        var visibleRangeForPosition = ctx.visibleRangeForPosition(this._viewPosition);
        if (!visibleRangeForPosition) {
            return [null, null];
        }
        var topForPosition = ctx.getVerticalOffsetForLineNumber(this._viewPosition.lineNumber) - ctx.scrollTop;
        var topLeft = new Coordinate(topForPosition, visibleRangeForPosition.left);
        var largestLineNumber = this._viewPosition.lineNumber;
        var smallestLeft = visibleRangeForPosition.left;
        if (this._viewRange) {
            var visibleRangesForRange = ctx.linesVisibleRangesForRange(this._viewRange, false);
            if (visibleRangesForRange && visibleRangesForRange.length > 0) {
                for (var i = visibleRangesForRange.length - 1; i >= 0; i--) {
                    var visibleRangesForLine = visibleRangesForRange[i];
                    if (visibleRangesForLine.lineNumber >= largestLineNumber) {
                        if (visibleRangesForLine.lineNumber > largestLineNumber) {
                            largestLineNumber = visibleRangesForLine.lineNumber;
                            smallestLeft = 1073741824 /* MAX_SAFE_SMALL_INTEGER */;
                        }
                        for (var j = 0, lenJ = visibleRangesForLine.ranges.length; j < lenJ; j++) {
                            var visibleRange = visibleRangesForLine.ranges[j];
                            if (visibleRange.left < smallestLeft) {
                                smallestLeft = visibleRange.left;
                            }
                        }
                    }
                }
            }
        }
        var topForBottomLine = ctx.getVerticalOffsetForLineNumber(largestLineNumber) - ctx.scrollTop;
        var bottomLeft = new Coordinate(topForBottomLine, smallestLeft);
        return [topLeft, bottomLeft];
    };
    Widget.prototype._prepareRenderWidget = function (ctx) {
        var _a = this._getTopAndBottomLeft(ctx), topLeft = _a[0], bottomLeft = _a[1];
        if (!topLeft || !bottomLeft) {
            return null;
        }
        if (this._cachedDomNodeClientWidth === -1 || this._cachedDomNodeClientHeight === -1) {
            var domNode = this.domNode.domNode;
            this._cachedDomNodeClientWidth = domNode.clientWidth;
            this._cachedDomNodeClientHeight = domNode.clientHeight;
        }
        var placement;
        if (this.allowEditorOverflow) {
            placement = this._layoutBoxInPage(topLeft, bottomLeft, this._cachedDomNodeClientWidth, this._cachedDomNodeClientHeight, ctx);
        }
        else {
            placement = this._layoutBoxInViewport(topLeft, bottomLeft, this._cachedDomNodeClientWidth, this._cachedDomNodeClientHeight, ctx);
        }
        // Do two passes, first for perfect fit, second picks first option
        if (this._preference) {
            for (var pass = 1; pass <= 2; pass++) {
                for (var i = 0; i < this._preference.length; i++) {
                    // placement
                    var pref = this._preference[i];
                    if (pref === 1 /* ABOVE */) {
                        if (!placement) {
                            // Widget outside of viewport
                            return null;
                        }
                        if (pass === 2 || placement.fitsAbove) {
                            return new Coordinate(placement.aboveTop, placement.aboveLeft);
                        }
                    }
                    else if (pref === 2 /* BELOW */) {
                        if (!placement) {
                            // Widget outside of viewport
                            return null;
                        }
                        if (pass === 2 || placement.fitsBelow) {
                            return new Coordinate(placement.belowTop, placement.belowLeft);
                        }
                    }
                    else {
                        if (this.allowEditorOverflow) {
                            return this._prepareRenderWidgetAtExactPositionOverflowing(topLeft);
                        }
                        else {
                            return topLeft;
                        }
                    }
                }
            }
        }
        return null;
    };
    /**
     * On this first pass, we ensure that the content widget (if it is in the viewport) has the max width set correctly.
     */
    Widget.prototype.onBeforeRender = function (viewportData) {
        if (!this._viewPosition || !this._preference) {
            return;
        }
        if (this._viewPosition.lineNumber < viewportData.startLineNumber || this._viewPosition.lineNumber > viewportData.endLineNumber) {
            // Outside of viewport
            return;
        }
        this.domNode.setMaxWidth(this._maxWidth);
    };
    Widget.prototype.prepareRender = function (ctx) {
        this._renderData = this._prepareRenderWidget(ctx);
    };
    Widget.prototype.render = function (ctx) {
        if (!this._renderData) {
            // This widget should be invisible
            if (this._isVisible) {
                this.domNode.removeAttribute('monaco-visible-content-widget');
                this._isVisible = false;
                this.domNode.setVisibility('hidden');
            }
            return;
        }
        // This widget should be visible
        if (this.allowEditorOverflow) {
            this.domNode.setTop(this._renderData.top);
            this.domNode.setLeft(this._renderData.left);
        }
        else {
            this.domNode.setTop(this._renderData.top + ctx.scrollTop - ctx.bigNumbersDelta);
            this.domNode.setLeft(this._renderData.left);
        }
        if (!this._isVisible) {
            this.domNode.setVisibility('inherit');
            this.domNode.setAttribute('monaco-visible-content-widget', 'true');
            this._isVisible = true;
        }
    };
    return Widget;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$B = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CurrentLineHighlightOverlay = /** @class */ (function (_super) {
    __extends$B(CurrentLineHighlightOverlay, _super);
    function CurrentLineHighlightOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._lineHeight = _this._context.configuration.editor.lineHeight;
        _this._renderLineHighlight = _this._context.configuration.editor.viewInfo.renderLineHighlight;
        _this._selectionIsEmpty = true;
        _this._primaryCursorLineNumber = 1;
        _this._scrollWidth = 0;
        _this._contentWidth = _this._context.configuration.editor.layoutInfo.contentWidth;
        _this._context.addEventHandler(_this);
        return _this;
    }
    CurrentLineHighlightOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    CurrentLineHighlightOverlay.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.viewInfo) {
            this._renderLineHighlight = this._context.configuration.editor.viewInfo.renderLineHighlight;
        }
        if (e.layoutInfo) {
            this._contentWidth = this._context.configuration.editor.layoutInfo.contentWidth;
        }
        return true;
    };
    CurrentLineHighlightOverlay.prototype.onCursorStateChanged = function (e) {
        var hasChanged = false;
        var primaryCursorLineNumber = e.selections[0].positionLineNumber;
        if (this._primaryCursorLineNumber !== primaryCursorLineNumber) {
            this._primaryCursorLineNumber = primaryCursorLineNumber;
            hasChanged = true;
        }
        var selectionIsEmpty = e.selections[0].isEmpty();
        if (this._selectionIsEmpty !== selectionIsEmpty) {
            this._selectionIsEmpty = selectionIsEmpty;
            hasChanged = true;
            return true;
        }
        return hasChanged;
    };
    CurrentLineHighlightOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    CurrentLineHighlightOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    CurrentLineHighlightOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    CurrentLineHighlightOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollWidthChanged;
    };
    CurrentLineHighlightOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    CurrentLineHighlightOverlay.prototype.prepareRender = function (ctx) {
        this._scrollWidth = ctx.scrollWidth;
    };
    CurrentLineHighlightOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (lineNumber === this._primaryCursorLineNumber) {
            if (this._shouldShowCurrentLine()) {
                var paintedInMargin = this._willRenderMarginCurrentLine();
                var className = 'current-line' + (paintedInMargin ? ' current-line-both' : '');
                return ('<div class="'
                    + className
                    + '" style="width:'
                    + String(Math.max(this._scrollWidth, this._contentWidth))
                    + 'px; height:'
                    + String(this._lineHeight)
                    + 'px;"></div>');
            }
            else {
                return '';
            }
        }
        return '';
    };
    CurrentLineHighlightOverlay.prototype._shouldShowCurrentLine = function () {
        return ((this._renderLineHighlight === 'line' || this._renderLineHighlight === 'all')
            && this._selectionIsEmpty);
    };
    CurrentLineHighlightOverlay.prototype._willRenderMarginCurrentLine = function () {
        return ((this._renderLineHighlight === 'gutter' || this._renderLineHighlight === 'all'));
    };
    return CurrentLineHighlightOverlay;
}(DynamicViewOverlay));
registerThemingParticipant(function (theme, collector) {
    var lineHighlight = theme.getColor(editorLineHighlight);
    if (lineHighlight) {
        collector.addRule(".monaco-editor .view-overlays .current-line { background-color: " + lineHighlight + "; }");
    }
    if (!lineHighlight || lineHighlight.isTransparent() || theme.defines(editorLineHighlightBorder)) {
        var lineHighlightBorder = theme.getColor(editorLineHighlightBorder);
        if (lineHighlightBorder) {
            collector.addRule(".monaco-editor .view-overlays .current-line { border: 2px solid " + lineHighlightBorder + "; }");
            if (theme.type === 'hc') {
                collector.addRule(".monaco-editor .view-overlays .current-line { border-width: 1px; }");
            }
        }
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$C = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CurrentLineMarginHighlightOverlay = /** @class */ (function (_super) {
    __extends$C(CurrentLineMarginHighlightOverlay, _super);
    function CurrentLineMarginHighlightOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._lineHeight = _this._context.configuration.editor.lineHeight;
        _this._renderLineHighlight = _this._context.configuration.editor.viewInfo.renderLineHighlight;
        _this._selectionIsEmpty = true;
        _this._primaryCursorLineNumber = 1;
        _this._contentLeft = _this._context.configuration.editor.layoutInfo.contentLeft;
        _this._context.addEventHandler(_this);
        return _this;
    }
    CurrentLineMarginHighlightOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    CurrentLineMarginHighlightOverlay.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.viewInfo) {
            this._renderLineHighlight = this._context.configuration.editor.viewInfo.renderLineHighlight;
        }
        if (e.layoutInfo) {
            this._contentLeft = this._context.configuration.editor.layoutInfo.contentLeft;
        }
        return true;
    };
    CurrentLineMarginHighlightOverlay.prototype.onCursorStateChanged = function (e) {
        var hasChanged = false;
        var primaryCursorLineNumber = e.selections[0].positionLineNumber;
        if (this._primaryCursorLineNumber !== primaryCursorLineNumber) {
            this._primaryCursorLineNumber = primaryCursorLineNumber;
            hasChanged = true;
        }
        var selectionIsEmpty = e.selections[0].isEmpty();
        if (this._selectionIsEmpty !== selectionIsEmpty) {
            this._selectionIsEmpty = selectionIsEmpty;
            hasChanged = true;
            return true;
        }
        return hasChanged;
    };
    CurrentLineMarginHighlightOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    CurrentLineMarginHighlightOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    CurrentLineMarginHighlightOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    CurrentLineMarginHighlightOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    CurrentLineMarginHighlightOverlay.prototype.prepareRender = function (ctx) {
    };
    CurrentLineMarginHighlightOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (lineNumber === this._primaryCursorLineNumber) {
            var className = 'current-line';
            if (this._shouldShowCurrentLine()) {
                var paintedInContent = this._willRenderContentCurrentLine();
                className = 'current-line current-line-margin' + (paintedInContent ? ' current-line-margin-both' : '');
            }
            return ('<div class="'
                + className
                + '" style="width:'
                + String(this._contentLeft)
                + 'px; height:'
                + String(this._lineHeight)
                + 'px;"></div>');
        }
        return '';
    };
    CurrentLineMarginHighlightOverlay.prototype._shouldShowCurrentLine = function () {
        return ((this._renderLineHighlight === 'gutter' || this._renderLineHighlight === 'all'));
    };
    CurrentLineMarginHighlightOverlay.prototype._willRenderContentCurrentLine = function () {
        return ((this._renderLineHighlight === 'line' || this._renderLineHighlight === 'all')
            && this._selectionIsEmpty);
    };
    return CurrentLineMarginHighlightOverlay;
}(DynamicViewOverlay));
registerThemingParticipant(function (theme, collector) {
    var lineHighlight = theme.getColor(editorLineHighlight);
    if (lineHighlight) {
        collector.addRule(".monaco-editor .margin-view-overlays .current-line-margin { background-color: " + lineHighlight + "; border: none; }");
    }
    else {
        var lineHighlightBorder = theme.getColor(editorLineHighlightBorder);
        if (lineHighlightBorder) {
            collector.addRule(".monaco-editor .margin-view-overlays .current-line-margin { border: 2px solid " + lineHighlightBorder + "; }");
        }
        if (theme.type === 'hc') {
            collector.addRule(".monaco-editor .margin-view-overlays .current-line-margin { border-width: 1px; }");
        }
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$D = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DecorationsOverlay = /** @class */ (function (_super) {
    __extends$D(DecorationsOverlay, _super);
    function DecorationsOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._lineHeight = _this._context.configuration.editor.lineHeight;
        _this._typicalHalfwidthCharacterWidth = _this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        _this._renderResult = null;
        _this._context.addEventHandler(_this);
        return _this;
    }
    DecorationsOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        this._renderResult = null;
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    DecorationsOverlay.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.fontInfo) {
            this._typicalHalfwidthCharacterWidth = this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        }
        return true;
    };
    DecorationsOverlay.prototype.onDecorationsChanged = function (e) {
        return true;
    };
    DecorationsOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    DecorationsOverlay.prototype.onLinesChanged = function (e) {
        return true;
    };
    DecorationsOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    DecorationsOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    DecorationsOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged || e.scrollWidthChanged;
    };
    DecorationsOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    DecorationsOverlay.prototype.prepareRender = function (ctx) {
        var _decorations = ctx.getDecorationsInViewport();
        // Keep only decorations with `className`
        var decorations = [], decorationsLen = 0;
        for (var i = 0, len = _decorations.length; i < len; i++) {
            var d = _decorations[i];
            if (d.options.className) {
                decorations[decorationsLen++] = d;
            }
        }
        // Sort decorations for consistent render output
        decorations = decorations.sort(function (a, b) {
            if (a.options.zIndex < b.options.zIndex) {
                return -1;
            }
            if (a.options.zIndex > b.options.zIndex) {
                return 1;
            }
            var aClassName = a.options.className;
            var bClassName = b.options.className;
            if (aClassName < bClassName) {
                return -1;
            }
            if (aClassName > bClassName) {
                return 1;
            }
            return Range.compareRangesUsingStarts(a.range, b.range);
        });
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        var output = [];
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var lineIndex = lineNumber - visibleStartLineNumber;
            output[lineIndex] = '';
        }
        // Render first whole line decorations and then regular decorations
        this._renderWholeLineDecorations(ctx, decorations, output);
        this._renderNormalDecorations(ctx, decorations, output);
        this._renderResult = output;
    };
    DecorationsOverlay.prototype._renderWholeLineDecorations = function (ctx, decorations, output) {
        var lineHeight = String(this._lineHeight);
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        for (var i = 0, lenI = decorations.length; i < lenI; i++) {
            var d = decorations[i];
            if (!d.options.isWholeLine) {
                continue;
            }
            var decorationOutput = ('<div class="cdr '
                + d.options.className
                + '" style="left:0;width:100%;height:'
                + lineHeight
                + 'px;"></div>');
            var startLineNumber = Math.max(d.range.startLineNumber, visibleStartLineNumber);
            var endLineNumber = Math.min(d.range.endLineNumber, visibleEndLineNumber);
            for (var j = startLineNumber; j <= endLineNumber; j++) {
                var lineIndex = j - visibleStartLineNumber;
                output[lineIndex] += decorationOutput;
            }
        }
    };
    DecorationsOverlay.prototype._renderNormalDecorations = function (ctx, decorations, output) {
        var lineHeight = String(this._lineHeight);
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var prevClassName = null;
        var prevShowIfCollapsed = false;
        var prevRange = null;
        for (var i = 0, lenI = decorations.length; i < lenI; i++) {
            var d = decorations[i];
            if (d.options.isWholeLine) {
                continue;
            }
            var className = d.options.className;
            var showIfCollapsed = Boolean(d.options.showIfCollapsed);
            var range = d.range;
            if (showIfCollapsed && range.endColumn === 1 && range.endLineNumber !== range.startLineNumber) {
                range = new Range(range.startLineNumber, range.startColumn, range.endLineNumber - 1, this._context.model.getLineMaxColumn(range.endLineNumber - 1));
            }
            if (prevClassName === className && prevShowIfCollapsed === showIfCollapsed && Range.areIntersectingOrTouching(prevRange, range)) {
                // merge into previous decoration
                prevRange = Range.plusRange(prevRange, range);
                continue;
            }
            // flush previous decoration
            if (prevClassName !== null) {
                this._renderNormalDecoration(ctx, prevRange, prevClassName, prevShowIfCollapsed, lineHeight, visibleStartLineNumber, output);
            }
            prevClassName = className;
            prevShowIfCollapsed = showIfCollapsed;
            prevRange = range;
        }
        if (prevClassName !== null) {
            this._renderNormalDecoration(ctx, prevRange, prevClassName, prevShowIfCollapsed, lineHeight, visibleStartLineNumber, output);
        }
    };
    DecorationsOverlay.prototype._renderNormalDecoration = function (ctx, range, className, showIfCollapsed, lineHeight, visibleStartLineNumber, output) {
        var linesVisibleRanges = ctx.linesVisibleRangesForRange(range, /*TODO@Alex*/ className === 'findMatch');
        if (!linesVisibleRanges) {
            return;
        }
        for (var j = 0, lenJ = linesVisibleRanges.length; j < lenJ; j++) {
            var lineVisibleRanges = linesVisibleRanges[j];
            var lineIndex = lineVisibleRanges.lineNumber - visibleStartLineNumber;
            if (showIfCollapsed && lineVisibleRanges.ranges.length === 1) {
                var singleVisibleRange = lineVisibleRanges.ranges[0];
                if (singleVisibleRange.width === 0) {
                    // collapsed range case => make the decoration visible by faking its width
                    lineVisibleRanges.ranges[0] = new HorizontalRange(singleVisibleRange.left, this._typicalHalfwidthCharacterWidth);
                }
            }
            for (var k = 0, lenK = lineVisibleRanges.ranges.length; k < lenK; k++) {
                var visibleRange = lineVisibleRanges.ranges[k];
                var decorationOutput = ('<div class="cdr '
                    + className
                    + '" style="left:'
                    + String(visibleRange.left)
                    + 'px;width:'
                    + String(visibleRange.width)
                    + 'px;height:'
                    + lineHeight
                    + 'px;"></div>');
                output[lineIndex] += decorationOutput;
            }
        }
    };
    DecorationsOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (!this._renderResult) {
            return '';
        }
        var lineIndex = lineNumber - startLineNumber;
        if (lineIndex < 0 || lineIndex >= this._renderResult.length) {
            return '';
        }
        return this._renderResult[lineIndex];
    };
    return DecorationsOverlay;
}(DynamicViewOverlay));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$E = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Widget$1 = /** @class */ (function (_super) {
    __extends$E(Widget, _super);
    function Widget() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Widget.prototype.onclick = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.CLICK, function (e) { return listener(new StandardMouseEvent(e)); }));
    };
    Widget.prototype.onmousedown = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.MOUSE_DOWN, function (e) { return listener(new StandardMouseEvent(e)); }));
    };
    Widget.prototype.onmouseover = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.MOUSE_OVER, function (e) { return listener(new StandardMouseEvent(e)); }));
    };
    Widget.prototype.onnonbubblingmouseout = function (domNode, listener) {
        this._register(addDisposableNonBubblingMouseOutListener(domNode, function (e) { return listener(new StandardMouseEvent(e)); }));
    };
    Widget.prototype.onkeydown = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.KEY_DOWN, function (e) { return listener(new StandardKeyboardEvent(e)); }));
    };
    Widget.prototype.onkeyup = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.KEY_UP, function (e) { return listener(new StandardKeyboardEvent(e)); }));
    };
    Widget.prototype.oninput = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.INPUT, listener));
    };
    Widget.prototype.onblur = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.BLUR, listener));
    };
    Widget.prototype.onfocus = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.FOCUS, listener));
    };
    Widget.prototype.onchange = function (domNode, listener) {
        this._register(addDisposableListener(domNode, EventType.CHANGE, listener));
    };
    return Widget;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$F = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * The arrow image size.
 */
var ARROW_IMG_SIZE = 11;
var ScrollbarArrow = /** @class */ (function (_super) {
    __extends$F(ScrollbarArrow, _super);
    function ScrollbarArrow(opts) {
        var _this = _super.call(this) || this;
        _this._onActivate = opts.onActivate;
        _this.bgDomNode = document.createElement('div');
        _this.bgDomNode.className = 'arrow-background';
        _this.bgDomNode.style.position = 'absolute';
        _this.bgDomNode.style.width = opts.bgWidth + 'px';
        _this.bgDomNode.style.height = opts.bgHeight + 'px';
        if (typeof opts.top !== 'undefined') {
            _this.bgDomNode.style.top = '0px';
        }
        if (typeof opts.left !== 'undefined') {
            _this.bgDomNode.style.left = '0px';
        }
        if (typeof opts.bottom !== 'undefined') {
            _this.bgDomNode.style.bottom = '0px';
        }
        if (typeof opts.right !== 'undefined') {
            _this.bgDomNode.style.right = '0px';
        }
        _this.domNode = document.createElement('div');
        _this.domNode.className = opts.className;
        _this.domNode.style.position = 'absolute';
        _this.domNode.style.width = ARROW_IMG_SIZE + 'px';
        _this.domNode.style.height = ARROW_IMG_SIZE + 'px';
        if (typeof opts.top !== 'undefined') {
            _this.domNode.style.top = opts.top + 'px';
        }
        if (typeof opts.left !== 'undefined') {
            _this.domNode.style.left = opts.left + 'px';
        }
        if (typeof opts.bottom !== 'undefined') {
            _this.domNode.style.bottom = opts.bottom + 'px';
        }
        if (typeof opts.right !== 'undefined') {
            _this.domNode.style.right = opts.right + 'px';
        }
        _this._mouseMoveMonitor = _this._register(new GlobalMouseMoveMonitor());
        _this.onmousedown(_this.bgDomNode, function (e) { return _this._arrowMouseDown(e); });
        _this.onmousedown(_this.domNode, function (e) { return _this._arrowMouseDown(e); });
        _this._mousedownRepeatTimer = _this._register(new IntervalTimer());
        _this._mousedownScheduleRepeatTimer = _this._register(new TimeoutTimer());
        return _this;
    }
    ScrollbarArrow.prototype._arrowMouseDown = function (e) {
        var _this = this;
        var scheduleRepeater = function () {
            _this._mousedownRepeatTimer.cancelAndSet(function () { return _this._onActivate(); }, 1000 / 24);
        };
        this._onActivate();
        this._mousedownRepeatTimer.cancel();
        this._mousedownScheduleRepeatTimer.cancelAndSet(scheduleRepeater, 200);
        this._mouseMoveMonitor.startMonitoring(standardMouseMoveMerger, function (mouseMoveData) {
            /* Intentional empty */
        }, function () {
            _this._mousedownRepeatTimer.cancel();
            _this._mousedownScheduleRepeatTimer.cancel();
        });
        e.preventDefault();
    };
    return ScrollbarArrow;
}(Widget$1));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$G = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ScrollbarVisibilityController = /** @class */ (function (_super) {
    __extends$G(ScrollbarVisibilityController, _super);
    function ScrollbarVisibilityController(visibility, visibleClassName, invisibleClassName) {
        var _this = _super.call(this) || this;
        _this._visibility = visibility;
        _this._visibleClassName = visibleClassName;
        _this._invisibleClassName = invisibleClassName;
        _this._domNode = null;
        _this._isVisible = false;
        _this._isNeeded = false;
        _this._shouldBeVisible = false;
        _this._revealTimer = _this._register(new TimeoutTimer());
        return _this;
    }
    // ----------------- Hide / Reveal
    ScrollbarVisibilityController.prototype.applyVisibilitySetting = function (shouldBeVisible) {
        if (this._visibility === 2 /* Hidden */) {
            return false;
        }
        if (this._visibility === 3 /* Visible */) {
            return true;
        }
        return shouldBeVisible;
    };
    ScrollbarVisibilityController.prototype.setShouldBeVisible = function (rawShouldBeVisible) {
        var shouldBeVisible = this.applyVisibilitySetting(rawShouldBeVisible);
        if (this._shouldBeVisible !== shouldBeVisible) {
            this._shouldBeVisible = shouldBeVisible;
            this.ensureVisibility();
        }
    };
    ScrollbarVisibilityController.prototype.setIsNeeded = function (isNeeded) {
        if (this._isNeeded !== isNeeded) {
            this._isNeeded = isNeeded;
            this.ensureVisibility();
        }
    };
    ScrollbarVisibilityController.prototype.setDomNode = function (domNode) {
        this._domNode = domNode;
        this._domNode.setClassName(this._invisibleClassName);
        // Now that the flags & the dom node are in a consistent state, ensure the Hidden/Visible configuration
        this.setShouldBeVisible(false);
    };
    ScrollbarVisibilityController.prototype.ensureVisibility = function () {
        if (!this._isNeeded) {
            // Nothing to be rendered
            this._hide(false);
            return;
        }
        if (this._shouldBeVisible) {
            this._reveal();
        }
        else {
            this._hide(true);
        }
    };
    ScrollbarVisibilityController.prototype._reveal = function () {
        var _this = this;
        if (this._isVisible) {
            return;
        }
        this._isVisible = true;
        // The CSS animation doesn't play otherwise
        this._revealTimer.setIfNotSet(function () {
            if (_this._domNode) {
                _this._domNode.setClassName(_this._visibleClassName);
            }
        }, 0);
    };
    ScrollbarVisibilityController.prototype._hide = function (withFadeAway) {
        this._revealTimer.cancel();
        if (!this._isVisible) {
            return;
        }
        this._isVisible = false;
        if (this._domNode) {
            this._domNode.setClassName(this._invisibleClassName + (withFadeAway ? ' fade' : ''));
        }
    };
    return ScrollbarVisibilityController;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$H = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * The orthogonal distance to the slider at which dragging "resets". This implements "snapping"
 */
var MOUSE_DRAG_RESET_DISTANCE = 140;
var AbstractScrollbar = /** @class */ (function (_super) {
    __extends$H(AbstractScrollbar, _super);
    function AbstractScrollbar(opts) {
        var _this = _super.call(this) || this;
        _this._lazyRender = opts.lazyRender;
        _this._host = opts.host;
        _this._scrollable = opts.scrollable;
        _this._scrollbarState = opts.scrollbarState;
        _this._visibilityController = _this._register(new ScrollbarVisibilityController(opts.visibility, 'visible scrollbar ' + opts.extraScrollbarClassName, 'invisible scrollbar ' + opts.extraScrollbarClassName));
        _this._mouseMoveMonitor = _this._register(new GlobalMouseMoveMonitor());
        _this._shouldRender = true;
        _this.domNode = createFastDomNode(document.createElement('div'));
        _this.domNode.setAttribute('role', 'presentation');
        _this.domNode.setAttribute('aria-hidden', 'true');
        _this._visibilityController.setDomNode(_this.domNode);
        _this.domNode.setPosition('absolute');
        _this.onmousedown(_this.domNode.domNode, function (e) { return _this._domNodeMouseDown(e); });
        return _this;
    }
    // ----------------- creation
    /**
     * Creates the dom node for an arrow & adds it to the container
     */
    AbstractScrollbar.prototype._createArrow = function (opts) {
        var arrow = this._register(new ScrollbarArrow(opts));
        this.domNode.domNode.appendChild(arrow.bgDomNode);
        this.domNode.domNode.appendChild(arrow.domNode);
    };
    /**
     * Creates the slider dom node, adds it to the container & hooks up the events
     */
    AbstractScrollbar.prototype._createSlider = function (top, left, width, height) {
        var _this = this;
        this.slider = createFastDomNode(document.createElement('div'));
        this.slider.setClassName('slider');
        this.slider.setPosition('absolute');
        this.slider.setTop(top);
        this.slider.setLeft(left);
        if (typeof width === 'number') {
            this.slider.setWidth(width);
        }
        if (typeof height === 'number') {
            this.slider.setHeight(height);
        }
        this.slider.setLayerHinting(true);
        this.domNode.domNode.appendChild(this.slider.domNode);
        this.onmousedown(this.slider.domNode, function (e) {
            if (e.leftButton) {
                e.preventDefault();
                _this._sliderMouseDown(e, function () { });
            }
        });
    };
    // ----------------- Update state
    AbstractScrollbar.prototype._onElementSize = function (visibleSize) {
        if (this._scrollbarState.setVisibleSize(visibleSize)) {
            this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded());
            this._shouldRender = true;
            if (!this._lazyRender) {
                this.render();
            }
        }
        return this._shouldRender;
    };
    AbstractScrollbar.prototype._onElementScrollSize = function (elementScrollSize) {
        if (this._scrollbarState.setScrollSize(elementScrollSize)) {
            this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded());
            this._shouldRender = true;
            if (!this._lazyRender) {
                this.render();
            }
        }
        return this._shouldRender;
    };
    AbstractScrollbar.prototype._onElementScrollPosition = function (elementScrollPosition) {
        if (this._scrollbarState.setScrollPosition(elementScrollPosition)) {
            this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded());
            this._shouldRender = true;
            if (!this._lazyRender) {
                this.render();
            }
        }
        return this._shouldRender;
    };
    // ----------------- rendering
    AbstractScrollbar.prototype.beginReveal = function () {
        this._visibilityController.setShouldBeVisible(true);
    };
    AbstractScrollbar.prototype.beginHide = function () {
        this._visibilityController.setShouldBeVisible(false);
    };
    AbstractScrollbar.prototype.render = function () {
        if (!this._shouldRender) {
            return;
        }
        this._shouldRender = false;
        this._renderDomNode(this._scrollbarState.getRectangleLargeSize(), this._scrollbarState.getRectangleSmallSize());
        this._updateSlider(this._scrollbarState.getSliderSize(), this._scrollbarState.getArrowSize() + this._scrollbarState.getSliderPosition());
    };
    // ----------------- DOM events
    AbstractScrollbar.prototype._domNodeMouseDown = function (e) {
        if (e.target !== this.domNode.domNode) {
            return;
        }
        this._onMouseDown(e);
    };
    AbstractScrollbar.prototype.delegateMouseDown = function (e) {
        var domTop = this.domNode.domNode.getClientRects()[0].top;
        var sliderStart = domTop + this._scrollbarState.getSliderPosition();
        var sliderStop = domTop + this._scrollbarState.getSliderPosition() + this._scrollbarState.getSliderSize();
        var mousePos = this._sliderMousePosition(e);
        if (sliderStart <= mousePos && mousePos <= sliderStop) {
            // Act as if it was a mouse down on the slider
            if (e.leftButton) {
                e.preventDefault();
                this._sliderMouseDown(e, function () { });
            }
        }
        else {
            // Act as if it was a mouse down on the scrollbar
            this._onMouseDown(e);
        }
    };
    AbstractScrollbar.prototype._onMouseDown = function (e) {
        var offsetX;
        var offsetY;
        if (e.target === this.domNode.domNode && typeof e.browserEvent.offsetX === 'number' && typeof e.browserEvent.offsetY === 'number') {
            offsetX = e.browserEvent.offsetX;
            offsetY = e.browserEvent.offsetY;
        }
        else {
            var domNodePosition = getDomNodePagePosition(this.domNode.domNode);
            offsetX = e.posx - domNodePosition.left;
            offsetY = e.posy - domNodePosition.top;
        }
        this._setDesiredScrollPositionNow(this._scrollbarState.getDesiredScrollPositionFromOffset(this._mouseDownRelativePosition(offsetX, offsetY)));
        if (e.leftButton) {
            e.preventDefault();
            this._sliderMouseDown(e, function () { });
        }
    };
    AbstractScrollbar.prototype._sliderMouseDown = function (e, onDragFinished) {
        var _this = this;
        var initialMousePosition = this._sliderMousePosition(e);
        var initialMouseOrthogonalPosition = this._sliderOrthogonalMousePosition(e);
        var initialScrollbarState = this._scrollbarState.clone();
        this.slider.toggleClassName('active', true);
        this._mouseMoveMonitor.startMonitoring(standardMouseMoveMerger, function (mouseMoveData) {
            var mouseOrthogonalPosition = _this._sliderOrthogonalMousePosition(mouseMoveData);
            var mouseOrthogonalDelta = Math.abs(mouseOrthogonalPosition - initialMouseOrthogonalPosition);
            if (isWindows && mouseOrthogonalDelta > MOUSE_DRAG_RESET_DISTANCE) {
                // The mouse has wondered away from the scrollbar => reset dragging
                _this._setDesiredScrollPositionNow(initialScrollbarState.getScrollPosition());
                return;
            }
            var mousePosition = _this._sliderMousePosition(mouseMoveData);
            var mouseDelta = mousePosition - initialMousePosition;
            _this._setDesiredScrollPositionNow(initialScrollbarState.getDesiredScrollPositionFromDelta(mouseDelta));
        }, function () {
            _this.slider.toggleClassName('active', false);
            _this._host.onDragEnd();
            onDragFinished();
        });
        this._host.onDragStart();
    };
    AbstractScrollbar.prototype._setDesiredScrollPositionNow = function (_desiredScrollPosition) {
        var desiredScrollPosition = {};
        this.writeScrollPosition(desiredScrollPosition, _desiredScrollPosition);
        this._scrollable.setScrollPositionNow(desiredScrollPosition);
    };
    return AbstractScrollbar;
}(Widget$1));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * The minimal size of the slider (such that it can still be clickable) -- it is artificially enlarged.
 */
var MINIMUM_SLIDER_SIZE = 20;
var ScrollbarState = /** @class */ (function () {
    function ScrollbarState(arrowSize, scrollbarSize, oppositeScrollbarSize) {
        this._scrollbarSize = Math.round(scrollbarSize);
        this._oppositeScrollbarSize = Math.round(oppositeScrollbarSize);
        this._arrowSize = Math.round(arrowSize);
        this._visibleSize = 0;
        this._scrollSize = 0;
        this._scrollPosition = 0;
        this._computedAvailableSize = 0;
        this._computedIsNeeded = false;
        this._computedSliderSize = 0;
        this._computedSliderRatio = 0;
        this._computedSliderPosition = 0;
        this._refreshComputedValues();
    }
    ScrollbarState.prototype.clone = function () {
        var r = new ScrollbarState(this._arrowSize, this._scrollbarSize, this._oppositeScrollbarSize);
        r.setVisibleSize(this._visibleSize);
        r.setScrollSize(this._scrollSize);
        r.setScrollPosition(this._scrollPosition);
        return r;
    };
    ScrollbarState.prototype.setVisibleSize = function (visibleSize) {
        var iVisibleSize = Math.round(visibleSize);
        if (this._visibleSize !== iVisibleSize) {
            this._visibleSize = iVisibleSize;
            this._refreshComputedValues();
            return true;
        }
        return false;
    };
    ScrollbarState.prototype.setScrollSize = function (scrollSize) {
        var iScrollSize = Math.round(scrollSize);
        if (this._scrollSize !== iScrollSize) {
            this._scrollSize = iScrollSize;
            this._refreshComputedValues();
            return true;
        }
        return false;
    };
    ScrollbarState.prototype.setScrollPosition = function (scrollPosition) {
        var iScrollPosition = Math.round(scrollPosition);
        if (this._scrollPosition !== iScrollPosition) {
            this._scrollPosition = iScrollPosition;
            this._refreshComputedValues();
            return true;
        }
        return false;
    };
    ScrollbarState._computeValues = function (oppositeScrollbarSize, arrowSize, visibleSize, scrollSize, scrollPosition) {
        var computedAvailableSize = Math.max(0, visibleSize - oppositeScrollbarSize);
        var computedRepresentableSize = Math.max(0, computedAvailableSize - 2 * arrowSize);
        var computedIsNeeded = (scrollSize > 0 && scrollSize > visibleSize);
        if (!computedIsNeeded) {
            // There is no need for a slider
            return {
                computedAvailableSize: Math.round(computedAvailableSize),
                computedIsNeeded: computedIsNeeded,
                computedSliderSize: Math.round(computedRepresentableSize),
                computedSliderRatio: 0,
                computedSliderPosition: 0,
            };
        }
        // We must artificially increase the size of the slider if needed, since the slider would be too small to grab with the mouse otherwise
        var computedSliderSize = Math.round(Math.max(MINIMUM_SLIDER_SIZE, Math.floor(visibleSize * computedRepresentableSize / scrollSize)));
        // The slider can move from 0 to `computedRepresentableSize` - `computedSliderSize`
        // in the same way `scrollPosition` can move from 0 to `scrollSize` - `visibleSize`.
        var computedSliderRatio = (computedRepresentableSize - computedSliderSize) / (scrollSize - visibleSize);
        var computedSliderPosition = (scrollPosition * computedSliderRatio);
        return {
            computedAvailableSize: Math.round(computedAvailableSize),
            computedIsNeeded: computedIsNeeded,
            computedSliderSize: Math.round(computedSliderSize),
            computedSliderRatio: computedSliderRatio,
            computedSliderPosition: Math.round(computedSliderPosition),
        };
    };
    ScrollbarState.prototype._refreshComputedValues = function () {
        var r = ScrollbarState._computeValues(this._oppositeScrollbarSize, this._arrowSize, this._visibleSize, this._scrollSize, this._scrollPosition);
        this._computedAvailableSize = r.computedAvailableSize;
        this._computedIsNeeded = r.computedIsNeeded;
        this._computedSliderSize = r.computedSliderSize;
        this._computedSliderRatio = r.computedSliderRatio;
        this._computedSliderPosition = r.computedSliderPosition;
    };
    ScrollbarState.prototype.getArrowSize = function () {
        return this._arrowSize;
    };
    ScrollbarState.prototype.getScrollPosition = function () {
        return this._scrollPosition;
    };
    ScrollbarState.prototype.getRectangleLargeSize = function () {
        return this._computedAvailableSize;
    };
    ScrollbarState.prototype.getRectangleSmallSize = function () {
        return this._scrollbarSize;
    };
    ScrollbarState.prototype.isNeeded = function () {
        return this._computedIsNeeded;
    };
    ScrollbarState.prototype.getSliderSize = function () {
        return this._computedSliderSize;
    };
    ScrollbarState.prototype.getSliderPosition = function () {
        return this._computedSliderPosition;
    };
    /**
     * Compute a desired `scrollPosition` such that `offset` ends up in the center of the slider.
     * `offset` is based on the same coordinate system as the `sliderPosition`.
     */
    ScrollbarState.prototype.getDesiredScrollPositionFromOffset = function (offset) {
        if (!this._computedIsNeeded) {
            // no need for a slider
            return 0;
        }
        var desiredSliderPosition = offset - this._arrowSize - this._computedSliderSize / 2;
        return Math.round(desiredSliderPosition / this._computedSliderRatio);
    };
    /**
     * Compute a desired `scrollPosition` such that the slider moves by `delta`.
     */
    ScrollbarState.prototype.getDesiredScrollPositionFromDelta = function (delta) {
        if (!this._computedIsNeeded) {
            // no need for a slider
            return 0;
        }
        var desiredSliderPosition = this._computedSliderPosition + delta;
        return Math.round(desiredSliderPosition / this._computedSliderRatio);
    };
    return ScrollbarState;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$I = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var HorizontalScrollbar = /** @class */ (function (_super) {
    __extends$I(HorizontalScrollbar, _super);
    function HorizontalScrollbar(scrollable, options, host) {
        var _this = _super.call(this, {
            lazyRender: options.lazyRender,
            host: host,
            scrollbarState: new ScrollbarState((options.horizontalHasArrows ? options.arrowSize : 0), (options.horizontal === 2 /* Hidden */ ? 0 : options.horizontalScrollbarSize), (options.vertical === 2 /* Hidden */ ? 0 : options.verticalScrollbarSize)),
            visibility: options.horizontal,
            extraScrollbarClassName: 'horizontal',
            scrollable: scrollable
        }) || this;
        if (options.horizontalHasArrows) {
            var arrowDelta = (options.arrowSize - ARROW_IMG_SIZE) / 2;
            var scrollbarDelta = (options.horizontalScrollbarSize - ARROW_IMG_SIZE) / 2;
            _this._createArrow({
                className: 'left-arrow',
                top: scrollbarDelta,
                left: arrowDelta,
                bottom: void 0,
                right: void 0,
                bgWidth: options.arrowSize,
                bgHeight: options.horizontalScrollbarSize,
                onActivate: function () { return _this._host.onMouseWheel(new StandardMouseWheelEvent(null, 1, 0)); },
            });
            _this._createArrow({
                className: 'right-arrow',
                top: scrollbarDelta,
                left: void 0,
                bottom: void 0,
                right: arrowDelta,
                bgWidth: options.arrowSize,
                bgHeight: options.horizontalScrollbarSize,
                onActivate: function () { return _this._host.onMouseWheel(new StandardMouseWheelEvent(null, -1, 0)); },
            });
        }
        _this._createSlider(Math.floor((options.horizontalScrollbarSize - options.horizontalSliderSize) / 2), 0, undefined, options.horizontalSliderSize);
        return _this;
    }
    HorizontalScrollbar.prototype._updateSlider = function (sliderSize, sliderPosition) {
        this.slider.setWidth(sliderSize);
        this.slider.setLeft(sliderPosition);
    };
    HorizontalScrollbar.prototype._renderDomNode = function (largeSize, smallSize) {
        this.domNode.setWidth(largeSize);
        this.domNode.setHeight(smallSize);
        this.domNode.setLeft(0);
        this.domNode.setBottom(0);
    };
    HorizontalScrollbar.prototype.onDidScroll = function (e) {
        this._shouldRender = this._onElementScrollSize(e.scrollWidth) || this._shouldRender;
        this._shouldRender = this._onElementScrollPosition(e.scrollLeft) || this._shouldRender;
        this._shouldRender = this._onElementSize(e.width) || this._shouldRender;
        return this._shouldRender;
    };
    HorizontalScrollbar.prototype._mouseDownRelativePosition = function (offsetX, offsetY) {
        return offsetX;
    };
    HorizontalScrollbar.prototype._sliderMousePosition = function (e) {
        return e.posx;
    };
    HorizontalScrollbar.prototype._sliderOrthogonalMousePosition = function (e) {
        return e.posy;
    };
    HorizontalScrollbar.prototype.writeScrollPosition = function (target, scrollPosition) {
        target.scrollLeft = scrollPosition;
    };
    return HorizontalScrollbar;
}(AbstractScrollbar));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$J = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var VerticalScrollbar = /** @class */ (function (_super) {
    __extends$J(VerticalScrollbar, _super);
    function VerticalScrollbar(scrollable, options, host) {
        var _this = _super.call(this, {
            lazyRender: options.lazyRender,
            host: host,
            scrollbarState: new ScrollbarState((options.verticalHasArrows ? options.arrowSize : 0), (options.vertical === 2 /* Hidden */ ? 0 : options.verticalScrollbarSize), 
            // give priority to vertical scroll bar over horizontal and let it scroll all the way to the bottom
            0),
            visibility: options.vertical,
            extraScrollbarClassName: 'vertical',
            scrollable: scrollable
        }) || this;
        if (options.verticalHasArrows) {
            var arrowDelta = (options.arrowSize - ARROW_IMG_SIZE) / 2;
            var scrollbarDelta = (options.verticalScrollbarSize - ARROW_IMG_SIZE) / 2;
            _this._createArrow({
                className: 'up-arrow',
                top: arrowDelta,
                left: scrollbarDelta,
                bottom: void 0,
                right: void 0,
                bgWidth: options.verticalScrollbarSize,
                bgHeight: options.arrowSize,
                onActivate: function () { return _this._host.onMouseWheel(new StandardMouseWheelEvent(null, 0, 1)); },
            });
            _this._createArrow({
                className: 'down-arrow',
                top: void 0,
                left: scrollbarDelta,
                bottom: arrowDelta,
                right: void 0,
                bgWidth: options.verticalScrollbarSize,
                bgHeight: options.arrowSize,
                onActivate: function () { return _this._host.onMouseWheel(new StandardMouseWheelEvent(null, 0, -1)); },
            });
        }
        _this._createSlider(0, Math.floor((options.verticalScrollbarSize - options.verticalSliderSize) / 2), options.verticalSliderSize, undefined);
        return _this;
    }
    VerticalScrollbar.prototype._updateSlider = function (sliderSize, sliderPosition) {
        this.slider.setHeight(sliderSize);
        this.slider.setTop(sliderPosition);
    };
    VerticalScrollbar.prototype._renderDomNode = function (largeSize, smallSize) {
        this.domNode.setWidth(smallSize);
        this.domNode.setHeight(largeSize);
        this.domNode.setRight(0);
        this.domNode.setTop(0);
    };
    VerticalScrollbar.prototype.onDidScroll = function (e) {
        this._shouldRender = this._onElementScrollSize(e.scrollHeight) || this._shouldRender;
        this._shouldRender = this._onElementScrollPosition(e.scrollTop) || this._shouldRender;
        this._shouldRender = this._onElementSize(e.height) || this._shouldRender;
        return this._shouldRender;
    };
    VerticalScrollbar.prototype._mouseDownRelativePosition = function (offsetX, offsetY) {
        return offsetY;
    };
    VerticalScrollbar.prototype._sliderMousePosition = function (e) {
        return e.posy;
    };
    VerticalScrollbar.prototype._sliderOrthogonalMousePosition = function (e) {
        return e.posx;
    };
    VerticalScrollbar.prototype.writeScrollPosition = function (target, scrollPosition) {
        target.scrollTop = scrollPosition;
    };
    return VerticalScrollbar;
}(AbstractScrollbar));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$K = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ScrollState = /** @class */ (function () {
    function ScrollState(width, scrollWidth, scrollLeft, height, scrollHeight, scrollTop) {
        width = width | 0;
        scrollWidth = scrollWidth | 0;
        scrollLeft = scrollLeft | 0;
        height = height | 0;
        scrollHeight = scrollHeight | 0;
        scrollTop = scrollTop | 0;
        if (width < 0) {
            width = 0;
        }
        if (scrollLeft + width > scrollWidth) {
            scrollLeft = scrollWidth - width;
        }
        if (scrollLeft < 0) {
            scrollLeft = 0;
        }
        if (height < 0) {
            height = 0;
        }
        if (scrollTop + height > scrollHeight) {
            scrollTop = scrollHeight - height;
        }
        if (scrollTop < 0) {
            scrollTop = 0;
        }
        this.width = width;
        this.scrollWidth = scrollWidth;
        this.scrollLeft = scrollLeft;
        this.height = height;
        this.scrollHeight = scrollHeight;
        this.scrollTop = scrollTop;
    }
    ScrollState.prototype.equals = function (other) {
        return (this.width === other.width
            && this.scrollWidth === other.scrollWidth
            && this.scrollLeft === other.scrollLeft
            && this.height === other.height
            && this.scrollHeight === other.scrollHeight
            && this.scrollTop === other.scrollTop);
    };
    ScrollState.prototype.withScrollDimensions = function (update) {
        return new ScrollState((typeof update.width !== 'undefined' ? update.width : this.width), (typeof update.scrollWidth !== 'undefined' ? update.scrollWidth : this.scrollWidth), this.scrollLeft, (typeof update.height !== 'undefined' ? update.height : this.height), (typeof update.scrollHeight !== 'undefined' ? update.scrollHeight : this.scrollHeight), this.scrollTop);
    };
    ScrollState.prototype.withScrollPosition = function (update) {
        return new ScrollState(this.width, this.scrollWidth, (typeof update.scrollLeft !== 'undefined' ? update.scrollLeft : this.scrollLeft), this.height, this.scrollHeight, (typeof update.scrollTop !== 'undefined' ? update.scrollTop : this.scrollTop));
    };
    ScrollState.prototype.createScrollEvent = function (previous) {
        var widthChanged = (this.width !== previous.width);
        var scrollWidthChanged = (this.scrollWidth !== previous.scrollWidth);
        var scrollLeftChanged = (this.scrollLeft !== previous.scrollLeft);
        var heightChanged = (this.height !== previous.height);
        var scrollHeightChanged = (this.scrollHeight !== previous.scrollHeight);
        var scrollTopChanged = (this.scrollTop !== previous.scrollTop);
        return {
            width: this.width,
            scrollWidth: this.scrollWidth,
            scrollLeft: this.scrollLeft,
            height: this.height,
            scrollHeight: this.scrollHeight,
            scrollTop: this.scrollTop,
            widthChanged: widthChanged,
            scrollWidthChanged: scrollWidthChanged,
            scrollLeftChanged: scrollLeftChanged,
            heightChanged: heightChanged,
            scrollHeightChanged: scrollHeightChanged,
            scrollTopChanged: scrollTopChanged,
        };
    };
    return ScrollState;
}());
var Scrollable = /** @class */ (function (_super) {
    __extends$K(Scrollable, _super);
    function Scrollable(smoothScrollDuration, scheduleAtNextAnimationFrame) {
        var _this = _super.call(this) || this;
        _this._onScroll = _this._register(new Emitter());
        _this.onScroll = _this._onScroll.event;
        _this._smoothScrollDuration = smoothScrollDuration;
        _this._scheduleAtNextAnimationFrame = scheduleAtNextAnimationFrame;
        _this._state = new ScrollState(0, 0, 0, 0, 0, 0);
        _this._smoothScrolling = null;
        return _this;
    }
    Scrollable.prototype.dispose = function () {
        if (this._smoothScrolling) {
            this._smoothScrolling.dispose();
            this._smoothScrolling = null;
        }
        _super.prototype.dispose.call(this);
    };
    Scrollable.prototype.setSmoothScrollDuration = function (smoothScrollDuration) {
        this._smoothScrollDuration = smoothScrollDuration;
    };
    Scrollable.prototype.validateScrollPosition = function (scrollPosition) {
        return this._state.withScrollPosition(scrollPosition);
    };
    Scrollable.prototype.getScrollDimensions = function () {
        return this._state;
    };
    Scrollable.prototype.setScrollDimensions = function (dimensions) {
        var newState = this._state.withScrollDimensions(dimensions);
        this._setState(newState);
        // Validate outstanding animated scroll position target
        if (this._smoothScrolling) {
            this._smoothScrolling.acceptScrollDimensions(this._state);
        }
    };
    /**
     * Returns the final scroll position that the instance will have once the smooth scroll animation concludes.
     * If no scroll animation is occurring, it will return the current scroll position instead.
     */
    Scrollable.prototype.getFutureScrollPosition = function () {
        if (this._smoothScrolling) {
            return this._smoothScrolling.to;
        }
        return this._state;
    };
    /**
     * Returns the current scroll position.
     * Note: This result might be an intermediate scroll position, as there might be an ongoing smooth scroll animation.
     */
    Scrollable.prototype.getCurrentScrollPosition = function () {
        return this._state;
    };
    Scrollable.prototype.setScrollPositionNow = function (update) {
        // no smooth scrolling requested
        var newState = this._state.withScrollPosition(update);
        // Terminate any outstanding smooth scrolling
        if (this._smoothScrolling) {
            this._smoothScrolling.dispose();
            this._smoothScrolling = null;
        }
        this._setState(newState);
    };
    Scrollable.prototype.setScrollPositionSmooth = function (update) {
        var _this = this;
        if (this._smoothScrollDuration === 0) {
            // Smooth scrolling not supported.
            return this.setScrollPositionNow(update);
        }
        if (this._smoothScrolling) {
            // Combine our pending scrollLeft/scrollTop with incoming scrollLeft/scrollTop
            update = {
                scrollLeft: (typeof update.scrollLeft === 'undefined' ? this._smoothScrolling.to.scrollLeft : update.scrollLeft),
                scrollTop: (typeof update.scrollTop === 'undefined' ? this._smoothScrolling.to.scrollTop : update.scrollTop)
            };
            // Validate `update`
            var validTarget = this._state.withScrollPosition(update);
            if (this._smoothScrolling.to.scrollLeft === validTarget.scrollLeft && this._smoothScrolling.to.scrollTop === validTarget.scrollTop) {
                // No need to interrupt or extend the current animation since we're going to the same place
                return;
            }
            var newSmoothScrolling = this._smoothScrolling.combine(this._state, validTarget, this._smoothScrollDuration);
            this._smoothScrolling.dispose();
            this._smoothScrolling = newSmoothScrolling;
        }
        else {
            // Validate `update`
            var validTarget = this._state.withScrollPosition(update);
            this._smoothScrolling = SmoothScrollingOperation.start(this._state, validTarget, this._smoothScrollDuration);
        }
        // Begin smooth scrolling animation
        this._smoothScrolling.animationFrameDisposable = this._scheduleAtNextAnimationFrame(function () {
            if (!_this._smoothScrolling) {
                return;
            }
            _this._smoothScrolling.animationFrameDisposable = null;
            _this._performSmoothScrolling();
        });
    };
    Scrollable.prototype._performSmoothScrolling = function () {
        var _this = this;
        if (!this._smoothScrolling) {
            return;
        }
        var update = this._smoothScrolling.tick();
        var newState = this._state.withScrollPosition(update);
        this._setState(newState);
        if (update.isDone) {
            this._smoothScrolling.dispose();
            this._smoothScrolling = null;
            return;
        }
        // Continue smooth scrolling animation
        this._smoothScrolling.animationFrameDisposable = this._scheduleAtNextAnimationFrame(function () {
            if (!_this._smoothScrolling) {
                return;
            }
            _this._smoothScrolling.animationFrameDisposable = null;
            _this._performSmoothScrolling();
        });
    };
    Scrollable.prototype._setState = function (newState) {
        var oldState = this._state;
        if (oldState.equals(newState)) {
            // no change
            return;
        }
        this._state = newState;
        this._onScroll.fire(this._state.createScrollEvent(oldState));
    };
    return Scrollable;
}(Disposable));
var SmoothScrollingUpdate = /** @class */ (function () {
    function SmoothScrollingUpdate(scrollLeft, scrollTop, isDone) {
        this.scrollLeft = scrollLeft;
        this.scrollTop = scrollTop;
        this.isDone = isDone;
    }
    return SmoothScrollingUpdate;
}());
function createEaseOutCubic(from, to) {
    var delta = to - from;
    return function (completion) {
        return from + delta * easeOutCubic(completion);
    };
}
function createComposed(a, b, cut) {
    return function (completion) {
        if (completion < cut) {
            return a(completion / cut);
        }
        return b((completion - cut) / (1 - cut));
    };
}
var SmoothScrollingOperation = /** @class */ (function () {
    function SmoothScrollingOperation(from, to, startTime, duration) {
        this.from = from;
        this.to = to;
        this.duration = duration;
        this._startTime = startTime;
        this.animationFrameDisposable = null;
        this._initAnimations();
    }
    SmoothScrollingOperation.prototype._initAnimations = function () {
        this.scrollLeft = this._initAnimation(this.from.scrollLeft, this.to.scrollLeft, this.to.width);
        this.scrollTop = this._initAnimation(this.from.scrollTop, this.to.scrollTop, this.to.height);
    };
    SmoothScrollingOperation.prototype._initAnimation = function (from, to, viewportSize) {
        var delta = Math.abs(from - to);
        if (delta > 2.5 * viewportSize) {
            var stop1 = void 0, stop2 = void 0;
            if (from < to) {
                // scroll to 75% of the viewportSize
                stop1 = from + 0.75 * viewportSize;
                stop2 = to - 0.75 * viewportSize;
            }
            else {
                stop1 = from - 0.75 * viewportSize;
                stop2 = to + 0.75 * viewportSize;
            }
            return createComposed(createEaseOutCubic(from, stop1), createEaseOutCubic(stop2, to), 0.33);
        }
        return createEaseOutCubic(from, to);
    };
    SmoothScrollingOperation.prototype.dispose = function () {
        if (this.animationFrameDisposable !== null) {
            this.animationFrameDisposable.dispose();
            this.animationFrameDisposable = null;
        }
    };
    SmoothScrollingOperation.prototype.acceptScrollDimensions = function (state) {
        this.to = state.withScrollPosition(this.to);
        this._initAnimations();
    };
    SmoothScrollingOperation.prototype.tick = function () {
        return this._tick(Date.now());
    };
    SmoothScrollingOperation.prototype._tick = function (now) {
        var completion = (now - this._startTime) / this.duration;
        if (completion < 1) {
            var newScrollLeft = this.scrollLeft(completion);
            var newScrollTop = this.scrollTop(completion);
            return new SmoothScrollingUpdate(newScrollLeft, newScrollTop, false);
        }
        return new SmoothScrollingUpdate(this.to.scrollLeft, this.to.scrollTop, true);
    };
    SmoothScrollingOperation.prototype.combine = function (from, to, duration) {
        return SmoothScrollingOperation.start(from, to, duration);
    };
    SmoothScrollingOperation.start = function (from, to, duration) {
        // +10 / -10 : pretend the animation already started for a quicker response to a scroll request
        duration = duration + 10;
        var startTime = Date.now() - 10;
        return new SmoothScrollingOperation(from, to, startTime, duration);
    };
    return SmoothScrollingOperation;
}());
function easeInCubic(t) {
    return Math.pow(t, 3);
}
function easeOutCubic(t) {
    return 1 - easeInCubic(1 - t);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$L = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var HIDE_TIMEOUT = 500;
var SCROLL_WHEEL_SENSITIVITY = 50;
var MouseWheelClassifierItem = /** @class */ (function () {
    function MouseWheelClassifierItem(timestamp, deltaX, deltaY) {
        this.timestamp = timestamp;
        this.deltaX = deltaX;
        this.deltaY = deltaY;
        this.score = 0;
    }
    return MouseWheelClassifierItem;
}());
var MouseWheelClassifier = /** @class */ (function () {
    function MouseWheelClassifier() {
        this._capacity = 5;
        this._memory = [];
        this._front = -1;
        this._rear = -1;
    }
    MouseWheelClassifier.prototype.isPhysicalMouseWheel = function () {
        if (this._front === -1 && this._rear === -1) {
            // no elements
            return false;
        }
        // 0.5 * last + 0.25 * before last + 0.125 * before before last + ...
        var remainingInfluence = 1;
        var score = 0;
        var iteration = 1;
        var index = this._rear;
        do {
            var influence = (index === this._front ? remainingInfluence : Math.pow(2, -iteration));
            remainingInfluence -= influence;
            score += this._memory[index].score * influence;
            if (index === this._front) {
                break;
            }
            index = (this._capacity + index - 1) % this._capacity;
            iteration++;
        } while (true);
        return (score <= 0.5);
    };
    MouseWheelClassifier.prototype.accept = function (timestamp, deltaX, deltaY) {
        var item = new MouseWheelClassifierItem(timestamp, deltaX, deltaY);
        item.score = this._computeScore(item);
        if (this._front === -1 && this._rear === -1) {
            this._memory[0] = item;
            this._front = 0;
            this._rear = 0;
        }
        else {
            this._rear = (this._rear + 1) % this._capacity;
            if (this._rear === this._front) {
                // Drop oldest
                this._front = (this._front + 1) % this._capacity;
            }
            this._memory[this._rear] = item;
        }
    };
    /**
     * A score between 0 and 1 for `item`.
     *  - a score towards 0 indicates that the source appears to be a physical mouse wheel
     *  - a score towards 1 indicates that the source appears to be a touchpad or magic mouse, etc.
     */
    MouseWheelClassifier.prototype._computeScore = function (item) {
        if (Math.abs(item.deltaX) > 0 && Math.abs(item.deltaY) > 0) {
            // both axes exercised => definitely not a physical mouse wheel
            return 1;
        }
        var score = 0.5;
        var prev = (this._front === -1 && this._rear === -1 ? null : this._memory[this._rear]);
        if (Math.abs(item.deltaX - Math.round(item.deltaX)) > 0 || Math.abs(item.deltaY - Math.round(item.deltaY)) > 0) {
            // non-integer deltas => indicator that this is not a physical mouse wheel
            score += 0.25;
        }
        return Math.min(Math.max(score, 0), 1);
    };
    MouseWheelClassifier.INSTANCE = new MouseWheelClassifier();
    return MouseWheelClassifier;
}());
var AbstractScrollableElement = /** @class */ (function (_super) {
    __extends$L(AbstractScrollableElement, _super);
    function AbstractScrollableElement(element, options, scrollable) {
        var _this = _super.call(this) || this;
        _this._onScroll = _this._register(new Emitter());
        _this.onScroll = _this._onScroll.event;
        element.style.overflow = 'hidden';
        _this._options = resolveOptions(options);
        _this._scrollable = scrollable;
        _this._register(_this._scrollable.onScroll(function (e) {
            _this._onDidScroll(e);
            _this._onScroll.fire(e);
        }));
        var scrollbarHost = {
            onMouseWheel: function (mouseWheelEvent) { return _this._onMouseWheel(mouseWheelEvent); },
            onDragStart: function () { return _this._onDragStart(); },
            onDragEnd: function () { return _this._onDragEnd(); },
        };
        _this._verticalScrollbar = _this._register(new VerticalScrollbar(_this._scrollable, _this._options, scrollbarHost));
        _this._horizontalScrollbar = _this._register(new HorizontalScrollbar(_this._scrollable, _this._options, scrollbarHost));
        _this._domNode = document.createElement('div');
        _this._domNode.className = 'monaco-scrollable-element ' + _this._options.className;
        _this._domNode.setAttribute('role', 'presentation');
        _this._domNode.style.position = 'relative';
        _this._domNode.style.overflow = 'hidden';
        _this._domNode.appendChild(element);
        _this._domNode.appendChild(_this._horizontalScrollbar.domNode.domNode);
        _this._domNode.appendChild(_this._verticalScrollbar.domNode.domNode);
        if (_this._options.useShadows) {
            _this._leftShadowDomNode = createFastDomNode(document.createElement('div'));
            _this._leftShadowDomNode.setClassName('shadow');
            _this._domNode.appendChild(_this._leftShadowDomNode.domNode);
            _this._topShadowDomNode = createFastDomNode(document.createElement('div'));
            _this._topShadowDomNode.setClassName('shadow');
            _this._domNode.appendChild(_this._topShadowDomNode.domNode);
            _this._topLeftShadowDomNode = createFastDomNode(document.createElement('div'));
            _this._topLeftShadowDomNode.setClassName('shadow top-left-corner');
            _this._domNode.appendChild(_this._topLeftShadowDomNode.domNode);
        }
        _this._listenOnDomNode = _this._options.listenOnDomNode || _this._domNode;
        _this._mouseWheelToDispose = [];
        _this._setListeningToMouseWheel(_this._options.handleMouseWheel);
        _this.onmouseover(_this._listenOnDomNode, function (e) { return _this._onMouseOver(e); });
        _this.onnonbubblingmouseout(_this._listenOnDomNode, function (e) { return _this._onMouseOut(e); });
        _this._hideTimeout = _this._register(new TimeoutTimer());
        _this._isDragging = false;
        _this._mouseIsOver = false;
        _this._shouldRender = true;
        _this._revealOnScroll = true;
        return _this;
    }
    AbstractScrollableElement.prototype.dispose = function () {
        this._mouseWheelToDispose = dispose(this._mouseWheelToDispose);
        _super.prototype.dispose.call(this);
    };
    /**
     * Get the generated 'scrollable' dom node
     */
    AbstractScrollableElement.prototype.getDomNode = function () {
        return this._domNode;
    };
    AbstractScrollableElement.prototype.getOverviewRulerLayoutInfo = function () {
        return {
            parent: this._domNode,
            insertBefore: this._verticalScrollbar.domNode.domNode,
        };
    };
    /**
     * Delegate a mouse down event to the vertical scrollbar.
     * This is to help with clicking somewhere else and having the scrollbar react.
     */
    AbstractScrollableElement.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
        this._verticalScrollbar.delegateMouseDown(browserEvent);
    };
    AbstractScrollableElement.prototype.getScrollDimensions = function () {
        return this._scrollable.getScrollDimensions();
    };
    AbstractScrollableElement.prototype.setScrollDimensions = function (dimensions) {
        this._scrollable.setScrollDimensions(dimensions);
    };
    /**
     * Update the class name of the scrollable element.
     */
    AbstractScrollableElement.prototype.updateClassName = function (newClassName) {
        this._options.className = newClassName;
        // Defaults are different on Macs
        if (isMacintosh) {
            this._options.className += ' mac';
        }
        this._domNode.className = 'monaco-scrollable-element ' + this._options.className;
    };
    /**
     * Update configuration options for the scrollbar.
     * Really this is Editor.IEditorScrollbarOptions, but base shouldn't
     * depend on Editor.
     */
    AbstractScrollableElement.prototype.updateOptions = function (newOptions) {
        var massagedOptions = resolveOptions(newOptions);
        this._options.handleMouseWheel = massagedOptions.handleMouseWheel;
        this._options.mouseWheelScrollSensitivity = massagedOptions.mouseWheelScrollSensitivity;
        this._setListeningToMouseWheel(this._options.handleMouseWheel);
        if (!this._options.lazyRender) {
            this._render();
        }
    };
    // -------------------- mouse wheel scrolling --------------------
    AbstractScrollableElement.prototype._setListeningToMouseWheel = function (shouldListen) {
        var _this = this;
        var isListening = (this._mouseWheelToDispose.length > 0);
        if (isListening === shouldListen) {
            // No change
            return;
        }
        // Stop listening (if necessary)
        this._mouseWheelToDispose = dispose(this._mouseWheelToDispose);
        // Start listening (if necessary)
        if (shouldListen) {
            var onMouseWheel = function (browserEvent) {
                var e = new StandardMouseWheelEvent(browserEvent);
                _this._onMouseWheel(e);
            };
            this._mouseWheelToDispose.push(addDisposableListener(this._listenOnDomNode, 'mousewheel', onMouseWheel));
            this._mouseWheelToDispose.push(addDisposableListener(this._listenOnDomNode, 'DOMMouseScroll', onMouseWheel));
        }
    };
    AbstractScrollableElement.prototype._onMouseWheel = function (e) {
        var _a;
        var classifier = MouseWheelClassifier.INSTANCE;
        {
            classifier.accept(Date.now(), e.deltaX, e.deltaY);
        }
        // console.log(`${Date.now()}, ${e.deltaY}, ${e.deltaX}`);
        if (e.deltaY || e.deltaX) {
            var deltaY = e.deltaY * this._options.mouseWheelScrollSensitivity;
            var deltaX = e.deltaX * this._options.mouseWheelScrollSensitivity;
            if (this._options.flipAxes) {
                _a = [deltaX, deltaY], deltaY = _a[0], deltaX = _a[1];
            }
            // Convert vertical scrolling to horizontal if shift is held, this
            // is handled at a higher level on Mac
            var shiftConvert = !isMacintosh && e.browserEvent && e.browserEvent.shiftKey;
            if ((this._options.scrollYToX || shiftConvert) && !deltaX) {
                deltaX = deltaY;
                deltaY = 0;
            }
            var futureScrollPosition = this._scrollable.getFutureScrollPosition();
            var desiredScrollPosition = {};
            if (deltaY) {
                var desiredScrollTop = futureScrollPosition.scrollTop - SCROLL_WHEEL_SENSITIVITY * deltaY;
                this._verticalScrollbar.writeScrollPosition(desiredScrollPosition, desiredScrollTop);
            }
            if (deltaX) {
                var desiredScrollLeft = futureScrollPosition.scrollLeft - SCROLL_WHEEL_SENSITIVITY * deltaX;
                this._horizontalScrollbar.writeScrollPosition(desiredScrollPosition, desiredScrollLeft);
            }
            // Check that we are scrolling towards a location which is valid
            desiredScrollPosition = this._scrollable.validateScrollPosition(desiredScrollPosition);
            if (futureScrollPosition.scrollLeft !== desiredScrollPosition.scrollLeft || futureScrollPosition.scrollTop !== desiredScrollPosition.scrollTop) {
                var canPerformSmoothScroll = (this._options.mouseWheelSmoothScroll
                    && classifier.isPhysicalMouseWheel());
                if (canPerformSmoothScroll) {
                    this._scrollable.setScrollPositionSmooth(desiredScrollPosition);
                }
                else {
                    this._scrollable.setScrollPositionNow(desiredScrollPosition);
                }
                this._shouldRender = true;
            }
        }
        if (this._options.alwaysConsumeMouseWheel || this._shouldRender) {
            e.preventDefault();
            e.stopPropagation();
        }
    };
    AbstractScrollableElement.prototype._onDidScroll = function (e) {
        this._shouldRender = this._horizontalScrollbar.onDidScroll(e) || this._shouldRender;
        this._shouldRender = this._verticalScrollbar.onDidScroll(e) || this._shouldRender;
        if (this._options.useShadows) {
            this._shouldRender = true;
        }
        if (this._revealOnScroll) {
            this._reveal();
        }
        if (!this._options.lazyRender) {
            this._render();
        }
    };
    /**
     * Render / mutate the DOM now.
     * Should be used together with the ctor option `lazyRender`.
     */
    AbstractScrollableElement.prototype.renderNow = function () {
        if (!this._options.lazyRender) {
            throw new Error('Please use `lazyRender` together with `renderNow`!');
        }
        this._render();
    };
    AbstractScrollableElement.prototype._render = function () {
        if (!this._shouldRender) {
            return;
        }
        this._shouldRender = false;
        this._horizontalScrollbar.render();
        this._verticalScrollbar.render();
        if (this._options.useShadows) {
            var scrollState = this._scrollable.getCurrentScrollPosition();
            var enableTop = scrollState.scrollTop > 0;
            var enableLeft = scrollState.scrollLeft > 0;
            this._leftShadowDomNode.setClassName('shadow' + (enableLeft ? ' left' : ''));
            this._topShadowDomNode.setClassName('shadow' + (enableTop ? ' top' : ''));
            this._topLeftShadowDomNode.setClassName('shadow top-left-corner' + (enableTop ? ' top' : '') + (enableLeft ? ' left' : ''));
        }
    };
    // -------------------- fade in / fade out --------------------
    AbstractScrollableElement.prototype._onDragStart = function () {
        this._isDragging = true;
        this._reveal();
    };
    AbstractScrollableElement.prototype._onDragEnd = function () {
        this._isDragging = false;
        this._hide();
    };
    AbstractScrollableElement.prototype._onMouseOut = function (e) {
        this._mouseIsOver = false;
        this._hide();
    };
    AbstractScrollableElement.prototype._onMouseOver = function (e) {
        this._mouseIsOver = true;
        this._reveal();
    };
    AbstractScrollableElement.prototype._reveal = function () {
        this._verticalScrollbar.beginReveal();
        this._horizontalScrollbar.beginReveal();
        this._scheduleHide();
    };
    AbstractScrollableElement.prototype._hide = function () {
        if (!this._mouseIsOver && !this._isDragging) {
            this._verticalScrollbar.beginHide();
            this._horizontalScrollbar.beginHide();
        }
    };
    AbstractScrollableElement.prototype._scheduleHide = function () {
        var _this = this;
        if (!this._mouseIsOver && !this._isDragging) {
            this._hideTimeout.cancelAndSet(function () { return _this._hide(); }, HIDE_TIMEOUT);
        }
    };
    return AbstractScrollableElement;
}(Widget$1));
var ScrollableElement = /** @class */ (function (_super) {
    __extends$L(ScrollableElement, _super);
    function ScrollableElement(element, options) {
        var _this = this;
        options = options || {};
        options.mouseWheelSmoothScroll = false;
        var scrollable = new Scrollable(0, function (callback) { return scheduleAtNextAnimationFrame(callback); });
        _this = _super.call(this, element, options, scrollable) || this;
        _this._register(scrollable);
        return _this;
    }
    ScrollableElement.prototype.setScrollPosition = function (update) {
        this._scrollable.setScrollPositionNow(update);
    };
    ScrollableElement.prototype.getScrollPosition = function () {
        return this._scrollable.getCurrentScrollPosition();
    };
    return ScrollableElement;
}(AbstractScrollableElement));
var SmoothScrollableElement = /** @class */ (function (_super) {
    __extends$L(SmoothScrollableElement, _super);
    function SmoothScrollableElement(element, options, scrollable) {
        return _super.call(this, element, options, scrollable) || this;
    }
    return SmoothScrollableElement;
}(AbstractScrollableElement));
var DomScrollableElement = /** @class */ (function (_super) {
    __extends$L(DomScrollableElement, _super);
    function DomScrollableElement(element, options) {
        var _this = _super.call(this, element, options) || this;
        _this._element = element;
        _this.onScroll(function (e) {
            if (e.scrollTopChanged) {
                _this._element.scrollTop = e.scrollTop;
            }
            if (e.scrollLeftChanged) {
                _this._element.scrollLeft = e.scrollLeft;
            }
        });
        _this.scanDomNode();
        return _this;
    }
    DomScrollableElement.prototype.scanDomNode = function () {
        // widh, scrollLeft, scrollWidth, height, scrollTop, scrollHeight
        this.setScrollDimensions({
            width: this._element.clientWidth,
            scrollWidth: this._element.scrollWidth,
            height: this._element.clientHeight,
            scrollHeight: this._element.scrollHeight
        });
        this.setScrollPosition({
            scrollLeft: this._element.scrollLeft,
            scrollTop: this._element.scrollTop,
        });
    };
    return DomScrollableElement;
}(ScrollableElement));
function resolveOptions(opts) {
    var result = {
        lazyRender: (typeof opts.lazyRender !== 'undefined' ? opts.lazyRender : false),
        className: (typeof opts.className !== 'undefined' ? opts.className : ''),
        useShadows: (typeof opts.useShadows !== 'undefined' ? opts.useShadows : true),
        handleMouseWheel: (typeof opts.handleMouseWheel !== 'undefined' ? opts.handleMouseWheel : true),
        flipAxes: (typeof opts.flipAxes !== 'undefined' ? opts.flipAxes : false),
        alwaysConsumeMouseWheel: (typeof opts.alwaysConsumeMouseWheel !== 'undefined' ? opts.alwaysConsumeMouseWheel : false),
        scrollYToX: (typeof opts.scrollYToX !== 'undefined' ? opts.scrollYToX : false),
        mouseWheelScrollSensitivity: (typeof opts.mouseWheelScrollSensitivity !== 'undefined' ? opts.mouseWheelScrollSensitivity : 1),
        mouseWheelSmoothScroll: (typeof opts.mouseWheelSmoothScroll !== 'undefined' ? opts.mouseWheelSmoothScroll : true),
        arrowSize: (typeof opts.arrowSize !== 'undefined' ? opts.arrowSize : 11),
        listenOnDomNode: (typeof opts.listenOnDomNode !== 'undefined' ? opts.listenOnDomNode : null),
        horizontal: (typeof opts.horizontal !== 'undefined' ? opts.horizontal : 1 /* Auto */),
        horizontalScrollbarSize: (typeof opts.horizontalScrollbarSize !== 'undefined' ? opts.horizontalScrollbarSize : 10),
        horizontalSliderSize: (typeof opts.horizontalSliderSize !== 'undefined' ? opts.horizontalSliderSize : 0),
        horizontalHasArrows: (typeof opts.horizontalHasArrows !== 'undefined' ? opts.horizontalHasArrows : false),
        vertical: (typeof opts.vertical !== 'undefined' ? opts.vertical : 1 /* Auto */),
        verticalScrollbarSize: (typeof opts.verticalScrollbarSize !== 'undefined' ? opts.verticalScrollbarSize : 10),
        verticalHasArrows: (typeof opts.verticalHasArrows !== 'undefined' ? opts.verticalHasArrows : false),
        verticalSliderSize: (typeof opts.verticalSliderSize !== 'undefined' ? opts.verticalSliderSize : 0)
    };
    result.horizontalSliderSize = (typeof opts.horizontalSliderSize !== 'undefined' ? opts.horizontalSliderSize : result.horizontalScrollbarSize);
    result.verticalSliderSize = (typeof opts.verticalSliderSize !== 'undefined' ? opts.verticalSliderSize : result.verticalScrollbarSize);
    // Defaults are different on Macs
    if (isMacintosh) {
        result.className += ' mac';
    }
    return result;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$M = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var EditorScrollbar = /** @class */ (function (_super) {
    __extends$M(EditorScrollbar, _super);
    function EditorScrollbar(context, linesContent, viewDomNode, overflowGuardDomNode) {
        var _this = _super.call(this, context) || this;
        var editor = _this._context.configuration.editor;
        var configScrollbarOpts = editor.viewInfo.scrollbar;
        var scrollbarOptions = {
            listenOnDomNode: viewDomNode.domNode,
            className: 'editor-scrollable' + ' ' + getThemeTypeSelector(context.theme.type),
            useShadows: false,
            lazyRender: true,
            vertical: configScrollbarOpts.vertical,
            horizontal: configScrollbarOpts.horizontal,
            verticalHasArrows: configScrollbarOpts.verticalHasArrows,
            horizontalHasArrows: configScrollbarOpts.horizontalHasArrows,
            verticalScrollbarSize: configScrollbarOpts.verticalScrollbarSize,
            verticalSliderSize: configScrollbarOpts.verticalSliderSize,
            horizontalScrollbarSize: configScrollbarOpts.horizontalScrollbarSize,
            horizontalSliderSize: configScrollbarOpts.horizontalSliderSize,
            handleMouseWheel: configScrollbarOpts.handleMouseWheel,
            arrowSize: configScrollbarOpts.arrowSize,
            mouseWheelScrollSensitivity: configScrollbarOpts.mouseWheelScrollSensitivity,
        };
        _this.scrollbar = _this._register(new SmoothScrollableElement(linesContent.domNode, scrollbarOptions, _this._context.viewLayout.scrollable));
        PartFingerprints.write(_this.scrollbar.getDomNode(), 5 /* ScrollableElement */);
        _this.scrollbarDomNode = createFastDomNode(_this.scrollbar.getDomNode());
        _this.scrollbarDomNode.setPosition('absolute');
        _this._setLayout();
        // When having a zone widget that calls .focus() on one of its dom elements,
        // the browser will try desperately to reveal that dom node, unexpectedly
        // changing the .scrollTop of this.linesContent
        var onBrowserDesperateReveal = function (domNode, lookAtScrollTop, lookAtScrollLeft) {
            var newScrollPosition = {};
            if (lookAtScrollTop) {
                var deltaTop = domNode.scrollTop;
                if (deltaTop) {
                    newScrollPosition.scrollTop = _this._context.viewLayout.getCurrentScrollTop() + deltaTop;
                    domNode.scrollTop = 0;
                }
            }
            if (lookAtScrollLeft) {
                var deltaLeft = domNode.scrollLeft;
                if (deltaLeft) {
                    newScrollPosition.scrollLeft = _this._context.viewLayout.getCurrentScrollLeft() + deltaLeft;
                    domNode.scrollLeft = 0;
                }
            }
            _this._context.viewLayout.setScrollPositionNow(newScrollPosition);
        };
        // I've seen this happen both on the view dom node & on the lines content dom node.
        _this._register(addDisposableListener(viewDomNode.domNode, 'scroll', function (e) { return onBrowserDesperateReveal(viewDomNode.domNode, true, true); }));
        _this._register(addDisposableListener(linesContent.domNode, 'scroll', function (e) { return onBrowserDesperateReveal(linesContent.domNode, true, false); }));
        _this._register(addDisposableListener(overflowGuardDomNode.domNode, 'scroll', function (e) { return onBrowserDesperateReveal(overflowGuardDomNode.domNode, true, false); }));
        _this._register(addDisposableListener(_this.scrollbarDomNode.domNode, 'scroll', function (e) { return onBrowserDesperateReveal(_this.scrollbarDomNode.domNode, true, false); }));
        return _this;
    }
    EditorScrollbar.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    EditorScrollbar.prototype._setLayout = function () {
        var layoutInfo = this._context.configuration.editor.layoutInfo;
        this.scrollbarDomNode.setLeft(layoutInfo.contentLeft);
        var side = this._context.configuration.editor.viewInfo.minimap.side;
        if (side === 'right') {
            this.scrollbarDomNode.setWidth(layoutInfo.contentWidth + layoutInfo.minimapWidth);
        }
        else {
            this.scrollbarDomNode.setWidth(layoutInfo.contentWidth);
        }
        this.scrollbarDomNode.setHeight(layoutInfo.contentHeight);
    };
    EditorScrollbar.prototype.getOverviewRulerLayoutInfo = function () {
        return this.scrollbar.getOverviewRulerLayoutInfo();
    };
    EditorScrollbar.prototype.getDomNode = function () {
        return this.scrollbarDomNode;
    };
    EditorScrollbar.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
        this.scrollbar.delegateVerticalScrollbarMouseDown(browserEvent);
    };
    // --- begin event handlers
    EditorScrollbar.prototype.onConfigurationChanged = function (e) {
        if (e.viewInfo) {
            var editor = this._context.configuration.editor;
            var newOpts = {
                handleMouseWheel: editor.viewInfo.scrollbar.handleMouseWheel,
                mouseWheelScrollSensitivity: editor.viewInfo.scrollbar.mouseWheelScrollSensitivity
            };
            this.scrollbar.updateOptions(newOpts);
        }
        if (e.layoutInfo) {
            this._setLayout();
        }
        return true;
    };
    EditorScrollbar.prototype.onScrollChanged = function (e) {
        return true;
    };
    EditorScrollbar.prototype.onThemeChanged = function (e) {
        this.scrollbar.updateClassName('editor-scrollable' + ' ' + getThemeTypeSelector(this._context.theme.type));
        return true;
    };
    // --- end event handlers
    EditorScrollbar.prototype.prepareRender = function (ctx) {
        // Nothing to do
    };
    EditorScrollbar.prototype.render = function (ctx) {
        this.scrollbar.renderNow();
    };
    return EditorScrollbar;
}(ViewPart));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$N = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DecorationToRender = /** @class */ (function () {
    function DecorationToRender(startLineNumber, endLineNumber, className) {
        this.startLineNumber = +startLineNumber;
        this.endLineNumber = +endLineNumber;
        this.className = String(className);
    }
    return DecorationToRender;
}());
var DedupOverlay = /** @class */ (function (_super) {
    __extends$N(DedupOverlay, _super);
    function DedupOverlay() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DedupOverlay.prototype._render = function (visibleStartLineNumber, visibleEndLineNumber, decorations) {
        var output = [];
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var lineIndex = lineNumber - visibleStartLineNumber;
            output[lineIndex] = [];
        }
        if (decorations.length === 0) {
            return output;
        }
        decorations.sort(function (a, b) {
            if (a.className === b.className) {
                if (a.startLineNumber === b.startLineNumber) {
                    return a.endLineNumber - b.endLineNumber;
                }
                return a.startLineNumber - b.startLineNumber;
            }
            return (a.className < b.className ? -1 : 1);
        });
        var prevClassName = null;
        var prevEndLineIndex = 0;
        for (var i = 0, len = decorations.length; i < len; i++) {
            var d = decorations[i];
            var className = d.className;
            var startLineIndex = Math.max(d.startLineNumber, visibleStartLineNumber) - visibleStartLineNumber;
            var endLineIndex = Math.min(d.endLineNumber, visibleEndLineNumber) - visibleStartLineNumber;
            if (prevClassName === className) {
                startLineIndex = Math.max(prevEndLineIndex + 1, startLineIndex);
                prevEndLineIndex = Math.max(prevEndLineIndex, endLineIndex);
            }
            else {
                prevClassName = className;
                prevEndLineIndex = endLineIndex;
            }
            for (var i_1 = startLineIndex; i_1 <= prevEndLineIndex; i_1++) {
                output[i_1].push(prevClassName);
            }
        }
        return output;
    };
    return DedupOverlay;
}(DynamicViewOverlay));
var GlyphMarginOverlay = /** @class */ (function (_super) {
    __extends$N(GlyphMarginOverlay, _super);
    function GlyphMarginOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._lineHeight = _this._context.configuration.editor.lineHeight;
        _this._glyphMargin = _this._context.configuration.editor.viewInfo.glyphMargin;
        _this._glyphMarginLeft = _this._context.configuration.editor.layoutInfo.glyphMarginLeft;
        _this._glyphMarginWidth = _this._context.configuration.editor.layoutInfo.glyphMarginWidth;
        _this._renderResult = null;
        _this._context.addEventHandler(_this);
        return _this;
    }
    GlyphMarginOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        this._renderResult = null;
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    GlyphMarginOverlay.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.viewInfo) {
            this._glyphMargin = this._context.configuration.editor.viewInfo.glyphMargin;
        }
        if (e.layoutInfo) {
            this._glyphMarginLeft = this._context.configuration.editor.layoutInfo.glyphMarginLeft;
            this._glyphMarginWidth = this._context.configuration.editor.layoutInfo.glyphMarginWidth;
        }
        return true;
    };
    GlyphMarginOverlay.prototype.onDecorationsChanged = function (e) {
        return true;
    };
    GlyphMarginOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    GlyphMarginOverlay.prototype.onLinesChanged = function (e) {
        return true;
    };
    GlyphMarginOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    GlyphMarginOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    GlyphMarginOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged;
    };
    GlyphMarginOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    GlyphMarginOverlay.prototype._getDecorations = function (ctx) {
        var decorations = ctx.getDecorationsInViewport();
        var r = [], rLen = 0;
        for (var i = 0, len = decorations.length; i < len; i++) {
            var d = decorations[i];
            var glyphMarginClassName = d.options.glyphMarginClassName;
            if (glyphMarginClassName) {
                r[rLen++] = new DecorationToRender(d.range.startLineNumber, d.range.endLineNumber, glyphMarginClassName);
            }
        }
        return r;
    };
    GlyphMarginOverlay.prototype.prepareRender = function (ctx) {
        if (!this._glyphMargin) {
            this._renderResult = null;
            return;
        }
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        var toRender = this._render(visibleStartLineNumber, visibleEndLineNumber, this._getDecorations(ctx));
        var lineHeight = this._lineHeight.toString();
        var left = this._glyphMarginLeft.toString();
        var width = this._glyphMarginWidth.toString();
        var common = '" style="left:' + left + 'px;width:' + width + 'px' + ';height:' + lineHeight + 'px;"></div>';
        var output = [];
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var lineIndex = lineNumber - visibleStartLineNumber;
            var classNames = toRender[lineIndex];
            if (classNames.length === 0) {
                output[lineIndex] = '';
            }
            else {
                output[lineIndex] = ('<div class="cgmr '
                    + classNames.join(' ')
                    + common);
            }
        }
        this._renderResult = output;
    };
    GlyphMarginOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (!this._renderResult) {
            return '';
        }
        var lineIndex = lineNumber - startLineNumber;
        if (lineIndex < 0 || lineIndex >= this._renderResult.length) {
            return '';
        }
        return this._renderResult[lineIndex];
    };
    return GlyphMarginOverlay;
}(DedupOverlay));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$O = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var IndentGuidesOverlay = /** @class */ (function (_super) {
    __extends$O(IndentGuidesOverlay, _super);
    function IndentGuidesOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._primaryLineNumber = 0;
        _this._lineHeight = _this._context.configuration.editor.lineHeight;
        _this._spaceWidth = _this._context.configuration.editor.fontInfo.spaceWidth;
        _this._enabled = _this._context.configuration.editor.viewInfo.renderIndentGuides;
        _this._activeIndentEnabled = _this._context.configuration.editor.viewInfo.highlightActiveIndentGuide;
        _this._renderResult = null;
        _this._context.addEventHandler(_this);
        return _this;
    }
    IndentGuidesOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        this._renderResult = null;
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    IndentGuidesOverlay.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.fontInfo) {
            this._spaceWidth = this._context.configuration.editor.fontInfo.spaceWidth;
        }
        if (e.viewInfo) {
            this._enabled = this._context.configuration.editor.viewInfo.renderIndentGuides;
            this._activeIndentEnabled = this._context.configuration.editor.viewInfo.highlightActiveIndentGuide;
        }
        return true;
    };
    IndentGuidesOverlay.prototype.onCursorStateChanged = function (e) {
        var selection = e.selections[0];
        var newPrimaryLineNumber = selection.isEmpty() ? selection.positionLineNumber : 0;
        if (this._primaryLineNumber !== newPrimaryLineNumber) {
            this._primaryLineNumber = newPrimaryLineNumber;
            return true;
        }
        return false;
    };
    IndentGuidesOverlay.prototype.onDecorationsChanged = function (e) {
        // true for inline decorations
        return true;
    };
    IndentGuidesOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    IndentGuidesOverlay.prototype.onLinesChanged = function (e) {
        return true;
    };
    IndentGuidesOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    IndentGuidesOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    IndentGuidesOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged; // || e.scrollWidthChanged;
    };
    IndentGuidesOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    IndentGuidesOverlay.prototype.onLanguageConfigurationChanged = function (e) {
        return true;
    };
    // --- end event handlers
    IndentGuidesOverlay.prototype.prepareRender = function (ctx) {
        if (!this._enabled) {
            this._renderResult = null;
            return;
        }
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        var tabSize = this._context.model.getTabSize();
        var tabWidth = tabSize * this._spaceWidth;
        var scrollWidth = ctx.scrollWidth;
        var lineHeight = this._lineHeight;
        var indentGuideWidth = tabWidth;
        var indents = this._context.model.getLinesIndentGuides(visibleStartLineNumber, visibleEndLineNumber);
        var activeIndentStartLineNumber = 0;
        var activeIndentEndLineNumber = 0;
        var activeIndentLevel = 0;
        if (this._activeIndentEnabled && this._primaryLineNumber) {
            var activeIndentInfo = this._context.model.getActiveIndentGuide(this._primaryLineNumber, visibleStartLineNumber, visibleEndLineNumber);
            activeIndentStartLineNumber = activeIndentInfo.startLineNumber;
            activeIndentEndLineNumber = activeIndentInfo.endLineNumber;
            activeIndentLevel = activeIndentInfo.indent;
        }
        var output = [];
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var containsActiveIndentGuide = (activeIndentStartLineNumber <= lineNumber && lineNumber <= activeIndentEndLineNumber);
            var lineIndex = lineNumber - visibleStartLineNumber;
            var indent = indents[lineIndex];
            var result = '';
            var leftMostVisiblePosition = ctx.visibleRangeForPosition(new Position(lineNumber, 1));
            var left = leftMostVisiblePosition ? leftMostVisiblePosition.left : 0;
            for (var i = 1; i <= indent; i++) {
                var className = (containsActiveIndentGuide && i === activeIndentLevel ? 'cigra' : 'cigr');
                result += "<div class=\"" + className + "\" style=\"left:" + left + "px;height:" + lineHeight + "px;width:" + indentGuideWidth + "px\"></div>";
                left += tabWidth;
                if (left > scrollWidth) {
                    break;
                }
            }
            output[lineIndex] = result;
        }
        this._renderResult = output;
    };
    IndentGuidesOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (!this._renderResult) {
            return '';
        }
        var lineIndex = lineNumber - startLineNumber;
        if (lineIndex < 0 || lineIndex >= this._renderResult.length) {
            return '';
        }
        return this._renderResult[lineIndex];
    };
    return IndentGuidesOverlay;
}(DynamicViewOverlay));
registerThemingParticipant(function (theme, collector) {
    var editorIndentGuidesColor = theme.getColor(editorIndentGuides);
    if (editorIndentGuidesColor) {
        collector.addRule(".monaco-editor .lines-content .cigr { box-shadow: 1px 0 0 0 " + editorIndentGuidesColor + " inset; }");
    }
    var editorActiveIndentGuidesColor = theme.getColor(editorActiveIndentGuides) || editorIndentGuidesColor;
    if (editorActiveIndentGuidesColor) {
        collector.addRule(".monaco-editor .lines-content .cigra { box-shadow: 1px 0 0 0 " + editorActiveIndentGuidesColor + " inset; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$P = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var LastRenderedData = /** @class */ (function () {
    function LastRenderedData() {
        this._currentVisibleRange = new Range(1, 1, 1, 1);
    }
    LastRenderedData.prototype.getCurrentVisibleRange = function () {
        return this._currentVisibleRange;
    };
    LastRenderedData.prototype.setCurrentVisibleRange = function (currentVisibleRange) {
        this._currentVisibleRange = currentVisibleRange;
    };
    return LastRenderedData;
}());
var HorizontalRevealRequest = /** @class */ (function () {
    function HorizontalRevealRequest(lineNumber, startColumn, endColumn, startScrollTop, stopScrollTop, scrollType) {
        this.lineNumber = lineNumber;
        this.startColumn = startColumn;
        this.endColumn = endColumn;
        this.startScrollTop = startScrollTop;
        this.stopScrollTop = stopScrollTop;
        this.scrollType = scrollType;
    }
    return HorizontalRevealRequest;
}());
var ViewLines = /** @class */ (function (_super) {
    __extends$P(ViewLines, _super);
    function ViewLines(context, linesContent) {
        var _this = _super.call(this, context) || this;
        _this._linesContent = linesContent;
        _this._textRangeRestingSpot = document.createElement('div');
        _this._visibleLines = new VisibleLinesCollection(_this);
        _this.domNode = _this._visibleLines.domNode;
        var conf = _this._context.configuration;
        _this._lineHeight = conf.editor.lineHeight;
        _this._typicalHalfwidthCharacterWidth = conf.editor.fontInfo.typicalHalfwidthCharacterWidth;
        _this._isViewportWrapping = conf.editor.wrappingInfo.isViewportWrapping;
        _this._revealHorizontalRightPadding = conf.editor.viewInfo.revealHorizontalRightPadding;
        _this._canUseLayerHinting = conf.editor.canUseLayerHinting;
        _this._viewLineOptions = new ViewLineOptions(conf, _this._context.theme.type);
        PartFingerprints.write(_this.domNode, 7 /* ViewLines */);
        _this.domNode.setClassName('view-lines');
        Configuration.applyFontInfo(_this.domNode, conf.editor.fontInfo);
        // --- width & height
        _this._maxLineWidth = 0;
        _this._asyncUpdateLineWidths = new RunOnceScheduler(function () {
            _this._updateLineWidthsSlow();
        }, 200);
        _this._lastRenderedData = new LastRenderedData();
        _this._horizontalRevealRequest = null;
        return _this;
    }
    ViewLines.prototype.dispose = function () {
        this._asyncUpdateLineWidths.dispose();
        _super.prototype.dispose.call(this);
    };
    ViewLines.prototype.getDomNode = function () {
        return this.domNode;
    };
    // ---- begin IVisibleLinesHost
    ViewLines.prototype.createVisibleLine = function () {
        return new ViewLine(this._viewLineOptions);
    };
    // ---- end IVisibleLinesHost
    // ---- begin view event handlers
    ViewLines.prototype.onConfigurationChanged = function (e) {
        this._visibleLines.onConfigurationChanged(e);
        if (e.wrappingInfo) {
            this._maxLineWidth = 0;
        }
        var conf = this._context.configuration;
        if (e.lineHeight) {
            this._lineHeight = conf.editor.lineHeight;
        }
        if (e.fontInfo) {
            this._typicalHalfwidthCharacterWidth = conf.editor.fontInfo.typicalHalfwidthCharacterWidth;
        }
        if (e.wrappingInfo) {
            this._isViewportWrapping = conf.editor.wrappingInfo.isViewportWrapping;
        }
        if (e.viewInfo) {
            this._revealHorizontalRightPadding = conf.editor.viewInfo.revealHorizontalRightPadding;
        }
        if (e.canUseLayerHinting) {
            this._canUseLayerHinting = conf.editor.canUseLayerHinting;
        }
        if (e.fontInfo) {
            Configuration.applyFontInfo(this.domNode, conf.editor.fontInfo);
        }
        this._onOptionsMaybeChanged();
        if (e.layoutInfo) {
            this._maxLineWidth = 0;
        }
        return true;
    };
    ViewLines.prototype._onOptionsMaybeChanged = function () {
        var conf = this._context.configuration;
        var newViewLineOptions = new ViewLineOptions(conf, this._context.theme.type);
        if (!this._viewLineOptions.equals(newViewLineOptions)) {
            this._viewLineOptions = newViewLineOptions;
            var startLineNumber = this._visibleLines.getStartLineNumber();
            var endLineNumber = this._visibleLines.getEndLineNumber();
            for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
                var line = this._visibleLines.getVisibleLine(lineNumber);
                line.onOptionsChanged(this._viewLineOptions);
            }
            return true;
        }
        return false;
    };
    ViewLines.prototype.onCursorStateChanged = function (e) {
        var rendStartLineNumber = this._visibleLines.getStartLineNumber();
        var rendEndLineNumber = this._visibleLines.getEndLineNumber();
        var r = false;
        for (var lineNumber = rendStartLineNumber; lineNumber <= rendEndLineNumber; lineNumber++) {
            r = this._visibleLines.getVisibleLine(lineNumber).onSelectionChanged() || r;
        }
        return r;
    };
    ViewLines.prototype.onDecorationsChanged = function (e) {
        {
            var rendStartLineNumber = this._visibleLines.getStartLineNumber();
            var rendEndLineNumber = this._visibleLines.getEndLineNumber();
            for (var lineNumber = rendStartLineNumber; lineNumber <= rendEndLineNumber; lineNumber++) {
                this._visibleLines.getVisibleLine(lineNumber).onDecorationsChanged();
            }
        }
        return true;
    };
    ViewLines.prototype.onFlushed = function (e) {
        var shouldRender = this._visibleLines.onFlushed(e);
        this._maxLineWidth = 0;
        return shouldRender;
    };
    ViewLines.prototype.onLinesChanged = function (e) {
        return this._visibleLines.onLinesChanged(e);
    };
    ViewLines.prototype.onLinesDeleted = function (e) {
        return this._visibleLines.onLinesDeleted(e);
    };
    ViewLines.prototype.onLinesInserted = function (e) {
        return this._visibleLines.onLinesInserted(e);
    };
    ViewLines.prototype.onRevealRangeRequest = function (e) {
        // Using the future viewport here in order to handle multiple
        // incoming reveal range requests that might all desire to be animated
        var desiredScrollTop = this._computeScrollTopToRevealRange(this._context.viewLayout.getFutureViewport(), e.range, e.verticalType);
        // validate the new desired scroll top
        var newScrollPosition = this._context.viewLayout.validateScrollPosition({ scrollTop: desiredScrollTop });
        if (e.revealHorizontal) {
            if (e.range.startLineNumber !== e.range.endLineNumber) {
                // Two or more lines? => scroll to base (That's how you see most of the two lines)
                newScrollPosition = {
                    scrollTop: newScrollPosition.scrollTop,
                    scrollLeft: 0
                };
            }
            else {
                // We don't necessarily know the horizontal offset of this range since the line might not be in the view...
                this._horizontalRevealRequest = new HorizontalRevealRequest(e.range.startLineNumber, e.range.startColumn, e.range.endColumn, this._context.viewLayout.getCurrentScrollTop(), newScrollPosition.scrollTop, e.scrollType);
            }
        }
        else {
            this._horizontalRevealRequest = null;
        }
        var scrollTopDelta = Math.abs(this._context.viewLayout.getCurrentScrollTop() - newScrollPosition.scrollTop);
        if (e.scrollType === 0 /* Smooth */ && scrollTopDelta > this._lineHeight) {
            this._context.viewLayout.setScrollPositionSmooth(newScrollPosition);
        }
        else {
            this._context.viewLayout.setScrollPositionNow(newScrollPosition);
        }
        return true;
    };
    ViewLines.prototype.onScrollChanged = function (e) {
        if (this._horizontalRevealRequest && e.scrollLeftChanged) {
            // cancel any outstanding horizontal reveal request if someone else scrolls horizontally.
            this._horizontalRevealRequest = null;
        }
        if (this._horizontalRevealRequest && e.scrollTopChanged) {
            var min = Math.min(this._horizontalRevealRequest.startScrollTop, this._horizontalRevealRequest.stopScrollTop);
            var max = Math.max(this._horizontalRevealRequest.startScrollTop, this._horizontalRevealRequest.stopScrollTop);
            if (e.scrollTop < min || e.scrollTop > max) {
                // cancel any outstanding horizontal reveal request if someone else scrolls vertically.
                this._horizontalRevealRequest = null;
            }
        }
        this.domNode.setWidth(e.scrollWidth);
        return this._visibleLines.onScrollChanged(e) || true;
    };
    ViewLines.prototype.onTokensChanged = function (e) {
        return this._visibleLines.onTokensChanged(e);
    };
    ViewLines.prototype.onZonesChanged = function (e) {
        this._context.viewLayout.onMaxLineWidthChanged(this._maxLineWidth);
        return this._visibleLines.onZonesChanged(e);
    };
    ViewLines.prototype.onThemeChanged = function (e) {
        return this._onOptionsMaybeChanged();
    };
    // ---- end view event handlers
    // ----------- HELPERS FOR OTHERS
    ViewLines.prototype.getPositionFromDOMInfo = function (spanNode, offset) {
        var viewLineDomNode = this._getViewLineDomNode(spanNode);
        if (viewLineDomNode === null) {
            // Couldn't find view line node
            return null;
        }
        var lineNumber = this._getLineNumberFor(viewLineDomNode);
        if (lineNumber === -1) {
            // Couldn't find view line node
            return null;
        }
        if (lineNumber < 1 || lineNumber > this._context.model.getLineCount()) {
            // lineNumber is outside range
            return null;
        }
        if (this._context.model.getLineMaxColumn(lineNumber) === 1) {
            // Line is empty
            return new Position(lineNumber, 1);
        }
        var rendStartLineNumber = this._visibleLines.getStartLineNumber();
        var rendEndLineNumber = this._visibleLines.getEndLineNumber();
        if (lineNumber < rendStartLineNumber || lineNumber > rendEndLineNumber) {
            // Couldn't find line
            return null;
        }
        var column = this._visibleLines.getVisibleLine(lineNumber).getColumnOfNodeOffset(lineNumber, spanNode, offset);
        var minColumn = this._context.model.getLineMinColumn(lineNumber);
        if (column < minColumn) {
            column = minColumn;
        }
        return new Position(lineNumber, column);
    };
    ViewLines.prototype._getViewLineDomNode = function (node) {
        while (node && node.nodeType === 1) {
            if (node.className === ViewLine.CLASS_NAME) {
                return node;
            }
            node = node.parentElement;
        }
        return null;
    };
    /**
     * @returns the line number of this view line dom node.
     */
    ViewLines.prototype._getLineNumberFor = function (domNode) {
        var startLineNumber = this._visibleLines.getStartLineNumber();
        var endLineNumber = this._visibleLines.getEndLineNumber();
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            var line = this._visibleLines.getVisibleLine(lineNumber);
            if (domNode === line.getDomNode()) {
                return lineNumber;
            }
        }
        return -1;
    };
    ViewLines.prototype.getLineWidth = function (lineNumber) {
        var rendStartLineNumber = this._visibleLines.getStartLineNumber();
        var rendEndLineNumber = this._visibleLines.getEndLineNumber();
        if (lineNumber < rendStartLineNumber || lineNumber > rendEndLineNumber) {
            // Couldn't find line
            return -1;
        }
        return this._visibleLines.getVisibleLine(lineNumber).getWidth();
    };
    ViewLines.prototype.linesVisibleRangesForRange = function (_range, includeNewLines) {
        if (this.shouldRender()) {
            // Cannot read from the DOM because it is dirty
            // i.e. the model & the dom are out of sync, so I'd be reading something stale
            return null;
        }
        var originalEndLineNumber = _range.endLineNumber;
        var range = Range.intersectRanges(_range, this._lastRenderedData.getCurrentVisibleRange());
        if (!range) {
            return null;
        }
        var visibleRanges = [], visibleRangesLen = 0;
        var domReadingContext = new DomReadingContext(this.domNode.domNode, this._textRangeRestingSpot);
        var nextLineModelLineNumber = 0;
        if (includeNewLines) {
            nextLineModelLineNumber = this._context.model.coordinatesConverter.convertViewPositionToModelPosition(new Position(range.startLineNumber, 1)).lineNumber;
        }
        var rendStartLineNumber = this._visibleLines.getStartLineNumber();
        var rendEndLineNumber = this._visibleLines.getEndLineNumber();
        for (var lineNumber = range.startLineNumber; lineNumber <= range.endLineNumber; lineNumber++) {
            if (lineNumber < rendStartLineNumber || lineNumber > rendEndLineNumber) {
                continue;
            }
            var startColumn = lineNumber === range.startLineNumber ? range.startColumn : 1;
            var endColumn = lineNumber === range.endLineNumber ? range.endColumn : this._context.model.getLineMaxColumn(lineNumber);
            var visibleRangesForLine = this._visibleLines.getVisibleLine(lineNumber).getVisibleRangesForRange(startColumn, endColumn, domReadingContext);
            if (!visibleRangesForLine || visibleRangesForLine.length === 0) {
                continue;
            }
            if (includeNewLines && lineNumber < originalEndLineNumber) {
                var currentLineModelLineNumber = nextLineModelLineNumber;
                nextLineModelLineNumber = this._context.model.coordinatesConverter.convertViewPositionToModelPosition(new Position(lineNumber + 1, 1)).lineNumber;
                if (currentLineModelLineNumber !== nextLineModelLineNumber) {
                    visibleRangesForLine[visibleRangesForLine.length - 1].width += this._typicalHalfwidthCharacterWidth;
                }
            }
            visibleRanges[visibleRangesLen++] = new LineVisibleRanges(lineNumber, visibleRangesForLine);
        }
        if (visibleRangesLen === 0) {
            return null;
        }
        return visibleRanges;
    };
    ViewLines.prototype.visibleRangesForRange2 = function (_range) {
        if (this.shouldRender()) {
            // Cannot read from the DOM because it is dirty
            // i.e. the model & the dom are out of sync, so I'd be reading something stale
            return null;
        }
        var range = Range.intersectRanges(_range, this._lastRenderedData.getCurrentVisibleRange());
        if (!range) {
            return null;
        }
        var result = [];
        var domReadingContext = new DomReadingContext(this.domNode.domNode, this._textRangeRestingSpot);
        var rendStartLineNumber = this._visibleLines.getStartLineNumber();
        var rendEndLineNumber = this._visibleLines.getEndLineNumber();
        for (var lineNumber = range.startLineNumber; lineNumber <= range.endLineNumber; lineNumber++) {
            if (lineNumber < rendStartLineNumber || lineNumber > rendEndLineNumber) {
                continue;
            }
            var startColumn = lineNumber === range.startLineNumber ? range.startColumn : 1;
            var endColumn = lineNumber === range.endLineNumber ? range.endColumn : this._context.model.getLineMaxColumn(lineNumber);
            var visibleRangesForLine = this._visibleLines.getVisibleLine(lineNumber).getVisibleRangesForRange(startColumn, endColumn, domReadingContext);
            if (!visibleRangesForLine || visibleRangesForLine.length === 0) {
                continue;
            }
            result = result.concat(visibleRangesForLine);
        }
        if (result.length === 0) {
            return null;
        }
        return result;
    };
    ViewLines.prototype.visibleRangeForPosition = function (position) {
        var visibleRanges = this.visibleRangesForRange2(new Range(position.lineNumber, position.column, position.lineNumber, position.column));
        if (!visibleRanges) {
            return null;
        }
        return visibleRanges[0];
    };
    // --- implementation
    ViewLines.prototype.updateLineWidths = function () {
        this._updateLineWidths(false);
    };
    /**
     * Updates the max line width if it is fast to compute.
     * Returns true if all lines were taken into account.
     * Returns false if some lines need to be reevaluated (in a slow fashion).
     */
    ViewLines.prototype._updateLineWidthsFast = function () {
        return this._updateLineWidths(true);
    };
    ViewLines.prototype._updateLineWidthsSlow = function () {
        this._updateLineWidths(false);
    };
    ViewLines.prototype._updateLineWidths = function (fast) {
        var rendStartLineNumber = this._visibleLines.getStartLineNumber();
        var rendEndLineNumber = this._visibleLines.getEndLineNumber();
        var localMaxLineWidth = 1;
        var allWidthsComputed = true;
        for (var lineNumber = rendStartLineNumber; lineNumber <= rendEndLineNumber; lineNumber++) {
            var visibleLine = this._visibleLines.getVisibleLine(lineNumber);
            if (fast && !visibleLine.getWidthIsFast()) {
                // Cannot compute width in a fast way for this line
                allWidthsComputed = false;
                continue;
            }
            localMaxLineWidth = Math.max(localMaxLineWidth, visibleLine.getWidth());
        }
        if (allWidthsComputed && rendStartLineNumber === 1 && rendEndLineNumber === this._context.model.getLineCount()) {
            // we know the max line width for all the lines
            this._maxLineWidth = 0;
        }
        this._ensureMaxLineWidth(localMaxLineWidth);
        return allWidthsComputed;
    };
    ViewLines.prototype.prepareRender = function () {
        throw new Error('Not supported');
    };
    ViewLines.prototype.render = function () {
        throw new Error('Not supported');
    };
    ViewLines.prototype.renderText = function (viewportData) {
        // (1) render lines - ensures lines are in the DOM
        this._visibleLines.renderLines(viewportData);
        this._lastRenderedData.setCurrentVisibleRange(viewportData.visibleRange);
        this.domNode.setWidth(this._context.viewLayout.getScrollWidth());
        this.domNode.setHeight(Math.min(this._context.viewLayout.getScrollHeight(), 1000000));
        // (2) compute horizontal scroll position:
        //  - this must happen after the lines are in the DOM since it might need a line that rendered just now
        //  - it might change `scrollWidth` and `scrollLeft`
        if (this._horizontalRevealRequest) {
            var revealLineNumber = this._horizontalRevealRequest.lineNumber;
            var revealStartColumn = this._horizontalRevealRequest.startColumn;
            var revealEndColumn = this._horizontalRevealRequest.endColumn;
            var scrollType = this._horizontalRevealRequest.scrollType;
            // Check that we have the line that contains the horizontal range in the viewport
            if (viewportData.startLineNumber <= revealLineNumber && revealLineNumber <= viewportData.endLineNumber) {
                this._horizontalRevealRequest = null;
                // allow `visibleRangesForRange2` to work
                this.onDidRender();
                // compute new scroll position
                var newScrollLeft = this._computeScrollLeftToRevealRange(revealLineNumber, revealStartColumn, revealEndColumn);
                var isViewportWrapping = this._isViewportWrapping;
                if (!isViewportWrapping) {
                    // ensure `scrollWidth` is large enough
                    this._ensureMaxLineWidth(newScrollLeft.maxHorizontalOffset);
                }
                // set `scrollLeft`
                if (scrollType === 0 /* Smooth */) {
                    this._context.viewLayout.setScrollPositionSmooth({
                        scrollLeft: newScrollLeft.scrollLeft
                    });
                }
                else {
                    this._context.viewLayout.setScrollPositionNow({
                        scrollLeft: newScrollLeft.scrollLeft
                    });
                }
            }
        }
        // Update max line width (not so important, it is just so the horizontal scrollbar doesn't get too small)
        if (!this._updateLineWidthsFast()) {
            // Computing the width of some lines would be slow => delay it
            this._asyncUpdateLineWidths.schedule();
        }
        // (3) handle scrolling
        this._linesContent.setLayerHinting(this._canUseLayerHinting);
        var adjustedScrollTop = this._context.viewLayout.getCurrentScrollTop() - viewportData.bigNumbersDelta;
        this._linesContent.setTop(-adjustedScrollTop);
        this._linesContent.setLeft(-this._context.viewLayout.getCurrentScrollLeft());
    };
    // --- width
    ViewLines.prototype._ensureMaxLineWidth = function (lineWidth) {
        var iLineWidth = Math.ceil(lineWidth);
        if (this._maxLineWidth < iLineWidth) {
            this._maxLineWidth = iLineWidth;
            this._context.viewLayout.onMaxLineWidthChanged(this._maxLineWidth);
        }
    };
    ViewLines.prototype._computeScrollTopToRevealRange = function (viewport, range, verticalType) {
        var viewportStartY = viewport.top;
        var viewportHeight = viewport.height;
        var viewportEndY = viewportStartY + viewportHeight;
        var boxStartY;
        var boxEndY;
        // Have a box that includes one extra line height (for the horizontal scrollbar)
        boxStartY = this._context.viewLayout.getVerticalOffsetForLineNumber(range.startLineNumber);
        boxEndY = this._context.viewLayout.getVerticalOffsetForLineNumber(range.endLineNumber) + this._lineHeight;
        if (verticalType === 0 /* Simple */ || verticalType === 4 /* Bottom */) {
            // Reveal one line more when the last line would be covered by the scrollbar - arrow down case or revealing a line explicitly at bottom
            boxEndY += this._lineHeight;
        }
        var newScrollTop;
        if (verticalType === 1 /* Center */ || verticalType === 2 /* CenterIfOutsideViewport */) {
            if (verticalType === 2 /* CenterIfOutsideViewport */ && viewportStartY <= boxStartY && boxEndY <= viewportEndY) {
                // Box is already in the viewport... do nothing
                newScrollTop = viewportStartY;
            }
            else {
                // Box is outside the viewport... center it
                var boxMiddleY = (boxStartY + boxEndY) / 2;
                newScrollTop = Math.max(0, boxMiddleY - viewportHeight / 2);
            }
        }
        else {
            newScrollTop = this._computeMinimumScrolling(viewportStartY, viewportEndY, boxStartY, boxEndY, verticalType === 3 /* Top */, verticalType === 4 /* Bottom */);
        }
        return newScrollTop;
    };
    ViewLines.prototype._computeScrollLeftToRevealRange = function (lineNumber, startColumn, endColumn) {
        var maxHorizontalOffset = 0;
        var viewport = this._context.viewLayout.getCurrentViewport();
        var viewportStartX = viewport.left;
        var viewportEndX = viewportStartX + viewport.width;
        var visibleRanges = this.visibleRangesForRange2(new Range(lineNumber, startColumn, lineNumber, endColumn));
        var boxStartX = Number.MAX_VALUE;
        var boxEndX = 0;
        if (!visibleRanges) {
            // Unknown
            return {
                scrollLeft: viewportStartX,
                maxHorizontalOffset: maxHorizontalOffset
            };
        }
        for (var i = 0; i < visibleRanges.length; i++) {
            var visibleRange = visibleRanges[i];
            if (visibleRange.left < boxStartX) {
                boxStartX = visibleRange.left;
            }
            if (visibleRange.left + visibleRange.width > boxEndX) {
                boxEndX = visibleRange.left + visibleRange.width;
            }
        }
        maxHorizontalOffset = boxEndX;
        boxStartX = Math.max(0, boxStartX - ViewLines.HORIZONTAL_EXTRA_PX);
        boxEndX += this._revealHorizontalRightPadding;
        var newScrollLeft = this._computeMinimumScrolling(viewportStartX, viewportEndX, boxStartX, boxEndX);
        return {
            scrollLeft: newScrollLeft,
            maxHorizontalOffset: maxHorizontalOffset
        };
    };
    ViewLines.prototype._computeMinimumScrolling = function (viewportStart, viewportEnd, boxStart, boxEnd, revealAtStart, revealAtEnd) {
        viewportStart = viewportStart | 0;
        viewportEnd = viewportEnd | 0;
        boxStart = boxStart | 0;
        boxEnd = boxEnd | 0;
        revealAtStart = !!revealAtStart;
        revealAtEnd = !!revealAtEnd;
        var viewportLength = viewportEnd - viewportStart;
        var boxLength = boxEnd - boxStart;
        if (boxLength < viewportLength) {
            // The box would fit in the viewport
            if (revealAtStart) {
                return boxStart;
            }
            if (revealAtEnd) {
                return Math.max(0, boxEnd - viewportLength);
            }
            if (boxStart < viewportStart) {
                // The box is above the viewport
                return boxStart;
            }
            else if (boxEnd > viewportEnd) {
                // The box is below the viewport
                return Math.max(0, boxEnd - viewportLength);
            }
        }
        else {
            // The box would not fit in the viewport
            // Reveal the beginning of the box
            return boxStart;
        }
        return viewportStart;
    };
    /**
     * Adds this amount of pixels to the right of lines (no-one wants to type near the edge of the viewport)
     */
    ViewLines.HORIZONTAL_EXTRA_PX = 30;
    return ViewLines;
}(ViewPart));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$Q = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var LinesDecorationsOverlay = /** @class */ (function (_super) {
    __extends$Q(LinesDecorationsOverlay, _super);
    function LinesDecorationsOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._decorationsLeft = _this._context.configuration.editor.layoutInfo.decorationsLeft;
        _this._decorationsWidth = _this._context.configuration.editor.layoutInfo.decorationsWidth;
        _this._renderResult = null;
        _this._context.addEventHandler(_this);
        return _this;
    }
    LinesDecorationsOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        this._renderResult = null;
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    LinesDecorationsOverlay.prototype.onConfigurationChanged = function (e) {
        if (e.layoutInfo) {
            this._decorationsLeft = this._context.configuration.editor.layoutInfo.decorationsLeft;
            this._decorationsWidth = this._context.configuration.editor.layoutInfo.decorationsWidth;
        }
        return true;
    };
    LinesDecorationsOverlay.prototype.onDecorationsChanged = function (e) {
        return true;
    };
    LinesDecorationsOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    LinesDecorationsOverlay.prototype.onLinesChanged = function (e) {
        return true;
    };
    LinesDecorationsOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    LinesDecorationsOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    LinesDecorationsOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged;
    };
    LinesDecorationsOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    LinesDecorationsOverlay.prototype._getDecorations = function (ctx) {
        var decorations = ctx.getDecorationsInViewport();
        var r = [], rLen = 0;
        for (var i = 0, len = decorations.length; i < len; i++) {
            var d = decorations[i];
            var linesDecorationsClassName = d.options.linesDecorationsClassName;
            if (linesDecorationsClassName) {
                r[rLen++] = new DecorationToRender(d.range.startLineNumber, d.range.endLineNumber, linesDecorationsClassName);
            }
        }
        return r;
    };
    LinesDecorationsOverlay.prototype.prepareRender = function (ctx) {
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        var toRender = this._render(visibleStartLineNumber, visibleEndLineNumber, this._getDecorations(ctx));
        var left = this._decorationsLeft.toString();
        var width = this._decorationsWidth.toString();
        var common = '" style="left:' + left + 'px;width:' + width + 'px;"></div>';
        var output = [];
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var lineIndex = lineNumber - visibleStartLineNumber;
            var classNames = toRender[lineIndex];
            var lineOutput = '';
            for (var i = 0, len = classNames.length; i < len; i++) {
                lineOutput += '<div class="cldr ' + classNames[i] + common;
            }
            output[lineIndex] = lineOutput;
        }
        this._renderResult = output;
    };
    LinesDecorationsOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (!this._renderResult) {
            return '';
        }
        return this._renderResult[lineNumber - startLineNumber];
    };
    return LinesDecorationsOverlay;
}(DedupOverlay));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$R = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var MarginViewLineDecorationsOverlay = /** @class */ (function (_super) {
    __extends$R(MarginViewLineDecorationsOverlay, _super);
    function MarginViewLineDecorationsOverlay(context) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._renderResult = null;
        _this._context.addEventHandler(_this);
        return _this;
    }
    MarginViewLineDecorationsOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        this._renderResult = null;
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    MarginViewLineDecorationsOverlay.prototype.onConfigurationChanged = function (e) {
        return true;
    };
    MarginViewLineDecorationsOverlay.prototype.onDecorationsChanged = function (e) {
        return true;
    };
    MarginViewLineDecorationsOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    MarginViewLineDecorationsOverlay.prototype.onLinesChanged = function (e) {
        return true;
    };
    MarginViewLineDecorationsOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    MarginViewLineDecorationsOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    MarginViewLineDecorationsOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged;
    };
    MarginViewLineDecorationsOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    MarginViewLineDecorationsOverlay.prototype._getDecorations = function (ctx) {
        var decorations = ctx.getDecorationsInViewport();
        var r = [], rLen = 0;
        for (var i = 0, len = decorations.length; i < len; i++) {
            var d = decorations[i];
            var marginClassName = d.options.marginClassName;
            if (marginClassName) {
                r[rLen++] = new DecorationToRender(d.range.startLineNumber, d.range.endLineNumber, marginClassName);
            }
        }
        return r;
    };
    MarginViewLineDecorationsOverlay.prototype.prepareRender = function (ctx) {
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        var toRender = this._render(visibleStartLineNumber, visibleEndLineNumber, this._getDecorations(ctx));
        var output = [];
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var lineIndex = lineNumber - visibleStartLineNumber;
            var classNames = toRender[lineIndex];
            var lineOutput = '';
            for (var i = 0, len = classNames.length; i < len; i++) {
                lineOutput += '<div class="cmdr ' + classNames[i] + '" style=""></div>';
            }
            output[lineIndex] = lineOutput;
        }
        this._renderResult = output;
    };
    MarginViewLineDecorationsOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (!this._renderResult) {
            return '';
        }
        return this._renderResult[lineNumber - startLineNumber];
    };
    return MarginViewLineDecorationsOverlay;
}(DedupOverlay));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * A very VM friendly rgba datastructure.
 * Please don't touch unless you take a look at the IR.
 */
var RGBA8 = /** @class */ (function () {
    function RGBA8(r, g, b, a) {
        this.r = RGBA8._clamp(r);
        this.g = RGBA8._clamp(g);
        this.b = RGBA8._clamp(b);
        this.a = RGBA8._clamp(a);
    }
    RGBA8._clamp = function (c) {
        if (c < 0) {
            return 0;
        }
        if (c > 255) {
            return 255;
        }
        return c | 0;
    };
    RGBA8.Empty = new RGBA8(0, 0, 0, 0);
    return RGBA8;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MinimapTokensColorTracker = /** @class */ (function () {
    function MinimapTokensColorTracker() {
        var _this = this;
        this._onDidChange = new Emitter();
        this.onDidChange = this._onDidChange.event;
        this._updateColorMap();
        TokenizationRegistry.onDidChange(function (e) {
            if (e.changedColorMap) {
                _this._updateColorMap();
            }
        });
    }
    MinimapTokensColorTracker.getInstance = function () {
        if (!this._INSTANCE) {
            this._INSTANCE = new MinimapTokensColorTracker();
        }
        return this._INSTANCE;
    };
    MinimapTokensColorTracker.prototype._updateColorMap = function () {
        var colorMap = TokenizationRegistry.getColorMap();
        if (!colorMap) {
            this._colors = [RGBA8.Empty];
            this._backgroundIsLight = true;
            return;
        }
        this._colors = [RGBA8.Empty];
        for (var colorId = 1; colorId < colorMap.length; colorId++) {
            var source = colorMap[colorId].rgba;
            // Use a VM friendly data-type
            this._colors[colorId] = new RGBA8(source.r, source.g, source.b, Math.round(source.a * 255));
        }
        var backgroundLuminosity = colorMap[2 /* DefaultBackground */].getRelativeLuminance();
        this._backgroundIsLight = (backgroundLuminosity >= 0.5);
        this._onDidChange.fire(void 0);
    };
    MinimapTokensColorTracker.prototype.getColor = function (colorId) {
        if (colorId < 1 || colorId >= this._colors.length) {
            // background color (basically invisible)
            colorId = 2 /* DefaultBackground */;
        }
        return this._colors[colorId];
    };
    MinimapTokensColorTracker.prototype.backgroundIsLight = function () {
        return this._backgroundIsLight;
    };
    MinimapTokensColorTracker._INSTANCE = null;
    return MinimapTokensColorTracker;
}());
var MinimapCharRenderer = /** @class */ (function () {
    function MinimapCharRenderer(x2CharData, x1CharData) {
        var x2ExpectedLen = 4 /* x2_CHAR_HEIGHT */ * 2 /* x2_CHAR_WIDTH */ * 95 /* CHAR_COUNT */;
        if (x2CharData.length !== x2ExpectedLen) {
            throw new Error('Invalid x2CharData');
        }
        var x1ExpectedLen = 2 /* x1_CHAR_HEIGHT */ * 1 /* x1_CHAR_WIDTH */ * 95 /* CHAR_COUNT */;
        if (x1CharData.length !== x1ExpectedLen) {
            throw new Error('Invalid x1CharData');
        }
        this.x2charData = x2CharData;
        this.x1charData = x1CharData;
        this.x2charDataLight = MinimapCharRenderer.soften(x2CharData, 12 / 15);
        this.x1charDataLight = MinimapCharRenderer.soften(x1CharData, 50 / 60);
    }
    MinimapCharRenderer.soften = function (input, ratio) {
        var result = new Uint8ClampedArray(input.length);
        for (var i = 0, len = input.length; i < len; i++) {
            result[i] = input[i] * ratio;
        }
        return result;
    };
    MinimapCharRenderer._getChIndex = function (chCode) {
        chCode -= 32 /* START_CH_CODE */;
        if (chCode < 0) {
            chCode += 95 /* CHAR_COUNT */;
        }
        return (chCode % 95 /* CHAR_COUNT */);
    };
    MinimapCharRenderer.prototype.x2RenderChar = function (target, dx, dy, chCode, color, backgroundColor, useLighterFont) {
        if (dx + 2 /* x2_CHAR_WIDTH */ > target.width || dy + 4 /* x2_CHAR_HEIGHT */ > target.height) {
            console.warn('bad render request outside image data');
            return;
        }
        var x2CharData = useLighterFont ? this.x2charDataLight : this.x2charData;
        var chIndex = MinimapCharRenderer._getChIndex(chCode);
        var outWidth = target.width * 4 /* RGBA_CHANNELS_CNT */;
        var backgroundR = backgroundColor.r;
        var backgroundG = backgroundColor.g;
        var backgroundB = backgroundColor.b;
        var deltaR = color.r - backgroundR;
        var deltaG = color.g - backgroundG;
        var deltaB = color.b - backgroundB;
        var dest = target.data;
        var sourceOffset = chIndex * 4 /* x2_CHAR_HEIGHT */ * 2 /* x2_CHAR_WIDTH */;
        var destOffset = dy * outWidth + dx * 4 /* RGBA_CHANNELS_CNT */;
        {
            var c = x2CharData[sourceOffset] / 255;
            dest[destOffset + 0] = backgroundR + deltaR * c;
            dest[destOffset + 1] = backgroundG + deltaG * c;
            dest[destOffset + 2] = backgroundB + deltaB * c;
        }
        {
            var c = x2CharData[sourceOffset + 1] / 255;
            dest[destOffset + 4] = backgroundR + deltaR * c;
            dest[destOffset + 5] = backgroundG + deltaG * c;
            dest[destOffset + 6] = backgroundB + deltaB * c;
        }
        destOffset += outWidth;
        {
            var c = x2CharData[sourceOffset + 2] / 255;
            dest[destOffset + 0] = backgroundR + deltaR * c;
            dest[destOffset + 1] = backgroundG + deltaG * c;
            dest[destOffset + 2] = backgroundB + deltaB * c;
        }
        {
            var c = x2CharData[sourceOffset + 3] / 255;
            dest[destOffset + 4] = backgroundR + deltaR * c;
            dest[destOffset + 5] = backgroundG + deltaG * c;
            dest[destOffset + 6] = backgroundB + deltaB * c;
        }
        destOffset += outWidth;
        {
            var c = x2CharData[sourceOffset + 4] / 255;
            dest[destOffset + 0] = backgroundR + deltaR * c;
            dest[destOffset + 1] = backgroundG + deltaG * c;
            dest[destOffset + 2] = backgroundB + deltaB * c;
        }
        {
            var c = x2CharData[sourceOffset + 5] / 255;
            dest[destOffset + 4] = backgroundR + deltaR * c;
            dest[destOffset + 5] = backgroundG + deltaG * c;
            dest[destOffset + 6] = backgroundB + deltaB * c;
        }
        destOffset += outWidth;
        {
            var c = x2CharData[sourceOffset + 6] / 255;
            dest[destOffset + 0] = backgroundR + deltaR * c;
            dest[destOffset + 1] = backgroundG + deltaG * c;
            dest[destOffset + 2] = backgroundB + deltaB * c;
        }
        {
            var c = x2CharData[sourceOffset + 7] / 255;
            dest[destOffset + 4] = backgroundR + deltaR * c;
            dest[destOffset + 5] = backgroundG + deltaG * c;
            dest[destOffset + 6] = backgroundB + deltaB * c;
        }
    };
    MinimapCharRenderer.prototype.x1RenderChar = function (target, dx, dy, chCode, color, backgroundColor, useLighterFont) {
        if (dx + 1 /* x1_CHAR_WIDTH */ > target.width || dy + 2 /* x1_CHAR_HEIGHT */ > target.height) {
            console.warn('bad render request outside image data');
            return;
        }
        var x1CharData = useLighterFont ? this.x1charDataLight : this.x1charData;
        var chIndex = MinimapCharRenderer._getChIndex(chCode);
        var outWidth = target.width * 4 /* RGBA_CHANNELS_CNT */;
        var backgroundR = backgroundColor.r;
        var backgroundG = backgroundColor.g;
        var backgroundB = backgroundColor.b;
        var deltaR = color.r - backgroundR;
        var deltaG = color.g - backgroundG;
        var deltaB = color.b - backgroundB;
        var dest = target.data;
        var sourceOffset = chIndex * 2 /* x1_CHAR_HEIGHT */ * 1 /* x1_CHAR_WIDTH */;
        var destOffset = dy * outWidth + dx * 4 /* RGBA_CHANNELS_CNT */;
        {
            var c = x1CharData[sourceOffset] / 255;
            dest[destOffset + 0] = backgroundR + deltaR * c;
            dest[destOffset + 1] = backgroundG + deltaG * c;
            dest[destOffset + 2] = backgroundB + deltaB * c;
        }
        destOffset += outWidth;
        {
            var c = x1CharData[sourceOffset + 1] / 255;
            dest[destOffset + 0] = backgroundR + deltaR * c;
            dest[destOffset + 1] = backgroundG + deltaG * c;
            dest[destOffset + 2] = backgroundB + deltaB * c;
        }
    };
    MinimapCharRenderer.prototype.x2BlockRenderChar = function (target, dx, dy, color, backgroundColor, useLighterFont) {
        if (dx + 2 /* x2_CHAR_WIDTH */ > target.width || dy + 4 /* x2_CHAR_HEIGHT */ > target.height) {
            console.warn('bad render request outside image data');
            return;
        }
        var outWidth = target.width * 4 /* RGBA_CHANNELS_CNT */;
        var c = 0.5;
        var backgroundR = backgroundColor.r;
        var backgroundG = backgroundColor.g;
        var backgroundB = backgroundColor.b;
        var deltaR = color.r - backgroundR;
        var deltaG = color.g - backgroundG;
        var deltaB = color.b - backgroundB;
        var colorR = backgroundR + deltaR * c;
        var colorG = backgroundG + deltaG * c;
        var colorB = backgroundB + deltaB * c;
        var dest = target.data;
        var destOffset = dy * outWidth + dx * 4 /* RGBA_CHANNELS_CNT */;
        {
            dest[destOffset + 0] = colorR;
            dest[destOffset + 1] = colorG;
            dest[destOffset + 2] = colorB;
        }
        {
            dest[destOffset + 4] = colorR;
            dest[destOffset + 5] = colorG;
            dest[destOffset + 6] = colorB;
        }
        destOffset += outWidth;
        {
            dest[destOffset + 0] = colorR;
            dest[destOffset + 1] = colorG;
            dest[destOffset + 2] = colorB;
        }
        {
            dest[destOffset + 4] = colorR;
            dest[destOffset + 5] = colorG;
            dest[destOffset + 6] = colorB;
        }
        destOffset += outWidth;
        {
            dest[destOffset + 0] = colorR;
            dest[destOffset + 1] = colorG;
            dest[destOffset + 2] = colorB;
        }
        {
            dest[destOffset + 4] = colorR;
            dest[destOffset + 5] = colorG;
            dest[destOffset + 6] = colorB;
        }
        destOffset += outWidth;
        {
            dest[destOffset + 0] = colorR;
            dest[destOffset + 1] = colorG;
            dest[destOffset + 2] = colorB;
        }
        {
            dest[destOffset + 4] = colorR;
            dest[destOffset + 5] = colorG;
            dest[destOffset + 6] = colorB;
        }
    };
    MinimapCharRenderer.prototype.x1BlockRenderChar = function (target, dx, dy, color, backgroundColor, useLighterFont) {
        if (dx + 1 /* x1_CHAR_WIDTH */ > target.width || dy + 2 /* x1_CHAR_HEIGHT */ > target.height) {
            console.warn('bad render request outside image data');
            return;
        }
        var outWidth = target.width * 4 /* RGBA_CHANNELS_CNT */;
        var c = 0.5;
        var backgroundR = backgroundColor.r;
        var backgroundG = backgroundColor.g;
        var backgroundB = backgroundColor.b;
        var deltaR = color.r - backgroundR;
        var deltaG = color.g - backgroundG;
        var deltaB = color.b - backgroundB;
        var colorR = backgroundR + deltaR * c;
        var colorG = backgroundG + deltaG * c;
        var colorB = backgroundB + deltaB * c;
        var dest = target.data;
        var destOffset = dy * outWidth + dx * 4 /* RGBA_CHANNELS_CNT */;
        {
            dest[destOffset + 0] = colorR;
            dest[destOffset + 1] = colorG;
            dest[destOffset + 2] = colorB;
        }
        destOffset += outWidth;
        {
            dest[destOffset + 0] = colorR;
            dest[destOffset + 1] = colorG;
            dest[destOffset + 2] = colorB;
        }
    };
    return MinimapCharRenderer;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function toUint8ClampedArrat(arr) {
    var r = new Uint8ClampedArray(arr.length);
    for (var i = 0, len = arr.length; i < len; i++) {
        r[i] = arr[i];
    }
    return r;
}
var minimapCharRenderer = null;
function getOrCreateMinimapCharRenderer() {
    if (!minimapCharRenderer) {
        var _x1Data = toUint8ClampedArrat(x1Data);
        x1Data = null;
        var _x2Data = toUint8ClampedArrat(x2Data);
        x2Data = null;
        minimapCharRenderer = new MinimapCharRenderer(_x2Data, _x1Data);
    }
    return minimapCharRenderer;
}
var x2Data = [
    //
    0, 0,
    0, 0,
    0, 0,
    0, 0,
    // !
    39, 14,
    39, 14,
    14, 5,
    29, 10,
    // "
    96, 96,
    29, 29,
    0, 0,
    0, 0,
    // #
    49, 113,
    195, 214,
    227, 166,
    135, 42,
    // $
    40, 29,
    194, 38,
    75, 148,
    197, 187,
    // %
    145, 0,
    160, 61,
    75, 143,
    2, 183,
    // &
    138, 58,
    163, 6,
    177, 223,
    197, 227,
    // '
    38, 13,
    11, 4,
    0, 0,
    0, 0,
    // (
    10, 54,
    52, 8,
    62, 4,
    71, 122,
    // )
    73, 2,
    19, 40,
    10, 50,
    155, 36,
    // *
    79, 70,
    145, 121,
    7, 5,
    0, 0,
    // +
    2, 1,
    36, 12,
    204, 166,
    16, 5,
    // ,
    0, 0,
    0, 0,
    1, 0,
    154, 34,
    // -
    0, 0,
    0, 0,
    96, 83,
    0, 0,
    // .
    0, 0,
    0, 0,
    0, 0,
    46, 34,
    // /
    0, 82,
    2, 56,
    53, 3,
    146, 0,
    // 0
    146, 119,
    152, 132,
    152, 131,
    145, 119,
    // 1
    170, 42,
    15, 42,
    15, 42,
    172, 194,
    // 2
    131, 132,
    0, 139,
    80, 28,
    227, 143,
    // 3
    159, 135,
    15, 118,
    11, 126,
    171, 144,
    // 4
    20, 124,
    88, 106,
    217, 196,
    0, 106,
    // 5
    189, 92,
    168, 43,
    5, 130,
    164, 133,
    // 6
    130, 115,
    183, 65,
    134, 120,
    141, 141,
    // 7
    170, 196,
    2, 106,
    31, 32,
    105, 2,
    // 8
    145, 130,
    116, 114,
    132, 135,
    138, 140,
    // 9
    138, 113,
    147, 137,
    81, 183,
    129, 94,
    // :
    0, 0,
    21, 16,
    4, 3,
    46, 34,
    // ;
    0, 0,
    45, 34,
    1, 0,
    160, 49,
    // <
    0, 0,
    43, 143,
    203, 23,
    1, 76,
    // =
    0, 0,
    38, 28,
    131, 96,
    38, 28,
    // >
    0, 0,
    168, 31,
    29, 191,
    98, 0,
    // ?
    118, 139,
    5, 113,
    45, 13,
    37, 6,
    // @
    97, 115,
    161, 179,
    204, 105,
    223, 224,
    // A
    83, 52,
    111, 100,
    184, 186,
    120, 132,
    // B
    212, 145,
    180, 139,
    174, 161,
    212, 182,
    // C
    104, 162,
    131, 0,
    131, 0,
    104, 161,
    // D
    219, 120,
    110, 116,
    110, 116,
    219, 120,
    // E
    207, 154,
    163, 40,
    147, 22,
    207, 154,
    // F
    202, 159,
    161, 47,
    145, 23,
    111, 0,
    // G
    139, 154,
    144, 30,
    144, 135,
    139, 187,
    // H
    110, 110,
    168, 161,
    150, 145,
    110, 110,
    // I
    185, 162,
    43, 16,
    43, 16,
    185, 162,
    // J
    73, 129,
    0, 110,
    0, 110,
    191, 87,
    // K
    149, 149,
    236, 48,
    195, 91,
    146, 149,
    // L
    146, 0,
    146, 0,
    146, 0,
    187, 173,
    // M
    200, 201,
    222, 215,
    172, 147,
    95, 95,
    // N
    193, 97,
    224, 129,
    159, 206,
    97, 192,
    // O
    155, 139,
    153, 115,
    153, 115,
    156, 140,
    // P
    189, 158,
    123, 136,
    190, 64,
    111, 0,
    // Q
    155, 139,
    153, 115,
    153, 114,
    156, 241,
    // R
    197, 148,
    150, 152,
    170, 116,
    110, 157,
    // S
    156, 128,
    169, 14,
    13, 159,
    158, 149,
    // T
    212, 189,
    43, 16,
    43, 16,
    43, 16,
    // U
    148, 110,
    148, 110,
    147, 109,
    182, 151,
    // V
    133, 121,
    106, 118,
    114, 103,
    89, 66,
    // W
    94, 94,
    211, 188,
    205, 207,
    139, 168,
    // X
    151, 152,
    87, 76,
    101, 79,
    151, 152,
    // Y
    130, 156,
    125, 116,
    47, 29,
    43, 16,
    // Z
    169, 228,
    11, 103,
    120, 6,
    230, 176,
    // [
    55, 49,
    55, 6,
    55, 6,
    193, 102,
    // \
    92, 0,
    71, 0,
    13, 30,
    0, 147,
    // ]
    63, 43,
    12, 43,
    12, 43,
    142, 152,
    // ^
    71, 53,
    61, 61,
    0, 0,
    0, 0,
    // _
    0, 0,
    0, 0,
    0, 0,
    158, 146,
    // `
    25, 2,
    0, 0,
    0, 0,
    0, 0,
    // a
    0, 0,
    107, 130,
    170, 194,
    176, 188,
    // b
    109, 0,
    203, 159,
    113, 111,
    202, 158,
    // c
    0, 0,
    135, 135,
    114, 0,
    136, 135,
    // d
    0, 109,
    187, 190,
    148, 126,
    177, 187,
    // e
    0, 0,
    149, 130,
    218, 105,
    169, 135,
    // f
    37, 113,
    146, 113,
    49, 13,
    49, 13,
    // g
    0, 0,
    178, 195,
    147, 114,
    255, 255,
    // h
    109, 0,
    193, 149,
    110, 109,
    109, 109,
    // i
    12, 15,
    125, 41,
    33, 41,
    144, 188,
    // j
    1, 6,
    75, 53,
    10, 53,
    210, 161,
    // k
    110, 0,
    152, 148,
    210, 60,
    110, 156,
    // l
    213, 5,
    63, 5,
    63, 5,
    45, 111,
    // m
    0, 0,
    232, 172,
    190, 168,
    190, 169,
    // n
    0, 0,
    190, 144,
    109, 109,
    109, 109,
    // o
    0, 0,
    168, 140,
    148, 111,
    168, 140,
    // p
    0, 0,
    200, 151,
    113, 110,
    255, 158,
    // q
    0, 0,
    184, 188,
    147, 139,
    186, 255,
    // r
    0, 0,
    122, 130,
    111, 0,
    109, 0,
    // s
    0, 0,
    132, 69,
    109, 93,
    110, 136,
    // t
    51, 5,
    205, 103,
    61, 6,
    47, 106,
    // u
    0, 0,
    110, 109,
    110, 122,
    155, 179,
    // v
    0, 0,
    132, 120,
    113, 114,
    84, 63,
    // w
    0, 0,
    124, 108,
    202, 189,
    160, 174,
    // x
    0, 0,
    144, 142,
    79, 57,
    159, 146,
    // y
    0, 0,
    138, 138,
    119, 117,
    255, 69,
    // z
    0, 0,
    97, 198,
    47, 38,
    208, 84,
    // {
    23, 112,
    41, 14,
    157, 7,
    121, 192,
    // |
    35, 11,
    35, 11,
    35, 11,
    160, 61,
    // }
    129, 9,
    40, 19,
    20, 139,
    236, 44,
    // ~
    0, 0,
    15, 3,
    97, 93,
    0, 0,
];
var x1Data = [
    //
    0,
    0,
    // !
    23,
    12,
    // "
    53,
    0,
    // #
    130,
    127,
    // $
    58,
    149,
    // %
    67,
    77,
    // &
    72,
    198,
    // '
    13,
    0,
    // (
    25,
    51,
    // )
    25,
    49,
    // *
    94,
    2,
    // +
    8,
    64,
    // ,
    0,
    24,
    // -
    0,
    21,
    // .
    0,
    9,
    // /
    19,
    27,
    // 0
    126,
    126,
    // 1
    51,
    80,
    // 2
    72,
    105,
    // 3
    87,
    98,
    // 4
    73,
    93,
    // 5
    106,
    85,
    // 6
    111,
    123,
    // 7
    87,
    30,
    // 8
    116,
    126,
    // 9
    123,
    110,
    // :
    4,
    16,
    // ;
    9,
    28,
    // <
    21,
    53,
    // =
    8,
    62,
    // >
    23,
    52,
    // ?
    73,
    21,
    // @
    132,
    183,
    // A
    78,
    142,
    // B
    168,
    175,
    // C
    70,
    70,
    // D
    128,
    128,
    // E
    123,
    110,
    // F
    125,
    43,
    // G
    100,
    139,
    // H
    125,
    119,
    // I
    78,
    78,
    // J
    54,
    77,
    // K
    139,
    139,
    // L
    33,
    87,
    // M
    201,
    117,
    // N
    162,
    149,
    // O
    130,
    130,
    // P
    138,
    60,
    // Q
    130,
    172,
    // R
    149,
    127,
    // S
    95,
    98,
    // T
    95,
    25,
    // U
    118,
    135,
    // V
    110,
    85,
    // W
    147,
    175,
    // X
    105,
    110,
    // Y
    121,
    30,
    // Z
    101,
    113,
    // [
    34,
    68,
    // \
    20,
    26,
    // ]
    34,
    68,
    // ^
    56,
    0,
    // _
    0,
    44,
    // `
    3,
    0,
    // a
    27,
    175,
    // b
    80,
    133,
    // c
    31,
    66,
    // d
    85,
    147,
    // e
    32,
    150,
    // f
    90,
    25,
    // g
    45,
    230,
    // h
    77,
    101,
    // i
    36,
    83,
    // j
    22,
    84,
    // k
    71,
    118,
    // l
    44,
    44,
    // m
    52,
    172,
    // n
    38,
    101,
    // o
    35,
    130,
    // p
    40,
    197,
    // q
    43,
    197,
    // r
    29,
    26,
    // s
    23,
    103,
    // t
    67,
    44,
    // u
    25,
    129,
    // v
    29,
    85,
    // w
    27,
    177,
    // x
    33,
    97,
    // y
    32,
    145,
    // z
    33,
    77,
    // {
    38,
    96,
    // |
    20,
    55,
    // }
    36,
    95,
    // ~
    2,
    22,
];

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$S = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function getMinimapLineHeight(renderMinimap) {
    if (renderMinimap === 2 /* Large */) {
        return 4 /* x2_CHAR_HEIGHT */;
    }
    if (renderMinimap === 4 /* LargeBlocks */) {
        return 4 /* x2_CHAR_HEIGHT */ + 2;
    }
    if (renderMinimap === 1 /* Small */) {
        return 2 /* x1_CHAR_HEIGHT */;
    }
    // RenderMinimap.SmallBlocks
    return 2 /* x1_CHAR_HEIGHT */ + 1;
}
function getMinimapCharWidth(renderMinimap) {
    if (renderMinimap === 2 /* Large */) {
        return 2 /* x2_CHAR_WIDTH */;
    }
    if (renderMinimap === 4 /* LargeBlocks */) {
        return 2 /* x2_CHAR_WIDTH */;
    }
    if (renderMinimap === 1 /* Small */) {
        return 1 /* x1_CHAR_WIDTH */;
    }
    // RenderMinimap.SmallBlocks
    return 1 /* x1_CHAR_WIDTH */;
}
/**
 * The orthogonal distance to the slider at which dragging "resets". This implements "snapping"
 */
var MOUSE_DRAG_RESET_DISTANCE$1 = 140;
var MinimapOptions = /** @class */ (function () {
    function MinimapOptions(configuration) {
        var pixelRatio = configuration.editor.pixelRatio;
        var layoutInfo = configuration.editor.layoutInfo;
        var viewInfo = configuration.editor.viewInfo;
        var fontInfo = configuration.editor.fontInfo;
        this.renderMinimap = layoutInfo.renderMinimap | 0;
        this.scrollBeyondLastLine = viewInfo.scrollBeyondLastLine;
        this.showSlider = viewInfo.minimap.showSlider;
        this.pixelRatio = pixelRatio;
        this.typicalHalfwidthCharacterWidth = fontInfo.typicalHalfwidthCharacterWidth;
        this.lineHeight = configuration.editor.lineHeight;
        this.minimapLeft = layoutInfo.minimapLeft;
        this.minimapWidth = layoutInfo.minimapWidth;
        this.minimapHeight = layoutInfo.height;
        this.canvasInnerWidth = Math.max(1, Math.floor(pixelRatio * this.minimapWidth));
        this.canvasInnerHeight = Math.max(1, Math.floor(pixelRatio * this.minimapHeight));
        this.canvasOuterWidth = this.canvasInnerWidth / pixelRatio;
        this.canvasOuterHeight = this.canvasInnerHeight / pixelRatio;
    }
    MinimapOptions.prototype.equals = function (other) {
        return (this.renderMinimap === other.renderMinimap
            && this.scrollBeyondLastLine === other.scrollBeyondLastLine
            && this.showSlider === other.showSlider
            && this.pixelRatio === other.pixelRatio
            && this.typicalHalfwidthCharacterWidth === other.typicalHalfwidthCharacterWidth
            && this.lineHeight === other.lineHeight
            && this.minimapLeft === other.minimapLeft
            && this.minimapWidth === other.minimapWidth
            && this.minimapHeight === other.minimapHeight
            && this.canvasInnerWidth === other.canvasInnerWidth
            && this.canvasInnerHeight === other.canvasInnerHeight
            && this.canvasOuterWidth === other.canvasOuterWidth
            && this.canvasOuterHeight === other.canvasOuterHeight);
    };
    return MinimapOptions;
}());
var MinimapLayout = /** @class */ (function () {
    function MinimapLayout(scrollTop, scrollHeight, computedSliderRatio, sliderTop, sliderHeight, startLineNumber, endLineNumber) {
        this.scrollTop = scrollTop;
        this.scrollHeight = scrollHeight;
        this._computedSliderRatio = computedSliderRatio;
        this.sliderTop = sliderTop;
        this.sliderHeight = sliderHeight;
        this.startLineNumber = startLineNumber;
        this.endLineNumber = endLineNumber;
    }
    /**
     * Compute a desired `scrollPosition` such that the slider moves by `delta`.
     */
    MinimapLayout.prototype.getDesiredScrollTopFromDelta = function (delta) {
        var desiredSliderPosition = this.sliderTop + delta;
        return Math.round(desiredSliderPosition / this._computedSliderRatio);
    };
    MinimapLayout.create = function (options, viewportStartLineNumber, viewportEndLineNumber, viewportHeight, viewportContainsWhitespaceGaps, lineCount, scrollTop, scrollHeight, previousLayout) {
        var pixelRatio = options.pixelRatio;
        var minimapLineHeight = getMinimapLineHeight(options.renderMinimap);
        var minimapLinesFitting = Math.floor(options.canvasInnerHeight / minimapLineHeight);
        var lineHeight = options.lineHeight;
        // The visible line count in a viewport can change due to a number of reasons:
        //  a) with the same viewport width, different scroll positions can result in partial lines being visible:
        //    e.g. for a line height of 20, and a viewport height of 600
        //          * scrollTop = 0  => visible lines are [1, 30]
        //          * scrollTop = 10 => visible lines are [1, 31] (with lines 1 and 31 partially visible)
        //          * scrollTop = 20 => visible lines are [2, 31]
        //  b) whitespace gaps might make their way in the viewport (which results in a decrease in the visible line count)
        //  c) we could be in the scroll beyond last line case (which also results in a decrease in the visible line count, down to possibly only one line being visible)
        // We must first establish a desirable slider height.
        var sliderHeight;
        if (viewportContainsWhitespaceGaps && viewportEndLineNumber !== lineCount) {
            // case b) from above: there are whitespace gaps in the viewport.
            // In this case, the height of the slider directly reflects the visible line count.
            var viewportLineCount = viewportEndLineNumber - viewportStartLineNumber + 1;
            sliderHeight = Math.floor(viewportLineCount * minimapLineHeight / pixelRatio);
        }
        else {
            // The slider has a stable height
            var expectedViewportLineCount = viewportHeight / lineHeight;
            sliderHeight = Math.floor(expectedViewportLineCount * minimapLineHeight / pixelRatio);
        }
        var maxMinimapSliderTop;
        if (options.scrollBeyondLastLine) {
            // The minimap slider, when dragged all the way down, will contain the last line at its top
            maxMinimapSliderTop = (lineCount - 1) * minimapLineHeight / pixelRatio;
        }
        else {
            // The minimap slider, when dragged all the way down, will contain the last line at its bottom
            maxMinimapSliderTop = Math.max(0, lineCount * minimapLineHeight / pixelRatio - sliderHeight);
        }
        maxMinimapSliderTop = Math.min(options.minimapHeight - sliderHeight, maxMinimapSliderTop);
        // The slider can move from 0 to `maxMinimapSliderTop`
        // in the same way `scrollTop` can move from 0 to `scrollHeight` - `viewportHeight`.
        var computedSliderRatio = (maxMinimapSliderTop) / (scrollHeight - viewportHeight);
        var sliderTop = (scrollTop * computedSliderRatio);
        if (minimapLinesFitting >= lineCount) {
            // All lines fit in the minimap
            var startLineNumber = 1;
            var endLineNumber = lineCount;
            return new MinimapLayout(scrollTop, scrollHeight, computedSliderRatio, sliderTop, sliderHeight, startLineNumber, endLineNumber);
        }
        else {
            var startLineNumber = Math.max(1, Math.floor(viewportStartLineNumber - sliderTop * pixelRatio / minimapLineHeight));
            // Avoid flickering caused by a partial viewport start line
            // by being consistent w.r.t. the previous layout decision
            if (previousLayout && previousLayout.scrollHeight === scrollHeight) {
                if (previousLayout.scrollTop > scrollTop) {
                    // Scrolling up => never increase `startLineNumber`
                    startLineNumber = Math.min(startLineNumber, previousLayout.startLineNumber);
                }
                if (previousLayout.scrollTop < scrollTop) {
                    // Scrolling down => never decrease `startLineNumber`
                    startLineNumber = Math.max(startLineNumber, previousLayout.startLineNumber);
                }
            }
            var endLineNumber = Math.min(lineCount, startLineNumber + minimapLinesFitting - 1);
            return new MinimapLayout(scrollTop, scrollHeight, computedSliderRatio, sliderTop, sliderHeight, startLineNumber, endLineNumber);
        }
    };
    return MinimapLayout;
}());
var MinimapLine = /** @class */ (function () {
    function MinimapLine(dy) {
        this.dy = dy;
    }
    MinimapLine.prototype.onContentChanged = function () {
        this.dy = -1;
    };
    MinimapLine.prototype.onTokensChanged = function () {
        this.dy = -1;
    };
    MinimapLine.INVALID = new MinimapLine(-1);
    return MinimapLine;
}());
var RenderData = /** @class */ (function () {
    function RenderData(renderedLayout, imageData, lines) {
        this.renderedLayout = renderedLayout;
        this._imageData = imageData;
        this._renderedLines = new RenderedLinesCollection(function () { return MinimapLine.INVALID; });
        this._renderedLines._set(renderedLayout.startLineNumber, lines);
    }
    /**
     * Check if the current RenderData matches accurately the new desired layout and no painting is needed.
     */
    RenderData.prototype.linesEquals = function (layout) {
        if (this.renderedLayout.startLineNumber !== layout.startLineNumber) {
            return false;
        }
        if (this.renderedLayout.endLineNumber !== layout.endLineNumber) {
            return false;
        }
        var tmp = this._renderedLines._get();
        var lines = tmp.lines;
        for (var i = 0, len = lines.length; i < len; i++) {
            if (lines[i].dy === -1) {
                // This line is invalid
                return false;
            }
        }
        return true;
    };
    RenderData.prototype._get = function () {
        var tmp = this._renderedLines._get();
        return {
            imageData: this._imageData,
            rendLineNumberStart: tmp.rendLineNumberStart,
            lines: tmp.lines
        };
    };
    RenderData.prototype.onLinesChanged = function (e) {
        return this._renderedLines.onLinesChanged(e.fromLineNumber, e.toLineNumber);
    };
    RenderData.prototype.onLinesDeleted = function (e) {
        this._renderedLines.onLinesDeleted(e.fromLineNumber, e.toLineNumber);
    };
    RenderData.prototype.onLinesInserted = function (e) {
        this._renderedLines.onLinesInserted(e.fromLineNumber, e.toLineNumber);
    };
    RenderData.prototype.onTokensChanged = function (e) {
        return this._renderedLines.onTokensChanged(e.ranges);
    };
    return RenderData;
}());
/**
 * Some sort of double buffering.
 *
 * Keeps two buffers around that will be rotated for painting.
 * Always gives a buffer that is filled with the background color.
 */
var MinimapBuffers = /** @class */ (function () {
    function MinimapBuffers(ctx, WIDTH, HEIGHT, background) {
        this._backgroundFillData = MinimapBuffers._createBackgroundFillData(WIDTH, HEIGHT, background);
        this._buffers = [
            ctx.createImageData(WIDTH, HEIGHT),
            ctx.createImageData(WIDTH, HEIGHT)
        ];
        this._lastUsedBuffer = 0;
    }
    MinimapBuffers.prototype.getBuffer = function () {
        // rotate buffers
        this._lastUsedBuffer = 1 - this._lastUsedBuffer;
        var result = this._buffers[this._lastUsedBuffer];
        // fill with background color
        result.data.set(this._backgroundFillData);
        return result;
    };
    MinimapBuffers._createBackgroundFillData = function (WIDTH, HEIGHT, background) {
        var backgroundR = background.r;
        var backgroundG = background.g;
        var backgroundB = background.b;
        var result = new Uint8ClampedArray(WIDTH * HEIGHT * 4);
        var offset = 0;
        for (var i = 0; i < HEIGHT; i++) {
            for (var j = 0; j < WIDTH; j++) {
                result[offset] = backgroundR;
                result[offset + 1] = backgroundG;
                result[offset + 2] = backgroundB;
                result[offset + 3] = 255;
                offset += 4;
            }
        }
        return result;
    };
    return MinimapBuffers;
}());
var Minimap = /** @class */ (function (_super) {
    __extends$S(Minimap, _super);
    function Minimap(context) {
        var _this = _super.call(this, context) || this;
        _this._options = new MinimapOptions(_this._context.configuration);
        _this._lastRenderData = null;
        _this._buffers = null;
        _this._domNode = createFastDomNode(document.createElement('div'));
        PartFingerprints.write(_this._domNode, 8 /* Minimap */);
        _this._domNode.setClassName(_this._getMinimapDomNodeClassName());
        _this._domNode.setPosition('absolute');
        _this._domNode.setAttribute('role', 'presentation');
        _this._domNode.setAttribute('aria-hidden', 'true');
        _this._shadow = createFastDomNode(document.createElement('div'));
        _this._shadow.setClassName('minimap-shadow-hidden');
        _this._domNode.appendChild(_this._shadow);
        _this._canvas = createFastDomNode(document.createElement('canvas'));
        _this._canvas.setPosition('absolute');
        _this._canvas.setLeft(0);
        _this._domNode.appendChild(_this._canvas);
        _this._slider = createFastDomNode(document.createElement('div'));
        _this._slider.setPosition('absolute');
        _this._slider.setClassName('minimap-slider');
        _this._slider.setLayerHinting(true);
        _this._domNode.appendChild(_this._slider);
        _this._sliderHorizontal = createFastDomNode(document.createElement('div'));
        _this._sliderHorizontal.setPosition('absolute');
        _this._sliderHorizontal.setClassName('minimap-slider-horizontal');
        _this._slider.appendChild(_this._sliderHorizontal);
        _this._tokensColorTracker = MinimapTokensColorTracker.getInstance();
        _this._applyLayout();
        _this._mouseDownListener = addStandardDisposableListener(_this._canvas.domNode, 'mousedown', function (e) {
            e.preventDefault();
            var renderMinimap = _this._options.renderMinimap;
            if (renderMinimap === 0 /* None */) {
                return;
            }
            if (!_this._lastRenderData) {
                return;
            }
            var minimapLineHeight = getMinimapLineHeight(renderMinimap);
            var internalOffsetY = _this._options.pixelRatio * e.browserEvent.offsetY;
            var lineIndex = Math.floor(internalOffsetY / minimapLineHeight);
            var lineNumber = lineIndex + _this._lastRenderData.renderedLayout.startLineNumber;
            lineNumber = Math.min(lineNumber, _this._context.model.getLineCount());
            _this._context.privateViewEventBus.emit(new ViewRevealRangeRequestEvent(new Range(lineNumber, 1, lineNumber, 1), 1 /* Center */, false, 0 /* Smooth */));
        });
        _this._sliderMouseMoveMonitor = new GlobalMouseMoveMonitor();
        _this._sliderMouseDownListener = addStandardDisposableListener(_this._slider.domNode, 'mousedown', function (e) {
            e.preventDefault();
            if (e.leftButton && _this._lastRenderData) {
                var initialMousePosition_1 = e.posy;
                var initialMouseOrthogonalPosition_1 = e.posx;
                var initialSliderState_1 = _this._lastRenderData.renderedLayout;
                _this._slider.toggleClassName('active', true);
                _this._sliderMouseMoveMonitor.startMonitoring(standardMouseMoveMerger, function (mouseMoveData) {
                    var mouseOrthogonalDelta = Math.abs(mouseMoveData.posx - initialMouseOrthogonalPosition_1);
                    if (isWindows && mouseOrthogonalDelta > MOUSE_DRAG_RESET_DISTANCE$1) {
                        // The mouse has wondered away from the scrollbar => reset dragging
                        _this._context.viewLayout.setScrollPositionNow({
                            scrollTop: initialSliderState_1.scrollTop
                        });
                        return;
                    }
                    var mouseDelta = mouseMoveData.posy - initialMousePosition_1;
                    _this._context.viewLayout.setScrollPositionNow({
                        scrollTop: initialSliderState_1.getDesiredScrollTopFromDelta(mouseDelta)
                    });
                }, function () {
                    _this._slider.toggleClassName('active', false);
                });
            }
        });
        return _this;
    }
    Minimap.prototype.dispose = function () {
        this._mouseDownListener.dispose();
        this._sliderMouseMoveMonitor.dispose();
        this._sliderMouseDownListener.dispose();
        _super.prototype.dispose.call(this);
    };
    Minimap.prototype._getMinimapDomNodeClassName = function () {
        if (this._options.showSlider === 'always') {
            return 'minimap slider-always';
        }
        return 'minimap slider-mouseover';
    };
    Minimap.prototype.getDomNode = function () {
        return this._domNode;
    };
    Minimap.prototype._applyLayout = function () {
        this._domNode.setLeft(this._options.minimapLeft);
        this._domNode.setWidth(this._options.minimapWidth);
        this._domNode.setHeight(this._options.minimapHeight);
        this._shadow.setHeight(this._options.minimapHeight);
        this._canvas.setWidth(this._options.canvasOuterWidth);
        this._canvas.setHeight(this._options.canvasOuterHeight);
        this._canvas.domNode.width = this._options.canvasInnerWidth;
        this._canvas.domNode.height = this._options.canvasInnerHeight;
        this._slider.setWidth(this._options.minimapWidth);
    };
    Minimap.prototype._getBuffer = function () {
        if (!this._buffers) {
            this._buffers = new MinimapBuffers(this._canvas.domNode.getContext('2d'), this._options.canvasInnerWidth, this._options.canvasInnerHeight, this._tokensColorTracker.getColor(2 /* DefaultBackground */));
        }
        return this._buffers.getBuffer();
    };
    Minimap.prototype._onOptionsMaybeChanged = function () {
        var opts = new MinimapOptions(this._context.configuration);
        if (this._options.equals(opts)) {
            return false;
        }
        this._options = opts;
        this._lastRenderData = null;
        this._buffers = null;
        this._applyLayout();
        this._domNode.setClassName(this._getMinimapDomNodeClassName());
        return true;
    };
    // ---- begin view event handlers
    Minimap.prototype.onConfigurationChanged = function (e) {
        return this._onOptionsMaybeChanged();
    };
    Minimap.prototype.onFlushed = function (e) {
        this._lastRenderData = null;
        return true;
    };
    Minimap.prototype.onLinesChanged = function (e) {
        if (this._lastRenderData) {
            return this._lastRenderData.onLinesChanged(e);
        }
        return false;
    };
    Minimap.prototype.onLinesDeleted = function (e) {
        if (this._lastRenderData) {
            this._lastRenderData.onLinesDeleted(e);
        }
        return true;
    };
    Minimap.prototype.onLinesInserted = function (e) {
        if (this._lastRenderData) {
            this._lastRenderData.onLinesInserted(e);
        }
        return true;
    };
    Minimap.prototype.onScrollChanged = function (e) {
        return true;
    };
    Minimap.prototype.onTokensChanged = function (e) {
        if (this._lastRenderData) {
            return this._lastRenderData.onTokensChanged(e);
        }
        return false;
    };
    Minimap.prototype.onTokensColorsChanged = function (e) {
        this._lastRenderData = null;
        this._buffers = null;
        return true;
    };
    Minimap.prototype.onZonesChanged = function (e) {
        this._lastRenderData = null;
        return true;
    };
    // --- end event handlers
    Minimap.prototype.prepareRender = function (ctx) {
        // Nothing to read
    };
    Minimap.prototype.render = function (renderingCtx) {
        var renderMinimap = this._options.renderMinimap;
        if (renderMinimap === 0 /* None */) {
            this._shadow.setClassName('minimap-shadow-hidden');
            this._sliderHorizontal.setWidth(0);
            this._sliderHorizontal.setHeight(0);
            return;
        }
        if (renderingCtx.scrollLeft + renderingCtx.viewportWidth >= renderingCtx.scrollWidth) {
            this._shadow.setClassName('minimap-shadow-hidden');
        }
        else {
            this._shadow.setClassName('minimap-shadow-visible');
        }
        var layout = MinimapLayout.create(this._options, renderingCtx.visibleRange.startLineNumber, renderingCtx.visibleRange.endLineNumber, renderingCtx.viewportHeight, (renderingCtx.viewportData.whitespaceViewportData.length > 0), this._context.model.getLineCount(), renderingCtx.scrollTop, renderingCtx.scrollHeight, this._lastRenderData ? this._lastRenderData.renderedLayout : null);
        this._slider.setTop(layout.sliderTop);
        this._slider.setHeight(layout.sliderHeight);
        // Compute horizontal slider coordinates
        var scrollLeftChars = renderingCtx.scrollLeft / this._options.typicalHalfwidthCharacterWidth;
        var horizontalSliderLeft = Math.min(this._options.minimapWidth, Math.round(scrollLeftChars * getMinimapCharWidth(this._options.renderMinimap) / this._options.pixelRatio));
        this._sliderHorizontal.setLeft(horizontalSliderLeft);
        this._sliderHorizontal.setWidth(this._options.minimapWidth - horizontalSliderLeft);
        this._sliderHorizontal.setTop(0);
        this._sliderHorizontal.setHeight(layout.sliderHeight);
        this._lastRenderData = this.renderLines(layout);
    };
    Minimap.prototype.renderLines = function (layout) {
        var renderMinimap = this._options.renderMinimap;
        var startLineNumber = layout.startLineNumber;
        var endLineNumber = layout.endLineNumber;
        var minimapLineHeight = getMinimapLineHeight(renderMinimap);
        // Check if nothing changed w.r.t. lines from last frame
        if (this._lastRenderData && this._lastRenderData.linesEquals(layout)) {
            var _lastData = this._lastRenderData._get();
            // Nice!! Nothing changed from last frame
            return new RenderData(layout, _lastData.imageData, _lastData.lines);
        }
        // Oh well!! We need to repaint some lines...
        var imageData = this._getBuffer();
        // Render untouched lines by using last rendered data.
        var _a = Minimap._renderUntouchedLines(imageData, startLineNumber, endLineNumber, minimapLineHeight, this._lastRenderData), _dirtyY1 = _a[0], _dirtyY2 = _a[1], needed = _a[2];
        // Fetch rendering info from view model for rest of lines that need rendering.
        var lineInfo = this._context.model.getMinimapLinesRenderingData(startLineNumber, endLineNumber, needed);
        var tabSize = lineInfo.tabSize;
        var background = this._tokensColorTracker.getColor(2 /* DefaultBackground */);
        var useLighterFont = this._tokensColorTracker.backgroundIsLight();
        // Render the rest of lines
        var dy = 0;
        var renderedLines = [];
        for (var lineIndex = 0, lineCount = endLineNumber - startLineNumber + 1; lineIndex < lineCount; lineIndex++) {
            if (needed[lineIndex]) {
                Minimap._renderLine(imageData, background, useLighterFont, renderMinimap, this._tokensColorTracker, getOrCreateMinimapCharRenderer(), dy, tabSize, lineInfo.data[lineIndex]);
            }
            renderedLines[lineIndex] = new MinimapLine(dy);
            dy += minimapLineHeight;
        }
        var dirtyY1 = (_dirtyY1 === -1 ? 0 : _dirtyY1);
        var dirtyY2 = (_dirtyY2 === -1 ? imageData.height : _dirtyY2);
        var dirtyHeight = dirtyY2 - dirtyY1;
        // Finally, paint to the canvas
        var ctx = this._canvas.domNode.getContext('2d');
        ctx.putImageData(imageData, 0, 0, 0, dirtyY1, imageData.width, dirtyHeight);
        // Save rendered data for reuse on next frame if possible
        return new RenderData(layout, imageData, renderedLines);
    };
    Minimap._renderUntouchedLines = function (target, startLineNumber, endLineNumber, minimapLineHeight, lastRenderData) {
        var needed = [];
        if (!lastRenderData) {
            for (var i = 0, len = endLineNumber - startLineNumber + 1; i < len; i++) {
                needed[i] = true;
            }
            return [-1, -1, needed];
        }
        var _lastData = lastRenderData._get();
        var lastTargetData = _lastData.imageData.data;
        var lastStartLineNumber = _lastData.rendLineNumberStart;
        var lastLines = _lastData.lines;
        var lastLinesLength = lastLines.length;
        var WIDTH = target.width;
        var targetData = target.data;
        var maxDestPixel = (endLineNumber - startLineNumber + 1) * minimapLineHeight * WIDTH * 4;
        var dirtyPixel1 = -1; // the pixel offset up to which all the data is equal to the prev frame
        var dirtyPixel2 = -1; // the pixel offset after which all the data is equal to the prev frame
        var copySourceStart = -1;
        var copySourceEnd = -1;
        var copyDestStart = -1;
        var copyDestEnd = -1;
        var dest_dy = 0;
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            var lineIndex = lineNumber - startLineNumber;
            var lastLineIndex = lineNumber - lastStartLineNumber;
            var source_dy = (lastLineIndex >= 0 && lastLineIndex < lastLinesLength ? lastLines[lastLineIndex].dy : -1);
            if (source_dy === -1) {
                needed[lineIndex] = true;
                dest_dy += minimapLineHeight;
                continue;
            }
            var sourceStart = source_dy * WIDTH * 4;
            var sourceEnd = (source_dy + minimapLineHeight) * WIDTH * 4;
            var destStart = dest_dy * WIDTH * 4;
            var destEnd = (dest_dy + minimapLineHeight) * WIDTH * 4;
            if (copySourceEnd === sourceStart && copyDestEnd === destStart) {
                // contiguous zone => extend copy request
                copySourceEnd = sourceEnd;
                copyDestEnd = destEnd;
            }
            else {
                if (copySourceStart !== -1) {
                    // flush existing copy request
                    targetData.set(lastTargetData.subarray(copySourceStart, copySourceEnd), copyDestStart);
                    if (dirtyPixel1 === -1 && copySourceStart === 0 && copySourceStart === copyDestStart) {
                        dirtyPixel1 = copySourceEnd;
                    }
                    if (dirtyPixel2 === -1 && copySourceEnd === maxDestPixel && copySourceStart === copyDestStart) {
                        dirtyPixel2 = copySourceStart;
                    }
                }
                copySourceStart = sourceStart;
                copySourceEnd = sourceEnd;
                copyDestStart = destStart;
                copyDestEnd = destEnd;
            }
            needed[lineIndex] = false;
            dest_dy += minimapLineHeight;
        }
        if (copySourceStart !== -1) {
            // flush existing copy request
            targetData.set(lastTargetData.subarray(copySourceStart, copySourceEnd), copyDestStart);
            if (dirtyPixel1 === -1 && copySourceStart === 0 && copySourceStart === copyDestStart) {
                dirtyPixel1 = copySourceEnd;
            }
            if (dirtyPixel2 === -1 && copySourceEnd === maxDestPixel && copySourceStart === copyDestStart) {
                dirtyPixel2 = copySourceStart;
            }
        }
        var dirtyY1 = (dirtyPixel1 === -1 ? -1 : dirtyPixel1 / (WIDTH * 4));
        var dirtyY2 = (dirtyPixel2 === -1 ? -1 : dirtyPixel2 / (WIDTH * 4));
        return [dirtyY1, dirtyY2, needed];
    };
    Minimap._renderLine = function (target, backgroundColor, useLighterFont, renderMinimap, colorTracker, minimapCharRenderer, dy, tabSize, lineData) {
        var content = lineData.content;
        var tokens = lineData.tokens;
        var charWidth = getMinimapCharWidth(renderMinimap);
        var maxDx = target.width - charWidth;
        var dx = 0;
        var charIndex = 0;
        var tabsCharDelta = 0;
        for (var tokenIndex = 0, tokensLen = tokens.getCount(); tokenIndex < tokensLen; tokenIndex++) {
            var tokenEndIndex = tokens.getEndOffset(tokenIndex);
            var tokenColorId = tokens.getForeground(tokenIndex);
            var tokenColor = colorTracker.getColor(tokenColorId);
            for (; charIndex < tokenEndIndex; charIndex++) {
                if (dx > maxDx) {
                    // hit edge of minimap
                    return;
                }
                var charCode = content.charCodeAt(charIndex);
                if (charCode === 9 /* Tab */) {
                    var insertSpacesCount = tabSize - (charIndex + tabsCharDelta) % tabSize;
                    tabsCharDelta += insertSpacesCount - 1;
                    // No need to render anything since tab is invisible
                    dx += insertSpacesCount * charWidth;
                }
                else if (charCode === 32 /* Space */) {
                    // No need to render anything since space is invisible
                    dx += charWidth;
                }
                else {
                    // Render twice for a full width character
                    var count = isFullWidthCharacter(charCode) ? 2 : 1;
                    for (var i = 0; i < count; i++) {
                        if (renderMinimap === 2 /* Large */) {
                            minimapCharRenderer.x2RenderChar(target, dx, dy, charCode, tokenColor, backgroundColor, useLighterFont);
                        }
                        else if (renderMinimap === 1 /* Small */) {
                            minimapCharRenderer.x1RenderChar(target, dx, dy, charCode, tokenColor, backgroundColor, useLighterFont);
                        }
                        else if (renderMinimap === 4 /* LargeBlocks */) {
                            minimapCharRenderer.x2BlockRenderChar(target, dx, dy, tokenColor, backgroundColor, useLighterFont);
                        }
                        else {
                            // RenderMinimap.SmallBlocks
                            minimapCharRenderer.x1BlockRenderChar(target, dx, dy, tokenColor, backgroundColor, useLighterFont);
                        }
                        dx += charWidth;
                        if (dx > maxDx) {
                            // hit edge of minimap
                            return;
                        }
                    }
                }
            }
        }
    };
    return Minimap;
}(ViewPart));
registerThemingParticipant(function (theme, collector) {
    var sliderBackground = theme.getColor(scrollbarSliderBackground);
    if (sliderBackground) {
        var halfSliderBackground = sliderBackground.transparent(0.5);
        collector.addRule(".monaco-editor .minimap-slider, .monaco-editor .minimap-slider .minimap-slider-horizontal { background: " + halfSliderBackground + "; }");
    }
    var sliderHoverBackground = theme.getColor(scrollbarSliderHoverBackground);
    if (sliderHoverBackground) {
        var halfSliderHoverBackground = sliderHoverBackground.transparent(0.5);
        collector.addRule(".monaco-editor .minimap-slider:hover, .monaco-editor .minimap-slider:hover .minimap-slider-horizontal { background: " + halfSliderHoverBackground + "; }");
    }
    var sliderActiveBackground = theme.getColor(scrollbarSliderActiveBackground);
    if (sliderActiveBackground) {
        var halfSliderActiveBackground = sliderActiveBackground.transparent(0.5);
        collector.addRule(".monaco-editor .minimap-slider.active, .monaco-editor .minimap-slider.active .minimap-slider-horizontal { background: " + halfSliderActiveBackground + "; }");
    }
    var shadow = theme.getColor(scrollbarShadow);
    if (shadow) {
        collector.addRule(".monaco-editor .minimap-shadow-visible { box-shadow: " + shadow + " -6px 0 6px -6px inset; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$T = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewOverlayWidgets = /** @class */ (function (_super) {
    __extends$T(ViewOverlayWidgets, _super);
    function ViewOverlayWidgets(context) {
        var _this = _super.call(this, context) || this;
        _this._widgets = {};
        _this._verticalScrollbarWidth = _this._context.configuration.editor.layoutInfo.verticalScrollbarWidth;
        _this._minimapWidth = _this._context.configuration.editor.layoutInfo.minimapWidth;
        _this._horizontalScrollbarHeight = _this._context.configuration.editor.layoutInfo.horizontalScrollbarHeight;
        _this._editorHeight = _this._context.configuration.editor.layoutInfo.height;
        _this._editorWidth = _this._context.configuration.editor.layoutInfo.width;
        _this._domNode = createFastDomNode(document.createElement('div'));
        PartFingerprints.write(_this._domNode, 4 /* OverlayWidgets */);
        _this._domNode.setClassName('overlayWidgets');
        return _this;
    }
    ViewOverlayWidgets.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this._widgets = {};
    };
    ViewOverlayWidgets.prototype.getDomNode = function () {
        return this._domNode;
    };
    // ---- begin view event handlers
    ViewOverlayWidgets.prototype.onConfigurationChanged = function (e) {
        if (e.layoutInfo) {
            this._verticalScrollbarWidth = this._context.configuration.editor.layoutInfo.verticalScrollbarWidth;
            this._minimapWidth = this._context.configuration.editor.layoutInfo.minimapWidth;
            this._horizontalScrollbarHeight = this._context.configuration.editor.layoutInfo.horizontalScrollbarHeight;
            this._editorHeight = this._context.configuration.editor.layoutInfo.height;
            this._editorWidth = this._context.configuration.editor.layoutInfo.width;
            return true;
        }
        return false;
    };
    // ---- end view event handlers
    ViewOverlayWidgets.prototype.addWidget = function (widget) {
        var domNode = createFastDomNode(widget.getDomNode());
        this._widgets[widget.getId()] = {
            widget: widget,
            preference: null,
            domNode: domNode
        };
        // This is sync because a widget wants to be in the dom
        domNode.setPosition('absolute');
        domNode.setAttribute('widgetId', widget.getId());
        this._domNode.appendChild(domNode);
        this.setShouldRender();
    };
    ViewOverlayWidgets.prototype.setWidgetPosition = function (widget, preference) {
        var widgetData = this._widgets[widget.getId()];
        if (widgetData.preference === preference) {
            return false;
        }
        widgetData.preference = preference;
        this.setShouldRender();
        return true;
    };
    ViewOverlayWidgets.prototype.removeWidget = function (widget) {
        var widgetId = widget.getId();
        if (this._widgets.hasOwnProperty(widgetId)) {
            var widgetData = this._widgets[widgetId];
            var domNode = widgetData.domNode.domNode;
            delete this._widgets[widgetId];
            domNode.parentNode.removeChild(domNode);
            this.setShouldRender();
        }
    };
    ViewOverlayWidgets.prototype._renderWidget = function (widgetData) {
        var domNode = widgetData.domNode;
        if (widgetData.preference === null) {
            domNode.unsetTop();
            return;
        }
        if (widgetData.preference === 0 /* TOP_RIGHT_CORNER */) {
            domNode.setTop(0);
            domNode.setRight((2 * this._verticalScrollbarWidth) + this._minimapWidth);
        }
        else if (widgetData.preference === 1 /* BOTTOM_RIGHT_CORNER */) {
            var widgetHeight = domNode.domNode.clientHeight;
            domNode.setTop((this._editorHeight - widgetHeight - 2 * this._horizontalScrollbarHeight));
            domNode.setRight((2 * this._verticalScrollbarWidth) + this._minimapWidth);
        }
        else if (widgetData.preference === 2 /* TOP_CENTER */) {
            domNode.setTop(0);
            domNode.domNode.style.right = '50%';
        }
    };
    ViewOverlayWidgets.prototype.prepareRender = function (ctx) {
        // Nothing to read
    };
    ViewOverlayWidgets.prototype.render = function (ctx) {
        this._domNode.setWidth(this._editorWidth);
        var keys = Object.keys(this._widgets);
        for (var i = 0, len = keys.length; i < len; i++) {
            var widgetId = keys[i];
            this._renderWidget(this._widgets[widgetId]);
        }
    };
    return ViewOverlayWidgets;
}(ViewPart));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$U = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Settings = /** @class */ (function () {
    function Settings(config, theme) {
        this.lineHeight = config.editor.lineHeight;
        this.pixelRatio = config.editor.pixelRatio;
        this.overviewRulerLanes = config.editor.viewInfo.overviewRulerLanes;
        this.renderBorder = config.editor.viewInfo.overviewRulerBorder;
        var borderColor = theme.getColor(editorOverviewRulerBorder);
        this.borderColor = borderColor ? borderColor.toString() : null;
        this.hideCursor = config.editor.viewInfo.hideCursorInOverviewRuler;
        var cursorColor = theme.getColor(editorCursorForeground);
        this.cursorColor = cursorColor ? cursorColor.transparent(0.7).toString() : null;
        this.themeType = theme.type;
        var minimapEnabled = config.editor.viewInfo.minimap.enabled;
        var minimapSide = config.editor.viewInfo.minimap.side;
        var backgroundColor = (minimapEnabled ? TokenizationRegistry.getDefaultBackground() : null);
        if (backgroundColor === null || minimapSide === 'left') {
            this.backgroundColor = null;
        }
        else {
            this.backgroundColor = Color.Format.CSS.formatHex(backgroundColor);
        }
        var position = config.editor.layoutInfo.overviewRuler;
        this.top = position.top;
        this.right = position.right;
        this.domWidth = position.width;
        this.domHeight = position.height;
        this.canvasWidth = (this.domWidth * this.pixelRatio) | 0;
        this.canvasHeight = (this.domHeight * this.pixelRatio) | 0;
        var _a = this._initLanes(1, this.canvasWidth, this.overviewRulerLanes), x = _a[0], w = _a[1];
        this.x = x;
        this.w = w;
    }
    Settings.prototype._initLanes = function (canvasLeftOffset, canvasWidth, laneCount) {
        var remainingWidth = canvasWidth - canvasLeftOffset;
        if (laneCount >= 3) {
            var leftWidth = Math.floor(remainingWidth / 3);
            var rightWidth = Math.floor(remainingWidth / 3);
            var centerWidth = remainingWidth - leftWidth - rightWidth;
            var leftOffset = canvasLeftOffset;
            var centerOffset = leftOffset + leftWidth;
            var rightOffset = leftOffset + leftWidth + centerWidth;
            return [
                [
                    0,
                    leftOffset,
                    centerOffset,
                    leftOffset,
                    rightOffset,
                    leftOffset,
                    centerOffset,
                    leftOffset,
                ], [
                    0,
                    leftWidth,
                    centerWidth,
                    leftWidth + centerWidth,
                    rightWidth,
                    leftWidth + centerWidth + rightWidth,
                    centerWidth + rightWidth,
                    leftWidth + centerWidth + rightWidth,
                ]
            ];
        }
        else if (laneCount === 2) {
            var leftWidth = Math.floor(remainingWidth / 2);
            var rightWidth = remainingWidth - leftWidth;
            var leftOffset = canvasLeftOffset;
            var rightOffset = leftOffset + leftWidth;
            return [
                [
                    0,
                    leftOffset,
                    leftOffset,
                    leftOffset,
                    rightOffset,
                    leftOffset,
                    leftOffset,
                    leftOffset,
                ], [
                    0,
                    leftWidth,
                    leftWidth,
                    leftWidth,
                    rightWidth,
                    leftWidth + rightWidth,
                    leftWidth + rightWidth,
                    leftWidth + rightWidth,
                ]
            ];
        }
        else {
            var offset = canvasLeftOffset;
            var width = remainingWidth;
            return [
                [
                    0,
                    offset,
                    offset,
                    offset,
                    offset,
                    offset,
                    offset,
                    offset,
                ], [
                    0,
                    width,
                    width,
                    width,
                    width,
                    width,
                    width,
                    width,
                ]
            ];
        }
    };
    Settings.prototype.equals = function (other) {
        return (this.lineHeight === other.lineHeight
            && this.pixelRatio === other.pixelRatio
            && this.overviewRulerLanes === other.overviewRulerLanes
            && this.renderBorder === other.renderBorder
            && this.borderColor === other.borderColor
            && this.hideCursor === other.hideCursor
            && this.cursorColor === other.cursorColor
            && this.themeType === other.themeType
            && this.backgroundColor === other.backgroundColor
            && this.top === other.top
            && this.right === other.right
            && this.domWidth === other.domWidth
            && this.domHeight === other.domHeight
            && this.canvasWidth === other.canvasWidth
            && this.canvasHeight === other.canvasHeight);
    };
    return Settings;
}());
var DecorationsOverviewRuler = /** @class */ (function (_super) {
    __extends$U(DecorationsOverviewRuler, _super);
    function DecorationsOverviewRuler(context) {
        var _this = _super.call(this, context) || this;
        _this._domNode = createFastDomNode(document.createElement('canvas'));
        _this._domNode.setClassName('decorationsOverviewRuler');
        _this._domNode.setPosition('absolute');
        _this._domNode.setLayerHinting(true);
        _this._domNode.setAttribute('aria-hidden', 'true');
        _this._updateSettings(false);
        _this._tokensColorTrackerListener = TokenizationRegistry.onDidChange(function (e) {
            if (e.changedColorMap) {
                _this._updateSettings(true);
            }
        });
        _this._cursorPositions = [];
        return _this;
    }
    DecorationsOverviewRuler.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this._tokensColorTrackerListener.dispose();
    };
    DecorationsOverviewRuler.prototype._updateSettings = function (renderNow) {
        var newSettings = new Settings(this._context.configuration, this._context.theme);
        if (this._settings && this._settings.equals(newSettings)) {
            // nothing to do
            return false;
        }
        this._settings = newSettings;
        this._domNode.setTop(this._settings.top);
        this._domNode.setRight(this._settings.right);
        this._domNode.setWidth(this._settings.domWidth);
        this._domNode.setHeight(this._settings.domHeight);
        this._domNode.domNode.width = this._settings.canvasWidth;
        this._domNode.domNode.height = this._settings.canvasHeight;
        if (renderNow) {
            this._render();
        }
        return true;
    };
    // ---- begin view event handlers
    DecorationsOverviewRuler.prototype.onConfigurationChanged = function (e) {
        return this._updateSettings(false);
    };
    DecorationsOverviewRuler.prototype.onCursorStateChanged = function (e) {
        this._cursorPositions = [];
        for (var i = 0, len = e.selections.length; i < len; i++) {
            this._cursorPositions[i] = e.selections[i].getPosition();
        }
        this._cursorPositions.sort(Position.compare);
        return true;
    };
    DecorationsOverviewRuler.prototype.onDecorationsChanged = function (e) {
        return true;
    };
    DecorationsOverviewRuler.prototype.onFlushed = function (e) {
        return true;
    };
    DecorationsOverviewRuler.prototype.onScrollChanged = function (e) {
        return e.scrollHeightChanged;
    };
    DecorationsOverviewRuler.prototype.onZonesChanged = function (e) {
        return true;
    };
    DecorationsOverviewRuler.prototype.onThemeChanged = function (e) {
        // invalidate color cache
        this._context.model.invalidateOverviewRulerColorCache();
        return this._updateSettings(false);
    };
    // ---- end view event handlers
    DecorationsOverviewRuler.prototype.getDomNode = function () {
        return this._domNode.domNode;
    };
    DecorationsOverviewRuler.prototype.prepareRender = function (ctx) {
        // Nothing to read
    };
    DecorationsOverviewRuler.prototype.render = function (editorCtx) {
        this._render();
    };
    DecorationsOverviewRuler.prototype._render = function () {
        var canvasWidth = this._settings.canvasWidth;
        var canvasHeight = this._settings.canvasHeight;
        var lineHeight = this._settings.lineHeight;
        var viewLayout = this._context.viewLayout;
        var outerHeight = this._context.viewLayout.getScrollHeight();
        var heightRatio = canvasHeight / outerHeight;
        var decorations = this._context.model.getAllOverviewRulerDecorations(this._context.theme);
        var minDecorationHeight = (6 /* MIN_DECORATION_HEIGHT */ * this._settings.pixelRatio) | 0;
        var halfMinDecorationHeight = (minDecorationHeight / 2) | 0;
        var canvasCtx = this._domNode.domNode.getContext('2d');
        if (this._settings.backgroundColor === null) {
            canvasCtx.clearRect(0, 0, canvasWidth, canvasHeight);
        }
        else {
            canvasCtx.fillStyle = this._settings.backgroundColor;
            canvasCtx.fillRect(0, 0, canvasWidth, canvasHeight);
        }
        var x = this._settings.x;
        var w = this._settings.w;
        // Avoid flickering by always rendering the colors in the same order
        // colors that don't use transparency will be sorted last (they start with #)
        var colors = Object.keys(decorations);
        colors.sort();
        for (var cIndex = 0, cLen = colors.length; cIndex < cLen; cIndex++) {
            var color = colors[cIndex];
            var colorDecorations = decorations[color];
            canvasCtx.fillStyle = color;
            var prevLane = 0;
            var prevY1 = 0;
            var prevY2 = 0;
            for (var i = 0, len = colorDecorations.length; i < len; i++) {
                var lane = colorDecorations[3 * i];
                var startLineNumber = colorDecorations[3 * i + 1];
                var endLineNumber = colorDecorations[3 * i + 2];
                var y1 = (viewLayout.getVerticalOffsetForLineNumber(startLineNumber) * heightRatio) | 0;
                var y2 = ((viewLayout.getVerticalOffsetForLineNumber(endLineNumber) + lineHeight) * heightRatio) | 0;
                var height = y2 - y1;
                if (height < minDecorationHeight) {
                    var yCenter = ((y1 + y2) / 2) | 0;
                    if (yCenter < halfMinDecorationHeight) {
                        yCenter = halfMinDecorationHeight;
                    }
                    else if (yCenter + halfMinDecorationHeight > canvasHeight) {
                        yCenter = canvasHeight - halfMinDecorationHeight;
                    }
                    y1 = yCenter - halfMinDecorationHeight;
                    y2 = yCenter + halfMinDecorationHeight;
                }
                if (y1 > prevY2 + 1 || lane !== prevLane) {
                    // flush prev
                    if (i !== 0) {
                        canvasCtx.fillRect(x[prevLane], prevY1, w[prevLane], prevY2 - prevY1);
                    }
                    prevLane = lane;
                    prevY1 = y1;
                    prevY2 = y2;
                }
                else {
                    // merge into prev
                    if (y2 > prevY2) {
                        prevY2 = y2;
                    }
                }
            }
            canvasCtx.fillRect(x[prevLane], prevY1, w[prevLane], prevY2 - prevY1);
        }
        // Draw cursors
        if (!this._settings.hideCursor && this._settings.cursorColor) {
            var cursorHeight = (2 * this._settings.pixelRatio) | 0;
            var halfCursorHeight = (cursorHeight / 2) | 0;
            var cursorX = this._settings.x[7 /* Full */];
            var cursorW = this._settings.w[7 /* Full */];
            canvasCtx.fillStyle = this._settings.cursorColor;
            var prevY1 = -100;
            var prevY2 = -100;
            for (var i = 0, len = this._cursorPositions.length; i < len; i++) {
                var cursor = this._cursorPositions[i];
                var yCenter = (viewLayout.getVerticalOffsetForLineNumber(cursor.lineNumber) * heightRatio) | 0;
                if (yCenter < halfCursorHeight) {
                    yCenter = halfCursorHeight;
                }
                else if (yCenter + halfCursorHeight > canvasHeight) {
                    yCenter = canvasHeight - halfCursorHeight;
                }
                var y1 = yCenter - halfCursorHeight;
                var y2 = y1 + cursorHeight;
                if (y1 > prevY2 + 1) {
                    // flush prev
                    if (i !== 0) {
                        canvasCtx.fillRect(cursorX, prevY1, cursorW, prevY2 - prevY1);
                    }
                    prevY1 = y1;
                    prevY2 = y2;
                }
                else {
                    // merge into prev
                    if (y2 > prevY2) {
                        prevY2 = y2;
                    }
                }
            }
            canvasCtx.fillRect(cursorX, prevY1, cursorW, prevY2 - prevY1);
        }
        if (this._settings.renderBorder && this._settings.borderColor && this._settings.overviewRulerLanes > 0) {
            canvasCtx.beginPath();
            canvasCtx.lineWidth = 1;
            canvasCtx.strokeStyle = this._settings.borderColor;
            canvasCtx.moveTo(0, 0);
            canvasCtx.lineTo(0, canvasHeight);
            canvasCtx.stroke();
            canvasCtx.moveTo(0, 0);
            canvasCtx.lineTo(canvasWidth, 0);
            canvasCtx.stroke();
        }
    };
    return DecorationsOverviewRuler;
}(ViewPart));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ColorZone = /** @class */ (function () {
    function ColorZone(from, to, colorId) {
        this.from = from | 0;
        this.to = to | 0;
        this.colorId = colorId | 0;
    }
    ColorZone.compare = function (a, b) {
        if (a.colorId === b.colorId) {
            if (a.from === b.from) {
                return a.to - b.to;
            }
            return a.from - b.from;
        }
        return a.colorId - b.colorId;
    };
    return ColorZone;
}());
/**
 * A zone in the overview ruler
 */
var OverviewRulerZone = /** @class */ (function () {
    function OverviewRulerZone(startLineNumber, endLineNumber, color) {
        this.startLineNumber = startLineNumber;
        this.endLineNumber = endLineNumber;
        this.color = color;
        this._colorZone = null;
    }
    OverviewRulerZone.compare = function (a, b) {
        if (a.color === b.color) {
            if (a.startLineNumber === b.startLineNumber) {
                return a.endLineNumber - b.endLineNumber;
            }
            return a.startLineNumber - b.startLineNumber;
        }
        return a.color < b.color ? -1 : 1;
    };
    OverviewRulerZone.prototype.setColorZone = function (colorZone) {
        this._colorZone = colorZone;
    };
    OverviewRulerZone.prototype.getColorZones = function () {
        return this._colorZone;
    };
    return OverviewRulerZone;
}());
var OverviewZoneManager = /** @class */ (function () {
    function OverviewZoneManager(getVerticalOffsetForLine) {
        this._getVerticalOffsetForLine = getVerticalOffsetForLine;
        this._zones = [];
        this._colorZonesInvalid = false;
        this._lineHeight = 0;
        this._domWidth = 0;
        this._domHeight = 0;
        this._outerHeight = 0;
        this._pixelRatio = 1;
        this._lastAssignedId = 0;
        this._color2Id = Object.create(null);
        this._id2Color = [];
    }
    OverviewZoneManager.prototype.getId2Color = function () {
        return this._id2Color;
    };
    OverviewZoneManager.prototype.setZones = function (newZones) {
        this._zones = newZones;
        this._zones.sort(OverviewRulerZone.compare);
    };
    OverviewZoneManager.prototype.setLineHeight = function (lineHeight) {
        if (this._lineHeight === lineHeight) {
            return false;
        }
        this._lineHeight = lineHeight;
        this._colorZonesInvalid = true;
        return true;
    };
    OverviewZoneManager.prototype.setPixelRatio = function (pixelRatio) {
        this._pixelRatio = pixelRatio;
        this._colorZonesInvalid = true;
    };
    OverviewZoneManager.prototype.getDOMWidth = function () {
        return this._domWidth;
    };
    OverviewZoneManager.prototype.getCanvasWidth = function () {
        return this._domWidth * this._pixelRatio;
    };
    OverviewZoneManager.prototype.setDOMWidth = function (width) {
        if (this._domWidth === width) {
            return false;
        }
        this._domWidth = width;
        this._colorZonesInvalid = true;
        return true;
    };
    OverviewZoneManager.prototype.getDOMHeight = function () {
        return this._domHeight;
    };
    OverviewZoneManager.prototype.getCanvasHeight = function () {
        return this._domHeight * this._pixelRatio;
    };
    OverviewZoneManager.prototype.setDOMHeight = function (height) {
        if (this._domHeight === height) {
            return false;
        }
        this._domHeight = height;
        this._colorZonesInvalid = true;
        return true;
    };
    OverviewZoneManager.prototype.getOuterHeight = function () {
        return this._outerHeight;
    };
    OverviewZoneManager.prototype.setOuterHeight = function (outerHeight) {
        if (this._outerHeight === outerHeight) {
            return false;
        }
        this._outerHeight = outerHeight;
        this._colorZonesInvalid = true;
        return true;
    };
    OverviewZoneManager.prototype.resolveColorZones = function () {
        var colorZonesInvalid = this._colorZonesInvalid;
        var lineHeight = Math.floor(this._lineHeight); // @perf
        var totalHeight = Math.floor(this.getCanvasHeight()); // @perf
        var outerHeight = Math.floor(this._outerHeight); // @perf
        var heightRatio = totalHeight / outerHeight;
        var halfMinimumHeight = Math.floor(4 /* MINIMUM_HEIGHT */ * this._pixelRatio / 2);
        var allColorZones = [];
        for (var i = 0, len = this._zones.length; i < len; i++) {
            var zone = this._zones[i];
            if (!colorZonesInvalid) {
                var colorZone_1 = zone.getColorZones();
                if (colorZone_1) {
                    allColorZones.push(colorZone_1);
                    continue;
                }
            }
            var y1 = Math.floor(heightRatio * (this._getVerticalOffsetForLine(zone.startLineNumber)));
            var y2 = Math.floor(heightRatio * (this._getVerticalOffsetForLine(zone.endLineNumber) + lineHeight));
            var ycenter = Math.floor((y1 + y2) / 2);
            var halfHeight = (y2 - ycenter);
            if (halfHeight < halfMinimumHeight) {
                halfHeight = halfMinimumHeight;
            }
            if (ycenter - halfHeight < 0) {
                ycenter = halfHeight;
            }
            if (ycenter + halfHeight > totalHeight) {
                ycenter = totalHeight - halfHeight;
            }
            var color = zone.color;
            var colorId = this._color2Id[color];
            if (!colorId) {
                colorId = (++this._lastAssignedId);
                this._color2Id[color] = colorId;
                this._id2Color[colorId] = color;
            }
            var colorZone = new ColorZone(ycenter - halfHeight, ycenter + halfHeight, colorId);
            zone.setColorZone(colorZone);
            allColorZones.push(colorZone);
        }
        this._colorZonesInvalid = false;
        allColorZones.sort(ColorZone.compare);
        return allColorZones;
    };
    return OverviewZoneManager;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$V = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var OverviewRuler = /** @class */ (function (_super) {
    __extends$V(OverviewRuler, _super);
    function OverviewRuler(context, cssClassName) {
        var _this = _super.call(this) || this;
        _this._context = context;
        _this._domNode = createFastDomNode(document.createElement('canvas'));
        _this._domNode.setClassName(cssClassName);
        _this._domNode.setPosition('absolute');
        _this._domNode.setLayerHinting(true);
        _this._zoneManager = new OverviewZoneManager(function (lineNumber) { return _this._context.viewLayout.getVerticalOffsetForLineNumber(lineNumber); });
        _this._zoneManager.setDOMWidth(0);
        _this._zoneManager.setDOMHeight(0);
        _this._zoneManager.setOuterHeight(_this._context.viewLayout.getScrollHeight());
        _this._zoneManager.setLineHeight(_this._context.configuration.editor.lineHeight);
        _this._zoneManager.setPixelRatio(_this._context.configuration.editor.pixelRatio);
        _this._context.addEventHandler(_this);
        return _this;
    }
    OverviewRuler.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        _super.prototype.dispose.call(this);
    };
    // ---- begin view event handlers
    OverviewRuler.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._zoneManager.setLineHeight(this._context.configuration.editor.lineHeight);
            this._render();
        }
        if (e.pixelRatio) {
            this._zoneManager.setPixelRatio(this._context.configuration.editor.pixelRatio);
            this._domNode.setWidth(this._zoneManager.getDOMWidth());
            this._domNode.setHeight(this._zoneManager.getDOMHeight());
            this._domNode.domNode.width = this._zoneManager.getCanvasWidth();
            this._domNode.domNode.height = this._zoneManager.getCanvasHeight();
            this._render();
        }
        return true;
    };
    OverviewRuler.prototype.onFlushed = function (e) {
        this._render();
        return true;
    };
    OverviewRuler.prototype.onScrollChanged = function (e) {
        if (e.scrollHeightChanged) {
            this._zoneManager.setOuterHeight(e.scrollHeight);
            this._render();
        }
        return true;
    };
    OverviewRuler.prototype.onZonesChanged = function (e) {
        this._render();
        return true;
    };
    // ---- end view event handlers
    OverviewRuler.prototype.getDomNode = function () {
        return this._domNode.domNode;
    };
    OverviewRuler.prototype.setLayout = function (position) {
        this._domNode.setTop(position.top);
        this._domNode.setRight(position.right);
        var hasChanged = false;
        hasChanged = this._zoneManager.setDOMWidth(position.width) || hasChanged;
        hasChanged = this._zoneManager.setDOMHeight(position.height) || hasChanged;
        if (hasChanged) {
            this._domNode.setWidth(this._zoneManager.getDOMWidth());
            this._domNode.setHeight(this._zoneManager.getDOMHeight());
            this._domNode.domNode.width = this._zoneManager.getCanvasWidth();
            this._domNode.domNode.height = this._zoneManager.getCanvasHeight();
            this._render();
        }
    };
    OverviewRuler.prototype.setZones = function (zones) {
        this._zoneManager.setZones(zones);
        this._render();
    };
    OverviewRuler.prototype._render = function () {
        if (this._zoneManager.getOuterHeight() === 0) {
            return false;
        }
        var width = this._zoneManager.getCanvasWidth();
        var height = this._zoneManager.getCanvasHeight();
        var colorZones = this._zoneManager.resolveColorZones();
        var id2Color = this._zoneManager.getId2Color();
        var ctx = this._domNode.domNode.getContext('2d');
        ctx.clearRect(0, 0, width, height);
        if (colorZones.length > 0) {
            this._renderOneLane(ctx, colorZones, id2Color, width);
        }
        return true;
    };
    OverviewRuler.prototype._renderOneLane = function (ctx, colorZones, id2Color, width) {
        var currentColorId = 0;
        var currentFrom = 0;
        var currentTo = 0;
        for (var i = 0, len = colorZones.length; i < len; i++) {
            var zone = colorZones[i];
            var zoneColorId = zone.colorId;
            var zoneFrom = zone.from;
            var zoneTo = zone.to;
            if (zoneColorId !== currentColorId) {
                ctx.fillRect(0, currentFrom, width, currentTo - currentFrom);
                currentColorId = zoneColorId;
                ctx.fillStyle = id2Color[currentColorId];
                currentFrom = zoneFrom;
                currentTo = zoneTo;
            }
            else {
                if (currentTo >= zoneFrom) {
                    currentTo = Math.max(currentTo, zoneTo);
                }
                else {
                    ctx.fillRect(0, currentFrom, width, currentTo - currentFrom);
                    currentFrom = zoneFrom;
                    currentTo = zoneTo;
                }
            }
        }
        ctx.fillRect(0, currentFrom, width, currentTo - currentFrom);
    };
    return OverviewRuler;
}(ViewEventHandler));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$W = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Rulers = /** @class */ (function (_super) {
    __extends$W(Rulers, _super);
    function Rulers(context) {
        var _this = _super.call(this, context) || this;
        _this.domNode = createFastDomNode(document.createElement('div'));
        _this.domNode.setAttribute('role', 'presentation');
        _this.domNode.setAttribute('aria-hidden', 'true');
        _this.domNode.setClassName('view-rulers');
        _this._renderedRulers = [];
        _this._rulers = _this._context.configuration.editor.viewInfo.rulers;
        _this._typicalHalfwidthCharacterWidth = _this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        return _this;
    }
    Rulers.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    Rulers.prototype.onConfigurationChanged = function (e) {
        if (e.viewInfo || e.layoutInfo || e.fontInfo) {
            this._rulers = this._context.configuration.editor.viewInfo.rulers;
            this._typicalHalfwidthCharacterWidth = this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
            return true;
        }
        return false;
    };
    Rulers.prototype.onScrollChanged = function (e) {
        return e.scrollHeightChanged;
    };
    // --- end event handlers
    Rulers.prototype.prepareRender = function (ctx) {
        // Nothing to read
    };
    Rulers.prototype._ensureRulersCount = function () {
        var currentCount = this._renderedRulers.length;
        var desiredCount = this._rulers.length;
        if (currentCount === desiredCount) {
            // Nothing to do
            return;
        }
        if (currentCount < desiredCount) {
            var rulerWidth = this._context.model.getTabSize();
            var addCount = desiredCount - currentCount;
            while (addCount > 0) {
                var node = createFastDomNode(document.createElement('div'));
                node.setClassName('view-ruler');
                node.setWidth(rulerWidth);
                this.domNode.appendChild(node);
                this._renderedRulers.push(node);
                addCount--;
            }
            return;
        }
        var removeCount = currentCount - desiredCount;
        while (removeCount > 0) {
            var node = this._renderedRulers.pop();
            this.domNode.removeChild(node);
            removeCount--;
        }
    };
    Rulers.prototype.render = function (ctx) {
        this._ensureRulersCount();
        for (var i = 0, len = this._rulers.length; i < len; i++) {
            var node = this._renderedRulers[i];
            node.setHeight(Math.min(ctx.scrollHeight, 1000000));
            node.setLeft(this._rulers[i] * this._typicalHalfwidthCharacterWidth);
        }
    };
    return Rulers;
}(ViewPart));
registerThemingParticipant(function (theme, collector) {
    var rulerColor = theme.getColor(editorRuler);
    if (rulerColor) {
        collector.addRule(".monaco-editor .view-ruler { box-shadow: 1px 0 0 0 " + rulerColor + " inset; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$X = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ScrollDecorationViewPart = /** @class */ (function (_super) {
    __extends$X(ScrollDecorationViewPart, _super);
    function ScrollDecorationViewPart(context) {
        var _this = _super.call(this, context) || this;
        _this._scrollTop = 0;
        _this._width = 0;
        _this._updateWidth();
        _this._shouldShow = false;
        _this._useShadows = _this._context.configuration.editor.viewInfo.scrollbar.useShadows;
        _this._domNode = createFastDomNode(document.createElement('div'));
        _this._domNode.setAttribute('role', 'presentation');
        _this._domNode.setAttribute('aria-hidden', 'true');
        return _this;
    }
    ScrollDecorationViewPart.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    ScrollDecorationViewPart.prototype._updateShouldShow = function () {
        var newShouldShow = (this._useShadows && this._scrollTop > 0);
        if (this._shouldShow !== newShouldShow) {
            this._shouldShow = newShouldShow;
            return true;
        }
        return false;
    };
    ScrollDecorationViewPart.prototype.getDomNode = function () {
        return this._domNode;
    };
    ScrollDecorationViewPart.prototype._updateWidth = function () {
        var layoutInfo = this._context.configuration.editor.layoutInfo;
        var newWidth = 0;
        if (layoutInfo.renderMinimap === 0 || (layoutInfo.minimapWidth > 0 && layoutInfo.minimapLeft === 0)) {
            newWidth = layoutInfo.width;
        }
        else {
            newWidth = layoutInfo.width - layoutInfo.minimapWidth - layoutInfo.verticalScrollbarWidth;
        }
        if (this._width !== newWidth) {
            this._width = newWidth;
            return true;
        }
        return false;
    };
    // --- begin event handlers
    ScrollDecorationViewPart.prototype.onConfigurationChanged = function (e) {
        var shouldRender = false;
        if (e.viewInfo) {
            this._useShadows = this._context.configuration.editor.viewInfo.scrollbar.useShadows;
        }
        if (e.layoutInfo) {
            shouldRender = this._updateWidth();
        }
        return this._updateShouldShow() || shouldRender;
    };
    ScrollDecorationViewPart.prototype.onScrollChanged = function (e) {
        this._scrollTop = e.scrollTop;
        return this._updateShouldShow();
    };
    // --- end event handlers
    ScrollDecorationViewPart.prototype.prepareRender = function (ctx) {
        // Nothing to read
    };
    ScrollDecorationViewPart.prototype.render = function (ctx) {
        this._domNode.setWidth(this._width);
        this._domNode.setClassName(this._shouldShow ? 'scroll-decoration' : '');
    };
    return ScrollDecorationViewPart;
}(ViewPart));
registerThemingParticipant(function (theme, collector) {
    var shadow = theme.getColor(scrollbarShadow);
    if (shadow) {
        collector.addRule(".monaco-editor .scroll-decoration { box-shadow: " + shadow + " 0 6px 6px -6px inset; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$Y = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var HorizontalRangeWithStyle = /** @class */ (function () {
    function HorizontalRangeWithStyle(other) {
        this.left = other.left;
        this.width = other.width;
        this.startStyle = null;
        this.endStyle = null;
    }
    return HorizontalRangeWithStyle;
}());
var LineVisibleRangesWithStyle = /** @class */ (function () {
    function LineVisibleRangesWithStyle(lineNumber, ranges) {
        this.lineNumber = lineNumber;
        this.ranges = ranges;
    }
    return LineVisibleRangesWithStyle;
}());
function toStyledRange(item) {
    return new HorizontalRangeWithStyle(item);
}
function toStyled(item) {
    return new LineVisibleRangesWithStyle(item.lineNumber, item.ranges.map(toStyledRange));
}
// TODO@Alex: Remove this once IE11 fixes Bug #524217
// The problem in IE11 is that it does some sort of auto-zooming to accomodate for displays with different pixel density.
// Unfortunately, this auto-zooming is buggy around dealing with rounded borders
var isIEWithZoomingIssuesNearRoundedBorders = isEdgeOrIE;
var SelectionsOverlay = /** @class */ (function (_super) {
    __extends$Y(SelectionsOverlay, _super);
    function SelectionsOverlay(context) {
        var _this = _super.call(this) || this;
        _this._previousFrameVisibleRangesWithStyle = [];
        _this._context = context;
        _this._lineHeight = _this._context.configuration.editor.lineHeight;
        _this._roundedSelection = _this._context.configuration.editor.viewInfo.roundedSelection;
        _this._typicalHalfwidthCharacterWidth = _this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        _this._selections = [];
        _this._renderResult = null;
        _this._context.addEventHandler(_this);
        return _this;
    }
    SelectionsOverlay.prototype.dispose = function () {
        this._context.removeEventHandler(this);
        this._renderResult = null;
        _super.prototype.dispose.call(this);
    };
    // --- begin event handlers
    SelectionsOverlay.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.viewInfo) {
            this._roundedSelection = this._context.configuration.editor.viewInfo.roundedSelection;
        }
        if (e.fontInfo) {
            this._typicalHalfwidthCharacterWidth = this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        }
        return true;
    };
    SelectionsOverlay.prototype.onCursorStateChanged = function (e) {
        this._selections = e.selections.slice(0);
        return true;
    };
    SelectionsOverlay.prototype.onDecorationsChanged = function (e) {
        // true for inline decorations that can end up relayouting text
        return true; //e.inlineDecorationsChanged;
    };
    SelectionsOverlay.prototype.onFlushed = function (e) {
        return true;
    };
    SelectionsOverlay.prototype.onLinesChanged = function (e) {
        return true;
    };
    SelectionsOverlay.prototype.onLinesDeleted = function (e) {
        return true;
    };
    SelectionsOverlay.prototype.onLinesInserted = function (e) {
        return true;
    };
    SelectionsOverlay.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged;
    };
    SelectionsOverlay.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    SelectionsOverlay.prototype._visibleRangesHaveGaps = function (linesVisibleRanges) {
        for (var i = 0, len = linesVisibleRanges.length; i < len; i++) {
            var lineVisibleRanges = linesVisibleRanges[i];
            if (lineVisibleRanges.ranges.length > 1) {
                // There are two ranges on the same line
                return true;
            }
        }
        return false;
    };
    SelectionsOverlay.prototype._enrichVisibleRangesWithStyle = function (viewport, linesVisibleRanges, previousFrame) {
        var epsilon = this._typicalHalfwidthCharacterWidth / 4;
        var previousFrameTop = null;
        var previousFrameBottom = null;
        if (previousFrame && previousFrame.length > 0 && linesVisibleRanges.length > 0) {
            var topLineNumber = linesVisibleRanges[0].lineNumber;
            if (topLineNumber === viewport.startLineNumber) {
                for (var i = 0; !previousFrameTop && i < previousFrame.length; i++) {
                    if (previousFrame[i].lineNumber === topLineNumber) {
                        previousFrameTop = previousFrame[i].ranges[0];
                    }
                }
            }
            var bottomLineNumber = linesVisibleRanges[linesVisibleRanges.length - 1].lineNumber;
            if (bottomLineNumber === viewport.endLineNumber) {
                for (var i = previousFrame.length - 1; !previousFrameBottom && i >= 0; i--) {
                    if (previousFrame[i].lineNumber === bottomLineNumber) {
                        previousFrameBottom = previousFrame[i].ranges[0];
                    }
                }
            }
            if (previousFrameTop && !previousFrameTop.startStyle) {
                previousFrameTop = null;
            }
            if (previousFrameBottom && !previousFrameBottom.startStyle) {
                previousFrameBottom = null;
            }
        }
        for (var i = 0, len = linesVisibleRanges.length; i < len; i++) {
            // We know for a fact that there is precisely one range on each line
            var curLineRange = linesVisibleRanges[i].ranges[0];
            var curLeft = curLineRange.left;
            var curRight = curLineRange.left + curLineRange.width;
            var startStyle = {
                top: 0 /* EXTERN */,
                bottom: 0 /* EXTERN */
            };
            var endStyle = {
                top: 0 /* EXTERN */,
                bottom: 0 /* EXTERN */
            };
            if (i > 0) {
                // Look above
                var prevLeft = linesVisibleRanges[i - 1].ranges[0].left;
                var prevRight = linesVisibleRanges[i - 1].ranges[0].left + linesVisibleRanges[i - 1].ranges[0].width;
                if (abs(curLeft - prevLeft) < epsilon) {
                    startStyle.top = 2 /* FLAT */;
                }
                else if (curLeft > prevLeft) {
                    startStyle.top = 1 /* INTERN */;
                }
                if (abs(curRight - prevRight) < epsilon) {
                    endStyle.top = 2 /* FLAT */;
                }
                else if (prevLeft < curRight && curRight < prevRight) {
                    endStyle.top = 1 /* INTERN */;
                }
            }
            else if (previousFrameTop) {
                // Accept some hick-ups near the viewport edges to save on repaints
                startStyle.top = previousFrameTop.startStyle.top;
                endStyle.top = previousFrameTop.endStyle.top;
            }
            if (i + 1 < len) {
                // Look below
                var nextLeft = linesVisibleRanges[i + 1].ranges[0].left;
                var nextRight = linesVisibleRanges[i + 1].ranges[0].left + linesVisibleRanges[i + 1].ranges[0].width;
                if (abs(curLeft - nextLeft) < epsilon) {
                    startStyle.bottom = 2 /* FLAT */;
                }
                else if (nextLeft < curLeft && curLeft < nextRight) {
                    startStyle.bottom = 1 /* INTERN */;
                }
                if (abs(curRight - nextRight) < epsilon) {
                    endStyle.bottom = 2 /* FLAT */;
                }
                else if (curRight < nextRight) {
                    endStyle.bottom = 1 /* INTERN */;
                }
            }
            else if (previousFrameBottom) {
                // Accept some hick-ups near the viewport edges to save on repaints
                startStyle.bottom = previousFrameBottom.startStyle.bottom;
                endStyle.bottom = previousFrameBottom.endStyle.bottom;
            }
            curLineRange.startStyle = startStyle;
            curLineRange.endStyle = endStyle;
        }
    };
    SelectionsOverlay.prototype._getVisibleRangesWithStyle = function (selection, ctx, previousFrame) {
        var _linesVisibleRanges = ctx.linesVisibleRangesForRange(selection, true) || [];
        var linesVisibleRanges = _linesVisibleRanges.map(toStyled);
        var visibleRangesHaveGaps = this._visibleRangesHaveGaps(linesVisibleRanges);
        if (!isIEWithZoomingIssuesNearRoundedBorders && !visibleRangesHaveGaps && this._roundedSelection) {
            this._enrichVisibleRangesWithStyle(ctx.visibleRange, linesVisibleRanges, previousFrame);
        }
        // The visible ranges are sorted TOP-BOTTOM and LEFT-RIGHT
        return linesVisibleRanges;
    };
    SelectionsOverlay.prototype._createSelectionPiece = function (top, height, className, left, width) {
        return ('<div class="cslr '
            + className
            + '" style="top:'
            + top.toString()
            + 'px;left:'
            + left.toString()
            + 'px;width:'
            + width.toString()
            + 'px;height:'
            + height
            + 'px;"></div>');
    };
    SelectionsOverlay.prototype._actualRenderOneSelection = function (output2, visibleStartLineNumber, hasMultipleSelections, visibleRanges) {
        var visibleRangesHaveStyle = (visibleRanges.length > 0 && visibleRanges[0].ranges[0].startStyle);
        var fullLineHeight = (this._lineHeight).toString();
        var reducedLineHeight = (this._lineHeight - 1).toString();
        var firstLineNumber = (visibleRanges.length > 0 ? visibleRanges[0].lineNumber : 0);
        var lastLineNumber = (visibleRanges.length > 0 ? visibleRanges[visibleRanges.length - 1].lineNumber : 0);
        for (var i = 0, len = visibleRanges.length; i < len; i++) {
            var lineVisibleRanges = visibleRanges[i];
            var lineNumber = lineVisibleRanges.lineNumber;
            var lineIndex = lineNumber - visibleStartLineNumber;
            var lineHeight = hasMultipleSelections ? (lineNumber === lastLineNumber || lineNumber === firstLineNumber ? reducedLineHeight : fullLineHeight) : fullLineHeight;
            var top_1 = hasMultipleSelections ? (lineNumber === firstLineNumber ? 1 : 0) : 0;
            var lineOutput = '';
            for (var j = 0, lenJ = lineVisibleRanges.ranges.length; j < lenJ; j++) {
                var visibleRange = lineVisibleRanges.ranges[j];
                if (visibleRangesHaveStyle) {
                    var startStyle = visibleRange.startStyle;
                    var endStyle = visibleRange.endStyle;
                    if (startStyle.top === 1 /* INTERN */ || startStyle.bottom === 1 /* INTERN */) {
                        // Reverse rounded corner to the left
                        // First comes the selection (blue layer)
                        lineOutput += this._createSelectionPiece(top_1, lineHeight, SelectionsOverlay.SELECTION_CLASS_NAME, visibleRange.left - SelectionsOverlay.ROUNDED_PIECE_WIDTH, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                        // Second comes the background (white layer) with inverse border radius
                        var className_1 = SelectionsOverlay.EDITOR_BACKGROUND_CLASS_NAME;
                        if (startStyle.top === 1 /* INTERN */) {
                            className_1 += ' ' + SelectionsOverlay.SELECTION_TOP_RIGHT;
                        }
                        if (startStyle.bottom === 1 /* INTERN */) {
                            className_1 += ' ' + SelectionsOverlay.SELECTION_BOTTOM_RIGHT;
                        }
                        lineOutput += this._createSelectionPiece(top_1, lineHeight, className_1, visibleRange.left - SelectionsOverlay.ROUNDED_PIECE_WIDTH, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                    }
                    if (endStyle.top === 1 /* INTERN */ || endStyle.bottom === 1 /* INTERN */) {
                        // Reverse rounded corner to the right
                        // First comes the selection (blue layer)
                        lineOutput += this._createSelectionPiece(top_1, lineHeight, SelectionsOverlay.SELECTION_CLASS_NAME, visibleRange.left + visibleRange.width, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                        // Second comes the background (white layer) with inverse border radius
                        var className_2 = SelectionsOverlay.EDITOR_BACKGROUND_CLASS_NAME;
                        if (endStyle.top === 1 /* INTERN */) {
                            className_2 += ' ' + SelectionsOverlay.SELECTION_TOP_LEFT;
                        }
                        if (endStyle.bottom === 1 /* INTERN */) {
                            className_2 += ' ' + SelectionsOverlay.SELECTION_BOTTOM_LEFT;
                        }
                        lineOutput += this._createSelectionPiece(top_1, lineHeight, className_2, visibleRange.left + visibleRange.width, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                    }
                }
                var className = SelectionsOverlay.SELECTION_CLASS_NAME;
                if (visibleRangesHaveStyle) {
                    var startStyle = visibleRange.startStyle;
                    var endStyle = visibleRange.endStyle;
                    if (startStyle.top === 0 /* EXTERN */) {
                        className += ' ' + SelectionsOverlay.SELECTION_TOP_LEFT;
                    }
                    if (startStyle.bottom === 0 /* EXTERN */) {
                        className += ' ' + SelectionsOverlay.SELECTION_BOTTOM_LEFT;
                    }
                    if (endStyle.top === 0 /* EXTERN */) {
                        className += ' ' + SelectionsOverlay.SELECTION_TOP_RIGHT;
                    }
                    if (endStyle.bottom === 0 /* EXTERN */) {
                        className += ' ' + SelectionsOverlay.SELECTION_BOTTOM_RIGHT;
                    }
                }
                lineOutput += this._createSelectionPiece(top_1, lineHeight, className, visibleRange.left, visibleRange.width);
            }
            output2[lineIndex] += lineOutput;
        }
    };
    SelectionsOverlay.prototype.prepareRender = function (ctx) {
        var output = [];
        var visibleStartLineNumber = ctx.visibleRange.startLineNumber;
        var visibleEndLineNumber = ctx.visibleRange.endLineNumber;
        for (var lineNumber = visibleStartLineNumber; lineNumber <= visibleEndLineNumber; lineNumber++) {
            var lineIndex = lineNumber - visibleStartLineNumber;
            output[lineIndex] = '';
        }
        var thisFrameVisibleRangesWithStyle = [];
        for (var i = 0, len = this._selections.length; i < len; i++) {
            var selection = this._selections[i];
            if (selection.isEmpty()) {
                thisFrameVisibleRangesWithStyle[i] = null;
                continue;
            }
            var visibleRangesWithStyle = this._getVisibleRangesWithStyle(selection, ctx, this._previousFrameVisibleRangesWithStyle[i]);
            thisFrameVisibleRangesWithStyle[i] = visibleRangesWithStyle;
            this._actualRenderOneSelection(output, visibleStartLineNumber, this._selections.length > 1, visibleRangesWithStyle);
        }
        this._previousFrameVisibleRangesWithStyle = thisFrameVisibleRangesWithStyle;
        this._renderResult = output;
    };
    SelectionsOverlay.prototype.render = function (startLineNumber, lineNumber) {
        if (!this._renderResult) {
            return '';
        }
        var lineIndex = lineNumber - startLineNumber;
        if (lineIndex < 0 || lineIndex >= this._renderResult.length) {
            return '';
        }
        return this._renderResult[lineIndex];
    };
    SelectionsOverlay.SELECTION_CLASS_NAME = 'selected-text';
    SelectionsOverlay.SELECTION_TOP_LEFT = 'top-left-radius';
    SelectionsOverlay.SELECTION_BOTTOM_LEFT = 'bottom-left-radius';
    SelectionsOverlay.SELECTION_TOP_RIGHT = 'top-right-radius';
    SelectionsOverlay.SELECTION_BOTTOM_RIGHT = 'bottom-right-radius';
    SelectionsOverlay.EDITOR_BACKGROUND_CLASS_NAME = 'monaco-editor-background';
    SelectionsOverlay.ROUNDED_PIECE_WIDTH = 10;
    return SelectionsOverlay;
}(DynamicViewOverlay));
registerThemingParticipant(function (theme, collector) {
    var editorSelectionColor = theme.getColor(editorSelectionBackground);
    if (editorSelectionColor) {
        collector.addRule(".monaco-editor .focused .selected-text { background-color: " + editorSelectionColor + "; }");
    }
    var editorInactiveSelectionColor = theme.getColor(editorInactiveSelection);
    if (editorInactiveSelectionColor) {
        collector.addRule(".monaco-editor .selected-text { background-color: " + editorInactiveSelectionColor + "; }");
    }
    var editorSelectionForegroundColor = theme.getColor(editorSelectionForeground);
    if (editorSelectionForegroundColor) {
        collector.addRule(".monaco-editor .view-line span.inline-selected-text { color: " + editorSelectionForegroundColor + "; }");
    }
});
function abs(n) {
    return n < 0 ? -n : n;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ViewCursorRenderData = /** @class */ (function () {
    function ViewCursorRenderData(top, left, width, height, textContent, textContentClassName) {
        this.top = top;
        this.left = left;
        this.width = width;
        this.height = height;
        this.textContent = textContent;
        this.textContentClassName = textContentClassName;
    }
    return ViewCursorRenderData;
}());
var ViewCursor = /** @class */ (function () {
    function ViewCursor(context) {
        this._context = context;
        this._cursorStyle = this._context.configuration.editor.viewInfo.cursorStyle;
        this._lineHeight = this._context.configuration.editor.lineHeight;
        this._typicalHalfwidthCharacterWidth = this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        this._lineCursorWidth = Math.min(this._context.configuration.editor.viewInfo.cursorWidth, this._typicalHalfwidthCharacterWidth);
        this._isVisible = true;
        // Create the dom node
        this._domNode = createFastDomNode(document.createElement('div'));
        this._domNode.setClassName('cursor');
        this._domNode.setHeight(this._lineHeight);
        this._domNode.setTop(0);
        this._domNode.setLeft(0);
        Configuration.applyFontInfo(this._domNode, this._context.configuration.editor.fontInfo);
        this._domNode.setDisplay('none');
        this.updatePosition(new Position(1, 1));
        this._lastRenderedContent = '';
        this._renderData = null;
    }
    ViewCursor.prototype.getDomNode = function () {
        return this._domNode;
    };
    ViewCursor.prototype.getPosition = function () {
        return this._position;
    };
    ViewCursor.prototype.show = function () {
        if (!this._isVisible) {
            this._domNode.setVisibility('inherit');
            this._isVisible = true;
        }
    };
    ViewCursor.prototype.hide = function () {
        if (this._isVisible) {
            this._domNode.setVisibility('hidden');
            this._isVisible = false;
        }
    };
    ViewCursor.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
        }
        if (e.fontInfo) {
            Configuration.applyFontInfo(this._domNode, this._context.configuration.editor.fontInfo);
            this._typicalHalfwidthCharacterWidth = this._context.configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
        }
        if (e.viewInfo) {
            this._cursorStyle = this._context.configuration.editor.viewInfo.cursorStyle;
            this._lineCursorWidth = Math.min(this._context.configuration.editor.viewInfo.cursorWidth, this._typicalHalfwidthCharacterWidth);
        }
        return true;
    };
    ViewCursor.prototype.onCursorPositionChanged = function (position) {
        this.updatePosition(position);
        return true;
    };
    ViewCursor.prototype._prepareRender = function (ctx) {
        var textContent = '';
        var textContentClassName = '';
        if (this._cursorStyle === TextEditorCursorStyle.Line || this._cursorStyle === TextEditorCursorStyle.LineThin) {
            var visibleRange = ctx.visibleRangeForPosition(this._position);
            if (!visibleRange) {
                // Outside viewport
                return null;
            }
            var width_1;
            if (this._cursorStyle === TextEditorCursorStyle.Line) {
                width_1 = computeScreenAwareSize(this._lineCursorWidth > 0 ? this._lineCursorWidth : 2);
                if (width_1 > 2) {
                    var lineContent = this._context.model.getLineContent(this._position.lineNumber);
                    textContent = lineContent.charAt(this._position.column - 1);
                }
            }
            else {
                width_1 = computeScreenAwareSize(1);
            }
            var left = visibleRange.left;
            if (width_1 >= 2 && left >= 1) {
                // try to center cursor
                left -= 1;
            }
            var top_1 = ctx.getVerticalOffsetForLineNumber(this._position.lineNumber) - ctx.bigNumbersDelta;
            return new ViewCursorRenderData(top_1, left, width_1, this._lineHeight, textContent, textContentClassName);
        }
        var visibleRangeForCharacter = ctx.linesVisibleRangesForRange(new Range(this._position.lineNumber, this._position.column, this._position.lineNumber, this._position.column + 1), false);
        if (!visibleRangeForCharacter || visibleRangeForCharacter.length === 0 || visibleRangeForCharacter[0].ranges.length === 0) {
            // Outside viewport
            return null;
        }
        var range = visibleRangeForCharacter[0].ranges[0];
        var width = range.width < 1 ? this._typicalHalfwidthCharacterWidth : range.width;
        if (this._cursorStyle === TextEditorCursorStyle.Block) {
            var lineData = this._context.model.getViewLineData(this._position.lineNumber);
            textContent = lineData.content.charAt(this._position.column - 1);
            if (isHighSurrogate(lineData.content.charCodeAt(this._position.column - 1))) {
                textContent += lineData.content.charAt(this._position.column);
            }
            var tokenIndex = lineData.tokens.findTokenIndexAtOffset(this._position.column - 1);
            textContentClassName = lineData.tokens.getClassName(tokenIndex);
        }
        var top = ctx.getVerticalOffsetForLineNumber(this._position.lineNumber) - ctx.bigNumbersDelta;
        var height = this._lineHeight;
        // Underline might interfere with clicking
        if (this._cursorStyle === TextEditorCursorStyle.Underline || this._cursorStyle === TextEditorCursorStyle.UnderlineThin) {
            top += this._lineHeight - 2;
            height = 2;
        }
        return new ViewCursorRenderData(top, range.left, width, height, textContent, textContentClassName);
    };
    ViewCursor.prototype.prepareRender = function (ctx) {
        this._renderData = this._prepareRender(ctx);
    };
    ViewCursor.prototype.render = function (ctx) {
        if (!this._renderData) {
            this._domNode.setDisplay('none');
            return null;
        }
        if (this._lastRenderedContent !== this._renderData.textContent) {
            this._lastRenderedContent = this._renderData.textContent;
            this._domNode.domNode.textContent = this._lastRenderedContent;
        }
        this._domNode.setClassName('cursor ' + this._renderData.textContentClassName);
        this._domNode.setDisplay('block');
        this._domNode.setTop(this._renderData.top);
        this._domNode.setLeft(this._renderData.left);
        this._domNode.setWidth(this._renderData.width);
        this._domNode.setLineHeight(this._renderData.height);
        this._domNode.setHeight(this._renderData.height);
        return {
            domNode: this._domNode.domNode,
            position: this._position,
            contentLeft: this._renderData.left,
            height: this._renderData.height,
            width: 2
        };
    };
    ViewCursor.prototype.updatePosition = function (newPosition) {
        this._position = newPosition;
    };
    return ViewCursor;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$Z = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewCursors = /** @class */ (function (_super) {
    __extends$Z(ViewCursors, _super);
    function ViewCursors(context) {
        var _this = _super.call(this, context) || this;
        _this._readOnly = _this._context.configuration.editor.readOnly;
        _this._cursorBlinking = _this._context.configuration.editor.viewInfo.cursorBlinking;
        _this._cursorStyle = _this._context.configuration.editor.viewInfo.cursorStyle;
        _this._selectionIsEmpty = true;
        _this._primaryCursor = new ViewCursor(_this._context);
        _this._secondaryCursors = [];
        _this._renderData = [];
        _this._domNode = createFastDomNode(document.createElement('div'));
        _this._domNode.setAttribute('role', 'presentation');
        _this._domNode.setAttribute('aria-hidden', 'true');
        _this._updateDomClassName();
        _this._domNode.appendChild(_this._primaryCursor.getDomNode());
        _this._startCursorBlinkAnimation = new TimeoutTimer();
        _this._cursorFlatBlinkInterval = new IntervalTimer();
        _this._blinkingEnabled = false;
        _this._editorHasFocus = false;
        _this._updateBlinking();
        return _this;
    }
    ViewCursors.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this._startCursorBlinkAnimation.dispose();
        this._cursorFlatBlinkInterval.dispose();
    };
    ViewCursors.prototype.getDomNode = function () {
        return this._domNode;
    };
    // --- begin event handlers
    ViewCursors.prototype.onConfigurationChanged = function (e) {
        if (e.readOnly) {
            this._readOnly = this._context.configuration.editor.readOnly;
        }
        if (e.viewInfo) {
            this._cursorBlinking = this._context.configuration.editor.viewInfo.cursorBlinking;
            this._cursorStyle = this._context.configuration.editor.viewInfo.cursorStyle;
        }
        this._primaryCursor.onConfigurationChanged(e);
        this._updateBlinking();
        if (e.viewInfo) {
            this._updateDomClassName();
        }
        for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
            this._secondaryCursors[i].onConfigurationChanged(e);
        }
        return true;
    };
    ViewCursors.prototype._onCursorPositionChanged = function (position, secondaryPositions) {
        this._primaryCursor.onCursorPositionChanged(position);
        this._updateBlinking();
        if (this._secondaryCursors.length < secondaryPositions.length) {
            // Create new cursors
            var addCnt = secondaryPositions.length - this._secondaryCursors.length;
            for (var i = 0; i < addCnt; i++) {
                var newCursor = new ViewCursor(this._context);
                this._domNode.domNode.insertBefore(newCursor.getDomNode().domNode, this._primaryCursor.getDomNode().domNode.nextSibling);
                this._secondaryCursors.push(newCursor);
            }
        }
        else if (this._secondaryCursors.length > secondaryPositions.length) {
            // Remove some cursors
            var removeCnt = this._secondaryCursors.length - secondaryPositions.length;
            for (var i = 0; i < removeCnt; i++) {
                this._domNode.removeChild(this._secondaryCursors[0].getDomNode());
                this._secondaryCursors.splice(0, 1);
            }
        }
        for (var i = 0; i < secondaryPositions.length; i++) {
            this._secondaryCursors[i].onCursorPositionChanged(secondaryPositions[i]);
        }
    };
    ViewCursors.prototype.onCursorStateChanged = function (e) {
        var positions = [];
        for (var i = 0, len = e.selections.length; i < len; i++) {
            positions[i] = e.selections[i].getPosition();
        }
        this._onCursorPositionChanged(positions[0], positions.slice(1));
        var selectionIsEmpty = e.selections[0].isEmpty();
        if (this._selectionIsEmpty !== selectionIsEmpty) {
            this._selectionIsEmpty = selectionIsEmpty;
            this._updateDomClassName();
        }
        return true;
    };
    ViewCursors.prototype.onDecorationsChanged = function (e) {
        // true for inline decorations that can end up relayouting text
        return true;
    };
    ViewCursors.prototype.onFlushed = function (e) {
        return true;
    };
    ViewCursors.prototype.onFocusChanged = function (e) {
        this._editorHasFocus = e.isFocused;
        this._updateBlinking();
        return false;
    };
    ViewCursors.prototype.onLinesChanged = function (e) {
        return true;
    };
    ViewCursors.prototype.onLinesDeleted = function (e) {
        return true;
    };
    ViewCursors.prototype.onLinesInserted = function (e) {
        return true;
    };
    ViewCursors.prototype.onScrollChanged = function (e) {
        return true;
    };
    ViewCursors.prototype.onTokensChanged = function (e) {
        var shouldRender = function (position) {
            for (var i = 0, len = e.ranges.length; i < len; i++) {
                if (e.ranges[i].fromLineNumber <= position.lineNumber && position.lineNumber <= e.ranges[i].toLineNumber) {
                    return true;
                }
            }
            return false;
        };
        if (shouldRender(this._primaryCursor.getPosition())) {
            return true;
        }
        for (var i = 0; i < this._secondaryCursors.length; i++) {
            if (shouldRender(this._secondaryCursors[i].getPosition())) {
                return true;
            }
        }
        return false;
    };
    ViewCursors.prototype.onZonesChanged = function (e) {
        return true;
    };
    // --- end event handlers
    // ---- blinking logic
    ViewCursors.prototype._getCursorBlinking = function () {
        if (!this._editorHasFocus) {
            return 0 /* Hidden */;
        }
        if (this._readOnly) {
            return 5 /* Solid */;
        }
        return this._cursorBlinking;
    };
    ViewCursors.prototype._updateBlinking = function () {
        var _this = this;
        this._startCursorBlinkAnimation.cancel();
        this._cursorFlatBlinkInterval.cancel();
        var blinkingStyle = this._getCursorBlinking();
        // hidden and solid are special as they involve no animations
        var isHidden = (blinkingStyle === 0 /* Hidden */);
        var isSolid = (blinkingStyle === 5 /* Solid */);
        if (isHidden) {
            this._hide();
        }
        else {
            this._show();
        }
        this._blinkingEnabled = false;
        this._updateDomClassName();
        if (!isHidden && !isSolid) {
            if (blinkingStyle === 1 /* Blink */) {
                // flat blinking is handled by JavaScript to save battery life due to Chromium step timing issue https://bugs.chromium.org/p/chromium/issues/detail?id=361587
                this._cursorFlatBlinkInterval.cancelAndSet(function () {
                    if (_this._isVisible) {
                        _this._hide();
                    }
                    else {
                        _this._show();
                    }
                }, ViewCursors.BLINK_INTERVAL);
            }
            else {
                this._startCursorBlinkAnimation.setIfNotSet(function () {
                    _this._blinkingEnabled = true;
                    _this._updateDomClassName();
                }, ViewCursors.BLINK_INTERVAL);
            }
        }
    };
    // --- end blinking logic
    ViewCursors.prototype._updateDomClassName = function () {
        this._domNode.setClassName(this._getClassName());
    };
    ViewCursors.prototype._getClassName = function () {
        var result = 'cursors-layer';
        if (!this._selectionIsEmpty) {
            result += ' has-selection';
        }
        switch (this._cursorStyle) {
            case TextEditorCursorStyle.Line:
                result += ' cursor-line-style';
                break;
            case TextEditorCursorStyle.Block:
                result += ' cursor-block-style';
                break;
            case TextEditorCursorStyle.Underline:
                result += ' cursor-underline-style';
                break;
            case TextEditorCursorStyle.LineThin:
                result += ' cursor-line-thin-style';
                break;
            case TextEditorCursorStyle.BlockOutline:
                result += ' cursor-block-outline-style';
                break;
            case TextEditorCursorStyle.UnderlineThin:
                result += ' cursor-underline-thin-style';
                break;
            default:
                result += ' cursor-line-style';
        }
        if (this._blinkingEnabled) {
            switch (this._getCursorBlinking()) {
                case 1 /* Blink */:
                    result += ' cursor-blink';
                    break;
                case 2 /* Smooth */:
                    result += ' cursor-smooth';
                    break;
                case 3 /* Phase */:
                    result += ' cursor-phase';
                    break;
                case 4 /* Expand */:
                    result += ' cursor-expand';
                    break;
                case 5 /* Solid */:
                    result += ' cursor-solid';
                    break;
                default:
                    result += ' cursor-solid';
            }
        }
        else {
            result += ' cursor-solid';
        }
        return result;
    };
    ViewCursors.prototype._show = function () {
        this._primaryCursor.show();
        for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
            this._secondaryCursors[i].show();
        }
        this._isVisible = true;
    };
    ViewCursors.prototype._hide = function () {
        this._primaryCursor.hide();
        for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
            this._secondaryCursors[i].hide();
        }
        this._isVisible = false;
    };
    // ---- IViewPart implementation
    ViewCursors.prototype.prepareRender = function (ctx) {
        this._primaryCursor.prepareRender(ctx);
        for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
            this._secondaryCursors[i].prepareRender(ctx);
        }
    };
    ViewCursors.prototype.render = function (ctx) {
        var renderData = [], renderDataLen = 0;
        var primaryRenderData = this._primaryCursor.render(ctx);
        if (primaryRenderData) {
            renderData[renderDataLen++] = primaryRenderData;
        }
        for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
            var secondaryRenderData = this._secondaryCursors[i].render(ctx);
            if (secondaryRenderData) {
                renderData[renderDataLen++] = secondaryRenderData;
            }
        }
        this._renderData = renderData;
    };
    ViewCursors.prototype.getLastRenderData = function () {
        return this._renderData;
    };
    ViewCursors.BLINK_INTERVAL = 500;
    return ViewCursors;
}(ViewPart));
registerThemingParticipant(function (theme, collector) {
    var caret = theme.getColor(editorCursorForeground);
    if (caret) {
        var caretBackground = theme.getColor(editorCursorBackground);
        if (!caretBackground) {
            caretBackground = caret.opposite();
        }
        collector.addRule(".monaco-editor .cursor { background-color: " + caret + "; border-color: " + caret + "; color: " + caretBackground + "; }");
        if (theme.type === 'hc') {
            collector.addRule(".monaco-editor .cursors-layer.has-selection .cursor { border-left: 1px solid " + caretBackground + "; border-right: 1px solid " + caretBackground + "; }");
        }
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$_ = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewZones = /** @class */ (function (_super) {
    __extends$_(ViewZones, _super);
    function ViewZones(context) {
        var _this = _super.call(this, context) || this;
        _this._lineHeight = _this._context.configuration.editor.lineHeight;
        _this._contentWidth = _this._context.configuration.editor.layoutInfo.contentWidth;
        _this._contentLeft = _this._context.configuration.editor.layoutInfo.contentLeft;
        _this.domNode = createFastDomNode(document.createElement('div'));
        _this.domNode.setClassName('view-zones');
        _this.domNode.setPosition('absolute');
        _this.domNode.setAttribute('role', 'presentation');
        _this.domNode.setAttribute('aria-hidden', 'true');
        _this.marginDomNode = createFastDomNode(document.createElement('div'));
        _this.marginDomNode.setClassName('margin-view-zones');
        _this.marginDomNode.setPosition('absolute');
        _this.marginDomNode.setAttribute('role', 'presentation');
        _this.marginDomNode.setAttribute('aria-hidden', 'true');
        _this._zones = {};
        return _this;
    }
    ViewZones.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this._zones = {};
    };
    // ---- begin view event handlers
    ViewZones.prototype._recomputeWhitespacesProps = function () {
        var hadAChange = false;
        var keys = Object.keys(this._zones);
        for (var i = 0, len = keys.length; i < len; i++) {
            var id = keys[i];
            var zone = this._zones[id];
            var props = this._computeWhitespaceProps(zone.delegate);
            if (this._context.viewLayout.changeWhitespace(parseInt(id, 10), props.afterViewLineNumber, props.heightInPx)) {
                this._safeCallOnComputedHeight(zone.delegate, props.heightInPx);
                hadAChange = true;
            }
        }
        return hadAChange;
    };
    ViewZones.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._lineHeight = this._context.configuration.editor.lineHeight;
            return this._recomputeWhitespacesProps();
        }
        if (e.layoutInfo) {
            this._contentWidth = this._context.configuration.editor.layoutInfo.contentWidth;
            this._contentLeft = this._context.configuration.editor.layoutInfo.contentLeft;
        }
        return true;
    };
    ViewZones.prototype.onLineMappingChanged = function (e) {
        var hadAChange = this._recomputeWhitespacesProps();
        if (hadAChange) {
            this._context.viewLayout.onHeightMaybeChanged();
        }
        return hadAChange;
    };
    ViewZones.prototype.onLinesDeleted = function (e) {
        return true;
    };
    ViewZones.prototype.onScrollChanged = function (e) {
        return e.scrollTopChanged || e.scrollWidthChanged;
    };
    ViewZones.prototype.onZonesChanged = function (e) {
        return true;
    };
    ViewZones.prototype.onLinesInserted = function (e) {
        return true;
    };
    // ---- end view event handlers
    ViewZones.prototype._getZoneOrdinal = function (zone) {
        if (typeof zone.afterColumn !== 'undefined') {
            return zone.afterColumn;
        }
        return 10000;
    };
    ViewZones.prototype._computeWhitespaceProps = function (zone) {
        if (zone.afterLineNumber === 0) {
            return {
                afterViewLineNumber: 0,
                heightInPx: this._heightInPixels(zone),
                minWidthInPx: this._minWidthInPixels(zone)
            };
        }
        var zoneAfterModelPosition;
        if (typeof zone.afterColumn !== 'undefined') {
            zoneAfterModelPosition = this._context.model.validateModelPosition({
                lineNumber: zone.afterLineNumber,
                column: zone.afterColumn
            });
        }
        else {
            var validAfterLineNumber = this._context.model.validateModelPosition({
                lineNumber: zone.afterLineNumber,
                column: 1
            }).lineNumber;
            zoneAfterModelPosition = new Position(validAfterLineNumber, this._context.model.getModelLineMaxColumn(validAfterLineNumber));
        }
        var zoneBeforeModelPosition;
        if (zoneAfterModelPosition.column === this._context.model.getModelLineMaxColumn(zoneAfterModelPosition.lineNumber)) {
            zoneBeforeModelPosition = this._context.model.validateModelPosition({
                lineNumber: zoneAfterModelPosition.lineNumber + 1,
                column: 1
            });
        }
        else {
            zoneBeforeModelPosition = this._context.model.validateModelPosition({
                lineNumber: zoneAfterModelPosition.lineNumber,
                column: zoneAfterModelPosition.column + 1
            });
        }
        var viewPosition = this._context.model.coordinatesConverter.convertModelPositionToViewPosition(zoneAfterModelPosition);
        var isVisible = this._context.model.coordinatesConverter.modelPositionIsVisible(zoneBeforeModelPosition);
        return {
            afterViewLineNumber: viewPosition.lineNumber,
            heightInPx: (isVisible ? this._heightInPixels(zone) : 0),
            minWidthInPx: this._minWidthInPixels(zone)
        };
    };
    ViewZones.prototype.addZone = function (zone) {
        var props = this._computeWhitespaceProps(zone);
        var whitespaceId = this._context.viewLayout.addWhitespace(props.afterViewLineNumber, this._getZoneOrdinal(zone), props.heightInPx, props.minWidthInPx);
        var myZone = {
            whitespaceId: whitespaceId,
            delegate: zone,
            isVisible: false,
            domNode: createFastDomNode(zone.domNode),
            marginDomNode: zone.marginDomNode ? createFastDomNode(zone.marginDomNode) : null
        };
        this._safeCallOnComputedHeight(myZone.delegate, props.heightInPx);
        myZone.domNode.setPosition('absolute');
        myZone.domNode.domNode.style.width = '100%';
        myZone.domNode.setDisplay('none');
        myZone.domNode.setAttribute('monaco-view-zone', myZone.whitespaceId.toString());
        this.domNode.appendChild(myZone.domNode);
        if (myZone.marginDomNode) {
            myZone.marginDomNode.setPosition('absolute');
            myZone.marginDomNode.domNode.style.width = '100%';
            myZone.marginDomNode.setDisplay('none');
            myZone.marginDomNode.setAttribute('monaco-view-zone', myZone.whitespaceId.toString());
            this.marginDomNode.appendChild(myZone.marginDomNode);
        }
        this._zones[myZone.whitespaceId.toString()] = myZone;
        this.setShouldRender();
        return myZone.whitespaceId;
    };
    ViewZones.prototype.removeZone = function (id) {
        if (this._zones.hasOwnProperty(id.toString())) {
            var zone = this._zones[id.toString()];
            delete this._zones[id.toString()];
            this._context.viewLayout.removeWhitespace(zone.whitespaceId);
            zone.domNode.removeAttribute('monaco-visible-view-zone');
            zone.domNode.removeAttribute('monaco-view-zone');
            zone.domNode.domNode.parentNode.removeChild(zone.domNode.domNode);
            if (zone.marginDomNode) {
                zone.marginDomNode.removeAttribute('monaco-visible-view-zone');
                zone.marginDomNode.removeAttribute('monaco-view-zone');
                zone.marginDomNode.domNode.parentNode.removeChild(zone.marginDomNode.domNode);
            }
            this.setShouldRender();
            return true;
        }
        return false;
    };
    ViewZones.prototype.layoutZone = function (id) {
        var changed = false;
        if (this._zones.hasOwnProperty(id.toString())) {
            var zone = this._zones[id.toString()];
            var props = this._computeWhitespaceProps(zone.delegate);
            // let newOrdinal = this._getZoneOrdinal(zone.delegate);
            changed = this._context.viewLayout.changeWhitespace(zone.whitespaceId, props.afterViewLineNumber, props.heightInPx) || changed;
            // TODO@Alex: change `newOrdinal` too
            if (changed) {
                this._safeCallOnComputedHeight(zone.delegate, props.heightInPx);
                this.setShouldRender();
            }
        }
        return changed;
    };
    ViewZones.prototype.shouldSuppressMouseDownOnViewZone = function (id) {
        if (this._zones.hasOwnProperty(id.toString())) {
            var zone = this._zones[id.toString()];
            return Boolean(zone.delegate.suppressMouseDown);
        }
        return false;
    };
    ViewZones.prototype._heightInPixels = function (zone) {
        if (typeof zone.heightInPx === 'number') {
            return zone.heightInPx;
        }
        if (typeof zone.heightInLines === 'number') {
            return this._lineHeight * zone.heightInLines;
        }
        return this._lineHeight;
    };
    ViewZones.prototype._minWidthInPixels = function (zone) {
        if (typeof zone.minWidthInPx === 'number') {
            return zone.minWidthInPx;
        }
        return 0;
    };
    ViewZones.prototype._safeCallOnComputedHeight = function (zone, height) {
        if (typeof zone.onComputedHeight === 'function') {
            try {
                zone.onComputedHeight(height);
            }
            catch (e) {
                onUnexpectedError(e);
            }
        }
    };
    ViewZones.prototype._safeCallOnDomNodeTop = function (zone, top) {
        if (typeof zone.onDomNodeTop === 'function') {
            try {
                zone.onDomNodeTop(top);
            }
            catch (e) {
                onUnexpectedError(e);
            }
        }
    };
    ViewZones.prototype.prepareRender = function (ctx) {
        // Nothing to read
    };
    ViewZones.prototype.render = function (ctx) {
        var visibleWhitespaces = ctx.viewportData.whitespaceViewportData;
        var visibleZones = {};
        var hasVisibleZone = false;
        for (var i = 0, len = visibleWhitespaces.length; i < len; i++) {
            visibleZones[visibleWhitespaces[i].id.toString()] = visibleWhitespaces[i];
            hasVisibleZone = true;
        }
        var keys = Object.keys(this._zones);
        for (var i = 0, len = keys.length; i < len; i++) {
            var id = keys[i];
            var zone = this._zones[id];
            var newTop = 0;
            var newHeight = 0;
            var newDisplay = 'none';
            if (visibleZones.hasOwnProperty(id)) {
                newTop = visibleZones[id].verticalOffset - ctx.bigNumbersDelta;
                newHeight = visibleZones[id].height;
                newDisplay = 'block';
                // zone is visible
                if (!zone.isVisible) {
                    zone.domNode.setAttribute('monaco-visible-view-zone', 'true');
                    zone.isVisible = true;
                }
                this._safeCallOnDomNodeTop(zone.delegate, ctx.getScrolledTopFromAbsoluteTop(visibleZones[id].verticalOffset));
            }
            else {
                if (zone.isVisible) {
                    zone.domNode.removeAttribute('monaco-visible-view-zone');
                    zone.isVisible = false;
                }
                this._safeCallOnDomNodeTop(zone.delegate, ctx.getScrolledTopFromAbsoluteTop(-1000000));
            }
            zone.domNode.setTop(newTop);
            zone.domNode.setHeight(newHeight);
            zone.domNode.setDisplay(newDisplay);
            if (zone.marginDomNode) {
                zone.marginDomNode.setTop(newTop);
                zone.marginDomNode.setHeight(newHeight);
                zone.marginDomNode.setDisplay(newDisplay);
            }
        }
        if (hasVisibleZone) {
            this.domNode.setWidth(Math.max(ctx.scrollWidth, this._contentWidth));
            this.marginDomNode.setWidth(this._contentLeft);
        }
    };
    return ViewZones;
}(ViewPart));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ViewContext = /** @class */ (function () {
    function ViewContext(configuration, theme, model, privateViewEventBus) {
        this.configuration = configuration;
        this.theme = theme;
        this.model = model;
        this.viewLayout = model.viewLayout;
        this.privateViewEventBus = privateViewEventBus;
    }
    ViewContext.prototype.addEventHandler = function (eventHandler) {
        this.privateViewEventBus.addEventHandler(eventHandler);
    };
    ViewContext.prototype.removeEventHandler = function (eventHandler) {
        this.privateViewEventBus.removeEventHandler(eventHandler);
    };
    return ViewContext;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ViewEventDispatcher = /** @class */ (function () {
    function ViewEventDispatcher(eventHandlerGateKeeper) {
        this._eventHandlerGateKeeper = eventHandlerGateKeeper;
        this._eventHandlers = [];
        this._eventQueue = null;
        this._isConsumingQueue = false;
    }
    ViewEventDispatcher.prototype.addEventHandler = function (eventHandler) {
        for (var i = 0, len = this._eventHandlers.length; i < len; i++) {
            if (this._eventHandlers[i] === eventHandler) {
                console.warn('Detected duplicate listener in ViewEventDispatcher', eventHandler);
            }
        }
        this._eventHandlers.push(eventHandler);
    };
    ViewEventDispatcher.prototype.removeEventHandler = function (eventHandler) {
        for (var i = 0; i < this._eventHandlers.length; i++) {
            if (this._eventHandlers[i] === eventHandler) {
                this._eventHandlers.splice(i, 1);
                break;
            }
        }
    };
    ViewEventDispatcher.prototype.emit = function (event) {
        if (this._eventQueue) {
            this._eventQueue.push(event);
        }
        else {
            this._eventQueue = [event];
        }
        if (!this._isConsumingQueue) {
            this.consumeQueue();
        }
    };
    ViewEventDispatcher.prototype.emitMany = function (events) {
        if (this._eventQueue) {
            this._eventQueue = this._eventQueue.concat(events);
        }
        else {
            this._eventQueue = events;
        }
        if (!this._isConsumingQueue) {
            this.consumeQueue();
        }
    };
    ViewEventDispatcher.prototype.consumeQueue = function () {
        var _this = this;
        this._eventHandlerGateKeeper(function () {
            try {
                _this._isConsumingQueue = true;
                _this._doConsumeQueue();
            }
            finally {
                _this._isConsumingQueue = false;
            }
        });
    };
    ViewEventDispatcher.prototype._doConsumeQueue = function () {
        while (this._eventQueue) {
            // Empty event queue, as events might come in while sending these off
            var events = this._eventQueue;
            this._eventQueue = null;
            // Use a clone of the event handlers list, as they might remove themselves
            var eventHandlers = this._eventHandlers.slice(0);
            for (var i = 0, len = eventHandlers.length; i < len; i++) {
                eventHandlers[i].handleEvents(events);
            }
        }
    };
    return ViewEventDispatcher;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Contains all data needed to render at a specific viewport.
 */
var ViewportData = /** @class */ (function () {
    function ViewportData(selections, partialData, whitespaceViewportData, model) {
        this.selections = selections;
        this.startLineNumber = partialData.startLineNumber | 0;
        this.endLineNumber = partialData.endLineNumber | 0;
        this.relativeVerticalOffset = partialData.relativeVerticalOffset;
        this.bigNumbersDelta = partialData.bigNumbersDelta | 0;
        this.whitespaceViewportData = whitespaceViewportData;
        this._model = model;
        this.visibleRange = new Range(partialData.startLineNumber, this._model.getLineMinColumn(partialData.startLineNumber), partialData.endLineNumber, this._model.getLineMaxColumn(partialData.endLineNumber));
    }
    ViewportData.prototype.getViewLineRenderingData = function (lineNumber) {
        return this._model.getViewLineRenderingData(this.visibleRange, lineNumber);
    };
    ViewportData.prototype.getDecorationsInViewport = function () {
        return this._model.getDecorationsInViewport(this.visibleRange);
    };
    return ViewportData;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$10 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var invalidFunc$1 = function () { throw new Error("Invalid change accessor"); };
var View = /** @class */ (function (_super) {
    __extends$10(View, _super);
    function View(commandDelegate, configuration, themeService, model, cursor, outgoingEvents) {
        var _this = _super.call(this) || this;
        _this._cursor = cursor;
        _this._renderAnimationFrame = null;
        _this.outgoingEvents = outgoingEvents;
        var viewController = new ViewController(configuration, model, _this.outgoingEvents, commandDelegate);
        // The event dispatcher will always go through _renderOnce before dispatching any events
        _this.eventDispatcher = new ViewEventDispatcher(function (callback) { return _this._renderOnce(callback); });
        // Ensure the view is the first event handler in order to update the layout
        _this.eventDispatcher.addEventHandler(_this);
        // The view context is passed on to most classes (basically to reduce param. counts in ctors)
        _this._context = new ViewContext(configuration, themeService.getTheme(), model, _this.eventDispatcher);
        _this._register(themeService.onThemeChange(function (theme) {
            _this._context.theme = theme;
            _this.eventDispatcher.emit(new ViewThemeChangedEvent());
            _this.render(true, false);
        }));
        _this.viewParts = [];
        // Keyboard handler
        _this._textAreaHandler = new TextAreaHandler(_this._context, viewController, _this.createTextAreaHandlerHelper());
        _this.viewParts.push(_this._textAreaHandler);
        _this.createViewParts();
        _this._setLayout();
        // Pointer handler
        _this.pointerHandler = new PointerHandler(_this._context, viewController, _this.createPointerHandlerHelper());
        _this._register(model.addEventListener(function (events) {
            _this.eventDispatcher.emitMany(events);
        }));
        _this._register(_this._cursor.addEventListener(function (events) {
            _this.eventDispatcher.emitMany(events);
        }));
        return _this;
    }
    View.prototype.createViewParts = function () {
        // These two dom nodes must be constructed up front, since references are needed in the layout provider (scrolling & co.)
        this.linesContent = createFastDomNode(document.createElement('div'));
        this.linesContent.setClassName('lines-content' + ' monaco-editor-background');
        this.linesContent.setPosition('absolute');
        this.domNode = createFastDomNode(document.createElement('div'));
        this.domNode.setClassName(this.getEditorClassName());
        this.overflowGuardContainer = createFastDomNode(document.createElement('div'));
        PartFingerprints.write(this.overflowGuardContainer, 3 /* OverflowGuard */);
        this.overflowGuardContainer.setClassName('overflow-guard');
        this._scrollbar = new EditorScrollbar(this._context, this.linesContent, this.domNode, this.overflowGuardContainer);
        this.viewParts.push(this._scrollbar);
        // View Lines
        this.viewLines = new ViewLines(this._context, this.linesContent);
        // View Zones
        this.viewZones = new ViewZones(this._context);
        this.viewParts.push(this.viewZones);
        // Decorations overview ruler
        var decorationsOverviewRuler = new DecorationsOverviewRuler(this._context);
        this.viewParts.push(decorationsOverviewRuler);
        var scrollDecoration = new ScrollDecorationViewPart(this._context);
        this.viewParts.push(scrollDecoration);
        var contentViewOverlays = new ContentViewOverlays(this._context);
        this.viewParts.push(contentViewOverlays);
        contentViewOverlays.addDynamicOverlay(new CurrentLineHighlightOverlay(this._context));
        contentViewOverlays.addDynamicOverlay(new SelectionsOverlay(this._context));
        contentViewOverlays.addDynamicOverlay(new IndentGuidesOverlay(this._context));
        contentViewOverlays.addDynamicOverlay(new DecorationsOverlay(this._context));
        var marginViewOverlays = new MarginViewOverlays(this._context);
        this.viewParts.push(marginViewOverlays);
        marginViewOverlays.addDynamicOverlay(new CurrentLineMarginHighlightOverlay(this._context));
        marginViewOverlays.addDynamicOverlay(new GlyphMarginOverlay(this._context));
        marginViewOverlays.addDynamicOverlay(new MarginViewLineDecorationsOverlay(this._context));
        marginViewOverlays.addDynamicOverlay(new LinesDecorationsOverlay(this._context));
        marginViewOverlays.addDynamicOverlay(new LineNumbersOverlay(this._context));
        var margin = new Margin(this._context);
        margin.getDomNode().appendChild(this.viewZones.marginDomNode);
        margin.getDomNode().appendChild(marginViewOverlays.getDomNode());
        this.viewParts.push(margin);
        // Content widgets
        this.contentWidgets = new ViewContentWidgets(this._context, this.domNode);
        this.viewParts.push(this.contentWidgets);
        this.viewCursors = new ViewCursors(this._context);
        this.viewParts.push(this.viewCursors);
        // Overlay widgets
        this.overlayWidgets = new ViewOverlayWidgets(this._context);
        this.viewParts.push(this.overlayWidgets);
        var rulers = new Rulers(this._context);
        this.viewParts.push(rulers);
        var minimap = new Minimap(this._context);
        this.viewParts.push(minimap);
        // -------------- Wire dom nodes up
        if (decorationsOverviewRuler) {
            var overviewRulerData = this._scrollbar.getOverviewRulerLayoutInfo();
            overviewRulerData.parent.insertBefore(decorationsOverviewRuler.getDomNode(), overviewRulerData.insertBefore);
        }
        this.linesContent.appendChild(contentViewOverlays.getDomNode());
        this.linesContent.appendChild(rulers.domNode);
        this.linesContent.appendChild(this.viewZones.domNode);
        this.linesContent.appendChild(this.viewLines.getDomNode());
        this.linesContent.appendChild(this.contentWidgets.domNode);
        this.linesContent.appendChild(this.viewCursors.getDomNode());
        this.overflowGuardContainer.appendChild(margin.getDomNode());
        this.overflowGuardContainer.appendChild(this._scrollbar.getDomNode());
        this.overflowGuardContainer.appendChild(scrollDecoration.getDomNode());
        this.overflowGuardContainer.appendChild(this._textAreaHandler.textArea);
        this.overflowGuardContainer.appendChild(this._textAreaHandler.textAreaCover);
        this.overflowGuardContainer.appendChild(this.overlayWidgets.getDomNode());
        this.overflowGuardContainer.appendChild(minimap.getDomNode());
        this.domNode.appendChild(this.overflowGuardContainer);
        this.domNode.appendChild(this.contentWidgets.overflowingContentWidgetsDomNode);
    };
    View.prototype._flushAccumulatedAndRenderNow = function () {
        this._renderNow();
    };
    View.prototype.createPointerHandlerHelper = function () {
        var _this = this;
        return {
            viewDomNode: this.domNode.domNode,
            linesContentDomNode: this.linesContent.domNode,
            focusTextArea: function () {
                _this.focus();
            },
            getLastViewCursorsRenderData: function () {
                return _this.viewCursors.getLastRenderData() || [];
            },
            shouldSuppressMouseDownOnViewZone: function (viewZoneId) {
                return _this.viewZones.shouldSuppressMouseDownOnViewZone(viewZoneId);
            },
            shouldSuppressMouseDownOnWidget: function (widgetId) {
                return _this.contentWidgets.shouldSuppressMouseDownOnWidget(widgetId);
            },
            getPositionFromDOMInfo: function (spanNode, offset) {
                _this._flushAccumulatedAndRenderNow();
                return _this.viewLines.getPositionFromDOMInfo(spanNode, offset);
            },
            visibleRangeForPosition2: function (lineNumber, column) {
                _this._flushAccumulatedAndRenderNow();
                return _this.viewLines.visibleRangeForPosition(new Position(lineNumber, column));
            },
            getLineWidth: function (lineNumber) {
                _this._flushAccumulatedAndRenderNow();
                return _this.viewLines.getLineWidth(lineNumber);
            }
        };
    };
    View.prototype.createTextAreaHandlerHelper = function () {
        var _this = this;
        return {
            visibleRangeForPositionRelativeToEditor: function (lineNumber, column) {
                _this._flushAccumulatedAndRenderNow();
                return _this.viewLines.visibleRangeForPosition(new Position(lineNumber, column));
            }
        };
    };
    View.prototype._setLayout = function () {
        var layoutInfo = this._context.configuration.editor.layoutInfo;
        this.domNode.setWidth(layoutInfo.width);
        this.domNode.setHeight(layoutInfo.height);
        this.overflowGuardContainer.setWidth(layoutInfo.width);
        this.overflowGuardContainer.setHeight(layoutInfo.height);
        this.linesContent.setWidth(1000000);
        this.linesContent.setHeight(1000000);
    };
    View.prototype.getEditorClassName = function () {
        var focused = this._textAreaHandler.isFocused() ? ' focused' : '';
        return this._context.configuration.editor.editorClassName + ' ' + getThemeTypeSelector(this._context.theme.type) + focused;
    };
    // --- begin event handlers
    View.prototype.onConfigurationChanged = function (e) {
        if (e.editorClassName) {
            this.domNode.setClassName(this.getEditorClassName());
        }
        if (e.layoutInfo) {
            this._setLayout();
        }
        return false;
    };
    View.prototype.onFocusChanged = function (e) {
        this.domNode.setClassName(this.getEditorClassName());
        this._context.model.setHasFocus(e.isFocused);
        if (e.isFocused) {
            this.outgoingEvents.emitViewFocusGained();
        }
        else {
            this.outgoingEvents.emitViewFocusLost();
        }
        return false;
    };
    View.prototype.onScrollChanged = function (e) {
        this.outgoingEvents.emitScrollChanged(e);
        return false;
    };
    View.prototype.onThemeChanged = function (e) {
        this.domNode.setClassName(this.getEditorClassName());
        return false;
    };
    // --- end event handlers
    View.prototype.dispose = function () {
        if (this._renderAnimationFrame !== null) {
            this._renderAnimationFrame.dispose();
            this._renderAnimationFrame = null;
        }
        this.eventDispatcher.removeEventHandler(this);
        this.outgoingEvents.dispose();
        this.pointerHandler.dispose();
        this.viewLines.dispose();
        // Destroy view parts
        for (var i = 0, len = this.viewParts.length; i < len; i++) {
            this.viewParts[i].dispose();
        }
        this.viewParts = [];
        _super.prototype.dispose.call(this);
    };
    View.prototype._renderOnce = function (callback) {
        var r = safeInvokeNoArg(callback);
        this._scheduleRender();
        return r;
    };
    View.prototype._scheduleRender = function () {
        if (this._renderAnimationFrame === null) {
            this._renderAnimationFrame = runAtThisOrScheduleAtNextAnimationFrame(this._onRenderScheduled.bind(this), 100);
        }
    };
    View.prototype._onRenderScheduled = function () {
        this._renderAnimationFrame = null;
        this._flushAccumulatedAndRenderNow();
    };
    View.prototype._renderNow = function () {
        var _this = this;
        safeInvokeNoArg(function () { return _this._actualRender(); });
    };
    View.prototype._getViewPartsToRender = function () {
        var result = [], resultLen = 0;
        for (var i = 0, len = this.viewParts.length; i < len; i++) {
            var viewPart = this.viewParts[i];
            if (viewPart.shouldRender()) {
                result[resultLen++] = viewPart;
            }
        }
        return result;
    };
    View.prototype._actualRender = function () {
        if (!isInDOM(this.domNode.domNode)) {
            return;
        }
        var viewPartsToRender = this._getViewPartsToRender();
        if (!this.viewLines.shouldRender() && viewPartsToRender.length === 0) {
            // Nothing to render
            return;
        }
        var partialViewportData = this._context.viewLayout.getLinesViewportData();
        this._context.model.setViewport(partialViewportData.startLineNumber, partialViewportData.endLineNumber, partialViewportData.centeredLineNumber);
        var viewportData = new ViewportData(this._cursor.getViewSelections(), partialViewportData, this._context.viewLayout.getWhitespaceViewportData(), this._context.model);
        if (this.contentWidgets.shouldRender()) {
            // Give the content widgets a chance to set their max width before a possible synchronous layout
            this.contentWidgets.onBeforeRender(viewportData);
        }
        if (this.viewLines.shouldRender()) {
            this.viewLines.renderText(viewportData);
            this.viewLines.onDidRender();
            // Rendering of viewLines might cause scroll events to occur, so collect view parts to render again
            viewPartsToRender = this._getViewPartsToRender();
        }
        var renderingContext = new RenderingContext(this._context.viewLayout, viewportData, this.viewLines);
        // Render the rest of the parts
        for (var i = 0, len = viewPartsToRender.length; i < len; i++) {
            var viewPart = viewPartsToRender[i];
            viewPart.prepareRender(renderingContext);
        }
        for (var i = 0, len = viewPartsToRender.length; i < len; i++) {
            var viewPart = viewPartsToRender[i];
            viewPart.render(renderingContext);
            viewPart.onDidRender();
        }
    };
    // --- BEGIN CodeEditor helpers
    View.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
        this._scrollbar.delegateVerticalScrollbarMouseDown(browserEvent);
    };
    View.prototype.restoreState = function (scrollPosition) {
        this._context.viewLayout.setScrollPositionNow({ scrollTop: scrollPosition.scrollTop });
        this._renderNow();
        this.viewLines.updateLineWidths();
        this._context.viewLayout.setScrollPositionNow({ scrollLeft: scrollPosition.scrollLeft });
    };
    View.prototype.getOffsetForColumn = function (modelLineNumber, modelColumn) {
        var modelPosition = this._context.model.validateModelPosition({
            lineNumber: modelLineNumber,
            column: modelColumn
        });
        var viewPosition = this._context.model.coordinatesConverter.convertModelPositionToViewPosition(modelPosition);
        this._flushAccumulatedAndRenderNow();
        var visibleRange = this.viewLines.visibleRangeForPosition(new Position(viewPosition.lineNumber, viewPosition.column));
        if (!visibleRange) {
            return -1;
        }
        return visibleRange.left;
    };
    View.prototype.getTargetAtClientPoint = function (clientX, clientY) {
        return this.pointerHandler.getTargetAtClientPoint(clientX, clientY);
    };
    View.prototype.createOverviewRuler = function (cssClassName) {
        return new OverviewRuler(this._context, cssClassName);
    };
    View.prototype.change = function (callback) {
        var _this = this;
        var zonesHaveChanged = false;
        this._renderOnce(function () {
            var changeAccessor = {
                addZone: function (zone) {
                    zonesHaveChanged = true;
                    return _this.viewZones.addZone(zone);
                },
                removeZone: function (id) {
                    if (!id) {
                        return;
                    }
                    zonesHaveChanged = _this.viewZones.removeZone(id) || zonesHaveChanged;
                },
                layoutZone: function (id) {
                    if (!id) {
                        return;
                    }
                    zonesHaveChanged = _this.viewZones.layoutZone(id) || zonesHaveChanged;
                }
            };
            safeInvoke1Arg(callback, changeAccessor);
            // Invalidate changeAccessor
            changeAccessor.addZone = invalidFunc$1;
            changeAccessor.removeZone = invalidFunc$1;
            changeAccessor.layoutZone = invalidFunc$1;
            if (zonesHaveChanged) {
                _this._context.viewLayout.onHeightMaybeChanged();
                _this._context.privateViewEventBus.emit(new ViewZonesChangedEvent());
            }
        });
        return zonesHaveChanged;
    };
    View.prototype.render = function (now, everything) {
        if (everything) {
            // Force everything to render...
            this.viewLines.forceShouldRender();
            for (var i = 0, len = this.viewParts.length; i < len; i++) {
                var viewPart = this.viewParts[i];
                viewPart.forceShouldRender();
            }
        }
        if (now) {
            this._flushAccumulatedAndRenderNow();
        }
        else {
            this._scheduleRender();
        }
    };
    View.prototype.focus = function () {
        this._textAreaHandler.focusTextArea();
    };
    View.prototype.isFocused = function () {
        return this._textAreaHandler.isFocused();
    };
    View.prototype.addContentWidget = function (widgetData) {
        this.contentWidgets.addWidget(widgetData.widget);
        this.layoutContentWidget(widgetData);
        this._scheduleRender();
    };
    View.prototype.layoutContentWidget = function (widgetData) {
        var newPosition = widgetData.position ? widgetData.position.position : null;
        var newRange = widgetData.position ? widgetData.position.range : null;
        var newPreference = widgetData.position ? widgetData.position.preference : null;
        this.contentWidgets.setWidgetPosition(widgetData.widget, newPosition, newRange, newPreference);
        this._scheduleRender();
    };
    View.prototype.removeContentWidget = function (widgetData) {
        this.contentWidgets.removeWidget(widgetData.widget);
        this._scheduleRender();
    };
    View.prototype.addOverlayWidget = function (widgetData) {
        this.overlayWidgets.addWidget(widgetData.widget);
        this.layoutOverlayWidget(widgetData);
        this._scheduleRender();
    };
    View.prototype.layoutOverlayWidget = function (widgetData) {
        var newPreference = widgetData.position ? widgetData.position.preference : null;
        var shouldRender = this.overlayWidgets.setWidgetPosition(widgetData.widget, newPreference);
        if (shouldRender) {
            this._scheduleRender();
        }
    };
    View.prototype.removeOverlayWidget = function (widgetData) {
        this.overlayWidgets.removeWidget(widgetData.widget);
        this._scheduleRender();
    };
    return View;
}(ViewEventHandler));
function safeInvokeNoArg(func) {
    try {
        return func();
    }
    catch (e) {
        onUnexpectedError(e);
    }
}
function safeInvoke1Arg(func, arg1) {
    try {
        return func(arg1);
    }
    catch (e) {
        onUnexpectedError(e);
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$11 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewOutgoingEvents = /** @class */ (function (_super) {
    __extends$11(ViewOutgoingEvents, _super);
    function ViewOutgoingEvents(viewModel) {
        var _this = _super.call(this) || this;
        _this.onDidScroll = null;
        _this.onDidGainFocus = null;
        _this.onDidLoseFocus = null;
        _this.onKeyDown = null;
        _this.onKeyUp = null;
        _this.onContextMenu = null;
        _this.onMouseMove = null;
        _this.onMouseLeave = null;
        _this.onMouseUp = null;
        _this.onMouseDown = null;
        _this.onMouseDrag = null;
        _this.onMouseDrop = null;
        _this._viewModel = viewModel;
        return _this;
    }
    ViewOutgoingEvents.prototype.emitScrollChanged = function (e) {
        if (this.onDidScroll) {
            this.onDidScroll(e);
        }
    };
    ViewOutgoingEvents.prototype.emitViewFocusGained = function () {
        if (this.onDidGainFocus) {
            this.onDidGainFocus(void 0);
        }
    };
    ViewOutgoingEvents.prototype.emitViewFocusLost = function () {
        if (this.onDidLoseFocus) {
            this.onDidLoseFocus(void 0);
        }
    };
    ViewOutgoingEvents.prototype.emitKeyDown = function (e) {
        if (this.onKeyDown) {
            this.onKeyDown(e);
        }
    };
    ViewOutgoingEvents.prototype.emitKeyUp = function (e) {
        if (this.onKeyUp) {
            this.onKeyUp(e);
        }
    };
    ViewOutgoingEvents.prototype.emitContextMenu = function (e) {
        if (this.onContextMenu) {
            this.onContextMenu(this._convertViewToModelMouseEvent(e));
        }
    };
    ViewOutgoingEvents.prototype.emitMouseMove = function (e) {
        if (this.onMouseMove) {
            this.onMouseMove(this._convertViewToModelMouseEvent(e));
        }
    };
    ViewOutgoingEvents.prototype.emitMouseLeave = function (e) {
        if (this.onMouseLeave) {
            this.onMouseLeave(this._convertViewToModelMouseEvent(e));
        }
    };
    ViewOutgoingEvents.prototype.emitMouseUp = function (e) {
        if (this.onMouseUp) {
            this.onMouseUp(this._convertViewToModelMouseEvent(e));
        }
    };
    ViewOutgoingEvents.prototype.emitMouseDown = function (e) {
        if (this.onMouseDown) {
            this.onMouseDown(this._convertViewToModelMouseEvent(e));
        }
    };
    ViewOutgoingEvents.prototype.emitMouseDrag = function (e) {
        if (this.onMouseDrag) {
            this.onMouseDrag(this._convertViewToModelMouseEvent(e));
        }
    };
    ViewOutgoingEvents.prototype.emitMouseDrop = function (e) {
        if (this.onMouseDrop) {
            this.onMouseDrop(this._convertViewToModelMouseEvent(e));
        }
    };
    ViewOutgoingEvents.prototype._convertViewToModelMouseEvent = function (e) {
        if (e.target) {
            return {
                event: e.event,
                target: this._convertViewToModelMouseTarget(e.target)
            };
        }
        return e;
    };
    ViewOutgoingEvents.prototype._convertViewToModelMouseTarget = function (target) {
        return new ExternalMouseTarget(target.element, target.type, target.mouseColumn, target.position ? this._convertViewToModelPosition(target.position) : null, target.range ? this._convertViewToModelRange(target.range) : null, target.detail);
    };
    ViewOutgoingEvents.prototype._convertViewToModelPosition = function (viewPosition) {
        return this._viewModel.coordinatesConverter.convertViewPositionToModelPosition(viewPosition);
    };
    ViewOutgoingEvents.prototype._convertViewToModelRange = function (viewRange) {
        return this._viewModel.coordinatesConverter.convertViewRangeToModelRange(viewRange);
    };
    return ViewOutgoingEvents;
}(Disposable));
var ExternalMouseTarget = /** @class */ (function () {
    function ExternalMouseTarget(element, type, mouseColumn, position, range, detail) {
        this.element = element;
        this.type = type;
        this.mouseColumn = mouseColumn;
        this.position = position;
        this.range = range;
        this.detail = detail;
    }
    ExternalMouseTarget.prototype.toString = function () {
        return MouseTarget.toString(this);
    };
    return ExternalMouseTarget;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var OneCursor = /** @class */ (function () {
    function OneCursor(context) {
        this._selTrackedRange = null;
        this._trackSelection = true;
        this._setState(context, new SingleCursorState(new Range(1, 1, 1, 1), 0, new Position(1, 1), 0), new SingleCursorState(new Range(1, 1, 1, 1), 0, new Position(1, 1), 0));
    }
    OneCursor.prototype.dispose = function (context) {
        this._removeTrackedRange(context);
    };
    OneCursor.prototype.startTrackingSelection = function (context) {
        this._trackSelection = true;
        this._updateTrackedRange(context);
    };
    OneCursor.prototype.stopTrackingSelection = function (context) {
        this._trackSelection = false;
        this._removeTrackedRange(context);
    };
    OneCursor.prototype._updateTrackedRange = function (context) {
        if (!this._trackSelection) {
            // don't track the selection
            return;
        }
        this._selTrackedRange = context.model._setTrackedRange(this._selTrackedRange, this.modelState.selection, 0 /* AlwaysGrowsWhenTypingAtEdges */);
    };
    OneCursor.prototype._removeTrackedRange = function (context) {
        this._selTrackedRange = context.model._setTrackedRange(this._selTrackedRange, null, 0 /* AlwaysGrowsWhenTypingAtEdges */);
    };
    OneCursor.prototype.asCursorState = function () {
        return new CursorState(this.modelState, this.viewState);
    };
    OneCursor.prototype.readSelectionFromMarkers = function (context) {
        var range = context.model._getTrackedRange(this._selTrackedRange);
        if (this.modelState.selection.getDirection() === 0 /* LTR */) {
            return new Selection(range.startLineNumber, range.startColumn, range.endLineNumber, range.endColumn);
        }
        return new Selection(range.endLineNumber, range.endColumn, range.startLineNumber, range.startColumn);
    };
    OneCursor.prototype.ensureValidState = function (context) {
        this._setState(context, this.modelState, this.viewState);
    };
    OneCursor.prototype.setState = function (context, modelState, viewState) {
        this._setState(context, modelState, viewState);
    };
    OneCursor.prototype._setState = function (context, modelState, viewState) {
        if (!modelState) {
            if (!viewState) {
                return;
            }
            // We only have the view state => compute the model state
            var selectionStart = context.model.validateRange(context.convertViewRangeToModelRange(viewState.selectionStart));
            var position = context.model.validatePosition(context.convertViewPositionToModelPosition(viewState.position.lineNumber, viewState.position.column));
            modelState = new SingleCursorState(selectionStart, viewState.selectionStartLeftoverVisibleColumns, position, viewState.leftoverVisibleColumns);
        }
        else {
            // Validate new model state
            var selectionStart = context.model.validateRange(modelState.selectionStart);
            var selectionStartLeftoverVisibleColumns = modelState.selectionStart.equalsRange(selectionStart) ? modelState.selectionStartLeftoverVisibleColumns : 0;
            var position = context.model.validatePosition(modelState.position);
            var leftoverVisibleColumns = modelState.position.equals(position) ? modelState.leftoverVisibleColumns : 0;
            modelState = new SingleCursorState(selectionStart, selectionStartLeftoverVisibleColumns, position, leftoverVisibleColumns);
        }
        if (!viewState) {
            // We only have the model state => compute the view state
            var viewSelectionStart1 = context.convertModelPositionToViewPosition(new Position(modelState.selectionStart.startLineNumber, modelState.selectionStart.startColumn));
            var viewSelectionStart2 = context.convertModelPositionToViewPosition(new Position(modelState.selectionStart.endLineNumber, modelState.selectionStart.endColumn));
            var viewSelectionStart = new Range(viewSelectionStart1.lineNumber, viewSelectionStart1.column, viewSelectionStart2.lineNumber, viewSelectionStart2.column);
            var viewPosition = context.convertModelPositionToViewPosition(modelState.position);
            viewState = new SingleCursorState(viewSelectionStart, modelState.selectionStartLeftoverVisibleColumns, viewPosition, modelState.leftoverVisibleColumns);
        }
        else {
            // Validate new view state
            var viewSelectionStart = context.validateViewRange(viewState.selectionStart, modelState.selectionStart);
            var viewPosition = context.validateViewPosition(viewState.position, modelState.position);
            viewState = new SingleCursorState(viewSelectionStart, modelState.selectionStartLeftoverVisibleColumns, viewPosition, modelState.leftoverVisibleColumns);
        }
        this.modelState = modelState;
        this.viewState = viewState;
        this._updateTrackedRange(context);
    };
    return OneCursor;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CursorCollection = /** @class */ (function () {
    function CursorCollection(context) {
        this.context = context;
        this.primaryCursor = new OneCursor(context);
        this.secondaryCursors = [];
        this.lastAddedCursorIndex = 0;
    }
    CursorCollection.prototype.dispose = function () {
        this.primaryCursor.dispose(this.context);
        this.killSecondaryCursors();
    };
    CursorCollection.prototype.startTrackingSelections = function () {
        this.primaryCursor.startTrackingSelection(this.context);
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            this.secondaryCursors[i].startTrackingSelection(this.context);
        }
    };
    CursorCollection.prototype.stopTrackingSelections = function () {
        this.primaryCursor.stopTrackingSelection(this.context);
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            this.secondaryCursors[i].stopTrackingSelection(this.context);
        }
    };
    CursorCollection.prototype.updateContext = function (context) {
        this.context = context;
    };
    CursorCollection.prototype.ensureValidState = function () {
        this.primaryCursor.ensureValidState(this.context);
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            this.secondaryCursors[i].ensureValidState(this.context);
        }
    };
    CursorCollection.prototype.readSelectionFromMarkers = function () {
        var result = [];
        result[0] = this.primaryCursor.readSelectionFromMarkers(this.context);
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            result[i + 1] = this.secondaryCursors[i].readSelectionFromMarkers(this.context);
        }
        return result;
    };
    CursorCollection.prototype.getAll = function () {
        var result = [];
        result[0] = this.primaryCursor.asCursorState();
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            result[i + 1] = this.secondaryCursors[i].asCursorState();
        }
        return result;
    };
    CursorCollection.prototype.getViewPositions = function () {
        var result = [];
        result[0] = this.primaryCursor.viewState.position;
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            result[i + 1] = this.secondaryCursors[i].viewState.position;
        }
        return result;
    };
    CursorCollection.prototype.getSelections = function () {
        var result = [];
        result[0] = this.primaryCursor.modelState.selection;
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            result[i + 1] = this.secondaryCursors[i].modelState.selection;
        }
        return result;
    };
    CursorCollection.prototype.getViewSelections = function () {
        var result = [];
        result[0] = this.primaryCursor.viewState.selection;
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            result[i + 1] = this.secondaryCursors[i].viewState.selection;
        }
        return result;
    };
    CursorCollection.prototype.setSelections = function (selections) {
        this.setStates(CursorState.fromModelSelections(selections));
    };
    CursorCollection.prototype.getPrimaryCursor = function () {
        return this.primaryCursor.asCursorState();
    };
    CursorCollection.prototype.setStates = function (states) {
        if (states === null) {
            return;
        }
        this.primaryCursor.setState(this.context, states[0].modelState, states[0].viewState);
        this._setSecondaryStates(states.slice(1));
    };
    /**
     * Creates or disposes secondary cursors as necessary to match the number of `secondarySelections`.
     */
    CursorCollection.prototype._setSecondaryStates = function (secondaryStates) {
        var secondaryCursorsLength = this.secondaryCursors.length;
        var secondaryStatesLength = secondaryStates.length;
        if (secondaryCursorsLength < secondaryStatesLength) {
            var createCnt = secondaryStatesLength - secondaryCursorsLength;
            for (var i = 0; i < createCnt; i++) {
                this._addSecondaryCursor();
            }
        }
        else if (secondaryCursorsLength > secondaryStatesLength) {
            var removeCnt = secondaryCursorsLength - secondaryStatesLength;
            for (var i = 0; i < removeCnt; i++) {
                this._removeSecondaryCursor(this.secondaryCursors.length - 1);
            }
        }
        for (var i = 0; i < secondaryStatesLength; i++) {
            this.secondaryCursors[i].setState(this.context, secondaryStates[i].modelState, secondaryStates[i].viewState);
        }
    };
    CursorCollection.prototype.killSecondaryCursors = function () {
        this._setSecondaryStates([]);
    };
    CursorCollection.prototype._addSecondaryCursor = function () {
        this.secondaryCursors.push(new OneCursor(this.context));
        this.lastAddedCursorIndex = this.secondaryCursors.length;
    };
    CursorCollection.prototype.getLastAddedCursorIndex = function () {
        if (this.secondaryCursors.length === 0 || this.lastAddedCursorIndex === 0) {
            return 0;
        }
        return this.lastAddedCursorIndex;
    };
    CursorCollection.prototype._removeSecondaryCursor = function (removeIndex) {
        if (this.lastAddedCursorIndex >= removeIndex + 1) {
            this.lastAddedCursorIndex--;
        }
        this.secondaryCursors[removeIndex].dispose(this.context);
        this.secondaryCursors.splice(removeIndex, 1);
    };
    CursorCollection.prototype._getAll = function () {
        var result = [];
        result[0] = this.primaryCursor;
        for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
            result[i + 1] = this.secondaryCursors[i];
        }
        return result;
    };
    CursorCollection.prototype.normalize = function () {
        if (this.secondaryCursors.length === 0) {
            return;
        }
        var cursors = this._getAll();
        var sortedCursors = [];
        for (var i = 0, len = cursors.length; i < len; i++) {
            sortedCursors.push({
                index: i,
                selection: cursors[i].modelState.selection,
            });
        }
        sortedCursors.sort(function (a, b) {
            if (a.selection.startLineNumber === b.selection.startLineNumber) {
                return a.selection.startColumn - b.selection.startColumn;
            }
            return a.selection.startLineNumber - b.selection.startLineNumber;
        });
        for (var sortedCursorIndex = 0; sortedCursorIndex < sortedCursors.length - 1; sortedCursorIndex++) {
            var current = sortedCursors[sortedCursorIndex];
            var next = sortedCursors[sortedCursorIndex + 1];
            var currentSelection = current.selection;
            var nextSelection = next.selection;
            if (!this.context.config.multiCursorMergeOverlapping) {
                continue;
            }
            var shouldMergeCursors = void 0;
            if (nextSelection.isEmpty() || currentSelection.isEmpty()) {
                // Merge touching cursors if one of them is collapsed
                shouldMergeCursors = nextSelection.getStartPosition().isBeforeOrEqual(currentSelection.getEndPosition());
            }
            else {
                // Merge only overlapping cursors (i.e. allow touching ranges)
                shouldMergeCursors = nextSelection.getStartPosition().isBefore(currentSelection.getEndPosition());
            }
            if (shouldMergeCursors) {
                var winnerSortedCursorIndex = current.index < next.index ? sortedCursorIndex : sortedCursorIndex + 1;
                var looserSortedCursorIndex = current.index < next.index ? sortedCursorIndex + 1 : sortedCursorIndex;
                var looserIndex = sortedCursors[looserSortedCursorIndex].index;
                var winnerIndex = sortedCursors[winnerSortedCursorIndex].index;
                var looserSelection = sortedCursors[looserSortedCursorIndex].selection;
                var winnerSelection = sortedCursors[winnerSortedCursorIndex].selection;
                if (!looserSelection.equalsSelection(winnerSelection)) {
                    var resultingRange = looserSelection.plusRange(winnerSelection);
                    var looserSelectionIsLTR = (looserSelection.selectionStartLineNumber === looserSelection.startLineNumber && looserSelection.selectionStartColumn === looserSelection.startColumn);
                    var winnerSelectionIsLTR = (winnerSelection.selectionStartLineNumber === winnerSelection.startLineNumber && winnerSelection.selectionStartColumn === winnerSelection.startColumn);
                    // Give more importance to the last added cursor (think Ctrl-dragging + hitting another cursor)
                    var resultingSelectionIsLTR = void 0;
                    if (looserIndex === this.lastAddedCursorIndex) {
                        resultingSelectionIsLTR = looserSelectionIsLTR;
                        this.lastAddedCursorIndex = winnerIndex;
                    }
                    else {
                        // Winner takes it all
                        resultingSelectionIsLTR = winnerSelectionIsLTR;
                    }
                    var resultingSelection = void 0;
                    if (resultingSelectionIsLTR) {
                        resultingSelection = new Selection(resultingRange.startLineNumber, resultingRange.startColumn, resultingRange.endLineNumber, resultingRange.endColumn);
                    }
                    else {
                        resultingSelection = new Selection(resultingRange.endLineNumber, resultingRange.endColumn, resultingRange.startLineNumber, resultingRange.startColumn);
                    }
                    sortedCursors[winnerSortedCursorIndex].selection = resultingSelection;
                    var resultingState = CursorState.fromModelSelection(resultingSelection);
                    cursors[winnerIndex].setState(this.context, resultingState.modelState, resultingState.viewState);
                }
                for (var j = 0; j < sortedCursors.length; j++) {
                    if (sortedCursors[j].index > looserIndex) {
                        sortedCursors[j].index--;
                    }
                }
                cursors.splice(looserIndex, 1);
                sortedCursors.splice(looserSortedCursorIndex, 1);
                this._removeSecondaryCursor(looserIndex - 1);
                sortedCursorIndex--;
            }
        }
    };
    return CursorCollection;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$12 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function containsLineMappingChanged(events) {
    for (var i = 0, len = events.length; i < len; i++) {
        if (events[i].type === 6 /* ViewLineMappingChanged */) {
            return true;
        }
    }
    return false;
}
var CursorStateChangedEvent = /** @class */ (function () {
    function CursorStateChangedEvent(selections, source, reason) {
        this.selections = selections;
        this.source = source;
        this.reason = reason;
    }
    return CursorStateChangedEvent;
}());
/**
 * A snapshot of the cursor and the model state
 */
var CursorModelState = /** @class */ (function () {
    function CursorModelState(model, cursor) {
        this.modelVersionId = model.getVersionId();
        this.cursorState = cursor.getAll();
    }
    CursorModelState.prototype.equals = function (other) {
        if (!other) {
            return false;
        }
        if (this.modelVersionId !== other.modelVersionId) {
            return false;
        }
        if (this.cursorState.length !== other.cursorState.length) {
            return false;
        }
        for (var i = 0, len = this.cursorState.length; i < len; i++) {
            if (!this.cursorState[i].equals(other.cursorState[i])) {
                return false;
            }
        }
        return true;
    };
    return CursorModelState;
}());
var Cursor = /** @class */ (function (_super) {
    __extends$12(Cursor, _super);
    function Cursor(configuration, model, viewModel) {
        var _this = _super.call(this) || this;
        _this._onDidReachMaxCursorCount = _this._register(new Emitter());
        _this.onDidReachMaxCursorCount = _this._onDidReachMaxCursorCount.event;
        _this._onDidAttemptReadOnlyEdit = _this._register(new Emitter());
        _this.onDidAttemptReadOnlyEdit = _this._onDidAttemptReadOnlyEdit.event;
        _this._onDidChange = _this._register(new Emitter());
        _this.onDidChange = _this._onDidChange.event;
        _this._configuration = configuration;
        _this._model = model;
        _this._knownModelVersionId = _this._model.getVersionId();
        _this._viewModel = viewModel;
        _this.context = new CursorContext(_this._configuration, _this._model, _this._viewModel);
        _this._cursors = new CursorCollection(_this.context);
        _this._isHandling = false;
        _this._isDoingComposition = false;
        _this._columnSelectData = null;
        _this._prevEditOperationType = 0 /* Other */;
        _this._register(_this._model.onDidChangeRawContent(function (e) {
            _this._knownModelVersionId = e.versionId;
            if (_this._isHandling) {
                return;
            }
            var hadFlushEvent = e.containsEvent(1 /* Flush */);
            _this._onModelContentChanged(hadFlushEvent);
        }));
        _this._register(viewModel.addEventListener(function (events) {
            if (!containsLineMappingChanged(events)) {
                return;
            }
            if (_this._knownModelVersionId !== _this._model.getVersionId()) {
                // There are model change events that I didn't yet receive.
                //
                // This can happen when editing the model, and the view model receives the change events first,
                // and the view model emits line mapping changed events, all before the cursor gets a chance to
                // recover from markers.
                //
                // The model change listener above will be called soon and we'll ensure a valid cursor state there.
                return;
            }
            // Ensure valid state
            _this.setStates('viewModel', 0 /* NotSet */, _this.getAll());
        }));
        var updateCursorContext = function () {
            _this.context = new CursorContext(_this._configuration, _this._model, _this._viewModel);
            _this._cursors.updateContext(_this.context);
        };
        _this._register(_this._model.onDidChangeLanguage(function (e) {
            updateCursorContext();
        }));
        _this._register(_this._model.onDidChangeLanguageConfiguration(function () {
            updateCursorContext();
        }));
        _this._register(_this._model.onDidChangeOptions(function () {
            updateCursorContext();
        }));
        _this._register(_this._configuration.onDidChange(function (e) {
            if (CursorConfiguration.shouldRecreate(e)) {
                updateCursorContext();
            }
        }));
        return _this;
    }
    Cursor.prototype.dispose = function () {
        this._cursors.dispose();
        _super.prototype.dispose.call(this);
    };
    // ------ some getters/setters
    Cursor.prototype.getPrimaryCursor = function () {
        return this._cursors.getPrimaryCursor();
    };
    Cursor.prototype.getLastAddedCursorIndex = function () {
        return this._cursors.getLastAddedCursorIndex();
    };
    Cursor.prototype.getAll = function () {
        return this._cursors.getAll();
    };
    Cursor.prototype.setStates = function (source, reason, states) {
        if (states !== null && states.length > Cursor.MAX_CURSOR_COUNT) {
            states = states.slice(0, Cursor.MAX_CURSOR_COUNT);
            this._onDidReachMaxCursorCount.fire(void 0);
        }
        var oldState = new CursorModelState(this._model, this);
        this._cursors.setStates(states);
        this._cursors.normalize();
        this._columnSelectData = null;
        this._emitStateChangedIfNecessary(source, reason, oldState);
    };
    Cursor.prototype.setColumnSelectData = function (columnSelectData) {
        this._columnSelectData = columnSelectData;
    };
    Cursor.prototype.reveal = function (horizontal, target, scrollType) {
        this._revealRange(target, 0 /* Simple */, horizontal, scrollType);
    };
    Cursor.prototype.revealRange = function (revealHorizontal, viewRange, verticalType, scrollType) {
        this.emitCursorRevealRange(viewRange, verticalType, revealHorizontal, scrollType);
    };
    Cursor.prototype.scrollTo = function (desiredScrollTop) {
        this._viewModel.viewLayout.setScrollPositionSmooth({
            scrollTop: desiredScrollTop
        });
    };
    Cursor.prototype.saveState = function () {
        var result = [];
        var selections = this._cursors.getSelections();
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            result.push({
                inSelectionMode: !selection.isEmpty(),
                selectionStart: {
                    lineNumber: selection.selectionStartLineNumber,
                    column: selection.selectionStartColumn,
                },
                position: {
                    lineNumber: selection.positionLineNumber,
                    column: selection.positionColumn,
                }
            });
        }
        return result;
    };
    Cursor.prototype.restoreState = function (states) {
        var desiredSelections = [];
        for (var i = 0, len = states.length; i < len; i++) {
            var state = states[i];
            var positionLineNumber = 1;
            var positionColumn = 1;
            // Avoid missing properties on the literal
            if (state.position && state.position.lineNumber) {
                positionLineNumber = state.position.lineNumber;
            }
            if (state.position && state.position.column) {
                positionColumn = state.position.column;
            }
            var selectionStartLineNumber = positionLineNumber;
            var selectionStartColumn = positionColumn;
            // Avoid missing properties on the literal
            if (state.selectionStart && state.selectionStart.lineNumber) {
                selectionStartLineNumber = state.selectionStart.lineNumber;
            }
            if (state.selectionStart && state.selectionStart.column) {
                selectionStartColumn = state.selectionStart.column;
            }
            desiredSelections.push({
                selectionStartLineNumber: selectionStartLineNumber,
                selectionStartColumn: selectionStartColumn,
                positionLineNumber: positionLineNumber,
                positionColumn: positionColumn
            });
        }
        this.setStates('restoreState', 0 /* NotSet */, CursorState.fromModelSelections(desiredSelections));
        this.reveal(true, 0 /* Primary */, 1 /* Immediate */);
    };
    Cursor.prototype._onModelContentChanged = function (hadFlushEvent) {
        this._prevEditOperationType = 0 /* Other */;
        if (hadFlushEvent) {
            // a model.setValue() was called
            this._cursors.dispose();
            this._cursors = new CursorCollection(this.context);
            this._emitStateChangedIfNecessary('model', 1 /* ContentFlush */, null);
        }
        else {
            var selectionsFromMarkers = this._cursors.readSelectionFromMarkers();
            this.setStates('modelChange', 2 /* RecoverFromMarkers */, CursorState.fromModelSelections(selectionsFromMarkers));
        }
    };
    Cursor.prototype.getSelection = function () {
        return this._cursors.getPrimaryCursor().modelState.selection;
    };
    Cursor.prototype.getColumnSelectData = function () {
        if (this._columnSelectData) {
            return this._columnSelectData;
        }
        var primaryCursor = this._cursors.getPrimaryCursor();
        var primaryPos = primaryCursor.viewState.position;
        return {
            toViewLineNumber: primaryPos.lineNumber,
            toViewVisualColumn: CursorColumns.visibleColumnFromColumn2(this.context.config, this.context.viewModel, primaryPos)
        };
    };
    Cursor.prototype.getSelections = function () {
        return this._cursors.getSelections();
    };
    Cursor.prototype.getViewSelections = function () {
        return this._cursors.getViewSelections();
    };
    Cursor.prototype.getPosition = function () {
        return this._cursors.getPrimaryCursor().modelState.position;
    };
    Cursor.prototype.setSelections = function (source, selections) {
        this.setStates(source, 0 /* NotSet */, CursorState.fromModelSelections(selections));
    };
    Cursor.prototype.getPrevEditOperationType = function () {
        return this._prevEditOperationType;
    };
    Cursor.prototype.setPrevEditOperationType = function (type) {
        this._prevEditOperationType = type;
    };
    // ------ auxiliary handling logic
    Cursor.prototype._executeEditOperation = function (opResult) {
        if (!opResult) {
            // Nothing to execute
            return;
        }
        if (opResult.shouldPushStackElementBefore) {
            this._model.pushStackElement();
        }
        var result = CommandExecutor.executeCommands(this._model, this._cursors.getSelections(), opResult.commands);
        if (result) {
            // The commands were applied correctly
            this._interpretCommandResult(result);
            this._prevEditOperationType = opResult.type;
        }
        if (opResult.shouldPushStackElementAfter) {
            this._model.pushStackElement();
        }
    };
    Cursor.prototype._interpretCommandResult = function (cursorState) {
        if (!cursorState || cursorState.length === 0) {
            cursorState = this._cursors.readSelectionFromMarkers();
        }
        this._columnSelectData = null;
        this._cursors.setSelections(cursorState);
        this._cursors.normalize();
    };
    // -----------------------------------------------------------------------------------------------------------
    // ----- emitting events
    Cursor.prototype._emitStateChangedIfNecessary = function (source, reason, oldState) {
        var newState = new CursorModelState(this._model, this);
        if (newState.equals(oldState)) {
            return false;
        }
        var selections = this._cursors.getSelections();
        var viewSelections = this._cursors.getViewSelections();
        // Let the view get the event first.
        try {
            var eventsCollector = this._beginEmit();
            eventsCollector.emit(new ViewCursorStateChangedEvent(viewSelections));
        }
        finally {
            this._endEmit();
        }
        // Only after the view has been notified, let the rest of the world know...
        if (!oldState
            || oldState.cursorState.length !== newState.cursorState.length
            || newState.cursorState.some(function (newCursorState, i) { return !newCursorState.modelState.equals(oldState.cursorState[i].modelState); })) {
            this._onDidChange.fire(new CursorStateChangedEvent(selections, source || 'keyboard', reason));
        }
        return true;
    };
    Cursor.prototype._revealRange = function (revealTarget, verticalType, revealHorizontal, scrollType) {
        var viewPositions = this._cursors.getViewPositions();
        var viewPosition = viewPositions[0];
        if (revealTarget === 1 /* TopMost */) {
            for (var i = 1; i < viewPositions.length; i++) {
                if (viewPositions[i].isBefore(viewPosition)) {
                    viewPosition = viewPositions[i];
                }
            }
        }
        else if (revealTarget === 2 /* BottomMost */) {
            for (var i = 1; i < viewPositions.length; i++) {
                if (viewPosition.isBeforeOrEqual(viewPositions[i])) {
                    viewPosition = viewPositions[i];
                }
            }
        }
        else {
            if (viewPositions.length > 1) {
                // no revealing!
                return;
            }
        }
        var viewRange = new Range(viewPosition.lineNumber, viewPosition.column, viewPosition.lineNumber, viewPosition.column);
        this.emitCursorRevealRange(viewRange, verticalType, revealHorizontal, scrollType);
    };
    Cursor.prototype.emitCursorRevealRange = function (viewRange, verticalType, revealHorizontal, scrollType) {
        try {
            var eventsCollector = this._beginEmit();
            eventsCollector.emit(new ViewRevealRangeRequestEvent(viewRange, verticalType, revealHorizontal, scrollType));
        }
        finally {
            this._endEmit();
        }
    };
    // -----------------------------------------------------------------------------------------------------------
    // ----- handlers beyond this point
    Cursor.prototype.trigger = function (source, handlerId, payload) {
        var H = Handler;
        if (handlerId === H.CompositionStart) {
            this._isDoingComposition = true;
            return;
        }
        if (handlerId === H.CompositionEnd) {
            this._isDoingComposition = false;
        }
        if (this._configuration.editor.readOnly) {
            // All the remaining handlers will try to edit the model,
            // but we cannot edit when read only...
            this._onDidAttemptReadOnlyEdit.fire(void 0);
            return;
        }
        var oldState = new CursorModelState(this._model, this);
        var cursorChangeReason = 0 /* NotSet */;
        if (handlerId !== H.Undo && handlerId !== H.Redo) {
            // TODO@Alex: if the undo/redo stack contains non-null selections
            // it would also be OK to stop tracking selections here
            this._cursors.stopTrackingSelections();
        }
        // ensure valid state on all cursors
        this._cursors.ensureValidState();
        this._isHandling = true;
        try {
            switch (handlerId) {
                case H.Type:
                    this._type(source, payload.text);
                    break;
                case H.ReplacePreviousChar:
                    this._replacePreviousChar(payload.text, payload.replaceCharCnt);
                    break;
                case H.Paste:
                    cursorChangeReason = 4 /* Paste */;
                    this._paste(payload.text, payload.pasteOnNewLine, payload.multicursorText);
                    break;
                case H.Cut:
                    this._cut();
                    break;
                case H.Undo:
                    cursorChangeReason = 5 /* Undo */;
                    this._interpretCommandResult(this._model.undo());
                    break;
                case H.Redo:
                    cursorChangeReason = 6 /* Redo */;
                    this._interpretCommandResult(this._model.redo());
                    break;
                case H.ExecuteCommand:
                    this._externalExecuteCommand(payload);
                    break;
                case H.ExecuteCommands:
                    this._externalExecuteCommands(payload);
                    break;
                case H.CompositionEnd:
                    this._interpretCompositionEnd(source);
                    break;
            }
        }
        catch (err) {
            onUnexpectedError(err);
        }
        this._isHandling = false;
        if (handlerId !== H.Undo && handlerId !== H.Redo) {
            this._cursors.startTrackingSelections();
        }
        if (this._emitStateChangedIfNecessary(source, cursorChangeReason, oldState)) {
            this._revealRange(0 /* Primary */, 0 /* Simple */, true, 0 /* Smooth */);
        }
    };
    Cursor.prototype._interpretCompositionEnd = function (source) {
        if (!this._isDoingComposition && source === 'keyboard') {
            // composition finishes, let's check if we need to auto complete if necessary.
            this._executeEditOperation(TypeOperations.compositionEndWithInterceptors(this._prevEditOperationType, this.context.config, this.context.model, this.getSelections()));
        }
    };
    Cursor.prototype._type = function (source, text) {
        if (!this._isDoingComposition && source === 'keyboard') {
            // If this event is coming straight from the keyboard, look for electric characters and enter
            for (var i = 0, len = text.length; i < len; i++) {
                var charCode = text.charCodeAt(i);
                var chr = void 0;
                if (isHighSurrogate(charCode) && i + 1 < len) {
                    chr = text.charAt(i) + text.charAt(i + 1);
                    i++;
                }
                else {
                    chr = text.charAt(i);
                }
                // Here we must interpret each typed character individually, that's why we create a new context
                this._executeEditOperation(TypeOperations.typeWithInterceptors(this._prevEditOperationType, this.context.config, this.context.model, this.getSelections(), chr));
            }
        }
        else {
            this._executeEditOperation(TypeOperations.typeWithoutInterceptors(this._prevEditOperationType, this.context.config, this.context.model, this.getSelections(), text));
        }
    };
    Cursor.prototype._replacePreviousChar = function (text, replaceCharCnt) {
        this._executeEditOperation(TypeOperations.replacePreviousChar(this._prevEditOperationType, this.context.config, this.context.model, this.getSelections(), text, replaceCharCnt));
    };
    Cursor.prototype._paste = function (text, pasteOnNewLine, multicursorText) {
        this._executeEditOperation(TypeOperations.paste(this.context.config, this.context.model, this.getSelections(), text, pasteOnNewLine, multicursorText));
    };
    Cursor.prototype._cut = function () {
        this._executeEditOperation(DeleteOperations.cut(this.context.config, this.context.model, this.getSelections()));
    };
    Cursor.prototype._externalExecuteCommand = function (command) {
        this._cursors.killSecondaryCursors();
        this._executeEditOperation(new EditOperationResult(0 /* Other */, [command], {
            shouldPushStackElementBefore: false,
            shouldPushStackElementAfter: false
        }));
    };
    Cursor.prototype._externalExecuteCommands = function (commands) {
        this._executeEditOperation(new EditOperationResult(0 /* Other */, commands, {
            shouldPushStackElementBefore: false,
            shouldPushStackElementAfter: false
        }));
    };
    Cursor.MAX_CURSOR_COUNT = 10000;
    return Cursor;
}(ViewEventEmitter));
var CommandExecutor = /** @class */ (function () {
    function CommandExecutor() {
    }
    CommandExecutor.executeCommands = function (model, selectionsBefore, commands) {
        var ctx = {
            model: model,
            selectionsBefore: selectionsBefore,
            trackedRanges: [],
            trackedRangesDirection: []
        };
        var result = this._innerExecuteCommands(ctx, commands);
        for (var i = 0, len = ctx.trackedRanges.length; i < len; i++) {
            ctx.model._setTrackedRange(ctx.trackedRanges[i], null, 0 /* AlwaysGrowsWhenTypingAtEdges */);
        }
        return result;
    };
    CommandExecutor._innerExecuteCommands = function (ctx, commands) {
        if (this._arrayIsEmpty(commands)) {
            return null;
        }
        var commandsData = this._getEditOperations(ctx, commands);
        if (commandsData.operations.length === 0) {
            return null;
        }
        var rawOperations = commandsData.operations;
        var loserCursorsMap = this._getLoserCursorMap(rawOperations);
        if (loserCursorsMap.hasOwnProperty('0')) {
            // These commands are very messed up
            console.warn('Ignoring commands');
            return null;
        }
        // Remove operations belonging to losing cursors
        var filteredOperations = [];
        for (var i = 0, len = rawOperations.length; i < len; i++) {
            if (!loserCursorsMap.hasOwnProperty(rawOperations[i].identifier.major.toString())) {
                filteredOperations.push(rawOperations[i]);
            }
        }
        // TODO@Alex: find a better way to do this.
        // give the hint that edit operations are tracked to the model
        if (commandsData.hadTrackedEditOperation && filteredOperations.length > 0) {
            filteredOperations[0]._isTracked = true;
        }
        var selectionsAfter = ctx.model.pushEditOperations(ctx.selectionsBefore, filteredOperations, function (inverseEditOperations) {
            var groupedInverseEditOperations = [];
            for (var i = 0; i < ctx.selectionsBefore.length; i++) {
                groupedInverseEditOperations[i] = [];
            }
            for (var i = 0; i < inverseEditOperations.length; i++) {
                var op = inverseEditOperations[i];
                if (!op.identifier) {
                    // perhaps auto whitespace trim edits
                    continue;
                }
                groupedInverseEditOperations[op.identifier.major].push(op);
            }
            var minorBasedSorter = function (a, b) {
                return a.identifier.minor - b.identifier.minor;
            };
            var cursorSelections = [];
            var _loop_1 = function (i) {
                if (groupedInverseEditOperations[i].length > 0) {
                    groupedInverseEditOperations[i].sort(minorBasedSorter);
                    cursorSelections[i] = commands[i].computeCursorState(ctx.model, {
                        getInverseEditOperations: function () {
                            return groupedInverseEditOperations[i];
                        },
                        getTrackedSelection: function (id) {
                            var idx = parseInt(id, 10);
                            var range = ctx.model._getTrackedRange(ctx.trackedRanges[idx]);
                            if (ctx.trackedRangesDirection[idx] === 0 /* LTR */) {
                                return new Selection(range.startLineNumber, range.startColumn, range.endLineNumber, range.endColumn);
                            }
                            return new Selection(range.endLineNumber, range.endColumn, range.startLineNumber, range.startColumn);
                        }
                    });
                }
                else {
                    cursorSelections[i] = ctx.selectionsBefore[i];
                }
            };
            for (var i = 0; i < ctx.selectionsBefore.length; i++) {
                _loop_1(i);
            }
            return cursorSelections;
        });
        if (!selectionsAfter) {
            selectionsAfter = ctx.selectionsBefore;
        }
        // Extract losing cursors
        var losingCursors = [];
        for (var losingCursorIndex in loserCursorsMap) {
            if (loserCursorsMap.hasOwnProperty(losingCursorIndex)) {
                losingCursors.push(parseInt(losingCursorIndex, 10));
            }
        }
        // Sort losing cursors descending
        losingCursors.sort(function (a, b) {
            return b - a;
        });
        // Remove losing cursors
        for (var i = 0; i < losingCursors.length; i++) {
            selectionsAfter.splice(losingCursors[i], 1);
        }
        return selectionsAfter;
    };
    CommandExecutor._arrayIsEmpty = function (commands) {
        for (var i = 0, len = commands.length; i < len; i++) {
            if (commands[i]) {
                return false;
            }
        }
        return true;
    };
    CommandExecutor._getEditOperations = function (ctx, commands) {
        var operations = [];
        var hadTrackedEditOperation = false;
        for (var i = 0, len = commands.length; i < len; i++) {
            var command = commands[i];
            if (command) {
                var r = this._getEditOperationsFromCommand(ctx, i, command);
                operations = operations.concat(r.operations);
                hadTrackedEditOperation = hadTrackedEditOperation || r.hadTrackedEditOperation;
            }
        }
        return {
            operations: operations,
            hadTrackedEditOperation: hadTrackedEditOperation
        };
    };
    CommandExecutor._getEditOperationsFromCommand = function (ctx, majorIdentifier, command) {
        // This method acts as a transaction, if the command fails
        // everything it has done is ignored
        var operations = [];
        var operationMinor = 0;
        var addEditOperation = function (selection, text) {
            if (selection.isEmpty() && text === '') {
                // This command wants to add a no-op => no thank you
                return;
            }
            operations.push({
                identifier: {
                    major: majorIdentifier,
                    minor: operationMinor++
                },
                range: selection,
                text: text,
                forceMoveMarkers: false,
                isAutoWhitespaceEdit: command.insertsAutoWhitespace
            });
        };
        var hadTrackedEditOperation = false;
        var addTrackedEditOperation = function (selection, text) {
            hadTrackedEditOperation = true;
            addEditOperation(selection, text);
        };
        var trackSelection = function (selection, trackPreviousOnEmpty) {
            var stickiness;
            if (selection.isEmpty()) {
                if (typeof trackPreviousOnEmpty === 'boolean') {
                    if (trackPreviousOnEmpty) {
                        stickiness = 2 /* GrowsOnlyWhenTypingBefore */;
                    }
                    else {
                        stickiness = 3 /* GrowsOnlyWhenTypingAfter */;
                    }
                }
                else {
                    // Try to lock it with surrounding text
                    var maxLineColumn = ctx.model.getLineMaxColumn(selection.startLineNumber);
                    if (selection.startColumn === maxLineColumn) {
                        stickiness = 2 /* GrowsOnlyWhenTypingBefore */;
                    }
                    else {
                        stickiness = 3 /* GrowsOnlyWhenTypingAfter */;
                    }
                }
            }
            else {
                stickiness = 1 /* NeverGrowsWhenTypingAtEdges */;
            }
            var l = ctx.trackedRanges.length;
            var id = ctx.model._setTrackedRange(null, selection, stickiness);
            ctx.trackedRanges[l] = id;
            ctx.trackedRangesDirection[l] = selection.getDirection();
            return l.toString();
        };
        var editOperationBuilder = {
            addEditOperation: addEditOperation,
            addTrackedEditOperation: addTrackedEditOperation,
            trackSelection: trackSelection
        };
        try {
            command.getEditOperations(ctx.model, editOperationBuilder);
        }
        catch (e) {
            e.friendlyMessage = localize('corrupt.commands', "Unexpected exception while executing command.");
            onUnexpectedError(e);
            return {
                operations: [],
                hadTrackedEditOperation: false
            };
        }
        return {
            operations: operations,
            hadTrackedEditOperation: hadTrackedEditOperation
        };
    };
    CommandExecutor._getLoserCursorMap = function (operations) {
        // This is destructive on the array
        operations = operations.slice(0);
        // Sort operations with last one first
        operations.sort(function (a, b) {
            // Note the minus!
            return -(Range.compareRangesUsingEnds(a.range, b.range));
        });
        // Operations can not overlap!
        var loserCursorsMap = {};
        for (var i = 1; i < operations.length; i++) {
            var previousOp = operations[i - 1];
            var currentOp = operations[i];
            if (previousOp.range.getStartPosition().isBefore(currentOp.range.getEndPosition())) {
                var loserMajor = void 0;
                if (previousOp.identifier.major > currentOp.identifier.major) {
                    // previousOp loses the battle
                    loserMajor = previousOp.identifier.major;
                }
                else {
                    loserMajor = currentOp.identifier.major;
                }
                loserCursorsMap[loserMajor.toString()] = true;
                for (var j = 0; j < operations.length; j++) {
                    if (operations[j].identifier.major === loserMajor) {
                        operations.splice(j, 1);
                        if (j < i) {
                            i--;
                        }
                        j--;
                    }
                }
                if (i > 0) {
                    i--;
                }
            }
        }
        return loserCursorsMap;
    };
    return CommandExecutor;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var InternalEditorAction = /** @class */ (function () {
    function InternalEditorAction(id, label, alias, precondition, run, contextKeyService) {
        this.id = id;
        this.label = label;
        this.alias = alias;
        this._precondition = precondition;
        this._run = run;
        this._contextKeyService = contextKeyService;
    }
    InternalEditorAction.prototype.isSupported = function () {
        return this._contextKeyService.contextMatchesRules(this._precondition);
    };
    InternalEditorAction.prototype.run = function () {
        if (!this.isSupported()) {
            return Promise.resolve(void 0);
        }
        var r = this._run();
        return r ? r : Promise.resolve(void 0);
    };
    return InternalEditorAction;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var fallback = {
    getInitialState: function () { return NULL_STATE; },
    tokenize2: function (buffer, state, deltaOffset) { return nullTokenize2(0 /* Null */, buffer, state, deltaOffset); }
};
function tokenizeToString(text, tokenizationSupport) {
    if (tokenizationSupport === void 0) { tokenizationSupport = fallback; }
    return _tokenizeToString(text, tokenizationSupport || fallback);
}
function tokenizeLineToHTML(text, viewLineTokens, colorMap, startOffset, endOffset, tabSize) {
    var result = "<div>";
    var charIndex = startOffset;
    var tabsCharDelta = 0;
    for (var tokenIndex = 0, tokenCount = viewLineTokens.getCount(); tokenIndex < tokenCount; tokenIndex++) {
        var tokenEndIndex = viewLineTokens.getEndOffset(tokenIndex);
        if (tokenEndIndex <= startOffset) {
            continue;
        }
        var partContent = '';
        for (; charIndex < tokenEndIndex && charIndex < endOffset; charIndex++) {
            var charCode = text.charCodeAt(charIndex);
            switch (charCode) {
                case 9 /* Tab */:
                    var insertSpacesCount = tabSize - (charIndex + tabsCharDelta) % tabSize;
                    tabsCharDelta += insertSpacesCount - 1;
                    while (insertSpacesCount > 0) {
                        partContent += '&nbsp;';
                        insertSpacesCount--;
                    }
                    break;
                case 60 /* LessThan */:
                    partContent += '&lt;';
                    break;
                case 62 /* GreaterThan */:
                    partContent += '&gt;';
                    break;
                case 38 /* Ampersand */:
                    partContent += '&amp;';
                    break;
                case 0 /* Null */:
                    partContent += '&#00;';
                    break;
                case 65279 /* UTF8_BOM */:
                case 8232 /* LINE_SEPARATOR_2028 */:
                    partContent += '\ufffd';
                    break;
                case 13 /* CarriageReturn */:
                    // zero width space, because carriage return would introduce a line break
                    partContent += '&#8203';
                    break;
                default:
                    partContent += String.fromCharCode(charCode);
            }
        }
        result += "<span style=\"" + viewLineTokens.getInlineStyle(tokenIndex, colorMap) + "\">" + partContent + "</span>";
        if (tokenEndIndex > endOffset || charIndex >= endOffset) {
            break;
        }
    }
    result += "</div>";
    return result;
}
function _tokenizeToString(text, tokenizationSupport) {
    var result = "<div class=\"monaco-tokenized-source\">";
    var lines = text.split(/\r\n|\r|\n/);
    var currentState = tokenizationSupport.getInitialState();
    for (var i = 0, len = lines.length; i < len; i++) {
        var line = lines[i];
        if (i > 0) {
            result += "<br/>";
        }
        var tokenizationResult = tokenizationSupport.tokenize2(line, currentState, 0);
        LineTokens.convertToEndOffset(tokenizationResult.tokens, line.length);
        var lineTokens = new LineTokens(tokenizationResult.tokens, line);
        var viewLineTokens = lineTokens.inflate();
        var startOffset = 0;
        for (var j = 0, lenJ = viewLineTokens.getCount(); j < lenJ; j++) {
            var type = viewLineTokens.getClassName(j);
            var endIndex = viewLineTokens.getEndOffset(j);
            result += "<span class=\"" + type + "\">" + escape(line.substring(startOffset, endIndex)) + "</span>";
            startOffset = endIndex;
        }
        currentState = tokenizationResult.endState;
    }
    result += "</div>";
    return result;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Represent whitespaces in between lines and provide fast CRUD management methods.
 * The whitespaces are sorted ascending by `afterLineNumber`.
 */
var WhitespaceComputer = /** @class */ (function () {
    function WhitespaceComputer() {
        this._heights = [];
        this._minWidths = [];
        this._ids = [];
        this._afterLineNumbers = [];
        this._ordinals = [];
        this._prefixSum = [];
        this._prefixSumValidIndex = -1;
        this._whitespaceId2Index = {};
        this._lastWhitespaceId = 0;
        this._minWidth = -1; /* marker for not being computed */
    }
    /**
     * Find the insertion index for a new value inside a sorted array of values.
     * If the value is already present in the sorted array, the insertion index will be after the already existing value.
     */
    WhitespaceComputer.findInsertionIndex = function (sortedArray, value, ordinals, valueOrdinal) {
        var low = 0;
        var high = sortedArray.length;
        while (low < high) {
            var mid = ((low + high) >>> 1);
            if (value === sortedArray[mid]) {
                if (valueOrdinal < ordinals[mid]) {
                    high = mid;
                }
                else {
                    low = mid + 1;
                }
            }
            else if (value < sortedArray[mid]) {
                high = mid;
            }
            else {
                low = mid + 1;
            }
        }
        return low;
    };
    /**
     * Insert a new whitespace of a certain height after a line number.
     * The whitespace has a "sticky" characteristic.
     * Irrespective of edits above or below `afterLineNumber`, the whitespace will follow the initial line.
     *
     * @param afterLineNumber The conceptual position of this whitespace. The whitespace will follow this line as best as possible even when deleting/inserting lines above/below.
     * @param heightInPx The height of the whitespace, in pixels.
     * @return An id that can be used later to mutate or delete the whitespace
     */
    WhitespaceComputer.prototype.insertWhitespace = function (afterLineNumber, ordinal, heightInPx, minWidth) {
        afterLineNumber = afterLineNumber | 0;
        ordinal = ordinal | 0;
        heightInPx = heightInPx | 0;
        minWidth = minWidth | 0;
        var id = (++this._lastWhitespaceId);
        var insertionIndex = WhitespaceComputer.findInsertionIndex(this._afterLineNumbers, afterLineNumber, this._ordinals, ordinal);
        this._insertWhitespaceAtIndex(id, insertionIndex, afterLineNumber, ordinal, heightInPx, minWidth);
        this._minWidth = -1; /* marker for not being computed */
        return id;
    };
    WhitespaceComputer.prototype._insertWhitespaceAtIndex = function (id, insertIndex, afterLineNumber, ordinal, heightInPx, minWidth) {
        id = id | 0;
        insertIndex = insertIndex | 0;
        afterLineNumber = afterLineNumber | 0;
        ordinal = ordinal | 0;
        heightInPx = heightInPx | 0;
        minWidth = minWidth | 0;
        this._heights.splice(insertIndex, 0, heightInPx);
        this._minWidths.splice(insertIndex, 0, minWidth);
        this._ids.splice(insertIndex, 0, id);
        this._afterLineNumbers.splice(insertIndex, 0, afterLineNumber);
        this._ordinals.splice(insertIndex, 0, ordinal);
        this._prefixSum.splice(insertIndex, 0, 0);
        var keys = Object.keys(this._whitespaceId2Index);
        for (var i = 0, len = keys.length; i < len; i++) {
            var sid = keys[i];
            var oldIndex = this._whitespaceId2Index[sid];
            if (oldIndex >= insertIndex) {
                this._whitespaceId2Index[sid] = oldIndex + 1;
            }
        }
        this._whitespaceId2Index[id.toString()] = insertIndex;
        this._prefixSumValidIndex = Math.min(this._prefixSumValidIndex, insertIndex - 1);
    };
    /**
     * Change properties associated with a certain whitespace.
     */
    WhitespaceComputer.prototype.changeWhitespace = function (id, newAfterLineNumber, newHeight) {
        id = id | 0;
        newAfterLineNumber = newAfterLineNumber | 0;
        newHeight = newHeight | 0;
        var hasChanges = false;
        hasChanges = this.changeWhitespaceHeight(id, newHeight) || hasChanges;
        hasChanges = this.changeWhitespaceAfterLineNumber(id, newAfterLineNumber) || hasChanges;
        return hasChanges;
    };
    /**
     * Change the height of an existing whitespace
     *
     * @param id The whitespace to change
     * @param newHeightInPx The new height of the whitespace, in pixels
     * @return Returns true if the whitespace is found and if the new height is different than the old height
     */
    WhitespaceComputer.prototype.changeWhitespaceHeight = function (id, newHeightInPx) {
        id = id | 0;
        newHeightInPx = newHeightInPx | 0;
        var sid = id.toString();
        if (this._whitespaceId2Index.hasOwnProperty(sid)) {
            var index = this._whitespaceId2Index[sid];
            if (this._heights[index] !== newHeightInPx) {
                this._heights[index] = newHeightInPx;
                this._prefixSumValidIndex = Math.min(this._prefixSumValidIndex, index - 1);
                return true;
            }
        }
        return false;
    };
    /**
     * Change the line number after which an existing whitespace flows.
     *
     * @param id The whitespace to change
     * @param newAfterLineNumber The new line number the whitespace will follow
     * @return Returns true if the whitespace is found and if the new line number is different than the old line number
     */
    WhitespaceComputer.prototype.changeWhitespaceAfterLineNumber = function (id, newAfterLineNumber) {
        id = id | 0;
        newAfterLineNumber = newAfterLineNumber | 0;
        var sid = id.toString();
        if (this._whitespaceId2Index.hasOwnProperty(sid)) {
            var index = this._whitespaceId2Index[sid];
            if (this._afterLineNumbers[index] !== newAfterLineNumber) {
                // `afterLineNumber` changed for this whitespace
                // Record old ordinal
                var ordinal = this._ordinals[index];
                // Record old height
                var heightInPx = this._heights[index];
                // Record old min width
                var minWidth = this._minWidths[index];
                // Since changing `afterLineNumber` can trigger a reordering, we're gonna remove this whitespace
                this.removeWhitespace(id);
                // And add it again
                var insertionIndex = WhitespaceComputer.findInsertionIndex(this._afterLineNumbers, newAfterLineNumber, this._ordinals, ordinal);
                this._insertWhitespaceAtIndex(id, insertionIndex, newAfterLineNumber, ordinal, heightInPx, minWidth);
                return true;
            }
        }
        return false;
    };
    /**
     * Remove an existing whitespace.
     *
     * @param id The whitespace to remove
     * @return Returns true if the whitespace is found and it is removed.
     */
    WhitespaceComputer.prototype.removeWhitespace = function (id) {
        id = id | 0;
        var sid = id.toString();
        if (this._whitespaceId2Index.hasOwnProperty(sid)) {
            var index = this._whitespaceId2Index[sid];
            delete this._whitespaceId2Index[sid];
            this._removeWhitespaceAtIndex(index);
            this._minWidth = -1; /* marker for not being computed */
            return true;
        }
        return false;
    };
    WhitespaceComputer.prototype._removeWhitespaceAtIndex = function (removeIndex) {
        removeIndex = removeIndex | 0;
        this._heights.splice(removeIndex, 1);
        this._minWidths.splice(removeIndex, 1);
        this._ids.splice(removeIndex, 1);
        this._afterLineNumbers.splice(removeIndex, 1);
        this._ordinals.splice(removeIndex, 1);
        this._prefixSum.splice(removeIndex, 1);
        this._prefixSumValidIndex = Math.min(this._prefixSumValidIndex, removeIndex - 1);
        var keys = Object.keys(this._whitespaceId2Index);
        for (var i = 0, len = keys.length; i < len; i++) {
            var sid = keys[i];
            var oldIndex = this._whitespaceId2Index[sid];
            if (oldIndex >= removeIndex) {
                this._whitespaceId2Index[sid] = oldIndex - 1;
            }
        }
    };
    /**
     * Notify the computer that lines have been deleted (a continuous zone of lines).
     * This gives it a chance to update `afterLineNumber` for whitespaces, giving the "sticky" characteristic.
     *
     * @param fromLineNumber The line number at which the deletion started, inclusive
     * @param toLineNumber The line number at which the deletion ended, inclusive
     */
    WhitespaceComputer.prototype.onLinesDeleted = function (fromLineNumber, toLineNumber) {
        fromLineNumber = fromLineNumber | 0;
        toLineNumber = toLineNumber | 0;
        for (var i = 0, len = this._afterLineNumbers.length; i < len; i++) {
            var afterLineNumber = this._afterLineNumbers[i];
            if (fromLineNumber <= afterLineNumber && afterLineNumber <= toLineNumber) {
                // The line this whitespace was after has been deleted
                //  => move whitespace to before first deleted line
                this._afterLineNumbers[i] = fromLineNumber - 1;
            }
            else if (afterLineNumber > toLineNumber) {
                // The line this whitespace was after has been moved up
                //  => move whitespace up
                this._afterLineNumbers[i] -= (toLineNumber - fromLineNumber + 1);
            }
        }
    };
    /**
     * Notify the computer that lines have been inserted (a continuous zone of lines).
     * This gives it a chance to update `afterLineNumber` for whitespaces, giving the "sticky" characteristic.
     *
     * @param fromLineNumber The line number at which the insertion started, inclusive
     * @param toLineNumber The line number at which the insertion ended, inclusive.
     */
    WhitespaceComputer.prototype.onLinesInserted = function (fromLineNumber, toLineNumber) {
        fromLineNumber = fromLineNumber | 0;
        toLineNumber = toLineNumber | 0;
        for (var i = 0, len = this._afterLineNumbers.length; i < len; i++) {
            var afterLineNumber = this._afterLineNumbers[i];
            if (fromLineNumber <= afterLineNumber) {
                this._afterLineNumbers[i] += (toLineNumber - fromLineNumber + 1);
            }
        }
    };
    /**
     * Get the sum of all the whitespaces.
     */
    WhitespaceComputer.prototype.getTotalHeight = function () {
        if (this._heights.length === 0) {
            return 0;
        }
        return this.getAccumulatedHeight(this._heights.length - 1);
    };
    /**
     * Return the sum of the heights of the whitespaces at [0..index].
     * This includes the whitespace at `index`.
     *
     * @param index The index of the whitespace.
     * @return The sum of the heights of all whitespaces before the one at `index`, including the one at `index`.
     */
    WhitespaceComputer.prototype.getAccumulatedHeight = function (index) {
        index = index | 0;
        var startIndex = Math.max(0, this._prefixSumValidIndex + 1);
        if (startIndex === 0) {
            this._prefixSum[0] = this._heights[0];
            startIndex++;
        }
        for (var i = startIndex; i <= index; i++) {
            this._prefixSum[i] = this._prefixSum[i - 1] + this._heights[i];
        }
        this._prefixSumValidIndex = Math.max(this._prefixSumValidIndex, index);
        return this._prefixSum[index];
    };
    /**
     * Find all whitespaces with `afterLineNumber` < `lineNumber` and return the sum of their heights.
     *
     * @param lineNumber The line number whitespaces should be before.
     * @return The sum of the heights of the whitespaces before `lineNumber`.
     */
    WhitespaceComputer.prototype.getAccumulatedHeightBeforeLineNumber = function (lineNumber) {
        lineNumber = lineNumber | 0;
        var lastWhitespaceBeforeLineNumber = this._findLastWhitespaceBeforeLineNumber(lineNumber);
        if (lastWhitespaceBeforeLineNumber === -1) {
            return 0;
        }
        return this.getAccumulatedHeight(lastWhitespaceBeforeLineNumber);
    };
    WhitespaceComputer.prototype._findLastWhitespaceBeforeLineNumber = function (lineNumber) {
        lineNumber = lineNumber | 0;
        // Find the whitespace before line number
        var afterLineNumbers = this._afterLineNumbers;
        var low = 0;
        var high = afterLineNumbers.length - 1;
        while (low <= high) {
            var delta = (high - low) | 0;
            var halfDelta = (delta / 2) | 0;
            var mid = (low + halfDelta) | 0;
            if (afterLineNumbers[mid] < lineNumber) {
                if (mid + 1 >= afterLineNumbers.length || afterLineNumbers[mid + 1] >= lineNumber) {
                    return mid;
                }
                else {
                    low = (mid + 1) | 0;
                }
            }
            else {
                high = (mid - 1) | 0;
            }
        }
        return -1;
    };
    WhitespaceComputer.prototype._findFirstWhitespaceAfterLineNumber = function (lineNumber) {
        lineNumber = lineNumber | 0;
        var lastWhitespaceBeforeLineNumber = this._findLastWhitespaceBeforeLineNumber(lineNumber);
        var firstWhitespaceAfterLineNumber = lastWhitespaceBeforeLineNumber + 1;
        if (firstWhitespaceAfterLineNumber < this._heights.length) {
            return firstWhitespaceAfterLineNumber;
        }
        return -1;
    };
    /**
     * Find the index of the first whitespace which has `afterLineNumber` >= `lineNumber`.
     * @return The index of the first whitespace with `afterLineNumber` >= `lineNumber` or -1 if no whitespace is found.
     */
    WhitespaceComputer.prototype.getFirstWhitespaceIndexAfterLineNumber = function (lineNumber) {
        lineNumber = lineNumber | 0;
        return this._findFirstWhitespaceAfterLineNumber(lineNumber);
    };
    /**
     * The number of whitespaces.
     */
    WhitespaceComputer.prototype.getCount = function () {
        return this._heights.length;
    };
    /**
     * The maximum min width for all whitespaces.
     */
    WhitespaceComputer.prototype.getMinWidth = function () {
        if (this._minWidth === -1) {
            var minWidth = 0;
            for (var i = 0, len = this._minWidths.length; i < len; i++) {
                minWidth = Math.max(minWidth, this._minWidths[i]);
            }
            this._minWidth = minWidth;
        }
        return this._minWidth;
    };
    /**
     * Get the `afterLineNumber` for whitespace at index `index`.
     *
     * @param index The index of the whitespace.
     * @return `afterLineNumber` of whitespace at `index`.
     */
    WhitespaceComputer.prototype.getAfterLineNumberForWhitespaceIndex = function (index) {
        index = index | 0;
        return this._afterLineNumbers[index];
    };
    /**
     * Get the `id` for whitespace at index `index`.
     *
     * @param index The index of the whitespace.
     * @return `id` of whitespace at `index`.
     */
    WhitespaceComputer.prototype.getIdForWhitespaceIndex = function (index) {
        index = index | 0;
        return this._ids[index];
    };
    /**
     * Get the `height` for whitespace at index `index`.
     *
     * @param index The index of the whitespace.
     * @return `height` of whitespace at `index`.
     */
    WhitespaceComputer.prototype.getHeightForWhitespaceIndex = function (index) {
        index = index | 0;
        return this._heights[index];
    };
    /**
     * Get all whitespaces.
     */
    WhitespaceComputer.prototype.getWhitespaces = function (deviceLineHeight) {
        deviceLineHeight = deviceLineHeight | 0;
        var result = [];
        for (var i = 0; i < this._heights.length; i++) {
            result.push({
                id: this._ids[i],
                afterLineNumber: this._afterLineNumbers[i],
                heightInLines: this._heights[i] / deviceLineHeight
            });
        }
        return result;
    };
    return WhitespaceComputer;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Layouting of objects that take vertical space (by having a height) and push down other objects.
 *
 * These objects are basically either text (lines) or spaces between those lines (whitespaces).
 * This provides commodity operations for working with lines that contain whitespace that pushes lines lower (vertically).
 * This is written with no knowledge of an editor in mind.
 */
var LinesLayout = /** @class */ (function () {
    function LinesLayout(lineCount, lineHeight) {
        this._lineCount = lineCount;
        this._lineHeight = lineHeight;
        this._whitespaces = new WhitespaceComputer();
    }
    /**
     * Change the height of a line in pixels.
     */
    LinesLayout.prototype.setLineHeight = function (lineHeight) {
        this._lineHeight = lineHeight;
    };
    /**
     * Set the number of lines.
     *
     * @param lineCount New number of lines.
     */
    LinesLayout.prototype.onFlushed = function (lineCount) {
        this._lineCount = lineCount;
    };
    /**
     * Insert a new whitespace of a certain height after a line number.
     * The whitespace has a "sticky" characteristic.
     * Irrespective of edits above or below `afterLineNumber`, the whitespace will follow the initial line.
     *
     * @param afterLineNumber The conceptual position of this whitespace. The whitespace will follow this line as best as possible even when deleting/inserting lines above/below.
     * @param heightInPx The height of the whitespace, in pixels.
     * @return An id that can be used later to mutate or delete the whitespace
     */
    LinesLayout.prototype.insertWhitespace = function (afterLineNumber, ordinal, heightInPx, minWidth) {
        return this._whitespaces.insertWhitespace(afterLineNumber, ordinal, heightInPx, minWidth);
    };
    /**
     * Change properties associated with a certain whitespace.
     */
    LinesLayout.prototype.changeWhitespace = function (id, newAfterLineNumber, newHeight) {
        return this._whitespaces.changeWhitespace(id, newAfterLineNumber, newHeight);
    };
    /**
     * Remove an existing whitespace.
     *
     * @param id The whitespace to remove
     * @return Returns true if the whitespace is found and it is removed.
     */
    LinesLayout.prototype.removeWhitespace = function (id) {
        return this._whitespaces.removeWhitespace(id);
    };
    /**
     * Notify the layouter that lines have been deleted (a continuous zone of lines).
     *
     * @param fromLineNumber The line number at which the deletion started, inclusive
     * @param toLineNumber The line number at which the deletion ended, inclusive
     */
    LinesLayout.prototype.onLinesDeleted = function (fromLineNumber, toLineNumber) {
        this._lineCount -= (toLineNumber - fromLineNumber + 1);
        this._whitespaces.onLinesDeleted(fromLineNumber, toLineNumber);
    };
    /**
     * Notify the layouter that lines have been inserted (a continuous zone of lines).
     *
     * @param fromLineNumber The line number at which the insertion started, inclusive
     * @param toLineNumber The line number at which the insertion ended, inclusive.
     */
    LinesLayout.prototype.onLinesInserted = function (fromLineNumber, toLineNumber) {
        this._lineCount += (toLineNumber - fromLineNumber + 1);
        this._whitespaces.onLinesInserted(fromLineNumber, toLineNumber);
    };
    /**
     * Get the sum of heights for all objects.
     *
     * @return The sum of heights for all objects.
     */
    LinesLayout.prototype.getLinesTotalHeight = function () {
        var linesHeight = this._lineHeight * this._lineCount;
        var whitespacesHeight = this._whitespaces.getTotalHeight();
        return linesHeight + whitespacesHeight;
    };
    /**
     * Get the vertical offset (the sum of heights for all objects above) a certain line number.
     *
     * @param lineNumber The line number
     * @return The sum of heights for all objects above `lineNumber`.
     */
    LinesLayout.prototype.getVerticalOffsetForLineNumber = function (lineNumber) {
        lineNumber = lineNumber | 0;
        var previousLinesHeight;
        if (lineNumber > 1) {
            previousLinesHeight = this._lineHeight * (lineNumber - 1);
        }
        else {
            previousLinesHeight = 0;
        }
        var previousWhitespacesHeight = this._whitespaces.getAccumulatedHeightBeforeLineNumber(lineNumber);
        return previousLinesHeight + previousWhitespacesHeight;
    };
    /**
     * Returns the accumulated height of whitespaces before the given line number.
     *
     * @param lineNumber The line number
     */
    LinesLayout.prototype.getWhitespaceAccumulatedHeightBeforeLineNumber = function (lineNumber) {
        return this._whitespaces.getAccumulatedHeightBeforeLineNumber(lineNumber);
    };
    LinesLayout.prototype.getWhitespaceMinWidth = function () {
        return this._whitespaces.getMinWidth();
    };
    /**
     * Check if `verticalOffset` is below all lines.
     */
    LinesLayout.prototype.isAfterLines = function (verticalOffset) {
        var totalHeight = this.getLinesTotalHeight();
        return verticalOffset > totalHeight;
    };
    /**
     * Find the first line number that is at or after vertical offset `verticalOffset`.
     * i.e. if getVerticalOffsetForLine(line) is x and getVerticalOffsetForLine(line + 1) is y, then
     * getLineNumberAtOrAfterVerticalOffset(i) = line, x <= i < y.
     *
     * @param verticalOffset The vertical offset to search at.
     * @return The line number at or after vertical offset `verticalOffset`.
     */
    LinesLayout.prototype.getLineNumberAtOrAfterVerticalOffset = function (verticalOffset) {
        verticalOffset = verticalOffset | 0;
        if (verticalOffset < 0) {
            return 1;
        }
        var linesCount = this._lineCount | 0;
        var lineHeight = this._lineHeight;
        var minLineNumber = 1;
        var maxLineNumber = linesCount;
        while (minLineNumber < maxLineNumber) {
            var midLineNumber = ((minLineNumber + maxLineNumber) / 2) | 0;
            var midLineNumberVerticalOffset = this.getVerticalOffsetForLineNumber(midLineNumber) | 0;
            if (verticalOffset >= midLineNumberVerticalOffset + lineHeight) {
                // vertical offset is after mid line number
                minLineNumber = midLineNumber + 1;
            }
            else if (verticalOffset >= midLineNumberVerticalOffset) {
                // Hit
                return midLineNumber;
            }
            else {
                // vertical offset is before mid line number, but mid line number could still be what we're searching for
                maxLineNumber = midLineNumber;
            }
        }
        if (minLineNumber > linesCount) {
            return linesCount;
        }
        return minLineNumber;
    };
    /**
     * Get all the lines and their relative vertical offsets that are positioned between `verticalOffset1` and `verticalOffset2`.
     *
     * @param verticalOffset1 The beginning of the viewport.
     * @param verticalOffset2 The end of the viewport.
     * @return A structure describing the lines positioned between `verticalOffset1` and `verticalOffset2`.
     */
    LinesLayout.prototype.getLinesViewportData = function (verticalOffset1, verticalOffset2) {
        verticalOffset1 = verticalOffset1 | 0;
        verticalOffset2 = verticalOffset2 | 0;
        var lineHeight = this._lineHeight;
        // Find first line number
        // We don't live in a perfect world, so the line number might start before or after verticalOffset1
        var startLineNumber = this.getLineNumberAtOrAfterVerticalOffset(verticalOffset1) | 0;
        var startLineNumberVerticalOffset = this.getVerticalOffsetForLineNumber(startLineNumber) | 0;
        var endLineNumber = this._lineCount | 0;
        // Also keep track of what whitespace we've got
        var whitespaceIndex = this._whitespaces.getFirstWhitespaceIndexAfterLineNumber(startLineNumber) | 0;
        var whitespaceCount = this._whitespaces.getCount() | 0;
        var currentWhitespaceHeight;
        var currentWhitespaceAfterLineNumber;
        if (whitespaceIndex === -1) {
            whitespaceIndex = whitespaceCount;
            currentWhitespaceAfterLineNumber = endLineNumber + 1;
            currentWhitespaceHeight = 0;
        }
        else {
            currentWhitespaceAfterLineNumber = this._whitespaces.getAfterLineNumberForWhitespaceIndex(whitespaceIndex) | 0;
            currentWhitespaceHeight = this._whitespaces.getHeightForWhitespaceIndex(whitespaceIndex) | 0;
        }
        var currentVerticalOffset = startLineNumberVerticalOffset;
        var currentLineRelativeOffset = currentVerticalOffset;
        // IE (all versions) cannot handle units above about 1,533,908 px, so every 500k pixels bring numbers down
        var STEP_SIZE = 500000;
        var bigNumbersDelta = 0;
        if (startLineNumberVerticalOffset >= STEP_SIZE) {
            // Compute a delta that guarantees that lines are positioned at `lineHeight` increments
            bigNumbersDelta = Math.floor(startLineNumberVerticalOffset / STEP_SIZE) * STEP_SIZE;
            bigNumbersDelta = Math.floor(bigNumbersDelta / lineHeight) * lineHeight;
            currentLineRelativeOffset -= bigNumbersDelta;
        }
        var linesOffsets = [];
        var verticalCenter = verticalOffset1 + (verticalOffset2 - verticalOffset1) / 2;
        var centeredLineNumber = -1;
        // Figure out how far the lines go
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            if (centeredLineNumber === -1) {
                var currentLineTop = currentVerticalOffset;
                var currentLineBottom = currentVerticalOffset + lineHeight;
                if ((currentLineTop <= verticalCenter && verticalCenter < currentLineBottom) || currentLineTop > verticalCenter) {
                    centeredLineNumber = lineNumber;
                }
            }
            // Count current line height in the vertical offsets
            currentVerticalOffset += lineHeight;
            linesOffsets[lineNumber - startLineNumber] = currentLineRelativeOffset;
            // Next line starts immediately after this one
            currentLineRelativeOffset += lineHeight;
            while (currentWhitespaceAfterLineNumber === lineNumber) {
                // Push down next line with the height of the current whitespace
                currentLineRelativeOffset += currentWhitespaceHeight;
                // Count current whitespace in the vertical offsets
                currentVerticalOffset += currentWhitespaceHeight;
                whitespaceIndex++;
                if (whitespaceIndex >= whitespaceCount) {
                    currentWhitespaceAfterLineNumber = endLineNumber + 1;
                }
                else {
                    currentWhitespaceAfterLineNumber = this._whitespaces.getAfterLineNumberForWhitespaceIndex(whitespaceIndex) | 0;
                    currentWhitespaceHeight = this._whitespaces.getHeightForWhitespaceIndex(whitespaceIndex) | 0;
                }
            }
            if (currentVerticalOffset >= verticalOffset2) {
                // We have covered the entire viewport area, time to stop
                endLineNumber = lineNumber;
                break;
            }
        }
        if (centeredLineNumber === -1) {
            centeredLineNumber = endLineNumber;
        }
        var endLineNumberVerticalOffset = this.getVerticalOffsetForLineNumber(endLineNumber) | 0;
        var completelyVisibleStartLineNumber = startLineNumber;
        var completelyVisibleEndLineNumber = endLineNumber;
        if (completelyVisibleStartLineNumber < completelyVisibleEndLineNumber) {
            if (startLineNumberVerticalOffset < verticalOffset1) {
                completelyVisibleStartLineNumber++;
            }
        }
        if (completelyVisibleStartLineNumber < completelyVisibleEndLineNumber) {
            if (endLineNumberVerticalOffset + lineHeight > verticalOffset2) {
                completelyVisibleEndLineNumber--;
            }
        }
        return {
            bigNumbersDelta: bigNumbersDelta,
            startLineNumber: startLineNumber,
            endLineNumber: endLineNumber,
            relativeVerticalOffset: linesOffsets,
            centeredLineNumber: centeredLineNumber,
            completelyVisibleStartLineNumber: completelyVisibleStartLineNumber,
            completelyVisibleEndLineNumber: completelyVisibleEndLineNumber
        };
    };
    LinesLayout.prototype.getVerticalOffsetForWhitespaceIndex = function (whitespaceIndex) {
        whitespaceIndex = whitespaceIndex | 0;
        var afterLineNumber = this._whitespaces.getAfterLineNumberForWhitespaceIndex(whitespaceIndex);
        var previousLinesHeight;
        if (afterLineNumber >= 1) {
            previousLinesHeight = this._lineHeight * afterLineNumber;
        }
        else {
            previousLinesHeight = 0;
        }
        var previousWhitespacesHeight;
        if (whitespaceIndex > 0) {
            previousWhitespacesHeight = this._whitespaces.getAccumulatedHeight(whitespaceIndex - 1);
        }
        else {
            previousWhitespacesHeight = 0;
        }
        return previousLinesHeight + previousWhitespacesHeight;
    };
    LinesLayout.prototype.getWhitespaceIndexAtOrAfterVerticallOffset = function (verticalOffset) {
        verticalOffset = verticalOffset | 0;
        var midWhitespaceIndex, minWhitespaceIndex = 0, maxWhitespaceIndex = this._whitespaces.getCount() - 1, midWhitespaceVerticalOffset, midWhitespaceHeight;
        if (maxWhitespaceIndex < 0) {
            return -1;
        }
        // Special case: nothing to be found
        var maxWhitespaceVerticalOffset = this.getVerticalOffsetForWhitespaceIndex(maxWhitespaceIndex);
        var maxWhitespaceHeight = this._whitespaces.getHeightForWhitespaceIndex(maxWhitespaceIndex);
        if (verticalOffset >= maxWhitespaceVerticalOffset + maxWhitespaceHeight) {
            return -1;
        }
        while (minWhitespaceIndex < maxWhitespaceIndex) {
            midWhitespaceIndex = Math.floor((minWhitespaceIndex + maxWhitespaceIndex) / 2);
            midWhitespaceVerticalOffset = this.getVerticalOffsetForWhitespaceIndex(midWhitespaceIndex);
            midWhitespaceHeight = this._whitespaces.getHeightForWhitespaceIndex(midWhitespaceIndex);
            if (verticalOffset >= midWhitespaceVerticalOffset + midWhitespaceHeight) {
                // vertical offset is after whitespace
                minWhitespaceIndex = midWhitespaceIndex + 1;
            }
            else if (verticalOffset >= midWhitespaceVerticalOffset) {
                // Hit
                return midWhitespaceIndex;
            }
            else {
                // vertical offset is before whitespace, but midWhitespaceIndex might still be what we're searching for
                maxWhitespaceIndex = midWhitespaceIndex;
            }
        }
        return minWhitespaceIndex;
    };
    /**
     * Get exactly the whitespace that is layouted at `verticalOffset`.
     *
     * @param verticalOffset The vertical offset.
     * @return Precisely the whitespace that is layouted at `verticaloffset` or null.
     */
    LinesLayout.prototype.getWhitespaceAtVerticalOffset = function (verticalOffset) {
        verticalOffset = verticalOffset | 0;
        var candidateIndex = this.getWhitespaceIndexAtOrAfterVerticallOffset(verticalOffset);
        if (candidateIndex < 0) {
            return null;
        }
        if (candidateIndex >= this._whitespaces.getCount()) {
            return null;
        }
        var candidateTop = this.getVerticalOffsetForWhitespaceIndex(candidateIndex);
        if (candidateTop > verticalOffset) {
            return null;
        }
        var candidateHeight = this._whitespaces.getHeightForWhitespaceIndex(candidateIndex);
        var candidateId = this._whitespaces.getIdForWhitespaceIndex(candidateIndex);
        var candidateAfterLineNumber = this._whitespaces.getAfterLineNumberForWhitespaceIndex(candidateIndex);
        return {
            id: candidateId,
            afterLineNumber: candidateAfterLineNumber,
            verticalOffset: candidateTop,
            height: candidateHeight
        };
    };
    /**
     * Get a list of whitespaces that are positioned between `verticalOffset1` and `verticalOffset2`.
     *
     * @param verticalOffset1 The beginning of the viewport.
     * @param verticalOffset2 The end of the viewport.
     * @return An array with all the whitespaces in the viewport. If no whitespace is in viewport, the array is empty.
     */
    LinesLayout.prototype.getWhitespaceViewportData = function (verticalOffset1, verticalOffset2) {
        verticalOffset1 = verticalOffset1 | 0;
        verticalOffset2 = verticalOffset2 | 0;
        var startIndex = this.getWhitespaceIndexAtOrAfterVerticallOffset(verticalOffset1);
        var endIndex = this._whitespaces.getCount() - 1;
        if (startIndex < 0) {
            return [];
        }
        var result = [];
        for (var i = startIndex; i <= endIndex; i++) {
            var top_1 = this.getVerticalOffsetForWhitespaceIndex(i);
            var height = this._whitespaces.getHeightForWhitespaceIndex(i);
            if (top_1 >= verticalOffset2) {
                break;
            }
            result.push({
                id: this._whitespaces.getIdForWhitespaceIndex(i),
                afterLineNumber: this._whitespaces.getAfterLineNumberForWhitespaceIndex(i),
                verticalOffset: top_1,
                height: height
            });
        }
        return result;
    };
    /**
     * Get all whitespaces.
     */
    LinesLayout.prototype.getWhitespaces = function () {
        return this._whitespaces.getWhitespaces(this._lineHeight);
    };
    return LinesLayout;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Viewport = /** @class */ (function () {
    function Viewport(top, left, width, height) {
        this.top = top | 0;
        this.left = left | 0;
        this.width = width | 0;
        this.height = height | 0;
    }
    return Viewport;
}());
var MinimapLinesRenderingData = /** @class */ (function () {
    function MinimapLinesRenderingData(tabSize, data) {
        this.tabSize = tabSize;
        this.data = data;
    }
    return MinimapLinesRenderingData;
}());
var ViewLineData = /** @class */ (function () {
    function ViewLineData(content, continuesWithWrappedLine, minColumn, maxColumn, tokens) {
        this.content = content;
        this.continuesWithWrappedLine = continuesWithWrappedLine;
        this.minColumn = minColumn;
        this.maxColumn = maxColumn;
        this.tokens = tokens;
    }
    return ViewLineData;
}());
var ViewLineRenderingData = /** @class */ (function () {
    function ViewLineRenderingData(minColumn, maxColumn, content, continuesWithWrappedLine, mightContainRTL, mightContainNonBasicASCII, tokens, inlineDecorations, tabSize) {
        this.minColumn = minColumn;
        this.maxColumn = maxColumn;
        this.content = content;
        this.continuesWithWrappedLine = continuesWithWrappedLine;
        this.isBasicASCII = ViewLineRenderingData.isBasicASCII(content, mightContainNonBasicASCII);
        this.containsRTL = ViewLineRenderingData.containsRTL(content, this.isBasicASCII, mightContainRTL);
        this.tokens = tokens;
        this.inlineDecorations = inlineDecorations;
        this.tabSize = tabSize;
    }
    ViewLineRenderingData.isBasicASCII = function (lineContent, mightContainNonBasicASCII) {
        if (mightContainNonBasicASCII) {
            return isBasicASCII(lineContent);
        }
        return true;
    };
    ViewLineRenderingData.containsRTL = function (lineContent, isBasicASCII$$1, mightContainRTL) {
        if (!isBasicASCII$$1 && mightContainRTL) {
            return containsRTL(lineContent);
        }
        return false;
    };
    return ViewLineRenderingData;
}());
var InlineDecoration = /** @class */ (function () {
    function InlineDecoration(range, inlineClassName, type) {
        this.range = range;
        this.inlineClassName = inlineClassName;
        this.type = type;
    }
    return InlineDecoration;
}());
var ViewModelDecoration = /** @class */ (function () {
    function ViewModelDecoration(range, options) {
        this.range = range;
        this.options = options;
    }
    return ViewModelDecoration;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$13 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var SMOOTH_SCROLLING_TIME = 125;
var ViewLayout = /** @class */ (function (_super) {
    __extends$13(ViewLayout, _super);
    function ViewLayout(configuration, lineCount, scheduleAtNextAnimationFrame) {
        var _this = _super.call(this) || this;
        _this._configuration = configuration;
        _this._linesLayout = new LinesLayout(lineCount, _this._configuration.editor.lineHeight);
        _this.scrollable = _this._register(new Scrollable(0, scheduleAtNextAnimationFrame));
        _this._configureSmoothScrollDuration();
        _this.scrollable.setScrollDimensions({
            width: configuration.editor.layoutInfo.contentWidth,
            height: configuration.editor.layoutInfo.contentHeight
        });
        _this.onDidScroll = _this.scrollable.onScroll;
        _this._updateHeight();
        return _this;
    }
    ViewLayout.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    ViewLayout.prototype.onHeightMaybeChanged = function () {
        this._updateHeight();
    };
    ViewLayout.prototype._configureSmoothScrollDuration = function () {
        this.scrollable.setSmoothScrollDuration(this._configuration.editor.viewInfo.smoothScrolling ? SMOOTH_SCROLLING_TIME : 0);
    };
    // ---- begin view event handlers
    ViewLayout.prototype.onConfigurationChanged = function (e) {
        if (e.lineHeight) {
            this._linesLayout.setLineHeight(this._configuration.editor.lineHeight);
        }
        if (e.layoutInfo) {
            this.scrollable.setScrollDimensions({
                width: this._configuration.editor.layoutInfo.contentWidth,
                height: this._configuration.editor.layoutInfo.contentHeight
            });
        }
        if (e.viewInfo) {
            this._configureSmoothScrollDuration();
        }
        this._updateHeight();
    };
    ViewLayout.prototype.onFlushed = function (lineCount) {
        this._linesLayout.onFlushed(lineCount);
    };
    ViewLayout.prototype.onLinesDeleted = function (fromLineNumber, toLineNumber) {
        this._linesLayout.onLinesDeleted(fromLineNumber, toLineNumber);
    };
    ViewLayout.prototype.onLinesInserted = function (fromLineNumber, toLineNumber) {
        this._linesLayout.onLinesInserted(fromLineNumber, toLineNumber);
    };
    // ---- end view event handlers
    ViewLayout.prototype._getHorizontalScrollbarHeight = function (scrollDimensions) {
        if (this._configuration.editor.viewInfo.scrollbar.horizontal === 2 /* Hidden */) {
            // horizontal scrollbar not visible
            return 0;
        }
        if (scrollDimensions.width >= scrollDimensions.scrollWidth) {
            // horizontal scrollbar not visible
            return 0;
        }
        return this._configuration.editor.viewInfo.scrollbar.horizontalScrollbarSize;
    };
    ViewLayout.prototype._getTotalHeight = function () {
        var scrollDimensions = this.scrollable.getScrollDimensions();
        var result = this._linesLayout.getLinesTotalHeight();
        if (this._configuration.editor.viewInfo.scrollBeyondLastLine) {
            result += scrollDimensions.height - this._configuration.editor.lineHeight;
        }
        else {
            result += this._getHorizontalScrollbarHeight(scrollDimensions);
        }
        return Math.max(scrollDimensions.height, result);
    };
    ViewLayout.prototype._updateHeight = function () {
        this.scrollable.setScrollDimensions({
            scrollHeight: this._getTotalHeight()
        });
    };
    // ---- Layouting logic
    ViewLayout.prototype.getCurrentViewport = function () {
        var scrollDimensions = this.scrollable.getScrollDimensions();
        var currentScrollPosition = this.scrollable.getCurrentScrollPosition();
        return new Viewport(currentScrollPosition.scrollTop, currentScrollPosition.scrollLeft, scrollDimensions.width, scrollDimensions.height);
    };
    ViewLayout.prototype.getFutureViewport = function () {
        var scrollDimensions = this.scrollable.getScrollDimensions();
        var currentScrollPosition = this.scrollable.getFutureScrollPosition();
        return new Viewport(currentScrollPosition.scrollTop, currentScrollPosition.scrollLeft, scrollDimensions.width, scrollDimensions.height);
    };
    ViewLayout.prototype._computeScrollWidth = function (maxLineWidth, viewportWidth) {
        var isViewportWrapping = this._configuration.editor.wrappingInfo.isViewportWrapping;
        if (!isViewportWrapping) {
            var extraHorizontalSpace = this._configuration.editor.viewInfo.scrollBeyondLastColumn * this._configuration.editor.fontInfo.typicalHalfwidthCharacterWidth;
            var whitespaceMinWidth = this._linesLayout.getWhitespaceMinWidth();
            return Math.max(maxLineWidth + extraHorizontalSpace, viewportWidth, whitespaceMinWidth);
        }
        return Math.max(maxLineWidth, viewportWidth);
    };
    ViewLayout.prototype.onMaxLineWidthChanged = function (maxLineWidth) {
        var newScrollWidth = this._computeScrollWidth(maxLineWidth, this.getCurrentViewport().width);
        this.scrollable.setScrollDimensions({
            scrollWidth: newScrollWidth
        });
        // The height might depend on the fact that there is a horizontal scrollbar or not
        this._updateHeight();
    };
    // ---- view state
    ViewLayout.prototype.saveState = function () {
        var currentScrollPosition = this.scrollable.getFutureScrollPosition();
        var scrollTop = currentScrollPosition.scrollTop;
        var firstLineNumberInViewport = this._linesLayout.getLineNumberAtOrAfterVerticalOffset(scrollTop);
        var whitespaceAboveFirstLine = this._linesLayout.getWhitespaceAccumulatedHeightBeforeLineNumber(firstLineNumberInViewport);
        return {
            scrollTop: scrollTop,
            scrollTopWithoutViewZones: scrollTop - whitespaceAboveFirstLine,
            scrollLeft: currentScrollPosition.scrollLeft
        };
    };
    // ---- IVerticalLayoutProvider
    ViewLayout.prototype.addWhitespace = function (afterLineNumber, ordinal, height, minWidth) {
        return this._linesLayout.insertWhitespace(afterLineNumber, ordinal, height, minWidth);
    };
    ViewLayout.prototype.changeWhitespace = function (id, newAfterLineNumber, newHeight) {
        return this._linesLayout.changeWhitespace(id, newAfterLineNumber, newHeight);
    };
    ViewLayout.prototype.removeWhitespace = function (id) {
        return this._linesLayout.removeWhitespace(id);
    };
    ViewLayout.prototype.getVerticalOffsetForLineNumber = function (lineNumber) {
        return this._linesLayout.getVerticalOffsetForLineNumber(lineNumber);
    };
    ViewLayout.prototype.isAfterLines = function (verticalOffset) {
        return this._linesLayout.isAfterLines(verticalOffset);
    };
    ViewLayout.prototype.getLineNumberAtVerticalOffset = function (verticalOffset) {
        return this._linesLayout.getLineNumberAtOrAfterVerticalOffset(verticalOffset);
    };
    ViewLayout.prototype.getWhitespaceAtVerticalOffset = function (verticalOffset) {
        return this._linesLayout.getWhitespaceAtVerticalOffset(verticalOffset);
    };
    ViewLayout.prototype.getLinesViewportData = function () {
        var visibleBox = this.getCurrentViewport();
        return this._linesLayout.getLinesViewportData(visibleBox.top, visibleBox.top + visibleBox.height);
    };
    ViewLayout.prototype.getLinesViewportDataAtScrollTop = function (scrollTop) {
        // do some minimal validations on scrollTop
        var scrollDimensions = this.scrollable.getScrollDimensions();
        if (scrollTop + scrollDimensions.height > scrollDimensions.scrollHeight) {
            scrollTop = scrollDimensions.scrollHeight - scrollDimensions.height;
        }
        if (scrollTop < 0) {
            scrollTop = 0;
        }
        return this._linesLayout.getLinesViewportData(scrollTop, scrollTop + scrollDimensions.height);
    };
    ViewLayout.prototype.getWhitespaceViewportData = function () {
        var visibleBox = this.getCurrentViewport();
        return this._linesLayout.getWhitespaceViewportData(visibleBox.top, visibleBox.top + visibleBox.height);
    };
    ViewLayout.prototype.getWhitespaces = function () {
        return this._linesLayout.getWhitespaces();
    };
    // ---- IScrollingProvider
    ViewLayout.prototype.getScrollWidth = function () {
        var scrollDimensions = this.scrollable.getScrollDimensions();
        return scrollDimensions.scrollWidth;
    };
    ViewLayout.prototype.getScrollHeight = function () {
        var scrollDimensions = this.scrollable.getScrollDimensions();
        return scrollDimensions.scrollHeight;
    };
    ViewLayout.prototype.getCurrentScrollLeft = function () {
        var currentScrollPosition = this.scrollable.getCurrentScrollPosition();
        return currentScrollPosition.scrollLeft;
    };
    ViewLayout.prototype.getCurrentScrollTop = function () {
        var currentScrollPosition = this.scrollable.getCurrentScrollPosition();
        return currentScrollPosition.scrollTop;
    };
    ViewLayout.prototype.validateScrollPosition = function (scrollPosition) {
        return this.scrollable.validateScrollPosition(scrollPosition);
    };
    ViewLayout.prototype.setScrollPositionNow = function (position) {
        this.scrollable.setScrollPositionNow(position);
    };
    ViewLayout.prototype.setScrollPositionSmooth = function (position) {
        this.scrollable.setScrollPositionSmooth(position);
    };
    ViewLayout.prototype.deltaScrollNow = function (deltaScrollLeft, deltaScrollTop) {
        var currentScrollPosition = this.scrollable.getCurrentScrollPosition();
        this.scrollable.setScrollPositionNow({
            scrollLeft: currentScrollPosition.scrollLeft + deltaScrollLeft,
            scrollTop: currentScrollPosition.scrollTop + deltaScrollTop
        });
    };
    return ViewLayout;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var PrefixSumIndexOfResult = /** @class */ (function () {
    function PrefixSumIndexOfResult(index, remainder) {
        this.index = index;
        this.remainder = remainder;
    }
    return PrefixSumIndexOfResult;
}());
var PrefixSumComputer = /** @class */ (function () {
    function PrefixSumComputer(values) {
        this.values = values;
        this.prefixSum = new Uint32Array(values.length);
        this.prefixSumValidIndex = new Int32Array(1);
        this.prefixSumValidIndex[0] = -1;
    }
    PrefixSumComputer.prototype.getCount = function () {
        return this.values.length;
    };
    PrefixSumComputer.prototype.insertValues = function (insertIndex, insertValues) {
        insertIndex = toUint32(insertIndex);
        var oldValues = this.values;
        var oldPrefixSum = this.prefixSum;
        var insertValuesLen = insertValues.length;
        if (insertValuesLen === 0) {
            return false;
        }
        this.values = new Uint32Array(oldValues.length + insertValuesLen);
        this.values.set(oldValues.subarray(0, insertIndex), 0);
        this.values.set(oldValues.subarray(insertIndex), insertIndex + insertValuesLen);
        this.values.set(insertValues, insertIndex);
        if (insertIndex - 1 < this.prefixSumValidIndex[0]) {
            this.prefixSumValidIndex[0] = insertIndex - 1;
        }
        this.prefixSum = new Uint32Array(this.values.length);
        if (this.prefixSumValidIndex[0] >= 0) {
            this.prefixSum.set(oldPrefixSum.subarray(0, this.prefixSumValidIndex[0] + 1));
        }
        return true;
    };
    PrefixSumComputer.prototype.changeValue = function (index, value) {
        index = toUint32(index);
        value = toUint32(value);
        if (this.values[index] === value) {
            return false;
        }
        this.values[index] = value;
        if (index - 1 < this.prefixSumValidIndex[0]) {
            this.prefixSumValidIndex[0] = index - 1;
        }
        return true;
    };
    PrefixSumComputer.prototype.removeValues = function (startIndex, cnt) {
        startIndex = toUint32(startIndex);
        cnt = toUint32(cnt);
        var oldValues = this.values;
        var oldPrefixSum = this.prefixSum;
        if (startIndex >= oldValues.length) {
            return false;
        }
        var maxCnt = oldValues.length - startIndex;
        if (cnt >= maxCnt) {
            cnt = maxCnt;
        }
        if (cnt === 0) {
            return false;
        }
        this.values = new Uint32Array(oldValues.length - cnt);
        this.values.set(oldValues.subarray(0, startIndex), 0);
        this.values.set(oldValues.subarray(startIndex + cnt), startIndex);
        this.prefixSum = new Uint32Array(this.values.length);
        if (startIndex - 1 < this.prefixSumValidIndex[0]) {
            this.prefixSumValidIndex[0] = startIndex - 1;
        }
        if (this.prefixSumValidIndex[0] >= 0) {
            this.prefixSum.set(oldPrefixSum.subarray(0, this.prefixSumValidIndex[0] + 1));
        }
        return true;
    };
    PrefixSumComputer.prototype.getTotalValue = function () {
        if (this.values.length === 0) {
            return 0;
        }
        return this._getAccumulatedValue(this.values.length - 1);
    };
    PrefixSumComputer.prototype.getAccumulatedValue = function (index) {
        if (index < 0) {
            return 0;
        }
        index = toUint32(index);
        return this._getAccumulatedValue(index);
    };
    PrefixSumComputer.prototype._getAccumulatedValue = function (index) {
        if (index <= this.prefixSumValidIndex[0]) {
            return this.prefixSum[index];
        }
        var startIndex = this.prefixSumValidIndex[0] + 1;
        if (startIndex === 0) {
            this.prefixSum[0] = this.values[0];
            startIndex++;
        }
        if (index >= this.values.length) {
            index = this.values.length - 1;
        }
        for (var i = startIndex; i <= index; i++) {
            this.prefixSum[i] = this.prefixSum[i - 1] + this.values[i];
        }
        this.prefixSumValidIndex[0] = Math.max(this.prefixSumValidIndex[0], index);
        return this.prefixSum[index];
    };
    PrefixSumComputer.prototype.getIndexOf = function (accumulatedValue) {
        accumulatedValue = Math.floor(accumulatedValue); //@perf
        // Compute all sums (to get a fully valid prefixSum)
        this.getTotalValue();
        var low = 0;
        var high = this.values.length - 1;
        var mid = 0;
        var midStop = 0;
        var midStart = 0;
        while (low <= high) {
            mid = low + ((high - low) / 2) | 0;
            midStop = this.prefixSum[mid];
            midStart = midStop - this.values[mid];
            if (accumulatedValue < midStart) {
                high = mid - 1;
            }
            else if (accumulatedValue >= midStop) {
                low = mid + 1;
            }
            else {
                break;
            }
        }
        return new PrefixSumIndexOfResult(mid, accumulatedValue - midStart);
    };
    return PrefixSumComputer;
}());
var PrefixSumComputerWithCache = /** @class */ (function () {
    function PrefixSumComputerWithCache(values) {
        this._cacheAccumulatedValueStart = 0;
        this._cache = null;
        this._actual = new PrefixSumComputer(values);
        this._bustCache();
    }
    PrefixSumComputerWithCache.prototype._bustCache = function () {
        this._cacheAccumulatedValueStart = 0;
        this._cache = null;
    };
    PrefixSumComputerWithCache.prototype.insertValues = function (insertIndex, insertValues) {
        if (this._actual.insertValues(insertIndex, insertValues)) {
            this._bustCache();
        }
    };
    PrefixSumComputerWithCache.prototype.changeValue = function (index, value) {
        if (this._actual.changeValue(index, value)) {
            this._bustCache();
        }
    };
    PrefixSumComputerWithCache.prototype.removeValues = function (startIndex, cnt) {
        if (this._actual.removeValues(startIndex, cnt)) {
            this._bustCache();
        }
    };
    PrefixSumComputerWithCache.prototype.getTotalValue = function () {
        return this._actual.getTotalValue();
    };
    PrefixSumComputerWithCache.prototype.getAccumulatedValue = function (index) {
        return this._actual.getAccumulatedValue(index);
    };
    PrefixSumComputerWithCache.prototype.getIndexOf = function (accumulatedValue) {
        accumulatedValue = Math.floor(accumulatedValue); //@perf
        if (this._cache !== null) {
            var cacheIndex = accumulatedValue - this._cacheAccumulatedValueStart;
            if (cacheIndex >= 0 && cacheIndex < this._cache.length) {
                // Cache hit!
                return this._cache[cacheIndex];
            }
        }
        // Cache miss!
        return this._actual.getIndexOf(accumulatedValue);
    };
    /**
     * Gives a hint that a lot of requests are about to come in for these accumulated values.
     */
    PrefixSumComputerWithCache.prototype.warmUpCache = function (accumulatedValueStart, accumulatedValueEnd) {
        var newCache = [];
        for (var accumulatedValue = accumulatedValueStart; accumulatedValue <= accumulatedValueEnd; accumulatedValue++) {
            newCache[accumulatedValue - accumulatedValueStart] = this.getIndexOf(accumulatedValue);
        }
        this._cache = newCache;
        this._cacheAccumulatedValueStart = accumulatedValueStart;
    };
    return PrefixSumComputerWithCache;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var OutputPosition = /** @class */ (function () {
    function OutputPosition(outputLineIndex, outputOffset) {
        this.outputLineIndex = outputLineIndex;
        this.outputOffset = outputOffset;
    }
    return OutputPosition;
}());
var CoordinatesConverter = /** @class */ (function () {
    function CoordinatesConverter(lines) {
        this._lines = lines;
    }
    // View -> Model conversion and related methods
    CoordinatesConverter.prototype.convertViewPositionToModelPosition = function (viewPosition) {
        return this._lines.convertViewPositionToModelPosition(viewPosition.lineNumber, viewPosition.column);
    };
    CoordinatesConverter.prototype.convertViewRangeToModelRange = function (viewRange) {
        var start = this._lines.convertViewPositionToModelPosition(viewRange.startLineNumber, viewRange.startColumn);
        var end = this._lines.convertViewPositionToModelPosition(viewRange.endLineNumber, viewRange.endColumn);
        return new Range(start.lineNumber, start.column, end.lineNumber, end.column);
    };
    CoordinatesConverter.prototype.validateViewPosition = function (viewPosition, expectedModelPosition) {
        return this._lines.validateViewPosition(viewPosition.lineNumber, viewPosition.column, expectedModelPosition);
    };
    CoordinatesConverter.prototype.validateViewRange = function (viewRange, expectedModelRange) {
        var validViewStart = this._lines.validateViewPosition(viewRange.startLineNumber, viewRange.startColumn, expectedModelRange.getStartPosition());
        var validViewEnd = this._lines.validateViewPosition(viewRange.endLineNumber, viewRange.endColumn, expectedModelRange.getEndPosition());
        return new Range(validViewStart.lineNumber, validViewStart.column, validViewEnd.lineNumber, validViewEnd.column);
    };
    // Model -> View conversion and related methods
    CoordinatesConverter.prototype.convertModelPositionToViewPosition = function (modelPosition) {
        return this._lines.convertModelPositionToViewPosition(modelPosition.lineNumber, modelPosition.column);
    };
    CoordinatesConverter.prototype.convertModelRangeToViewRange = function (modelRange) {
        var start = this._lines.convertModelPositionToViewPosition(modelRange.startLineNumber, modelRange.startColumn);
        var end = this._lines.convertModelPositionToViewPosition(modelRange.endLineNumber, modelRange.endColumn);
        return new Range(start.lineNumber, start.column, end.lineNumber, end.column);
    };
    CoordinatesConverter.prototype.modelPositionIsVisible = function (modelPosition) {
        return this._lines.modelPositionIsVisible(modelPosition.lineNumber, modelPosition.column);
    };
    return CoordinatesConverter;
}());
var SplitLinesCollection = /** @class */ (function () {
    function SplitLinesCollection(model, linePositionMapperFactory, tabSize, wrappingColumn, columnsForFullWidthChar, wrappingIndent) {
        this.model = model;
        this._validModelVersionId = -1;
        this.tabSize = tabSize;
        this.wrappingColumn = wrappingColumn;
        this.columnsForFullWidthChar = columnsForFullWidthChar;
        this.wrappingIndent = wrappingIndent;
        this.linePositionMapperFactory = linePositionMapperFactory;
        this._constructLines(true);
    }
    SplitLinesCollection.prototype.dispose = function () {
        this.hiddenAreasIds = this.model.deltaDecorations(this.hiddenAreasIds, []);
    };
    SplitLinesCollection.prototype.createCoordinatesConverter = function () {
        return new CoordinatesConverter(this);
    };
    SplitLinesCollection.prototype._ensureValidState = function () {
        var modelVersion = this.model.getVersionId();
        if (modelVersion !== this._validModelVersionId) {
            // This is pretty bad, it means we lost track of the model...
            throw new Error("ViewModel is out of sync with Model!");
        }
        if (this.lines.length !== this.model.getLineCount()) {
            // This is pretty bad, it means we lost track of the model...
            this._constructLines(false);
        }
    };
    SplitLinesCollection.prototype._constructLines = function (resetHiddenAreas) {
        var _this = this;
        this.lines = [];
        if (resetHiddenAreas) {
            this.hiddenAreasIds = [];
        }
        var linesContent = this.model.getLinesContent();
        var lineCount = linesContent.length;
        var values = new Uint32Array(lineCount);
        var hiddenAreas = this.hiddenAreasIds.map(function (areaId) { return _this.model.getDecorationRange(areaId); }).sort(Range.compareRangesUsingStarts);
        var hiddenAreaStart = 1, hiddenAreaEnd = 0;
        var hiddenAreaIdx = -1;
        var nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : lineCount + 2;
        for (var i = 0; i < lineCount; i++) {
            var lineNumber = i + 1;
            if (lineNumber === nextLineNumberToUpdateHiddenArea) {
                hiddenAreaIdx++;
                hiddenAreaStart = hiddenAreas[hiddenAreaIdx].startLineNumber;
                hiddenAreaEnd = hiddenAreas[hiddenAreaIdx].endLineNumber;
                nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : lineCount + 2;
            }
            var isInHiddenArea = (lineNumber >= hiddenAreaStart && lineNumber <= hiddenAreaEnd);
            var line = createSplitLine(this.linePositionMapperFactory, linesContent[i], this.tabSize, this.wrappingColumn, this.columnsForFullWidthChar, this.wrappingIndent, !isInHiddenArea);
            values[i] = line.getViewLineCount();
            this.lines[i] = line;
        }
        this._validModelVersionId = this.model.getVersionId();
        this.prefixSumComputer = new PrefixSumComputerWithCache(values);
    };
    SplitLinesCollection.prototype.getHiddenAreas = function () {
        var _this = this;
        return this.hiddenAreasIds.map(function (decId) {
            return _this.model.getDecorationRange(decId);
        });
    };
    SplitLinesCollection.prototype._reduceRanges = function (_ranges) {
        var _this = this;
        if (_ranges.length === 0) {
            return [];
        }
        var ranges = _ranges.map(function (r) { return _this.model.validateRange(r); }).sort(Range.compareRangesUsingStarts);
        var result = [];
        var currentRangeStart = ranges[0].startLineNumber;
        var currentRangeEnd = ranges[0].endLineNumber;
        for (var i = 1, len = ranges.length; i < len; i++) {
            var range = ranges[i];
            if (range.startLineNumber > currentRangeEnd + 1) {
                result.push(new Range(currentRangeStart, 1, currentRangeEnd, 1));
                currentRangeStart = range.startLineNumber;
                currentRangeEnd = range.endLineNumber;
            }
            else if (range.endLineNumber > currentRangeEnd) {
                currentRangeEnd = range.endLineNumber;
            }
        }
        result.push(new Range(currentRangeStart, 1, currentRangeEnd, 1));
        return result;
    };
    SplitLinesCollection.prototype.setHiddenAreas = function (_ranges) {
        var _this = this;
        var newRanges = this._reduceRanges(_ranges);
        // BEGIN TODO@Martin: Please stop calling this method on each model change!
        var oldRanges = this.hiddenAreasIds.map(function (areaId) { return _this.model.getDecorationRange(areaId); }).sort(Range.compareRangesUsingStarts);
        if (newRanges.length === oldRanges.length) {
            var hasDifference = false;
            for (var i = 0; i < newRanges.length; i++) {
                if (!newRanges[i].equalsRange(oldRanges[i])) {
                    hasDifference = true;
                    break;
                }
            }
            if (!hasDifference) {
                return false;
            }
        }
        // END TODO@Martin: Please stop calling this method on each model change!
        var newDecorations = [];
        for (var i = 0; i < newRanges.length; i++) {
            newDecorations.push({
                range: newRanges[i],
                options: ModelDecorationOptions.EMPTY
            });
        }
        this.hiddenAreasIds = this.model.deltaDecorations(this.hiddenAreasIds, newDecorations);
        var hiddenAreas = newRanges;
        var hiddenAreaStart = 1, hiddenAreaEnd = 0;
        var hiddenAreaIdx = -1;
        var nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : this.lines.length + 2;
        var hasVisibleLine = false;
        for (var i = 0; i < this.lines.length; i++) {
            var lineNumber = i + 1;
            if (lineNumber === nextLineNumberToUpdateHiddenArea) {
                hiddenAreaIdx++;
                hiddenAreaStart = hiddenAreas[hiddenAreaIdx].startLineNumber;
                hiddenAreaEnd = hiddenAreas[hiddenAreaIdx].endLineNumber;
                nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : this.lines.length + 2;
            }
            var lineChanged = false;
            if (lineNumber >= hiddenAreaStart && lineNumber <= hiddenAreaEnd) {
                // Line should be hidden
                if (this.lines[i].isVisible()) {
                    this.lines[i] = this.lines[i].setVisible(false);
                    lineChanged = true;
                }
            }
            else {
                hasVisibleLine = true;
                // Line should be visible
                if (!this.lines[i].isVisible()) {
                    this.lines[i] = this.lines[i].setVisible(true);
                    lineChanged = true;
                }
            }
            if (lineChanged) {
                var newOutputLineCount = this.lines[i].getViewLineCount();
                this.prefixSumComputer.changeValue(i, newOutputLineCount);
            }
        }
        if (!hasVisibleLine) {
            // Cannot have everything be hidden => reveal everything!
            this.setHiddenAreas([]);
        }
        return true;
    };
    SplitLinesCollection.prototype.modelPositionIsVisible = function (modelLineNumber, _modelColumn) {
        if (modelLineNumber < 1 || modelLineNumber > this.lines.length) {
            // invalid arguments
            return false;
        }
        return this.lines[modelLineNumber - 1].isVisible();
    };
    SplitLinesCollection.prototype.setTabSize = function (newTabSize) {
        if (this.tabSize === newTabSize) {
            return false;
        }
        this.tabSize = newTabSize;
        this._constructLines(false);
        return true;
    };
    SplitLinesCollection.prototype.setWrappingSettings = function (wrappingIndent, wrappingColumn, columnsForFullWidthChar) {
        if (this.wrappingIndent === wrappingIndent && this.wrappingColumn === wrappingColumn && this.columnsForFullWidthChar === columnsForFullWidthChar) {
            return false;
        }
        this.wrappingIndent = wrappingIndent;
        this.wrappingColumn = wrappingColumn;
        this.columnsForFullWidthChar = columnsForFullWidthChar;
        this._constructLines(false);
        return true;
    };
    SplitLinesCollection.prototype.onModelFlushed = function () {
        this._constructLines(true);
    };
    SplitLinesCollection.prototype.onModelLinesDeleted = function (versionId, fromLineNumber, toLineNumber) {
        if (versionId <= this._validModelVersionId) {
            // Here we check for versionId in case the lines were reconstructed in the meantime.
            // We don't want to apply stale change events on top of a newer read model state.
            return null;
        }
        var outputFromLineNumber = (fromLineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(fromLineNumber - 2) + 1);
        var outputToLineNumber = this.prefixSumComputer.getAccumulatedValue(toLineNumber - 1);
        this.lines.splice(fromLineNumber - 1, toLineNumber - fromLineNumber + 1);
        this.prefixSumComputer.removeValues(fromLineNumber - 1, toLineNumber - fromLineNumber + 1);
        return new ViewLinesDeletedEvent(outputFromLineNumber, outputToLineNumber);
    };
    SplitLinesCollection.prototype.onModelLinesInserted = function (versionId, fromLineNumber, _toLineNumber, text) {
        if (versionId <= this._validModelVersionId) {
            // Here we check for versionId in case the lines were reconstructed in the meantime.
            // We don't want to apply stale change events on top of a newer read model state.
            return null;
        }
        var hiddenAreas = this.getHiddenAreas();
        var isInHiddenArea = false;
        var testPosition = new Position(fromLineNumber, 1);
        for (var i = 0; i < hiddenAreas.length; i++) {
            if (hiddenAreas[i].containsPosition(testPosition)) {
                isInHiddenArea = true;
                break;
            }
        }
        var outputFromLineNumber = (fromLineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(fromLineNumber - 2) + 1);
        var totalOutputLineCount = 0;
        var insertLines = [];
        var insertPrefixSumValues = new Uint32Array(text.length);
        for (var i = 0, len = text.length; i < len; i++) {
            var line = createSplitLine(this.linePositionMapperFactory, text[i], this.tabSize, this.wrappingColumn, this.columnsForFullWidthChar, this.wrappingIndent, !isInHiddenArea);
            insertLines.push(line);
            var outputLineCount = line.getViewLineCount();
            totalOutputLineCount += outputLineCount;
            insertPrefixSumValues[i] = outputLineCount;
        }
        // TODO@Alex: use arrays.arrayInsert
        this.lines = this.lines.slice(0, fromLineNumber - 1).concat(insertLines).concat(this.lines.slice(fromLineNumber - 1));
        this.prefixSumComputer.insertValues(fromLineNumber - 1, insertPrefixSumValues);
        return new ViewLinesInsertedEvent(outputFromLineNumber, outputFromLineNumber + totalOutputLineCount - 1);
    };
    SplitLinesCollection.prototype.onModelLineChanged = function (versionId, lineNumber, newText) {
        if (versionId <= this._validModelVersionId) {
            // Here we check for versionId in case the lines were reconstructed in the meantime.
            // We don't want to apply stale change events on top of a newer read model state.
            return [false, null, null, null];
        }
        var lineIndex = lineNumber - 1;
        var oldOutputLineCount = this.lines[lineIndex].getViewLineCount();
        var isVisible = this.lines[lineIndex].isVisible();
        var line = createSplitLine(this.linePositionMapperFactory, newText, this.tabSize, this.wrappingColumn, this.columnsForFullWidthChar, this.wrappingIndent, isVisible);
        this.lines[lineIndex] = line;
        var newOutputLineCount = this.lines[lineIndex].getViewLineCount();
        var lineMappingChanged = false;
        var changeFrom = 0;
        var changeTo = -1;
        var insertFrom = 0;
        var insertTo = -1;
        var deleteFrom = 0;
        var deleteTo = -1;
        if (oldOutputLineCount > newOutputLineCount) {
            changeFrom = (lineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(lineNumber - 2) + 1);
            changeTo = changeFrom + newOutputLineCount - 1;
            deleteFrom = changeTo + 1;
            deleteTo = deleteFrom + (oldOutputLineCount - newOutputLineCount) - 1;
            lineMappingChanged = true;
        }
        else if (oldOutputLineCount < newOutputLineCount) {
            changeFrom = (lineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(lineNumber - 2) + 1);
            changeTo = changeFrom + oldOutputLineCount - 1;
            insertFrom = changeTo + 1;
            insertTo = insertFrom + (newOutputLineCount - oldOutputLineCount) - 1;
            lineMappingChanged = true;
        }
        else {
            changeFrom = (lineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(lineNumber - 2) + 1);
            changeTo = changeFrom + newOutputLineCount - 1;
        }
        this.prefixSumComputer.changeValue(lineIndex, newOutputLineCount);
        var viewLinesChangedEvent = (changeFrom <= changeTo ? new ViewLinesChangedEvent(changeFrom, changeTo) : null);
        var viewLinesInsertedEvent = (insertFrom <= insertTo ? new ViewLinesInsertedEvent(insertFrom, insertTo) : null);
        var viewLinesDeletedEvent = (deleteFrom <= deleteTo ? new ViewLinesDeletedEvent(deleteFrom, deleteTo) : null);
        return [lineMappingChanged, viewLinesChangedEvent, viewLinesInsertedEvent, viewLinesDeletedEvent];
    };
    SplitLinesCollection.prototype.acceptVersionId = function (versionId) {
        this._validModelVersionId = versionId;
        if (this.lines.length === 1 && !this.lines[0].isVisible()) {
            // At least one line must be visible => reset hidden areas
            this.setHiddenAreas([]);
        }
    };
    SplitLinesCollection.prototype.getViewLineCount = function () {
        this._ensureValidState();
        return this.prefixSumComputer.getTotalValue();
    };
    SplitLinesCollection.prototype._toValidViewLineNumber = function (viewLineNumber) {
        if (viewLineNumber < 1) {
            return 1;
        }
        var viewLineCount = this.getViewLineCount();
        if (viewLineNumber > viewLineCount) {
            return viewLineCount;
        }
        return viewLineNumber;
    };
    /**
     * Gives a hint that a lot of requests are about to come in for these line numbers.
     */
    SplitLinesCollection.prototype.warmUpLookupCache = function (viewStartLineNumber, viewEndLineNumber) {
        this.prefixSumComputer.warmUpCache(viewStartLineNumber - 1, viewEndLineNumber - 1);
    };
    SplitLinesCollection.prototype.getActiveIndentGuide = function (viewLineNumber, minLineNumber, maxLineNumber) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        minLineNumber = this._toValidViewLineNumber(minLineNumber);
        maxLineNumber = this._toValidViewLineNumber(maxLineNumber);
        var modelPosition = this.convertViewPositionToModelPosition(viewLineNumber, this.getViewLineMinColumn(viewLineNumber));
        var modelMinPosition = this.convertViewPositionToModelPosition(minLineNumber, this.getViewLineMinColumn(minLineNumber));
        var modelMaxPosition = this.convertViewPositionToModelPosition(maxLineNumber, this.getViewLineMinColumn(maxLineNumber));
        var result = this.model.getActiveIndentGuide(modelPosition.lineNumber, modelMinPosition.lineNumber, modelMaxPosition.lineNumber);
        var viewStartPosition = this.convertModelPositionToViewPosition(result.startLineNumber, 1);
        var viewEndPosition = this.convertModelPositionToViewPosition(result.endLineNumber, this.model.getLineMaxColumn(result.endLineNumber));
        return {
            startLineNumber: viewStartPosition.lineNumber,
            endLineNumber: viewEndPosition.lineNumber,
            indent: result.indent
        };
    };
    SplitLinesCollection.prototype.getViewLinesIndentGuides = function (viewStartLineNumber, viewEndLineNumber) {
        this._ensureValidState();
        viewStartLineNumber = this._toValidViewLineNumber(viewStartLineNumber);
        viewEndLineNumber = this._toValidViewLineNumber(viewEndLineNumber);
        var modelStart = this.convertViewPositionToModelPosition(viewStartLineNumber, this.getViewLineMinColumn(viewStartLineNumber));
        var modelEnd = this.convertViewPositionToModelPosition(viewEndLineNumber, this.getViewLineMaxColumn(viewEndLineNumber));
        var result = [];
        var resultRepeatCount = [];
        var resultRepeatOption = [];
        var modelStartLineIndex = modelStart.lineNumber - 1;
        var modelEndLineIndex = modelEnd.lineNumber - 1;
        var reqStart = null;
        for (var modelLineIndex = modelStartLineIndex; modelLineIndex <= modelEndLineIndex; modelLineIndex++) {
            var line = this.lines[modelLineIndex];
            if (line.isVisible()) {
                var viewLineStartIndex = line.getViewLineNumberOfModelPosition(0, modelLineIndex === modelStartLineIndex ? modelStart.column : 1);
                var viewLineEndIndex = line.getViewLineNumberOfModelPosition(0, this.model.getLineMaxColumn(modelLineIndex + 1));
                var count = viewLineEndIndex - viewLineStartIndex + 1;
                var option = 0 /* BlockNone */;
                if (count > 1 && line.getViewLineMinColumn(this.model, modelLineIndex + 1, viewLineEndIndex) === 1) {
                    // wrapped lines should block indent guides
                    option = (viewLineStartIndex === 0 ? 1 /* BlockSubsequent */ : 2 /* BlockAll */);
                }
                resultRepeatCount.push(count);
                resultRepeatOption.push(option);
                // merge into previous request
                if (reqStart === null) {
                    reqStart = new Position(modelLineIndex + 1, 0);
                }
            }
            else {
                // hit invisible line => flush request
                if (reqStart !== null) {
                    result = result.concat(this.model.getLinesIndentGuides(reqStart.lineNumber, modelLineIndex));
                    reqStart = null;
                }
            }
        }
        if (reqStart !== null) {
            result = result.concat(this.model.getLinesIndentGuides(reqStart.lineNumber, modelEnd.lineNumber));
            reqStart = null;
        }
        var viewLineCount = viewEndLineNumber - viewStartLineNumber + 1;
        var viewIndents = new Array(viewLineCount);
        var currIndex = 0;
        for (var i = 0, len = result.length; i < len; i++) {
            var value = result[i];
            var count = Math.min(viewLineCount - currIndex, resultRepeatCount[i]);
            var option = resultRepeatOption[i];
            var blockAtIndex = void 0;
            if (option === 2 /* BlockAll */) {
                blockAtIndex = 0;
            }
            else if (option === 1 /* BlockSubsequent */) {
                blockAtIndex = 1;
            }
            else {
                blockAtIndex = count;
            }
            for (var j = 0; j < count; j++) {
                if (j === blockAtIndex) {
                    value = 0;
                }
                viewIndents[currIndex++] = value;
            }
        }
        return viewIndents;
    };
    SplitLinesCollection.prototype.getViewLineContent = function (viewLineNumber) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        var r = this.prefixSumComputer.getIndexOf(viewLineNumber - 1);
        var lineIndex = r.index;
        var remainder = r.remainder;
        return this.lines[lineIndex].getViewLineContent(this.model, lineIndex + 1, remainder);
    };
    SplitLinesCollection.prototype.getViewLineLength = function (viewLineNumber) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        var r = this.prefixSumComputer.getIndexOf(viewLineNumber - 1);
        var lineIndex = r.index;
        var remainder = r.remainder;
        return this.lines[lineIndex].getViewLineLength(this.model, lineIndex + 1, remainder);
    };
    SplitLinesCollection.prototype.getViewLineMinColumn = function (viewLineNumber) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        var r = this.prefixSumComputer.getIndexOf(viewLineNumber - 1);
        var lineIndex = r.index;
        var remainder = r.remainder;
        return this.lines[lineIndex].getViewLineMinColumn(this.model, lineIndex + 1, remainder);
    };
    SplitLinesCollection.prototype.getViewLineMaxColumn = function (viewLineNumber) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        var r = this.prefixSumComputer.getIndexOf(viewLineNumber - 1);
        var lineIndex = r.index;
        var remainder = r.remainder;
        return this.lines[lineIndex].getViewLineMaxColumn(this.model, lineIndex + 1, remainder);
    };
    SplitLinesCollection.prototype.getViewLineData = function (viewLineNumber) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        var r = this.prefixSumComputer.getIndexOf(viewLineNumber - 1);
        var lineIndex = r.index;
        var remainder = r.remainder;
        return this.lines[lineIndex].getViewLineData(this.model, lineIndex + 1, remainder);
    };
    SplitLinesCollection.prototype.getViewLinesData = function (viewStartLineNumber, viewEndLineNumber, needed) {
        this._ensureValidState();
        viewStartLineNumber = this._toValidViewLineNumber(viewStartLineNumber);
        viewEndLineNumber = this._toValidViewLineNumber(viewEndLineNumber);
        var start = this.prefixSumComputer.getIndexOf(viewStartLineNumber - 1);
        var viewLineNumber = viewStartLineNumber;
        var startModelLineIndex = start.index;
        var startRemainder = start.remainder;
        var result = [];
        for (var modelLineIndex = startModelLineIndex, len = this.model.getLineCount(); modelLineIndex < len; modelLineIndex++) {
            var line = this.lines[modelLineIndex];
            if (!line.isVisible()) {
                continue;
            }
            var fromViewLineIndex = (modelLineIndex === startModelLineIndex ? startRemainder : 0);
            var remainingViewLineCount = line.getViewLineCount() - fromViewLineIndex;
            var lastLine = false;
            if (viewLineNumber + remainingViewLineCount > viewEndLineNumber) {
                lastLine = true;
                remainingViewLineCount = viewEndLineNumber - viewLineNumber + 1;
            }
            var toViewLineIndex = fromViewLineIndex + remainingViewLineCount;
            line.getViewLinesData(this.model, modelLineIndex + 1, fromViewLineIndex, toViewLineIndex, viewLineNumber - viewStartLineNumber, needed, result);
            viewLineNumber += remainingViewLineCount;
            if (lastLine) {
                break;
            }
        }
        return result;
    };
    SplitLinesCollection.prototype.validateViewPosition = function (viewLineNumber, viewColumn, expectedModelPosition) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        var r = this.prefixSumComputer.getIndexOf(viewLineNumber - 1);
        var lineIndex = r.index;
        var remainder = r.remainder;
        var line = this.lines[lineIndex];
        var minColumn = line.getViewLineMinColumn(this.model, lineIndex + 1, remainder);
        var maxColumn = line.getViewLineMaxColumn(this.model, lineIndex + 1, remainder);
        if (viewColumn < minColumn) {
            viewColumn = minColumn;
        }
        if (viewColumn > maxColumn) {
            viewColumn = maxColumn;
        }
        var computedModelColumn = line.getModelColumnOfViewPosition(remainder, viewColumn);
        var computedModelPosition = this.model.validatePosition(new Position(lineIndex + 1, computedModelColumn));
        if (computedModelPosition.equals(expectedModelPosition)) {
            return new Position(viewLineNumber, viewColumn);
        }
        return this.convertModelPositionToViewPosition(expectedModelPosition.lineNumber, expectedModelPosition.column);
    };
    SplitLinesCollection.prototype.convertViewPositionToModelPosition = function (viewLineNumber, viewColumn) {
        this._ensureValidState();
        viewLineNumber = this._toValidViewLineNumber(viewLineNumber);
        var r = this.prefixSumComputer.getIndexOf(viewLineNumber - 1);
        var lineIndex = r.index;
        var remainder = r.remainder;
        var inputColumn = this.lines[lineIndex].getModelColumnOfViewPosition(remainder, viewColumn);
        // console.log('out -> in ' + viewLineNumber + ',' + viewColumn + ' ===> ' + (lineIndex+1) + ',' + inputColumn);
        return this.model.validatePosition(new Position(lineIndex + 1, inputColumn));
    };
    SplitLinesCollection.prototype.convertModelPositionToViewPosition = function (_modelLineNumber, _modelColumn) {
        this._ensureValidState();
        var validPosition = this.model.validatePosition(new Position(_modelLineNumber, _modelColumn));
        var inputLineNumber = validPosition.lineNumber;
        var inputColumn = validPosition.column;
        var lineIndex = inputLineNumber - 1, lineIndexChanged = false;
        while (lineIndex > 0 && !this.lines[lineIndex].isVisible()) {
            lineIndex--;
            lineIndexChanged = true;
        }
        if (lineIndex === 0 && !this.lines[lineIndex].isVisible()) {
            // Could not reach a real line
            // console.log('in -> out ' + inputLineNumber + ',' + inputColumn + ' ===> ' + 1 + ',' + 1);
            return new Position(1, 1);
        }
        var deltaLineNumber = 1 + (lineIndex === 0 ? 0 : this.prefixSumComputer.getAccumulatedValue(lineIndex - 1));
        var r;
        if (lineIndexChanged) {
            r = this.lines[lineIndex].getViewPositionOfModelPosition(deltaLineNumber, this.model.getLineMaxColumn(lineIndex + 1));
        }
        else {
            r = this.lines[inputLineNumber - 1].getViewPositionOfModelPosition(deltaLineNumber, inputColumn);
        }
        // console.log('in -> out ' + inputLineNumber + ',' + inputColumn + ' ===> ' + r.lineNumber + ',' + r);
        return r;
    };
    SplitLinesCollection.prototype._getViewLineNumberForModelPosition = function (inputLineNumber, inputColumn) {
        var lineIndex = inputLineNumber - 1;
        if (this.lines[lineIndex].isVisible()) {
            // this model line is visible
            var deltaLineNumber_1 = 1 + (lineIndex === 0 ? 0 : this.prefixSumComputer.getAccumulatedValue(lineIndex - 1));
            return this.lines[lineIndex].getViewLineNumberOfModelPosition(deltaLineNumber_1, inputColumn);
        }
        // this model line is not visible
        while (lineIndex > 0 && !this.lines[lineIndex].isVisible()) {
            lineIndex--;
        }
        if (lineIndex === 0 && !this.lines[lineIndex].isVisible()) {
            // Could not reach a real line
            return 1;
        }
        var deltaLineNumber = 1 + (lineIndex === 0 ? 0 : this.prefixSumComputer.getAccumulatedValue(lineIndex - 1));
        return this.lines[lineIndex].getViewLineNumberOfModelPosition(deltaLineNumber, this.model.getLineMaxColumn(lineIndex + 1));
    };
    SplitLinesCollection.prototype.getAllOverviewRulerDecorations = function (ownerId, filterOutValidation, theme) {
        var decorations = this.model.getOverviewRulerDecorations(ownerId, filterOutValidation);
        var result = new OverviewRulerDecorations();
        for (var i = 0, len = decorations.length; i < len; i++) {
            var decoration = decorations[i];
            var opts = decoration.options.overviewRuler;
            var lane = opts ? opts.position : 0;
            if (lane === 0) {
                continue;
            }
            var color = opts.getColor(theme);
            var viewStartLineNumber = this._getViewLineNumberForModelPosition(decoration.range.startLineNumber, decoration.range.startColumn);
            var viewEndLineNumber = this._getViewLineNumberForModelPosition(decoration.range.endLineNumber, decoration.range.endColumn);
            result.accept(color, viewStartLineNumber, viewEndLineNumber, lane);
        }
        return result.result;
    };
    SplitLinesCollection.prototype.getDecorationsInRange = function (range, ownerId, filterOutValidation) {
        var modelStart = this.convertViewPositionToModelPosition(range.startLineNumber, range.startColumn);
        var modelEnd = this.convertViewPositionToModelPosition(range.endLineNumber, range.endColumn);
        if (modelEnd.lineNumber - modelStart.lineNumber <= range.endLineNumber - range.startLineNumber) {
            // most likely there are no hidden lines => fast path
            // fetch decorations from column 1 to cover the case of wrapped lines that have whole line decorations at column 1
            return this.model.getDecorationsInRange(new Range(modelStart.lineNumber, 1, modelEnd.lineNumber, modelEnd.column), ownerId, filterOutValidation);
        }
        var result = [];
        var modelStartLineIndex = modelStart.lineNumber - 1;
        var modelEndLineIndex = modelEnd.lineNumber - 1;
        var reqStart = null;
        for (var modelLineIndex = modelStartLineIndex; modelLineIndex <= modelEndLineIndex; modelLineIndex++) {
            var line = this.lines[modelLineIndex];
            if (line.isVisible()) {
                // merge into previous request
                if (reqStart === null) {
                    reqStart = new Position(modelLineIndex + 1, modelLineIndex === modelStartLineIndex ? modelStart.column : 1);
                }
            }
            else {
                // hit invisible line => flush request
                if (reqStart !== null) {
                    var maxLineColumn = this.model.getLineMaxColumn(modelLineIndex);
                    result = result.concat(this.model.getDecorationsInRange(new Range(reqStart.lineNumber, reqStart.column, modelLineIndex, maxLineColumn), ownerId, filterOutValidation));
                    reqStart = null;
                }
            }
        }
        if (reqStart !== null) {
            result = result.concat(this.model.getDecorationsInRange(new Range(reqStart.lineNumber, reqStart.column, modelEnd.lineNumber, modelEnd.column), ownerId, filterOutValidation));
            reqStart = null;
        }
        result.sort(function (a, b) {
            var res = Range.compareRangesUsingStarts(a.range, b.range);
            if (res === 0) {
                if (a.id < b.id) {
                    return -1;
                }
                if (a.id > b.id) {
                    return 1;
                }
                return 0;
            }
            return res;
        });
        // Eliminate duplicate decorations that might have intersected our visible ranges multiple times
        var finalResult = [], finalResultLen = 0;
        var prevDecId = null;
        for (var i = 0, len = result.length; i < len; i++) {
            var dec = result[i];
            var decId = dec.id;
            if (prevDecId === decId) {
                // skip
                continue;
            }
            prevDecId = decId;
            finalResult[finalResultLen++] = dec;
        }
        return finalResult;
    };
    return SplitLinesCollection;
}());
var VisibleIdentitySplitLine = /** @class */ (function () {
    function VisibleIdentitySplitLine() {
    }
    VisibleIdentitySplitLine.prototype.isVisible = function () {
        return true;
    };
    VisibleIdentitySplitLine.prototype.setVisible = function (isVisible) {
        if (isVisible) {
            return this;
        }
        return InvisibleIdentitySplitLine.INSTANCE;
    };
    VisibleIdentitySplitLine.prototype.getViewLineCount = function () {
        return 1;
    };
    VisibleIdentitySplitLine.prototype.getViewLineContent = function (model, modelLineNumber, _outputLineIndex) {
        return model.getLineContent(modelLineNumber);
    };
    VisibleIdentitySplitLine.prototype.getViewLineLength = function (model, modelLineNumber, _outputLineIndex) {
        return model.getLineLength(modelLineNumber);
    };
    VisibleIdentitySplitLine.prototype.getViewLineMinColumn = function (model, modelLineNumber, _outputLineIndex) {
        return model.getLineMinColumn(modelLineNumber);
    };
    VisibleIdentitySplitLine.prototype.getViewLineMaxColumn = function (model, modelLineNumber, _outputLineIndex) {
        return model.getLineMaxColumn(modelLineNumber);
    };
    VisibleIdentitySplitLine.prototype.getViewLineData = function (model, modelLineNumber, _outputLineIndex) {
        var lineTokens = model.getLineTokens(modelLineNumber);
        var lineContent = lineTokens.getLineContent();
        return new ViewLineData(lineContent, false, 1, lineContent.length + 1, lineTokens.inflate());
    };
    VisibleIdentitySplitLine.prototype.getViewLinesData = function (model, modelLineNumber, _fromOuputLineIndex, _toOutputLineIndex, globalStartIndex, needed, result) {
        if (!needed[globalStartIndex]) {
            result[globalStartIndex] = null;
            return;
        }
        result[globalStartIndex] = this.getViewLineData(model, modelLineNumber, 0);
    };
    VisibleIdentitySplitLine.prototype.getModelColumnOfViewPosition = function (_outputLineIndex, outputColumn) {
        return outputColumn;
    };
    VisibleIdentitySplitLine.prototype.getViewPositionOfModelPosition = function (deltaLineNumber, inputColumn) {
        return new Position(deltaLineNumber, inputColumn);
    };
    VisibleIdentitySplitLine.prototype.getViewLineNumberOfModelPosition = function (deltaLineNumber, _inputColumn) {
        return deltaLineNumber;
    };
    VisibleIdentitySplitLine.INSTANCE = new VisibleIdentitySplitLine();
    return VisibleIdentitySplitLine;
}());
var InvisibleIdentitySplitLine = /** @class */ (function () {
    function InvisibleIdentitySplitLine() {
    }
    InvisibleIdentitySplitLine.prototype.isVisible = function () {
        return false;
    };
    InvisibleIdentitySplitLine.prototype.setVisible = function (isVisible) {
        if (!isVisible) {
            return this;
        }
        return VisibleIdentitySplitLine.INSTANCE;
    };
    InvisibleIdentitySplitLine.prototype.getViewLineCount = function () {
        return 0;
    };
    InvisibleIdentitySplitLine.prototype.getViewLineContent = function (_model, _modelLineNumber, _outputLineIndex) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getViewLineLength = function (_model, _modelLineNumber, _outputLineIndex) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getViewLineMinColumn = function (_model, _modelLineNumber, _outputLineIndex) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getViewLineMaxColumn = function (_model, _modelLineNumber, _outputLineIndex) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getViewLineData = function (_model, _modelLineNumber, _outputLineIndex) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getViewLinesData = function (_model, _modelLineNumber, _fromOuputLineIndex, _toOutputLineIndex, _globalStartIndex, _needed, _result) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getModelColumnOfViewPosition = function (_outputLineIndex, _outputColumn) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getViewPositionOfModelPosition = function (_deltaLineNumber, _inputColumn) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.prototype.getViewLineNumberOfModelPosition = function (_deltaLineNumber, _inputColumn) {
        throw new Error('Not supported');
    };
    InvisibleIdentitySplitLine.INSTANCE = new InvisibleIdentitySplitLine();
    return InvisibleIdentitySplitLine;
}());
var SplitLine = /** @class */ (function () {
    function SplitLine(positionMapper, isVisible) {
        this.positionMapper = positionMapper;
        this.wrappedIndent = this.positionMapper.getWrappedLinesIndent();
        this.wrappedIndentLength = this.wrappedIndent.length;
        this.outputLineCount = this.positionMapper.getOutputLineCount();
        this._isVisible = isVisible;
    }
    SplitLine.prototype.isVisible = function () {
        return this._isVisible;
    };
    SplitLine.prototype.setVisible = function (isVisible) {
        this._isVisible = isVisible;
        return this;
    };
    SplitLine.prototype.getViewLineCount = function () {
        if (!this._isVisible) {
            return 0;
        }
        return this.outputLineCount;
    };
    SplitLine.prototype.getInputStartOffsetOfOutputLineIndex = function (outputLineIndex) {
        return this.positionMapper.getInputOffsetOfOutputPosition(outputLineIndex, 0);
    };
    SplitLine.prototype.getInputEndOffsetOfOutputLineIndex = function (model, modelLineNumber, outputLineIndex) {
        if (outputLineIndex + 1 === this.outputLineCount) {
            return model.getLineMaxColumn(modelLineNumber) - 1;
        }
        return this.positionMapper.getInputOffsetOfOutputPosition(outputLineIndex + 1, 0);
    };
    SplitLine.prototype.getViewLineContent = function (model, modelLineNumber, outputLineIndex) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        var startOffset = this.getInputStartOffsetOfOutputLineIndex(outputLineIndex);
        var endOffset = this.getInputEndOffsetOfOutputLineIndex(model, modelLineNumber, outputLineIndex);
        var r = model.getValueInRange({
            startLineNumber: modelLineNumber,
            startColumn: startOffset + 1,
            endLineNumber: modelLineNumber,
            endColumn: endOffset + 1
        });
        if (outputLineIndex > 0) {
            r = this.wrappedIndent + r;
        }
        return r;
    };
    SplitLine.prototype.getViewLineLength = function (model, modelLineNumber, outputLineIndex) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        var startOffset = this.getInputStartOffsetOfOutputLineIndex(outputLineIndex);
        var endOffset = this.getInputEndOffsetOfOutputLineIndex(model, modelLineNumber, outputLineIndex);
        var r = endOffset - startOffset;
        if (outputLineIndex > 0) {
            r = this.wrappedIndent.length + r;
        }
        return r;
    };
    SplitLine.prototype.getViewLineMinColumn = function (_model, _modelLineNumber, outputLineIndex) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        if (outputLineIndex > 0) {
            return this.wrappedIndentLength + 1;
        }
        return 1;
    };
    SplitLine.prototype.getViewLineMaxColumn = function (model, modelLineNumber, outputLineIndex) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        return this.getViewLineContent(model, modelLineNumber, outputLineIndex).length + 1;
    };
    SplitLine.prototype.getViewLineData = function (model, modelLineNumber, outputLineIndex) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        var startOffset = this.getInputStartOffsetOfOutputLineIndex(outputLineIndex);
        var endOffset = this.getInputEndOffsetOfOutputLineIndex(model, modelLineNumber, outputLineIndex);
        var lineContent = model.getValueInRange({
            startLineNumber: modelLineNumber,
            startColumn: startOffset + 1,
            endLineNumber: modelLineNumber,
            endColumn: endOffset + 1
        });
        if (outputLineIndex > 0) {
            lineContent = this.wrappedIndent + lineContent;
        }
        var minColumn = (outputLineIndex > 0 ? this.wrappedIndentLength + 1 : 1);
        var maxColumn = lineContent.length + 1;
        var continuesWithWrappedLine = (outputLineIndex + 1 < this.getViewLineCount());
        var deltaStartIndex = 0;
        if (outputLineIndex > 0) {
            deltaStartIndex = this.wrappedIndentLength;
        }
        var lineTokens = model.getLineTokens(modelLineNumber);
        return new ViewLineData(lineContent, continuesWithWrappedLine, minColumn, maxColumn, lineTokens.sliceAndInflate(startOffset, endOffset, deltaStartIndex));
    };
    SplitLine.prototype.getViewLinesData = function (model, modelLineNumber, fromOuputLineIndex, toOutputLineIndex, globalStartIndex, needed, result) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        for (var outputLineIndex = fromOuputLineIndex; outputLineIndex < toOutputLineIndex; outputLineIndex++) {
            var globalIndex = globalStartIndex + outputLineIndex - fromOuputLineIndex;
            if (!needed[globalIndex]) {
                result[globalIndex] = null;
                continue;
            }
            result[globalIndex] = this.getViewLineData(model, modelLineNumber, outputLineIndex);
        }
    };
    SplitLine.prototype.getModelColumnOfViewPosition = function (outputLineIndex, outputColumn) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        var adjustedColumn = outputColumn - 1;
        if (outputLineIndex > 0) {
            if (adjustedColumn < this.wrappedIndentLength) {
                adjustedColumn = 0;
            }
            else {
                adjustedColumn -= this.wrappedIndentLength;
            }
        }
        return this.positionMapper.getInputOffsetOfOutputPosition(outputLineIndex, adjustedColumn) + 1;
    };
    SplitLine.prototype.getViewPositionOfModelPosition = function (deltaLineNumber, inputColumn) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        var r = this.positionMapper.getOutputPositionOfInputOffset(inputColumn - 1);
        var outputLineIndex = r.outputLineIndex;
        var outputColumn = r.outputOffset + 1;
        if (outputLineIndex > 0) {
            outputColumn += this.wrappedIndentLength;
        }
        //		console.log('in -> out ' + deltaLineNumber + ',' + inputColumn + ' ===> ' + (deltaLineNumber+outputLineIndex) + ',' + outputColumn);
        return new Position(deltaLineNumber + outputLineIndex, outputColumn);
    };
    SplitLine.prototype.getViewLineNumberOfModelPosition = function (deltaLineNumber, inputColumn) {
        if (!this._isVisible) {
            throw new Error('Not supported');
        }
        var r = this.positionMapper.getOutputPositionOfInputOffset(inputColumn - 1);
        return (deltaLineNumber + r.outputLineIndex);
    };
    return SplitLine;
}());
function createSplitLine(linePositionMapperFactory, text, tabSize, wrappingColumn, columnsForFullWidthChar, wrappingIndent, isVisible) {
    var positionMapper = linePositionMapperFactory.createLineMapping(text, tabSize, wrappingColumn, columnsForFullWidthChar, wrappingIndent);
    if (positionMapper === null) {
        // No mapping needed
        if (isVisible) {
            return VisibleIdentitySplitLine.INSTANCE;
        }
        return InvisibleIdentitySplitLine.INSTANCE;
    }
    else {
        return new SplitLine(positionMapper, isVisible);
    }
}
var IdentityCoordinatesConverter = /** @class */ (function () {
    function IdentityCoordinatesConverter(lines) {
        this._lines = lines;
    }
    IdentityCoordinatesConverter.prototype._validPosition = function (pos) {
        return this._lines.model.validatePosition(pos);
    };
    IdentityCoordinatesConverter.prototype._validRange = function (range) {
        return this._lines.model.validateRange(range);
    };
    // View -> Model conversion and related methods
    IdentityCoordinatesConverter.prototype.convertViewPositionToModelPosition = function (viewPosition) {
        return this._validPosition(viewPosition);
    };
    IdentityCoordinatesConverter.prototype.convertViewRangeToModelRange = function (viewRange) {
        return this._validRange(viewRange);
    };
    IdentityCoordinatesConverter.prototype.validateViewPosition = function (_viewPosition, expectedModelPosition) {
        return this._validPosition(expectedModelPosition);
    };
    IdentityCoordinatesConverter.prototype.validateViewRange = function (_viewRange, expectedModelRange) {
        return this._validRange(expectedModelRange);
    };
    // Model -> View conversion and related methods
    IdentityCoordinatesConverter.prototype.convertModelPositionToViewPosition = function (modelPosition) {
        return this._validPosition(modelPosition);
    };
    IdentityCoordinatesConverter.prototype.convertModelRangeToViewRange = function (modelRange) {
        return this._validRange(modelRange);
    };
    IdentityCoordinatesConverter.prototype.modelPositionIsVisible = function (modelPosition) {
        var lineCount = this._lines.model.getLineCount();
        if (modelPosition.lineNumber < 1 || modelPosition.lineNumber > lineCount) {
            // invalid arguments
            return false;
        }
        return true;
    };
    return IdentityCoordinatesConverter;
}());
var IdentityLinesCollection = /** @class */ (function () {
    function IdentityLinesCollection(model) {
        this.model = model;
    }
    IdentityLinesCollection.prototype.dispose = function () {
    };
    IdentityLinesCollection.prototype.createCoordinatesConverter = function () {
        return new IdentityCoordinatesConverter(this);
    };
    IdentityLinesCollection.prototype.getHiddenAreas = function () {
        return [];
    };
    IdentityLinesCollection.prototype.setHiddenAreas = function (_ranges) {
        return false;
    };
    IdentityLinesCollection.prototype.setTabSize = function (_newTabSize) {
        return false;
    };
    IdentityLinesCollection.prototype.setWrappingSettings = function (_wrappingIndent, _wrappingColumn, _columnsForFullWidthChar) {
        return false;
    };
    IdentityLinesCollection.prototype.onModelFlushed = function () {
    };
    IdentityLinesCollection.prototype.onModelLinesDeleted = function (_versionId, fromLineNumber, toLineNumber) {
        return new ViewLinesDeletedEvent(fromLineNumber, toLineNumber);
    };
    IdentityLinesCollection.prototype.onModelLinesInserted = function (_versionId, fromLineNumber, toLineNumber, _text) {
        return new ViewLinesInsertedEvent(fromLineNumber, toLineNumber);
    };
    IdentityLinesCollection.prototype.onModelLineChanged = function (_versionId, lineNumber, _newText) {
        return [false, new ViewLinesChangedEvent(lineNumber, lineNumber), null, null];
    };
    IdentityLinesCollection.prototype.acceptVersionId = function (_versionId) {
    };
    IdentityLinesCollection.prototype.getViewLineCount = function () {
        return this.model.getLineCount();
    };
    IdentityLinesCollection.prototype.warmUpLookupCache = function (_viewStartLineNumber, _viewEndLineNumber) {
    };
    IdentityLinesCollection.prototype.getActiveIndentGuide = function (viewLineNumber, _minLineNumber, _maxLineNumber) {
        return {
            startLineNumber: viewLineNumber,
            endLineNumber: viewLineNumber,
            indent: 0
        };
    };
    IdentityLinesCollection.prototype.getViewLinesIndentGuides = function (viewStartLineNumber, viewEndLineNumber) {
        var viewLineCount = viewEndLineNumber - viewStartLineNumber + 1;
        var result = new Array(viewLineCount);
        for (var i = 0; i < viewLineCount; i++) {
            result[i] = 0;
        }
        return result;
    };
    IdentityLinesCollection.prototype.getViewLineContent = function (viewLineNumber) {
        return this.model.getLineContent(viewLineNumber);
    };
    IdentityLinesCollection.prototype.getViewLineLength = function (viewLineNumber) {
        return this.model.getLineLength(viewLineNumber);
    };
    IdentityLinesCollection.prototype.getViewLineMinColumn = function (viewLineNumber) {
        return this.model.getLineMinColumn(viewLineNumber);
    };
    IdentityLinesCollection.prototype.getViewLineMaxColumn = function (viewLineNumber) {
        return this.model.getLineMaxColumn(viewLineNumber);
    };
    IdentityLinesCollection.prototype.getViewLineData = function (viewLineNumber) {
        var lineTokens = this.model.getLineTokens(viewLineNumber);
        var lineContent = lineTokens.getLineContent();
        return new ViewLineData(lineContent, false, 1, lineContent.length + 1, lineTokens.inflate());
    };
    IdentityLinesCollection.prototype.getViewLinesData = function (viewStartLineNumber, viewEndLineNumber, needed) {
        var lineCount = this.model.getLineCount();
        viewStartLineNumber = Math.min(Math.max(1, viewStartLineNumber), lineCount);
        viewEndLineNumber = Math.min(Math.max(1, viewEndLineNumber), lineCount);
        var result = [];
        for (var lineNumber = viewStartLineNumber; lineNumber <= viewEndLineNumber; lineNumber++) {
            var idx = lineNumber - viewStartLineNumber;
            if (!needed[idx]) {
                result[idx] = null;
            }
            result[idx] = this.getViewLineData(lineNumber);
        }
        return result;
    };
    IdentityLinesCollection.prototype.getAllOverviewRulerDecorations = function (ownerId, filterOutValidation, theme) {
        var decorations = this.model.getOverviewRulerDecorations(ownerId, filterOutValidation);
        var result = new OverviewRulerDecorations();
        for (var i = 0, len = decorations.length; i < len; i++) {
            var decoration = decorations[i];
            var opts = decoration.options.overviewRuler;
            var lane = opts ? opts.position : 0;
            if (lane === 0) {
                continue;
            }
            var color = opts.getColor(theme);
            var viewStartLineNumber = decoration.range.startLineNumber;
            var viewEndLineNumber = decoration.range.endLineNumber;
            result.accept(color, viewStartLineNumber, viewEndLineNumber, lane);
        }
        return result.result;
    };
    IdentityLinesCollection.prototype.getDecorationsInRange = function (range, ownerId, filterOutValidation) {
        return this.model.getDecorationsInRange(range, ownerId, filterOutValidation);
    };
    return IdentityLinesCollection;
}());
var OverviewRulerDecorations = /** @class */ (function () {
    function OverviewRulerDecorations() {
        this.result = Object.create(null);
    }
    OverviewRulerDecorations.prototype.accept = function (color, startLineNumber, endLineNumber, lane) {
        var prev = this.result[color];
        if (prev) {
            var prevLane = prev[prev.length - 3];
            var prevEndLineNumber = prev[prev.length - 1];
            if (prevLane === lane && prevEndLineNumber + 1 >= startLineNumber) {
                // merge into prev
                if (endLineNumber > prevEndLineNumber) {
                    prev[prev.length - 1] = endLineNumber;
                }
                return;
            }
            // push
            prev.push(lane, startLineNumber, endLineNumber);
        }
        else {
            this.result[color] = [lane, startLineNumber, endLineNumber];
        }
    };
    return OverviewRulerDecorations;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$14 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var WrappingCharacterClassifier = /** @class */ (function (_super) {
    __extends$14(WrappingCharacterClassifier, _super);
    function WrappingCharacterClassifier(BREAK_BEFORE, BREAK_AFTER, BREAK_OBTRUSIVE) {
        var _this = _super.call(this, 0 /* NONE */) || this;
        for (var i = 0; i < BREAK_BEFORE.length; i++) {
            _this.set(BREAK_BEFORE.charCodeAt(i), 1 /* BREAK_BEFORE */);
        }
        for (var i = 0; i < BREAK_AFTER.length; i++) {
            _this.set(BREAK_AFTER.charCodeAt(i), 2 /* BREAK_AFTER */);
        }
        for (var i = 0; i < BREAK_OBTRUSIVE.length; i++) {
            _this.set(BREAK_OBTRUSIVE.charCodeAt(i), 3 /* BREAK_OBTRUSIVE */);
        }
        return _this;
    }
    WrappingCharacterClassifier.prototype.get = function (charCode) {
        // Initialize CharacterClass.BREAK_IDEOGRAPHIC for these Unicode ranges:
        // 1. CJK Unified Ideographs (0x4E00 -- 0x9FFF)
        // 2. CJK Unified Ideographs Extension A (0x3400 -- 0x4DBF)
        // 3. Hiragana and Katakana (0x3040 -- 0x30FF)
        if ((charCode >= 0x3040 && charCode <= 0x30FF)
            || (charCode >= 0x3400 && charCode <= 0x4DBF)
            || (charCode >= 0x4E00 && charCode <= 0x9FFF)) {
            return 4 /* BREAK_IDEOGRAPHIC */;
        }
        return _super.prototype.get.call(this, charCode);
    };
    return WrappingCharacterClassifier;
}(CharacterClassifier));
var CharacterHardWrappingLineMapperFactory = /** @class */ (function () {
    function CharacterHardWrappingLineMapperFactory(breakBeforeChars, breakAfterChars, breakObtrusiveChars) {
        this.classifier = new WrappingCharacterClassifier(breakBeforeChars, breakAfterChars, breakObtrusiveChars);
    }
    // TODO@Alex -> duplicated in lineCommentCommand
    CharacterHardWrappingLineMapperFactory.nextVisibleColumn = function (currentVisibleColumn, tabSize, isTab, columnSize) {
        currentVisibleColumn = +currentVisibleColumn; //@perf
        tabSize = +tabSize; //@perf
        columnSize = +columnSize; //@perf
        if (isTab) {
            return currentVisibleColumn + (tabSize - (currentVisibleColumn % tabSize));
        }
        return currentVisibleColumn + columnSize;
    };
    CharacterHardWrappingLineMapperFactory.prototype.createLineMapping = function (lineText, tabSize, breakingColumn, columnsForFullWidthChar, hardWrappingIndent) {
        if (breakingColumn === -1) {
            return null;
        }
        tabSize = +tabSize; //@perf
        breakingColumn = +breakingColumn; //@perf
        columnsForFullWidthChar = +columnsForFullWidthChar; //@perf
        hardWrappingIndent = +hardWrappingIndent; //@perf
        var wrappedTextIndentVisibleColumn = 0;
        var wrappedTextIndent = '';
        var firstNonWhitespaceIndex$$1 = -1;
        if (hardWrappingIndent !== 0 /* None */) {
            firstNonWhitespaceIndex$$1 = firstNonWhitespaceIndex(lineText);
            if (firstNonWhitespaceIndex$$1 !== -1) {
                // Track existing indent
                wrappedTextIndent = lineText.substring(0, firstNonWhitespaceIndex$$1);
                for (var i = 0; i < firstNonWhitespaceIndex$$1; i++) {
                    wrappedTextIndentVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(wrappedTextIndentVisibleColumn, tabSize, lineText.charCodeAt(i) === 9 /* Tab */, 1);
                }
                // Increase indent of continuation lines, if desired
                var numberOfAdditionalTabs = 0;
                if (hardWrappingIndent === 2 /* Indent */) {
                    numberOfAdditionalTabs = 1;
                }
                else if (hardWrappingIndent === 3 /* DeepIndent */) {
                    numberOfAdditionalTabs = 2;
                }
                for (var i = 0; i < numberOfAdditionalTabs; i++) {
                    wrappedTextIndent += '\t';
                    wrappedTextIndentVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(wrappedTextIndentVisibleColumn, tabSize, true, 1);
                }
                // Force sticking to beginning of line if no character would fit except for the indentation
                if (wrappedTextIndentVisibleColumn + columnsForFullWidthChar > breakingColumn) {
                    wrappedTextIndent = '';
                    wrappedTextIndentVisibleColumn = 0;
                }
            }
        }
        var classifier = this.classifier;
        var lastBreakingOffset = 0; // Last 0-based offset in the lineText at which a break happened
        var breakingLengths = []; // The length of each broken-up line text
        var breakingLengthsIndex = 0; // The count of breaks already done
        var visibleColumn = 0; // Visible column since the beginning of the current line
        var niceBreakOffset = -1; // Last index of a character that indicates a break should happen before it (more desirable)
        var niceBreakVisibleColumn = 0; // visible column if a break were to be later introduced before `niceBreakOffset`
        var obtrusiveBreakOffset = -1; // Last index of a character that indicates a break should happen before it (less desirable)
        var obtrusiveBreakVisibleColumn = 0; // visible column if a break were to be later introduced before `obtrusiveBreakOffset`
        var len = lineText.length;
        for (var i = 0; i < len; i++) {
            // At this point, there is a certainty that the character before `i` fits on the current line,
            // but the character at `i` might not fit
            var charCode = lineText.charCodeAt(i);
            var charCodeIsTab = (charCode === 9 /* Tab */);
            var charCodeClass = classifier.get(charCode);
            if (charCodeClass === 1 /* BREAK_BEFORE */) {
                // This is a character that indicates that a break should happen before it
                // Since we are certain the character before `i` fits, there's no extra checking needed,
                // just mark it as a nice breaking opportunity
                niceBreakOffset = i;
                niceBreakVisibleColumn = wrappedTextIndentVisibleColumn;
            }
            // CJK breaking : before break
            if (charCodeClass === 4 /* BREAK_IDEOGRAPHIC */ && i > 0) {
                var prevCode = lineText.charCodeAt(i - 1);
                var prevClass = classifier.get(prevCode);
                if (prevClass !== 1 /* BREAK_BEFORE */) { // Kinsoku Shori: Don't break after a leading character, like an open bracket
                    niceBreakOffset = i;
                    niceBreakVisibleColumn = wrappedTextIndentVisibleColumn;
                }
            }
            var charColumnSize = 1;
            if (isFullWidthCharacter(charCode)) {
                charColumnSize = columnsForFullWidthChar;
            }
            // Advance visibleColumn with character at `i`
            visibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(visibleColumn, tabSize, charCodeIsTab, charColumnSize);
            if (visibleColumn > breakingColumn && i !== 0) {
                // We need to break at least before character at `i`:
                //  - break before niceBreakLastOffset if it exists (and re-establish a correct visibleColumn by using niceBreakVisibleColumn + charAt(i))
                //  - otherwise, break before obtrusiveBreakLastOffset if it exists (and re-establish a correct visibleColumn by using obtrusiveBreakVisibleColumn + charAt(i))
                //  - otherwise, break before i (and re-establish a correct visibleColumn by charAt(i))
                var breakBeforeOffset = void 0;
                var restoreVisibleColumnFrom = void 0;
                if (niceBreakOffset !== -1 && niceBreakVisibleColumn <= breakingColumn) {
                    // We will break before `niceBreakLastOffset`
                    breakBeforeOffset = niceBreakOffset;
                    restoreVisibleColumnFrom = niceBreakVisibleColumn;
                }
                else if (obtrusiveBreakOffset !== -1 && obtrusiveBreakVisibleColumn <= breakingColumn) {
                    // We will break before `obtrusiveBreakLastOffset`
                    breakBeforeOffset = obtrusiveBreakOffset;
                    restoreVisibleColumnFrom = obtrusiveBreakVisibleColumn;
                }
                else {
                    // We will break before `i`
                    breakBeforeOffset = i;
                    restoreVisibleColumnFrom = wrappedTextIndentVisibleColumn;
                }
                // Break before character at `breakBeforeOffset`
                breakingLengths[breakingLengthsIndex++] = breakBeforeOffset - lastBreakingOffset;
                lastBreakingOffset = breakBeforeOffset;
                // Re-establish visibleColumn by taking character at `i` into account
                visibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(restoreVisibleColumnFrom, tabSize, charCodeIsTab, charColumnSize);
                // Reset markers
                niceBreakOffset = -1;
                niceBreakVisibleColumn = 0;
                obtrusiveBreakOffset = -1;
                obtrusiveBreakVisibleColumn = 0;
            }
            // At this point, there is a certainty that the character at `i` fits on the current line
            if (niceBreakOffset !== -1) {
                // Advance niceBreakVisibleColumn
                niceBreakVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(niceBreakVisibleColumn, tabSize, charCodeIsTab, charColumnSize);
            }
            if (obtrusiveBreakOffset !== -1) {
                // Advance obtrusiveBreakVisibleColumn
                obtrusiveBreakVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(obtrusiveBreakVisibleColumn, tabSize, charCodeIsTab, charColumnSize);
            }
            if (charCodeClass === 2 /* BREAK_AFTER */ && (hardWrappingIndent === 0 /* None */ || i >= firstNonWhitespaceIndex$$1)) {
                // This is a character that indicates that a break should happen after it
                niceBreakOffset = i + 1;
                niceBreakVisibleColumn = wrappedTextIndentVisibleColumn;
            }
            // CJK breaking : after break
            if (charCodeClass === 4 /* BREAK_IDEOGRAPHIC */ && i < len - 1) {
                var nextCode = lineText.charCodeAt(i + 1);
                var nextClass = classifier.get(nextCode);
                if (nextClass !== 2 /* BREAK_AFTER */) { // Kinsoku Shori: Don't break before a trailing character, like a period
                    niceBreakOffset = i + 1;
                    niceBreakVisibleColumn = wrappedTextIndentVisibleColumn;
                }
            }
            if (charCodeClass === 3 /* BREAK_OBTRUSIVE */) {
                // This is an obtrusive character that indicates that a break should happen after it
                obtrusiveBreakOffset = i + 1;
                obtrusiveBreakVisibleColumn = wrappedTextIndentVisibleColumn;
            }
        }
        if (breakingLengthsIndex === 0) {
            return null;
        }
        // Add last segment
        breakingLengths[breakingLengthsIndex++] = len - lastBreakingOffset;
        return new CharacterHardWrappingLineMapping(new PrefixSumComputer(toUint32Array(breakingLengths)), wrappedTextIndent);
    };
    return CharacterHardWrappingLineMapperFactory;
}());
var CharacterHardWrappingLineMapping = /** @class */ (function () {
    function CharacterHardWrappingLineMapping(prefixSums, wrappedLinesIndent) {
        this._prefixSums = prefixSums;
        this._wrappedLinesIndent = wrappedLinesIndent;
    }
    CharacterHardWrappingLineMapping.prototype.getOutputLineCount = function () {
        return this._prefixSums.getCount();
    };
    CharacterHardWrappingLineMapping.prototype.getWrappedLinesIndent = function () {
        return this._wrappedLinesIndent;
    };
    CharacterHardWrappingLineMapping.prototype.getInputOffsetOfOutputPosition = function (outputLineIndex, outputOffset) {
        if (outputLineIndex === 0) {
            return outputOffset;
        }
        else {
            return this._prefixSums.getAccumulatedValue(outputLineIndex - 1) + outputOffset;
        }
    };
    CharacterHardWrappingLineMapping.prototype.getOutputPositionOfInputOffset = function (inputOffset) {
        var r = this._prefixSums.getIndexOf(inputOffset);
        return new OutputPosition(r.index, r.remainder);
    };
    return CharacterHardWrappingLineMapping;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ViewModelDecorations = /** @class */ (function () {
    function ViewModelDecorations(editorId, model, configuration, linesCollection, coordinatesConverter) {
        this.editorId = editorId;
        this.model = model;
        this.configuration = configuration;
        this._linesCollection = linesCollection;
        this._coordinatesConverter = coordinatesConverter;
        this._decorationsCache = Object.create(null);
        this._clearCachedModelDecorationsResolver();
    }
    ViewModelDecorations.prototype._clearCachedModelDecorationsResolver = function () {
        this._cachedModelDecorationsResolver = null;
        this._cachedModelDecorationsResolverViewRange = null;
    };
    ViewModelDecorations.prototype.dispose = function () {
        this._decorationsCache = Object.create(null);
        this._clearCachedModelDecorationsResolver();
    };
    ViewModelDecorations.prototype.reset = function () {
        this._decorationsCache = Object.create(null);
        this._clearCachedModelDecorationsResolver();
    };
    ViewModelDecorations.prototype.onModelDecorationsChanged = function () {
        this._decorationsCache = Object.create(null);
        this._clearCachedModelDecorationsResolver();
    };
    ViewModelDecorations.prototype.onLineMappingChanged = function () {
        this._decorationsCache = Object.create(null);
        this._clearCachedModelDecorationsResolver();
    };
    ViewModelDecorations.prototype._getOrCreateViewModelDecoration = function (modelDecoration) {
        var id = modelDecoration.id;
        var r = this._decorationsCache[id];
        if (!r) {
            var modelRange = modelDecoration.range;
            var options = modelDecoration.options;
            var viewRange = void 0;
            if (options.isWholeLine) {
                var start = this._coordinatesConverter.convertModelPositionToViewPosition(new Position(modelRange.startLineNumber, 1));
                var end = this._coordinatesConverter.convertModelPositionToViewPosition(new Position(modelRange.endLineNumber, this.model.getLineMaxColumn(modelRange.endLineNumber)));
                viewRange = new Range(start.lineNumber, start.column, end.lineNumber, end.column);
            }
            else {
                viewRange = this._coordinatesConverter.convertModelRangeToViewRange(modelRange);
            }
            r = new ViewModelDecoration(viewRange, options);
            this._decorationsCache[id] = r;
        }
        return r;
    };
    ViewModelDecorations.prototype.getDecorationsViewportData = function (viewRange) {
        var cacheIsValid = true;
        cacheIsValid = cacheIsValid && (this._cachedModelDecorationsResolver !== null);
        cacheIsValid = cacheIsValid && (viewRange.equalsRange(this._cachedModelDecorationsResolverViewRange));
        if (!cacheIsValid) {
            this._cachedModelDecorationsResolver = this._getDecorationsViewportData(viewRange);
            this._cachedModelDecorationsResolverViewRange = viewRange;
        }
        return this._cachedModelDecorationsResolver;
    };
    ViewModelDecorations.prototype._getDecorationsViewportData = function (viewportRange) {
        var modelDecorations = this._linesCollection.getDecorationsInRange(viewportRange, this.editorId, this.configuration.editor.readOnly);
        var startLineNumber = viewportRange.startLineNumber;
        var endLineNumber = viewportRange.endLineNumber;
        var decorationsInViewport = [], decorationsInViewportLen = 0;
        var inlineDecorations = [];
        for (var j = startLineNumber; j <= endLineNumber; j++) {
            inlineDecorations[j - startLineNumber] = [];
        }
        for (var i = 0, len = modelDecorations.length; i < len; i++) {
            var modelDecoration = modelDecorations[i];
            var decorationOptions = modelDecoration.options;
            var viewModelDecoration = this._getOrCreateViewModelDecoration(modelDecoration);
            var viewRange = viewModelDecoration.range;
            decorationsInViewport[decorationsInViewportLen++] = viewModelDecoration;
            if (decorationOptions.inlineClassName) {
                var inlineDecoration = new InlineDecoration(viewRange, decorationOptions.inlineClassName, decorationOptions.inlineClassNameAffectsLetterSpacing ? 3 /* RegularAffectingLetterSpacing */ : 0 /* Regular */);
                var intersectedStartLineNumber = Math.max(startLineNumber, viewRange.startLineNumber);
                var intersectedEndLineNumber = Math.min(endLineNumber, viewRange.endLineNumber);
                for (var j = intersectedStartLineNumber; j <= intersectedEndLineNumber; j++) {
                    inlineDecorations[j - startLineNumber].push(inlineDecoration);
                }
            }
            if (decorationOptions.beforeContentClassName) {
                if (startLineNumber <= viewRange.startLineNumber && viewRange.startLineNumber <= endLineNumber) {
                    var inlineDecoration = new InlineDecoration(new Range(viewRange.startLineNumber, viewRange.startColumn, viewRange.startLineNumber, viewRange.startColumn), decorationOptions.beforeContentClassName, 1 /* Before */);
                    inlineDecorations[viewRange.startLineNumber - startLineNumber].push(inlineDecoration);
                }
            }
            if (decorationOptions.afterContentClassName) {
                if (startLineNumber <= viewRange.endLineNumber && viewRange.endLineNumber <= endLineNumber) {
                    var inlineDecoration = new InlineDecoration(new Range(viewRange.endLineNumber, viewRange.endColumn, viewRange.endLineNumber, viewRange.endColumn), decorationOptions.afterContentClassName, 2 /* After */);
                    inlineDecorations[viewRange.endLineNumber - startLineNumber].push(inlineDecoration);
                }
            }
        }
        return {
            decorations: decorationsInViewport,
            inlineDecorations: inlineDecorations
        };
    };
    return ViewModelDecorations;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$15 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ViewModel = /** @class */ (function (_super) {
    __extends$15(ViewModel, _super);
    function ViewModel(editorId, configuration, model, scheduleAtNextAnimationFrame) {
        var _this = _super.call(this) || this;
        _this.editorId = editorId;
        _this.configuration = configuration;
        _this.model = model;
        _this.hasFocus = false;
        _this.viewportStartLine = -1;
        _this.viewportStartLineTrackedRange = null;
        _this.viewportStartLineDelta = 0;
        if (_this.model.isTooLargeForTokenization()) {
            _this.lines = new IdentityLinesCollection(_this.model);
        }
        else {
            var conf = _this.configuration.editor;
            var hardWrappingLineMapperFactory = new CharacterHardWrappingLineMapperFactory(conf.wrappingInfo.wordWrapBreakBeforeCharacters, conf.wrappingInfo.wordWrapBreakAfterCharacters, conf.wrappingInfo.wordWrapBreakObtrusiveCharacters);
            _this.lines = new SplitLinesCollection(_this.model, hardWrappingLineMapperFactory, _this.model.getOptions().tabSize, conf.wrappingInfo.wrappingColumn, conf.fontInfo.typicalFullwidthCharacterWidth / conf.fontInfo.typicalHalfwidthCharacterWidth, conf.wrappingInfo.wrappingIndent);
        }
        _this.coordinatesConverter = _this.lines.createCoordinatesConverter();
        _this.viewLayout = _this._register(new ViewLayout(_this.configuration, _this.getLineCount(), scheduleAtNextAnimationFrame));
        _this._register(_this.viewLayout.onDidScroll(function (e) {
            try {
                var eventsCollector = _this._beginEmit();
                eventsCollector.emit(new ViewScrollChangedEvent(e));
            }
            finally {
                _this._endEmit();
            }
        }));
        _this.decorations = new ViewModelDecorations(_this.editorId, _this.model, _this.configuration, _this.lines, _this.coordinatesConverter);
        _this._registerModelEvents();
        _this._register(_this.configuration.onDidChange(function (e) {
            try {
                var eventsCollector = _this._beginEmit();
                _this._onConfigurationChanged(eventsCollector, e);
            }
            finally {
                _this._endEmit();
            }
        }));
        _this._register(MinimapTokensColorTracker.getInstance().onDidChange(function () {
            try {
                var eventsCollector = _this._beginEmit();
                eventsCollector.emit(new ViewTokensColorsChangedEvent());
            }
            finally {
                _this._endEmit();
            }
        }));
        return _this;
    }
    ViewModel.prototype.dispose = function () {
        // First remove listeners, as disposing the lines might end up sending
        // model decoration changed events ... and we no longer care about them ...
        _super.prototype.dispose.call(this);
        this.decorations.dispose();
        this.lines.dispose();
        this.viewportStartLineTrackedRange = this.model._setTrackedRange(this.viewportStartLineTrackedRange, null, 1 /* NeverGrowsWhenTypingAtEdges */);
    };
    ViewModel.prototype.setHasFocus = function (hasFocus) {
        this.hasFocus = hasFocus;
    };
    ViewModel.prototype._onConfigurationChanged = function (eventsCollector, e) {
        // We might need to restore the current centered view range, so save it (if available)
        var previousViewportStartModelPosition = null;
        if (this.viewportStartLine !== -1) {
            var previousViewportStartViewPosition = new Position(this.viewportStartLine, this.getLineMinColumn(this.viewportStartLine));
            previousViewportStartModelPosition = this.coordinatesConverter.convertViewPositionToModelPosition(previousViewportStartViewPosition);
        }
        var restorePreviousViewportStart = false;
        var conf = this.configuration.editor;
        if (this.lines.setWrappingSettings(conf.wrappingInfo.wrappingIndent, conf.wrappingInfo.wrappingColumn, conf.fontInfo.typicalFullwidthCharacterWidth / conf.fontInfo.typicalHalfwidthCharacterWidth)) {
            eventsCollector.emit(new ViewFlushedEvent());
            eventsCollector.emit(new ViewLineMappingChangedEvent());
            eventsCollector.emit(new ViewDecorationsChangedEvent());
            this.decorations.onLineMappingChanged();
            this.viewLayout.onFlushed(this.getLineCount());
            if (this.viewLayout.getCurrentScrollTop() !== 0) {
                // Never change the scroll position from 0 to something else...
                restorePreviousViewportStart = true;
            }
        }
        if (e.readOnly) {
            // Must read again all decorations due to readOnly filtering
            this.decorations.reset();
            eventsCollector.emit(new ViewDecorationsChangedEvent());
        }
        eventsCollector.emit(new ViewConfigurationChangedEvent(e));
        this.viewLayout.onConfigurationChanged(e);
        if (restorePreviousViewportStart && previousViewportStartModelPosition) {
            var viewPosition = this.coordinatesConverter.convertModelPositionToViewPosition(previousViewportStartModelPosition);
            var viewPositionTop = this.viewLayout.getVerticalOffsetForLineNumber(viewPosition.lineNumber);
            this.viewLayout.setScrollPositionNow({ scrollTop: viewPositionTop + this.viewportStartLineDelta });
        }
    };
    ViewModel.prototype._registerModelEvents = function () {
        var _this = this;
        this._register(this.model.onDidChangeRawContentFast(function (e) {
            try {
                var eventsCollector = _this._beginEmit();
                var hadOtherModelChange = false;
                var hadModelLineChangeThatChangedLineMapping = false;
                var changes = e.changes;
                var versionId = e.versionId;
                for (var j = 0, lenJ = changes.length; j < lenJ; j++) {
                    var change = changes[j];
                    switch (change.changeType) {
                        case 1 /* Flush */: {
                            _this.lines.onModelFlushed();
                            eventsCollector.emit(new ViewFlushedEvent());
                            _this.decorations.reset();
                            _this.viewLayout.onFlushed(_this.getLineCount());
                            hadOtherModelChange = true;
                            break;
                        }
                        case 3 /* LinesDeleted */: {
                            var linesDeletedEvent = _this.lines.onModelLinesDeleted(versionId, change.fromLineNumber, change.toLineNumber);
                            if (linesDeletedEvent !== null) {
                                eventsCollector.emit(linesDeletedEvent);
                                _this.viewLayout.onLinesDeleted(linesDeletedEvent.fromLineNumber, linesDeletedEvent.toLineNumber);
                            }
                            hadOtherModelChange = true;
                            break;
                        }
                        case 4 /* LinesInserted */: {
                            var linesInsertedEvent = _this.lines.onModelLinesInserted(versionId, change.fromLineNumber, change.toLineNumber, change.detail);
                            if (linesInsertedEvent !== null) {
                                eventsCollector.emit(linesInsertedEvent);
                                _this.viewLayout.onLinesInserted(linesInsertedEvent.fromLineNumber, linesInsertedEvent.toLineNumber);
                            }
                            hadOtherModelChange = true;
                            break;
                        }
                        case 2 /* LineChanged */: {
                            var _a = _this.lines.onModelLineChanged(versionId, change.lineNumber, change.detail), lineMappingChanged = _a[0], linesChangedEvent = _a[1], linesInsertedEvent = _a[2], linesDeletedEvent = _a[3];
                            hadModelLineChangeThatChangedLineMapping = lineMappingChanged;
                            if (linesChangedEvent) {
                                eventsCollector.emit(linesChangedEvent);
                            }
                            if (linesInsertedEvent) {
                                eventsCollector.emit(linesInsertedEvent);
                                _this.viewLayout.onLinesInserted(linesInsertedEvent.fromLineNumber, linesInsertedEvent.toLineNumber);
                            }
                            if (linesDeletedEvent) {
                                eventsCollector.emit(linesDeletedEvent);
                                _this.viewLayout.onLinesDeleted(linesDeletedEvent.fromLineNumber, linesDeletedEvent.toLineNumber);
                            }
                            break;
                        }
                        case 5 /* EOLChanged */: {
                            // Nothing to do. The new version will be accepted below
                            break;
                        }
                    }
                }
                _this.lines.acceptVersionId(versionId);
                _this.viewLayout.onHeightMaybeChanged();
                if (!hadOtherModelChange && hadModelLineChangeThatChangedLineMapping) {
                    eventsCollector.emit(new ViewLineMappingChangedEvent());
                    eventsCollector.emit(new ViewDecorationsChangedEvent());
                    _this.decorations.onLineMappingChanged();
                }
            }
            finally {
                _this._endEmit();
            }
            // Update the configuration and reset the centered view line
            _this.viewportStartLine = -1;
            _this.configuration.setMaxLineNumber(_this.model.getLineCount());
            // Recover viewport
            if (!_this.hasFocus && _this.model.getAttachedEditorCount() >= 2 && _this.viewportStartLineTrackedRange) {
                var modelRange = _this.model._getTrackedRange(_this.viewportStartLineTrackedRange);
                if (modelRange) {
                    var viewPosition = _this.coordinatesConverter.convertModelPositionToViewPosition(modelRange.getStartPosition());
                    var viewPositionTop = _this.viewLayout.getVerticalOffsetForLineNumber(viewPosition.lineNumber);
                    _this.viewLayout.setScrollPositionNow({ scrollTop: viewPositionTop + _this.viewportStartLineDelta });
                }
            }
        }));
        this._register(this.model.onDidChangeTokens(function (e) {
            var viewRanges = [];
            for (var j = 0, lenJ = e.ranges.length; j < lenJ; j++) {
                var modelRange = e.ranges[j];
                var viewStartLineNumber = _this.coordinatesConverter.convertModelPositionToViewPosition(new Position(modelRange.fromLineNumber, 1)).lineNumber;
                var viewEndLineNumber = _this.coordinatesConverter.convertModelPositionToViewPosition(new Position(modelRange.toLineNumber, _this.model.getLineMaxColumn(modelRange.toLineNumber))).lineNumber;
                viewRanges[j] = {
                    fromLineNumber: viewStartLineNumber,
                    toLineNumber: viewEndLineNumber
                };
            }
            try {
                var eventsCollector = _this._beginEmit();
                eventsCollector.emit(new ViewTokensChangedEvent(viewRanges));
            }
            finally {
                _this._endEmit();
            }
        }));
        this._register(this.model.onDidChangeLanguageConfiguration(function (e) {
            try {
                var eventsCollector = _this._beginEmit();
                eventsCollector.emit(new ViewLanguageConfigurationEvent());
            }
            finally {
                _this._endEmit();
            }
        }));
        this._register(this.model.onDidChangeOptions(function (e) {
            // A tab size change causes a line mapping changed event => all view parts will repaint OK, no further event needed here
            if (_this.lines.setTabSize(_this.model.getOptions().tabSize)) {
                _this.decorations.onLineMappingChanged();
                _this.viewLayout.onFlushed(_this.getLineCount());
                try {
                    var eventsCollector = _this._beginEmit();
                    eventsCollector.emit(new ViewFlushedEvent());
                    eventsCollector.emit(new ViewLineMappingChangedEvent());
                    eventsCollector.emit(new ViewDecorationsChangedEvent());
                }
                finally {
                    _this._endEmit();
                }
            }
        }));
        this._register(this.model.onDidChangeDecorations(function (e) {
            _this.decorations.onModelDecorationsChanged();
            try {
                var eventsCollector = _this._beginEmit();
                eventsCollector.emit(new ViewDecorationsChangedEvent());
            }
            finally {
                _this._endEmit();
            }
        }));
    };
    ViewModel.prototype.setHiddenAreas = function (ranges) {
        try {
            var eventsCollector = this._beginEmit();
            var lineMappingChanged = this.lines.setHiddenAreas(ranges);
            if (lineMappingChanged) {
                eventsCollector.emit(new ViewFlushedEvent());
                eventsCollector.emit(new ViewLineMappingChangedEvent());
                eventsCollector.emit(new ViewDecorationsChangedEvent());
                this.decorations.onLineMappingChanged();
                this.viewLayout.onFlushed(this.getLineCount());
                this.viewLayout.onHeightMaybeChanged();
            }
        }
        finally {
            this._endEmit();
        }
    };
    ViewModel.prototype.getVisibleRanges = function () {
        var visibleViewRange = this.getCompletelyVisibleViewRange();
        var visibleRange = this.coordinatesConverter.convertViewRangeToModelRange(visibleViewRange);
        var hiddenAreas = this.lines.getHiddenAreas();
        if (hiddenAreas.length === 0) {
            return [visibleRange];
        }
        var result = [], resultLen = 0;
        var startLineNumber = visibleRange.startLineNumber;
        var startColumn = visibleRange.startColumn;
        var endLineNumber = visibleRange.endLineNumber;
        var endColumn = visibleRange.endColumn;
        for (var i = 0, len = hiddenAreas.length; i < len; i++) {
            var hiddenStartLineNumber = hiddenAreas[i].startLineNumber;
            var hiddenEndLineNumber = hiddenAreas[i].endLineNumber;
            if (hiddenEndLineNumber < startLineNumber) {
                continue;
            }
            if (hiddenStartLineNumber > endLineNumber) {
                continue;
            }
            if (startLineNumber < hiddenStartLineNumber) {
                result[resultLen++] = new Range(startLineNumber, startColumn, hiddenStartLineNumber - 1, this.model.getLineMaxColumn(hiddenStartLineNumber - 1));
            }
            startLineNumber = hiddenEndLineNumber + 1;
            startColumn = 1;
        }
        if (startLineNumber < endLineNumber || (startLineNumber === endLineNumber && startColumn < endColumn)) {
            result[resultLen++] = new Range(startLineNumber, startColumn, endLineNumber, endColumn);
        }
        return result;
    };
    ViewModel.prototype.getCompletelyVisibleViewRange = function () {
        var partialData = this.viewLayout.getLinesViewportData();
        var startViewLineNumber = partialData.completelyVisibleStartLineNumber;
        var endViewLineNumber = partialData.completelyVisibleEndLineNumber;
        return new Range(startViewLineNumber, this.getLineMinColumn(startViewLineNumber), endViewLineNumber, this.getLineMaxColumn(endViewLineNumber));
    };
    ViewModel.prototype.getCompletelyVisibleViewRangeAtScrollTop = function (scrollTop) {
        var partialData = this.viewLayout.getLinesViewportDataAtScrollTop(scrollTop);
        var startViewLineNumber = partialData.completelyVisibleStartLineNumber;
        var endViewLineNumber = partialData.completelyVisibleEndLineNumber;
        return new Range(startViewLineNumber, this.getLineMinColumn(startViewLineNumber), endViewLineNumber, this.getLineMaxColumn(endViewLineNumber));
    };
    ViewModel.prototype.saveState = function () {
        var compatViewState = this.viewLayout.saveState();
        var scrollTop = compatViewState.scrollTop;
        var firstViewLineNumber = this.viewLayout.getLineNumberAtVerticalOffset(scrollTop);
        var firstPosition = this.coordinatesConverter.convertViewPositionToModelPosition(new Position(firstViewLineNumber, this.getLineMinColumn(firstViewLineNumber)));
        var firstPositionDeltaTop = this.viewLayout.getVerticalOffsetForLineNumber(firstViewLineNumber) - scrollTop;
        return {
            scrollLeft: compatViewState.scrollLeft,
            firstPosition: firstPosition,
            firstPositionDeltaTop: firstPositionDeltaTop
        };
    };
    ViewModel.prototype.reduceRestoreState = function (state) {
        if (typeof state.firstPosition === 'undefined') {
            // This is a view state serialized by an older version
            return this._reduceRestoreStateCompatibility(state);
        }
        var modelPosition = this.model.validatePosition(state.firstPosition);
        var viewPosition = this.coordinatesConverter.convertModelPositionToViewPosition(modelPosition);
        var scrollTop = this.viewLayout.getVerticalOffsetForLineNumber(viewPosition.lineNumber) - state.firstPositionDeltaTop;
        return {
            scrollLeft: state.scrollLeft,
            scrollTop: scrollTop
        };
    };
    ViewModel.prototype._reduceRestoreStateCompatibility = function (state) {
        return {
            scrollLeft: state.scrollLeft,
            scrollTop: state.scrollTopWithoutViewZones
        };
    };
    ViewModel.prototype.getTabSize = function () {
        return this.model.getOptions().tabSize;
    };
    ViewModel.prototype.getLineCount = function () {
        return this.lines.getViewLineCount();
    };
    /**
     * Gives a hint that a lot of requests are about to come in for these line numbers.
     */
    ViewModel.prototype.setViewport = function (startLineNumber, endLineNumber, centeredLineNumber) {
        this.lines.warmUpLookupCache(startLineNumber, endLineNumber);
        this.viewportStartLine = startLineNumber;
        var position = this.coordinatesConverter.convertViewPositionToModelPosition(new Position(startLineNumber, this.getLineMinColumn(startLineNumber)));
        this.viewportStartLineTrackedRange = this.model._setTrackedRange(this.viewportStartLineTrackedRange, new Range(position.lineNumber, position.column, position.lineNumber, position.column), 1 /* NeverGrowsWhenTypingAtEdges */);
        var viewportStartLineTop = this.viewLayout.getVerticalOffsetForLineNumber(startLineNumber);
        var scrollTop = this.viewLayout.getCurrentScrollTop();
        this.viewportStartLineDelta = scrollTop - viewportStartLineTop;
    };
    ViewModel.prototype.getActiveIndentGuide = function (lineNumber, minLineNumber, maxLineNumber) {
        return this.lines.getActiveIndentGuide(lineNumber, minLineNumber, maxLineNumber);
    };
    ViewModel.prototype.getLinesIndentGuides = function (startLineNumber, endLineNumber) {
        return this.lines.getViewLinesIndentGuides(startLineNumber, endLineNumber);
    };
    ViewModel.prototype.getLineContent = function (lineNumber) {
        return this.lines.getViewLineContent(lineNumber);
    };
    ViewModel.prototype.getLineLength = function (lineNumber) {
        return this.lines.getViewLineLength(lineNumber);
    };
    ViewModel.prototype.getLineMinColumn = function (lineNumber) {
        return this.lines.getViewLineMinColumn(lineNumber);
    };
    ViewModel.prototype.getLineMaxColumn = function (lineNumber) {
        return this.lines.getViewLineMaxColumn(lineNumber);
    };
    ViewModel.prototype.getLineFirstNonWhitespaceColumn = function (lineNumber) {
        var result = firstNonWhitespaceIndex(this.getLineContent(lineNumber));
        if (result === -1) {
            return 0;
        }
        return result + 1;
    };
    ViewModel.prototype.getLineLastNonWhitespaceColumn = function (lineNumber) {
        var result = lastNonWhitespaceIndex(this.getLineContent(lineNumber));
        if (result === -1) {
            return 0;
        }
        return result + 2;
    };
    ViewModel.prototype.getDecorationsInViewport = function (visibleRange) {
        return this.decorations.getDecorationsViewportData(visibleRange).decorations;
    };
    ViewModel.prototype.getViewLineRenderingData = function (visibleRange, lineNumber) {
        var mightContainRTL = this.model.mightContainRTL();
        var mightContainNonBasicASCII = this.model.mightContainNonBasicASCII();
        var tabSize = this.getTabSize();
        var lineData = this.lines.getViewLineData(lineNumber);
        var allInlineDecorations = this.decorations.getDecorationsViewportData(visibleRange).inlineDecorations;
        var inlineDecorations = allInlineDecorations[lineNumber - visibleRange.startLineNumber];
        return new ViewLineRenderingData(lineData.minColumn, lineData.maxColumn, lineData.content, lineData.continuesWithWrappedLine, mightContainRTL, mightContainNonBasicASCII, lineData.tokens, inlineDecorations, tabSize);
    };
    ViewModel.prototype.getViewLineData = function (lineNumber) {
        return this.lines.getViewLineData(lineNumber);
    };
    ViewModel.prototype.getMinimapLinesRenderingData = function (startLineNumber, endLineNumber, needed) {
        var result = this.lines.getViewLinesData(startLineNumber, endLineNumber, needed);
        return new MinimapLinesRenderingData(this.getTabSize(), result);
    };
    ViewModel.prototype.getAllOverviewRulerDecorations = function (theme) {
        return this.lines.getAllOverviewRulerDecorations(this.editorId, this.configuration.editor.readOnly, theme);
    };
    ViewModel.prototype.invalidateOverviewRulerColorCache = function () {
        var decorations = this.model.getOverviewRulerDecorations();
        for (var i = 0, len = decorations.length; i < len; i++) {
            var decoration = decorations[i];
            var opts = decoration.options.overviewRuler;
            if (opts) {
                opts.invalidateCachedColor();
            }
        }
    };
    ViewModel.prototype.getValueInRange = function (range, eol) {
        var modelRange = this.coordinatesConverter.convertViewRangeToModelRange(range);
        return this.model.getValueInRange(modelRange, eol);
    };
    ViewModel.prototype.getModelLineMaxColumn = function (modelLineNumber) {
        return this.model.getLineMaxColumn(modelLineNumber);
    };
    ViewModel.prototype.validateModelPosition = function (position) {
        return this.model.validatePosition(position);
    };
    ViewModel.prototype.validateModelRange = function (range) {
        return this.model.validateRange(range);
    };
    ViewModel.prototype.deduceModelPositionRelativeToViewPosition = function (viewAnchorPosition, deltaOffset, lineFeedCnt) {
        var modelAnchor = this.coordinatesConverter.convertViewPositionToModelPosition(viewAnchorPosition);
        if (this.model.getEOL().length === 2) {
            // This model uses CRLF, so the delta must take that into account
            if (deltaOffset < 0) {
                deltaOffset -= lineFeedCnt;
            }
            else {
                deltaOffset += lineFeedCnt;
            }
        }
        var modelAnchorOffset = this.model.getOffsetAt(modelAnchor);
        var resultOffset = modelAnchorOffset + deltaOffset;
        return this.model.getPositionAt(resultOffset);
    };
    ViewModel.prototype.getEOL = function () {
        return this.model.getEOL();
    };
    ViewModel.prototype.getPlainTextToCopy = function (ranges, emptySelectionClipboard, forceCRLF) {
        var _this = this;
        var newLineCharacter = forceCRLF ? '\r\n' : this.model.getEOL();
        ranges = ranges.slice(0);
        ranges.sort(Range.compareRangesUsingStarts);
        var nonEmptyRanges = ranges.filter(function (r) { return !r.isEmpty(); });
        if (nonEmptyRanges.length === 0) {
            if (!emptySelectionClipboard) {
                return '';
            }
            var modelLineNumbers = ranges.map(function (r) {
                var viewLineStart = new Position(r.startLineNumber, 1);
                return _this.coordinatesConverter.convertViewPositionToModelPosition(viewLineStart).lineNumber;
            });
            var result_1 = '';
            for (var i = 0; i < modelLineNumbers.length; i++) {
                if (i > 0 && modelLineNumbers[i - 1] === modelLineNumbers[i]) {
                    continue;
                }
                result_1 += this.model.getLineContent(modelLineNumbers[i]) + newLineCharacter;
            }
            return result_1;
        }
        var result = [];
        for (var i = 0; i < nonEmptyRanges.length; i++) {
            result.push(this.getValueInRange(nonEmptyRanges[i], forceCRLF ? 2 /* CRLF */ : 0 /* TextDefined */));
        }
        return result.length === 1 ? result[0] : result;
    };
    ViewModel.prototype.getHTMLToCopy = function (viewRanges, emptySelectionClipboard) {
        if (this.model.getLanguageIdentifier().id === 1 /* PlainText */) {
            return null;
        }
        if (viewRanges.length !== 1) {
            // no multiple selection support at this time
            return null;
        }
        var range = this.coordinatesConverter.convertViewRangeToModelRange(viewRanges[0]);
        if (range.isEmpty()) {
            if (!emptySelectionClipboard) {
                // nothing to copy
                return null;
            }
            var lineNumber = range.startLineNumber;
            range = new Range(lineNumber, this.model.getLineMinColumn(lineNumber), lineNumber, this.model.getLineMaxColumn(lineNumber));
        }
        var fontInfo = this.configuration.editor.fontInfo;
        var colorMap = this._getColorMap();
        return ("<div style=\""
            + ("color: " + colorMap[1 /* DefaultForeground */] + ";")
            + ("background-color: " + colorMap[2 /* DefaultBackground */] + ";")
            + ("font-family: " + fontInfo.fontFamily + ";")
            + ("font-weight: " + fontInfo.fontWeight + ";")
            + ("font-size: " + fontInfo.fontSize + "px;")
            + ("line-height: " + fontInfo.lineHeight + "px;")
            + "white-space: pre;"
            + "\">"
            + this._getHTMLToCopy(range, colorMap)
            + '</div>');
    };
    ViewModel.prototype._getHTMLToCopy = function (modelRange, colorMap) {
        var startLineNumber = modelRange.startLineNumber;
        var startColumn = modelRange.startColumn;
        var endLineNumber = modelRange.endLineNumber;
        var endColumn = modelRange.endColumn;
        var tabSize = this.getTabSize();
        var result = '';
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            var lineTokens = this.model.getLineTokens(lineNumber);
            var lineContent = lineTokens.getLineContent();
            var startOffset = (lineNumber === startLineNumber ? startColumn - 1 : 0);
            var endOffset = (lineNumber === endLineNumber ? endColumn - 1 : lineContent.length);
            if (lineContent === '') {
                result += '<br>';
            }
            else {
                result += tokenizeLineToHTML(lineContent, lineTokens.inflate(), colorMap, startOffset, endOffset, tabSize);
            }
        }
        return result;
    };
    ViewModel.prototype._getColorMap = function () {
        var colorMap = TokenizationRegistry.getColorMap();
        var result = ['#000000'];
        if (colorMap) {
            for (var i = 1, len = colorMap.length; i < len; i++) {
                result[i] = Color.Format.CSS.formatHex(colorMap[i]);
            }
        }
        return result;
    };
    return ViewModel;
}(ViewEventEmitter));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ServiceCollection = /** @class */ (function () {
    function ServiceCollection() {
        var entries = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            entries[_i] = arguments[_i];
        }
        this._entries = new Map();
        for (var _a = 0, entries_1 = entries; _a < entries_1.length; _a++) {
            var _b = entries_1[_a], id = _b[0], service = _b[1];
            this.set(id, service);
        }
    }
    ServiceCollection.prototype.set = function (id, instanceOrDescriptor) {
        var result = this._entries.get(id);
        this._entries.set(id, instanceOrDescriptor);
        return result;
    };
    ServiceCollection.prototype.has = function (id) {
        return this._entries.has(id);
    };
    ServiceCollection.prototype.get = function (id) {
        return this._entries.get(id);
    };
    return ServiceCollection;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Severity;
(function (Severity) {
    Severity[Severity["Ignore"] = 0] = "Ignore";
    Severity[Severity["Info"] = 1] = "Info";
    Severity[Severity["Warning"] = 2] = "Warning";
    Severity[Severity["Error"] = 3] = "Error";
})(Severity || (Severity = {}));
(function (Severity) {
    var _error = 'error';
    var _warning = 'warning';
    var _warn = 'warn';
    var _info = 'info';
    var _displayStrings = Object.create(null);
    _displayStrings[Severity.Error] = localize('sev.error', "Error");
    _displayStrings[Severity.Warning] = localize('sev.warning', "Warning");
    _displayStrings[Severity.Info] = localize('sev.info', "Info");
    /**
     * Parses 'error', 'warning', 'warn', 'info' in call casings
     * and falls back to ignore.
     */
    function fromValue(value) {
        if (!value) {
            return Severity.Ignore;
        }
        if (equalsIgnoreCase(_error, value)) {
            return Severity.Error;
        }
        if (equalsIgnoreCase(_warning, value) || equalsIgnoreCase(_warn, value)) {
            return Severity.Warning;
        }
        if (equalsIgnoreCase(_info, value)) {
            return Severity.Info;
        }
        return Severity.Ignore;
    }
    Severity.fromValue = fromValue;
})(Severity || (Severity = {}));
var Severity$1 = Severity;

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var INotificationService = createDecorator('notificationService');
var NoOpNotification = /** @class */ (function () {
    function NoOpNotification() {
    }
    return NoOpNotification;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$16 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$2 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$1 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var EDITOR_ID = 0;
var SHOW_UNUSED_ENABLED_CLASS = 'showUnused';
var ModelData = /** @class */ (function () {
    function ModelData(model, viewModel, cursor, view, hasRealView, listenersToRemove) {
        this.model = model;
        this.viewModel = viewModel;
        this.cursor = cursor;
        this.view = view;
        this.hasRealView = hasRealView;
        this.listenersToRemove = listenersToRemove;
    }
    ModelData.prototype.dispose = function () {
        dispose(this.listenersToRemove);
        this.model.onBeforeDetached();
        if (this.hasRealView) {
            this.view.dispose();
        }
        this.cursor.dispose();
        this.viewModel.dispose();
    };
    return ModelData;
}());
var CodeEditorWidget = /** @class */ (function (_super) {
    __extends$16(CodeEditorWidget, _super);
    function CodeEditorWidget(domElement, options, codeEditorWidgetOptions, instantiationService, codeEditorService, commandService, contextKeyService, themeService, notificationService) {
        var _this = _super.call(this) || this;
        //#region Eventing
        _this._onDidDispose = _this._register(new Emitter());
        _this.onDidDispose = _this._onDidDispose.event;
        _this._onDidChangeModelContent = _this._register(new Emitter());
        _this.onDidChangeModelContent = _this._onDidChangeModelContent.event;
        _this._onDidChangeModelLanguage = _this._register(new Emitter());
        _this.onDidChangeModelLanguage = _this._onDidChangeModelLanguage.event;
        _this._onDidChangeModelLanguageConfiguration = _this._register(new Emitter());
        _this.onDidChangeModelLanguageConfiguration = _this._onDidChangeModelLanguageConfiguration.event;
        _this._onDidChangeModelOptions = _this._register(new Emitter());
        _this.onDidChangeModelOptions = _this._onDidChangeModelOptions.event;
        _this._onDidChangeModelDecorations = _this._register(new Emitter());
        _this.onDidChangeModelDecorations = _this._onDidChangeModelDecorations.event;
        _this._onDidChangeConfiguration = _this._register(new Emitter());
        _this.onDidChangeConfiguration = _this._onDidChangeConfiguration.event;
        _this._onDidChangeModel = _this._register(new Emitter());
        _this.onDidChangeModel = _this._onDidChangeModel.event;
        _this._onDidChangeCursorPosition = _this._register(new Emitter());
        _this.onDidChangeCursorPosition = _this._onDidChangeCursorPosition.event;
        _this._onDidChangeCursorSelection = _this._register(new Emitter());
        _this.onDidChangeCursorSelection = _this._onDidChangeCursorSelection.event;
        _this._onDidAttemptReadOnlyEdit = _this._register(new Emitter());
        _this.onDidAttemptReadOnlyEdit = _this._onDidAttemptReadOnlyEdit.event;
        _this._onDidLayoutChange = _this._register(new Emitter());
        _this.onDidLayoutChange = _this._onDidLayoutChange.event;
        _this._editorTextFocus = _this._register(new BooleanEventEmitter());
        _this.onDidFocusEditorText = _this._editorTextFocus.onDidChangeToTrue;
        _this.onDidBlurEditorText = _this._editorTextFocus.onDidChangeToFalse;
        _this._editorWidgetFocus = _this._register(new BooleanEventEmitter());
        _this.onDidFocusEditorWidget = _this._editorWidgetFocus.onDidChangeToTrue;
        _this.onDidBlurEditorWidget = _this._editorWidgetFocus.onDidChangeToFalse;
        _this._onWillType = _this._register(new Emitter());
        _this.onWillType = _this._onWillType.event;
        _this._onDidType = _this._register(new Emitter());
        _this.onDidType = _this._onDidType.event;
        _this._onCompositionStart = _this._register(new Emitter());
        _this.onCompositionStart = _this._onCompositionStart.event;
        _this._onCompositionEnd = _this._register(new Emitter());
        _this.onCompositionEnd = _this._onCompositionEnd.event;
        _this._onDidPaste = _this._register(new Emitter());
        _this.onDidPaste = _this._onDidPaste.event;
        _this._onMouseUp = _this._register(new Emitter());
        _this.onMouseUp = _this._onMouseUp.event;
        _this._onMouseDown = _this._register(new Emitter());
        _this.onMouseDown = _this._onMouseDown.event;
        _this._onMouseDrag = _this._register(new Emitter());
        _this.onMouseDrag = _this._onMouseDrag.event;
        _this._onMouseDrop = _this._register(new Emitter());
        _this.onMouseDrop = _this._onMouseDrop.event;
        _this._onContextMenu = _this._register(new Emitter());
        _this.onContextMenu = _this._onContextMenu.event;
        _this._onMouseMove = _this._register(new Emitter());
        _this.onMouseMove = _this._onMouseMove.event;
        _this._onMouseLeave = _this._register(new Emitter());
        _this.onMouseLeave = _this._onMouseLeave.event;
        _this._onKeyUp = _this._register(new Emitter());
        _this.onKeyUp = _this._onKeyUp.event;
        _this._onKeyDown = _this._register(new Emitter());
        _this.onKeyDown = _this._onKeyDown.event;
        _this._onDidScrollChange = _this._register(new Emitter());
        _this.onDidScrollChange = _this._onDidScrollChange.event;
        _this._onDidChangeViewZones = _this._register(new Emitter());
        _this.onDidChangeViewZones = _this._onDidChangeViewZones.event;
        _this._domElement = domElement;
        _this._id = (++EDITOR_ID);
        _this._decorationTypeKeysToIds = {};
        _this._decorationTypeSubtypes = {};
        _this.isSimpleWidget = codeEditorWidgetOptions.isSimpleWidget || false;
        _this._telemetryData = codeEditorWidgetOptions.telemetryData || null;
        options = options || {};
        _this._configuration = _this._register(_this._createConfiguration(options));
        _this._register(_this._configuration.onDidChange(function (e) {
            _this._onDidChangeConfiguration.fire(e);
            if (e.layoutInfo) {
                _this._onDidLayoutChange.fire(_this._configuration.editor.layoutInfo);
            }
            if (_this._configuration.editor.showUnused) {
                _this._domElement.classList.add(SHOW_UNUSED_ENABLED_CLASS);
            }
            else {
                _this._domElement.classList.remove(SHOW_UNUSED_ENABLED_CLASS);
            }
        }));
        _this._contextKeyService = _this._register(contextKeyService.createScoped(_this._domElement));
        _this._notificationService = notificationService;
        _this._codeEditorService = codeEditorService;
        _this._commandService = commandService;
        _this._themeService = themeService;
        _this._register(new EditorContextKeysManager(_this, _this._contextKeyService));
        _this._register(new EditorModeContext(_this, _this._contextKeyService));
        _this._instantiationService = instantiationService.createChild(new ServiceCollection([IContextKeyService, _this._contextKeyService]));
        _this._attachModel(null);
        _this._contributions = {};
        _this._actions = {};
        _this._focusTracker = new CodeEditorWidgetFocusTracker(domElement);
        _this._focusTracker.onChange(function () {
            _this._editorWidgetFocus.setValue(_this._focusTracker.hasFocus());
        });
        _this._contentWidgets = {};
        _this._overlayWidgets = {};
        var contributions;
        if (Array.isArray(codeEditorWidgetOptions.contributions)) {
            contributions = codeEditorWidgetOptions.contributions;
        }
        else {
            contributions = EditorExtensionsRegistry.getEditorContributions();
        }
        for (var i = 0, len = contributions.length; i < len; i++) {
            var ctor = contributions[i];
            try {
                var contribution = _this._instantiationService.createInstance(ctor, _this);
                _this._contributions[contribution.getId()] = contribution;
            }
            catch (err) {
                onUnexpectedError(err);
            }
        }
        EditorExtensionsRegistry.getEditorActions().forEach(function (action) {
            var internalAction = new InternalEditorAction(action.id, action.label, action.alias, action.precondition, function () {
                return _this._instantiationService.invokeFunction(function (accessor) {
                    return Promise.resolve(action.runEditorCommand(accessor, _this, null));
                });
            }, _this._contextKeyService);
            _this._actions[internalAction.id] = internalAction;
        });
        _this._codeEditorService.addCodeEditor(_this);
        return _this;
    }
    CodeEditorWidget.prototype._createConfiguration = function (options) {
        return new Configuration(options, this._domElement);
    };
    CodeEditorWidget.prototype.getId = function () {
        return this.getEditorType() + ':' + this._id;
    };
    CodeEditorWidget.prototype.getEditorType = function () {
        return EditorType.ICodeEditor;
    };
    CodeEditorWidget.prototype.dispose = function () {
        this._codeEditorService.removeCodeEditor(this);
        this._focusTracker.dispose();
        var keys = Object.keys(this._contributions);
        for (var i = 0, len = keys.length; i < len; i++) {
            var contributionId = keys[i];
            this._contributions[contributionId].dispose();
        }
        this._removeDecorationTypes();
        this._postDetachModelCleanup(this._detachModel());
        this._onDidDispose.fire();
        _super.prototype.dispose.call(this);
    };
    CodeEditorWidget.prototype.invokeWithinContext = function (fn) {
        return this._instantiationService.invokeFunction(fn);
    };
    CodeEditorWidget.prototype.updateOptions = function (newOptions) {
        this._configuration.updateOptions(newOptions);
    };
    CodeEditorWidget.prototype.getConfiguration = function () {
        return this._configuration.editor;
    };
    CodeEditorWidget.prototype.getRawConfiguration = function () {
        return this._configuration.getRawOptions();
    };
    CodeEditorWidget.prototype.getValue = function (options) {
        if (options === void 0) { options = null; }
        if (!this._modelData) {
            return '';
        }
        var preserveBOM = (options && options.preserveBOM) ? true : false;
        var eolPreference = 0 /* TextDefined */;
        if (options && options.lineEnding && options.lineEnding === '\n') {
            eolPreference = 1 /* LF */;
        }
        else if (options && options.lineEnding && options.lineEnding === '\r\n') {
            eolPreference = 2 /* CRLF */;
        }
        return this._modelData.model.getValue(eolPreference, preserveBOM);
    };
    CodeEditorWidget.prototype.setValue = function (newValue) {
        if (!this._modelData) {
            return;
        }
        this._modelData.model.setValue(newValue);
    };
    CodeEditorWidget.prototype.getModel = function () {
        if (!this._modelData) {
            return null;
        }
        return this._modelData.model;
    };
    CodeEditorWidget.prototype.setModel = function (model) {
        if (model === void 0) { model = null; }
        if (this._modelData === null && model === null) {
            // Current model is the new model
            return;
        }
        if (this._modelData && this._modelData.model === model) {
            // Current model is the new model
            return;
        }
        var detachedModel = this._detachModel();
        this._attachModel(model);
        var e = {
            oldModelUrl: detachedModel ? detachedModel.uri : null,
            newModelUrl: model ? model.uri : null
        };
        this._removeDecorationTypes();
        this._onDidChangeModel.fire(e);
        this._postDetachModelCleanup(detachedModel);
    };
    CodeEditorWidget.prototype._removeDecorationTypes = function () {
        this._decorationTypeKeysToIds = {};
        if (this._decorationTypeSubtypes) {
            for (var decorationType in this._decorationTypeSubtypes) {
                var subTypes = this._decorationTypeSubtypes[decorationType];
                for (var subType in subTypes) {
                    this._removeDecorationType(decorationType + '-' + subType);
                }
            }
            this._decorationTypeSubtypes = {};
        }
    };
    CodeEditorWidget.prototype.getVisibleRanges = function () {
        if (!this._modelData) {
            return [];
        }
        return this._modelData.viewModel.getVisibleRanges();
    };
    CodeEditorWidget.prototype.getWhitespaces = function () {
        if (!this._modelData) {
            return [];
        }
        return this._modelData.viewModel.viewLayout.getWhitespaces();
    };
    CodeEditorWidget._getVerticalOffsetForPosition = function (modelData, modelLineNumber, modelColumn) {
        var modelPosition = modelData.model.validatePosition({
            lineNumber: modelLineNumber,
            column: modelColumn
        });
        var viewPosition = modelData.viewModel.coordinatesConverter.convertModelPositionToViewPosition(modelPosition);
        return modelData.viewModel.viewLayout.getVerticalOffsetForLineNumber(viewPosition.lineNumber);
    };
    CodeEditorWidget.prototype.getTopForLineNumber = function (lineNumber) {
        if (!this._modelData) {
            return -1;
        }
        return CodeEditorWidget._getVerticalOffsetForPosition(this._modelData, lineNumber, 1);
    };
    CodeEditorWidget.prototype.getTopForPosition = function (lineNumber, column) {
        if (!this._modelData) {
            return -1;
        }
        return CodeEditorWidget._getVerticalOffsetForPosition(this._modelData, lineNumber, column);
    };
    CodeEditorWidget.prototype.setHiddenAreas = function (ranges) {
        if (this._modelData) {
            this._modelData.viewModel.setHiddenAreas(ranges.map(function (r) { return Range.lift(r); }));
        }
    };
    CodeEditorWidget.prototype.getVisibleColumnFromPosition = function (rawPosition) {
        if (!this._modelData) {
            return rawPosition.column;
        }
        var position = this._modelData.model.validatePosition(rawPosition);
        var tabSize = this._modelData.model.getOptions().tabSize;
        return CursorColumns.visibleColumnFromColumn(this._modelData.model.getLineContent(position.lineNumber), position.column, tabSize) + 1;
    };
    CodeEditorWidget.prototype.getPosition = function () {
        if (!this._modelData) {
            return null;
        }
        return this._modelData.cursor.getPosition();
    };
    CodeEditorWidget.prototype.setPosition = function (position) {
        if (!this._modelData) {
            return;
        }
        if (!Position.isIPosition(position)) {
            throw new Error('Invalid arguments');
        }
        this._modelData.cursor.setSelections('api', [{
                selectionStartLineNumber: position.lineNumber,
                selectionStartColumn: position.column,
                positionLineNumber: position.lineNumber,
                positionColumn: position.column
            }]);
    };
    CodeEditorWidget.prototype._sendRevealRange = function (modelRange, verticalType, revealHorizontal, scrollType) {
        if (!this._modelData) {
            return;
        }
        if (!Range.isIRange(modelRange)) {
            throw new Error('Invalid arguments');
        }
        var validatedModelRange = this._modelData.model.validateRange(modelRange);
        var viewRange = this._modelData.viewModel.coordinatesConverter.convertModelRangeToViewRange(validatedModelRange);
        this._modelData.cursor.emitCursorRevealRange(viewRange, verticalType, revealHorizontal, scrollType);
    };
    CodeEditorWidget.prototype.revealLine = function (lineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealLine(lineNumber, 0 /* Simple */, scrollType);
    };
    CodeEditorWidget.prototype.revealLineInCenter = function (lineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealLine(lineNumber, 1 /* Center */, scrollType);
    };
    CodeEditorWidget.prototype.revealLineInCenterIfOutsideViewport = function (lineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealLine(lineNumber, 2 /* CenterIfOutsideViewport */, scrollType);
    };
    CodeEditorWidget.prototype._revealLine = function (lineNumber, revealType, scrollType) {
        if (typeof lineNumber !== 'number') {
            throw new Error('Invalid arguments');
        }
        this._sendRevealRange(new Range(lineNumber, 1, lineNumber, 1), revealType, false, scrollType);
    };
    CodeEditorWidget.prototype.revealPosition = function (position, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealPosition(position, 0 /* Simple */, true, scrollType);
    };
    CodeEditorWidget.prototype.revealPositionInCenter = function (position, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealPosition(position, 1 /* Center */, true, scrollType);
    };
    CodeEditorWidget.prototype.revealPositionInCenterIfOutsideViewport = function (position, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealPosition(position, 2 /* CenterIfOutsideViewport */, true, scrollType);
    };
    CodeEditorWidget.prototype._revealPosition = function (position, verticalType, revealHorizontal, scrollType) {
        if (!Position.isIPosition(position)) {
            throw new Error('Invalid arguments');
        }
        this._sendRevealRange(new Range(position.lineNumber, position.column, position.lineNumber, position.column), verticalType, revealHorizontal, scrollType);
    };
    CodeEditorWidget.prototype.getSelection = function () {
        if (!this._modelData) {
            return null;
        }
        return this._modelData.cursor.getSelection();
    };
    CodeEditorWidget.prototype.getSelections = function () {
        if (!this._modelData) {
            return null;
        }
        return this._modelData.cursor.getSelections();
    };
    CodeEditorWidget.prototype.setSelection = function (something) {
        var isSelection = Selection.isISelection(something);
        var isRange = Range.isIRange(something);
        if (!isSelection && !isRange) {
            throw new Error('Invalid arguments');
        }
        if (isSelection) {
            this._setSelectionImpl(something);
        }
        else if (isRange) {
            // act as if it was an IRange
            var selection = {
                selectionStartLineNumber: something.startLineNumber,
                selectionStartColumn: something.startColumn,
                positionLineNumber: something.endLineNumber,
                positionColumn: something.endColumn
            };
            this._setSelectionImpl(selection);
        }
    };
    CodeEditorWidget.prototype._setSelectionImpl = function (sel) {
        if (!this._modelData) {
            return;
        }
        var selection = new Selection(sel.selectionStartLineNumber, sel.selectionStartColumn, sel.positionLineNumber, sel.positionColumn);
        this._modelData.cursor.setSelections('api', [selection]);
    };
    CodeEditorWidget.prototype.revealLines = function (startLineNumber, endLineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealLines(startLineNumber, endLineNumber, 0 /* Simple */, scrollType);
    };
    CodeEditorWidget.prototype.revealLinesInCenter = function (startLineNumber, endLineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealLines(startLineNumber, endLineNumber, 1 /* Center */, scrollType);
    };
    CodeEditorWidget.prototype.revealLinesInCenterIfOutsideViewport = function (startLineNumber, endLineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealLines(startLineNumber, endLineNumber, 2 /* CenterIfOutsideViewport */, scrollType);
    };
    CodeEditorWidget.prototype._revealLines = function (startLineNumber, endLineNumber, verticalType, scrollType) {
        if (typeof startLineNumber !== 'number' || typeof endLineNumber !== 'number') {
            throw new Error('Invalid arguments');
        }
        this._sendRevealRange(new Range(startLineNumber, 1, endLineNumber, 1), verticalType, false, scrollType);
    };
    CodeEditorWidget.prototype.revealRange = function (range, scrollType, revealVerticalInCenter, revealHorizontal) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
        if (revealHorizontal === void 0) { revealHorizontal = true; }
        this._revealRange(range, revealVerticalInCenter ? 1 /* Center */ : 0 /* Simple */, revealHorizontal, scrollType);
    };
    CodeEditorWidget.prototype.revealRangeInCenter = function (range, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealRange(range, 1 /* Center */, true, scrollType);
    };
    CodeEditorWidget.prototype.revealRangeInCenterIfOutsideViewport = function (range, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealRange(range, 2 /* CenterIfOutsideViewport */, true, scrollType);
    };
    CodeEditorWidget.prototype.revealRangeAtTop = function (range, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._revealRange(range, 3 /* Top */, true, scrollType);
    };
    CodeEditorWidget.prototype._revealRange = function (range, verticalType, revealHorizontal, scrollType) {
        if (!Range.isIRange(range)) {
            throw new Error('Invalid arguments');
        }
        this._sendRevealRange(Range.lift(range), verticalType, revealHorizontal, scrollType);
    };
    CodeEditorWidget.prototype.setSelections = function (ranges, source) {
        if (source === void 0) { source = 'api'; }
        if (!this._modelData) {
            return;
        }
        if (!ranges || ranges.length === 0) {
            throw new Error('Invalid arguments');
        }
        for (var i = 0, len = ranges.length; i < len; i++) {
            if (!Selection.isISelection(ranges[i])) {
                throw new Error('Invalid arguments');
            }
        }
        this._modelData.cursor.setSelections(source, ranges);
    };
    CodeEditorWidget.prototype.getScrollWidth = function () {
        if (!this._modelData) {
            return -1;
        }
        return this._modelData.viewModel.viewLayout.getScrollWidth();
    };
    CodeEditorWidget.prototype.getScrollLeft = function () {
        if (!this._modelData) {
            return -1;
        }
        return this._modelData.viewModel.viewLayout.getCurrentScrollLeft();
    };
    CodeEditorWidget.prototype.getScrollHeight = function () {
        if (!this._modelData) {
            return -1;
        }
        return this._modelData.viewModel.viewLayout.getScrollHeight();
    };
    CodeEditorWidget.prototype.getScrollTop = function () {
        if (!this._modelData) {
            return -1;
        }
        return this._modelData.viewModel.viewLayout.getCurrentScrollTop();
    };
    CodeEditorWidget.prototype.setScrollLeft = function (newScrollLeft) {
        if (!this._modelData) {
            return;
        }
        if (typeof newScrollLeft !== 'number') {
            throw new Error('Invalid arguments');
        }
        this._modelData.viewModel.viewLayout.setScrollPositionNow({
            scrollLeft: newScrollLeft
        });
    };
    CodeEditorWidget.prototype.setScrollTop = function (newScrollTop) {
        if (!this._modelData) {
            return;
        }
        if (typeof newScrollTop !== 'number') {
            throw new Error('Invalid arguments');
        }
        this._modelData.viewModel.viewLayout.setScrollPositionNow({
            scrollTop: newScrollTop
        });
    };
    CodeEditorWidget.prototype.setScrollPosition = function (position) {
        if (!this._modelData) {
            return;
        }
        this._modelData.viewModel.viewLayout.setScrollPositionNow(position);
    };
    CodeEditorWidget.prototype.saveViewState = function () {
        if (!this._modelData) {
            return null;
        }
        var contributionsState = {};
        var keys = Object.keys(this._contributions);
        for (var i = 0, len = keys.length; i < len; i++) {
            var id = keys[i];
            var contribution = this._contributions[id];
            if (typeof contribution.saveViewState === 'function') {
                contributionsState[id] = contribution.saveViewState();
            }
        }
        var cursorState = this._modelData.cursor.saveState();
        var viewState = this._modelData.viewModel.saveState();
        return {
            cursorState: cursorState,
            viewState: viewState,
            contributionsState: contributionsState
        };
    };
    CodeEditorWidget.prototype.restoreViewState = function (s) {
        if (!this._modelData || !this._modelData.hasRealView) {
            return;
        }
        if (s && s.cursorState && s.viewState) {
            var codeEditorState = s;
            var cursorState = codeEditorState.cursorState;
            if (Array.isArray(cursorState)) {
                this._modelData.cursor.restoreState(cursorState);
            }
            else {
                // Backwards compatibility
                this._modelData.cursor.restoreState([cursorState]);
            }
            var contributionsState = s.contributionsState || {};
            var keys = Object.keys(this._contributions);
            for (var i = 0, len = keys.length; i < len; i++) {
                var id = keys[i];
                var contribution = this._contributions[id];
                if (typeof contribution.restoreViewState === 'function') {
                    contribution.restoreViewState(contributionsState[id]);
                }
            }
            var reducedState = this._modelData.viewModel.reduceRestoreState(s.viewState);
            var linesViewportData = this._modelData.viewModel.viewLayout.getLinesViewportDataAtScrollTop(reducedState.scrollTop);
            var startPosition = this._modelData.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new Position(linesViewportData.startLineNumber, 1));
            var endPosition = this._modelData.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new Position(linesViewportData.endLineNumber, 1));
            this._modelData.model.tokenizeViewport(startPosition.lineNumber, endPosition.lineNumber);
            this._modelData.view.restoreState(reducedState);
        }
    };
    CodeEditorWidget.prototype.getContribution = function (id) {
        return (this._contributions[id] || null);
    };
    CodeEditorWidget.prototype.getActions = function () {
        var result = [];
        var keys = Object.keys(this._actions);
        for (var i = 0, len = keys.length; i < len; i++) {
            var id = keys[i];
            result.push(this._actions[id]);
        }
        return result;
    };
    CodeEditorWidget.prototype.getSupportedActions = function () {
        var result = this.getActions();
        result = result.filter(function (action) { return action.isSupported(); });
        return result;
    };
    CodeEditorWidget.prototype.getAction = function (id) {
        return this._actions[id] || null;
    };
    CodeEditorWidget.prototype.trigger = function (source, handlerId, payload) {
        payload = payload || {};
        // Special case for typing
        if (handlerId === Handler.Type) {
            if (!this._modelData || typeof payload.text !== 'string' || payload.text.length === 0) {
                // nothing to do
                return;
            }
            if (source === 'keyboard') {
                this._onWillType.fire(payload.text);
            }
            this._modelData.cursor.trigger(source, handlerId, payload);
            if (source === 'keyboard') {
                this._onDidType.fire(payload.text);
            }
            return;
        }
        // Special case for pasting
        if (handlerId === Handler.Paste) {
            if (!this._modelData || typeof payload.text !== 'string' || payload.text.length === 0) {
                // nothing to do
                return;
            }
            var startPosition = this._modelData.cursor.getSelection().getStartPosition();
            this._modelData.cursor.trigger(source, handlerId, payload);
            var endPosition = this._modelData.cursor.getSelection().getStartPosition();
            if (source === 'keyboard') {
                this._onDidPaste.fire(new Range(startPosition.lineNumber, startPosition.column, endPosition.lineNumber, endPosition.column));
            }
            return;
        }
        if (handlerId === Handler.CompositionStart) {
            this._onCompositionStart.fire();
        }
        if (handlerId === Handler.CompositionEnd) {
            this._onCompositionEnd.fire();
        }
        var action = this.getAction(handlerId);
        if (action) {
            Promise.resolve(action.run()).then(null, onUnexpectedError);
            return;
        }
        if (!this._modelData) {
            return;
        }
        if (this._triggerEditorCommand(source, handlerId, payload)) {
            return;
        }
        this._modelData.cursor.trigger(source, handlerId, payload);
    };
    CodeEditorWidget.prototype._triggerEditorCommand = function (source, handlerId, payload) {
        var _this = this;
        var command = EditorExtensionsRegistry.getEditorCommand(handlerId);
        if (command) {
            payload = payload || {};
            payload.source = source;
            this._instantiationService.invokeFunction(function (accessor) {
                Promise.resolve(command.runEditorCommand(accessor, _this, payload)).then(null, onUnexpectedError);
            });
            return true;
        }
        return false;
    };
    CodeEditorWidget.prototype._getCursors = function () {
        if (!this._modelData) {
            return null;
        }
        return this._modelData.cursor;
    };
    CodeEditorWidget.prototype.pushUndoStop = function () {
        if (!this._modelData) {
            return false;
        }
        if (this._configuration.editor.readOnly) {
            // read only editor => sorry!
            return false;
        }
        this._modelData.model.pushStackElement();
        return true;
    };
    CodeEditorWidget.prototype.executeEdits = function (source, edits, endCursorState) {
        if (!this._modelData) {
            return false;
        }
        if (this._configuration.editor.readOnly) {
            // read only editor => sorry!
            return false;
        }
        this._modelData.model.pushEditOperations(this._modelData.cursor.getSelections(), edits, function () {
            return endCursorState ? endCursorState : null;
        });
        if (endCursorState) {
            this._modelData.cursor.setSelections(source, endCursorState);
        }
        return true;
    };
    CodeEditorWidget.prototype.executeCommand = function (source, command) {
        if (!this._modelData) {
            return;
        }
        this._modelData.cursor.trigger(source, Handler.ExecuteCommand, command);
    };
    CodeEditorWidget.prototype.executeCommands = function (source, commands) {
        if (!this._modelData) {
            return;
        }
        this._modelData.cursor.trigger(source, Handler.ExecuteCommands, commands);
    };
    CodeEditorWidget.prototype.changeDecorations = function (callback) {
        if (!this._modelData) {
            // callback will not be called
            return null;
        }
        return this._modelData.model.changeDecorations(callback, this._id);
    };
    CodeEditorWidget.prototype.getLineDecorations = function (lineNumber) {
        if (!this._modelData) {
            return null;
        }
        return this._modelData.model.getLineDecorations(lineNumber, this._id, this._configuration.editor.readOnly);
    };
    CodeEditorWidget.prototype.deltaDecorations = function (oldDecorations, newDecorations) {
        if (!this._modelData) {
            return [];
        }
        if (oldDecorations.length === 0 && newDecorations.length === 0) {
            return oldDecorations;
        }
        return this._modelData.model.deltaDecorations(oldDecorations, newDecorations, this._id);
    };
    CodeEditorWidget.prototype.removeDecorations = function (decorationTypeKey) {
        // remove decorations for type and sub type
        var oldDecorationsIds = this._decorationTypeKeysToIds[decorationTypeKey];
        if (oldDecorationsIds) {
            this.deltaDecorations(oldDecorationsIds, []);
        }
        if (this._decorationTypeKeysToIds.hasOwnProperty(decorationTypeKey)) {
            delete this._decorationTypeKeysToIds[decorationTypeKey];
        }
        if (this._decorationTypeSubtypes.hasOwnProperty(decorationTypeKey)) {
            delete this._decorationTypeSubtypes[decorationTypeKey];
        }
    };
    CodeEditorWidget.prototype.getLayoutInfo = function () {
        return this._configuration.editor.layoutInfo;
    };
    CodeEditorWidget.prototype.createOverviewRuler = function (cssClassName) {
        if (!this._modelData || !this._modelData.hasRealView) {
            return null;
        }
        return this._modelData.view.createOverviewRuler(cssClassName);
    };
    CodeEditorWidget.prototype.getDomNode = function () {
        if (!this._modelData || !this._modelData.hasRealView) {
            return null;
        }
        return this._modelData.view.domNode.domNode;
    };
    CodeEditorWidget.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
        if (!this._modelData || !this._modelData.hasRealView) {
            return;
        }
        this._modelData.view.delegateVerticalScrollbarMouseDown(browserEvent);
    };
    CodeEditorWidget.prototype.layout = function (dimension) {
        this._configuration.observeReferenceElement(dimension);
        this.render();
    };
    CodeEditorWidget.prototype.focus = function () {
        if (!this._modelData || !this._modelData.hasRealView) {
            return;
        }
        this._modelData.view.focus();
    };
    CodeEditorWidget.prototype.hasTextFocus = function () {
        if (!this._modelData || !this._modelData.hasRealView) {
            return false;
        }
        return this._modelData.view.isFocused();
    };
    CodeEditorWidget.prototype.hasWidgetFocus = function () {
        return this._focusTracker && this._focusTracker.hasFocus();
    };
    CodeEditorWidget.prototype.addContentWidget = function (widget) {
        var widgetData = {
            widget: widget,
            position: widget.getPosition()
        };
        if (this._contentWidgets.hasOwnProperty(widget.getId())) {
            console.warn('Overwriting a content widget with the same id.');
        }
        this._contentWidgets[widget.getId()] = widgetData;
        if (this._modelData && this._modelData.hasRealView) {
            this._modelData.view.addContentWidget(widgetData);
        }
    };
    CodeEditorWidget.prototype.layoutContentWidget = function (widget) {
        var widgetId = widget.getId();
        if (this._contentWidgets.hasOwnProperty(widgetId)) {
            var widgetData = this._contentWidgets[widgetId];
            widgetData.position = widget.getPosition();
            if (this._modelData && this._modelData.hasRealView) {
                this._modelData.view.layoutContentWidget(widgetData);
            }
        }
    };
    CodeEditorWidget.prototype.removeContentWidget = function (widget) {
        var widgetId = widget.getId();
        if (this._contentWidgets.hasOwnProperty(widgetId)) {
            var widgetData = this._contentWidgets[widgetId];
            delete this._contentWidgets[widgetId];
            if (this._modelData && this._modelData.hasRealView) {
                this._modelData.view.removeContentWidget(widgetData);
            }
        }
    };
    CodeEditorWidget.prototype.addOverlayWidget = function (widget) {
        var widgetData = {
            widget: widget,
            position: widget.getPosition()
        };
        if (this._overlayWidgets.hasOwnProperty(widget.getId())) {
            console.warn('Overwriting an overlay widget with the same id.');
        }
        this._overlayWidgets[widget.getId()] = widgetData;
        if (this._modelData && this._modelData.hasRealView) {
            this._modelData.view.addOverlayWidget(widgetData);
        }
    };
    CodeEditorWidget.prototype.layoutOverlayWidget = function (widget) {
        var widgetId = widget.getId();
        if (this._overlayWidgets.hasOwnProperty(widgetId)) {
            var widgetData = this._overlayWidgets[widgetId];
            widgetData.position = widget.getPosition();
            if (this._modelData && this._modelData.hasRealView) {
                this._modelData.view.layoutOverlayWidget(widgetData);
            }
        }
    };
    CodeEditorWidget.prototype.removeOverlayWidget = function (widget) {
        var widgetId = widget.getId();
        if (this._overlayWidgets.hasOwnProperty(widgetId)) {
            var widgetData = this._overlayWidgets[widgetId];
            delete this._overlayWidgets[widgetId];
            if (this._modelData && this._modelData.hasRealView) {
                this._modelData.view.removeOverlayWidget(widgetData);
            }
        }
    };
    CodeEditorWidget.prototype.changeViewZones = function (callback) {
        if (!this._modelData || !this._modelData.hasRealView) {
            return;
        }
        var hasChanges = this._modelData.view.change(callback);
        if (hasChanges) {
            this._onDidChangeViewZones.fire();
        }
    };
    CodeEditorWidget.prototype.getTargetAtClientPoint = function (clientX, clientY) {
        if (!this._modelData || !this._modelData.hasRealView) {
            return null;
        }
        return this._modelData.view.getTargetAtClientPoint(clientX, clientY);
    };
    CodeEditorWidget.prototype.getScrolledVisiblePosition = function (rawPosition) {
        if (!this._modelData || !this._modelData.hasRealView) {
            return null;
        }
        var position = this._modelData.model.validatePosition(rawPosition);
        var layoutInfo = this._configuration.editor.layoutInfo;
        var top = CodeEditorWidget._getVerticalOffsetForPosition(this._modelData, position.lineNumber, position.column) - this.getScrollTop();
        var left = this._modelData.view.getOffsetForColumn(position.lineNumber, position.column) + layoutInfo.glyphMarginWidth + layoutInfo.lineNumbersWidth + layoutInfo.decorationsWidth - this.getScrollLeft();
        return {
            top: top,
            left: left,
            height: this._configuration.editor.lineHeight
        };
    };
    CodeEditorWidget.prototype.getOffsetForColumn = function (lineNumber, column) {
        if (!this._modelData || !this._modelData.hasRealView) {
            return -1;
        }
        return this._modelData.view.getOffsetForColumn(lineNumber, column);
    };
    CodeEditorWidget.prototype.render = function () {
        if (!this._modelData || !this._modelData.hasRealView) {
            return;
        }
        this._modelData.view.render(true, false);
    };
    CodeEditorWidget.prototype.applyFontInfo = function (target) {
        Configuration.applyFontInfoSlow(target, this._configuration.editor.fontInfo);
    };
    CodeEditorWidget.prototype._attachModel = function (model) {
        var _this = this;
        if (!model) {
            this._modelData = null;
            return;
        }
        var listenersToRemove = [];
        this._domElement.setAttribute('data-mode-id', model.getLanguageIdentifier().language);
        this._configuration.setIsDominatedByLongLines(model.isDominatedByLongLines());
        this._configuration.setMaxLineNumber(model.getLineCount());
        model.onBeforeAttached();
        var viewModel = new ViewModel(this._id, this._configuration, model, function (callback) { return scheduleAtNextAnimationFrame(callback); });
        listenersToRemove.push(model.onDidChangeDecorations(function (e) { return _this._onDidChangeModelDecorations.fire(e); }));
        listenersToRemove.push(model.onDidChangeLanguage(function (e) {
            _this._domElement.setAttribute('data-mode-id', model.getLanguageIdentifier().language);
            _this._onDidChangeModelLanguage.fire(e);
        }));
        listenersToRemove.push(model.onDidChangeLanguageConfiguration(function (e) { return _this._onDidChangeModelLanguageConfiguration.fire(e); }));
        listenersToRemove.push(model.onDidChangeContent(function (e) { return _this._onDidChangeModelContent.fire(e); }));
        listenersToRemove.push(model.onDidChangeOptions(function (e) { return _this._onDidChangeModelOptions.fire(e); }));
        // Someone might destroy the model from under the editor, so prevent any exceptions by setting a null model
        listenersToRemove.push(model.onWillDispose(function () { return _this.setModel(null); }));
        var cursor = new Cursor(this._configuration, model, viewModel);
        listenersToRemove.push(cursor.onDidReachMaxCursorCount(function () {
            _this._notificationService.warn(localize('cursors.maximum', "The number of cursors has been limited to {0}.", Cursor.MAX_CURSOR_COUNT));
        }));
        listenersToRemove.push(cursor.onDidAttemptReadOnlyEdit(function () {
            _this._onDidAttemptReadOnlyEdit.fire(void 0);
        }));
        listenersToRemove.push(cursor.onDidChange(function (e) {
            var positions = [];
            for (var i = 0, len = e.selections.length; i < len; i++) {
                positions[i] = e.selections[i].getPosition();
            }
            var e1 = {
                position: positions[0],
                secondaryPositions: positions.slice(1),
                reason: e.reason,
                source: e.source
            };
            _this._onDidChangeCursorPosition.fire(e1);
            var e2 = {
                selection: e.selections[0],
                secondarySelections: e.selections.slice(1),
                source: e.source,
                reason: e.reason
            };
            _this._onDidChangeCursorSelection.fire(e2);
        }));
        var _a = this._createView(viewModel, cursor), view = _a[0], hasRealView = _a[1];
        if (hasRealView) {
            this._domElement.appendChild(view.domNode.domNode);
            var keys = Object.keys(this._contentWidgets);
            for (var i = 0, len = keys.length; i < len; i++) {
                var widgetId = keys[i];
                view.addContentWidget(this._contentWidgets[widgetId]);
            }
            keys = Object.keys(this._overlayWidgets);
            for (var i = 0, len = keys.length; i < len; i++) {
                var widgetId = keys[i];
                view.addOverlayWidget(this._overlayWidgets[widgetId]);
            }
            view.render(false, true);
            view.domNode.domNode.setAttribute('data-uri', model.uri.toString());
        }
        this._modelData = new ModelData(model, viewModel, cursor, view, hasRealView, listenersToRemove);
    };
    CodeEditorWidget.prototype._createView = function (viewModel, cursor) {
        var _this = this;
        var commandDelegate;
        if (this.isSimpleWidget) {
            commandDelegate = {
                executeEditorCommand: function (editorCommand, args) {
                    editorCommand.runCoreEditorCommand(cursor, args);
                },
                paste: function (source, text, pasteOnNewLine, multicursorText) {
                    _this.trigger(source, Handler.Paste, { text: text, pasteOnNewLine: pasteOnNewLine, multicursorText: multicursorText });
                },
                type: function (source, text) {
                    _this.trigger(source, Handler.Type, { text: text });
                },
                replacePreviousChar: function (source, text, replaceCharCnt) {
                    _this.trigger(source, Handler.ReplacePreviousChar, { text: text, replaceCharCnt: replaceCharCnt });
                },
                compositionStart: function (source) {
                    _this.trigger(source, Handler.CompositionStart, undefined);
                },
                compositionEnd: function (source) {
                    _this.trigger(source, Handler.CompositionEnd, undefined);
                },
                cut: function (source) {
                    _this.trigger(source, Handler.Cut, undefined);
                }
            };
        }
        else {
            commandDelegate = {
                executeEditorCommand: function (editorCommand, args) {
                    editorCommand.runCoreEditorCommand(cursor, args);
                },
                paste: function (source, text, pasteOnNewLine, multicursorText) {
                    _this._commandService.executeCommand(Handler.Paste, {
                        text: text,
                        pasteOnNewLine: pasteOnNewLine,
                        multicursorText: multicursorText
                    });
                },
                type: function (source, text) {
                    _this._commandService.executeCommand(Handler.Type, {
                        text: text
                    });
                },
                replacePreviousChar: function (source, text, replaceCharCnt) {
                    _this._commandService.executeCommand(Handler.ReplacePreviousChar, {
                        text: text,
                        replaceCharCnt: replaceCharCnt
                    });
                },
                compositionStart: function (source) {
                    _this._commandService.executeCommand(Handler.CompositionStart, {});
                },
                compositionEnd: function (source) {
                    _this._commandService.executeCommand(Handler.CompositionEnd, {});
                },
                cut: function (source) {
                    _this._commandService.executeCommand(Handler.Cut, {});
                }
            };
        }
        var viewOutgoingEvents = new ViewOutgoingEvents(viewModel);
        viewOutgoingEvents.onDidGainFocus = function () {
            _this._editorTextFocus.setValue(true);
            // In IE, the focus is not synchronous, so we give it a little help
            _this._editorWidgetFocus.setValue(true);
        };
        viewOutgoingEvents.onDidScroll = function (e) { return _this._onDidScrollChange.fire(e); };
        viewOutgoingEvents.onDidLoseFocus = function () { return _this._editorTextFocus.setValue(false); };
        viewOutgoingEvents.onContextMenu = function (e) { return _this._onContextMenu.fire(e); };
        viewOutgoingEvents.onMouseDown = function (e) { return _this._onMouseDown.fire(e); };
        viewOutgoingEvents.onMouseUp = function (e) { return _this._onMouseUp.fire(e); };
        viewOutgoingEvents.onMouseDrag = function (e) { return _this._onMouseDrag.fire(e); };
        viewOutgoingEvents.onMouseDrop = function (e) { return _this._onMouseDrop.fire(e); };
        viewOutgoingEvents.onKeyUp = function (e) { return _this._onKeyUp.fire(e); };
        viewOutgoingEvents.onMouseMove = function (e) { return _this._onMouseMove.fire(e); };
        viewOutgoingEvents.onMouseLeave = function (e) { return _this._onMouseLeave.fire(e); };
        viewOutgoingEvents.onKeyDown = function (e) { return _this._onKeyDown.fire(e); };
        var view = new View(commandDelegate, this._configuration, this._themeService, viewModel, cursor, viewOutgoingEvents);
        return [view, true];
    };
    CodeEditorWidget.prototype._postDetachModelCleanup = function (detachedModel) {
        if (detachedModel) {
            detachedModel.removeAllDecorationsWithOwnerId(this._id);
        }
    };
    CodeEditorWidget.prototype._detachModel = function () {
        if (!this._modelData) {
            return null;
        }
        var model = this._modelData.model;
        var removeDomNode = this._modelData.hasRealView ? this._modelData.view.domNode.domNode : null;
        this._modelData.dispose();
        this._modelData = null;
        this._domElement.removeAttribute('data-mode-id');
        if (removeDomNode) {
            this._domElement.removeChild(removeDomNode);
        }
        return model;
    };
    CodeEditorWidget.prototype._removeDecorationType = function (key) {
        this._codeEditorService.removeDecorationType(key);
    };
    /* __GDPR__FRAGMENT__
        "EditorTelemetryData" : {}
    */
    CodeEditorWidget.prototype.getTelemetryData = function () {
        return this._telemetryData;
    };
    CodeEditorWidget.prototype.hasModel = function () {
        return (this._modelData !== null);
    };
    CodeEditorWidget = __decorate$2([
        __param$1(3, IInstantiationService),
        __param$1(4, ICodeEditorService),
        __param$1(5, ICommandService),
        __param$1(6, IContextKeyService),
        __param$1(7, IThemeService),
        __param$1(8, INotificationService)
    ], CodeEditorWidget);
    return CodeEditorWidget;
}(Disposable));
var BooleanEventEmitter = /** @class */ (function (_super) {
    __extends$16(BooleanEventEmitter, _super);
    function BooleanEventEmitter() {
        var _this = _super.call(this) || this;
        _this._onDidChangeToTrue = _this._register(new Emitter());
        _this.onDidChangeToTrue = _this._onDidChangeToTrue.event;
        _this._onDidChangeToFalse = _this._register(new Emitter());
        _this.onDidChangeToFalse = _this._onDidChangeToFalse.event;
        _this._value = 0 /* NotSet */;
        return _this;
    }
    BooleanEventEmitter.prototype.setValue = function (_value) {
        var value = (_value ? 2 /* True */ : 1 /* False */);
        if (this._value === value) {
            return;
        }
        this._value = value;
        if (this._value === 2 /* True */) {
            this._onDidChangeToTrue.fire();
        }
        else if (this._value === 1 /* False */) {
            this._onDidChangeToFalse.fire();
        }
    };
    return BooleanEventEmitter;
}(Disposable));
var EditorContextKeysManager = /** @class */ (function (_super) {
    __extends$16(EditorContextKeysManager, _super);
    function EditorContextKeysManager(editor, contextKeyService) {
        var _this = _super.call(this) || this;
        _this._editor = editor;
        contextKeyService.createKey('editorId', editor.getId());
        _this._editorFocus = EditorContextKeys.focus.bindTo(contextKeyService);
        _this._textInputFocus = EditorContextKeys.textInputFocus.bindTo(contextKeyService);
        _this._editorTextFocus = EditorContextKeys.editorTextFocus.bindTo(contextKeyService);
        _this._editorTabMovesFocus = EditorContextKeys.tabMovesFocus.bindTo(contextKeyService);
        _this._editorReadonly = EditorContextKeys.readOnly.bindTo(contextKeyService);
        _this._hasMultipleSelections = EditorContextKeys.hasMultipleSelections.bindTo(contextKeyService);
        _this._hasNonEmptySelection = EditorContextKeys.hasNonEmptySelection.bindTo(contextKeyService);
        _this._canUndo = EditorContextKeys.canUndo.bindTo(contextKeyService);
        _this._canRedo = EditorContextKeys.canRedo.bindTo(contextKeyService);
        _this._register(_this._editor.onDidChangeConfiguration(function () { return _this._updateFromConfig(); }));
        _this._register(_this._editor.onDidChangeCursorSelection(function () { return _this._updateFromSelection(); }));
        _this._register(_this._editor.onDidFocusEditorWidget(function () { return _this._updateFromFocus(); }));
        _this._register(_this._editor.onDidBlurEditorWidget(function () { return _this._updateFromFocus(); }));
        _this._register(_this._editor.onDidFocusEditorText(function () { return _this._updateFromFocus(); }));
        _this._register(_this._editor.onDidBlurEditorText(function () { return _this._updateFromFocus(); }));
        _this._register(_this._editor.onDidChangeModel(function () { return _this._updateFromModel(); }));
        _this._register(_this._editor.onDidChangeConfiguration(function () { return _this._updateFromModel(); }));
        _this._updateFromConfig();
        _this._updateFromSelection();
        _this._updateFromFocus();
        _this._updateFromModel();
        return _this;
    }
    EditorContextKeysManager.prototype._updateFromConfig = function () {
        var config = this._editor.getConfiguration();
        this._editorTabMovesFocus.set(config.tabFocusMode);
        this._editorReadonly.set(config.readOnly);
    };
    EditorContextKeysManager.prototype._updateFromSelection = function () {
        var selections = this._editor.getSelections();
        if (!selections) {
            this._hasMultipleSelections.reset();
            this._hasNonEmptySelection.reset();
        }
        else {
            this._hasMultipleSelections.set(selections.length > 1);
            this._hasNonEmptySelection.set(selections.some(function (s) { return !s.isEmpty(); }));
        }
    };
    EditorContextKeysManager.prototype._updateFromFocus = function () {
        this._editorFocus.set(this._editor.hasWidgetFocus() && !this._editor.isSimpleWidget);
        this._editorTextFocus.set(this._editor.hasTextFocus() && !this._editor.isSimpleWidget);
        this._textInputFocus.set(this._editor.hasTextFocus());
    };
    EditorContextKeysManager.prototype._updateFromModel = function () {
        var model = this._editor.getModel();
        this._canUndo.set(Boolean(model && model.canUndo()));
        this._canRedo.set(Boolean(model && model.canRedo()));
    };
    return EditorContextKeysManager;
}(Disposable));
var EditorModeContext = /** @class */ (function (_super) {
    __extends$16(EditorModeContext, _super);
    function EditorModeContext(editor, contextKeyService) {
        var _this = _super.call(this) || this;
        _this._editor = editor;
        _this._langId = EditorContextKeys.languageId.bindTo(contextKeyService);
        _this._hasCompletionItemProvider = EditorContextKeys.hasCompletionItemProvider.bindTo(contextKeyService);
        _this._hasCodeActionsProvider = EditorContextKeys.hasCodeActionsProvider.bindTo(contextKeyService);
        _this._hasCodeLensProvider = EditorContextKeys.hasCodeLensProvider.bindTo(contextKeyService);
        _this._hasDefinitionProvider = EditorContextKeys.hasDefinitionProvider.bindTo(contextKeyService);
        _this._hasImplementationProvider = EditorContextKeys.hasImplementationProvider.bindTo(contextKeyService);
        _this._hasTypeDefinitionProvider = EditorContextKeys.hasTypeDefinitionProvider.bindTo(contextKeyService);
        _this._hasHoverProvider = EditorContextKeys.hasHoverProvider.bindTo(contextKeyService);
        _this._hasDocumentHighlightProvider = EditorContextKeys.hasDocumentHighlightProvider.bindTo(contextKeyService);
        _this._hasDocumentSymbolProvider = EditorContextKeys.hasDocumentSymbolProvider.bindTo(contextKeyService);
        _this._hasReferenceProvider = EditorContextKeys.hasReferenceProvider.bindTo(contextKeyService);
        _this._hasRenameProvider = EditorContextKeys.hasRenameProvider.bindTo(contextKeyService);
        _this._hasDocumentFormattingProvider = EditorContextKeys.hasDocumentFormattingProvider.bindTo(contextKeyService);
        _this._hasDocumentSelectionFormattingProvider = EditorContextKeys.hasDocumentSelectionFormattingProvider.bindTo(contextKeyService);
        _this._hasSignatureHelpProvider = EditorContextKeys.hasSignatureHelpProvider.bindTo(contextKeyService);
        _this._isInWalkThrough = EditorContextKeys.isInEmbeddedEditor.bindTo(contextKeyService);
        var update = function () { return _this._update(); };
        // update when model/mode changes
        _this._register(editor.onDidChangeModel(update));
        _this._register(editor.onDidChangeModelLanguage(update));
        // update when registries change
        _this._register(CompletionProviderRegistry.onDidChange(update));
        _this._register(CodeActionProviderRegistry.onDidChange(update));
        _this._register(CodeLensProviderRegistry.onDidChange(update));
        _this._register(DefinitionProviderRegistry.onDidChange(update));
        _this._register(ImplementationProviderRegistry.onDidChange(update));
        _this._register(TypeDefinitionProviderRegistry.onDidChange(update));
        _this._register(HoverProviderRegistry.onDidChange(update));
        _this._register(DocumentHighlightProviderRegistry.onDidChange(update));
        _this._register(DocumentSymbolProviderRegistry.onDidChange(update));
        _this._register(ReferenceProviderRegistry.onDidChange(update));
        _this._register(RenameProviderRegistry.onDidChange(update));
        _this._register(DocumentFormattingEditProviderRegistry.onDidChange(update));
        _this._register(DocumentRangeFormattingEditProviderRegistry.onDidChange(update));
        _this._register(SignatureHelpProviderRegistry.onDidChange(update));
        update();
        return _this;
    }
    EditorModeContext.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    EditorModeContext.prototype.reset = function () {
        this._langId.reset();
        this._hasCompletionItemProvider.reset();
        this._hasCodeActionsProvider.reset();
        this._hasCodeLensProvider.reset();
        this._hasDefinitionProvider.reset();
        this._hasImplementationProvider.reset();
        this._hasTypeDefinitionProvider.reset();
        this._hasHoverProvider.reset();
        this._hasDocumentHighlightProvider.reset();
        this._hasDocumentSymbolProvider.reset();
        this._hasReferenceProvider.reset();
        this._hasRenameProvider.reset();
        this._hasDocumentFormattingProvider.reset();
        this._hasDocumentSelectionFormattingProvider.reset();
        this._hasSignatureHelpProvider.reset();
        this._isInWalkThrough.reset();
    };
    EditorModeContext.prototype._update = function () {
        var model = this._editor.getModel();
        if (!model) {
            this.reset();
            return;
        }
        this._langId.set(model.getLanguageIdentifier().language);
        this._hasCompletionItemProvider.set(CompletionProviderRegistry.has(model));
        this._hasCodeActionsProvider.set(CodeActionProviderRegistry.has(model));
        this._hasCodeLensProvider.set(CodeLensProviderRegistry.has(model));
        this._hasDefinitionProvider.set(DefinitionProviderRegistry.has(model));
        this._hasImplementationProvider.set(ImplementationProviderRegistry.has(model));
        this._hasTypeDefinitionProvider.set(TypeDefinitionProviderRegistry.has(model));
        this._hasHoverProvider.set(HoverProviderRegistry.has(model));
        this._hasDocumentHighlightProvider.set(DocumentHighlightProviderRegistry.has(model));
        this._hasDocumentSymbolProvider.set(DocumentSymbolProviderRegistry.has(model));
        this._hasReferenceProvider.set(ReferenceProviderRegistry.has(model));
        this._hasRenameProvider.set(RenameProviderRegistry.has(model));
        this._hasSignatureHelpProvider.set(SignatureHelpProviderRegistry.has(model));
        this._hasDocumentFormattingProvider.set(DocumentFormattingEditProviderRegistry.has(model) || DocumentRangeFormattingEditProviderRegistry.has(model));
        this._hasDocumentSelectionFormattingProvider.set(DocumentRangeFormattingEditProviderRegistry.has(model));
        this._isInWalkThrough.set(model.uri.scheme === Schemas.walkThroughSnippet);
    };
    return EditorModeContext;
}(Disposable));
var CodeEditorWidgetFocusTracker = /** @class */ (function (_super) {
    __extends$16(CodeEditorWidgetFocusTracker, _super);
    function CodeEditorWidgetFocusTracker(domElement) {
        var _this = _super.call(this) || this;
        _this._onChange = _this._register(new Emitter());
        _this.onChange = _this._onChange.event;
        _this._hasFocus = false;
        _this._domFocusTracker = _this._register(trackFocus(domElement));
        _this._register(_this._domFocusTracker.onDidFocus(function () {
            _this._hasFocus = true;
            _this._onChange.fire(void 0);
        }));
        _this._register(_this._domFocusTracker.onDidBlur(function () {
            _this._hasFocus = false;
            _this._onChange.fire(void 0);
        }));
        return _this;
    }
    CodeEditorWidgetFocusTracker.prototype.hasFocus = function () {
        return this._hasFocus;
    };
    return CodeEditorWidgetFocusTracker;
}(Disposable));
var squigglyStart = encodeURIComponent("<svg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 6 3' enable-background='new 0 0 6 3' height='3' width='6'><g fill='");
var squigglyEnd = encodeURIComponent("'><polygon points='5.5,0 2.5,3 1.1,3 4.1,0'/><polygon points='4,0 6,2 6,0.6 5.4,0'/><polygon points='0,2 1,3 2.4,3 0,0.6'/></g></svg>");
function getSquigglySVGData(color) {
    return squigglyStart + encodeURIComponent(color.toString()) + squigglyEnd;
}
var dotdotdotStart = encodeURIComponent("<svg xmlns=\"http://www.w3.org/2000/svg\" height=\"3\" width=\"12\"><g fill=\"");
var dotdotdotEnd = encodeURIComponent("\"><circle cx=\"1\" cy=\"1\" r=\"1\"/><circle cx=\"5\" cy=\"1\" r=\"1\"/><circle cx=\"9\" cy=\"1\" r=\"1\"/></g></svg>");
function getDotDotDotSVGData(color) {
    return dotdotdotStart + encodeURIComponent(color.toString()) + dotdotdotEnd;
}
registerThemingParticipant(function (theme, collector) {
    var errorBorderColor = theme.getColor(editorErrorBorder);
    if (errorBorderColor) {
        collector.addRule(".monaco-editor ." + "squiggly-error" /* EditorErrorDecoration */ + " { border-bottom: 4px double " + errorBorderColor + "; }");
    }
    var errorForeground = theme.getColor(editorErrorForeground);
    if (errorForeground) {
        collector.addRule(".monaco-editor ." + "squiggly-error" /* EditorErrorDecoration */ + " { background: url(\"data:image/svg+xml," + getSquigglySVGData(errorForeground) + "\") repeat-x bottom left; }");
    }
    var warningBorderColor = theme.getColor(editorWarningBorder);
    if (warningBorderColor) {
        collector.addRule(".monaco-editor ." + "squiggly-warning" /* EditorWarningDecoration */ + " { border-bottom: 4px double " + warningBorderColor + "; }");
    }
    var warningForeground = theme.getColor(editorWarningForeground);
    if (warningForeground) {
        collector.addRule(".monaco-editor ." + "squiggly-warning" /* EditorWarningDecoration */ + " { background: url(\"data:image/svg+xml," + getSquigglySVGData(warningForeground) + "\") repeat-x bottom left; }");
    }
    var infoBorderColor = theme.getColor(editorInfoBorder);
    if (infoBorderColor) {
        collector.addRule(".monaco-editor ." + "squiggly-info" /* EditorInfoDecoration */ + " { border-bottom: 4px double " + infoBorderColor + "; }");
    }
    var infoForeground = theme.getColor(editorInfoForeground);
    if (infoForeground) {
        collector.addRule(".monaco-editor ." + "squiggly-info" /* EditorInfoDecoration */ + " { background: url(\"data:image/svg+xml," + getSquigglySVGData(infoForeground) + "\") repeat-x bottom left; }");
    }
    var hintBorderColor = theme.getColor(editorHintBorder);
    if (hintBorderColor) {
        collector.addRule(".monaco-editor ." + "squiggly-hint" /* EditorHintDecoration */ + " { border-bottom: 2px dotted " + hintBorderColor + "; }");
    }
    var hintForeground = theme.getColor(editorHintForeground);
    if (hintForeground) {
        collector.addRule(".monaco-editor ." + "squiggly-hint" /* EditorHintDecoration */ + " { background: url(\"data:image/svg+xml," + getDotDotDotSVGData(hintForeground) + "\") no-repeat bottom left; }");
    }
    var unnecessaryForeground = theme.getColor(editorUnnecessaryCodeOpacity);
    if (unnecessaryForeground) {
        collector.addRule("." + SHOW_UNUSED_ENABLED_CLASS + " .monaco-editor ." + "squiggly-inline-unnecessary" /* EditorUnnecessaryInlineDecoration */ + " { opacity: " + unnecessaryForeground.rgba.a + "; will-change: opacity; }"); // TODO@Ben: 'will-change: opacity' is a workaround for https://github.com/Microsoft/vscode/issues/52196
    }
    var unnecessaryBorder = theme.getColor(editorUnnecessaryCodeBorder);
    if (unnecessaryBorder) {
        collector.addRule("." + SHOW_UNUSED_ENABLED_CLASS + " .monaco-editor ." + "squiggly-unnecessary" /* EditorUnnecessaryDecoration */ + " { border-bottom: 2px dashed " + unnecessaryBorder + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$17 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DEBUG = false;
var Sash = /** @class */ (function (_super) {
    __extends$17(Sash, _super);
    function Sash(container, layoutProvider, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this._state = 3 /* Enabled */;
        _this._onDidEnablementChange = _this._register(new Emitter());
        _this.onDidEnablementChange = _this._onDidEnablementChange.event;
        _this._onDidStart = _this._register(new Emitter());
        _this.onDidStart = _this._onDidStart.event;
        _this._onDidChange = _this._register(new Emitter());
        _this.onDidChange = _this._onDidChange.event;
        _this._onDidReset = _this._register(new Emitter());
        _this.onDidReset = _this._onDidReset.event;
        _this._onDidEnd = _this._register(new Emitter());
        _this.onDidEnd = _this._onDidEnd.event;
        _this.linkedSash = undefined;
        _this.orthogonalStartSashDisposables = [];
        _this.orthogonalEndSashDisposables = [];
        _this.el = append(container, $('.monaco-sash'));
        if (isMacintosh) {
            addClass(_this.el, 'mac');
        }
        _this._register(domEvent(_this.el, 'mousedown')(_this.onMouseDown, _this));
        _this._register(domEvent(_this.el, 'dblclick')(_this.onMouseDoubleClick, _this));
        Gesture.addTarget(_this.el);
        _this._register(domEvent(_this.el, EventType$1.Start)(_this.onTouchStart, _this));
        if (isIPad) {
            // see also http://ux.stackexchange.com/questions/39023/what-is-the-optimum-button-size-of-touch-screen-applications
            addClass(_this.el, 'touch');
        }
        _this.setOrientation(options.orientation || 0 /* VERTICAL */);
        _this.hidden = false;
        _this.layoutProvider = layoutProvider;
        _this.orthogonalStartSash = options.orthogonalStartSash;
        _this.orthogonalEndSash = options.orthogonalEndSash;
        toggleClass(_this.el, 'debug', DEBUG);
        return _this;
    }
    Object.defineProperty(Sash.prototype, "state", {
        get: function () { return this._state; },
        set: function (state) {
            if (this._state === state) {
                return;
            }
            toggleClass(this.el, 'disabled', state === 0 /* Disabled */);
            toggleClass(this.el, 'minimum', state === 1 /* Minimum */);
            toggleClass(this.el, 'maximum', state === 2 /* Maximum */);
            this._state = state;
            this._onDidEnablementChange.fire(state);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sash.prototype, "orthogonalStartSash", {
        get: function () { return this._orthogonalStartSash; },
        set: function (sash) {
            this.orthogonalStartSashDisposables = dispose(this.orthogonalStartSashDisposables);
            if (sash) {
                sash.onDidEnablementChange(this.onOrthogonalStartSashEnablementChange, this, this.orthogonalStartSashDisposables);
                this.onOrthogonalStartSashEnablementChange(sash.state);
            }
            else {
                this.onOrthogonalStartSashEnablementChange(0 /* Disabled */);
            }
            this._orthogonalStartSash = sash;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sash.prototype, "orthogonalEndSash", {
        get: function () { return this._orthogonalEndSash; },
        set: function (sash) {
            this.orthogonalEndSashDisposables = dispose(this.orthogonalEndSashDisposables);
            if (sash) {
                sash.onDidEnablementChange(this.onOrthogonalEndSashEnablementChange, this, this.orthogonalEndSashDisposables);
                this.onOrthogonalEndSashEnablementChange(sash.state);
            }
            else {
                this.onOrthogonalEndSashEnablementChange(0 /* Disabled */);
            }
            this._orthogonalEndSash = sash;
        },
        enumerable: true,
        configurable: true
    });
    Sash.prototype.setOrientation = function (orientation) {
        this.orientation = orientation;
        if (this.orientation === 1 /* HORIZONTAL */) {
            addClass(this.el, 'horizontal');
            removeClass(this.el, 'vertical');
        }
        else {
            removeClass(this.el, 'horizontal');
            addClass(this.el, 'vertical');
        }
        if (this.layoutProvider) {
            this.layout();
        }
    };
    Sash.prototype.onMouseDown = function (e) {
        var _this = this;
        EventHelper.stop(e, false);
        var isMultisashResize = false;
        if (this.linkedSash && !e.__linkedSashEvent) {
            e.__linkedSashEvent = true;
            this.linkedSash.onMouseDown(e);
        }
        if (!e.__orthogonalSashEvent) {
            var orthogonalSash = void 0;
            if (this.orientation === 0 /* VERTICAL */) {
                if (e.offsetY <= 4) {
                    orthogonalSash = this.orthogonalStartSash;
                }
                else if (e.offsetY >= this.el.clientHeight - 4) {
                    orthogonalSash = this.orthogonalEndSash;
                }
            }
            else {
                if (e.offsetX <= 4) {
                    orthogonalSash = this.orthogonalStartSash;
                }
                else if (e.offsetX >= this.el.clientWidth - 4) {
                    orthogonalSash = this.orthogonalEndSash;
                }
            }
            if (orthogonalSash) {
                isMultisashResize = true;
                e.__orthogonalSashEvent = true;
                orthogonalSash.onMouseDown(e);
            }
        }
        if (!this.state) {
            return;
        }
        var iframes = getElementsByTagName('iframe');
        for (var _i = 0, iframes_1 = iframes; _i < iframes_1.length; _i++) {
            var iframe = iframes_1[_i];
            iframe.style.pointerEvents = 'none'; // disable mouse events on iframes as long as we drag the sash
        }
        var mouseDownEvent = new StandardMouseEvent(e);
        var startX = mouseDownEvent.posx;
        var startY = mouseDownEvent.posy;
        var altKey = mouseDownEvent.altKey;
        var startEvent = { startX: startX, currentX: startX, startY: startY, currentY: startY, altKey: altKey };
        addClass(this.el, 'active');
        this._onDidStart.fire(startEvent);
        // fix https://github.com/Microsoft/vscode/issues/21675
        var style = createStyleSheet(this.el);
        var updateStyle = function () {
            var cursor = '';
            if (isMultisashResize) {
                cursor = 'all-scroll';
            }
            else if (_this.orientation === 1 /* HORIZONTAL */) {
                if (_this.state === 1 /* Minimum */) {
                    cursor = 's-resize';
                }
                else if (_this.state === 2 /* Maximum */) {
                    cursor = 'n-resize';
                }
                else {
                    cursor = isMacintosh ? 'row-resize' : 'ns-resize';
                }
            }
            else {
                if (_this.state === 1 /* Minimum */) {
                    cursor = 'e-resize';
                }
                else if (_this.state === 2 /* Maximum */) {
                    cursor = 'w-resize';
                }
                else {
                    cursor = isMacintosh ? 'col-resize' : 'ew-resize';
                }
            }
            style.innerHTML = "* { cursor: " + cursor + " !important; }";
        };
        var disposables = [];
        updateStyle();
        if (!isMultisashResize) {
            this.onDidEnablementChange(updateStyle, null, disposables);
        }
        var onMouseMove = function (e) {
            EventHelper.stop(e, false);
            var mouseMoveEvent = new StandardMouseEvent(e);
            var event = { startX: startX, currentX: mouseMoveEvent.posx, startY: startY, currentY: mouseMoveEvent.posy, altKey: altKey };
            _this._onDidChange.fire(event);
        };
        var onMouseUp = function (e) {
            EventHelper.stop(e, false);
            _this.el.removeChild(style);
            removeClass(_this.el, 'active');
            _this._onDidEnd.fire();
            dispose(disposables);
            var iframes = getElementsByTagName('iframe');
            for (var _i = 0, iframes_2 = iframes; _i < iframes_2.length; _i++) {
                var iframe = iframes_2[_i];
                iframe.style.pointerEvents = 'auto';
            }
        };
        domEvent(window, 'mousemove')(onMouseMove, null, disposables);
        domEvent(window, 'mouseup')(onMouseUp, null, disposables);
    };
    Sash.prototype.onMouseDoubleClick = function (event) {
        this._onDidReset.fire();
    };
    Sash.prototype.onTouchStart = function (event) {
        var _this = this;
        EventHelper.stop(event);
        var listeners = [];
        var startX = event.pageX;
        var startY = event.pageY;
        var altKey = event.altKey;
        this._onDidStart.fire({
            startX: startX,
            currentX: startX,
            startY: startY,
            currentY: startY,
            altKey: altKey
        });
        listeners.push(addDisposableListener(this.el, EventType$1.Change, function (event) {
            if (isNumber(event.pageX) && isNumber(event.pageY)) {
                _this._onDidChange.fire({
                    startX: startX,
                    currentX: event.pageX,
                    startY: startY,
                    currentY: event.pageY,
                    altKey: altKey
                });
            }
        }));
        listeners.push(addDisposableListener(this.el, EventType$1.End, function (event) {
            _this._onDidEnd.fire();
            dispose(listeners);
        }));
    };
    Sash.prototype.layout = function () {
        var size = isIPad ? 20 : 4;
        if (this.orientation === 0 /* VERTICAL */) {
            var verticalProvider = this.layoutProvider;
            this.el.style.left = verticalProvider.getVerticalSashLeft(this) - (size / 2) + 'px';
            if (verticalProvider.getVerticalSashTop) {
                this.el.style.top = verticalProvider.getVerticalSashTop(this) + 'px';
            }
            if (verticalProvider.getVerticalSashHeight) {
                this.el.style.height = verticalProvider.getVerticalSashHeight(this) + 'px';
            }
        }
        else {
            var horizontalProvider = this.layoutProvider;
            this.el.style.top = horizontalProvider.getHorizontalSashTop(this) - (size / 2) + 'px';
            if (horizontalProvider.getHorizontalSashLeft) {
                this.el.style.left = horizontalProvider.getHorizontalSashLeft(this) + 'px';
            }
            if (horizontalProvider.getHorizontalSashWidth) {
                this.el.style.width = horizontalProvider.getHorizontalSashWidth(this) + 'px';
            }
        }
    };
    Sash.prototype.hide = function () {
        this.hidden = true;
        this.el.style.display = 'none';
        this.el.setAttribute('aria-hidden', 'true');
    };
    Sash.prototype.onOrthogonalStartSashEnablementChange = function (state) {
        toggleClass(this.el, 'orthogonal-start', state !== 0 /* Disabled */);
    };
    Sash.prototype.onOrthogonalEndSashEnablementChange = function (state) {
        toggleClass(this.el, 'orthogonal-end', state !== 0 /* Disabled */);
    };
    Sash.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this.orthogonalStartSashDisposables = dispose(this.orthogonalStartSashDisposables);
        this.orthogonalEndSashDisposables = dispose(this.orthogonalEndSashDisposables);
        if (this.el && this.el.parentElement) {
            this.el.parentElement.removeChild(this.el);
        }
        this.el = null; // StrictNullOverride: nulling out ok in dispose
    };
    return Sash;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var EditorState = /** @class */ (function () {
    function EditorState(editor, flags) {
        this.flags = flags;
        if ((this.flags & 1 /* Value */) !== 0) {
            var model = editor.getModel();
            this.modelVersionId = model ? format('{0}#{1}', model.uri.toString(), model.getVersionId()) : null;
        }
        if ((this.flags & 4 /* Position */) !== 0) {
            this.position = editor.getPosition();
        }
        if ((this.flags & 2 /* Selection */) !== 0) {
            this.selection = editor.getSelection();
        }
        if ((this.flags & 8 /* Scroll */) !== 0) {
            this.scrollLeft = editor.getScrollLeft();
            this.scrollTop = editor.getScrollTop();
        }
    }
    EditorState.prototype._equals = function (other) {
        if (!(other instanceof EditorState)) {
            return false;
        }
        var state = other;
        if (this.modelVersionId !== state.modelVersionId) {
            return false;
        }
        if (this.scrollLeft !== state.scrollLeft || this.scrollTop !== state.scrollTop) {
            return false;
        }
        if (!this.position && state.position || this.position && !state.position || this.position && state.position && !this.position.equals(state.position)) {
            return false;
        }
        if (!this.selection && state.selection || this.selection && !state.selection || this.selection && state.selection && !this.selection.equalsRange(state.selection)) {
            return false;
        }
        return true;
    };
    EditorState.prototype.validate = function (editor) {
        return this._equals(new EditorState(editor, this.flags));
    };
    return EditorState;
}());
var StableEditorScrollState = /** @class */ (function () {
    function StableEditorScrollState(_visiblePosition, _visiblePositionScrollDelta) {
        this._visiblePosition = _visiblePosition;
        this._visiblePositionScrollDelta = _visiblePositionScrollDelta;
    }
    StableEditorScrollState.capture = function (editor) {
        var visiblePosition = null;
        var visiblePositionScrollDelta = 0;
        if (editor.getScrollTop() !== 0) {
            var visibleRanges = editor.getVisibleRanges();
            if (visibleRanges.length > 0) {
                visiblePosition = visibleRanges[0].getStartPosition();
                var visiblePositionScrollTop = editor.getTopForPosition(visiblePosition.lineNumber, visiblePosition.column);
                visiblePositionScrollDelta = editor.getScrollTop() - visiblePositionScrollTop;
            }
        }
        return new StableEditorScrollState(visiblePosition, visiblePositionScrollDelta);
    };
    StableEditorScrollState.prototype.restore = function (editor) {
        if (this._visiblePosition) {
            var visiblePositionScrollTop = editor.getTopForPosition(this._visiblePosition.lineNumber, this._visiblePosition.column);
            editor.setScrollTop(visiblePositionScrollTop + this._visiblePositionScrollDelta);
        }
    };
    return StableEditorScrollState;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$18 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var BaseActionItem = /** @class */ (function (_super) {
    __extends$18(BaseActionItem, _super);
    function BaseActionItem(context, action, options) {
        var _this = _super.call(this) || this;
        _this.options = options;
        _this._context = context || _this;
        _this._action = action;
        if (action instanceof Action) {
            _this._register(action.onDidChange(function (event) {
                if (!_this.element) {
                    // we have not been rendered yet, so there
                    // is no point in updating the UI
                    return;
                }
                _this.handleActionChangeEvent(event);
            }));
        }
        return _this;
    }
    BaseActionItem.prototype.handleActionChangeEvent = function (event) {
        if (event.enabled !== void 0) {
            this.updateEnabled();
        }
        if (event.checked !== void 0) {
            this.updateChecked();
        }
        if (event.class !== void 0) {
            this.updateClass();
        }
        if (event.label !== void 0) {
            this.updateLabel();
            this.updateTooltip();
        }
        if (event.tooltip !== void 0) {
            this.updateTooltip();
        }
    };
    Object.defineProperty(BaseActionItem.prototype, "actionRunner", {
        get: function () {
            return this._actionRunner;
        },
        set: function (actionRunner) {
            this._actionRunner = actionRunner;
        },
        enumerable: true,
        configurable: true
    });
    BaseActionItem.prototype.getAction = function () {
        return this._action;
    };
    BaseActionItem.prototype.isEnabled = function () {
        return this._action.enabled;
    };
    BaseActionItem.prototype.setActionContext = function (newContext) {
        this._context = newContext;
    };
    BaseActionItem.prototype.render = function (container) {
        var _this = this;
        this.element = container;
        Gesture.addTarget(container);
        var enableDragging = this.options && this.options.draggable;
        if (enableDragging) {
            container.draggable = true;
        }
        this._register(addDisposableListener(this.element, EventType$1.Tap, function (e) { return _this.onClick(e); }));
        this._register(addDisposableListener(this.element, EventType.MOUSE_DOWN, function (e) {
            if (!enableDragging) {
                EventHelper.stop(e, true); // do not run when dragging is on because that would disable it
            }
            var mouseEvent = e;
            if (_this._action.enabled && mouseEvent.button === 0) {
                addClass(_this.element, 'active');
            }
        }));
        this._register(addDisposableListener(this.element, EventType.CLICK, function (e) {
            EventHelper.stop(e, true);
            // See https://developer.mozilla.org/en-US/Add-ons/WebExtensions/Interact_with_the_clipboard
            // > Writing to the clipboard
            // > You can use the "cut" and "copy" commands without any special
            // permission if you are using them in a short-lived event handler
            // for a user action (for example, a click handler).
            // => to get the Copy and Paste context menu actions working on Firefox,
            // there should be no timeout here
            if (_this.options && _this.options.isMenu) {
                _this.onClick(e);
            }
            else {
                setImmediate(function () { return _this.onClick(e); });
            }
        }));
        this._register(addDisposableListener(this.element, EventType.DBLCLICK, function (e) {
            EventHelper.stop(e, true);
        }));
        [EventType.MOUSE_UP, EventType.MOUSE_OUT].forEach(function (event) {
            _this._register(addDisposableListener(_this.element, event, function (e) {
                EventHelper.stop(e);
                removeClass(_this.element, 'active');
            }));
        });
    };
    BaseActionItem.prototype.onClick = function (event) {
        EventHelper.stop(event, true);
        var context;
        if (isUndefinedOrNull(this._context)) {
            context = event;
        }
        else {
            context = this._context;
            if (isObject(context)) {
                context.event = event;
            }
        }
        this._actionRunner.run(this._action, context);
    };
    BaseActionItem.prototype.focus = function () {
        if (this.element) {
            this.element.focus();
            addClass(this.element, 'focused');
        }
    };
    BaseActionItem.prototype.blur = function () {
        if (this.element) {
            this.element.blur();
            removeClass(this.element, 'focused');
        }
    };
    BaseActionItem.prototype.updateEnabled = function () {
        // implement in subclass
    };
    BaseActionItem.prototype.updateLabel = function () {
        // implement in subclass
    };
    BaseActionItem.prototype.updateTooltip = function () {
        // implement in subclass
    };
    BaseActionItem.prototype.updateClass = function () {
        // implement in subclass
    };
    BaseActionItem.prototype.updateChecked = function () {
        // implement in subclass
    };
    BaseActionItem.prototype.dispose = function () {
        if (this.element) {
            removeNode(this.element);
            this.element = null;
        }
        _super.prototype.dispose.call(this);
    };
    return BaseActionItem;
}(Disposable));
var Separator = /** @class */ (function (_super) {
    __extends$18(Separator, _super);
    function Separator(label) {
        var _this = _super.call(this, Separator.ID, label, label ? 'separator text' : 'separator') || this;
        _this.checked = false;
        _this.radio = false;
        _this.enabled = false;
        return _this;
    }
    Separator.ID = 'vs.actions.separator';
    return Separator;
}(Action));
var ActionItem = /** @class */ (function (_super) {
    __extends$18(ActionItem, _super);
    function ActionItem(context, action, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, context, action, options) || this;
        _this.options = options;
        _this.options.icon = options.icon !== undefined ? options.icon : false;
        _this.options.label = options.label !== undefined ? options.label : true;
        _this.cssClass = '';
        return _this;
    }
    ActionItem.prototype.render = function (container) {
        _super.prototype.render.call(this, container);
        this.label = append(this.element, $('a.action-label'));
        if (this._action.id === Separator.ID) {
            this.label.setAttribute('role', 'presentation'); // A separator is a presentation item
        }
        else {
            if (this.options.isMenu) {
                this.label.setAttribute('role', 'menuitem');
            }
            else {
                this.label.setAttribute('role', 'button');
            }
        }
        if (this.options.label && this.options.keybinding) {
            append(this.element, $('span.keybinding')).textContent = this.options.keybinding;
        }
        this.updateClass();
        this.updateLabel();
        this.updateTooltip();
        this.updateEnabled();
        this.updateChecked();
    };
    ActionItem.prototype.focus = function () {
        _super.prototype.focus.call(this);
        this.label.focus();
    };
    ActionItem.prototype.updateLabel = function () {
        if (this.options.label) {
            this.label.textContent = this.getAction().label;
        }
    };
    ActionItem.prototype.updateTooltip = function () {
        var title = null;
        if (this.getAction().tooltip) {
            title = this.getAction().tooltip;
        }
        else if (!this.options.label && this.getAction().label && this.options.icon) {
            title = this.getAction().label;
            if (this.options.keybinding) {
                title = localize({ key: 'titleLabel', comment: ['action title', 'action keybinding'] }, "{0} ({1})", title, this.options.keybinding);
            }
        }
        if (title) {
            this.label.title = title;
        }
    };
    ActionItem.prototype.updateClass = function () {
        if (this.cssClass) {
            removeClasses(this.label, this.cssClass);
        }
        if (this.options.icon) {
            this.cssClass = this.getAction().class;
            addClass(this.label, 'icon');
            if (this.cssClass) {
                addClasses(this.label, this.cssClass);
            }
            this.updateEnabled();
        }
        else {
            removeClass(this.label, 'icon');
        }
    };
    ActionItem.prototype.updateEnabled = function () {
        if (this.getAction().enabled) {
            this.label.removeAttribute('aria-disabled');
            removeClass(this.element, 'disabled');
            removeClass(this.label, 'disabled');
            this.label.tabIndex = 0;
        }
        else {
            this.label.setAttribute('aria-disabled', 'true');
            addClass(this.element, 'disabled');
            addClass(this.label, 'disabled');
            removeTabIndexAndUpdateFocus(this.label);
        }
    };
    ActionItem.prototype.updateChecked = function () {
        if (this.getAction().checked) {
            addClass(this.label, 'checked');
        }
        else {
            removeClass(this.label, 'checked');
        }
    };
    return ActionItem;
}(BaseActionItem));
var defaultOptions = {
    orientation: 0 /* HORIZONTAL */,
    context: null,
    triggerKeys: {
        keys: [3 /* Enter */, 10 /* Space */],
        keyDown: false
    }
};
var ActionBar = /** @class */ (function (_super) {
    __extends$18(ActionBar, _super);
    function ActionBar(container, options) {
        if (options === void 0) { options = defaultOptions; }
        var _this = _super.call(this) || this;
        _this._onDidBlur = _this._register(new Emitter());
        _this._onDidCancel = _this._register(new Emitter());
        _this._onDidRun = _this._register(new Emitter());
        _this._onDidBeforeRun = _this._register(new Emitter());
        _this.options = options;
        _this._context = options.context;
        _this._actionRunner = _this.options.actionRunner;
        if (!_this.options.triggerKeys) {
            _this.options.triggerKeys = defaultOptions.triggerKeys;
        }
        if (!_this._actionRunner) {
            _this._actionRunner = new ActionRunner();
            _this._register(_this._actionRunner);
        }
        _this._register(_this._actionRunner.onDidRun(function (e) { return _this._onDidRun.fire(e); }));
        _this._register(_this._actionRunner.onDidBeforeRun(function (e) { return _this._onDidBeforeRun.fire(e); }));
        _this.items = [];
        _this.focusedItem = undefined;
        _this.domNode = document.createElement('div');
        _this.domNode.className = 'monaco-action-bar';
        if (options.animated !== false) {
            addClass(_this.domNode, 'animated');
        }
        var previousKey;
        var nextKey;
        switch (_this.options.orientation) {
            case 0 /* HORIZONTAL */:
                previousKey = 15 /* LeftArrow */;
                nextKey = 17 /* RightArrow */;
                break;
            case 1 /* HORIZONTAL_REVERSE */:
                previousKey = 17 /* RightArrow */;
                nextKey = 15 /* LeftArrow */;
                _this.domNode.className += ' reverse';
                break;
            case 2 /* VERTICAL */:
                previousKey = 16 /* UpArrow */;
                nextKey = 18 /* DownArrow */;
                _this.domNode.className += ' vertical';
                break;
            case 3 /* VERTICAL_REVERSE */:
                previousKey = 18 /* DownArrow */;
                nextKey = 16 /* UpArrow */;
                _this.domNode.className += ' vertical reverse';
                break;
        }
        _this._register(addDisposableListener(_this.domNode, EventType.KEY_DOWN, function (e) {
            var event = new StandardKeyboardEvent(e);
            var eventHandled = true;
            if (event.equals(previousKey)) {
                _this.focusPrevious();
            }
            else if (event.equals(nextKey)) {
                _this.focusNext();
            }
            else if (event.equals(9 /* Escape */)) {
                _this.cancel();
            }
            else if (_this.isTriggerKeyEvent(event)) {
                // Staying out of the else branch even if not triggered
                if (_this.options.triggerKeys && _this.options.triggerKeys.keyDown) {
                    _this.doTrigger(event);
                }
            }
            else {
                eventHandled = false;
            }
            if (eventHandled) {
                event.preventDefault();
                event.stopPropagation();
            }
        }));
        _this._register(addDisposableListener(_this.domNode, EventType.KEY_UP, function (e) {
            var event = new StandardKeyboardEvent(e);
            // Run action on Enter/Space
            if (_this.isTriggerKeyEvent(event)) {
                if (!_this.options.triggerKeys.keyDown) {
                    _this.doTrigger(event);
                }
                event.preventDefault();
                event.stopPropagation();
            }
            // Recompute focused item
            else if (event.equals(2 /* Tab */) || event.equals(1024 /* Shift */ | 2 /* Tab */)) {
                _this.updateFocusedItem();
            }
        }));
        _this.focusTracker = _this._register(trackFocus(_this.domNode));
        _this._register(_this.focusTracker.onDidBlur(function () {
            if (document.activeElement === _this.domNode || !isAncestor(document.activeElement, _this.domNode)) {
                _this._onDidBlur.fire();
                _this.focusedItem = undefined;
            }
        }));
        _this._register(_this.focusTracker.onDidFocus(function () { return _this.updateFocusedItem(); }));
        _this.actionsList = document.createElement('ul');
        _this.actionsList.className = 'actions-container';
        _this.actionsList.setAttribute('role', 'toolbar');
        if (_this.options.ariaLabel) {
            _this.actionsList.setAttribute('aria-label', _this.options.ariaLabel);
        }
        _this.domNode.appendChild(_this.actionsList);
        container.appendChild(_this.domNode);
        return _this;
    }
    Object.defineProperty(ActionBar.prototype, "onDidBlur", {
        get: function () { return this._onDidBlur.event; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActionBar.prototype, "onDidCancel", {
        get: function () { return this._onDidCancel.event; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActionBar.prototype, "onDidRun", {
        get: function () { return this._onDidRun.event; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActionBar.prototype, "onDidBeforeRun", {
        get: function () { return this._onDidBeforeRun.event; },
        enumerable: true,
        configurable: true
    });
    ActionBar.prototype.isTriggerKeyEvent = function (event) {
        var ret = false;
        if (this.options.triggerKeys) {
            this.options.triggerKeys.keys.forEach(function (keyCode) {
                ret = ret || event.equals(keyCode);
            });
        }
        return ret;
    };
    ActionBar.prototype.updateFocusedItem = function () {
        for (var i = 0; i < this.actionsList.children.length; i++) {
            var elem = this.actionsList.children[i];
            if (isAncestor(document.activeElement, elem)) {
                this.focusedItem = i;
                break;
            }
        }
    };
    Object.defineProperty(ActionBar.prototype, "context", {
        get: function () {
            return this._context;
        },
        set: function (context) {
            this._context = context;
            this.items.forEach(function (i) { return i.setActionContext(context); });
        },
        enumerable: true,
        configurable: true
    });
    ActionBar.prototype.getContainer = function () {
        return this.domNode;
    };
    ActionBar.prototype.push = function (arg, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        var actions = !Array.isArray(arg) ? [arg] : arg;
        var index = isNumber(options.index) ? options.index : null;
        actions.forEach(function (action) {
            var actionItemElement = document.createElement('li');
            actionItemElement.className = 'action-item';
            actionItemElement.setAttribute('role', 'presentation');
            // Prevent native context menu on actions
            _this._register(addDisposableListener(actionItemElement, EventType.CONTEXT_MENU, function (e) {
                e.preventDefault();
                e.stopPropagation();
            }));
            var item = null;
            if (_this.options.actionItemProvider) {
                item = _this.options.actionItemProvider(action);
            }
            if (!item) {
                item = new ActionItem(_this.context, action, options);
            }
            item.actionRunner = _this._actionRunner;
            item.setActionContext(_this.context);
            item.render(actionItemElement);
            if (index === null || index < 0 || index >= _this.actionsList.children.length) {
                _this.actionsList.appendChild(actionItemElement);
                _this.items.push(item);
            }
            else {
                _this.actionsList.insertBefore(actionItemElement, _this.actionsList.children[index]);
                _this.items.splice(index, 0, item);
                index++;
            }
        });
    };
    ActionBar.prototype.clear = function () {
        this.items = dispose(this.items);
        clearNode(this.actionsList);
    };
    ActionBar.prototype.isEmpty = function () {
        return this.items.length === 0;
    };
    ActionBar.prototype.focus = function (selectFirst) {
        if (selectFirst && typeof this.focusedItem === 'undefined') {
            // Focus the first enabled item
            this.focusedItem = this.items.length - 1;
            this.focusNext();
        }
        else {
            this.updateFocus();
        }
    };
    ActionBar.prototype.focusNext = function () {
        if (typeof this.focusedItem === 'undefined') {
            this.focusedItem = this.items.length - 1;
        }
        var startIndex = this.focusedItem;
        var item;
        do {
            this.focusedItem = (this.focusedItem + 1) % this.items.length;
            item = this.items[this.focusedItem];
        } while (this.focusedItem !== startIndex && !item.isEnabled());
        if (this.focusedItem === startIndex && !item.isEnabled()) {
            this.focusedItem = undefined;
        }
        this.updateFocus();
    };
    ActionBar.prototype.focusPrevious = function () {
        if (typeof this.focusedItem === 'undefined') {
            this.focusedItem = 0;
        }
        var startIndex = this.focusedItem;
        var item;
        do {
            this.focusedItem = this.focusedItem - 1;
            if (this.focusedItem < 0) {
                this.focusedItem = this.items.length - 1;
            }
            item = this.items[this.focusedItem];
        } while (this.focusedItem !== startIndex && !item.isEnabled());
        if (this.focusedItem === startIndex && !item.isEnabled()) {
            this.focusedItem = undefined;
        }
        this.updateFocus(true);
    };
    ActionBar.prototype.updateFocus = function (fromRight) {
        if (typeof this.focusedItem === 'undefined') {
            this.actionsList.focus();
        }
        for (var i = 0; i < this.items.length; i++) {
            var item = this.items[i];
            var actionItem = item;
            if (i === this.focusedItem) {
                if (isFunction(actionItem.isEnabled)) {
                    if (actionItem.isEnabled() && isFunction(actionItem.focus)) {
                        actionItem.focus(fromRight);
                    }
                    else {
                        this.actionsList.focus();
                    }
                }
            }
            else {
                if (isFunction(actionItem.blur)) {
                    actionItem.blur();
                }
            }
        }
    };
    ActionBar.prototype.doTrigger = function (event) {
        if (typeof this.focusedItem === 'undefined') {
            return; //nothing to focus
        }
        // trigger action
        var actionItem = this.items[this.focusedItem];
        if (actionItem instanceof BaseActionItem) {
            var context_1 = (actionItem._context === null || actionItem._context === undefined) ? event : actionItem._context;
            this.run(actionItem._action, context_1);
        }
    };
    ActionBar.prototype.cancel = function () {
        if (document.activeElement instanceof HTMLElement) {
            document.activeElement.blur(); // remove focus from focused action
        }
        this._onDidCancel.fire();
    };
    ActionBar.prototype.run = function (action, context) {
        return this._actionRunner.run(action, context);
    };
    ActionBar.prototype.dispose = function () {
        if (this.items !== null) {
            dispose(this.items);
        }
        this.items = null;
        removeNode(this.getContainer());
        _super.prototype.dispose.call(this);
    };
    return ActionBar;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$19 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DIFF_LINES_PADDING = 3;
var DiffEntry = /** @class */ (function () {
    function DiffEntry(originalLineStart, originalLineEnd, modifiedLineStart, modifiedLineEnd) {
        this.originalLineStart = originalLineStart;
        this.originalLineEnd = originalLineEnd;
        this.modifiedLineStart = modifiedLineStart;
        this.modifiedLineEnd = modifiedLineEnd;
    }
    DiffEntry.prototype.getType = function () {
        if (this.originalLineStart === 0) {
            return 1 /* Insert */;
        }
        if (this.modifiedLineStart === 0) {
            return 2 /* Delete */;
        }
        return 0 /* Equal */;
    };
    return DiffEntry;
}());
var Diff = /** @class */ (function () {
    function Diff(entries) {
        this.entries = entries;
    }
    return Diff;
}());
var DiffReview = /** @class */ (function (_super) {
    __extends$19(DiffReview, _super);
    function DiffReview(diffEditor) {
        var _this = _super.call(this) || this;
        _this._width = 0;
        _this._diffEditor = diffEditor;
        _this._isVisible = false;
        _this.shadow = createFastDomNode(document.createElement('div'));
        _this.shadow.setClassName('diff-review-shadow');
        _this.actionBarContainer = createFastDomNode(document.createElement('div'));
        _this.actionBarContainer.setClassName('diff-review-actions');
        _this._actionBar = _this._register(new ActionBar(_this.actionBarContainer.domNode));
        _this._actionBar.push(new Action('diffreview.close', localize('label.close', "Close"), 'close-diff-review', true, function () {
            _this.hide();
            return null;
        }), { label: false, icon: true });
        _this.domNode = createFastDomNode(document.createElement('div'));
        _this.domNode.setClassName('diff-review monaco-editor-background');
        _this._content = createFastDomNode(document.createElement('div'));
        _this._content.setClassName('diff-review-content');
        _this.scrollbar = _this._register(new DomScrollableElement(_this._content.domNode, {}));
        _this.domNode.domNode.appendChild(_this.scrollbar.getDomNode());
        _this._register(diffEditor.onDidUpdateDiff(function () {
            if (!_this._isVisible) {
                return;
            }
            _this._diffs = _this._compute();
            _this._render();
        }));
        _this._register(diffEditor.getModifiedEditor().onDidChangeCursorPosition(function () {
            if (!_this._isVisible) {
                return;
            }
            _this._render();
        }));
        _this._register(diffEditor.getOriginalEditor().onDidFocusEditorWidget(function () {
            if (_this._isVisible) {
                _this.hide();
            }
        }));
        _this._register(diffEditor.getModifiedEditor().onDidFocusEditorWidget(function () {
            if (_this._isVisible) {
                _this.hide();
            }
        }));
        _this._register(addStandardDisposableListener(_this.domNode.domNode, 'click', function (e) {
            e.preventDefault();
            var row = findParentWithClass(e.target, 'diff-review-row');
            if (row) {
                _this._goToRow(row);
            }
        }));
        _this._register(addStandardDisposableListener(_this.domNode.domNode, 'keydown', function (e) {
            if (e.equals(18 /* DownArrow */)
                || e.equals(2048 /* CtrlCmd */ | 18 /* DownArrow */)
                || e.equals(512 /* Alt */ | 18 /* DownArrow */)) {
                e.preventDefault();
                _this._goToRow(_this._getNextRow());
            }
            if (e.equals(16 /* UpArrow */)
                || e.equals(2048 /* CtrlCmd */ | 16 /* UpArrow */)
                || e.equals(512 /* Alt */ | 16 /* UpArrow */)) {
                e.preventDefault();
                _this._goToRow(_this._getPrevRow());
            }
            if (e.equals(9 /* Escape */)
                || e.equals(2048 /* CtrlCmd */ | 9 /* Escape */)
                || e.equals(512 /* Alt */ | 9 /* Escape */)
                || e.equals(1024 /* Shift */ | 9 /* Escape */)) {
                e.preventDefault();
                _this.hide();
            }
            if (e.equals(10 /* Space */)
                || e.equals(3 /* Enter */)) {
                e.preventDefault();
                _this.accept();
            }
        }));
        _this._diffs = [];
        _this._currentDiff = null;
        return _this;
    }
    DiffReview.prototype.prev = function () {
        var index = 0;
        if (!this._isVisible) {
            this._diffs = this._compute();
        }
        if (this._isVisible) {
            var currentIndex = -1;
            for (var i = 0, len = this._diffs.length; i < len; i++) {
                if (this._diffs[i] === this._currentDiff) {
                    currentIndex = i;
                    break;
                }
            }
            index = (this._diffs.length + currentIndex - 1);
        }
        else {
            index = this._findDiffIndex(this._diffEditor.getPosition());
        }
        if (this._diffs.length === 0) {
            // Nothing to do
            return;
        }
        index = index % this._diffs.length;
        this._diffEditor.setPosition(new Position(this._diffs[index].entries[0].modifiedLineStart, 1));
        this._isVisible = true;
        this._diffEditor.doLayout();
        this._render();
        this._goToRow(this._getNextRow());
    };
    DiffReview.prototype.next = function () {
        var index = 0;
        if (!this._isVisible) {
            this._diffs = this._compute();
        }
        if (this._isVisible) {
            var currentIndex = -1;
            for (var i = 0, len = this._diffs.length; i < len; i++) {
                if (this._diffs[i] === this._currentDiff) {
                    currentIndex = i;
                    break;
                }
            }
            index = (currentIndex + 1);
        }
        else {
            index = this._findDiffIndex(this._diffEditor.getPosition());
        }
        if (this._diffs.length === 0) {
            // Nothing to do
            return;
        }
        index = index % this._diffs.length;
        this._diffEditor.setPosition(new Position(this._diffs[index].entries[0].modifiedLineStart, 1));
        this._isVisible = true;
        this._diffEditor.doLayout();
        this._render();
        this._goToRow(this._getNextRow());
    };
    DiffReview.prototype.accept = function () {
        var jumpToLineNumber = -1;
        var current = this._getCurrentFocusedRow();
        if (current) {
            var lineNumber = parseInt(current.getAttribute('data-line'), 10);
            if (!isNaN(lineNumber)) {
                jumpToLineNumber = lineNumber;
            }
        }
        this.hide();
        if (jumpToLineNumber !== -1) {
            this._diffEditor.setPosition(new Position(jumpToLineNumber, 1));
            this._diffEditor.revealPosition(new Position(jumpToLineNumber, 1), 1 /* Immediate */);
        }
    };
    DiffReview.prototype.hide = function () {
        this._isVisible = false;
        this._diffEditor.focus();
        this._diffEditor.doLayout();
        this._render();
    };
    DiffReview.prototype._getPrevRow = function () {
        var current = this._getCurrentFocusedRow();
        if (!current) {
            return this._getFirstRow();
        }
        if (current.previousElementSibling) {
            return current.previousElementSibling;
        }
        return current;
    };
    DiffReview.prototype._getNextRow = function () {
        var current = this._getCurrentFocusedRow();
        if (!current) {
            return this._getFirstRow();
        }
        if (current.nextElementSibling) {
            return current.nextElementSibling;
        }
        return current;
    };
    DiffReview.prototype._getFirstRow = function () {
        return this.domNode.domNode.querySelector('.diff-review-row');
    };
    DiffReview.prototype._getCurrentFocusedRow = function () {
        var result = document.activeElement;
        if (result && /diff-review-row/.test(result.className)) {
            return result;
        }
        return null;
    };
    DiffReview.prototype._goToRow = function (row) {
        var prev = this._getCurrentFocusedRow();
        row.tabIndex = 0;
        row.focus();
        if (prev && prev !== row) {
            prev.tabIndex = -1;
        }
        this.scrollbar.scanDomNode();
    };
    DiffReview.prototype.isVisible = function () {
        return this._isVisible;
    };
    DiffReview.prototype.layout = function (top, width, height) {
        this._width = width;
        this.shadow.setTop(top - 6);
        this.shadow.setWidth(width);
        this.shadow.setHeight(this._isVisible ? 6 : 0);
        this.domNode.setTop(top);
        this.domNode.setWidth(width);
        this.domNode.setHeight(height);
        this._content.setHeight(height);
        this._content.setWidth(width);
        if (this._isVisible) {
            this.actionBarContainer.setAttribute('aria-hidden', 'false');
            this.actionBarContainer.setDisplay('block');
        }
        else {
            this.actionBarContainer.setAttribute('aria-hidden', 'true');
            this.actionBarContainer.setDisplay('none');
        }
    };
    DiffReview.prototype._compute = function () {
        var lineChanges = this._diffEditor.getLineChanges();
        if (!lineChanges || lineChanges.length === 0) {
            return [];
        }
        var originalModel = this._diffEditor.getOriginalEditor().getModel();
        var modifiedModel = this._diffEditor.getModifiedEditor().getModel();
        if (!originalModel || !modifiedModel) {
            return [];
        }
        return DiffReview._mergeAdjacent(lineChanges, originalModel.getLineCount(), modifiedModel.getLineCount());
    };
    DiffReview._mergeAdjacent = function (lineChanges, originalLineCount, modifiedLineCount) {
        if (!lineChanges || lineChanges.length === 0) {
            return [];
        }
        var diffs = [], diffsLength = 0;
        for (var i = 0, len = lineChanges.length; i < len; i++) {
            var lineChange = lineChanges[i];
            var originalStart = lineChange.originalStartLineNumber;
            var originalEnd = lineChange.originalEndLineNumber;
            var modifiedStart = lineChange.modifiedStartLineNumber;
            var modifiedEnd = lineChange.modifiedEndLineNumber;
            var r_1 = [], rLength_1 = 0;
            // Emit before anchors
            {
                var originalEqualAbove = (originalEnd === 0 ? originalStart : originalStart - 1);
                var modifiedEqualAbove = (modifiedEnd === 0 ? modifiedStart : modifiedStart - 1);
                // Make sure we don't step into the previous diff
                var minOriginal = 1;
                var minModified = 1;
                if (i > 0) {
                    var prevLineChange = lineChanges[i - 1];
                    if (prevLineChange.originalEndLineNumber === 0) {
                        minOriginal = prevLineChange.originalStartLineNumber + 1;
                    }
                    else {
                        minOriginal = prevLineChange.originalEndLineNumber + 1;
                    }
                    if (prevLineChange.modifiedEndLineNumber === 0) {
                        minModified = prevLineChange.modifiedStartLineNumber + 1;
                    }
                    else {
                        minModified = prevLineChange.modifiedEndLineNumber + 1;
                    }
                }
                var fromOriginal = originalEqualAbove - DIFF_LINES_PADDING + 1;
                var fromModified = modifiedEqualAbove - DIFF_LINES_PADDING + 1;
                if (fromOriginal < minOriginal) {
                    var delta = minOriginal - fromOriginal;
                    fromOriginal = fromOriginal + delta;
                    fromModified = fromModified + delta;
                }
                if (fromModified < minModified) {
                    var delta = minModified - fromModified;
                    fromOriginal = fromOriginal + delta;
                    fromModified = fromModified + delta;
                }
                r_1[rLength_1++] = new DiffEntry(fromOriginal, originalEqualAbove, fromModified, modifiedEqualAbove);
            }
            // Emit deleted lines
            {
                if (originalEnd !== 0) {
                    r_1[rLength_1++] = new DiffEntry(originalStart, originalEnd, 0, 0);
                }
            }
            // Emit inserted lines
            {
                if (modifiedEnd !== 0) {
                    r_1[rLength_1++] = new DiffEntry(0, 0, modifiedStart, modifiedEnd);
                }
            }
            // Emit after anchors
            {
                var originalEqualBelow = (originalEnd === 0 ? originalStart + 1 : originalEnd + 1);
                var modifiedEqualBelow = (modifiedEnd === 0 ? modifiedStart + 1 : modifiedEnd + 1);
                // Make sure we don't step into the next diff
                var maxOriginal = originalLineCount;
                var maxModified = modifiedLineCount;
                if (i + 1 < len) {
                    var nextLineChange = lineChanges[i + 1];
                    if (nextLineChange.originalEndLineNumber === 0) {
                        maxOriginal = nextLineChange.originalStartLineNumber;
                    }
                    else {
                        maxOriginal = nextLineChange.originalStartLineNumber - 1;
                    }
                    if (nextLineChange.modifiedEndLineNumber === 0) {
                        maxModified = nextLineChange.modifiedStartLineNumber;
                    }
                    else {
                        maxModified = nextLineChange.modifiedStartLineNumber - 1;
                    }
                }
                var toOriginal = originalEqualBelow + DIFF_LINES_PADDING - 1;
                var toModified = modifiedEqualBelow + DIFF_LINES_PADDING - 1;
                if (toOriginal > maxOriginal) {
                    var delta = maxOriginal - toOriginal;
                    toOriginal = toOriginal + delta;
                    toModified = toModified + delta;
                }
                if (toModified > maxModified) {
                    var delta = maxModified - toModified;
                    toOriginal = toOriginal + delta;
                    toModified = toModified + delta;
                }
                r_1[rLength_1++] = new DiffEntry(originalEqualBelow, toOriginal, modifiedEqualBelow, toModified);
            }
            diffs[diffsLength++] = new Diff(r_1);
        }
        // Merge adjacent diffs
        var curr = diffs[0].entries;
        var r = [], rLength = 0;
        for (var i = 1, len = diffs.length; i < len; i++) {
            var thisDiff = diffs[i].entries;
            var currLast = curr[curr.length - 1];
            var thisFirst = thisDiff[0];
            if (currLast.getType() === 0 /* Equal */
                && thisFirst.getType() === 0 /* Equal */
                && thisFirst.originalLineStart <= currLast.originalLineEnd) {
                // We are dealing with equal lines that overlap
                curr[curr.length - 1] = new DiffEntry(currLast.originalLineStart, thisFirst.originalLineEnd, currLast.modifiedLineStart, thisFirst.modifiedLineEnd);
                curr = curr.concat(thisDiff.slice(1));
                continue;
            }
            r[rLength++] = new Diff(curr);
            curr = thisDiff;
        }
        r[rLength++] = new Diff(curr);
        return r;
    };
    DiffReview.prototype._findDiffIndex = function (pos) {
        var lineNumber = pos.lineNumber;
        for (var i = 0, len = this._diffs.length; i < len; i++) {
            var diff = this._diffs[i].entries;
            var lastModifiedLine = diff[diff.length - 1].modifiedLineEnd;
            if (lineNumber <= lastModifiedLine) {
                return i;
            }
        }
        return 0;
    };
    DiffReview.prototype._render = function () {
        var originalOpts = this._diffEditor.getOriginalEditor().getConfiguration();
        var modifiedOpts = this._diffEditor.getModifiedEditor().getConfiguration();
        var originalModel = this._diffEditor.getOriginalEditor().getModel();
        var modifiedModel = this._diffEditor.getModifiedEditor().getModel();
        var originalModelOpts = originalModel.getOptions();
        var modifiedModelOpts = modifiedModel.getOptions();
        if (!this._isVisible || !originalModel || !modifiedModel) {
            clearNode(this._content.domNode);
            this._currentDiff = null;
            this.scrollbar.scanDomNode();
            return;
        }
        var pos = this._diffEditor.getPosition();
        var diffIndex = this._findDiffIndex(pos);
        if (this._diffs[diffIndex] === this._currentDiff) {
            return;
        }
        this._currentDiff = this._diffs[diffIndex];
        var diffs = this._diffs[diffIndex].entries;
        var container = document.createElement('div');
        container.className = 'diff-review-table';
        container.setAttribute('role', 'list');
        Configuration.applyFontInfoSlow(container, modifiedOpts.fontInfo);
        var minOriginalLine = 0;
        var maxOriginalLine = 0;
        var minModifiedLine = 0;
        var maxModifiedLine = 0;
        for (var i = 0, len = diffs.length; i < len; i++) {
            var diffEntry = diffs[i];
            var originalLineStart = diffEntry.originalLineStart;
            var originalLineEnd = diffEntry.originalLineEnd;
            var modifiedLineStart = diffEntry.modifiedLineStart;
            var modifiedLineEnd = diffEntry.modifiedLineEnd;
            if (originalLineStart !== 0 && ((minOriginalLine === 0 || originalLineStart < minOriginalLine))) {
                minOriginalLine = originalLineStart;
            }
            if (originalLineEnd !== 0 && ((maxOriginalLine === 0 || originalLineEnd > maxOriginalLine))) {
                maxOriginalLine = originalLineEnd;
            }
            if (modifiedLineStart !== 0 && ((minModifiedLine === 0 || modifiedLineStart < minModifiedLine))) {
                minModifiedLine = modifiedLineStart;
            }
            if (modifiedLineEnd !== 0 && ((maxModifiedLine === 0 || modifiedLineEnd > maxModifiedLine))) {
                maxModifiedLine = modifiedLineEnd;
            }
        }
        var header = document.createElement('div');
        header.className = 'diff-review-row';
        var cell = document.createElement('div');
        cell.className = 'diff-review-cell diff-review-summary';
        var originalChangedLinesCnt = maxOriginalLine - minOriginalLine + 1;
        var modifiedChangedLinesCnt = maxModifiedLine - minModifiedLine + 1;
        cell.appendChild(document.createTextNode(diffIndex + 1 + "/" + this._diffs.length + ": @@ -" + minOriginalLine + "," + originalChangedLinesCnt + " +" + minModifiedLine + "," + modifiedChangedLinesCnt + " @@"));
        header.setAttribute('data-line', String(minModifiedLine));
        var getAriaLines = function (lines) {
            if (lines === 0) {
                return localize('no_lines', "no lines");
            }
            else if (lines === 1) {
                return localize('one_line', "1 line");
            }
            else {
                return localize('more_lines', "{0} lines", lines);
            }
        };
        var originalChangedLinesCntAria = getAriaLines(originalChangedLinesCnt);
        var modifiedChangedLinesCntAria = getAriaLines(modifiedChangedLinesCnt);
        header.setAttribute('aria-label', localize({
            key: 'header',
            comment: [
                'This is the ARIA label for a git diff header.',
                'A git diff header looks like this: @@ -154,12 +159,39 @@.',
                'That encodes that at original line 154 (which is now line 159), 12 lines were removed/changed with 39 lines.',
                'Variables 0 and 1 refer to the diff index out of total number of diffs.',
                'Variables 2 and 4 will be numbers (a line number).',
                'Variables 3 and 5 will be "no lines", "1 line" or "X lines", localized separately.'
            ]
        }, "Difference {0} of {1}: original {2}, {3}, modified {4}, {5}", (diffIndex + 1), this._diffs.length, minOriginalLine, originalChangedLinesCntAria, minModifiedLine, modifiedChangedLinesCntAria));
        header.appendChild(cell);
        // @@ -504,7 +517,7 @@
        header.setAttribute('role', 'listitem');
        container.appendChild(header);
        var modLine = minModifiedLine;
        for (var i = 0, len = diffs.length; i < len; i++) {
            var diffEntry = diffs[i];
            DiffReview._renderSection(container, diffEntry, modLine, this._width, originalOpts, originalModel, originalModelOpts, modifiedOpts, modifiedModel, modifiedModelOpts);
            if (diffEntry.modifiedLineStart !== 0) {
                modLine = diffEntry.modifiedLineEnd;
            }
        }
        clearNode(this._content.domNode);
        this._content.domNode.appendChild(container);
        this.scrollbar.scanDomNode();
    };
    DiffReview._renderSection = function (dest, diffEntry, modLine, width, originalOpts, originalModel, originalModelOpts, modifiedOpts, modifiedModel, modifiedModelOpts) {
        var type = diffEntry.getType();
        var rowClassName = 'diff-review-row';
        var lineNumbersExtraClassName = '';
        var spacerClassName = 'diff-review-spacer';
        switch (type) {
            case 1 /* Insert */:
                rowClassName = 'diff-review-row line-insert';
                lineNumbersExtraClassName = ' char-insert';
                spacerClassName = 'diff-review-spacer insert-sign';
                break;
            case 2 /* Delete */:
                rowClassName = 'diff-review-row line-delete';
                lineNumbersExtraClassName = ' char-delete';
                spacerClassName = 'diff-review-spacer delete-sign';
                break;
        }
        var originalLineStart = diffEntry.originalLineStart;
        var originalLineEnd = diffEntry.originalLineEnd;
        var modifiedLineStart = diffEntry.modifiedLineStart;
        var modifiedLineEnd = diffEntry.modifiedLineEnd;
        var cnt = Math.max(modifiedLineEnd - modifiedLineStart, originalLineEnd - originalLineStart);
        var originalLineNumbersWidth = originalOpts.layoutInfo.glyphMarginWidth + originalOpts.layoutInfo.lineNumbersWidth;
        var modifiedLineNumbersWidth = 10 + modifiedOpts.layoutInfo.glyphMarginWidth + modifiedOpts.layoutInfo.lineNumbersWidth;
        for (var i = 0; i <= cnt; i++) {
            var originalLine = (originalLineStart === 0 ? 0 : originalLineStart + i);
            var modifiedLine = (modifiedLineStart === 0 ? 0 : modifiedLineStart + i);
            var row = document.createElement('div');
            row.style.minWidth = width + 'px';
            row.className = rowClassName;
            row.setAttribute('role', 'listitem');
            if (modifiedLine !== 0) {
                modLine = modifiedLine;
            }
            row.setAttribute('data-line', String(modLine));
            var cell = document.createElement('div');
            cell.className = 'diff-review-cell';
            row.appendChild(cell);
            var originalLineNumber = document.createElement('span');
            originalLineNumber.style.width = (originalLineNumbersWidth + 'px');
            originalLineNumber.style.minWidth = (originalLineNumbersWidth + 'px');
            originalLineNumber.className = 'diff-review-line-number' + lineNumbersExtraClassName;
            if (originalLine !== 0) {
                originalLineNumber.appendChild(document.createTextNode(String(originalLine)));
            }
            else {
                originalLineNumber.innerHTML = '&nbsp;';
            }
            cell.appendChild(originalLineNumber);
            var modifiedLineNumber = document.createElement('span');
            modifiedLineNumber.style.width = (modifiedLineNumbersWidth + 'px');
            modifiedLineNumber.style.minWidth = (modifiedLineNumbersWidth + 'px');
            modifiedLineNumber.style.paddingRight = '10px';
            modifiedLineNumber.className = 'diff-review-line-number' + lineNumbersExtraClassName;
            if (modifiedLine !== 0) {
                modifiedLineNumber.appendChild(document.createTextNode(String(modifiedLine)));
            }
            else {
                modifiedLineNumber.innerHTML = '&nbsp;';
            }
            cell.appendChild(modifiedLineNumber);
            var spacer = document.createElement('span');
            spacer.className = spacerClassName;
            spacer.innerHTML = '&nbsp;&nbsp;';
            cell.appendChild(spacer);
            var lineContent = void 0;
            if (modifiedLine !== 0) {
                cell.insertAdjacentHTML('beforeend', this._renderLine(modifiedModel, modifiedOpts, modifiedModelOpts.tabSize, modifiedLine));
                lineContent = modifiedModel.getLineContent(modifiedLine);
            }
            else {
                cell.insertAdjacentHTML('beforeend', this._renderLine(originalModel, originalOpts, originalModelOpts.tabSize, originalLine));
                lineContent = originalModel.getLineContent(originalLine);
            }
            if (lineContent.length === 0) {
                lineContent = localize('blankLine', "blank");
            }
            var ariaLabel = void 0;
            switch (type) {
                case 0 /* Equal */:
                    ariaLabel = localize('equalLine', "original {0}, modified {1}: {2}", originalLine, modifiedLine, lineContent);
                    break;
                case 1 /* Insert */:
                    ariaLabel = localize('insertLine', "+ modified {0}: {1}", modifiedLine, lineContent);
                    break;
                case 2 /* Delete */:
                    ariaLabel = localize('deleteLine', "- original {0}: {1}", originalLine, lineContent);
                    break;
            }
            row.setAttribute('aria-label', ariaLabel);
            dest.appendChild(row);
        }
    };
    DiffReview._renderLine = function (model, config, tabSize, lineNumber) {
        var lineContent = model.getLineContent(lineNumber);
        var defaultMetadata = ((0 /* None */ << 11 /* FONT_STYLE_OFFSET */)
            | (1 /* DefaultForeground */ << 14 /* FOREGROUND_OFFSET */)
            | (2 /* DefaultBackground */ << 23 /* BACKGROUND_OFFSET */)) >>> 0;
        var tokens = new Uint32Array(2);
        tokens[0] = lineContent.length;
        tokens[1] = defaultMetadata;
        var lineTokens = new LineTokens(tokens, lineContent);
        var isBasicASCII = ViewLineRenderingData.isBasicASCII(lineContent, model.mightContainNonBasicASCII());
        var containsRTL = ViewLineRenderingData.containsRTL(lineContent, isBasicASCII, model.mightContainRTL());
        var r = renderViewLine2(new RenderLineInput((config.fontInfo.isMonospace && !config.viewInfo.disableMonospaceOptimizations), config.fontInfo.canUseHalfwidthRightwardsArrow, lineContent, false, isBasicASCII, containsRTL, 0, lineTokens, [], tabSize, config.fontInfo.spaceWidth, config.viewInfo.stopRenderingLineAfter, config.viewInfo.renderWhitespace, config.viewInfo.renderControlCharacters, config.viewInfo.fontLigatures));
        return r.html;
    };
    return DiffReview;
}(Disposable));
// theming
registerThemingParticipant(function (theme, collector) {
    var lineNumbers = theme.getColor(editorLineNumbers);
    if (lineNumbers) {
        collector.addRule(".monaco-diff-editor .diff-review-line-number { color: " + lineNumbers + "; }");
    }
    var shadow = theme.getColor(scrollbarShadow);
    if (shadow) {
        collector.addRule(".monaco-diff-editor .diff-review-shadow { box-shadow: " + shadow + " 0 -6px 6px -6px inset; }");
    }
});
var DiffReviewNext = /** @class */ (function (_super) {
    __extends$19(DiffReviewNext, _super);
    function DiffReviewNext() {
        return _super.call(this, {
            id: 'editor.action.diffReview.next',
            label: localize('editor.action.diffReview.next', "Go to Next Difference"),
            alias: 'Go to Next Difference',
            precondition: ContextKeyExpr.has('isInDiffEditor'),
            kbOpts: {
                kbExpr: null,
                primary: 65 /* F7 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    DiffReviewNext.prototype.run = function (accessor, editor) {
        var diffEditor = findFocusedDiffEditor(accessor);
        if (diffEditor) {
            diffEditor.diffReviewNext();
        }
    };
    return DiffReviewNext;
}(EditorAction));
var DiffReviewPrev = /** @class */ (function (_super) {
    __extends$19(DiffReviewPrev, _super);
    function DiffReviewPrev() {
        return _super.call(this, {
            id: 'editor.action.diffReview.prev',
            label: localize('editor.action.diffReview.prev', "Go to Previous Difference"),
            alias: 'Go to Previous Difference',
            precondition: ContextKeyExpr.has('isInDiffEditor'),
            kbOpts: {
                kbExpr: null,
                primary: 1024 /* Shift */ | 65 /* F7 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    DiffReviewPrev.prototype.run = function (accessor, editor) {
        var diffEditor = findFocusedDiffEditor(accessor);
        if (diffEditor) {
            diffEditor.diffReviewPrev();
        }
    };
    return DiffReviewPrev;
}(EditorAction));
function findFocusedDiffEditor(accessor) {
    var codeEditorService = accessor.get(ICodeEditorService);
    var diffEditors = codeEditorService.listDiffEditors();
    for (var i = 0, len = diffEditors.length; i < len; i++) {
        var diffEditor = diffEditors[i];
        if (diffEditor.hasWidgetFocus()) {
            return diffEditor;
        }
    }
    return null;
}
registerEditorAction(DiffReviewNext);
registerEditorAction(DiffReviewPrev);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ID_EDITOR_WORKER_SERVICE = 'editorWorkerService';
var IEditorWorkerService = createDecorator(ID_EDITOR_WORKER_SERVICE);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1a = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$3 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$2 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var VisualEditorState = /** @class */ (function () {
    function VisualEditorState() {
        this._zones = [];
        this._zonesMap = {};
        this._decorations = [];
    }
    VisualEditorState.prototype.getForeignViewZones = function (allViewZones) {
        var _this = this;
        return allViewZones.filter(function (z) { return !_this._zonesMap[String(z.id)]; });
    };
    VisualEditorState.prototype.clean = function (editor) {
        var _this = this;
        // (1) View zones
        if (this._zones.length > 0) {
            editor.changeViewZones(function (viewChangeAccessor) {
                for (var i = 0, length_1 = _this._zones.length; i < length_1; i++) {
                    viewChangeAccessor.removeZone(_this._zones[i]);
                }
            });
        }
        this._zones = [];
        this._zonesMap = {};
        // (2) Model decorations
        this._decorations = editor.deltaDecorations(this._decorations, []);
    };
    VisualEditorState.prototype.apply = function (editor, overviewRuler, newDecorations, restoreScrollState) {
        var _this = this;
        var scrollState = restoreScrollState ? StableEditorScrollState.capture(editor) : null;
        // view zones
        editor.changeViewZones(function (viewChangeAccessor) {
            for (var i = 0, length_2 = _this._zones.length; i < length_2; i++) {
                viewChangeAccessor.removeZone(_this._zones[i]);
            }
            _this._zones = [];
            _this._zonesMap = {};
            for (var i = 0, length_3 = newDecorations.zones.length; i < length_3; i++) {
                newDecorations.zones[i].suppressMouseDown = true;
                var zoneId = viewChangeAccessor.addZone(newDecorations.zones[i]);
                _this._zones.push(zoneId);
                _this._zonesMap[String(zoneId)] = true;
            }
        });
        if (scrollState) {
            scrollState.restore(editor);
        }
        // decorations
        this._decorations = editor.deltaDecorations(this._decorations, newDecorations.decorations);
        // overview ruler
        if (overviewRuler) {
            overviewRuler.setZones(newDecorations.overviewZones);
        }
    };
    return VisualEditorState;
}());
var DIFF_EDITOR_ID = 0;
var DiffEditorWidget = /** @class */ (function (_super) {
    __extends$1a(DiffEditorWidget, _super);
    function DiffEditorWidget(domElement, options, editorWorkerService, contextKeyService, instantiationService, codeEditorService, themeService, notificationService) {
        var _this = _super.call(this) || this;
        _this._onDidDispose = _this._register(new Emitter());
        _this.onDidDispose = _this._onDidDispose.event;
        _this._onDidUpdateDiff = _this._register(new Emitter());
        _this.onDidUpdateDiff = _this._onDidUpdateDiff.event;
        _this._lastOriginalWarning = null;
        _this._lastModifiedWarning = null;
        _this._editorWorkerService = editorWorkerService;
        _this._codeEditorService = codeEditorService;
        _this._contextKeyService = _this._register(contextKeyService.createScoped(domElement));
        _this._contextKeyService.createKey('isInDiffEditor', true);
        _this._themeService = themeService;
        _this._notificationService = notificationService;
        _this.id = (++DIFF_EDITOR_ID);
        _this._domElement = domElement;
        options = options || {};
        // renderSideBySide
        _this._renderSideBySide = true;
        if (typeof options.renderSideBySide !== 'undefined') {
            _this._renderSideBySide = options.renderSideBySide;
        }
        // ignoreTrimWhitespace
        _this._ignoreTrimWhitespace = true;
        if (typeof options.ignoreTrimWhitespace !== 'undefined') {
            _this._ignoreTrimWhitespace = options.ignoreTrimWhitespace;
        }
        // renderIndicators
        _this._renderIndicators = true;
        if (typeof options.renderIndicators !== 'undefined') {
            _this._renderIndicators = options.renderIndicators;
        }
        _this._originalIsEditable = false;
        if (typeof options.originalEditable !== 'undefined') {
            _this._originalIsEditable = Boolean(options.originalEditable);
        }
        _this._updateDecorationsRunner = _this._register(new RunOnceScheduler(function () { return _this._updateDecorations(); }, 0));
        _this._containerDomElement = document.createElement('div');
        _this._containerDomElement.className = DiffEditorWidget._getClassName(_this._themeService.getTheme(), _this._renderSideBySide);
        _this._containerDomElement.style.position = 'relative';
        _this._containerDomElement.style.height = '100%';
        _this._domElement.appendChild(_this._containerDomElement);
        _this._overviewViewportDomElement = createFastDomNode(document.createElement('div'));
        _this._overviewViewportDomElement.setClassName('diffViewport');
        _this._overviewViewportDomElement.setPosition('absolute');
        _this._overviewDomElement = document.createElement('div');
        _this._overviewDomElement.className = 'diffOverview';
        _this._overviewDomElement.style.position = 'absolute';
        _this._overviewDomElement.appendChild(_this._overviewViewportDomElement.domNode);
        _this._register(addStandardDisposableListener(_this._overviewDomElement, 'mousedown', function (e) {
            _this.modifiedEditor.delegateVerticalScrollbarMouseDown(e);
        }));
        _this._containerDomElement.appendChild(_this._overviewDomElement);
        _this._createLeftHandSide();
        _this._createRightHandSide();
        _this._beginUpdateDecorationsTimeout = -1;
        _this._currentlyChangingViewZones = false;
        _this._diffComputationToken = 0;
        _this._originalEditorState = new VisualEditorState();
        _this._modifiedEditorState = new VisualEditorState();
        _this._isVisible = true;
        _this._isHandlingScrollEvent = false;
        _this._width = 0;
        _this._height = 0;
        _this._reviewHeight = 0;
        _this._diffComputationResult = null;
        var leftContextKeyService = _this._contextKeyService.createScoped();
        leftContextKeyService.createKey('isInDiffLeftEditor', true);
        var leftServices = new ServiceCollection();
        leftServices.set(IContextKeyService, leftContextKeyService);
        var leftScopedInstantiationService = instantiationService.createChild(leftServices);
        var rightContextKeyService = _this._contextKeyService.createScoped();
        rightContextKeyService.createKey('isInDiffRightEditor', true);
        var rightServices = new ServiceCollection();
        rightServices.set(IContextKeyService, rightContextKeyService);
        var rightScopedInstantiationService = instantiationService.createChild(rightServices);
        _this._createLeftHandSideEditor(options, leftScopedInstantiationService);
        _this._createRightHandSideEditor(options, rightScopedInstantiationService);
        _this._reviewPane = new DiffReview(_this);
        _this._containerDomElement.appendChild(_this._reviewPane.domNode.domNode);
        _this._containerDomElement.appendChild(_this._reviewPane.shadow.domNode);
        _this._containerDomElement.appendChild(_this._reviewPane.actionBarContainer.domNode);
        if (options.automaticLayout) {
            _this._measureDomElementToken = window.setInterval(function () { return _this._measureDomElement(false); }, 100);
        }
        // enableSplitViewResizing
        _this._enableSplitViewResizing = true;
        if (typeof options.enableSplitViewResizing !== 'undefined') {
            _this._enableSplitViewResizing = options.enableSplitViewResizing;
        }
        if (_this._renderSideBySide) {
            _this._setStrategy(new DiffEdtorWidgetSideBySide(_this._createDataSource(), _this._enableSplitViewResizing));
        }
        else {
            _this._setStrategy(new DiffEdtorWidgetInline(_this._createDataSource(), _this._enableSplitViewResizing));
        }
        _this._register(themeService.onThemeChange(function (t) {
            if (_this._strategy && _this._strategy.applyColors(t)) {
                _this._updateDecorationsRunner.schedule();
            }
            _this._containerDomElement.className = DiffEditorWidget._getClassName(_this._themeService.getTheme(), _this._renderSideBySide);
        }));
        _this._codeEditorService.addDiffEditor(_this);
        return _this;
    }
    DiffEditorWidget.prototype.hasWidgetFocus = function () {
        return isAncestor(document.activeElement, this._domElement);
    };
    DiffEditorWidget.prototype.diffReviewNext = function () {
        this._reviewPane.next();
    };
    DiffEditorWidget.prototype.diffReviewPrev = function () {
        this._reviewPane.prev();
    };
    DiffEditorWidget._getClassName = function (theme, renderSideBySide) {
        var result = 'monaco-diff-editor monaco-editor-background ';
        if (renderSideBySide) {
            result += 'side-by-side ';
        }
        result += getThemeTypeSelector(theme.type);
        return result;
    };
    DiffEditorWidget.prototype._recreateOverviewRulers = function () {
        if (this._originalOverviewRuler) {
            this._overviewDomElement.removeChild(this._originalOverviewRuler.getDomNode());
            this._originalOverviewRuler.dispose();
        }
        this._originalOverviewRuler = this.originalEditor.createOverviewRuler('original diffOverviewRuler');
        this._overviewDomElement.appendChild(this._originalOverviewRuler.getDomNode());
        if (this._modifiedOverviewRuler) {
            this._overviewDomElement.removeChild(this._modifiedOverviewRuler.getDomNode());
            this._modifiedOverviewRuler.dispose();
        }
        this._modifiedOverviewRuler = this.modifiedEditor.createOverviewRuler('modified diffOverviewRuler');
        this._overviewDomElement.appendChild(this._modifiedOverviewRuler.getDomNode());
        this._layoutOverviewRulers();
    };
    DiffEditorWidget.prototype._createLeftHandSide = function () {
        this._originalDomNode = document.createElement('div');
        this._originalDomNode.className = 'editor original';
        this._originalDomNode.style.position = 'absolute';
        this._originalDomNode.style.height = '100%';
        this._containerDomElement.appendChild(this._originalDomNode);
    };
    DiffEditorWidget.prototype._createRightHandSide = function () {
        this._modifiedDomNode = document.createElement('div');
        this._modifiedDomNode.className = 'editor modified';
        this._modifiedDomNode.style.position = 'absolute';
        this._modifiedDomNode.style.height = '100%';
        this._containerDomElement.appendChild(this._modifiedDomNode);
    };
    DiffEditorWidget.prototype._createLeftHandSideEditor = function (options, instantiationService) {
        var _this = this;
        this.originalEditor = this._createInnerEditor(instantiationService, this._originalDomNode, this._adjustOptionsForLeftHandSide(options, this._originalIsEditable));
        this._register(this.originalEditor.onDidScrollChange(function (e) {
            if (_this._isHandlingScrollEvent) {
                return;
            }
            if (!e.scrollTopChanged && !e.scrollLeftChanged && !e.scrollHeightChanged) {
                return;
            }
            _this._isHandlingScrollEvent = true;
            _this.modifiedEditor.setScrollPosition({
                scrollLeft: e.scrollLeft,
                scrollTop: e.scrollTop
            });
            _this._isHandlingScrollEvent = false;
            _this._layoutOverviewViewport();
        }));
        this._register(this.originalEditor.onDidChangeViewZones(function () {
            _this._onViewZonesChanged();
        }));
        this._register(this.originalEditor.onDidChangeModelContent(function () {
            if (_this._isVisible) {
                _this._beginUpdateDecorationsSoon();
            }
        }));
    };
    DiffEditorWidget.prototype._createRightHandSideEditor = function (options, instantiationService) {
        var _this = this;
        this.modifiedEditor = this._createInnerEditor(instantiationService, this._modifiedDomNode, this._adjustOptionsForRightHandSide(options));
        this._register(this.modifiedEditor.onDidScrollChange(function (e) {
            if (_this._isHandlingScrollEvent) {
                return;
            }
            if (!e.scrollTopChanged && !e.scrollLeftChanged && !e.scrollHeightChanged) {
                return;
            }
            _this._isHandlingScrollEvent = true;
            _this.originalEditor.setScrollPosition({
                scrollLeft: e.scrollLeft,
                scrollTop: e.scrollTop
            });
            _this._isHandlingScrollEvent = false;
            _this._layoutOverviewViewport();
        }));
        this._register(this.modifiedEditor.onDidChangeViewZones(function () {
            _this._onViewZonesChanged();
        }));
        this._register(this.modifiedEditor.onDidChangeConfiguration(function (e) {
            if (e.fontInfo && _this.modifiedEditor.getModel()) {
                _this._onViewZonesChanged();
            }
        }));
        this._register(this.modifiedEditor.onDidChangeModelContent(function () {
            if (_this._isVisible) {
                _this._beginUpdateDecorationsSoon();
            }
        }));
    };
    DiffEditorWidget.prototype._createInnerEditor = function (instantiationService, container, options) {
        return instantiationService.createInstance(CodeEditorWidget, container, options, {});
    };
    DiffEditorWidget.prototype.dispose = function () {
        this._codeEditorService.removeDiffEditor(this);
        if (this._beginUpdateDecorationsTimeout !== -1) {
            window.clearTimeout(this._beginUpdateDecorationsTimeout);
            this._beginUpdateDecorationsTimeout = -1;
        }
        window.clearInterval(this._measureDomElementToken);
        this._cleanViewZonesAndDecorations();
        if (this._originalOverviewRuler) {
            this._overviewDomElement.removeChild(this._originalOverviewRuler.getDomNode());
            this._originalOverviewRuler.dispose();
        }
        if (this._modifiedOverviewRuler) {
            this._overviewDomElement.removeChild(this._modifiedOverviewRuler.getDomNode());
            this._modifiedOverviewRuler.dispose();
        }
        this._overviewDomElement.removeChild(this._overviewViewportDomElement.domNode);
        this._containerDomElement.removeChild(this._overviewDomElement);
        this._containerDomElement.removeChild(this._originalDomNode);
        this.originalEditor.dispose();
        this._containerDomElement.removeChild(this._modifiedDomNode);
        this.modifiedEditor.dispose();
        this._strategy.dispose();
        this._containerDomElement.removeChild(this._reviewPane.domNode.domNode);
        this._containerDomElement.removeChild(this._reviewPane.shadow.domNode);
        this._containerDomElement.removeChild(this._reviewPane.actionBarContainer.domNode);
        this._reviewPane.dispose();
        this._domElement.removeChild(this._containerDomElement);
        this._onDidDispose.fire();
        _super.prototype.dispose.call(this);
    };
    //------------ begin IDiffEditor methods
    DiffEditorWidget.prototype.getId = function () {
        return this.getEditorType() + ':' + this.id;
    };
    DiffEditorWidget.prototype.getEditorType = function () {
        return EditorType.IDiffEditor;
    };
    DiffEditorWidget.prototype.getLineChanges = function () {
        if (!this._diffComputationResult) {
            return null;
        }
        return this._diffComputationResult.changes;
    };
    DiffEditorWidget.prototype.getOriginalEditor = function () {
        return this.originalEditor;
    };
    DiffEditorWidget.prototype.getModifiedEditor = function () {
        return this.modifiedEditor;
    };
    DiffEditorWidget.prototype.updateOptions = function (newOptions) {
        // Handle side by side
        var renderSideBySideChanged = false;
        if (typeof newOptions.renderSideBySide !== 'undefined') {
            if (this._renderSideBySide !== newOptions.renderSideBySide) {
                this._renderSideBySide = newOptions.renderSideBySide;
                renderSideBySideChanged = true;
            }
        }
        var beginUpdateDecorations = false;
        if (typeof newOptions.ignoreTrimWhitespace !== 'undefined') {
            if (this._ignoreTrimWhitespace !== newOptions.ignoreTrimWhitespace) {
                this._ignoreTrimWhitespace = newOptions.ignoreTrimWhitespace;
                // Begin comparing
                beginUpdateDecorations = true;
            }
        }
        if (typeof newOptions.renderIndicators !== 'undefined') {
            if (this._renderIndicators !== newOptions.renderIndicators) {
                this._renderIndicators = newOptions.renderIndicators;
                beginUpdateDecorations = true;
            }
        }
        if (beginUpdateDecorations) {
            this._beginUpdateDecorations();
        }
        if (typeof newOptions.originalEditable !== 'undefined') {
            this._originalIsEditable = Boolean(newOptions.originalEditable);
        }
        this.modifiedEditor.updateOptions(this._adjustOptionsForRightHandSide(newOptions));
        this.originalEditor.updateOptions(this._adjustOptionsForLeftHandSide(newOptions, this._originalIsEditable));
        // enableSplitViewResizing
        if (typeof newOptions.enableSplitViewResizing !== 'undefined') {
            this._enableSplitViewResizing = newOptions.enableSplitViewResizing;
        }
        this._strategy.setEnableSplitViewResizing(this._enableSplitViewResizing);
        // renderSideBySide
        if (renderSideBySideChanged) {
            if (this._renderSideBySide) {
                this._setStrategy(new DiffEdtorWidgetSideBySide(this._createDataSource(), this._enableSplitViewResizing));
            }
            else {
                this._setStrategy(new DiffEdtorWidgetInline(this._createDataSource(), this._enableSplitViewResizing));
            }
            // Update class name
            this._containerDomElement.className = DiffEditorWidget._getClassName(this._themeService.getTheme(), this._renderSideBySide);
        }
    };
    DiffEditorWidget.prototype.getModel = function () {
        return {
            original: this.originalEditor.getModel(),
            modified: this.modifiedEditor.getModel()
        };
    };
    DiffEditorWidget.prototype.setModel = function (model) {
        // Guard us against partial null model
        if (model && (!model.original || !model.modified)) {
            throw new Error(!model.original ? 'DiffEditorWidget.setModel: Original model is null' : 'DiffEditorWidget.setModel: Modified model is null');
        }
        // Remove all view zones & decorations
        this._cleanViewZonesAndDecorations();
        // Update code editor models
        this.originalEditor.setModel(model ? model.original : null);
        this.modifiedEditor.setModel(model ? model.modified : null);
        this._updateDecorationsRunner.cancel();
        if (model) {
            this.originalEditor.setScrollTop(0);
            this.modifiedEditor.setScrollTop(0);
        }
        // Disable any diff computations that will come in
        this._diffComputationResult = null;
        this._diffComputationToken++;
        if (model) {
            this._recreateOverviewRulers();
            // Begin comparing
            this._beginUpdateDecorations();
        }
        else {
            this._diffComputationResult = null;
        }
        this._layoutOverviewViewport();
    };
    DiffEditorWidget.prototype.getDomNode = function () {
        return this._domElement;
    };
    DiffEditorWidget.prototype.getVisibleColumnFromPosition = function (position) {
        return this.modifiedEditor.getVisibleColumnFromPosition(position);
    };
    DiffEditorWidget.prototype.getPosition = function () {
        return this.modifiedEditor.getPosition();
    };
    DiffEditorWidget.prototype.setPosition = function (position) {
        this.modifiedEditor.setPosition(position);
    };
    DiffEditorWidget.prototype.revealLine = function (lineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealLine(lineNumber, scrollType);
    };
    DiffEditorWidget.prototype.revealLineInCenter = function (lineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealLineInCenter(lineNumber, scrollType);
    };
    DiffEditorWidget.prototype.revealLineInCenterIfOutsideViewport = function (lineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealLineInCenterIfOutsideViewport(lineNumber, scrollType);
    };
    DiffEditorWidget.prototype.revealPosition = function (position, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealPosition(position, scrollType);
    };
    DiffEditorWidget.prototype.revealPositionInCenter = function (position, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealPositionInCenter(position, scrollType);
    };
    DiffEditorWidget.prototype.revealPositionInCenterIfOutsideViewport = function (position, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealPositionInCenterIfOutsideViewport(position, scrollType);
    };
    DiffEditorWidget.prototype.getSelection = function () {
        return this.modifiedEditor.getSelection();
    };
    DiffEditorWidget.prototype.getSelections = function () {
        return this.modifiedEditor.getSelections();
    };
    DiffEditorWidget.prototype.setSelection = function (something) {
        this.modifiedEditor.setSelection(something);
    };
    DiffEditorWidget.prototype.setSelections = function (ranges) {
        this.modifiedEditor.setSelections(ranges);
    };
    DiffEditorWidget.prototype.revealLines = function (startLineNumber, endLineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealLines(startLineNumber, endLineNumber, scrollType);
    };
    DiffEditorWidget.prototype.revealLinesInCenter = function (startLineNumber, endLineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealLinesInCenter(startLineNumber, endLineNumber, scrollType);
    };
    DiffEditorWidget.prototype.revealLinesInCenterIfOutsideViewport = function (startLineNumber, endLineNumber, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealLinesInCenterIfOutsideViewport(startLineNumber, endLineNumber, scrollType);
    };
    DiffEditorWidget.prototype.revealRange = function (range, scrollType, revealVerticalInCenter, revealHorizontal) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
        if (revealHorizontal === void 0) { revealHorizontal = true; }
        this.modifiedEditor.revealRange(range, scrollType, revealVerticalInCenter, revealHorizontal);
    };
    DiffEditorWidget.prototype.revealRangeInCenter = function (range, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealRangeInCenter(range, scrollType);
    };
    DiffEditorWidget.prototype.revealRangeInCenterIfOutsideViewport = function (range, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealRangeInCenterIfOutsideViewport(range, scrollType);
    };
    DiffEditorWidget.prototype.revealRangeAtTop = function (range, scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this.modifiedEditor.revealRangeAtTop(range, scrollType);
    };
    DiffEditorWidget.prototype.getSupportedActions = function () {
        return this.modifiedEditor.getSupportedActions();
    };
    DiffEditorWidget.prototype.saveViewState = function () {
        var originalViewState = this.originalEditor.saveViewState();
        var modifiedViewState = this.modifiedEditor.saveViewState();
        return {
            original: originalViewState,
            modified: modifiedViewState
        };
    };
    DiffEditorWidget.prototype.restoreViewState = function (s) {
        if (s.original && s.original) {
            var diffEditorState = s;
            this.originalEditor.restoreViewState(diffEditorState.original);
            this.modifiedEditor.restoreViewState(diffEditorState.modified);
        }
    };
    DiffEditorWidget.prototype.layout = function (dimension) {
        this._measureDomElement(false, dimension);
    };
    DiffEditorWidget.prototype.focus = function () {
        this.modifiedEditor.focus();
    };
    DiffEditorWidget.prototype.hasTextFocus = function () {
        return this.originalEditor.hasTextFocus() || this.modifiedEditor.hasTextFocus();
    };
    DiffEditorWidget.prototype.trigger = function (source, handlerId, payload) {
        this.modifiedEditor.trigger(source, handlerId, payload);
    };
    DiffEditorWidget.prototype.changeDecorations = function (callback) {
        return this.modifiedEditor.changeDecorations(callback);
    };
    //------------ end IDiffEditor methods
    //------------ begin layouting methods
    DiffEditorWidget.prototype._measureDomElement = function (forceDoLayoutCall, dimensions) {
        dimensions = dimensions || {
            width: this._containerDomElement.clientWidth,
            height: this._containerDomElement.clientHeight
        };
        if (dimensions.width <= 0) {
            this._width = 0;
            this._height = 0;
            this._reviewHeight = 0;
            return;
        }
        if (!forceDoLayoutCall && dimensions.width === this._width && dimensions.height === this._height) {
            // Nothing has changed
            return;
        }
        this._width = dimensions.width;
        this._height = dimensions.height;
        this._reviewHeight = this._reviewPane.isVisible() ? this._height : 0;
        this._doLayout();
    };
    DiffEditorWidget.prototype._layoutOverviewRulers = function () {
        var freeSpace = DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH - 2 * DiffEditorWidget.ONE_OVERVIEW_WIDTH;
        var layoutInfo = this.modifiedEditor.getLayoutInfo();
        if (layoutInfo) {
            this._originalOverviewRuler.setLayout({
                top: 0,
                width: DiffEditorWidget.ONE_OVERVIEW_WIDTH,
                right: freeSpace + DiffEditorWidget.ONE_OVERVIEW_WIDTH,
                height: (this._height - this._reviewHeight)
            });
            this._modifiedOverviewRuler.setLayout({
                top: 0,
                right: 0,
                width: DiffEditorWidget.ONE_OVERVIEW_WIDTH,
                height: (this._height - this._reviewHeight)
            });
        }
    };
    //------------ end layouting methods
    DiffEditorWidget.prototype._onViewZonesChanged = function () {
        if (this._currentlyChangingViewZones) {
            return;
        }
        this._updateDecorationsRunner.schedule();
    };
    DiffEditorWidget.prototype._beginUpdateDecorationsSoon = function () {
        var _this = this;
        // Clear previous timeout if necessary
        if (this._beginUpdateDecorationsTimeout !== -1) {
            window.clearTimeout(this._beginUpdateDecorationsTimeout);
            this._beginUpdateDecorationsTimeout = -1;
        }
        this._beginUpdateDecorationsTimeout = window.setTimeout(function () { return _this._beginUpdateDecorations(); }, DiffEditorWidget.UPDATE_DIFF_DECORATIONS_DELAY);
    };
    DiffEditorWidget._equals = function (a, b) {
        if (!a && !b) {
            return true;
        }
        if (!a || !b) {
            return false;
        }
        return (a.toString() === b.toString());
    };
    DiffEditorWidget.prototype._beginUpdateDecorations = function () {
        var _this = this;
        this._beginUpdateDecorationsTimeout = -1;
        var currentOriginalModel = this.originalEditor.getModel();
        var currentModifiedModel = this.modifiedEditor.getModel();
        if (!currentOriginalModel || !currentModifiedModel) {
            return;
        }
        // Prevent old diff requests to come if a new request has been initiated
        // The best method would be to call cancel on the Promise, but this is not
        // yet supported, so using tokens for now.
        this._diffComputationToken++;
        var currentToken = this._diffComputationToken;
        if (!this._editorWorkerService.canComputeDiff(currentOriginalModel.uri, currentModifiedModel.uri)) {
            if (!DiffEditorWidget._equals(currentOriginalModel.uri, this._lastOriginalWarning)
                || !DiffEditorWidget._equals(currentModifiedModel.uri, this._lastModifiedWarning)) {
                this._lastOriginalWarning = currentOriginalModel.uri;
                this._lastModifiedWarning = currentModifiedModel.uri;
                this._notificationService.warn(localize("diff.tooLarge", "Cannot compare files because one file is too large."));
            }
            return;
        }
        this._editorWorkerService.computeDiff(currentOriginalModel.uri, currentModifiedModel.uri, this._ignoreTrimWhitespace).then(function (result) {
            if (currentToken === _this._diffComputationToken
                && currentOriginalModel === _this.originalEditor.getModel()
                && currentModifiedModel === _this.modifiedEditor.getModel()) {
                _this._diffComputationResult = result;
                _this._updateDecorationsRunner.schedule();
                _this._onDidUpdateDiff.fire();
            }
        }, function (error) {
            if (currentToken === _this._diffComputationToken
                && currentOriginalModel === _this.originalEditor.getModel()
                && currentModifiedModel === _this.modifiedEditor.getModel()) {
                _this._diffComputationResult = null;
                _this._updateDecorationsRunner.schedule();
            }
        });
    };
    DiffEditorWidget.prototype._cleanViewZonesAndDecorations = function () {
        this._originalEditorState.clean(this.originalEditor);
        this._modifiedEditorState.clean(this.modifiedEditor);
    };
    DiffEditorWidget.prototype._updateDecorations = function () {
        if (!this.originalEditor.getModel() || !this.modifiedEditor.getModel()) {
            return;
        }
        var lineChanges = (this._diffComputationResult ? this._diffComputationResult.changes : []);
        var foreignOriginal = this._originalEditorState.getForeignViewZones(this.originalEditor.getWhitespaces());
        var foreignModified = this._modifiedEditorState.getForeignViewZones(this.modifiedEditor.getWhitespaces());
        var diffDecorations = this._strategy.getEditorsDiffDecorations(lineChanges, this._ignoreTrimWhitespace, this._renderIndicators, foreignOriginal, foreignModified, this.originalEditor, this.modifiedEditor);
        try {
            this._currentlyChangingViewZones = true;
            this._originalEditorState.apply(this.originalEditor, this._originalOverviewRuler, diffDecorations.original, false);
            this._modifiedEditorState.apply(this.modifiedEditor, this._modifiedOverviewRuler, diffDecorations.modified, true);
        }
        finally {
            this._currentlyChangingViewZones = false;
        }
    };
    DiffEditorWidget.prototype._adjustOptionsForSubEditor = function (options) {
        var clonedOptions = deepClone(options || {});
        clonedOptions.inDiffEditor = true;
        clonedOptions.wordWrap = 'off';
        clonedOptions.wordWrapMinified = false;
        clonedOptions.automaticLayout = false;
        clonedOptions.scrollbar = clonedOptions.scrollbar || {};
        clonedOptions.scrollbar.vertical = 'visible';
        clonedOptions.folding = false;
        clonedOptions.codeLens = false;
        clonedOptions.fixedOverflowWidgets = true;
        // clonedOptions.lineDecorationsWidth = '2ch';
        if (!clonedOptions.minimap) {
            clonedOptions.minimap = {};
        }
        clonedOptions.minimap.enabled = false;
        return clonedOptions;
    };
    DiffEditorWidget.prototype._adjustOptionsForLeftHandSide = function (options, isEditable) {
        var result = this._adjustOptionsForSubEditor(options);
        result.readOnly = !isEditable;
        result.overviewRulerLanes = 1;
        result.extraEditorClassName = 'original-in-monaco-diff-editor';
        return result;
    };
    DiffEditorWidget.prototype._adjustOptionsForRightHandSide = function (options) {
        var result = this._adjustOptionsForSubEditor(options);
        result.revealHorizontalRightPadding = EDITOR_DEFAULTS.viewInfo.revealHorizontalRightPadding + DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH;
        result.scrollbar.verticalHasArrows = false;
        result.extraEditorClassName = 'modified-in-monaco-diff-editor';
        return result;
    };
    DiffEditorWidget.prototype.doLayout = function () {
        this._measureDomElement(true);
    };
    DiffEditorWidget.prototype._doLayout = function () {
        var splitPoint = this._strategy.layout();
        this._originalDomNode.style.width = splitPoint + 'px';
        this._originalDomNode.style.left = '0px';
        this._modifiedDomNode.style.width = (this._width - splitPoint) + 'px';
        this._modifiedDomNode.style.left = splitPoint + 'px';
        this._overviewDomElement.style.top = '0px';
        this._overviewDomElement.style.height = (this._height - this._reviewHeight) + 'px';
        this._overviewDomElement.style.width = DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH + 'px';
        this._overviewDomElement.style.left = (this._width - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH) + 'px';
        this._overviewViewportDomElement.setWidth(DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH);
        this._overviewViewportDomElement.setHeight(30);
        this.originalEditor.layout({ width: splitPoint, height: (this._height - this._reviewHeight) });
        this.modifiedEditor.layout({ width: this._width - splitPoint - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH, height: (this._height - this._reviewHeight) });
        if (this._originalOverviewRuler || this._modifiedOverviewRuler) {
            this._layoutOverviewRulers();
        }
        this._reviewPane.layout(this._height - this._reviewHeight, this._width, this._reviewHeight);
        this._layoutOverviewViewport();
    };
    DiffEditorWidget.prototype._layoutOverviewViewport = function () {
        var layout = this._computeOverviewViewport();
        if (!layout) {
            this._overviewViewportDomElement.setTop(0);
            this._overviewViewportDomElement.setHeight(0);
        }
        else {
            this._overviewViewportDomElement.setTop(layout.top);
            this._overviewViewportDomElement.setHeight(layout.height);
        }
    };
    DiffEditorWidget.prototype._computeOverviewViewport = function () {
        var layoutInfo = this.modifiedEditor.getLayoutInfo();
        if (!layoutInfo) {
            return null;
        }
        var scrollTop = this.modifiedEditor.getScrollTop();
        var scrollHeight = this.modifiedEditor.getScrollHeight();
        var computedAvailableSize = Math.max(0, layoutInfo.contentHeight);
        var computedRepresentableSize = Math.max(0, computedAvailableSize - 2 * 0);
        var computedRatio = scrollHeight > 0 ? (computedRepresentableSize / scrollHeight) : 0;
        var computedSliderSize = Math.max(0, Math.floor(layoutInfo.contentHeight * computedRatio));
        var computedSliderPosition = Math.floor(scrollTop * computedRatio);
        return {
            height: computedSliderSize,
            top: computedSliderPosition
        };
    };
    DiffEditorWidget.prototype._createDataSource = function () {
        var _this = this;
        return {
            getWidth: function () {
                return _this._width;
            },
            getHeight: function () {
                return (_this._height - _this._reviewHeight);
            },
            getContainerDomNode: function () {
                return _this._containerDomElement;
            },
            relayoutEditors: function () {
                _this._doLayout();
            },
            getOriginalEditor: function () {
                return _this.originalEditor;
            },
            getModifiedEditor: function () {
                return _this.modifiedEditor;
            }
        };
    };
    DiffEditorWidget.prototype._setStrategy = function (newStrategy) {
        if (this._strategy) {
            this._strategy.dispose();
        }
        this._strategy = newStrategy;
        newStrategy.applyColors(this._themeService.getTheme());
        if (this._diffComputationResult) {
            this._updateDecorations();
        }
        // Just do a layout, the strategy might need it
        this._measureDomElement(true);
    };
    DiffEditorWidget.prototype._getLineChangeAtOrBeforeLineNumber = function (lineNumber, startLineNumberExtractor) {
        var lineChanges = (this._diffComputationResult ? this._diffComputationResult.changes : []);
        if (lineChanges.length === 0 || lineNumber < startLineNumberExtractor(lineChanges[0])) {
            // There are no changes or `lineNumber` is before the first change
            return null;
        }
        var min = 0, max = lineChanges.length - 1;
        while (min < max) {
            var mid = Math.floor((min + max) / 2);
            var midStart = startLineNumberExtractor(lineChanges[mid]);
            var midEnd = (mid + 1 <= max ? startLineNumberExtractor(lineChanges[mid + 1]) : Number.MAX_VALUE);
            if (lineNumber < midStart) {
                max = mid - 1;
            }
            else if (lineNumber >= midEnd) {
                min = mid + 1;
            }
            else {
                // HIT!
                min = mid;
                max = mid;
            }
        }
        return lineChanges[min];
    };
    DiffEditorWidget.prototype._getEquivalentLineForOriginalLineNumber = function (lineNumber) {
        var lineChange = this._getLineChangeAtOrBeforeLineNumber(lineNumber, function (lineChange) { return lineChange.originalStartLineNumber; });
        if (!lineChange) {
            return lineNumber;
        }
        var originalEquivalentLineNumber = lineChange.originalStartLineNumber + (lineChange.originalEndLineNumber > 0 ? -1 : 0);
        var modifiedEquivalentLineNumber = lineChange.modifiedStartLineNumber + (lineChange.modifiedEndLineNumber > 0 ? -1 : 0);
        var lineChangeOriginalLength = (lineChange.originalEndLineNumber > 0 ? (lineChange.originalEndLineNumber - lineChange.originalStartLineNumber + 1) : 0);
        var lineChangeModifiedLength = (lineChange.modifiedEndLineNumber > 0 ? (lineChange.modifiedEndLineNumber - lineChange.modifiedStartLineNumber + 1) : 0);
        var delta = lineNumber - originalEquivalentLineNumber;
        if (delta <= lineChangeOriginalLength) {
            return modifiedEquivalentLineNumber + Math.min(delta, lineChangeModifiedLength);
        }
        return modifiedEquivalentLineNumber + lineChangeModifiedLength - lineChangeOriginalLength + delta;
    };
    DiffEditorWidget.prototype._getEquivalentLineForModifiedLineNumber = function (lineNumber) {
        var lineChange = this._getLineChangeAtOrBeforeLineNumber(lineNumber, function (lineChange) { return lineChange.modifiedStartLineNumber; });
        if (!lineChange) {
            return lineNumber;
        }
        var originalEquivalentLineNumber = lineChange.originalStartLineNumber + (lineChange.originalEndLineNumber > 0 ? -1 : 0);
        var modifiedEquivalentLineNumber = lineChange.modifiedStartLineNumber + (lineChange.modifiedEndLineNumber > 0 ? -1 : 0);
        var lineChangeOriginalLength = (lineChange.originalEndLineNumber > 0 ? (lineChange.originalEndLineNumber - lineChange.originalStartLineNumber + 1) : 0);
        var lineChangeModifiedLength = (lineChange.modifiedEndLineNumber > 0 ? (lineChange.modifiedEndLineNumber - lineChange.modifiedStartLineNumber + 1) : 0);
        var delta = lineNumber - modifiedEquivalentLineNumber;
        if (delta <= lineChangeModifiedLength) {
            return originalEquivalentLineNumber + Math.min(delta, lineChangeOriginalLength);
        }
        return originalEquivalentLineNumber + lineChangeOriginalLength - lineChangeModifiedLength + delta;
    };
    DiffEditorWidget.prototype.getDiffLineInformationForOriginal = function (lineNumber) {
        if (!this._diffComputationResult) {
            // Cannot answer that which I don't know
            return null;
        }
        return {
            equivalentLineNumber: this._getEquivalentLineForOriginalLineNumber(lineNumber)
        };
    };
    DiffEditorWidget.prototype.getDiffLineInformationForModified = function (lineNumber) {
        if (!this._diffComputationResult) {
            // Cannot answer that which I don't know
            return null;
        }
        return {
            equivalentLineNumber: this._getEquivalentLineForModifiedLineNumber(lineNumber)
        };
    };
    DiffEditorWidget.ONE_OVERVIEW_WIDTH = 15;
    DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH = 30;
    DiffEditorWidget.UPDATE_DIFF_DECORATIONS_DELAY = 200; // ms
    DiffEditorWidget = __decorate$3([
        __param$2(2, IEditorWorkerService),
        __param$2(3, IContextKeyService),
        __param$2(4, IInstantiationService),
        __param$2(5, ICodeEditorService),
        __param$2(6, IThemeService),
        __param$2(7, INotificationService)
    ], DiffEditorWidget);
    return DiffEditorWidget;
}(Disposable));
var DiffEditorWidgetStyle = /** @class */ (function (_super) {
    __extends$1a(DiffEditorWidgetStyle, _super);
    function DiffEditorWidgetStyle(dataSource) {
        var _this = _super.call(this) || this;
        _this._dataSource = dataSource;
        return _this;
    }
    DiffEditorWidgetStyle.prototype.applyColors = function (theme) {
        var newInsertColor = (theme.getColor(diffInserted) || defaultInsertColor).transparent(2);
        var newRemoveColor = (theme.getColor(diffRemoved) || defaultRemoveColor).transparent(2);
        var hasChanges = !newInsertColor.equals(this._insertColor) || !newRemoveColor.equals(this._removeColor);
        this._insertColor = newInsertColor;
        this._removeColor = newRemoveColor;
        return hasChanges;
    };
    DiffEditorWidgetStyle.prototype.getEditorsDiffDecorations = function (lineChanges, ignoreTrimWhitespace, renderIndicators, originalWhitespaces, modifiedWhitespaces, originalEditor, modifiedEditor) {
        // Get view zones
        modifiedWhitespaces = modifiedWhitespaces.sort(function (a, b) {
            return a.afterLineNumber - b.afterLineNumber;
        });
        originalWhitespaces = originalWhitespaces.sort(function (a, b) {
            return a.afterLineNumber - b.afterLineNumber;
        });
        var zones = this._getViewZones(lineChanges, originalWhitespaces, modifiedWhitespaces, originalEditor, modifiedEditor, renderIndicators);
        // Get decorations & overview ruler zones
        var originalDecorations = this._getOriginalEditorDecorations(lineChanges, ignoreTrimWhitespace, renderIndicators, originalEditor, modifiedEditor);
        var modifiedDecorations = this._getModifiedEditorDecorations(lineChanges, ignoreTrimWhitespace, renderIndicators, originalEditor, modifiedEditor);
        return {
            original: {
                decorations: originalDecorations.decorations,
                overviewZones: originalDecorations.overviewZones,
                zones: zones.original
            },
            modified: {
                decorations: modifiedDecorations.decorations,
                overviewZones: modifiedDecorations.overviewZones,
                zones: zones.modified
            }
        };
    };
    return DiffEditorWidgetStyle;
}(Disposable));
var ForeignViewZonesIterator = /** @class */ (function () {
    function ForeignViewZonesIterator(source) {
        this._source = source;
        this._index = -1;
        this.advance();
    }
    ForeignViewZonesIterator.prototype.advance = function () {
        this._index++;
        if (this._index < this._source.length) {
            this.current = this._source[this._index];
        }
        else {
            this.current = null;
        }
    };
    return ForeignViewZonesIterator;
}());
var ViewZonesComputer = /** @class */ (function () {
    function ViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ) {
        this.lineChanges = lineChanges;
        this.originalForeignVZ = originalForeignVZ;
        this.modifiedForeignVZ = modifiedForeignVZ;
    }
    ViewZonesComputer.prototype.getViewZones = function () {
        var result = {
            original: [],
            modified: []
        };
        var lineChangeModifiedLength = 0;
        var lineChangeOriginalLength = 0;
        var originalEquivalentLineNumber = 0;
        var modifiedEquivalentLineNumber = 0;
        var originalEndEquivalentLineNumber = 0;
        var modifiedEndEquivalentLineNumber = 0;
        var sortMyViewZones = function (a, b) {
            return a.afterLineNumber - b.afterLineNumber;
        };
        var addAndCombineIfPossible = function (destination, item) {
            if (item.domNode === null && destination.length > 0) {
                var lastItem = destination[destination.length - 1];
                if (lastItem.afterLineNumber === item.afterLineNumber && lastItem.domNode === null) {
                    lastItem.heightInLines += item.heightInLines;
                    return;
                }
            }
            destination.push(item);
        };
        var modifiedForeignVZ = new ForeignViewZonesIterator(this.modifiedForeignVZ);
        var originalForeignVZ = new ForeignViewZonesIterator(this.originalForeignVZ);
        // In order to include foreign view zones after the last line change, the for loop will iterate once more after the end of the `lineChanges` array
        for (var i = 0, length_4 = this.lineChanges.length; i <= length_4; i++) {
            var lineChange = (i < length_4 ? this.lineChanges[i] : null);
            if (lineChange !== null) {
                originalEquivalentLineNumber = lineChange.originalStartLineNumber + (lineChange.originalEndLineNumber > 0 ? -1 : 0);
                modifiedEquivalentLineNumber = lineChange.modifiedStartLineNumber + (lineChange.modifiedEndLineNumber > 0 ? -1 : 0);
                lineChangeOriginalLength = (lineChange.originalEndLineNumber > 0 ? (lineChange.originalEndLineNumber - lineChange.originalStartLineNumber + 1) : 0);
                lineChangeModifiedLength = (lineChange.modifiedEndLineNumber > 0 ? (lineChange.modifiedEndLineNumber - lineChange.modifiedStartLineNumber + 1) : 0);
                originalEndEquivalentLineNumber = Math.max(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber);
                modifiedEndEquivalentLineNumber = Math.max(lineChange.modifiedStartLineNumber, lineChange.modifiedEndLineNumber);
            }
            else {
                // Increase to very large value to get the producing tests of foreign view zones running
                originalEquivalentLineNumber += 10000000 + lineChangeOriginalLength;
                modifiedEquivalentLineNumber += 10000000 + lineChangeModifiedLength;
                originalEndEquivalentLineNumber = originalEquivalentLineNumber;
                modifiedEndEquivalentLineNumber = modifiedEquivalentLineNumber;
            }
            // Each step produces view zones, and after producing them, we try to cancel them out, to avoid empty-empty view zone cases
            var stepOriginal = [];
            var stepModified = [];
            // ---------------------------- PRODUCE VIEW ZONES
            // [PRODUCE] View zone(s) in original-side due to foreign view zone(s) in modified-side
            while (modifiedForeignVZ.current && modifiedForeignVZ.current.afterLineNumber <= modifiedEndEquivalentLineNumber) {
                var viewZoneLineNumber = void 0;
                if (modifiedForeignVZ.current.afterLineNumber <= modifiedEquivalentLineNumber) {
                    viewZoneLineNumber = originalEquivalentLineNumber - modifiedEquivalentLineNumber + modifiedForeignVZ.current.afterLineNumber;
                }
                else {
                    viewZoneLineNumber = originalEndEquivalentLineNumber;
                }
                var marginDomNode = null;
                if (lineChange && lineChange.modifiedStartLineNumber <= modifiedForeignVZ.current.afterLineNumber && modifiedForeignVZ.current.afterLineNumber <= lineChange.modifiedEndLineNumber) {
                    marginDomNode = this._createOriginalMarginDomNodeForModifiedForeignViewZoneInAddedRegion();
                }
                stepOriginal.push({
                    afterLineNumber: viewZoneLineNumber,
                    heightInLines: modifiedForeignVZ.current.heightInLines,
                    domNode: null,
                    marginDomNode: marginDomNode
                });
                modifiedForeignVZ.advance();
            }
            // [PRODUCE] View zone(s) in modified-side due to foreign view zone(s) in original-side
            while (originalForeignVZ.current && originalForeignVZ.current.afterLineNumber <= originalEndEquivalentLineNumber) {
                var viewZoneLineNumber = void 0;
                if (originalForeignVZ.current.afterLineNumber <= originalEquivalentLineNumber) {
                    viewZoneLineNumber = modifiedEquivalentLineNumber - originalEquivalentLineNumber + originalForeignVZ.current.afterLineNumber;
                }
                else {
                    viewZoneLineNumber = modifiedEndEquivalentLineNumber;
                }
                stepModified.push({
                    afterLineNumber: viewZoneLineNumber,
                    heightInLines: originalForeignVZ.current.heightInLines,
                    domNode: null
                });
                originalForeignVZ.advance();
            }
            if (lineChange !== null && isChangeOrInsert(lineChange)) {
                var r = this._produceOriginalFromDiff(lineChange, lineChangeOriginalLength, lineChangeModifiedLength);
                if (r) {
                    stepOriginal.push(r);
                }
            }
            if (lineChange !== null && isChangeOrDelete(lineChange)) {
                var r = this._produceModifiedFromDiff(lineChange, lineChangeOriginalLength, lineChangeModifiedLength);
                if (r) {
                    stepModified.push(r);
                }
            }
            // ---------------------------- END PRODUCE VIEW ZONES
            // ---------------------------- EMIT MINIMAL VIEW ZONES
            // [CANCEL & EMIT] Try to cancel view zones out
            var stepOriginalIndex = 0;
            var stepModifiedIndex = 0;
            stepOriginal = stepOriginal.sort(sortMyViewZones);
            stepModified = stepModified.sort(sortMyViewZones);
            while (stepOriginalIndex < stepOriginal.length && stepModifiedIndex < stepModified.length) {
                var original = stepOriginal[stepOriginalIndex];
                var modified = stepModified[stepModifiedIndex];
                var originalDelta = original.afterLineNumber - originalEquivalentLineNumber;
                var modifiedDelta = modified.afterLineNumber - modifiedEquivalentLineNumber;
                if (originalDelta < modifiedDelta) {
                    addAndCombineIfPossible(result.original, original);
                    stepOriginalIndex++;
                }
                else if (modifiedDelta < originalDelta) {
                    addAndCombineIfPossible(result.modified, modified);
                    stepModifiedIndex++;
                }
                else if (original.shouldNotShrink) {
                    addAndCombineIfPossible(result.original, original);
                    stepOriginalIndex++;
                }
                else if (modified.shouldNotShrink) {
                    addAndCombineIfPossible(result.modified, modified);
                    stepModifiedIndex++;
                }
                else {
                    if (original.heightInLines >= modified.heightInLines) {
                        // modified view zone gets removed
                        original.heightInLines -= modified.heightInLines;
                        stepModifiedIndex++;
                    }
                    else {
                        // original view zone gets removed
                        modified.heightInLines -= original.heightInLines;
                        stepOriginalIndex++;
                    }
                }
            }
            // [EMIT] Remaining original view zones
            while (stepOriginalIndex < stepOriginal.length) {
                addAndCombineIfPossible(result.original, stepOriginal[stepOriginalIndex]);
                stepOriginalIndex++;
            }
            // [EMIT] Remaining modified view zones
            while (stepModifiedIndex < stepModified.length) {
                addAndCombineIfPossible(result.modified, stepModified[stepModifiedIndex]);
                stepModifiedIndex++;
            }
            // ---------------------------- END EMIT MINIMAL VIEW ZONES
        }
        var ensureDomNode = function (z) {
            if (!z.domNode) {
                z.domNode = createFakeLinesDiv();
            }
        };
        result.original.forEach(ensureDomNode);
        result.modified.forEach(ensureDomNode);
        return result;
    };
    return ViewZonesComputer;
}());
function createDecoration(startLineNumber, startColumn, endLineNumber, endColumn, options) {
    return {
        range: new Range(startLineNumber, startColumn, endLineNumber, endColumn),
        options: options
    };
}
var DECORATIONS = {
    charDelete: ModelDecorationOptions.register({
        className: 'char-delete'
    }),
    charDeleteWholeLine: ModelDecorationOptions.register({
        className: 'char-delete',
        isWholeLine: true
    }),
    charInsert: ModelDecorationOptions.register({
        className: 'char-insert'
    }),
    charInsertWholeLine: ModelDecorationOptions.register({
        className: 'char-insert',
        isWholeLine: true
    }),
    lineInsert: ModelDecorationOptions.register({
        className: 'line-insert',
        marginClassName: 'line-insert',
        isWholeLine: true
    }),
    lineInsertWithSign: ModelDecorationOptions.register({
        className: 'line-insert',
        linesDecorationsClassName: 'insert-sign',
        marginClassName: 'line-insert',
        isWholeLine: true
    }),
    lineDelete: ModelDecorationOptions.register({
        className: 'line-delete',
        marginClassName: 'line-delete',
        isWholeLine: true
    }),
    lineDeleteWithSign: ModelDecorationOptions.register({
        className: 'line-delete',
        linesDecorationsClassName: 'delete-sign',
        marginClassName: 'line-delete',
        isWholeLine: true
    }),
    lineDeleteMargin: ModelDecorationOptions.register({
        marginClassName: 'line-delete',
    })
};
var DiffEdtorWidgetSideBySide = /** @class */ (function (_super) {
    __extends$1a(DiffEdtorWidgetSideBySide, _super);
    function DiffEdtorWidgetSideBySide(dataSource, enableSplitViewResizing) {
        var _this = _super.call(this, dataSource) || this;
        _this._disableSash = (enableSplitViewResizing === false);
        _this._sashRatio = null;
        _this._sashPosition = null;
        _this._sash = _this._register(new Sash(_this._dataSource.getContainerDomNode(), _this));
        if (_this._disableSash) {
            _this._sash.state = 0 /* Disabled */;
        }
        _this._sash.onDidStart(function () { return _this.onSashDragStart(); });
        _this._sash.onDidChange(function (e) { return _this.onSashDrag(e); });
        _this._sash.onDidEnd(function () { return _this.onSashDragEnd(); });
        _this._sash.onDidReset(function () { return _this.onSashReset(); });
        return _this;
    }
    DiffEdtorWidgetSideBySide.prototype.setEnableSplitViewResizing = function (enableSplitViewResizing) {
        var newDisableSash = (enableSplitViewResizing === false);
        if (this._disableSash !== newDisableSash) {
            this._disableSash = newDisableSash;
            this._sash.state = this._disableSash ? 0 /* Disabled */ : 3 /* Enabled */;
        }
    };
    DiffEdtorWidgetSideBySide.prototype.layout = function (sashRatio) {
        if (sashRatio === void 0) { sashRatio = this._sashRatio; }
        var w = this._dataSource.getWidth();
        var contentWidth = w - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH;
        var sashPosition = Math.floor((sashRatio || 0.5) * contentWidth);
        var midPoint = Math.floor(0.5 * contentWidth);
        sashPosition = this._disableSash ? midPoint : sashPosition || midPoint;
        if (contentWidth > DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH * 2) {
            if (sashPosition < DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH) {
                sashPosition = DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH;
            }
            if (sashPosition > contentWidth - DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH) {
                sashPosition = contentWidth - DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH;
            }
        }
        else {
            sashPosition = midPoint;
        }
        if (this._sashPosition !== sashPosition) {
            this._sashPosition = sashPosition;
            this._sash.layout();
        }
        return this._sashPosition;
    };
    DiffEdtorWidgetSideBySide.prototype.onSashDragStart = function () {
        this._startSashPosition = this._sashPosition;
    };
    DiffEdtorWidgetSideBySide.prototype.onSashDrag = function (e) {
        var w = this._dataSource.getWidth();
        var contentWidth = w - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH;
        var sashPosition = this.layout((this._startSashPosition + (e.currentX - e.startX)) / contentWidth);
        this._sashRatio = sashPosition / contentWidth;
        this._dataSource.relayoutEditors();
    };
    DiffEdtorWidgetSideBySide.prototype.onSashDragEnd = function () {
        this._sash.layout();
    };
    DiffEdtorWidgetSideBySide.prototype.onSashReset = function () {
        this._sashRatio = 0.5;
        this._dataSource.relayoutEditors();
        this._sash.layout();
    };
    DiffEdtorWidgetSideBySide.prototype.getVerticalSashTop = function (sash) {
        return 0;
    };
    DiffEdtorWidgetSideBySide.prototype.getVerticalSashLeft = function (sash) {
        return this._sashPosition;
    };
    DiffEdtorWidgetSideBySide.prototype.getVerticalSashHeight = function (sash) {
        return this._dataSource.getHeight();
    };
    DiffEdtorWidgetSideBySide.prototype._getViewZones = function (lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor) {
        var c = new SideBySideViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ);
        return c.getViewZones();
    };
    DiffEdtorWidgetSideBySide.prototype._getOriginalEditorDecorations = function (lineChanges, ignoreTrimWhitespace, renderIndicators, originalEditor, modifiedEditor) {
        var overviewZoneColor = this._removeColor.toString();
        var result = {
            decorations: [],
            overviewZones: []
        };
        var originalModel = originalEditor.getModel();
        for (var i = 0, length_5 = lineChanges.length; i < length_5; i++) {
            var lineChange = lineChanges[i];
            if (isChangeOrDelete(lineChange)) {
                result.decorations.push({
                    range: new Range(lineChange.originalStartLineNumber, 1, lineChange.originalEndLineNumber, Number.MAX_VALUE),
                    options: (renderIndicators ? DECORATIONS.lineDeleteWithSign : DECORATIONS.lineDelete)
                });
                if (!isChangeOrInsert(lineChange) || !lineChange.charChanges) {
                    result.decorations.push(createDecoration(lineChange.originalStartLineNumber, 1, lineChange.originalEndLineNumber, Number.MAX_VALUE, DECORATIONS.charDeleteWholeLine));
                }
                result.overviewZones.push(new OverviewRulerZone(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber, overviewZoneColor));
                if (lineChange.charChanges) {
                    for (var j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                        var charChange = lineChange.charChanges[j];
                        if (isChangeOrDelete(charChange)) {
                            if (ignoreTrimWhitespace) {
                                for (var lineNumber = charChange.originalStartLineNumber; lineNumber <= charChange.originalEndLineNumber; lineNumber++) {
                                    var startColumn = void 0;
                                    var endColumn = void 0;
                                    if (lineNumber === charChange.originalStartLineNumber) {
                                        startColumn = charChange.originalStartColumn;
                                    }
                                    else {
                                        startColumn = originalModel.getLineFirstNonWhitespaceColumn(lineNumber);
                                    }
                                    if (lineNumber === charChange.originalEndLineNumber) {
                                        endColumn = charChange.originalEndColumn;
                                    }
                                    else {
                                        endColumn = originalModel.getLineLastNonWhitespaceColumn(lineNumber);
                                    }
                                    result.decorations.push(createDecoration(lineNumber, startColumn, lineNumber, endColumn, DECORATIONS.charDelete));
                                }
                            }
                            else {
                                result.decorations.push(createDecoration(charChange.originalStartLineNumber, charChange.originalStartColumn, charChange.originalEndLineNumber, charChange.originalEndColumn, DECORATIONS.charDelete));
                            }
                        }
                    }
                }
            }
        }
        return result;
    };
    DiffEdtorWidgetSideBySide.prototype._getModifiedEditorDecorations = function (lineChanges, ignoreTrimWhitespace, renderIndicators, originalEditor, modifiedEditor) {
        var overviewZoneColor = this._insertColor.toString();
        var result = {
            decorations: [],
            overviewZones: []
        };
        var modifiedModel = modifiedEditor.getModel();
        for (var i = 0, length_6 = lineChanges.length; i < length_6; i++) {
            var lineChange = lineChanges[i];
            if (isChangeOrInsert(lineChange)) {
                result.decorations.push({
                    range: new Range(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE),
                    options: (renderIndicators ? DECORATIONS.lineInsertWithSign : DECORATIONS.lineInsert)
                });
                if (!isChangeOrDelete(lineChange) || !lineChange.charChanges) {
                    result.decorations.push(createDecoration(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE, DECORATIONS.charInsertWholeLine));
                }
                result.overviewZones.push(new OverviewRulerZone(lineChange.modifiedStartLineNumber, lineChange.modifiedEndLineNumber, overviewZoneColor));
                if (lineChange.charChanges) {
                    for (var j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                        var charChange = lineChange.charChanges[j];
                        if (isChangeOrInsert(charChange)) {
                            if (ignoreTrimWhitespace) {
                                for (var lineNumber = charChange.modifiedStartLineNumber; lineNumber <= charChange.modifiedEndLineNumber; lineNumber++) {
                                    var startColumn = void 0;
                                    var endColumn = void 0;
                                    if (lineNumber === charChange.modifiedStartLineNumber) {
                                        startColumn = charChange.modifiedStartColumn;
                                    }
                                    else {
                                        startColumn = modifiedModel.getLineFirstNonWhitespaceColumn(lineNumber);
                                    }
                                    if (lineNumber === charChange.modifiedEndLineNumber) {
                                        endColumn = charChange.modifiedEndColumn;
                                    }
                                    else {
                                        endColumn = modifiedModel.getLineLastNonWhitespaceColumn(lineNumber);
                                    }
                                    result.decorations.push(createDecoration(lineNumber, startColumn, lineNumber, endColumn, DECORATIONS.charInsert));
                                }
                            }
                            else {
                                result.decorations.push(createDecoration(charChange.modifiedStartLineNumber, charChange.modifiedStartColumn, charChange.modifiedEndLineNumber, charChange.modifiedEndColumn, DECORATIONS.charInsert));
                            }
                        }
                    }
                }
            }
        }
        return result;
    };
    DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH = 100;
    return DiffEdtorWidgetSideBySide;
}(DiffEditorWidgetStyle));
var SideBySideViewZonesComputer = /** @class */ (function (_super) {
    __extends$1a(SideBySideViewZonesComputer, _super);
    function SideBySideViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ) {
        return _super.call(this, lineChanges, originalForeignVZ, modifiedForeignVZ) || this;
    }
    SideBySideViewZonesComputer.prototype._createOriginalMarginDomNodeForModifiedForeignViewZoneInAddedRegion = function () {
        return null;
    };
    SideBySideViewZonesComputer.prototype._produceOriginalFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
        if (lineChangeModifiedLength > lineChangeOriginalLength) {
            return {
                afterLineNumber: Math.max(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber),
                heightInLines: (lineChangeModifiedLength - lineChangeOriginalLength),
                domNode: null
            };
        }
        return null;
    };
    SideBySideViewZonesComputer.prototype._produceModifiedFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
        if (lineChangeOriginalLength > lineChangeModifiedLength) {
            return {
                afterLineNumber: Math.max(lineChange.modifiedStartLineNumber, lineChange.modifiedEndLineNumber),
                heightInLines: (lineChangeOriginalLength - lineChangeModifiedLength),
                domNode: null
            };
        }
        return null;
    };
    return SideBySideViewZonesComputer;
}(ViewZonesComputer));
var DiffEdtorWidgetInline = /** @class */ (function (_super) {
    __extends$1a(DiffEdtorWidgetInline, _super);
    function DiffEdtorWidgetInline(dataSource, enableSplitViewResizing) {
        var _this = _super.call(this, dataSource) || this;
        _this.decorationsLeft = dataSource.getOriginalEditor().getLayoutInfo().decorationsLeft;
        _this._register(dataSource.getOriginalEditor().onDidLayoutChange(function (layoutInfo) {
            if (_this.decorationsLeft !== layoutInfo.decorationsLeft) {
                _this.decorationsLeft = layoutInfo.decorationsLeft;
                dataSource.relayoutEditors();
            }
        }));
        return _this;
    }
    DiffEdtorWidgetInline.prototype.setEnableSplitViewResizing = function (enableSplitViewResizing) {
        // Nothing to do..
    };
    DiffEdtorWidgetInline.prototype._getViewZones = function (lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor, renderIndicators) {
        var computer = new InlineViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor, renderIndicators);
        return computer.getViewZones();
    };
    DiffEdtorWidgetInline.prototype._getOriginalEditorDecorations = function (lineChanges, ignoreTrimWhitespace, renderIndicators, originalEditor, modifiedEditor) {
        var overviewZoneColor = this._removeColor.toString();
        var result = {
            decorations: [],
            overviewZones: []
        };
        for (var i = 0, length_7 = lineChanges.length; i < length_7; i++) {
            var lineChange = lineChanges[i];
            // Add overview zones in the overview ruler
            if (isChangeOrDelete(lineChange)) {
                result.decorations.push({
                    range: new Range(lineChange.originalStartLineNumber, 1, lineChange.originalEndLineNumber, Number.MAX_VALUE),
                    options: DECORATIONS.lineDeleteMargin
                });
                result.overviewZones.push(new OverviewRulerZone(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber, overviewZoneColor));
            }
        }
        return result;
    };
    DiffEdtorWidgetInline.prototype._getModifiedEditorDecorations = function (lineChanges, ignoreTrimWhitespace, renderIndicators, originalEditor, modifiedEditor) {
        var overviewZoneColor = this._insertColor.toString();
        var result = {
            decorations: [],
            overviewZones: []
        };
        var modifiedModel = modifiedEditor.getModel();
        for (var i = 0, length_8 = lineChanges.length; i < length_8; i++) {
            var lineChange = lineChanges[i];
            // Add decorations & overview zones
            if (isChangeOrInsert(lineChange)) {
                result.decorations.push({
                    range: new Range(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE),
                    options: (renderIndicators ? DECORATIONS.lineInsertWithSign : DECORATIONS.lineInsert)
                });
                result.overviewZones.push(new OverviewRulerZone(lineChange.modifiedStartLineNumber, lineChange.modifiedEndLineNumber, overviewZoneColor));
                if (lineChange.charChanges) {
                    for (var j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                        var charChange = lineChange.charChanges[j];
                        if (isChangeOrInsert(charChange)) {
                            if (ignoreTrimWhitespace) {
                                for (var lineNumber = charChange.modifiedStartLineNumber; lineNumber <= charChange.modifiedEndLineNumber; lineNumber++) {
                                    var startColumn = void 0;
                                    var endColumn = void 0;
                                    if (lineNumber === charChange.modifiedStartLineNumber) {
                                        startColumn = charChange.modifiedStartColumn;
                                    }
                                    else {
                                        startColumn = modifiedModel.getLineFirstNonWhitespaceColumn(lineNumber);
                                    }
                                    if (lineNumber === charChange.modifiedEndLineNumber) {
                                        endColumn = charChange.modifiedEndColumn;
                                    }
                                    else {
                                        endColumn = modifiedModel.getLineLastNonWhitespaceColumn(lineNumber);
                                    }
                                    result.decorations.push(createDecoration(lineNumber, startColumn, lineNumber, endColumn, DECORATIONS.charInsert));
                                }
                            }
                            else {
                                result.decorations.push(createDecoration(charChange.modifiedStartLineNumber, charChange.modifiedStartColumn, charChange.modifiedEndLineNumber, charChange.modifiedEndColumn, DECORATIONS.charInsert));
                            }
                        }
                    }
                }
                else {
                    result.decorations.push(createDecoration(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE, DECORATIONS.charInsertWholeLine));
                }
            }
        }
        return result;
    };
    DiffEdtorWidgetInline.prototype.layout = function () {
        // An editor should not be smaller than 5px
        return Math.max(5, this.decorationsLeft);
    };
    return DiffEdtorWidgetInline;
}(DiffEditorWidgetStyle));
var InlineViewZonesComputer = /** @class */ (function (_super) {
    __extends$1a(InlineViewZonesComputer, _super);
    function InlineViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor, renderIndicators) {
        var _this = _super.call(this, lineChanges, originalForeignVZ, modifiedForeignVZ) || this;
        _this.originalModel = originalEditor.getModel();
        _this.modifiedEditorConfiguration = modifiedEditor.getConfiguration();
        _this.modifiedEditorTabSize = modifiedEditor.getModel().getOptions().tabSize;
        _this.renderIndicators = renderIndicators;
        return _this;
    }
    InlineViewZonesComputer.prototype._createOriginalMarginDomNodeForModifiedForeignViewZoneInAddedRegion = function () {
        var result = document.createElement('div');
        result.className = 'inline-added-margin-view-zone';
        return result;
    };
    InlineViewZonesComputer.prototype._produceOriginalFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
        var marginDomNode = document.createElement('div');
        marginDomNode.className = 'inline-added-margin-view-zone';
        return {
            afterLineNumber: Math.max(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber),
            heightInLines: lineChangeModifiedLength,
            domNode: document.createElement('div'),
            marginDomNode: marginDomNode
        };
    };
    InlineViewZonesComputer.prototype._produceModifiedFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
        var decorations = [];
        if (lineChange.charChanges) {
            for (var j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                var charChange = lineChange.charChanges[j];
                if (isChangeOrDelete(charChange)) {
                    decorations.push(new InlineDecoration(new Range(charChange.originalStartLineNumber, charChange.originalStartColumn, charChange.originalEndLineNumber, charChange.originalEndColumn), 'char-delete', 0 /* Regular */));
                }
            }
        }
        var sb = createStringBuilder(10000);
        var marginHTML = [];
        var lineDecorationsWidth = this.modifiedEditorConfiguration.layoutInfo.decorationsWidth;
        var lineHeight = this.modifiedEditorConfiguration.lineHeight;
        var typicalHalfwidthCharacterWidth = this.modifiedEditorConfiguration.fontInfo.typicalHalfwidthCharacterWidth;
        var maxCharsPerLine = 0;
        for (var lineNumber = lineChange.originalStartLineNumber; lineNumber <= lineChange.originalEndLineNumber; lineNumber++) {
            maxCharsPerLine = Math.max(maxCharsPerLine, this._renderOriginalLine(lineNumber - lineChange.originalStartLineNumber, this.originalModel, this.modifiedEditorConfiguration, this.modifiedEditorTabSize, lineNumber, decorations, sb));
            if (this.renderIndicators) {
                var index = lineNumber - lineChange.originalStartLineNumber;
                marginHTML = marginHTML.concat([
                    "<div class=\"delete-sign\" style=\"position:absolute;top:" + index * lineHeight + "px;width:" + lineDecorationsWidth + "px;height:" + lineHeight + "px;right:0;\"></div>"
                ]);
            }
        }
        maxCharsPerLine += this.modifiedEditorConfiguration.viewInfo.scrollBeyondLastColumn;
        var domNode = document.createElement('div');
        domNode.className = 'view-lines line-delete';
        domNode.innerHTML = sb.build();
        Configuration.applyFontInfoSlow(domNode, this.modifiedEditorConfiguration.fontInfo);
        var marginDomNode = document.createElement('div');
        marginDomNode.className = 'inline-deleted-margin-view-zone';
        marginDomNode.innerHTML = marginHTML.join('');
        Configuration.applyFontInfoSlow(marginDomNode, this.modifiedEditorConfiguration.fontInfo);
        return {
            shouldNotShrink: true,
            afterLineNumber: (lineChange.modifiedEndLineNumber === 0 ? lineChange.modifiedStartLineNumber : lineChange.modifiedStartLineNumber - 1),
            heightInLines: lineChangeOriginalLength,
            minWidthInPx: (maxCharsPerLine * typicalHalfwidthCharacterWidth),
            domNode: domNode,
            marginDomNode: marginDomNode
        };
    };
    InlineViewZonesComputer.prototype._renderOriginalLine = function (count, originalModel, config, tabSize, lineNumber, decorations, sb) {
        var lineTokens = originalModel.getLineTokens(lineNumber);
        var lineContent = lineTokens.getLineContent();
        var actualDecorations = LineDecoration.filter(decorations, lineNumber, 1, lineContent.length + 1);
        sb.appendASCIIString('<div class="view-line');
        if (decorations.length === 0) {
            // No char changes
            sb.appendASCIIString(' char-delete');
        }
        sb.appendASCIIString('" style="top:');
        sb.appendASCIIString(String(count * config.lineHeight));
        sb.appendASCIIString('px;width:1000000px;">');
        var isBasicASCII = ViewLineRenderingData.isBasicASCII(lineContent, originalModel.mightContainNonBasicASCII());
        var containsRTL = ViewLineRenderingData.containsRTL(lineContent, isBasicASCII, originalModel.mightContainRTL());
        var output = renderViewLine(new RenderLineInput((config.fontInfo.isMonospace && !config.viewInfo.disableMonospaceOptimizations), config.fontInfo.canUseHalfwidthRightwardsArrow, lineContent, false, isBasicASCII, containsRTL, 0, lineTokens, actualDecorations, tabSize, config.fontInfo.spaceWidth, config.viewInfo.stopRenderingLineAfter, config.viewInfo.renderWhitespace, config.viewInfo.renderControlCharacters, config.viewInfo.fontLigatures), sb);
        sb.appendASCIIString('</div>');
        var absoluteOffsets = output.characterMapping.getAbsoluteOffsets();
        return absoluteOffsets.length > 0 ? absoluteOffsets[absoluteOffsets.length - 1] : 0;
    };
    return InlineViewZonesComputer;
}(ViewZonesComputer));
function isChangeOrInsert(lineChange) {
    return lineChange.modifiedEndLineNumber > 0;
}
function isChangeOrDelete(lineChange) {
    return lineChange.originalEndLineNumber > 0;
}
function createFakeLinesDiv() {
    var r = document.createElement('div');
    r.className = 'diagonal-fill';
    return r;
}
registerThemingParticipant(function (theme, collector) {
    var added = theme.getColor(diffInserted);
    if (added) {
        collector.addRule(".monaco-editor .line-insert, .monaco-editor .char-insert { background-color: " + added + "; }");
        collector.addRule(".monaco-diff-editor .line-insert, .monaco-diff-editor .char-insert { background-color: " + added + "; }");
        collector.addRule(".monaco-editor .inline-added-margin-view-zone { background-color: " + added + "; }");
    }
    var removed = theme.getColor(diffRemoved);
    if (removed) {
        collector.addRule(".monaco-editor .line-delete, .monaco-editor .char-delete { background-color: " + removed + "; }");
        collector.addRule(".monaco-diff-editor .line-delete, .monaco-diff-editor .char-delete { background-color: " + removed + "; }");
        collector.addRule(".monaco-editor .inline-deleted-margin-view-zone { background-color: " + removed + "; }");
    }
    var addedOutline = theme.getColor(diffInsertedOutline);
    if (addedOutline) {
        collector.addRule(".monaco-editor .line-insert, .monaco-editor .char-insert { border: 1px " + (theme.type === 'hc' ? 'dashed' : 'solid') + " " + addedOutline + "; }");
    }
    var removedOutline = theme.getColor(diffRemovedOutline);
    if (removedOutline) {
        collector.addRule(".monaco-editor .line-delete, .monaco-editor .char-delete { border: 1px " + (theme.type === 'hc' ? 'dashed' : 'solid') + " " + removedOutline + "; }");
    }
    var shadow = theme.getColor(scrollbarShadow);
    if (shadow) {
        collector.addRule(".monaco-diff-editor.side-by-side .editor.modified { box-shadow: -6px 0 5px -5px " + shadow + "; }");
    }
    var border = theme.getColor(diffBorder);
    if (border) {
        collector.addRule(".monaco-diff-editor.side-by-side .editor.modified { border-left: 1px solid " + border + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var defaultOptions$1 = {
    followsCaret: true,
    ignoreCharChanges: true,
    alwaysRevealFirst: true
};
/**
 * Create a new diff navigator for the provided diff editor.
 */
var DiffNavigator = /** @class */ (function () {
    function DiffNavigator(editor, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this._onDidUpdate = new Emitter();
        this._editor = editor;
        this._options = mixin(options, defaultOptions$1, false);
        this.disposed = false;
        this._disposables = [];
        this.nextIdx = -1;
        this.ranges = [];
        this.ignoreSelectionChange = false;
        this.revealFirst = Boolean(this._options.alwaysRevealFirst);
        // hook up to diff editor for diff, disposal, and caret move
        this._disposables.push(this._editor.onDidDispose(function () { return _this.dispose(); }));
        this._disposables.push(this._editor.onDidUpdateDiff(function () { return _this._onDiffUpdated(); }));
        if (this._options.followsCaret) {
            this._disposables.push(this._editor.getModifiedEditor().onDidChangeCursorPosition(function (e) {
                if (_this.ignoreSelectionChange) {
                    return;
                }
                _this.nextIdx = -1;
            }));
        }
        if (this._options.alwaysRevealFirst) {
            this._disposables.push(this._editor.getModifiedEditor().onDidChangeModel(function (e) {
                _this.revealFirst = true;
            }));
        }
        // init things
        this._init();
    }
    DiffNavigator.prototype._init = function () {
        var changes = this._editor.getLineChanges();
        if (!changes) {
            return;
        }
    };
    DiffNavigator.prototype._onDiffUpdated = function () {
        this._init();
        this._compute(this._editor.getLineChanges());
        if (this.revealFirst) {
            // Only reveal first on first non-null changes
            if (this._editor.getLineChanges() !== null) {
                this.revealFirst = false;
                this.nextIdx = -1;
                this.next(1 /* Immediate */);
            }
        }
    };
    DiffNavigator.prototype._compute = function (lineChanges) {
        var _this = this;
        // new ranges
        this.ranges = [];
        if (lineChanges) {
            // create ranges from changes
            lineChanges.forEach(function (lineChange) {
                if (!_this._options.ignoreCharChanges && lineChange.charChanges) {
                    lineChange.charChanges.forEach(function (charChange) {
                        _this.ranges.push({
                            rhs: true,
                            range: new Range(charChange.modifiedStartLineNumber, charChange.modifiedStartColumn, charChange.modifiedEndLineNumber, charChange.modifiedEndColumn)
                        });
                    });
                }
                else {
                    _this.ranges.push({
                        rhs: true,
                        range: new Range(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedStartLineNumber, 1)
                    });
                }
            });
        }
        // sort
        this.ranges.sort(function (left, right) {
            if (left.range.getStartPosition().isBeforeOrEqual(right.range.getStartPosition())) {
                return -1;
            }
            else if (right.range.getStartPosition().isBeforeOrEqual(left.range.getStartPosition())) {
                return 1;
            }
            else {
                return 0;
            }
        });
        this._onDidUpdate.fire(this);
    };
    DiffNavigator.prototype._initIdx = function (fwd) {
        var found = false;
        var position = this._editor.getPosition();
        if (!position) {
            this.nextIdx = 0;
            return;
        }
        for (var i = 0, len = this.ranges.length; i < len && !found; i++) {
            var range = this.ranges[i].range;
            if (position.isBeforeOrEqual(range.getStartPosition())) {
                this.nextIdx = i + (fwd ? 0 : -1);
                found = true;
            }
        }
        if (!found) {
            // after the last change
            this.nextIdx = fwd ? 0 : this.ranges.length - 1;
        }
        if (this.nextIdx < 0) {
            this.nextIdx = this.ranges.length - 1;
        }
    };
    DiffNavigator.prototype._move = function (fwd, scrollType) {
        ok(!this.disposed, 'Illegal State - diff navigator has been disposed');
        if (!this.canNavigate()) {
            return;
        }
        if (this.nextIdx === -1) {
            this._initIdx(fwd);
        }
        else if (fwd) {
            this.nextIdx += 1;
            if (this.nextIdx >= this.ranges.length) {
                this.nextIdx = 0;
            }
        }
        else {
            this.nextIdx -= 1;
            if (this.nextIdx < 0) {
                this.nextIdx = this.ranges.length - 1;
            }
        }
        var info = this.ranges[this.nextIdx];
        this.ignoreSelectionChange = true;
        try {
            var pos = info.range.getStartPosition();
            this._editor.setPosition(pos);
            this._editor.revealPositionInCenter(pos, scrollType);
        }
        finally {
            this.ignoreSelectionChange = false;
        }
    };
    DiffNavigator.prototype.canNavigate = function () {
        return this.ranges && this.ranges.length > 0;
    };
    DiffNavigator.prototype.next = function (scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._move(true, scrollType);
    };
    DiffNavigator.prototype.previous = function (scrollType) {
        if (scrollType === void 0) { scrollType = 0 /* Smooth */; }
        this._move(false, scrollType);
    };
    DiffNavigator.prototype.dispose = function () {
        dispose(this._disposables);
        this._disposables.length = 0;
        this._onDidUpdate.dispose();
        this.ranges = [];
        this.disposed = true;
    };
    return DiffNavigator;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1b = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var overviewRulerBracketMatchForeground = registerColor('editorOverviewRuler.bracketMatchForeground', { dark: '#A0A0A0', light: '#A0A0A0', hc: '#A0A0A0' }, localize('overviewRulerBracketMatchForeground', 'Overview ruler marker color for matching brackets.'));
var JumpToBracketAction = /** @class */ (function (_super) {
    __extends$1b(JumpToBracketAction, _super);
    function JumpToBracketAction() {
        return _super.call(this, {
            id: 'editor.action.jumpToBracket',
            label: localize('smartSelect.jumpBracket', "Go to Bracket"),
            alias: 'Go to Bracket',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 88 /* US_BACKSLASH */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    JumpToBracketAction.prototype.run = function (accessor, editor) {
        var controller = BracketMatchingController.get(editor);
        if (!controller) {
            return;
        }
        controller.jumpToBracket();
    };
    return JumpToBracketAction;
}(EditorAction));
var SelectToBracketAction = /** @class */ (function (_super) {
    __extends$1b(SelectToBracketAction, _super);
    function SelectToBracketAction() {
        return _super.call(this, {
            id: 'editor.action.selectToBracket',
            label: localize('smartSelect.selectToBracket', "Select to Bracket"),
            alias: 'Select to Bracket',
            precondition: null
        }) || this;
    }
    SelectToBracketAction.prototype.run = function (accessor, editor) {
        var controller = BracketMatchingController.get(editor);
        if (!controller) {
            return;
        }
        controller.selectToBracket();
    };
    return SelectToBracketAction;
}(EditorAction));
var BracketsData = /** @class */ (function () {
    function BracketsData(position, brackets) {
        this.position = position;
        this.brackets = brackets;
    }
    return BracketsData;
}());
var BracketMatchingController = /** @class */ (function (_super) {
    __extends$1b(BracketMatchingController, _super);
    function BracketMatchingController(editor) {
        var _this = _super.call(this) || this;
        _this._editor = editor;
        _this._lastBracketsData = [];
        _this._lastVersionId = 0;
        _this._decorations = [];
        _this._updateBracketsSoon = _this._register(new RunOnceScheduler(function () { return _this._updateBrackets(); }, 50));
        _this._matchBrackets = _this._editor.getConfiguration().contribInfo.matchBrackets;
        _this._updateBracketsSoon.schedule();
        _this._register(editor.onDidChangeCursorPosition(function (e) {
            if (!_this._matchBrackets) {
                // Early exit if nothing needs to be done!
                // Leave some form of early exit check here if you wish to continue being a cursor position change listener ;)
                return;
            }
            _this._updateBracketsSoon.schedule();
        }));
        _this._register(editor.onDidChangeModelContent(function (e) {
            _this._updateBracketsSoon.schedule();
        }));
        _this._register(editor.onDidChangeModel(function (e) {
            _this._lastBracketsData = [];
            _this._decorations = [];
            _this._updateBracketsSoon.schedule();
        }));
        _this._register(editor.onDidChangeModelLanguageConfiguration(function (e) {
            _this._lastBracketsData = [];
            _this._updateBracketsSoon.schedule();
        }));
        _this._register(editor.onDidChangeConfiguration(function (e) {
            _this._matchBrackets = _this._editor.getConfiguration().contribInfo.matchBrackets;
            if (!_this._matchBrackets && _this._decorations.length > 0) {
                // Remove existing decorations if bracket matching is off
                _this._decorations = _this._editor.deltaDecorations(_this._decorations, []);
            }
            _this._updateBracketsSoon.schedule();
        }));
        return _this;
    }
    BracketMatchingController.get = function (editor) {
        return editor.getContribution(BracketMatchingController.ID);
    };
    BracketMatchingController.prototype.getId = function () {
        return BracketMatchingController.ID;
    };
    BracketMatchingController.prototype.jumpToBracket = function () {
        if (!this._editor.hasModel()) {
            return;
        }
        var model = this._editor.getModel();
        var newSelections = this._editor.getSelections().map(function (selection) {
            var position = selection.getStartPosition();
            // find matching brackets if position is on a bracket
            var brackets = model.matchBracket(position);
            var newCursorPosition = null;
            if (brackets) {
                if (brackets[0].containsPosition(position)) {
                    newCursorPosition = brackets[1].getStartPosition();
                }
                else if (brackets[1].containsPosition(position)) {
                    newCursorPosition = brackets[0].getStartPosition();
                }
            }
            else {
                // find the next bracket if the position isn't on a matching bracket
                var nextBracket = model.findNextBracket(position);
                if (nextBracket && nextBracket.range) {
                    newCursorPosition = nextBracket.range.getStartPosition();
                }
            }
            if (newCursorPosition) {
                return new Selection(newCursorPosition.lineNumber, newCursorPosition.column, newCursorPosition.lineNumber, newCursorPosition.column);
            }
            return new Selection(position.lineNumber, position.column, position.lineNumber, position.column);
        });
        this._editor.setSelections(newSelections);
        this._editor.revealRange(newSelections[0]);
    };
    BracketMatchingController.prototype.selectToBracket = function () {
        if (!this._editor.hasModel()) {
            return;
        }
        var model = this._editor.getModel();
        var newSelections = [];
        this._editor.getSelections().forEach(function (selection) {
            var position = selection.getStartPosition();
            var brackets = model.matchBracket(position);
            var openBracket = null;
            var closeBracket = null;
            if (!brackets) {
                var nextBracket = model.findNextBracket(position);
                if (nextBracket && nextBracket.range) {
                    brackets = model.matchBracket(nextBracket.range.getStartPosition());
                }
            }
            if (brackets) {
                if (brackets[0].startLineNumber === brackets[1].startLineNumber) {
                    openBracket = brackets[1].startColumn < brackets[0].startColumn ?
                        brackets[1].getStartPosition() : brackets[0].getStartPosition();
                    closeBracket = brackets[1].startColumn < brackets[0].startColumn ?
                        brackets[0].getEndPosition() : brackets[1].getEndPosition();
                }
                else {
                    openBracket = brackets[1].startLineNumber < brackets[0].startLineNumber ?
                        brackets[1].getStartPosition() : brackets[0].getStartPosition();
                    closeBracket = brackets[1].startLineNumber < brackets[0].startLineNumber ?
                        brackets[0].getEndPosition() : brackets[1].getEndPosition();
                }
            }
            if (openBracket && closeBracket) {
                newSelections.push(new Selection(openBracket.lineNumber, openBracket.column, closeBracket.lineNumber, closeBracket.column));
            }
        });
        if (newSelections.length > 0) {
            this._editor.setSelections(newSelections);
            this._editor.revealRange(newSelections[0]);
        }
    };
    BracketMatchingController.prototype._updateBrackets = function () {
        if (!this._matchBrackets) {
            return;
        }
        this._recomputeBrackets();
        var newDecorations = [], newDecorationsLen = 0;
        for (var i = 0, len = this._lastBracketsData.length; i < len; i++) {
            var brackets = this._lastBracketsData[i].brackets;
            if (brackets) {
                newDecorations[newDecorationsLen++] = { range: brackets[0], options: BracketMatchingController._DECORATION_OPTIONS };
                newDecorations[newDecorationsLen++] = { range: brackets[1], options: BracketMatchingController._DECORATION_OPTIONS };
            }
        }
        this._decorations = this._editor.deltaDecorations(this._decorations, newDecorations);
    };
    BracketMatchingController.prototype._recomputeBrackets = function () {
        if (!this._editor.hasModel()) {
            // no model => no brackets!
            this._lastBracketsData = [];
            this._lastVersionId = 0;
            return;
        }
        var model = this._editor.getModel();
        var versionId = model.getVersionId();
        var previousData = [];
        if (this._lastVersionId === versionId) {
            // use the previous data only if the model is at the same version id
            previousData = this._lastBracketsData;
        }
        var selections = this._editor.getSelections();
        var positions = [], positionsLen = 0;
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (selection.isEmpty()) {
                // will bracket match a cursor only if the selection is collapsed
                positions[positionsLen++] = selection.getStartPosition();
            }
        }
        // sort positions for `previousData` cache hits
        if (positions.length > 1) {
            positions.sort(Position.compare);
        }
        var newData = [], newDataLen = 0;
        var previousIndex = 0, previousLen = previousData.length;
        for (var i = 0, len = positions.length; i < len; i++) {
            var position = positions[i];
            while (previousIndex < previousLen && previousData[previousIndex].position.isBefore(position)) {
                previousIndex++;
            }
            if (previousIndex < previousLen && previousData[previousIndex].position.equals(position)) {
                newData[newDataLen++] = previousData[previousIndex];
            }
            else {
                var brackets = model.matchBracket(position);
                newData[newDataLen++] = new BracketsData(position, brackets);
            }
        }
        this._lastBracketsData = newData;
        this._lastVersionId = versionId;
    };
    BracketMatchingController.ID = 'editor.contrib.bracketMatchingController';
    BracketMatchingController._DECORATION_OPTIONS = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        className: 'bracket-match',
        overviewRuler: {
            color: themeColorFromId(overviewRulerBracketMatchForeground),
            position: OverviewRulerLane.Center
        }
    });
    return BracketMatchingController;
}(Disposable));
registerEditorContribution(BracketMatchingController);
registerEditorAction(SelectToBracketAction);
registerEditorAction(JumpToBracketAction);
registerThemingParticipant(function (theme, collector) {
    var bracketMatchBackground = theme.getColor(editorBracketMatchBackground);
    if (bracketMatchBackground) {
        collector.addRule(".monaco-editor .bracket-match { background-color: " + bracketMatchBackground + "; }");
    }
    var bracketMatchBorder = theme.getColor(editorBracketMatchBorder);
    if (bracketMatchBorder) {
        collector.addRule(".monaco-editor .bracket-match { border: 1px solid " + bracketMatchBorder + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MoveCaretCommand = /** @class */ (function () {
    function MoveCaretCommand(selection, isMovingLeft) {
        this._selection = selection;
        this._isMovingLeft = isMovingLeft;
    }
    MoveCaretCommand.prototype.getEditOperations = function (model, builder) {
        var s = this._selection;
        this._selectionId = builder.trackSelection(s);
        if (s.startLineNumber !== s.endLineNumber) {
            return;
        }
        if (this._isMovingLeft && s.startColumn === 0) {
            return;
        }
        else if (!this._isMovingLeft && s.endColumn === model.getLineMaxColumn(s.startLineNumber)) {
            return;
        }
        var lineNumber = s.selectionStartLineNumber;
        var lineContent = model.getLineContent(lineNumber);
        var left;
        var middle;
        var right;
        if (this._isMovingLeft) {
            left = lineContent.substring(0, s.startColumn - 2);
            middle = lineContent.substring(s.startColumn - 1, s.endColumn - 1);
            right = lineContent.substring(s.startColumn - 2, s.startColumn - 1) + lineContent.substring(s.endColumn - 1);
        }
        else {
            left = lineContent.substring(0, s.startColumn - 1) + lineContent.substring(s.endColumn - 1, s.endColumn);
            middle = lineContent.substring(s.startColumn - 1, s.endColumn - 1);
            right = lineContent.substring(s.endColumn);
        }
        var newLineContent = left + middle + right;
        builder.addEditOperation(new Range(lineNumber, 1, lineNumber, model.getLineMaxColumn(lineNumber)), null);
        builder.addEditOperation(new Range(lineNumber, 1, lineNumber, 1), newLineContent);
        this._cutStartIndex = s.startColumn + (this._isMovingLeft ? -1 : 1);
        this._cutEndIndex = this._cutStartIndex + s.endColumn - s.startColumn;
        this._moved = true;
    };
    MoveCaretCommand.prototype.computeCursorState = function (model, helper) {
        var result = helper.getTrackedSelection(this._selectionId);
        if (this._moved) {
            result = result.setStartPosition(result.startLineNumber, this._cutStartIndex);
            result = result.setEndPosition(result.startLineNumber, this._cutEndIndex);
        }
        return result;
    };
    return MoveCaretCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1c = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var MoveCaretAction = /** @class */ (function (_super) {
    __extends$1c(MoveCaretAction, _super);
    function MoveCaretAction(left, opts) {
        var _this = _super.call(this, opts) || this;
        _this.left = left;
        return _this;
    }
    MoveCaretAction.prototype.run = function (accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        var commands = [];
        var selections = editor.getSelections();
        for (var i = 0; i < selections.length; i++) {
            commands.push(new MoveCaretCommand(selections[i], this.left));
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return MoveCaretAction;
}(EditorAction));
var MoveCaretLeftAction = /** @class */ (function (_super) {
    __extends$1c(MoveCaretLeftAction, _super);
    function MoveCaretLeftAction() {
        return _super.call(this, true, {
            id: 'editor.action.moveCarretLeftAction',
            label: localize('caret.moveLeft', "Move Caret Left"),
            alias: 'Move Caret Left',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return MoveCaretLeftAction;
}(MoveCaretAction));
var MoveCaretRightAction = /** @class */ (function (_super) {
    __extends$1c(MoveCaretRightAction, _super);
    function MoveCaretRightAction() {
        return _super.call(this, false, {
            id: 'editor.action.moveCarretRightAction',
            label: localize('caret.moveRight', "Move Caret Right"),
            alias: 'Move Caret Right',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return MoveCaretRightAction;
}(MoveCaretAction));
registerEditorAction(MoveCaretLeftAction);
registerEditorAction(MoveCaretRightAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1d = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var TransposeLettersAction = /** @class */ (function (_super) {
    __extends$1d(TransposeLettersAction, _super);
    function TransposeLettersAction() {
        return _super.call(this, {
            id: 'editor.action.transposeLetters',
            label: localize('transposeLetters.label', "Transpose Letters"),
            alias: 'Transpose Letters',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 0,
                mac: {
                    primary: 256 /* WinCtrl */ | 50 /* KEY_T */
                },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    TransposeLettersAction.prototype.positionLeftOf = function (start, model) {
        var column = start.column;
        var lineNumber = start.lineNumber;
        if (column > model.getLineMinColumn(lineNumber)) {
            if (isLowSurrogate(model.getLineContent(lineNumber).charCodeAt(column - 2))) {
                // character before column is a low surrogate
                column = column - 2;
            }
            else {
                column = column - 1;
            }
        }
        else if (lineNumber > 1) {
            lineNumber = lineNumber - 1;
            column = model.getLineMaxColumn(lineNumber);
        }
        return new Position(lineNumber, column);
    };
    TransposeLettersAction.prototype.positionRightOf = function (start, model) {
        var column = start.column;
        var lineNumber = start.lineNumber;
        if (column < model.getLineMaxColumn(lineNumber)) {
            if (isHighSurrogate(model.getLineContent(lineNumber).charCodeAt(column - 1))) {
                // character after column is a high surrogate
                column = column + 2;
            }
            else {
                column = column + 1;
            }
        }
        else if (lineNumber < model.getLineCount()) {
            lineNumber = lineNumber + 1;
            column = 0;
        }
        return new Position(lineNumber, column);
    };
    TransposeLettersAction.prototype.run = function (accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        var model = editor.getModel();
        var commands = [];
        var selections = editor.getSelections();
        for (var _i = 0, selections_1 = selections; _i < selections_1.length; _i++) {
            var selection = selections_1[_i];
            if (!selection.isEmpty()) {
                continue;
            }
            var lineNumber = selection.startLineNumber;
            var column = selection.startColumn;
            var lastColumn = model.getLineMaxColumn(lineNumber);
            if (lineNumber === 1 && (column === 1 || (column === 2 && lastColumn === 2))) {
                // at beginning of file, nothing to do
                continue;
            }
            // handle special case: when at end of line, transpose left two chars
            // otherwise, transpose left and right chars
            var endPosition = (column === lastColumn) ?
                selection.getPosition() :
                this.positionRightOf(selection.getPosition(), model);
            var middlePosition = this.positionLeftOf(endPosition, model);
            var beginPosition = this.positionLeftOf(middlePosition, model);
            var leftChar = model.getValueInRange(Range.fromPositions(beginPosition, middlePosition));
            var rightChar = model.getValueInRange(Range.fromPositions(middlePosition, endPosition));
            var replaceRange = Range.fromPositions(beginPosition, endPosition);
            commands.push(new ReplaceCommand(replaceRange, rightChar + leftChar));
        }
        if (commands.length > 0) {
            editor.pushUndoStop();
            editor.executeCommands(this.id, commands);
            editor.pushUndoStop();
        }
    };
    return TransposeLettersAction;
}(EditorAction));
registerEditorAction(TransposeLettersAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1e = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CLIPBOARD_CONTEXT_MENU_GROUP = '9_cutcopypaste';
var supportsCut = (isNative || document.queryCommandSupported('cut'));
var supportsCopy = (isNative || document.queryCommandSupported('copy'));
// IE and Edge have trouble with setting html content in clipboard
var supportsCopyWithSyntaxHighlighting = (supportsCopy && !isEdgeOrIE);
// Chrome incorrectly returns true for document.queryCommandSupported('paste')
// when the paste feature is available but the calling script has insufficient
// privileges to actually perform the action
var supportsPaste = (isNative || (!isChrome && document.queryCommandSupported('paste')));
var ExecCommandAction = /** @class */ (function (_super) {
    __extends$1e(ExecCommandAction, _super);
    function ExecCommandAction(browserCommand, opts) {
        var _this = _super.call(this, opts) || this;
        _this.browserCommand = browserCommand;
        return _this;
    }
    ExecCommandAction.prototype.runCommand = function (accessor, args) {
        var focusedEditor = accessor.get(ICodeEditorService).getFocusedCodeEditor();
        // Only if editor text focus (i.e. not if editor has widget focus).
        if (focusedEditor && focusedEditor.hasTextFocus()) {
            focusedEditor.trigger('keyboard', this.id, args);
            return;
        }
        document.execCommand(this.browserCommand);
    };
    ExecCommandAction.prototype.run = function (accessor, editor) {
        editor.focus();
        document.execCommand(this.browserCommand);
    };
    return ExecCommandAction;
}(EditorAction));
var ExecCommandCutAction = /** @class */ (function (_super) {
    __extends$1e(ExecCommandCutAction, _super);
    function ExecCommandCutAction() {
        var _this = this;
        var kbOpts = {
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 54 /* KEY_X */,
            win: { primary: 2048 /* CtrlCmd */ | 54 /* KEY_X */, secondary: [1024 /* Shift */ | 20 /* Delete */] },
            weight: 100 /* EditorContrib */
        };
        // Do not bind cut keybindings in the browser,
        // since browsers do that for us and it avoids security prompts
        if (!isNative) {
            kbOpts = null;
        }
        _this = _super.call(this, 'cut', {
            id: 'editor.action.clipboardCutAction',
            label: localize('actions.clipboard.cutLabel', "Cut"),
            alias: 'Cut',
            precondition: EditorContextKeys.writable,
            kbOpts: kbOpts,
            menuOpts: {
                group: CLIPBOARD_CONTEXT_MENU_GROUP,
                order: 1
            },
            menubarOpts: {
                menuId: MenuId.MenubarEditMenu,
                group: '2_ccp',
                title: localize({ key: 'miCut', comment: ['&& denotes a mnemonic'] }, "Cu&&t"),
                order: 1
            }
        }) || this;
        return _this;
    }
    ExecCommandCutAction.prototype.run = function (accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        var emptySelectionClipboard = editor.getConfiguration().emptySelectionClipboard;
        if (!emptySelectionClipboard && editor.getSelection().isEmpty()) {
            return;
        }
        _super.prototype.run.call(this, accessor, editor);
    };
    return ExecCommandCutAction;
}(ExecCommandAction));
var ExecCommandCopyAction = /** @class */ (function (_super) {
    __extends$1e(ExecCommandCopyAction, _super);
    function ExecCommandCopyAction() {
        var _this = this;
        var kbOpts = {
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 33 /* KEY_C */,
            win: { primary: 2048 /* CtrlCmd */ | 33 /* KEY_C */, secondary: [2048 /* CtrlCmd */ | 19 /* Insert */] },
            weight: 100 /* EditorContrib */
        };
        // Do not bind copy keybindings in the browser,
        // since browsers do that for us and it avoids security prompts
        if (!isNative) {
            kbOpts = null;
        }
        _this = _super.call(this, 'copy', {
            id: 'editor.action.clipboardCopyAction',
            label: localize('actions.clipboard.copyLabel', "Copy"),
            alias: 'Copy',
            precondition: null,
            kbOpts: kbOpts,
            menuOpts: {
                group: CLIPBOARD_CONTEXT_MENU_GROUP,
                order: 2
            },
            menubarOpts: {
                menuId: MenuId.MenubarEditMenu,
                group: '2_ccp',
                title: localize({ key: 'miCopy', comment: ['&& denotes a mnemonic'] }, "&&Copy"),
                order: 2
            }
        }) || this;
        return _this;
    }
    ExecCommandCopyAction.prototype.run = function (accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        var emptySelectionClipboard = editor.getConfiguration().emptySelectionClipboard;
        if (!emptySelectionClipboard && editor.getSelection().isEmpty()) {
            return;
        }
        _super.prototype.run.call(this, accessor, editor);
    };
    return ExecCommandCopyAction;
}(ExecCommandAction));
var ExecCommandPasteAction = /** @class */ (function (_super) {
    __extends$1e(ExecCommandPasteAction, _super);
    function ExecCommandPasteAction() {
        var _this = this;
        var kbOpts = {
            kbExpr: EditorContextKeys.textInputFocus,
            primary: 2048 /* CtrlCmd */ | 52 /* KEY_V */,
            win: { primary: 2048 /* CtrlCmd */ | 52 /* KEY_V */, secondary: [1024 /* Shift */ | 19 /* Insert */] },
            weight: 100 /* EditorContrib */
        };
        // Do not bind paste keybindings in the browser,
        // since browsers do that for us and it avoids security prompts
        if (!isNative) {
            kbOpts = null;
        }
        _this = _super.call(this, 'paste', {
            id: 'editor.action.clipboardPasteAction',
            label: localize('actions.clipboard.pasteLabel', "Paste"),
            alias: 'Paste',
            precondition: EditorContextKeys.writable,
            kbOpts: kbOpts,
            menuOpts: {
                group: CLIPBOARD_CONTEXT_MENU_GROUP,
                order: 3
            },
            menubarOpts: {
                menuId: MenuId.MenubarEditMenu,
                group: '2_ccp',
                title: localize({ key: 'miPaste', comment: ['&& denotes a mnemonic'] }, "&&Paste"),
                order: 3
            }
        }) || this;
        return _this;
    }
    return ExecCommandPasteAction;
}(ExecCommandAction));
var ExecCommandCopyWithSyntaxHighlightingAction = /** @class */ (function (_super) {
    __extends$1e(ExecCommandCopyWithSyntaxHighlightingAction, _super);
    function ExecCommandCopyWithSyntaxHighlightingAction() {
        return _super.call(this, 'copy', {
            id: 'editor.action.clipboardCopyWithSyntaxHighlightingAction',
            label: localize('actions.clipboard.copyWithSyntaxHighlightingLabel', "Copy With Syntax Highlighting"),
            alias: 'Copy With Syntax Highlighting',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 0,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    ExecCommandCopyWithSyntaxHighlightingAction.prototype.run = function (accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        var emptySelectionClipboard = editor.getConfiguration().emptySelectionClipboard;
        if (!emptySelectionClipboard && editor.getSelection().isEmpty()) {
            return;
        }
        CopyOptions.forceCopyWithSyntaxHighlighting = true;
        _super.prototype.run.call(this, accessor, editor);
        CopyOptions.forceCopyWithSyntaxHighlighting = false;
    };
    return ExecCommandCopyWithSyntaxHighlightingAction;
}(ExecCommandAction));
if (supportsCut) {
    registerEditorAction(ExecCommandCutAction);
}
if (supportsCopy) {
    registerEditorAction(ExecCommandCopyAction);
}
if (supportsPaste) {
    registerEditorAction(ExecCommandPasteAction);
}
if (supportsCopyWithSyntaxHighlighting) {
    registerEditorAction(ExecCommandCopyWithSyntaxHighlightingAction);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var EditOperation = /** @class */ (function () {
    function EditOperation() {
    }
    EditOperation.insert = function (position, text) {
        return {
            range: new Range(position.lineNumber, position.column, position.lineNumber, position.column),
            text: text,
            forceMoveMarkers: true
        };
    };
    EditOperation.delete = function (range) {
        return {
            range: range,
            text: null
        };
    };
    EditOperation.replace = function (range, text) {
        return {
            range: range,
            text: text
        };
    };
    EditOperation.replaceMove = function (range, text) {
        return {
            range: range,
            text: text,
            forceMoveMarkers: true
        };
    };
    return EditOperation;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var BlockCommentCommand = /** @class */ (function () {
    function BlockCommentCommand(selection) {
        this._selection = selection;
        this._usedEndToken = null;
    }
    BlockCommentCommand._haystackHasNeedleAtOffset = function (haystack, needle, offset) {
        if (offset < 0) {
            return false;
        }
        var needleLength = needle.length;
        var haystackLength = haystack.length;
        if (offset + needleLength > haystackLength) {
            return false;
        }
        for (var i = 0; i < needleLength; i++) {
            var codeA = haystack.charCodeAt(offset + i);
            var codeB = needle.charCodeAt(i);
            if (codeA === codeB) {
                continue;
            }
            if (codeA >= 65 /* A */ && codeA <= 90 /* Z */ && codeA + 32 === codeB) {
                // codeA is upper-case variant of codeB
                continue;
            }
            if (codeB >= 65 /* A */ && codeB <= 90 /* Z */ && codeB + 32 === codeA) {
                // codeB is upper-case variant of codeA
                continue;
            }
            return false;
        }
        return true;
    };
    BlockCommentCommand.prototype._createOperationsForBlockComment = function (selection, startToken, endToken, model, builder) {
        var startLineNumber = selection.startLineNumber;
        var startColumn = selection.startColumn;
        var endLineNumber = selection.endLineNumber;
        var endColumn = selection.endColumn;
        var startLineText = model.getLineContent(startLineNumber);
        var endLineText = model.getLineContent(endLineNumber);
        var startTokenIndex = startLineText.lastIndexOf(startToken, startColumn - 1 + startToken.length);
        var endTokenIndex = endLineText.indexOf(endToken, endColumn - 1 - endToken.length);
        if (startTokenIndex !== -1 && endTokenIndex !== -1) {
            if (startLineNumber === endLineNumber) {
                var lineBetweenTokens = startLineText.substring(startTokenIndex + startToken.length, endTokenIndex);
                if (lineBetweenTokens.indexOf(endToken) >= 0) {
                    // force to add a block comment
                    startTokenIndex = -1;
                    endTokenIndex = -1;
                }
            }
            else {
                var startLineAfterStartToken = startLineText.substring(startTokenIndex + startToken.length);
                var endLineBeforeEndToken = endLineText.substring(0, endTokenIndex);
                if (startLineAfterStartToken.indexOf(endToken) >= 0 || endLineBeforeEndToken.indexOf(endToken) >= 0) {
                    // force to add a block comment
                    startTokenIndex = -1;
                    endTokenIndex = -1;
                }
            }
        }
        var ops;
        if (startTokenIndex !== -1 && endTokenIndex !== -1) {
            // Consider spaces as part of the comment tokens
            if (startTokenIndex + startToken.length < startLineText.length) {
                if (startLineText.charCodeAt(startTokenIndex + startToken.length) === 32 /* Space */) {
                    // Pretend the start token contains a trailing space
                    startToken = startToken + ' ';
                }
            }
            if (endTokenIndex > 0) {
                if (endLineText.charCodeAt(endTokenIndex - 1) === 32 /* Space */) {
                    // Pretend the end token contains a leading space
                    endToken = ' ' + endToken;
                    endTokenIndex -= 1;
                }
            }
            ops = BlockCommentCommand._createRemoveBlockCommentOperations(new Range(startLineNumber, startTokenIndex + startToken.length + 1, endLineNumber, endTokenIndex + 1), startToken, endToken);
        }
        else {
            ops = BlockCommentCommand._createAddBlockCommentOperations(selection, startToken, endToken);
            this._usedEndToken = ops.length === 1 ? endToken : null;
        }
        for (var i = 0; i < ops.length; i++) {
            builder.addTrackedEditOperation(ops[i].range, ops[i].text);
        }
    };
    BlockCommentCommand._createRemoveBlockCommentOperations = function (r, startToken, endToken) {
        var res = [];
        if (!Range.isEmpty(r)) {
            // Remove block comment start
            res.push(EditOperation.delete(new Range(r.startLineNumber, r.startColumn - startToken.length, r.startLineNumber, r.startColumn)));
            // Remove block comment end
            res.push(EditOperation.delete(new Range(r.endLineNumber, r.endColumn, r.endLineNumber, r.endColumn + endToken.length)));
        }
        else {
            // Remove both continuously
            res.push(EditOperation.delete(new Range(r.startLineNumber, r.startColumn - startToken.length, r.endLineNumber, r.endColumn + endToken.length)));
        }
        return res;
    };
    BlockCommentCommand._createAddBlockCommentOperations = function (r, startToken, endToken) {
        var res = [];
        if (!Range.isEmpty(r)) {
            // Insert block comment start
            res.push(EditOperation.insert(new Position(r.startLineNumber, r.startColumn), startToken + ' '));
            // Insert block comment end
            res.push(EditOperation.insert(new Position(r.endLineNumber, r.endColumn), ' ' + endToken));
        }
        else {
            // Insert both continuously
            res.push(EditOperation.replace(new Range(r.startLineNumber, r.startColumn, r.endLineNumber, r.endColumn), startToken + '  ' + endToken));
        }
        return res;
    };
    BlockCommentCommand.prototype.getEditOperations = function (model, builder) {
        var startLineNumber = this._selection.startLineNumber;
        var startColumn = this._selection.startColumn;
        model.tokenizeIfCheap(startLineNumber);
        var languageId = model.getLanguageIdAtPosition(startLineNumber, startColumn);
        var config = LanguageConfigurationRegistry.getComments(languageId);
        if (!config || !config.blockCommentStartToken || !config.blockCommentEndToken) {
            // Mode does not support block comments
            return;
        }
        this._createOperationsForBlockComment(this._selection, config.blockCommentStartToken, config.blockCommentEndToken, model, builder);
    };
    BlockCommentCommand.prototype.computeCursorState = function (model, helper) {
        var inverseEditOperations = helper.getInverseEditOperations();
        if (inverseEditOperations.length === 2) {
            var startTokenEditOperation = inverseEditOperations[0];
            var endTokenEditOperation = inverseEditOperations[1];
            return new Selection(startTokenEditOperation.range.endLineNumber, startTokenEditOperation.range.endColumn, endTokenEditOperation.range.startLineNumber, endTokenEditOperation.range.startColumn);
        }
        else {
            var srcRange = inverseEditOperations[0].range;
            var deltaColumn = this._usedEndToken ? -this._usedEndToken.length - 1 : 0; // minus 1 space before endToken
            return new Selection(srcRange.endLineNumber, srcRange.endColumn + deltaColumn, srcRange.endLineNumber, srcRange.endColumn + deltaColumn);
        }
    };
    return BlockCommentCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var LineCommentCommand = /** @class */ (function () {
    function LineCommentCommand(selection, tabSize, type) {
        this._selection = selection;
        this._tabSize = tabSize;
        this._type = type;
        this._deltaColumn = 0;
    }
    /**
     * Do an initial pass over the lines and gather info about the line comment string.
     * Returns null if any of the lines doesn't support a line comment string.
     */
    LineCommentCommand._gatherPreflightCommentStrings = function (model, startLineNumber, endLineNumber) {
        model.tokenizeIfCheap(startLineNumber);
        var languageId = model.getLanguageIdAtPosition(startLineNumber, 1);
        var config = LanguageConfigurationRegistry.getComments(languageId);
        var commentStr = (config ? config.lineCommentToken : null);
        if (!commentStr) {
            // Mode does not support line comments
            return null;
        }
        var lines = [];
        for (var i = 0, lineCount = endLineNumber - startLineNumber + 1; i < lineCount; i++) {
            lines[i] = {
                ignore: false,
                commentStr: commentStr,
                commentStrOffset: 0,
                commentStrLength: commentStr.length
            };
        }
        return lines;
    };
    /**
     * Analyze lines and decide which lines are relevant and what the toggle should do.
     * Also, build up several offsets and lengths useful in the generation of editor operations.
     */
    LineCommentCommand._analyzeLines = function (type, model, lines, startLineNumber) {
        var onlyWhitespaceLines = true;
        var shouldRemoveComments;
        if (type === 0 /* Toggle */) {
            shouldRemoveComments = true;
        }
        else if (type === 1 /* ForceAdd */) {
            shouldRemoveComments = false;
        }
        else {
            shouldRemoveComments = true;
        }
        for (var i = 0, lineCount = lines.length; i < lineCount; i++) {
            var lineData = lines[i];
            var lineNumber = startLineNumber + i;
            var lineContent = model.getLineContent(lineNumber);
            var lineContentStartOffset = firstNonWhitespaceIndex(lineContent);
            if (lineContentStartOffset === -1) {
                // Empty or whitespace only line
                if (type === 0 /* Toggle */) {
                    lineData.ignore = true;
                }
                else if (type === 1 /* ForceAdd */) {
                    lineData.ignore = true;
                }
                else {
                    lineData.ignore = true;
                }
                lineData.commentStrOffset = lineContent.length;
                continue;
            }
            onlyWhitespaceLines = false;
            lineData.ignore = false;
            lineData.commentStrOffset = lineContentStartOffset;
            if (shouldRemoveComments && !BlockCommentCommand._haystackHasNeedleAtOffset(lineContent, lineData.commentStr, lineContentStartOffset)) {
                if (type === 0 /* Toggle */) {
                    // Every line so far has been a line comment, but this one is not
                    shouldRemoveComments = false;
                }
                else if (type === 1 /* ForceAdd */) ;
                else {
                    lineData.ignore = true;
                }
            }
            if (shouldRemoveComments) {
                var commentStrEndOffset = lineContentStartOffset + lineData.commentStrLength;
                if (commentStrEndOffset < lineContent.length && lineContent.charCodeAt(commentStrEndOffset) === 32 /* Space */) {
                    lineData.commentStrLength += 1;
                }
            }
        }
        if (type === 0 /* Toggle */ && onlyWhitespaceLines) {
            // For only whitespace lines, we insert comments
            shouldRemoveComments = false;
            // Also, no longer ignore them
            for (var i = 0, lineCount = lines.length; i < lineCount; i++) {
                lines[i].ignore = false;
            }
        }
        return {
            supported: true,
            shouldRemoveComments: shouldRemoveComments,
            lines: lines
        };
    };
    /**
     * Analyze all lines and decide exactly what to do => not supported | insert line comments | remove line comments
     */
    LineCommentCommand._gatherPreflightData = function (type, model, startLineNumber, endLineNumber) {
        var lines = LineCommentCommand._gatherPreflightCommentStrings(model, startLineNumber, endLineNumber);
        if (lines === null) {
            return {
                supported: false
            };
        }
        return LineCommentCommand._analyzeLines(type, model, lines, startLineNumber);
    };
    /**
     * Given a successful analysis, execute either insert line comments, either remove line comments
     */
    LineCommentCommand.prototype._executeLineComments = function (model, builder, data, s) {
        var ops;
        if (data.shouldRemoveComments) {
            ops = LineCommentCommand._createRemoveLineCommentsOperations(data.lines, s.startLineNumber);
        }
        else {
            LineCommentCommand._normalizeInsertionPoint(model, data.lines, s.startLineNumber, this._tabSize);
            ops = LineCommentCommand._createAddLineCommentsOperations(data.lines, s.startLineNumber);
        }
        var cursorPosition = new Position(s.positionLineNumber, s.positionColumn);
        for (var i = 0, len = ops.length; i < len; i++) {
            builder.addEditOperation(ops[i].range, ops[i].text);
            if (ops[i].range.isEmpty() && ops[i].range.getStartPosition().equals(cursorPosition)) {
                var lineContent = model.getLineContent(cursorPosition.lineNumber);
                if (lineContent.length + 1 === cursorPosition.column) {
                    this._deltaColumn = (ops[i].text || '').length;
                }
            }
        }
        this._selectionId = builder.trackSelection(s);
    };
    LineCommentCommand.prototype._attemptRemoveBlockComment = function (model, s, startToken, endToken) {
        var startLineNumber = s.startLineNumber;
        var endLineNumber = s.endLineNumber;
        var startTokenAllowedBeforeColumn = endToken.length + Math.max(model.getLineFirstNonWhitespaceColumn(s.startLineNumber), s.startColumn);
        var startTokenIndex = model.getLineContent(startLineNumber).lastIndexOf(startToken, startTokenAllowedBeforeColumn - 1);
        var endTokenIndex = model.getLineContent(endLineNumber).indexOf(endToken, s.endColumn - 1 - startToken.length);
        if (startTokenIndex !== -1 && endTokenIndex === -1) {
            endTokenIndex = model.getLineContent(startLineNumber).indexOf(endToken, startTokenIndex + startToken.length);
            endLineNumber = startLineNumber;
        }
        if (startTokenIndex === -1 && endTokenIndex !== -1) {
            startTokenIndex = model.getLineContent(endLineNumber).lastIndexOf(startToken, endTokenIndex);
            startLineNumber = endLineNumber;
        }
        if (s.isEmpty() && (startTokenIndex === -1 || endTokenIndex === -1)) {
            startTokenIndex = model.getLineContent(startLineNumber).indexOf(startToken);
            if (startTokenIndex !== -1) {
                endTokenIndex = model.getLineContent(startLineNumber).indexOf(endToken, startTokenIndex + startToken.length);
            }
        }
        // We have to adjust to possible inner white space.
        // For Space after startToken, add Space to startToken - range math will work out.
        if (startTokenIndex !== -1 && model.getLineContent(startLineNumber).charCodeAt(startTokenIndex + startToken.length) === 32 /* Space */) {
            startToken += ' ';
        }
        // For Space before endToken, add Space before endToken and shift index one left.
        if (endTokenIndex !== -1 && model.getLineContent(endLineNumber).charCodeAt(endTokenIndex - 1) === 32 /* Space */) {
            endToken = ' ' + endToken;
            endTokenIndex -= 1;
        }
        if (startTokenIndex !== -1 && endTokenIndex !== -1) {
            return BlockCommentCommand._createRemoveBlockCommentOperations(new Range(startLineNumber, startTokenIndex + startToken.length + 1, endLineNumber, endTokenIndex + 1), startToken, endToken);
        }
        return null;
    };
    /**
     * Given an unsuccessful analysis, delegate to the block comment command
     */
    LineCommentCommand.prototype._executeBlockComment = function (model, builder, s) {
        model.tokenizeIfCheap(s.startLineNumber);
        var languageId = model.getLanguageIdAtPosition(s.startLineNumber, 1);
        var config = LanguageConfigurationRegistry.getComments(languageId);
        if (!config || !config.blockCommentStartToken || !config.blockCommentEndToken) {
            // Mode does not support block comments
            return;
        }
        var startToken = config.blockCommentStartToken;
        var endToken = config.blockCommentEndToken;
        var ops = this._attemptRemoveBlockComment(model, s, startToken, endToken);
        if (!ops) {
            if (s.isEmpty()) {
                var lineContent = model.getLineContent(s.startLineNumber);
                var firstNonWhitespaceIndex$$1 = firstNonWhitespaceIndex(lineContent);
                if (firstNonWhitespaceIndex$$1 === -1) {
                    // Line is empty or contains only whitespace
                    firstNonWhitespaceIndex$$1 = lineContent.length;
                }
                ops = BlockCommentCommand._createAddBlockCommentOperations(new Range(s.startLineNumber, firstNonWhitespaceIndex$$1 + 1, s.startLineNumber, lineContent.length + 1), startToken, endToken);
            }
            else {
                ops = BlockCommentCommand._createAddBlockCommentOperations(new Range(s.startLineNumber, model.getLineFirstNonWhitespaceColumn(s.startLineNumber), s.endLineNumber, model.getLineMaxColumn(s.endLineNumber)), startToken, endToken);
            }
            if (ops.length === 1) {
                // Leave cursor after token and Space
                this._deltaColumn = startToken.length + 1;
            }
        }
        this._selectionId = builder.trackSelection(s);
        for (var i = 0; i < ops.length; i++) {
            builder.addEditOperation(ops[i].range, ops[i].text);
        }
    };
    LineCommentCommand.prototype.getEditOperations = function (model, builder) {
        var s = this._selection;
        this._moveEndPositionDown = false;
        if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
            this._moveEndPositionDown = true;
            s = s.setEndPosition(s.endLineNumber - 1, model.getLineMaxColumn(s.endLineNumber - 1));
        }
        var data = LineCommentCommand._gatherPreflightData(this._type, model, s.startLineNumber, s.endLineNumber);
        if (data.supported) {
            return this._executeLineComments(model, builder, data, s);
        }
        return this._executeBlockComment(model, builder, s);
    };
    LineCommentCommand.prototype.computeCursorState = function (model, helper) {
        var result = helper.getTrackedSelection(this._selectionId);
        if (this._moveEndPositionDown) {
            result = result.setEndPosition(result.endLineNumber + 1, 1);
        }
        return new Selection(result.selectionStartLineNumber, result.selectionStartColumn + this._deltaColumn, result.positionLineNumber, result.positionColumn + this._deltaColumn);
    };
    /**
     * Generate edit operations in the remove line comment case
     */
    LineCommentCommand._createRemoveLineCommentsOperations = function (lines, startLineNumber) {
        var res = [];
        for (var i = 0, len = lines.length; i < len; i++) {
            var lineData = lines[i];
            if (lineData.ignore) {
                continue;
            }
            res.push(EditOperation.delete(new Range(startLineNumber + i, lineData.commentStrOffset + 1, startLineNumber + i, lineData.commentStrOffset + lineData.commentStrLength + 1)));
        }
        return res;
    };
    /**
     * Generate edit operations in the add line comment case
     */
    LineCommentCommand._createAddLineCommentsOperations = function (lines, startLineNumber) {
        var res = [];
        for (var i = 0, len = lines.length; i < len; i++) {
            var lineData = lines[i];
            if (lineData.ignore) {
                continue;
            }
            res.push(EditOperation.insert(new Position(startLineNumber + i, lineData.commentStrOffset + 1), lineData.commentStr + ' '));
        }
        return res;
    };
    // TODO@Alex -> duplicated in characterHardWrappingLineMapper
    LineCommentCommand.nextVisibleColumn = function (currentVisibleColumn, tabSize, isTab, columnSize) {
        if (isTab) {
            return currentVisibleColumn + (tabSize - (currentVisibleColumn % tabSize));
        }
        return currentVisibleColumn + columnSize;
    };
    /**
     * Adjust insertion points to have them vertically aligned in the add line comment case
     */
    LineCommentCommand._normalizeInsertionPoint = function (model, lines, startLineNumber, tabSize) {
        var minVisibleColumn = Number.MAX_VALUE;
        var j;
        var lenJ;
        for (var i = 0, len = lines.length; i < len; i++) {
            if (lines[i].ignore) {
                continue;
            }
            var lineContent = model.getLineContent(startLineNumber + i);
            var currentVisibleColumn = 0;
            for (var j_1 = 0, lenJ_1 = lines[i].commentStrOffset; currentVisibleColumn < minVisibleColumn && j_1 < lenJ_1; j_1++) {
                currentVisibleColumn = LineCommentCommand.nextVisibleColumn(currentVisibleColumn, tabSize, lineContent.charCodeAt(j_1) === 9 /* Tab */, 1);
            }
            if (currentVisibleColumn < minVisibleColumn) {
                minVisibleColumn = currentVisibleColumn;
            }
        }
        minVisibleColumn = Math.floor(minVisibleColumn / tabSize) * tabSize;
        for (var i = 0, len = lines.length; i < len; i++) {
            if (lines[i].ignore) {
                continue;
            }
            var lineContent = model.getLineContent(startLineNumber + i);
            var currentVisibleColumn = 0;
            for (j = 0, lenJ = lines[i].commentStrOffset; currentVisibleColumn < minVisibleColumn && j < lenJ; j++) {
                currentVisibleColumn = LineCommentCommand.nextVisibleColumn(currentVisibleColumn, tabSize, lineContent.charCodeAt(j) === 9 /* Tab */, 1);
            }
            if (currentVisibleColumn > minVisibleColumn) {
                lines[i].commentStrOffset = j - 1;
            }
            else {
                lines[i].commentStrOffset = j;
            }
        }
    };
    return LineCommentCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1f = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CommentLineAction = /** @class */ (function (_super) {
    __extends$1f(CommentLineAction, _super);
    function CommentLineAction(type, opts) {
        var _this = _super.call(this, opts) || this;
        _this._type = type;
        return _this;
    }
    CommentLineAction.prototype.run = function (accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        var model = editor.getModel();
        var commands = [];
        var selections = editor.getSelections();
        var opts = model.getOptions();
        for (var i = 0; i < selections.length; i++) {
            commands.push(new LineCommentCommand(selections[i], opts.tabSize, this._type));
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return CommentLineAction;
}(EditorAction));
var ToggleCommentLineAction = /** @class */ (function (_super) {
    __extends$1f(ToggleCommentLineAction, _super);
    function ToggleCommentLineAction() {
        return _super.call(this, 0 /* Toggle */, {
            id: 'editor.action.commentLine',
            label: localize('comment.line', "Toggle Line Comment"),
            alias: 'Toggle Line Comment',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 85 /* US_SLASH */,
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarEditMenu,
                group: '5_insert',
                title: localize({ key: 'miToggleLineComment', comment: ['&& denotes a mnemonic'] }, "&&Toggle Line Comment"),
                order: 1
            }
        }) || this;
    }
    return ToggleCommentLineAction;
}(CommentLineAction));
var AddLineCommentAction = /** @class */ (function (_super) {
    __extends$1f(AddLineCommentAction, _super);
    function AddLineCommentAction() {
        return _super.call(this, 1 /* ForceAdd */, {
            id: 'editor.action.addCommentLine',
            label: localize('comment.line.add', "Add Line Comment"),
            alias: 'Add Line Comment',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 33 /* KEY_C */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return AddLineCommentAction;
}(CommentLineAction));
var RemoveLineCommentAction = /** @class */ (function (_super) {
    __extends$1f(RemoveLineCommentAction, _super);
    function RemoveLineCommentAction() {
        return _super.call(this, 2 /* ForceRemove */, {
            id: 'editor.action.removeCommentLine',
            label: localize('comment.line.remove', "Remove Line Comment"),
            alias: 'Remove Line Comment',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 51 /* KEY_U */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return RemoveLineCommentAction;
}(CommentLineAction));
var BlockCommentAction = /** @class */ (function (_super) {
    __extends$1f(BlockCommentAction, _super);
    function BlockCommentAction() {
        return _super.call(this, {
            id: 'editor.action.blockComment',
            label: localize('comment.block', "Toggle Block Comment"),
            alias: 'Toggle Block Comment',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 1024 /* Shift */ | 512 /* Alt */ | 31 /* KEY_A */,
                linux: { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 31 /* KEY_A */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarEditMenu,
                group: '5_insert',
                title: localize({ key: 'miToggleBlockComment', comment: ['&& denotes a mnemonic'] }, "Toggle &&Block Comment"),
                order: 2
            }
        }) || this;
    }
    BlockCommentAction.prototype.run = function (accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        var commands = [];
        var selections = editor.getSelections();
        for (var i = 0; i < selections.length; i++) {
            commands.push(new BlockCommentCommand(selections[i]));
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return BlockCommentAction;
}(EditorAction));
registerEditorAction(ToggleCommentLineAction);
registerEditorAction(AddLineCommentAction);
registerEditorAction(RemoveLineCommentAction);
registerEditorAction(BlockCommentAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IContextViewService = createDecorator('contextViewService');
var IContextMenuService = createDecorator('contextMenuService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IKeybindingService = createDecorator('keybindingService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1g = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$4 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$3 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ContextMenuController = /** @class */ (function () {
    function ContextMenuController(editor, _contextMenuService, _contextViewService, _contextKeyService, _keybindingService, _menuService) {
        var _this = this;
        this._contextMenuService = _contextMenuService;
        this._contextViewService = _contextViewService;
        this._contextKeyService = _contextKeyService;
        this._keybindingService = _keybindingService;
        this._menuService = _menuService;
        this._toDispose = [];
        this._contextMenuIsBeingShownCount = 0;
        this._editor = editor;
        this._toDispose.push(this._editor.onContextMenu(function (e) { return _this._onContextMenu(e); }));
        this._toDispose.push(this._editor.onDidScrollChange(function (e) {
            if (_this._contextMenuIsBeingShownCount > 0) {
                _this._contextViewService.hideContextView();
            }
        }));
        this._toDispose.push(this._editor.onKeyDown(function (e) {
            if (e.keyCode === 58 /* ContextMenu */) {
                // Chrome is funny like that
                e.preventDefault();
                e.stopPropagation();
                _this.showContextMenu();
            }
        }));
    }
    ContextMenuController.get = function (editor) {
        return editor.getContribution(ContextMenuController.ID);
    };
    ContextMenuController.prototype._onContextMenu = function (e) {
        if (!this._editor.getConfiguration().contribInfo.contextmenu) {
            this._editor.focus();
            // Ensure the cursor is at the position of the mouse click
            if (e.target.position && !this._editor.getSelection().containsPosition(e.target.position)) {
                this._editor.setPosition(e.target.position);
            }
            return; // Context menu is turned off through configuration
        }
        if (e.target.type === 12 /* OVERLAY_WIDGET */) {
            return; // allow native menu on widgets to support right click on input field for example in find
        }
        e.event.preventDefault();
        if (e.target.type !== 6 /* CONTENT_TEXT */ && e.target.type !== 7 /* CONTENT_EMPTY */ && e.target.type !== 1 /* TEXTAREA */) {
            return; // only support mouse click into text or native context menu key for now
        }
        // Ensure the editor gets focus if it hasn't, so the right events are being sent to other contributions
        this._editor.focus();
        // Ensure the cursor is at the position of the mouse click
        if (e.target.position && !this._editor.getSelection().containsPosition(e.target.position)) {
            this._editor.setPosition(e.target.position);
        }
        // Unless the user triggerd the context menu through Shift+F10, use the mouse position as menu position
        var anchor;
        if (e.target.type !== 1 /* TEXTAREA */) {
            anchor = { x: e.event.posx - 1, width: 2, y: e.event.posy - 1, height: 2 };
        }
        // Show the context menu
        this.showContextMenu(anchor);
    };
    ContextMenuController.prototype.showContextMenu = function (anchor) {
        if (!this._editor.getConfiguration().contribInfo.contextmenu) {
            return; // Context menu is turned off through configuration
        }
        if (!this._contextMenuService) {
            this._editor.focus();
            return; // We need the context menu service to function
        }
        // Find actions available for menu
        var menuActions = this._getMenuActions();
        // Show menu if we have actions to show
        if (menuActions.length > 0) {
            this._doShowContextMenu(menuActions, anchor);
        }
    };
    ContextMenuController.prototype._getMenuActions = function () {
        var result = [];
        var contextMenu = this._menuService.createMenu(MenuId.EditorContext, this._contextKeyService);
        var groups = contextMenu.getActions({ arg: this._editor.getModel().uri });
        contextMenu.dispose();
        for (var _i = 0, groups_1 = groups; _i < groups_1.length; _i++) {
            var group = groups_1[_i];
            var actions = group[1];
            result.push.apply(result, actions);
            result.push(new Separator());
        }
        result.pop(); // remove last separator
        return result;
    };
    ContextMenuController.prototype._doShowContextMenu = function (actions, anchor) {
        var _this = this;
        if (anchor === void 0) { anchor = null; }
        // Disable hover
        var oldHoverSetting = this._editor.getConfiguration().contribInfo.hover;
        this._editor.updateOptions({
            hover: {
                enabled: false
            }
        });
        if (!anchor) {
            // Ensure selection is visible
            this._editor.revealPosition(this._editor.getPosition(), 1 /* Immediate */);
            this._editor.render();
            var cursorCoords = this._editor.getScrolledVisiblePosition(this._editor.getPosition());
            // Translate to absolute editor position
            var editorCoords = getDomNodePagePosition(this._editor.getDomNode());
            var posx = editorCoords.left + cursorCoords.left;
            var posy = editorCoords.top + cursorCoords.top + cursorCoords.height;
            anchor = { x: posx, y: posy };
        }
        // Show menu
        this._contextMenuIsBeingShownCount++;
        this._contextMenuService.showContextMenu({
            getAnchor: function () { return anchor; },
            getActions: function () {
                return Promise.resolve(actions);
            },
            getActionItem: function (action) {
                var keybinding = _this._keybindingFor(action);
                if (keybinding) {
                    return new ActionItem(action, action, { label: true, keybinding: keybinding.getLabel(), isMenu: true });
                }
                var customActionItem = action;
                if (typeof customActionItem.getActionItem === 'function') {
                    return customActionItem.getActionItem();
                }
                return new ActionItem(action, action, { icon: true, label: true, isMenu: true });
            },
            getKeyBinding: function (action) {
                return _this._keybindingFor(action);
            },
            onHide: function (wasCancelled) {
                _this._contextMenuIsBeingShownCount--;
                _this._editor.focus();
                _this._editor.updateOptions({
                    hover: oldHoverSetting
                });
            }
        });
    };
    ContextMenuController.prototype._keybindingFor = function (action) {
        return this._keybindingService.lookupKeybinding(action.id);
    };
    ContextMenuController.prototype.getId = function () {
        return ContextMenuController.ID;
    };
    ContextMenuController.prototype.dispose = function () {
        if (this._contextMenuIsBeingShownCount > 0) {
            this._contextViewService.hideContextView();
        }
        this._toDispose = dispose(this._toDispose);
    };
    ContextMenuController.ID = 'editor.contrib.contextmenu';
    ContextMenuController = __decorate$4([
        __param$3(1, IContextMenuService),
        __param$3(2, IContextViewService),
        __param$3(3, IContextKeyService),
        __param$3(4, IKeybindingService),
        __param$3(5, IMenuService)
    ], ContextMenuController);
    return ContextMenuController;
}());
var ShowContextMenu = /** @class */ (function (_super) {
    __extends$1g(ShowContextMenu, _super);
    function ShowContextMenu() {
        return _super.call(this, {
            id: 'editor.action.showContextMenu',
            label: localize('action.showContextMenu.label', "Show Editor Context Menu"),
            alias: 'Show Editor Context Menu',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 1024 /* Shift */ | 68 /* F10 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    ShowContextMenu.prototype.run = function (accessor, editor) {
        var contribution = ContextMenuController.get(editor);
        contribution.showContextMenu();
    };
    return ShowContextMenu;
}(EditorAction));
registerEditorContribution(ContextMenuController);
registerEditorAction(ShowContextMenu);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var FindDecorations = /** @class */ (function () {
    function FindDecorations(editor) {
        this._editor = editor;
        this._decorations = [];
        this._overviewRulerApproximateDecorations = [];
        this._findScopeDecorationId = null;
        this._rangeHighlightDecorationId = null;
        this._highlightedDecorationId = null;
        this._startPosition = this._editor.getPosition();
    }
    FindDecorations.prototype.dispose = function () {
        this._editor.deltaDecorations(this._allDecorations(), []);
        this._decorations = [];
        this._overviewRulerApproximateDecorations = [];
        this._findScopeDecorationId = null;
        this._rangeHighlightDecorationId = null;
        this._highlightedDecorationId = null;
    };
    FindDecorations.prototype.reset = function () {
        this._decorations = [];
        this._overviewRulerApproximateDecorations = [];
        this._findScopeDecorationId = null;
        this._rangeHighlightDecorationId = null;
        this._highlightedDecorationId = null;
    };
    FindDecorations.prototype.getCount = function () {
        return this._decorations.length;
    };
    FindDecorations.prototype.getFindScope = function () {
        if (this._findScopeDecorationId) {
            return this._editor.getModel().getDecorationRange(this._findScopeDecorationId);
        }
        return null;
    };
    FindDecorations.prototype.getStartPosition = function () {
        return this._startPosition;
    };
    FindDecorations.prototype.setStartPosition = function (newStartPosition) {
        this._startPosition = newStartPosition;
        this.setCurrentFindMatch(null);
    };
    FindDecorations.prototype._getDecorationIndex = function (decorationId) {
        var index = this._decorations.indexOf(decorationId);
        if (index >= 0) {
            return index + 1;
        }
        return 1;
    };
    FindDecorations.prototype.getCurrentMatchesPosition = function (desiredRange) {
        var candidates = this._editor.getModel().getDecorationsInRange(desiredRange);
        for (var i = 0, len = candidates.length; i < len; i++) {
            var candidate = candidates[i];
            var candidateOpts = candidate.options;
            if (candidateOpts === FindDecorations._FIND_MATCH_DECORATION || candidateOpts === FindDecorations._CURRENT_FIND_MATCH_DECORATION) {
                return this._getDecorationIndex(candidate.id);
            }
        }
        return 1;
    };
    FindDecorations.prototype.setCurrentFindMatch = function (nextMatch) {
        var _this = this;
        var newCurrentDecorationId = null;
        var matchPosition = 0;
        if (nextMatch) {
            for (var i = 0, len = this._decorations.length; i < len; i++) {
                var range = this._editor.getModel().getDecorationRange(this._decorations[i]);
                if (nextMatch.equalsRange(range)) {
                    newCurrentDecorationId = this._decorations[i];
                    matchPosition = (i + 1);
                    break;
                }
            }
        }
        if (this._highlightedDecorationId !== null || newCurrentDecorationId !== null) {
            this._editor.changeDecorations(function (changeAccessor) {
                if (_this._highlightedDecorationId !== null) {
                    changeAccessor.changeDecorationOptions(_this._highlightedDecorationId, FindDecorations._FIND_MATCH_DECORATION);
                    _this._highlightedDecorationId = null;
                }
                if (newCurrentDecorationId !== null) {
                    _this._highlightedDecorationId = newCurrentDecorationId;
                    changeAccessor.changeDecorationOptions(_this._highlightedDecorationId, FindDecorations._CURRENT_FIND_MATCH_DECORATION);
                }
                if (_this._rangeHighlightDecorationId !== null) {
                    changeAccessor.removeDecoration(_this._rangeHighlightDecorationId);
                    _this._rangeHighlightDecorationId = null;
                }
                if (newCurrentDecorationId !== null) {
                    var rng = _this._editor.getModel().getDecorationRange(newCurrentDecorationId);
                    if (rng.startLineNumber !== rng.endLineNumber && rng.endColumn === 1) {
                        var lineBeforeEnd = rng.endLineNumber - 1;
                        var lineBeforeEndMaxColumn = _this._editor.getModel().getLineMaxColumn(lineBeforeEnd);
                        rng = new Range(rng.startLineNumber, rng.startColumn, lineBeforeEnd, lineBeforeEndMaxColumn);
                    }
                    _this._rangeHighlightDecorationId = changeAccessor.addDecoration(rng, FindDecorations._RANGE_HIGHLIGHT_DECORATION);
                }
            });
        }
        return matchPosition;
    };
    FindDecorations.prototype.set = function (findMatches, findScope) {
        var _this = this;
        this._editor.changeDecorations(function (accessor) {
            var findMatchesOptions = FindDecorations._FIND_MATCH_DECORATION;
            var newOverviewRulerApproximateDecorations = [];
            if (findMatches.length > 1000) {
                // we go into a mode where the overview ruler gets "approximate" decorations
                // the reason is that the overview ruler paints all the decorations in the file and we don't want to cause freezes
                findMatchesOptions = FindDecorations._FIND_MATCH_NO_OVERVIEW_DECORATION;
                // approximate a distance in lines where matches should be merged
                var lineCount = _this._editor.getModel().getLineCount();
                var height = _this._editor.getLayoutInfo().height;
                var approxPixelsPerLine = height / lineCount;
                var mergeLinesDelta = Math.max(2, Math.ceil(3 / approxPixelsPerLine));
                // merge decorations as much as possible
                var prevStartLineNumber = findMatches[0].range.startLineNumber;
                var prevEndLineNumber = findMatches[0].range.endLineNumber;
                for (var i = 1, len = findMatches.length; i < len; i++) {
                    var range = findMatches[i].range;
                    if (prevEndLineNumber + mergeLinesDelta >= range.startLineNumber) {
                        if (range.endLineNumber > prevEndLineNumber) {
                            prevEndLineNumber = range.endLineNumber;
                        }
                    }
                    else {
                        newOverviewRulerApproximateDecorations.push({
                            range: new Range(prevStartLineNumber, 1, prevEndLineNumber, 1),
                            options: FindDecorations._FIND_MATCH_ONLY_OVERVIEW_DECORATION
                        });
                        prevStartLineNumber = range.startLineNumber;
                        prevEndLineNumber = range.endLineNumber;
                    }
                }
                newOverviewRulerApproximateDecorations.push({
                    range: new Range(prevStartLineNumber, 1, prevEndLineNumber, 1),
                    options: FindDecorations._FIND_MATCH_ONLY_OVERVIEW_DECORATION
                });
            }
            // Find matches
            var newFindMatchesDecorations = new Array(findMatches.length);
            for (var i = 0, len = findMatches.length; i < len; i++) {
                newFindMatchesDecorations[i] = {
                    range: findMatches[i].range,
                    options: findMatchesOptions
                };
            }
            _this._decorations = accessor.deltaDecorations(_this._decorations, newFindMatchesDecorations);
            // Overview ruler approximate decorations
            _this._overviewRulerApproximateDecorations = accessor.deltaDecorations(_this._overviewRulerApproximateDecorations, newOverviewRulerApproximateDecorations);
            // Range highlight
            if (_this._rangeHighlightDecorationId) {
                accessor.removeDecoration(_this._rangeHighlightDecorationId);
                _this._rangeHighlightDecorationId = null;
            }
            // Find scope
            if (_this._findScopeDecorationId) {
                accessor.removeDecoration(_this._findScopeDecorationId);
                _this._findScopeDecorationId = null;
            }
            if (findScope) {
                _this._findScopeDecorationId = accessor.addDecoration(findScope, FindDecorations._FIND_SCOPE_DECORATION);
            }
        });
    };
    FindDecorations.prototype.matchBeforePosition = function (position) {
        if (this._decorations.length === 0) {
            return null;
        }
        for (var i = this._decorations.length - 1; i >= 0; i--) {
            var decorationId = this._decorations[i];
            var r = this._editor.getModel().getDecorationRange(decorationId);
            if (!r || r.endLineNumber > position.lineNumber) {
                continue;
            }
            if (r.endLineNumber < position.lineNumber) {
                return r;
            }
            if (r.endColumn > position.column) {
                continue;
            }
            return r;
        }
        return this._editor.getModel().getDecorationRange(this._decorations[this._decorations.length - 1]);
    };
    FindDecorations.prototype.matchAfterPosition = function (position) {
        if (this._decorations.length === 0) {
            return null;
        }
        for (var i = 0, len = this._decorations.length; i < len; i++) {
            var decorationId = this._decorations[i];
            var r = this._editor.getModel().getDecorationRange(decorationId);
            if (!r || r.startLineNumber < position.lineNumber) {
                continue;
            }
            if (r.startLineNumber > position.lineNumber) {
                return r;
            }
            if (r.startColumn < position.column) {
                continue;
            }
            return r;
        }
        return this._editor.getModel().getDecorationRange(this._decorations[0]);
    };
    FindDecorations.prototype._allDecorations = function () {
        var result = [];
        result = result.concat(this._decorations);
        result = result.concat(this._overviewRulerApproximateDecorations);
        if (this._findScopeDecorationId) {
            result.push(this._findScopeDecorationId);
        }
        if (this._rangeHighlightDecorationId) {
            result.push(this._rangeHighlightDecorationId);
        }
        return result;
    };
    FindDecorations._CURRENT_FIND_MATCH_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        zIndex: 13,
        className: 'currentFindMatch',
        showIfCollapsed: true,
        overviewRuler: {
            color: themeColorFromId(overviewRulerFindMatchForeground),
            position: OverviewRulerLane.Center
        }
    });
    FindDecorations._FIND_MATCH_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        className: 'findMatch',
        showIfCollapsed: true,
        overviewRuler: {
            color: themeColorFromId(overviewRulerFindMatchForeground),
            position: OverviewRulerLane.Center
        }
    });
    FindDecorations._FIND_MATCH_NO_OVERVIEW_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        className: 'findMatch',
        showIfCollapsed: true
    });
    FindDecorations._FIND_MATCH_ONLY_OVERVIEW_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        overviewRuler: {
            color: themeColorFromId(overviewRulerFindMatchForeground),
            position: OverviewRulerLane.Center
        }
    });
    FindDecorations._RANGE_HIGHLIGHT_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        className: 'rangeHighlight',
        isWholeLine: true
    });
    FindDecorations._FIND_SCOPE_DECORATION = ModelDecorationOptions.register({
        className: 'findScope',
        isWholeLine: true
    });
    return FindDecorations;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ReplaceAllCommand = /** @class */ (function () {
    function ReplaceAllCommand(editorSelection, ranges, replaceStrings) {
        this._editorSelection = editorSelection;
        this._ranges = ranges;
        this._replaceStrings = replaceStrings;
    }
    ReplaceAllCommand.prototype.getEditOperations = function (model, builder) {
        if (this._ranges.length > 0) {
            // Collect all edit operations
            var ops = [];
            for (var i = 0; i < this._ranges.length; i++) {
                ops.push({
                    range: this._ranges[i],
                    text: this._replaceStrings[i]
                });
            }
            // Sort them in ascending order by range starts
            ops.sort(function (o1, o2) {
                return Range.compareRangesUsingStarts(o1.range, o2.range);
            });
            // Merge operations that touch each other
            var resultOps = [];
            var previousOp = ops[0];
            for (var i = 1; i < ops.length; i++) {
                if (previousOp.range.endLineNumber === ops[i].range.startLineNumber && previousOp.range.endColumn === ops[i].range.startColumn) {
                    // These operations are one after another and can be merged
                    previousOp.range = previousOp.range.plusRange(ops[i].range);
                    previousOp.text = previousOp.text + ops[i].text;
                }
                else {
                    resultOps.push(previousOp);
                    previousOp = ops[i];
                }
            }
            resultOps.push(previousOp);
            for (var i = 0; i < resultOps.length; i++) {
                builder.addEditOperation(resultOps[i].range, resultOps[i].text);
            }
        }
        this._trackedEditorSelectionId = builder.trackSelection(this._editorSelection);
    };
    ReplaceAllCommand.prototype.computeCursorState = function (model, helper) {
        return helper.getTrackedSelection(this._trackedEditorSelectionId);
    };
    return ReplaceAllCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Assigned when the replace pattern is entirely static.
 */
var StaticValueReplacePattern = /** @class */ (function () {
    function StaticValueReplacePattern(staticValue) {
        this.staticValue = staticValue;
        this.kind = 0 /* StaticValue */;
    }
    return StaticValueReplacePattern;
}());
/**
 * Assigned when the replace pattern has replacemend patterns.
 */
var DynamicPiecesReplacePattern = /** @class */ (function () {
    function DynamicPiecesReplacePattern(pieces) {
        this.pieces = pieces;
        this.kind = 1 /* DynamicPieces */;
    }
    return DynamicPiecesReplacePattern;
}());
var ReplacePattern = /** @class */ (function () {
    function ReplacePattern(pieces) {
        if (!pieces || pieces.length === 0) {
            this._state = new StaticValueReplacePattern('');
        }
        else if (pieces.length === 1 && pieces[0].staticValue !== null) {
            this._state = new StaticValueReplacePattern(pieces[0].staticValue);
        }
        else {
            this._state = new DynamicPiecesReplacePattern(pieces);
        }
    }
    ReplacePattern.fromStaticValue = function (value) {
        return new ReplacePattern([ReplacePiece.staticValue(value)]);
    };
    Object.defineProperty(ReplacePattern.prototype, "hasReplacementPatterns", {
        get: function () {
            return (this._state.kind === 1 /* DynamicPieces */);
        },
        enumerable: true,
        configurable: true
    });
    ReplacePattern.prototype.buildReplaceString = function (matches) {
        if (this._state.kind === 0 /* StaticValue */) {
            return this._state.staticValue;
        }
        var result = '';
        for (var i = 0, len = this._state.pieces.length; i < len; i++) {
            var piece = this._state.pieces[i];
            if (piece.staticValue !== null) {
                // static value ReplacePiece
                result += piece.staticValue;
                continue;
            }
            // match index ReplacePiece
            result += ReplacePattern._substitute(piece.matchIndex, matches);
        }
        return result;
    };
    ReplacePattern._substitute = function (matchIndex, matches) {
        if (matches === null) {
            return '';
        }
        if (matchIndex === 0) {
            return matches[0];
        }
        var remainder = '';
        while (matchIndex > 0) {
            if (matchIndex < matches.length) {
                // A match can be undefined
                var match = (matches[matchIndex] || '');
                return match + remainder;
            }
            remainder = String(matchIndex % 10) + remainder;
            matchIndex = Math.floor(matchIndex / 10);
        }
        return '$' + remainder;
    };
    return ReplacePattern;
}());
/**
 * A replace piece can either be a static string or an index to a specific match.
 */
var ReplacePiece = /** @class */ (function () {
    function ReplacePiece(staticValue, matchIndex) {
        this.staticValue = staticValue;
        this.matchIndex = matchIndex;
    }
    ReplacePiece.staticValue = function (value) {
        return new ReplacePiece(value, -1);
    };
    ReplacePiece.matchIndex = function (index) {
        return new ReplacePiece(null, index);
    };
    return ReplacePiece;
}());
var ReplacePieceBuilder = /** @class */ (function () {
    function ReplacePieceBuilder(source) {
        this._source = source;
        this._lastCharIndex = 0;
        this._result = [];
        this._resultLen = 0;
        this._currentStaticPiece = '';
    }
    ReplacePieceBuilder.prototype.emitUnchanged = function (toCharIndex) {
        this._emitStatic(this._source.substring(this._lastCharIndex, toCharIndex));
        this._lastCharIndex = toCharIndex;
    };
    ReplacePieceBuilder.prototype.emitStatic = function (value, toCharIndex) {
        this._emitStatic(value);
        this._lastCharIndex = toCharIndex;
    };
    ReplacePieceBuilder.prototype._emitStatic = function (value) {
        if (value.length === 0) {
            return;
        }
        this._currentStaticPiece += value;
    };
    ReplacePieceBuilder.prototype.emitMatchIndex = function (index, toCharIndex) {
        if (this._currentStaticPiece.length !== 0) {
            this._result[this._resultLen++] = ReplacePiece.staticValue(this._currentStaticPiece);
            this._currentStaticPiece = '';
        }
        this._result[this._resultLen++] = ReplacePiece.matchIndex(index);
        this._lastCharIndex = toCharIndex;
    };
    ReplacePieceBuilder.prototype.finalize = function () {
        this.emitUnchanged(this._source.length);
        if (this._currentStaticPiece.length !== 0) {
            this._result[this._resultLen++] = ReplacePiece.staticValue(this._currentStaticPiece);
            this._currentStaticPiece = '';
        }
        return new ReplacePattern(this._result);
    };
    return ReplacePieceBuilder;
}());
/**
 * \n			=> inserts a LF
 * \t			=> inserts a TAB
 * \\			=> inserts a "\".
 * $$			=> inserts a "$".
 * $& and $0	=> inserts the matched substring.
 * $n			=> Where n is a non-negative integer lesser than 100, inserts the nth parenthesized submatch string
 * everything else stays untouched
 *
 * Also see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/replace#Specifying_a_string_as_a_parameter
 */
function parseReplaceString(replaceString) {
    if (!replaceString || replaceString.length === 0) {
        return new ReplacePattern(null);
    }
    var result = new ReplacePieceBuilder(replaceString);
    for (var i = 0, len = replaceString.length; i < len; i++) {
        var chCode = replaceString.charCodeAt(i);
        if (chCode === 92 /* Backslash */) {
            // move to next char
            i++;
            if (i >= len) {
                // string ends with a \
                break;
            }
            var nextChCode = replaceString.charCodeAt(i);
            // let replaceWithCharacter: string | null = null;
            switch (nextChCode) {
                case 92 /* Backslash */:
                    // \\ => inserts a "\"
                    result.emitUnchanged(i - 1);
                    result.emitStatic('\\', i + 1);
                    break;
                case 110 /* n */:
                    // \n => inserts a LF
                    result.emitUnchanged(i - 1);
                    result.emitStatic('\n', i + 1);
                    break;
                case 116 /* t */:
                    // \t => inserts a TAB
                    result.emitUnchanged(i - 1);
                    result.emitStatic('\t', i + 1);
                    break;
            }
            continue;
        }
        if (chCode === 36 /* DollarSign */) {
            // move to next char
            i++;
            if (i >= len) {
                // string ends with a $
                break;
            }
            var nextChCode = replaceString.charCodeAt(i);
            if (nextChCode === 36 /* DollarSign */) {
                // $$ => inserts a "$"
                result.emitUnchanged(i - 1);
                result.emitStatic('$', i + 1);
                continue;
            }
            if (nextChCode === 48 /* Digit0 */ || nextChCode === 38 /* Ampersand */) {
                // $& and $0 => inserts the matched substring.
                result.emitUnchanged(i - 1);
                result.emitMatchIndex(0, i + 1);
                continue;
            }
            if (49 /* Digit1 */ <= nextChCode && nextChCode <= 57 /* Digit9 */) {
                // $n
                var matchIndex = nextChCode - 48 /* Digit0 */;
                // peek next char to probe for $nn
                if (i + 1 < len) {
                    var nextNextChCode = replaceString.charCodeAt(i + 1);
                    if (48 /* Digit0 */ <= nextNextChCode && nextNextChCode <= 57 /* Digit9 */) {
                        // $nn
                        // move to next char
                        i++;
                        matchIndex = matchIndex * 10 + (nextNextChCode - 48 /* Digit0 */);
                        result.emitUnchanged(i - 2);
                        result.emitMatchIndex(matchIndex, i + 1);
                        continue;
                    }
                }
                result.emitUnchanged(i - 1);
                result.emitMatchIndex(matchIndex, i + 1);
                continue;
            }
        }
    }
    return result.finalize();
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CONTEXT_FIND_WIDGET_VISIBLE = new RawContextKey('findWidgetVisible', false);
// Keep ContextKey use of 'Focussed' to not break when clauses
var CONTEXT_FIND_INPUT_FOCUSED = new RawContextKey('findInputFocussed', false);
var CONTEXT_REPLACE_INPUT_FOCUSED = new RawContextKey('replaceInputFocussed', false);
var ToggleCaseSensitiveKeybinding = {
    primary: 512 /* Alt */ | 33 /* KEY_C */,
    mac: { primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 33 /* KEY_C */ }
};
var ToggleWholeWordKeybinding = {
    primary: 512 /* Alt */ | 53 /* KEY_W */,
    mac: { primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 53 /* KEY_W */ }
};
var ToggleRegexKeybinding = {
    primary: 512 /* Alt */ | 48 /* KEY_R */,
    mac: { primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 48 /* KEY_R */ }
};
var ToggleSearchScopeKeybinding = {
    primary: 512 /* Alt */ | 42 /* KEY_L */,
    mac: { primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 42 /* KEY_L */ }
};
var FIND_IDS = {
    StartFindAction: 'actions.find',
    StartFindWithSelection: 'actions.findWithSelection',
    NextMatchFindAction: 'editor.action.nextMatchFindAction',
    PreviousMatchFindAction: 'editor.action.previousMatchFindAction',
    NextSelectionMatchFindAction: 'editor.action.nextSelectionMatchFindAction',
    PreviousSelectionMatchFindAction: 'editor.action.previousSelectionMatchFindAction',
    StartFindReplaceAction: 'editor.action.startFindReplaceAction',
    CloseFindWidgetCommand: 'closeFindWidget',
    ToggleCaseSensitiveCommand: 'toggleFindCaseSensitive',
    ToggleWholeWordCommand: 'toggleFindWholeWord',
    ToggleRegexCommand: 'toggleFindRegex',
    ToggleSearchScopeCommand: 'toggleFindInSelection',
    ReplaceOneAction: 'editor.action.replaceOne',
    ReplaceAllAction: 'editor.action.replaceAll',
    SelectAllMatchesAction: 'editor.action.selectAllMatches'
};
var MATCHES_LIMIT = 19999;
var RESEARCH_DELAY = 240;
var FindModelBoundToEditorModel = /** @class */ (function () {
    function FindModelBoundToEditorModel(editor, state) {
        var _this = this;
        this._editor = editor;
        this._state = state;
        this._toDispose = [];
        this._isDisposed = false;
        this._startSearchingTimer = new TimeoutTimer();
        this._decorations = new FindDecorations(editor);
        this._toDispose.push(this._decorations);
        this._updateDecorationsScheduler = new RunOnceScheduler(function () { return _this.research(false); }, 100);
        this._toDispose.push(this._updateDecorationsScheduler);
        this._toDispose.push(this._editor.onDidChangeCursorPosition(function (e) {
            if (e.reason === 3 /* Explicit */
                || e.reason === 5 /* Undo */
                || e.reason === 6 /* Redo */) {
                _this._decorations.setStartPosition(_this._editor.getPosition());
            }
        }));
        this._ignoreModelContentChanged = false;
        this._toDispose.push(this._editor.onDidChangeModelContent(function (e) {
            if (_this._ignoreModelContentChanged) {
                return;
            }
            if (e.isFlush) {
                // a model.setValue() was called
                _this._decorations.reset();
            }
            _this._decorations.setStartPosition(_this._editor.getPosition());
            _this._updateDecorationsScheduler.schedule();
        }));
        this._toDispose.push(this._state.onFindReplaceStateChange(function (e) { return _this._onStateChanged(e); }));
        this.research(false, this._state.searchScope);
    }
    FindModelBoundToEditorModel.prototype.dispose = function () {
        this._isDisposed = true;
        dispose(this._startSearchingTimer);
        this._toDispose = dispose(this._toDispose);
    };
    FindModelBoundToEditorModel.prototype._onStateChanged = function (e) {
        var _this = this;
        if (this._isDisposed) {
            // The find model is disposed during a find state changed event
            return;
        }
        if (!this._editor.getModel()) {
            // The find model will be disposed momentarily
            return;
        }
        if (e.searchString || e.isReplaceRevealed || e.isRegex || e.wholeWord || e.matchCase || e.searchScope) {
            var model = this._editor.getModel();
            if (model.isTooLargeForSyncing()) {
                this._startSearchingTimer.cancel();
                this._startSearchingTimer.setIfNotSet(function () {
                    if (e.searchScope) {
                        _this.research(e.moveCursor, _this._state.searchScope);
                    }
                    else {
                        _this.research(e.moveCursor);
                    }
                }, RESEARCH_DELAY);
            }
            else {
                if (e.searchScope) {
                    this.research(e.moveCursor, this._state.searchScope);
                }
                else {
                    this.research(e.moveCursor);
                }
            }
        }
    };
    FindModelBoundToEditorModel._getSearchRange = function (model, findScope) {
        // If we have set now or before a find scope, use it for computing the search range
        if (findScope) {
            return findScope;
        }
        return model.getFullModelRange();
    };
    FindModelBoundToEditorModel.prototype.research = function (moveCursor, newFindScope) {
        var findScope = null;
        if (typeof newFindScope !== 'undefined') {
            findScope = newFindScope;
        }
        else {
            findScope = this._decorations.getFindScope();
        }
        if (findScope !== null) {
            if (findScope.startLineNumber !== findScope.endLineNumber) {
                if (findScope.endColumn === 1) {
                    findScope = new Range(findScope.startLineNumber, 1, findScope.endLineNumber - 1, this._editor.getModel().getLineMaxColumn(findScope.endLineNumber - 1));
                }
                else {
                    // multiline find scope => expand to line starts / ends
                    findScope = new Range(findScope.startLineNumber, 1, findScope.endLineNumber, this._editor.getModel().getLineMaxColumn(findScope.endLineNumber));
                }
            }
        }
        var findMatches = this._findMatches(findScope, false, MATCHES_LIMIT);
        this._decorations.set(findMatches, findScope);
        this._state.changeMatchInfo(this._decorations.getCurrentMatchesPosition(this._editor.getSelection()), this._decorations.getCount(), undefined);
        if (moveCursor) {
            this._moveToNextMatch(this._decorations.getStartPosition());
        }
    };
    FindModelBoundToEditorModel.prototype._hasMatches = function () {
        return (this._state.matchesCount > 0);
    };
    FindModelBoundToEditorModel.prototype._cannotFind = function () {
        if (!this._hasMatches()) {
            var findScope = this._decorations.getFindScope();
            if (findScope) {
                // Reveal the selection so user is reminded that 'selection find' is on.
                this._editor.revealRangeInCenterIfOutsideViewport(findScope, 0 /* Smooth */);
            }
            return true;
        }
        return false;
    };
    FindModelBoundToEditorModel.prototype._setCurrentFindMatch = function (match) {
        var matchesPosition = this._decorations.setCurrentFindMatch(match);
        this._state.changeMatchInfo(matchesPosition, this._decorations.getCount(), match);
        this._editor.setSelection(match);
        this._editor.revealRangeInCenterIfOutsideViewport(match, 0 /* Smooth */);
    };
    FindModelBoundToEditorModel.prototype._prevSearchPosition = function (before) {
        var isUsingLineStops = this._state.isRegex && (this._state.searchString.indexOf('^') >= 0
            || this._state.searchString.indexOf('$') >= 0);
        var lineNumber = before.lineNumber, column = before.column;
        var model = this._editor.getModel();
        if (isUsingLineStops || column === 1) {
            if (lineNumber === 1) {
                lineNumber = model.getLineCount();
            }
            else {
                lineNumber--;
            }
            column = model.getLineMaxColumn(lineNumber);
        }
        else {
            column--;
        }
        return new Position(lineNumber, column);
    };
    FindModelBoundToEditorModel.prototype._moveToPrevMatch = function (before, isRecursed) {
        if (isRecursed === void 0) { isRecursed = false; }
        if (this._decorations.getCount() < MATCHES_LIMIT) {
            var prevMatchRange = this._decorations.matchBeforePosition(before);
            if (prevMatchRange && prevMatchRange.isEmpty() && prevMatchRange.getStartPosition().equals(before)) {
                before = this._prevSearchPosition(before);
                prevMatchRange = this._decorations.matchBeforePosition(before);
            }
            if (prevMatchRange) {
                this._setCurrentFindMatch(prevMatchRange);
            }
            return;
        }
        if (this._cannotFind()) {
            return;
        }
        var findScope = this._decorations.getFindScope();
        var searchRange = FindModelBoundToEditorModel._getSearchRange(this._editor.getModel(), findScope);
        // ...(----)...|...
        if (searchRange.getEndPosition().isBefore(before)) {
            before = searchRange.getEndPosition();
        }
        // ...|...(----)...
        if (before.isBefore(searchRange.getStartPosition())) {
            before = searchRange.getEndPosition();
        }
        var lineNumber = before.lineNumber, column = before.column;
        var model = this._editor.getModel();
        var position = new Position(lineNumber, column);
        var prevMatch = model.findPreviousMatch(this._state.searchString, position, this._state.isRegex, this._state.matchCase, this._state.wholeWord ? this._editor.getConfiguration().wordSeparators : null, false);
        if (prevMatch && prevMatch.range.isEmpty() && prevMatch.range.getStartPosition().equals(position)) {
            // Looks like we're stuck at this position, unacceptable!
            position = this._prevSearchPosition(position);
            prevMatch = model.findPreviousMatch(this._state.searchString, position, this._state.isRegex, this._state.matchCase, this._state.wholeWord ? this._editor.getConfiguration().wordSeparators : null, false);
        }
        if (!prevMatch) {
            // there is precisely one match and selection is on top of it
            return;
        }
        if (!isRecursed && !searchRange.containsRange(prevMatch.range)) {
            return this._moveToPrevMatch(prevMatch.range.getStartPosition(), true);
        }
        this._setCurrentFindMatch(prevMatch.range);
    };
    FindModelBoundToEditorModel.prototype.moveToPrevMatch = function () {
        this._moveToPrevMatch(this._editor.getSelection().getStartPosition());
    };
    FindModelBoundToEditorModel.prototype._nextSearchPosition = function (after) {
        var isUsingLineStops = this._state.isRegex && (this._state.searchString.indexOf('^') >= 0
            || this._state.searchString.indexOf('$') >= 0);
        var lineNumber = after.lineNumber, column = after.column;
        var model = this._editor.getModel();
        if (isUsingLineStops || column === model.getLineMaxColumn(lineNumber)) {
            if (lineNumber === model.getLineCount()) {
                lineNumber = 1;
            }
            else {
                lineNumber++;
            }
            column = 1;
        }
        else {
            column++;
        }
        return new Position(lineNumber, column);
    };
    FindModelBoundToEditorModel.prototype._moveToNextMatch = function (after) {
        if (this._decorations.getCount() < MATCHES_LIMIT) {
            var nextMatchRange = this._decorations.matchAfterPosition(after);
            if (nextMatchRange && nextMatchRange.isEmpty() && nextMatchRange.getStartPosition().equals(after)) {
                // Looks like we're stuck at this position, unacceptable!
                after = this._nextSearchPosition(after);
                nextMatchRange = this._decorations.matchAfterPosition(after);
            }
            if (nextMatchRange) {
                this._setCurrentFindMatch(nextMatchRange);
            }
            return;
        }
        var nextMatch = this._getNextMatch(after, false, true);
        if (nextMatch) {
            this._setCurrentFindMatch(nextMatch.range);
        }
    };
    FindModelBoundToEditorModel.prototype._getNextMatch = function (after, captureMatches, forceMove, isRecursed) {
        if (isRecursed === void 0) { isRecursed = false; }
        if (this._cannotFind()) {
            return null;
        }
        var findScope = this._decorations.getFindScope();
        var searchRange = FindModelBoundToEditorModel._getSearchRange(this._editor.getModel(), findScope);
        // ...(----)...|...
        if (searchRange.getEndPosition().isBefore(after)) {
            after = searchRange.getStartPosition();
        }
        // ...|...(----)...
        if (after.isBefore(searchRange.getStartPosition())) {
            after = searchRange.getStartPosition();
        }
        var lineNumber = after.lineNumber, column = after.column;
        var model = this._editor.getModel();
        var position = new Position(lineNumber, column);
        var nextMatch = model.findNextMatch(this._state.searchString, position, this._state.isRegex, this._state.matchCase, this._state.wholeWord ? this._editor.getConfiguration().wordSeparators : null, captureMatches);
        if (forceMove && nextMatch && nextMatch.range.isEmpty() && nextMatch.range.getStartPosition().equals(position)) {
            // Looks like we're stuck at this position, unacceptable!
            position = this._nextSearchPosition(position);
            nextMatch = model.findNextMatch(this._state.searchString, position, this._state.isRegex, this._state.matchCase, this._state.wholeWord ? this._editor.getConfiguration().wordSeparators : null, captureMatches);
        }
        if (!nextMatch) {
            // there is precisely one match and selection is on top of it
            return null;
        }
        if (!isRecursed && !searchRange.containsRange(nextMatch.range)) {
            return this._getNextMatch(nextMatch.range.getEndPosition(), captureMatches, forceMove, true);
        }
        return nextMatch;
    };
    FindModelBoundToEditorModel.prototype.moveToNextMatch = function () {
        this._moveToNextMatch(this._editor.getSelection().getEndPosition());
    };
    FindModelBoundToEditorModel.prototype._getReplacePattern = function () {
        if (this._state.isRegex) {
            return parseReplaceString(this._state.replaceString);
        }
        return ReplacePattern.fromStaticValue(this._state.replaceString);
    };
    FindModelBoundToEditorModel.prototype.replace = function () {
        if (!this._hasMatches()) {
            return;
        }
        var replacePattern = this._getReplacePattern();
        var selection = this._editor.getSelection();
        var nextMatch = this._getNextMatch(selection.getStartPosition(), replacePattern.hasReplacementPatterns, false);
        if (nextMatch) {
            if (selection.equalsRange(nextMatch.range)) {
                // selection sits on a find match => replace it!
                var replaceString = replacePattern.buildReplaceString(nextMatch.matches);
                var command = new ReplaceCommand(selection, replaceString);
                this._executeEditorCommand('replace', command);
                this._decorations.setStartPosition(new Position(selection.startLineNumber, selection.startColumn + replaceString.length));
                this.research(true);
            }
            else {
                this._decorations.setStartPosition(this._editor.getPosition());
                this._setCurrentFindMatch(nextMatch.range);
            }
        }
    };
    FindModelBoundToEditorModel.prototype._findMatches = function (findScope, captureMatches, limitResultCount) {
        var searchRange = FindModelBoundToEditorModel._getSearchRange(this._editor.getModel(), findScope);
        return this._editor.getModel().findMatches(this._state.searchString, searchRange, this._state.isRegex, this._state.matchCase, this._state.wholeWord ? this._editor.getConfiguration().wordSeparators : null, captureMatches, limitResultCount);
    };
    FindModelBoundToEditorModel.prototype.replaceAll = function () {
        if (!this._hasMatches()) {
            return;
        }
        var findScope = this._decorations.getFindScope();
        if (findScope === null && this._state.matchesCount >= MATCHES_LIMIT) {
            // Doing a replace on the entire file that is over ${MATCHES_LIMIT} matches
            this._largeReplaceAll();
        }
        else {
            this._regularReplaceAll(findScope);
        }
        this.research(false);
    };
    FindModelBoundToEditorModel.prototype._largeReplaceAll = function () {
        var searchParams = new SearchParams(this._state.searchString, this._state.isRegex, this._state.matchCase, this._state.wholeWord ? this._editor.getConfiguration().wordSeparators : null);
        var searchData = searchParams.parseSearchRequest();
        if (!searchData) {
            return;
        }
        var searchRegex = searchData.regex;
        if (!searchRegex.multiline) {
            var mod = 'm';
            if (searchRegex.ignoreCase) {
                mod += 'i';
            }
            if (searchRegex.global) {
                mod += 'g';
            }
            searchRegex = new RegExp(searchRegex.source, mod);
        }
        var model = this._editor.getModel();
        var modelText = model.getValue(1 /* LF */);
        var fullModelRange = model.getFullModelRange();
        var replacePattern = this._getReplacePattern();
        var resultText;
        if (replacePattern.hasReplacementPatterns) {
            resultText = modelText.replace(searchRegex, function () {
                return replacePattern.buildReplaceString(arguments);
            });
        }
        else {
            resultText = modelText.replace(searchRegex, replacePattern.buildReplaceString(null));
        }
        var command = new ReplaceCommandThatPreservesSelection(fullModelRange, resultText, this._editor.getSelection());
        this._executeEditorCommand('replaceAll', command);
    };
    FindModelBoundToEditorModel.prototype._regularReplaceAll = function (findScope) {
        var replacePattern = this._getReplacePattern();
        // Get all the ranges (even more than the highlighted ones)
        var matches = this._findMatches(findScope, replacePattern.hasReplacementPatterns, 1073741824 /* MAX_SAFE_SMALL_INTEGER */);
        var replaceStrings = [];
        for (var i = 0, len = matches.length; i < len; i++) {
            replaceStrings[i] = replacePattern.buildReplaceString(matches[i].matches);
        }
        var command = new ReplaceAllCommand(this._editor.getSelection(), matches.map(function (m) { return m.range; }), replaceStrings);
        this._executeEditorCommand('replaceAll', command);
    };
    FindModelBoundToEditorModel.prototype.selectAllMatches = function () {
        if (!this._hasMatches()) {
            return;
        }
        var findScope = this._decorations.getFindScope();
        // Get all the ranges (even more than the highlighted ones)
        var matches = this._findMatches(findScope, false, 1073741824 /* MAX_SAFE_SMALL_INTEGER */);
        var selections = matches.map(function (m) { return new Selection(m.range.startLineNumber, m.range.startColumn, m.range.endLineNumber, m.range.endColumn); });
        // If one of the ranges is the editor selection, then maintain it as primary
        var editorSelection = this._editor.getSelection();
        for (var i = 0, len = selections.length; i < len; i++) {
            var sel = selections[i];
            if (sel.equalsRange(editorSelection)) {
                selections = [editorSelection].concat(selections.slice(0, i)).concat(selections.slice(i + 1));
                break;
            }
        }
        this._editor.setSelections(selections);
    };
    FindModelBoundToEditorModel.prototype._executeEditorCommand = function (source, command) {
        try {
            this._ignoreModelContentChanged = true;
            this._editor.pushUndoStop();
            this._editor.executeCommand(source, command);
            this._editor.pushUndoStop();
        }
        finally {
            this._ignoreModelContentChanged = false;
        }
    };
    return FindModelBoundToEditorModel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1h = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var defaultOpts = {
    inputActiveOptionBorder: Color.fromHex('#007ACC')
};
var Checkbox = /** @class */ (function (_super) {
    __extends$1h(Checkbox, _super);
    function Checkbox(opts) {
        var _this = _super.call(this) || this;
        _this._onChange = _this._register(new Emitter());
        _this._onKeyDown = _this._register(new Emitter());
        _this._opts = deepClone(opts);
        mixin(_this._opts, defaultOpts, false);
        _this._checked = _this._opts.isChecked;
        _this.domNode = document.createElement('div');
        _this.domNode.title = _this._opts.title;
        _this.domNode.className = 'monaco-custom-checkbox ' + _this._opts.actionClassName + ' ' + (_this._checked ? 'checked' : 'unchecked');
        _this.domNode.tabIndex = 0;
        _this.domNode.setAttribute('role', 'checkbox');
        _this.domNode.setAttribute('aria-checked', String(_this._checked));
        _this.domNode.setAttribute('aria-label', _this._opts.title);
        _this.applyStyles();
        _this.onclick(_this.domNode, function (ev) {
            _this.checked = !_this._checked;
            _this._onChange.fire(false);
            ev.preventDefault();
        });
        _this.onkeydown(_this.domNode, function (keyboardEvent) {
            if (keyboardEvent.keyCode === 10 /* Space */ || keyboardEvent.keyCode === 3 /* Enter */) {
                _this.checked = !_this._checked;
                _this._onChange.fire(true);
                keyboardEvent.preventDefault();
                return;
            }
            _this._onKeyDown.fire(keyboardEvent);
        });
        return _this;
    }
    Object.defineProperty(Checkbox.prototype, "onChange", {
        get: function () { return this._onChange.event; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Checkbox.prototype, "onKeyDown", {
        get: function () { return this._onKeyDown.event; },
        enumerable: true,
        configurable: true
    });
    Checkbox.prototype.focus = function () {
        this.domNode.focus();
    };
    Object.defineProperty(Checkbox.prototype, "checked", {
        get: function () {
            return this._checked;
        },
        set: function (newIsChecked) {
            this._checked = newIsChecked;
            this.domNode.setAttribute('aria-checked', String(this._checked));
            if (this._checked) {
                this.domNode.classList.add('checked');
            }
            else {
                this.domNode.classList.remove('checked');
            }
            this.applyStyles();
        },
        enumerable: true,
        configurable: true
    });
    Checkbox.prototype.width = function () {
        return 2 /*marginleft*/ + 2 /*border*/ + 2 /*padding*/ + 16 /* icon width */;
    };
    Checkbox.prototype.style = function (styles) {
        if (styles.inputActiveOptionBorder) {
            this._opts.inputActiveOptionBorder = styles.inputActiveOptionBorder;
        }
        this.applyStyles();
    };
    Checkbox.prototype.applyStyles = function () {
        if (this.domNode) {
            this.domNode.style.borderColor = this._checked && this._opts.inputActiveOptionBorder ? this._opts.inputActiveOptionBorder.toString() : 'transparent';
        }
    };
    Checkbox.prototype.enable = function () {
        this.domNode.tabIndex = 0;
        this.domNode.setAttribute('aria-disabled', String(false));
    };
    Checkbox.prototype.disable = function () {
        removeTabIndexAndUpdateFocus(this.domNode);
        this.domNode.setAttribute('aria-disabled', String(true));
    };
    return Checkbox;
}(Widget$1));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1i = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var NLS_CASE_SENSITIVE_CHECKBOX_LABEL = localize('caseDescription', "Match Case");
var NLS_WHOLE_WORD_CHECKBOX_LABEL = localize('wordsDescription', "Match Whole Word");
var NLS_REGEX_CHECKBOX_LABEL = localize('regexDescription', "Use Regular Expression");
var CaseSensitiveCheckbox = /** @class */ (function (_super) {
    __extends$1i(CaseSensitiveCheckbox, _super);
    function CaseSensitiveCheckbox(opts) {
        return _super.call(this, {
            actionClassName: 'monaco-case-sensitive',
            title: NLS_CASE_SENSITIVE_CHECKBOX_LABEL + opts.appendTitle,
            isChecked: opts.isChecked,
            inputActiveOptionBorder: opts.inputActiveOptionBorder
        }) || this;
    }
    return CaseSensitiveCheckbox;
}(Checkbox));
var WholeWordsCheckbox = /** @class */ (function (_super) {
    __extends$1i(WholeWordsCheckbox, _super);
    function WholeWordsCheckbox(opts) {
        return _super.call(this, {
            actionClassName: 'monaco-whole-word',
            title: NLS_WHOLE_WORD_CHECKBOX_LABEL + opts.appendTitle,
            isChecked: opts.isChecked,
            inputActiveOptionBorder: opts.inputActiveOptionBorder
        }) || this;
    }
    return WholeWordsCheckbox;
}(Checkbox));
var RegexCheckbox = /** @class */ (function (_super) {
    __extends$1i(RegexCheckbox, _super);
    function RegexCheckbox(opts) {
        return _super.call(this, {
            actionClassName: 'monaco-regex',
            title: NLS_REGEX_CHECKBOX_LABEL + opts.appendTitle,
            isChecked: opts.isChecked,
            inputActiveOptionBorder: opts.inputActiveOptionBorder
        }) || this;
    }
    return RegexCheckbox;
}(Checkbox));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1j = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var FindOptionsWidget = /** @class */ (function (_super) {
    __extends$1j(FindOptionsWidget, _super);
    function FindOptionsWidget(editor, state, keybindingService, themeService) {
        var _this = _super.call(this) || this;
        _this._hideSoon = _this._register(new RunOnceScheduler(function () { return _this._hide(); }, 2000));
        _this._isVisible = false;
        _this._editor = editor;
        _this._state = state;
        _this._keybindingService = keybindingService;
        _this._domNode = document.createElement('div');
        _this._domNode.className = 'findOptionsWidget';
        _this._domNode.style.display = 'none';
        _this._domNode.style.top = '10px';
        _this._domNode.setAttribute('role', 'presentation');
        _this._domNode.setAttribute('aria-hidden', 'true');
        var inputActiveOptionBorderColor = themeService.getTheme().getColor(inputActiveOptionBorder);
        _this.caseSensitive = _this._register(new CaseSensitiveCheckbox({
            appendTitle: _this._keybindingLabelFor(FIND_IDS.ToggleCaseSensitiveCommand),
            isChecked: _this._state.matchCase,
            inputActiveOptionBorder: inputActiveOptionBorderColor
        }));
        _this._domNode.appendChild(_this.caseSensitive.domNode);
        _this._register(_this.caseSensitive.onChange(function () {
            _this._state.change({
                matchCase: _this.caseSensitive.checked
            }, false);
        }));
        _this.wholeWords = _this._register(new WholeWordsCheckbox({
            appendTitle: _this._keybindingLabelFor(FIND_IDS.ToggleWholeWordCommand),
            isChecked: _this._state.wholeWord,
            inputActiveOptionBorder: inputActiveOptionBorderColor
        }));
        _this._domNode.appendChild(_this.wholeWords.domNode);
        _this._register(_this.wholeWords.onChange(function () {
            _this._state.change({
                wholeWord: _this.wholeWords.checked
            }, false);
        }));
        _this.regex = _this._register(new RegexCheckbox({
            appendTitle: _this._keybindingLabelFor(FIND_IDS.ToggleRegexCommand),
            isChecked: _this._state.isRegex,
            inputActiveOptionBorder: inputActiveOptionBorderColor
        }));
        _this._domNode.appendChild(_this.regex.domNode);
        _this._register(_this.regex.onChange(function () {
            _this._state.change({
                isRegex: _this.regex.checked
            }, false);
        }));
        _this._editor.addOverlayWidget(_this);
        _this._register(_this._state.onFindReplaceStateChange(function (e) {
            var somethingChanged = false;
            if (e.isRegex) {
                _this.regex.checked = _this._state.isRegex;
                somethingChanged = true;
            }
            if (e.wholeWord) {
                _this.wholeWords.checked = _this._state.wholeWord;
                somethingChanged = true;
            }
            if (e.matchCase) {
                _this.caseSensitive.checked = _this._state.matchCase;
                somethingChanged = true;
            }
            if (!_this._state.isRevealed && somethingChanged) {
                _this._revealTemporarily();
            }
        }));
        _this._register(addDisposableNonBubblingMouseOutListener(_this._domNode, function (e) { return _this._onMouseOut(); }));
        _this._register(addDisposableListener(_this._domNode, 'mouseover', function (e) { return _this._onMouseOver(); }));
        _this._applyTheme(themeService.getTheme());
        _this._register(themeService.onThemeChange(_this._applyTheme.bind(_this)));
        return _this;
    }
    FindOptionsWidget.prototype._keybindingLabelFor = function (actionId) {
        var kb = this._keybindingService.lookupKeybinding(actionId);
        if (!kb) {
            return '';
        }
        return " (" + kb.getLabel() + ")";
    };
    FindOptionsWidget.prototype.dispose = function () {
        this._editor.removeOverlayWidget(this);
        _super.prototype.dispose.call(this);
    };
    // ----- IOverlayWidget API
    FindOptionsWidget.prototype.getId = function () {
        return FindOptionsWidget.ID;
    };
    FindOptionsWidget.prototype.getDomNode = function () {
        return this._domNode;
    };
    FindOptionsWidget.prototype.getPosition = function () {
        return {
            preference: 0 /* TOP_RIGHT_CORNER */
        };
    };
    FindOptionsWidget.prototype.highlightFindOptions = function () {
        this._revealTemporarily();
    };
    FindOptionsWidget.prototype._revealTemporarily = function () {
        this._show();
        this._hideSoon.schedule();
    };
    FindOptionsWidget.prototype._onMouseOut = function () {
        this._hideSoon.schedule();
    };
    FindOptionsWidget.prototype._onMouseOver = function () {
        this._hideSoon.cancel();
    };
    FindOptionsWidget.prototype._show = function () {
        if (this._isVisible) {
            return;
        }
        this._isVisible = true;
        this._domNode.style.display = 'block';
    };
    FindOptionsWidget.prototype._hide = function () {
        if (!this._isVisible) {
            return;
        }
        this._isVisible = false;
        this._domNode.style.display = 'none';
    };
    FindOptionsWidget.prototype._applyTheme = function (theme) {
        var inputStyles = { inputActiveOptionBorder: theme.getColor(inputActiveOptionBorder) };
        this.caseSensitive.style(inputStyles);
        this.wholeWords.style(inputStyles);
        this.regex.style(inputStyles);
    };
    FindOptionsWidget.ID = 'editor.contrib.findOptionsWidget';
    return FindOptionsWidget;
}(Widget$1));
registerThemingParticipant(function (theme, collector) {
    var widgetBackground = theme.getColor(editorWidgetBackground);
    if (widgetBackground) {
        collector.addRule(".monaco-editor .findOptionsWidget { background-color: " + widgetBackground + "; }");
    }
    var widgetShadowColor = theme.getColor(widgetShadow);
    if (widgetShadowColor) {
        collector.addRule(".monaco-editor .findOptionsWidget { box-shadow: 0 2px 8px " + widgetShadowColor + "; }");
    }
    var hcBorder = theme.getColor(contrastBorder);
    if (hcBorder) {
        collector.addRule(".monaco-editor .findOptionsWidget { border: 2px solid " + hcBorder + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function effectiveOptionValue(override, value) {
    if (override === 1 /* True */) {
        return true;
    }
    if (override === 2 /* False */) {
        return false;
    }
    return value;
}
var FindReplaceState = /** @class */ (function () {
    function FindReplaceState() {
        this._searchString = '';
        this._replaceString = '';
        this._isRevealed = false;
        this._isReplaceRevealed = false;
        this._isRegex = false;
        this._isRegexOverride = 0 /* NotSet */;
        this._wholeWord = false;
        this._wholeWordOverride = 0 /* NotSet */;
        this._matchCase = false;
        this._matchCaseOverride = 0 /* NotSet */;
        this._searchScope = null;
        this._matchesPosition = 0;
        this._matchesCount = 0;
        this._currentMatch = null;
        this._onFindReplaceStateChange = new Emitter();
    }
    Object.defineProperty(FindReplaceState.prototype, "searchString", {
        get: function () { return this._searchString; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "replaceString", {
        get: function () { return this._replaceString; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "isRevealed", {
        get: function () { return this._isRevealed; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "isReplaceRevealed", {
        get: function () { return this._isReplaceRevealed; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "isRegex", {
        get: function () { return effectiveOptionValue(this._isRegexOverride, this._isRegex); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "wholeWord", {
        get: function () { return effectiveOptionValue(this._wholeWordOverride, this._wholeWord); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "matchCase", {
        get: function () { return effectiveOptionValue(this._matchCaseOverride, this._matchCase); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "actualIsRegex", {
        get: function () { return this._isRegex; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "actualWholeWord", {
        get: function () { return this._wholeWord; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "actualMatchCase", {
        get: function () { return this._matchCase; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "searchScope", {
        get: function () { return this._searchScope; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "matchesPosition", {
        get: function () { return this._matchesPosition; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "matchesCount", {
        get: function () { return this._matchesCount; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "currentMatch", {
        get: function () { return this._currentMatch; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FindReplaceState.prototype, "onFindReplaceStateChange", {
        get: function () { return this._onFindReplaceStateChange.event; },
        enumerable: true,
        configurable: true
    });
    FindReplaceState.prototype.dispose = function () {
    };
    FindReplaceState.prototype.changeMatchInfo = function (matchesPosition, matchesCount, currentMatch) {
        var changeEvent = {
            moveCursor: false,
            updateHistory: false,
            searchString: false,
            replaceString: false,
            isRevealed: false,
            isReplaceRevealed: false,
            isRegex: false,
            wholeWord: false,
            matchCase: false,
            searchScope: false,
            matchesPosition: false,
            matchesCount: false,
            currentMatch: false
        };
        var somethingChanged = false;
        if (matchesCount === 0) {
            matchesPosition = 0;
        }
        if (matchesPosition > matchesCount) {
            matchesPosition = matchesCount;
        }
        if (this._matchesPosition !== matchesPosition) {
            this._matchesPosition = matchesPosition;
            changeEvent.matchesPosition = true;
            somethingChanged = true;
        }
        if (this._matchesCount !== matchesCount) {
            this._matchesCount = matchesCount;
            changeEvent.matchesCount = true;
            somethingChanged = true;
        }
        if (typeof currentMatch !== 'undefined') {
            if (!Range.equalsRange(this._currentMatch, currentMatch)) {
                this._currentMatch = currentMatch;
                changeEvent.currentMatch = true;
                somethingChanged = true;
            }
        }
        if (somethingChanged) {
            this._onFindReplaceStateChange.fire(changeEvent);
        }
    };
    FindReplaceState.prototype.change = function (newState, moveCursor, updateHistory) {
        if (updateHistory === void 0) { updateHistory = true; }
        var changeEvent = {
            moveCursor: moveCursor,
            updateHistory: updateHistory,
            searchString: false,
            replaceString: false,
            isRevealed: false,
            isReplaceRevealed: false,
            isRegex: false,
            wholeWord: false,
            matchCase: false,
            searchScope: false,
            matchesPosition: false,
            matchesCount: false,
            currentMatch: false
        };
        var somethingChanged = false;
        var oldEffectiveIsRegex = this.isRegex;
        var oldEffectiveWholeWords = this.wholeWord;
        var oldEffectiveMatchCase = this.matchCase;
        if (typeof newState.searchString !== 'undefined') {
            if (this._searchString !== newState.searchString) {
                this._searchString = newState.searchString;
                changeEvent.searchString = true;
                somethingChanged = true;
            }
        }
        if (typeof newState.replaceString !== 'undefined') {
            if (this._replaceString !== newState.replaceString) {
                this._replaceString = newState.replaceString;
                changeEvent.replaceString = true;
                somethingChanged = true;
            }
        }
        if (typeof newState.isRevealed !== 'undefined') {
            if (this._isRevealed !== newState.isRevealed) {
                this._isRevealed = newState.isRevealed;
                changeEvent.isRevealed = true;
                somethingChanged = true;
            }
        }
        if (typeof newState.isReplaceRevealed !== 'undefined') {
            if (this._isReplaceRevealed !== newState.isReplaceRevealed) {
                this._isReplaceRevealed = newState.isReplaceRevealed;
                changeEvent.isReplaceRevealed = true;
                somethingChanged = true;
            }
        }
        if (typeof newState.isRegex !== 'undefined') {
            this._isRegex = newState.isRegex;
        }
        if (typeof newState.wholeWord !== 'undefined') {
            this._wholeWord = newState.wholeWord;
        }
        if (typeof newState.matchCase !== 'undefined') {
            this._matchCase = newState.matchCase;
        }
        if (typeof newState.searchScope !== 'undefined') {
            if (!Range.equalsRange(this._searchScope, newState.searchScope)) {
                this._searchScope = newState.searchScope;
                changeEvent.searchScope = true;
                somethingChanged = true;
            }
        }
        // Overrides get set when they explicitly come in and get reset anytime something else changes
        this._isRegexOverride = (typeof newState.isRegexOverride !== 'undefined' ? newState.isRegexOverride : 0 /* NotSet */);
        this._wholeWordOverride = (typeof newState.wholeWordOverride !== 'undefined' ? newState.wholeWordOverride : 0 /* NotSet */);
        this._matchCaseOverride = (typeof newState.matchCaseOverride !== 'undefined' ? newState.matchCaseOverride : 0 /* NotSet */);
        if (oldEffectiveIsRegex !== this.isRegex) {
            somethingChanged = true;
            changeEvent.isRegex = true;
        }
        if (oldEffectiveWholeWords !== this.wholeWord) {
            somethingChanged = true;
            changeEvent.wholeWord = true;
        }
        if (oldEffectiveMatchCase !== this.matchCase) {
            somethingChanged = true;
            changeEvent.matchCase = true;
        }
        if (somethingChanged) {
            this._onFindReplaceStateChange.fire(changeEvent);
        }
    };
    return FindReplaceState;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IdGenerator = /** @class */ (function () {
    function IdGenerator(prefix) {
        this._prefix = prefix;
        this._lastId = 0;
    }
    IdGenerator.prototype.nextId = function () {
        return this._prefix + (++this._lastId);
    };
    return IdGenerator;
}());
var defaultGenerator = new IdGenerator('id#');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MarkdownString = /** @class */ (function () {
    function MarkdownString(value) {
        if (value === void 0) { value = ''; }
        this.value = value;
    }
    MarkdownString.prototype.appendText = function (value) {
        // escape markdown syntax tokens: http://daringfireball.net/projects/markdown/syntax#backslash
        this.value += value.replace(/[\\`*_{}[\]()#+\-.!]/g, '\\$&');
        return this;
    };
    MarkdownString.prototype.appendMarkdown = function (value) {
        this.value += value;
        return this;
    };
    MarkdownString.prototype.appendCodeblock = function (langId, code) {
        this.value += '\n```';
        this.value += langId;
        this.value += '\n';
        this.value += code;
        this.value += '\n```\n';
        return this;
    };
    return MarkdownString;
}());
function isEmptyMarkdownString(oneOrMany) {
    if (isMarkdownString(oneOrMany)) {
        return !oneOrMany.value;
    }
    else if (Array.isArray(oneOrMany)) {
        return oneOrMany.every(isEmptyMarkdownString);
    }
    else {
        return true;
    }
}
function isMarkdownString(thing) {
    if (thing instanceof MarkdownString) {
        return true;
    }
    else if (thing && typeof thing === 'object') {
        return typeof thing.value === 'string'
            && (typeof thing.isTrusted === 'boolean' || thing.isTrusted === void 0);
    }
    return false;
}
function markedStringsEquals(a, b) {
    if (!a && !b) {
        return true;
    }
    else if (!a || !b) {
        return false;
    }
    else if (Array.isArray(a) && Array.isArray(b)) {
        return equals(a, b, markdownStringEqual);
    }
    else if (isMarkdownString(a) && isMarkdownString(b)) {
        return markdownStringEqual(a, b);
    }
    else {
        return false;
    }
}
function markdownStringEqual(a, b) {
    if (a === b) {
        return true;
    }
    else if (!a || !b) {
        return false;
    }
    else {
        return a.value === b.value && a.isTrusted === b.isTrusted;
    }
}
function removeMarkdownEscapes(text) {
    if (!text) {
        return text;
    }
    return text.replace(/\\([\\`*_{}[\]()#+\-.!])/g, '$1');
}

/**
 * marked - a markdown parser
 * Copyright (c) 2011-2014, Christopher Jeffrey. (MIT Licensed)
 * https://github.com/markedjs/marked
 */

// BEGIN MONACOCHANGE
var __marked_exports;
(function(root) {

/**
 * Block-Level Grammar
 */

var block = {
  newline: /^\n+/,
  code: /^( {4}[^\n]+\n*)+/,
  fences: noop,
  hr: /^ {0,3}((?:- *){3,}|(?:_ *){3,}|(?:\* *){3,})(?:\n+|$)/,
  heading: /^ *(#{1,6}) *([^\n]+?) *(?:#+ *)?(?:\n+|$)/,
  nptable: noop,
  blockquote: /^( {0,3}> ?(paragraph|[^\n]*)(?:\n|$))+/,
  list: /^( *)(bull) [\s\S]+?(?:hr|def|\n{2,}(?! )(?!\1bull )\n*|\s*$)/,
  html: '^ {0,3}(?:' // optional indentation
    + '<(script|pre|style)[\\s>][\\s\\S]*?(?:</\\1>[^\\n]*\\n+|$)' // (1)
    + '|comment[^\\n]*(\\n+|$)' // (2)
    + '|<\\?[\\s\\S]*?\\?>\\n*' // (3)
    + '|<![A-Z][\\s\\S]*?>\\n*' // (4)
    + '|<!\\[CDATA\\[[\\s\\S]*?\\]\\]>\\n*' // (5)
    + '|</?(tag)(?: +|\\n|/?>)[\\s\\S]*?(?:\\n{2,}|$)' // (6)
    + '|<(?!script|pre|style)([a-z][\\w-]*)(?:attribute)*? */?>(?=\\h*\\n)[\\s\\S]*?(?:\\n{2,}|$)' // (7) open tag
    + '|</(?!script|pre|style)[a-z][\\w-]*\\s*>(?=\\h*\\n)[\\s\\S]*?(?:\\n{2,}|$)' // (7) closing tag
    + ')',
  def: /^ {0,3}\[(label)\]: *\n? *<?([^\s>]+)>?(?:(?: +\n? *| *\n *)(title))? *(?:\n+|$)/,
  table: noop,
  lheading: /^([^\n]+)\n *(=|-){2,} *(?:\n+|$)/,
  paragraph: /^([^\n]+(?:\n(?!hr|heading|lheading| {0,3}>|<\/?(?:tag)(?: +|\n|\/?>)|<(?:script|pre|style|!--))[^\n]+)*)/,
  text: /^[^\n]+/
};

block._label = /(?!\s*\])(?:\\[\[\]]|[^\[\]])+/;
block._title = /(?:"(?:\\"?|[^"\\])*"|'[^'\n]*(?:\n[^'\n]+)*\n?'|\([^()]*\))/;
block.def = edit(block.def)
  .replace('label', block._label)
  .replace('title', block._title)
  .getRegex();

block.bullet = /(?:[*+-]|\d+\.)/;
block.item = /^( *)(bull) [^\n]*(?:\n(?!\1bull )[^\n]*)*/;
block.item = edit(block.item, 'gm')
  .replace(/bull/g, block.bullet)
  .getRegex();

block.list = edit(block.list)
  .replace(/bull/g, block.bullet)
  .replace('hr', '\\n+(?=\\1?(?:(?:- *){3,}|(?:_ *){3,}|(?:\\* *){3,})(?:\\n+|$))')
  .replace('def', '\\n+(?=' + block.def.source + ')')
  .getRegex();

block._tag = 'address|article|aside|base|basefont|blockquote|body|caption'
  + '|center|col|colgroup|dd|details|dialog|dir|div|dl|dt|fieldset|figcaption'
  + '|figure|footer|form|frame|frameset|h[1-6]|head|header|hr|html|iframe'
  + '|legend|li|link|main|menu|menuitem|meta|nav|noframes|ol|optgroup|option'
  + '|p|param|section|source|summary|table|tbody|td|tfoot|th|thead|title|tr'
  + '|track|ul';
block._comment = /<!--(?!-?>)[\s\S]*?-->/;
block.html = edit(block.html, 'i')
  .replace('comment', block._comment)
  .replace('tag', block._tag)
  .replace('attribute', / +[a-zA-Z:_][\w.:-]*(?: *= *"[^"\n]*"| *= *'[^'\n]*'| *= *[^\s"'=<>`]+)?/)
  .getRegex();

block.paragraph = edit(block.paragraph)
  .replace('hr', block.hr)
  .replace('heading', block.heading)
  .replace('lheading', block.lheading)
  .replace('tag', block._tag) // pars can be interrupted by type (6) html blocks
  .getRegex();

block.blockquote = edit(block.blockquote)
  .replace('paragraph', block.paragraph)
  .getRegex();

/**
 * Normal Block Grammar
 */

block.normal = merge({}, block);

/**
 * GFM Block Grammar
 */

block.gfm = merge({}, block.normal, {
  fences: /^ *(`{3,}|~{3,})[ \.]*(\S+)? *\n([\s\S]*?)\n? *\1 *(?:\n+|$)/,
  paragraph: /^/,
  heading: /^ *(#{1,6}) +([^\n]+?) *#* *(?:\n+|$)/
});

block.gfm.paragraph = edit(block.paragraph)
  .replace('(?!', '(?!'
    + block.gfm.fences.source.replace('\\1', '\\2') + '|'
    + block.list.source.replace('\\1', '\\3') + '|')
  .getRegex();

/**
 * GFM + Tables Block Grammar
 */

block.tables = merge({}, block.gfm, {
  nptable: /^ *([^|\n ].*\|.*)\n *([-:]+ *\|[-| :]*)(?:\n((?:.*[^>\n ].*(?:\n|$))*)\n*|$)/,
  table: /^ *\|(.+)\n *\|?( *[-:]+[-| :]*)(?:\n((?: *[^>\n ].*(?:\n|$))*)\n*|$)/
});

/**
 * Pedantic grammar
 */

block.pedantic = merge({}, block.normal, {
  html: edit(
    '^ *(?:comment *(?:\\n|\\s*$)'
    + '|<(tag)[\\s\\S]+?</\\1> *(?:\\n{2,}|\\s*$)' // closed tag
    + '|<tag(?:"[^"]*"|\'[^\']*\'|\\s[^\'"/>\\s]*)*?/?> *(?:\\n{2,}|\\s*$))')
    .replace('comment', block._comment)
    .replace(/tag/g, '(?!(?:'
      + 'a|em|strong|small|s|cite|q|dfn|abbr|data|time|code|var|samp|kbd|sub'
      + '|sup|i|b|u|mark|ruby|rt|rp|bdi|bdo|span|br|wbr|ins|del|img)'
      + '\\b)\\w+(?!:|[^\\w\\s@]*@)\\b')
    .getRegex(),
  def: /^ *\[([^\]]+)\]: *<?([^\s>]+)>?(?: +(["(][^\n]+[")]))? *(?:\n+|$)/
});

/**
 * Block Lexer
 */

function Lexer(options) {
  this.tokens = [];
  this.tokens.links = Object.create(null);
  this.options = options || marked.defaults;
  this.rules = block.normal;

  if (this.options.pedantic) {
    this.rules = block.pedantic;
  } else if (this.options.gfm) {
    if (this.options.tables) {
      this.rules = block.tables;
    } else {
      this.rules = block.gfm;
    }
  }
}

/**
 * Expose Block Rules
 */

Lexer.rules = block;

/**
 * Static Lex Method
 */

Lexer.lex = function(src, options) {
  var lexer = new Lexer(options);
  return lexer.lex(src);
};

/**
 * Preprocessing
 */

Lexer.prototype.lex = function(src) {
  src = src
    .replace(/\r\n|\r/g, '\n')
    .replace(/\t/g, '    ')
    .replace(/\u00a0/g, ' ')
    .replace(/\u2424/g, '\n');

  return this.token(src, true);
};

/**
 * Lexing
 */

Lexer.prototype.token = function(src, top) {
  src = src.replace(/^ +$/gm, '');
  var next,
      loose,
      cap,
      bull,
      b,
      item,
      listStart,
      listItems,
      t,
      space,
      i,
      tag,
      l,
      isordered,
      istask,
      ischecked;

  while (src) {
    // newline
    if (cap = this.rules.newline.exec(src)) {
      src = src.substring(cap[0].length);
      if (cap[0].length > 1) {
        this.tokens.push({
          type: 'space'
        });
      }
    }

    // code
    if (cap = this.rules.code.exec(src)) {
      src = src.substring(cap[0].length);
      cap = cap[0].replace(/^ {4}/gm, '');
      this.tokens.push({
        type: 'code',
        text: !this.options.pedantic
          ? rtrim(cap, '\n')
          : cap
      });
      continue;
    }

    // fences (gfm)
    if (cap = this.rules.fences.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'code',
        lang: cap[2],
        text: cap[3] || ''
      });
      continue;
    }

    // heading
    if (cap = this.rules.heading.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'heading',
        depth: cap[1].length,
        text: cap[2]
      });
      continue;
    }

    // table no leading pipe (gfm)
    if (top && (cap = this.rules.nptable.exec(src))) {
      item = {
        type: 'table',
        header: splitCells(cap[1].replace(/^ *| *\| *$/g, '')),
        align: cap[2].replace(/^ *|\| *$/g, '').split(/ *\| */),
        cells: cap[3] ? cap[3].replace(/\n$/, '').split('\n') : []
      };

      if (item.header.length === item.align.length) {
        src = src.substring(cap[0].length);

        for (i = 0; i < item.align.length; i++) {
          if (/^ *-+: *$/.test(item.align[i])) {
            item.align[i] = 'right';
          } else if (/^ *:-+: *$/.test(item.align[i])) {
            item.align[i] = 'center';
          } else if (/^ *:-+ *$/.test(item.align[i])) {
            item.align[i] = 'left';
          } else {
            item.align[i] = null;
          }
        }

        for (i = 0; i < item.cells.length; i++) {
          item.cells[i] = splitCells(item.cells[i], item.header.length);
        }

        this.tokens.push(item);

        continue;
      }
    }

    // hr
    if (cap = this.rules.hr.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'hr'
      });
      continue;
    }

    // blockquote
    if (cap = this.rules.blockquote.exec(src)) {
      src = src.substring(cap[0].length);

      this.tokens.push({
        type: 'blockquote_start'
      });

      cap = cap[0].replace(/^ *> ?/gm, '');

      // Pass `top` to keep the current
      // "toplevel" state. This is exactly
      // how markdown.pl works.
      this.token(cap, top);

      this.tokens.push({
        type: 'blockquote_end'
      });

      continue;
    }

    // list
    if (cap = this.rules.list.exec(src)) {
      src = src.substring(cap[0].length);
      bull = cap[2];
      isordered = bull.length > 1;

      listStart = {
        type: 'list_start',
        ordered: isordered,
        start: isordered ? +bull : '',
        loose: false
      };

      this.tokens.push(listStart);

      // Get each top-level item.
      cap = cap[0].match(this.rules.item);

      listItems = [];
      next = false;
      l = cap.length;
      i = 0;

      for (; i < l; i++) {
        item = cap[i];

        // Remove the list item's bullet
        // so it is seen as the next token.
        space = item.length;
        item = item.replace(/^ *([*+-]|\d+\.) +/, '');

        // Outdent whatever the
        // list item contains. Hacky.
        if (~item.indexOf('\n ')) {
          space -= item.length;
          item = !this.options.pedantic
            ? item.replace(new RegExp('^ {1,' + space + '}', 'gm'), '')
            : item.replace(/^ {1,4}/gm, '');
        }

        // Determine whether the next list item belongs here.
        // Backpedal if it does not belong in this list.
        if (this.options.smartLists && i !== l - 1) {
          b = block.bullet.exec(cap[i + 1])[0];
          if (bull !== b && !(bull.length > 1 && b.length > 1)) {
            src = cap.slice(i + 1).join('\n') + src;
            i = l - 1;
          }
        }

        // Determine whether item is loose or not.
        // Use: /(^|\n)(?! )[^\n]+\n\n(?!\s*$)/
        // for discount behavior.
        loose = next || /\n\n(?!\s*$)/.test(item);
        if (i !== l - 1) {
          next = item.charAt(item.length - 1) === '\n';
          if (!loose) loose = next;
        }

        if (loose) {
          listStart.loose = true;
        }

        // Check for task list items
        istask = /^\[[ xX]\] /.test(item);
        ischecked = undefined;
        if (istask) {
          ischecked = item[1] !== ' ';
          item = item.replace(/^\[[ xX]\] +/, '');
        }

        t = {
          type: 'list_item_start',
          task: istask,
          checked: ischecked,
          loose: loose
        };

        listItems.push(t);
        this.tokens.push(t);

        // Recurse.
        this.token(item, false);

        this.tokens.push({
          type: 'list_item_end'
        });
      }

      if (listStart.loose) {
        l = listItems.length;
        i = 0;
        for (; i < l; i++) {
          listItems[i].loose = true;
        }
      }

      this.tokens.push({
        type: 'list_end'
      });

      continue;
    }

    // html
    if (cap = this.rules.html.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: this.options.sanitize
          ? 'paragraph'
          : 'html',
        pre: !this.options.sanitizer
          && (cap[1] === 'pre' || cap[1] === 'script' || cap[1] === 'style'),
        text: cap[0]
      });
      continue;
    }

    // def
    if (top && (cap = this.rules.def.exec(src))) {
      src = src.substring(cap[0].length);
      if (cap[3]) cap[3] = cap[3].substring(1, cap[3].length - 1);
      tag = cap[1].toLowerCase().replace(/\s+/g, ' ');
      if (!this.tokens.links[tag]) {
        this.tokens.links[tag] = {
          href: cap[2],
          title: cap[3]
        };
      }
      continue;
    }

    // table (gfm)
    if (top && (cap = this.rules.table.exec(src))) {
      item = {
        type: 'table',
        header: splitCells(cap[1].replace(/^ *| *\| *$/g, '')),
        align: cap[2].replace(/^ *|\| *$/g, '').split(/ *\| */),
        cells: cap[3] ? cap[3].replace(/(?: *\| *)?\n$/, '').split('\n') : []
      };

      if (item.header.length === item.align.length) {
        src = src.substring(cap[0].length);

        for (i = 0; i < item.align.length; i++) {
          if (/^ *-+: *$/.test(item.align[i])) {
            item.align[i] = 'right';
          } else if (/^ *:-+: *$/.test(item.align[i])) {
            item.align[i] = 'center';
          } else if (/^ *:-+ *$/.test(item.align[i])) {
            item.align[i] = 'left';
          } else {
            item.align[i] = null;
          }
        }

        for (i = 0; i < item.cells.length; i++) {
          item.cells[i] = splitCells(
            item.cells[i].replace(/^ *\| *| *\| *$/g, ''),
            item.header.length);
        }

        this.tokens.push(item);

        continue;
      }
    }

    // lheading
    if (cap = this.rules.lheading.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'heading',
        depth: cap[2] === '=' ? 1 : 2,
        text: cap[1]
      });
      continue;
    }

    // top-level paragraph
    if (top && (cap = this.rules.paragraph.exec(src))) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'paragraph',
        text: cap[1].charAt(cap[1].length - 1) === '\n'
          ? cap[1].slice(0, -1)
          : cap[1]
      });
      continue;
    }

    // text
    if (cap = this.rules.text.exec(src)) {
      // Top-level should never reach here.
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'text',
        text: cap[0]
      });
      continue;
    }

    if (src) {
      throw new Error('Infinite loop on byte: ' + src.charCodeAt(0));
    }
  }

  return this.tokens;
};

/**
 * Inline-Level Grammar
 */

var inline = {
  escape: /^\\([!"#$%&'()*+,\-./:;<=>?@\[\]\\^_`{|}~])/,
  autolink: /^<(scheme:[^\s\x00-\x1f<>]*|email)>/,
  url: noop,
  tag: '^comment'
    + '|^</[a-zA-Z][\\w:-]*\\s*>' // self-closing tag
    + '|^<[a-zA-Z][\\w-]*(?:attribute)*?\\s*/?>' // open tag
    + '|^<\\?[\\s\\S]*?\\?>' // processing instruction, e.g. <?php ?>
    + '|^<![a-zA-Z]+\\s[\\s\\S]*?>' // declaration, e.g. <!DOCTYPE html>
    + '|^<!\\[CDATA\\[[\\s\\S]*?\\]\\]>', // CDATA section
  link: /^!?\[(label)\]\(href(?:\s+(title))?\s*\)/,
  reflink: /^!?\[(label)\]\[(?!\s*\])((?:\\[\[\]]?|[^\[\]\\])+)\]/,
  nolink: /^!?\[(?!\s*\])((?:\[[^\[\]]*\]|\\[\[\]]|[^\[\]])*)\](?:\[\])?/,
  strong: /^__([^\s])__(?!_)|^\*\*([^\s])\*\*(?!\*)|^__([^\s][\s\S]*?[^\s])__(?!_)|^\*\*([^\s][\s\S]*?[^\s])\*\*(?!\*)/,
  em: /^_([^\s_])_(?!_)|^\*([^\s*"<\[])\*(?!\*)|^_([^\s][\s\S]*?[^\s_])_(?!_)|^_([^\s_][\s\S]*?[^\s])_(?!_)|^\*([^\s"<\[][\s\S]*?[^\s*])\*(?!\*)|^\*([^\s*"<\[][\s\S]*?[^\s])\*(?!\*)/,
  code: /^(`+)\s*([\s\S]*?[^`]?)\s*\1(?!`)/,
  br: /^( {2,}|\\)\n(?!\s*$)/,
  del: noop,
  text: /^[\s\S]+?(?=[\\<!\[`*]|\b_| {2,}\n|$)/
};

inline._escapes = /\\([!"#$%&'()*+,\-./:;<=>?@\[\]\\^_`{|}~])/g;

inline._scheme = /[a-zA-Z][a-zA-Z0-9+.-]{1,31}/;
inline._email = /[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+(@)[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+(?![-_])/;
inline.autolink = edit(inline.autolink)
  .replace('scheme', inline._scheme)
  .replace('email', inline._email)
  .getRegex();

inline._attribute = /\s+[a-zA-Z:_][\w.:-]*(?:\s*=\s*"[^"]*"|\s*=\s*'[^']*'|\s*=\s*[^\s"'=<>`]+)?/;

inline.tag = edit(inline.tag)
  .replace('comment', block._comment)
  .replace('attribute', inline._attribute)
  .getRegex();

inline._label = /(?:\[[^\[\]]*\]|\\[\[\]]?|`[^`]*`|[^\[\]\\])*?/;
inline._href = /\s*(<(?:\\[<>]?|[^\s<>\\])*>|(?:\\[()]?|\([^\s\x00-\x1f\\]*\)|[^\s\x00-\x1f()\\])*?)/;
inline._title = /"(?:\\"?|[^"\\])*"|'(?:\\'?|[^'\\])*'|\((?:\\\)?|[^)\\])*\)/;

inline.link = edit(inline.link)
  .replace('label', inline._label)
  .replace('href', inline._href)
  .replace('title', inline._title)
  .getRegex();

inline.reflink = edit(inline.reflink)
  .replace('label', inline._label)
  .getRegex();

/**
 * Normal Inline Grammar
 */

inline.normal = merge({}, inline);

/**
 * Pedantic Inline Grammar
 */

inline.pedantic = merge({}, inline.normal, {
  strong: /^__(?=\S)([\s\S]*?\S)__(?!_)|^\*\*(?=\S)([\s\S]*?\S)\*\*(?!\*)/,
  em: /^_(?=\S)([\s\S]*?\S)_(?!_)|^\*(?=\S)([\s\S]*?\S)\*(?!\*)/,
  link: edit(/^!?\[(label)\]\((.*?)\)/)
    .replace('label', inline._label)
    .getRegex(),
  reflink: edit(/^!?\[(label)\]\s*\[([^\]]*)\]/)
    .replace('label', inline._label)
    .getRegex()
});

/**
 * GFM Inline Grammar
 */

inline.gfm = merge({}, inline.normal, {
  escape: edit(inline.escape).replace('])', '~|])').getRegex(),
  url: edit(/^((?:ftp|https?):\/\/|www\.)(?:[a-zA-Z0-9\-]+\.?)+[^\s<]*|^email/)
    .replace('email', inline._email)
    .getRegex(),
  _backpedal: /(?:[^?!.,:;*_~()&]+|\([^)]*\)|&(?![a-zA-Z0-9]+;$)|[?!.,:;*_~)]+(?!$))+/,
  del: /^~+(?=\S)([\s\S]*?\S)~+/,
  text: edit(inline.text)
    .replace(']|', '~]|')
    .replace('|', '|https?://|ftp://|www\\.|[a-zA-Z0-9.!#$%&\'*+/=?^_`{\\|}~-]+@|')
    .getRegex()
});

/**
 * GFM + Line Breaks Inline Grammar
 */

inline.breaks = merge({}, inline.gfm, {
  br: edit(inline.br).replace('{2,}', '*').getRegex(),
  text: edit(inline.gfm.text).replace('{2,}', '*').getRegex()
});

/**
 * Inline Lexer & Compiler
 */

function InlineLexer(links, options) {
  this.options = options || marked.defaults;
  this.links = links;
  this.rules = inline.normal;
  this.renderer = this.options.renderer || new Renderer();
  this.renderer.options = this.options;

  if (!this.links) {
    throw new Error('Tokens array requires a `links` property.');
  }

  if (this.options.pedantic) {
    this.rules = inline.pedantic;
  } else if (this.options.gfm) {
    if (this.options.breaks) {
      this.rules = inline.breaks;
    } else {
      this.rules = inline.gfm;
    }
  }
}

/**
 * Expose Inline Rules
 */

InlineLexer.rules = inline;

/**
 * Static Lexing/Compiling Method
 */

InlineLexer.output = function(src, links, options) {
  var inline = new InlineLexer(links, options);
  return inline.output(src);
};

/**
 * Lexing/Compiling
 */

InlineLexer.prototype.output = function(src) {
  var out = '',
      link,
      text,
      href,
      title,
      cap,
      prevCapZero;

  while (src) {
    // escape
    if (cap = this.rules.escape.exec(src)) {
      src = src.substring(cap[0].length);
      out += cap[1];
      continue;
    }

    // autolink
    if (cap = this.rules.autolink.exec(src)) {
      src = src.substring(cap[0].length);
      if (cap[2] === '@') {
        text = escape(this.mangle(cap[1]));
        href = 'mailto:' + text;
      } else {
        text = escape(cap[1]);
        href = text;
      }
      out += this.renderer.link(href, null, text);
      continue;
    }

    // url (gfm)
    if (!this.inLink && (cap = this.rules.url.exec(src))) {
      do {
        prevCapZero = cap[0];
        cap[0] = this.rules._backpedal.exec(cap[0])[0];
      } while (prevCapZero !== cap[0]);
      src = src.substring(cap[0].length);
      if (cap[2] === '@') {
        text = escape(cap[0]);
        href = 'mailto:' + text;
      } else {
        text = escape(cap[0]);
        if (cap[1] === 'www.') {
          href = 'http://' + text;
        } else {
          href = text;
        }
      }
      out += this.renderer.link(href, null, text);
      continue;
    }

    // tag
    if (cap = this.rules.tag.exec(src)) {
      if (!this.inLink && /^<a /i.test(cap[0])) {
        this.inLink = true;
      } else if (this.inLink && /^<\/a>/i.test(cap[0])) {
        this.inLink = false;
      }
      src = src.substring(cap[0].length);
      out += this.options.sanitize
        ? this.options.sanitizer
          ? this.options.sanitizer(cap[0])
          : escape(cap[0])
        : cap[0];
      continue;
    }

    // link
    if (cap = this.rules.link.exec(src)) {
      src = src.substring(cap[0].length);
      this.inLink = true;
      href = cap[2];
      if (this.options.pedantic) {
        link = /^([^'"]*[^\s])\s+(['"])(.*)\2/.exec(href);

        if (link) {
          href = link[1];
          title = link[3];
        } else {
          title = '';
        }
      } else {
        title = cap[3] ? cap[3].slice(1, -1) : '';
      }
      href = href.trim().replace(/^<([\s\S]*)>$/, '$1');
      out += this.outputLink(cap, {
        href: InlineLexer.escapes(href),
        title: InlineLexer.escapes(title)
      });
      this.inLink = false;
      continue;
    }

    // reflink, nolink
    if ((cap = this.rules.reflink.exec(src))
        || (cap = this.rules.nolink.exec(src))) {
      src = src.substring(cap[0].length);
      link = (cap[2] || cap[1]).replace(/\s+/g, ' ');
      link = this.links[link.toLowerCase()];
      if (!link || !link.href) {
        out += cap[0].charAt(0);
        src = cap[0].substring(1) + src;
        continue;
      }
      this.inLink = true;
      out += this.outputLink(cap, link);
      this.inLink = false;
      continue;
    }

    // strong
    if (cap = this.rules.strong.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.strong(this.output(cap[4] || cap[3] || cap[2] || cap[1]));
      continue;
    }

    // em
    if (cap = this.rules.em.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.em(this.output(cap[6] || cap[5] || cap[4] || cap[3] || cap[2] || cap[1]));
      continue;
    }

    // code
    if (cap = this.rules.code.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.codespan(escape(cap[2].trim(), true));
      continue;
    }

    // br
    if (cap = this.rules.br.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.br();
      continue;
    }

    // del (gfm)
    if (cap = this.rules.del.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.del(this.output(cap[1]));
      continue;
    }

    // text
    if (cap = this.rules.text.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.text(escape(this.smartypants(cap[0])));
      continue;
    }

    if (src) {
      throw new Error('Infinite loop on byte: ' + src.charCodeAt(0));
    }
  }

  return out;
};

InlineLexer.escapes = function(text) {
  return text ? text.replace(InlineLexer.rules._escapes, '$1') : text;
};

/**
 * Compile Link
 */

InlineLexer.prototype.outputLink = function(cap, link) {
  var href = link.href,
      title = link.title ? escape(link.title) : null;

  return cap[0].charAt(0) !== '!'
    ? this.renderer.link(href, title, this.output(cap[1]))
    : this.renderer.image(href, title, escape(cap[1]));
};

/**
 * Smartypants Transformations
 */

InlineLexer.prototype.smartypants = function(text) {
  if (!this.options.smartypants) return text;
  return text
    // em-dashes
    .replace(/---/g, '\u2014')
    // en-dashes
    .replace(/--/g, '\u2013')
    // opening singles
    .replace(/(^|[-\u2014/(\[{"\s])'/g, '$1\u2018')
    // closing singles & apostrophes
    .replace(/'/g, '\u2019')
    // opening doubles
    .replace(/(^|[-\u2014/(\[{\u2018\s])"/g, '$1\u201c')
    // closing doubles
    .replace(/"/g, '\u201d')
    // ellipses
    .replace(/\.{3}/g, '\u2026');
};

/**
 * Mangle Links
 */

InlineLexer.prototype.mangle = function(text) {
  if (!this.options.mangle) return text;
  var out = '',
      l = text.length,
      i = 0,
      ch;

  for (; i < l; i++) {
    ch = text.charCodeAt(i);
    if (Math.random() > 0.5) {
      ch = 'x' + ch.toString(16);
    }
    out += '&#' + ch + ';';
  }

  return out;
};

/**
 * Renderer
 */

function Renderer(options) {
  this.options = options || marked.defaults;
}

Renderer.prototype.code = function(code, lang, escaped) {
  if (this.options.highlight) {
    var out = this.options.highlight(code, lang);
    if (out != null && out !== code) {
      escaped = true;
      code = out;
    }
  }

  if (!lang) {
    return '<pre><code>'
      + (escaped ? code : escape(code, true))
      + '</code></pre>';
  }

  return '<pre><code class="'
    + this.options.langPrefix
    + escape(lang, true)
    + '">'
    + (escaped ? code : escape(code, true))
    + '</code></pre>\n';
};

Renderer.prototype.blockquote = function(quote) {
  return '<blockquote>\n' + quote + '</blockquote>\n';
};

Renderer.prototype.html = function(html) {
  return html;
};

Renderer.prototype.heading = function(text, level, raw) {
  if (this.options.headerIds) {
    return '<h'
      + level
      + ' id="'
      + this.options.headerPrefix
      + raw.toLowerCase().replace(/[^\w]+/g, '-')
      + '">'
      + text
      + '</h'
      + level
      + '>\n';
  }
  // ignore IDs
  return '<h' + level + '>' + text + '</h' + level + '>\n';
};

Renderer.prototype.hr = function() {
  return this.options.xhtml ? '<hr/>\n' : '<hr>\n';
};

Renderer.prototype.list = function(body, ordered, start) {
  var type = ordered ? 'ol' : 'ul',
      startatt = (ordered && start !== 1) ? (' start="' + start + '"') : '';
  return '<' + type + startatt + '>\n' + body + '</' + type + '>\n';
};

Renderer.prototype.listitem = function(text) {
  return '<li>' + text + '</li>\n';
};

Renderer.prototype.checkbox = function(checked) {
  return '<input '
    + (checked ? 'checked="" ' : '')
    + 'disabled="" type="checkbox"'
    + (this.options.xhtml ? ' /' : '')
    + '> ';
};

Renderer.prototype.paragraph = function(text) {
  return '<p>' + text + '</p>\n';
};

Renderer.prototype.table = function(header, body) {
  if (body) body = '<tbody>' + body + '</tbody>';

  return '<table>\n'
    + '<thead>\n'
    + header
    + '</thead>\n'
    + body
    + '</table>\n';
};

Renderer.prototype.tablerow = function(content) {
  return '<tr>\n' + content + '</tr>\n';
};

Renderer.prototype.tablecell = function(content, flags) {
  var type = flags.header ? 'th' : 'td';
  var tag = flags.align
    ? '<' + type + ' align="' + flags.align + '">'
    : '<' + type + '>';
  return tag + content + '</' + type + '>\n';
};

// span level renderer
Renderer.prototype.strong = function(text) {
  return '<strong>' + text + '</strong>';
};

Renderer.prototype.em = function(text) {
  return '<em>' + text + '</em>';
};

Renderer.prototype.codespan = function(text) {
  return '<code>' + text + '</code>';
};

Renderer.prototype.br = function() {
  return this.options.xhtml ? '<br/>' : '<br>';
};

Renderer.prototype.del = function(text) {
  return '<del>' + text + '</del>';
};

Renderer.prototype.link = function(href, title, text) {
  if (this.options.sanitize) {
    try {
      var prot = decodeURIComponent(unescape(href))
        .replace(/[^\w:]/g, '')
        .toLowerCase();
    } catch (e) {
      return text;
    }
    if (prot.indexOf('javascript:') === 0 || prot.indexOf('vbscript:') === 0 || prot.indexOf('data:') === 0) {
      return text;
    }
  }
  if (this.options.baseUrl && !originIndependentUrl.test(href)) {
    href = resolveUrl(this.options.baseUrl, href);
  }
  try {
    href = encodeURI(href).replace(/%25/g, '%');
  } catch (e) {
    return text;
  }
  var out = '<a href="' + escape(href) + '"';
  if (title) {
    out += ' title="' + title + '"';
  }
  out += '>' + text + '</a>';
  return out;
};

Renderer.prototype.image = function(href, title, text) {
  if (this.options.baseUrl && !originIndependentUrl.test(href)) {
    href = resolveUrl(this.options.baseUrl, href);
  }
  var out = '<img src="' + href + '" alt="' + text + '"';
  if (title) {
    out += ' title="' + title + '"';
  }
  out += this.options.xhtml ? '/>' : '>';
  return out;
};

Renderer.prototype.text = function(text) {
  return text;
};

/**
 * TextRenderer
 * returns only the textual part of the token
 */

function TextRenderer() {}

// no need for block level renderers

TextRenderer.prototype.strong =
TextRenderer.prototype.em =
TextRenderer.prototype.codespan =
TextRenderer.prototype.del =
TextRenderer.prototype.text = function (text) {
  return text;
};

TextRenderer.prototype.link =
TextRenderer.prototype.image = function(href, title, text) {
  return '' + text;
};

TextRenderer.prototype.br = function() {
  return '';
};

/**
 * Parsing & Compiling
 */

function Parser(options) {
  this.tokens = [];
  this.token = null;
  this.options = options || marked.defaults;
  this.options.renderer = this.options.renderer || new Renderer();
  this.renderer = this.options.renderer;
  this.renderer.options = this.options;
}

/**
 * Static Parse Method
 */

Parser.parse = function(src, options) {
  var parser = new Parser(options);
  return parser.parse(src);
};

/**
 * Parse Loop
 */

Parser.prototype.parse = function(src) {
  this.inline = new InlineLexer(src.links, this.options);
  // use an InlineLexer with a TextRenderer to extract pure text
  this.inlineText = new InlineLexer(
    src.links,
    merge({}, this.options, {renderer: new TextRenderer()})
  );
  this.tokens = src.reverse();

  var out = '';
  while (this.next()) {
    out += this.tok();
  }

  return out;
};

/**
 * Next Token
 */

Parser.prototype.next = function() {
  return this.token = this.tokens.pop();
};

/**
 * Preview Next Token
 */

Parser.prototype.peek = function() {
  return this.tokens[this.tokens.length - 1] || 0;
};

/**
 * Parse Text Tokens
 */

Parser.prototype.parseText = function() {
  var body = this.token.text;

  while (this.peek().type === 'text') {
    body += '\n' + this.next().text;
  }

  return this.inline.output(body);
};

/**
 * Parse Current Token
 */

Parser.prototype.tok = function() {
  switch (this.token.type) {
    case 'space': {
      return '';
    }
    case 'hr': {
      return this.renderer.hr();
    }
    case 'heading': {
      return this.renderer.heading(
        this.inline.output(this.token.text),
        this.token.depth,
        unescape(this.inlineText.output(this.token.text)));
    }
    case 'code': {
      return this.renderer.code(this.token.text,
        this.token.lang,
        this.token.escaped);
    }
    case 'table': {
      var header = '',
          body = '',
          i,
          row,
          cell,
          j;

      // header
      cell = '';
      for (i = 0; i < this.token.header.length; i++) {
        cell += this.renderer.tablecell(
          this.inline.output(this.token.header[i]),
          { header: true, align: this.token.align[i] }
        );
      }
      header += this.renderer.tablerow(cell);

      for (i = 0; i < this.token.cells.length; i++) {
        row = this.token.cells[i];

        cell = '';
        for (j = 0; j < row.length; j++) {
          cell += this.renderer.tablecell(
            this.inline.output(row[j]),
            { header: false, align: this.token.align[j] }
          );
        }

        body += this.renderer.tablerow(cell);
      }
      return this.renderer.table(header, body);
    }
    case 'blockquote_start': {
      body = '';

      while (this.next().type !== 'blockquote_end') {
        body += this.tok();
      }

      return this.renderer.blockquote(body);
    }
    case 'list_start': {
      body = '';
      var ordered = this.token.ordered,
          start = this.token.start;

      while (this.next().type !== 'list_end') {
        body += this.tok();
      }

      return this.renderer.list(body, ordered, start);
    }
    case 'list_item_start': {
      body = '';
      var loose = this.token.loose;

      if (this.token.task) {
        body += this.renderer.checkbox(this.token.checked);
      }

      while (this.next().type !== 'list_item_end') {
        body += !loose && this.token.type === 'text'
          ? this.parseText()
          : this.tok();
      }

      return this.renderer.listitem(body);
    }
    case 'html': {
      // TODO parse inline content if parameter markdown=1
      return this.renderer.html(this.token.text);
    }
    case 'paragraph': {
      return this.renderer.paragraph(this.inline.output(this.token.text));
    }
    case 'text': {
      return this.renderer.paragraph(this.parseText());
    }
  }
};

/**
 * Helpers
 */

function escape(html, encode) {
  return html
    .replace(!encode ? /&(?!#?\w+;)/g : /&/g, '&amp;')
    .replace(/</g, '&lt;')
    .replace(/>/g, '&gt;')
    .replace(/"/g, '&quot;')
    .replace(/'/g, '&#39;');
}

function unescape(html) {
  // explicitly match decimal, hex, and named HTML entities
  return html.replace(/&(#(?:\d+)|(?:#x[0-9A-Fa-f]+)|(?:\w+));?/ig, function(_, n) {
    n = n.toLowerCase();
    if (n === 'colon') return ':';
    if (n.charAt(0) === '#') {
      return n.charAt(1) === 'x'
        ? String.fromCharCode(parseInt(n.substring(2), 16))
        : String.fromCharCode(+n.substring(1));
    }
    return '';
  });
}

function edit(regex, opt) {
  regex = regex.source || regex;
  opt = opt || '';
  return {
    replace: function(name, val) {
      val = val.source || val;
      val = val.replace(/(^|[^\[])\^/g, '$1');
      regex = regex.replace(name, val);
      return this;
    },
    getRegex: function() {
      return new RegExp(regex, opt);
    }
  };
}

function resolveUrl(base, href) {
  if (!baseUrls[' ' + base]) {
    // we can ignore everything in base after the last slash of its path component,
    // but we might need to add _that_
    // https://tools.ietf.org/html/rfc3986#section-3
    if (/^[^:]+:\/*[^/]*$/.test(base)) {
      baseUrls[' ' + base] = base + '/';
    } else {
      baseUrls[' ' + base] = rtrim(base, '/', true);
    }
  }
  base = baseUrls[' ' + base];

  if (href.slice(0, 2) === '//') {
    return base.replace(/:[\s\S]*/, ':') + href;
  } else if (href.charAt(0) === '/') {
    return base.replace(/(:\/*[^/]*)[\s\S]*/, '$1') + href;
  } else {
    return base + href;
  }
}
var baseUrls = {};
var originIndependentUrl = /^$|^[a-z][a-z0-9+.-]*:|^[?#]/i;

function noop() {}
noop.exec = noop;

function merge(obj) {
  var i = 1,
      target,
      key;

  for (; i < arguments.length; i++) {
    target = arguments[i];
    for (key in target) {
      if (Object.prototype.hasOwnProperty.call(target, key)) {
        obj[key] = target[key];
      }
    }
  }

  return obj;
}

function splitCells(tableRow, count) {
  // ensure that every cell-delimiting pipe has a space
  // before it to distinguish it from an escaped pipe
  var row = tableRow.replace(/\|/g, function (match, offset, str) {
        var escaped = false,
            curr = offset;
        while (--curr >= 0 && str[curr] === '\\') escaped = !escaped;
        if (escaped) {
          // odd number of slashes means | is escaped
          // so we leave it alone
          return '|';
        } else {
          // add space before unescaped |
          return ' |';
        }
      }),
      cells = row.split(/ \|/),
      i = 0;

  if (cells.length > count) {
    cells.splice(count);
  } else {
    while (cells.length < count) cells.push('');
  }

  for (; i < cells.length; i++) {
    // leading or trailing whitespace is ignored per the gfm spec
    cells[i] = cells[i].trim().replace(/\\\|/g, '|');
  }
  return cells;
}

// Remove trailing 'c's. Equivalent to str.replace(/c*$/, '').
// /c*$/ is vulnerable to REDOS.
// invert: Remove suffix of non-c chars instead. Default falsey.
function rtrim(str, c, invert) {
  if (str.length === 0) {
    return '';
  }

  // Length of suffix matching the invert condition.
  var suffLen = 0;

  // Step left until we fail to match the invert condition.
  while (suffLen < str.length) {
    var currChar = str.charAt(str.length - suffLen - 1);
    if (currChar === c && !invert) {
      suffLen++;
    } else if (currChar !== c && invert) {
      suffLen++;
    } else {
      break;
    }
  }

  return str.substr(0, str.length - suffLen);
}

/**
 * Marked
 */

function marked(src, opt, callback) {
  // throw error in case of non string input
  if (typeof src === 'undefined' || src === null) {
    throw new Error('marked(): input parameter is undefined or null');
  }
  if (typeof src !== 'string') {
    throw new Error('marked(): input parameter is of type '
      + Object.prototype.toString.call(src) + ', string expected');
  }

  if (callback || typeof opt === 'function') {
    if (!callback) {
      callback = opt;
      opt = null;
    }

    opt = merge({}, marked.defaults, opt || {});

    var highlight = opt.highlight,
        tokens,
        pending,
        i = 0;

    try {
      tokens = Lexer.lex(src, opt);
    } catch (e) {
      return callback(e);
    }

    pending = tokens.length;

    var done = function(err) {
      if (err) {
        opt.highlight = highlight;
        return callback(err);
      }

      var out;

      try {
        out = Parser.parse(tokens, opt);
      } catch (e) {
        err = e;
      }

      opt.highlight = highlight;

      return err
        ? callback(err)
        : callback(null, out);
    };

    if (!highlight || highlight.length < 3) {
      return done();
    }

    delete opt.highlight;

    if (!pending) return done();

    for (; i < tokens.length; i++) {
      (function(token) {
        if (token.type !== 'code') {
          return --pending || done();
        }
        return highlight(token.text, token.lang, function(err, code) {
          if (err) return done(err);
          if (code == null || code === token.text) {
            return --pending || done();
          }
          token.text = code;
          token.escaped = true;
          --pending || done();
        });
      })(tokens[i]);
    }

    return;
  }
  try {
    if (opt) opt = merge({}, marked.defaults, opt);
    return Parser.parse(Lexer.lex(src, opt), opt);
  } catch (e) {
    e.message += '\nPlease report this to https://github.com/markedjs/marked.';
    if ((opt || marked.defaults).silent) {
      return '<p>An error occurred:</p><pre>'
        + escape(e.message + '', true)
        + '</pre>';
    }
    throw e;
  }
}

/**
 * Options
 */

marked.options =
marked.setOptions = function(opt) {
  merge(marked.defaults, opt);
  return marked;
};

marked.getDefaults = function () {
  return {
    baseUrl: null,
    breaks: false,
    gfm: true,
    headerIds: true,
    headerPrefix: '',
    highlight: null,
    langPrefix: 'language-',
    mangle: true,
    pedantic: false,
    renderer: new Renderer(),
    sanitize: false,
    sanitizer: null,
    silent: false,
    smartLists: false,
    smartypants: false,
    tables: true,
    xhtml: false
  };
};

marked.defaults = marked.getDefaults();

/**
 * Expose
 */

marked.Parser = Parser;
marked.parser = Parser.parse;

marked.Renderer = Renderer;
marked.TextRenderer = TextRenderer;

marked.Lexer = Lexer;
marked.lexer = Lexer.lex;

marked.InlineLexer = InlineLexer;
marked.inlineLexer = InlineLexer.output;

marked.parse = marked;

// BEGIN MONACOCHANGE
// if (typeof module !== 'undefined' && typeof exports === 'object') {
//   module.exports = marked;
// } else if (typeof define === 'function' && define.amd) {
//   define(function() { return marked; });
// } else {
//   root.marked = marked;
// }
// })(this || (typeof window !== 'undefined' ? window : global));
__marked_exports = marked;
}).call(undefined);
var Parser = __marked_exports.Parser;
var parser = __marked_exports.parser;
var Renderer = __marked_exports.Renderer;
var TextRenderer = __marked_exports.TextRenderer;
var Lexer = __marked_exports.Lexer;
var lexer = __marked_exports.lexer;
var InlineLexer = __marked_exports.InlineLexer;
var inlineLexer = __marked_exports.inlineLexer;
var parse$1 = __marked_exports.parse;
// ESM-uncomment-end
// END MONACOCHANGE

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function createElement(options) {
    var tagName = options.inline ? 'span' : 'div';
    var element = document.createElement(tagName);
    if (options.className) {
        element.className = options.className;
    }
    return element;
}
function renderText(text, options) {
    if (options === void 0) { options = {}; }
    var element = createElement(options);
    element.textContent = text;
    return element;
}
function renderFormattedText(formattedText, options) {
    if (options === void 0) { options = {}; }
    var element = createElement(options);
    _renderFormattedText(element, parseFormattedText(formattedText), options.actionHandler);
    return element;
}
/**
 * Create html nodes for the given content element.
 */
function renderMarkdown(markdown, options) {
    if (options === void 0) { options = {}; }
    var element = createElement(options);
    // signal to code-block render that the
    // element has been created
    var signalInnerHTML;
    var withInnerHTML = new Promise(function (c) { return signalInnerHTML = c; });
    var renderer = new Renderer();
    renderer.image = function (href, title, text) {
        var dimensions = [];
        if (href) {
            var splitted = href.split('|').map(function (s) { return s.trim(); });
            href = splitted[0];
            var parameters = splitted[1];
            if (parameters) {
                var heightFromParams = /height=(\d+)/.exec(parameters);
                var widthFromParams = /width=(\d+)/.exec(parameters);
                var height = heightFromParams ? heightFromParams[1] : '';
                var width = widthFromParams ? widthFromParams[1] : '';
                var widthIsFinite = isFinite(parseInt(width));
                var heightIsFinite = isFinite(parseInt(height));
                if (widthIsFinite) {
                    dimensions.push("width=\"" + width + "\"");
                }
                if (heightIsFinite) {
                    dimensions.push("height=\"" + height + "\"");
                }
            }
        }
        var attributes = [];
        if (href) {
            attributes.push("src=\"" + href + "\"");
        }
        if (text) {
            attributes.push("alt=\"" + text + "\"");
        }
        if (title) {
            attributes.push("title=\"" + title + "\"");
        }
        if (dimensions.length) {
            attributes = attributes.concat(dimensions);
        }
        return '<img ' + attributes.join(' ') + '>';
    };
    renderer.link = function (href, title, text) {
        // Remove markdown escapes. Workaround for https://github.com/chjj/marked/issues/829
        if (href === text) { // raw link case
            text = removeMarkdownEscapes(text);
        }
        title = removeMarkdownEscapes(title);
        href = removeMarkdownEscapes(href);
        if (!href
            || href.match(/^data:|javascript:/i)
            || (href.match(/^command:/i) && !markdown.isTrusted)
            || href.match(/^command:(\/\/\/)?_workbench\.downloadResource/i)) {
            // drop the link
            return text;
        }
        else {
            return "<a href=\"#\" data-href=\"" + href + "\" title=\"" + (title || href) + "\">" + text + "</a>";
        }
    };
    renderer.paragraph = function (text) {
        return "<p>" + text + "</p>";
    };
    if (options.codeBlockRenderer) {
        renderer.code = function (code, lang) {
            var value = options.codeBlockRenderer(lang, code);
            // when code-block rendering is async we return sync
            // but update the node with the real result later.
            var id = defaultGenerator.nextId();
            var promise = Promise.all([value, withInnerHTML]).then(function (values) {
                var strValue = values[0];
                var span = element.querySelector("div[data-code=\"" + id + "\"]");
                if (span) {
                    span.innerHTML = strValue;
                }
            }).catch(function (err) {
                // ignore
            });
            if (options.codeBlockRenderCallback) {
                promise.then(options.codeBlockRenderCallback);
            }
            return "<div class=\"code\" data-code=\"" + id + "\">" + escape(code) + "</div>";
        };
    }
    if (options.actionHandler) {
        options.actionHandler.disposeables.push(addStandardDisposableListener(element, 'click', function (event) {
            var target = event.target;
            if (target.tagName !== 'A') {
                target = target.parentElement;
                if (!target || target.tagName !== 'A') {
                    return;
                }
            }
            try {
                var href = target.dataset['href'];
                if (href) {
                    options.actionHandler.callback(href, event);
                }
            }
            catch (err) {
                onUnexpectedError(err);
            }
            finally {
                event.preventDefault();
            }
        }));
    }
    var markedOptions = {
        sanitize: true,
        renderer: renderer
    };
    element.innerHTML = parse$1(markdown.value, markedOptions);
    signalInnerHTML();
    return element;
}
// --- formatted string parsing
var StringStream = /** @class */ (function () {
    function StringStream(source) {
        this.source = source;
        this.index = 0;
    }
    StringStream.prototype.eos = function () {
        return this.index >= this.source.length;
    };
    StringStream.prototype.next = function () {
        var next = this.peek();
        this.advance();
        return next;
    };
    StringStream.prototype.peek = function () {
        return this.source[this.index];
    };
    StringStream.prototype.advance = function () {
        this.index++;
    };
    return StringStream;
}());
function _renderFormattedText(element, treeNode, actionHandler) {
    var child;
    if (treeNode.type === 2 /* Text */) {
        child = document.createTextNode(treeNode.content || '');
    }
    else if (treeNode.type === 3 /* Bold */) {
        child = document.createElement('b');
    }
    else if (treeNode.type === 4 /* Italics */) {
        child = document.createElement('i');
    }
    else if (treeNode.type === 5 /* Action */ && actionHandler) {
        var a = document.createElement('a');
        a.href = '#';
        actionHandler.disposeables.push(addStandardDisposableListener(a, 'click', function (event) {
            actionHandler.callback(String(treeNode.index), event);
        }));
        child = a;
    }
    else if (treeNode.type === 7 /* NewLine */) {
        child = document.createElement('br');
    }
    else if (treeNode.type === 1 /* Root */) {
        child = element;
    }
    if (child && element !== child) {
        element.appendChild(child);
    }
    if (child && Array.isArray(treeNode.children)) {
        treeNode.children.forEach(function (nodeChild) {
            _renderFormattedText(child, nodeChild, actionHandler);
        });
    }
}
function parseFormattedText(content) {
    var root = {
        type: 1 /* Root */,
        children: []
    };
    var actionItemIndex = 0;
    var current = root;
    var stack = [];
    var stream = new StringStream(content);
    while (!stream.eos()) {
        var next = stream.next();
        var isEscapedFormatType = (next === '\\' && formatTagType(stream.peek()) !== 0 /* Invalid */);
        if (isEscapedFormatType) {
            next = stream.next(); // unread the backslash if it escapes a format tag type
        }
        if (!isEscapedFormatType && isFormatTag(next) && next === stream.peek()) {
            stream.advance();
            if (current.type === 2 /* Text */) {
                current = stack.pop();
            }
            var type = formatTagType(next);
            if (current.type === type || (current.type === 5 /* Action */ && type === 6 /* ActionClose */)) {
                current = stack.pop();
            }
            else {
                var newCurrent = {
                    type: type,
                    children: []
                };
                if (type === 5 /* Action */) {
                    newCurrent.index = actionItemIndex;
                    actionItemIndex++;
                }
                current.children.push(newCurrent);
                stack.push(current);
                current = newCurrent;
            }
        }
        else if (next === '\n') {
            if (current.type === 2 /* Text */) {
                current = stack.pop();
            }
            current.children.push({
                type: 7 /* NewLine */
            });
        }
        else {
            if (current.type !== 2 /* Text */) {
                var textCurrent = {
                    type: 2 /* Text */,
                    content: next
                };
                current.children.push(textCurrent);
                stack.push(current);
                current = textCurrent;
            }
            else {
                current.content += next;
            }
        }
    }
    if (current.type === 2 /* Text */) {
        current = stack.pop();
    }
    return root;
}
function isFormatTag(char) {
    return formatTagType(char) !== 0 /* Invalid */;
}
function formatTagType(char) {
    switch (char) {
        case '*':
            return 3 /* Bold */;
        case '_':
            return 4 /* Italics */;
        case '[':
            return 5 /* Action */;
        case ']':
            return 6 /* ActionClose */;
        default:
            return 0 /* Invalid */;
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ariaContainer;
var alertContainer;
var statusContainer;
function setARIAContainer(parent) {
    ariaContainer = document.createElement('div');
    ariaContainer.className = 'monaco-aria-container';
    alertContainer = document.createElement('div');
    alertContainer.className = 'monaco-alert';
    alertContainer.setAttribute('role', 'alert');
    alertContainer.setAttribute('aria-atomic', 'true');
    ariaContainer.appendChild(alertContainer);
    statusContainer = document.createElement('div');
    statusContainer.className = 'monaco-status';
    statusContainer.setAttribute('role', 'status');
    statusContainer.setAttribute('aria-atomic', 'true');
    ariaContainer.appendChild(statusContainer);
    parent.appendChild(ariaContainer);
}
/**
 * Given the provided message, will make sure that it is read as alert to screen readers.
 */
function alert(msg) {
    insertMessage(alertContainer, msg);
}
/**
 * Given the provided message, will make sure that it is read as status to screen readers.
 */
function status(msg) {
    if (isMacintosh) {
        alert(msg); // VoiceOver does not seem to support status role
    }
    else {
        insertMessage(statusContainer, msg);
    }
}
var repeatedTimes = 0;
var prevText = undefined;
function insertMessage(target, msg) {
    if (!ariaContainer) {
        // console.warn('ARIA support needs a container. Call setARIAContainer() first.');
        return;
    }
    if (prevText === msg) {
        repeatedTimes++;
    }
    else {
        prevText = msg;
        repeatedTimes = 0;
    }
    switch (repeatedTimes) {
        case 0: break;
        case 1:
            msg = localize('repeated', "{0} (occurred again)", msg);
            break;
        default:
            msg = localize('repeatedNtimes', "{0} (occurred {1} times)", msg, repeatedTimes);
            break;
    }
    clearNode(target);
    target.textContent = msg;
    // See https://www.paciellogroup.com/blog/2012/06/html5-accessibility-chops-aria-rolealert-browser-support/
    target.style.visibility = 'hidden';
    target.style.visibility = 'visible';
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var HistoryNavigator = /** @class */ (function () {
    function HistoryNavigator(history, limit) {
        if (history === void 0) { history = []; }
        if (limit === void 0) { limit = 10; }
        this._initialize(history);
        this._limit = limit;
        this._onChange();
    }
    HistoryNavigator.prototype.add = function (t) {
        this._history.delete(t);
        this._history.add(t);
        this._onChange();
    };
    HistoryNavigator.prototype.next = function () {
        return this._navigator.next();
    };
    HistoryNavigator.prototype.previous = function () {
        return this._navigator.previous();
    };
    HistoryNavigator.prototype.current = function () {
        return this._navigator.current();
    };
    HistoryNavigator.prototype.parent = function () {
        return null;
    };
    HistoryNavigator.prototype.first = function () {
        return this._navigator.first();
    };
    HistoryNavigator.prototype.last = function () {
        return this._navigator.last();
    };
    HistoryNavigator.prototype.has = function (t) {
        return this._history.has(t);
    };
    HistoryNavigator.prototype._onChange = function () {
        this._reduceToLimit();
        this._navigator = new ArrayNavigator(this._elements, 0, this._elements.length, this._elements.length);
    };
    HistoryNavigator.prototype._reduceToLimit = function () {
        var data = this._elements;
        if (data.length > this._limit) {
            this._initialize(data.slice(data.length - this._limit));
        }
    };
    HistoryNavigator.prototype._initialize = function (history) {
        this._history = new Set();
        for (var _i = 0, history_1 = history; _i < history_1.length; _i++) {
            var entry = history_1[_i];
            this._history.add(entry);
        }
    };
    Object.defineProperty(HistoryNavigator.prototype, "_elements", {
        get: function () {
            var elements = [];
            this._history.forEach(function (e) { return elements.push(e); });
            return elements;
        },
        enumerable: true,
        configurable: true
    });
    return HistoryNavigator;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1k = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var $$1 = $;
var defaultOpts$1 = {
    inputBackground: Color.fromHex('#3C3C3C'),
    inputForeground: Color.fromHex('#CCCCCC'),
    inputValidationInfoBorder: Color.fromHex('#55AAFF'),
    inputValidationInfoBackground: Color.fromHex('#063B49'),
    inputValidationWarningBorder: Color.fromHex('#B89500'),
    inputValidationWarningBackground: Color.fromHex('#352A05'),
    inputValidationErrorBorder: Color.fromHex('#BE1100'),
    inputValidationErrorBackground: Color.fromHex('#5A1D1D')
};
var InputBox = /** @class */ (function (_super) {
    __extends$1k(InputBox, _super);
    function InputBox(container, contextViewProvider, options) {
        var _this = _super.call(this) || this;
        _this.state = 'idle';
        _this._onDidChange = _this._register(new Emitter());
        _this.onDidChange = _this._onDidChange.event;
        _this._onDidHeightChange = _this._register(new Emitter());
        _this.contextViewProvider = contextViewProvider;
        _this.options = options || Object.create(null);
        mixin(_this.options, defaultOpts$1, false);
        _this.message = null;
        _this.cachedHeight = null;
        _this.placeholder = _this.options.placeholder || '';
        _this.ariaLabel = _this.options.ariaLabel || '';
        _this.inputBackground = _this.options.inputBackground;
        _this.inputForeground = _this.options.inputForeground;
        _this.inputBorder = _this.options.inputBorder;
        _this.inputValidationInfoBorder = _this.options.inputValidationInfoBorder;
        _this.inputValidationInfoBackground = _this.options.inputValidationInfoBackground;
        _this.inputValidationInfoForeground = _this.options.inputValidationInfoForeground;
        _this.inputValidationWarningBorder = _this.options.inputValidationWarningBorder;
        _this.inputValidationWarningBackground = _this.options.inputValidationWarningBackground;
        _this.inputValidationWarningForeground = _this.options.inputValidationWarningForeground;
        _this.inputValidationErrorBorder = _this.options.inputValidationErrorBorder;
        _this.inputValidationErrorBackground = _this.options.inputValidationErrorBackground;
        _this.inputValidationErrorForeground = _this.options.inputValidationErrorForeground;
        if (_this.options.validationOptions) {
            _this.validation = _this.options.validationOptions.validation;
        }
        _this.element = append(container, $$1('.monaco-inputbox.idle'));
        var tagName = _this.options.flexibleHeight ? 'textarea' : 'input';
        var wrapper = append(_this.element, $$1('.wrapper'));
        _this.input = append(wrapper, $$1(tagName + '.input'));
        _this.input.setAttribute('autocorrect', 'off');
        _this.input.setAttribute('autocapitalize', 'off');
        _this.input.setAttribute('spellcheck', 'false');
        _this.onfocus(_this.input, function () { return addClass(_this.element, 'synthetic-focus'); });
        _this.onblur(_this.input, function () { return removeClass(_this.element, 'synthetic-focus'); });
        if (_this.options.flexibleHeight) {
            _this.mirror = append(wrapper, $$1('div.mirror'));
        }
        else {
            _this.input.type = _this.options.type || 'text';
            _this.input.setAttribute('wrap', 'off');
        }
        if (_this.ariaLabel) {
            _this.input.setAttribute('aria-label', _this.ariaLabel);
        }
        if (_this.placeholder) {
            _this.setPlaceHolder(_this.placeholder);
        }
        _this.oninput(_this.input, function () { return _this.onValueChange(); });
        _this.onblur(_this.input, function () { return _this.onBlur(); });
        _this.onfocus(_this.input, function () { return _this.onFocus(); });
        // Add placeholder shim for IE because IE decides to hide the placeholder on focus (we dont want that!)
        if (_this.placeholder && isIE) {
            _this.onclick(_this.input, function (e) {
                EventHelper.stop(e, true);
                _this.input.focus();
            });
        }
        setTimeout(function () {
            if (!_this.input) {
                return;
            }
            _this.updateMirror();
        }, 0);
        // Support actions
        if (_this.options.actions) {
            _this.actionbar = _this._register(new ActionBar(_this.element));
            _this.actionbar.push(_this.options.actions, { icon: true, label: false });
        }
        _this.applyStyles();
        return _this;
    }
    InputBox.prototype.onBlur = function () {
        this._hideMessage();
    };
    InputBox.prototype.onFocus = function () {
        this._showMessage();
    };
    InputBox.prototype.setPlaceHolder = function (placeHolder) {
        if (this.input) {
            this.input.setAttribute('placeholder', placeHolder);
            this.input.title = placeHolder;
        }
    };
    InputBox.prototype.setAriaLabel = function (label) {
        this.ariaLabel = label;
        if (this.input) {
            if (label) {
                this.input.setAttribute('aria-label', this.ariaLabel);
            }
            else {
                this.input.removeAttribute('aria-label');
            }
        }
    };
    Object.defineProperty(InputBox.prototype, "inputElement", {
        get: function () {
            return this.input;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputBox.prototype, "value", {
        get: function () {
            return this.input.value;
        },
        set: function (newValue) {
            if (this.input.value !== newValue) {
                this.input.value = newValue;
                this.onValueChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    InputBox.prototype.focus = function () {
        this.input.focus();
    };
    InputBox.prototype.blur = function () {
        this.input.blur();
    };
    InputBox.prototype.hasFocus = function () {
        return document.activeElement === this.input;
    };
    InputBox.prototype.select = function (range) {
        if (range === void 0) { range = null; }
        this.input.select();
        if (range) {
            this.input.setSelectionRange(range.start, range.end);
        }
    };
    InputBox.prototype.enable = function () {
        this.input.removeAttribute('disabled');
    };
    InputBox.prototype.disable = function () {
        this.input.disabled = true;
        this._hideMessage();
    };
    InputBox.prototype.setEnabled = function (enabled) {
        if (enabled) {
            this.enable();
        }
        else {
            this.disable();
        }
    };
    Object.defineProperty(InputBox.prototype, "width", {
        get: function () {
            return getTotalWidth(this.input);
        },
        set: function (width) {
            this.input.style.width = width + 'px';
        },
        enumerable: true,
        configurable: true
    });
    InputBox.prototype.showMessage = function (message, force) {
        this.message = message;
        removeClass(this.element, 'idle');
        removeClass(this.element, 'info');
        removeClass(this.element, 'warning');
        removeClass(this.element, 'error');
        addClass(this.element, this.classForType(message.type));
        var styles = this.stylesForType(this.message.type);
        this.element.style.border = styles.border ? "1px solid " + styles.border : null;
        // ARIA Support
        var alertText;
        if (message.type === 3 /* ERROR */) {
            alertText = localize('alertErrorMessage', "Error: {0}", message.content);
        }
        else if (message.type === 2 /* WARNING */) {
            alertText = localize('alertWarningMessage', "Warning: {0}", message.content);
        }
        else {
            alertText = localize('alertInfoMessage', "Info: {0}", message.content);
        }
        alert(alertText);
        if (this.hasFocus() || force) {
            this._showMessage();
        }
    };
    InputBox.prototype.hideMessage = function () {
        this.message = null;
        removeClass(this.element, 'info');
        removeClass(this.element, 'warning');
        removeClass(this.element, 'error');
        addClass(this.element, 'idle');
        this._hideMessage();
        this.applyStyles();
    };
    InputBox.prototype.validate = function () {
        var errorMsg = null;
        if (this.validation) {
            errorMsg = this.validation(this.value);
            if (errorMsg) {
                this.inputElement.setAttribute('aria-invalid', 'true');
                this.showMessage(errorMsg);
            }
            else if (this.inputElement.hasAttribute('aria-invalid')) {
                this.inputElement.removeAttribute('aria-invalid');
                this.hideMessage();
            }
        }
        return !errorMsg;
    };
    InputBox.prototype.stylesForType = function (type) {
        switch (type) {
            case 1 /* INFO */: return { border: this.inputValidationInfoBorder, background: this.inputValidationInfoBackground, foreground: this.inputValidationInfoForeground };
            case 2 /* WARNING */: return { border: this.inputValidationWarningBorder, background: this.inputValidationWarningBackground, foreground: this.inputValidationWarningForeground };
            default: return { border: this.inputValidationErrorBorder, background: this.inputValidationErrorBackground, foreground: this.inputValidationErrorForeground };
        }
    };
    InputBox.prototype.classForType = function (type) {
        switch (type) {
            case 1 /* INFO */: return 'info';
            case 2 /* WARNING */: return 'warning';
            default: return 'error';
        }
    };
    InputBox.prototype._showMessage = function () {
        var _this = this;
        if (!this.contextViewProvider || !this.message) {
            return;
        }
        var div;
        var layout = function () { return div.style.width = getTotalWidth(_this.element) + 'px'; };
        this.state = 'open';
        this.contextViewProvider.showContextView({
            getAnchor: function () { return _this.element; },
            anchorAlignment: 1 /* RIGHT */,
            render: function (container) {
                div = append(container, $$1('.monaco-inputbox-container'));
                layout();
                var renderOptions = {
                    inline: true,
                    className: 'monaco-inputbox-message'
                };
                var spanElement = (_this.message.formatContent
                    ? renderFormattedText(_this.message.content, renderOptions)
                    : renderText(_this.message.content, renderOptions));
                addClass(spanElement, _this.classForType(_this.message.type));
                var styles = _this.stylesForType(_this.message.type);
                spanElement.style.backgroundColor = styles.background ? styles.background.toString() : null;
                spanElement.style.color = styles.foreground ? styles.foreground.toString() : null;
                spanElement.style.border = styles.border ? "1px solid " + styles.border : null;
                append(div, spanElement);
                return null;
            },
            layout: layout
        });
    };
    InputBox.prototype._hideMessage = function () {
        if (!this.contextViewProvider || this.state !== 'open') {
            return;
        }
        this.state = 'idle';
        this.contextViewProvider.hideContextView();
    };
    InputBox.prototype.onValueChange = function () {
        this._onDidChange.fire(this.value);
        this.validate();
        this.updateMirror();
        if (this.state === 'open') {
            this.contextViewProvider.layout();
        }
    };
    InputBox.prototype.updateMirror = function () {
        if (!this.mirror) {
            return;
        }
        var value = this.value || this.placeholder;
        var lastCharCode = value.charCodeAt(value.length - 1);
        var suffix = lastCharCode === 10 ? ' ' : '';
        this.mirror.textContent = value + suffix;
        this.layout();
    };
    InputBox.prototype.style = function (styles) {
        this.inputBackground = styles.inputBackground;
        this.inputForeground = styles.inputForeground;
        this.inputBorder = styles.inputBorder;
        this.inputValidationInfoBackground = styles.inputValidationInfoBackground;
        this.inputValidationInfoForeground = styles.inputValidationInfoForeground;
        this.inputValidationInfoBorder = styles.inputValidationInfoBorder;
        this.inputValidationWarningBackground = styles.inputValidationWarningBackground;
        this.inputValidationWarningForeground = styles.inputValidationWarningForeground;
        this.inputValidationWarningBorder = styles.inputValidationWarningBorder;
        this.inputValidationErrorBackground = styles.inputValidationErrorBackground;
        this.inputValidationErrorForeground = styles.inputValidationErrorForeground;
        this.inputValidationErrorBorder = styles.inputValidationErrorBorder;
        this.applyStyles();
    };
    InputBox.prototype.applyStyles = function () {
        if (this.element) {
            var background = this.inputBackground ? this.inputBackground.toString() : null;
            var foreground = this.inputForeground ? this.inputForeground.toString() : null;
            var border = this.inputBorder ? this.inputBorder.toString() : null;
            this.element.style.backgroundColor = background;
            this.element.style.color = foreground;
            this.input.style.backgroundColor = background;
            this.input.style.color = foreground;
            this.element.style.borderWidth = border ? '1px' : null;
            this.element.style.borderStyle = border ? 'solid' : null;
            this.element.style.borderColor = border;
        }
    };
    InputBox.prototype.layout = function () {
        if (!this.mirror) {
            return;
        }
        var previousHeight = this.cachedHeight;
        this.cachedHeight = getTotalHeight(this.mirror);
        if (previousHeight !== this.cachedHeight) {
            this.input.style.height = this.cachedHeight + 'px';
            this._onDidHeightChange.fire(this.cachedHeight);
        }
    };
    InputBox.prototype.dispose = function () {
        this._hideMessage();
        this.element = null;
        this.input = null;
        this.contextViewProvider = null;
        this.message = null;
        this.placeholder = null;
        this.ariaLabel = null;
        this.validation = null;
        this.state = null;
        this.actionbar = null;
        _super.prototype.dispose.call(this);
    };
    return InputBox;
}(Widget$1));
var HistoryInputBox = /** @class */ (function (_super) {
    __extends$1k(HistoryInputBox, _super);
    function HistoryInputBox(container, contextViewProvider, options) {
        var _this = _super.call(this, container, contextViewProvider, options) || this;
        _this.history = new HistoryNavigator(options.history, 100);
        return _this;
    }
    HistoryInputBox.prototype.addToHistory = function () {
        if (this.value && this.value !== this.getCurrentValue()) {
            this.history.add(this.value);
        }
    };
    HistoryInputBox.prototype.showNextValue = function () {
        if (!this.history.has(this.value)) {
            this.addToHistory();
        }
        var next = this.getNextValue();
        if (next) {
            next = next === this.value ? this.getNextValue() : next;
        }
        if (next) {
            this.value = next;
            status(this.value);
        }
    };
    HistoryInputBox.prototype.showPreviousValue = function () {
        if (!this.history.has(this.value)) {
            this.addToHistory();
        }
        var previous = this.getPreviousValue();
        if (previous) {
            previous = previous === this.value ? this.getPreviousValue() : previous;
        }
        if (previous) {
            this.value = previous;
            status(this.value);
        }
    };
    HistoryInputBox.prototype.getCurrentValue = function () {
        var currentValue = this.history.current();
        if (!currentValue) {
            currentValue = this.history.last();
            this.history.next();
        }
        return currentValue;
    };
    HistoryInputBox.prototype.getPreviousValue = function () {
        return this.history.previous() || this.history.first();
    };
    HistoryInputBox.prototype.getNextValue = function () {
        return this.history.next() || this.history.last();
    };
    return HistoryInputBox;
}(InputBox));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1l = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var NLS_DEFAULT_LABEL = localize('defaultLabel', "input");
var FindInput = /** @class */ (function (_super) {
    __extends$1l(FindInput, _super);
    function FindInput(parent, contextViewProvider, _showOptionButtons, options) {
        var _this = _super.call(this) || this;
        _this._showOptionButtons = _showOptionButtons;
        _this.fixFocusOnOptionClickEnabled = true;
        _this._onDidOptionChange = _this._register(new Emitter());
        _this.onDidOptionChange = _this._onDidOptionChange.event;
        _this._onKeyDown = _this._register(new Emitter());
        _this.onKeyDown = _this._onKeyDown.event;
        _this._onMouseDown = _this._register(new Emitter());
        _this.onMouseDown = _this._onMouseDown.event;
        _this._onInput = _this._register(new Emitter());
        _this._onKeyUp = _this._register(new Emitter());
        _this._onCaseSensitiveKeyDown = _this._register(new Emitter());
        _this.onCaseSensitiveKeyDown = _this._onCaseSensitiveKeyDown.event;
        _this._onRegexKeyDown = _this._register(new Emitter());
        _this._lastHighlightFindOptions = 0;
        _this.contextViewProvider = contextViewProvider;
        _this.width = options.width || 100;
        _this.placeholder = options.placeholder || '';
        _this.validation = options.validation;
        _this.label = options.label || NLS_DEFAULT_LABEL;
        _this.inputActiveOptionBorder = options.inputActiveOptionBorder;
        _this.inputBackground = options.inputBackground;
        _this.inputForeground = options.inputForeground;
        _this.inputBorder = options.inputBorder;
        _this.inputValidationInfoBorder = options.inputValidationInfoBorder;
        _this.inputValidationInfoBackground = options.inputValidationInfoBackground;
        _this.inputValidationInfoForeground = options.inputValidationInfoForeground;
        _this.inputValidationWarningBorder = options.inputValidationWarningBorder;
        _this.inputValidationWarningBackground = options.inputValidationWarningBackground;
        _this.inputValidationWarningForeground = options.inputValidationWarningForeground;
        _this.inputValidationErrorBorder = options.inputValidationErrorBorder;
        _this.inputValidationErrorBackground = options.inputValidationErrorBackground;
        _this.inputValidationErrorForeground = options.inputValidationErrorForeground;
        _this.regex = null;
        _this.wholeWords = null;
        _this.caseSensitive = null;
        _this.domNode = null;
        _this.inputBox = null;
        _this.buildDomNode(options.appendCaseSensitiveLabel || '', options.appendWholeWordsLabel || '', options.appendRegexLabel || '', options.history);
        if (Boolean(parent)) {
            parent.appendChild(_this.domNode);
        }
        _this.onkeydown(_this.inputBox.inputElement, function (e) { return _this._onKeyDown.fire(e); });
        _this.onkeyup(_this.inputBox.inputElement, function (e) { return _this._onKeyUp.fire(e); });
        _this.oninput(_this.inputBox.inputElement, function (e) { return _this._onInput.fire(); });
        _this.onmousedown(_this.inputBox.inputElement, function (e) { return _this._onMouseDown.fire(e); });
        return _this;
    }
    FindInput.prototype.enable = function () {
        removeClass(this.domNode, 'disabled');
        this.inputBox.enable();
        this.regex.enable();
        this.wholeWords.enable();
        this.caseSensitive.enable();
    };
    FindInput.prototype.disable = function () {
        addClass(this.domNode, 'disabled');
        this.inputBox.disable();
        this.regex.disable();
        this.wholeWords.disable();
        this.caseSensitive.disable();
    };
    FindInput.prototype.setFocusInputOnOptionClick = function (value) {
        this.fixFocusOnOptionClickEnabled = value;
    };
    FindInput.prototype.setEnabled = function (enabled) {
        if (enabled) {
            this.enable();
        }
        else {
            this.disable();
        }
    };
    FindInput.prototype.getValue = function () {
        return this.inputBox.value;
    };
    FindInput.prototype.setValue = function (value) {
        if (this.inputBox.value !== value) {
            this.inputBox.value = value;
        }
    };
    FindInput.prototype.style = function (styles) {
        this.inputActiveOptionBorder = styles.inputActiveOptionBorder;
        this.inputBackground = styles.inputBackground;
        this.inputForeground = styles.inputForeground;
        this.inputBorder = styles.inputBorder;
        this.inputValidationInfoBackground = styles.inputValidationInfoBackground;
        this.inputValidationInfoForeground = styles.inputValidationInfoForeground;
        this.inputValidationInfoBorder = styles.inputValidationInfoBorder;
        this.inputValidationWarningBackground = styles.inputValidationWarningBackground;
        this.inputValidationWarningForeground = styles.inputValidationWarningForeground;
        this.inputValidationWarningBorder = styles.inputValidationWarningBorder;
        this.inputValidationErrorBackground = styles.inputValidationErrorBackground;
        this.inputValidationErrorForeground = styles.inputValidationErrorForeground;
        this.inputValidationErrorBorder = styles.inputValidationErrorBorder;
        this.applyStyles();
    };
    FindInput.prototype.applyStyles = function () {
        if (this.domNode) {
            var checkBoxStyles = {
                inputActiveOptionBorder: this.inputActiveOptionBorder,
            };
            this.regex.style(checkBoxStyles);
            this.wholeWords.style(checkBoxStyles);
            this.caseSensitive.style(checkBoxStyles);
            var inputBoxStyles = {
                inputBackground: this.inputBackground,
                inputForeground: this.inputForeground,
                inputBorder: this.inputBorder,
                inputValidationInfoBackground: this.inputValidationInfoBackground,
                inputValidationInfoForeground: this.inputValidationInfoForeground,
                inputValidationInfoBorder: this.inputValidationInfoBorder,
                inputValidationWarningBackground: this.inputValidationWarningBackground,
                inputValidationWarningForeground: this.inputValidationWarningForeground,
                inputValidationWarningBorder: this.inputValidationWarningBorder,
                inputValidationErrorBackground: this.inputValidationErrorBackground,
                inputValidationErrorForeground: this.inputValidationErrorForeground,
                inputValidationErrorBorder: this.inputValidationErrorBorder
            };
            this.inputBox.style(inputBoxStyles);
        }
    };
    FindInput.prototype.select = function () {
        this.inputBox.select();
    };
    FindInput.prototype.focus = function () {
        this.inputBox.focus();
    };
    FindInput.prototype.getCaseSensitive = function () {
        return this.caseSensitive.checked;
    };
    FindInput.prototype.setCaseSensitive = function (value) {
        this.caseSensitive.checked = value;
        this.setInputWidth();
    };
    FindInput.prototype.getWholeWords = function () {
        return this.wholeWords.checked;
    };
    FindInput.prototype.setWholeWords = function (value) {
        this.wholeWords.checked = value;
        this.setInputWidth();
    };
    FindInput.prototype.getRegex = function () {
        return this.regex.checked;
    };
    FindInput.prototype.setRegex = function (value) {
        this.regex.checked = value;
        this.setInputWidth();
        this.validate();
    };
    FindInput.prototype.focusOnCaseSensitive = function () {
        this.caseSensitive.focus();
    };
    FindInput.prototype.highlightFindOptions = function () {
        removeClass(this.domNode, 'highlight-' + (this._lastHighlightFindOptions));
        this._lastHighlightFindOptions = 1 - this._lastHighlightFindOptions;
        addClass(this.domNode, 'highlight-' + (this._lastHighlightFindOptions));
    };
    FindInput.prototype.setInputWidth = function () {
        var w = this.width - this.caseSensitive.width() - this.wholeWords.width() - this.regex.width();
        this.inputBox.width = w;
    };
    FindInput.prototype.buildDomNode = function (appendCaseSensitiveLabel, appendWholeWordsLabel, appendRegexLabel, history) {
        var _this = this;
        this.domNode = document.createElement('div');
        this.domNode.style.width = this.width + 'px';
        addClass(this.domNode, 'monaco-findInput');
        this.inputBox = this._register(new HistoryInputBox(this.domNode, this.contextViewProvider, {
            placeholder: this.placeholder || '',
            ariaLabel: this.label || '',
            validationOptions: {
                validation: this.validation || null
            },
            inputBackground: this.inputBackground,
            inputForeground: this.inputForeground,
            inputBorder: this.inputBorder,
            inputValidationInfoBackground: this.inputValidationInfoBackground,
            inputValidationInfoForeground: this.inputValidationInfoForeground,
            inputValidationInfoBorder: this.inputValidationInfoBorder,
            inputValidationWarningBackground: this.inputValidationWarningBackground,
            inputValidationWarningForeground: this.inputValidationWarningForeground,
            inputValidationWarningBorder: this.inputValidationWarningBorder,
            inputValidationErrorBackground: this.inputValidationErrorBackground,
            inputValidationErrorForeground: this.inputValidationErrorForeground,
            inputValidationErrorBorder: this.inputValidationErrorBorder,
            history: history
        }));
        this.regex = this._register(new RegexCheckbox({
            appendTitle: appendRegexLabel,
            isChecked: false,
            inputActiveOptionBorder: this.inputActiveOptionBorder
        }));
        this._register(this.regex.onChange(function (viaKeyboard) {
            _this._onDidOptionChange.fire(viaKeyboard);
            if (!viaKeyboard && _this.fixFocusOnOptionClickEnabled) {
                _this.inputBox.focus();
            }
            _this.setInputWidth();
            _this.validate();
        }));
        this._register(this.regex.onKeyDown(function (e) {
            _this._onRegexKeyDown.fire(e);
        }));
        this.wholeWords = this._register(new WholeWordsCheckbox({
            appendTitle: appendWholeWordsLabel,
            isChecked: false,
            inputActiveOptionBorder: this.inputActiveOptionBorder
        }));
        this._register(this.wholeWords.onChange(function (viaKeyboard) {
            _this._onDidOptionChange.fire(viaKeyboard);
            if (!viaKeyboard && _this.fixFocusOnOptionClickEnabled) {
                _this.inputBox.focus();
            }
            _this.setInputWidth();
            _this.validate();
        }));
        this.caseSensitive = this._register(new CaseSensitiveCheckbox({
            appendTitle: appendCaseSensitiveLabel,
            isChecked: false,
            inputActiveOptionBorder: this.inputActiveOptionBorder
        }));
        this._register(this.caseSensitive.onChange(function (viaKeyboard) {
            _this._onDidOptionChange.fire(viaKeyboard);
            if (!viaKeyboard && _this.fixFocusOnOptionClickEnabled) {
                _this.inputBox.focus();
            }
            _this.setInputWidth();
            _this.validate();
        }));
        this._register(this.caseSensitive.onKeyDown(function (e) {
            _this._onCaseSensitiveKeyDown.fire(e);
        }));
        // Arrow-Key support to navigate between options
        var indexes = [this.caseSensitive.domNode, this.wholeWords.domNode, this.regex.domNode];
        this.onkeydown(this.domNode, function (event) {
            if (event.equals(15 /* LeftArrow */) || event.equals(17 /* RightArrow */) || event.equals(9 /* Escape */)) {
                var index = indexes.indexOf(document.activeElement);
                if (index >= 0) {
                    var newIndex = void 0;
                    if (event.equals(17 /* RightArrow */)) {
                        newIndex = (index + 1) % indexes.length;
                    }
                    else if (event.equals(15 /* LeftArrow */)) {
                        if (index === 0) {
                            newIndex = indexes.length - 1;
                        }
                        else {
                            newIndex = index - 1;
                        }
                    }
                    if (event.equals(9 /* Escape */)) {
                        indexes[index].blur();
                    }
                    else if (newIndex >= 0) {
                        indexes[newIndex].focus();
                    }
                    EventHelper.stop(event, true);
                }
            }
        });
        this.setInputWidth();
        var controls = document.createElement('div');
        controls.className = 'controls';
        controls.style.display = this._showOptionButtons ? 'block' : 'none';
        controls.appendChild(this.caseSensitive.domNode);
        controls.appendChild(this.wholeWords.domNode);
        controls.appendChild(this.regex.domNode);
        this.domNode.appendChild(controls);
    };
    FindInput.prototype.validate = function () {
        this.inputBox.validate();
    };
    FindInput.prototype.clearMessage = function () {
        this.inputBox.hideMessage();
    };
    FindInput.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    return FindInput;
}(Widget$1));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function bindContextScopedWidget(contextKeyService, widget, contextKey) {
    new RawContextKey(contextKey, widget).bindTo(contextKeyService);
}
function createWidgetScopedContextKeyService(contextKeyService, widget) {
    return contextKeyService.createScoped(widget.target);
}
function getContextScopedWidget(contextKeyService, contextKey) {
    return contextKeyService.getContext(document.activeElement).getValue(contextKey);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1m = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$5 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$4 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var HistoryNavigationWidgetContext = 'historyNavigationWidget';
var HistoryNavigationEnablementContext = 'historyNavigationEnabled';
function createAndBindHistoryNavigationWidgetScopedContextKeyService(contextKeyService, widget) {
    var scopedContextKeyService = createWidgetScopedContextKeyService(contextKeyService, widget);
    bindContextScopedWidget(scopedContextKeyService, widget, HistoryNavigationWidgetContext);
    var historyNavigationEnablement = new RawContextKey(HistoryNavigationEnablementContext, true).bindTo(scopedContextKeyService);
    return { scopedContextKeyService: scopedContextKeyService, historyNavigationEnablement: historyNavigationEnablement };
}
var ContextScopedHistoryInputBox = /** @class */ (function (_super) {
    __extends$1m(ContextScopedHistoryInputBox, _super);
    function ContextScopedHistoryInputBox(container, contextViewProvider, options, contextKeyService) {
        var _this = _super.call(this, container, contextViewProvider, options) || this;
        _this._register(createAndBindHistoryNavigationWidgetScopedContextKeyService(contextKeyService, { target: _this.element, historyNavigator: _this }).scopedContextKeyService);
        return _this;
    }
    ContextScopedHistoryInputBox = __decorate$5([
        __param$4(3, IContextKeyService)
    ], ContextScopedHistoryInputBox);
    return ContextScopedHistoryInputBox;
}(HistoryInputBox));
var ContextScopedFindInput = /** @class */ (function (_super) {
    __extends$1m(ContextScopedFindInput, _super);
    function ContextScopedFindInput(container, contextViewProvider, options, contextKeyService, showFindOptions) {
        if (showFindOptions === void 0) { showFindOptions = false; }
        var _this = _super.call(this, container, contextViewProvider, showFindOptions, options) || this;
        _this._register(createAndBindHistoryNavigationWidgetScopedContextKeyService(contextKeyService, { target: _this.inputBox.element, historyNavigator: _this.inputBox }).scopedContextKeyService);
        return _this;
    }
    ContextScopedFindInput = __decorate$5([
        __param$4(3, IContextKeyService)
    ], ContextScopedFindInput);
    return ContextScopedFindInput;
}(FindInput));
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: 'history.showPrevious',
    weight: 200 /* WorkbenchContrib */,
    when: ContextKeyExpr.and(new ContextKeyDefinedExpr(HistoryNavigationWidgetContext), new ContextKeyEqualsExpr(HistoryNavigationEnablementContext, true)),
    primary: 16 /* UpArrow */,
    secondary: [512 /* Alt */ | 16 /* UpArrow */],
    handler: function (accessor, arg2) {
        var historyInputBox = getContextScopedWidget(accessor.get(IContextKeyService), HistoryNavigationWidgetContext).historyNavigator;
        historyInputBox.showPreviousValue();
    }
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: 'history.showNext',
    weight: 200 /* WorkbenchContrib */,
    when: new ContextKeyAndExpr([new ContextKeyDefinedExpr(HistoryNavigationWidgetContext), new ContextKeyEqualsExpr(HistoryNavigationEnablementContext, true)]),
    primary: 18 /* DownArrow */,
    secondary: [512 /* Alt */ | 18 /* DownArrow */],
    handler: function (accessor, arg2) {
        var historyInputBox = getContextScopedWidget(accessor.get(IContextKeyService), HistoryNavigationWidgetContext).historyNavigator;
        historyInputBox.showNextValue();
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1n = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var NLS_FIND_INPUT_LABEL = localize('label.find', "Find");
var NLS_FIND_INPUT_PLACEHOLDER = localize('placeholder.find', "Find");
var NLS_PREVIOUS_MATCH_BTN_LABEL = localize('label.previousMatchButton', "Previous match");
var NLS_NEXT_MATCH_BTN_LABEL = localize('label.nextMatchButton', "Next match");
var NLS_TOGGLE_SELECTION_FIND_TITLE = localize('label.toggleSelectionFind', "Find in selection");
var NLS_CLOSE_BTN_LABEL = localize('label.closeButton', "Close");
var NLS_REPLACE_INPUT_LABEL = localize('label.replace', "Replace");
var NLS_REPLACE_INPUT_PLACEHOLDER = localize('placeholder.replace', "Replace");
var NLS_REPLACE_BTN_LABEL = localize('label.replaceButton', "Replace");
var NLS_REPLACE_ALL_BTN_LABEL = localize('label.replaceAllButton', "Replace All");
var NLS_TOGGLE_REPLACE_MODE_BTN_LABEL = localize('label.toggleReplaceButton', "Toggle Replace mode");
var NLS_MATCHES_COUNT_LIMIT_TITLE = localize('title.matchesCountLimit', "Only the first {0} results are highlighted, but all find operations work on the entire text.", MATCHES_LIMIT);
var NLS_MATCHES_LOCATION = localize('label.matchesLocation', "{0} of {1}");
var NLS_NO_RESULTS = localize('label.noResults', "No Results");
var FIND_WIDGET_INITIAL_WIDTH = 411;
var PART_WIDTH = 275;
var FIND_INPUT_AREA_WIDTH = PART_WIDTH - 54;
var REPLACE_INPUT_AREA_WIDTH = FIND_INPUT_AREA_WIDTH;
var MAX_MATCHES_COUNT_WIDTH = 69;
var FIND_ALL_CONTROLS_WIDTH = 17 /** Find Input margin-left */ + (MAX_MATCHES_COUNT_WIDTH + 3 + 1) /** Match Results */ + 23 /** Button */ * 4 + 2 /** sash */;
var FIND_INPUT_AREA_HEIGHT = 34; // The height of Find Widget when Replace Input is not visible.
var FIND_REPLACE_AREA_HEIGHT = 64; // The height of Find Widget when Replace Input is  visible.
var FindWidgetViewZone = /** @class */ (function () {
    function FindWidgetViewZone(afterLineNumber) {
        this.afterLineNumber = afterLineNumber;
        this.heightInPx = FIND_INPUT_AREA_HEIGHT;
        this.suppressMouseDown = false;
        this.domNode = document.createElement('div');
        this.domNode.className = 'dock-find-viewzone';
    }
    return FindWidgetViewZone;
}());
var FindWidget = /** @class */ (function (_super) {
    __extends$1n(FindWidget, _super);
    function FindWidget(codeEditor, controller, state, contextViewProvider, keybindingService, contextKeyService, themeService) {
        var _this = _super.call(this) || this;
        _this._codeEditor = codeEditor;
        _this._controller = controller;
        _this._state = state;
        _this._contextViewProvider = contextViewProvider;
        _this._keybindingService = keybindingService;
        _this._contextKeyService = contextKeyService;
        _this._isVisible = false;
        _this._isReplaceVisible = false;
        _this._ignoreChangeEvent = false;
        _this._updateHistoryDelayer = new Delayer(500);
        _this._register(toDisposable(function () { return _this._updateHistoryDelayer.cancel(); }));
        _this._register(_this._state.onFindReplaceStateChange(function (e) { return _this._onStateChanged(e); }));
        _this._buildDomNode();
        _this._updateButtons();
        _this._tryUpdateWidgetWidth();
        _this._register(_this._codeEditor.onDidChangeConfiguration(function (e) {
            if (e.readOnly) {
                if (_this._codeEditor.getConfiguration().readOnly) {
                    // Hide replace part if editor becomes read only
                    _this._state.change({ isReplaceRevealed: false }, false);
                }
                _this._updateButtons();
            }
            if (e.layoutInfo) {
                _this._tryUpdateWidgetWidth();
            }
            if (e.accessibilitySupport) {
                _this.updateAccessibilitySupport();
            }
        }));
        _this.updateAccessibilitySupport();
        _this._register(_this._codeEditor.onDidChangeCursorSelection(function () {
            if (_this._isVisible) {
                _this._updateToggleSelectionFindButton();
            }
        }));
        _this._register(_this._codeEditor.onDidFocusEditorWidget(function () {
            if (_this._isVisible) {
                var globalBufferTerm = _this._controller.getGlobalBufferTerm();
                if (globalBufferTerm && globalBufferTerm !== _this._state.searchString) {
                    _this._state.change({ searchString: globalBufferTerm }, true);
                    _this._findInput.select();
                }
            }
        }));
        _this._findInputFocused = CONTEXT_FIND_INPUT_FOCUSED.bindTo(contextKeyService);
        _this._findFocusTracker = _this._register(trackFocus(_this._findInput.inputBox.inputElement));
        _this._register(_this._findFocusTracker.onDidFocus(function () {
            _this._findInputFocused.set(true);
            _this._updateSearchScope();
        }));
        _this._register(_this._findFocusTracker.onDidBlur(function () {
            _this._findInputFocused.set(false);
        }));
        _this._replaceInputFocused = CONTEXT_REPLACE_INPUT_FOCUSED.bindTo(contextKeyService);
        _this._replaceFocusTracker = _this._register(trackFocus(_this._replaceInputBox.inputElement));
        _this._register(_this._replaceFocusTracker.onDidFocus(function () {
            _this._replaceInputFocused.set(true);
            _this._updateSearchScope();
        }));
        _this._register(_this._replaceFocusTracker.onDidBlur(function () {
            _this._replaceInputFocused.set(false);
        }));
        _this._codeEditor.addOverlayWidget(_this);
        _this._viewZone = new FindWidgetViewZone(0); // Put it before the first line then users can scroll beyond the first line.
        _this._applyTheme(themeService.getTheme());
        _this._register(themeService.onThemeChange(_this._applyTheme.bind(_this)));
        _this._register(_this._codeEditor.onDidChangeModel(function () {
            if (!_this._isVisible) {
                return;
            }
            if (_this._viewZoneId === undefined) {
                return;
            }
            _this._codeEditor.changeViewZones(function (accessor) {
                accessor.removeZone(_this._viewZoneId);
                _this._viewZoneId = undefined;
            });
        }));
        _this._register(_this._codeEditor.onDidScrollChange(function (e) {
            if (e.scrollTopChanged) {
                _this._layoutViewZone();
                return;
            }
            // for other scroll changes, layout the viewzone in next tick to avoid ruining current rendering.
            setTimeout(function () {
                _this._layoutViewZone();
            }, 0);
        }));
        return _this;
    }
    // ----- IOverlayWidget API
    FindWidget.prototype.getId = function () {
        return FindWidget.ID;
    };
    FindWidget.prototype.getDomNode = function () {
        return this._domNode;
    };
    FindWidget.prototype.getPosition = function () {
        if (this._isVisible) {
            return {
                preference: 0 /* TOP_RIGHT_CORNER */
            };
        }
        return null;
    };
    // ----- React to state changes
    FindWidget.prototype._onStateChanged = function (e) {
        if (e.searchString) {
            try {
                this._ignoreChangeEvent = true;
                this._findInput.setValue(this._state.searchString);
            }
            finally {
                this._ignoreChangeEvent = false;
            }
            this._updateButtons();
        }
        if (e.replaceString) {
            this._replaceInputBox.value = this._state.replaceString;
        }
        if (e.isRevealed) {
            if (this._state.isRevealed) {
                this._reveal();
            }
            else {
                this._hide(true);
            }
        }
        if (e.isReplaceRevealed) {
            if (this._state.isReplaceRevealed) {
                if (!this._codeEditor.getConfiguration().readOnly && !this._isReplaceVisible) {
                    this._isReplaceVisible = true;
                    this._replaceInputBox.width = this._findInput.inputBox.width;
                    this._updateButtons();
                }
            }
            else {
                if (this._isReplaceVisible) {
                    this._isReplaceVisible = false;
                    this._updateButtons();
                }
            }
        }
        if (e.isRegex) {
            this._findInput.setRegex(this._state.isRegex);
        }
        if (e.wholeWord) {
            this._findInput.setWholeWords(this._state.wholeWord);
        }
        if (e.matchCase) {
            this._findInput.setCaseSensitive(this._state.matchCase);
        }
        if (e.searchScope) {
            if (this._state.searchScope) {
                this._toggleSelectionFind.checked = true;
            }
            else {
                this._toggleSelectionFind.checked = false;
            }
            this._updateToggleSelectionFindButton();
        }
        if (e.searchString || e.matchesCount || e.matchesPosition) {
            var showRedOutline = (this._state.searchString.length > 0 && this._state.matchesCount === 0);
            toggleClass(this._domNode, 'no-results', showRedOutline);
            this._updateMatchesCount();
            this._updateButtons();
        }
        if (e.searchString || e.currentMatch) {
            this._layoutViewZone();
        }
        if (e.updateHistory) {
            this._delayedUpdateHistory();
        }
    };
    FindWidget.prototype._delayedUpdateHistory = function () {
        this._updateHistoryDelayer.trigger(this._updateHistory.bind(this));
    };
    FindWidget.prototype._updateHistory = function () {
        if (this._state.searchString) {
            this._findInput.inputBox.addToHistory();
        }
        if (this._state.replaceString) {
            this._replaceInputBox.addToHistory();
        }
    };
    FindWidget.prototype._updateMatchesCount = function () {
        this._matchesCount.style.minWidth = MAX_MATCHES_COUNT_WIDTH + 'px';
        if (this._state.matchesCount >= MATCHES_LIMIT) {
            this._matchesCount.title = NLS_MATCHES_COUNT_LIMIT_TITLE;
        }
        else {
            this._matchesCount.title = '';
        }
        // remove previous content
        if (this._matchesCount.firstChild) {
            this._matchesCount.removeChild(this._matchesCount.firstChild);
        }
        var label;
        if (this._state.matchesCount > 0) {
            var matchesCount = String(this._state.matchesCount);
            if (this._state.matchesCount >= MATCHES_LIMIT) {
                matchesCount += '+';
            }
            var matchesPosition = String(this._state.matchesPosition);
            if (matchesPosition === '0') {
                matchesPosition = '?';
            }
            label = format(NLS_MATCHES_LOCATION, matchesPosition, matchesCount);
        }
        else {
            label = NLS_NO_RESULTS;
        }
        this._matchesCount.appendChild(document.createTextNode(label));
        MAX_MATCHES_COUNT_WIDTH = Math.max(MAX_MATCHES_COUNT_WIDTH, this._matchesCount.clientWidth);
    };
    // ----- actions
    /**
     * If 'selection find' is ON we should not disable the button (its function is to cancel 'selection find').
     * If 'selection find' is OFF we enable the button only if there is a selection.
     */
    FindWidget.prototype._updateToggleSelectionFindButton = function () {
        var selection = this._codeEditor.getSelection();
        var isSelection = selection ? (selection.startLineNumber !== selection.endLineNumber || selection.startColumn !== selection.endColumn) : false;
        var isChecked = this._toggleSelectionFind.checked;
        this._toggleSelectionFind.setEnabled(this._isVisible && (isChecked || isSelection));
    };
    FindWidget.prototype._updateButtons = function () {
        this._findInput.setEnabled(this._isVisible);
        this._replaceInputBox.setEnabled(this._isVisible && this._isReplaceVisible);
        this._updateToggleSelectionFindButton();
        this._closeBtn.setEnabled(this._isVisible);
        var findInputIsNonEmpty = (this._state.searchString.length > 0);
        var matchesCount = this._state.matchesCount ? true : false;
        this._prevBtn.setEnabled(this._isVisible && findInputIsNonEmpty && matchesCount);
        this._nextBtn.setEnabled(this._isVisible && findInputIsNonEmpty && matchesCount);
        this._replaceBtn.setEnabled(this._isVisible && this._isReplaceVisible && findInputIsNonEmpty);
        this._replaceAllBtn.setEnabled(this._isVisible && this._isReplaceVisible && findInputIsNonEmpty);
        toggleClass(this._domNode, 'replaceToggled', this._isReplaceVisible);
        this._toggleReplaceBtn.toggleClass('collapse', !this._isReplaceVisible);
        this._toggleReplaceBtn.toggleClass('expand', this._isReplaceVisible);
        this._toggleReplaceBtn.setExpanded(this._isReplaceVisible);
        var canReplace = !this._codeEditor.getConfiguration().readOnly;
        this._toggleReplaceBtn.setEnabled(this._isVisible && canReplace);
    };
    FindWidget.prototype._reveal = function () {
        var _this = this;
        if (!this._isVisible) {
            this._isVisible = true;
            var selection = this._codeEditor.getSelection();
            var isSelection = selection ? (selection.startLineNumber !== selection.endLineNumber || selection.startColumn !== selection.endColumn) : false;
            if (isSelection && this._codeEditor.getConfiguration().contribInfo.find.autoFindInSelection) {
                this._toggleSelectionFind.checked = true;
            }
            else {
                this._toggleSelectionFind.checked = false;
            }
            this._tryUpdateWidgetWidth();
            this._updateButtons();
            setTimeout(function () {
                addClass(_this._domNode, 'visible');
                _this._domNode.setAttribute('aria-hidden', 'false');
            }, 0);
            // validate query again as it's being dismissed when we hide the find widget.
            setTimeout(function () {
                _this._findInput.validate();
            }, 200);
            this._codeEditor.layoutOverlayWidget(this);
            var adjustEditorScrollTop = true;
            if (this._codeEditor.getConfiguration().contribInfo.find.seedSearchStringFromSelection && selection) {
                var editorCoords = getDomNodePagePosition(this._codeEditor.getDomNode());
                var startCoords = this._codeEditor.getScrolledVisiblePosition(selection.getStartPosition());
                var startLeft = editorCoords.left + startCoords.left;
                var startTop = startCoords.top;
                if (startTop < this._viewZone.heightInPx) {
                    if (selection.endLineNumber > selection.startLineNumber) {
                        adjustEditorScrollTop = false;
                    }
                    var leftOfFindWidget = getTopLeftOffset(this._domNode).left;
                    if (startLeft > leftOfFindWidget) {
                        adjustEditorScrollTop = false;
                    }
                    var endCoords = this._codeEditor.getScrolledVisiblePosition(selection.getEndPosition());
                    var endLeft = editorCoords.left + endCoords.left;
                    if (endLeft > leftOfFindWidget) {
                        adjustEditorScrollTop = false;
                    }
                }
            }
            this._showViewZone(adjustEditorScrollTop);
        }
    };
    FindWidget.prototype._hide = function (focusTheEditor) {
        var _this = this;
        if (this._isVisible) {
            this._isVisible = false;
            this._updateButtons();
            removeClass(this._domNode, 'visible');
            this._domNode.setAttribute('aria-hidden', 'true');
            this._findInput.clearMessage();
            if (focusTheEditor) {
                this._codeEditor.focus();
            }
            this._codeEditor.layoutOverlayWidget(this);
            this._codeEditor.changeViewZones(function (accessor) {
                if (_this._viewZoneId !== undefined) {
                    accessor.removeZone(_this._viewZoneId);
                    _this._viewZoneId = undefined;
                    _this._codeEditor.setScrollTop(_this._codeEditor.getScrollTop() - _this._viewZone.heightInPx);
                }
            });
        }
    };
    FindWidget.prototype._layoutViewZone = function () {
        var _this = this;
        if (!this._isVisible) {
            return;
        }
        if (this._viewZoneId !== undefined) {
            return;
        }
        this._codeEditor.changeViewZones(function (accessor) {
            if (_this._state.isReplaceRevealed) {
                _this._viewZone.heightInPx = FIND_REPLACE_AREA_HEIGHT;
            }
            else {
                _this._viewZone.heightInPx = FIND_INPUT_AREA_HEIGHT;
            }
            _this._viewZoneId = accessor.addZone(_this._viewZone);
            // scroll top adjust to make sure the editor doesn't scroll when adding viewzone at the beginning.
            _this._codeEditor.setScrollTop(_this._codeEditor.getScrollTop() + _this._viewZone.heightInPx);
        });
    };
    FindWidget.prototype._showViewZone = function (adjustScroll) {
        var _this = this;
        if (adjustScroll === void 0) { adjustScroll = true; }
        if (!this._isVisible) {
            return;
        }
        this._codeEditor.changeViewZones(function (accessor) {
            var scrollAdjustment = FIND_INPUT_AREA_HEIGHT;
            if (_this._viewZoneId !== undefined) {
                if (_this._state.isReplaceRevealed) {
                    _this._viewZone.heightInPx = FIND_REPLACE_AREA_HEIGHT;
                    scrollAdjustment = FIND_REPLACE_AREA_HEIGHT - FIND_INPUT_AREA_HEIGHT;
                }
                else {
                    _this._viewZone.heightInPx = FIND_INPUT_AREA_HEIGHT;
                    scrollAdjustment = FIND_INPUT_AREA_HEIGHT - FIND_REPLACE_AREA_HEIGHT;
                }
                accessor.removeZone(_this._viewZoneId);
            }
            else {
                _this._viewZone.heightInPx = FIND_INPUT_AREA_HEIGHT;
            }
            _this._viewZoneId = accessor.addZone(_this._viewZone);
            if (adjustScroll) {
                _this._codeEditor.setScrollTop(_this._codeEditor.getScrollTop() + scrollAdjustment);
            }
        });
    };
    FindWidget.prototype._applyTheme = function (theme) {
        var inputStyles = {
            inputActiveOptionBorder: theme.getColor(inputActiveOptionBorder),
            inputBackground: theme.getColor(inputBackground),
            inputForeground: theme.getColor(inputForeground),
            inputBorder: theme.getColor(inputBorder),
            inputValidationInfoBackground: theme.getColor(inputValidationInfoBackground),
            inputValidationInfoForeground: theme.getColor(inputValidationInfoForeground),
            inputValidationInfoBorder: theme.getColor(inputValidationInfoBorder),
            inputValidationWarningBackground: theme.getColor(inputValidationWarningBackground),
            inputValidationWarningForeground: theme.getColor(inputValidationWarningForeground),
            inputValidationWarningBorder: theme.getColor(inputValidationWarningBorder),
            inputValidationErrorBackground: theme.getColor(inputValidationErrorBackground),
            inputValidationErrorForeground: theme.getColor(inputValidationErrorForeground),
            inputValidationErrorBorder: theme.getColor(inputValidationErrorBorder)
        };
        this._findInput.style(inputStyles);
        this._replaceInputBox.style(inputStyles);
    };
    FindWidget.prototype._tryUpdateWidgetWidth = function () {
        if (!this._isVisible) {
            return;
        }
        var editorWidth = this._codeEditor.getConfiguration().layoutInfo.width;
        var minimapWidth = this._codeEditor.getConfiguration().layoutInfo.minimapWidth;
        var collapsedFindWidget = false;
        var reducedFindWidget = false;
        var narrowFindWidget = false;
        if (this._resized) {
            var widgetWidth = getTotalWidth(this._domNode);
            if (widgetWidth > FIND_WIDGET_INITIAL_WIDTH) {
                // as the widget is resized by users, we may need to change the max width of the widget as the editor width changes.
                this._domNode.style.maxWidth = editorWidth - 28 - minimapWidth - 15 + "px";
                this._replaceInputBox.inputElement.style.width = getTotalWidth(this._findInput.inputBox.inputElement) + "px";
                return;
            }
        }
        if (FIND_WIDGET_INITIAL_WIDTH + 28 + minimapWidth >= editorWidth) {
            reducedFindWidget = true;
        }
        if (FIND_WIDGET_INITIAL_WIDTH + 28 + minimapWidth - MAX_MATCHES_COUNT_WIDTH >= editorWidth) {
            narrowFindWidget = true;
        }
        if (FIND_WIDGET_INITIAL_WIDTH + 28 + minimapWidth - MAX_MATCHES_COUNT_WIDTH >= editorWidth + 50) {
            collapsedFindWidget = true;
        }
        toggleClass(this._domNode, 'collapsed-find-widget', collapsedFindWidget);
        toggleClass(this._domNode, 'narrow-find-widget', narrowFindWidget);
        toggleClass(this._domNode, 'reduced-find-widget', reducedFindWidget);
        if (!narrowFindWidget && !collapsedFindWidget) {
            // the minimal left offset of findwidget is 15px.
            this._domNode.style.maxWidth = editorWidth - 28 - minimapWidth - 15 + "px";
        }
        if (this._resized) {
            var findInputWidth = getTotalWidth(this._findInput.inputBox.inputElement);
            if (findInputWidth > 0) {
                this._replaceInputBox.inputElement.style.width = findInputWidth + "px";
            }
        }
    };
    // ----- Public
    FindWidget.prototype.focusFindInput = function () {
        this._findInput.select();
        // Edge browser requires focus() in addition to select()
        this._findInput.focus();
    };
    FindWidget.prototype.focusReplaceInput = function () {
        this._replaceInputBox.select();
        // Edge browser requires focus() in addition to select()
        this._replaceInputBox.focus();
    };
    FindWidget.prototype.highlightFindOptions = function () {
        this._findInput.highlightFindOptions();
    };
    FindWidget.prototype._updateSearchScope = function () {
        if (this._toggleSelectionFind.checked) {
            var selection = this._codeEditor.getSelection();
            if (selection.endColumn === 1 && selection.endLineNumber > selection.startLineNumber) {
                selection = selection.setEndPosition(selection.endLineNumber - 1, this._codeEditor.getModel().getLineMaxColumn(selection.endLineNumber - 1));
            }
            var currentMatch = this._state.currentMatch;
            if (selection.startLineNumber !== selection.endLineNumber) {
                if (!Range.equalsRange(selection, currentMatch)) {
                    // Reseed find scope
                    this._state.change({ searchScope: selection }, true);
                }
            }
        }
    };
    FindWidget.prototype._onFindInputMouseDown = function (e) {
        // on linux, middle key does pasting.
        if (e.middleButton) {
            e.stopPropagation();
        }
    };
    FindWidget.prototype._onFindInputKeyDown = function (e) {
        if (e.equals(3 /* Enter */)) {
            this._codeEditor.getAction(FIND_IDS.NextMatchFindAction).run().then(null, onUnexpectedError);
            e.preventDefault();
            return;
        }
        if (e.equals(1024 /* Shift */ | 3 /* Enter */)) {
            this._codeEditor.getAction(FIND_IDS.PreviousMatchFindAction).run().then(null, onUnexpectedError);
            e.preventDefault();
            return;
        }
        if (e.equals(2 /* Tab */)) {
            if (this._isReplaceVisible) {
                this._replaceInputBox.focus();
            }
            else {
                this._findInput.focusOnCaseSensitive();
            }
            e.preventDefault();
            return;
        }
        if (e.equals(2048 /* CtrlCmd */ | 18 /* DownArrow */)) {
            this._codeEditor.focus();
            e.preventDefault();
            return;
        }
    };
    FindWidget.prototype._onReplaceInputKeyDown = function (e) {
        if (e.equals(3 /* Enter */)) {
            this._controller.replace();
            e.preventDefault();
            return;
        }
        if (e.equals(2048 /* CtrlCmd */ | 3 /* Enter */)) {
            this._controller.replaceAll();
            e.preventDefault();
            return;
        }
        if (e.equals(2 /* Tab */)) {
            this._findInput.focusOnCaseSensitive();
            e.preventDefault();
            return;
        }
        if (e.equals(1024 /* Shift */ | 2 /* Tab */)) {
            this._findInput.focus();
            e.preventDefault();
            return;
        }
        if (e.equals(2048 /* CtrlCmd */ | 18 /* DownArrow */)) {
            this._codeEditor.focus();
            e.preventDefault();
            return;
        }
    };
    // ----- sash
    FindWidget.prototype.getHorizontalSashTop = function (_sash) {
        return 0;
    };
    FindWidget.prototype.getHorizontalSashLeft = function (_sash) {
        return 0;
    };
    FindWidget.prototype.getHorizontalSashWidth = function (_sash) {
        return 500;
    };
    // ----- initialization
    FindWidget.prototype._keybindingLabelFor = function (actionId) {
        var kb = this._keybindingService.lookupKeybinding(actionId);
        if (!kb) {
            return '';
        }
        return " (" + kb.getLabel() + ")";
    };
    FindWidget.prototype._buildFindPart = function () {
        var _this = this;
        // Find input
        this._findInput = this._register(new ContextScopedFindInput(null, this._contextViewProvider, {
            width: FIND_INPUT_AREA_WIDTH,
            label: NLS_FIND_INPUT_LABEL,
            placeholder: NLS_FIND_INPUT_PLACEHOLDER,
            appendCaseSensitiveLabel: this._keybindingLabelFor(FIND_IDS.ToggleCaseSensitiveCommand),
            appendWholeWordsLabel: this._keybindingLabelFor(FIND_IDS.ToggleWholeWordCommand),
            appendRegexLabel: this._keybindingLabelFor(FIND_IDS.ToggleRegexCommand),
            validation: function (value) {
                if (value.length === 0) {
                    return null;
                }
                if (!_this._findInput.getRegex()) {
                    return null;
                }
                try {
                    /* tslint:enable:no-unused-expression */
                    return null;
                }
                catch (e) {
                    return { content: e.message };
                }
            }
        }, this._contextKeyService, true));
        this._findInput.setRegex(!!this._state.isRegex);
        this._findInput.setCaseSensitive(!!this._state.matchCase);
        this._findInput.setWholeWords(!!this._state.wholeWord);
        this._register(this._findInput.onKeyDown(function (e) { return _this._onFindInputKeyDown(e); }));
        this._register(this._findInput.inputBox.onDidChange(function () {
            if (_this._ignoreChangeEvent) {
                return;
            }
            _this._state.change({ searchString: _this._findInput.getValue() }, true);
        }));
        this._register(this._findInput.onDidOptionChange(function () {
            _this._state.change({
                isRegex: _this._findInput.getRegex(),
                wholeWord: _this._findInput.getWholeWords(),
                matchCase: _this._findInput.getCaseSensitive()
            }, true);
        }));
        this._register(this._findInput.onCaseSensitiveKeyDown(function (e) {
            if (e.equals(1024 /* Shift */ | 2 /* Tab */)) {
                if (_this._isReplaceVisible) {
                    _this._replaceInputBox.focus();
                    e.preventDefault();
                }
            }
        }));
        if (isLinux) {
            this._register(this._findInput.onMouseDown(function (e) { return _this._onFindInputMouseDown(e); }));
        }
        this._matchesCount = document.createElement('div');
        this._matchesCount.className = 'matchesCount';
        this._updateMatchesCount();
        // Previous button
        this._prevBtn = this._register(new SimpleButton({
            label: NLS_PREVIOUS_MATCH_BTN_LABEL + this._keybindingLabelFor(FIND_IDS.PreviousMatchFindAction),
            className: 'previous',
            onTrigger: function () {
                _this._codeEditor.getAction(FIND_IDS.PreviousMatchFindAction).run().then(null, onUnexpectedError);
            }
        }));
        // Next button
        this._nextBtn = this._register(new SimpleButton({
            label: NLS_NEXT_MATCH_BTN_LABEL + this._keybindingLabelFor(FIND_IDS.NextMatchFindAction),
            className: 'next',
            onTrigger: function () {
                _this._codeEditor.getAction(FIND_IDS.NextMatchFindAction).run().then(null, onUnexpectedError);
            }
        }));
        var findPart = document.createElement('div');
        findPart.className = 'find-part';
        findPart.appendChild(this._findInput.domNode);
        findPart.appendChild(this._matchesCount);
        findPart.appendChild(this._prevBtn.domNode);
        findPart.appendChild(this._nextBtn.domNode);
        // Toggle selection button
        this._toggleSelectionFind = this._register(new SimpleCheckbox({
            parent: findPart,
            title: NLS_TOGGLE_SELECTION_FIND_TITLE + this._keybindingLabelFor(FIND_IDS.ToggleSearchScopeCommand),
            onChange: function () {
                if (_this._toggleSelectionFind.checked) {
                    var selection = _this._codeEditor.getSelection();
                    if (selection.endColumn === 1 && selection.endLineNumber > selection.startLineNumber) {
                        selection = selection.setEndPosition(selection.endLineNumber - 1, _this._codeEditor.getModel().getLineMaxColumn(selection.endLineNumber - 1));
                    }
                    if (!selection.isEmpty()) {
                        _this._state.change({ searchScope: selection }, true);
                    }
                }
                else {
                    _this._state.change({ searchScope: null }, true);
                }
            }
        }));
        // Close button
        this._closeBtn = this._register(new SimpleButton({
            label: NLS_CLOSE_BTN_LABEL + this._keybindingLabelFor(FIND_IDS.CloseFindWidgetCommand),
            className: 'close-fw',
            onTrigger: function () {
                _this._state.change({ isRevealed: false, searchScope: null }, false);
            },
            onKeyDown: function (e) {
                if (e.equals(2 /* Tab */)) {
                    if (_this._isReplaceVisible) {
                        if (_this._replaceBtn.isEnabled()) {
                            _this._replaceBtn.focus();
                        }
                        else {
                            _this._codeEditor.focus();
                        }
                        e.preventDefault();
                    }
                }
            }
        }));
        findPart.appendChild(this._closeBtn.domNode);
        return findPart;
    };
    FindWidget.prototype._buildReplacePart = function () {
        var _this = this;
        // Replace input
        var replaceInput = document.createElement('div');
        replaceInput.className = 'replace-input';
        replaceInput.style.width = REPLACE_INPUT_AREA_WIDTH + 'px';
        this._replaceInputBox = this._register(new ContextScopedHistoryInputBox(replaceInput, null, {
            ariaLabel: NLS_REPLACE_INPUT_LABEL,
            placeholder: NLS_REPLACE_INPUT_PLACEHOLDER,
            history: []
        }, this._contextKeyService));
        this._register(addStandardDisposableListener(this._replaceInputBox.inputElement, 'keydown', function (e) { return _this._onReplaceInputKeyDown(e); }));
        this._register(this._replaceInputBox.onDidChange(function () {
            _this._state.change({ replaceString: _this._replaceInputBox.value }, false);
        }));
        // Replace one button
        this._replaceBtn = this._register(new SimpleButton({
            label: NLS_REPLACE_BTN_LABEL + this._keybindingLabelFor(FIND_IDS.ReplaceOneAction),
            className: 'replace',
            onTrigger: function () {
                _this._controller.replace();
            },
            onKeyDown: function (e) {
                if (e.equals(1024 /* Shift */ | 2 /* Tab */)) {
                    _this._closeBtn.focus();
                    e.preventDefault();
                }
            }
        }));
        // Replace all button
        this._replaceAllBtn = this._register(new SimpleButton({
            label: NLS_REPLACE_ALL_BTN_LABEL + this._keybindingLabelFor(FIND_IDS.ReplaceAllAction),
            className: 'replace-all',
            onTrigger: function () {
                _this._controller.replaceAll();
            }
        }));
        var replacePart = document.createElement('div');
        replacePart.className = 'replace-part';
        replacePart.appendChild(replaceInput);
        replacePart.appendChild(this._replaceBtn.domNode);
        replacePart.appendChild(this._replaceAllBtn.domNode);
        return replacePart;
    };
    FindWidget.prototype._buildDomNode = function () {
        var _this = this;
        // Find part
        var findPart = this._buildFindPart();
        // Replace part
        var replacePart = this._buildReplacePart();
        // Toggle replace button
        this._toggleReplaceBtn = this._register(new SimpleButton({
            label: NLS_TOGGLE_REPLACE_MODE_BTN_LABEL,
            className: 'toggle left',
            onTrigger: function () {
                _this._state.change({ isReplaceRevealed: !_this._isReplaceVisible }, false);
                if (_this._isReplaceVisible) {
                    _this._replaceInputBox.width = _this._findInput.inputBox.width;
                }
                _this._showViewZone();
            }
        }));
        this._toggleReplaceBtn.toggleClass('expand', this._isReplaceVisible);
        this._toggleReplaceBtn.toggleClass('collapse', !this._isReplaceVisible);
        this._toggleReplaceBtn.setExpanded(this._isReplaceVisible);
        // Widget
        this._domNode = document.createElement('div');
        this._domNode.className = 'editor-widget find-widget';
        this._domNode.setAttribute('aria-hidden', 'true');
        // We need to set this explicitly, otherwise on IE11, the width inheritence of flex doesn't work.
        this._domNode.style.width = FIND_WIDGET_INITIAL_WIDTH + "px";
        this._domNode.appendChild(this._toggleReplaceBtn.domNode);
        this._domNode.appendChild(findPart);
        this._domNode.appendChild(replacePart);
        this._buildSash();
    };
    FindWidget.prototype._buildSash = function () {
        var _this = this;
        this._resizeSash = new Sash(this._domNode, this, { orientation: 0 /* VERTICAL */ });
        this._resized = false;
        var originalWidth = FIND_WIDGET_INITIAL_WIDTH;
        this._register(this._resizeSash.onDidStart(function () {
            originalWidth = getTotalWidth(_this._domNode);
        }));
        this._register(this._resizeSash.onDidChange(function (evt) {
            _this._resized = true;
            var width = originalWidth + evt.startX - evt.currentX;
            if (width < FIND_WIDGET_INITIAL_WIDTH) {
                // narrow down the find widget should be handled by CSS.
                return;
            }
            var inputBoxWidth = width - FIND_ALL_CONTROLS_WIDTH;
            var maxWidth = parseFloat(getComputedStyle(_this._domNode).maxWidth) || 0;
            if (width > maxWidth) {
                return;
            }
            _this._domNode.style.width = width + "px";
            if (_this._isReplaceVisible) {
                _this._replaceInputBox.width = inputBoxWidth;
            }
        }));
    };
    FindWidget.prototype.updateAccessibilitySupport = function () {
        var value = this._codeEditor.getConfiguration().accessibilitySupport;
        this._findInput.setFocusInputOnOptionClick(value !== 2 /* Enabled */);
    };
    FindWidget.ID = 'editor.contrib.findWidget';
    return FindWidget;
}(Widget$1));
var SimpleCheckbox = /** @class */ (function (_super) {
    __extends$1n(SimpleCheckbox, _super);
    function SimpleCheckbox(opts) {
        var _this = _super.call(this) || this;
        _this._opts = opts;
        _this._domNode = document.createElement('div');
        _this._domNode.className = 'monaco-checkbox';
        _this._domNode.title = _this._opts.title;
        _this._domNode.tabIndex = 0;
        _this._checkbox = document.createElement('input');
        _this._checkbox.type = 'checkbox';
        _this._checkbox.className = 'checkbox';
        _this._checkbox.id = 'checkbox-' + SimpleCheckbox._COUNTER++;
        _this._checkbox.tabIndex = -1;
        _this._label = document.createElement('label');
        _this._label.className = 'label';
        // Connect the label and the checkbox. Checkbox will get checked when the label receives a click.
        _this._label.htmlFor = _this._checkbox.id;
        _this._label.tabIndex = -1;
        _this._domNode.appendChild(_this._checkbox);
        _this._domNode.appendChild(_this._label);
        _this._opts.parent.appendChild(_this._domNode);
        _this.onchange(_this._checkbox, function () {
            _this._opts.onChange();
        });
        return _this;
    }
    Object.defineProperty(SimpleCheckbox.prototype, "domNode", {
        get: function () {
            return this._domNode;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleCheckbox.prototype, "checked", {
        get: function () {
            return this._checkbox.checked;
        },
        set: function (newValue) {
            this._checkbox.checked = newValue;
        },
        enumerable: true,
        configurable: true
    });
    SimpleCheckbox.prototype.enable = function () {
        this._checkbox.removeAttribute('disabled');
    };
    SimpleCheckbox.prototype.disable = function () {
        this._checkbox.disabled = true;
    };
    SimpleCheckbox.prototype.setEnabled = function (enabled) {
        if (enabled) {
            this.enable();
            this.domNode.tabIndex = 0;
        }
        else {
            this.disable();
            this.domNode.tabIndex = -1;
        }
    };
    SimpleCheckbox._COUNTER = 0;
    return SimpleCheckbox;
}(Widget$1));
var SimpleButton = /** @class */ (function (_super) {
    __extends$1n(SimpleButton, _super);
    function SimpleButton(opts) {
        var _this = _super.call(this) || this;
        _this._opts = opts;
        _this._domNode = document.createElement('div');
        _this._domNode.title = _this._opts.label;
        _this._domNode.tabIndex = 0;
        _this._domNode.className = 'button ' + _this._opts.className;
        _this._domNode.setAttribute('role', 'button');
        _this._domNode.setAttribute('aria-label', _this._opts.label);
        _this.onclick(_this._domNode, function (e) {
            _this._opts.onTrigger();
            e.preventDefault();
        });
        _this.onkeydown(_this._domNode, function (e) {
            if (e.equals(10 /* Space */) || e.equals(3 /* Enter */)) {
                _this._opts.onTrigger();
                e.preventDefault();
                return;
            }
            if (_this._opts.onKeyDown) {
                _this._opts.onKeyDown(e);
            }
        });
        return _this;
    }
    Object.defineProperty(SimpleButton.prototype, "domNode", {
        get: function () {
            return this._domNode;
        },
        enumerable: true,
        configurable: true
    });
    SimpleButton.prototype.isEnabled = function () {
        return (this._domNode.tabIndex >= 0);
    };
    SimpleButton.prototype.focus = function () {
        this._domNode.focus();
    };
    SimpleButton.prototype.setEnabled = function (enabled) {
        toggleClass(this._domNode, 'disabled', !enabled);
        this._domNode.setAttribute('aria-disabled', String(!enabled));
        this._domNode.tabIndex = enabled ? 0 : -1;
    };
    SimpleButton.prototype.setExpanded = function (expanded) {
        this._domNode.setAttribute('aria-expanded', String(!!expanded));
    };
    SimpleButton.prototype.toggleClass = function (className, shouldHaveIt) {
        toggleClass(this._domNode, className, shouldHaveIt);
    };
    return SimpleButton;
}(Widget$1));
// theming
registerThemingParticipant(function (theme, collector) {
    var addBackgroundColorRule = function (selector, color) {
        if (color) {
            collector.addRule(".monaco-editor " + selector + " { background-color: " + color + "; }");
        }
    };
    addBackgroundColorRule('.findMatch', theme.getColor(editorFindMatchHighlight));
    addBackgroundColorRule('.currentFindMatch', theme.getColor(editorFindMatch));
    addBackgroundColorRule('.findScope', theme.getColor(editorFindRangeHighlight));
    var widgetBackground = theme.getColor(editorWidgetBackground);
    addBackgroundColorRule('.find-widget', widgetBackground);
    var widgetShadowColor = theme.getColor(widgetShadow);
    if (widgetShadowColor) {
        collector.addRule(".monaco-editor .find-widget { box-shadow: 0 2px 8px " + widgetShadowColor + "; }");
    }
    var findMatchHighlightBorder = theme.getColor(editorFindMatchHighlightBorder);
    if (findMatchHighlightBorder) {
        collector.addRule(".monaco-editor .findMatch { border: 1px " + (theme.type === 'hc' ? 'dotted' : 'solid') + " " + findMatchHighlightBorder + "; box-sizing: border-box; }");
    }
    var findMatchBorder = theme.getColor(editorFindMatchBorder);
    if (findMatchBorder) {
        collector.addRule(".monaco-editor .currentFindMatch { border: 2px solid " + findMatchBorder + "; padding: 1px; box-sizing: border-box; }");
    }
    var findRangeHighlightBorder = theme.getColor(editorFindRangeHighlightBorder);
    if (findRangeHighlightBorder) {
        collector.addRule(".monaco-editor .findScope { border: 1px " + (theme.type === 'hc' ? 'dashed' : 'solid') + " " + findRangeHighlightBorder + "; }");
    }
    var hcBorder = theme.getColor(contrastBorder);
    if (hcBorder) {
        collector.addRule(".monaco-editor .find-widget { border: 2px solid " + hcBorder + "; }");
    }
    var error = theme.getColor(errorForeground);
    if (error) {
        collector.addRule(".monaco-editor .find-widget.no-results .matchesCount { color: " + error + "; }");
    }
    var resizeBorderBackground = theme.getColor(editorWidgetResizeBorder);
    if (resizeBorderBackground) {
        collector.addRule(".monaco-editor .find-widget .monaco-sash { background-color: " + resizeBorderBackground + "; width: 3px !important; margin-left: -4px;}");
    }
    else {
        var border = theme.getColor(editorWidgetBorder);
        if (border) {
            collector.addRule(".monaco-editor .find-widget .monaco-sash { background-color: " + border + "; width: 3px !important; margin-left: -4px;}");
        }
    }
    var inputActiveBorder = theme.getColor(inputActiveOptionBorder);
    if (inputActiveBorder) {
        collector.addRule(".monaco-editor .find-widget .monaco-checkbox .checkbox:checked + .label { border: 1px solid " + inputActiveBorder.toString() + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IClipboardService = createDecorator('clipboardService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IStorageService = createDecorator('storageService');
var NullStorageService = new /** @class */ (function () {
    function class_1() {
        this._serviceBrand = undefined;
        this.onDidChangeStorage = Event.None;
        this.onWillSaveState = Event.None;
    }
    class_1.prototype.get = function (key, scope, fallbackValue) {
        return fallbackValue;
    };
    class_1.prototype.getBoolean = function (key, scope, fallbackValue) {
        return fallbackValue;
    };
    class_1.prototype.getInteger = function (key, scope, fallbackValue) {
        return fallbackValue;
    };
    class_1.prototype.store = function (key, value, scope) {
        return Promise.resolve();
    };
    class_1.prototype.remove = function (key, scope) {
        return Promise.resolve();
    };
    return class_1;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1o = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$6 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$5 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var SEARCH_STRING_MAX_LENGTH = 524288;
function getSelectionSearchString(editor) {
    var selection = editor.getSelection();
    // if selection spans multiple lines, default search string to empty
    if (selection.startLineNumber === selection.endLineNumber) {
        if (selection.isEmpty()) {
            var wordAtPosition = editor.getModel().getWordAtPosition(selection.getStartPosition());
            if (wordAtPosition) {
                return wordAtPosition.word;
            }
        }
        else {
            if (editor.getModel().getValueLengthInRange(selection) < SEARCH_STRING_MAX_LENGTH) {
                return editor.getModel().getValueInRange(selection);
            }
        }
    }
    return null;
}
var CommonFindController = /** @class */ (function (_super) {
    __extends$1o(CommonFindController, _super);
    function CommonFindController(editor, contextKeyService, storageService, clipboardService) {
        var _this = _super.call(this) || this;
        _this._editor = editor;
        _this._findWidgetVisible = CONTEXT_FIND_WIDGET_VISIBLE.bindTo(contextKeyService);
        _this._contextKeyService = contextKeyService;
        _this._storageService = storageService;
        _this._clipboardService = clipboardService;
        _this._updateHistoryDelayer = new Delayer(500);
        _this._state = _this._register(new FindReplaceState());
        _this.loadQueryState();
        _this._register(_this._state.onFindReplaceStateChange(function (e) { return _this._onStateChanged(e); }));
        _this._model = null;
        _this._register(_this._editor.onDidChangeModel(function () {
            var shouldRestartFind = (_this._editor.getModel() && _this._state.isRevealed);
            _this.disposeModel();
            _this._state.change({
                searchScope: null,
                matchCase: _this._storageService.getBoolean('editor.matchCase', 1 /* WORKSPACE */, false),
                wholeWord: _this._storageService.getBoolean('editor.wholeWord', 1 /* WORKSPACE */, false),
                isRegex: _this._storageService.getBoolean('editor.isRegex', 1 /* WORKSPACE */, false)
            }, false);
            if (shouldRestartFind) {
                _this._start({
                    forceRevealReplace: false,
                    seedSearchStringFromSelection: false && _this._editor.getConfiguration().contribInfo.find.seedSearchStringFromSelection,
                    seedSearchStringFromGlobalClipboard: false,
                    shouldFocus: 0 /* NoFocusChange */,
                    shouldAnimate: false,
                    updateSearchScope: false
                });
            }
        }));
        return _this;
    }
    CommonFindController.get = function (editor) {
        return editor.getContribution(CommonFindController.ID);
    };
    CommonFindController.prototype.dispose = function () {
        this.disposeModel();
        _super.prototype.dispose.call(this);
    };
    CommonFindController.prototype.disposeModel = function () {
        if (this._model) {
            this._model.dispose();
            this._model = null;
        }
    };
    CommonFindController.prototype.getId = function () {
        return CommonFindController.ID;
    };
    CommonFindController.prototype._onStateChanged = function (e) {
        this.saveQueryState(e);
        if (e.isRevealed) {
            if (this._state.isRevealed) {
                this._findWidgetVisible.set(true);
            }
            else {
                this._findWidgetVisible.reset();
                this.disposeModel();
            }
        }
        if (e.searchString) {
            this.setGlobalBufferTerm(this._state.searchString);
        }
    };
    CommonFindController.prototype.saveQueryState = function (e) {
        if (e.isRegex) {
            this._storageService.store('editor.isRegex', this._state.actualIsRegex, 1 /* WORKSPACE */);
        }
        if (e.wholeWord) {
            this._storageService.store('editor.wholeWord', this._state.actualWholeWord, 1 /* WORKSPACE */);
        }
        if (e.matchCase) {
            this._storageService.store('editor.matchCase', this._state.actualMatchCase, 1 /* WORKSPACE */);
        }
    };
    CommonFindController.prototype.loadQueryState = function () {
        this._state.change({
            matchCase: this._storageService.getBoolean('editor.matchCase', 1 /* WORKSPACE */, this._state.matchCase),
            wholeWord: this._storageService.getBoolean('editor.wholeWord', 1 /* WORKSPACE */, this._state.wholeWord),
            isRegex: this._storageService.getBoolean('editor.isRegex', 1 /* WORKSPACE */, this._state.isRegex)
        }, false);
    };
    CommonFindController.prototype.isFindInputFocused = function () {
        return CONTEXT_FIND_INPUT_FOCUSED.getValue(this._contextKeyService);
    };
    CommonFindController.prototype.getState = function () {
        return this._state;
    };
    CommonFindController.prototype.closeFindWidget = function () {
        this._state.change({
            isRevealed: false,
            searchScope: null
        }, false);
        this._editor.focus();
    };
    CommonFindController.prototype.toggleCaseSensitive = function () {
        this._state.change({ matchCase: !this._state.matchCase }, false);
        if (!this._state.isRevealed) {
            this.highlightFindOptions();
        }
    };
    CommonFindController.prototype.toggleWholeWords = function () {
        this._state.change({ wholeWord: !this._state.wholeWord }, false);
        if (!this._state.isRevealed) {
            this.highlightFindOptions();
        }
    };
    CommonFindController.prototype.toggleRegex = function () {
        this._state.change({ isRegex: !this._state.isRegex }, false);
        if (!this._state.isRevealed) {
            this.highlightFindOptions();
        }
    };
    CommonFindController.prototype.toggleSearchScope = function () {
        if (this._state.searchScope) {
            this._state.change({ searchScope: null }, true);
        }
        else {
            var selection = this._editor.getSelection();
            if (selection.endColumn === 1 && selection.endLineNumber > selection.startLineNumber) {
                selection = selection.setEndPosition(selection.endLineNumber - 1, this._editor.getModel().getLineMaxColumn(selection.endLineNumber - 1));
            }
            if (!selection.isEmpty()) {
                this._state.change({ searchScope: selection }, true);
            }
        }
    };
    CommonFindController.prototype.setSearchString = function (searchString) {
        if (this._state.isRegex) {
            searchString = escapeRegExpCharacters(searchString);
        }
        this._state.change({ searchString: searchString }, false);
    };
    CommonFindController.prototype.highlightFindOptions = function () {
        // overwritten in subclass
    };
    CommonFindController.prototype._start = function (opts) {
        this.disposeModel();
        if (!this._editor.getModel()) {
            // cannot do anything with an editor that doesn't have a model...
            return;
        }
        var stateChanges = {
            isRevealed: true
        };
        if (opts.seedSearchStringFromSelection) {
            var selectionSearchString = getSelectionSearchString(this._editor);
            if (selectionSearchString) {
                if (this._state.isRegex) {
                    stateChanges.searchString = escapeRegExpCharacters(selectionSearchString);
                }
                else {
                    stateChanges.searchString = selectionSearchString;
                }
            }
        }
        if (!stateChanges.searchString && opts.seedSearchStringFromGlobalClipboard) {
            var selectionSearchString = this.getGlobalBufferTerm();
            if (selectionSearchString) {
                stateChanges.searchString = selectionSearchString;
            }
        }
        // Overwrite isReplaceRevealed
        if (opts.forceRevealReplace) {
            stateChanges.isReplaceRevealed = true;
        }
        else if (!this._findWidgetVisible.get()) {
            stateChanges.isReplaceRevealed = false;
        }
        if (opts.updateSearchScope) {
            var currentSelection = this._editor.getSelection();
            if (!currentSelection.isEmpty()) {
                stateChanges.searchScope = currentSelection;
            }
        }
        this._state.change(stateChanges, false);
        if (!this._model) {
            this._model = new FindModelBoundToEditorModel(this._editor, this._state);
        }
    };
    CommonFindController.prototype.start = function (opts) {
        this._start(opts);
    };
    CommonFindController.prototype.moveToNextMatch = function () {
        if (this._model) {
            this._model.moveToNextMatch();
            return true;
        }
        return false;
    };
    CommonFindController.prototype.moveToPrevMatch = function () {
        if (this._model) {
            this._model.moveToPrevMatch();
            return true;
        }
        return false;
    };
    CommonFindController.prototype.replace = function () {
        if (this._model) {
            this._model.replace();
            return true;
        }
        return false;
    };
    CommonFindController.prototype.replaceAll = function () {
        if (this._model) {
            this._model.replaceAll();
            return true;
        }
        return false;
    };
    CommonFindController.prototype.selectAllMatches = function () {
        if (this._model) {
            this._model.selectAllMatches();
            this._editor.focus();
            return true;
        }
        return false;
    };
    CommonFindController.prototype.getGlobalBufferTerm = function () {
        if (this._editor.getConfiguration().contribInfo.find.globalFindClipboard
            && this._clipboardService
            && !this._editor.getModel().isTooLargeForSyncing()) {
            return this._clipboardService.readFindText();
        }
        return '';
    };
    CommonFindController.prototype.setGlobalBufferTerm = function (text) {
        if (this._editor.getConfiguration().contribInfo.find.globalFindClipboard
            && this._clipboardService
            && !this._editor.getModel().isTooLargeForSyncing()) {
            this._clipboardService.writeFindText(text);
        }
    };
    CommonFindController.ID = 'editor.contrib.findController';
    CommonFindController = __decorate$6([
        __param$5(1, IContextKeyService),
        __param$5(2, IStorageService),
        __param$5(3, IClipboardService)
    ], CommonFindController);
    return CommonFindController;
}(Disposable));
var FindController = /** @class */ (function (_super) {
    __extends$1o(FindController, _super);
    function FindController(editor, _contextViewService, _contextKeyService, _keybindingService, _themeService, storageService, clipboardService) {
        var _this = _super.call(this, editor, _contextKeyService, storageService, clipboardService) || this;
        _this._contextViewService = _contextViewService;
        _this._keybindingService = _keybindingService;
        _this._themeService = _themeService;
        return _this;
    }
    FindController.prototype._start = function (opts) {
        if (!this._widget) {
            this._createFindWidget();
        }
        if (!this._widget.getPosition() && this._editor.getConfiguration().contribInfo.find.autoFindInSelection) {
            // not visible yet so we need to set search scope if `editor.find.autoFindInSelection` is `true`
            opts.updateSearchScope = true;
        }
        _super.prototype._start.call(this, opts);
        if (opts.shouldFocus === 2 /* FocusReplaceInput */) {
            this._widget.focusReplaceInput();
        }
        else if (opts.shouldFocus === 1 /* FocusFindInput */) {
            this._widget.focusFindInput();
        }
    };
    FindController.prototype.highlightFindOptions = function () {
        if (!this._widget) {
            this._createFindWidget();
        }
        if (this._state.isRevealed) {
            this._widget.highlightFindOptions();
        }
        else {
            this._findOptionsWidget.highlightFindOptions();
        }
    };
    FindController.prototype._createFindWidget = function () {
        this._widget = this._register(new FindWidget(this._editor, this, this._state, this._contextViewService, this._keybindingService, this._contextKeyService, this._themeService));
        this._findOptionsWidget = this._register(new FindOptionsWidget(this._editor, this._state, this._keybindingService, this._themeService));
    };
    FindController = __decorate$6([
        __param$5(1, IContextViewService),
        __param$5(2, IContextKeyService),
        __param$5(3, IKeybindingService),
        __param$5(4, IThemeService),
        __param$5(5, IStorageService),
        __param$5(6, optional(IClipboardService))
    ], FindController);
    return FindController;
}(CommonFindController));
var StartFindAction = /** @class */ (function (_super) {
    __extends$1o(StartFindAction, _super);
    function StartFindAction() {
        return _super.call(this, {
            id: FIND_IDS.StartFindAction,
            label: localize('startFindAction', "Find"),
            alias: 'Find',
            precondition: null,
            kbOpts: {
                kbExpr: null,
                primary: 2048 /* CtrlCmd */ | 36 /* KEY_F */,
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarEditMenu,
                group: '3_find',
                title: localize({ key: 'miFind', comment: ['&& denotes a mnemonic'] }, "&&Find"),
                order: 1
            }
        }) || this;
    }
    StartFindAction.prototype.run = function (accessor, editor) {
        var controller = CommonFindController.get(editor);
        if (controller) {
            controller.start({
                forceRevealReplace: false,
                seedSearchStringFromSelection: editor.getConfiguration().contribInfo.find.seedSearchStringFromSelection,
                seedSearchStringFromGlobalClipboard: editor.getConfiguration().contribInfo.find.globalFindClipboard,
                shouldFocus: 1 /* FocusFindInput */,
                shouldAnimate: true,
                updateSearchScope: false
            });
        }
    };
    return StartFindAction;
}(EditorAction));
var StartFindWithSelectionAction = /** @class */ (function (_super) {
    __extends$1o(StartFindWithSelectionAction, _super);
    function StartFindWithSelectionAction() {
        return _super.call(this, {
            id: FIND_IDS.StartFindWithSelection,
            label: localize('startFindWithSelectionAction', "Find With Selection"),
            alias: 'Find With Selection',
            precondition: null,
            kbOpts: {
                kbExpr: null,
                primary: 0,
                mac: {
                    primary: 2048 /* CtrlCmd */ | 35 /* KEY_E */,
                },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    StartFindWithSelectionAction.prototype.run = function (accessor, editor) {
        var controller = CommonFindController.get(editor);
        if (controller) {
            controller.start({
                forceRevealReplace: false,
                seedSearchStringFromSelection: true,
                seedSearchStringFromGlobalClipboard: false,
                shouldFocus: 1 /* FocusFindInput */,
                shouldAnimate: true,
                updateSearchScope: false
            });
            controller.setGlobalBufferTerm(controller.getState().searchString);
        }
    };
    return StartFindWithSelectionAction;
}(EditorAction));
var MatchFindAction = /** @class */ (function (_super) {
    __extends$1o(MatchFindAction, _super);
    function MatchFindAction() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MatchFindAction.prototype.run = function (accessor, editor) {
        var controller = CommonFindController.get(editor);
        if (controller && !this._run(controller)) {
            controller.start({
                forceRevealReplace: false,
                seedSearchStringFromSelection: (controller.getState().searchString.length === 0) && editor.getConfiguration().contribInfo.find.seedSearchStringFromSelection,
                seedSearchStringFromGlobalClipboard: true,
                shouldFocus: 0 /* NoFocusChange */,
                shouldAnimate: true,
                updateSearchScope: false
            });
            this._run(controller);
        }
    };
    return MatchFindAction;
}(EditorAction));
var NextMatchFindAction = /** @class */ (function (_super) {
    __extends$1o(NextMatchFindAction, _super);
    function NextMatchFindAction() {
        return _super.call(this, {
            id: FIND_IDS.NextMatchFindAction,
            label: localize('findNextMatchAction', "Find Next"),
            alias: 'Find Next',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 61 /* F3 */,
                mac: { primary: 2048 /* CtrlCmd */ | 37 /* KEY_G */, secondary: [61 /* F3 */] },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    NextMatchFindAction.prototype._run = function (controller) {
        return controller.moveToNextMatch();
    };
    return NextMatchFindAction;
}(MatchFindAction));
var PreviousMatchFindAction = /** @class */ (function (_super) {
    __extends$1o(PreviousMatchFindAction, _super);
    function PreviousMatchFindAction() {
        return _super.call(this, {
            id: FIND_IDS.PreviousMatchFindAction,
            label: localize('findPreviousMatchAction', "Find Previous"),
            alias: 'Find Previous',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 1024 /* Shift */ | 61 /* F3 */,
                mac: { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 37 /* KEY_G */, secondary: [1024 /* Shift */ | 61 /* F3 */] },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    PreviousMatchFindAction.prototype._run = function (controller) {
        return controller.moveToPrevMatch();
    };
    return PreviousMatchFindAction;
}(MatchFindAction));
var SelectionMatchFindAction = /** @class */ (function (_super) {
    __extends$1o(SelectionMatchFindAction, _super);
    function SelectionMatchFindAction() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SelectionMatchFindAction.prototype.run = function (accessor, editor) {
        var controller = CommonFindController.get(editor);
        if (!controller) {
            return;
        }
        var selectionSearchString = getSelectionSearchString(editor);
        if (selectionSearchString) {
            controller.setSearchString(selectionSearchString);
        }
        if (!this._run(controller)) {
            controller.start({
                forceRevealReplace: false,
                seedSearchStringFromSelection: editor.getConfiguration().contribInfo.find.seedSearchStringFromSelection,
                seedSearchStringFromGlobalClipboard: false,
                shouldFocus: 0 /* NoFocusChange */,
                shouldAnimate: true,
                updateSearchScope: false
            });
            this._run(controller);
        }
    };
    return SelectionMatchFindAction;
}(EditorAction));
var NextSelectionMatchFindAction = /** @class */ (function (_super) {
    __extends$1o(NextSelectionMatchFindAction, _super);
    function NextSelectionMatchFindAction() {
        return _super.call(this, {
            id: FIND_IDS.NextSelectionMatchFindAction,
            label: localize('nextSelectionMatchFindAction', "Find Next Selection"),
            alias: 'Find Next Selection',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 2048 /* CtrlCmd */ | 61 /* F3 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    NextSelectionMatchFindAction.prototype._run = function (controller) {
        return controller.moveToNextMatch();
    };
    return NextSelectionMatchFindAction;
}(SelectionMatchFindAction));
var PreviousSelectionMatchFindAction = /** @class */ (function (_super) {
    __extends$1o(PreviousSelectionMatchFindAction, _super);
    function PreviousSelectionMatchFindAction() {
        return _super.call(this, {
            id: FIND_IDS.PreviousSelectionMatchFindAction,
            label: localize('previousSelectionMatchFindAction', "Find Previous Selection"),
            alias: 'Find Previous Selection',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 61 /* F3 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    PreviousSelectionMatchFindAction.prototype._run = function (controller) {
        return controller.moveToPrevMatch();
    };
    return PreviousSelectionMatchFindAction;
}(SelectionMatchFindAction));
var StartFindReplaceAction = /** @class */ (function (_super) {
    __extends$1o(StartFindReplaceAction, _super);
    function StartFindReplaceAction() {
        return _super.call(this, {
            id: FIND_IDS.StartFindReplaceAction,
            label: localize('startReplace', "Replace"),
            alias: 'Replace',
            precondition: null,
            kbOpts: {
                kbExpr: null,
                primary: 2048 /* CtrlCmd */ | 38 /* KEY_H */,
                mac: { primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 36 /* KEY_F */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarEditMenu,
                group: '3_find',
                title: localize({ key: 'miReplace', comment: ['&& denotes a mnemonic'] }, "&&Replace"),
                order: 2
            }
        }) || this;
    }
    StartFindReplaceAction.prototype.run = function (accessor, editor) {
        if (editor.getConfiguration().readOnly) {
            return;
        }
        var controller = CommonFindController.get(editor);
        var currentSelection = editor.getSelection();
        var findInputFocused = controller.isFindInputFocused();
        // we only seed search string from selection when the current selection is single line and not empty,
        // + the find input is not focused
        var seedSearchStringFromSelection = !currentSelection.isEmpty()
            && currentSelection.startLineNumber === currentSelection.endLineNumber && editor.getConfiguration().contribInfo.find.seedSearchStringFromSelection
            && !findInputFocused;
        /*
         * if the existing search string in find widget is empty and we don't seed search string from selection, it means the Find Input is still empty, so we should focus the Find Input instead of Replace Input.

         * findInputFocused true -> seedSearchStringFromSelection false, FocusReplaceInput
         * findInputFocused false, seedSearchStringFromSelection true FocusReplaceInput
         * findInputFocused false seedSearchStringFromSelection false FocusFindInput
         */
        var shouldFocus = (findInputFocused || seedSearchStringFromSelection) ?
            2 /* FocusReplaceInput */ : 1 /* FocusFindInput */;
        if (controller) {
            controller.start({
                forceRevealReplace: true,
                seedSearchStringFromSelection: seedSearchStringFromSelection,
                seedSearchStringFromGlobalClipboard: editor.getConfiguration().contribInfo.find.seedSearchStringFromSelection,
                shouldFocus: shouldFocus,
                shouldAnimate: true,
                updateSearchScope: false
            });
        }
    };
    return StartFindReplaceAction;
}(EditorAction));
registerEditorContribution(FindController);
registerEditorAction(StartFindAction);
registerEditorAction(StartFindWithSelectionAction);
registerEditorAction(NextMatchFindAction);
registerEditorAction(PreviousMatchFindAction);
registerEditorAction(NextSelectionMatchFindAction);
registerEditorAction(PreviousSelectionMatchFindAction);
registerEditorAction(StartFindReplaceAction);
var FindCommand = EditorCommand.bindToContribution(CommonFindController.get);
registerEditorCommand(new FindCommand({
    id: FIND_IDS.CloseFindWidgetCommand,
    precondition: CONTEXT_FIND_WIDGET_VISIBLE,
    handler: function (x) { return x.closeFindWidget(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: 9 /* Escape */,
        secondary: [1024 /* Shift */ | 9 /* Escape */]
    }
}));
registerEditorCommand(new FindCommand({
    id: FIND_IDS.ToggleCaseSensitiveCommand,
    precondition: null,
    handler: function (x) { return x.toggleCaseSensitive(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: ToggleCaseSensitiveKeybinding.primary,
        mac: ToggleCaseSensitiveKeybinding.mac,
        win: ToggleCaseSensitiveKeybinding.win,
        linux: ToggleCaseSensitiveKeybinding.linux
    }
}));
registerEditorCommand(new FindCommand({
    id: FIND_IDS.ToggleWholeWordCommand,
    precondition: null,
    handler: function (x) { return x.toggleWholeWords(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: ToggleWholeWordKeybinding.primary,
        mac: ToggleWholeWordKeybinding.mac,
        win: ToggleWholeWordKeybinding.win,
        linux: ToggleWholeWordKeybinding.linux
    }
}));
registerEditorCommand(new FindCommand({
    id: FIND_IDS.ToggleRegexCommand,
    precondition: null,
    handler: function (x) { return x.toggleRegex(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: ToggleRegexKeybinding.primary,
        mac: ToggleRegexKeybinding.mac,
        win: ToggleRegexKeybinding.win,
        linux: ToggleRegexKeybinding.linux
    }
}));
registerEditorCommand(new FindCommand({
    id: FIND_IDS.ToggleSearchScopeCommand,
    precondition: null,
    handler: function (x) { return x.toggleSearchScope(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: ToggleSearchScopeKeybinding.primary,
        mac: ToggleSearchScopeKeybinding.mac,
        win: ToggleSearchScopeKeybinding.win,
        linux: ToggleSearchScopeKeybinding.linux
    }
}));
registerEditorCommand(new FindCommand({
    id: FIND_IDS.ReplaceOneAction,
    precondition: CONTEXT_FIND_WIDGET_VISIBLE,
    handler: function (x) { return x.replace(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 22 /* KEY_1 */
    }
}));
registerEditorCommand(new FindCommand({
    id: FIND_IDS.ReplaceAllAction,
    precondition: CONTEXT_FIND_WIDGET_VISIBLE,
    handler: function (x) { return x.replaceAll(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 3 /* Enter */
    }
}));
registerEditorCommand(new FindCommand({
    id: FIND_IDS.SelectAllMatchesAction,
    precondition: CONTEXT_FIND_WIDGET_VISIBLE,
    handler: function (x) { return x.selectAllMatches(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 5,
        kbExpr: EditorContextKeys.focus,
        primary: 512 /* Alt */ | 3 /* Enter */
    }
}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MAX_FOLDING_REGIONS = 0xFFFF;
var MAX_LINE_NUMBER = 0xFFFFFF;
var MASK_INDENT = 0xFF000000;
var FoldingRegions = /** @class */ (function () {
    function FoldingRegions(startIndexes, endIndexes, types) {
        if (startIndexes.length !== endIndexes.length || startIndexes.length > MAX_FOLDING_REGIONS) {
            throw new Error('invalid startIndexes or endIndexes size');
        }
        this._startIndexes = startIndexes;
        this._endIndexes = endIndexes;
        this._collapseStates = new Uint32Array(Math.ceil(startIndexes.length / 32));
        this._types = types;
    }
    FoldingRegions.prototype.ensureParentIndices = function () {
        var _this = this;
        if (!this._parentsComputed) {
            this._parentsComputed = true;
            var parentIndexes_1 = [];
            var isInsideLast = function (startLineNumber, endLineNumber) {
                var index = parentIndexes_1[parentIndexes_1.length - 1];
                return _this.getStartLineNumber(index) <= startLineNumber && _this.getEndLineNumber(index) >= endLineNumber;
            };
            for (var i = 0, len = this._startIndexes.length; i < len; i++) {
                var startLineNumber = this._startIndexes[i];
                var endLineNumber = this._endIndexes[i];
                if (startLineNumber > MAX_LINE_NUMBER || endLineNumber > MAX_LINE_NUMBER) {
                    throw new Error('startLineNumber or endLineNumber must not exceed ' + MAX_LINE_NUMBER);
                }
                while (parentIndexes_1.length > 0 && !isInsideLast(startLineNumber, endLineNumber)) {
                    parentIndexes_1.pop();
                }
                var parentIndex = parentIndexes_1.length > 0 ? parentIndexes_1[parentIndexes_1.length - 1] : -1;
                parentIndexes_1.push(i);
                this._startIndexes[i] = startLineNumber + ((parentIndex & 0xFF) << 24);
                this._endIndexes[i] = endLineNumber + ((parentIndex & 0xFF00) << 16);
            }
        }
    };
    Object.defineProperty(FoldingRegions.prototype, "length", {
        get: function () {
            return this._startIndexes.length;
        },
        enumerable: true,
        configurable: true
    });
    FoldingRegions.prototype.getStartLineNumber = function (index) {
        return this._startIndexes[index] & MAX_LINE_NUMBER;
    };
    FoldingRegions.prototype.getEndLineNumber = function (index) {
        return this._endIndexes[index] & MAX_LINE_NUMBER;
    };
    FoldingRegions.prototype.getType = function (index) {
        return this._types ? this._types[index] : void 0;
    };
    FoldingRegions.prototype.hasTypes = function () {
        return !!this._types;
    };
    FoldingRegions.prototype.isCollapsed = function (index) {
        var arrayIndex = (index / 32) | 0;
        var bit = index % 32;
        return (this._collapseStates[arrayIndex] & (1 << bit)) !== 0;
    };
    FoldingRegions.prototype.setCollapsed = function (index, newState) {
        var arrayIndex = (index / 32) | 0;
        var bit = index % 32;
        var value = this._collapseStates[arrayIndex];
        if (newState) {
            this._collapseStates[arrayIndex] = value | (1 << bit);
        }
        else {
            this._collapseStates[arrayIndex] = value & ~(1 << bit);
        }
    };
    FoldingRegions.prototype.toRegion = function (index) {
        return new FoldingRegion(this, index);
    };
    FoldingRegions.prototype.getParentIndex = function (index) {
        this.ensureParentIndices();
        var parent = ((this._startIndexes[index] & MASK_INDENT) >>> 24) + ((this._endIndexes[index] & MASK_INDENT) >>> 16);
        if (parent === MAX_FOLDING_REGIONS) {
            return -1;
        }
        return parent;
    };
    FoldingRegions.prototype.contains = function (index, line) {
        return this.getStartLineNumber(index) <= line && this.getEndLineNumber(index) >= line;
    };
    FoldingRegions.prototype.findIndex = function (line) {
        var low = 0, high = this._startIndexes.length;
        if (high === 0) {
            return -1; // no children
        }
        while (low < high) {
            var mid = Math.floor((low + high) / 2);
            if (line < this.getStartLineNumber(mid)) {
                high = mid;
            }
            else {
                low = mid + 1;
            }
        }
        return low - 1;
    };
    FoldingRegions.prototype.findRange = function (line) {
        var index = this.findIndex(line);
        if (index >= 0) {
            var endLineNumber = this.getEndLineNumber(index);
            if (endLineNumber >= line) {
                return index;
            }
            index = this.getParentIndex(index);
            while (index !== -1) {
                if (this.contains(index, line)) {
                    return index;
                }
                index = this.getParentIndex(index);
            }
        }
        return -1;
    };
    FoldingRegions.prototype.toString = function () {
        var res = [];
        for (var i = 0; i < this.length; i++) {
            res[i] = "[" + (this.isCollapsed(i) ? '+' : '-') + "] " + this.getStartLineNumber(i) + "/" + this.getEndLineNumber(i);
        }
        return res.join(', ');
    };
    return FoldingRegions;
}());
var FoldingRegion = /** @class */ (function () {
    function FoldingRegion(ranges, index) {
        this.ranges = ranges;
        this.index = index;
    }
    Object.defineProperty(FoldingRegion.prototype, "startLineNumber", {
        get: function () {
            return this.ranges.getStartLineNumber(this.index);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingRegion.prototype, "endLineNumber", {
        get: function () {
            return this.ranges.getEndLineNumber(this.index);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingRegion.prototype, "regionIndex", {
        get: function () {
            return this.index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingRegion.prototype, "parentIndex", {
        get: function () {
            return this.ranges.getParentIndex(this.index);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingRegion.prototype, "isCollapsed", {
        get: function () {
            return this.ranges.isCollapsed(this.index);
        },
        enumerable: true,
        configurable: true
    });
    FoldingRegion.prototype.containedBy = function (range) {
        return range.startLineNumber <= this.startLineNumber && range.endLineNumber >= this.endLineNumber;
    };
    FoldingRegion.prototype.containsLine = function (lineNumber) {
        return this.startLineNumber <= lineNumber && lineNumber <= this.endLineNumber;
    };
    return FoldingRegion;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var FoldingModel = /** @class */ (function () {
    function FoldingModel(textModel, decorationProvider) {
        this._updateEventEmitter = new Emitter();
        this._textModel = textModel;
        this._decorationProvider = decorationProvider;
        this._regions = new FoldingRegions(new Uint32Array(0), new Uint32Array(0));
        this._editorDecorationIds = [];
        this._isInitialized = false;
    }
    Object.defineProperty(FoldingModel.prototype, "regions", {
        get: function () { return this._regions; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingModel.prototype, "onDidChange", {
        get: function () { return this._updateEventEmitter.event; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingModel.prototype, "textModel", {
        get: function () { return this._textModel; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingModel.prototype, "isInitialized", {
        get: function () { return this._isInitialized; },
        enumerable: true,
        configurable: true
    });
    FoldingModel.prototype.toggleCollapseState = function (regions) {
        var _this = this;
        if (!regions.length) {
            return;
        }
        var processed = {};
        this._decorationProvider.changeDecorations(function (accessor) {
            for (var _i = 0, regions_1 = regions; _i < regions_1.length; _i++) {
                var region = regions_1[_i];
                var index = region.regionIndex;
                var editorDecorationId = _this._editorDecorationIds[index];
                if (editorDecorationId && !processed[editorDecorationId]) {
                    processed[editorDecorationId] = true;
                    var newCollapseState = !_this._regions.isCollapsed(index);
                    _this._regions.setCollapsed(index, newCollapseState);
                    accessor.changeDecorationOptions(editorDecorationId, _this._decorationProvider.getDecorationOption(newCollapseState));
                }
            }
        });
        this._updateEventEmitter.fire({ model: this, collapseStateChanged: regions });
    };
    FoldingModel.prototype.update = function (newRegions, blockedLineNumers) {
        var _this = this;
        if (blockedLineNumers === void 0) { blockedLineNumers = []; }
        var newEditorDecorations = [];
        var isBlocked = function (startLineNumber, endLineNumber) {
            for (var _i = 0, blockedLineNumers_1 = blockedLineNumers; _i < blockedLineNumers_1.length; _i++) {
                var blockedLineNumber = blockedLineNumers_1[_i];
                if (startLineNumber < blockedLineNumber && blockedLineNumber <= endLineNumber) { // first line is visible
                    return true;
                }
            }
            return false;
        };
        var initRange = function (index, isCollapsed) {
            var startLineNumber = newRegions.getStartLineNumber(index);
            if (isCollapsed && isBlocked(startLineNumber, newRegions.getEndLineNumber(index))) {
                isCollapsed = false;
            }
            newRegions.setCollapsed(index, isCollapsed);
            var maxColumn = _this._textModel.getLineMaxColumn(startLineNumber);
            var decorationRange = {
                startLineNumber: startLineNumber,
                startColumn: maxColumn,
                endLineNumber: startLineNumber,
                endColumn: maxColumn
            };
            newEditorDecorations.push({ range: decorationRange, options: _this._decorationProvider.getDecorationOption(isCollapsed) });
        };
        var i = 0;
        var nextCollapsed = function () {
            while (i < _this._regions.length) {
                var isCollapsed = _this._regions.isCollapsed(i);
                i++;
                if (isCollapsed) {
                    return i - 1;
                }
            }
            return -1;
        };
        var k = 0;
        var collapsedIndex = nextCollapsed();
        while (collapsedIndex !== -1 && k < newRegions.length) {
            // get the latest range
            var decRange = this._textModel.getDecorationRange(this._editorDecorationIds[collapsedIndex]);
            if (decRange) {
                var collapsedStartLineNumber = decRange.startLineNumber;
                if (this._textModel.getLineMaxColumn(collapsedStartLineNumber) === decRange.startColumn) { // test that the decoration is still at the end otherwise it got deleted
                    while (k < newRegions.length) {
                        var startLineNumber = newRegions.getStartLineNumber(k);
                        if (collapsedStartLineNumber >= startLineNumber) {
                            initRange(k, collapsedStartLineNumber === startLineNumber);
                            k++;
                        }
                        else {
                            break;
                        }
                    }
                }
            }
            collapsedIndex = nextCollapsed();
        }
        while (k < newRegions.length) {
            initRange(k, false);
            k++;
        }
        this._editorDecorationIds = this._decorationProvider.deltaDecorations(this._editorDecorationIds, newEditorDecorations);
        this._regions = newRegions;
        this._isInitialized = true;
        this._updateEventEmitter.fire({ model: this });
    };
    /**
     * Collapse state memento, for persistence only
     */
    FoldingModel.prototype.getMemento = function () {
        var collapsedRanges = [];
        for (var i = 0; i < this._regions.length; i++) {
            if (this._regions.isCollapsed(i)) {
                var range = this._textModel.getDecorationRange(this._editorDecorationIds[i]);
                if (range) {
                    var startLineNumber = range.startLineNumber;
                    var endLineNumber = range.endLineNumber + this._regions.getEndLineNumber(i) - this._regions.getStartLineNumber(i);
                    collapsedRanges.push({ startLineNumber: startLineNumber, endLineNumber: endLineNumber });
                }
            }
        }
        if (collapsedRanges.length > 0) {
            return collapsedRanges;
        }
        return void 0;
    };
    /**
     * Apply persisted state, for persistence only
     */
    FoldingModel.prototype.applyMemento = function (state) {
        if (!Array.isArray(state)) {
            return;
        }
        var toToogle = [];
        for (var _i = 0, state_1 = state; _i < state_1.length; _i++) {
            var range = state_1[_i];
            var region = this.getRegionAtLine(range.startLineNumber);
            if (region && !region.isCollapsed) {
                toToogle.push(region);
            }
        }
        this.toggleCollapseState(toToogle);
    };
    FoldingModel.prototype.dispose = function () {
        this._decorationProvider.deltaDecorations(this._editorDecorationIds, []);
    };
    FoldingModel.prototype.getAllRegionsAtLine = function (lineNumber, filter) {
        var result = [];
        if (this._regions) {
            var index = this._regions.findRange(lineNumber);
            var level = 1;
            while (index >= 0) {
                var current = this._regions.toRegion(index);
                if (!filter || filter(current, level)) {
                    result.push(current);
                }
                level++;
                index = current.parentIndex;
            }
        }
        return result;
    };
    FoldingModel.prototype.getRegionAtLine = function (lineNumber) {
        if (this._regions) {
            var index = this._regions.findRange(lineNumber);
            if (index >= 0) {
                return this._regions.toRegion(index);
            }
        }
        return null;
    };
    FoldingModel.prototype.getRegionsInside = function (region, filter) {
        var result = [];
        var index = region ? region.regionIndex + 1 : 0;
        var endLineNumber = region ? region.endLineNumber : Number.MAX_VALUE;
        if (filter && filter.length === 2) {
            var levelStack = [];
            for (var i = index, len = this._regions.length; i < len; i++) {
                var current = this._regions.toRegion(i);
                if (this._regions.getStartLineNumber(i) < endLineNumber) {
                    while (levelStack.length > 0 && !current.containedBy(levelStack[levelStack.length - 1])) {
                        levelStack.pop();
                    }
                    levelStack.push(current);
                    if (filter(current, levelStack.length)) {
                        result.push(current);
                    }
                }
                else {
                    break;
                }
            }
        }
        else {
            for (var i = index, len = this._regions.length; i < len; i++) {
                var current = this._regions.toRegion(i);
                if (this._regions.getStartLineNumber(i) < endLineNumber) {
                    if (!filter || filter(current)) {
                        result.push(current);
                    }
                }
                else {
                    break;
                }
            }
        }
        return result;
    };
    return FoldingModel;
}());
/**
 * Collapse or expand the regions at the given locations including all children.
 * @param doCollapse Wheter to collase or expand
 * @param levels The number of levels. Use 1 to only impact the regions at the location, use Number.MAX_VALUE for all levels.
 * @param lineNumbers the location of the regions to collapse or expand, or if not set, all regions in the model.
 */
function setCollapseStateLevelsDown(foldingModel, doCollapse, levels, lineNumbers) {
    if (levels === void 0) { levels = Number.MAX_VALUE; }
    var toToggle = [];
    if (lineNumbers && lineNumbers.length > 0) {
        for (var _i = 0, lineNumbers_1 = lineNumbers; _i < lineNumbers_1.length; _i++) {
            var lineNumber = lineNumbers_1[_i];
            var region = foldingModel.getRegionAtLine(lineNumber);
            if (region) {
                if (region.isCollapsed !== doCollapse) {
                    toToggle.push(region);
                }
                if (levels > 1) {
                    var regionsInside = foldingModel.getRegionsInside(region, function (r, level) { return r.isCollapsed !== doCollapse && level < levels; });
                    toToggle.push.apply(toToggle, regionsInside);
                }
            }
        }
    }
    else {
        var regionsInside = foldingModel.getRegionsInside(null, function (r, level) { return r.isCollapsed !== doCollapse && level < levels; });
        toToggle.push.apply(toToggle, regionsInside);
    }
    foldingModel.toggleCollapseState(toToggle);
}
/**
 * Collapse or expand the regions at the given locations including all parents.
 * @param doCollapse Wheter to collase or expand
 * @param levels The number of levels. Use 1 to only impact the regions at the location, use Number.MAX_VALUE for all levels.
 * @param lineNumbers the location of the regions to collapse or expand, or if not set, all regions in the model.
 */
function setCollapseStateLevelsUp(foldingModel, doCollapse, levels, lineNumbers) {
    var toToggle = [];
    for (var _i = 0, lineNumbers_2 = lineNumbers; _i < lineNumbers_2.length; _i++) {
        var lineNumber = lineNumbers_2[_i];
        var regions = foldingModel.getAllRegionsAtLine(lineNumber, function (region, level) { return region.isCollapsed !== doCollapse && level <= levels; });
        toToggle.push.apply(toToggle, regions);
    }
    foldingModel.toggleCollapseState(toToggle);
}
/**
 * Folds or unfolds all regions that have a given level, except if they contain one of the blocked lines.
 * @param foldLevel level. Level == 1 is the top level
 * @param doCollapse Wheter to collase or expand
* @param blockedLineNumbers
*/
function setCollapseStateAtLevel(foldingModel, foldLevel, doCollapse, blockedLineNumbers) {
    var filter = function (region, level) { return level === foldLevel && region.isCollapsed !== doCollapse && !blockedLineNumbers.some(function (line) { return region.containsLine(line); }); };
    var toToggle = foldingModel.getRegionsInside(null, filter);
    foldingModel.toggleCollapseState(toToggle);
}
/**
 * Folds all regions for which the lines start with a given regex
 * @param foldingModel the folding model
 */
function setCollapseStateForMatchingLines(foldingModel, regExp, doCollapse) {
    var editorModel = foldingModel.textModel;
    var regions = foldingModel.regions;
    var toToggle = [];
    for (var i = regions.length - 1; i >= 0; i--) {
        if (doCollapse !== regions.isCollapsed(i)) {
            var startLineNumber = regions.getStartLineNumber(i);
            if (regExp.test(editorModel.getLineContent(startLineNumber))) {
                toToggle.push(regions.toRegion(i));
            }
        }
    }
    foldingModel.toggleCollapseState(toToggle);
}
/**
 * Folds all regions of the given type
 * @param foldingModel the folding model
 */
function setCollapseStateForType(foldingModel, type, doCollapse) {
    var regions = foldingModel.regions;
    var toToggle = [];
    for (var i = regions.length - 1; i >= 0; i--) {
        if (doCollapse !== regions.isCollapsed(i) && type === regions.getType(i)) {
            toToggle.push(regions.toRegion(i));
        }
    }
    foldingModel.toggleCollapseState(toToggle);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var FoldingDecorationProvider = /** @class */ (function () {
    function FoldingDecorationProvider(editor) {
        this.editor = editor;
        this.autoHideFoldingControls = true;
    }
    FoldingDecorationProvider.prototype.getDecorationOption = function (isCollapsed) {
        if (isCollapsed) {
            return FoldingDecorationProvider.COLLAPSED_VISUAL_DECORATION;
        }
        else if (this.autoHideFoldingControls) {
            return FoldingDecorationProvider.EXPANDED_AUTO_HIDE_VISUAL_DECORATION;
        }
        else {
            return FoldingDecorationProvider.EXPANDED_VISUAL_DECORATION;
        }
    };
    FoldingDecorationProvider.prototype.deltaDecorations = function (oldDecorations, newDecorations) {
        return this.editor.deltaDecorations(oldDecorations, newDecorations);
    };
    FoldingDecorationProvider.prototype.changeDecorations = function (callback) {
        return this.editor.changeDecorations(callback);
    };
    FoldingDecorationProvider.COLLAPSED_VISUAL_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        afterContentClassName: 'inline-folded',
        linesDecorationsClassName: 'folding collapsed'
    });
    FoldingDecorationProvider.EXPANDED_AUTO_HIDE_VISUAL_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        linesDecorationsClassName: 'folding'
    });
    FoldingDecorationProvider.EXPANDED_VISUAL_DECORATION = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        linesDecorationsClassName: 'folding alwaysShowFoldIcons'
    });
    return FoldingDecorationProvider;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var HiddenRangeModel = /** @class */ (function () {
    function HiddenRangeModel(model) {
        var _this = this;
        this._updateEventEmitter = new Emitter();
        this._foldingModel = model;
        this._foldingModelListener = model.onDidChange(function (_) { return _this.updateHiddenRanges(); });
        this._hiddenRanges = [];
        if (model.regions.length) {
            this.updateHiddenRanges();
        }
    }
    Object.defineProperty(HiddenRangeModel.prototype, "onDidChange", {
        get: function () { return this._updateEventEmitter.event; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HiddenRangeModel.prototype, "hiddenRanges", {
        get: function () { return this._hiddenRanges; },
        enumerable: true,
        configurable: true
    });
    HiddenRangeModel.prototype.updateHiddenRanges = function () {
        var updateHiddenAreas = false;
        var newHiddenAreas = [];
        var i = 0; // index into hidden
        var k = 0;
        var lastCollapsedStart = Number.MAX_VALUE;
        var lastCollapsedEnd = -1;
        var ranges = this._foldingModel.regions;
        for (; i < ranges.length; i++) {
            if (!ranges.isCollapsed(i)) {
                continue;
            }
            var startLineNumber = ranges.getStartLineNumber(i) + 1; // the first line is not hidden
            var endLineNumber = ranges.getEndLineNumber(i);
            if (lastCollapsedStart <= startLineNumber && endLineNumber <= lastCollapsedEnd) {
                // ignore ranges contained in collapsed regions
                continue;
            }
            if (!updateHiddenAreas && k < this._hiddenRanges.length && this._hiddenRanges[k].startLineNumber === startLineNumber && this._hiddenRanges[k].endLineNumber === endLineNumber) {
                // reuse the old ranges
                newHiddenAreas.push(this._hiddenRanges[k]);
                k++;
            }
            else {
                updateHiddenAreas = true;
                newHiddenAreas.push(new Range(startLineNumber, 1, endLineNumber, 1));
            }
            lastCollapsedStart = startLineNumber;
            lastCollapsedEnd = endLineNumber;
        }
        if (updateHiddenAreas || k < this._hiddenRanges.length) {
            this.applyHiddenRanges(newHiddenAreas);
        }
    };
    HiddenRangeModel.prototype.applyMemento = function (state) {
        if (!Array.isArray(state) || state.length === 0) {
            return false;
        }
        var hiddenRanges = [];
        for (var _i = 0, state_1 = state; _i < state_1.length; _i++) {
            var r = state_1[_i];
            if (!r.startLineNumber || !r.endLineNumber) {
                return false;
            }
            hiddenRanges.push(new Range(r.startLineNumber + 1, 1, r.endLineNumber, 1));
        }
        this.applyHiddenRanges(hiddenRanges);
        return true;
    };
    /**
     * Collapse state memento, for persistence only, only used if folding model is not yet initialized
     */
    HiddenRangeModel.prototype.getMemento = function () {
        return this._hiddenRanges.map(function (r) { return ({ startLineNumber: r.startLineNumber - 1, endLineNumber: r.endLineNumber }); });
    };
    HiddenRangeModel.prototype.applyHiddenRanges = function (newHiddenAreas) {
        this._hiddenRanges = newHiddenAreas;
        this._updateEventEmitter.fire(newHiddenAreas);
    };
    HiddenRangeModel.prototype.hasRanges = function () {
        return this._hiddenRanges.length > 0;
    };
    HiddenRangeModel.prototype.isHidden = function (line) {
        return findRange(this._hiddenRanges, line) !== null;
    };
    HiddenRangeModel.prototype.adjustSelections = function (selections) {
        var _this = this;
        var hasChanges = false;
        var editorModel = this._foldingModel.textModel;
        var lastRange = null;
        var adjustLine = function (line) {
            if (!lastRange || !isInside(line, lastRange)) {
                lastRange = findRange(_this._hiddenRanges, line);
            }
            if (lastRange) {
                return lastRange.startLineNumber - 1;
            }
            return null;
        };
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            var adjustedStartLine = adjustLine(selection.startLineNumber);
            if (adjustedStartLine) {
                selection = selection.setStartPosition(adjustedStartLine, editorModel.getLineMaxColumn(adjustedStartLine));
                hasChanges = true;
            }
            var adjustedEndLine = adjustLine(selection.endLineNumber);
            if (adjustedEndLine) {
                selection = selection.setEndPosition(adjustedEndLine, editorModel.getLineMaxColumn(adjustedEndLine));
                hasChanges = true;
            }
            selections[i] = selection;
        }
        return hasChanges;
    };
    HiddenRangeModel.prototype.dispose = function () {
        if (this.hiddenRanges.length > 0) {
            this._hiddenRanges = [];
            this._updateEventEmitter.fire(this._hiddenRanges);
        }
        if (this._foldingModelListener) {
            this._foldingModelListener.dispose();
            this._foldingModelListener = null;
        }
    };
    return HiddenRangeModel;
}());
function isInside(line, range$$1) {
    return line >= range$$1.startLineNumber && line <= range$$1.endLineNumber;
}
function findRange(ranges, line) {
    var i = findFirstInSorted(ranges, function (r) { return line < r.startLineNumber; }) - 1;
    if (i >= 0 && ranges[i].endLineNumber >= line) {
        return ranges[i];
    }
    return null;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MAX_FOLDING_REGIONS_FOR_INDENT_LIMIT = 5000;
var ID_INDENT_PROVIDER = 'indent';
var IndentRangeProvider = /** @class */ (function () {
    function IndentRangeProvider(editorModel) {
        this.editorModel = editorModel;
        this.id = ID_INDENT_PROVIDER;
    }
    IndentRangeProvider.prototype.dispose = function () {
    };
    IndentRangeProvider.prototype.compute = function (cancelationToken) {
        var foldingRules = LanguageConfigurationRegistry.getFoldingRules(this.editorModel.getLanguageIdentifier().id);
        var offSide = foldingRules && !!foldingRules.offSide;
        var markers = foldingRules && foldingRules.markers;
        return Promise.resolve(computeRanges(this.editorModel, offSide, markers));
    };
    return IndentRangeProvider;
}());
// public only for testing
var RangesCollector = /** @class */ (function () {
    function RangesCollector(foldingRangesLimit) {
        this._startIndexes = [];
        this._endIndexes = [];
        this._indentOccurrences = [];
        this._length = 0;
        this._foldingRangesLimit = foldingRangesLimit;
    }
    RangesCollector.prototype.insertFirst = function (startLineNumber, endLineNumber, indent) {
        if (startLineNumber > MAX_LINE_NUMBER || endLineNumber > MAX_LINE_NUMBER) {
            return;
        }
        var index = this._length;
        this._startIndexes[index] = startLineNumber;
        this._endIndexes[index] = endLineNumber;
        this._length++;
        if (indent < 1000) {
            this._indentOccurrences[indent] = (this._indentOccurrences[indent] || 0) + 1;
        }
    };
    RangesCollector.prototype.toIndentRanges = function (model) {
        if (this._length <= this._foldingRangesLimit) {
            // reverse and create arrays of the exact length
            var startIndexes = new Uint32Array(this._length);
            var endIndexes = new Uint32Array(this._length);
            for (var i = this._length - 1, k = 0; i >= 0; i--, k++) {
                startIndexes[k] = this._startIndexes[i];
                endIndexes[k] = this._endIndexes[i];
            }
            return new FoldingRegions(startIndexes, endIndexes);
        }
        else {
            var entries = 0;
            var maxIndent = this._indentOccurrences.length;
            for (var i = 0; i < this._indentOccurrences.length; i++) {
                var n = this._indentOccurrences[i];
                if (n) {
                    if (n + entries > this._foldingRangesLimit) {
                        maxIndent = i;
                        break;
                    }
                    entries += n;
                }
            }
            var tabSize = model.getOptions().tabSize;
            // reverse and create arrays of the exact length
            var startIndexes = new Uint32Array(this._foldingRangesLimit);
            var endIndexes = new Uint32Array(this._foldingRangesLimit);
            for (var i = this._length - 1, k = 0; i >= 0; i--) {
                var startIndex = this._startIndexes[i];
                var lineContent = model.getLineContent(startIndex);
                var indent = TextModel.computeIndentLevel(lineContent, tabSize);
                if (indent < maxIndent || (indent === maxIndent && entries++ < this._foldingRangesLimit)) {
                    startIndexes[k] = startIndex;
                    endIndexes[k] = this._endIndexes[i];
                    k++;
                }
            }
            return new FoldingRegions(startIndexes, endIndexes);
        }
    };
    return RangesCollector;
}());
function computeRanges(model, offSide, markers, foldingRangesLimit) {
    if (foldingRangesLimit === void 0) { foldingRangesLimit = MAX_FOLDING_REGIONS_FOR_INDENT_LIMIT; }
    var tabSize = model.getOptions().tabSize;
    var result = new RangesCollector(foldingRangesLimit);
    var pattern = void 0;
    if (markers) {
        pattern = new RegExp("(" + markers.start.source + ")|(?:" + markers.end.source + ")");
    }
    var previousRegions = [];
    previousRegions.push({ indent: -1, line: model.getLineCount() + 1, marker: false }); // sentinel, to make sure there's at least one entry
    for (var line = model.getLineCount(); line > 0; line--) {
        var lineContent = model.getLineContent(line);
        var indent = TextModel.computeIndentLevel(lineContent, tabSize);
        var previous = previousRegions[previousRegions.length - 1];
        if (indent === -1) {
            if (offSide && !previous.marker) {
                // for offSide languages, empty lines are associated to the next block
                previous.line = line;
            }
            continue; // only whitespace
        }
        var m = void 0;
        if (pattern && (m = lineContent.match(pattern))) {
            // folding pattern match
            if (m[1]) { // start pattern match
                // discard all regions until the folding pattern
                var i = previousRegions.length - 1;
                while (i > 0 && !previousRegions[i].marker) {
                    i--;
                }
                if (i > 0) {
                    previousRegions.length = i + 1;
                    previous = previousRegions[i];
                    // new folding range from pattern, includes the end line
                    result.insertFirst(line, previous.line, indent);
                    previous.marker = false;
                    previous.indent = indent;
                    previous.line = line;
                    continue;
                }
            }
            else { // end pattern match
                previousRegions.push({ indent: -2, line: line, marker: true });
                continue;
            }
        }
        if (previous.indent > indent) {
            // discard all regions with larger indent
            do {
                previousRegions.pop();
                previous = previousRegions[previousRegions.length - 1];
            } while (previous.indent > indent);
            // new folding range
            var endLineNumber = previous.line - 1;
            if (endLineNumber - line >= 1) { // needs at east size 1
                result.insertFirst(line, endLineNumber, indent);
            }
        }
        if (previous.indent === indent) {
            previous.line = line;
        }
        else { // previous.indent < indent
            // new region with a bigger indent
            previousRegions.push({ indent: indent, line: line, marker: false });
        }
    }
    return result.toIndentRanges(model);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MAX_FOLDING_REGIONS$1 = 5000;
var foldingContext = {};
var ID_SYNTAX_PROVIDER = 'syntax';
var SyntaxRangeProvider = /** @class */ (function () {
    function SyntaxRangeProvider(editorModel, providers, limit) {
        if (limit === void 0) { limit = MAX_FOLDING_REGIONS$1; }
        this.editorModel = editorModel;
        this.providers = providers;
        this.limit = limit;
        this.id = ID_SYNTAX_PROVIDER;
    }
    SyntaxRangeProvider.prototype.compute = function (cancellationToken) {
        var _this = this;
        return collectSyntaxRanges(this.providers, this.editorModel, cancellationToken).then(function (ranges) {
            if (ranges) {
                var res = sanitizeRanges(ranges, _this.limit);
                return res;
            }
            return null;
        });
    };
    SyntaxRangeProvider.prototype.dispose = function () {
    };
    return SyntaxRangeProvider;
}());
function collectSyntaxRanges(providers, model, cancellationToken) {
    var rangeData = null;
    var promises = providers.map(function (provider, i) {
        return Promise.resolve(provider.provideFoldingRanges(model, foldingContext, cancellationToken)).then(function (ranges) {
            if (cancellationToken.isCancellationRequested) {
                return;
            }
            if (Array.isArray(ranges)) {
                if (!Array.isArray(rangeData)) {
                    rangeData = [];
                }
                var nLines = model.getLineCount();
                for (var _i = 0, ranges_1 = ranges; _i < ranges_1.length; _i++) {
                    var r = ranges_1[_i];
                    if (r.start > 0 && r.end > r.start && r.end <= nLines) {
                        rangeData.push({ start: r.start, end: r.end, rank: i, kind: r.kind });
                    }
                }
            }
        }, onUnexpectedExternalError);
    });
    return Promise.all(promises).then(function (_) {
        return rangeData;
    });
}
var RangesCollector$1 = /** @class */ (function () {
    function RangesCollector(foldingRangesLimit) {
        this._startIndexes = [];
        this._endIndexes = [];
        this._nestingLevels = [];
        this._nestingLevelCounts = [];
        this._types = [];
        this._length = 0;
        this._foldingRangesLimit = foldingRangesLimit;
    }
    RangesCollector.prototype.add = function (startLineNumber, endLineNumber, type, nestingLevel) {
        if (startLineNumber > MAX_LINE_NUMBER || endLineNumber > MAX_LINE_NUMBER) {
            return;
        }
        var index = this._length;
        this._startIndexes[index] = startLineNumber;
        this._endIndexes[index] = endLineNumber;
        this._nestingLevels[index] = nestingLevel;
        this._types[index] = type;
        this._length++;
        if (nestingLevel < 30) {
            this._nestingLevelCounts[nestingLevel] = (this._nestingLevelCounts[nestingLevel] || 0) + 1;
        }
    };
    RangesCollector.prototype.toIndentRanges = function () {
        if (this._length <= this._foldingRangesLimit) {
            var startIndexes = new Uint32Array(this._length);
            var endIndexes = new Uint32Array(this._length);
            for (var i = 0; i < this._length; i++) {
                startIndexes[i] = this._startIndexes[i];
                endIndexes[i] = this._endIndexes[i];
            }
            return new FoldingRegions(startIndexes, endIndexes, this._types);
        }
        else {
            var entries = 0;
            var maxLevel = this._nestingLevelCounts.length;
            for (var i = 0; i < this._nestingLevelCounts.length; i++) {
                var n = this._nestingLevelCounts[i];
                if (n) {
                    if (n + entries > this._foldingRangesLimit) {
                        maxLevel = i;
                        break;
                    }
                    entries += n;
                }
            }
            var startIndexes = new Uint32Array(this._foldingRangesLimit);
            var endIndexes = new Uint32Array(this._foldingRangesLimit);
            var types = [];
            for (var i = 0, k = 0; i < this._length; i++) {
                var level = this._nestingLevels[i];
                if (level < maxLevel || (level === maxLevel && entries++ < this._foldingRangesLimit)) {
                    startIndexes[k] = this._startIndexes[i];
                    endIndexes[k] = this._endIndexes[i];
                    types[k] = this._types[i];
                    k++;
                }
            }
            return new FoldingRegions(startIndexes, endIndexes, types);
        }
    };
    return RangesCollector;
}());
function sanitizeRanges(rangeData, limit) {
    var sorted = rangeData.sort(function (d1, d2) {
        var diff = d1.start - d2.start;
        if (diff === 0) {
            diff = d1.rank - d2.rank;
        }
        return diff;
    });
    var collector = new RangesCollector$1(limit);
    var top = void 0;
    var previous = [];
    for (var _i = 0, sorted_1 = sorted; _i < sorted_1.length; _i++) {
        var entry = sorted_1[_i];
        if (!top) {
            top = entry;
            collector.add(entry.start, entry.end, entry.kind && entry.kind.value, previous.length);
        }
        else {
            if (entry.start > top.start) {
                if (entry.end <= top.end) {
                    previous.push(top);
                    top = entry;
                    collector.add(entry.start, entry.end, entry.kind && entry.kind.value, previous.length);
                }
                else {
                    if (entry.start > top.end) {
                        do {
                            top = previous.pop();
                        } while (top && entry.start > top.end);
                        if (top) {
                            previous.push(top);
                        }
                        top = entry;
                    }
                    collector.add(entry.start, entry.end, entry.kind && entry.kind.value, previous.length);
                }
            }
        }
    }
    return collector.toIndentRanges();
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ID_INIT_PROVIDER = 'init';
var InitializingRangeProvider = /** @class */ (function () {
    function InitializingRangeProvider(editorModel, initialRanges, onTimeout, timeoutTime) {
        this.editorModel = editorModel;
        this.id = ID_INIT_PROVIDER;
        if (initialRanges.length) {
            var toDecorationRange = function (range) {
                return {
                    range: {
                        startLineNumber: range.startLineNumber,
                        startColumn: 0,
                        endLineNumber: range.endLineNumber,
                        endColumn: editorModel.getLineLength(range.endLineNumber)
                    },
                    options: {
                        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */
                    }
                };
            };
            this.decorationIds = editorModel.deltaDecorations([], initialRanges.map(toDecorationRange));
            this.timeout = setTimeout(onTimeout, timeoutTime);
        }
    }
    InitializingRangeProvider.prototype.dispose = function () {
        if (this.decorationIds) {
            this.editorModel.deltaDecorations(this.decorationIds, []);
            this.decorationIds = void 0;
        }
        if (typeof this.timeout === 'number') {
            clearTimeout(this.timeout);
            this.timeout = void 0;
        }
    };
    InitializingRangeProvider.prototype.compute = function (cancelationToken) {
        var foldingRangeData = [];
        if (this.decorationIds) {
            for (var _i = 0, _a = this.decorationIds; _i < _a.length; _i++) {
                var id = _a[_i];
                var range = this.editorModel.getDecorationRange(id);
                if (range) {
                    foldingRangeData.push({ start: range.startLineNumber, end: range.endLineNumber, rank: 1 });
                }
            }
        }
        return Promise.resolve(sanitizeRanges(foldingRangeData, Number.MAX_VALUE));
    };
    return InitializingRangeProvider;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1p = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ID = 'editor.contrib.folding';
var FoldingController = /** @class */ (function () {
    function FoldingController(editor) {
        var _this = this;
        this.editor = editor;
        this._isEnabled = this.editor.getConfiguration().contribInfo.folding;
        this._autoHideFoldingControls = this.editor.getConfiguration().contribInfo.showFoldingControls === 'mouseover';
        this._useFoldingProviders = this.editor.getConfiguration().contribInfo.foldingStrategy !== 'indentation';
        this.globalToDispose = [];
        this.localToDispose = [];
        this.foldingDecorationProvider = new FoldingDecorationProvider(editor);
        this.foldingDecorationProvider.autoHideFoldingControls = this._autoHideFoldingControls;
        this.globalToDispose.push(this.editor.onDidChangeModel(function () { return _this.onModelChanged(); }));
        this.globalToDispose.push(FoldingRangeProviderRegistry.onDidChange(function () { return _this.onFoldingStrategyChanged(); }));
        this.globalToDispose.push(this.editor.onDidChangeConfiguration(function (e) {
            if (e.contribInfo) {
                var oldIsEnabled = _this._isEnabled;
                _this._isEnabled = _this.editor.getConfiguration().contribInfo.folding;
                if (oldIsEnabled !== _this._isEnabled) {
                    _this.onModelChanged();
                }
                var oldShowFoldingControls = _this._autoHideFoldingControls;
                _this._autoHideFoldingControls = _this.editor.getConfiguration().contribInfo.showFoldingControls === 'mouseover';
                if (oldShowFoldingControls !== _this._autoHideFoldingControls) {
                    _this.foldingDecorationProvider.autoHideFoldingControls = _this._autoHideFoldingControls;
                    _this.onModelContentChanged();
                }
                var oldUseFoldingProviders = _this._useFoldingProviders;
                _this._useFoldingProviders = _this.editor.getConfiguration().contribInfo.foldingStrategy !== 'indentation';
                if (oldUseFoldingProviders !== _this._useFoldingProviders) {
                    _this.onFoldingStrategyChanged();
                }
            }
        }));
        this.globalToDispose.push({ dispose: function () { return dispose(_this.localToDispose); } });
        this.onModelChanged();
    }
    FoldingController.get = function (editor) {
        return editor.getContribution(ID);
    };
    FoldingController.prototype.getId = function () {
        return ID;
    };
    FoldingController.prototype.dispose = function () {
        this.globalToDispose = dispose(this.globalToDispose);
    };
    /**
     * Store view state.
     */
    FoldingController.prototype.saveViewState = function () {
        var model = this.editor.getModel();
        if (!model || !this._isEnabled || model.isTooLargeForTokenization()) {
            return {};
        }
        if (this.foldingModel) { // disposed ?
            var collapsedRegions = this.foldingModel.isInitialized ? this.foldingModel.getMemento() : this.hiddenRangeModel.getMemento();
            var provider = this.rangeProvider ? this.rangeProvider.id : void 0;
            return { collapsedRegions: collapsedRegions, lineCount: model.getLineCount(), provider: provider };
        }
        return void 0;
    };
    /**
     * Restore view state.
     */
    FoldingController.prototype.restoreViewState = function (state) {
        var model = this.editor.getModel();
        if (!model || !this._isEnabled || model.isTooLargeForTokenization() || !this.hiddenRangeModel) {
            return;
        }
        if (!state || !state.collapsedRegions || state.lineCount !== model.getLineCount()) {
            return;
        }
        if (state.provider === ID_SYNTAX_PROVIDER || state.provider === ID_INIT_PROVIDER) {
            this.foldingStateMemento = state;
        }
        var collapsedRegions = state.collapsedRegions;
        // set the hidden ranges right away, before waiting for the folding model.
        if (this.hiddenRangeModel.applyMemento(collapsedRegions)) {
            var foldingModel = this.getFoldingModel();
            if (foldingModel) {
                foldingModel.then(function (foldingModel) {
                    if (foldingModel) {
                        foldingModel.applyMemento(collapsedRegions);
                    }
                }).then(undefined, onUnexpectedError);
            }
        }
    };
    FoldingController.prototype.onModelChanged = function () {
        var _this = this;
        this.localToDispose = dispose(this.localToDispose);
        var model = this.editor.getModel();
        if (!this._isEnabled || !model || model.isTooLargeForTokenization()) {
            // huge files get no view model, so they cannot support hidden areas
            return;
        }
        this.foldingModel = new FoldingModel(model, this.foldingDecorationProvider);
        this.localToDispose.push(this.foldingModel);
        this.hiddenRangeModel = new HiddenRangeModel(this.foldingModel);
        this.localToDispose.push(this.hiddenRangeModel);
        this.localToDispose.push(this.hiddenRangeModel.onDidChange(function (hr) { return _this.onHiddenRangesChanges(hr); }));
        this.updateScheduler = new Delayer(200);
        this.cursorChangedScheduler = new RunOnceScheduler(function () { return _this.revealCursor(); }, 200);
        this.localToDispose.push(this.cursorChangedScheduler);
        this.localToDispose.push(this.editor.onDidChangeModelLanguageConfiguration(function () { return _this.onModelContentChanged(); })); // covers model language changes as well
        this.localToDispose.push(this.editor.onDidChangeModelContent(function () { return _this.onModelContentChanged(); }));
        this.localToDispose.push(this.editor.onDidChangeCursorPosition(function () { return _this.onCursorPositionChanged(); }));
        this.localToDispose.push(this.editor.onMouseDown(function (e) { return _this.onEditorMouseDown(e); }));
        this.localToDispose.push(this.editor.onMouseUp(function (e) { return _this.onEditorMouseUp(e); }));
        this.localToDispose.push({
            dispose: function () {
                if (_this.foldingRegionPromise) {
                    _this.foldingRegionPromise.cancel();
                    _this.foldingRegionPromise = null;
                }
                if (_this.updateScheduler) {
                    _this.updateScheduler.cancel();
                }
                _this.updateScheduler = null;
                _this.foldingModel = null;
                _this.foldingModelPromise = null;
                _this.hiddenRangeModel = null;
                _this.cursorChangedScheduler = null;
                _this.foldingStateMemento = null;
                if (_this.rangeProvider) {
                    _this.rangeProvider.dispose();
                }
                _this.rangeProvider = null;
            }
        });
        this.onModelContentChanged();
    };
    FoldingController.prototype.onFoldingStrategyChanged = function () {
        if (this.rangeProvider) {
            this.rangeProvider.dispose();
        }
        this.rangeProvider = null;
        this.onModelContentChanged();
    };
    FoldingController.prototype.getRangeProvider = function (editorModel) {
        var _this = this;
        if (this.rangeProvider) {
            return this.rangeProvider;
        }
        this.rangeProvider = new IndentRangeProvider(editorModel); // fallback
        if (this._useFoldingProviders && this.foldingModel) {
            var foldingProviders = FoldingRangeProviderRegistry.ordered(this.foldingModel.textModel);
            if (foldingProviders.length === 0 && this.foldingStateMemento && this.foldingStateMemento.collapsedRegions) {
                var rangeProvider = this.rangeProvider = new InitializingRangeProvider(editorModel, this.foldingStateMemento.collapsedRegions, function () {
                    // if after 30 the InitializingRangeProvider is still not replaced, force a refresh
                    _this.foldingStateMemento = null;
                    _this.onFoldingStrategyChanged();
                }, 30000);
                return rangeProvider; // keep memento in case there are still no foldingProviders on the next request.
            }
            else if (foldingProviders.length > 0) {
                this.rangeProvider = new SyntaxRangeProvider(editorModel, foldingProviders);
            }
        }
        this.foldingStateMemento = null;
        return this.rangeProvider;
    };
    FoldingController.prototype.getFoldingModel = function () {
        return this.foldingModelPromise;
    };
    FoldingController.prototype.onModelContentChanged = function () {
        var _this = this;
        if (this.updateScheduler) {
            if (this.foldingRegionPromise) {
                this.foldingRegionPromise.cancel();
                this.foldingRegionPromise = null;
            }
            this.foldingModelPromise = this.updateScheduler.trigger(function () {
                var foldingModel = _this.foldingModel;
                if (!foldingModel) { // null if editor has been disposed, or folding turned off
                    return null;
                }
                var foldingRegionPromise = _this.foldingRegionPromise = createCancelablePromise(function (token) { return _this.getRangeProvider(foldingModel.textModel).compute(token); });
                return foldingRegionPromise.then(function (foldingRanges) {
                    if (foldingRanges && foldingRegionPromise === _this.foldingRegionPromise) { // new request or cancelled in the meantime?
                        // some cursors might have moved into hidden regions, make sure they are in expanded regions
                        var selections = _this.editor.getSelections();
                        var selectionLineNumbers = selections ? selections.map(function (s) { return s.startLineNumber; }) : [];
                        foldingModel.update(foldingRanges, selectionLineNumbers);
                    }
                    return foldingModel;
                });
            });
        }
    };
    FoldingController.prototype.onHiddenRangesChanges = function (hiddenRanges) {
        if (this.hiddenRangeModel && hiddenRanges.length) {
            var selections = this.editor.getSelections();
            if (selections) {
                if (this.hiddenRangeModel.adjustSelections(selections)) {
                    this.editor.setSelections(selections);
                }
            }
        }
        this.editor.setHiddenAreas(hiddenRanges);
    };
    FoldingController.prototype.onCursorPositionChanged = function () {
        if (this.hiddenRangeModel && this.hiddenRangeModel.hasRanges()) {
            this.cursorChangedScheduler.schedule();
        }
    };
    FoldingController.prototype.revealCursor = function () {
        var _this = this;
        var foldingModel = this.getFoldingModel();
        if (!foldingModel) {
            return;
        }
        foldingModel.then(function (foldingModel) {
            if (foldingModel) {
                var selections = _this.editor.getSelections();
                if (selections && selections.length > 0) {
                    var toToggle = [];
                    var _loop_1 = function (selection) {
                        var lineNumber = selection.selectionStartLineNumber;
                        if (_this.hiddenRangeModel && _this.hiddenRangeModel.isHidden(lineNumber)) {
                            toToggle.push.apply(toToggle, foldingModel.getAllRegionsAtLine(lineNumber, function (r) { return r.isCollapsed && lineNumber > r.startLineNumber; }));
                        }
                    };
                    for (var _i = 0, selections_1 = selections; _i < selections_1.length; _i++) {
                        var selection = selections_1[_i];
                        _loop_1(selection);
                    }
                    if (toToggle.length) {
                        foldingModel.toggleCollapseState(toToggle);
                        _this.reveal(selections[0].getPosition());
                    }
                }
            }
        }).then(undefined, onUnexpectedError);
    };
    FoldingController.prototype.onEditorMouseDown = function (e) {
        this.mouseDownInfo = null;
        if (!this.hiddenRangeModel || !e.target || !e.target.range) {
            return;
        }
        if (!e.event.leftButton && !e.event.middleButton) {
            return;
        }
        var range = e.target.range;
        var iconClicked = false;
        switch (e.target.type) {
            case 4 /* GUTTER_LINE_DECORATIONS */:
                var data = e.target.detail;
                var offsetLeftInGutter = e.target.element.offsetLeft;
                var gutterOffsetX = data.offsetX - offsetLeftInGutter;
                // const gutterOffsetX = data.offsetX - data.glyphMarginWidth - data.lineNumbersWidth - data.glyphMarginLeft;
                // TODO@joao TODO@alex TODO@martin this is such that we don't collide with dirty diff
                if (gutterOffsetX < 5) { // the whitespace between the border and the real folding icon border is 5px
                    return;
                }
                iconClicked = true;
                break;
            case 7 /* CONTENT_EMPTY */: {
                if (this.hiddenRangeModel.hasRanges()) {
                    var data_1 = e.target.detail;
                    if (!data_1.isAfterLines) {
                        break;
                    }
                }
                return;
            }
            case 6 /* CONTENT_TEXT */: {
                if (this.hiddenRangeModel.hasRanges()) {
                    var model = this.editor.getModel();
                    if (model && range.startColumn === model.getLineMaxColumn(range.startLineNumber)) {
                        break;
                    }
                }
                return;
            }
            default:
                return;
        }
        this.mouseDownInfo = { lineNumber: range.startLineNumber, iconClicked: iconClicked };
    };
    FoldingController.prototype.onEditorMouseUp = function (e) {
        var _this = this;
        var foldingModel = this.getFoldingModel();
        if (!foldingModel || !this.mouseDownInfo || !e.target) {
            return;
        }
        var lineNumber = this.mouseDownInfo.lineNumber;
        var iconClicked = this.mouseDownInfo.iconClicked;
        var range = e.target.range;
        if (!range || range.startLineNumber !== lineNumber) {
            return;
        }
        if (iconClicked) {
            if (e.target.type !== 4 /* GUTTER_LINE_DECORATIONS */) {
                return;
            }
        }
        else {
            var model = this.editor.getModel();
            if (!model || range.startColumn !== model.getLineMaxColumn(lineNumber)) {
                return;
            }
        }
        foldingModel.then(function (foldingModel) {
            if (foldingModel) {
                var region = foldingModel.getRegionAtLine(lineNumber);
                if (region && region.startLineNumber === lineNumber) {
                    var isCollapsed_1 = region.isCollapsed;
                    if (iconClicked || isCollapsed_1) {
                        var toToggle = [region];
                        if (e.event.middleButton || e.event.shiftKey) {
                            toToggle.push.apply(toToggle, foldingModel.getRegionsInside(region, function (r) { return r.isCollapsed === isCollapsed_1; }));
                        }
                        foldingModel.toggleCollapseState(toToggle);
                        _this.reveal({ lineNumber: lineNumber, column: 1 });
                    }
                }
            }
        }).then(undefined, onUnexpectedError);
    };
    FoldingController.prototype.reveal = function (position) {
        this.editor.revealPositionInCenterIfOutsideViewport(position, 0 /* Smooth */);
    };
    return FoldingController;
}());
var FoldingAction = /** @class */ (function (_super) {
    __extends$1p(FoldingAction, _super);
    function FoldingAction() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FoldingAction.prototype.runEditorCommand = function (accessor, editor, args) {
        var _this = this;
        var foldingController = FoldingController.get(editor);
        if (!foldingController) {
            return;
        }
        var foldingModelPromise = foldingController.getFoldingModel();
        if (foldingModelPromise) {
            this.reportTelemetry(accessor, editor);
            return foldingModelPromise.then(function (foldingModel) {
                if (foldingModel) {
                    _this.invoke(foldingController, foldingModel, editor, args);
                    var selection = editor.getSelection();
                    if (selection) {
                        foldingController.reveal(selection.getStartPosition());
                    }
                }
            });
        }
    };
    FoldingAction.prototype.getSelectedLines = function (editor) {
        var selections = editor.getSelections();
        return selections ? selections.map(function (s) { return s.startLineNumber; }) : [];
    };
    FoldingAction.prototype.getLineNumbers = function (args, editor) {
        if (args && args.selectionLines) {
            return args.selectionLines.map(function (l) { return l + 1; }); // to 0-bases line numbers
        }
        return this.getSelectedLines(editor);
    };
    FoldingAction.prototype.run = function (_accessor, _editor) {
    };
    return FoldingAction;
}(EditorAction));
function foldingArgumentsConstraint(args) {
    if (!isUndefined(args)) {
        if (!isObject(args)) {
            return false;
        }
        var foldingArgs = args;
        if (!isUndefined(foldingArgs.levels) && !isNumber(foldingArgs.levels)) {
            return false;
        }
        if (!isUndefined(foldingArgs.direction) && !isString(foldingArgs.direction)) {
            return false;
        }
        if (!isUndefined(foldingArgs.selectionLines) && (!isArray(foldingArgs.selectionLines) || !foldingArgs.selectionLines.every(isNumber))) {
            return false;
        }
    }
    return true;
}
var UnfoldAction = /** @class */ (function (_super) {
    __extends$1p(UnfoldAction, _super);
    function UnfoldAction() {
        return _super.call(this, {
            id: 'editor.unfold',
            label: localize('unfoldAction.label', "Unfold"),
            alias: 'Unfold',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 89 /* US_CLOSE_SQUARE_BRACKET */,
                mac: {
                    primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 89 /* US_CLOSE_SQUARE_BRACKET */
                },
                weight: 100 /* EditorContrib */
            },
            description: {
                description: 'Unfold the content in the editor',
                args: [
                    {
                        name: 'Unfold editor argument',
                        description: "Property-value pairs that can be passed through this argument:\n\t\t\t\t\t\t* 'levels': Number of levels to unfold. If not set, defaults to 1.\n\t\t\t\t\t\t* 'direction': If 'up', unfold given number of levels up otherwise unfolds down.\n\t\t\t\t\t\t* 'selectionLines': The start lines (0-based) of the editor selections to apply the unfold action to. If not set, the active selection(s) will be used.\n\t\t\t\t\t\t",
                        constraint: foldingArgumentsConstraint
                    }
                ]
            }
        }) || this;
    }
    UnfoldAction.prototype.invoke = function (_foldingController, foldingModel, editor, args) {
        var levels = args && args.levels || 1;
        var lineNumbers = this.getLineNumbers(args, editor);
        if (args && args.direction === 'up') {
            setCollapseStateLevelsUp(foldingModel, false, levels, lineNumbers);
        }
        else {
            setCollapseStateLevelsDown(foldingModel, false, levels, lineNumbers);
        }
    };
    return UnfoldAction;
}(FoldingAction));
var UnFoldRecursivelyAction = /** @class */ (function (_super) {
    __extends$1p(UnFoldRecursivelyAction, _super);
    function UnFoldRecursivelyAction() {
        return _super.call(this, {
            id: 'editor.unfoldRecursively',
            label: localize('unFoldRecursivelyAction.label', "Unfold Recursively"),
            alias: 'Unfold Recursively',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 89 /* US_CLOSE_SQUARE_BRACKET */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    UnFoldRecursivelyAction.prototype.invoke = function (_foldingController, foldingModel, editor, _args) {
        setCollapseStateLevelsDown(foldingModel, false, Number.MAX_VALUE, this.getSelectedLines(editor));
    };
    return UnFoldRecursivelyAction;
}(FoldingAction));
var FoldAction = /** @class */ (function (_super) {
    __extends$1p(FoldAction, _super);
    function FoldAction() {
        return _super.call(this, {
            id: 'editor.fold',
            label: localize('foldAction.label', "Fold"),
            alias: 'Fold',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 87 /* US_OPEN_SQUARE_BRACKET */,
                mac: {
                    primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 87 /* US_OPEN_SQUARE_BRACKET */
                },
                weight: 100 /* EditorContrib */
            },
            description: {
                description: 'Fold the content in the editor',
                args: [
                    {
                        name: 'Fold editor argument',
                        description: "Property-value pairs that can be passed through this argument:\n\t\t\t\t\t\t\t* 'levels': Number of levels to fold. Defaults to 1.\n\t\t\t\t\t\t\t* 'direction': If 'up', folds given number of levels up otherwise folds down.\n\t\t\t\t\t\t\t* 'selectionLines': The start lines (0-based) of the editor selections to apply the fold action to. If not set, the active selection(s) will be used.\n\t\t\t\t\t\t",
                        constraint: foldingArgumentsConstraint
                    }
                ]
            }
        }) || this;
    }
    FoldAction.prototype.invoke = function (_foldingController, foldingModel, editor, args) {
        var levels = args && args.levels || 1;
        var lineNumbers = this.getLineNumbers(args, editor);
        if (args && args.direction === 'up') {
            setCollapseStateLevelsUp(foldingModel, true, levels, lineNumbers);
        }
        else {
            setCollapseStateLevelsDown(foldingModel, true, levels, lineNumbers);
        }
    };
    return FoldAction;
}(FoldingAction));
var FoldRecursivelyAction = /** @class */ (function (_super) {
    __extends$1p(FoldRecursivelyAction, _super);
    function FoldRecursivelyAction() {
        return _super.call(this, {
            id: 'editor.foldRecursively',
            label: localize('foldRecursivelyAction.label', "Fold Recursively"),
            alias: 'Fold Recursively',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 87 /* US_OPEN_SQUARE_BRACKET */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    FoldRecursivelyAction.prototype.invoke = function (_foldingController, foldingModel, editor) {
        var selectedLines = this.getSelectedLines(editor);
        setCollapseStateLevelsDown(foldingModel, true, Number.MAX_VALUE, selectedLines);
    };
    return FoldRecursivelyAction;
}(FoldingAction));
var FoldAllBlockCommentsAction = /** @class */ (function (_super) {
    __extends$1p(FoldAllBlockCommentsAction, _super);
    function FoldAllBlockCommentsAction() {
        return _super.call(this, {
            id: 'editor.foldAllBlockComments',
            label: localize('foldAllBlockComments.label', "Fold All Block Comments"),
            alias: 'Fold All Block Comments',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 85 /* US_SLASH */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    FoldAllBlockCommentsAction.prototype.invoke = function (_foldingController, foldingModel, editor) {
        if (foldingModel.regions.hasTypes()) {
            setCollapseStateForType(foldingModel, FoldingRangeKind.Comment.value, true);
        }
        else {
            var editorModel = editor.getModel();
            if (!editorModel) {
                return;
            }
            var comments = LanguageConfigurationRegistry.getComments(editorModel.getLanguageIdentifier().id);
            if (comments && comments.blockCommentStartToken) {
                var regExp = new RegExp('^\\s*' + escapeRegExpCharacters(comments.blockCommentStartToken));
                setCollapseStateForMatchingLines(foldingModel, regExp, true);
            }
        }
    };
    return FoldAllBlockCommentsAction;
}(FoldingAction));
var FoldAllRegionsAction = /** @class */ (function (_super) {
    __extends$1p(FoldAllRegionsAction, _super);
    function FoldAllRegionsAction() {
        return _super.call(this, {
            id: 'editor.foldAllMarkerRegions',
            label: localize('foldAllMarkerRegions.label', "Fold All Regions"),
            alias: 'Fold All Regions',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 29 /* KEY_8 */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    FoldAllRegionsAction.prototype.invoke = function (_foldingController, foldingModel, editor) {
        if (foldingModel.regions.hasTypes()) {
            setCollapseStateForType(foldingModel, FoldingRangeKind.Region.value, true);
        }
        else {
            var editorModel = editor.getModel();
            if (!editorModel) {
                return;
            }
            var foldingRules = LanguageConfigurationRegistry.getFoldingRules(editorModel.getLanguageIdentifier().id);
            if (foldingRules && foldingRules.markers && foldingRules.markers.start) {
                var regExp = new RegExp(foldingRules.markers.start);
                setCollapseStateForMatchingLines(foldingModel, regExp, true);
            }
        }
    };
    return FoldAllRegionsAction;
}(FoldingAction));
var UnfoldAllRegionsAction = /** @class */ (function (_super) {
    __extends$1p(UnfoldAllRegionsAction, _super);
    function UnfoldAllRegionsAction() {
        return _super.call(this, {
            id: 'editor.unfoldAllMarkerRegions',
            label: localize('unfoldAllMarkerRegions.label', "Unfold All Regions"),
            alias: 'Unfold All Regions',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 30 /* KEY_9 */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    UnfoldAllRegionsAction.prototype.invoke = function (_foldingController, foldingModel, editor) {
        if (foldingModel.regions.hasTypes()) {
            setCollapseStateForType(foldingModel, FoldingRangeKind.Region.value, false);
        }
        else {
            var editorModel = editor.getModel();
            if (!editorModel) {
                return;
            }
            var foldingRules = LanguageConfigurationRegistry.getFoldingRules(editorModel.getLanguageIdentifier().id);
            if (foldingRules && foldingRules.markers && foldingRules.markers.start) {
                var regExp = new RegExp(foldingRules.markers.start);
                setCollapseStateForMatchingLines(foldingModel, regExp, false);
            }
        }
    };
    return UnfoldAllRegionsAction;
}(FoldingAction));
var FoldAllAction = /** @class */ (function (_super) {
    __extends$1p(FoldAllAction, _super);
    function FoldAllAction() {
        return _super.call(this, {
            id: 'editor.foldAll',
            label: localize('foldAllAction.label', "Fold All"),
            alias: 'Fold All',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 21 /* KEY_0 */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    FoldAllAction.prototype.invoke = function (_foldingController, foldingModel, _editor) {
        setCollapseStateLevelsDown(foldingModel, true);
    };
    return FoldAllAction;
}(FoldingAction));
var UnfoldAllAction = /** @class */ (function (_super) {
    __extends$1p(UnfoldAllAction, _super);
    function UnfoldAllAction() {
        return _super.call(this, {
            id: 'editor.unfoldAll',
            label: localize('unfoldAllAction.label', "Unfold All"),
            alias: 'Unfold All',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 40 /* KEY_J */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    UnfoldAllAction.prototype.invoke = function (_foldingController, foldingModel, _editor) {
        setCollapseStateLevelsDown(foldingModel, false);
    };
    return UnfoldAllAction;
}(FoldingAction));
var FoldLevelAction = /** @class */ (function (_super) {
    __extends$1p(FoldLevelAction, _super);
    function FoldLevelAction() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FoldLevelAction.prototype.getFoldingLevel = function () {
        return parseInt(this.id.substr(FoldLevelAction.ID_PREFIX.length));
    };
    FoldLevelAction.prototype.invoke = function (_foldingController, foldingModel, editor) {
        setCollapseStateAtLevel(foldingModel, this.getFoldingLevel(), true, this.getSelectedLines(editor));
    };
    FoldLevelAction.ID_PREFIX = 'editor.foldLevel';
    FoldLevelAction.ID = function (level) { return FoldLevelAction.ID_PREFIX + level; };
    return FoldLevelAction;
}(FoldingAction));
registerEditorContribution(FoldingController);
registerEditorAction(UnfoldAction);
registerEditorAction(UnFoldRecursivelyAction);
registerEditorAction(FoldAction);
registerEditorAction(FoldRecursivelyAction);
registerEditorAction(FoldAllAction);
registerEditorAction(UnfoldAllAction);
registerEditorAction(FoldAllBlockCommentsAction);
registerEditorAction(FoldAllRegionsAction);
registerEditorAction(UnfoldAllRegionsAction);
for (var i = 1; i <= 7; i++) {
    registerInstantiatedEditorAction(new FoldLevelAction({
        id: FoldLevelAction.ID(i),
        label: localize('foldLevelAction.label', "Fold Level {0}", i),
        alias: "Fold Level " + i,
        precondition: null,
        kbOpts: {
            kbExpr: EditorContextKeys.editorTextFocus,
            primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | (21 /* KEY_0 */ + i)),
            weight: 100 /* EditorContrib */
        }
    }));
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1q = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var NoProviderError = /** @class */ (function (_super) {
    __extends$1q(NoProviderError, _super);
    function NoProviderError(message) {
        var _this = _super.call(this) || this;
        _this.name = NoProviderError.Name;
        _this.message = message;
        return _this;
    }
    NoProviderError.Name = 'NOPRO';
    return NoProviderError;
}(Error));
function getDocumentRangeFormattingEdits(model, range$$1, options, token) {
    var providers = DocumentRangeFormattingEditProviderRegistry.ordered(model);
    if (providers.length === 0) {
        return Promise.reject(new NoProviderError());
    }
    return first(providers.map(function (provider) { return function () {
        return Promise.resolve(provider.provideDocumentRangeFormattingEdits(model, range$$1, options, token))
            .then(undefined, onUnexpectedExternalError);
    }; }), function (result) { return !isFalsyOrEmpty(result); });
}
function getDocumentFormattingEdits(model, options, token) {
    var providers = DocumentFormattingEditProviderRegistry.ordered(model);
    // try range formatters when no document formatter is registered
    if (providers.length === 0) {
        return getDocumentRangeFormattingEdits(model, model.getFullModelRange(), options, token);
    }
    return first(providers.map(function (provider) { return function () {
        return Promise.resolve(provider.provideDocumentFormattingEdits(model, options, token))
            .then(undefined, onUnexpectedExternalError);
    }; }), function (result) { return !isFalsyOrEmpty(result); });
}
function getOnTypeFormattingEdits(model, position, ch, options) {
    var support = OnTypeFormattingEditProviderRegistry.ordered(model)[0];
    if (!support) {
        return Promise.resolve(undefined);
    }
    if (support.autoFormatTriggerCharacters.indexOf(ch) < 0) {
        return Promise.resolve(undefined);
    }
    return Promise.resolve(support.provideOnTypeFormattingEdits(model, position, ch, options, CancellationToken.None)).then(function (r) { return r; }, onUnexpectedExternalError);
}
registerLanguageCommand('_executeFormatRangeProvider', function (accessor, args) {
    var resource = args.resource, range$$1 = args.range, options = args.options;
    if (!(resource instanceof URI) || !Range.isIRange(range$$1)) {
        throw illegalArgument();
    }
    var model = accessor.get(IModelService).getModel(resource);
    if (!model) {
        throw illegalArgument('resource');
    }
    return getDocumentRangeFormattingEdits(model, Range.lift(range$$1), options, CancellationToken.None);
});
registerLanguageCommand('_executeFormatDocumentProvider', function (accessor, args) {
    var resource = args.resource, options = args.options;
    if (!(resource instanceof URI)) {
        throw illegalArgument('resource');
    }
    var model = accessor.get(IModelService).getModel(resource);
    if (!model) {
        throw illegalArgument('resource');
    }
    return getDocumentFormattingEdits(model, options, CancellationToken.None);
});
registerDefaultLanguageCommand('_executeFormatOnTypeProvider', function (model, position, args) {
    var ch = args.ch, options = args.options;
    if (typeof ch !== 'string') {
        throw illegalArgument('ch');
    }
    return getOnTypeFormattingEdits(model, position, ch, options);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var FormattingEdit = /** @class */ (function () {
    function FormattingEdit() {
    }
    FormattingEdit._handleEolEdits = function (editor, edits) {
        var newEol = undefined;
        var singleEdits = [];
        for (var _i = 0, edits_1 = edits; _i < edits_1.length; _i++) {
            var edit = edits_1[_i];
            if (typeof edit.eol === 'number') {
                newEol = edit.eol;
            }
            if (edit.range && typeof edit.text === 'string') {
                singleEdits.push(edit);
            }
        }
        if (typeof newEol === 'number') {
            editor.getModel().pushEOL(newEol);
        }
        return singleEdits;
    };
    FormattingEdit._isFullModelReplaceEdit = function (editor, edit) {
        var model = editor.getModel();
        var editRange = model.validateRange(edit.range);
        var fullModelRange = model.getFullModelRange();
        return fullModelRange.equalsRange(editRange);
    };
    FormattingEdit.execute = function (editor, _edits) {
        editor.pushUndoStop();
        var edits = FormattingEdit._handleEolEdits(editor, _edits);
        if (edits.length === 1 && FormattingEdit._isFullModelReplaceEdit(editor, edits[0])) {
            // We use replace semantics and hope that markers stay put...
            editor.executeEdits('formatEditsCommand', edits.map(function (edit) { return EditOperation.replace(Range.lift(edit.range), edit.text); }));
        }
        else {
            editor.executeEdits('formatEditsCommand', edits.map(function (edit) { return EditOperation.replaceMove(Range.lift(edit.range), edit.text); }));
        }
        editor.pushUndoStop();
    };
    return FormattingEdit;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1r = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$7 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$6 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
function alertFormattingEdits(edits) {
    edits = edits.filter(function (edit) { return edit.range; });
    if (!edits.length) {
        return;
    }
    var range$$1 = edits[0].range;
    for (var i = 1; i < edits.length; i++) {
        range$$1 = Range.plusRange(range$$1, edits[i].range);
    }
    var startLineNumber = range$$1.startLineNumber, endLineNumber = range$$1.endLineNumber;
    if (startLineNumber === endLineNumber) {
        if (edits.length === 1) {
            alert(localize('hint11', "Made 1 formatting edit on line {0}", startLineNumber));
        }
        else {
            alert(localize('hintn1', "Made {0} formatting edits on line {1}", edits.length, startLineNumber));
        }
    }
    else {
        if (edits.length === 1) {
            alert(localize('hint1n', "Made 1 formatting edit between lines {0} and {1}", startLineNumber, endLineNumber));
        }
        else {
            alert(localize('hintnn', "Made {0} formatting edits between lines {1} and {2}", edits.length, startLineNumber, endLineNumber));
        }
    }
}
var FormatOnType = /** @class */ (function () {
    function FormatOnType(editor, workerService) {
        var _this = this;
        this.editor = editor;
        this.workerService = workerService;
        this.callOnDispose = [];
        this.callOnModel = [];
        this.callOnDispose.push(editor.onDidChangeConfiguration(function () { return _this.update(); }));
        this.callOnDispose.push(editor.onDidChangeModel(function () { return _this.update(); }));
        this.callOnDispose.push(editor.onDidChangeModelLanguage(function () { return _this.update(); }));
        this.callOnDispose.push(OnTypeFormattingEditProviderRegistry.onDidChange(this.update, this));
    }
    FormatOnType.prototype.update = function () {
        var _this = this;
        // clean up
        this.callOnModel = dispose(this.callOnModel);
        // we are disabled
        if (!this.editor.getConfiguration().contribInfo.formatOnType) {
            return;
        }
        // no model
        if (!this.editor.getModel()) {
            return;
        }
        var model = this.editor.getModel();
        // no support
        var support = OnTypeFormattingEditProviderRegistry.ordered(model)[0];
        if (!support || !support.autoFormatTriggerCharacters) {
            return;
        }
        // register typing listeners that will trigger the format
        var triggerChars = new CharacterSet();
        for (var _i = 0, _a = support.autoFormatTriggerCharacters; _i < _a.length; _i++) {
            var ch = _a[_i];
            triggerChars.add(ch.charCodeAt(0));
        }
        this.callOnModel.push(this.editor.onDidType(function (text) {
            var lastCharCode = text.charCodeAt(text.length - 1);
            if (triggerChars.has(lastCharCode)) {
                _this.trigger(String.fromCharCode(lastCharCode));
            }
        }));
    };
    FormatOnType.prototype.trigger = function (ch) {
        var _this = this;
        if (this.editor.getSelections().length > 1) {
            return;
        }
        var model = this.editor.getModel();
        var position = this.editor.getPosition();
        var canceled = false;
        // install a listener that checks if edits happens before the
        // position on which we format right now. If so, we won't
        // apply the format edits
        var unbind = this.editor.onDidChangeModelContent(function (e) {
            if (e.isFlush) {
                // a model.setValue() was called
                // cancel only once
                canceled = true;
                unbind.dispose();
                return;
            }
            for (var i = 0, len = e.changes.length; i < len; i++) {
                var change = e.changes[i];
                if (change.range.endLineNumber <= position.lineNumber) {
                    // cancel only once
                    canceled = true;
                    unbind.dispose();
                    return;
                }
            }
        });
        var modelOpts = model.getOptions();
        getOnTypeFormattingEdits(model, position, ch, {
            tabSize: modelOpts.tabSize,
            insertSpaces: modelOpts.insertSpaces
        }).then(function (edits) {
            return _this.workerService.computeMoreMinimalEdits(model.uri, edits);
        }).then(function (edits) {
            unbind.dispose();
            if (canceled || isFalsyOrEmpty(edits)) {
                return;
            }
            FormattingEdit.execute(_this.editor, edits);
            alertFormattingEdits(edits);
        }, function (err) {
            unbind.dispose();
            throw err;
        });
    };
    FormatOnType.prototype.getId = function () {
        return FormatOnType.ID;
    };
    FormatOnType.prototype.dispose = function () {
        this.callOnDispose = dispose(this.callOnDispose);
        this.callOnModel = dispose(this.callOnModel);
    };
    FormatOnType.ID = 'editor.contrib.autoFormat';
    FormatOnType = __decorate$7([
        __param$6(1, IEditorWorkerService)
    ], FormatOnType);
    return FormatOnType;
}());
var FormatOnPaste = /** @class */ (function () {
    function FormatOnPaste(editor, workerService) {
        var _this = this;
        this.editor = editor;
        this.workerService = workerService;
        this.callOnDispose = [];
        this.callOnModel = [];
        this.callOnDispose.push(editor.onDidChangeConfiguration(function () { return _this.update(); }));
        this.callOnDispose.push(editor.onDidChangeModel(function () { return _this.update(); }));
        this.callOnDispose.push(editor.onDidChangeModelLanguage(function () { return _this.update(); }));
        this.callOnDispose.push(DocumentRangeFormattingEditProviderRegistry.onDidChange(this.update, this));
    }
    FormatOnPaste.prototype.update = function () {
        var _this = this;
        // clean up
        this.callOnModel = dispose(this.callOnModel);
        // we are disabled
        if (!this.editor.getConfiguration().contribInfo.formatOnPaste) {
            return;
        }
        // no model
        if (!this.editor.getModel()) {
            return;
        }
        var model = this.editor.getModel();
        // no support
        var support = DocumentRangeFormattingEditProviderRegistry.ordered(model)[0];
        if (!support || !support.provideDocumentRangeFormattingEdits) {
            return;
        }
        this.callOnModel.push(this.editor.onDidPaste(function (range$$1) {
            _this.trigger(range$$1);
        }));
    };
    FormatOnPaste.prototype.trigger = function (range$$1) {
        var _this = this;
        if (this.editor.getSelections().length > 1) {
            return;
        }
        var model = this.editor.getModel();
        var _a = model.getOptions(), tabSize = _a.tabSize, insertSpaces = _a.insertSpaces;
        var state = new EditorState(this.editor, 1 /* Value */ | 4 /* Position */);
        getDocumentRangeFormattingEdits(model, range$$1, { tabSize: tabSize, insertSpaces: insertSpaces }, CancellationToken.None).then(function (edits) {
            return _this.workerService.computeMoreMinimalEdits(model.uri, edits);
        }).then(function (edits) {
            if (!state.validate(_this.editor) || isFalsyOrEmpty(edits)) {
                return;
            }
            FormattingEdit.execute(_this.editor, edits);
            alertFormattingEdits(edits);
        });
    };
    FormatOnPaste.prototype.getId = function () {
        return FormatOnPaste.ID;
    };
    FormatOnPaste.prototype.dispose = function () {
        this.callOnDispose = dispose(this.callOnDispose);
        this.callOnModel = dispose(this.callOnModel);
    };
    FormatOnPaste.ID = 'editor.contrib.formatOnPaste';
    FormatOnPaste = __decorate$7([
        __param$6(1, IEditorWorkerService)
    ], FormatOnPaste);
    return FormatOnPaste;
}());
var AbstractFormatAction = /** @class */ (function (_super) {
    __extends$1r(AbstractFormatAction, _super);
    function AbstractFormatAction() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AbstractFormatAction.prototype.run = function (accessor, editor) {
        var _this = this;
        var workerService = accessor.get(IEditorWorkerService);
        var notificationService = accessor.get(INotificationService);
        var formattingPromise = this._getFormattingEdits(editor, CancellationToken.None);
        if (!formattingPromise) {
            return Promise.resolve(void 0);
        }
        // Capture the state of the editor
        var state = new EditorState(editor, 1 /* Value */ | 4 /* Position */);
        // Receive formatted value from worker
        return formattingPromise.then(function (edits) { return workerService.computeMoreMinimalEdits(editor.getModel().uri, edits); }).then(function (edits) {
            if (!state.validate(editor) || isFalsyOrEmpty(edits)) {
                return;
            }
            FormattingEdit.execute(editor, edits);
            alertFormattingEdits(edits);
            editor.focus();
            editor.revealPositionInCenterIfOutsideViewport(editor.getPosition(), 1 /* Immediate */);
        }, function (err) {
            if (err instanceof Error && err.name === NoProviderError.Name) {
                _this._notifyNoProviderError(notificationService, editor.getModel().getLanguageIdentifier().language);
            }
            else {
                throw err;
            }
        });
    };
    AbstractFormatAction.prototype._notifyNoProviderError = function (notificationService, language) {
        notificationService.info(localize('no.provider', "There is no formatter for '{0}'-files installed.", language));
    };
    return AbstractFormatAction;
}(EditorAction));
var FormatDocumentAction = /** @class */ (function (_super) {
    __extends$1r(FormatDocumentAction, _super);
    function FormatDocumentAction() {
        return _super.call(this, {
            id: 'editor.action.formatDocument',
            label: localize('formatDocument.label', "Format Document"),
            alias: 'Format Document',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 1024 /* Shift */ | 512 /* Alt */ | 36 /* KEY_F */,
                // secondary: [KeyChord(KeyMod.CtrlCmd | KeyCode.KEY_K, KeyMod.CtrlCmd | KeyCode.KEY_D)],
                linux: { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 39 /* KEY_I */ },
                weight: 100 /* EditorContrib */
            },
            menuOpts: {
                when: EditorContextKeys.hasDocumentFormattingProvider,
                group: '1_modification',
                order: 1.3
            }
        }) || this;
    }
    FormatDocumentAction.prototype._getFormattingEdits = function (editor, token) {
        var model = editor.getModel();
        var _a = model.getOptions(), tabSize = _a.tabSize, insertSpaces = _a.insertSpaces;
        return getDocumentFormattingEdits(model, { tabSize: tabSize, insertSpaces: insertSpaces }, token);
    };
    FormatDocumentAction.prototype._notifyNoProviderError = function (notificationService, language) {
        notificationService.info(localize('no.documentprovider', "There is no document formatter for '{0}'-files installed.", language));
    };
    return FormatDocumentAction;
}(AbstractFormatAction));
var FormatSelectionAction = /** @class */ (function (_super) {
    __extends$1r(FormatSelectionAction, _super);
    function FormatSelectionAction() {
        return _super.call(this, {
            id: 'editor.action.formatSelection',
            label: localize('formatSelection.label', "Format Selection"),
            alias: 'Format Code',
            precondition: ContextKeyExpr.and(EditorContextKeys.writable),
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 36 /* KEY_F */),
                weight: 100 /* EditorContrib */
            },
            menuOpts: {
                when: ContextKeyExpr.and(EditorContextKeys.hasDocumentSelectionFormattingProvider, EditorContextKeys.hasNonEmptySelection),
                group: '1_modification',
                order: 1.31
            }
        }) || this;
    }
    FormatSelectionAction.prototype._getFormattingEdits = function (editor, token) {
        var model = editor.getModel();
        var selection = editor.getSelection();
        if (selection.isEmpty()) {
            var maxColumn = model.getLineMaxColumn(selection.startLineNumber);
            selection = selection.setStartPosition(selection.startLineNumber, 1);
            selection = selection.setEndPosition(selection.endLineNumber, maxColumn);
        }
        var _a = model.getOptions(), tabSize = _a.tabSize, insertSpaces = _a.insertSpaces;
        return getDocumentRangeFormattingEdits(model, selection, { tabSize: tabSize, insertSpaces: insertSpaces }, token);
    };
    FormatSelectionAction.prototype._notifyNoProviderError = function (notificationService, language) {
        notificationService.info(localize('no.selectionprovider', "There is no selection formatter for '{0}'-files installed.", language));
    };
    return FormatSelectionAction;
}(AbstractFormatAction));
registerEditorContribution(FormatOnType);
registerEditorContribution(FormatOnPaste);
registerEditorAction(FormatDocumentAction);
registerEditorAction(FormatSelectionAction);
// this is the old format action that does both (format document OR format selection)
// and we keep it here such that existing keybinding configurations etc will still work
CommandsRegistry.registerCommand('editor.action.format', function (accessor) {
    var editor = accessor.get(ICodeEditorService).getFocusedCodeEditor();
    if (editor) {
        return new /** @class */ (function (_super) {
            __extends$1r(class_1, _super);
            function class_1() {
                return _super.call(this, {}) || this;
            }
            class_1.prototype._getFormattingEdits = function (editor, token) {
                var model = editor.getModel();
                var editorSelection = editor.getSelection();
                var _a = model.getOptions(), tabSize = _a.tabSize, insertSpaces = _a.insertSpaces;
                return editorSelection.isEmpty()
                    ? getDocumentFormattingEdits(model, { tabSize: tabSize, insertSpaces: insertSpaces }, token)
                    : getDocumentRangeFormattingEdits(model, editorSelection, { tabSize: tabSize, insertSpaces: insertSpaces }, token);
            };
            return class_1;
        }(AbstractFormatAction))().run(accessor, editor);
    }
    return undefined;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MarkerSeverity;
(function (MarkerSeverity) {
    MarkerSeverity[MarkerSeverity["Hint"] = 1] = "Hint";
    MarkerSeverity[MarkerSeverity["Info"] = 2] = "Info";
    MarkerSeverity[MarkerSeverity["Warning"] = 4] = "Warning";
    MarkerSeverity[MarkerSeverity["Error"] = 8] = "Error";
})(MarkerSeverity || (MarkerSeverity = {}));
(function (MarkerSeverity) {
    function compare(a, b) {
        return b - a;
    }
    MarkerSeverity.compare = compare;
    var _displayStrings = Object.create(null);
    _displayStrings[MarkerSeverity.Error] = localize('sev.error', "Error");
    _displayStrings[MarkerSeverity.Warning] = localize('sev.warning', "Warning");
    _displayStrings[MarkerSeverity.Info] = localize('sev.info', "Info");
    function toString(a) {
        return _displayStrings[a] || '';
    }
    MarkerSeverity.toString = toString;
    function fromSeverity(severity) {
        switch (severity) {
            case Severity$1.Error: return MarkerSeverity.Error;
            case Severity$1.Warning: return MarkerSeverity.Warning;
            case Severity$1.Info: return MarkerSeverity.Info;
            case Severity$1.Ignore: return MarkerSeverity.Hint;
        }
    }
    MarkerSeverity.fromSeverity = fromSeverity;
})(MarkerSeverity || (MarkerSeverity = {}));
var IMarkerData;
(function (IMarkerData) {
    var emptyString = '';
    function makeKey(markerData) {
        var result = [emptyString];
        if (markerData.source) {
            result.push(markerData.source.replace('¦', '\¦'));
        }
        else {
            result.push(emptyString);
        }
        if (markerData.code) {
            result.push(markerData.code.replace('¦', '\¦'));
        }
        else {
            result.push(emptyString);
        }
        if (markerData.severity !== void 0 && markerData.severity !== null) {
            result.push(MarkerSeverity.toString(markerData.severity));
        }
        else {
            result.push(emptyString);
        }
        if (markerData.message) {
            result.push(markerData.message.replace('¦', '\¦'));
        }
        else {
            result.push(emptyString);
        }
        if (markerData.startLineNumber !== void 0 && markerData.startLineNumber !== null) {
            result.push(markerData.startLineNumber.toString());
        }
        else {
            result.push(emptyString);
        }
        if (markerData.startColumn !== void 0 && markerData.startColumn !== null) {
            result.push(markerData.startColumn.toString());
        }
        else {
            result.push(emptyString);
        }
        if (markerData.endLineNumber !== void 0 && markerData.endLineNumber !== null) {
            result.push(markerData.endLineNumber.toString());
        }
        else {
            result.push(emptyString);
        }
        if (markerData.endColumn !== void 0 && markerData.endColumn !== null) {
            result.push(markerData.endColumn.toString());
        }
        else {
            result.push(emptyString);
        }
        result.push(emptyString);
        return result.join('¦');
    }
    IMarkerData.makeKey = makeKey;
})(IMarkerData || (IMarkerData = {}));
var IMarkerService = createDecorator('markerService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var defaultColor = new Color(new RGBA(0, 122, 204));
var defaultOptions$2 = {
    showArrow: true,
    showFrame: true,
    className: '',
    frameColor: defaultColor,
    arrowColor: defaultColor,
    keepEditorSelection: false
};
var WIDGET_ID = 'vs.editor.contrib.zoneWidget';
var ViewZoneDelegate = /** @class */ (function () {
    function ViewZoneDelegate(domNode, afterLineNumber, afterColumn, heightInLines, onDomNodeTop, onComputedHeight) {
        this.domNode = domNode;
        this.afterLineNumber = afterLineNumber;
        this.afterColumn = afterColumn;
        this.heightInLines = heightInLines;
        this._onDomNodeTop = onDomNodeTop;
        this._onComputedHeight = onComputedHeight;
    }
    ViewZoneDelegate.prototype.onDomNodeTop = function (top) {
        this._onDomNodeTop(top);
    };
    ViewZoneDelegate.prototype.onComputedHeight = function (height) {
        this._onComputedHeight(height);
    };
    return ViewZoneDelegate;
}());
var OverlayWidgetDelegate = /** @class */ (function () {
    function OverlayWidgetDelegate(id, domNode) {
        this._id = id;
        this._domNode = domNode;
    }
    OverlayWidgetDelegate.prototype.getId = function () {
        return this._id;
    };
    OverlayWidgetDelegate.prototype.getDomNode = function () {
        return this._domNode;
    };
    OverlayWidgetDelegate.prototype.getPosition = function () {
        return null;
    };
    return OverlayWidgetDelegate;
}());
var Arrow = /** @class */ (function () {
    function Arrow(_editor) {
        this._editor = _editor;
        this._ruleName = Arrow._IdGenerator.nextId();
        this._decorations = [];
        //
    }
    Arrow.prototype.dispose = function () {
        this.hide();
        removeCSSRulesContainingSelector(this._ruleName);
    };
    Object.defineProperty(Arrow.prototype, "color", {
        set: function (value) {
            if (this._color !== value) {
                this._color = value;
                this._updateStyle();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Arrow.prototype, "height", {
        set: function (value) {
            if (this._height !== value) {
                this._height = value;
                this._updateStyle();
            }
        },
        enumerable: true,
        configurable: true
    });
    Arrow.prototype._updateStyle = function () {
        removeCSSRulesContainingSelector(this._ruleName);
        createCSSRule(".monaco-editor " + this._ruleName, "border-style: solid; border-color: transparent; border-bottom-color: " + this._color + "; border-width: " + this._height + "px; bottom: -" + this._height + "px; margin-left: -" + this._height + "px; ");
    };
    Arrow.prototype.show = function (where) {
        this._decorations = this._editor.deltaDecorations(this._decorations, [{ range: Range.fromPositions(where), options: { className: this._ruleName, stickiness: 1 /* NeverGrowsWhenTypingAtEdges */ } }]);
    };
    Arrow.prototype.hide = function () {
        this._editor.deltaDecorations(this._decorations, []);
    };
    Arrow._IdGenerator = new IdGenerator('.arrow-decoration-');
    return Arrow;
}());
var ZoneWidget = /** @class */ (function () {
    function ZoneWidget(editor, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this._positionMarkerId = [];
        this._disposables = [];
        this._isShowing = false;
        this.editor = editor;
        this.options = deepClone(options);
        mixin(this.options, defaultOptions$2, false);
        this.domNode = document.createElement('div');
        if (!this.options.isAccessible) {
            this.domNode.setAttribute('aria-hidden', 'true');
            this.domNode.setAttribute('role', 'presentation');
        }
        this._disposables.push(this.editor.onDidLayoutChange(function (info) {
            var width = _this._getWidth(info);
            _this.domNode.style.width = width + 'px';
            _this.domNode.style.left = _this._getLeft(info) + 'px';
            _this._onWidth(width);
        }));
    }
    ZoneWidget.prototype.dispose = function () {
        var _this = this;
        dispose(this._disposables);
        if (this._overlayWidget) {
            this.editor.removeOverlayWidget(this._overlayWidget);
            this._overlayWidget = null;
        }
        if (this._viewZone) {
            this.editor.changeViewZones(function (accessor) {
                if (_this._viewZone) {
                    accessor.removeZone(_this._viewZone.id);
                }
                _this._viewZone = null;
            });
        }
        this.editor.deltaDecorations(this._positionMarkerId, []);
        this._positionMarkerId = [];
    };
    ZoneWidget.prototype.create = function () {
        addClass(this.domNode, 'zone-widget');
        if (this.options.className) {
            addClass(this.domNode, this.options.className);
        }
        this.container = document.createElement('div');
        addClass(this.container, 'zone-widget-container');
        this.domNode.appendChild(this.container);
        if (this.options.showArrow) {
            this._arrow = new Arrow(this.editor);
            this._disposables.push(this._arrow);
        }
        this._fillContainer(this.container);
        this._initSash();
        this._applyStyles();
    };
    ZoneWidget.prototype.style = function (styles) {
        if (styles.frameColor) {
            this.options.frameColor = styles.frameColor;
        }
        if (styles.arrowColor) {
            this.options.arrowColor = styles.arrowColor;
        }
        this._applyStyles();
    };
    ZoneWidget.prototype._applyStyles = function () {
        if (this.container && this.options.frameColor) {
            var frameColor = this.options.frameColor.toString();
            this.container.style.borderTopColor = frameColor;
            this.container.style.borderBottomColor = frameColor;
        }
        if (this._arrow && this.options.arrowColor) {
            var arrowColor = this.options.arrowColor.toString();
            this._arrow.color = arrowColor;
        }
    };
    ZoneWidget.prototype._getWidth = function (info) {
        return info.width - info.minimapWidth - info.verticalScrollbarWidth;
    };
    ZoneWidget.prototype._getLeft = function (info) {
        // If minimap is to the left, we move beyond it
        if (info.minimapWidth > 0 && info.minimapLeft === 0) {
            return info.minimapWidth;
        }
        return 0;
    };
    ZoneWidget.prototype._onViewZoneTop = function (top) {
        this.domNode.style.top = top + 'px';
    };
    ZoneWidget.prototype._onViewZoneHeight = function (height) {
        this.domNode.style.height = height + "px";
        var containerHeight = height - this._decoratingElementsHeight();
        this.container.style.height = containerHeight + "px";
        var layoutInfo = this.editor.getLayoutInfo();
        this._doLayout(containerHeight, this._getWidth(layoutInfo));
        this._resizeSash.layout();
    };
    Object.defineProperty(ZoneWidget.prototype, "position", {
        get: function () {
            var id = this._positionMarkerId[0];
            if (!id) {
                return undefined;
            }
            var model = this.editor.getModel();
            if (!model) {
                return undefined;
            }
            var range = model.getDecorationRange(id);
            if (!range) {
                return undefined;
            }
            return range.getStartPosition();
        },
        enumerable: true,
        configurable: true
    });
    ZoneWidget.prototype.show = function (rangeOrPos, heightInLines) {
        var range = Range.isIRange(rangeOrPos)
            ? rangeOrPos
            : new Range(rangeOrPos.lineNumber, rangeOrPos.column, rangeOrPos.lineNumber, rangeOrPos.column);
        this._isShowing = true;
        this._showImpl(range, heightInLines);
        this._isShowing = false;
        this._positionMarkerId = this.editor.deltaDecorations(this._positionMarkerId, [{ range: range, options: ModelDecorationOptions.EMPTY }]);
    };
    ZoneWidget.prototype.hide = function () {
        var _this = this;
        if (this._viewZone) {
            this.editor.changeViewZones(function (accessor) {
                if (_this._viewZone) {
                    accessor.removeZone(_this._viewZone.id);
                }
            });
            this._viewZone = null;
        }
        if (this._overlayWidget) {
            this.editor.removeOverlayWidget(this._overlayWidget);
            this._overlayWidget = null;
        }
        if (this._arrow) {
            this._arrow.hide();
        }
    };
    ZoneWidget.prototype._decoratingElementsHeight = function () {
        var lineHeight = this.editor.getConfiguration().lineHeight;
        var result = 0;
        if (this.options.showArrow) {
            var arrowHeight = Math.round(lineHeight / 3);
            result += 2 * arrowHeight;
        }
        if (this.options.showFrame) {
            var frameThickness = Math.round(lineHeight / 9);
            result += 2 * frameThickness;
        }
        return result;
    };
    ZoneWidget.prototype._showImpl = function (where, heightInLines) {
        var _this = this;
        var position = {
            lineNumber: where.startLineNumber,
            column: where.startColumn
        };
        var layoutInfo = this.editor.getLayoutInfo();
        var width = this._getWidth(layoutInfo);
        this.domNode.style.width = width + "px";
        this.domNode.style.left = this._getLeft(layoutInfo) + 'px';
        // Render the widget as zone (rendering) and widget (lifecycle)
        var viewZoneDomNode = document.createElement('div');
        viewZoneDomNode.style.overflow = 'hidden';
        var lineHeight = this.editor.getConfiguration().lineHeight;
        // adjust heightInLines to viewport
        var maxHeightInLines = (this.editor.getLayoutInfo().height / lineHeight) * .8;
        if (heightInLines >= maxHeightInLines) {
            heightInLines = maxHeightInLines;
        }
        var arrowHeight = 0;
        var frameThickness = 0;
        // Render the arrow one 1/3 of an editor line height
        if (this.options.showArrow) {
            arrowHeight = Math.round(lineHeight / 3);
            this._arrow.height = arrowHeight;
            this._arrow.show(position);
        }
        // Render the frame as 1/9 of an editor line height
        if (this.options.showFrame) {
            frameThickness = Math.round(lineHeight / 9);
        }
        // insert zone widget
        this.editor.changeViewZones(function (accessor) {
            if (_this._viewZone) {
                accessor.removeZone(_this._viewZone.id);
            }
            if (_this._overlayWidget) {
                _this.editor.removeOverlayWidget(_this._overlayWidget);
                _this._overlayWidget = null;
            }
            _this.domNode.style.top = '-1000px';
            _this._viewZone = new ViewZoneDelegate(viewZoneDomNode, position.lineNumber, position.column, heightInLines, function (top) { return _this._onViewZoneTop(top); }, function (height) { return _this._onViewZoneHeight(height); });
            _this._viewZone.id = accessor.addZone(_this._viewZone);
            _this._overlayWidget = new OverlayWidgetDelegate(WIDGET_ID + _this._viewZone.id, _this.domNode);
            _this.editor.addOverlayWidget(_this._overlayWidget);
        });
        if (this.options.showFrame) {
            var width_1 = this.options.frameWidth ? this.options.frameWidth : frameThickness;
            this.container.style.borderTopWidth = width_1 + 'px';
            this.container.style.borderBottomWidth = width_1 + 'px';
        }
        var containerHeight = heightInLines * lineHeight - this._decoratingElementsHeight();
        this.container.style.top = arrowHeight + 'px';
        this.container.style.height = containerHeight + 'px';
        this.container.style.overflow = 'hidden';
        this._doLayout(containerHeight, width);
        if (!this.options.keepEditorSelection) {
            this.editor.setSelection(where);
        }
        var model = this.editor.getModel();
        if (model) {
            // Reveal the line above or below the zone widget, to get the zone widget in the viewport
            var revealLineNumber = Math.min(model.getLineCount(), Math.max(1, where.endLineNumber + 1));
            this.revealLine(revealLineNumber);
        }
    };
    ZoneWidget.prototype.revealLine = function (lineNumber) {
        this.editor.revealLine(lineNumber, 0 /* Smooth */);
    };
    ZoneWidget.prototype.setCssClass = function (className, classToReplace) {
        if (classToReplace) {
            this.container.classList.remove(classToReplace);
        }
        addClass(this.container, className);
    };
    ZoneWidget.prototype._onWidth = function (widthInPixel) {
        // implement in subclass
    };
    ZoneWidget.prototype._doLayout = function (heightInPixel, widthInPixel) {
        // implement in subclass
    };
    ZoneWidget.prototype._relayout = function (newHeightInLines) {
        var _this = this;
        if (this._viewZone && this._viewZone.heightInLines !== newHeightInLines) {
            this.editor.changeViewZones(function (accessor) {
                if (_this._viewZone) {
                    _this._viewZone.heightInLines = newHeightInLines;
                    accessor.layoutZone(_this._viewZone.id);
                }
            });
        }
    };
    // --- sash
    ZoneWidget.prototype._initSash = function () {
        var _this = this;
        this._resizeSash = new Sash(this.domNode, this, { orientation: 1 /* HORIZONTAL */ });
        if (!this.options.isResizeable) {
            this._resizeSash.hide();
            this._resizeSash.state = 0 /* Disabled */;
        }
        var data;
        this._disposables.push(this._resizeSash.onDidStart(function (e) {
            if (_this._viewZone) {
                data = {
                    startY: e.startY,
                    heightInLines: _this._viewZone.heightInLines,
                };
            }
        }));
        this._disposables.push(this._resizeSash.onDidEnd(function () {
            data = undefined;
        }));
        this._disposables.push(this._resizeSash.onDidChange(function (evt) {
            if (data) {
                var lineDelta = (evt.currentY - data.startY) / _this.editor.getConfiguration().lineHeight;
                var roundedLineDelta = lineDelta < 0 ? Math.ceil(lineDelta) : Math.floor(lineDelta);
                var newHeightInLines = data.heightInLines + roundedLineDelta;
                if (newHeightInLines > 5 && newHeightInLines < 35) {
                    _this._relayout(newHeightInLines);
                }
            }
        }));
    };
    ZoneWidget.prototype.getHorizontalSashLeft = function () {
        return 0;
    };
    ZoneWidget.prototype.getHorizontalSashTop = function () {
        return (this.domNode.style.height === null ? 0 : parseInt(this.domNode.style.height)) - (this._decoratingElementsHeight() / 2);
    };
    ZoneWidget.prototype.getHorizontalSashWidth = function () {
        var layoutInfo = this.editor.getLayoutInfo();
        return layoutInfo.width - layoutInfo.minimapWidth;
    };
    return ZoneWidget;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function hasToIgnoreCase(resource) {
    // A file scheme resource is in the same platform as code, so ignore case for non linux platforms
    // Resource can be from another platform. Lowering the case as an hack. Should come from File system provider
    return resource && resource.scheme === Schemas.file ? !isLinux : true;
}
function basenameOrAuthority(resource) {
    return basename$1(resource) || resource.authority;
}
function isEqual(first, second, ignoreCase) {
    if (ignoreCase === void 0) { ignoreCase = hasToIgnoreCase(first); }
    var identityEquals = (first === second);
    if (identityEquals) {
        return true;
    }
    if (!first || !second) {
        return false;
    }
    if (ignoreCase) {
        return equalsIgnoreCase(first.toString(), second.toString());
    }
    return first.toString() === second.toString();
}
function basename$1(resource) {
    return basename(resource.path);
}
/**
 * Join a URI path with a path fragment and normalizes the resulting path.
 *
 * @param resource The input URI.
 * @param pathFragment The path fragment to add to the URI path.
 * @returns The resulting URI.
 */
function joinPath(resource, pathFragment) {
    var joinedPath;
    if (resource.scheme === Schemas.file) {
        joinedPath = URI.file(join(fsPath(resource), pathFragment)).path;
    }
    else {
        joinedPath = join(resource.path, pathFragment);
    }
    return resource.with({
        path: joinedPath
    });
}
/**
 * Normalizes the path part of a URI: Resolves `.` and `..` elements with directory names.
 *
 * @param resource The URI to normalize the path.
 * @returns The URI with the normalized path.
 */
function normalizePath(resource) {
    var normalizedPath;
    if (resource.scheme === Schemas.file) {
        normalizedPath = URI.file(normalize(fsPath(resource))).path;
    }
    else {
        normalizedPath = normalize(resource.path);
    }
    return resource.with({
        path: normalizedPath
    });
}
/**
 * Returns the fsPath of an URI where the drive letter is not normalized.
 * See #56403.
 */
function fsPath(uri) {
    var value;
    if (uri.authority && uri.path.length > 1 && uri.scheme === 'file') {
        // unc path: file://shares/c$/far/boo
        value = "//" + uri.authority + uri.path;
    }
    else if (isWindows
        && uri.path.charCodeAt(0) === 47 /* Slash */
        && (uri.path.charCodeAt(1) >= 65 /* A */ && uri.path.charCodeAt(1) <= 90 /* Z */ || uri.path.charCodeAt(1) >= 97 /* a */ && uri.path.charCodeAt(1) <= 122 /* z */)
        && uri.path.charCodeAt(2) === 58 /* Colon */) {
        value = uri.path.substr(1);
    }
    else {
        // other path
        value = uri.path;
    }
    if (isWindows) {
        value = value.replace(/\//g, '\\');
    }
    return value;
}
/**
 * Data URI related helpers.
 */
var DataUri;
(function (DataUri) {
    DataUri.META_DATA_LABEL = 'label';
    DataUri.META_DATA_DESCRIPTION = 'description';
    DataUri.META_DATA_SIZE = 'size';
    DataUri.META_DATA_MIME = 'mime';
    function parseMetaData(dataUri) {
        var metadata = new Map();
        // Given a URI of:  data:image/png;size:2313;label:SomeLabel;description:SomeDescription;base64,77+9UE5...
        // the metadata is: size:2313;label:SomeLabel;description:SomeDescription
        var meta = dataUri.path.substring(dataUri.path.indexOf(';') + 1, dataUri.path.lastIndexOf(';'));
        meta.split(';').forEach(function (property) {
            var _a = property.split(':'), key = _a[0], value = _a[1];
            if (key && value) {
                metadata.set(key, value);
            }
        });
        // Given a URI of:  data:image/png;size:2313;label:SomeLabel;description:SomeDescription;base64,77+9UE5...
        // the mime is: image/png
        var mime = dataUri.path.substring(0, dataUri.path.indexOf(';'));
        if (mime) {
            metadata.set(DataUri.META_DATA_MIME, mime);
        }
        return metadata;
    }
    DataUri.parseMetaData = parseMetaData;
})(DataUri || (DataUri = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * @deprecated use LabelService instead
 */
function getPathLabel(resource, userHomeProvider, rootProvider) {
    if (typeof resource === 'string') {
        resource = URI.file(resource);
    }
    // return early if we can resolve a relative path label from the root
    if (rootProvider) {
        var baseResource = rootProvider.getWorkspaceFolder(resource);
        if (baseResource) {
            var hasMultipleRoots = rootProvider.getWorkspace().folders.length > 1;
            var pathLabel = void 0;
            if (isEqual(baseResource.uri, resource, !isLinux)) {
                pathLabel = ''; // no label if paths are identical
            }
            else {
                pathLabel = normalize(ltrim(resource.path.substr(baseResource.uri.path.length), sep), true);
            }
            if (hasMultipleRoots) {
                var rootName = (baseResource && baseResource.name) ? baseResource.name : basename(baseResource.uri.fsPath);
                pathLabel = pathLabel ? (rootName + ' • ' + pathLabel) : rootName; // always show root basename if there are multiple
            }
            return pathLabel;
        }
    }
    // return if the resource is neither file:// nor untitled:// and no baseResource was provided
    if (resource.scheme !== Schemas.file && resource.scheme !== Schemas.untitled) {
        return resource.with({ query: null, fragment: null }).toString(true);
    }
    // convert c:\something => C:\something
    if (hasDriveLetter(resource.fsPath)) {
        return normalize(normalizeDriveLetter(resource.fsPath), true);
    }
    // normalize and tildify (macOS, Linux only)
    var res = normalize(resource.fsPath, true);
    if (!isWindows && userHomeProvider) {
        res = tildify(res, userHomeProvider.userHome);
    }
    return res;
}
function getBaseLabel(resource) {
    if (!resource) {
        return undefined;
    }
    if (typeof resource === 'string') {
        resource = URI.file(resource);
    }
    var base = basename(resource.path) || (resource.scheme === Schemas.file ? resource.fsPath : resource.path) /* can be empty string if '/' is passed in */;
    // convert c: => C:
    if (hasDriveLetter(base)) {
        return normalizeDriveLetter(base);
    }
    return base;
}
function hasDriveLetter(path) {
    return !!(isWindows && path && path[1] === ':');
}
function normalizeDriveLetter(path) {
    if (hasDriveLetter(path)) {
        return path.charAt(0).toUpperCase() + path.slice(1);
    }
    return path;
}
var normalizedUserHomeCached = Object.create(null);
function tildify(path, userHome) {
    if (isWindows || !path || !userHome) {
        return path; // unsupported
    }
    // Keep a normalized user home path as cache to prevent accumulated string creation
    var normalizedUserHome = normalizedUserHomeCached.original === userHome ? normalizedUserHomeCached.normalized : void 0;
    if (!normalizedUserHome) {
        normalizedUserHome = "" + rtrim(userHome, sep) + sep;
        normalizedUserHomeCached = { original: userHome, normalized: normalizedUserHome };
    }
    // Linux: case sensitive, macOS: case insensitive
    if (isLinux ? startsWith(path, normalizedUserHome) : startsWithIgnoreCase(path, normalizedUserHome)) {
        path = "~/" + path.substr(normalizedUserHome.length);
    }
    return path;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1s = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var MessageWidget = /** @class */ (function () {
    function MessageWidget(parent, editor, onRelatedInformation) {
        var _this = this;
        this._lines = 0;
        this._longestLineLength = 0;
        this._relatedDiagnostics = new WeakMap();
        this._disposables = [];
        this._editor = editor;
        var domNode = document.createElement('div');
        domNode.className = 'descriptioncontainer';
        domNode.setAttribute('aria-live', 'assertive');
        domNode.setAttribute('role', 'alert');
        this._messageBlock = document.createElement('div');
        domNode.appendChild(this._messageBlock);
        this._relatedBlock = document.createElement('div');
        domNode.appendChild(this._relatedBlock);
        this._disposables.push(addStandardDisposableListener(this._relatedBlock, 'click', function (event) {
            event.preventDefault();
            var related = _this._relatedDiagnostics.get(event.target);
            if (related) {
                onRelatedInformation(related);
            }
        }));
        this._scrollable = new ScrollableElement(domNode, {
            horizontal: 1 /* Auto */,
            vertical: 1 /* Auto */,
            useShadows: false,
            horizontalScrollbarSize: 3,
            verticalScrollbarSize: 3
        });
        addClass(this._scrollable.getDomNode(), 'block');
        parent.appendChild(this._scrollable.getDomNode());
        this._disposables.push(this._scrollable.onScroll(function (e) {
            domNode.style.left = "-" + e.scrollLeft + "px";
            domNode.style.top = "-" + e.scrollTop + "px";
        }));
        this._disposables.push(this._scrollable);
    }
    MessageWidget.prototype.dispose = function () {
        dispose(this._disposables);
    };
    MessageWidget.prototype.update = function (_a) {
        var source = _a.source, message = _a.message, relatedInformation = _a.relatedInformation, code = _a.code;
        if (source) {
            this._lines = 0;
            this._longestLineLength = 0;
            var indent = new Array(source.length + 3 + 1).join(' ');
            var lines = message.split(/\r\n|\r|\n/g);
            for (var i = 0; i < lines.length; i++) {
                var line = lines[i];
                this._lines += 1;
                if (code && i === lines.length - 1) {
                    line += " [" + code + "]";
                }
                this._longestLineLength = Math.max(line.length, this._longestLineLength);
                if (i === 0) {
                    message = "[" + source + "] " + line;
                }
                else {
                    message += "\n" + indent + line;
                }
            }
        }
        else {
            this._lines = 1;
            if (code) {
                message += " [" + code + "]";
            }
            this._longestLineLength = message.length;
        }
        clearNode(this._relatedBlock);
        if (!isFalsyOrEmpty(relatedInformation)) {
            this._relatedBlock.style.paddingTop = Math.floor(this._editor.getConfiguration().lineHeight * .66) + "px";
            this._lines += 1;
            for (var _i = 0, _b = relatedInformation || []; _i < _b.length; _i++) {
                var related = _b[_i];
                var container = document.createElement('div');
                var relatedResource = document.createElement('span');
                addClass(relatedResource, 'filename');
                relatedResource.innerHTML = getBaseLabel(related.resource) + "(" + related.startLineNumber + ", " + related.startColumn + "): ";
                relatedResource.title = getPathLabel(related.resource, undefined);
                this._relatedDiagnostics.set(relatedResource, related);
                var relatedMessage = document.createElement('span');
                relatedMessage.innerText = related.message;
                this._editor.applyFontInfo(relatedMessage);
                container.appendChild(relatedResource);
                container.appendChild(relatedMessage);
                this._lines += 1;
                this._relatedBlock.appendChild(container);
            }
        }
        this._messageBlock.innerText = message;
        this._editor.applyFontInfo(this._messageBlock);
        var fontInfo = this._editor.getConfiguration().fontInfo;
        var scrollWidth = Math.ceil(fontInfo.typicalFullwidthCharacterWidth * this._longestLineLength * 0.75);
        var scrollHeight = fontInfo.lineHeight * this._lines;
        this._scrollable.setScrollDimensions({ scrollWidth: scrollWidth, scrollHeight: scrollHeight });
    };
    MessageWidget.prototype.layout = function (height, width) {
        this._scrollable.getDomNode().style.height = height + "px";
        this._scrollable.setScrollDimensions({ width: width, height: height });
    };
    MessageWidget.prototype.getHeightInLines = function () {
        return Math.min(17, this._lines);
    };
    return MessageWidget;
}());
var MarkerNavigationWidget = /** @class */ (function (_super) {
    __extends$1s(MarkerNavigationWidget, _super);
    function MarkerNavigationWidget(editor, _themeService) {
        var _this = _super.call(this, editor, { showArrow: true, showFrame: true, isAccessible: true }) || this;
        _this._themeService = _themeService;
        _this._callOnDispose = [];
        _this._onDidSelectRelatedInformation = new Emitter();
        _this.onDidSelectRelatedInformation = _this._onDidSelectRelatedInformation.event;
        _this._severity = MarkerSeverity.Warning;
        _this._backgroundColor = Color.white;
        _this._applyTheme(_themeService.getTheme());
        _this._callOnDispose.push(_themeService.onThemeChange(_this._applyTheme.bind(_this)));
        _this.create();
        return _this;
    }
    MarkerNavigationWidget.prototype._applyTheme = function (theme) {
        this._backgroundColor = theme.getColor(editorMarkerNavigationBackground);
        var colorId = editorMarkerNavigationError;
        if (this._severity === MarkerSeverity.Warning) {
            colorId = editorMarkerNavigationWarning;
        }
        else if (this._severity === MarkerSeverity.Info) {
            colorId = editorMarkerNavigationInfo;
        }
        var frameColor = theme.getColor(colorId);
        this.style({
            arrowColor: frameColor,
            frameColor: frameColor
        }); // style() will trigger _applyStyles
    };
    MarkerNavigationWidget.prototype._applyStyles = function () {
        if (this._parentContainer) {
            this._parentContainer.style.backgroundColor = this._backgroundColor ? this._backgroundColor.toString() : '';
        }
        _super.prototype._applyStyles.call(this);
    };
    MarkerNavigationWidget.prototype.dispose = function () {
        this._callOnDispose = dispose(this._callOnDispose);
        _super.prototype.dispose.call(this);
    };
    MarkerNavigationWidget.prototype.focus = function () {
        this._parentContainer.focus();
    };
    MarkerNavigationWidget.prototype._fillContainer = function (container) {
        var _this = this;
        this._parentContainer = container;
        addClass(container, 'marker-widget');
        this._parentContainer.tabIndex = 0;
        this._parentContainer.setAttribute('role', 'tooltip');
        this._container = document.createElement('div');
        container.appendChild(this._container);
        this._title = document.createElement('div');
        this._title.className = 'block title';
        this._container.appendChild(this._title);
        this._message = new MessageWidget(this._container, this.editor, function (related) { return _this._onDidSelectRelatedInformation.fire(related); });
        this._disposables.push(this._message);
    };
    MarkerNavigationWidget.prototype.show = function (where, heightInLines) {
        throw new Error('call showAtMarker');
    };
    MarkerNavigationWidget.prototype.showAtMarker = function (marker, markerIdx, markerCount) {
        // update:
        // * title
        // * message
        this._container.classList.remove('stale');
        this._title.innerHTML = localize('title.wo_source', "({0}/{1})", markerIdx, markerCount);
        this._message.update(marker);
        // update frame color (only applied on 'show')
        this._severity = marker.severity;
        this._applyTheme(this._themeService.getTheme());
        // show
        var range$$1 = Range.lift(marker);
        var editorPosition = this.editor.getPosition();
        var position = editorPosition && range$$1.containsPosition(editorPosition) ? editorPosition : range$$1.getStartPosition();
        _super.prototype.show.call(this, position, this.computeRequiredHeight());
        this.editor.revealPositionInCenter(position, 0 /* Smooth */);
        if (this.editor.getConfiguration().accessibilitySupport !== 1 /* Disabled */) {
            this.focus();
        }
    };
    MarkerNavigationWidget.prototype.updateMarker = function (marker) {
        this._container.classList.remove('stale');
        this._message.update(marker);
    };
    MarkerNavigationWidget.prototype.showStale = function () {
        this._container.classList.add('stale');
        this._relayout();
    };
    MarkerNavigationWidget.prototype._doLayout = function (heightInPixel, widthInPixel) {
        this._message.layout(heightInPixel, widthInPixel);
        this._container.style.height = heightInPixel + "px";
    };
    MarkerNavigationWidget.prototype._relayout = function () {
        _super.prototype._relayout.call(this, this.computeRequiredHeight());
    };
    MarkerNavigationWidget.prototype.computeRequiredHeight = function () {
        return 1 + this._message.getHeightInLines();
    };
    return MarkerNavigationWidget;
}(ZoneWidget));
// theming
var errorDefault = oneOf(editorErrorForeground, editorErrorBorder);
var warningDefault = oneOf(editorWarningForeground, editorWarningBorder);
var infoDefault = oneOf(editorInfoForeground, editorInfoBorder);
var editorMarkerNavigationError = registerColor('editorMarkerNavigationError.background', { dark: errorDefault, light: errorDefault, hc: errorDefault }, localize('editorMarkerNavigationError', 'Editor marker navigation widget error color.'));
var editorMarkerNavigationWarning = registerColor('editorMarkerNavigationWarning.background', { dark: warningDefault, light: warningDefault, hc: warningDefault }, localize('editorMarkerNavigationWarning', 'Editor marker navigation widget warning color.'));
var editorMarkerNavigationInfo = registerColor('editorMarkerNavigationInfo.background', { dark: infoDefault, light: infoDefault, hc: infoDefault }, localize('editorMarkerNavigationInfo', 'Editor marker navigation widget info color.'));
var editorMarkerNavigationBackground = registerColor('editorMarkerNavigation.background', { dark: '#2D2D30', light: Color.white, hc: '#0C141F' }, localize('editorMarkerNavigationBackground', 'Editor marker navigation widget background.'));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1t = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$8 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$7 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MarkerModel = /** @class */ (function () {
    function MarkerModel(editor, markers) {
        var _this = this;
        this._editor = editor;
        this._markers = null;
        this._nextIdx = -1;
        this._toUnbind = [];
        this._ignoreSelectionChange = false;
        this._onCurrentMarkerChanged = new Emitter();
        this._onMarkerSetChanged = new Emitter();
        this.setMarkers(markers);
        // listen on editor
        this._toUnbind.push(this._editor.onDidDispose(function () { return _this.dispose(); }));
        this._toUnbind.push(this._editor.onDidChangeCursorPosition(function () {
            if (_this._ignoreSelectionChange) {
                return;
            }
            if (_this.currentMarker && Range.containsPosition(_this.currentMarker, _this._editor.getPosition())) {
                return;
            }
            _this._nextIdx = -1;
        }));
    }
    Object.defineProperty(MarkerModel.prototype, "onCurrentMarkerChanged", {
        get: function () {
            return this._onCurrentMarkerChanged.event;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MarkerModel.prototype, "onMarkerSetChanged", {
        get: function () {
            return this._onMarkerSetChanged.event;
        },
        enumerable: true,
        configurable: true
    });
    MarkerModel.prototype.setMarkers = function (markers) {
        var oldMarker = this._nextIdx >= 0 ? this._markers[this._nextIdx] : undefined;
        this._markers = markers || [];
        this._markers.sort(MarkerNavigationAction.compareMarker);
        if (!oldMarker) {
            this._nextIdx = -1;
        }
        else {
            this._nextIdx = Math.max(-1, binarySearch(this._markers, oldMarker, MarkerNavigationAction.compareMarker));
        }
        this._onMarkerSetChanged.fire(this);
    };
    MarkerModel.prototype.withoutWatchingEditorPosition = function (callback) {
        this._ignoreSelectionChange = true;
        try {
            callback();
        }
        finally {
            this._ignoreSelectionChange = false;
        }
    };
    MarkerModel.prototype._initIdx = function (fwd) {
        var found = false;
        var position = this._editor.getPosition();
        for (var i = 0; i < this._markers.length; i++) {
            var range$$1 = Range.lift(this._markers[i]);
            if (range$$1.isEmpty()) {
                var word = this._editor.getModel().getWordAtPosition(range$$1.getStartPosition());
                if (word) {
                    range$$1 = new Range(range$$1.startLineNumber, word.startColumn, range$$1.startLineNumber, word.endColumn);
                }
            }
            if (range$$1.containsPosition(position) || position.isBeforeOrEqual(range$$1.getStartPosition())) {
                this._nextIdx = i;
                found = true;
                break;
            }
        }
        if (!found) {
            // after the last change
            this._nextIdx = fwd ? 0 : this._markers.length - 1;
        }
        if (this._nextIdx < 0) {
            this._nextIdx = this._markers.length - 1;
        }
    };
    Object.defineProperty(MarkerModel.prototype, "currentMarker", {
        get: function () {
            return this.canNavigate() ? this._markers[this._nextIdx] : undefined;
        },
        enumerable: true,
        configurable: true
    });
    MarkerModel.prototype.move = function (fwd, inCircles) {
        if (!this.canNavigate()) {
            this._onCurrentMarkerChanged.fire(undefined);
            return !inCircles;
        }
        var oldIdx = this._nextIdx;
        var atEdge = false;
        if (this._nextIdx === -1) {
            this._initIdx(fwd);
        }
        else if (fwd) {
            if (inCircles || this._nextIdx + 1 < this._markers.length) {
                this._nextIdx = (this._nextIdx + 1) % this._markers.length;
            }
            else {
                atEdge = true;
            }
        }
        else if (!fwd) {
            if (inCircles || this._nextIdx > 0) {
                this._nextIdx = (this._nextIdx - 1 + this._markers.length) % this._markers.length;
            }
            else {
                atEdge = true;
            }
        }
        if (oldIdx !== this._nextIdx) {
            var marker = this._markers[this._nextIdx];
            this._onCurrentMarkerChanged.fire(marker);
        }
        return atEdge;
    };
    MarkerModel.prototype.canNavigate = function () {
        return this._markers.length > 0;
    };
    MarkerModel.prototype.findMarkerAtPosition = function (pos) {
        for (var _i = 0, _a = this._markers; _i < _a.length; _i++) {
            var marker = _a[_i];
            if (Range.containsPosition(marker, pos)) {
                return marker;
            }
        }
        return undefined;
    };
    Object.defineProperty(MarkerModel.prototype, "total", {
        get: function () {
            return this._markers.length;
        },
        enumerable: true,
        configurable: true
    });
    MarkerModel.prototype.indexOf = function (marker) {
        return 1 + this._markers.indexOf(marker);
    };
    MarkerModel.prototype.dispose = function () {
        this._toUnbind = dispose(this._toUnbind);
    };
    return MarkerModel;
}());
var MarkerController = /** @class */ (function () {
    function MarkerController(editor, _markerService, _contextKeyService, _themeService, _editorService) {
        this._markerService = _markerService;
        this._contextKeyService = _contextKeyService;
        this._themeService = _themeService;
        this._editorService = _editorService;
        this._disposeOnClose = [];
        this._editor = editor;
        this._widgetVisible = CONTEXT_MARKERS_NAVIGATION_VISIBLE.bindTo(this._contextKeyService);
    }
    MarkerController.get = function (editor) {
        return editor.getContribution(MarkerController.ID);
    };
    MarkerController.prototype.getId = function () {
        return MarkerController.ID;
    };
    MarkerController.prototype.dispose = function () {
        this._cleanUp();
    };
    MarkerController.prototype._cleanUp = function () {
        this._widgetVisible.reset();
        this._disposeOnClose = dispose(this._disposeOnClose);
        this._widget = null;
        this._model = null;
    };
    MarkerController.prototype.getOrCreateModel = function () {
        var _this = this;
        if (this._model) {
            return this._model;
        }
        var markers = this._getMarkers();
        this._model = new MarkerModel(this._editor, markers);
        this._markerService.onMarkerChanged(this._onMarkerChanged, this, this._disposeOnClose);
        this._widget = new MarkerNavigationWidget(this._editor, this._themeService);
        this._widgetVisible.set(true);
        this._disposeOnClose.push(this._model);
        this._disposeOnClose.push(this._widget);
        this._disposeOnClose.push(this._widget.onDidSelectRelatedInformation(function (related) {
            _this._editorService.openCodeEditor({
                resource: related.resource,
                options: { pinned: true, revealIfOpened: true, selection: Range.lift(related).collapseToStart() }
            }, _this._editor).then(undefined, onUnexpectedError);
            _this.closeMarkersNavigation(false);
        }));
        this._disposeOnClose.push(this._editor.onDidChangeModel(function () { return _this._cleanUp(); }));
        this._disposeOnClose.push(this._model.onCurrentMarkerChanged(function (marker) {
            if (!marker) {
                _this._cleanUp();
            }
            else {
                _this._model.withoutWatchingEditorPosition(function () {
                    _this._widget.showAtMarker(marker, _this._model.indexOf(marker), _this._model.total);
                });
            }
        }));
        this._disposeOnClose.push(this._model.onMarkerSetChanged(function () {
            var marker = _this._model.findMarkerAtPosition(_this._widget.position);
            if (marker) {
                _this._widget.updateMarker(marker);
            }
            else {
                _this._widget.showStale();
            }
        }));
        return this._model;
    };
    MarkerController.prototype.closeMarkersNavigation = function (focusEditor) {
        if (focusEditor === void 0) { focusEditor = true; }
        this._cleanUp();
        if (focusEditor) {
            this._editor.focus();
        }
    };
    MarkerController.prototype._onMarkerChanged = function (changedResources) {
        var _this = this;
        if (!changedResources.some(function (r) { return _this._editor.getModel().uri.toString() === r.toString(); })) {
            return;
        }
        this._model.setMarkers(this._getMarkers());
    };
    MarkerController.prototype._getMarkers = function () {
        return this._markerService.read({
            resource: this._editor.getModel().uri,
            severities: MarkerSeverity.Error | MarkerSeverity.Warning | MarkerSeverity.Info
        });
    };
    MarkerController.ID = 'editor.contrib.markerController';
    MarkerController = __decorate$8([
        __param$7(1, IMarkerService),
        __param$7(2, IContextKeyService),
        __param$7(3, IThemeService),
        __param$7(4, ICodeEditorService)
    ], MarkerController);
    return MarkerController;
}());
var MarkerNavigationAction = /** @class */ (function (_super) {
    __extends$1t(MarkerNavigationAction, _super);
    function MarkerNavigationAction(next, multiFile, opts) {
        var _this = _super.call(this, opts) || this;
        _this._isNext = next;
        _this._multiFile = multiFile;
        return _this;
    }
    MarkerNavigationAction.prototype.run = function (accessor, editor) {
        var _this = this;
        var markerService = accessor.get(IMarkerService);
        var editorService = accessor.get(ICodeEditorService);
        var controller = MarkerController.get(editor);
        if (!controller) {
            return undefined;
        }
        var model = controller.getOrCreateModel();
        var atEdge = model.move(this._isNext, !this._multiFile);
        if (!atEdge || !this._multiFile) {
            return undefined;
        }
        // try with the next/prev file
        var markers = markerService.read({ severities: MarkerSeverity.Error | MarkerSeverity.Warning | MarkerSeverity.Info }).sort(MarkerNavigationAction.compareMarker);
        if (markers.length === 0) {
            return undefined;
        }
        var oldMarker = model.currentMarker || { resource: editor.getModel().uri, severity: MarkerSeverity.Error, startLineNumber: 1, startColumn: 1, endLineNumber: 1, endColumn: 1 };
        var idx = binarySearch(markers, oldMarker, MarkerNavigationAction.compareMarker);
        if (idx < 0) {
            // find best match...
            idx = ~idx;
            idx %= markers.length;
        }
        else if (this._isNext) {
            idx = (idx + 1) % markers.length;
        }
        else {
            idx = (idx + markers.length - 1) % markers.length;
        }
        var newMarker = markers[idx];
        if (newMarker.resource.toString() === editor.getModel().uri.toString()) {
            // the next `resource` is this resource which
            // means we cycle within this file
            model.move(this._isNext, true);
            return undefined;
        }
        // close the widget for this editor-instance, open the resource
        // for the next marker and re-start marker navigation in there
        controller.closeMarkersNavigation();
        return editorService.openCodeEditor({
            resource: newMarker.resource,
            options: { pinned: false, revealIfOpened: true, revealInCenterIfOutsideViewport: true, selection: newMarker }
        }, editor).then(function (editor) {
            if (!editor) {
                return undefined;
            }
            return editor.getAction(_this.id).run();
        });
    };
    MarkerNavigationAction.compareMarker = function (a, b) {
        var res = compare(a.resource.toString(), b.resource.toString());
        if (res === 0) {
            res = MarkerSeverity.compare(a.severity, b.severity);
        }
        if (res === 0) {
            res = Range.compareRangesUsingStarts(a, b);
        }
        return res;
    };
    return MarkerNavigationAction;
}(EditorAction));
var NextMarkerAction = /** @class */ (function (_super) {
    __extends$1t(NextMarkerAction, _super);
    function NextMarkerAction() {
        return _super.call(this, true, false, {
            id: 'editor.action.marker.next',
            label: localize('markerAction.next.label', "Go to Next Problem (Error, Warning, Info)"),
            alias: 'Go to Next Error or Warning',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return NextMarkerAction;
}(MarkerNavigationAction));
var PrevMarkerAction = /** @class */ (function (_super) {
    __extends$1t(PrevMarkerAction, _super);
    function PrevMarkerAction() {
        return _super.call(this, false, false, {
            id: 'editor.action.marker.prev',
            label: localize('markerAction.previous.label', "Go to Previous Problem (Error, Warning, Info)"),
            alias: 'Go to Previous Error or Warning',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return PrevMarkerAction;
}(MarkerNavigationAction));
var NextMarkerInFilesAction = /** @class */ (function (_super) {
    __extends$1t(NextMarkerInFilesAction, _super);
    function NextMarkerInFilesAction() {
        return _super.call(this, true, true, {
            id: 'editor.action.marker.nextInFiles',
            label: localize('markerAction.nextInFiles.label', "Go to Next Problem in Files (Error, Warning, Info)"),
            alias: 'Go to Next Error or Warning in Files',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 66 /* F8 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return NextMarkerInFilesAction;
}(MarkerNavigationAction));
var PrevMarkerInFilesAction = /** @class */ (function (_super) {
    __extends$1t(PrevMarkerInFilesAction, _super);
    function PrevMarkerInFilesAction() {
        return _super.call(this, false, true, {
            id: 'editor.action.marker.prevInFiles',
            label: localize('markerAction.previousInFiles.label', "Go to Previous Problem in Files (Error, Warning, Info)"),
            alias: 'Go to Previous Error or Warning in Files',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 1024 /* Shift */ | 66 /* F8 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return PrevMarkerInFilesAction;
}(MarkerNavigationAction));
registerEditorContribution(MarkerController);
registerEditorAction(NextMarkerAction);
registerEditorAction(PrevMarkerAction);
registerEditorAction(NextMarkerInFilesAction);
registerEditorAction(PrevMarkerInFilesAction);
var CONTEXT_MARKERS_NAVIGATION_VISIBLE = new RawContextKey('markersNavigationVisible', false);
var MarkerCommand = EditorCommand.bindToContribution(MarkerController.get);
registerEditorCommand(new MarkerCommand({
    id: 'closeMarkersNavigation',
    precondition: CONTEXT_MARKERS_NAVIGATION_VISIBLE,
    handler: function (x) { return x.closeMarkersNavigation(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 50,
        kbExpr: EditorContextKeys.focus,
        primary: 9 /* Escape */,
        secondary: [1024 /* Shift */ | 9 /* Escape */]
    }
}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IModeService = createDecorator('modeService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getColors(model, token) {
    var colors = [];
    var providers = ColorProviderRegistry.ordered(model).reverse();
    var promises = providers.map(function (provider) { return Promise.resolve(provider.provideDocumentColors(model, token)).then(function (result) {
        if (Array.isArray(result)) {
            for (var _i = 0, result_1 = result; _i < result_1.length; _i++) {
                var colorInfo = result_1[_i];
                colors.push({ colorInfo: colorInfo, provider: provider });
            }
        }
    }); });
    return Promise.all(promises).then(function () { return colors; });
}
function getColorPresentations(model, colorInfo, provider, token) {
    return Promise.resolve(provider.provideColorPresentations(model, colorInfo, token));
}
registerLanguageCommand('_executeDocumentColorProvider', function (accessor, args) {
    var resource = args.resource;
    if (!(resource instanceof URI)) {
        throw illegalArgument();
    }
    var model = accessor.get(IModelService).getModel(resource);
    if (!model) {
        throw illegalArgument();
    }
    var rawCIs = [];
    var providers = ColorProviderRegistry.ordered(model).reverse();
    var promises = providers.map(function (provider) { return Promise.resolve(provider.provideDocumentColors(model, CancellationToken.None)).then(function (result) {
        if (Array.isArray(result)) {
            for (var _i = 0, result_2 = result; _i < result_2.length; _i++) {
                var ci = result_2[_i];
                rawCIs.push({ range: ci.range, color: [ci.color.red, ci.color.green, ci.color.blue, ci.color.alpha] });
            }
        }
    }); });
    return Promise.all(promises).then(function () { return rawCIs; });
});
registerLanguageCommand('_executeColorPresentationProvider', function (accessor, args) {
    var resource = args.resource, color = args.color, range = args.range;
    if (!(resource instanceof URI) || !Array.isArray(color) || color.length !== 4 || !Range.isIRange(range)) {
        throw illegalArgument();
    }
    var red = color[0], green = color[1], blue = color[2], alpha = color[3];
    var model = accessor.get(IModelService).getModel(resource);
    if (!model) {
        throw illegalArgument();
    }
    var colorInfo = {
        range: range,
        color: { red: red, green: green, blue: blue, alpha: alpha }
    };
    var presentations = [];
    var providers = ColorProviderRegistry.ordered(model).reverse();
    var promises = providers.map(function (provider) { return Promise.resolve(provider.provideColorPresentations(model, colorInfo, CancellationToken.None)).then(function (result) {
        if (Array.isArray(result)) {
            presentations.push.apply(presentations, result);
        }
    }); });
    return Promise.all(promises).then(function () { return presentations; });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Return a hash value for an object.
 */
function hash(obj, hashVal) {
    if (hashVal === void 0) { hashVal = 0; }
    switch (typeof obj) {
        case 'object':
            if (obj === null) {
                return numberHash(349, hashVal);
            }
            else if (Array.isArray(obj)) {
                return arrayHash(obj, hashVal);
            }
            return objectHash(obj, hashVal);
        case 'string':
            return stringHash(obj, hashVal);
        case 'boolean':
            return booleanHash(obj, hashVal);
        case 'number':
            return numberHash(obj, hashVal);
        case 'undefined':
            return numberHash(0, 937);
        default:
            return numberHash(0, 617);
    }
}
function numberHash(val, initialHashVal) {
    return (((initialHashVal << 5) - initialHashVal) + val) | 0; // hashVal * 31 + ch, keep as int32
}
function booleanHash(b, initialHashVal) {
    return numberHash(b ? 433 : 863, initialHashVal);
}
function stringHash(s, hashVal) {
    hashVal = numberHash(149417, hashVal);
    for (var i = 0, length_1 = s.length; i < length_1; i++) {
        hashVal = numberHash(s.charCodeAt(i), hashVal);
    }
    return hashVal;
}
function arrayHash(arr, initialHashVal) {
    initialHashVal = numberHash(104579, initialHashVal);
    return arr.reduce(function (hashVal, item) { return hash(item, hashVal); }, initialHashVal);
}
function objectHash(obj, initialHashVal) {
    initialHashVal = numberHash(181387, initialHashVal);
    return Object.keys(obj).sort().reduce(function (hashVal, key) {
        hashVal = stringHash(key, hashVal);
        return hash(obj[key], hashVal);
    }, initialHashVal);
}

var IConfigurationService = createDecorator('configurationService');
function toValuesTree(properties, conflictReporter) {
    var root = Object.create(null);
    for (var key in properties) {
        addToValueTree(root, key, properties[key], conflictReporter);
    }
    return root;
}
function addToValueTree(settingsTreeRoot, key, value, conflictReporter) {
    var segments = key.split('.');
    var last = segments.pop();
    var curr = settingsTreeRoot;
    for (var i = 0; i < segments.length; i++) {
        var s = segments[i];
        var obj = curr[s];
        switch (typeof obj) {
            case 'undefined':
                obj = curr[s] = Object.create(null);
                break;
            case 'object':
                break;
            default:
                conflictReporter("Ignoring " + key + " as " + segments.slice(0, i + 1).join('.') + " is " + JSON.stringify(obj));
                return;
        }
        curr = obj;
    }
    if (typeof curr === 'object') {
        curr[last] = value; // workaround https://github.com/Microsoft/vscode/issues/13606
    }
    else {
        conflictReporter("Ignoring " + key + " as " + segments.join('.') + " is " + JSON.stringify(curr));
    }
}
function removeFromValueTree(valueTree, key) {
    var segments = key.split('.');
    doRemoveFromValueTree(valueTree, segments);
}
function doRemoveFromValueTree(valueTree, segments) {
    var first = segments.shift();
    if (segments.length === 0) {
        // Reached last segment
        delete valueTree[first];
        return;
    }
    if (Object.keys(valueTree).indexOf(first) !== -1) {
        var value = valueTree[first];
        if (typeof value === 'object' && !Array.isArray(value)) {
            doRemoveFromValueTree(value, segments);
            if (Object.keys(value).length === 0) {
                delete valueTree[first];
            }
        }
    }
}
/**
 * A helper function to get the configuration value with a specific settings path (e.g. config.some.setting)
 */
function getConfigurationValue(config, settingPath, defaultValue) {
    function accessSetting(config, path) {
        var current = config;
        for (var i = 0; i < path.length; i++) {
            if (typeof current !== 'object' || current === null) {
                return undefined;
            }
            current = current[path[i]];
        }
        return current;
    }
    var path = settingPath.split('.');
    var result = accessSetting(config, path);
    return typeof result === 'undefined' ? defaultValue : result;
}
function getConfigurationKeys() {
    var properties = Registry.as(Extensions$3.Configuration).getConfigurationProperties();
    return Object.keys(properties);
}
function getDefaultValues() {
    var valueTreeRoot = Object.create(null);
    var properties = Registry.as(Extensions$3.Configuration).getConfigurationProperties();
    for (var key in properties) {
        var value = properties[key].default;
        addToValueTree(valueTreeRoot, key, value, function (message) { return console.error("Conflict in default settings: " + message); });
    }
    return valueTreeRoot;
}
function overrideIdentifierFromKey(key) {
    return key.substring(1, key.length - 1);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$9 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$8 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MAX_DECORATORS = 500;
var ColorDetector = /** @class */ (function () {
    function ColorDetector(_editor, _codeEditorService, _configurationService) {
        var _this = this;
        this._editor = _editor;
        this._codeEditorService = _codeEditorService;
        this._configurationService = _configurationService;
        this._globalToDispose = [];
        this._localToDispose = [];
        this._decorationsIds = [];
        this._colorDatas = new Map();
        this._colorDecoratorIds = [];
        this._decorationsTypes = {};
        this._globalToDispose.push(_editor.onDidChangeModel(function (e) {
            _this._isEnabled = _this.isEnabled();
            _this.onModelChanged();
        }));
        this._globalToDispose.push(_editor.onDidChangeModelLanguage(function (e) { return _this.onModelChanged(); }));
        this._globalToDispose.push(ColorProviderRegistry.onDidChange(function (e) { return _this.onModelChanged(); }));
        this._globalToDispose.push(_editor.onDidChangeConfiguration(function (e) {
            var prevIsEnabled = _this._isEnabled;
            _this._isEnabled = _this.isEnabled();
            if (prevIsEnabled !== _this._isEnabled) {
                if (_this._isEnabled) {
                    _this.onModelChanged();
                }
                else {
                    _this.removeAllDecorations();
                }
            }
        }));
        this._timeoutTimer = null;
        this._computePromise = null;
        this._isEnabled = this.isEnabled();
        this.onModelChanged();
    }
    ColorDetector.prototype.isEnabled = function () {
        var model = this._editor.getModel();
        if (!model) {
            return false;
        }
        var languageId = model.getLanguageIdentifier();
        // handle deprecated settings. [languageId].colorDecorators.enable
        var deprecatedConfig = this._configurationService.getValue(languageId.language);
        if (deprecatedConfig) {
            var colorDecorators = deprecatedConfig['colorDecorators']; // deprecatedConfig.valueOf('.colorDecorators.enable');
            if (colorDecorators && colorDecorators['enable'] !== undefined && !colorDecorators['enable']) {
                return colorDecorators['enable'];
            }
        }
        return this._editor.getConfiguration().contribInfo.colorDecorators;
    };
    ColorDetector.prototype.getId = function () {
        return ColorDetector.ID;
    };
    ColorDetector.get = function (editor) {
        return editor.getContribution(this.ID);
    };
    ColorDetector.prototype.dispose = function () {
        this.stop();
        this.removeAllDecorations();
        this._globalToDispose = dispose(this._globalToDispose);
    };
    ColorDetector.prototype.onModelChanged = function () {
        var _this = this;
        this.stop();
        if (!this._isEnabled) {
            return;
        }
        var model = this._editor.getModel();
        if (!model || !ColorProviderRegistry.has(model)) {
            return;
        }
        this._localToDispose.push(this._editor.onDidChangeModelContent(function (e) {
            if (!_this._timeoutTimer) {
                _this._timeoutTimer = new TimeoutTimer();
                _this._timeoutTimer.cancelAndSet(function () {
                    _this._timeoutTimer = null;
                    _this.beginCompute();
                }, ColorDetector.RECOMPUTE_TIME);
            }
        }));
        this.beginCompute();
    };
    ColorDetector.prototype.beginCompute = function () {
        var _this = this;
        this._computePromise = createCancelablePromise(function (token) {
            var model = _this._editor.getModel();
            if (!model) {
                return Promise.resolve([]);
            }
            return getColors(model, token);
        });
        this._computePromise.then(function (colorInfos) {
            _this.updateDecorations(colorInfos);
            _this.updateColorDecorators(colorInfos);
            _this._computePromise = null;
        }, onUnexpectedError);
    };
    ColorDetector.prototype.stop = function () {
        if (this._timeoutTimer) {
            this._timeoutTimer.cancel();
            this._timeoutTimer = null;
        }
        if (this._computePromise) {
            this._computePromise.cancel();
            this._computePromise = null;
        }
        this._localToDispose = dispose(this._localToDispose);
    };
    ColorDetector.prototype.updateDecorations = function (colorDatas) {
        var _this = this;
        var decorations = colorDatas.map(function (c) { return ({
            range: {
                startLineNumber: c.colorInfo.range.startLineNumber,
                startColumn: c.colorInfo.range.startColumn,
                endLineNumber: c.colorInfo.range.endLineNumber,
                endColumn: c.colorInfo.range.endColumn
            },
            options: ModelDecorationOptions.EMPTY
        }); });
        this._decorationsIds = this._editor.deltaDecorations(this._decorationsIds, decorations);
        this._colorDatas = new Map();
        this._decorationsIds.forEach(function (id, i) { return _this._colorDatas.set(id, colorDatas[i]); });
    };
    ColorDetector.prototype.updateColorDecorators = function (colorData) {
        var decorations = [];
        var newDecorationsTypes = {};
        for (var i = 0; i < colorData.length && decorations.length < MAX_DECORATORS; i++) {
            var _a = colorData[i].colorInfo.color, red = _a.red, green = _a.green, blue = _a.blue, alpha = _a.alpha;
            var rgba = new RGBA(Math.round(red * 255), Math.round(green * 255), Math.round(blue * 255), alpha);
            var subKey = hash(rgba).toString(16);
            var color = "rgba(" + rgba.r + ", " + rgba.g + ", " + rgba.b + ", " + rgba.a + ")";
            var key = 'colorBox-' + subKey;
            if (!this._decorationsTypes[key] && !newDecorationsTypes[key]) {
                this._codeEditorService.registerDecorationType(key, {
                    before: {
                        contentText: ' ',
                        border: 'solid 0.1em #000',
                        margin: '0.1em 0.2em 0 0.2em',
                        width: '0.8em',
                        height: '0.8em',
                        backgroundColor: color
                    },
                    dark: {
                        before: {
                            border: 'solid 0.1em #eee'
                        }
                    }
                });
            }
            newDecorationsTypes[key] = true;
            decorations.push({
                range: {
                    startLineNumber: colorData[i].colorInfo.range.startLineNumber,
                    startColumn: colorData[i].colorInfo.range.startColumn,
                    endLineNumber: colorData[i].colorInfo.range.endLineNumber,
                    endColumn: colorData[i].colorInfo.range.endColumn
                },
                options: this._codeEditorService.resolveDecorationOptions(key, true)
            });
        }
        for (var subType in this._decorationsTypes) {
            if (!newDecorationsTypes[subType]) {
                this._codeEditorService.removeDecorationType(subType);
            }
        }
        this._colorDecoratorIds = this._editor.deltaDecorations(this._colorDecoratorIds, decorations);
    };
    ColorDetector.prototype.removeAllDecorations = function () {
        this._decorationsIds = this._editor.deltaDecorations(this._decorationsIds, []);
        this._colorDecoratorIds = this._editor.deltaDecorations(this._colorDecoratorIds, []);
        for (var subType in this._decorationsTypes) {
            this._codeEditorService.removeDecorationType(subType);
        }
    };
    ColorDetector.prototype.getColorData = function (position) {
        var _this = this;
        var model = this._editor.getModel();
        if (!model) {
            return null;
        }
        var decorations = model
            .getDecorationsInRange(Range.fromPositions(position, position))
            .filter(function (d) { return _this._colorDatas.has(d.id); });
        if (decorations.length === 0) {
            return null;
        }
        return this._colorDatas.get(decorations[0].id);
    };
    ColorDetector.ID = 'editor.contrib.colorDetector';
    ColorDetector.RECOMPUTE_TIME = 1000; // ms
    ColorDetector = __decorate$9([
        __param$8(1, ICodeEditorService),
        __param$8(2, IConfigurationService)
    ], ColorDetector);
    return ColorDetector;
}());
registerEditorContribution(ColorDetector);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ColorPickerModel = /** @class */ (function () {
    function ColorPickerModel(color, availableColorPresentations, presentationIndex) {
        this.presentationIndex = presentationIndex;
        this._onColorFlushed = new Emitter();
        this.onColorFlushed = this._onColorFlushed.event;
        this._onDidChangeColor = new Emitter();
        this.onDidChangeColor = this._onDidChangeColor.event;
        this._onDidChangePresentation = new Emitter();
        this.onDidChangePresentation = this._onDidChangePresentation.event;
        this.originalColor = color;
        this._color = color;
        this._colorPresentations = availableColorPresentations;
    }
    Object.defineProperty(ColorPickerModel.prototype, "color", {
        get: function () {
            return this._color;
        },
        set: function (color) {
            if (this._color.equals(color)) {
                return;
            }
            this._color = color;
            this._onDidChangeColor.fire(color);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ColorPickerModel.prototype, "presentation", {
        get: function () { return this.colorPresentations[this.presentationIndex]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ColorPickerModel.prototype, "colorPresentations", {
        get: function () {
            return this._colorPresentations;
        },
        set: function (colorPresentations) {
            this._colorPresentations = colorPresentations;
            if (this.presentationIndex > colorPresentations.length - 1) {
                this.presentationIndex = 0;
            }
            this._onDidChangePresentation.fire(this.presentation);
        },
        enumerable: true,
        configurable: true
    });
    ColorPickerModel.prototype.selectNextColorPresentation = function () {
        this.presentationIndex = (this.presentationIndex + 1) % this.colorPresentations.length;
        this.flushColor();
        this._onDidChangePresentation.fire(this.presentation);
    };
    ColorPickerModel.prototype.guessColorPresentation = function (color, originalText) {
        for (var i = 0; i < this.colorPresentations.length; i++) {
            if (originalText === this.colorPresentations[i].label) {
                this.presentationIndex = i;
                this._onDidChangePresentation.fire(this.presentation);
                break;
            }
        }
    };
    ColorPickerModel.prototype.flushColor = function () {
        this._onColorFlushed.fire(this._color);
    };
    return ColorPickerModel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1u = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var $$2 = $;
var ColorPickerHeader = /** @class */ (function (_super) {
    __extends$1u(ColorPickerHeader, _super);
    function ColorPickerHeader(container, model, themeService) {
        var _this = _super.call(this) || this;
        _this.model = model;
        _this.domNode = $$2('.colorpicker-header');
        append(container, _this.domNode);
        _this.pickedColorNode = append(_this.domNode, $$2('.picked-color'));
        var colorBox = append(_this.domNode, $$2('.original-color'));
        colorBox.style.backgroundColor = Color.Format.CSS.format(_this.model.originalColor);
        _this.backgroundColor = themeService.getTheme().getColor(editorHoverBackground) || Color.white;
        _this._register(registerThemingParticipant(function (theme, collector) {
            _this.backgroundColor = theme.getColor(editorHoverBackground) || Color.white;
        }));
        _this._register(addDisposableListener(_this.pickedColorNode, EventType.CLICK, function () { return _this.model.selectNextColorPresentation(); }));
        _this._register(addDisposableListener(colorBox, EventType.CLICK, function () {
            _this.model.color = _this.model.originalColor;
            _this.model.flushColor();
        }));
        _this._register(model.onDidChangeColor(_this.onDidChangeColor, _this));
        _this._register(model.onDidChangePresentation(_this.onDidChangePresentation, _this));
        _this.pickedColorNode.style.backgroundColor = Color.Format.CSS.format(model.color);
        toggleClass(_this.pickedColorNode, 'light', model.color.rgba.a < 0.5 ? _this.backgroundColor.isLighter() : model.color.isLighter());
        return _this;
    }
    ColorPickerHeader.prototype.onDidChangeColor = function (color) {
        this.pickedColorNode.style.backgroundColor = Color.Format.CSS.format(color);
        toggleClass(this.pickedColorNode, 'light', color.rgba.a < 0.5 ? this.backgroundColor.isLighter() : color.isLighter());
        this.onDidChangePresentation();
    };
    ColorPickerHeader.prototype.onDidChangePresentation = function () {
        this.pickedColorNode.textContent = this.model.presentation ? this.model.presentation.label : '';
    };
    return ColorPickerHeader;
}(Disposable));
var ColorPickerBody = /** @class */ (function (_super) {
    __extends$1u(ColorPickerBody, _super);
    function ColorPickerBody(container, model, pixelRatio) {
        var _this = _super.call(this) || this;
        _this.model = model;
        _this.pixelRatio = pixelRatio;
        _this.domNode = $$2('.colorpicker-body');
        append(container, _this.domNode);
        _this.saturationBox = new SaturationBox(_this.domNode, _this.model, _this.pixelRatio);
        _this._register(_this.saturationBox);
        _this._register(_this.saturationBox.onDidChange(_this.onDidSaturationValueChange, _this));
        _this._register(_this.saturationBox.onColorFlushed(_this.flushColor, _this));
        _this.opacityStrip = new OpacityStrip(_this.domNode, _this.model);
        _this._register(_this.opacityStrip);
        _this._register(_this.opacityStrip.onDidChange(_this.onDidOpacityChange, _this));
        _this._register(_this.opacityStrip.onColorFlushed(_this.flushColor, _this));
        _this.hueStrip = new HueStrip(_this.domNode, _this.model);
        _this._register(_this.hueStrip);
        _this._register(_this.hueStrip.onDidChange(_this.onDidHueChange, _this));
        _this._register(_this.hueStrip.onColorFlushed(_this.flushColor, _this));
        return _this;
    }
    ColorPickerBody.prototype.flushColor = function () {
        this.model.flushColor();
    };
    ColorPickerBody.prototype.onDidSaturationValueChange = function (_a) {
        var s = _a.s, v = _a.v;
        var hsva = this.model.color.hsva;
        this.model.color = new Color(new HSVA(hsva.h, s, v, hsva.a));
    };
    ColorPickerBody.prototype.onDidOpacityChange = function (a) {
        var hsva = this.model.color.hsva;
        this.model.color = new Color(new HSVA(hsva.h, hsva.s, hsva.v, a));
    };
    ColorPickerBody.prototype.onDidHueChange = function (value) {
        var hsva = this.model.color.hsva;
        var h = (1 - value) * 360;
        this.model.color = new Color(new HSVA(h === 360 ? 0 : h, hsva.s, hsva.v, hsva.a));
    };
    ColorPickerBody.prototype.layout = function () {
        this.saturationBox.layout();
        this.opacityStrip.layout();
        this.hueStrip.layout();
    };
    return ColorPickerBody;
}(Disposable));
var SaturationBox = /** @class */ (function (_super) {
    __extends$1u(SaturationBox, _super);
    function SaturationBox(container, model, pixelRatio) {
        var _this = _super.call(this) || this;
        _this.model = model;
        _this.pixelRatio = pixelRatio;
        _this._onDidChange = new Emitter();
        _this.onDidChange = _this._onDidChange.event;
        _this._onColorFlushed = new Emitter();
        _this.onColorFlushed = _this._onColorFlushed.event;
        _this.domNode = $$2('.saturation-wrap');
        append(container, _this.domNode);
        // Create canvas, draw selected color
        _this.canvas = document.createElement('canvas');
        _this.canvas.className = 'saturation-box';
        append(_this.domNode, _this.canvas);
        // Add selection circle
        _this.selection = $$2('.saturation-selection');
        append(_this.domNode, _this.selection);
        _this.layout();
        _this._register(addDisposableListener(_this.domNode, EventType.MOUSE_DOWN, function (e) { return _this.onMouseDown(e); }));
        _this._register(_this.model.onDidChangeColor(_this.onDidChangeColor, _this));
        _this.monitor = null;
        return _this;
    }
    SaturationBox.prototype.onMouseDown = function (e) {
        var _this = this;
        this.monitor = this._register(new GlobalMouseMoveMonitor());
        var origin = getDomNodePagePosition(this.domNode);
        if (e.target !== this.selection) {
            this.onDidChangePosition(e.offsetX, e.offsetY);
        }
        this.monitor.startMonitoring(standardMouseMoveMerger, function (event) { return _this.onDidChangePosition(event.posx - origin.left, event.posy - origin.top); }, function () { return null; });
        var mouseUpListener = addDisposableListener(document, EventType.MOUSE_UP, function () {
            _this._onColorFlushed.fire();
            mouseUpListener.dispose();
            _this.monitor.stopMonitoring(true);
            _this.monitor = null;
        }, true);
    };
    SaturationBox.prototype.onDidChangePosition = function (left, top) {
        var s = Math.max(0, Math.min(1, left / this.width));
        var v = Math.max(0, Math.min(1, 1 - (top / this.height)));
        this.paintSelection(s, v);
        this._onDidChange.fire({ s: s, v: v });
    };
    SaturationBox.prototype.layout = function () {
        this.width = this.domNode.offsetWidth;
        this.height = this.domNode.offsetHeight;
        this.canvas.width = this.width * this.pixelRatio;
        this.canvas.height = this.height * this.pixelRatio;
        this.paint();
        var hsva = this.model.color.hsva;
        this.paintSelection(hsva.s, hsva.v);
    };
    SaturationBox.prototype.paint = function () {
        var hsva = this.model.color.hsva;
        var saturatedColor = new Color(new HSVA(hsva.h, 1, 1, 1));
        var ctx = this.canvas.getContext('2d');
        var whiteGradient = ctx.createLinearGradient(0, 0, this.canvas.width, 0);
        whiteGradient.addColorStop(0, 'rgba(255, 255, 255, 1)');
        whiteGradient.addColorStop(0.5, 'rgba(255, 255, 255, 0.5)');
        whiteGradient.addColorStop(1, 'rgba(255, 255, 255, 0)');
        var blackGradient = ctx.createLinearGradient(0, 0, 0, this.canvas.height);
        blackGradient.addColorStop(0, 'rgba(0, 0, 0, 0)');
        blackGradient.addColorStop(1, 'rgba(0, 0, 0, 1)');
        ctx.rect(0, 0, this.canvas.width, this.canvas.height);
        ctx.fillStyle = Color.Format.CSS.format(saturatedColor);
        ctx.fill();
        ctx.fillStyle = whiteGradient;
        ctx.fill();
        ctx.fillStyle = blackGradient;
        ctx.fill();
    };
    SaturationBox.prototype.paintSelection = function (s, v) {
        this.selection.style.left = s * this.width + "px";
        this.selection.style.top = this.height - v * this.height + "px";
    };
    SaturationBox.prototype.onDidChangeColor = function () {
        if (this.monitor && this.monitor.isMonitoring()) {
            return;
        }
        this.paint();
    };
    return SaturationBox;
}(Disposable));
var Strip = /** @class */ (function (_super) {
    __extends$1u(Strip, _super);
    function Strip(container, model) {
        var _this = _super.call(this) || this;
        _this.model = model;
        _this._onDidChange = new Emitter();
        _this.onDidChange = _this._onDidChange.event;
        _this._onColorFlushed = new Emitter();
        _this.onColorFlushed = _this._onColorFlushed.event;
        _this.domNode = append(container, $$2('.strip'));
        _this.overlay = append(_this.domNode, $$2('.overlay'));
        _this.slider = append(_this.domNode, $$2('.slider'));
        _this.slider.style.top = "0px";
        _this._register(addDisposableListener(_this.domNode, EventType.MOUSE_DOWN, function (e) { return _this.onMouseDown(e); }));
        _this.layout();
        return _this;
    }
    Strip.prototype.layout = function () {
        this.height = this.domNode.offsetHeight - this.slider.offsetHeight;
        var value = this.getValue(this.model.color);
        this.updateSliderPosition(value);
    };
    Strip.prototype.onMouseDown = function (e) {
        var _this = this;
        var monitor = this._register(new GlobalMouseMoveMonitor());
        var origin = getDomNodePagePosition(this.domNode);
        addClass(this.domNode, 'grabbing');
        if (e.target !== this.slider) {
            this.onDidChangeTop(e.offsetY);
        }
        monitor.startMonitoring(standardMouseMoveMerger, function (event) { return _this.onDidChangeTop(event.posy - origin.top); }, function () { return null; });
        var mouseUpListener = addDisposableListener(document, EventType.MOUSE_UP, function () {
            _this._onColorFlushed.fire();
            mouseUpListener.dispose();
            monitor.stopMonitoring(true);
            removeClass(_this.domNode, 'grabbing');
        }, true);
    };
    Strip.prototype.onDidChangeTop = function (top) {
        var value = Math.max(0, Math.min(1, 1 - (top / this.height)));
        this.updateSliderPosition(value);
        this._onDidChange.fire(value);
    };
    Strip.prototype.updateSliderPosition = function (value) {
        this.slider.style.top = (1 - value) * this.height + "px";
    };
    return Strip;
}(Disposable));
var OpacityStrip = /** @class */ (function (_super) {
    __extends$1u(OpacityStrip, _super);
    function OpacityStrip(container, model) {
        var _this = _super.call(this, container, model) || this;
        addClass(_this.domNode, 'opacity-strip');
        _this._register(model.onDidChangeColor(_this.onDidChangeColor, _this));
        _this.onDidChangeColor(_this.model.color);
        return _this;
    }
    OpacityStrip.prototype.onDidChangeColor = function (color) {
        var _a = color.rgba, r = _a.r, g = _a.g, b = _a.b;
        var opaque = new Color(new RGBA(r, g, b, 1));
        var transparent$$1 = new Color(new RGBA(r, g, b, 0));
        this.overlay.style.background = "linear-gradient(to bottom, " + opaque + " 0%, " + transparent$$1 + " 100%)";
    };
    OpacityStrip.prototype.getValue = function (color) {
        return color.hsva.a;
    };
    return OpacityStrip;
}(Strip));
var HueStrip = /** @class */ (function (_super) {
    __extends$1u(HueStrip, _super);
    function HueStrip(container, model) {
        var _this = _super.call(this, container, model) || this;
        addClass(_this.domNode, 'hue-strip');
        return _this;
    }
    HueStrip.prototype.getValue = function (color) {
        return 1 - (color.hsva.h / 360);
    };
    return HueStrip;
}(Strip));
var ColorPickerWidget = /** @class */ (function (_super) {
    __extends$1u(ColorPickerWidget, _super);
    function ColorPickerWidget(container, model, pixelRatio, themeService) {
        var _this = _super.call(this) || this;
        _this.model = model;
        _this.pixelRatio = pixelRatio;
        _this._register(onDidChangeZoomLevel(function () { return _this.layout(); }));
        var element = $$2('.colorpicker-widget');
        container.appendChild(element);
        var header = new ColorPickerHeader(element, _this.model, themeService);
        _this.body = new ColorPickerBody(element, _this.model, _this.pixelRatio);
        _this._register(header);
        _this._register(_this.body);
        return _this;
    }
    ColorPickerWidget.prototype.layout = function () {
        this.body.layout();
    };
    return ColorPickerWidget;
}(Widget$1));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getHover(model, position, token) {
    var supports = HoverProviderRegistry.ordered(model);
    var promises = supports.map(function (support) {
        return Promise.resolve(support.provideHover(model, position, token)).then(function (hover) {
            return hover && isValid(hover) ? hover : undefined;
        }, function (err) {
            onUnexpectedExternalError(err);
            return undefined;
        });
    });
    return Promise.all(promises).then(coalesce);
}
registerDefaultLanguageCommand('_executeHoverProvider', function (model, position) { return getHover(model, position, CancellationToken.None); });
function isValid(result) {
    var hasRange = (typeof result.range !== 'undefined');
    var hasHtmlContent = typeof result.contents !== 'undefined' && result.contents && result.contents.length > 0;
    return hasRange && hasHtmlContent;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var HoverOperation = /** @class */ (function () {
    function HoverOperation(computer, success, error, progress) {
        var _this = this;
        this._computer = computer;
        this._state = 0 /* IDLE */;
        this._hoverTime = HoverOperation.HOVER_TIME;
        this._firstWaitScheduler = new RunOnceScheduler(function () { return _this._triggerAsyncComputation(); }, 0);
        this._secondWaitScheduler = new RunOnceScheduler(function () { return _this._triggerSyncComputation(); }, 0);
        this._loadingMessageScheduler = new RunOnceScheduler(function () { return _this._showLoadingMessage(); }, 0);
        this._asyncComputationPromise = null;
        this._asyncComputationPromiseDone = false;
        this._completeCallback = success;
        this._errorCallback = error;
        this._progressCallback = progress;
    }
    HoverOperation.prototype.setHoverTime = function (hoverTime) {
        this._hoverTime = hoverTime;
    };
    HoverOperation.prototype._firstWaitTime = function () {
        return this._hoverTime / 2;
    };
    HoverOperation.prototype._secondWaitTime = function () {
        return this._hoverTime / 2;
    };
    HoverOperation.prototype._loadingMessageTime = function () {
        return 3 * this._hoverTime;
    };
    HoverOperation.prototype._triggerAsyncComputation = function () {
        var _this = this;
        this._state = 2 /* SECOND_WAIT */;
        this._secondWaitScheduler.schedule(this._secondWaitTime());
        if (this._computer.computeAsync) {
            this._asyncComputationPromiseDone = false;
            this._asyncComputationPromise = createCancelablePromise(function (token) { return _this._computer.computeAsync(token); });
            this._asyncComputationPromise.then(function (asyncResult) {
                _this._asyncComputationPromiseDone = true;
                _this._withAsyncResult(asyncResult);
            }, function (e) { return _this._onError(e); });
        }
        else {
            this._asyncComputationPromiseDone = true;
        }
    };
    HoverOperation.prototype._triggerSyncComputation = function () {
        if (this._computer.computeSync) {
            this._computer.onResult(this._computer.computeSync(), true);
        }
        if (this._asyncComputationPromiseDone) {
            this._state = 0 /* IDLE */;
            this._onComplete(this._computer.getResult());
        }
        else {
            this._state = 3 /* WAITING_FOR_ASYNC_COMPUTATION */;
            this._onProgress(this._computer.getResult());
        }
    };
    HoverOperation.prototype._showLoadingMessage = function () {
        if (this._state === 3 /* WAITING_FOR_ASYNC_COMPUTATION */) {
            this._onProgress(this._computer.getResultWithLoadingMessage());
        }
    };
    HoverOperation.prototype._withAsyncResult = function (asyncResult) {
        if (asyncResult) {
            this._computer.onResult(asyncResult, false);
        }
        if (this._state === 3 /* WAITING_FOR_ASYNC_COMPUTATION */) {
            this._state = 0 /* IDLE */;
            this._onComplete(this._computer.getResult());
        }
    };
    HoverOperation.prototype._onComplete = function (value) {
        if (this._completeCallback) {
            this._completeCallback(value);
        }
    };
    HoverOperation.prototype._onError = function (error) {
        if (this._errorCallback) {
            this._errorCallback(error);
        }
        else {
            onUnexpectedError(error);
        }
    };
    HoverOperation.prototype._onProgress = function (value) {
        if (this._progressCallback) {
            this._progressCallback(value);
        }
    };
    HoverOperation.prototype.start = function (mode) {
        if (mode === 0 /* Delayed */) {
            if (this._state === 0 /* IDLE */) {
                this._state = 1 /* FIRST_WAIT */;
                this._firstWaitScheduler.schedule(this._firstWaitTime());
                this._loadingMessageScheduler.schedule(this._loadingMessageTime());
            }
        }
        else {
            switch (this._state) {
                case 0 /* IDLE */:
                    this._triggerAsyncComputation();
                    this._secondWaitScheduler.cancel();
                    this._triggerSyncComputation();
                    break;
                case 2 /* SECOND_WAIT */:
                    this._secondWaitScheduler.cancel();
                    this._triggerSyncComputation();
                    break;
            }
        }
    };
    HoverOperation.prototype.cancel = function () {
        this._loadingMessageScheduler.cancel();
        if (this._state === 1 /* FIRST_WAIT */) {
            this._firstWaitScheduler.cancel();
        }
        if (this._state === 2 /* SECOND_WAIT */) {
            this._secondWaitScheduler.cancel();
            if (this._asyncComputationPromise) {
                this._asyncComputationPromise.cancel();
                this._asyncComputationPromise = null;
            }
        }
        if (this._state === 3 /* WAITING_FOR_ASYNC_COMPUTATION */) {
            if (this._asyncComputationPromise) {
                this._asyncComputationPromise.cancel();
                this._asyncComputationPromise = null;
            }
        }
        this._state = 0 /* IDLE */;
    };
    HoverOperation.HOVER_TIME = 300;
    return HoverOperation;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1v = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ContentHoverWidget = /** @class */ (function (_super) {
    __extends$1v(ContentHoverWidget, _super);
    function ContentHoverWidget(id, editor) {
        var _this = _super.call(this) || this;
        _this.disposables = [];
        // Editor.IContentWidget.allowEditorOverflow
        _this.allowEditorOverflow = true;
        _this._id = id;
        _this._editor = editor;
        _this._isVisible = false;
        _this._containerDomNode = document.createElement('div');
        _this._containerDomNode.className = 'monaco-editor-hover hidden';
        _this._containerDomNode.tabIndex = 0;
        _this._domNode = document.createElement('div');
        _this._domNode.className = 'monaco-editor-hover-content';
        _this.scrollbar = new DomScrollableElement(_this._domNode, {});
        _this.disposables.push(_this.scrollbar);
        _this._containerDomNode.appendChild(_this.scrollbar.getDomNode());
        _this.onkeydown(_this._containerDomNode, function (e) {
            if (e.equals(9 /* Escape */)) {
                _this.hide();
            }
        });
        _this._register(_this._editor.onDidChangeConfiguration(function (e) {
            if (e.fontInfo) {
                _this.updateFont();
            }
        }));
        _this._editor.onDidLayoutChange(function (e) { return _this.updateMaxHeight(); });
        _this.updateMaxHeight();
        _this._editor.addContentWidget(_this);
        _this._showAtPosition = null;
        _this._showAtRange = null;
        return _this;
    }
    Object.defineProperty(ContentHoverWidget.prototype, "isVisible", {
        get: function () {
            return this._isVisible;
        },
        set: function (value) {
            this._isVisible = value;
            toggleClass(this._containerDomNode, 'hidden', !this._isVisible);
        },
        enumerable: true,
        configurable: true
    });
    ContentHoverWidget.prototype.getId = function () {
        return this._id;
    };
    ContentHoverWidget.prototype.getDomNode = function () {
        return this._containerDomNode;
    };
    ContentHoverWidget.prototype.showAt = function (position, range, focus) {
        // Position has changed
        this._showAtPosition = position;
        this._showAtRange = range;
        this.isVisible = true;
        this._editor.layoutContentWidget(this);
        // Simply force a synchronous render on the editor
        // such that the widget does not really render with left = '0px'
        this._editor.render();
        this._stoleFocus = focus;
        if (focus) {
            this._containerDomNode.focus();
        }
    };
    ContentHoverWidget.prototype.hide = function () {
        if (!this.isVisible) {
            return;
        }
        this.isVisible = false;
        this._editor.layoutContentWidget(this);
        if (this._stoleFocus) {
            this._editor.focus();
        }
    };
    ContentHoverWidget.prototype.getPosition = function () {
        if (this.isVisible) {
            return {
                position: this._showAtPosition,
                range: this._showAtRange,
                preference: [
                    1 /* ABOVE */,
                    2 /* BELOW */
                ]
            };
        }
        return null;
    };
    ContentHoverWidget.prototype.dispose = function () {
        this._editor.removeContentWidget(this);
        this.disposables = dispose(this.disposables);
        _super.prototype.dispose.call(this);
    };
    ContentHoverWidget.prototype.updateFont = function () {
        var _this = this;
        var codeClasses = Array.prototype.slice.call(this._domNode.getElementsByClassName('code'));
        codeClasses.forEach(function (node) { return _this._editor.applyFontInfo(node); });
    };
    ContentHoverWidget.prototype.updateContents = function (node) {
        this._domNode.textContent = '';
        this._domNode.appendChild(node);
        this.updateFont();
        this._editor.layoutContentWidget(this);
        this.onContentsChange();
    };
    ContentHoverWidget.prototype.onContentsChange = function () {
        this.scrollbar.scanDomNode();
    };
    ContentHoverWidget.prototype.updateMaxHeight = function () {
        var height = Math.max(this._editor.getLayoutInfo().height / 4, 250);
        var _a = this._editor.getConfiguration().fontInfo, fontSize = _a.fontSize, lineHeight = _a.lineHeight;
        this._domNode.style.fontSize = fontSize + "px";
        this._domNode.style.lineHeight = lineHeight + "px";
        this._domNode.style.maxHeight = height + "px";
    };
    return ContentHoverWidget;
}(Widget$1));
var GlyphHoverWidget = /** @class */ (function (_super) {
    __extends$1v(GlyphHoverWidget, _super);
    function GlyphHoverWidget(id, editor) {
        var _this = _super.call(this) || this;
        _this._id = id;
        _this._editor = editor;
        _this._isVisible = false;
        _this._domNode = document.createElement('div');
        _this._domNode.className = 'monaco-editor-hover hidden';
        _this._domNode.setAttribute('aria-hidden', 'true');
        _this._domNode.setAttribute('role', 'presentation');
        _this._showAtLineNumber = -1;
        _this._register(_this._editor.onDidChangeConfiguration(function (e) {
            if (e.fontInfo) {
                _this.updateFont();
            }
        }));
        _this._editor.addOverlayWidget(_this);
        return _this;
    }
    Object.defineProperty(GlyphHoverWidget.prototype, "isVisible", {
        get: function () {
            return this._isVisible;
        },
        set: function (value) {
            this._isVisible = value;
            toggleClass(this._domNode, 'hidden', !this._isVisible);
        },
        enumerable: true,
        configurable: true
    });
    GlyphHoverWidget.prototype.getId = function () {
        return this._id;
    };
    GlyphHoverWidget.prototype.getDomNode = function () {
        return this._domNode;
    };
    GlyphHoverWidget.prototype.showAt = function (lineNumber) {
        this._showAtLineNumber = lineNumber;
        if (!this.isVisible) {
            this.isVisible = true;
        }
        var editorLayout = this._editor.getLayoutInfo();
        var topForLineNumber = this._editor.getTopForLineNumber(this._showAtLineNumber);
        var editorScrollTop = this._editor.getScrollTop();
        var lineHeight = this._editor.getConfiguration().lineHeight;
        var nodeHeight = this._domNode.clientHeight;
        var top = topForLineNumber - editorScrollTop - ((nodeHeight - lineHeight) / 2);
        this._domNode.style.left = editorLayout.glyphMarginLeft + editorLayout.glyphMarginWidth + "px";
        this._domNode.style.top = Math.max(Math.round(top), 0) + "px";
    };
    GlyphHoverWidget.prototype.hide = function () {
        if (!this.isVisible) {
            return;
        }
        this.isVisible = false;
    };
    GlyphHoverWidget.prototype.getPosition = function () {
        return null;
    };
    GlyphHoverWidget.prototype.dispose = function () {
        this._editor.removeOverlayWidget(this);
        _super.prototype.dispose.call(this);
    };
    GlyphHoverWidget.prototype.updateFont = function () {
        var _this = this;
        var codeTags = Array.prototype.slice.call(this._domNode.getElementsByTagName('code'));
        var codeClasses = Array.prototype.slice.call(this._domNode.getElementsByClassName('code'));
        codeTags.concat(codeClasses).forEach(function (node) { return _this._editor.applyFontInfo(node); });
    };
    GlyphHoverWidget.prototype.updateContents = function (node) {
        this._domNode.textContent = '';
        this._domNode.appendChild(node);
        this.updateFont();
    };
    return GlyphHoverWidget;
}(Widget$1));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1w = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var $$3 = $;
var ColorHover = /** @class */ (function () {
    function ColorHover(range, color, provider) {
        this.range = range;
        this.color = color;
        this.provider = provider;
    }
    return ColorHover;
}());
var ModesContentComputer = /** @class */ (function () {
    function ModesContentComputer(editor) {
        this._editor = editor;
        this._range = null;
    }
    ModesContentComputer.prototype.setRange = function (range) {
        this._range = range;
        this._result = [];
    };
    ModesContentComputer.prototype.clearResult = function () {
        this._result = [];
    };
    ModesContentComputer.prototype.computeAsync = function (token) {
        var model = this._editor.getModel();
        if (!HoverProviderRegistry.has(model)) {
            return Promise.resolve(null);
        }
        return getHover(model, new Position(this._range.startLineNumber, this._range.startColumn), token);
    };
    ModesContentComputer.prototype.computeSync = function () {
        var _this = this;
        var lineNumber = this._range.startLineNumber;
        if (lineNumber > this._editor.getModel().getLineCount()) {
            // Illegal line number => no results
            return [];
        }
        var colorDetector = ColorDetector.get(this._editor);
        var maxColumn = this._editor.getModel().getLineMaxColumn(lineNumber);
        var lineDecorations = this._editor.getLineDecorations(lineNumber);
        var didFindColor = false;
        var result = lineDecorations.map(function (d) {
            var startColumn = (d.range.startLineNumber === lineNumber) ? d.range.startColumn : 1;
            var endColumn = (d.range.endLineNumber === lineNumber) ? d.range.endColumn : maxColumn;
            if (startColumn > _this._range.startColumn || _this._range.endColumn > endColumn) {
                return null;
            }
            var range = new Range(_this._range.startLineNumber, startColumn, _this._range.startLineNumber, endColumn);
            var colorData = colorDetector.getColorData(d.range.getStartPosition());
            if (!didFindColor && colorData) {
                didFindColor = true;
                var _a = colorData.colorInfo, color = _a.color, range_1 = _a.range;
                return new ColorHover(range_1, color, colorData.provider);
            }
            else {
                if (isEmptyMarkdownString(d.options.hoverMessage)) {
                    return null;
                }
                var contents = void 0;
                if (d.options.hoverMessage) {
                    if (Array.isArray(d.options.hoverMessage)) {
                        contents = d.options.hoverMessage.slice();
                    }
                    else {
                        contents = [d.options.hoverMessage];
                    }
                }
                return { contents: contents, range: range };
            }
        });
        return result.filter(function (d) { return !!d; });
    };
    ModesContentComputer.prototype.onResult = function (result, isFromSynchronousComputation) {
        // Always put synchronous messages before asynchronous ones
        if (isFromSynchronousComputation) {
            this._result = result.concat(this._result.sort(function (a, b) {
                if (a instanceof ColorHover) { // sort picker messages at to the top
                    return -1;
                }
                else if (b instanceof ColorHover) {
                    return 1;
                }
                return 0;
            }));
        }
        else {
            this._result = this._result.concat(result);
        }
    };
    ModesContentComputer.prototype.getResult = function () {
        return this._result.slice(0);
    };
    ModesContentComputer.prototype.getResultWithLoadingMessage = function () {
        return this._result.slice(0).concat([this._getLoadingMessage()]);
    };
    ModesContentComputer.prototype._getLoadingMessage = function () {
        return {
            range: this._range,
            contents: [new MarkdownString().appendText(localize('modesContentHover.loading', "Loading..."))]
        };
    };
    return ModesContentComputer;
}());
var ModesContentHoverWidget = /** @class */ (function (_super) {
    __extends$1w(ModesContentHoverWidget, _super);
    function ModesContentHoverWidget(editor, markdownRenderer, _themeService) {
        var _this = _super.call(this, ModesContentHoverWidget.ID, editor) || this;
        _this._themeService = _themeService;
        _this.renderDisposable = Disposable.None;
        _this._computer = new ModesContentComputer(_this._editor);
        _this._highlightDecorations = [];
        _this._isChangingDecorations = false;
        _this._markdownRenderer = markdownRenderer;
        _this._register(markdownRenderer.onDidRenderCodeBlock(_this.onContentsChange, _this));
        _this._hoverOperation = new HoverOperation(_this._computer, function (result) { return _this._withResult(result, true); }, null, function (result) { return _this._withResult(result, false); });
        _this._register(addStandardDisposableListener(_this.getDomNode(), EventType.FOCUS, function () {
            if (_this._colorPicker) {
                addClass(_this.getDomNode(), 'colorpicker-hover');
            }
        }));
        _this._register(addStandardDisposableListener(_this.getDomNode(), EventType.BLUR, function () {
            removeClass(_this.getDomNode(), 'colorpicker-hover');
        }));
        _this._register(editor.onDidChangeConfiguration(function (e) {
            _this._hoverOperation.setHoverTime(_this._editor.getConfiguration().contribInfo.hover.delay);
        }));
        return _this;
    }
    ModesContentHoverWidget.prototype.dispose = function () {
        this.renderDisposable.dispose();
        this.renderDisposable = Disposable.None;
        this._hoverOperation.cancel();
        _super.prototype.dispose.call(this);
    };
    ModesContentHoverWidget.prototype.onModelDecorationsChanged = function () {
        if (this._isChangingDecorations) {
            return;
        }
        if (this.isVisible) {
            // The decorations have changed and the hover is visible,
            // we need to recompute the displayed text
            this._hoverOperation.cancel();
            this._computer.clearResult();
            if (!this._colorPicker) { // TODO@Michel ensure that displayed text for other decorations is computed even if color picker is in place
                this._hoverOperation.start(0 /* Delayed */);
            }
        }
    };
    ModesContentHoverWidget.prototype.startShowingAt = function (range, mode, focus) {
        if (this._lastRange && this._lastRange.equalsRange(range)) {
            // We have to show the widget at the exact same range as before, so no work is needed
            return;
        }
        this._hoverOperation.cancel();
        if (this.isVisible) {
            // The range might have changed, but the hover is visible
            // Instead of hiding it completely, filter out messages that are still in the new range and
            // kick off a new computation
            if (this._showAtPosition.lineNumber !== range.startLineNumber) {
                this.hide();
            }
            else {
                var filteredMessages = [];
                for (var i = 0, len = this._messages.length; i < len; i++) {
                    var msg = this._messages[i];
                    var rng = msg.range;
                    if (rng.startColumn <= range.startColumn && rng.endColumn >= range.endColumn) {
                        filteredMessages.push(msg);
                    }
                }
                if (filteredMessages.length > 0) {
                    if (hoverContentsEquals(filteredMessages, this._messages)) {
                        return;
                    }
                    this._renderMessages(range, filteredMessages);
                }
                else {
                    this.hide();
                }
            }
        }
        this._lastRange = range;
        this._computer.setRange(range);
        this._shouldFocus = focus;
        this._hoverOperation.start(mode);
    };
    ModesContentHoverWidget.prototype.hide = function () {
        this._lastRange = null;
        this._hoverOperation.cancel();
        _super.prototype.hide.call(this);
        this._isChangingDecorations = true;
        this._highlightDecorations = this._editor.deltaDecorations(this._highlightDecorations, []);
        this._isChangingDecorations = false;
        this.renderDisposable.dispose();
        this.renderDisposable = Disposable.None;
        this._colorPicker = null;
    };
    ModesContentHoverWidget.prototype.isColorPickerVisible = function () {
        if (this._colorPicker) {
            return true;
        }
        return false;
    };
    ModesContentHoverWidget.prototype._withResult = function (result, complete) {
        this._messages = result;
        if (this._lastRange && this._messages.length > 0) {
            this._renderMessages(this._lastRange, this._messages);
        }
        else if (complete) {
            this.hide();
        }
    };
    ModesContentHoverWidget.prototype._renderMessages = function (renderRange, messages) {
        var _this = this;
        this.renderDisposable.dispose();
        this._colorPicker = null;
        // update column from which to show
        var renderColumn = Number.MAX_VALUE;
        var highlightRange = Range.lift(messages[0].range);
        var fragment = document.createDocumentFragment();
        var isEmptyHoverContent = true;
        var containColorPicker = false;
        var markdownDisposeable;
        messages.forEach(function (msg) {
            if (!msg.range) {
                return;
            }
            renderColumn = Math.min(renderColumn, msg.range.startColumn);
            highlightRange = Range.plusRange(highlightRange, msg.range);
            if (!(msg instanceof ColorHover)) {
                msg.contents
                    .filter(function (contents) { return !isEmptyMarkdownString(contents); })
                    .forEach(function (contents) {
                    var renderedContents = _this._markdownRenderer.render(contents);
                    markdownDisposeable = renderedContents;
                    fragment.appendChild($$3('div.hover-row', null, renderedContents.element));
                    isEmptyHoverContent = false;
                });
            }
            else {
                containColorPicker = true;
                var _a = msg.color, red = _a.red, green = _a.green, blue = _a.blue, alpha = _a.alpha;
                var rgba = new RGBA(red * 255, green * 255, blue * 255, alpha);
                var color_1 = new Color(rgba);
                var editorModel_1 = _this._editor.getModel();
                var range_2 = new Range(msg.range.startLineNumber, msg.range.startColumn, msg.range.endLineNumber, msg.range.endColumn);
                var colorInfo = { range: msg.range, color: msg.color };
                // create blank olor picker model and widget first to ensure it's positioned correctly.
                var model_1 = new ColorPickerModel(color_1, [], 0);
                var widget_1 = new ColorPickerWidget(fragment, model_1, _this._editor.getConfiguration().pixelRatio, _this._themeService);
                getColorPresentations(editorModel_1, colorInfo, msg.provider, CancellationToken.None).then(function (colorPresentations) {
                    model_1.colorPresentations = colorPresentations;
                    var originalText = _this._editor.getModel().getValueInRange(msg.range);
                    model_1.guessColorPresentation(color_1, originalText);
                    var updateEditorModel = function () {
                        var textEdits;
                        var newRange;
                        if (model_1.presentation.textEdit) {
                            textEdits = [model_1.presentation.textEdit];
                            newRange = new Range(model_1.presentation.textEdit.range.startLineNumber, model_1.presentation.textEdit.range.startColumn, model_1.presentation.textEdit.range.endLineNumber, model_1.presentation.textEdit.range.endColumn);
                            newRange = newRange.setEndPosition(newRange.endLineNumber, newRange.startColumn + model_1.presentation.textEdit.text.length);
                        }
                        else {
                            textEdits = [{ identifier: null, range: range_2, text: model_1.presentation.label, forceMoveMarkers: false }];
                            newRange = range_2.setEndPosition(range_2.endLineNumber, range_2.startColumn + model_1.presentation.label.length);
                        }
                        _this._editor.pushUndoStop();
                        _this._editor.executeEdits('colorpicker', textEdits);
                        if (model_1.presentation.additionalTextEdits) {
                            textEdits = model_1.presentation.additionalTextEdits.slice();
                            _this._editor.executeEdits('colorpicker', textEdits);
                            _this.hide();
                        }
                        _this._editor.pushUndoStop();
                        range_2 = newRange;
                    };
                    var updateColorPresentations = function (color) {
                        return getColorPresentations(editorModel_1, {
                            range: range_2,
                            color: {
                                red: color.rgba.r / 255,
                                green: color.rgba.g / 255,
                                blue: color.rgba.b / 255,
                                alpha: color.rgba.a
                            }
                        }, msg.provider, CancellationToken.None).then(function (colorPresentations) {
                            model_1.colorPresentations = colorPresentations;
                        });
                    };
                    var colorListener = model_1.onColorFlushed(function (color) {
                        updateColorPresentations(color).then(updateEditorModel);
                    });
                    var colorChangeListener = model_1.onDidChangeColor(updateColorPresentations);
                    _this._colorPicker = widget_1;
                    _this.showAt(range_2.getStartPosition(), range_2, _this._shouldFocus);
                    _this.updateContents(fragment);
                    _this._colorPicker.layout();
                    _this.renderDisposable = combinedDisposable([colorListener, colorChangeListener, widget_1, markdownDisposeable]);
                });
            }
        });
        // show
        if (!containColorPicker && !isEmptyHoverContent) {
            this.showAt(new Position(renderRange.startLineNumber, renderColumn), highlightRange, this._shouldFocus);
            this.updateContents(fragment);
        }
        this._isChangingDecorations = true;
        this._highlightDecorations = this._editor.deltaDecorations(this._highlightDecorations, [{
                range: highlightRange,
                options: ModesContentHoverWidget._DECORATION_OPTIONS
            }]);
        this._isChangingDecorations = false;
    };
    ModesContentHoverWidget.ID = 'editor.contrib.modesContentHoverWidget';
    ModesContentHoverWidget._DECORATION_OPTIONS = ModelDecorationOptions.register({
        className: 'hoverHighlight'
    });
    return ModesContentHoverWidget;
}(ContentHoverWidget));
function hoverContentsEquals(first, second) {
    if ((!first && second) || (first && !second) || first.length !== second.length) {
        return false;
    }
    for (var i = 0; i < first.length; i++) {
        var firstElement = first[i];
        var secondElement = second[i];
        if (firstElement instanceof ColorHover) {
            return false;
        }
        if (secondElement instanceof ColorHover) {
            return false;
        }
        if (!markedStringsEquals(firstElement.contents, secondElement.contents)) {
            return false;
        }
    }
    return true;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1x = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var MarginComputer = /** @class */ (function () {
    function MarginComputer(editor) {
        this._editor = editor;
        this._lineNumber = -1;
    }
    MarginComputer.prototype.setLineNumber = function (lineNumber) {
        this._lineNumber = lineNumber;
        this._result = [];
    };
    MarginComputer.prototype.clearResult = function () {
        this._result = [];
    };
    MarginComputer.prototype.computeSync = function () {
        var toHoverMessage = function (contents) {
            return {
                value: contents
            };
        };
        var lineDecorations = this._editor.getLineDecorations(this._lineNumber);
        var result = [];
        if (!lineDecorations) {
            return result;
        }
        for (var i = 0, len = lineDecorations.length; i < len; i++) {
            var d = lineDecorations[i];
            if (!d.options.glyphMarginClassName) {
                continue;
            }
            var hoverMessage = d.options.glyphMarginHoverMessage;
            if (!hoverMessage || isEmptyMarkdownString(hoverMessage)) {
                continue;
            }
            if (Array.isArray(hoverMessage)) {
                result = result.concat(hoverMessage.map(toHoverMessage));
            }
            else {
                result.push(toHoverMessage(hoverMessage));
            }
        }
        return result;
    };
    MarginComputer.prototype.onResult = function (result, isFromSynchronousComputation) {
        this._result = this._result.concat(result);
    };
    MarginComputer.prototype.getResult = function () {
        return this._result;
    };
    MarginComputer.prototype.getResultWithLoadingMessage = function () {
        return this.getResult();
    };
    return MarginComputer;
}());
var ModesGlyphHoverWidget = /** @class */ (function (_super) {
    __extends$1x(ModesGlyphHoverWidget, _super);
    function ModesGlyphHoverWidget(editor, markdownRenderer) {
        var _this = _super.call(this, ModesGlyphHoverWidget.ID, editor) || this;
        _this._lastLineNumber = -1;
        _this._markdownRenderer = markdownRenderer;
        _this._computer = new MarginComputer(_this._editor);
        _this._hoverOperation = new HoverOperation(_this._computer, function (result) { return _this._withResult(result); }, undefined, function (result) { return _this._withResult(result); });
        return _this;
    }
    ModesGlyphHoverWidget.prototype.dispose = function () {
        this._renderDisposeables = dispose(this._renderDisposeables);
        this._hoverOperation.cancel();
        _super.prototype.dispose.call(this);
    };
    ModesGlyphHoverWidget.prototype.onModelDecorationsChanged = function () {
        if (this.isVisible) {
            // The decorations have changed and the hover is visible,
            // we need to recompute the displayed text
            this._hoverOperation.cancel();
            this._computer.clearResult();
            this._hoverOperation.start(0 /* Delayed */);
        }
    };
    ModesGlyphHoverWidget.prototype.startShowingAt = function (lineNumber) {
        if (this._lastLineNumber === lineNumber) {
            // We have to show the widget at the exact same line number as before, so no work is needed
            return;
        }
        this._hoverOperation.cancel();
        this.hide();
        this._lastLineNumber = lineNumber;
        this._computer.setLineNumber(lineNumber);
        this._hoverOperation.start(0 /* Delayed */);
    };
    ModesGlyphHoverWidget.prototype.hide = function () {
        this._lastLineNumber = -1;
        this._hoverOperation.cancel();
        _super.prototype.hide.call(this);
    };
    ModesGlyphHoverWidget.prototype._withResult = function (result) {
        this._messages = result;
        if (this._messages.length > 0) {
            this._renderMessages(this._lastLineNumber, this._messages);
        }
        else {
            this.hide();
        }
    };
    ModesGlyphHoverWidget.prototype._renderMessages = function (lineNumber, messages) {
        var _this = this;
        dispose(this._renderDisposeables);
        this._renderDisposeables = [];
        var fragment = document.createDocumentFragment();
        messages.forEach(function (msg) {
            var renderedContents = _this._markdownRenderer.render(msg.value);
            _this._renderDisposeables.push(renderedContents);
            fragment.appendChild($('div.hover-row', undefined, renderedContents.element));
        });
        this.updateContents(fragment);
        this.showAt(lineNumber);
    };
    ModesGlyphHoverWidget.ID = 'editor.contrib.modesGlyphHoverWidget';
    return ModesGlyphHoverWidget;
}(GlyphHoverWidget));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IOpenerService = createDecorator('openerService');
var NullOpenerService = Object.freeze({
    _serviceBrand: undefined,
    open: function () { return Promise.resolve(undefined); }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$a = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$9 = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MarkdownRenderer = /** @class */ (function () {
    function MarkdownRenderer(_editor, _modeService, _openerService) {
        if (_openerService === void 0) { _openerService = NullOpenerService; }
        this._editor = _editor;
        this._modeService = _modeService;
        this._openerService = _openerService;
        this._onDidRenderCodeBlock = new Emitter();
        this.onDidRenderCodeBlock = this._onDidRenderCodeBlock.event;
    }
    MarkdownRenderer.prototype.getOptions = function (disposeables) {
        var _this = this;
        return {
            codeBlockRenderer: function (languageAlias, value) {
                // In markdown,
                // it is possible that we stumble upon language aliases (e.g.js instead of javascript)
                // it is possible no alias is given in which case we fall back to the current editor lang
                var modeId = null;
                if (languageAlias) {
                    modeId = _this._modeService.getModeIdForLanguageName(languageAlias);
                }
                else {
                    var model = _this._editor.getModel();
                    if (model) {
                        modeId = model.getLanguageIdentifier().language;
                    }
                }
                _this._modeService.triggerMode(modeId || '');
                return Promise.resolve(true).then(function (_) {
                    var promise = TokenizationRegistry.getPromise(modeId || '');
                    if (promise) {
                        return promise.then(function (support) { return tokenizeToString(value, support); });
                    }
                    return tokenizeToString(value, undefined);
                }).then(function (code) {
                    return "<span style=\"font-family: " + _this._editor.getConfiguration().fontInfo.fontFamily + "\">" + code + "</span>";
                });
            },
            codeBlockRenderCallback: function () { return _this._onDidRenderCodeBlock.fire(); },
            actionHandler: {
                callback: function (content) {
                    var uri;
                    try {
                        uri = URI.parse(content);
                    }
                    catch (_a) {
                        // ignore
                    }
                    if (uri && _this._openerService) {
                        _this._openerService.open(uri).catch(onUnexpectedError);
                    }
                },
                disposeables: disposeables
            }
        };
    };
    MarkdownRenderer.prototype.render = function (markdown) {
        var disposeables = [];
        var element;
        if (!markdown) {
            element = document.createElement('span');
        }
        else {
            element = renderMarkdown(markdown, this.getOptions(disposeables));
        }
        return {
            element: element,
            dispose: function () { return dispose(disposeables); }
        };
    };
    MarkdownRenderer = __decorate$a([
        __param$9(1, IModeService),
        __param$9(2, optional(IOpenerService))
    ], MarkdownRenderer);
    return MarkdownRenderer;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1y = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$b = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$a = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ModesHoverController = /** @class */ (function () {
    function ModesHoverController(_editor, _openerService, _modeService, _themeService) {
        var _this = this;
        this._editor = _editor;
        this._openerService = _openerService;
        this._modeService = _modeService;
        this._themeService = _themeService;
        this._toUnhook = [];
        this._isMouseDown = false;
        this._hoverClicked = false;
        this._hookEvents();
        this._didChangeConfigurationHandler = this._editor.onDidChangeConfiguration(function (e) {
            if (e.contribInfo) {
                _this._hideWidgets();
                _this._unhookEvents();
                _this._hookEvents();
            }
        });
    }
    Object.defineProperty(ModesHoverController.prototype, "contentWidget", {
        get: function () {
            if (!this._contentWidget) {
                this._createHoverWidget();
            }
            return this._contentWidget;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ModesHoverController.prototype, "glyphWidget", {
        get: function () {
            if (!this._glyphWidget) {
                this._createHoverWidget();
            }
            return this._glyphWidget;
        },
        enumerable: true,
        configurable: true
    });
    ModesHoverController.get = function (editor) {
        return editor.getContribution(ModesHoverController.ID);
    };
    ModesHoverController.prototype._hookEvents = function () {
        var _this = this;
        var hideWidgetsEventHandler = function () { return _this._hideWidgets(); };
        var hoverOpts = this._editor.getConfiguration().contribInfo.hover;
        this._isHoverEnabled = hoverOpts.enabled;
        this._isHoverSticky = hoverOpts.sticky;
        if (this._isHoverEnabled) {
            this._toUnhook.push(this._editor.onMouseDown(function (e) { return _this._onEditorMouseDown(e); }));
            this._toUnhook.push(this._editor.onMouseUp(function (e) { return _this._onEditorMouseUp(e); }));
            this._toUnhook.push(this._editor.onMouseMove(function (e) { return _this._onEditorMouseMove(e); }));
            this._toUnhook.push(this._editor.onKeyDown(function (e) { return _this._onKeyDown(e); }));
            this._toUnhook.push(this._editor.onDidChangeModelDecorations(function () { return _this._onModelDecorationsChanged(); }));
        }
        else {
            this._toUnhook.push(this._editor.onMouseMove(hideWidgetsEventHandler));
        }
        this._toUnhook.push(this._editor.onMouseLeave(hideWidgetsEventHandler));
        this._toUnhook.push(this._editor.onDidChangeModel(hideWidgetsEventHandler));
        this._toUnhook.push(this._editor.onDidScrollChange(function (e) { return _this._onEditorScrollChanged(e); }));
    };
    ModesHoverController.prototype._unhookEvents = function () {
        this._toUnhook = dispose(this._toUnhook);
    };
    ModesHoverController.prototype._onModelDecorationsChanged = function () {
        this.contentWidget.onModelDecorationsChanged();
        this.glyphWidget.onModelDecorationsChanged();
    };
    ModesHoverController.prototype._onEditorScrollChanged = function (e) {
        if (e.scrollTopChanged || e.scrollLeftChanged) {
            this._hideWidgets();
        }
    };
    ModesHoverController.prototype._onEditorMouseDown = function (mouseEvent) {
        this._isMouseDown = true;
        var targetType = mouseEvent.target.type;
        if (targetType === 9 /* CONTENT_WIDGET */ && mouseEvent.target.detail === ModesContentHoverWidget.ID) {
            this._hoverClicked = true;
            // mouse down on top of content hover widget
            return;
        }
        if (targetType === 12 /* OVERLAY_WIDGET */ && mouseEvent.target.detail === ModesGlyphHoverWidget.ID) {
            // mouse down on top of overlay hover widget
            return;
        }
        if (targetType !== 12 /* OVERLAY_WIDGET */ && mouseEvent.target.detail !== ModesGlyphHoverWidget.ID) {
            this._hoverClicked = false;
        }
        this._hideWidgets();
    };
    ModesHoverController.prototype._onEditorMouseUp = function (mouseEvent) {
        this._isMouseDown = false;
    };
    ModesHoverController.prototype._onEditorMouseMove = function (mouseEvent) {
        // const this._editor.getConfiguration().contribInfo.hover.sticky;
        var targetType = mouseEvent.target.type;
        var hasStopKey = (isMacintosh ? mouseEvent.event.metaKey : mouseEvent.event.ctrlKey);
        if (this._isMouseDown && this._hoverClicked && this.contentWidget.isColorPickerVisible()) {
            return;
        }
        if (this._isHoverSticky && targetType === 9 /* CONTENT_WIDGET */ && mouseEvent.target.detail === ModesContentHoverWidget.ID && !hasStopKey) {
            // mouse moved on top of content hover widget
            return;
        }
        if (this._isHoverSticky && targetType === 12 /* OVERLAY_WIDGET */ && mouseEvent.target.detail === ModesGlyphHoverWidget.ID && !hasStopKey) {
            // mouse moved on top of overlay hover widget
            return;
        }
        if (targetType === 7 /* CONTENT_EMPTY */) {
            var epsilon = this._editor.getConfiguration().fontInfo.typicalHalfwidthCharacterWidth / 2;
            var data = mouseEvent.target.detail;
            if (data && !data.isAfterLines && typeof data.horizontalDistanceToText === 'number' && data.horizontalDistanceToText < epsilon) {
                // Let hover kick in even when the mouse is technically in the empty area after a line, given the distance is small enough
                targetType = 6 /* CONTENT_TEXT */;
            }
        }
        if (targetType === 6 /* CONTENT_TEXT */) {
            this.glyphWidget.hide();
            if (this._isHoverEnabled) {
                this.contentWidget.startShowingAt(mouseEvent.target.range, 0 /* Delayed */, false);
            }
        }
        else if (targetType === 2 /* GUTTER_GLYPH_MARGIN */) {
            this.contentWidget.hide();
            if (this._isHoverEnabled) {
                this.glyphWidget.startShowingAt(mouseEvent.target.position.lineNumber);
            }
        }
        else {
            this._hideWidgets();
        }
    };
    ModesHoverController.prototype._onKeyDown = function (e) {
        if (e.keyCode !== 5 /* Ctrl */ && e.keyCode !== 6 /* Alt */ && e.keyCode !== 57 /* Meta */ && e.keyCode !== 4 /* Shift */) {
            // Do not hide hover when a modifier key is pressed
            this._hideWidgets();
        }
    };
    ModesHoverController.prototype._hideWidgets = function () {
        if (!this._contentWidget || (this._isMouseDown && this._hoverClicked && this._contentWidget.isColorPickerVisible())) {
            return;
        }
        this._glyphWidget.hide();
        this._contentWidget.hide();
    };
    ModesHoverController.prototype._createHoverWidget = function () {
        var renderer = new MarkdownRenderer(this._editor, this._modeService, this._openerService);
        this._contentWidget = new ModesContentHoverWidget(this._editor, renderer, this._themeService);
        this._glyphWidget = new ModesGlyphHoverWidget(this._editor, renderer);
    };
    ModesHoverController.prototype.showContentHover = function (range, mode, focus) {
        this.contentWidget.startShowingAt(range, mode, focus);
    };
    ModesHoverController.prototype.getId = function () {
        return ModesHoverController.ID;
    };
    ModesHoverController.prototype.dispose = function () {
        this._unhookEvents();
        this._didChangeConfigurationHandler.dispose();
        if (this._glyphWidget) {
            this._glyphWidget.dispose();
            this._glyphWidget = null;
        }
        if (this._contentWidget) {
            this._contentWidget.dispose();
            this._contentWidget = null;
        }
    };
    ModesHoverController.ID = 'editor.contrib.hover';
    ModesHoverController = __decorate$b([
        __param$a(1, IOpenerService),
        __param$a(2, IModeService),
        __param$a(3, IThemeService)
    ], ModesHoverController);
    return ModesHoverController;
}());
var ShowHoverAction = /** @class */ (function (_super) {
    __extends$1y(ShowHoverAction, _super);
    function ShowHoverAction() {
        return _super.call(this, {
            id: 'editor.action.showHover',
            label: localize({
                key: 'showHover',
                comment: [
                    'Label for action that will trigger the showing of a hover in the editor.',
                    'This allows for users to show the hover without using the mouse.'
                ]
            }, "Show Hover"),
            alias: 'Show Hover',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 39 /* KEY_I */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    ShowHoverAction.prototype.run = function (accessor, editor) {
        var controller = ModesHoverController.get(editor);
        if (!controller) {
            return;
        }
        var position = editor.getPosition();
        var range = new Range(position.lineNumber, position.column, position.lineNumber, position.column);
        controller.showContentHover(range, 1 /* Immediate */, true);
    };
    return ShowHoverAction;
}(EditorAction));
registerEditorContribution(ModesHoverController);
registerEditorAction(ShowHoverAction);
// theming
registerThemingParticipant(function (theme, collector) {
    var editorHoverHighlightColor = theme.getColor(editorHoverHighlight);
    if (editorHoverHighlightColor) {
        collector.addRule(".monaco-editor .hoverHighlight { background-color: " + editorHoverHighlightColor + "; }");
    }
    var hoverBackground = theme.getColor(editorHoverBackground);
    if (hoverBackground) {
        collector.addRule(".monaco-editor .monaco-editor-hover { background-color: " + hoverBackground + "; }");
    }
    var hoverBorder = theme.getColor(editorHoverBorder);
    if (hoverBorder) {
        collector.addRule(".monaco-editor .monaco-editor-hover { border: 1px solid " + hoverBorder + "; }");
        collector.addRule(".monaco-editor .monaco-editor-hover .hover-row:not(:first-child):not(:empty) { border-top: 1px solid " + hoverBorder.transparent(0.5) + "; }");
    }
    var link = theme.getColor(textLinkForeground);
    if (link) {
        collector.addRule(".monaco-editor .monaco-editor-hover a { color: " + link + "; }");
    }
    var codeBackground = theme.getColor(textCodeBlockBackground);
    if (codeBackground) {
        collector.addRule(".monaco-editor .monaco-editor-hover code { background-color: " + codeBackground + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var InPlaceReplaceCommand = /** @class */ (function () {
    function InPlaceReplaceCommand(editRange, originalSelection, text) {
        this._editRange = editRange;
        this._originalSelection = originalSelection;
        this._text = text;
    }
    InPlaceReplaceCommand.prototype.getEditOperations = function (model, builder) {
        builder.addTrackedEditOperation(this._editRange, this._text);
    };
    InPlaceReplaceCommand.prototype.computeCursorState = function (model, helper) {
        var inverseEditOperations = helper.getInverseEditOperations();
        var srcRange = inverseEditOperations[0].range;
        if (!this._originalSelection.isEmpty()) {
            // Preserve selection and extends to typed text
            return new Selection(srcRange.endLineNumber, srcRange.endColumn - this._text.length, srcRange.endLineNumber, srcRange.endColumn);
        }
        return new Selection(srcRange.endLineNumber, Math.min(this._originalSelection.positionColumn, srcRange.endColumn), srcRange.endLineNumber, Math.min(this._originalSelection.positionColumn, srcRange.endColumn));
    };
    return InPlaceReplaceCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1z = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$c = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$b = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var InPlaceReplaceController = /** @class */ (function () {
    function InPlaceReplaceController(editor, editorWorkerService) {
        this.decorationIds = [];
        this.editor = editor;
        this.editorWorkerService = editorWorkerService;
    }
    InPlaceReplaceController.get = function (editor) {
        return editor.getContribution(InPlaceReplaceController.ID);
    };
    InPlaceReplaceController.prototype.dispose = function () {
    };
    InPlaceReplaceController.prototype.getId = function () {
        return InPlaceReplaceController.ID;
    };
    InPlaceReplaceController.prototype.run = function (source, up) {
        var _this = this;
        // cancel any pending request
        if (this.currentRequest) {
            this.currentRequest.cancel();
        }
        var selection = this.editor.getSelection();
        var model = this.editor.getModel();
        var modelURI = model.uri;
        if (selection.startLineNumber !== selection.endLineNumber) {
            // Can't accept multiline selection
            return null;
        }
        var state = new EditorState(this.editor, 1 /* Value */ | 4 /* Position */);
        if (!this.editorWorkerService.canNavigateValueSet(modelURI)) {
            return undefined;
        }
        this.currentRequest = createCancelablePromise(function (token) { return _this.editorWorkerService.navigateValueSet(modelURI, selection, up); });
        return this.currentRequest.then(function (result) {
            if (!result || !result.range || !result.value) {
                // No proper result
                return;
            }
            if (!state.validate(_this.editor)) {
                // state has changed
                return;
            }
            // Selection
            var editRange = Range.lift(result.range);
            var highlightRange = result.range;
            var diff = result.value.length - (selection.endColumn - selection.startColumn);
            // highlight
            highlightRange = {
                startLineNumber: highlightRange.startLineNumber,
                startColumn: highlightRange.startColumn,
                endLineNumber: highlightRange.endLineNumber,
                endColumn: highlightRange.startColumn + result.value.length
            };
            if (diff > 1) {
                selection = new Selection(selection.startLineNumber, selection.startColumn, selection.endLineNumber, selection.endColumn + diff - 1);
            }
            // Insert new text
            var command = new InPlaceReplaceCommand(editRange, selection, result.value);
            _this.editor.pushUndoStop();
            _this.editor.executeCommand(source, command);
            _this.editor.pushUndoStop();
            // add decoration
            _this.decorationIds = _this.editor.deltaDecorations(_this.decorationIds, [{
                    range: highlightRange,
                    options: InPlaceReplaceController.DECORATION
                }]);
            // remove decoration after delay
            if (_this.decorationRemover) {
                _this.decorationRemover.cancel();
            }
            _this.decorationRemover = timeout(350);
            _this.decorationRemover.then(function () { return _this.decorationIds = _this.editor.deltaDecorations(_this.decorationIds, []); }).catch(onUnexpectedError);
        }).catch(onUnexpectedError);
    };
    InPlaceReplaceController.ID = 'editor.contrib.inPlaceReplaceController';
    InPlaceReplaceController.DECORATION = ModelDecorationOptions.register({
        className: 'valueSetReplacement'
    });
    InPlaceReplaceController = __decorate$c([
        __param$b(1, IEditorWorkerService)
    ], InPlaceReplaceController);
    return InPlaceReplaceController;
}());
var InPlaceReplaceUp = /** @class */ (function (_super) {
    __extends$1z(InPlaceReplaceUp, _super);
    function InPlaceReplaceUp() {
        return _super.call(this, {
            id: 'editor.action.inPlaceReplace.up',
            label: localize('InPlaceReplaceAction.previous.label', "Replace with Previous Value"),
            alias: 'Replace with Previous Value',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 82 /* US_COMMA */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    InPlaceReplaceUp.prototype.run = function (accessor, editor) {
        var controller = InPlaceReplaceController.get(editor);
        if (!controller) {
            return undefined;
        }
        return controller.run(this.id, true);
    };
    return InPlaceReplaceUp;
}(EditorAction));
var InPlaceReplaceDown = /** @class */ (function (_super) {
    __extends$1z(InPlaceReplaceDown, _super);
    function InPlaceReplaceDown() {
        return _super.call(this, {
            id: 'editor.action.inPlaceReplace.down',
            label: localize('InPlaceReplaceAction.next.label', "Replace with Next Value"),
            alias: 'Replace with Next Value',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 84 /* US_DOT */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    InPlaceReplaceDown.prototype.run = function (accessor, editor) {
        var controller = InPlaceReplaceController.get(editor);
        if (!controller) {
            return undefined;
        }
        return controller.run(this.id, false);
    };
    return InPlaceReplaceDown;
}(EditorAction));
registerEditorContribution(InPlaceReplaceController);
registerEditorAction(InPlaceReplaceUp);
registerEditorAction(InPlaceReplaceDown);
registerThemingParticipant(function (theme, collector) {
    var border = theme.getColor(editorBracketMatchBorder);
    if (border) {
        collector.addRule(".monaco-editor.vs .valueSetReplacement { outline: solid 2px " + border + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var TrimTrailingWhitespaceCommand = /** @class */ (function () {
    function TrimTrailingWhitespaceCommand(selection, cursors) {
        this.selection = selection;
        this.cursors = cursors;
    }
    TrimTrailingWhitespaceCommand.prototype.getEditOperations = function (model, builder) {
        var ops = trimTrailingWhitespace(model, this.cursors);
        for (var i = 0, len = ops.length; i < len; i++) {
            var op = ops[i];
            builder.addEditOperation(op.range, op.text);
        }
        this.selectionId = builder.trackSelection(this.selection);
    };
    TrimTrailingWhitespaceCommand.prototype.computeCursorState = function (model, helper) {
        return helper.getTrackedSelection(this.selectionId);
    };
    return TrimTrailingWhitespaceCommand;
}());
/**
 * Generate commands for trimming trailing whitespace on a model and ignore lines on which cursors are sitting.
 */
function trimTrailingWhitespace(model, cursors) {
    // Sort cursors ascending
    cursors.sort(function (a, b) {
        if (a.lineNumber === b.lineNumber) {
            return a.column - b.column;
        }
        return a.lineNumber - b.lineNumber;
    });
    // Reduce multiple cursors on the same line and only keep the last one on the line
    for (var i = cursors.length - 2; i >= 0; i--) {
        if (cursors[i].lineNumber === cursors[i + 1].lineNumber) {
            // Remove cursor at `i`
            cursors.splice(i, 1);
        }
    }
    var r = [];
    var rLen = 0;
    var cursorIndex = 0;
    var cursorLen = cursors.length;
    for (var lineNumber = 1, lineCount = model.getLineCount(); lineNumber <= lineCount; lineNumber++) {
        var lineContent = model.getLineContent(lineNumber);
        var maxLineColumn = lineContent.length + 1;
        var minEditColumn = 0;
        if (cursorIndex < cursorLen && cursors[cursorIndex].lineNumber === lineNumber) {
            minEditColumn = cursors[cursorIndex].column;
            cursorIndex++;
            if (minEditColumn === maxLineColumn) {
                // The cursor is at the end of the line => no edits for sure on this line
                continue;
            }
        }
        if (lineContent.length === 0) {
            continue;
        }
        var lastNonWhitespaceIndex$$1 = lastNonWhitespaceIndex(lineContent);
        var fromColumn = 0;
        if (lastNonWhitespaceIndex$$1 === -1) {
            // Entire line is whitespace
            fromColumn = 1;
        }
        else if (lastNonWhitespaceIndex$$1 !== lineContent.length - 1) {
            // There is trailing whitespace
            fromColumn = lastNonWhitespaceIndex$$1 + 2;
        }
        else {
            // There is no trailing whitespace
            continue;
        }
        fromColumn = Math.max(minEditColumn, fromColumn);
        r[rLen++] = EditOperation.delete(new Range(lineNumber, fromColumn, lineNumber, maxLineColumn));
    }
    return r;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CopyLinesCommand = /** @class */ (function () {
    function CopyLinesCommand(selection, isCopyingDown) {
        this._selection = selection;
        this._isCopyingDown = isCopyingDown;
    }
    CopyLinesCommand.prototype.getEditOperations = function (model, builder) {
        var s = this._selection;
        this._startLineNumberDelta = 0;
        this._endLineNumberDelta = 0;
        if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
            this._endLineNumberDelta = 1;
            s = s.setEndPosition(s.endLineNumber - 1, model.getLineMaxColumn(s.endLineNumber - 1));
        }
        var sourceLines = [];
        for (var i = s.startLineNumber; i <= s.endLineNumber; i++) {
            sourceLines.push(model.getLineContent(i));
        }
        var sourceText = sourceLines.join('\n');
        if (sourceText === '') {
            // Duplicating empty line
            if (this._isCopyingDown) {
                this._startLineNumberDelta++;
                this._endLineNumberDelta++;
            }
        }
        if (!this._isCopyingDown) {
            builder.addEditOperation(new Range(s.endLineNumber, model.getLineMaxColumn(s.endLineNumber), s.endLineNumber, model.getLineMaxColumn(s.endLineNumber)), '\n' + sourceText);
        }
        else {
            builder.addEditOperation(new Range(s.startLineNumber, 1, s.startLineNumber, 1), sourceText + '\n');
        }
        this._selectionId = builder.trackSelection(s);
        this._selectionDirection = this._selection.getDirection();
    };
    CopyLinesCommand.prototype.computeCursorState = function (model, helper) {
        var result = helper.getTrackedSelection(this._selectionId);
        if (this._startLineNumberDelta !== 0 || this._endLineNumberDelta !== 0) {
            var startLineNumber = result.startLineNumber;
            var startColumn = result.startColumn;
            var endLineNumber = result.endLineNumber;
            var endColumn = result.endColumn;
            if (this._startLineNumberDelta !== 0) {
                startLineNumber = startLineNumber + this._startLineNumberDelta;
                startColumn = 1;
            }
            if (this._endLineNumberDelta !== 0) {
                endLineNumber = endLineNumber + this._endLineNumberDelta;
                endColumn = 1;
            }
            result = Selection.createWithDirection(startLineNumber, startColumn, endLineNumber, endColumn, this._selectionDirection);
        }
        return result;
    };
    return CopyLinesCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var DeleteLinesCommand = /** @class */ (function () {
    function DeleteLinesCommand(startLineNumber, endLineNumber, restoreCursorToColumn) {
        this.startLineNumber = startLineNumber;
        this.endLineNumber = endLineNumber;
        this.restoreCursorToColumn = restoreCursorToColumn;
    }
    DeleteLinesCommand.prototype.getEditOperations = function (model, builder) {
        if (model.getLineCount() === 1 && model.getLineMaxColumn(1) === 1) {
            // Model is empty
            return;
        }
        var startLineNumber = this.startLineNumber;
        var endLineNumber = this.endLineNumber;
        var startColumn = 1;
        var endColumn = model.getLineMaxColumn(endLineNumber);
        if (endLineNumber < model.getLineCount()) {
            endLineNumber += 1;
            endColumn = 1;
        }
        else if (startLineNumber > 1) {
            startLineNumber -= 1;
            startColumn = model.getLineMaxColumn(startLineNumber);
        }
        builder.addTrackedEditOperation(new Range(startLineNumber, startColumn, endLineNumber, endColumn), null);
    };
    DeleteLinesCommand.prototype.computeCursorState = function (model, helper) {
        var inverseEditOperations = helper.getInverseEditOperations();
        var srcRange = inverseEditOperations[0].range;
        return new Selection(srcRange.endLineNumber, this.restoreCursorToColumn, srcRange.endLineNumber, this.restoreCursorToColumn);
    };
    return DeleteLinesCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getSpaceCnt(str, tabSize) {
    var spacesCnt = 0;
    for (var i = 0; i < str.length; i++) {
        if (str.charAt(i) === '\t') {
            spacesCnt += tabSize;
        }
        else {
            spacesCnt++;
        }
    }
    return spacesCnt;
}
function generateIndent(spacesCnt, tabSize, insertSpaces) {
    spacesCnt = spacesCnt < 0 ? 0 : spacesCnt;
    var result = '';
    if (!insertSpaces) {
        var tabsCnt = Math.floor(spacesCnt / tabSize);
        spacesCnt = spacesCnt % tabSize;
        for (var i = 0; i < tabsCnt; i++) {
            result += '\t';
        }
    }
    for (var i = 0; i < spacesCnt; i++) {
        result += ' ';
    }
    return result;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MoveLinesCommand = /** @class */ (function () {
    function MoveLinesCommand(selection, isMovingDown, autoIndent) {
        this._selection = selection;
        this._isMovingDown = isMovingDown;
        this._autoIndent = autoIndent;
        this._moveEndLineSelectionShrink = false;
    }
    MoveLinesCommand.prototype.getEditOperations = function (model, builder) {
        var modelLineCount = model.getLineCount();
        if (this._isMovingDown && this._selection.endLineNumber === modelLineCount) {
            return;
        }
        if (!this._isMovingDown && this._selection.startLineNumber === 1) {
            return;
        }
        this._moveEndPositionDown = false;
        var s = this._selection;
        if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
            this._moveEndPositionDown = true;
            s = s.setEndPosition(s.endLineNumber - 1, model.getLineMaxColumn(s.endLineNumber - 1));
        }
        var tabSize = model.getOptions().tabSize;
        var insertSpaces = model.getOptions().insertSpaces;
        var indentConverter = this.buildIndentConverter(tabSize);
        var virtualModel = {
            getLineTokens: function (lineNumber) {
                return model.getLineTokens(lineNumber);
            },
            getLanguageIdentifier: function () {
                return model.getLanguageIdentifier();
            },
            getLanguageIdAtPosition: function (lineNumber, column) {
                return model.getLanguageIdAtPosition(lineNumber, column);
            },
            getLineContent: null,
        };
        if (s.startLineNumber === s.endLineNumber && model.getLineMaxColumn(s.startLineNumber) === 1) {
            // Current line is empty
            var lineNumber = s.startLineNumber;
            var otherLineNumber = (this._isMovingDown ? lineNumber + 1 : lineNumber - 1);
            if (model.getLineMaxColumn(otherLineNumber) === 1) {
                // Other line number is empty too, so no editing is needed
                // Add a no-op to force running by the model
                builder.addEditOperation(new Range(1, 1, 1, 1), null);
            }
            else {
                // Type content from other line number on line number
                builder.addEditOperation(new Range(lineNumber, 1, lineNumber, 1), model.getLineContent(otherLineNumber));
                // Remove content from other line number
                builder.addEditOperation(new Range(otherLineNumber, 1, otherLineNumber, model.getLineMaxColumn(otherLineNumber)), null);
            }
            // Track selection at the other line number
            s = new Selection(otherLineNumber, 1, otherLineNumber, 1);
        }
        else {
            var movingLineNumber_1;
            var movingLineText = void 0;
            if (this._isMovingDown) {
                movingLineNumber_1 = s.endLineNumber + 1;
                movingLineText = model.getLineContent(movingLineNumber_1);
                // Delete line that needs to be moved
                builder.addEditOperation(new Range(movingLineNumber_1 - 1, model.getLineMaxColumn(movingLineNumber_1 - 1), movingLineNumber_1, model.getLineMaxColumn(movingLineNumber_1)), null);
                var insertingText_1 = movingLineText;
                if (this.shouldAutoIndent(model, s)) {
                    var movingLineMatchResult = this.matchEnterRule(model, indentConverter, tabSize, movingLineNumber_1, s.startLineNumber - 1);
                    // if s.startLineNumber - 1 matches onEnter rule, we still honor that.
                    if (movingLineMatchResult !== null) {
                        var oldIndentation = getLeadingWhitespace(model.getLineContent(movingLineNumber_1));
                        var newSpaceCnt = movingLineMatchResult + getSpaceCnt(oldIndentation, tabSize);
                        var newIndentation = generateIndent(newSpaceCnt, tabSize, insertSpaces);
                        insertingText_1 = newIndentation + this.trimLeft(movingLineText);
                    }
                    else {
                        // no enter rule matches, let's check indentatin rules then.
                        virtualModel.getLineContent = function (lineNumber) {
                            if (lineNumber === s.startLineNumber) {
                                return model.getLineContent(movingLineNumber_1);
                            }
                            else {
                                return model.getLineContent(lineNumber);
                            }
                        };
                        var indentOfMovingLine = LanguageConfigurationRegistry.getGoodIndentForLine(virtualModel, model.getLanguageIdAtPosition(movingLineNumber_1, 1), s.startLineNumber, indentConverter);
                        if (indentOfMovingLine !== null) {
                            var oldIndentation = getLeadingWhitespace(model.getLineContent(movingLineNumber_1));
                            var newSpaceCnt = getSpaceCnt(indentOfMovingLine, tabSize);
                            var oldSpaceCnt = getSpaceCnt(oldIndentation, tabSize);
                            if (newSpaceCnt !== oldSpaceCnt) {
                                var newIndentation = generateIndent(newSpaceCnt, tabSize, insertSpaces);
                                insertingText_1 = newIndentation + this.trimLeft(movingLineText);
                            }
                        }
                    }
                    // add edit operations for moving line first to make sure it's executed after we make indentation change
                    // to s.startLineNumber
                    builder.addEditOperation(new Range(s.startLineNumber, 1, s.startLineNumber, 1), insertingText_1 + '\n');
                    var ret = this.matchEnterRule(model, indentConverter, tabSize, s.startLineNumber, s.startLineNumber, insertingText_1);
                    // check if the line being moved before matches onEnter rules, if so let's adjust the indentation by onEnter rules.
                    if (ret !== null) {
                        if (ret !== 0) {
                            this.getIndentEditsOfMovingBlock(model, builder, s, tabSize, insertSpaces, ret);
                        }
                    }
                    else {
                        // it doesn't match onEnter rules, let's check indentation rules then.
                        virtualModel.getLineContent = function (lineNumber) {
                            if (lineNumber === s.startLineNumber) {
                                return insertingText_1;
                            }
                            else if (lineNumber >= s.startLineNumber + 1 && lineNumber <= s.endLineNumber + 1) {
                                return model.getLineContent(lineNumber - 1);
                            }
                            else {
                                return model.getLineContent(lineNumber);
                            }
                        };
                        var newIndentatOfMovingBlock = LanguageConfigurationRegistry.getGoodIndentForLine(virtualModel, model.getLanguageIdAtPosition(movingLineNumber_1, 1), s.startLineNumber + 1, indentConverter);
                        if (newIndentatOfMovingBlock !== null) {
                            var oldIndentation = getLeadingWhitespace(model.getLineContent(s.startLineNumber));
                            var newSpaceCnt = getSpaceCnt(newIndentatOfMovingBlock, tabSize);
                            var oldSpaceCnt = getSpaceCnt(oldIndentation, tabSize);
                            if (newSpaceCnt !== oldSpaceCnt) {
                                var spaceCntOffset = newSpaceCnt - oldSpaceCnt;
                                this.getIndentEditsOfMovingBlock(model, builder, s, tabSize, insertSpaces, spaceCntOffset);
                            }
                        }
                    }
                }
                else {
                    // Insert line that needs to be moved before
                    builder.addEditOperation(new Range(s.startLineNumber, 1, s.startLineNumber, 1), insertingText_1 + '\n');
                }
            }
            else {
                movingLineNumber_1 = s.startLineNumber - 1;
                movingLineText = model.getLineContent(movingLineNumber_1);
                // Delete line that needs to be moved
                builder.addEditOperation(new Range(movingLineNumber_1, 1, movingLineNumber_1 + 1, 1), null);
                // Insert line that needs to be moved after
                builder.addEditOperation(new Range(s.endLineNumber, model.getLineMaxColumn(s.endLineNumber), s.endLineNumber, model.getLineMaxColumn(s.endLineNumber)), '\n' + movingLineText);
                if (this.shouldAutoIndent(model, s)) {
                    virtualModel.getLineContent = function (lineNumber) {
                        if (lineNumber === movingLineNumber_1) {
                            return model.getLineContent(s.startLineNumber);
                        }
                        else {
                            return model.getLineContent(lineNumber);
                        }
                    };
                    var ret = this.matchEnterRule(model, indentConverter, tabSize, s.startLineNumber, s.startLineNumber - 2);
                    // check if s.startLineNumber - 2 matches onEnter rules, if so adjust the moving block by onEnter rules.
                    if (ret !== null) {
                        if (ret !== 0) {
                            this.getIndentEditsOfMovingBlock(model, builder, s, tabSize, insertSpaces, ret);
                        }
                    }
                    else {
                        // it doesn't match any onEnter rule, let's check indentation rules then.
                        var indentOfFirstLine = LanguageConfigurationRegistry.getGoodIndentForLine(virtualModel, model.getLanguageIdAtPosition(s.startLineNumber, 1), movingLineNumber_1, indentConverter);
                        if (indentOfFirstLine !== null) {
                            // adjust the indentation of the moving block
                            var oldIndent = getLeadingWhitespace(model.getLineContent(s.startLineNumber));
                            var newSpaceCnt = getSpaceCnt(indentOfFirstLine, tabSize);
                            var oldSpaceCnt = getSpaceCnt(oldIndent, tabSize);
                            if (newSpaceCnt !== oldSpaceCnt) {
                                var spaceCntOffset = newSpaceCnt - oldSpaceCnt;
                                this.getIndentEditsOfMovingBlock(model, builder, s, tabSize, insertSpaces, spaceCntOffset);
                            }
                        }
                    }
                }
            }
        }
        this._selectionId = builder.trackSelection(s);
    };
    MoveLinesCommand.prototype.buildIndentConverter = function (tabSize) {
        return {
            shiftIndent: function (indentation) {
                var desiredIndentCount = ShiftCommand.shiftIndentCount(indentation, indentation.length + 1, tabSize);
                var newIndentation = '';
                for (var i = 0; i < desiredIndentCount; i++) {
                    newIndentation += '\t';
                }
                return newIndentation;
            },
            unshiftIndent: function (indentation) {
                var desiredIndentCount = ShiftCommand.unshiftIndentCount(indentation, indentation.length + 1, tabSize);
                var newIndentation = '';
                for (var i = 0; i < desiredIndentCount; i++) {
                    newIndentation += '\t';
                }
                return newIndentation;
            }
        };
    };
    MoveLinesCommand.prototype.matchEnterRule = function (model, indentConverter, tabSize, line, oneLineAbove, oneLineAboveText) {
        var validPrecedingLine = oneLineAbove;
        while (validPrecedingLine >= 1) {
            // ship empty lines as empty lines just inherit indentation
            var lineContent = void 0;
            if (validPrecedingLine === oneLineAbove && oneLineAboveText !== undefined) {
                lineContent = oneLineAboveText;
            }
            else {
                lineContent = model.getLineContent(validPrecedingLine);
            }
            var nonWhitespaceIdx = lastNonWhitespaceIndex(lineContent);
            if (nonWhitespaceIdx >= 0) {
                break;
            }
            validPrecedingLine--;
        }
        if (validPrecedingLine < 1 || line > model.getLineCount()) {
            return null;
        }
        var maxColumn = model.getLineMaxColumn(validPrecedingLine);
        var enter = LanguageConfigurationRegistry.getEnterAction(model, new Range(validPrecedingLine, maxColumn, validPrecedingLine, maxColumn));
        if (enter) {
            var enterPrefix = enter.indentation;
            var enterAction = enter.enterAction;
            if (enterAction.indentAction === IndentAction.None) {
                enterPrefix = enter.indentation + enterAction.appendText;
            }
            else if (enterAction.indentAction === IndentAction.Indent) {
                enterPrefix = enter.indentation + enterAction.appendText;
            }
            else if (enterAction.indentAction === IndentAction.IndentOutdent) {
                enterPrefix = enter.indentation;
            }
            else if (enterAction.indentAction === IndentAction.Outdent) {
                enterPrefix = indentConverter.unshiftIndent(enter.indentation) + enterAction.appendText;
            }
            var movingLineText = model.getLineContent(line);
            if (this.trimLeft(movingLineText).indexOf(this.trimLeft(enterPrefix)) >= 0) {
                var oldIndentation = getLeadingWhitespace(model.getLineContent(line));
                var newIndentation = getLeadingWhitespace(enterPrefix);
                var indentMetadataOfMovelingLine = LanguageConfigurationRegistry.getIndentMetadata(model, line);
                if (indentMetadataOfMovelingLine !== null && indentMetadataOfMovelingLine & 2 /* DECREASE_MASK */) {
                    newIndentation = indentConverter.unshiftIndent(newIndentation);
                }
                var newSpaceCnt = getSpaceCnt(newIndentation, tabSize);
                var oldSpaceCnt = getSpaceCnt(oldIndentation, tabSize);
                return newSpaceCnt - oldSpaceCnt;
            }
        }
        return null;
    };
    MoveLinesCommand.prototype.trimLeft = function (str) {
        return str.replace(/^\s+/, '');
    };
    MoveLinesCommand.prototype.shouldAutoIndent = function (model, selection) {
        if (!this._autoIndent) {
            return false;
        }
        // if it's not easy to tokenize, we stop auto indent.
        if (!model.isCheapToTokenize(selection.startLineNumber)) {
            return false;
        }
        var languageAtSelectionStart = model.getLanguageIdAtPosition(selection.startLineNumber, 1);
        var languageAtSelectionEnd = model.getLanguageIdAtPosition(selection.endLineNumber, 1);
        if (languageAtSelectionStart !== languageAtSelectionEnd) {
            return false;
        }
        if (LanguageConfigurationRegistry.getIndentRulesSupport(languageAtSelectionStart) === null) {
            return false;
        }
        return true;
    };
    MoveLinesCommand.prototype.getIndentEditsOfMovingBlock = function (model, builder, s, tabSize, insertSpaces, offset) {
        for (var i = s.startLineNumber; i <= s.endLineNumber; i++) {
            var lineContent = model.getLineContent(i);
            var originalIndent = getLeadingWhitespace(lineContent);
            var originalSpacesCnt = getSpaceCnt(originalIndent, tabSize);
            var newSpacesCnt = originalSpacesCnt + offset;
            var newIndent = generateIndent(newSpacesCnt, tabSize, insertSpaces);
            if (newIndent !== originalIndent) {
                builder.addEditOperation(new Range(i, 1, i, originalIndent.length + 1), newIndent);
                if (i === s.endLineNumber && s.endColumn <= originalIndent.length + 1 && newIndent === '') {
                    // as users select part of the original indent white spaces
                    // when we adjust the indentation of endLine, we should adjust the cursor position as well.
                    this._moveEndLineSelectionShrink = true;
                }
            }
        }
    };
    MoveLinesCommand.prototype.computeCursorState = function (model, helper) {
        var result = helper.getTrackedSelection(this._selectionId);
        if (this._moveEndPositionDown) {
            result = result.setEndPosition(result.endLineNumber + 1, 1);
        }
        if (this._moveEndLineSelectionShrink && result.startLineNumber < result.endLineNumber) {
            result = result.setEndPosition(result.endLineNumber, 2);
        }
        return result;
    };
    return MoveLinesCommand;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var SortLinesCommand = /** @class */ (function () {
    function SortLinesCommand(selection, descending) {
        this.selection = selection;
        this.descending = descending;
    }
    SortLinesCommand.prototype.getEditOperations = function (model, builder) {
        var op = sortLines(model, this.selection, this.descending);
        if (op) {
            builder.addEditOperation(op.range, op.text);
        }
        this.selectionId = builder.trackSelection(this.selection);
    };
    SortLinesCommand.prototype.computeCursorState = function (model, helper) {
        return helper.getTrackedSelection(this.selectionId);
    };
    SortLinesCommand.canRun = function (model, selection, descending) {
        var data = getSortData(model, selection, descending);
        if (!data) {
            return false;
        }
        for (var i = 0, len = data.before.length; i < len; i++) {
            if (data.before[i] !== data.after[i]) {
                return true;
            }
        }
        return false;
    };
    return SortLinesCommand;
}());
function getSortData(model, selection, descending) {
    var startLineNumber = selection.startLineNumber;
    var endLineNumber = selection.endLineNumber;
    if (selection.endColumn === 1) {
        endLineNumber--;
    }
    // Nothing to sort if user didn't select anything.
    if (startLineNumber >= endLineNumber) {
        return null;
    }
    var linesToSort = [];
    // Get the contents of the selection to be sorted.
    for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
        linesToSort.push(model.getLineContent(lineNumber));
    }
    var sorted = linesToSort.slice(0);
    sorted.sort(function (a, b) {
        return a.toLowerCase().localeCompare(b.toLowerCase());
    });
    // If descending, reverse the order.
    if (descending === true) {
        sorted = sorted.reverse();
    }
    return {
        startLineNumber: startLineNumber,
        endLineNumber: endLineNumber,
        before: linesToSort,
        after: sorted
    };
}
/**
 * Generate commands for sorting lines on a model.
 */
function sortLines(model, selection, descending) {
    var data = getSortData(model, selection, descending);
    if (!data) {
        return null;
    }
    return EditOperation.replace(new Range(data.startLineNumber, 1, data.endLineNumber, model.getLineMaxColumn(data.endLineNumber)), data.after.join('\n'));
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1A = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// copy lines
var AbstractCopyLinesAction = /** @class */ (function (_super) {
    __extends$1A(AbstractCopyLinesAction, _super);
    function AbstractCopyLinesAction(down, opts) {
        var _this = _super.call(this, opts) || this;
        _this.down = down;
        return _this;
    }
    AbstractCopyLinesAction.prototype.run = function (_accessor, editor) {
        var commands = [];
        var selections = editor.getSelections() || [];
        for (var i = 0; i < selections.length; i++) {
            commands.push(new CopyLinesCommand(selections[i], this.down));
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return AbstractCopyLinesAction;
}(EditorAction));
var CopyLinesUpAction = /** @class */ (function (_super) {
    __extends$1A(CopyLinesUpAction, _super);
    function CopyLinesUpAction() {
        return _super.call(this, false, {
            id: 'editor.action.copyLinesUpAction',
            label: localize('lines.copyUp', "Copy Line Up"),
            alias: 'Copy Line Up',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 512 /* Alt */ | 1024 /* Shift */ | 16 /* UpArrow */,
                linux: { primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 1024 /* Shift */ | 16 /* UpArrow */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarSelectionMenu,
                group: '2_line',
                title: localize({ key: 'miCopyLinesUp', comment: ['&& denotes a mnemonic'] }, "&&Copy Line Up"),
                order: 1
            }
        }) || this;
    }
    return CopyLinesUpAction;
}(AbstractCopyLinesAction));
var CopyLinesDownAction = /** @class */ (function (_super) {
    __extends$1A(CopyLinesDownAction, _super);
    function CopyLinesDownAction() {
        return _super.call(this, true, {
            id: 'editor.action.copyLinesDownAction',
            label: localize('lines.copyDown', "Copy Line Down"),
            alias: 'Copy Line Down',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 512 /* Alt */ | 1024 /* Shift */ | 18 /* DownArrow */,
                linux: { primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 1024 /* Shift */ | 18 /* DownArrow */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarSelectionMenu,
                group: '2_line',
                title: localize({ key: 'miCopyLinesDown', comment: ['&& denotes a mnemonic'] }, "Co&&py Line Down"),
                order: 2
            }
        }) || this;
    }
    return CopyLinesDownAction;
}(AbstractCopyLinesAction));
// move lines
var AbstractMoveLinesAction = /** @class */ (function (_super) {
    __extends$1A(AbstractMoveLinesAction, _super);
    function AbstractMoveLinesAction(down, opts) {
        var _this = _super.call(this, opts) || this;
        _this.down = down;
        return _this;
    }
    AbstractMoveLinesAction.prototype.run = function (_accessor, editor) {
        var commands = [];
        var selections = editor.getSelections() || [];
        var autoIndent = editor.getConfiguration().autoIndent;
        for (var i = 0; i < selections.length; i++) {
            commands.push(new MoveLinesCommand(selections[i], this.down, autoIndent));
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return AbstractMoveLinesAction;
}(EditorAction));
var MoveLinesUpAction = /** @class */ (function (_super) {
    __extends$1A(MoveLinesUpAction, _super);
    function MoveLinesUpAction() {
        return _super.call(this, false, {
            id: 'editor.action.moveLinesUpAction',
            label: localize('lines.moveUp', "Move Line Up"),
            alias: 'Move Line Up',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 512 /* Alt */ | 16 /* UpArrow */,
                linux: { primary: 512 /* Alt */ | 16 /* UpArrow */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarSelectionMenu,
                group: '2_line',
                title: localize({ key: 'miMoveLinesUp', comment: ['&& denotes a mnemonic'] }, "Mo&&ve Line Up"),
                order: 3
            }
        }) || this;
    }
    return MoveLinesUpAction;
}(AbstractMoveLinesAction));
var MoveLinesDownAction = /** @class */ (function (_super) {
    __extends$1A(MoveLinesDownAction, _super);
    function MoveLinesDownAction() {
        return _super.call(this, true, {
            id: 'editor.action.moveLinesDownAction',
            label: localize('lines.moveDown', "Move Line Down"),
            alias: 'Move Line Down',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 512 /* Alt */ | 18 /* DownArrow */,
                linux: { primary: 512 /* Alt */ | 18 /* DownArrow */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarSelectionMenu,
                group: '2_line',
                title: localize({ key: 'miMoveLinesDown', comment: ['&& denotes a mnemonic'] }, "Move &&Line Down"),
                order: 4
            }
        }) || this;
    }
    return MoveLinesDownAction;
}(AbstractMoveLinesAction));
var AbstractSortLinesAction = /** @class */ (function (_super) {
    __extends$1A(AbstractSortLinesAction, _super);
    function AbstractSortLinesAction(descending, opts) {
        var _this = _super.call(this, opts) || this;
        _this.descending = descending;
        return _this;
    }
    AbstractSortLinesAction.prototype.run = function (_accessor, editor) {
        var selections = editor.getSelections() || [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (!SortLinesCommand.canRun(editor.getModel(), selection, this.descending)) {
                return;
            }
        }
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            commands[i] = new SortLinesCommand(selections[i], this.descending);
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return AbstractSortLinesAction;
}(EditorAction));
var SortLinesAscendingAction = /** @class */ (function (_super) {
    __extends$1A(SortLinesAscendingAction, _super);
    function SortLinesAscendingAction() {
        return _super.call(this, false, {
            id: 'editor.action.sortLinesAscending',
            label: localize('lines.sortAscending', "Sort Lines Ascending"),
            alias: 'Sort Lines Ascending',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return SortLinesAscendingAction;
}(AbstractSortLinesAction));
var SortLinesDescendingAction = /** @class */ (function (_super) {
    __extends$1A(SortLinesDescendingAction, _super);
    function SortLinesDescendingAction() {
        return _super.call(this, true, {
            id: 'editor.action.sortLinesDescending',
            label: localize('lines.sortDescending', "Sort Lines Descending"),
            alias: 'Sort Lines Descending',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return SortLinesDescendingAction;
}(AbstractSortLinesAction));
var TrimTrailingWhitespaceAction = /** @class */ (function (_super) {
    __extends$1A(TrimTrailingWhitespaceAction, _super);
    function TrimTrailingWhitespaceAction() {
        return _super.call(this, {
            id: TrimTrailingWhitespaceAction.ID,
            label: localize('lines.trimTrailingWhitespace', "Trim Trailing Whitespace"),
            alias: 'Trim Trailing Whitespace',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: KeyChord(2048 /* CtrlCmd */ | 41 /* KEY_K */, 2048 /* CtrlCmd */ | 54 /* KEY_X */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    TrimTrailingWhitespaceAction.prototype.run = function (_accessor, editor, args) {
        var cursors = [];
        if (args.reason === 'auto-save') {
            // See https://github.com/editorconfig/editorconfig-vscode/issues/47
            // It is very convenient for the editor config extension to invoke this action.
            // So, if we get a reason:'auto-save' passed in, let's preserve cursor positions.
            cursors = (editor.getSelections() || []).map(function (s) { return new Position(s.positionLineNumber, s.positionColumn); });
        }
        var command = new TrimTrailingWhitespaceCommand(editor.getSelection(), cursors);
        editor.pushUndoStop();
        editor.executeCommands(this.id, [command]);
        editor.pushUndoStop();
    };
    TrimTrailingWhitespaceAction.ID = 'editor.action.trimTrailingWhitespace';
    return TrimTrailingWhitespaceAction;
}(EditorAction));
var DeleteLinesAction = /** @class */ (function (_super) {
    __extends$1A(DeleteLinesAction, _super);
    function DeleteLinesAction() {
        return _super.call(this, {
            id: 'editor.action.deleteLines',
            label: localize('lines.delete', "Delete Line"),
            alias: 'Delete Line',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 41 /* KEY_K */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    DeleteLinesAction.prototype.run = function (_accessor, editor) {
        var ops = this._getLinesToRemove(editor);
        // Finally, construct the delete lines commands
        var commands = ops.map(function (op) {
            return new DeleteLinesCommand(op.startLineNumber, op.endLineNumber, op.positionColumn);
        });
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    DeleteLinesAction.prototype._getLinesToRemove = function (editor) {
        // Construct delete operations
        var operations = editor.getSelections().map(function (s) {
            var endLineNumber = s.endLineNumber;
            if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
                endLineNumber -= 1;
            }
            return {
                startLineNumber: s.startLineNumber,
                endLineNumber: endLineNumber,
                positionColumn: s.positionColumn
            };
        });
        // Sort delete operations
        operations.sort(function (a, b) {
            return a.startLineNumber - b.startLineNumber;
        });
        // Merge delete operations on consecutive lines
        var mergedOperations = [];
        var previousOperation = operations[0];
        for (var i = 1; i < operations.length; i++) {
            if (previousOperation.endLineNumber + 1 === operations[i].startLineNumber) {
                // Merge current operations into the previous one
                previousOperation.endLineNumber = operations[i].endLineNumber;
            }
            else {
                // Push previous operation
                mergedOperations.push(previousOperation);
                previousOperation = operations[i];
            }
        }
        // Push the last operation
        mergedOperations.push(previousOperation);
        return mergedOperations;
    };
    return DeleteLinesAction;
}(EditorAction));
var IndentLinesAction = /** @class */ (function (_super) {
    __extends$1A(IndentLinesAction, _super);
    function IndentLinesAction() {
        return _super.call(this, {
            id: 'editor.action.indentLines',
            label: localize('lines.indent', "Indent Line"),
            alias: 'Indent Line',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 89 /* US_CLOSE_SQUARE_BRACKET */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    IndentLinesAction.prototype.run = function (_accessor, editor) {
        var cursors = editor._getCursors();
        if (!cursors) {
            return;
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, TypeOperations.indent(cursors.context.config, editor.getModel(), editor.getSelections()));
        editor.pushUndoStop();
    };
    return IndentLinesAction;
}(EditorAction));
var OutdentLinesAction = /** @class */ (function (_super) {
    __extends$1A(OutdentLinesAction, _super);
    function OutdentLinesAction() {
        return _super.call(this, {
            id: 'editor.action.outdentLines',
            label: localize('lines.outdent', "Outdent Line"),
            alias: 'Outdent Line',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 87 /* US_OPEN_SQUARE_BRACKET */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    OutdentLinesAction.prototype.run = function (_accessor, editor) {
        CoreEditingCommands.Outdent.runEditorCommand(null, editor, null);
    };
    return OutdentLinesAction;
}(EditorAction));
var InsertLineBeforeAction = /** @class */ (function (_super) {
    __extends$1A(InsertLineBeforeAction, _super);
    function InsertLineBeforeAction() {
        return _super.call(this, {
            id: 'editor.action.insertLineBefore',
            label: localize('lines.insertBefore', "Insert Line Above"),
            alias: 'Insert Line Above',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 3 /* Enter */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    InsertLineBeforeAction.prototype.run = function (_accessor, editor) {
        var cursors = editor._getCursors();
        if (!cursors) {
            return;
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, TypeOperations.lineInsertBefore(cursors.context.config, editor.getModel(), editor.getSelections()));
    };
    return InsertLineBeforeAction;
}(EditorAction));
var InsertLineAfterAction = /** @class */ (function (_super) {
    __extends$1A(InsertLineAfterAction, _super);
    function InsertLineAfterAction() {
        return _super.call(this, {
            id: 'editor.action.insertLineAfter',
            label: localize('lines.insertAfter', "Insert Line Below"),
            alias: 'Insert Line Below',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 2048 /* CtrlCmd */ | 3 /* Enter */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    InsertLineAfterAction.prototype.run = function (_accessor, editor) {
        var cursors = editor._getCursors();
        if (!cursors) {
            return;
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, TypeOperations.lineInsertAfter(cursors.context.config, editor.getModel(), editor.getSelections()));
    };
    return InsertLineAfterAction;
}(EditorAction));
var AbstractDeleteAllToBoundaryAction = /** @class */ (function (_super) {
    __extends$1A(AbstractDeleteAllToBoundaryAction, _super);
    function AbstractDeleteAllToBoundaryAction() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AbstractDeleteAllToBoundaryAction.prototype.run = function (_accessor, editor) {
        var primaryCursor = editor.getSelection();
        var rangesToDelete = this._getRangesToDelete(editor);
        // merge overlapping selections
        var effectiveRanges = [];
        for (var i = 0, count = rangesToDelete.length - 1; i < count; i++) {
            var range = rangesToDelete[i];
            var nextRange = rangesToDelete[i + 1];
            if (Range.intersectRanges(range, nextRange) === null) {
                effectiveRanges.push(range);
            }
            else {
                rangesToDelete[i + 1] = Range.plusRange(range, nextRange);
            }
        }
        effectiveRanges.push(rangesToDelete[rangesToDelete.length - 1]);
        var endCursorState = this._getEndCursorState(primaryCursor, effectiveRanges);
        var edits = effectiveRanges.map(function (range) {
            return EditOperation.replace(range, '');
        });
        editor.pushUndoStop();
        editor.executeEdits(this.id, edits, endCursorState);
        editor.pushUndoStop();
    };
    return AbstractDeleteAllToBoundaryAction;
}(EditorAction));
var DeleteAllLeftAction = /** @class */ (function (_super) {
    __extends$1A(DeleteAllLeftAction, _super);
    function DeleteAllLeftAction() {
        return _super.call(this, {
            id: 'deleteAllLeft',
            label: localize('lines.deleteAllLeft', "Delete All Left"),
            alias: 'Delete All Left',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 0,
                mac: { primary: 2048 /* CtrlCmd */ | 1 /* Backspace */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    DeleteAllLeftAction.prototype._getEndCursorState = function (primaryCursor, rangesToDelete) {
        var endPrimaryCursor;
        var endCursorState = [];
        var deletedLines = 0;
        rangesToDelete.forEach(function (range) {
            var endCursor;
            if (range.endColumn === 1 && deletedLines > 0) {
                var newStartLine = range.startLineNumber - deletedLines;
                endCursor = new Selection(newStartLine, range.startColumn, newStartLine, range.startColumn);
            }
            else {
                endCursor = new Selection(range.startLineNumber, range.startColumn, range.startLineNumber, range.startColumn);
            }
            deletedLines += range.endLineNumber - range.startLineNumber;
            if (range.intersectRanges(primaryCursor)) {
                endPrimaryCursor = endCursor;
            }
            else {
                endCursorState.push(endCursor);
            }
        });
        if (endPrimaryCursor) {
            endCursorState.unshift(endPrimaryCursor);
        }
        return endCursorState;
    };
    DeleteAllLeftAction.prototype._getRangesToDelete = function (editor) {
        var rangesToDelete = editor.getSelections();
        var model = editor.getModel();
        rangesToDelete.sort(Range.compareRangesUsingStarts);
        rangesToDelete = rangesToDelete.map(function (selection) {
            if (selection.isEmpty()) {
                if (selection.startColumn === 1) {
                    var deleteFromLine = Math.max(1, selection.startLineNumber - 1);
                    var deleteFromColumn = selection.startLineNumber === 1 ? 1 : model.getLineContent(deleteFromLine).length + 1;
                    return new Range(deleteFromLine, deleteFromColumn, selection.startLineNumber, 1);
                }
                else {
                    return new Range(selection.startLineNumber, 1, selection.startLineNumber, selection.startColumn);
                }
            }
            else {
                return selection;
            }
        });
        return rangesToDelete;
    };
    return DeleteAllLeftAction;
}(AbstractDeleteAllToBoundaryAction));
var DeleteAllRightAction = /** @class */ (function (_super) {
    __extends$1A(DeleteAllRightAction, _super);
    function DeleteAllRightAction() {
        return _super.call(this, {
            id: 'deleteAllRight',
            label: localize('lines.deleteAllRight', "Delete All Right"),
            alias: 'Delete All Right',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 0,
                mac: { primary: 256 /* WinCtrl */ | 41 /* KEY_K */, secondary: [2048 /* CtrlCmd */ | 20 /* Delete */] },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    DeleteAllRightAction.prototype._getEndCursorState = function (primaryCursor, rangesToDelete) {
        var endPrimaryCursor;
        var endCursorState = [];
        for (var i = 0, len = rangesToDelete.length, offset = 0; i < len; i++) {
            var range = rangesToDelete[i];
            var endCursor = new Selection(range.startLineNumber - offset, range.startColumn, range.startLineNumber - offset, range.startColumn);
            if (range.intersectRanges(primaryCursor)) {
                endPrimaryCursor = endCursor;
            }
            else {
                endCursorState.push(endCursor);
            }
        }
        if (endPrimaryCursor) {
            endCursorState.unshift(endPrimaryCursor);
        }
        return endCursorState;
    };
    DeleteAllRightAction.prototype._getRangesToDelete = function (editor) {
        var model = editor.getModel();
        var rangesToDelete = editor.getSelections().map(function (sel) {
            if (sel.isEmpty()) {
                var maxColumn = model.getLineMaxColumn(sel.startLineNumber);
                if (sel.startColumn === maxColumn) {
                    return new Range(sel.startLineNumber, sel.startColumn, sel.startLineNumber + 1, 1);
                }
                else {
                    return new Range(sel.startLineNumber, sel.startColumn, sel.startLineNumber, maxColumn);
                }
            }
            return sel;
        });
        rangesToDelete.sort(Range.compareRangesUsingStarts);
        return rangesToDelete;
    };
    return DeleteAllRightAction;
}(AbstractDeleteAllToBoundaryAction));
var JoinLinesAction = /** @class */ (function (_super) {
    __extends$1A(JoinLinesAction, _super);
    function JoinLinesAction() {
        return _super.call(this, {
            id: 'editor.action.joinLines',
            label: localize('lines.joinLines', "Join Lines"),
            alias: 'Join Lines',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 0,
                mac: { primary: 256 /* WinCtrl */ | 40 /* KEY_J */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    JoinLinesAction.prototype.run = function (_accessor, editor) {
        var selections = editor.getSelections();
        var primaryCursor = editor.getSelection();
        selections.sort(Range.compareRangesUsingStarts);
        var reducedSelections = [];
        var lastSelection = selections.reduce(function (previousValue, currentValue) {
            if (previousValue.isEmpty()) {
                if (previousValue.endLineNumber === currentValue.startLineNumber) {
                    if (primaryCursor.equalsSelection(previousValue)) {
                        primaryCursor = currentValue;
                    }
                    return currentValue;
                }
                if (currentValue.startLineNumber > previousValue.endLineNumber + 1) {
                    reducedSelections.push(previousValue);
                    return currentValue;
                }
                else {
                    return new Selection(previousValue.startLineNumber, previousValue.startColumn, currentValue.endLineNumber, currentValue.endColumn);
                }
            }
            else {
                if (currentValue.startLineNumber > previousValue.endLineNumber) {
                    reducedSelections.push(previousValue);
                    return currentValue;
                }
                else {
                    return new Selection(previousValue.startLineNumber, previousValue.startColumn, currentValue.endLineNumber, currentValue.endColumn);
                }
            }
        });
        reducedSelections.push(lastSelection);
        var model = editor.getModel();
        var edits = [];
        var endCursorState = [];
        var endPrimaryCursor = primaryCursor;
        var lineOffset = 0;
        for (var i = 0, len = reducedSelections.length; i < len; i++) {
            var selection = reducedSelections[i];
            var startLineNumber = selection.startLineNumber;
            var startColumn = 1;
            var columnDeltaOffset = 0;
            var endLineNumber = void 0, endColumn = void 0;
            var selectionEndPositionOffset = model.getLineContent(selection.endLineNumber).length - selection.endColumn;
            if (selection.isEmpty() || selection.startLineNumber === selection.endLineNumber) {
                var position = selection.getStartPosition();
                if (position.lineNumber < model.getLineCount()) {
                    endLineNumber = startLineNumber + 1;
                    endColumn = model.getLineMaxColumn(endLineNumber);
                }
                else {
                    endLineNumber = position.lineNumber;
                    endColumn = model.getLineMaxColumn(position.lineNumber);
                }
            }
            else {
                endLineNumber = selection.endLineNumber;
                endColumn = model.getLineMaxColumn(endLineNumber);
            }
            var trimmedLinesContent = model.getLineContent(startLineNumber);
            for (var i_1 = startLineNumber + 1; i_1 <= endLineNumber; i_1++) {
                var lineText = model.getLineContent(i_1);
                var firstNonWhitespaceIdx = model.getLineFirstNonWhitespaceColumn(i_1);
                if (firstNonWhitespaceIdx >= 1) {
                    var insertSpace = true;
                    if (trimmedLinesContent === '') {
                        insertSpace = false;
                    }
                    if (insertSpace && (trimmedLinesContent.charAt(trimmedLinesContent.length - 1) === ' ' ||
                        trimmedLinesContent.charAt(trimmedLinesContent.length - 1) === '\t')) {
                        insertSpace = false;
                        trimmedLinesContent = trimmedLinesContent.replace(/[\s\uFEFF\xA0]+$/g, ' ');
                    }
                    var lineTextWithoutIndent = lineText.substr(firstNonWhitespaceIdx - 1);
                    trimmedLinesContent += (insertSpace ? ' ' : '') + lineTextWithoutIndent;
                    if (insertSpace) {
                        columnDeltaOffset = lineTextWithoutIndent.length + 1;
                    }
                    else {
                        columnDeltaOffset = lineTextWithoutIndent.length;
                    }
                }
                else {
                    columnDeltaOffset = 0;
                }
            }
            var deleteSelection = new Range(startLineNumber, startColumn, endLineNumber, endColumn);
            if (!deleteSelection.isEmpty()) {
                var resultSelection = void 0;
                if (selection.isEmpty()) {
                    edits.push(EditOperation.replace(deleteSelection, trimmedLinesContent));
                    resultSelection = new Selection(deleteSelection.startLineNumber - lineOffset, trimmedLinesContent.length - columnDeltaOffset + 1, startLineNumber - lineOffset, trimmedLinesContent.length - columnDeltaOffset + 1);
                }
                else {
                    if (selection.startLineNumber === selection.endLineNumber) {
                        edits.push(EditOperation.replace(deleteSelection, trimmedLinesContent));
                        resultSelection = new Selection(selection.startLineNumber - lineOffset, selection.startColumn, selection.endLineNumber - lineOffset, selection.endColumn);
                    }
                    else {
                        edits.push(EditOperation.replace(deleteSelection, trimmedLinesContent));
                        resultSelection = new Selection(selection.startLineNumber - lineOffset, selection.startColumn, selection.startLineNumber - lineOffset, trimmedLinesContent.length - selectionEndPositionOffset);
                    }
                }
                if (Range.intersectRanges(deleteSelection, primaryCursor) !== null) {
                    endPrimaryCursor = resultSelection;
                }
                else {
                    endCursorState.push(resultSelection);
                }
            }
            lineOffset += deleteSelection.endLineNumber - deleteSelection.startLineNumber;
        }
        endCursorState.unshift(endPrimaryCursor);
        editor.pushUndoStop();
        editor.executeEdits(this.id, edits, endCursorState);
        editor.pushUndoStop();
    };
    return JoinLinesAction;
}(EditorAction));
var TransposeAction = /** @class */ (function (_super) {
    __extends$1A(TransposeAction, _super);
    function TransposeAction() {
        return _super.call(this, {
            id: 'editor.action.transpose',
            label: localize('editor.transpose', "Transpose characters around the cursor"),
            alias: 'Transpose characters around the cursor',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    TransposeAction.prototype.run = function (_accessor, editor) {
        var selections = editor.getSelections();
        var model = editor.getModel();
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (!selection.isEmpty()) {
                continue;
            }
            var cursor = selection.getStartPosition();
            var maxColumn = model.getLineMaxColumn(cursor.lineNumber);
            if (cursor.column >= maxColumn) {
                if (cursor.lineNumber === model.getLineCount()) {
                    continue;
                }
                // The cursor is at the end of current line and current line is not empty
                // then we transpose the character before the cursor and the line break if there is any following line.
                var deleteSelection = new Range(cursor.lineNumber, Math.max(1, cursor.column - 1), cursor.lineNumber + 1, 1);
                var chars = model.getValueInRange(deleteSelection).split('').reverse().join('');
                commands.push(new ReplaceCommand(new Selection(cursor.lineNumber, Math.max(1, cursor.column - 1), cursor.lineNumber + 1, 1), chars));
            }
            else {
                var deleteSelection = new Range(cursor.lineNumber, Math.max(1, cursor.column - 1), cursor.lineNumber, cursor.column + 1);
                var chars = model.getValueInRange(deleteSelection).split('').reverse().join('');
                commands.push(new ReplaceCommandThatPreservesSelection(deleteSelection, chars, new Selection(cursor.lineNumber, cursor.column + 1, cursor.lineNumber, cursor.column + 1)));
            }
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return TransposeAction;
}(EditorAction));
var AbstractCaseAction = /** @class */ (function (_super) {
    __extends$1A(AbstractCaseAction, _super);
    function AbstractCaseAction() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AbstractCaseAction.prototype.run = function (_accessor, editor) {
        var selections = editor.getSelections();
        var model = editor.getModel();
        var commands = [];
        for (var i = 0, len = selections.length; i < len; i++) {
            var selection = selections[i];
            if (selection.isEmpty()) {
                var cursor = selection.getStartPosition();
                var word = model.getWordAtPosition(cursor);
                if (!word) {
                    continue;
                }
                var wordRange = new Range(cursor.lineNumber, word.startColumn, cursor.lineNumber, word.endColumn);
                var text = model.getValueInRange(wordRange);
                commands.push(new ReplaceCommandThatPreservesSelection(wordRange, this._modifyText(text), new Selection(cursor.lineNumber, cursor.column, cursor.lineNumber, cursor.column)));
            }
            else {
                var text = model.getValueInRange(selection);
                commands.push(new ReplaceCommandThatPreservesSelection(selection, this._modifyText(text), selection));
            }
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return AbstractCaseAction;
}(EditorAction));
var UpperCaseAction = /** @class */ (function (_super) {
    __extends$1A(UpperCaseAction, _super);
    function UpperCaseAction() {
        return _super.call(this, {
            id: 'editor.action.transformToUppercase',
            label: localize('editor.transformToUppercase', "Transform to Uppercase"),
            alias: 'Transform to Uppercase',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    UpperCaseAction.prototype._modifyText = function (text) {
        return text.toLocaleUpperCase();
    };
    return UpperCaseAction;
}(AbstractCaseAction));
var LowerCaseAction = /** @class */ (function (_super) {
    __extends$1A(LowerCaseAction, _super);
    function LowerCaseAction() {
        return _super.call(this, {
            id: 'editor.action.transformToLowercase',
            label: localize('editor.transformToLowercase', "Transform to Lowercase"),
            alias: 'Transform to Lowercase',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    LowerCaseAction.prototype._modifyText = function (text) {
        return text.toLocaleLowerCase();
    };
    return LowerCaseAction;
}(AbstractCaseAction));
registerEditorAction(CopyLinesUpAction);
registerEditorAction(CopyLinesDownAction);
registerEditorAction(MoveLinesUpAction);
registerEditorAction(MoveLinesDownAction);
registerEditorAction(SortLinesAscendingAction);
registerEditorAction(SortLinesDescendingAction);
registerEditorAction(TrimTrailingWhitespaceAction);
registerEditorAction(DeleteLinesAction);
registerEditorAction(IndentLinesAction);
registerEditorAction(OutdentLinesAction);
registerEditorAction(InsertLineBeforeAction);
registerEditorAction(InsertLineAfterAction);
registerEditorAction(DeleteAllLeftAction);
registerEditorAction(DeleteAllRightAction);
registerEditorAction(JoinLinesAction);
registerEditorAction(TransposeAction);
registerEditorAction(UpperCaseAction);
registerEditorAction(LowerCaseAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1B = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Node$2 = /** @class */ (function () {
    function Node() {
    }
    Object.defineProperty(Node.prototype, "range", {
        get: function () {
            return new Range(this.start.lineNumber, this.start.column, this.end.lineNumber, this.end.column);
        },
        enumerable: true,
        configurable: true
    });
    return Node;
}());
var NodeList = /** @class */ (function (_super) {
    __extends$1B(NodeList, _super);
    function NodeList() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(NodeList.prototype, "hasChildren", {
        get: function () {
            return this.children && this.children.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NodeList.prototype, "isEmpty", {
        get: function () {
            return !this.hasChildren && !this.parent;
        },
        enumerable: true,
        configurable: true
    });
    NodeList.prototype.append = function (node) {
        if (!node) {
            return false;
        }
        node.parent = this;
        if (!this.children) {
            this.children = [];
        }
        if (node instanceof NodeList) {
            if (node.children) {
                this.children.push.apply(this.children, node.children);
            }
        }
        else {
            this.children.push(node);
        }
        return true;
    };
    return NodeList;
}(Node$2));
var Block = /** @class */ (function (_super) {
    __extends$1B(Block, _super);
    function Block() {
        var _this = _super.call(this) || this;
        _this.elements = new NodeList();
        _this.elements.parent = _this;
        return _this;
    }
    return Block;
}(Node$2));
var Token$1 = /** @class */ (function () {
    function Token(range, bracket, bracketType) {
        this.range = range;
        this.bracket = bracket;
        this.bracketType = bracketType;
    }
    return Token;
}());
function newNode(token) {
    var node = new Node$2();
    node.start = token.range.getStartPosition();
    node.end = token.range.getEndPosition();
    return node;
}
var RawToken = /** @class */ (function () {
    function RawToken(source, tokenIndex, lineNumber) {
        this.lineNumber = lineNumber;
        this.lineText = source.getLineContent();
        this.startOffset = source.getStartOffset(tokenIndex);
        this.endOffset = source.getEndOffset(tokenIndex);
        this.type = source.getStandardTokenType(tokenIndex);
        this.languageId = source.getLanguageId(tokenIndex);
    }
    return RawToken;
}());
var ModelRawTokenScanner = /** @class */ (function () {
    function ModelRawTokenScanner(model) {
        this._model = model;
        this._lineCount = this._model.getLineCount();
        this._versionId = this._model.getVersionId();
        this._lineNumber = 0;
        this._tokenIndex = 0;
        this._lineTokens = null;
        this._advance();
    }
    ModelRawTokenScanner.prototype._advance = function () {
        if (this._lineTokens) {
            this._tokenIndex++;
            if (this._tokenIndex >= this._lineTokens.getCount()) {
                this._lineTokens = null;
            }
        }
        while (this._lineNumber < this._lineCount && !this._lineTokens) {
            this._lineNumber++;
            this._model.forceTokenization(this._lineNumber);
            this._lineTokens = this._model.getLineTokens(this._lineNumber);
            this._tokenIndex = 0;
            if (this._lineTokens.getLineContent().length === 0) {
                // Skip empty lines
                this._lineTokens = null;
            }
        }
    };
    ModelRawTokenScanner.prototype.next = function () {
        if (!this._lineTokens) {
            return null;
        }
        if (this._model.getVersionId() !== this._versionId) {
            return null;
        }
        var result = new RawToken(this._lineTokens, this._tokenIndex, this._lineNumber);
        this._advance();
        return result;
    };
    return ModelRawTokenScanner;
}());
var TokenScanner = /** @class */ (function () {
    function TokenScanner(model) {
        this._rawTokenScanner = new ModelRawTokenScanner(model);
        this._nextBuff = [];
        this._cachedLanguageBrackets = null;
        this._cachedLanguageId = -1;
    }
    TokenScanner.prototype.next = function () {
        if (this._nextBuff.length > 0) {
            return this._nextBuff.shift();
        }
        var token = this._rawTokenScanner.next();
        if (!token) {
            return null;
        }
        var lineNumber = token.lineNumber;
        var lineText = token.lineText;
        var tokenType = token.type;
        var startOffset = token.startOffset;
        var endOffset = token.endOffset;
        if (this._cachedLanguageId !== token.languageId) {
            this._cachedLanguageId = token.languageId;
            this._cachedLanguageBrackets = LanguageConfigurationRegistry.getBracketsSupport(this._cachedLanguageId);
        }
        var modeBrackets = this._cachedLanguageBrackets;
        if (!modeBrackets || ignoreBracketsInToken(tokenType)) {
            return new Token$1(new Range(lineNumber, startOffset + 1, lineNumber, endOffset + 1), 0 /* None */, null);
        }
        var foundBracket;
        do {
            foundBracket = BracketsUtils.findNextBracketInToken(modeBrackets.forwardRegex, lineNumber, lineText, startOffset, endOffset);
            if (foundBracket) {
                var foundBracketStartOffset = foundBracket.startColumn - 1;
                var foundBracketEndOffset = foundBracket.endColumn - 1;
                if (startOffset < foundBracketStartOffset) {
                    // there is some text before this bracket in this token
                    this._nextBuff.push(new Token$1(new Range(lineNumber, startOffset + 1, lineNumber, foundBracketStartOffset + 1), 0 /* None */, null));
                }
                var bracketText = lineText.substring(foundBracketStartOffset, foundBracketEndOffset);
                bracketText = bracketText.toLowerCase();
                var bracketData = modeBrackets.textIsBracket[bracketText];
                var bracketIsOpen = modeBrackets.textIsOpenBracket[bracketText];
                this._nextBuff.push(new Token$1(new Range(lineNumber, foundBracketStartOffset + 1, lineNumber, foundBracketEndOffset + 1), bracketIsOpen ? 1 /* Open */ : -1 /* Close */, bracketData.languageIdentifier.language + ";" + bracketData.open + ";" + bracketData.close));
                startOffset = foundBracketEndOffset;
            }
        } while (foundBracket);
        if (startOffset < endOffset) {
            // there is some remaining none-bracket text in this token
            this._nextBuff.push(new Token$1(new Range(lineNumber, startOffset + 1, lineNumber, endOffset + 1), 0 /* None */, null));
        }
        return this._nextBuff.shift() || null;
    };
    return TokenScanner;
}());
var TokenTreeBuilder = /** @class */ (function () {
    function TokenTreeBuilder(model) {
        this._stack = [];
        this._scanner = new TokenScanner(model);
    }
    TokenTreeBuilder.prototype.build = function () {
        var node = new NodeList();
        while (node.append(this._line() || this._any())) {
            // accept all
        }
        return node;
    };
    TokenTreeBuilder.prototype._accept = function (condt) {
        var token = this._stack.pop() || this._scanner.next();
        if (!token) {
            return false;
        }
        var accepted = condt(token);
        if (!accepted) {
            this._stack.push(token);
            // this._currentToken = null;
        }
        else {
            this._currentToken = token;
            //			console.log('accepted: ' + token.__debugContent);
        }
        return accepted;
    };
    TokenTreeBuilder.prototype._peek = function (condt) {
        var ret = false;
        this._accept(function (info) {
            ret = condt(info);
            return false;
        });
        return ret;
    };
    TokenTreeBuilder.prototype._line = function () {
        var node = new NodeList();
        var lineNumber;
        // capture current linenumber
        this._peek(function (info) {
            lineNumber = info.range.startLineNumber;
            return false;
        });
        while (this._peek(function (info) { return info.range.startLineNumber === lineNumber; })
            && node.append(this._token() || this._block())) {
            // all children that started on this line
        }
        if (!node.children || node.children.length === 0) {
            return null;
        }
        else if (node.children.length === 1) {
            return node.children[0];
        }
        else {
            return node;
        }
    };
    TokenTreeBuilder.prototype._token = function () {
        if (!this._accept(function (token) { return token.bracket === 0 /* None */; })) {
            return null;
        }
        return newNode(this._currentToken);
    };
    TokenTreeBuilder.prototype._block = function () {
        var bracketType;
        var accepted;
        accepted = this._accept(function (token) {
            bracketType = token.bracketType;
            return token.bracket === 1 /* Open */;
        });
        if (!accepted) {
            return null;
        }
        var bracket = new Block();
        bracket.open = newNode(this._currentToken);
        while (bracket.elements.append(this._line())) {
            // inside brackets
        }
        if (!this._accept(function (token) { return token.bracket === -1 /* Close */ && token.bracketType === bracketType; })) {
            // missing closing bracket -> return just a node list
            var nodelist = new NodeList();
            nodelist.append(bracket.open);
            nodelist.append(bracket.elements);
            return nodelist;
        }
        bracket.close = newNode(this._currentToken);
        return bracket;
    };
    TokenTreeBuilder.prototype._any = function () {
        if (!this._accept(function (_) { return true; })) {
            return null;
        }
        return newNode(this._currentToken);
    };
    return TokenTreeBuilder;
}());
/**
 * Parses this grammar:
 *	grammer = { line }
 *	line = { block | "token" }
 *	block = "open_bracket" { line } "close_bracket"
 */
function build(model) {
    var node = new TokenTreeBuilder(model).build();
    return node;
}
function find(node, position) {
    if (node instanceof NodeList && node.isEmpty) {
        return null;
    }
    if (!Range.containsPosition(node.range, position)) {
        return null;
    }
    var result = null;
    if (node instanceof NodeList) {
        if (node.hasChildren) {
            for (var i = 0, len = node.children.length; i < len && !result; i++) {
                result = find(node.children[i], position);
            }
        }
    }
    else if (node instanceof Block) {
        result = find(node.elements, position) || find(node.open, position) || find(node.close, position);
    }
    return result || node;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$d = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$c = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var TokenSelectionSupport = /** @class */ (function () {
    function TokenSelectionSupport(modelService) {
        this._modelService = modelService;
    }
    TokenSelectionSupport.prototype.getRangesToPositionSync = function (resource, position) {
        var model = this._modelService.getModel(resource);
        var entries = [];
        if (model) {
            this._doGetRangesToPosition(model, position).forEach(function (range) {
                entries.push({
                    type: void 0,
                    range: range
                });
            });
        }
        return entries;
    };
    TokenSelectionSupport.prototype._doGetRangesToPosition = function (model, position) {
        var tree = build(model);
        var node;
        var lastRange;
        node = find(tree, position);
        var ranges = [];
        while (node) {
            if (!lastRange || !Range.equalsRange(lastRange, node.range)) {
                ranges.push(node.range);
            }
            lastRange = node.range;
            node = node.parent;
        }
        ranges = ranges.reverse();
        return ranges;
    };
    TokenSelectionSupport = __decorate$d([
        __param$c(0, IModelService)
    ], TokenSelectionSupport);
    return TokenSelectionSupport;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1C = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$e = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$d = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
// --- selection state machine
var State = /** @class */ (function () {
    function State(editor) {
        this.editor = editor;
        this.next = null;
        this.previous = null;
        this.selection = editor.getSelection();
    }
    return State;
}());
// -- action implementation
var SmartSelectController = /** @class */ (function () {
    function SmartSelectController(editor, instantiationService) {
        this.editor = editor;
        this._tokenSelectionSupport = instantiationService.createInstance(TokenSelectionSupport);
        this._state = null;
        this._ignoreSelection = false;
    }
    SmartSelectController.get = function (editor) {
        return editor.getContribution(SmartSelectController.ID);
    };
    SmartSelectController.prototype.dispose = function () {
    };
    SmartSelectController.prototype.getId = function () {
        return SmartSelectController.ID;
    };
    SmartSelectController.prototype.run = function (forward) {
        var _this = this;
        var selection = this.editor.getSelection();
        var model = this.editor.getModel();
        // forget about current state
        if (this._state) {
            if (this._state.editor !== this.editor) {
                this._state = null;
            }
        }
        var promise = Promise.resolve(null);
        if (!this._state) {
            promise = Promise.resolve(this._tokenSelectionSupport.getRangesToPositionSync(model.uri, selection.getStartPosition())).then(function (elements) {
                if (isFalsyOrEmpty(elements)) {
                    return;
                }
                var lastState;
                elements.filter(function (element) {
                    // filter ranges inside the selection
                    var selection = _this.editor.getSelection();
                    var range$$1 = new Range(element.range.startLineNumber, element.range.startColumn, element.range.endLineNumber, element.range.endColumn);
                    return range$$1.containsPosition(selection.getStartPosition()) && range$$1.containsPosition(selection.getEndPosition());
                }).forEach(function (element) {
                    // create ranges
                    var range$$1 = element.range;
                    var state = new State(_this.editor);
                    state.selection = new Range(range$$1.startLineNumber, range$$1.startColumn, range$$1.endLineNumber, range$$1.endColumn);
                    if (lastState) {
                        state.next = lastState;
                        lastState.previous = state;
                    }
                    lastState = state;
                });
                // insert current selection
                var editorState = new State(_this.editor);
                editorState.next = lastState;
                if (lastState) {
                    lastState.previous = editorState;
                }
                _this._state = editorState;
                // listen to caret move and forget about state
                var unhook = _this.editor.onDidChangeCursorPosition(function (e) {
                    if (_this._ignoreSelection) {
                        return;
                    }
                    _this._state = null;
                    unhook.dispose();
                });
            });
        }
        return promise.then(function () {
            if (!_this._state) {
                return;
            }
            _this._state = forward ? _this._state.next : _this._state.previous;
            if (!_this._state) {
                return;
            }
            _this._ignoreSelection = true;
            try {
                _this.editor.setSelection(_this._state.selection);
            }
            finally {
                _this._ignoreSelection = false;
            }
            return;
        });
    };
    SmartSelectController.ID = 'editor.contrib.smartSelectController';
    SmartSelectController = __decorate$e([
        __param$d(1, IInstantiationService)
    ], SmartSelectController);
    return SmartSelectController;
}());
var AbstractSmartSelect = /** @class */ (function (_super) {
    __extends$1C(AbstractSmartSelect, _super);
    function AbstractSmartSelect(forward, opts) {
        var _this = _super.call(this, opts) || this;
        _this._forward = forward;
        return _this;
    }
    AbstractSmartSelect.prototype.run = function (accessor, editor) {
        var controller = SmartSelectController.get(editor);
        if (controller) {
            return controller.run(this._forward);
        }
        return undefined;
    };
    return AbstractSmartSelect;
}(EditorAction));
var GrowSelectionAction = /** @class */ (function (_super) {
    __extends$1C(GrowSelectionAction, _super);
    function GrowSelectionAction() {
        return _super.call(this, true, {
            id: 'editor.action.smartSelect.grow',
            label: localize('smartSelect.grow', "Expand Select"),
            alias: 'Expand Select',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 1024 /* Shift */ | 512 /* Alt */ | 17 /* RightArrow */,
                mac: { primary: 2048 /* CtrlCmd */ | 256 /* WinCtrl */ | 1024 /* Shift */ | 17 /* RightArrow */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarSelectionMenu,
                group: '1_basic',
                title: localize({ key: 'miSmartSelectGrow', comment: ['&& denotes a mnemonic'] }, "&&Expand Selection"),
                order: 2
            }
        }) || this;
    }
    return GrowSelectionAction;
}(AbstractSmartSelect));
var ShrinkSelectionAction = /** @class */ (function (_super) {
    __extends$1C(ShrinkSelectionAction, _super);
    function ShrinkSelectionAction() {
        return _super.call(this, false, {
            id: 'editor.action.smartSelect.shrink',
            label: localize('smartSelect.shrink', "Shrink Select"),
            alias: 'Shrink Select',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 1024 /* Shift */ | 512 /* Alt */ | 15 /* LeftArrow */,
                mac: { primary: 2048 /* CtrlCmd */ | 256 /* WinCtrl */ | 1024 /* Shift */ | 15 /* LeftArrow */ },
                weight: 100 /* EditorContrib */
            },
            menubarOpts: {
                menuId: MenuId.MenubarSelectionMenu,
                group: '1_basic',
                title: localize({ key: 'miSmartSelectShrink', comment: ['&& denotes a mnemonic'] }, "&&Shrink Selection"),
                order: 3
            }
        }) || this;
    }
    return ShrinkSelectionAction;
}(AbstractSmartSelect));
registerEditorContribution(SmartSelectController);
registerEditorAction(GrowSelectionAction);
registerEditorAction(ShrinkSelectionAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Context = {
    Visible: new RawContextKey('suggestWidgetVisible', false),
    MultipleSuggestions: new RawContextKey('suggestWidgetMultipleSuggestions', false),
    MakesTextEdit: new RawContextKey('suggestionMakesTextEdit', true),
    AcceptSuggestionsOnEnter: new RawContextKey('acceptSuggestionOnEnter', true)
};
var _snippetSuggestSupport;
function getSnippetSuggestSupport() {
    return _snippetSuggestSupport;
}
function provideSuggestionItems(model, position, snippetConfig, onlyFrom, context, token) {
    if (snippetConfig === void 0) { snippetConfig = 'bottom'; }
    if (token === void 0) { token = CancellationToken.None; }
    var allSuggestions = [];
    var acceptSuggestion = createSuggesionFilter(snippetConfig);
    var wordUntil = model.getWordUntilPosition(position);
    var defaultRange = new Range(position.lineNumber, wordUntil.startColumn, position.lineNumber, wordUntil.endColumn);
    position = position.clone();
    // get provider groups, always add snippet suggestion provider
    var supports = CompletionProviderRegistry.orderedGroups(model);
    // add snippets provider unless turned off
    if (snippetConfig !== 'none' && _snippetSuggestSupport) {
        supports.unshift([_snippetSuggestSupport]);
    }
    var suggestConext = context || { triggerKind: 0 /* Invoke */ };
    // add suggestions from contributed providers - providers are ordered in groups of
    // equal score and once a group produces a result the process stops
    var hasResult = false;
    var factory = supports.map(function (supports) { return function () {
        // for each support in the group ask for suggestions
        return Promise.all(supports.map(function (support) {
            if (!isFalsyOrEmpty(onlyFrom) && onlyFrom.indexOf(support) < 0) {
                return undefined;
            }
            return Promise.resolve(support.provideCompletionItems(model, position, suggestConext, token)).then(function (container) {
                var len = allSuggestions.length;
                if (container && !isFalsyOrEmpty(container.suggestions)) {
                    for (var _i = 0, _a = container.suggestions; _i < _a.length; _i++) {
                        var suggestion = _a[_i];
                        if (acceptSuggestion(suggestion)) {
                            // fill in default range when missing
                            if (!suggestion.range) {
                                suggestion.range = defaultRange;
                            }
                            // fill in lower-case text
                            ensureLowerCaseVariants(suggestion);
                            allSuggestions.push({
                                position: position,
                                container: container,
                                suggestion: suggestion,
                                support: support,
                                resolve: createSuggestionResolver(support, suggestion, model, position)
                            });
                        }
                    }
                }
                if (len !== allSuggestions.length && support !== _snippetSuggestSupport) {
                    hasResult = true;
                }
            }, onUnexpectedExternalError);
        }));
    }; });
    var result = first(factory, function () {
        // stop on result or cancellation
        return hasResult || token.isCancellationRequested;
    }).then(function () {
        if (token.isCancellationRequested) {
            return Promise.reject(canceled());
        }
        return allSuggestions.sort(getSuggestionComparator(snippetConfig));
    });
    // result.then(items => {
    // 	console.log(model.getWordUntilPosition(position), items.map(item => `${item.suggestion.label}, type=${item.suggestion.type}, incomplete?${item.container.incomplete}, overwriteBefore=${item.suggestion.overwriteBefore}`));
    // 	return items;
    // }, err => {
    // 	console.warn(model.getWordUntilPosition(position), err);
    // });
    return result;
}
function ensureLowerCaseVariants(suggestion) {
    if (!suggestion._labelLow) {
        suggestion._labelLow = suggestion.label.toLowerCase();
    }
    if (suggestion.sortText && !suggestion._sortTextLow) {
        suggestion._sortTextLow = suggestion.sortText.toLowerCase();
    }
    if (suggestion.filterText && !suggestion._filterTextLow) {
        suggestion._filterTextLow = suggestion.filterText.toLowerCase();
    }
}
function createSuggestionResolver(provider, suggestion, model, position) {
    var cached;
    return function (token) {
        if (!cached) {
            if (typeof provider.resolveCompletionItem === 'function') {
                cached = Promise.resolve(provider.resolveCompletionItem(model, position, suggestion, token)).then(function (value) { assign(suggestion, value); });
            }
            else {
                cached = Promise.resolve(void 0);
            }
        }
        return cached;
    };
}
function createSuggesionFilter(snippetConfig) {
    if (snippetConfig === 'none') {
        return function (suggestion) { return suggestion.kind !== 25 /* Snippet */; };
    }
    else {
        return function () { return true; };
    }
}
function defaultComparator(a, b) {
    // check with 'sortText'
    if (a.suggestion._sortTextLow && b.suggestion._sortTextLow) {
        if (a.suggestion._sortTextLow < b.suggestion._sortTextLow) {
            return -1;
        }
        else if (a.suggestion._sortTextLow > b.suggestion._sortTextLow) {
            return 1;
        }
    }
    // check with 'label'
    if (a.suggestion.label < b.suggestion.label) {
        return -1;
    }
    else if (a.suggestion.label > b.suggestion.label) {
        return 1;
    }
    // check with 'type'
    return a.suggestion.kind - b.suggestion.kind;
}
function snippetUpComparator(a, b) {
    if (a.suggestion.kind !== b.suggestion.kind) {
        if (a.suggestion.kind === 25 /* Snippet */) {
            return -1;
        }
        else if (b.suggestion.kind === 25 /* Snippet */) {
            return 1;
        }
    }
    return defaultComparator(a, b);
}
function snippetDownComparator(a, b) {
    if (a.suggestion.kind !== b.suggestion.kind) {
        if (a.suggestion.kind === 25 /* Snippet */) {
            return 1;
        }
        else if (b.suggestion.kind === 25 /* Snippet */) {
            return -1;
        }
    }
    return defaultComparator(a, b);
}
function getSuggestionComparator(snippetConfig) {
    if (snippetConfig === 'top') {
        return snippetUpComparator;
    }
    else if (snippetConfig === 'bottom') {
        return snippetDownComparator;
    }
    else {
        return defaultComparator;
    }
}
registerDefaultLanguageCommand('_executeCompletionItemProvider', function (model, position, args) {
    var result = {
        incomplete: false,
        suggestions: []
    };
    var resolving = [];
    var maxItemsToResolve = args['maxItemsToResolve'] || 0;
    return provideSuggestionItems(model, position).then(function (items) {
        for (var _i = 0, items_1 = items; _i < items_1.length; _i++) {
            var item = items_1[_i];
            if (resolving.length < maxItemsToResolve) {
                resolving.push(item.resolve(CancellationToken.None));
            }
            result.incomplete = result.incomplete || item.container.incomplete;
            result.suggestions.push(item.suggestion);
        }
    }).then(function () {
        return Promise.all(resolving);
    }).then(function () {
        return result;
    });
});
var _provider = new /** @class */ (function () {
    function class_1() {
        this.onlyOnceSuggestions = [];
    }
    class_1.prototype.provideCompletionItems = function () {
        var suggestions = this.onlyOnceSuggestions.slice(0);
        var result = { suggestions: suggestions };
        this.onlyOnceSuggestions.length = 0;
        return result;
    };
    return class_1;
}());
CompletionProviderRegistry.register('*', _provider);
function showSimpleSuggestions(editor, suggestions) {
    setTimeout(function () {
        var _a;
        (_a = _provider.onlyOnceSuggestions).push.apply(_a, suggestions);
        editor.getContribution('editor.contrib.suggestController').triggerSuggest([_provider]);
    }, 0);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ILogService = createDecorator('logService');
var NullLogService = /** @class */ (function () {
    function NullLogService() {
    }
    NullLogService.prototype.trace = function (message) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
    };
    NullLogService.prototype.error = function (message) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
    };
    NullLogService.prototype.dispose = function () { };
    return NullLogService;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1D = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var _a$1;
var Scanner = /** @class */ (function () {
    function Scanner() {
        this.text('');
    }
    Scanner.isDigitCharacter = function (ch) {
        return ch >= 48 /* Digit0 */ && ch <= 57 /* Digit9 */;
    };
    Scanner.isVariableCharacter = function (ch) {
        return ch === 95 /* Underline */
            || (ch >= 97 /* a */ && ch <= 122 /* z */)
            || (ch >= 65 /* A */ && ch <= 90 /* Z */);
    };
    Scanner.prototype.text = function (value) {
        this.value = value;
        this.pos = 0;
    };
    Scanner.prototype.tokenText = function (token) {
        return this.value.substr(token.pos, token.len);
    };
    Scanner.prototype.next = function () {
        if (this.pos >= this.value.length) {
            return { type: 14 /* EOF */, pos: this.pos, len: 0 };
        }
        var pos = this.pos;
        var len = 0;
        var ch = this.value.charCodeAt(pos);
        var type;
        // static types
        type = Scanner._table[ch];
        if (typeof type === 'number') {
            this.pos += 1;
            return { type: type, pos: pos, len: 1 };
        }
        // number
        if (Scanner.isDigitCharacter(ch)) {
            type = 8 /* Int */;
            do {
                len += 1;
                ch = this.value.charCodeAt(pos + len);
            } while (Scanner.isDigitCharacter(ch));
            this.pos += len;
            return { type: type, pos: pos, len: len };
        }
        // variable name
        if (Scanner.isVariableCharacter(ch)) {
            type = 9 /* VariableName */;
            do {
                ch = this.value.charCodeAt(pos + (++len));
            } while (Scanner.isVariableCharacter(ch) || Scanner.isDigitCharacter(ch));
            this.pos += len;
            return { type: type, pos: pos, len: len };
        }
        // format
        type = 10 /* Format */;
        do {
            len += 1;
            ch = this.value.charCodeAt(pos + len);
        } while (!isNaN(ch)
            && typeof Scanner._table[ch] === 'undefined' // not static token
            && !Scanner.isDigitCharacter(ch) // not number
            && !Scanner.isVariableCharacter(ch) // not variable
        );
        this.pos += len;
        return { type: type, pos: pos, len: len };
    };
    Scanner._table = (_a$1 = {},
        _a$1[36 /* DollarSign */] = 0 /* Dollar */,
        _a$1[58 /* Colon */] = 1 /* Colon */,
        _a$1[44 /* Comma */] = 2 /* Comma */,
        _a$1[123 /* OpenCurlyBrace */] = 3 /* CurlyOpen */,
        _a$1[125 /* CloseCurlyBrace */] = 4 /* CurlyClose */,
        _a$1[92 /* Backslash */] = 5 /* Backslash */,
        _a$1[47 /* Slash */] = 6 /* Forwardslash */,
        _a$1[124 /* Pipe */] = 7 /* Pipe */,
        _a$1[43 /* Plus */] = 11 /* Plus */,
        _a$1[45 /* Dash */] = 12 /* Dash */,
        _a$1[63 /* QuestionMark */] = 13 /* QuestionMark */,
        _a$1);
    return Scanner;
}());
var Marker = /** @class */ (function () {
    function Marker() {
        this._children = [];
    }
    Marker.prototype.appendChild = function (child) {
        if (child instanceof Text && this._children[this._children.length - 1] instanceof Text) {
            // this and previous child are text -> merge them
            this._children[this._children.length - 1].value += child.value;
        }
        else {
            // normal adoption of child
            child.parent = this;
            this._children.push(child);
        }
        return this;
    };
    Marker.prototype.replace = function (child, others) {
        var parent = child.parent;
        var idx = parent.children.indexOf(child);
        var newChildren = parent.children.slice(0);
        newChildren.splice.apply(newChildren, [idx, 1].concat(others));
        parent._children = newChildren;
        (function _fixParent(children, parent) {
            for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
                var child_1 = children_1[_i];
                child_1.parent = parent;
                _fixParent(child_1.children, child_1);
            }
        })(others, parent);
    };
    Object.defineProperty(Marker.prototype, "children", {
        get: function () {
            return this._children;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Marker.prototype, "snippet", {
        get: function () {
            var candidate = this;
            while (true) {
                if (!candidate) {
                    return undefined;
                }
                if (candidate instanceof TextmateSnippet) {
                    return candidate;
                }
                candidate = candidate.parent;
            }
        },
        enumerable: true,
        configurable: true
    });
    Marker.prototype.toString = function () {
        return this.children.reduce(function (prev, cur) { return prev + cur.toString(); }, '');
    };
    Marker.prototype.len = function () {
        return 0;
    };
    return Marker;
}());
var Text = /** @class */ (function (_super) {
    __extends$1D(Text, _super);
    function Text(value) {
        var _this_1 = _super.call(this) || this;
        _this_1.value = value;
        return _this_1;
    }
    Text.prototype.toString = function () {
        return this.value;
    };
    Text.prototype.len = function () {
        return this.value.length;
    };
    Text.prototype.clone = function () {
        return new Text(this.value);
    };
    return Text;
}(Marker));
var TransformableMarker = /** @class */ (function (_super) {
    __extends$1D(TransformableMarker, _super);
    function TransformableMarker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return TransformableMarker;
}(Marker));
var Placeholder = /** @class */ (function (_super) {
    __extends$1D(Placeholder, _super);
    function Placeholder(index) {
        var _this_1 = _super.call(this) || this;
        _this_1.index = index;
        return _this_1;
    }
    Placeholder.compareByIndex = function (a, b) {
        if (a.index === b.index) {
            return 0;
        }
        else if (a.isFinalTabstop) {
            return 1;
        }
        else if (b.isFinalTabstop) {
            return -1;
        }
        else if (a.index < b.index) {
            return -1;
        }
        else if (a.index > b.index) {
            return 1;
        }
        else {
            return 0;
        }
    };
    Object.defineProperty(Placeholder.prototype, "isFinalTabstop", {
        get: function () {
            return this.index === 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Placeholder.prototype, "choice", {
        get: function () {
            return this._children.length === 1 && this._children[0] instanceof Choice
                ? this._children[0]
                : undefined;
        },
        enumerable: true,
        configurable: true
    });
    Placeholder.prototype.clone = function () {
        var ret = new Placeholder(this.index);
        if (this.transform) {
            ret.transform = this.transform.clone();
        }
        ret._children = this.children.map(function (child) { return child.clone(); });
        return ret;
    };
    return Placeholder;
}(TransformableMarker));
var Choice = /** @class */ (function (_super) {
    __extends$1D(Choice, _super);
    function Choice() {
        var _this_1 = _super !== null && _super.apply(this, arguments) || this;
        _this_1.options = [];
        return _this_1;
    }
    Choice.prototype.appendChild = function (marker) {
        if (marker instanceof Text) {
            marker.parent = this;
            this.options.push(marker);
        }
        return this;
    };
    Choice.prototype.toString = function () {
        return this.options[0].value;
    };
    Choice.prototype.len = function () {
        return this.options[0].len();
    };
    Choice.prototype.clone = function () {
        var ret = new Choice();
        this.options.forEach(ret.appendChild, ret);
        return ret;
    };
    return Choice;
}(Marker));
var Transform = /** @class */ (function (_super) {
    __extends$1D(Transform, _super);
    function Transform() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Transform.prototype.resolve = function (value) {
        var _this = this;
        var didMatch = false;
        var ret = value.replace(this.regexp, function () {
            didMatch = true;
            return _this._replace(Array.prototype.slice.call(arguments, 0, -2));
        });
        // when the regex didn't match and when the transform has
        // else branches, then run those
        if (!didMatch && this._children.some(function (child) { return child instanceof FormatString && Boolean(child.elseValue); })) {
            ret = this._replace([]);
        }
        return ret;
    };
    Transform.prototype._replace = function (groups) {
        var ret = '';
        for (var _i = 0, _a = this._children; _i < _a.length; _i++) {
            var marker = _a[_i];
            if (marker instanceof FormatString) {
                var value = groups[marker.index] || '';
                value = marker.resolve(value);
                ret += value;
            }
            else {
                ret += marker.toString();
            }
        }
        return ret;
    };
    Transform.prototype.toString = function () {
        return '';
    };
    Transform.prototype.clone = function () {
        var ret = new Transform();
        ret.regexp = new RegExp(this.regexp.source, '' + (this.regexp.ignoreCase ? 'i' : '') + (this.regexp.global ? 'g' : ''));
        ret._children = this.children.map(function (child) { return child.clone(); });
        return ret;
    };
    return Transform;
}(Marker));
var FormatString = /** @class */ (function (_super) {
    __extends$1D(FormatString, _super);
    function FormatString(index, shorthandName, ifValue, elseValue) {
        var _this_1 = _super.call(this) || this;
        _this_1.index = index;
        _this_1.shorthandName = shorthandName;
        _this_1.ifValue = ifValue;
        _this_1.elseValue = elseValue;
        return _this_1;
    }
    FormatString.prototype.resolve = function (value) {
        if (this.shorthandName === 'upcase') {
            return !value ? '' : value.toLocaleUpperCase();
        }
        else if (this.shorthandName === 'downcase') {
            return !value ? '' : value.toLocaleLowerCase();
        }
        else if (this.shorthandName === 'capitalize') {
            return !value ? '' : (value[0].toLocaleUpperCase() + value.substr(1));
        }
        else if (this.shorthandName === 'pascalcase') {
            return !value ? '' : this._toPascalCase(value);
        }
        else if (Boolean(value) && typeof this.ifValue === 'string') {
            return this.ifValue;
        }
        else if (!Boolean(value) && typeof this.elseValue === 'string') {
            return this.elseValue;
        }
        else {
            return value || '';
        }
    };
    FormatString.prototype._toPascalCase = function (value) {
        var match = value.match(/[a-z]+/gi);
        if (!match) {
            return value;
        }
        return match.map(function (word) {
            return word.charAt(0).toUpperCase()
                + word.substr(1).toLowerCase();
        })
            .join('');
    };
    FormatString.prototype.clone = function () {
        var ret = new FormatString(this.index, this.shorthandName, this.ifValue, this.elseValue);
        return ret;
    };
    return FormatString;
}(Marker));
var Variable = /** @class */ (function (_super) {
    __extends$1D(Variable, _super);
    function Variable(name) {
        var _this_1 = _super.call(this) || this;
        _this_1.name = name;
        return _this_1;
    }
    Variable.prototype.resolve = function (resolver) {
        var value = resolver.resolve(this);
        if (this.transform) {
            value = this.transform.resolve(value || '');
        }
        if (value !== undefined) {
            this._children = [new Text(value)];
            return true;
        }
        return false;
    };
    Variable.prototype.clone = function () {
        var ret = new Variable(this.name);
        if (this.transform) {
            ret.transform = this.transform.clone();
        }
        ret._children = this.children.map(function (child) { return child.clone(); });
        return ret;
    };
    return Variable;
}(TransformableMarker));
function walk(marker, visitor) {
    var stack = marker.slice();
    while (stack.length > 0) {
        var marker_1 = stack.shift();
        var recurse = visitor(marker_1);
        if (!recurse) {
            break;
        }
        stack.unshift.apply(stack, marker_1.children);
    }
}
var TextmateSnippet = /** @class */ (function (_super) {
    __extends$1D(TextmateSnippet, _super);
    function TextmateSnippet() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(TextmateSnippet.prototype, "placeholderInfo", {
        get: function () {
            if (!this._placeholders) {
                // fill in placeholders
                var all_1 = [];
                var last_1;
                this.walk(function (candidate) {
                    if (candidate instanceof Placeholder) {
                        all_1.push(candidate);
                        last_1 = !last_1 || last_1.index < candidate.index ? candidate : last_1;
                    }
                    return true;
                });
                this._placeholders = { all: all_1, last: last_1 };
            }
            return this._placeholders;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextmateSnippet.prototype, "placeholders", {
        get: function () {
            var all = this.placeholderInfo.all;
            return all;
        },
        enumerable: true,
        configurable: true
    });
    TextmateSnippet.prototype.offset = function (marker) {
        var pos = 0;
        var found = false;
        this.walk(function (candidate) {
            if (candidate === marker) {
                found = true;
                return false;
            }
            pos += candidate.len();
            return true;
        });
        if (!found) {
            return -1;
        }
        return pos;
    };
    TextmateSnippet.prototype.fullLen = function (marker) {
        var ret = 0;
        walk([marker], function (marker) {
            ret += marker.len();
            return true;
        });
        return ret;
    };
    TextmateSnippet.prototype.enclosingPlaceholders = function (placeholder) {
        var ret = [];
        var parent = placeholder.parent;
        while (parent) {
            if (parent instanceof Placeholder) {
                ret.push(parent);
            }
            parent = parent.parent;
        }
        return ret;
    };
    TextmateSnippet.prototype.resolveVariables = function (resolver) {
        var _this_1 = this;
        this.walk(function (candidate) {
            if (candidate instanceof Variable) {
                if (candidate.resolve(resolver)) {
                    _this_1._placeholders = undefined;
                }
            }
            return true;
        });
        return this;
    };
    TextmateSnippet.prototype.appendChild = function (child) {
        this._placeholders = undefined;
        return _super.prototype.appendChild.call(this, child);
    };
    TextmateSnippet.prototype.replace = function (child, others) {
        this._placeholders = undefined;
        return _super.prototype.replace.call(this, child, others);
    };
    TextmateSnippet.prototype.clone = function () {
        var ret = new TextmateSnippet();
        this._children = this.children.map(function (child) { return child.clone(); });
        return ret;
    };
    TextmateSnippet.prototype.walk = function (visitor) {
        walk(this.children, visitor);
    };
    return TextmateSnippet;
}(Marker));
var SnippetParser = /** @class */ (function () {
    function SnippetParser() {
        this._scanner = new Scanner();
    }
    SnippetParser.escape = function (value) {
        return value.replace(/\$|}|\\/g, '\\$&');
    };
    SnippetParser.prototype.parse = function (value, insertFinalTabstop, enforceFinalTabstop) {
        this._scanner.text(value);
        this._token = this._scanner.next();
        var snippet = new TextmateSnippet();
        while (this._parse(snippet)) {
            // nothing
        }
        // fill in values for placeholders. the first placeholder of an index
        // that has a value defines the value for all placeholders with that index
        var placeholderDefaultValues = new Map();
        var incompletePlaceholders = [];
        var placeholderCount = 0;
        snippet.walk(function (marker) {
            if (marker instanceof Placeholder) {
                placeholderCount += 1;
                if (marker.isFinalTabstop) {
                    placeholderDefaultValues.set(0, undefined);
                }
                else if (!placeholderDefaultValues.has(marker.index) && marker.children.length > 0) {
                    placeholderDefaultValues.set(marker.index, marker.children);
                }
                else {
                    incompletePlaceholders.push(marker);
                }
            }
            return true;
        });
        for (var _i = 0, incompletePlaceholders_1 = incompletePlaceholders; _i < incompletePlaceholders_1.length; _i++) {
            var placeholder = incompletePlaceholders_1[_i];
            if (placeholderDefaultValues.has(placeholder.index)) {
                var clone = new Placeholder(placeholder.index);
                clone.transform = placeholder.transform;
                for (var _a = 0, _b = placeholderDefaultValues.get(placeholder.index); _a < _b.length; _a++) {
                    var child = _b[_a];
                    clone.appendChild(child.clone());
                }
                snippet.replace(placeholder, [clone]);
            }
        }
        if (!enforceFinalTabstop) {
            enforceFinalTabstop = placeholderCount > 0 && insertFinalTabstop;
        }
        if (!placeholderDefaultValues.has(0) && enforceFinalTabstop) {
            // the snippet uses placeholders but has no
            // final tabstop defined -> insert at the end
            snippet.appendChild(new Placeholder(0));
        }
        return snippet;
    };
    SnippetParser.prototype._accept = function (type, value) {
        if (type === undefined || this._token.type === type) {
            var ret = !value ? true : this._scanner.tokenText(this._token);
            this._token = this._scanner.next();
            return ret;
        }
        return false;
    };
    SnippetParser.prototype._backTo = function (token) {
        this._scanner.pos = token.pos + token.len;
        this._token = token;
        return false;
    };
    SnippetParser.prototype._until = function (type) {
        if (this._token.type === 14 /* EOF */) {
            return false;
        }
        var start = this._token;
        while (this._token.type !== type) {
            this._token = this._scanner.next();
            if (this._token.type === 14 /* EOF */) {
                return false;
            }
        }
        var value = this._scanner.value.substring(start.pos, this._token.pos);
        this._token = this._scanner.next();
        return value;
    };
    SnippetParser.prototype._parse = function (marker) {
        return this._parseEscaped(marker)
            || this._parseTabstopOrVariableName(marker)
            || this._parseComplexPlaceholder(marker)
            || this._parseComplexVariable(marker)
            || this._parseAnything(marker);
    };
    // \$, \\, \} -> just text
    SnippetParser.prototype._parseEscaped = function (marker) {
        var value;
        if (value = this._accept(5 /* Backslash */, true)) {
            // saw a backslash, append escaped token or that backslash
            value = this._accept(0 /* Dollar */, true)
                || this._accept(4 /* CurlyClose */, true)
                || this._accept(5 /* Backslash */, true)
                || value;
            marker.appendChild(new Text(value));
            return true;
        }
        return false;
    };
    // $foo -> variable, $1 -> tabstop
    SnippetParser.prototype._parseTabstopOrVariableName = function (parent) {
        var value;
        var token = this._token;
        var match = this._accept(0 /* Dollar */)
            && (value = this._accept(9 /* VariableName */, true) || this._accept(8 /* Int */, true));
        if (!match) {
            return this._backTo(token);
        }
        parent.appendChild(/^\d+$/.test(value)
            ? new Placeholder(Number(value))
            : new Variable(value));
        return true;
    };
    // ${1:<children>}, ${1} -> placeholder
    SnippetParser.prototype._parseComplexPlaceholder = function (parent) {
        var index;
        var token = this._token;
        var match = this._accept(0 /* Dollar */)
            && this._accept(3 /* CurlyOpen */)
            && (index = this._accept(8 /* Int */, true));
        if (!match) {
            return this._backTo(token);
        }
        var placeholder = new Placeholder(Number(index));
        if (this._accept(1 /* Colon */)) {
            // ${1:<children>}
            while (true) {
                // ...} -> done
                if (this._accept(4 /* CurlyClose */)) {
                    parent.appendChild(placeholder);
                    return true;
                }
                if (this._parse(placeholder)) {
                    continue;
                }
                // fallback
                parent.appendChild(new Text('${' + index + ':'));
                placeholder.children.forEach(parent.appendChild, parent);
                return true;
            }
        }
        else if (placeholder.index > 0 && this._accept(7 /* Pipe */)) {
            // ${1|one,two,three|}
            var choice = new Choice();
            while (true) {
                if (this._parseChoiceElement(choice)) {
                    if (this._accept(2 /* Comma */)) {
                        // opt, -> more
                        continue;
                    }
                    if (this._accept(7 /* Pipe */)) {
                        placeholder.appendChild(choice);
                        if (this._accept(4 /* CurlyClose */)) {
                            // ..|} -> done
                            parent.appendChild(placeholder);
                            return true;
                        }
                    }
                }
                this._backTo(token);
                return false;
            }
        }
        else if (this._accept(6 /* Forwardslash */)) {
            // ${1/<regex>/<format>/<options>}
            if (this._parseTransform(placeholder)) {
                parent.appendChild(placeholder);
                return true;
            }
            this._backTo(token);
            return false;
        }
        else if (this._accept(4 /* CurlyClose */)) {
            // ${1}
            parent.appendChild(placeholder);
            return true;
        }
        else {
            // ${1 <- missing curly or colon
            return this._backTo(token);
        }
    };
    SnippetParser.prototype._parseChoiceElement = function (parent) {
        var token = this._token;
        var values = [];
        while (true) {
            if (this._token.type === 2 /* Comma */ || this._token.type === 7 /* Pipe */) {
                break;
            }
            var value = void 0;
            if (value = this._accept(5 /* Backslash */, true)) {
                // \, \|, or \\
                value = this._accept(2 /* Comma */, true)
                    || this._accept(7 /* Pipe */, true)
                    || this._accept(5 /* Backslash */, true)
                    || value;
            }
            else {
                value = this._accept(undefined, true);
            }
            if (!value) {
                // EOF
                this._backTo(token);
                return false;
            }
            values.push(value);
        }
        if (values.length === 0) {
            this._backTo(token);
            return false;
        }
        parent.appendChild(new Text(values.join('')));
        return true;
    };
    // ${foo:<children>}, ${foo} -> variable
    SnippetParser.prototype._parseComplexVariable = function (parent) {
        var name;
        var token = this._token;
        var match = this._accept(0 /* Dollar */)
            && this._accept(3 /* CurlyOpen */)
            && (name = this._accept(9 /* VariableName */, true));
        if (!match) {
            return this._backTo(token);
        }
        var variable = new Variable(name);
        if (this._accept(1 /* Colon */)) {
            // ${foo:<children>}
            while (true) {
                // ...} -> done
                if (this._accept(4 /* CurlyClose */)) {
                    parent.appendChild(variable);
                    return true;
                }
                if (this._parse(variable)) {
                    continue;
                }
                // fallback
                parent.appendChild(new Text('${' + name + ':'));
                variable.children.forEach(parent.appendChild, parent);
                return true;
            }
        }
        else if (this._accept(6 /* Forwardslash */)) {
            // ${foo/<regex>/<format>/<options>}
            if (this._parseTransform(variable)) {
                parent.appendChild(variable);
                return true;
            }
            this._backTo(token);
            return false;
        }
        else if (this._accept(4 /* CurlyClose */)) {
            // ${foo}
            parent.appendChild(variable);
            return true;
        }
        else {
            // ${foo <- missing curly or colon
            return this._backTo(token);
        }
    };
    SnippetParser.prototype._parseTransform = function (parent) {
        // ...<regex>/<format>/<options>}
        var transform = new Transform();
        var regexValue = '';
        var regexOptions = '';
        // (1) /regex
        while (true) {
            if (this._accept(6 /* Forwardslash */)) {
                break;
            }
            var escaped = void 0;
            if (escaped = this._accept(5 /* Backslash */, true)) {
                escaped = this._accept(6 /* Forwardslash */, true) || escaped;
                regexValue += escaped;
                continue;
            }
            if (this._token.type !== 14 /* EOF */) {
                regexValue += this._accept(undefined, true);
                continue;
            }
            return false;
        }
        // (2) /format
        while (true) {
            if (this._accept(6 /* Forwardslash */)) {
                break;
            }
            var escaped = void 0;
            if (escaped = this._accept(5 /* Backslash */, true)) {
                escaped = this._accept(6 /* Forwardslash */, true) || escaped;
                transform.appendChild(new Text(escaped));
                continue;
            }
            if (this._parseFormatString(transform) || this._parseAnything(transform)) {
                continue;
            }
            return false;
        }
        // (3) /option
        while (true) {
            if (this._accept(4 /* CurlyClose */)) {
                break;
            }
            if (this._token.type !== 14 /* EOF */) {
                regexOptions += this._accept(undefined, true);
                continue;
            }
            return false;
        }
        try {
            transform.regexp = new RegExp(regexValue, regexOptions);
        }
        catch (e) {
            // invalid regexp
            return false;
        }
        parent.transform = transform;
        return true;
    };
    SnippetParser.prototype._parseFormatString = function (parent) {
        var token = this._token;
        if (!this._accept(0 /* Dollar */)) {
            return false;
        }
        var complex = false;
        if (this._accept(3 /* CurlyOpen */)) {
            complex = true;
        }
        var index = this._accept(8 /* Int */, true);
        if (!index) {
            this._backTo(token);
            return false;
        }
        else if (!complex) {
            // $1
            parent.appendChild(new FormatString(Number(index)));
            return true;
        }
        else if (this._accept(4 /* CurlyClose */)) {
            // ${1}
            parent.appendChild(new FormatString(Number(index)));
            return true;
        }
        else if (!this._accept(1 /* Colon */)) {
            this._backTo(token);
            return false;
        }
        if (this._accept(6 /* Forwardslash */)) {
            // ${1:/upcase}
            var shorthand = this._accept(9 /* VariableName */, true);
            if (!shorthand || !this._accept(4 /* CurlyClose */)) {
                this._backTo(token);
                return false;
            }
            else {
                parent.appendChild(new FormatString(Number(index), shorthand));
                return true;
            }
        }
        else if (this._accept(11 /* Plus */)) {
            // ${1:+<if>}
            var ifValue = this._until(4 /* CurlyClose */);
            if (ifValue) {
                parent.appendChild(new FormatString(Number(index), undefined, ifValue, undefined));
                return true;
            }
        }
        else if (this._accept(12 /* Dash */)) {
            // ${2:-<else>}
            var elseValue = this._until(4 /* CurlyClose */);
            if (elseValue) {
                parent.appendChild(new FormatString(Number(index), undefined, undefined, elseValue));
                return true;
            }
        }
        else if (this._accept(13 /* QuestionMark */)) {
            // ${2:?<if>:<else>}
            var ifValue = this._until(1 /* Colon */);
            if (ifValue) {
                var elseValue = this._until(4 /* CurlyClose */);
                if (elseValue) {
                    parent.appendChild(new FormatString(Number(index), undefined, ifValue, elseValue));
                    return true;
                }
            }
        }
        else {
            // ${1:<else>}
            var elseValue = this._until(4 /* CurlyClose */);
            if (elseValue) {
                parent.appendChild(new FormatString(Number(index), undefined, undefined, elseValue));
                return true;
            }
        }
        this._backTo(token);
        return false;
    };
    SnippetParser.prototype._parseAnything = function (marker) {
        if (this._token.type !== 14 /* EOF */) {
            marker.appendChild(new Text(this._scanner.tokenText(this._token)));
            this._accept(undefined);
            return true;
        }
        return false;
    };
    return SnippetParser;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CompositeSnippetVariableResolver = /** @class */ (function () {
    function CompositeSnippetVariableResolver(_delegates) {
        this._delegates = _delegates;
        //
    }
    CompositeSnippetVariableResolver.prototype.resolve = function (variable) {
        for (var _i = 0, _a = this._delegates; _i < _a.length; _i++) {
            var delegate = _a[_i];
            var value = delegate.resolve(variable);
            if (value !== void 0) {
                return value;
            }
        }
        return undefined;
    };
    return CompositeSnippetVariableResolver;
}());
var SelectionBasedVariableResolver = /** @class */ (function () {
    function SelectionBasedVariableResolver(_model, _selection) {
        this._model = _model;
        this._selection = _selection;
        //
    }
    SelectionBasedVariableResolver.prototype.resolve = function (variable) {
        var name = variable.name;
        if (name === 'SELECTION' || name === 'TM_SELECTED_TEXT') {
            var value = this._model.getValueInRange(this._selection) || undefined;
            if (value && this._selection.startLineNumber !== this._selection.endLineNumber) {
                // Selection is a multiline string which we indentation we now
                // need to adjust. We compare the indentation of this variable
                // with the indentation at the editor position and add potential
                // extra indentation to the value
                var line = this._model.getLineContent(this._selection.startLineNumber);
                var lineLeadingWhitespace = getLeadingWhitespace(line, 0, this._selection.startColumn - 1);
                var varLeadingWhitespace_1 = lineLeadingWhitespace;
                variable.snippet.walk(function (marker) {
                    if (marker === variable) {
                        return false;
                    }
                    if (marker instanceof Text) {
                        varLeadingWhitespace_1 = getLeadingWhitespace(marker.value.split(/\r\n|\r|\n/).pop());
                    }
                    return true;
                });
                var whitespaceCommonLength_1 = commonPrefixLength(varLeadingWhitespace_1, lineLeadingWhitespace);
                value = value.replace(/(\r\n|\r|\n)(.*)/g, function (m, newline, rest) { return "" + newline + varLeadingWhitespace_1.substr(whitespaceCommonLength_1) + rest; });
            }
            return value;
        }
        else if (name === 'TM_CURRENT_LINE') {
            return this._model.getLineContent(this._selection.positionLineNumber);
        }
        else if (name === 'TM_CURRENT_WORD') {
            var info = this._model.getWordAtPosition({
                lineNumber: this._selection.positionLineNumber,
                column: this._selection.positionColumn
            });
            return info && info.word || undefined;
        }
        else if (name === 'TM_LINE_INDEX') {
            return String(this._selection.positionLineNumber - 1);
        }
        else if (name === 'TM_LINE_NUMBER') {
            return String(this._selection.positionLineNumber);
        }
        return undefined;
    };
    return SelectionBasedVariableResolver;
}());
var ModelBasedVariableResolver = /** @class */ (function () {
    function ModelBasedVariableResolver(_model) {
        this._model = _model;
        //
    }
    ModelBasedVariableResolver.prototype.resolve = function (variable) {
        var name = variable.name;
        if (name === 'TM_FILENAME') {
            return basename(this._model.uri.fsPath);
        }
        else if (name === 'TM_FILENAME_BASE') {
            var name_1 = basename(this._model.uri.fsPath);
            var idx = name_1.lastIndexOf('.');
            if (idx <= 0) {
                return name_1;
            }
            else {
                return name_1.slice(0, idx);
            }
        }
        else if (name === 'TM_DIRECTORY') {
            var dir = dirname(this._model.uri.fsPath);
            return dir !== '.' ? dir : '';
        }
        else if (name === 'TM_FILEPATH') {
            return this._model.uri.fsPath;
        }
        return undefined;
    };
    return ModelBasedVariableResolver;
}());
var ClipboardBasedVariableResolver = /** @class */ (function () {
    function ClipboardBasedVariableResolver(_clipboardService, _selectionIdx, _selectionCount) {
        this._clipboardService = _clipboardService;
        this._selectionIdx = _selectionIdx;
        this._selectionCount = _selectionCount;
        //
    }
    ClipboardBasedVariableResolver.prototype.resolve = function (variable) {
        if (variable.name !== 'CLIPBOARD' || !this._clipboardService) {
            return undefined;
        }
        var text = this._clipboardService.readText();
        if (!text) {
            return undefined;
        }
        var lines = text.split(/\r\n|\n|\r/).filter(function (s) { return !isFalsyOrWhitespace(s); });
        if (lines.length === this._selectionCount) {
            return lines[this._selectionIdx];
        }
        else {
            return text;
        }
    };
    return ClipboardBasedVariableResolver;
}());
var TimeBasedVariableResolver = /** @class */ (function () {
    function TimeBasedVariableResolver() {
    }
    TimeBasedVariableResolver.prototype.resolve = function (variable) {
        var name = variable.name;
        if (name === 'CURRENT_YEAR') {
            return String(new Date().getFullYear());
        }
        else if (name === 'CURRENT_YEAR_SHORT') {
            return String(new Date().getFullYear()).slice(-2);
        }
        else if (name === 'CURRENT_MONTH') {
            return pad((new Date().getMonth().valueOf() + 1), 2);
        }
        else if (name === 'CURRENT_DATE') {
            return pad(new Date().getDate().valueOf(), 2);
        }
        else if (name === 'CURRENT_HOUR') {
            return pad(new Date().getHours().valueOf(), 2);
        }
        else if (name === 'CURRENT_MINUTE') {
            return pad(new Date().getMinutes().valueOf(), 2);
        }
        else if (name === 'CURRENT_SECOND') {
            return pad(new Date().getSeconds().valueOf(), 2);
        }
        else if (name === 'CURRENT_DAY_NAME') {
            return TimeBasedVariableResolver.dayNames[new Date().getDay()];
        }
        else if (name === 'CURRENT_DAY_NAME_SHORT') {
            return TimeBasedVariableResolver.dayNamesShort[new Date().getDay()];
        }
        else if (name === 'CURRENT_MONTH_NAME') {
            return TimeBasedVariableResolver.monthNames[new Date().getMonth()];
        }
        else if (name === 'CURRENT_MONTH_NAME_SHORT') {
            return TimeBasedVariableResolver.monthNamesShort[new Date().getMonth()];
        }
        return undefined;
    };
    TimeBasedVariableResolver.dayNames = [localize('Sunday', "Sunday"), localize('Monday', "Monday"), localize('Tuesday', "Tuesday"), localize('Wednesday', "Wednesday"), localize('Thursday', "Thursday"), localize('Friday', "Friday"), localize('Saturday', "Saturday")];
    TimeBasedVariableResolver.dayNamesShort = [localize('SundayShort', "Sun"), localize('MondayShort', "Mon"), localize('TuesdayShort', "Tue"), localize('WednesdayShort', "Wed"), localize('ThursdayShort', "Thu"), localize('FridayShort', "Fri"), localize('SaturdayShort', "Sat")];
    TimeBasedVariableResolver.monthNames = [localize('January', "January"), localize('February', "February"), localize('March', "March"), localize('April', "April"), localize('May', "May"), localize('June', "June"), localize('July', "July"), localize('August', "August"), localize('September', "September"), localize('October', "October"), localize('November', "November"), localize('December', "December")];
    TimeBasedVariableResolver.monthNamesShort = [localize('JanuaryShort', "Jan"), localize('FebruaryShort', "Feb"), localize('MarchShort', "Mar"), localize('AprilShort', "Apr"), localize('MayShort', "May"), localize('JuneShort', "Jun"), localize('JulyShort', "Jul"), localize('AugustShort', "Aug"), localize('SeptemberShort', "Sep"), localize('OctoberShort', "Oct"), localize('NovemberShort', "Nov"), localize('DecemberShort', "Dec")];
    return TimeBasedVariableResolver;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
registerThemingParticipant(function (theme, collector) {
    function getColorGraceful(name) {
        var color = theme.getColor(name);
        return color ? color.toString() : 'transparent';
    }
    collector.addRule(".monaco-editor .snippet-placeholder { background-color: " + getColorGraceful(snippetTabstopHighlightBackground) + "; outline-color: " + getColorGraceful(snippetTabstopHighlightBorder) + "; }");
    collector.addRule(".monaco-editor .finish-snippet-placeholder { background-color: " + getColorGraceful(snippetFinalTabstopHighlightBackground) + "; outline-color: " + getColorGraceful(snippetFinalTabstopHighlightBorder) + "; }");
});
var OneSnippet = /** @class */ (function () {
    function OneSnippet(editor, snippet, offset) {
        this._nestingLevel = 1;
        this._editor = editor;
        this._snippet = snippet;
        this._offset = offset;
        this._placeholderGroups = groupBy(snippet.placeholders, Placeholder.compareByIndex);
        this._placeholderGroupsIdx = -1;
    }
    OneSnippet.prototype.dispose = function () {
        if (this._placeholderDecorations) {
            var toRemove_1 = [];
            this._placeholderDecorations.forEach(function (handle) { return toRemove_1.push(handle); });
            this._editor.deltaDecorations(toRemove_1, []);
        }
        this._placeholderGroups.length = 0;
    };
    OneSnippet.prototype._initDecorations = function () {
        var _this = this;
        if (this._placeholderDecorations) {
            // already initialized
            return;
        }
        this._placeholderDecorations = new Map();
        var model = this._editor.getModel();
        this._editor.changeDecorations(function (accessor) {
            // create a decoration for each placeholder
            for (var _i = 0, _a = _this._snippet.placeholders; _i < _a.length; _i++) {
                var placeholder = _a[_i];
                var placeholderOffset = _this._snippet.offset(placeholder);
                var placeholderLen = _this._snippet.fullLen(placeholder);
                var range$$1 = Range.fromPositions(model.getPositionAt(_this._offset + placeholderOffset), model.getPositionAt(_this._offset + placeholderOffset + placeholderLen));
                var options = placeholder.isFinalTabstop ? OneSnippet._decor.inactiveFinal : OneSnippet._decor.inactive;
                var handle = accessor.addDecoration(range$$1, options);
                _this._placeholderDecorations.set(placeholder, handle);
            }
        });
    };
    OneSnippet.prototype.move = function (fwd) {
        var _this = this;
        this._initDecorations();
        // Transform placeholder text if necessary
        if (this._placeholderGroupsIdx >= 0) {
            var operations = [];
            for (var _i = 0, _a = this._placeholderGroups[this._placeholderGroupsIdx]; _i < _a.length; _i++) {
                var placeholder = _a[_i];
                // Check if the placeholder has a transformation
                if (placeholder.transform) {
                    var id = this._placeholderDecorations.get(placeholder);
                    var range$$1 = this._editor.getModel().getDecorationRange(id);
                    var currentValue = this._editor.getModel().getValueInRange(range$$1);
                    operations.push(EditOperation.replaceMove(range$$1, placeholder.transform.resolve(currentValue)));
                }
            }
            if (operations.length > 0) {
                this._editor.executeEdits('snippet.placeholderTransform', operations);
            }
        }
        var skipThisPlaceholder = false;
        if (fwd === true && this._placeholderGroupsIdx < this._placeholderGroups.length - 1) {
            this._placeholderGroupsIdx += 1;
            skipThisPlaceholder = true;
        }
        else if (fwd === false && this._placeholderGroupsIdx > 0) {
            this._placeholderGroupsIdx -= 1;
            skipThisPlaceholder = true;
        }
        var newSelections = this._editor.getModel().changeDecorations(function (accessor) {
            var activePlaceholders = new Set();
            // change stickiness to always grow when typing at its edges
            // because these decorations represent the currently active
            // tabstop.
            // Special case #1: reaching the final tabstop
            // Special case #2: placeholders enclosing active placeholders
            var selections = [];
            for (var _i = 0, _a = _this._placeholderGroups[_this._placeholderGroupsIdx]; _i < _a.length; _i++) {
                var placeholder = _a[_i];
                var id = _this._placeholderDecorations.get(placeholder);
                var range$$1 = _this._editor.getModel().getDecorationRange(id);
                selections.push(new Selection(range$$1.startLineNumber, range$$1.startColumn, range$$1.endLineNumber, range$$1.endColumn));
                // consider to skip this placeholder index when the decoration
                // range is empty but when the placeholder wasn't. that's a strong
                // hint that the placeholder has been deleted. (all placeholder must match this)
                skipThisPlaceholder = skipThisPlaceholder && (range$$1.isEmpty() && placeholder.toString().length > 0);
                accessor.changeDecorationOptions(id, placeholder.isFinalTabstop ? OneSnippet._decor.activeFinal : OneSnippet._decor.active);
                activePlaceholders.add(placeholder);
                for (var _b = 0, _c = _this._snippet.enclosingPlaceholders(placeholder); _b < _c.length; _b++) {
                    var enclosingPlaceholder = _c[_b];
                    var id_1 = _this._placeholderDecorations.get(enclosingPlaceholder);
                    accessor.changeDecorationOptions(id_1, enclosingPlaceholder.isFinalTabstop ? OneSnippet._decor.activeFinal : OneSnippet._decor.active);
                    activePlaceholders.add(enclosingPlaceholder);
                }
            }
            // change stickness to never grow when typing at its edges
            // so that in-active tabstops never grow
            _this._placeholderDecorations.forEach(function (id, placeholder) {
                if (!activePlaceholders.has(placeholder)) {
                    accessor.changeDecorationOptions(id, placeholder.isFinalTabstop ? OneSnippet._decor.inactiveFinal : OneSnippet._decor.inactive);
                }
            });
            return selections;
        });
        return !skipThisPlaceholder ? newSelections : this.move(fwd);
    };
    Object.defineProperty(OneSnippet.prototype, "isAtFirstPlaceholder", {
        get: function () {
            return this._placeholderGroupsIdx <= 0 || this._placeholderGroups.length === 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OneSnippet.prototype, "isAtLastPlaceholder", {
        get: function () {
            return this._placeholderGroupsIdx === this._placeholderGroups.length - 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OneSnippet.prototype, "hasPlaceholder", {
        get: function () {
            return this._snippet.placeholders.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    OneSnippet.prototype.computePossibleSelections = function () {
        var result = new Map();
        for (var _i = 0, _a = this._placeholderGroups; _i < _a.length; _i++) {
            var placeholdersWithEqualIndex = _a[_i];
            var ranges = void 0;
            for (var _b = 0, placeholdersWithEqualIndex_1 = placeholdersWithEqualIndex; _b < placeholdersWithEqualIndex_1.length; _b++) {
                var placeholder = placeholdersWithEqualIndex_1[_b];
                if (placeholder.isFinalTabstop) {
                    // ignore those
                    break;
                }
                if (!ranges) {
                    ranges = [];
                    result.set(placeholder.index, ranges);
                }
                var id = this._placeholderDecorations.get(placeholder);
                var range$$1 = this._editor.getModel().getDecorationRange(id);
                if (!range$$1) {
                    // one of the placeholder lost its decoration and
                    // therefore we bail out and pretend the placeholder
                    // (with its mirrors) doesn't exist anymore.
                    result.delete(placeholder.index);
                    break;
                }
                ranges.push(range$$1);
            }
        }
        return result;
    };
    Object.defineProperty(OneSnippet.prototype, "choice", {
        get: function () {
            return this._placeholderGroups[this._placeholderGroupsIdx][0].choice;
        },
        enumerable: true,
        configurable: true
    });
    OneSnippet.prototype.merge = function (others) {
        var _this = this;
        var model = this._editor.getModel();
        this._nestingLevel *= 10;
        this._editor.changeDecorations(function (accessor) {
            // For each active placeholder take one snippet and merge it
            // in that the placeholder (can be many for `$1foo$1foo`). Because
            // everything is sorted by editor selection we can simply remove
            // elements from the beginning of the array
            for (var _i = 0, _a = _this._placeholderGroups[_this._placeholderGroupsIdx]; _i < _a.length; _i++) {
                var placeholder = _a[_i];
                var nested = others.shift();
                console.assert(!nested._placeholderDecorations);
                // Massage placeholder-indicies of the nested snippet to be
                // sorted right after the insertion point. This ensures we move
                // through the placeholders in the correct order
                var indexLastPlaceholder = nested._snippet.placeholderInfo.last.index;
                for (var _b = 0, _c = nested._snippet.placeholderInfo.all; _b < _c.length; _b++) {
                    var nestedPlaceholder = _c[_b];
                    if (nestedPlaceholder.isFinalTabstop) {
                        nestedPlaceholder.index = placeholder.index + ((indexLastPlaceholder + 1) / _this._nestingLevel);
                    }
                    else {
                        nestedPlaceholder.index = placeholder.index + (nestedPlaceholder.index / _this._nestingLevel);
                    }
                }
                _this._snippet.replace(placeholder, nested._snippet.children);
                // Remove the placeholder at which position are inserting
                // the snippet and also remove its decoration.
                var id = _this._placeholderDecorations.get(placeholder);
                accessor.removeDecoration(id);
                _this._placeholderDecorations.delete(placeholder);
                // For each *new* placeholder we create decoration to monitor
                // how and if it grows/shrinks.
                for (var _d = 0, _e = nested._snippet.placeholders; _d < _e.length; _d++) {
                    var placeholder_1 = _e[_d];
                    var placeholderOffset = nested._snippet.offset(placeholder_1);
                    var placeholderLen = nested._snippet.fullLen(placeholder_1);
                    var range$$1 = Range.fromPositions(model.getPositionAt(nested._offset + placeholderOffset), model.getPositionAt(nested._offset + placeholderOffset + placeholderLen));
                    var handle = accessor.addDecoration(range$$1, OneSnippet._decor.inactive);
                    _this._placeholderDecorations.set(placeholder_1, handle);
                }
            }
            // Last, re-create the placeholder groups by sorting placeholders by their index.
            _this._placeholderGroups = groupBy(_this._snippet.placeholders, Placeholder.compareByIndex);
        });
    };
    OneSnippet._decor = {
        active: ModelDecorationOptions.register({ stickiness: 0 /* AlwaysGrowsWhenTypingAtEdges */, className: 'snippet-placeholder' }),
        inactive: ModelDecorationOptions.register({ stickiness: 1 /* NeverGrowsWhenTypingAtEdges */, className: 'snippet-placeholder' }),
        activeFinal: ModelDecorationOptions.register({ stickiness: 1 /* NeverGrowsWhenTypingAtEdges */, className: 'finish-snippet-placeholder' }),
        inactiveFinal: ModelDecorationOptions.register({ stickiness: 1 /* NeverGrowsWhenTypingAtEdges */, className: 'finish-snippet-placeholder' }),
    };
    return OneSnippet;
}());
var SnippetSession = /** @class */ (function () {
    function SnippetSession(editor, template, overwriteBefore, overwriteAfter, adjustWhitespace) {
        if (overwriteBefore === void 0) { overwriteBefore = 0; }
        if (overwriteAfter === void 0) { overwriteAfter = 0; }
        if (adjustWhitespace === void 0) { adjustWhitespace = true; }
        this._templateMerges = [];
        this._snippets = [];
        this._editor = editor;
        this._template = template;
        this._overwriteBefore = overwriteBefore;
        this._overwriteAfter = overwriteAfter;
        this._adjustWhitespace = adjustWhitespace;
    }
    SnippetSession.adjustWhitespace = function (model, position, snippet) {
        var line = model.getLineContent(position.lineNumber);
        var lineLeadingWhitespace = getLeadingWhitespace(line, 0, position.column - 1);
        snippet.walk(function (marker) {
            if (marker instanceof Text && !(marker.parent instanceof Choice)) {
                // adjust indentation of text markers, except for choise elements
                // which get adjusted when being selected
                var lines = marker.value.split(/\r\n|\r|\n/);
                for (var i = 1; i < lines.length; i++) {
                    var templateLeadingWhitespace = getLeadingWhitespace(lines[i]);
                    lines[i] = model.normalizeIndentation(lineLeadingWhitespace + templateLeadingWhitespace) + lines[i].substr(templateLeadingWhitespace.length);
                }
                var newValue = lines.join(model.getEOL());
                if (newValue !== marker.value) {
                    marker.parent.replace(marker, [new Text(newValue)]);
                }
            }
            return true;
        });
    };
    SnippetSession.adjustSelection = function (model, selection, overwriteBefore, overwriteAfter) {
        if (overwriteBefore !== 0 || overwriteAfter !== 0) {
            // overwrite[Before|After] is compute using the position, not the whole
            // selection. therefore we adjust the selection around that position
            var positionLineNumber = selection.positionLineNumber, positionColumn = selection.positionColumn;
            var positionColumnBefore = positionColumn - overwriteBefore;
            var positionColumnAfter = positionColumn + overwriteAfter;
            var range$$1 = model.validateRange({
                startLineNumber: positionLineNumber,
                startColumn: positionColumnBefore,
                endLineNumber: positionLineNumber,
                endColumn: positionColumnAfter
            });
            selection = Selection.createWithDirection(range$$1.startLineNumber, range$$1.startColumn, range$$1.endLineNumber, range$$1.endColumn, selection.getDirection());
        }
        return selection;
    };
    SnippetSession.createEditsAndSnippets = function (editor, template, overwriteBefore, overwriteAfter, enforceFinalTabstop, adjustWhitespace) {
        var model = editor.getModel();
        var edits = [];
        var snippets = [];
        var modelBasedVariableResolver = new ModelBasedVariableResolver(model);
        var clipboardService = editor.invokeWithinContext(function (accessor) { return accessor.get(IClipboardService, optional); });
        var delta = 0;
        // know what text the overwrite[Before|After] extensions
        // of the primary curser have selected because only when
        // secondary selections extend to the same text we can grow them
        var firstBeforeText = model.getValueInRange(SnippetSession.adjustSelection(model, editor.getSelection(), overwriteBefore, 0));
        var firstAfterText = model.getValueInRange(SnippetSession.adjustSelection(model, editor.getSelection(), 0, overwriteAfter));
        // remember the first non-whitespace column to decide if
        // `keepWhitespace` should be overruled for secondary selections
        var firstLineFirstNonWhitespace = model.getLineFirstNonWhitespaceColumn(editor.getSelection().positionLineNumber);
        // sort selections by their start position but remeber
        // the original index. that allows you to create correct
        // offset-based selection logic without changing the
        // primary selection
        var indexedSelections = editor.getSelections()
            .map(function (selection, idx) { return ({ selection: selection, idx: idx }); })
            .sort(function (a, b) { return Range.compareRangesUsingStarts(a.selection, b.selection); });
        for (var _i = 0, indexedSelections_1 = indexedSelections; _i < indexedSelections_1.length; _i++) {
            var _a = indexedSelections_1[_i], selection = _a.selection, idx = _a.idx;
            // extend selection with the `overwriteBefore` and `overwriteAfter` and then
            // compare if this matches the extensions of the primary selection
            var extensionBefore = SnippetSession.adjustSelection(model, selection, overwriteBefore, 0);
            var extensionAfter = SnippetSession.adjustSelection(model, selection, 0, overwriteAfter);
            if (firstBeforeText !== model.getValueInRange(extensionBefore)) {
                extensionBefore = selection;
            }
            if (firstAfterText !== model.getValueInRange(extensionAfter)) {
                extensionAfter = selection;
            }
            // merge the before and after selection into one
            var snippetSelection = selection
                .setStartPosition(extensionBefore.startLineNumber, extensionBefore.startColumn)
                .setEndPosition(extensionAfter.endLineNumber, extensionAfter.endColumn);
            var snippet = new SnippetParser().parse(template, true, enforceFinalTabstop);
            // adjust the template string to match the indentation and
            // whitespace rules of this insert location (can be different for each cursor)
            // happens when being asked for (default) or when this is a secondary
            // cursor and the leading whitespace is different
            var start = snippetSelection.getStartPosition();
            if (adjustWhitespace || (idx > 0 && firstLineFirstNonWhitespace !== model.getLineFirstNonWhitespaceColumn(selection.positionLineNumber))) {
                SnippetSession.adjustWhitespace(model, start, snippet);
            }
            snippet.resolveVariables(new CompositeSnippetVariableResolver([
                modelBasedVariableResolver,
                new ClipboardBasedVariableResolver(clipboardService, idx, indexedSelections.length),
                new SelectionBasedVariableResolver(model, selection),
                new TimeBasedVariableResolver
            ]));
            var offset = model.getOffsetAt(start) + delta;
            delta += snippet.toString().length - model.getValueLengthInRange(snippetSelection);
            // store snippets with the index of their originating selection.
            // that ensures the primiary cursor stays primary despite not being
            // the one with lowest start position
            edits[idx] = EditOperation.replace(snippetSelection, snippet.toString());
            snippets[idx] = new OneSnippet(editor, snippet, offset);
        }
        return { edits: edits, snippets: snippets };
    };
    SnippetSession.prototype.dispose = function () {
        dispose(this._snippets);
    };
    SnippetSession.prototype._logInfo = function () {
        return "template=\"" + this._template + "\", merged_templates=\"" + this._templateMerges.join(' -> ') + "\"";
    };
    SnippetSession.prototype.insert = function () {
        var _this = this;
        var model = this._editor.getModel();
        // make insert edit and start with first selections
        var _a = SnippetSession.createEditsAndSnippets(this._editor, this._template, this._overwriteBefore, this._overwriteAfter, false, this._adjustWhitespace), edits = _a.edits, snippets = _a.snippets;
        this._snippets = snippets;
        var selections = model.pushEditOperations(this._editor.getSelections(), edits, function (undoEdits) {
            if (_this._snippets[0].hasPlaceholder) {
                return _this._move(true);
            }
            else {
                return undoEdits.map(function (edit) { return Selection.fromPositions(edit.range.getEndPosition()); });
            }
        });
        this._editor.setSelections(selections);
        this._editor.revealRange(selections[0]);
    };
    SnippetSession.prototype.merge = function (template, overwriteBefore, overwriteAfter, adjustWhitespace) {
        var _this = this;
        if (overwriteBefore === void 0) { overwriteBefore = 0; }
        if (overwriteAfter === void 0) { overwriteAfter = 0; }
        if (adjustWhitespace === void 0) { adjustWhitespace = true; }
        this._templateMerges.push([this._snippets[0]._nestingLevel, this._snippets[0]._placeholderGroupsIdx, template]);
        var _a = SnippetSession.createEditsAndSnippets(this._editor, template, overwriteBefore, overwriteAfter, true, adjustWhitespace), edits = _a.edits, snippets = _a.snippets;
        this._editor.setSelections(this._editor.getModel().pushEditOperations(this._editor.getSelections(), edits, function (undoEdits) {
            for (var _i = 0, _a = _this._snippets; _i < _a.length; _i++) {
                var snippet = _a[_i];
                snippet.merge(snippets);
            }
            console.assert(snippets.length === 0);
            if (_this._snippets[0].hasPlaceholder) {
                return _this._move(undefined);
            }
            else {
                return undoEdits.map(function (edit) { return Selection.fromPositions(edit.range.getEndPosition()); });
            }
        }));
    };
    SnippetSession.prototype.next = function () {
        var newSelections = this._move(true);
        this._editor.setSelections(newSelections);
        this._editor.revealPositionInCenterIfOutsideViewport(newSelections[0].getPosition());
    };
    SnippetSession.prototype.prev = function () {
        var newSelections = this._move(false);
        this._editor.setSelections(newSelections);
        this._editor.revealPositionInCenterIfOutsideViewport(newSelections[0].getPosition());
    };
    SnippetSession.prototype._move = function (fwd) {
        var selections = [];
        for (var _i = 0, _a = this._snippets; _i < _a.length; _i++) {
            var snippet = _a[_i];
            var oneSelection = snippet.move(fwd);
            selections.push.apply(selections, oneSelection);
        }
        return selections;
    };
    Object.defineProperty(SnippetSession.prototype, "isAtFirstPlaceholder", {
        get: function () {
            return this._snippets[0].isAtFirstPlaceholder;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SnippetSession.prototype, "isAtLastPlaceholder", {
        get: function () {
            return this._snippets[0].isAtLastPlaceholder;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SnippetSession.prototype, "hasPlaceholder", {
        get: function () {
            return this._snippets[0].hasPlaceholder;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SnippetSession.prototype, "choice", {
        get: function () {
            return this._snippets[0].choice;
        },
        enumerable: true,
        configurable: true
    });
    SnippetSession.prototype.isSelectionWithinPlaceholders = function () {
        if (!this.hasPlaceholder) {
            return false;
        }
        var selections = this._editor.getSelections();
        if (selections.length < this._snippets.length) {
            // this means we started snippet mode with N
            // selections and have M (N > M) selections.
            // So one snippet is without selection -> cancel
            return false;
        }
        var allPossibleSelections;
        var _loop_1 = function (snippet) {
            var possibleSelections = snippet.computePossibleSelections();
            // for the first snippet find the placeholder (and its ranges)
            // that contain at least one selection. for all remaining snippets
            // the same placeholder (and their ranges) must be used.
            if (!allPossibleSelections) {
                allPossibleSelections = new Map();
                possibleSelections.forEach(function (ranges, index) {
                    ranges.sort(Range.compareRangesUsingStarts);
                    for (var _i = 0, selections_1 = selections; _i < selections_1.length; _i++) {
                        var selection = selections_1[_i];
                        if (ranges[0].containsRange(selection)) {
                            allPossibleSelections.set(index, []);
                            break;
                        }
                    }
                });
            }
            if (allPossibleSelections.size === 0) {
                return { value: false };
            }
            // add selections from 'this' snippet so that we know all
            // selections for this placeholder
            allPossibleSelections.forEach(function (array, index) {
                array.push.apply(array, possibleSelections.get(index));
            });
        };
        for (var _i = 0, _a = this._snippets; _i < _a.length; _i++) {
            var snippet = _a[_i];
            var state_1 = _loop_1(snippet);
            if (typeof state_1 === "object")
                return state_1.value;
        }
        // sort selections (and later placeholder-ranges). then walk both
        // arrays and make sure the placeholder-ranges contain the corresponding
        // selection
        selections.sort(Range.compareRangesUsingStarts);
        allPossibleSelections.forEach(function (ranges, index) {
            if (ranges.length !== selections.length) {
                allPossibleSelections.delete(index);
                return;
            }
            ranges.sort(Range.compareRangesUsingStarts);
            for (var i = 0; i < ranges.length; i++) {
                if (!ranges[i].containsRange(selections[i])) {
                    allPossibleSelections.delete(index);
                    return;
                }
            }
        });
        // from all possible selections we have deleted those
        // that don't match with the current selection. if we don't
        // have any left, we don't have a selection anymore
        return allPossibleSelections.size > 0;
    };
    return SnippetSession;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$f = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$e = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var SnippetController2 = /** @class */ (function () {
    function SnippetController2(_editor, _logService, contextKeyService) {
        this._editor = _editor;
        this._logService = _logService;
        this._snippetListener = [];
        this._inSnippet = SnippetController2.InSnippetMode.bindTo(contextKeyService);
        this._hasNextTabstop = SnippetController2.HasNextTabstop.bindTo(contextKeyService);
        this._hasPrevTabstop = SnippetController2.HasPrevTabstop.bindTo(contextKeyService);
    }
    SnippetController2.get = function (editor) {
        return editor.getContribution('snippetController2');
    };
    SnippetController2.prototype.dispose = function () {
        this._inSnippet.reset();
        this._hasPrevTabstop.reset();
        this._hasNextTabstop.reset();
        dispose(this._session);
    };
    SnippetController2.prototype.getId = function () {
        return 'snippetController2';
    };
    SnippetController2.prototype.insert = function (template, overwriteBefore, overwriteAfter, undoStopBefore, undoStopAfter, adjustWhitespace) {
        if (overwriteBefore === void 0) { overwriteBefore = 0; }
        if (overwriteAfter === void 0) { overwriteAfter = 0; }
        if (undoStopBefore === void 0) { undoStopBefore = true; }
        if (undoStopAfter === void 0) { undoStopAfter = true; }
        if (adjustWhitespace === void 0) { adjustWhitespace = true; }
        // this is here to find out more about the yet-not-understood
        // error that sometimes happens when we fail to inserted a nested
        // snippet
        try {
            this._doInsert(template, overwriteBefore, overwriteAfter, undoStopBefore, undoStopAfter, adjustWhitespace);
        }
        catch (e) {
            this.cancel();
            this._logService.error(e);
            this._logService.error('snippet_error');
            this._logService.error('insert_template=', template);
            this._logService.error('existing_template=', this._session ? this._session._logInfo() : '<no_session>');
        }
    };
    SnippetController2.prototype._doInsert = function (template, overwriteBefore, overwriteAfter, undoStopBefore, undoStopAfter, adjustWhitespace) {
        var _this = this;
        if (overwriteBefore === void 0) { overwriteBefore = 0; }
        if (overwriteAfter === void 0) { overwriteAfter = 0; }
        if (undoStopBefore === void 0) { undoStopBefore = true; }
        if (undoStopAfter === void 0) { undoStopAfter = true; }
        if (adjustWhitespace === void 0) { adjustWhitespace = true; }
        // don't listen while inserting the snippet
        // as that is the inflight state causing cancelation
        this._snippetListener = dispose(this._snippetListener);
        if (undoStopBefore) {
            this._editor.getModel().pushStackElement();
        }
        if (!this._session) {
            this._modelVersionId = this._editor.getModel().getAlternativeVersionId();
            this._session = new SnippetSession(this._editor, template, overwriteBefore, overwriteAfter, adjustWhitespace);
            this._session.insert();
        }
        else {
            this._session.merge(template, overwriteBefore, overwriteAfter, adjustWhitespace);
        }
        if (undoStopAfter) {
            this._editor.getModel().pushStackElement();
        }
        this._updateState();
        this._snippetListener = [
            this._editor.onDidChangeModelContent(function (e) { return e.isFlush && _this.cancel(); }),
            this._editor.onDidChangeModel(function () { return _this.cancel(); }),
            this._editor.onDidChangeCursorSelection(function () { return _this._updateState(); })
        ];
    };
    SnippetController2.prototype._updateState = function () {
        if (!this._session) {
            // canceled in the meanwhile
            return;
        }
        if (this._modelVersionId === this._editor.getModel().getAlternativeVersionId()) {
            // undo until the 'before' state happened
            // and makes use cancel snippet mode
            return this.cancel();
        }
        if (!this._session.hasPlaceholder) {
            // don't listen for selection changes and don't
            // update context keys when the snippet is plain text
            return this.cancel();
        }
        if (this._session.isAtLastPlaceholder || !this._session.isSelectionWithinPlaceholders()) {
            return this.cancel();
        }
        this._inSnippet.set(true);
        this._hasPrevTabstop.set(!this._session.isAtFirstPlaceholder);
        this._hasNextTabstop.set(!this._session.isAtLastPlaceholder);
        this._handleChoice();
    };
    SnippetController2.prototype._handleChoice = function () {
        var _this = this;
        var choice = this._session.choice;
        if (!choice) {
            this._currentChoice = undefined;
            return;
        }
        if (this._currentChoice !== choice) {
            this._currentChoice = choice;
            this._editor.setSelections(this._editor.getSelections()
                .map(function (s) { return Selection.fromPositions(s.getStartPosition()); }));
            var first_1 = choice.options[0];
            showSimpleSuggestions(this._editor, choice.options.map(function (option, i) {
                // let before = choice.options.slice(0, i);
                // let after = choice.options.slice(i);
                return {
                    kind: 13 /* Value */,
                    label: option.value,
                    insertText: option.value,
                    // insertText: `\${1|${after.concat(before).join(',')}|}$0`,
                    // snippetType: 'textmate',
                    sortText: repeat('a', i),
                    range: Range.fromPositions(_this._editor.getPosition(), _this._editor.getPosition().delta(0, first_1.value.length))
                };
            }));
        }
    };
    SnippetController2.prototype.finish = function () {
        while (this._inSnippet.get()) {
            this.next();
        }
    };
    SnippetController2.prototype.cancel = function () {
        this._inSnippet.reset();
        this._hasPrevTabstop.reset();
        this._hasNextTabstop.reset();
        dispose(this._snippetListener);
        dispose(this._session);
        this._session = undefined;
        this._modelVersionId = -1;
    };
    SnippetController2.prototype.prev = function () {
        this._session.prev();
        this._updateState();
    };
    SnippetController2.prototype.next = function () {
        this._session.next();
        this._updateState();
    };
    SnippetController2.prototype.isInSnippet = function () {
        return this._inSnippet.get();
    };
    SnippetController2.InSnippetMode = new RawContextKey('inSnippetMode', false);
    SnippetController2.HasNextTabstop = new RawContextKey('hasNextTabstop', false);
    SnippetController2.HasPrevTabstop = new RawContextKey('hasPrevTabstop', false);
    SnippetController2 = __decorate$f([
        __param$e(1, ILogService),
        __param$e(2, IContextKeyService)
    ], SnippetController2);
    return SnippetController2;
}());
registerEditorContribution(SnippetController2);
var CommandCtor = EditorCommand.bindToContribution(SnippetController2.get);
registerEditorCommand(new CommandCtor({
    id: 'jumpToNextSnippetPlaceholder',
    precondition: ContextKeyExpr.and(SnippetController2.InSnippetMode, SnippetController2.HasNextTabstop),
    handler: function (ctrl) { return ctrl.next(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 30,
        kbExpr: EditorContextKeys.editorTextFocus,
        primary: 2 /* Tab */
    }
}));
registerEditorCommand(new CommandCtor({
    id: 'jumpToPrevSnippetPlaceholder',
    precondition: ContextKeyExpr.and(SnippetController2.InSnippetMode, SnippetController2.HasPrevTabstop),
    handler: function (ctrl) { return ctrl.prev(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 30,
        kbExpr: EditorContextKeys.editorTextFocus,
        primary: 1024 /* Shift */ | 2 /* Tab */
    }
}));
registerEditorCommand(new CommandCtor({
    id: 'leaveSnippet',
    precondition: SnippetController2.InSnippetMode,
    handler: function (ctrl) { return ctrl.cancel(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 30,
        kbExpr: EditorContextKeys.editorTextFocus,
        primary: 9 /* Escape */,
        secondary: [1024 /* Shift */ | 9 /* Escape */]
    }
}));
registerEditorCommand(new CommandCtor({
    id: 'acceptSnippet',
    precondition: SnippetController2.InSnippetMode,
    handler: function (ctrl) { return ctrl.finish(); },
}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1E = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$g = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$f = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var Memory = /** @class */ (function () {
    function Memory() {
    }
    Memory.prototype.select = function (model, pos, items) {
        if (items.length === 0) {
            return 0;
        }
        var topScore = items[0].score;
        for (var i = 1; i < items.length; i++) {
            var _a = items[i], score = _a.score, suggestion = _a.suggestion;
            if (score !== topScore) {
                // stop when leaving the group of top matches
                break;
            }
            if (suggestion.preselect) {
                // stop when seeing an auto-select-item
                return i;
            }
        }
        return 0;
    };
    return Memory;
}());
var NoMemory = /** @class */ (function (_super) {
    __extends$1E(NoMemory, _super);
    function NoMemory() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NoMemory.prototype.memorize = function (model, pos, item) {
        // no-op
    };
    NoMemory.prototype.toJSON = function () {
        return undefined;
    };
    NoMemory.prototype.fromJSON = function () {
        //
    };
    return NoMemory;
}(Memory));
var LRUMemory = /** @class */ (function (_super) {
    __extends$1E(LRUMemory, _super);
    function LRUMemory() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._cache = new LRUCache(300, .66);
        _this._seq = 0;
        return _this;
    }
    LRUMemory.prototype.memorize = function (model, pos, item) {
        var label = item.suggestion.label;
        var key = model.getLanguageIdentifier().language + "/" + label;
        this._cache.set(key, {
            touch: this._seq++,
            type: item.suggestion.kind,
            insertText: item.suggestion.insertText
        });
    };
    LRUMemory.prototype.select = function (model, pos, items) {
        // in order of completions, select the first
        // that has been used in the past
        var word = model.getWordUntilPosition(pos).word;
        if (word.length !== 0) {
            return _super.prototype.select.call(this, model, pos, items);
        }
        var lineSuffix = model.getLineContent(pos.lineNumber).substr(pos.column - 10, pos.column - 1);
        if (/\s$/.test(lineSuffix)) {
            return _super.prototype.select.call(this, model, pos, items);
        }
        var res = -1;
        var seq = -1;
        for (var i = 0; i < items.length; i++) {
            var suggestion = items[i].suggestion;
            var key = model.getLanguageIdentifier().language + "/" + suggestion.label;
            var item = this._cache.get(key);
            if (item && item.touch > seq && item.type === suggestion.kind && item.insertText === suggestion.insertText) {
                seq = item.touch;
                res = i;
            }
        }
        if (res === -1) {
            return _super.prototype.select.call(this, model, pos, items);
        }
        else {
            return res;
        }
    };
    LRUMemory.prototype.toJSON = function () {
        var data = [];
        this._cache.forEach(function (value, key) {
            data.push([key, value]);
        });
        return data;
    };
    LRUMemory.prototype.fromJSON = function (data) {
        this._cache.clear();
        var seq = 0;
        for (var _i = 0, data_1 = data; _i < data_1.length; _i++) {
            var _a = data_1[_i], key = _a[0], value = _a[1];
            value.touch = seq;
            value.type = typeof value.type === 'number' ? value.type : completionKindFromLegacyString(value.type);
            this._cache.set(key, value);
        }
        this._seq = this._cache.size;
    };
    return LRUMemory;
}(Memory));
var PrefixMemory = /** @class */ (function (_super) {
    __extends$1E(PrefixMemory, _super);
    function PrefixMemory() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._trie = TernarySearchTree.forStrings();
        _this._seq = 0;
        return _this;
    }
    PrefixMemory.prototype.memorize = function (model, pos, item) {
        var word = model.getWordUntilPosition(pos).word;
        var key = model.getLanguageIdentifier().language + "/" + word;
        this._trie.set(key, {
            type: item.suggestion.kind,
            insertText: item.suggestion.insertText,
            touch: this._seq++
        });
    };
    PrefixMemory.prototype.select = function (model, pos, items) {
        var word = model.getWordUntilPosition(pos).word;
        if (!word) {
            return _super.prototype.select.call(this, model, pos, items);
        }
        var key = model.getLanguageIdentifier().language + "/" + word;
        var item = this._trie.get(key);
        if (!item) {
            item = this._trie.findSubstr(key);
        }
        if (item) {
            for (var i = 0; i < items.length; i++) {
                var _a = items[i].suggestion, kind = _a.kind, insertText = _a.insertText;
                if (kind === item.type && insertText === item.insertText) {
                    return i;
                }
            }
        }
        return _super.prototype.select.call(this, model, pos, items);
    };
    PrefixMemory.prototype.toJSON = function () {
        var entries = [];
        this._trie.forEach(function (value, key) { return entries.push([key, value]); });
        // sort by last recently used (touch), then
        // take the top 200 item and normalize their
        // touch
        entries
            .sort(function (a, b) { return -(a[1].touch - b[1].touch); })
            .forEach(function (value, i) { return value[1].touch = i; });
        return entries.slice(0, 200);
    };
    PrefixMemory.prototype.fromJSON = function (data) {
        this._trie.clear();
        if (data.length > 0) {
            this._seq = data[0][1].touch + 1;
            for (var _i = 0, data_2 = data; _i < data_2.length; _i++) {
                var _a = data_2[_i], key = _a[0], value = _a[1];
                value.type = typeof value.type === 'number' ? value.type : completionKindFromLegacyString(value.type);
                this._trie.set(key, value);
            }
        }
    };
    return PrefixMemory;
}(Memory));
var SuggestMemories = /** @class */ (function (_super) {
    __extends$1E(SuggestMemories, _super);
    function SuggestMemories(editor, _storageService) {
        var _this = _super.call(this) || this;
        _this._storageService = _storageService;
        _this._storagePrefix = 'suggest/memories';
        _this._setMode(editor.getConfiguration().contribInfo.suggestSelection);
        _this._register(editor.onDidChangeConfiguration(function (e) { return e.contribInfo && _this._setMode(editor.getConfiguration().contribInfo.suggestSelection); }));
        _this._register(_storageService.onWillSaveState(function () { return _this._saveState(); }));
        return _this;
    }
    SuggestMemories.prototype._setMode = function (mode) {
        if (this._mode === mode) {
            return;
        }
        this._mode = mode;
        this._strategy = mode === 'recentlyUsedByPrefix' ? new PrefixMemory() : mode === 'recentlyUsed' ? new LRUMemory() : new NoMemory();
        try {
            var raw = this._storageService.get(this._storagePrefix + "/" + this._mode, 1 /* WORKSPACE */);
            if (raw) {
                this._strategy.fromJSON(JSON.parse(raw));
            }
        }
        catch (e) {
            // things can go wrong with JSON...
        }
    };
    SuggestMemories.prototype.memorize = function (model, pos, item) {
        this._strategy.memorize(model, pos, item);
    };
    SuggestMemories.prototype.select = function (model, pos, items) {
        return this._strategy.select(model, pos, items);
    };
    SuggestMemories.prototype._saveState = function () {
        var raw = JSON.stringify(this._strategy);
        this._storageService.store(this._storagePrefix + "/" + this._mode, raw, 1 /* WORKSPACE */);
    };
    SuggestMemories = __decorate$g([
        __param$f(1, IStorageService)
    ], SuggestMemories);
    return SuggestMemories;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$h = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$g = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var SuggestAlternatives = /** @class */ (function () {
    function SuggestAlternatives(_editor, contextKeyService) {
        this._editor = _editor;
        this._ckOtherSuggestions = SuggestAlternatives.OtherSuggestions.bindTo(contextKeyService);
    }
    SuggestAlternatives.prototype.dispose = function () {
        this.reset();
    };
    SuggestAlternatives.prototype.reset = function () {
        this._ckOtherSuggestions.reset();
        dispose(this._listener);
        this._model = undefined;
        this._acceptNext = undefined;
        this._ignore = false;
    };
    SuggestAlternatives.prototype.set = function (_a, acceptNext) {
        var _this = this;
        var model = _a.model, index = _a.index;
        // no suggestions -> nothing to do
        if (model.items.length === 0) {
            this.reset();
            return;
        }
        // no alternative suggestions -> nothing to do
        var nextIndex = SuggestAlternatives._moveIndex(true, model, index);
        if (nextIndex === index) {
            this.reset();
            return;
        }
        this._acceptNext = acceptNext;
        this._model = model;
        this._index = index;
        this._listener = this._editor.onDidChangeCursorPosition(function () {
            if (!_this._ignore) {
                _this.reset();
            }
        });
        this._ckOtherSuggestions.set(true);
    };
    SuggestAlternatives._moveIndex = function (fwd, model, index) {
        var newIndex = index;
        while (true) {
            newIndex = (newIndex + model.items.length + (fwd ? +1 : -1)) % model.items.length;
            if (newIndex === index) {
                break;
            }
            if (!model.items[newIndex].suggestion.additionalTextEdits) {
                break;
            }
        }
        return newIndex;
    };
    SuggestAlternatives.prototype.next = function () {
        this._move(true);
    };
    SuggestAlternatives.prototype.prev = function () {
        this._move(false);
    };
    SuggestAlternatives.prototype._move = function (fwd) {
        if (!this._model) {
            // nothing to reason about
            return;
        }
        try {
            this._ignore = true;
            this._index = SuggestAlternatives._moveIndex(fwd, this._model, this._index);
            this._acceptNext({ index: this._index, item: this._model.items[this._index], model: this._model });
        }
        finally {
            this._ignore = false;
        }
    };
    SuggestAlternatives.OtherSuggestions = new RawContextKey('hasOtherSuggestions', false);
    SuggestAlternatives = __decorate$h([
        __param$g(1, IContextKeyService)
    ], SuggestAlternatives);
    return SuggestAlternatives;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Combined filters
/**
 * @returns A filter which combines the provided set
 * of filters with an or. The *first* filters that
 * matches defined the return value of the returned
 * filter.
 */
function or() {
    var filter = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        filter[_i] = arguments[_i];
    }
    return function (word, wordToMatchAgainst) {
        for (var i = 0, len = filter.length; i < len; i++) {
            var match = filter[i](word, wordToMatchAgainst);
            if (match) {
                return match;
            }
        }
        return null;
    };
}
var matchesPrefix = _matchesPrefix.bind(undefined, true);
function _matchesPrefix(ignoreCase, word, wordToMatchAgainst) {
    if (!wordToMatchAgainst || wordToMatchAgainst.length < word.length) {
        return null;
    }
    var matches;
    if (ignoreCase) {
        matches = startsWithIgnoreCase(wordToMatchAgainst, word);
    }
    else {
        matches = wordToMatchAgainst.indexOf(word) === 0;
    }
    if (!matches) {
        return null;
    }
    return word.length > 0 ? [{ start: 0, end: word.length }] : [];
}
// Contiguous Substring
function matchesContiguousSubString(word, wordToMatchAgainst) {
    var index = wordToMatchAgainst.toLowerCase().indexOf(word.toLowerCase());
    if (index === -1) {
        return null;
    }
    return [{ start: index, end: index + word.length }];
}
// Substring
function matchesSubString(word, wordToMatchAgainst) {
    return _matchesSubString(word.toLowerCase(), wordToMatchAgainst.toLowerCase(), 0, 0);
}
function _matchesSubString(word, wordToMatchAgainst, i, j) {
    if (i === word.length) {
        return [];
    }
    else if (j === wordToMatchAgainst.length) {
        return null;
    }
    else {
        if (word[i] === wordToMatchAgainst[j]) {
            var result = null;
            if (result = _matchesSubString(word, wordToMatchAgainst, i + 1, j + 1)) {
                return join$1({ start: j, end: j + 1 }, result);
            }
            return null;
        }
        return _matchesSubString(word, wordToMatchAgainst, i, j + 1);
    }
}
// CamelCase
function isLower(code) {
    return 97 /* a */ <= code && code <= 122 /* z */;
}
function isUpper(code) {
    return 65 /* A */ <= code && code <= 90 /* Z */;
}
function isNumber$1(code) {
    return 48 /* Digit0 */ <= code && code <= 57 /* Digit9 */;
}
function isWhitespace(code) {
    return (code === 32 /* Space */
        || code === 9 /* Tab */
        || code === 10 /* LineFeed */
        || code === 13 /* CarriageReturn */);
}
function isAlphanumeric(code) {
    return isLower(code) || isUpper(code) || isNumber$1(code);
}
function join$1(head, tail) {
    if (tail.length === 0) {
        tail = [head];
    }
    else if (head.end === tail[0].start) {
        tail[0].start = head.start;
    }
    else {
        tail.unshift(head);
    }
    return tail;
}
function nextAnchor(camelCaseWord, start) {
    for (var i = start; i < camelCaseWord.length; i++) {
        var c = camelCaseWord.charCodeAt(i);
        if (isUpper(c) || isNumber$1(c) || (i > 0 && !isAlphanumeric(camelCaseWord.charCodeAt(i - 1)))) {
            return i;
        }
    }
    return camelCaseWord.length;
}
function _matchesCamelCase(word, camelCaseWord, i, j) {
    if (i === word.length) {
        return [];
    }
    else if (j === camelCaseWord.length) {
        return null;
    }
    else if (word[i] !== camelCaseWord[j].toLowerCase()) {
        return null;
    }
    else {
        var result = null;
        var nextUpperIndex = j + 1;
        result = _matchesCamelCase(word, camelCaseWord, i + 1, j + 1);
        while (!result && (nextUpperIndex = nextAnchor(camelCaseWord, nextUpperIndex)) < camelCaseWord.length) {
            result = _matchesCamelCase(word, camelCaseWord, i + 1, nextUpperIndex);
            nextUpperIndex++;
        }
        return result === null ? null : join$1({ start: j, end: j + 1 }, result);
    }
}
// Heuristic to avoid computing camel case matcher for words that don't
// look like camelCaseWords.
function analyzeCamelCaseWord(word) {
    var upper = 0, lower = 0, alpha = 0, numeric = 0, code = 0;
    for (var i = 0; i < word.length; i++) {
        code = word.charCodeAt(i);
        if (isUpper(code)) {
            upper++;
        }
        if (isLower(code)) {
            lower++;
        }
        if (isAlphanumeric(code)) {
            alpha++;
        }
        if (isNumber$1(code)) {
            numeric++;
        }
    }
    var upperPercent = upper / word.length;
    var lowerPercent = lower / word.length;
    var alphaPercent = alpha / word.length;
    var numericPercent = numeric / word.length;
    return { upperPercent: upperPercent, lowerPercent: lowerPercent, alphaPercent: alphaPercent, numericPercent: numericPercent };
}
function isUpperCaseWord(analysis) {
    var upperPercent = analysis.upperPercent, lowerPercent = analysis.lowerPercent;
    return lowerPercent === 0 && upperPercent > 0.6;
}
function isCamelCaseWord(analysis) {
    var upperPercent = analysis.upperPercent, lowerPercent = analysis.lowerPercent, alphaPercent = analysis.alphaPercent, numericPercent = analysis.numericPercent;
    return lowerPercent > 0.2 && upperPercent < 0.8 && alphaPercent > 0.6 && numericPercent < 0.2;
}
// Heuristic to avoid computing camel case matcher for words that don't
// look like camel case patterns.
function isCamelCasePattern(word) {
    var upper = 0, lower = 0, code = 0, whitespace = 0;
    for (var i = 0; i < word.length; i++) {
        code = word.charCodeAt(i);
        if (isUpper(code)) {
            upper++;
        }
        if (isLower(code)) {
            lower++;
        }
        if (isWhitespace(code)) {
            whitespace++;
        }
    }
    if ((upper === 0 || lower === 0) && whitespace === 0) {
        return word.length <= 30;
    }
    else {
        return upper <= 5;
    }
}
function matchesCamelCase(word, camelCaseWord) {
    if (!camelCaseWord) {
        return null;
    }
    camelCaseWord = camelCaseWord.trim();
    if (camelCaseWord.length === 0) {
        return null;
    }
    if (!isCamelCasePattern(word)) {
        return null;
    }
    if (camelCaseWord.length > 60) {
        return null;
    }
    var analysis = analyzeCamelCaseWord(camelCaseWord);
    if (!isCamelCaseWord(analysis)) {
        if (!isUpperCaseWord(analysis)) {
            return null;
        }
        camelCaseWord = camelCaseWord.toLowerCase();
    }
    var result = null;
    var i = 0;
    word = word.toLowerCase();
    while (i < camelCaseWord.length && (result = _matchesCamelCase(word, camelCaseWord, 0, i)) === null) {
        i = nextAnchor(camelCaseWord, i + 1);
    }
    return result;
}
// Fuzzy
var fuzzyContiguousFilter = or(matchesPrefix, matchesCamelCase, matchesContiguousSubString);
var fuzzySeparateFilter = or(matchesPrefix, matchesCamelCase, matchesSubString);
var fuzzyRegExpCache = new LRUCache(10000); // bounded to 10000 elements
function matchesFuzzy(word, wordToMatchAgainst, enableSeparateSubstringMatching) {
    if (enableSeparateSubstringMatching === void 0) { enableSeparateSubstringMatching = false; }
    if (typeof word !== 'string' || typeof wordToMatchAgainst !== 'string') {
        return null; // return early for invalid input
    }
    // Form RegExp for wildcard matches
    var regexp = fuzzyRegExpCache.get(word);
    if (!regexp) {
        regexp = new RegExp(convertSimple2RegExpPattern(word), 'i');
        fuzzyRegExpCache.set(word, regexp);
    }
    // RegExp Filter
    var match = regexp.exec(wordToMatchAgainst);
    if (match) {
        return [{ start: match.index, end: match.index + match[0].length }];
    }
    // Default Filter
    return enableSeparateSubstringMatching ? fuzzySeparateFilter(word, wordToMatchAgainst) : fuzzyContiguousFilter(word, wordToMatchAgainst);
}
function anyScore(pattern, word, patternMaxWhitespaceIgnore) {
    pattern = pattern.toLowerCase();
    word = word.toLowerCase();
    var matches = [];
    var idx = 0;
    for (var pos = 0; pos < pattern.length; ++pos) {
        var thisIdx = word.indexOf(pattern.charAt(pos), idx);
        if (thisIdx >= 0) {
            matches.push(thisIdx);
            idx = thisIdx + 1;
        }
    }
    return [matches.length, matches];
}
//#region --- fuzzyScore ---
function createMatches(offsetOrScore) {
    var ret = [];
    if (!offsetOrScore) {
        return ret;
    }
    var offsets;
    if (Array.isArray(offsetOrScore[1])) {
        offsets = offsetOrScore[1];
    }
    else {
        offsets = offsetOrScore;
    }
    var last;
    for (var _i = 0, offsets_1 = offsets; _i < offsets_1.length; _i++) {
        var pos = offsets_1[_i];
        if (last && last.end === pos) {
            last.end += 1;
        }
        else {
            last = { start: pos, end: pos + 1 };
            ret.push(last);
        }
    }
    return ret;
}
function initTable() {
    var table = [];
    var row = [0];
    for (var i = 1; i <= 100; i++) {
        row.push(-i);
    }
    for (var i = 0; i <= 100; i++) {
        var thisRow = row.slice(0);
        thisRow[0] = -i;
        table.push(thisRow);
    }
    return table;
}
var _table = initTable();
var _scores = initTable();
var _arrows = initTable();
function isSeparatorAtPos(value, index) {
    if (index < 0 || index >= value.length) {
        return false;
    }
    var code = value.charCodeAt(index);
    switch (code) {
        case 95 /* Underline */:
        case 45 /* Dash */:
        case 46 /* Period */:
        case 32 /* Space */:
        case 47 /* Slash */:
        case 92 /* Backslash */:
        case 39 /* SingleQuote */:
        case 34 /* DoubleQuote */:
        case 58 /* Colon */:
            return true;
        default:
            return false;
    }
}
function isWhitespaceAtPos(value, index) {
    if (index < 0 || index >= value.length) {
        return false;
    }
    var code = value.charCodeAt(index);
    switch (code) {
        case 32 /* Space */:
        case 9 /* Tab */:
            return true;
        default:
            return false;
    }
}
function fuzzyScore(pattern, lowPattern, patternPos, word, lowWord, wordPos, firstMatchCanBeWeak) {
    var patternLen = pattern.length > 100 ? 100 : pattern.length;
    var wordLen = word.length > 100 ? 100 : word.length;
    if (patternPos >= patternLen || wordPos >= wordLen || patternLen > wordLen) {
        return undefined;
    }
    // Run a simple check if the characters of pattern occur
    // (in order) at all in word. If that isn't the case we
    // stop because no match will be possible
    var patternStartPos = patternPos;
    var wordStartPos = wordPos;
    while (patternPos < patternLen && wordPos < wordLen) {
        if (lowPattern[patternPos] === lowWord[wordPos]) {
            patternPos += 1;
        }
        wordPos += 1;
    }
    if (patternPos !== patternLen) {
        return undefined;
    }
    patternPos = patternStartPos;
    wordPos = wordStartPos;
    // There will be a mach, fill in tables
    for (patternPos = patternStartPos + 1; patternPos <= patternLen; patternPos++) {
        for (wordPos = 1; wordPos <= wordLen; wordPos++) {
            var score = -1;
            var lowWordChar = lowWord[wordPos - 1];
            if (lowPattern[patternPos - 1] === lowWordChar) {
                if (wordPos === (patternPos - patternStartPos)) {
                    // common prefix: `foobar <-> foobaz`
                    if (pattern[patternPos - 1] === word[wordPos - 1]) {
                        score = 7;
                    }
                    else {
                        score = 5;
                    }
                }
                else if (lowWordChar !== word[wordPos - 1] && (wordPos === 1 || lowWord[wordPos - 2] === word[wordPos - 2])) {
                    // hitting upper-case: `foo <-> forOthers`
                    if (pattern[patternPos - 1] === word[wordPos - 1]) {
                        score = 7;
                    }
                    else {
                        score = 5;
                    }
                }
                else if (isSeparatorAtPos(lowWord, wordPos - 2) || isWhitespaceAtPos(lowWord, wordPos - 2)) {
                    // post separator: `foo <-> bar_foo`
                    score = 5;
                }
                else {
                    score = 1;
                }
            }
            _scores[patternPos][wordPos] = score;
            var diag = _table[patternPos - 1][wordPos - 1] + (score > 1 ? 1 : score);
            var top_1 = _table[patternPos - 1][wordPos] + -1;
            var left = _table[patternPos][wordPos - 1] + -1;
            if (left >= top_1) {
                // left or diag
                if (left > diag) {
                    _table[patternPos][wordPos] = left;
                    _arrows[patternPos][wordPos] = 4 /* Left */;
                }
                else if (left === diag) {
                    _table[patternPos][wordPos] = left;
                    _arrows[patternPos][wordPos] = 4 /* Left */ | 2 /* Diag */;
                }
                else {
                    _table[patternPos][wordPos] = diag;
                    _arrows[patternPos][wordPos] = 2 /* Diag */;
                }
            }
            else {
                // top or diag
                if (top_1 > diag) {
                    _table[patternPos][wordPos] = top_1;
                    _arrows[patternPos][wordPos] = 1 /* Top */;
                }
                else if (top_1 === diag) {
                    _table[patternPos][wordPos] = top_1;
                    _arrows[patternPos][wordPos] = 1 /* Top */ | 2 /* Diag */;
                }
                else {
                    _table[patternPos][wordPos] = diag;
                    _arrows[patternPos][wordPos] = 2 /* Diag */;
                }
            }
        }
    }
    // _bucket is an array of [PrefixArray] we use to keep
    // track of scores and matches. After calling `_findAllMatches`
    // the best match (if available) is the first item in the array
    _matchesCount = 0;
    _topScore = -100;
    _patternStartPos = patternStartPos;
    _firstMatchCanBeWeak = firstMatchCanBeWeak;
    _findAllMatches(patternLen, wordLen, patternLen === wordLen ? 1 : 0, new LazyArray(), false);
    if (_matchesCount === 0) {
        return undefined;
    }
    return [_topScore, _topMatch.toArray()];
}
var _matchesCount = 0;
var _topMatch;
var _topScore = 0;
var _patternStartPos = 0;
var _firstMatchCanBeWeak = false;
function _findAllMatches(patternPos, wordPos, total, matches, lastMatched) {
    if (_matchesCount >= 10 || total < -25) {
        // stop when having already 10 results, or
        // when a potential alignment as already 5 gaps
        return;
    }
    var simpleMatchCount = 0;
    while (patternPos > _patternStartPos && wordPos > 0) {
        var score = _scores[patternPos][wordPos];
        var arrow = _arrows[patternPos][wordPos];
        if (arrow === 4 /* Left */) {
            // left
            wordPos -= 1;
            if (lastMatched) {
                total -= 5; // new gap penalty
            }
            else if (!matches.isEmpty()) {
                total -= 1; // gap penalty after first match
            }
            lastMatched = false;
            simpleMatchCount = 0;
        }
        else if (arrow & 2 /* Diag */) {
            if (arrow & 4 /* Left */) {
                // left
                _findAllMatches(patternPos, wordPos - 1, !matches.isEmpty() ? total - 1 : total, // gap penalty after first match
                matches.slice(), lastMatched);
            }
            // diag
            total += score;
            patternPos -= 1;
            wordPos -= 1;
            matches.unshift(wordPos);
            lastMatched = true;
            // count simple matches and boost a row of
            // simple matches when they yield in a
            // strong match.
            if (score === 1) {
                simpleMatchCount += 1;
                if (patternPos === _patternStartPos && !_firstMatchCanBeWeak) {
                    // when the first match is a weak
                    // match we discard it
                    return undefined;
                }
            }
            else {
                // boost
                total += 1 + (simpleMatchCount * (score - 1));
                simpleMatchCount = 0;
            }
        }
        else {
            return undefined;
        }
    }
    total -= wordPos >= 3 ? 9 : wordPos * 3; // late start penalty
    // dynamically keep track of the current top score
    // and insert the current best score at head, the rest at tail
    _matchesCount += 1;
    if (total > _topScore) {
        _topScore = total;
        _topMatch = matches;
    }
}
var LazyArray = /** @class */ (function () {
    function LazyArray() {
    }
    LazyArray.prototype.isEmpty = function () {
        return !this._data && (!this._parent || this._parent.isEmpty());
    };
    LazyArray.prototype.unshift = function (n) {
        if (!this._data) {
            this._data = [n];
        }
        else {
            this._data.unshift(n);
        }
    };
    LazyArray.prototype.slice = function () {
        var ret = new LazyArray();
        ret._parent = this;
        ret._parentLen = this._data ? this._data.length : 0;
        return ret;
    };
    LazyArray.prototype.toArray = function () {
        if (!this._data) {
            return this._parent.toArray();
        }
        var bucket = [];
        var element = this;
        while (element) {
            if (element._parent && element._parent._data) {
                bucket.push(element._parent._data.slice(element._parent._data.length - element._parentLen));
            }
            element = element._parent;
        }
        return Array.prototype.concat.apply(this._data, bucket);
    };
    return LazyArray;
}());
//#endregion
//#region --- graceful ---
function fuzzyScoreGracefulAggressive(pattern, lowPattern, patternPos, word, lowWord, wordPos, firstMatchCanBeWeak) {
    return fuzzyScoreWithPermutations(pattern, lowPattern, patternPos, word, lowWord, wordPos, true, firstMatchCanBeWeak);
}
function fuzzyScoreWithPermutations(pattern, lowPattern, patternPos, word, lowWord, wordPos, aggressive, firstMatchCanBeWeak) {
    var top = fuzzyScore(pattern, lowPattern, patternPos, word, lowWord, wordPos, firstMatchCanBeWeak);
    if (top && !aggressive) {
        // when using the original pattern yield a result we`
        // return it unless we are aggressive and try to find
        // a better alignment, e.g. `cno` -> `^co^ns^ole` or `^c^o^nsole`.
        return top;
    }
    if (pattern.length >= 3) {
        // When the pattern is long enough then try a few (max 7)
        // permutations of the pattern to find a better match. The
        // permutations only swap neighbouring characters, e.g
        // `cnoso` becomes `conso`, `cnsoo`, `cnoos`.
        var tries = Math.min(7, pattern.length - 1);
        for (var movingPatternPos = patternPos + 1; movingPatternPos < tries; movingPatternPos++) {
            var newPattern = nextTypoPermutation(pattern, movingPatternPos);
            if (newPattern) {
                var candidate = fuzzyScore(newPattern, newPattern.toLowerCase(), patternPos, word, lowWord, wordPos, firstMatchCanBeWeak);
                if (candidate) {
                    candidate[0] -= 3; // permutation penalty
                    if (!top || candidate[0] > top[0]) {
                        top = candidate;
                    }
                }
            }
        }
    }
    return top;
}
function nextTypoPermutation(pattern, patternPos) {
    if (patternPos + 1 >= pattern.length) {
        return undefined;
    }
    var swap1 = pattern[patternPos];
    var swap2 = pattern[patternPos + 1];
    if (swap1 === swap2) {
        return undefined;
    }
    return pattern.slice(0, patternPos)
        + swap2
        + swap1
        + pattern.slice(patternPos + 2);
}
//#endregion

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CompletionModel = /** @class */ (function () {
    function CompletionModel(items, column, lineContext, wordDistance, options) {
        if (options === void 0) { options = EDITOR_DEFAULTS.contribInfo.suggest; }
        this._snippetCompareFn = CompletionModel._compareCompletionItems;
        this._items = items;
        this._column = column;
        this._wordDistance = wordDistance;
        this._options = options;
        this._refilterKind = 1 /* All */;
        this._lineContext = lineContext;
        if (options.snippets === 'top') {
            this._snippetCompareFn = CompletionModel._compareCompletionItemsSnippetsUp;
        }
        else if (options.snippets === 'bottom') {
            this._snippetCompareFn = CompletionModel._compareCompletionItemsSnippetsDown;
        }
    }
    CompletionModel.prototype.dispose = function () {
        var seen = new Set();
        for (var _i = 0, _a = this._items; _i < _a.length; _i++) {
            var container = _a[_i].container;
            if (!seen.has(container)) {
                seen.add(container);
                if (isDisposable(container)) {
                    container.dispose();
                }
            }
        }
    };
    Object.defineProperty(CompletionModel.prototype, "lineContext", {
        get: function () {
            return this._lineContext;
        },
        set: function (value) {
            if (this._lineContext.leadingLineContent !== value.leadingLineContent
                || this._lineContext.characterCountDelta !== value.characterCountDelta) {
                this._refilterKind = this._lineContext.characterCountDelta < value.characterCountDelta && this._filteredItems ? 2 /* Incr */ : 1 /* All */;
                this._lineContext = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CompletionModel.prototype, "items", {
        get: function () {
            this._ensureCachedState();
            return this._filteredItems;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CompletionModel.prototype, "incomplete", {
        get: function () {
            this._ensureCachedState();
            return this._isIncomplete;
        },
        enumerable: true,
        configurable: true
    });
    CompletionModel.prototype.adopt = function (except) {
        var res = new Array();
        for (var i = 0; i < this._items.length;) {
            if (!except.has(this._items[i].support)) {
                res.push(this._items[i]);
                // unordered removed
                this._items[i] = this._items[this._items.length - 1];
                this._items.pop();
            }
            else {
                // continue with next item
                i++;
            }
        }
        this._refilterKind = 1 /* All */;
        return res;
    };
    Object.defineProperty(CompletionModel.prototype, "stats", {
        get: function () {
            this._ensureCachedState();
            return this._stats;
        },
        enumerable: true,
        configurable: true
    });
    CompletionModel.prototype._ensureCachedState = function () {
        if (this._refilterKind !== 0 /* Nothing */) {
            this._createCachedState();
        }
    };
    CompletionModel.prototype._createCachedState = function () {
        this._isIncomplete = new Set();
        this._stats = { suggestionCount: 0, snippetCount: 0, textCount: 0 };
        var _a = this._lineContext, leadingLineContent = _a.leadingLineContent, characterCountDelta = _a.characterCountDelta;
        var word = '';
        var wordLow = '';
        // incrementally filter less
        var source = this._refilterKind === 1 /* All */ ? this._items : this._filteredItems;
        var target = [];
        // picks a score function based on the number of
        // items that we have to score/filter and based on the
        // user-configuration
        var scoreFn = (!this._options.filterGraceful || source.length > 2000) ? fuzzyScore : fuzzyScoreGracefulAggressive;
        for (var i = 0; i < source.length; i++) {
            var item = source[i];
            var suggestion = item.suggestion, container = item.container;
            // make sure _labelLow, _filterTextLow, _sortTextLow exist
            ensureLowerCaseVariants(suggestion);
            // collect those supports that signaled having
            // an incomplete result
            if (container.incomplete) {
                this._isIncomplete.add(item.support);
            }
            // 'word' is that remainder of the current line that we
            // filter and score against. In theory each suggestion uses a
            // different word, but in practice not - that's why we cache
            var overwriteBefore = item.position.column - suggestion.range.startColumn;
            var wordLen = overwriteBefore + characterCountDelta - (item.position.column - this._column);
            if (word.length !== wordLen) {
                word = wordLen === 0 ? '' : leadingLineContent.slice(-wordLen);
                wordLow = word.toLowerCase();
            }
            // remember the word against which this item was
            // scored
            item.word = word;
            if (wordLen === 0) {
                // when there is nothing to score against, don't
                // event try to do. Use a const rank and rely on
                // the fallback-sort using the initial sort order.
                // use a score of `-100` because that is out of the
                // bound of values `fuzzyScore` will return
                item.score = -100;
                item.matches = undefined;
            }
            else {
                // skip word characters that are whitespace until
                // we have hit the replace range (overwriteBefore)
                var wordPos = 0;
                while (wordPos < overwriteBefore) {
                    var ch = word.charCodeAt(wordPos);
                    if (ch === 32 /* Space */ || ch === 9 /* Tab */) {
                        wordPos += 1;
                    }
                    else {
                        break;
                    }
                }
                if (wordPos >= wordLen) {
                    // the wordPos at which scoring starts is the whole word
                    // and therefore the same rules as not having a word apply
                    item.score = -100;
                    item.matches = [];
                }
                else if (typeof suggestion.filterText === 'string') {
                    // when there is a `filterText` it must match the `word`.
                    // if it matches we check with the label to compute highlights
                    // and if that doesn't yield a result we have no highlights,
                    // despite having the match
                    var match = scoreFn(word, wordLow, wordPos, suggestion.filterText, suggestion._filterTextLow, 0, false);
                    if (!match) {
                        continue;
                    }
                    item.score = match[0];
                    item.matches = (fuzzyScore(word, wordLow, 0, suggestion.label, suggestion._labelLow, 0, true) || anyScore(word, suggestion.label))[1];
                }
                else {
                    // by default match `word` against the `label`
                    var match = scoreFn(word, wordLow, wordPos, suggestion.label, suggestion._labelLow, 0, false);
                    if (match) {
                        item.score = match[0];
                        item.matches = match[1];
                    }
                    else {
                        continue;
                    }
                }
            }
            item.idx = i;
            item.distance = this._wordDistance.distance(item.position, suggestion);
            target.push(item);
            // update stats
            this._stats.suggestionCount++;
            switch (suggestion.kind) {
                case 25 /* Snippet */:
                    this._stats.snippetCount++;
                    break;
                case 18 /* Text */:
                    this._stats.textCount++;
                    break;
            }
        }
        this._filteredItems = target.sort(this._snippetCompareFn);
        this._refilterKind = 0 /* Nothing */;
    };
    CompletionModel._compareCompletionItems = function (a, b) {
        if (a.score > b.score) {
            return -1;
        }
        else if (a.score < b.score) {
            return 1;
        }
        else if (a.distance < b.distance) {
            return -1;
        }
        else if (a.distance > b.distance) {
            return 1;
        }
        else if (a.idx < b.idx) {
            return -1;
        }
        else if (a.idx > b.idx) {
            return 1;
        }
        else {
            return 0;
        }
    };
    CompletionModel._compareCompletionItemsSnippetsDown = function (a, b) {
        if (a.suggestion.kind !== b.suggestion.kind) {
            if (a.suggestion.kind === 25 /* Snippet */) {
                return 1;
            }
            else if (b.suggestion.kind === 25 /* Snippet */) {
                return -1;
            }
        }
        return CompletionModel._compareCompletionItems(a, b);
    };
    CompletionModel._compareCompletionItemsSnippetsUp = function (a, b) {
        if (a.suggestion.kind !== b.suggestion.kind) {
            if (a.suggestion.kind === 25 /* Snippet */) {
                return -1;
            }
            else if (b.suggestion.kind === 25 /* Snippet */) {
                return 1;
            }
        }
        return CompletionModel._compareCompletionItems(a, b);
    };
    return CompletionModel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1F = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var WordDistance = /** @class */ (function () {
    function WordDistance() {
    }
    WordDistance.create = function (service, editor) {
        if (!editor.getConfiguration().contribInfo.suggest.localityBonus) {
            return Promise.resolve(WordDistance.None);
        }
        if (!editor.hasModel()) {
            return Promise.resolve(WordDistance.None);
        }
        var model = editor.getModel();
        var position = editor.getPosition();
        if (!service.canComputeWordRanges(model.uri)) {
            return Promise.resolve(WordDistance.None);
        }
        // use token tree ranges
        var node = find(build(model), position);
        var ranges = [];
        while (node) {
            if (!node.range.isEmpty()) {
                ranges.push(node.range);
            }
            if (node.end.lineNumber - node.start.lineNumber >= 100) {
                break;
            }
            node = node.parent;
        }
        ranges.reverse();
        if (ranges.length === 0) {
            return Promise.resolve(WordDistance.None);
        }
        return service.computeWordRanges(model.uri, ranges[0]).then(function (wordRanges) {
            return new /** @class */ (function (_super) {
                __extends$1F(class_1, _super);
                function class_1() {
                    return _super !== null && _super.apply(this, arguments) || this;
                }
                class_1.prototype.distance = function (anchor, suggestion) {
                    if (!wordRanges || !position.equals(editor.getPosition())) {
                        return 0;
                    }
                    if (suggestion.kind === 17 /* Keyword */) {
                        return 2 << 20;
                    }
                    var word = suggestion.label;
                    var wordLines = wordRanges[word];
                    if (isFalsyOrEmpty(wordLines)) {
                        return 2 << 20;
                    }
                    var idx = binarySearch(wordLines, Range.fromPositions(anchor), Range.compareRangesUsingStarts);
                    var bestWordRange = idx >= 0 ? wordLines[idx] : wordLines[Math.max(0, ~idx - 1)];
                    var blockDistance = ranges.length;
                    for (var _i = 0, ranges_1 = ranges; _i < ranges_1.length; _i++) {
                        var range$$1 = ranges_1[_i];
                        if (!Range.containsRange(range$$1, bestWordRange)) {
                            break;
                        }
                        blockDistance -= 1;
                    }
                    return blockDistance;
                };
                return class_1;
            }(WordDistance));
        });
    };
    WordDistance.None = new /** @class */ (function (_super) {
        __extends$1F(class_2, _super);
        function class_2() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_2.prototype.distance = function () { return 0; };
        return class_2;
    }(WordDistance));
    return WordDistance;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var LineContext$1 = /** @class */ (function () {
    function LineContext$$1(model, position, auto, shy) {
        this.leadingLineContent = model.getLineContent(position.lineNumber).substr(0, position.column - 1);
        this.leadingWord = model.getWordUntilPosition(position);
        this.lineNumber = position.lineNumber;
        this.column = position.column;
        this.auto = auto;
        this.shy = shy;
    }
    LineContext$$1.shouldAutoTrigger = function (editor) {
        var model = editor.getModel();
        if (!model) {
            return false;
        }
        var pos = editor.getPosition();
        model.tokenizeIfCheap(pos.lineNumber);
        var word = model.getWordAtPosition(pos);
        if (!word) {
            return false;
        }
        if (word.endColumn !== pos.column) {
            return false;
        }
        if (!isNaN(Number(word.word))) {
            return false;
        }
        return true;
    };
    return LineContext$$1;
}());
var SuggestModel = /** @class */ (function () {
    function SuggestModel(_editor, _editorWorker) {
        var _this = this;
        this._editor = _editor;
        this._editorWorker = _editorWorker;
        this._toDispose = [];
        this._triggerQuickSuggest = new TimeoutTimer();
        this._triggerRefilter = new TimeoutTimer();
        this._state = 0 /* Idle */;
        this._onDidCancel = new Emitter();
        this._onDidTrigger = new Emitter();
        this._onDidSuggest = new Emitter();
        this.onDidCancel = this._onDidCancel.event;
        this.onDidTrigger = this._onDidTrigger.event;
        this.onDidSuggest = this._onDidSuggest.event;
        this._completionModel = null;
        this._context = null;
        this._currentSelection = this._editor.getSelection() || new Selection(1, 1, 1, 1);
        // wire up various listeners
        this._toDispose.push(this._editor.onDidChangeModel(function () {
            _this._updateTriggerCharacters();
            _this.cancel();
        }));
        this._toDispose.push(this._editor.onDidChangeModelLanguage(function () {
            _this._updateTriggerCharacters();
            _this.cancel();
        }));
        this._toDispose.push(this._editor.onDidChangeConfiguration(function () {
            _this._updateTriggerCharacters();
            _this._updateQuickSuggest();
        }));
        this._toDispose.push(CompletionProviderRegistry.onDidChange(function () {
            _this._updateTriggerCharacters();
            _this._updateActiveSuggestSession();
        }));
        this._toDispose.push(this._editor.onDidChangeCursorSelection(function (e) {
            _this._onCursorChange(e);
        }));
        var editorIsComposing = false;
        this._toDispose.push(this._editor.onCompositionStart(function () {
            editorIsComposing = true;
        }));
        this._toDispose.push(this._editor.onCompositionEnd(function () {
            // refilter when composition ends
            editorIsComposing = false;
            _this._refilterCompletionItems();
        }));
        this._toDispose.push(this._editor.onDidChangeModelContent(function () {
            // only filter completions when the editor isn't
            // composing a character, e.g. ¨ + u makes ü but just
            // ¨ cannot be used for filtering
            if (!editorIsComposing) {
                _this._refilterCompletionItems();
            }
        }));
        this._updateTriggerCharacters();
        this._updateQuickSuggest();
    }
    SuggestModel.prototype.dispose = function () {
        dispose([this._onDidCancel, this._onDidSuggest, this._onDidTrigger, this._triggerCharacterListener, this._triggerQuickSuggest, this._triggerRefilter]);
        this._toDispose = dispose(this._toDispose);
        dispose(this._completionModel);
        this.cancel();
    };
    // --- handle configuration & precondition changes
    SuggestModel.prototype._updateQuickSuggest = function () {
        this._quickSuggestDelay = this._editor.getConfiguration().contribInfo.quickSuggestionsDelay;
        if (isNaN(this._quickSuggestDelay) || (!this._quickSuggestDelay && this._quickSuggestDelay !== 0) || this._quickSuggestDelay < 0) {
            this._quickSuggestDelay = 10;
        }
    };
    SuggestModel.prototype._updateTriggerCharacters = function () {
        var _this = this;
        dispose(this._triggerCharacterListener);
        if (this._editor.getConfiguration().readOnly
            || !this._editor.getModel()
            || !this._editor.getConfiguration().contribInfo.suggestOnTriggerCharacters) {
            return;
        }
        var supportsByTriggerCharacter = Object.create(null);
        for (var _i = 0, _a = CompletionProviderRegistry.all(this._editor.getModel()); _i < _a.length; _i++) {
            var support = _a[_i];
            if (isFalsyOrEmpty(support.triggerCharacters)) {
                continue;
            }
            for (var _b = 0, _c = support.triggerCharacters; _b < _c.length; _b++) {
                var ch = _c[_b];
                var set = supportsByTriggerCharacter[ch];
                if (!set) {
                    set = supportsByTriggerCharacter[ch] = new Set();
                    set.add(getSnippetSuggestSupport());
                }
                set.add(support);
            }
        }
        this._triggerCharacterListener = this._editor.onDidType(function (text) {
            var lastChar = text.charAt(text.length - 1);
            var supports = supportsByTriggerCharacter[lastChar];
            if (supports) {
                // keep existing items that where not computed by the
                // supports/providers that want to trigger now
                var items = _this._completionModel ? _this._completionModel.adopt(supports) : undefined;
                _this.trigger({ auto: true, triggerCharacter: lastChar }, Boolean(_this._completionModel), values(supports), items);
            }
        });
    };
    Object.defineProperty(SuggestModel.prototype, "state", {
        // --- trigger/retrigger/cancel suggest
        get: function () {
            return this._state;
        },
        enumerable: true,
        configurable: true
    });
    SuggestModel.prototype.cancel = function (retrigger) {
        if (retrigger === void 0) { retrigger = false; }
        this._triggerRefilter.cancel();
        if (this._triggerQuickSuggest) {
            this._triggerQuickSuggest.cancel();
        }
        if (this._requestToken) {
            this._requestToken.cancel();
        }
        this._state = 0 /* Idle */;
        dispose(this._completionModel);
        this._completionModel = null;
        this._context = null;
        this._onDidCancel.fire({ retrigger: retrigger });
    };
    SuggestModel.prototype._updateActiveSuggestSession = function () {
        if (this._state !== 0 /* Idle */) {
            if (!CompletionProviderRegistry.has(this._editor.getModel())) {
                this.cancel();
            }
            else {
                this.trigger({ auto: this._state === 2 /* Auto */ }, true);
            }
        }
    };
    SuggestModel.prototype._onCursorChange = function (e) {
        var _this = this;
        var prevSelection = this._currentSelection;
        this._currentSelection = this._editor.getSelection();
        if (!e.selection.isEmpty()
            || e.reason !== 0 /* NotSet */
            || (e.source !== 'keyboard' && e.source !== 'deleteLeft')) {
            // Early exit if nothing needs to be done!
            // Leave some form of early exit check here if you wish to continue being a cursor position change listener ;)
            if (this._state !== 0 /* Idle */) {
                this.cancel();
            }
            return;
        }
        if (!CompletionProviderRegistry.has(this._editor.getModel())) {
            return;
        }
        var model = this._editor.getModel();
        if (!model) {
            return;
        }
        if (this._state === 0 /* Idle */) {
            if (this._editor.getConfiguration().contribInfo.quickSuggestions === false) {
                // not enabled
                return;
            }
            if (!prevSelection.containsRange(this._currentSelection) && !prevSelection.getEndPosition().isBeforeOrEqual(this._currentSelection.getPosition())) {
                // cursor didn't move RIGHT
                return;
            }
            if (this._editor.getConfiguration().contribInfo.suggest.snippetsPreventQuickSuggestions && SnippetController2.get(this._editor).isInSnippet()) {
                // no quick suggestion when in snippet mode
                return;
            }
            this.cancel();
            this._triggerQuickSuggest.cancelAndSet(function () {
                if (!LineContext$1.shouldAutoTrigger(_this._editor)) {
                    return;
                }
                var model = _this._editor.getModel();
                var pos = _this._editor.getPosition();
                if (!model) {
                    return;
                }
                // validate enabled now
                var quickSuggestions = _this._editor.getConfiguration().contribInfo.quickSuggestions;
                if (quickSuggestions === false) {
                    return;
                }
                else if (quickSuggestions === true) ;
                else {
                    // Check the type of the token that triggered this
                    model.tokenizeIfCheap(pos.lineNumber);
                    var lineTokens = model.getLineTokens(pos.lineNumber);
                    var tokenType = lineTokens.getStandardTokenType(lineTokens.findTokenIndexAtOffset(Math.max(pos.column - 1 - 1, 0)));
                    var inValidScope = quickSuggestions.other && tokenType === 0 /* Other */
                        || quickSuggestions.comments && tokenType === 1 /* Comment */
                        || quickSuggestions.strings && tokenType === 2 /* String */;
                    if (!inValidScope) {
                        return;
                    }
                }
                // we made it till here -> trigger now
                _this.trigger({ auto: true });
            }, this._quickSuggestDelay);
        }
    };
    SuggestModel.prototype._refilterCompletionItems = function () {
        var _this = this;
        if (this._state === 0 /* Idle */) {
            return;
        }
        var model = this._editor.getModel();
        if (model) {
            // refine active suggestion
            this._triggerRefilter.cancelAndSet(function () {
                var position = _this._editor.getPosition();
                var ctx = new LineContext$1(model, position, _this._state === 2 /* Auto */, false);
                _this._onNewContext(ctx);
            }, 25);
        }
    };
    SuggestModel.prototype.trigger = function (context, retrigger, onlyFrom, existingItems) {
        var _this = this;
        if (retrigger === void 0) { retrigger = false; }
        var model = this._editor.getModel();
        if (!model) {
            return;
        }
        var auto = context.auto;
        var ctx = new LineContext$1(model, this._editor.getPosition(), auto, context.shy);
        // Cancel previous requests, change state & update UI
        this.cancel(retrigger);
        this._state = auto ? 2 /* Auto */ : 1 /* Manual */;
        this._onDidTrigger.fire({ auto: auto, shy: context.shy });
        // Capture context when request was sent
        this._context = ctx;
        // Build context for request
        var suggestCtx;
        if (context.triggerCharacter) {
            suggestCtx = {
                triggerKind: 1 /* TriggerCharacter */,
                triggerCharacter: context.triggerCharacter
            };
        }
        else if (onlyFrom && onlyFrom.length) {
            suggestCtx = { triggerKind: 2 /* TriggerForIncompleteCompletions */ };
        }
        else {
            suggestCtx = { triggerKind: 0 /* Invoke */ };
        }
        this._requestToken = new CancellationTokenSource();
        // TODO: Remove this workaround - https://github.com/Microsoft/vscode/issues/61917
        // let wordDistance = Promise.resolve().then(() => WordDistance.create(this._editorWorker, this._editor));
        var wordDistance = WordDistance.create(this._editorWorker, this._editor);
        var items = provideSuggestionItems(model, this._editor.getPosition(), this._editor.getConfiguration().contribInfo.suggest.snippets, onlyFrom, suggestCtx, this._requestToken.token);
        Promise.all([items, wordDistance]).then(function (_a) {
            var items = _a[0], wordDistance = _a[1];
            _this._requestToken.dispose();
            if (_this._state === 0 /* Idle */) {
                return;
            }
            var model = _this._editor.getModel();
            if (!model) {
                return;
            }
            if (!isFalsyOrEmpty(existingItems)) {
                var cmpFn = getSuggestionComparator(_this._editor.getConfiguration().contribInfo.suggest.snippets);
                items = items.concat(existingItems).sort(cmpFn);
            }
            var ctx = new LineContext$1(model, _this._editor.getPosition(), auto, context.shy);
            dispose(_this._completionModel);
            _this._completionModel = new CompletionModel(items, _this._context.column, {
                leadingLineContent: ctx.leadingLineContent,
                characterCountDelta: _this._context ? ctx.column - _this._context.column : 0
            }, wordDistance, _this._editor.getConfiguration().contribInfo.suggest);
            _this._onNewContext(ctx);
        }).catch(onUnexpectedError);
    };
    SuggestModel.prototype._onNewContext = function (ctx) {
        if (!this._context) {
            // happens when 24x7 IntelliSense is enabled and still in its delay
            return;
        }
        if (ctx.lineNumber !== this._context.lineNumber) {
            // e.g. happens when pressing Enter while IntelliSense is computed
            this.cancel();
            return;
        }
        if (ctx.leadingWord.startColumn < this._context.leadingWord.startColumn) {
            // happens when the current word gets outdented
            this.cancel();
            return;
        }
        if (ctx.column < this._context.column) {
            // typed -> moved cursor LEFT -> retrigger if still on a word
            if (ctx.leadingWord.word) {
                this.trigger({ auto: this._context.auto }, true);
            }
            else {
                this.cancel();
            }
            return;
        }
        if (!this._completionModel) {
            // happens when IntelliSense is not yet computed
            return;
        }
        if (ctx.column > this._context.column && this._completionModel.incomplete.size > 0 && ctx.leadingWord.word.length !== 0) {
            // typed -> moved cursor RIGHT & incomple model & still on a word -> retrigger
            var incomplete = this._completionModel.incomplete;
            var adopted = this._completionModel.adopt(incomplete);
            this.trigger({ auto: this._state === 2 /* Auto */ }, true, values(incomplete), adopted);
        }
        else {
            // typed -> moved cursor RIGHT -> update UI
            var oldLineContext = this._completionModel.lineContext;
            var isFrozen = false;
            this._completionModel.lineContext = {
                leadingLineContent: ctx.leadingLineContent,
                characterCountDelta: ctx.column - this._context.column
            };
            if (this._completionModel.items.length === 0) {
                if (LineContext$1.shouldAutoTrigger(this._editor) && this._context.leadingWord.endColumn < ctx.leadingWord.startColumn) {
                    // retrigger when heading into a new word
                    this.trigger({ auto: this._context.auto }, true);
                    return;
                }
                if (!this._context.auto) {
                    // freeze when IntelliSense was manually requested
                    this._completionModel.lineContext = oldLineContext;
                    isFrozen = this._completionModel.items.length > 0;
                    if (isFrozen && ctx.leadingWord.word.length === 0) {
                        // there were results before but now there aren't
                        // and also we are not on a word anymore -> cancel
                        this.cancel();
                        return;
                    }
                }
                else {
                    // nothing left
                    this.cancel();
                    return;
                }
            }
            this._onDidSuggest.fire({
                completionModel: this._completionModel,
                auto: this._context.auto,
                shy: this._context.shy,
                isFrozen: isFrozen,
            });
        }
    };
    return SuggestModel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Range$1;
(function (Range) {
    /**
     * Returns the intersection between two ranges as a range itself.
     * Returns `{ start: 0, end: 0 }` if the intersection is empty.
     */
    function intersect(one, other) {
        if (one.start >= other.end || other.start >= one.end) {
            return { start: 0, end: 0 };
        }
        var start = Math.max(one.start, other.start);
        var end = Math.min(one.end, other.end);
        if (end - start <= 0) {
            return { start: 0, end: 0 };
        }
        return { start: start, end: end };
    }
    Range.intersect = intersect;
    function isEmpty(range) {
        return range.end - range.start <= 0;
    }
    Range.isEmpty = isEmpty;
    function intersects(one, other) {
        return !isEmpty(intersect(one, other));
    }
    Range.intersects = intersects;
    function relativeComplement(one, other) {
        var result = [];
        var first = { start: one.start, end: Math.min(other.start, one.end) };
        var second = { start: Math.max(other.end, one.start), end: one.end };
        if (!isEmpty(first)) {
            result.push(first);
        }
        if (!isEmpty(second)) {
            result.push(second);
        }
        return result;
    }
    Range.relativeComplement = relativeComplement;
})(Range$1 || (Range$1 = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Returns the intersection between a ranged group and a range.
 * Returns `[]` if the intersection is empty.
 */
function groupIntersect(range, groups) {
    var result = [];
    for (var _i = 0, groups_1 = groups; _i < groups_1.length; _i++) {
        var r = groups_1[_i];
        if (range.start >= r.range.end) {
            continue;
        }
        if (range.end < r.range.start) {
            break;
        }
        var intersection = Range$1.intersect(range, r.range);
        if (Range$1.isEmpty(intersection)) {
            continue;
        }
        result.push({
            range: intersection,
            size: r.size
        });
    }
    return result;
}
/**
 * Shifts a range by that `much`.
 */
function shift(_a, much) {
    var start = _a.start, end = _a.end;
    return { start: start + much, end: end + much };
}
/**
 * Consolidates a collection of ranged groups.
 *
 * Consolidation is the process of merging consecutive ranged groups
 * that share the same `size`.
 */
function consolidate(groups) {
    var result = [];
    var previousGroup = null;
    for (var _i = 0, groups_2 = groups; _i < groups_2.length; _i++) {
        var group = groups_2[_i];
        var start = group.range.start;
        var end = group.range.end;
        var size = group.size;
        if (previousGroup && size === previousGroup.size) {
            previousGroup.range.end = end;
            continue;
        }
        previousGroup = { range: { start: start, end: end }, size: size };
        result.push(previousGroup);
    }
    return result;
}
/**
 * Concatenates several collections of ranged groups into a single
 * collection.
 */
function concat() {
    var groups = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        groups[_i] = arguments[_i];
    }
    return consolidate(groups.reduce(function (r, g) { return r.concat(g); }, []));
}
var RangeMap = /** @class */ (function () {
    function RangeMap() {
        this.groups = [];
        this._size = 0;
    }
    RangeMap.prototype.splice = function (index, deleteCount, items) {
        if (items === void 0) { items = []; }
        var diff = items.length - deleteCount;
        var before = groupIntersect({ start: 0, end: index }, this.groups);
        var after = groupIntersect({ start: index + deleteCount, end: Number.POSITIVE_INFINITY }, this.groups)
            .map(function (g) { return ({ range: shift(g.range, diff), size: g.size }); });
        var middle = items.map(function (item, i) { return ({
            range: { start: index + i, end: index + i + 1 },
            size: item.size
        }); });
        this.groups = concat(before, middle, after);
        this._size = this.groups.reduce(function (t, g) { return t + (g.size * (g.range.end - g.range.start)); }, 0);
    };
    Object.defineProperty(RangeMap.prototype, "count", {
        /**
         * Returns the number of items in the range map.
         */
        get: function () {
            var len = this.groups.length;
            if (!len) {
                return 0;
            }
            return this.groups[len - 1].range.end;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RangeMap.prototype, "size", {
        /**
         * Returns the sum of the sizes of all items in the range map.
         */
        get: function () {
            return this._size;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns the index of the item at the given position.
     */
    RangeMap.prototype.indexAt = function (position) {
        if (position < 0) {
            return -1;
        }
        var index = 0;
        var size = 0;
        for (var _i = 0, _a = this.groups; _i < _a.length; _i++) {
            var group = _a[_i];
            var count = group.range.end - group.range.start;
            var newSize = size + (count * group.size);
            if (position < newSize) {
                return index + Math.floor((position - size) / group.size);
            }
            index += count;
            size = newSize;
        }
        return index;
    };
    /**
     * Returns the index of the item right after the item at the
     * index of the given position.
     */
    RangeMap.prototype.indexAfter = function (position) {
        return Math.min(this.indexAt(position) + 1, this.count);
    };
    /**
     * Returns the start position of the item at the given index.
     */
    RangeMap.prototype.positionAt = function (index) {
        if (index < 0) {
            return -1;
        }
        var position = 0;
        var count = 0;
        for (var _i = 0, _a = this.groups; _i < _a.length; _i++) {
            var group = _a[_i];
            var groupCount = group.range.end - group.range.start;
            var newCount = count + groupCount;
            if (index < newCount) {
                return position + ((index - count) * group.size);
            }
            position += groupCount * group.size;
            count = newCount;
        }
        return -1;
    };
    RangeMap.prototype.dispose = function () {
        this.groups = null; // StrictNullOverride: nulling out ok in dispose
    };
    return RangeMap;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function removeFromParent(element) {
    try {
        if (element.parentElement) {
            element.parentElement.removeChild(element);
        }
    }
    catch (e) {
        // this will throw if this happens due to a blur event, nasty business
    }
}
var RowCache = /** @class */ (function () {
    function RowCache(renderers) {
        this.renderers = renderers;
        this.cache = new Map();
    }
    /**
     * Returns a row either by creating a new one or reusing
     * a previously released row which shares the same templateId.
     */
    RowCache.prototype.alloc = function (templateId) {
        var result = this.getTemplateCache(templateId).pop();
        if (!result) {
            var domNode = $('.monaco-list-row');
            var renderer = this.renderers.get(templateId);
            var templateData = renderer.renderTemplate(domNode);
            result = { domNode: domNode, templateId: templateId, templateData: templateData };
        }
        return result;
    };
    /**
     * Releases the row for eventual reuse.
     */
    RowCache.prototype.release = function (row) {
        if (!row) {
            return;
        }
        this.releaseRow(row);
    };
    RowCache.prototype.releaseRow = function (row) {
        var domNode = row.domNode, templateId = row.templateId;
        if (domNode) {
            removeClass(domNode, 'scrolling');
            removeFromParent(domNode);
        }
        var cache = this.getTemplateCache(templateId);
        cache.push(row);
    };
    RowCache.prototype.getTemplateCache = function (templateId) {
        var result = this.cache.get(templateId);
        if (!result) {
            result = [];
            this.cache.set(templateId, result);
        }
        return result;
    };
    RowCache.prototype.garbageCollect = function () {
        var _this = this;
        if (!this.renderers) {
            return;
        }
        this.cache.forEach(function (cachedRows, templateId) {
            for (var _i = 0, cachedRows_1 = cachedRows; _i < cachedRows_1.length; _i++) {
                var cachedRow = cachedRows_1[_i];
                var renderer = _this.renderers.get(templateId);
                renderer.disposeTemplate(cachedRow.templateData);
                cachedRow.domNode = null;
                cachedRow.templateData = null;
            }
        });
        this.cache.clear();
    };
    RowCache.prototype.dispose = function () {
        this.garbageCollect();
        this.cache.clear();
        this.renderers = null; // StrictNullOverride: nulling out ok in dispose
    };
    return RowCache;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$i = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
function canUseTranslate3d() {
    if (isFirefox) {
        return false;
    }
    if (getZoomLevel() !== 0) {
        return false;
    }
    return true;
}
var DefaultOptions = {
    useShadows: true,
    verticalScrollMode: 1 /* Auto */,
    setRowLineHeight: true
};
var ListView = /** @class */ (function () {
    function ListView(container, virtualDelegate, renderers, options) {
        if (options === void 0) { options = DefaultOptions; }
        this.virtualDelegate = virtualDelegate;
        this.renderers = new Map();
        this.didRequestScrollableElementUpdate = false;
        this.splicing = false;
        this.items = [];
        this.itemId = 0;
        this.rangeMap = new RangeMap();
        for (var _i = 0, renderers_1 = renderers; _i < renderers_1.length; _i++) {
            var renderer = renderers_1[_i];
            this.renderers.set(renderer.templateId, renderer);
        }
        this.cache = new RowCache(this.renderers);
        this.lastRenderTop = 0;
        this.lastRenderHeight = 0;
        this._domNode = document.createElement('div');
        this._domNode.className = 'monaco-list';
        this.rowsContainer = document.createElement('div');
        this.rowsContainer.className = 'monaco-list-rows';
        Gesture.addTarget(this.rowsContainer);
        this.scrollableElement = new ScrollableElement(this.rowsContainer, {
            alwaysConsumeMouseWheel: true,
            horizontal: 2 /* Hidden */,
            vertical: getOrDefault(options, function (o) { return o.verticalScrollMode; }, DefaultOptions.verticalScrollMode),
            useShadows: getOrDefault(options, function (o) { return o.useShadows; }, DefaultOptions.useShadows)
        });
        this._domNode.appendChild(this.scrollableElement.getDomNode());
        container.appendChild(this._domNode);
        this.disposables = [this.rangeMap, this.gesture, this.scrollableElement, this.cache];
        this.scrollableElement.onScroll(this.onScroll, this, this.disposables);
        domEvent(this.rowsContainer, EventType$1.Change)(this.onTouchChange, this, this.disposables);
        // Prevent the monaco-scrollable-element from scrolling
        // https://github.com/Microsoft/vscode/issues/44181
        domEvent(this.scrollableElement.getDomNode(), 'scroll')(function (e) { return e.target.scrollTop = 0; }, null, this.disposables);
        var onDragOver = mapEvent(domEvent(this.rowsContainer, 'dragover'), function (e) { return new DragMouseEvent(e); });
        onDragOver(this.onDragOver, this, this.disposables);
        this.setRowLineHeight = getOrDefault(options, function (o) { return o.setRowLineHeight; }, DefaultOptions.setRowLineHeight);
        this.layout();
    }
    Object.defineProperty(ListView.prototype, "domNode", {
        get: function () {
            return this._domNode;
        },
        enumerable: true,
        configurable: true
    });
    ListView.prototype.splice = function (start, deleteCount, elements) {
        if (elements === void 0) { elements = []; }
        if (this.splicing) {
            throw new Error('Can\'t run recursive splices.');
        }
        this.splicing = true;
        try {
            return this._splice(start, deleteCount, elements);
        }
        finally {
            this.splicing = false;
        }
    };
    ListView.prototype._splice = function (start, deleteCount, elements) {
        var _this = this;
        if (elements === void 0) { elements = []; }
        var _a;
        var previousRenderRange = this.getRenderRange(this.lastRenderTop, this.lastRenderHeight);
        var deleteRange = { start: start, end: start + deleteCount };
        var removeRange = Range$1.intersect(previousRenderRange, deleteRange);
        for (var i = removeRange.start; i < removeRange.end; i++) {
            this.removeItemFromDOM(i);
        }
        var previousRestRange = { start: start + deleteCount, end: this.items.length };
        var previousRenderedRestRange = Range$1.intersect(previousRestRange, previousRenderRange);
        var previousUnrenderedRestRanges = Range$1.relativeComplement(previousRestRange, previousRenderRange);
        var inserted = elements.map(function (element) { return ({
            id: String(_this.itemId++),
            element: element,
            size: _this.virtualDelegate.getHeight(element),
            templateId: _this.virtualDelegate.getTemplateId(element),
            row: null
        }); });
        var deleted;
        // TODO@joao: improve this optimization to catch even more cases
        if (start === 0 && deleteCount >= this.items.length) {
            this.rangeMap = new RangeMap();
            this.rangeMap.splice(0, 0, inserted);
            this.items = inserted;
            deleted = [];
        }
        else {
            this.rangeMap.splice(start, deleteCount, inserted);
            deleted = (_a = this.items).splice.apply(_a, [start, deleteCount].concat(inserted));
        }
        var delta = elements.length - deleteCount;
        var renderRange = this.getRenderRange(this.lastRenderTop, this.lastRenderHeight);
        var renderedRestRange = shift(previousRenderedRestRange, delta);
        var updateRange = Range$1.intersect(renderRange, renderedRestRange);
        for (var i = updateRange.start; i < updateRange.end; i++) {
            this.updateItemInDOM(this.items[i], i);
        }
        var removeRanges = Range$1.relativeComplement(renderedRestRange, renderRange);
        for (var r = 0; r < removeRanges.length; r++) {
            var removeRange_1 = removeRanges[r];
            for (var i = removeRange_1.start; i < removeRange_1.end; i++) {
                this.removeItemFromDOM(i);
            }
        }
        var unrenderedRestRanges = previousUnrenderedRestRanges.map(function (r) { return shift(r, delta); });
        var elementsRange = { start: start, end: start + elements.length };
        var insertRanges = [elementsRange].concat(unrenderedRestRanges).map(function (r) { return Range$1.intersect(renderRange, r); });
        var beforeElement = this.getNextToLastElement(insertRanges);
        for (var r = 0; r < insertRanges.length; r++) {
            var insertRange = insertRanges[r];
            for (var i = insertRange.start; i < insertRange.end; i++) {
                this.insertItemInDOM(i, beforeElement);
            }
        }
        this.scrollHeight = this.getContentHeight();
        this.rowsContainer.style.height = this.scrollHeight + "px";
        if (!this.didRequestScrollableElementUpdate) {
            scheduleAtNextAnimationFrame(function () {
                _this.scrollableElement.setScrollDimensions({ scrollHeight: _this.scrollHeight });
                _this.didRequestScrollableElementUpdate = false;
            });
            this.didRequestScrollableElementUpdate = true;
        }
        return deleted.map(function (i) { return i.element; });
    };
    Object.defineProperty(ListView.prototype, "length", {
        get: function () {
            return this.items.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListView.prototype, "renderHeight", {
        get: function () {
            var scrollDimensions = this.scrollableElement.getScrollDimensions();
            return scrollDimensions.height;
        },
        enumerable: true,
        configurable: true
    });
    ListView.prototype.element = function (index) {
        return this.items[index].element;
    };
    ListView.prototype.domElement = function (index) {
        var row = this.items[index].row;
        return row && row.domNode;
    };
    ListView.prototype.elementHeight = function (index) {
        return this.items[index].size;
    };
    ListView.prototype.elementTop = function (index) {
        return this.rangeMap.positionAt(index);
    };
    ListView.prototype.indexAt = function (position) {
        return this.rangeMap.indexAt(position);
    };
    ListView.prototype.indexAfter = function (position) {
        return this.rangeMap.indexAfter(position);
    };
    ListView.prototype.layout = function (height) {
        this.scrollableElement.setScrollDimensions({
            height: height || getContentHeight(this._domNode)
        });
    };
    // Render
    ListView.prototype.render = function (renderTop, renderHeight) {
        var previousRenderRange = this.getRenderRange(this.lastRenderTop, this.lastRenderHeight);
        var renderRange = this.getRenderRange(renderTop, renderHeight);
        var rangesToInsert = Range$1.relativeComplement(renderRange, previousRenderRange);
        var rangesToRemove = Range$1.relativeComplement(previousRenderRange, renderRange);
        var beforeElement = this.getNextToLastElement(rangesToInsert);
        for (var _i = 0, rangesToInsert_1 = rangesToInsert; _i < rangesToInsert_1.length; _i++) {
            var range = rangesToInsert_1[_i];
            for (var i = range.start; i < range.end; i++) {
                this.insertItemInDOM(i, beforeElement);
            }
        }
        for (var _a = 0, rangesToRemove_1 = rangesToRemove; _a < rangesToRemove_1.length; _a++) {
            var range = rangesToRemove_1[_a];
            for (var i = range.start; i < range.end; i++) {
                this.removeItemFromDOM(i);
            }
        }
        if (canUseTranslate3d() && !isWindows /* Windows: translate3d breaks subpixel-antialias (ClearType) unless a background is defined */) {
            var transform = "translate3d(0px, -" + renderTop + "px, 0px)";
            this.rowsContainer.style.transform = transform;
            this.rowsContainer.style.webkitTransform = transform;
        }
        else {
            this.rowsContainer.style.top = "-" + renderTop + "px";
        }
        this.lastRenderTop = renderTop;
        this.lastRenderHeight = renderHeight;
    };
    // DOM operations
    ListView.prototype.insertItemInDOM = function (index, beforeElement) {
        var item = this.items[index];
        if (!item.row) {
            item.row = this.cache.alloc(item.templateId);
        }
        if (!item.row.domNode.parentElement) {
            if (beforeElement) {
                this.rowsContainer.insertBefore(item.row.domNode, beforeElement);
            }
            else {
                this.rowsContainer.appendChild(item.row.domNode);
            }
        }
        item.row.domNode.style.height = item.size + "px";
        if (this.setRowLineHeight) {
            item.row.domNode.style.lineHeight = item.size + "px";
        }
        this.updateItemInDOM(item, index);
        var renderer = this.renderers.get(item.templateId);
        renderer.renderElement(item.element, index, item.row.templateData);
    };
    ListView.prototype.updateItemInDOM = function (item, index) {
        item.row.domNode.style.top = this.elementTop(index) + "px";
        item.row.domNode.setAttribute('data-index', "" + index);
        item.row.domNode.setAttribute('data-last-element', index === this.length - 1 ? 'true' : 'false');
        item.row.domNode.setAttribute('aria-setsize', "" + this.length);
        item.row.domNode.setAttribute('aria-posinset', "" + (index + 1));
    };
    ListView.prototype.removeItemFromDOM = function (index) {
        var item = this.items[index];
        var renderer = this.renderers.get(item.templateId);
        if (renderer.disposeElement) {
            renderer.disposeElement(item.element, index, item.row.templateData);
        }
        this.cache.release(item.row);
        item.row = null;
    };
    ListView.prototype.getContentHeight = function () {
        return this.rangeMap.size;
    };
    ListView.prototype.getScrollTop = function () {
        var scrollPosition = this.scrollableElement.getScrollPosition();
        return scrollPosition.scrollTop;
    };
    ListView.prototype.setScrollTop = function (scrollTop) {
        this.scrollableElement.setScrollPosition({ scrollTop: scrollTop });
    };
    Object.defineProperty(ListView.prototype, "scrollTop", {
        get: function () {
            return this.getScrollTop();
        },
        set: function (scrollTop) {
            this.setScrollTop(scrollTop);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListView.prototype, "onMouseClick", {
        // Events
        get: function () {
            var _this = this;
            return filterEvent(mapEvent(domEvent(this.domNode, 'click'), function (e) { return _this.toMouseEvent(e); }), function (e) { return e.index >= 0; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListView.prototype, "onMouseDblClick", {
        get: function () {
            var _this = this;
            return filterEvent(mapEvent(domEvent(this.domNode, 'dblclick'), function (e) { return _this.toMouseEvent(e); }), function (e) { return e.index >= 0; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListView.prototype, "onMouseDown", {
        get: function () {
            var _this = this;
            return filterEvent(mapEvent(domEvent(this.domNode, 'mousedown'), function (e) { return _this.toMouseEvent(e); }), function (e) { return e.index >= 0; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListView.prototype, "onContextMenu", {
        get: function () {
            var _this = this;
            return filterEvent(mapEvent(domEvent(this.domNode, 'contextmenu'), function (e) { return _this.toMouseEvent(e); }), function (e) { return e.index >= 0; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListView.prototype, "onTouchStart", {
        get: function () {
            var _this = this;
            return filterEvent(mapEvent(domEvent(this.domNode, 'touchstart'), function (e) { return _this.toTouchEvent(e); }), function (e) { return e.index >= 0; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListView.prototype, "onTap", {
        get: function () {
            var _this = this;
            return filterEvent(mapEvent(domEvent(this.rowsContainer, EventType$1.Tap), function (e) { return _this.toGestureEvent(e); }), function (e) { return e.index >= 0; });
        },
        enumerable: true,
        configurable: true
    });
    ListView.prototype.toMouseEvent = function (browserEvent) {
        var index = this.getItemIndexFromEventTarget(browserEvent.target);
        var item = index < 0 ? undefined : this.items[index];
        var element = item && item.element;
        return { browserEvent: browserEvent, index: index, element: element };
    };
    ListView.prototype.toTouchEvent = function (browserEvent) {
        var index = this.getItemIndexFromEventTarget(browserEvent.target);
        var item = index < 0 ? undefined : this.items[index];
        var element = item && item.element;
        return { browserEvent: browserEvent, index: index, element: element };
    };
    ListView.prototype.toGestureEvent = function (browserEvent) {
        var index = this.getItemIndexFromEventTarget(browserEvent.initialTarget);
        var item = index < 0 ? undefined : this.items[index];
        var element = item && item.element;
        return { browserEvent: browserEvent, index: index, element: element };
    };
    ListView.prototype.onScroll = function (e) {
        try {
            this.render(e.scrollTop, e.height);
        }
        catch (err) {
            console.log('Got bad scroll event:', e);
            throw err;
        }
    };
    ListView.prototype.onTouchChange = function (event) {
        event.preventDefault();
        event.stopPropagation();
        this.scrollTop -= event.translationY;
    };
    ListView.prototype.onDragOver = function (event) {
        this.setupDragAndDropScrollInterval();
        this.dragAndDropMouseY = event.posy;
    };
    ListView.prototype.setupDragAndDropScrollInterval = function () {
        var _this = this;
        var viewTop = getTopLeftOffset(this._domNode).top;
        if (!this.dragAndDropScrollInterval) {
            this.dragAndDropScrollInterval = window.setInterval(function () {
                if (_this.dragAndDropMouseY === undefined) {
                    return;
                }
                var diff = _this.dragAndDropMouseY - viewTop;
                var scrollDiff = 0;
                var upperLimit = _this.renderHeight - 35;
                if (diff < 35) {
                    scrollDiff = Math.max(-14, 0.2 * (diff - 35));
                }
                else if (diff > upperLimit) {
                    scrollDiff = Math.min(14, 0.2 * (diff - upperLimit));
                }
                _this.scrollTop += scrollDiff;
            }, 10);
            this.cancelDragAndDropScrollTimeout();
            this.dragAndDropScrollTimeout = window.setTimeout(function () {
                _this.cancelDragAndDropScrollInterval();
                _this.dragAndDropScrollTimeout = null;
            }, 1000);
        }
    };
    ListView.prototype.cancelDragAndDropScrollInterval = function () {
        if (this.dragAndDropScrollInterval) {
            window.clearInterval(this.dragAndDropScrollInterval);
            this.dragAndDropScrollInterval = null;
        }
        this.cancelDragAndDropScrollTimeout();
    };
    ListView.prototype.cancelDragAndDropScrollTimeout = function () {
        if (this.dragAndDropScrollTimeout) {
            window.clearTimeout(this.dragAndDropScrollTimeout);
            this.dragAndDropScrollTimeout = null;
        }
    };
    // Util
    ListView.prototype.getItemIndexFromEventTarget = function (target) {
        while (target instanceof HTMLElement && target !== this.rowsContainer) {
            var element = target;
            var rawIndex = element.getAttribute('data-index');
            if (rawIndex) {
                var index = Number(rawIndex);
                if (!isNaN(index)) {
                    return index;
                }
            }
            target = element.parentElement;
        }
        return -1;
    };
    ListView.prototype.getRenderRange = function (renderTop, renderHeight) {
        return {
            start: this.rangeMap.indexAt(renderTop),
            end: this.rangeMap.indexAfter(renderTop + renderHeight - 1)
        };
    };
    ListView.prototype.getNextToLastElement = function (ranges) {
        var lastRange = ranges[ranges.length - 1];
        if (!lastRange) {
            return null;
        }
        var nextToLastItem = this.items[lastRange.end];
        if (!nextToLastItem) {
            return null;
        }
        if (!nextToLastItem.row) {
            return null;
        }
        return nextToLastItem.row.domNode;
    };
    // Dispose
    ListView.prototype.dispose = function () {
        if (this.items) {
            for (var _i = 0, _a = this.items; _i < _a.length; _i++) {
                var item = _a[_i];
                if (item.row) {
                    var renderer = this.renderers.get(item.row.templateId);
                    renderer.disposeTemplate(item.row.templateData);
                    item.row = null;
                }
            }
            this.items = null;
        }
        if (this._domNode && this._domNode.parentElement) {
            this._domNode.parentNode.removeChild(this._domNode);
            this._domNode = null;
        }
        this.disposables = dispose(this.disposables);
    };
    __decorate$i([
        memoize
    ], ListView.prototype, "onMouseClick", null);
    __decorate$i([
        memoize
    ], ListView.prototype, "onMouseDblClick", null);
    __decorate$i([
        memoize
    ], ListView.prototype, "onMouseDown", null);
    __decorate$i([
        memoize
    ], ListView.prototype, "onContextMenu", null);
    __decorate$i([
        memoize
    ], ListView.prototype, "onTouchStart", null);
    __decorate$i([
        memoize
    ], ListView.prototype, "onTap", null);
    return ListView;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CombinedSpliceable = /** @class */ (function () {
    function CombinedSpliceable(spliceables) {
        this.spliceables = spliceables;
    }
    CombinedSpliceable.prototype.splice = function (start, deleteCount, elements) {
        this.spliceables.forEach(function (s) { return s.splice(start, deleteCount, elements); });
    };
    return CombinedSpliceable;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function clamp$1(value, min, max) {
    return Math.min(Math.max(value, min), max);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1G = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$j = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var TraitRenderer = /** @class */ (function () {
    function TraitRenderer(trait) {
        this.trait = trait;
        this.renderedElements = [];
    }
    Object.defineProperty(TraitRenderer.prototype, "templateId", {
        get: function () {
            return "template:" + this.trait.trait;
        },
        enumerable: true,
        configurable: true
    });
    TraitRenderer.prototype.renderTemplate = function (container) {
        return container;
    };
    TraitRenderer.prototype.renderElement = function (element, index, templateData) {
        var renderedElementIndex = firstIndex(this.renderedElements, function (el) { return el.templateData === templateData; });
        if (renderedElementIndex >= 0) {
            var rendered = this.renderedElements[renderedElementIndex];
            this.trait.unrender(templateData);
            rendered.index = index;
        }
        else {
            var rendered = { index: index, templateData: templateData };
            this.renderedElements.push(rendered);
        }
        this.trait.renderIndex(index, templateData);
    };
    TraitRenderer.prototype.disposeElement = function () {
        // noop
    };
    TraitRenderer.prototype.splice = function (start, deleteCount, insertCount) {
        var rendered = [];
        for (var i = 0; i < this.renderedElements.length; i++) {
            var renderedElement = this.renderedElements[i];
            if (renderedElement.index < start) {
                rendered.push(renderedElement);
            }
            else if (renderedElement.index >= start + deleteCount) {
                rendered.push({
                    index: renderedElement.index + insertCount - deleteCount,
                    templateData: renderedElement.templateData
                });
            }
        }
        this.renderedElements = rendered;
    };
    TraitRenderer.prototype.renderIndexes = function (indexes) {
        for (var _i = 0, _a = this.renderedElements; _i < _a.length; _i++) {
            var _b = _a[_i], index = _b.index, templateData = _b.templateData;
            if (indexes.indexOf(index) > -1) {
                this.trait.renderIndex(index, templateData);
            }
        }
    };
    TraitRenderer.prototype.disposeTemplate = function (templateData) {
        var index = firstIndex(this.renderedElements, function (el) { return el.templateData === templateData; });
        if (index < 0) {
            return;
        }
        this.renderedElements.splice(index, 1);
    };
    return TraitRenderer;
}());
var Trait = /** @class */ (function () {
    function Trait(_trait) {
        this._trait = _trait;
        this._onChange = new Emitter();
        this.indexes = [];
    }
    Object.defineProperty(Trait.prototype, "onChange", {
        get: function () { return this._onChange.event; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trait.prototype, "trait", {
        get: function () { return this._trait; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trait.prototype, "renderer", {
        get: function () {
            return new TraitRenderer(this);
        },
        enumerable: true,
        configurable: true
    });
    Trait.prototype.splice = function (start, deleteCount, elements) {
        var diff = elements.length - deleteCount;
        var end = start + deleteCount;
        var indexes = this.indexes.filter(function (i) { return i < start; }).concat(elements.map(function (hasTrait, i) { return hasTrait ? i + start : -1; }).filter(function (i) { return i !== -1; }), this.indexes.filter(function (i) { return i >= end; }).map(function (i) { return i + diff; }));
        this.renderer.splice(start, deleteCount, elements.length);
        this.set(indexes);
    };
    Trait.prototype.renderIndex = function (index, container) {
        toggleClass(container, this._trait, this.contains(index));
    };
    Trait.prototype.unrender = function (container) {
        removeClass(container, this._trait);
    };
    /**
     * Sets the indexes which should have this trait.
     *
     * @param indexes Indexes which should have this trait.
     * @return The old indexes which had this trait.
     */
    Trait.prototype.set = function (indexes, browserEvent) {
        var result = this.indexes;
        this.indexes = indexes;
        var toRender = disjunction(result, indexes);
        this.renderer.renderIndexes(toRender);
        this._onChange.fire({ indexes: indexes, browserEvent: browserEvent });
        return result;
    };
    Trait.prototype.get = function () {
        return this.indexes;
    };
    Trait.prototype.contains = function (index) {
        return this.indexes.some(function (i) { return i === index; });
    };
    Trait.prototype.dispose = function () {
        this.indexes = null;
        this._onChange = dispose(this._onChange);
    };
    __decorate$j([
        memoize
    ], Trait.prototype, "renderer", null);
    return Trait;
}());
var FocusTrait = /** @class */ (function (_super) {
    __extends$1G(FocusTrait, _super);
    function FocusTrait(getDomId) {
        var _this = _super.call(this, 'focused') || this;
        _this.getDomId = getDomId;
        return _this;
    }
    FocusTrait.prototype.renderIndex = function (index, container) {
        _super.prototype.renderIndex.call(this, index, container);
        container.setAttribute('role', 'treeitem');
        container.setAttribute('id', this.getDomId(index));
        if (this.contains(index)) {
            container.setAttribute('aria-selected', 'true');
        }
        else {
            container.removeAttribute('aria-selected');
        }
    };
    return FocusTrait;
}(Trait));
/**
 * The TraitSpliceable is used as a util class to be able
 * to preserve traits across splice calls, given an identity
 * provider.
 */
var TraitSpliceable = /** @class */ (function () {
    function TraitSpliceable(trait, view, getId) {
        this.trait = trait;
        this.view = view;
        this.getId = getId;
    }
    TraitSpliceable.prototype.splice = function (start, deleteCount, elements) {
        var _this = this;
        if (!this.getId) {
            return this.trait.splice(start, deleteCount, elements.map(function (e) { return false; }));
        }
        var pastElementsWithTrait = this.trait.get().map(function (i) { return _this.getId(_this.view.element(i)); });
        var elementsWithTrait = elements.map(function (e) { return pastElementsWithTrait.indexOf(_this.getId(e)) > -1; });
        this.trait.splice(start, deleteCount, elementsWithTrait);
    };
    return TraitSpliceable;
}());
function isInputElement(e) {
    return e.tagName === 'INPUT' || e.tagName === 'TEXTAREA';
}
var KeyboardController = /** @class */ (function () {
    function KeyboardController(list, view, options) {
        this.list = list;
        this.view = view;
        var multipleSelectionSupport = !(options.multipleSelectionSupport === false);
        this.disposables = [];
        this.openController = options.openController || DefaultOpenController;
        var onKeyDown = chain(domEvent(view.domNode, 'keydown'))
            .filter(function (e) { return !isInputElement(e.target); })
            .map(function (e) { return new StandardKeyboardEvent(e); });
        onKeyDown.filter(function (e) { return e.keyCode === 3 /* Enter */; }).on(this.onEnter, this, this.disposables);
        onKeyDown.filter(function (e) { return e.keyCode === 16 /* UpArrow */; }).on(this.onUpArrow, this, this.disposables);
        onKeyDown.filter(function (e) { return e.keyCode === 18 /* DownArrow */; }).on(this.onDownArrow, this, this.disposables);
        onKeyDown.filter(function (e) { return e.keyCode === 11 /* PageUp */; }).on(this.onPageUpArrow, this, this.disposables);
        onKeyDown.filter(function (e) { return e.keyCode === 12 /* PageDown */; }).on(this.onPageDownArrow, this, this.disposables);
        onKeyDown.filter(function (e) { return e.keyCode === 9 /* Escape */; }).on(this.onEscape, this, this.disposables);
        if (multipleSelectionSupport) {
            onKeyDown.filter(function (e) { return (isMacintosh ? e.metaKey : e.ctrlKey) && e.keyCode === 31 /* KEY_A */; }).on(this.onCtrlA, this, this.disposables);
        }
    }
    KeyboardController.prototype.onEnter = function (e) {
        e.preventDefault();
        e.stopPropagation();
        this.list.setSelection(this.list.getFocus(), e.browserEvent);
        if (this.openController.shouldOpen(e.browserEvent)) {
            this.list.open(this.list.getFocus(), e.browserEvent);
        }
    };
    KeyboardController.prototype.onUpArrow = function (e) {
        e.preventDefault();
        e.stopPropagation();
        this.list.focusPrevious(1, false, e.browserEvent);
        this.list.reveal(this.list.getFocus()[0]);
        this.view.domNode.focus();
    };
    KeyboardController.prototype.onDownArrow = function (e) {
        e.preventDefault();
        e.stopPropagation();
        this.list.focusNext(1, false, e.browserEvent);
        this.list.reveal(this.list.getFocus()[0]);
        this.view.domNode.focus();
    };
    KeyboardController.prototype.onPageUpArrow = function (e) {
        e.preventDefault();
        e.stopPropagation();
        this.list.focusPreviousPage(e.browserEvent);
        this.list.reveal(this.list.getFocus()[0]);
        this.view.domNode.focus();
    };
    KeyboardController.prototype.onPageDownArrow = function (e) {
        e.preventDefault();
        e.stopPropagation();
        this.list.focusNextPage(e.browserEvent);
        this.list.reveal(this.list.getFocus()[0]);
        this.view.domNode.focus();
    };
    KeyboardController.prototype.onCtrlA = function (e) {
        e.preventDefault();
        e.stopPropagation();
        this.list.setSelection(range(this.list.length), e.browserEvent);
        this.view.domNode.focus();
    };
    KeyboardController.prototype.onEscape = function (e) {
        e.preventDefault();
        e.stopPropagation();
        this.list.setSelection([], e.browserEvent);
        this.view.domNode.focus();
    };
    KeyboardController.prototype.dispose = function () {
        this.disposables = dispose(this.disposables);
    };
    return KeyboardController;
}());
var DOMFocusController = /** @class */ (function () {
    function DOMFocusController(list, view) {
        this.list = list;
        this.view = view;
        this.disposables = [];
        this.disposables = [];
        var onKeyDown = chain(domEvent(view.domNode, 'keydown'))
            .filter(function (e) { return !isInputElement(e.target); })
            .map(function (e) { return new StandardKeyboardEvent(e); });
        onKeyDown.filter(function (e) { return e.keyCode === 2 /* Tab */ && !e.ctrlKey && !e.metaKey && !e.shiftKey && !e.altKey; })
            .on(this.onTab, this, this.disposables);
    }
    DOMFocusController.prototype.onTab = function (e) {
        if (e.target !== this.view.domNode) {
            return;
        }
        var focus = this.list.getFocus();
        if (focus.length === 0) {
            return;
        }
        var focusedDomElement = this.view.domElement(focus[0]);
        var tabIndexElement = focusedDomElement.querySelector('[tabIndex]');
        if (!tabIndexElement || !(tabIndexElement instanceof HTMLElement) || tabIndexElement.tabIndex === -1) {
            return;
        }
        var style = window.getComputedStyle(tabIndexElement);
        if (style.visibility === 'hidden' || style.display === 'none') {
            return;
        }
        e.preventDefault();
        e.stopPropagation();
        tabIndexElement.focus();
    };
    DOMFocusController.prototype.dispose = function () {
        this.disposables = dispose(this.disposables);
    };
    return DOMFocusController;
}());
function isSelectionSingleChangeEvent(event) {
    return isMacintosh ? event.browserEvent.metaKey : event.browserEvent.ctrlKey;
}
function isSelectionRangeChangeEvent(event) {
    return event.browserEvent.shiftKey;
}
function isMouseRightClick(event) {
    return event instanceof MouseEvent && event.button === 2;
}
var DefaultMultipleSelectionContoller = {
    isSelectionSingleChangeEvent: isSelectionSingleChangeEvent,
    isSelectionRangeChangeEvent: isSelectionRangeChangeEvent
};
var DefaultOpenController = {
    shouldOpen: function (event) {
        if (event instanceof MouseEvent) {
            return !isMouseRightClick(event);
        }
        return true;
    }
};
var MouseController = /** @class */ (function () {
    function MouseController(list, view, options) {
        if (options === void 0) { options = {}; }
        this.list = list;
        this.view = view;
        this.options = options;
        this.didJustPressContextMenuKey = false;
        this.disposables = [];
        this.multipleSelectionSupport = !(options.multipleSelectionSupport === false);
        if (this.multipleSelectionSupport) {
            this.multipleSelectionController = options.multipleSelectionController || DefaultMultipleSelectionContoller;
        }
        this.openController = options.openController || DefaultOpenController;
        view.onMouseDown(this.onMouseDown, this, this.disposables);
        view.onMouseClick(this.onPointer, this, this.disposables);
        view.onMouseDblClick(this.onDoubleClick, this, this.disposables);
        view.onTouchStart(this.onMouseDown, this, this.disposables);
        view.onTap(this.onPointer, this, this.disposables);
        Gesture.addTarget(view.domNode);
    }
    Object.defineProperty(MouseController.prototype, "onContextMenu", {
        get: function () {
            var _this = this;
            var fromKeydown = chain(domEvent(this.view.domNode, 'keydown'))
                .map(function (e) { return new StandardKeyboardEvent(e); })
                .filter(function (e) { return _this.didJustPressContextMenuKey = e.keyCode === 58 /* ContextMenu */ || (e.shiftKey && e.keyCode === 68 /* F10 */); })
                .filter(function (e) { e.preventDefault(); e.stopPropagation(); return false; })
                .map(function (event) {
                var index = _this.list.getFocus()[0];
                var element = _this.view.element(index);
                var anchor = _this.view.domElement(index);
                return { index: index, element: element, anchor: anchor, browserEvent: event.browserEvent };
            })
                .event;
            var fromKeyup = chain(domEvent(this.view.domNode, 'keyup'))
                .filter(function () {
                var didJustPressContextMenuKey = _this.didJustPressContextMenuKey;
                _this.didJustPressContextMenuKey = false;
                return didJustPressContextMenuKey;
            })
                .filter(function () { return _this.list.getFocus().length > 0; })
                .map(function (browserEvent) {
                var index = _this.list.getFocus()[0];
                var element = _this.view.element(index);
                var anchor = _this.view.domElement(index);
                return { index: index, element: element, anchor: anchor, browserEvent: browserEvent };
            })
                .filter(function (_a) {
                var anchor = _a.anchor;
                return !!anchor;
            })
                .event;
            var fromMouse = chain(this.view.onContextMenu)
                .filter(function () { return !_this.didJustPressContextMenuKey; })
                .map(function (_a) {
                var element = _a.element, index = _a.index, browserEvent = _a.browserEvent;
                return ({ element: element, index: index, anchor: { x: browserEvent.clientX + 1, y: browserEvent.clientY }, browserEvent: browserEvent });
            })
                .event;
            return anyEvent(fromKeydown, fromKeyup, fromMouse);
        },
        enumerable: true,
        configurable: true
    });
    MouseController.prototype.isSelectionSingleChangeEvent = function (event) {
        if (this.multipleSelectionController) {
            return this.multipleSelectionController.isSelectionSingleChangeEvent(event);
        }
        return isMacintosh ? event.browserEvent.metaKey : event.browserEvent.ctrlKey;
    };
    MouseController.prototype.isSelectionRangeChangeEvent = function (event) {
        if (this.multipleSelectionController) {
            return this.multipleSelectionController.isSelectionRangeChangeEvent(event);
        }
        return event.browserEvent.shiftKey;
    };
    MouseController.prototype.isSelectionChangeEvent = function (event) {
        return this.isSelectionSingleChangeEvent(event) || this.isSelectionRangeChangeEvent(event);
    };
    MouseController.prototype.onMouseDown = function (e) {
        if (this.options.focusOnMouseDown === false) {
            e.browserEvent.preventDefault();
            e.browserEvent.stopPropagation();
        }
        else if (document.activeElement !== e.browserEvent.target) {
            this.view.domNode.focus();
        }
        var reference = this.list.getFocus()[0];
        var selection = this.list.getSelection();
        reference = reference === undefined ? selection[0] : reference;
        if (this.multipleSelectionSupport && this.isSelectionRangeChangeEvent(e)) {
            return this.changeSelection(e, reference);
        }
        var focus = e.index;
        if (selection.every(function (s) { return s !== focus; })) {
            this.list.setFocus([focus], e.browserEvent);
        }
        if (this.multipleSelectionSupport && this.isSelectionChangeEvent(e)) {
            return this.changeSelection(e, reference);
        }
        if (this.options.selectOnMouseDown && !isMouseRightClick(e.browserEvent)) {
            this.list.setSelection([focus], e.browserEvent);
            if (this.openController.shouldOpen(e.browserEvent)) {
                this.list.open([focus], e.browserEvent);
            }
        }
    };
    MouseController.prototype.onPointer = function (e) {
        if (this.multipleSelectionSupport && this.isSelectionChangeEvent(e)) {
            return;
        }
        if (!this.options.selectOnMouseDown) {
            var focus_1 = this.list.getFocus();
            this.list.setSelection(focus_1, e.browserEvent);
            if (this.openController.shouldOpen(e.browserEvent)) {
                this.list.open(focus_1, e.browserEvent);
            }
        }
    };
    MouseController.prototype.onDoubleClick = function (e) {
        if (this.multipleSelectionSupport && this.isSelectionChangeEvent(e)) {
            return;
        }
        var focus = this.list.getFocus();
        this.list.setSelection(focus, e.browserEvent);
        this.list.pin(focus);
    };
    MouseController.prototype.changeSelection = function (e, reference) {
        var focus = e.index;
        if (this.isSelectionRangeChangeEvent(e) && reference !== undefined) {
            var min = Math.min(reference, focus);
            var max = Math.max(reference, focus);
            var rangeSelection = range(min, max + 1);
            var selection = this.list.getSelection();
            var contiguousRange = getContiguousRangeContaining(disjunction(selection, [reference]), reference);
            if (contiguousRange.length === 0) {
                return;
            }
            var newSelection = disjunction(rangeSelection, relativeComplement(selection, contiguousRange));
            this.list.setSelection(newSelection, e.browserEvent);
        }
        else if (this.isSelectionSingleChangeEvent(e)) {
            var selection = this.list.getSelection();
            var newSelection = selection.filter(function (i) { return i !== focus; });
            if (selection.length === newSelection.length) {
                this.list.setSelection(newSelection.concat([focus]), e.browserEvent);
            }
            else {
                this.list.setSelection(newSelection, e.browserEvent);
            }
        }
    };
    MouseController.prototype.dispose = function () {
        this.disposables = dispose(this.disposables);
    };
    __decorate$j([
        memoize
    ], MouseController.prototype, "onContextMenu", null);
    return MouseController;
}());
var DefaultStyleController = /** @class */ (function () {
    function DefaultStyleController(styleElement, selectorSuffix) {
        this.styleElement = styleElement;
        this.selectorSuffix = selectorSuffix;
    }
    DefaultStyleController.prototype.style = function (styles) {
        var suffix = this.selectorSuffix ? "." + this.selectorSuffix : '';
        var content = [];
        if (styles.listFocusBackground) {
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.focused { background-color: " + styles.listFocusBackground + "; }");
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.focused:hover { background-color: " + styles.listFocusBackground + "; }"); // overwrite :hover style in this case!
        }
        if (styles.listFocusForeground) {
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.focused { color: " + styles.listFocusForeground + "; }");
        }
        if (styles.listActiveSelectionBackground) {
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.selected { background-color: " + styles.listActiveSelectionBackground + "; }");
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.selected:hover { background-color: " + styles.listActiveSelectionBackground + "; }"); // overwrite :hover style in this case!
        }
        if (styles.listActiveSelectionForeground) {
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.selected { color: " + styles.listActiveSelectionForeground + "; }");
        }
        if (styles.listFocusAndSelectionBackground) {
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.selected.focused { background-color: " + styles.listFocusAndSelectionBackground + "; }");
        }
        if (styles.listFocusAndSelectionForeground) {
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.selected.focused { color: " + styles.listFocusAndSelectionForeground + "; }");
        }
        if (styles.listInactiveFocusBackground) {
            content.push(".monaco-list" + suffix + " .monaco-list-row.focused { background-color:  " + styles.listInactiveFocusBackground + "; }");
            content.push(".monaco-list" + suffix + " .monaco-list-row.focused:hover { background-color:  " + styles.listInactiveFocusBackground + "; }"); // overwrite :hover style in this case!
        }
        if (styles.listInactiveSelectionBackground) {
            content.push(".monaco-list" + suffix + " .monaco-list-row.selected { background-color:  " + styles.listInactiveSelectionBackground + "; }");
            content.push(".monaco-list" + suffix + " .monaco-list-row.selected:hover { background-color:  " + styles.listInactiveSelectionBackground + "; }"); // overwrite :hover style in this case!
        }
        if (styles.listInactiveSelectionForeground) {
            content.push(".monaco-list" + suffix + " .monaco-list-row.selected { color: " + styles.listInactiveSelectionForeground + "; }");
        }
        if (styles.listHoverBackground) {
            content.push(".monaco-list" + suffix + " .monaco-list-row:hover { background-color:  " + styles.listHoverBackground + "; }");
        }
        if (styles.listHoverForeground) {
            content.push(".monaco-list" + suffix + " .monaco-list-row:hover { color:  " + styles.listHoverForeground + "; }");
        }
        if (styles.listSelectionOutline) {
            content.push(".monaco-list" + suffix + " .monaco-list-row.selected { outline: 1px dotted " + styles.listSelectionOutline + "; outline-offset: -1px; }");
        }
        if (styles.listFocusOutline) {
            content.push(".monaco-list" + suffix + ":focus .monaco-list-row.focused { outline: 1px solid " + styles.listFocusOutline + "; outline-offset: -1px; }");
        }
        if (styles.listInactiveFocusOutline) {
            content.push(".monaco-list" + suffix + " .monaco-list-row.focused { outline: 1px dotted " + styles.listInactiveFocusOutline + "; outline-offset: -1px; }");
        }
        if (styles.listHoverOutline) {
            content.push(".monaco-list" + suffix + " .monaco-list-row:hover { outline: 1px dashed " + styles.listHoverOutline + "; outline-offset: -1px; }");
        }
        var newStyles = content.join('\n');
        if (newStyles !== this.styleElement.innerHTML) {
            this.styleElement.innerHTML = newStyles;
        }
    };
    return DefaultStyleController;
}());
var defaultStyles = {
    listFocusBackground: Color.fromHex('#073655'),
    listActiveSelectionBackground: Color.fromHex('#0E639C'),
    listActiveSelectionForeground: Color.fromHex('#FFFFFF'),
    listFocusAndSelectionBackground: Color.fromHex('#094771'),
    listFocusAndSelectionForeground: Color.fromHex('#FFFFFF'),
    listInactiveSelectionBackground: Color.fromHex('#3F3F46'),
    listHoverBackground: Color.fromHex('#2A2D2E'),
    listDropBackground: Color.fromHex('#383B3D')
};
var DefaultOptions$1 = {
    keyboardSupport: true,
    mouseSupport: true,
    multipleSelectionSupport: true
};
// TODO@Joao: move these utils into a SortedArray class
function getContiguousRangeContaining(range$$1, value) {
    var index = range$$1.indexOf(value);
    if (index === -1) {
        return [];
    }
    var result = [];
    var i = index - 1;
    while (i >= 0 && range$$1[i] === value - (index - i)) {
        result.push(range$$1[i--]);
    }
    result.reverse();
    i = index;
    while (i < range$$1.length && range$$1[i] === value + (i - index)) {
        result.push(range$$1[i++]);
    }
    return result;
}
/**
 * Given two sorted collections of numbers, returns the intersection
 * betweem them (OR).
 */
function disjunction(one, other) {
    var result = [];
    var i = 0, j = 0;
    while (i < one.length || j < other.length) {
        if (i >= one.length) {
            result.push(other[j++]);
        }
        else if (j >= other.length) {
            result.push(one[i++]);
        }
        else if (one[i] === other[j]) {
            result.push(one[i]);
            i++;
            j++;
            continue;
        }
        else if (one[i] < other[j]) {
            result.push(one[i++]);
        }
        else {
            result.push(other[j++]);
        }
    }
    return result;
}
/**
 * Given two sorted collections of numbers, returns the relative
 * complement between them (XOR).
 */
function relativeComplement(one, other) {
    var result = [];
    var i = 0, j = 0;
    while (i < one.length || j < other.length) {
        if (i >= one.length) {
            result.push(other[j++]);
        }
        else if (j >= other.length) {
            result.push(one[i++]);
        }
        else if (one[i] === other[j]) {
            i++;
            j++;
            continue;
        }
        else if (one[i] < other[j]) {
            result.push(one[i++]);
        }
        else {
            j++;
        }
    }
    return result;
}
var numericSort = function (a, b) { return a - b; };
var PipelineRenderer = /** @class */ (function () {
    function PipelineRenderer(_templateId, renderers) {
        this._templateId = _templateId;
        this.renderers = renderers;
    }
    Object.defineProperty(PipelineRenderer.prototype, "templateId", {
        get: function () {
            return this._templateId;
        },
        enumerable: true,
        configurable: true
    });
    PipelineRenderer.prototype.renderTemplate = function (container) {
        return this.renderers.map(function (r) { return r.renderTemplate(container); });
    };
    PipelineRenderer.prototype.renderElement = function (element, index, templateData) {
        var i = 0;
        for (var _i = 0, _a = this.renderers; _i < _a.length; _i++) {
            var renderer = _a[_i];
            renderer.renderElement(element, index, templateData[i++]);
        }
    };
    PipelineRenderer.prototype.disposeElement = function (element, index, templateData) {
        var i = 0;
        for (var _i = 0, _a = this.renderers; _i < _a.length; _i++) {
            var renderer = _a[_i];
            renderer.disposeElement(element, index, templateData[i++]);
        }
    };
    PipelineRenderer.prototype.disposeTemplate = function (templateData) {
        var i = 0;
        for (var _i = 0, _a = this.renderers; _i < _a.length; _i++) {
            var renderer = _a[_i];
            renderer.disposeTemplate(templateData[i++]);
        }
    };
    return PipelineRenderer;
}());
var AccessibiltyRenderer = /** @class */ (function () {
    function AccessibiltyRenderer(accessibilityProvider) {
        this.accessibilityProvider = accessibilityProvider;
        this.templateId = 'a18n';
    }
    AccessibiltyRenderer.prototype.renderTemplate = function (container) {
        return container;
    };
    AccessibiltyRenderer.prototype.renderElement = function (element, index, container) {
        var ariaLabel = this.accessibilityProvider.getAriaLabel(element);
        if (ariaLabel) {
            container.setAttribute('aria-label', ariaLabel);
        }
        else {
            container.removeAttribute('aria-label');
        }
    };
    AccessibiltyRenderer.prototype.disposeElement = function (element, index, container) {
        // noop
    };
    AccessibiltyRenderer.prototype.disposeTemplate = function (templateData) {
        // noop
    };
    return AccessibiltyRenderer;
}());
var List = /** @class */ (function () {
    function List(container, virtualDelegate, renderers, options) {
        if (options === void 0) { options = DefaultOptions$1; }
        var _this = this;
        this.idPrefix = "list_id_" + ++List.InstanceCount;
        this.eventBufferer = new EventBufferer();
        this.onContextMenu = Event.None;
        this._onOpen = new Emitter();
        this._onPin = new Emitter();
        this._onDidDispose = new Emitter();
        this.focus = new FocusTrait(function (i) { return _this.getElementDomId(i); });
        this.selection = new Trait('selected');
        mixin(options, defaultStyles, false);
        var baseRenderers = [this.focus.renderer, this.selection.renderer];
        if (options.accessibilityProvider) {
            baseRenderers.push(new AccessibiltyRenderer(options.accessibilityProvider));
        }
        renderers = renderers.map(function (r) { return new PipelineRenderer(r.templateId, baseRenderers.concat([r])); });
        this.view = new ListView(container, virtualDelegate, renderers, options);
        this.view.domNode.setAttribute('role', 'tree');
        addClass(this.view.domNode, this.idPrefix);
        this.view.domNode.tabIndex = 0;
        this.styleElement = createStyleSheet(this.view.domNode);
        this.styleController = options.styleController;
        if (!this.styleController) {
            this.styleController = new DefaultStyleController(this.styleElement, this.idPrefix);
        }
        this.spliceable = new CombinedSpliceable([
            new TraitSpliceable(this.focus, this.view, options.identityProvider),
            new TraitSpliceable(this.selection, this.view, options.identityProvider),
            this.view
        ]);
        this.disposables = [this.focus, this.selection, this.view, this._onDidDispose];
        this.onDidFocus = mapEvent(domEvent(this.view.domNode, 'focus', true), function () { return null; });
        this.onDidBlur = mapEvent(domEvent(this.view.domNode, 'blur', true), function () { return null; });
        this.disposables.push(new DOMFocusController(this, this.view));
        if (typeof options.keyboardSupport !== 'boolean' || options.keyboardSupport) {
            var controller = new KeyboardController(this, this.view, options);
            this.disposables.push(controller);
        }
        if (typeof options.mouseSupport !== 'boolean' || options.mouseSupport) {
            this.mouseController = new MouseController(this, this.view, options);
            this.disposables.push(this.mouseController);
            this.onContextMenu = this.mouseController.onContextMenu;
        }
        this.onFocusChange(this._onFocusChange, this, this.disposables);
        this.onSelectionChange(this._onSelectionChange, this, this.disposables);
        if (options.ariaLabel) {
            this.view.domNode.setAttribute('aria-label', localize('aria list', "{0}. Use the navigation keys to navigate.", options.ariaLabel));
        }
        this.style(options);
    }
    Object.defineProperty(List.prototype, "onFocusChange", {
        get: function () {
            var _this = this;
            return mapEvent(this.eventBufferer.wrapEvent(this.focus.onChange), function (e) { return _this.toListEvent(e); });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(List.prototype, "onSelectionChange", {
        get: function () {
            var _this = this;
            return mapEvent(this.eventBufferer.wrapEvent(this.selection.onChange), function (e) { return _this.toListEvent(e); });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(List.prototype, "onMouseClick", {
        get: function () { return this.view.onMouseClick; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(List.prototype, "onKeyDown", {
        get: function () { return domEvent(this.view.domNode, 'keydown'); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(List.prototype, "onDidDispose", {
        get: function () { return this._onDidDispose.event; },
        enumerable: true,
        configurable: true
    });
    List.prototype.splice = function (start, deleteCount, elements) {
        var _this = this;
        if (elements === void 0) { elements = []; }
        if (start < 0 || start > this.view.length) {
            throw new Error("Invalid start index: " + start);
        }
        if (deleteCount < 0) {
            throw new Error("Invalid delete count: " + deleteCount);
        }
        if (deleteCount === 0 && elements.length === 0) {
            return;
        }
        this.eventBufferer.bufferEvents(function () { return _this.spliceable.splice(start, deleteCount, elements); });
    };
    Object.defineProperty(List.prototype, "length", {
        get: function () {
            return this.view.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(List.prototype, "contentHeight", {
        get: function () {
            return this.view.getContentHeight();
        },
        enumerable: true,
        configurable: true
    });
    List.prototype.layout = function (height) {
        this.view.layout(height);
    };
    List.prototype.setSelection = function (indexes, browserEvent) {
        for (var _i = 0, indexes_1 = indexes; _i < indexes_1.length; _i++) {
            var index = indexes_1[_i];
            if (index < 0 || index >= this.length) {
                throw new Error("Invalid index " + index);
            }
        }
        indexes = indexes.sort(numericSort);
        this.selection.set(indexes, browserEvent);
    };
    List.prototype.getSelection = function () {
        return this.selection.get();
    };
    List.prototype.setFocus = function (indexes, browserEvent) {
        for (var _i = 0, indexes_2 = indexes; _i < indexes_2.length; _i++) {
            var index = indexes_2[_i];
            if (index < 0 || index >= this.length) {
                throw new Error("Invalid index " + index);
            }
        }
        indexes = indexes.sort(numericSort);
        this.focus.set(indexes, browserEvent);
    };
    List.prototype.focusNext = function (n, loop, browserEvent) {
        if (n === void 0) { n = 1; }
        if (loop === void 0) { loop = false; }
        if (this.length === 0) {
            return;
        }
        var focus = this.focus.get();
        var index = focus.length > 0 ? focus[0] + n : 0;
        this.setFocus(loop ? [index % this.length] : [Math.min(index, this.length - 1)], browserEvent);
    };
    List.prototype.focusPrevious = function (n, loop, browserEvent) {
        if (n === void 0) { n = 1; }
        if (loop === void 0) { loop = false; }
        if (this.length === 0) {
            return;
        }
        var focus = this.focus.get();
        var index = focus.length > 0 ? focus[0] - n : 0;
        if (loop && index < 0) {
            index = (this.length + (index % this.length)) % this.length;
        }
        this.setFocus([Math.max(index, 0)], browserEvent);
    };
    List.prototype.focusNextPage = function (browserEvent) {
        var _this = this;
        var lastPageIndex = this.view.indexAt(this.view.getScrollTop() + this.view.renderHeight);
        lastPageIndex = lastPageIndex === 0 ? 0 : lastPageIndex - 1;
        var lastPageElement = this.view.element(lastPageIndex);
        var currentlyFocusedElement = this.getFocusedElements()[0];
        if (currentlyFocusedElement !== lastPageElement) {
            this.setFocus([lastPageIndex], browserEvent);
        }
        else {
            var previousScrollTop = this.view.getScrollTop();
            this.view.setScrollTop(previousScrollTop + this.view.renderHeight - this.view.elementHeight(lastPageIndex));
            if (this.view.getScrollTop() !== previousScrollTop) {
                // Let the scroll event listener run
                setTimeout(function () { return _this.focusNextPage(browserEvent); }, 0);
            }
        }
    };
    List.prototype.focusPreviousPage = function (browserEvent) {
        var _this = this;
        var firstPageIndex;
        var scrollTop = this.view.getScrollTop();
        if (scrollTop === 0) {
            firstPageIndex = this.view.indexAt(scrollTop);
        }
        else {
            firstPageIndex = this.view.indexAfter(scrollTop - 1);
        }
        var firstPageElement = this.view.element(firstPageIndex);
        var currentlyFocusedElement = this.getFocusedElements()[0];
        if (currentlyFocusedElement !== firstPageElement) {
            this.setFocus([firstPageIndex], browserEvent);
        }
        else {
            var previousScrollTop = scrollTop;
            this.view.setScrollTop(scrollTop - this.view.renderHeight);
            if (this.view.getScrollTop() !== previousScrollTop) {
                // Let the scroll event listener run
                setTimeout(function () { return _this.focusPreviousPage(browserEvent); }, 0);
            }
        }
    };
    List.prototype.focusLast = function (browserEvent) {
        if (this.length === 0) {
            return;
        }
        this.setFocus([this.length - 1], browserEvent);
    };
    List.prototype.focusFirst = function (browserEvent) {
        if (this.length === 0) {
            return;
        }
        this.setFocus([0], browserEvent);
    };
    List.prototype.getFocus = function () {
        return this.focus.get();
    };
    List.prototype.getFocusedElements = function () {
        var _this = this;
        return this.getFocus().map(function (i) { return _this.view.element(i); });
    };
    List.prototype.reveal = function (index, relativeTop) {
        if (index < 0 || index >= this.length) {
            throw new Error("Invalid index " + index);
        }
        var scrollTop = this.view.getScrollTop();
        var elementTop = this.view.elementTop(index);
        var elementHeight = this.view.elementHeight(index);
        if (isNumber(relativeTop)) {
            // y = mx + b
            var m = elementHeight - this.view.renderHeight;
            this.view.setScrollTop(m * clamp$1(relativeTop, 0, 1) + elementTop);
        }
        else {
            var viewItemBottom = elementTop + elementHeight;
            var wrapperBottom = scrollTop + this.view.renderHeight;
            if (elementTop < scrollTop) {
                this.view.setScrollTop(elementTop);
            }
            else if (viewItemBottom >= wrapperBottom) {
                this.view.setScrollTop(viewItemBottom - this.view.renderHeight);
            }
        }
    };
    List.prototype.getElementDomId = function (index) {
        return this.idPrefix + "_" + index;
    };
    List.prototype.getHTMLElement = function () {
        return this.view.domNode;
    };
    List.prototype.open = function (indexes, browserEvent) {
        var _this = this;
        for (var _i = 0, indexes_3 = indexes; _i < indexes_3.length; _i++) {
            var index = indexes_3[_i];
            if (index < 0 || index >= this.length) {
                throw new Error("Invalid index " + index);
            }
        }
        this._onOpen.fire({ indexes: indexes, elements: indexes.map(function (i) { return _this.view.element(i); }), browserEvent: browserEvent });
    };
    List.prototype.pin = function (indexes) {
        for (var _i = 0, indexes_4 = indexes; _i < indexes_4.length; _i++) {
            var index = indexes_4[_i];
            if (index < 0 || index >= this.length) {
                throw new Error("Invalid index " + index);
            }
        }
        this._onPin.fire(indexes);
    };
    List.prototype.style = function (styles) {
        this.styleController.style(styles);
    };
    List.prototype.toListEvent = function (_a) {
        var _this = this;
        var indexes = _a.indexes, browserEvent = _a.browserEvent;
        return { indexes: indexes, elements: indexes.map(function (i) { return _this.view.element(i); }), browserEvent: browserEvent };
    };
    List.prototype._onFocusChange = function () {
        var focus = this.focus.get();
        if (focus.length > 0) {
            this.view.domNode.setAttribute('aria-activedescendant', this.getElementDomId(focus[0]));
        }
        else {
            this.view.domNode.removeAttribute('aria-activedescendant');
        }
        this.view.domNode.setAttribute('role', 'tree');
        toggleClass(this.view.domNode, 'element-focused', focus.length > 0);
    };
    List.prototype._onSelectionChange = function () {
        var selection = this.selection.get();
        toggleClass(this.view.domNode, 'selection-none', selection.length === 0);
        toggleClass(this.view.domNode, 'selection-single', selection.length === 1);
        toggleClass(this.view.domNode, 'selection-multiple', selection.length > 1);
    };
    List.prototype.dispose = function () {
        this._onDidDispose.fire();
        this.disposables = dispose(this.disposables);
        this._onOpen.dispose();
        this._onPin.dispose();
        this._onDidDispose.dispose();
    };
    List.InstanceCount = 0;
    __decorate$j([
        memoize
    ], List.prototype, "onFocusChange", null);
    __decorate$j([
        memoize
    ], List.prototype, "onSelectionChange", null);
    return List;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __assign$2 = (undefined && undefined.__assign) || function () {
    __assign$2 = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign$2.apply(this, arguments);
};
function computeStyles(theme, styleMap) {
    var styles = Object.create(null);
    for (var key in styleMap) {
        var value = styleMap[key];
        if (typeof value === 'string') {
            styles[key] = theme.getColor(value);
        }
        else if (typeof value === 'function') {
            styles[key] = value(theme);
        }
    }
    return styles;
}
function attachStyler(themeService, styleMap, widgetOrCallback) {
    function applyStyles(theme) {
        var styles = computeStyles(themeService.getTheme(), styleMap);
        if (typeof widgetOrCallback === 'function') {
            widgetOrCallback(styles);
        }
        else {
            widgetOrCallback.style(styles);
        }
    }
    applyStyles(themeService.getTheme());
    return themeService.onThemeChange(applyStyles);
}
function attachQuickOpenStyler(widget, themeService, style) {
    return attachStyler(themeService, {
        foreground: (style && style.foreground) || foreground,
        background: (style && style.background) || editorBackground,
        borderColor: style && style.borderColor || contrastBorder,
        widgetShadow: style && style.widgetShadow || widgetShadow,
        progressBarBackground: style && style.progressBarBackground || progressBarBackground,
        pickerGroupForeground: style && style.pickerGroupForeground || pickerGroupForeground,
        pickerGroupBorder: style && style.pickerGroupBorder || pickerGroupBorder,
        inputBackground: (style && style.inputBackground) || inputBackground,
        inputForeground: (style && style.inputForeground) || inputForeground,
        inputBorder: (style && style.inputBorder) || inputBorder,
        inputValidationInfoBorder: (style && style.inputValidationInfoBorder) || inputValidationInfoBorder,
        inputValidationInfoBackground: (style && style.inputValidationInfoBackground) || inputValidationInfoBackground,
        inputValidationInfoForeground: (style && style.inputValidationInfoForeground) || inputValidationInfoForeground,
        inputValidationWarningBorder: (style && style.inputValidationWarningBorder) || inputValidationWarningBorder,
        inputValidationWarningBackground: (style && style.inputValidationWarningBackground) || inputValidationWarningBackground,
        inputValidationWarningForeground: (style && style.inputValidationWarningForeground) || inputValidationWarningForeground,
        inputValidationErrorBorder: (style && style.inputValidationErrorBorder) || inputValidationErrorBorder,
        inputValidationErrorBackground: (style && style.inputValidationErrorBackground) || inputValidationErrorBackground,
        inputValidationErrorForeground: (style && style.inputValidationErrorForeground) || inputValidationErrorForeground,
        listFocusBackground: (style && style.listFocusBackground) || listFocusBackground,
        listFocusForeground: (style && style.listFocusForeground) || listFocusForeground,
        listActiveSelectionBackground: (style && style.listActiveSelectionBackground) || listActiveSelectionBackground,
        listActiveSelectionForeground: (style && style.listActiveSelectionForeground) || listActiveSelectionForeground,
        listFocusAndSelectionBackground: style && style.listFocusAndSelectionBackground || listActiveSelectionBackground,
        listFocusAndSelectionForeground: (style && style.listFocusAndSelectionForeground) || listActiveSelectionForeground,
        listInactiveSelectionBackground: (style && style.listInactiveSelectionBackground) || listInactiveSelectionBackground,
        listInactiveSelectionForeground: (style && style.listInactiveSelectionForeground) || listInactiveSelectionForeground,
        listInactiveFocusBackground: (style && style.listInactiveFocusBackground) || listInactiveFocusBackground,
        listHoverBackground: (style && style.listHoverBackground) || listHoverBackground,
        listHoverForeground: (style && style.listHoverForeground) || listHoverForeground,
        listDropBackground: (style && style.listDropBackground) || listDropBackground,
        listFocusOutline: (style && style.listFocusOutline) || activeContrastBorder,
        listSelectionOutline: (style && style.listSelectionOutline) || activeContrastBorder,
        listHoverOutline: (style && style.listHoverOutline) || activeContrastBorder
    }, widget);
}
function attachListStyler(widget, themeService, overrides) {
    return attachStyler(themeService, mixin(overrides || Object.create(null), defaultListStyles, false), widget);
}
var defaultListStyles = {
    listFocusBackground: listFocusBackground,
    listFocusForeground: listFocusForeground,
    listActiveSelectionBackground: listActiveSelectionBackground,
    listActiveSelectionForeground: listActiveSelectionForeground,
    listFocusAndSelectionBackground: listActiveSelectionBackground,
    listFocusAndSelectionForeground: listActiveSelectionForeground,
    listInactiveSelectionBackground: listInactiveSelectionBackground,
    listInactiveSelectionForeground: listInactiveSelectionForeground,
    listInactiveFocusBackground: listInactiveFocusBackground,
    listHoverBackground: listHoverBackground,
    listHoverForeground: listHoverForeground,
    listDropBackground: listDropBackground,
    listFocusOutline: activeContrastBorder,
    listSelectionOutline: activeContrastBorder,
    listHoverOutline: activeContrastBorder
};
var defaultMenuStyles = {
    shadowColor: widgetShadow,
    borderColor: menuBorder,
    foregroundColor: menuForeground,
    backgroundColor: menuBackground,
    selectionForegroundColor: menuSelectionForeground,
    selectionBackgroundColor: menuSelectionBackground,
    selectionBorderColor: menuSelectionBorder,
    separatorColor: menuSeparatorBackground
};
function attachMenuStyler(widget, themeService, style) {
    var styles = __assign$2({}, defaultMenuStyles, style);
    var fallback = {
        foregroundColor: !!styles.foregroundColor && !!themeService && !!themeService.getTheme().getColor(styles.foregroundColor) ? styles.foregroundColor : foreground
    };
    return attachStyler(themeService, __assign$2({}, styles, fallback), widget);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function renderOcticons(text) {
    return escape(text);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var HighlightedLabel = /** @class */ (function () {
    function HighlightedLabel(container, supportOcticons) {
        this.supportOcticons = supportOcticons;
        this.domNode = document.createElement('span');
        this.domNode.className = 'monaco-highlighted-label';
        this.didEverRender = false;
        container.appendChild(this.domNode);
    }
    Object.defineProperty(HighlightedLabel.prototype, "element", {
        get: function () {
            return this.domNode;
        },
        enumerable: true,
        configurable: true
    });
    HighlightedLabel.prototype.set = function (text, highlights, title, escapeNewLines) {
        if (highlights === void 0) { highlights = []; }
        if (title === void 0) { title = ''; }
        if (!text) {
            text = '';
        }
        if (escapeNewLines) {
            // adjusts highlights inplace
            text = HighlightedLabel.escapeNewLines(text, highlights);
        }
        if (this.didEverRender && this.text === text && this.title === title && equals$1(this.highlights, highlights)) {
            return;
        }
        if (!Array.isArray(highlights)) {
            highlights = [];
        }
        this.text = text;
        this.title = title;
        this.highlights = highlights;
        this.render();
    };
    HighlightedLabel.prototype.render = function () {
        clearNode(this.domNode);
        var htmlContent = [], highlight, pos = 0;
        for (var i = 0; i < this.highlights.length; i++) {
            highlight = this.highlights[i];
            if (highlight.end === highlight.start) {
                continue;
            }
            if (pos < highlight.start) {
                htmlContent.push('<span>');
                var substring_1 = this.text.substring(pos, highlight.start);
                htmlContent.push(this.supportOcticons ? renderOcticons(substring_1) : substring_1);
                htmlContent.push('</span>');
                pos = highlight.end;
            }
            htmlContent.push('<span class="highlight">');
            var substring = this.text.substring(highlight.start, highlight.end);
            htmlContent.push(this.supportOcticons ? renderOcticons(substring) : substring);
            htmlContent.push('</span>');
            pos = highlight.end;
        }
        if (pos < this.text.length) {
            htmlContent.push('<span>');
            var substring = this.text.substring(pos);
            htmlContent.push(this.supportOcticons ? renderOcticons(substring) : substring);
            htmlContent.push('</span>');
        }
        this.domNode.innerHTML = htmlContent.join('');
        this.domNode.title = this.title;
        this.didEverRender = true;
    };
    HighlightedLabel.prototype.dispose = function () {
        this.text = null; // StrictNullOverride: nulling out ok in dispose
        this.highlights = null; // StrictNullOverride: nulling out ok in dispose
    };
    HighlightedLabel.escapeNewLines = function (text, highlights) {
        var total = 0;
        var extra = 0;
        return text.replace(/\r\n|\r|\n/, function (match, offset) {
            extra = match === '\r\n' ? -1 : 0;
            offset += total;
            for (var _i = 0, highlights_1 = highlights; _i < highlights_1.length; _i++) {
                var highlight = highlights_1[_i];
                if (highlight.end <= offset) {
                    continue;
                }
                if (highlight.start >= offset) {
                    highlight.start += extra;
                }
                if (highlight.end >= offset) {
                    highlight.end += extra;
                }
            }
            total += extra;
            return '\u23CE';
        });
    };
    return HighlightedLabel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1H = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var FastLabelNode = /** @class */ (function () {
    function FastLabelNode(_element) {
        this._element = _element;
    }
    Object.defineProperty(FastLabelNode.prototype, "element", {
        get: function () {
            return this._element;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FastLabelNode.prototype, "textContent", {
        set: function (content) {
            if (this.disposed || content === this._textContent) {
                return;
            }
            this._textContent = content;
            this._element.textContent = content;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FastLabelNode.prototype, "className", {
        set: function (className) {
            if (this.disposed || className === this._className) {
                return;
            }
            this._className = className;
            this._element.className = className;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FastLabelNode.prototype, "title", {
        set: function (title) {
            if (this.disposed || title === this._title) {
                return;
            }
            this._title = title;
            if (this._title) {
                this._element.title = title;
            }
            else {
                this._element.removeAttribute('title');
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FastLabelNode.prototype, "empty", {
        set: function (empty) {
            if (this.disposed || empty === this._empty) {
                return;
            }
            this._empty = empty;
            this._element.style.marginLeft = empty ? '0' : null;
        },
        enumerable: true,
        configurable: true
    });
    FastLabelNode.prototype.dispose = function () {
        this.disposed = true;
    };
    return FastLabelNode;
}());
var IconLabel = /** @class */ (function (_super) {
    __extends$1H(IconLabel, _super);
    function IconLabel(container, options) {
        var _this = _super.call(this) || this;
        _this.domNode = _this._register(new FastLabelNode(append(container, $('.monaco-icon-label'))));
        _this.labelDescriptionContainer = _this._register(new FastLabelNode(append(_this.domNode.element, $('.monaco-icon-label-description-container'))));
        if (options && options.supportHighlights) {
            _this.labelNode = _this._register(new HighlightedLabel(append(_this.labelDescriptionContainer.element, $('a.label-name')), !options.donotSupportOcticons));
        }
        else {
            _this.labelNode = _this._register(new FastLabelNode(append(_this.labelDescriptionContainer.element, $('a.label-name'))));
        }
        if (options && options.supportDescriptionHighlights) {
            _this.descriptionNodeFactory = function () { return _this._register(new HighlightedLabel(append(_this.labelDescriptionContainer.element, $('span.label-description')), !options.donotSupportOcticons)); };
        }
        else {
            _this.descriptionNodeFactory = function () { return _this._register(new FastLabelNode(append(_this.labelDescriptionContainer.element, $('span.label-description')))); };
        }
        return _this;
    }
    IconLabel.prototype.setValue = function (label, description, options) {
        var classes = ['monaco-icon-label'];
        if (options) {
            if (options.extraClasses) {
                classes.push.apply(classes, options.extraClasses);
            }
            if (options.italic) {
                classes.push('italic');
            }
        }
        this.domNode.className = classes.join(' ');
        this.domNode.title = options && options.title ? options.title : '';
        if (this.labelNode instanceof HighlightedLabel) {
            this.labelNode.set(label || '', options ? options.matches : void 0, options && options.title ? options.title : void 0, options && options.labelEscapeNewLines);
        }
        else {
            this.labelNode.textContent = label || '';
        }
        if (description || this.descriptionNode) {
            if (!this.descriptionNode) {
                this.descriptionNode = this.descriptionNodeFactory(); // description node is created lazily on demand
            }
            if (this.descriptionNode instanceof HighlightedLabel) {
                this.descriptionNode.set(description || '', options ? options.descriptionMatches : void 0);
                if (options && options.descriptionTitle) {
                    this.descriptionNode.element.title = options.descriptionTitle;
                }
                else {
                    this.descriptionNode.element.removeAttribute('title');
                }
            }
            else {
                this.descriptionNode.textContent = description || '';
                this.descriptionNode.title = options && options.descriptionTitle ? options.descriptionTitle : '';
                this.descriptionNode.empty = !description;
            }
        }
    };
    return IconLabel;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Define extension point ids
var Extensions$6 = {
    ModesRegistry: 'editor.modesRegistry'
};
var EditorModesRegistry = /** @class */ (function () {
    function EditorModesRegistry() {
        this._onDidAddLanguages = new Emitter();
        this.onDidAddLanguages = this._onDidAddLanguages.event;
        this._languages = [];
    }
    // --- languages
    EditorModesRegistry.prototype.registerLanguage = function (def) {
        this._languages.push(def);
        this._onDidAddLanguages.fire([def]);
    };
    EditorModesRegistry.prototype.getLanguages = function () {
        return this._languages.slice(0);
    };
    return EditorModesRegistry;
}());
var ModesRegistry = new EditorModesRegistry();
Registry.add(Extensions$6.ModesRegistry, ModesRegistry);
var PLAINTEXT_MODE_ID = 'plaintext';
var PLAINTEXT_LANGUAGE_IDENTIFIER = new LanguageIdentifier(PLAINTEXT_MODE_ID, 1 /* PlainText */);
ModesRegistry.registerLanguage({
    id: PLAINTEXT_MODE_ID,
    extensions: ['.txt', '.gitignore'],
    aliases: [localize('plainText.alias', "Plain Text"), 'text'],
    mimetypes: ['text/plain']
});
LanguageConfigurationRegistry.register(PLAINTEXT_LANGUAGE_IDENTIFIER, {
    brackets: [
        ['(', ')'],
        ['[', ']'],
        ['{', '}'],
    ]
});

var FileKind;
(function (FileKind) {
    FileKind[FileKind["FILE"] = 0] = "FILE";
    FileKind[FileKind["FOLDER"] = 1] = "FOLDER";
    FileKind[FileKind["ROOT_FOLDER"] = 2] = "ROOT_FOLDER";
})(FileKind || (FileKind = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getIconClasses(modelService, modeService, resource, fileKind) {
    // we always set these base classes even if we do not have a path
    var classes = fileKind === FileKind.ROOT_FOLDER ? ['rootfolder-icon'] : fileKind === FileKind.FOLDER ? ['folder-icon'] : ['file-icon'];
    if (resource) {
        // Get the path and name of the resource. For data-URIs, we need to parse specially
        var name_1;
        var path = void 0;
        if (resource.scheme === Schemas.data) {
            var metadata = DataUri.parseMetaData(resource);
            name_1 = metadata.get(DataUri.META_DATA_LABEL);
            path = name_1;
        }
        else {
            name_1 = cssEscape(basenameOrAuthority(resource).toLowerCase());
            path = resource.path.toLowerCase();
        }
        // Folders
        if (fileKind === FileKind.FOLDER) {
            classes.push(name_1 + "-name-folder-icon");
        }
        // Files
        else {
            // Name & Extension(s)
            if (name_1) {
                classes.push(name_1 + "-name-file-icon");
                var dotSegments = name_1.split('.');
                for (var i = 1; i < dotSegments.length; i++) {
                    classes.push(dotSegments.slice(i).join('.') + "-ext-file-icon"); // add each combination of all found extensions if more than one
                }
                classes.push("ext-file-icon"); // extra segment to increase file-ext score
            }
            // Configured Language
            var configuredLangId = getConfiguredLangId(modelService, resource);
            configuredLangId = configuredLangId || modeService.getModeIdByFilepathOrFirstLine(path);
            if (configuredLangId) {
                classes.push(cssEscape(configuredLangId) + "-lang-file-icon");
            }
        }
    }
    return classes;
}
function getConfiguredLangId(modelService, resource) {
    var configuredLangId = null;
    if (resource) {
        var model = modelService.getModel(resource);
        if (model) {
            var modeId = model.getLanguageIdentifier().language;
            if (modeId && modeId !== PLAINTEXT_MODE_ID) {
                configuredLangId = modeId; // only take if the mode is specific (aka no just plain text)
            }
        }
    }
    return configuredLangId;
}
function cssEscape(val) {
    return val.replace(/\s/g, '\\$&'); // make sure to not introduce CSS classes from files that contain whitespace
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __assign$3 = (undefined && undefined.__assign) || function () {
    __assign$3 = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign$3.apply(this, arguments);
};
var __decorate$k = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$h = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var expandSuggestionDocsByDefault = false;
var maxSuggestionsToShow = 12;
/**
 * Suggest widget colors
 */
var editorSuggestWidgetBackground = registerColor('editorSuggestWidget.background', { dark: editorWidgetBackground, light: editorWidgetBackground, hc: editorWidgetBackground }, localize('editorSuggestWidgetBackground', 'Background color of the suggest widget.'));
var editorSuggestWidgetBorder = registerColor('editorSuggestWidget.border', { dark: editorWidgetBorder, light: editorWidgetBorder, hc: editorWidgetBorder }, localize('editorSuggestWidgetBorder', 'Border color of the suggest widget.'));
var editorSuggestWidgetForeground = registerColor('editorSuggestWidget.foreground', { dark: editorForeground, light: editorForeground, hc: editorForeground }, localize('editorSuggestWidgetForeground', 'Foreground color of the suggest widget.'));
var editorSuggestWidgetSelectedBackground = registerColor('editorSuggestWidget.selectedBackground', { dark: listFocusBackground, light: listFocusBackground, hc: listFocusBackground }, localize('editorSuggestWidgetSelectedBackground', 'Background color of the selected entry in the suggest widget.'));
var editorSuggestWidgetHighlightForeground = registerColor('editorSuggestWidget.highlightForeground', { dark: listHighlightForeground, light: listHighlightForeground, hc: listHighlightForeground }, localize('editorSuggestWidgetHighlightForeground', 'Color of the match highlights in the suggest widget.'));
var colorRegExp = /^(#([\da-f]{3}){1,2}|(rgb|hsl)a\(\s*(\d{1,3}%?\s*,\s*){3}(1|0?\.\d+)\)|(rgb|hsl)\(\s*\d{1,3}%?(\s*,\s*\d{1,3}%?){2}\s*\))$/i;
function matchesColor(text) {
    return text && text.match(colorRegExp) ? text : null;
}
function canExpandCompletionItem(item) {
    if (!item) {
        return false;
    }
    var suggestion = item.suggestion;
    if (suggestion.documentation) {
        return true;
    }
    return (suggestion.detail && suggestion.detail !== suggestion.label);
}
var Renderer$1 = /** @class */ (function () {
    function Renderer(widget, editor, triggerKeybindingLabel, _modelService, _modeService, _themeService) {
        this.widget = widget;
        this.editor = editor;
        this.triggerKeybindingLabel = triggerKeybindingLabel;
        this._modelService = _modelService;
        this._modeService = _modeService;
        this._themeService = _themeService;
    }
    Object.defineProperty(Renderer.prototype, "templateId", {
        get: function () {
            return 'suggestion';
        },
        enumerable: true,
        configurable: true
    });
    Renderer.prototype.renderTemplate = function (container) {
        var _this = this;
        var data = Object.create(null);
        data.disposables = [];
        data.root = container;
        addClass(data.root, 'show-file-icons');
        data.icon = append(container, $('.icon'));
        data.colorspan = append(data.icon, $('span.colorspan'));
        var text = append(container, $('.contents'));
        var main = append(text, $('.main'));
        data.iconLabel = new IconLabel(main, { supportHighlights: true });
        data.disposables.push(data.iconLabel);
        data.typeLabel = append(main, $('span.type-label'));
        data.readMore = append(main, $('span.readMore'));
        data.readMore.title = localize('readMore', "Read More...{0}", this.triggerKeybindingLabel);
        var configureFont = function () {
            var configuration = _this.editor.getConfiguration();
            var fontFamily = configuration.fontInfo.fontFamily;
            var fontSize = configuration.contribInfo.suggestFontSize || configuration.fontInfo.fontSize;
            var lineHeight = configuration.contribInfo.suggestLineHeight || configuration.fontInfo.lineHeight;
            var fontWeight = configuration.fontInfo.fontWeight;
            var fontSizePx = fontSize + "px";
            var lineHeightPx = lineHeight + "px";
            data.root.style.fontSize = fontSizePx;
            data.root.style.fontWeight = fontWeight;
            main.style.fontFamily = fontFamily;
            main.style.lineHeight = lineHeightPx;
            data.icon.style.height = lineHeightPx;
            data.icon.style.width = lineHeightPx;
            data.readMore.style.height = lineHeightPx;
            data.readMore.style.width = lineHeightPx;
        };
        configureFont();
        chain(this.editor.onDidChangeConfiguration.bind(this.editor))
            .filter(function (e) { return e.fontInfo || e.contribInfo; })
            .on(configureFont, null, data.disposables);
        return data;
    };
    Renderer.prototype.renderElement = function (element, _index, templateData) {
        var _this = this;
        var data = templateData;
        var suggestion = element.suggestion;
        data.icon.className = 'icon ' + completionKindToCssClass(suggestion.kind);
        data.colorspan.style.backgroundColor = '';
        var labelOptions = {
            labelEscapeNewLines: true,
            matches: createMatches(element.matches)
        };
        var color;
        if (suggestion.kind === 19 /* Color */ && (color = matchesColor(suggestion.label) || typeof suggestion.documentation === 'string' && matchesColor(suggestion.documentation))) {
            // special logic for 'color' completion items
            data.icon.className = 'icon customcolor';
            data.colorspan.style.backgroundColor = color;
        }
        else if (suggestion.kind === 20 /* File */ && this._themeService.getIconTheme().hasFileIcons) {
            // special logic for 'file' completion items
            data.icon.className = 'icon hide';
            labelOptions.extraClasses = [].concat(getIconClasses(this._modelService, this._modeService, URI.from({ scheme: 'fake', path: suggestion.label }), FileKind.FILE), getIconClasses(this._modelService, this._modeService, URI.from({ scheme: 'fake', path: suggestion.detail }), FileKind.FILE));
        }
        else if (suggestion.kind === 23 /* Folder */ && this._themeService.getIconTheme().hasFolderIcons) {
            // special logic for 'folder' completion items
            data.icon.className = 'icon hide';
            labelOptions.extraClasses = [].concat(getIconClasses(this._modelService, this._modeService, URI.from({ scheme: 'fake', path: suggestion.label }), FileKind.FOLDER), getIconClasses(this._modelService, this._modeService, URI.from({ scheme: 'fake', path: suggestion.detail }), FileKind.FOLDER));
        }
        else {
            // normal icon
            data.icon.className = 'icon hide';
            labelOptions.extraClasses = [
                "suggest-icon " + completionKindToCssClass(suggestion.kind)
            ];
        }
        data.iconLabel.setValue(suggestion.label, undefined, labelOptions);
        data.typeLabel.textContent = (suggestion.detail || '').replace(/\n.*$/m, '');
        if (canExpandCompletionItem(element)) {
            show(data.readMore);
            data.readMore.onmousedown = function (e) {
                e.stopPropagation();
                e.preventDefault();
            };
            data.readMore.onclick = function (e) {
                e.stopPropagation();
                e.preventDefault();
                _this.widget.toggleDetails();
            };
        }
        else {
            hide(data.readMore);
            data.readMore.onmousedown = null;
            data.readMore.onclick = null;
        }
    };
    Renderer.prototype.disposeElement = function () {
        // noop
    };
    Renderer.prototype.disposeTemplate = function (templateData) {
        templateData.disposables = dispose(templateData.disposables);
    };
    Renderer = __decorate$k([
        __param$h(3, IModelService),
        __param$h(4, IModeService),
        __param$h(5, IThemeService)
    ], Renderer);
    return Renderer;
}());
var SuggestionDetails = /** @class */ (function () {
    function SuggestionDetails(container, widget, editor, markdownRenderer, triggerKeybindingLabel) {
        var _this = this;
        this.widget = widget;
        this.editor = editor;
        this.markdownRenderer = markdownRenderer;
        this.triggerKeybindingLabel = triggerKeybindingLabel;
        this.borderWidth = 1;
        this.disposables = [];
        this.el = append(container, $('.details'));
        this.disposables.push(toDisposable(function () { return container.removeChild(_this.el); }));
        this.body = $('.body');
        this.scrollbar = new DomScrollableElement(this.body, {});
        append(this.el, this.scrollbar.getDomNode());
        this.disposables.push(this.scrollbar);
        this.header = append(this.body, $('.header'));
        this.close = append(this.header, $('span.close'));
        this.close.title = localize('readLess', "Read less...{0}", this.triggerKeybindingLabel);
        this.type = append(this.header, $('p.type'));
        this.docs = append(this.body, $('p.docs'));
        this.ariaLabel = null;
        this.configureFont();
        chain(this.editor.onDidChangeConfiguration.bind(this.editor))
            .filter(function (e) { return e.fontInfo; })
            .on(this.configureFont, this, this.disposables);
        markdownRenderer.onDidRenderCodeBlock(function () { return _this.scrollbar.scanDomNode(); }, this, this.disposables);
    }
    Object.defineProperty(SuggestionDetails.prototype, "element", {
        get: function () {
            return this.el;
        },
        enumerable: true,
        configurable: true
    });
    SuggestionDetails.prototype.render = function (item) {
        var _this = this;
        this.renderDisposeable = dispose(this.renderDisposeable);
        if (!item || !canExpandCompletionItem(item)) {
            this.type.textContent = '';
            this.docs.textContent = '';
            addClass(this.el, 'no-docs');
            this.ariaLabel = null;
            return;
        }
        removeClass(this.el, 'no-docs');
        if (typeof item.suggestion.documentation === 'string') {
            removeClass(this.docs, 'markdown-docs');
            this.docs.textContent = item.suggestion.documentation;
        }
        else {
            addClass(this.docs, 'markdown-docs');
            this.docs.innerHTML = '';
            var renderedContents = this.markdownRenderer.render(item.suggestion.documentation);
            this.renderDisposeable = renderedContents;
            this.docs.appendChild(renderedContents.element);
        }
        if (item.suggestion.detail) {
            this.type.innerText = item.suggestion.detail;
            show(this.type);
        }
        else {
            this.type.innerText = '';
            hide(this.type);
        }
        this.el.style.height = this.header.offsetHeight + this.docs.offsetHeight + (this.borderWidth * 2) + 'px';
        this.close.onmousedown = function (e) {
            e.preventDefault();
            e.stopPropagation();
        };
        this.close.onclick = function (e) {
            e.preventDefault();
            e.stopPropagation();
            _this.widget.toggleDetails();
        };
        this.body.scrollTop = 0;
        this.scrollbar.scanDomNode();
        this.ariaLabel = format('{0}{1}', item.suggestion.detail || '', item.suggestion.documentation ? (typeof item.suggestion.documentation === 'string' ? item.suggestion.documentation : item.suggestion.documentation.value) : '');
    };
    SuggestionDetails.prototype.getAriaLabel = function () {
        return this.ariaLabel;
    };
    SuggestionDetails.prototype.scrollDown = function (much) {
        if (much === void 0) { much = 8; }
        this.body.scrollTop += much;
    };
    SuggestionDetails.prototype.scrollUp = function (much) {
        if (much === void 0) { much = 8; }
        this.body.scrollTop -= much;
    };
    SuggestionDetails.prototype.scrollTop = function () {
        this.body.scrollTop = 0;
    };
    SuggestionDetails.prototype.scrollBottom = function () {
        this.body.scrollTop = this.body.scrollHeight;
    };
    SuggestionDetails.prototype.pageDown = function () {
        this.scrollDown(80);
    };
    SuggestionDetails.prototype.pageUp = function () {
        this.scrollUp(80);
    };
    SuggestionDetails.prototype.setBorderWidth = function (width) {
        this.borderWidth = width;
    };
    SuggestionDetails.prototype.configureFont = function () {
        var configuration = this.editor.getConfiguration();
        var fontFamily = configuration.fontInfo.fontFamily;
        var fontSize = configuration.contribInfo.suggestFontSize || configuration.fontInfo.fontSize;
        var lineHeight = configuration.contribInfo.suggestLineHeight || configuration.fontInfo.lineHeight;
        var fontWeight = configuration.fontInfo.fontWeight;
        var fontSizePx = fontSize + "px";
        var lineHeightPx = lineHeight + "px";
        this.el.style.fontSize = fontSizePx;
        this.el.style.fontWeight = fontWeight;
        this.type.style.fontFamily = fontFamily;
        this.close.style.height = lineHeightPx;
        this.close.style.width = lineHeightPx;
    };
    SuggestionDetails.prototype.dispose = function () {
        this.disposables = dispose(this.disposables);
        this.renderDisposeable = dispose(this.renderDisposeable);
    };
    return SuggestionDetails;
}());
var SuggestWidget = /** @class */ (function () {
    function SuggestWidget(editor, telemetryService, contextKeyService, themeService, storageService, keybindingService, modeService, openerService, instantiationService) {
        var _this = this;
        this.editor = editor;
        this.telemetryService = telemetryService;
        // Editor.IContentWidget.allowEditorOverflow
        this.allowEditorOverflow = true;
        this.ignoreFocusEvents = false;
        this.editorBlurTimeout = new TimeoutTimer();
        this.showTimeout = new TimeoutTimer();
        this.onDidSelectEmitter = new Emitter();
        this.onDidFocusEmitter = new Emitter();
        this.onDidHideEmitter = new Emitter();
        this.onDidShowEmitter = new Emitter();
        this.onDidSelect = this.onDidSelectEmitter.event;
        this.onDidFocus = this.onDidFocusEmitter.event;
        this.onDidHide = this.onDidHideEmitter.event;
        this.onDidShow = this.onDidShowEmitter.event;
        this.maxWidgetWidth = 660;
        this.listWidth = 330;
        this.storageServiceAvailable = true;
        this.expandSuggestionDocs = false;
        this.firstFocusInCurrentList = false;
        var kb = keybindingService.lookupKeybinding('editor.action.triggerSuggest');
        var triggerKeybindingLabel = !kb ? '' : " (" + kb.getLabel() + ")";
        var markdownRenderer = new MarkdownRenderer(editor, modeService, openerService);
        this.isAuto = false;
        this.focusedItem = null;
        this.storageService = storageService;
        // :facepalm:
        this.storageService.store('___suggest___', true, 0 /* GLOBAL */);
        if (!this.storageService.get('___suggest___', 0 /* GLOBAL */)) {
            this.storageServiceAvailable = false;
        }
        this.storageService.remove('___suggest___', 0 /* GLOBAL */);
        this.element = $('.editor-widget.suggest-widget');
        if (!this.editor.getConfiguration().contribInfo.iconsInSuggestions) {
            addClass(this.element, 'no-icons');
        }
        this.messageElement = append(this.element, $('.message'));
        this.listElement = append(this.element, $('.tree'));
        this.details = new SuggestionDetails(this.element, this, this.editor, markdownRenderer, triggerKeybindingLabel);
        var renderer = instantiationService.createInstance(Renderer$1, this, this.editor, triggerKeybindingLabel);
        this.list = new List(this.listElement, this, [renderer], {
            useShadows: false,
            selectOnMouseDown: true,
            focusOnMouseDown: false,
            openController: { shouldOpen: function () { return false; } }
        });
        this.toDispose = [
            attachListStyler(this.list, themeService, {
                listInactiveFocusBackground: editorSuggestWidgetSelectedBackground,
                listInactiveFocusOutline: activeContrastBorder
            }),
            themeService.onThemeChange(function (t) { return _this.onThemeChange(t); }),
            editor.onDidLayoutChange(function () { return _this.onEditorLayoutChange(); }),
            this.list.onSelectionChange(function (e) { return _this.onListSelection(e); }),
            this.list.onFocusChange(function (e) { return _this.onListFocus(e); }),
            this.editor.onDidChangeCursorSelection(function () { return _this.onCursorSelectionChanged(); })
        ];
        this.suggestWidgetVisible = Context.Visible.bindTo(contextKeyService);
        this.suggestWidgetMultipleSuggestions = Context.MultipleSuggestions.bindTo(contextKeyService);
        this.editor.addContentWidget(this);
        this.setState(0 /* Hidden */);
        this.onThemeChange(themeService.getTheme());
    }
    SuggestWidget.prototype.onCursorSelectionChanged = function () {
        if (this.state === 0 /* Hidden */) {
            return;
        }
        this.editor.layoutContentWidget(this);
    };
    SuggestWidget.prototype.onEditorLayoutChange = function () {
        if ((this.state === 3 /* Open */ || this.state === 5 /* Details */) && this.expandDocsSettingFromStorage()) {
            this.expandSideOrBelow();
        }
    };
    SuggestWidget.prototype.onListSelection = function (e) {
        var _this = this;
        if (!e.elements.length) {
            return;
        }
        var item = e.elements[0];
        var index = e.indexes[0];
        item.resolve(CancellationToken.None).then(function () {
            _this.onDidSelectEmitter.fire({ item: item, index: index, model: _this.completionModel });
            alert(localize('suggestionAriaAccepted', "{0}, accepted", item.suggestion.label));
            _this.editor.focus();
        });
    };
    SuggestWidget.prototype._getSuggestionAriaAlertLabel = function (item) {
        var isSnippet = item.suggestion.kind === 25 /* Snippet */;
        if (!canExpandCompletionItem(item)) {
            return isSnippet ? localize('ariaCurrentSnippetSuggestion', "{0}, snippet suggestion", item.suggestion.label)
                : localize('ariaCurrentSuggestion', "{0}, suggestion", item.suggestion.label);
        }
        else if (this.expandDocsSettingFromStorage()) {
            return isSnippet ? localize('ariaCurrentSnippeSuggestionReadDetails', "{0}, snippet suggestion. Reading details. {1}", item.suggestion.label, this.details.getAriaLabel())
                : localize('ariaCurrenttSuggestionReadDetails', "{0}, suggestion. Reading details. {1}", item.suggestion.label, this.details.getAriaLabel());
        }
        else {
            return isSnippet ? localize('ariaCurrentSnippetSuggestionWithDetails', "{0}, snippet suggestion, has details", item.suggestion.label)
                : localize('ariaCurrentSuggestionWithDetails', "{0}, suggestion, has details", item.suggestion.label);
        }
    };
    SuggestWidget.prototype._ariaAlert = function (newAriaAlertLabel) {
        if (this._lastAriaAlertLabel === newAriaAlertLabel) {
            return;
        }
        this._lastAriaAlertLabel = newAriaAlertLabel;
        if (this._lastAriaAlertLabel) {
            alert(this._lastAriaAlertLabel);
        }
    };
    SuggestWidget.prototype.onThemeChange = function (theme) {
        var backgroundColor = theme.getColor(editorSuggestWidgetBackground);
        if (backgroundColor) {
            this.listElement.style.backgroundColor = backgroundColor.toString();
            this.details.element.style.backgroundColor = backgroundColor.toString();
            this.messageElement.style.backgroundColor = backgroundColor.toString();
        }
        var borderColor = theme.getColor(editorSuggestWidgetBorder);
        if (borderColor) {
            this.listElement.style.borderColor = borderColor.toString();
            this.details.element.style.borderColor = borderColor.toString();
            this.messageElement.style.borderColor = borderColor.toString();
            this.detailsBorderColor = borderColor.toString();
        }
        var focusBorderColor = theme.getColor(focusBorder);
        if (focusBorderColor) {
            this.detailsFocusBorderColor = focusBorderColor.toString();
        }
        this.details.setBorderWidth(theme.type === 'hc' ? 2 : 1);
    };
    SuggestWidget.prototype.onListFocus = function (e) {
        var _this = this;
        if (this.ignoreFocusEvents) {
            return;
        }
        if (!e.elements.length) {
            if (this.currentSuggestionDetails) {
                this.currentSuggestionDetails.cancel();
                this.currentSuggestionDetails = null;
                this.focusedItem = null;
            }
            this._ariaAlert(null);
            return;
        }
        var item = e.elements[0];
        var index = e.indexes[0];
        this.firstFocusInCurrentList = !this.focusedItem;
        if (item !== this.focusedItem) {
            if (this.currentSuggestionDetails) {
                this.currentSuggestionDetails.cancel();
                this.currentSuggestionDetails = null;
            }
            this.focusedItem = item;
            this.list.reveal(index);
            this.currentSuggestionDetails = createCancelablePromise(function (token) { return item.resolve(token); });
            this.currentSuggestionDetails.then(function () {
                if (_this.list.length < index) {
                    return;
                }
                // item can have extra information, so re-render
                _this.ignoreFocusEvents = true;
                _this.list.splice(index, 1, [item]);
                _this.list.setFocus([index]);
                _this.ignoreFocusEvents = false;
                if (_this.expandDocsSettingFromStorage()) {
                    _this.showDetails();
                }
                else {
                    removeClass(_this.element, 'docs-side');
                }
                _this._ariaAlert(_this._getSuggestionAriaAlertLabel(item));
            }).catch(onUnexpectedError).then(function () {
                if (_this.focusedItem === item) {
                    _this.currentSuggestionDetails = null;
                }
            });
        }
        // emit an event
        this.onDidFocusEmitter.fire({ item: item, index: index, model: this.completionModel });
    };
    SuggestWidget.prototype.setState = function (state) {
        if (!this.element) {
            return;
        }
        var stateChanged = this.state !== state;
        this.state = state;
        toggleClass(this.element, 'frozen', state === 4 /* Frozen */);
        switch (state) {
            case 0 /* Hidden */:
                hide(this.messageElement, this.details.element, this.listElement);
                this.hide();
                this.listHeight = 0;
                if (stateChanged) {
                    this.list.splice(0, this.list.length);
                }
                this.focusedItem = null;
                break;
            case 1 /* Loading */:
                this.messageElement.textContent = SuggestWidget.LOADING_MESSAGE;
                hide(this.listElement, this.details.element);
                show(this.messageElement);
                removeClass(this.element, 'docs-side');
                this.show();
                this.focusedItem = null;
                break;
            case 2 /* Empty */:
                this.messageElement.textContent = SuggestWidget.NO_SUGGESTIONS_MESSAGE;
                hide(this.listElement, this.details.element);
                show(this.messageElement);
                removeClass(this.element, 'docs-side');
                this.show();
                this.focusedItem = null;
                break;
            case 3 /* Open */:
                hide(this.messageElement);
                show(this.listElement);
                this.show();
                break;
            case 4 /* Frozen */:
                hide(this.messageElement);
                show(this.listElement);
                this.show();
                break;
            case 5 /* Details */:
                hide(this.messageElement);
                show(this.details.element, this.listElement);
                this.show();
                this._ariaAlert(this.details.getAriaLabel());
                break;
        }
    };
    SuggestWidget.prototype.showTriggered = function (auto, delay) {
        var _this = this;
        if (this.state !== 0 /* Hidden */) {
            return;
        }
        this.isAuto = !!auto;
        if (!this.isAuto) {
            this.loadingTimeout = setTimeout(function () {
                _this.loadingTimeout = null;
                _this.setState(1 /* Loading */);
            }, delay);
        }
    };
    SuggestWidget.prototype.showSuggestions = function (completionModel, selectionIndex, isFrozen, isAuto) {
        if (this.loadingTimeout) {
            clearTimeout(this.loadingTimeout);
            this.loadingTimeout = null;
        }
        if (this.currentSuggestionDetails) {
            this.currentSuggestionDetails.cancel();
            this.currentSuggestionDetails = null;
        }
        if (this.completionModel !== completionModel) {
            this.completionModel = completionModel;
        }
        if (isFrozen && this.state !== 2 /* Empty */ && this.state !== 0 /* Hidden */) {
            this.setState(4 /* Frozen */);
            return;
        }
        var visibleCount = this.completionModel.items.length;
        var isEmpty = visibleCount === 0;
        this.suggestWidgetMultipleSuggestions.set(visibleCount > 1);
        if (isEmpty) {
            if (isAuto) {
                this.setState(0 /* Hidden */);
            }
            else {
                this.setState(2 /* Empty */);
            }
            this.completionModel = null;
        }
        else {
            if (this.state !== 3 /* Open */) {
                var stats = this.completionModel.stats;
                stats['wasAutomaticallyTriggered'] = !!isAuto;
                /* __GDPR__
                    "suggestWidget" : {
                        "wasAutomaticallyTriggered" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
                        "${include}": [
                            "${ICompletionStats}",
                            "${EditorTelemetryData}"
                        ]
                    }
                */
                this.telemetryService.publicLog('suggestWidget', __assign$3({}, stats, this.editor.getTelemetryData()));
            }
            this.focusedItem = null;
            this.list.splice(0, this.list.length, this.completionModel.items);
            if (isFrozen) {
                this.setState(4 /* Frozen */);
            }
            else {
                this.setState(3 /* Open */);
            }
            this.list.reveal(selectionIndex, selectionIndex);
            this.list.setFocus([selectionIndex]);
            // Reset focus border
            if (this.detailsBorderColor) {
                this.details.element.style.borderColor = this.detailsBorderColor;
            }
        }
    };
    SuggestWidget.prototype.selectNextPage = function () {
        switch (this.state) {
            case 0 /* Hidden */:
                return false;
            case 5 /* Details */:
                this.details.pageDown();
                return true;
            case 1 /* Loading */:
                return !this.isAuto;
            default:
                this.list.focusNextPage();
                return true;
        }
    };
    SuggestWidget.prototype.selectNext = function () {
        switch (this.state) {
            case 0 /* Hidden */:
                return false;
            case 1 /* Loading */:
                return !this.isAuto;
            default:
                this.list.focusNext(1, true);
                return true;
        }
    };
    SuggestWidget.prototype.selectLast = function () {
        switch (this.state) {
            case 0 /* Hidden */:
                return false;
            case 5 /* Details */:
                this.details.scrollBottom();
                return true;
            case 1 /* Loading */:
                return !this.isAuto;
            default:
                this.list.focusLast();
                return true;
        }
    };
    SuggestWidget.prototype.selectPreviousPage = function () {
        switch (this.state) {
            case 0 /* Hidden */:
                return false;
            case 5 /* Details */:
                this.details.pageUp();
                return true;
            case 1 /* Loading */:
                return !this.isAuto;
            default:
                this.list.focusPreviousPage();
                return true;
        }
    };
    SuggestWidget.prototype.selectPrevious = function () {
        switch (this.state) {
            case 0 /* Hidden */:
                return false;
            case 1 /* Loading */:
                return !this.isAuto;
            default:
                this.list.focusPrevious(1, true);
                return false;
        }
    };
    SuggestWidget.prototype.selectFirst = function () {
        switch (this.state) {
            case 0 /* Hidden */:
                return false;
            case 5 /* Details */:
                this.details.scrollTop();
                return true;
            case 1 /* Loading */:
                return !this.isAuto;
            default:
                this.list.focusFirst();
                return true;
        }
    };
    SuggestWidget.prototype.getFocusedItem = function () {
        if (this.state !== 0 /* Hidden */
            && this.state !== 2 /* Empty */
            && this.state !== 1 /* Loading */) {
            return {
                item: this.list.getFocusedElements()[0],
                index: this.list.getFocus()[0],
                model: this.completionModel
            };
        }
        return undefined;
    };
    SuggestWidget.prototype.toggleDetailsFocus = function () {
        if (this.state === 5 /* Details */) {
            this.setState(3 /* Open */);
            if (this.detailsBorderColor) {
                this.details.element.style.borderColor = this.detailsBorderColor;
            }
        }
        else if (this.state === 3 /* Open */ && this.expandDocsSettingFromStorage()) {
            this.setState(5 /* Details */);
            if (this.detailsFocusBorderColor) {
                this.details.element.style.borderColor = this.detailsFocusBorderColor;
            }
        }
        /* __GDPR__
            "suggestWidget:toggleDetailsFocus" : {
                "${include}": [
                    "${EditorTelemetryData}"
                ]
            }
        */
        this.telemetryService.publicLog('suggestWidget:toggleDetailsFocus', this.editor.getTelemetryData());
    };
    SuggestWidget.prototype.toggleDetails = function () {
        if (!canExpandCompletionItem(this.list.getFocusedElements()[0])) {
            return;
        }
        if (this.expandDocsSettingFromStorage()) {
            this.updateExpandDocsSetting(false);
            hide(this.details.element);
            removeClass(this.element, 'docs-side');
            removeClass(this.element, 'docs-below');
            this.editor.layoutContentWidget(this);
            /* __GDPR__
                "suggestWidget:collapseDetails" : {
                    "${include}": [
                        "${EditorTelemetryData}"
                    ]
                }
            */
            this.telemetryService.publicLog('suggestWidget:collapseDetails', this.editor.getTelemetryData());
        }
        else {
            if (this.state !== 3 /* Open */ && this.state !== 5 /* Details */ && this.state !== 4 /* Frozen */) {
                return;
            }
            this.updateExpandDocsSetting(true);
            this.showDetails();
            this._ariaAlert(this.details.getAriaLabel());
            /* __GDPR__
                "suggestWidget:expandDetails" : {
                    "${include}": [
                        "${EditorTelemetryData}"
                    ]
                }
            */
            this.telemetryService.publicLog('suggestWidget:expandDetails', this.editor.getTelemetryData());
        }
    };
    SuggestWidget.prototype.showDetails = function () {
        this.expandSideOrBelow();
        show(this.details.element);
        this.details.render(this.list.getFocusedElements()[0]);
        this.details.element.style.maxHeight = this.maxWidgetHeight + 'px';
        // Reset margin-top that was set as Fix for #26416
        this.listElement.style.marginTop = '0px';
        // with docs showing up widget width/height may change, so reposition the widget
        this.editor.layoutContentWidget(this);
        this.adjustDocsPosition();
        this.editor.focus();
    };
    SuggestWidget.prototype.show = function () {
        var _this = this;
        var newHeight = this.updateListHeight();
        if (newHeight !== this.listHeight) {
            this.editor.layoutContentWidget(this);
            this.listHeight = newHeight;
        }
        this.suggestWidgetVisible.set(true);
        this.showTimeout.cancelAndSet(function () {
            addClass(_this.element, 'visible');
            _this.onDidShowEmitter.fire(_this);
        }, 100);
    };
    SuggestWidget.prototype.hide = function () {
        this.suggestWidgetVisible.reset();
        this.suggestWidgetMultipleSuggestions.reset();
        removeClass(this.element, 'visible');
    };
    SuggestWidget.prototype.hideWidget = function () {
        clearTimeout(this.loadingTimeout);
        this.setState(0 /* Hidden */);
        this.onDidHideEmitter.fire(this);
    };
    SuggestWidget.prototype.getPosition = function () {
        if (this.state === 0 /* Hidden */) {
            return null;
        }
        return {
            position: this.editor.getPosition(),
            preference: [2 /* BELOW */, 1 /* ABOVE */]
        };
    };
    SuggestWidget.prototype.getDomNode = function () {
        return this.element;
    };
    SuggestWidget.prototype.getId = function () {
        return SuggestWidget.ID;
    };
    SuggestWidget.prototype.updateListHeight = function () {
        var height = 0;
        if (this.state === 2 /* Empty */ || this.state === 1 /* Loading */) {
            height = this.unfocusedHeight;
        }
        else {
            var suggestionCount = this.list.contentHeight / this.unfocusedHeight;
            height = Math.min(suggestionCount, maxSuggestionsToShow) * this.unfocusedHeight;
        }
        this.element.style.lineHeight = this.unfocusedHeight + "px";
        this.listElement.style.height = height + "px";
        this.list.layout(height);
        return height;
    };
    SuggestWidget.prototype.adjustDocsPosition = function () {
        var lineHeight = this.editor.getConfiguration().fontInfo.lineHeight;
        var cursorCoords = this.editor.getScrolledVisiblePosition(this.editor.getPosition());
        var editorCoords = getDomNodePagePosition(this.editor.getDomNode());
        var cursorX = editorCoords.left + cursorCoords.left;
        var cursorY = editorCoords.top + cursorCoords.top + cursorCoords.height;
        var widgetCoords = getDomNodePagePosition(this.element);
        var widgetX = widgetCoords.left;
        var widgetY = widgetCoords.top;
        if (widgetX < cursorX - this.listWidth) {
            // Widget is too far to the left of cursor, swap list and docs
            addClass(this.element, 'list-right');
        }
        else {
            removeClass(this.element, 'list-right');
        }
        // Compare top of the cursor (cursorY - lineheight) with widgetTop to determine if
        // margin-top needs to be applied on list to make it appear right above the cursor
        // Cannot compare cursorY directly as it may be a few decimals off due to zoooming
        if (hasClass(this.element, 'docs-side')
            && cursorY - lineHeight > widgetY
            && this.details.element.offsetHeight > this.listElement.offsetHeight) {
            // Fix for #26416
            // Docs is bigger than list and widget is above cursor, apply margin-top so that list appears right above cursor
            this.listElement.style.marginTop = this.details.element.offsetHeight - this.listElement.offsetHeight + "px";
        }
    };
    SuggestWidget.prototype.expandSideOrBelow = function () {
        if (!canExpandCompletionItem(this.focusedItem) && this.firstFocusInCurrentList) {
            removeClass(this.element, 'docs-side');
            removeClass(this.element, 'docs-below');
            return;
        }
        var matches = this.element.style.maxWidth.match(/(\d+)px/);
        if (!matches || Number(matches[1]) < this.maxWidgetWidth) {
            addClass(this.element, 'docs-below');
            removeClass(this.element, 'docs-side');
        }
        else if (canExpandCompletionItem(this.focusedItem)) {
            addClass(this.element, 'docs-side');
            removeClass(this.element, 'docs-below');
        }
    };
    Object.defineProperty(SuggestWidget.prototype, "maxWidgetHeight", {
        // Heights
        get: function () {
            return this.unfocusedHeight * maxSuggestionsToShow;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuggestWidget.prototype, "unfocusedHeight", {
        get: function () {
            var configuration = this.editor.getConfiguration();
            return configuration.contribInfo.suggestLineHeight || configuration.fontInfo.lineHeight;
        },
        enumerable: true,
        configurable: true
    });
    // IDelegate
    SuggestWidget.prototype.getHeight = function (element) {
        return this.unfocusedHeight;
    };
    SuggestWidget.prototype.getTemplateId = function (element) {
        return 'suggestion';
    };
    SuggestWidget.prototype.expandDocsSettingFromStorage = function () {
        if (this.storageServiceAvailable) {
            return this.storageService.getBoolean('expandSuggestionDocs', 0 /* GLOBAL */, expandSuggestionDocsByDefault);
        }
        else {
            return this.expandSuggestionDocs;
        }
    };
    SuggestWidget.prototype.updateExpandDocsSetting = function (value) {
        if (this.storageServiceAvailable) {
            this.storageService.store('expandSuggestionDocs', value, 0 /* GLOBAL */);
        }
        else {
            this.expandSuggestionDocs = value;
        }
    };
    SuggestWidget.prototype.dispose = function () {
        this.state = null;
        this.currentSuggestionDetails = null;
        this.focusedItem = null;
        this.element = null;
        this.messageElement = null;
        this.listElement = null;
        this.details.dispose();
        this.details = null;
        this.list.dispose();
        this.list = null;
        this.toDispose = dispose(this.toDispose);
        if (this.loadingTimeout) {
            clearTimeout(this.loadingTimeout);
            this.loadingTimeout = null;
        }
        this.editorBlurTimeout.dispose();
        this.showTimeout.dispose();
    };
    SuggestWidget.ID = 'editor.widget.suggestWidget';
    SuggestWidget.LOADING_MESSAGE = localize('suggestWidget.loading', "Loading...");
    SuggestWidget.NO_SUGGESTIONS_MESSAGE = localize('suggestWidget.noSuggestions', "No suggestions.");
    SuggestWidget = __decorate$k([
        __param$h(1, ITelemetryService),
        __param$h(2, IContextKeyService),
        __param$h(3, IThemeService),
        __param$h(4, IStorageService),
        __param$h(5, IKeybindingService),
        __param$h(6, IModeService),
        __param$h(7, IOpenerService),
        __param$h(8, IInstantiationService)
    ], SuggestWidget);
    return SuggestWidget;
}());
registerThemingParticipant(function (theme, collector) {
    var matchHighlight = theme.getColor(editorSuggestWidgetHighlightForeground);
    if (matchHighlight) {
        collector.addRule(".monaco-editor .suggest-widget .monaco-list .monaco-list-row .monaco-highlighted-label .highlight { color: " + matchHighlight + "; }");
    }
    var foreground$$1 = theme.getColor(editorSuggestWidgetForeground);
    if (foreground$$1) {
        collector.addRule(".monaco-editor .suggest-widget { color: " + foreground$$1 + "; }");
    }
    var link = theme.getColor(textLinkForeground);
    if (link) {
        collector.addRule(".monaco-editor .suggest-widget a { color: " + link + "; }");
    }
    var codeBackground = theme.getColor(textCodeBlockBackground);
    if (codeBackground) {
        collector.addRule(".monaco-editor .suggest-widget code { background-color: " + codeBackground + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$l = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$i = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var WordContextKey = /** @class */ (function () {
    function WordContextKey(_editor, contextKeyService) {
        var _this = this;
        this._editor = _editor;
        this._ckAtEnd = WordContextKey.AtEnd.bindTo(contextKeyService);
        this._editor.onDidChangeConfiguration(function (e) { return e.contribInfo && _this._update(); });
        this._update();
    }
    WordContextKey.prototype.dispose = function () {
        dispose(this._confListener, this._selectionListener);
        this._ckAtEnd.reset();
    };
    WordContextKey.prototype._update = function () {
        var _this = this;
        // only update this when tab completions are enabled
        var enabled = this._editor.getConfiguration().contribInfo.tabCompletion === 'on';
        if (this._enabled === enabled) {
            return;
        }
        this._enabled = enabled;
        if (this._enabled) {
            var checkForWordEnd = function () {
                if (!_this._editor.hasModel()) {
                    _this._ckAtEnd.set(false);
                    return;
                }
                var model = _this._editor.getModel();
                var selection = _this._editor.getSelection();
                var word = model.getWordAtPosition(selection.getStartPosition());
                if (!word) {
                    _this._ckAtEnd.set(false);
                    return;
                }
                _this._ckAtEnd.set(word.endColumn === selection.getStartPosition().column);
            };
            this._selectionListener = this._editor.onDidChangeCursorSelection(checkForWordEnd);
            checkForWordEnd();
        }
        else if (this._selectionListener) {
            this._ckAtEnd.reset();
            this._selectionListener.dispose();
            this._selectionListener = undefined;
        }
    };
    WordContextKey.AtEnd = new RawContextKey('atEndOfWord', false);
    WordContextKey = __decorate$l([
        __param$i(1, IContextKeyService)
    ], WordContextKey);
    return WordContextKey;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1I = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$m = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$j = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var AcceptOnCharacterOracle = /** @class */ (function () {
    function AcceptOnCharacterOracle(editor, widget, accept) {
        var _this = this;
        this._disposables = [];
        this._activeAcceptCharacters = new Set();
        this._disposables.push(widget.onDidShow(function () { return _this._onItem(widget.getFocusedItem()); }));
        this._disposables.push(widget.onDidFocus(this._onItem, this));
        this._disposables.push(widget.onDidHide(this.reset, this));
        this._disposables.push(editor.onWillType(function (text) {
            if (_this._activeItem) {
                var ch = text[text.length - 1];
                if (_this._activeAcceptCharacters.has(ch) && editor.getConfiguration().contribInfo.acceptSuggestionOnCommitCharacter) {
                    accept(_this._activeItem);
                }
            }
        }));
    }
    AcceptOnCharacterOracle.prototype._onItem = function (selected) {
        if (!selected || isFalsyOrEmpty(selected.item.suggestion.commitCharacters)) {
            this.reset();
            return;
        }
        this._activeItem = selected;
        this._activeAcceptCharacters.clear();
        for (var _i = 0, _a = selected.item.suggestion.commitCharacters; _i < _a.length; _i++) {
            var ch = _a[_i];
            if (ch.length > 0) {
                this._activeAcceptCharacters.add(ch[0]);
            }
        }
    };
    AcceptOnCharacterOracle.prototype.reset = function () {
        this._activeItem = undefined;
    };
    AcceptOnCharacterOracle.prototype.dispose = function () {
        dispose(this._disposables);
    };
    return AcceptOnCharacterOracle;
}());
var SuggestController = /** @class */ (function () {
    function SuggestController(_editor, editorWorker, _commandService, _contextKeyService, _instantiationService) {
        var _this = this;
        this._editor = _editor;
        this._commandService = _commandService;
        this._contextKeyService = _contextKeyService;
        this._instantiationService = _instantiationService;
        this._toDispose = [];
        this._sticky = false; // for development purposes only
        this._model = new SuggestModel(this._editor, editorWorker);
        this._memory = new IdleValue(function () {
            var res = _instantiationService.createInstance(SuggestMemories, _this._editor);
            _this._toDispose.push(res);
            return res;
        });
        this._alternatives = new IdleValue(function () {
            var res = new SuggestAlternatives(_this._editor, _this._contextKeyService);
            _this._toDispose.push(res);
            return res;
        });
        this._toDispose.push(_instantiationService.createInstance(WordContextKey, _editor));
        this._toDispose.push(this._model.onDidTrigger(function (e) {
            if (!_this._widget) {
                _this._createSuggestWidget();
            }
            _this._widget.showTriggered(e.auto, e.shy ? 250 : 50);
        }));
        this._toDispose.push(this._model.onDidSuggest(function (e) {
            if (!e.shy) {
                var index = _this._memory.getValue().select(_this._editor.getModel(), _this._editor.getPosition(), e.completionModel.items);
                _this._widget.showSuggestions(e.completionModel, index, e.isFrozen, e.auto);
            }
        }));
        this._toDispose.push(this._model.onDidCancel(function (e) {
            if (_this._widget && !e.retrigger) {
                _this._widget.hideWidget();
            }
        }));
        this._toDispose.push(this._editor.onDidBlurEditorText(function () {
            if (!_this._sticky) {
                _this._model.cancel();
            }
        }));
        // Manage the acceptSuggestionsOnEnter context key
        var acceptSuggestionsOnEnter = Context.AcceptSuggestionsOnEnter.bindTo(_contextKeyService);
        var updateFromConfig = function () {
            var acceptSuggestionOnEnter = _this._editor.getConfiguration().contribInfo.acceptSuggestionOnEnter;
            acceptSuggestionsOnEnter.set(acceptSuggestionOnEnter === 'on' || acceptSuggestionOnEnter === 'smart');
        };
        this._toDispose.push(this._editor.onDidChangeConfiguration(function (e) { return updateFromConfig(); }));
        updateFromConfig();
    }
    SuggestController.get = function (editor) {
        return editor.getContribution(SuggestController.ID);
    };
    SuggestController.prototype._createSuggestWidget = function () {
        var _this = this;
        this._widget = this._instantiationService.createInstance(SuggestWidget, this._editor);
        this._toDispose.push(this._widget.onDidSelect(function (item) { return _this._onDidSelectItem(item, false, true); }, this));
        // Wire up logic to accept a suggestion on certain characters
        var autoAcceptOracle = new AcceptOnCharacterOracle(this._editor, this._widget, function (item) { return _this._onDidSelectItem(item, false, true); });
        this._toDispose.push(autoAcceptOracle, this._model.onDidSuggest(function (e) {
            if (e.completionModel.items.length === 0) {
                autoAcceptOracle.reset();
            }
        }));
        var makesTextEdit = Context.MakesTextEdit.bindTo(this._contextKeyService);
        this._toDispose.push(this._widget.onDidFocus(function (_a) {
            var item = _a.item;
            var position = _this._editor.getPosition();
            var startColumn = item.suggestion.range.startColumn;
            var endColumn = position.column;
            var value = true;
            if (_this._editor.getConfiguration().contribInfo.acceptSuggestionOnEnter === 'smart'
                && _this._model.state === 2 /* Auto */
                && !item.suggestion.command
                && !item.suggestion.additionalTextEdits
                && !(item.suggestion.insertTextRules & 4 /* InsertAsSnippet */)
                && endColumn - startColumn === item.suggestion.insertText.length) {
                var oldText = _this._editor.getModel().getValueInRange({
                    startLineNumber: position.lineNumber,
                    startColumn: startColumn,
                    endLineNumber: position.lineNumber,
                    endColumn: endColumn
                });
                value = oldText !== item.suggestion.insertText;
            }
            makesTextEdit.set(value);
        }));
        this._toDispose.push({
            dispose: function () { makesTextEdit.reset(); }
        });
    };
    SuggestController.prototype.getId = function () {
        return SuggestController.ID;
    };
    SuggestController.prototype.dispose = function () {
        this._toDispose = dispose(this._toDispose);
        if (this._widget) {
            this._widget.dispose();
            this._widget = null;
        }
        if (this._model) {
            this._model.dispose();
            this._model = null;
        }
    };
    SuggestController.prototype._onDidSelectItem = function (event, keepAlternativeSuggestions, undoStops) {
        var _this = this;
        var _a;
        if (!event || !event.item) {
            this._alternatives.getValue().reset();
            this._model.cancel();
            return;
        }
        var model = this._editor.getModel();
        var modelVersionNow = model.getAlternativeVersionId();
        var _b = event.item, suggestion = _b.suggestion, position = _b.position;
        var editorColumn = this._editor.getPosition().column;
        var columnDelta = editorColumn - position.column;
        // pushing undo stops *before* additional text edits and
        // *after* the main edit
        if (undoStops) {
            this._editor.pushUndoStop();
        }
        if (Array.isArray(suggestion.additionalTextEdits)) {
            this._editor.executeEdits('suggestController.additionalTextEdits', suggestion.additionalTextEdits.map(function (edit) { return EditOperation.replace(Range.lift(edit.range), edit.text); }));
        }
        // keep item in memory
        this._memory.getValue().memorize(model, this._editor.getPosition(), event.item);
        var insertText = suggestion.insertText;
        if (!(suggestion.insertTextRules & 4 /* InsertAsSnippet */)) {
            insertText = SnippetParser.escape(insertText);
        }
        var overwriteBefore = position.column - suggestion.range.startColumn;
        var overwriteAfter = suggestion.range.endColumn - position.column;
        SnippetController2.get(this._editor).insert(insertText, overwriteBefore + columnDelta, overwriteAfter, false, false, !(suggestion.insertTextRules & 1 /* KeepWhitespace */));
        if (undoStops) {
            this._editor.pushUndoStop();
        }
        if (!suggestion.command) {
            // done
            this._model.cancel();
        }
        else if (suggestion.command.id === TriggerSuggestAction.id) {
            // retigger
            this._model.trigger({ auto: true }, true);
        }
        else {
            // exec command, done
            (_a = this._commandService).executeCommand.apply(_a, [suggestion.command.id].concat(suggestion.command.arguments)).then(undefined, onUnexpectedError);
            this._model.cancel();
        }
        if (keepAlternativeSuggestions) {
            this._alternatives.getValue().set(event, function (next) {
                // this is not so pretty. when inserting the 'next'
                // suggestion we undo until we are at the state at
                // which we were before inserting the previous suggestion...
                while (model.canUndo()) {
                    if (modelVersionNow !== model.getAlternativeVersionId()) {
                        model.undo();
                    }
                    _this._onDidSelectItem(next, false, false);
                    break;
                }
            });
        }
        this._alertCompletionItem(event.item);
    };
    SuggestController.prototype._alertCompletionItem = function (_a) {
        var suggestion = _a.suggestion;
        var msg = localize('arai.alert.snippet', "Accepting '{0}' did insert the following text: {1}", suggestion.label, suggestion.insertText);
        alert(msg);
    };
    SuggestController.prototype.triggerSuggest = function (onlyFrom) {
        this._model.trigger({ auto: false }, false, onlyFrom);
        this._editor.revealLine(this._editor.getPosition().lineNumber, 0 /* Smooth */);
        this._editor.focus();
    };
    SuggestController.prototype.triggerSuggestAndAcceptBest = function (defaultTypeText) {
        var _this = this;
        var positionNow = this._editor.getPosition();
        var fallback = function () {
            if (positionNow.equals(_this._editor.getPosition())) {
                _this._editor.trigger('suggest', Handler.Type, { text: defaultTypeText });
            }
        };
        var makesTextEdit = function (item) {
            if (item.suggestion.insertTextRules & 4 /* InsertAsSnippet */ || item.suggestion.additionalTextEdits) {
                // snippet, other editor -> makes edit
                return true;
            }
            var position = _this._editor.getPosition();
            var startColumn = item.suggestion.range.startColumn;
            var endColumn = position.column;
            if (endColumn - startColumn !== item.suggestion.insertText.length) {
                // unequal lengths -> makes edit
                return true;
            }
            var textNow = _this._editor.getModel().getValueInRange({
                startLineNumber: position.lineNumber,
                startColumn: startColumn,
                endLineNumber: position.lineNumber,
                endColumn: endColumn
            });
            // unequal text -> makes edit
            return textNow !== item.suggestion.insertText;
        };
        once$1(this._model.onDidTrigger)(function (_) {
            // wait for trigger because only then the cancel-event is trustworthy
            var listener = [];
            anyEvent(_this._model.onDidTrigger, _this._model.onDidCancel)(function () {
                // retrigger or cancel -> try to type default text
                dispose(listener);
                fallback();
            }, undefined, listener);
            _this._model.onDidSuggest(function (_a) {
                var completionModel = _a.completionModel;
                dispose(listener);
                if (completionModel.items.length === 0) {
                    fallback();
                    return;
                }
                var index = _this._memory.getValue().select(_this._editor.getModel(), _this._editor.getPosition(), completionModel.items);
                var item = completionModel.items[index];
                if (!makesTextEdit(item)) {
                    fallback();
                    return;
                }
                _this._editor.pushUndoStop();
                _this._onDidSelectItem({ index: index, item: item, model: completionModel }, true, false);
            }, undefined, listener);
        });
        this._model.trigger({ auto: false, shy: true });
        this._editor.revealLine(positionNow.lineNumber, 0 /* Smooth */);
        this._editor.focus();
    };
    SuggestController.prototype.acceptSelectedSuggestion = function (keepAlternativeSuggestions) {
        if (this._widget) {
            var item = this._widget.getFocusedItem();
            this._onDidSelectItem(item, keepAlternativeSuggestions, true);
        }
    };
    SuggestController.prototype.acceptNextSuggestion = function () {
        this._alternatives.getValue().next();
    };
    SuggestController.prototype.acceptPrevSuggestion = function () {
        this._alternatives.getValue().prev();
    };
    SuggestController.prototype.cancelSuggestWidget = function () {
        if (this._widget) {
            this._model.cancel();
            this._widget.hideWidget();
        }
    };
    SuggestController.prototype.selectNextSuggestion = function () {
        if (this._widget) {
            this._widget.selectNext();
        }
    };
    SuggestController.prototype.selectNextPageSuggestion = function () {
        if (this._widget) {
            this._widget.selectNextPage();
        }
    };
    SuggestController.prototype.selectLastSuggestion = function () {
        if (this._widget) {
            this._widget.selectLast();
        }
    };
    SuggestController.prototype.selectPrevSuggestion = function () {
        if (this._widget) {
            this._widget.selectPrevious();
        }
    };
    SuggestController.prototype.selectPrevPageSuggestion = function () {
        if (this._widget) {
            this._widget.selectPreviousPage();
        }
    };
    SuggestController.prototype.selectFirstSuggestion = function () {
        if (this._widget) {
            this._widget.selectFirst();
        }
    };
    SuggestController.prototype.toggleSuggestionDetails = function () {
        if (this._widget) {
            this._widget.toggleDetails();
        }
    };
    SuggestController.prototype.toggleSuggestionFocus = function () {
        if (this._widget) {
            this._widget.toggleDetailsFocus();
        }
    };
    SuggestController.ID = 'editor.contrib.suggestController';
    SuggestController = __decorate$m([
        __param$j(1, IEditorWorkerService),
        __param$j(2, ICommandService),
        __param$j(3, IContextKeyService),
        __param$j(4, IInstantiationService)
    ], SuggestController);
    return SuggestController;
}());
var TriggerSuggestAction = /** @class */ (function (_super) {
    __extends$1I(TriggerSuggestAction, _super);
    function TriggerSuggestAction() {
        return _super.call(this, {
            id: TriggerSuggestAction.id,
            label: localize('suggest.trigger.label', "Trigger Suggest"),
            alias: 'Trigger Suggest',
            precondition: ContextKeyExpr.and(EditorContextKeys.writable, EditorContextKeys.hasCompletionItemProvider),
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 10 /* Space */,
                mac: { primary: 256 /* WinCtrl */ | 10 /* Space */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    TriggerSuggestAction.prototype.run = function (accessor, editor) {
        var controller = SuggestController.get(editor);
        if (!controller) {
            return;
        }
        controller.triggerSuggest();
    };
    TriggerSuggestAction.id = 'editor.action.triggerSuggest';
    return TriggerSuggestAction;
}(EditorAction));
registerEditorContribution(SuggestController);
registerEditorAction(TriggerSuggestAction);
var weight = 100 /* EditorContrib */ + 90;
var SuggestCommand = EditorCommand.bindToContribution(SuggestController.get);
registerEditorCommand(new SuggestCommand({
    id: 'acceptSelectedSuggestion',
    precondition: Context.Visible,
    handler: function (x) { return x.acceptSelectedSuggestion(true); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 2 /* Tab */
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'acceptSelectedSuggestionOnEnter',
    precondition: Context.Visible,
    handler: function (x) { return x.acceptSelectedSuggestion(false); },
    kbOpts: {
        weight: weight,
        kbExpr: ContextKeyExpr.and(EditorContextKeys.textInputFocus, Context.AcceptSuggestionsOnEnter, Context.MakesTextEdit),
        primary: 3 /* Enter */
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'hideSuggestWidget',
    precondition: Context.Visible,
    handler: function (x) { return x.cancelSuggestWidget(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 9 /* Escape */,
        secondary: [1024 /* Shift */ | 9 /* Escape */]
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'selectNextSuggestion',
    precondition: ContextKeyExpr.and(Context.Visible, Context.MultipleSuggestions),
    handler: function (c) { return c.selectNextSuggestion(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 18 /* DownArrow */,
        secondary: [2048 /* CtrlCmd */ | 18 /* DownArrow */],
        mac: { primary: 18 /* DownArrow */, secondary: [2048 /* CtrlCmd */ | 18 /* DownArrow */, 256 /* WinCtrl */ | 44 /* KEY_N */] }
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'selectNextPageSuggestion',
    precondition: ContextKeyExpr.and(Context.Visible, Context.MultipleSuggestions),
    handler: function (c) { return c.selectNextPageSuggestion(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 12 /* PageDown */,
        secondary: [2048 /* CtrlCmd */ | 12 /* PageDown */]
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'selectLastSuggestion',
    precondition: ContextKeyExpr.and(Context.Visible, Context.MultipleSuggestions),
    handler: function (c) { return c.selectLastSuggestion(); }
}));
registerEditorCommand(new SuggestCommand({
    id: 'selectPrevSuggestion',
    precondition: ContextKeyExpr.and(Context.Visible, Context.MultipleSuggestions),
    handler: function (c) { return c.selectPrevSuggestion(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 16 /* UpArrow */,
        secondary: [2048 /* CtrlCmd */ | 16 /* UpArrow */],
        mac: { primary: 16 /* UpArrow */, secondary: [2048 /* CtrlCmd */ | 16 /* UpArrow */, 256 /* WinCtrl */ | 46 /* KEY_P */] }
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'selectPrevPageSuggestion',
    precondition: ContextKeyExpr.and(Context.Visible, Context.MultipleSuggestions),
    handler: function (c) { return c.selectPrevPageSuggestion(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 11 /* PageUp */,
        secondary: [2048 /* CtrlCmd */ | 11 /* PageUp */]
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'selectFirstSuggestion',
    precondition: ContextKeyExpr.and(Context.Visible, Context.MultipleSuggestions),
    handler: function (c) { return c.selectFirstSuggestion(); }
}));
registerEditorCommand(new SuggestCommand({
    id: 'toggleSuggestionDetails',
    precondition: Context.Visible,
    handler: function (x) { return x.toggleSuggestionDetails(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 2048 /* CtrlCmd */ | 10 /* Space */,
        mac: { primary: 256 /* WinCtrl */ | 10 /* Space */ }
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'toggleSuggestionFocus',
    precondition: Context.Visible,
    handler: function (x) { return x.toggleSuggestionFocus(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 2048 /* CtrlCmd */ | 512 /* Alt */ | 10 /* Space */,
        mac: { primary: 256 /* WinCtrl */ | 512 /* Alt */ | 10 /* Space */ }
    }
}));
//#region tab completions
registerEditorCommand(new SuggestCommand({
    id: 'insertBestCompletion',
    precondition: ContextKeyExpr.and(ContextKeyExpr.equals('config.editor.tabCompletion', 'on'), WordContextKey.AtEnd, Context.Visible.toNegated(), SuggestAlternatives.OtherSuggestions.toNegated(), SnippetController2.InSnippetMode.toNegated()),
    handler: function (x) { return x.triggerSuggestAndAcceptBest('\t'); },
    kbOpts: {
        weight: weight,
        primary: 2 /* Tab */
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'insertNextSuggestion',
    precondition: ContextKeyExpr.and(ContextKeyExpr.equals('config.editor.tabCompletion', 'on'), SuggestAlternatives.OtherSuggestions, Context.Visible.toNegated(), SnippetController2.InSnippetMode.toNegated()),
    handler: function (x) { return x.acceptNextSuggestion(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 2 /* Tab */
    }
}));
registerEditorCommand(new SuggestCommand({
    id: 'insertPrevSuggestion',
    precondition: ContextKeyExpr.and(ContextKeyExpr.equals('config.editor.tabCompletion', 'on'), SuggestAlternatives.OtherSuggestions, Context.Visible.toNegated(), SnippetController2.InSnippetMode.toNegated()),
    handler: function (x) { return x.acceptPrevSuggestion(); },
    kbOpts: {
        weight: weight,
        kbExpr: EditorContextKeys.textInputFocus,
        primary: 1024 /* Shift */ | 2 /* Tab */
    }
}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1J = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$n = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$k = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var editorWordHighlight = registerColor('editor.wordHighlightBackground', { dark: '#575757B8', light: '#57575740', hc: null }, localize('wordHighlight', 'Background color of a symbol during read-access, like reading a variable. The color must not be opaque to not hide underlying decorations.'), true);
var editorWordHighlightStrong = registerColor('editor.wordHighlightStrongBackground', { dark: '#004972B8', light: '#0e639c40', hc: null }, localize('wordHighlightStrong', 'Background color of a symbol during write-access, like writing to a variable. The color must not be opaque to not hide underlying decorations.'), true);
var editorWordHighlightBorder = registerColor('editor.wordHighlightBorder', { light: null, dark: null, hc: activeContrastBorder }, localize('wordHighlightBorder', 'Border color of a symbol during read-access, like reading a variable.'));
var editorWordHighlightStrongBorder = registerColor('editor.wordHighlightStrongBorder', { light: null, dark: null, hc: activeContrastBorder }, localize('wordHighlightStrongBorder', 'Border color of a symbol during write-access, like writing to a variable.'));
var overviewRulerWordHighlightForeground = registerColor('editorOverviewRuler.wordHighlightForeground', { dark: '#A0A0A0CC', light: '#A0A0A0CC', hc: '#A0A0A0CC' }, localize('overviewRulerWordHighlightForeground', 'Overview ruler marker color for symbol highlights. The color must not be opaque to not hide underlying decorations.'), true);
var overviewRulerWordHighlightStrongForeground = registerColor('editorOverviewRuler.wordHighlightStrongForeground', { dark: '#C0A0C0CC', light: '#C0A0C0CC', hc: '#C0A0C0CC' }, localize('overviewRulerWordHighlightStrongForeground', 'Overview ruler marker color for write-access symbol highlights. The color must not be opaque to not hide underlying decorations.'), true);
var ctxHasWordHighlights = new RawContextKey('hasWordHighlights', false);
function getOccurrencesAtPosition(model, position, token) {
    var orderedByScore = DocumentHighlightProviderRegistry.ordered(model);
    // in order of score ask the occurrences provider
    // until someone response with a good result
    // (good = none empty array)
    return first(orderedByScore.map(function (provider) { return function () {
        return Promise.resolve(provider.provideDocumentHighlights(model, position, token))
            .then(undefined, onUnexpectedExternalError);
    }; }), function (result) { return !isFalsyOrEmpty(result); });
}
var OccurenceAtPositionRequest = /** @class */ (function () {
    function OccurenceAtPositionRequest(model, selection, wordSeparators) {
        var _this = this;
        this._wordRange = this._getCurrentWordRange(model, selection);
        this.result = createCancelablePromise(function (token) { return _this._compute(model, selection, wordSeparators, token); });
    }
    OccurenceAtPositionRequest.prototype._getCurrentWordRange = function (model, selection) {
        var word = model.getWordAtPosition(selection.getPosition());
        if (word) {
            return new Range(selection.startLineNumber, word.startColumn, selection.startLineNumber, word.endColumn);
        }
        return null;
    };
    OccurenceAtPositionRequest.prototype.isValid = function (model, selection, decorationIds) {
        var lineNumber = selection.startLineNumber;
        var startColumn = selection.startColumn;
        var endColumn = selection.endColumn;
        var currentWordRange = this._getCurrentWordRange(model, selection);
        var requestIsValid = Boolean(this._wordRange && this._wordRange.equalsRange(currentWordRange));
        // Even if we are on a different word, if that word is in the decorations ranges, the request is still valid
        // (Same symbol)
        for (var i = 0, len = decorationIds.length; !requestIsValid && i < len; i++) {
            var range$$1 = model.getDecorationRange(decorationIds[i]);
            if (range$$1 && range$$1.startLineNumber === lineNumber) {
                if (range$$1.startColumn <= startColumn && range$$1.endColumn >= endColumn) {
                    requestIsValid = true;
                }
            }
        }
        return requestIsValid;
    };
    OccurenceAtPositionRequest.prototype.cancel = function () {
        this.result.cancel();
    };
    return OccurenceAtPositionRequest;
}());
var SemanticOccurenceAtPositionRequest = /** @class */ (function (_super) {
    __extends$1J(SemanticOccurenceAtPositionRequest, _super);
    function SemanticOccurenceAtPositionRequest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SemanticOccurenceAtPositionRequest.prototype._compute = function (model, selection, wordSeparators, token) {
        return getOccurrencesAtPosition(model, selection.getPosition(), token);
    };
    return SemanticOccurenceAtPositionRequest;
}(OccurenceAtPositionRequest));
var TextualOccurenceAtPositionRequest = /** @class */ (function (_super) {
    __extends$1J(TextualOccurenceAtPositionRequest, _super);
    function TextualOccurenceAtPositionRequest(model, selection, wordSeparators) {
        var _this = _super.call(this, model, selection, wordSeparators) || this;
        _this._selectionIsEmpty = selection.isEmpty();
        return _this;
    }
    TextualOccurenceAtPositionRequest.prototype._compute = function (model, selection, wordSeparators, token) {
        return timeout(250, token).then(function () {
            if (!selection.isEmpty()) {
                return [];
            }
            var word = model.getWordAtPosition(selection.getPosition());
            if (!word) {
                return [];
            }
            var matches = model.findMatches(word.word, true, false, true, wordSeparators, false);
            return matches.map(function (m) {
                return {
                    range: m.range,
                    kind: DocumentHighlightKind.Text
                };
            });
        });
    };
    TextualOccurenceAtPositionRequest.prototype.isValid = function (model, selection, decorationIds) {
        var currentSelectionIsEmpty = selection.isEmpty();
        if (this._selectionIsEmpty !== currentSelectionIsEmpty) {
            return false;
        }
        return _super.prototype.isValid.call(this, model, selection, decorationIds);
    };
    return TextualOccurenceAtPositionRequest;
}(OccurenceAtPositionRequest));
function computeOccurencesAtPosition(model, selection, wordSeparators) {
    if (DocumentHighlightProviderRegistry.has(model)) {
        return new SemanticOccurenceAtPositionRequest(model, selection, wordSeparators);
    }
    return new TextualOccurenceAtPositionRequest(model, selection, wordSeparators);
}
registerDefaultLanguageCommand('_executeDocumentHighlights', function (model, position) { return getOccurrencesAtPosition(model, position, CancellationToken.None); });
var WordHighlighter = /** @class */ (function () {
    function WordHighlighter(editor, contextKeyService) {
        var _this = this;
        this.workerRequestTokenId = 0;
        this.workerRequestCompleted = false;
        this.workerRequestValue = [];
        this.lastCursorPositionChangeTime = 0;
        this.renderDecorationsTimer = -1;
        this.editor = editor;
        this._hasWordHighlights = ctxHasWordHighlights.bindTo(contextKeyService);
        this._ignorePositionChangeEvent = false;
        this.occurrencesHighlight = this.editor.getConfiguration().contribInfo.occurrencesHighlight;
        this.model = this.editor.getModel();
        this.toUnhook = [];
        this.toUnhook.push(editor.onDidChangeCursorPosition(function (e) {
            if (_this._ignorePositionChangeEvent) {
                // We are changing the position => ignore this event
                return;
            }
            if (!_this.occurrencesHighlight) {
                // Early exit if nothing needs to be done!
                // Leave some form of early exit check here if you wish to continue being a cursor position change listener ;)
                return;
            }
            _this._onPositionChanged(e);
        }));
        this.toUnhook.push(editor.onDidChangeModelContent(function (e) {
            _this._stopAll();
        }));
        this.toUnhook.push(editor.onDidChangeConfiguration(function (e) {
            var newValue = _this.editor.getConfiguration().contribInfo.occurrencesHighlight;
            if (_this.occurrencesHighlight !== newValue) {
                _this.occurrencesHighlight = newValue;
                _this._stopAll();
            }
        }));
        this._decorationIds = [];
        this.workerRequestTokenId = 0;
        this.workerRequest = null;
        this.workerRequestCompleted = false;
        this.lastCursorPositionChangeTime = 0;
        this.renderDecorationsTimer = -1;
    }
    WordHighlighter.prototype.hasDecorations = function () {
        return (this._decorationIds.length > 0);
    };
    WordHighlighter.prototype.restore = function () {
        if (!this.occurrencesHighlight) {
            return;
        }
        this._run();
    };
    WordHighlighter.prototype._getSortedHighlights = function () {
        var _this = this;
        return coalesce(this._decorationIds
            .map(function (id) { return _this.model.getDecorationRange(id); })
            .sort(Range.compareRangesUsingStarts));
    };
    WordHighlighter.prototype.moveNext = function () {
        var _this = this;
        var highlights = this._getSortedHighlights();
        var index = firstIndex(highlights, function (range$$1) { return range$$1.containsPosition(_this.editor.getPosition()); });
        var newIndex = ((index + 1) % highlights.length);
        var dest = highlights[newIndex];
        try {
            this._ignorePositionChangeEvent = true;
            this.editor.setPosition(dest.getStartPosition());
            this.editor.revealRangeInCenterIfOutsideViewport(dest);
        }
        finally {
            this._ignorePositionChangeEvent = false;
        }
    };
    WordHighlighter.prototype.moveBack = function () {
        var _this = this;
        var highlights = this._getSortedHighlights();
        var index = firstIndex(highlights, function (range$$1) { return range$$1.containsPosition(_this.editor.getPosition()); });
        var newIndex = ((index - 1 + highlights.length) % highlights.length);
        var dest = highlights[newIndex];
        try {
            this._ignorePositionChangeEvent = true;
            this.editor.setPosition(dest.getStartPosition());
            this.editor.revealRangeInCenterIfOutsideViewport(dest);
        }
        finally {
            this._ignorePositionChangeEvent = false;
        }
    };
    WordHighlighter.prototype._removeDecorations = function () {
        if (this._decorationIds.length > 0) {
            // remove decorations
            this._decorationIds = this.editor.deltaDecorations(this._decorationIds, []);
            this._hasWordHighlights.set(false);
        }
    };
    WordHighlighter.prototype._stopAll = function () {
        // Remove any existing decorations
        this._removeDecorations();
        // Cancel any renderDecorationsTimer
        if (this.renderDecorationsTimer !== -1) {
            clearTimeout(this.renderDecorationsTimer);
            this.renderDecorationsTimer = -1;
        }
        // Cancel any worker request
        if (this.workerRequest !== null) {
            this.workerRequest.cancel();
            this.workerRequest = null;
        }
        // Invalidate any worker request callback
        if (!this.workerRequestCompleted) {
            this.workerRequestTokenId++;
            this.workerRequestCompleted = true;
        }
    };
    WordHighlighter.prototype._onPositionChanged = function (e) {
        // disabled
        if (!this.occurrencesHighlight) {
            this._stopAll();
            return;
        }
        // ignore typing & other
        if (e.reason !== 3 /* Explicit */) {
            this._stopAll();
            return;
        }
        this._run();
    };
    WordHighlighter.prototype._run = function () {
        var _this = this;
        var editorSelection = this.editor.getSelection();
        // ignore multiline selection
        if (editorSelection.startLineNumber !== editorSelection.endLineNumber) {
            this._stopAll();
            return;
        }
        var lineNumber = editorSelection.startLineNumber;
        var startColumn = editorSelection.startColumn;
        var endColumn = editorSelection.endColumn;
        var word = this.model.getWordAtPosition({
            lineNumber: lineNumber,
            column: startColumn
        });
        // The selection must be inside a word or surround one word at most
        if (!word || word.startColumn > startColumn || word.endColumn < endColumn) {
            this._stopAll();
            return;
        }
        // All the effort below is trying to achieve this:
        // - when cursor is moved to a word, trigger immediately a findOccurrences request
        // - 250ms later after the last cursor move event, render the occurrences
        // - no flickering!
        var workerRequestIsValid = (this.workerRequest && this.workerRequest.isValid(this.model, editorSelection, this._decorationIds));
        // There are 4 cases:
        // a) old workerRequest is valid & completed, renderDecorationsTimer fired
        // b) old workerRequest is valid & completed, renderDecorationsTimer not fired
        // c) old workerRequest is valid, but not completed
        // d) old workerRequest is not valid
        // For a) no action is needed
        // For c), member 'lastCursorPositionChangeTime' will be used when installing the timer so no action is needed
        this.lastCursorPositionChangeTime = (new Date()).getTime();
        if (workerRequestIsValid) {
            if (this.workerRequestCompleted && this.renderDecorationsTimer !== -1) {
                // case b)
                // Delay the firing of renderDecorationsTimer by an extra 250 ms
                clearTimeout(this.renderDecorationsTimer);
                this.renderDecorationsTimer = -1;
                this._beginRenderDecorations();
            }
        }
        else {
            // case d)
            // Stop all previous actions and start fresh
            this._stopAll();
            var myRequestId_1 = ++this.workerRequestTokenId;
            this.workerRequestCompleted = false;
            this.workerRequest = computeOccurencesAtPosition(this.model, this.editor.getSelection(), this.editor.getConfiguration().wordSeparators);
            this.workerRequest.result.then(function (data) {
                if (myRequestId_1 === _this.workerRequestTokenId) {
                    _this.workerRequestCompleted = true;
                    _this.workerRequestValue = data || [];
                    _this._beginRenderDecorations();
                }
            }, onUnexpectedError);
        }
    };
    WordHighlighter.prototype._beginRenderDecorations = function () {
        var _this = this;
        var currentTime = (new Date()).getTime();
        var minimumRenderTime = this.lastCursorPositionChangeTime + 250;
        if (currentTime >= minimumRenderTime) {
            // Synchronous
            this.renderDecorationsTimer = -1;
            this.renderDecorations();
        }
        else {
            // Asynchronous
            this.renderDecorationsTimer = setTimeout(function () {
                _this.renderDecorations();
            }, (minimumRenderTime - currentTime));
        }
    };
    WordHighlighter.prototype.renderDecorations = function () {
        this.renderDecorationsTimer = -1;
        var decorations = [];
        for (var i = 0, len = this.workerRequestValue.length; i < len; i++) {
            var info = this.workerRequestValue[i];
            decorations.push({
                range: info.range,
                options: WordHighlighter._getDecorationOptions(info.kind)
            });
        }
        this._decorationIds = this.editor.deltaDecorations(this._decorationIds, decorations);
        this._hasWordHighlights.set(this.hasDecorations());
    };
    WordHighlighter._getDecorationOptions = function (kind) {
        if (kind === DocumentHighlightKind.Write) {
            return this._WRITE_OPTIONS;
        }
        else if (kind === DocumentHighlightKind.Text) {
            return this._TEXT_OPTIONS;
        }
        else {
            return this._REGULAR_OPTIONS;
        }
    };
    WordHighlighter.prototype.dispose = function () {
        this._stopAll();
        this.toUnhook = dispose(this.toUnhook);
    };
    WordHighlighter._WRITE_OPTIONS = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        className: 'wordHighlightStrong',
        overviewRuler: {
            color: themeColorFromId(overviewRulerWordHighlightStrongForeground),
            position: OverviewRulerLane.Center
        }
    });
    WordHighlighter._TEXT_OPTIONS = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        className: 'selectionHighlight',
        overviewRuler: {
            color: themeColorFromId(overviewRulerSelectionHighlightForeground),
            position: OverviewRulerLane.Center
        }
    });
    WordHighlighter._REGULAR_OPTIONS = ModelDecorationOptions.register({
        stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
        className: 'wordHighlight',
        overviewRuler: {
            color: themeColorFromId(overviewRulerWordHighlightForeground),
            position: OverviewRulerLane.Center
        }
    });
    return WordHighlighter;
}());
var WordHighlighterContribution = /** @class */ (function (_super) {
    __extends$1J(WordHighlighterContribution, _super);
    function WordHighlighterContribution(editor, contextKeyService) {
        var _this = _super.call(this) || this;
        var createWordHighlighterIfPossible = function () {
            if (editor.hasModel()) {
                _this.wordHighligher = new WordHighlighter(editor, contextKeyService);
            }
        };
        _this._register(editor.onDidChangeModel(function (e) {
            if (_this.wordHighligher) {
                _this.wordHighligher.dispose();
                _this.wordHighligher = null;
            }
            createWordHighlighterIfPossible();
        }));
        createWordHighlighterIfPossible();
        return _this;
    }
    WordHighlighterContribution.get = function (editor) {
        return editor.getContribution(WordHighlighterContribution.ID);
    };
    WordHighlighterContribution.prototype.getId = function () {
        return WordHighlighterContribution.ID;
    };
    WordHighlighterContribution.prototype.saveViewState = function () {
        if (this.wordHighligher && this.wordHighligher.hasDecorations()) {
            return true;
        }
        return false;
    };
    WordHighlighterContribution.prototype.moveNext = function () {
        if (this.wordHighligher) {
            this.wordHighligher.moveNext();
        }
    };
    WordHighlighterContribution.prototype.moveBack = function () {
        if (this.wordHighligher) {
            this.wordHighligher.moveBack();
        }
    };
    WordHighlighterContribution.prototype.restoreViewState = function (state) {
        if (this.wordHighligher && state) {
            this.wordHighligher.restore();
        }
    };
    WordHighlighterContribution.prototype.dispose = function () {
        if (this.wordHighligher) {
            this.wordHighligher.dispose();
            this.wordHighligher = null;
        }
        _super.prototype.dispose.call(this);
    };
    WordHighlighterContribution.ID = 'editor.contrib.wordHighlighter';
    WordHighlighterContribution = __decorate$n([
        __param$k(1, IContextKeyService)
    ], WordHighlighterContribution);
    return WordHighlighterContribution;
}(Disposable));
var WordHighlightNavigationAction = /** @class */ (function (_super) {
    __extends$1J(WordHighlightNavigationAction, _super);
    function WordHighlightNavigationAction(next, opts) {
        var _this = _super.call(this, opts) || this;
        _this._isNext = next;
        return _this;
    }
    WordHighlightNavigationAction.prototype.run = function (accessor, editor) {
        var controller = WordHighlighterContribution.get(editor);
        if (!controller) {
            return;
        }
        if (this._isNext) {
            controller.moveNext();
        }
        else {
            controller.moveBack();
        }
    };
    return WordHighlightNavigationAction;
}(EditorAction));
var NextWordHighlightAction = /** @class */ (function (_super) {
    __extends$1J(NextWordHighlightAction, _super);
    function NextWordHighlightAction() {
        return _super.call(this, true, {
            id: 'editor.action.wordHighlight.next',
            label: localize('wordHighlight.next.label', "Go to Next Symbol Highlight"),
            alias: 'Go to Next Symbol Highlight',
            precondition: ctxHasWordHighlights,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 65 /* F7 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return NextWordHighlightAction;
}(WordHighlightNavigationAction));
var PrevWordHighlightAction = /** @class */ (function (_super) {
    __extends$1J(PrevWordHighlightAction, _super);
    function PrevWordHighlightAction() {
        return _super.call(this, false, {
            id: 'editor.action.wordHighlight.prev',
            label: localize('wordHighlight.previous.label', "Go to Previous Symbol Highlight"),
            alias: 'Go to Previous Symbol Highlight',
            precondition: ctxHasWordHighlights,
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 1024 /* Shift */ | 65 /* F7 */,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return PrevWordHighlightAction;
}(WordHighlightNavigationAction));
var TriggerWordHighlightAction = /** @class */ (function (_super) {
    __extends$1J(TriggerWordHighlightAction, _super);
    function TriggerWordHighlightAction() {
        return _super.call(this, {
            id: 'editor.action.wordHighlight.trigger',
            label: localize('wordHighlight.trigger.label', "Trigger Symbol Highlight"),
            alias: 'Trigger Symbol Highlight',
            precondition: ctxHasWordHighlights.toNegated(),
            kbOpts: {
                kbExpr: EditorContextKeys.editorTextFocus,
                primary: 0,
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    TriggerWordHighlightAction.prototype.run = function (accessor, editor, args) {
        var controller = WordHighlighterContribution.get(editor);
        if (!controller) {
            return;
        }
        controller.restoreViewState(true);
    };
    return TriggerWordHighlightAction;
}(EditorAction));
registerEditorContribution(WordHighlighterContribution);
registerEditorAction(NextWordHighlightAction);
registerEditorAction(PrevWordHighlightAction);
registerEditorAction(TriggerWordHighlightAction);
registerThemingParticipant(function (theme, collector) {
    var selectionHighlight = theme.getColor(editorSelectionHighlight);
    if (selectionHighlight) {
        collector.addRule(".monaco-editor .focused .selectionHighlight { background-color: " + selectionHighlight + "; }");
        collector.addRule(".monaco-editor .selectionHighlight { background-color: " + selectionHighlight.transparent(0.5) + "; }");
    }
    var wordHighlight = theme.getColor(editorWordHighlight);
    if (wordHighlight) {
        collector.addRule(".monaco-editor .wordHighlight { background-color: " + wordHighlight + "; }");
    }
    var wordHighlightStrong = theme.getColor(editorWordHighlightStrong);
    if (wordHighlightStrong) {
        collector.addRule(".monaco-editor .wordHighlightStrong { background-color: " + wordHighlightStrong + "; }");
    }
    var selectionHighlightBorder = theme.getColor(editorSelectionHighlightBorder);
    if (selectionHighlightBorder) {
        collector.addRule(".monaco-editor .selectionHighlight { border: 1px " + (theme.type === 'hc' ? 'dotted' : 'solid') + " " + selectionHighlightBorder + "; box-sizing: border-box; }");
    }
    var wordHighlightBorder = theme.getColor(editorWordHighlightBorder);
    if (wordHighlightBorder) {
        collector.addRule(".monaco-editor .wordHighlight { border: 1px " + (theme.type === 'hc' ? 'dashed' : 'solid') + " " + wordHighlightBorder + "; box-sizing: border-box; }");
    }
    var wordHighlightStrongBorder = theme.getColor(editorWordHighlightStrongBorder);
    if (wordHighlightStrongBorder) {
        collector.addRule(".monaco-editor .wordHighlightStrong { border: 1px " + (theme.type === 'hc' ? 'dashed' : 'solid') + " " + wordHighlightStrongBorder + "; box-sizing: border-box; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1K = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var MoveWordCommand = /** @class */ (function (_super) {
    __extends$1K(MoveWordCommand, _super);
    function MoveWordCommand(opts) {
        var _this = _super.call(this, opts) || this;
        _this._inSelectionMode = opts.inSelectionMode;
        _this._wordNavigationType = opts.wordNavigationType;
        return _this;
    }
    MoveWordCommand.prototype.runEditorCommand = function (accessor, editor, args) {
        var _this = this;
        if (!editor.hasModel()) {
            return;
        }
        var config = editor.getConfiguration();
        var wordSeparators = getMapForWordSeparators(config.wordSeparators);
        var model = editor.getModel();
        var selections = editor.getSelections();
        var result = selections.map(function (sel) {
            var inPosition = new Position(sel.positionLineNumber, sel.positionColumn);
            var outPosition = _this._move(wordSeparators, model, inPosition, _this._wordNavigationType);
            return _this._moveTo(sel, outPosition, _this._inSelectionMode);
        });
        editor._getCursors().setStates('moveWordCommand', 0 /* NotSet */, result.map(function (r) { return CursorState.fromModelSelection(r); }));
        if (result.length === 1) {
            var pos = new Position(result[0].positionLineNumber, result[0].positionColumn);
            editor.revealPosition(pos, 0 /* Smooth */);
        }
    };
    MoveWordCommand.prototype._moveTo = function (from, to, inSelectionMode) {
        if (inSelectionMode) {
            // move just position
            return new Selection(from.selectionStartLineNumber, from.selectionStartColumn, to.lineNumber, to.column);
        }
        else {
            // move everything
            return new Selection(to.lineNumber, to.column, to.lineNumber, to.column);
        }
    };
    return MoveWordCommand;
}(EditorCommand));
var WordLeftCommand = /** @class */ (function (_super) {
    __extends$1K(WordLeftCommand, _super);
    function WordLeftCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WordLeftCommand.prototype._move = function (wordSeparators, model, position, wordNavigationType) {
        return WordOperations.moveWordLeft(wordSeparators, model, position, wordNavigationType);
    };
    return WordLeftCommand;
}(MoveWordCommand));
var WordRightCommand = /** @class */ (function (_super) {
    __extends$1K(WordRightCommand, _super);
    function WordRightCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WordRightCommand.prototype._move = function (wordSeparators, model, position, wordNavigationType) {
        return WordOperations.moveWordRight(wordSeparators, model, position, wordNavigationType);
    };
    return WordRightCommand;
}(MoveWordCommand));
var CursorWordStartLeft = /** @class */ (function (_super) {
    __extends$1K(CursorWordStartLeft, _super);
    function CursorWordStartLeft() {
        return _super.call(this, {
            inSelectionMode: false,
            wordNavigationType: 0 /* WordStart */,
            id: 'cursorWordStartLeft',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 15 /* LeftArrow */,
                mac: { primary: 512 /* Alt */ | 15 /* LeftArrow */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return CursorWordStartLeft;
}(WordLeftCommand));
var CursorWordEndLeft = /** @class */ (function (_super) {
    __extends$1K(CursorWordEndLeft, _super);
    function CursorWordEndLeft() {
        return _super.call(this, {
            inSelectionMode: false,
            wordNavigationType: 2 /* WordEnd */,
            id: 'cursorWordEndLeft',
            precondition: null
        }) || this;
    }
    return CursorWordEndLeft;
}(WordLeftCommand));
var CursorWordLeft = /** @class */ (function (_super) {
    __extends$1K(CursorWordLeft, _super);
    function CursorWordLeft() {
        return _super.call(this, {
            inSelectionMode: false,
            wordNavigationType: 1 /* WordStartFast */,
            id: 'cursorWordLeft',
            precondition: null
        }) || this;
    }
    return CursorWordLeft;
}(WordLeftCommand));
var CursorWordStartLeftSelect = /** @class */ (function (_super) {
    __extends$1K(CursorWordStartLeftSelect, _super);
    function CursorWordStartLeftSelect() {
        return _super.call(this, {
            inSelectionMode: true,
            wordNavigationType: 0 /* WordStart */,
            id: 'cursorWordStartLeftSelect',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 15 /* LeftArrow */,
                mac: { primary: 512 /* Alt */ | 1024 /* Shift */ | 15 /* LeftArrow */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return CursorWordStartLeftSelect;
}(WordLeftCommand));
var CursorWordEndLeftSelect = /** @class */ (function (_super) {
    __extends$1K(CursorWordEndLeftSelect, _super);
    function CursorWordEndLeftSelect() {
        return _super.call(this, {
            inSelectionMode: true,
            wordNavigationType: 2 /* WordEnd */,
            id: 'cursorWordEndLeftSelect',
            precondition: null
        }) || this;
    }
    return CursorWordEndLeftSelect;
}(WordLeftCommand));
var CursorWordLeftSelect = /** @class */ (function (_super) {
    __extends$1K(CursorWordLeftSelect, _super);
    function CursorWordLeftSelect() {
        return _super.call(this, {
            inSelectionMode: true,
            wordNavigationType: 0 /* WordStart */,
            id: 'cursorWordLeftSelect',
            precondition: null
        }) || this;
    }
    return CursorWordLeftSelect;
}(WordLeftCommand));
var CursorWordStartRight = /** @class */ (function (_super) {
    __extends$1K(CursorWordStartRight, _super);
    function CursorWordStartRight() {
        return _super.call(this, {
            inSelectionMode: false,
            wordNavigationType: 0 /* WordStart */,
            id: 'cursorWordStartRight',
            precondition: null
        }) || this;
    }
    return CursorWordStartRight;
}(WordRightCommand));
var CursorWordEndRight = /** @class */ (function (_super) {
    __extends$1K(CursorWordEndRight, _super);
    function CursorWordEndRight() {
        return _super.call(this, {
            inSelectionMode: false,
            wordNavigationType: 2 /* WordEnd */,
            id: 'cursorWordEndRight',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 17 /* RightArrow */,
                mac: { primary: 512 /* Alt */ | 17 /* RightArrow */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return CursorWordEndRight;
}(WordRightCommand));
var CursorWordRight = /** @class */ (function (_super) {
    __extends$1K(CursorWordRight, _super);
    function CursorWordRight() {
        return _super.call(this, {
            inSelectionMode: false,
            wordNavigationType: 2 /* WordEnd */,
            id: 'cursorWordRight',
            precondition: null
        }) || this;
    }
    return CursorWordRight;
}(WordRightCommand));
var CursorWordStartRightSelect = /** @class */ (function (_super) {
    __extends$1K(CursorWordStartRightSelect, _super);
    function CursorWordStartRightSelect() {
        return _super.call(this, {
            inSelectionMode: true,
            wordNavigationType: 0 /* WordStart */,
            id: 'cursorWordStartRightSelect',
            precondition: null
        }) || this;
    }
    return CursorWordStartRightSelect;
}(WordRightCommand));
var CursorWordEndRightSelect = /** @class */ (function (_super) {
    __extends$1K(CursorWordEndRightSelect, _super);
    function CursorWordEndRightSelect() {
        return _super.call(this, {
            inSelectionMode: true,
            wordNavigationType: 2 /* WordEnd */,
            id: 'cursorWordEndRightSelect',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 17 /* RightArrow */,
                mac: { primary: 512 /* Alt */ | 1024 /* Shift */ | 17 /* RightArrow */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return CursorWordEndRightSelect;
}(WordRightCommand));
var CursorWordRightSelect = /** @class */ (function (_super) {
    __extends$1K(CursorWordRightSelect, _super);
    function CursorWordRightSelect() {
        return _super.call(this, {
            inSelectionMode: true,
            wordNavigationType: 2 /* WordEnd */,
            id: 'cursorWordRightSelect',
            precondition: null
        }) || this;
    }
    return CursorWordRightSelect;
}(WordRightCommand));
var DeleteWordCommand = /** @class */ (function (_super) {
    __extends$1K(DeleteWordCommand, _super);
    function DeleteWordCommand(opts) {
        var _this = _super.call(this, opts) || this;
        _this._whitespaceHeuristics = opts.whitespaceHeuristics;
        _this._wordNavigationType = opts.wordNavigationType;
        return _this;
    }
    DeleteWordCommand.prototype.runEditorCommand = function (accessor, editor, args) {
        var _this = this;
        if (!editor.hasModel()) {
            return;
        }
        var config = editor.getConfiguration();
        var wordSeparators = getMapForWordSeparators(config.wordSeparators);
        var model = editor.getModel();
        var selections = editor.getSelections();
        var commands = selections.map(function (sel) {
            var deleteRange = _this._delete(wordSeparators, model, sel, _this._whitespaceHeuristics, _this._wordNavigationType);
            return new ReplaceCommand(deleteRange, '');
        });
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    };
    return DeleteWordCommand;
}(EditorCommand));
var DeleteWordLeftCommand = /** @class */ (function (_super) {
    __extends$1K(DeleteWordLeftCommand, _super);
    function DeleteWordLeftCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DeleteWordLeftCommand.prototype._delete = function (wordSeparators, model, selection, whitespaceHeuristics, wordNavigationType) {
        var r = WordOperations.deleteWordLeft(wordSeparators, model, selection, whitespaceHeuristics, wordNavigationType);
        if (r) {
            return r;
        }
        return new Range(1, 1, 1, 1);
    };
    return DeleteWordLeftCommand;
}(DeleteWordCommand));
var DeleteWordRightCommand = /** @class */ (function (_super) {
    __extends$1K(DeleteWordRightCommand, _super);
    function DeleteWordRightCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DeleteWordRightCommand.prototype._delete = function (wordSeparators, model, selection, whitespaceHeuristics, wordNavigationType) {
        var r = WordOperations.deleteWordRight(wordSeparators, model, selection, whitespaceHeuristics, wordNavigationType);
        if (r) {
            return r;
        }
        var lineCount = model.getLineCount();
        var maxColumn = model.getLineMaxColumn(lineCount);
        return new Range(lineCount, maxColumn, lineCount, maxColumn);
    };
    return DeleteWordRightCommand;
}(DeleteWordCommand));
var DeleteWordStartLeft = /** @class */ (function (_super) {
    __extends$1K(DeleteWordStartLeft, _super);
    function DeleteWordStartLeft() {
        return _super.call(this, {
            whitespaceHeuristics: false,
            wordNavigationType: 0 /* WordStart */,
            id: 'deleteWordStartLeft',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return DeleteWordStartLeft;
}(DeleteWordLeftCommand));
var DeleteWordEndLeft = /** @class */ (function (_super) {
    __extends$1K(DeleteWordEndLeft, _super);
    function DeleteWordEndLeft() {
        return _super.call(this, {
            whitespaceHeuristics: false,
            wordNavigationType: 2 /* WordEnd */,
            id: 'deleteWordEndLeft',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return DeleteWordEndLeft;
}(DeleteWordLeftCommand));
var DeleteWordLeft = /** @class */ (function (_super) {
    __extends$1K(DeleteWordLeft, _super);
    function DeleteWordLeft() {
        return _super.call(this, {
            whitespaceHeuristics: true,
            wordNavigationType: 0 /* WordStart */,
            id: 'deleteWordLeft',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 1 /* Backspace */,
                mac: { primary: 512 /* Alt */ | 1 /* Backspace */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return DeleteWordLeft;
}(DeleteWordLeftCommand));
var DeleteWordStartRight = /** @class */ (function (_super) {
    __extends$1K(DeleteWordStartRight, _super);
    function DeleteWordStartRight() {
        return _super.call(this, {
            whitespaceHeuristics: false,
            wordNavigationType: 0 /* WordStart */,
            id: 'deleteWordStartRight',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return DeleteWordStartRight;
}(DeleteWordRightCommand));
var DeleteWordEndRight = /** @class */ (function (_super) {
    __extends$1K(DeleteWordEndRight, _super);
    function DeleteWordEndRight() {
        return _super.call(this, {
            whitespaceHeuristics: false,
            wordNavigationType: 2 /* WordEnd */,
            id: 'deleteWordEndRight',
            precondition: EditorContextKeys.writable
        }) || this;
    }
    return DeleteWordEndRight;
}(DeleteWordRightCommand));
var DeleteWordRight = /** @class */ (function (_super) {
    __extends$1K(DeleteWordRight, _super);
    function DeleteWordRight() {
        return _super.call(this, {
            whitespaceHeuristics: true,
            wordNavigationType: 2 /* WordEnd */,
            id: 'deleteWordRight',
            precondition: EditorContextKeys.writable,
            kbOpts: {
                kbExpr: EditorContextKeys.textInputFocus,
                primary: 2048 /* CtrlCmd */ | 20 /* Delete */,
                mac: { primary: 512 /* Alt */ | 20 /* Delete */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    return DeleteWordRight;
}(DeleteWordRightCommand));
registerEditorCommand(new CursorWordStartLeft());
registerEditorCommand(new CursorWordEndLeft());
registerEditorCommand(new CursorWordLeft());
registerEditorCommand(new CursorWordStartLeftSelect());
registerEditorCommand(new CursorWordEndLeftSelect());
registerEditorCommand(new CursorWordLeftSelect());
registerEditorCommand(new CursorWordStartRight());
registerEditorCommand(new CursorWordEndRight());
registerEditorCommand(new CursorWordRight());
registerEditorCommand(new CursorWordStartRightSelect());
registerEditorCommand(new CursorWordEndRightSelect());
registerEditorCommand(new CursorWordRightSelect());
registerEditorCommand(new DeleteWordStartLeft());
registerEditorCommand(new DeleteWordEndLeft());
registerEditorCommand(new DeleteWordLeft());
registerEditorCommand(new DeleteWordStartRight());
registerEditorCommand(new DeleteWordEndRight());
registerEditorCommand(new DeleteWordRight());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1L = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ToggleTabFocusModeAction = /** @class */ (function (_super) {
    __extends$1L(ToggleTabFocusModeAction, _super);
    function ToggleTabFocusModeAction() {
        return _super.call(this, {
            id: ToggleTabFocusModeAction.ID,
            label: localize({ key: 'toggle.tabMovesFocus', comment: ['Turn on/off use of tab key for moving focus around VS Code'] }, "Toggle Tab Key Moves Focus"),
            alias: 'Toggle Tab Key Moves Focus',
            precondition: null,
            kbOpts: {
                kbExpr: null,
                primary: 2048 /* CtrlCmd */ | 43 /* KEY_M */,
                mac: { primary: 256 /* WinCtrl */ | 1024 /* Shift */ | 43 /* KEY_M */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    ToggleTabFocusModeAction.prototype.run = function (accessor, editor) {
        var oldValue = TabFocus.getTabFocusMode();
        var newValue = !oldValue;
        TabFocus.setTabFocusMode(newValue);
        if (newValue) {
            alert(localize('toggle.tabMovesFocus.on', "Pressing Tab will now move focus to the next focusable element"));
        }
        else {
            alert(localize('toggle.tabMovesFocus.off', "Pressing Tab will now insert the tab character"));
        }
    };
    ToggleTabFocusModeAction.ID = 'editor.action.toggleTabFocusMode';
    return ToggleTabFocusModeAction;
}(EditorAction));
registerEditorAction(ToggleTabFocusModeAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1M = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$o = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$l = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var CONTEXT_ACCESSIBILITY_WIDGET_VISIBLE = new RawContextKey('accessibilityHelpWidgetVisible', false);
var AccessibilityHelpController = /** @class */ (function (_super) {
    __extends$1M(AccessibilityHelpController, _super);
    function AccessibilityHelpController(editor, instantiationService) {
        var _this = _super.call(this) || this;
        _this._editor = editor;
        _this._widget = _this._register(instantiationService.createInstance(AccessibilityHelpWidget, _this._editor));
        return _this;
    }
    AccessibilityHelpController.get = function (editor) {
        return editor.getContribution(AccessibilityHelpController.ID);
    };
    AccessibilityHelpController.prototype.getId = function () {
        return AccessibilityHelpController.ID;
    };
    AccessibilityHelpController.prototype.show = function () {
        this._widget.show();
    };
    AccessibilityHelpController.prototype.hide = function () {
        this._widget.hide();
    };
    AccessibilityHelpController.ID = 'editor.contrib.accessibilityHelpController';
    AccessibilityHelpController = __decorate$o([
        __param$l(1, IInstantiationService)
    ], AccessibilityHelpController);
    return AccessibilityHelpController;
}(Disposable));
var nlsNoSelection = localize("noSelection", "No selection");
var nlsSingleSelectionRange = localize("singleSelectionRange", "Line {0}, Column {1} ({2} selected)");
var nlsSingleSelection = localize("singleSelection", "Line {0}, Column {1}");
var nlsMultiSelectionRange = localize("multiSelectionRange", "{0} selections ({1} characters selected)");
var nlsMultiSelection = localize("multiSelection", "{0} selections");
function getSelectionLabel(selections, charactersSelected) {
    if (!selections || selections.length === 0) {
        return nlsNoSelection;
    }
    if (selections.length === 1) {
        if (charactersSelected) {
            return format(nlsSingleSelectionRange, selections[0].positionLineNumber, selections[0].positionColumn, charactersSelected);
        }
        return format(nlsSingleSelection, selections[0].positionLineNumber, selections[0].positionColumn);
    }
    if (charactersSelected) {
        return format(nlsMultiSelectionRange, selections.length, charactersSelected);
    }
    if (selections.length > 0) {
        return format(nlsMultiSelection, selections.length);
    }
    return null;
}
var AccessibilityHelpWidget = /** @class */ (function (_super) {
    __extends$1M(AccessibilityHelpWidget, _super);
    function AccessibilityHelpWidget(editor, _contextKeyService, _keybindingService, _openerService) {
        var _this = _super.call(this) || this;
        _this._contextKeyService = _contextKeyService;
        _this._keybindingService = _keybindingService;
        _this._openerService = _openerService;
        _this._editor = editor;
        _this._isVisibleKey = CONTEXT_ACCESSIBILITY_WIDGET_VISIBLE.bindTo(_this._contextKeyService);
        _this._domNode = createFastDomNode(document.createElement('div'));
        _this._domNode.setClassName('accessibilityHelpWidget');
        _this._domNode.setDisplay('none');
        _this._domNode.setAttribute('role', 'dialog');
        _this._domNode.setAttribute('aria-hidden', 'true');
        _this._contentDomNode = createFastDomNode(document.createElement('div'));
        _this._contentDomNode.setAttribute('role', 'document');
        _this._domNode.appendChild(_this._contentDomNode);
        _this._isVisible = false;
        _this._register(_this._editor.onDidLayoutChange(function () {
            if (_this._isVisible) {
                _this._layout();
            }
        }));
        // Intentionally not configurable!
        _this._register(addStandardDisposableListener(_this._contentDomNode.domNode, 'keydown', function (e) {
            if (!_this._isVisible) {
                return;
            }
            if (e.equals(2048 /* CtrlCmd */ | 35 /* KEY_E */)) {
                alert(localize("emergencyConfOn", "Now changing the setting `accessibilitySupport` to 'on'."));
                _this._editor.updateOptions({
                    accessibilitySupport: 'on'
                });
                clearNode(_this._contentDomNode.domNode);
                _this._buildContent();
                _this._contentDomNode.domNode.focus();
                e.preventDefault();
                e.stopPropagation();
            }
            if (e.equals(2048 /* CtrlCmd */ | 38 /* KEY_H */)) {
                alert(localize("openingDocs", "Now opening the Editor Accessibility documentation page."));
                var url = _this._editor.getRawConfiguration().accessibilityHelpUrl;
                if (typeof url === 'undefined') {
                    url = 'https://go.microsoft.com/fwlink/?linkid=852450';
                }
                _this._openerService.open(URI.parse(url));
                e.preventDefault();
                e.stopPropagation();
            }
        }));
        _this.onblur(_this._contentDomNode.domNode, function () {
            _this.hide();
        });
        _this._editor.addOverlayWidget(_this);
        return _this;
    }
    AccessibilityHelpWidget.prototype.dispose = function () {
        this._editor.removeOverlayWidget(this);
        _super.prototype.dispose.call(this);
    };
    AccessibilityHelpWidget.prototype.getId = function () {
        return AccessibilityHelpWidget.ID;
    };
    AccessibilityHelpWidget.prototype.getDomNode = function () {
        return this._domNode.domNode;
    };
    AccessibilityHelpWidget.prototype.getPosition = function () {
        return {
            preference: null
        };
    };
    AccessibilityHelpWidget.prototype.show = function () {
        if (this._isVisible) {
            return;
        }
        this._isVisible = true;
        this._isVisibleKey.set(true);
        this._layout();
        this._domNode.setDisplay('block');
        this._domNode.setAttribute('aria-hidden', 'false');
        this._contentDomNode.domNode.tabIndex = 0;
        this._buildContent();
        this._contentDomNode.domNode.focus();
    };
    AccessibilityHelpWidget.prototype._descriptionForCommand = function (commandId, msg, noKbMsg) {
        var kb = this._keybindingService.lookupKeybinding(commandId);
        if (kb) {
            return format(msg, kb.getAriaLabel());
        }
        return format(noKbMsg, commandId);
    };
    AccessibilityHelpWidget.prototype._buildContent = function () {
        var opts = this._editor.getConfiguration();
        var selections = this._editor.getSelections();
        var charactersSelected = 0;
        if (selections) {
            var model_1 = this._editor.getModel();
            if (model_1) {
                selections.forEach(function (selection) {
                    charactersSelected += model_1.getValueLengthInRange(selection);
                });
            }
        }
        var text = getSelectionLabel(selections, charactersSelected);
        if (opts.wrappingInfo.inDiffEditor) {
            if (opts.readOnly) {
                text += localize("readonlyDiffEditor", " in a read-only pane of a diff editor.");
            }
            else {
                text += localize("editableDiffEditor", " in a pane of a diff editor.");
            }
        }
        else {
            if (opts.readOnly) {
                text += localize("readonlyEditor", " in a read-only code editor");
            }
            else {
                text += localize("editableEditor", " in a code editor");
            }
        }
        var turnOnMessage = (isMacintosh
            ? localize("changeConfigToOnMac", "To configure the editor to be optimized for usage with a Screen Reader press Command+E now.")
            : localize("changeConfigToOnWinLinux", "To configure the editor to be optimized for usage with a Screen Reader press Control+E now."));
        switch (opts.accessibilitySupport) {
            case 0 /* Unknown */:
                text += '\n\n - ' + turnOnMessage;
                break;
            case 2 /* Enabled */:
                text += '\n\n - ' + localize("auto_on", "The editor is configured to be optimized for usage with a Screen Reader.");
                break;
            case 1 /* Disabled */:
                text += '\n\n - ' + localize("auto_off", "The editor is configured to never be optimized for usage with a Screen Reader, which is not the case at this time.");
                text += ' ' + turnOnMessage;
                break;
        }
        var NLS_TAB_FOCUS_MODE_ON = localize("tabFocusModeOnMsg", "Pressing Tab in the current editor will move focus to the next focusable element. Toggle this behavior by pressing {0}.");
        var NLS_TAB_FOCUS_MODE_ON_NO_KB = localize("tabFocusModeOnMsgNoKb", "Pressing Tab in the current editor will move focus to the next focusable element. The command {0} is currently not triggerable by a keybinding.");
        var NLS_TAB_FOCUS_MODE_OFF = localize("tabFocusModeOffMsg", "Pressing Tab in the current editor will insert the tab character. Toggle this behavior by pressing {0}.");
        var NLS_TAB_FOCUS_MODE_OFF_NO_KB = localize("tabFocusModeOffMsgNoKb", "Pressing Tab in the current editor will insert the tab character. The command {0} is currently not triggerable by a keybinding.");
        if (opts.tabFocusMode) {
            text += '\n\n - ' + this._descriptionForCommand(ToggleTabFocusModeAction.ID, NLS_TAB_FOCUS_MODE_ON, NLS_TAB_FOCUS_MODE_ON_NO_KB);
        }
        else {
            text += '\n\n - ' + this._descriptionForCommand(ToggleTabFocusModeAction.ID, NLS_TAB_FOCUS_MODE_OFF, NLS_TAB_FOCUS_MODE_OFF_NO_KB);
        }
        var openDocMessage = (isMacintosh
            ? localize("openDocMac", "Press Command+H now to open a browser window with more information related to editor accessibility.")
            : localize("openDocWinLinux", "Press Control+H now to open a browser window with more information related to editor accessibility."));
        text += '\n\n - ' + openDocMessage;
        text += '\n\n' + localize("outroMsg", "You can dismiss this tooltip and return to the editor by pressing Escape or Shift+Escape.");
        this._contentDomNode.domNode.appendChild(renderFormattedText(text));
        // Per https://www.w3.org/TR/wai-aria/roles#document, Authors SHOULD provide a title or label for documents
        this._contentDomNode.domNode.setAttribute('aria-label', text);
    };
    AccessibilityHelpWidget.prototype.hide = function () {
        if (!this._isVisible) {
            return;
        }
        this._isVisible = false;
        this._isVisibleKey.reset();
        this._domNode.setDisplay('none');
        this._domNode.setAttribute('aria-hidden', 'true');
        this._contentDomNode.domNode.tabIndex = -1;
        clearNode(this._contentDomNode.domNode);
        this._editor.focus();
    };
    AccessibilityHelpWidget.prototype._layout = function () {
        var editorLayout = this._editor.getLayoutInfo();
        var w = Math.max(5, Math.min(AccessibilityHelpWidget.WIDTH, editorLayout.width - 40));
        var h = Math.max(5, Math.min(AccessibilityHelpWidget.HEIGHT, editorLayout.height - 40));
        this._domNode.setWidth(w);
        this._domNode.setHeight(h);
        var top = Math.round((editorLayout.height - h) / 2);
        this._domNode.setTop(top);
        var left = Math.round((editorLayout.width - w) / 2);
        this._domNode.setLeft(left);
    };
    AccessibilityHelpWidget.ID = 'editor.contrib.accessibilityHelpWidget';
    AccessibilityHelpWidget.WIDTH = 500;
    AccessibilityHelpWidget.HEIGHT = 300;
    AccessibilityHelpWidget = __decorate$o([
        __param$l(1, IContextKeyService),
        __param$l(2, IKeybindingService),
        __param$l(3, IOpenerService)
    ], AccessibilityHelpWidget);
    return AccessibilityHelpWidget;
}(Widget$1));
var ShowAccessibilityHelpAction = /** @class */ (function (_super) {
    __extends$1M(ShowAccessibilityHelpAction, _super);
    function ShowAccessibilityHelpAction() {
        return _super.call(this, {
            id: 'editor.action.showAccessibilityHelp',
            label: localize("ShowAccessibilityHelpAction", "Show Accessibility Help"),
            alias: 'Show Accessibility Help',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: (isIE ? 2048 /* CtrlCmd */ | 59 /* F1 */ : 512 /* Alt */ | 59 /* F1 */),
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    ShowAccessibilityHelpAction.prototype.run = function (accessor, editor) {
        var controller = AccessibilityHelpController.get(editor);
        if (controller) {
            controller.show();
        }
    };
    return ShowAccessibilityHelpAction;
}(EditorAction));
registerEditorContribution(AccessibilityHelpController);
registerEditorAction(ShowAccessibilityHelpAction);
var AccessibilityHelpCommand = EditorCommand.bindToContribution(AccessibilityHelpController.get);
registerEditorCommand(new AccessibilityHelpCommand({
    id: 'closeAccessibilityHelp',
    precondition: CONTEXT_ACCESSIBILITY_WIDGET_VISIBLE,
    handler: function (x) { return x.hide(); },
    kbOpts: {
        weight: 100 /* EditorContrib */ + 100,
        kbExpr: EditorContextKeys.focus,
        primary: 9 /* Escape */,
        secondary: [1024 /* Shift */ | 9 /* Escape */]
    }
}));
registerThemingParticipant(function (theme, collector) {
    var widgetBackground = theme.getColor(editorWidgetBackground);
    if (widgetBackground) {
        collector.addRule(".monaco-editor .accessibilityHelpWidget { background-color: " + widgetBackground + "; }");
    }
    var widgetShadowColor = theme.getColor(widgetShadow);
    if (widgetShadowColor) {
        collector.addRule(".monaco-editor .accessibilityHelpWidget { box-shadow: 0 2px 8px " + widgetShadowColor + "; }");
    }
    var hcBorder = theme.getColor(contrastBorder);
    if (hcBorder) {
        collector.addRule(".monaco-editor .accessibilityHelpWidget { border: 2px solid " + hcBorder + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IStandaloneThemeService = createDecorator('themeService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1N = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$p = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$m = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var InspectTokensController = /** @class */ (function (_super) {
    __extends$1N(InspectTokensController, _super);
    function InspectTokensController(editor, standaloneColorService, modeService) {
        var _this = _super.call(this) || this;
        _this._editor = editor;
        _this._modeService = modeService;
        _this._widget = null;
        _this._register(_this._editor.onDidChangeModel(function (e) { return _this.stop(); }));
        _this._register(_this._editor.onDidChangeModelLanguage(function (e) { return _this.stop(); }));
        _this._register(TokenizationRegistry.onDidChange(function (e) { return _this.stop(); }));
        return _this;
    }
    InspectTokensController.get = function (editor) {
        return editor.getContribution(InspectTokensController.ID);
    };
    InspectTokensController.prototype.getId = function () {
        return InspectTokensController.ID;
    };
    InspectTokensController.prototype.dispose = function () {
        this.stop();
        _super.prototype.dispose.call(this);
    };
    InspectTokensController.prototype.launch = function () {
        if (this._widget) {
            return;
        }
        if (!this._editor.hasModel()) {
            return;
        }
        this._widget = new InspectTokensWidget(this._editor, this._modeService);
    };
    InspectTokensController.prototype.stop = function () {
        if (this._widget) {
            this._widget.dispose();
            this._widget = null;
        }
    };
    InspectTokensController.ID = 'editor.contrib.inspectTokens';
    InspectTokensController = __decorate$p([
        __param$m(1, IStandaloneThemeService),
        __param$m(2, IModeService)
    ], InspectTokensController);
    return InspectTokensController;
}(Disposable));
var InspectTokens = /** @class */ (function (_super) {
    __extends$1N(InspectTokens, _super);
    function InspectTokens() {
        return _super.call(this, {
            id: 'editor.action.inspectTokens',
            label: localize('inspectTokens', "Developer: Inspect Tokens"),
            alias: 'Developer: Inspect Tokens',
            precondition: null
        }) || this;
    }
    InspectTokens.prototype.run = function (accessor, editor) {
        var controller = InspectTokensController.get(editor);
        if (controller) {
            controller.launch();
        }
    };
    return InspectTokens;
}(EditorAction));
function renderTokenText(tokenText) {
    var result = '';
    for (var charIndex = 0, len = tokenText.length; charIndex < len; charIndex++) {
        var charCode = tokenText.charCodeAt(charIndex);
        switch (charCode) {
            case 9 /* Tab */:
                result += '&rarr;';
                break;
            case 32 /* Space */:
                result += '&middot;';
                break;
            case 60 /* LessThan */:
                result += '&lt;';
                break;
            case 62 /* GreaterThan */:
                result += '&gt;';
                break;
            case 38 /* Ampersand */:
                result += '&amp;';
                break;
            default:
                result += String.fromCharCode(charCode);
        }
    }
    return result;
}
function getSafeTokenizationSupport(languageIdentifier) {
    var tokenizationSupport = TokenizationRegistry.get(languageIdentifier.language);
    if (tokenizationSupport) {
        return tokenizationSupport;
    }
    return {
        getInitialState: function () { return NULL_STATE; },
        tokenize: function (line, state, deltaOffset) { return nullTokenize(languageIdentifier.language, line, state, deltaOffset); },
        tokenize2: function (line, state, deltaOffset) { return nullTokenize2(languageIdentifier.id, line, state, deltaOffset); }
    };
}
var InspectTokensWidget = /** @class */ (function (_super) {
    __extends$1N(InspectTokensWidget, _super);
    function InspectTokensWidget(editor, modeService) {
        var _this = _super.call(this) || this;
        // Editor.IContentWidget.allowEditorOverflow
        _this.allowEditorOverflow = true;
        _this._editor = editor;
        _this._modeService = modeService;
        _this._model = _this._editor.getModel();
        _this._domNode = document.createElement('div');
        _this._domNode.className = 'tokens-inspect-widget';
        _this._tokenizationSupport = getSafeTokenizationSupport(_this._model.getLanguageIdentifier());
        _this._compute(_this._editor.getPosition());
        _this._register(_this._editor.onDidChangeCursorPosition(function (e) { return _this._compute(_this._editor.getPosition()); }));
        _this._editor.addContentWidget(_this);
        return _this;
    }
    InspectTokensWidget.prototype.dispose = function () {
        this._editor.removeContentWidget(this);
        _super.prototype.dispose.call(this);
    };
    InspectTokensWidget.prototype.getId = function () {
        return InspectTokensWidget._ID;
    };
    InspectTokensWidget.prototype._compute = function (position) {
        var data = this._getTokensAtLine(position.lineNumber);
        var token1Index = 0;
        for (var i = data.tokens1.length - 1; i >= 0; i--) {
            var t = data.tokens1[i];
            if (position.column - 1 >= t.offset) {
                token1Index = i;
                break;
            }
        }
        var token2Index = 0;
        for (var i = (data.tokens2.length >>> 1); i >= 0; i--) {
            if (position.column - 1 >= data.tokens2[(i << 1)]) {
                token2Index = i;
                break;
            }
        }
        var result = '';
        var lineContent = this._model.getLineContent(position.lineNumber);
        var tokenText = '';
        if (token1Index < data.tokens1.length) {
            var tokenStartIndex = data.tokens1[token1Index].offset;
            var tokenEndIndex = token1Index + 1 < data.tokens1.length ? data.tokens1[token1Index + 1].offset : lineContent.length;
            tokenText = lineContent.substring(tokenStartIndex, tokenEndIndex);
        }
        result += "<h2 class=\"tm-token\">" + renderTokenText(tokenText) + "<span class=\"tm-token-length\">(" + tokenText.length + " " + (tokenText.length === 1 ? 'char' : 'chars') + ")</span></h2>";
        result += "<hr class=\"tokens-inspect-separator\" style=\"clear:both\"/>";
        var metadata = this._decodeMetadata(data.tokens2[(token2Index << 1) + 1]);
        result += "<table class=\"tm-metadata-table\"><tbody>";
        result += "<tr><td class=\"tm-metadata-key\">language</td><td class=\"tm-metadata-value\">" + escape(metadata.languageIdentifier.language) + "</td>";
        result += "<tr><td class=\"tm-metadata-key\">token type</td><td class=\"tm-metadata-value\">" + this._tokenTypeToString(metadata.tokenType) + "</td>";
        result += "<tr><td class=\"tm-metadata-key\">font style</td><td class=\"tm-metadata-value\">" + this._fontStyleToString(metadata.fontStyle) + "</td>";
        result += "<tr><td class=\"tm-metadata-key\">foreground</td><td class=\"tm-metadata-value\">" + Color.Format.CSS.formatHex(metadata.foreground) + "</td>";
        result += "<tr><td class=\"tm-metadata-key\">background</td><td class=\"tm-metadata-value\">" + Color.Format.CSS.formatHex(metadata.background) + "</td>";
        result += "</tbody></table>";
        result += "<hr class=\"tokens-inspect-separator\"/>";
        if (token1Index < data.tokens1.length) {
            result += "<span class=\"tm-token-type\">" + escape(data.tokens1[token1Index].type) + "</span>";
        }
        this._domNode.innerHTML = result;
        this._editor.layoutContentWidget(this);
    };
    InspectTokensWidget.prototype._decodeMetadata = function (metadata) {
        var colorMap = TokenizationRegistry.getColorMap();
        var languageId = TokenMetadata.getLanguageId(metadata);
        var tokenType = TokenMetadata.getTokenType(metadata);
        var fontStyle = TokenMetadata.getFontStyle(metadata);
        var foreground$$1 = TokenMetadata.getForeground(metadata);
        var background = TokenMetadata.getBackground(metadata);
        return {
            languageIdentifier: this._modeService.getLanguageIdentifier(languageId),
            tokenType: tokenType,
            fontStyle: fontStyle,
            foreground: colorMap[foreground$$1],
            background: colorMap[background]
        };
    };
    InspectTokensWidget.prototype._tokenTypeToString = function (tokenType) {
        switch (tokenType) {
            case 0 /* Other */: return 'Other';
            case 1 /* Comment */: return 'Comment';
            case 2 /* String */: return 'String';
            case 4 /* RegEx */: return 'RegEx';
        }
        return '??';
    };
    InspectTokensWidget.prototype._fontStyleToString = function (fontStyle) {
        var r = '';
        if (fontStyle & 1 /* Italic */) {
            r += 'italic ';
        }
        if (fontStyle & 2 /* Bold */) {
            r += 'bold ';
        }
        if (fontStyle & 4 /* Underline */) {
            r += 'underline ';
        }
        if (r.length === 0) {
            r = '---';
        }
        return r;
    };
    InspectTokensWidget.prototype._getTokensAtLine = function (lineNumber) {
        var stateBeforeLine = this._getStateBeforeLine(lineNumber);
        var tokenizationResult1 = this._tokenizationSupport.tokenize(this._model.getLineContent(lineNumber), stateBeforeLine, 0);
        var tokenizationResult2 = this._tokenizationSupport.tokenize2(this._model.getLineContent(lineNumber), stateBeforeLine, 0);
        return {
            startState: stateBeforeLine,
            tokens1: tokenizationResult1.tokens,
            tokens2: tokenizationResult2.tokens,
            endState: tokenizationResult1.endState
        };
    };
    InspectTokensWidget.prototype._getStateBeforeLine = function (lineNumber) {
        var state = this._tokenizationSupport.getInitialState();
        for (var i = 1; i < lineNumber; i++) {
            var tokenizationResult = this._tokenizationSupport.tokenize(this._model.getLineContent(i), state, 0);
            state = tokenizationResult.endState;
        }
        return state;
    };
    InspectTokensWidget.prototype.getDomNode = function () {
        return this._domNode;
    };
    InspectTokensWidget.prototype.getPosition = function () {
        return {
            position: this._editor.getPosition(),
            preference: [2 /* BELOW */, 1 /* ABOVE */]
        };
    };
    InspectTokensWidget._ID = 'editor.contrib.inspectTokensWidget';
    return InspectTokensWidget;
}(Disposable));
registerEditorContribution(InspectTokensController);
registerEditorAction(InspectTokens);
registerThemingParticipant(function (theme, collector) {
    var border = theme.getColor(editorHoverBorder);
    if (border) {
        var borderWidth = theme.type === HIGH_CONTRAST ? 2 : 1;
        collector.addRule(".monaco-editor .tokens-inspect-widget { border: " + borderWidth + "px solid " + border + "; }");
        collector.addRule(".monaco-editor .tokens-inspect-widget .tokens-inspect-separator { background-color: " + border + "; }");
    }
    var background = theme.getColor(editorHoverBackground);
    if (background) {
        collector.addRule(".monaco-editor .tokens-inspect-widget { background-color: " + background + "; }");
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IPadShowKeyboard = /** @class */ (function () {
    function IPadShowKeyboard(editor) {
        var _this = this;
        this.editor = editor;
        this.toDispose = [];
        if (isIPad) {
            this.toDispose.push(editor.onDidChangeConfiguration(function () { return _this.update(); }));
            this.update();
        }
    }
    IPadShowKeyboard.prototype.update = function () {
        var shouldHaveWidget = (!this.editor.getConfiguration().readOnly);
        if (!this.widget && shouldHaveWidget) {
            this.widget = new ShowKeyboardWidget(this.editor);
        }
        else if (this.widget && !shouldHaveWidget) {
            this.widget.dispose();
            this.widget = null;
        }
    };
    IPadShowKeyboard.prototype.getId = function () {
        return IPadShowKeyboard.ID;
    };
    IPadShowKeyboard.prototype.dispose = function () {
        this.toDispose = dispose(this.toDispose);
        if (this.widget) {
            this.widget.dispose();
            this.widget = null;
        }
    };
    IPadShowKeyboard.ID = 'editor.contrib.iPadShowKeyboard';
    return IPadShowKeyboard;
}());
var ShowKeyboardWidget = /** @class */ (function () {
    function ShowKeyboardWidget(editor) {
        var _this = this;
        this.editor = editor;
        this._domNode = document.createElement('textarea');
        this._domNode.className = 'iPadShowKeyboard';
        this._toDispose = [];
        this._toDispose.push(addDisposableListener(this._domNode, 'touchstart', function (e) {
            _this.editor.focus();
        }));
        this._toDispose.push(addDisposableListener(this._domNode, 'focus', function (e) {
            _this.editor.focus();
        }));
        this.editor.addOverlayWidget(this);
    }
    ShowKeyboardWidget.prototype.dispose = function () {
        this.editor.removeOverlayWidget(this);
        this._toDispose = dispose(this._toDispose);
    };
    // ----- IOverlayWidget API
    ShowKeyboardWidget.prototype.getId = function () {
        return ShowKeyboardWidget.ID;
    };
    ShowKeyboardWidget.prototype.getDomNode = function () {
        return this._domNode;
    };
    ShowKeyboardWidget.prototype.getPosition = function () {
        return {
            preference: 1 /* BOTTOM_RIGHT_CORNER */
        };
    };
    ShowKeyboardWidget.ID = 'editor.contrib.ShowKeyboardWidget';
    return ShowKeyboardWidget;
}());
registerEditorContribution(IPadShowKeyboard);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ModifierLabelProvider = /** @class */ (function () {
    function ModifierLabelProvider(mac, windows, linux) {
        if (linux === void 0) { linux = windows; }
        this.modifierLabels = [null]; // index 0 will never me accessed.
        this.modifierLabels[2 /* Macintosh */] = mac;
        this.modifierLabels[1 /* Windows */] = windows;
        this.modifierLabels[3 /* Linux */] = linux;
    }
    ModifierLabelProvider.prototype.toLabel = function (firstPartMod, firstPartKey, chordPartMod, chordPartKey, OS) {
        if (firstPartMod === null || firstPartKey === null) {
            return null;
        }
        return _asString(firstPartMod, firstPartKey, chordPartMod, chordPartKey, this.modifierLabels[OS]);
    };
    return ModifierLabelProvider;
}());
/**
 * A label provider that prints modifiers in a suitable format for displaying in the UI.
 */
var UILabelProvider = new ModifierLabelProvider({
    ctrlKey: '⌃',
    shiftKey: '⇧',
    altKey: '⌥',
    metaKey: '⌘',
    separator: '',
}, {
    ctrlKey: localize({ key: 'ctrlKey', comment: ['This is the short form for the Control key on the keyboard'] }, "Ctrl"),
    shiftKey: localize({ key: 'shiftKey', comment: ['This is the short form for the Shift key on the keyboard'] }, "Shift"),
    altKey: localize({ key: 'altKey', comment: ['This is the short form for the Alt key on the keyboard'] }, "Alt"),
    metaKey: localize({ key: 'windowsKey', comment: ['This is the short form for the Windows key on the keyboard'] }, "Windows"),
    separator: '+',
}, {
    ctrlKey: localize({ key: 'ctrlKey', comment: ['This is the short form for the Control key on the keyboard'] }, "Ctrl"),
    shiftKey: localize({ key: 'shiftKey', comment: ['This is the short form for the Shift key on the keyboard'] }, "Shift"),
    altKey: localize({ key: 'altKey', comment: ['This is the short form for the Alt key on the keyboard'] }, "Alt"),
    metaKey: localize({ key: 'superKey', comment: ['This is the short form for the Super key on the keyboard'] }, "Super"),
    separator: '+',
});
/**
 * A label provider that prints modifiers in a suitable format for ARIA.
 */
var AriaLabelProvider = new ModifierLabelProvider({
    ctrlKey: localize({ key: 'ctrlKey.long', comment: ['This is the long form for the Control key on the keyboard'] }, "Control"),
    shiftKey: localize({ key: 'shiftKey.long', comment: ['This is the long form for the Shift key on the keyboard'] }, "Shift"),
    altKey: localize({ key: 'altKey.long', comment: ['This is the long form for the Alt key on the keyboard'] }, "Alt"),
    metaKey: localize({ key: 'cmdKey.long', comment: ['This is the long form for the Command key on the keyboard'] }, "Command"),
    separator: '+',
}, {
    ctrlKey: localize({ key: 'ctrlKey.long', comment: ['This is the long form for the Control key on the keyboard'] }, "Control"),
    shiftKey: localize({ key: 'shiftKey.long', comment: ['This is the long form for the Shift key on the keyboard'] }, "Shift"),
    altKey: localize({ key: 'altKey.long', comment: ['This is the long form for the Alt key on the keyboard'] }, "Alt"),
    metaKey: localize({ key: 'windowsKey.long', comment: ['This is the long form for the Windows key on the keyboard'] }, "Windows"),
    separator: '+',
}, {
    ctrlKey: localize({ key: 'ctrlKey.long', comment: ['This is the long form for the Control key on the keyboard'] }, "Control"),
    shiftKey: localize({ key: 'shiftKey.long', comment: ['This is the long form for the Shift key on the keyboard'] }, "Shift"),
    altKey: localize({ key: 'altKey.long', comment: ['This is the long form for the Alt key on the keyboard'] }, "Alt"),
    metaKey: localize({ key: 'superKey.long', comment: ['This is the long form for the Super key on the keyboard'] }, "Super"),
    separator: '+',
});
function _simpleAsString(modifiers, key, labels) {
    if (key === null) {
        return '';
    }
    var result = [];
    // translate modifier keys: Ctrl-Shift-Alt-Meta
    if (modifiers.ctrlKey) {
        result.push(labels.ctrlKey);
    }
    if (modifiers.shiftKey) {
        result.push(labels.shiftKey);
    }
    if (modifiers.altKey) {
        result.push(labels.altKey);
    }
    if (modifiers.metaKey) {
        result.push(labels.metaKey);
    }
    // the actual key
    result.push(key);
    return result.join(labels.separator);
}
function _asString(firstPartMod, firstPartKey, chordPartMod, chordPartKey, labels) {
    var result = _simpleAsString(firstPartMod, firstPartKey, labels);
    if (chordPartMod !== null && chordPartKey !== null) {
        result += ' ';
        result += _simpleAsString(chordPartMod, chordPartKey, labels);
    }
    return result;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var $$4 = $;
var KeybindingLabel = /** @class */ (function () {
    function KeybindingLabel(container, os) {
        this.os = os;
        this.domNode = append(container, $$4('.monaco-keybinding'));
        this.didEverRender = false;
        container.appendChild(this.domNode);
    }
    KeybindingLabel.prototype.set = function (keybinding, matches) {
        if (this.didEverRender && this.keybinding === keybinding && KeybindingLabel.areSame(this.matches, matches)) {
            return;
        }
        this.keybinding = keybinding;
        this.matches = matches;
        this.render();
    };
    KeybindingLabel.prototype.render = function () {
        clearNode(this.domNode);
        if (this.keybinding) {
            var _a = this.keybinding.getParts(), firstPart = _a[0], chordPart = _a[1];
            if (firstPart) {
                this.renderPart(this.domNode, firstPart, this.matches ? this.matches.firstPart : null);
            }
            if (chordPart) {
                append(this.domNode, $$4('span.monaco-keybinding-key-chord-separator', undefined, ' '));
                this.renderPart(this.domNode, chordPart, this.matches ? this.matches.chordPart : null);
            }
            this.domNode.title = this.keybinding.getAriaLabel() || '';
        }
        this.didEverRender = true;
    };
    KeybindingLabel.prototype.renderPart = function (parent, part, match) {
        var modifierLabels = UILabelProvider.modifierLabels[this.os];
        if (part.ctrlKey) {
            this.renderKey(parent, modifierLabels.ctrlKey, Boolean(match && match.ctrlKey), modifierLabels.separator);
        }
        if (part.shiftKey) {
            this.renderKey(parent, modifierLabels.shiftKey, Boolean(match && match.shiftKey), modifierLabels.separator);
        }
        if (part.altKey) {
            this.renderKey(parent, modifierLabels.altKey, Boolean(match && match.altKey), modifierLabels.separator);
        }
        if (part.metaKey) {
            this.renderKey(parent, modifierLabels.metaKey, Boolean(match && match.metaKey), modifierLabels.separator);
        }
        var keyLabel = part.keyLabel;
        if (keyLabel) {
            this.renderKey(parent, keyLabel, Boolean(match && match.keyCode), '');
        }
    };
    KeybindingLabel.prototype.renderKey = function (parent, label, highlight, separator) {
        append(parent, $$4('span.monaco-keybinding-key' + (highlight ? '.highlight' : ''), undefined, label));
        if (separator) {
            append(parent, $$4('span.monaco-keybinding-key-separator', undefined, separator));
        }
    };
    KeybindingLabel.prototype.dispose = function () {
    };
    KeybindingLabel.areSame = function (a, b) {
        if (a === b || (!a && !b)) {
            return true;
        }
        return !!a && !!b && equals$1(a.firstPart, b.firstPart) && equals$1(a.chordPart, b.chordPart);
    };
    return KeybindingLabel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1O = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var IDS = 0;
var QuickOpenEntry = /** @class */ (function () {
    function QuickOpenEntry(highlights) {
        if (highlights === void 0) { highlights = []; }
        this.id = (IDS++).toString();
        this.labelHighlights = highlights;
        this.descriptionHighlights = [];
    }
    /**
     * A unique identifier for the entry
     */
    QuickOpenEntry.prototype.getId = function () {
        return this.id;
    };
    /**
     * The label of the entry to identify it from others in the list
     */
    QuickOpenEntry.prototype.getLabel = function () {
        return null;
    };
    /**
     * The options for the label to use for this entry
     */
    QuickOpenEntry.prototype.getLabelOptions = function () {
        return null;
    };
    /**
     * The label of the entry to use when a screen reader wants to read about the entry
     */
    QuickOpenEntry.prototype.getAriaLabel = function () {
        return [this.getLabel(), this.getDescription(), this.getDetail()]
            .filter(function (s) { return !!s; })
            .join(', ');
    };
    /**
     * Detail information about the entry that is optional and can be shown below the label
     */
    QuickOpenEntry.prototype.getDetail = function () {
        return null;
    };
    /**
     * The icon of the entry to identify it from others in the list
     */
    QuickOpenEntry.prototype.getIcon = function () {
        return null;
    };
    /**
     * A secondary description that is optional and can be shown right to the label
     */
    QuickOpenEntry.prototype.getDescription = function () {
        return null;
    };
    /**
     * A tooltip to show when hovering over the entry.
     */
    QuickOpenEntry.prototype.getTooltip = function () {
        return null;
    };
    /**
     * A tooltip to show when hovering over the description portion of the entry.
     */
    QuickOpenEntry.prototype.getDescriptionTooltip = function () {
        return null;
    };
    /**
     * An optional keybinding to show for an entry.
     */
    QuickOpenEntry.prototype.getKeybinding = function () {
        return null;
    };
    /**
     * Allows to reuse the same model while filtering. Hidden entries will not show up in the viewer.
     */
    QuickOpenEntry.prototype.isHidden = function () {
        return this.hidden;
    };
    /**
     * Allows to set highlight ranges that should show up for the entry label and optionally description if set.
     */
    QuickOpenEntry.prototype.setHighlights = function (labelHighlights, descriptionHighlights, detailHighlights) {
        this.labelHighlights = labelHighlights;
        this.descriptionHighlights = descriptionHighlights;
        this.detailHighlights = detailHighlights;
    };
    /**
     * Allows to return highlight ranges that should show up for the entry label and description.
     */
    QuickOpenEntry.prototype.getHighlights = function () {
        return [this.labelHighlights, this.descriptionHighlights, this.detailHighlights];
    };
    /**
     * Called when the entry is selected for opening. Returns a boolean value indicating if an action was performed or not.
     * The mode parameter gives an indication if the element is previewed (using arrow keys) or opened.
     *
     * The context parameter provides additional context information how the run was triggered.
     */
    QuickOpenEntry.prototype.run = function (mode, context) {
        return false;
    };
    return QuickOpenEntry;
}());
var QuickOpenEntryGroup = /** @class */ (function (_super) {
    __extends$1O(QuickOpenEntryGroup, _super);
    function QuickOpenEntryGroup(entry, groupLabel, withBorder) {
        var _this = _super.call(this) || this;
        _this.entry = entry;
        _this.groupLabel = groupLabel;
        _this.withBorder = withBorder;
        return _this;
    }
    /**
     * The label of the group or null if none.
     */
    QuickOpenEntryGroup.prototype.getGroupLabel = function () {
        return this.groupLabel;
    };
    QuickOpenEntryGroup.prototype.setGroupLabel = function (groupLabel) {
        this.groupLabel = groupLabel;
    };
    /**
     * Whether to show a border on top of the group entry or not.
     */
    QuickOpenEntryGroup.prototype.showBorder = function () {
        return this.withBorder;
    };
    QuickOpenEntryGroup.prototype.setShowBorder = function (showBorder) {
        this.withBorder = showBorder;
    };
    QuickOpenEntryGroup.prototype.getLabel = function () {
        return this.entry ? this.entry.getLabel() : _super.prototype.getLabel.call(this);
    };
    QuickOpenEntryGroup.prototype.getLabelOptions = function () {
        return this.entry ? this.entry.getLabelOptions() : _super.prototype.getLabelOptions.call(this);
    };
    QuickOpenEntryGroup.prototype.getAriaLabel = function () {
        return this.entry ? this.entry.getAriaLabel() : _super.prototype.getAriaLabel.call(this);
    };
    QuickOpenEntryGroup.prototype.getDetail = function () {
        return this.entry ? this.entry.getDetail() : _super.prototype.getDetail.call(this);
    };
    QuickOpenEntryGroup.prototype.getIcon = function () {
        return this.entry ? this.entry.getIcon() : _super.prototype.getIcon.call(this);
    };
    QuickOpenEntryGroup.prototype.getDescription = function () {
        return this.entry ? this.entry.getDescription() : _super.prototype.getDescription.call(this);
    };
    QuickOpenEntryGroup.prototype.getHighlights = function () {
        return this.entry ? this.entry.getHighlights() : _super.prototype.getHighlights.call(this);
    };
    QuickOpenEntryGroup.prototype.isHidden = function () {
        return this.entry ? this.entry.isHidden() : _super.prototype.isHidden.call(this);
    };
    QuickOpenEntryGroup.prototype.setHighlights = function (labelHighlights, descriptionHighlights, detailHighlights) {
        this.entry ? this.entry.setHighlights(labelHighlights, descriptionHighlights, detailHighlights) : _super.prototype.setHighlights.call(this, labelHighlights, descriptionHighlights, detailHighlights);
    };
    QuickOpenEntryGroup.prototype.run = function (mode, context) {
        return this.entry ? this.entry.run(mode, context) : _super.prototype.run.call(this, mode, context);
    };
    return QuickOpenEntryGroup;
}(QuickOpenEntry));
var NoActionProvider = /** @class */ (function () {
    function NoActionProvider() {
    }
    NoActionProvider.prototype.hasActions = function (tree, element) {
        return false;
    };
    NoActionProvider.prototype.getActions = function (tree, element) {
        return TPromise.as(null);
    };
    return NoActionProvider;
}());
var templateEntry = 'quickOpenEntry';
var templateEntryGroup = 'quickOpenEntryGroup';
var Renderer$2 = /** @class */ (function () {
    function Renderer(actionProvider, actionRunner) {
        if (actionProvider === void 0) { actionProvider = new NoActionProvider(); }
        if (actionRunner === void 0) { actionRunner = null; }
        this.actionProvider = actionProvider;
        this.actionRunner = actionRunner;
    }
    Renderer.prototype.getHeight = function (entry) {
        if (entry.getDetail()) {
            return 44;
        }
        return 22;
    };
    Renderer.prototype.getTemplateId = function (entry) {
        if (entry instanceof QuickOpenEntryGroup) {
            return templateEntryGroup;
        }
        return templateEntry;
    };
    Renderer.prototype.renderTemplate = function (templateId, container, styles) {
        var entryContainer = document.createElement('div');
        addClass(entryContainer, 'sub-content');
        container.appendChild(entryContainer);
        // Entry
        var row1 = $('.quick-open-row');
        var row2 = $('.quick-open-row');
        var entry = $('.quick-open-entry', null, row1, row2);
        entryContainer.appendChild(entry);
        // Icon
        var icon = document.createElement('span');
        row1.appendChild(icon);
        // Label
        var label = new IconLabel(row1, { supportHighlights: true, supportDescriptionHighlights: true });
        // Keybinding
        var keybindingContainer = document.createElement('span');
        row1.appendChild(keybindingContainer);
        addClass(keybindingContainer, 'quick-open-entry-keybinding');
        var keybinding = new KeybindingLabel(keybindingContainer, OS);
        // Detail
        var detailContainer = document.createElement('div');
        row2.appendChild(detailContainer);
        addClass(detailContainer, 'quick-open-entry-meta');
        var detail = new HighlightedLabel(detailContainer, true);
        // Entry Group
        var group;
        if (templateId === templateEntryGroup) {
            group = document.createElement('div');
            addClass(group, 'results-group');
            container.appendChild(group);
        }
        // Actions
        addClass(container, 'actions');
        var actionBarContainer = document.createElement('div');
        addClass(actionBarContainer, 'primary-action-bar');
        container.appendChild(actionBarContainer);
        var actionBar = new ActionBar(actionBarContainer, {
            actionRunner: this.actionRunner
        });
        return {
            container: container,
            entry: entry,
            icon: icon,
            label: label,
            detail: detail,
            keybinding: keybinding,
            group: group,
            actionBar: actionBar
        };
    };
    Renderer.prototype.renderElement = function (entry, templateId, data, styles) {
        // Action Bar
        if (this.actionProvider.hasActions(null, entry)) {
            addClass(data.container, 'has-actions');
        }
        else {
            removeClass(data.container, 'has-actions');
        }
        data.actionBar.context = entry; // make sure the context is the current element
        this.actionProvider.getActions(null, entry).then(function (actions) {
            if (data.actionBar.isEmpty() && actions && actions.length > 0) {
                data.actionBar.push(actions, { icon: true, label: false });
            }
            else if (!data.actionBar.isEmpty() && (!actions || actions.length === 0)) {
                data.actionBar.clear();
            }
        });
        // Entry group class
        if (entry instanceof QuickOpenEntryGroup && entry.getGroupLabel()) {
            addClass(data.container, 'has-group-label');
        }
        else {
            removeClass(data.container, 'has-group-label');
        }
        // Entry group
        if (entry instanceof QuickOpenEntryGroup) {
            var group = entry;
            var groupData = data;
            // Border
            if (group.showBorder()) {
                addClass(groupData.container, 'results-group-separator');
                groupData.container.style.borderTopColor = styles.pickerGroupBorder.toString();
            }
            else {
                removeClass(groupData.container, 'results-group-separator');
                groupData.container.style.borderTopColor = null;
            }
            // Group Label
            var groupLabel = group.getGroupLabel() || '';
            groupData.group.textContent = groupLabel;
            groupData.group.style.color = styles.pickerGroupForeground.toString();
        }
        // Normal Entry
        if (entry instanceof QuickOpenEntry) {
            var _a = entry.getHighlights(), labelHighlights = _a[0], descriptionHighlights = _a[1], detailHighlights = _a[2];
            // Icon
            var iconClass = entry.getIcon() ? ('quick-open-entry-icon ' + entry.getIcon()) : '';
            data.icon.className = iconClass;
            // Label
            var options = entry.getLabelOptions() || Object.create(null);
            options.matches = labelHighlights || [];
            options.title = entry.getTooltip();
            options.descriptionTitle = entry.getDescriptionTooltip() || entry.getDescription(); // tooltip over description because it could overflow
            options.descriptionMatches = descriptionHighlights || [];
            data.label.setValue(entry.getLabel(), entry.getDescription(), options);
            // Meta
            data.detail.set(entry.getDetail(), detailHighlights);
            // Keybinding
            data.keybinding.set(entry.getKeybinding(), null);
        }
    };
    Renderer.prototype.disposeTemplate = function (templateId, templateData) {
        var data = templateData;
        data.actionBar.dispose();
        data.actionBar = null;
        data.container = null;
        data.entry = null;
        data.keybinding.dispose();
        data.keybinding = null;
        data.detail.dispose();
        data.detail = null;
        data.group = null;
        data.icon = null;
        data.label.dispose();
        data.label = null;
    };
    return Renderer;
}());
var QuickOpenModel = /** @class */ (function () {
    function QuickOpenModel(entries, actionProvider) {
        if (entries === void 0) { entries = []; }
        if (actionProvider === void 0) { actionProvider = new NoActionProvider(); }
        this._entries = entries;
        this._dataSource = this;
        this._renderer = new Renderer$2(actionProvider);
        this._filter = this;
        this._runner = this;
        this._accessibilityProvider = this;
    }
    Object.defineProperty(QuickOpenModel.prototype, "entries", {
        get: function () { return this._entries; },
        set: function (entries) {
            this._entries = entries;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(QuickOpenModel.prototype, "dataSource", {
        get: function () { return this._dataSource; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(QuickOpenModel.prototype, "renderer", {
        get: function () { return this._renderer; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(QuickOpenModel.prototype, "filter", {
        get: function () { return this._filter; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(QuickOpenModel.prototype, "runner", {
        get: function () { return this._runner; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(QuickOpenModel.prototype, "accessibilityProvider", {
        get: function () { return this._accessibilityProvider; },
        enumerable: true,
        configurable: true
    });
    QuickOpenModel.prototype.getId = function (entry) {
        return entry.getId();
    };
    QuickOpenModel.prototype.getLabel = function (entry) {
        return entry.getLabel();
    };
    QuickOpenModel.prototype.getAriaLabel = function (entry) {
        var ariaLabel = entry.getAriaLabel();
        if (ariaLabel) {
            return localize('quickOpenAriaLabelEntry', "{0}, picker", entry.getAriaLabel());
        }
        return localize('quickOpenAriaLabel', "picker");
    };
    QuickOpenModel.prototype.isVisible = function (entry) {
        return !entry.isHidden();
    };
    QuickOpenModel.prototype.run = function (entry, mode, context) {
        return entry.run(mode, context);
    };
    return QuickOpenModel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getDocumentSymbols(model, flat, token) {
    var roots = [];
    var promises = DocumentSymbolProviderRegistry.all(model).map(function (support) {
        return Promise.resolve(support.provideDocumentSymbols(model, token)).then(function (result) {
            if (Array.isArray(result)) {
                roots.push.apply(roots, result);
            }
        }, function (err) {
            onUnexpectedExternalError(err);
        });
    });
    return Promise.all(promises).then(function () {
        var flatEntries = [];
        if (token.isCancellationRequested) {
            return flatEntries;
        }
        if (flat) {
            flatten$1(flatEntries, roots, '');
        }
        else {
            flatEntries = roots;
        }
        flatEntries.sort(compareEntriesUsingStart);
        return flatEntries;
    });
}
function compareEntriesUsingStart(a, b) {
    return Range.compareRangesUsingStarts(a.range, b.range);
}
function flatten$1(bucket, entries, overrideContainerLabel) {
    for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
        var entry = entries_1[_i];
        bucket.push({
            kind: entry.kind,
            name: entry.name,
            detail: entry.detail,
            containerName: entry.containerName || overrideContainerLabel,
            range: entry.range,
            selectionRange: entry.selectionRange,
            children: undefined,
        });
        if (entry.children) {
            flatten$1(bucket, entry.children, entry.name);
        }
    }
}
registerLanguageCommand('_executeDocumentSymbolProvider', function (accessor, args) {
    var resource = args.resource;
    if (!(resource instanceof URI)) {
        throw illegalArgument('resource');
    }
    var model = accessor.get(IModelService).getModel(resource);
    if (!model) {
        throw illegalArgument('resource');
    }
    return getDocumentSymbols(model, false, CancellationToken.None);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var DataSource = /** @class */ (function () {
    function DataSource(arg) {
        this.modelProvider = isFunction(arg.getModel) ? arg : { getModel: function () { return arg; } };
    }
    DataSource.prototype.getId = function (tree, element) {
        if (!element) {
            return null;
        }
        var model = this.modelProvider.getModel();
        return model === element ? '__root__' : model.dataSource.getId(element);
    };
    DataSource.prototype.hasChildren = function (tree, element) {
        var model = this.modelProvider.getModel();
        return model && model === element && model.entries.length > 0;
    };
    DataSource.prototype.getChildren = function (tree, element) {
        var model = this.modelProvider.getModel();
        return TPromise.as(model === element ? model.entries : []);
    };
    DataSource.prototype.getParent = function (tree, element) {
        return TPromise.as(null);
    };
    return DataSource;
}());
var AccessibilityProvider = /** @class */ (function () {
    function AccessibilityProvider(modelProvider) {
        this.modelProvider = modelProvider;
    }
    AccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
        var model = this.modelProvider.getModel();
        return model.accessibilityProvider && model.accessibilityProvider.getAriaLabel(element);
    };
    AccessibilityProvider.prototype.getPosInSet = function (tree, element) {
        var model = this.modelProvider.getModel();
        var i = 0;
        if (model.filter) {
            for (var _i = 0, _a = model.entries; _i < _a.length; _i++) {
                var entry = _a[_i];
                if (model.filter.isVisible(entry)) {
                    i++;
                }
                if (entry === element) {
                    break;
                }
            }
        }
        else {
            i = model.entries.indexOf(element) + 1;
        }
        return String(i);
    };
    AccessibilityProvider.prototype.getSetSize = function () {
        var model = this.modelProvider.getModel();
        var n = 0;
        if (model.filter) {
            for (var _i = 0, _a = model.entries; _i < _a.length; _i++) {
                var entry = _a[_i];
                if (model.filter.isVisible(entry)) {
                    n++;
                }
            }
        }
        else {
            n = model.entries.length;
        }
        return String(n);
    };
    return AccessibilityProvider;
}());
var Filter = /** @class */ (function () {
    function Filter(modelProvider) {
        this.modelProvider = modelProvider;
    }
    Filter.prototype.isVisible = function (tree, element) {
        var model = this.modelProvider.getModel();
        if (!model.filter) {
            return true;
        }
        return model.filter.isVisible(element);
    };
    return Filter;
}());
var Renderer$3 = /** @class */ (function () {
    function Renderer(modelProvider, styles) {
        this.modelProvider = modelProvider;
        this.styles = styles;
    }
    Renderer.prototype.updateStyles = function (styles) {
        this.styles = styles;
    };
    Renderer.prototype.getHeight = function (tree, element) {
        var model = this.modelProvider.getModel();
        return model.renderer.getHeight(element);
    };
    Renderer.prototype.getTemplateId = function (tree, element) {
        var model = this.modelProvider.getModel();
        return model.renderer.getTemplateId(element);
    };
    Renderer.prototype.renderTemplate = function (tree, templateId, container) {
        var model = this.modelProvider.getModel();
        return model.renderer.renderTemplate(templateId, container, this.styles);
    };
    Renderer.prototype.renderElement = function (tree, element, templateId, templateData) {
        var model = this.modelProvider.getModel();
        model.renderer.renderElement(element, templateId, templateData, this.styles);
    };
    Renderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
        var model = this.modelProvider.getModel();
        model.renderer.disposeTemplate(templateId, templateData);
    };
    return Renderer;
}());

var KeybindingDispatcher = /** @class */ (function () {
    function KeybindingDispatcher() {
        this._arr = [];
    }
    KeybindingDispatcher.prototype.set = function (keybinding, callback) {
        this._arr.push({
            keybinding: createKeybinding(keybinding, OS),
            callback: callback
        });
    };
    KeybindingDispatcher.prototype.dispatch = function (keybinding) {
        // Loop from the last to the first to handle overwrites
        for (var i = this._arr.length - 1; i >= 0; i--) {
            var item = this._arr[i];
            if (keybinding.equals(item.keybinding)) {
                return item.callback;
            }
        }
        return null;
    };
    return KeybindingDispatcher;
}());
var DefaultController = /** @class */ (function () {
    function DefaultController(options) {
        if (options === void 0) { options = { clickBehavior: 0 /* ON_MOUSE_DOWN */, keyboardSupport: true, openMode: 0 /* SINGLE_CLICK */ }; }
        var _this = this;
        this.options = options;
        this.downKeyBindingDispatcher = new KeybindingDispatcher();
        this.upKeyBindingDispatcher = new KeybindingDispatcher();
        if (typeof options.keyboardSupport !== 'boolean' || options.keyboardSupport) {
            this.downKeyBindingDispatcher.set(16 /* UpArrow */, function (t, e) { return _this.onUp(t, e); });
            this.downKeyBindingDispatcher.set(18 /* DownArrow */, function (t, e) { return _this.onDown(t, e); });
            this.downKeyBindingDispatcher.set(15 /* LeftArrow */, function (t, e) { return _this.onLeft(t, e); });
            this.downKeyBindingDispatcher.set(17 /* RightArrow */, function (t, e) { return _this.onRight(t, e); });
            if (isMacintosh) {
                this.downKeyBindingDispatcher.set(2048 /* CtrlCmd */ | 16 /* UpArrow */, function (t, e) { return _this.onLeft(t, e); });
                this.downKeyBindingDispatcher.set(256 /* WinCtrl */ | 44 /* KEY_N */, function (t, e) { return _this.onDown(t, e); });
                this.downKeyBindingDispatcher.set(256 /* WinCtrl */ | 46 /* KEY_P */, function (t, e) { return _this.onUp(t, e); });
            }
            this.downKeyBindingDispatcher.set(11 /* PageUp */, function (t, e) { return _this.onPageUp(t, e); });
            this.downKeyBindingDispatcher.set(12 /* PageDown */, function (t, e) { return _this.onPageDown(t, e); });
            this.downKeyBindingDispatcher.set(14 /* Home */, function (t, e) { return _this.onHome(t, e); });
            this.downKeyBindingDispatcher.set(13 /* End */, function (t, e) { return _this.onEnd(t, e); });
            this.downKeyBindingDispatcher.set(10 /* Space */, function (t, e) { return _this.onSpace(t, e); });
            this.downKeyBindingDispatcher.set(9 /* Escape */, function (t, e) { return _this.onEscape(t, e); });
            this.upKeyBindingDispatcher.set(3 /* Enter */, this.onEnter.bind(this));
            this.upKeyBindingDispatcher.set(2048 /* CtrlCmd */ | 3 /* Enter */, this.onEnter.bind(this));
        }
    }
    DefaultController.prototype.onMouseDown = function (tree, element, event, origin) {
        if (origin === void 0) { origin = 'mouse'; }
        if (this.options.clickBehavior === 0 /* ON_MOUSE_DOWN */ && (event.leftButton || event.middleButton)) {
            if (event.target) {
                if (event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                    return false; // Ignore event if target is a form input field (avoids browser specific issues)
                }
                if (findParentWithClass(event.target, 'scrollbar', 'monaco-tree')) {
                    return false;
                }
                if (findParentWithClass(event.target, 'monaco-action-bar', 'row')) { // TODO@Joao not very nice way of checking for the action bar (implicit knowledge)
                    return false; // Ignore event if target is over an action bar of the row
                }
            }
            // Propagate to onLeftClick now
            return this.onLeftClick(tree, element, event, origin);
        }
        return false;
    };
    DefaultController.prototype.onClick = function (tree, element, event) {
        var isMac = isMacintosh;
        // A Ctrl click on the Mac is a context menu event
        if (isMac && event.ctrlKey) {
            event.preventDefault();
            event.stopPropagation();
            return false;
        }
        if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
            return false; // Ignore event if target is a form input field (avoids browser specific issues)
        }
        if (this.options.clickBehavior === 0 /* ON_MOUSE_DOWN */ && (event.leftButton || event.middleButton)) {
            return false; // Already handled by onMouseDown
        }
        return this.onLeftClick(tree, element, event);
    };
    DefaultController.prototype.onLeftClick = function (tree, element, eventish, origin) {
        if (origin === void 0) { origin = 'mouse'; }
        var event = eventish;
        var payload = { origin: origin, originalEvent: eventish, didClickOnTwistie: this.isClickOnTwistie(event) };
        if (tree.getInput() === element) {
            tree.clearFocus(payload);
            tree.clearSelection(payload);
        }
        else {
            var isSingleMouseDown = eventish && event.browserEvent && event.browserEvent.type === 'mousedown' && event.browserEvent.detail === 1;
            if (!isSingleMouseDown) {
                eventish.preventDefault(); // we cannot preventDefault onMouseDown with single click because this would break DND otherwise
            }
            eventish.stopPropagation();
            tree.domFocus();
            tree.setSelection([element], payload);
            tree.setFocus(element, payload);
            if (this.shouldToggleExpansion(element, event, origin)) {
                if (tree.isExpanded(element)) {
                    tree.collapse(element).then(null, onUnexpectedError);
                }
                else {
                    tree.expand(element).then(null, onUnexpectedError);
                }
            }
        }
        return true;
    };
    DefaultController.prototype.shouldToggleExpansion = function (element, event, origin) {
        var isDoubleClick = (origin === 'mouse' && event.detail === 2);
        return this.openOnSingleClick || isDoubleClick || this.isClickOnTwistie(event);
    };
    DefaultController.prototype.setOpenMode = function (openMode) {
        this.options.openMode = openMode;
    };
    Object.defineProperty(DefaultController.prototype, "openOnSingleClick", {
        get: function () {
            return this.options.openMode === 0 /* SINGLE_CLICK */;
        },
        enumerable: true,
        configurable: true
    });
    DefaultController.prototype.isClickOnTwistie = function (event) {
        var element = event.target;
        if (!hasClass(element, 'content')) {
            return false;
        }
        var twistieStyle = window.getComputedStyle(element, ':before');
        if (twistieStyle.backgroundImage === 'none' || twistieStyle.display === 'none') {
            return false;
        }
        var twistieWidth = parseInt(twistieStyle.width) + parseInt(twistieStyle.paddingRight);
        return event.browserEvent.offsetX <= twistieWidth;
    };
    DefaultController.prototype.onContextMenu = function (tree, element, event) {
        if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
            return false; // allow context menu on input fields
        }
        // Prevent native context menu from showing up
        if (event) {
            event.preventDefault();
            event.stopPropagation();
        }
        return false;
    };
    DefaultController.prototype.onTap = function (tree, element, event) {
        var target = event.initialTarget;
        if (target && target.tagName && target.tagName.toLowerCase() === 'input') {
            return false; // Ignore event if target is a form input field (avoids browser specific issues)
        }
        return this.onLeftClick(tree, element, event, 'touch');
    };
    DefaultController.prototype.onKeyDown = function (tree, event) {
        return this.onKey(this.downKeyBindingDispatcher, tree, event);
    };
    DefaultController.prototype.onKeyUp = function (tree, event) {
        return this.onKey(this.upKeyBindingDispatcher, tree, event);
    };
    DefaultController.prototype.onKey = function (bindings, tree, event) {
        var handler = bindings.dispatch(event.toKeybinding());
        if (handler) {
            // TODO: TS 3.1 upgrade. Why are we checking against void?
            if (handler(tree, event)) {
                event.preventDefault();
                event.stopPropagation();
                return true;
            }
        }
        return false;
    };
    DefaultController.prototype.onUp = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            tree.focusPrevious(1, payload);
            tree.reveal(tree.getFocus()).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onPageUp = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            tree.focusPreviousPage(payload);
            tree.reveal(tree.getFocus()).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onDown = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            tree.focusNext(1, payload);
            tree.reveal(tree.getFocus()).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onPageDown = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            tree.focusNextPage(payload);
            tree.reveal(tree.getFocus()).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onHome = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            tree.focusFirst(payload);
            tree.reveal(tree.getFocus()).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onEnd = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            tree.focusLast(payload);
            tree.reveal(tree.getFocus()).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onLeft = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            var focus_1 = tree.getFocus();
            tree.collapse(focus_1).then(function (didCollapse) {
                if (focus_1 && !didCollapse) {
                    tree.focusParent(payload);
                    return tree.reveal(tree.getFocus());
                }
                return undefined;
            }).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onRight = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
        }
        else {
            var focus_2 = tree.getFocus();
            tree.expand(focus_2).then(function (didExpand) {
                if (focus_2 && !didExpand) {
                    tree.focusFirstChild(payload);
                    return tree.reveal(tree.getFocus());
                }
                return undefined;
            }).then(null, onUnexpectedError);
        }
        return true;
    };
    DefaultController.prototype.onEnter = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            return false;
        }
        var focus = tree.getFocus();
        if (focus) {
            tree.setSelection([focus], payload);
        }
        return true;
    };
    DefaultController.prototype.onSpace = function (tree, event) {
        if (tree.getHighlight()) {
            return false;
        }
        var focus = tree.getFocus();
        if (focus) {
            tree.toggleExpansion(focus);
        }
        return true;
    };
    DefaultController.prototype.onEscape = function (tree, event) {
        var payload = { origin: 'keyboard', originalEvent: event };
        if (tree.getHighlight()) {
            tree.clearHighlight(payload);
            return true;
        }
        if (tree.getSelection().length) {
            tree.clearSelection(payload);
            return true;
        }
        if (tree.getFocus()) {
            tree.clearFocus(payload);
            return true;
        }
        return false;
    };
    return DefaultController;
}());
var DefaultDragAndDrop = /** @class */ (function () {
    function DefaultDragAndDrop() {
    }
    DefaultDragAndDrop.prototype.getDragURI = function (tree, element) {
        return null;
    };
    DefaultDragAndDrop.prototype.onDragStart = function (tree, data, originalEvent) {
        return;
    };
    DefaultDragAndDrop.prototype.onDragOver = function (tree, data, targetElement, originalEvent) {
        return null;
    };
    DefaultDragAndDrop.prototype.drop = function (tree, data, targetElement, originalEvent) {
        return;
    };
    return DefaultDragAndDrop;
}());
var DefaultFilter = /** @class */ (function () {
    function DefaultFilter() {
    }
    DefaultFilter.prototype.isVisible = function (tree, element) {
        return true;
    };
    return DefaultFilter;
}());
var DefaultAccessibilityProvider = /** @class */ (function () {
    function DefaultAccessibilityProvider() {
    }
    DefaultAccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
        return null;
    };
    return DefaultAccessibilityProvider;
}());
var DefaultTreestyler = /** @class */ (function () {
    function DefaultTreestyler(styleElement, selectorSuffix) {
        this.styleElement = styleElement;
        this.selectorSuffix = selectorSuffix;
    }
    DefaultTreestyler.prototype.style = function (styles) {
        var suffix = this.selectorSuffix ? "." + this.selectorSuffix : '';
        var content = [];
        if (styles.listFocusBackground) {
            content.push(".monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.focused:not(.highlighted) { background-color: " + styles.listFocusBackground + "; }");
        }
        if (styles.listFocusForeground) {
            content.push(".monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.focused:not(.highlighted) { color: " + styles.listFocusForeground + "; }");
        }
        if (styles.listActiveSelectionBackground) {
            content.push(".monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.selected:not(.highlighted) { background-color: " + styles.listActiveSelectionBackground + "; }");
        }
        if (styles.listActiveSelectionForeground) {
            content.push(".monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.selected:not(.highlighted) { color: " + styles.listActiveSelectionForeground + "; }");
        }
        if (styles.listFocusAndSelectionBackground) {
            content.push("\n\t\t\t\t.monaco-tree-drag-image,\n\t\t\t\t.monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.focused.selected:not(.highlighted) { background-color: " + styles.listFocusAndSelectionBackground + "; }\n\t\t\t");
        }
        if (styles.listFocusAndSelectionForeground) {
            content.push("\n\t\t\t\t.monaco-tree-drag-image,\n\t\t\t\t.monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.focused.selected:not(.highlighted) { color: " + styles.listFocusAndSelectionForeground + "; }\n\t\t\t");
        }
        if (styles.listInactiveSelectionBackground) {
            content.push(".monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row.selected:not(.highlighted) { background-color: " + styles.listInactiveSelectionBackground + "; }");
        }
        if (styles.listInactiveSelectionForeground) {
            content.push(".monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row.selected:not(.highlighted) { color: " + styles.listInactiveSelectionForeground + "; }");
        }
        if (styles.listHoverBackground) {
            content.push(".monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row:hover:not(.highlighted):not(.selected):not(.focused) { background-color: " + styles.listHoverBackground + "; }");
        }
        if (styles.listHoverForeground) {
            content.push(".monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row:hover:not(.highlighted):not(.selected):not(.focused) { color: " + styles.listHoverForeground + "; }");
        }
        if (styles.listDropBackground) {
            content.push("\n\t\t\t\t.monaco-tree" + suffix + " .monaco-tree-wrapper.drop-target,\n\t\t\t\t.monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row.drop-target { background-color: " + styles.listDropBackground + " !important; color: inherit !important; }\n\t\t\t");
        }
        if (styles.listFocusOutline) {
            content.push("\n\t\t\t\t.monaco-tree-drag-image\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t{ border: 1px solid " + styles.listFocusOutline + "; background: #000; }\n\t\t\t\t.monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row \t\t\t\t\t\t\t\t\t\t\t\t\t\t{ border: 1px solid transparent; }\n\t\t\t\t.monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.focused:not(.highlighted) \t\t\t\t\t\t{ border: 1px dotted " + styles.listFocusOutline + "; }\n\t\t\t\t.monaco-tree" + suffix + ".focused .monaco-tree-rows > .monaco-tree-row.selected:not(.highlighted) \t\t\t\t\t\t{ border: 1px solid " + styles.listFocusOutline + "; }\n\t\t\t\t.monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row.selected:not(.highlighted)  \t\t\t\t\t\t\t{ border: 1px solid " + styles.listFocusOutline + "; }\n\t\t\t\t.monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row:hover:not(.highlighted):not(.selected):not(.focused)  \t{ border: 1px dashed " + styles.listFocusOutline + "; }\n\t\t\t\t.monaco-tree" + suffix + " .monaco-tree-wrapper.drop-target,\n\t\t\t\t.monaco-tree" + suffix + " .monaco-tree-rows > .monaco-tree-row.drop-target\t\t\t\t\t\t\t\t\t\t\t\t{ border: 1px dashed " + styles.listFocusOutline + "; }\n\t\t\t");
        }
        var newStyles = content.join('\n');
        if (newStyles !== this.styleElement.innerHTML) {
            this.styleElement.innerHTML = newStyles;
        }
    };
    return DefaultTreestyler;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1P = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var LockData = /** @class */ (function () {
    function LockData(item) {
        this._onDispose = new Emitter();
        this.onDispose = this._onDispose.event;
        this._item = item;
    }
    Object.defineProperty(LockData.prototype, "item", {
        get: function () {
            return this._item;
        },
        enumerable: true,
        configurable: true
    });
    LockData.prototype.dispose = function () {
        if (this._onDispose) {
            this._onDispose.fire();
            this._onDispose.dispose();
            this._onDispose = null;
        }
    };
    return LockData;
}());
var Lock = /** @class */ (function () {
    function Lock() {
        this.locks = Object.create({});
    }
    Lock.prototype.isLocked = function (item) {
        return !!this.locks[item.id];
    };
    Lock.prototype.run = function (item, fn) {
        var _this = this;
        var lock = this.getLock(item);
        if (lock) {
            return new TPromise(function (c, e) {
                once$1(lock.onDispose)(function () {
                    return _this.run(item, fn).then(c, e);
                });
            });
        }
        var result;
        return new TPromise(function (c, e) {
            if (item.isDisposed()) {
                return e(new Error('Item is disposed.'));
            }
            var lock = _this.locks[item.id] = new LockData(item);
            result = fn().then(function (r) {
                delete _this.locks[item.id];
                lock.dispose();
                return r;
            }).then(c, e);
            return result;
        });
    };
    Lock.prototype.getLock = function (item) {
        var key;
        for (key in this.locks) {
            var lock = this.locks[key];
            if (item.intersects(lock.item)) {
                return lock;
            }
        }
        return null;
    };
    return Lock;
}());
var ItemRegistry = /** @class */ (function () {
    function ItemRegistry() {
        this._isDisposed = false;
        this._onDidRevealItem = new EventMultiplexer();
        this.onDidRevealItem = this._onDidRevealItem.event;
        this._onExpandItem = new EventMultiplexer();
        this.onExpandItem = this._onExpandItem.event;
        this._onDidExpandItem = new EventMultiplexer();
        this.onDidExpandItem = this._onDidExpandItem.event;
        this._onCollapseItem = new EventMultiplexer();
        this.onCollapseItem = this._onCollapseItem.event;
        this._onDidCollapseItem = new EventMultiplexer();
        this.onDidCollapseItem = this._onDidCollapseItem.event;
        this._onDidAddTraitItem = new EventMultiplexer();
        this.onDidAddTraitItem = this._onDidAddTraitItem.event;
        this._onDidRemoveTraitItem = new EventMultiplexer();
        this.onDidRemoveTraitItem = this._onDidRemoveTraitItem.event;
        this._onDidRefreshItem = new EventMultiplexer();
        this.onDidRefreshItem = this._onDidRefreshItem.event;
        this._onRefreshItemChildren = new EventMultiplexer();
        this.onRefreshItemChildren = this._onRefreshItemChildren.event;
        this._onDidRefreshItemChildren = new EventMultiplexer();
        this.onDidRefreshItemChildren = this._onDidRefreshItemChildren.event;
        this._onDidDisposeItem = new EventMultiplexer();
        this.onDidDisposeItem = this._onDidDisposeItem.event;
        this.items = {};
    }
    ItemRegistry.prototype.register = function (item) {
        ok(!this.isRegistered(item.id), 'item already registered: ' + item.id);
        var disposable = combinedDisposable([
            this._onDidRevealItem.add(item.onDidReveal),
            this._onExpandItem.add(item.onExpand),
            this._onDidExpandItem.add(item.onDidExpand),
            this._onCollapseItem.add(item.onCollapse),
            this._onDidCollapseItem.add(item.onDidCollapse),
            this._onDidAddTraitItem.add(item.onDidAddTrait),
            this._onDidRemoveTraitItem.add(item.onDidRemoveTrait),
            this._onDidRefreshItem.add(item.onDidRefresh),
            this._onRefreshItemChildren.add(item.onRefreshChildren),
            this._onDidRefreshItemChildren.add(item.onDidRefreshChildren),
            this._onDidDisposeItem.add(item.onDidDispose)
        ]);
        this.items[item.id] = { item: item, disposable: disposable };
    };
    ItemRegistry.prototype.deregister = function (item) {
        ok(this.isRegistered(item.id), 'item not registered: ' + item.id);
        this.items[item.id].disposable.dispose();
        delete this.items[item.id];
    };
    ItemRegistry.prototype.isRegistered = function (id) {
        return this.items.hasOwnProperty(id);
    };
    ItemRegistry.prototype.getItem = function (id) {
        var result = this.items[id];
        return result ? result.item : null;
    };
    ItemRegistry.prototype.dispose = function () {
        this.items = null;
        this._onDidRevealItem.dispose();
        this._onExpandItem.dispose();
        this._onDidExpandItem.dispose();
        this._onCollapseItem.dispose();
        this._onDidCollapseItem.dispose();
        this._onDidAddTraitItem.dispose();
        this._onDidRemoveTraitItem.dispose();
        this._onDidRefreshItem.dispose();
        this._onRefreshItemChildren.dispose();
        this._onDidRefreshItemChildren.dispose();
        this._isDisposed = true;
    };
    ItemRegistry.prototype.isDisposed = function () {
        return this._isDisposed;
    };
    return ItemRegistry;
}());
var Item = /** @class */ (function () {
    function Item(id, registry, context, lock, element) {
        this._onDidCreate = new Emitter();
        this._onDidReveal = new Emitter();
        this.onDidReveal = this._onDidReveal.event;
        this._onExpand = new Emitter();
        this.onExpand = this._onExpand.event;
        this._onDidExpand = new Emitter();
        this.onDidExpand = this._onDidExpand.event;
        this._onCollapse = new Emitter();
        this.onCollapse = this._onCollapse.event;
        this._onDidCollapse = new Emitter();
        this.onDidCollapse = this._onDidCollapse.event;
        this._onDidAddTrait = new Emitter();
        this.onDidAddTrait = this._onDidAddTrait.event;
        this._onDidRemoveTrait = new Emitter();
        this.onDidRemoveTrait = this._onDidRemoveTrait.event;
        this._onDidRefresh = new Emitter();
        this.onDidRefresh = this._onDidRefresh.event;
        this._onRefreshChildren = new Emitter();
        this.onRefreshChildren = this._onRefreshChildren.event;
        this._onDidRefreshChildren = new Emitter();
        this.onDidRefreshChildren = this._onDidRefreshChildren.event;
        this._onDidDispose = new Emitter();
        this.onDidDispose = this._onDidDispose.event;
        this.registry = registry;
        this.context = context;
        this.lock = lock;
        this.element = element;
        this.id = id;
        this.registry.register(this);
        this.doesHaveChildren = this.context.dataSource.hasChildren(this.context.tree, this.element);
        this.needsChildrenRefresh = true;
        this.parent = null;
        this.previous = null;
        this.next = null;
        this.firstChild = null;
        this.lastChild = null;
        this.traits = {};
        this.depth = 0;
        this.expanded = this.context.dataSource.shouldAutoexpand && this.context.dataSource.shouldAutoexpand(this.context.tree, element);
        this._onDidCreate.fire(this);
        this.visible = this._isVisible();
        this.height = this._getHeight();
        this._isDisposed = false;
    }
    Item.prototype.getElement = function () {
        return this.element;
    };
    Item.prototype.hasChildren = function () {
        return this.doesHaveChildren;
    };
    Item.prototype.getDepth = function () {
        return this.depth;
    };
    Item.prototype.isVisible = function () {
        return this.visible;
    };
    Item.prototype.setVisible = function (value) {
        this.visible = value;
    };
    Item.prototype.isExpanded = function () {
        return this.expanded;
    };
    /* protected */ Item.prototype._setExpanded = function (value) {
        this.expanded = value;
    };
    Item.prototype.reveal = function (relativeTop) {
        if (relativeTop === void 0) { relativeTop = null; }
        var eventData = { item: this, relativeTop: relativeTop };
        this._onDidReveal.fire(eventData);
    };
    Item.prototype.expand = function () {
        var _this = this;
        if (this.isExpanded() || !this.doesHaveChildren || this.lock.isLocked(this)) {
            return TPromise.as(false);
        }
        var result = this.lock.run(this, function () {
            if (_this.isExpanded() || !_this.doesHaveChildren) {
                return TPromise.as(false);
            }
            var eventData = { item: _this };
            var result;
            _this._onExpand.fire(eventData);
            if (_this.needsChildrenRefresh) {
                result = _this.refreshChildren(false, true, true);
            }
            else {
                result = TPromise.as(null);
            }
            return result.then(function () {
                _this._setExpanded(true);
                _this._onDidExpand.fire(eventData);
                return true;
            });
        });
        return result.then(function (r) {
            if (_this.isDisposed()) {
                return false;
            }
            // Auto expand single child folders
            if (_this.context.options.autoExpandSingleChildren && r && _this.firstChild !== null && _this.firstChild === _this.lastChild && _this.firstChild.isVisible()) {
                return _this.firstChild.expand().then(function () { return true; });
            }
            return r;
        });
    };
    Item.prototype.collapse = function (recursive) {
        var _this = this;
        if (recursive === void 0) { recursive = false; }
        if (recursive) {
            var collapseChildrenPromise = TPromise.as(null);
            this.forEachChild(function (child) {
                collapseChildrenPromise = collapseChildrenPromise.then(function () { return child.collapse(true); });
            });
            return collapseChildrenPromise.then(function () {
                return _this.collapse(false);
            });
        }
        else {
            if (!this.isExpanded() || this.lock.isLocked(this)) {
                return TPromise.as(false);
            }
            return this.lock.run(this, function () {
                var eventData = { item: _this };
                _this._onCollapse.fire(eventData);
                _this._setExpanded(false);
                _this._onDidCollapse.fire(eventData);
                return TPromise.as(true);
            });
        }
    };
    Item.prototype.addTrait = function (trait) {
        var eventData = { item: this, trait: trait };
        this.traits[trait] = true;
        this._onDidAddTrait.fire(eventData);
    };
    Item.prototype.removeTrait = function (trait) {
        var eventData = { item: this, trait: trait };
        delete this.traits[trait];
        this._onDidRemoveTrait.fire(eventData);
    };
    Item.prototype.hasTrait = function (trait) {
        return this.traits[trait] || false;
    };
    Item.prototype.getAllTraits = function () {
        var result = [];
        var trait;
        for (trait in this.traits) {
            if (this.traits.hasOwnProperty(trait) && this.traits[trait]) {
                result.push(trait);
            }
        }
        return result;
    };
    Item.prototype.getHeight = function () {
        return this.height;
    };
    Item.prototype.refreshChildren = function (recursive, safe, force) {
        var _this = this;
        if (safe === void 0) { safe = false; }
        if (force === void 0) { force = false; }
        if (!force && !this.isExpanded()) {
            var setNeedsChildrenRefresh_1 = function (item) {
                item.needsChildrenRefresh = true;
                item.forEachChild(setNeedsChildrenRefresh_1);
            };
            setNeedsChildrenRefresh_1(this);
            return TPromise.as(this);
        }
        this.needsChildrenRefresh = false;
        var doRefresh = function () {
            var eventData = { item: _this, isNested: safe };
            _this._onRefreshChildren.fire(eventData);
            var childrenPromise;
            if (_this.doesHaveChildren) {
                childrenPromise = _this.context.dataSource.getChildren(_this.context.tree, _this.element);
            }
            else {
                childrenPromise = TPromise.as([]);
            }
            var result = childrenPromise.then(function (elements) {
                if (_this.isDisposed() || _this.registry.isDisposed()) {
                    return TPromise.as(null);
                }
                if (!Array.isArray(elements)) {
                    return TPromise.wrapError(new Error('Please return an array of children.'));
                }
                elements = !elements ? [] : elements.slice(0);
                elements = _this.sort(elements);
                var staleItems = {};
                while (_this.firstChild !== null) {
                    staleItems[_this.firstChild.id] = _this.firstChild;
                    _this.removeChild(_this.firstChild);
                }
                for (var i = 0, len = elements.length; i < len; i++) {
                    var element = elements[i];
                    var id = _this.context.dataSource.getId(_this.context.tree, element);
                    var item = staleItems[id] || new Item(id, _this.registry, _this.context, _this.lock, element);
                    item.element = element;
                    if (recursive) {
                        item.needsChildrenRefresh = recursive;
                    }
                    delete staleItems[id];
                    _this.addChild(item);
                }
                for (var staleItemId in staleItems) {
                    if (staleItems.hasOwnProperty(staleItemId)) {
                        staleItems[staleItemId].dispose();
                    }
                }
                if (recursive) {
                    return Promise$1.join(_this.mapEachChild(function (child) {
                        return child.doRefresh(recursive, true);
                    }));
                }
                else {
                    return Promise$1.join(_this.mapEachChild(function (child) {
                        if (child.isExpanded() && child.needsChildrenRefresh) {
                            return child.doRefresh(recursive, true);
                        }
                        else {
                            child.updateVisibility();
                            return TPromise.as(null);
                        }
                    }));
                }
            });
            return result
                .then(null, onUnexpectedError)
                .then(function () { return _this._onDidRefreshChildren.fire(eventData); });
        };
        return safe ? doRefresh() : this.lock.run(this, doRefresh);
    };
    Item.prototype.doRefresh = function (recursive, safe) {
        if (safe === void 0) { safe = false; }
        this.doesHaveChildren = this.context.dataSource.hasChildren(this.context.tree, this.element);
        this.height = this._getHeight();
        this.updateVisibility();
        this._onDidRefresh.fire(this);
        return this.refreshChildren(recursive, safe);
    };
    Item.prototype.updateVisibility = function () {
        this.setVisible(this._isVisible());
    };
    Item.prototype.refresh = function (recursive) {
        return this.doRefresh(recursive);
    };
    Item.prototype.getNavigator = function () {
        return new TreeNavigator(this);
    };
    Item.prototype.intersects = function (other) {
        return this.isAncestorOf(other) || other.isAncestorOf(this);
    };
    Item.prototype.isAncestorOf = function (item) {
        while (item) {
            if (item.id === this.id) {
                return true;
            }
            item = item.parent;
        }
        return false;
    };
    Item.prototype.addChild = function (item, afterItem) {
        if (afterItem === void 0) { afterItem = this.lastChild; }
        var isEmpty = this.firstChild === null;
        var atHead = afterItem === null;
        var atTail = afterItem === this.lastChild;
        if (isEmpty) {
            this.firstChild = this.lastChild = item;
            item.next = item.previous = null;
        }
        else if (atHead) {
            this.firstChild.previous = item;
            item.next = this.firstChild;
            item.previous = null;
            this.firstChild = item;
        }
        else if (atTail) {
            this.lastChild.next = item;
            item.next = null;
            item.previous = this.lastChild;
            this.lastChild = item;
        }
        else {
            item.previous = afterItem;
            item.next = afterItem.next;
            afterItem.next.previous = item;
            afterItem.next = item;
        }
        item.parent = this;
        item.depth = this.depth + 1;
    };
    Item.prototype.removeChild = function (item) {
        var isFirstChild = this.firstChild === item;
        var isLastChild = this.lastChild === item;
        if (isFirstChild && isLastChild) {
            this.firstChild = this.lastChild = null;
        }
        else if (isFirstChild) {
            item.next.previous = null;
            this.firstChild = item.next;
        }
        else if (isLastChild) {
            item.previous.next = null;
            this.lastChild = item.previous;
        }
        else {
            item.next.previous = item.previous;
            item.previous.next = item.next;
        }
        item.parent = null;
        item.depth = null;
    };
    Item.prototype.forEachChild = function (fn) {
        var child = this.firstChild, next;
        while (child) {
            next = child.next;
            fn(child);
            child = next;
        }
    };
    Item.prototype.mapEachChild = function (fn) {
        var result = [];
        this.forEachChild(function (child) {
            result.push(fn(child));
        });
        return result;
    };
    Item.prototype.sort = function (elements) {
        var _this = this;
        if (this.context.sorter) {
            return elements.sort(function (element, otherElement) {
                return _this.context.sorter.compare(_this.context.tree, element, otherElement);
            });
        }
        return elements;
    };
    /* protected */ Item.prototype._getHeight = function () {
        return this.context.renderer.getHeight(this.context.tree, this.element);
    };
    /* protected */ Item.prototype._isVisible = function () {
        return this.context.filter.isVisible(this.context.tree, this.element);
    };
    Item.prototype.isDisposed = function () {
        return this._isDisposed;
    };
    Item.prototype.dispose = function () {
        this.forEachChild(function (child) { return child.dispose(); });
        this.parent = null;
        this.previous = null;
        this.next = null;
        this.firstChild = null;
        this.lastChild = null;
        this._onDidDispose.fire(this);
        this.registry.deregister(this);
        this._onDidCreate.dispose();
        this._onDidReveal.dispose();
        this._onExpand.dispose();
        this._onDidExpand.dispose();
        this._onCollapse.dispose();
        this._onDidCollapse.dispose();
        this._onDidAddTrait.dispose();
        this._onDidRemoveTrait.dispose();
        this._onDidRefresh.dispose();
        this._onRefreshChildren.dispose();
        this._onDidRefreshChildren.dispose();
        this._onDidDispose.dispose();
        this._isDisposed = true;
    };
    return Item;
}());
var RootItem = /** @class */ (function (_super) {
    __extends$1P(RootItem, _super);
    function RootItem(id, registry, context, lock, element) {
        return _super.call(this, id, registry, context, lock, element) || this;
    }
    RootItem.prototype.isVisible = function () {
        return false;
    };
    RootItem.prototype.setVisible = function (value) {
        // no-op
    };
    RootItem.prototype.isExpanded = function () {
        return true;
    };
    /* protected */ RootItem.prototype._setExpanded = function (value) {
        // no-op
    };
    /* protected */ RootItem.prototype._getHeight = function () {
        return 0;
    };
    /* protected */ RootItem.prototype._isVisible = function () {
        return false;
    };
    return RootItem;
}(Item));
var TreeNavigator = /** @class */ (function () {
    function TreeNavigator(item, subTreeOnly) {
        if (subTreeOnly === void 0) { subTreeOnly = true; }
        this.item = item;
        this.start = subTreeOnly ? item : null;
    }
    TreeNavigator.lastDescendantOf = function (item) {
        if (!item) {
            return null;
        }
        if (item instanceof RootItem) {
            return TreeNavigator.lastDescendantOf(item.lastChild);
        }
        if (!item.isVisible()) {
            return TreeNavigator.lastDescendantOf(item.previous);
        }
        if (!item.isExpanded() || item.lastChild === null) {
            return item;
        }
        return TreeNavigator.lastDescendantOf(item.lastChild);
    };
    TreeNavigator.prototype.current = function () {
        return this.item || null;
    };
    TreeNavigator.prototype.next = function () {
        if (this.item) {
            do {
                if ((this.item instanceof RootItem || (this.item.isVisible() && this.item.isExpanded())) && this.item.firstChild) {
                    this.item = this.item.firstChild;
                }
                else if (this.item === this.start) {
                    this.item = null;
                }
                else {
                    // select next brother, next uncle, next great-uncle, etc...
                    while (this.item && this.item !== this.start && !this.item.next) {
                        this.item = this.item.parent;
                    }
                    if (this.item === this.start) {
                        this.item = null;
                    }
                    this.item = !this.item ? null : this.item.next;
                }
            } while (this.item && !this.item.isVisible());
        }
        return this.item || null;
    };
    TreeNavigator.prototype.previous = function () {
        if (this.item) {
            do {
                var previous = TreeNavigator.lastDescendantOf(this.item.previous);
                if (previous) {
                    this.item = previous;
                }
                else if (this.item.parent && this.item.parent !== this.start && this.item.parent.isVisible()) {
                    this.item = this.item.parent;
                }
                else {
                    this.item = null;
                }
            } while (this.item && !this.item.isVisible());
        }
        return this.item || null;
    };
    TreeNavigator.prototype.parent = function () {
        if (this.item) {
            var parent = this.item.parent;
            if (parent && parent !== this.start && parent.isVisible()) {
                this.item = parent;
            }
            else {
                this.item = null;
            }
        }
        return this.item || null;
    };
    TreeNavigator.prototype.first = function () {
        this.item = this.start;
        this.next();
        return this.item || null;
    };
    TreeNavigator.prototype.last = function () {
        return TreeNavigator.lastDescendantOf(this.start);
    };
    return TreeNavigator;
}());
var TreeModel = /** @class */ (function () {
    function TreeModel(context) {
        this._onSetInput = new Emitter();
        this.onSetInput = this._onSetInput.event;
        this._onDidSetInput = new Emitter();
        this.onDidSetInput = this._onDidSetInput.event;
        this._onRefresh = new Emitter();
        this.onRefresh = this._onRefresh.event;
        this._onDidRefresh = new Emitter();
        this.onDidRefresh = this._onDidRefresh.event;
        this._onDidHighlight = new Emitter();
        this.onDidHighlight = this._onDidHighlight.event;
        this._onDidSelect = new Emitter();
        this.onDidSelect = this._onDidSelect.event;
        this._onDidFocus = new Emitter();
        this.onDidFocus = this._onDidFocus.event;
        this._onDidRevealItem = new Relay();
        this.onDidRevealItem = this._onDidRevealItem.event;
        this._onExpandItem = new Relay();
        this.onExpandItem = this._onExpandItem.event;
        this._onDidExpandItem = new Relay();
        this.onDidExpandItem = this._onDidExpandItem.event;
        this._onCollapseItem = new Relay();
        this.onCollapseItem = this._onCollapseItem.event;
        this._onDidCollapseItem = new Relay();
        this.onDidCollapseItem = this._onDidCollapseItem.event;
        this._onDidAddTraitItem = new Relay();
        this.onDidAddTraitItem = this._onDidAddTraitItem.event;
        this._onDidRemoveTraitItem = new Relay();
        this.onDidRemoveTraitItem = this._onDidRemoveTraitItem.event;
        this._onDidRefreshItem = new Relay();
        this.onDidRefreshItem = this._onDidRefreshItem.event;
        this._onRefreshItemChildren = new Relay();
        this.onRefreshItemChildren = this._onRefreshItemChildren.event;
        this._onDidRefreshItemChildren = new Relay();
        this.onDidRefreshItemChildren = this._onDidRefreshItemChildren.event;
        this._onDidDisposeItem = new Relay();
        this.context = context;
        this.input = null;
        this.traitsToItems = {};
    }
    TreeModel.prototype.setInput = function (element) {
        var _this = this;
        var eventData = { item: this.input };
        this._onSetInput.fire(eventData);
        this.setSelection([]);
        this.setFocus();
        this.setHighlight();
        this.lock = new Lock();
        if (this.input) {
            this.input.dispose();
        }
        if (this.registry) {
            this.registry.dispose();
            this.registryDisposable.dispose();
        }
        this.registry = new ItemRegistry();
        this._onDidRevealItem.input = this.registry.onDidRevealItem;
        this._onExpandItem.input = this.registry.onExpandItem;
        this._onDidExpandItem.input = this.registry.onDidExpandItem;
        this._onCollapseItem.input = this.registry.onCollapseItem;
        this._onDidCollapseItem.input = this.registry.onDidCollapseItem;
        this._onDidAddTraitItem.input = this.registry.onDidAddTraitItem;
        this._onDidRemoveTraitItem.input = this.registry.onDidRemoveTraitItem;
        this._onDidRefreshItem.input = this.registry.onDidRefreshItem;
        this._onRefreshItemChildren.input = this.registry.onRefreshItemChildren;
        this._onDidRefreshItemChildren.input = this.registry.onDidRefreshItemChildren;
        this._onDidDisposeItem.input = this.registry.onDidDisposeItem;
        this.registryDisposable = this.registry
            .onDidDisposeItem(function (item) { return item.getAllTraits().forEach(function (trait) { return delete _this.traitsToItems[trait][item.id]; }); });
        var id = this.context.dataSource.getId(this.context.tree, element);
        this.input = new RootItem(id, this.registry, this.context, this.lock, element);
        eventData = { item: this.input };
        this._onDidSetInput.fire(eventData);
        return this.refresh(this.input);
    };
    TreeModel.prototype.getInput = function () {
        return this.input ? this.input.getElement() : null;
    };
    TreeModel.prototype.refresh = function (element, recursive) {
        var _this = this;
        if (element === void 0) { element = null; }
        if (recursive === void 0) { recursive = true; }
        var item = this.getItem(element);
        if (!item) {
            return TPromise.as(null);
        }
        var eventData = { item: item, recursive: recursive };
        this._onRefresh.fire(eventData);
        return item.refresh(recursive).then(function () {
            _this._onDidRefresh.fire(eventData);
        });
    };
    TreeModel.prototype.expand = function (element) {
        var item = this.getItem(element);
        if (!item) {
            return TPromise.as(false);
        }
        return item.expand();
    };
    TreeModel.prototype.collapse = function (element, recursive) {
        if (recursive === void 0) { recursive = false; }
        var item = this.getItem(element);
        if (!item) {
            return TPromise.as(false);
        }
        return item.collapse(recursive);
    };
    TreeModel.prototype.toggleExpansion = function (element, recursive) {
        if (recursive === void 0) { recursive = false; }
        return this.isExpanded(element) ? this.collapse(element, recursive) : this.expand(element);
    };
    TreeModel.prototype.isExpanded = function (element) {
        var item = this.getItem(element);
        if (!item) {
            return false;
        }
        return item.isExpanded();
    };
    TreeModel.prototype.reveal = function (element, relativeTop) {
        var _this = this;
        if (relativeTop === void 0) { relativeTop = null; }
        return this.resolveUnknownParentChain(element).then(function (chain$$1) {
            var result = TPromise.as(null);
            chain$$1.forEach(function (e) {
                result = result.then(function () { return _this.expand(e); });
            });
            return result;
        }).then(function () {
            var item = _this.getItem(element);
            if (item) {
                return item.reveal(relativeTop);
            }
        });
    };
    TreeModel.prototype.resolveUnknownParentChain = function (element) {
        var _this = this;
        return this.context.dataSource.getParent(this.context.tree, element).then(function (parent) {
            if (!parent) {
                return TPromise.as([]);
            }
            return _this.resolveUnknownParentChain(parent).then(function (result) {
                result.push(parent);
                return result;
            });
        });
    };
    TreeModel.prototype.setHighlight = function (element, eventPayload) {
        this.setTraits('highlighted', element ? [element] : []);
        var eventData = { highlight: this.getHighlight(), payload: eventPayload };
        this._onDidHighlight.fire(eventData);
    };
    TreeModel.prototype.getHighlight = function (includeHidden) {
        var result = this.getElementsWithTrait('highlighted', includeHidden);
        return result.length === 0 ? null : result[0];
    };
    TreeModel.prototype.setSelection = function (elements, eventPayload) {
        this.setTraits('selected', elements);
        var eventData = { selection: this.getSelection(), payload: eventPayload };
        this._onDidSelect.fire(eventData);
    };
    TreeModel.prototype.getSelection = function (includeHidden) {
        return this.getElementsWithTrait('selected', includeHidden);
    };
    TreeModel.prototype.setFocus = function (element, eventPayload) {
        this.setTraits('focused', element ? [element] : []);
        var eventData = { focus: this.getFocus(), payload: eventPayload };
        this._onDidFocus.fire(eventData);
    };
    TreeModel.prototype.getFocus = function (includeHidden) {
        var result = this.getElementsWithTrait('focused', includeHidden);
        return result.length === 0 ? null : result[0];
    };
    TreeModel.prototype.focusNext = function (count, eventPayload) {
        if (count === void 0) { count = 1; }
        var item = this.getFocus() || this.input;
        var nextItem;
        var nav = this.getNavigator(item, false);
        for (var i = 0; i < count; i++) {
            nextItem = nav.next();
            if (!nextItem) {
                break;
            }
            item = nextItem;
        }
        this.setFocus(item, eventPayload);
    };
    TreeModel.prototype.focusPrevious = function (count, eventPayload) {
        if (count === void 0) { count = 1; }
        var item = this.getFocus() || this.input;
        var previousItem;
        var nav = this.getNavigator(item, false);
        for (var i = 0; i < count; i++) {
            previousItem = nav.previous();
            if (!previousItem) {
                break;
            }
            item = previousItem;
        }
        this.setFocus(item, eventPayload);
    };
    TreeModel.prototype.focusParent = function (eventPayload) {
        var item = this.getFocus() || this.input;
        var nav = this.getNavigator(item, false);
        var parent = nav.parent();
        if (parent) {
            this.setFocus(parent, eventPayload);
        }
    };
    TreeModel.prototype.focusFirstChild = function (eventPayload) {
        var item = this.getItem(this.getFocus() || this.input);
        var nav = this.getNavigator(item, false);
        var next = nav.next();
        var parent = nav.parent();
        if (parent === item) {
            this.setFocus(next, eventPayload);
        }
    };
    TreeModel.prototype.focusFirst = function (eventPayload, from) {
        this.focusNth(0, eventPayload, from);
    };
    TreeModel.prototype.focusNth = function (index, eventPayload, from) {
        var navItem = this.getParent(from);
        var nav = this.getNavigator(navItem);
        var item = nav.first();
        for (var i = 0; i < index; i++) {
            item = nav.next();
        }
        if (item) {
            this.setFocus(item, eventPayload);
        }
    };
    TreeModel.prototype.focusLast = function (eventPayload, from) {
        var navItem = this.getParent(from);
        var item;
        if (from) {
            item = navItem.lastChild;
        }
        else {
            var nav = this.getNavigator(navItem);
            item = nav.last();
        }
        if (item) {
            this.setFocus(item, eventPayload);
        }
    };
    TreeModel.prototype.getParent = function (from) {
        if (from) {
            var fromItem = this.getItem(from);
            if (fromItem && fromItem.parent) {
                return fromItem.parent;
            }
        }
        return this.getItem(this.input);
    };
    TreeModel.prototype.getNavigator = function (element, subTreeOnly) {
        if (element === void 0) { element = null; }
        if (subTreeOnly === void 0) { subTreeOnly = true; }
        return new TreeNavigator(this.getItem(element), subTreeOnly);
    };
    TreeModel.prototype.getItem = function (element) {
        if (element === void 0) { element = null; }
        if (element === null) {
            return this.input;
        }
        else if (element instanceof Item) {
            return element;
        }
        else if (typeof element === 'string') {
            return this.registry.getItem(element);
        }
        else {
            return this.registry.getItem(this.context.dataSource.getId(this.context.tree, element));
        }
    };
    TreeModel.prototype.removeTraits = function (trait, elements) {
        var items = this.traitsToItems[trait] || {};
        var item;
        var id;
        if (elements.length === 0) {
            for (id in items) {
                if (items.hasOwnProperty(id)) {
                    item = items[id];
                    item.removeTrait(trait);
                }
            }
            delete this.traitsToItems[trait];
        }
        else {
            for (var i = 0, len = elements.length; i < len; i++) {
                item = this.getItem(elements[i]);
                if (item) {
                    item.removeTrait(trait);
                    delete items[item.id];
                }
            }
        }
    };
    TreeModel.prototype.setTraits = function (trait, elements) {
        if (elements.length === 0) {
            this.removeTraits(trait, elements);
        }
        else {
            var items = {};
            var item;
            for (var i = 0, len = elements.length; i < len; i++) {
                item = this.getItem(elements[i]);
                if (item) {
                    items[item.id] = item;
                }
            }
            var traitItems = this.traitsToItems[trait] || {};
            var itemsToRemoveTrait = [];
            var id;
            for (id in traitItems) {
                if (traitItems.hasOwnProperty(id)) {
                    if (items.hasOwnProperty(id)) {
                        delete items[id];
                    }
                    else {
                        itemsToRemoveTrait.push(traitItems[id]);
                    }
                }
            }
            for (var i = 0, len = itemsToRemoveTrait.length; i < len; i++) {
                item = itemsToRemoveTrait[i];
                item.removeTrait(trait);
                delete traitItems[item.id];
            }
            for (id in items) {
                if (items.hasOwnProperty(id)) {
                    item = items[id];
                    item.addTrait(trait);
                    traitItems[id] = item;
                }
            }
            this.traitsToItems[trait] = traitItems;
        }
    };
    TreeModel.prototype.getElementsWithTrait = function (trait, includeHidden) {
        var elements = [];
        var items = this.traitsToItems[trait] || {};
        var id;
        for (id in items) {
            if (items.hasOwnProperty(id) && (items[id].isVisible() || includeHidden)) {
                elements.push(items[id].getElement());
            }
        }
        return elements;
    };
    TreeModel.prototype.dispose = function () {
        if (this.registry) {
            this.registry.dispose();
            this.registry = null;
        }
        this._onSetInput.dispose();
        this._onDidSetInput.dispose();
        this._onRefresh.dispose();
        this._onDidRefresh.dispose();
        this._onDidHighlight.dispose();
        this._onDidSelect.dispose();
        this._onDidFocus.dispose();
        this._onDidRevealItem.dispose();
        this._onExpandItem.dispose();
        this._onDidExpandItem.dispose();
        this._onCollapseItem.dispose();
        this._onDidCollapseItem.dispose();
        this._onDidAddTraitItem.dispose();
        this._onDidRemoveTraitItem.dispose();
        this._onDidRefreshItem.dispose();
        this._onRefreshItemChildren.dispose();
        this._onDidRefreshItemChildren.dispose();
        this._onDidDisposeItem.dispose();
    };
    return TreeModel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Represents information about a specific difference between two sequences.
 */
var DiffChange = /** @class */ (function () {
    /**
     * Constructs a new DiffChange with the given sequence information
     * and content.
     */
    function DiffChange(originalStart, originalLength, modifiedStart, modifiedLength) {
        //Debug.Assert(originalLength > 0 || modifiedLength > 0, "originalLength and modifiedLength cannot both be <= 0");
        this.originalStart = originalStart;
        this.originalLength = originalLength;
        this.modifiedStart = modifiedStart;
        this.modifiedLength = modifiedLength;
    }
    /**
     * The end point (exclusive) of the change in the original sequence.
     */
    DiffChange.prototype.getOriginalEnd = function () {
        return this.originalStart + this.originalLength;
    };
    /**
     * The end point (exclusive) of the change in the modified sequence.
     */
    DiffChange.prototype.getModifiedEnd = function () {
        return this.modifiedStart + this.modifiedLength;
    };
    return DiffChange;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function createStringSequence(a) {
    return {
        getLength: function () { return a.length; },
        getElementAtIndex: function (pos) { return a.charCodeAt(pos); }
    };
}
function stringDiff(original, modified, pretty) {
    return new LcsDiff(createStringSequence(original), createStringSequence(modified)).ComputeDiff(pretty);
}
//
// The code below has been ported from a C# implementation in VS
//
var Debug = /** @class */ (function () {
    function Debug() {
    }
    Debug.Assert = function (condition, message) {
        if (!condition) {
            throw new Error(message);
        }
    };
    return Debug;
}());
var MyArray = /** @class */ (function () {
    function MyArray() {
    }
    /**
     * Copies a range of elements from an Array starting at the specified source index and pastes
     * them to another Array starting at the specified destination index. The length and the indexes
     * are specified as 64-bit integers.
     * sourceArray:
     *		The Array that contains the data to copy.
     * sourceIndex:
     *		A 64-bit integer that represents the index in the sourceArray at which copying begins.
     * destinationArray:
     *		The Array that receives the data.
     * destinationIndex:
     *		A 64-bit integer that represents the index in the destinationArray at which storing begins.
     * length:
     *		A 64-bit integer that represents the number of elements to copy.
     */
    MyArray.Copy = function (sourceArray, sourceIndex, destinationArray, destinationIndex, length) {
        for (var i = 0; i < length; i++) {
            destinationArray[destinationIndex + i] = sourceArray[sourceIndex + i];
        }
    };
    return MyArray;
}());
//*****************************************************************************
// LcsDiff.cs
//
// An implementation of the difference algorithm described in
// "An O(ND) Difference Algorithm and its variations" by Eugene W. Myers
//
// Copyright (C) 2008 Microsoft Corporation @minifier_do_not_preserve
//*****************************************************************************
// Our total memory usage for storing history is (worst-case):
// 2 * [(MaxDifferencesHistory + 1) * (MaxDifferencesHistory + 1) - 1] * sizeof(int)
// 2 * [1448*1448 - 1] * 4 = 16773624 = 16MB
var MaxDifferencesHistory = 1447;
//let MaxDifferencesHistory = 100;
/**
 * A utility class which helps to create the set of DiffChanges from
 * a difference operation. This class accepts original DiffElements and
 * modified DiffElements that are involved in a particular change. The
 * MarktNextChange() method can be called to mark the separation between
 * distinct changes. At the end, the Changes property can be called to retrieve
 * the constructed changes.
 */
var DiffChangeHelper = /** @class */ (function () {
    /**
     * Constructs a new DiffChangeHelper for the given DiffSequences.
     */
    function DiffChangeHelper() {
        this.m_changes = [];
        this.m_originalStart = Number.MAX_VALUE;
        this.m_modifiedStart = Number.MAX_VALUE;
        this.m_originalCount = 0;
        this.m_modifiedCount = 0;
    }
    /**
     * Marks the beginning of the next change in the set of differences.
     */
    DiffChangeHelper.prototype.MarkNextChange = function () {
        // Only add to the list if there is something to add
        if (this.m_originalCount > 0 || this.m_modifiedCount > 0) {
            // Add the new change to our list
            this.m_changes.push(new DiffChange(this.m_originalStart, this.m_originalCount, this.m_modifiedStart, this.m_modifiedCount));
        }
        // Reset for the next change
        this.m_originalCount = 0;
        this.m_modifiedCount = 0;
        this.m_originalStart = Number.MAX_VALUE;
        this.m_modifiedStart = Number.MAX_VALUE;
    };
    /**
     * Adds the original element at the given position to the elements
     * affected by the current change. The modified index gives context
     * to the change position with respect to the original sequence.
     * @param originalIndex The index of the original element to add.
     * @param modifiedIndex The index of the modified element that provides corresponding position in the modified sequence.
     */
    DiffChangeHelper.prototype.AddOriginalElement = function (originalIndex, modifiedIndex) {
        // The 'true' start index is the smallest of the ones we've seen
        this.m_originalStart = Math.min(this.m_originalStart, originalIndex);
        this.m_modifiedStart = Math.min(this.m_modifiedStart, modifiedIndex);
        this.m_originalCount++;
    };
    /**
     * Adds the modified element at the given position to the elements
     * affected by the current change. The original index gives context
     * to the change position with respect to the modified sequence.
     * @param originalIndex The index of the original element that provides corresponding position in the original sequence.
     * @param modifiedIndex The index of the modified element to add.
     */
    DiffChangeHelper.prototype.AddModifiedElement = function (originalIndex, modifiedIndex) {
        // The 'true' start index is the smallest of the ones we've seen
        this.m_originalStart = Math.min(this.m_originalStart, originalIndex);
        this.m_modifiedStart = Math.min(this.m_modifiedStart, modifiedIndex);
        this.m_modifiedCount++;
    };
    /**
     * Retrieves all of the changes marked by the class.
     */
    DiffChangeHelper.prototype.getChanges = function () {
        if (this.m_originalCount > 0 || this.m_modifiedCount > 0) {
            // Finish up on whatever is left
            this.MarkNextChange();
        }
        return this.m_changes;
    };
    /**
     * Retrieves all of the changes marked by the class in the reverse order
     */
    DiffChangeHelper.prototype.getReverseChanges = function () {
        if (this.m_originalCount > 0 || this.m_modifiedCount > 0) {
            // Finish up on whatever is left
            this.MarkNextChange();
        }
        this.m_changes.reverse();
        return this.m_changes;
    };
    return DiffChangeHelper;
}());
/**
 * An implementation of the difference algorithm described in
 * "An O(ND) Difference Algorithm and its variations" by Eugene W. Myers
 */
var LcsDiff = /** @class */ (function () {
    /**
     * Constructs the DiffFinder
     */
    function LcsDiff(originalSequence, newSequence, continueProcessingPredicate) {
        if (continueProcessingPredicate === void 0) { continueProcessingPredicate = null; }
        this.OriginalSequence = originalSequence;
        this.ModifiedSequence = newSequence;
        this.ContinueProcessingPredicate = continueProcessingPredicate;
        this.m_forwardHistory = [];
        this.m_reverseHistory = [];
    }
    LcsDiff.prototype.ElementsAreEqual = function (originalIndex, newIndex) {
        return (this.OriginalSequence.getElementAtIndex(originalIndex) === this.ModifiedSequence.getElementAtIndex(newIndex));
    };
    LcsDiff.prototype.OriginalElementsAreEqual = function (index1, index2) {
        return (this.OriginalSequence.getElementAtIndex(index1) === this.OriginalSequence.getElementAtIndex(index2));
    };
    LcsDiff.prototype.ModifiedElementsAreEqual = function (index1, index2) {
        return (this.ModifiedSequence.getElementAtIndex(index1) === this.ModifiedSequence.getElementAtIndex(index2));
    };
    LcsDiff.prototype.ComputeDiff = function (pretty) {
        return this._ComputeDiff(0, this.OriginalSequence.getLength() - 1, 0, this.ModifiedSequence.getLength() - 1, pretty);
    };
    /**
     * Computes the differences between the original and modified input
     * sequences on the bounded range.
     * @returns An array of the differences between the two input sequences.
     */
    LcsDiff.prototype._ComputeDiff = function (originalStart, originalEnd, modifiedStart, modifiedEnd, pretty) {
        var quitEarlyArr = [false];
        var changes = this.ComputeDiffRecursive(originalStart, originalEnd, modifiedStart, modifiedEnd, quitEarlyArr);
        if (pretty) {
            // We have to clean up the computed diff to be more intuitive
            // but it turns out this cannot be done correctly until the entire set
            // of diffs have been computed
            return this.PrettifyChanges(changes);
        }
        return changes;
    };
    /**
     * Private helper method which computes the differences on the bounded range
     * recursively.
     * @returns An array of the differences between the two input sequences.
     */
    LcsDiff.prototype.ComputeDiffRecursive = function (originalStart, originalEnd, modifiedStart, modifiedEnd, quitEarlyArr) {
        quitEarlyArr[0] = false;
        // Find the start of the differences
        while (originalStart <= originalEnd && modifiedStart <= modifiedEnd && this.ElementsAreEqual(originalStart, modifiedStart)) {
            originalStart++;
            modifiedStart++;
        }
        // Find the end of the differences
        while (originalEnd >= originalStart && modifiedEnd >= modifiedStart && this.ElementsAreEqual(originalEnd, modifiedEnd)) {
            originalEnd--;
            modifiedEnd--;
        }
        // In the special case where we either have all insertions or all deletions or the sequences are identical
        if (originalStart > originalEnd || modifiedStart > modifiedEnd) {
            var changes = void 0;
            if (modifiedStart <= modifiedEnd) {
                Debug.Assert(originalStart === originalEnd + 1, 'originalStart should only be one more than originalEnd');
                // All insertions
                changes = [
                    new DiffChange(originalStart, 0, modifiedStart, modifiedEnd - modifiedStart + 1)
                ];
            }
            else if (originalStart <= originalEnd) {
                Debug.Assert(modifiedStart === modifiedEnd + 1, 'modifiedStart should only be one more than modifiedEnd');
                // All deletions
                changes = [
                    new DiffChange(originalStart, originalEnd - originalStart + 1, modifiedStart, 0)
                ];
            }
            else {
                Debug.Assert(originalStart === originalEnd + 1, 'originalStart should only be one more than originalEnd');
                Debug.Assert(modifiedStart === modifiedEnd + 1, 'modifiedStart should only be one more than modifiedEnd');
                // Identical sequences - No differences
                changes = [];
            }
            return changes;
        }
        // This problem can be solved using the Divide-And-Conquer technique.
        var midOriginalArr = [0], midModifiedArr = [0];
        var result = this.ComputeRecursionPoint(originalStart, originalEnd, modifiedStart, modifiedEnd, midOriginalArr, midModifiedArr, quitEarlyArr);
        var midOriginal = midOriginalArr[0];
        var midModified = midModifiedArr[0];
        if (result !== null) {
            // Result is not-null when there was enough memory to compute the changes while
            // searching for the recursion point
            return result;
        }
        else if (!quitEarlyArr[0]) {
            // We can break the problem down recursively by finding the changes in the
            // First Half:   (originalStart, modifiedStart) to (midOriginal, midModified)
            // Second Half:  (midOriginal + 1, minModified + 1) to (originalEnd, modifiedEnd)
            // NOTE: ComputeDiff() is inclusive, therefore the second range starts on the next point
            var leftChanges = this.ComputeDiffRecursive(originalStart, midOriginal, modifiedStart, midModified, quitEarlyArr);
            var rightChanges = [];
            if (!quitEarlyArr[0]) {
                rightChanges = this.ComputeDiffRecursive(midOriginal + 1, originalEnd, midModified + 1, modifiedEnd, quitEarlyArr);
            }
            else {
                // We did't have time to finish the first half, so we don't have time to compute this half.
                // Consider the entire rest of the sequence different.
                rightChanges = [
                    new DiffChange(midOriginal + 1, originalEnd - (midOriginal + 1) + 1, midModified + 1, modifiedEnd - (midModified + 1) + 1)
                ];
            }
            return this.ConcatenateChanges(leftChanges, rightChanges);
        }
        // If we hit here, we quit early, and so can't return anything meaningful
        return [
            new DiffChange(originalStart, originalEnd - originalStart + 1, modifiedStart, modifiedEnd - modifiedStart + 1)
        ];
    };
    LcsDiff.prototype.WALKTRACE = function (diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr) {
        var forwardChanges = null, reverseChanges = null;
        // First, walk backward through the forward diagonals history
        var changeHelper = new DiffChangeHelper();
        var diagonalMin = diagonalForwardStart;
        var diagonalMax = diagonalForwardEnd;
        var diagonalRelative = (midOriginalArr[0] - midModifiedArr[0]) - diagonalForwardOffset;
        var lastOriginalIndex = Number.MIN_VALUE;
        var historyIndex = this.m_forwardHistory.length - 1;
        var diagonal;
        do {
            // Get the diagonal index from the relative diagonal number
            diagonal = diagonalRelative + diagonalForwardBase;
            // Figure out where we came from
            if (diagonal === diagonalMin || (diagonal < diagonalMax && forwardPoints[diagonal - 1] < forwardPoints[diagonal + 1])) {
                // Vertical line (the element is an insert)
                originalIndex = forwardPoints[diagonal + 1];
                modifiedIndex = originalIndex - diagonalRelative - diagonalForwardOffset;
                if (originalIndex < lastOriginalIndex) {
                    changeHelper.MarkNextChange();
                }
                lastOriginalIndex = originalIndex;
                changeHelper.AddModifiedElement(originalIndex + 1, modifiedIndex);
                diagonalRelative = (diagonal + 1) - diagonalForwardBase; //Setup for the next iteration
            }
            else {
                // Horizontal line (the element is a deletion)
                originalIndex = forwardPoints[diagonal - 1] + 1;
                modifiedIndex = originalIndex - diagonalRelative - diagonalForwardOffset;
                if (originalIndex < lastOriginalIndex) {
                    changeHelper.MarkNextChange();
                }
                lastOriginalIndex = originalIndex - 1;
                changeHelper.AddOriginalElement(originalIndex, modifiedIndex + 1);
                diagonalRelative = (diagonal - 1) - diagonalForwardBase; //Setup for the next iteration
            }
            if (historyIndex >= 0) {
                forwardPoints = this.m_forwardHistory[historyIndex];
                diagonalForwardBase = forwardPoints[0]; //We stored this in the first spot
                diagonalMin = 1;
                diagonalMax = forwardPoints.length - 1;
            }
        } while (--historyIndex >= -1);
        // Ironically, we get the forward changes as the reverse of the
        // order we added them since we technically added them backwards
        forwardChanges = changeHelper.getReverseChanges();
        if (quitEarlyArr[0]) {
            // TODO: Calculate a partial from the reverse diagonals.
            //       For now, just assume everything after the midOriginal/midModified point is a diff
            var originalStartPoint = midOriginalArr[0] + 1;
            var modifiedStartPoint = midModifiedArr[0] + 1;
            if (forwardChanges !== null && forwardChanges.length > 0) {
                var lastForwardChange = forwardChanges[forwardChanges.length - 1];
                originalStartPoint = Math.max(originalStartPoint, lastForwardChange.getOriginalEnd());
                modifiedStartPoint = Math.max(modifiedStartPoint, lastForwardChange.getModifiedEnd());
            }
            reverseChanges = [
                new DiffChange(originalStartPoint, originalEnd - originalStartPoint + 1, modifiedStartPoint, modifiedEnd - modifiedStartPoint + 1)
            ];
        }
        else {
            // Now walk backward through the reverse diagonals history
            changeHelper = new DiffChangeHelper();
            diagonalMin = diagonalReverseStart;
            diagonalMax = diagonalReverseEnd;
            diagonalRelative = (midOriginalArr[0] - midModifiedArr[0]) - diagonalReverseOffset;
            lastOriginalIndex = Number.MAX_VALUE;
            historyIndex = (deltaIsEven) ? this.m_reverseHistory.length - 1 : this.m_reverseHistory.length - 2;
            do {
                // Get the diagonal index from the relative diagonal number
                diagonal = diagonalRelative + diagonalReverseBase;
                // Figure out where we came from
                if (diagonal === diagonalMin || (diagonal < diagonalMax && reversePoints[diagonal - 1] >= reversePoints[diagonal + 1])) {
                    // Horizontal line (the element is a deletion))
                    originalIndex = reversePoints[diagonal + 1] - 1;
                    modifiedIndex = originalIndex - diagonalRelative - diagonalReverseOffset;
                    if (originalIndex > lastOriginalIndex) {
                        changeHelper.MarkNextChange();
                    }
                    lastOriginalIndex = originalIndex + 1;
                    changeHelper.AddOriginalElement(originalIndex + 1, modifiedIndex + 1);
                    diagonalRelative = (diagonal + 1) - diagonalReverseBase; //Setup for the next iteration
                }
                else {
                    // Vertical line (the element is an insertion)
                    originalIndex = reversePoints[diagonal - 1];
                    modifiedIndex = originalIndex - diagonalRelative - diagonalReverseOffset;
                    if (originalIndex > lastOriginalIndex) {
                        changeHelper.MarkNextChange();
                    }
                    lastOriginalIndex = originalIndex;
                    changeHelper.AddModifiedElement(originalIndex + 1, modifiedIndex + 1);
                    diagonalRelative = (diagonal - 1) - diagonalReverseBase; //Setup for the next iteration
                }
                if (historyIndex >= 0) {
                    reversePoints = this.m_reverseHistory[historyIndex];
                    diagonalReverseBase = reversePoints[0]; //We stored this in the first spot
                    diagonalMin = 1;
                    diagonalMax = reversePoints.length - 1;
                }
            } while (--historyIndex >= -1);
            // There are cases where the reverse history will find diffs that
            // are correct, but not intuitive, so we need shift them.
            reverseChanges = changeHelper.getChanges();
        }
        return this.ConcatenateChanges(forwardChanges, reverseChanges);
    };
    /**
     * Given the range to compute the diff on, this method finds the point:
     * (midOriginal, midModified)
     * that exists in the middle of the LCS of the two sequences and
     * is the point at which the LCS problem may be broken down recursively.
     * This method will try to keep the LCS trace in memory. If the LCS recursion
     * point is calculated and the full trace is available in memory, then this method
     * will return the change list.
     * @param originalStart The start bound of the original sequence range
     * @param originalEnd The end bound of the original sequence range
     * @param modifiedStart The start bound of the modified sequence range
     * @param modifiedEnd The end bound of the modified sequence range
     * @param midOriginal The middle point of the original sequence range
     * @param midModified The middle point of the modified sequence range
     * @returns The diff changes, if available, otherwise null
     */
    LcsDiff.prototype.ComputeRecursionPoint = function (originalStart, originalEnd, modifiedStart, modifiedEnd, midOriginalArr, midModifiedArr, quitEarlyArr) {
        var originalIndex = 0, modifiedIndex = 0;
        var diagonalForwardStart = 0, diagonalForwardEnd = 0;
        var diagonalReverseStart = 0, diagonalReverseEnd = 0;
        var numDifferences;
        // To traverse the edit graph and produce the proper LCS, our actual
        // start position is just outside the given boundary
        originalStart--;
        modifiedStart--;
        // We set these up to make the compiler happy, but they will
        // be replaced before we return with the actual recursion point
        midOriginalArr[0] = 0;
        midModifiedArr[0] = 0;
        // Clear out the history
        this.m_forwardHistory = [];
        this.m_reverseHistory = [];
        // Each cell in the two arrays corresponds to a diagonal in the edit graph.
        // The integer value in the cell represents the originalIndex of the furthest
        // reaching point found so far that ends in that diagonal.
        // The modifiedIndex can be computed mathematically from the originalIndex and the diagonal number.
        var maxDifferences = (originalEnd - originalStart) + (modifiedEnd - modifiedStart);
        var numDiagonals = maxDifferences + 1;
        var forwardPoints = new Array(numDiagonals);
        var reversePoints = new Array(numDiagonals);
        // diagonalForwardBase: Index into forwardPoints of the diagonal which passes through (originalStart, modifiedStart)
        // diagonalReverseBase: Index into reversePoints of the diagonal which passes through (originalEnd, modifiedEnd)
        var diagonalForwardBase = (modifiedEnd - modifiedStart);
        var diagonalReverseBase = (originalEnd - originalStart);
        // diagonalForwardOffset: Geometric offset which allows modifiedIndex to be computed from originalIndex and the
        //    diagonal number (relative to diagonalForwardBase)
        // diagonalReverseOffset: Geometric offset which allows modifiedIndex to be computed from originalIndex and the
        //    diagonal number (relative to diagonalReverseBase)
        var diagonalForwardOffset = (originalStart - modifiedStart);
        var diagonalReverseOffset = (originalEnd - modifiedEnd);
        // delta: The difference between the end diagonal and the start diagonal. This is used to relate diagonal numbers
        //   relative to the start diagonal with diagonal numbers relative to the end diagonal.
        // The Even/Oddn-ness of this delta is important for determining when we should check for overlap
        var delta = diagonalReverseBase - diagonalForwardBase;
        var deltaIsEven = (delta % 2 === 0);
        // Here we set up the start and end points as the furthest points found so far
        // in both the forward and reverse directions, respectively
        forwardPoints[diagonalForwardBase] = originalStart;
        reversePoints[diagonalReverseBase] = originalEnd;
        // Remember if we quit early, and thus need to do a best-effort result instead of a real result.
        quitEarlyArr[0] = false;
        // A couple of points:
        // --With this method, we iterate on the number of differences between the two sequences.
        //   The more differences there actually are, the longer this will take.
        // --Also, as the number of differences increases, we have to search on diagonals further
        //   away from the reference diagonal (which is diagonalForwardBase for forward, diagonalReverseBase for reverse).
        // --We extend on even diagonals (relative to the reference diagonal) only when numDifferences
        //   is even and odd diagonals only when numDifferences is odd.
        var diagonal, tempOriginalIndex;
        for (numDifferences = 1; numDifferences <= (maxDifferences / 2) + 1; numDifferences++) {
            var furthestOriginalIndex = 0;
            var furthestModifiedIndex = 0;
            // Run the algorithm in the forward direction
            diagonalForwardStart = this.ClipDiagonalBound(diagonalForwardBase - numDifferences, numDifferences, diagonalForwardBase, numDiagonals);
            diagonalForwardEnd = this.ClipDiagonalBound(diagonalForwardBase + numDifferences, numDifferences, diagonalForwardBase, numDiagonals);
            for (diagonal = diagonalForwardStart; diagonal <= diagonalForwardEnd; diagonal += 2) {
                // STEP 1: We extend the furthest reaching point in the present diagonal
                // by looking at the diagonals above and below and picking the one whose point
                // is further away from the start point (originalStart, modifiedStart)
                if (diagonal === diagonalForwardStart || (diagonal < diagonalForwardEnd && forwardPoints[diagonal - 1] < forwardPoints[diagonal + 1])) {
                    originalIndex = forwardPoints[diagonal + 1];
                }
                else {
                    originalIndex = forwardPoints[diagonal - 1] + 1;
                }
                modifiedIndex = originalIndex - (diagonal - diagonalForwardBase) - diagonalForwardOffset;
                // Save the current originalIndex so we can test for false overlap in step 3
                tempOriginalIndex = originalIndex;
                // STEP 2: We can continue to extend the furthest reaching point in the present diagonal
                // so long as the elements are equal.
                while (originalIndex < originalEnd && modifiedIndex < modifiedEnd && this.ElementsAreEqual(originalIndex + 1, modifiedIndex + 1)) {
                    originalIndex++;
                    modifiedIndex++;
                }
                forwardPoints[diagonal] = originalIndex;
                if (originalIndex + modifiedIndex > furthestOriginalIndex + furthestModifiedIndex) {
                    furthestOriginalIndex = originalIndex;
                    furthestModifiedIndex = modifiedIndex;
                }
                // STEP 3: If delta is odd (overlap first happens on forward when delta is odd)
                // and diagonal is in the range of reverse diagonals computed for numDifferences-1
                // (the previous iteration; we haven't computed reverse diagonals for numDifferences yet)
                // then check for overlap.
                if (!deltaIsEven && Math.abs(diagonal - diagonalReverseBase) <= (numDifferences - 1)) {
                    if (originalIndex >= reversePoints[diagonal]) {
                        midOriginalArr[0] = originalIndex;
                        midModifiedArr[0] = modifiedIndex;
                        if (tempOriginalIndex <= reversePoints[diagonal] && MaxDifferencesHistory > 0 && numDifferences <= (MaxDifferencesHistory + 1)) {
                            // BINGO! We overlapped, and we have the full trace in memory!
                            return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
                        }
                        else {
                            // Either false overlap, or we didn't have enough memory for the full trace
                            // Just return the recursion point
                            return null;
                        }
                    }
                }
            }
            // Check to see if we should be quitting early, before moving on to the next iteration.
            var matchLengthOfLongest = ((furthestOriginalIndex - originalStart) + (furthestModifiedIndex - modifiedStart) - numDifferences) / 2;
            if (this.ContinueProcessingPredicate !== null && !this.ContinueProcessingPredicate(furthestOriginalIndex, this.OriginalSequence, matchLengthOfLongest)) {
                // We can't finish, so skip ahead to generating a result from what we have.
                quitEarlyArr[0] = true;
                // Use the furthest distance we got in the forward direction.
                midOriginalArr[0] = furthestOriginalIndex;
                midModifiedArr[0] = furthestModifiedIndex;
                if (matchLengthOfLongest > 0 && MaxDifferencesHistory > 0 && numDifferences <= (MaxDifferencesHistory + 1)) {
                    // Enough of the history is in memory to walk it backwards
                    return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
                }
                else {
                    // We didn't actually remember enough of the history.
                    //Since we are quiting the diff early, we need to shift back the originalStart and modified start
                    //back into the boundary limits since we decremented their value above beyond the boundary limit.
                    originalStart++;
                    modifiedStart++;
                    return [
                        new DiffChange(originalStart, originalEnd - originalStart + 1, modifiedStart, modifiedEnd - modifiedStart + 1)
                    ];
                }
            }
            // Run the algorithm in the reverse direction
            diagonalReverseStart = this.ClipDiagonalBound(diagonalReverseBase - numDifferences, numDifferences, diagonalReverseBase, numDiagonals);
            diagonalReverseEnd = this.ClipDiagonalBound(diagonalReverseBase + numDifferences, numDifferences, diagonalReverseBase, numDiagonals);
            for (diagonal = diagonalReverseStart; diagonal <= diagonalReverseEnd; diagonal += 2) {
                // STEP 1: We extend the furthest reaching point in the present diagonal
                // by looking at the diagonals above and below and picking the one whose point
                // is further away from the start point (originalEnd, modifiedEnd)
                if (diagonal === diagonalReverseStart || (diagonal < diagonalReverseEnd && reversePoints[diagonal - 1] >= reversePoints[diagonal + 1])) {
                    originalIndex = reversePoints[diagonal + 1] - 1;
                }
                else {
                    originalIndex = reversePoints[diagonal - 1];
                }
                modifiedIndex = originalIndex - (diagonal - diagonalReverseBase) - diagonalReverseOffset;
                // Save the current originalIndex so we can test for false overlap
                tempOriginalIndex = originalIndex;
                // STEP 2: We can continue to extend the furthest reaching point in the present diagonal
                // as long as the elements are equal.
                while (originalIndex > originalStart && modifiedIndex > modifiedStart && this.ElementsAreEqual(originalIndex, modifiedIndex)) {
                    originalIndex--;
                    modifiedIndex--;
                }
                reversePoints[diagonal] = originalIndex;
                // STEP 4: If delta is even (overlap first happens on reverse when delta is even)
                // and diagonal is in the range of forward diagonals computed for numDifferences
                // then check for overlap.
                if (deltaIsEven && Math.abs(diagonal - diagonalForwardBase) <= numDifferences) {
                    if (originalIndex <= forwardPoints[diagonal]) {
                        midOriginalArr[0] = originalIndex;
                        midModifiedArr[0] = modifiedIndex;
                        if (tempOriginalIndex >= forwardPoints[diagonal] && MaxDifferencesHistory > 0 && numDifferences <= (MaxDifferencesHistory + 1)) {
                            // BINGO! We overlapped, and we have the full trace in memory!
                            return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
                        }
                        else {
                            // Either false overlap, or we didn't have enough memory for the full trace
                            // Just return the recursion point
                            return null;
                        }
                    }
                }
            }
            // Save current vectors to history before the next iteration
            if (numDifferences <= MaxDifferencesHistory) {
                // We are allocating space for one extra int, which we fill with
                // the index of the diagonal base index
                var temp = new Array(diagonalForwardEnd - diagonalForwardStart + 2);
                temp[0] = diagonalForwardBase - diagonalForwardStart + 1;
                MyArray.Copy(forwardPoints, diagonalForwardStart, temp, 1, diagonalForwardEnd - diagonalForwardStart + 1);
                this.m_forwardHistory.push(temp);
                temp = new Array(diagonalReverseEnd - diagonalReverseStart + 2);
                temp[0] = diagonalReverseBase - diagonalReverseStart + 1;
                MyArray.Copy(reversePoints, diagonalReverseStart, temp, 1, diagonalReverseEnd - diagonalReverseStart + 1);
                this.m_reverseHistory.push(temp);
            }
        }
        // If we got here, then we have the full trace in history. We just have to convert it to a change list
        // NOTE: This part is a bit messy
        return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
    };
    /**
     * Shifts the given changes to provide a more intuitive diff.
     * While the first element in a diff matches the first element after the diff,
     * we shift the diff down.
     *
     * @param changes The list of changes to shift
     * @returns The shifted changes
     */
    LcsDiff.prototype.PrettifyChanges = function (changes) {
        // Shift all the changes down first
        for (var i = 0; i < changes.length; i++) {
            var change = changes[i];
            var originalStop = (i < changes.length - 1) ? changes[i + 1].originalStart : this.OriginalSequence.getLength();
            var modifiedStop = (i < changes.length - 1) ? changes[i + 1].modifiedStart : this.ModifiedSequence.getLength();
            var checkOriginal = change.originalLength > 0;
            var checkModified = change.modifiedLength > 0;
            while (change.originalStart + change.originalLength < originalStop &&
                change.modifiedStart + change.modifiedLength < modifiedStop &&
                (!checkOriginal || this.OriginalElementsAreEqual(change.originalStart, change.originalStart + change.originalLength)) &&
                (!checkModified || this.ModifiedElementsAreEqual(change.modifiedStart, change.modifiedStart + change.modifiedLength))) {
                change.originalStart++;
                change.modifiedStart++;
            }
            var mergedChangeArr = [null];
            if (i < changes.length - 1 && this.ChangesOverlap(changes[i], changes[i + 1], mergedChangeArr)) {
                changes[i] = mergedChangeArr[0];
                changes.splice(i + 1, 1);
                i--;
                continue;
            }
        }
        // Shift changes back up until we hit empty or whitespace-only lines
        for (var i = changes.length - 1; i >= 0; i--) {
            var change = changes[i];
            var originalStop = 0;
            var modifiedStop = 0;
            if (i > 0) {
                var prevChange = changes[i - 1];
                if (prevChange.originalLength > 0) {
                    originalStop = prevChange.originalStart + prevChange.originalLength;
                }
                if (prevChange.modifiedLength > 0) {
                    modifiedStop = prevChange.modifiedStart + prevChange.modifiedLength;
                }
            }
            var checkOriginal = change.originalLength > 0;
            var checkModified = change.modifiedLength > 0;
            var bestDelta = 0;
            var bestScore = this._boundaryScore(change.originalStart, change.originalLength, change.modifiedStart, change.modifiedLength);
            for (var delta = 1;; delta++) {
                var originalStart = change.originalStart - delta;
                var modifiedStart = change.modifiedStart - delta;
                if (originalStart < originalStop || modifiedStart < modifiedStop) {
                    break;
                }
                if (checkOriginal && !this.OriginalElementsAreEqual(originalStart, originalStart + change.originalLength)) {
                    break;
                }
                if (checkModified && !this.ModifiedElementsAreEqual(modifiedStart, modifiedStart + change.modifiedLength)) {
                    break;
                }
                var score = this._boundaryScore(originalStart, change.originalLength, modifiedStart, change.modifiedLength);
                if (score > bestScore) {
                    bestScore = score;
                    bestDelta = delta;
                }
            }
            change.originalStart -= bestDelta;
            change.modifiedStart -= bestDelta;
        }
        return changes;
    };
    LcsDiff.prototype._OriginalIsBoundary = function (index) {
        if (index <= 0 || index >= this.OriginalSequence.getLength() - 1) {
            return true;
        }
        var element = this.OriginalSequence.getElementAtIndex(index);
        return (typeof element === 'string' && /^\s*$/.test(element));
    };
    LcsDiff.prototype._OriginalRegionIsBoundary = function (originalStart, originalLength) {
        if (this._OriginalIsBoundary(originalStart) || this._OriginalIsBoundary(originalStart - 1)) {
            return true;
        }
        if (originalLength > 0) {
            var originalEnd = originalStart + originalLength;
            if (this._OriginalIsBoundary(originalEnd - 1) || this._OriginalIsBoundary(originalEnd)) {
                return true;
            }
        }
        return false;
    };
    LcsDiff.prototype._ModifiedIsBoundary = function (index) {
        if (index <= 0 || index >= this.ModifiedSequence.getLength() - 1) {
            return true;
        }
        var element = this.ModifiedSequence.getElementAtIndex(index);
        return (typeof element === 'string' && /^\s*$/.test(element));
    };
    LcsDiff.prototype._ModifiedRegionIsBoundary = function (modifiedStart, modifiedLength) {
        if (this._ModifiedIsBoundary(modifiedStart) || this._ModifiedIsBoundary(modifiedStart - 1)) {
            return true;
        }
        if (modifiedLength > 0) {
            var modifiedEnd = modifiedStart + modifiedLength;
            if (this._ModifiedIsBoundary(modifiedEnd - 1) || this._ModifiedIsBoundary(modifiedEnd)) {
                return true;
            }
        }
        return false;
    };
    LcsDiff.prototype._boundaryScore = function (originalStart, originalLength, modifiedStart, modifiedLength) {
        var originalScore = (this._OriginalRegionIsBoundary(originalStart, originalLength) ? 1 : 0);
        var modifiedScore = (this._ModifiedRegionIsBoundary(modifiedStart, modifiedLength) ? 1 : 0);
        return (originalScore + modifiedScore);
    };
    /**
     * Concatenates the two input DiffChange lists and returns the resulting
     * list.
     * @param The left changes
     * @param The right changes
     * @returns The concatenated list
     */
    LcsDiff.prototype.ConcatenateChanges = function (left, right) {
        var mergedChangeArr = [];
        if (left.length === 0 || right.length === 0) {
            return (right.length > 0) ? right : left;
        }
        else if (this.ChangesOverlap(left[left.length - 1], right[0], mergedChangeArr)) {
            // Since we break the problem down recursively, it is possible that we
            // might recurse in the middle of a change thereby splitting it into
            // two changes. Here in the combining stage, we detect and fuse those
            // changes back together
            var result = new Array(left.length + right.length - 1);
            MyArray.Copy(left, 0, result, 0, left.length - 1);
            result[left.length - 1] = mergedChangeArr[0];
            MyArray.Copy(right, 1, result, left.length, right.length - 1);
            return result;
        }
        else {
            var result = new Array(left.length + right.length);
            MyArray.Copy(left, 0, result, 0, left.length);
            MyArray.Copy(right, 0, result, left.length, right.length);
            return result;
        }
    };
    /**
     * Returns true if the two changes overlap and can be merged into a single
     * change
     * @param left The left change
     * @param right The right change
     * @param mergedChange The merged change if the two overlap, null otherwise
     * @returns True if the two changes overlap
     */
    LcsDiff.prototype.ChangesOverlap = function (left, right, mergedChangeArr) {
        Debug.Assert(left.originalStart <= right.originalStart, 'Left change is not less than or equal to right change');
        Debug.Assert(left.modifiedStart <= right.modifiedStart, 'Left change is not less than or equal to right change');
        if (left.originalStart + left.originalLength >= right.originalStart || left.modifiedStart + left.modifiedLength >= right.modifiedStart) {
            var originalStart = left.originalStart;
            var originalLength = left.originalLength;
            var modifiedStart = left.modifiedStart;
            var modifiedLength = left.modifiedLength;
            if (left.originalStart + left.originalLength >= right.originalStart) {
                originalLength = right.originalStart + right.originalLength - left.originalStart;
            }
            if (left.modifiedStart + left.modifiedLength >= right.modifiedStart) {
                modifiedLength = right.modifiedStart + right.modifiedLength - left.modifiedStart;
            }
            mergedChangeArr[0] = new DiffChange(originalStart, originalLength, modifiedStart, modifiedLength);
            return true;
        }
        else {
            mergedChangeArr[0] = null;
            return false;
        }
    };
    /**
     * Helper method used to clip a diagonal index to the range of valid
     * diagonals. This also decides whether or not the diagonal index,
     * if it exceeds the boundary, should be clipped to the boundary or clipped
     * one inside the boundary depending on the Even/Odd status of the boundary
     * and numDifferences.
     * @param diagonal The index of the diagonal to clip.
     * @param numDifferences The current number of differences being iterated upon.
     * @param diagonalBaseIndex The base reference diagonal.
     * @param numDiagonals The total number of diagonals.
     * @returns The clipped diagonal index.
     */
    LcsDiff.prototype.ClipDiagonalBound = function (diagonal, numDifferences, diagonalBaseIndex, numDiagonals) {
        if (diagonal >= 0 && diagonal < numDiagonals) {
            // Nothing to clip, its in range
            return diagonal;
        }
        // diagonalsBelow: The number of diagonals below the reference diagonal
        // diagonalsAbove: The number of diagonals above the reference diagonal
        var diagonalsBelow = diagonalBaseIndex;
        var diagonalsAbove = numDiagonals - diagonalBaseIndex - 1;
        var diffEven = (numDifferences % 2 === 0);
        if (diagonal < 0) {
            var lowerBoundEven = (diagonalsBelow % 2 === 0);
            return (diffEven === lowerBoundEven) ? 0 : 1;
        }
        else {
            var upperBoundEven = (diagonalsAbove % 2 === 0);
            return (diffEven === upperBoundEven) ? numDiagonals - 1 : numDiagonals - 2;
        }
    };
    return LcsDiff;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ElementsDragAndDropData = /** @class */ (function () {
    function ElementsDragAndDropData(elements) {
        this.elements = elements;
    }
    ElementsDragAndDropData.prototype.update = function (event) {
        // no-op
    };
    return ElementsDragAndDropData;
}());
var ExternalElementsDragAndDropData = /** @class */ (function () {
    function ExternalElementsDragAndDropData(elements) {
        this.elements = elements;
    }
    ExternalElementsDragAndDropData.prototype.update = function (event) {
        // no-op
    };
    return ExternalElementsDragAndDropData;
}());
var DesktopDragAndDropData = /** @class */ (function () {
    function DesktopDragAndDropData() {
        this.types = [];
        this.files = [];
    }
    DesktopDragAndDropData.prototype.update = function (event) {
        if (event.dataTransfer.types) {
            this.types = [];
            Array.prototype.push.apply(this.types, event.dataTransfer.types);
        }
        if (event.dataTransfer.files) {
            this.files = [];
            Array.prototype.push.apply(this.files, event.dataTransfer.files);
            this.files = this.files.filter(function (f) { return f.size || f.type; });
        }
    };
    return DesktopDragAndDropData;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var HeightMap = /** @class */ (function () {
    function HeightMap() {
        this.heightMap = [];
        this.indexes = {};
    }
    HeightMap.prototype.getContentHeight = function () {
        var last = this.heightMap[this.heightMap.length - 1];
        return !last ? 0 : last.top + last.height;
    };
    HeightMap.prototype.onInsertItems = function (iterator, afterItemId) {
        if (afterItemId === void 0) { afterItemId = null; }
        var item;
        var viewItem;
        var i, j;
        var totalSize;
        var sizeDiff = 0;
        if (afterItemId === null) {
            i = 0;
            totalSize = 0;
        }
        else {
            i = this.indexes[afterItemId] + 1;
            viewItem = this.heightMap[i - 1];
            if (!viewItem) {
                console.error('view item doesnt exist');
                return undefined;
            }
            totalSize = viewItem.top + viewItem.height;
        }
        var boundSplice = this.heightMap.splice.bind(this.heightMap, i, 0);
        var itemsToInsert = [];
        while (item = iterator.next()) {
            viewItem = this.createViewItem(item);
            viewItem.top = totalSize + sizeDiff;
            this.indexes[item.id] = i++;
            itemsToInsert.push(viewItem);
            sizeDiff += viewItem.height;
        }
        boundSplice.apply(this.heightMap, itemsToInsert);
        for (j = i; j < this.heightMap.length; j++) {
            viewItem = this.heightMap[j];
            viewItem.top += sizeDiff;
            this.indexes[viewItem.model.id] = j;
        }
        for (j = itemsToInsert.length - 1; j >= 0; j--) {
            this.onInsertItem(itemsToInsert[j]);
        }
        for (j = this.heightMap.length - 1; j >= i; j--) {
            this.onRefreshItem(this.heightMap[j]);
        }
        return sizeDiff;
    };
    HeightMap.prototype.onInsertItem = function (item) {
        // noop
    };
    // Contiguous items
    HeightMap.prototype.onRemoveItems = function (iterator) {
        var itemId;
        var viewItem;
        var startIndex = null;
        var i;
        var sizeDiff = 0;
        while (itemId = iterator.next()) {
            i = this.indexes[itemId];
            viewItem = this.heightMap[i];
            if (!viewItem) {
                console.error('view item doesnt exist');
                return;
            }
            sizeDiff -= viewItem.height;
            delete this.indexes[itemId];
            this.onRemoveItem(viewItem);
            if (startIndex === null) {
                startIndex = i;
            }
        }
        if (sizeDiff === 0) {
            return;
        }
        this.heightMap.splice(startIndex, i - startIndex + 1);
        for (i = startIndex; i < this.heightMap.length; i++) {
            viewItem = this.heightMap[i];
            viewItem.top += sizeDiff;
            this.indexes[viewItem.model.id] = i;
            this.onRefreshItem(viewItem);
        }
    };
    HeightMap.prototype.onRemoveItem = function (item) {
        // noop
    };
    HeightMap.prototype.onRefreshItemSet = function (items) {
        var _this = this;
        var sortedItems = items.sort(function (a, b) { return _this.indexes[a.id] - _this.indexes[b.id]; });
        this.onRefreshItems(new ArrayIterator(sortedItems));
    };
    // Ordered, but not necessarily contiguous items
    HeightMap.prototype.onRefreshItems = function (iterator) {
        var item;
        var viewItem;
        var newHeight;
        var i, j = null;
        var cummDiff = 0;
        while (item = iterator.next()) {
            i = this.indexes[item.id];
            for (; cummDiff !== 0 && j !== null && j < i; j++) {
                viewItem = this.heightMap[j];
                viewItem.top += cummDiff;
                this.onRefreshItem(viewItem);
            }
            viewItem = this.heightMap[i];
            newHeight = item.getHeight();
            viewItem.top += cummDiff;
            cummDiff += newHeight - viewItem.height;
            viewItem.height = newHeight;
            this.onRefreshItem(viewItem, true);
            j = i + 1;
        }
        if (cummDiff !== 0 && j !== null) {
            for (; j < this.heightMap.length; j++) {
                viewItem = this.heightMap[j];
                viewItem.top += cummDiff;
                this.onRefreshItem(viewItem);
            }
        }
    };
    HeightMap.prototype.onRefreshItem = function (item, needsRender) {
        if (needsRender === void 0) { needsRender = false; }
        // noop
    };
    HeightMap.prototype.indexAt = function (position) {
        var left = 0;
        var right = this.heightMap.length;
        var center;
        var item;
        // Binary search
        while (left < right) {
            center = Math.floor((left + right) / 2);
            item = this.heightMap[center];
            if (position < item.top) {
                right = center;
            }
            else if (position >= item.top + item.height) {
                if (left === center) {
                    break;
                }
                left = center;
            }
            else {
                return center;
            }
        }
        return this.heightMap.length;
    };
    HeightMap.prototype.indexAfter = function (position) {
        return Math.min(this.indexAt(position) + 1, this.heightMap.length);
    };
    HeightMap.prototype.itemAtIndex = function (index) {
        return this.heightMap[index];
    };
    HeightMap.prototype.itemAfter = function (item) {
        return this.heightMap[this.indexes[item.model.id] + 1] || null;
    };
    HeightMap.prototype.createViewItem = function (item) {
        throw new Error('not implemented');
    };
    HeightMap.prototype.dispose = function () {
        this.heightMap = null;
        this.indexes = null;
    };
    return HeightMap;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1Q = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ContextMenuEvent = /** @class */ (function () {
    function ContextMenuEvent(posx, posy, target) {
        this._posx = posx;
        this._posy = posy;
        this._target = target;
    }
    ContextMenuEvent.prototype.preventDefault = function () {
        // no-op
    };
    ContextMenuEvent.prototype.stopPropagation = function () {
        // no-op
    };
    Object.defineProperty(ContextMenuEvent.prototype, "target", {
        get: function () {
            return this._target;
        },
        enumerable: true,
        configurable: true
    });
    return ContextMenuEvent;
}());
var MouseContextMenuEvent = /** @class */ (function (_super) {
    __extends$1Q(MouseContextMenuEvent, _super);
    function MouseContextMenuEvent(originalEvent) {
        var _this = _super.call(this, originalEvent.posx, originalEvent.posy, originalEvent.target) || this;
        _this.originalEvent = originalEvent;
        return _this;
    }
    MouseContextMenuEvent.prototype.preventDefault = function () {
        this.originalEvent.preventDefault();
    };
    MouseContextMenuEvent.prototype.stopPropagation = function () {
        this.originalEvent.stopPropagation();
    };
    return MouseContextMenuEvent;
}(ContextMenuEvent));
var KeyboardContextMenuEvent = /** @class */ (function (_super) {
    __extends$1Q(KeyboardContextMenuEvent, _super);
    function KeyboardContextMenuEvent(posx, posy, originalEvent) {
        var _this = _super.call(this, posx, posy, originalEvent.target) || this;
        _this.originalEvent = originalEvent;
        return _this;
    }
    KeyboardContextMenuEvent.prototype.preventDefault = function () {
        this.originalEvent.preventDefault();
    };
    KeyboardContextMenuEvent.prototype.stopPropagation = function () {
        this.originalEvent.stopPropagation();
    };
    return KeyboardContextMenuEvent;
}(ContextMenuEvent));

// Common data transfers
var DataTransfers = {
    /**
     * Application specific resource transfer type
     */
    RESOURCES: 'ResourceURLs',
    /**
     * Browser specific transfer type to download
     */
    DOWNLOAD_URL: 'DownloadURL',
    /**
     * Browser specific transfer type for files
     */
    FILES: 'Files',
    /**
     * Typicaly transfer type for copy/paste transfers.
     */
    TEXT: 'text/plain'
};

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1R = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function removeFromParent$1(element) {
    try {
        element.parentElement.removeChild(element);
    }
    catch (e) {
        // this will throw if this happens due to a blur event, nasty business
    }
}
var RowCache$1 = /** @class */ (function () {
    function RowCache(context) {
        this.context = context;
        this._cache = { '': [] };
    }
    RowCache.prototype.alloc = function (templateId) {
        var result = this.cache(templateId).pop();
        if (!result) {
            var content = document.createElement('div');
            content.className = 'content';
            var row = document.createElement('div');
            row.appendChild(content);
            var templateData = null;
            try {
                templateData = this.context.renderer.renderTemplate(this.context.tree, templateId, content);
            }
            catch (err) {
                console.error('Tree usage error: exception while rendering template');
                console.error(err);
            }
            result = {
                element: row,
                templateId: templateId,
                templateData: templateData
            };
        }
        return result;
    };
    RowCache.prototype.release = function (templateId, row) {
        removeFromParent$1(row.element);
        this.cache(templateId).push(row);
    };
    RowCache.prototype.cache = function (templateId) {
        return this._cache[templateId] || (this._cache[templateId] = []);
    };
    RowCache.prototype.garbageCollect = function () {
        var _this = this;
        if (this._cache) {
            Object.keys(this._cache).forEach(function (templateId) {
                _this._cache[templateId].forEach(function (cachedRow) {
                    _this.context.renderer.disposeTemplate(_this.context.tree, templateId, cachedRow.templateData);
                    cachedRow.element = null;
                    cachedRow.templateData = null;
                });
                delete _this._cache[templateId];
            });
        }
    };
    RowCache.prototype.dispose = function () {
        this.garbageCollect();
        this._cache = null;
        this.context = null;
    };
    return RowCache;
}());
var ViewItem = /** @class */ (function () {
    function ViewItem(context, model) {
        var _this = this;
        this.width = 0;
        this.context = context;
        this.model = model;
        this.id = this.model.id;
        this.row = null;
        this.top = 0;
        this.height = model.getHeight();
        this._styles = {};
        model.getAllTraits().forEach(function (t) { return _this._styles[t] = true; });
        if (model.isExpanded()) {
            this.addClass('expanded');
        }
    }
    Object.defineProperty(ViewItem.prototype, "expanded", {
        set: function (value) {
            value ? this.addClass('expanded') : this.removeClass('expanded');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ViewItem.prototype, "loading", {
        set: function (value) {
            value ? this.addClass('loading') : this.removeClass('loading');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ViewItem.prototype, "draggable", {
        get: function () {
            return this._draggable;
        },
        set: function (value) {
            this._draggable = value;
            this.render(true);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ViewItem.prototype, "dropTarget", {
        set: function (value) {
            value ? this.addClass('drop-target') : this.removeClass('drop-target');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ViewItem.prototype, "element", {
        get: function () {
            return this.row && this.row.element;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ViewItem.prototype, "templateId", {
        get: function () {
            return this._templateId || (this._templateId = (this.context.renderer.getTemplateId && this.context.renderer.getTemplateId(this.context.tree, this.model.getElement())));
        },
        enumerable: true,
        configurable: true
    });
    ViewItem.prototype.addClass = function (name) {
        this._styles[name] = true;
        this.render(true);
    };
    ViewItem.prototype.removeClass = function (name) {
        delete this._styles[name]; // is this slow?
        this.render(true);
    };
    ViewItem.prototype.render = function (skipUserRender) {
        var _this = this;
        if (skipUserRender === void 0) { skipUserRender = false; }
        if (!this.model || !this.element) {
            return;
        }
        var classes = ['monaco-tree-row'];
        classes.push.apply(classes, Object.keys(this._styles));
        if (this.model.hasChildren()) {
            classes.push('has-children');
        }
        this.element.className = classes.join(' ');
        this.element.draggable = this.draggable;
        this.element.style.height = this.height + 'px';
        // ARIA
        this.element.setAttribute('role', 'treeitem');
        var accessibility = this.context.accessibilityProvider;
        var ariaLabel = accessibility.getAriaLabel(this.context.tree, this.model.getElement());
        if (ariaLabel) {
            this.element.setAttribute('aria-label', ariaLabel);
        }
        if (accessibility.getPosInSet && accessibility.getSetSize) {
            this.element.setAttribute('aria-setsize', accessibility.getSetSize());
            this.element.setAttribute('aria-posinset', accessibility.getPosInSet(this.context.tree, this.model.getElement()));
        }
        if (this.model.hasTrait('focused')) {
            var base64Id = safeBtoa(this.model.id);
            this.element.setAttribute('aria-selected', 'true');
            this.element.setAttribute('id', base64Id);
        }
        else {
            this.element.setAttribute('aria-selected', 'false');
            this.element.removeAttribute('id');
        }
        if (this.model.hasChildren()) {
            this.element.setAttribute('aria-expanded', String(!!this._styles['expanded']));
        }
        else {
            this.element.removeAttribute('aria-expanded');
        }
        this.element.setAttribute('aria-level', String(this.model.getDepth()));
        if (this.context.options.paddingOnRow) {
            this.element.style.paddingLeft = this.context.options.twistiePixels + ((this.model.getDepth() - 1) * this.context.options.indentPixels) + 'px';
        }
        else {
            this.element.style.paddingLeft = ((this.model.getDepth() - 1) * this.context.options.indentPixels) + 'px';
            this.row.element.firstElementChild.style.paddingLeft = this.context.options.twistiePixels + 'px';
        }
        var uri = this.context.dnd.getDragURI(this.context.tree, this.model.getElement());
        if (uri !== this.uri) {
            if (this.unbindDragStart) {
                this.unbindDragStart.dispose();
                this.unbindDragStart = null;
            }
            if (uri) {
                this.uri = uri;
                this.draggable = true;
                this.unbindDragStart = addDisposableListener(this.element, 'dragstart', function (e) {
                    _this.onDragStart(e);
                });
            }
            else {
                this.uri = null;
            }
        }
        if (!skipUserRender && this.element) {
            var style = window.getComputedStyle(this.element);
            var paddingLeft = parseFloat(style.paddingLeft);
            if (this.context.horizontalScrolling) {
                this.element.style.width = 'fit-content';
            }
            try {
                this.context.renderer.renderElement(this.context.tree, this.model.getElement(), this.templateId, this.row.templateData);
            }
            catch (err) {
                console.error('Tree usage error: exception while rendering element');
                console.error(err);
            }
            if (this.context.horizontalScrolling) {
                this.width = getContentWidth(this.element) + paddingLeft;
                this.element.style.width = '';
            }
        }
    };
    ViewItem.prototype.insertInDOM = function (container, afterElement) {
        if (!this.row) {
            this.row = this.context.cache.alloc(this.templateId);
            // used in reverse lookup from HTMLElement to Item
            this.element[TreeView.BINDING] = this;
        }
        if (this.element.parentElement) {
            return;
        }
        if (afterElement === null) {
            container.appendChild(this.element);
        }
        else {
            try {
                container.insertBefore(this.element, afterElement);
            }
            catch (e) {
                console.warn('Failed to locate previous tree element');
                container.appendChild(this.element);
            }
        }
        this.render();
    };
    ViewItem.prototype.removeFromDOM = function () {
        if (!this.row) {
            return;
        }
        if (this.unbindDragStart) {
            this.unbindDragStart.dispose();
            this.unbindDragStart = null;
        }
        this.uri = null;
        this.element[TreeView.BINDING] = null;
        this.context.cache.release(this.templateId, this.row);
        this.row = null;
    };
    ViewItem.prototype.dispose = function () {
        this.row = null;
        this.model = null;
    };
    return ViewItem;
}());
var RootViewItem = /** @class */ (function (_super) {
    __extends$1R(RootViewItem, _super);
    function RootViewItem(context, model, wrapper) {
        var _this = _super.call(this, context, model) || this;
        _this.row = {
            element: wrapper,
            templateData: null,
            templateId: null
        };
        return _this;
    }
    RootViewItem.prototype.render = function () {
        if (!this.model || !this.element) {
            return;
        }
        var classes = ['monaco-tree-wrapper'];
        classes.push.apply(classes, Object.keys(this._styles));
        if (this.model.hasChildren()) {
            classes.push('has-children');
        }
        this.element.className = classes.join(' ');
    };
    RootViewItem.prototype.insertInDOM = function (container, afterElement) {
        // noop
    };
    RootViewItem.prototype.removeFromDOM = function () {
        // noop
    };
    return RootViewItem;
}(ViewItem));
function reactionEquals(one, other) {
    if (!one && !other) {
        return true;
    }
    else if (!one || !other) {
        return false;
    }
    else if (one.accept !== other.accept) {
        return false;
    }
    else if (one.bubble !== other.bubble) {
        return false;
    }
    else if (one.effect !== other.effect) {
        return false;
    }
    else {
        return true;
    }
}
var TreeView = /** @class */ (function (_super) {
    __extends$1R(TreeView, _super);
    function TreeView(context, container) {
        var _this = _super.call(this) || this;
        _this.lastClickTimeStamp = 0;
        _this.contentWidthUpdateDelayer = new Delayer(50);
        _this.isRefreshing = false;
        _this.refreshingPreviousChildrenIds = {};
        _this.currentDropDisposable = Disposable.None;
        _this._onDOMFocus = new Emitter();
        _this._onDOMBlur = new Emitter();
        _this._onDidScroll = new Emitter();
        TreeView.counter++;
        _this.instance = TreeView.counter;
        var horizontalScrollMode = typeof context.options.horizontalScrollMode === 'undefined' ? 2 /* Hidden */ : context.options.horizontalScrollMode;
        _this.horizontalScrolling = horizontalScrollMode !== 2 /* Hidden */;
        _this.context = {
            dataSource: context.dataSource,
            renderer: context.renderer,
            controller: context.controller,
            dnd: context.dnd,
            filter: context.filter,
            sorter: context.sorter,
            tree: context.tree,
            accessibilityProvider: context.accessibilityProvider,
            options: context.options,
            cache: new RowCache$1(context),
            horizontalScrolling: _this.horizontalScrolling
        };
        _this.modelListeners = [];
        _this.viewListeners = [];
        _this.model = null;
        _this.items = {};
        _this.domNode = document.createElement('div');
        _this.domNode.className = "monaco-tree no-focused-item monaco-tree-instance-" + _this.instance;
        // to allow direct tabbing into the tree instead of first focusing the tree
        _this.domNode.tabIndex = context.options.preventRootFocus ? -1 : 0;
        _this.styleElement = createStyleSheet(_this.domNode);
        _this.treeStyler = context.styler;
        if (!_this.treeStyler) {
            _this.treeStyler = new DefaultTreestyler(_this.styleElement, "monaco-tree-instance-" + _this.instance);
        }
        // ARIA
        _this.domNode.setAttribute('role', 'tree');
        if (_this.context.options.ariaLabel) {
            _this.domNode.setAttribute('aria-label', _this.context.options.ariaLabel);
        }
        if (_this.context.options.alwaysFocused) {
            addClass(_this.domNode, 'focused');
        }
        if (!_this.context.options.paddingOnRow) {
            addClass(_this.domNode, 'no-row-padding');
        }
        _this.wrapper = document.createElement('div');
        _this.wrapper.className = 'monaco-tree-wrapper';
        _this.scrollableElement = new ScrollableElement(_this.wrapper, {
            alwaysConsumeMouseWheel: true,
            horizontal: horizontalScrollMode,
            vertical: (typeof context.options.verticalScrollMode !== 'undefined' ? context.options.verticalScrollMode : 1 /* Auto */),
            useShadows: context.options.useShadows
        });
        _this.scrollableElement.onScroll(function (e) {
            _this.render(e.scrollTop, e.height, e.scrollLeft, e.width, e.scrollWidth);
            _this._onDidScroll.fire();
        });
        if (isIE) {
            _this.wrapper.style.msTouchAction = 'none';
            _this.wrapper.style.msContentZooming = 'none';
        }
        else {
            Gesture.addTarget(_this.wrapper);
        }
        _this.rowsContainer = document.createElement('div');
        _this.rowsContainer.className = 'monaco-tree-rows';
        if (context.options.showTwistie) {
            _this.rowsContainer.className += ' show-twisties';
        }
        var focusTracker = trackFocus(_this.domNode);
        _this.viewListeners.push(focusTracker.onDidFocus(function () { return _this.onFocus(); }));
        _this.viewListeners.push(focusTracker.onDidBlur(function () { return _this.onBlur(); }));
        _this.viewListeners.push(focusTracker);
        _this.viewListeners.push(addDisposableListener(_this.domNode, 'keydown', function (e) { return _this.onKeyDown(e); }));
        _this.viewListeners.push(addDisposableListener(_this.domNode, 'keyup', function (e) { return _this.onKeyUp(e); }));
        _this.viewListeners.push(addDisposableListener(_this.domNode, 'mousedown', function (e) { return _this.onMouseDown(e); }));
        _this.viewListeners.push(addDisposableListener(_this.domNode, 'mouseup', function (e) { return _this.onMouseUp(e); }));
        _this.viewListeners.push(addDisposableListener(_this.wrapper, 'auxclick', function (e) {
            if (e && e.button === 1) {
                _this.onMouseMiddleClick(e);
            }
        }));
        _this.viewListeners.push(addDisposableListener(_this.wrapper, 'click', function (e) { return _this.onClick(e); }));
        _this.viewListeners.push(addDisposableListener(_this.domNode, 'contextmenu', function (e) { return _this.onContextMenu(e); }));
        _this.viewListeners.push(addDisposableListener(_this.wrapper, EventType$1.Tap, function (e) { return _this.onTap(e); }));
        _this.viewListeners.push(addDisposableListener(_this.wrapper, EventType$1.Change, function (e) { return _this.onTouchChange(e); }));
        if (isIE) {
            _this.viewListeners.push(addDisposableListener(_this.wrapper, 'MSPointerDown', function (e) { return _this.onMsPointerDown(e); }));
            _this.viewListeners.push(addDisposableListener(_this.wrapper, 'MSGestureTap', function (e) { return _this.onMsGestureTap(e); }));
            // these events come too fast, we throttle them
            _this.viewListeners.push(addDisposableThrottledListener(_this.wrapper, 'MSGestureChange', function (e) { return _this.onThrottledMsGestureChange(e); }, function (lastEvent, event) {
                event.stopPropagation();
                event.preventDefault();
                var result = { translationY: event.translationY, translationX: event.translationX };
                if (lastEvent) {
                    result.translationY += lastEvent.translationY;
                    result.translationX += lastEvent.translationX;
                }
                return result;
            }));
        }
        _this.viewListeners.push(addDisposableListener(window, 'dragover', function (e) { return _this.onDragOver(e); }));
        _this.viewListeners.push(addDisposableListener(_this.wrapper, 'drop', function (e) { return _this.onDrop(e); }));
        _this.viewListeners.push(addDisposableListener(window, 'dragend', function (e) { return _this.onDragEnd(e); }));
        _this.viewListeners.push(addDisposableListener(window, 'dragleave', function (e) { return _this.onDragOver(e); }));
        _this.wrapper.appendChild(_this.rowsContainer);
        _this.domNode.appendChild(_this.scrollableElement.getDomNode());
        container.appendChild(_this.domNode);
        _this.lastRenderTop = 0;
        _this.lastRenderHeight = 0;
        _this.didJustPressContextMenuKey = false;
        _this.currentDropTarget = null;
        _this.currentDropTargets = [];
        _this.shouldInvalidateDropReaction = false;
        _this.dragAndDropScrollInterval = null;
        _this.dragAndDropScrollTimeout = null;
        _this.onHiddenScrollTop = null;
        _this.onRowsChanged();
        _this.layout();
        _this.setupMSGesture();
        _this.applyStyles(context.options);
        return _this;
    }
    Object.defineProperty(TreeView.prototype, "onDOMFocus", {
        get: function () { return this._onDOMFocus.event; },
        enumerable: true,
        configurable: true
    });
    TreeView.prototype.applyStyles = function (styles) {
        this.treeStyler.style(styles);
    };
    TreeView.prototype.createViewItem = function (item) {
        return new ViewItem(this.context, item);
    };
    TreeView.prototype.getHTMLElement = function () {
        return this.domNode;
    };
    TreeView.prototype.focus = function () {
        this.domNode.focus();
    };
    TreeView.prototype.isFocused = function () {
        return document.activeElement === this.domNode;
    };
    TreeView.prototype.blur = function () {
        this.domNode.blur();
    };
    TreeView.prototype.setupMSGesture = function () {
        var _this = this;
        if (window.MSGesture) {
            this.msGesture = new MSGesture();
            setTimeout(function () { return _this.msGesture.target = _this.wrapper; }, 100); // TODO@joh, TODO@IETeam
        }
    };
    TreeView.prototype.isTreeVisible = function () {
        return this.onHiddenScrollTop === null;
    };
    TreeView.prototype.layout = function (height, width) {
        if (!this.isTreeVisible()) {
            return;
        }
        this.viewHeight = height || getContentHeight(this.wrapper); // render
        this.scrollHeight = this.getContentHeight();
        if (this.horizontalScrolling) {
            this.viewWidth = width || getContentWidth(this.wrapper);
        }
    };
    TreeView.prototype.render = function (scrollTop, viewHeight, scrollLeft, viewWidth, scrollWidth) {
        var i;
        var stop;
        var renderTop = scrollTop;
        var renderBottom = scrollTop + viewHeight;
        var thisRenderBottom = this.lastRenderTop + this.lastRenderHeight;
        // when view scrolls down, start rendering from the renderBottom
        for (i = this.indexAfter(renderBottom) - 1, stop = this.indexAt(Math.max(thisRenderBottom, renderTop)); i >= stop; i--) {
            this.insertItemInDOM(this.itemAtIndex(i));
        }
        // when view scrolls up, start rendering from either this.renderTop or renderBottom
        for (i = Math.min(this.indexAt(this.lastRenderTop), this.indexAfter(renderBottom)) - 1, stop = this.indexAt(renderTop); i >= stop; i--) {
            this.insertItemInDOM(this.itemAtIndex(i));
        }
        // when view scrolls down, start unrendering from renderTop
        for (i = this.indexAt(this.lastRenderTop), stop = Math.min(this.indexAt(renderTop), this.indexAfter(thisRenderBottom)); i < stop; i++) {
            this.removeItemFromDOM(this.itemAtIndex(i));
        }
        // when view scrolls up, start unrendering from either renderBottom this.renderTop
        for (i = Math.max(this.indexAfter(renderBottom), this.indexAt(this.lastRenderTop)), stop = this.indexAfter(thisRenderBottom); i < stop; i++) {
            this.removeItemFromDOM(this.itemAtIndex(i));
        }
        var topItem = this.itemAtIndex(this.indexAt(renderTop));
        if (topItem) {
            this.rowsContainer.style.top = (topItem.top - renderTop) + 'px';
        }
        if (this.horizontalScrolling) {
            this.rowsContainer.style.left = -scrollLeft + 'px';
            this.rowsContainer.style.width = Math.max(scrollWidth, viewWidth) + "px";
        }
        this.lastRenderTop = renderTop;
        this.lastRenderHeight = renderBottom - renderTop;
    };
    TreeView.prototype.setModel = function (newModel) {
        this.releaseModel();
        this.model = newModel;
        this.model.onRefresh(this.onRefreshing, this, this.modelListeners);
        this.model.onDidRefresh(this.onRefreshed, this, this.modelListeners);
        this.model.onSetInput(this.onClearingInput, this, this.modelListeners);
        this.model.onDidSetInput(this.onSetInput, this, this.modelListeners);
        this.model.onDidFocus(this.onModelFocusChange, this, this.modelListeners);
        this.model.onRefreshItemChildren(this.onItemChildrenRefreshing, this, this.modelListeners);
        this.model.onDidRefreshItemChildren(this.onItemChildrenRefreshed, this, this.modelListeners);
        this.model.onDidRefreshItem(this.onItemRefresh, this, this.modelListeners);
        this.model.onExpandItem(this.onItemExpanding, this, this.modelListeners);
        this.model.onDidExpandItem(this.onItemExpanded, this, this.modelListeners);
        this.model.onCollapseItem(this.onItemCollapsing, this, this.modelListeners);
        this.model.onDidRevealItem(this.onItemReveal, this, this.modelListeners);
        this.model.onDidAddTraitItem(this.onItemAddTrait, this, this.modelListeners);
        this.model.onDidRemoveTraitItem(this.onItemRemoveTrait, this, this.modelListeners);
    };
    TreeView.prototype.onRefreshing = function () {
        this.isRefreshing = true;
    };
    TreeView.prototype.onRefreshed = function () {
        this.isRefreshing = false;
        this.onRowsChanged();
    };
    TreeView.prototype.onRowsChanged = function (scrollTop) {
        if (scrollTop === void 0) { scrollTop = this.scrollTop; }
        if (this.isRefreshing) {
            return;
        }
        this.scrollTop = scrollTop;
        this.updateScrollWidth();
    };
    TreeView.prototype.updateScrollWidth = function () {
        var _this = this;
        if (!this.horizontalScrolling) {
            return;
        }
        this.contentWidthUpdateDelayer.trigger(function () {
            var keys = Object.keys(_this.items);
            var scrollWidth = 0;
            for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
                var key = keys_1[_i];
                scrollWidth = Math.max(scrollWidth, _this.items[key].width);
            }
            _this.scrollWidth = scrollWidth + 10 /* scrollbar */;
        });
    };
    TreeView.prototype.focusNextPage = function (eventPayload) {
        var _this = this;
        var lastPageIndex = this.indexAt(this.scrollTop + this.viewHeight);
        lastPageIndex = lastPageIndex === 0 ? 0 : lastPageIndex - 1;
        var lastPageElement = this.itemAtIndex(lastPageIndex).model.getElement();
        var currentlyFocusedElement = this.model.getFocus();
        if (currentlyFocusedElement !== lastPageElement) {
            this.model.setFocus(lastPageElement, eventPayload);
        }
        else {
            var previousScrollTop = this.scrollTop;
            this.scrollTop += this.viewHeight;
            if (this.scrollTop !== previousScrollTop) {
                // Let the scroll event listener run
                setTimeout(function () {
                    _this.focusNextPage(eventPayload);
                }, 0);
            }
        }
    };
    TreeView.prototype.focusPreviousPage = function (eventPayload) {
        var _this = this;
        var firstPageIndex;
        if (this.scrollTop === 0) {
            firstPageIndex = this.indexAt(this.scrollTop);
        }
        else {
            firstPageIndex = this.indexAfter(this.scrollTop - 1);
        }
        var firstPageElement = this.itemAtIndex(firstPageIndex).model.getElement();
        var currentlyFocusedElement = this.model.getFocus();
        if (currentlyFocusedElement !== firstPageElement) {
            this.model.setFocus(firstPageElement, eventPayload);
        }
        else {
            var previousScrollTop = this.scrollTop;
            this.scrollTop -= this.viewHeight;
            if (this.scrollTop !== previousScrollTop) {
                // Let the scroll event listener run
                setTimeout(function () {
                    _this.focusPreviousPage(eventPayload);
                }, 0);
            }
        }
    };
    Object.defineProperty(TreeView.prototype, "viewHeight", {
        get: function () {
            var scrollDimensions = this.scrollableElement.getScrollDimensions();
            return scrollDimensions.height;
        },
        set: function (height) {
            this.scrollableElement.setScrollDimensions({ height: height });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeView.prototype, "scrollHeight", {
        set: function (scrollHeight) {
            this.scrollableElement.setScrollDimensions({ scrollHeight: scrollHeight });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeView.prototype, "viewWidth", {
        get: function () {
            var scrollDimensions = this.scrollableElement.getScrollDimensions();
            return scrollDimensions.width;
        },
        set: function (viewWidth) {
            this.scrollableElement.setScrollDimensions({ width: viewWidth });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeView.prototype, "scrollWidth", {
        set: function (scrollWidth) {
            this.scrollableElement.setScrollDimensions({ scrollWidth: scrollWidth });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeView.prototype, "scrollTop", {
        get: function () {
            var scrollPosition = this.scrollableElement.getScrollPosition();
            return scrollPosition.scrollTop;
        },
        set: function (scrollTop) {
            this.scrollableElement.setScrollDimensions({
                scrollHeight: this.getContentHeight()
            });
            this.scrollableElement.setScrollPosition({
                scrollTop: scrollTop
            });
        },
        enumerable: true,
        configurable: true
    });
    // Events
    TreeView.prototype.onClearingInput = function (e) {
        var item = e.item;
        if (item) {
            this.onRemoveItems(new MappedIterator(item.getNavigator(), function (item) { return item && item.id; }));
            this.onRowsChanged();
        }
    };
    TreeView.prototype.onSetInput = function (e) {
        this.context.cache.garbageCollect();
        this.inputItem = new RootViewItem(this.context, e.item, this.wrapper);
    };
    TreeView.prototype.onItemChildrenRefreshing = function (e) {
        var item = e.item;
        var viewItem = this.items[item.id];
        if (viewItem && this.context.options.showLoading) {
            viewItem.loadingTimer = setTimeout(function () {
                viewItem.loadingTimer = 0;
                viewItem.loading = true;
            }, TreeView.LOADING_DECORATION_DELAY);
        }
        if (!e.isNested) {
            var childrenIds = [];
            var navigator = item.getNavigator();
            var childItem;
            while (childItem = navigator.next()) {
                childrenIds.push(childItem.id);
            }
            this.refreshingPreviousChildrenIds[item.id] = childrenIds;
        }
    };
    TreeView.prototype.onItemChildrenRefreshed = function (e) {
        var _this = this;
        var item = e.item;
        var viewItem = this.items[item.id];
        if (viewItem) {
            if (viewItem.loadingTimer) {
                clearTimeout(viewItem.loadingTimer);
                viewItem.loadingTimer = 0;
            }
            viewItem.loading = false;
        }
        if (!e.isNested) {
            var previousChildrenIds = this.refreshingPreviousChildrenIds[item.id];
            var afterModelItems = [];
            var navigator = item.getNavigator();
            var childItem;
            while (childItem = navigator.next()) {
                afterModelItems.push(childItem);
            }
            var skipDiff = Math.abs(previousChildrenIds.length - afterModelItems.length) > 1000;
            var diff = void 0;
            var doToInsertItemsAlreadyExist = void 0;
            if (!skipDiff) {
                var lcs = new LcsDiff({
                    getLength: function () { return previousChildrenIds.length; },
                    getElementAtIndex: function (i) { return previousChildrenIds[i]; }
                }, {
                    getLength: function () { return afterModelItems.length; },
                    getElementAtIndex: function (i) { return afterModelItems[i].id; }
                }, null);
                diff = lcs.ComputeDiff(false);
                // this means that the result of the diff algorithm would result
                // in inserting items that were already registered. this can only
                // happen if the data provider returns bad ids OR if the sorting
                // of the elements has changed
                doToInsertItemsAlreadyExist = diff.some(function (d) {
                    if (d.modifiedLength > 0) {
                        for (var i = d.modifiedStart, len = d.modifiedStart + d.modifiedLength; i < len; i++) {
                            if (_this.items.hasOwnProperty(afterModelItems[i].id)) {
                                return true;
                            }
                        }
                    }
                    return false;
                });
            }
            // 50 is an optimization number, at some point we're better off
            // just replacing everything
            if (!skipDiff && !doToInsertItemsAlreadyExist && diff.length < 50) {
                for (var i = 0, len = diff.length; i < len; i++) {
                    var diffChange = diff[i];
                    if (diffChange.originalLength > 0) {
                        this.onRemoveItems(new ArrayIterator(previousChildrenIds, diffChange.originalStart, diffChange.originalStart + diffChange.originalLength));
                    }
                    if (diffChange.modifiedLength > 0) {
                        var beforeItem = afterModelItems[diffChange.modifiedStart - 1] || item;
                        beforeItem = beforeItem.getDepth() > 0 ? beforeItem : null;
                        this.onInsertItems(new ArrayIterator(afterModelItems, diffChange.modifiedStart, diffChange.modifiedStart + diffChange.modifiedLength), beforeItem ? beforeItem.id : null);
                    }
                }
            }
            else if (skipDiff || diff.length) {
                this.onRemoveItems(new ArrayIterator(previousChildrenIds));
                this.onInsertItems(new ArrayIterator(afterModelItems), item.getDepth() > 0 ? item.id : null);
            }
            if (skipDiff || diff.length) {
                this.onRowsChanged();
            }
        }
    };
    TreeView.prototype.onItemRefresh = function (item) {
        this.onItemsRefresh([item]);
    };
    TreeView.prototype.onItemsRefresh = function (items) {
        var _this = this;
        this.onRefreshItemSet(items.filter(function (item) { return _this.items.hasOwnProperty(item.id); }));
        this.onRowsChanged();
    };
    TreeView.prototype.onItemExpanding = function (e) {
        var viewItem = this.items[e.item.id];
        if (viewItem) {
            viewItem.expanded = true;
        }
    };
    TreeView.prototype.onItemExpanded = function (e) {
        var item = e.item;
        var viewItem = this.items[item.id];
        if (viewItem) {
            viewItem.expanded = true;
            var height = this.onInsertItems(item.getNavigator(), item.id);
            var scrollTop = this.scrollTop;
            if (viewItem.top + viewItem.height <= this.scrollTop) {
                scrollTop += height;
            }
            this.onRowsChanged(scrollTop);
        }
    };
    TreeView.prototype.onItemCollapsing = function (e) {
        var item = e.item;
        var viewItem = this.items[item.id];
        if (viewItem) {
            viewItem.expanded = false;
            this.onRemoveItems(new MappedIterator(item.getNavigator(), function (item) { return item && item.id; }));
            this.onRowsChanged();
        }
    };
    TreeView.prototype.onItemReveal = function (e) {
        var item = e.item;
        var relativeTop = e.relativeTop;
        var viewItem = this.items[item.id];
        if (viewItem) {
            if (relativeTop !== null) {
                relativeTop = relativeTop < 0 ? 0 : relativeTop;
                relativeTop = relativeTop > 1 ? 1 : relativeTop;
                // y = mx + b
                var m = viewItem.height - this.viewHeight;
                this.scrollTop = m * relativeTop + viewItem.top;
            }
            else {
                var viewItemBottom = viewItem.top + viewItem.height;
                var wrapperBottom = this.scrollTop + this.viewHeight;
                if (viewItem.top < this.scrollTop) {
                    this.scrollTop = viewItem.top;
                }
                else if (viewItemBottom >= wrapperBottom) {
                    this.scrollTop = viewItemBottom - this.viewHeight;
                }
            }
        }
    };
    TreeView.prototype.onItemAddTrait = function (e) {
        var item = e.item;
        var trait = e.trait;
        var viewItem = this.items[item.id];
        if (viewItem) {
            viewItem.addClass(trait);
        }
        if (trait === 'highlighted') {
            addClass(this.domNode, trait);
            // Ugly Firefox fix: input fields can't be selected if parent nodes are draggable
            if (viewItem) {
                this.highlightedItemWasDraggable = !!viewItem.draggable;
                if (viewItem.draggable) {
                    viewItem.draggable = false;
                }
            }
        }
    };
    TreeView.prototype.onItemRemoveTrait = function (e) {
        var item = e.item;
        var trait = e.trait;
        var viewItem = this.items[item.id];
        if (viewItem) {
            viewItem.removeClass(trait);
        }
        if (trait === 'highlighted') {
            removeClass(this.domNode, trait);
            // Ugly Firefox fix: input fields can't be selected if parent nodes are draggable
            if (this.highlightedItemWasDraggable) {
                viewItem.draggable = true;
            }
            this.highlightedItemWasDraggable = false;
        }
    };
    TreeView.prototype.onModelFocusChange = function () {
        var focus = this.model && this.model.getFocus();
        toggleClass(this.domNode, 'no-focused-item', !focus);
        // ARIA
        if (focus) {
            this.domNode.setAttribute('aria-activedescendant', safeBtoa(this.context.dataSource.getId(this.context.tree, focus)));
        }
        else {
            this.domNode.removeAttribute('aria-activedescendant');
        }
    };
    // HeightMap "events"
    TreeView.prototype.onInsertItem = function (item) {
        var _this = this;
        item.onDragStart = function (e) { _this.onDragStart(item, e); };
        item.needsRender = true;
        this.refreshViewItem(item);
        this.items[item.id] = item;
    };
    TreeView.prototype.onRefreshItem = function (item, needsRender) {
        if (needsRender === void 0) { needsRender = false; }
        item.needsRender = item.needsRender || needsRender;
        this.refreshViewItem(item);
    };
    TreeView.prototype.onRemoveItem = function (item) {
        this.removeItemFromDOM(item);
        item.dispose();
        delete this.items[item.id];
    };
    // ViewItem refresh
    TreeView.prototype.refreshViewItem = function (item) {
        item.render();
        if (this.shouldBeRendered(item)) {
            this.insertItemInDOM(item);
        }
        else {
            this.removeItemFromDOM(item);
        }
    };
    // DOM Events
    TreeView.prototype.onClick = function (e) {
        if (this.lastPointerType && this.lastPointerType !== 'mouse') {
            return;
        }
        var event = new StandardMouseEvent(e);
        var item = this.getItemAround(event.target);
        if (!item) {
            return;
        }
        if (isIE && Date.now() - this.lastClickTimeStamp < 300) {
            // IE10+ doesn't set the detail property correctly. While IE10 simply
            // counts the number of clicks, IE11 reports always 1. To align with
            // other browser, we set the value to 2 if clicks events come in a 300ms
            // sequence.
            event.detail = 2;
        }
        this.lastClickTimeStamp = Date.now();
        this.context.controller.onClick(this.context.tree, item.model.getElement(), event);
    };
    TreeView.prototype.onMouseMiddleClick = function (e) {
        if (!this.context.controller.onMouseMiddleClick) {
            return;
        }
        var event = new StandardMouseEvent(e);
        var item = this.getItemAround(event.target);
        if (!item) {
            return;
        }
        this.context.controller.onMouseMiddleClick(this.context.tree, item.model.getElement(), event);
    };
    TreeView.prototype.onMouseDown = function (e) {
        this.didJustPressContextMenuKey = false;
        if (!this.context.controller.onMouseDown) {
            return;
        }
        if (this.lastPointerType && this.lastPointerType !== 'mouse') {
            return;
        }
        var event = new StandardMouseEvent(e);
        if (event.ctrlKey && isNative && isMacintosh) {
            return;
        }
        var item = this.getItemAround(event.target);
        if (!item) {
            return;
        }
        this.context.controller.onMouseDown(this.context.tree, item.model.getElement(), event);
    };
    TreeView.prototype.onMouseUp = function (e) {
        if (!this.context.controller.onMouseUp) {
            return;
        }
        if (this.lastPointerType && this.lastPointerType !== 'mouse') {
            return;
        }
        var event = new StandardMouseEvent(e);
        if (event.ctrlKey && isNative && isMacintosh) {
            return;
        }
        var item = this.getItemAround(event.target);
        if (!item) {
            return;
        }
        this.context.controller.onMouseUp(this.context.tree, item.model.getElement(), event);
    };
    TreeView.prototype.onTap = function (e) {
        var item = this.getItemAround(e.initialTarget);
        if (!item) {
            return;
        }
        this.context.controller.onTap(this.context.tree, item.model.getElement(), e);
    };
    TreeView.prototype.onTouchChange = function (event) {
        event.preventDefault();
        event.stopPropagation();
        this.scrollTop -= event.translationY;
    };
    TreeView.prototype.onContextMenu = function (event) {
        var resultEvent;
        var element;
        if (event instanceof KeyboardEvent || this.didJustPressContextMenuKey) {
            this.didJustPressContextMenuKey = false;
            var keyboardEvent = new StandardKeyboardEvent(event);
            element = this.model.getFocus();
            var position;
            if (!element) {
                element = this.model.getInput();
                position = getDomNodePagePosition(this.inputItem.element);
            }
            else {
                var id = this.context.dataSource.getId(this.context.tree, element);
                var viewItem = this.items[id];
                position = getDomNodePagePosition(viewItem.element);
            }
            resultEvent = new KeyboardContextMenuEvent(position.left + position.width, position.top, keyboardEvent);
        }
        else {
            var mouseEvent = new StandardMouseEvent(event);
            var item = this.getItemAround(mouseEvent.target);
            if (!item) {
                return;
            }
            element = item.model.getElement();
            resultEvent = new MouseContextMenuEvent(mouseEvent);
        }
        this.context.controller.onContextMenu(this.context.tree, element, resultEvent);
    };
    TreeView.prototype.onKeyDown = function (e) {
        var event = new StandardKeyboardEvent(e);
        this.didJustPressContextMenuKey = event.keyCode === 58 /* ContextMenu */ || (event.shiftKey && event.keyCode === 68 /* F10 */);
        if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
            return; // Ignore event if target is a form input field (avoids browser specific issues)
        }
        if (this.didJustPressContextMenuKey) {
            event.preventDefault();
            event.stopPropagation();
        }
        this.context.controller.onKeyDown(this.context.tree, event);
    };
    TreeView.prototype.onKeyUp = function (e) {
        if (this.didJustPressContextMenuKey) {
            this.onContextMenu(e);
        }
        this.didJustPressContextMenuKey = false;
        this.context.controller.onKeyUp(this.context.tree, new StandardKeyboardEvent(e));
    };
    TreeView.prototype.onDragStart = function (item, e) {
        if (this.model.getHighlight()) {
            return;
        }
        var element = item.model.getElement();
        var selection = this.model.getSelection();
        var elements;
        if (selection.indexOf(element) > -1) {
            elements = selection;
        }
        else {
            elements = [element];
        }
        e.dataTransfer.effectAllowed = 'copyMove';
        e.dataTransfer.setData(DataTransfers.RESOURCES, JSON.stringify([item.uri]));
        if (e.dataTransfer.setDragImage) {
            var label = void 0;
            if (this.context.dnd.getDragLabel) {
                label = this.context.dnd.getDragLabel(this.context.tree, elements);
            }
            else {
                label = String(elements.length);
            }
            var dragImage_1 = document.createElement('div');
            dragImage_1.className = 'monaco-tree-drag-image';
            dragImage_1.textContent = label;
            document.body.appendChild(dragImage_1);
            e.dataTransfer.setDragImage(dragImage_1, -10, -10);
            setTimeout(function () { return document.body.removeChild(dragImage_1); }, 0);
        }
        this.currentDragAndDropData = new ElementsDragAndDropData(elements);
        TreeView.currentExternalDragAndDropData = new ExternalElementsDragAndDropData(elements);
        this.context.dnd.onDragStart(this.context.tree, this.currentDragAndDropData, new DragMouseEvent(e));
    };
    TreeView.prototype.setupDragAndDropScrollInterval = function () {
        var _this = this;
        var viewTop = getTopLeftOffset(this.wrapper).top;
        if (!this.dragAndDropScrollInterval) {
            this.dragAndDropScrollInterval = window.setInterval(function () {
                if (_this.dragAndDropMouseY === undefined) {
                    return;
                }
                var diff = _this.dragAndDropMouseY - viewTop;
                var scrollDiff = 0;
                var upperLimit = _this.viewHeight - 35;
                if (diff < 35) {
                    scrollDiff = Math.max(-14, 0.2 * (diff - 35));
                }
                else if (diff > upperLimit) {
                    scrollDiff = Math.min(14, 0.2 * (diff - upperLimit));
                }
                _this.scrollTop += scrollDiff;
            }, 10);
            this.cancelDragAndDropScrollTimeout();
            this.dragAndDropScrollTimeout = window.setTimeout(function () {
                _this.cancelDragAndDropScrollInterval();
                _this.dragAndDropScrollTimeout = null;
            }, 1000);
        }
    };
    TreeView.prototype.cancelDragAndDropScrollInterval = function () {
        if (this.dragAndDropScrollInterval) {
            window.clearInterval(this.dragAndDropScrollInterval);
            this.dragAndDropScrollInterval = null;
        }
        this.cancelDragAndDropScrollTimeout();
    };
    TreeView.prototype.cancelDragAndDropScrollTimeout = function () {
        if (this.dragAndDropScrollTimeout) {
            window.clearTimeout(this.dragAndDropScrollTimeout);
            this.dragAndDropScrollTimeout = null;
        }
    };
    TreeView.prototype.onDragOver = function (e) {
        var _this = this;
        var event = new DragMouseEvent(e);
        var viewItem = this.getItemAround(event.target);
        if (!viewItem || (event.posx === 0 && event.posy === 0 && event.browserEvent.type === EventType.DRAG_LEAVE)) {
            // dragging outside of tree
            if (this.currentDropTarget) {
                // clear previously hovered element feedback
                this.currentDropTargets.forEach(function (i) { return i.dropTarget = false; });
                this.currentDropTargets = [];
                this.currentDropDisposable.dispose();
            }
            this.cancelDragAndDropScrollInterval();
            this.currentDropTarget = null;
            this.currentDropElement = null;
            this.dragAndDropMouseY = null;
            return false;
        }
        // dragging inside the tree
        this.setupDragAndDropScrollInterval();
        this.dragAndDropMouseY = event.posy;
        if (!this.currentDragAndDropData) {
            // just started dragging
            if (TreeView.currentExternalDragAndDropData) {
                this.currentDragAndDropData = TreeView.currentExternalDragAndDropData;
            }
            else {
                if (!event.dataTransfer.types) {
                    return false;
                }
                this.currentDragAndDropData = new DesktopDragAndDropData();
            }
        }
        this.currentDragAndDropData.update(event);
        var element;
        var item = viewItem.model;
        var reaction;
        // check the bubble up behavior
        do {
            element = item ? item.getElement() : this.model.getInput();
            reaction = this.context.dnd.onDragOver(this.context.tree, this.currentDragAndDropData, element, event);
            if (!reaction || reaction.bubble !== 1 /* BUBBLE_UP */) {
                break;
            }
            item = item && item.parent;
        } while (item);
        if (!item) {
            this.currentDropElement = null;
            return false;
        }
        var canDrop = reaction && reaction.accept;
        if (canDrop) {
            this.currentDropElement = item.getElement();
            event.preventDefault();
            event.dataTransfer.dropEffect = reaction.effect === 0 /* COPY */ ? 'copy' : 'move';
        }
        else {
            this.currentDropElement = null;
        }
        // item is the model item where drop() should be called
        // can be null
        var currentDropTarget = item.id === this.inputItem.id ? this.inputItem : this.items[item.id];
        if (this.shouldInvalidateDropReaction || this.currentDropTarget !== currentDropTarget || !reactionEquals(this.currentDropElementReaction, reaction)) {
            this.shouldInvalidateDropReaction = false;
            if (this.currentDropTarget) {
                this.currentDropTargets.forEach(function (i) { return i.dropTarget = false; });
                this.currentDropTargets = [];
                this.currentDropDisposable.dispose();
            }
            this.currentDropTarget = currentDropTarget;
            this.currentDropElementReaction = reaction;
            if (canDrop) {
                // setup hover feedback for drop target
                if (this.currentDropTarget) {
                    this.currentDropTarget.dropTarget = true;
                    this.currentDropTargets.push(this.currentDropTarget);
                }
                if (reaction.bubble === 0 /* BUBBLE_DOWN */) {
                    var nav = item.getNavigator();
                    var child;
                    while (child = nav.next()) {
                        viewItem = this.items[child.id];
                        if (viewItem) {
                            viewItem.dropTarget = true;
                            this.currentDropTargets.push(viewItem);
                        }
                    }
                }
                if (reaction.autoExpand) {
                    var timeoutPromise_1 = timeout(500);
                    this.currentDropDisposable = toDisposable(function () { return timeoutPromise_1.cancel(); });
                    timeoutPromise_1
                        .then(function () { return _this.context.tree.expand(_this.currentDropElement); })
                        .then(function () { return _this.shouldInvalidateDropReaction = true; });
                }
            }
        }
        return true;
    };
    TreeView.prototype.onDrop = function (e) {
        if (this.currentDropElement) {
            var event = new DragMouseEvent(e);
            event.preventDefault();
            this.currentDragAndDropData.update(event);
            this.context.dnd.drop(this.context.tree, this.currentDragAndDropData, this.currentDropElement, event);
            this.onDragEnd(e);
        }
        this.cancelDragAndDropScrollInterval();
    };
    TreeView.prototype.onDragEnd = function (e) {
        if (this.currentDropTarget) {
            this.currentDropTargets.forEach(function (i) { return i.dropTarget = false; });
            this.currentDropTargets = [];
        }
        this.currentDropDisposable.dispose();
        this.cancelDragAndDropScrollInterval();
        this.currentDragAndDropData = null;
        TreeView.currentExternalDragAndDropData = null;
        this.currentDropElement = null;
        this.currentDropTarget = null;
        this.dragAndDropMouseY = null;
    };
    TreeView.prototype.onFocus = function () {
        if (!this.context.options.alwaysFocused) {
            addClass(this.domNode, 'focused');
        }
        this._onDOMFocus.fire();
    };
    TreeView.prototype.onBlur = function () {
        if (!this.context.options.alwaysFocused) {
            removeClass(this.domNode, 'focused');
        }
        this.domNode.removeAttribute('aria-activedescendant'); // ARIA
        this._onDOMBlur.fire();
    };
    // MS specific DOM Events
    TreeView.prototype.onMsPointerDown = function (event) {
        if (!this.msGesture) {
            return;
        }
        // Circumvent IE11 breaking change in e.pointerType & TypeScript's stale definitions
        var pointerType = event.pointerType;
        if (pointerType === (event.MSPOINTER_TYPE_MOUSE || 'mouse')) {
            this.lastPointerType = 'mouse';
            return;
        }
        else if (pointerType === (event.MSPOINTER_TYPE_TOUCH || 'touch')) {
            this.lastPointerType = 'touch';
        }
        else {
            return;
        }
        event.stopPropagation();
        event.preventDefault();
        this.msGesture.addPointer(event.pointerId);
    };
    TreeView.prototype.onThrottledMsGestureChange = function (event) {
        this.scrollTop -= event.translationY;
    };
    TreeView.prototype.onMsGestureTap = function (event) {
        event.initialTarget = document.elementFromPoint(event.clientX, event.clientY);
        this.onTap(event);
    };
    // DOM changes
    TreeView.prototype.insertItemInDOM = function (item) {
        var elementAfter = null;
        var itemAfter = this.itemAfter(item);
        if (itemAfter && itemAfter.element) {
            elementAfter = itemAfter.element;
        }
        item.insertInDOM(this.rowsContainer, elementAfter);
    };
    TreeView.prototype.removeItemFromDOM = function (item) {
        if (!item) {
            return;
        }
        item.removeFromDOM();
    };
    // Helpers
    TreeView.prototype.shouldBeRendered = function (item) {
        return item.top < this.lastRenderTop + this.lastRenderHeight && item.top + item.height > this.lastRenderTop;
    };
    TreeView.prototype.getItemAround = function (element) {
        var candidate = this.inputItem;
        do {
            if (element[TreeView.BINDING]) {
                candidate = element[TreeView.BINDING];
            }
            if (element === this.wrapper || element === this.domNode) {
                return candidate;
            }
            if (element === this.scrollableElement.getDomNode() || element === document.body) {
                return null;
            }
        } while (element = element.parentElement);
        return undefined;
    };
    // Cleanup
    TreeView.prototype.releaseModel = function () {
        if (this.model) {
            this.modelListeners = dispose(this.modelListeners);
            this.model = null;
        }
    };
    TreeView.prototype.dispose = function () {
        var _this = this;
        // TODO@joao: improve
        this.scrollableElement.dispose();
        this.releaseModel();
        this.modelListeners = null;
        this.viewListeners = dispose(this.viewListeners);
        this._onDOMFocus.dispose();
        this._onDOMBlur.dispose();
        if (this.domNode.parentNode) {
            this.domNode.parentNode.removeChild(this.domNode);
        }
        this.domNode = null;
        if (this.items) {
            Object.keys(this.items).forEach(function (key) { return _this.items[key].removeFromDOM(); });
            this.items = null;
        }
        if (this.context.cache) {
            this.context.cache.dispose();
            this.context.cache = null;
        }
        _super.prototype.dispose.call(this);
    };
    TreeView.BINDING = 'monaco-tree-row';
    TreeView.LOADING_DECORATION_DELAY = 800;
    TreeView.counter = 0;
    TreeView.currentExternalDragAndDropData = null;
    return TreeView;
}(HeightMap));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var TreeContext = /** @class */ (function () {
    function TreeContext(tree, configuration, options) {
        if (options === void 0) { options = {}; }
        this.tree = tree;
        this.configuration = configuration;
        this.options = options;
        if (!configuration.dataSource) {
            throw new Error('You must provide a Data Source to the tree.');
        }
        this.dataSource = configuration.dataSource;
        this.renderer = configuration.renderer;
        this.controller = configuration.controller || new DefaultController({ clickBehavior: 1 /* ON_MOUSE_UP */, keyboardSupport: typeof options.keyboardSupport !== 'boolean' || options.keyboardSupport });
        this.dnd = configuration.dnd || new DefaultDragAndDrop();
        this.filter = configuration.filter || new DefaultFilter();
        this.sorter = configuration.sorter || null;
        this.accessibilityProvider = configuration.accessibilityProvider || new DefaultAccessibilityProvider();
        this.styler = configuration.styler || null;
    }
    return TreeContext;
}());
var defaultStyles$1 = {
    listFocusBackground: Color.fromHex('#073655'),
    listActiveSelectionBackground: Color.fromHex('#0E639C'),
    listActiveSelectionForeground: Color.fromHex('#FFFFFF'),
    listFocusAndSelectionBackground: Color.fromHex('#094771'),
    listFocusAndSelectionForeground: Color.fromHex('#FFFFFF'),
    listInactiveSelectionBackground: Color.fromHex('#3F3F46'),
    listHoverBackground: Color.fromHex('#2A2D2E'),
    listDropBackground: Color.fromHex('#383B3D')
};
var Tree = /** @class */ (function () {
    function Tree(container, configuration, options) {
        if (options === void 0) { options = {}; }
        this._onDidChangeFocus = new Relay();
        this.onDidChangeFocus = this._onDidChangeFocus.event;
        this._onDidChangeSelection = new Relay();
        this.onDidChangeSelection = this._onDidChangeSelection.event;
        this._onHighlightChange = new Relay();
        this._onDidExpandItem = new Relay();
        this._onDidCollapseItem = new Relay();
        this._onDispose = new Emitter();
        this.onDidDispose = this._onDispose.event;
        this.container = container;
        mixin(options, defaultStyles$1, false);
        options.twistiePixels = typeof options.twistiePixels === 'number' ? options.twistiePixels : 32;
        options.showTwistie = options.showTwistie === false ? false : true;
        options.indentPixels = typeof options.indentPixels === 'number' ? options.indentPixels : 12;
        options.alwaysFocused = options.alwaysFocused === true ? true : false;
        options.useShadows = options.useShadows === false ? false : true;
        options.paddingOnRow = options.paddingOnRow === false ? false : true;
        options.showLoading = options.showLoading === false ? false : true;
        this.context = new TreeContext(this, configuration, options);
        this.model = new TreeModel(this.context);
        this.view = new TreeView(this.context, this.container);
        this.view.setModel(this.model);
        this._onDidChangeFocus.input = this.model.onDidFocus;
        this._onDidChangeSelection.input = this.model.onDidSelect;
        this._onHighlightChange.input = this.model.onDidHighlight;
        this._onDidExpandItem.input = this.model.onDidExpandItem;
        this._onDidCollapseItem.input = this.model.onDidCollapseItem;
    }
    Tree.prototype.style = function (styles) {
        this.view.applyStyles(styles);
    };
    Object.defineProperty(Tree.prototype, "onDidFocus", {
        get: function () {
            return this.view && this.view.onDOMFocus;
        },
        enumerable: true,
        configurable: true
    });
    Tree.prototype.getHTMLElement = function () {
        return this.view.getHTMLElement();
    };
    Tree.prototype.layout = function (height, width) {
        this.view.layout(height, width);
    };
    Tree.prototype.domFocus = function () {
        this.view.focus();
    };
    Tree.prototype.isDOMFocused = function () {
        return this.view.isFocused();
    };
    Tree.prototype.domBlur = function () {
        this.view.blur();
    };
    Tree.prototype.setInput = function (element) {
        return this.model.setInput(element);
    };
    Tree.prototype.getInput = function () {
        return this.model.getInput();
    };
    Tree.prototype.refresh = function (element, recursive) {
        if (element === void 0) { element = null; }
        if (recursive === void 0) { recursive = true; }
        return this.model.refresh(element, recursive);
    };
    Tree.prototype.expand = function (element) {
        return this.model.expand(element);
    };
    Tree.prototype.collapse = function (element, recursive) {
        if (recursive === void 0) { recursive = false; }
        return this.model.collapse(element, recursive);
    };
    Tree.prototype.toggleExpansion = function (element, recursive) {
        if (recursive === void 0) { recursive = false; }
        return this.model.toggleExpansion(element, recursive);
    };
    Tree.prototype.isExpanded = function (element) {
        return this.model.isExpanded(element);
    };
    Tree.prototype.reveal = function (element, relativeTop) {
        if (relativeTop === void 0) { relativeTop = null; }
        return this.model.reveal(element, relativeTop);
    };
    Tree.prototype.getHighlight = function () {
        return this.model.getHighlight();
    };
    Tree.prototype.clearHighlight = function (eventPayload) {
        this.model.setHighlight(null, eventPayload);
    };
    Tree.prototype.setSelection = function (elements, eventPayload) {
        this.model.setSelection(elements, eventPayload);
    };
    Tree.prototype.getSelection = function () {
        return this.model.getSelection();
    };
    Tree.prototype.clearSelection = function (eventPayload) {
        this.model.setSelection([], eventPayload);
    };
    Tree.prototype.setFocus = function (element, eventPayload) {
        this.model.setFocus(element, eventPayload);
    };
    Tree.prototype.getFocus = function () {
        return this.model.getFocus();
    };
    Tree.prototype.focusNext = function (count, eventPayload) {
        this.model.focusNext(count, eventPayload);
    };
    Tree.prototype.focusPrevious = function (count, eventPayload) {
        this.model.focusPrevious(count, eventPayload);
    };
    Tree.prototype.focusParent = function (eventPayload) {
        this.model.focusParent(eventPayload);
    };
    Tree.prototype.focusFirstChild = function (eventPayload) {
        this.model.focusFirstChild(eventPayload);
    };
    Tree.prototype.focusFirst = function (eventPayload, from) {
        this.model.focusFirst(eventPayload, from);
    };
    Tree.prototype.focusNth = function (index, eventPayload) {
        this.model.focusNth(index, eventPayload);
    };
    Tree.prototype.focusLast = function (eventPayload, from) {
        this.model.focusLast(eventPayload, from);
    };
    Tree.prototype.focusNextPage = function (eventPayload) {
        this.view.focusNextPage(eventPayload);
    };
    Tree.prototype.focusPreviousPage = function (eventPayload) {
        this.view.focusPreviousPage(eventPayload);
    };
    Tree.prototype.clearFocus = function (eventPayload) {
        this.model.setFocus(null, eventPayload);
    };
    Tree.prototype.dispose = function () {
        this._onDispose.fire();
        if (this.model !== null) {
            this.model.dispose();
            this.model = null;
        }
        if (this.view !== null) {
            this.view.dispose();
            this.view = null;
        }
        this._onDidChangeFocus.dispose();
        this._onDidChangeSelection.dispose();
        this._onHighlightChange.dispose();
        this._onDidExpandItem.dispose();
        this._onDidCollapseItem.dispose();
        this._onDispose.dispose();
    };
    return Tree;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1S = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var css_done = 'done';
var css_active = 'active';
var css_infinite = 'infinite';
var css_discrete = 'discrete';
var css_progress_container = 'monaco-progress-container';
var css_progress_bit = 'progress-bit';
var defaultOpts$2 = {
    progressBarBackground: Color.fromHex('#0E70C0')
};
/**
 * A progress bar with support for infinite or discrete progress.
 */
var ProgressBar = /** @class */ (function (_super) {
    __extends$1S(ProgressBar, _super);
    function ProgressBar(container, options) {
        var _this = _super.call(this) || this;
        _this.options = options || Object.create(null);
        mixin(_this.options, defaultOpts$2, false);
        _this.workedVal = 0;
        _this.progressBarBackground = _this.options.progressBarBackground;
        _this._register(_this.showDelayedScheduler = new RunOnceScheduler(function () { return show(_this.element); }, 0));
        _this.create(container);
        return _this;
    }
    ProgressBar.prototype.create = function (container) {
        this.element = document.createElement('div');
        addClass(this.element, css_progress_container);
        container.appendChild(this.element);
        this.bit = document.createElement('div');
        addClass(this.bit, css_progress_bit);
        this.element.appendChild(this.bit);
        this.applyStyles();
    };
    ProgressBar.prototype.off = function () {
        this.bit.style.width = 'inherit';
        this.bit.style.opacity = '1';
        removeClasses(this.element, css_active, css_infinite, css_discrete);
        this.workedVal = 0;
        this.totalWork = undefined;
    };
    /**
     * Stops the progressbar from showing any progress instantly without fading out.
     */
    ProgressBar.prototype.stop = function () {
        return this.doDone(false);
    };
    ProgressBar.prototype.doDone = function (delayed) {
        var _this = this;
        addClass(this.element, css_done);
        // let it grow to 100% width and hide afterwards
        if (!hasClass(this.element, css_infinite)) {
            this.bit.style.width = 'inherit';
            if (delayed) {
                setTimeout(function () { return _this.off(); }, 200);
            }
            else {
                this.off();
            }
        }
        // let it fade out and hide afterwards
        else {
            this.bit.style.opacity = '0';
            if (delayed) {
                setTimeout(function () { return _this.off(); }, 200);
            }
            else {
                this.off();
            }
        }
        return this;
    };
    ProgressBar.prototype.hide = function () {
        hide(this.element);
        this.showDelayedScheduler.cancel();
    };
    ProgressBar.prototype.style = function (styles) {
        this.progressBarBackground = styles.progressBarBackground;
        this.applyStyles();
    };
    ProgressBar.prototype.applyStyles = function () {
        if (this.bit) {
            var background = this.progressBarBackground ? this.progressBarBackground.toString() : null;
            this.bit.style.backgroundColor = background;
        }
    };
    return ProgressBar;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1T = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var QuickOpenController = /** @class */ (function (_super) {
    __extends$1T(QuickOpenController, _super);
    function QuickOpenController() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    QuickOpenController.prototype.onContextMenu = function (tree, element, event) {
        if (isMacintosh) {
            return this.onLeftClick(tree, element, event); // https://github.com/Microsoft/vscode/issues/1011
        }
        return _super.prototype.onContextMenu.call(this, tree, element, event);
    };
    return QuickOpenController;
}(DefaultController));
var defaultStyles$2 = {
    background: Color.fromHex('#1E1E1E'),
    foreground: Color.fromHex('#CCCCCC'),
    pickerGroupForeground: Color.fromHex('#0097FB'),
    pickerGroupBorder: Color.fromHex('#3F3F46'),
    widgetShadow: Color.fromHex('#000000'),
    progressBarBackground: Color.fromHex('#0E70C0')
};
var DEFAULT_INPUT_ARIA_LABEL = localize('quickOpenAriaLabel', "Quick picker. Type to narrow down results.");
var QuickOpenWidget = /** @class */ (function (_super) {
    __extends$1T(QuickOpenWidget, _super);
    function QuickOpenWidget(container, callbacks, options) {
        var _this = _super.call(this) || this;
        _this.isDisposed = false;
        _this.container = container;
        _this.callbacks = callbacks;
        _this.options = options;
        _this.styles = options || Object.create(null);
        mixin(_this.styles, defaultStyles$2, false);
        _this.model = null;
        return _this;
    }
    QuickOpenWidget.prototype.getModel = function () {
        return this.model;
    };
    QuickOpenWidget.prototype.create = function () {
        var _this = this;
        // Container
        this.element = document.createElement('div');
        addClass(this.element, 'monaco-quick-open-widget');
        this.container.appendChild(this.element);
        this._register(addDisposableListener(this.element, EventType.CONTEXT_MENU, function (e) { return EventHelper.stop(e, true); })); // Do this to fix an issue on Mac where the menu goes into the way
        this._register(addDisposableListener(this.element, EventType.FOCUS, function (e) { return _this.gainingFocus(); }, true));
        this._register(addDisposableListener(this.element, EventType.BLUR, function (e) { return _this.loosingFocus(e); }, true));
        this._register(addDisposableListener(this.element, EventType.KEY_DOWN, function (e) {
            var keyboardEvent = new StandardKeyboardEvent(e);
            if (keyboardEvent.keyCode === 9 /* Escape */) {
                EventHelper.stop(e, true);
                _this.hide(2 /* CANCELED */);
            }
            else if (keyboardEvent.keyCode === 2 /* Tab */ && !keyboardEvent.altKey && !keyboardEvent.ctrlKey && !keyboardEvent.metaKey) {
                var stops = e.currentTarget.querySelectorAll('input, .monaco-tree, .monaco-tree-row.focused .action-label.icon');
                if (keyboardEvent.shiftKey && keyboardEvent.target === stops[0]) {
                    EventHelper.stop(e, true);
                    stops[stops.length - 1].focus();
                }
                else if (!keyboardEvent.shiftKey && keyboardEvent.target === stops[stops.length - 1]) {
                    EventHelper.stop(e, true);
                    stops[0].focus();
                }
            }
        }));
        // Progress Bar
        this.progressBar = this._register(new ProgressBar(this.element, { progressBarBackground: this.styles.progressBarBackground }));
        this.progressBar.hide();
        // Input Field
        this.inputContainer = document.createElement('div');
        addClass(this.inputContainer, 'quick-open-input');
        this.element.appendChild(this.inputContainer);
        this.inputBox = this._register(new InputBox(this.inputContainer, null, {
            placeholder: this.options.inputPlaceHolder || '',
            ariaLabel: DEFAULT_INPUT_ARIA_LABEL,
            inputBackground: this.styles.inputBackground,
            inputForeground: this.styles.inputForeground,
            inputBorder: this.styles.inputBorder,
            inputValidationInfoBackground: this.styles.inputValidationInfoBackground,
            inputValidationInfoForeground: this.styles.inputValidationInfoForeground,
            inputValidationInfoBorder: this.styles.inputValidationInfoBorder,
            inputValidationWarningBackground: this.styles.inputValidationWarningBackground,
            inputValidationWarningForeground: this.styles.inputValidationWarningForeground,
            inputValidationWarningBorder: this.styles.inputValidationWarningBorder,
            inputValidationErrorBackground: this.styles.inputValidationErrorBackground,
            inputValidationErrorForeground: this.styles.inputValidationErrorForeground,
            inputValidationErrorBorder: this.styles.inputValidationErrorBorder
        }));
        this.inputElement = this.inputBox.inputElement;
        this.inputElement.setAttribute('role', 'combobox');
        this.inputElement.setAttribute('aria-haspopup', 'false');
        this.inputElement.setAttribute('aria-autocomplete', 'list');
        this._register(addDisposableListener(this.inputBox.inputElement, EventType.INPUT, function (e) { return _this.onType(); }));
        this._register(addDisposableListener(this.inputBox.inputElement, EventType.KEY_DOWN, function (e) {
            var keyboardEvent = new StandardKeyboardEvent(e);
            var shouldOpenInBackground = _this.shouldOpenInBackground(keyboardEvent);
            // Do not handle Tab: It is used to navigate between elements without mouse
            if (keyboardEvent.keyCode === 2 /* Tab */) {
                return;
            }
            // Pass tree navigation keys to the tree but leave focus in input field
            else if (keyboardEvent.keyCode === 18 /* DownArrow */ || keyboardEvent.keyCode === 16 /* UpArrow */ || keyboardEvent.keyCode === 12 /* PageDown */ || keyboardEvent.keyCode === 11 /* PageUp */) {
                EventHelper.stop(e, true);
                _this.navigateInTree(keyboardEvent.keyCode, keyboardEvent.shiftKey);
                // Position cursor at the end of input to allow right arrow (open in background)
                // to function immediately unless the user has made a selection
                if (_this.inputBox.inputElement.selectionStart === _this.inputBox.inputElement.selectionEnd) {
                    _this.inputBox.inputElement.selectionStart = _this.inputBox.value.length;
                }
            }
            // Select element on Enter or on Arrow-Right if we are at the end of the input
            else if (keyboardEvent.keyCode === 3 /* Enter */ || shouldOpenInBackground) {
                EventHelper.stop(e, true);
                var focus_1 = _this.tree.getFocus();
                if (focus_1) {
                    _this.elementSelected(focus_1, e, shouldOpenInBackground ? 2 /* OPEN_IN_BACKGROUND */ : 1 /* OPEN */);
                }
            }
        }));
        // Result count for screen readers
        this.resultCount = document.createElement('div');
        addClass(this.resultCount, 'quick-open-result-count');
        this.resultCount.setAttribute('aria-live', 'polite');
        this.resultCount.setAttribute('aria-atomic', 'true');
        this.element.appendChild(this.resultCount);
        // Tree
        this.treeContainer = document.createElement('div');
        addClass(this.treeContainer, 'quick-open-tree');
        this.element.appendChild(this.treeContainer);
        var createTree = this.options.treeCreator || (function (container, config, opts) { return new Tree(container, config, opts); });
        this.tree = this._register(createTree(this.treeContainer, {
            dataSource: new DataSource(this),
            controller: new QuickOpenController({ clickBehavior: 1 /* ON_MOUSE_UP */, keyboardSupport: this.options.keyboardSupport }),
            renderer: (this.renderer = new Renderer$3(this, this.styles)),
            filter: new Filter(this),
            accessibilityProvider: new AccessibilityProvider(this)
        }, {
            twistiePixels: 11,
            indentPixels: 0,
            alwaysFocused: true,
            verticalScrollMode: 3 /* Visible */,
            horizontalScrollMode: 2 /* Hidden */,
            ariaLabel: localize('treeAriaLabel', "Quick Picker"),
            keyboardSupport: this.options.keyboardSupport,
            preventRootFocus: false
        }));
        this.treeElement = this.tree.getHTMLElement();
        // Handle Focus and Selection event
        this._register(this.tree.onDidChangeFocus(function (event) {
            _this.elementFocused(event.focus, event);
        }));
        this._register(this.tree.onDidChangeSelection(function (event) {
            if (event.selection && event.selection.length > 0) {
                var mouseEvent = event.payload && event.payload.originalEvent instanceof StandardMouseEvent ? event.payload.originalEvent : void 0;
                var shouldOpenInBackground = mouseEvent ? _this.shouldOpenInBackground(mouseEvent) : false;
                _this.elementSelected(event.selection[0], event, shouldOpenInBackground ? 2 /* OPEN_IN_BACKGROUND */ : 1 /* OPEN */);
            }
        }));
        this._register(addDisposableListener(this.treeContainer, EventType.KEY_DOWN, function (e) {
            var keyboardEvent = new StandardKeyboardEvent(e);
            // Only handle when in quick navigation mode
            if (!_this.quickNavigateConfiguration) {
                return;
            }
            // Support keyboard navigation in quick navigation mode
            if (keyboardEvent.keyCode === 18 /* DownArrow */ || keyboardEvent.keyCode === 16 /* UpArrow */ || keyboardEvent.keyCode === 12 /* PageDown */ || keyboardEvent.keyCode === 11 /* PageUp */) {
                EventHelper.stop(e, true);
                _this.navigateInTree(keyboardEvent.keyCode);
            }
        }));
        this._register(addDisposableListener(this.treeContainer, EventType.KEY_UP, function (e) {
            var keyboardEvent = new StandardKeyboardEvent(e);
            var keyCode = keyboardEvent.keyCode;
            // Only handle when in quick navigation mode
            if (!_this.quickNavigateConfiguration) {
                return;
            }
            // Select element when keys are pressed that signal it
            var quickNavKeys = _this.quickNavigateConfiguration.keybindings;
            var wasTriggerKeyPressed = keyCode === 3 /* Enter */ || quickNavKeys.some(function (k) {
                var _a = k.getParts(), firstPart = _a[0], chordPart = _a[1];
                if (chordPart) {
                    return false;
                }
                if (firstPart.shiftKey && keyCode === 4 /* Shift */) {
                    if (keyboardEvent.ctrlKey || keyboardEvent.altKey || keyboardEvent.metaKey) {
                        return false; // this is an optimistic check for the shift key being used to navigate back in quick open
                    }
                    return true;
                }
                if (firstPart.altKey && keyCode === 6 /* Alt */) {
                    return true;
                }
                if (firstPart.ctrlKey && keyCode === 5 /* Ctrl */) {
                    return true;
                }
                if (firstPart.metaKey && keyCode === 57 /* Meta */) {
                    return true;
                }
                return false;
            });
            if (wasTriggerKeyPressed) {
                var focus_2 = _this.tree.getFocus();
                if (focus_2) {
                    _this.elementSelected(focus_2, e);
                }
            }
        }));
        // Support layout
        if (this.layoutDimensions) {
            this.layout(this.layoutDimensions);
        }
        this.applyStyles();
        // Allows focus to switch to next/previous entry after tab into an actionbar item
        this._register(addDisposableListener(this.treeContainer, EventType.KEY_DOWN, function (e) {
            var keyboardEvent = new StandardKeyboardEvent(e);
            // Only handle when not in quick navigation mode
            if (_this.quickNavigateConfiguration) {
                return;
            }
            if (keyboardEvent.keyCode === 18 /* DownArrow */ || keyboardEvent.keyCode === 16 /* UpArrow */ || keyboardEvent.keyCode === 12 /* PageDown */ || keyboardEvent.keyCode === 11 /* PageUp */) {
                EventHelper.stop(e, true);
                _this.navigateInTree(keyboardEvent.keyCode, keyboardEvent.shiftKey);
                _this.treeElement.focus();
            }
        }));
        return this.element;
    };
    QuickOpenWidget.prototype.style = function (styles) {
        this.styles = styles;
        this.applyStyles();
    };
    QuickOpenWidget.prototype.applyStyles = function () {
        if (this.element) {
            var foreground = this.styles.foreground ? this.styles.foreground.toString() : null;
            var background = this.styles.background ? this.styles.background.toString() : null;
            var borderColor = this.styles.borderColor ? this.styles.borderColor.toString() : null;
            var widgetShadow = this.styles.widgetShadow ? this.styles.widgetShadow.toString() : null;
            this.element.style.color = foreground;
            this.element.style.backgroundColor = background;
            this.element.style.borderColor = borderColor;
            this.element.style.borderWidth = borderColor ? '1px' : null;
            this.element.style.borderStyle = borderColor ? 'solid' : null;
            this.element.style.boxShadow = widgetShadow ? "0 5px 8px " + widgetShadow : null;
        }
        if (this.progressBar) {
            this.progressBar.style({
                progressBarBackground: this.styles.progressBarBackground
            });
        }
        if (this.inputBox) {
            this.inputBox.style({
                inputBackground: this.styles.inputBackground,
                inputForeground: this.styles.inputForeground,
                inputBorder: this.styles.inputBorder,
                inputValidationInfoBackground: this.styles.inputValidationInfoBackground,
                inputValidationInfoForeground: this.styles.inputValidationInfoForeground,
                inputValidationInfoBorder: this.styles.inputValidationInfoBorder,
                inputValidationWarningBackground: this.styles.inputValidationWarningBackground,
                inputValidationWarningForeground: this.styles.inputValidationWarningForeground,
                inputValidationWarningBorder: this.styles.inputValidationWarningBorder,
                inputValidationErrorBackground: this.styles.inputValidationErrorBackground,
                inputValidationErrorForeground: this.styles.inputValidationErrorForeground,
                inputValidationErrorBorder: this.styles.inputValidationErrorBorder
            });
        }
        if (this.tree && !this.options.treeCreator) {
            this.tree.style(this.styles);
        }
        if (this.renderer) {
            this.renderer.updateStyles(this.styles);
        }
    };
    QuickOpenWidget.prototype.shouldOpenInBackground = function (e) {
        // Keyboard
        if (e instanceof StandardKeyboardEvent) {
            if (e.keyCode !== 17 /* RightArrow */) {
                return false; // only for right arrow
            }
            if (e.metaKey || e.ctrlKey || e.shiftKey || e.altKey) {
                return false; // no modifiers allowed
            }
            // validate the cursor is at the end of the input and there is no selection,
            // and if not prevent opening in the background such as the selection can be changed
            var element = this.inputBox.inputElement;
            return element.selectionEnd === this.inputBox.value.length && element.selectionStart === element.selectionEnd;
        }
        // Mouse
        return e.middleButton;
    };
    QuickOpenWidget.prototype.onType = function () {
        var value = this.inputBox.value;
        // Adjust help text as needed if present
        if (this.helpText) {
            if (value) {
                hide(this.helpText);
            }
            else {
                show(this.helpText);
            }
        }
        // Send to callbacks
        this.callbacks.onType(value);
    };
    QuickOpenWidget.prototype.navigateInTree = function (keyCode, isShift) {
        var model = this.tree.getInput();
        var entries = model ? model.entries : [];
        var oldFocus = this.tree.getFocus();
        // Normal Navigation
        switch (keyCode) {
            case 18 /* DownArrow */:
                this.tree.focusNext();
                break;
            case 16 /* UpArrow */:
                this.tree.focusPrevious();
                break;
            case 12 /* PageDown */:
                this.tree.focusNextPage();
                break;
            case 11 /* PageUp */:
                this.tree.focusPreviousPage();
                break;
            case 2 /* Tab */:
                if (isShift) {
                    this.tree.focusPrevious();
                }
                else {
                    this.tree.focusNext();
                }
                break;
        }
        var newFocus = this.tree.getFocus();
        // Support cycle-through navigation if focus did not change
        if (entries.length > 1 && oldFocus === newFocus) {
            // Up from no entry or first entry goes down to last
            if (keyCode === 16 /* UpArrow */ || (keyCode === 2 /* Tab */ && isShift)) {
                this.tree.focusLast();
            }
            // Down from last entry goes to up to first
            else if (keyCode === 18 /* DownArrow */ || keyCode === 2 /* Tab */ && !isShift) {
                this.tree.focusFirst();
            }
        }
        // Reveal
        newFocus = this.tree.getFocus();
        if (newFocus) {
            this.tree.reveal(newFocus);
        }
    };
    QuickOpenWidget.prototype.elementFocused = function (value, event) {
        if (!value || !this.isVisible()) {
            return;
        }
        // ARIA
        this.inputElement.setAttribute('aria-activedescendant', this.treeElement.getAttribute('aria-activedescendant'));
        var context = { event: event, keymods: this.extractKeyMods(event), quickNavigateConfiguration: this.quickNavigateConfiguration };
        this.model.runner.run(value, 0 /* PREVIEW */, context);
    };
    QuickOpenWidget.prototype.elementSelected = function (value, event, preferredMode) {
        var hide$$1 = true;
        // Trigger open of element on selection
        if (this.isVisible()) {
            var mode = preferredMode || 1 /* OPEN */;
            var context_1 = { event: event, keymods: this.extractKeyMods(event), quickNavigateConfiguration: this.quickNavigateConfiguration };
            hide$$1 = this.model.runner.run(value, mode, context_1);
        }
        // Hide if command was run successfully
        if (hide$$1) {
            this.hide(0 /* ELEMENT_SELECTED */);
        }
    };
    QuickOpenWidget.prototype.extractKeyMods = function (event) {
        return {
            ctrlCmd: event && (event.ctrlKey || event.metaKey || (event.payload && event.payload.originalEvent && (event.payload.originalEvent.ctrlKey || event.payload.originalEvent.metaKey))),
            alt: event && (event.altKey || (event.payload && event.payload.originalEvent && event.payload.originalEvent.altKey))
        };
    };
    QuickOpenWidget.prototype.show = function (param, options) {
        this.visible = true;
        this.isLoosingFocus = false;
        this.quickNavigateConfiguration = options ? options.quickNavigateConfiguration : void 0;
        // Adjust UI for quick navigate mode
        if (this.quickNavigateConfiguration) {
            hide(this.inputContainer);
            show(this.element);
            this.tree.domFocus();
        }
        // Otherwise use normal UI
        else {
            show(this.inputContainer);
            show(this.element);
            this.inputBox.focus();
        }
        // Adjust Help text for IE
        if (this.helpText) {
            if (this.quickNavigateConfiguration || isString(param)) {
                hide(this.helpText);
            }
            else {
                show(this.helpText);
            }
        }
        // Show based on param
        if (isString(param)) {
            this.doShowWithPrefix(param);
        }
        else {
            if (options.value) {
                this.restoreLastInput(options.value);
            }
            this.doShowWithInput(param, options && options.autoFocus ? options.autoFocus : {});
        }
        // Respect selectAll option
        if (options && options.inputSelection && !this.quickNavigateConfiguration) {
            this.inputBox.select(options.inputSelection);
        }
        if (this.callbacks.onShow) {
            this.callbacks.onShow();
        }
    };
    QuickOpenWidget.prototype.restoreLastInput = function (lastInput) {
        this.inputBox.value = lastInput;
        this.inputBox.select();
        this.callbacks.onType(lastInput);
    };
    QuickOpenWidget.prototype.doShowWithPrefix = function (prefix) {
        this.inputBox.value = prefix;
        this.callbacks.onType(prefix);
    };
    QuickOpenWidget.prototype.doShowWithInput = function (input, autoFocus) {
        this.setInput(input, autoFocus);
    };
    QuickOpenWidget.prototype.setInputAndLayout = function (input, autoFocus) {
        var _this = this;
        this.treeContainer.style.height = this.getHeight(input) + "px";
        this.tree.setInput(null).then(function () {
            _this.model = input;
            // ARIA
            _this.inputElement.setAttribute('aria-haspopup', String(input && input.entries && input.entries.length > 0));
            return _this.tree.setInput(input);
        }).then(function () {
            // Indicate entries to tree
            _this.tree.layout();
            var entries = input ? input.entries.filter(function (e) { return _this.isElementVisible(input, e); }) : [];
            _this.updateResultCount(entries.length);
            // Handle auto focus
            if (entries.length) {
                _this.autoFocus(input, entries, autoFocus);
            }
        });
    };
    QuickOpenWidget.prototype.isElementVisible = function (input, e) {
        if (!input.filter) {
            return true;
        }
        return input.filter.isVisible(e);
    };
    QuickOpenWidget.prototype.autoFocus = function (input, entries, autoFocus) {
        if (autoFocus === void 0) { autoFocus = {}; }
        // First check for auto focus of prefix matches
        if (autoFocus.autoFocusPrefixMatch) {
            var caseSensitiveMatch = void 0;
            var caseInsensitiveMatch = void 0;
            var prefix = autoFocus.autoFocusPrefixMatch;
            var lowerCasePrefix = prefix.toLowerCase();
            for (var i = 0; i < entries.length; i++) {
                var entry = entries[i];
                var label = input.dataSource.getLabel(entry);
                if (!caseSensitiveMatch && label.indexOf(prefix) === 0) {
                    caseSensitiveMatch = entry;
                }
                else if (!caseInsensitiveMatch && label.toLowerCase().indexOf(lowerCasePrefix) === 0) {
                    caseInsensitiveMatch = entry;
                }
                if (caseSensitiveMatch && caseInsensitiveMatch) {
                    break;
                }
            }
            var entryToFocus = caseSensitiveMatch || caseInsensitiveMatch;
            if (entryToFocus) {
                this.tree.setFocus(entryToFocus);
                this.tree.reveal(entryToFocus, 0.5);
                return;
            }
        }
        // Second check for auto focus of first entry
        if (autoFocus.autoFocusFirstEntry) {
            this.tree.focusFirst();
            this.tree.reveal(this.tree.getFocus());
        }
        // Third check for specific index option
        else if (typeof autoFocus.autoFocusIndex === 'number') {
            if (entries.length > autoFocus.autoFocusIndex) {
                this.tree.focusNth(autoFocus.autoFocusIndex);
                this.tree.reveal(this.tree.getFocus());
            }
        }
        // Check for auto focus of second entry
        else if (autoFocus.autoFocusSecondEntry) {
            if (entries.length > 1) {
                this.tree.focusNth(1);
            }
        }
        // Finally check for auto focus of last entry
        else if (autoFocus.autoFocusLastEntry) {
            if (entries.length > 1) {
                this.tree.focusLast();
            }
        }
    };
    QuickOpenWidget.prototype.getHeight = function (input) {
        var _this = this;
        var renderer = input.renderer;
        if (!input) {
            var itemHeight = renderer.getHeight(null);
            return this.options.minItemsToShow ? this.options.minItemsToShow * itemHeight : 0;
        }
        var height = 0;
        var preferredItemsHeight;
        if (this.layoutDimensions && this.layoutDimensions.height) {
            preferredItemsHeight = (this.layoutDimensions.height - 50 /* subtract height of input field (30px) and some spacing (drop shadow) to fit */) * 0.40 /* max 40% of screen */;
        }
        if (!preferredItemsHeight || preferredItemsHeight > QuickOpenWidget.MAX_ITEMS_HEIGHT) {
            preferredItemsHeight = QuickOpenWidget.MAX_ITEMS_HEIGHT;
        }
        var entries = input.entries.filter(function (e) { return _this.isElementVisible(input, e); });
        var maxEntries = this.options.maxItemsToShow || entries.length;
        for (var i = 0; i < maxEntries && i < entries.length; i++) {
            var entryHeight = renderer.getHeight(entries[i]);
            if (height + entryHeight <= preferredItemsHeight) {
                height += entryHeight;
            }
            else {
                break;
            }
        }
        return height;
    };
    QuickOpenWidget.prototype.updateResultCount = function (count) {
        this.resultCount.textContent = localize({ key: 'quickInput.visibleCount', comment: ['This tells the user how many items are shown in a list of items to select from. The items can be anything. Currently not visible, but read by screen readers.'] }, "{0} Results", count);
    };
    QuickOpenWidget.prototype.hide = function (reason) {
        if (!this.isVisible()) {
            return;
        }
        this.visible = false;
        hide(this.element);
        this.element.blur();
        // Clear input field and clear tree
        this.inputBox.value = '';
        this.tree.setInput(null);
        // ARIA
        this.inputElement.setAttribute('aria-haspopup', 'false');
        // Reset Tree Height
        this.treeContainer.style.height = (this.options.minItemsToShow ? this.options.minItemsToShow * 22 : 0) + "px";
        // Clear any running Progress
        this.progressBar.stop().hide();
        // Clear Focus
        if (this.tree.isDOMFocused()) {
            this.tree.domBlur();
        }
        else if (this.inputBox.hasFocus()) {
            this.inputBox.blur();
        }
        // Callbacks
        if (reason === 0 /* ELEMENT_SELECTED */) {
            this.callbacks.onOk();
        }
        else {
            this.callbacks.onCancel();
        }
        if (this.callbacks.onHide) {
            this.callbacks.onHide(reason);
        }
    };
    QuickOpenWidget.prototype.setInput = function (input, autoFocus, ariaLabel) {
        if (!this.isVisible()) {
            return;
        }
        // If the input changes, indicate this to the tree
        if (!!this.getInput()) {
            this.onInputChanging();
        }
        // Adapt tree height to entries and apply input
        this.setInputAndLayout(input, autoFocus);
        // Apply ARIA
        if (this.inputBox) {
            this.inputBox.setAriaLabel(ariaLabel || DEFAULT_INPUT_ARIA_LABEL);
        }
    };
    QuickOpenWidget.prototype.onInputChanging = function () {
        var _this = this;
        if (this.inputChangingTimeoutHandle) {
            clearTimeout(this.inputChangingTimeoutHandle);
            this.inputChangingTimeoutHandle = null;
        }
        // when the input is changing in quick open, we indicate this as CSS class to the widget
        // for a certain timeout. this helps reducing some hectic UI updates when input changes quickly
        addClass(this.element, 'content-changing');
        this.inputChangingTimeoutHandle = setTimeout(function () {
            removeClass(_this.element, 'content-changing');
        }, 500);
    };
    QuickOpenWidget.prototype.getInput = function () {
        return this.tree.getInput();
    };
    QuickOpenWidget.prototype.isVisible = function () {
        return this.visible;
    };
    QuickOpenWidget.prototype.layout = function (dimension) {
        this.layoutDimensions = dimension;
        // Apply to quick open width (height is dynamic by number of items to show)
        var quickOpenWidth = Math.min(this.layoutDimensions.width * 0.62 /* golden cut */, QuickOpenWidget.MAX_WIDTH);
        if (this.element) {
            // quick open
            this.element.style.width = quickOpenWidth + "px";
            this.element.style.marginLeft = "-" + quickOpenWidth / 2 + "px";
            // input field
            this.inputContainer.style.width = quickOpenWidth - 12 + "px";
        }
    };
    QuickOpenWidget.prototype.gainingFocus = function () {
        this.isLoosingFocus = false;
    };
    QuickOpenWidget.prototype.loosingFocus = function (e) {
        var _this = this;
        if (!this.isVisible()) {
            return;
        }
        var relatedTarget = e.relatedTarget;
        if (!this.quickNavigateConfiguration && isAncestor(relatedTarget, this.element)) {
            return; // user clicked somewhere into quick open widget, do not close thereby
        }
        this.isLoosingFocus = true;
        setTimeout(function () {
            if (!_this.isLoosingFocus || _this.isDisposed) {
                return;
            }
            var veto = _this.callbacks.onFocusLost && _this.callbacks.onFocusLost();
            if (!veto) {
                _this.hide(1 /* FOCUS_LOST */);
            }
        }, 0);
    };
    QuickOpenWidget.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this.isDisposed = true;
    };
    QuickOpenWidget.MAX_WIDTH = 600; // Max total width of quick open widget
    QuickOpenWidget.MAX_ITEMS_HEIGHT = 20 * 22; // Max height of item list below input field
    return QuickOpenWidget;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var QuickOpenEditorWidget = /** @class */ (function () {
    function QuickOpenEditorWidget(codeEditor, onOk, onCancel, onType, configuration, themeService) {
        this.codeEditor = codeEditor;
        this.themeService = themeService;
        this.create(onOk, onCancel, onType, configuration);
    }
    QuickOpenEditorWidget.prototype.create = function (onOk, onCancel, onType, configuration) {
        this.domNode = document.createElement('div');
        this.quickOpenWidget = new QuickOpenWidget(this.domNode, {
            onOk: onOk,
            onCancel: onCancel,
            onType: onType
        }, {
            inputPlaceHolder: null,
            inputAriaLabel: configuration.inputAriaLabel,
            keyboardSupport: true
        });
        this.styler = attachQuickOpenStyler(this.quickOpenWidget, this.themeService, {
            pickerGroupForeground: foreground
        });
        this.quickOpenWidget.create();
        this.codeEditor.addOverlayWidget(this);
    };
    QuickOpenEditorWidget.prototype.setInput = function (model, focus) {
        this.quickOpenWidget.setInput(model, focus);
    };
    QuickOpenEditorWidget.prototype.getId = function () {
        return QuickOpenEditorWidget.ID;
    };
    QuickOpenEditorWidget.prototype.getDomNode = function () {
        return this.domNode;
    };
    QuickOpenEditorWidget.prototype.destroy = function () {
        this.codeEditor.removeOverlayWidget(this);
        this.quickOpenWidget.dispose();
        this.styler.dispose();
    };
    QuickOpenEditorWidget.prototype.show = function (value) {
        this.visible = true;
        var editorLayout = this.codeEditor.getLayoutInfo();
        if (editorLayout) {
            this.quickOpenWidget.layout(new Dimension(editorLayout.width, editorLayout.height));
        }
        this.quickOpenWidget.show(value);
        this.codeEditor.layoutOverlayWidget(this);
    };
    QuickOpenEditorWidget.prototype.getPosition = function () {
        if (this.visible) {
            return {
                preference: 2 /* TOP_CENTER */
            };
        }
        return null;
    };
    QuickOpenEditorWidget.ID = 'editor.contrib.quickOpenEditorWidget';
    return QuickOpenEditorWidget;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1U = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$q = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$n = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var QuickOpenController$1 = /** @class */ (function () {
    function QuickOpenController(editor, themeService) {
        this.themeService = themeService;
        this.editor = editor;
    }
    QuickOpenController.get = function (editor) {
        return editor.getContribution(QuickOpenController.ID);
    };
    QuickOpenController.prototype.getId = function () {
        return QuickOpenController.ID;
    };
    QuickOpenController.prototype.dispose = function () {
        // Dispose widget
        if (this.widget) {
            this.widget.destroy();
            this.widget = null;
        }
    };
    QuickOpenController.prototype.run = function (opts) {
        var _this = this;
        if (this.widget) {
            this.widget.destroy();
            this.widget = null;
        }
        // Create goto line widget
        var onClose = function (canceled) {
            // Clear Highlight Decorations if present
            _this.clearDecorations();
            // Restore selection if canceled
            if (canceled && _this.lastKnownEditorSelection) {
                _this.editor.setSelection(_this.lastKnownEditorSelection);
                _this.editor.revealRangeInCenterIfOutsideViewport(_this.lastKnownEditorSelection, 0 /* Smooth */);
            }
            _this.lastKnownEditorSelection = null;
            // Return focus to the editor if
            // - focus is back on the <body> element because no other focusable element was clicked
            // - a command was picked from the picker which indicates the editor should get focused
            if (document.activeElement === document.body || !canceled) {
                _this.editor.focus();
            }
        };
        this.widget = new QuickOpenEditorWidget(this.editor, function () { return onClose(false); }, function () { return onClose(true); }, function (value) {
            _this.widget.setInput(opts.getModel(value), opts.getAutoFocus(value));
        }, {
            inputAriaLabel: opts.inputAriaLabel
        }, this.themeService);
        // Remember selection to be able to restore on cancel
        if (!this.lastKnownEditorSelection) {
            this.lastKnownEditorSelection = this.editor.getSelection();
        }
        // Show
        this.widget.show('');
    };
    QuickOpenController.prototype.decorateLine = function (range, editor) {
        var oldDecorations = [];
        if (this.rangeHighlightDecorationId) {
            oldDecorations.push(this.rangeHighlightDecorationId);
            this.rangeHighlightDecorationId = null;
        }
        var newDecorations = [
            {
                range: range,
                options: QuickOpenController._RANGE_HIGHLIGHT_DECORATION
            }
        ];
        var decorations = editor.deltaDecorations(oldDecorations, newDecorations);
        this.rangeHighlightDecorationId = decorations[0];
    };
    QuickOpenController.prototype.clearDecorations = function () {
        if (this.rangeHighlightDecorationId) {
            this.editor.deltaDecorations([this.rangeHighlightDecorationId], []);
            this.rangeHighlightDecorationId = null;
        }
    };
    QuickOpenController.ID = 'editor.controller.quickOpenController';
    QuickOpenController._RANGE_HIGHLIGHT_DECORATION = ModelDecorationOptions.register({
        className: 'rangeHighlight',
        isWholeLine: true
    });
    QuickOpenController = __decorate$q([
        __param$n(1, IThemeService)
    ], QuickOpenController);
    return QuickOpenController;
}());
/**
 * Base class for providing quick open in the editor.
 */
var BaseEditorQuickOpenAction = /** @class */ (function (_super) {
    __extends$1U(BaseEditorQuickOpenAction, _super);
    function BaseEditorQuickOpenAction(inputAriaLabel, opts) {
        var _this = _super.call(this, opts) || this;
        _this._inputAriaLabel = inputAriaLabel;
        return _this;
    }
    BaseEditorQuickOpenAction.prototype.getController = function (editor) {
        return QuickOpenController$1.get(editor);
    };
    BaseEditorQuickOpenAction.prototype._show = function (controller, opts) {
        controller.run({
            inputAriaLabel: this._inputAriaLabel,
            getModel: function (value) { return opts.getModel(value); },
            getAutoFocus: function (searchValue) { return opts.getAutoFocus(searchValue); }
        });
    };
    return BaseEditorQuickOpenAction;
}(EditorAction));
registerEditorContribution(QuickOpenController$1);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1V = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var SCOPE_PREFIX = ':';
var SymbolEntry = /** @class */ (function (_super) {
    __extends$1V(SymbolEntry, _super);
    function SymbolEntry(name, type, description, range, highlights, editor, decorator) {
        var _this = _super.call(this) || this;
        _this.name = name;
        _this.type = type;
        _this.description = description;
        _this.range = range;
        _this.setHighlights(highlights);
        _this.editor = editor;
        _this.decorator = decorator;
        return _this;
    }
    SymbolEntry.prototype.getLabel = function () {
        return this.name;
    };
    SymbolEntry.prototype.getAriaLabel = function () {
        return localize('entryAriaLabel', "{0}, symbols", this.name);
    };
    SymbolEntry.prototype.getIcon = function () {
        return this.type;
    };
    SymbolEntry.prototype.getDescription = function () {
        return this.description;
    };
    SymbolEntry.prototype.getType = function () {
        return this.type;
    };
    SymbolEntry.prototype.getRange = function () {
        return this.range;
    };
    SymbolEntry.prototype.run = function (mode, context) {
        if (mode === 1 /* OPEN */) {
            return this.runOpen(context);
        }
        return this.runPreview();
    };
    SymbolEntry.prototype.runOpen = function (context) {
        // Apply selection and focus
        var range = this.toSelection();
        this.editor.setSelection(range);
        this.editor.revealRangeInCenter(range, 0 /* Smooth */);
        this.editor.focus();
        return true;
    };
    SymbolEntry.prototype.runPreview = function () {
        // Select Outline Position
        var range = this.toSelection();
        this.editor.revealRangeInCenter(range, 0 /* Smooth */);
        // Decorate if possible
        this.decorator.decorateLine(this.range, this.editor);
        return false;
    };
    SymbolEntry.prototype.toSelection = function () {
        return new Range(this.range.startLineNumber, this.range.startColumn || 1, this.range.startLineNumber, this.range.startColumn || 1);
    };
    return SymbolEntry;
}(QuickOpenEntryGroup));
var QuickOutlineAction = /** @class */ (function (_super) {
    __extends$1V(QuickOutlineAction, _super);
    function QuickOutlineAction() {
        return _super.call(this, localize('quickOutlineActionInput', "Type the name of an identifier you wish to navigate to"), {
            id: 'editor.action.quickOutline',
            label: localize('QuickOutlineAction.label', "Go to Symbol..."),
            alias: 'Go to Symbol...',
            precondition: EditorContextKeys.hasDocumentSymbolProvider,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 45 /* KEY_O */,
                weight: 100 /* EditorContrib */
            },
            menuOpts: {
                group: 'navigation',
                order: 3
            }
        }) || this;
    }
    QuickOutlineAction.prototype.run = function (accessor, editor) {
        var _this = this;
        var model = editor.getModel();
        if (!DocumentSymbolProviderRegistry.has(model)) {
            return null;
        }
        // Resolve outline
        return getDocumentSymbols(model, true, CancellationToken.None).then(function (result) {
            if (result.length === 0) {
                return;
            }
            _this._run(editor, result);
        });
    };
    QuickOutlineAction.prototype._run = function (editor, result) {
        var _this = this;
        this._show(this.getController(editor), {
            getModel: function (value) {
                return new QuickOpenModel(_this.toQuickOpenEntries(editor, result, value));
            },
            getAutoFocus: function (searchValue) {
                // Remove any type pattern (:) from search value as needed
                if (searchValue.indexOf(SCOPE_PREFIX) === 0) {
                    searchValue = searchValue.substr(SCOPE_PREFIX.length);
                }
                return {
                    autoFocusPrefixMatch: searchValue,
                    autoFocusFirstEntry: !!searchValue
                };
            }
        });
    };
    QuickOutlineAction.prototype.symbolEntry = function (name, type, description, range, highlights, editor, decorator) {
        return new SymbolEntry(name, type, description, Range.lift(range), highlights, editor, decorator);
    };
    QuickOutlineAction.prototype.toQuickOpenEntries = function (editor, flattened, searchValue) {
        var controller = this.getController(editor);
        var results = [];
        // Convert to Entries
        var normalizedSearchValue = searchValue;
        if (searchValue.indexOf(SCOPE_PREFIX) === 0) {
            normalizedSearchValue = normalizedSearchValue.substr(SCOPE_PREFIX.length);
        }
        for (var i = 0; i < flattened.length; i++) {
            var element = flattened[i];
            var label = trim(element.name);
            // Check for meatch
            var highlights = matchesFuzzy(normalizedSearchValue, label);
            if (highlights) {
                // Show parent scope as description
                var description = null;
                if (element.containerName) {
                    description = element.containerName;
                }
                // Add
                results.push(this.symbolEntry(label, symbolKindToCssClass(element.kind), description, element.range, highlights, editor, controller));
            }
        }
        // Sort properly if actually searching
        if (searchValue) {
            if (searchValue.indexOf(SCOPE_PREFIX) === 0) {
                results = results.sort(this.sortScoped.bind(this, searchValue.toLowerCase()));
            }
            else {
                results = results.sort(this.sortNormal.bind(this, searchValue.toLowerCase()));
            }
        }
        // Mark all type groups
        if (results.length > 0 && searchValue.indexOf(SCOPE_PREFIX) === 0) {
            var currentType = null;
            var currentResult = null;
            var typeCounter = 0;
            for (var i = 0; i < results.length; i++) {
                var result = results[i];
                // Found new type
                if (currentType !== result.getType()) {
                    // Update previous result with count
                    if (currentResult) {
                        currentResult.setGroupLabel(this.typeToLabel(currentType, typeCounter));
                    }
                    currentType = result.getType();
                    currentResult = result;
                    typeCounter = 1;
                    result.setShowBorder(i > 0);
                }
                // Existing type, keep counting
                else {
                    typeCounter++;
                }
            }
            // Update previous result with count
            if (currentResult) {
                currentResult.setGroupLabel(this.typeToLabel(currentType, typeCounter));
            }
        }
        // Mark first entry as outline
        else if (results.length > 0) {
            results[0].setGroupLabel(localize('symbols', "symbols ({0})", results.length));
        }
        return results;
    };
    QuickOutlineAction.prototype.typeToLabel = function (type, count) {
        switch (type) {
            case 'module': return localize('modules', "modules ({0})", count);
            case 'class': return localize('class', "classes ({0})", count);
            case 'interface': return localize('interface', "interfaces ({0})", count);
            case 'method': return localize('method', "methods ({0})", count);
            case 'function': return localize('function', "functions ({0})", count);
            case 'property': return localize('property', "properties ({0})", count);
            case 'variable': return localize('variable', "variables ({0})", count);
            case 'var': return localize('variable2', "variables ({0})", count);
            case 'constructor': return localize('_constructor', "constructors ({0})", count);
            case 'call': return localize('call', "calls ({0})", count);
        }
        return type;
    };
    QuickOutlineAction.prototype.sortNormal = function (searchValue, elementA, elementB) {
        var elementAName = elementA.getLabel().toLowerCase();
        var elementBName = elementB.getLabel().toLowerCase();
        // Compare by name
        var r = elementAName.localeCompare(elementBName);
        if (r !== 0) {
            return r;
        }
        // If name identical sort by range instead
        var elementARange = elementA.getRange();
        var elementBRange = elementB.getRange();
        return elementARange.startLineNumber - elementBRange.startLineNumber;
    };
    QuickOutlineAction.prototype.sortScoped = function (searchValue, elementA, elementB) {
        // Remove scope char
        searchValue = searchValue.substr(SCOPE_PREFIX.length);
        // Sort by type first if scoped search
        var elementAType = elementA.getType();
        var elementBType = elementB.getType();
        var r = elementAType.localeCompare(elementBType);
        if (r !== 0) {
            return r;
        }
        // Special sort when searching in scoped mode
        if (searchValue) {
            var elementAName = elementA.getLabel().toLowerCase();
            var elementBName = elementB.getLabel().toLowerCase();
            // Compare by name
            var r_1 = elementAName.localeCompare(elementBName);
            if (r_1 !== 0) {
                return r_1;
            }
        }
        // Default to sort by range
        var elementARange = elementA.getRange();
        var elementBRange = elementB.getRange();
        return elementARange.startLineNumber - elementBRange.startLineNumber;
    };
    return QuickOutlineAction;
}(BaseEditorQuickOpenAction));
registerEditorAction(QuickOutlineAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 *@internal
 */
function isCodeEditor(thing) {
    if (thing && typeof thing.getEditorType === 'function') {
        return thing.getEditorType() === EditorType.ICodeEditor;
    }
    else {
        return false;
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1W = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var GotoLineEntry = /** @class */ (function (_super) {
    __extends$1W(GotoLineEntry, _super);
    function GotoLineEntry(line, editor, decorator) {
        var _this = _super.call(this) || this;
        _this.editor = editor;
        _this.decorator = decorator;
        _this._parseResult = _this._parseInput(line);
        return _this;
    }
    GotoLineEntry.prototype._parseInput = function (line) {
        var numbers = line.split(',').map(function (part) { return parseInt(part, 10); }).filter(function (part) { return !isNaN(part); }), position;
        if (numbers.length === 0) {
            position = new Position(-1, -1);
        }
        else if (numbers.length === 1) {
            position = new Position(numbers[0], 1);
        }
        else {
            position = new Position(numbers[0], numbers[1]);
        }
        var model;
        if (isCodeEditor(this.editor)) {
            model = this.editor.getModel();
        }
        else {
            model = this.editor.getModel().modified;
        }
        var isValid = model.validatePosition(position).equals(position), label;
        if (isValid) {
            if (position.column && position.column > 1) {
                label = localize('gotoLineLabelValidLineAndColumn', "Go to line {0} and character {1}", position.lineNumber, position.column);
            }
            else {
                label = localize('gotoLineLabelValidLine', "Go to line {0}", position.lineNumber, position.column);
            }
        }
        else if (position.lineNumber < 1 || position.lineNumber > model.getLineCount()) {
            label = localize('gotoLineLabelEmptyWithLineLimit', "Type a line number between 1 and {0} to navigate to", model.getLineCount());
        }
        else {
            label = localize('gotoLineLabelEmptyWithLineAndColumnLimit', "Type a character between 1 and {0} to navigate to", model.getLineMaxColumn(position.lineNumber));
        }
        return {
            position: position,
            isValid: isValid,
            label: label
        };
    };
    GotoLineEntry.prototype.getLabel = function () {
        return this._parseResult.label;
    };
    GotoLineEntry.prototype.getAriaLabel = function () {
        return localize('gotoLineAriaLabel', "Go to line {0}", this._parseResult.label);
    };
    GotoLineEntry.prototype.run = function (mode, context) {
        if (mode === 1 /* OPEN */) {
            return this.runOpen();
        }
        return this.runPreview();
    };
    GotoLineEntry.prototype.runOpen = function () {
        // No-op if range is not valid
        if (!this._parseResult.isValid) {
            return false;
        }
        // Apply selection and focus
        var range = this.toSelection();
        this.editor.setSelection(range);
        this.editor.revealRangeInCenter(range, 0 /* Smooth */);
        this.editor.focus();
        return true;
    };
    GotoLineEntry.prototype.runPreview = function () {
        // No-op if range is not valid
        if (!this._parseResult.isValid) {
            this.decorator.clearDecorations();
            return false;
        }
        // Select Line Position
        var range = this.toSelection();
        this.editor.revealRangeInCenter(range, 0 /* Smooth */);
        // Decorate if possible
        this.decorator.decorateLine(range, this.editor);
        return false;
    };
    GotoLineEntry.prototype.toSelection = function () {
        return new Range(this._parseResult.position.lineNumber, this._parseResult.position.column, this._parseResult.position.lineNumber, this._parseResult.position.column);
    };
    return GotoLineEntry;
}(QuickOpenEntry));
var GotoLineAction = /** @class */ (function (_super) {
    __extends$1W(GotoLineAction, _super);
    function GotoLineAction() {
        return _super.call(this, localize('gotoLineActionInput', "Type a line number, followed by an optional colon and a character number to navigate to"), {
            id: 'editor.action.gotoLine',
            label: localize('GotoLineAction.label', "Go to Line..."),
            alias: 'Go to Line...',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: 2048 /* CtrlCmd */ | 37 /* KEY_G */,
                mac: { primary: 256 /* WinCtrl */ | 37 /* KEY_G */ },
                weight: 100 /* EditorContrib */
            }
        }) || this;
    }
    GotoLineAction.prototype.run = function (accessor, editor) {
        var _this = this;
        this._show(this.getController(editor), {
            getModel: function (value) {
                return new QuickOpenModel([new GotoLineEntry(value, editor, _this.getController(editor))]);
            },
            getAutoFocus: function (searchValue) {
                return {
                    autoFocusFirstEntry: searchValue.length > 0
                };
            }
        });
    };
    return GotoLineAction;
}(BaseEditorQuickOpenAction));
registerEditorAction(GotoLineAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1X = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var EditorActionCommandEntry = /** @class */ (function (_super) {
    __extends$1X(EditorActionCommandEntry, _super);
    function EditorActionCommandEntry(key, highlights, action, editor) {
        var _this = _super.call(this) || this;
        _this.key = key;
        _this.setHighlights(highlights);
        _this.action = action;
        _this.editor = editor;
        return _this;
    }
    EditorActionCommandEntry.prototype.getLabel = function () {
        return this.action.label;
    };
    EditorActionCommandEntry.prototype.getAriaLabel = function () {
        return localize('ariaLabelEntry', "{0}, commands", this.getLabel());
    };
    EditorActionCommandEntry.prototype.getGroupLabel = function () {
        return this.key;
    };
    EditorActionCommandEntry.prototype.run = function (mode, context) {
        var _this = this;
        if (mode === 1 /* OPEN */) {
            // Use a timeout to give the quick open widget a chance to close itself first
            setTimeout(function () {
                // Some actions are enabled only when editor has focus
                _this.editor.focus();
                try {
                    var promise = _this.action.run() || Promise.resolve();
                    promise.then(null, onUnexpectedError);
                }
                catch (error) {
                    onUnexpectedError(error);
                }
            }, 50);
            return true;
        }
        return false;
    };
    return EditorActionCommandEntry;
}(QuickOpenEntryGroup));
var QuickCommandAction = /** @class */ (function (_super) {
    __extends$1X(QuickCommandAction, _super);
    function QuickCommandAction() {
        return _super.call(this, localize('quickCommandActionInput', "Type the name of an action you want to execute"), {
            id: 'editor.action.quickCommand',
            label: localize('QuickCommandAction.label', "Command Palette"),
            alias: 'Command Palette',
            precondition: null,
            kbOpts: {
                kbExpr: EditorContextKeys.focus,
                primary: (isIE ? 512 /* Alt */ | 59 /* F1 */ : 59 /* F1 */),
                weight: 100 /* EditorContrib */
            },
            menuOpts: {
                group: 'z_commands',
                order: 1
            }
        }) || this;
    }
    QuickCommandAction.prototype.run = function (accessor, editor) {
        var _this = this;
        var keybindingService = accessor.get(IKeybindingService);
        this._show(this.getController(editor), {
            getModel: function (value) {
                return new QuickOpenModel(_this._editorActionsToEntries(keybindingService, editor, value));
            },
            getAutoFocus: function (searchValue) {
                return {
                    autoFocusFirstEntry: true,
                    autoFocusPrefixMatch: searchValue
                };
            }
        });
    };
    QuickCommandAction.prototype._sort = function (elementA, elementB) {
        var elementAName = elementA.getLabel().toLowerCase();
        var elementBName = elementB.getLabel().toLowerCase();
        return elementAName.localeCompare(elementBName);
    };
    QuickCommandAction.prototype._editorActionsToEntries = function (keybindingService, editor, searchValue) {
        var actions = editor.getSupportedActions();
        var entries = [];
        for (var i = 0; i < actions.length; i++) {
            var action = actions[i];
            var keybind = keybindingService.lookupKeybinding(action.id);
            if (action.label) {
                var highlights = matchesFuzzy(searchValue, action.label);
                if (highlights) {
                    entries.push(new EditorActionCommandEntry(keybind ? keybind.getLabel() : '', highlights, action, editor));
                }
            }
        }
        // Sort by name
        entries = entries.sort(this._sort);
        return entries;
    };
    return QuickCommandAction;
}(BaseEditorQuickOpenAction));
registerEditorAction(QuickCommandAction);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function isWinJSPromise(candidate) {
    return isThenable(candidate) && typeof candidate.done === 'function';
}
/**
 * A polyfill for the native promises. The implementation is based on
 * WinJS promises but tries to gap differences between winjs promises
 * and native promises.
 */
var PolyfillPromise = /** @class */ (function () {
    function PolyfillPromise(initOrPromise) {
        if (isWinJSPromise(initOrPromise)) {
            this._winjsPromise = initOrPromise;
        }
        else {
            this._winjsPromise = new Promise$1(function (resolve, reject) {
                var initializing = true;
                initOrPromise(function (value) {
                    if (!initializing) {
                        resolve(value);
                    }
                    else {
                        setImmediate(function () { return resolve(value); });
                    }
                }, function (err) {
                    if (!initializing) {
                        reject(err);
                    }
                    else {
                        setImmediate(function () { return reject(err); });
                    }
                });
                initializing = false;
            });
        }
    }
    PolyfillPromise.all = function (thenables) {
        return new PolyfillPromise(Promise$1.join(thenables).then(null, function (values) {
            // WinJSPromise returns a sparse array whereas
            // native promises return the *first* error
            for (var key in values) {
                if (values.hasOwnProperty(key)) {
                    return values[key];
                }
            }
        }));
    };
    PolyfillPromise.race = function (thenables) {
        // WinJSPromise returns `{ key: <index/key>, value: <promise> }`
        // from the `any` call and Promise.race just wants the value
        return new PolyfillPromise(Promise$1.any(thenables).then(function (entry) { return entry.value; }, function (err) { return err.value; }));
    };
    PolyfillPromise.resolve = function (value) {
        return new PolyfillPromise(Promise$1.wrap(value));
    };
    PolyfillPromise.reject = function (value) {
        return new PolyfillPromise(Promise$1.wrapError(value));
    };
    PolyfillPromise.prototype.then = function (onFulfilled, onRejected) {
        var sync = true;
        // To support chaining, we need to return the value of the
        // onFulfilled and onRejected callback.
        // WinJSPromise supports a flat-map style #then, ie. the callbacks
        // passed to WinJSPromise#then can return a Promise.
        var promise = new PolyfillPromise(this._winjsPromise.then(onFulfilled && function (value) {
            if (!sync) {
                return onFulfilled(value);
            }
            else {
                return new Promise$1(function (resolve, reject) {
                    setImmediate(function () {
                        var result;
                        try {
                            result = onFulfilled(value);
                        }
                        catch (err2) {
                            reject(err2);
                            return;
                        }
                        resolve(result);
                    });
                });
            }
        }, onRejected && function (err) {
            if (!sync) {
                return onRejected(err);
            }
            else {
                return new Promise$1(function (resolve, reject) {
                    setImmediate(function () {
                        var result;
                        try {
                            result = onRejected(err);
                        }
                        catch (err2) {
                            reject(err2);
                            return;
                        }
                        resolve(result);
                    });
                });
            }
        }));
        sync = false;
        return promise;
    };
    PolyfillPromise.prototype.catch = function (onRejected) {
        return this.then(null, onRejected);
    };
    return PolyfillPromise;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// THIS IS A GENERATED FILE. DO NOT EDIT DIRECTLY.
var MarkerTag;
(function (MarkerTag) {
    MarkerTag[MarkerTag["Unnecessary"] = 1] = "Unnecessary";
})(MarkerTag || (MarkerTag = {}));
var MarkerSeverity$1;
(function (MarkerSeverity) {
    MarkerSeverity[MarkerSeverity["Hint"] = 1] = "Hint";
    MarkerSeverity[MarkerSeverity["Info"] = 2] = "Info";
    MarkerSeverity[MarkerSeverity["Warning"] = 4] = "Warning";
    MarkerSeverity[MarkerSeverity["Error"] = 8] = "Error";
})(MarkerSeverity$1 || (MarkerSeverity$1 = {}));
/**
 * Virtual Key Codes, the value does not hold any inherent meaning.
 * Inspired somewhat from https://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx
 * But these are "more general", as they should work across browsers & OS`s.
 */
var KeyCode;
(function (KeyCode) {
    /**
     * Placed first to cover the 0 value of the enum.
     */
    KeyCode[KeyCode["Unknown"] = 0] = "Unknown";
    KeyCode[KeyCode["Backspace"] = 1] = "Backspace";
    KeyCode[KeyCode["Tab"] = 2] = "Tab";
    KeyCode[KeyCode["Enter"] = 3] = "Enter";
    KeyCode[KeyCode["Shift"] = 4] = "Shift";
    KeyCode[KeyCode["Ctrl"] = 5] = "Ctrl";
    KeyCode[KeyCode["Alt"] = 6] = "Alt";
    KeyCode[KeyCode["PauseBreak"] = 7] = "PauseBreak";
    KeyCode[KeyCode["CapsLock"] = 8] = "CapsLock";
    KeyCode[KeyCode["Escape"] = 9] = "Escape";
    KeyCode[KeyCode["Space"] = 10] = "Space";
    KeyCode[KeyCode["PageUp"] = 11] = "PageUp";
    KeyCode[KeyCode["PageDown"] = 12] = "PageDown";
    KeyCode[KeyCode["End"] = 13] = "End";
    KeyCode[KeyCode["Home"] = 14] = "Home";
    KeyCode[KeyCode["LeftArrow"] = 15] = "LeftArrow";
    KeyCode[KeyCode["UpArrow"] = 16] = "UpArrow";
    KeyCode[KeyCode["RightArrow"] = 17] = "RightArrow";
    KeyCode[KeyCode["DownArrow"] = 18] = "DownArrow";
    KeyCode[KeyCode["Insert"] = 19] = "Insert";
    KeyCode[KeyCode["Delete"] = 20] = "Delete";
    KeyCode[KeyCode["KEY_0"] = 21] = "KEY_0";
    KeyCode[KeyCode["KEY_1"] = 22] = "KEY_1";
    KeyCode[KeyCode["KEY_2"] = 23] = "KEY_2";
    KeyCode[KeyCode["KEY_3"] = 24] = "KEY_3";
    KeyCode[KeyCode["KEY_4"] = 25] = "KEY_4";
    KeyCode[KeyCode["KEY_5"] = 26] = "KEY_5";
    KeyCode[KeyCode["KEY_6"] = 27] = "KEY_6";
    KeyCode[KeyCode["KEY_7"] = 28] = "KEY_7";
    KeyCode[KeyCode["KEY_8"] = 29] = "KEY_8";
    KeyCode[KeyCode["KEY_9"] = 30] = "KEY_9";
    KeyCode[KeyCode["KEY_A"] = 31] = "KEY_A";
    KeyCode[KeyCode["KEY_B"] = 32] = "KEY_B";
    KeyCode[KeyCode["KEY_C"] = 33] = "KEY_C";
    KeyCode[KeyCode["KEY_D"] = 34] = "KEY_D";
    KeyCode[KeyCode["KEY_E"] = 35] = "KEY_E";
    KeyCode[KeyCode["KEY_F"] = 36] = "KEY_F";
    KeyCode[KeyCode["KEY_G"] = 37] = "KEY_G";
    KeyCode[KeyCode["KEY_H"] = 38] = "KEY_H";
    KeyCode[KeyCode["KEY_I"] = 39] = "KEY_I";
    KeyCode[KeyCode["KEY_J"] = 40] = "KEY_J";
    KeyCode[KeyCode["KEY_K"] = 41] = "KEY_K";
    KeyCode[KeyCode["KEY_L"] = 42] = "KEY_L";
    KeyCode[KeyCode["KEY_M"] = 43] = "KEY_M";
    KeyCode[KeyCode["KEY_N"] = 44] = "KEY_N";
    KeyCode[KeyCode["KEY_O"] = 45] = "KEY_O";
    KeyCode[KeyCode["KEY_P"] = 46] = "KEY_P";
    KeyCode[KeyCode["KEY_Q"] = 47] = "KEY_Q";
    KeyCode[KeyCode["KEY_R"] = 48] = "KEY_R";
    KeyCode[KeyCode["KEY_S"] = 49] = "KEY_S";
    KeyCode[KeyCode["KEY_T"] = 50] = "KEY_T";
    KeyCode[KeyCode["KEY_U"] = 51] = "KEY_U";
    KeyCode[KeyCode["KEY_V"] = 52] = "KEY_V";
    KeyCode[KeyCode["KEY_W"] = 53] = "KEY_W";
    KeyCode[KeyCode["KEY_X"] = 54] = "KEY_X";
    KeyCode[KeyCode["KEY_Y"] = 55] = "KEY_Y";
    KeyCode[KeyCode["KEY_Z"] = 56] = "KEY_Z";
    KeyCode[KeyCode["Meta"] = 57] = "Meta";
    KeyCode[KeyCode["ContextMenu"] = 58] = "ContextMenu";
    KeyCode[KeyCode["F1"] = 59] = "F1";
    KeyCode[KeyCode["F2"] = 60] = "F2";
    KeyCode[KeyCode["F3"] = 61] = "F3";
    KeyCode[KeyCode["F4"] = 62] = "F4";
    KeyCode[KeyCode["F5"] = 63] = "F5";
    KeyCode[KeyCode["F6"] = 64] = "F6";
    KeyCode[KeyCode["F7"] = 65] = "F7";
    KeyCode[KeyCode["F8"] = 66] = "F8";
    KeyCode[KeyCode["F9"] = 67] = "F9";
    KeyCode[KeyCode["F10"] = 68] = "F10";
    KeyCode[KeyCode["F11"] = 69] = "F11";
    KeyCode[KeyCode["F12"] = 70] = "F12";
    KeyCode[KeyCode["F13"] = 71] = "F13";
    KeyCode[KeyCode["F14"] = 72] = "F14";
    KeyCode[KeyCode["F15"] = 73] = "F15";
    KeyCode[KeyCode["F16"] = 74] = "F16";
    KeyCode[KeyCode["F17"] = 75] = "F17";
    KeyCode[KeyCode["F18"] = 76] = "F18";
    KeyCode[KeyCode["F19"] = 77] = "F19";
    KeyCode[KeyCode["NumLock"] = 78] = "NumLock";
    KeyCode[KeyCode["ScrollLock"] = 79] = "ScrollLock";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     * For the US standard keyboard, the ';:' key
     */
    KeyCode[KeyCode["US_SEMICOLON"] = 80] = "US_SEMICOLON";
    /**
     * For any country/region, the '+' key
     * For the US standard keyboard, the '=+' key
     */
    KeyCode[KeyCode["US_EQUAL"] = 81] = "US_EQUAL";
    /**
     * For any country/region, the ',' key
     * For the US standard keyboard, the ',<' key
     */
    KeyCode[KeyCode["US_COMMA"] = 82] = "US_COMMA";
    /**
     * For any country/region, the '-' key
     * For the US standard keyboard, the '-_' key
     */
    KeyCode[KeyCode["US_MINUS"] = 83] = "US_MINUS";
    /**
     * For any country/region, the '.' key
     * For the US standard keyboard, the '.>' key
     */
    KeyCode[KeyCode["US_DOT"] = 84] = "US_DOT";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     * For the US standard keyboard, the '/?' key
     */
    KeyCode[KeyCode["US_SLASH"] = 85] = "US_SLASH";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     * For the US standard keyboard, the '`~' key
     */
    KeyCode[KeyCode["US_BACKTICK"] = 86] = "US_BACKTICK";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     * For the US standard keyboard, the '[{' key
     */
    KeyCode[KeyCode["US_OPEN_SQUARE_BRACKET"] = 87] = "US_OPEN_SQUARE_BRACKET";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     * For the US standard keyboard, the '\|' key
     */
    KeyCode[KeyCode["US_BACKSLASH"] = 88] = "US_BACKSLASH";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     * For the US standard keyboard, the ']}' key
     */
    KeyCode[KeyCode["US_CLOSE_SQUARE_BRACKET"] = 89] = "US_CLOSE_SQUARE_BRACKET";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     * For the US standard keyboard, the ''"' key
     */
    KeyCode[KeyCode["US_QUOTE"] = 90] = "US_QUOTE";
    /**
     * Used for miscellaneous characters; it can vary by keyboard.
     */
    KeyCode[KeyCode["OEM_8"] = 91] = "OEM_8";
    /**
     * Either the angle bracket key or the backslash key on the RT 102-key keyboard.
     */
    KeyCode[KeyCode["OEM_102"] = 92] = "OEM_102";
    KeyCode[KeyCode["NUMPAD_0"] = 93] = "NUMPAD_0";
    KeyCode[KeyCode["NUMPAD_1"] = 94] = "NUMPAD_1";
    KeyCode[KeyCode["NUMPAD_2"] = 95] = "NUMPAD_2";
    KeyCode[KeyCode["NUMPAD_3"] = 96] = "NUMPAD_3";
    KeyCode[KeyCode["NUMPAD_4"] = 97] = "NUMPAD_4";
    KeyCode[KeyCode["NUMPAD_5"] = 98] = "NUMPAD_5";
    KeyCode[KeyCode["NUMPAD_6"] = 99] = "NUMPAD_6";
    KeyCode[KeyCode["NUMPAD_7"] = 100] = "NUMPAD_7";
    KeyCode[KeyCode["NUMPAD_8"] = 101] = "NUMPAD_8";
    KeyCode[KeyCode["NUMPAD_9"] = 102] = "NUMPAD_9";
    KeyCode[KeyCode["NUMPAD_MULTIPLY"] = 103] = "NUMPAD_MULTIPLY";
    KeyCode[KeyCode["NUMPAD_ADD"] = 104] = "NUMPAD_ADD";
    KeyCode[KeyCode["NUMPAD_SEPARATOR"] = 105] = "NUMPAD_SEPARATOR";
    KeyCode[KeyCode["NUMPAD_SUBTRACT"] = 106] = "NUMPAD_SUBTRACT";
    KeyCode[KeyCode["NUMPAD_DECIMAL"] = 107] = "NUMPAD_DECIMAL";
    KeyCode[KeyCode["NUMPAD_DIVIDE"] = 108] = "NUMPAD_DIVIDE";
    /**
     * Cover all key codes when IME is processing input.
     */
    KeyCode[KeyCode["KEY_IN_COMPOSITION"] = 109] = "KEY_IN_COMPOSITION";
    KeyCode[KeyCode["ABNT_C1"] = 110] = "ABNT_C1";
    KeyCode[KeyCode["ABNT_C2"] = 111] = "ABNT_C2";
    /**
     * Placed last to cover the length of the enum.
     * Please do not depend on this value!
     */
    KeyCode[KeyCode["MAX_VALUE"] = 112] = "MAX_VALUE";
})(KeyCode || (KeyCode = {}));
/**
 * The direction of a selection.
 */
var SelectionDirection;
(function (SelectionDirection) {
    /**
     * The selection starts above where it ends.
     */
    SelectionDirection[SelectionDirection["LTR"] = 0] = "LTR";
    /**
     * The selection starts below where it ends.
     */
    SelectionDirection[SelectionDirection["RTL"] = 1] = "RTL";
})(SelectionDirection || (SelectionDirection = {}));
var ScrollbarVisibility;
(function (ScrollbarVisibility) {
    ScrollbarVisibility[ScrollbarVisibility["Auto"] = 1] = "Auto";
    ScrollbarVisibility[ScrollbarVisibility["Hidden"] = 2] = "Hidden";
    ScrollbarVisibility[ScrollbarVisibility["Visible"] = 3] = "Visible";
})(ScrollbarVisibility || (ScrollbarVisibility = {}));
/**
 * Vertical Lane in the overview ruler of the editor.
 */
var OverviewRulerLane$1;
(function (OverviewRulerLane) {
    OverviewRulerLane[OverviewRulerLane["Left"] = 1] = "Left";
    OverviewRulerLane[OverviewRulerLane["Center"] = 2] = "Center";
    OverviewRulerLane[OverviewRulerLane["Right"] = 4] = "Right";
    OverviewRulerLane[OverviewRulerLane["Full"] = 7] = "Full";
})(OverviewRulerLane$1 || (OverviewRulerLane$1 = {}));
/**
 * End of line character preference.
 */
var EndOfLinePreference;
(function (EndOfLinePreference) {
    /**
     * Use the end of line character identified in the text buffer.
     */
    EndOfLinePreference[EndOfLinePreference["TextDefined"] = 0] = "TextDefined";
    /**
     * Use line feed (\n) as the end of line character.
     */
    EndOfLinePreference[EndOfLinePreference["LF"] = 1] = "LF";
    /**
     * Use carriage return and line feed (\r\n) as the end of line character.
     */
    EndOfLinePreference[EndOfLinePreference["CRLF"] = 2] = "CRLF";
})(EndOfLinePreference || (EndOfLinePreference = {}));
/**
 * The default end of line to use when instantiating models.
 */
var DefaultEndOfLine;
(function (DefaultEndOfLine) {
    /**
     * Use line feed (\n) as the end of line character.
     */
    DefaultEndOfLine[DefaultEndOfLine["LF"] = 1] = "LF";
    /**
     * Use carriage return and line feed (\r\n) as the end of line character.
     */
    DefaultEndOfLine[DefaultEndOfLine["CRLF"] = 2] = "CRLF";
})(DefaultEndOfLine || (DefaultEndOfLine = {}));
/**
 * End of line character preference.
 */
var EndOfLineSequence;
(function (EndOfLineSequence) {
    /**
     * Use line feed (\n) as the end of line character.
     */
    EndOfLineSequence[EndOfLineSequence["LF"] = 0] = "LF";
    /**
     * Use carriage return and line feed (\r\n) as the end of line character.
     */
    EndOfLineSequence[EndOfLineSequence["CRLF"] = 1] = "CRLF";
})(EndOfLineSequence || (EndOfLineSequence = {}));
/**
 * Describes the behavior of decorations when typing/editing near their edges.
 * Note: Please do not edit the values, as they very carefully match `DecorationRangeBehavior`
 */
var TrackedRangeStickiness;
(function (TrackedRangeStickiness) {
    TrackedRangeStickiness[TrackedRangeStickiness["AlwaysGrowsWhenTypingAtEdges"] = 0] = "AlwaysGrowsWhenTypingAtEdges";
    TrackedRangeStickiness[TrackedRangeStickiness["NeverGrowsWhenTypingAtEdges"] = 1] = "NeverGrowsWhenTypingAtEdges";
    TrackedRangeStickiness[TrackedRangeStickiness["GrowsOnlyWhenTypingBefore"] = 2] = "GrowsOnlyWhenTypingBefore";
    TrackedRangeStickiness[TrackedRangeStickiness["GrowsOnlyWhenTypingAfter"] = 3] = "GrowsOnlyWhenTypingAfter";
})(TrackedRangeStickiness || (TrackedRangeStickiness = {}));
var ScrollType;
(function (ScrollType) {
    ScrollType[ScrollType["Smooth"] = 0] = "Smooth";
    ScrollType[ScrollType["Immediate"] = 1] = "Immediate";
})(ScrollType || (ScrollType = {}));
/**
 * Describes the reason the cursor has changed its position.
 */
var CursorChangeReason;
(function (CursorChangeReason) {
    /**
     * Unknown or not set.
     */
    CursorChangeReason[CursorChangeReason["NotSet"] = 0] = "NotSet";
    /**
     * A `model.setValue()` was called.
     */
    CursorChangeReason[CursorChangeReason["ContentFlush"] = 1] = "ContentFlush";
    /**
     * The `model` has been changed outside of this cursor and the cursor recovers its position from associated markers.
     */
    CursorChangeReason[CursorChangeReason["RecoverFromMarkers"] = 2] = "RecoverFromMarkers";
    /**
     * There was an explicit user gesture.
     */
    CursorChangeReason[CursorChangeReason["Explicit"] = 3] = "Explicit";
    /**
     * There was a Paste.
     */
    CursorChangeReason[CursorChangeReason["Paste"] = 4] = "Paste";
    /**
     * There was an Undo.
     */
    CursorChangeReason[CursorChangeReason["Undo"] = 5] = "Undo";
    /**
     * There was a Redo.
     */
    CursorChangeReason[CursorChangeReason["Redo"] = 6] = "Redo";
})(CursorChangeReason || (CursorChangeReason = {}));
var RenderMinimap;
(function (RenderMinimap) {
    RenderMinimap[RenderMinimap["None"] = 0] = "None";
    RenderMinimap[RenderMinimap["Small"] = 1] = "Small";
    RenderMinimap[RenderMinimap["Large"] = 2] = "Large";
    RenderMinimap[RenderMinimap["SmallBlocks"] = 3] = "SmallBlocks";
    RenderMinimap[RenderMinimap["LargeBlocks"] = 4] = "LargeBlocks";
})(RenderMinimap || (RenderMinimap = {}));
/**
 * Describes how to indent wrapped lines.
 */
var WrappingIndent;
(function (WrappingIndent) {
    /**
     * No indentation => wrapped lines begin at column 1.
     */
    WrappingIndent[WrappingIndent["None"] = 0] = "None";
    /**
     * Same => wrapped lines get the same indentation as the parent.
     */
    WrappingIndent[WrappingIndent["Same"] = 1] = "Same";
    /**
     * Indent => wrapped lines get +1 indentation toward the parent.
     */
    WrappingIndent[WrappingIndent["Indent"] = 2] = "Indent";
    /**
     * DeepIndent => wrapped lines get +2 indentation toward the parent.
     */
    WrappingIndent[WrappingIndent["DeepIndent"] = 3] = "DeepIndent";
})(WrappingIndent || (WrappingIndent = {}));
/**
 * The kind of animation in which the editor's cursor should be rendered.
 */
var TextEditorCursorBlinkingStyle;
(function (TextEditorCursorBlinkingStyle) {
    /**
     * Hidden
     */
    TextEditorCursorBlinkingStyle[TextEditorCursorBlinkingStyle["Hidden"] = 0] = "Hidden";
    /**
     * Blinking
     */
    TextEditorCursorBlinkingStyle[TextEditorCursorBlinkingStyle["Blink"] = 1] = "Blink";
    /**
     * Blinking with smooth fading
     */
    TextEditorCursorBlinkingStyle[TextEditorCursorBlinkingStyle["Smooth"] = 2] = "Smooth";
    /**
     * Blinking with prolonged filled state and smooth fading
     */
    TextEditorCursorBlinkingStyle[TextEditorCursorBlinkingStyle["Phase"] = 3] = "Phase";
    /**
     * Expand collapse animation on the y axis
     */
    TextEditorCursorBlinkingStyle[TextEditorCursorBlinkingStyle["Expand"] = 4] = "Expand";
    /**
     * No-Blinking
     */
    TextEditorCursorBlinkingStyle[TextEditorCursorBlinkingStyle["Solid"] = 5] = "Solid";
})(TextEditorCursorBlinkingStyle || (TextEditorCursorBlinkingStyle = {}));
/**
 * The style in which the editor's cursor should be rendered.
 */
var TextEditorCursorStyle$1;
(function (TextEditorCursorStyle) {
    /**
     * As a vertical line (sitting between two characters).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["Line"] = 1] = "Line";
    /**
     * As a block (sitting on top of a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["Block"] = 2] = "Block";
    /**
     * As a horizontal line (sitting under a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["Underline"] = 3] = "Underline";
    /**
     * As a thin vertical line (sitting between two characters).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["LineThin"] = 4] = "LineThin";
    /**
     * As an outlined block (sitting on top of a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["BlockOutline"] = 5] = "BlockOutline";
    /**
     * As a thin horizontal line (sitting under a character).
     */
    TextEditorCursorStyle[TextEditorCursorStyle["UnderlineThin"] = 6] = "UnderlineThin";
})(TextEditorCursorStyle$1 || (TextEditorCursorStyle$1 = {}));
var RenderLineNumbersType;
(function (RenderLineNumbersType) {
    RenderLineNumbersType[RenderLineNumbersType["Off"] = 0] = "Off";
    RenderLineNumbersType[RenderLineNumbersType["On"] = 1] = "On";
    RenderLineNumbersType[RenderLineNumbersType["Relative"] = 2] = "Relative";
    RenderLineNumbersType[RenderLineNumbersType["Interval"] = 3] = "Interval";
    RenderLineNumbersType[RenderLineNumbersType["Custom"] = 4] = "Custom";
})(RenderLineNumbersType || (RenderLineNumbersType = {}));
/**
 * A positioning preference for rendering content widgets.
 */
var ContentWidgetPositionPreference;
(function (ContentWidgetPositionPreference) {
    /**
     * Place the content widget exactly at a position
     */
    ContentWidgetPositionPreference[ContentWidgetPositionPreference["EXACT"] = 0] = "EXACT";
    /**
     * Place the content widget above a position
     */
    ContentWidgetPositionPreference[ContentWidgetPositionPreference["ABOVE"] = 1] = "ABOVE";
    /**
     * Place the content widget below a position
     */
    ContentWidgetPositionPreference[ContentWidgetPositionPreference["BELOW"] = 2] = "BELOW";
})(ContentWidgetPositionPreference || (ContentWidgetPositionPreference = {}));
/**
 * A positioning preference for rendering overlay widgets.
 */
var OverlayWidgetPositionPreference;
(function (OverlayWidgetPositionPreference) {
    /**
     * Position the overlay widget in the top right corner
     */
    OverlayWidgetPositionPreference[OverlayWidgetPositionPreference["TOP_RIGHT_CORNER"] = 0] = "TOP_RIGHT_CORNER";
    /**
     * Position the overlay widget in the bottom right corner
     */
    OverlayWidgetPositionPreference[OverlayWidgetPositionPreference["BOTTOM_RIGHT_CORNER"] = 1] = "BOTTOM_RIGHT_CORNER";
    /**
     * Position the overlay widget in the top center
     */
    OverlayWidgetPositionPreference[OverlayWidgetPositionPreference["TOP_CENTER"] = 2] = "TOP_CENTER";
})(OverlayWidgetPositionPreference || (OverlayWidgetPositionPreference = {}));
/**
 * Type of hit element with the mouse in the editor.
 */
var MouseTargetType;
(function (MouseTargetType) {
    /**
     * Mouse is on top of an unknown element.
     */
    MouseTargetType[MouseTargetType["UNKNOWN"] = 0] = "UNKNOWN";
    /**
     * Mouse is on top of the textarea used for input.
     */
    MouseTargetType[MouseTargetType["TEXTAREA"] = 1] = "TEXTAREA";
    /**
     * Mouse is on top of the glyph margin
     */
    MouseTargetType[MouseTargetType["GUTTER_GLYPH_MARGIN"] = 2] = "GUTTER_GLYPH_MARGIN";
    /**
     * Mouse is on top of the line numbers
     */
    MouseTargetType[MouseTargetType["GUTTER_LINE_NUMBERS"] = 3] = "GUTTER_LINE_NUMBERS";
    /**
     * Mouse is on top of the line decorations
     */
    MouseTargetType[MouseTargetType["GUTTER_LINE_DECORATIONS"] = 4] = "GUTTER_LINE_DECORATIONS";
    /**
     * Mouse is on top of the whitespace left in the gutter by a view zone.
     */
    MouseTargetType[MouseTargetType["GUTTER_VIEW_ZONE"] = 5] = "GUTTER_VIEW_ZONE";
    /**
     * Mouse is on top of text in the content.
     */
    MouseTargetType[MouseTargetType["CONTENT_TEXT"] = 6] = "CONTENT_TEXT";
    /**
     * Mouse is on top of empty space in the content (e.g. after line text or below last line)
     */
    MouseTargetType[MouseTargetType["CONTENT_EMPTY"] = 7] = "CONTENT_EMPTY";
    /**
     * Mouse is on top of a view zone in the content.
     */
    MouseTargetType[MouseTargetType["CONTENT_VIEW_ZONE"] = 8] = "CONTENT_VIEW_ZONE";
    /**
     * Mouse is on top of a content widget.
     */
    MouseTargetType[MouseTargetType["CONTENT_WIDGET"] = 9] = "CONTENT_WIDGET";
    /**
     * Mouse is on top of the decorations overview ruler.
     */
    MouseTargetType[MouseTargetType["OVERVIEW_RULER"] = 10] = "OVERVIEW_RULER";
    /**
     * Mouse is on top of a scrollbar.
     */
    MouseTargetType[MouseTargetType["SCROLLBAR"] = 11] = "SCROLLBAR";
    /**
     * Mouse is on top of an overlay widget.
     */
    MouseTargetType[MouseTargetType["OVERLAY_WIDGET"] = 12] = "OVERLAY_WIDGET";
    /**
     * Mouse is outside of the editor.
     */
    MouseTargetType[MouseTargetType["OUTSIDE_EDITOR"] = 13] = "OUTSIDE_EDITOR";
})(MouseTargetType || (MouseTargetType = {}));
/**
 * Describes what to do with the indentation when pressing Enter.
 */
var IndentAction$1;
(function (IndentAction) {
    /**
     * Insert new line and copy the previous line's indentation.
     */
    IndentAction[IndentAction["None"] = 0] = "None";
    /**
     * Insert new line and indent once (relative to the previous line's indentation).
     */
    IndentAction[IndentAction["Indent"] = 1] = "Indent";
    /**
     * Insert two new lines:
     *  - the first one indented which will hold the cursor
     *  - the second one at the same indentation level
     */
    IndentAction[IndentAction["IndentOutdent"] = 2] = "IndentOutdent";
    /**
     * Insert new line and outdent once (relative to the previous line's indentation).
     */
    IndentAction[IndentAction["Outdent"] = 3] = "Outdent";
})(IndentAction$1 || (IndentAction$1 = {}));
var CompletionItemKind;
(function (CompletionItemKind) {
    CompletionItemKind[CompletionItemKind["Method"] = 0] = "Method";
    CompletionItemKind[CompletionItemKind["Function"] = 1] = "Function";
    CompletionItemKind[CompletionItemKind["Constructor"] = 2] = "Constructor";
    CompletionItemKind[CompletionItemKind["Field"] = 3] = "Field";
    CompletionItemKind[CompletionItemKind["Variable"] = 4] = "Variable";
    CompletionItemKind[CompletionItemKind["Class"] = 5] = "Class";
    CompletionItemKind[CompletionItemKind["Struct"] = 6] = "Struct";
    CompletionItemKind[CompletionItemKind["Interface"] = 7] = "Interface";
    CompletionItemKind[CompletionItemKind["Module"] = 8] = "Module";
    CompletionItemKind[CompletionItemKind["Property"] = 9] = "Property";
    CompletionItemKind[CompletionItemKind["Event"] = 10] = "Event";
    CompletionItemKind[CompletionItemKind["Operator"] = 11] = "Operator";
    CompletionItemKind[CompletionItemKind["Unit"] = 12] = "Unit";
    CompletionItemKind[CompletionItemKind["Value"] = 13] = "Value";
    CompletionItemKind[CompletionItemKind["Constant"] = 14] = "Constant";
    CompletionItemKind[CompletionItemKind["Enum"] = 15] = "Enum";
    CompletionItemKind[CompletionItemKind["EnumMember"] = 16] = "EnumMember";
    CompletionItemKind[CompletionItemKind["Keyword"] = 17] = "Keyword";
    CompletionItemKind[CompletionItemKind["Text"] = 18] = "Text";
    CompletionItemKind[CompletionItemKind["Color"] = 19] = "Color";
    CompletionItemKind[CompletionItemKind["File"] = 20] = "File";
    CompletionItemKind[CompletionItemKind["Reference"] = 21] = "Reference";
    CompletionItemKind[CompletionItemKind["Customcolor"] = 22] = "Customcolor";
    CompletionItemKind[CompletionItemKind["Folder"] = 23] = "Folder";
    CompletionItemKind[CompletionItemKind["TypeParameter"] = 24] = "TypeParameter";
    CompletionItemKind[CompletionItemKind["Snippet"] = 25] = "Snippet";
})(CompletionItemKind || (CompletionItemKind = {}));
var CompletionItemInsertTextRule;
(function (CompletionItemInsertTextRule) {
    /**
     * Adjust whitespace/indentation of multiline insert texts to
     * match the current line indentation.
     */
    CompletionItemInsertTextRule[CompletionItemInsertTextRule["KeepWhitespace"] = 1] = "KeepWhitespace";
    /**
     * `insertText` is a snippet.
     */
    CompletionItemInsertTextRule[CompletionItemInsertTextRule["InsertAsSnippet"] = 4] = "InsertAsSnippet";
})(CompletionItemInsertTextRule || (CompletionItemInsertTextRule = {}));
/**
 * How a suggest provider was triggered.
 */
var CompletionTriggerKind;
(function (CompletionTriggerKind) {
    CompletionTriggerKind[CompletionTriggerKind["Invoke"] = 0] = "Invoke";
    CompletionTriggerKind[CompletionTriggerKind["TriggerCharacter"] = 1] = "TriggerCharacter";
    CompletionTriggerKind[CompletionTriggerKind["TriggerForIncompleteCompletions"] = 2] = "TriggerForIncompleteCompletions";
})(CompletionTriggerKind || (CompletionTriggerKind = {}));
var SignatureHelpTriggerReason$1;
(function (SignatureHelpTriggerReason) {
    SignatureHelpTriggerReason[SignatureHelpTriggerReason["Invoke"] = 1] = "Invoke";
    SignatureHelpTriggerReason[SignatureHelpTriggerReason["TriggerCharacter"] = 2] = "TriggerCharacter";
    SignatureHelpTriggerReason[SignatureHelpTriggerReason["ContentChange"] = 3] = "ContentChange";
})(SignatureHelpTriggerReason$1 || (SignatureHelpTriggerReason$1 = {}));
/**
 * A document highlight kind.
 */
var DocumentHighlightKind$1;
(function (DocumentHighlightKind) {
    /**
     * A textual occurrence.
     */
    DocumentHighlightKind[DocumentHighlightKind["Text"] = 0] = "Text";
    /**
     * Read-access of a symbol, like reading a variable.
     */
    DocumentHighlightKind[DocumentHighlightKind["Read"] = 1] = "Read";
    /**
     * Write-access of a symbol, like writing to a variable.
     */
    DocumentHighlightKind[DocumentHighlightKind["Write"] = 2] = "Write";
})(DocumentHighlightKind$1 || (DocumentHighlightKind$1 = {}));
/**
 * A symbol kind.
 */
var SymbolKind;
(function (SymbolKind) {
    SymbolKind[SymbolKind["File"] = 0] = "File";
    SymbolKind[SymbolKind["Module"] = 1] = "Module";
    SymbolKind[SymbolKind["Namespace"] = 2] = "Namespace";
    SymbolKind[SymbolKind["Package"] = 3] = "Package";
    SymbolKind[SymbolKind["Class"] = 4] = "Class";
    SymbolKind[SymbolKind["Method"] = 5] = "Method";
    SymbolKind[SymbolKind["Property"] = 6] = "Property";
    SymbolKind[SymbolKind["Field"] = 7] = "Field";
    SymbolKind[SymbolKind["Constructor"] = 8] = "Constructor";
    SymbolKind[SymbolKind["Enum"] = 9] = "Enum";
    SymbolKind[SymbolKind["Interface"] = 10] = "Interface";
    SymbolKind[SymbolKind["Function"] = 11] = "Function";
    SymbolKind[SymbolKind["Variable"] = 12] = "Variable";
    SymbolKind[SymbolKind["Constant"] = 13] = "Constant";
    SymbolKind[SymbolKind["String"] = 14] = "String";
    SymbolKind[SymbolKind["Number"] = 15] = "Number";
    SymbolKind[SymbolKind["Boolean"] = 16] = "Boolean";
    SymbolKind[SymbolKind["Array"] = 17] = "Array";
    SymbolKind[SymbolKind["Object"] = 18] = "Object";
    SymbolKind[SymbolKind["Key"] = 19] = "Key";
    SymbolKind[SymbolKind["Null"] = 20] = "Null";
    SymbolKind[SymbolKind["EnumMember"] = 21] = "EnumMember";
    SymbolKind[SymbolKind["Struct"] = 22] = "Struct";
    SymbolKind[SymbolKind["Event"] = 23] = "Event";
    SymbolKind[SymbolKind["Operator"] = 24] = "Operator";
    SymbolKind[SymbolKind["TypeParameter"] = 25] = "TypeParameter";
})(SymbolKind || (SymbolKind = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var KeyMod = /** @class */ (function () {
    function KeyMod() {
    }
    KeyMod.chord = function (firstPart, secondPart) {
        return KeyChord(firstPart, secondPart);
    };
    KeyMod.CtrlCmd = 2048 /* CtrlCmd */;
    KeyMod.Shift = 1024 /* Shift */;
    KeyMod.Alt = 512 /* Alt */;
    KeyMod.WinCtrl = 256 /* WinCtrl */;
    return KeyMod;
}());
function createMonacoBaseAPI() {
    return {
        editor: undefined,
        languages: undefined,
        CancellationTokenSource: CancellationTokenSource,
        Emitter: Emitter,
        KeyCode: KeyCode,
        KeyMod: KeyMod,
        Position: Position,
        Range: Range,
        Selection: Selection,
        SelectionDirection: SelectionDirection,
        MarkerSeverity: MarkerSeverity$1,
        MarkerTag: MarkerTag,
        Promise: TPromise,
        Uri: URI,
        Token: Token
    };
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function parse$2(text) {
    var data = JSON.parse(text);
    data = revive(data, 0);
    return data;
}
function revive(obj, depth) {
    if (!obj || depth > 200) {
        return obj;
    }
    if (typeof obj === 'object') {
        switch (obj.$mid) {
            case 1: return URI.revive(obj);
            case 2: return new RegExp(obj.source, obj.flags);
        }
        // walk object (or array)
        for (var key in obj) {
            if (Object.hasOwnProperty.call(obj, key)) {
                obj[key] = revive(obj[key], depth + 1);
            }
        }
    }
    return obj;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var NullTelemetryService = new /** @class */ (function () {
    function class_1() {
    }
    class_1.prototype.publicLog = function (eventName, data) {
        return TPromise.wrap(void 0);
    };
    class_1.prototype.getTelemetryInfo = function () {
        return TPromise.wrap({
            instanceId: 'someValue.instanceId',
            sessionId: 'someValue.sessionId',
            machineId: 'someValue.machineId'
        });
    };
    return class_1;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$r = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$o = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var OpenerService = /** @class */ (function () {
    function OpenerService(_editorService, _commandService, _telemetryService) {
        if (_telemetryService === void 0) { _telemetryService = NullTelemetryService; }
        this._editorService = _editorService;
        this._commandService = _commandService;
        this._telemetryService = _telemetryService;
        //
    }
    OpenerService.prototype.open = function (resource, options) {
        var _a;
        if (this._telemetryService) {
            /* __GDPR__
                "openerService" : {
                    "scheme" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" }
                }
            */
            this._telemetryService.publicLog('openerService', { scheme: resource.scheme });
        }
        var scheme = resource.scheme, path = resource.path, query = resource.query, fragment = resource.fragment;
        var promise = undefined;
        if (scheme === Schemas.http || scheme === Schemas.https || scheme === Schemas.mailto) {
            // open http or default mail application
            windowOpenNoOpener(resource.toString(true));
        }
        else if (scheme === 'command' && CommandsRegistry.getCommand(path)) {
            // execute as command
            var args = [];
            try {
                args = parse$2(query);
                if (!Array.isArray(args)) {
                    args = [args];
                }
            }
            catch (e) {
                //
            }
            promise = (_a = this._commandService).executeCommand.apply(_a, [path].concat(args));
        }
        else {
            var selection = undefined;
            var match = /^L?(\d+)(?:,(\d+))?/.exec(fragment);
            if (match) {
                // support file:///some/file.js#73,84
                // support file:///some/file.js#L73
                selection = {
                    startLineNumber: parseInt(match[1]),
                    startColumn: match[2] ? parseInt(match[2]) : 1
                };
                // remove fragment
                resource = resource.with({ fragment: '' });
            }
            if (!resource.scheme) {
                // we cannot handle those
                return Promise.resolve(undefined);
            }
            else if (resource.scheme === Schemas.file) {
                resource = normalizePath(resource); // workaround for non-normalized paths (https://github.com/Microsoft/vscode/issues/12954)
            }
            promise = this._editorService.openCodeEditor({ resource: resource, options: { selection: selection, } }, this._editorService.getFocusedCodeEditor(), options && options.openToSide);
        }
        return Promise.resolve(promise);
    };
    OpenerService = __decorate$r([
        __param$o(0, ICodeEditorService),
        __param$o(1, ICommandService),
        __param$o(2, optional(ITelemetryService))
    ], OpenerService);
    return OpenerService;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1Y = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var global$1 = self;
// When missing, polyfill the native promise
// with our winjs-based polyfill
if (typeof global$1.Promise === 'undefined') {
    global$1.Promise = PolyfillPromise;
}
var INITIALIZE = '$initialize';
var webWorkerWarningLogged = false;
function logOnceWebWorkerWarning(err) {
    if (!isWeb) {
        // running tests
        return;
    }
    if (!webWorkerWarningLogged) {
        webWorkerWarningLogged = true;
        console.warn('Could not create web worker(s). Falling back to loading web worker code in main thread, which might cause UI freezes. Please see https://github.com/Microsoft/monaco-editor#faq');
    }
    console.warn(err.message);
}
var SimpleWorkerProtocol = /** @class */ (function () {
    function SimpleWorkerProtocol(handler) {
        this._workerId = -1;
        this._handler = handler;
        this._lastSentReq = 0;
        this._pendingReplies = Object.create(null);
    }
    SimpleWorkerProtocol.prototype.setWorkerId = function (workerId) {
        this._workerId = workerId;
    };
    SimpleWorkerProtocol.prototype.sendMessage = function (method, args) {
        var _this = this;
        var req = String(++this._lastSentReq);
        return new Promise(function (resolve, reject) {
            _this._pendingReplies[req] = {
                resolve: resolve,
                reject: reject
            };
            _this._send({
                vsWorker: _this._workerId,
                req: req,
                method: method,
                args: args
            });
        });
    };
    SimpleWorkerProtocol.prototype.handleMessage = function (serializedMessage) {
        var message;
        try {
            message = JSON.parse(serializedMessage);
        }
        catch (e) {
            // nothing
            return;
        }
        if (!message || !message.vsWorker) {
            return;
        }
        if (this._workerId !== -1 && message.vsWorker !== this._workerId) {
            return;
        }
        this._handleMessage(message);
    };
    SimpleWorkerProtocol.prototype._handleMessage = function (msg) {
        var _this = this;
        if (msg.seq) {
            var replyMessage = msg;
            if (!this._pendingReplies[replyMessage.seq]) {
                console.warn('Got reply to unknown seq');
                return;
            }
            var reply = this._pendingReplies[replyMessage.seq];
            delete this._pendingReplies[replyMessage.seq];
            if (replyMessage.err) {
                var err = replyMessage.err;
                if (replyMessage.err.$isError) {
                    err = new Error();
                    err.name = replyMessage.err.name;
                    err.message = replyMessage.err.message;
                    err.stack = replyMessage.err.stack;
                }
                reply.reject(err);
                return;
            }
            reply.resolve(replyMessage.res);
            return;
        }
        var requestMessage = msg;
        var req = requestMessage.req;
        var result = this._handler.handleMessage(requestMessage.method, requestMessage.args);
        result.then(function (r) {
            _this._send({
                vsWorker: _this._workerId,
                seq: req,
                res: r,
                err: undefined
            });
        }, function (e) {
            if (e.detail instanceof Error) {
                // Loading errors have a detail property that points to the actual error
                e.detail = transformErrorForSerialization(e.detail);
            }
            _this._send({
                vsWorker: _this._workerId,
                seq: req,
                res: undefined,
                err: transformErrorForSerialization(e)
            });
        });
    };
    SimpleWorkerProtocol.prototype._send = function (msg) {
        var strMsg = JSON.stringify(msg);
        // console.log('SENDING: ' + strMsg);
        this._handler.sendMessage(strMsg);
    };
    return SimpleWorkerProtocol;
}());
/**
 * Main thread side
 */
var SimpleWorkerClient = /** @class */ (function (_super) {
    __extends$1Y(SimpleWorkerClient, _super);
    function SimpleWorkerClient(workerFactory, moduleId) {
        var _this = _super.call(this) || this;
        var lazyProxyReject = null;
        _this._worker = _this._register(workerFactory.create('vs/base/common/worker/simpleWorker', function (msg) {
            _this._protocol.handleMessage(msg);
        }, function (err) {
            // in Firefox, web workers fail lazily :(
            // we will reject the proxy
            if (lazyProxyReject) {
                lazyProxyReject(err);
            }
        }));
        _this._protocol = new SimpleWorkerProtocol({
            sendMessage: function (msg) {
                _this._worker.postMessage(msg);
            },
            handleMessage: function (method, args) {
                // Intentionally not supporting worker -> main requests
                return Promise.resolve(null);
            }
        });
        _this._protocol.setWorkerId(_this._worker.getId());
        // Gather loader configuration
        var loaderConfiguration = null;
        if (typeof self.require !== 'undefined' && typeof self.require.getConfig === 'function') {
            // Get the configuration from the Monaco AMD Loader
            loaderConfiguration = self.require.getConfig();
        }
        else if (typeof self.requirejs !== 'undefined') {
            // Get the configuration from requirejs
            loaderConfiguration = self.requirejs.s.contexts._.config;
        }
        // Send initialize message
        _this._onModuleLoaded = _this._protocol.sendMessage(INITIALIZE, [
            _this._worker.getId(),
            moduleId,
            loaderConfiguration
        ]);
        _this._lazyProxy = new Promise(function (resolve, reject) {
            lazyProxyReject = reject;
            _this._onModuleLoaded.then(function (availableMethods) {
                var proxy = {};
                for (var i = 0; i < availableMethods.length; i++) {
                    proxy[availableMethods[i]] = createProxyMethod(availableMethods[i], proxyMethodRequest);
                }
                resolve(proxy);
            }, function (e) {
                reject(e);
                _this._onError('Worker failed to load ' + moduleId, e);
            });
        });
        // Create proxy to loaded code
        var proxyMethodRequest = function (method, args) {
            return _this._request(method, args);
        };
        var createProxyMethod = function (method, proxyMethodRequest) {
            return function () {
                var args = Array.prototype.slice.call(arguments, 0);
                return proxyMethodRequest(method, args);
            };
        };
        return _this;
    }
    SimpleWorkerClient.prototype.getProxyObject = function () {
        return this._lazyProxy;
    };
    SimpleWorkerClient.prototype._request = function (method, args) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this._onModuleLoaded.then(function () {
                _this._protocol.sendMessage(method, args).then(resolve, reject);
            }, reject);
        });
    };
    SimpleWorkerClient.prototype._onError = function (message, error) {
        console.error(message);
        console.info(error);
    };
    return SimpleWorkerClient;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function getWorker(workerId, label) {
    // Option for hosts to overwrite the worker script (used in the standalone editor)
    if (globals.MonacoEnvironment) {
        if (typeof globals.MonacoEnvironment.getWorker === 'function') {
            return globals.MonacoEnvironment.getWorker(workerId, label);
        }
        if (typeof globals.MonacoEnvironment.getWorkerUrl === 'function') {
            return new Worker(globals.MonacoEnvironment.getWorkerUrl(workerId, label));
        }
    }
    // ESM-comment-begin
    // 	if (typeof require === 'function') {
    // 		return new Worker(require.toUrl('./' + workerId) + '#' + label);
    // 	}
    // ESM-comment-end
    throw new Error("You must define a function MonacoEnvironment.getWorkerUrl or MonacoEnvironment.getWorker");
}
/**
 * A worker that uses HTML5 web workers so that is has
 * its own global scope and its own thread.
 */
var WebWorker = /** @class */ (function () {
    function WebWorker(moduleId, id, label, onMessageCallback, onErrorCallback) {
        this.id = id;
        this.worker = getWorker('workerMain.js', label);
        this.postMessage(moduleId);
        this.worker.onmessage = function (ev) {
            onMessageCallback(ev.data);
        };
        if (typeof this.worker.addEventListener === 'function') {
            this.worker.addEventListener('error', onErrorCallback);
        }
    }
    WebWorker.prototype.getId = function () {
        return this.id;
    };
    WebWorker.prototype.postMessage = function (msg) {
        if (this.worker) {
            this.worker.postMessage(msg);
        }
    };
    WebWorker.prototype.dispose = function () {
        if (this.worker) {
            this.worker.terminate();
        }
        this.worker = null;
    };
    return WebWorker;
}());
var DefaultWorkerFactory = /** @class */ (function () {
    function DefaultWorkerFactory(label) {
        this._label = label;
        this._webWorkerFailedBeforeError = false;
    }
    DefaultWorkerFactory.prototype.create = function (moduleId, onMessageCallback, onErrorCallback) {
        var _this = this;
        var workerId = (++DefaultWorkerFactory.LAST_WORKER_ID);
        if (this._webWorkerFailedBeforeError) {
            throw this._webWorkerFailedBeforeError;
        }
        return new WebWorker(moduleId, workerId, this._label || 'anonymous' + workerId, onMessageCallback, function (err) {
            logOnceWebWorkerWarning(err);
            _this._webWorkerFailedBeforeError = err;
            onErrorCallback(err);
        });
    };
    DefaultWorkerFactory.LAST_WORKER_ID = 0;
    return DefaultWorkerFactory;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MAXIMUM_RUN_TIME = 5000; // 5 seconds
var MINIMUM_MATCHING_CHARACTER_LENGTH = 3;
function computeDiff(originalSequence, modifiedSequence, continueProcessingPredicate, pretty) {
    var diffAlgo = new LcsDiff(originalSequence, modifiedSequence, continueProcessingPredicate);
    return diffAlgo.ComputeDiff(pretty);
}
var LineMarkerSequence = /** @class */ (function () {
    function LineMarkerSequence(lines) {
        var startColumns = [];
        var endColumns = [];
        for (var i = 0, length_1 = lines.length; i < length_1; i++) {
            startColumns[i] = LineMarkerSequence._getFirstNonBlankColumn(lines[i], 1);
            endColumns[i] = LineMarkerSequence._getLastNonBlankColumn(lines[i], 1);
        }
        this._lines = lines;
        this._startColumns = startColumns;
        this._endColumns = endColumns;
    }
    LineMarkerSequence.prototype.getLength = function () {
        return this._lines.length;
    };
    LineMarkerSequence.prototype.getElementAtIndex = function (i) {
        return this._lines[i].substring(this._startColumns[i] - 1, this._endColumns[i] - 1);
    };
    LineMarkerSequence.prototype.getStartLineNumber = function (i) {
        return i + 1;
    };
    LineMarkerSequence.prototype.getEndLineNumber = function (i) {
        return i + 1;
    };
    LineMarkerSequence._getFirstNonBlankColumn = function (txt, defaultValue) {
        var r = firstNonWhitespaceIndex(txt);
        if (r === -1) {
            return defaultValue;
        }
        return r + 1;
    };
    LineMarkerSequence._getLastNonBlankColumn = function (txt, defaultValue) {
        var r = lastNonWhitespaceIndex(txt);
        if (r === -1) {
            return defaultValue;
        }
        return r + 2;
    };
    LineMarkerSequence.prototype.getCharSequence = function (shouldIgnoreTrimWhitespace, startIndex, endIndex) {
        var charCodes = [];
        var lineNumbers = [];
        var columns = [];
        var len = 0;
        for (var index = startIndex; index <= endIndex; index++) {
            var lineContent = this._lines[index];
            var startColumn = (shouldIgnoreTrimWhitespace ? this._startColumns[index] : 1);
            var endColumn = (shouldIgnoreTrimWhitespace ? this._endColumns[index] : lineContent.length + 1);
            for (var col = startColumn; col < endColumn; col++) {
                charCodes[len] = lineContent.charCodeAt(col - 1);
                lineNumbers[len] = index + 1;
                columns[len] = col;
                len++;
            }
        }
        return new CharSequence(charCodes, lineNumbers, columns);
    };
    return LineMarkerSequence;
}());
var CharSequence = /** @class */ (function () {
    function CharSequence(charCodes, lineNumbers, columns) {
        this._charCodes = charCodes;
        this._lineNumbers = lineNumbers;
        this._columns = columns;
    }
    CharSequence.prototype.getLength = function () {
        return this._charCodes.length;
    };
    CharSequence.prototype.getElementAtIndex = function (i) {
        return this._charCodes[i];
    };
    CharSequence.prototype.getStartLineNumber = function (i) {
        return this._lineNumbers[i];
    };
    CharSequence.prototype.getStartColumn = function (i) {
        return this._columns[i];
    };
    CharSequence.prototype.getEndLineNumber = function (i) {
        return this._lineNumbers[i];
    };
    CharSequence.prototype.getEndColumn = function (i) {
        return this._columns[i] + 1;
    };
    return CharSequence;
}());
var CharChange = /** @class */ (function () {
    function CharChange(originalStartLineNumber, originalStartColumn, originalEndLineNumber, originalEndColumn, modifiedStartLineNumber, modifiedStartColumn, modifiedEndLineNumber, modifiedEndColumn) {
        this.originalStartLineNumber = originalStartLineNumber;
        this.originalStartColumn = originalStartColumn;
        this.originalEndLineNumber = originalEndLineNumber;
        this.originalEndColumn = originalEndColumn;
        this.modifiedStartLineNumber = modifiedStartLineNumber;
        this.modifiedStartColumn = modifiedStartColumn;
        this.modifiedEndLineNumber = modifiedEndLineNumber;
        this.modifiedEndColumn = modifiedEndColumn;
    }
    CharChange.createFromDiffChange = function (diffChange, originalCharSequence, modifiedCharSequence) {
        var originalStartLineNumber;
        var originalStartColumn;
        var originalEndLineNumber;
        var originalEndColumn;
        var modifiedStartLineNumber;
        var modifiedStartColumn;
        var modifiedEndLineNumber;
        var modifiedEndColumn;
        if (diffChange.originalLength === 0) {
            originalStartLineNumber = 0;
            originalStartColumn = 0;
            originalEndLineNumber = 0;
            originalEndColumn = 0;
        }
        else {
            originalStartLineNumber = originalCharSequence.getStartLineNumber(diffChange.originalStart);
            originalStartColumn = originalCharSequence.getStartColumn(diffChange.originalStart);
            originalEndLineNumber = originalCharSequence.getEndLineNumber(diffChange.originalStart + diffChange.originalLength - 1);
            originalEndColumn = originalCharSequence.getEndColumn(diffChange.originalStart + diffChange.originalLength - 1);
        }
        if (diffChange.modifiedLength === 0) {
            modifiedStartLineNumber = 0;
            modifiedStartColumn = 0;
            modifiedEndLineNumber = 0;
            modifiedEndColumn = 0;
        }
        else {
            modifiedStartLineNumber = modifiedCharSequence.getStartLineNumber(diffChange.modifiedStart);
            modifiedStartColumn = modifiedCharSequence.getStartColumn(diffChange.modifiedStart);
            modifiedEndLineNumber = modifiedCharSequence.getEndLineNumber(diffChange.modifiedStart + diffChange.modifiedLength - 1);
            modifiedEndColumn = modifiedCharSequence.getEndColumn(diffChange.modifiedStart + diffChange.modifiedLength - 1);
        }
        return new CharChange(originalStartLineNumber, originalStartColumn, originalEndLineNumber, originalEndColumn, modifiedStartLineNumber, modifiedStartColumn, modifiedEndLineNumber, modifiedEndColumn);
    };
    return CharChange;
}());
function postProcessCharChanges(rawChanges) {
    if (rawChanges.length <= 1) {
        return rawChanges;
    }
    var result = [rawChanges[0]];
    var prevChange = result[0];
    for (var i = 1, len = rawChanges.length; i < len; i++) {
        var currChange = rawChanges[i];
        var originalMatchingLength = currChange.originalStart - (prevChange.originalStart + prevChange.originalLength);
        var modifiedMatchingLength = currChange.modifiedStart - (prevChange.modifiedStart + prevChange.modifiedLength);
        // Both of the above should be equal, but the continueProcessingPredicate may prevent this from being true
        var matchingLength = Math.min(originalMatchingLength, modifiedMatchingLength);
        if (matchingLength < MINIMUM_MATCHING_CHARACTER_LENGTH) {
            // Merge the current change into the previous one
            prevChange.originalLength = (currChange.originalStart + currChange.originalLength) - prevChange.originalStart;
            prevChange.modifiedLength = (currChange.modifiedStart + currChange.modifiedLength) - prevChange.modifiedStart;
        }
        else {
            // Add the current change
            result.push(currChange);
            prevChange = currChange;
        }
    }
    return result;
}
var LineChange = /** @class */ (function () {
    function LineChange(originalStartLineNumber, originalEndLineNumber, modifiedStartLineNumber, modifiedEndLineNumber, charChanges) {
        this.originalStartLineNumber = originalStartLineNumber;
        this.originalEndLineNumber = originalEndLineNumber;
        this.modifiedStartLineNumber = modifiedStartLineNumber;
        this.modifiedEndLineNumber = modifiedEndLineNumber;
        this.charChanges = charChanges;
    }
    LineChange.createFromDiffResult = function (shouldIgnoreTrimWhitespace, diffChange, originalLineSequence, modifiedLineSequence, continueProcessingPredicate, shouldComputeCharChanges, shouldPostProcessCharChanges) {
        var originalStartLineNumber;
        var originalEndLineNumber;
        var modifiedStartLineNumber;
        var modifiedEndLineNumber;
        var charChanges = undefined;
        if (diffChange.originalLength === 0) {
            originalStartLineNumber = originalLineSequence.getStartLineNumber(diffChange.originalStart) - 1;
            originalEndLineNumber = 0;
        }
        else {
            originalStartLineNumber = originalLineSequence.getStartLineNumber(diffChange.originalStart);
            originalEndLineNumber = originalLineSequence.getEndLineNumber(diffChange.originalStart + diffChange.originalLength - 1);
        }
        if (diffChange.modifiedLength === 0) {
            modifiedStartLineNumber = modifiedLineSequence.getStartLineNumber(diffChange.modifiedStart) - 1;
            modifiedEndLineNumber = 0;
        }
        else {
            modifiedStartLineNumber = modifiedLineSequence.getStartLineNumber(diffChange.modifiedStart);
            modifiedEndLineNumber = modifiedLineSequence.getEndLineNumber(diffChange.modifiedStart + diffChange.modifiedLength - 1);
        }
        if (shouldComputeCharChanges && diffChange.originalLength !== 0 && diffChange.modifiedLength !== 0 && continueProcessingPredicate()) {
            var originalCharSequence = originalLineSequence.getCharSequence(shouldIgnoreTrimWhitespace, diffChange.originalStart, diffChange.originalStart + diffChange.originalLength - 1);
            var modifiedCharSequence = modifiedLineSequence.getCharSequence(shouldIgnoreTrimWhitespace, diffChange.modifiedStart, diffChange.modifiedStart + diffChange.modifiedLength - 1);
            var rawChanges = computeDiff(originalCharSequence, modifiedCharSequence, continueProcessingPredicate, true);
            if (shouldPostProcessCharChanges) {
                rawChanges = postProcessCharChanges(rawChanges);
            }
            charChanges = [];
            for (var i = 0, length_2 = rawChanges.length; i < length_2; i++) {
                charChanges.push(CharChange.createFromDiffChange(rawChanges[i], originalCharSequence, modifiedCharSequence));
            }
        }
        return new LineChange(originalStartLineNumber, originalEndLineNumber, modifiedStartLineNumber, modifiedEndLineNumber, charChanges);
    };
    return LineChange;
}());
var DiffComputer = /** @class */ (function () {
    function DiffComputer(originalLines, modifiedLines, opts) {
        this.shouldComputeCharChanges = opts.shouldComputeCharChanges;
        this.shouldPostProcessCharChanges = opts.shouldPostProcessCharChanges;
        this.shouldIgnoreTrimWhitespace = opts.shouldIgnoreTrimWhitespace;
        this.shouldMakePrettyDiff = opts.shouldMakePrettyDiff;
        this.maximumRunTimeMs = MAXIMUM_RUN_TIME;
        this.originalLines = originalLines;
        this.modifiedLines = modifiedLines;
        this.original = new LineMarkerSequence(originalLines);
        this.modified = new LineMarkerSequence(modifiedLines);
    }
    DiffComputer.prototype.computeDiff = function () {
        if (this.original.getLength() === 1 && this.original.getElementAtIndex(0).length === 0) {
            // empty original => fast path
            return [{
                    originalStartLineNumber: 1,
                    originalEndLineNumber: 1,
                    modifiedStartLineNumber: 1,
                    modifiedEndLineNumber: this.modified.getLength(),
                    charChanges: [{
                            modifiedEndColumn: 0,
                            modifiedEndLineNumber: 0,
                            modifiedStartColumn: 0,
                            modifiedStartLineNumber: 0,
                            originalEndColumn: 0,
                            originalEndLineNumber: 0,
                            originalStartColumn: 0,
                            originalStartLineNumber: 0
                        }]
                }];
        }
        if (this.modified.getLength() === 1 && this.modified.getElementAtIndex(0).length === 0) {
            // empty modified => fast path
            return [{
                    originalStartLineNumber: 1,
                    originalEndLineNumber: this.original.getLength(),
                    modifiedStartLineNumber: 1,
                    modifiedEndLineNumber: 1,
                    charChanges: [{
                            modifiedEndColumn: 0,
                            modifiedEndLineNumber: 0,
                            modifiedStartColumn: 0,
                            modifiedStartLineNumber: 0,
                            originalEndColumn: 0,
                            originalEndLineNumber: 0,
                            originalStartColumn: 0,
                            originalStartLineNumber: 0
                        }]
                }];
        }
        this.computationStartTime = (new Date()).getTime();
        var rawChanges = computeDiff(this.original, this.modified, this._continueProcessingPredicate.bind(this), this.shouldMakePrettyDiff);
        // The diff is always computed with ignoring trim whitespace
        // This ensures we get the prettiest diff
        if (this.shouldIgnoreTrimWhitespace) {
            var lineChanges = [];
            for (var i = 0, length_3 = rawChanges.length; i < length_3; i++) {
                lineChanges.push(LineChange.createFromDiffResult(this.shouldIgnoreTrimWhitespace, rawChanges[i], this.original, this.modified, this._continueProcessingPredicate.bind(this), this.shouldComputeCharChanges, this.shouldPostProcessCharChanges));
            }
            return lineChanges;
        }
        // Need to post-process and introduce changes where the trim whitespace is different
        // Note that we are looping starting at -1 to also cover the lines before the first change
        var result = [];
        var originalLineIndex = 0;
        var modifiedLineIndex = 0;
        for (var i = -1 /* !!!! */, len = rawChanges.length; i < len; i++) {
            var nextChange = (i + 1 < len ? rawChanges[i + 1] : null);
            var originalStop = (nextChange ? nextChange.originalStart : this.originalLines.length);
            var modifiedStop = (nextChange ? nextChange.modifiedStart : this.modifiedLines.length);
            while (originalLineIndex < originalStop && modifiedLineIndex < modifiedStop) {
                var originalLine = this.originalLines[originalLineIndex];
                var modifiedLine = this.modifiedLines[modifiedLineIndex];
                if (originalLine !== modifiedLine) {
                    // These lines differ only in trim whitespace
                    // Check the leading whitespace
                    {
                        var originalStartColumn = LineMarkerSequence._getFirstNonBlankColumn(originalLine, 1);
                        var modifiedStartColumn = LineMarkerSequence._getFirstNonBlankColumn(modifiedLine, 1);
                        while (originalStartColumn > 1 && modifiedStartColumn > 1) {
                            var originalChar = originalLine.charCodeAt(originalStartColumn - 2);
                            var modifiedChar = modifiedLine.charCodeAt(modifiedStartColumn - 2);
                            if (originalChar !== modifiedChar) {
                                break;
                            }
                            originalStartColumn--;
                            modifiedStartColumn--;
                        }
                        if (originalStartColumn > 1 || modifiedStartColumn > 1) {
                            this._pushTrimWhitespaceCharChange(result, originalLineIndex + 1, 1, originalStartColumn, modifiedLineIndex + 1, 1, modifiedStartColumn);
                        }
                    }
                    // Check the trailing whitespace
                    {
                        var originalEndColumn = LineMarkerSequence._getLastNonBlankColumn(originalLine, 1);
                        var modifiedEndColumn = LineMarkerSequence._getLastNonBlankColumn(modifiedLine, 1);
                        var originalMaxColumn = originalLine.length + 1;
                        var modifiedMaxColumn = modifiedLine.length + 1;
                        while (originalEndColumn < originalMaxColumn && modifiedEndColumn < modifiedMaxColumn) {
                            var originalChar = originalLine.charCodeAt(originalEndColumn - 1);
                            var modifiedChar = originalLine.charCodeAt(modifiedEndColumn - 1);
                            if (originalChar !== modifiedChar) {
                                break;
                            }
                            originalEndColumn++;
                            modifiedEndColumn++;
                        }
                        if (originalEndColumn < originalMaxColumn || modifiedEndColumn < modifiedMaxColumn) {
                            this._pushTrimWhitespaceCharChange(result, originalLineIndex + 1, originalEndColumn, originalMaxColumn, modifiedLineIndex + 1, modifiedEndColumn, modifiedMaxColumn);
                        }
                    }
                }
                originalLineIndex++;
                modifiedLineIndex++;
            }
            if (nextChange) {
                // Emit the actual change
                result.push(LineChange.createFromDiffResult(this.shouldIgnoreTrimWhitespace, nextChange, this.original, this.modified, this._continueProcessingPredicate.bind(this), this.shouldComputeCharChanges, this.shouldPostProcessCharChanges));
                originalLineIndex += nextChange.originalLength;
                modifiedLineIndex += nextChange.modifiedLength;
            }
        }
        return result;
    };
    DiffComputer.prototype._pushTrimWhitespaceCharChange = function (result, originalLineNumber, originalStartColumn, originalEndColumn, modifiedLineNumber, modifiedStartColumn, modifiedEndColumn) {
        if (this._mergeTrimWhitespaceCharChange(result, originalLineNumber, originalStartColumn, originalEndColumn, modifiedLineNumber, modifiedStartColumn, modifiedEndColumn)) {
            // Merged into previous
            return;
        }
        var charChanges = undefined;
        if (this.shouldComputeCharChanges) {
            charChanges = [new CharChange(originalLineNumber, originalStartColumn, originalLineNumber, originalEndColumn, modifiedLineNumber, modifiedStartColumn, modifiedLineNumber, modifiedEndColumn)];
        }
        result.push(new LineChange(originalLineNumber, originalLineNumber, modifiedLineNumber, modifiedLineNumber, charChanges));
    };
    DiffComputer.prototype._mergeTrimWhitespaceCharChange = function (result, originalLineNumber, originalStartColumn, originalEndColumn, modifiedLineNumber, modifiedStartColumn, modifiedEndColumn) {
        var len = result.length;
        if (len === 0) {
            return false;
        }
        var prevChange = result[len - 1];
        if (prevChange.originalEndLineNumber === 0 || prevChange.modifiedEndLineNumber === 0) {
            // Don't merge with inserts/deletes
            return false;
        }
        if (prevChange.originalEndLineNumber + 1 === originalLineNumber && prevChange.modifiedEndLineNumber + 1 === modifiedLineNumber) {
            prevChange.originalEndLineNumber = originalLineNumber;
            prevChange.modifiedEndLineNumber = modifiedLineNumber;
            if (this.shouldComputeCharChanges) {
                prevChange.charChanges.push(new CharChange(originalLineNumber, originalStartColumn, originalLineNumber, originalEndColumn, modifiedLineNumber, modifiedStartColumn, modifiedLineNumber, modifiedEndColumn));
            }
            return true;
        }
        return false;
    };
    DiffComputer.prototype._continueProcessingPredicate = function () {
        if (this.maximumRunTimeMs === 0) {
            return true;
        }
        var now = (new Date()).getTime();
        return now - this.computationStartTime < this.maximumRunTimeMs;
    };
    return DiffComputer;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MirrorTextModel = /** @class */ (function () {
    function MirrorTextModel(uri, lines, eol, versionId) {
        this._uri = uri;
        this._lines = lines;
        this._eol = eol;
        this._versionId = versionId;
        this._lineStarts = null;
    }
    MirrorTextModel.prototype.dispose = function () {
        this._lines.length = 0;
    };
    MirrorTextModel.prototype.getText = function () {
        return this._lines.join(this._eol);
    };
    MirrorTextModel.prototype.onEvents = function (e) {
        if (e.eol && e.eol !== this._eol) {
            this._eol = e.eol;
            this._lineStarts = null;
        }
        // Update my lines
        var changes = e.changes;
        for (var i = 0, len = changes.length; i < len; i++) {
            var change = changes[i];
            this._acceptDeleteRange(change.range);
            this._acceptInsertText(new Position(change.range.startLineNumber, change.range.startColumn), change.text);
        }
        this._versionId = e.versionId;
    };
    MirrorTextModel.prototype._ensureLineStarts = function () {
        if (!this._lineStarts) {
            var eolLength = this._eol.length;
            var linesLength = this._lines.length;
            var lineStartValues = new Uint32Array(linesLength);
            for (var i = 0; i < linesLength; i++) {
                lineStartValues[i] = this._lines[i].length + eolLength;
            }
            this._lineStarts = new PrefixSumComputer(lineStartValues);
        }
    };
    /**
     * All changes to a line's text go through this method
     */
    MirrorTextModel.prototype._setLineText = function (lineIndex, newValue) {
        this._lines[lineIndex] = newValue;
        if (this._lineStarts) {
            // update prefix sum
            this._lineStarts.changeValue(lineIndex, this._lines[lineIndex].length + this._eol.length);
        }
    };
    MirrorTextModel.prototype._acceptDeleteRange = function (range) {
        if (range.startLineNumber === range.endLineNumber) {
            if (range.startColumn === range.endColumn) {
                // Nothing to delete
                return;
            }
            // Delete text on the affected line
            this._setLineText(range.startLineNumber - 1, this._lines[range.startLineNumber - 1].substring(0, range.startColumn - 1)
                + this._lines[range.startLineNumber - 1].substring(range.endColumn - 1));
            return;
        }
        // Take remaining text on last line and append it to remaining text on first line
        this._setLineText(range.startLineNumber - 1, this._lines[range.startLineNumber - 1].substring(0, range.startColumn - 1)
            + this._lines[range.endLineNumber - 1].substring(range.endColumn - 1));
        // Delete middle lines
        this._lines.splice(range.startLineNumber, range.endLineNumber - range.startLineNumber);
        if (this._lineStarts) {
            // update prefix sum
            this._lineStarts.removeValues(range.startLineNumber, range.endLineNumber - range.startLineNumber);
        }
    };
    MirrorTextModel.prototype._acceptInsertText = function (position, insertText) {
        if (insertText.length === 0) {
            // Nothing to insert
            return;
        }
        var insertLines = insertText.split(/\r\n|\r|\n/);
        if (insertLines.length === 1) {
            // Inserting text on one line
            this._setLineText(position.lineNumber - 1, this._lines[position.lineNumber - 1].substring(0, position.column - 1)
                + insertLines[0]
                + this._lines[position.lineNumber - 1].substring(position.column - 1));
            return;
        }
        // Append overflowing text from first line to the end of text to insert
        insertLines[insertLines.length - 1] += this._lines[position.lineNumber - 1].substring(position.column - 1);
        // Delete overflowing text from first line and insert text on first line
        this._setLineText(position.lineNumber - 1, this._lines[position.lineNumber - 1].substring(0, position.column - 1)
            + insertLines[0]);
        // Insert new lines & store lengths
        var newLengths = new Uint32Array(insertLines.length - 1);
        for (var i = 1; i < insertLines.length; i++) {
            this._lines.splice(position.lineNumber + i - 1, 0, insertLines[i]);
            newLengths[i - 1] = insertLines[i].length + this._eol.length;
        }
        if (this._lineStarts) {
            // update prefix sum
            this._lineStarts.insertValues(position.lineNumber, newLengths);
        }
    };
    return MirrorTextModel;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var StateMachine = /** @class */ (function () {
    function StateMachine(edges) {
        var maxCharCode = 0;
        var maxState = 0 /* Invalid */;
        for (var i = 0, len = edges.length; i < len; i++) {
            var _a = edges[i], from = _a[0], chCode = _a[1], to = _a[2];
            if (chCode > maxCharCode) {
                maxCharCode = chCode;
            }
            if (from > maxState) {
                maxState = from;
            }
            if (to > maxState) {
                maxState = to;
            }
        }
        maxCharCode++;
        maxState++;
        var states = new Uint8Matrix(maxState, maxCharCode, 0 /* Invalid */);
        for (var i = 0, len = edges.length; i < len; i++) {
            var _b = edges[i], from = _b[0], chCode = _b[1], to = _b[2];
            states.set(from, chCode, to);
        }
        this._states = states;
        this._maxCharCode = maxCharCode;
    }
    StateMachine.prototype.nextState = function (currentState, chCode) {
        if (chCode < 0 || chCode >= this._maxCharCode) {
            return 0 /* Invalid */;
        }
        return this._states.get(currentState, chCode);
    };
    return StateMachine;
}());
// State machine for http:// or https:// or file://
var _stateMachine = null;
function getStateMachine() {
    if (_stateMachine === null) {
        _stateMachine = new StateMachine([
            [1 /* Start */, 104 /* h */, 2 /* H */],
            [1 /* Start */, 72 /* H */, 2 /* H */],
            [1 /* Start */, 102 /* f */, 6 /* F */],
            [1 /* Start */, 70 /* F */, 6 /* F */],
            [2 /* H */, 116 /* t */, 3 /* HT */],
            [2 /* H */, 84 /* T */, 3 /* HT */],
            [3 /* HT */, 116 /* t */, 4 /* HTT */],
            [3 /* HT */, 84 /* T */, 4 /* HTT */],
            [4 /* HTT */, 112 /* p */, 5 /* HTTP */],
            [4 /* HTT */, 80 /* P */, 5 /* HTTP */],
            [5 /* HTTP */, 115 /* s */, 9 /* BeforeColon */],
            [5 /* HTTP */, 83 /* S */, 9 /* BeforeColon */],
            [5 /* HTTP */, 58 /* Colon */, 10 /* AfterColon */],
            [6 /* F */, 105 /* i */, 7 /* FI */],
            [6 /* F */, 73 /* I */, 7 /* FI */],
            [7 /* FI */, 108 /* l */, 8 /* FIL */],
            [7 /* FI */, 76 /* L */, 8 /* FIL */],
            [8 /* FIL */, 101 /* e */, 9 /* BeforeColon */],
            [8 /* FIL */, 69 /* E */, 9 /* BeforeColon */],
            [9 /* BeforeColon */, 58 /* Colon */, 10 /* AfterColon */],
            [10 /* AfterColon */, 47 /* Slash */, 11 /* AlmostThere */],
            [11 /* AlmostThere */, 47 /* Slash */, 12 /* End */],
        ]);
    }
    return _stateMachine;
}
var _classifier = null;
function getClassifier() {
    if (_classifier === null) {
        _classifier = new CharacterClassifier(0 /* None */);
        var FORCE_TERMINATION_CHARACTERS = ' \t<>\'\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…';
        for (var i = 0; i < FORCE_TERMINATION_CHARACTERS.length; i++) {
            _classifier.set(FORCE_TERMINATION_CHARACTERS.charCodeAt(i), 1 /* ForceTermination */);
        }
        var CANNOT_END_WITH_CHARACTERS = '.,;';
        for (var i = 0; i < CANNOT_END_WITH_CHARACTERS.length; i++) {
            _classifier.set(CANNOT_END_WITH_CHARACTERS.charCodeAt(i), 2 /* CannotEndIn */);
        }
    }
    return _classifier;
}
var LinkComputer = /** @class */ (function () {
    function LinkComputer() {
    }
    LinkComputer._createLink = function (classifier, line, lineNumber, linkBeginIndex, linkEndIndex) {
        // Do not allow to end link in certain characters...
        var lastIncludedCharIndex = linkEndIndex - 1;
        do {
            var chCode = line.charCodeAt(lastIncludedCharIndex);
            var chClass = classifier.get(chCode);
            if (chClass !== 2 /* CannotEndIn */) {
                break;
            }
            lastIncludedCharIndex--;
        } while (lastIncludedCharIndex > linkBeginIndex);
        // Handle links enclosed in parens, square brackets and curlys.
        if (linkBeginIndex > 0) {
            var charCodeBeforeLink = line.charCodeAt(linkBeginIndex - 1);
            var lastCharCodeInLink = line.charCodeAt(lastIncludedCharIndex);
            if ((charCodeBeforeLink === 40 /* OpenParen */ && lastCharCodeInLink === 41 /* CloseParen */)
                || (charCodeBeforeLink === 91 /* OpenSquareBracket */ && lastCharCodeInLink === 93 /* CloseSquareBracket */)
                || (charCodeBeforeLink === 123 /* OpenCurlyBrace */ && lastCharCodeInLink === 125 /* CloseCurlyBrace */)) {
                // Do not end in ) if ( is before the link start
                // Do not end in ] if [ is before the link start
                // Do not end in } if { is before the link start
                lastIncludedCharIndex--;
            }
        }
        return {
            range: {
                startLineNumber: lineNumber,
                startColumn: linkBeginIndex + 1,
                endLineNumber: lineNumber,
                endColumn: lastIncludedCharIndex + 2
            },
            url: line.substring(linkBeginIndex, lastIncludedCharIndex + 1)
        };
    };
    LinkComputer.computeLinks = function (model) {
        var stateMachine = getStateMachine();
        var classifier = getClassifier();
        var result = [];
        for (var i = 1, lineCount = model.getLineCount(); i <= lineCount; i++) {
            var line = model.getLineContent(i);
            var len = line.length;
            var j = 0;
            var linkBeginIndex = 0;
            var linkBeginChCode = 0;
            var state = 1 /* Start */;
            var hasOpenParens = false;
            var hasOpenSquareBracket = false;
            var hasOpenCurlyBracket = false;
            while (j < len) {
                var resetStateMachine = false;
                var chCode = line.charCodeAt(j);
                if (state === 13 /* Accept */) {
                    var chClass = void 0;
                    switch (chCode) {
                        case 40 /* OpenParen */:
                            hasOpenParens = true;
                            chClass = 0 /* None */;
                            break;
                        case 41 /* CloseParen */:
                            chClass = (hasOpenParens ? 0 /* None */ : 1 /* ForceTermination */);
                            break;
                        case 91 /* OpenSquareBracket */:
                            hasOpenSquareBracket = true;
                            chClass = 0 /* None */;
                            break;
                        case 93 /* CloseSquareBracket */:
                            chClass = (hasOpenSquareBracket ? 0 /* None */ : 1 /* ForceTermination */);
                            break;
                        case 123 /* OpenCurlyBrace */:
                            hasOpenCurlyBracket = true;
                            chClass = 0 /* None */;
                            break;
                        case 125 /* CloseCurlyBrace */:
                            chClass = (hasOpenCurlyBracket ? 0 /* None */ : 1 /* ForceTermination */);
                            break;
                        /* The following three rules make it that ' or " or ` are allowed inside links if the link began with a different one */
                        case 39 /* SingleQuote */:
                            chClass = (linkBeginChCode === 34 /* DoubleQuote */ || linkBeginChCode === 96 /* BackTick */) ? 0 /* None */ : 1 /* ForceTermination */;
                            break;
                        case 34 /* DoubleQuote */:
                            chClass = (linkBeginChCode === 39 /* SingleQuote */ || linkBeginChCode === 96 /* BackTick */) ? 0 /* None */ : 1 /* ForceTermination */;
                            break;
                        case 96 /* BackTick */:
                            chClass = (linkBeginChCode === 39 /* SingleQuote */ || linkBeginChCode === 34 /* DoubleQuote */) ? 0 /* None */ : 1 /* ForceTermination */;
                            break;
                        default:
                            chClass = classifier.get(chCode);
                    }
                    // Check if character terminates link
                    if (chClass === 1 /* ForceTermination */) {
                        result.push(LinkComputer._createLink(classifier, line, i, linkBeginIndex, j));
                        resetStateMachine = true;
                    }
                }
                else if (state === 12 /* End */) {
                    var chClass = classifier.get(chCode);
                    // Check if character terminates link
                    if (chClass === 1 /* ForceTermination */) {
                        resetStateMachine = true;
                    }
                    else {
                        state = 13 /* Accept */;
                    }
                }
                else {
                    state = stateMachine.nextState(state, chCode);
                    if (state === 0 /* Invalid */) {
                        resetStateMachine = true;
                    }
                }
                if (resetStateMachine) {
                    state = 1 /* Start */;
                    hasOpenParens = false;
                    hasOpenSquareBracket = false;
                    hasOpenCurlyBracket = false;
                    // Record where the link started
                    linkBeginIndex = j + 1;
                    linkBeginChCode = chCode;
                }
                j++;
            }
            if (state === 13 /* Accept */) {
                result.push(LinkComputer._createLink(classifier, line, i, linkBeginIndex, len));
            }
        }
        return result;
    };
    return LinkComputer;
}());
/**
 * Returns an array of all links contains in the provided
 * document. *Note* that this operation is computational
 * expensive and should not run in the UI thread.
 */
function computeLinks(model) {
    if (!model || typeof model.getLineCount !== 'function' || typeof model.getLineContent !== 'function') {
        // Unknown caller!
        return [];
    }
    return LinkComputer.computeLinks(model);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var BasicInplaceReplace = /** @class */ (function () {
    function BasicInplaceReplace() {
        this._defaultValueSet = [
            ['true', 'false'],
            ['True', 'False'],
            ['Private', 'Public', 'Friend', 'ReadOnly', 'Partial', 'Protected', 'WriteOnly'],
            ['public', 'protected', 'private'],
        ];
    }
    BasicInplaceReplace.prototype.navigateValueSet = function (range1, text1, range2, text2, up) {
        if (range1 && text1) {
            var result = this.doNavigateValueSet(text1, up);
            if (result) {
                return {
                    range: range1,
                    value: result
                };
            }
        }
        if (range2 && text2) {
            var result = this.doNavigateValueSet(text2, up);
            if (result) {
                return {
                    range: range2,
                    value: result
                };
            }
        }
        return null;
    };
    BasicInplaceReplace.prototype.doNavigateValueSet = function (text, up) {
        var numberResult = this.numberReplace(text, up);
        if (numberResult !== null) {
            return numberResult;
        }
        return this.textReplace(text, up);
    };
    BasicInplaceReplace.prototype.numberReplace = function (value, up) {
        var precision = Math.pow(10, value.length - (value.lastIndexOf('.') + 1));
        var n1 = Number(value);
        var n2 = parseFloat(value);
        if (!isNaN(n1) && !isNaN(n2) && n1 === n2) {
            if (n1 === 0 && !up) {
                return null; // don't do negative
                //			} else if(n1 === 9 && up) {
                //				return null; // don't insert 10 into a number
            }
            else {
                n1 = Math.floor(n1 * precision);
                n1 += up ? precision : -precision;
                return String(n1 / precision);
            }
        }
        return null;
    };
    BasicInplaceReplace.prototype.textReplace = function (value, up) {
        return this.valueSetsReplace(this._defaultValueSet, value, up);
    };
    BasicInplaceReplace.prototype.valueSetsReplace = function (valueSets, value, up) {
        var result = null;
        for (var i = 0, len = valueSets.length; result === null && i < len; i++) {
            result = this.valueSetReplace(valueSets[i], value, up);
        }
        return result;
    };
    BasicInplaceReplace.prototype.valueSetReplace = function (valueSet, value, up) {
        var idx = valueSet.indexOf(value);
        if (idx >= 0) {
            idx += up ? +1 : -1;
            if (idx < 0) {
                idx = valueSet.length - 1;
            }
            else {
                idx %= valueSet.length;
            }
            return valueSet[idx];
        }
        return null;
    };
    BasicInplaceReplace.INSTANCE = new BasicInplaceReplace();
    return BasicInplaceReplace;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1Z = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * @internal
 */
var MirrorModel = /** @class */ (function (_super) {
    __extends$1Z(MirrorModel, _super);
    function MirrorModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(MirrorModel.prototype, "uri", {
        get: function () {
            return this._uri;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MirrorModel.prototype, "version", {
        get: function () {
            return this._versionId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MirrorModel.prototype, "eol", {
        get: function () {
            return this._eol;
        },
        enumerable: true,
        configurable: true
    });
    MirrorModel.prototype.getValue = function () {
        return this.getText();
    };
    MirrorModel.prototype.getLinesContent = function () {
        return this._lines.slice(0);
    };
    MirrorModel.prototype.getLineCount = function () {
        return this._lines.length;
    };
    MirrorModel.prototype.getLineContent = function (lineNumber) {
        return this._lines[lineNumber - 1];
    };
    MirrorModel.prototype.getWordAtPosition = function (position, wordDefinition) {
        var wordAtText = getWordAtText(position.column, ensureValidWordDefinition(wordDefinition), this._lines[position.lineNumber - 1], 0);
        if (wordAtText) {
            return new Range(position.lineNumber, wordAtText.startColumn, position.lineNumber, wordAtText.endColumn);
        }
        return null;
    };
    MirrorModel.prototype.getWordUntilPosition = function (position, wordDefinition) {
        var wordAtPosition = this.getWordAtPosition(position, wordDefinition);
        if (!wordAtPosition) {
            return {
                word: '',
                startColumn: position.column,
                endColumn: position.column
            };
        }
        return {
            word: this._lines[position.lineNumber - 1].substring(wordAtPosition.startColumn - 1, position.column - 1),
            startColumn: wordAtPosition.startColumn,
            endColumn: position.column
        };
    };
    MirrorModel.prototype.createWordIterator = function (wordDefinition) {
        var _this = this;
        var obj;
        var lineNumber = 0;
        var lineText;
        var wordRangesIdx = 0;
        var wordRanges = [];
        var next = function () {
            if (wordRangesIdx < wordRanges.length) {
                var value = lineText.substring(wordRanges[wordRangesIdx].start, wordRanges[wordRangesIdx].end);
                wordRangesIdx += 1;
                if (!obj) {
                    obj = { done: false, value: value };
                }
                else {
                    obj.value = value;
                }
                return obj;
            }
            else if (lineNumber >= _this._lines.length) {
                return FIN;
            }
            else {
                lineText = _this._lines[lineNumber];
                wordRanges = _this._wordenize(lineText, wordDefinition);
                wordRangesIdx = 0;
                lineNumber += 1;
                return next();
            }
        };
        return { next: next };
    };
    MirrorModel.prototype.getLineWords = function (lineNumber, wordDefinition) {
        var content = this._lines[lineNumber - 1];
        var ranges = this._wordenize(content, wordDefinition);
        var words = [];
        for (var _i = 0, ranges_1 = ranges; _i < ranges_1.length; _i++) {
            var range$$1 = ranges_1[_i];
            words.push({
                word: content.substring(range$$1.start, range$$1.end),
                startColumn: range$$1.start + 1,
                endColumn: range$$1.end + 1
            });
        }
        return words;
    };
    MirrorModel.prototype._wordenize = function (content, wordDefinition) {
        var result = [];
        var match;
        wordDefinition.lastIndex = 0; // reset lastIndex just to be sure
        while (match = wordDefinition.exec(content)) {
            if (match[0].length === 0) {
                // it did match the empty string
                break;
            }
            result.push({ start: match.index, end: match.index + match[0].length });
        }
        return result;
    };
    MirrorModel.prototype.getValueInRange = function (range$$1) {
        range$$1 = this._validateRange(range$$1);
        if (range$$1.startLineNumber === range$$1.endLineNumber) {
            return this._lines[range$$1.startLineNumber - 1].substring(range$$1.startColumn - 1, range$$1.endColumn - 1);
        }
        var lineEnding = this._eol;
        var startLineIndex = range$$1.startLineNumber - 1;
        var endLineIndex = range$$1.endLineNumber - 1;
        var resultLines = [];
        resultLines.push(this._lines[startLineIndex].substring(range$$1.startColumn - 1));
        for (var i = startLineIndex + 1; i < endLineIndex; i++) {
            resultLines.push(this._lines[i]);
        }
        resultLines.push(this._lines[endLineIndex].substring(0, range$$1.endColumn - 1));
        return resultLines.join(lineEnding);
    };
    MirrorModel.prototype.offsetAt = function (position) {
        position = this._validatePosition(position);
        this._ensureLineStarts();
        return this._lineStarts.getAccumulatedValue(position.lineNumber - 2) + (position.column - 1);
    };
    MirrorModel.prototype.positionAt = function (offset) {
        offset = Math.floor(offset);
        offset = Math.max(0, offset);
        this._ensureLineStarts();
        var out = this._lineStarts.getIndexOf(offset);
        var lineLength = this._lines[out.index].length;
        // Ensure we return a valid position
        return {
            lineNumber: 1 + out.index,
            column: 1 + Math.min(out.remainder, lineLength)
        };
    };
    MirrorModel.prototype._validateRange = function (range$$1) {
        var start = this._validatePosition({ lineNumber: range$$1.startLineNumber, column: range$$1.startColumn });
        var end = this._validatePosition({ lineNumber: range$$1.endLineNumber, column: range$$1.endColumn });
        if (start.lineNumber !== range$$1.startLineNumber
            || start.column !== range$$1.startColumn
            || end.lineNumber !== range$$1.endLineNumber
            || end.column !== range$$1.endColumn) {
            return {
                startLineNumber: start.lineNumber,
                startColumn: start.column,
                endLineNumber: end.lineNumber,
                endColumn: end.column
            };
        }
        return range$$1;
    };
    MirrorModel.prototype._validatePosition = function (position) {
        if (!Position.isIPosition(position)) {
            throw new Error('bad position');
        }
        var lineNumber = position.lineNumber, column = position.column;
        var hasChanged = false;
        if (lineNumber < 1) {
            lineNumber = 1;
            column = 1;
            hasChanged = true;
        }
        else if (lineNumber > this._lines.length) {
            lineNumber = this._lines.length;
            column = this._lines[lineNumber - 1].length + 1;
            hasChanged = true;
        }
        else {
            var maxCharacter = this._lines[lineNumber - 1].length + 1;
            if (column < 1) {
                column = 1;
                hasChanged = true;
            }
            else if (column > maxCharacter) {
                column = maxCharacter;
                hasChanged = true;
            }
        }
        if (!hasChanged) {
            return position;
        }
        else {
            return { lineNumber: lineNumber, column: column };
        }
    };
    return MirrorModel;
}(MirrorTextModel));
/**
 * @internal
 */
var BaseEditorSimpleWorker = /** @class */ (function () {
    function BaseEditorSimpleWorker(foreignModuleFactory) {
        this._foreignModuleFactory = foreignModuleFactory;
        this._foreignModule = null;
    }
    // ---- BEGIN diff --------------------------------------------------------------------------
    BaseEditorSimpleWorker.prototype.computeDiff = function (originalUrl, modifiedUrl, ignoreTrimWhitespace) {
        var original = this._getModel(originalUrl);
        var modified = this._getModel(modifiedUrl);
        if (!original || !modified) {
            return Promise.resolve(null);
        }
        var originalLines = original.getLinesContent();
        var modifiedLines = modified.getLinesContent();
        var diffComputer = new DiffComputer(originalLines, modifiedLines, {
            shouldComputeCharChanges: true,
            shouldPostProcessCharChanges: true,
            shouldIgnoreTrimWhitespace: ignoreTrimWhitespace,
            shouldMakePrettyDiff: true
        });
        var changes = diffComputer.computeDiff();
        var identical = (changes.length > 0 ? false : this._modelsAreIdentical(original, modified));
        return Promise.resolve({
            identical: identical,
            changes: changes
        });
    };
    BaseEditorSimpleWorker.prototype._modelsAreIdentical = function (original, modified) {
        var originalLineCount = original.getLineCount();
        var modifiedLineCount = modified.getLineCount();
        if (originalLineCount !== modifiedLineCount) {
            return false;
        }
        for (var line = 1; line <= originalLineCount; line++) {
            var originalLine = original.getLineContent(line);
            var modifiedLine = modified.getLineContent(line);
            if (originalLine !== modifiedLine) {
                return false;
            }
        }
        return true;
    };
    BaseEditorSimpleWorker.prototype.computeMoreMinimalEdits = function (modelUrl, edits) {
        var model = this._getModel(modelUrl);
        if (!model) {
            return Promise.resolve(edits);
        }
        var result = [];
        var lastEol = undefined;
        edits = mergeSort(edits, function (a, b) {
            if (a.range && b.range) {
                return Range.compareRangesUsingStarts(a.range, b.range);
            }
            // eol only changes should go to the end
            var aRng = a.range ? 0 : 1;
            var bRng = b.range ? 0 : 1;
            return aRng - bRng;
        });
        for (var _i = 0, edits_1 = edits; _i < edits_1.length; _i++) {
            var _a = edits_1[_i], range$$1 = _a.range, text = _a.text, eol = _a.eol;
            if (typeof eol === 'number') {
                lastEol = eol;
            }
            if (!range$$1) {
                // eol-change only
                continue;
            }
            var original = model.getValueInRange(range$$1);
            text = text.replace(/\r\n|\n|\r/g, model.eol);
            if (original === text) {
                // noop
                continue;
            }
            // make sure diff won't take too long
            if (Math.max(text.length, original.length) > BaseEditorSimpleWorker._diffLimit) {
                result.push({ range: range$$1, text: text });
                continue;
            }
            // compute diff between original and edit.text
            var changes = stringDiff(original, text, false);
            var editOffset = model.offsetAt(Range.lift(range$$1).getStartPosition());
            for (var _b = 0, changes_1 = changes; _b < changes_1.length; _b++) {
                var change = changes_1[_b];
                var start = model.positionAt(editOffset + change.originalStart);
                var end = model.positionAt(editOffset + change.originalStart + change.originalLength);
                var newEdit = {
                    text: text.substr(change.modifiedStart, change.modifiedLength),
                    range: { startLineNumber: start.lineNumber, startColumn: start.column, endLineNumber: end.lineNumber, endColumn: end.column }
                };
                if (model.getValueInRange(newEdit.range) !== newEdit.text) {
                    result.push(newEdit);
                }
            }
        }
        if (typeof lastEol === 'number') {
            result.push({ eol: lastEol, text: undefined, range: undefined });
        }
        return Promise.resolve(result);
    };
    // ---- END minimal edits ---------------------------------------------------------------
    BaseEditorSimpleWorker.prototype.computeLinks = function (modelUrl) {
        var model = this._getModel(modelUrl);
        if (!model) {
            return Promise.resolve(null);
        }
        return Promise.resolve(computeLinks(model));
    };
    BaseEditorSimpleWorker.prototype.textualSuggest = function (modelUrl, position, wordDef, wordDefFlags) {
        var model = this._getModel(modelUrl);
        if (!model) {
            return Promise.resolve(null);
        }
        var suggestions = [];
        var wordDefRegExp = new RegExp(wordDef, wordDefFlags);
        var currentWord = model.getWordUntilPosition(position, wordDefRegExp);
        var seen = Object.create(null);
        seen[currentWord.word] = true;
        for (var iter = model.createWordIterator(wordDefRegExp), e = iter.next(); !e.done && suggestions.length <= BaseEditorSimpleWorker._suggestionsLimit; e = iter.next()) {
            var word = e.value;
            if (seen[word]) {
                continue;
            }
            seen[word] = true;
            if (!isNaN(Number(word))) {
                continue;
            }
            suggestions.push({
                kind: 18 /* Text */,
                label: word,
                insertText: word,
                range: { startLineNumber: position.lineNumber, startColumn: currentWord.startColumn, endLineNumber: position.lineNumber, endColumn: currentWord.endColumn }
            });
        }
        return Promise.resolve({ suggestions: suggestions });
    };
    // ---- END suggest --------------------------------------------------------------------------
    //#region -- word ranges --
    BaseEditorSimpleWorker.prototype.computeWordRanges = function (modelUrl, range$$1, wordDef, wordDefFlags) {
        var model = this._getModel(modelUrl);
        if (!model) {
            return Promise.resolve(Object.create(null));
        }
        var wordDefRegExp = new RegExp(wordDef, wordDefFlags);
        var result = Object.create(null);
        for (var line = range$$1.startLineNumber; line < range$$1.endLineNumber; line++) {
            var words = model.getLineWords(line, wordDefRegExp);
            for (var _i = 0, words_1 = words; _i < words_1.length; _i++) {
                var word = words_1[_i];
                if (!isNaN(Number(word.word))) {
                    continue;
                }
                var array = result[word.word];
                if (!array) {
                    array = [];
                    result[word.word] = array;
                }
                array.push({
                    startLineNumber: line,
                    startColumn: word.startColumn,
                    endLineNumber: line,
                    endColumn: word.endColumn
                });
            }
        }
        return Promise.resolve(result);
    };
    //#endregion
    BaseEditorSimpleWorker.prototype.navigateValueSet = function (modelUrl, range$$1, up, wordDef, wordDefFlags) {
        var model = this._getModel(modelUrl);
        if (!model) {
            return Promise.resolve(null);
        }
        var wordDefRegExp = new RegExp(wordDef, wordDefFlags);
        if (range$$1.startColumn === range$$1.endColumn) {
            range$$1 = {
                startLineNumber: range$$1.startLineNumber,
                startColumn: range$$1.startColumn,
                endLineNumber: range$$1.endLineNumber,
                endColumn: range$$1.endColumn + 1
            };
        }
        var selectionText = model.getValueInRange(range$$1);
        var wordRange = model.getWordAtPosition({ lineNumber: range$$1.startLineNumber, column: range$$1.startColumn }, wordDefRegExp);
        if (!wordRange) {
            return Promise.resolve(null);
        }
        var word = model.getValueInRange(wordRange);
        var result = BasicInplaceReplace.INSTANCE.navigateValueSet(range$$1, selectionText, wordRange, word, up);
        return Promise.resolve(result);
    };
    // ---- BEGIN foreign module support --------------------------------------------------------------------------
    BaseEditorSimpleWorker.prototype.loadForeignModule = function (moduleId, createData) {
        var _this = this;
        var ctx = {
            getMirrorModels: function () {
                return _this._getModels();
            }
        };
        if (this._foreignModuleFactory) {
            this._foreignModule = this._foreignModuleFactory(ctx, createData);
            // static foreing module
            var methods = [];
            for (var prop in this._foreignModule) {
                if (typeof this._foreignModule[prop] === 'function') {
                    methods.push(prop);
                }
            }
            return Promise.resolve(methods);
        }
        // ESM-comment-begin
        // 		return new Promise<any>((resolve, reject) => {
        // 			require([moduleId], (foreignModule: { create: IForeignModuleFactory }) => {
        // 				this._foreignModule = foreignModule.create(ctx, createData);
        // 
        // 				let methods: string[] = [];
        // 				for (let prop in this._foreignModule) {
        // 					if (typeof this._foreignModule[prop] === 'function') {
        // 						methods.push(prop);
        // 					}
        // 				}
        // 
        // 				resolve(methods);
        // 
        // 			}, reject);
        // 		});
        // ESM-comment-end
        // ESM-uncomment-begin
        return Promise.reject(new Error("Unexpected usage"));
        // ESM-uncomment-end
    };
    // foreign method request
    BaseEditorSimpleWorker.prototype.fmr = function (method, args) {
        if (!this._foreignModule || typeof this._foreignModule[method] !== 'function') {
            return Promise.reject(new Error('Missing requestHandler or method: ' + method));
        }
        try {
            return Promise.resolve(this._foreignModule[method].apply(this._foreignModule, args));
        }
        catch (e) {
            return Promise.reject(e);
        }
    };
    // ---- END diff --------------------------------------------------------------------------
    // ---- BEGIN minimal edits ---------------------------------------------------------------
    BaseEditorSimpleWorker._diffLimit = 10000;
    // ---- BEGIN suggest --------------------------------------------------------------------------
    BaseEditorSimpleWorker._suggestionsLimit = 10000;
    return BaseEditorSimpleWorker;
}());
/**
 * @internal
 */
var EditorSimpleWorkerImpl = /** @class */ (function (_super) {
    __extends$1Z(EditorSimpleWorkerImpl, _super);
    function EditorSimpleWorkerImpl(foreignModuleFactory) {
        var _this = _super.call(this, foreignModuleFactory) || this;
        _this._models = Object.create(null);
        return _this;
    }
    EditorSimpleWorkerImpl.prototype.dispose = function () {
        this._models = Object.create(null);
    };
    EditorSimpleWorkerImpl.prototype._getModel = function (uri) {
        return this._models[uri];
    };
    EditorSimpleWorkerImpl.prototype._getModels = function () {
        var _this = this;
        var all = [];
        Object.keys(this._models).forEach(function (key) { return all.push(_this._models[key]); });
        return all;
    };
    EditorSimpleWorkerImpl.prototype.acceptNewModel = function (data) {
        this._models[data.url] = new MirrorModel(URI.parse(data.url), data.lines, data.EOL, data.versionId);
    };
    EditorSimpleWorkerImpl.prototype.acceptModelChanged = function (strURL, e) {
        if (!this._models[strURL]) {
            return;
        }
        var model = this._models[strURL];
        model.onEvents(e);
    };
    EditorSimpleWorkerImpl.prototype.acceptRemovedModel = function (strURL) {
        if (!this._models[strURL]) {
            return;
        }
        delete this._models[strURL];
    };
    return EditorSimpleWorkerImpl;
}(BaseEditorSimpleWorker));
if (typeof importScripts === 'function') {
    // Running in a web worker
    globals.monaco = createMonacoBaseAPI();
}

var ITextResourceConfigurationService = createDecorator('textResourceConfigurationService');
var ITextResourcePropertiesService = createDecorator('textResourcePropertiesService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$1_ = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$s = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$p = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
/**
 * Stop syncing a model to the worker if it was not needed for 1 min.
 */
var STOP_SYNC_MODEL_DELTA_TIME_MS = 60 * 1000;
/**
 * Stop the worker if it was not needed for 5 min.
 */
var STOP_WORKER_DELTA_TIME_MS = 5 * 60 * 1000;
function canSyncModel(modelService, resource) {
    var model = modelService.getModel(resource);
    if (!model) {
        return false;
    }
    if (model.isTooLargeForSyncing()) {
        return false;
    }
    return true;
}
var EditorWorkerServiceImpl = /** @class */ (function (_super) {
    __extends$1_(EditorWorkerServiceImpl, _super);
    function EditorWorkerServiceImpl(modelService, configurationService) {
        var _this = _super.call(this) || this;
        _this._modelService = modelService;
        _this._workerManager = _this._register(new WorkerManager(_this._modelService));
        // todo@joh make sure this happens only once
        _this._register(LinkProviderRegistry.register('*', {
            provideLinks: function (model, token) {
                if (!canSyncModel(_this._modelService, model.uri)) {
                    return Promise.resolve([]); // File too large
                }
                return _this._workerManager.withWorker().then(function (client) { return client.computeLinks(model.uri); });
            }
        }));
        _this._register(CompletionProviderRegistry.register('*', new WordBasedCompletionItemProvider(_this._workerManager, configurationService, _this._modelService)));
        return _this;
    }
    EditorWorkerServiceImpl.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    EditorWorkerServiceImpl.prototype.canComputeDiff = function (original, modified) {
        return (canSyncModel(this._modelService, original) && canSyncModel(this._modelService, modified));
    };
    EditorWorkerServiceImpl.prototype.computeDiff = function (original, modified, ignoreTrimWhitespace) {
        return this._workerManager.withWorker().then(function (client) { return client.computeDiff(original, modified, ignoreTrimWhitespace); });
    };
    EditorWorkerServiceImpl.prototype.computeMoreMinimalEdits = function (resource, edits) {
        if (!Array.isArray(edits) || edits.length === 0) {
            return Promise.resolve(edits);
        }
        else {
            if (!canSyncModel(this._modelService, resource)) {
                return Promise.resolve(edits); // File too large
            }
            return this._workerManager.withWorker().then(function (client) { return client.computeMoreMinimalEdits(resource, edits); });
        }
    };
    EditorWorkerServiceImpl.prototype.canNavigateValueSet = function (resource) {
        return (canSyncModel(this._modelService, resource));
    };
    EditorWorkerServiceImpl.prototype.navigateValueSet = function (resource, range, up) {
        return this._workerManager.withWorker().then(function (client) { return client.navigateValueSet(resource, range, up); });
    };
    EditorWorkerServiceImpl.prototype.canComputeWordRanges = function (resource) {
        return canSyncModel(this._modelService, resource);
    };
    EditorWorkerServiceImpl.prototype.computeWordRanges = function (resource, range) {
        return this._workerManager.withWorker().then(function (client) { return client.computeWordRanges(resource, range); });
    };
    EditorWorkerServiceImpl = __decorate$s([
        __param$p(0, IModelService),
        __param$p(1, ITextResourceConfigurationService)
    ], EditorWorkerServiceImpl);
    return EditorWorkerServiceImpl;
}(Disposable));
var WordBasedCompletionItemProvider = /** @class */ (function () {
    function WordBasedCompletionItemProvider(workerManager, configurationService, modelService) {
        this._workerManager = workerManager;
        this._configurationService = configurationService;
        this._modelService = modelService;
    }
    WordBasedCompletionItemProvider.prototype.provideCompletionItems = function (model, position) {
        var wordBasedSuggestions = this._configurationService.getValue(model.uri, position, 'editor').wordBasedSuggestions;
        if (!wordBasedSuggestions) {
            return undefined;
        }
        if (!canSyncModel(this._modelService, model.uri)) {
            return undefined; // File too large
        }
        return this._workerManager.withWorker().then(function (client) { return client.textualSuggest(model.uri, position); });
    };
    return WordBasedCompletionItemProvider;
}());
var WorkerManager = /** @class */ (function (_super) {
    __extends$1_(WorkerManager, _super);
    function WorkerManager(modelService) {
        var _this = _super.call(this) || this;
        _this._modelService = modelService;
        _this._editorWorkerClient = null;
        var stopWorkerInterval = _this._register(new IntervalTimer());
        stopWorkerInterval.cancelAndSet(function () { return _this._checkStopIdleWorker(); }, Math.round(STOP_WORKER_DELTA_TIME_MS / 2));
        _this._register(_this._modelService.onModelRemoved(function (_) { return _this._checkStopEmptyWorker(); }));
        return _this;
    }
    WorkerManager.prototype.dispose = function () {
        if (this._editorWorkerClient) {
            this._editorWorkerClient.dispose();
            this._editorWorkerClient = null;
        }
        _super.prototype.dispose.call(this);
    };
    /**
     * Check if the model service has no more models and stop the worker if that is the case.
     */
    WorkerManager.prototype._checkStopEmptyWorker = function () {
        if (!this._editorWorkerClient) {
            return;
        }
        var models = this._modelService.getModels();
        if (models.length === 0) {
            // There are no more models => nothing possible for me to do
            this._editorWorkerClient.dispose();
            this._editorWorkerClient = null;
        }
    };
    /**
     * Check if the worker has been idle for a while and then stop it.
     */
    WorkerManager.prototype._checkStopIdleWorker = function () {
        if (!this._editorWorkerClient) {
            return;
        }
        var timeSinceLastWorkerUsedTime = (new Date()).getTime() - this._lastWorkerUsedTime;
        if (timeSinceLastWorkerUsedTime > STOP_WORKER_DELTA_TIME_MS) {
            this._editorWorkerClient.dispose();
            this._editorWorkerClient = null;
        }
    };
    WorkerManager.prototype.withWorker = function () {
        this._lastWorkerUsedTime = (new Date()).getTime();
        if (!this._editorWorkerClient) {
            this._editorWorkerClient = new EditorWorkerClient(this._modelService, 'editorWorkerService');
        }
        return Promise.resolve(this._editorWorkerClient);
    };
    return WorkerManager;
}(Disposable));
var EditorModelManager = /** @class */ (function (_super) {
    __extends$1_(EditorModelManager, _super);
    function EditorModelManager(proxy, modelService, keepIdleModels) {
        var _this = _super.call(this) || this;
        _this._syncedModels = Object.create(null);
        _this._syncedModelsLastUsedTime = Object.create(null);
        _this._proxy = proxy;
        _this._modelService = modelService;
        if (!keepIdleModels) {
            var timer = new IntervalTimer();
            timer.cancelAndSet(function () { return _this._checkStopModelSync(); }, Math.round(STOP_SYNC_MODEL_DELTA_TIME_MS / 2));
            _this._register(timer);
        }
        return _this;
    }
    EditorModelManager.prototype.dispose = function () {
        for (var modelUrl in this._syncedModels) {
            dispose(this._syncedModels[modelUrl]);
        }
        this._syncedModels = Object.create(null);
        this._syncedModelsLastUsedTime = Object.create(null);
        _super.prototype.dispose.call(this);
    };
    EditorModelManager.prototype.esureSyncedResources = function (resources) {
        for (var i = 0; i < resources.length; i++) {
            var resource = resources[i];
            var resourceStr = resource.toString();
            if (!this._syncedModels[resourceStr]) {
                this._beginModelSync(resource);
            }
            if (this._syncedModels[resourceStr]) {
                this._syncedModelsLastUsedTime[resourceStr] = (new Date()).getTime();
            }
        }
    };
    EditorModelManager.prototype._checkStopModelSync = function () {
        var currentTime = (new Date()).getTime();
        var toRemove = [];
        for (var modelUrl in this._syncedModelsLastUsedTime) {
            var elapsedTime = currentTime - this._syncedModelsLastUsedTime[modelUrl];
            if (elapsedTime > STOP_SYNC_MODEL_DELTA_TIME_MS) {
                toRemove.push(modelUrl);
            }
        }
        for (var i = 0; i < toRemove.length; i++) {
            this._stopModelSync(toRemove[i]);
        }
    };
    EditorModelManager.prototype._beginModelSync = function (resource) {
        var _this = this;
        var model = this._modelService.getModel(resource);
        if (!model) {
            return;
        }
        if (model.isTooLargeForSyncing()) {
            return;
        }
        var modelUrl = resource.toString();
        this._proxy.acceptNewModel({
            url: model.uri.toString(),
            lines: model.getLinesContent(),
            EOL: model.getEOL(),
            versionId: model.getVersionId()
        });
        var toDispose = [];
        toDispose.push(model.onDidChangeContent(function (e) {
            _this._proxy.acceptModelChanged(modelUrl.toString(), e);
        }));
        toDispose.push(model.onWillDispose(function () {
            _this._stopModelSync(modelUrl);
        }));
        toDispose.push(toDisposable(function () {
            _this._proxy.acceptRemovedModel(modelUrl);
        }));
        this._syncedModels[modelUrl] = toDispose;
    };
    EditorModelManager.prototype._stopModelSync = function (modelUrl) {
        var toDispose = this._syncedModels[modelUrl];
        delete this._syncedModels[modelUrl];
        delete this._syncedModelsLastUsedTime[modelUrl];
        dispose(toDispose);
    };
    return EditorModelManager;
}(Disposable));
var SynchronousWorkerClient = /** @class */ (function () {
    function SynchronousWorkerClient(instance) {
        this._instance = instance;
        this._proxyObj = Promise.resolve(this._instance);
    }
    SynchronousWorkerClient.prototype.dispose = function () {
        this._instance.dispose();
    };
    SynchronousWorkerClient.prototype.getProxyObject = function () {
        return this._proxyObj;
    };
    return SynchronousWorkerClient;
}());
var EditorWorkerClient = /** @class */ (function (_super) {
    __extends$1_(EditorWorkerClient, _super);
    function EditorWorkerClient(modelService, label) {
        var _this = _super.call(this) || this;
        _this._modelService = modelService;
        _this._workerFactory = new DefaultWorkerFactory(label);
        _this._worker = null;
        _this._modelManager = null;
        return _this;
    }
    EditorWorkerClient.prototype._getOrCreateWorker = function () {
        if (!this._worker) {
            try {
                this._worker = this._register(new SimpleWorkerClient(this._workerFactory, 'vs/editor/common/services/editorSimpleWorker'));
            }
            catch (err) {
                logOnceWebWorkerWarning(err);
                this._worker = new SynchronousWorkerClient(new EditorSimpleWorkerImpl(null));
            }
        }
        return this._worker;
    };
    EditorWorkerClient.prototype._getProxy = function () {
        var _this = this;
        return this._getOrCreateWorker().getProxyObject().then(null, function (err) {
            logOnceWebWorkerWarning(err);
            _this._worker = new SynchronousWorkerClient(new EditorSimpleWorkerImpl(null));
            return _this._getOrCreateWorker().getProxyObject();
        });
    };
    EditorWorkerClient.prototype._getOrCreateModelManager = function (proxy) {
        if (!this._modelManager) {
            this._modelManager = this._register(new EditorModelManager(proxy, this._modelService, false));
        }
        return this._modelManager;
    };
    EditorWorkerClient.prototype._withSyncedResources = function (resources) {
        var _this = this;
        return this._getProxy().then(function (proxy) {
            _this._getOrCreateModelManager(proxy).esureSyncedResources(resources);
            return proxy;
        });
    };
    EditorWorkerClient.prototype.computeDiff = function (original, modified, ignoreTrimWhitespace) {
        return this._withSyncedResources([original, modified]).then(function (proxy) {
            return proxy.computeDiff(original.toString(), modified.toString(), ignoreTrimWhitespace);
        });
    };
    EditorWorkerClient.prototype.computeMoreMinimalEdits = function (resource, edits) {
        return this._withSyncedResources([resource]).then(function (proxy) {
            return proxy.computeMoreMinimalEdits(resource.toString(), edits);
        });
    };
    EditorWorkerClient.prototype.computeLinks = function (resource) {
        return this._withSyncedResources([resource]).then(function (proxy) {
            return proxy.computeLinks(resource.toString());
        });
    };
    EditorWorkerClient.prototype.textualSuggest = function (resource, position) {
        var _this = this;
        return this._withSyncedResources([resource]).then(function (proxy) {
            var model = _this._modelService.getModel(resource);
            if (!model) {
                return null;
            }
            var wordDefRegExp = LanguageConfigurationRegistry.getWordDefinition(model.getLanguageIdentifier().id);
            var wordDef = wordDefRegExp.source;
            var wordDefFlags = (wordDefRegExp.global ? 'g' : '') + (wordDefRegExp.ignoreCase ? 'i' : '') + (wordDefRegExp.multiline ? 'm' : '');
            return proxy.textualSuggest(resource.toString(), position, wordDef, wordDefFlags);
        });
    };
    EditorWorkerClient.prototype.computeWordRanges = function (resource, range) {
        var _this = this;
        return this._withSyncedResources([resource]).then(function (proxy) {
            var model = _this._modelService.getModel(resource);
            if (!model) {
                return Promise.resolve(null);
            }
            var wordDefRegExp = LanguageConfigurationRegistry.getWordDefinition(model.getLanguageIdentifier().id);
            var wordDef = wordDefRegExp.source;
            var wordDefFlags = (wordDefRegExp.global ? 'g' : '') + (wordDefRegExp.ignoreCase ? 'i' : '') + (wordDefRegExp.multiline ? 'm' : '');
            return proxy.computeWordRanges(resource.toString(), range, wordDef, wordDefFlags);
        });
    };
    EditorWorkerClient.prototype.navigateValueSet = function (resource, range, up) {
        var _this = this;
        return this._withSyncedResources([resource]).then(function (proxy) {
            var model = _this._modelService.getModel(resource);
            if (!model) {
                return null;
            }
            var wordDefRegExp = LanguageConfigurationRegistry.getWordDefinition(model.getLanguageIdentifier().id);
            var wordDef = wordDefRegExp.source;
            var wordDefFlags = (wordDefRegExp.global ? 'g' : '') + (wordDefRegExp.ignoreCase ? 'i' : '') + (wordDefRegExp.multiline ? 'm' : '');
            return proxy.navigateValueSet(resource.toString(), range, up, wordDef, wordDefFlags);
        });
    };
    return EditorWorkerClient;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$20 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * Create a new web worker that has model syncing capabilities built in.
 * Specify an AMD module to load that will `create` an object that will be proxied.
 */
function createWebWorker(modelService, opts) {
    return new MonacoWebWorkerImpl(modelService, opts);
}
var MonacoWebWorkerImpl = /** @class */ (function (_super) {
    __extends$20(MonacoWebWorkerImpl, _super);
    function MonacoWebWorkerImpl(modelService, opts) {
        var _this = _super.call(this, modelService, opts.label) || this;
        _this._foreignModuleId = opts.moduleId;
        _this._foreignModuleCreateData = opts.createData || null;
        _this._foreignProxy = null;
        return _this;
    }
    MonacoWebWorkerImpl.prototype._getForeignProxy = function () {
        var _this = this;
        if (!this._foreignProxy) {
            this._foreignProxy = this._getProxy().then(function (proxy) {
                return proxy.loadForeignModule(_this._foreignModuleId, _this._foreignModuleCreateData).then(function (foreignMethods) {
                    _this._foreignModuleCreateData = null;
                    var proxyMethodRequest = function (method, args) {
                        return proxy.fmr(method, args);
                    };
                    var createProxyMethod = function (method, proxyMethodRequest) {
                        return function () {
                            var args = Array.prototype.slice.call(arguments, 0);
                            return proxyMethodRequest(method, args);
                        };
                    };
                    var foreignProxy = {};
                    for (var i = 0; i < foreignMethods.length; i++) {
                        foreignProxy[foreignMethods[i]] = createProxyMethod(foreignMethods[i], proxyMethodRequest);
                    }
                    return foreignProxy;
                });
            });
        }
        return this._foreignProxy;
    };
    MonacoWebWorkerImpl.prototype.getProxy = function () {
        return this._getForeignProxy();
    };
    MonacoWebWorkerImpl.prototype.withSyncedResources = function (resources) {
        var _this = this;
        return this._withSyncedResources(resources).then(function (_) { return _this.getProxy(); });
    };
    return MonacoWebWorkerImpl;
}(EditorWorkerClient));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Colorizer = /** @class */ (function () {
    function Colorizer() {
    }
    Colorizer.colorizeElement = function (themeService, modeService, domNode, options) {
        options = options || {};
        var theme = options.theme || 'vs';
        var mimeType = options.mimeType || domNode.getAttribute('lang') || domNode.getAttribute('data-lang');
        if (!mimeType) {
            console.error('Mode not detected');
            return Promise.resolve();
        }
        themeService.setTheme(theme);
        var text = domNode.firstChild ? domNode.firstChild.nodeValue : '';
        domNode.className += ' ' + theme;
        var render = function (str) {
            domNode.innerHTML = str;
        };
        return this.colorize(modeService, text || '', mimeType, options).then(render, function (err) { return console.error(err); });
    };
    Colorizer.colorize = function (modeService, text, mimeType, options) {
        var tabSize = 4;
        if (options && typeof options.tabSize === 'number') {
            tabSize = options.tabSize;
        }
        if (startsWithUTF8BOM(text)) {
            text = text.substr(1);
        }
        var lines = text.split(/\r\n|\r|\n/);
        var language = modeService.getModeId(mimeType);
        if (!language) {
            return Promise.resolve(_fakeColorize(lines, tabSize));
        }
        // Send out the event to create the mode
        modeService.triggerMode(language);
        var tokenizationSupport = TokenizationRegistry.get(language);
        if (tokenizationSupport) {
            return Promise.resolve(_colorize(lines, tabSize, tokenizationSupport));
        }
        return new Promise(function (resolve, reject) {
            var listener = null;
            var timeout$$1 = null;
            var execute = function () {
                if (listener) {
                    listener.dispose();
                    listener = null;
                }
                if (timeout$$1) {
                    timeout$$1.dispose();
                    timeout$$1 = null;
                }
                var tokenizationSupport = TokenizationRegistry.get(language);
                if (tokenizationSupport) {
                    return resolve(_colorize(lines, tabSize, tokenizationSupport));
                }
                return resolve(_fakeColorize(lines, tabSize));
            };
            // wait 500ms for mode to load, then give up
            timeout$$1 = new TimeoutTimer();
            timeout$$1.cancelAndSet(execute, 500);
            listener = TokenizationRegistry.onDidChange(function (e) {
                if (e.changedLanguages.indexOf(language) >= 0) {
                    execute();
                }
            });
        });
    };
    Colorizer.colorizeLine = function (line, mightContainNonBasicASCII, mightContainRTL, tokens, tabSize) {
        if (tabSize === void 0) { tabSize = 4; }
        var isBasicASCII$$1 = ViewLineRenderingData.isBasicASCII(line, mightContainNonBasicASCII);
        var containsRTL$$1 = ViewLineRenderingData.containsRTL(line, isBasicASCII$$1, mightContainRTL);
        var renderResult = renderViewLine2(new RenderLineInput(false, true, line, false, isBasicASCII$$1, containsRTL$$1, 0, tokens, [], tabSize, 0, -1, 'none', false, false));
        return renderResult.html;
    };
    Colorizer.colorizeModelLine = function (model, lineNumber, tabSize) {
        if (tabSize === void 0) { tabSize = 4; }
        var content = model.getLineContent(lineNumber);
        model.forceTokenization(lineNumber);
        var tokens = model.getLineTokens(lineNumber);
        var inflatedTokens = tokens.inflate();
        return this.colorizeLine(content, model.mightContainNonBasicASCII(), model.mightContainRTL(), inflatedTokens, tabSize);
    };
    return Colorizer;
}());
function _colorize(lines, tabSize, tokenizationSupport) {
    return _actualColorize(lines, tabSize, tokenizationSupport);
}
function _fakeColorize(lines, tabSize) {
    var html = [];
    var defaultMetadata = ((0 /* None */ << 11 /* FONT_STYLE_OFFSET */)
        | (1 /* DefaultForeground */ << 14 /* FOREGROUND_OFFSET */)
        | (2 /* DefaultBackground */ << 23 /* BACKGROUND_OFFSET */)) >>> 0;
    var tokens = new Uint32Array(2);
    tokens[0] = 0;
    tokens[1] = defaultMetadata;
    for (var i = 0, length_1 = lines.length; i < length_1; i++) {
        var line = lines[i];
        tokens[0] = line.length;
        var lineTokens = new LineTokens(tokens, line);
        var isBasicASCII$$1 = ViewLineRenderingData.isBasicASCII(line, /* check for basic ASCII */ true);
        var containsRTL$$1 = ViewLineRenderingData.containsRTL(line, isBasicASCII$$1, /* check for RTL */ true);
        var renderResult = renderViewLine2(new RenderLineInput(false, true, line, false, isBasicASCII$$1, containsRTL$$1, 0, lineTokens, [], tabSize, 0, -1, 'none', false, false));
        html = html.concat(renderResult.html);
        html.push('<br/>');
    }
    return html.join('');
}
function _actualColorize(lines, tabSize, tokenizationSupport) {
    var html = [];
    var state = tokenizationSupport.getInitialState();
    for (var i = 0, length_2 = lines.length; i < length_2; i++) {
        var line = lines[i];
        var tokenizeResult = tokenizationSupport.tokenize2(line, state, 0);
        LineTokens.convertToEndOffset(tokenizeResult.tokens, line.length);
        var lineTokens = new LineTokens(tokenizeResult.tokens, line);
        var isBasicASCII$$1 = ViewLineRenderingData.isBasicASCII(line, /* check for basic ASCII */ true);
        var containsRTL$$1 = ViewLineRenderingData.containsRTL(line, isBasicASCII$$1, /* check for RTL */ true);
        var renderResult = renderViewLine2(new RenderLineInput(false, true, line, false, isBasicASCII$$1, containsRTL$$1, 0, lineTokens.inflate(), [], tabSize, 0, -1, 'none', false, false));
        html = html.concat(renderResult.html);
        html.push('<br/>');
        state = tokenizeResult.endState;
    }
    return html.join('');
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate$t = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$q = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var Menu = /** @class */ (function () {
    function Menu(id, startupSignal, _commandService, _contextKeyService) {
        var _this = this;
        this._commandService = _commandService;
        this._contextKeyService = _contextKeyService;
        this._menuGroups = [];
        this._disposables = [];
        this._onDidChange = new Emitter();
        startupSignal.then(function (_) {
            var menuItems = MenuRegistry.getMenuItems(id);
            var keysFilter = new Set();
            var group;
            menuItems.sort(Menu._compareMenuItems);
            for (var _i = 0, menuItems_1 = menuItems; _i < menuItems_1.length; _i++) {
                var item = menuItems_1[_i];
                // group by groupId
                var groupName = item.group;
                if (!group || group[0] !== groupName) {
                    group = [groupName || '', []];
                    _this._menuGroups.push(group);
                }
                group[1].push(item);
                // keep keys for eventing
                Menu._fillInKbExprKeys(item.when, keysFilter);
                // keep precondition keys for event if applicable
                if (isIMenuItem(item) && item.command.precondition) {
                    Menu._fillInKbExprKeys(item.command.precondition, keysFilter);
                }
                // keep toggled keys for event if applicable
                if (isIMenuItem(item) && item.command.toggled) {
                    Menu._fillInKbExprKeys(item.command.toggled, keysFilter);
                }
            }
            // subscribe to context changes
            _this._disposables.push(_this._contextKeyService.onDidChangeContext(function (event) {
                if (event.affectsSome(keysFilter)) {
                    _this._onDidChange.fire();
                }
            }));
            _this._onDidChange.fire(_this);
        });
    }
    Menu.prototype.dispose = function () {
        this._disposables = dispose(this._disposables);
        this._onDidChange.dispose();
    };
    Menu.prototype.getActions = function (options) {
        var result = [];
        for (var _i = 0, _a = this._menuGroups; _i < _a.length; _i++) {
            var group = _a[_i];
            var id = group[0], items = group[1];
            var activeActions = [];
            for (var _b = 0, items_1 = items; _b < items_1.length; _b++) {
                var item = items_1[_b];
                if (this._contextKeyService.contextMatchesRules(item.when || null)) {
                    var action = isIMenuItem(item) ? new MenuItemAction(item.command, item.alt, options, this._contextKeyService, this._commandService) : new SubmenuItemAction(item);
                    activeActions.push(action);
                }
            }
            if (activeActions.length > 0) {
                result.push([id, activeActions]);
            }
        }
        return result;
    };
    Menu._fillInKbExprKeys = function (exp, set) {
        if (exp) {
            for (var _i = 0, _a = exp.keys(); _i < _a.length; _i++) {
                var key = _a[_i];
                set.add(key);
            }
        }
    };
    Menu._compareMenuItems = function (a, b) {
        var aGroup = a.group;
        var bGroup = b.group;
        if (aGroup !== bGroup) {
            // Falsy groups come last
            if (!aGroup) {
                return 1;
            }
            else if (!bGroup) {
                return -1;
            }
            // 'navigation' group comes first
            if (aGroup === 'navigation') {
                return -1;
            }
            else if (bGroup === 'navigation') {
                return 1;
            }
            // lexical sort for groups
            var value = aGroup.localeCompare(bGroup);
            if (value !== 0) {
                return value;
            }
        }
        // sort on priority - default is 0
        var aPrio = a.order || 0;
        var bPrio = b.order || 0;
        if (aPrio < bPrio) {
            return -1;
        }
        else if (aPrio > bPrio) {
            return 1;
        }
        // sort on titles
        var aTitle = typeof a.command.title === 'string' ? a.command.title : a.command.title.value;
        var bTitle = typeof b.command.title === 'string' ? b.command.title : b.command.title.value;
        return aTitle.localeCompare(bTitle);
    };
    Menu = __decorate$t([
        __param$q(2, ICommandService),
        __param$q(3, IContextKeyService)
    ], Menu);
    return Menu;
}());

var __extends$21 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ConfigurationModel = /** @class */ (function () {
    function ConfigurationModel(_contents, _keys, _overrides) {
        if (_contents === void 0) { _contents = {}; }
        if (_keys === void 0) { _keys = []; }
        if (_overrides === void 0) { _overrides = []; }
        this._contents = _contents;
        this._keys = _keys;
        this._overrides = _overrides;
        this.isFrozen = false;
    }
    Object.defineProperty(ConfigurationModel.prototype, "contents", {
        get: function () {
            return this.checkAndFreeze(this._contents);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigurationModel.prototype, "overrides", {
        get: function () {
            return this.checkAndFreeze(this._overrides);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigurationModel.prototype, "keys", {
        get: function () {
            return this.checkAndFreeze(this._keys);
        },
        enumerable: true,
        configurable: true
    });
    ConfigurationModel.prototype.getValue = function (section) {
        return section ? getConfigurationValue(this.contents, section) : this.contents;
    };
    ConfigurationModel.prototype.override = function (identifier) {
        var overrideContents = this.getContentsForOverrideIdentifer(identifier);
        if (!overrideContents || typeof overrideContents !== 'object' || !Object.keys(overrideContents).length) {
            // If there are no valid overrides, return self
            return this;
        }
        var contents = {};
        for (var _i = 0, _a = distinct(Object.keys(this.contents).concat(Object.keys(overrideContents))); _i < _a.length; _i++) {
            var key = _a[_i];
            var contentsForKey = this.contents[key];
            var overrideContentsForKey = overrideContents[key];
            // If there are override contents for the key, clone and merge otherwise use base contents
            if (overrideContentsForKey) {
                // Clone and merge only if base contents and override contents are of type object otherwise just override
                if (typeof contentsForKey === 'object' && typeof overrideContentsForKey === 'object') {
                    contentsForKey = deepClone(contentsForKey);
                    this.mergeContents(contentsForKey, overrideContentsForKey);
                }
                else {
                    contentsForKey = overrideContentsForKey;
                }
            }
            contents[key] = contentsForKey;
        }
        return new ConfigurationModel(contents);
    };
    ConfigurationModel.prototype.merge = function () {
        var others = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            others[_i] = arguments[_i];
        }
        var contents = deepClone(this.contents);
        var overrides = deepClone(this.overrides);
        var keys$$1 = this.keys.slice();
        for (var _a = 0, others_1 = others; _a < others_1.length; _a++) {
            var other = others_1[_a];
            this.mergeContents(contents, other.contents);
            var _loop_1 = function (otherOverride) {
                var override = overrides.filter(function (o) { return equals(o.identifiers, otherOverride.identifiers); })[0];
                if (override) {
                    this_1.mergeContents(override.contents, otherOverride.contents);
                }
                else {
                    overrides.push(deepClone(otherOverride));
                }
            };
            var this_1 = this;
            for (var _b = 0, _c = other.overrides; _b < _c.length; _b++) {
                var otherOverride = _c[_b];
                _loop_1(otherOverride);
            }
            for (var _d = 0, _e = other.keys; _d < _e.length; _d++) {
                var key = _e[_d];
                if (keys$$1.indexOf(key) === -1) {
                    keys$$1.push(key);
                }
            }
        }
        return new ConfigurationModel(contents, keys$$1, overrides);
    };
    ConfigurationModel.prototype.freeze = function () {
        this.isFrozen = true;
        return this;
    };
    ConfigurationModel.prototype.mergeContents = function (source, target) {
        for (var _i = 0, _a = Object.keys(target); _i < _a.length; _i++) {
            var key = _a[_i];
            if (key in source) {
                if (isObject(source[key]) && isObject(target[key])) {
                    this.mergeContents(source[key], target[key]);
                    continue;
                }
            }
            source[key] = deepClone(target[key]);
        }
    };
    ConfigurationModel.prototype.checkAndFreeze = function (data) {
        if (this.isFrozen && !Object.isFrozen(data)) {
            return deepFreeze(data);
        }
        return data;
    };
    ConfigurationModel.prototype.getContentsForOverrideIdentifer = function (identifier) {
        for (var _i = 0, _a = this.overrides; _i < _a.length; _i++) {
            var override = _a[_i];
            if (override.identifiers.indexOf(identifier) !== -1) {
                return override.contents;
            }
        }
        return null;
    };
    ConfigurationModel.prototype.toJSON = function () {
        return {
            contents: this.contents,
            overrides: this.overrides,
            keys: this.keys
        };
    };
    // Update methods
    ConfigurationModel.prototype.setValue = function (key, value) {
        this.addKey(key);
        addToValueTree(this.contents, key, value, function (e) { throw new Error(e); });
    };
    ConfigurationModel.prototype.removeValue = function (key) {
        if (this.removeKey(key)) {
            removeFromValueTree(this.contents, key);
        }
    };
    ConfigurationModel.prototype.addKey = function (key) {
        var index = this.keys.length;
        for (var i = 0; i < index; i++) {
            if (key.indexOf(this.keys[i]) === 0) {
                index = i;
            }
        }
        this.keys.splice(index, 1, key);
    };
    ConfigurationModel.prototype.removeKey = function (key) {
        var index = this.keys.indexOf(key);
        if (index !== -1) {
            this.keys.splice(index, 1);
            return true;
        }
        return false;
    };
    return ConfigurationModel;
}());
var DefaultConfigurationModel = /** @class */ (function (_super) {
    __extends$21(DefaultConfigurationModel, _super);
    function DefaultConfigurationModel() {
        var _this = this;
        var contents = getDefaultValues();
        var keys$$1 = getConfigurationKeys();
        var overrides = [];
        for (var _i = 0, _a = Object.keys(contents); _i < _a.length; _i++) {
            var key = _a[_i];
            if (OVERRIDE_PROPERTY_PATTERN.test(key)) {
                overrides.push({
                    identifiers: [overrideIdentifierFromKey(key).trim()],
                    contents: toValuesTree(contents[key], function (message) { return console.error("Conflict in default settings file: " + message); })
                });
            }
        }
        _this = _super.call(this, contents, keys$$1, overrides) || this;
        return _this;
    }
    return DefaultConfigurationModel;
}(ConfigurationModel));
var Configuration$1 = /** @class */ (function () {
    function Configuration(_defaultConfiguration, _userConfiguration, _workspaceConfiguration, _folderConfigurations, _memoryConfiguration, _memoryConfigurationByResource, _freeze) {
        if (_workspaceConfiguration === void 0) { _workspaceConfiguration = new ConfigurationModel(); }
        if (_folderConfigurations === void 0) { _folderConfigurations = new ResourceMap(); }
        if (_memoryConfiguration === void 0) { _memoryConfiguration = new ConfigurationModel(); }
        if (_memoryConfigurationByResource === void 0) { _memoryConfigurationByResource = new ResourceMap(); }
        if (_freeze === void 0) { _freeze = true; }
        this._defaultConfiguration = _defaultConfiguration;
        this._userConfiguration = _userConfiguration;
        this._workspaceConfiguration = _workspaceConfiguration;
        this._folderConfigurations = _folderConfigurations;
        this._memoryConfiguration = _memoryConfiguration;
        this._memoryConfigurationByResource = _memoryConfigurationByResource;
        this._freeze = _freeze;
        this._workspaceConsolidatedConfiguration = null;
        this._foldersConsolidatedConfigurations = new ResourceMap();
    }
    Configuration.prototype.getValue = function (section, overrides, workspace) {
        var consolidateConfigurationModel = this.getConsolidateConfigurationModel(overrides, workspace);
        return consolidateConfigurationModel.getValue(section);
    };
    Configuration.prototype.updateValue = function (key, value, overrides) {
        if (overrides === void 0) { overrides = {}; }
        var memoryConfiguration;
        if (overrides.resource) {
            memoryConfiguration = this._memoryConfigurationByResource.get(overrides.resource);
            if (!memoryConfiguration) {
                memoryConfiguration = new ConfigurationModel();
                this._memoryConfigurationByResource.set(overrides.resource, memoryConfiguration);
            }
        }
        else {
            memoryConfiguration = this._memoryConfiguration;
        }
        if (value === void 0) {
            memoryConfiguration.removeValue(key);
        }
        else {
            memoryConfiguration.setValue(key, value);
        }
        if (!overrides.resource) {
            this._workspaceConsolidatedConfiguration = null;
        }
    };
    Configuration.prototype.getConsolidateConfigurationModel = function (overrides, workspace) {
        var configurationModel = this.getConsolidatedConfigurationModelForResource(overrides, workspace);
        return overrides.overrideIdentifier ? configurationModel.override(overrides.overrideIdentifier) : configurationModel;
    };
    Configuration.prototype.getConsolidatedConfigurationModelForResource = function (_a, workspace) {
        var resource = _a.resource;
        var consolidateConfiguration = this.getWorkspaceConsolidatedConfiguration();
        if (workspace && resource) {
            var root = workspace.getFolder(resource);
            if (root) {
                consolidateConfiguration = this.getFolderConsolidatedConfiguration(root.uri) || consolidateConfiguration;
            }
            var memoryConfigurationForResource = this._memoryConfigurationByResource.get(resource);
            if (memoryConfigurationForResource) {
                consolidateConfiguration = consolidateConfiguration.merge(memoryConfigurationForResource);
            }
        }
        return consolidateConfiguration;
    };
    Configuration.prototype.getWorkspaceConsolidatedConfiguration = function () {
        if (!this._workspaceConsolidatedConfiguration) {
            this._workspaceConsolidatedConfiguration = this._defaultConfiguration.merge(this._userConfiguration, this._workspaceConfiguration, this._memoryConfiguration);
            if (this._freeze) {
                this._workspaceConfiguration = this._workspaceConfiguration.freeze();
            }
        }
        return this._workspaceConsolidatedConfiguration;
    };
    Configuration.prototype.getFolderConsolidatedConfiguration = function (folder) {
        var folderConsolidatedConfiguration = this._foldersConsolidatedConfigurations.get(folder);
        if (!folderConsolidatedConfiguration) {
            var workspaceConsolidateConfiguration = this.getWorkspaceConsolidatedConfiguration();
            var folderConfiguration = this._folderConfigurations.get(folder);
            if (folderConfiguration) {
                folderConsolidatedConfiguration = workspaceConsolidateConfiguration.merge(folderConfiguration);
                if (this._freeze) {
                    folderConsolidatedConfiguration = folderConsolidatedConfiguration.freeze();
                }
                this._foldersConsolidatedConfigurations.set(folder, folderConsolidatedConfiguration);
            }
            else {
                folderConsolidatedConfiguration = workspaceConsolidateConfiguration;
            }
        }
        return folderConsolidatedConfiguration;
    };
    return Configuration;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$22 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var AbstractKeybindingService = /** @class */ (function (_super) {
    __extends$22(AbstractKeybindingService, _super);
    function AbstractKeybindingService(contextKeyService, commandService, telemetryService, notificationService, statusService) {
        var _this = _super.call(this) || this;
        _this._contextKeyService = contextKeyService;
        _this._commandService = commandService;
        _this._telemetryService = telemetryService;
        _this._statusService = statusService;
        _this._notificationService = notificationService;
        _this._currentChord = null;
        _this._currentChordChecker = new IntervalTimer();
        _this._currentChordStatusMessage = null;
        _this._onDidUpdateKeybindings = _this._register(new Emitter());
        return _this;
    }
    AbstractKeybindingService.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    Object.defineProperty(AbstractKeybindingService.prototype, "onDidUpdateKeybindings", {
        get: function () {
            return this._onDidUpdateKeybindings ? this._onDidUpdateKeybindings.event : Event.None; // Sinon stubbing walks properties on prototype
        },
        enumerable: true,
        configurable: true
    });
    AbstractKeybindingService.prototype.lookupKeybinding = function (commandId) {
        var result = this._getResolver().lookupPrimaryKeybinding(commandId);
        if (!result) {
            return null;
        }
        return result.resolvedKeybinding;
    };
    AbstractKeybindingService.prototype._enterChordMode = function (firstPart, keypressLabel) {
        var _this = this;
        this._currentChord = {
            keypress: firstPart,
            label: keypressLabel
        };
        if (this._statusService) {
            this._currentChordStatusMessage = this._statusService.setStatusMessage(localize('first.chord', "({0}) was pressed. Waiting for second key of chord...", keypressLabel));
        }
        var chordEnterTime = Date.now();
        this._currentChordChecker.cancelAndSet(function () {
            if (!_this._documentHasFocus()) {
                // Focus has been lost => leave chord mode
                _this._leaveChordMode();
                return;
            }
            if (Date.now() - chordEnterTime > 5000) {
                // 5 seconds elapsed => leave chord mode
                _this._leaveChordMode();
            }
        }, 500);
    };
    AbstractKeybindingService.prototype._leaveChordMode = function () {
        if (this._currentChordStatusMessage) {
            this._currentChordStatusMessage.dispose();
            this._currentChordStatusMessage = null;
        }
        this._currentChordChecker.cancel();
        this._currentChord = null;
    };
    AbstractKeybindingService.prototype._dispatch = function (e, target) {
        var _this = this;
        var shouldPreventDefault = false;
        var keybinding = this.resolveKeyboardEvent(e);
        if (keybinding.isChord()) {
            console.warn('Unexpected keyboard event mapped to a chord');
            return false;
        }
        var firstPart = keybinding.getDispatchParts()[0];
        if (firstPart === null) {
            // cannot be dispatched, probably only modifier keys
            return shouldPreventDefault;
        }
        var contextValue = this._contextKeyService.getContext(target);
        var currentChord = this._currentChord ? this._currentChord.keypress : null;
        var keypressLabel = keybinding.getLabel();
        var resolveResult = this._getResolver().resolve(contextValue, currentChord, firstPart);
        if (resolveResult && resolveResult.enterChord) {
            shouldPreventDefault = true;
            this._enterChordMode(firstPart, keypressLabel);
            return shouldPreventDefault;
        }
        if (this._statusService && this._currentChord) {
            if (!resolveResult || !resolveResult.commandId) {
                this._statusService.setStatusMessage(localize('missing.chord', "The key combination ({0}, {1}) is not a command.", this._currentChord.label, keypressLabel), 10 * 1000 /* 10s */);
                shouldPreventDefault = true;
            }
        }
        this._leaveChordMode();
        if (resolveResult && resolveResult.commandId) {
            if (!resolveResult.bubble) {
                shouldPreventDefault = true;
            }
            if (typeof resolveResult.commandArgs === 'undefined') {
                this._commandService.executeCommand(resolveResult.commandId).then(undefined, function (err) { return _this._notificationService.warn(err); });
            }
            else {
                this._commandService.executeCommand(resolveResult.commandId, resolveResult.commandArgs).then(undefined, function (err) { return _this._notificationService.warn(err); });
            }
            /* __GDPR__
                "workbenchActionExecuted" : {
                    "id" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" },
                    "from": { "classification": "SystemMetaData", "purpose": "FeatureInsight" }
                }
            */
            this._telemetryService.publicLog('workbenchActionExecuted', { id: resolveResult.commandId, from: 'keybinding' });
        }
        return shouldPreventDefault;
    };
    return AbstractKeybindingService;
}(Disposable));

var KeybindingResolver = /** @class */ (function () {
    function KeybindingResolver(defaultKeybindings, overrides) {
        this._defaultKeybindings = defaultKeybindings;
        this._defaultBoundCommands = new Map();
        for (var i = 0, len = defaultKeybindings.length; i < len; i++) {
            var command = defaultKeybindings[i].command;
            if (command) {
                this._defaultBoundCommands.set(command, true);
            }
        }
        this._map = new Map();
        this._lookupMap = new Map();
        this._keybindings = KeybindingResolver.combine(defaultKeybindings, overrides);
        for (var i = 0, len = this._keybindings.length; i < len; i++) {
            var k = this._keybindings[i];
            if (k.keypressFirstPart === null) {
                // unbound
                continue;
            }
            this._addKeyPress(k.keypressFirstPart, k);
        }
    }
    KeybindingResolver._isTargetedForRemoval = function (defaultKb, keypressFirstPart, keypressChordPart, command, when) {
        if (defaultKb.command !== command) {
            return false;
        }
        if (keypressFirstPart && defaultKb.keypressFirstPart !== keypressFirstPart) {
            return false;
        }
        if (keypressChordPart && defaultKb.keypressChordPart !== keypressChordPart) {
            return false;
        }
        if (when) {
            if (!defaultKb.when) {
                return false;
            }
            if (!when.equals(defaultKb.when)) {
                return false;
            }
        }
        return true;
    };
    /**
     * Looks for rules containing -command in `overrides` and removes them directly from `defaults`.
     */
    KeybindingResolver.combine = function (defaults, rawOverrides) {
        defaults = defaults.slice(0);
        var overrides = [];
        for (var i = 0, len = rawOverrides.length; i < len; i++) {
            var override = rawOverrides[i];
            if (!override.command || override.command.length === 0 || override.command.charAt(0) !== '-') {
                overrides.push(override);
                continue;
            }
            var command = override.command.substr(1);
            var keypressFirstPart = override.keypressFirstPart;
            var keypressChordPart = override.keypressChordPart;
            var when = override.when;
            for (var j = defaults.length - 1; j >= 0; j--) {
                if (this._isTargetedForRemoval(defaults[j], keypressFirstPart, keypressChordPart, command, when)) {
                    defaults.splice(j, 1);
                }
            }
        }
        return defaults.concat(overrides);
    };
    KeybindingResolver.prototype._addKeyPress = function (keypress, item) {
        var conflicts = this._map.get(keypress);
        if (typeof conflicts === 'undefined') {
            // There is no conflict so far
            this._map.set(keypress, [item]);
            this._addToLookupMap(item);
            return;
        }
        for (var i = conflicts.length - 1; i >= 0; i--) {
            var conflict = conflicts[i];
            if (conflict.command === item.command) {
                continue;
            }
            var conflictIsChord = (conflict.keypressChordPart !== null);
            var itemIsChord = (item.keypressChordPart !== null);
            if (conflictIsChord && itemIsChord && conflict.keypressChordPart !== item.keypressChordPart) {
                // The conflict only shares the chord start with this command
                continue;
            }
            if (KeybindingResolver.whenIsEntirelyIncluded(conflict.when, item.when)) {
                // `item` completely overwrites `conflict`
                // Remove conflict from the lookupMap
                this._removeFromLookupMap(conflict);
            }
        }
        conflicts.push(item);
        this._addToLookupMap(item);
    };
    KeybindingResolver.prototype._addToLookupMap = function (item) {
        if (!item.command) {
            return;
        }
        var arr = this._lookupMap.get(item.command);
        if (typeof arr === 'undefined') {
            arr = [item];
            this._lookupMap.set(item.command, arr);
        }
        else {
            arr.push(item);
        }
    };
    KeybindingResolver.prototype._removeFromLookupMap = function (item) {
        if (!item.command) {
            return;
        }
        var arr = this._lookupMap.get(item.command);
        if (typeof arr === 'undefined') {
            return;
        }
        for (var i = 0, len = arr.length; i < len; i++) {
            if (arr[i] === item) {
                arr.splice(i, 1);
                return;
            }
        }
    };
    /**
     * Returns true if it is provable `a` implies `b`.
     * **Precondition**: Assumes `a` and `b` are normalized!
     */
    KeybindingResolver.whenIsEntirelyIncluded = function (a, b) {
        if (!b) {
            return true;
        }
        if (!a) {
            return false;
        }
        var aExpressions = ((a instanceof ContextKeyAndExpr) ? a.expr : [a]);
        var bExpressions = ((b instanceof ContextKeyAndExpr) ? b.expr : [b]);
        var aIndex = 0;
        for (var bIndex = 0; bIndex < bExpressions.length; bIndex++) {
            var bExpr = bExpressions[bIndex];
            var bExprMatched = false;
            while (!bExprMatched && aIndex < aExpressions.length) {
                var aExpr = aExpressions[aIndex];
                if (aExpr.equals(bExpr)) {
                    bExprMatched = true;
                }
                aIndex++;
            }
            if (!bExprMatched) {
                return false;
            }
        }
        return true;
    };
    KeybindingResolver.prototype.lookupPrimaryKeybinding = function (commandId) {
        var items = this._lookupMap.get(commandId);
        if (typeof items === 'undefined' || items.length === 0) {
            return null;
        }
        return items[items.length - 1];
    };
    KeybindingResolver.prototype.resolve = function (context, currentChord, keypress) {
        var lookupMap = null;
        if (currentChord !== null) {
            // Fetch all chord bindings for `currentChord`
            var candidates = this._map.get(currentChord);
            if (typeof candidates === 'undefined') {
                // No chords starting with `currentChord`
                return null;
            }
            lookupMap = [];
            for (var i = 0, len = candidates.length; i < len; i++) {
                var candidate = candidates[i];
                if (candidate.keypressChordPart === keypress) {
                    lookupMap.push(candidate);
                }
            }
        }
        else {
            var candidates = this._map.get(keypress);
            if (typeof candidates === 'undefined') {
                // No bindings with `keypress`
                return null;
            }
            lookupMap = candidates;
        }
        var result = this._findCommand(context, lookupMap);
        if (!result) {
            return null;
        }
        if (currentChord === null && result.keypressChordPart !== null) {
            return {
                enterChord: true,
                commandId: null,
                commandArgs: null,
                bubble: false
            };
        }
        return {
            enterChord: false,
            commandId: result.command,
            commandArgs: result.commandArgs,
            bubble: result.bubble
        };
    };
    KeybindingResolver.prototype._findCommand = function (context, matches) {
        for (var i = matches.length - 1; i >= 0; i--) {
            var k = matches[i];
            if (!KeybindingResolver.contextMatchesRules(context, k.when)) {
                continue;
            }
            return k;
        }
        return null;
    };
    KeybindingResolver.contextMatchesRules = function (context, rules) {
        if (!rules) {
            return true;
        }
        return rules.evaluate(context);
    };
    return KeybindingResolver;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ResolvedKeybindingItem = /** @class */ (function () {
    function ResolvedKeybindingItem(resolvedKeybinding, command, commandArgs, when, isDefault) {
        this.resolvedKeybinding = resolvedKeybinding;
        if (resolvedKeybinding) {
            var _a = resolvedKeybinding.getDispatchParts(), keypressFirstPart = _a[0], keypressChordPart = _a[1];
            this.keypressFirstPart = keypressFirstPart;
            this.keypressChordPart = keypressChordPart;
        }
        else {
            this.keypressFirstPart = null;
            this.keypressChordPart = null;
        }
        this.bubble = (command ? command.charCodeAt(0) === 94 /* Caret */ : false);
        this.command = this.bubble ? command.substr(1) : command;
        this.commandArgs = commandArgs;
        this.when = when;
        this.isDefault = isDefault;
    }
    return ResolvedKeybindingItem;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$23 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * Do not instantiate. Use KeybindingService to get a ResolvedKeybinding seeded with information about the current kb layout.
 */
var USLayoutResolvedKeybinding = /** @class */ (function (_super) {
    __extends$23(USLayoutResolvedKeybinding, _super);
    function USLayoutResolvedKeybinding(actual, OS) {
        var _this = _super.call(this) || this;
        _this._os = OS;
        if (!actual) {
            throw new Error("Invalid USLayoutResolvedKeybinding");
        }
        else if (actual.type === 2 /* Chord */) {
            _this._firstPart = actual.firstPart;
            _this._chordPart = actual.chordPart;
        }
        else {
            _this._firstPart = actual;
            _this._chordPart = null;
        }
        return _this;
    }
    USLayoutResolvedKeybinding.prototype._keyCodeToUILabel = function (keyCode) {
        if (this._os === 2 /* Macintosh */) {
            switch (keyCode) {
                case 15 /* LeftArrow */:
                    return '←';
                case 16 /* UpArrow */:
                    return '↑';
                case 17 /* RightArrow */:
                    return '→';
                case 18 /* DownArrow */:
                    return '↓';
            }
        }
        return KeyCodeUtils.toString(keyCode);
    };
    USLayoutResolvedKeybinding.prototype._getUILabelForKeybinding = function (keybinding) {
        if (!keybinding) {
            return null;
        }
        if (keybinding.isDuplicateModifierCase()) {
            return '';
        }
        return this._keyCodeToUILabel(keybinding.keyCode);
    };
    USLayoutResolvedKeybinding.prototype.getLabel = function () {
        var firstPart = this._getUILabelForKeybinding(this._firstPart);
        var chordPart = this._getUILabelForKeybinding(this._chordPart);
        return UILabelProvider.toLabel(this._firstPart, firstPart, this._chordPart, chordPart, this._os);
    };
    USLayoutResolvedKeybinding.prototype._getAriaLabelForKeybinding = function (keybinding) {
        if (!keybinding) {
            return null;
        }
        if (keybinding.isDuplicateModifierCase()) {
            return '';
        }
        return KeyCodeUtils.toString(keybinding.keyCode);
    };
    USLayoutResolvedKeybinding.prototype.getAriaLabel = function () {
        var firstPart = this._getAriaLabelForKeybinding(this._firstPart);
        var chordPart = this._getAriaLabelForKeybinding(this._chordPart);
        return AriaLabelProvider.toLabel(this._firstPart, firstPart, this._chordPart, chordPart, this._os);
    };
    USLayoutResolvedKeybinding.prototype.isChord = function () {
        return (this._chordPart ? true : false);
    };
    USLayoutResolvedKeybinding.prototype.getParts = function () {
        return [
            this._toResolvedKeybindingPart(this._firstPart),
            this._chordPart ? this._toResolvedKeybindingPart(this._chordPart) : null
        ];
    };
    USLayoutResolvedKeybinding.prototype._toResolvedKeybindingPart = function (keybinding) {
        return new ResolvedKeybindingPart(keybinding.ctrlKey, keybinding.shiftKey, keybinding.altKey, keybinding.metaKey, this._getUILabelForKeybinding(keybinding), this._getAriaLabelForKeybinding(keybinding));
    };
    USLayoutResolvedKeybinding.prototype.getDispatchParts = function () {
        var firstPart = this._firstPart ? USLayoutResolvedKeybinding.getDispatchStr(this._firstPart) : null;
        var chordPart = this._chordPart ? USLayoutResolvedKeybinding.getDispatchStr(this._chordPart) : null;
        return [firstPart, chordPart];
    };
    USLayoutResolvedKeybinding.getDispatchStr = function (keybinding) {
        if (keybinding.isModifierKey()) {
            return null;
        }
        var result = '';
        if (keybinding.ctrlKey) {
            result += 'ctrl+';
        }
        if (keybinding.shiftKey) {
            result += 'shift+';
        }
        if (keybinding.altKey) {
            result += 'alt+';
        }
        if (keybinding.metaKey) {
            result += 'meta+';
        }
        result += KeyCodeUtils.toString(keybinding.keyCode);
        return result;
    };
    return USLayoutResolvedKeybinding;
}(ResolvedKeybinding));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IWorkspaceContextService = createDecorator('contextService');
var IWorkspace;
(function (IWorkspace) {
    function isIWorkspace(thing) {
        return thing && typeof thing === 'object'
            && typeof thing.id === 'string'
            && Array.isArray(thing.folders);
    }
    IWorkspace.isIWorkspace = isIWorkspace;
})(IWorkspace || (IWorkspace = {}));
var IWorkspaceFolder;
(function (IWorkspaceFolder) {
    function isIWorkspaceFolder(thing) {
        return thing && typeof thing === 'object'
            && URI.isUri(thing.uri)
            && typeof thing.name === 'string'
            && typeof thing.toResource === 'function';
    }
    IWorkspaceFolder.isIWorkspaceFolder = isIWorkspaceFolder;
})(IWorkspaceFolder || (IWorkspaceFolder = {}));
var Workspace = /** @class */ (function () {
    function Workspace(_id, folders, _configuration) {
        if (folders === void 0) { folders = []; }
        if (_configuration === void 0) { _configuration = null; }
        this._id = _id;
        this._configuration = _configuration;
        this._foldersMap = TernarySearchTree.forPaths();
        this.folders = folders;
    }
    Object.defineProperty(Workspace.prototype, "folders", {
        get: function () {
            return this._folders;
        },
        set: function (folders) {
            this._folders = folders;
            this.updateFoldersMap();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Workspace.prototype, "id", {
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Workspace.prototype, "configuration", {
        get: function () {
            return this._configuration;
        },
        set: function (configuration) {
            this._configuration = configuration;
        },
        enumerable: true,
        configurable: true
    });
    Workspace.prototype.getFolder = function (resource) {
        if (!resource) {
            return null;
        }
        return this._foldersMap.findSubstr(resource.toString());
    };
    Workspace.prototype.updateFoldersMap = function () {
        this._foldersMap = TernarySearchTree.forPaths();
        for (var _i = 0, _a = this.folders; _i < _a.length; _i++) {
            var folder = _a[_i];
            this._foldersMap.set(folder.uri.toString(), folder);
        }
    };
    Workspace.prototype.toJSON = function () {
        return { id: this.id, folders: this.folders, configuration: this.configuration };
    };
    return Workspace;
}());
var WorkspaceFolder = /** @class */ (function () {
    function WorkspaceFolder(data, raw) {
        this.raw = raw;
        this.uri = data.uri;
        this.index = data.index;
        this.name = data.name;
    }
    WorkspaceFolder.prototype.toResource = function (relativePath) {
        return joinPath(this.uri, relativePath);
    };
    WorkspaceFolder.prototype.toJSON = function () {
        return { uri: this.uri, name: this.name, index: this.index };
    };
    return WorkspaceFolder;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$24 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$u = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$r = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var SimpleModel = /** @class */ (function () {
    function SimpleModel(model) {
        this.model = model;
        this._onDispose = new Emitter();
    }
    Object.defineProperty(SimpleModel.prototype, "textEditorModel", {
        get: function () {
            return this.model;
        },
        enumerable: true,
        configurable: true
    });
    SimpleModel.prototype.dispose = function () {
        this._onDispose.fire();
    };
    return SimpleModel;
}());
function withTypedEditor(widget, codeEditorCallback, diffEditorCallback) {
    if (isCodeEditor(widget)) {
        // Single Editor
        return codeEditorCallback(widget);
    }
    else {
        // Diff Editor
        return diffEditorCallback(widget);
    }
}
var SimpleEditorModelResolverService = /** @class */ (function () {
    function SimpleEditorModelResolverService() {
    }
    SimpleEditorModelResolverService.prototype.setEditor = function (editor) {
        this.editor = editor;
    };
    SimpleEditorModelResolverService.prototype.createModelReference = function (resource) {
        var _this = this;
        var model;
        model = withTypedEditor(this.editor, function (editor) { return _this.findModel(editor, resource); }, function (diffEditor) { return _this.findModel(diffEditor.getOriginalEditor(), resource) || _this.findModel(diffEditor.getModifiedEditor(), resource); });
        if (!model) {
            return TPromise.as(new ImmortalReference(null));
        }
        return TPromise.as(new ImmortalReference(new SimpleModel(model)));
    };
    SimpleEditorModelResolverService.prototype.findModel = function (editor, resource) {
        var model = editor.getModel();
        if (model && model.uri.toString() !== resource.toString()) {
            return null;
        }
        return model;
    };
    return SimpleEditorModelResolverService;
}());
var SimpleProgressService = /** @class */ (function () {
    function SimpleProgressService() {
    }
    SimpleProgressService.prototype.showWhile = function (promise, delay) {
        return null;
    };
    return SimpleProgressService;
}());
var SimpleDialogService = /** @class */ (function () {
    function SimpleDialogService() {
    }
    return SimpleDialogService;
}());
var SimpleNotificationService = /** @class */ (function () {
    function SimpleNotificationService() {
    }
    SimpleNotificationService.prototype.info = function (message) {
        return this.notify({ severity: Severity$1.Info, message: message });
    };
    SimpleNotificationService.prototype.warn = function (message) {
        return this.notify({ severity: Severity$1.Warning, message: message });
    };
    SimpleNotificationService.prototype.error = function (error) {
        return this.notify({ severity: Severity$1.Error, message: error });
    };
    SimpleNotificationService.prototype.notify = function (notification) {
        switch (notification.severity) {
            case Severity$1.Error:
                console.error(notification.message);
                break;
            case Severity$1.Warning:
                console.warn(notification.message);
                break;
            default:
                console.log(notification.message);
                break;
        }
        return SimpleNotificationService.NO_OP;
    };
    SimpleNotificationService.NO_OP = new NoOpNotification();
    return SimpleNotificationService;
}());
var StandaloneCommandService = /** @class */ (function () {
    function StandaloneCommandService(instantiationService) {
        this._onWillExecuteCommand = new Emitter();
        this._instantiationService = instantiationService;
        this._dynamicCommands = Object.create(null);
    }
    StandaloneCommandService.prototype.addCommand = function (command) {
        var _this = this;
        var id = command.id;
        this._dynamicCommands[id] = command;
        return toDisposable(function () {
            delete _this._dynamicCommands[id];
        });
    };
    StandaloneCommandService.prototype.executeCommand = function (id) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var command = (CommandsRegistry.getCommand(id) || this._dynamicCommands[id]);
        if (!command) {
            return Promise.reject(new Error("command '" + id + "' not found"));
        }
        try {
            this._onWillExecuteCommand.fire({ commandId: id });
            var result = this._instantiationService.invokeFunction.apply(this._instantiationService, [command.handler].concat(args));
            return Promise.resolve(result);
        }
        catch (err) {
            return Promise.reject(err);
        }
    };
    return StandaloneCommandService;
}());
var StandaloneKeybindingService = /** @class */ (function (_super) {
    __extends$24(StandaloneKeybindingService, _super);
    function StandaloneKeybindingService(contextKeyService, commandService, telemetryService, notificationService, domNode) {
        var _this = _super.call(this, contextKeyService, commandService, telemetryService, notificationService) || this;
        _this._cachedResolver = null;
        _this._dynamicKeybindings = [];
        _this._register(addDisposableListener(domNode, EventType.KEY_DOWN, function (e) {
            var keyEvent = new StandardKeyboardEvent(e);
            var shouldPreventDefault = _this._dispatch(keyEvent, keyEvent.target);
            if (shouldPreventDefault) {
                keyEvent.preventDefault();
            }
        }));
        return _this;
    }
    StandaloneKeybindingService.prototype.addDynamicKeybinding = function (commandId, keybinding, handler, when) {
        var _this = this;
        var toDispose = [];
        this._dynamicKeybindings.push({
            keybinding: createKeybinding(keybinding, OS),
            command: commandId,
            when: when,
            weight1: 1000,
            weight2: 0
        });
        toDispose.push(toDisposable(function () {
            for (var i = 0; i < _this._dynamicKeybindings.length; i++) {
                var kb = _this._dynamicKeybindings[i];
                if (kb.command === commandId) {
                    _this._dynamicKeybindings.splice(i, 1);
                    _this.updateResolver({ source: 1 /* Default */ });
                    return;
                }
            }
        }));
        var commandService = this._commandService;
        if (commandService instanceof StandaloneCommandService) {
            toDispose.push(commandService.addCommand({
                id: commandId,
                handler: handler
            }));
        }
        else {
            throw new Error('Unknown command service!');
        }
        this.updateResolver({ source: 1 /* Default */ });
        return combinedDisposable(toDispose);
    };
    StandaloneKeybindingService.prototype.updateResolver = function (event) {
        this._cachedResolver = null;
        this._onDidUpdateKeybindings.fire(event);
    };
    StandaloneKeybindingService.prototype._getResolver = function () {
        if (!this._cachedResolver) {
            var defaults = this._toNormalizedKeybindingItems(KeybindingsRegistry.getDefaultKeybindings(), true);
            var overrides = this._toNormalizedKeybindingItems(this._dynamicKeybindings, false);
            this._cachedResolver = new KeybindingResolver(defaults, overrides);
        }
        return this._cachedResolver;
    };
    StandaloneKeybindingService.prototype._documentHasFocus = function () {
        return document.hasFocus();
    };
    StandaloneKeybindingService.prototype._toNormalizedKeybindingItems = function (items, isDefault) {
        var result = [], resultLen = 0;
        for (var i = 0, len = items.length; i < len; i++) {
            var item = items[i];
            var when = (item.when ? item.when.normalize() : null);
            var keybinding = item.keybinding;
            if (!keybinding) {
                // This might be a removal keybinding item in user settings => accept it
                result[resultLen++] = new ResolvedKeybindingItem(null, item.command, item.commandArgs, when, isDefault);
            }
            else {
                var resolvedKeybindings = this.resolveKeybinding(keybinding);
                for (var j = 0; j < resolvedKeybindings.length; j++) {
                    result[resultLen++] = new ResolvedKeybindingItem(resolvedKeybindings[j], item.command, item.commandArgs, when, isDefault);
                }
            }
        }
        return result;
    };
    StandaloneKeybindingService.prototype.resolveKeybinding = function (keybinding) {
        return [new USLayoutResolvedKeybinding(keybinding, OS)];
    };
    StandaloneKeybindingService.prototype.resolveKeyboardEvent = function (keyboardEvent) {
        var keybinding = new SimpleKeybinding(keyboardEvent.ctrlKey, keyboardEvent.shiftKey, keyboardEvent.altKey, keyboardEvent.metaKey, keyboardEvent.keyCode);
        return new USLayoutResolvedKeybinding(keybinding, OS);
    };
    return StandaloneKeybindingService;
}(AbstractKeybindingService));
function isConfigurationOverrides(thing) {
    return thing
        && typeof thing === 'object'
        && (!thing.overrideIdentifier || typeof thing.overrideIdentifier === 'string')
        && (!thing.resource || thing.resource instanceof URI);
}
var SimpleConfigurationService = /** @class */ (function () {
    function SimpleConfigurationService() {
        this._onDidChangeConfiguration = new Emitter();
        this.onDidChangeConfiguration = this._onDidChangeConfiguration.event;
        this._configuration = new Configuration$1(new DefaultConfigurationModel(), new ConfigurationModel());
    }
    SimpleConfigurationService.prototype.configuration = function () {
        return this._configuration;
    };
    SimpleConfigurationService.prototype.getValue = function (arg1, arg2) {
        var section = typeof arg1 === 'string' ? arg1 : void 0;
        var overrides = isConfigurationOverrides(arg1) ? arg1 : isConfigurationOverrides(arg2) ? arg2 : {};
        return this.configuration().getValue(section, overrides, null);
    };
    SimpleConfigurationService.prototype.updateValue = function (key, value, arg3, arg4) {
        this.configuration().updateValue(key, value);
        return Promise.resolve();
    };
    return SimpleConfigurationService;
}());
var SimpleResourceConfigurationService = /** @class */ (function () {
    function SimpleResourceConfigurationService(configurationService) {
        var _this = this;
        this.configurationService = configurationService;
        this._onDidChangeConfigurationEmitter = new Emitter();
        this.configurationService.onDidChangeConfiguration(function (e) {
            _this._onDidChangeConfigurationEmitter.fire(e);
        });
    }
    SimpleResourceConfigurationService.prototype.getValue = function (resource, arg2, arg3) {
        var position = Position.isIPosition(arg2) ? arg2 : null;
        var section = position ? (typeof arg3 === 'string' ? arg3 : void 0) : (typeof arg2 === 'string' ? arg2 : void 0);
        return this.configurationService.getValue(section);
    };
    return SimpleResourceConfigurationService;
}());
var SimpleResourcePropertiesService = /** @class */ (function () {
    function SimpleResourcePropertiesService(configurationService) {
        this.configurationService = configurationService;
    }
    SimpleResourcePropertiesService.prototype.getEOL = function (resource) {
        var filesConfiguration = this.configurationService.getValue('files');
        if (filesConfiguration && filesConfiguration.eol) {
            if (filesConfiguration.eol !== 'auto') {
                return filesConfiguration.eol;
            }
        }
        return (isLinux || isMacintosh) ? '\n' : '\r\n';
    };
    SimpleResourcePropertiesService = __decorate$u([
        __param$r(0, IConfigurationService)
    ], SimpleResourcePropertiesService);
    return SimpleResourcePropertiesService;
}());
var SimpleMenuService = /** @class */ (function () {
    function SimpleMenuService(commandService) {
        this._commandService = commandService;
    }
    SimpleMenuService.prototype.createMenu = function (id, contextKeyService) {
        return new Menu(id, TPromise.as(true), this._commandService, contextKeyService);
    };
    return SimpleMenuService;
}());
var StandaloneTelemetryService = /** @class */ (function () {
    function StandaloneTelemetryService() {
    }
    StandaloneTelemetryService.prototype.publicLog = function (eventName, data) {
        return TPromise.wrap(null);
    };
    return StandaloneTelemetryService;
}());
var SimpleWorkspaceContextService = /** @class */ (function () {
    function SimpleWorkspaceContextService() {
        var resource = URI.from({ scheme: SimpleWorkspaceContextService.SCHEME, authority: 'model', path: '/' });
        this.workspace = { id: '4064f6ec-cb38-4ad0-af64-ee6467e63c82', folders: [new WorkspaceFolder({ uri: resource, name: '', index: 0 })] };
    }
    SimpleWorkspaceContextService.prototype.getWorkspace = function () {
        return this.workspace;
    };
    SimpleWorkspaceContextService.prototype.getWorkspaceFolder = function (resource) {
        return resource && resource.scheme === SimpleWorkspaceContextService.SCHEME ? this.workspace.folders[0] : void 0;
    };
    SimpleWorkspaceContextService.SCHEME = 'inmemory';
    return SimpleWorkspaceContextService;
}());
function applyConfigurationValues(configurationService, source, isDiffEditor) {
    if (!source) {
        return;
    }
    if (!(configurationService instanceof SimpleConfigurationService)) {
        return;
    }
    Object.keys(source).forEach(function (key) {
        if (isEditorConfigurationKey(key)) {
            configurationService.updateValue("editor." + key, source[key]);
        }
        if (isDiffEditor && isDiffEditorConfigurationKey(key)) {
            configurationService.updateValue("diffEditor." + key, source[key]);
        }
    });
}
var SimpleBulkEditService = /** @class */ (function () {
    function SimpleBulkEditService(_modelService) {
        this._modelService = _modelService;
        //
    }
    SimpleBulkEditService.prototype.apply = function (workspaceEdit, options) {
        var edits = new Map();
        for (var _i = 0, _a = workspaceEdit.edits; _i < _a.length; _i++) {
            var edit = _a[_i];
            if (!isResourceTextEdit(edit)) {
                return Promise.reject(new Error('bad edit - only text edits are supported'));
            }
            var model = this._modelService.getModel(edit.resource);
            if (!model) {
                return Promise.reject(new Error('bad edit - model not found'));
            }
            var array = edits.get(model);
            if (!array) {
                array = [];
            }
            edits.set(model, array.concat(edit.edits));
        }
        var totalEdits = 0;
        var totalFiles = 0;
        edits.forEach(function (edits, model) {
            model.applyEdits(edits.map(function (edit) { return EditOperation.replaceMove(Range.lift(edit.range), edit.text); }));
            totalFiles += 1;
            totalEdits += edits.length;
        });
        return Promise.resolve({
            selection: undefined,
            ariaSummary: localize('summary', 'Made {0} edits in {1} files', totalEdits, totalFiles)
        });
    };
    return SimpleBulkEditService;
}());
var SimpleUriLabelService = /** @class */ (function () {
    function SimpleUriLabelService() {
    }
    SimpleUriLabelService.prototype.getUriLabel = function (resource, options) {
        if (resource.scheme === 'file') {
            return resource.fsPath;
        }
        return resource.path;
    };
    return SimpleUriLabelService;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$25 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$v = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$s = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var LAST_GENERATED_COMMAND_ID = 0;
var ariaDomNodeCreated = false;
function createAriaDomNode() {
    if (ariaDomNodeCreated) {
        return;
    }
    ariaDomNodeCreated = true;
    setARIAContainer(document.body);
}
/**
 * A code editor to be used both by the standalone editor and the standalone diff editor.
 */
var StandaloneCodeEditor = /** @class */ (function (_super) {
    __extends$25(StandaloneCodeEditor, _super);
    function StandaloneCodeEditor(domElement, options, instantiationService, codeEditorService, commandService, contextKeyService, keybindingService, themeService, notificationService) {
        var _this = this;
        options = options || {};
        options.ariaLabel = options.ariaLabel || localize('editorViewAccessibleLabel', "Editor content");
        options.ariaLabel = options.ariaLabel + ';' + (isIE
            ? localize('accessibilityHelpMessageIE', "Press Ctrl+F1 for Accessibility Options.")
            : localize('accessibilityHelpMessage', "Press Alt+F1 for Accessibility Options."));
        _this = _super.call(this, domElement, options, {}, instantiationService, codeEditorService, commandService, contextKeyService, themeService, notificationService) || this;
        if (keybindingService instanceof StandaloneKeybindingService) {
            _this._standaloneKeybindingService = keybindingService;
        }
        // Create the ARIA dom node as soon as the first editor is instantiated
        createAriaDomNode();
        return _this;
    }
    StandaloneCodeEditor.prototype.addCommand = function (keybinding, handler, context) {
        if (!this._standaloneKeybindingService) {
            console.warn('Cannot add command because the editor is configured with an unrecognized KeybindingService');
            return null;
        }
        var commandId = 'DYNAMIC_' + (++LAST_GENERATED_COMMAND_ID);
        var whenExpression = ContextKeyExpr.deserialize(context);
        this._standaloneKeybindingService.addDynamicKeybinding(commandId, keybinding, handler, whenExpression);
        return commandId;
    };
    StandaloneCodeEditor.prototype.createContextKey = function (key, defaultValue) {
        return this._contextKeyService.createKey(key, defaultValue);
    };
    StandaloneCodeEditor.prototype.addAction = function (_descriptor) {
        var _this = this;
        if ((typeof _descriptor.id !== 'string') || (typeof _descriptor.label !== 'string') || (typeof _descriptor.run !== 'function')) {
            throw new Error('Invalid action descriptor, `id`, `label` and `run` are required properties!');
        }
        if (!this._standaloneKeybindingService) {
            console.warn('Cannot add keybinding because the editor is configured with an unrecognized KeybindingService');
            return Disposable.None;
        }
        // Read descriptor options
        var id = _descriptor.id;
        var label = _descriptor.label;
        var precondition = ContextKeyExpr.and(ContextKeyExpr.equals('editorId', this.getId()), ContextKeyExpr.deserialize(_descriptor.precondition));
        var keybindings = _descriptor.keybindings;
        var keybindingsWhen = ContextKeyExpr.and(precondition, ContextKeyExpr.deserialize(_descriptor.keybindingContext));
        var contextMenuGroupId = _descriptor.contextMenuGroupId || null;
        var contextMenuOrder = _descriptor.contextMenuOrder || 0;
        var run = function () {
            return Promise.resolve(_descriptor.run(_this));
        };
        var toDispose = [];
        // Generate a unique id to allow the same descriptor.id across multiple editor instances
        var uniqueId = this.getId() + ':' + id;
        // Register the command
        toDispose.push(CommandsRegistry.registerCommand(uniqueId, run));
        // Register the context menu item
        if (contextMenuGroupId) {
            var menuItem = {
                command: {
                    id: uniqueId,
                    title: label
                },
                when: precondition,
                group: contextMenuGroupId,
                order: contextMenuOrder
            };
            toDispose.push(MenuRegistry.appendMenuItem(MenuId.EditorContext, menuItem));
        }
        // Register the keybindings
        if (Array.isArray(keybindings)) {
            toDispose = toDispose.concat(keybindings.map(function (kb) {
                return _this._standaloneKeybindingService.addDynamicKeybinding(uniqueId, kb, run, keybindingsWhen);
            }));
        }
        // Finally, register an internal editor action
        var internalAction = new InternalEditorAction(uniqueId, label, label, precondition, run, this._contextKeyService);
        // Store it under the original id, such that trigger with the original id will work
        this._actions[id] = internalAction;
        toDispose.push(toDisposable(function () {
            delete _this._actions[id];
        }));
        return combinedDisposable(toDispose);
    };
    StandaloneCodeEditor = __decorate$v([
        __param$s(2, IInstantiationService),
        __param$s(3, ICodeEditorService),
        __param$s(4, ICommandService),
        __param$s(5, IContextKeyService),
        __param$s(6, IKeybindingService),
        __param$s(7, IThemeService),
        __param$s(8, INotificationService)
    ], StandaloneCodeEditor);
    return StandaloneCodeEditor;
}(CodeEditorWidget));
var StandaloneEditor = /** @class */ (function (_super) {
    __extends$25(StandaloneEditor, _super);
    function StandaloneEditor(domElement, options, toDispose, instantiationService, codeEditorService, commandService, contextKeyService, keybindingService, contextViewService, themeService, notificationService, configurationService) {
        var _this = this;
        applyConfigurationValues(configurationService, options, false);
        options = options || {};
        if (typeof options.theme === 'string') {
            themeService.setTheme(options.theme);
        }
        var _model = options.model;
        delete options.model;
        _this = _super.call(this, domElement, options, instantiationService, codeEditorService, commandService, contextKeyService, keybindingService, themeService, notificationService) || this;
        _this._contextViewService = contextViewService;
        _this._configurationService = configurationService;
        _this._register(toDispose);
        var model;
        if (typeof _model === 'undefined') {
            model = self.monaco.editor.createModel(options.value || '', options.language || 'text/plain');
            _this._ownsModel = true;
        }
        else {
            model = _model;
            _this._ownsModel = false;
        }
        _this._attachModel(model);
        if (model) {
            var e = {
                oldModelUrl: null,
                newModelUrl: model.uri
            };
            _this._onDidChangeModel.fire(e);
        }
        return _this;
    }
    StandaloneEditor.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    StandaloneEditor.prototype.updateOptions = function (newOptions) {
        applyConfigurationValues(this._configurationService, newOptions, false);
        _super.prototype.updateOptions.call(this, newOptions);
    };
    StandaloneEditor.prototype._attachModel = function (model) {
        _super.prototype._attachModel.call(this, model);
        if (this._modelData) {
            this._contextViewService.setContainer(this._modelData.view.domNode.domNode);
        }
    };
    StandaloneEditor.prototype._postDetachModelCleanup = function (detachedModel) {
        _super.prototype._postDetachModelCleanup.call(this, detachedModel);
        if (detachedModel && this._ownsModel) {
            detachedModel.dispose();
            this._ownsModel = false;
        }
    };
    StandaloneEditor = __decorate$v([
        __param$s(3, IInstantiationService),
        __param$s(4, ICodeEditorService),
        __param$s(5, ICommandService),
        __param$s(6, IContextKeyService),
        __param$s(7, IKeybindingService),
        __param$s(8, IContextViewService),
        __param$s(9, IStandaloneThemeService),
        __param$s(10, INotificationService),
        __param$s(11, IConfigurationService)
    ], StandaloneEditor);
    return StandaloneEditor;
}(StandaloneCodeEditor));
var StandaloneDiffEditor = /** @class */ (function (_super) {
    __extends$25(StandaloneDiffEditor, _super);
    function StandaloneDiffEditor(domElement, options, toDispose, instantiationService, contextKeyService, keybindingService, contextViewService, editorWorkerService, codeEditorService, themeService, notificationService, configurationService) {
        var _this = this;
        applyConfigurationValues(configurationService, options, true);
        options = options || {};
        if (typeof options.theme === 'string') {
            options.theme = themeService.setTheme(options.theme);
        }
        _this = _super.call(this, domElement, options, editorWorkerService, contextKeyService, instantiationService, codeEditorService, themeService, notificationService) || this;
        _this._contextViewService = contextViewService;
        _this._configurationService = configurationService;
        _this._register(toDispose);
        _this._contextViewService.setContainer(_this._containerDomElement);
        return _this;
    }
    StandaloneDiffEditor.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
    };
    StandaloneDiffEditor.prototype.updateOptions = function (newOptions) {
        applyConfigurationValues(this._configurationService, newOptions, true);
        _super.prototype.updateOptions.call(this, newOptions);
    };
    StandaloneDiffEditor.prototype._createInnerEditor = function (instantiationService, container, options) {
        return instantiationService.createInstance(StandaloneCodeEditor, container, options);
    };
    StandaloneDiffEditor.prototype.getOriginalEditor = function () {
        return _super.prototype.getOriginalEditor.call(this);
    };
    StandaloneDiffEditor.prototype.getModifiedEditor = function () {
        return _super.prototype.getModifiedEditor.call(this);
    };
    StandaloneDiffEditor.prototype.addCommand = function (keybinding, handler, context) {
        return this.getModifiedEditor().addCommand(keybinding, handler, context);
    };
    StandaloneDiffEditor.prototype.createContextKey = function (key, defaultValue) {
        return this.getModifiedEditor().createContextKey(key, defaultValue);
    };
    StandaloneDiffEditor.prototype.addAction = function (descriptor) {
        return this.getModifiedEditor().addAction(descriptor);
    };
    StandaloneDiffEditor = __decorate$v([
        __param$s(3, IInstantiationService),
        __param$s(4, IContextKeyService),
        __param$s(5, IKeybindingService),
        __param$s(6, IContextViewService),
        __param$s(7, IEditorWorkerService),
        __param$s(8, ICodeEditorService),
        __param$s(9, IStandaloneThemeService),
        __param$s(10, INotificationService),
        __param$s(11, IConfigurationService)
    ], StandaloneDiffEditor);
    return StandaloneDiffEditor;
}(DiffEditorWidget));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IBulkEditService = createDecorator('IWorkspaceEditService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var FrankensteinMode = /** @class */ (function () {
    function FrankensteinMode(languageIdentifier) {
        this._languageIdentifier = languageIdentifier;
    }
    FrankensteinMode.prototype.getId = function () {
        return this._languageIdentifier.language;
    };
    return FrankensteinMode;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MIME_TEXT = 'text/plain';
var MIME_UNKNOWN = 'application/unknown';
var registeredAssociations = [];
var nonUserRegisteredAssociations = [];
var userRegisteredAssociations = [];
/**
 * Associate a text mime to the registry.
 */
function registerTextMime(association, warnOnOverwrite) {
    if (warnOnOverwrite === void 0) { warnOnOverwrite = false; }
    // Register
    var associationItem = toTextMimeAssociationItem(association);
    registeredAssociations.push(associationItem);
    if (!associationItem.userConfigured) {
        nonUserRegisteredAssociations.push(associationItem);
    }
    else {
        userRegisteredAssociations.push(associationItem);
    }
    // Check for conflicts unless this is a user configured association
    if (warnOnOverwrite && !associationItem.userConfigured) {
        registeredAssociations.forEach(function (a) {
            if (a.mime === associationItem.mime || a.userConfigured) {
                return; // same mime or userConfigured is ok
            }
            if (associationItem.extension && a.extension === associationItem.extension) {
                console.warn("Overwriting extension <<" + associationItem.extension + ">> to now point to mime <<" + associationItem.mime + ">>");
            }
            if (associationItem.filename && a.filename === associationItem.filename) {
                console.warn("Overwriting filename <<" + associationItem.filename + ">> to now point to mime <<" + associationItem.mime + ">>");
            }
            if (associationItem.filepattern && a.filepattern === associationItem.filepattern) {
                console.warn("Overwriting filepattern <<" + associationItem.filepattern + ">> to now point to mime <<" + associationItem.mime + ">>");
            }
            if (associationItem.firstline && a.firstline === associationItem.firstline) {
                console.warn("Overwriting firstline <<" + associationItem.firstline + ">> to now point to mime <<" + associationItem.mime + ">>");
            }
        });
    }
}
function toTextMimeAssociationItem(association) {
    return {
        id: association.id,
        mime: association.mime,
        filename: association.filename,
        extension: association.extension,
        filepattern: association.filepattern,
        firstline: association.firstline,
        userConfigured: association.userConfigured,
        filenameLowercase: association.filename ? association.filename.toLowerCase() : void 0,
        extensionLowercase: association.extension ? association.extension.toLowerCase() : void 0,
        filepatternLowercase: association.filepattern ? association.filepattern.toLowerCase() : void 0,
        filepatternOnPath: association.filepattern ? association.filepattern.indexOf(sep) >= 0 : false
    };
}
/**
 * Given a file, return the best matching mime type for it
 */
function guessMimeTypes(path, firstLine) {
    if (!path) {
        return [MIME_UNKNOWN];
    }
    path = path.toLowerCase();
    var filename = basename(path);
    // 1.) User configured mappings have highest priority
    var configuredMime = guessMimeTypeByPath(path, filename, userRegisteredAssociations);
    if (configuredMime) {
        return [configuredMime, MIME_TEXT];
    }
    // 2.) Registered mappings have middle priority
    var registeredMime = guessMimeTypeByPath(path, filename, nonUserRegisteredAssociations);
    if (registeredMime) {
        return [registeredMime, MIME_TEXT];
    }
    // 3.) Firstline has lowest priority
    if (firstLine) {
        var firstlineMime = guessMimeTypeByFirstline(firstLine);
        if (firstlineMime) {
            return [firstlineMime, MIME_TEXT];
        }
    }
    return [MIME_UNKNOWN];
}
function guessMimeTypeByPath(path, filename, associations) {
    var filenameMatch = null;
    var patternMatch = null;
    var extensionMatch = null;
    // We want to prioritize associations based on the order they are registered so that the last registered
    // association wins over all other. This is for https://github.com/Microsoft/vscode/issues/20074
    for (var i = associations.length - 1; i >= 0; i--) {
        var association = associations[i];
        // First exact name match
        if (filename === association.filenameLowercase) {
            filenameMatch = association;
            break; // take it!
        }
        // Longest pattern match
        if (association.filepattern) {
            if (!patternMatch || association.filepattern.length > patternMatch.filepattern.length) {
                var target = association.filepatternOnPath ? path : filename; // match on full path if pattern contains path separator
                if (match(association.filepatternLowercase, target)) {
                    patternMatch = association;
                }
            }
        }
        // Longest extension match
        if (association.extension) {
            if (!extensionMatch || association.extension.length > extensionMatch.extension.length) {
                if (endsWith(filename, association.extensionLowercase)) {
                    extensionMatch = association;
                }
            }
        }
    }
    // 1.) Exact name match has second highest prio
    if (filenameMatch) {
        return filenameMatch.mime;
    }
    // 2.) Match on pattern
    if (patternMatch) {
        return patternMatch.mime;
    }
    // 3.) Match on extension comes next
    if (extensionMatch) {
        return extensionMatch.mime;
    }
    return null;
}
function guessMimeTypeByFirstline(firstLine) {
    if (startsWithUTF8BOM(firstLine)) {
        firstLine = firstLine.substr(1);
    }
    if (firstLine.length > 0) {
        for (var i = 0; i < registeredAssociations.length; ++i) {
            var association = registeredAssociations[i];
            if (!association.firstline) {
                continue;
            }
            var matches = firstLine.match(association.firstline);
            if (matches && matches.length > 0) {
                return association.mime;
            }
        }
    }
    return null;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$26 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var hasOwnProperty$2 = Object.prototype.hasOwnProperty;
var LanguagesRegistry = /** @class */ (function (_super) {
    __extends$26(LanguagesRegistry, _super);
    function LanguagesRegistry(useModesRegistry, warnOnOverwrite) {
        if (useModesRegistry === void 0) { useModesRegistry = true; }
        if (warnOnOverwrite === void 0) { warnOnOverwrite = false; }
        var _this = _super.call(this) || this;
        _this._onDidChange = _this._register(new Emitter());
        _this.onDidChange = _this._onDidChange.event;
        _this._nextLanguageId = 1;
        _this._languages = {};
        _this._mimeTypesMap = {};
        _this._nameMap = {};
        _this._lowercaseNameMap = {};
        _this._languageIds = [];
        _this._warnOnOverwrite = warnOnOverwrite;
        if (useModesRegistry) {
            _this._registerLanguages(ModesRegistry.getLanguages());
            _this._register(ModesRegistry.onDidAddLanguages(function (m) { return _this._registerLanguages(m); }));
        }
        return _this;
    }
    LanguagesRegistry.prototype._registerLanguages = function (desc) {
        var _this = this;
        if (desc.length === 0) {
            return;
        }
        for (var i = 0; i < desc.length; i++) {
            this._registerLanguage(desc[i]);
        }
        // Rebuild fast path maps
        this._mimeTypesMap = {};
        this._nameMap = {};
        this._lowercaseNameMap = {};
        Object.keys(this._languages).forEach(function (langId) {
            var language = _this._languages[langId];
            if (language.name) {
                _this._nameMap[language.name] = language.identifier;
            }
            language.aliases.forEach(function (alias) {
                _this._lowercaseNameMap[alias.toLowerCase()] = language.identifier;
            });
            language.mimetypes.forEach(function (mimetype) {
                _this._mimeTypesMap[mimetype] = language.identifier;
            });
        });
        Registry.as(Extensions$3.Configuration).registerOverrideIdentifiers(ModesRegistry.getLanguages().map(function (language) { return language.id; }));
        this._onDidChange.fire();
    };
    LanguagesRegistry.prototype._registerLanguage = function (lang) {
        var langId = lang.id;
        var resolvedLanguage;
        if (hasOwnProperty$2.call(this._languages, langId)) {
            resolvedLanguage = this._languages[langId];
        }
        else {
            var languageId = this._nextLanguageId++;
            resolvedLanguage = {
                identifier: new LanguageIdentifier(langId, languageId),
                name: null,
                mimetypes: [],
                aliases: [],
                extensions: [],
                filenames: [],
                configurationFiles: []
            };
            this._languageIds[languageId] = langId;
            this._languages[langId] = resolvedLanguage;
        }
        this._mergeLanguage(resolvedLanguage, lang);
    };
    LanguagesRegistry.prototype._mergeLanguage = function (resolvedLanguage, lang) {
        var _a;
        var langId = lang.id;
        var primaryMime = null;
        if (Array.isArray(lang.mimetypes) && lang.mimetypes.length > 0) {
            (_a = resolvedLanguage.mimetypes).push.apply(_a, lang.mimetypes);
            primaryMime = lang.mimetypes[0];
        }
        if (!primaryMime) {
            primaryMime = "text/x-" + langId;
            resolvedLanguage.mimetypes.push(primaryMime);
        }
        if (Array.isArray(lang.extensions)) {
            for (var _i = 0, _b = lang.extensions; _i < _b.length; _i++) {
                var extension = _b[_i];
                registerTextMime({ id: langId, mime: primaryMime, extension: extension }, this._warnOnOverwrite);
                resolvedLanguage.extensions.push(extension);
            }
        }
        if (Array.isArray(lang.filenames)) {
            for (var _c = 0, _d = lang.filenames; _c < _d.length; _c++) {
                var filename = _d[_c];
                registerTextMime({ id: langId, mime: primaryMime, filename: filename }, this._warnOnOverwrite);
                resolvedLanguage.filenames.push(filename);
            }
        }
        if (Array.isArray(lang.filenamePatterns)) {
            for (var _e = 0, _f = lang.filenamePatterns; _e < _f.length; _e++) {
                var filenamePattern = _f[_e];
                registerTextMime({ id: langId, mime: primaryMime, filepattern: filenamePattern }, this._warnOnOverwrite);
            }
        }
        if (typeof lang.firstLine === 'string' && lang.firstLine.length > 0) {
            var firstLineRegexStr = lang.firstLine;
            if (firstLineRegexStr.charAt(0) !== '^') {
                firstLineRegexStr = '^' + firstLineRegexStr;
            }
            try {
                var firstLineRegex = new RegExp(firstLineRegexStr);
                if (!regExpLeadsToEndlessLoop(firstLineRegex)) {
                    registerTextMime({ id: langId, mime: primaryMime, firstline: firstLineRegex }, this._warnOnOverwrite);
                }
            }
            catch (err) {
                // Most likely, the regex was bad
                onUnexpectedError(err);
            }
        }
        resolvedLanguage.aliases.push(langId);
        var langAliases = null;
        if (typeof lang.aliases !== 'undefined' && Array.isArray(lang.aliases)) {
            if (lang.aliases.length === 0) {
                // signal that this language should not get a name
                langAliases = [null];
            }
            else {
                langAliases = lang.aliases;
            }
        }
        if (langAliases !== null) {
            for (var i = 0; i < langAliases.length; i++) {
                var langAlias = langAliases[i];
                if (!langAlias || langAlias.length === 0) {
                    continue;
                }
                resolvedLanguage.aliases.push(langAlias);
            }
        }
        var containsAliases = (langAliases !== null && langAliases.length > 0);
        if (containsAliases && langAliases[0] === null) ;
        else {
            var bestName = (containsAliases ? langAliases[0] : null) || langId;
            if (containsAliases || !resolvedLanguage.name) {
                resolvedLanguage.name = bestName;
            }
        }
        if (lang.configuration) {
            resolvedLanguage.configurationFiles.push(lang.configuration);
        }
    };
    LanguagesRegistry.prototype.isRegisteredMode = function (mimetypeOrModeId) {
        // Is this a known mime type ?
        if (hasOwnProperty$2.call(this._mimeTypesMap, mimetypeOrModeId)) {
            return true;
        }
        // Is this a known mode id ?
        return hasOwnProperty$2.call(this._languages, mimetypeOrModeId);
    };
    LanguagesRegistry.prototype.getModeIdForLanguageNameLowercase = function (languageNameLower) {
        if (!hasOwnProperty$2.call(this._lowercaseNameMap, languageNameLower)) {
            return null;
        }
        return this._lowercaseNameMap[languageNameLower].language;
    };
    LanguagesRegistry.prototype.extractModeIds = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
        var _this = this;
        if (!commaSeparatedMimetypesOrCommaSeparatedIds) {
            return [];
        }
        return (commaSeparatedMimetypesOrCommaSeparatedIds.
            split(',').
            map(function (mimeTypeOrId) { return mimeTypeOrId.trim(); }).
            map(function (mimeTypeOrId) {
            if (hasOwnProperty$2.call(_this._mimeTypesMap, mimeTypeOrId)) {
                return _this._mimeTypesMap[mimeTypeOrId].language;
            }
            return mimeTypeOrId;
        }).
            filter(function (modeId) {
            return hasOwnProperty$2.call(_this._languages, modeId);
        }));
    };
    LanguagesRegistry.prototype.getLanguageIdentifier = function (_modeId) {
        if (_modeId === NULL_MODE_ID || _modeId === 0 /* Null */) {
            return NULL_LANGUAGE_IDENTIFIER;
        }
        var modeId;
        if (typeof _modeId === 'string') {
            modeId = _modeId;
        }
        else {
            modeId = this._languageIds[_modeId];
            if (!modeId) {
                return null;
            }
        }
        if (!hasOwnProperty$2.call(this._languages, modeId)) {
            return null;
        }
        return this._languages[modeId].identifier;
    };
    LanguagesRegistry.prototype.getModeIdsFromFilepathOrFirstLine = function (filepath, firstLine) {
        if (!filepath && !firstLine) {
            return [];
        }
        var mimeTypes = guessMimeTypes(filepath, firstLine);
        return this.extractModeIds(mimeTypes.join(','));
    };
    return LanguagesRegistry;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$27 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var LanguageSelection = /** @class */ (function (_super) {
    __extends$27(LanguageSelection, _super);
    function LanguageSelection(onLanguagesMaybeChanged, selector) {
        var _this = _super.call(this) || this;
        _this._onDidChange = _this._register(new Emitter());
        _this.onDidChange = _this._onDidChange.event;
        _this._selector = selector;
        _this.languageIdentifier = _this._selector();
        _this._register(onLanguagesMaybeChanged(function () { return _this._evaluate(); }));
        return _this;
    }
    LanguageSelection.prototype._evaluate = function () {
        var languageIdentifier = this._selector();
        if (languageIdentifier.id === this.languageIdentifier.id) {
            // no change
            return;
        }
        this.languageIdentifier = languageIdentifier;
        this._onDidChange.fire(this.languageIdentifier);
    };
    return LanguageSelection;
}(Disposable));
var ModeServiceImpl = /** @class */ (function () {
    function ModeServiceImpl(warnOnOverwrite) {
        if (warnOnOverwrite === void 0) { warnOnOverwrite = false; }
        var _this = this;
        this._onDidCreateMode = new Emitter();
        this.onDidCreateMode = this._onDidCreateMode.event;
        this._onLanguagesMaybeChanged = new Emitter();
        this.onLanguagesMaybeChanged = this._onLanguagesMaybeChanged.event;
        this._instantiatedModes = {};
        this._registry = new LanguagesRegistry(true, warnOnOverwrite);
        this._registry.onDidChange(function () { return _this._onLanguagesMaybeChanged.fire(); });
    }
    ModeServiceImpl.prototype.isRegisteredMode = function (mimetypeOrModeId) {
        return this._registry.isRegisteredMode(mimetypeOrModeId);
    };
    ModeServiceImpl.prototype.getModeIdForLanguageName = function (alias) {
        return this._registry.getModeIdForLanguageNameLowercase(alias);
    };
    ModeServiceImpl.prototype.getModeIdByFilepathOrFirstLine = function (filepath, firstLine) {
        var modeIds = this._registry.getModeIdsFromFilepathOrFirstLine(filepath, firstLine);
        if (modeIds.length > 0) {
            return modeIds[0];
        }
        return null;
    };
    ModeServiceImpl.prototype.getModeId = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
        var modeIds = this._registry.extractModeIds(commaSeparatedMimetypesOrCommaSeparatedIds);
        if (modeIds.length > 0) {
            return modeIds[0];
        }
        return null;
    };
    ModeServiceImpl.prototype.getLanguageIdentifier = function (modeId) {
        return this._registry.getLanguageIdentifier(modeId);
    };
    // --- instantiation
    ModeServiceImpl.prototype.create = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
        var _this = this;
        return new LanguageSelection(this.onLanguagesMaybeChanged, function () {
            var modeId = _this.getModeId(commaSeparatedMimetypesOrCommaSeparatedIds);
            return _this._createModeAndGetLanguageIdentifier(modeId);
        });
    };
    ModeServiceImpl.prototype.createByFilepathOrFirstLine = function (filepath, firstLine) {
        var _this = this;
        return new LanguageSelection(this.onLanguagesMaybeChanged, function () {
            var modeId = _this.getModeIdByFilepathOrFirstLine(filepath, firstLine);
            return _this._createModeAndGetLanguageIdentifier(modeId);
        });
    };
    ModeServiceImpl.prototype._createModeAndGetLanguageIdentifier = function (modeId) {
        // Fall back to plain text if no mode was found
        var languageIdentifier = this.getLanguageIdentifier(modeId || 'plaintext') || NULL_LANGUAGE_IDENTIFIER;
        this._getOrCreateMode(languageIdentifier.language);
        return languageIdentifier;
    };
    ModeServiceImpl.prototype.triggerMode = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
        var modeId = this.getModeId(commaSeparatedMimetypesOrCommaSeparatedIds);
        // Fall back to plain text if no mode was found
        this._getOrCreateMode(modeId || 'plaintext');
    };
    ModeServiceImpl.prototype._getOrCreateMode = function (modeId) {
        if (!this._instantiatedModes.hasOwnProperty(modeId)) {
            var languageIdentifier = this.getLanguageIdentifier(modeId) || NULL_LANGUAGE_IDENTIFIER;
            this._instantiatedModes[modeId] = new FrankensteinMode(languageIdentifier);
            this._onDidCreateMode.fire(this._instantiatedModes[modeId]);
        }
        return this._instantiatedModes[modeId];
    };
    return ModeServiceImpl;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$28 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$w = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$t = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
function MODEL_ID$1(resource) {
    return resource.toString();
}
var ModelData$1 = /** @class */ (function () {
    function ModelData(model, onWillDispose, onDidChangeLanguage) {
        this.model = model;
        this._languageSelection = null;
        this._languageSelectionListener = null;
        this._markerDecorations = [];
        this._modelEventListeners = [];
        this._modelEventListeners.push(model.onWillDispose(function () { return onWillDispose(model); }));
        this._modelEventListeners.push(model.onDidChangeLanguage(function (e) { return onDidChangeLanguage(model, e); }));
    }
    ModelData.prototype._disposeLanguageSelection = function () {
        if (this._languageSelectionListener) {
            this._languageSelectionListener.dispose();
            this._languageSelectionListener = null;
        }
        if (this._languageSelection) {
            this._languageSelection.dispose();
            this._languageSelection = null;
        }
    };
    ModelData.prototype.dispose = function () {
        this._markerDecorations = this.model.deltaDecorations(this._markerDecorations, []);
        this._modelEventListeners = dispose(this._modelEventListeners);
        this._disposeLanguageSelection();
    };
    ModelData.prototype.acceptMarkerDecorations = function (newDecorations) {
        this._markerDecorations = this.model.deltaDecorations(this._markerDecorations, newDecorations);
    };
    ModelData.prototype.setLanguage = function (languageSelection) {
        var _this = this;
        this._disposeLanguageSelection();
        this._languageSelection = languageSelection;
        this._languageSelectionListener = this._languageSelection.onDidChange(function () { return _this.model.setMode(languageSelection.languageIdentifier); });
        this.model.setMode(languageSelection.languageIdentifier);
    };
    return ModelData;
}());
var ModelMarkerHandler = /** @class */ (function () {
    function ModelMarkerHandler() {
    }
    ModelMarkerHandler.setMarkers = function (modelData, markerService) {
        // Limit to the first 500 errors/warnings
        var markers = markerService.read({ resource: modelData.model.uri, take: 500 });
        var newModelDecorations = markers.map(function (marker) {
            return {
                range: ModelMarkerHandler._createDecorationRange(modelData.model, marker),
                options: ModelMarkerHandler._createDecorationOption(marker)
            };
        });
        modelData.acceptMarkerDecorations(newModelDecorations);
    };
    ModelMarkerHandler._createDecorationRange = function (model, rawMarker) {
        var ret = Range.lift(rawMarker);
        if (rawMarker.severity === MarkerSeverity.Hint) {
            if (!rawMarker.tags || rawMarker.tags.indexOf(1 /* Unnecessary */) === -1) {
                // * never render hints on multiple lines
                // * make enough space for three dots
                ret = ret.setEndPosition(ret.startLineNumber, ret.startColumn + 2);
            }
        }
        ret = model.validateRange(ret);
        if (ret.isEmpty()) {
            var word = model.getWordAtPosition(ret.getStartPosition());
            if (word) {
                ret = new Range(ret.startLineNumber, word.startColumn, ret.endLineNumber, word.endColumn);
            }
            else {
                var maxColumn = model.getLineLastNonWhitespaceColumn(ret.startLineNumber) ||
                    model.getLineMaxColumn(ret.startLineNumber);
                if (maxColumn === 1) ;
                else if (ret.endColumn >= maxColumn) {
                    // behind eol
                    ret = new Range(ret.startLineNumber, maxColumn - 1, ret.endLineNumber, maxColumn);
                }
                else {
                    // extend marker to width = 1
                    ret = new Range(ret.startLineNumber, ret.startColumn, ret.endLineNumber, ret.endColumn + 1);
                }
            }
        }
        else if (rawMarker.endColumn === Number.MAX_VALUE && rawMarker.startColumn === 1 && ret.startLineNumber === ret.endLineNumber) {
            var minColumn = model.getLineFirstNonWhitespaceColumn(rawMarker.startLineNumber);
            if (minColumn < ret.endColumn) {
                ret = new Range(ret.startLineNumber, minColumn, ret.endLineNumber, ret.endColumn);
                rawMarker.startColumn = minColumn;
            }
        }
        return ret;
    };
    ModelMarkerHandler._createDecorationOption = function (marker) {
        var className;
        var color = undefined;
        var zIndex;
        var inlineClassName = undefined;
        switch (marker.severity) {
            case MarkerSeverity.Hint:
                if (marker.tags && marker.tags.indexOf(1 /* Unnecessary */) >= 0) {
                    className = "squiggly-unnecessary" /* EditorUnnecessaryDecoration */;
                }
                else {
                    className = "squiggly-hint" /* EditorHintDecoration */;
                }
                zIndex = 0;
                break;
            case MarkerSeverity.Warning:
                className = "squiggly-warning" /* EditorWarningDecoration */;
                color = themeColorFromId(overviewRulerWarning);
                zIndex = 20;
                break;
            case MarkerSeverity.Info:
                className = "squiggly-info" /* EditorInfoDecoration */;
                color = themeColorFromId(overviewRulerInfo);
                zIndex = 10;
                break;
            case MarkerSeverity.Error:
            default:
                className = "squiggly-error" /* EditorErrorDecoration */;
                color = themeColorFromId(overviewRulerError);
                zIndex = 30;
                break;
        }
        if (marker.tags) {
            if (marker.tags.indexOf(1 /* Unnecessary */) !== -1) {
                inlineClassName = "squiggly-inline-unnecessary" /* EditorUnnecessaryInlineDecoration */;
            }
        }
        var hoverMessage = null;
        var message = marker.message, source = marker.source, relatedInformation = marker.relatedInformation, code = marker.code;
        if (typeof message === 'string') {
            message = message.trim();
            if (source) {
                if (/\n/g.test(message)) {
                    if (code) {
                        message = localize('diagAndSourceAndCodeMultiline', "[{0}]\n{1} [{2}]", source, message, code);
                    }
                    else {
                        message = localize('diagAndSourceMultiline', "[{0}]\n{1}", source, message);
                    }
                }
                else {
                    if (code) {
                        message = localize('diagAndSourceAndCode', "[{0}] {1} [{2}]", source, message, code);
                    }
                    else {
                        message = localize('diagAndSource', "[{0}] {1}", source, message);
                    }
                }
            }
            hoverMessage = new MarkdownString().appendCodeblock('_', message);
            if (!isFalsyOrEmpty(relatedInformation)) {
                hoverMessage.appendMarkdown('\n');
                for (var _i = 0, _a = relatedInformation; _i < _a.length; _i++) {
                    var _b = _a[_i], message_1 = _b.message, resource = _b.resource, startLineNumber = _b.startLineNumber, startColumn = _b.startColumn;
                    hoverMessage.appendMarkdown("* [" + basename(resource.path) + "(" + startLineNumber + ", " + startColumn + ")](" + resource.toString(false) + "#" + startLineNumber + "," + startColumn + "): ");
                    hoverMessage.appendText("" + message_1);
                    hoverMessage.appendMarkdown('\n');
                }
                hoverMessage.appendMarkdown('\n');
            }
        }
        return {
            stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
            className: className,
            hoverMessage: hoverMessage,
            showIfCollapsed: true,
            overviewRuler: {
                color: color,
                position: OverviewRulerLane.Right
            },
            zIndex: zIndex,
            inlineClassName: inlineClassName,
        };
    };
    return ModelMarkerHandler;
}());
var DEFAULT_EOL = (isLinux || isMacintosh) ? 1 /* LF */ : 2 /* CRLF */;
var ModelServiceImpl = /** @class */ (function (_super) {
    __extends$28(ModelServiceImpl, _super);
    function ModelServiceImpl(markerService, configurationService, resourcePropertiesService) {
        var _this = _super.call(this) || this;
        _this._onModelAdded = _this._register(new Emitter());
        _this.onModelAdded = _this._onModelAdded.event;
        _this._onModelRemoved = _this._register(new Emitter());
        _this.onModelRemoved = _this._onModelRemoved.event;
        _this._onModelModeChanged = _this._register(new Emitter());
        _this.onModelModeChanged = _this._onModelModeChanged.event;
        _this._markerService = markerService;
        _this._configurationService = configurationService;
        _this._resourcePropertiesService = resourcePropertiesService;
        _this._models = {};
        _this._modelCreationOptionsByLanguageAndResource = Object.create(null);
        if (_this._markerService) {
            _this._markerServiceSubscription = _this._markerService.onMarkerChanged(_this._handleMarkerChange, _this);
        }
        _this._configurationServiceSubscription = _this._configurationService.onDidChangeConfiguration(function (e) { return _this._updateModelOptions(); });
        _this._updateModelOptions();
        return _this;
    }
    ModelServiceImpl._readModelOptions = function (config, isForSimpleWidget) {
        var tabSize = EDITOR_MODEL_DEFAULTS.tabSize;
        if (config.editor && typeof config.editor.tabSize !== 'undefined') {
            var parsedTabSize = parseInt(config.editor.tabSize, 10);
            if (!isNaN(parsedTabSize)) {
                tabSize = parsedTabSize;
            }
            if (tabSize < 1) {
                tabSize = 1;
            }
        }
        var insertSpaces = EDITOR_MODEL_DEFAULTS.insertSpaces;
        if (config.editor && typeof config.editor.insertSpaces !== 'undefined') {
            insertSpaces = (config.editor.insertSpaces === 'false' ? false : Boolean(config.editor.insertSpaces));
        }
        var newDefaultEOL = DEFAULT_EOL;
        var eol = config.eol;
        if (eol === '\r\n') {
            newDefaultEOL = 2 /* CRLF */;
        }
        else if (eol === '\n') {
            newDefaultEOL = 1 /* LF */;
        }
        var trimAutoWhitespace = EDITOR_MODEL_DEFAULTS.trimAutoWhitespace;
        if (config.editor && typeof config.editor.trimAutoWhitespace !== 'undefined') {
            trimAutoWhitespace = (config.editor.trimAutoWhitespace === 'false' ? false : Boolean(config.editor.trimAutoWhitespace));
        }
        var detectIndentation = EDITOR_MODEL_DEFAULTS.detectIndentation;
        if (config.editor && typeof config.editor.detectIndentation !== 'undefined') {
            detectIndentation = (config.editor.detectIndentation === 'false' ? false : Boolean(config.editor.detectIndentation));
        }
        var largeFileOptimizations = EDITOR_MODEL_DEFAULTS.largeFileOptimizations;
        if (config.editor && typeof config.editor.largeFileOptimizations !== 'undefined') {
            largeFileOptimizations = (config.editor.largeFileOptimizations === 'false' ? false : Boolean(config.editor.largeFileOptimizations));
        }
        return {
            isForSimpleWidget: isForSimpleWidget,
            tabSize: tabSize,
            insertSpaces: insertSpaces,
            detectIndentation: detectIndentation,
            defaultEOL: newDefaultEOL,
            trimAutoWhitespace: trimAutoWhitespace,
            largeFileOptimizations: largeFileOptimizations
        };
    };
    ModelServiceImpl.prototype.getCreationOptions = function (language, resource, isForSimpleWidget) {
        var creationOptions = this._modelCreationOptionsByLanguageAndResource[language + resource];
        if (!creationOptions) {
            var editor = this._configurationService.getValue('editor', { overrideIdentifier: language, resource: resource });
            var eol = this._resourcePropertiesService.getEOL(resource, language);
            creationOptions = ModelServiceImpl._readModelOptions({ editor: editor, eol: eol }, isForSimpleWidget);
            this._modelCreationOptionsByLanguageAndResource[language + resource] = creationOptions;
        }
        return creationOptions;
    };
    ModelServiceImpl.prototype._updateModelOptions = function () {
        var oldOptionsByLanguageAndResource = this._modelCreationOptionsByLanguageAndResource;
        this._modelCreationOptionsByLanguageAndResource = Object.create(null);
        // Update options on all models
        var keys = Object.keys(this._models);
        for (var i = 0, len = keys.length; i < len; i++) {
            var modelId = keys[i];
            var modelData = this._models[modelId];
            var language = modelData.model.getLanguageIdentifier().language;
            var uri = modelData.model.uri;
            var oldOptions = oldOptionsByLanguageAndResource[language + uri];
            var newOptions = this.getCreationOptions(language, uri, modelData.model.isForSimpleWidget);
            ModelServiceImpl._setModelOptionsForModel(modelData.model, newOptions, oldOptions);
        }
    };
    ModelServiceImpl._setModelOptionsForModel = function (model, newOptions, currentOptions) {
        if (currentOptions
            && (currentOptions.detectIndentation === newOptions.detectIndentation)
            && (currentOptions.insertSpaces === newOptions.insertSpaces)
            && (currentOptions.tabSize === newOptions.tabSize)
            && (currentOptions.trimAutoWhitespace === newOptions.trimAutoWhitespace)) {
            // Same indent opts, no need to touch the model
            return;
        }
        if (newOptions.detectIndentation) {
            model.detectIndentation(newOptions.insertSpaces, newOptions.tabSize);
            model.updateOptions({
                trimAutoWhitespace: newOptions.trimAutoWhitespace
            });
        }
        else {
            model.updateOptions({
                insertSpaces: newOptions.insertSpaces,
                tabSize: newOptions.tabSize,
                trimAutoWhitespace: newOptions.trimAutoWhitespace
            });
        }
    };
    ModelServiceImpl.prototype.dispose = function () {
        if (this._markerServiceSubscription) {
            this._markerServiceSubscription.dispose();
        }
        this._configurationServiceSubscription.dispose();
        _super.prototype.dispose.call(this);
    };
    ModelServiceImpl.prototype._handleMarkerChange = function (changedResources) {
        var _this = this;
        changedResources.forEach(function (resource) {
            var modelId = MODEL_ID$1(resource);
            var modelData = _this._models[modelId];
            if (!modelData) {
                return;
            }
            ModelMarkerHandler.setMarkers(modelData, _this._markerService);
        });
    };
    ModelServiceImpl.prototype._cleanUp = function (model) {
        var _this = this;
        // clean up markers for internal, transient models
        if (model.uri.scheme === Schemas.inMemory
            || model.uri.scheme === Schemas.internal
            || model.uri.scheme === Schemas.vscode) {
            if (this._markerService) {
                this._markerService.read({ resource: model.uri }).map(function (marker) { return marker.owner; }).forEach(function (owner) { return _this._markerService.remove(owner, [model.uri]); });
            }
        }
        // clean up cache
        delete this._modelCreationOptionsByLanguageAndResource[model.getLanguageIdentifier().language + model.uri];
    };
    // --- begin IModelService
    ModelServiceImpl.prototype._createModelData = function (value, languageIdentifier, resource, isForSimpleWidget) {
        var _this = this;
        // create & save the model
        var options = this.getCreationOptions(languageIdentifier.language, resource, isForSimpleWidget);
        var model = new TextModel(value, options, languageIdentifier, resource);
        var modelId = MODEL_ID$1(model.uri);
        if (this._models[modelId]) {
            // There already exists a model with this id => this is a programmer error
            throw new Error('ModelService: Cannot add model because it already exists!');
        }
        var modelData = new ModelData$1(model, function (model) { return _this._onWillDispose(model); }, function (model, e) { return _this._onDidChangeLanguage(model, e); });
        this._models[modelId] = modelData;
        return modelData;
    };
    ModelServiceImpl.prototype.createModel = function (value, languageSelection, resource, isForSimpleWidget) {
        if (isForSimpleWidget === void 0) { isForSimpleWidget = false; }
        var modelData;
        if (languageSelection) {
            modelData = this._createModelData(value, languageSelection.languageIdentifier, resource, isForSimpleWidget);
            this.setMode(modelData.model, languageSelection);
        }
        else {
            modelData = this._createModelData(value, PLAINTEXT_LANGUAGE_IDENTIFIER, resource, isForSimpleWidget);
        }
        // handle markers (marker service => model)
        if (this._markerService) {
            ModelMarkerHandler.setMarkers(modelData, this._markerService);
        }
        this._onModelAdded.fire(modelData.model);
        return modelData.model;
    };
    ModelServiceImpl.prototype.setMode = function (model, languageSelection) {
        if (!languageSelection) {
            return;
        }
        var modelData = this._models[MODEL_ID$1(model.uri)];
        if (!modelData) {
            return;
        }
        modelData.setLanguage(languageSelection);
    };
    ModelServiceImpl.prototype.getModels = function () {
        var ret = [];
        var keys = Object.keys(this._models);
        for (var i = 0, len = keys.length; i < len; i++) {
            var modelId = keys[i];
            ret.push(this._models[modelId].model);
        }
        return ret;
    };
    ModelServiceImpl.prototype.getModel = function (resource) {
        var modelId = MODEL_ID$1(resource);
        var modelData = this._models[modelId];
        if (!modelData) {
            return null;
        }
        return modelData.model;
    };
    // --- end IModelService
    ModelServiceImpl.prototype._onWillDispose = function (model) {
        var modelId = MODEL_ID$1(model.uri);
        var modelData = this._models[modelId];
        delete this._models[modelId];
        modelData.dispose();
        this._cleanUp(model);
        this._onModelRemoved.fire(model);
    };
    ModelServiceImpl.prototype._onDidChangeLanguage = function (model, e) {
        var oldModeId = e.oldLanguage;
        var newModeId = model.getLanguageIdentifier().language;
        var oldOptions = this.getCreationOptions(oldModeId, model.uri, model.isForSimpleWidget);
        var newOptions = this.getCreationOptions(newModeId, model.uri, model.isForSimpleWidget);
        ModelServiceImpl._setModelOptionsForModel(model, newOptions, oldOptions);
        this._onModelModeChanged.fire({ model: model, oldModeId: oldModeId });
    };
    ModelServiceImpl = __decorate$w([
        __param$t(0, IMarkerService),
        __param$t(1, IConfigurationService),
        __param$t(2, ITextResourcePropertiesService)
    ], ModelServiceImpl);
    return ModelServiceImpl;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$29 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var AbstractCodeEditorService = /** @class */ (function (_super) {
    __extends$29(AbstractCodeEditorService, _super);
    function AbstractCodeEditorService() {
        var _this = _super.call(this) || this;
        _this._onCodeEditorAdd = _this._register(new Emitter());
        _this.onCodeEditorAdd = _this._onCodeEditorAdd.event;
        _this._onCodeEditorRemove = _this._register(new Emitter());
        _this._onDiffEditorAdd = _this._register(new Emitter());
        _this._onDiffEditorRemove = _this._register(new Emitter());
        _this._codeEditors = Object.create(null);
        _this._diffEditors = Object.create(null);
        return _this;
    }
    AbstractCodeEditorService.prototype.addCodeEditor = function (editor) {
        this._codeEditors[editor.getId()] = editor;
        this._onCodeEditorAdd.fire(editor);
    };
    AbstractCodeEditorService.prototype.removeCodeEditor = function (editor) {
        if (delete this._codeEditors[editor.getId()]) {
            this._onCodeEditorRemove.fire(editor);
        }
    };
    AbstractCodeEditorService.prototype.listCodeEditors = function () {
        var _this = this;
        return Object.keys(this._codeEditors).map(function (id) { return _this._codeEditors[id]; });
    };
    AbstractCodeEditorService.prototype.addDiffEditor = function (editor) {
        this._diffEditors[editor.getId()] = editor;
        this._onDiffEditorAdd.fire(editor);
    };
    AbstractCodeEditorService.prototype.removeDiffEditor = function (editor) {
        if (delete this._diffEditors[editor.getId()]) {
            this._onDiffEditorRemove.fire(editor);
        }
    };
    AbstractCodeEditorService.prototype.listDiffEditors = function () {
        var _this = this;
        return Object.keys(this._diffEditors).map(function (id) { return _this._diffEditors[id]; });
    };
    AbstractCodeEditorService.prototype.getFocusedCodeEditor = function () {
        var editorWithWidgetFocus = null;
        var editors = this.listCodeEditors();
        for (var i = 0; i < editors.length; i++) {
            var editor = editors[i];
            if (editor.hasTextFocus()) {
                // bingo!
                return editor;
            }
            if (editor.hasWidgetFocus()) {
                editorWithWidgetFocus = editor;
            }
        }
        return editorWithWidgetFocus;
    };
    return AbstractCodeEditorService;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2a = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$x = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$u = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var CodeEditorServiceImpl = /** @class */ (function (_super) {
    __extends$2a(CodeEditorServiceImpl, _super);
    function CodeEditorServiceImpl(themeService, styleSheet) {
        if (styleSheet === void 0) { styleSheet = createStyleSheet(); }
        var _this = _super.call(this) || this;
        _this._styleSheet = styleSheet;
        _this._decorationOptionProviders = Object.create(null);
        _this._themeService = themeService;
        return _this;
    }
    CodeEditorServiceImpl.prototype.registerDecorationType = function (key, options, parentTypeKey) {
        var provider = this._decorationOptionProviders[key];
        if (!provider) {
            var providerArgs = {
                styleSheet: this._styleSheet,
                key: key,
                parentTypeKey: parentTypeKey,
                options: options || Object.create(null)
            };
            if (!parentTypeKey) {
                provider = new DecorationTypeOptionsProvider(this._themeService, providerArgs);
            }
            else {
                provider = new DecorationSubTypeOptionsProvider(this._themeService, providerArgs);
            }
            this._decorationOptionProviders[key] = provider;
        }
        provider.refCount++;
    };
    CodeEditorServiceImpl.prototype.removeDecorationType = function (key) {
        var provider = this._decorationOptionProviders[key];
        if (provider) {
            provider.refCount--;
            if (provider.refCount <= 0) {
                delete this._decorationOptionProviders[key];
                provider.dispose();
                this.listCodeEditors().forEach(function (ed) { return ed.removeDecorations(key); });
            }
        }
    };
    CodeEditorServiceImpl.prototype.resolveDecorationOptions = function (decorationTypeKey, writable) {
        var provider = this._decorationOptionProviders[decorationTypeKey];
        if (!provider) {
            throw new Error('Unknown decoration type key: ' + decorationTypeKey);
        }
        return provider.getOptions(this, writable);
    };
    CodeEditorServiceImpl = __decorate$x([
        __param$u(0, IThemeService)
    ], CodeEditorServiceImpl);
    return CodeEditorServiceImpl;
}(AbstractCodeEditorService));
var DecorationSubTypeOptionsProvider = /** @class */ (function () {
    function DecorationSubTypeOptionsProvider(themeService, providerArgs) {
        this._parentTypeKey = providerArgs.parentTypeKey;
        this.refCount = 0;
        this._beforeContentRules = new DecorationCSSRules(3 /* BeforeContentClassName */, providerArgs, themeService);
        this._afterContentRules = new DecorationCSSRules(4 /* AfterContentClassName */, providerArgs, themeService);
    }
    DecorationSubTypeOptionsProvider.prototype.getOptions = function (codeEditorService, writable) {
        var options = codeEditorService.resolveDecorationOptions(this._parentTypeKey, true);
        if (this._beforeContentRules) {
            options.beforeContentClassName = this._beforeContentRules.className;
        }
        if (this._afterContentRules) {
            options.afterContentClassName = this._afterContentRules.className;
        }
        return options;
    };
    DecorationSubTypeOptionsProvider.prototype.dispose = function () {
        if (this._beforeContentRules) {
            this._beforeContentRules.dispose();
            this._beforeContentRules = null;
        }
        if (this._afterContentRules) {
            this._afterContentRules.dispose();
            this._afterContentRules = null;
        }
    };
    return DecorationSubTypeOptionsProvider;
}());
var DecorationTypeOptionsProvider = /** @class */ (function () {
    function DecorationTypeOptionsProvider(themeService, providerArgs) {
        var _this = this;
        this.refCount = 0;
        this._disposables = [];
        var createCSSRules = function (type) {
            var rules = new DecorationCSSRules(type, providerArgs, themeService);
            if (rules.hasContent) {
                _this._disposables.push(rules);
                return rules.className;
            }
            return void 0;
        };
        var createInlineCSSRules = function (type) {
            var rules = new DecorationCSSRules(type, providerArgs, themeService);
            if (rules.hasContent) {
                _this._disposables.push(rules);
                return { className: rules.className, hasLetterSpacing: rules.hasLetterSpacing };
            }
            return null;
        };
        this.className = createCSSRules(0 /* ClassName */);
        var inlineData = createInlineCSSRules(1 /* InlineClassName */);
        if (inlineData) {
            this.inlineClassName = inlineData.className;
            this.inlineClassNameAffectsLetterSpacing = inlineData.hasLetterSpacing;
        }
        this.beforeContentClassName = createCSSRules(3 /* BeforeContentClassName */);
        this.afterContentClassName = createCSSRules(4 /* AfterContentClassName */);
        this.glyphMarginClassName = createCSSRules(2 /* GlyphMarginClassName */);
        var options = providerArgs.options;
        this.isWholeLine = Boolean(options.isWholeLine);
        this.stickiness = options.rangeBehavior;
        var lightOverviewRulerColor = options.light && options.light.overviewRulerColor || options.overviewRulerColor;
        var darkOverviewRulerColor = options.dark && options.dark.overviewRulerColor || options.overviewRulerColor;
        if (typeof lightOverviewRulerColor !== 'undefined'
            || typeof darkOverviewRulerColor !== 'undefined') {
            this.overviewRuler = {
                color: lightOverviewRulerColor || darkOverviewRulerColor,
                darkColor: darkOverviewRulerColor || lightOverviewRulerColor,
                position: options.overviewRulerLane || OverviewRulerLane.Center
            };
        }
    }
    DecorationTypeOptionsProvider.prototype.getOptions = function (codeEditorService, writable) {
        if (!writable) {
            return this;
        }
        return {
            inlineClassName: this.inlineClassName,
            beforeContentClassName: this.beforeContentClassName,
            afterContentClassName: this.afterContentClassName,
            className: this.className,
            glyphMarginClassName: this.glyphMarginClassName,
            isWholeLine: this.isWholeLine,
            overviewRuler: this.overviewRuler,
            stickiness: this.stickiness
        };
    };
    DecorationTypeOptionsProvider.prototype.dispose = function () {
        this._disposables = dispose(this._disposables);
    };
    return DecorationTypeOptionsProvider;
}());
var _CSS_MAP = {
    color: 'color:{0} !important;',
    opacity: 'opacity:{0}; will-change: opacity;',
    backgroundColor: 'background-color:{0};',
    outline: 'outline:{0};',
    outlineColor: 'outline-color:{0};',
    outlineStyle: 'outline-style:{0};',
    outlineWidth: 'outline-width:{0};',
    border: 'border:{0};',
    borderColor: 'border-color:{0};',
    borderRadius: 'border-radius:{0};',
    borderSpacing: 'border-spacing:{0};',
    borderStyle: 'border-style:{0};',
    borderWidth: 'border-width:{0};',
    fontStyle: 'font-style:{0};',
    fontWeight: 'font-weight:{0};',
    textDecoration: 'text-decoration:{0};',
    cursor: 'cursor:{0};',
    letterSpacing: 'letter-spacing:{0};',
    gutterIconPath: 'background:url(\'{0}\') center center no-repeat;',
    gutterIconSize: 'background-size:{0};',
    contentText: 'content:\'{0}\';',
    contentIconPath: 'content:url(\'{0}\');',
    margin: 'margin:{0};',
    width: 'width:{0};',
    height: 'height:{0};'
};
var DecorationCSSRules = /** @class */ (function () {
    function DecorationCSSRules(ruleType, providerArgs, themeService) {
        var _this = this;
        this._theme = themeService.getTheme();
        this._ruleType = ruleType;
        this._providerArgs = providerArgs;
        this._usesThemeColors = false;
        this._hasContent = false;
        this._hasLetterSpacing = false;
        var className = CSSNameHelper.getClassName(this._providerArgs.key, ruleType);
        if (this._providerArgs.parentTypeKey) {
            className = className + ' ' + CSSNameHelper.getClassName(this._providerArgs.parentTypeKey, ruleType);
        }
        this._className = className;
        this._unThemedSelector = CSSNameHelper.getSelector(this._providerArgs.key, this._providerArgs.parentTypeKey, ruleType);
        this._buildCSS();
        if (this._usesThemeColors) {
            this._themeListener = themeService.onThemeChange(function (theme) {
                _this._theme = themeService.getTheme();
                _this._removeCSS();
                _this._buildCSS();
            });
        }
        else {
            this._themeListener = null;
        }
    }
    DecorationCSSRules.prototype.dispose = function () {
        if (this._hasContent) {
            this._removeCSS();
            this._hasContent = false;
        }
        if (this._themeListener) {
            this._themeListener.dispose();
            this._themeListener = null;
        }
    };
    Object.defineProperty(DecorationCSSRules.prototype, "hasContent", {
        get: function () {
            return this._hasContent;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DecorationCSSRules.prototype, "hasLetterSpacing", {
        get: function () {
            return this._hasLetterSpacing;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DecorationCSSRules.prototype, "className", {
        get: function () {
            return this._className;
        },
        enumerable: true,
        configurable: true
    });
    DecorationCSSRules.prototype._buildCSS = function () {
        var options = this._providerArgs.options;
        var unthemedCSS, lightCSS, darkCSS;
        switch (this._ruleType) {
            case 0 /* ClassName */:
                unthemedCSS = this.getCSSTextForModelDecorationClassName(options);
                lightCSS = this.getCSSTextForModelDecorationClassName(options.light);
                darkCSS = this.getCSSTextForModelDecorationClassName(options.dark);
                break;
            case 1 /* InlineClassName */:
                unthemedCSS = this.getCSSTextForModelDecorationInlineClassName(options);
                lightCSS = this.getCSSTextForModelDecorationInlineClassName(options.light);
                darkCSS = this.getCSSTextForModelDecorationInlineClassName(options.dark);
                break;
            case 2 /* GlyphMarginClassName */:
                unthemedCSS = this.getCSSTextForModelDecorationGlyphMarginClassName(options);
                lightCSS = this.getCSSTextForModelDecorationGlyphMarginClassName(options.light);
                darkCSS = this.getCSSTextForModelDecorationGlyphMarginClassName(options.dark);
                break;
            case 3 /* BeforeContentClassName */:
                unthemedCSS = this.getCSSTextForModelDecorationContentClassName(options.before);
                lightCSS = this.getCSSTextForModelDecorationContentClassName(options.light && options.light.before);
                darkCSS = this.getCSSTextForModelDecorationContentClassName(options.dark && options.dark.before);
                break;
            case 4 /* AfterContentClassName */:
                unthemedCSS = this.getCSSTextForModelDecorationContentClassName(options.after);
                lightCSS = this.getCSSTextForModelDecorationContentClassName(options.light && options.light.after);
                darkCSS = this.getCSSTextForModelDecorationContentClassName(options.dark && options.dark.after);
                break;
            default:
                throw new Error('Unknown rule type: ' + this._ruleType);
        }
        var sheet = this._providerArgs.styleSheet.sheet;
        var hasContent = false;
        if (unthemedCSS.length > 0) {
            sheet.insertRule(this._unThemedSelector + " {" + unthemedCSS + "}", 0);
            hasContent = true;
        }
        if (lightCSS.length > 0) {
            sheet.insertRule(".vs" + this._unThemedSelector + " {" + lightCSS + "}", 0);
            hasContent = true;
        }
        if (darkCSS.length > 0) {
            sheet.insertRule(".vs-dark" + this._unThemedSelector + ", .hc-black" + this._unThemedSelector + " {" + darkCSS + "}", 0);
            hasContent = true;
        }
        this._hasContent = hasContent;
    };
    DecorationCSSRules.prototype._removeCSS = function () {
        removeCSSRulesContainingSelector(this._unThemedSelector, this._providerArgs.styleSheet);
    };
    /**
     * Build the CSS for decorations styled via `className`.
     */
    DecorationCSSRules.prototype.getCSSTextForModelDecorationClassName = function (opts) {
        if (!opts) {
            return '';
        }
        var cssTextArr = [];
        this.collectCSSText(opts, ['backgroundColor'], cssTextArr);
        this.collectCSSText(opts, ['outline', 'outlineColor', 'outlineStyle', 'outlineWidth'], cssTextArr);
        this.collectBorderSettingsCSSText(opts, cssTextArr);
        return cssTextArr.join('');
    };
    /**
     * Build the CSS for decorations styled via `inlineClassName`.
     */
    DecorationCSSRules.prototype.getCSSTextForModelDecorationInlineClassName = function (opts) {
        if (!opts) {
            return '';
        }
        var cssTextArr = [];
        this.collectCSSText(opts, ['fontStyle', 'fontWeight', 'textDecoration', 'cursor', 'color', 'opacity', 'letterSpacing'], cssTextArr);
        if (opts.letterSpacing) {
            this._hasLetterSpacing = true;
        }
        return cssTextArr.join('');
    };
    /**
     * Build the CSS for decorations styled before or after content.
     */
    DecorationCSSRules.prototype.getCSSTextForModelDecorationContentClassName = function (opts) {
        if (!opts) {
            return '';
        }
        var cssTextArr = [];
        if (typeof opts !== 'undefined') {
            this.collectBorderSettingsCSSText(opts, cssTextArr);
            if (typeof opts.contentIconPath !== 'undefined') {
                if (typeof opts.contentIconPath === 'string') {
                    cssTextArr.push(format(_CSS_MAP.contentIconPath, URI.file(opts.contentIconPath).toString().replace(/'/g, '%27')));
                }
                else {
                    cssTextArr.push(format(_CSS_MAP.contentIconPath, URI.revive(opts.contentIconPath).toString(true).replace(/'/g, '%27')));
                }
            }
            if (typeof opts.contentText === 'string') {
                var truncated = opts.contentText.match(/^.*$/m)[0]; // only take first line
                var escaped = truncated.replace(/['\\]/g, '\\$&');
                cssTextArr.push(format(_CSS_MAP.contentText, escaped));
            }
            this.collectCSSText(opts, ['fontStyle', 'fontWeight', 'textDecoration', 'color', 'opacity', 'backgroundColor', 'margin'], cssTextArr);
            if (this.collectCSSText(opts, ['width', 'height'], cssTextArr)) {
                cssTextArr.push('display:inline-block;');
            }
        }
        return cssTextArr.join('');
    };
    /**
     * Build the CSS for decorations styled via `glpyhMarginClassName`.
     */
    DecorationCSSRules.prototype.getCSSTextForModelDecorationGlyphMarginClassName = function (opts) {
        if (!opts) {
            return '';
        }
        var cssTextArr = [];
        if (typeof opts.gutterIconPath !== 'undefined') {
            if (typeof opts.gutterIconPath === 'string') {
                cssTextArr.push(format(_CSS_MAP.gutterIconPath, URI.file(opts.gutterIconPath).toString()));
            }
            else {
                cssTextArr.push(format(_CSS_MAP.gutterIconPath, URI.revive(opts.gutterIconPath).toString(true).replace(/'/g, '%27')));
            }
            if (typeof opts.gutterIconSize !== 'undefined') {
                cssTextArr.push(format(_CSS_MAP.gutterIconSize, opts.gutterIconSize));
            }
        }
        return cssTextArr.join('');
    };
    DecorationCSSRules.prototype.collectBorderSettingsCSSText = function (opts, cssTextArr) {
        if (this.collectCSSText(opts, ['border', 'borderColor', 'borderRadius', 'borderSpacing', 'borderStyle', 'borderWidth'], cssTextArr)) {
            cssTextArr.push(format('box-sizing: border-box;'));
            return true;
        }
        return false;
    };
    DecorationCSSRules.prototype.collectCSSText = function (opts, properties, cssTextArr) {
        var lenBefore = cssTextArr.length;
        for (var _i = 0, properties_1 = properties; _i < properties_1.length; _i++) {
            var property = properties_1[_i];
            var value = this.resolveValue(opts[property]);
            if (typeof value === 'string') {
                cssTextArr.push(format(_CSS_MAP[property], value));
            }
        }
        return cssTextArr.length !== lenBefore;
    };
    DecorationCSSRules.prototype.resolveValue = function (value) {
        if (isThemeColor(value)) {
            this._usesThemeColors = true;
            var color = this._theme.getColor(value.id);
            if (color) {
                return color.toString();
            }
            return 'transparent';
        }
        return value;
    };
    return DecorationCSSRules;
}());
var CSSNameHelper = /** @class */ (function () {
    function CSSNameHelper() {
    }
    CSSNameHelper.getClassName = function (key, type) {
        return 'ced-' + key + '-' + type;
    };
    CSSNameHelper.getSelector = function (key, parentKey, ruleType) {
        var selector = '.monaco-editor .' + this.getClassName(key, ruleType);
        if (parentKey) {
            selector = selector + '.' + this.getClassName(parentKey, ruleType);
        }
        if (ruleType === 3 /* BeforeContentClassName */) {
            selector += '::before';
        }
        else if (ruleType === 4 /* AfterContentClassName */) {
            selector += '::after';
        }
        return selector;
    };
    return CSSNameHelper;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2b = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var StandaloneCodeEditorServiceImpl = /** @class */ (function (_super) {
    __extends$2b(StandaloneCodeEditorServiceImpl, _super);
    function StandaloneCodeEditorServiceImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StandaloneCodeEditorServiceImpl.prototype.getActiveCodeEditor = function () {
        return null; // not supported in the standalone case
    };
    StandaloneCodeEditorServiceImpl.prototype.openCodeEditor = function (input, source, sideBySide) {
        if (!source) {
            return Promise.resolve(null);
        }
        return Promise.resolve(this.doOpenEditor(source, input));
    };
    StandaloneCodeEditorServiceImpl.prototype.doOpenEditor = function (editor, input) {
        var model = this.findModel(editor, input.resource);
        if (!model) {
            if (input.resource) {
                var schema = input.resource.scheme;
                if (schema === Schemas.http || schema === Schemas.https) {
                    // This is a fully qualified http or https URL
                    windowOpenNoOpener(input.resource.toString());
                    return editor;
                }
            }
            return null;
        }
        var selection = (input.options ? input.options.selection : null);
        if (selection) {
            if (typeof selection.endLineNumber === 'number' && typeof selection.endColumn === 'number') {
                editor.setSelection(selection);
                editor.revealRangeInCenter(selection, 1 /* Immediate */);
            }
            else {
                var pos = {
                    lineNumber: selection.startLineNumber,
                    column: selection.startColumn
                };
                editor.setPosition(pos);
                editor.revealPositionInCenter(pos, 1 /* Immediate */);
            }
        }
        return editor;
    };
    StandaloneCodeEditorServiceImpl.prototype.findModel = function (editor, resource) {
        var model = editor.getModel();
        if (model && model.uri.toString() !== resource.toString()) {
            return null;
        }
        return model;
    };
    return StandaloneCodeEditorServiceImpl;
}(CodeEditorServiceImpl));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ParsedTokenThemeRule = /** @class */ (function () {
    function ParsedTokenThemeRule(token, index, fontStyle, foreground, background) {
        this.token = token;
        this.index = index;
        this.fontStyle = fontStyle;
        this.foreground = foreground;
        this.background = background;
    }
    return ParsedTokenThemeRule;
}());
/**
 * Parse a raw theme into rules.
 */
function parseTokenTheme(source) {
    if (!source || !Array.isArray(source)) {
        return [];
    }
    var result = [], resultLen = 0;
    for (var i = 0, len = source.length; i < len; i++) {
        var entry = source[i];
        var fontStyle = -1 /* NotSet */;
        if (typeof entry.fontStyle === 'string') {
            fontStyle = 0 /* None */;
            var segments = entry.fontStyle.split(' ');
            for (var j = 0, lenJ = segments.length; j < lenJ; j++) {
                var segment = segments[j];
                switch (segment) {
                    case 'italic':
                        fontStyle = fontStyle | 1 /* Italic */;
                        break;
                    case 'bold':
                        fontStyle = fontStyle | 2 /* Bold */;
                        break;
                    case 'underline':
                        fontStyle = fontStyle | 4 /* Underline */;
                        break;
                }
            }
        }
        var foreground = null;
        if (typeof entry.foreground === 'string') {
            foreground = entry.foreground;
        }
        var background = null;
        if (typeof entry.background === 'string') {
            background = entry.background;
        }
        result[resultLen++] = new ParsedTokenThemeRule(entry.token || '', i, fontStyle, foreground, background);
    }
    return result;
}
/**
 * Resolve rules (i.e. inheritance).
 */
function resolveParsedTokenThemeRules(parsedThemeRules, customTokenColors) {
    // Sort rules lexicographically, and then by index if necessary
    parsedThemeRules.sort(function (a, b) {
        var r = strcmp(a.token, b.token);
        if (r !== 0) {
            return r;
        }
        return a.index - b.index;
    });
    // Determine defaults
    var defaultFontStyle = 0 /* None */;
    var defaultForeground = '000000';
    var defaultBackground = 'ffffff';
    while (parsedThemeRules.length >= 1 && parsedThemeRules[0].token === '') {
        var incomingDefaults = parsedThemeRules.shift();
        if (incomingDefaults.fontStyle !== -1 /* NotSet */) {
            defaultFontStyle = incomingDefaults.fontStyle;
        }
        if (incomingDefaults.foreground !== null) {
            defaultForeground = incomingDefaults.foreground;
        }
        if (incomingDefaults.background !== null) {
            defaultBackground = incomingDefaults.background;
        }
    }
    var colorMap = new ColorMap();
    // start with token colors from custom token themes
    for (var _i = 0, customTokenColors_1 = customTokenColors; _i < customTokenColors_1.length; _i++) {
        var color = customTokenColors_1[_i];
        colorMap.getId(color);
    }
    var foregroundColorId = colorMap.getId(defaultForeground);
    var backgroundColorId = colorMap.getId(defaultBackground);
    var defaults = new ThemeTrieElementRule(defaultFontStyle, foregroundColorId, backgroundColorId);
    var root = new ThemeTrieElement(defaults);
    for (var i = 0, len = parsedThemeRules.length; i < len; i++) {
        var rule = parsedThemeRules[i];
        root.insert(rule.token, rule.fontStyle, colorMap.getId(rule.foreground), colorMap.getId(rule.background));
    }
    return new TokenTheme(colorMap, root);
}
var colorRegExp$1 = /^#?([0-9A-Fa-f]{6})([0-9A-Fa-f]{2})?$/;
var ColorMap = /** @class */ (function () {
    function ColorMap() {
        this._lastColorId = 0;
        this._id2color = [];
        this._color2id = new Map();
    }
    ColorMap.prototype.getId = function (color) {
        if (color === null) {
            return 0;
        }
        var match = color.match(colorRegExp$1);
        if (!match) {
            throw new Error('Illegal value for token color: ' + color);
        }
        color = match[1].toUpperCase();
        var value = this._color2id.get(color);
        if (value) {
            return value;
        }
        value = ++this._lastColorId;
        this._color2id.set(color, value);
        this._id2color[value] = Color.fromHex('#' + color);
        return value;
    };
    ColorMap.prototype.getColorMap = function () {
        return this._id2color.slice(0);
    };
    return ColorMap;
}());
var TokenTheme = /** @class */ (function () {
    function TokenTheme(colorMap, root) {
        this._colorMap = colorMap;
        this._root = root;
        this._cache = new Map();
    }
    TokenTheme.createFromRawTokenTheme = function (source, customTokenColors) {
        return this.createFromParsedTokenTheme(parseTokenTheme(source), customTokenColors);
    };
    TokenTheme.createFromParsedTokenTheme = function (source, customTokenColors) {
        return resolveParsedTokenThemeRules(source, customTokenColors);
    };
    TokenTheme.prototype.getColorMap = function () {
        return this._colorMap.getColorMap();
    };
    TokenTheme.prototype._match = function (token) {
        return this._root.match(token);
    };
    TokenTheme.prototype.match = function (languageId, token) {
        // The cache contains the metadata without the language bits set.
        var result = this._cache.get(token);
        if (typeof result === 'undefined') {
            var rule = this._match(token);
            var standardToken = toStandardTokenType(token);
            result = (rule.metadata
                | (standardToken << 8 /* TOKEN_TYPE_OFFSET */)) >>> 0;
            this._cache.set(token, result);
        }
        return (result
            | (languageId << 0 /* LANGUAGEID_OFFSET */)) >>> 0;
    };
    return TokenTheme;
}());
var STANDARD_TOKEN_TYPE_REGEXP = /\b(comment|string|regex)\b/;
function toStandardTokenType(tokenType) {
    var m = tokenType.match(STANDARD_TOKEN_TYPE_REGEXP);
    if (!m) {
        return 0 /* Other */;
    }
    switch (m[1]) {
        case 'comment':
            return 1 /* Comment */;
        case 'string':
            return 2 /* String */;
        case 'regex':
            return 4 /* RegEx */;
    }
    throw new Error('Unexpected match for standard token type!');
}
function strcmp(a, b) {
    if (a < b) {
        return -1;
    }
    if (a > b) {
        return 1;
    }
    return 0;
}
var ThemeTrieElementRule = /** @class */ (function () {
    function ThemeTrieElementRule(fontStyle, foreground, background) {
        this._fontStyle = fontStyle;
        this._foreground = foreground;
        this._background = background;
        this.metadata = ((this._fontStyle << 11 /* FONT_STYLE_OFFSET */)
            | (this._foreground << 14 /* FOREGROUND_OFFSET */)
            | (this._background << 23 /* BACKGROUND_OFFSET */)) >>> 0;
    }
    ThemeTrieElementRule.prototype.clone = function () {
        return new ThemeTrieElementRule(this._fontStyle, this._foreground, this._background);
    };
    ThemeTrieElementRule.prototype.acceptOverwrite = function (fontStyle, foreground, background) {
        if (fontStyle !== -1 /* NotSet */) {
            this._fontStyle = fontStyle;
        }
        if (foreground !== 0 /* None */) {
            this._foreground = foreground;
        }
        if (background !== 0 /* None */) {
            this._background = background;
        }
        this.metadata = ((this._fontStyle << 11 /* FONT_STYLE_OFFSET */)
            | (this._foreground << 14 /* FOREGROUND_OFFSET */)
            | (this._background << 23 /* BACKGROUND_OFFSET */)) >>> 0;
    };
    return ThemeTrieElementRule;
}());
var ThemeTrieElement = /** @class */ (function () {
    function ThemeTrieElement(mainRule) {
        this._mainRule = mainRule;
        this._children = new Map();
    }
    ThemeTrieElement.prototype.match = function (token) {
        if (token === '') {
            return this._mainRule;
        }
        var dotIndex = token.indexOf('.');
        var head;
        var tail;
        if (dotIndex === -1) {
            head = token;
            tail = '';
        }
        else {
            head = token.substring(0, dotIndex);
            tail = token.substring(dotIndex + 1);
        }
        var child = this._children.get(head);
        if (typeof child !== 'undefined') {
            return child.match(tail);
        }
        return this._mainRule;
    };
    ThemeTrieElement.prototype.insert = function (token, fontStyle, foreground, background) {
        if (token === '') {
            // Merge into the main rule
            this._mainRule.acceptOverwrite(fontStyle, foreground, background);
            return;
        }
        var dotIndex = token.indexOf('.');
        var head;
        var tail;
        if (dotIndex === -1) {
            head = token;
            tail = '';
        }
        else {
            head = token.substring(0, dotIndex);
            tail = token.substring(dotIndex + 1);
        }
        var child = this._children.get(head);
        if (typeof child === 'undefined') {
            child = new ThemeTrieElement(this._mainRule.clone());
            this._children.set(head, child);
        }
        child.insert(tail, fontStyle, foreground, background);
    };
    return ThemeTrieElement;
}());
function generateTokensCSSForColorMap(colorMap) {
    var rules = [];
    for (var i = 1, len = colorMap.length; i < len; i++) {
        var color = colorMap[i];
        rules[i] = ".mtk" + i + " { color: " + color + "; }";
    }
    rules.push('.mtki { font-style: italic; }');
    rules.push('.mtkb { font-weight: bold; }');
    rules.push('.mtku { text-decoration: underline; text-underline-position: under; }');
    return rules.join('\n');
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var _a$2, _b, _c;
/* -------------------------------- Begin vs theme -------------------------------- */
var vs = {
    base: 'vs',
    inherit: false,
    rules: [
        { token: '', foreground: '000000', background: 'fffffe' },
        { token: 'invalid', foreground: 'cd3131' },
        { token: 'emphasis', fontStyle: 'italic' },
        { token: 'strong', fontStyle: 'bold' },
        { token: 'variable', foreground: '001188' },
        { token: 'variable.predefined', foreground: '4864AA' },
        { token: 'constant', foreground: 'dd0000' },
        { token: 'comment', foreground: '008000' },
        { token: 'number', foreground: '09885A' },
        { token: 'number.hex', foreground: '3030c0' },
        { token: 'regexp', foreground: '800000' },
        { token: 'annotation', foreground: '808080' },
        { token: 'type', foreground: '008080' },
        { token: 'delimiter', foreground: '000000' },
        { token: 'delimiter.html', foreground: '383838' },
        { token: 'delimiter.xml', foreground: '0000FF' },
        { token: 'tag', foreground: '800000' },
        { token: 'tag.id.pug', foreground: '4F76AC' },
        { token: 'tag.class.pug', foreground: '4F76AC' },
        { token: 'meta.scss', foreground: '800000' },
        { token: 'metatag', foreground: 'e00000' },
        { token: 'metatag.content.html', foreground: 'FF0000' },
        { token: 'metatag.html', foreground: '808080' },
        { token: 'metatag.xml', foreground: '808080' },
        { token: 'metatag.php', fontStyle: 'bold' },
        { token: 'key', foreground: '863B00' },
        { token: 'string.key.json', foreground: 'A31515' },
        { token: 'string.value.json', foreground: '0451A5' },
        { token: 'attribute.name', foreground: 'FF0000' },
        { token: 'attribute.value', foreground: '0451A5' },
        { token: 'attribute.value.number', foreground: '09885A' },
        { token: 'attribute.value.unit', foreground: '09885A' },
        { token: 'attribute.value.html', foreground: '0000FF' },
        { token: 'attribute.value.xml', foreground: '0000FF' },
        { token: 'string', foreground: 'A31515' },
        { token: 'string.html', foreground: '0000FF' },
        { token: 'string.sql', foreground: 'FF0000' },
        { token: 'string.yaml', foreground: '0451A5' },
        { token: 'keyword', foreground: '0000FF' },
        { token: 'keyword.json', foreground: '0451A5' },
        { token: 'keyword.flow', foreground: 'AF00DB' },
        { token: 'keyword.flow.scss', foreground: '0000FF' },
        { token: 'operator.scss', foreground: '666666' },
        { token: 'operator.sql', foreground: '778899' },
        { token: 'operator.swift', foreground: '666666' },
        { token: 'predefined.sql', foreground: 'FF00FF' },
    ],
    colors: (_a$2 = {},
        _a$2[editorBackground] = '#FFFFFE',
        _a$2[editorForeground] = '#000000',
        _a$2[editorInactiveSelection] = '#E5EBF1',
        _a$2[editorIndentGuides] = '#D3D3D3',
        _a$2[editorActiveIndentGuides] = '#939393',
        _a$2[editorSelectionHighlight] = '#ADD6FF4D',
        _a$2)
};
/* -------------------------------- End vs theme -------------------------------- */
/* -------------------------------- Begin vs-dark theme -------------------------------- */
var vs_dark = {
    base: 'vs-dark',
    inherit: false,
    rules: [
        { token: '', foreground: 'D4D4D4', background: '1E1E1E' },
        { token: 'invalid', foreground: 'f44747' },
        { token: 'emphasis', fontStyle: 'italic' },
        { token: 'strong', fontStyle: 'bold' },
        { token: 'variable', foreground: '74B0DF' },
        { token: 'variable.predefined', foreground: '4864AA' },
        { token: 'variable.parameter', foreground: '9CDCFE' },
        { token: 'constant', foreground: '569CD6' },
        { token: 'comment', foreground: '608B4E' },
        { token: 'number', foreground: 'B5CEA8' },
        { token: 'number.hex', foreground: '5BB498' },
        { token: 'regexp', foreground: 'B46695' },
        { token: 'annotation', foreground: 'cc6666' },
        { token: 'type', foreground: '3DC9B0' },
        { token: 'delimiter', foreground: 'DCDCDC' },
        { token: 'delimiter.html', foreground: '808080' },
        { token: 'delimiter.xml', foreground: '808080' },
        { token: 'tag', foreground: '569CD6' },
        { token: 'tag.id.pug', foreground: '4F76AC' },
        { token: 'tag.class.pug', foreground: '4F76AC' },
        { token: 'meta.scss', foreground: 'A79873' },
        { token: 'meta.tag', foreground: 'CE9178' },
        { token: 'metatag', foreground: 'DD6A6F' },
        { token: 'metatag.content.html', foreground: '9CDCFE' },
        { token: 'metatag.html', foreground: '569CD6' },
        { token: 'metatag.xml', foreground: '569CD6' },
        { token: 'metatag.php', fontStyle: 'bold' },
        { token: 'key', foreground: '9CDCFE' },
        { token: 'string.key.json', foreground: '9CDCFE' },
        { token: 'string.value.json', foreground: 'CE9178' },
        { token: 'attribute.name', foreground: '9CDCFE' },
        { token: 'attribute.value', foreground: 'CE9178' },
        { token: 'attribute.value.number.css', foreground: 'B5CEA8' },
        { token: 'attribute.value.unit.css', foreground: 'B5CEA8' },
        { token: 'attribute.value.hex.css', foreground: 'D4D4D4' },
        { token: 'string', foreground: 'CE9178' },
        { token: 'string.sql', foreground: 'FF0000' },
        { token: 'keyword', foreground: '569CD6' },
        { token: 'keyword.flow', foreground: 'C586C0' },
        { token: 'keyword.json', foreground: 'CE9178' },
        { token: 'keyword.flow.scss', foreground: '569CD6' },
        { token: 'operator.scss', foreground: '909090' },
        { token: 'operator.sql', foreground: '778899' },
        { token: 'operator.swift', foreground: '909090' },
        { token: 'predefined.sql', foreground: 'FF00FF' },
    ],
    colors: (_b = {},
        _b[editorBackground] = '#1E1E1E',
        _b[editorForeground] = '#D4D4D4',
        _b[editorInactiveSelection] = '#3A3D41',
        _b[editorIndentGuides] = '#404040',
        _b[editorActiveIndentGuides] = '#707070',
        _b[editorSelectionHighlight] = '#ADD6FF26',
        _b)
};
/* -------------------------------- End vs-dark theme -------------------------------- */
/* -------------------------------- Begin hc-black theme -------------------------------- */
var hc_black = {
    base: 'hc-black',
    inherit: false,
    rules: [
        { token: '', foreground: 'FFFFFF', background: '000000' },
        { token: 'invalid', foreground: 'f44747' },
        { token: 'emphasis', fontStyle: 'italic' },
        { token: 'strong', fontStyle: 'bold' },
        { token: 'variable', foreground: '1AEBFF' },
        { token: 'variable.parameter', foreground: '9CDCFE' },
        { token: 'constant', foreground: '569CD6' },
        { token: 'comment', foreground: '608B4E' },
        { token: 'number', foreground: 'FFFFFF' },
        { token: 'regexp', foreground: 'C0C0C0' },
        { token: 'annotation', foreground: '569CD6' },
        { token: 'type', foreground: '3DC9B0' },
        { token: 'delimiter', foreground: 'FFFF00' },
        { token: 'delimiter.html', foreground: 'FFFF00' },
        { token: 'tag', foreground: '569CD6' },
        { token: 'tag.id.pug', foreground: '4F76AC' },
        { token: 'tag.class.pug', foreground: '4F76AC' },
        { token: 'meta', foreground: 'D4D4D4' },
        { token: 'meta.tag', foreground: 'CE9178' },
        { token: 'metatag', foreground: '569CD6' },
        { token: 'metatag.content.html', foreground: '1AEBFF' },
        { token: 'metatag.html', foreground: '569CD6' },
        { token: 'metatag.xml', foreground: '569CD6' },
        { token: 'metatag.php', fontStyle: 'bold' },
        { token: 'key', foreground: '9CDCFE' },
        { token: 'string.key', foreground: '9CDCFE' },
        { token: 'string.value', foreground: 'CE9178' },
        { token: 'attribute.name', foreground: '569CD6' },
        { token: 'attribute.value', foreground: '3FF23F' },
        { token: 'string', foreground: 'CE9178' },
        { token: 'string.sql', foreground: 'FF0000' },
        { token: 'keyword', foreground: '569CD6' },
        { token: 'keyword.flow', foreground: 'C586C0' },
        { token: 'operator.sql', foreground: '778899' },
        { token: 'operator.swift', foreground: '909090' },
        { token: 'predefined.sql', foreground: 'FF00FF' },
    ],
    colors: (_c = {},
        _c[editorBackground] = '#000000',
        _c[editorForeground] = '#FFFFFF',
        _c[editorIndentGuides] = '#FFFFFF',
        _c[editorActiveIndentGuides] = '#FFFFFF',
        _c)
};
/* -------------------------------- End hc-black theme -------------------------------- */

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var VS_THEME_NAME = 'vs';
var VS_DARK_THEME_NAME = 'vs-dark';
var HC_BLACK_THEME_NAME = 'hc-black';
var colorRegistry$1 = Registry.as(Extensions$5.ColorContribution);
var themingRegistry$1 = Registry.as(Extensions$4.ThemingContribution);
var StandaloneTheme = /** @class */ (function () {
    function StandaloneTheme(name, standaloneThemeData) {
        this.themeData = standaloneThemeData;
        var base = standaloneThemeData.base;
        if (name.length > 0) {
            this.id = base + ' ' + name;
            this.themeName = name;
        }
        else {
            this.id = base;
            this.themeName = base;
        }
        this.colors = null;
        this.defaultColors = Object.create(null);
        this._tokenTheme = null;
    }
    Object.defineProperty(StandaloneTheme.prototype, "base", {
        get: function () {
            return this.themeData.base;
        },
        enumerable: true,
        configurable: true
    });
    StandaloneTheme.prototype.notifyBaseUpdated = function () {
        if (this.themeData.inherit) {
            this.colors = null;
            this._tokenTheme = null;
        }
    };
    StandaloneTheme.prototype.getColors = function () {
        if (!this.colors) {
            var colors = Object.create(null);
            for (var id in this.themeData.colors) {
                colors[id] = Color.fromHex(this.themeData.colors[id]);
            }
            if (this.themeData.inherit) {
                var baseData = getBuiltinRules(this.themeData.base);
                for (var id in baseData.colors) {
                    if (!colors[id]) {
                        colors[id] = Color.fromHex(baseData.colors[id]);
                    }
                }
            }
            this.colors = colors;
        }
        return this.colors;
    };
    StandaloneTheme.prototype.getColor = function (colorId, useDefault) {
        var color = this.getColors()[colorId];
        if (color) {
            return color;
        }
        if (useDefault !== false) {
            return this.getDefault(colorId);
        }
        return null;
    };
    StandaloneTheme.prototype.getDefault = function (colorId) {
        var color = this.defaultColors[colorId];
        if (color) {
            return color;
        }
        color = colorRegistry$1.resolveDefaultColor(colorId, this);
        this.defaultColors[colorId] = color;
        return color;
    };
    StandaloneTheme.prototype.defines = function (colorId) {
        return Object.prototype.hasOwnProperty.call(this.getColors(), colorId);
    };
    Object.defineProperty(StandaloneTheme.prototype, "type", {
        get: function () {
            switch (this.base) {
                case VS_THEME_NAME: return 'light';
                case HC_BLACK_THEME_NAME: return 'hc';
                default: return 'dark';
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StandaloneTheme.prototype, "tokenTheme", {
        get: function () {
            if (!this._tokenTheme) {
                var rules = [];
                var encodedTokensColors = [];
                if (this.themeData.inherit) {
                    var baseData = getBuiltinRules(this.themeData.base);
                    rules = baseData.rules;
                    if (baseData.encodedTokensColors) {
                        encodedTokensColors = baseData.encodedTokensColors;
                    }
                }
                rules = rules.concat(this.themeData.rules);
                if (this.themeData.encodedTokensColors) {
                    encodedTokensColors = this.themeData.encodedTokensColors;
                }
                this._tokenTheme = TokenTheme.createFromRawTokenTheme(rules, encodedTokensColors);
            }
            return this._tokenTheme;
        },
        enumerable: true,
        configurable: true
    });
    return StandaloneTheme;
}());
function isBuiltinTheme(themeName) {
    return (themeName === VS_THEME_NAME
        || themeName === VS_DARK_THEME_NAME
        || themeName === HC_BLACK_THEME_NAME);
}
function getBuiltinRules(builtinTheme) {
    switch (builtinTheme) {
        case VS_THEME_NAME:
            return vs;
        case VS_DARK_THEME_NAME:
            return vs_dark;
        case HC_BLACK_THEME_NAME:
            return hc_black;
    }
}
function newBuiltInTheme(builtinTheme) {
    var themeData = getBuiltinRules(builtinTheme);
    return new StandaloneTheme(builtinTheme, themeData);
}
var StandaloneThemeServiceImpl = /** @class */ (function () {
    function StandaloneThemeServiceImpl() {
        this.environment = Object.create(null);
        this._onThemeChange = new Emitter();
        this._onIconThemeChange = new Emitter();
        this._knownThemes = new Map();
        this._knownThemes.set(VS_THEME_NAME, newBuiltInTheme(VS_THEME_NAME));
        this._knownThemes.set(VS_DARK_THEME_NAME, newBuiltInTheme(VS_DARK_THEME_NAME));
        this._knownThemes.set(HC_BLACK_THEME_NAME, newBuiltInTheme(HC_BLACK_THEME_NAME));
        this._styleElement = createStyleSheet();
        this._styleElement.className = 'monaco-colors';
        this.setTheme(VS_THEME_NAME);
    }
    Object.defineProperty(StandaloneThemeServiceImpl.prototype, "onThemeChange", {
        get: function () {
            return this._onThemeChange.event;
        },
        enumerable: true,
        configurable: true
    });
    StandaloneThemeServiceImpl.prototype.defineTheme = function (themeName, themeData) {
        if (!/^[a-z0-9\-]+$/i.test(themeName)) {
            throw new Error('Illegal theme name!');
        }
        if (!isBuiltinTheme(themeData.base) && !isBuiltinTheme(themeName)) {
            throw new Error('Illegal theme base!');
        }
        // set or replace theme
        this._knownThemes.set(themeName, new StandaloneTheme(themeName, themeData));
        if (isBuiltinTheme(themeName)) {
            this._knownThemes.forEach(function (theme) {
                if (theme.base === themeName) {
                    theme.notifyBaseUpdated();
                }
            });
        }
        if (this._theme && this._theme.themeName === themeName) {
            this.setTheme(themeName); // refresh theme
        }
    };
    StandaloneThemeServiceImpl.prototype.getTheme = function () {
        return this._theme;
    };
    StandaloneThemeServiceImpl.prototype.setTheme = function (themeName) {
        var _this = this;
        var theme;
        if (this._knownThemes.has(themeName)) {
            theme = this._knownThemes.get(themeName);
        }
        else {
            theme = this._knownThemes.get(VS_THEME_NAME);
        }
        if (this._theme === theme) {
            // Nothing to do
            return theme.id;
        }
        this._theme = theme;
        var cssRules = [];
        var hasRule = {};
        var ruleCollector = {
            addRule: function (rule) {
                if (!hasRule[rule]) {
                    cssRules.push(rule);
                    hasRule[rule] = true;
                }
            }
        };
        themingRegistry$1.getThemingParticipants().forEach(function (p) { return p(theme, ruleCollector, _this.environment); });
        var tokenTheme = theme.tokenTheme;
        var colorMap = tokenTheme.getColorMap();
        ruleCollector.addRule(generateTokensCSSForColorMap(colorMap));
        this._styleElement.innerHTML = cssRules.join('\n');
        TokenizationRegistry.setColorMap(colorMap);
        this._onThemeChange.fire(theme);
        return theme.id;
    };
    StandaloneThemeServiceImpl.prototype.getIconTheme = function () {
        return {
            hasFileIcons: false,
            hasFolderIcons: false,
            hidesExplorerArrows: false
        };
    };
    return StandaloneThemeServiceImpl;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2c = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$y = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$v = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var KEYBINDING_CONTEXT_ATTR = 'data-keybinding-context';
var Context$1 = /** @class */ (function () {
    function Context(id, parent) {
        this._id = id;
        this._parent = parent;
        this._value = Object.create(null);
        this._value['_contextId'] = id;
    }
    Context.prototype.setValue = function (key, value) {
        // console.log('SET ' + key + ' = ' + value + ' ON ' + this._id);
        if (this._value[key] !== value) {
            this._value[key] = value;
            return true;
        }
        return false;
    };
    Context.prototype.removeValue = function (key) {
        // console.log('REMOVE ' + key + ' FROM ' + this._id);
        if (key in this._value) {
            delete this._value[key];
            return true;
        }
        return false;
    };
    Context.prototype.getValue = function (key) {
        var ret = this._value[key];
        if (typeof ret === 'undefined' && this._parent) {
            return this._parent.getValue(key);
        }
        return ret;
    };
    return Context;
}());
var NullContext = /** @class */ (function (_super) {
    __extends$2c(NullContext, _super);
    function NullContext() {
        return _super.call(this, -1, null) || this;
    }
    NullContext.prototype.setValue = function (key, value) {
        return false;
    };
    NullContext.prototype.removeValue = function (key) {
        return false;
    };
    NullContext.prototype.getValue = function (key) {
        return undefined;
    };
    NullContext.INSTANCE = new NullContext();
    return NullContext;
}(Context$1));
var ConfigAwareContextValuesContainer = /** @class */ (function (_super) {
    __extends$2c(ConfigAwareContextValuesContainer, _super);
    function ConfigAwareContextValuesContainer(id, _configurationService, emitter) {
        var _this = _super.call(this, id, null) || this;
        _this._configurationService = _configurationService;
        _this._values = new Map();
        _this._listener = _this._configurationService.onDidChangeConfiguration(function (event) {
            if (event.source === 4 /* DEFAULT */) {
                // new setting, reset everything
                var allKeys = keys(_this._values);
                _this._values.clear();
                emitter.fire(allKeys);
            }
            else {
                var changedKeys = [];
                for (var _i = 0, _a = event.affectedKeys; _i < _a.length; _i++) {
                    var configKey = _a[_i];
                    var contextKey = "config." + configKey;
                    if (_this._values.has(contextKey)) {
                        _this._values.delete(contextKey);
                        changedKeys.push(contextKey);
                    }
                }
                emitter.fire(changedKeys);
            }
        });
        return _this;
    }
    ConfigAwareContextValuesContainer.prototype.dispose = function () {
        this._listener.dispose();
    };
    ConfigAwareContextValuesContainer.prototype.getValue = function (key) {
        if (key.indexOf(ConfigAwareContextValuesContainer._keyPrefix) !== 0) {
            return _super.prototype.getValue.call(this, key);
        }
        if (this._values.has(key)) {
            return this._values.get(key);
        }
        var configKey = key.substr(ConfigAwareContextValuesContainer._keyPrefix.length);
        var configValue = this._configurationService.getValue(configKey);
        var value = undefined;
        switch (typeof configValue) {
            case 'number':
            case 'boolean':
            case 'string':
                value = configValue;
                break;
        }
        this._values.set(key, value);
        return value;
    };
    ConfigAwareContextValuesContainer.prototype.setValue = function (key, value) {
        return _super.prototype.setValue.call(this, key, value);
    };
    ConfigAwareContextValuesContainer.prototype.removeValue = function (key) {
        return _super.prototype.removeValue.call(this, key);
    };
    ConfigAwareContextValuesContainer._keyPrefix = 'config.';
    return ConfigAwareContextValuesContainer;
}(Context$1));
var ContextKey = /** @class */ (function () {
    function ContextKey(parent, key, defaultValue) {
        this._parent = parent;
        this._key = key;
        this._defaultValue = defaultValue;
        this.reset();
    }
    ContextKey.prototype.set = function (value) {
        this._parent.setContext(this._key, value);
    };
    ContextKey.prototype.reset = function () {
        if (typeof this._defaultValue === 'undefined') {
            this._parent.removeContext(this._key);
        }
        else {
            this._parent.setContext(this._key, this._defaultValue);
        }
    };
    ContextKey.prototype.get = function () {
        return this._parent.getContextKeyValue(this._key);
    };
    return ContextKey;
}());
var ContextKeyChangeEvent = /** @class */ (function () {
    function ContextKeyChangeEvent() {
        this._keys = [];
    }
    ContextKeyChangeEvent.prototype.collect = function (oneOrManyKeys) {
        this._keys = this._keys.concat(oneOrManyKeys);
    };
    ContextKeyChangeEvent.prototype.affectsSome = function (keys$$1) {
        for (var _i = 0, _a = this._keys; _i < _a.length; _i++) {
            var key = _a[_i];
            if (keys$$1.has(key)) {
                return true;
            }
        }
        return false;
    };
    return ContextKeyChangeEvent;
}());
var AbstractContextKeyService = /** @class */ (function () {
    function AbstractContextKeyService(myContextId) {
        this._isDisposed = false;
        this._myContextId = myContextId;
        this._onDidChangeContextKey = new Emitter();
    }
    AbstractContextKeyService.prototype.createKey = function (key, defaultValue) {
        if (this._isDisposed) {
            throw new Error("AbstractContextKeyService has been disposed");
        }
        return new ContextKey(this, key, defaultValue);
    };
    Object.defineProperty(AbstractContextKeyService.prototype, "onDidChangeContext", {
        get: function () {
            if (!this._onDidChangeContext) {
                this._onDidChangeContext = debounceEvent(this._onDidChangeContextKey.event, function (prev, cur) {
                    if (!prev) {
                        prev = new ContextKeyChangeEvent();
                    }
                    prev.collect(cur);
                    return prev;
                }, 25);
            }
            return this._onDidChangeContext;
        },
        enumerable: true,
        configurable: true
    });
    AbstractContextKeyService.prototype.createScoped = function (domNode) {
        if (this._isDisposed) {
            throw new Error("AbstractContextKeyService has been disposed");
        }
        return new ScopedContextKeyService(this, this._onDidChangeContextKey, domNode);
    };
    AbstractContextKeyService.prototype.contextMatchesRules = function (rules) {
        if (this._isDisposed) {
            throw new Error("AbstractContextKeyService has been disposed");
        }
        var context = this.getContextValuesContainer(this._myContextId);
        var result = KeybindingResolver.contextMatchesRules(context, rules);
        // console.group(rules.serialize() + ' -> ' + result);
        // rules.keys().forEach(key => { console.log(key, ctx[key]); });
        // console.groupEnd();
        return result;
    };
    AbstractContextKeyService.prototype.getContextKeyValue = function (key) {
        if (this._isDisposed) {
            return undefined;
        }
        return this.getContextValuesContainer(this._myContextId).getValue(key);
    };
    AbstractContextKeyService.prototype.setContext = function (key, value) {
        if (this._isDisposed) {
            return;
        }
        var myContext = this.getContextValuesContainer(this._myContextId);
        if (!myContext) {
            return;
        }
        if (myContext.setValue(key, value)) {
            this._onDidChangeContextKey.fire(key);
        }
    };
    AbstractContextKeyService.prototype.removeContext = function (key) {
        if (this._isDisposed) {
            return;
        }
        if (this.getContextValuesContainer(this._myContextId).removeValue(key)) {
            this._onDidChangeContextKey.fire(key);
        }
    };
    AbstractContextKeyService.prototype.getContext = function (target) {
        if (this._isDisposed) {
            return NullContext.INSTANCE;
        }
        return this.getContextValuesContainer(findContextAttr(target));
    };
    return AbstractContextKeyService;
}());
var ContextKeyService = /** @class */ (function (_super) {
    __extends$2c(ContextKeyService, _super);
    function ContextKeyService(configurationService) {
        var _this = _super.call(this, 0) || this;
        _this._toDispose = [];
        _this._lastContextId = 0;
        _this._contexts = Object.create(null);
        var myContext = new ConfigAwareContextValuesContainer(_this._myContextId, configurationService, _this._onDidChangeContextKey);
        _this._contexts[String(_this._myContextId)] = myContext;
        _this._toDispose.push(myContext);
        return _this;
        // Uncomment this to see the contexts continuously logged
        // let lastLoggedValue: string | null = null;
        // setInterval(() => {
        // 	let values = Object.keys(this._contexts).map((key) => this._contexts[key]);
        // 	let logValue = values.map(v => JSON.stringify(v._value, null, '\t')).join('\n');
        // 	if (lastLoggedValue !== logValue) {
        // 		lastLoggedValue = logValue;
        // 		console.log(lastLoggedValue);
        // 	}
        // }, 2000);
    }
    ContextKeyService.prototype.dispose = function () {
        this._isDisposed = true;
        this._toDispose = dispose(this._toDispose);
    };
    ContextKeyService.prototype.getContextValuesContainer = function (contextId) {
        if (this._isDisposed) {
            return NullContext.INSTANCE;
        }
        return this._contexts[String(contextId)];
    };
    ContextKeyService.prototype.createChildContext = function (parentContextId) {
        if (parentContextId === void 0) { parentContextId = this._myContextId; }
        if (this._isDisposed) {
            throw new Error("ContextKeyService has been disposed");
        }
        var id = (++this._lastContextId);
        this._contexts[String(id)] = new Context$1(id, this.getContextValuesContainer(parentContextId));
        return id;
    };
    ContextKeyService.prototype.disposeContext = function (contextId) {
        if (this._isDisposed) {
            return;
        }
        delete this._contexts[String(contextId)];
    };
    ContextKeyService = __decorate$y([
        __param$v(0, IConfigurationService)
    ], ContextKeyService);
    return ContextKeyService;
}(AbstractContextKeyService));
var ScopedContextKeyService = /** @class */ (function (_super) {
    __extends$2c(ScopedContextKeyService, _super);
    function ScopedContextKeyService(parent, emitter, domNode) {
        var _this = _super.call(this, parent.createChildContext()) || this;
        _this._parent = parent;
        _this._onDidChangeContextKey = emitter;
        if (domNode) {
            _this._domNode = domNode;
            _this._domNode.setAttribute(KEYBINDING_CONTEXT_ATTR, String(_this._myContextId));
        }
        return _this;
    }
    ScopedContextKeyService.prototype.dispose = function () {
        this._isDisposed = true;
        this._parent.disposeContext(this._myContextId);
        if (this._domNode) {
            this._domNode.removeAttribute(KEYBINDING_CONTEXT_ATTR);
            this._domNode = undefined;
        }
    };
    Object.defineProperty(ScopedContextKeyService.prototype, "onDidChangeContext", {
        get: function () {
            return this._parent.onDidChangeContext;
        },
        enumerable: true,
        configurable: true
    });
    ScopedContextKeyService.prototype.getContextValuesContainer = function (contextId) {
        if (this._isDisposed) {
            return NullContext.INSTANCE;
        }
        return this._parent.getContextValuesContainer(contextId);
    };
    ScopedContextKeyService.prototype.createChildContext = function (parentContextId) {
        if (parentContextId === void 0) { parentContextId = this._myContextId; }
        if (this._isDisposed) {
            throw new Error("ScopedContextKeyService has been disposed");
        }
        return this._parent.createChildContext(parentContextId);
    };
    ScopedContextKeyService.prototype.disposeContext = function (contextId) {
        if (this._isDisposed) {
            return;
        }
        this._parent.disposeContext(contextId);
    };
    return ScopedContextKeyService;
}(AbstractContextKeyService));
function findContextAttr(domNode) {
    while (domNode) {
        if (domNode.hasAttribute(KEYBINDING_CONTEXT_ATTR)) {
            var attr = domNode.getAttribute(KEYBINDING_CONTEXT_ATTR);
            if (attr) {
                return parseInt(attr, 10);
            }
            return NaN;
        }
        domNode = domNode.parentElement;
    }
    return 0;
}
CommandsRegistry.registerCommand(SET_CONTEXT_COMMAND_ID, function (accessor, contextKey, contextValue) {
    accessor.get(IContextKeyService).createKey(String(contextKey), contextValue);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2d = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var MENU_MNEMONIC_REGEX = /\(&{1,2}(.)\)|&{1,2}(.)/;
var MENU_ESCAPED_MNEMONIC_REGEX = /(?:&amp;){1,2}(.)/;
var SubmenuAction = /** @class */ (function (_super) {
    __extends$2d(SubmenuAction, _super);
    function SubmenuAction(label, entries, cssClass) {
        var _this = _super.call(this, !!cssClass ? cssClass : 'submenu', label, '', true) || this;
        _this.entries = entries;
        return _this;
    }
    return SubmenuAction;
}(Action));
var Menu$1 = /** @class */ (function (_super) {
    __extends$2d(Menu, _super);
    function Menu(container, actions, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        addClass(container, 'monaco-menu-container');
        container.setAttribute('role', 'presentation');
        var menuContainer = document.createElement('div');
        addClass(menuContainer, 'monaco-menu');
        menuContainer.setAttribute('role', 'presentation');
        container.appendChild(menuContainer);
        _this = _super.call(this, menuContainer, {
            orientation: 2 /* VERTICAL */,
            actionItemProvider: function (action) { return _this.doGetActionItem(action, options, parentData); },
            context: options.context,
            actionRunner: options.actionRunner,
            ariaLabel: options.ariaLabel,
            triggerKeys: { keys: [3 /* Enter */], keyDown: true }
        }) || this;
        _this.actionsList.setAttribute('role', 'menu');
        _this.actionsList.tabIndex = 0;
        _this.menuDisposables = [];
        if (options.enableMnemonics) {
            _this.menuDisposables.push(addDisposableListener(menuContainer, EventType.KEY_DOWN, function (e) {
                var key = KeyCodeUtils.fromString(e.key);
                if (_this.mnemonics.has(key)) {
                    EventHelper.stop(e, true);
                    var actions_1 = _this.mnemonics.get(key);
                    if (actions_1.length === 1) {
                        if (actions_1[0] instanceof SubmenuActionItem) {
                            _this.focusItemByElement(actions_1[0].container);
                        }
                        actions_1[0].onClick(event);
                    }
                    if (actions_1.length > 1) {
                        var action = actions_1.shift();
                        _this.focusItemByElement(action.container);
                        actions_1.push(action);
                        _this.mnemonics.set(key, actions_1);
                    }
                }
            }));
        }
        _this._register(addDisposableListener(_this.domNode, EventType.MOUSE_OUT, function (e) {
            var relatedTarget = e.relatedTarget;
            if (!isAncestor(relatedTarget, _this.domNode)) {
                _this.focusedItem = undefined;
                _this.updateFocus();
                e.stopPropagation();
            }
        }));
        _this._register(addDisposableListener(_this.actionsList, EventType.MOUSE_OVER, function (e) {
            var target = e.target;
            if (!target || !isAncestor(target, _this.actionsList) || target === _this.actionsList) {
                return;
            }
            while (target.parentElement !== _this.actionsList) {
                target = target.parentElement;
            }
            if (hasClass(target, 'action-item')) {
                var lastFocusedItem = _this.focusedItem;
                _this.setFocusedItem(target);
                if (lastFocusedItem !== _this.focusedItem) {
                    _this.updateFocus();
                }
            }
        }));
        var parentData = {
            parent: _this
        };
        _this.mnemonics = new Map();
        _this.push(actions, { icon: true, label: true, isMenu: true });
        return _this;
    }
    Menu.prototype.style = function (style) {
        var container = this.getContainer();
        var fgColor = style.foregroundColor ? "" + style.foregroundColor : null;
        var bgColor = style.backgroundColor ? "" + style.backgroundColor : null;
        var border = style.borderColor ? "2px solid " + style.borderColor : null;
        var shadow = style.shadowColor ? "0 2px 4px " + style.shadowColor : null;
        container.style.border = border;
        this.domNode.style.color = fgColor;
        this.domNode.style.backgroundColor = bgColor;
        container.style.boxShadow = shadow;
        if (this.items) {
            this.items.forEach(function (item) {
                if (item instanceof MenuActionItem || item instanceof MenuSeparatorActionItem) {
                    item.style(style);
                }
            });
        }
    };
    Menu.prototype.focusItemByElement = function (element) {
        var lastFocusedItem = this.focusedItem;
        this.setFocusedItem(element);
        if (lastFocusedItem !== this.focusedItem) {
            this.updateFocus();
        }
    };
    Menu.prototype.setFocusedItem = function (element) {
        for (var i = 0; i < this.actionsList.children.length; i++) {
            var elem = this.actionsList.children[i];
            if (element === elem) {
                this.focusedItem = i;
                break;
            }
        }
    };
    Menu.prototype.doGetActionItem = function (action, options, parentData) {
        if (action instanceof Separator) {
            return new MenuSeparatorActionItem(options.context, action, { icon: true });
        }
        else if (action instanceof SubmenuAction) {
            var menuActionItem = new SubmenuActionItem(action, action.entries, parentData, options);
            if (options.enableMnemonics) {
                var mnemonic = menuActionItem.getMnemonic();
                if (mnemonic && menuActionItem.isEnabled()) {
                    var actionItems = [];
                    if (this.mnemonics.has(mnemonic)) {
                        actionItems = this.mnemonics.get(mnemonic);
                    }
                    actionItems.push(menuActionItem);
                    this.mnemonics.set(mnemonic, actionItems);
                }
            }
            return menuActionItem;
        }
        else {
            var menuItemOptions = { enableMnemonics: options.enableMnemonics };
            if (options.getKeyBinding) {
                var keybinding = options.getKeyBinding(action);
                if (keybinding) {
                    menuItemOptions.keybinding = keybinding.getLabel();
                }
            }
            var menuActionItem = new MenuActionItem(options.context, action, menuItemOptions);
            if (options.enableMnemonics) {
                var mnemonic = menuActionItem.getMnemonic();
                if (mnemonic && menuActionItem.isEnabled()) {
                    var actionItems = [];
                    if (this.mnemonics.has(mnemonic)) {
                        actionItems = this.mnemonics.get(mnemonic);
                    }
                    actionItems.push(menuActionItem);
                    this.mnemonics.set(mnemonic, actionItems);
                }
            }
            return menuActionItem;
        }
    };
    Menu.prototype.focus = function (selectFirst) {
        if (selectFirst === void 0) { selectFirst = true; }
        _super.prototype.focus.call(this, selectFirst);
    };
    return Menu;
}(ActionBar));
var MenuActionItem = /** @class */ (function (_super) {
    __extends$2d(MenuActionItem, _super);
    function MenuActionItem(ctx, action, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        options.isMenu = true;
        _this = _super.call(this, action, action, options) || this;
        _this.options = options;
        _this.options.icon = options.icon !== undefined ? options.icon : false;
        _this.options.label = options.label !== undefined ? options.label : true;
        _this.cssClass = '';
        // Set mnemonic
        if (_this.options.label && options.enableMnemonics) {
            var label = _this.getAction().label;
            if (label) {
                var matches = MENU_MNEMONIC_REGEX.exec(label);
                if (matches) {
                    _this.mnemonic = KeyCodeUtils.fromString((!!matches[1] ? matches[1] : matches[2]).toLocaleLowerCase());
                }
            }
        }
        return _this;
    }
    MenuActionItem.prototype.render = function (container) {
        var _this = this;
        _super.prototype.render.call(this, container);
        this.container = container;
        this.item = append(this.element, $('a.action-menu-item'));
        if (this._action.id === Separator.ID) {
            // A separator is a presentation item
            this.item.setAttribute('role', 'presentation');
        }
        else {
            this.item.setAttribute('role', 'menuitem');
            if (this.mnemonic) {
                this.item.setAttribute('aria-keyshortcuts', "" + this.mnemonic);
            }
        }
        this.check = append(this.item, $('span.menu-item-check'));
        this.check.setAttribute('role', 'none');
        this.label = append(this.item, $('span.action-label'));
        if (this.options.label && this.options.keybinding) {
            append(this.item, $('span.keybinding')).textContent = this.options.keybinding;
        }
        this._register(addDisposableListener(this.element, EventType.MOUSE_UP, function (e) {
            EventHelper.stop(e, true);
            _this.onClick(e);
        }));
        this.updateClass();
        this.updateLabel();
        this.updateTooltip();
        this.updateEnabled();
        this.updateChecked();
    };
    MenuActionItem.prototype.blur = function () {
        _super.prototype.blur.call(this);
        this.applyStyle();
    };
    MenuActionItem.prototype.focus = function () {
        _super.prototype.focus.call(this);
        this.item.focus();
        this.applyStyle();
    };
    MenuActionItem.prototype.updateLabel = function () {
        if (this.options.label) {
            var label = this.getAction().label;
            if (label) {
                var cleanLabel = cleanMnemonic(label);
                if (!this.options.enableMnemonics) {
                    label = cleanLabel;
                }
                this.label.setAttribute('aria-label', cleanLabel);
                var matches = MENU_MNEMONIC_REGEX.exec(label);
                if (matches) {
                    label = escape(label).replace(MENU_ESCAPED_MNEMONIC_REGEX, '<u aria-hidden="true">$1</u>');
                    this.item.setAttribute('aria-keyshortcuts', (!!matches[1] ? matches[1] : matches[2]).toLocaleLowerCase());
                }
            }
            this.label.innerHTML = label.trim();
        }
    };
    MenuActionItem.prototype.updateTooltip = function () {
        var title = null;
        if (this.getAction().tooltip) {
            title = this.getAction().tooltip;
        }
        else if (!this.options.label && this.getAction().label && this.options.icon) {
            title = this.getAction().label;
            if (this.options.keybinding) {
                title = localize({ key: 'titleLabel', comment: ['action title', 'action keybinding'] }, "{0} ({1})", title, this.options.keybinding);
            }
        }
        if (title) {
            this.item.title = title;
        }
    };
    MenuActionItem.prototype.updateClass = function () {
        if (this.cssClass) {
            removeClasses(this.item, this.cssClass);
        }
        if (this.options.icon) {
            this.cssClass = this.getAction().class;
            addClass(this.label, 'icon');
            if (this.cssClass) {
                addClasses(this.label, this.cssClass);
            }
            this.updateEnabled();
        }
        else {
            removeClass(this.label, 'icon');
        }
    };
    MenuActionItem.prototype.updateEnabled = function () {
        if (this.getAction().enabled) {
            removeClass(this.element, 'disabled');
            removeClass(this.item, 'disabled');
            this.item.tabIndex = 0;
        }
        else {
            addClass(this.element, 'disabled');
            addClass(this.item, 'disabled');
            removeTabIndexAndUpdateFocus(this.item);
        }
    };
    MenuActionItem.prototype.updateChecked = function () {
        if (this.getAction().checked) {
            addClass(this.item, 'checked');
            this.item.setAttribute('role', 'menuitemcheckbox');
            this.item.setAttribute('aria-checked', 'true');
        }
        else {
            removeClass(this.item, 'checked');
            this.item.setAttribute('role', 'menuitem');
            this.item.setAttribute('aria-checked', 'false');
        }
    };
    MenuActionItem.prototype.getMnemonic = function () {
        return this.mnemonic;
    };
    MenuActionItem.prototype.applyStyle = function () {
        var isSelected = hasClass(this.element, 'focused');
        var fgColor = isSelected && this.menuStyle.selectionForegroundColor ? this.menuStyle.selectionForegroundColor : this.menuStyle.foregroundColor;
        var bgColor = isSelected && this.menuStyle.selectionBackgroundColor ? this.menuStyle.selectionBackgroundColor : this.menuStyle.backgroundColor;
        var border = isSelected && this.menuStyle.selectionBorderColor ? "1px solid " + this.menuStyle.selectionBorderColor : null;
        this.item.style.color = fgColor ? "" + fgColor : null;
        this.check.style.backgroundColor = fgColor ? "" + fgColor : null;
        this.item.style.backgroundColor = bgColor ? "" + bgColor : null;
        this.container.style.border = border;
    };
    MenuActionItem.prototype.style = function (style) {
        this.menuStyle = style;
        this.applyStyle();
    };
    return MenuActionItem;
}(BaseActionItem));
var SubmenuActionItem = /** @class */ (function (_super) {
    __extends$2d(SubmenuActionItem, _super);
    function SubmenuActionItem(action, submenuActions, parentData, submenuOptions) {
        var _this = _super.call(this, action, action, submenuOptions) || this;
        _this.submenuActions = submenuActions;
        _this.parentData = parentData;
        _this.submenuOptions = submenuOptions;
        _this.submenuDisposables = [];
        _this.showScheduler = new RunOnceScheduler(function () {
            if (_this.mouseOver) {
                _this.cleanupExistingSubmenu(false);
                _this.createSubmenu(false);
            }
        }, 250);
        _this.hideScheduler = new RunOnceScheduler(function () {
            if ((!isAncestor(document.activeElement, _this.element) && _this.parentData.submenu === _this.mysubmenu)) {
                _this.parentData.parent.focus(false);
                _this.cleanupExistingSubmenu(true);
            }
        }, 750);
        return _this;
    }
    SubmenuActionItem.prototype.render = function (container) {
        var _this = this;
        _super.prototype.render.call(this, container);
        addClass(this.item, 'monaco-submenu-item');
        this.item.setAttribute('aria-haspopup', 'true');
        this.submenuIndicator = append(this.item, $('span.submenu-indicator'));
        this.submenuIndicator.setAttribute('aria-hidden', 'true');
        this._register(addDisposableListener(this.element, EventType.KEY_UP, function (e) {
            var event = new StandardKeyboardEvent(e);
            if (event.equals(17 /* RightArrow */) || event.equals(3 /* Enter */)) {
                EventHelper.stop(e, true);
                _this.createSubmenu(true);
            }
        }));
        this._register(addDisposableListener(this.element, EventType.KEY_DOWN, function (e) {
            var event = new StandardKeyboardEvent(e);
            if (event.equals(17 /* RightArrow */) || event.equals(3 /* Enter */)) {
                EventHelper.stop(e, true);
            }
        }));
        this._register(addDisposableListener(this.element, EventType.MOUSE_OVER, function (e) {
            if (!_this.mouseOver) {
                _this.mouseOver = true;
                _this.showScheduler.schedule();
            }
        }));
        this._register(addDisposableListener(this.element, EventType.MOUSE_LEAVE, function (e) {
            _this.mouseOver = false;
        }));
        this._register(addDisposableListener(this.element, EventType.FOCUS_OUT, function (e) {
            if (!isAncestor(document.activeElement, _this.element)) {
                _this.hideScheduler.schedule();
            }
        }));
    };
    SubmenuActionItem.prototype.onClick = function (e) {
        // stop clicking from trying to run an action
        EventHelper.stop(e, true);
        this.cleanupExistingSubmenu(false);
        this.createSubmenu(false);
    };
    SubmenuActionItem.prototype.cleanupExistingSubmenu = function (force) {
        if (this.parentData.submenu && (force || (this.parentData.submenu !== this.mysubmenu))) {
            this.parentData.submenu.dispose();
            this.parentData.submenu = null;
            if (this.submenuContainer) {
                this.submenuDisposables = dispose(this.submenuDisposables);
                this.submenuContainer = null;
            }
        }
    };
    SubmenuActionItem.prototype.createSubmenu = function (selectFirstItem) {
        var _this = this;
        if (selectFirstItem === void 0) { selectFirstItem = true; }
        if (!this.parentData.submenu) {
            this.submenuContainer = append(this.element, $('div.monaco-submenu'));
            addClasses(this.submenuContainer, 'menubar-menu-items-holder', 'context-view');
            this.submenuContainer.style.left = getClientArea(this.element).width + "px";
            this.submenuDisposables.push(addDisposableListener(this.submenuContainer, EventType.KEY_UP, function (e) {
                var event = new StandardKeyboardEvent(e);
                if (event.equals(15 /* LeftArrow */)) {
                    EventHelper.stop(e, true);
                    _this.parentData.parent.focus();
                    _this.parentData.submenu.dispose();
                    _this.parentData.submenu = null;
                    _this.submenuDisposables = dispose(_this.submenuDisposables);
                    _this.submenuContainer = null;
                }
            }));
            this.submenuDisposables.push(addDisposableListener(this.submenuContainer, EventType.KEY_DOWN, function (e) {
                var event = new StandardKeyboardEvent(e);
                if (event.equals(15 /* LeftArrow */)) {
                    EventHelper.stop(e, true);
                }
            }));
            this.parentData.submenu = new Menu$1(this.submenuContainer, this.submenuActions, this.submenuOptions);
            if (this.menuStyle) {
                this.parentData.submenu.style(this.menuStyle);
            }
            this.submenuDisposables.push(this.parentData.submenu.onDidCancel(function () {
                _this.parentData.parent.focus();
                _this.parentData.submenu.dispose();
                _this.parentData.submenu = null;
                _this.submenuDisposables = dispose(_this.submenuDisposables);
                _this.submenuContainer = null;
            }));
            this.parentData.submenu.focus(selectFirstItem);
            this.mysubmenu = this.parentData.submenu;
        }
        else {
            this.parentData.submenu.focus(false);
        }
    };
    SubmenuActionItem.prototype.applyStyle = function () {
        _super.prototype.applyStyle.call(this);
        var isSelected = hasClass(this.element, 'focused');
        var fgColor = isSelected && this.menuStyle.selectionForegroundColor ? this.menuStyle.selectionForegroundColor : this.menuStyle.foregroundColor;
        this.submenuIndicator.style.backgroundColor = fgColor ? "" + fgColor : null;
        if (this.parentData.submenu) {
            this.parentData.submenu.style(this.menuStyle);
        }
    };
    SubmenuActionItem.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this.hideScheduler.dispose();
        if (this.mysubmenu) {
            this.mysubmenu.dispose();
            this.mysubmenu = null;
        }
        if (this.submenuContainer) {
            this.submenuDisposables = dispose(this.submenuDisposables);
            this.submenuContainer = null;
        }
    };
    return SubmenuActionItem;
}(MenuActionItem));
var MenuSeparatorActionItem = /** @class */ (function (_super) {
    __extends$2d(MenuSeparatorActionItem, _super);
    function MenuSeparatorActionItem() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MenuSeparatorActionItem.prototype.style = function (style) {
        this.label.style.borderBottomColor = style.separatorColor ? "" + style.separatorColor : null;
    };
    return MenuSeparatorActionItem;
}(ActionItem));
function cleanMnemonic(label) {
    var regex = MENU_MNEMONIC_REGEX;
    var matches = regex.exec(label);
    if (!matches) {
        return label;
    }
    var mnemonicInText = matches[0].charAt(0) === '&';
    return label.replace(regex, mnemonicInText ? '$2' : '').trim();
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var ContextMenuHandler = /** @class */ (function () {
    function ContextMenuHandler(element, contextViewService, telemetryService, notificationService, keybindingService, themeService) {
        this.contextViewService = contextViewService;
        this.telemetryService = telemetryService;
        this.notificationService = notificationService;
        this.keybindingService = keybindingService;
        this.themeService = themeService;
        this.setContainer(element);
    }
    ContextMenuHandler.prototype.setContainer = function (container) {
        var _this = this;
        if (this.element) {
            this.elementDisposable = dispose(this.elementDisposable);
            this.element = null;
        }
        if (container) {
            this.element = container;
            this.elementDisposable = addDisposableListener(this.element, EventType.MOUSE_DOWN, function (e) { return _this.onMouseDown(e); });
        }
    };
    ContextMenuHandler.prototype.showContextMenu = function (delegate) {
        var _this = this;
        delegate.getActions().then(function (actions) {
            if (!actions.length) {
                return; // Don't render an empty context menu
            }
            _this.focusToReturn = document.activeElement;
            var menu;
            _this.contextViewService.showContextView({
                getAnchor: function () { return delegate.getAnchor(); },
                canRelayout: false,
                render: function (container) {
                    _this.menuContainerElement = container;
                    var className = delegate.getMenuClassName ? delegate.getMenuClassName() : '';
                    if (className) {
                        container.className += ' ' + className;
                    }
                    var menuDisposables = [];
                    var actionRunner = delegate.actionRunner || new ActionRunner();
                    actionRunner.onDidBeforeRun(_this.onActionRun, _this, menuDisposables);
                    actionRunner.onDidRun(_this.onDidActionRun, _this, menuDisposables);
                    menu = new Menu$1(container, actions, {
                        actionItemProvider: delegate.getActionItem,
                        context: delegate.getActionsContext ? delegate.getActionsContext() : null,
                        actionRunner: actionRunner,
                        getKeyBinding: delegate.getKeyBinding ? delegate.getKeyBinding : function (action) { return _this.keybindingService.lookupKeybinding(action.id); }
                    });
                    menuDisposables.push(attachMenuStyler(menu, _this.themeService));
                    menu.onDidCancel(function () { return _this.contextViewService.hideContextView(true); }, null, menuDisposables);
                    menu.onDidBlur(function () { return _this.contextViewService.hideContextView(true); }, null, menuDisposables);
                    domEvent(window, EventType.BLUR)(function () { _this.contextViewService.hideContextView(true); }, null, menuDisposables);
                    return combinedDisposable(menuDisposables.concat([menu]));
                },
                focus: function () {
                    menu.focus(!!delegate.autoSelectFirstItem);
                },
                onHide: function (didCancel) {
                    if (delegate.onHide) {
                        delegate.onHide(didCancel);
                    }
                    _this.menuContainerElement = null;
                }
            });
        });
    };
    ContextMenuHandler.prototype.onActionRun = function (e) {
        if (this.telemetryService) {
            /* __GDPR__
                "workbenchActionExecuted" : {
                    "id" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" },
                    "from": { "classification": "SystemMetaData", "purpose": "FeatureInsight" }
                }
            */
            this.telemetryService.publicLog('workbenchActionExecuted', { id: e.action.id, from: 'contextMenu' });
        }
        this.contextViewService.hideContextView(false);
        // Restore focus here
        if (this.focusToReturn) {
            this.focusToReturn.focus();
        }
    };
    ContextMenuHandler.prototype.onDidActionRun = function (e) {
        if (e.error && this.notificationService) {
            this.notificationService.error(e.error);
        }
    };
    ContextMenuHandler.prototype.onMouseDown = function (e) {
        if (!this.menuContainerElement) {
            return;
        }
        var event = new StandardMouseEvent(e);
        var element = event.target;
        while (element) {
            if (element === this.menuContainerElement) {
                return;
            }
            element = element.parentElement;
        }
        this.contextViewService.hideContextView();
    };
    ContextMenuHandler.prototype.dispose = function () {
        this.setContainer(null);
    };
    return ContextMenuHandler;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2e = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$z = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$w = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ContextMenuService = /** @class */ (function (_super) {
    __extends$2e(ContextMenuService, _super);
    function ContextMenuService(container, telemetryService, notificationService, contextViewService, keybindingService, themeService) {
        var _this = _super.call(this) || this;
        _this._onDidContextMenu = _this._register(new Emitter());
        _this.contextMenuHandler = _this._register(new ContextMenuHandler(container, contextViewService, telemetryService, notificationService, keybindingService, themeService));
        return _this;
    }
    ContextMenuService.prototype.dispose = function () {
        this.contextMenuHandler.dispose();
    };
    // ContextMenu
    ContextMenuService.prototype.showContextMenu = function (delegate) {
        this.contextMenuHandler.showContextMenu(delegate);
        this._onDidContextMenu.fire();
    };
    ContextMenuService = __decorate$z([
        __param$w(1, ITelemetryService),
        __param$w(2, INotificationService),
        __param$w(3, IContextViewService),
        __param$w(4, IKeybindingService),
        __param$w(5, IThemeService)
    ], ContextMenuService);
    return ContextMenuService;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2f = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * Lays out a one dimensional view next to an anchor in a viewport.
 *
 * @returns The view offset within the viewport.
 */
function layout(viewportSize, viewSize, anchor) {
    var anchorEnd = anchor.offset + anchor.size;
    if (anchor.position === 0 /* Before */) {
        if (viewSize <= viewportSize - anchorEnd) {
            return anchorEnd; // happy case, lay it out after the anchor
        }
        if (viewSize <= anchor.offset) {
            return anchor.offset - viewSize; // ok case, lay it out before the anchor
        }
        return Math.max(viewportSize - viewSize, 0); // sad case, lay it over the anchor
    }
    else {
        if (viewSize <= anchor.offset) {
            return anchor.offset - viewSize; // happy case, lay it out before the anchor
        }
        if (viewSize <= viewportSize - anchorEnd) {
            return anchorEnd; // ok case, lay it out after the anchor
        }
        return 0; // sad case, lay it over the anchor
    }
}
var ContextView = /** @class */ (function (_super) {
    __extends$2f(ContextView, _super);
    function ContextView(container) {
        var _this = _super.call(this) || this;
        _this.view = $('.context-view');
        hide(_this.view);
        _this.setContainer(container);
        _this._register(toDisposable(function () { return _this.setContainer(null); }));
        return _this;
    }
    ContextView.prototype.setContainer = function (container) {
        var _this = this;
        if (this.container) {
            this.toDisposeOnSetContainer = dispose(this.toDisposeOnSetContainer);
            this.container.removeChild(this.view);
            this.container = null;
        }
        if (container) {
            this.container = container;
            this.container.appendChild(this.view);
            var toDisposeOnSetContainer_1 = [];
            ContextView.BUBBLE_UP_EVENTS.forEach(function (event) {
                toDisposeOnSetContainer_1.push(addStandardDisposableListener(_this.container, event, function (e) {
                    _this.onDOMEvent(e, document.activeElement, false);
                }));
            });
            ContextView.BUBBLE_DOWN_EVENTS.forEach(function (event) {
                toDisposeOnSetContainer_1.push(addStandardDisposableListener(_this.container, event, function (e) {
                    _this.onDOMEvent(e, document.activeElement, true);
                }, true));
            });
            this.toDisposeOnSetContainer = combinedDisposable(toDisposeOnSetContainer_1);
        }
    };
    ContextView.prototype.show = function (delegate) {
        if (this.isVisible()) {
            this.hide();
        }
        // Show static box
        clearNode(this.view);
        this.view.className = 'context-view';
        this.view.style.top = '0px';
        this.view.style.left = '0px';
        show(this.view);
        // Render content
        this.toDisposeOnClean = delegate.render(this.view);
        // Set active delegate
        this.delegate = delegate;
        // Layout
        this.doLayout();
        // Focus
        if (this.delegate.focus) {
            this.delegate.focus();
        }
    };
    ContextView.prototype.layout = function () {
        if (!this.isVisible()) {
            return;
        }
        if (this.delegate.canRelayout === false) {
            this.hide();
            return;
        }
        if (this.delegate.layout) {
            this.delegate.layout();
        }
        this.doLayout();
    };
    ContextView.prototype.doLayout = function () {
        // Check that we still have a delegate - this.delegate.layout may have hidden
        if (!this.isVisible()) {
            return;
        }
        // Get anchor
        var anchor = this.delegate.getAnchor();
        // Compute around
        var around;
        // Get the element's position and size (to anchor the view)
        if (isHTMLElement(anchor)) {
            var elementPosition = getDomNodePagePosition(anchor);
            around = {
                top: elementPosition.top,
                left: elementPosition.left,
                width: elementPosition.width,
                height: elementPosition.height
            };
        }
        else {
            var realAnchor = anchor;
            around = {
                top: realAnchor.y,
                left: realAnchor.x,
                width: realAnchor.width || 1,
                height: realAnchor.height || 2
            };
        }
        var viewSizeWidth = getTotalWidth(this.view);
        var viewSizeHeight = getTotalHeight(this.view);
        var anchorPosition = this.delegate.anchorPosition || 0 /* BELOW */;
        var anchorAlignment = this.delegate.anchorAlignment || 0 /* LEFT */;
        var verticalAnchor = { offset: around.top - window.pageYOffset, size: around.height, position: anchorPosition === 0 /* BELOW */ ? 0 /* Before */ : 1 /* After */ };
        var horizontalAnchor;
        if (anchorAlignment === 0 /* LEFT */) {
            horizontalAnchor = { offset: around.left, size: 0, position: 0 /* Before */ };
        }
        else {
            horizontalAnchor = { offset: around.left + around.width, size: 0, position: 1 /* After */ };
        }
        var top = layout(window.innerHeight, viewSizeHeight, verticalAnchor) + window.pageYOffset;
        // if view intersects vertically with anchor, shift it horizontally
        if (Range$1.intersects({ start: top, end: top + viewSizeHeight }, { start: verticalAnchor.offset, end: verticalAnchor.offset + verticalAnchor.size })) {
            horizontalAnchor.size = around.width;
        }
        var left = layout(window.innerWidth, viewSizeWidth, horizontalAnchor);
        removeClasses(this.view, 'top', 'bottom', 'left', 'right');
        addClass(this.view, anchorPosition === 0 /* BELOW */ ? 'bottom' : 'top');
        addClass(this.view, anchorAlignment === 0 /* LEFT */ ? 'left' : 'right');
        var containerPosition = getDomNodePagePosition(this.container);
        this.view.style.top = top - containerPosition.top + "px";
        this.view.style.left = left - containerPosition.left + "px";
        this.view.style.width = 'initial';
    };
    ContextView.prototype.hide = function (data) {
        if (this.delegate && this.delegate.onHide) {
            this.delegate.onHide(data);
        }
        this.delegate = null;
        if (this.toDisposeOnClean) {
            this.toDisposeOnClean.dispose();
            this.toDisposeOnClean = null;
        }
        hide(this.view);
    };
    ContextView.prototype.isVisible = function () {
        return !!this.delegate;
    };
    ContextView.prototype.onDOMEvent = function (e, element, onCapture) {
        if (this.delegate) {
            if (this.delegate.onDOMEvent) {
                this.delegate.onDOMEvent(e, document.activeElement);
            }
            else if (onCapture && !isAncestor(e.target, this.container)) {
                this.hide();
            }
        }
    };
    ContextView.prototype.dispose = function () {
        this.hide();
        _super.prototype.dispose.call(this);
    };
    ContextView.BUBBLE_UP_EVENTS = ['click', 'keydown', 'focus', 'blur'];
    ContextView.BUBBLE_DOWN_EVENTS = ['click'];
    return ContextView;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2g = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$A = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$x = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ContextViewService = /** @class */ (function (_super) {
    __extends$2g(ContextViewService, _super);
    function ContextViewService(container, telemetryService, logService) {
        var _this = _super.call(this) || this;
        _this.logService = logService;
        _this.contextView = _this._register(new ContextView(container));
        return _this;
    }
    // ContextView
    ContextViewService.prototype.setContainer = function (container) {
        this.logService.trace('ContextViewService#setContainer');
        this.contextView.setContainer(container);
    };
    ContextViewService.prototype.showContextView = function (delegate) {
        this.logService.trace('ContextViewService#showContextView');
        this.contextView.show(delegate);
    };
    ContextViewService.prototype.layout = function () {
        this.contextView.layout();
    };
    ContextViewService.prototype.hideContextView = function (data) {
        this.logService.trace('ContextViewService#hideContextView');
        this.contextView.hide(data);
    };
    ContextViewService = __decorate$A([
        __param$x(1, ITelemetryService),
        __param$x(2, ILogService)
    ], ContextViewService);
    return ContextViewService;
}(Disposable));

var IDialogService = createDecorator('dialogService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var hasOwnProperty$3 = Object.prototype.hasOwnProperty;
/**
 * Iterates over each entry in the provided set. The iterator allows
 * to remove elements and will stop when the callback returns {{false}}.
 */
function forEach(from, callback) {
    var _loop_1 = function (key) {
        if (hasOwnProperty$3.call(from, key)) {
            var result = callback({ key: key, value: from[key] }, function () {
                delete from[key];
            });
            if (result === false) {
                return { value: void 0 };
            }
        }
    };
    for (var key in from) {
        var state_1 = _loop_1(key);
        if (typeof state_1 === "object")
            return state_1.value;
    }
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function newNode$1(data) {
    return {
        data: data,
        incoming: Object.create(null),
        outgoing: Object.create(null)
    };
}
var Graph = /** @class */ (function () {
    function Graph(_hashFn) {
        this._hashFn = _hashFn;
        this._nodes = Object.create(null);
        // empty
    }
    Graph.prototype.roots = function () {
        var ret = [];
        forEach(this._nodes, function (entry) {
            if (isEmptyObject(entry.value.outgoing)) {
                ret.push(entry.value);
            }
        });
        return ret;
    };
    Graph.prototype.insertEdge = function (from, to) {
        var fromNode = this.lookupOrInsertNode(from), toNode = this.lookupOrInsertNode(to);
        fromNode.outgoing[this._hashFn(to)] = toNode;
        toNode.incoming[this._hashFn(from)] = fromNode;
    };
    Graph.prototype.removeNode = function (data) {
        var key = this._hashFn(data);
        delete this._nodes[key];
        forEach(this._nodes, function (entry) {
            delete entry.value.outgoing[key];
            delete entry.value.incoming[key];
        });
    };
    Graph.prototype.lookupOrInsertNode = function (data) {
        var key = this._hashFn(data);
        var node = this._nodes[key];
        if (!node) {
            node = newNode$1(data);
            this._nodes[key] = node;
        }
        return node;
    };
    Graph.prototype.isEmpty = function () {
        for (var _key in this._nodes) {
            return false;
        }
        return true;
    };
    Graph.prototype.toString = function () {
        var data = [];
        forEach(this._nodes, function (entry) {
            data.push(entry.key + ", (incoming)[" + Object.keys(entry.value.incoming).join(', ') + "], (outgoing)[" + Object.keys(entry.value.outgoing).join(',') + "]");
        });
        return data.join('\n');
    };
    return Graph;
}());

var SyncDescriptor = /** @class */ (function () {
    function SyncDescriptor(ctor, staticArguments, supportsDelayedInstantiation) {
        if (staticArguments === void 0) { staticArguments = []; }
        if (supportsDelayedInstantiation === void 0) { supportsDelayedInstantiation = false; }
        this.ctor = ctor;
        this.staticArguments = staticArguments;
        this.supportsDelayedInstantiation = supportsDelayedInstantiation;
    }
    return SyncDescriptor;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2h = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var InstantiationService = /** @class */ (function () {
    function InstantiationService(services, strict, parent) {
        if (services === void 0) { services = new ServiceCollection(); }
        if (strict === void 0) { strict = false; }
        this._services = services;
        this._strict = strict;
        this._parent = parent;
        this._services.set(IInstantiationService, this);
    }
    InstantiationService.prototype.createChild = function (services) {
        return new InstantiationService(services, this._strict, this);
    };
    InstantiationService.prototype.invokeFunction = function (fn) {
        var _this = this;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var _trace = Trace.traceInvocation(fn);
        var _done = false;
        try {
            var accessor = {
                get: function (id, isOptional) {
                    if (_done) {
                        throw illegalState('service accessor is only valid during the invocation of its target method');
                    }
                    var result = _this._getOrCreateServiceInstance(id, _trace);
                    if (!result && isOptional !== optional) {
                        throw new Error("[invokeFunction] unknown service '" + id + "'");
                    }
                    return result;
                }
            };
            return fn.apply(undefined, [accessor].concat(args));
        }
        finally {
            _done = true;
            _trace.stop();
        }
    };
    InstantiationService.prototype.createInstance = function (ctorOrDescriptor) {
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        var _trace;
        var result;
        if (ctorOrDescriptor instanceof SyncDescriptor) {
            _trace = Trace.traceCreation(ctorOrDescriptor.ctor);
            result = this._createInstance(ctorOrDescriptor.ctor, ctorOrDescriptor.staticArguments.concat(rest), _trace);
        }
        else {
            _trace = Trace.traceCreation(ctorOrDescriptor);
            result = this._createInstance(ctorOrDescriptor, rest, _trace);
        }
        _trace.stop();
        return result;
    };
    InstantiationService.prototype._createInstance = function (ctor, args, _trace) {
        if (args === void 0) { args = []; }
        // arguments defined by service decorators
        var serviceDependencies = _util.getServiceDependencies(ctor).sort(function (a, b) { return a.index - b.index; });
        var serviceArgs = [];
        for (var _i = 0, serviceDependencies_1 = serviceDependencies; _i < serviceDependencies_1.length; _i++) {
            var dependency = serviceDependencies_1[_i];
            var service = this._getOrCreateServiceInstance(dependency.id, _trace);
            if (!service && this._strict && !dependency.optional) {
                throw new Error("[createInstance] " + ctor.name + " depends on UNKNOWN service " + dependency.id + ".");
            }
            serviceArgs.push(service);
        }
        var firstServiceArgPos = serviceDependencies.length > 0 ? serviceDependencies[0].index : args.length;
        // check for argument mismatches, adjust static args if needed
        if (args.length !== firstServiceArgPos) {
            console.warn("[createInstance] First service dependency of " + ctor.name + " at position " + (firstServiceArgPos + 1) + " conflicts with " + args.length + " static arguments");
            var delta = firstServiceArgPos - args.length;
            if (delta > 0) {
                args = args.concat(new Array(delta));
            }
            else {
                args = args.slice(0, firstServiceArgPos);
            }
        }
        // now create the instance
        return create.apply(null, [ctor].concat(args, serviceArgs));
    };
    InstantiationService.prototype._setServiceInstance = function (id, instance) {
        if (this._services.get(id) instanceof SyncDescriptor) {
            this._services.set(id, instance);
        }
        else if (this._parent) {
            this._parent._setServiceInstance(id, instance);
        }
        else {
            throw new Error('illegalState - setting UNKNOWN service instance');
        }
    };
    InstantiationService.prototype._getServiceInstanceOrDescriptor = function (id) {
        var instanceOrDesc = this._services.get(id);
        if (!instanceOrDesc && this._parent) {
            return this._parent._getServiceInstanceOrDescriptor(id);
        }
        else {
            return instanceOrDesc;
        }
    };
    InstantiationService.prototype._getOrCreateServiceInstance = function (id, _trace) {
        var thing = this._getServiceInstanceOrDescriptor(id);
        if (thing instanceof SyncDescriptor) {
            return this._createAndCacheServiceInstance(id, thing, _trace.branch(id, true));
        }
        else {
            _trace.branch(id, false);
            return thing;
        }
    };
    InstantiationService.prototype._createAndCacheServiceInstance = function (id, desc, _trace) {
        var graph = new Graph(function (data) { return data.id.toString(); });
        function throwCycleError() {
            var err = new Error('[createInstance] cyclic dependency between services');
            err.message = graph.toString();
            throw err;
        }
        var count = 0;
        var stack = [{ id: id, desc: desc, _trace: _trace }];
        while (stack.length) {
            var item = stack.pop();
            graph.lookupOrInsertNode(item);
            // TODO@joh use the graph to find a cycle
            // a weak heuristic for cycle checks
            if (count++ > 100) {
                throwCycleError();
            }
            // check all dependencies for existence and if they need to be created first
            var dependencies = _util.getServiceDependencies(item.desc.ctor);
            for (var _i = 0, dependencies_1 = dependencies; _i < dependencies_1.length; _i++) {
                var dependency = dependencies_1[_i];
                var instanceOrDesc = this._getServiceInstanceOrDescriptor(dependency.id);
                if (!instanceOrDesc && !dependency.optional) {
                    console.warn("[createInstance] " + id + " depends on " + dependency.id + " which is NOT registered.");
                }
                if (instanceOrDesc instanceof SyncDescriptor) {
                    var d = { id: dependency.id, desc: instanceOrDesc, _trace: item._trace.branch(dependency.id, true) };
                    graph.insertEdge(item, d);
                    stack.push(d);
                }
            }
        }
        while (true) {
            var roots = graph.roots();
            // if there is no more roots but still
            // nodes in the graph we have a cycle
            if (roots.length === 0) {
                if (!graph.isEmpty()) {
                    throwCycleError();
                }
                break;
            }
            for (var _a = 0, roots_1 = roots; _a < roots_1.length; _a++) {
                var data = roots_1[_a].data;
                // create instance and overwrite the service collections
                var instance = this._createServiceInstanceWithOwner(data.id, data.desc.ctor, data.desc.staticArguments, false, data._trace);
                this._setServiceInstance(data.id, instance);
                graph.removeNode(data);
            }
        }
        return this._getServiceInstanceOrDescriptor(id);
    };
    InstantiationService.prototype._createServiceInstanceWithOwner = function (id, ctor, args, supportsDelayedInstantiation, _trace) {
        if (args === void 0) { args = []; }
        if (this._services.get(id) instanceof SyncDescriptor) {
            return this._createServiceInstance(ctor, args, supportsDelayedInstantiation, _trace);
        }
        else if (this._parent) {
            return this._parent._createServiceInstanceWithOwner(id, ctor, args, supportsDelayedInstantiation, _trace);
        }
        else {
            throw new Error('illegalState - creating UNKNOWN service instance');
        }
    };
    InstantiationService.prototype._createServiceInstance = function (ctor, args, supportsDelayedInstantiation, _trace) {
        if (args === void 0) { args = []; }
        return this._createInstance(ctor, args, _trace);
    };
    return InstantiationService;
}());
var Trace = /** @class */ (function () {
    function Trace(type, name) {
        this.type = type;
        this.name = name;
        this._start = Date.now();
        this._dep = [];
    }
    Trace.traceInvocation = function (ctor) {
        return Trace._None;
    };
    Trace.traceCreation = function (ctor) {
        return Trace._None;
    };
    Trace.prototype.branch = function (id, first) {
        var child = new Trace(2 /* Branch */, id.toString());
        this._dep.push([id, first, child]);
        return child;
    };
    Trace.prototype.stop = function () {
        var dur = Date.now() - this._start;
        Trace._totals += dur;
        var causedCreation = false;
        function printChild(n, trace) {
            var res = [];
            var prefix = new Array(n + 1).join('\t');
            for (var _i = 0, _a = trace._dep; _i < _a.length; _i++) {
                var _b = _a[_i], id = _b[0], first = _b[1], child = _b[2];
                if (first && child) {
                    causedCreation = true;
                    res.push(prefix + "CREATES -> " + id);
                    var nested = printChild(n + 1, child);
                    if (nested) {
                        res.push(nested);
                    }
                }
                else {
                    res.push(prefix + "uses -> " + id);
                }
            }
            return res.join('\n');
        }
        var lines = [
            (this.type === 0 /* Creation */ ? 'CREATE' : 'CALL') + " " + this.name,
            "" + printChild(1, this),
            "DONE, took " + dur.toFixed(2) + "ms (grand total " + Trace._totals.toFixed(2) + "ms)"
        ];
        if (dur > 2 || causedCreation) {
            console.log(lines.join('\n'));
        }
    };
    Trace._None = new /** @class */ (function (_super) {
        __extends$2h(class_1, _super);
        function class_1() {
            return _super.call(this, -1, null) || this;
        }
        class_1.prototype.stop = function () { };
        class_1.prototype.branch = function () { return this; };
        return class_1;
    }(Trace));
    Trace._totals = 0;
    return Trace;
}());
//#endregion

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var LABEL_SERVICE_ID = 'label';
var ILabelService = createDecorator(LABEL_SERVICE_ID);

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2i = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$4 = (undefined && undefined.__assign) || function () {
    __assign$4 = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign$4.apply(this, arguments);
};
var __decorate$B = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$y = (undefined && undefined.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var _a$3;
var IListService = createDecorator('listService');
var ListService = /** @class */ (function () {
    function ListService(contextKeyService) {
        this.lists = [];
        this._lastFocusedWidget = undefined;
    }
    Object.defineProperty(ListService.prototype, "lastFocusedList", {
        get: function () {
            return this._lastFocusedWidget;
        },
        enumerable: true,
        configurable: true
    });
    ListService.prototype.register = function (widget, extraContextKeys) {
        var _this = this;
        if (this.lists.some(function (l) { return l.widget === widget; })) {
            throw new Error('Cannot register the same widget multiple times');
        }
        // Keep in our lists list
        var registeredList = { widget: widget, extraContextKeys: extraContextKeys };
        this.lists.push(registeredList);
        // Check for currently being focused
        if (widget.getHTMLElement() === document.activeElement) {
            this._lastFocusedWidget = widget;
        }
        var result = combinedDisposable([
            widget.onDidFocus(function () { return _this._lastFocusedWidget = widget; }),
            toDisposable(function () { return _this.lists.splice(_this.lists.indexOf(registeredList), 1); }),
            widget.onDidDispose(function () {
                _this.lists = _this.lists.filter(function (l) { return l !== registeredList; });
                if (_this._lastFocusedWidget === widget) {
                    _this._lastFocusedWidget = undefined;
                }
            })
        ]);
        return result;
    };
    ListService = __decorate$B([
        __param$y(0, IContextKeyService)
    ], ListService);
    return ListService;
}());
var RawWorkbenchListFocusContextKey = new RawContextKey('listFocus', true);
var WorkbenchListSupportsMultiSelectContextKey = new RawContextKey('listSupportsMultiselect', true);
var WorkbenchListHasSelectionOrFocus = new RawContextKey('listHasSelectionOrFocus', false);
var WorkbenchListDoubleSelection = new RawContextKey('listDoubleSelection', false);
var WorkbenchListMultiSelection = new RawContextKey('listMultiSelection', false);
function createScopedContextKeyService(contextKeyService, widget) {
    var result = contextKeyService.createScoped(widget.getHTMLElement());
    RawWorkbenchListFocusContextKey.bindTo(result);
    return result;
}
var multiSelectModifierSettingKey = 'workbench.list.multiSelectModifier';
var openModeSettingKey = 'workbench.list.openMode';
var horizontalScrollingKey = 'workbench.tree.horizontalScrolling';
function useAltAsMultipleSelectionModifier(configurationService) {
    return configurationService.getValue(multiSelectModifierSettingKey) === 'alt';
}
function useSingleClickToOpen(configurationService) {
    return configurationService.getValue(openModeSettingKey) !== 'doubleClick';
}
var sharedTreeStyleSheet;
function getSharedTreeStyleSheet() {
    if (!sharedTreeStyleSheet) {
        sharedTreeStyleSheet = createStyleSheet();
    }
    return sharedTreeStyleSheet;
}
function handleTreeController(configuration, instantiationService) {
    if (!configuration.controller) {
        configuration.controller = instantiationService.createInstance(WorkbenchTreeController, {});
    }
    if (!configuration.styler) {
        configuration.styler = new DefaultTreestyler(getSharedTreeStyleSheet());
    }
    return configuration;
}
var WorkbenchTree = /** @class */ (function (_super) {
    __extends$2i(WorkbenchTree, _super);
    function WorkbenchTree(container, configuration, options, contextKeyService, listService, themeService, instantiationService, configurationService) {
        var _this = this;
        var config = handleTreeController(configuration, instantiationService);
        var horizontalScrollMode = configurationService.getValue(horizontalScrollingKey) ? 1 /* Auto */ : 2 /* Hidden */;
        var opts = __assign$4({ horizontalScrollMode: horizontalScrollMode, keyboardSupport: false }, computeStyles(themeService.getTheme(), defaultListStyles), options);
        _this = _super.call(this, container, config, opts) || this;
        _this.disposables = [];
        _this.contextKeyService = createScopedContextKeyService(contextKeyService, _this);
        WorkbenchListSupportsMultiSelectContextKey.bindTo(_this.contextKeyService);
        _this.listHasSelectionOrFocus = WorkbenchListHasSelectionOrFocus.bindTo(_this.contextKeyService);
        _this.listDoubleSelection = WorkbenchListDoubleSelection.bindTo(_this.contextKeyService);
        _this.listMultiSelection = WorkbenchListMultiSelection.bindTo(_this.contextKeyService);
        _this._openOnSingleClick = useSingleClickToOpen(configurationService);
        _this._useAltAsMultipleSelectionModifier = useAltAsMultipleSelectionModifier(configurationService);
        _this.disposables.push(_this.contextKeyService, listService.register(_this), attachListStyler(_this, themeService));
        _this.disposables.push(_this.onDidChangeSelection(function () {
            var selection = _this.getSelection();
            var focus = _this.getFocus();
            _this.listHasSelectionOrFocus.set((selection && selection.length > 0) || !!focus);
            _this.listDoubleSelection.set(selection && selection.length === 2);
            _this.listMultiSelection.set(selection && selection.length > 1);
        }));
        _this.disposables.push(_this.onDidChangeFocus(function () {
            var selection = _this.getSelection();
            var focus = _this.getFocus();
            _this.listHasSelectionOrFocus.set((selection && selection.length > 0) || !!focus);
        }));
        _this.disposables.push(configurationService.onDidChangeConfiguration(function (e) {
            if (e.affectsConfiguration(openModeSettingKey)) {
                _this._openOnSingleClick = useSingleClickToOpen(configurationService);
            }
            if (e.affectsConfiguration(multiSelectModifierSettingKey)) {
                _this._useAltAsMultipleSelectionModifier = useAltAsMultipleSelectionModifier(configurationService);
            }
        }));
        return _this;
    }
    WorkbenchTree.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this.disposables = dispose(this.disposables);
    };
    WorkbenchTree = __decorate$B([
        __param$y(3, IContextKeyService),
        __param$y(4, IListService),
        __param$y(5, IThemeService),
        __param$y(6, IInstantiationService),
        __param$y(7, IConfigurationService)
    ], WorkbenchTree);
    return WorkbenchTree;
}(Tree));
function massageControllerOptions(options) {
    if (typeof options.keyboardSupport !== 'boolean') {
        options.keyboardSupport = false;
    }
    if (typeof options.clickBehavior !== 'number') {
        options.clickBehavior = 0 /* ON_MOUSE_DOWN */;
    }
    return options;
}
var WorkbenchTreeController = /** @class */ (function (_super) {
    __extends$2i(WorkbenchTreeController, _super);
    function WorkbenchTreeController(options, configurationService) {
        var _this = _super.call(this, massageControllerOptions(options)) || this;
        _this.configurationService = configurationService;
        _this.disposables = [];
        // if the open mode is not set, we configure it based on settings
        if (isUndefinedOrNull(options.openMode)) {
            _this.setOpenMode(_this.getOpenModeSetting());
            _this.registerListeners();
        }
        return _this;
    }
    WorkbenchTreeController.prototype.registerListeners = function () {
        var _this = this;
        this.disposables.push(this.configurationService.onDidChangeConfiguration(function (e) {
            if (e.affectsConfiguration(openModeSettingKey)) {
                _this.setOpenMode(_this.getOpenModeSetting());
            }
        }));
    };
    WorkbenchTreeController.prototype.getOpenModeSetting = function () {
        return useSingleClickToOpen(this.configurationService) ? 0 /* SINGLE_CLICK */ : 1 /* DOUBLE_CLICK */;
    };
    WorkbenchTreeController.prototype.dispose = function () {
        this.disposables = dispose(this.disposables);
    };
    WorkbenchTreeController = __decorate$B([
        __param$y(1, IConfigurationService)
    ], WorkbenchTreeController);
    return WorkbenchTreeController;
}(DefaultController));
var configurationRegistry$2 = Registry.as(Extensions$3.Configuration);
configurationRegistry$2.registerConfiguration({
    'id': 'workbench',
    'order': 7,
    'title': localize('workbenchConfigurationTitle', "Workbench"),
    'type': 'object',
    'properties': (_a$3 = {},
        _a$3[multiSelectModifierSettingKey] = {
            'type': 'string',
            'enum': ['ctrlCmd', 'alt'],
            'enumDescriptions': [
                localize('multiSelectModifier.ctrlCmd', "Maps to `Control` on Windows and Linux and to `Command` on macOS."),
                localize('multiSelectModifier.alt', "Maps to `Alt` on Windows and Linux and to `Option` on macOS.")
            ],
            'default': 'ctrlCmd',
            'description': localize({
                key: 'multiSelectModifier',
                comment: [
                    '- `ctrlCmd` refers to a value the setting can take and should not be localized.',
                    '- `Control` and `Command` refer to the modifier keys Ctrl or Cmd on the keyboard and can be localized.'
                ]
            }, "The modifier to be used to add an item in trees and lists to a multi-selection with the mouse (for example in the explorer, open editors and scm view). The 'Open to Side' mouse gestures - if supported - will adapt such that they do not conflict with the multiselect modifier.")
        },
        _a$3[openModeSettingKey] = {
            'type': 'string',
            'enum': ['singleClick', 'doubleClick'],
            'default': 'singleClick',
            'description': localize({
                key: 'openModeModifier',
                comment: ['`singleClick` and `doubleClick` refers to a value the setting can take and should not be localized.']
            }, "Controls how to open items in trees and lists using the mouse (if supported). For parents with children in trees, this setting will control if a single click expands the parent or a double click. Note that some trees and lists might choose to ignore this setting if it is not applicable. ")
        },
        _a$3[horizontalScrollingKey] = {
            'type': 'boolean',
            'default': false,
            'description': localize('horizontalScrolling setting', "Controls whether trees support horizontal scrolling in the workbench.")
        },
        _a$3)
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var MapMap;
(function (MapMap) {
    function get(map, key1, key2) {
        if (map[key1]) {
            return map[key1][key2];
        }
        return undefined;
    }
    MapMap.get = get;
    function set(map, key1, key2, value) {
        if (!map[key1]) {
            map[key1] = Object.create(null);
        }
        map[key1][key2] = value;
    }
    MapMap.set = set;
    function remove(map, key1, key2) {
        if (map[key1] && map[key1][key2]) {
            delete map[key1][key2];
            if (isEmptyObject(map[key1])) {
                delete map[key1];
            }
            return true;
        }
        return false;
    }
    MapMap.remove = remove;
})(MapMap || (MapMap = {}));
var MarkerStats = /** @class */ (function () {
    function MarkerStats(service) {
        this.errors = 0;
        this.infos = 0;
        this.warnings = 0;
        this.unknowns = 0;
        this._data = Object.create(null);
        this._service = service;
        this._subscription = service.onMarkerChanged(this._update, this);
    }
    MarkerStats.prototype.dispose = function () {
        this._subscription.dispose();
        this._data = undefined;
    };
    MarkerStats.prototype._update = function (resources) {
        if (!this._data) {
            return;
        }
        for (var _i = 0, resources_1 = resources; _i < resources_1.length; _i++) {
            var resource = resources_1[_i];
            var key = resource.toString();
            var oldStats = this._data[key];
            if (oldStats) {
                this._substract(oldStats);
            }
            var newStats = this._resourceStats(resource);
            this._add(newStats);
            this._data[key] = newStats;
        }
    };
    MarkerStats.prototype._resourceStats = function (resource) {
        var result = { errors: 0, warnings: 0, infos: 0, unknowns: 0 };
        // TODO this is a hack
        if (resource.scheme === Schemas.inMemory || resource.scheme === Schemas.walkThrough || resource.scheme === Schemas.walkThroughSnippet) {
            return result;
        }
        for (var _i = 0, _a = this._service.read({ resource: resource }); _i < _a.length; _i++) {
            var severity = _a[_i].severity;
            if (severity === MarkerSeverity.Error) {
                result.errors += 1;
            }
            else if (severity === MarkerSeverity.Warning) {
                result.warnings += 1;
            }
            else if (severity === MarkerSeverity.Info) {
                result.infos += 1;
            }
            else {
                result.unknowns += 1;
            }
        }
        return result;
    };
    MarkerStats.prototype._substract = function (op) {
        this.errors -= op.errors;
        this.warnings -= op.warnings;
        this.infos -= op.infos;
        this.unknowns -= op.unknowns;
    };
    MarkerStats.prototype._add = function (op) {
        this.errors += op.errors;
        this.warnings += op.warnings;
        this.infos += op.infos;
        this.unknowns += op.unknowns;
    };
    return MarkerStats;
}());
var MarkerService = /** @class */ (function () {
    function MarkerService() {
        this._onMarkerChanged = new Emitter();
        this._onMarkerChangedEvent = debounceEvent(this._onMarkerChanged.event, MarkerService._debouncer, 0);
        this._byResource = Object.create(null);
        this._byOwner = Object.create(null);
        this._stats = new MarkerStats(this);
    }
    MarkerService.prototype.dispose = function () {
        this._stats.dispose();
    };
    Object.defineProperty(MarkerService.prototype, "onMarkerChanged", {
        get: function () {
            return this._onMarkerChangedEvent;
        },
        enumerable: true,
        configurable: true
    });
    MarkerService.prototype.remove = function (owner, resources) {
        if (!isFalsyOrEmpty(resources)) {
            for (var _i = 0, resources_2 = resources; _i < resources_2.length; _i++) {
                var resource = resources_2[_i];
                this.changeOne(owner, resource, []);
            }
        }
    };
    MarkerService.prototype.changeOne = function (owner, resource, markerData) {
        if (isFalsyOrEmpty(markerData)) {
            // remove marker for this (owner,resource)-tuple
            var a = MapMap.remove(this._byResource, resource.toString(), owner);
            var b = MapMap.remove(this._byOwner, owner, resource.toString());
            if (a !== b) {
                throw new Error('invalid marker service state');
            }
            if (a && b) {
                this._onMarkerChanged.fire([resource]);
            }
        }
        else {
            // insert marker for this (owner,resource)-tuple
            var markers = [];
            for (var _i = 0, markerData_1 = markerData; _i < markerData_1.length; _i++) {
                var data = markerData_1[_i];
                var marker = MarkerService._toMarker(owner, resource, data);
                if (marker) {
                    markers.push(marker);
                }
            }
            MapMap.set(this._byResource, resource.toString(), owner, markers);
            MapMap.set(this._byOwner, owner, resource.toString(), markers);
            this._onMarkerChanged.fire([resource]);
        }
    };
    MarkerService._toMarker = function (owner, resource, data) {
        var code = data.code, severity = data.severity, message = data.message, source = data.source, startLineNumber = data.startLineNumber, startColumn = data.startColumn, endLineNumber = data.endLineNumber, endColumn = data.endColumn, relatedInformation = data.relatedInformation, tags = data.tags;
        if (!message) {
            return undefined;
        }
        // santize data
        startLineNumber = startLineNumber > 0 ? startLineNumber : 1;
        startColumn = startColumn > 0 ? startColumn : 1;
        endLineNumber = endLineNumber >= startLineNumber ? endLineNumber : startLineNumber;
        endColumn = endColumn > 0 ? endColumn : startColumn;
        return {
            resource: resource,
            owner: owner,
            code: code || undefined,
            severity: severity,
            message: message,
            source: source,
            startLineNumber: startLineNumber,
            startColumn: startColumn,
            endLineNumber: endLineNumber,
            endColumn: endColumn,
            relatedInformation: relatedInformation,
            tags: tags,
        };
    };
    MarkerService.prototype.read = function (filter) {
        if (filter === void 0) { filter = Object.create(null); }
        var owner = filter.owner, resource = filter.resource, severities = filter.severities, take = filter.take;
        if (!take || take < 0) {
            take = -1;
        }
        if (owner && resource) {
            // exactly one owner AND resource
            var data = MapMap.get(this._byResource, resource.toString(), owner);
            if (!data) {
                return [];
            }
            else {
                var result = [];
                for (var _i = 0, data_1 = data; _i < data_1.length; _i++) {
                    var marker = data_1[_i];
                    if (MarkerService._accept(marker, severities)) {
                        var newLen = result.push(marker);
                        if (take > 0 && newLen === take) {
                            break;
                        }
                    }
                }
                return result;
            }
        }
        else if (!owner && !resource) {
            // all
            var result = [];
            for (var key1 in this._byResource) {
                for (var key2 in this._byResource[key1]) {
                    for (var _a = 0, _b = this._byResource[key1][key2]; _a < _b.length; _a++) {
                        var data = _b[_a];
                        if (MarkerService._accept(data, severities)) {
                            var newLen = result.push(data);
                            if (take > 0 && newLen === take) {
                                return result;
                            }
                        }
                    }
                }
            }
            return result;
        }
        else {
            // of one resource OR owner
            var map = owner
                ? this._byOwner[owner]
                : resource ? this._byResource[resource.toString()] : undefined;
            if (!map) {
                return [];
            }
            var result = [];
            for (var key in map) {
                for (var _c = 0, _d = map[key]; _c < _d.length; _c++) {
                    var data = _d[_c];
                    if (MarkerService._accept(data, severities)) {
                        var newLen = result.push(data);
                        if (take > 0 && newLen === take) {
                            return result;
                        }
                    }
                }
            }
            return result;
        }
    };
    MarkerService._accept = function (marker, severities) {
        return severities === void 0 || (severities & marker.severity) === marker.severity;
    };
    MarkerService._debouncer = function (last, event) {
        if (!last) {
            MarkerService._dedupeMap = Object.create(null);
            last = [];
        }
        for (var _i = 0, event_1 = event; _i < event_1.length; _i++) {
            var uri = event_1[_i];
            if (MarkerService._dedupeMap[uri.toString()] === void 0) {
                MarkerService._dedupeMap[uri.toString()] = true;
                last.push(uri);
            }
        }
        return last;
    };
    return MarkerService;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var IProgressService = createDecorator('progressService');

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2j = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var StaticServices;
(function (StaticServices) {
    var _serviceCollection = new ServiceCollection();
    var LazyStaticService = /** @class */ (function () {
        function LazyStaticService(serviceId, factory) {
            this._serviceId = serviceId;
            this._factory = factory;
            this._value = null;
        }
        Object.defineProperty(LazyStaticService.prototype, "id", {
            get: function () { return this._serviceId; },
            enumerable: true,
            configurable: true
        });
        LazyStaticService.prototype.get = function (overrides) {
            if (!this._value) {
                if (overrides) {
                    this._value = overrides[this._serviceId.toString()];
                }
                if (!this._value) {
                    this._value = this._factory(overrides);
                }
                if (!this._value) {
                    throw new Error('Service ' + this._serviceId + ' is missing!');
                }
                _serviceCollection.set(this._serviceId, this._value);
            }
            return this._value;
        };
        return LazyStaticService;
    }());
    StaticServices.LazyStaticService = LazyStaticService;
    var _all = [];
    function define(serviceId, factory) {
        var r = new LazyStaticService(serviceId, factory);
        _all.push(r);
        return r;
    }
    function init(overrides) {
        // Create a fresh service collection
        var result = new ServiceCollection();
        // Initialize the service collection with the overrides
        for (var serviceId in overrides) {
            if (overrides.hasOwnProperty(serviceId)) {
                result.set(createDecorator(serviceId), overrides[serviceId]);
            }
        }
        // Make sure the same static services are present in all service collections
        _all.forEach(function (service) { return result.set(service.id, service.get(overrides)); });
        // Ensure the collection gets the correct instantiation service
        var instantiationService = new InstantiationService(result, true);
        result.set(IInstantiationService, instantiationService);
        return [result, instantiationService];
    }
    StaticServices.init = init;
    StaticServices.instantiationService = define(IInstantiationService, function () { return new InstantiationService(_serviceCollection, true); });
    var configurationServiceImpl = new SimpleConfigurationService();
    StaticServices.configurationService = define(IConfigurationService, function () { return configurationServiceImpl; });
    StaticServices.resourceConfigurationService = define(ITextResourceConfigurationService, function () { return new SimpleResourceConfigurationService(configurationServiceImpl); });
    StaticServices.resourcePropertiesService = define(ITextResourcePropertiesService, function () { return new SimpleResourcePropertiesService(configurationServiceImpl); });
    StaticServices.contextService = define(IWorkspaceContextService, function () { return new SimpleWorkspaceContextService(); });
    StaticServices.labelService = define(ILabelService, function () { return new SimpleUriLabelService(); });
    StaticServices.telemetryService = define(ITelemetryService, function () { return new StandaloneTelemetryService(); });
    StaticServices.dialogService = define(IDialogService, function () { return new SimpleDialogService(); });
    StaticServices.notificationService = define(INotificationService, function () { return new SimpleNotificationService(); });
    StaticServices.markerService = define(IMarkerService, function () { return new MarkerService(); });
    StaticServices.modeService = define(IModeService, function (o) { return new ModeServiceImpl(); });
    StaticServices.modelService = define(IModelService, function (o) { return new ModelServiceImpl(StaticServices.markerService.get(o), StaticServices.configurationService.get(o), StaticServices.resourcePropertiesService.get(o)); });
    StaticServices.editorWorkerService = define(IEditorWorkerService, function (o) { return new EditorWorkerServiceImpl(StaticServices.modelService.get(o), StaticServices.resourceConfigurationService.get(o)); });
    StaticServices.standaloneThemeService = define(IStandaloneThemeService, function () { return new StandaloneThemeServiceImpl(); });
    StaticServices.codeEditorService = define(ICodeEditorService, function (o) { return new StandaloneCodeEditorServiceImpl(StaticServices.standaloneThemeService.get(o)); });
    StaticServices.progressService = define(IProgressService, function () { return new SimpleProgressService(); });
    StaticServices.storageService = define(IStorageService, function () { return NullStorageService; });
    StaticServices.logService = define(ILogService, function () { return new NullLogService(); });
})(StaticServices || (StaticServices = {}));
var DynamicStandaloneServices = /** @class */ (function (_super) {
    __extends$2j(DynamicStandaloneServices, _super);
    function DynamicStandaloneServices(domElement, overrides) {
        var _this = _super.call(this) || this;
        var _a = StaticServices.init(overrides), _serviceCollection = _a[0], _instantiationService = _a[1];
        _this._serviceCollection = _serviceCollection;
        _this._instantiationService = _instantiationService;
        var configurationService = _this.get(IConfigurationService);
        var notificationService = _this.get(INotificationService);
        var telemetryService = _this.get(ITelemetryService);
        var themeService = _this.get(IThemeService);
        var ensure = function (serviceId, factory) {
            var value = null;
            if (overrides) {
                value = overrides[serviceId.toString()];
            }
            if (!value) {
                value = factory();
            }
            _this._serviceCollection.set(serviceId, value);
            return value;
        };
        var contextKeyService = ensure(IContextKeyService, function () { return _this._register(new ContextKeyService(configurationService)); });
        ensure(IListService, function () { return new ListService(contextKeyService); });
        var commandService = ensure(ICommandService, function () { return new StandaloneCommandService(_this._instantiationService); });
        var keybindingService = ensure(IKeybindingService, function () { return _this._register(new StandaloneKeybindingService(contextKeyService, commandService, telemetryService, notificationService, domElement)); });
        var contextViewService = ensure(IContextViewService, function () { return _this._register(new ContextViewService(domElement, telemetryService, new NullLogService())); });
        ensure(IContextMenuService, function () { return _this._register(new ContextMenuService(domElement, telemetryService, notificationService, contextViewService, keybindingService, themeService)); });
        ensure(IMenuService, function () { return new SimpleMenuService(commandService); });
        ensure(IBulkEditService, function () { return new SimpleBulkEditService(StaticServices.modelService.get(IModelService)); });
        return _this;
    }
    DynamicStandaloneServices.prototype.get = function (serviceId) {
        var r = this._serviceCollection.get(serviceId);
        if (!r) {
            throw new Error('Missing service ' + serviceId);
        }
        return r;
    };
    DynamicStandaloneServices.prototype.set = function (serviceId, instance) {
        this._serviceCollection.set(serviceId, instance);
    };
    DynamicStandaloneServices.prototype.has = function (serviceId) {
        return this._serviceCollection.has(serviceId);
    };
    return DynamicStandaloneServices;
}(Disposable));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function withAllStandaloneServices(domElement, override, callback) {
    var services = new DynamicStandaloneServices(domElement, override);
    var simpleEditorModelResolverService = null;
    if (!services.has(ITextModelService)) {
        simpleEditorModelResolverService = new SimpleEditorModelResolverService();
        services.set(ITextModelService, simpleEditorModelResolverService);
    }
    if (!services.has(IOpenerService)) {
        services.set(IOpenerService, new OpenerService(services.get(ICodeEditorService), services.get(ICommandService)));
    }
    var result = callback(services);
    if (simpleEditorModelResolverService) {
        simpleEditorModelResolverService.setEditor(result);
    }
    return result;
}
/**
 * Create a new editor under `domElement`.
 * `domElement` should be empty (not contain other dom nodes).
 * The editor will read the size of `domElement`.
 */
function create$3(domElement, options, override) {
    return withAllStandaloneServices(domElement, override || {}, function (services) {
        return new StandaloneEditor(domElement, options, services, services.get(IInstantiationService), services.get(ICodeEditorService), services.get(ICommandService), services.get(IContextKeyService), services.get(IKeybindingService), services.get(IContextViewService), services.get(IStandaloneThemeService), services.get(INotificationService), services.get(IConfigurationService));
    });
}
/**
 * Emitted when an editor is created.
 * Creating a diff editor might cause this listener to be invoked with the two editors.
 * @event
 */
function onDidCreateEditor(listener) {
    return StaticServices.codeEditorService.get().onCodeEditorAdd(function (editor) {
        listener(editor);
    });
}
/**
 * Create a new diff editor under `domElement`.
 * `domElement` should be empty (not contain other dom nodes).
 * The editor will read the size of `domElement`.
 */
function createDiffEditor(domElement, options, override) {
    return withAllStandaloneServices(domElement, override, function (services) {
        return new StandaloneDiffEditor(domElement, options, services, services.get(IInstantiationService), services.get(IContextKeyService), services.get(IKeybindingService), services.get(IContextViewService), services.get(IEditorWorkerService), services.get(ICodeEditorService), services.get(IStandaloneThemeService), services.get(INotificationService), services.get(IConfigurationService));
    });
}
function createDiffNavigator(diffEditor, opts) {
    return new DiffNavigator(diffEditor, opts);
}
function doCreateModel(value, languageSelection, uri) {
    return StaticServices.modelService.get().createModel(value, languageSelection, uri);
}
/**
 * Create a new editor model.
 * You can specify the language that should be set for this model or let the language be inferred from the `uri`.
 */
function createModel(value, language, uri) {
    value = value || '';
    if (!language) {
        var path = uri ? uri.path : null;
        var firstLF = value.indexOf('\n');
        var firstLine = value;
        if (firstLF !== -1) {
            firstLine = value.substring(0, firstLF);
        }
        return doCreateModel(value, StaticServices.modeService.get().createByFilepathOrFirstLine(path, firstLine), uri);
    }
    return doCreateModel(value, StaticServices.modeService.get().create(language), uri);
}
/**
 * Change the language for a model.
 */
function setModelLanguage(model, languageId) {
    StaticServices.modelService.get().setMode(model, StaticServices.modeService.get().create(languageId));
}
/**
 * Set the markers for a model.
 */
function setModelMarkers(model, owner, markers) {
    if (model) {
        StaticServices.markerService.get().changeOne(owner, model.uri, markers);
    }
}
/**
 * Get markers for owner and/or resource
 * @returns {IMarker[]} list of markers
 * @param filter
 */
function getModelMarkers(filter) {
    return StaticServices.markerService.get().read(filter);
}
/**
 * Get the model that has `uri` if it exists.
 */
function getModel(uri) {
    return StaticServices.modelService.get().getModel(uri);
}
/**
 * Get all the created models.
 */
function getModels() {
    return StaticServices.modelService.get().getModels();
}
/**
 * Emitted when a model is created.
 * @event
 */
function onDidCreateModel(listener) {
    return StaticServices.modelService.get().onModelAdded(listener);
}
/**
 * Emitted right before a model is disposed.
 * @event
 */
function onWillDisposeModel(listener) {
    return StaticServices.modelService.get().onModelRemoved(listener);
}
/**
 * Emitted when a different language is set to a model.
 * @event
 */
function onDidChangeModelLanguage(listener) {
    return StaticServices.modelService.get().onModelModeChanged(function (e) {
        listener({
            model: e.model,
            oldLanguage: e.oldModeId
        });
    });
}
/**
 * Create a new web worker that has model syncing capabilities built in.
 * Specify an AMD module to load that will `create` an object that will be proxied.
 */
function createWebWorker$1(opts) {
    return createWebWorker(StaticServices.modelService.get(), opts);
}
/**
 * Colorize the contents of `domNode` using attribute `data-lang`.
 */
function colorizeElement(domNode, options) {
    return Colorizer.colorizeElement(StaticServices.standaloneThemeService.get(), StaticServices.modeService.get(), domNode, options);
}
/**
 * Colorize `text` using language `languageId`.
 */
function colorize(text, languageId, options) {
    return Colorizer.colorize(StaticServices.modeService.get(), text, languageId, options);
}
/**
 * Colorize a line in a model.
 */
function colorizeModelLine(model, lineNumber, tabSize) {
    if (tabSize === void 0) { tabSize = 4; }
    return Colorizer.colorizeModelLine(model, lineNumber, tabSize);
}
/**
 * @internal
 */
function getSafeTokenizationSupport$1(language) {
    var tokenizationSupport = TokenizationRegistry.get(language);
    if (tokenizationSupport) {
        return tokenizationSupport;
    }
    return {
        getInitialState: function () { return NULL_STATE; },
        tokenize: function (line, state, deltaOffset) { return nullTokenize(language, line, state, deltaOffset); },
        tokenize2: undefined,
    };
}
/**
 * Tokenize `text` using language `languageId`
 */
function tokenize(text, languageId) {
    var modeService = StaticServices.modeService.get();
    // Needed in order to get the mode registered for subsequent look-ups
    modeService.triggerMode(languageId);
    var tokenizationSupport = getSafeTokenizationSupport$1(languageId);
    var lines = text.split(/\r\n|\r|\n/);
    var result = [];
    var state = tokenizationSupport.getInitialState();
    for (var i = 0, len = lines.length; i < len; i++) {
        var line = lines[i];
        var tokenizationResult = tokenizationSupport.tokenize(line, state, 0);
        result[i] = tokenizationResult.tokens;
        state = tokenizationResult.endState;
    }
    return result;
}
/**
 * Define a new theme or update an existing theme.
 */
function defineTheme(themeName, themeData) {
    StaticServices.standaloneThemeService.get().defineTheme(themeName, themeData);
}
/**
 * Switches to a theme.
 */
function setTheme(themeName) {
    StaticServices.standaloneThemeService.get().setTheme(themeName);
}
/**
 * @internal
 */
function createMonacoEditorAPI() {
    return {
        // methods
        create: create$3,
        onDidCreateEditor: onDidCreateEditor,
        createDiffEditor: createDiffEditor,
        createDiffNavigator: createDiffNavigator,
        createModel: createModel,
        setModelLanguage: setModelLanguage,
        setModelMarkers: setModelMarkers,
        getModelMarkers: getModelMarkers,
        getModels: getModels,
        getModel: getModel,
        onDidCreateModel: onDidCreateModel,
        onWillDisposeModel: onWillDisposeModel,
        onDidChangeModelLanguage: onDidChangeModelLanguage,
        createWebWorker: createWebWorker$1,
        colorizeElement: colorizeElement,
        colorize: colorize,
        colorizeModelLine: colorizeModelLine,
        tokenize: tokenize,
        defineTheme: defineTheme,
        setTheme: setTheme,
        // enums
        ScrollbarVisibility: ScrollbarVisibility,
        WrappingIndent: WrappingIndent,
        OverviewRulerLane: OverviewRulerLane$1,
        EndOfLinePreference: EndOfLinePreference,
        DefaultEndOfLine: DefaultEndOfLine,
        EndOfLineSequence: EndOfLineSequence,
        TrackedRangeStickiness: TrackedRangeStickiness,
        CursorChangeReason: CursorChangeReason,
        MouseTargetType: MouseTargetType,
        TextEditorCursorStyle: TextEditorCursorStyle$1,
        TextEditorCursorBlinkingStyle: TextEditorCursorBlinkingStyle,
        ContentWidgetPositionPreference: ContentWidgetPositionPreference,
        OverlayWidgetPositionPreference: OverlayWidgetPositionPreference,
        RenderMinimap: RenderMinimap,
        ScrollType: ScrollType,
        RenderLineNumbersType: RenderLineNumbersType,
        // classes
        InternalEditorOptions: InternalEditorOptions,
        BareFontInfo: BareFontInfo,
        FontInfo: FontInfo,
        TextModelResolvedOptions: TextModelResolvedOptions,
        FindMatch: FindMatch,
        // vars
        EditorType: EditorType
    };
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function isFuzzyActionArr(what) {
    return (Array.isArray(what));
}
function isFuzzyAction(what) {
    return !isFuzzyActionArr(what);
}
function isString$1(what) {
    return (typeof what === 'string');
}
function isIAction(what) {
    return !isString$1(what);
}
// Small helper functions
/**
 * Is a string null, undefined, or empty?
 */
function empty$1(s) {
    return (s ? false : true);
}
/**
 * Puts a string to lower case if 'ignoreCase' is set.
 */
function fixCase(lexer, str) {
    return (lexer.ignoreCase && str ? str.toLowerCase() : str);
}
/**
 * Ensures there are no bad characters in a CSS token class.
 */
function sanitize(s) {
    return s.replace(/[&<>'"_]/g, '-'); // used on all output token CSS classes
}
// Logging
/**
 * Logs a message.
 */
function log(lexer, msg) {
    console.log(lexer.languageId + ": " + msg);
}
// Throwing errors
function createError(lexer, msg) {
    return new Error(lexer.languageId + ": " + msg);
}
// Helper functions for rule finding and substitution
/**
 * substituteMatches is used on lexer strings and can substitutes predefined patterns:
 * 		$$  => $
 * 		$#  => id
 * 		$n  => matched entry n
 * 		@attr => contents of lexer[attr]
 *
 * See documentation for more info
 */
function substituteMatches(lexer, str, id, matches, state) {
    var re = /\$((\$)|(#)|(\d\d?)|[sS](\d\d?)|@(\w+))/g;
    var stateMatches = null;
    return str.replace(re, function (full, sub, dollar, hash, n, s, attr, ofs, total) {
        if (!empty$1(dollar)) {
            return '$'; // $$
        }
        if (!empty$1(hash)) {
            return fixCase(lexer, id); // default $#
        }
        if (!empty$1(n) && n < matches.length) {
            return fixCase(lexer, matches[n]); // $n
        }
        if (!empty$1(attr) && lexer && typeof (lexer[attr]) === 'string') {
            return lexer[attr]; //@attribute
        }
        if (stateMatches === null) { // split state on demand
            stateMatches = state.split('.');
            stateMatches.unshift(state);
        }
        if (!empty$1(s) && s < stateMatches.length) {
            return fixCase(lexer, stateMatches[s]); //$Sn
        }
        return '';
    });
}
/**
 * Find the tokenizer rules for a specific state (i.e. next action)
 */
function findRules(lexer, inState) {
    var state = inState;
    while (state && state.length > 0) {
        var rules = lexer.tokenizer[state];
        if (rules) {
            return rules;
        }
        var idx = state.lastIndexOf('.');
        if (idx < 0) {
            state = null; // no further parent
        }
        else {
            state = state.substr(0, idx);
        }
    }
    return null;
}
/**
 * Is a certain state defined? In contrast to 'findRules' this works on a ILexerMin.
 * This is used during compilation where we may know the defined states
 * but not yet whether the corresponding rules are correct.
 */
function stateExists(lexer, inState) {
    var state = inState;
    while (state && state.length > 0) {
        var exist = lexer.stateNames[state];
        if (exist) {
            return true;
        }
        var idx = state.lastIndexOf('.');
        if (idx < 0) {
            state = null; // no further parent
        }
        else {
            state = state.substr(0, idx);
        }
    }
    return false;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/*
 * Type helpers
 *
 * Note: this is just for sanity checks on the JSON description which is
 * helpful for the programmer. No checks are done anymore once the lexer is
 * already 'compiled and checked'.
 *
 */
function isArrayOf(elemType, obj) {
    if (!obj) {
        return false;
    }
    if (!(Array.isArray(obj))) {
        return false;
    }
    for (var idx in obj) {
        if (obj.hasOwnProperty(idx)) {
            if (!(elemType(obj[idx]))) {
                return false;
            }
        }
    }
    return true;
}
function bool(prop, defValue) {
    if (typeof prop === 'boolean') {
        return prop;
    }
    return defValue;
}
function string(prop, defValue) {
    if (typeof (prop) === 'string') {
        return prop;
    }
    return defValue;
}
// Lexer helpers
/**
 * Compiles a regular expression string, adding the 'i' flag if 'ignoreCase' is set.
 * Also replaces @\w+ or sequences with the content of the specified attribute
 */
function compileRegExp(lexer, str) {
    var n = 0;
    while (str.indexOf('@') >= 0 && n < 5) { // at most 5 expansions
        n++;
        str = str.replace(/@(\w+)/g, function (s, attr) {
            var sub = '';
            if (typeof (lexer[attr]) === 'string') {
                sub = lexer[attr];
            }
            else if (lexer[attr] && lexer[attr] instanceof RegExp) {
                sub = lexer[attr].source;
            }
            else {
                if (lexer[attr] === undefined) {
                    throw createError(lexer, 'language definition does not contain attribute \'' + attr + '\', used at: ' + str);
                }
                else {
                    throw createError(lexer, 'attribute reference \'' + attr + '\' must be a string, used at: ' + str);
                }
            }
            return (empty$1(sub) ? '' : '(?:' + sub + ')');
        });
    }
    return new RegExp(str, (lexer.ignoreCase ? 'i' : ''));
}
/**
 * Compiles guard functions for case matches.
 * This compiles 'cases' attributes into efficient match functions.
 *
 */
function selectScrutinee(id, matches, state, num) {
    if (num < 0) {
        return id;
    }
    if (num < matches.length) {
        return matches[num];
    }
    if (num >= 100) {
        num = num - 100;
        var parts = state.split('.');
        parts.unshift(state);
        if (num < parts.length) {
            return parts[num];
        }
    }
    return null;
}
function createGuard(lexer, ruleName, tkey, val) {
    // get the scrutinee and pattern
    var scrut = -1; // -1: $!, 0-99: $n, 100+n: $Sn
    var oppat = tkey;
    var matches = tkey.match(/^\$(([sS]?)(\d\d?)|#)(.*)$/);
    if (matches) {
        if (matches[3]) { // if digits
            scrut = parseInt(matches[3]);
            if (matches[2]) {
                scrut = scrut + 100; // if [sS] present
            }
        }
        oppat = matches[4];
    }
    // get operator
    var op = '~';
    var pat = oppat;
    if (!oppat || oppat.length === 0) {
        op = '!=';
        pat = '';
    }
    else if (/^\w*$/.test(pat)) { // just a word
        op = '==';
    }
    else {
        matches = oppat.match(/^(@|!@|~|!~|==|!=)(.*)$/);
        if (matches) {
            op = matches[1];
            pat = matches[2];
        }
    }
    // set the tester function
    var tester;
    // special case a regexp that matches just words
    if ((op === '~' || op === '!~') && /^(\w|\|)*$/.test(pat)) {
        var inWords_1 = createKeywordMatcher(pat.split('|'), lexer.ignoreCase);
        tester = function (s) { return (op === '~' ? inWords_1(s) : !inWords_1(s)); };
    }
    else if (op === '@' || op === '!@') {
        var words = lexer[pat];
        if (!words) {
            throw createError(lexer, 'the @ match target \'' + pat + '\' is not defined, in rule: ' + ruleName);
        }
        if (!(isArrayOf(function (elem) { return (typeof (elem) === 'string'); }, words))) {
            throw createError(lexer, 'the @ match target \'' + pat + '\' must be an array of strings, in rule: ' + ruleName);
        }
        var inWords_2 = createKeywordMatcher(words, lexer.ignoreCase);
        tester = function (s) { return (op === '@' ? inWords_2(s) : !inWords_2(s)); };
    }
    else if (op === '~' || op === '!~') {
        if (pat.indexOf('$') < 0) {
            // precompile regular expression
            var re_1 = compileRegExp(lexer, '^' + pat + '$');
            tester = function (s) { return (op === '~' ? re_1.test(s) : !re_1.test(s)); };
        }
        else {
            tester = function (s, id, matches, state) {
                var re = compileRegExp(lexer, '^' + substituteMatches(lexer, pat, id, matches, state) + '$');
                return re.test(s);
            };
        }
    }
    else { // if (op==='==' || op==='!=') {
        if (pat.indexOf('$') < 0) {
            var patx_1 = fixCase(lexer, pat);
            tester = function (s) { return (op === '==' ? s === patx_1 : s !== patx_1); };
        }
        else {
            var patx_2 = fixCase(lexer, pat);
            tester = function (s, id, matches, state, eos) {
                var patexp = substituteMatches(lexer, patx_2, id, matches, state);
                return (op === '==' ? s === patexp : s !== patexp);
            };
        }
    }
    // return the branch object
    if (scrut === -1) {
        return {
            name: tkey, value: val, test: function (id, matches, state, eos) {
                return tester(id, id, matches, state, eos);
            }
        };
    }
    else {
        return {
            name: tkey, value: val, test: function (id, matches, state, eos) {
                var scrutinee = selectScrutinee(id, matches, state, scrut);
                return tester(!scrutinee ? '' : scrutinee, id, matches, state, eos);
            }
        };
    }
}
/**
 * Compiles an action: i.e. optimize regular expressions and case matches
 * and do many sanity checks.
 *
 * This is called only during compilation but if the lexer definition
 * contains user functions as actions (which is usually not allowed), then this
 * may be called during lexing. It is important therefore to compile common cases efficiently
 */
function compileAction(lexer, ruleName, action) {
    if (!action) {
        return { token: '' };
    }
    else if (typeof (action) === 'string') {
        return action; // { token: action };
    }
    else if (action.token || action.token === '') {
        if (typeof (action.token) !== 'string') {
            throw createError(lexer, 'a \'token\' attribute must be of type string, in rule: ' + ruleName);
            return { token: '' };
        }
        else {
            // only copy specific typed fields (only happens once during compile Lexer)
            var newAction = { token: action.token };
            if (action.token.indexOf('$') >= 0) {
                newAction.tokenSubst = true;
            }
            if (typeof (action.bracket) === 'string') {
                if (action.bracket === '@open') {
                    newAction.bracket = 1 /* Open */;
                }
                else if (action.bracket === '@close') {
                    newAction.bracket = -1 /* Close */;
                }
                else {
                    throw createError(lexer, 'a \'bracket\' attribute must be either \'@open\' or \'@close\', in rule: ' + ruleName);
                }
            }
            if (action.next) {
                if (typeof (action.next) !== 'string') {
                    throw createError(lexer, 'the next state must be a string value in rule: ' + ruleName);
                }
                else {
                    var next = action.next;
                    if (!/^(@pop|@push|@popall)$/.test(next)) {
                        if (next[0] === '@') {
                            next = next.substr(1); // peel off starting @ sign
                        }
                        if (next.indexOf('$') < 0) { // no dollar substitution, we can check if the state exists
                            if (!stateExists(lexer, substituteMatches(lexer, next, '', [], ''))) {
                                throw createError(lexer, 'the next state \'' + action.next + '\' is not defined in rule: ' + ruleName);
                            }
                        }
                    }
                    newAction.next = next;
                }
            }
            if (typeof (action.goBack) === 'number') {
                newAction.goBack = action.goBack;
            }
            if (typeof (action.switchTo) === 'string') {
                newAction.switchTo = action.switchTo;
            }
            if (typeof (action.log) === 'string') {
                newAction.log = action.log;
            }
            if (typeof (action.nextEmbedded) === 'string') {
                newAction.nextEmbedded = action.nextEmbedded;
                lexer.usesEmbedded = true;
            }
            return newAction;
        }
    }
    else if (Array.isArray(action)) {
        var results = [];
        for (var idx in action) {
            if (action.hasOwnProperty(idx)) {
                results[idx] = compileAction(lexer, ruleName, action[idx]);
            }
        }
        return { group: results };
    }
    else if (action.cases) {
        // build an array of test cases
        var cases_1 = [];
        // for each case, push a test function and result value
        for (var tkey in action.cases) {
            if (action.cases.hasOwnProperty(tkey)) {
                var val = compileAction(lexer, ruleName, action.cases[tkey]);
                // what kind of case
                if (tkey === '@default' || tkey === '@' || tkey === '') {
                    cases_1.push({ test: undefined, value: val, name: tkey });
                }
                else if (tkey === '@eos') {
                    cases_1.push({ test: function (id, matches, state, eos) { return eos; }, value: val, name: tkey });
                }
                else {
                    cases_1.push(createGuard(lexer, ruleName, tkey, val)); // call separate function to avoid local variable capture
                }
            }
        }
        // create a matching function
        var def_1 = lexer.defaultToken;
        return {
            test: function (id, matches, state, eos) {
                for (var idx in cases_1) {
                    if (cases_1.hasOwnProperty(idx)) {
                        var _case = cases_1[idx];
                        var didmatch = (!_case.test || _case.test(id, matches, state, eos));
                        if (didmatch) {
                            return _case.value;
                        }
                    }
                }
                return def_1;
            }
        };
    }
    else {
        throw createError(lexer, 'an action must be a string, an object with a \'token\' or \'cases\' attribute, or an array of actions; in rule: ' + ruleName);
        return '';
    }
}
/**
 * Helper class for creating matching rules
 */
var Rule = /** @class */ (function () {
    function Rule(name) {
        this.regex = new RegExp('');
        this.action = { token: '' };
        this.matchOnlyAtLineStart = false;
        this.name = '';
        this.name = name;
    }
    Rule.prototype.setRegex = function (lexer, re) {
        var sregex;
        if (typeof (re) === 'string') {
            sregex = re;
        }
        else if (re instanceof RegExp) {
            sregex = re.source;
        }
        else {
            throw createError(lexer, 'rules must start with a match string or regular expression: ' + this.name);
        }
        this.matchOnlyAtLineStart = (sregex.length > 0 && sregex[0] === '^');
        this.name = this.name + ': ' + sregex;
        this.regex = compileRegExp(lexer, '^(?:' + (this.matchOnlyAtLineStart ? sregex.substr(1) : sregex) + ')');
    };
    Rule.prototype.setAction = function (lexer, act) {
        this.action = compileAction(lexer, this.name, act);
    };
    return Rule;
}());
/**
 * Compiles a json description function into json where all regular expressions,
 * case matches etc, are compiled and all include rules are expanded.
 * We also compile the bracket definitions, supply defaults, and do many sanity checks.
 * If the 'jsonStrict' parameter is 'false', we allow at certain locations
 * regular expression objects and functions that get called during lexing.
 * (Currently we have no samples that need this so perhaps we should always have
 * jsonStrict to true).
 */
function compile(languageId, json) {
    if (!json || typeof (json) !== 'object') {
        throw new Error('Monarch: expecting a language definition object');
    }
    // Create our lexer
    var lexer = {};
    lexer.languageId = languageId;
    lexer.noThrow = false; // raise exceptions during compilation
    lexer.maxStack = 100;
    // Set standard fields: be defensive about types
    lexer.start = (typeof json.start === 'string' ? json.start : null);
    lexer.ignoreCase = bool(json.ignoreCase, false);
    lexer.tokenPostfix = string(json.tokenPostfix, '.' + lexer.languageId);
    lexer.defaultToken = string(json.defaultToken, 'source');
    lexer.usesEmbedded = false; // becomes true if we find a nextEmbedded action
    // For calling compileAction later on
    var lexerMin = json;
    lexerMin.languageId = languageId;
    lexerMin.ignoreCase = lexer.ignoreCase;
    lexerMin.noThrow = lexer.noThrow;
    lexerMin.usesEmbedded = lexer.usesEmbedded;
    lexerMin.stateNames = json.tokenizer;
    lexerMin.defaultToken = lexer.defaultToken;
    // Compile an array of rules into newrules where RegExp objects are created.
    function addRules(state, newrules, rules) {
        for (var idx in rules) {
            if (rules.hasOwnProperty(idx)) {
                var rule = rules[idx];
                var include = rule.include;
                if (include) {
                    if (typeof (include) !== 'string') {
                        throw createError(lexer, 'an \'include\' attribute must be a string at: ' + state);
                    }
                    if (include[0] === '@') {
                        include = include.substr(1); // peel off starting @
                    }
                    if (!json.tokenizer[include]) {
                        throw createError(lexer, 'include target \'' + include + '\' is not defined at: ' + state);
                    }
                    addRules(state + '.' + include, newrules, json.tokenizer[include]);
                }
                else {
                    var newrule = new Rule(state);
                    // Set up new rule attributes
                    if (Array.isArray(rule) && rule.length >= 1 && rule.length <= 3) {
                        newrule.setRegex(lexerMin, rule[0]);
                        if (rule.length >= 3) {
                            if (typeof (rule[1]) === 'string') {
                                newrule.setAction(lexerMin, { token: rule[1], next: rule[2] });
                            }
                            else if (typeof (rule[1]) === 'object') {
                                var rule1 = rule[1];
                                rule1.next = rule[2];
                                newrule.setAction(lexerMin, rule1);
                            }
                            else {
                                throw createError(lexer, 'a next state as the last element of a rule can only be given if the action is either an object or a string, at: ' + state);
                            }
                        }
                        else {
                            newrule.setAction(lexerMin, rule[1]);
                        }
                    }
                    else {
                        if (!rule.regex) {
                            throw createError(lexer, 'a rule must either be an array, or an object with a \'regex\' or \'include\' field at: ' + state);
                        }
                        if (rule.name) {
                            if (typeof rule.name === 'string') {
                                newrule.name = rule.name;
                            }
                        }
                        if (rule.matchOnlyAtStart) {
                            newrule.matchOnlyAtLineStart = bool(rule.matchOnlyAtLineStart, false);
                        }
                        newrule.setRegex(lexerMin, rule.regex);
                        newrule.setAction(lexerMin, rule.action);
                    }
                    newrules.push(newrule);
                }
            }
        }
    }
    // compile the tokenizer rules
    if (!json.tokenizer || typeof (json.tokenizer) !== 'object') {
        throw createError(lexer, 'a language definition must define the \'tokenizer\' attribute as an object');
    }
    lexer.tokenizer = [];
    for (var key in json.tokenizer) {
        if (json.tokenizer.hasOwnProperty(key)) {
            if (!lexer.start) {
                lexer.start = key;
            }
            var rules = json.tokenizer[key];
            lexer.tokenizer[key] = new Array();
            addRules('tokenizer.' + key, lexer.tokenizer[key], rules);
        }
    }
    lexer.usesEmbedded = lexerMin.usesEmbedded; // can be set during compileAction
    // Set simple brackets
    if (json.brackets) {
        if (!(Array.isArray(json.brackets))) {
            throw createError(lexer, 'the \'brackets\' attribute must be defined as an array');
        }
    }
    else {
        json.brackets = [
            { open: '{', close: '}', token: 'delimiter.curly' },
            { open: '[', close: ']', token: 'delimiter.square' },
            { open: '(', close: ')', token: 'delimiter.parenthesis' },
            { open: '<', close: '>', token: 'delimiter.angle' }
        ];
    }
    var brackets = [];
    for (var bracketIdx in json.brackets) {
        if (json.brackets.hasOwnProperty(bracketIdx)) {
            var desc = json.brackets[bracketIdx];
            if (desc && Array.isArray(desc) && desc.length === 3) {
                desc = { token: desc[2], open: desc[0], close: desc[1] };
            }
            if (desc.open === desc.close) {
                throw createError(lexer, 'open and close brackets in a \'brackets\' attribute must be different: ' + desc.open +
                    '\n hint: use the \'bracket\' attribute if matching on equal brackets is required.');
            }
            if (typeof desc.open === 'string' && typeof desc.token === 'string' && typeof desc.close === 'string') {
                brackets.push({
                    token: desc.token + lexer.tokenPostfix,
                    open: fixCase(lexer, desc.open),
                    close: fixCase(lexer, desc.close)
                });
            }
            else {
                throw createError(lexer, 'every element in the \'brackets\' array must be a \'{open,close,token}\' object or array');
            }
        }
    }
    lexer.brackets = brackets;
    // Disable throw so the syntax highlighter goes, no matter what
    lexer.noThrow = true;
    return lexer;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var CACHE_STACK_DEPTH = 5;
/**
 * Reuse the same stack elements up to a certain depth.
 */
var MonarchStackElementFactory = /** @class */ (function () {
    function MonarchStackElementFactory(maxCacheDepth) {
        this._maxCacheDepth = maxCacheDepth;
        this._entries = Object.create(null);
    }
    MonarchStackElementFactory.create = function (parent, state) {
        return this._INSTANCE.create(parent, state);
    };
    MonarchStackElementFactory.prototype.create = function (parent, state) {
        if (parent !== null && parent.depth >= this._maxCacheDepth) {
            // no caching above a certain depth
            return new MonarchStackElement(parent, state);
        }
        var stackElementId = MonarchStackElement.getStackElementId(parent);
        if (stackElementId.length > 0) {
            stackElementId += '|';
        }
        stackElementId += state;
        var result = this._entries[stackElementId];
        if (result) {
            return result;
        }
        result = new MonarchStackElement(parent, state);
        this._entries[stackElementId] = result;
        return result;
    };
    MonarchStackElementFactory._INSTANCE = new MonarchStackElementFactory(CACHE_STACK_DEPTH);
    return MonarchStackElementFactory;
}());
var MonarchStackElement = /** @class */ (function () {
    function MonarchStackElement(parent, state) {
        this.parent = parent;
        this.state = state;
        this.depth = (this.parent ? this.parent.depth : 0) + 1;
    }
    MonarchStackElement.getStackElementId = function (element) {
        var result = '';
        while (element !== null) {
            if (result.length > 0) {
                result += '|';
            }
            result += element.state;
            element = element.parent;
        }
        return result;
    };
    MonarchStackElement._equals = function (a, b) {
        while (a !== null && b !== null) {
            if (a === b) {
                return true;
            }
            if (a.state !== b.state) {
                return false;
            }
            a = a.parent;
            b = b.parent;
        }
        if (a === null && b === null) {
            return true;
        }
        return false;
    };
    MonarchStackElement.prototype.equals = function (other) {
        return MonarchStackElement._equals(this, other);
    };
    MonarchStackElement.prototype.push = function (state) {
        return MonarchStackElementFactory.create(this, state);
    };
    MonarchStackElement.prototype.pop = function () {
        return this.parent;
    };
    MonarchStackElement.prototype.popall = function () {
        var result = this;
        while (result.parent) {
            result = result.parent;
        }
        return result;
    };
    MonarchStackElement.prototype.switchTo = function (state) {
        return MonarchStackElementFactory.create(this.parent, state);
    };
    return MonarchStackElement;
}());
var EmbeddedModeData = /** @class */ (function () {
    function EmbeddedModeData(modeId, state) {
        this.modeId = modeId;
        this.state = state;
    }
    EmbeddedModeData.prototype.equals = function (other) {
        return (this.modeId === other.modeId
            && this.state.equals(other.state));
    };
    EmbeddedModeData.prototype.clone = function () {
        var stateClone = this.state.clone();
        // save an object
        if (stateClone === this.state) {
            return this;
        }
        return new EmbeddedModeData(this.modeId, this.state);
    };
    return EmbeddedModeData;
}());
/**
 * Reuse the same line states up to a certain depth.
 */
var MonarchLineStateFactory = /** @class */ (function () {
    function MonarchLineStateFactory(maxCacheDepth) {
        this._maxCacheDepth = maxCacheDepth;
        this._entries = Object.create(null);
    }
    MonarchLineStateFactory.create = function (stack, embeddedModeData) {
        return this._INSTANCE.create(stack, embeddedModeData);
    };
    MonarchLineStateFactory.prototype.create = function (stack, embeddedModeData) {
        if (embeddedModeData !== null) {
            // no caching when embedding
            return new MonarchLineState(stack, embeddedModeData);
        }
        if (stack !== null && stack.depth >= this._maxCacheDepth) {
            // no caching above a certain depth
            return new MonarchLineState(stack, embeddedModeData);
        }
        var stackElementId = MonarchStackElement.getStackElementId(stack);
        var result = this._entries[stackElementId];
        if (result) {
            return result;
        }
        result = new MonarchLineState(stack, null);
        this._entries[stackElementId] = result;
        return result;
    };
    MonarchLineStateFactory._INSTANCE = new MonarchLineStateFactory(CACHE_STACK_DEPTH);
    return MonarchLineStateFactory;
}());
var MonarchLineState = /** @class */ (function () {
    function MonarchLineState(stack, embeddedModeData) {
        this.stack = stack;
        this.embeddedModeData = embeddedModeData;
    }
    MonarchLineState.prototype.clone = function () {
        var embeddedModeDataClone = this.embeddedModeData ? this.embeddedModeData.clone() : null;
        // save an object
        if (embeddedModeDataClone === this.embeddedModeData) {
            return this;
        }
        return MonarchLineStateFactory.create(this.stack, this.embeddedModeData);
    };
    MonarchLineState.prototype.equals = function (other) {
        if (!(other instanceof MonarchLineState)) {
            return false;
        }
        if (!this.stack.equals(other.stack)) {
            return false;
        }
        if (this.embeddedModeData === null && other.embeddedModeData === null) {
            return true;
        }
        if (this.embeddedModeData === null || other.embeddedModeData === null) {
            return false;
        }
        return this.embeddedModeData.equals(other.embeddedModeData);
    };
    return MonarchLineState;
}());
var hasOwnProperty$4 = Object.hasOwnProperty;
var MonarchClassicTokensCollector = /** @class */ (function () {
    function MonarchClassicTokensCollector() {
        this._tokens = [];
        this._language = null;
        this._lastTokenType = null;
        this._lastTokenLanguage = null;
    }
    MonarchClassicTokensCollector.prototype.enterMode = function (startOffset, modeId) {
        this._language = modeId;
    };
    MonarchClassicTokensCollector.prototype.emit = function (startOffset, type) {
        if (this._lastTokenType === type && this._lastTokenLanguage === this._language) {
            return;
        }
        this._lastTokenType = type;
        this._lastTokenLanguage = this._language;
        this._tokens.push(new Token(startOffset, type, this._language));
    };
    MonarchClassicTokensCollector.prototype.nestedModeTokenize = function (embeddedModeLine, embeddedModeData, offsetDelta) {
        var nestedModeId = embeddedModeData.modeId;
        var embeddedModeState = embeddedModeData.state;
        var nestedModeTokenizationSupport = TokenizationRegistry.get(nestedModeId);
        if (!nestedModeTokenizationSupport) {
            this.enterMode(offsetDelta, nestedModeId);
            this.emit(offsetDelta, '');
            return embeddedModeState;
        }
        var nestedResult = nestedModeTokenizationSupport.tokenize(embeddedModeLine, embeddedModeState, offsetDelta);
        this._tokens = this._tokens.concat(nestedResult.tokens);
        this._lastTokenType = null;
        this._lastTokenLanguage = null;
        this._language = null;
        return nestedResult.endState;
    };
    MonarchClassicTokensCollector.prototype.finalize = function (endState) {
        return new TokenizationResult(this._tokens, endState);
    };
    return MonarchClassicTokensCollector;
}());
var MonarchModernTokensCollector = /** @class */ (function () {
    function MonarchModernTokensCollector(modeService, theme) {
        this._modeService = modeService;
        this._theme = theme;
        this._prependTokens = null;
        this._tokens = [];
        this._currentLanguageId = 0 /* Null */;
        this._lastTokenMetadata = 0;
    }
    MonarchModernTokensCollector.prototype.enterMode = function (startOffset, modeId) {
        this._currentLanguageId = this._modeService.getLanguageIdentifier(modeId).id;
    };
    MonarchModernTokensCollector.prototype.emit = function (startOffset, type) {
        var metadata = this._theme.match(this._currentLanguageId, type);
        if (this._lastTokenMetadata === metadata) {
            return;
        }
        this._lastTokenMetadata = metadata;
        this._tokens.push(startOffset);
        this._tokens.push(metadata);
    };
    MonarchModernTokensCollector._merge = function (a, b, c) {
        var aLen = (a !== null ? a.length : 0);
        var bLen = b.length;
        var cLen = (c !== null ? c.length : 0);
        if (aLen === 0 && bLen === 0 && cLen === 0) {
            return new Uint32Array(0);
        }
        if (aLen === 0 && bLen === 0) {
            return c;
        }
        if (bLen === 0 && cLen === 0) {
            return a;
        }
        var result = new Uint32Array(aLen + bLen + cLen);
        if (a !== null) {
            result.set(a);
        }
        for (var i = 0; i < bLen; i++) {
            result[aLen + i] = b[i];
        }
        if (c !== null) {
            result.set(c, aLen + bLen);
        }
        return result;
    };
    MonarchModernTokensCollector.prototype.nestedModeTokenize = function (embeddedModeLine, embeddedModeData, offsetDelta) {
        var nestedModeId = embeddedModeData.modeId;
        var embeddedModeState = embeddedModeData.state;
        var nestedModeTokenizationSupport = TokenizationRegistry.get(nestedModeId);
        if (!nestedModeTokenizationSupport) {
            this.enterMode(offsetDelta, nestedModeId);
            this.emit(offsetDelta, '');
            return embeddedModeState;
        }
        var nestedResult = nestedModeTokenizationSupport.tokenize2(embeddedModeLine, embeddedModeState, offsetDelta);
        this._prependTokens = MonarchModernTokensCollector._merge(this._prependTokens, this._tokens, nestedResult.tokens);
        this._tokens = [];
        this._currentLanguageId = 0;
        this._lastTokenMetadata = 0;
        return nestedResult.endState;
    };
    MonarchModernTokensCollector.prototype.finalize = function (endState) {
        return new TokenizationResult2(MonarchModernTokensCollector._merge(this._prependTokens, this._tokens, null), endState);
    };
    return MonarchModernTokensCollector;
}());
var MonarchTokenizer = /** @class */ (function () {
    function MonarchTokenizer(modeService, standaloneThemeService, modeId, lexer) {
        var _this = this;
        this._modeService = modeService;
        this._standaloneThemeService = standaloneThemeService;
        this._modeId = modeId;
        this._lexer = lexer;
        this._embeddedModes = Object.create(null);
        // Set up listening for embedded modes
        var emitting = false;
        this._tokenizationRegistryListener = TokenizationRegistry.onDidChange(function (e) {
            if (emitting) {
                return;
            }
            var isOneOfMyEmbeddedModes = false;
            for (var i = 0, len = e.changedLanguages.length; i < len; i++) {
                var language = e.changedLanguages[i];
                if (_this._embeddedModes[language]) {
                    isOneOfMyEmbeddedModes = true;
                    break;
                }
            }
            if (isOneOfMyEmbeddedModes) {
                emitting = true;
                TokenizationRegistry.fire([_this._modeId]);
                emitting = false;
            }
        });
    }
    MonarchTokenizer.prototype.dispose = function () {
        this._tokenizationRegistryListener.dispose();
    };
    MonarchTokenizer.prototype.getInitialState = function () {
        var rootState = MonarchStackElementFactory.create(null, this._lexer.start);
        return MonarchLineStateFactory.create(rootState, null);
    };
    MonarchTokenizer.prototype.tokenize = function (line, lineState, offsetDelta) {
        var tokensCollector = new MonarchClassicTokensCollector();
        var endLineState = this._tokenize(line, lineState, offsetDelta, tokensCollector);
        return tokensCollector.finalize(endLineState);
    };
    MonarchTokenizer.prototype.tokenize2 = function (line, lineState, offsetDelta) {
        var tokensCollector = new MonarchModernTokensCollector(this._modeService, this._standaloneThemeService.getTheme().tokenTheme);
        var endLineState = this._tokenize(line, lineState, offsetDelta, tokensCollector);
        return tokensCollector.finalize(endLineState);
    };
    MonarchTokenizer.prototype._tokenize = function (line, lineState, offsetDelta, collector) {
        if (lineState.embeddedModeData) {
            return this._nestedTokenize(line, lineState, offsetDelta, collector);
        }
        else {
            return this._myTokenize(line, lineState, offsetDelta, collector);
        }
    };
    MonarchTokenizer.prototype._findLeavingNestedModeOffset = function (line, state) {
        var rules = this._lexer.tokenizer[state.stack.state];
        if (!rules) {
            rules = findRules(this._lexer, state.stack.state); // do parent matching
            if (!rules) {
                throw createError(this._lexer, 'tokenizer state is not defined: ' + state.stack.state);
            }
        }
        var popOffset = -1;
        var hasEmbeddedPopRule = false;
        for (var idx in rules) {
            if (!hasOwnProperty$4.call(rules, idx)) {
                continue;
            }
            var rule = rules[idx];
            if (!isIAction(rule.action) || rule.action.nextEmbedded !== '@pop') {
                continue;
            }
            hasEmbeddedPopRule = true;
            var regex = rule.regex;
            var regexSource = rule.regex.source;
            if (regexSource.substr(0, 4) === '^(?:' && regexSource.substr(regexSource.length - 1, 1) === ')') {
                regex = new RegExp(regexSource.substr(4, regexSource.length - 5), regex.ignoreCase ? 'i' : '');
            }
            var result = line.search(regex);
            if (result === -1) {
                continue;
            }
            if (popOffset === -1 || result < popOffset) {
                popOffset = result;
            }
        }
        if (!hasEmbeddedPopRule) {
            throw createError(this._lexer, 'no rule containing nextEmbedded: "@pop" in tokenizer embedded state: ' + state.stack.state);
        }
        return popOffset;
    };
    MonarchTokenizer.prototype._nestedTokenize = function (line, lineState, offsetDelta, tokensCollector) {
        var popOffset = this._findLeavingNestedModeOffset(line, lineState);
        if (popOffset === -1) {
            // tokenization will not leave nested mode
            var nestedEndState = tokensCollector.nestedModeTokenize(line, lineState.embeddedModeData, offsetDelta);
            return MonarchLineStateFactory.create(lineState.stack, new EmbeddedModeData(lineState.embeddedModeData.modeId, nestedEndState));
        }
        var nestedModeLine = line.substring(0, popOffset);
        if (nestedModeLine.length > 0) {
            // tokenize with the nested mode
            tokensCollector.nestedModeTokenize(nestedModeLine, lineState.embeddedModeData, offsetDelta);
        }
        var restOfTheLine = line.substring(popOffset);
        return this._myTokenize(restOfTheLine, lineState, offsetDelta + popOffset, tokensCollector);
    };
    MonarchTokenizer.prototype._myTokenize = function (line, lineState, offsetDelta, tokensCollector) {
        tokensCollector.enterMode(offsetDelta, this._modeId);
        var lineLength = line.length;
        var embeddedModeData = lineState.embeddedModeData;
        var stack = lineState.stack;
        var pos = 0;
        var groupMatching = null;
        while (pos < lineLength) {
            var pos0 = pos;
            var stackLen0 = stack.depth;
            var groupLen0 = groupMatching ? groupMatching.groups.length : 0;
            var state = stack.state;
            var matches = null;
            var matched = null;
            var action = null;
            var rule = null;
            var enteringEmbeddedMode = null;
            // check if we need to process group matches first
            if (groupMatching) {
                matches = groupMatching.matches;
                var groupEntry = groupMatching.groups.shift();
                matched = groupEntry.matched;
                action = groupEntry.action;
                rule = groupMatching.rule;
                // cleanup if necessary
                if (groupMatching.groups.length === 0) {
                    groupMatching = null;
                }
            }
            else {
                // otherwise we match on the token stream
                if (pos >= lineLength) {
                    // nothing to do
                    break;
                }
                // get the rules for this state
                var rules = this._lexer.tokenizer[state];
                if (!rules) {
                    rules = findRules(this._lexer, state); // do parent matching
                    if (!rules) {
                        throw createError(this._lexer, 'tokenizer state is not defined: ' + state);
                    }
                }
                // try each rule until we match
                var restOfLine = line.substr(pos);
                for (var idx in rules) {
                    if (hasOwnProperty$4.call(rules, idx)) {
                        var rule_1 = rules[idx];
                        if (pos === 0 || !rule_1.matchOnlyAtLineStart) {
                            matches = restOfLine.match(rule_1.regex);
                            if (matches) {
                                matched = matches[0];
                                action = rule_1.action;
                                break;
                            }
                        }
                    }
                }
            }
            // We matched 'rule' with 'matches' and 'action'
            if (!matches) {
                matches = [''];
                matched = '';
            }
            if (!action) {
                // bad: we didn't match anything, and there is no action to take
                // we need to advance the stream or we get progress trouble
                if (pos < lineLength) {
                    matches = [line.charAt(pos)];
                    matched = matches[0];
                }
                action = this._lexer.defaultToken;
            }
            // advance stream
            pos += matched.length;
            // maybe call action function (used for 'cases')
            while (isFuzzyAction(action) && isIAction(action) && action.test) {
                action = action.test(matched, matches, state, pos === lineLength);
            }
            var result = null;
            // set the result: either a string or an array of actions
            if (typeof action === 'string' || Array.isArray(action)) {
                result = action;
            }
            else if (action.group) {
                result = action.group;
            }
            else if (action.token !== null && action.token !== undefined) {
                // do $n replacements?
                if (action.tokenSubst) {
                    result = substituteMatches(this._lexer, action.token, matched, matches, state);
                }
                else {
                    result = action.token;
                }
                // enter embedded mode?
                if (action.nextEmbedded) {
                    if (action.nextEmbedded === '@pop') {
                        if (!embeddedModeData) {
                            throw createError(this._lexer, 'cannot pop embedded mode if not inside one');
                        }
                        embeddedModeData = null;
                    }
                    else if (embeddedModeData) {
                        throw createError(this._lexer, 'cannot enter embedded mode from within an embedded mode');
                    }
                    else {
                        enteringEmbeddedMode = substituteMatches(this._lexer, action.nextEmbedded, matched, matches, state);
                    }
                }
                // state transformations
                if (action.goBack) { // back up the stream..
                    pos = Math.max(0, pos - action.goBack);
                }
                if (action.switchTo && typeof action.switchTo === 'string') {
                    var nextState = substituteMatches(this._lexer, action.switchTo, matched, matches, state); // switch state without a push...
                    if (nextState[0] === '@') {
                        nextState = nextState.substr(1); // peel off starting '@'
                    }
                    if (!findRules(this._lexer, nextState)) {
                        throw createError(this._lexer, 'trying to switch to a state \'' + nextState + '\' that is undefined in rule: ' + rule.name);
                    }
                    else {
                        stack = stack.switchTo(nextState);
                    }
                }
                else if (action.transform && typeof action.transform === 'function') {
                    throw createError(this._lexer, 'action.transform not supported');
                }
                else if (action.next) {
                    if (action.next === '@push') {
                        if (stack.depth >= this._lexer.maxStack) {
                            throw createError(this._lexer, 'maximum tokenizer stack size reached: [' +
                                stack.state + ',' + stack.parent.state + ',...]');
                        }
                        else {
                            stack = stack.push(state);
                        }
                    }
                    else if (action.next === '@pop') {
                        if (stack.depth <= 1) {
                            throw createError(this._lexer, 'trying to pop an empty stack in rule: ' + rule.name);
                        }
                        else {
                            stack = stack.pop();
                        }
                    }
                    else if (action.next === '@popall') {
                        stack = stack.popall();
                    }
                    else {
                        var nextState = substituteMatches(this._lexer, action.next, matched, matches, state);
                        if (nextState[0] === '@') {
                            nextState = nextState.substr(1); // peel off starting '@'
                        }
                        if (!findRules(this._lexer, nextState)) {
                            throw createError(this._lexer, 'trying to set a next state \'' + nextState + '\' that is undefined in rule: ' + rule.name);
                        }
                        else {
                            stack = stack.push(nextState);
                        }
                    }
                }
                if (action.log && typeof (action.log) === 'string') {
                    log(this._lexer, this._lexer.languageId + ': ' + substituteMatches(this._lexer, action.log, matched, matches, state));
                }
            }
            // check result
            if (result === null) {
                throw createError(this._lexer, 'lexer rule has no well-defined action in rule: ' + rule.name);
            }
            // is the result a group match?
            if (Array.isArray(result)) {
                if (groupMatching && groupMatching.groups.length > 0) {
                    throw createError(this._lexer, 'groups cannot be nested: ' + rule.name);
                }
                if (matches.length !== result.length + 1) {
                    throw createError(this._lexer, 'matched number of groups does not match the number of actions in rule: ' + rule.name);
                }
                var totalLen = 0;
                for (var i = 1; i < matches.length; i++) {
                    totalLen += matches[i].length;
                }
                if (totalLen !== matched.length) {
                    throw createError(this._lexer, 'with groups, all characters should be matched in consecutive groups in rule: ' + rule.name);
                }
                groupMatching = {
                    rule: rule,
                    matches: matches,
                    groups: []
                };
                for (var i = 0; i < result.length; i++) {
                    groupMatching.groups[i] = {
                        action: result[i],
                        matched: matches[i + 1]
                    };
                }
                pos -= matched.length;
                // call recursively to initiate first result match
                continue;
            }
            else {
                // regular result
                // check for '@rematch'
                if (result === '@rematch') {
                    pos -= matched.length;
                    matched = ''; // better set the next state too..
                    matches = null;
                    result = '';
                }
                // check progress
                if (matched.length === 0) {
                    if (stackLen0 !== stack.depth || state !== stack.state || (!groupMatching ? 0 : groupMatching.groups.length) !== groupLen0) {
                        continue;
                    }
                    else {
                        throw createError(this._lexer, 'no progress in tokenizer in rule: ' + rule.name);
                        pos = lineLength; // must make progress or editor loops
                    }
                }
                // return the result (and check for brace matching)
                // todo: for efficiency we could pre-sanitize tokenPostfix and substitutions
                var tokenType = null;
                if (isString$1(result) && result.indexOf('@brackets') === 0) {
                    var rest = result.substr('@brackets'.length);
                    var bracket = findBracket(this._lexer, matched);
                    if (!bracket) {
                        throw createError(this._lexer, '@brackets token returned but no bracket defined as: ' + matched);
                        bracket = { token: '', bracketType: 0 /* None */ };
                    }
                    tokenType = sanitize(bracket.token + rest);
                }
                else {
                    var token = (result === '' ? '' : result + this._lexer.tokenPostfix);
                    tokenType = sanitize(token);
                }
                tokensCollector.emit(pos0 + offsetDelta, tokenType);
            }
            if (enteringEmbeddedMode !== null) {
                // substitute language alias to known modes to support syntax highlighting
                var enteringEmbeddedModeId = this._modeService.getModeIdForLanguageName(enteringEmbeddedMode);
                if (enteringEmbeddedModeId) {
                    enteringEmbeddedMode = enteringEmbeddedModeId;
                }
                var embeddedModeData_1 = this._getNestedEmbeddedModeData(enteringEmbeddedMode);
                if (pos < lineLength) {
                    // there is content from the embedded mode on this line
                    var restOfLine = line.substr(pos);
                    return this._nestedTokenize(restOfLine, MonarchLineStateFactory.create(stack, embeddedModeData_1), offsetDelta + pos, tokensCollector);
                }
                else {
                    return MonarchLineStateFactory.create(stack, embeddedModeData_1);
                }
            }
        }
        return MonarchLineStateFactory.create(stack, embeddedModeData);
    };
    MonarchTokenizer.prototype._getNestedEmbeddedModeData = function (mimetypeOrModeId) {
        var nestedModeId = this._locateMode(mimetypeOrModeId);
        if (nestedModeId) {
            var tokenizationSupport = TokenizationRegistry.get(nestedModeId);
            if (tokenizationSupport) {
                return new EmbeddedModeData(nestedModeId, tokenizationSupport.getInitialState());
            }
        }
        return new EmbeddedModeData(nestedModeId || NULL_MODE_ID, NULL_STATE);
    };
    MonarchTokenizer.prototype._locateMode = function (mimetypeOrModeId) {
        if (!mimetypeOrModeId || !this._modeService.isRegisteredMode(mimetypeOrModeId)) {
            return null;
        }
        var modeId = this._modeService.getModeId(mimetypeOrModeId);
        if (modeId) {
            // Fire mode loading event
            this._modeService.triggerMode(modeId);
            this._embeddedModes[modeId] = true;
        }
        return modeId;
    };
    return MonarchTokenizer;
}());
/**
 * Searches for a bracket in the 'brackets' attribute that matches the input.
 */
function findBracket(lexer, matched) {
    if (!matched) {
        return null;
    }
    matched = fixCase(lexer, matched);
    var brackets = lexer.brackets;
    for (var i = 0; i < brackets.length; i++) {
        var bracket = brackets[i];
        if (bracket.open === matched) {
            return { token: bracket.token, bracketType: 1 /* Open */ };
        }
        else if (bracket.close === matched) {
            return { token: bracket.token, bracketType: -1 /* Close */ };
        }
    }
    return null;
}
function createTokenizationSupport(modeService, standaloneThemeService, modeId, lexer) {
    return new MonarchTokenizer(modeService, standaloneThemeService, modeId, lexer);
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Register information about a new language.
 */
function register(language) {
    ModesRegistry.registerLanguage(language);
}
/**
 * Get the information of all the registered languages.
 */
function getLanguages() {
    var result = [];
    result = result.concat(ModesRegistry.getLanguages());
    return result;
}
function getEncodedLanguageId(languageId) {
    var lid = StaticServices.modeService.get().getLanguageIdentifier(languageId);
    return lid ? lid.id : 0;
}
/**
 * An event emitted when a language is first time needed (e.g. a model has it set).
 * @event
 */
function onLanguage(languageId, callback) {
    var disposable = StaticServices.modeService.get().onDidCreateMode(function (mode) {
        if (mode.getId() === languageId) {
            // stop listening
            disposable.dispose();
            // invoke actual listener
            callback();
        }
    });
    return disposable;
}
/**
 * Set the editing configuration for a language.
 */
function setLanguageConfiguration(languageId, configuration) {
    var languageIdentifier = StaticServices.modeService.get().getLanguageIdentifier(languageId);
    if (!languageIdentifier) {
        throw new Error("Cannot set configuration for unknown language " + languageId);
    }
    return LanguageConfigurationRegistry.register(languageIdentifier, configuration);
}
/**
 * @internal
 */
var EncodedTokenizationSupport2Adapter = /** @class */ (function () {
    function EncodedTokenizationSupport2Adapter(actual) {
        this._actual = actual;
    }
    EncodedTokenizationSupport2Adapter.prototype.getInitialState = function () {
        return this._actual.getInitialState();
    };
    EncodedTokenizationSupport2Adapter.prototype.tokenize = function (line, state, offsetDelta) {
        throw new Error('Not supported!');
    };
    EncodedTokenizationSupport2Adapter.prototype.tokenize2 = function (line, state) {
        var result = this._actual.tokenizeEncoded(line, state);
        return new TokenizationResult2(result.tokens, result.endState);
    };
    return EncodedTokenizationSupport2Adapter;
}());
/**
 * @internal
 */
var TokenizationSupport2Adapter = /** @class */ (function () {
    function TokenizationSupport2Adapter(standaloneThemeService, languageIdentifier, actual) {
        this._standaloneThemeService = standaloneThemeService;
        this._languageIdentifier = languageIdentifier;
        this._actual = actual;
    }
    TokenizationSupport2Adapter.prototype.getInitialState = function () {
        return this._actual.getInitialState();
    };
    TokenizationSupport2Adapter.prototype._toClassicTokens = function (tokens, language, offsetDelta) {
        var result = [];
        var previousStartIndex = 0;
        for (var i = 0, len = tokens.length; i < len; i++) {
            var t = tokens[i];
            var startIndex = t.startIndex;
            // Prevent issues stemming from a buggy external tokenizer.
            if (i === 0) {
                // Force first token to start at first index!
                startIndex = 0;
            }
            else if (startIndex < previousStartIndex) {
                // Force tokens to be after one another!
                startIndex = previousStartIndex;
            }
            result[i] = new Token(startIndex + offsetDelta, t.scopes, language);
            previousStartIndex = startIndex;
        }
        return result;
    };
    TokenizationSupport2Adapter.prototype.tokenize = function (line, state, offsetDelta) {
        var actualResult = this._actual.tokenize(line, state);
        var tokens = this._toClassicTokens(actualResult.tokens, this._languageIdentifier.language, offsetDelta);
        var endState;
        // try to save an object if possible
        if (actualResult.endState.equals(state)) {
            endState = state;
        }
        else {
            endState = actualResult.endState;
        }
        return new TokenizationResult(tokens, endState);
    };
    TokenizationSupport2Adapter.prototype._toBinaryTokens = function (tokens, offsetDelta) {
        var languageId = this._languageIdentifier.id;
        var tokenTheme = this._standaloneThemeService.getTheme().tokenTheme;
        var result = [], resultLen = 0;
        var previousStartIndex = 0;
        for (var i = 0, len = tokens.length; i < len; i++) {
            var t = tokens[i];
            var metadata = tokenTheme.match(languageId, t.scopes);
            if (resultLen > 0 && result[resultLen - 1] === metadata) {
                // same metadata
                continue;
            }
            var startIndex = t.startIndex;
            // Prevent issues stemming from a buggy external tokenizer.
            if (i === 0) {
                // Force first token to start at first index!
                startIndex = 0;
            }
            else if (startIndex < previousStartIndex) {
                // Force tokens to be after one another!
                startIndex = previousStartIndex;
            }
            result[resultLen++] = startIndex + offsetDelta;
            result[resultLen++] = metadata;
            previousStartIndex = startIndex;
        }
        var actualResult = new Uint32Array(resultLen);
        for (var i = 0; i < resultLen; i++) {
            actualResult[i] = result[i];
        }
        return actualResult;
    };
    TokenizationSupport2Adapter.prototype.tokenize2 = function (line, state, offsetDelta) {
        var actualResult = this._actual.tokenize(line, state);
        var tokens = this._toBinaryTokens(actualResult.tokens, offsetDelta);
        var endState;
        // try to save an object if possible
        if (actualResult.endState.equals(state)) {
            endState = state;
        }
        else {
            endState = actualResult.endState;
        }
        return new TokenizationResult2(tokens, endState);
    };
    return TokenizationSupport2Adapter;
}());
function isEncodedTokensProvider(provider) {
    return provider['tokenizeEncoded'];
}
/**
 * Set the tokens provider for a language (manual implementation).
 */
function setTokensProvider(languageId, provider) {
    var languageIdentifier = StaticServices.modeService.get().getLanguageIdentifier(languageId);
    if (!languageIdentifier) {
        throw new Error("Cannot set tokens provider for unknown language " + languageId);
    }
    var adapter;
    if (isEncodedTokensProvider(provider)) {
        adapter = new EncodedTokenizationSupport2Adapter(provider);
    }
    else {
        adapter = new TokenizationSupport2Adapter(StaticServices.standaloneThemeService.get(), languageIdentifier, provider);
    }
    return TokenizationRegistry.register(languageId, adapter);
}
/**
 * Set the tokens provider for a language (monarch implementation).
 */
function setMonarchTokensProvider(languageId, languageDef) {
    var lexer = compile(languageId, languageDef);
    var adapter = createTokenizationSupport(StaticServices.modeService.get(), StaticServices.standaloneThemeService.get(), languageId, lexer);
    return TokenizationRegistry.register(languageId, adapter);
}
/**
 * Register a reference provider (used by e.g. reference search).
 */
function registerReferenceProvider(languageId, provider) {
    return ReferenceProviderRegistry.register(languageId, provider);
}
/**
 * Register a rename provider (used by e.g. rename symbol).
 */
function registerRenameProvider(languageId, provider) {
    return RenameProviderRegistry.register(languageId, provider);
}
/**
 * Register a signature help provider (used by e.g. parameter hints).
 */
function registerSignatureHelpProvider(languageId, provider) {
    return SignatureHelpProviderRegistry.register(languageId, provider);
}
/**
 * Register a hover provider (used by e.g. editor hover).
 */
function registerHoverProvider(languageId, provider) {
    return HoverProviderRegistry.register(languageId, {
        provideHover: function (model, position, token) {
            var word = model.getWordAtPosition(position);
            return Promise.resolve(provider.provideHover(model, position, token)).then(function (value) {
                if (!value) {
                    return undefined;
                }
                if (!value.range && word) {
                    value.range = new Range(position.lineNumber, word.startColumn, position.lineNumber, word.endColumn);
                }
                if (!value.range) {
                    value.range = new Range(position.lineNumber, position.column, position.lineNumber, position.column);
                }
                return value;
            });
        }
    });
}
/**
 * Register a document symbol provider (used by e.g. outline).
 */
function registerDocumentSymbolProvider(languageId, provider) {
    return DocumentSymbolProviderRegistry.register(languageId, provider);
}
/**
 * Register a document highlight provider (used by e.g. highlight occurrences).
 */
function registerDocumentHighlightProvider(languageId, provider) {
    return DocumentHighlightProviderRegistry.register(languageId, provider);
}
/**
 * Register a definition provider (used by e.g. go to definition).
 */
function registerDefinitionProvider(languageId, provider) {
    return DefinitionProviderRegistry.register(languageId, provider);
}
/**
 * Register a implementation provider (used by e.g. go to implementation).
 */
function registerImplementationProvider(languageId, provider) {
    return ImplementationProviderRegistry.register(languageId, provider);
}
/**
 * Register a type definition provider (used by e.g. go to type definition).
 */
function registerTypeDefinitionProvider(languageId, provider) {
    return TypeDefinitionProviderRegistry.register(languageId, provider);
}
/**
 * Register a code lens provider (used by e.g. inline code lenses).
 */
function registerCodeLensProvider(languageId, provider) {
    return CodeLensProviderRegistry.register(languageId, provider);
}
/**
 * Register a code action provider (used by e.g. quick fix).
 */
function registerCodeActionProvider(languageId, provider) {
    return CodeActionProviderRegistry.register(languageId, {
        provideCodeActions: function (model, range, context, token) {
            var markers = StaticServices.markerService.get().read({ resource: model.uri }).filter(function (m) {
                return Range.areIntersectingOrTouching(m, range);
            });
            return provider.provideCodeActions(model, range, { markers: markers, only: context.only }, token);
        }
    });
}
/**
 * Register a formatter that can handle only entire models.
 */
function registerDocumentFormattingEditProvider(languageId, provider) {
    return DocumentFormattingEditProviderRegistry.register(languageId, provider);
}
/**
 * Register a formatter that can handle a range inside a model.
 */
function registerDocumentRangeFormattingEditProvider(languageId, provider) {
    return DocumentRangeFormattingEditProviderRegistry.register(languageId, provider);
}
/**
 * Register a formatter than can do formatting as the user types.
 */
function registerOnTypeFormattingEditProvider(languageId, provider) {
    return OnTypeFormattingEditProviderRegistry.register(languageId, provider);
}
/**
 * Register a link provider that can find links in text.
 */
function registerLinkProvider(languageId, provider) {
    return LinkProviderRegistry.register(languageId, provider);
}
/**
 * Register a completion item provider (use by e.g. suggestions).
 */
function registerCompletionItemProvider(languageId, provider) {
    return CompletionProviderRegistry.register(languageId, provider);
}
/**
 * Register a document color provider (used by Color Picker, Color Decorator).
 */
function registerColorProvider(languageId, provider) {
    return ColorProviderRegistry.register(languageId, provider);
}
/**
 * Register a folding range provider
 */
function registerFoldingRangeProvider(languageId, provider) {
    return FoldingRangeProviderRegistry.register(languageId, provider);
}
/**
 * @internal
 */
function createMonacoLanguagesAPI() {
    return {
        register: register,
        getLanguages: getLanguages,
        onLanguage: onLanguage,
        getEncodedLanguageId: getEncodedLanguageId,
        // provider methods
        setLanguageConfiguration: setLanguageConfiguration,
        setTokensProvider: setTokensProvider,
        setMonarchTokensProvider: setMonarchTokensProvider,
        registerReferenceProvider: registerReferenceProvider,
        registerRenameProvider: registerRenameProvider,
        registerCompletionItemProvider: registerCompletionItemProvider,
        registerSignatureHelpProvider: registerSignatureHelpProvider,
        registerHoverProvider: registerHoverProvider,
        registerDocumentSymbolProvider: registerDocumentSymbolProvider,
        registerDocumentHighlightProvider: registerDocumentHighlightProvider,
        registerDefinitionProvider: registerDefinitionProvider,
        registerImplementationProvider: registerImplementationProvider,
        registerTypeDefinitionProvider: registerTypeDefinitionProvider,
        registerCodeLensProvider: registerCodeLensProvider,
        registerCodeActionProvider: registerCodeActionProvider,
        registerDocumentFormattingEditProvider: registerDocumentFormattingEditProvider,
        registerDocumentRangeFormattingEditProvider: registerDocumentRangeFormattingEditProvider,
        registerOnTypeFormattingEditProvider: registerOnTypeFormattingEditProvider,
        registerLinkProvider: registerLinkProvider,
        registerColorProvider: registerColorProvider,
        registerFoldingRangeProvider: registerFoldingRangeProvider,
        // enums
        DocumentHighlightKind: DocumentHighlightKind$1,
        CompletionItemKind: CompletionItemKind,
        CompletionItemInsertTextRule: CompletionItemInsertTextRule,
        SymbolKind: SymbolKind,
        IndentAction: IndentAction$1,
        CompletionTriggerKind: CompletionTriggerKind,
        SignatureHelpTriggerReason: SignatureHelpTriggerReason$1,
        // classes
        FoldingRangeKind: FoldingRangeKind,
    };
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var global$2 = self;
// When missing, polyfill the native promise
// with our winjs-based polyfill
if (typeof global$2.Promise === 'undefined') {
    global$2.Promise = PolyfillPromise;
}
// Set defaults for standalone editor
EDITOR_DEFAULTS.wrappingIndent = 0 /* None */;
EDITOR_DEFAULTS.viewInfo.glyphMargin = false;
EDITOR_DEFAULTS.autoIndent = false;
var api = createMonacoBaseAPI();
api.editor = createMonacoEditorAPI();
api.languages = createMonacoLanguagesAPI();
var CancellationTokenSource$1 = api.CancellationTokenSource;
var Emitter$1 = api.Emitter;
var KeyCode$1 = api.KeyCode;
var KeyMod$1 = api.KeyMod;
var Position$1 = api.Position;
var Range$2 = api.Range;
var Selection$1 = api.Selection;
var SelectionDirection$1 = api.SelectionDirection;
var MarkerSeverity$2 = api.MarkerSeverity;
var MarkerTag$1 = api.MarkerTag;
var Promise$2 = api.Promise;
var Uri = api.Uri;
var Token$2 = api.Token;
var editor = api.editor;
var languages = api.languages;
global$2.monaco = api;
if (typeof global$2.require !== 'undefined' && typeof global$2.require.config === 'function') {
    global$2.require.config({
        ignoreDuplicateModules: [
            'vscode-languageserver-types',
            'vscode-languageserver-types/main',
            'vscode-nls',
            'vscode-nls/vscode-nls',
            'jsonc-parser',
            'jsonc-parser/main',
            'vscode-uri',
            'vscode-uri/index',
            'vs/basic-languages/typescript/typescript'
        ]
    });
}

var monaco$1 = /*#__PURE__*/Object.freeze({
    CancellationTokenSource: CancellationTokenSource$1,
    Emitter: Emitter$1,
    KeyCode: KeyCode$1,
    KeyMod: KeyMod$1,
    Position: Position$1,
    Range: Range$2,
    Selection: Selection$1,
    SelectionDirection: SelectionDirection$1,
    MarkerSeverity: MarkerSeverity$2,
    MarkerTag: MarkerTag$1,
    Promise: Promise$2,
    Uri: Uri,
    Token: Token$2,
    editor: editor,
    languages: languages
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco = (typeof monaco === 'undefined' ? self.monaco : monaco);
var languageDefinitions = {};
function _loadLanguage(languageId) {
    var loader = languageDefinitions[languageId].loader;
    return loader().then(function (mod) {
        _monaco.languages.setMonarchTokensProvider(languageId, mod.language);
        _monaco.languages.setLanguageConfiguration(languageId, mod.conf);
    });
}
var languagePromises = {};
function loadLanguage(languageId) {
    if (!languagePromises[languageId]) {
        languagePromises[languageId] = _loadLanguage(languageId);
    }
    return languagePromises[languageId];
}
function registerLanguage(def) {
    var languageId = def.id;
    languageDefinitions[languageId] = def;
    _monaco.languages.register(def);
    _monaco.languages.onLanguage(languageId, function () {
        loadLanguage(languageId);
    });
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco$1 = (typeof monaco === 'undefined' ? self.monaco : monaco);
registerLanguage({
    id: 'javascript',
    extensions: ['.js', '.es6', '.jsx'],
    firstLine: '^#!.*\\bnode',
    filenames: ['jakefile'],
    aliases: ['JavaScript', 'javascript', 'js'],
    mimetypes: ['text/javascript'],
    loader: function () { return _monaco$1.Promise.wrap(Promise.resolve().then(function () { return javascript; })); }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco$2 = (typeof monaco === 'undefined' ? self.monaco : monaco);
registerLanguage({
    id: 'typescript',
    extensions: ['.ts', '.tsx'],
    aliases: ['TypeScript', 'ts', 'typescript'],
    mimetypes: ['text/typescript'],
    loader: function () { return _monaco$2.Promise.wrap(Promise.resolve().then(function () { return typescript; })); }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco$3 = (typeof monaco === 'undefined' ? self.monaco : monaco);
registerLanguage({
    id: 'css',
    extensions: ['.css'],
    aliases: ['CSS', 'css'],
    mimetypes: ['text/css'],
    loader: function () { return _monaco$3.Promise.wrap(Promise.resolve().then(function () { return css; })); }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco$4 = (typeof monaco === 'undefined' ? self.monaco : monaco);
registerLanguage({
    id: 'html',
    extensions: ['.html', '.htm', '.shtml', '.xhtml', '.mdoc', '.jsp', '.asp', '.aspx', '.jshtm'],
    aliases: ['HTML', 'htm', 'html', 'xhtml'],
    mimetypes: ['text/html', 'text/x-jshtm', 'text/template', 'text/ng-template'],
    loader: function () { return _monaco$4.Promise.wrap(Promise.resolve().then(function () { return html; })); }
});

var Emitter$2 = monaco.Emitter;
// --- TypeScript configuration and defaults ---------
var LanguageServiceDefaultsImpl = /** @class */ (function () {
    function LanguageServiceDefaultsImpl(compilerOptions, diagnosticsOptions) {
        this._onDidChange = new Emitter$2();
        this._extraLibs = Object.create(null);
        this._workerMaxIdleTime = 2 * 60 * 1000;
        this.setCompilerOptions(compilerOptions);
        this.setDiagnosticsOptions(diagnosticsOptions);
    }
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "onDidChange", {
        get: function () {
            return this._onDidChange.event;
        },
        enumerable: true,
        configurable: true
    });
    LanguageServiceDefaultsImpl.prototype.getExtraLibs = function () {
        var result = Object.create(null);
        for (var key in this._extraLibs) {
            result[key] = this._extraLibs[key];
        }
        return Object.freeze(result);
    };
    LanguageServiceDefaultsImpl.prototype.addExtraLib = function (content, filePath) {
        var _this = this;
        if (typeof filePath === 'undefined') {
            filePath = "ts:extralib-" + Date.now();
        }
        if (this._extraLibs[filePath]) {
            throw new Error(filePath + " already a extra lib");
        }
        this._extraLibs[filePath] = content;
        this._onDidChange.fire(this);
        return {
            dispose: function () {
                if (delete _this._extraLibs[filePath]) {
                    _this._onDidChange.fire(_this);
                }
            }
        };
    };
    LanguageServiceDefaultsImpl.prototype.getCompilerOptions = function () {
        return this._compilerOptions;
    };
    LanguageServiceDefaultsImpl.prototype.setCompilerOptions = function (options) {
        this._compilerOptions = options || Object.create(null);
        this._onDidChange.fire(this);
    };
    LanguageServiceDefaultsImpl.prototype.getDiagnosticsOptions = function () {
        return this._diagnosticsOptions;
    };
    LanguageServiceDefaultsImpl.prototype.setDiagnosticsOptions = function (options) {
        this._diagnosticsOptions = options || Object.create(null);
        this._onDidChange.fire(this);
    };
    LanguageServiceDefaultsImpl.prototype.setMaximumWorkerIdleTime = function (value) {
        // doesn't fire an event since no
        // worker restart is required here
        this._workerMaxIdleTime = value;
    };
    LanguageServiceDefaultsImpl.prototype.getWorkerMaxIdleTime = function () {
        return this._workerMaxIdleTime;
    };
    LanguageServiceDefaultsImpl.prototype.setEagerModelSync = function (value) {
        // doesn't fire an event since no
        // worker restart is required here
        this._eagerModelSync = value;
    };
    LanguageServiceDefaultsImpl.prototype.getEagerModelSync = function () {
        return this._eagerModelSync;
    };
    return LanguageServiceDefaultsImpl;
}());
//#region enums copied from typescript to prevent loading the entire typescriptServices ---
var ModuleKind;
(function (ModuleKind) {
    ModuleKind[ModuleKind["None"] = 0] = "None";
    ModuleKind[ModuleKind["CommonJS"] = 1] = "CommonJS";
    ModuleKind[ModuleKind["AMD"] = 2] = "AMD";
    ModuleKind[ModuleKind["UMD"] = 3] = "UMD";
    ModuleKind[ModuleKind["System"] = 4] = "System";
    ModuleKind[ModuleKind["ES2015"] = 5] = "ES2015";
    ModuleKind[ModuleKind["ESNext"] = 6] = "ESNext";
})(ModuleKind || (ModuleKind = {}));
var JsxEmit;
(function (JsxEmit) {
    JsxEmit[JsxEmit["None"] = 0] = "None";
    JsxEmit[JsxEmit["Preserve"] = 1] = "Preserve";
    JsxEmit[JsxEmit["React"] = 2] = "React";
    JsxEmit[JsxEmit["ReactNative"] = 3] = "ReactNative";
})(JsxEmit || (JsxEmit = {}));
var NewLineKind;
(function (NewLineKind) {
    NewLineKind[NewLineKind["CarriageReturnLineFeed"] = 0] = "CarriageReturnLineFeed";
    NewLineKind[NewLineKind["LineFeed"] = 1] = "LineFeed";
})(NewLineKind || (NewLineKind = {}));
var ScriptTarget;
(function (ScriptTarget) {
    ScriptTarget[ScriptTarget["ES3"] = 0] = "ES3";
    ScriptTarget[ScriptTarget["ES5"] = 1] = "ES5";
    ScriptTarget[ScriptTarget["ES2015"] = 2] = "ES2015";
    ScriptTarget[ScriptTarget["ES2016"] = 3] = "ES2016";
    ScriptTarget[ScriptTarget["ES2017"] = 4] = "ES2017";
    ScriptTarget[ScriptTarget["ES2018"] = 5] = "ES2018";
    ScriptTarget[ScriptTarget["ESNext"] = 6] = "ESNext";
    ScriptTarget[ScriptTarget["JSON"] = 100] = "JSON";
    ScriptTarget[ScriptTarget["Latest"] = 6] = "Latest";
})(ScriptTarget || (ScriptTarget = {}));
var ModuleResolutionKind;
(function (ModuleResolutionKind) {
    ModuleResolutionKind[ModuleResolutionKind["Classic"] = 1] = "Classic";
    ModuleResolutionKind[ModuleResolutionKind["NodeJs"] = 2] = "NodeJs";
})(ModuleResolutionKind || (ModuleResolutionKind = {}));
//#endregion
var typescriptDefaults = new LanguageServiceDefaultsImpl({ allowNonTsExtensions: true, target: ScriptTarget.Latest }, { noSemanticValidation: false, noSyntaxValidation: false });
var javascriptDefaults = new LanguageServiceDefaultsImpl({ allowNonTsExtensions: true, allowJs: true, target: ScriptTarget.Latest }, { noSemanticValidation: true, noSyntaxValidation: false });
function getTypeScriptWorker() {
    return getMode().then(function (mode) { return mode.getTypeScriptWorker(); });
}
function getJavaScriptWorker() {
    return getMode().then(function (mode) { return mode.getJavaScriptWorker(); });
}
// Export API
function createAPI() {
    return {
        ModuleKind: ModuleKind,
        JsxEmit: JsxEmit,
        NewLineKind: NewLineKind,
        ScriptTarget: ScriptTarget,
        ModuleResolutionKind: ModuleResolutionKind,
        typescriptDefaults: typescriptDefaults,
        javascriptDefaults: javascriptDefaults,
        getTypeScriptWorker: getTypeScriptWorker,
        getJavaScriptWorker: getJavaScriptWorker
    };
}
monaco.languages.typescript = createAPI();
// --- Registration to monaco editor ---
function getMode() {
    return monaco.Promise.wrap(Promise.resolve().then(function () { return tsMode; }));
}
monaco.languages.onLanguage('typescript', function () {
    return getMode().then(function (mode) { return mode.setupTypeScript(typescriptDefaults); });
});
monaco.languages.onLanguage('javascript', function () {
    return getMode().then(function (mode) { return mode.setupJavaScript(javascriptDefaults); });
});

var Emitter$3 = monaco.Emitter;
// --- CSS configuration and defaults ---------
var LanguageServiceDefaultsImpl$1 = /** @class */ (function () {
    function LanguageServiceDefaultsImpl(languageId, diagnosticsOptions) {
        this._onDidChange = new Emitter$3();
        this._languageId = languageId;
        this.setDiagnosticsOptions(diagnosticsOptions);
    }
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "onDidChange", {
        get: function () {
            return this._onDidChange.event;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "languageId", {
        get: function () {
            return this._languageId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "diagnosticsOptions", {
        get: function () {
            return this._diagnosticsOptions;
        },
        enumerable: true,
        configurable: true
    });
    LanguageServiceDefaultsImpl.prototype.setDiagnosticsOptions = function (options) {
        this._diagnosticsOptions = options || Object.create(null);
        this._onDidChange.fire(this);
    };
    return LanguageServiceDefaultsImpl;
}());
var diagnosticDefault = {
    validate: true,
    lint: {
        compatibleVendorPrefixes: 'ignore',
        vendorPrefix: 'warning',
        duplicateProperties: 'warning',
        emptyRules: 'warning',
        importStatement: 'ignore',
        boxModel: 'ignore',
        universalSelector: 'ignore',
        zeroUnits: 'ignore',
        fontFaceProperties: 'warning',
        hexColorLength: 'error',
        argumentsInColorFunction: 'error',
        unknownProperties: 'warning',
        ieHack: 'ignore',
        unknownVendorSpecificProperties: 'ignore',
        propertyIgnoredDueToDisplay: 'warning',
        important: 'ignore',
        float: 'ignore',
        idSelector: 'ignore'
    }
};
var cssDefaults = new LanguageServiceDefaultsImpl$1('css', diagnosticDefault);
var scssDefaults = new LanguageServiceDefaultsImpl$1('scss', diagnosticDefault);
var lessDefaults = new LanguageServiceDefaultsImpl$1('less', diagnosticDefault);
// Export API
function createAPI$1() {
    return {
        cssDefaults: cssDefaults,
        lessDefaults: lessDefaults,
        scssDefaults: scssDefaults
    };
}
monaco.languages.css = createAPI$1();
// --- Registration to monaco editor ---
function getMode$1() {
    return monaco.Promise.wrap(Promise.resolve().then(function () { return cssMode; }));
}
monaco.languages.onLanguage('less', function () {
    getMode$1().then(function (mode) { return mode.setupMode(lessDefaults); });
});
monaco.languages.onLanguage('scss', function () {
    getMode$1().then(function (mode) { return mode.setupMode(scssDefaults); });
});
monaco.languages.onLanguage('css', function () {
    getMode$1().then(function (mode) { return mode.setupMode(cssDefaults); });
});

var Emitter$4 = monaco.Emitter;
// --- JSON configuration and defaults ---------
var LanguageServiceDefaultsImpl$2 = /** @class */ (function () {
    function LanguageServiceDefaultsImpl(languageId, diagnosticsOptions) {
        this._onDidChange = new Emitter$4();
        this._languageId = languageId;
        this.setDiagnosticsOptions(diagnosticsOptions);
    }
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "onDidChange", {
        get: function () {
            return this._onDidChange.event;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "languageId", {
        get: function () {
            return this._languageId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "diagnosticsOptions", {
        get: function () {
            return this._diagnosticsOptions;
        },
        enumerable: true,
        configurable: true
    });
    LanguageServiceDefaultsImpl.prototype.setDiagnosticsOptions = function (options) {
        this._diagnosticsOptions = options || Object.create(null);
        this._onDidChange.fire(this);
    };
    return LanguageServiceDefaultsImpl;
}());
var diagnosticDefault$1 = {
    validate: true,
    allowComments: true,
    schemas: [],
    enableSchemaRequest: false
};
var jsonDefaults = new LanguageServiceDefaultsImpl$2('json', diagnosticDefault$1);
// Export API
function createAPI$2() {
    return {
        jsonDefaults: jsonDefaults,
    };
}
monaco.languages.json = createAPI$2();
// --- Registration to monaco editor ---
function getMode$2() {
    return monaco.Promise.wrap(Promise.resolve().then(function () { return jsonMode; }));
}
monaco.languages.register({
    id: 'json',
    extensions: ['.json', '.bowerrc', '.jshintrc', '.jscsrc', '.eslintrc', '.babelrc'],
    aliases: ['JSON', 'json'],
    mimetypes: ['application/json'],
});
monaco.languages.onLanguage('json', function () {
    getMode$2().then(function (mode) { return mode.setupMode(jsonDefaults); });
});

var Emitter$5 = monaco.Emitter;
// --- HTML configuration and defaults ---------
var LanguageServiceDefaultsImpl$3 = /** @class */ (function () {
    function LanguageServiceDefaultsImpl(languageId, options) {
        this._onDidChange = new Emitter$5();
        this._languageId = languageId;
        this.setOptions(options);
    }
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "onDidChange", {
        get: function () {
            return this._onDidChange.event;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "languageId", {
        get: function () {
            return this._languageId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LanguageServiceDefaultsImpl.prototype, "options", {
        get: function () {
            return this._options;
        },
        enumerable: true,
        configurable: true
    });
    LanguageServiceDefaultsImpl.prototype.setOptions = function (options) {
        this._options = options || Object.create(null);
        this._onDidChange.fire(this);
    };
    return LanguageServiceDefaultsImpl;
}());
var formatDefaults = {
    tabSize: 4,
    insertSpaces: false,
    wrapLineLength: 120,
    unformatted: 'default": "a, abbr, acronym, b, bdo, big, br, button, cite, code, dfn, em, i, img, input, kbd, label, map, object, q, samp, select, small, span, strong, sub, sup, textarea, tt, var',
    contentUnformatted: 'pre',
    indentInnerHtml: false,
    preserveNewLines: true,
    maxPreserveNewLines: null,
    indentHandlebars: false,
    endWithNewline: false,
    extraLiners: 'head, body, /html',
    wrapAttributes: 'auto'
};
var htmlOptionsDefault = {
    format: formatDefaults,
    suggest: { html5: true, angular1: true, ionic: true }
};
var handlebarOptionsDefault = {
    format: formatDefaults,
    suggest: { html5: true }
};
var razorOptionsDefault = {
    format: formatDefaults,
    suggest: { html5: true, razor: true }
};
var htmlLanguageId = 'html';
var handlebarsLanguageId = 'handlebars';
var razorLanguageId = 'razor';
var htmlDefaults = new LanguageServiceDefaultsImpl$3(htmlLanguageId, htmlOptionsDefault);
var handlebarDefaults = new LanguageServiceDefaultsImpl$3(handlebarsLanguageId, handlebarOptionsDefault);
var razorDefaults = new LanguageServiceDefaultsImpl$3(razorLanguageId, razorOptionsDefault);
// Export API
function createAPI$3() {
    return {
        htmlDefaults: htmlDefaults,
        razorDefaults: razorDefaults,
        handlebarDefaults: handlebarDefaults
    };
}
monaco.languages.html = createAPI$3();
// --- Registration to monaco editor ---
function getMode$3() {
    return monaco.Promise.wrap(Promise.resolve().then(function () { return htmlMode; }));
}
monaco.languages.onLanguage(htmlLanguageId, function () {
    getMode$3().then(function (mode) { return mode.setupMode(htmlDefaults); });
});
monaco.languages.onLanguage(handlebarsLanguageId, function () {
    getMode$3().then(function (mode) { return mode.setupMode(handlebarDefaults); });
});
monaco.languages.onLanguage(razorLanguageId, function () {
    getMode$3().then(function (mode) { return mode.setupMode(razorDefaults); });
});

var editorWorker = URL.createObjectURL(new Blob(["var editorworker=function(e){\"use strict\";var t;!function(){var e=Object.create(null);e[\"WinJS/Core/_WinJS\"]={};var n=function(t,n,r){var i={},o=!1,s=n.map(function(t){return\"exports\"===t?(o=!0,i):e[t]}),u=r.apply({},s);e[t]=o?i:u};n(\"WinJS/Core/_Global\",[],function(){return\"undefined\"!=typeof window?window:\"undefined\"!=typeof self?self:\"undefined\"!=typeof global?global:{}}),n(\"WinJS/Core/_BaseCoreUtils\",[\"WinJS/Core/_Global\"],function(e){var t=!!e.Windows;var n=null;return{hasWinRT:t,markSupportedForProcessing:function(e){return e.supportedForProcessing=!0,e},_setImmediate:function(t){null===n&&(n=e.setImmediate?e.setImmediate.bind(e):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):e.setTimeout.bind(e)),n(t)}}}),n(\"WinJS/Core/_WriteProfilerMark\",[\"WinJS/Core/_Global\"],function(e){return e.msWriteProfilerMark||function(){}}),n(\"WinJS/Core/_Base\",[\"WinJS/Core/_WinJS\",\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_WriteProfilerMark\"],function(e,t,n,r){function i(e,t,n){var r,i,o,s=Object.keys(t),u=Array.isArray(e);for(i=0,o=s.length;i<o;i++){var a=s[i],l=95!==a.charCodeAt(0),c=t[a];!c||\"object\"!=typeof c||void 0===c.value&&\"function\"!=typeof c.get&&\"function\"!=typeof c.set?l?u?e.forEach(function(e){e[a]=c}):e[a]=c:(r=r||{})[a]={value:c,enumerable:l,configurable:!0,writable:!0}:(void 0===c.enumerable&&(c.enumerable=l),n&&c.setName&&\"function\"==typeof c.setName&&c.setName(n+\".\"+a),(r=r||{})[a]=c)}r&&(u?e.forEach(function(e){Object.defineProperties(e,r)}):Object.defineProperties(e,r))}return function(){var n=e;function o(n,r){var i=n||{};if(r){var o=r.split(\".\");i===t&&\"WinJS\"===o[0]&&(i=e,o.splice(0,1));for(var s=0,u=o.length;s<u;s++){var a=o[s];i[a]||Object.defineProperty(i,a,{value:{},writable:!1,enumerable:!0,configurable:!0}),i=i[a]}}return i}function s(e,t,n){var r=o(e,t);return n&&i(r,n,t||\"<ANONYMOUS>\"),r}n.Namespace||(n.Namespace=Object.create(Object.prototype));var u={uninitialized:1,working:2,initialized:3};Object.defineProperties(n.Namespace,{defineWithParent:{value:s,writable:!0,enumerable:!0,configurable:!0},define:{value:function(e,n){return s(t,e,n)},writable:!0,enumerable:!0,configurable:!0},_lazy:{value:function(e){var t,n,i=u.uninitialized;return{setName:function(e){t=e},get:function(){switch(i){case u.initialized:return n;case u.uninitialized:i=u.working;try{r(\"WinJS.Namespace._lazy:\"+t+\",StartTM\"),n=e()}finally{r(\"WinJS.Namespace._lazy:\"+t+\",StopTM\"),i=u.uninitialized}return e=null,i=u.initialized,n;case u.working:throw\"Illegal: reentrancy on initialization\";default:throw\"Illegal\"}},set:function(e){switch(i){case u.working:throw\"Illegal: reentrancy on initialization\";default:i=u.initialized,n=e}},enumerable:!0,configurable:!0}},writable:!0,enumerable:!0,configurable:!0},_moduleDefine:{value:function(e,n,r){var s=[e],u=null;return n&&(u=o(t,n),s.push(u)),i(s,r,n||\"<ANONYMOUS>\"),u},writable:!0,enumerable:!0,configurable:!0}})}(),function(){function t(e,t,r){return e=e||function(){},n.markSupportedForProcessing(e),t&&i(e.prototype,t),r&&i(e,r),e}e.Namespace.define(\"WinJS.Class\",{define:t,derive:function(e,r,o,s){if(e){r=r||function(){};var u=e.prototype;return r.prototype=Object.create(u),n.markSupportedForProcessing(r),Object.defineProperty(r.prototype,\"constructor\",{value:r,writable:!0,configurable:!0,enumerable:!0}),o&&i(r.prototype,o),s&&i(r,s),r}return t(r,o,s)},mix:function(e){var t,n;for(e=e||function(){},t=1,n=arguments.length;t<n;t++)i(e.prototype,arguments[t]);return e}})}(),{Namespace:e.Namespace,Class:e.Class}}),n(\"WinJS/Core/_ErrorFromName\",[\"WinJS/Core/_Base\"],function(e){var t=e.Class.derive(Error,function(e,t){this.name=e,this.message=t||e},{},{supportedForProcessing:!1});return e.Namespace.define(\"WinJS\",{ErrorFromName:t}),t}),n(\"WinJS/Core/_Events\",[\"exports\",\"WinJS/Core/_Base\"],function(e,t){function n(e){var t=\"_on\"+e+\"state\";return{get:function(){var e=this[t];return e&&e.userHandler},set:function(n){var r=this[t];n?(r||(r={wrapper:function(e){return r.userHandler(e)},userHandler:n},Object.defineProperty(this,t,{value:r,enumerable:!1,writable:!0,configurable:!0}),this.addEventListener(e,r.wrapper,!1)),r.userHandler=n):r&&(this.removeEventListener(e,r.wrapper,!1),this[t]=null)},enumerable:!0}}var r=t.Class.define(function(e,t,n){this.detail=t,this.target=n,this.timeStamp=Date.now(),this.type=e},{bubbles:{value:!1,writable:!1},cancelable:{value:!1,writable:!1},currentTarget:{get:function(){return this.target}},defaultPrevented:{get:function(){return this._preventDefaultCalled}},trusted:{value:!1,writable:!1},eventPhase:{value:0,writable:!1},target:null,timeStamp:null,type:null,preventDefault:function(){this._preventDefaultCalled=!0},stopImmediatePropagation:function(){this._stopImmediatePropagationCalled=!0},stopPropagation:function(){}},{supportedForProcessing:!1}),i={_listeners:null,addEventListener:function(e,t,n){n=n||!1,this._listeners=this._listeners||{};for(var r=this._listeners[e]=this._listeners[e]||[],i=0,o=r.length;i<o;i++){var s=r[i];if(s.useCapture===n&&s.listener===t)return}r.push({listener:t,useCapture:n})},dispatchEvent:function(e,t){var n=this._listeners&&this._listeners[e];if(n){for(var i=new r(e,t,this),o=0,s=(n=n.slice(0,n.length)).length;o<s&&!i._stopImmediatePropagationCalled;o++)n[o].listener(i);return i.defaultPrevented||!1}return!1},removeEventListener:function(e,t,n){n=n||!1;var r=this._listeners&&this._listeners[e];if(r)for(var i=0,o=r.length;i<o;i++){var s=r[i];if(s.listener===t&&s.useCapture===n){r.splice(i,1),0===r.length&&delete this._listeners[e];break}}}};t.Namespace._moduleDefine(e,\"WinJS.Utilities\",{_createEventProperty:n,createEventProperties:function(){for(var e={},t=0,r=arguments.length;t<r;t++){var i=arguments[t];e[\"on\"+i]=n(i)}return e},eventMixin:i})}),n(\"WinJS/Core/_Trace\",[\"WinJS/Core/_Global\"],function(e){function t(e){return e}return{_traceAsyncOperationStarting:e.Debug&&e.Debug.msTraceAsyncOperationStarting&&e.Debug.msTraceAsyncOperationStarting.bind(e.Debug)||t,_traceAsyncOperationCompleted:e.Debug&&e.Debug.msTraceAsyncOperationCompleted&&e.Debug.msTraceAsyncOperationCompleted.bind(e.Debug)||t,_traceAsyncCallbackStarting:e.Debug&&e.Debug.msTraceAsyncCallbackStarting&&e.Debug.msTraceAsyncCallbackStarting.bind(e.Debug)||t,_traceAsyncCallbackCompleted:e.Debug&&e.Debug.msTraceAsyncCallbackCompleted&&e.Debug.msTraceAsyncCallbackCompleted.bind(e.Debug)||t}}),n(\"WinJS/Promise/_StateMachine\",[\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_Base\",\"WinJS/Core/_ErrorFromName\",\"WinJS/Core/_Events\",\"WinJS/Core/_Trace\"],function(e,t,n,r,i,o){e.Debug&&(e.Debug.setNonUserCodeExceptions=!0);var s=new(n.Class.mix(n.Class.define(null,{},{supportedForProcessing:!1}),i.eventMixin));s._listeners={};var u=\"error\",a=\"Canceled\",l=!1,c={promise:1,thenPromise:2,errorPromise:4,exceptionPromise:8,completePromise:16};c.all=c.promise|c.thenPromise|c.errorPromise|c.exceptionPromise|c.completePromise;var h,f,d,m,p,_,g,v,y,C,b=1;function S(){}h={name:\"created\",enter:function(e){e._setState(f)},cancel:S,done:S,then:S,_completed:S,_error:S,_notify:S,_progress:S,_setCompleteValue:S,_setErrorValue:S},f={name:\"working\",enter:S,cancel:function(e){e._setState(p)},done:M,then:V,_completed:N,_error:x,_notify:S,_progress:R,_setCompleteValue:W,_setErrorValue:F},d={name:\"waiting\",enter:function(e){var t=e._value;if(t instanceof q&&t._state!==C&&t._state!==v)U(t,{promise:e});else{var n=function(r){t._errorId?e._chainedError(r,t):(k(e,r,A,t,n),e._error(r))};n.handlesOnError=!0,t.then(e._completed.bind(e),n,e._progress.bind(e))}},cancel:function(e){e._setState(m)},done:M,then:V,_completed:N,_error:x,_notify:S,_progress:R,_setCompleteValue:W,_setErrorValue:F},m={name:\"waiting_canceled\",enter:function(e){e._setState(_);var t=e._value;t.cancel&&t.cancel()},cancel:S,done:M,then:V,_completed:N,_error:x,_notify:S,_progress:R,_setCompleteValue:W,_setErrorValue:F},p={name:\"canceled\",enter:function(e){e._setState(_),e._cancelAction()},cancel:S,done:M,then:V,_completed:N,_error:x,_notify:S,_progress:R,_setCompleteValue:W,_setErrorValue:F},_={name:\"canceling\",enter:function(e){var t=new Error(a);t.name=t.message,e._value=t,e._setState(y)},cancel:S,done:S,then:S,_completed:S,_error:S,_notify:S,_progress:S,_setCompleteValue:S,_setErrorValue:S},g={name:\"complete_notify\",enter:function(e){if(e.done=Y.prototype.done,e.then=Y.prototype.then,e._listeners)for(var t,n=[e];n.length;)(t=n.shift())._state._notify(t,n);e._setState(v)},cancel:S,done:null,then:null,_completed:S,_error:S,_notify:T,_progress:S,_setCompleteValue:S,_setErrorValue:S},v={name:\"success\",enter:function(e){e.done=Y.prototype.done,e.then=Y.prototype.then,e._cleanupAction()},cancel:S,done:null,then:null,_completed:S,_error:S,_notify:T,_progress:S,_setCompleteValue:S,_setErrorValue:S},y={name:\"error_notify\",enter:function(e){if(e.done=B.prototype.done,e.then=B.prototype.then,e._listeners)for(var t,n=[e];n.length;)(t=n.shift())._state._notify(t,n);e._setState(C)},cancel:S,done:null,then:null,_completed:S,_error:S,_notify:I,_progress:S,_setCompleteValue:S,_setErrorValue:S},C={name:\"error\",enter:function(e){e.done=B.prototype.done,e.then=B.prototype.then,e._cleanupAction()},cancel:S,done:null,then:null,_completed:S,_error:S,_notify:I,_progress:S,_setCompleteValue:S,_setErrorValue:S};var E=n.Class.define(null,{_listeners:null,_nextState:null,_state:null,_value:null,cancel:function(){this._state.cancel(this),this._run()},done:function(e,t,n){this._state.done(this,e,t,n)},then:function e(t,n,r){if(this.then===e)return this._state.then(this,t,n,r);this.then(t,n,r)},_chainedError:function(e,t){var n=this._state._error(this,e,w,t);return this._run(),n},_completed:function(e){var t=this._state._completed(this,e);return this._run(),t},_error:function(e){var t=this._state._error(this,e,P);return this._run(),t},_progress:function(e){this._state._progress(this,e)},_setState:function(e){this._nextState=e},_setCompleteValue:function(e){this._state._setCompleteValue(this,e),this._run()},_setChainedErrorValue:function(e,t){var n=this._state._setErrorValue(this,e,w,t);return this._run(),n},_setExceptionValue:function(e){var t=this._state._setErrorValue(this,e,O);return this._run(),t},_run:function(){for(;this._nextState;)this._state=this._nextState,this._nextState=null,this._state.enter(this)}},{supportedForProcessing:!1});function N(e,t){var n;n=t&&\"object\"==typeof t&&\"function\"==typeof t.then?d:g,e._value=t,e._setState(n)}function L(e,t,n,r,i,o){return{exception:e,error:t,promise:n,handler:o,id:r,parent:i}}function A(e,t,n,r){var i=n._isException;return L(i?t:null,i?null:t,e,n._errorId,n,r)}function w(e,t,n){var r=n._isException,i=n._errorId;return D(e,i,r),L(r?t:null,r?null:t,e,i,n)}function P(e,t){var n=++b;return D(e,n),L(null,t,e,n)}function O(e,t){var n=++b;return D(e,n,!0),L(t,null,e,n)}function M(e,t,n,r){U(e,{c:t,e:n,p:r,asyncOpID:o._traceAsyncOperationStarting(\"WinJS.Promise.done\")})}function x(e,t,n,r){e._value=t,k(e,t,n,r),e._setState(y)}function T(t,n){var r,i,s=t._value,u=t._listeners;if(u)for(t._listeners=null,r=0,i=Array.isArray(u)?u.length:1;r<i;r++){var a=1===i?u:u[r],l=a.c,c=a.promise;if(o._traceAsyncOperationCompleted(a.asyncOpID,e.Debug&&e.Debug.MS_ASYNC_OP_STATUS_SUCCESS),c){o._traceAsyncCallbackStarting(a.asyncOpID);try{c._setCompleteValue(l?l(s):s)}catch(e){c._setExceptionValue(e)}finally{o._traceAsyncCallbackCompleted()}c._state!==d&&c._listeners&&n.push(c)}else Y.prototype.done.call(t,l)}}function I(t,n){var r,i,s=t._value,u=t._listeners;if(u)for(t._listeners=null,r=0,i=Array.isArray(u)?u.length:1;r<i;r++){var l=1===i?u:u[r],c=l.e,h=l.promise,f=e.Debug&&(s&&s.name===a?e.Debug.MS_ASYNC_OP_STATUS_CANCELED:e.Debug.MS_ASYNC_OP_STATUS_ERROR);if(o._traceAsyncOperationCompleted(l.asyncOpID,f),h){var m=!1;try{c?(o._traceAsyncCallbackStarting(l.asyncOpID),m=!0,c.handlesOnError||k(h,s,A,t,c),h._setCompleteValue(c(s))):h._setChainedErrorValue(s,t)}catch(e){h._setExceptionValue(e)}finally{m&&o._traceAsyncCallbackCompleted()}h._state!==d&&h._listeners&&n.push(h)}else B.prototype.done.call(t,null,c)}}function k(e,t,n,r,i){if(s._listeners[u]){if(t instanceof Error&&t.message===a)return;s.dispatchEvent(u,n(e,t,r,i))}}function R(e,t){var n,r,i=e._listeners;if(i)for(n=0,r=Array.isArray(i)?i.length:1;n<r;n++){var o=1===r?i:i[n],s=o.p;if(s)try{s(t)}catch(e){}o.c||o.e||!o.promise||o.promise._progress(t)}}function U(e,t){var n=e._listeners;n?(n=Array.isArray(n)?n:[n]).push(t):n=t,e._listeners=n}function D(e,t,n){e._isException=n||!1,e._errorId=t}function F(e,t,n,r){e._value=t,k(e,t,n,r),e._setState(C)}function W(e,t){var n;n=t&&\"object\"==typeof t&&\"function\"==typeof t.then?d:v,e._value=t,e._setState(n)}function V(e,t,n,r){var i=new q(e);return U(e,{promise:i,c:t,e:n,p:r,asyncOpID:o._traceAsyncOperationStarting(\"WinJS.Promise.then\")}),i}var K,q=n.Class.derive(E,function(e){l&&(!0===l||l&c.thenPromise)&&(this._stack=H._getStack()),this._creator=e,this._setState(h),this._run()},{_creator:null,_cancelAction:function(){this._creator&&this._creator.cancel()},_cleanupAction:function(){this._creator=null}},{supportedForProcessing:!1}),B=n.Class.define(function(e){l&&(!0===l||l&c.errorPromise)&&(this._stack=H._getStack()),this._value=e,k(this,e,P)},{cancel:function(){},done:function(e,t){var n=this._value;if(t)try{t.handlesOnError||k(null,n,A,this,t);var r=t(n);return void(r&&\"object\"==typeof r&&\"function\"==typeof r.done&&r.done())}catch(e){n=e}n instanceof Error&&n.message===a||H._doneHandler(n)},then:function(e,t){if(!t)return this;var n,r=this._value;try{t.handlesOnError||k(null,r,A,this,t),n=new Y(t(r))}catch(e){n=e===r?this:new j(e)}return n}},{supportedForProcessing:!1}),j=n.Class.derive(B,function(e){l&&(!0===l||l&c.exceptionPromise)&&(this._stack=H._getStack()),this._value=e,k(this,e,O)},{},{supportedForProcessing:!1}),Y=n.Class.define(function(e){if(l&&(!0===l||l&c.completePromise)&&(this._stack=H._getStack()),e&&\"object\"==typeof e&&\"function\"==typeof e.then){var t=new q(null);return t._setCompleteValue(e),t}this._value=e},{cancel:function(){},done:function(e){if(e)try{var t=e(this._value);t&&\"object\"==typeof t&&\"function\"==typeof t.done&&t.done()}catch(e){H._doneHandler(e)}},then:function(e){try{var t=e?e(this._value):this._value;return t===this._value?this:new Y(t)}catch(e){return new j(e)}}},{supportedForProcessing:!1});var H=n.Class.derive(E,function(e,t){l&&(!0===l||l&c.promise)&&(this._stack=H._getStack()),this._oncancel=t,this._setState(h),this._run();try{e(this._completed.bind(this),this._error.bind(this),this._progress.bind(this))}catch(e){this._setExceptionValue(e)}},{_oncancel:null,_cancelAction:function(){try{if(!this._oncancel)throw new Error(\"Promise did not implement oncancel\");this._oncancel()}catch(e){e.message,e.stack;s.dispatchEvent(\"error\",e)}},_cleanupAction:function(){this._oncancel=null}},{addEventListener:function(e,t,n){s.addEventListener(e,t,n)},any:function(e){return new H(function(t,n){var r=Object.keys(e);0===r.length&&t();var i=0;r.forEach(function(o){H.as(e[o]).then(function(){t({key:o,value:e[o]})},function(s){s instanceof Error&&s.name===a?++i===r.length&&t(H.cancel):n({key:o,value:e[o]})})})},function(){Object.keys(e).forEach(function(t){var n=H.as(e[t]);\"function\"==typeof n.cancel&&n.cancel()})})},as:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then?e:new Y(e)},cancel:{get:function(){return K=K||new B(new r(a))}},dispatchEvent:function(e,t){return s.dispatchEvent(e,t)},is:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then},join:function(e){return new H(function(t,n,r){var i=Object.keys(e),o=Array.isArray(e)?[]:{},s=Array.isArray(e)?[]:{},u=0,l=i.length,c=function(e){if(0==--l){var u=Object.keys(o).length;if(0===u)t(s);else{var c=0;i.forEach(function(e){var t=o[e];t instanceof Error&&t.name===a&&c++}),c===u?t(H.cancel):n(o)}}else r({Key:e,Done:!0})};i.forEach(function(t){var n=e[t];void 0===n?u++:H.then(n,function(e){s[t]=e,c(t)},function(e){o[t]=e,c(t)})}),0!==(l-=u)||t(s)},function(){Object.keys(e).forEach(function(t){var n=H.as(e[t]);\"function\"==typeof n.cancel&&n.cancel()})})},removeEventListener:function(e,t,n){s.removeEventListener(e,t,n)},supportedForProcessing:!1,then:function(e,t,n,r){return H.as(e).then(t,n,r)},thenEach:function(e,t,n,r){var i=Array.isArray(e)?[]:{};return Object.keys(e).forEach(function(o){i[o]=H.as(e[o]).then(t,n,r)}),H.join(i)},timeout:function(n,r){var i,o,s=(i=n,new H(function(n){i?o=e.setTimeout(n,i):t._setImmediate(n)},function(){o&&e.clearTimeout(o)}));return r?function(e,t){var n=function(){e.cancel()};return e.then(function(){t.cancel()}),t.then(n,n),t}(s,r):s},wrap:function(e){return new Y(e)},wrapError:function(e){return new B(e)},_veryExpensiveTagWithStack:{get:function(){return l},set:function(e){l=e}},_veryExpensiveTagWithStack_tag:c,_getStack:function(){if(e.Debug&&e.Debug.debuggerEnabled)try{throw new Error}catch(e){return e.stack}},_cancelBlocker:function(e,t){if(!H.is(e))return H.wrap(e);var n,r,i=new H(function(e,t){n=e,r=t},function(){n=null,r=null,t&&t()});return e.then(function(e){n&&n(e)},function(e){r&&r(e)}),i}});return Object.defineProperties(H,i.createEventProperties(u)),H._doneHandler=function(e){t._setImmediate(function(){throw e})},{PromiseStateMachine:E,Promise:H,state_created:h}}),n(\"WinJS/Promise\",[\"WinJS/Core/_Base\",\"WinJS/Promise/_StateMachine\"],function(e,t){return e.Namespace.define(\"WinJS\",{Promise:t.Promise}),t.Promise}),(t=e[\"WinJS/Core/_WinJS\"]).TPromise=t.Promise,t.PPromise=t.Promise}();var n=t.Promise,r=t.TPromise,i=(t.PPromise,{});r.addEventListener(\"error\",function(e){var t=e.detail,n=t.id;t.parent?t.handler&&i&&delete i[n]:(i[n]=t,1===Object.keys(i).length&&setTimeout(function(){var e=i;i={},Object.keys(e).forEach(function(t){var n=e[t];n.exception?s(n.exception):n.error&&s(n.error),console.log(\"WARNING: Promise with no error callback:\"+n.id),console.log(n),n.exception&&console.log(n.exception.stack)})},0))});var o=new(function(){function e(){this.listeners=[],this.unexpectedErrorHandler=function(e){setTimeout(function(){if(e.stack)throw new Error(e.message+\"\\n\\n\"+e.stack);throw e},0)}}return e.prototype.emit=function(e){this.listeners.forEach(function(t){t(e)})},e.prototype.onUnexpectedError=function(e){this.unexpectedErrorHandler(e),this.emit(e)},e.prototype.onUnexpectedExternalError=function(e){this.unexpectedErrorHandler(e)},e}());function s(e){var t;(t=e)instanceof Error&&t.name===a&&t.message===a||o.onUnexpectedError(e)}function u(e){return e instanceof Error?{$isError:!0,name:e.name,message:e.message,stack:e.stacktrace||e.stack}:e}var a=\"Canceled\";var l=function(){function e(){this._toDispose=[]}return e.prototype.dispose=function(){this._toDispose=function e(t){for(var n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];return Array.isArray(t)?(t.forEach(function(e){return e&&e.dispose()}),[]):0===n.length?t?(t.dispose(),t):void 0:(e(t),e(n),[])}(this._toDispose)},e.prototype._register=function(e){return this._toDispose.push(e),e},e.None=Object.freeze({dispose:function(){}}),e}(),c=!1,h=\"undefined\"!=typeof process&&void 0!==process.versions&&void 0!==process.versions.electron&&\"renderer\"===process.type;if(\"object\"!=typeof navigator||h){if(\"object\"==typeof process){c=\"win32\"===process.platform,\"darwin\"===process.platform,\"linux\"===process.platform,\"en\";var f=process.env.VSCODE_NLS_CONFIG;if(f)try{var d=JSON.parse(f);d.availableLanguages[\"*\"];d.locale,d._translationsConfigFile}catch(e){}}}else{var m=navigator.userAgent;c=m.indexOf(\"Windows\")>=0,m.indexOf(\"Macintosh\")>=0,m.indexOf(\"Linux\")>=0,navigator.language}var p=c,_=\"object\"==typeof self?self:\"object\"==typeof global?global:{},g=null;function v(e){return null===g&&(g=_.setImmediate?_.setImmediate.bind(_):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):_.setTimeout.bind(_)),g(e)}var y,C,b=(y=function(e,t){return(y=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}y(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),S={done:!0,value:void 0};!function(e){var t={next:function(){return S}};function n(e,t){for(var n=e.next();!n.done;n=e.next())t(n.value)}e.empty=function(){return t},e.fromArray=function(e,t,n){return void 0===t&&(t=0),void 0===n&&(n=e.length),{next:function(){return t>=n?S:{done:!1,value:e[t++]}}}},e.from=function(t){return t?Array.isArray(t)?e.fromArray(t):t:e.empty()},e.map=function(e,t){return{next:function(){var n=e.next();return n.done?S:{done:!1,value:t(n.value)}}}},e.filter=function(e,t){return{next:function(){for(;;){var n=e.next();if(n.done)return S;if(t(n.value))return{done:!1,value:n.value}}}}},e.forEach=n,e.collect=function(e){var t=[];return n(e,function(e){return t.push(e)}),t}}(C||(C={}));!function(e){function t(t,n,r,i){return void 0===n&&(n=0),void 0===r&&(r=t.length),void 0===i&&(i=n-1),e.call(this,t,n,r,i)||this}b(t,e),t.prototype.current=function(){return e.prototype.current.call(this)},t.prototype.previous=function(){return this.index=Math.max(this.index-1,this.start-1),this.current()},t.prototype.first=function(){return this.index=this.start,this.current()},t.prototype.last=function(){return this.index=this.end-1,this.current()},t.prototype.parent=function(){return null}}(function(){function e(e,t,n,r){void 0===t&&(t=0),void 0===n&&(n=e.length),void 0===r&&(r=t-1),this.items=e,this.start=t,this.end=n,this.index=r}return e.prototype.next=function(){return this.index=Math.min(this.index+1,this.end),this.current()},e.prototype.current=function(){return this.index===this.start-1||this.index===this.end?null:this.items[this.index]},e}());var E,N=function(){return function(e){this.element=e}}(),L=function(){function e(){}return e.prototype.isEmpty=function(){return!this._first},e.prototype.unshift=function(e){return this.insert(e,!1)},e.prototype.push=function(e){return this.insert(e,!0)},e.prototype.insert=function(e,t){var n=this,r=new N(e);if(this._first)if(t){var i=this._last;this._last=r,r.prev=i,i.next=r}else{var o=this._first;this._first=r,r.next=o,o.prev=r}else this._first=r,this._last=r;return function(){for(var e=n._first;e instanceof N;){if(e===r){if(e.prev&&e.next){var t=e.prev;t.next=e.next,e.next.prev=t}else e.prev||e.next?e.next?e.prev||(n._first=n._first.next,n._first.prev=void 0):(n._last=n._last.prev,n._last.next=void 0):(n._first=void 0,n._last=void 0);break}e=e.next}}},e.prototype.iterator=function(){var e,t=this._first;return{next:function(){return t?(e?e.value=t.element:e={done:!1,value:t.element},t=t.next,e):S}}},e}();!function(e){var t={dispose:function(){}};e.None=function(){return t}}(E||(E={}));var A=function(){function e(e){void 0===e&&(e=null),this._options=e,this._event=null,this._disposed=!1,this._deliveryQueue=null,this._listeners=null}return Object.defineProperty(e.prototype,\"event\",{get:function(){var t=this;return this._event||(this._event=function(n,r,i){t._listeners||(t._listeners=new L);var o=t._listeners.isEmpty();o&&t._options&&t._options.onFirstListenerAdd&&t._options.onFirstListenerAdd(t);var s,u=t._listeners.push(r?[n,r]:n);return o&&t._options&&t._options.onFirstListenerDidAdd&&t._options.onFirstListenerDidAdd(t),t._options&&t._options.onListenerDidAdd&&t._options.onListenerDidAdd(t,n,r),s={dispose:function(){(s.dispose=e._noop,t._disposed)||(u(),t._options&&t._options.onLastListenerRemove&&(t._listeners&&!t._listeners.isEmpty()||t._options.onLastListenerRemove(t)))}},Array.isArray(i)&&i.push(s),s}),this._event},enumerable:!0,configurable:!0}),e.prototype.fire=function(e){if(this._listeners){this._deliveryQueue||(this._deliveryQueue=[]);for(var t=this._listeners.iterator(),n=t.next();!n.done;n=t.next())this._deliveryQueue.push([n.value,e]);for(;this._deliveryQueue.length>0;){var r=this._deliveryQueue.shift(),i=r[0],o=r[1];try{\"function\"==typeof i?i.call(void 0,o):i[0].call(i[1],o)}catch(n){s(n)}}}},e.prototype.dispose=function(){this._listeners&&(this._listeners=null),this._deliveryQueue&&(this._deliveryQueue.length=0),this._disposed=!0},e._noop=function(){},e}();!function(){function e(){var e=this;this.hasListeners=!1,this.events=[],this.emitter=new A({onFirstListenerAdd:function(){return e.onFirstListenerAdd()},onLastListenerRemove:function(){return e.onLastListenerRemove()}})}Object.defineProperty(e.prototype,\"event\",{get:function(){return this.emitter.event},enumerable:!0,configurable:!0}),e.prototype.add=function(e){var t=this,n={event:e,listener:null};this.events.push(n),this.hasListeners&&this.hook(n);var r;return r=function(e){var t,n=this,r=!1;return function(){return r?t:(r=!0,t=e.apply(n,arguments))}}(function(){t.hasListeners&&t.unhook(n);var e=t.events.indexOf(n);t.events.splice(e,1)}),{dispose:function(){r()}}},e.prototype.onFirstListenerAdd=function(){var e=this;this.hasListeners=!0,this.events.forEach(function(t){return e.hook(t)})},e.prototype.onLastListenerRemove=function(){var e=this;this.hasListeners=!1,this.events.forEach(function(t){return e.unhook(t)})},e.prototype.hook=function(e){var t=this;e.listener=e.event(function(e){return t.emitter.fire(e)})},e.prototype.unhook=function(e){e.listener&&e.listener.dispose(),e.listener=null},e.prototype.dispose=function(){this.emitter.dispose()}}();(function(){function e(e){this._event=e}Object.defineProperty(e.prototype,\"event\",{get:function(){return this._event},enumerable:!0,configurable:!0}),e.prototype.map=function(t){return new e((n=this._event,r=t,function(e,t,i){return void 0===t&&(t=null),n(function(n){return e.call(t,r(n))},null,i)}));var n,r},e.prototype.filter=function(t){return new e((n=this._event,r=t,function(e,t,i){return void 0===t&&(t=null),n(function(n){return r(n)&&e.call(t,n)},null,i)}));var n,r},e.prototype.on=function(e,t,n){return this._event(e,t,n)}})(),function(){function e(){var e=this;this.listening=!1,this.inputEvent=E.None,this.inputEventListener=l.None,this.emitter=new A({onFirstListenerDidAdd:function(){e.listening=!0,e.inputEventListener=e.inputEvent(e.emitter.fire,e.emitter)},onLastListenerRemove:function(){e.listening=!1,e.inputEventListener.dispose()}}),this.event=this.emitter.event}Object.defineProperty(e.prototype,\"input\",{set:function(e){this.inputEvent=e,this.listening&&(this.inputEventListener.dispose(),this.inputEventListener=e(this.emitter.fire,this.emitter))},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.inputEventListener.dispose(),this.emitter.dispose()}}();var w,P=Object.freeze(function(e,t){var n=setTimeout(e.bind(t),0);return{dispose:function(){clearTimeout(n)}}});!function(e){e.isCancellationToken=function(t){return t===e.None||t===e.Cancelled||t instanceof O||!(!t||\"object\"!=typeof t)&&\"boolean\"==typeof t.isCancellationRequested&&\"function\"==typeof t.onCancellationRequested},e.None=Object.freeze({isCancellationRequested:!1,onCancellationRequested:E.None}),e.Cancelled=Object.freeze({isCancellationRequested:!0,onCancellationRequested:P})}(w||(w={}));var O=function(){function e(){this._isCancelled=!1,this._emitter=null}return e.prototype.cancel=function(){this._isCancelled||(this._isCancelled=!0,this._emitter&&(this._emitter.fire(void 0),this.dispose()))},Object.defineProperty(e.prototype,\"isCancellationRequested\",{get:function(){return this._isCancelled},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"onCancellationRequested\",{get:function(){return this._isCancelled?P:(this._emitter||(this._emitter=new A),this._emitter.event)},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this._emitter&&(this._emitter.dispose(),this._emitter=null)},e}(),M=function(){function e(){}return Object.defineProperty(e.prototype,\"token\",{get:function(){return this._token||(this._token=new O),this._token},enumerable:!0,configurable:!0}),e.prototype.cancel=function(){this._token?this._token instanceof O&&this._token.cancel():this._token=w.Cancelled},e.prototype.dispose=function(){this._token?this._token instanceof O&&this._token.dispose():this._token=w.None},e}(),x=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();(function(e){function t(t,n){var r=e.call(this)||this;return r._token=-1,\"function\"==typeof t&&\"number\"==typeof n&&r.setIfNotSet(t,n),r}x(t,e),t.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},t.prototype.cancel=function(){-1!==this._token&&(clearTimeout(this._token),this._token=-1)},t.prototype.cancelAndSet=function(e,t){var n=this;this.cancel(),this._token=setTimeout(function(){n._token=-1,e()},t)},t.prototype.setIfNotSet=function(e,t){var n=this;-1===this._token&&(this._token=setTimeout(function(){n._token=-1,e()},t))}})(l),function(e){function t(){var t=e.call(this)||this;return t._token=-1,t}x(t,e),t.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},t.prototype.cancel=function(){-1!==this._token&&(clearInterval(this._token),this._token=-1)},t.prototype.cancelAndSet=function(e,t){this.cancel(),this._token=setInterval(function(){e()},t)}}(l);!function(){if(\"function\"!=typeof requestIdleCallback||\"function\"!=typeof cancelIdleCallback)Object.freeze({didTimeout:!0,timeRemaining:function(){return 15}})}();var T=function(){function e(e){var t,r;(r=t=e)&&\"function\"==typeof r.then&&\"function\"==typeof t.done?this._winjsPromise=e:this._winjsPromise=new n(function(t,n){var r=!0;e(function(e){r?v(function(){return t(e)}):t(e)},function(e){r?v(function(){return n(e)}):n(e)}),r=!1})}return e.all=function(t){return new e(n.join(t).then(null,function(e){for(var t in e)if(e.hasOwnProperty(t))return e[t]}))},e.race=function(t){return new e(n.any(t).then(function(e){return e.value},function(e){return e.value}))},e.resolve=function(t){return new e(n.wrap(t))},e.reject=function(t){return new e(n.wrapError(t))},e.prototype.then=function(t,r){var i=!0,o=new e(this._winjsPromise.then(t&&function(e){return i?new n(function(n,r){v(function(){var i;try{i=t(e)}catch(e){return void r(e)}n(i)})}):t(e)},r&&function(e){return i?new n(function(t,n){v(function(){var i;try{i=r(e)}catch(e){return void n(e)}t(i)})}):r(e)}));return i=!1,o},e.prototype.catch=function(e){return this.then(null,e)},e}(),I=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),k=self;void 0===k.Promise&&(k.Promise=T);var R=\"$initialize\",U=function(){function e(e){this._workerId=-1,this._handler=e,this._lastSentReq=0,this._pendingReplies=Object.create(null)}return e.prototype.setWorkerId=function(e){this._workerId=e},e.prototype.sendMessage=function(e,t){var n=this,r=String(++this._lastSentReq);return new Promise(function(i,o){n._pendingReplies[r]={resolve:i,reject:o},n._send({vsWorker:n._workerId,req:r,method:e,args:t})})},e.prototype.handleMessage=function(e){var t;try{t=JSON.parse(e)}catch(e){return}t&&t.vsWorker&&(-1!==this._workerId&&t.vsWorker!==this._workerId||this._handleMessage(t))},e.prototype._handleMessage=function(e){var t=this;if(e.seq){var n=e;if(!this._pendingReplies[n.seq])return void console.warn(\"Got reply to unknown seq\");var r=this._pendingReplies[n.seq];if(delete this._pendingReplies[n.seq],n.err){var i=n.err;return n.err.$isError&&((i=new Error).name=n.err.name,i.message=n.err.message,i.stack=n.err.stack),void r.reject(i)}r.resolve(n.res)}else{var o=e,s=o.req;this._handler.handleMessage(o.method,o.args).then(function(e){t._send({vsWorker:t._workerId,seq:s,res:e,err:void 0})},function(e){e.detail instanceof Error&&(e.detail=u(e.detail)),t._send({vsWorker:t._workerId,seq:s,res:void 0,err:u(e)})})}},e.prototype._send=function(e){var t=JSON.stringify(e);this._handler.sendMessage(t)},e}(),D=(function(e){function t(t,n){var r=e.call(this)||this,i=null;r._worker=r._register(t.create(\"vs/base/common/worker/simpleWorker\",function(e){r._protocol.handleMessage(e)},function(e){i&&i(e)})),r._protocol=new U({sendMessage:function(e){r._worker.postMessage(e)},handleMessage:function(e,t){return Promise.resolve(null)}}),r._protocol.setWorkerId(r._worker.getId());var o=null;void 0!==self.require&&\"function\"==typeof self.require.getConfig?o=self.require.getConfig():void 0!==self.requirejs&&(o=self.requirejs.s.contexts._.config),r._onModuleLoaded=r._protocol.sendMessage(R,[r._worker.getId(),n,o]),r._lazyProxy=new Promise(function(e,t){i=t,r._onModuleLoaded.then(function(t){for(var n={},r=0;r<t.length;r++)n[t[r]]=u(t[r],s);e(n)},function(e){t(e),r._onError(\"Worker failed to load \"+n,e)})});var s=function(e,t){return r._request(e,t)},u=function(e,t){return function(){var n=Array.prototype.slice.call(arguments,0);return t(e,n)}};return r}I(t,e),t.prototype.getProxyObject=function(){return this._lazyProxy},t.prototype._request=function(e,t){var n=this;return new Promise(function(r,i){n._onModuleLoaded.then(function(){n._protocol.sendMessage(e,t).then(r,i)},i)})},t.prototype._onError=function(e,t){console.error(e),console.info(t)}}(l),function(){function e(e,t){var n=this;this._requestHandler=t,this._protocol=new U({sendMessage:function(t){e(t)},handleMessage:function(e,t){return n._handleMessage(e,t)}})}return e.prototype.onmessage=function(e){this._protocol.handleMessage(e)},e.prototype._handleMessage=function(e,t){if(e===R)return this.initialize(t[0],t[1],t[2]);if(!this._requestHandler||\"function\"!=typeof this._requestHandler[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._requestHandler[e].apply(this._requestHandler,t))}catch(e){return Promise.reject(e)}},e.prototype.initialize=function(e,t,n){var r=this;if(this._protocol.setWorkerId(e),this._requestHandler){var i=[];for(var o in this._requestHandler)\"function\"==typeof this._requestHandler[o]&&i.push(o);return Promise.resolve(i)}return n&&(void 0!==n.baseUrl&&delete n.baseUrl,void 0!==n.paths&&void 0!==n.paths.vs&&delete n.paths.vs,n.catchError=!0,self.require.config(n)),new Promise(function(e,n){self.require([t],function(){for(var t=[],i=0;i<arguments.length;i++)t[i]=arguments[i];var o=t[0];if(r._requestHandler=o.create(),r._requestHandler){var s=[];for(var u in r._requestHandler)\"function\"==typeof r._requestHandler[u]&&s.push(u);e(s)}else n(new Error(\"No RequestHandler!\"))},n)})},e}());function F(e,t){return function e(t,n,r,i,o){if(i<=r)return;var s=r+(i-r)/2|0;e(t,n,r,s,o);e(t,n,s+1,i,o);if(n(t[s],t[s+1])<=0)return;!function(e,t,n,r,i,o){for(var s=n,u=r+1,a=n;a<=i;a++)o[a]=e[a];for(var a=n;a<=i;a++)s>r?e[a]=o[u++]:u>i?e[a]=o[s++]:t(o[u],o[s])<0?e[a]=o[u++]:e[a]=o[s++]}(t,n,r,s,i,o)}(e,t,0,e.length-1,[]),e}var W=function(){function e(e,t,n,r){this.originalStart=e,this.originalLength=t,this.modifiedStart=n,this.modifiedLength=r}return e.prototype.getOriginalEnd=function(){return this.originalStart+this.originalLength},e.prototype.getModifiedEnd=function(){return this.modifiedStart+this.modifiedLength},e}();function V(e){return{getLength:function(){return e.length},getElementAtIndex:function(t){return e.charCodeAt(t)}}}function K(e,t,n){return new H(V(e),V(t)).ComputeDiff(n)}var q,B=function(){function e(){}return e.Assert=function(e,t){if(!e)throw new Error(t)},e}(),j=function(){function e(){}return e.Copy=function(e,t,n,r,i){for(var o=0;o<i;o++)n[r+o]=e[t+o]},e}(),Y=function(){function e(){this.m_changes=[],this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE,this.m_originalCount=0,this.m_modifiedCount=0}return e.prototype.MarkNextChange=function(){(this.m_originalCount>0||this.m_modifiedCount>0)&&this.m_changes.push(new W(this.m_originalStart,this.m_originalCount,this.m_modifiedStart,this.m_modifiedCount)),this.m_originalCount=0,this.m_modifiedCount=0,this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE},e.prototype.AddOriginalElement=function(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_originalCount++},e.prototype.AddModifiedElement=function(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_modifiedCount++},e.prototype.getChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes},e.prototype.getReverseChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes.reverse(),this.m_changes},e}(),H=function(){function e(e,t,n){void 0===n&&(n=null),this.OriginalSequence=e,this.ModifiedSequence=t,this.ContinueProcessingPredicate=n,this.m_forwardHistory=[],this.m_reverseHistory=[]}return e.prototype.ElementsAreEqual=function(e,t){return this.OriginalSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(t)},e.prototype.OriginalElementsAreEqual=function(e,t){return this.OriginalSequence.getElementAtIndex(e)===this.OriginalSequence.getElementAtIndex(t)},e.prototype.ModifiedElementsAreEqual=function(e,t){return this.ModifiedSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(t)},e.prototype.ComputeDiff=function(e){return this._ComputeDiff(0,this.OriginalSequence.getLength()-1,0,this.ModifiedSequence.getLength()-1,e)},e.prototype._ComputeDiff=function(e,t,n,r,i){var o=this.ComputeDiffRecursive(e,t,n,r,[!1]);return i?this.PrettifyChanges(o):o},e.prototype.ComputeDiffRecursive=function(e,t,n,r,i){for(i[0]=!1;e<=t&&n<=r&&this.ElementsAreEqual(e,n);)e++,n++;for(;t>=e&&r>=n&&this.ElementsAreEqual(t,r);)t--,r--;if(e>t||n>r){var o=void 0;return n<=r?(B.Assert(e===t+1,\"originalStart should only be one more than originalEnd\"),o=[new W(e,0,n,r-n+1)]):e<=t?(B.Assert(n===r+1,\"modifiedStart should only be one more than modifiedEnd\"),o=[new W(e,t-e+1,n,0)]):(B.Assert(e===t+1,\"originalStart should only be one more than originalEnd\"),B.Assert(n===r+1,\"modifiedStart should only be one more than modifiedEnd\"),o=[]),o}var s=[0],u=[0],a=this.ComputeRecursionPoint(e,t,n,r,s,u,i),l=s[0],c=u[0];if(null!==a)return a;if(!i[0]){var h=this.ComputeDiffRecursive(e,l,n,c,i),f=[];return f=i[0]?[new W(l+1,t-(l+1)+1,c+1,r-(c+1)+1)]:this.ComputeDiffRecursive(l+1,t,c+1,r,i),this.ConcatenateChanges(h,f)}return[new W(e,t-e+1,n,r-n+1)]},e.prototype.WALKTRACE=function(e,t,n,r,i,o,s,u,a,l,c,h,f,d,m,p,_,g){var v,y,C=null,b=new Y,S=t,E=n,N=f[0]-p[0]-r,L=Number.MIN_VALUE,A=this.m_forwardHistory.length-1;do{(y=N+e)===S||y<E&&a[y-1]<a[y+1]?(d=(c=a[y+1])-N-r,c<L&&b.MarkNextChange(),L=c,b.AddModifiedElement(c+1,d),N=y+1-e):(d=(c=a[y-1]+1)-N-r,c<L&&b.MarkNextChange(),L=c-1,b.AddOriginalElement(c,d+1),N=y-1-e),A>=0&&(e=(a=this.m_forwardHistory[A])[0],S=1,E=a.length-1)}while(--A>=-1);if(v=b.getReverseChanges(),g[0]){var w=f[0]+1,P=p[0]+1;if(null!==v&&v.length>0){var O=v[v.length-1];w=Math.max(w,O.getOriginalEnd()),P=Math.max(P,O.getModifiedEnd())}C=[new W(w,h-w+1,P,m-P+1)]}else{b=new Y,S=o,E=s,N=f[0]-p[0]-u,L=Number.MAX_VALUE,A=_?this.m_reverseHistory.length-1:this.m_reverseHistory.length-2;do{(y=N+i)===S||y<E&&l[y-1]>=l[y+1]?(d=(c=l[y+1]-1)-N-u,c>L&&b.MarkNextChange(),L=c+1,b.AddOriginalElement(c+1,d+1),N=y+1-i):(d=(c=l[y-1])-N-u,c>L&&b.MarkNextChange(),L=c,b.AddModifiedElement(c+1,d+1),N=y-1-i),A>=0&&(i=(l=this.m_reverseHistory[A])[0],S=1,E=l.length-1)}while(--A>=-1);C=b.getChanges()}return this.ConcatenateChanges(v,C)},e.prototype.ComputeRecursionPoint=function(e,t,n,r,i,o,s){var u,a=0,l=0,c=0,h=0,f=0,d=0;e--,n--,i[0]=0,o[0]=0,this.m_forwardHistory=[],this.m_reverseHistory=[];var m,p,_=t-e+(r-n),g=_+1,v=new Array(g),y=new Array(g),C=r-n,b=t-e,S=e-n,E=t-r,N=(b-C)%2==0;for(v[C]=e,y[b]=t,s[0]=!1,u=1;u<=_/2+1;u++){var L=0,A=0;for(c=this.ClipDiagonalBound(C-u,u,C,g),h=this.ClipDiagonalBound(C+u,u,C,g),m=c;m<=h;m+=2){for(l=(a=m===c||m<h&&v[m-1]<v[m+1]?v[m+1]:v[m-1]+1)-(m-C)-S,p=a;a<t&&l<r&&this.ElementsAreEqual(a+1,l+1);)a++,l++;if(v[m]=a,a+l>L+A&&(L=a,A=l),!N&&Math.abs(m-b)<=u-1&&a>=y[m])return i[0]=a,o[0]=l,p<=y[m]&&u<=1448?this.WALKTRACE(C,c,h,S,b,f,d,E,v,y,a,t,i,l,r,o,N,s):null}var w=(L-e+(A-n)-u)/2;if(null!==this.ContinueProcessingPredicate&&!this.ContinueProcessingPredicate(L,this.OriginalSequence,w))return s[0]=!0,i[0]=L,o[0]=A,w>0&&u<=1448?this.WALKTRACE(C,c,h,S,b,f,d,E,v,y,a,t,i,l,r,o,N,s):[new W(++e,t-e+1,++n,r-n+1)];for(f=this.ClipDiagonalBound(b-u,u,b,g),d=this.ClipDiagonalBound(b+u,u,b,g),m=f;m<=d;m+=2){for(l=(a=m===f||m<d&&y[m-1]>=y[m+1]?y[m+1]-1:y[m-1])-(m-b)-E,p=a;a>e&&l>n&&this.ElementsAreEqual(a,l);)a--,l--;if(y[m]=a,N&&Math.abs(m-C)<=u&&a<=v[m])return i[0]=a,o[0]=l,p>=v[m]&&u<=1448?this.WALKTRACE(C,c,h,S,b,f,d,E,v,y,a,t,i,l,r,o,N,s):null}if(u<=1447){var P=new Array(h-c+2);P[0]=C-c+1,j.Copy(v,c,P,1,h-c+1),this.m_forwardHistory.push(P),(P=new Array(d-f+2))[0]=b-f+1,j.Copy(y,f,P,1,d-f+1),this.m_reverseHistory.push(P)}}return this.WALKTRACE(C,c,h,S,b,f,d,E,v,y,a,t,i,l,r,o,N,s)},e.prototype.PrettifyChanges=function(e){for(var t=0;t<e.length;t++){for(var n=e[t],r=t<e.length-1?e[t+1].originalStart:this.OriginalSequence.getLength(),i=t<e.length-1?e[t+1].modifiedStart:this.ModifiedSequence.getLength(),o=n.originalLength>0,s=n.modifiedLength>0;n.originalStart+n.originalLength<r&&n.modifiedStart+n.modifiedLength<i&&(!o||this.OriginalElementsAreEqual(n.originalStart,n.originalStart+n.originalLength))&&(!s||this.ModifiedElementsAreEqual(n.modifiedStart,n.modifiedStart+n.modifiedLength));)n.originalStart++,n.modifiedStart++;var u=[null];t<e.length-1&&this.ChangesOverlap(e[t],e[t+1],u)&&(e[t]=u[0],e.splice(t+1,1),t--)}for(t=e.length-1;t>=0;t--){n=e[t],r=0,i=0;if(t>0){var a=e[t-1];a.originalLength>0&&(r=a.originalStart+a.originalLength),a.modifiedLength>0&&(i=a.modifiedStart+a.modifiedLength)}o=n.originalLength>0,s=n.modifiedLength>0;for(var l=0,c=this._boundaryScore(n.originalStart,n.originalLength,n.modifiedStart,n.modifiedLength),h=1;;h++){var f=n.originalStart-h,d=n.modifiedStart-h;if(f<r||d<i)break;if(o&&!this.OriginalElementsAreEqual(f,f+n.originalLength))break;if(s&&!this.ModifiedElementsAreEqual(d,d+n.modifiedLength))break;var m=this._boundaryScore(f,n.originalLength,d,n.modifiedLength);m>c&&(c=m,l=h)}n.originalStart-=l,n.modifiedStart-=l}return e},e.prototype._OriginalIsBoundary=function(e){if(e<=0||e>=this.OriginalSequence.getLength()-1)return!0;var t=this.OriginalSequence.getElementAtIndex(e);return\"string\"==typeof t&&/^\\s*$/.test(t)},e.prototype._OriginalRegionIsBoundary=function(e,t){if(this._OriginalIsBoundary(e)||this._OriginalIsBoundary(e-1))return!0;if(t>0){var n=e+t;if(this._OriginalIsBoundary(n-1)||this._OriginalIsBoundary(n))return!0}return!1},e.prototype._ModifiedIsBoundary=function(e){if(e<=0||e>=this.ModifiedSequence.getLength()-1)return!0;var t=this.ModifiedSequence.getElementAtIndex(e);return\"string\"==typeof t&&/^\\s*$/.test(t)},e.prototype._ModifiedRegionIsBoundary=function(e,t){if(this._ModifiedIsBoundary(e)||this._ModifiedIsBoundary(e-1))return!0;if(t>0){var n=e+t;if(this._ModifiedIsBoundary(n-1)||this._ModifiedIsBoundary(n))return!0}return!1},e.prototype._boundaryScore=function(e,t,n,r){return(this._OriginalRegionIsBoundary(e,t)?1:0)+(this._ModifiedRegionIsBoundary(n,r)?1:0)},e.prototype.ConcatenateChanges=function(e,t){var n=[];if(0===e.length||0===t.length)return t.length>0?t:e;if(this.ChangesOverlap(e[e.length-1],t[0],n)){var r=new Array(e.length+t.length-1);return j.Copy(e,0,r,0,e.length-1),r[e.length-1]=n[0],j.Copy(t,1,r,e.length,t.length-1),r}r=new Array(e.length+t.length);return j.Copy(e,0,r,0,e.length),j.Copy(t,0,r,e.length,t.length),r},e.prototype.ChangesOverlap=function(e,t,n){if(B.Assert(e.originalStart<=t.originalStart,\"Left change is not less than or equal to right change\"),B.Assert(e.modifiedStart<=t.modifiedStart,\"Left change is not less than or equal to right change\"),e.originalStart+e.originalLength>=t.originalStart||e.modifiedStart+e.modifiedLength>=t.modifiedStart){var r=e.originalStart,i=e.originalLength,o=e.modifiedStart,s=e.modifiedLength;return e.originalStart+e.originalLength>=t.originalStart&&(i=t.originalStart+t.originalLength-e.originalStart),e.modifiedStart+e.modifiedLength>=t.modifiedStart&&(s=t.modifiedStart+t.modifiedLength-e.modifiedStart),n[0]=new W(r,i,o,s),!0}return n[0]=null,!1},e.prototype.ClipDiagonalBound=function(e,t,n,r){if(e>=0&&e<r)return e;var i=t%2==0;return e<0?i===(n%2==0)?0:1:i===((r-n-1)%2==0)?r-1:r-2},e}(),J=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),G=/^\\w[\\w\\d+.-]*$/,z=/^\\//,Q=/^\\/\\//;var X=\"\",$=\"/\",Z=/^(([^:\\/?#]+?):)?(\\/\\/([^\\/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?/,ee=function(){function e(e,t,n,r,i){\"object\"==typeof e?(this.scheme=e.scheme||X,this.authority=e.authority||X,this.path=e.path||X,this.query=e.query||X,this.fragment=e.fragment||X):(this.scheme=e||X,this.authority=t||X,this.path=function(e,t){switch(e){case\"https\":case\"http\":case\"file\":t?t[0]!==$&&(t=$+t):t=$}return t}(this.scheme,n||X),this.query=r||X,this.fragment=i||X,function(e){if(!e.scheme)throw new Error('[UriError]: Scheme is missing: {scheme: \"\", authority: \"'+e.authority+'\", path: \"'+e.path+'\", query: \"'+e.query+'\", fragment: \"'+e.fragment+'\"}');if(e.scheme&&!G.test(e.scheme))throw new Error(\"[UriError]: Scheme contains illegal characters.\");if(e.path)if(e.authority){if(!z.test(e.path))throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash (\"/\") character')}else if(Q.test(e.path))throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters (\"//\")')}(this))}return e.isUri=function(t){return t instanceof e||!!t&&(\"string\"==typeof t.authority&&\"string\"==typeof t.fragment&&\"string\"==typeof t.path&&\"string\"==typeof t.query&&\"string\"==typeof t.scheme)},Object.defineProperty(e.prototype,\"fsPath\",{get:function(){return oe(this)},enumerable:!0,configurable:!0}),e.prototype.with=function(e){if(!e)return this;var t=e.scheme,n=e.authority,r=e.path,i=e.query,o=e.fragment;return void 0===t?t=this.scheme:null===t&&(t=X),void 0===n?n=this.authority:null===n&&(n=X),void 0===r?r=this.path:null===r&&(r=X),void 0===i?i=this.query:null===i&&(i=X),void 0===o?o=this.fragment:null===o&&(o=X),t===this.scheme&&n===this.authority&&r===this.path&&i===this.query&&o===this.fragment?this:new te(t,n,r,i,o)},e.parse=function(e){var t=Z.exec(e);return t?new te(t[2]||X,decodeURIComponent(t[4]||X),decodeURIComponent(t[5]||X),decodeURIComponent(t[7]||X),decodeURIComponent(t[9]||X)):new te(X,X,X,X,X)},e.file=function(e){var t=X;if(p&&(e=e.replace(/\\\\/g,$)),e[0]===$&&e[1]===$){var n=e.indexOf($,2);-1===n?(t=e.substring(2),e=$):(t=e.substring(2,n),e=e.substring(n)||$)}return new te(\"file\",t,e,X,X)},e.from=function(e){return new te(e.scheme,e.authority,e.path,e.query,e.fragment)},e.prototype.toString=function(e){return void 0===e&&(e=!1),se(this,e)},e.prototype.toJSON=function(){return this},e.revive=function(t){if(t){if(t instanceof e)return t;var n=new te(t);return n._fsPath=t.fsPath,n._formatted=t.external,n}return t},e}(),te=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._formatted=null,t._fsPath=null,t}return J(t,e),Object.defineProperty(t.prototype,\"fsPath\",{get:function(){return this._fsPath||(this._fsPath=oe(this)),this._fsPath},enumerable:!0,configurable:!0}),t.prototype.toString=function(e){return void 0===e&&(e=!1),e?se(this,!0):(this._formatted||(this._formatted=se(this,!1)),this._formatted)},t.prototype.toJSON=function(){var e={$mid:1};return this._fsPath&&(e.fsPath=this._fsPath),this._formatted&&(e.external=this._formatted),this.path&&(e.path=this.path),this.scheme&&(e.scheme=this.scheme),this.authority&&(e.authority=this.authority),this.query&&(e.query=this.query),this.fragment&&(e.fragment=this.fragment),e},t}(ee),ne=((q={})[58]=\"%3A\",q[47]=\"%2F\",q[63]=\"%3F\",q[35]=\"%23\",q[91]=\"%5B\",q[93]=\"%5D\",q[64]=\"%40\",q[33]=\"%21\",q[36]=\"%24\",q[38]=\"%26\",q[39]=\"%27\",q[40]=\"%28\",q[41]=\"%29\",q[42]=\"%2A\",q[43]=\"%2B\",q[44]=\"%2C\",q[59]=\"%3B\",q[61]=\"%3D\",q[32]=\"%20\",q);function re(e,t){for(var n=void 0,r=-1,i=0;i<e.length;i++){var o=e.charCodeAt(i);if(o>=97&&o<=122||o>=65&&o<=90||o>=48&&o<=57||45===o||46===o||95===o||126===o||t&&47===o)-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),void 0!==n&&(n+=e.charAt(i));else{void 0===n&&(n=e.substr(0,i));var s=ne[o];void 0!==s?(-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),n+=s):-1===r&&(r=i)}}return-1!==r&&(n+=encodeURIComponent(e.substring(r))),void 0!==n?n:e}function ie(e){for(var t=void 0,n=0;n<e.length;n++){var r=e.charCodeAt(n);35===r||63===r?(void 0===t&&(t=e.substr(0,n)),t+=ne[r]):void 0!==t&&(t+=e[n])}return void 0!==t?t:e}function oe(e){var t;return t=e.authority&&e.path.length>1&&\"file\"===e.scheme?\"//\"+e.authority+e.path:47===e.path.charCodeAt(0)&&(e.path.charCodeAt(1)>=65&&e.path.charCodeAt(1)<=90||e.path.charCodeAt(1)>=97&&e.path.charCodeAt(1)<=122)&&58===e.path.charCodeAt(2)?e.path[1].toLowerCase()+e.path.substr(2):e.path,p&&(t=t.replace(/\\//g,\"\\\\\")),t}function se(e,t){var n=t?ie:re,r=\"\",i=e.scheme,o=e.authority,s=e.path,u=e.query,a=e.fragment;if(i&&(r+=i,r+=\":\"),(o||\"file\"===i)&&(r+=$,r+=$),o){var l=o.indexOf(\"@\");if(-1!==l){var c=o.substr(0,l);o=o.substr(l+1),-1===(l=c.indexOf(\":\"))?r+=n(c,!1):(r+=n(c.substr(0,l),!1),r+=\":\",r+=n(c.substr(l+1),!1)),r+=\"@\"}-1===(l=(o=o.toLowerCase()).indexOf(\":\"))?r+=n(o,!1):(r+=n(o.substr(0,l),!1),r+=o.substr(l))}if(s){if(s.length>=3&&47===s.charCodeAt(0)&&58===s.charCodeAt(2))(h=s.charCodeAt(1))>=65&&h<=90&&(s=\"/\"+String.fromCharCode(h+32)+\":\"+s.substr(3));else if(s.length>=2&&58===s.charCodeAt(1)){var h;(h=s.charCodeAt(0))>=65&&h<=90&&(s=String.fromCharCode(h+32)+\":\"+s.substr(2))}r+=n(s,!0)}return u&&(r+=\"?\",r+=n(u,!1)),a&&(r+=\"#\",r+=t?a:re(a,!1)),r}var ue=function(){function e(e,t){this.lineNumber=e,this.column=t}return e.prototype.with=function(t,n){return void 0===t&&(t=this.lineNumber),void 0===n&&(n=this.column),t===this.lineNumber&&n===this.column?this:new e(t,n)},e.prototype.delta=function(e,t){return void 0===e&&(e=0),void 0===t&&(t=0),this.with(this.lineNumber+e,this.column+t)},e.prototype.equals=function(t){return e.equals(this,t)},e.equals=function(e,t){return!e&&!t||!!e&&!!t&&e.lineNumber===t.lineNumber&&e.column===t.column},e.prototype.isBefore=function(t){return e.isBefore(this,t)},e.isBefore=function(e,t){return e.lineNumber<t.lineNumber||!(t.lineNumber<e.lineNumber)&&e.column<t.column},e.prototype.isBeforeOrEqual=function(t){return e.isBeforeOrEqual(this,t)},e.isBeforeOrEqual=function(e,t){return e.lineNumber<t.lineNumber||!(t.lineNumber<e.lineNumber)&&e.column<=t.column},e.compare=function(e,t){var n=0|e.lineNumber,r=0|t.lineNumber;return n===r?(0|e.column)-(0|t.column):n-r},e.prototype.clone=function(){return new e(this.lineNumber,this.column)},e.prototype.toString=function(){return\"(\"+this.lineNumber+\",\"+this.column+\")\"},e.lift=function(t){return new e(t.lineNumber,t.column)},e.isIPosition=function(e){return e&&\"number\"==typeof e.lineNumber&&\"number\"==typeof e.column},e}(),ae=function(){function e(e,t,n,r){e>n||e===n&&t>r?(this.startLineNumber=n,this.startColumn=r,this.endLineNumber=e,this.endColumn=t):(this.startLineNumber=e,this.startColumn=t,this.endLineNumber=n,this.endColumn=r)}return e.prototype.isEmpty=function(){return e.isEmpty(this)},e.isEmpty=function(e){return e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn},e.prototype.containsPosition=function(t){return e.containsPosition(this,t)},e.containsPosition=function(e,t){return!(t.lineNumber<e.startLineNumber||t.lineNumber>e.endLineNumber)&&(!(t.lineNumber===e.startLineNumber&&t.column<e.startColumn)&&!(t.lineNumber===e.endLineNumber&&t.column>e.endColumn))},e.prototype.containsRange=function(t){return e.containsRange(this,t)},e.containsRange=function(e,t){return!(t.startLineNumber<e.startLineNumber||t.endLineNumber<e.startLineNumber)&&(!(t.startLineNumber>e.endLineNumber||t.endLineNumber>e.endLineNumber)&&(!(t.startLineNumber===e.startLineNumber&&t.startColumn<e.startColumn)&&!(t.endLineNumber===e.endLineNumber&&t.endColumn>e.endColumn)))},e.prototype.plusRange=function(t){return e.plusRange(this,t)},e.plusRange=function(t,n){var r,i,o,s;return n.startLineNumber<t.startLineNumber?(r=n.startLineNumber,i=n.startColumn):n.startLineNumber===t.startLineNumber?(r=n.startLineNumber,i=Math.min(n.startColumn,t.startColumn)):(r=t.startLineNumber,i=t.startColumn),n.endLineNumber>t.endLineNumber?(o=n.endLineNumber,s=n.endColumn):n.endLineNumber===t.endLineNumber?(o=n.endLineNumber,s=Math.max(n.endColumn,t.endColumn)):(o=t.endLineNumber,s=t.endColumn),new e(r,i,o,s)},e.prototype.intersectRanges=function(t){return e.intersectRanges(this,t)},e.intersectRanges=function(t,n){var r=t.startLineNumber,i=t.startColumn,o=t.endLineNumber,s=t.endColumn,u=n.startLineNumber,a=n.startColumn,l=n.endLineNumber,c=n.endColumn;return r<u?(r=u,i=a):r===u&&(i=Math.max(i,a)),o>l?(o=l,s=c):o===l&&(s=Math.min(s,c)),r>o?null:r===o&&i>s?null:new e(r,i,o,s)},e.prototype.equalsRange=function(t){return e.equalsRange(this,t)},e.equalsRange=function(e,t){return!!e&&!!t&&e.startLineNumber===t.startLineNumber&&e.startColumn===t.startColumn&&e.endLineNumber===t.endLineNumber&&e.endColumn===t.endColumn},e.prototype.getEndPosition=function(){return new ue(this.endLineNumber,this.endColumn)},e.prototype.getStartPosition=function(){return new ue(this.startLineNumber,this.startColumn)},e.prototype.toString=function(){return\"[\"+this.startLineNumber+\",\"+this.startColumn+\" -> \"+this.endLineNumber+\",\"+this.endColumn+\"]\"},e.prototype.setEndPosition=function(t,n){return new e(this.startLineNumber,this.startColumn,t,n)},e.prototype.setStartPosition=function(t,n){return new e(t,n,this.endLineNumber,this.endColumn)},e.prototype.collapseToStart=function(){return e.collapseToStart(this)},e.collapseToStart=function(t){return new e(t.startLineNumber,t.startColumn,t.startLineNumber,t.startColumn)},e.fromPositions=function(t,n){return void 0===n&&(n=t),new e(t.lineNumber,t.column,n.lineNumber,n.column)},e.lift=function(t){return t?new e(t.startLineNumber,t.startColumn,t.endLineNumber,t.endColumn):null},e.isIRange=function(e){return e&&\"number\"==typeof e.startLineNumber&&\"number\"==typeof e.startColumn&&\"number\"==typeof e.endLineNumber&&\"number\"==typeof e.endColumn},e.areIntersectingOrTouching=function(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<t.startColumn)&&!(t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<e.startColumn)},e.areIntersecting=function(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<=t.startColumn)&&!(t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<=e.startColumn)},e.compareRangesUsingStarts=function(e,t){if(e&&t){var n=0|e.startLineNumber,r=0|t.startLineNumber;if(n===r){var i=0|e.startColumn,o=0|t.startColumn;if(i===o){var s=0|e.endLineNumber,u=0|t.endLineNumber;return s===u?(0|e.endColumn)-(0|t.endColumn):s-u}return i-o}return n-r}return(e?1:0)-(t?1:0)},e.compareRangesUsingEnds=function(e,t){return e.endLineNumber===t.endLineNumber?e.endColumn===t.endColumn?e.startLineNumber===t.startLineNumber?e.startColumn-t.startColumn:e.startLineNumber-t.startLineNumber:e.endColumn-t.endColumn:e.endLineNumber-t.endLineNumber},e.spansMultipleLines=function(e){return e.endLineNumber>e.startLineNumber},e}();var le=5e3,ce=3;function he(e,t,n,r){return new H(e,t,n).ComputeDiff(r)}var fe=function(){function e(t){for(var n=[],r=[],i=0,o=t.length;i<o;i++)n[i]=e._getFirstNonBlankColumn(t[i],1),r[i]=e._getLastNonBlankColumn(t[i],1);this._lines=t,this._startColumns=n,this._endColumns=r}return e.prototype.getLength=function(){return this._lines.length},e.prototype.getElementAtIndex=function(e){return this._lines[e].substring(this._startColumns[e]-1,this._endColumns[e]-1)},e.prototype.getStartLineNumber=function(e){return e+1},e.prototype.getEndLineNumber=function(e){return e+1},e._getFirstNonBlankColumn=function(e,t){var n=function(e){for(var t=0,n=e.length;t<n;t++){var r=e.charCodeAt(t);if(32!==r&&9!==r)return t}return-1}(e);return-1===n?t:n+1},e._getLastNonBlankColumn=function(e,t){var n=function(e,t){void 0===t&&(t=e.length-1);for(var n=t;n>=0;n--){var r=e.charCodeAt(n);if(32!==r&&9!==r)return n}return-1}(e);return-1===n?t:n+2},e.prototype.getCharSequence=function(e,t,n){for(var r=[],i=[],o=[],s=0,u=t;u<=n;u++)for(var a=this._lines[u],l=e?this._startColumns[u]:1,c=e?this._endColumns[u]:a.length+1,h=l;h<c;h++)r[s]=a.charCodeAt(h-1),i[s]=u+1,o[s]=h,s++;return new de(r,i,o)},e}(),de=function(){function e(e,t,n){this._charCodes=e,this._lineNumbers=t,this._columns=n}return e.prototype.getLength=function(){return this._charCodes.length},e.prototype.getElementAtIndex=function(e){return this._charCodes[e]},e.prototype.getStartLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getStartColumn=function(e){return this._columns[e]},e.prototype.getEndLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getEndColumn=function(e){return this._columns[e]+1},e}(),me=function(){function e(e,t,n,r,i,o,s,u){this.originalStartLineNumber=e,this.originalStartColumn=t,this.originalEndLineNumber=n,this.originalEndColumn=r,this.modifiedStartLineNumber=i,this.modifiedStartColumn=o,this.modifiedEndLineNumber=s,this.modifiedEndColumn=u}return e.createFromDiffChange=function(t,n,r){var i,o,s,u,a,l,c,h;return 0===t.originalLength?(i=0,o=0,s=0,u=0):(i=n.getStartLineNumber(t.originalStart),o=n.getStartColumn(t.originalStart),s=n.getEndLineNumber(t.originalStart+t.originalLength-1),u=n.getEndColumn(t.originalStart+t.originalLength-1)),0===t.modifiedLength?(a=0,l=0,c=0,h=0):(a=r.getStartLineNumber(t.modifiedStart),l=r.getStartColumn(t.modifiedStart),c=r.getEndLineNumber(t.modifiedStart+t.modifiedLength-1),h=r.getEndColumn(t.modifiedStart+t.modifiedLength-1)),new e(i,o,s,u,a,l,c,h)},e}();var pe=function(){function e(e,t,n,r,i){this.originalStartLineNumber=e,this.originalEndLineNumber=t,this.modifiedStartLineNumber=n,this.modifiedEndLineNumber=r,this.charChanges=i}return e.createFromDiffResult=function(t,n,r,i,o,s,u){var a,l,c,h,f=void 0;if(0===n.originalLength?(a=r.getStartLineNumber(n.originalStart)-1,l=0):(a=r.getStartLineNumber(n.originalStart),l=r.getEndLineNumber(n.originalStart+n.originalLength-1)),0===n.modifiedLength?(c=i.getStartLineNumber(n.modifiedStart)-1,h=0):(c=i.getStartLineNumber(n.modifiedStart),h=i.getEndLineNumber(n.modifiedStart+n.modifiedLength-1)),s&&0!==n.originalLength&&0!==n.modifiedLength&&o()){var d=r.getCharSequence(t,n.originalStart,n.originalStart+n.originalLength-1),m=i.getCharSequence(t,n.modifiedStart,n.modifiedStart+n.modifiedLength-1),p=he(d,m,o,!0);u&&(p=function(e){if(e.length<=1)return e;for(var t=[e[0]],n=t[0],r=1,i=e.length;r<i;r++){var o=e[r],s=o.originalStart-(n.originalStart+n.originalLength),u=o.modifiedStart-(n.modifiedStart+n.modifiedLength);Math.min(s,u)<ce?(n.originalLength=o.originalStart+o.originalLength-n.originalStart,n.modifiedLength=o.modifiedStart+o.modifiedLength-n.modifiedStart):(t.push(o),n=o)}return t}(p)),f=[];for(var _=0,g=p.length;_<g;_++)f.push(me.createFromDiffChange(p[_],d,m))}return new e(a,l,c,h,f)},e}(),_e=function(){function e(e,t,n){this.shouldComputeCharChanges=n.shouldComputeCharChanges,this.shouldPostProcessCharChanges=n.shouldPostProcessCharChanges,this.shouldIgnoreTrimWhitespace=n.shouldIgnoreTrimWhitespace,this.shouldMakePrettyDiff=n.shouldMakePrettyDiff,this.maximumRunTimeMs=le,this.originalLines=e,this.modifiedLines=t,this.original=new fe(e),this.modified=new fe(t)}return e.prototype.computeDiff=function(){if(1===this.original.getLength()&&0===this.original.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:1,modifiedStartLineNumber:1,modifiedEndLineNumber:this.modified.getLength(),charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];if(1===this.modified.getLength()&&0===this.modified.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:this.original.getLength(),modifiedStartLineNumber:1,modifiedEndLineNumber:1,charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];this.computationStartTime=(new Date).getTime();var e=he(this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldMakePrettyDiff);if(this.shouldIgnoreTrimWhitespace){for(var t=[],n=0,r=e.length;n<r;n++)t.push(pe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,e[n],this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges));return t}for(var i=[],o=0,s=0,u=(n=-1,e.length);n<u;n++){for(var a=n+1<u?e[n+1]:null,l=a?a.originalStart:this.originalLines.length,c=a?a.modifiedStart:this.modifiedLines.length;o<l&&s<c;){var h=this.originalLines[o],f=this.modifiedLines[s];if(h!==f){for(var d=fe._getFirstNonBlankColumn(h,1),m=fe._getFirstNonBlankColumn(f,1);d>1&&m>1;){if(h.charCodeAt(d-2)!==f.charCodeAt(m-2))break;d--,m--}(d>1||m>1)&&this._pushTrimWhitespaceCharChange(i,o+1,1,d,s+1,1,m);for(var p=fe._getLastNonBlankColumn(h,1),_=fe._getLastNonBlankColumn(f,1),g=h.length+1,v=f.length+1;p<g&&_<v;){if(h.charCodeAt(p-1)!==h.charCodeAt(_-1))break;p++,_++}(p<g||_<v)&&this._pushTrimWhitespaceCharChange(i,o+1,p,g,s+1,_,v)}o++,s++}a&&(i.push(pe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,a,this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges)),o+=a.originalLength,s+=a.modifiedLength)}return i},e.prototype._pushTrimWhitespaceCharChange=function(e,t,n,r,i,o,s){if(!this._mergeTrimWhitespaceCharChange(e,t,n,r,i,o,s)){var u=void 0;this.shouldComputeCharChanges&&(u=[new me(t,n,t,r,i,o,i,s)]),e.push(new pe(t,t,i,i,u))}},e.prototype._mergeTrimWhitespaceCharChange=function(e,t,n,r,i,o,s){var u=e.length;if(0===u)return!1;var a=e[u-1];return 0!==a.originalEndLineNumber&&0!==a.modifiedEndLineNumber&&(a.originalEndLineNumber+1===t&&a.modifiedEndLineNumber+1===i&&(a.originalEndLineNumber=t,a.modifiedEndLineNumber=i,this.shouldComputeCharChanges&&a.charChanges.push(new me(t,n,t,r,i,o,i,s)),!0))},e.prototype._continueProcessingPredicate=function(){return 0===this.maximumRunTimeMs||(new Date).getTime()-this.computationStartTime<this.maximumRunTimeMs},e}(),ge=function(){function e(e,t,n){for(var r=new Uint8Array(e*t),i=0,o=e*t;i<o;i++)r[i]=n;this._data=r,this.rows=e,this.cols=t}return e.prototype.get=function(e,t){return this._data[e*this.cols+t]},e.prototype.set=function(e,t,n){this._data[e*this.cols+t]=n},e}();function ve(e){return e<0?0:e>255?255:0|e}function ye(e){return e<0?0:e>4294967295?4294967295:0|e}var Ce=function(){return function(e,t){this.index=e,this.remainder=t}}(),be=function(){function e(e){this.values=e,this.prefixSum=new Uint32Array(e.length),this.prefixSumValidIndex=new Int32Array(1),this.prefixSumValidIndex[0]=-1}return e.prototype.getCount=function(){return this.values.length},e.prototype.insertValues=function(e,t){e=ye(e);var n=this.values,r=this.prefixSum,i=t.length;return 0!==i&&(this.values=new Uint32Array(n.length+i),this.values.set(n.subarray(0,e),0),this.values.set(n.subarray(e),e+i),this.values.set(t,e),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSum=new Uint32Array(this.values.length),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.changeValue=function(e,t){return e=ye(e),t=ye(t),this.values[e]!==t&&(this.values[e]=t,e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),!0)},e.prototype.removeValues=function(e,t){e=ye(e),t=ye(t);var n=this.values,r=this.prefixSum;if(e>=n.length)return!1;var i=n.length-e;return t>=i&&(t=i),0!==t&&(this.values=new Uint32Array(n.length-t),this.values.set(n.subarray(0,e),0),this.values.set(n.subarray(e+t),e),this.prefixSum=new Uint32Array(this.values.length),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.getTotalValue=function(){return 0===this.values.length?0:this._getAccumulatedValue(this.values.length-1)},e.prototype.getAccumulatedValue=function(e){return e<0?0:(e=ye(e),this._getAccumulatedValue(e))},e.prototype._getAccumulatedValue=function(e){if(e<=this.prefixSumValidIndex[0])return this.prefixSum[e];var t=this.prefixSumValidIndex[0]+1;0===t&&(this.prefixSum[0]=this.values[0],t++),e>=this.values.length&&(e=this.values.length-1);for(var n=t;n<=e;n++)this.prefixSum[n]=this.prefixSum[n-1]+this.values[n];return this.prefixSumValidIndex[0]=Math.max(this.prefixSumValidIndex[0],e),this.prefixSum[e]},e.prototype.getIndexOf=function(e){e=Math.floor(e),this.getTotalValue();for(var t=0,n=this.values.length-1,r=0,i=0,o=0;t<=n;)if(r=t+(n-t)/2|0,e<(o=(i=this.prefixSum[r])-this.values[r]))n=r-1;else{if(!(e>=i))break;t=r+1}return new Ce(r,e-o)},e}(),Se=function(){function e(e,t,n,r){this._uri=e,this._lines=t,this._eol=n,this._versionId=r,this._lineStarts=null}return e.prototype.dispose=function(){this._lines.length=0},e.prototype.getText=function(){return this._lines.join(this._eol)},e.prototype.onEvents=function(e){e.eol&&e.eol!==this._eol&&(this._eol=e.eol,this._lineStarts=null);for(var t=e.changes,n=0,r=t.length;n<r;n++){var i=t[n];this._acceptDeleteRange(i.range),this._acceptInsertText(new ue(i.range.startLineNumber,i.range.startColumn),i.text)}this._versionId=e.versionId},e.prototype._ensureLineStarts=function(){if(!this._lineStarts){for(var e=this._eol.length,t=this._lines.length,n=new Uint32Array(t),r=0;r<t;r++)n[r]=this._lines[r].length+e;this._lineStarts=new be(n)}},e.prototype._setLineText=function(e,t){this._lines[e]=t,this._lineStarts&&this._lineStarts.changeValue(e,this._lines[e].length+this._eol.length)},e.prototype._acceptDeleteRange=function(e){if(e.startLineNumber!==e.endLineNumber)this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.endLineNumber-1].substring(e.endColumn-1)),this._lines.splice(e.startLineNumber,e.endLineNumber-e.startLineNumber),this._lineStarts&&this._lineStarts.removeValues(e.startLineNumber,e.endLineNumber-e.startLineNumber);else{if(e.startColumn===e.endColumn)return;this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.startLineNumber-1].substring(e.endColumn-1))}},e.prototype._acceptInsertText=function(e,t){if(0!==t.length){var n=t.split(/\\r\\n|\\r|\\n/);if(1!==n.length){n[n.length-1]+=this._lines[e.lineNumber-1].substring(e.column-1),this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+n[0]);for(var r=new Uint32Array(n.length-1),i=1;i<n.length;i++)this._lines.splice(e.lineNumber+i-1,0,n[i]),r[i-1]=n[i].length+this._eol.length;this._lineStarts&&this._lineStarts.insertValues(e.lineNumber,r)}else this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+n[0]+this._lines[e.lineNumber-1].substring(e.column-1))}},e}(),Ee=\"`~!@#$%^&*()-=+[{]}\\\\|;:'\\\",.<>/?\";var Ne=function(e){void 0===e&&(e=\"\");for(var t=\"(-?\\\\d*\\\\.\\\\d\\\\w*)|([^\",n=0;n<Ee.length;n++)e.indexOf(Ee[n])>=0||(t+=\"\\\\\"+Ee[n]);return t+=\"\\\\s]+)\",new RegExp(t,\"g\")}();var Le=function(){function e(t){var n=ve(t);this._defaultValue=n,this._asciiMap=e._createAsciiMap(n),this._map=new Map}return e._createAsciiMap=function(e){for(var t=new Uint8Array(256),n=0;n<256;n++)t[n]=e;return t},e.prototype.set=function(e,t){var n=ve(t);e>=0&&e<256?this._asciiMap[e]=n:this._map.set(e,n)},e.prototype.get=function(e){return e>=0&&e<256?this._asciiMap[e]:this._map.get(e)||this._defaultValue},e}(),Ae=function(){function e(e){for(var t=0,n=0,r=0,i=e.length;r<i;r++){var o=e[r],s=o[0];(l=o[1])>t&&(t=l),s>n&&(n=s),(c=o[2])>n&&(n=c)}var u=new ge(++n,++t,0);for(r=0,i=e.length;r<i;r++){var a=e[r],l=(s=a[0],a[1]),c=a[2];u.set(s,l,c)}this._states=u,this._maxCharCode=t}return e.prototype.nextState=function(e,t){return t<0||t>=this._maxCharCode?0:this._states.get(e,t)},e}(),we=null;var Pe=null;var Oe=function(){function e(){}return e._createLink=function(e,t,n,r,i){var o=i-1;do{var s=t.charCodeAt(o);if(2!==e.get(s))break;o--}while(o>r);if(r>0){var u=t.charCodeAt(r-1),a=t.charCodeAt(o);(40===u&&41===a||91===u&&93===a||123===u&&125===a)&&o--}return{range:{startLineNumber:n,startColumn:r+1,endLineNumber:n,endColumn:o+2},url:t.substring(r,o+1)}},e.computeLinks=function(t){for(var n=(null===we&&(we=new Ae([[1,104,2],[1,72,2],[1,102,6],[1,70,6],[2,116,3],[2,84,3],[3,116,4],[3,84,4],[4,112,5],[4,80,5],[5,115,9],[5,83,9],[5,58,10],[6,105,7],[6,73,7],[7,108,8],[7,76,8],[8,101,9],[8,69,9],[9,58,10],[10,47,11],[11,47,12]])),we),r=function(){if(null===Pe){Pe=new Le(0);for(var e=0;e<\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".length;e++)Pe.set(\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".charCodeAt(e),1);for(e=0;e<\".,;\".length;e++)Pe.set(\".,;\".charCodeAt(e),2)}return Pe}(),i=[],o=1,s=t.getLineCount();o<=s;o++){for(var u=t.getLineContent(o),a=u.length,l=0,c=0,h=0,f=1,d=!1,m=!1,p=!1;l<a;){var _=!1,g=u.charCodeAt(l);if(13===f){var v=void 0;switch(g){case 40:d=!0,v=0;break;case 41:v=d?0:1;break;case 91:m=!0,v=0;break;case 93:v=m?0:1;break;case 123:p=!0,v=0;break;case 125:v=p?0:1;break;case 39:v=34===h||96===h?0:1;break;case 34:v=39===h||96===h?0:1;break;case 96:v=39===h||34===h?0:1;break;default:v=r.get(g)}1===v&&(i.push(e._createLink(r,u,o,c,l)),_=!0)}else if(12===f){1===(v=r.get(g))?_=!0:f=13}else 0===(f=n.nextState(f,g))&&(_=!0);_&&(f=1,d=!1,m=!1,p=!1,c=l+1,h=g),l++}13===f&&i.push(e._createLink(r,u,o,c,a))}return i},e}();var Me,xe=function(){function e(){this._defaultValueSet=[[\"true\",\"false\"],[\"True\",\"False\"],[\"Private\",\"Public\",\"Friend\",\"ReadOnly\",\"Partial\",\"Protected\",\"WriteOnly\"],[\"public\",\"protected\",\"private\"]]}return e.prototype.navigateValueSet=function(e,t,n,r,i){var o;if(e&&t&&(o=this.doNavigateValueSet(t,i)))return{range:e,value:o};if(n&&r&&(o=this.doNavigateValueSet(r,i)))return{range:n,value:o};return null},e.prototype.doNavigateValueSet=function(e,t){var n=this.numberReplace(e,t);return null!==n?n:this.textReplace(e,t)},e.prototype.numberReplace=function(e,t){var n=Math.pow(10,e.length-(e.lastIndexOf(\".\")+1)),r=Number(e),i=parseFloat(e);return isNaN(r)||isNaN(i)||r!==i?null:0!==r||t?(r=Math.floor(r*n),r+=t?n:-n,String(r/n)):null},e.prototype.textReplace=function(e,t){return this.valueSetsReplace(this._defaultValueSet,e,t)},e.prototype.valueSetsReplace=function(e,t,n){for(var r=null,i=0,o=e.length;null===r&&i<o;i++)r=this.valueSetReplace(e[i],t,n);return r},e.prototype.valueSetReplace=function(e,t,n){var r=e.indexOf(t);return r>=0?((r+=n?1:-1)<0?r=e.length-1:r%=e.length,e[r]):null},e.INSTANCE=new e,e}(),Te=function(){function e(){this._keyCodeToStr=[],this._strToKeyCode=Object.create(null)}return e.prototype.define=function(e,t){this._keyCodeToStr[e]=t,this._strToKeyCode[t.toLowerCase()]=e},e.prototype.keyCodeToStr=function(e){return this._keyCodeToStr[e]},e.prototype.strToKeyCode=function(e){return this._strToKeyCode[e.toLowerCase()]||0},e}(),Ie=new Te,ke=new Te,Re=new Te;!function(){function e(e,t,n,r){void 0===n&&(n=t),void 0===r&&(r=n),Ie.define(e,t),ke.define(e,n),Re.define(e,r)}e(0,\"unknown\"),e(1,\"Backspace\"),e(2,\"Tab\"),e(3,\"Enter\"),e(4,\"Shift\"),e(5,\"Ctrl\"),e(6,\"Alt\"),e(7,\"PauseBreak\"),e(8,\"CapsLock\"),e(9,\"Escape\"),e(10,\"Space\"),e(11,\"PageUp\"),e(12,\"PageDown\"),e(13,\"End\"),e(14,\"Home\"),e(15,\"LeftArrow\",\"Left\"),e(16,\"UpArrow\",\"Up\"),e(17,\"RightArrow\",\"Right\"),e(18,\"DownArrow\",\"Down\"),e(19,\"Insert\"),e(20,\"Delete\"),e(21,\"0\"),e(22,\"1\"),e(23,\"2\"),e(24,\"3\"),e(25,\"4\"),e(26,\"5\"),e(27,\"6\"),e(28,\"7\"),e(29,\"8\"),e(30,\"9\"),e(31,\"A\"),e(32,\"B\"),e(33,\"C\"),e(34,\"D\"),e(35,\"E\"),e(36,\"F\"),e(37,\"G\"),e(38,\"H\"),e(39,\"I\"),e(40,\"J\"),e(41,\"K\"),e(42,\"L\"),e(43,\"M\"),e(44,\"N\"),e(45,\"O\"),e(46,\"P\"),e(47,\"Q\"),e(48,\"R\"),e(49,\"S\"),e(50,\"T\"),e(51,\"U\"),e(52,\"V\"),e(53,\"W\"),e(54,\"X\"),e(55,\"Y\"),e(56,\"Z\"),e(57,\"Meta\"),e(58,\"ContextMenu\"),e(59,\"F1\"),e(60,\"F2\"),e(61,\"F3\"),e(62,\"F4\"),e(63,\"F5\"),e(64,\"F6\"),e(65,\"F7\"),e(66,\"F8\"),e(67,\"F9\"),e(68,\"F10\"),e(69,\"F11\"),e(70,\"F12\"),e(71,\"F13\"),e(72,\"F14\"),e(73,\"F15\"),e(74,\"F16\"),e(75,\"F17\"),e(76,\"F18\"),e(77,\"F19\"),e(78,\"NumLock\"),e(79,\"ScrollLock\"),e(80,\";\",\";\",\"OEM_1\"),e(81,\"=\",\"=\",\"OEM_PLUS\"),e(82,\",\",\",\",\"OEM_COMMA\"),e(83,\"-\",\"-\",\"OEM_MINUS\"),e(84,\".\",\".\",\"OEM_PERIOD\"),e(85,\"/\",\"/\",\"OEM_2\"),e(86,\"`\",\"`\",\"OEM_3\"),e(110,\"ABNT_C1\"),e(111,\"ABNT_C2\"),e(87,\"[\",\"[\",\"OEM_4\"),e(88,\"\\\\\",\"\\\\\",\"OEM_5\"),e(89,\"]\",\"]\",\"OEM_6\"),e(90,\"'\",\"'\",\"OEM_7\"),e(91,\"OEM_8\"),e(92,\"OEM_102\"),e(93,\"NumPad0\"),e(94,\"NumPad1\"),e(95,\"NumPad2\"),e(96,\"NumPad3\"),e(97,\"NumPad4\"),e(98,\"NumPad5\"),e(99,\"NumPad6\"),e(100,\"NumPad7\"),e(101,\"NumPad8\"),e(102,\"NumPad9\"),e(103,\"NumPad_Multiply\"),e(104,\"NumPad_Add\"),e(105,\"NumPad_Separator\"),e(106,\"NumPad_Subtract\"),e(107,\"NumPad_Decimal\"),e(108,\"NumPad_Divide\")}(),function(e){e.toString=function(e){return Ie.keyCodeToStr(e)},e.fromString=function(e){return Ie.strToKeyCode(e)},e.toUserSettingsUS=function(e){return ke.keyCodeToStr(e)},e.toUserSettingsGeneral=function(e){return Re.keyCodeToStr(e)},e.fromUserSettings=function(e){return ke.strToKeyCode(e)||Re.strToKeyCode(e)}}(Me||(Me={}));var Ue,De,Fe,We,Ve,Ke,qe,Be,je,Ye,He,Je,Ge,ze,Qe,Xe,$e,Ze,et,tt,nt,rt,it,ot,st,ut,at,lt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ct=function(e){function t(t,n,r,i){var o=e.call(this,t,n,r,i)||this;return o.selectionStartLineNumber=t,o.selectionStartColumn=n,o.positionLineNumber=r,o.positionColumn=i,o}return lt(t,e),t.prototype.clone=function(){return new t(this.selectionStartLineNumber,this.selectionStartColumn,this.positionLineNumber,this.positionColumn)},t.prototype.toString=function(){return\"[\"+this.selectionStartLineNumber+\",\"+this.selectionStartColumn+\" -> \"+this.positionLineNumber+\",\"+this.positionColumn+\"]\"},t.prototype.equalsSelection=function(e){return t.selectionsEqual(this,e)},t.selectionsEqual=function(e,t){return e.selectionStartLineNumber===t.selectionStartLineNumber&&e.selectionStartColumn===t.selectionStartColumn&&e.positionLineNumber===t.positionLineNumber&&e.positionColumn===t.positionColumn},t.prototype.getDirection=function(){return this.selectionStartLineNumber===this.startLineNumber&&this.selectionStartColumn===this.startColumn?0:1},t.prototype.setEndPosition=function(e,n){return 0===this.getDirection()?new t(this.startLineNumber,this.startColumn,e,n):new t(e,n,this.startLineNumber,this.startColumn)},t.prototype.getPosition=function(){return new ue(this.positionLineNumber,this.positionColumn)},t.prototype.setStartPosition=function(e,n){return 0===this.getDirection()?new t(e,n,this.endLineNumber,this.endColumn):new t(this.endLineNumber,this.endColumn,e,n)},t.fromPositions=function(e,n){return void 0===n&&(n=e),new t(e.lineNumber,e.column,n.lineNumber,n.column)},t.liftSelection=function(e){return new t(e.selectionStartLineNumber,e.selectionStartColumn,e.positionLineNumber,e.positionColumn)},t.selectionsArrEqual=function(e,t){if(e&&!t||!e&&t)return!1;if(!e&&!t)return!0;if(e.length!==t.length)return!1;for(var n=0,r=e.length;n<r;n++)if(!this.selectionsEqual(e[n],t[n]))return!1;return!0},t.isISelection=function(e){return e&&\"number\"==typeof e.selectionStartLineNumber&&\"number\"==typeof e.selectionStartColumn&&\"number\"==typeof e.positionLineNumber&&\"number\"==typeof e.positionColumn},t.createWithDirection=function(e,n,r,i,o){return 0===o?new t(e,n,r,i):new t(r,i,e,n)},t}(ae),ht=function(){function e(e,t,n){this.offset=0|e,this.type=t,this.language=n}return e.prototype.toString=function(){return\"(\"+this.offset+\", \"+this.type+\")\"},e}();!function(e){e[e.Unnecessary=1]=\"Unnecessary\"}(Ue||(Ue={})),function(e){e[e.Hint=1]=\"Hint\",e[e.Info=2]=\"Info\",e[e.Warning=4]=\"Warning\",e[e.Error=8]=\"Error\"}(De||(De={})),function(e){e[e.Unknown=0]=\"Unknown\",e[e.Backspace=1]=\"Backspace\",e[e.Tab=2]=\"Tab\",e[e.Enter=3]=\"Enter\",e[e.Shift=4]=\"Shift\",e[e.Ctrl=5]=\"Ctrl\",e[e.Alt=6]=\"Alt\",e[e.PauseBreak=7]=\"PauseBreak\",e[e.CapsLock=8]=\"CapsLock\",e[e.Escape=9]=\"Escape\",e[e.Space=10]=\"Space\",e[e.PageUp=11]=\"PageUp\",e[e.PageDown=12]=\"PageDown\",e[e.End=13]=\"End\",e[e.Home=14]=\"Home\",e[e.LeftArrow=15]=\"LeftArrow\",e[e.UpArrow=16]=\"UpArrow\",e[e.RightArrow=17]=\"RightArrow\",e[e.DownArrow=18]=\"DownArrow\",e[e.Insert=19]=\"Insert\",e[e.Delete=20]=\"Delete\",e[e.KEY_0=21]=\"KEY_0\",e[e.KEY_1=22]=\"KEY_1\",e[e.KEY_2=23]=\"KEY_2\",e[e.KEY_3=24]=\"KEY_3\",e[e.KEY_4=25]=\"KEY_4\",e[e.KEY_5=26]=\"KEY_5\",e[e.KEY_6=27]=\"KEY_6\",e[e.KEY_7=28]=\"KEY_7\",e[e.KEY_8=29]=\"KEY_8\",e[e.KEY_9=30]=\"KEY_9\",e[e.KEY_A=31]=\"KEY_A\",e[e.KEY_B=32]=\"KEY_B\",e[e.KEY_C=33]=\"KEY_C\",e[e.KEY_D=34]=\"KEY_D\",e[e.KEY_E=35]=\"KEY_E\",e[e.KEY_F=36]=\"KEY_F\",e[e.KEY_G=37]=\"KEY_G\",e[e.KEY_H=38]=\"KEY_H\",e[e.KEY_I=39]=\"KEY_I\",e[e.KEY_J=40]=\"KEY_J\",e[e.KEY_K=41]=\"KEY_K\",e[e.KEY_L=42]=\"KEY_L\",e[e.KEY_M=43]=\"KEY_M\",e[e.KEY_N=44]=\"KEY_N\",e[e.KEY_O=45]=\"KEY_O\",e[e.KEY_P=46]=\"KEY_P\",e[e.KEY_Q=47]=\"KEY_Q\",e[e.KEY_R=48]=\"KEY_R\",e[e.KEY_S=49]=\"KEY_S\",e[e.KEY_T=50]=\"KEY_T\",e[e.KEY_U=51]=\"KEY_U\",e[e.KEY_V=52]=\"KEY_V\",e[e.KEY_W=53]=\"KEY_W\",e[e.KEY_X=54]=\"KEY_X\",e[e.KEY_Y=55]=\"KEY_Y\",e[e.KEY_Z=56]=\"KEY_Z\",e[e.Meta=57]=\"Meta\",e[e.ContextMenu=58]=\"ContextMenu\",e[e.F1=59]=\"F1\",e[e.F2=60]=\"F2\",e[e.F3=61]=\"F3\",e[e.F4=62]=\"F4\",e[e.F5=63]=\"F5\",e[e.F6=64]=\"F6\",e[e.F7=65]=\"F7\",e[e.F8=66]=\"F8\",e[e.F9=67]=\"F9\",e[e.F10=68]=\"F10\",e[e.F11=69]=\"F11\",e[e.F12=70]=\"F12\",e[e.F13=71]=\"F13\",e[e.F14=72]=\"F14\",e[e.F15=73]=\"F15\",e[e.F16=74]=\"F16\",e[e.F17=75]=\"F17\",e[e.F18=76]=\"F18\",e[e.F19=77]=\"F19\",e[e.NumLock=78]=\"NumLock\",e[e.ScrollLock=79]=\"ScrollLock\",e[e.US_SEMICOLON=80]=\"US_SEMICOLON\",e[e.US_EQUAL=81]=\"US_EQUAL\",e[e.US_COMMA=82]=\"US_COMMA\",e[e.US_MINUS=83]=\"US_MINUS\",e[e.US_DOT=84]=\"US_DOT\",e[e.US_SLASH=85]=\"US_SLASH\",e[e.US_BACKTICK=86]=\"US_BACKTICK\",e[e.US_OPEN_SQUARE_BRACKET=87]=\"US_OPEN_SQUARE_BRACKET\",e[e.US_BACKSLASH=88]=\"US_BACKSLASH\",e[e.US_CLOSE_SQUARE_BRACKET=89]=\"US_CLOSE_SQUARE_BRACKET\",e[e.US_QUOTE=90]=\"US_QUOTE\",e[e.OEM_8=91]=\"OEM_8\",e[e.OEM_102=92]=\"OEM_102\",e[e.NUMPAD_0=93]=\"NUMPAD_0\",e[e.NUMPAD_1=94]=\"NUMPAD_1\",e[e.NUMPAD_2=95]=\"NUMPAD_2\",e[e.NUMPAD_3=96]=\"NUMPAD_3\",e[e.NUMPAD_4=97]=\"NUMPAD_4\",e[e.NUMPAD_5=98]=\"NUMPAD_5\",e[e.NUMPAD_6=99]=\"NUMPAD_6\",e[e.NUMPAD_7=100]=\"NUMPAD_7\",e[e.NUMPAD_8=101]=\"NUMPAD_8\",e[e.NUMPAD_9=102]=\"NUMPAD_9\",e[e.NUMPAD_MULTIPLY=103]=\"NUMPAD_MULTIPLY\",e[e.NUMPAD_ADD=104]=\"NUMPAD_ADD\",e[e.NUMPAD_SEPARATOR=105]=\"NUMPAD_SEPARATOR\",e[e.NUMPAD_SUBTRACT=106]=\"NUMPAD_SUBTRACT\",e[e.NUMPAD_DECIMAL=107]=\"NUMPAD_DECIMAL\",e[e.NUMPAD_DIVIDE=108]=\"NUMPAD_DIVIDE\",e[e.KEY_IN_COMPOSITION=109]=\"KEY_IN_COMPOSITION\",e[e.ABNT_C1=110]=\"ABNT_C1\",e[e.ABNT_C2=111]=\"ABNT_C2\",e[e.MAX_VALUE=112]=\"MAX_VALUE\"}(Fe||(Fe={})),function(e){e[e.LTR=0]=\"LTR\",e[e.RTL=1]=\"RTL\"}(We||(We={})),function(e){e[e.Auto=1]=\"Auto\",e[e.Hidden=2]=\"Hidden\",e[e.Visible=3]=\"Visible\"}(Ve||(Ve={})),function(e){e[e.Left=1]=\"Left\",e[e.Center=2]=\"Center\",e[e.Right=4]=\"Right\",e[e.Full=7]=\"Full\"}(Ke||(Ke={})),function(e){e[e.TextDefined=0]=\"TextDefined\",e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(qe||(qe={})),function(e){e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(Be||(Be={})),function(e){e[e.LF=0]=\"LF\",e[e.CRLF=1]=\"CRLF\"}(je||(je={})),function(e){e[e.AlwaysGrowsWhenTypingAtEdges=0]=\"AlwaysGrowsWhenTypingAtEdges\",e[e.NeverGrowsWhenTypingAtEdges=1]=\"NeverGrowsWhenTypingAtEdges\",e[e.GrowsOnlyWhenTypingBefore=2]=\"GrowsOnlyWhenTypingBefore\",e[e.GrowsOnlyWhenTypingAfter=3]=\"GrowsOnlyWhenTypingAfter\"}(Ye||(Ye={})),function(e){e[e.Smooth=0]=\"Smooth\",e[e.Immediate=1]=\"Immediate\"}(He||(He={})),function(e){e[e.NotSet=0]=\"NotSet\",e[e.ContentFlush=1]=\"ContentFlush\",e[e.RecoverFromMarkers=2]=\"RecoverFromMarkers\",e[e.Explicit=3]=\"Explicit\",e[e.Paste=4]=\"Paste\",e[e.Undo=5]=\"Undo\",e[e.Redo=6]=\"Redo\"}(Je||(Je={})),function(e){e[e.None=0]=\"None\",e[e.Small=1]=\"Small\",e[e.Large=2]=\"Large\",e[e.SmallBlocks=3]=\"SmallBlocks\",e[e.LargeBlocks=4]=\"LargeBlocks\"}(Ge||(Ge={})),function(e){e[e.None=0]=\"None\",e[e.Same=1]=\"Same\",e[e.Indent=2]=\"Indent\",e[e.DeepIndent=3]=\"DeepIndent\"}(ze||(ze={})),function(e){e[e.Hidden=0]=\"Hidden\",e[e.Blink=1]=\"Blink\",e[e.Smooth=2]=\"Smooth\",e[e.Phase=3]=\"Phase\",e[e.Expand=4]=\"Expand\",e[e.Solid=5]=\"Solid\"}(Qe||(Qe={})),function(e){e[e.Line=1]=\"Line\",e[e.Block=2]=\"Block\",e[e.Underline=3]=\"Underline\",e[e.LineThin=4]=\"LineThin\",e[e.BlockOutline=5]=\"BlockOutline\",e[e.UnderlineThin=6]=\"UnderlineThin\"}(Xe||(Xe={})),function(e){e[e.Off=0]=\"Off\",e[e.On=1]=\"On\",e[e.Relative=2]=\"Relative\",e[e.Interval=3]=\"Interval\",e[e.Custom=4]=\"Custom\"}($e||($e={})),function(e){e[e.EXACT=0]=\"EXACT\",e[e.ABOVE=1]=\"ABOVE\",e[e.BELOW=2]=\"BELOW\"}(Ze||(Ze={})),function(e){e[e.TOP_RIGHT_CORNER=0]=\"TOP_RIGHT_CORNER\",e[e.BOTTOM_RIGHT_CORNER=1]=\"BOTTOM_RIGHT_CORNER\",e[e.TOP_CENTER=2]=\"TOP_CENTER\"}(et||(et={})),function(e){e[e.UNKNOWN=0]=\"UNKNOWN\",e[e.TEXTAREA=1]=\"TEXTAREA\",e[e.GUTTER_GLYPH_MARGIN=2]=\"GUTTER_GLYPH_MARGIN\",e[e.GUTTER_LINE_NUMBERS=3]=\"GUTTER_LINE_NUMBERS\",e[e.GUTTER_LINE_DECORATIONS=4]=\"GUTTER_LINE_DECORATIONS\",e[e.GUTTER_VIEW_ZONE=5]=\"GUTTER_VIEW_ZONE\",e[e.CONTENT_TEXT=6]=\"CONTENT_TEXT\",e[e.CONTENT_EMPTY=7]=\"CONTENT_EMPTY\",e[e.CONTENT_VIEW_ZONE=8]=\"CONTENT_VIEW_ZONE\",e[e.CONTENT_WIDGET=9]=\"CONTENT_WIDGET\",e[e.OVERVIEW_RULER=10]=\"OVERVIEW_RULER\",e[e.SCROLLBAR=11]=\"SCROLLBAR\",e[e.OVERLAY_WIDGET=12]=\"OVERLAY_WIDGET\",e[e.OUTSIDE_EDITOR=13]=\"OUTSIDE_EDITOR\"}(tt||(tt={})),function(e){e[e.None=0]=\"None\",e[e.Indent=1]=\"Indent\",e[e.IndentOutdent=2]=\"IndentOutdent\",e[e.Outdent=3]=\"Outdent\"}(nt||(nt={})),function(e){e[e.Method=0]=\"Method\",e[e.Function=1]=\"Function\",e[e.Constructor=2]=\"Constructor\",e[e.Field=3]=\"Field\",e[e.Variable=4]=\"Variable\",e[e.Class=5]=\"Class\",e[e.Struct=6]=\"Struct\",e[e.Interface=7]=\"Interface\",e[e.Module=8]=\"Module\",e[e.Property=9]=\"Property\",e[e.Event=10]=\"Event\",e[e.Operator=11]=\"Operator\",e[e.Unit=12]=\"Unit\",e[e.Value=13]=\"Value\",e[e.Constant=14]=\"Constant\",e[e.Enum=15]=\"Enum\",e[e.EnumMember=16]=\"EnumMember\",e[e.Keyword=17]=\"Keyword\",e[e.Text=18]=\"Text\",e[e.Color=19]=\"Color\",e[e.File=20]=\"File\",e[e.Reference=21]=\"Reference\",e[e.Customcolor=22]=\"Customcolor\",e[e.Folder=23]=\"Folder\",e[e.TypeParameter=24]=\"TypeParameter\",e[e.Snippet=25]=\"Snippet\"}(rt||(rt={})),function(e){e[e.KeepWhitespace=1]=\"KeepWhitespace\",e[e.InsertAsSnippet=4]=\"InsertAsSnippet\"}(it||(it={})),function(e){e[e.Invoke=0]=\"Invoke\",e[e.TriggerCharacter=1]=\"TriggerCharacter\",e[e.TriggerForIncompleteCompletions=2]=\"TriggerForIncompleteCompletions\"}(ot||(ot={})),function(e){e[e.Invoke=1]=\"Invoke\",e[e.TriggerCharacter=2]=\"TriggerCharacter\",e[e.ContentChange=3]=\"ContentChange\"}(st||(st={})),function(e){e[e.Text=0]=\"Text\",e[e.Read=1]=\"Read\",e[e.Write=2]=\"Write\"}(ut||(ut={})),function(e){e[e.File=0]=\"File\",e[e.Module=1]=\"Module\",e[e.Namespace=2]=\"Namespace\",e[e.Package=3]=\"Package\",e[e.Class=4]=\"Class\",e[e.Method=5]=\"Method\",e[e.Property=6]=\"Property\",e[e.Field=7]=\"Field\",e[e.Constructor=8]=\"Constructor\",e[e.Enum=9]=\"Enum\",e[e.Interface=10]=\"Interface\",e[e.Function=11]=\"Function\",e[e.Variable=12]=\"Variable\",e[e.Constant=13]=\"Constant\",e[e.String=14]=\"String\",e[e.Number=15]=\"Number\",e[e.Boolean=16]=\"Boolean\",e[e.Array=17]=\"Array\",e[e.Object=18]=\"Object\",e[e.Key=19]=\"Key\",e[e.Null=20]=\"Null\",e[e.EnumMember=21]=\"EnumMember\",e[e.Struct=22]=\"Struct\",e[e.Event=23]=\"Event\",e[e.Operator=24]=\"Operator\",e[e.TypeParameter=25]=\"TypeParameter\"}(at||(at={}));var ft=function(){function e(){}return e.chord=function(e,t){return function(e,t){return(e|(65535&t)<<16>>>0)>>>0}(e,t)},e.CtrlCmd=2048,e.Shift=1024,e.Alt=512,e.WinCtrl=256,e}();var dt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),mt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return dt(t,e),Object.defineProperty(t.prototype,\"uri\",{get:function(){return this._uri},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,\"version\",{get:function(){return this._versionId},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,\"eol\",{get:function(){return this._eol},enumerable:!0,configurable:!0}),t.prototype.getValue=function(){return this.getText()},t.prototype.getLinesContent=function(){return this._lines.slice(0)},t.prototype.getLineCount=function(){return this._lines.length},t.prototype.getLineContent=function(e){return this._lines[e-1]},t.prototype.getWordAtPosition=function(e,t){var n=function(e,t,n,r){t.lastIndex=0;var i=t.exec(n);if(!i)return null;var o=i[0].indexOf(\" \")>=0?function(e,t,n,r){var i,o=e-1-r;for(t.lastIndex=0;i=t.exec(n);){var s=i.index||0;if(s>o)return null;if(t.lastIndex>=o)return{word:i[0],startColumn:r+1+s,endColumn:r+1+t.lastIndex}}return null}(e,t,n,r):function(e,t,n,r){var i,o=e-1-r,s=n.lastIndexOf(\" \",o-1)+1,u=n.indexOf(\" \",o);for(-1===u&&(u=n.length),t.lastIndex=s;i=t.exec(n);){var a=i.index||0;if(a<=o&&t.lastIndex>=o)return{word:i[0],startColumn:r+1+a,endColumn:r+1+t.lastIndex}}return null}(e,t,n,r);return t.lastIndex=0,o}(e.column,function(e){var t=Ne;if(e&&e instanceof RegExp)if(e.global)t=e;else{var n=\"g\";e.ignoreCase&&(n+=\"i\"),e.multiline&&(n+=\"m\"),t=new RegExp(e.source,n)}return t.lastIndex=0,t}(t),this._lines[e.lineNumber-1],0);return n?new ae(e.lineNumber,n.startColumn,e.lineNumber,n.endColumn):null},t.prototype.getWordUntilPosition=function(e,t){var n=this.getWordAtPosition(e,t);return n?{word:this._lines[e.lineNumber-1].substring(n.startColumn-1,e.column-1),startColumn:n.startColumn,endColumn:e.column}:{word:\"\",startColumn:e.column,endColumn:e.column}},t.prototype.createWordIterator=function(e){var t,n,r=this,i=0,o=0,s=[],u=function(){if(o<s.length){var a=n.substring(s[o].start,s[o].end);return o+=1,t?t.value=a:t={done:!1,value:a},t}return i>=r._lines.length?S:(n=r._lines[i],s=r._wordenize(n,e),o=0,i+=1,u())};return{next:u}},t.prototype.getLineWords=function(e,t){for(var n=this._lines[e-1],r=[],i=0,o=this._wordenize(n,t);i<o.length;i++){var s=o[i];r.push({word:n.substring(s.start,s.end),startColumn:s.start+1,endColumn:s.end+1})}return r},t.prototype._wordenize=function(e,t){var n,r=[];for(t.lastIndex=0;(n=t.exec(e))&&0!==n[0].length;)r.push({start:n.index,end:n.index+n[0].length});return r},t.prototype.getValueInRange=function(e){if((e=this._validateRange(e)).startLineNumber===e.endLineNumber)return this._lines[e.startLineNumber-1].substring(e.startColumn-1,e.endColumn-1);var t=this._eol,n=e.startLineNumber-1,r=e.endLineNumber-1,i=[];i.push(this._lines[n].substring(e.startColumn-1));for(var o=n+1;o<r;o++)i.push(this._lines[o]);return i.push(this._lines[r].substring(0,e.endColumn-1)),i.join(t)},t.prototype.offsetAt=function(e){return e=this._validatePosition(e),this._ensureLineStarts(),this._lineStarts.getAccumulatedValue(e.lineNumber-2)+(e.column-1)},t.prototype.positionAt=function(e){e=Math.floor(e),e=Math.max(0,e),this._ensureLineStarts();var t=this._lineStarts.getIndexOf(e),n=this._lines[t.index].length;return{lineNumber:1+t.index,column:1+Math.min(t.remainder,n)}},t.prototype._validateRange=function(e){var t=this._validatePosition({lineNumber:e.startLineNumber,column:e.startColumn}),n=this._validatePosition({lineNumber:e.endLineNumber,column:e.endColumn});return t.lineNumber!==e.startLineNumber||t.column!==e.startColumn||n.lineNumber!==e.endLineNumber||n.column!==e.endColumn?{startLineNumber:t.lineNumber,startColumn:t.column,endLineNumber:n.lineNumber,endColumn:n.column}:e},t.prototype._validatePosition=function(e){if(!ue.isIPosition(e))throw new Error(\"bad position\");var t=e.lineNumber,n=e.column,r=!1;if(t<1)t=1,n=1,r=!0;else if(t>this._lines.length)t=this._lines.length,n=this._lines[t-1].length+1,r=!0;else{var i=this._lines[t-1].length+1;n<1?(n=1,r=!0):n>i&&(n=i,r=!0)}return r?{lineNumber:t,column:n}:e},t}(Se),pt=function(e){function t(t){var n=e.call(this,t)||this;return n._models=Object.create(null),n}return dt(t,e),t.prototype.dispose=function(){this._models=Object.create(null)},t.prototype._getModel=function(e){return this._models[e]},t.prototype._getModels=function(){var e=this,t=[];return Object.keys(this._models).forEach(function(n){return t.push(e._models[n])}),t},t.prototype.acceptNewModel=function(e){this._models[e.url]=new mt(ee.parse(e.url),e.lines,e.EOL,e.versionId)},t.prototype.acceptModelChanged=function(e,t){this._models[e]&&this._models[e].onEvents(t)},t.prototype.acceptRemovedModel=function(e){this._models[e]&&delete this._models[e]},t}(function(){function e(e){this._foreignModuleFactory=e,this._foreignModule=null}return e.prototype.computeDiff=function(e,t,n){var r=this._getModel(e),i=this._getModel(t);if(!r||!i)return Promise.resolve(null);var o=r.getLinesContent(),s=i.getLinesContent(),u=new _e(o,s,{shouldComputeCharChanges:!0,shouldPostProcessCharChanges:!0,shouldIgnoreTrimWhitespace:n,shouldMakePrettyDiff:!0}).computeDiff(),a=!(u.length>0)&&this._modelsAreIdentical(r,i);return Promise.resolve({identical:a,changes:u})},e.prototype._modelsAreIdentical=function(e,t){var n=e.getLineCount();if(n!==t.getLineCount())return!1;for(var r=1;r<=n;r++){if(e.getLineContent(r)!==t.getLineContent(r))return!1}return!0},e.prototype.computeMoreMinimalEdits=function(t,n){var r=this._getModel(t);if(!r)return Promise.resolve(n);for(var i=[],o=void 0,s=0,u=n=F(n,function(e,t){return e.range&&t.range?ae.compareRangesUsingStarts(e.range,t.range):(e.range?0:1)-(t.range?0:1)});s<u.length;s++){var a=u[s],l=a.range,c=a.text,h=a.eol;if(\"number\"==typeof h&&(o=h),l){var f=r.getValueInRange(l);if(f!==(c=c.replace(/\\r\\n|\\n|\\r/g,r.eol)))if(Math.max(c.length,f.length)>e._diffLimit)i.push({range:l,text:c});else for(var d=K(f,c,!1),m=r.offsetAt(ae.lift(l).getStartPosition()),p=0,_=d;p<_.length;p++){var g=_[p],v=r.positionAt(m+g.originalStart),y=r.positionAt(m+g.originalStart+g.originalLength),C={text:c.substr(g.modifiedStart,g.modifiedLength),range:{startLineNumber:v.lineNumber,startColumn:v.column,endLineNumber:y.lineNumber,endColumn:y.column}};r.getValueInRange(C.range)!==C.text&&i.push(C)}}}return\"number\"==typeof o&&i.push({eol:o,text:void 0,range:void 0}),Promise.resolve(i)},e.prototype.computeLinks=function(e){var t=this._getModel(e);return t?Promise.resolve(function(e){return e&&\"function\"==typeof e.getLineCount&&\"function\"==typeof e.getLineContent?Oe.computeLinks(e):[]}(t)):Promise.resolve(null)},e.prototype.textualSuggest=function(t,n,r,i){var o=this._getModel(t);if(!o)return Promise.resolve(null);var s=[],u=new RegExp(r,i),a=o.getWordUntilPosition(n,u),l=Object.create(null);l[a.word]=!0;for(var c=o.createWordIterator(u),h=c.next();!h.done&&s.length<=e._suggestionsLimit;h=c.next()){var f=h.value;l[f]||(l[f]=!0,isNaN(Number(f))&&s.push({kind:18,label:f,insertText:f,range:{startLineNumber:n.lineNumber,startColumn:a.startColumn,endLineNumber:n.lineNumber,endColumn:a.endColumn}}))}return Promise.resolve({suggestions:s})},e.prototype.computeWordRanges=function(e,t,n,r){var i=this._getModel(e);if(!i)return Promise.resolve(Object.create(null));for(var o=new RegExp(n,r),s=Object.create(null),u=t.startLineNumber;u<t.endLineNumber;u++)for(var a=0,l=i.getLineWords(u,o);a<l.length;a++){var c=l[a];if(isNaN(Number(c.word))){var h=s[c.word];h||(h=[],s[c.word]=h),h.push({startLineNumber:u,startColumn:c.startColumn,endLineNumber:u,endColumn:c.endColumn})}}return Promise.resolve(s)},e.prototype.navigateValueSet=function(e,t,n,r,i){var o=this._getModel(e);if(!o)return Promise.resolve(null);var s=new RegExp(r,i);t.startColumn===t.endColumn&&(t={startLineNumber:t.startLineNumber,startColumn:t.startColumn,endLineNumber:t.endLineNumber,endColumn:t.endColumn+1});var u=o.getValueInRange(t),a=o.getWordAtPosition({lineNumber:t.startLineNumber,column:t.startColumn},s);if(!a)return Promise.resolve(null);var l=o.getValueInRange(a),c=xe.INSTANCE.navigateValueSet(t,u,a,l,n);return Promise.resolve(c)},e.prototype.loadForeignModule=function(e,t){var n=this,r={getMirrorModels:function(){return n._getModels()}};if(this._foreignModuleFactory){this._foreignModule=this._foreignModuleFactory(r,t);var i=[];for(var o in this._foreignModule)\"function\"==typeof this._foreignModule[o]&&i.push(o);return Promise.resolve(i)}return Promise.reject(new Error(\"Unexpected usage\"))},e.prototype.fmr=function(e,t){if(!this._foreignModule||\"function\"!=typeof this._foreignModule[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._foreignModule[e].apply(this._foreignModule,t))}catch(e){return Promise.reject(e)}},e._diffLimit=1e4,e._suggestionsLimit=1e4,e}());\"function\"==typeof importScripts&&(_.monaco={editor:void 0,languages:void 0,CancellationTokenSource:M,Emitter:A,KeyCode:Fe,KeyMod:ft,Position:ue,Range:ae,Selection:ct,SelectionDirection:We,MarkerSeverity:De,MarkerTag:Ue,Promise:r,Uri:ee,Token:ht});var _t=!1;function gt(e){if(!_t){_t=!0;var t=new pt(e),n=new D(function(e){self.postMessage(e)},t);self.onmessage=function(e){n.onmessage(e.data)}}}return self.onmessage=function(e){_t||gt(null)},e.initialize=gt,e}({});\n"], { type: "text/javascript" }));

var cssWorker = URL.createObjectURL(new Blob(["!function(){\"use strict\";var e;!function(){var t=Object.create(null);t[\"WinJS/Core/_WinJS\"]={};var n=function(e,n,r){var i={},o=!1,s=n.map(function(e){return\"exports\"===e?(o=!0,i):t[e]}),a=r.apply({},s);t[e]=o?i:a};n(\"WinJS/Core/_Global\",[],function(){return\"undefined\"!=typeof window?window:\"undefined\"!=typeof self?self:\"undefined\"!=typeof global?global:{}}),n(\"WinJS/Core/_BaseCoreUtils\",[\"WinJS/Core/_Global\"],function(e){var t=!!e.Windows;var n=null;return{hasWinRT:t,markSupportedForProcessing:function(e){return e.supportedForProcessing=!0,e},_setImmediate:function(t){null===n&&(n=e.setImmediate?e.setImmediate.bind(e):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):e.setTimeout.bind(e)),n(t)}}}),n(\"WinJS/Core/_WriteProfilerMark\",[\"WinJS/Core/_Global\"],function(e){return e.msWriteProfilerMark||function(){}}),n(\"WinJS/Core/_Base\",[\"WinJS/Core/_WinJS\",\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_WriteProfilerMark\"],function(e,t,n,r){function i(e,t,n){var r,i,o,s=Object.keys(t),a=Array.isArray(e);for(i=0,o=s.length;i<o;i++){var l=s[i],c=95!==l.charCodeAt(0),h=t[l];!h||\"object\"!=typeof h||void 0===h.value&&\"function\"!=typeof h.get&&\"function\"!=typeof h.set?c?a?e.forEach(function(e){e[l]=h}):e[l]=h:(r=r||{})[l]={value:h,enumerable:c,configurable:!0,writable:!0}:(void 0===h.enumerable&&(h.enumerable=c),n&&h.setName&&\"function\"==typeof h.setName&&h.setName(n+\".\"+l),(r=r||{})[l]=h)}r&&(a?e.forEach(function(e){Object.defineProperties(e,r)}):Object.defineProperties(e,r))}return function(){var n=e;function o(n,r){var i=n||{};if(r){var o=r.split(\".\");i===t&&\"WinJS\"===o[0]&&(i=e,o.splice(0,1));for(var s=0,a=o.length;s<a;s++){var l=o[s];i[l]||Object.defineProperty(i,l,{value:{},writable:!1,enumerable:!0,configurable:!0}),i=i[l]}}return i}function s(e,t,n){var r=o(e,t);return n&&i(r,n,t||\"<ANONYMOUS>\"),r}n.Namespace||(n.Namespace=Object.create(Object.prototype));var a={uninitialized:1,working:2,initialized:3};Object.defineProperties(n.Namespace,{defineWithParent:{value:s,writable:!0,enumerable:!0,configurable:!0},define:{value:function(e,n){return s(t,e,n)},writable:!0,enumerable:!0,configurable:!0},_lazy:{value:function(e){var t,n,i=a.uninitialized;return{setName:function(e){t=e},get:function(){switch(i){case a.initialized:return n;case a.uninitialized:i=a.working;try{r(\"WinJS.Namespace._lazy:\"+t+\",StartTM\"),n=e()}finally{r(\"WinJS.Namespace._lazy:\"+t+\",StopTM\"),i=a.uninitialized}return e=null,i=a.initialized,n;case a.working:throw\"Illegal: reentrancy on initialization\";default:throw\"Illegal\"}},set:function(e){switch(i){case a.working:throw\"Illegal: reentrancy on initialization\";default:i=a.initialized,n=e}},enumerable:!0,configurable:!0}},writable:!0,enumerable:!0,configurable:!0},_moduleDefine:{value:function(e,n,r){var s=[e],a=null;return n&&(a=o(t,n),s.push(a)),i(s,r,n||\"<ANONYMOUS>\"),a},writable:!0,enumerable:!0,configurable:!0}})}(),function(){function t(e,t,r){return e=e||function(){},n.markSupportedForProcessing(e),t&&i(e.prototype,t),r&&i(e,r),e}e.Namespace.define(\"WinJS.Class\",{define:t,derive:function(e,r,o,s){if(e){r=r||function(){};var a=e.prototype;return r.prototype=Object.create(a),n.markSupportedForProcessing(r),Object.defineProperty(r.prototype,\"constructor\",{value:r,writable:!0,configurable:!0,enumerable:!0}),o&&i(r.prototype,o),s&&i(r,s),r}return t(r,o,s)},mix:function(e){var t,n;for(e=e||function(){},t=1,n=arguments.length;t<n;t++)i(e.prototype,arguments[t]);return e}})}(),{Namespace:e.Namespace,Class:e.Class}}),n(\"WinJS/Core/_ErrorFromName\",[\"WinJS/Core/_Base\"],function(e){var t=e.Class.derive(Error,function(e,t){this.name=e,this.message=t||e},{},{supportedForProcessing:!1});return e.Namespace.define(\"WinJS\",{ErrorFromName:t}),t}),n(\"WinJS/Core/_Events\",[\"exports\",\"WinJS/Core/_Base\"],function(e,t){function n(e){var t=\"_on\"+e+\"state\";return{get:function(){var e=this[t];return e&&e.userHandler},set:function(n){var r=this[t];n?(r||(r={wrapper:function(e){return r.userHandler(e)},userHandler:n},Object.defineProperty(this,t,{value:r,enumerable:!1,writable:!0,configurable:!0}),this.addEventListener(e,r.wrapper,!1)),r.userHandler=n):r&&(this.removeEventListener(e,r.wrapper,!1),this[t]=null)},enumerable:!0}}var r=t.Class.define(function(e,t,n){this.detail=t,this.target=n,this.timeStamp=Date.now(),this.type=e},{bubbles:{value:!1,writable:!1},cancelable:{value:!1,writable:!1},currentTarget:{get:function(){return this.target}},defaultPrevented:{get:function(){return this._preventDefaultCalled}},trusted:{value:!1,writable:!1},eventPhase:{value:0,writable:!1},target:null,timeStamp:null,type:null,preventDefault:function(){this._preventDefaultCalled=!0},stopImmediatePropagation:function(){this._stopImmediatePropagationCalled=!0},stopPropagation:function(){}},{supportedForProcessing:!1}),i={_listeners:null,addEventListener:function(e,t,n){n=n||!1,this._listeners=this._listeners||{};for(var r=this._listeners[e]=this._listeners[e]||[],i=0,o=r.length;i<o;i++){var s=r[i];if(s.useCapture===n&&s.listener===t)return}r.push({listener:t,useCapture:n})},dispatchEvent:function(e,t){var n=this._listeners&&this._listeners[e];if(n){for(var i=new r(e,t,this),o=0,s=(n=n.slice(0,n.length)).length;o<s&&!i._stopImmediatePropagationCalled;o++)n[o].listener(i);return i.defaultPrevented||!1}return!1},removeEventListener:function(e,t,n){n=n||!1;var r=this._listeners&&this._listeners[e];if(r)for(var i=0,o=r.length;i<o;i++){var s=r[i];if(s.listener===t&&s.useCapture===n){r.splice(i,1),0===r.length&&delete this._listeners[e];break}}}};t.Namespace._moduleDefine(e,\"WinJS.Utilities\",{_createEventProperty:n,createEventProperties:function(){for(var e={},t=0,r=arguments.length;t<r;t++){var i=arguments[t];e[\"on\"+i]=n(i)}return e},eventMixin:i})}),n(\"WinJS/Core/_Trace\",[\"WinJS/Core/_Global\"],function(e){function t(e){return e}return{_traceAsyncOperationStarting:e.Debug&&e.Debug.msTraceAsyncOperationStarting&&e.Debug.msTraceAsyncOperationStarting.bind(e.Debug)||t,_traceAsyncOperationCompleted:e.Debug&&e.Debug.msTraceAsyncOperationCompleted&&e.Debug.msTraceAsyncOperationCompleted.bind(e.Debug)||t,_traceAsyncCallbackStarting:e.Debug&&e.Debug.msTraceAsyncCallbackStarting&&e.Debug.msTraceAsyncCallbackStarting.bind(e.Debug)||t,_traceAsyncCallbackCompleted:e.Debug&&e.Debug.msTraceAsyncCallbackCompleted&&e.Debug.msTraceAsyncCallbackCompleted.bind(e.Debug)||t}}),n(\"WinJS/Promise/_StateMachine\",[\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_Base\",\"WinJS/Core/_ErrorFromName\",\"WinJS/Core/_Events\",\"WinJS/Core/_Trace\"],function(e,t,n,r,i,o){e.Debug&&(e.Debug.setNonUserCodeExceptions=!0);var s=new(n.Class.mix(n.Class.define(null,{},{supportedForProcessing:!1}),i.eventMixin));s._listeners={};var a=\"error\",l=\"Canceled\",c=!1,h={promise:1,thenPromise:2,errorPromise:4,exceptionPromise:8,completePromise:16};h.all=h.promise|h.thenPromise|h.errorPromise|h.exceptionPromise|h.completePromise;var d,u,m,p,f,g,b,y,w,v,x=1;function k(){}d={name:\"created\",enter:function(e){e._setState(u)},cancel:k,done:k,then:k,_completed:k,_error:k,_notify:k,_progress:k,_setCompleteValue:k,_setErrorValue:k},u={name:\"working\",enter:k,cancel:function(e){e._setState(f)},done:A,then:j,_completed:S,_error:P,_notify:k,_progress:R,_setCompleteValue:M,_setErrorValue:U},m={name:\"waiting\",enter:function(e){var t=e._value;if(t instanceof B&&t._state!==v&&t._state!==y)z(t,{promise:e});else{var n=function(r){t._errorId?e._chainedError(r,t):(L(e,r,F,t,n),e._error(r))};n.handlesOnError=!0,t.then(e._completed.bind(e),n,e._progress.bind(e))}},cancel:function(e){e._setState(p)},done:A,then:j,_completed:S,_error:P,_notify:k,_progress:R,_setCompleteValue:M,_setErrorValue:U},p={name:\"waiting_canceled\",enter:function(e){e._setState(g);var t=e._value;t.cancel&&t.cancel()},cancel:k,done:A,then:j,_completed:S,_error:P,_notify:k,_progress:R,_setCompleteValue:M,_setErrorValue:U},f={name:\"canceled\",enter:function(e){e._setState(g),e._cancelAction()},cancel:k,done:A,then:j,_completed:S,_error:P,_notify:k,_progress:R,_setCompleteValue:M,_setErrorValue:U},g={name:\"canceling\",enter:function(e){var t=new Error(l);t.name=t.message,e._value=t,e._setState(w)},cancel:k,done:k,then:k,_completed:k,_error:k,_notify:k,_progress:k,_setCompleteValue:k,_setErrorValue:k},b={name:\"complete_notify\",enter:function(e){if(e.done=W.prototype.done,e.then=W.prototype.then,e._listeners)for(var t,n=[e];n.length;)(t=n.shift())._state._notify(t,n);e._setState(y)},cancel:k,done:null,then:null,_completed:k,_error:k,_notify:O,_progress:k,_setCompleteValue:k,_setErrorValue:k},y={name:\"success\",enter:function(e){e.done=W.prototype.done,e.then=W.prototype.then,e._cleanupAction()},cancel:k,done:null,then:null,_completed:k,_error:k,_notify:O,_progress:k,_setCompleteValue:k,_setErrorValue:k},w={name:\"error_notify\",enter:function(e){if(e.done=K.prototype.done,e.then=K.prototype.then,e._listeners)for(var t,n=[e];n.length;)(t=n.shift())._state._notify(t,n);e._setState(v)},cancel:k,done:null,then:null,_completed:k,_error:k,_notify:N,_progress:k,_setCompleteValue:k,_setErrorValue:k},v={name:\"error\",enter:function(e){e.done=K.prototype.done,e.then=K.prototype.then,e._cleanupAction()},cancel:k,done:null,then:null,_completed:k,_error:k,_notify:N,_progress:k,_setCompleteValue:k,_setErrorValue:k};var C=n.Class.define(null,{_listeners:null,_nextState:null,_state:null,_value:null,cancel:function(){this._state.cancel(this),this._run()},done:function(e,t,n){this._state.done(this,e,t,n)},then:function e(t,n,r){if(this.then===e)return this._state.then(this,t,n,r);this.then(t,n,r)},_chainedError:function(e,t){var n=this._state._error(this,e,E,t);return this._run(),n},_completed:function(e){var t=this._state._completed(this,e);return this._run(),t},_error:function(e){var t=this._state._error(this,e,T);return this._run(),t},_progress:function(e){this._state._progress(this,e)},_setState:function(e){this._nextState=e},_setCompleteValue:function(e){this._state._setCompleteValue(this,e),this._run()},_setChainedErrorValue:function(e,t){var n=this._state._setErrorValue(this,e,E,t);return this._run(),n},_setExceptionValue:function(e){var t=this._state._setErrorValue(this,e,I);return this._run(),t},_run:function(){for(;this._nextState;)this._state=this._nextState,this._nextState=null,this._state.enter(this)}},{supportedForProcessing:!1});function S(e,t){var n;n=t&&\"object\"==typeof t&&\"function\"==typeof t.then?m:b,e._value=t,e._setState(n)}function _(e,t,n,r,i,o){return{exception:e,error:t,promise:n,handler:o,id:r,parent:i}}function F(e,t,n,r){var i=n._isException;return _(i?t:null,i?null:t,e,n._errorId,n,r)}function E(e,t,n){var r=n._isException,i=n._errorId;return D(e,i,r),_(r?t:null,r?null:t,e,i,n)}function T(e,t){var n=++x;return D(e,n),_(null,t,e,n)}function I(e,t){var n=++x;return D(e,n,!0),_(t,null,e,n)}function A(e,t,n,r){z(e,{c:t,e:n,p:r,asyncOpID:o._traceAsyncOperationStarting(\"WinJS.Promise.done\")})}function P(e,t,n,r){e._value=t,L(e,t,n,r),e._setState(w)}function O(t,n){var r,i,s=t._value,a=t._listeners;if(a)for(t._listeners=null,r=0,i=Array.isArray(a)?a.length:1;r<i;r++){var l=1===i?a:a[r],c=l.c,h=l.promise;if(o._traceAsyncOperationCompleted(l.asyncOpID,e.Debug&&e.Debug.MS_ASYNC_OP_STATUS_SUCCESS),h){o._traceAsyncCallbackStarting(l.asyncOpID);try{h._setCompleteValue(c?c(s):s)}catch(e){h._setExceptionValue(e)}finally{o._traceAsyncCallbackCompleted()}h._state!==m&&h._listeners&&n.push(h)}else W.prototype.done.call(t,c)}}function N(t,n){var r,i,s=t._value,a=t._listeners;if(a)for(t._listeners=null,r=0,i=Array.isArray(a)?a.length:1;r<i;r++){var c=1===i?a:a[r],h=c.e,d=c.promise,u=e.Debug&&(s&&s.name===l?e.Debug.MS_ASYNC_OP_STATUS_CANCELED:e.Debug.MS_ASYNC_OP_STATUS_ERROR);if(o._traceAsyncOperationCompleted(c.asyncOpID,u),d){var p=!1;try{h?(o._traceAsyncCallbackStarting(c.asyncOpID),p=!0,h.handlesOnError||L(d,s,F,t,h),d._setCompleteValue(h(s))):d._setChainedErrorValue(s,t)}catch(e){d._setExceptionValue(e)}finally{p&&o._traceAsyncCallbackCompleted()}d._state!==m&&d._listeners&&n.push(d)}else K.prototype.done.call(t,null,h)}}function L(e,t,n,r,i){if(s._listeners[a]){if(t instanceof Error&&t.message===l)return;s.dispatchEvent(a,n(e,t,r,i))}}function R(e,t){var n,r,i=e._listeners;if(i)for(n=0,r=Array.isArray(i)?i.length:1;n<r;n++){var o=1===r?i:i[n],s=o.p;if(s)try{s(t)}catch(e){}o.c||o.e||!o.promise||o.promise._progress(t)}}function z(e,t){var n=e._listeners;n?(n=Array.isArray(n)?n:[n]).push(t):n=t,e._listeners=n}function D(e,t,n){e._isException=n||!1,e._errorId=t}function U(e,t,n,r){e._value=t,L(e,t,n,r),e._setState(v)}function M(e,t){var n;n=t&&\"object\"==typeof t&&\"function\"==typeof t.then?m:y,e._value=t,e._setState(n)}function j(e,t,n,r){var i=new B(e);return z(e,{promise:i,c:t,e:n,p:r,asyncOpID:o._traceAsyncOperationStarting(\"WinJS.Promise.then\")}),i}var V,B=n.Class.derive(C,function(e){c&&(!0===c||c&h.thenPromise)&&(this._stack=$._getStack()),this._creator=e,this._setState(d),this._run()},{_creator:null,_cancelAction:function(){this._creator&&this._creator.cancel()},_cleanupAction:function(){this._creator=null}},{supportedForProcessing:!1}),K=n.Class.define(function(e){c&&(!0===c||c&h.errorPromise)&&(this._stack=$._getStack()),this._value=e,L(this,e,T)},{cancel:function(){},done:function(e,t){var n=this._value;if(t)try{t.handlesOnError||L(null,n,F,this,t);var r=t(n);return void(r&&\"object\"==typeof r&&\"function\"==typeof r.done&&r.done())}catch(e){n=e}n instanceof Error&&n.message===l||$._doneHandler(n)},then:function(e,t){if(!t)return this;var n,r=this._value;try{t.handlesOnError||L(null,r,F,this,t),n=new W(t(r))}catch(e){n=e===r?this:new q(e)}return n}},{supportedForProcessing:!1}),q=n.Class.derive(K,function(e){c&&(!0===c||c&h.exceptionPromise)&&(this._stack=$._getStack()),this._value=e,L(this,e,I)},{},{supportedForProcessing:!1}),W=n.Class.define(function(e){if(c&&(!0===c||c&h.completePromise)&&(this._stack=$._getStack()),e&&\"object\"==typeof e&&\"function\"==typeof e.then){var t=new B(null);return t._setCompleteValue(e),t}this._value=e},{cancel:function(){},done:function(e){if(e)try{var t=e(this._value);t&&\"object\"==typeof t&&\"function\"==typeof t.done&&t.done()}catch(e){$._doneHandler(e)}},then:function(e){try{var t=e?e(this._value):this._value;return t===this._value?this:new W(t)}catch(e){return new q(e)}}},{supportedForProcessing:!1});var $=n.Class.derive(C,function(e,t){c&&(!0===c||c&h.promise)&&(this._stack=$._getStack()),this._oncancel=t,this._setState(d),this._run();try{e(this._completed.bind(this),this._error.bind(this),this._progress.bind(this))}catch(e){this._setExceptionValue(e)}},{_oncancel:null,_cancelAction:function(){try{if(!this._oncancel)throw new Error(\"Promise did not implement oncancel\");this._oncancel()}catch(e){e.message,e.stack;s.dispatchEvent(\"error\",e)}},_cleanupAction:function(){this._oncancel=null}},{addEventListener:function(e,t,n){s.addEventListener(e,t,n)},any:function(e){return new $(function(t,n){var r=Object.keys(e);0===r.length&&t();var i=0;r.forEach(function(o){$.as(e[o]).then(function(){t({key:o,value:e[o]})},function(s){s instanceof Error&&s.name===l?++i===r.length&&t($.cancel):n({key:o,value:e[o]})})})},function(){Object.keys(e).forEach(function(t){var n=$.as(e[t]);\"function\"==typeof n.cancel&&n.cancel()})})},as:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then?e:new W(e)},cancel:{get:function(){return V=V||new K(new r(l))}},dispatchEvent:function(e,t){return s.dispatchEvent(e,t)},is:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then},join:function(e){return new $(function(t,n,r){var i=Object.keys(e),o=Array.isArray(e)?[]:{},s=Array.isArray(e)?[]:{},a=0,c=i.length,h=function(e){if(0==--c){var a=Object.keys(o).length;if(0===a)t(s);else{var h=0;i.forEach(function(e){var t=o[e];t instanceof Error&&t.name===l&&h++}),h===a?t($.cancel):n(o)}}else r({Key:e,Done:!0})};i.forEach(function(t){var n=e[t];void 0===n?a++:$.then(n,function(e){s[t]=e,h(t)},function(e){o[t]=e,h(t)})}),0!==(c-=a)||t(s)},function(){Object.keys(e).forEach(function(t){var n=$.as(e[t]);\"function\"==typeof n.cancel&&n.cancel()})})},removeEventListener:function(e,t,n){s.removeEventListener(e,t,n)},supportedForProcessing:!1,then:function(e,t,n,r){return $.as(e).then(t,n,r)},thenEach:function(e,t,n,r){var i=Array.isArray(e)?[]:{};return Object.keys(e).forEach(function(o){i[o]=$.as(e[o]).then(t,n,r)}),$.join(i)},timeout:function(n,r){var i,o,s=(i=n,new $(function(n){i?o=e.setTimeout(n,i):t._setImmediate(n)},function(){o&&e.clearTimeout(o)}));return r?function(e,t){var n=function(){e.cancel()};return e.then(function(){t.cancel()}),t.then(n,n),t}(s,r):s},wrap:function(e){return new W(e)},wrapError:function(e){return new K(e)},_veryExpensiveTagWithStack:{get:function(){return c},set:function(e){c=e}},_veryExpensiveTagWithStack_tag:h,_getStack:function(){if(e.Debug&&e.Debug.debuggerEnabled)try{throw new Error}catch(e){return e.stack}},_cancelBlocker:function(e,t){if(!$.is(e))return $.wrap(e);var n,r,i=new $(function(e,t){n=e,r=t},function(){n=null,r=null,t&&t()});return e.then(function(e){n&&n(e)},function(e){r&&r(e)}),i}});return Object.defineProperties($,i.createEventProperties(a)),$._doneHandler=function(e){t._setImmediate(function(){throw e})},{PromiseStateMachine:C,Promise:$,state_created:d}}),n(\"WinJS/Promise\",[\"WinJS/Core/_Base\",\"WinJS/Promise/_StateMachine\"],function(e,t){return e.Namespace.define(\"WinJS\",{Promise:t.Promise}),t.Promise}),(e=t[\"WinJS/Core/_WinJS\"]).TPromise=e.Promise,e.PPromise=e.Promise}();var t=e.Promise,n=e.TPromise,r=(e.PPromise,{});n.addEventListener(\"error\",function(e){var t=e.detail,n=t.id;t.parent?t.handler&&r&&delete r[n]:(r[n]=t,1===Object.keys(r).length&&setTimeout(function(){var e=r;r={},Object.keys(e).forEach(function(t){var n=e[t];n.exception?o(n.exception):n.error&&o(n.error),console.log(\"WARNING: Promise with no error callback:\"+n.id),console.log(n),n.exception&&console.log(n.exception.stack)})},0))});var i=new(function(){function e(){this.listeners=[],this.unexpectedErrorHandler=function(e){setTimeout(function(){if(e.stack)throw new Error(e.message+\"\\n\\n\"+e.stack);throw e},0)}}return e.prototype.emit=function(e){this.listeners.forEach(function(t){t(e)})},e.prototype.onUnexpectedError=function(e){this.unexpectedErrorHandler(e),this.emit(e)},e.prototype.onUnexpectedExternalError=function(e){this.unexpectedErrorHandler(e)},e}());function o(e){var t;(t=e)instanceof Error&&t.name===a&&t.message===a||i.onUnexpectedError(e)}function s(e){return e instanceof Error?{$isError:!0,name:e.name,message:e.message,stack:e.stacktrace||e.stack}:e}var a=\"Canceled\";var l=function(){function e(){this._toDispose=[]}return e.prototype.dispose=function(){this._toDispose=function e(t){for(var n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];return Array.isArray(t)?(t.forEach(function(e){return e&&e.dispose()}),[]):0===n.length?t?(t.dispose(),t):void 0:(e(t),e(n),[])}(this._toDispose)},e.prototype._register=function(e){return this._toDispose.push(e),e},e.None=Object.freeze({dispose:function(){}}),e}(),c=!1,h=\"undefined\"!=typeof process&&void 0!==process.versions&&void 0!==process.versions.electron&&\"renderer\"===process.type;if(\"object\"!=typeof navigator||h){if(\"object\"==typeof process){c=\"win32\"===process.platform,\"darwin\"===process.platform,\"linux\"===process.platform,\"en\";var d=process.env.VSCODE_NLS_CONFIG;if(d)try{var u=JSON.parse(d);u.availableLanguages[\"*\"];u.locale,u._translationsConfigFile}catch(e){}}}else{var m=navigator.userAgent;c=m.indexOf(\"Windows\")>=0,m.indexOf(\"Macintosh\")>=0,m.indexOf(\"Linux\")>=0,navigator.language}var p=c,f=\"object\"==typeof self?self:\"object\"==typeof global?global:{},g=null;function b(e){return null===g&&(g=f.setImmediate?f.setImmediate.bind(f):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):f.setTimeout.bind(f)),g(e)}var y,w,v=(y=function(e,t){return(y=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}y(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),x={done:!0,value:void 0};!function(e){var t={next:function(){return x}};function n(e,t){for(var n=e.next();!n.done;n=e.next())t(n.value)}e.empty=function(){return t},e.fromArray=function(e,t,n){return void 0===t&&(t=0),void 0===n&&(n=e.length),{next:function(){return t>=n?x:{done:!1,value:e[t++]}}}},e.from=function(t){return t?Array.isArray(t)?e.fromArray(t):t:e.empty()},e.map=function(e,t){return{next:function(){var n=e.next();return n.done?x:{done:!1,value:t(n.value)}}}},e.filter=function(e,t){return{next:function(){for(;;){var n=e.next();if(n.done)return x;if(t(n.value))return{done:!1,value:n.value}}}}},e.forEach=n,e.collect=function(e){var t=[];return n(e,function(e){return t.push(e)}),t}}(w||(w={}));!function(e){function t(t,n,r,i){return void 0===n&&(n=0),void 0===r&&(r=t.length),void 0===i&&(i=n-1),e.call(this,t,n,r,i)||this}v(t,e),t.prototype.current=function(){return e.prototype.current.call(this)},t.prototype.previous=function(){return this.index=Math.max(this.index-1,this.start-1),this.current()},t.prototype.first=function(){return this.index=this.start,this.current()},t.prototype.last=function(){return this.index=this.end-1,this.current()},t.prototype.parent=function(){return null}}(function(){function e(e,t,n,r){void 0===t&&(t=0),void 0===n&&(n=e.length),void 0===r&&(r=t-1),this.items=e,this.start=t,this.end=n,this.index=r}return e.prototype.next=function(){return this.index=Math.min(this.index+1,this.end),this.current()},e.prototype.current=function(){return this.index===this.start-1||this.index===this.end?null:this.items[this.index]},e}());var k,C=function(){return function(e){this.element=e}}(),S=function(){function e(){}return e.prototype.isEmpty=function(){return!this._first},e.prototype.unshift=function(e){return this.insert(e,!1)},e.prototype.push=function(e){return this.insert(e,!0)},e.prototype.insert=function(e,t){var n=this,r=new C(e);if(this._first)if(t){var i=this._last;this._last=r,r.prev=i,i.next=r}else{var o=this._first;this._first=r,r.next=o,o.prev=r}else this._first=r,this._last=r;return function(){for(var e=n._first;e instanceof C;){if(e===r){if(e.prev&&e.next){var t=e.prev;t.next=e.next,e.next.prev=t}else e.prev||e.next?e.next?e.prev||(n._first=n._first.next,n._first.prev=void 0):(n._last=n._last.prev,n._last.next=void 0):(n._first=void 0,n._last=void 0);break}e=e.next}}},e.prototype.iterator=function(){var e,t=this._first;return{next:function(){return t?(e?e.value=t.element:e={done:!1,value:t.element},t=t.next,e):x}}},e}();!function(e){var t={dispose:function(){}};e.None=function(){return t}}(k||(k={}));var _=function(){function e(e){void 0===e&&(e=null),this._options=e,this._event=null,this._disposed=!1,this._deliveryQueue=null,this._listeners=null}return Object.defineProperty(e.prototype,\"event\",{get:function(){var t=this;return this._event||(this._event=function(n,r,i){t._listeners||(t._listeners=new S);var o=t._listeners.isEmpty();o&&t._options&&t._options.onFirstListenerAdd&&t._options.onFirstListenerAdd(t);var s,a=t._listeners.push(r?[n,r]:n);return o&&t._options&&t._options.onFirstListenerDidAdd&&t._options.onFirstListenerDidAdd(t),t._options&&t._options.onListenerDidAdd&&t._options.onListenerDidAdd(t,n,r),s={dispose:function(){(s.dispose=e._noop,t._disposed)||(a(),t._options&&t._options.onLastListenerRemove&&(t._listeners&&!t._listeners.isEmpty()||t._options.onLastListenerRemove(t)))}},Array.isArray(i)&&i.push(s),s}),this._event},enumerable:!0,configurable:!0}),e.prototype.fire=function(e){if(this._listeners){this._deliveryQueue||(this._deliveryQueue=[]);for(var t=this._listeners.iterator(),n=t.next();!n.done;n=t.next())this._deliveryQueue.push([n.value,e]);for(;this._deliveryQueue.length>0;){var r=this._deliveryQueue.shift(),i=r[0],s=r[1];try{\"function\"==typeof i?i.call(void 0,s):i[0].call(i[1],s)}catch(n){o(n)}}}},e.prototype.dispose=function(){this._listeners&&(this._listeners=null),this._deliveryQueue&&(this._deliveryQueue.length=0),this._disposed=!0},e._noop=function(){},e}();!function(){function e(){var e=this;this.hasListeners=!1,this.events=[],this.emitter=new _({onFirstListenerAdd:function(){return e.onFirstListenerAdd()},onLastListenerRemove:function(){return e.onLastListenerRemove()}})}Object.defineProperty(e.prototype,\"event\",{get:function(){return this.emitter.event},enumerable:!0,configurable:!0}),e.prototype.add=function(e){var t=this,n={event:e,listener:null};this.events.push(n),this.hasListeners&&this.hook(n);var r;return r=function(e){var t,n=this,r=!1;return function(){return r?t:(r=!0,t=e.apply(n,arguments))}}(function(){t.hasListeners&&t.unhook(n);var e=t.events.indexOf(n);t.events.splice(e,1)}),{dispose:function(){r()}}},e.prototype.onFirstListenerAdd=function(){var e=this;this.hasListeners=!0,this.events.forEach(function(t){return e.hook(t)})},e.prototype.onLastListenerRemove=function(){var e=this;this.hasListeners=!1,this.events.forEach(function(t){return e.unhook(t)})},e.prototype.hook=function(e){var t=this;e.listener=e.event(function(e){return t.emitter.fire(e)})},e.prototype.unhook=function(e){e.listener&&e.listener.dispose(),e.listener=null},e.prototype.dispose=function(){this.emitter.dispose()}}();(function(){function e(e){this._event=e}Object.defineProperty(e.prototype,\"event\",{get:function(){return this._event},enumerable:!0,configurable:!0}),e.prototype.map=function(t){return new e((n=this._event,r=t,function(e,t,i){return void 0===t&&(t=null),n(function(n){return e.call(t,r(n))},null,i)}));var n,r},e.prototype.filter=function(t){return new e((n=this._event,r=t,function(e,t,i){return void 0===t&&(t=null),n(function(n){return r(n)&&e.call(t,n)},null,i)}));var n,r},e.prototype.on=function(e,t,n){return this._event(e,t,n)}})(),function(){function e(){var e=this;this.listening=!1,this.inputEvent=k.None,this.inputEventListener=l.None,this.emitter=new _({onFirstListenerDidAdd:function(){e.listening=!0,e.inputEventListener=e.inputEvent(e.emitter.fire,e.emitter)},onLastListenerRemove:function(){e.listening=!1,e.inputEventListener.dispose()}}),this.event=this.emitter.event}Object.defineProperty(e.prototype,\"input\",{set:function(e){this.inputEvent=e,this.listening&&(this.inputEventListener.dispose(),this.inputEventListener=e(this.emitter.fire,this.emitter))},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.inputEventListener.dispose(),this.emitter.dispose()}}();var F,E=Object.freeze(function(e,t){var n=setTimeout(e.bind(t),0);return{dispose:function(){clearTimeout(n)}}});!function(e){e.isCancellationToken=function(t){return t===e.None||t===e.Cancelled||t instanceof T||!(!t||\"object\"!=typeof t)&&\"boolean\"==typeof t.isCancellationRequested&&\"function\"==typeof t.onCancellationRequested},e.None=Object.freeze({isCancellationRequested:!1,onCancellationRequested:k.None}),e.Cancelled=Object.freeze({isCancellationRequested:!0,onCancellationRequested:E})}(F||(F={}));var T=function(){function e(){this._isCancelled=!1,this._emitter=null}return e.prototype.cancel=function(){this._isCancelled||(this._isCancelled=!0,this._emitter&&(this._emitter.fire(void 0),this.dispose()))},Object.defineProperty(e.prototype,\"isCancellationRequested\",{get:function(){return this._isCancelled},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"onCancellationRequested\",{get:function(){return this._isCancelled?E:(this._emitter||(this._emitter=new _),this._emitter.event)},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this._emitter&&(this._emitter.dispose(),this._emitter=null)},e}(),I=function(){function e(){}return Object.defineProperty(e.prototype,\"token\",{get:function(){return this._token||(this._token=new T),this._token},enumerable:!0,configurable:!0}),e.prototype.cancel=function(){this._token?this._token instanceof T&&this._token.cancel():this._token=F.Cancelled},e.prototype.dispose=function(){this._token?this._token instanceof T&&this._token.dispose():this._token=F.None},e}(),A=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();(function(e){function t(t,n){var r=e.call(this)||this;return r._token=-1,\"function\"==typeof t&&\"number\"==typeof n&&r.setIfNotSet(t,n),r}A(t,e),t.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},t.prototype.cancel=function(){-1!==this._token&&(clearTimeout(this._token),this._token=-1)},t.prototype.cancelAndSet=function(e,t){var n=this;this.cancel(),this._token=setTimeout(function(){n._token=-1,e()},t)},t.prototype.setIfNotSet=function(e,t){var n=this;-1===this._token&&(this._token=setTimeout(function(){n._token=-1,e()},t))}})(l),function(e){function t(){var t=e.call(this)||this;return t._token=-1,t}A(t,e),t.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},t.prototype.cancel=function(){-1!==this._token&&(clearInterval(this._token),this._token=-1)},t.prototype.cancelAndSet=function(e,t){this.cancel(),this._token=setInterval(function(){e()},t)}}(l);!function(){if(\"function\"!=typeof requestIdleCallback||\"function\"!=typeof cancelIdleCallback)Object.freeze({didTimeout:!0,timeRemaining:function(){return 15}})}();var P=function(){function e(e){var n,r;(r=n=e)&&\"function\"==typeof r.then&&\"function\"==typeof n.done?this._winjsPromise=e:this._winjsPromise=new t(function(t,n){var r=!0;e(function(e){r?b(function(){return t(e)}):t(e)},function(e){r?b(function(){return n(e)}):n(e)}),r=!1})}return e.all=function(n){return new e(t.join(n).then(null,function(e){for(var t in e)if(e.hasOwnProperty(t))return e[t]}))},e.race=function(n){return new e(t.any(n).then(function(e){return e.value},function(e){return e.value}))},e.resolve=function(n){return new e(t.wrap(n))},e.reject=function(n){return new e(t.wrapError(n))},e.prototype.then=function(n,r){var i=!0,o=new e(this._winjsPromise.then(n&&function(e){return i?new t(function(t,r){b(function(){var i;try{i=n(e)}catch(e){return void r(e)}t(i)})}):n(e)},r&&function(e){return i?new t(function(t,n){b(function(){var i;try{i=r(e)}catch(e){return void n(e)}t(i)})}):r(e)}));return i=!1,o},e.prototype.catch=function(e){return this.then(null,e)},e}(),O=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),N=self;void 0===N.Promise&&(N.Promise=P);var L=\"$initialize\",R=function(){function e(e){this._workerId=-1,this._handler=e,this._lastSentReq=0,this._pendingReplies=Object.create(null)}return e.prototype.setWorkerId=function(e){this._workerId=e},e.prototype.sendMessage=function(e,t){var n=this,r=String(++this._lastSentReq);return new Promise(function(i,o){n._pendingReplies[r]={resolve:i,reject:o},n._send({vsWorker:n._workerId,req:r,method:e,args:t})})},e.prototype.handleMessage=function(e){var t;try{t=JSON.parse(e)}catch(e){return}t&&t.vsWorker&&(-1!==this._workerId&&t.vsWorker!==this._workerId||this._handleMessage(t))},e.prototype._handleMessage=function(e){var t=this;if(e.seq){var n=e;if(!this._pendingReplies[n.seq])return void console.warn(\"Got reply to unknown seq\");var r=this._pendingReplies[n.seq];if(delete this._pendingReplies[n.seq],n.err){var i=n.err;return n.err.$isError&&((i=new Error).name=n.err.name,i.message=n.err.message,i.stack=n.err.stack),void r.reject(i)}r.resolve(n.res)}else{var o=e,a=o.req;this._handler.handleMessage(o.method,o.args).then(function(e){t._send({vsWorker:t._workerId,seq:a,res:e,err:void 0})},function(e){e.detail instanceof Error&&(e.detail=s(e.detail)),t._send({vsWorker:t._workerId,seq:a,res:void 0,err:s(e)})})}},e.prototype._send=function(e){var t=JSON.stringify(e);this._handler.sendMessage(t)},e}(),z=(function(e){function t(t,n){var r=e.call(this)||this,i=null;r._worker=r._register(t.create(\"vs/base/common/worker/simpleWorker\",function(e){r._protocol.handleMessage(e)},function(e){i&&i(e)})),r._protocol=new R({sendMessage:function(e){r._worker.postMessage(e)},handleMessage:function(e,t){return Promise.resolve(null)}}),r._protocol.setWorkerId(r._worker.getId());var o=null;void 0!==self.require&&\"function\"==typeof self.require.getConfig?o=self.require.getConfig():void 0!==self.requirejs&&(o=self.requirejs.s.contexts._.config),r._onModuleLoaded=r._protocol.sendMessage(L,[r._worker.getId(),n,o]),r._lazyProxy=new Promise(function(e,t){i=t,r._onModuleLoaded.then(function(t){for(var n={},r=0;r<t.length;r++)n[t[r]]=a(t[r],s);e(n)},function(e){t(e),r._onError(\"Worker failed to load \"+n,e)})});var s=function(e,t){return r._request(e,t)},a=function(e,t){return function(){var n=Array.prototype.slice.call(arguments,0);return t(e,n)}};return r}O(t,e),t.prototype.getProxyObject=function(){return this._lazyProxy},t.prototype._request=function(e,t){var n=this;return new Promise(function(r,i){n._onModuleLoaded.then(function(){n._protocol.sendMessage(e,t).then(r,i)},i)})},t.prototype._onError=function(e,t){console.error(e),console.info(t)}}(l),function(){function e(e,t){var n=this;this._requestHandler=t,this._protocol=new R({sendMessage:function(t){e(t)},handleMessage:function(e,t){return n._handleMessage(e,t)}})}return e.prototype.onmessage=function(e){this._protocol.handleMessage(e)},e.prototype._handleMessage=function(e,t){if(e===L)return this.initialize(t[0],t[1],t[2]);if(!this._requestHandler||\"function\"!=typeof this._requestHandler[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._requestHandler[e].apply(this._requestHandler,t))}catch(e){return Promise.reject(e)}},e.prototype.initialize=function(e,t,n){var r=this;if(this._protocol.setWorkerId(e),this._requestHandler){var i=[];for(var o in this._requestHandler)\"function\"==typeof this._requestHandler[o]&&i.push(o);return Promise.resolve(i)}return n&&(void 0!==n.baseUrl&&delete n.baseUrl,void 0!==n.paths&&void 0!==n.paths.vs&&delete n.paths.vs,n.catchError=!0,self.require.config(n)),new Promise(function(e,n){self.require([t],function(){for(var t=[],i=0;i<arguments.length;i++)t[i]=arguments[i];var o=t[0];if(r._requestHandler=o.create(),r._requestHandler){var s=[];for(var a in r._requestHandler)\"function\"==typeof r._requestHandler[a]&&s.push(a);e(s)}else n(new Error(\"No RequestHandler!\"))},n)})},e}());function D(e,t){return function e(t,n,r,i,o){if(i<=r)return;var s=r+(i-r)/2|0;e(t,n,r,s,o);e(t,n,s+1,i,o);if(n(t[s],t[s+1])<=0)return;!function(e,t,n,r,i,o){for(var s=n,a=r+1,l=n;l<=i;l++)o[l]=e[l];for(var l=n;l<=i;l++)s>r?e[l]=o[a++]:a>i?e[l]=o[s++]:t(o[a],o[s])<0?e[l]=o[a++]:e[l]=o[s++]}(t,n,r,s,i,o)}(e,t,0,e.length-1,[]),e}var U=function(){function e(e,t,n,r){this.originalStart=e,this.originalLength=t,this.modifiedStart=n,this.modifiedLength=r}return e.prototype.getOriginalEnd=function(){return this.originalStart+this.originalLength},e.prototype.getModifiedEnd=function(){return this.modifiedStart+this.modifiedLength},e}();function M(e){return{getLength:function(){return e.length},getElementAtIndex:function(t){return e.charCodeAt(t)}}}function j(e,t,n){return new W(M(e),M(t)).ComputeDiff(n)}var V,B=function(){function e(){}return e.Assert=function(e,t){if(!e)throw new Error(t)},e}(),K=function(){function e(){}return e.Copy=function(e,t,n,r,i){for(var o=0;o<i;o++)n[r+o]=e[t+o]},e}(),q=function(){function e(){this.m_changes=[],this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE,this.m_originalCount=0,this.m_modifiedCount=0}return e.prototype.MarkNextChange=function(){(this.m_originalCount>0||this.m_modifiedCount>0)&&this.m_changes.push(new U(this.m_originalStart,this.m_originalCount,this.m_modifiedStart,this.m_modifiedCount)),this.m_originalCount=0,this.m_modifiedCount=0,this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE},e.prototype.AddOriginalElement=function(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_originalCount++},e.prototype.AddModifiedElement=function(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_modifiedCount++},e.prototype.getChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes},e.prototype.getReverseChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes.reverse(),this.m_changes},e}(),W=function(){function e(e,t,n){void 0===n&&(n=null),this.OriginalSequence=e,this.ModifiedSequence=t,this.ContinueProcessingPredicate=n,this.m_forwardHistory=[],this.m_reverseHistory=[]}return e.prototype.ElementsAreEqual=function(e,t){return this.OriginalSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(t)},e.prototype.OriginalElementsAreEqual=function(e,t){return this.OriginalSequence.getElementAtIndex(e)===this.OriginalSequence.getElementAtIndex(t)},e.prototype.ModifiedElementsAreEqual=function(e,t){return this.ModifiedSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(t)},e.prototype.ComputeDiff=function(e){return this._ComputeDiff(0,this.OriginalSequence.getLength()-1,0,this.ModifiedSequence.getLength()-1,e)},e.prototype._ComputeDiff=function(e,t,n,r,i){var o=this.ComputeDiffRecursive(e,t,n,r,[!1]);return i?this.PrettifyChanges(o):o},e.prototype.ComputeDiffRecursive=function(e,t,n,r,i){for(i[0]=!1;e<=t&&n<=r&&this.ElementsAreEqual(e,n);)e++,n++;for(;t>=e&&r>=n&&this.ElementsAreEqual(t,r);)t--,r--;if(e>t||n>r){var o=void 0;return n<=r?(B.Assert(e===t+1,\"originalStart should only be one more than originalEnd\"),o=[new U(e,0,n,r-n+1)]):e<=t?(B.Assert(n===r+1,\"modifiedStart should only be one more than modifiedEnd\"),o=[new U(e,t-e+1,n,0)]):(B.Assert(e===t+1,\"originalStart should only be one more than originalEnd\"),B.Assert(n===r+1,\"modifiedStart should only be one more than modifiedEnd\"),o=[]),o}var s=[0],a=[0],l=this.ComputeRecursionPoint(e,t,n,r,s,a,i),c=s[0],h=a[0];if(null!==l)return l;if(!i[0]){var d=this.ComputeDiffRecursive(e,c,n,h,i),u=[];return u=i[0]?[new U(c+1,t-(c+1)+1,h+1,r-(h+1)+1)]:this.ComputeDiffRecursive(c+1,t,h+1,r,i),this.ConcatenateChanges(d,u)}return[new U(e,t-e+1,n,r-n+1)]},e.prototype.WALKTRACE=function(e,t,n,r,i,o,s,a,l,c,h,d,u,m,p,f,g,b){var y,w,v=null,x=new q,k=t,C=n,S=u[0]-f[0]-r,_=Number.MIN_VALUE,F=this.m_forwardHistory.length-1;do{(w=S+e)===k||w<C&&l[w-1]<l[w+1]?(m=(h=l[w+1])-S-r,h<_&&x.MarkNextChange(),_=h,x.AddModifiedElement(h+1,m),S=w+1-e):(m=(h=l[w-1]+1)-S-r,h<_&&x.MarkNextChange(),_=h-1,x.AddOriginalElement(h,m+1),S=w-1-e),F>=0&&(e=(l=this.m_forwardHistory[F])[0],k=1,C=l.length-1)}while(--F>=-1);if(y=x.getReverseChanges(),b[0]){var E=u[0]+1,T=f[0]+1;if(null!==y&&y.length>0){var I=y[y.length-1];E=Math.max(E,I.getOriginalEnd()),T=Math.max(T,I.getModifiedEnd())}v=[new U(E,d-E+1,T,p-T+1)]}else{x=new q,k=o,C=s,S=u[0]-f[0]-a,_=Number.MAX_VALUE,F=g?this.m_reverseHistory.length-1:this.m_reverseHistory.length-2;do{(w=S+i)===k||w<C&&c[w-1]>=c[w+1]?(m=(h=c[w+1]-1)-S-a,h>_&&x.MarkNextChange(),_=h+1,x.AddOriginalElement(h+1,m+1),S=w+1-i):(m=(h=c[w-1])-S-a,h>_&&x.MarkNextChange(),_=h,x.AddModifiedElement(h+1,m+1),S=w-1-i),F>=0&&(i=(c=this.m_reverseHistory[F])[0],k=1,C=c.length-1)}while(--F>=-1);v=x.getChanges()}return this.ConcatenateChanges(y,v)},e.prototype.ComputeRecursionPoint=function(e,t,n,r,i,o,s){var a,l=0,c=0,h=0,d=0,u=0,m=0;e--,n--,i[0]=0,o[0]=0,this.m_forwardHistory=[],this.m_reverseHistory=[];var p,f,g=t-e+(r-n),b=g+1,y=new Array(b),w=new Array(b),v=r-n,x=t-e,k=e-n,C=t-r,S=(x-v)%2==0;for(y[v]=e,w[x]=t,s[0]=!1,a=1;a<=g/2+1;a++){var _=0,F=0;for(h=this.ClipDiagonalBound(v-a,a,v,b),d=this.ClipDiagonalBound(v+a,a,v,b),p=h;p<=d;p+=2){for(c=(l=p===h||p<d&&y[p-1]<y[p+1]?y[p+1]:y[p-1]+1)-(p-v)-k,f=l;l<t&&c<r&&this.ElementsAreEqual(l+1,c+1);)l++,c++;if(y[p]=l,l+c>_+F&&(_=l,F=c),!S&&Math.abs(p-x)<=a-1&&l>=w[p])return i[0]=l,o[0]=c,f<=w[p]&&a<=1448?this.WALKTRACE(v,h,d,k,x,u,m,C,y,w,l,t,i,c,r,o,S,s):null}var E=(_-e+(F-n)-a)/2;if(null!==this.ContinueProcessingPredicate&&!this.ContinueProcessingPredicate(_,this.OriginalSequence,E))return s[0]=!0,i[0]=_,o[0]=F,E>0&&a<=1448?this.WALKTRACE(v,h,d,k,x,u,m,C,y,w,l,t,i,c,r,o,S,s):[new U(++e,t-e+1,++n,r-n+1)];for(u=this.ClipDiagonalBound(x-a,a,x,b),m=this.ClipDiagonalBound(x+a,a,x,b),p=u;p<=m;p+=2){for(c=(l=p===u||p<m&&w[p-1]>=w[p+1]?w[p+1]-1:w[p-1])-(p-x)-C,f=l;l>e&&c>n&&this.ElementsAreEqual(l,c);)l--,c--;if(w[p]=l,S&&Math.abs(p-v)<=a&&l<=y[p])return i[0]=l,o[0]=c,f>=y[p]&&a<=1448?this.WALKTRACE(v,h,d,k,x,u,m,C,y,w,l,t,i,c,r,o,S,s):null}if(a<=1447){var T=new Array(d-h+2);T[0]=v-h+1,K.Copy(y,h,T,1,d-h+1),this.m_forwardHistory.push(T),(T=new Array(m-u+2))[0]=x-u+1,K.Copy(w,u,T,1,m-u+1),this.m_reverseHistory.push(T)}}return this.WALKTRACE(v,h,d,k,x,u,m,C,y,w,l,t,i,c,r,o,S,s)},e.prototype.PrettifyChanges=function(e){for(var t=0;t<e.length;t++){for(var n=e[t],r=t<e.length-1?e[t+1].originalStart:this.OriginalSequence.getLength(),i=t<e.length-1?e[t+1].modifiedStart:this.ModifiedSequence.getLength(),o=n.originalLength>0,s=n.modifiedLength>0;n.originalStart+n.originalLength<r&&n.modifiedStart+n.modifiedLength<i&&(!o||this.OriginalElementsAreEqual(n.originalStart,n.originalStart+n.originalLength))&&(!s||this.ModifiedElementsAreEqual(n.modifiedStart,n.modifiedStart+n.modifiedLength));)n.originalStart++,n.modifiedStart++;var a=[null];t<e.length-1&&this.ChangesOverlap(e[t],e[t+1],a)&&(e[t]=a[0],e.splice(t+1,1),t--)}for(t=e.length-1;t>=0;t--){n=e[t],r=0,i=0;if(t>0){var l=e[t-1];l.originalLength>0&&(r=l.originalStart+l.originalLength),l.modifiedLength>0&&(i=l.modifiedStart+l.modifiedLength)}o=n.originalLength>0,s=n.modifiedLength>0;for(var c=0,h=this._boundaryScore(n.originalStart,n.originalLength,n.modifiedStart,n.modifiedLength),d=1;;d++){var u=n.originalStart-d,m=n.modifiedStart-d;if(u<r||m<i)break;if(o&&!this.OriginalElementsAreEqual(u,u+n.originalLength))break;if(s&&!this.ModifiedElementsAreEqual(m,m+n.modifiedLength))break;var p=this._boundaryScore(u,n.originalLength,m,n.modifiedLength);p>h&&(h=p,c=d)}n.originalStart-=c,n.modifiedStart-=c}return e},e.prototype._OriginalIsBoundary=function(e){if(e<=0||e>=this.OriginalSequence.getLength()-1)return!0;var t=this.OriginalSequence.getElementAtIndex(e);return\"string\"==typeof t&&/^\\s*$/.test(t)},e.prototype._OriginalRegionIsBoundary=function(e,t){if(this._OriginalIsBoundary(e)||this._OriginalIsBoundary(e-1))return!0;if(t>0){var n=e+t;if(this._OriginalIsBoundary(n-1)||this._OriginalIsBoundary(n))return!0}return!1},e.prototype._ModifiedIsBoundary=function(e){if(e<=0||e>=this.ModifiedSequence.getLength()-1)return!0;var t=this.ModifiedSequence.getElementAtIndex(e);return\"string\"==typeof t&&/^\\s*$/.test(t)},e.prototype._ModifiedRegionIsBoundary=function(e,t){if(this._ModifiedIsBoundary(e)||this._ModifiedIsBoundary(e-1))return!0;if(t>0){var n=e+t;if(this._ModifiedIsBoundary(n-1)||this._ModifiedIsBoundary(n))return!0}return!1},e.prototype._boundaryScore=function(e,t,n,r){return(this._OriginalRegionIsBoundary(e,t)?1:0)+(this._ModifiedRegionIsBoundary(n,r)?1:0)},e.prototype.ConcatenateChanges=function(e,t){var n=[];if(0===e.length||0===t.length)return t.length>0?t:e;if(this.ChangesOverlap(e[e.length-1],t[0],n)){var r=new Array(e.length+t.length-1);return K.Copy(e,0,r,0,e.length-1),r[e.length-1]=n[0],K.Copy(t,1,r,e.length,t.length-1),r}r=new Array(e.length+t.length);return K.Copy(e,0,r,0,e.length),K.Copy(t,0,r,e.length,t.length),r},e.prototype.ChangesOverlap=function(e,t,n){if(B.Assert(e.originalStart<=t.originalStart,\"Left change is not less than or equal to right change\"),B.Assert(e.modifiedStart<=t.modifiedStart,\"Left change is not less than or equal to right change\"),e.originalStart+e.originalLength>=t.originalStart||e.modifiedStart+e.modifiedLength>=t.modifiedStart){var r=e.originalStart,i=e.originalLength,o=e.modifiedStart,s=e.modifiedLength;return e.originalStart+e.originalLength>=t.originalStart&&(i=t.originalStart+t.originalLength-e.originalStart),e.modifiedStart+e.modifiedLength>=t.modifiedStart&&(s=t.modifiedStart+t.modifiedLength-e.modifiedStart),n[0]=new U(r,i,o,s),!0}return n[0]=null,!1},e.prototype.ClipDiagonalBound=function(e,t,n,r){if(e>=0&&e<r)return e;var i=t%2==0;return e<0?i===(n%2==0)?0:1:i===((r-n-1)%2==0)?r-1:r-2},e}(),$=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),G=/^\\w[\\w\\d+.-]*$/,H=/^\\//,Y=/^\\/\\//;var J=\"\",X=\"/\",Q=/^(([^:\\/?#]+?):)?(\\/\\/([^\\/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?/,Z=function(){function e(e,t,n,r,i){\"object\"==typeof e?(this.scheme=e.scheme||J,this.authority=e.authority||J,this.path=e.path||J,this.query=e.query||J,this.fragment=e.fragment||J):(this.scheme=e||J,this.authority=t||J,this.path=function(e,t){switch(e){case\"https\":case\"http\":case\"file\":t?t[0]!==X&&(t=X+t):t=X}return t}(this.scheme,n||J),this.query=r||J,this.fragment=i||J,function(e){if(!e.scheme)throw new Error('[UriError]: Scheme is missing: {scheme: \"\", authority: \"'+e.authority+'\", path: \"'+e.path+'\", query: \"'+e.query+'\", fragment: \"'+e.fragment+'\"}');if(e.scheme&&!G.test(e.scheme))throw new Error(\"[UriError]: Scheme contains illegal characters.\");if(e.path)if(e.authority){if(!H.test(e.path))throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash (\"/\") character')}else if(Y.test(e.path))throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters (\"//\")')}(this))}return e.isUri=function(t){return t instanceof e||!!t&&(\"string\"==typeof t.authority&&\"string\"==typeof t.fragment&&\"string\"==typeof t.path&&\"string\"==typeof t.query&&\"string\"==typeof t.scheme)},Object.defineProperty(e.prototype,\"fsPath\",{get:function(){return ie(this)},enumerable:!0,configurable:!0}),e.prototype.with=function(e){if(!e)return this;var t=e.scheme,n=e.authority,r=e.path,i=e.query,o=e.fragment;return void 0===t?t=this.scheme:null===t&&(t=J),void 0===n?n=this.authority:null===n&&(n=J),void 0===r?r=this.path:null===r&&(r=J),void 0===i?i=this.query:null===i&&(i=J),void 0===o?o=this.fragment:null===o&&(o=J),t===this.scheme&&n===this.authority&&r===this.path&&i===this.query&&o===this.fragment?this:new ee(t,n,r,i,o)},e.parse=function(e){var t=Q.exec(e);return t?new ee(t[2]||J,decodeURIComponent(t[4]||J),decodeURIComponent(t[5]||J),decodeURIComponent(t[7]||J),decodeURIComponent(t[9]||J)):new ee(J,J,J,J,J)},e.file=function(e){var t=J;if(p&&(e=e.replace(/\\\\/g,X)),e[0]===X&&e[1]===X){var n=e.indexOf(X,2);-1===n?(t=e.substring(2),e=X):(t=e.substring(2,n),e=e.substring(n)||X)}return new ee(\"file\",t,e,J,J)},e.from=function(e){return new ee(e.scheme,e.authority,e.path,e.query,e.fragment)},e.prototype.toString=function(e){return void 0===e&&(e=!1),oe(this,e)},e.prototype.toJSON=function(){return this},e.revive=function(t){if(t){if(t instanceof e)return t;var n=new ee(t);return n._fsPath=t.fsPath,n._formatted=t.external,n}return t},e}(),ee=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._formatted=null,t._fsPath=null,t}return $(t,e),Object.defineProperty(t.prototype,\"fsPath\",{get:function(){return this._fsPath||(this._fsPath=ie(this)),this._fsPath},enumerable:!0,configurable:!0}),t.prototype.toString=function(e){return void 0===e&&(e=!1),e?oe(this,!0):(this._formatted||(this._formatted=oe(this,!1)),this._formatted)},t.prototype.toJSON=function(){var e={$mid:1};return this._fsPath&&(e.fsPath=this._fsPath),this._formatted&&(e.external=this._formatted),this.path&&(e.path=this.path),this.scheme&&(e.scheme=this.scheme),this.authority&&(e.authority=this.authority),this.query&&(e.query=this.query),this.fragment&&(e.fragment=this.fragment),e},t}(Z),te=((V={})[58]=\"%3A\",V[47]=\"%2F\",V[63]=\"%3F\",V[35]=\"%23\",V[91]=\"%5B\",V[93]=\"%5D\",V[64]=\"%40\",V[33]=\"%21\",V[36]=\"%24\",V[38]=\"%26\",V[39]=\"%27\",V[40]=\"%28\",V[41]=\"%29\",V[42]=\"%2A\",V[43]=\"%2B\",V[44]=\"%2C\",V[59]=\"%3B\",V[61]=\"%3D\",V[32]=\"%20\",V);function ne(e,t){for(var n=void 0,r=-1,i=0;i<e.length;i++){var o=e.charCodeAt(i);if(o>=97&&o<=122||o>=65&&o<=90||o>=48&&o<=57||45===o||46===o||95===o||126===o||t&&47===o)-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),void 0!==n&&(n+=e.charAt(i));else{void 0===n&&(n=e.substr(0,i));var s=te[o];void 0!==s?(-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),n+=s):-1===r&&(r=i)}}return-1!==r&&(n+=encodeURIComponent(e.substring(r))),void 0!==n?n:e}function re(e){for(var t=void 0,n=0;n<e.length;n++){var r=e.charCodeAt(n);35===r||63===r?(void 0===t&&(t=e.substr(0,n)),t+=te[r]):void 0!==t&&(t+=e[n])}return void 0!==t?t:e}function ie(e){var t;return t=e.authority&&e.path.length>1&&\"file\"===e.scheme?\"//\"+e.authority+e.path:47===e.path.charCodeAt(0)&&(e.path.charCodeAt(1)>=65&&e.path.charCodeAt(1)<=90||e.path.charCodeAt(1)>=97&&e.path.charCodeAt(1)<=122)&&58===e.path.charCodeAt(2)?e.path[1].toLowerCase()+e.path.substr(2):e.path,p&&(t=t.replace(/\\//g,\"\\\\\")),t}function oe(e,t){var n=t?re:ne,r=\"\",i=e.scheme,o=e.authority,s=e.path,a=e.query,l=e.fragment;if(i&&(r+=i,r+=\":\"),(o||\"file\"===i)&&(r+=X,r+=X),o){var c=o.indexOf(\"@\");if(-1!==c){var h=o.substr(0,c);o=o.substr(c+1),-1===(c=h.indexOf(\":\"))?r+=n(h,!1):(r+=n(h.substr(0,c),!1),r+=\":\",r+=n(h.substr(c+1),!1)),r+=\"@\"}-1===(c=(o=o.toLowerCase()).indexOf(\":\"))?r+=n(o,!1):(r+=n(o.substr(0,c),!1),r+=o.substr(c))}if(s){if(s.length>=3&&47===s.charCodeAt(0)&&58===s.charCodeAt(2))(d=s.charCodeAt(1))>=65&&d<=90&&(s=\"/\"+String.fromCharCode(d+32)+\":\"+s.substr(3));else if(s.length>=2&&58===s.charCodeAt(1)){var d;(d=s.charCodeAt(0))>=65&&d<=90&&(s=String.fromCharCode(d+32)+\":\"+s.substr(2))}r+=n(s,!0)}return a&&(r+=\"?\",r+=n(a,!1)),l&&(r+=\"#\",r+=t?l:ne(l,!1)),r}var se=function(){function e(e,t){this.lineNumber=e,this.column=t}return e.prototype.with=function(t,n){return void 0===t&&(t=this.lineNumber),void 0===n&&(n=this.column),t===this.lineNumber&&n===this.column?this:new e(t,n)},e.prototype.delta=function(e,t){return void 0===e&&(e=0),void 0===t&&(t=0),this.with(this.lineNumber+e,this.column+t)},e.prototype.equals=function(t){return e.equals(this,t)},e.equals=function(e,t){return!e&&!t||!!e&&!!t&&e.lineNumber===t.lineNumber&&e.column===t.column},e.prototype.isBefore=function(t){return e.isBefore(this,t)},e.isBefore=function(e,t){return e.lineNumber<t.lineNumber||!(t.lineNumber<e.lineNumber)&&e.column<t.column},e.prototype.isBeforeOrEqual=function(t){return e.isBeforeOrEqual(this,t)},e.isBeforeOrEqual=function(e,t){return e.lineNumber<t.lineNumber||!(t.lineNumber<e.lineNumber)&&e.column<=t.column},e.compare=function(e,t){var n=0|e.lineNumber,r=0|t.lineNumber;return n===r?(0|e.column)-(0|t.column):n-r},e.prototype.clone=function(){return new e(this.lineNumber,this.column)},e.prototype.toString=function(){return\"(\"+this.lineNumber+\",\"+this.column+\")\"},e.lift=function(t){return new e(t.lineNumber,t.column)},e.isIPosition=function(e){return e&&\"number\"==typeof e.lineNumber&&\"number\"==typeof e.column},e}(),ae=function(){function e(e,t,n,r){e>n||e===n&&t>r?(this.startLineNumber=n,this.startColumn=r,this.endLineNumber=e,this.endColumn=t):(this.startLineNumber=e,this.startColumn=t,this.endLineNumber=n,this.endColumn=r)}return e.prototype.isEmpty=function(){return e.isEmpty(this)},e.isEmpty=function(e){return e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn},e.prototype.containsPosition=function(t){return e.containsPosition(this,t)},e.containsPosition=function(e,t){return!(t.lineNumber<e.startLineNumber||t.lineNumber>e.endLineNumber)&&(!(t.lineNumber===e.startLineNumber&&t.column<e.startColumn)&&!(t.lineNumber===e.endLineNumber&&t.column>e.endColumn))},e.prototype.containsRange=function(t){return e.containsRange(this,t)},e.containsRange=function(e,t){return!(t.startLineNumber<e.startLineNumber||t.endLineNumber<e.startLineNumber)&&(!(t.startLineNumber>e.endLineNumber||t.endLineNumber>e.endLineNumber)&&(!(t.startLineNumber===e.startLineNumber&&t.startColumn<e.startColumn)&&!(t.endLineNumber===e.endLineNumber&&t.endColumn>e.endColumn)))},e.prototype.plusRange=function(t){return e.plusRange(this,t)},e.plusRange=function(t,n){var r,i,o,s;return n.startLineNumber<t.startLineNumber?(r=n.startLineNumber,i=n.startColumn):n.startLineNumber===t.startLineNumber?(r=n.startLineNumber,i=Math.min(n.startColumn,t.startColumn)):(r=t.startLineNumber,i=t.startColumn),n.endLineNumber>t.endLineNumber?(o=n.endLineNumber,s=n.endColumn):n.endLineNumber===t.endLineNumber?(o=n.endLineNumber,s=Math.max(n.endColumn,t.endColumn)):(o=t.endLineNumber,s=t.endColumn),new e(r,i,o,s)},e.prototype.intersectRanges=function(t){return e.intersectRanges(this,t)},e.intersectRanges=function(t,n){var r=t.startLineNumber,i=t.startColumn,o=t.endLineNumber,s=t.endColumn,a=n.startLineNumber,l=n.startColumn,c=n.endLineNumber,h=n.endColumn;return r<a?(r=a,i=l):r===a&&(i=Math.max(i,l)),o>c?(o=c,s=h):o===c&&(s=Math.min(s,h)),r>o?null:r===o&&i>s?null:new e(r,i,o,s)},e.prototype.equalsRange=function(t){return e.equalsRange(this,t)},e.equalsRange=function(e,t){return!!e&&!!t&&e.startLineNumber===t.startLineNumber&&e.startColumn===t.startColumn&&e.endLineNumber===t.endLineNumber&&e.endColumn===t.endColumn},e.prototype.getEndPosition=function(){return new se(this.endLineNumber,this.endColumn)},e.prototype.getStartPosition=function(){return new se(this.startLineNumber,this.startColumn)},e.prototype.toString=function(){return\"[\"+this.startLineNumber+\",\"+this.startColumn+\" -> \"+this.endLineNumber+\",\"+this.endColumn+\"]\"},e.prototype.setEndPosition=function(t,n){return new e(this.startLineNumber,this.startColumn,t,n)},e.prototype.setStartPosition=function(t,n){return new e(t,n,this.endLineNumber,this.endColumn)},e.prototype.collapseToStart=function(){return e.collapseToStart(this)},e.collapseToStart=function(t){return new e(t.startLineNumber,t.startColumn,t.startLineNumber,t.startColumn)},e.fromPositions=function(t,n){return void 0===n&&(n=t),new e(t.lineNumber,t.column,n.lineNumber,n.column)},e.lift=function(t){return t?new e(t.startLineNumber,t.startColumn,t.endLineNumber,t.endColumn):null},e.isIRange=function(e){return e&&\"number\"==typeof e.startLineNumber&&\"number\"==typeof e.startColumn&&\"number\"==typeof e.endLineNumber&&\"number\"==typeof e.endColumn},e.areIntersectingOrTouching=function(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<t.startColumn)&&!(t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<e.startColumn)},e.areIntersecting=function(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<=t.startColumn)&&!(t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<=e.startColumn)},e.compareRangesUsingStarts=function(e,t){if(e&&t){var n=0|e.startLineNumber,r=0|t.startLineNumber;if(n===r){var i=0|e.startColumn,o=0|t.startColumn;if(i===o){var s=0|e.endLineNumber,a=0|t.endLineNumber;return s===a?(0|e.endColumn)-(0|t.endColumn):s-a}return i-o}return n-r}return(e?1:0)-(t?1:0)},e.compareRangesUsingEnds=function(e,t){return e.endLineNumber===t.endLineNumber?e.endColumn===t.endColumn?e.startLineNumber===t.startLineNumber?e.startColumn-t.startColumn:e.startLineNumber-t.startLineNumber:e.endColumn-t.endColumn:e.endLineNumber-t.endLineNumber},e.spansMultipleLines=function(e){return e.endLineNumber>e.startLineNumber},e}();var le=5e3,ce=3;function he(e,t,n,r){return new W(e,t,n).ComputeDiff(r)}var de=function(){function e(t){for(var n=[],r=[],i=0,o=t.length;i<o;i++)n[i]=e._getFirstNonBlankColumn(t[i],1),r[i]=e._getLastNonBlankColumn(t[i],1);this._lines=t,this._startColumns=n,this._endColumns=r}return e.prototype.getLength=function(){return this._lines.length},e.prototype.getElementAtIndex=function(e){return this._lines[e].substring(this._startColumns[e]-1,this._endColumns[e]-1)},e.prototype.getStartLineNumber=function(e){return e+1},e.prototype.getEndLineNumber=function(e){return e+1},e._getFirstNonBlankColumn=function(e,t){var n=function(e){for(var t=0,n=e.length;t<n;t++){var r=e.charCodeAt(t);if(32!==r&&9!==r)return t}return-1}(e);return-1===n?t:n+1},e._getLastNonBlankColumn=function(e,t){var n=function(e,t){void 0===t&&(t=e.length-1);for(var n=t;n>=0;n--){var r=e.charCodeAt(n);if(32!==r&&9!==r)return n}return-1}(e);return-1===n?t:n+2},e.prototype.getCharSequence=function(e,t,n){for(var r=[],i=[],o=[],s=0,a=t;a<=n;a++)for(var l=this._lines[a],c=e?this._startColumns[a]:1,h=e?this._endColumns[a]:l.length+1,d=c;d<h;d++)r[s]=l.charCodeAt(d-1),i[s]=a+1,o[s]=d,s++;return new ue(r,i,o)},e}(),ue=function(){function e(e,t,n){this._charCodes=e,this._lineNumbers=t,this._columns=n}return e.prototype.getLength=function(){return this._charCodes.length},e.prototype.getElementAtIndex=function(e){return this._charCodes[e]},e.prototype.getStartLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getStartColumn=function(e){return this._columns[e]},e.prototype.getEndLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getEndColumn=function(e){return this._columns[e]+1},e}(),me=function(){function e(e,t,n,r,i,o,s,a){this.originalStartLineNumber=e,this.originalStartColumn=t,this.originalEndLineNumber=n,this.originalEndColumn=r,this.modifiedStartLineNumber=i,this.modifiedStartColumn=o,this.modifiedEndLineNumber=s,this.modifiedEndColumn=a}return e.createFromDiffChange=function(t,n,r){var i,o,s,a,l,c,h,d;return 0===t.originalLength?(i=0,o=0,s=0,a=0):(i=n.getStartLineNumber(t.originalStart),o=n.getStartColumn(t.originalStart),s=n.getEndLineNumber(t.originalStart+t.originalLength-1),a=n.getEndColumn(t.originalStart+t.originalLength-1)),0===t.modifiedLength?(l=0,c=0,h=0,d=0):(l=r.getStartLineNumber(t.modifiedStart),c=r.getStartColumn(t.modifiedStart),h=r.getEndLineNumber(t.modifiedStart+t.modifiedLength-1),d=r.getEndColumn(t.modifiedStart+t.modifiedLength-1)),new e(i,o,s,a,l,c,h,d)},e}();var pe=function(){function e(e,t,n,r,i){this.originalStartLineNumber=e,this.originalEndLineNumber=t,this.modifiedStartLineNumber=n,this.modifiedEndLineNumber=r,this.charChanges=i}return e.createFromDiffResult=function(t,n,r,i,o,s,a){var l,c,h,d,u=void 0;if(0===n.originalLength?(l=r.getStartLineNumber(n.originalStart)-1,c=0):(l=r.getStartLineNumber(n.originalStart),c=r.getEndLineNumber(n.originalStart+n.originalLength-1)),0===n.modifiedLength?(h=i.getStartLineNumber(n.modifiedStart)-1,d=0):(h=i.getStartLineNumber(n.modifiedStart),d=i.getEndLineNumber(n.modifiedStart+n.modifiedLength-1)),s&&0!==n.originalLength&&0!==n.modifiedLength&&o()){var m=r.getCharSequence(t,n.originalStart,n.originalStart+n.originalLength-1),p=i.getCharSequence(t,n.modifiedStart,n.modifiedStart+n.modifiedLength-1),f=he(m,p,o,!0);a&&(f=function(e){if(e.length<=1)return e;for(var t=[e[0]],n=t[0],r=1,i=e.length;r<i;r++){var o=e[r],s=o.originalStart-(n.originalStart+n.originalLength),a=o.modifiedStart-(n.modifiedStart+n.modifiedLength);Math.min(s,a)<ce?(n.originalLength=o.originalStart+o.originalLength-n.originalStart,n.modifiedLength=o.modifiedStart+o.modifiedLength-n.modifiedStart):(t.push(o),n=o)}return t}(f)),u=[];for(var g=0,b=f.length;g<b;g++)u.push(me.createFromDiffChange(f[g],m,p))}return new e(l,c,h,d,u)},e}(),fe=function(){function e(e,t,n){this.shouldComputeCharChanges=n.shouldComputeCharChanges,this.shouldPostProcessCharChanges=n.shouldPostProcessCharChanges,this.shouldIgnoreTrimWhitespace=n.shouldIgnoreTrimWhitespace,this.shouldMakePrettyDiff=n.shouldMakePrettyDiff,this.maximumRunTimeMs=le,this.originalLines=e,this.modifiedLines=t,this.original=new de(e),this.modified=new de(t)}return e.prototype.computeDiff=function(){if(1===this.original.getLength()&&0===this.original.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:1,modifiedStartLineNumber:1,modifiedEndLineNumber:this.modified.getLength(),charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];if(1===this.modified.getLength()&&0===this.modified.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:this.original.getLength(),modifiedStartLineNumber:1,modifiedEndLineNumber:1,charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];this.computationStartTime=(new Date).getTime();var e=he(this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldMakePrettyDiff);if(this.shouldIgnoreTrimWhitespace){for(var t=[],n=0,r=e.length;n<r;n++)t.push(pe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,e[n],this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges));return t}for(var i=[],o=0,s=0,a=(n=-1,e.length);n<a;n++){for(var l=n+1<a?e[n+1]:null,c=l?l.originalStart:this.originalLines.length,h=l?l.modifiedStart:this.modifiedLines.length;o<c&&s<h;){var d=this.originalLines[o],u=this.modifiedLines[s];if(d!==u){for(var m=de._getFirstNonBlankColumn(d,1),p=de._getFirstNonBlankColumn(u,1);m>1&&p>1;){if(d.charCodeAt(m-2)!==u.charCodeAt(p-2))break;m--,p--}(m>1||p>1)&&this._pushTrimWhitespaceCharChange(i,o+1,1,m,s+1,1,p);for(var f=de._getLastNonBlankColumn(d,1),g=de._getLastNonBlankColumn(u,1),b=d.length+1,y=u.length+1;f<b&&g<y;){if(d.charCodeAt(f-1)!==d.charCodeAt(g-1))break;f++,g++}(f<b||g<y)&&this._pushTrimWhitespaceCharChange(i,o+1,f,b,s+1,g,y)}o++,s++}l&&(i.push(pe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,l,this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges)),o+=l.originalLength,s+=l.modifiedLength)}return i},e.prototype._pushTrimWhitespaceCharChange=function(e,t,n,r,i,o,s){if(!this._mergeTrimWhitespaceCharChange(e,t,n,r,i,o,s)){var a=void 0;this.shouldComputeCharChanges&&(a=[new me(t,n,t,r,i,o,i,s)]),e.push(new pe(t,t,i,i,a))}},e.prototype._mergeTrimWhitespaceCharChange=function(e,t,n,r,i,o,s){var a=e.length;if(0===a)return!1;var l=e[a-1];return 0!==l.originalEndLineNumber&&0!==l.modifiedEndLineNumber&&(l.originalEndLineNumber+1===t&&l.modifiedEndLineNumber+1===i&&(l.originalEndLineNumber=t,l.modifiedEndLineNumber=i,this.shouldComputeCharChanges&&l.charChanges.push(new me(t,n,t,r,i,o,i,s)),!0))},e.prototype._continueProcessingPredicate=function(){return 0===this.maximumRunTimeMs||(new Date).getTime()-this.computationStartTime<this.maximumRunTimeMs},e}(),ge=function(){function e(e,t,n){for(var r=new Uint8Array(e*t),i=0,o=e*t;i<o;i++)r[i]=n;this._data=r,this.rows=e,this.cols=t}return e.prototype.get=function(e,t){return this._data[e*this.cols+t]},e.prototype.set=function(e,t,n){this._data[e*this.cols+t]=n},e}();function be(e){return e<0?0:e>255?255:0|e}function ye(e){return e<0?0:e>4294967295?4294967295:0|e}var we=function(){return function(e,t){this.index=e,this.remainder=t}}(),ve=function(){function e(e){this.values=e,this.prefixSum=new Uint32Array(e.length),this.prefixSumValidIndex=new Int32Array(1),this.prefixSumValidIndex[0]=-1}return e.prototype.getCount=function(){return this.values.length},e.prototype.insertValues=function(e,t){e=ye(e);var n=this.values,r=this.prefixSum,i=t.length;return 0!==i&&(this.values=new Uint32Array(n.length+i),this.values.set(n.subarray(0,e),0),this.values.set(n.subarray(e),e+i),this.values.set(t,e),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSum=new Uint32Array(this.values.length),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.changeValue=function(e,t){return e=ye(e),t=ye(t),this.values[e]!==t&&(this.values[e]=t,e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),!0)},e.prototype.removeValues=function(e,t){e=ye(e),t=ye(t);var n=this.values,r=this.prefixSum;if(e>=n.length)return!1;var i=n.length-e;return t>=i&&(t=i),0!==t&&(this.values=new Uint32Array(n.length-t),this.values.set(n.subarray(0,e),0),this.values.set(n.subarray(e+t),e),this.prefixSum=new Uint32Array(this.values.length),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.getTotalValue=function(){return 0===this.values.length?0:this._getAccumulatedValue(this.values.length-1)},e.prototype.getAccumulatedValue=function(e){return e<0?0:(e=ye(e),this._getAccumulatedValue(e))},e.prototype._getAccumulatedValue=function(e){if(e<=this.prefixSumValidIndex[0])return this.prefixSum[e];var t=this.prefixSumValidIndex[0]+1;0===t&&(this.prefixSum[0]=this.values[0],t++),e>=this.values.length&&(e=this.values.length-1);for(var n=t;n<=e;n++)this.prefixSum[n]=this.prefixSum[n-1]+this.values[n];return this.prefixSumValidIndex[0]=Math.max(this.prefixSumValidIndex[0],e),this.prefixSum[e]},e.prototype.getIndexOf=function(e){e=Math.floor(e),this.getTotalValue();for(var t=0,n=this.values.length-1,r=0,i=0,o=0;t<=n;)if(r=t+(n-t)/2|0,e<(o=(i=this.prefixSum[r])-this.values[r]))n=r-1;else{if(!(e>=i))break;t=r+1}return new we(r,e-o)},e}(),xe=function(){function e(e,t,n,r){this._uri=e,this._lines=t,this._eol=n,this._versionId=r,this._lineStarts=null}return e.prototype.dispose=function(){this._lines.length=0},e.prototype.getText=function(){return this._lines.join(this._eol)},e.prototype.onEvents=function(e){e.eol&&e.eol!==this._eol&&(this._eol=e.eol,this._lineStarts=null);for(var t=e.changes,n=0,r=t.length;n<r;n++){var i=t[n];this._acceptDeleteRange(i.range),this._acceptInsertText(new se(i.range.startLineNumber,i.range.startColumn),i.text)}this._versionId=e.versionId},e.prototype._ensureLineStarts=function(){if(!this._lineStarts){for(var e=this._eol.length,t=this._lines.length,n=new Uint32Array(t),r=0;r<t;r++)n[r]=this._lines[r].length+e;this._lineStarts=new ve(n)}},e.prototype._setLineText=function(e,t){this._lines[e]=t,this._lineStarts&&this._lineStarts.changeValue(e,this._lines[e].length+this._eol.length)},e.prototype._acceptDeleteRange=function(e){if(e.startLineNumber!==e.endLineNumber)this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.endLineNumber-1].substring(e.endColumn-1)),this._lines.splice(e.startLineNumber,e.endLineNumber-e.startLineNumber),this._lineStarts&&this._lineStarts.removeValues(e.startLineNumber,e.endLineNumber-e.startLineNumber);else{if(e.startColumn===e.endColumn)return;this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.startLineNumber-1].substring(e.endColumn-1))}},e.prototype._acceptInsertText=function(e,t){if(0!==t.length){var n=t.split(/\\r\\n|\\r|\\n/);if(1!==n.length){n[n.length-1]+=this._lines[e.lineNumber-1].substring(e.column-1),this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+n[0]);for(var r=new Uint32Array(n.length-1),i=1;i<n.length;i++)this._lines.splice(e.lineNumber+i-1,0,n[i]),r[i-1]=n[i].length+this._eol.length;this._lineStarts&&this._lineStarts.insertValues(e.lineNumber,r)}else this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+n[0]+this._lines[e.lineNumber-1].substring(e.column-1))}},e}(),ke=\"`~!@#$%^&*()-=+[{]}\\\\|;:'\\\",.<>/?\";var Ce=function(e){void 0===e&&(e=\"\");for(var t=\"(-?\\\\d*\\\\.\\\\d\\\\w*)|([^\",n=0;n<ke.length;n++)e.indexOf(ke[n])>=0||(t+=\"\\\\\"+ke[n]);return t+=\"\\\\s]+)\",new RegExp(t,\"g\")}();var Se=function(){function e(t){var n=be(t);this._defaultValue=n,this._asciiMap=e._createAsciiMap(n),this._map=new Map}return e._createAsciiMap=function(e){for(var t=new Uint8Array(256),n=0;n<256;n++)t[n]=e;return t},e.prototype.set=function(e,t){var n=be(t);e>=0&&e<256?this._asciiMap[e]=n:this._map.set(e,n)},e.prototype.get=function(e){return e>=0&&e<256?this._asciiMap[e]:this._map.get(e)||this._defaultValue},e}(),_e=function(){function e(e){for(var t=0,n=0,r=0,i=e.length;r<i;r++){var o=e[r],s=o[0];(c=o[1])>t&&(t=c),s>n&&(n=s),(h=o[2])>n&&(n=h)}var a=new ge(++n,++t,0);for(r=0,i=e.length;r<i;r++){var l=e[r],c=(s=l[0],l[1]),h=l[2];a.set(s,c,h)}this._states=a,this._maxCharCode=t}return e.prototype.nextState=function(e,t){return t<0||t>=this._maxCharCode?0:this._states.get(e,t)},e}(),Fe=null;var Ee=null;var Te=function(){function e(){}return e._createLink=function(e,t,n,r,i){var o=i-1;do{var s=t.charCodeAt(o);if(2!==e.get(s))break;o--}while(o>r);if(r>0){var a=t.charCodeAt(r-1),l=t.charCodeAt(o);(40===a&&41===l||91===a&&93===l||123===a&&125===l)&&o--}return{range:{startLineNumber:n,startColumn:r+1,endLineNumber:n,endColumn:o+2},url:t.substring(r,o+1)}},e.computeLinks=function(t){for(var n=(null===Fe&&(Fe=new _e([[1,104,2],[1,72,2],[1,102,6],[1,70,6],[2,116,3],[2,84,3],[3,116,4],[3,84,4],[4,112,5],[4,80,5],[5,115,9],[5,83,9],[5,58,10],[6,105,7],[6,73,7],[7,108,8],[7,76,8],[8,101,9],[8,69,9],[9,58,10],[10,47,11],[11,47,12]])),Fe),r=function(){if(null===Ee){Ee=new Se(0);for(var e=0;e<\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".length;e++)Ee.set(\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".charCodeAt(e),1);for(e=0;e<\".,;\".length;e++)Ee.set(\".,;\".charCodeAt(e),2)}return Ee}(),i=[],o=1,s=t.getLineCount();o<=s;o++){for(var a=t.getLineContent(o),l=a.length,c=0,h=0,d=0,u=1,m=!1,p=!1,f=!1;c<l;){var g=!1,b=a.charCodeAt(c);if(13===u){var y=void 0;switch(b){case 40:m=!0,y=0;break;case 41:y=m?0:1;break;case 91:p=!0,y=0;break;case 93:y=p?0:1;break;case 123:f=!0,y=0;break;case 125:y=f?0:1;break;case 39:y=34===d||96===d?0:1;break;case 34:y=39===d||96===d?0:1;break;case 96:y=39===d||34===d?0:1;break;default:y=r.get(b)}1===y&&(i.push(e._createLink(r,a,o,h,c)),g=!0)}else if(12===u){1===(y=r.get(b))?g=!0:u=13}else 0===(u=n.nextState(u,b))&&(g=!0);g&&(u=1,m=!1,p=!1,f=!1,h=c+1,d=b),c++}13===u&&i.push(e._createLink(r,a,o,h,l))}return i},e}();var Ie,Ae=function(){function e(){this._defaultValueSet=[[\"true\",\"false\"],[\"True\",\"False\"],[\"Private\",\"Public\",\"Friend\",\"ReadOnly\",\"Partial\",\"Protected\",\"WriteOnly\"],[\"public\",\"protected\",\"private\"]]}return e.prototype.navigateValueSet=function(e,t,n,r,i){var o;if(e&&t&&(o=this.doNavigateValueSet(t,i)))return{range:e,value:o};if(n&&r&&(o=this.doNavigateValueSet(r,i)))return{range:n,value:o};return null},e.prototype.doNavigateValueSet=function(e,t){var n=this.numberReplace(e,t);return null!==n?n:this.textReplace(e,t)},e.prototype.numberReplace=function(e,t){var n=Math.pow(10,e.length-(e.lastIndexOf(\".\")+1)),r=Number(e),i=parseFloat(e);return isNaN(r)||isNaN(i)||r!==i?null:0!==r||t?(r=Math.floor(r*n),r+=t?n:-n,String(r/n)):null},e.prototype.textReplace=function(e,t){return this.valueSetsReplace(this._defaultValueSet,e,t)},e.prototype.valueSetsReplace=function(e,t,n){for(var r=null,i=0,o=e.length;null===r&&i<o;i++)r=this.valueSetReplace(e[i],t,n);return r},e.prototype.valueSetReplace=function(e,t,n){var r=e.indexOf(t);return r>=0?((r+=n?1:-1)<0?r=e.length-1:r%=e.length,e[r]):null},e.INSTANCE=new e,e}(),Pe=function(){function e(){this._keyCodeToStr=[],this._strToKeyCode=Object.create(null)}return e.prototype.define=function(e,t){this._keyCodeToStr[e]=t,this._strToKeyCode[t.toLowerCase()]=e},e.prototype.keyCodeToStr=function(e){return this._keyCodeToStr[e]},e.prototype.strToKeyCode=function(e){return this._strToKeyCode[e.toLowerCase()]||0},e}(),Oe=new Pe,Ne=new Pe,Le=new Pe;!function(){function e(e,t,n,r){void 0===n&&(n=t),void 0===r&&(r=n),Oe.define(e,t),Ne.define(e,n),Le.define(e,r)}e(0,\"unknown\"),e(1,\"Backspace\"),e(2,\"Tab\"),e(3,\"Enter\"),e(4,\"Shift\"),e(5,\"Ctrl\"),e(6,\"Alt\"),e(7,\"PauseBreak\"),e(8,\"CapsLock\"),e(9,\"Escape\"),e(10,\"Space\"),e(11,\"PageUp\"),e(12,\"PageDown\"),e(13,\"End\"),e(14,\"Home\"),e(15,\"LeftArrow\",\"Left\"),e(16,\"UpArrow\",\"Up\"),e(17,\"RightArrow\",\"Right\"),e(18,\"DownArrow\",\"Down\"),e(19,\"Insert\"),e(20,\"Delete\"),e(21,\"0\"),e(22,\"1\"),e(23,\"2\"),e(24,\"3\"),e(25,\"4\"),e(26,\"5\"),e(27,\"6\"),e(28,\"7\"),e(29,\"8\"),e(30,\"9\"),e(31,\"A\"),e(32,\"B\"),e(33,\"C\"),e(34,\"D\"),e(35,\"E\"),e(36,\"F\"),e(37,\"G\"),e(38,\"H\"),e(39,\"I\"),e(40,\"J\"),e(41,\"K\"),e(42,\"L\"),e(43,\"M\"),e(44,\"N\"),e(45,\"O\"),e(46,\"P\"),e(47,\"Q\"),e(48,\"R\"),e(49,\"S\"),e(50,\"T\"),e(51,\"U\"),e(52,\"V\"),e(53,\"W\"),e(54,\"X\"),e(55,\"Y\"),e(56,\"Z\"),e(57,\"Meta\"),e(58,\"ContextMenu\"),e(59,\"F1\"),e(60,\"F2\"),e(61,\"F3\"),e(62,\"F4\"),e(63,\"F5\"),e(64,\"F6\"),e(65,\"F7\"),e(66,\"F8\"),e(67,\"F9\"),e(68,\"F10\"),e(69,\"F11\"),e(70,\"F12\"),e(71,\"F13\"),e(72,\"F14\"),e(73,\"F15\"),e(74,\"F16\"),e(75,\"F17\"),e(76,\"F18\"),e(77,\"F19\"),e(78,\"NumLock\"),e(79,\"ScrollLock\"),e(80,\";\",\";\",\"OEM_1\"),e(81,\"=\",\"=\",\"OEM_PLUS\"),e(82,\",\",\",\",\"OEM_COMMA\"),e(83,\"-\",\"-\",\"OEM_MINUS\"),e(84,\".\",\".\",\"OEM_PERIOD\"),e(85,\"/\",\"/\",\"OEM_2\"),e(86,\"`\",\"`\",\"OEM_3\"),e(110,\"ABNT_C1\"),e(111,\"ABNT_C2\"),e(87,\"[\",\"[\",\"OEM_4\"),e(88,\"\\\\\",\"\\\\\",\"OEM_5\"),e(89,\"]\",\"]\",\"OEM_6\"),e(90,\"'\",\"'\",\"OEM_7\"),e(91,\"OEM_8\"),e(92,\"OEM_102\"),e(93,\"NumPad0\"),e(94,\"NumPad1\"),e(95,\"NumPad2\"),e(96,\"NumPad3\"),e(97,\"NumPad4\"),e(98,\"NumPad5\"),e(99,\"NumPad6\"),e(100,\"NumPad7\"),e(101,\"NumPad8\"),e(102,\"NumPad9\"),e(103,\"NumPad_Multiply\"),e(104,\"NumPad_Add\"),e(105,\"NumPad_Separator\"),e(106,\"NumPad_Subtract\"),e(107,\"NumPad_Decimal\"),e(108,\"NumPad_Divide\")}(),function(e){e.toString=function(e){return Oe.keyCodeToStr(e)},e.fromString=function(e){return Oe.strToKeyCode(e)},e.toUserSettingsUS=function(e){return Ne.keyCodeToStr(e)},e.toUserSettingsGeneral=function(e){return Le.keyCodeToStr(e)},e.fromUserSettings=function(e){return Ne.strToKeyCode(e)||Le.strToKeyCode(e)}}(Ie||(Ie={}));var Re,ze,De,Ue,Me,je,Ve,Be,Ke,qe,We,$e,Ge,He,Ye,Je,Xe,Qe,Ze,et,tt,nt,rt,it,ot,st,at,lt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ct=function(e){function t(t,n,r,i){var o=e.call(this,t,n,r,i)||this;return o.selectionStartLineNumber=t,o.selectionStartColumn=n,o.positionLineNumber=r,o.positionColumn=i,o}return lt(t,e),t.prototype.clone=function(){return new t(this.selectionStartLineNumber,this.selectionStartColumn,this.positionLineNumber,this.positionColumn)},t.prototype.toString=function(){return\"[\"+this.selectionStartLineNumber+\",\"+this.selectionStartColumn+\" -> \"+this.positionLineNumber+\",\"+this.positionColumn+\"]\"},t.prototype.equalsSelection=function(e){return t.selectionsEqual(this,e)},t.selectionsEqual=function(e,t){return e.selectionStartLineNumber===t.selectionStartLineNumber&&e.selectionStartColumn===t.selectionStartColumn&&e.positionLineNumber===t.positionLineNumber&&e.positionColumn===t.positionColumn},t.prototype.getDirection=function(){return this.selectionStartLineNumber===this.startLineNumber&&this.selectionStartColumn===this.startColumn?0:1},t.prototype.setEndPosition=function(e,n){return 0===this.getDirection()?new t(this.startLineNumber,this.startColumn,e,n):new t(e,n,this.startLineNumber,this.startColumn)},t.prototype.getPosition=function(){return new se(this.positionLineNumber,this.positionColumn)},t.prototype.setStartPosition=function(e,n){return 0===this.getDirection()?new t(e,n,this.endLineNumber,this.endColumn):new t(this.endLineNumber,this.endColumn,e,n)},t.fromPositions=function(e,n){return void 0===n&&(n=e),new t(e.lineNumber,e.column,n.lineNumber,n.column)},t.liftSelection=function(e){return new t(e.selectionStartLineNumber,e.selectionStartColumn,e.positionLineNumber,e.positionColumn)},t.selectionsArrEqual=function(e,t){if(e&&!t||!e&&t)return!1;if(!e&&!t)return!0;if(e.length!==t.length)return!1;for(var n=0,r=e.length;n<r;n++)if(!this.selectionsEqual(e[n],t[n]))return!1;return!0},t.isISelection=function(e){return e&&\"number\"==typeof e.selectionStartLineNumber&&\"number\"==typeof e.selectionStartColumn&&\"number\"==typeof e.positionLineNumber&&\"number\"==typeof e.positionColumn},t.createWithDirection=function(e,n,r,i,o){return 0===o?new t(e,n,r,i):new t(r,i,e,n)},t}(ae),ht=function(){function e(e,t,n){this.offset=0|e,this.type=t,this.language=n}return e.prototype.toString=function(){return\"(\"+this.offset+\", \"+this.type+\")\"},e}();!function(e){e[e.Unnecessary=1]=\"Unnecessary\"}(Re||(Re={})),function(e){e[e.Hint=1]=\"Hint\",e[e.Info=2]=\"Info\",e[e.Warning=4]=\"Warning\",e[e.Error=8]=\"Error\"}(ze||(ze={})),function(e){e[e.Unknown=0]=\"Unknown\",e[e.Backspace=1]=\"Backspace\",e[e.Tab=2]=\"Tab\",e[e.Enter=3]=\"Enter\",e[e.Shift=4]=\"Shift\",e[e.Ctrl=5]=\"Ctrl\",e[e.Alt=6]=\"Alt\",e[e.PauseBreak=7]=\"PauseBreak\",e[e.CapsLock=8]=\"CapsLock\",e[e.Escape=9]=\"Escape\",e[e.Space=10]=\"Space\",e[e.PageUp=11]=\"PageUp\",e[e.PageDown=12]=\"PageDown\",e[e.End=13]=\"End\",e[e.Home=14]=\"Home\",e[e.LeftArrow=15]=\"LeftArrow\",e[e.UpArrow=16]=\"UpArrow\",e[e.RightArrow=17]=\"RightArrow\",e[e.DownArrow=18]=\"DownArrow\",e[e.Insert=19]=\"Insert\",e[e.Delete=20]=\"Delete\",e[e.KEY_0=21]=\"KEY_0\",e[e.KEY_1=22]=\"KEY_1\",e[e.KEY_2=23]=\"KEY_2\",e[e.KEY_3=24]=\"KEY_3\",e[e.KEY_4=25]=\"KEY_4\",e[e.KEY_5=26]=\"KEY_5\",e[e.KEY_6=27]=\"KEY_6\",e[e.KEY_7=28]=\"KEY_7\",e[e.KEY_8=29]=\"KEY_8\",e[e.KEY_9=30]=\"KEY_9\",e[e.KEY_A=31]=\"KEY_A\",e[e.KEY_B=32]=\"KEY_B\",e[e.KEY_C=33]=\"KEY_C\",e[e.KEY_D=34]=\"KEY_D\",e[e.KEY_E=35]=\"KEY_E\",e[e.KEY_F=36]=\"KEY_F\",e[e.KEY_G=37]=\"KEY_G\",e[e.KEY_H=38]=\"KEY_H\",e[e.KEY_I=39]=\"KEY_I\",e[e.KEY_J=40]=\"KEY_J\",e[e.KEY_K=41]=\"KEY_K\",e[e.KEY_L=42]=\"KEY_L\",e[e.KEY_M=43]=\"KEY_M\",e[e.KEY_N=44]=\"KEY_N\",e[e.KEY_O=45]=\"KEY_O\",e[e.KEY_P=46]=\"KEY_P\",e[e.KEY_Q=47]=\"KEY_Q\",e[e.KEY_R=48]=\"KEY_R\",e[e.KEY_S=49]=\"KEY_S\",e[e.KEY_T=50]=\"KEY_T\",e[e.KEY_U=51]=\"KEY_U\",e[e.KEY_V=52]=\"KEY_V\",e[e.KEY_W=53]=\"KEY_W\",e[e.KEY_X=54]=\"KEY_X\",e[e.KEY_Y=55]=\"KEY_Y\",e[e.KEY_Z=56]=\"KEY_Z\",e[e.Meta=57]=\"Meta\",e[e.ContextMenu=58]=\"ContextMenu\",e[e.F1=59]=\"F1\",e[e.F2=60]=\"F2\",e[e.F3=61]=\"F3\",e[e.F4=62]=\"F4\",e[e.F5=63]=\"F5\",e[e.F6=64]=\"F6\",e[e.F7=65]=\"F7\",e[e.F8=66]=\"F8\",e[e.F9=67]=\"F9\",e[e.F10=68]=\"F10\",e[e.F11=69]=\"F11\",e[e.F12=70]=\"F12\",e[e.F13=71]=\"F13\",e[e.F14=72]=\"F14\",e[e.F15=73]=\"F15\",e[e.F16=74]=\"F16\",e[e.F17=75]=\"F17\",e[e.F18=76]=\"F18\",e[e.F19=77]=\"F19\",e[e.NumLock=78]=\"NumLock\",e[e.ScrollLock=79]=\"ScrollLock\",e[e.US_SEMICOLON=80]=\"US_SEMICOLON\",e[e.US_EQUAL=81]=\"US_EQUAL\",e[e.US_COMMA=82]=\"US_COMMA\",e[e.US_MINUS=83]=\"US_MINUS\",e[e.US_DOT=84]=\"US_DOT\",e[e.US_SLASH=85]=\"US_SLASH\",e[e.US_BACKTICK=86]=\"US_BACKTICK\",e[e.US_OPEN_SQUARE_BRACKET=87]=\"US_OPEN_SQUARE_BRACKET\",e[e.US_BACKSLASH=88]=\"US_BACKSLASH\",e[e.US_CLOSE_SQUARE_BRACKET=89]=\"US_CLOSE_SQUARE_BRACKET\",e[e.US_QUOTE=90]=\"US_QUOTE\",e[e.OEM_8=91]=\"OEM_8\",e[e.OEM_102=92]=\"OEM_102\",e[e.NUMPAD_0=93]=\"NUMPAD_0\",e[e.NUMPAD_1=94]=\"NUMPAD_1\",e[e.NUMPAD_2=95]=\"NUMPAD_2\",e[e.NUMPAD_3=96]=\"NUMPAD_3\",e[e.NUMPAD_4=97]=\"NUMPAD_4\",e[e.NUMPAD_5=98]=\"NUMPAD_5\",e[e.NUMPAD_6=99]=\"NUMPAD_6\",e[e.NUMPAD_7=100]=\"NUMPAD_7\",e[e.NUMPAD_8=101]=\"NUMPAD_8\",e[e.NUMPAD_9=102]=\"NUMPAD_9\",e[e.NUMPAD_MULTIPLY=103]=\"NUMPAD_MULTIPLY\",e[e.NUMPAD_ADD=104]=\"NUMPAD_ADD\",e[e.NUMPAD_SEPARATOR=105]=\"NUMPAD_SEPARATOR\",e[e.NUMPAD_SUBTRACT=106]=\"NUMPAD_SUBTRACT\",e[e.NUMPAD_DECIMAL=107]=\"NUMPAD_DECIMAL\",e[e.NUMPAD_DIVIDE=108]=\"NUMPAD_DIVIDE\",e[e.KEY_IN_COMPOSITION=109]=\"KEY_IN_COMPOSITION\",e[e.ABNT_C1=110]=\"ABNT_C1\",e[e.ABNT_C2=111]=\"ABNT_C2\",e[e.MAX_VALUE=112]=\"MAX_VALUE\"}(De||(De={})),function(e){e[e.LTR=0]=\"LTR\",e[e.RTL=1]=\"RTL\"}(Ue||(Ue={})),function(e){e[e.Auto=1]=\"Auto\",e[e.Hidden=2]=\"Hidden\",e[e.Visible=3]=\"Visible\"}(Me||(Me={})),function(e){e[e.Left=1]=\"Left\",e[e.Center=2]=\"Center\",e[e.Right=4]=\"Right\",e[e.Full=7]=\"Full\"}(je||(je={})),function(e){e[e.TextDefined=0]=\"TextDefined\",e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(Ve||(Ve={})),function(e){e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(Be||(Be={})),function(e){e[e.LF=0]=\"LF\",e[e.CRLF=1]=\"CRLF\"}(Ke||(Ke={})),function(e){e[e.AlwaysGrowsWhenTypingAtEdges=0]=\"AlwaysGrowsWhenTypingAtEdges\",e[e.NeverGrowsWhenTypingAtEdges=1]=\"NeverGrowsWhenTypingAtEdges\",e[e.GrowsOnlyWhenTypingBefore=2]=\"GrowsOnlyWhenTypingBefore\",e[e.GrowsOnlyWhenTypingAfter=3]=\"GrowsOnlyWhenTypingAfter\"}(qe||(qe={})),function(e){e[e.Smooth=0]=\"Smooth\",e[e.Immediate=1]=\"Immediate\"}(We||(We={})),function(e){e[e.NotSet=0]=\"NotSet\",e[e.ContentFlush=1]=\"ContentFlush\",e[e.RecoverFromMarkers=2]=\"RecoverFromMarkers\",e[e.Explicit=3]=\"Explicit\",e[e.Paste=4]=\"Paste\",e[e.Undo=5]=\"Undo\",e[e.Redo=6]=\"Redo\"}($e||($e={})),function(e){e[e.None=0]=\"None\",e[e.Small=1]=\"Small\",e[e.Large=2]=\"Large\",e[e.SmallBlocks=3]=\"SmallBlocks\",e[e.LargeBlocks=4]=\"LargeBlocks\"}(Ge||(Ge={})),function(e){e[e.None=0]=\"None\",e[e.Same=1]=\"Same\",e[e.Indent=2]=\"Indent\",e[e.DeepIndent=3]=\"DeepIndent\"}(He||(He={})),function(e){e[e.Hidden=0]=\"Hidden\",e[e.Blink=1]=\"Blink\",e[e.Smooth=2]=\"Smooth\",e[e.Phase=3]=\"Phase\",e[e.Expand=4]=\"Expand\",e[e.Solid=5]=\"Solid\"}(Ye||(Ye={})),function(e){e[e.Line=1]=\"Line\",e[e.Block=2]=\"Block\",e[e.Underline=3]=\"Underline\",e[e.LineThin=4]=\"LineThin\",e[e.BlockOutline=5]=\"BlockOutline\",e[e.UnderlineThin=6]=\"UnderlineThin\"}(Je||(Je={})),function(e){e[e.Off=0]=\"Off\",e[e.On=1]=\"On\",e[e.Relative=2]=\"Relative\",e[e.Interval=3]=\"Interval\",e[e.Custom=4]=\"Custom\"}(Xe||(Xe={})),function(e){e[e.EXACT=0]=\"EXACT\",e[e.ABOVE=1]=\"ABOVE\",e[e.BELOW=2]=\"BELOW\"}(Qe||(Qe={})),function(e){e[e.TOP_RIGHT_CORNER=0]=\"TOP_RIGHT_CORNER\",e[e.BOTTOM_RIGHT_CORNER=1]=\"BOTTOM_RIGHT_CORNER\",e[e.TOP_CENTER=2]=\"TOP_CENTER\"}(Ze||(Ze={})),function(e){e[e.UNKNOWN=0]=\"UNKNOWN\",e[e.TEXTAREA=1]=\"TEXTAREA\",e[e.GUTTER_GLYPH_MARGIN=2]=\"GUTTER_GLYPH_MARGIN\",e[e.GUTTER_LINE_NUMBERS=3]=\"GUTTER_LINE_NUMBERS\",e[e.GUTTER_LINE_DECORATIONS=4]=\"GUTTER_LINE_DECORATIONS\",e[e.GUTTER_VIEW_ZONE=5]=\"GUTTER_VIEW_ZONE\",e[e.CONTENT_TEXT=6]=\"CONTENT_TEXT\",e[e.CONTENT_EMPTY=7]=\"CONTENT_EMPTY\",e[e.CONTENT_VIEW_ZONE=8]=\"CONTENT_VIEW_ZONE\",e[e.CONTENT_WIDGET=9]=\"CONTENT_WIDGET\",e[e.OVERVIEW_RULER=10]=\"OVERVIEW_RULER\",e[e.SCROLLBAR=11]=\"SCROLLBAR\",e[e.OVERLAY_WIDGET=12]=\"OVERLAY_WIDGET\",e[e.OUTSIDE_EDITOR=13]=\"OUTSIDE_EDITOR\"}(et||(et={})),function(e){e[e.None=0]=\"None\",e[e.Indent=1]=\"Indent\",e[e.IndentOutdent=2]=\"IndentOutdent\",e[e.Outdent=3]=\"Outdent\"}(tt||(tt={})),function(e){e[e.Method=0]=\"Method\",e[e.Function=1]=\"Function\",e[e.Constructor=2]=\"Constructor\",e[e.Field=3]=\"Field\",e[e.Variable=4]=\"Variable\",e[e.Class=5]=\"Class\",e[e.Struct=6]=\"Struct\",e[e.Interface=7]=\"Interface\",e[e.Module=8]=\"Module\",e[e.Property=9]=\"Property\",e[e.Event=10]=\"Event\",e[e.Operator=11]=\"Operator\",e[e.Unit=12]=\"Unit\",e[e.Value=13]=\"Value\",e[e.Constant=14]=\"Constant\",e[e.Enum=15]=\"Enum\",e[e.EnumMember=16]=\"EnumMember\",e[e.Keyword=17]=\"Keyword\",e[e.Text=18]=\"Text\",e[e.Color=19]=\"Color\",e[e.File=20]=\"File\",e[e.Reference=21]=\"Reference\",e[e.Customcolor=22]=\"Customcolor\",e[e.Folder=23]=\"Folder\",e[e.TypeParameter=24]=\"TypeParameter\",e[e.Snippet=25]=\"Snippet\"}(nt||(nt={})),function(e){e[e.KeepWhitespace=1]=\"KeepWhitespace\",e[e.InsertAsSnippet=4]=\"InsertAsSnippet\"}(rt||(rt={})),function(e){e[e.Invoke=0]=\"Invoke\",e[e.TriggerCharacter=1]=\"TriggerCharacter\",e[e.TriggerForIncompleteCompletions=2]=\"TriggerForIncompleteCompletions\"}(it||(it={})),function(e){e[e.Invoke=1]=\"Invoke\",e[e.TriggerCharacter=2]=\"TriggerCharacter\",e[e.ContentChange=3]=\"ContentChange\"}(ot||(ot={})),function(e){e[e.Text=0]=\"Text\",e[e.Read=1]=\"Read\",e[e.Write=2]=\"Write\"}(st||(st={})),function(e){e[e.File=0]=\"File\",e[e.Module=1]=\"Module\",e[e.Namespace=2]=\"Namespace\",e[e.Package=3]=\"Package\",e[e.Class=4]=\"Class\",e[e.Method=5]=\"Method\",e[e.Property=6]=\"Property\",e[e.Field=7]=\"Field\",e[e.Constructor=8]=\"Constructor\",e[e.Enum=9]=\"Enum\",e[e.Interface=10]=\"Interface\",e[e.Function=11]=\"Function\",e[e.Variable=12]=\"Variable\",e[e.Constant=13]=\"Constant\",e[e.String=14]=\"String\",e[e.Number=15]=\"Number\",e[e.Boolean=16]=\"Boolean\",e[e.Array=17]=\"Array\",e[e.Object=18]=\"Object\",e[e.Key=19]=\"Key\",e[e.Null=20]=\"Null\",e[e.EnumMember=21]=\"EnumMember\",e[e.Struct=22]=\"Struct\",e[e.Event=23]=\"Event\",e[e.Operator=24]=\"Operator\",e[e.TypeParameter=25]=\"TypeParameter\"}(at||(at={}));var dt=function(){function e(){}return e.chord=function(e,t){return function(e,t){return(e|(65535&t)<<16>>>0)>>>0}(e,t)},e.CtrlCmd=2048,e.Shift=1024,e.Alt=512,e.WinCtrl=256,e}();var ut=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),mt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ut(t,e),Object.defineProperty(t.prototype,\"uri\",{get:function(){return this._uri},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,\"version\",{get:function(){return this._versionId},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,\"eol\",{get:function(){return this._eol},enumerable:!0,configurable:!0}),t.prototype.getValue=function(){return this.getText()},t.prototype.getLinesContent=function(){return this._lines.slice(0)},t.prototype.getLineCount=function(){return this._lines.length},t.prototype.getLineContent=function(e){return this._lines[e-1]},t.prototype.getWordAtPosition=function(e,t){var n=function(e,t,n,r){t.lastIndex=0;var i=t.exec(n);if(!i)return null;var o=i[0].indexOf(\" \")>=0?function(e,t,n,r){var i,o=e-1-r;for(t.lastIndex=0;i=t.exec(n);){var s=i.index||0;if(s>o)return null;if(t.lastIndex>=o)return{word:i[0],startColumn:r+1+s,endColumn:r+1+t.lastIndex}}return null}(e,t,n,r):function(e,t,n,r){var i,o=e-1-r,s=n.lastIndexOf(\" \",o-1)+1,a=n.indexOf(\" \",o);for(-1===a&&(a=n.length),t.lastIndex=s;i=t.exec(n);){var l=i.index||0;if(l<=o&&t.lastIndex>=o)return{word:i[0],startColumn:r+1+l,endColumn:r+1+t.lastIndex}}return null}(e,t,n,r);return t.lastIndex=0,o}(e.column,function(e){var t=Ce;if(e&&e instanceof RegExp)if(e.global)t=e;else{var n=\"g\";e.ignoreCase&&(n+=\"i\"),e.multiline&&(n+=\"m\"),t=new RegExp(e.source,n)}return t.lastIndex=0,t}(t),this._lines[e.lineNumber-1],0);return n?new ae(e.lineNumber,n.startColumn,e.lineNumber,n.endColumn):null},t.prototype.getWordUntilPosition=function(e,t){var n=this.getWordAtPosition(e,t);return n?{word:this._lines[e.lineNumber-1].substring(n.startColumn-1,e.column-1),startColumn:n.startColumn,endColumn:e.column}:{word:\"\",startColumn:e.column,endColumn:e.column}},t.prototype.createWordIterator=function(e){var t,n,r=this,i=0,o=0,s=[],a=function(){if(o<s.length){var l=n.substring(s[o].start,s[o].end);return o+=1,t?t.value=l:t={done:!1,value:l},t}return i>=r._lines.length?x:(n=r._lines[i],s=r._wordenize(n,e),o=0,i+=1,a())};return{next:a}},t.prototype.getLineWords=function(e,t){for(var n=this._lines[e-1],r=[],i=0,o=this._wordenize(n,t);i<o.length;i++){var s=o[i];r.push({word:n.substring(s.start,s.end),startColumn:s.start+1,endColumn:s.end+1})}return r},t.prototype._wordenize=function(e,t){var n,r=[];for(t.lastIndex=0;(n=t.exec(e))&&0!==n[0].length;)r.push({start:n.index,end:n.index+n[0].length});return r},t.prototype.getValueInRange=function(e){if((e=this._validateRange(e)).startLineNumber===e.endLineNumber)return this._lines[e.startLineNumber-1].substring(e.startColumn-1,e.endColumn-1);var t=this._eol,n=e.startLineNumber-1,r=e.endLineNumber-1,i=[];i.push(this._lines[n].substring(e.startColumn-1));for(var o=n+1;o<r;o++)i.push(this._lines[o]);return i.push(this._lines[r].substring(0,e.endColumn-1)),i.join(t)},t.prototype.offsetAt=function(e){return e=this._validatePosition(e),this._ensureLineStarts(),this._lineStarts.getAccumulatedValue(e.lineNumber-2)+(e.column-1)},t.prototype.positionAt=function(e){e=Math.floor(e),e=Math.max(0,e),this._ensureLineStarts();var t=this._lineStarts.getIndexOf(e),n=this._lines[t.index].length;return{lineNumber:1+t.index,column:1+Math.min(t.remainder,n)}},t.prototype._validateRange=function(e){var t=this._validatePosition({lineNumber:e.startLineNumber,column:e.startColumn}),n=this._validatePosition({lineNumber:e.endLineNumber,column:e.endColumn});return t.lineNumber!==e.startLineNumber||t.column!==e.startColumn||n.lineNumber!==e.endLineNumber||n.column!==e.endColumn?{startLineNumber:t.lineNumber,startColumn:t.column,endLineNumber:n.lineNumber,endColumn:n.column}:e},t.prototype._validatePosition=function(e){if(!se.isIPosition(e))throw new Error(\"bad position\");var t=e.lineNumber,n=e.column,r=!1;if(t<1)t=1,n=1,r=!0;else if(t>this._lines.length)t=this._lines.length,n=this._lines[t-1].length+1,r=!0;else{var i=this._lines[t-1].length+1;n<1?(n=1,r=!0):n>i&&(n=i,r=!0)}return r?{lineNumber:t,column:n}:e},t}(xe),pt=function(e){function t(t){var n=e.call(this,t)||this;return n._models=Object.create(null),n}return ut(t,e),t.prototype.dispose=function(){this._models=Object.create(null)},t.prototype._getModel=function(e){return this._models[e]},t.prototype._getModels=function(){var e=this,t=[];return Object.keys(this._models).forEach(function(n){return t.push(e._models[n])}),t},t.prototype.acceptNewModel=function(e){this._models[e.url]=new mt(Z.parse(e.url),e.lines,e.EOL,e.versionId)},t.prototype.acceptModelChanged=function(e,t){this._models[e]&&this._models[e].onEvents(t)},t.prototype.acceptRemovedModel=function(e){this._models[e]&&delete this._models[e]},t}(function(){function e(e){this._foreignModuleFactory=e,this._foreignModule=null}return e.prototype.computeDiff=function(e,t,n){var r=this._getModel(e),i=this._getModel(t);if(!r||!i)return Promise.resolve(null);var o=r.getLinesContent(),s=i.getLinesContent(),a=new fe(o,s,{shouldComputeCharChanges:!0,shouldPostProcessCharChanges:!0,shouldIgnoreTrimWhitespace:n,shouldMakePrettyDiff:!0}).computeDiff(),l=!(a.length>0)&&this._modelsAreIdentical(r,i);return Promise.resolve({identical:l,changes:a})},e.prototype._modelsAreIdentical=function(e,t){var n=e.getLineCount();if(n!==t.getLineCount())return!1;for(var r=1;r<=n;r++){if(e.getLineContent(r)!==t.getLineContent(r))return!1}return!0},e.prototype.computeMoreMinimalEdits=function(t,n){var r=this._getModel(t);if(!r)return Promise.resolve(n);for(var i=[],o=void 0,s=0,a=n=D(n,function(e,t){return e.range&&t.range?ae.compareRangesUsingStarts(e.range,t.range):(e.range?0:1)-(t.range?0:1)});s<a.length;s++){var l=a[s],c=l.range,h=l.text,d=l.eol;if(\"number\"==typeof d&&(o=d),c){var u=r.getValueInRange(c);if(u!==(h=h.replace(/\\r\\n|\\n|\\r/g,r.eol)))if(Math.max(h.length,u.length)>e._diffLimit)i.push({range:c,text:h});else for(var m=j(u,h,!1),p=r.offsetAt(ae.lift(c).getStartPosition()),f=0,g=m;f<g.length;f++){var b=g[f],y=r.positionAt(p+b.originalStart),w=r.positionAt(p+b.originalStart+b.originalLength),v={text:h.substr(b.modifiedStart,b.modifiedLength),range:{startLineNumber:y.lineNumber,startColumn:y.column,endLineNumber:w.lineNumber,endColumn:w.column}};r.getValueInRange(v.range)!==v.text&&i.push(v)}}}return\"number\"==typeof o&&i.push({eol:o,text:void 0,range:void 0}),Promise.resolve(i)},e.prototype.computeLinks=function(e){var t=this._getModel(e);return t?Promise.resolve(function(e){return e&&\"function\"==typeof e.getLineCount&&\"function\"==typeof e.getLineContent?Te.computeLinks(e):[]}(t)):Promise.resolve(null)},e.prototype.textualSuggest=function(t,n,r,i){var o=this._getModel(t);if(!o)return Promise.resolve(null);var s=[],a=new RegExp(r,i),l=o.getWordUntilPosition(n,a),c=Object.create(null);c[l.word]=!0;for(var h=o.createWordIterator(a),d=h.next();!d.done&&s.length<=e._suggestionsLimit;d=h.next()){var u=d.value;c[u]||(c[u]=!0,isNaN(Number(u))&&s.push({kind:18,label:u,insertText:u,range:{startLineNumber:n.lineNumber,startColumn:l.startColumn,endLineNumber:n.lineNumber,endColumn:l.endColumn}}))}return Promise.resolve({suggestions:s})},e.prototype.computeWordRanges=function(e,t,n,r){var i=this._getModel(e);if(!i)return Promise.resolve(Object.create(null));for(var o=new RegExp(n,r),s=Object.create(null),a=t.startLineNumber;a<t.endLineNumber;a++)for(var l=0,c=i.getLineWords(a,o);l<c.length;l++){var h=c[l];if(isNaN(Number(h.word))){var d=s[h.word];d||(d=[],s[h.word]=d),d.push({startLineNumber:a,startColumn:h.startColumn,endLineNumber:a,endColumn:h.endColumn})}}return Promise.resolve(s)},e.prototype.navigateValueSet=function(e,t,n,r,i){var o=this._getModel(e);if(!o)return Promise.resolve(null);var s=new RegExp(r,i);t.startColumn===t.endColumn&&(t={startLineNumber:t.startLineNumber,startColumn:t.startColumn,endLineNumber:t.endLineNumber,endColumn:t.endColumn+1});var a=o.getValueInRange(t),l=o.getWordAtPosition({lineNumber:t.startLineNumber,column:t.startColumn},s);if(!l)return Promise.resolve(null);var c=o.getValueInRange(l),h=Ae.INSTANCE.navigateValueSet(t,a,l,c,n);return Promise.resolve(h)},e.prototype.loadForeignModule=function(e,t){var n=this,r={getMirrorModels:function(){return n._getModels()}};if(this._foreignModuleFactory){this._foreignModule=this._foreignModuleFactory(r,t);var i=[];for(var o in this._foreignModule)\"function\"==typeof this._foreignModule[o]&&i.push(o);return Promise.resolve(i)}return Promise.reject(new Error(\"Unexpected usage\"))},e.prototype.fmr=function(e,t){if(!this._foreignModule||\"function\"!=typeof this._foreignModule[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._foreignModule[e].apply(this._foreignModule,t))}catch(e){return Promise.reject(e)}},e._diffLimit=1e4,e._suggestionsLimit=1e4,e}());\"function\"==typeof importScripts&&(f.monaco={editor:void 0,languages:void 0,CancellationTokenSource:I,Emitter:_,KeyCode:De,KeyMod:dt,Position:se,Range:ae,Selection:ct,SelectionDirection:Ue,MarkerSeverity:ze,MarkerTag:Re,Promise:n,Uri:Z,Token:ht});var ft,gt=!1;function bt(e){if(!gt){gt=!0;var t=new pt(e),n=new z(function(e){self.postMessage(e)},t);self.onmessage=function(e){n.onmessage(e.data)}}}self.onmessage=function(e){gt||bt(null)},function(e){e[e.Ident=0]=\"Ident\",e[e.AtKeyword=1]=\"AtKeyword\",e[e.String=2]=\"String\",e[e.BadString=3]=\"BadString\",e[e.UnquotedString=4]=\"UnquotedString\",e[e.Hash=5]=\"Hash\",e[e.Num=6]=\"Num\",e[e.Percentage=7]=\"Percentage\",e[e.Dimension=8]=\"Dimension\",e[e.UnicodeRange=9]=\"UnicodeRange\",e[e.CDO=10]=\"CDO\",e[e.CDC=11]=\"CDC\",e[e.Colon=12]=\"Colon\",e[e.SemiColon=13]=\"SemiColon\",e[e.CurlyL=14]=\"CurlyL\",e[e.CurlyR=15]=\"CurlyR\",e[e.ParenthesisL=16]=\"ParenthesisL\",e[e.ParenthesisR=17]=\"ParenthesisR\",e[e.BracketL=18]=\"BracketL\",e[e.BracketR=19]=\"BracketR\",e[e.Whitespace=20]=\"Whitespace\",e[e.Includes=21]=\"Includes\",e[e.Dashmatch=22]=\"Dashmatch\",e[e.SubstringOperator=23]=\"SubstringOperator\",e[e.PrefixOperator=24]=\"PrefixOperator\",e[e.SuffixOperator=25]=\"SuffixOperator\",e[e.Delim=26]=\"Delim\",e[e.EMS=27]=\"EMS\",e[e.EXS=28]=\"EXS\",e[e.Length=29]=\"Length\",e[e.Angle=30]=\"Angle\",e[e.Time=31]=\"Time\",e[e.Freq=32]=\"Freq\",e[e.Exclamation=33]=\"Exclamation\",e[e.Resolution=34]=\"Resolution\",e[e.Comma=35]=\"Comma\",e[e.Charset=36]=\"Charset\",e[e.EscapedJavaScript=37]=\"EscapedJavaScript\",e[e.BadEscapedJavaScript=38]=\"BadEscapedJavaScript\",e[e.Comment=39]=\"Comment\",e[e.SingleLineComment=40]=\"SingleLineComment\",e[e.EOF=41]=\"EOF\",e[e.CustomToken=42]=\"CustomToken\"}(ft||(ft={}));var yt=function(){function e(e){this.source=e,this.len=e.length,this.position=0}return e.prototype.substring=function(e,t){return void 0===t&&(t=this.position),this.source.substring(e,t)},e.prototype.eos=function(){return this.len<=this.position},e.prototype.pos=function(){return this.position},e.prototype.goBackTo=function(e){this.position=e},e.prototype.goBack=function(e){this.position-=e},e.prototype.advance=function(e){this.position+=e},e.prototype.nextChar=function(){return this.source.charCodeAt(this.position++)||0},e.prototype.peekChar=function(e){return void 0===e&&(e=0),this.source.charCodeAt(this.position+e)||0},e.prototype.lookbackChar=function(e){return void 0===e&&(e=0),this.source.charCodeAt(this.position-e)||0},e.prototype.advanceIfChar=function(e){return e===this.source.charCodeAt(this.position)&&(this.position++,!0)},e.prototype.advanceIfChars=function(e){var t;if(this.position+e.length>this.source.length)return!1;for(t=0;t<e.length;t++)if(this.source.charCodeAt(this.position+t)!==e[t])return!1;return this.advance(t),!0},e.prototype.advanceWhileChar=function(e){for(var t=this.position;this.position<this.len&&e(this.source.charCodeAt(this.position));)this.position++;return this.position-t},e}(),wt=\"a\".charCodeAt(0),vt=\"f\".charCodeAt(0),xt=\"z\".charCodeAt(0),kt=\"A\".charCodeAt(0),Ct=\"F\".charCodeAt(0),St=\"Z\".charCodeAt(0),_t=\"0\".charCodeAt(0),Ft=\"9\".charCodeAt(0),Et=\"~\".charCodeAt(0),Tt=\"^\".charCodeAt(0),It=\"=\".charCodeAt(0),At=\"|\".charCodeAt(0),Pt=\"-\".charCodeAt(0),Ot=\"_\".charCodeAt(0),Nt=\"%\".charCodeAt(0),Lt=\"*\".charCodeAt(0),Rt=\"(\".charCodeAt(0),zt=\")\".charCodeAt(0),Dt=\"<\".charCodeAt(0),Ut=\">\".charCodeAt(0),Mt=\"@\".charCodeAt(0),jt=\"#\".charCodeAt(0),Vt=\"$\".charCodeAt(0),Bt=\"\\\\\".charCodeAt(0),Kt=\"/\".charCodeAt(0),qt=\"\\n\".charCodeAt(0),Wt=\"\\r\".charCodeAt(0),$t=\"\\f\".charCodeAt(0),Gt='\"'.charCodeAt(0),Ht=\"'\".charCodeAt(0),Yt=\" \".charCodeAt(0),Jt=\"\\t\".charCodeAt(0),Xt=\";\".charCodeAt(0),Qt=\":\".charCodeAt(0),Zt=\"{\".charCodeAt(0),en=\"}\".charCodeAt(0),tn=\"[\".charCodeAt(0),nn=\"]\".charCodeAt(0),rn=\",\".charCodeAt(0),on=\".\".charCodeAt(0),sn=\"!\".charCodeAt(0),an={};an[Xt]=ft.SemiColon,an[Qt]=ft.Colon,an[Zt]=ft.CurlyL,an[en]=ft.CurlyR,an[nn]=ft.BracketR,an[tn]=ft.BracketL,an[Rt]=ft.ParenthesisL,an[zt]=ft.ParenthesisR,an[rn]=ft.Comma;var ln={};ln.em=ft.EMS,ln.ex=ft.EXS,ln.px=ft.Length,ln.cm=ft.Length,ln.mm=ft.Length,ln.in=ft.Length,ln.pt=ft.Length,ln.pc=ft.Length,ln.deg=ft.Angle,ln.rad=ft.Angle,ln.grad=ft.Angle,ln.ms=ft.Time,ln.s=ft.Time,ln.hz=ft.Freq,ln.khz=ft.Freq,ln[\"%\"]=ft.Percentage,ln.fr=ft.Percentage,ln.dpi=ft.Resolution,ln.dpcm=ft.Resolution;var cn,hn,dn=function(){function e(){this.stream=new yt(\"\"),this.ignoreComment=!0,this.ignoreWhitespace=!0,this.inURL=!1}return e.prototype.setSource=function(e){this.stream=new yt(e)},e.prototype.finishToken=function(e,t,n){return{offset:e,len:this.stream.pos()-e,type:t,text:n||this.stream.substring(e)}},e.prototype.substring=function(e,t){return this.stream.substring(e,e+t)},e.prototype.pos=function(){return this.stream.pos()},e.prototype.goBackTo=function(e){this.stream.goBackTo(e)},e.prototype.scanUnquotedString=function(){var e=this.stream.pos(),t=[];return this._unquotedString(t)?this.finishToken(e,ft.UnquotedString,t.join(\"\")):null},e.prototype.scan=function(){var e=this.trivia();if(null!==e)return e;var t=this.stream.pos();return this.stream.eos()?this.finishToken(t,ft.EOF):this.scanNext(t)},e.prototype.scanNext=function(e){if(this.stream.advanceIfChars([Dt,sn,Pt,Pt]))return this.finishToken(e,ft.CDO);if(this.stream.advanceIfChars([Pt,Pt,Ut]))return this.finishToken(e,ft.CDC);var t=[];if(this.ident(t))return this.finishToken(e,ft.Ident,t.join(\"\"));if(this.stream.advanceIfChar(Mt)){if(t=[\"@\"],this._name(t)){var n=t.join(\"\");return\"@charset\"===n?this.finishToken(e,ft.Charset,n):this.finishToken(e,ft.AtKeyword,n)}return this.finishToken(e,ft.Delim)}if(this.stream.advanceIfChar(jt))return t=[\"#\"],this._name(t)?this.finishToken(e,ft.Hash,t.join(\"\")):this.finishToken(e,ft.Delim);if(this.stream.advanceIfChar(sn))return this.finishToken(e,ft.Exclamation);if(this._number()){var r=this.stream.pos();if(t=[this.stream.substring(e,r)],this.stream.advanceIfChar(Nt))return this.finishToken(e,ft.Percentage);if(this.ident(t)){var i=this.stream.substring(r).toLowerCase(),o=ln[i];return void 0!==o?this.finishToken(e,o,t.join(\"\")):this.finishToken(e,ft.Dimension,t.join(\"\"))}return this.finishToken(e,ft.Num)}t=[];var s=this._string(t);return null!==s?this.finishToken(e,s,t.join(\"\")):void 0!==(s=an[this.stream.peekChar()])?(this.stream.advance(1),this.finishToken(e,s)):this.stream.peekChar(0)===Et&&this.stream.peekChar(1)===It?(this.stream.advance(2),this.finishToken(e,ft.Includes)):this.stream.peekChar(0)===At&&this.stream.peekChar(1)===It?(this.stream.advance(2),this.finishToken(e,ft.Dashmatch)):this.stream.peekChar(0)===Lt&&this.stream.peekChar(1)===It?(this.stream.advance(2),this.finishToken(e,ft.SubstringOperator)):this.stream.peekChar(0)===Tt&&this.stream.peekChar(1)===It?(this.stream.advance(2),this.finishToken(e,ft.PrefixOperator)):this.stream.peekChar(0)===Vt&&this.stream.peekChar(1)===It?(this.stream.advance(2),this.finishToken(e,ft.SuffixOperator)):(this.stream.nextChar(),this.finishToken(e,ft.Delim))},e.prototype._matchWordAnyCase=function(e){var t=0;return this.stream.advanceWhileChar(function(n){var r=e[t]===n||e[t+1]===n;return r&&(t+=2),r}),t===e.length||(this.stream.goBack(t/2),!1)},e.prototype.trivia=function(){for(;;){var e=this.stream.pos();if(this._whitespace()){if(!this.ignoreWhitespace)return this.finishToken(e,ft.Whitespace)}else{if(!this.comment())return null;if(!this.ignoreComment)return this.finishToken(e,ft.Comment)}}},e.prototype.comment=function(){if(this.stream.advanceIfChars([Kt,Lt])){var e=!1,t=!1;return this.stream.advanceWhileChar(function(n){return t&&n===Kt?(e=!0,!1):(t=n===Lt,!0)}),e&&this.stream.advance(1),!0}return!1},e.prototype._number=function(){var e,t=0;return this.stream.peekChar()===on&&(t=1),(e=this.stream.peekChar(t))>=_t&&e<=Ft&&(this.stream.advance(t+1),this.stream.advanceWhileChar(function(e){return e>=_t&&e<=Ft||0===t&&e===on}),!0)},e.prototype._newline=function(e){var t=this.stream.peekChar();switch(t){case Wt:case $t:case qt:return this.stream.advance(1),e.push(String.fromCharCode(t)),t===Wt&&this.stream.advanceIfChar(qt)&&e.push(\"\\n\"),!0}return!1},e.prototype._escape=function(e,t){var n=this.stream.peekChar();if(n===Bt){this.stream.advance(1),n=this.stream.peekChar();for(var r=0;r<6&&(n>=_t&&n<=Ft||n>=wt&&n<=vt||n>=kt&&n<=Ct);)this.stream.advance(1),n=this.stream.peekChar(),r++;if(r>0){try{var i=parseInt(this.stream.substring(this.stream.pos()-r),16);i&&e.push(String.fromCharCode(i))}catch(e){}return n===Yt||n===Jt?this.stream.advance(1):this._newline([]),!0}if(n!==Wt&&n!==$t&&n!==qt)return this.stream.advance(1),e.push(String.fromCharCode(n)),!0;if(t)return this._newline(e)}return!1},e.prototype._stringChar=function(e,t){var n=this.stream.peekChar();return 0!==n&&n!==e&&n!==Bt&&n!==Wt&&n!==$t&&n!==qt&&(this.stream.advance(1),t.push(String.fromCharCode(n)),!0)},e.prototype._string=function(e){if(this.stream.peekChar()===Ht||this.stream.peekChar()===Gt){var t=this.stream.nextChar();for(e.push(String.fromCharCode(t));this._stringChar(t,e)||this._escape(e,!0););return this.stream.peekChar()===t?(this.stream.nextChar(),e.push(String.fromCharCode(t)),ft.String):ft.BadString}return null},e.prototype._unquotedChar=function(e){var t=this.stream.peekChar();return 0!==t&&t!==Bt&&t!==Ht&&t!==Gt&&t!==Rt&&t!==zt&&t!==Yt&&t!==Jt&&t!==qt&&t!==$t&&t!==Wt&&(this.stream.advance(1),e.push(String.fromCharCode(t)),!0)},e.prototype._unquotedString=function(e){for(var t=!1;this._unquotedChar(e)||this._escape(e);)t=!0;return t},e.prototype._whitespace=function(){return this.stream.advanceWhileChar(function(e){return e===Yt||e===Jt||e===qt||e===$t||e===Wt})>0},e.prototype._name=function(e){for(var t=!1;this._identChar(e)||this._escape(e);)t=!0;return t},e.prototype.ident=function(e){var t=this.stream.pos();if(this._minus(e)&&this._minus(e)){if(this._identFirstChar(e)||this._escape(e)){for(;this._identChar(e)||this._escape(e););return!0}}else if(this._identFirstChar(e)||this._escape(e)){for(;this._identChar(e)||this._escape(e););return!0}return this.stream.goBackTo(t),!1},e.prototype._identFirstChar=function(e){var t=this.stream.peekChar();return(t===Ot||t>=wt&&t<=xt||t>=kt&&t<=St||t>=128&&t<=65535)&&(this.stream.advance(1),e.push(String.fromCharCode(t)),!0)},e.prototype._minus=function(e){var t=this.stream.peekChar();return t===Pt&&(this.stream.advance(1),e.push(String.fromCharCode(t)),!0)},e.prototype._identChar=function(e){var t=this.stream.peekChar();return(t===Ot||t===Pt||t>=wt&&t<=xt||t>=kt&&t<=St||t>=_t&&t<=Ft||t>=128&&t<=65535)&&(this.stream.advance(1),e.push(String.fromCharCode(t)),!0)},e}(),un=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();function mn(e,t){var n=null;return!e||t<e.offset||t>e.end?null:(e.accept(function(e){return-1===e.offset&&-1===e.length||e.offset<=t&&e.end>=t&&(n?e.length<=n.length&&(n=e):n=e,!0)}),n)}function pn(e,t){for(var n=mn(e,t),r=[];n;)r.unshift(n),n=n.parent;return r}!function(e){e[e.Undefined=0]=\"Undefined\",e[e.Identifier=1]=\"Identifier\",e[e.Stylesheet=2]=\"Stylesheet\",e[e.Ruleset=3]=\"Ruleset\",e[e.Selector=4]=\"Selector\",e[e.SimpleSelector=5]=\"SimpleSelector\",e[e.SelectorInterpolation=6]=\"SelectorInterpolation\",e[e.SelectorCombinator=7]=\"SelectorCombinator\",e[e.SelectorCombinatorParent=8]=\"SelectorCombinatorParent\",e[e.SelectorCombinatorSibling=9]=\"SelectorCombinatorSibling\",e[e.SelectorCombinatorAllSiblings=10]=\"SelectorCombinatorAllSiblings\",e[e.SelectorCombinatorShadowPiercingDescendant=11]=\"SelectorCombinatorShadowPiercingDescendant\",e[e.Page=12]=\"Page\",e[e.PageBoxMarginBox=13]=\"PageBoxMarginBox\",e[e.ClassSelector=14]=\"ClassSelector\",e[e.IdentifierSelector=15]=\"IdentifierSelector\",e[e.ElementNameSelector=16]=\"ElementNameSelector\",e[e.PseudoSelector=17]=\"PseudoSelector\",e[e.AttributeSelector=18]=\"AttributeSelector\",e[e.Declaration=19]=\"Declaration\",e[e.Declarations=20]=\"Declarations\",e[e.Property=21]=\"Property\",e[e.Expression=22]=\"Expression\",e[e.BinaryExpression=23]=\"BinaryExpression\",e[e.Term=24]=\"Term\",e[e.Operator=25]=\"Operator\",e[e.Value=26]=\"Value\",e[e.StringLiteral=27]=\"StringLiteral\",e[e.URILiteral=28]=\"URILiteral\",e[e.EscapedValue=29]=\"EscapedValue\",e[e.Function=30]=\"Function\",e[e.NumericValue=31]=\"NumericValue\",e[e.HexColorValue=32]=\"HexColorValue\",e[e.MixinDeclaration=33]=\"MixinDeclaration\",e[e.MixinReference=34]=\"MixinReference\",e[e.VariableName=35]=\"VariableName\",e[e.VariableDeclaration=36]=\"VariableDeclaration\",e[e.Prio=37]=\"Prio\",e[e.Interpolation=38]=\"Interpolation\",e[e.NestedProperties=39]=\"NestedProperties\",e[e.ExtendsReference=40]=\"ExtendsReference\",e[e.SelectorPlaceholder=41]=\"SelectorPlaceholder\",e[e.Debug=42]=\"Debug\",e[e.If=43]=\"If\",e[e.Else=44]=\"Else\",e[e.For=45]=\"For\",e[e.Each=46]=\"Each\",e[e.While=47]=\"While\",e[e.MixinContent=48]=\"MixinContent\",e[e.Media=49]=\"Media\",e[e.Keyframe=50]=\"Keyframe\",e[e.FontFace=51]=\"FontFace\",e[e.Import=52]=\"Import\",e[e.Namespace=53]=\"Namespace\",e[e.Invocation=54]=\"Invocation\",e[e.FunctionDeclaration=55]=\"FunctionDeclaration\",e[e.ReturnStatement=56]=\"ReturnStatement\",e[e.MediaQuery=57]=\"MediaQuery\",e[e.FunctionParameter=58]=\"FunctionParameter\",e[e.FunctionArgument=59]=\"FunctionArgument\",e[e.KeyframeSelector=60]=\"KeyframeSelector\",e[e.ViewPort=61]=\"ViewPort\",e[e.Document=62]=\"Document\",e[e.AtApplyRule=63]=\"AtApplyRule\",e[e.CustomPropertyDeclaration=64]=\"CustomPropertyDeclaration\",e[e.CustomPropertySet=65]=\"CustomPropertySet\",e[e.ListEntry=66]=\"ListEntry\",e[e.Supports=67]=\"Supports\",e[e.SupportsCondition=68]=\"SupportsCondition\",e[e.NamespacePrefix=69]=\"NamespacePrefix\",e[e.GridLine=70]=\"GridLine\",e[e.Plugin=71]=\"Plugin\",e[e.UnknownAtRule=72]=\"UnknownAtRule\"}(cn||(cn={})),function(e){e[e.Mixin=0]=\"Mixin\",e[e.Rule=1]=\"Rule\",e[e.Variable=2]=\"Variable\",e[e.Function=3]=\"Function\",e[e.Keyframe=4]=\"Keyframe\",e[e.Unknown=5]=\"Unknown\"}(hn||(hn={}));var fn,gn=function(){function e(e,t,n){void 0===e&&(e=-1),void 0===t&&(t=-1),this.parent=null,this.offset=e,this.length=t,n&&(this.nodeType=n)}return Object.defineProperty(e.prototype,\"end\",{get:function(){return this.offset+this.length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"type\",{get:function(){return this.nodeType||cn.Undefined},set:function(e){this.nodeType=e},enumerable:!0,configurable:!0}),e.prototype.getTextProvider=function(){for(var e=this;e&&!e.textProvider;)e=e.parent;return e?e.textProvider:function(){return\"unknown\"}},e.prototype.getText=function(){return this.getTextProvider()(this.offset,this.length)},e.prototype.matches=function(e){return this.length===e.length&&this.getTextProvider()(this.offset,this.length)===e},e.prototype.startsWith=function(e){return this.length>=e.length&&this.getTextProvider()(this.offset,e.length)===e},e.prototype.endsWith=function(e){return this.length>=e.length&&this.getTextProvider()(this.end-e.length,e.length)===e},e.prototype.accept=function(e){if(e(this)&&this.children)for(var t=0,n=this.children;t<n.length;t++){n[t].accept(e)}},e.prototype.acceptVisitor=function(e){this.accept(e.visitNode.bind(e))},e.prototype.adoptChild=function(e,t){if(void 0===t&&(t=-1),e.parent&&e.parent.children){var n=e.parent.children.indexOf(e);n>=0&&e.parent.children.splice(n,1)}e.parent=this;var r=this.children;return r||(r=this.children=[]),-1!==t?r.splice(t,0,e):r.push(e),e},e.prototype.attachTo=function(e,t){return void 0===t&&(t=-1),e&&e.adoptChild(this,t),this},e.prototype.collectIssues=function(e){this.issues&&e.push.apply(e,this.issues)},e.prototype.addIssue=function(e){this.issues||(this.issues=[]),this.issues.push(e)},e.prototype.hasIssue=function(e){return Array.isArray(this.issues)&&this.issues.some(function(t){return t.getRule()===e})},e.prototype.isErroneous=function(e){return void 0===e&&(e=!1),!!(this.issues&&this.issues.length>0)||e&&Array.isArray(this.children)&&this.children.some(function(e){return e.isErroneous(!0)})},e.prototype.setNode=function(e,t,n){return void 0===n&&(n=-1),!!t&&(t.attachTo(this,n),this[e]=t,!0)},e.prototype.addChild=function(e){return!!e&&(this.children||(this.children=[]),e.attachTo(this),this.updateOffsetAndLength(e),!0)},e.prototype.updateOffsetAndLength=function(e){(e.offset<this.offset||-1===this.offset)&&(this.offset=e.offset);var t=e.end;(t>this.end||-1===this.length)&&(this.length=t-this.offset)},e.prototype.hasChildren=function(){return this.children&&this.children.length>0},e.prototype.getChildren=function(){return this.children?this.children.slice(0):[]},e.prototype.getChild=function(e){return this.children&&e<this.children.length?this.children[e]:null},e.prototype.addChildren=function(e){for(var t=0,n=e;t<n.length;t++){var r=n[t];this.addChild(r)}},e.prototype.findFirstChildBeforeOffset=function(e){if(this.children)for(var t=null,n=this.children.length-1;n>=0;n--)if((t=this.children[n]).offset<=e)return t;return null},e.prototype.findChildAtOffset=function(e,t){var n=this.findFirstChildBeforeOffset(e);return n&&n.end>=e?t&&n.findChildAtOffset(e,!0)||n:null},e.prototype.encloses=function(e){return this.offset<=e.offset&&this.offset+this.length>=e.offset+e.length},e.prototype.getParent=function(){for(var e=this.parent;e instanceof bn;)e=e.parent;return e},e.prototype.findParent=function(e){for(var t=this;t&&t.type!==e;)t=t.parent;return t},e.prototype.setData=function(e,t){this.options||(this.options={}),this.options[e]=t},e.prototype.getData=function(e){return this.options&&this.options.hasOwnProperty(e)?this.options[e]:null},e}(),bn=function(e){function t(t,n){void 0===n&&(n=-1);var r=e.call(this,-1,-1)||this;return r.attachTo(t,n),r.offset=-1,r.length=-1,r}return un(t,e),t}(gn),yn=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.isCustomProperty=!1,r}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Identifier},enumerable:!0,configurable:!0}),t.prototype.containsInterpolation=function(){return this.hasChildren()},t}(gn),wn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Stylesheet},enumerable:!0,configurable:!0}),t.prototype.setName=function(e){this.name=e},t}(gn),vn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Declarations},enumerable:!0,configurable:!0}),t}(gn),xn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),t.prototype.getDeclarations=function(){return this.declarations},t.prototype.setDeclarations=function(e){return this.setNode(\"declarations\",e)},t}(gn),kn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Ruleset},enumerable:!0,configurable:!0}),t.prototype.getSelectors=function(){return this.selectors||(this.selectors=new bn(this)),this.selectors},t.prototype.isNested=function(){return!!this.parent&&null!==this.parent.findParent(cn.Declarations)},t}(xn),Cn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Selector},enumerable:!0,configurable:!0}),t}(gn),Sn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.SimpleSelector},enumerable:!0,configurable:!0}),t}(gn),_n=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.AtApplyRule},enumerable:!0,configurable:!0}),t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t}(gn),Fn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),t}(gn),En=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.CustomPropertyDeclaration},enumerable:!0,configurable:!0}),t.prototype.setProperty=function(e){return this.setNode(\"property\",e)},t.prototype.getProperty=function(){return this.property},t.prototype.setValue=function(e){return this.setNode(\"value\",e)},t.prototype.getValue=function(){return this.value},t.prototype.setPropertySet=function(e){return this.setNode(\"propertySet\",e)},t.prototype.getPropertySet=function(){return this.propertySet},t}(Fn),Tn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.CustomPropertySet},enumerable:!0,configurable:!0}),t}(xn),In=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Declaration},enumerable:!0,configurable:!0}),t.prototype.setProperty=function(e){return this.setNode(\"property\",e)},t.prototype.getProperty=function(){return this.property},t.prototype.getFullPropertyName=function(){var e=this.property?this.property.getName():\"unknown\";if(this.parent instanceof vn&&this.parent.getParent()instanceof Bn){var n=this.parent.getParent().getParent();if(n instanceof t)return n.getFullPropertyName()+e}return e},t.prototype.getNonPrefixedPropertyName=function(){var e=this.getFullPropertyName();if(e&&\"-\"===e.charAt(0)){var t=e.indexOf(\"-\",1);if(-1!==t)return e.substring(t+1)}return e},t.prototype.setValue=function(e){return this.setNode(\"value\",e)},t.prototype.getValue=function(){return this.value},t.prototype.setNestedProperties=function(e){return this.setNode(\"nestedProprties\",e)},t.prototype.getNestedProperties=function(){return this.nestedProprties},t}(Fn),An=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Property},enumerable:!0,configurable:!0}),t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.getText()},t.prototype.isCustomProperty=function(){return this.identifier.isCustomProperty},t}(gn),Pn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Function},enumerable:!0,configurable:!0}),t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t}(function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Invocation},enumerable:!0,configurable:!0}),t.prototype.getArguments=function(){return this.arguments||(this.arguments=new bn(this)),this.arguments},t}(gn)),On=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.FunctionParameter},enumerable:!0,configurable:!0}),t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t.prototype.setDefaultValue=function(e){return this.setNode(\"defaultValue\",e,0)},t.prototype.getDefaultValue=function(){return this.defaultValue},t}(gn),Nn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.FunctionArgument},enumerable:!0,configurable:!0}),t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t.prototype.setValue=function(e){return this.setNode(\"value\",e,0)},t.prototype.getValue=function(){return this.value},t}(gn),Ln=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.If},enumerable:!0,configurable:!0}),t.prototype.setExpression=function(e){return this.setNode(\"expression\",e,0)},t.prototype.setElseClause=function(e){return this.setNode(\"elseClause\",e)},t}(xn),Rn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.For},enumerable:!0,configurable:!0}),t.prototype.setVariable=function(e){return this.setNode(\"variable\",e,0)},t}(xn),zn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Each},enumerable:!0,configurable:!0}),t.prototype.getVariables=function(){return this.variables||(this.variables=new bn(this)),this.variables},t}(xn),Dn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.While},enumerable:!0,configurable:!0}),t}(xn),Un=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Else},enumerable:!0,configurable:!0}),t}(xn),Mn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.FunctionDeclaration},enumerable:!0,configurable:!0}),t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t.prototype.getParameters=function(){return this.parameters||(this.parameters=new bn(this)),this.parameters},t}(xn),jn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.ViewPort},enumerable:!0,configurable:!0}),t}(xn),Vn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.FontFace},enumerable:!0,configurable:!0}),t}(xn),Bn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.NestedProperties},enumerable:!0,configurable:!0}),t}(xn),Kn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Keyframe},enumerable:!0,configurable:!0}),t.prototype.setKeyword=function(e){return this.setNode(\"keyword\",e,0)},t.prototype.getKeyword=function(){return this.keyword},t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t}(xn),qn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.KeyframeSelector},enumerable:!0,configurable:!0}),t}(xn),Wn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Import},enumerable:!0,configurable:!0}),t.prototype.setMedialist=function(e){return!!e&&(e.attachTo(this),this.medialist=e,!0)},t}(gn),$n=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Namespace},enumerable:!0,configurable:!0}),t}(gn),Gn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Media},enumerable:!0,configurable:!0}),t}(xn),Hn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Supports},enumerable:!0,configurable:!0}),t}(xn),Yn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Document},enumerable:!0,configurable:!0}),t}(xn),Jn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),t.prototype.getMediums=function(){return this.mediums||(this.mediums=new bn(this)),this.mediums},t}(gn),Xn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.MediaQuery},enumerable:!0,configurable:!0}),t}(gn),Qn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.SupportsCondition},enumerable:!0,configurable:!0}),t}(gn),Zn=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Page},enumerable:!0,configurable:!0}),t}(xn),er=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.PageBoxMarginBox},enumerable:!0,configurable:!0}),t}(xn),tr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Expression},enumerable:!0,configurable:!0}),t}(gn),nr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.BinaryExpression},enumerable:!0,configurable:!0}),t.prototype.setLeft=function(e){return this.setNode(\"left\",e)},t.prototype.getLeft=function(){return this.left},t.prototype.setRight=function(e){return this.setNode(\"right\",e)},t.prototype.getRight=function(){return this.right},t.prototype.setOperator=function(e){return this.setNode(\"operator\",e)},t.prototype.getOperator=function(){return this.operator},t}(gn),rr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Term},enumerable:!0,configurable:!0}),t.prototype.setOperator=function(e){return this.setNode(\"operator\",e)},t.prototype.getOperator=function(){return this.operator},t.prototype.setExpression=function(e){return this.setNode(\"expression\",e)},t.prototype.getExpression=function(){return this.expression},t}(gn),ir=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.AttributeSelector},enumerable:!0,configurable:!0}),t.prototype.setNamespacePrefix=function(e){return this.setNode(\"namespacePrefix\",e)},t.prototype.getNamespacePrefix=function(){return this.namespacePrefix},t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.setOperator=function(e){return this.setNode(\"operator\",e)},t.prototype.getOperator=function(){return this.operator},t.prototype.setValue=function(e){return this.setNode(\"value\",e)},t.prototype.getValue=function(){return this.value},t}(gn),or=(function(e){function t(t,n){return e.call(this,t,n)||this}un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Operator},enumerable:!0,configurable:!0})}(gn),function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.HexColorValue},enumerable:!0,configurable:!0}),t}(gn)),sr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.NumericValue},enumerable:!0,configurable:!0}),t.prototype.getValue=function(){for(var e,t=this.getText(),n=0,r=\".\".charCodeAt(0),i=\"0\".charCodeAt(0),o=\"9\".charCodeAt(0),s=0,a=t.length;s<a&&(i<=(e=t.charCodeAt(s))&&e<=o||e===r);s++)n+=1;return{value:t.substring(0,n),unit:n<t.length?t.substring(n):void 0}},t}(gn),ar=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.VariableDeclaration},enumerable:!0,configurable:!0}),t.prototype.setVariable=function(e){return!!e&&(e.attachTo(this),this.variable=e,!0)},t.prototype.getVariable=function(){return this.variable},t.prototype.getName=function(){return this.variable?this.variable.getName():\"\"},t.prototype.setValue=function(e){return!!e&&(e.attachTo(this),this.value=e,!0)},t.prototype.getValue=function(){return this.value},t}(Fn),lr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.Interpolation},enumerable:!0,configurable:!0}),t}(gn),cr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.VariableName},enumerable:!0,configurable:!0}),t.prototype.getName=function(){return this.getText()},t}(gn),hr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.ExtendsReference},enumerable:!0,configurable:!0}),t.prototype.getSelectors=function(){return this.selectors||(this.selectors=new bn(this)),this.selectors},t}(gn),dr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.MixinReference},enumerable:!0,configurable:!0}),t.prototype.getNamespaces=function(){return this.namespaces||(this.namespaces=new bn(this)),this.namespaces},t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t.prototype.getArguments=function(){return this.arguments||(this.arguments=new bn(this)),this.arguments},t.prototype.setContent=function(e){return this.setNode(\"content\",e)},t.prototype.getContent=function(){return this.content},t}(gn),ur=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.MixinDeclaration},enumerable:!0,configurable:!0}),t.prototype.setIdentifier=function(e){return this.setNode(\"identifier\",e,0)},t.prototype.getIdentifier=function(){return this.identifier},t.prototype.getName=function(){return this.identifier?this.identifier.getText():\"\"},t.prototype.getParameters=function(){return this.parameters||(this.parameters=new bn(this)),this.parameters},t.prototype.setGuard=function(e){return e&&(e.attachTo(this),this.guard=e),!1},t}(xn),mr=function(e){function t(t,n){return e.call(this,t,n)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.UnknownAtRule},enumerable:!0,configurable:!0}),t.prototype.setAtRuleName=function(e){this.atRuleName=e},t.prototype.getAtRuleName=function(e){return this.atRuleName},t}(xn),pr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return un(t,e),Object.defineProperty(t.prototype,\"type\",{get:function(){return cn.ListEntry},enumerable:!0,configurable:!0}),t.prototype.setKey=function(e){return this.setNode(\"key\",e,0)},t.prototype.setValue=function(e){return this.setNode(\"value\",e,1)},t}(gn),fr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return un(t,e),t.prototype.getConditions=function(){return this.conditions||(this.conditions=new bn(this)),this.conditions},t}(gn),gr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return un(t,e),t.prototype.setVariable=function(e){return this.setNode(\"variable\",e)},t}(gn);!function(e){e[e.Ignore=1]=\"Ignore\",e[e.Warning=2]=\"Warning\",e[e.Error=4]=\"Error\"}(fn||(fn={}));var br=function(){function e(e,t,n,r,i,o){void 0===i&&(i=e.offset),void 0===o&&(o=e.length),this.node=e,this.rule=t,this.level=n,this.message=r||t.message,this.offset=i,this.length=o}return e.prototype.getRule=function(){return this.rule},e.prototype.getLevel=function(){return this.level},e.prototype.getOffset=function(){return this.offset},e.prototype.getLength=function(){return this.length},e.prototype.getNode=function(){return this.node},e.prototype.getMessage=function(){return this.message},e}(),yr=function(){function e(){this.entries=[]}return e.entries=function(t){var n=new e;return t.acceptVisitor(n),n.entries},e.prototype.visitNode=function(e){return e.isErroneous()&&e.collectIssues(this.entries),!0},e}();function wr(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];return function(e,t){return 0===t.length?e:e.replace(/\\{(\\d+)\\}/g,function(e,n){var r=n[0];return void 0!==t[r]?t[r]:e})}(t,n)}function vr(e){return wr}var xr=vr(),kr=function(){return function(e,t){this.id=e,this.message=t}}(),Cr={NumberExpected:new kr(\"css-numberexpected\",xr(\"expected.number\",\"number expected\")),ConditionExpected:new kr(\"css-conditionexpected\",xr(\"expected.condt\",\"condition expected\")),RuleOrSelectorExpected:new kr(\"css-ruleorselectorexpected\",xr(\"expected.ruleorselector\",\"at-rule or selector expected\")),DotExpected:new kr(\"css-dotexpected\",xr(\"expected.dot\",\"dot expected\")),ColonExpected:new kr(\"css-colonexpected\",xr(\"expected.colon\",\"colon expected\")),SemiColonExpected:new kr(\"css-semicolonexpected\",xr(\"expected.semicolon\",\"semi-colon expected\")),TermExpected:new kr(\"css-termexpected\",xr(\"expected.term\",\"term expected\")),ExpressionExpected:new kr(\"css-expressionexpected\",xr(\"expected.expression\",\"expression expected\")),OperatorExpected:new kr(\"css-operatorexpected\",xr(\"expected.operator\",\"operator expected\")),IdentifierExpected:new kr(\"css-identifierexpected\",xr(\"expected.ident\",\"identifier expected\")),PercentageExpected:new kr(\"css-percentageexpected\",xr(\"expected.percentage\",\"percentage expected\")),URIOrStringExpected:new kr(\"css-uriorstringexpected\",xr(\"expected.uriorstring\",\"uri or string expected\")),URIExpected:new kr(\"css-uriexpected\",xr(\"expected.uri\",\"URI expected\")),VariableNameExpected:new kr(\"css-varnameexpected\",xr(\"expected.varname\",\"variable name expected\")),VariableValueExpected:new kr(\"css-varvalueexpected\",xr(\"expected.varvalue\",\"variable value expected\")),PropertyValueExpected:new kr(\"css-propertyvalueexpected\",xr(\"expected.propvalue\",\"property value expected\")),LeftCurlyExpected:new kr(\"css-lcurlyexpected\",xr(\"expected.lcurly\",\"{ expected\")),RightCurlyExpected:new kr(\"css-rcurlyexpected\",xr(\"expected.rcurly\",\"} expected\")),LeftSquareBracketExpected:new kr(\"css-rbracketexpected\",xr(\"expected.lsquare\",\"[ expected\")),RightSquareBracketExpected:new kr(\"css-lbracketexpected\",xr(\"expected.rsquare\",\"] expected\")),LeftParenthesisExpected:new kr(\"css-lparentexpected\",xr(\"expected.lparen\",\"( expected\")),RightParenthesisExpected:new kr(\"css-rparentexpected\",xr(\"expected.rparent\",\") expected\")),CommaExpected:new kr(\"css-commaexpected\",xr(\"expected.comma\",\"comma expected\")),PageDirectiveOrDeclarationExpected:new kr(\"css-pagedirordeclexpected\",xr(\"expected.pagedirordecl\",\"page directive or declaraton expected\")),UnknownAtRule:new kr(\"css-unknownatrule\",xr(\"unknown.atrule\",\"at-rule unknown\")),UnknownKeyword:new kr(\"css-unknownkeyword\",xr(\"unknown.keyword\",\"unknown keyword\")),SelectorExpected:new kr(\"css-selectorexpected\",xr(\"expected.selector\",\"selector expected\")),StringLiteralExpected:new kr(\"css-stringliteralexpected\",xr(\"expected.stringliteral\",\"string literal expected\")),WhitespaceExpected:new kr(\"css-whitespaceexpected\",xr(\"expected.whitespace\",\"whitespace expected\")),MediaQueryExpected:new kr(\"css-mediaqueryexpected\",xr(\"expected.mediaquery\",\"media query expected\"))},Sr={atdirectives:[{name:\"@charset\",desc:\"Defines character set of the document.\",browsers:\"FF1.5,S4,C2,IE5.5,O9\"},{name:\"@counter-style\",desc:\"Defines a custom counter style.\",browsers:\"FF33\"},{name:\"@font-face\",desc:\"Allows for linking to fonts that are automatically activated when needed. This permits authors to work around the limitation of 'web-safe' fonts, allowing for consistent rendering independent of the fonts available in a given user's environment.\",browsers:\"all\"},{name:\"@font-feature-values\",desc:\"Defines named values for the indices used to select alternate glyphs for a given font family.\",browsers:\"FF34,S9.1\"},{name:\"@import\",desc:\"Includes content of another file.\",browsers:\"all\"},{name:\"@keyframes\",desc:\"Defines set of animation key frames.\",browsers:\"all\"},{name:\"@media\",desc:\"Defines a stylesheet for a particular media type.\",browsers:\"all\"},{name:\"@-moz-document\",desc:\"Gecko-specific at-rule that restricts the style rules contained within it based on the URL of the document.\",browsers:\"FF1.8\"},{name:\"@-moz-keyframes\",desc:\"Defines set of animation key frames.\",browsers:\"FF5\"},{name:\"@-ms-viewport\",desc:\"Specifies the size, zoom factor, and orientation of the viewport.\",browsers:\"E,IE10\"},{name:\"@namespace\",desc:\"Declares a prefix and associates it with a namespace name.\",browsers:\"all\"},{name:\"@-o-keyframes\",desc:\"Defines set of animation key frames.\",browsers:\"O12\"},{name:\"@-o-viewport\",desc:\"Specifies the size, zoom factor, and orientation of the viewport.\",browsers:\"O11\"},{name:\"@page\",desc:\"Directive defines various page parameters.\",browsers:\"E,FF19,C2,IE8,O6\"},{name:\"@supports\",desc:\"A conditional group rule whose condition tests whether the user agent supports CSS property:value pairs.\",browsers:\"E12,FF22,S9,C28,O12.1\"},{name:\"@-webkit-keyframes\",desc:\"Defines set of animation key frames.\",browsers:\"C,S4\"}],pseudoclasses:[{name:\":active\",desc:\"Applies while an element is being activated by the user. For example, between the times the user presses the mouse button and releases it.\",browsers:\"all\"},{name:\":any-link\",desc:\"Represents an element that acts as the source anchor of a hyperlink. Applies to both visited and unvisited links.\",browsers:\"FF,S,C,O\"},{name:\":checked\",desc:\"Radio and checkbox elements can be toggled by the user. Some menu items are 'checked' when the user selects them. When such elements are toggled 'on' the :checked pseudo-class applies.\",browsers:\"all\"},{name:\":corner-present\",desc:\"Non-standard. Indicates whether or not a scrollbar corner is present.\",browsers:\"C,S5\"},{name:\":decrement\",desc:\"Non-standard. Applies to buttons and track pieces. Indicates whether or not the button or track piece will decrement the view’s position when used.\",browsers:\"C,S5\"},{name:\":default\",desc:\"Applies to the one or more UI elements that are the default among a set of similar elements. Typically applies to context menu items, buttons, and select lists/menus.\",browsers:\"FF4,S5,C10,O10\"},{name:\":disabled\",desc:\"Represents user interface elements that are in a disabled state; such elements have a corresponding enabled state.\",browsers:\"all\"},{name:\":double-button\",desc:\"Non-standard. Applies to buttons and track pieces. Applies when both buttons are displayed together at the same end of the scrollbar.\",browsers:\"C,S5\"},{name:\":empty\",desc:\"Represents an element that has no children at all.\",browsers:\"all\"},{name:\":enabled\",desc:\"Represents user interface elements that are in an enabled state; such elements have a corresponding disabled state.\",browsers:\"all\"},{name:\":end\",desc:\"Non-standard. Applies to buttons and track pieces. Indicates whether the object is placed after the thumb.\",browsers:\"C,S5\"},{name:\":first\",desc:\"When printing double-sided documents, the page boxes on left and right pages may be different. This can be expressed through CSS pseudo-classes defined in the  page context.\",browsers:\"E,IE8,O9.2\"},{name:\":first-child\",desc:\"Same as :nth-child(1). Represents an element that is the first child of some other element.\",browsers:\"all\"},{name:\":first-of-type\",desc:\"Same as :nth-of-type(1). Represents an element that is the first sibling of its type in the list of children of its parent element.\",browsers:\"all\"},{name:\":focus\",desc:\"Applies while an element has the focus (accepts keyboard or mouse events, or other forms of input).\",browsers:\"all\"},{name:\":fullscreen\",desc:\"Matches any element that has its fullscreen flag set.\",browsers:\"E12,FF9,S6,C15,IE11\"},{name:\":future\",desc:\"Represents any element that is defined to occur entirely after a :current element.\",browsers:\"C,O16,S6\"},{name:\":horizontal\",desc:\"Non-standard. Applies to any scrollbar pieces that have a horizontal orientation.\",browsers:\"C,S5\"},{name:\":host\",desc:\"When evaluated in the context of a shadow tree, matches the shadow tree’s host element.\",browsers:\"FF61,S,C,O\"},{name:\":host()\",desc:\"When evaluated in the context of a shadow tree, it matches the shadow tree’s host element if the host element, in its normal context, matches the selector argument.\",browsers:\"C35,O22\"},{name:\":host-context()\",desc:\"Tests whether there is an ancestor, outside the shadow tree, which matches a particular selector.\",browsers:\"C35,O22\"},{name:\":hover\",desc:\"Applies while the user designates an element with a pointing device, but does not necessarily activate it. For example, a visual user agent could apply this pseudo-class when the cursor (mouse pointer) hovers over a box generated by the element.\",browsers:\"all\"},{name:\":increment\",desc:\"Non-standard. Applies to buttons and track pieces. Indicates whether or not the button or track piece will increment the view’s position when used.\",browsers:\"C,S5\"},{name:\":indeterminate\",desc:\"Applies to UI elements whose value is in an indeterminate state.\",browsers:\"all\"},{name:\":in-range\",desc:\"Used in conjunction with the min and max attributes, whether on a range input, a number field, or any other types that accept those attributes.\",browsers:\"E,FF29,S,C10,O11\"},{name:\":invalid\",desc:\"An element is :valid or :invalid when it is, respectively, valid or invalid with respect to data validity semantics defined by a different specification.\",browsers:\"all\"},{name:\":lang()\",desc:\"Represents an element that is in language specified.\",browsers:\"E,C,FF1,IE8,O8,S3\"},{name:\":last-child\",desc:\"Same as :nth-last-child(1). Represents an element that is the last child of some other element.\",browsers:\"all\"},{name:\":last-of-type\",desc:\"Same as :nth-last-of-type(1). Represents an element that is the last sibling of its type in the list of children of its parent element.\",browsers:\"all\"},{name:\":left\",desc:\"When printing double-sided documents, the page boxes on left and right pages may be different. This can be expressed through CSS pseudo-classes defined in the  page context.\",browsers:\"E,IE8,O9.2\"},{name:\":link\",desc:\"Applies to links that have not yet been visited.\",browsers:\"all\"},{name:\":matches()\",desc:\"Takes a selector list as its argument. It represents an element that is represented by its argument.\",browsers:\"S9\"},{name:\":-moz-any()\",desc:\"Represents an element that is represented by the selector list passed as its argument. Standardized as :matches().\",browsers:\"FF4\"},{name:\":-moz-any-link\",desc:\"Represents an element that acts as the source anchor of a hyperlink. Applies to both visited and unvisited links.\",browsers:\"FF1\"},{name:\":-moz-broken\",desc:\"Non-standard. Matches elements representing broken images.\",browsers:\"FF3\"},{name:\":-moz-drag-over\",desc:\"Non-standard. Matches elements when a drag-over event applies to it.\",browsers:\"FF1\"},{name:\":-moz-first-node\",desc:\"Non-standard. Represents an element that is the first child node of some other element.\",browsers:\"FF1\"},{name:\":-moz-focusring\",desc:\"Non-standard. Matches an element that has focus and focus ring drawing is enabled in the browser.\",browsers:\"FF4\"},{name:\":-moz-full-screen\",desc:\"Matches any element that has its fullscreen flag set. Standardized as :fullscreen.\",browsers:\"FF9\"},{name:\":-moz-last-node\",desc:\"Non-standard. Represents an element that is the last child node of some other element.\",browsers:\"FF1\"},{name:\":-moz-loading\",desc:\"Non-standard. Matches elements, such as images, that haven’t started loading yet.\",browsers:\"FF3\"},{name:\":-moz-only-whitespace\",desc:\"The same as :empty, except that it additionally matches elements that only contain code points affected by whitespace processing. Standardized as :blank.\",browsers:\"FF1.5\"},{name:\":-moz-placeholder\",desc:\"Deprecated. Represents placeholder text in an input field. Use ::-moz-placeholder for Firefox 19+.\",browsers:\"FF4\"},{name:\":-moz-submit-invalid\",desc:\"Non-standard. Represents any submit button when the contents of the associated form are not valid.\",browsers:\"FF4\"},{name:\":-moz-suppressed\",desc:\"Non-standard. Matches elements representing images that have been blocked from loading.\",browsers:\"FF3\"},{name:\":-moz-ui-invalid\",desc:\"Non-standard. Represents any validated form element whose value isn't valid \",browsers:\"FF4\"},{name:\":-moz-ui-valid\",desc:\"Non-standard. Represents any validated form element whose value is valid \",browsers:\"FF4\"},{name:\":-moz-user-disabled\",desc:\"Non-standard. Matches elements representing images that have been disabled due to the user’s preferences.\",browsers:\"FF3\"},{name:\":-moz-window-inactive\",desc:\"Non-standard. Matches elements in an inactive window.\",browsers:\"FF4\"},{name:\":-ms-fullscreen\",desc:\"Matches any element that has its fullscreen flag set.\",browsers:\"IE11\"},{name:\":-ms-input-placeholder\",desc:\"Represents placeholder text in an input field. Note: for Edge use the pseudo-element ::-ms-input-placeholder. Standardized as ::placeholder.\",browsers:\"IE10\"},{name:\":-ms-keyboard-active\",desc:\"Windows Store apps only. Applies one or more styles to an element when it has focus and the user presses the space bar.\",browsers:\"IE10\"},{name:\":-ms-lang()\",desc:\"Represents an element that is in the language specified. Accepts a comma seperated list of language tokens.\",browsers:\"E,IE10\"},{name:\":no-button\",desc:\"Non-standard. Applies to track pieces. Applies when there is no button at that end of the track.\",browsers:\"C,S5\"},{name:\":not()\",desc:\"The negation pseudo-class, :not(X), is a functional notation taking a simple selector (excluding the negation pseudo-class itself) as an argument. It represents an element that is not represented by its argument.\",browsers:\"E,C,FF1,IE9,O9.5,S2\"},{name:\":nth-child()\",desc:\"Represents an element that has an+b-1 siblings before it in the document tree, for any positive integer or zero value of n, and has a parent element.\",browsers:\"E,C,FF3.5,IE9,O9.5,S3.1\"},{name:\":nth-last-child()\",desc:\"Represents an element that has an+b-1 siblings after it in the document tree, for any positive integer or zero value of n, and has a parent element.\",browsers:\"E,C,FF3.5,IE9,O9.5,S3.1\"},{name:\":nth-last-of-type()\",desc:\"Represents an element that has an+b-1 siblings with the same expanded element name after it in the document tree, for any zero or positive integer value of n, and has a parent element.\",browsers:\"E,C,FF3.5,IE9,O9.5,S3.1\"},{name:\":nth-of-type()\",desc:\"Represents an element that has an+b-1 siblings with the same expanded element name before it in the document tree, for any zero or positive integer value of n, and has a parent element.\",browsers:\"E,C,FF3.5,IE9,O9.5,S3.1\"},{name:\":only-child\",desc:\"Represents an element that has a parent element and whose parent element has no other element children. Same as :first-child:last-child or :nth-child(1):nth-last-child(1), but with a lower specificity.\",browsers:\"all\"},{name:\":only-of-type\",desc:\"Matches every element that is the only child of its type, of its parent. Same as :first-of-type:last-of-type or :nth-of-type(1):nth-last-of-type(1), but with a lower specificity.\",browsers:\"all\"},{name:\":optional\",desc:\"A form element is :required or :optional if a value for it is, respectively, required or optional before the form it belongs to is submitted. Elements that are not form elements are neither required nor optional.\",browsers:\"all\"},{name:\":out-of-range\",desc:\"Used in conjunction with the min and max attributes, whether on a range input, a number field, or any other types that accept those attributes.\",browsers:\"E,FF29,S,C10,O11\"},{name:\":past\",desc:\"Represents any element that is defined to occur entirely prior to a :current element.\",browsers:\"C,O16,S6\"},{name:\":read-only\",desc:\"An element whose contents are not user-alterable is :read-only. However, elements whose contents are user-alterable (such as text input fields) are considered to be in a :read-write state. In typical documents, most elements are :read-only.\",browsers:\"E,FF,S,C,O\"},{name:\":read-write\",desc:\"An element whose contents are not user-alterable is :read-only. However, elements whose contents are user-alterable (such as text input fields) are considered to be in a :read-write state. In typical documents, most elements are :read-only.\",browsers:\"E,FF,S,C,O\"},{name:\":required\",desc:\"A form element is :required or :optional if a value for it is, respectively, required or optional before the form it belongs to is submitted. Elements that are not form elements are neither required nor optional.\",browsers:\"all\"},{name:\":right\",desc:\"When printing double-sided documents, the page boxes on left and right pages may be different. This can be expressed through CSS pseudo-classes defined in the  page context.\",browsers:\"E,IE8,O9.2\"},{name:\":root\",desc:\"Represents an element that is the root of the document. In HTML 4, this is always the HTML element.\",browsers:\"FF1,S1,C1,IE9,O9.5\"},{name:\":scope\",desc:\"Represents any element that is in the contextual reference element set.\",browsers:\"FF32,S7,O15\"},{name:\":single-button\",desc:\"Non-standard. Applies to buttons and track pieces. Applies when both buttons are displayed separately at either end of the scrollbar.\",browsers:\"C,S5\"},{name:\":start\",desc:\"Non-standard. Applies to buttons and track pieces. Indicates whether the object is placed before the thumb.\",browsers:\"C,S5\"},{name:\":target\",desc:\"Some URIs refer to a location within a resource. This kind of URI ends with a 'number sign' (#) followed by an anchor identifier (called the fragment identifier).\",browsers:\"all\"},{name:\":valid\",desc:\"An element is :valid or :invalid when it is, respectively, valid or invalid with respect to data validity semantics defined by a different specification.\",browsers:\"all\"},{name:\":vertical\",desc:\"Non-standard. Applies to any scrollbar pieces that have a vertical orientation.\",browsers:\"C,S5\"},{name:\":visited\",desc:\"Applies once the link has been visited by the user.\",browsers:\"all\"},{name:\":-webkit-any()\",desc:\"Represents an element that is represented by the selector list passed as its argument. Standardized as :matches().\",browsers:\"C,S5\"},{name:\":-webkit-full-screen\",desc:\"Matches any element that has its fullscreen flag set. Standardized as :fullscreen.\",browsers:\"C,S6\"},{name:\":window-inactive\",desc:\"Non-standard. Applies to all scrollbar pieces. Indicates whether or not the window containing the scrollbar is currently active.\",browsers:\"C,S3\"},{name:\":defined\",desc:\"The :defined CSS pseudo-class represents any element that has been defined. This includes any standard element built in to the browser, and custom elements that have been successfully defined (i.e. with the CustomElementRegistry.define() method).\",browsers:\"S,C,O\"},{name:\":dir\",desc:\"The :dir() CSS pseudo-class matches elements based on the directionality of the text contained in them.\",browsers:\"FF49\"},{name:\":focus-visible\",desc:\"The :focus-visible pseudo-class applies while an element matches the :focus pseudo-class and the UA determines via heuristics that the focus should be made evident on the element.\"},{name:\":focus-within\",desc:\"The :focus-within pseudo-class applies to any element for which the :focus pseudo class applies as well as to an element whose descendant in the flat tree (including non-element nodes, such as text nodes) matches the conditions for matching :focus.\",browsers:\"FF52,S10.1,C60,O47\"},{name:\":placeholder-shown\",desc:\"The :placeholder-shown CSS pseudo-class represents any <input> or <textarea> element that is currently displaying placeholder text.\",browsers:\"FF51,S9,C47,O34\"}],pseudoelements:[{name:\"::after\",desc:\"Represents a styleable child pseudo-element immediately after the originating element’s actual content.\",browsers:\"all\"},{name:\"::backdrop\",desc:\"Used to create a backdrop that hides the underlying document for an element in a top layer (such as an element that is displayed fullscreen).\",browsers:\"E,FF47,C37,IE11\"},{name:\"::before\",desc:\"Represents a styleable child pseudo-element immediately before the originating element’s actual content.\",browsers:\"all\"},{name:\"::content\",desc:\"Deprecated. Matches the distribution list itself, on elements that have one. Use ::slotted for forward compatibility.\",browsers:\"C35,O22\"},{name:\"::cue\",browsers:\"FF55,C\"},{name:\"::cue()\",browsers:\"C,O16,S6\"},{name:\"::cue-region\",browsers:\"C,O16,S6\"},{name:\"::cue-region()\",browsers:\"C,O16,S6\"},{name:\"::first-letter\",desc:\"Represents the first letter of an element, if it is not preceded by any other content (such as images or inline tables) on its line.\",browsers:\"all\"},{name:\"::first-line\",desc:\"Describes the contents of the first formatted line of its originating element.\",browsers:\"all\"},{name:\"::-moz-focus-inner\",browsers:\"FF4\"},{name:\"::-moz-focus-outer\",browsers:\"FF4\"},{name:\"::-moz-list-bullet\",desc:\"Used to style the bullet of a list element. Similar to the standardized ::marker.\",browsers:\"FF1\"},{name:\"::-moz-list-number\",desc:\"Used to style the numbers of a list element. Similar to the standardized ::marker.\",browsers:\"FF1\"},{name:\"::-moz-placeholder\",desc:\"Represents placeholder text in an input field\",browsers:\"FF19\"},{name:\"::-moz-progress-bar\",desc:\"Represents the bar portion of a progress bar.\",browsers:\"FF9\"},{name:\"::-moz-selection\",desc:\"Represents the portion of a document that has been highlighted by the user.\",browsers:\"FF1\"},{name:\"::-ms-backdrop\",desc:\"Used to create a backdrop that hides the underlying document for an element in a top layer (such as an element that is displayed fullscreen).\",browsers:\"IE11\"},{name:\"::-ms-browse\",desc:\"Represents the browse button of an input type=file control.\",browsers:\"E,IE10\"},{name:\"::-ms-check\",desc:\"Represents the check of a checkbox or radio button input control.\",browsers:\"E,IE10\"},{name:\"::-ms-clear\",desc:\"Represents the clear button of a text input control\",browsers:\"E,IE10\"},{name:\"::-ms-expand\",desc:\"Represents the drop-down button of a select control.\",browsers:\"E,IE10\"},{name:\"::-ms-fill\",desc:\"Represents the bar portion of a progress bar.\",browsers:\"E,IE10\"},{name:\"::-ms-fill-lower\",desc:\"Represents the portion of the slider track from its smallest value up to the value currently selected by the thumb. In a left-to-right layout, this is the portion of the slider track to the left of the thumb.\",browsers:\"E,IE10\"},{name:\"::-ms-fill-upper\",desc:\"Represents the portion of the slider track from the value currently selected by the thumb up to the slider's largest value. In a left-to-right layout, this is the portion of the slider track to the right of the thumb.\",browsers:\"E,IE10\"},{name:\"::-ms-reveal\",desc:\"Represents the password reveal button of an input type=password control.\",browsers:\"E,IE10\"},{name:\"::-ms-thumb\",desc:\"Represents the portion of range input control (also known as a slider control) that the user drags.\",browsers:\"E,IE10\"},{name:\"::-ms-ticks-after\",desc:\"Represents the tick marks of a slider that begin just after the thumb and continue up to the slider's largest value. In a left-to-right layout, these are the ticks to the right of the thumb.\",browsers:\"E,IE10\"},{name:\"::-ms-ticks-before\",desc:\"Represents the tick marks of a slider that represent its smallest values up to the value currently selected by the thumb. In a left-to-right layout, these are the ticks to the left of the thumb.\",browsers:\"E,IE10\"},{name:\"::-ms-tooltip\",desc:\"Represents the tooltip of a slider (input type=range).\",browsers:\"E,IE10\"},{name:\"::-ms-track\",desc:\"Represents the track of a slider.\",browsers:\"E,IE10\"},{name:\"::-ms-value\",desc:\"Represents the content of a text or password input control, or a select control.\",browsers:\"E,IE10\"},{name:\"::selection\",desc:\"Represents the portion of a document that has been highlighted by the user.\",browsers:\"all\"},{name:\"::shadow\",desc:\"Matches the shadow root if an element has a shadow tree.\",browsers:\"C35,O22\"},{name:\"::-webkit-file-upload-button\",browsers:\"C,O,S6\"},{name:\"::-webkit-inner-spin-button\",browsers:\"C,O,S6\"},{name:\"::-webkit-input-placeholder\",browsers:\"C,S4\"},{name:\"::-webkit-keygen-select\",browsers:\"C,O,S6\"},{name:\"::-webkit-meter-bar\",browsers:\"S,C,O\"},{name:\"::-webkit-meter-even-less-good-value\",browsers:\"S,C,O\"},{name:\"::-webkit-meter-optimum-value\",browsers:\"S,C,O\"},{name:\"::-webkit-meter-suboptimal-value\",browsers:\"E13,C,O15,S6\"},{name:\"::-webkit-outer-spin-button\",browsers:\"C,O,S6\"},{name:\"::-webkit-progress-bar\",browsers:\"S,C,O\"},{name:\"::-webkit-progress-inner-element\",browsers:\"S,C,O\"},{name:\"::-webkit-progress-value\",browsers:\"S,C,O\"},{name:\"::-webkit-resizer\",browsers:\"C,S5\"},{name:\"::-webkit-scrollbar\",browsers:\"C,S5\"},{name:\"::-webkit-scrollbar-button\",browsers:\"C,S5\"},{name:\"::-webkit-scrollbar-corner\",browsers:\"C,S5\"},{name:\"::-webkit-scrollbar-thumb\",browsers:\"C,S5\"},{name:\"::-webkit-scrollbar-track\",browsers:\"C,S5\"},{name:\"::-webkit-scrollbar-track-piece\",browsers:\"C,S5\"},{name:\"::-webkit-search-cancel-button\",browsers:\"S,C\"},{name:\"::-webkit-search-decoration\",browsers:\"C,S4\"},{name:\"::-webkit-search-results-button\",browsers:\"S,C\"},{name:\"::-webkit-search-results-decoration\",browsers:\"C,S4\"},{name:\"::-webkit-slider-runnable-track\",browsers:\"C,O,S6\"},{name:\"::-webkit-slider-thumb\",browsers:\"C,O,S6\"},{name:\"::-webkit-textfield-decoration-container\",browsers:\"C,O,S6\"},{name:\"::-webkit-validation-bubble\",browsers:\"C,O,S6\"},{name:\"::-webkit-validation-bubble-arrow\",browsers:\"C,O,S6\"},{name:\"::-webkit-validation-bubble-arrow-clipper\",browsers:\"C,O,S6\"},{name:\"::-webkit-validation-bubble-heading\",browsers:\"C,O,S6\"},{name:\"::-webkit-validation-bubble-message\",browsers:\"C,O,S6\"},{name:\"::-webkit-validation-bubble-text-block\",browsers:\"C,O,S6\"},{name:\"::-moz-range-progress\",desc:'The ::-moz-range-progress CSS pseudo-element is a Mozilla extension that represents the lower portion of the track (i.e., groove) in which the indicator slides in an <input> of type=\"range\". This portion corresponds to values lower than the value currently selected by the thumb (i.e., virtual knob).'},{name:\"::-moz-range-thumb\",desc:'The ::-moz-range-thumb CSS pseudo-element is a Mozilla extension that represents the thumb (i.e., virtual knob) of an <input> of type=\"range\". The user can move the thumb along the input\\'s track to alter its numerical value.'},{name:\"::-moz-range-track\",desc:'The ::-moz-range-track CSS pseudo-element is a Mozilla extension that represents the track (i.e., groove) in which the indicator slides in an <input> of type=\"range\".'},{name:\"::-webkit-progress-inner-value\",desc:\"The ::-webkit-progress-value CSS pseudo-element represents the filled-in portion of the bar of a <progress> element. It is a child of the ::-webkit-progress-bar pseudo-element.\\n\\nIn order to let ::-webkit-progress-value take effect, -webkit-appearance needs to be set to none on the <progress> element.\"},{name:\"::grammar-error\",desc:\"The ::grammar-error CSS pseudo-element represents a text segment which the user agent has flagged as grammatically incorrect.\"},{name:\"::placeholder\",desc:\"The ::placeholder CSS pseudo-element represents the placeholder text of a form element.\",browsers:\"all\"},{name:\"::spelling-error\",desc:\"The ::spelling-error CSS pseudo-element represents a text segment which the user agent has flagged as incorrectly spelled.\"}],properties:[{name:\"additive-symbols\",desc:\"@counter-style descriptor. Specifies the symbols used by the marker-construction algorithm specified by the system descriptor. Needs to be specified if the counter system is 'additive'.\",browsers:\"FF33\",restriction:\"integer, string, image, identifier\",syntax:\"[ <integer> && <symbol> ]#\"},{name:\"align-content\",desc:\"Aligns a flex container’s lines within the flex container when there is extra space in the cross-axis, similar to how 'justify-content' aligns individual items within the main-axis.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"center\",desc:\"Lines are packed toward the center of the flex container.\"},{name:\"flex-end\",desc:\"Lines are packed toward the end of the flex container.\"},{name:\"flex-start\",desc:\"Lines are packed toward the start of the flex container.\"},{name:\"space-around\",desc:\"Lines are evenly distributed in the flex container, with half-size spaces on either end.\"},{name:\"space-between\",desc:\"Lines are evenly distributed in the flex container.\"},{name:\"stretch\",desc:\"Lines stretch to take up the remaining space.\"}],syntax:\"normal | <baseline-position> | <content-distribution> | <overflow-position>? <content-position>\"},{name:\"align-items\",desc:\"Aligns flex items along the cross axis of the current line of the flex container.\",browsers:\"E,C29,FF22,IE11,O12.1,S9\",restriction:\"enum\",values:[{name:\"baseline\",desc:\"If the flex item’s inline axis is the same as the cross axis, this value is identical to 'flex-start'. Otherwise, it participates in baseline alignment.\"},{name:\"center\",desc:\"The flex item’s margin box is centered in the cross axis within the line.\"},{name:\"flex-end\",desc:\"The cross-end margin edge of the flex item is placed flush with the cross-end edge of the line.\"},{name:\"flex-start\",desc:\"The cross-start margin edge of the flex item is placed flush with the cross-start edge of the line.\"},{name:\"stretch\",desc:\"If the cross size property of the flex item computes to auto, and neither of the cross-axis margins are auto, the flex item is stretched.\"}],syntax:\"normal | stretch | <baseline-position> | [ <overflow-position>? <self-position> ]\"},{name:\"justify-items\",desc:\"Defines the default justify-self for all items of the box, given them the default way of justifying each box along the appropriate axi\",browsers:\"E16,FF45\",restriction:\"enum\",values:[{name:\"auto\"},{name:\"normal\"},{name:\"end\"},{name:\"start\"},{name:\"flex-end\",desc:'\"Flex items are packed toward the end of the line.\"'},{name:\"flex-start\",desc:'\"Flex items are packed toward the start of the line.\"'},{name:\"self-end\"},{name:\"self-start\"},{name:\"center\",desc:\"The items are packed flush to each other toward the center of the of the alignment container.\"},{name:\"left\"},{name:\"right\"},{name:\"baseline\"},{name:\"first baseline\"},{name:\"last baseline\"},{name:\"stretch\",desc:\"If the cross size property of the flex item computes to auto, and neither of the cross-axis margins are auto, the flex item is stretched.\"},{name:\"save\"},{name:\"unsave\"},{name:\"legacy\"}],syntax:\"normal | stretch | <baseline-position> | <overflow-position>? [ <self-position> | left | right ] | legacy | legacy && [ left | right | center ]\"},{name:\"justify-self\",desc:\"Defines the way of justifying a box inside its container along the appropriate axis.\",browsers:\"E16,FF45\",restriction:\"enum\",values:[{name:\"auto\"},{name:\"normal\"},{name:\"end\"},{name:\"start\"},{name:\"flex-end\",desc:'\"Flex items are packed toward the end of the line.\"'},{name:\"flex-start\",desc:'\"Flex items are packed toward the start of the line.\"'},{name:\"self-end\"},{name:\"self-start\"},{name:\"center\",desc:\"The items are packed flush to each other toward the center of the of the alignment container.\"},{name:\"left\"},{name:\"right\"},{name:\"baseline\"},{name:\"first baseline\"},{name:\"last baseline\"},{name:\"stretch\",desc:\"If the cross size property of the flex item computes to auto, and neither of the cross-axis margins are auto, the flex item is stretched.\"},{name:\"save\"},{name:\"unsave\"}],syntax:\"auto | normal | stretch | <baseline-position> | <overflow-position>? [ <self-position> | left | right ]\"},{name:\"align-self\",desc:\"Allows the default alignment along the cross axis to be overridden for individual flex items.\",browsers:\"E12,FF20,C36,IE11,O12.1\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Computes to the value of 'align-items' on the element’s parent, or 'stretch' if the element has no parent. On absolutely positioned elements, it computes to itself.\"},{name:\"baseline\",desc:\"If the flex item’s inline axis is the same as the cross axis, this value is identical to 'flex-start'. Otherwise, it participates in baseline alignment.\"},{name:\"center\",desc:\"The flex item’s margin box is centered in the cross axis within the line.\"},{name:\"flex-end\",desc:\"The cross-end margin edge of the flex item is placed flush with the cross-end edge of the line.\"},{name:\"flex-start\",desc:\"The cross-start margin edge of the flex item is placed flush with the cross-start edge of the line.\"},{name:\"stretch\",desc:\"If the cross size property of the flex item computes to auto, and neither of the cross-axis margins are auto, the flex item is stretched.\"}],syntax:\"auto | normal | stretch | <baseline-position> | <overflow-position>? <self-position>\"},{name:\"all\",desc:\"Shorthand that resets all properties except 'direction' and 'unicode-bidi'.\",browsers:\"FF27,C37,O24\",restriction:\"enum\",values:[],syntax:\"initial | inherit | unset | revert\"},{name:\"alt\",desc:\"Provides alternative text for assistive technology to replace the genenerated content of a ::before or ::after element.\",browsers:\"S9\",restriction:\"string, enum\",values:[]},{name:\"animation\",desc:\"Shorthand property combines six of the animation properties into a single property.\",browsers:\"all\",restriction:\"time, timing-function, enum, identifier, number\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"backwards\"},{name:\"both\",desc:\"Both forwards and backwards fill modes are applied.\"},{name:\"forwards\"},{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"},{name:\"none\",desc:\"No animation is performed\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}],syntax:\"<single-animation>#\"},{name:\"animation-delay\",desc:\"Defines when the animation will start.\",browsers:\"all\",restriction:\"time\",syntax:\"<time>#\"},{name:\"animation-direction\",desc:\"Defines whether or not the animation should play in reverse on alternate cycles.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}],syntax:\"<single-animation-direction>#\"},{name:\"animation-duration\",desc:\"Defines the length of time that an animation takes to complete one cycle.\",browsers:\"all\",restriction:\"time\",syntax:\"<time>#\"},{name:\"animation-fill-mode\",desc:\"Defines what values are applied by the animation outside the time it is executing.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"backwards\"},{name:\"both\",desc:\"Both forwards and backwards fill modes are applied.\"},{name:\"forwards\"},{name:\"none\",desc:\"There is no change to the property value between the time the animation is applied and the time the animation begins playing or after the animation completes.\"}],syntax:\"<single-animation-fill-mode>#\"},{name:\"animation-iteration-count\",desc:\"Defines the number of times an animation cycle is played. The default value is one, meaning the animation will play from beginning to end once.\",browsers:\"all\",restriction:\"number, enum\",values:[{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"}],syntax:\"<single-animation-iteration-count>#\"},{name:\"animation-name\",desc:\"Defines a list of animations that apply. Each name is used to select the keyframe at-rule that provides the property values for the animation.\",browsers:\"all\",restriction:\"identifier, enum\",values:[{name:\"none\",desc:\"No animation is performed\"}],syntax:\"[ none | <keyframes-name> ]#\"},{name:\"animation-play-state\",desc:\"Defines whether the animation is running or paused.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"paused\"},{name:\"running\"}],syntax:\"<single-animation-play-state>#\"},{name:\"animation-timing-function\",desc:\"Describes how the animation will progress over one cycle of its duration.\",browsers:\"all\",restriction:\"timing-function\",syntax:\"<single-timing-function>#\"},{name:\"backface-visibility\",desc:\"Determines whether or not the 'back' side of a transformed element is visible when facing the viewer. With an identity transform, the front side of an element faces the viewer.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"hidden\",desc:\"Back side is hidden.\"},{name:\"visible\",desc:\"Back side is visible.\"}],syntax:\"visible | hidden\"},{name:\"background\",desc:\"Shorthand property for setting most background properties at the same place in the style sheet.\",browsers:\"all\",restriction:\"enum, image, color, position, length, repeat, percentage, box\",values:[{name:\"fixed\",desc:\"The background is fixed with regard to the viewport. In paged media where there is no viewport, a 'fixed' background is fixed with respect to the page box and therefore replicated on every page.\"},{name:\"local\",desc:\"The background is fixed with regard to the element's contents: if the element has a scrolling mechanism, the background scrolls with the element's contents.\"},{name:\"scroll\",desc:\"The background is fixed with regard to the element itself and does not scroll with its contents. (It is effectively attached to the element's border.)\"}],syntax:\"[ <bg-layer> , ]* <final-bg-layer>\"},{name:\"background-attachment\",desc:\"Specifies whether the background images are fixed with regard to the viewport ('fixed') or scroll along with the element ('scroll') or its contents ('local').\",browsers:\"all\",restriction:\"enum\",values:[{name:\"fixed\",desc:\"The background is fixed with regard to the viewport. In paged media where there is no viewport, a 'fixed' background is fixed with respect to the page box and therefore replicated on every page.\"},{name:\"local\",desc:\"The background is fixed with regard to the element’s contents: if the element has a scrolling mechanism, the background scrolls with the element’s contents.\",browsers:\"E,C,FF25,IE9,O11.5,S5\"},{name:\"scroll\",desc:\"The background is fixed with regard to the element itself and does not scroll with its contents. (It is effectively attached to the element’s border.)\"}],syntax:\"<attachment>#\"},{name:\"background-blend-mode\",desc:\"Defines the blending mode of each background layer.\",browsers:\"FF30,S,C35,O22\",restriction:\"enum\",values:[{name:\"normal\",desc:\"Default attribute which specifies no blending\"},{name:\"multiply\"},{name:\"screen\"},{name:\"overlay\"},{name:\"darken\"},{name:\"lighten\"},{name:\"color-dodge\"},{name:\"color-burn\"},{name:\"hard-light\"},{name:\"soft-light\"},{name:\"difference\"},{name:\"exclusion\"},{name:\"hue\",browsers:\"C35,FF30,O22\"},{name:\"saturation\",browsers:\"C35,FF30,O22\"},{name:\"color\",browsers:\"C35,FF30,O22\"},{name:\"luminosity\",browsers:\"C35,FF30,O22\"}],syntax:\"<blend-mode>#\"},{name:\"background-clip\",desc:\"Determines the background painting area.\",browsers:\"all\",restriction:\"box\",syntax:\"<box>#\"},{name:\"background-color\",desc:\"Sets the background color of an element.\",browsers:\"all\",restriction:\"color\",syntax:\"<color>\"},{name:\"background-image\",desc:\"Sets the background image(s) of an element.\",browsers:\"all\",restriction:\"image, enum\",values:[{name:\"none\",desc:\"Counts as an image layer but draws nothing.\"}],syntax:\"<bg-image>#\"},{name:\"background-origin\",desc:\"For elements rendered as a single box, specifies the background positioning area. For elements rendered as multiple boxes (e.g., inline boxes on several lines, boxes on several pages) specifies which boxes 'box-decoration-break' operates on to determine the background positioning area(s).\",browsers:\"all\",restriction:\"box\",syntax:\"<box>#\"},{name:\"background-position\",desc:\"Specifies the initial position of the background image(s) (after any resizing) within their corresponding background positioning area.\",browsers:\"all\",restriction:\"position, length, percentage\",syntax:\"<bg-position>#\"},{name:\"background-position-x\",desc:\"If background images have been specified, this property specifies their initial position (after any resizing) within their corresponding background positioning area.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"center\",desc:\"Equivalent to '50%' ('left 50%') for the horizontal position if the horizontal position is not otherwise specified, or '50%' ('top 50%') for the vertical position if it is.\"},{name:\"left\",desc:\"Equivalent to '0%' for the horizontal position if one or two values are given, otherwise specifies the left edge as the origin for the next offset.\"},{name:\"right\",desc:\"Equivalent to '100%' for the horizontal position if one or two values are given, otherwise specifies the right edge as the origin for the next offset.\"}],status:\"e\",syntax:\"[ center | [ left | right | x-start | x-end ]? <length-percentage>? ]#\"},{name:\"background-position-y\",desc:\"If background images have been specified, this property specifies their initial position (after any resizing) within their corresponding background positioning area.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"bottom\",desc:\"Equivalent to '100%' for the vertical position if one or two values are given, otherwise specifies the bottom edge as the origin for the next offset.\"},{name:\"center\",desc:\"Equivalent to '50%' ('left 50%') for the horizontal position if the horizontal position is not otherwise specified, or '50%' ('top 50%') for the vertical position if it is.\"},{name:\"top\",desc:\"Equivalent to '0%' for the vertical position if one or two values are given, otherwise specifies the top edge as the origin for the next offset.\"}],status:\"e\",syntax:\"[ center | [ top | bottom | y-start | y-end ]? <length-percentage>? ]#\"},{name:\"background-repeat\",desc:\"Specifies how background images are tiled after they have been sized and positioned.\",browsers:\"all\",restriction:\"repeat\",values:[],syntax:\"<repeat-style>#\"},{name:\"background-size\",desc:\"Specifies the size of the background images.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"Resolved by using the image’s intrinsic ratio and the size of the other dimension, or failing that, using the image’s intrinsic size, or failing that, treating it as 100%.\"},{name:\"contain\",desc:\"Scale the image, while preserving its intrinsic aspect ratio (if any), to the largest size such that both its width and its height can fit inside the background positioning area.\"},{name:\"cover\",desc:\"Scale the image, while preserving its intrinsic aspect ratio (if any), to the smallest size such that both its width and its height can completely cover the background positioning area.\"}],syntax:\"<bg-size>#\"},{name:\"behavior\",desc:\"IE only. Used to extend behaviors of the browser.\",browsers:\"IE6\",restriction:\"url\"},{name:\"block-size\",desc:\"Logical 'width'. Mapping depends on the element’s 'writing-mode'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"Depends on the values of other properties.\"}],syntax:\"<'width'>\"},{name:\"border\",desc:\"Shorthand property for setting border width, style, and color.\",browsers:\"all\",restriction:\"length, line-width, line-style, color\",syntax:\"<br-width> || <br-style> || <color>\"},{name:\"border-block-end\",desc:\"Logical 'border-bottom'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width, line-style, color\",syntax:\"<'border-width'> || <'border-style'> || <'color'>\"},{name:\"border-block-start\",desc:\"Logical 'border-top'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width, line-style, color\",syntax:\"<'border-width'> || <'border-style'> || <'color'>\"},{name:\"border-block-end-color\",desc:\"Logical 'border-bottom-color'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"color\",syntax:\"<'color'>\"},{name:\"border-block-start-color\",desc:\"Logical 'border-top-color'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"color\",syntax:\"<'color'>\"},{name:\"border-block-end-style\",desc:\"Logical 'border-bottom-style'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"line-style\",syntax:\"<'border-style'>\"},{name:\"border-block-start-style\",desc:\"Logical 'border-top-style'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"lline-style\",syntax:\"<'border-style'>\"},{name:\"border-block-end-width\",desc:\"Logical 'border-bottom-width'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width\",syntax:\"<'border-width'>\"},{name:\"border-block-start-width\",desc:\"Logical 'border-top-width'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width\",syntax:\"<'border-width'>\"},{name:\"border-bottom\",desc:\"Shorthand property for setting border width, style and color.\",browsers:\"all\",restriction:\"length, line-width, line-style, color\",syntax:\"<br-width> || <br-style> || <color>\"},{name:\"border-bottom-color\",desc:\"Sets the color of the bottom border.\",browsers:\"all\",restriction:\"color\",syntax:\"<color>\"},{name:\"border-bottom-left-radius\",desc:\"Defines the radii of the bottom left outer border edge.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length-percentage>{1,2}\"},{name:\"border-bottom-right-radius\",desc:\"Defines the radii of the bottom right outer border edge.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length-percentage>{1,2}\"},{name:\"border-bottom-style\",desc:\"Sets the style of the bottom border.\",browsers:\"all\",restriction:\"line-style\",syntax:\"<br-style>\"},{name:\"border-bottom-width\",desc:\"Sets the thickness of the bottom border.\",browsers:\"all\",restriction:\"length, line-width\",syntax:\"<br-width>\"},{name:\"border-collapse\",desc:\"Selects a table's border model.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"collapse\",desc:\"Selects the collapsing borders model.\"},{name:\"separate\",desc:\"Selects the separated borders border model.\"}],syntax:\"collapse | separate\"},{name:\"border-color\",desc:\"The color of the border around all four edges of an element.\",browsers:\"all\",restriction:\"color\",values:[],syntax:\"<color>{1,4}\"},{name:\"border-image\",desc:\"Shorthand property for setting 'border-image-source', 'border-image-slice', 'border-image-width', 'border-image-outset' and 'border-image-repeat'. Omitted values are set to their initial values.\",browsers:\"all\",restriction:\"length, percentage, number, url, enum\",values:[{name:\"auto\",desc:\"If 'auto' is specified then the border image width is the intrinsic width or height (whichever is applicable) of the corresponding image slice. If the image does not have the required intrinsic dimension then the corresponding border-width is used instead.\"},{name:\"fill\",desc:\"Causes the middle part of the border-image to be preserved.\"},{name:\"none\",desc:\"Use the border styles.\"},{name:\"repeat\"},{name:\"round\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the image is rescaled so that it does.\"},{name:\"space\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the extra space is distributed around the tiles.\"},{name:\"stretch\",desc:\"The image is stretched to fill the area.\"},{name:\"url()\"}],syntax:\"<'border-image-source'> || <'border-image-slice'> [ / <'border-image-width'> | / <'border-image-width'>? / <'border-image-outset'> ]? || <'border-image-repeat'>\"},{name:\"border-image-outset\",desc:\"The values specify the amount by which the border image area extends beyond the border box on the top, right, bottom, and left sides respectively. If the fourth value is absent, it is the same as the second. If the third one is also absent, it is the same as the first. If the second one is also absent, it is the same as the first. Numbers represent multiples of the corresponding border-width.\",browsers:\"all\",restriction:\"length, number\",syntax:\"[ <length> | <number> ]{1,4}\"},{name:\"border-image-repeat\",desc:\"Specifies how the images for the sides and the middle part of the border image are scaled and tiled. If the second keyword is absent, it is assumed to be the same as the first.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"repeat\"},{name:\"round\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the image is rescaled so that it does.\"},{name:\"space\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the extra space is distributed around the tiles.\"},{name:\"stretch\",desc:\"The image is stretched to fill the area.\"}],syntax:\"[ stretch | repeat | round | space ]{1,2}\"},{name:\"border-image-slice\",desc:\"Specifies inward offsets from the top, right, bottom, and left edges of the image, dividing it into nine regions: four corners, four edges and a middle.\",browsers:\"all\",restriction:\"number, percentage\",values:[{name:\"fill\",desc:\"Causes the middle part of the border-image to be preserved.\"}],syntax:\"<number-percentage>{1,4} && fill?\"},{name:\"border-image-source\",desc:\"Specifies an image to use instead of the border styles given by the 'border-style' properties and as an additional background layer for the element. If the value is 'none' or if the image cannot be displayed, the border styles will be used.\",browsers:\"all\",restriction:\"image\",values:[{name:\"none\",desc:\"Use the border styles.\"}],syntax:\"none | <image>\"},{name:\"border-image-width\",desc:\"The four values of 'border-image-width' specify offsets that are used to divide the border image area into nine parts. They represent inward distances from the top, right, bottom, and left sides of the area, respectively.\",browsers:\"all\",restriction:\"length, percentage, number\",values:[{name:\"auto\",desc:\"The border image width is the intrinsic width or height (whichever is applicable) of the corresponding image slice. If the image does not have the required intrinsic dimension then the corresponding border-width is used instead.\"}],syntax:\"[ <length-percentage> | <number> | auto ]{1,4}\"},{name:\"border-inline-end\",desc:\"Logical 'border-right'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width, line-style, color\",syntax:\"<'border-width'> || <'border-style'> || <'color'>\"},{name:\"border-inline-start\",desc:\"Logical 'border-left'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width, line-style, color\",syntax:\"<'border-width'> || <'border-style'> || <'color'>\"},{name:\"border-inline-end-color\",desc:\"Logical 'border-right-color'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"color\",syntax:\"<'color'>\"},{name:\"border-inline-start-color\",desc:\"Logical 'border-left-color'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"color\",syntax:\"<'color'>\"},{name:\"border-inline-end-style\",desc:\"Logical 'border-right-style'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"line-style\",syntax:\"<'border-style'>\"},{name:\"border-inline-start-style\",desc:\"Logical 'border-left-style'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"lline-style\",syntax:\"<'border-style'>\"},{name:\"border-inline-end-width\",desc:\"Logical 'border-right-width'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width\",syntax:\"<'border-width'>\"},{name:\"border-inline-start-width\",desc:\"Logical 'border-left-width'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, line-width\",syntax:\"<'border-width'>\"},{name:\"border-left\",desc:\"Shorthand property for setting border width, style and color\",browsers:\"all\",restriction:\"length, line-width, line-style, color\",syntax:\"<br-width> || <br-style> || <color>\"},{name:\"border-left-color\",desc:\"Sets the color of the left border.\",browsers:\"all\",restriction:\"color\",syntax:\"<color>\"},{name:\"border-left-style\",desc:\"Sets the style of the left border.\",browsers:\"all\",restriction:\"line-style\",syntax:\"<br-style>\"},{name:\"border-left-width\",desc:\"Sets the thickness of the left border.\",browsers:\"all\",restriction:\"length, line-width\",syntax:\"<br-width>\"},{name:\"border-radius\",desc:\"Defines the radii of the outer border edge.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length-percentage>{1,4} [ / <length-percentage>{1,4} ]?\"},{name:\"border-right\",desc:\"Shorthand property for setting border width, style and color\",browsers:\"all\",restriction:\"length, line-width, line-style, color\",syntax:\"<br-width> || <br-style> || <color>\"},{name:\"border-right-color\",desc:\"Sets the color of the right border.\",browsers:\"all\",restriction:\"color\",syntax:\"<color>\"},{name:\"border-right-style\",desc:\"Sets the style of the right border.\",browsers:\"all\",restriction:\"line-style\",syntax:\"<br-style>\"},{name:\"border-right-width\",desc:\"Sets the thickness of the right border.\",browsers:\"all\",restriction:\"length, line-width\",syntax:\"<br-width>\"},{name:\"border-spacing\",desc:\"The lengths specify the distance that separates adjoining cell borders. If one length is specified, it gives both the horizontal and vertical spacing. If two are specified, the first gives the horizontal spacing and the second the vertical spacing. Lengths may not be negative.\",browsers:\"all\",restriction:\"length\",syntax:\"<length> <length>?\"},{name:\"border-style\",desc:\"The style of the border around edges of an element.\",browsers:\"all\",restriction:\"line-style\",values:[],syntax:\"<br-style>{1,4}\"},{name:\"border-top\",desc:\"Shorthand property for setting border width, style and color\",browsers:\"all\",restriction:\"length, line-width, line-style, color\",syntax:\"<br-width> || <br-style> || <color>\"},{name:\"border-top-color\",desc:\"Sets the color of the top border.\",browsers:\"all\",restriction:\"color\",syntax:\"<color>\"},{name:\"border-top-left-radius\",desc:\"Defines the radii of the top left outer border edge.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length-percentage>{1,2}\"},{name:\"border-top-right-radius\",desc:\"Defines the radii of the top right outer border edge.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length-percentage>{1,2}\"},{name:\"border-top-style\",desc:\"Sets the style of the top border.\",browsers:\"all\",restriction:\"line-style\",syntax:\"<br-style>\"},{name:\"border-top-width\",desc:\"Sets the thickness of the top border.\",browsers:\"all\",restriction:\"length, line-width\",syntax:\"<br-width>\"},{name:\"border-width\",desc:\"Shorthand that sets the four 'border-*-width' properties. If it has four values, they set top, right, bottom and left in that order. If left is missing, it is the same as right; if bottom is missing, it is the same as top; if right is missing, it is the same as top.\",browsers:\"all\",restriction:\"length, line-width\",values:[],syntax:\"<br-width>{1,4}\"},{name:\"bottom\",desc:\"Specifies how far an absolutely positioned box's bottom margin edge is offset above the bottom edge of the box's 'containing block'.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"box-decoration-break\",desc:\"Specifies whether individual boxes are treated as broken pieces of one continuous box, or whether each box is individually wrapped with the border and padding.\",browsers:\"FF32,S6.1,C22,O15\",restriction:\"enum\",values:[{name:\"clone\"},{name:\"slice\"}],syntax:\"slice | clone\"},{name:\"box-shadow\",desc:\"Attaches one or more drop-shadows to the box. The property is a comma-separated list of shadows, each specified by 2-4 length values, an optional color, and an optional 'inset' keyword. Omitted lengths are 0; omitted colors are a user agent chosen color.\",browsers:\"all\",restriction:\"length, color, enum\",values:[{name:\"inset\"}],syntax:\"none | <shadow>#\"},{name:\"box-sizing\",desc:\"Specifies the behavior of the 'width' and 'height' properties.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"border-box\"},{name:\"content-box\"}],syntax:\"content-box | border-box\"},{name:\"break-after\",desc:\"Describes the page/column/region break behavior after the generated box.\",browsers:\"E12,S10,C50,IE10,O37\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break before/after the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page/column break before/after the principal box.\"},{name:\"avoid\",desc:\"Avoid a break before/after the principal box.\"},{name:\"avoid-column\",desc:\"Avoid a column break before/after the principal box.\"},{name:\"avoid-page\",desc:\"Avoid a page break before/after the principal box.\"},{name:\"column\",desc:\"Always force a column break before/after the principal box.\"},{name:\"left\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a left page.\"},{name:\"page\",desc:\"Always force a page break before/after the principal box.\"},{name:\"right\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a right page.\"}],syntax:\"auto | avoid | avoid-page | page | left | right | recto | verso | avoid-column | column | avoid-region | region\"},{name:\"break-before\",desc:\"Describes the page/column/region break behavior before the generated box.\",browsers:\"E12,S10,C50,IE10,O37\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break before/after the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page/column break before/after the principal box.\"},{name:\"avoid\",desc:\"Avoid a break before/after the principal box.\"},{name:\"avoid-column\",desc:\"Avoid a column break before/after the principal box.\"},{name:\"avoid-page\",desc:\"Avoid a page break before/after the principal box.\"},{name:\"column\",desc:\"Always force a column break before/after the principal box.\"},{name:\"left\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a left page.\"},{name:\"page\",desc:\"Always force a page break before/after the principal box.\"},{name:\"right\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a right page.\"}],syntax:\"auto | avoid | avoid-page | page | left | right | recto | verso | avoid-column | column | avoid-region | region\"},{name:\"break-inside\",desc:\"Describes the page/column/region break behavior inside the principal box.\",browsers:\"E12,S10,C50,IE10,O37\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Impose no additional breaking constraints within the box.\"},{name:\"avoid\",desc:\"Avoid breaks within the box.\"},{name:\"avoid-column\",desc:\"Avoid a column break within the box.\"},{name:\"avoid-page\",desc:\"Avoid a page break within the box.\"}],syntax:\"auto | avoid | avoid-page | avoid-column | avoid-region\"},{name:\"caption-side\",desc:\"Specifies the position of the caption box with respect to the table box.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"bottom\",desc:\"Positions the caption box below the table box.\"},{name:\"top\",desc:\"Positions the caption box above the table box.\"}],syntax:\"top | bottom | block-start | block-end | inline-start | inline-end\"},{name:\"caret-color\",desc:\"Controls the color of the text insertion indicator.\",browsers:\"FF53,S11.1,C57,O44\",restriction:\"color, enum\",values:[{name:\"auto\",desc:\"The user agent selects an appropriate color for the caret. This is generally currentcolor, but the user agent may choose a different color to ensure good visibility and contrast with the surrounding content, taking into account the value of currentcolor, the background, shadows, and other factors.\"}],syntax:\"auto | <color>\"},{name:\"clear\",desc:\"Indicates which sides of an element's box(es) may not be adjacent to an earlier floating box. The 'clear' property does not consider floats inside the element itself or in other block formatting contexts.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"both\",desc:\"The clearance of the generated box is set to the amount necessary to place the top border edge below the bottom outer edge of any right-floating and left-floating boxes that resulted from elements earlier in the source document.\"},{name:\"left\",desc:\"The clearance of the generated box is set to the amount necessary to place the top border edge below the bottom outer edge of any left-floating boxes that resulted from elements earlier in the source document.\"},{name:\"none\",desc:\"No constraint on the box's position with respect to floats.\"},{name:\"right\",desc:\"The clearance of the generated box is set to the amount necessary to place the top border edge below the bottom outer edge of any right-floating boxes that resulted from elements earlier in the source document.\"}],syntax:\"none | left | right | both | inline-start | inline-end\"},{name:\"clip\",desc:\"Deprecated. Use the 'clip-path' property when support allows. Defines the visible portion of an element’s box.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The element does not clip.\"},{name:\"rect()\"}],syntax:\"<shape> | auto\"},{name:\"clip-path\",desc:\"Specifies a clipping path where everything inside the path is visable and everything outside is clipped out.\",browsers:\"FF3.5,C55,O42\",restriction:\"url, shape, geometry-box, enum\",values:[{name:\"none\",desc:\"No clipping path gets created.\"},{name:\"url()\",desc:\"References a <clipPath> element to create a clipping path.\"}],syntax:\"<clip-source> | [ <basic-shape> || <geometry-box> ] | none\"},{name:\"clip-rule\",desc:\"Indicates the algorithm which is to be used to determine what parts of the canvas are included inside the shape.\",browsers:\"E,C5,FF3,IE10,O9,S6\",restriction:\"enum\",values:[{name:\"evenodd\"},{name:\"nonzero\"}]},{name:\"color\",desc:\"Color of an element's text\",browsers:\"all\",restriction:\"color\",syntax:\"<color>\"},{name:\"color-interpolation-filters\",desc:\"Specifies the color space for imaging operations performed via filter effects.\",browsers:\"E,C5,FF3,IE10,O9,S6\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Color operations are not required to occur in a particular color space.\"},{name:\"linearRGB\"},{name:\"sRGB\"}]},{name:\"column-count\",desc:\"Describes the optimal number of columns into which the content of the element will be flowed.\",browsers:\"all\",restriction:\"integer, enum\",values:[{name:\"auto\",desc:\"Determines the number of columns by the 'column-width' property and the element width.\"}],syntax:\"<integer> | auto\"},{name:\"column-fill\",desc:\"In continuous media, this property will only be consulted if the length of columns has been constrained. Otherwise, columns will automatically be balanced.\",browsers:\"E12,FF52,C\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Fills columns sequentially.\"},{name:\"balance\"}],syntax:\"auto | balance | balance-all\"},{name:\"column-gap\",desc:\"Sets the gap between columns. If there is a column rule between columns, it will appear in the middle of the gap.\",browsers:\"E,IE10,O11.5,S9\",restriction:\"length, enum\",values:[{name:\"normal\",desc:\"User agent specific and typically equivalent to 1em.\"}],syntax:\"normal | <length-percentage>\"},{name:\"column-rule\",desc:\"Shorthand for setting 'column-rule-width', 'column-rule-style', and 'column-rule-color' at the same place in the style sheet. Omitted values are set to their initial values.\",browsers:\"all\",restriction:\"length, line-width, line-style, color\",syntax:\"<'column-rule-width'> || <'column-rule-style'> || <'column-rule-color'>\"},{name:\"column-rule-color\",desc:\"Sets the color of the column rule\",browsers:\"all\",restriction:\"color\",syntax:\"<color>\"},{name:\"column-rule-style\",desc:\"Sets the style of the rule between columns of an element.\",browsers:\"all\",restriction:\"line-style\",syntax:\"<'border-style'>\"},{name:\"column-rule-width\",desc:\"Sets the width of the rule between columns. Negative values are not allowed.\",browsers:\"all\",restriction:\"length, line-width\",syntax:\"<'border-width'>\"},{name:\"columns\",desc:\"A shorthand property which sets both 'column-width' and 'column-count'.\",browsers:\"all\",restriction:\"length, integer, enum\",values:[{name:\"auto\",desc:\"The width depends on the values of other properties.\"}],syntax:\"<'column-width'> || <'column-count'>\"},{name:\"column-span\",desc:\"Describes the page/column break behavior after the generated box.\",browsers:\"E12,S,C50,IE10,O11.1\",restriction:\"enum\",values:[{name:\"all\",desc:\"The element spans across all columns. Content in the normal flow that appears before the element is automatically balanced across all columns before the element appear.\"},{name:\"none\",desc:\"The element does not span multiple columns.\"}],syntax:\"none | all\"},{name:\"column-width\",desc:\"Describes the width of columns in multicol elements.\",browsers:\"all\",restriction:\"length, enum\",values:[{name:\"auto\",desc:\"The width depends on the values of other properties.\"}],syntax:\"<length> | auto\"},{name:\"contain\",desc:\"Indicates that an element and its contents are, as much as possible, independent of the rest of the document tree.\",browsers:\"C52,O40\",restriction:\"enum\",values:[{name:\"none\",desc:\"Indicates that the property has no effect.\"},{name:\"strict\",desc:\"Turns on all forms of containment for the element.\"},{name:\"content\",desc:\"All containment rules except size are applied to the element.\"},{name:\"size\"},{name:\"layout\"},{name:\"style\",desc:\"Turns on style containment for the element.\"},{name:\"paint\"}],status:\"e\",syntax:\"none | strict | content | [ size || layout || style || paint ]\"},{name:\"content\",desc:\"Determines which page-based occurrence of a given element is applied to a counter or string value.\",browsers:\"all\",restriction:\"string, url\",values:[{name:\"attr()\"},{name:\"counter(name)\"},{name:\"icon\",desc:\"The (pseudo-)element is replaced in its entirety by the resource referenced by its 'icon' property, and treated as a replaced element.\"},{name:\"none\",desc:\"On elements, this inhibits the children of the element from being rendered as children of this element, as if the element was empty. On pseudo-elements it causes the pseudo-element to have no content.\"},{name:\"normal\",desc:\"See http://www.w3.org/TR/css3-content/#content for computation rules.\"},{name:\"url()\"}],syntax:\"normal | none | [ <content-replacement> | <content-list> ] [/ <string> ]?\"},{name:\"counter-increment\",desc:\"Manipulate the value of existing counters.\",browsers:\"FF1,S3,C2,IE8,O9.2\",restriction:\"identifier, integer\",values:[{name:\"none\",desc:\"This element does not alter the value of any counters.\"}],syntax:\"[ <custom-ident> <integer>? ]+ | none\"},{name:\"counter-reset\",desc:\"Property accepts one or more names of counters (identifiers), each one optionally followed by an integer. The integer gives the value that the counter is set to on each occurrence of the element.\",browsers:\"all\",restriction:\"identifier, integer\",values:[{name:\"none\",desc:\"The counter is not modified.\"}],syntax:\"[ <custom-ident> <integer>? ]+ | none\"},{name:\"cursor\",desc:\"Allows control over cursor appearance in an element\",browsers:\"all\",restriction:\"url, number, enum\",values:[{name:\"alias\"},{name:\"all-scroll\"},{name:\"auto\",desc:\"The UA determines the cursor to display based on the current context.\"},{name:\"cell\"},{name:\"col-resize\"},{name:\"context-menu\"},{name:\"copy\"},{name:\"crosshair\"},{name:\"default\",desc:\"The platform-dependent default cursor. Often rendered as an arrow.\"},{name:\"e-resize\"},{name:\"ew-resize\"},{name:\"grab\",browsers:\"FF27\"},{name:\"grabbing\",browsers:\"FF27\"},{name:\"help\"},{name:\"move\"},{name:\"-moz-grab\",browsers:\"FF1.5\"},{name:\"-moz-grabbing\",browsers:\"FF1.5\"},{name:\"-moz-zoom-in\",browsers:\"FF\"},{name:\"-moz-zoom-out\",browsers:\"FF\"},{name:\"ne-resize\"},{name:\"nesw-resize\"},{name:\"no-drop\"},{name:\"none\",desc:\"No cursor is rendered for the element.\"},{name:\"not-allowed\"},{name:\"n-resize\"},{name:\"ns-resize\"},{name:\"nw-resize\"},{name:\"nwse-resize\"},{name:\"pointer\"},{name:\"progress\"},{name:\"row-resize\"},{name:\"se-resize\"},{name:\"s-resize\"},{name:\"sw-resize\"},{name:\"text\",desc:\"Indicates text that may be selected. Often rendered as a vertical I-beam.\"},{name:\"vertical-text\"},{name:\"wait\"},{name:\"-webkit-grab\",browsers:\"C,S4\"},{name:\"-webkit-grabbing\",browsers:\"C,S4\"},{name:\"-webkit-zoom-in\",browsers:\"C,S1.2\"},{name:\"-webkit-zoom-out\",browsers:\"C,S1.2\"},{name:\"w-resize\"},{name:\"zoom-in\",browsers:\"E,C37,FF24,O12.1,S9\"},{name:\"zoom-out\",browsers:\"E,C37,FF24,O12.1,S9\"}],syntax:\"[ [ <url> [ <x> <y> ]? , ]* [ auto | default | none | context-menu | help | pointer | progress | wait | cell | crosshair | text | vertical-text | alias | copy | move | no-drop | not-allowed | e-resize | n-resize | ne-resize | nw-resize | s-resize | se-resize | sw-resize | w-resize | ew-resize | ns-resize | nesw-resize | nwse-resize | col-resize | row-resize | all-scroll | zoom-in | zoom-out | grab | grabbing ] ]\"},{name:\"direction\",desc:\"Specifies the inline base direction or directionality of any bidi paragraph, embedding, isolate, or override established by the box. Note: for HTML content use the 'dir' attribute and 'bdo' element rather than this property.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"ltr\"},{name:\"rtl\"}],syntax:\"ltr | rtl\"},{name:\"display\",desc:\"In combination with 'float' and 'position', determines the type of box or boxes that are generated for an element.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"block\"},{name:\"contents\",desc:\"The element itself does not generate any boxes, but its children and pseudo-elements still generate boxes as normal.\",browsers:\"FF37\"},{name:\"flex\",browsers:\"E,C29,FF22,IE11,O12.1,S9\"},{name:\"flexbox\",browsers:\"O12.1\"},{name:\"flow-root\",browsers:\"C58,FF53,O45\"},{name:\"grid\",browsers:\"FF52,C57,S10.1,O44\"},{name:\"inline\",desc:\"The element generates an inline-level box.\"},{name:\"inline-block\"},{name:\"inline-flex\",browsers:\"E,C29,FF22,IE11,O12.1,S9\"},{name:\"inline-flexbox\",browsers:\"O12.1\"},{name:\"inline-table\"},{name:\"list-item\"},{name:\"-moz-box\",browsers:\"FF\"},{name:\"-moz-deck\",browsers:\"FF\"},{name:\"-moz-grid\",browsers:\"FF\"},{name:\"-moz-grid-group\",browsers:\"FF\"},{name:\"-moz-grid-line\",browsers:\"FF\"},{name:\"-moz-groupbox\",browsers:\"FF\"},{name:\"-moz-inline-box\",browsers:\"FF\"},{name:\"-moz-inline-grid\",browsers:\"FF\"},{name:\"-moz-inline-stack\",browsers:\"FF\"},{name:\"-moz-marker\",browsers:\"FF\"},{name:\"-moz-popup\",browsers:\"FF\"},{name:\"-moz-stack\",browsers:\"FF\"},{name:\"-ms-flexbox\",browsers:\"IE10\"},{name:\"-ms-grid\",browsers:\"E,IE10\"},{name:\"-ms-inline-flexbox\",browsers:\"IE10\"},{name:\"-ms-inline-grid\",browsers:\"E,IE10\"},{name:\"none\",desc:\"The element and its descendants generates no boxes.\"},{name:\"ruby\",desc:\"The element generates a principal ruby container box, and establishes a ruby formatting context.\"},{name:\"ruby-base\"},{name:\"ruby-base-container\"},{name:\"ruby-text\"},{name:\"ruby-text-container\"},{name:\"run-in\",browsers:\"IE8\"},{name:\"table\"},{name:\"table-caption\"},{name:\"table-cell\"},{name:\"table-column\"},{name:\"table-column-group\"},{name:\"table-footer-group\"},{name:\"table-header-group\"},{name:\"table-row\"},{name:\"table-row-group\"},{name:\"-webkit-box\",browsers:\"C,S1\"},{name:\"-webkit-flex\",browsers:\"C21,O15,S6.1\"},{name:\"-webkit-inline-box\",browsers:\"C,S1\"},{name:\"-webkit-inline-flex\",browsers:\"C21,O15,S6.1\"}],syntax:\"[ <display-outside> || <display-inside> ] | <display-listitem> | <display-internal> | <display-box> | <display-legacy>\"},{name:\"empty-cells\",desc:\"In the separated borders model, this property controls the rendering of borders and backgrounds around cells that have no visible content.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"hide\"},{name:\"-moz-show-background\",browsers:\"FF\"},{name:\"show\"}],syntax:\"show | hide\"},{name:\"enable-background\",desc:\"Deprecated. Use 'isolation' property instead when support allows. Specifies how the accumulation of the background image is managed.\",restriction:\"integer, length, percentage, enum\",values:[{name:\"accumulate\"},{name:\"new\"}]},{name:\"fallback\",desc:\"@counter-style descriptor. Specifies a fallback counter style to be used when the current counter style can’t create a representation for a given counter value.\",browsers:\"FF33\",restriction:\"identifier\",syntax:\"<counter-style-name>\"},{name:\"fill\",desc:\"Paints the interior of the given graphical element.\",restriction:\"color, enum, url\",values:[{name:\"url()\",desc:\"A URL reference to a paint server element, which is an element that defines a paint server: ‘hatch’, ‘linearGradient’, ‘mesh’, ‘pattern’, ‘radialGradient’ and ‘solidcolor’.\"}]},{name:\"fill-opacity\",desc:\"Specifies the opacity of the painting operation used to paint the interior the current object.\",restriction:\"number(0-1)\"},{name:\"fill-rule\",desc:\"Indicates the algorithm (or winding rule) which is to be used to determine what parts of the canvas are included inside the shape.\",restriction:\"enum\",values:[{name:\"evenodd\"},{name:\"nonzero\"}]},{name:\"filter\",desc:\"Processes an element’s rendering before it is displayed in the document, by applying one or more filter effects.\",browsers:\"E12,FF35,S6,C53,O40\",restriction:\"enum, url\",values:[{name:\"none\",desc:\"No filter effects are applied.\"},{name:\"blur()\"},{name:\"brightness()\"},{name:\"contrast()\"},{name:\"drop-shadow()\"},{name:\"grayscale()\"},{name:\"hue-rotate()\"},{name:\"invert()\"},{name:\"opacity()\"},{name:\"saturate()\"},{name:\"sepia()\"},{name:\"url()\",desc:\"A filter reference to a <filter> element.\",browsers:\"FF3.6\"}],syntax:\"none | <filter-function-list>\"},{name:\"flex\",desc:\"Specifies the components of a flexible length: the flex grow factor and flex shrink factor, and the flex basis.\",browsers:\"all\",restriction:\"length, number, percentage\",values:[{name:\"auto\",desc:\"Retrieves the value of the main size property as the used 'flex-basis'.\"},{name:\"content\",desc:\"Indicates automatic sizing, based on the flex item’s content.\",browsers:\"E,IE11\"},{name:\"none\",desc:\"Expands to '0 0 auto'.\"}],syntax:\"none | [ <'flex-grow'> <'flex-shrink'>? || <'flex-basis'> ]\"},{name:\"flex-basis\",desc:\"Sets the flex basis.\",browsers:\"all\",restriction:\"length, number, percentage\",values:[{name:\"auto\",desc:\"Retrieves the value of the main size property as the used 'flex-basis'.\"},{name:\"content\",desc:\"Indicates automatic sizing, based on the flex item’s content.\",browsers:\"E,IE11\"}],syntax:\"content | <'width'>\"},{name:\"flex-direction\",desc:\"Specifies how flex items are placed in the flex container, by setting the direction of the flex container’s main axis.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"column\",desc:\"The flex container’s main axis has the same orientation as the block axis of the current writing mode.\"},{name:\"column-reverse\"},{name:\"row\",desc:\"The flex container’s main axis has the same orientation as the inline axis of the current writing mode.\"},{name:\"row-reverse\"}],syntax:\"row | row-reverse | column | column-reverse\"},{name:\"flex-flow\",desc:\"Specifies how flexbox items are placed in the flexbox.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"column\",desc:\"The flex container’s main axis has the same orientation as the block axis of the current writing mode.\"},{name:\"column-reverse\"},{name:\"nowrap\",desc:\"The flex container is single-line.\"},{name:\"row\",desc:\"The flex container’s main axis has the same orientation as the inline axis of the current writing mode.\"},{name:\"row-reverse\"},{name:\"wrap\",desc:\"The flexbox is multi-line.\"},{name:\"wrap-reverse\"}],syntax:\"<'flex-direction'> || <'flex-wrap'>\"},{name:\"flex-grow\",desc:\"Sets the flex grow factor. Negative numbers are invalid.\",browsers:\"all\",restriction:\"number\",syntax:\"<number>\"},{name:\"flex-shrink\",desc:\"Sets the flex shrink factor. Negative numbers are invalid.\",browsers:\"all\",restriction:\"number\",syntax:\"<number>\"},{name:\"flex-wrap\",desc:\"Controls whether the flex container is single-line or multi-line, and the direction of the cross-axis, which determines the direction new lines are stacked in.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"nowrap\",desc:\"The flex container is single-line.\"},{name:\"wrap\",desc:\"The flexbox is multi-line.\"},{name:\"wrap-reverse\"}],syntax:\"nowrap | wrap | wrap-reverse\"},{name:\"float\",desc:\"Specifies how a box should be floated. It may be set for any element, but only applies to elements that generate boxes that are not absolutely positioned.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"inline-end\",browsers:\"FF55\"},{name:\"inline-start\",browsers:\"FF55\"},{name:\"left\",desc:\"The element generates a block box that is floated to the left. Content flows on the right side of the box, starting at the top (subject to the 'clear' property).\"},{name:\"none\",desc:\"The box is not floated.\"},{name:\"right\",desc:\"Similar to 'left', except the box is floated to the right, and content flows on the left side of the box, starting at the top.\"}],syntax:\"left | right | none | inline-start | inline-end\"},{name:\"flood-color\",desc:\"Indicates what color to use to flood the current filter primitive subregion.\",browsers:\"E,C5,FF3,IE10,O9,S6\",restriction:\"color\"},{name:\"flood-opacity\",desc:\"Indicates what opacity to use to flood the current filter primitive subregion.\",browsers:\"E,C5,FF3,IE10,O9,S6\",restriction:\"number(0-1), percentage\"},{name:\"font\",desc:\"Shorthand property for setting 'font-style', 'font-variant', 'font-weight', 'font-size', 'line-height', and 'font-family', at the same place in the style sheet. The syntax of this property is based on a traditional typographical shorthand notation to set multiple properties related to fonts.\",browsers:\"all\",restriction:\"font\",values:[{name:\"100\"},{name:\"200\"},{name:\"300\"},{name:\"400\"},{name:\"500\"},{name:\"600\"},{name:\"700\"},{name:\"800\"},{name:\"900\"},{name:\"bold\"},{name:\"bolder\"},{name:\"caption\"},{name:\"icon\",desc:\"The font used to label icons.\"},{name:\"italic\",desc:\"Selects a font that is labeled 'italic', or, if that is not available, one labeled 'oblique'.\"},{name:\"large\"},{name:\"larger\"},{name:\"lighter\"},{name:\"medium\"},{name:\"menu\"},{name:\"message-box\"},{name:\"normal\",desc:\"Specifies a face that is not labeled as a small-caps font.\"},{name:\"oblique\",desc:\"Selects a font that is labeled 'oblique'.\"},{name:\"small\"},{name:\"small-caps\",desc:\"Specifies a font that is labeled as a small-caps font. If a genuine small-caps font is not available, user agents should simulate a small-caps font.\"},{name:\"small-caption\"},{name:\"smaller\"},{name:\"status-bar\"},{name:\"x-large\"},{name:\"x-small\"},{name:\"xx-large\"},{name:\"xx-small\"}],syntax:\"[ [ <'font-style'> || <font-variant-css21> || <'font-weight'> || <'font-stretch'> ]? <'font-size'> [ / <'line-height'> ]? <'font-family'> ] | caption | icon | menu | message-box | small-caption | status-bar\"},{name:\"font-family\",desc:\"Specifies a prioritized list of font family names or generic family names. A user agent iterates through the list of family names until it matches an available font that contains a glyph for the character to be rendered.\",browsers:\"all\",restriction:\"font\",values:[{name:\"-apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen, Ubuntu, Cantarell, 'Open Sans', 'Helvetica Neue', sans-serif\"},{name:\"Arial, Helvetica, sans-serif\"},{name:\"Cambria, Cochin, Georgia, Times, 'Times New Roman', serif\"},{name:\"'Courier New', Courier, monospace\"},{name:\"cursive\"},{name:\"fantasy\"},{name:\"'Franklin Gothic Medium', 'Arial Narrow', Arial, sans-serif\"},{name:\"Georgia, 'Times New Roman', Times, serif\"},{name:\"'Gill Sans', 'Gill Sans MT', Calibri, 'Trebuchet MS', sans-serif\"},{name:\"Impact, Haettenschweiler, 'Arial Narrow Bold', sans-serif\"},{name:\"'Lucida Sans', 'Lucida Sans Regular', 'Lucida Grande', 'Lucida Sans Unicode', Geneva, Verdana, sans-serif\"},{name:\"monospace\"},{name:\"sans-serif\"},{name:\"'Segoe UI', Tahoma, Geneva, Verdana, sans-serif\"},{name:\"serif\"},{name:\"'Times New Roman', Times, serif\"},{name:\"'Trebuchet MS', 'Lucida Sans Unicode', 'Lucida Grande', 'Lucida Sans', Arial, sans-serif\"},{name:\"Verdana, Geneva, Tahoma, sans-serif\"}],syntax:\"<family-name>\"},{name:\"font-feature-settings\",desc:\"Provides low-level control over OpenType font features. It is intended as a way of providing access to font features that are not widely used but are needed for a particular use case.\",browsers:\"all\",restriction:\"string, integer\",values:[{name:'\"aalt\"'},{name:'\"abvf\"'},{name:'\"abvm\"'},{name:'\"abvs\"'},{name:'\"afrc\"'},{name:'\"akhn\"'},{name:'\"blwf\"'},{name:'\"blwm\"'},{name:'\"blws\"'},{name:'\"calt\"'},{name:'\"case\"'},{name:'\"ccmp\"'},{name:'\"cfar\"'},{name:'\"cjct\"'},{name:'\"clig\"'},{name:'\"cpct\"'},{name:'\"cpsp\"'},{name:'\"cswh\"'},{name:'\"curs\"'},{name:'\"c2pc\"'},{name:'\"c2cs\"',desc:\"Small Capitals From Capitals. Applies only to bicameral scripts.\"},{name:'\"dist\"'},{name:'\"dlig\"',desc:\"Discretionary ligatures.\"},{name:'\"dnom\"'},{name:'\"dtls\"'},{name:'\"expt\"'},{name:'\"falt\"'},{name:'\"fin2\"'},{name:'\"fin3\"'},{name:'\"fina\"'},{name:'\"flac\"'},{name:'\"frac\"'},{name:'\"fwid\"'},{name:'\"half\"'},{name:'\"haln\"'},{name:'\"halt\"'},{name:'\"hist\"'},{name:'\"hkna\"'},{name:'\"hlig\"'},{name:'\"hngl\"'},{name:'\"hojo\"'},{name:'\"hwid\"'},{name:'\"init\"'},{name:'\"isol\"'},{name:'\"ital\"'},{name:'\"jalt\"'},{name:'\"jp78\"'},{name:'\"jp83\"'},{name:'\"jp90\"'},{name:'\"jp04\"'},{name:'\"kern\"',desc:\"Kerning.\"},{name:'\"lfbd\"'},{name:'\"liga\"',desc:\"Standard Ligatures.\"},{name:'\"ljmo\"'},{name:'\"lnum\"',desc:\"Lining Figures.\"},{name:'\"locl\"'},{name:'\"ltra\"'},{name:'\"ltrm\"'},{name:'\"mark\"'},{name:'\"med2\"'},{name:'\"medi\"'},{name:'\"mgrk\"'},{name:'\"mkmk\"'},{name:'\"nalt\"'},{name:'\"nlck\"'},{name:'\"nukt\"'},{name:'\"numr\"'},{name:'\"onum\"',desc:\"Oldstyle Figures.\"},{name:'\"opbd\"'},{name:'\"ordn\"'},{name:'\"ornm\"'},{name:'\"palt\"'},{name:'\"pcap\"'},{name:'\"pkna\"'},{name:'\"pnum\"'},{name:'\"pref\"'},{name:'\"pres\"'},{name:'\"pstf\"'},{name:'\"psts\"'},{name:'\"pwid\"'},{name:'\"qwid\"'},{name:'\"rand\"'},{name:'\"rclt\"'},{name:'\"rlig\"'},{name:'\"rkrf\"'},{name:'\"rphf\"'},{name:'\"rtbd\"'},{name:'\"rtla\"'},{name:'\"rtlm\"'},{name:'\"ruby\"'},{name:'\"salt\"'},{name:'\"sinf\"'},{name:'\"size\"'},{name:'\"smcp\"',desc:\"Small Capitals. Applies only to bicameral scripts.\"},{name:'\"smpl\"'},{name:'\"ssty\"'},{name:'\"stch\"'},{name:'\"subs\"'},{name:'\"sups\"'},{name:'\"swsh\"',desc:\"Swash. Does not apply to ideographic scripts.\"},{name:'\"titl\"'},{name:'\"tjmo\"'},{name:'\"tnam\"'},{name:'\"tnum\"',desc:\"Tabular Figures.\"},{name:'\"trad\"'},{name:'\"twid\"'},{name:'\"unic\"'},{name:'\"valt\"'},{name:'\"vatu\"'},{name:'\"vert\"'},{name:'\"vhal\"'},{name:'\"vjmo\"'},{name:'\"vkna\"'},{name:'\"vkrn\"'},{name:'\"vpal\"'},{name:'\"vrt2\"'},{name:'\"zero\"'},{name:\"normal\",desc:\"No change in glyph substitution or positioning occurs.\"},{name:\"off\",desc:\"Disable feature.\"},{name:\"on\",desc:\"Enable feature.\"}],syntax:\"normal | <feature-tag-value>#\"},{name:\"font-kerning\",desc:\"Kerning is the contextual adjustment of inter-glyph spacing. This property controls metric kerning, kerning that utilizes adjustment data contained in the font.\",browsers:\"FF32,S7,C32\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Specifies that kerning is applied at the discretion of the user agent.\"},{name:\"none\",desc:\"Specifies that kerning is not applied.\"},{name:\"normal\",desc:\"Specifies that kerning is applied.\"}],syntax:\"auto | normal | none\"},{name:\"font-language-override\",desc:\"The value of 'normal' implies that when rendering with OpenType fonts the language of the document is used to infer the OpenType language system, used to select language specific features when rendering.\",browsers:\"FF34\",restriction:\"string\",values:[{name:\"normal\",desc:\"Implies that when rendering with OpenType fonts the language of the document is used to infer the OpenType language system, used to select language specific features when rendering.\"}],syntax:\"normal | <string>\"},{name:\"font-size\",desc:\"Indicates the desired height of glyphs from the font. For scalable fonts, the font-size is a scale factor applied to the EM unit of the font. (Note that certain glyphs may bleed outside their EM box.) For non-scalable fonts, the font-size is converted into absolute units and matched against the declared font-size of the font, using the same absolute coordinate space for both of the matched values.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"large\"},{name:\"larger\"},{name:\"medium\"},{name:\"small\"},{name:\"smaller\"},{name:\"x-large\"},{name:\"x-small\"},{name:\"xx-large\"},{name:\"xx-small\"}],syntax:\"<absolute-size> | <relative-size> | <length-percentage>\"},{name:\"font-size-adjust\",desc:\"Preserves the readability of text when font fallback occurs by adjusting the font-size so that the x-height is the same irregardless of the font used.\",browsers:\"FF40,C43,O30\",restriction:\"number\",values:[{name:\"none\",desc:\"Do not preserve the font’s x-height.\"}],syntax:\"none | <number>\"},{name:\"font-stretch\",desc:\"Selects a normal, condensed, or expanded face from a font family.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"condensed\"},{name:\"expanded\"},{name:\"extra-condensed\"},{name:\"extra-expanded\"},{name:\"narrower\",browsers:\"E,IE10\"},{name:\"normal\"},{name:\"semi-condensed\"},{name:\"semi-expanded\"},{name:\"ultra-condensed\"},{name:\"ultra-expanded\"},{name:\"wider\",browsers:\"E,IE10\"}],syntax:\"normal | ultra-condensed | extra-condensed | condensed | semi-condensed | semi-expanded | expanded | extra-expanded | ultra-expanded\"},{name:\"font-style\",desc:\"Allows italic or oblique faces to be selected. Italic forms are generally cursive in nature while oblique faces are typically sloped versions of the regular face.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"italic\",desc:\"Selects a font that is labeled as an 'italic' face, or an 'oblique' face if one is not\"},{name:\"normal\",desc:\"Selects a face that is classified as 'normal'.\"},{name:\"oblique\",desc:\"Selects a font that is labeled as an 'oblique' face, or an 'italic' face if one is not.\"}],syntax:\"normal | italic | oblique\"},{name:\"font-synthesis\",desc:\"Controls whether user agents are allowed to synthesize bold or oblique font faces when a font family lacks bold or italic faces.\",browsers:\"FF34,S9\",restriction:\"enum\",values:[{name:\"none\",desc:\"Disallow all synthetic faces.\"},{name:\"style\",desc:\"Allow synthetic italic faces.\"},{name:\"weight\"}],syntax:\"none | [ weight || style ]\"},{name:\"font-variant\",desc:\"Specifies variant representations of the font\",browsers:\"all\",restriction:\"enum\",values:[{name:\"normal\",desc:\"Specifies a face that is not labeled as a small-caps font.\"},{name:\"small-caps\",desc:\"Specifies a font that is labeled as a small-caps font. If a genuine small-caps font is not available, user agents should simulate a small-caps font.\"}],syntax:\"normal | none | [ <common-lig-values> || <discretionary-lig-values> || <historical-lig-values> || <contextual-alt-values> || stylistic(<feature-value-name>) || historical-forms || styleset(<feature-value-name>#) || character-variant(<feature-value-name>#) || swash(<feature-value-name>) || ornaments(<feature-value-name>) || annotation(<feature-value-name>) || [ small-caps | all-small-caps | petite-caps | all-petite-caps | unicase | titling-caps ] || <numeric-figure-values> || <numeric-spacing-values> || <numeric-fraction-values> || ordinal || slashed-zero || <east-asian-variant-values> || <east-asian-width-values> || ruby ]\"},{name:\"font-variant-alternates\",desc:\"For any given character, fonts can provide a variety of alternate glyphs in addition to the default glyph for that character. This property provides control over the selection of these alternate glyphs.\",browsers:\"FF34\",restriction:\"enum\",values:[{name:\"annotation()\"},{name:\"character-variant()\"},{name:\"historical-forms\"},{name:\"normal\",desc:\"None of the features are enabled.\"},{name:\"ornaments()\"},{name:\"styleset()\"},{name:\"stylistic()\"},{name:\"swash()\"}],syntax:\"normal | [ stylistic( <feature-value-name> ) || historical-forms || styleset( <feature-value-name># ) || character-variant( <feature-value-name># ) || swash( <feature-value-name> ) || ornaments( <feature-value-name> ) || annotation( <feature-value-name> ) ]\"},{name:\"font-variant-caps\",desc:\"Specifies control over capitalized forms.\",browsers:\"FF34,C52,O39\",restriction:\"enum\",values:[{name:\"all-petite-caps\"},{name:\"all-small-caps\"},{name:\"normal\",desc:\"None of the features are enabled.\"},{name:\"petite-caps\"},{name:\"small-caps\",desc:\"Enables display of small capitals. Small-caps glyphs typically use the form of uppercase letters but are reduced to the size of lowercase letters.\"},{name:\"titling-caps\"},{name:\"unicase\"}],syntax:\"normal | small-caps | all-small-caps | petite-caps | all-petite-caps | unicase | titling-caps\"},{name:\"font-variant-east-asian\",desc:\"Allows control of glyph substitute and positioning in East Asian text.\",browsers:\"FF34,C63,O50\",restriction:\"enum\",values:[{name:\"full-width\"},{name:\"jis04\"},{name:\"jis78\"},{name:\"jis83\"},{name:\"jis90\"},{name:\"normal\",desc:\"None of the features are enabled.\"},{name:\"proportional-width\"},{name:\"ruby\",desc:\"Enables display of ruby variant glyphs.\"},{name:\"simplified\"},{name:\"traditional\"}],syntax:\"normal | [ <east-asian-variant-values> || <east-asian-width-values> || ruby ]\"},{name:\"font-variant-ligatures\",desc:\"Specifies control over which ligatures are enabled or disabled. A value of ‘normal’ implies that the defaults set by the font are used.\",browsers:\"FF34,S9.1,C34,O21\",restriction:\"enum\",values:[{name:\"additional-ligatures\"},{name:\"common-ligatures\"},{name:\"contextual\",browsers:\"C35,F34,O22\"},{name:\"discretionary-ligatures\"},{name:\"historical-ligatures\"},{name:\"no-additional-ligatures\"},{name:\"no-common-ligatures\"},{name:\"no-contextual\",browsers:\"C35,F34,O22\"},{name:\"no-discretionary-ligatures\"},{name:\"no-historical-ligatures\"},{name:\"none\",desc:\"Disables all ligatures.\",browsers:\"FF34\"},{name:\"normal\",desc:\"Implies that the defaults set by the font are used.\"}],syntax:\"normal | none | [ <common-lig-values> || <discretionary-lig-values> || <historical-lig-values> || <contextual-alt-values> ]\"},{name:\"font-variant-numeric\",desc:\"Specifies control over numerical forms.\",browsers:\"FF34,S9.1,C52,O39\",restriction:\"enum\",values:[{name:\"diagonal-fractions\"},{name:\"lining-nums\"},{name:\"normal\",desc:\"None of the features are enabled.\"},{name:\"oldstyle-nums\"},{name:\"ordinal\"},{name:\"proportional-nums\"},{name:\"slashed-zero\"},{name:\"stacked-fractions\"},{name:\"tabular-nums\"}],syntax:\"normal | [ <numeric-figure-values> || <numeric-spacing-values> || <numeric-fraction-values> || ordinal || slashed-zero ]\"},{name:\"font-variant-position\",desc:\"Specifies the vertical position\",browsers:\"FF34\",restriction:\"enum\",values:[{name:\"normal\",desc:\"None of the features are enabled.\"},{name:\"sub\",desc:\"Enables display of subscript variants (OpenType feature: subs).\"},{name:\"super\",desc:\"Enables display of superscript variants (OpenType feature: sups).\"}],syntax:\"normal | sub | super\"},{name:\"font-weight\",desc:\"Specifies weight of glyphs in the font, their degree of blackness or stroke thickness.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"100\"},{name:\"200\"},{name:\"300\"},{name:\"400\"},{name:\"500\"},{name:\"600\"},{name:\"700\"},{name:\"800\"},{name:\"900\"},{name:\"bold\"},{name:\"bolder\"},{name:\"lighter\"},{name:\"normal\",desc:\"Same as 400\"}],syntax:\"normal | bold | 100 | 200 | 300 | 400 | 500 | 600 | 700 | 800 | 900\"},{name:\"glyph-orientation-horizontal\",desc:\"Controls glyph orientation when the inline-progression-direction is horizontal.\",restriction:\"angle, number\"},{name:\"glyph-orientation-vertical\",desc:\"Controls glyph orientation when the inline-progression-direction is vertical.\",restriction:\"angle, number, enum\",values:[{name:\"auto\",desc:\"Sets the orientation based on the fullwidth or non-fullwidth characters and the most common orientation.\"}]},{name:\"grid-area\",desc:\"Determine a grid item’s size and location within the grid by contributing a line, a span, or nothing (automatic) to its grid placement. Shorthand for 'grid-row-start', 'grid-column-start', 'grid-row-end', and 'grid-column-end'.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, integer\",values:[{name:\"auto\",desc:\"The property contributes nothing to the grid item’s placement, indicating auto-placement, an automatic span, or a default span of one.\"},{name:\"span\"}],syntax:\"<grid-line> [ / <grid-line> ]{0,3}\"},{name:\"grid\",desc:\"The grid CSS property is a shorthand property that sets all of the explicit grid properties ('grid-template-rows', 'grid-template-columns', and 'grid-template-areas'), and all the implicit grid properties ('grid-auto-rows', 'grid-auto-columns', and 'grid-auto-flow'), in a single declaration.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, length, percentage, string, enum\",syntax:\"<'grid-template'> | <'grid-template-rows'> / [ auto-flow && dense? ] <'grid-auto-columns'>? | [ auto-flow && dense? ] <'grid-auto-rows'>? / <'grid-template-columns'>\"},{name:\"grid-auto-columns\",desc:\"Specifies the size of implicitly created columns.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"min-content\",desc:\"Represents the largest min-content contribution of the grid items occupying the grid track.\"},{name:\"max-content\",desc:\"Represents the largest max-content contribution of the grid items occupying the grid track.\"},{name:\"auto\",desc:\"As a maximum, identical to 'max-content'. As a minimum, represents the largest minimum size (as specified by min-width/min-height) of the grid items occupying the grid track.\"},{name:\"minmax()\"}],syntax:\"<track-size>+\"},{name:\"grid-auto-flow\",desc:\"Controls how the auto-placement algorithm works, specifying exactly how auto-placed items get flowed into the grid.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"enum\",values:[{name:\"row\",desc:\"The auto-placement algorithm places items by filling each row in turn, adding new rows as necessary.\"},{name:\"column\",desc:\"The auto-placement algorithm places items by filling each column in turn, adding new columns as necessary.\"},{name:\"dense\"}],syntax:\"[ row | column ] || dense\"},{name:\"grid-auto-rows\",desc:\"Specifies the size of implicitly created rows.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"min-content\",desc:\"Represents the largest min-content contribution of the grid items occupying the grid track.\"},{name:\"max-content\",desc:\"Represents the largest max-content contribution of the grid items occupying the grid track.\"},{name:\"auto\",desc:\"As a maximum, identical to 'max-content'. As a minimum, represents the largest minimum size (as specified by min-width/min-height) of the grid items occupying the grid track.\"},{name:\"minmax()\"}],syntax:\"<track-size>+\"},{name:\"grid-column\",desc:\"Shorthand for 'grid-column-start' and 'grid-column-end'.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, integer, enum\",values:[{name:\"auto\",desc:\"The property contributes nothing to the grid item’s placement, indicating auto-placement, an automatic span, or a default span of one.\"},{name:\"span\"}],syntax:\"<grid-line> [ / <grid-line> ]?\"},{name:\"grid-column-end\",desc:\"Determine a grid item’s size and location within the grid by contributing a line, a span, or nothing (automatic) to its grid placement.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, integer, enum\",values:[{name:\"auto\",desc:\"The property contributes nothing to the grid item’s placement, indicating auto-placement, an automatic span, or a default span of one.\"},{name:\"span\"}],syntax:\"<grid-line>\"},{name:\"grid-column-gap\",desc:\"Specifies the gutters between grid columns.\",browsers:\"FF52,C57,S10.1,O44\",restriction:\"length\",status:\"o\",syntax:\"<length-percentage>\"},{name:\"grid-column-start\",desc:\"Determine a grid item’s size and location within the grid by contributing a line, a span, or nothing (automatic) to its grid placement.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, integer, enum\",values:[{name:\"auto\",desc:\"The property contributes nothing to the grid item’s placement, indicating auto-placement, an automatic span, or a default span of one.\"},{name:\"span\"}],syntax:\"<grid-line>\"},{name:\"grid-gap\",desc:\"Shorthand that specifies the gutters between grid columns and grid rows in one declaration.\",browsers:\"FF52,C57,S10.1,O44\",restriction:\"length\",status:\"o\",syntax:\"<'grid-row-gap'> <'grid-column-gap'>?\"},{name:\"grid-row\",desc:\"Shorthand for 'grid-row-start' and 'grid-row-end'.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, integer, enum\",values:[{name:\"auto\",desc:\"The property contributes nothing to the grid item’s placement, indicating auto-placement, an automatic span, or a default span of one.\"},{name:\"span\"}],syntax:\"<grid-line> [ / <grid-line> ]?\"},{name:\"grid-row-end\",desc:\"Determine a grid item’s size and location within the grid by contributing a line, a span, or nothing (automatic) to its grid placement.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, integer, enum\",values:[{name:\"auto\",desc:\"The property contributes nothing to the grid item’s placement, indicating auto-placement, an automatic span, or a default span of one.\"},{name:\"span\"}],syntax:\"<grid-line>\"},{name:\"grid-row-gap\",desc:\"Specifies the gutters between grid rows.\",browsers:\"FF52,C57,S10.1,O44\",restriction:\"length\",status:\"o\",syntax:\"<length-percentage>\"},{name:\"grid-row-start\",desc:\"Determine a grid item’s size and location within the grid by contributing a line, a span, or nothing (automatic) to its grid placement.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, integer, enum\",values:[{name:\"auto\",desc:\"The property contributes nothing to the grid item’s placement, indicating auto-placement, an automatic span, or a default span of one.\"},{name:\"span\"}],syntax:\"<grid-line>\"},{name:\"grid-template\",desc:\"Shorthand for setting grid-template-columns, grid-template-rows, and grid-template-areas in a single declaration.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, length, percentage, string, enum\",values:[{name:\"none\",desc:\"Sets all three properties to their initial values.\"},{name:\"min-content\",desc:\"Represents the largest min-content contribution of the grid items occupying the grid track.\"},{name:\"max-content\",desc:\"Represents the largest max-content contribution of the grid items occupying the grid track.\"},{name:\"auto\",desc:\"As a maximum, identical to 'max-content'. As a minimum, represents the largest minimum size (as specified by min-width/min-height) of the grid items occupying the grid track.\"},{name:\"subgrid\",desc:\"Sets 'grid-template-rows' and 'grid-template-columns' to 'subgrid', and 'grid-template-areas' to its initial value.\"},{name:\"minmax()\"},{name:\"repeat()\",desc:\"Represents a repeated fragment of the track list, allowing a large number of columns or rows that exhibit a recurring pattern to be written in a more compact form.\"}],syntax:\"none | [ <'grid-template-rows'> / <'grid-template-columns'> ] | [ <line-names>? <string> <track-size>? <line-names>? ]+ [ / <explicit-track-list> ]?\"},{name:\"grid-template-areas\",desc:\"Specifies named grid areas, which are not associated with any particular grid item, but can be referenced from the grid-placement properties.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"string\",values:[{name:\"none\",desc:\"The grid container doesn’t define any named grid areas.\"}],syntax:\"none | <string>+\"},{name:\"grid-template-columns\",desc:\"specifies, as a space-separated track list, the line names and track sizing functions of the grid.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, length, percentage, enum\",values:[{name:\"none\",desc:\"There is no explicit grid; any rows/columns will be implicitly generated.\"},{name:\"min-content\",desc:\"Represents the largest min-content contribution of the grid items occupying the grid track.\"},{name:\"max-content\",desc:\"Represents the largest max-content contribution of the grid items occupying the grid track.\"},{name:\"auto\",desc:\"As a maximum, identical to 'max-content'. As a minimum, represents the largest minimum size (as specified by min-width/min-height) of the grid items occupying the grid track.\"},{name:\"subgrid\",desc:\"Indicates that the grid will align to its parent grid in that axis.\"},{name:\"minmax()\"},{name:\"repeat()\",desc:\"Represents a repeated fragment of the track list, allowing a large number of columns or rows that exhibit a recurring pattern to be written in a more compact form.\"}],syntax:\"none | <track-list> | <auto-track-list>\"},{name:\"grid-template-rows\",desc:\"specifies, as a space-separated track list, the line names and track sizing functions of the grid.\",browsers:\"E16,FF52,S10.1,C57,O44\",restriction:\"identifier, length, percentage, string, enum\",values:[{name:\"none\",desc:\"There is no explicit grid; any rows/columns will be implicitly generated.\"},{name:\"min-content\",desc:\"Represents the largest min-content contribution of the grid items occupying the grid track.\"},{name:\"max-content\",desc:\"Represents the largest max-content contribution of the grid items occupying the grid track.\"},{name:\"auto\",desc:\"As a maximum, identical to 'max-content'. As a minimum, represents the largest minimum size (as specified by min-width/min-height) of the grid items occupying the grid track.\"},{name:\"subgrid\",desc:\"Indicates that the grid will align to its parent grid in that axis.\"},{name:\"minmax()\"},{name:\"repeat()\",desc:\"Represents a repeated fragment of the track list, allowing a large number of columns or rows that exhibit a recurring pattern to be written in a more compact form.\"}],syntax:\"none | <track-list> | <auto-track-list>\"},{name:\"height\",desc:\"Specifies the height of the content area, padding area or border area (depending on 'box-sizing') of certain boxes.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"The height depends on the values of other properties.\"},{name:\"fit-content\",browsers:\"C46,O33\"},{name:\"max-content\",desc:\"Use the max-content inline size or max-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"},{name:\"min-content\",desc:\"Use the min-content inline size or min-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"}],syntax:\"<viewport-length>{1,2}\"},{name:\"hyphens\",desc:\"Controls whether hyphenation is allowed to create more break opportunities within a line of text.\",browsers:\"FF43,S5.1,C55,IE10,O44\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Conditional hyphenation characters inside a word, if present, take priority over automatic resources when determining hyphenation points within the word.\"},{name:\"manual\"},{name:\"none\",desc:\"Words are not broken at line breaks, even if characters inside the word suggest line break points.\"}],syntax:\"none | manual | auto\"},{name:\"image-orientation\",desc:\"Specifies an orthogonal rotation to be applied to an image before it is laid out.\",browsers:\"FF26\",restriction:\"angle\",values:[{name:\"flip\"},{name:\"from-image\"}],syntax:\"from-image | <angle> | [ <angle>? flip ]\"},{name:\"image-rendering\",desc:\"Provides a hint to the user-agent about what aspects of an image are most important to preserve when the image is scaled, to aid the user-agent in the choice of an appropriate scaling algorithm.\",browsers:\"FF3.6,S,C,O\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The image should be scaled with an algorithm that maximizes the appearance of the image.\"},{name:\"crisp-edges\"},{name:\"-moz-crisp-edges\",browsers:\"FF\"},{name:\"optimizeQuality\"},{name:\"optimizeSpeed\",desc:\"Deprecated.\"},{name:\"pixelated\"}],syntax:\"auto | crisp-edges | pixelated\"},{name:\"ime-mode\",desc:\"Controls the state of the input method editor for text fields.\",browsers:\"FF3,IE5\",restriction:\"enum\",values:[{name:\"active\"},{name:\"auto\",desc:\"No change is made to the current input method editor state. This is the default.\"},{name:\"disabled\"},{name:\"inactive\"},{name:\"normal\",desc:\"The IME state should be normal; this value can be used in a user style sheet to override the page setting.\"}],status:\"o\",syntax:\"auto | normal | active | inactive | disabled\"},{name:\"inline-size\",desc:\"Logical 'height'. Mapping depends on the element’s 'writing-mode'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"Depends on the values of other properties.\"}],syntax:\"<'width'>\"},{name:\"isolation\",desc:\"In CSS setting to 'isolate' will turn the element into a stacking context. In SVG, it defines whether an element is isolated or not.\",browsers:\"FF36,S,C41,O30\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Elements are not isolated unless an operation is applied that causes the creation of a stacking context.\"},{name:\"isolate\",desc:\"In CSS will turn the element into a stacking context.\"}],syntax:\"auto | isolate\"},{name:\"justify-content\",desc:\"Aligns flex items along the main axis of the current line of the flex container.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"center\",desc:\"Flex items are packed toward the center of the line.\"},{name:\"start\",desc:\"The items are packed flush to each other toward the start edge of the alignment container in the main axis.\"},{name:\"end\",desc:\"The items are packed flush to each other toward the end edge of the alignment container in the main axis.\"},{name:\"left\",desc:\"The items are packed flush to each other toward the left edge of the alignment container in the main axis.\"},{name:\"right\",desc:\"The items are packed flush to each other toward the right edge of the alignment container in the main axis.\"},{name:\"safe\"},{name:\"unsafe\"},{name:\"stretch\",desc:\"If the combined size of the alignment subjects is less than the size of the alignment container, any auto-sized alignment subjects have their size increased equally (not proportionally), while still respecting the constraints imposed by max-height/max-width (or equivalent functionality), so that the combined size exactly fills the alignment container.\"},{name:\"space-evenly\"},{name:\"flex-end\",desc:\"Flex items are packed toward the end of the line.\"},{name:\"flex-start\",desc:\"Flex items are packed toward the start of the line.\"},{name:\"space-around\",desc:\"Flex items are evenly distributed in the line, with half-size spaces on either end.\"},{name:\"space-between\",desc:\"Flex items are evenly distributed in the line.\"},{name:\"baseline\",desc:\"Specifies participation in first-baseline alignment.\"},{name:\"first baseline\",desc:\"Specifies participation in first-baseline alignment.\"},{name:\"last baseline\",desc:\"Specifies participation in last-baseline alignment.\"}],syntax:\"normal | <content-distribution> | <overflow-position>? [ <content-position> | left | right ]\"},{name:\"kerning\",desc:\"Indicates whether the user agent should adjust inter-glyph spacing based on kerning tables that are included in the relevant font or instead disable auto-kerning and set inter-character spacing to a specific length.\",restriction:\"length, enum\",values:[{name:\"auto\",desc:\"Indicates that the user agent should adjust inter-glyph spacing based on kerning tables that are included in the font that will be used.\"}]},{name:\"left\",desc:\"Specifies how far an absolutely positioned box's left margin edge is offset to the right of the left edge of the box's 'containing block'.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"letter-spacing\",desc:\"Specifies the minimum, maximum, and optimal spacing between grapheme clusters.\",browsers:\"all\",restriction:\"length\",values:[{name:\"normal\",desc:\"The spacing is the normal spacing for the current font. It is typically zero-length.\"}],syntax:\"normal | <length>\"},{name:\"lighting-color\",desc:\"Defines the color of the light source for filter primitives 'feDiffuseLighting' and 'feSpecularLighting'.\",browsers:\"E,C5,FF3,IE10,O9,S6\",restriction:\"color\"},{name:\"line-break\",desc:\"Specifies what set of line breaking restrictions are in effect within the element.\",browsers:\"E14,S,C58,IE5.5,O45\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The UA determines the set of line-breaking restrictions to use for CJK scripts, and it may vary the restrictions based on the length of the line; e.g., use a less restrictive set of line-break rules for short lines.\"},{name:\"loose\",desc:\"Breaks text using the least restrictive set of line-breaking rules. Typically used for short lines, such as in newspapers.\"},{name:\"normal\",desc:\"Breaks text using the most common set of line-breaking rules.\"},{name:\"strict\",desc:\"Breaks CJK scripts using a more restrictive set of line-breaking rules than 'normal'.\"}],syntax:\"auto | loose | normal | strict\"},{name:\"line-height\",desc:\"Determines the block-progression dimension of the text content area of an inline box.\",browsers:\"all\",restriction:\"number, length, percentage\",values:[{name:\"normal\",desc:\"Tells user agents to set the computed value to a 'reasonable' value based on the font size of the element.\"}],syntax:\"normal | <number> | <length> | <percentage>\"},{name:\"list-style\",desc:\"Shorthand for setting 'list-style-type', 'list-style-position' and 'list-style-image'\",browsers:\"all\",restriction:\"image, enum, url\",values:[{name:\"armenian\"},{name:\"circle\"},{name:\"decimal\"},{name:\"decimal-leading-zero\"},{name:\"disc\"},{name:\"georgian\"},{name:\"inside\"},{name:\"lower-alpha\"},{name:\"lower-greek\"},{name:\"lower-latin\"},{name:\"lower-roman\"},{name:\"none\"},{name:\"outside\"},{name:\"square\",desc:\"A filled square.\"},{name:\"symbols()\",browsers:\"FF35\"},{name:\"upper-alpha\"},{name:\"upper-latin\"},{name:\"upper-roman\"},{name:\"url()\"}],syntax:\"<'list-style-type'> || <'list-style-position'> || <'list-style-image'>\"},{name:\"list-style-image\",desc:\"Sets the image that will be used as the list item marker. When the image is available, it will replace the marker set with the 'list-style-type' marker.\",browsers:\"all\",restriction:\"image\",values:[{name:\"none\",desc:\"The default contents of the of the list item’s marker are given by 'list-style-type' instead.\"}],syntax:\"<url> | none\"},{name:\"list-style-position\",desc:\"Specifies the position of the '::marker' pseudo-element's box in the list item.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"inside\"},{name:\"outside\"}],syntax:\"inside | outside\"},{name:\"list-style-type\",desc:\"Used to construct the default contents of a list item’s marker\",browsers:\"all\",restriction:\"enum, string\",values:[{name:\"armenian\",desc:\"Traditional uppercase Armenian numbering.\"},{name:\"circle\"},{name:\"decimal\",desc:\"Western decimal numbers.\"},{name:\"decimal-leading-zero\",desc:\"Decimal numbers padded by initial zeros.\"},{name:\"disc\"},{name:\"georgian\",desc:\"Traditional Georgian numbering.\"},{name:\"lower-alpha\",desc:\"Lowercase ASCII letters.\"},{name:\"lower-greek\",desc:\"Lowercase classical Greek.\"},{name:\"lower-latin\",desc:\"Lowercase ASCII letters.\"},{name:\"lower-roman\",desc:\"Lowercase ASCII Roman numerals.\"},{name:\"none\",desc:\"No marker\"},{name:\"square\",desc:\"A filled square.\"},{name:\"symbols()\",browsers:\"FF35\"},{name:\"upper-alpha\",desc:\"Uppercase ASCII letters.\"},{name:\"upper-latin\",desc:\"Uppercase ASCII letters.\"},{name:\"upper-roman\",desc:\"Uppercase ASCII Roman numerals.\"}],syntax:\"<counter-style> | <string> | none\"},{name:\"margin\",desc:\"Shorthand property to set values the thickness of the margin area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. Negative values for margin properties are allowed, but there may be implementation-specific limits.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"[ <length> | <percentage> | auto ]{1,4}\"},{name:\"margin-block-end\",desc:\"Logical 'margin-bottom'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<'margin-left'>\"},{name:\"margin-block-start\",desc:\"Logical 'margin-top'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<'margin-left'>\"},{name:\"margin-bottom\",desc:\"Shorthand property to set values the thickness of the margin area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. Negative values for margin properties are allowed, but there may be implementation-specific limits..\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"margin-inline-end\",desc:\"Logical 'margin-right'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41,S3,C2\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<'margin-left'>\"},{name:\"margin-inline-start\",desc:\"Logical 'margin-left'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41,S3,C2\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<'margin-left'>\"},{name:\"margin-left\",desc:\"Shorthand property to set values the thickness of the margin area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. Negative values for margin properties are allowed, but there may be implementation-specific limits..\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"margin-right\",desc:\"Shorthand property to set values the thickness of the margin area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. Negative values for margin properties are allowed, but there may be implementation-specific limits..\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"margin-top\",desc:\"Shorthand property to set values the thickness of the margin area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. Negative values for margin properties are allowed, but there may be implementation-specific limits..\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"marker\",desc:\"Specifies the marker symbol that shall be used for all points on the sets the value for all vertices on the given ‘path’ element or basic shape.\",restriction:\"url\",values:[{name:\"none\",desc:\"Indicates that no marker symbol will be drawn at the given vertex or vertices.\"},{name:\"url()\",desc:\"Indicates that the <marker> element referenced will be used.\"}]},{name:\"marker-end\",desc:\"Specifies the marker that will be drawn at the last vertices of the given markable element.\",restriction:\"url\",values:[{name:\"none\",desc:\"Indicates that no marker symbol will be drawn at the given vertex or vertices.\"},{name:\"url()\",desc:\"Indicates that the <marker> element referenced will be used.\"}]},{name:\"marker-mid\",desc:\"Specifies the marker that will be drawn at all vertices except the first and last.\",restriction:\"url\",values:[{name:\"none\",desc:\"Indicates that no marker symbol will be drawn at the given vertex or vertices.\"},{name:\"url()\",desc:\"Indicates that the <marker> element referenced will be used.\"}]},{name:\"marker-start\",desc:\"Specifies the marker that will be drawn at the first vertices of the given markable element.\",restriction:\"url\",values:[{name:\"none\",desc:\"Indicates that no marker symbol will be drawn at the given vertex or vertices.\"},{name:\"url()\",desc:\"Indicates that the <marker> element referenced will be used.\"}]},{name:\"mask-image\",desc:\"Sets the mask layer image of an element.\",browsers:\"E16,FF53,S4,C1,O\",restriction:\"url, image, enum\",values:[{name:\"none\",desc:\"Counts as a transparent black image layer.\"},{name:\"url()\",desc:\"Reference to a <mask element or to a CSS image.\"}],syntax:\"<mask-reference>#\"},{name:\"mask-mode\",desc:\"Indicates whether the mask layer image is treated as luminance mask or alpha mask.\",browsers:\"FF53\",restriction:\"url, image, enum\",values:[{name:\"alpha\",desc:\"Alpha values of the mask layer image should be used as the mask values.\"},{name:\"auto\",desc:\"Use alpha values if 'mask-image' is an image, luminance if a <mask> element or a CSS image.\"},{name:\"luminance\",desc:\"Luminance values of the mask layer image should be used as the mask values.\"}],syntax:\"<masking-mode>#\"},{name:\"mask-origin\",desc:\"Specifies the mask positioning area.\",browsers:\"FF53,S,C,O\",restriction:\"geometry-box, enum\",syntax:\"<geometry-box>#\"},{name:\"mask-position\",desc:\"Specifies how mask layer images are positioned.\",browsers:\"FF53,S4,C1\",restriction:\"position, length, percentage\",syntax:\"<position>#\"},{name:\"mask-repeat\",desc:\"Specifies how mask layer images are tiled after they have been sized and positioned.\",browsers:\"FF53,S4,C1\",restriction:\"repeat\",syntax:\"<repeat-style>#\"},{name:\"mask-size\",desc:\"Specifies the size of the mask layer images.\",browsers:\"FF53\",restriction:\"length, percentage, enum\",values:[{name:\"auto\",desc:\"Resolved by using the image’s intrinsic ratio and the size of the other dimension, or failing that, using the image’s intrinsic size, or failing that, treating it as 100%.\"},{name:\"contain\",desc:\"Scale the image, while preserving its intrinsic aspect ratio (if any), to the largest size such that both its width and its height can fit inside the background positioning area.\"},{name:\"cover\",desc:\"Scale the image, while preserving its intrinsic aspect ratio (if any), to the smallest size such that both its width and its height can completely cover the background positioning area.\"}],syntax:\"<bg-size>#\"},{name:\"mask-type\",desc:\"Defines whether the content of the <mask> element is treated as as luminance mask or alpha mask.\",browsers:\"FF35,C24\",restriction:\"enum\",values:[{name:\"alpha\",desc:\"Indicates that the alpha values of the mask should be used.\"},{name:\"luminance\",desc:\"Indicates that the luminance values of the mask should be used.\"}],syntax:\"luminance | alpha\"},{name:\"max-block-size\",desc:\"Logical 'max-width'. Mapping depends on the element’s 'writing-mode'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"none\",desc:\"No limit on the width of the box.\"}],status:\"e\",syntax:\"<'max-width'>\"},{name:\"max-height\",desc:\"Allows authors to constrain content height to a certain range.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"none\",desc:\"No limit on the height of the box.\"},{name:\"fit-content\",browsers:\"C46,O33\"},{name:\"max-content\",desc:\"Use the max-content inline size or max-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"},{name:\"min-content\",desc:\"Use the min-content inline size or min-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"}],syntax:\"<viewport-length>\"},{name:\"max-inline-size\",desc:\"Logical 'max-height'. Mapping depends on the element’s 'writing-mode'.\",browsers:\"FF41,S10.1,C,O\",restriction:\"length, percentage\",values:[{name:\"none\",desc:\"No limit on the height of the box.\"}],status:\"e\",syntax:\"<'max-width'>\"},{name:\"max-width\",desc:\"Allows authors to constrain content width to a certain range.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"none\",desc:\"No limit on the width of the box.\"},{name:\"fit-content\",browsers:\"C46,O33\"},{name:\"max-content\",desc:\"Use the max-content inline size or max-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"},{name:\"min-content\",desc:\"Use the min-content inline size or min-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"}],syntax:\"<viewport-length>\"},{name:\"min-block-size\",desc:\"Logical 'min-width'. Mapping depends on the element’s 'writing-mode'.\",browsers:\"FF41\",restriction:\"length, percentage\",syntax:\"<'min-width'>\"},{name:\"min-height\",desc:\"Allows authors to constrain content height to a certain range.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",browsers:\"E,IE11\"},{name:\"fit-content\",browsers:\"C46,O33\"},{name:\"max-content\",desc:\"Use the max-content inline size or max-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"},{name:\"min-content\",desc:\"Use the min-content inline size or min-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"}],syntax:\"<viewport-length>\"},{name:\"min-inline-size\",desc:\"Logical 'min-height'. Mapping depends on the element’s 'writing-mode'.\",browsers:\"FF41\",restriction:\"length, percentage\",syntax:\"<'min-width'>\"},{name:\"min-width\",desc:\"Allows authors to constrain content width to a certain range.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",browsers:\"E,IE11\"},{name:\"fit-content\",browsers:\"C46,O33\"},{name:\"max-content\",desc:\"Use the max-content inline size or max-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"},{name:\"min-content\",desc:\"Use the min-content inline size or min-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"}],syntax:\"<viewport-length>\"},{name:\"mix-blend-mode\",desc:\"Defines the formula that must be used to mix the colors with the backdrop.\",browsers:\"FF32,S8,C41,O\",restriction:\"enum\",values:[{name:\"normal\",desc:\"Default attribute which specifies no blending\"},{name:\"multiply\"},{name:\"screen\"},{name:\"overlay\"},{name:\"darken\"},{name:\"lighten\"},{name:\"color-dodge\"},{name:\"color-burn\"},{name:\"hard-light\"},{name:\"soft-light\"},{name:\"difference\"},{name:\"exclusion\"},{name:\"hue\",browsers:\"C41,FF32,O29\"},{name:\"saturation\",browsers:\"C41,FF32,O29\"},{name:\"color\",browsers:\"C41,FF32,O29\"},{name:\"luminosity\",browsers:\"C41,FF32,O29\"}],syntax:\"<blend-mode>\"},{name:\"motion\",desc:\"Shorthand property for setting 'motion-path', 'motion-offset' and 'motion-rotation'.\",browsers:\"C46,O33\",restriction:\"url, length, percentage, angle, shape, geometry-box, enum\",values:[{name:\"none\",desc:\"No motion path gets created.\"},{name:\"path()\"},{name:\"auto\",desc:\"Indicates that the object is rotated by the angle of the direction of the motion path.\"},{name:\"reverse\",desc:\"Indicates that the object is rotated by the angle of the direction of the motion path plus 180 degrees.\"}]},{name:\"motion-offset\",desc:\"A distance that describes the position along the specified motion path.\",browsers:\"C46,O33\",restriction:\"length, percentage\"},{name:\"motion-path\",desc:\"Specifies the motion path the element gets positioned at.\",browsers:\"C46,O33\",restriction:\"url, shape, geometry-box, enum\",values:[{name:\"none\",desc:\"No motion path gets created.\"},{name:\"path()\"}]},{name:\"motion-rotation\",desc:\"Defines the direction of the element while positioning along the motion path.\",browsers:\"C46,O33\",restriction:\"angle\",values:[{name:\"auto\",desc:\"Indicates that the object is rotated by the angle of the direction of the motion path.\"},{name:\"reverse\",desc:\"Indicates that the object is rotated by the angle of the direction of the motion path plus 180 degrees.\"}]},{name:\"-moz-animation\",desc:\"Shorthand property combines six of the animation properties into a single property.\",browsers:\"FF9\",restriction:\"time, enum, timing-function, identifier, number\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"backwards\"},{name:\"both\",desc:\"Both forwards and backwards fill modes are applied.\"},{name:\"forwards\"},{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"},{name:\"none\",desc:\"No animation is performed\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}]},{name:\"-moz-animation-delay\",desc:\"Defines when the animation will start.\",browsers:\"FF9\",restriction:\"time\"},{name:\"-moz-animation-direction\",desc:\"Defines whether or not the animation should play in reverse on alternate cycles.\",browsers:\"FF9\",restriction:\"enum\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}]},{name:\"-moz-animation-duration\",desc:\"Defines the length of time that an animation takes to complete one cycle.\",browsers:\"FF9\",restriction:\"time\"},{name:\"-moz-animation-iteration-count\",desc:\"Defines the number of times an animation cycle is played. The default value is one, meaning the animation will play from beginning to end once.\",browsers:\"FF9\",restriction:\"number, enum\",values:[{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"}]},{name:\"-moz-animation-name\",desc:\"Defines a list of animations that apply. Each name is used to select the keyframe at-rule that provides the property values for the animation.\",browsers:\"FF9\",restriction:\"identifier, enum\",values:[{name:\"none\",desc:\"No animation is performed\"}]},{name:\"-moz-animation-play-state\",desc:\"Defines whether the animation is running or paused.\",browsers:\"FF9\",restriction:\"enum\",values:[{name:\"paused\"},{name:\"running\"}]},{name:\"-moz-animation-timing-function\",desc:\"Describes how the animation will progress over one cycle of its duration. See the 'transition-timing-function'.\",browsers:\"FF9\",restriction:\"timing-function\"},{name:\"-moz-appearance\",desc:\"Used in Gecko (Firefox) to display an element using a platform-native styling based on the operating system's theme.\",browsers:\"FF54\",restriction:\"enum\",values:[{name:\"button\"},{name:\"button-arrow-down\"},{name:\"button-arrow-next\"},{name:\"button-arrow-previous\"},{name:\"button-arrow-up\"},{name:\"button-bevel\"},{name:\"checkbox\"},{name:\"checkbox-container\"},{name:\"checkbox-label\"},{name:\"dialog\"},{name:\"groupbox\"},{name:\"listbox\"},{name:\"menuarrow\"},{name:\"menuimage\"},{name:\"menuitem\"},{name:\"menuitemtext\"},{name:\"menulist\"},{name:\"menulist-button\"},{name:\"menulist-text\"},{name:\"menulist-textfield\"},{name:\"menupopup\"},{name:\"menuradio\"},{name:\"menuseparator\"},{name:\"-moz-mac-unified-toolbar\"},{name:\"-moz-win-borderless-glass\"},{name:\"-moz-win-browsertabbar-toolbox\"},{name:\"-moz-win-communications-toolbox\"},{name:\"-moz-win-glass\"},{name:\"-moz-win-media-toolbox\"},{name:\"none\"},{name:\"progressbar\"},{name:\"progresschunk\"},{name:\"radio\"},{name:\"radio-container\"},{name:\"radio-label\"},{name:\"radiomenuitem\"},{name:\"resizer\"},{name:\"resizerpanel\"},{name:\"scrollbarbutton-down\"},{name:\"scrollbarbutton-left\"},{name:\"scrollbarbutton-right\"},{name:\"scrollbarbutton-up\"},{name:\"scrollbar-small\"},{name:\"scrollbartrack-horizontal\"},{name:\"scrollbartrack-vertical\"},{name:\"separator\"},{name:\"spinner\"},{name:\"spinner-downbutton\"},{name:\"spinner-textfield\"},{name:\"spinner-upbutton\"},{name:\"statusbar\"},{name:\"statusbarpanel\"},{name:\"tab\"},{name:\"tabpanels\"},{name:\"tab-scroll-arrow-back\"},{name:\"tab-scroll-arrow-forward\"},{name:\"textfield\"},{name:\"textfield-multiline\"},{name:\"toolbar\"},{name:\"toolbox\"},{name:\"tooltip\"},{name:\"treeheadercell\"},{name:\"treeheadersortarrow\"},{name:\"treeitem\"},{name:\"treetwistyopen\"},{name:\"treeview\"},{name:\"treewisty\"},{name:\"window\"}],status:\"n\",syntax:\"none | button | button-arrow-down | button-arrow-next | button-arrow-previous | button-arrow-up | button-bevel | button-focus | caret | checkbox | checkbox-container | checkbox-label | checkmenuitem | dualbutton | groupbox | listbox | listitem | menuarrow | menubar | menucheckbox | menuimage | menuitem | menuitemtext | menulist | menulist-button | menulist-text | menulist-textfield | menupopup | menuradio | menuseparator | meterbar | meterchunk | progressbar | progressbar-vertical | progresschunk | progresschunk-vertical | radio | radio-container | radio-label | radiomenuitem | range | range-thumb | resizer | resizerpanel | scale-horizontal | scalethumbend | scalethumb-horizontal | scalethumbstart | scalethumbtick | scalethumb-vertical | scale-vertical | scrollbarbutton-down | scrollbarbutton-left | scrollbarbutton-right | scrollbarbutton-up | scrollbarthumb-horizontal | scrollbarthumb-vertical | scrollbartrack-horizontal | scrollbartrack-vertical | searchfield | separator | sheet | spinner | spinner-downbutton | spinner-textfield | spinner-upbutton | splitter | statusbar | statusbarpanel | tab | tabpanel | tabpanels | tab-scroll-arrow-back | tab-scroll-arrow-forward | textfield | textfield-multiline | toolbar | toolbarbutton | toolbarbutton-dropdown | toolbargripper | toolbox | tooltip | treeheader | treeheadercell | treeheadersortarrow | treeitem | treeline | treetwisty | treetwistyopen | treeview | -moz-mac-unified-toolbar | -moz-win-borderless-glass | -moz-win-browsertabbar-toolbox | -moz-win-communicationstext | -moz-win-communications-toolbox | -moz-win-exclude-glass | -moz-win-glass | -moz-win-mediatext | -moz-win-media-toolbox | -moz-window-button-box | -moz-window-button-box-maximized | -moz-window-button-close | -moz-window-button-maximize | -moz-window-button-minimize | -moz-window-button-restore | -moz-window-frame-bottom | -moz-window-frame-left | -moz-window-frame-right | -moz-window-titlebar | -moz-window-titlebar-maximized\"},{name:\"-moz-backface-visibility\",desc:\"Determines whether or not the 'back' side of a transformed element is visible when facing the viewer. With an identity transform, the front side of an element faces the viewer.\",browsers:\"FF10\",restriction:\"enum\",values:[{name:\"hidden\"},{name:\"visible\"}]},{name:\"-moz-background-clip\",desc:\"Determines the background painting area.\",browsers:\"FF1-3.6\",restriction:\"box, enum\",values:[{name:\"padding\"}]},{name:\"-moz-background-inline-policy\",desc:\"In Gecko-based applications like Firefox, the -moz-background-inline-policy CSS property specifies how the background image of an inline element is determined when the content of the inline element wraps onto multiple lines. The choice of position has significant effects on repetition.\",browsers:\"FF1\",restriction:\"enum\",values:[{name:\"bounding-box\"},{name:\"continuous\"},{name:\"each-box\"}]},{name:\"-moz-background-origin\",desc:\"For elements rendered as a single box, specifies the background positioning area. For elements rendered as multiple boxes (e.g., inline boxes on several lines, boxes on several pages) specifies which boxes 'box-decoration-break' operates on to determine the background positioning area(s).\",browsers:\"FF1\",restriction:\"box\"},{name:\"-moz-border-bottom-colors\",desc:\"Sets a list of colors for the bottom border.\",browsers:\"FF1\",restriction:\"color\",status:\"n\",syntax:\"<color>+ | none\"},{name:\"-moz-border-image\",desc:\"Shorthand property for setting 'border-image-source', 'border-image-slice', 'border-image-width', 'border-image-outset' and 'border-image-repeat'. Omitted values are set to their initial values.\",browsers:\"FF3.6\",restriction:\"length, percentage, number, url, enum\",values:[{name:\"auto\",desc:\"If 'auto' is specified then the border image width is the intrinsic width or height (whichever is applicable) of the corresponding image slice. If the image does not have the required intrinsic dimension then the corresponding border-width is used instead.\"},{name:\"fill\",desc:\"Causes the middle part of the border-image to be preserved.\"},{name:\"none\"},{name:\"repeat\"},{name:\"round\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the image is rescaled so that it does.\"},{name:\"space\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the extra space is distributed around the tiles.\"},{name:\"stretch\",desc:\"The image is stretched to fill the area.\"},{name:\"url()\"}]},{name:\"-moz-border-left-colors\",desc:\"Sets a list of colors for the bottom border.\",browsers:\"FF1\",restriction:\"color\",status:\"n\",syntax:\"<color>+ | none\"},{name:\"-moz-border-right-colors\",desc:\"Sets a list of colors for the bottom border.\",browsers:\"FF1\",restriction:\"color\",status:\"n\",syntax:\"<color>+ | none\"},{name:\"-moz-border-top-colors\",desc:\"Ske Firefox, -moz-border-bottom-colors sets a list of colors for the bottom border.\",browsers:\"FF1\",restriction:\"color\",status:\"n\",syntax:\"<color>+ | none\"},{name:\"-moz-box-align\",desc:\"Specifies how a XUL box aligns its contents across (perpendicular to) the direction of its layout. The effect of this is only visible if there is extra space in the box.\",browsers:\"FF1\",restriction:\"enum\",values:[{name:\"baseline\",desc:\"If this box orientation is inline-axis or horizontal, all children are placed with their baselines aligned, and extra space placed before or after as necessary. For block flows, the baseline of the first non-empty line box located within the element is used. For tables, the baseline of the first cell is used.\"},{name:\"center\",desc:\"Any extra space is divided evenly, with half placed above the child and the other half placed after the child.\"},{name:\"end\",desc:\"For normal direction boxes, the bottom edge of each child is placed along the bottom of the box. Extra space is placed above the element. For reverse direction boxes, the top edge of each child is placed along the top of the box. Extra space is placed below the element.\"},{name:\"start\",desc:\"For normal direction boxes, the top edge of each child is placed along the top of the box. Extra space is placed below the element. For reverse direction boxes, the bottom edge of each child is placed along the bottom of the box. Extra space is placed above the element.\"},{name:\"stretch\",desc:\"The height of each child is adjusted to that of the containing block.\"}]},{name:\"-moz-box-direction\",desc:\"Specifies whether a box lays out its contents normally (from the top or left edge), or in reverse (from the bottom or right edge).\",browsers:\"FF1\",restriction:\"enum\",values:[{name:\"normal\",desc:\"A box with a computed value of horizontal for box-orient displays its children from left to right. A box with a computed value of vertical displays its children from top to bottom.\"},{name:\"reverse\",desc:\"A box with a computed value of horizontal for box-orient displays its children from right to left. A box with a computed value of vertical displays its children from bottom to top.\"}]},{name:\"-moz-box-flex\",desc:\"Specifies how a box grows to fill the box that contains it, in the direction of the containing box's layout.\",browsers:\"FF1\",restriction:\"number\"},{name:\"-moz-box-flexgroup\",desc:\"Flexible elements can be assigned to flex groups using the 'box-flex-group' property.\",browsers:\"FF1\",restriction:\"integer\"},{name:\"-moz-box-ordinal-group\",desc:\"Indicates the ordinal group the element belongs to. Elements with a lower ordinal group are displayed before those with a higher ordinal group.\",browsers:\"FF1\",restriction:\"integer\"},{name:\"-moz-box-orient\",desc:\"In Mozilla applications, -moz-box-orient specifies whether a box lays out its contents horizontally or vertically.\",browsers:\"FF1\",restriction:\"enum\",values:[{name:\"block-axis\"},{name:\"horizontal\",desc:\"The box displays its children from left to right in a horizontal line.\"},{name:\"inline-axis\"},{name:\"vertical\",desc:\"The box displays its children from stacked from top to bottom vertically.\"}]},{name:\"-moz-box-pack\",desc:\"Specifies how a box packs its contents in the direction of its layout. The effect of this is only visible if there is extra space in the box.\",browsers:\"FF1\",restriction:\"enum\",values:[{name:\"center\",desc:\"The extra space is divided evenly, with half placed before the first child and the other half placed after the last child.\"},{name:\"end\",desc:\"For normal direction boxes, the right edge of the last child is placed at the right side, with all extra space placed before the first child. For reverse direction boxes, the left edge of the first child is placed at the left side, with all extra space placed after the last child.\"},{name:\"justify\",desc:\"The space is divided evenly in-between each child, with none of the extra space placed before the first child or after the last child. If there is only one child, treat the pack value as if it were start.\"},{name:\"start\",desc:\"For normal direction boxes, the left edge of the first child is placed at the left side, with all extra space placed after the last child. For reverse direction boxes, the right edge of the last child is placed at the right side, with all extra space placed before the first child.\"}]},{name:\"-moz-box-sizing\",desc:\"Box Model addition in CSS3.\",browsers:\"FF1\",restriction:\"enum\",values:[{name:\"border-box\"},{name:\"content-box\"},{name:\"padding-box\"}]},{name:\"-moz-column-count\",desc:\"Describes the optimal number of columns into which the content of the element will be flowed.\",browsers:\"FF3.5\",restriction:\"integer\",values:[{name:\"auto\",desc:\"Determines the number of columns by the 'column-width' property and the element width.\"}]},{name:\"-moz-column-gap\",desc:\"Sets the gap between columns. If there is a column rule between columns, it will appear in the middle of the gap.\",browsers:\"FF3.5\",restriction:\"length\",values:[{name:\"normal\",desc:\"User agent specific and typically equivalent to 1em.\"}]},{name:\"-moz-column-rule\",desc:\"Shorthand for setting 'column-rule-width', 'column-rule-style', and 'column-rule-color' at the same place in the style sheet. Omitted values are set to their initial values.\",browsers:\"FF3.5\",restriction:\"length, line-width, line-style, color\"},{name:\"-moz-column-rule-color\",desc:\"Sets the color of the column rule\",browsers:\"FF3.5\",restriction:\"color\"},{name:\"-moz-column-rule-style\",desc:\"Sets the style of the rule between columns of an element.\",browsers:\"FF3.5\",restriction:\"line-style\"},{name:\"-moz-column-rule-width\",desc:\"Sets the width of the rule between columns. Negative values are not allowed.\",browsers:\"FF3.5\",restriction:\"length, line-width\"},{name:\"-moz-columns\",desc:\"A shorthand property which sets both 'column-width' and 'column-count'.\",browsers:\"FF9\",restriction:\"length, integer\",values:[{name:\"auto\",desc:\"The width depends on the values of other properties.\"}]},{name:\"-moz-column-width\",desc:\"This property describes the width of columns in multicol elements.\",browsers:\"FF3.5\",restriction:\"length\",values:[{name:\"auto\",desc:\"The width depends on the values of other properties.\"}]},{name:\"-moz-font-feature-settings\",desc:\"Provides low-level control over OpenType font features. It is intended as a way of providing access to font features that are not widely used but are needed for a particular use case.\",browsers:\"FF4\",restriction:\"string, integer\",values:[{name:'\"c2cs\"'},{name:'\"dlig\"'},{name:'\"kern\"'},{name:'\"liga\"'},{name:'\"lnum\"'},{name:'\"onum\"'},{name:'\"smcp\"'},{name:'\"swsh\"'},{name:'\"tnum\"'},{name:\"normal\",desc:\"No change in glyph substitution or positioning occurs.\"},{name:\"off\",browsers:\"FF15\"},{name:\"on\",browsers:\"FF15\"}]},{name:\"-moz-hyphens\",desc:\"Controls whether hyphenation is allowed to create more break opportunities within a line of text.\",browsers:\"FF9\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Conditional hyphenation characters inside a word, if present, take priority over automatic resources when determining hyphenation points within the word.\"},{name:\"manual\"},{name:\"none\",desc:\"Words are not broken at line breaks, even if characters inside the word suggest line break points.\"}]},{name:\"-moz-perspective\",desc:\"Applies the same transform as the perspective(<number>) transform function, except that it applies only to the positioned or transformed children of the element, not to the transform on the element itself.\",browsers:\"FF10\",restriction:\"length\",values:[{name:\"none\",desc:\"No perspective transform is applied.\"}]},{name:\"-moz-perspective-origin\",desc:\"Establishes the origin for the perspective property. It effectively sets the X and Y position at which the viewer appears to be looking at the children of the element.\",browsers:\"FF10\",restriction:\"position, percentage, length\"},{name:\"-moz-text-align-last\",desc:\"Describes how the last line of a block or a line right before a forced line break is aligned when 'text-align' is set to 'justify'.\",browsers:\"FF12\",restriction:\"enum\",values:[{name:\"auto\"},{name:\"center\",desc:\"The inline contents are centered within the line box.\"},{name:\"justify\",desc:\"The text is justified according to the method specified by the 'text-justify' property.\"},{name:\"left\",desc:\"The inline contents are aligned to the left edge of the line box. In vertical text, 'left' aligns to the edge of the line box that would be the start edge for left-to-right text.\"},{name:\"right\",desc:\"The inline contents are aligned to the right edge of the line box. In vertical text, 'right' aligns to the edge of the line box that would be the end edge for left-to-right text.\"}]},{name:\"-moz-text-decoration-color\",desc:\"Specifies the color of text decoration (underlines overlines, and line-throughs) set on the element with text-decoration-line.\",browsers:\"FF6\",restriction:\"color\"},{name:\"-moz-text-decoration-line\",desc:\"Specifies what line decorations, if any, are added to the element.\",browsers:\"FF6\",restriction:\"enum\",values:[{name:\"line-through\"},{name:\"none\",desc:\"Neither produces nor inhibits text decoration.\"},{name:\"overline\"},{name:\"underline\"}]},{name:\"-moz-text-decoration-style\",desc:\"Specifies the line style for underline, line-through and overline text decoration.\",browsers:\"FF6\",restriction:\"enum\",values:[{name:\"dashed\"},{name:\"dotted\"},{name:\"double\"},{name:\"none\",desc:\"Produces no line.\"},{name:\"solid\"},{name:\"wavy\"}]},{name:\"-moz-text-size-adjust\",desc:\"Specifies a size adjustment for displaying text content in mobile browsers.\",browsers:\"FF\",restriction:\"enum, percentage\",values:[{name:\"auto\",desc:\"Renderers must use the default size adjustment when displaying on a small device.\"},{name:\"none\",desc:\"Renderers must not do size adjustment when displaying on a small device.\"}]},{name:\"-moz-transform\",desc:\"A two-dimensional transformation is applied to an element through the 'transform' property. This property contains a list of transform functions similar to those allowed by SVG.\",browsers:\"FF3.5\",restriction:\"enum\",values:[{name:\"matrix()\"},{name:\"matrix3d()\"},{name:\"none\"},{name:\"perspective\"},{name:\"rotate()\"},{name:\"rotate3d()\"},{name:\"rotateX('angle')\"},{name:\"rotateY('angle')\"},{name:\"rotateZ('angle')\"},{name:\"scale()\"},{name:\"scale3d()\"},{name:\"scaleX()\"},{name:\"scaleY()\"},{name:\"scaleZ()\"},{name:\"skew()\"},{name:\"skewX()\"},{name:\"skewY()\"},{name:\"translate()\"},{name:\"translate3d()\"},{name:\"translateX()\"},{name:\"translateY()\"},{name:\"translateZ()\"}]},{name:\"-moz-transform-origin\",desc:\"Establishes the origin of transformation for an element.\",browsers:\"FF3.5\",restriction:\"position, length, percentage\"},{name:\"-moz-transition\",desc:\"Shorthand property combines four of the transition properties into a single property.\",browsers:\"FF4\",restriction:\"time, property, timing-function, enum\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}]},{name:\"-moz-transition-delay\",desc:\"Defines when the transition will start. It allows a transition to begin execution some period of time from when it is applied.\",browsers:\"FF4\",restriction:\"time\"},{name:\"-moz-transition-duration\",desc:\"Specifies how long the transition from the old value to the new value should take.\",browsers:\"FF4\",restriction:\"time\"},{name:\"-moz-transition-property\",desc:\"Specifies the name of the CSS property to which the transition is applied.\",browsers:\"FF4\",restriction:\"property\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}]},{name:\"-moz-transition-timing-function\",desc:\"Describes how the intermediate values used during a transition will be calculated.\",browsers:\"FF4\",restriction:\"timing-function\"},{name:\"-moz-user-focus\",desc:\"Used to indicate whether the element can have focus.\",browsers:\"FF1.5\",values:[{name:\"ignore\"},{name:\"normal\"}],status:\"n\",syntax:\"ignore | normal | select-after | select-before | select-menu | select-same | select-all | none\"},{name:\"-moz-user-select\",desc:\"Controls the appearance of selection.\",browsers:\"FF1.5\",restriction:\"enum\",values:[{name:\"all\"},{name:\"element\"},{name:\"elements\"},{name:\"-moz-all\"},{name:\"-moz-none\"},{name:\"none\"},{name:\"text\"},{name:\"toggle\"}]},{name:\"-ms-accelerator\",desc:\"IE only. Has the ability to turn off its system underlines for accelerator keys until the ALT key is pressed\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"false\"},{name:\"true\"}],status:\"n\",syntax:\"false | true\"},{name:\"-ms-behavior\",desc:\"IE only. Used to extend behaviors of the browser\",browsers:\"IE8\",restriction:\"url\"},{name:\"-ms-block-progression\",desc:\"Sets the block-progression value and the flow orientation\",browsers:\"IE8\",restriction:\"enum\",values:[{name:\"bt\"},{name:\"lr\"},{name:\"rl\"},{name:\"tb\"}],status:\"n\",syntax:\"tb | rl | bt | lr\"},{name:\"-ms-content-zoom-chaining\",desc:\"Specifies the zoom behavior that occurs when a user hits the zoom limit during a manipulation.\",browsers:\"E,IE10\",values:[{name:\"chained\",desc:\"The nearest zoomable parent element begins zooming when the user hits a zoom limit during a manipulation. No bounce effect is shown.\"},{name:\"none\",desc:\"A bounce effect is shown when the user hits a zoom limit during a manipulation.\"}],status:\"n\",syntax:\"none | chained\"},{name:\"-ms-content-zooming\",desc:\"Specifies whether zooming is enabled.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"none\",desc:\"The element is not zoomable.\"},{name:\"zoom\"}],status:\"n\",syntax:\"none | zoom\"},{name:\"-ms-content-zoom-limit\",desc:\"Shorthand property for the -ms-content-zoom-limit-min and -ms-content-zoom-limit-max properties.\",browsers:\"E,IE10\",restriction:\"percentage\",status:\"n\",syntax:\"<'-ms-content-zoom-limit-min'> <'-ms-content-zoom-limit-max'>\"},{name:\"-ms-content-zoom-limit-max\",desc:\"Specifies the maximum zoom factor.\",browsers:\"E,IE10\",restriction:\"percentage\",status:\"n\",syntax:\"<percentage>\"},{name:\"-ms-content-zoom-limit-min\",desc:\"Specifies the minimum zoom factor.\",browsers:\"E,IE10\",restriction:\"percentage\",status:\"n\",syntax:\"<percentage>\"},{name:\"-ms-content-zoom-snap\",desc:\"Shorthand property for the -ms-content-zoom-snap-type and -ms-content-zoom-snap-points properties.\",browsers:\"E,IE10\",values:[{name:\"mandatory\",desc:\"Indicates that the motion of the content after the contact is picked up is always adjusted so that it lands on a snap-point.\"},{name:\"none\",desc:\"Indicates that zooming is unaffected by any defined snap-points.\"},{name:\"proximity\",desc:'Indicates that the motion of the content after the contact is picked up may be adjusted if the content would normally stop \"close enough\" to a snap-point.'},{name:\"snapInterval(100%, 100%)\",desc:\"Specifies where the snap-points will be placed.\"},{name:\"snapList()\",desc:\"Specifies the position of individual snap-points as a comma-separated list of zoom factors.\"}],status:\"n\",syntax:\"<'-ms-content-zoom-snap-type'> || <'-ms-content-zoom-snap-points'>\"},{name:\"-ms-content-zoom-snap-points\",desc:\"Defines where zoom snap-points are located.\",browsers:\"E,IE10\",values:[{name:\"snapInterval(100%, 100%)\",desc:\"Specifies where the snap-points will be placed.\"},{name:\"snapList()\",desc:\"Specifies the position of individual snap-points as a comma-separated list of zoom factors.\"}],status:\"n\",syntax:\"snapInterval( <percentage>, <percentage> ) | snapList( <percentage># )\"},{name:\"-ms-content-zoom-snap-type\",desc:\"Specifies how zooming is affected by defined snap-points.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"mandatory\",desc:\"Indicates that the motion of the content after the contact is picked up is always adjusted so that it lands on a snap-point.\"},{name:\"none\",desc:\"Indicates that zooming is unaffected by any defined snap-points.\"},{name:\"proximity\",desc:'Indicates that the motion of the content after the contact is picked up may be adjusted if the content would normally stop \"close enough\" to a snap-point.'}],status:\"n\",syntax:\"none | proximity | mandatory\"},{name:\"-ms-filter\",desc:\"IE only. Used to produce visual effects.\",browsers:\"IE8-9\",restriction:\"string\",status:\"n\",syntax:\"<string>\"},{name:\"-ms-flex\",desc:\"specifies the parameters of a flexible length: the positive and negative flexibility, and the preferred size.\",browsers:\"IE10\",restriction:\"length, number, percentage\",values:[{name:\"auto\",desc:\"Retrieves the value of the main size property as the used 'flex-basis'.\"},{name:\"none\",desc:\"Expands to '0 0 auto'.\"}]},{name:\"-ms-flex-align\",desc:\"Aligns flex items along the cross axis of the current line of the flex container.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"baseline\",desc:\"If the flex item’s inline axis is the same as the cross axis, this value is identical to 'flex-start'. Otherwise, it participates in baseline alignment.\"},{name:\"center\",desc:\"The flex item’s margin box is centered in the cross axis within the line.\"},{name:\"end\",desc:\"The cross-end margin edge of the flex item is placed flush with the cross-end edge of the line.\"},{name:\"start\",desc:\"The cross-start margin edge of the flexbox item is placed flush with the cross-start edge of the line.\"},{name:\"stretch\",desc:\"If the cross size property of the flexbox item is anything other than 'auto', this value is identical to 'start'.\"}]},{name:\"-ms-flex-direction\",desc:\"Specifies how flex items are placed in the flex container, by setting the direction of the flex container’s main axis.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"column\",desc:\"The flex container’s main axis has the same orientation as the block axis of the current writing mode.\"},{name:\"column-reverse\"},{name:\"row\",desc:\"The flex container’s main axis has the same orientation as the inline axis of the current writing mode.\"},{name:\"row-reverse\"}]},{name:\"-ms-flex-flow\",desc:\"Specifies how flexbox items are placed in the flexbox.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"column\",desc:\"The flex container’s main axis has the same orientation as the block axis of the current writing mode.\"},{name:\"column-reverse\"},{name:\"nowrap\",desc:\"The flex container is single-line.\"},{name:\"row\",desc:\"The flex container’s main axis has the same orientation as the inline axis of the current writing mode.\"},{name:\"wrap\",desc:\"The flexbox is multi-line.\"},{name:\"wrap-reverse\"}]},{name:\"-ms-flex-item-align\",desc:\"Allows the default alignment along the cross axis to be overridden for individual flex items.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Computes to the value of 'align-items' on the element’s parent, or 'stretch' if the element has no parent. On absolutely positioned elements, it computes to itself.\"},{name:\"baseline\",desc:\"If the flex item’s inline axis is the same as the cross axis, this value is identical to 'flex-start'. Otherwise, it participates in baseline alignment.\"},{name:\"center\",desc:\"The flex item’s margin box is centered in the cross axis within the line.\"},{name:\"end\",desc:\"The cross-end margin edge of the flex item is placed flush with the cross-end edge of the line.\"},{name:\"start\",desc:\"The cross-start margin edge of the flex item is placed flush with the cross-start edge of the line.\"},{name:\"stretch\",desc:\"If the cross size property of the flex item computes to auto, and neither of the cross-axis margins are auto, the flex item is stretched.\"}]},{name:\"-ms-flex-line-pack\",desc:\"Aligns a flex container’s lines within the flex container when there is extra space in the cross-axis, similar to how 'justify-content' aligns individual items within the main-axis.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"center\",desc:\"Lines are packed toward the center of the flex container.\"},{name:\"distribute\",desc:\"Lines are evenly distributed in the flex container, with half-size spaces on either end.\"},{name:\"end\",desc:\"Lines are packed toward the end of the flex container.\"},{name:\"justify\",desc:\"Lines are evenly distributed in the flex container.\"},{name:\"start\",desc:\"Lines are packed toward the start of the flex container.\"},{name:\"stretch\",desc:\"Lines stretch to take up the remaining space.\"}]},{name:\"-ms-flex-order\",desc:\"Controls the order in which children of a flex container appear within the flex container, by assigning them to ordinal groups.\",browsers:\"IE10\",restriction:\"integer\"},{name:\"-ms-flex-pack\",desc:\"Aligns flex items along the main axis of the current line of the flex container.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"center\",desc:\"Flex items are packed toward the center of the line.\"},{name:\"distribute\",desc:\"Flex items are evenly distributed in the line, with half-size spaces on either end.\"},{name:\"end\",desc:\"Flex items are packed toward the end of the line.\"},{name:\"justify\",desc:\"Flex items are evenly distributed in the line.\"},{name:\"start\",desc:\"Flex items are packed toward the start of the line.\"}]},{name:\"-ms-flex-wrap\",desc:\"Controls whether the flex container is single-line or multi-line, and the direction of the cross-axis, which determines the direction new lines are stacked in.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"nowrap\",desc:\"The flex container is single-line.\"},{name:\"wrap\",desc:\"The flexbox is multi-line.\"},{name:\"wrap-reverse\"}]},{name:\"-ms-flow-from\",desc:\"Makes a block container a region and associates it with a named flow.\",browsers:\"E,IE10\",restriction:\"identifier\",values:[{name:\"none\",desc:\"The block container is not a CSS Region.\"}],status:\"n\",syntax:\"[ none | <custom-ident> ]#\"},{name:\"-ms-flow-into\",desc:\"Places an element or its contents into a named flow.\",browsers:\"E,IE10\",restriction:\"identifier\",values:[{name:\"none\",desc:\"The element is not moved to a named flow and normal CSS processing takes place.\"}],status:\"n\",syntax:\"[ none | <custom-ident> ]#\"},{name:\"-ms-grid-column\",desc:\"Used to place grid items and explicitly defined grid cells in the Grid.\",browsers:\"E,IE10\",restriction:\"integer, string, enum\",values:[{name:\"auto\"},{name:\"end\"},{name:\"start\"}]},{name:\"-ms-grid-column-align\",desc:\"Aligns the columns in a grid.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"center\",desc:\"Places the center of the Grid Item's margin box at the center of the Grid Item's column.\"},{name:\"end\",desc:\"Aligns the end edge of the Grid Item's margin box to the end edge of the Grid Item's column.\"},{name:\"start\",desc:\"Aligns the starting edge of the Grid Item's margin box to the starting edge of the Grid Item's column.\"},{name:\"stretch\",desc:\"Ensures that the Grid Item's margin box is equal to the size of the Grid Item's column.\"}]},{name:\"-ms-grid-columns\",desc:\"Lays out the columns of the grid.\",browsers:\"E,IE10\"},{name:\"-ms-grid-column-span\",desc:\"Specifies the number of columns to span.\",browsers:\"E,IE10\",restriction:\"integer\"},{name:\"-ms-grid-layer\",desc:\"Grid-layer is similar in concept to z-index, but avoids overloading the meaning of the z-index property, which is applicable only to positioned elements.\",browsers:\"E,IE10\",restriction:\"integer\"},{name:\"-ms-grid-row\",desc:\"grid-row is used to place grid items and explicitly defined grid cells in the Grid.\",browsers:\"E,IE10\",restriction:\"integer, string, enum\",values:[{name:\"auto\"},{name:\"end\"},{name:\"start\"}]},{name:\"-ms-grid-row-align\",desc:\"Aligns the rows in a grid.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"center\",desc:\"Places the center of the Grid Item's margin box at the center of the Grid Item's row.\"},{name:\"end\",desc:\"Aligns the end edge of the Grid Item's margin box to the end edge of the Grid Item's row.\"},{name:\"start\",desc:\"Aligns the starting edge of the Grid Item's margin box to the starting edge of the Grid Item's row.\"},{name:\"stretch\",desc:\"Ensures that the Grid Item's margin box is equal to the size of the Grid Item's row.\"}]},{name:\"-ms-grid-rows\",desc:\"Lays out the columns of the grid.\",browsers:\"E,IE10\"},{name:\"-ms-grid-row-span\",desc:\"Specifies the number of rows to span.\",browsers:\"E,IE10\",restriction:\"integer\"},{name:\"-ms-high-contrast-adjust\",desc:\"Specifies if properties should be adjusted in high contrast mode.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Properties will be adjusted as applicable.\"},{name:\"none\",desc:\"No adjustments will be applied.\"}],status:\"n\",syntax:\"auto | none\"},{name:\"-ms-hyphenate-limit-chars\",desc:\"Specifies the minimum number of characters in a hyphenated word.\",browsers:\"E,IE10\",restriction:\"integer\",values:[{name:\"auto\",desc:\"The user agent chooses a value that adapts to the current layout.\"}],status:\"n\",syntax:\"auto | <integer>{1,3}\"},{name:\"-ms-hyphenate-limit-lines\",desc:\"Indicates the maximum number of successive hyphenated lines in an element.\",browsers:\"E,IE10\",restriction:\"integer\",values:[{name:\"no-limit\"}],status:\"n\",syntax:\"no-limit | <integer>\"},{name:\"-ms-hyphenate-limit-zone\",desc:\"Specifies the maximum amount of unfilled space (before justification) that may be left in the line box before hyphenation is triggered to pull part of a word from the next line back up into the current line.\",browsers:\"E,IE10\",restriction:\"percentage, length\",status:\"n\",syntax:\"<percentage> | <length>\"},{name:\"-ms-hyphens\",desc:\"Controls whether hyphenation is allowed to create more break opportunities within a line of text.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Conditional hyphenation characters inside a word, if present, take priority over automatic resources when determining hyphenation points within the word.\"},{name:\"manual\"},{name:\"none\",desc:\"Words are not broken at line breaks, even if characters inside the word suggest line break points.\"}]},{name:\"-ms-ime-mode\",desc:\"Controls the state of the input method editor for text fields.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"active\"},{name:\"auto\",desc:\"No change is made to the current input method editor state. This is the default.\"},{name:\"disabled\"},{name:\"inactive\"},{name:\"normal\",desc:\"The IME state should be normal; this value can be used in a user style sheet to override the page setting.\"}]},{name:\"-ms-interpolation-mode\",desc:\"Gets or sets the interpolation (resampling) method used to stretch images.\",browsers:\"IE7\",restriction:\"enum\",values:[{name:\"bicubic\"},{name:\"nearest-neighbor\"}]},{name:\"-ms-layout-grid\",desc:\"Sets or retrieves the composite document grid properties that specify the layout of text characters.\",browsers:\"E,IE10\",values:[{name:\"char\",desc:\"Any of the range of character values available to the -ms-layout-grid-char property.\"},{name:\"line\",desc:\"Any of the range of line values available to the -ms-layout-grid-line property.\"},{name:\"mode\"},{name:\"type\"}]},{name:\"-ms-layout-grid-char\",desc:\"Sets or retrieves the size of the character grid used for rendering the text content of an element.\",browsers:\"E,IE10\",restriction:\"enum, length, percentage\",values:[{name:\"auto\",desc:\"Largest character in the font of the element is used to set the character grid.\"},{name:\"none\",desc:\"Default. No character grid is set.\"}]},{name:\"-ms-layout-grid-line\",desc:\"Sets or retrieves the gridline value used for rendering the text content of an element.\",browsers:\"E,IE10\",restriction:\"length\",values:[{name:\"auto\",desc:\"Largest character in the font of the element is used to set the character grid.\"},{name:\"none\",desc:\"Default. No grid line is set.\"}]},{name:\"-ms-layout-grid-mode\",desc:\"Gets or sets whether the text layout grid uses two dimensions.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"both\",desc:\"Default. Both the char and line grid modes are enabled. This setting is necessary to fully enable the layout grid on an element.\"},{name:\"char\",desc:\"Only a character grid is used. This is recommended for use with block-level elements, such as a blockquote, where the line grid is intended to be disabled.\"},{name:\"line\",desc:\"Only a line grid is used. This is recommended for use with inline elements, such as a span, to disable the horizontal grid on runs of text that act as a single entity in the grid layout.\"},{name:\"none\",desc:\"No grid is used.\"}]},{name:\"-ms-layout-grid-type\",desc:\"Sets or retrieves the type of grid used for rendering the text content of an element.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"fixed\",desc:\"Grid used for monospaced layout. All noncursive characters are treated as equal; every character is centered within a single grid space by default.\"},{name:\"loose\",desc:\"Default. Grid used for Japanese and Korean characters.\"},{name:\"strict\",desc:\"Grid used for Chinese, as well as Japanese (Genko) and Korean characters. Only the ideographs, kanas, and wide characters are snapped to the grid.\"}]},{name:\"-ms-line-break\",desc:\"Specifies what set of line breaking restrictions are in effect within the element.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The UA determines the set of line-breaking restrictions to use for CJK scripts, and it may vary the restrictions based on the length of the line; e.g., use a less restrictive set of line-break rules for short lines.\"},{name:\"keep-all\",desc:\"Sequences of CJK characters can no longer break on implied break points. This option should only be used where the presence of word separator characters still creates line-breaking opportunities, as in Korean.\"},{name:\"newspaper\",desc:\"Breaks CJK scripts using the least restrictive set of line-breaking rules. Typically used for short lines, such as in newspapers.\"},{name:\"normal\",desc:\"Breaks CJK scripts using a normal set of line-breaking rules.\"},{name:\"strict\",desc:\"Breaks CJK scripts using a more restrictive set of line-breaking rules than 'normal'.\"}]},{name:\"-ms-overflow-style\",desc:\"Specify whether content is clipped when it overflows the element's content area.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"auto\",desc:\"No preference, UA should use the first scrolling method in the list that it supports.\"},{name:\"-ms-autohiding-scrollbar\"},{name:\"none\",desc:\"Indicates the element does not display scrollbars or panning indicators, even when its content overflows.\"},{name:\"scrollbar\"}],status:\"n\",syntax:\"auto | none | scrollbar | -ms-autohiding-scrollbar\"},{name:\"-ms-perspective\",desc:\"Applies the same transform as the perspective(<number>) transform function, except that it applies only to the positioned or transformed children of the element, not to the transform on the element itself.\",browsers:\"IE10\",restriction:\"length\",values:[{name:\"none\",desc:\"No perspective transform is applied.\"}]},{name:\"-ms-perspective-origin\",desc:\"Establishes the origin for the perspective property. It effectively sets the X and Y position at which the viewer appears to be looking at the children of the element.\",browsers:\"IE10\",restriction:\"position, percentage, length\"},{name:\"-ms-perspective-origin-x\",desc:\"Establishes the origin for the perspective property. It effectively sets the X  position at which the viewer appears to be looking at the children of the element.\",browsers:\"IE10\",restriction:\"position, percentage, length\"},{name:\"-ms-perspective-origin-y\",desc:\"Establishes the origin for the perspective property. It effectively sets the Y position at which the viewer appears to be looking at the children of the element.\",browsers:\"IE10\",restriction:\"position, percentage, length\"},{name:\"-ms-progress-appearance\",desc:\"Gets or sets a value that specifies whether a progress control displays as a bar or a ring.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"bar\"},{name:\"ring\"}]},{name:\"-ms-scrollbar-3dlight-color\",desc:\"Determines the color of the top and left edges of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scrollbar-arrow-color\",desc:\"Determines the color of the arrow elements of a scroll arrow.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scrollbar-base-color\",desc:\"Determines the color of the main elements of a scroll bar, which include the scroll box, track, and scroll arrows.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scrollbar-darkshadow-color\",desc:\"Determines the color of the gutter of a scroll bar.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scrollbar-face-color\",desc:\"Determines the color of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scrollbar-highlight-color\",desc:\"Determines the color of the top and left edges of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scrollbar-shadow-color\",desc:\"Determines the color of the bottom and right edges of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scrollbar-track-color\",desc:\"Determines the color of the track element of a scroll bar.\",browsers:\"IE8\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-ms-scroll-chaining\",desc:\"Gets or sets a value that indicates the scrolling behavior that occurs when a user hits the content boundary during a manipulation.\",browsers:\"E,IE10\",restriction:\"enum, length\",values:[{name:\"chained\"},{name:\"none\"}],status:\"n\",syntax:\"chained | none\"},{name:\"-ms-scroll-limit\",desc:\"Gets or sets a shorthand value that sets values for the -ms-scroll-limit-x-min, -ms-scroll-limit-y-min, -ms-scroll-limit-x-max, and -ms-scroll-limit-y-max properties.\",browsers:\"E,IE10\",restriction:\"length\",values:[{name:\"auto\"}],status:\"n\",syntax:\"<'-ms-scroll-limit-x-min'> <'-ms-scroll-limit-y-min'> <'-ms-scroll-limit-x-max'> <'-ms-scroll-limit-y-max'>\"},{name:\"-ms-scroll-limit-x-max\",desc:\"Gets or sets a value that specifies the maximum value for the scrollLeft property.\",browsers:\"E,IE10\",restriction:\"length\",values:[{name:\"auto\"}],status:\"n\",syntax:\"auto | <length>\"},{name:\"-ms-scroll-limit-x-min\",desc:\"Gets or sets a value that specifies the minimum value for the scrollLeft property.\",browsers:\"E,IE10\",restriction:\"length\",status:\"n\",syntax:\"<length>\"},{name:\"-ms-scroll-limit-y-max\",desc:\"Gets or sets a value that specifies the maximum value for the scrollTop property.\",browsers:\"E,IE10\",restriction:\"length\",values:[{name:\"auto\"}],status:\"n\",syntax:\"auto | <length>\"},{name:\"-ms-scroll-limit-y-min\",desc:\"Gets or sets a value that specifies the minimum value for the scrollTop property.\",browsers:\"E,IE10\",restriction:\"length\",status:\"n\",syntax:\"<length>\"},{name:\"-ms-scroll-rails\",desc:\"Gets or sets a value that indicates whether or not small motions perpendicular to the primary axis of motion will result in either changes to both the scrollTop and scrollLeft properties or a change to the primary axis (for instance, either the scrollTop or scrollLeft properties will change, but not both).\",browsers:\"E,IE10\",restriction:\"enum, length\",values:[{name:\"none\"},{name:\"railed\"}],status:\"n\",syntax:\"none | railed\"},{name:\"-ms-scroll-snap-points-x\",desc:\"Gets or sets a value that defines where snap-points will be located along the x-axis.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"snapInterval(100%, 100%)\"},{name:\"snapList()\"}],status:\"n\",syntax:\"snapInterval( <length-percentage>, <length-percentage> ) | snapList( <length-percentage># )\"},{name:\"-ms-scroll-snap-points-y\",desc:\"Gets or sets a value that defines where snap-points will be located along the y-axis.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"snapInterval(100%, 100%)\"},{name:\"snapList()\"}],status:\"n\",syntax:\"snapInterval( <length-percentage>, <length-percentage> ) | snapList( <length-percentage># )\"},{name:\"-ms-scroll-snap-type\",desc:\"Gets or sets a value that defines what type of snap-point should be used for the current element. There are two type of snap-points, with the primary difference being whether or not the user is guaranteed to always stop on a snap-point.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"none\",desc:\"The visual viewport of this scroll container must ignore snap points, if any, when scrolled.\"},{name:\"mandatory\",desc:\"The visual viewport of this scroll container is guaranteed to rest on a snap point when there are no active scrolling operations.\"},{name:\"proximity\",desc:\"The visual viewport of this scroll container may come to rest on a snap point at the termination of a scroll at the discretion of the UA given the parameters of the scroll.\"}],status:\"n\",syntax:\"none | proximity | mandatory\"},{name:\"-ms-scroll-snap-x\",desc:\"Gets or sets a shorthand value that sets values for the -ms-scroll-snap-type and -ms-scroll-snap-points-x properties.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"mandatory\"},{name:\"none\"},{name:\"proximity\"},{name:\"snapInterval(100%, 100%)\"},{name:\"snapList()\"}],status:\"n\",syntax:\"<'-ms-scroll-snap-type'> <'-ms-scroll-snap-points-x'>\"},{name:\"-ms-scroll-snap-y\",desc:\"Gets or sets a shorthand value that sets values for the -ms-scroll-snap-type and -ms-scroll-snap-points-y properties.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"mandatory\"},{name:\"none\"},{name:\"proximity\"},{name:\"snapInterval(100%, 100%)\"},{name:\"snapList()\"}],status:\"n\",syntax:\"<'-ms-scroll-snap-type'> <'-ms-scroll-snap-points-y'>\"},{name:\"-ms-scroll-translation\",desc:\"Gets or sets a value that specifies whether vertical-to-horizontal scroll wheel translation occurs on the specified element.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"none\"},{name:\"vertical-to-horizontal\"}],status:\"n\",syntax:\"none | vertical-to-horizontal\"},{name:\"-ms-text-align-last\",desc:\"Describes how the last line of a block or a line right before a forced line break is aligned when 'text-align' is set to 'justify'.\",browsers:\"E,IE8\",restriction:\"enum\",values:[{name:\"auto\"},{name:\"center\",desc:\"The inline contents are centered within the line box.\"},{name:\"justify\",desc:\"The text is justified according to the method specified by the 'text-justify' property.\"},{name:\"left\",desc:\"The inline contents are aligned to the left edge of the line box. In vertical text, 'left' aligns to the edge of the line box that would be the start edge for left-to-right text.\"},{name:\"right\",desc:\"The inline contents are aligned to the right edge of the line box. In vertical text, 'right' aligns to the edge of the line box that would be the end edge for left-to-right text.\"}]},{name:\"-ms-text-autospace\",desc:\"Determines whether or not a full-width punctuation mark character should be trimmed if it appears at the beginning of a line, so that its 'ink' lines up with the first glyph in the line above and below.\",browsers:\"E,IE8\",restriction:\"enum\",values:[{name:\"ideograph-alpha\"},{name:\"ideograph-numeric\"},{name:\"ideograph-parenthesis\"},{name:\"ideograph-space\"},{name:\"none\",desc:\"No extra space is created.\"},{name:\"punctuation\"}],status:\"n\",syntax:\"none | ideograph-alpha | ideograph-numeric | ideograph-parenthesis | ideograph-space\"},{name:\"-ms-text-combine-horizontal\",desc:\"This property specifies the combination of multiple characters into the space of a single character.\",browsers:\"E,IE11\",restriction:\"enum, integer\",values:[{name:\"all\",desc:\"Attempt to typeset horizontally all consecutive characters within the box such that they take up the space of a single character within the vertical line box.\"},{name:\"digits\"},{name:\"none\",desc:\"No special processing.\"}]},{name:\"-ms-text-justify\",desc:\"Selects the justification algorithm used when 'text-align' is set to 'justify'. The property applies to block containers, but the UA may (but is not required to) also support it on inline elements.\",browsers:\"E,IE8\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The UA determines the justification algorithm to follow, based on a balance between performance and adequate presentation quality.\"},{name:\"distribute\",desc:\"Justification primarily changes spacing both at word separators and at grapheme cluster boundaries in all scripts except those in the connected and cursive groups. This value is sometimes used in e.g. Japanese, often with the 'text-align-last' property.\"},{name:\"inter-cluster\"},{name:\"inter-ideograph\"},{name:\"inter-word\"},{name:\"kashida\"}]},{name:\"-ms-text-kashida-space\",desc:\"Sets or retrieves the ratio of kashida expansion to white space expansion when justifying lines of text in the object.\",browsers:\"E,IE10\",restriction:\"percentage\"},{name:\"-ms-text-overflow\",desc:\"Text can overflow for example when it is prevented from wrapping\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"clip\"},{name:\"ellipsis\"}]},{name:\"-ms-text-size-adjust\",desc:\"Specifies a size adjustment for displaying text content in mobile browsers.\",browsers:\"E,IE10\",restriction:\"enum, percentage\",values:[{name:\"auto\",desc:\"Renderers must use the default size adjustment when displaying on a small device.\"},{name:\"none\",desc:\"Renderers must not do size adjustment when displaying on a small device.\"}]},{name:\"-ms-text-underline-position\",desc:\"Sets the position of an underline specified on the same element: it does not affect underlines specified by ancestor elements.This property is typically used in vertical writing contexts such as in Japanese documents where it often desired to have the underline appear 'over' (to the right of) the affected run of text\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"alphabetic\",desc:\"The underline is aligned with the alphabetic baseline. In this case the underline is likely to cross some descenders.\"},{name:\"auto\",desc:\"The user agent may use any algorithm to determine the underline's position. In horizontal line layout, the underline should be aligned as for alphabetic. In vertical line layout, if the language is set to Japanese or Korean, the underline should be aligned as for over.\"},{name:\"over\"},{name:\"under\"}]},{name:\"-ms-touch-action\",desc:\"Gets or sets a value that indicates whether and how a given region can be manipulated by the user.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The element is a passive element, with several exceptions.\"},{name:\"double-tap-zoom\",desc:\"The element will zoom on double-tap.\"},{name:\"manipulation\",desc:\"The element is a manipulation-causing element.\"},{name:\"none\",desc:\"The element is a manipulation-blocking element.\"},{name:\"pan-x\",desc:\"The element permits touch-driven panning on the horizontal axis. The touch pan is performed on the nearest ancestor with horizontally scrollable content.\"},{name:\"pan-y\",desc:\"The element permits touch-driven panning on the vertical axis. The touch pan is performed on the nearest ancestor with vertically scrollable content.\"},{name:\"pinch-zoom\",desc:\"The element permits pinch-zooming. The pinch-zoom is performed on the nearest ancestor with zoomable content.\"}]},{name:\"-ms-touch-select\",desc:\"Gets or sets a value that toggles the 'gripper' visual elements that enable touch text selection.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"grippers\"},{name:\"none\",desc:\"Grippers are always off.\"}],status:\"n\",syntax:\"grippers | none\"},{name:\"-ms-transform\",desc:\"A two-dimensional transformation is applied to an element through the 'transform' property. This property contains a list of transform functions similar to those allowed by SVG.\",browsers:\"IE9-9\",restriction:\"enum\",values:[{name:\"matrix()\"},{name:\"matrix3d()\"},{name:\"none\"},{name:\"rotate()\"},{name:\"rotate3d()\"},{name:\"rotateX('angle')\"},{name:\"rotateY('angle')\"},{name:\"rotateZ('angle')\"},{name:\"scale()\"},{name:\"scale3d()\"},{name:\"scaleX()\"},{name:\"scaleY()\"},{name:\"scaleZ()\"},{name:\"skew()\"},{name:\"skewX()\"},{name:\"skewY()\"},{name:\"translate()\"},{name:\"translate3d()\"},{name:\"translateX()\"},{name:\"translateY()\"},{name:\"translateZ()\"}]},{name:\"-ms-transform-origin\",desc:\"Establishes the origin of transformation for an element.\",browsers:\"IE9-9\",restriction:\"position, length, percentage\"},{name:\"-ms-transform-origin-x\",desc:\"The x coordinate of the origin for transforms applied to an element with respect to its border box.\",browsers:\"IE10\",restriction:\"length, percentage\"},{name:\"-ms-transform-origin-y\",desc:\"The y coordinate of the origin for transforms applied to an element with respect to its border box.\",browsers:\"IE10\",restriction:\"length, percentage\"},{name:\"-ms-transform-origin-z\",desc:\"The z coordinate of the origin for transforms applied to an element with respect to its border box.\",browsers:\"IE10\",restriction:\"length, percentage\"},{name:\"-ms-user-select\",desc:\"Controls the appearance of selection.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"element\"},{name:\"none\"},{name:\"text\"}],status:\"n\",syntax:\"none | element | text\"},{name:\"-ms-word-break\",desc:\"Specifies line break opportunities for non-CJK scripts.\",browsers:\"IE8\",restriction:\"enum\",values:[{name:\"break-all\"},{name:\"keep-all\",desc:\"Block characters can no longer create implied break points.\"},{name:\"normal\",desc:\"Breaks non-CJK scripts according to their own rules.\"}]},{name:\"-ms-word-wrap\",desc:\"Specifies whether the UA may break within a word to prevent overflow when an otherwise-unbreakable string is too long to fit.\",browsers:\"IE8\",restriction:\"enum\",values:[{name:\"break-word\",desc:\"An unbreakable 'word' may be broken at an arbitrary point if there are no otherwise-acceptable break points in the line.\"},{name:\"normal\",desc:\"Lines may break only at allowed break points.\"}]},{name:\"-ms-wrap-flow\",desc:\"An element becomes an exclusion when its 'wrap-flow' property has a computed value other than 'auto'.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"auto\",desc:\"For floats an exclusion is created, for all other elements an exclusion is not created.\"},{name:\"both\",desc:\"Inline flow content can flow on all sides of the exclusion.\"},{name:\"clear\"},{name:\"end\",desc:\"Inline flow content can wrap on the end side of the exclusion area but must leave the area to the start edge of the exclusion area empty.\"},{name:\"maximum\"},{name:\"minimum\"},{name:\"start\",desc:\"Inline flow content can wrap on the start edge of the exclusion area but must leave the area to end edge of the exclusion area empty.\"}],status:\"n\",syntax:\"auto | both | start | end | maximum | clear\"},{name:\"-ms-wrap-margin\",desc:\"Gets or sets a value that is used to offset the inner wrap shape from other shapes.\",browsers:\"E,IE10\",restriction:\"length, percentage\",status:\"n\",syntax:\"<length>\"},{name:\"-ms-wrap-through\",desc:\"Specifies if an element inherits its parent wrapping context. In other words if it is subject to the exclusions defined outside the element.\",browsers:\"E,IE10\",restriction:\"enum\",values:[{name:\"none\",desc:\"The exclusion element does not inherit its parent node's wrapping context. Its descendants are only subject to exclusion shapes defined inside the element.\"},{name:\"wrap\",desc:\"The exclusion element inherits its parent node's wrapping context. Its descendant inline content wraps around exclusions defined outside the element.\"}],status:\"n\",syntax:\"wrap | none\"},{name:\"-ms-writing-mode\",desc:\"Shorthand property for both 'direction' and 'block-progression'.\",browsers:\"IE8\",restriction:\"enum\",values:[{name:\"bt-lr\"},{name:\"bt-rl\"},{name:\"lr-bt\"},{name:\"lr-tb\"},{name:\"rl-bt\"},{name:\"rl-tb\"},{name:\"tb-lr\"},{name:\"tb-rl\"}]},{name:\"-ms-zoom\",desc:\"Sets or retrieves the magnification scale of the object.\",browsers:\"IE8\",restriction:\"enum, integer, number, percentage\",values:[{name:\"normal\"}]},{name:\"-ms-zoom-animation\",desc:\"Gets or sets a value that indicates whether an animation is used when zooming.\",browsers:\"IE10\",restriction:\"enum\",values:[{name:\"default\"},{name:\"none\"}]},{name:\"nav-down\",desc:\"Provides an way to control directional focus navigation.\",browsers:\"O9.5\",restriction:\"enum, identifier, string\",values:[{name:\"auto\",desc:\"The user agent automatically determines which element to navigate the focus to in response to directional navigational input.\"},{name:\"current\"},{name:\"root\"}]},{name:\"nav-index\",desc:\"Provides an input-method-neutral way of specifying the sequential navigation order (also known as 'tabbing order').\",browsers:\"O9.5\",restriction:\"number\",values:[{name:\"auto\",desc:\"The element's sequential navigation order is assigned automatically by the user agent.\"}]},{name:\"nav-left\",desc:\"Provides an way to control directional focus navigation.\",browsers:\"O9.5\",restriction:\"enum, identifier, string\",values:[{name:\"auto\",desc:\"The user agent automatically determines which element to navigate the focus to in response to directional navigational input.\"},{name:\"current\"},{name:\"root\"}]},{name:\"nav-right\",desc:\"Provides an way to control directional focus navigation.\",browsers:\"O9.5\",restriction:\"enum, identifier, string\",values:[{name:\"auto\",desc:\"The user agent automatically determines which element to navigate the focus to in response to directional navigational input.\"},{name:\"current\"},{name:\"root\"}]},{name:\"nav-up\",desc:\"Provides an way to control directional focus navigation.\",browsers:\"O9.5\",restriction:\"enum, identifier, string\",values:[{name:\"auto\",desc:\"The user agent automatically determines which element to navigate the focus to in response to directional navigational input.\"},{name:\"current\"},{name:\"root\"}]},{name:\"negative\",desc:\"@counter-style descriptor. Defines how to alter the representation when the counter value is negative.\",browsers:\"FF33\",restriction:\"image, identifier, string\",syntax:\"<symbol> <symbol>?\"},{name:\"-o-animation\",desc:\"Shorthand property combines six of the animation properties into a single property.\",browsers:\"O12\",restriction:\"time, enum, timing-function, identifier, number\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"backwards\"},{name:\"both\",desc:\"Both forwards and backwards fill modes are applied.\"},{name:\"forwards\"},{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"},{name:\"none\",desc:\"No animation is performed\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}]},{name:\"-o-animation-delay\",desc:\"Defines when the animation will start.\",browsers:\"O12\",restriction:\"time\"},{name:\"-o-animation-direction\",desc:\"Defines whether or not the animation should play in reverse on alternate cycles.\",browsers:\"O12\",restriction:\"enum\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}]},{name:\"-o-animation-duration\",desc:\"Defines the length of time that an animation takes to complete one cycle.\",browsers:\"O12\",restriction:\"time\"},{name:\"-o-animation-fill-mode\",desc:\"Defines what values are applied by the animation outside the time it is executing.\",browsers:\"O12\",restriction:\"enum\",values:[{name:\"backwards\"},{name:\"both\",desc:\"Both forwards and backwards fill modes are applied.\"},{name:\"forwards\"},{name:\"none\",desc:\"There is no change to the property value between the time the animation is applied and the time the animation begins playing or after the animation completes.\"}]},{name:\"-o-animation-iteration-count\",desc:\"Defines the number of times an animation cycle is played. The default value is one, meaning the animation will play from beginning to end once.\",browsers:\"O12\",restriction:\"number, enum\",values:[{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"}]},{name:\"-o-animation-name\",desc:\"Defines a list of animations that apply. Each name is used to select the keyframe at-rule that provides the property values for the animation.\",browsers:\"O12\",restriction:\"identifier, enum\",values:[{name:\"none\",desc:\"No animation is performed\"}]},{name:\"-o-animation-play-state\",desc:\"Defines whether the animation is running or paused.\",browsers:\"O12\",restriction:\"enum\",values:[{name:\"paused\"},{name:\"running\"}]},{name:\"-o-animation-timing-function\",desc:\"Describes how the animation will progress over one cycle of its duration. See the 'transition-timing-function'.\",browsers:\"O12\",restriction:\"timing-function\"},{name:\"object-fit\",desc:\"Specifies how the contents of a replaced element should be scaled relative to the box established by its used height and width.\",browsers:\"E16,FF36,S,C31,O19\",restriction:\"enum\",values:[{name:\"contain\",desc:\"The replaced content is sized to maintain its aspect ratio while fitting within the element’s content box: its concrete object size is resolved as a contain constraint against the element's used width and height.\"},{name:\"cover\",desc:\"The replaced content is sized to maintain its aspect ratio while filling the element's entire content box: its concrete object size is resolved as a cover constraint against the element’s used width and height.\"},{name:\"fill\",desc:\"The replaced content is sized to fill the element’s content box: the object's concrete object size is the element's used width and height.\"},{name:\"none\",desc:\"The replaced content is not resized to fit inside the element's content box\"},{name:\"scale-down\"}],syntax:\"fill | contain | cover | none | scale-down\"},{name:\"object-position\",desc:\"Determines the alignment of the replaced element inside its box.\",browsers:\"E16,FF36,S10,C31,O19\",restriction:\"position, length, percentage\",syntax:\"<position>\"},{name:\"-o-border-image\",desc:\"Shorthand property for setting 'border-image-source', 'border-image-slice', 'border-image-width', 'border-image-outset' and 'border-image-repeat'. Omitted values are set to their initial values.\",browsers:\"O11.6\",restriction:\"length, percentage, number, image, enum\",values:[{name:\"auto\",desc:\"If 'auto' is specified then the border image width is the intrinsic width or height (whichever is applicable) of the corresponding image slice. If the image does not have the required intrinsic dimension then the corresponding border-width is used instead.\"},{name:\"fill\",desc:\"Causes the middle part of the border-image to be preserved.\"},{name:\"none\"},{name:\"repeat\"},{name:\"round\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the image is rescaled so that it does.\"},{name:\"space\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the extra space is distributed around the tiles.\"},{name:\"stretch\",desc:\"The image is stretched to fill the area.\"}]},{name:\"-o-object-fit\",desc:\"Specifies how the contents of a replaced element should be scaled relative to the box established by its used height and width.\",browsers:\"O10.6\",restriction:\"enum\",values:[{name:\"contain\",desc:\"The replaced content is sized to maintain its aspect ratio while fitting within the element’s content box: its concrete object size is resolved as a contain constraint against the element's used width and height.\"},{name:\"cover\",desc:\"The replaced content is sized to maintain its aspect ratio while filling the element's entire content box: its concrete object size is resolved as a cover constraint against the element’s used width and height.\"},{name:\"fill\",desc:\"The replaced content is sized to fill the element’s content box: the object's concrete object size is the element's used width and height.\"},{name:\"none\",desc:\"The replaced content is not resized to fit inside the element's content box\"},{name:\"scale-down\"}]},{name:\"-o-object-position\",desc:\"Determines the alignment of the replaced element inside its box.\",browsers:\"O10.6\",restriction:\"position, length, percentage\"},{name:\"opacity\",desc:\"Opacity of an element's text, where 1 is opaque and 0 is entirely transparent.\",browsers:\"all\",restriction:\"number(0-1)\",syntax:\"<number>\"},{name:\"order\",desc:\"Controls the order in which children of a flex container appear within the flex container, by assigning them to ordinal groups.\",browsers:\"all\",restriction:\"integer\",syntax:\"<integer>\"},{name:\"orphans\",desc:\"Specifies the minimum number of line boxes in a block container that must be left in a fragment before a fragmentation break.\",browsers:\"E12,C25,IE8,O9.2\",restriction:\"integer\",syntax:\"<integer>\"},{name:\"-o-table-baseline\",desc:\"Determines which row of a inline-table should be used as baseline of inline-table.\",browsers:\"O9.6\",restriction:\"integer\"},{name:\"-o-tab-size\",desc:\"This property determines the width of the tab character (U+0009), in space characters (U+0020), when rendered.\",browsers:\"O10.6\",restriction:\"integer, length\"},{name:\"-o-text-overflow\",desc:\"Text can overflow for example when it is prevented from wrapping\",browsers:\"O10\",restriction:\"enum\",values:[{name:\"clip\"},{name:\"ellipsis\"}]},{name:\"-o-transform\",desc:\"A two-dimensional transformation is applied to an element through the 'transform' property. This property contains a list of transform functions similar to those allowed by SVG.\",browsers:\"O10.5\",restriction:\"enum\",values:[{name:\"matrix()\"},{name:\"matrix3d()\"},{name:\"none\"},{name:\"rotate()\"},{name:\"rotate3d()\"},{name:\"rotateX('angle')\"},{name:\"rotateY('angle')\"},{name:\"rotateZ('angle')\"},{name:\"scale()\"},{name:\"scale3d()\"},{name:\"scaleX()\"},{name:\"scaleY()\"},{name:\"scaleZ()\"},{name:\"skew()\"},{name:\"skewX()\"},{name:\"skewY()\"},{name:\"translate()\"},{name:\"translate3d()\"},{name:\"translateX()\"},{name:\"translateY()\"},{name:\"translateZ()\"}]},{name:\"-o-transform-origin\",desc:\"Establishes the origin of transformation for an element.\",browsers:\"O10.5\",restriction:\"positon, length, percentage\"},{name:\"-o-transition\",desc:\"Shorthand property combines four of the transition properties into a single property.\",browsers:\"O11.5\",restriction:\"time, property, timing-function, enum\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}]},{name:\"-o-transition-delay\",desc:\"Defines when the transition will start. It allows a transition to begin execution some period of time from when it is applied.\",browsers:\"O11.5\",restriction:\"time\"},{name:\"-o-transition-duration\",desc:\"Specifies how long the transition from the old value to the new value should take.\",browsers:\"O11.5\",restriction:\"time\"},{name:\"-o-transition-property\",desc:\"Specifies the name of the CSS property to which the transition is applied.\",browsers:\"O11.5\",restriction:\"property\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}]},{name:\"-o-transition-timing-function\",desc:\"Describes how the intermediate values used during a transition will be calculated.\",browsers:\"O11.5\",restriction:\"timing-function\"},{name:\"offset-block-end\",desc:\"Logical 'bottom'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well.\"}],syntax:\"<'left'>\"},{name:\"offset-block-start\",desc:\"Logical 'top'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well.\"}],syntax:\"<'left'>\"},{name:\"offset-inline-end\",desc:\"Logical 'right'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well.\"}],syntax:\"<'left'>\"},{name:\"offset-inline-start\",desc:\"Logical 'left'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well.\"}],syntax:\"<'left'>\"},{name:\"outline\",desc:\"Shorthand property for 'outline-style', 'outline-width', and 'outline-color'.\",browsers:\"all\",restriction:\"length, line-width, line-style, color, enum\",values:[{name:\"auto\",desc:\"Permits the user agent to render a custom outline style, typically the default platform style.\"},{name:\"invert\",browsers:\"E,IE8,O\"}],syntax:\"[ <'outline-color'> || <'outline-style'> || <'outline-width'> ]\"},{name:\"outline-color\",desc:\"The color of the outline.\",browsers:\"all\",restriction:\"enum, color\",values:[{name:\"invert\",browsers:\"E,IE8,O\"}],syntax:\"<color> | invert\"},{name:\"outline-offset\",desc:\"Offset the outline and draw it beyond the border edge.\",browsers:\"FF1.5,S1.2,C1,O9.5\",restriction:\"length\",syntax:\"<length>\"},{name:\"outline-style\",desc:\"Style of the outline.\",browsers:\"all\",restriction:\"line-style, enum\",values:[{name:\"auto\",desc:\"Permits the user agent to render a custom outline style, typically the default platform style.\"}],syntax:\"auto | <br-style>\"},{name:\"outline-width\",desc:\"Width of the outline.\",browsers:\"all\",restriction:\"length, line-width\",syntax:\"<br-width>\"},{name:\"overflow\",desc:\"Shorthand for setting 'overflow-x' and 'overflow-y'.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The behavior of the 'auto' value is UA-dependent, but should cause a scrolling mechanism to be provided for overflowing boxes.\"},{name:\"hidden\",desc:\"Content is clipped and no scrolling mechanism should be provided to view the content outside the clipping region.\"},{name:\"-moz-hidden-unscrollable\",browsers:\"FF\"},{name:\"scroll\",desc:\"Content is clipped and if the user agent uses a scrolling mechanism that is visible on the screen (such as a scroll bar or a panner), that mechanism should be displayed for a box whether or not any of its content is clipped.\"},{name:\"visible\",desc:\"Content is not clipped, i.e., it may be rendered outside the content box.\"}],syntax:\"[ visible | hidden | clip | scroll | auto ]{1,2}\"},{name:\"overflow-wrap\",desc:\"Specifies whether the UA may break within a word to prevent overflow when an otherwise-unbreakable string is too long to fit within the line box.\",browsers:\"FF49,S,C,IE5.5,O\",restriction:\"enum\",values:[{name:\"break-word\",desc:\"An otherwise unbreakable sequence of characters may be broken at an arbitrary point if there are no otherwise-acceptable break points in the line.\"},{name:\"normal\",desc:\"Lines may break only at allowed break points.\"}],syntax:\"normal | break-word\"},{name:\"overflow-x\",desc:\"Specifies the handling of overflow in the horizontal direction.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The behavior of the 'auto' value is UA-dependent, but should cause a scrolling mechanism to be provided for overflowing boxes.\"},{name:\"hidden\",desc:\"Content is clipped and no scrolling mechanism should be provided to view the content outside the clipping region.\"},{name:\"scroll\",desc:\"Content is clipped and if the user agent uses a scrolling mechanism that is visible on the screen (such as a scroll bar or a panner), that mechanism should be displayed for a box whether or not any of its content is clipped.\"},{name:\"visible\",desc:\"Content is not clipped, i.e., it may be rendered outside the content box.\"}],syntax:\"visible | hidden | clip | scroll | auto\"},{name:\"overflow-y\",desc:\"Specifies the handling of overflow in the vertical direction.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The behavior of the 'auto' value is UA-dependent, but should cause a scrolling mechanism to be provided for overflowing boxes.\"},{name:\"hidden\",desc:\"Content is clipped and no scrolling mechanism should be provided to view the content outside the clipping region.\"},{name:\"scroll\",desc:\"Content is clipped and if the user agent uses a scrolling mechanism that is visible on the screen (such as a scroll bar or a panner), that mechanism should be displayed for a box whether or not any of its content is clipped.\"},{name:\"visible\",desc:\"Content is not clipped, i.e., it may be rendered outside the content box.\"}],syntax:\"visible | hidden | clip | scroll | auto\"},{name:\"pad\",desc:\"@counter-style descriptor. Specifies a “fixed-width” counter style, where representations shorter than the pad value are padded with a particular <symbol>\",browsers:\"FF33\",restriction:\"integer, image, string, identifier\",syntax:\"<integer> && <symbol>\"},{name:\"padding\",desc:\"Shorthand property to set values the thickness of the padding area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. The value may not be negative.\",browsers:\"all\",restriction:\"length, percentage\",values:[],syntax:\"[ <length> | <percentage> ]{1,4}\"},{name:\"padding-bottom\",desc:\"Shorthand property to set values the thickness of the padding area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. The value may not be negative.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length> | <percentage>\"},{name:\"padding-block-end\",desc:\"Logical 'padding-bottom'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",syntax:\"<'padding-left'>\"},{name:\"padding-block-start\",desc:\"Logical 'padding-top'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41\",restriction:\"length, percentage\",syntax:\"<'padding-left'>\"},{name:\"padding-inline-end\",desc:\"Logical 'padding-right'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41,S3,C2,O15\",restriction:\"length, percentage\",syntax:\"<'padding-left'>\"},{name:\"padding-inline-start\",desc:\"Logical 'padding-left'. Mapping depends on the parent element’s 'writing-mode', 'direction', and 'text-orientation'.\",browsers:\"FF41,S3,C2,O\",restriction:\"length, percentage\",syntax:\"<'padding-left'>\"},{name:\"padding-left\",desc:\"Shorthand property to set values the thickness of the padding area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. The value may not be negative.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length> | <percentage>\"},{name:\"padding-right\",desc:\"Shorthand property to set values the thickness of the padding area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. The value may not be negative.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length> | <percentage>\"},{name:\"padding-top\",desc:\"Shorthand property to set values the thickness of the padding area. If left is omitted, it is the same as right. If bottom is omitted it is the same as top, if right is omitted it is the same as top. The value may not be negative.\",browsers:\"all\",restriction:\"length, percentage\",syntax:\"<length> | <percentage>\"},{name:\"page-break-after\",desc:\"Defines rules for page breaks after an element.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break after the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page break after generated box.\"},{name:\"avoid\",desc:\"Avoid a page break after the generated box.\"},{name:\"left\",desc:\"Force one or two page breaks after the generated box so that the next page is formatted as a left page.\"},{name:\"right\",desc:\"Force one or two page breaks after the generated box so that the next page is formatted as a right page.\"}],syntax:\"auto | always | avoid | left | right | recto | verso\"},{name:\"page-break-before\",desc:\"Defines rules for page breaks before an element.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break before the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page break before the generated box.\"},{name:\"avoid\",desc:\"Avoid a page break before the generated box.\"},{name:\"left\",desc:\"Force one or two page breaks before the generated box so that the next page is formatted as a left page.\"},{name:\"right\",desc:\"Force one or two page breaks before the generated box so that the next page is formatted as a right page.\"}],syntax:\"auto | always | avoid | left | right | recto | verso\"},{name:\"page-break-inside\",desc:\"Defines rules for page breaks inside an element.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Neither force nor forbid a page break inside the generated box.\"},{name:\"avoid\",desc:\"Avoid a page break inside the generated box.\"}],syntax:\"auto | avoid\"},{name:\"paint-order\",desc:\"Controls the order that the three paint operations that shapes and text are rendered with: their fill, their stroke and any markers they might have.\",browsers:\"FF60,S\",restriction:\"enum\",values:[{name:\"fill\"},{name:\"markers\"},{name:\"normal\",desc:\"The element is painted with the standard order of painting operations: the 'fill' is painted first, then its 'stroke' and finally its markers.\"},{name:\"stroke\"}],status:\"e\",syntax:\"normal | [ fill || stroke || markers ]\"},{name:\"perspective\",desc:\"Applies the same transform as the perspective(<number>) transform function, except that it applies only to the positioned or transformed children of the element, not to the transform on the element itself.\",browsers:\"all\",restriction:\"length, enum\",values:[{name:\"none\",desc:\"No perspective transform is applied.\"}],syntax:\"none | <length>\"},{name:\"perspective-origin\",desc:\"Establishes the origin for the perspective property. It effectively sets the X and Y position at which the viewer appears to be looking at the children of the element.\",browsers:\"all\",restriction:\"position, percentage, length\",syntax:\"<position>\"},{name:\"pointer-events\",desc:\"Specifies under what circumstances a given element can be the target element for a pointer event.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"all\",desc:\"The given element can be the target element for pointer events whenever the pointer is over either the interior or the perimeter of the element.\"},{name:\"fill\",desc:\"The given element can be the target element for pointer events whenever the pointer is over the interior of the element.\"},{name:\"none\",desc:\"The given element does not receive pointer events.\"},{name:\"painted\"},{name:\"stroke\",desc:\"The given element can be the target element for pointer events whenever the pointer is over the perimeter of the element.\"},{name:\"visible\",desc:\"The given element can be the target element for pointer events when the ‘visibility’ property is set to visible and the pointer is over either the interior or the perimete of the element.\"},{name:\"visibleFill\"},{name:\"visiblePainted\"},{name:\"visibleStroke\"}],syntax:\"auto | none | visiblePainted | visibleFill | visibleStroke | visible | painted | fill | stroke | all | inherit\"},{name:\"position\",browsers:\"all\",restriction:\"enum\",values:[{name:\"absolute\"},{name:\"fixed\",desc:\"The box's position is calculated according to the 'absolute' model, but in addition, the box is fixed with respect to some reference. As with the 'absolute' model, the box's margins do not collapse with any other margins.\"},{name:\"-ms-page\",browsers:\"E,IE10\"},{name:\"relative\"},{name:\"static\"},{name:\"sticky\",browsers:\"C56,FF32\"},{name:\"-webkit-sticky\",browsers:\"S6.1\"}],syntax:\"static | relative | absolute | sticky | fixed\"},{name:\"prefix\",desc:\"@counter-style descriptor. Specifies a <symbol> that is prepended to the marker representation.\",browsers:\"FF33\",restriction:\"image, string, identifier\",syntax:\"<symbol>\"},{name:\"quotes\",desc:\"Specifies quotation marks for any number of embedded quotations.\",browsers:\"all\",restriction:\"string\",values:[{name:\"none\",desc:\"The 'open-quote' and 'close-quote' values of the 'content' property produce no quotations marks, as if they were 'no-open-quote' and 'no-close-quote' respectively.\"}],syntax:\"none | [ <string> <string> ]+\"},{name:\"range\",desc:\"@counter-style descriptor. Defines the ranges over which the counter style is defined.\",browsers:\"FF33\",restriction:\"integer, enum\",values:[{name:\"auto\",desc:\"The range depends on the counter system.\"},{name:\"infinite\",desc:\"If used as the first value in a range, it represents negative infinity; if used as the second value, it represents positive infinity.\"}],syntax:\"[ [ <integer> | infinite ]{2} ]# | auto\"},{name:\"resize\",desc:\"Specifies whether or not an element is resizable by the user, and if so, along which axis/axes.\",browsers:\"FF,S3,C1,O12.1\",restriction:\"enum\",values:[{name:\"both\",desc:\"The UA presents a bidirectional resizing mechanism to allow the user to adjust both the height and the width of the element.\"},{name:\"horizontal\",desc:\"The UA presents a unidirectional horizontal resizing mechanism to allow the user to adjust only the width of the element.\"},{name:\"none\",desc:\"The UA does not present a resizing mechanism on the element, and the user is given no direct manipulation mechanism to resize the element.\"},{name:\"vertical\",desc:\"The UA presents a unidirectional vertical resizing mechanism to allow the user to adjust only the height of the element.\"}],syntax:\"none | both | horizontal | vertical\"},{name:\"right\",desc:\"Specifies how far an absolutely positioned box's right margin edge is offset to the left of the right edge of the box's 'containing block'.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"ruby-align\",desc:\"Specifies how text is distributed within the various ruby boxes when their contents do not exactly fill their respective boxes.\",browsers:\"FF38\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The user agent determines how the ruby contents are aligned. This is the initial value.\",browsers:\"E,IE5\"},{name:\"center\",desc:\"The ruby content is centered within its box.\"},{name:\"distribute-letter\",browsers:\"E,IE5\"},{name:\"distribute-space\",browsers:\"E,IE5\"},{name:\"left\",desc:\"The ruby text content is aligned with the start edge of the base.\"},{name:\"line-edge\",browsers:\"E,IE5\"},{name:\"right\",desc:\"The ruby text content is aligned with the end edge of the base.\",browsers:\"E,IE5\"},{name:\"start\",desc:\"The ruby text content is aligned with the start edge of the base.\",browsers:\"FF10\"},{name:\"space-between\",desc:\"The ruby content expands as defined for normal text justification (as defined by 'text-justify'),\",browsers:\"FF10\"},{name:\"space-around\",desc:\"As for 'space-between' except that there exists an extra justification opportunities whose space is distributed half before and half after the ruby content.\",browsers:\"FF10\"}],status:\"e\",syntax:\"start | center | space-between | space-around\"},{name:\"ruby-overhang\",desc:\"Determines whether, and on which side, ruby text is allowed to partially overhang any adjacent text in addition to its own base, when the ruby text is wider than the ruby base.\",browsers:\"FF10,IE5\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The ruby text can overhang text adjacent to the base on either side. This is the initial value.\"},{name:\"end\",desc:\"The ruby text can overhang the text that follows it.\"},{name:\"none\",desc:\"The ruby text cannot overhang any text adjacent to its base, only its own base.\"},{name:\"start\",desc:\"The ruby text can overhang the text that precedes it.\"}]},{name:\"ruby-position\",desc:\"Used by the parent of elements with display: ruby-text to control the position of the ruby text with respect to its base.\",browsers:\"E12,FF38\",restriction:\"enum\",values:[{name:\"after\"},{name:\"before\"},{name:\"inline\"},{name:\"right\",desc:\"The ruby text appears on the right of the base. Unlike 'before' and 'after', this value is not relative to the text flow direction.\"}],status:\"e\",syntax:\"over | under | inter-character\"},{name:\"ruby-span\",desc:\"Determines whether, and on which side, ruby text is allowed to partially overhang any adjacent text in addition to its own base, when the ruby text is wider than the ruby base.\",browsers:\"FF10\",restriction:\"enum\",values:[{name:\"attr(x)\"},{name:\"none\",desc:\"No spanning. The computed value is '1'.\"}]},{name:\"scrollbar-3dlight-color\",desc:\"Determines the color of the top and left edges of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scrollbar-arrow-color\",desc:\"Determines the color of the arrow elements of a scroll arrow.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scrollbar-base-color\",desc:\"Determines the color of the main elements of a scroll bar, which include the scroll box, track, and scroll arrows.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scrollbar-darkshadow-color\",desc:\"Determines the color of the gutter of a scroll bar.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scrollbar-face-color\",desc:\"Determines the color of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scrollbar-highlight-color\",desc:\"Determines the color of the top and left edges of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scrollbar-shadow-color\",desc:\"Determines the color of the bottom and right edges of the scroll box and scroll arrows of a scroll bar.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scrollbar-track-color\",desc:\"Determines the color of the track element of a scroll bar.\",browsers:\"IE6\",restriction:\"color\"},{name:\"scroll-behavior\",desc:\"Specifies the scrolling behavior for a scrolling box, when scrolling happens due to navigation or CSSOM scrolling APIs.\",browsers:\"FF36,C61,O\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Scrolls in an instant fashion.\"},{name:\"smooth\"}],syntax:\"auto | smooth\"},{name:\"scroll-snap-coordinate\",desc:\"Defines the x and y coordinate within the element which will align with the nearest ancestor scroll container’s snap-destination for the respective axis.\",browsers:\"FF39\",restriction:\"position, length, percentage, enum\",values:[{name:\"none\",desc:\"Specifies that this element does not contribute a snap point.\"}],syntax:\"none | <position>#\"},{name:\"scroll-snap-destination\",desc:\"Define the x and y coordinate within the scroll container’s visual viewport which element snap points will align with.\",browsers:\"FF39\",restriction:\"position, length, percentage\",syntax:\"<position>\"},{name:\"scroll-snap-points-x\",desc:\"Defines the positioning of snap points along the x axis of the scroll container it is applied to.\",browsers:\"FF39,S9\",restriction:\"enum\",values:[{name:\"none\",desc:\"No snap points are defined by this scroll container.\"},{name:\"repeat()\",desc:\"Defines an interval at which snap points are defined, starting from the container’s relevant start edge.\"}],status:\"o\",syntax:\"none | repeat( <length-percentage> )\"},{name:\"scroll-snap-points-y\",desc:\"Defines the positioning of snap points alobg the y axis of the scroll container it is applied to.\",browsers:\"FF39,S9\",restriction:\"enum\",values:[{name:\"none\",desc:\"No snap points are defined by this scroll container.\"},{name:\"repeat()\",desc:\"Defines an interval at which snap points are defined, starting from the container’s relevant start edge.\"}],status:\"o\",syntax:\"none | repeat( <length-percentage> )\"},{name:\"scroll-snap-type\",desc:\"Defines how strictly snap points are enforced on the scroll container.\",browsers:\"E12,FF39,S9,IE10\",restriction:\"enum\",values:[{name:\"none\",desc:\"The visual viewport of this scroll container must ignore snap points, if any, when scrolled.\"},{name:\"mandatory\",desc:\"The visual viewport of this scroll container is guaranteed to rest on a snap point when there are no active scrolling operations.\"},{name:\"proximity\",desc:\"The visual viewport of this scroll container may come to rest on a snap point at the termination of a scroll at the discretion of the UA given the parameters of the scroll.\"}],syntax:\"none | mandatory | proximity\"},{name:\"shape-image-threshold\",desc:\"Defines the alpha channel threshold used to extract the shape using an image. A value of 0.5 means that the shape will enclose all the pixels that are more than 50% opaque.\",browsers:\"FF61,S10.1,C37,O24\",restriction:\"number\",syntax:\"<number>\"},{name:\"shape-margin\",desc:\"Adds a margin to a 'shape-outside'. This defines a new shape that is the smallest contour that includes all the points that are the 'shape-margin' distance outward in the perpendicular direction from a point on the underlying shape.\",browsers:\"FF61,S10.1,C37,O24\",restriction:\"url, length, percentage\",syntax:\"<length-percentage>\"},{name:\"shape-outside\",desc:\"Specifies an orthogonal rotation to be applied to an image before it is laid out.\",browsers:\"FF61,S10.1,C37,O24\",restriction:\"image, box, shape, enum\",values:[{name:\"margin-box\"},{name:\"none\",desc:\"The float area is unaffected.\"}],syntax:\"none | <shape-box> || <basic-shape> | <image>\"},{name:\"shape-rendering\",desc:\"Provides hints about what tradeoffs to make as it renders vector graphics elements such as <path> elements and basic shapes such as circles and rectangles.\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Suppresses aural rendering.\"},{name:\"crispEdges\"},{name:\"geometricPrecision\",desc:\"Emphasize geometric precision over speed and crisp edges.\"},{name:\"optimizeSpeed\",desc:\"Emphasize rendering speed over geometric precision and crisp edges.\"}]},{name:\"size\",browsers:\"C,O8\",restriction:\"length\"},{name:\"src\",desc:\"@font-face descriptor. Specifies the resource containing font data. It is required, whether the font is downloadable or locally installed.\",restriction:\"enum, url, identifier\",values:[{name:\"url()\",desc:\"Reference font by URL\"},{name:\"format()\"},{name:\"local()\"}],syntax:\"[ <url> [ format( <string># ) ]? | local( <family-name> ) ]#\"},{name:\"stop-color\",desc:\"Indicates what color to use at that gradient stop.\",restriction:\"color\"},{name:\"stop-opacity\",desc:\"Defines the opacity of a given gradient stop.\",restriction:\"number(0-1)\"},{name:\"stroke\",desc:\"Paints along the outline of the given graphical element.\",restriction:\"color, enum, url\",values:[{name:\"url()\",desc:\"A URL reference to a paint server element, which is an element that defines a paint server: ‘hatch’, ‘linearGradient’, ‘mesh’, ‘pattern’, ‘radialGradient’ and ‘solidcolor’.\"}]},{name:\"stroke-dasharray\",desc:\"Controls the pattern of dashes and gaps used to stroke paths.\",restriction:\"length, percentage, number, enum\",values:[{name:\"none\",desc:\"Indicates that no dashing is used.\"}]},{name:\"stroke-dashoffset\",desc:\"Specifies the distance into the dash pattern to start the dash.\",restriction:\"percentage, length\"},{name:\"stroke-linecap\",desc:\"Specifies the shape to be used at the end of open subpaths when they are stroked.\",restriction:\"enum\",values:[{name:\"butt\"},{name:\"round\",desc:\"Indicates that at each end of each subpath, the shape representing the stroke will be extended by a half circle with a radius equal to the stroke width.\"},{name:\"square\",desc:\"Indicates that at the end of each subpath, the shape representing the stroke will be extended by a rectangle with the same width as the stroke width and whose length is half of the stroke width.\"}]},{name:\"stroke-linejoin\",desc:\"Specifies the shape to be used at the corners of paths or basic shapes when they are stroked.\",restriction:\"enum\",values:[{name:\"bevel\"},{name:\"miter\"},{name:\"round\",desc:\"Indicates that a round corner is to be used to join path segments.\"}]},{name:\"stroke-miterlimit\",desc:\"When two line segments meet at a sharp angle and miter joins have been specified for 'stroke-linejoin', it is possible for the miter to extend far beyond the thickness of the line stroking the path.\",restriction:\"number\"},{name:\"stroke-opacity\",desc:\"Specifies the opacity of the painting operation used to stroke the current object.\",restriction:\"number(0-1)\"},{name:\"stroke-width\",desc:\"Specifies the width of the stroke on the current object.\",restriction:\"percentage, length\"},{name:\"suffix\",desc:\"@counter-style descriptor. Specifies a <symbol> that is appended to the marker representation.\",browsers:\"FF33\",restriction:\"image, string, identifier\",syntax:\"<symbol>\"},{name:\"system\",desc:\"@counter-style descriptor. Specifies which algorithm will be used to construct the counter’s representation based on the counter value.\",browsers:\"FF33\",restriction:\"enum, integer\",values:[{name:\"additive\"},{name:\"alphabetic\",desc:'Interprets the list of counter symbols as digits to an alphabetic numbering system, similar to the default lower-alpha counter style, which wraps from \"a\", \"b\", \"c\", to \"aa\", \"ab\", \"ac\".'},{name:\"cyclic\"},{name:\"extends\"},{name:\"fixed\",desc:\"Runs through its list of counter symbols once, then falls back.\"},{name:\"numeric\"},{name:\"symbolic\"}],syntax:\"cyclic | numeric | alphabetic | symbolic | additive | [ fixed <integer>? ] | [ extends <counter-style-name> ]\"},{name:\"symbols\",desc:\"@counter-style descriptor. Specifies the symbols used by the marker-construction algorithm specified by the system descriptor.\",browsers:\"FF33\",restriction:\"image, string, identifier\",syntax:\"<symbol>+\"},{name:\"table-layout\",desc:\"Controls the algorithm used to lay out the table cells, rows, and columns.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Use any automatic table layout algorithm.\"},{name:\"fixed\",desc:\"Use the fixed table layout algorithm.\"}],syntax:\"auto | fixed\"},{name:\"tab-size\",desc:\"Determines the width of the tab character (U+0009), in space characters (U+0020), when rendered.\",browsers:\"FF4,S6.1,C21,O15\",restriction:\"integer, length\",syntax:\"<integer> | <length>\"},{name:\"text-align\",desc:\"Describes how inline contents of a block are horizontally aligned if the contents do not completely fill the line box.\",browsers:\"all\",restriction:\"string\",values:[{name:\"center\",desc:\"The inline contents are centered within the line box.\"},{name:\"end\",desc:\"The inline contents are aligned to the end edge of the line box.\",browsers:\"C,FF3.6,O15,S3.1\"},{name:\"justify\",desc:\"The text is justified according to the method specified by the 'text-justify' property.\"},{name:\"left\",desc:\"The inline contents are aligned to the left edge of the line box. In vertical text, 'left' aligns to the edge of the line box that would be the start edge for left-to-right text.\"},{name:\"right\",desc:\"The inline contents are aligned to the right edge of the line box. In vertical text, 'right' aligns to the edge of the line box that would be the end edge for left-to-right text.\"},{name:\"start\",desc:\"The inline contents are aligned to the start edge of the line box.\",browsers:\"C,FF1,O15,S3.1\"}],syntax:\"start | end | left | right | center | justify | match-parent\"},{name:\"text-align-last\",desc:\"Describes how the last line of a block or a line right before a forced line break is aligned when 'text-align' is set to 'justify'.\",browsers:\"E12,FF49,C47,O\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Content on the affected line is aligned per 'text-align' unless 'text-align' is set to 'justify', in which case it is 'start-aligned'.\"},{name:\"center\",desc:\"The inline contents are centered within the line box.\"},{name:\"justify\",desc:\"The text is justified according to the method specified by the 'text-justify' property.\"},{name:\"left\",desc:\"The inline contents are aligned to the left edge of the line box. In vertical text, 'left' aligns to the edge of the line box that would be the start edge for left-to-right text.\"},{name:\"right\",desc:\"The inline contents are aligned to the right edge of the line box. In vertical text, 'right' aligns to the edge of the line box that would be the end edge for left-to-right text.\"}],syntax:\"auto | start | end | left | right | center | justify\"},{name:\"text-anchor\",desc:\"Used to align (start-, middle- or end-alignment) a string of text relative to a given point.\",restriction:\"enum\",values:[{name:\"end\",desc:\"The rendered characters are aligned such that the end of the resulting rendered text is at the initial current text position.\"},{name:\"middle\",desc:\"The rendered characters are aligned such that the geometric middle of the resulting rendered text is at the initial current text position.\"},{name:\"start\",desc:\"The rendered characters are aligned such that the start of the resulting rendered text is at the initial current text position.\"}]},{name:\"text-decoration\",desc:\"Decorations applied to font used for an element's text.\",browsers:\"all\",restriction:\"enum, color\",values:[{name:\"dashed\"},{name:\"dotted\"},{name:\"double\"},{name:\"line-through\"},{name:\"overline\"},{name:\"solid\"},{name:\"underline\"},{name:\"wavy\"}],syntax:\"<'text-decoration-line'> || <'text-decoration-style'> || <'text-decoration-color'>\"},{name:\"text-decoration-color\",desc:\"Specifies the color of text decoration (underlines overlines, and line-throughs) set on the element with text-decoration-line.\",browsers:\"FF36,S,C57,O44\",restriction:\"color\",syntax:\"<color>\"},{name:\"text-decoration-line\",desc:\"Specifies what line decorations, if any, are added to the element.\",browsers:\"FF36,S,C,O\",restriction:\"enum\",values:[{name:\"line-through\"},{name:\"none\",desc:\"Neither produces nor inhibits text decoration.\"},{name:\"overline\"},{name:\"underline\"}],syntax:\"none | [ underline || overline || line-through || blink ]\"},{name:\"text-decoration-style\",desc:\"Specifies the line style for underline, line-through and overline text decoration.\",browsers:\"FF36,S,C57,O44\",restriction:\"enum\",values:[{name:\"dashed\"},{name:\"dotted\"},{name:\"double\"},{name:\"none\",desc:\"Produces no line.\"},{name:\"solid\"},{name:\"wavy\"}],syntax:\"solid | double | dotted | dashed | wavy\"},{name:\"text-indent\",desc:\"Specifies the indentation applied to lines of inline content in a block. The indentation only affects the first line of inline content in the block unless the 'hanging' keyword is specified, in which case it affects all lines except the first.\",browsers:\"all\",restriction:\"percentage, length\",values:[],syntax:\"<length-percentage> && hanging? && each-line?\"},{name:\"text-justify\",desc:\"Selects the justification algorithm used when 'text-align' is set to 'justify'. The property applies to block containers, but the UA may (but is not required to) also support it on inline elements.\",browsers:\"E14,FF55,C,IE11,O\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The UA determines the justification algorithm to follow, based on a balance between performance and adequate presentation quality.\"},{name:\"distribute\",desc:\"Justification primarily changes spacing both at word separators and at grapheme cluster boundaries in all scripts except those in the connected and cursive groups. This value is sometimes used in e.g. Japanese, often with the 'text-align-last' property.\"},{name:\"distribute-all-lines\"},{name:\"inter-cluster\"},{name:\"inter-ideograph\"},{name:\"inter-word\"},{name:\"kashida\"},{name:\"newspaper\"}],syntax:\"auto | inter-character | inter-word | none\"},{name:\"text-orientation\",desc:\"Specifies the orientation of text within a line.\",browsers:\"FF41,C48,O\",restriction:\"enum\",values:[{name:\"sideways\",browsers:\"C25,O15,S6.1\"},{name:\"sideways-right\",browsers:\"C25,O15,S6.1\"},{name:\"upright\"}],syntax:\"mixed | upright | sideways\"},{name:\"text-overflow\",desc:\"Text can overflow for example when it is prevented from wrapping.\",browsers:\"all\",restriction:\"enum, string\",values:[{name:\"clip\"},{name:\"ellipsis\"}],syntax:\"[ clip | ellipsis | <string> ]{1,2}\"},{name:\"text-rendering\",desc:\"The creator of SVG content might want to provide a hint to the implementation about what tradeoffs to make as it renders text. The ‘text-rendering’ property provides these hints.\",browsers:\"FF3,S5,C4,O15\",restriction:\"enum\",values:[{name:\"auto\"},{name:\"geometricPrecision\",desc:\"Indicates that the user agent shall emphasize geometric precision over legibility and rendering speed.\"},{name:\"optimizeLegibility\"},{name:\"optimizeSpeed\",desc:\"Indicates that the user agent shall emphasize rendering speed over legibility and geometric precision.\"}],syntax:\"auto | optimizeSpeed | optimizeLegibility | geometricPrecision\"},{name:\"text-shadow\",desc:\"Enables shadow effects to be applied to the text of the element.\",browsers:\"all\",restriction:\"length, color\",values:[],syntax:\"none | <shadow-t>#\"},{name:\"text-transform\",desc:\"Controls capitalization effects of an element’s text.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"capitalize\"},{name:\"lowercase\"},{name:\"none\",desc:\"No effects.\"},{name:\"uppercase\"}],syntax:\"none | capitalize | uppercase | lowercase | full-width\"},{name:\"text-underline-position\",desc:\"Sets the position of an underline specified on the same element: it does not affect underlines specified by ancestor elements. This property is typically used in vertical writing contexts such as in Japanese documents where it often desired to have the underline appear 'over' (to the right of) the affected run of text\",browsers:\"E12,C33,IE6\",restriction:\"enum\",values:[{name:\"above\"},{name:\"auto\",desc:\"The user agent may use any algorithm to determine the underline’s position. In horizontal line layout, the underline should be aligned as for alphabetic. In vertical line layout, if the language is set to Japanese or Korean, the underline should be aligned as for over.\"},{name:\"below\",desc:\"The underline is aligned with the under edge of the element’s content box.\"}],syntax:\"auto | [ under || [ left | right ] ]\"},{name:\"top\",desc:\"Specifies how far an absolutely positioned box's top margin edge is offset below the top edge of the box's 'containing block'.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"For non-replaced elements, the effect of this value depends on which of related properties have the value 'auto' as well\"}],syntax:\"<length> | <percentage> | auto\"},{name:\"touch-action\",desc:\"Determines whether touch input may trigger default behavior supplied by user agent.\",browsers:\"E12,FF52,C36,IE11,O23\",restriction:\"enum\",values:[{name:\"auto\",desc:\"The user agent may determine any permitted touch behaviors for touches that begin on the element.\"},{name:\"cross-slide-x\",browsers:\"E,IE11\"},{name:\"cross-slide-y\",browsers:\"E,IE11\"},{name:\"double-tap-zoom\",browsers:\"E,IE11\"},{name:\"manipulation\",desc:\"The user agent may consider touches that begin on the element only for the purposes of scrolling and continuous zooming.\"},{name:\"none\",desc:\"Touches that begin on the element must not trigger default touch behaviors.\"},{name:\"pan-x\",desc:\"The user agent may consider touches that begin on the element only for the purposes of horizontally scrolling the element’s nearest ancestor with horizontally scrollable content.\"},{name:\"pan-y\",desc:\"The user agent may consider touches that begin on the element only for the purposes of vertically scrolling the element’s nearest ancestor with vertically scrollable content.\"},{name:\"pinch-zoom\",browsers:\"E,IE11\"}],syntax:\"auto | none | [ [ pan-x | pan-left | pan-right ] || [ pan-y | pan-up | pan-down ] || pinch-zoom ] | manipulation\"},{name:\"transform\",desc:\"A two-dimensional transformation is applied to an element through the 'transform' property. This property contains a list of transform functions similar to those allowed by SVG.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"matrix()\"},{name:\"matrix3d()\"},{name:\"none\"},{name:\"perspective()\"},{name:\"rotate()\"},{name:\"rotate3d()\"},{name:\"rotateX('angle')\"},{name:\"rotateY('angle')\"},{name:\"rotateZ('angle')\"},{name:\"scale()\"},{name:\"scale3d()\"},{name:\"scaleX()\"},{name:\"scaleY()\"},{name:\"scaleZ()\"},{name:\"skew()\"},{name:\"skewX()\"},{name:\"skewY()\"},{name:\"translate()\"},{name:\"translate3d()\"},{name:\"translateX()\"},{name:\"translateY()\"},{name:\"translateZ()\"}],syntax:\"none | <transform-list>\"},{name:\"transform-origin\",desc:\"Establishes the origin of transformation for an element.\",browsers:\"all\",restriction:\"position, length, percentage\",syntax:\"[ <length-percentage> | left | center | right | top | bottom ] | [ [ <length-percentage> | left | center | right ] && [ <length-percentage> | top | center | bottom ] ] <length>?\"},{name:\"transform-style\",desc:\"Defines how nested elements are rendered in 3D space.\",browsers:\"E12,FF16,S,C12,O15\",restriction:\"enum\",values:[{name:\"flat\"},{name:\"preserve-3d\",browsers:\"E,C36,FF16,O23,S9\"}],syntax:\"flat | preserve-3d\"},{name:\"transition\",desc:\"Shorthand property combines four of the transition properties into a single property.\",browsers:\"all\",restriction:\"time, property, timing-function, enum\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}],syntax:\"<single-transition>#\"},{name:\"transition-delay\",desc:\"Defines when the transition will start. It allows a transition to begin execution some period of time from when it is applied.\",browsers:\"all\",restriction:\"time\",syntax:\"<time>#\"},{name:\"transition-duration\",desc:\"Specifies how long the transition from the old value to the new value should take.\",browsers:\"all\",restriction:\"time\",syntax:\"<time>#\"},{name:\"transition-property\",desc:\"Specifies the name of the CSS property to which the transition is applied.\",browsers:\"all\",restriction:\"property\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}],syntax:\"none | <single-transition-property>#\"},{name:\"transition-timing-function\",desc:\"Describes how the intermediate values used during a transition will be calculated.\",browsers:\"all\",restriction:\"timing-function\",syntax:\"<single-transition-timing-function>#\"},{name:\"unicode-bidi\",desc:\"The level of embedding with respect to the bidirectional algorithm.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"bidi-override\"},{name:\"embed\"},{name:\"isolate\",desc:\"The contents of the element are considered to be inside a separate, independent paragraph.\",browsers:\"C,FF10,O15,S5.1\"},{name:\"isolate-override\",browsers:\"C,FF17,O15,S6.1\"},{name:\"normal\",desc:\"The element does not open an additional level of embedding with respect to the bidirectional algorithm. For inline-level elements, implicit reordering works across element boundaries.\"},{name:\"plaintext\",browsers:\"C,FF10,O15,S6\"}],syntax:\"normal | embed | isolate | bidi-override | isolate-override | plaintext\"},{name:\"unicode-range\",desc:\"@font-face descriptor. Defines the set of Unicode codepoints that may be supported by the font face for which it is declared.\",restriction:\"unicode-range\",values:[{name:\"U+26\"},{name:\"U+20-24F, U+2B0-2FF, U+370-4FF, U+1E00-1EFF, U+2000-20CF, U+2100-23FF, U+2500-26FF, U+E000-F8FF, U+FB00–FB4F\"},{name:\"U+20-17F, U+2B0-2FF, U+2000-206F, U+20A0-20CF, U+2100-21FF, U+2600-26FF\"},{name:\"U+20-2FF, U+370-4FF, U+1E00-20CF, U+2100-23FF, U+2500-26FF, U+FB00-FB4F, U+FFF0-FFFD\"},{name:\"U+20-4FF, U+530-58F, U+10D0-10FF, U+1E00-23FF, U+2440-245F, U+2500-26FF, U+FB00-FB4F, U+FE20-FE2F, U+FFF0-FFFD\"},{name:\"U+00-7F\"},{name:\"U+80-FF\"},{name:\"U+100-17F\"},{name:\"U+180-24F\"},{name:\"U+1E00-1EFF\"},{name:\"U+250-2AF\"},{name:\"U+370-3FF\"},{name:\"U+1F00-1FFF\"},{name:\"U+400-4FF\"},{name:\"U+500-52F\"},{name:\"U+00-52F, U+1E00-1FFF, U+2200–22FF\"},{name:\"U+530–58F\"},{name:\"U+590–5FF\"},{name:\"U+600–6FF\"},{name:\"U+750–77F\"},{name:\"U+8A0–8FF\"},{name:\"U+700–74F\"},{name:\"U+900–97F\"},{name:\"U+980–9FF\"},{name:\"U+A00–A7F\"},{name:\"U+A80–AFF\"},{name:\"U+B00–B7F\"},{name:\"U+B80–BFF\"},{name:\"U+C00–C7F\"},{name:\"U+C80–CFF\"},{name:\"U+D00–D7F\"},{name:\"U+D80–DFF\"},{name:\"U+118A0–118FF\"},{name:\"U+E00–E7F\"},{name:\"U+1A20–1AAF\"},{name:\"U+AA80–AADF\"},{name:\"U+E80–EFF\"},{name:\"U+F00–FFF\"},{name:\"U+1000–109F\"},{name:\"U+10A0–10FF\"},{name:\"U+1200–137F\"},{name:\"U+1380–139F\"},{name:\"U+2D80–2DDF\"},{name:\"U+AB00–AB2F\"},{name:\"U+1780–17FF\"},{name:\"U+1800–18AF\"},{name:\"U+1B80–1BBF\"},{name:\"U+1CC0–1CCF\"},{name:\"U+4E00–9FD5\"},{name:\"U+3400–4DB5\"},{name:\"U+2F00–2FDF\"},{name:\"U+2E80–2EFF\"},{name:\"U+1100–11FF\"},{name:\"U+AC00–D7AF\"},{name:\"U+3040–309F\"},{name:\"U+30A0–30FF\"},{name:\"U+A5, U+4E00-9FFF, U+30??, U+FF00-FF9F\"},{name:\"U+A4D0–A4FF\"},{name:\"U+A000–A48F\"},{name:\"U+A490–A4CF\"},{name:\"U+2000-206F\"},{name:\"U+3000–303F\"},{name:\"U+2070–209F\"},{name:\"U+20A0–20CF\"},{name:\"U+2100–214F\"},{name:\"U+2150–218F\"},{name:\"U+2190–21FF\"},{name:\"U+2200–22FF\"},{name:\"U+2300–23FF\"},{name:\"U+E000-F8FF\"},{name:\"U+FB00–FB4F\"},{name:\"U+FB50–FDFF\"},{name:\"U+1F600–1F64F\"},{name:\"U+2600–26FF\"},{name:\"U+1F300–1F5FF\"},{name:\"U+1F900–1F9FF\"},{name:\"U+1F680–1F6FF\"}],syntax:\"<unicode-range>#\"},{name:\"user-select\",desc:\"Controls the appearance of selection.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"all\",desc:\"The content of the element must be selected atomically\"},{name:\"auto\"},{name:\"contain\",desc:\"UAs must not allow a selection which is started in this element to be extended outside of this element.\"},{name:\"none\",desc:\"The UA must not allow selections to be started in this element.\"},{name:\"text\",desc:\"The element imposes no constraint on the selection.\"}],status:\"n\",syntax:\"auto | text | none | contain | all\"},{name:\"vertical-align\",desc:\"Affects the vertical positioning of the inline boxes generated by an inline-level element inside a line box.\",browsers:\"all\",restriction:\"percentage, length\",values:[{name:\"auto\",desc:\"Align the dominant baseline of the parent box with the equivalent, or heuristically reconstructed, baseline of the element inline box.\"},{name:\"baseline\",desc:\"Align the 'alphabetic' baseline of the element with the 'alphabetic' baseline of the parent element.\"},{name:\"bottom\",desc:\"Align the after edge of the extended inline box with the after-edge of the line box.\"},{name:\"middle\",desc:\"Align the 'middle' baseline of the inline element with the middle baseline of the parent.\"},{name:\"sub\",desc:\"Lower the baseline of the box to the proper position for subscripts of the parent's box. (This value has no effect on the font size of the element's text.)\"},{name:\"super\",desc:\"Raise the baseline of the box to the proper position for superscripts of the parent's box. (This value has no effect on the font size of the element's text.)\"},{name:\"text-bottom\"},{name:\"text-top\"},{name:\"top\",desc:\"Align the before edge of the extended inline box with the before-edge of the line box.\"},{name:\"-webkit-baseline-middle\",browsers:\"C,S1\"}],syntax:\"baseline | sub | super | text-top | text-bottom | middle | top | bottom | <percentage> | <length>\"},{name:\"visibility\",desc:\"Specifies whether the boxes generated by an element are rendered. Invisible boxes still affect layout (set the ‘display’ property to ‘none’ to suppress box generation altogether).\",browsers:\"all\",restriction:\"enum\",values:[{name:\"collapse\",desc:\"Table-specific. If used on elements other than rows, row groups, columns, or column groups, 'collapse' has the same meaning as 'hidden'.\"},{name:\"hidden\",desc:\"The generated box is invisible (fully transparent, nothing is drawn), but still affects layout.\"},{name:\"visible\",desc:\"The generated box is visible.\"}],syntax:\"visible | hidden | collapse\"},{name:\"-webkit-animation\",desc:\"Shorthand property combines six of the animation properties into a single property.\",browsers:\"C,S5\",restriction:\"time, enum, timing-function, identifier, number\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"backwards\"},{name:\"both\",desc:\"Both forwards and backwards fill modes are applied.\"},{name:\"forwards\"},{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"},{name:\"none\",desc:\"No animation is performed\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}]},{name:\"-webkit-animation-delay\",desc:\"Defines when the animation will start.\",browsers:\"C,S5\",restriction:\"time\"},{name:\"-webkit-animation-direction\",desc:\"Defines whether or not the animation should play in reverse on alternate cycles.\",browsers:\"C,S5\",restriction:\"enum\",values:[{name:\"alternate\"},{name:\"alternate-reverse\"},{name:\"normal\",desc:\"Normal playback.\"},{name:\"reverse\",desc:\"All iterations of the animation are played in the reverse direction from the way they were specified.\"}]},{name:\"-webkit-animation-duration\",desc:\"Defines the length of time that an animation takes to complete one cycle.\",browsers:\"C,S5\",restriction:\"time\"},{name:\"-webkit-animation-fill-mode\",desc:\"Defines what values are applied by the animation outside the time it is executing.\",browsers:\"C,S5\",restriction:\"enum\",values:[{name:\"backwards\"},{name:\"both\",desc:\"Both forwards and backwards fill modes are applied.\"},{name:\"forwards\"},{name:\"none\",desc:\"There is no change to the property value between the time the animation is applied and the time the animation begins playing or after the animation completes.\"}]},{name:\"-webkit-animation-iteration-count\",desc:\"Defines the number of times an animation cycle is played. The default value is one, meaning the animation will play from beginning to end once.\",browsers:\"C,S5\",restriction:\"number, enum\",values:[{name:\"infinite\",desc:\"Causes the animation to repeat forever.\"}]},{name:\"-webkit-animation-name\",desc:\"Defines a list of animations that apply. Each name is used to select the keyframe at-rule that provides the property values for the animation.\",browsers:\"C,S5\",restriction:\"identifier, enum\",values:[{name:\"none\",desc:\"No animation is performed\"}]},{name:\"-webkit-animation-play-state\",desc:\"Defines whether the animation is running or paused.\",browsers:\"C,S5\",restriction:\"enum\",values:[{name:\"paused\"},{name:\"running\"}]},{name:\"-webkit-animation-timing-function\",desc:\"Describes how the animation will progress over one cycle of its duration. See the 'transition-timing-function'.\",browsers:\"C,S5\",restriction:\"timing-function\"},{name:\"-webkit-appearance\",desc:\"Changes the appearance of buttons and other controls to resemble native controls.\",browsers:\"E12,S3,C1,O15\",restriction:\"enum\",values:[{name:\"button\"},{name:\"button-bevel\"},{name:\"caps-lock-indicator\"},{name:\"caret\"},{name:\"checkbox\"},{name:\"default-button\"},{name:\"listbox\"},{name:\"listitem\"},{name:\"media-fullscreen-button\"},{name:\"media-mute-button\"},{name:\"media-play-button\"},{name:\"media-seek-back-button\"},{name:\"media-seek-forward-button\"},{name:\"media-slider\"},{name:\"media-sliderthumb\"},{name:\"menulist\"},{name:\"menulist-button\"},{name:\"menulist-text\"},{name:\"menulist-textfield\"},{name:\"none\"},{name:\"push-button\"},{name:\"radio\"},{name:\"scrollbarbutton-down\"},{name:\"scrollbarbutton-left\"},{name:\"scrollbarbutton-right\"},{name:\"scrollbarbutton-up\"},{name:\"scrollbargripper-horizontal\"},{name:\"scrollbargripper-vertical\"},{name:\"scrollbarthumb-horizontal\"},{name:\"scrollbarthumb-vertical\"},{name:\"scrollbartrack-horizontal\"},{name:\"scrollbartrack-vertical\"},{name:\"searchfield\"},{name:\"searchfield-cancel-button\"},{name:\"searchfield-decoration\"},{name:\"searchfield-results-button\"},{name:\"searchfield-results-decoration\"},{name:\"slider-horizontal\"},{name:\"sliderthumb-horizontal\"},{name:\"sliderthumb-vertical\"},{name:\"slider-vertical\"},{name:\"square-button\"},{name:\"textarea\"},{name:\"textfield\"}],status:\"n\",syntax:\"none | button | button-bevel | caret | checkbox | default-button | inner-spin-button | listbox | listitem | media-controls-background | media-controls-fullscreen-background | media-current-time-display | media-enter-fullscreen-button | media-exit-fullscreen-button | media-fullscreen-button | media-mute-button | media-overlay-play-button | media-play-button | media-seek-back-button | media-seek-forward-button | media-slider | media-sliderthumb | media-time-remaining-display | media-toggle-closed-captions-button | media-volume-slider | media-volume-slider-container | media-volume-sliderthumb | menulist | menulist-button | menulist-text | menulist-textfield | meter | progress-bar | progress-bar-value | push-button | radio | searchfield | searchfield-cancel-button | searchfield-decoration | searchfield-results-button | searchfield-results-decoration | slider-horizontal | slider-vertical | sliderthumb-horizontal | sliderthumb-vertical | square-button | textarea | textfield\"},{name:\"-webkit-backdrop-filter\",desc:\"Applies a filter effect where the first filter in the list takes the element's background image as the input image.\",browsers:\"S9\",restriction:\"enum, url\",values:[{name:\"none\",desc:\"No filter effects are applied.\"},{name:\"blur()\"},{name:\"brightness()\"},{name:\"contrast()\"},{name:\"drop-shadow()\"},{name:\"grayscale()\"},{name:\"hue-rotate()\"},{name:\"invert()\"},{name:\"opacity()\"},{name:\"saturate()\"},{name:\"sepia()\"},{name:\"url()\",desc:\"A filter reference to a <filter> element.\"}]},{name:\"-webkit-backface-visibility\",desc:\"Determines whether or not the 'back' side of a transformed element is visible when facing the viewer. With an identity transform, the front side of an element faces the viewer.\",browsers:\"C,S5\",restriction:\"enum\",values:[{name:\"hidden\"},{name:\"visible\"}]},{name:\"-webkit-background-clip\",desc:\"Determines the background painting area.\",browsers:\"C,S3\",restriction:\"box\"},{name:\"-webkit-background-composite\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"border\"},{name:\"padding\"}]},{name:\"-webkit-background-origin\",desc:\"For elements rendered as a single box, specifies the background positioning area. For elements rendered as multiple boxes (e.g., inline boxes on several lines, boxes on several pages) specifies which boxes 'box-decoration-break' operates on to determine the background positioning area(s).\",browsers:\"C,S3\",restriction:\"box\"},{name:\"-webkit-border-image\",desc:\"Shorthand property for setting 'border-image-source', 'border-image-slice', 'border-image-width', 'border-image-outset' and 'border-image-repeat'. Omitted values are set to their initial values.\",browsers:\"C,S5\",restriction:\"length, percentage, number, url, enum\",values:[{name:\"auto\",desc:\"If 'auto' is specified then the border image width is the intrinsic width or height (whichever is applicable) of the corresponding image slice. If the image does not have the required intrinsic dimension then the corresponding border-width is used instead.\"},{name:\"fill\",desc:\"Causes the middle part of the border-image to be preserved.\"},{name:\"none\"},{name:\"repeat\"},{name:\"round\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the image is rescaled so that it does.\"},{name:\"space\",desc:\"The image is tiled (repeated) to fill the area. If it does not fill the area with a whole number of tiles, the extra space is distributed around the tiles.\"},{name:\"stretch\",desc:\"The image is stretched to fill the area.\"},{name:\"url()\"}]},{name:\"-webkit-box-align\",desc:\"Specifies the alignment of nested elements within an outer flexible box element.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"baseline\",desc:\"If this box orientation is inline-axis or horizontal, all children are placed with their baselines aligned, and extra space placed before or after as necessary. For block flows, the baseline of the first non-empty line box located within the element is used. For tables, the baseline of the first cell is used.\"},{name:\"center\",desc:\"Any extra space is divided evenly, with half placed above the child and the other half placed after the child.\"},{name:\"end\",desc:\"For normal direction boxes, the bottom edge of each child is placed along the bottom of the box. Extra space is placed above the element. For reverse direction boxes, the top edge of each child is placed along the top of the box. Extra space is placed below the element.\"},{name:\"start\",desc:\"For normal direction boxes, the top edge of each child is placed along the top of the box. Extra space is placed below the element. For reverse direction boxes, the bottom edge of each child is placed along the bottom of the box. Extra space is placed above the element.\"},{name:\"stretch\",desc:\"The height of each child is adjusted to that of the containing block.\"}]},{name:\"-webkit-box-direction\",desc:\"In webkit applications, -webkit-box-direction specifies whether a box lays out its contents normally (from the top or left edge), or in reverse (from the bottom or right edge).\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"normal\",desc:\"A box with a computed value of horizontal for box-orient displays its children from left to right. A box with a computed value of vertical displays its children from top to bottom.\"},{name:\"reverse\",desc:\"A box with a computed value of horizontal for box-orient displays its children from right to left. A box with a computed value of vertical displays its children from bottom to top.\"}]},{name:\"-webkit-box-flex\",desc:\"Specifies an element's flexibility.\",browsers:\"C,S3\",restriction:\"number\"},{name:\"-webkit-box-flex-group\",desc:\"Flexible elements can be assigned to flex groups using the 'box-flex-group' property.\",browsers:\"C,S3\",restriction:\"integer\"},{name:\"-webkit-box-ordinal-group\",desc:\"Indicates the ordinal group the element belongs to. Elements with a lower ordinal group are displayed before those with a higher ordinal group.\",browsers:\"C,S3\",restriction:\"integer\"},{name:\"-webkit-box-orient\",desc:\"In webkit applications, -webkit-box-orient specifies whether a box lays out its contents horizontally or vertically.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"block-axis\"},{name:\"horizontal\",desc:\"The box displays its children from left to right in a horizontal line.\"},{name:\"inline-axis\"},{name:\"vertical\",desc:\"The box displays its children from stacked from top to bottom vertically.\"}]},{name:\"-webkit-box-pack\",desc:\"Specifies alignment of child elements within the current element in the direction of orientation.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"center\",desc:\"The extra space is divided evenly, with half placed before the first child and the other half placed after the last child.\"},{name:\"end\",desc:\"For normal direction boxes, the right edge of the last child is placed at the right side, with all extra space placed before the first child. For reverse direction boxes, the left edge of the first child is placed at the left side, with all extra space placed after the last child.\"},{name:\"justify\",desc:\"The space is divided evenly in-between each child, with none of the extra space placed before the first child or after the last child. If there is only one child, treat the pack value as if it were start.\"},{name:\"start\",desc:\"For normal direction boxes, the left edge of the first child is placed at the left side, with all extra space placed after the last child. For reverse direction boxes, the right edge of the last child is placed at the right side, with all extra space placed before the first child.\"}]},{name:\"-webkit-box-reflect\",desc:\"Defines a reflection of a border box.\",browsers:\"C,S4\",values:[{name:\"above\",desc:\"The reflection appears above the border box.\"},{name:\"below\",desc:\"The reflection appears below the border box.\"},{name:\"left\",desc:\"The reflection appears to the left of the border box.\"},{name:\"right\",desc:\"The reflection appears to the right of the border box.\"}],status:\"n\",syntax:\"[ above | below | right | left ]? <length>? <image>?\"},{name:\"-webkit-box-sizing\",desc:\"Box Model addition in CSS3.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"border-box\"},{name:\"content-box\"}]},{name:\"-webkit-break-after\",desc:\"Describes the page/column break behavior before the generated box.\",browsers:\"S7\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break before/after the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page/column break before/after the generated box.\"},{name:\"avoid\",desc:\"Avoid a page/column break before/after the generated box.\"},{name:\"avoid-column\",desc:\"Avoid a column break before/after the generated box.\"},{name:\"avoid-page\",desc:\"Avoid a page break before/after the generated box.\"},{name:\"avoid-region\"},{name:\"column\",desc:\"Always force a column break before/after the generated box.\"},{name:\"left\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a left page.\"},{name:\"page\",desc:\"Always force a page break before/after the generated box.\"},{name:\"region\"},{name:\"right\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a right page.\"}]},{name:\"-webkit-break-before\",desc:\"Describes the page/column break behavior before the generated box.\",browsers:\"S7\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break before/after the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page/column break before/after the generated box.\"},{name:\"avoid\",desc:\"Avoid a page/column break before/after the generated box.\"},{name:\"avoid-column\",desc:\"Avoid a column break before/after the generated box.\"},{name:\"avoid-page\",desc:\"Avoid a page break before/after the generated box.\"},{name:\"avoid-region\"},{name:\"column\",desc:\"Always force a column break before/after the generated box.\"},{name:\"left\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a left page.\"},{name:\"page\",desc:\"Always force a page break before/after the generated box.\"},{name:\"region\"},{name:\"right\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a right page.\"}]},{name:\"-webkit-break-inside\",desc:\"Describes the page/column break behavior inside the generated box.\",browsers:\"S7\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Neither force nor forbid a page/column break inside the generated box.\"},{name:\"avoid\",desc:\"Avoid a page/column break inside the generated box.\"},{name:\"avoid-column\",desc:\"Avoid a column break inside the generated box.\"},{name:\"avoid-page\",desc:\"Avoid a page break inside the generated box.\"},{name:\"avoid-region\"}]},{name:\"-webkit-column-break-after\",desc:\"Describes the page/column break behavior before the generated box.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break before/after the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page/column break before/after the generated box.\"},{name:\"avoid\",desc:\"Avoid a page/column break before/after the generated box.\"},{name:\"avoid-column\",desc:\"Avoid a column break before/after the generated box.\"},{name:\"avoid-page\",desc:\"Avoid a page break before/after the generated box.\"},{name:\"avoid-region\"},{name:\"column\",desc:\"Always force a column break before/after the generated box.\"},{name:\"left\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a left page.\"},{name:\"page\",desc:\"Always force a page break before/after the generated box.\"},{name:\"region\"},{name:\"right\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a right page.\"}]},{name:\"-webkit-column-break-before\",desc:\"Describes the page/column break behavior before the generated box.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"always\",desc:\"Always force a page break before/after the generated box.\"},{name:\"auto\",desc:\"Neither force nor forbid a page/column break before/after the generated box.\"},{name:\"avoid\",desc:\"Avoid a page/column break before/after the generated box.\"},{name:\"avoid-column\",desc:\"Avoid a column break before/after the generated box.\"},{name:\"avoid-page\",desc:\"Avoid a page break before/after the generated box.\"},{name:\"avoid-region\"},{name:\"column\",desc:\"Always force a column break before/after the generated box.\"},{name:\"left\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a left page.\"},{name:\"page\",desc:\"Always force a page break before/after the generated box.\"},{name:\"region\"},{name:\"right\",desc:\"Force one or two page breaks before/after the generated box so that the next page is formatted as a right page.\"}]},{name:\"-webkit-column-break-inside\",desc:\"Describes the page/column break behavior inside the generated box.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Neither force nor forbid a page/column break inside the generated box.\"},{name:\"avoid\",desc:\"Avoid a page/column break inside the generated box.\"},{name:\"avoid-column\",desc:\"Avoid a column break inside the generated box.\"},{name:\"avoid-page\",desc:\"Avoid a page break inside the generated box.\"},{name:\"avoid-region\"}]},{name:\"-webkit-column-count\",desc:\"Describes the optimal number of columns into which the content of the element will be flowed.\",browsers:\"C,S3\",restriction:\"integer\",values:[{name:\"auto\",desc:\"Determines the number of columns by the 'column-width' property and the element width.\"}]},{name:\"-webkit-column-gap\",desc:\"Sets the gap between columns. If there is a column rule between columns, it will appear in the middle of the gap.\",browsers:\"C,S3\",restriction:\"length\",values:[{name:\"normal\",desc:\"User agent specific and typically equivalent to 1em.\"}]},{name:\"-webkit-column-rule\",desc:\"This property is a shorthand for setting 'column-rule-width', 'column-rule-style', and 'column-rule-color' at the same place in the style sheet. Omitted values are set to their initial values.\",browsers:\"C,S3\",restriction:\"length, line-width, line-style, color\"},{name:\"-webkit-column-rule-color\",desc:\"Sets the color of the column rule\",browsers:\"C,S3\",restriction:\"color\"},{name:\"-webkit-column-rule-style\",desc:\"Sets the style of the rule between columns of an element.\",browsers:\"C,S3\",restriction:\"line-style\"},{name:\"-webkit-column-rule-width\",desc:\"Sets the width of the rule between columns. Negative values are not allowed.\",browsers:\"C,S3\",restriction:\"length, line-width\"},{name:\"-webkit-columns\",desc:\"A shorthand property which sets both 'column-width' and 'column-count'.\",browsers:\"C,S3\",restriction:\"length, integer\",values:[{name:\"auto\",desc:\"The width depends on the values of other properties.\"}]},{name:\"-webkit-column-span\",desc:\"Describes the page/column break behavior after the generated box.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"all\",desc:\"The element spans across all columns. Content in the normal flow that appears before the element is automatically balanced across all columns before the element appear.\"},{name:\"none\",desc:\"The element does not span multiple columns.\"}]},{name:\"-webkit-column-width\",desc:\"This property describes the width of columns in multicol elements.\",browsers:\"C,S3\",restriction:\"length\",values:[{name:\"auto\",desc:\"The width depends on the values of other properties.\"}]},{name:\"-webkit-filter\",desc:\"Processes an element’s rendering before it is displayed in the document, by applying one or more filter effects.\",browsers:\"C18,O15,S6\",restriction:\"enum, url\",values:[{name:\"none\",desc:\"No filter effects are applied.\"},{name:\"blur()\"},{name:\"brightness()\"},{name:\"contrast()\"},{name:\"drop-shadow()\"},{name:\"grayscale()\"},{name:\"hue-rotate()\"},{name:\"invert()\"},{name:\"opacity()\"},{name:\"saturate()\"},{name:\"sepia()\"},{name:\"url()\",desc:\"A filter reference to a <filter> element.\"}]},{name:\"-webkit-flow-from\",desc:\"Makes a block container a region and associates it with a named flow.\",browsers:\"S6.1\",restriction:\"identifier\",values:[{name:\"none\",desc:\"The block container is not a CSS Region.\"}]},{name:\"-webkit-flow-into\",desc:\"Places an element or its contents into a named flow.\",browsers:\"S6.1\",restriction:\"identifier\",values:[{name:\"none\",desc:\"The element is not moved to a named flow and normal CSS processing takes place.\"}]},{name:\"-webkit-font-feature-settings\",desc:\"This property provides low-level control over OpenType font features. It is intended as a way of providing access to font features that are not widely used but are needed for a particular use case.\",browsers:\"C16\",restriction:\"string, integer\",values:[{name:'\"c2cs\"'},{name:'\"dlig\"'},{name:'\"kern\"'},{name:'\"liga\"'},{name:'\"lnum\"'},{name:'\"onum\"'},{name:'\"smcp\"'},{name:'\"swsh\"'},{name:'\"tnum\"'},{name:\"normal\",desc:\"No change in glyph substitution or positioning occurs.\"},{name:\"off\"},{name:\"on\"}]},{name:\"-webkit-hyphens\",desc:\"Controls whether hyphenation is allowed to create more break opportunities within a line of text.\",browsers:\"S5.1\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Conditional hyphenation characters inside a word, if present, take priority over automatic resources when determining hyphenation points within the word.\"},{name:\"manual\"},{name:\"none\",desc:\"Words are not broken at line breaks, even if characters inside the word suggest line break points.\"}]},{name:\"-webkit-line-break\",desc:\"Specifies line-breaking rules for CJK (Chinese, Japanese, and Korean) text.\",browsers:\"C,S3\",values:[{name:\"after-white-space\"},{name:\"normal\"}]},{name:\"-webkit-margin-bottom-collapse\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"collapse\"},{name:\"discard\"},{name:\"separate\"}]},{name:\"-webkit-margin-collapse\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"collapse\"},{name:\"discard\"},{name:\"separate\"}]},{name:\"-webkit-margin-start\",browsers:\"C,S3\",restriction:\"percentage, length\",values:[{name:\"auto\"}]},{name:\"-webkit-margin-top-collapse\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"collapse\"},{name:\"discard\"},{name:\"separate\"}]},{name:\"-webkit-mask-clip\",desc:\"Determines the mask painting area, which determines the area that is affected by the mask.\",browsers:\"C,O15,S4\",restriction:\"box\",status:\"n\",syntax:\"[ <box> | border | padding | content | text ]#\"},{name:\"-webkit-mask-image\",desc:\"Sets the mask layer image of an element.\",browsers:\"C,O15,S4\",restriction:\"url, image, enum\",values:[{name:\"none\",desc:\"Counts as a transparent black image layer.\"},{name:\"url()\",desc:\"Reference to a <mask element or to a CSS image.\"}],status:\"n\",syntax:\"<mask-reference>#\"},{name:\"-webkit-mask-origin\",desc:\"Specifies the mask positioning area.\",browsers:\"C,O15,S4\",restriction:\"box\",status:\"n\",syntax:\"[ <box> | border | padding | content ]#\"},{name:\"-webkit-mask-repeat\",desc:\"Specifies how mask layer images are tiled after they have been sized and positioned.\",browsers:\"C,O15,S4\",restriction:\"repeat\",status:\"n\",syntax:\"<repeat-style>#\"},{name:\"-webkit-mask-size\",desc:\"Specifies the size of the mask layer images.\",browsers:\"C,O15,S4\",restriction:\"length, percentage, enum\",values:[{name:\"auto\",desc:\"Resolved by using the image’s intrinsic ratio and the size of the other dimension, or failing that, using the image’s intrinsic size, or failing that, treating it as 100%.\"},{name:\"contain\",desc:\"Scale the image, while preserving its intrinsic aspect ratio (if any), to the largest size such that both its width and its height can fit inside the background positioning area.\"},{name:\"cover\",desc:\"Scale the image, while preserving its intrinsic aspect ratio (if any), to the smallest size such that both its width and its height can completely cover the background positioning area.\"}],status:\"n\",syntax:\"<bg-size>#\"},{name:\"-webkit-nbsp-mode\",desc:\"Defines the behavior of nonbreaking spaces within text.\",browsers:\"C,S3\",values:[{name:\"normal\"},{name:\"space\"}]},{name:\"-webkit-overflow-scrolling\",desc:\"Specifies whether to use native-style scrolling in an overflow:scroll element.\",browsers:\"C,S5\",values:[{name:\"auto\"},{name:\"touch\"}],status:\"n\",syntax:\"auto | touch\"},{name:\"-webkit-padding-start\",browsers:\"C,S3\",restriction:\"percentage, length\"},{name:\"-webkit-perspective\",desc:\"Applies the same transform as the perspective(<number>) transform function, except that it applies only to the positioned or transformed children of the element, not to the transform on the element itself.\",browsers:\"C,S4\",restriction:\"length\",values:[{name:\"none\",desc:\"No perspective transform is applied.\"}]},{name:\"-webkit-perspective-origin\",desc:\"Establishes the origin for the perspective property. It effectively sets the X and Y position at which the viewer appears to be looking at the children of the element.\",browsers:\"C,S4\",restriction:\"position, percentage, length\"},{name:\"-webkit-region-fragment\",desc:\"The 'region-fragment' property controls the behavior of the last region associated with a named flow.\",browsers:\"S7\",restriction:\"enum\",values:[{name:\"auto\",desc:\"Content flows as it would in a regular content box.\"},{name:\"break\"}]},{name:\"-webkit-tap-highlight-color\",browsers:\"E,C,S3.1\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-webkit-text-fill-color\",browsers:\"E12,FF49,S,C,O\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-webkit-text-size-adjust\",desc:\"Specifies a size adjustment for displaying text content in mobile browsers.\",browsers:\"E,C,S3\",restriction:\"percentage\",values:[{name:\"auto\",desc:\"Renderers must use the default size adjustment when displaying on a small device.\"},{name:\"none\",desc:\"Renderers must not do size adjustment when displaying on a small device.\"}]},{name:\"-webkit-text-stroke\",browsers:\"E15,FF49,S3.1,C4,O15\",restriction:\"length, line-width, color, percentage\",status:\"n\",syntax:\"<length> || <color>\"},{name:\"-webkit-text-stroke-color\",browsers:\"E15,FF49,S,C,O\",restriction:\"color\",status:\"n\",syntax:\"<color>\"},{name:\"-webkit-text-stroke-width\",browsers:\"E15,FF49,S,C,O\",restriction:\"length, line-width, percentage\",status:\"n\",syntax:\"<length>\"},{name:\"-webkit-touch-callout\",browsers:\"S4\",restriction:\"enum\",values:[{name:\"none\"}],status:\"n\",syntax:\"default | none\"},{name:\"-webkit-transform\",desc:\"A two-dimensional transformation is applied to an element through the 'transform' property. This property contains a list of transform functions similar to those allowed by SVG.\",browsers:\"C,O12,S3.1\",restriction:\"enum\",values:[{name:\"matrix()\"},{name:\"matrix3d()\"},{name:\"none\"},{name:\"perspective()\"},{name:\"rotate()\"},{name:\"rotate3d()\"},{name:\"rotateX('angle')\"},{name:\"rotateY('angle')\"},{name:\"rotateZ('angle')\"},{name:\"scale()\"},{name:\"scale3d()\"},{name:\"scaleX()\"},{name:\"scaleY()\"},{name:\"scaleZ()\"},{name:\"skew()\"},{name:\"skewX()\"},{name:\"skewY()\"},{name:\"translate()\"},{name:\"translate3d()\"},{name:\"translateX()\"},{name:\"translateY()\"},{name:\"translateZ()\"}]},{name:\"-webkit-transform-origin\",desc:\"Establishes the origin of transformation for an element.\",browsers:\"C,O15,S3.1\",restriction:\"position, length, percentage\"},{name:\"-webkit-transform-origin-x\",desc:\"The x coordinate of the origin for transforms applied to an element with respect to its border box.\",browsers:\"C,S3.1\",restriction:\"length, percentage\"},{name:\"-webkit-transform-origin-y\",desc:\"The y coordinate of the origin for transforms applied to an element with respect to its border box.\",browsers:\"C,S3.1\",restriction:\"length, percentage\"},{name:\"-webkit-transform-origin-z\",desc:\"The z coordinate of the origin for transforms applied to an element with respect to its border box.\",browsers:\"C,S4\",restriction:\"length, percentage\"},{name:\"-webkit-transform-style\",desc:\"Defines how nested elements are rendered in 3D space.\",browsers:\"C,S4\",restriction:\"enum\",values:[{name:\"flat\"}]},{name:\"-webkit-transition\",desc:\"Shorthand property combines four of the transition properties into a single property.\",browsers:\"C,O12,S5\",restriction:\"time, property, timing-function, enum\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}]},{name:\"-webkit-transition-delay\",desc:\"Defines when the transition will start. It allows a transition to begin execution some period of time from when it is applied.\",browsers:\"C,O12,S5\",restriction:\"time\"},{name:\"-webkit-transition-duration\",desc:\"Specifies how long the transition from the old value to the new value should take.\",browsers:\"C,O12,S5\",restriction:\"time\"},{name:\"-webkit-transition-property\",desc:\"Specifies the name of the CSS property to which the transition is applied.\",browsers:\"C,O12,S5\",restriction:\"property\",values:[{name:\"all\",desc:\"Every property that is able to undergo a transition will do so.\"},{name:\"none\",desc:\"No property will transition.\"}]},{name:\"-webkit-transition-timing-function\",desc:\"Describes how the intermediate values used during a transition will be calculated.\",browsers:\"C,O12,S5\",restriction:\"timing-function\"},{name:\"-webkit-user-drag\",browsers:\"S3\",restriction:\"enum\",values:[{name:\"auto\"},{name:\"element\"},{name:\"none\"}]},{name:\"-webkit-user-modify\",desc:\"Determines whether a user can edit the content of an element.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"read-only\"},{name:\"read-write\"},{name:\"read-write-plaintext-only\"}],status:\"n\",syntax:\"read-only | read-write | read-write-plaintext-only\"},{name:\"-webkit-user-select\",desc:\"Controls the appearance of selection.\",browsers:\"C,S3\",restriction:\"enum\",values:[{name:\"auto\"},{name:\"none\"},{name:\"text\"}]},{name:\"white-space\",desc:\"Shorthand property for the 'white-space-collapsing' and 'text-wrap' properties.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"normal\",desc:\"Sets 'white-space-collapsing' to 'collapse' and 'text-wrap' to 'normal'.\"},{name:\"nowrap\",desc:\"Sets 'white-space-collapsing' to 'collapse' and 'text-wrap' to 'none'.\"},{name:\"pre\"},{name:\"pre-line\"},{name:\"pre-wrap\"}],syntax:\"normal | pre | nowrap | pre-wrap | pre-line\"},{name:\"widows\",desc:\"Specifies the minimum number of line boxes of a block container that must be left in a fragment after a break.\",browsers:\"E12,C25,IE8,O9.2\",restriction:\"integer\",syntax:\"<integer>\"},{name:\"width\",desc:\"Specifies the width of the content area, padding area or border area (depending on 'box-sizing') of certain boxes.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"auto\",desc:\"The width depends on the values of other properties.\"},{name:\"fit-content\",browsers:\"C46,O33\"},{name:\"max-content\",desc:\"Use the max-content inline size or max-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"},{name:\"min-content\",desc:\"Use the min-content inline size or min-content block size, as appropriate to the writing mode.\",browsers:\"C46,O33\"}],syntax:\"<viewport-length>{1,2}\"},{name:\"will-change\",desc:\"Provides a rendering hint to the user agent, stating what kinds of changes the author expects to perform on the element.\",browsers:\"FF36,S9.1,C36,O24\",restriction:\"enum, identifier\",values:[{name:\"auto\",desc:\"Expresses no particular intent.\"},{name:\"contents\",desc:\"Indicates that the author expects to animate or change something about the element’s contents in the near future.\"},{name:\"scroll-position\"}],syntax:\"auto | <animateable-feature>#\"},{name:\"word-break\",desc:\"Specifies line break opportunities for non-CJK scripts.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"break-all\"},{name:\"keep-all\",desc:\"Block characters can no longer create implied break points.\"},{name:\"normal\",desc:\"Breaks non-CJK scripts according to their own rules.\"}],syntax:\"normal | break-all | keep-all | break-word\"},{name:\"word-spacing\",desc:\"Specifies additional spacing between “words”.\",browsers:\"all\",restriction:\"length, percentage\",values:[{name:\"normal\",desc:\"No additional spacing is applied. Computes to zero.\"}],syntax:\"normal | <length-percentage>\"},{name:\"word-wrap\",desc:\"Specifies whether the UA may break within a word to prevent overflow when an otherwise-unbreakable string is too long to fit.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"break-word\",desc:\"An otherwise unbreakable sequence of characters may be broken at an arbitrary point if there are no otherwise-acceptable break points in the line.\"},{name:\"normal\",desc:\"Lines may break only at allowed break points.\"}],syntax:\"normal | break-word\"},{name:\"writing-mode\",desc:\"This is a shorthand property for both 'direction' and 'block-progression'.\",browsers:\"all\",restriction:\"enum\",values:[{name:\"horizontal-tb\"},{name:\"sideways-lr\",browsers:\"FF43\"},{name:\"sideways-rl\",browsers:\"FF43\"},{name:\"vertical-lr\"},{name:\"vertical-rl\"}],syntax:\"horizontal-tb | vertical-rl | vertical-lr | sideways-rl | sideways-lr\"},{name:\"z-index\",desc:\"For a positioned box, the 'z-index' property specifies the stack level of the box in the current stacking context and whether the box establishes a local stacking context.\",browsers:\"all\",restriction:\"integer\",values:[{name:\"auto\",desc:\"The stack level of the generated box in the current stacking context is 0. The box does not establish a new stacking context unless it is the root element.\"}],syntax:\"auto | <integer>\"},{name:\"zoom\",desc:\"Non-standard. Specifies the magnification scale of the object. See 'transform: scale()' for a standards-based alternative.\",browsers:\"E,C,IE6,O15,S4\",restriction:\"enum, integer, number, percentage\",values:[{name:\"normal\"}],syntax:\"auto | <number> | <percentage>\"},{name:\"-ms-ime-align\",desc:\"Aligns the Input Method Editor (IME) candidate window box relative to the element on which the IME composition is active.\",restriction:\"none\",status:\"n\",syntax:\"auto | after\"},{name:\"-moz-binding\",desc:\"The -moz-binding CSS property is used by Mozilla-based applications to attach an XBL binding to a DOM element.\",restriction:\"none\",status:\"n\",syntax:\"<url> | none\"},{name:\"-moz-context-properties\",desc:\"If you reference an SVG image in a webpage (such as with the <img> element or as a background image), the SVG image can coordinate with the embedding element (its context) to have the image adopt property values set on the embedding element. To do this the embedding element needs to list the properties that are to be made available to the image by listing them as values of the -moz-context-properties property, and the image needs to opt in to using those properties by using values such as the context-fill value.\\n\\nThis feature is available since Firefox 55, but is only currently supported with SVG images loaded via chrome:// or resource:// URLs. To experiment with the feature in SVG on the Web it is necessary to set the svg.context-properties.content.enabled pref to true.\",restriction:\"none\",status:\"n\",syntax:\"none | [ fill | fill-opacity | stroke | stroke-opacity ]#\",browsers:\"FF55\"},{name:\"-moz-float-edge\",desc:\"The non-standard -moz-float-edge CSS property specifies whether the height and width properties of the element include the margin, border, or padding thickness.\",restriction:\"none\",status:\"n\",syntax:\"border-box | content-box | margin-box | padding-box\"},{name:\"-moz-force-broken-image-icon\",desc:\"The -moz-force-broken-image-icon extended CSS property can be used to force the broken image icon to be shown even when a broken image has an alt attribute.\",restriction:\"none\",status:\"n\",syntax:\"<integer>\"},{name:\"-moz-image-region\",desc:\"For certain XUL elements and pseudo-elements that use an image from the list-style-image property, this property specifies a region of the image that is used in place of the whole image. This allows elements to use different pieces of the same image to improve performance.\",restriction:\"none\",status:\"n\",syntax:\"<shape> | auto\"},{name:\"-moz-orient\",desc:\"The -moz-orient CSS property specifies the orientation of the element to which it's applied.\",restriction:\"none\",status:\"n\",syntax:\"inline | block | horizontal | vertical\"},{name:\"-moz-outline-radius\",desc:\"In Mozilla applications like Firefox, the -moz-outline-radius CSS property can be used to give an element's outline rounded corners.\",restriction:\"none\",status:\"n\",syntax:\"<outline-radius>{1,4} [ / <outline-radius>{1,4} ]?\"},{name:\"-moz-outline-radius-bottomleft\",desc:\"In Mozilla applications, the -moz-outline-radius-bottomleft CSS property can be used to round the bottom-left corner of an element's outline.\",restriction:\"none\",status:\"n\",syntax:\"<outline-radius>\"},{name:\"-moz-outline-radius-bottomright\",desc:\"In Mozilla applications, the -moz-outline-radius-bottomright CSS property can be used to round the bottom-right corner of an element's outline.\",restriction:\"none\",status:\"n\",syntax:\"<outline-radius>\"},{name:\"-moz-outline-radius-topleft\",desc:\"In Mozilla applications, the -moz-outline-radius-topleft CSS property can be used to round the top-left corner of an element's outline.\",restriction:\"none\",status:\"n\",syntax:\"<outline-radius>\"},{name:\"-moz-outline-radius-topright\",desc:\"In Mozilla applications, the -moz-outline-radius-topright CSS property can be used to round the top-right corner of an element's outline.\",restriction:\"none\",status:\"n\",syntax:\"<outline-radius>\"},{name:\"-moz-stack-sizing\",desc:\"-moz-stack-sizing is an extended CSS property. Normally, a stack will change its size so that all of its child elements are completely visible. For example, moving a child of the stack far to the right will widen the stack so the child remains visible.\",restriction:\"none\",status:\"n\",syntax:\"ignore | stretch-to-fit\"},{name:\"-moz-text-blink\",desc:\"The -moz-text-blink non-standard Mozilla CSS extension specifies the blink mode.\",restriction:\"none\",status:\"n\",syntax:\"none | blink\"},{name:\"-moz-user-input\",desc:\"In Mozilla applications, -moz-user-input determines if an element will accept user input.\",restriction:\"none\",status:\"n\",syntax:\"auto | none | enabled | disabled\"},{name:\"-moz-user-modify\",desc:\"The -moz-user-modify property has no effect. It was originally planned to determine whether or not the content of an element can be edited by a user.\",restriction:\"none\",status:\"n\",syntax:\"read-only | read-write | write-only\"},{name:\"-moz-window-dragging\",desc:\"The -moz-window-dragging CSS property specifies whether a window is draggable or not. It only works in Chrome code, and only on Mac OS X.\",restriction:\"none\",status:\"n\",syntax:\"drag | no-drag\"},{name:\"-moz-window-shadow\",desc:\"The -moz-window-shadow CSS property specifies whether a window will have a shadow. It only works on Mac OS X.\",restriction:\"none\",status:\"n\",syntax:\"default | menu | tooltip | sheet | none\"},{name:\"-webkit-border-before\",desc:\"The -webkit-border-before CSS property is a shorthand property for setting the individual logical block start border property values in a single place in the style sheet.\",restriction:\"none\",status:\"n\",syntax:\"<'border-width'> || <'border-style'> || <'color'>\"},{name:\"-webkit-border-before-color\",desc:\"The -webkit-border-before-color CSS property sets the color of the individual logical block start border in a single place in the style sheet.\",restriction:\"none\",status:\"n\",syntax:\"<'color'>\"},{name:\"-webkit-border-before-style\",desc:\"The -webkit-border-before-style CSS property sets the style of the individual logical block start border in a single place in the style sheet.\",restriction:\"none\",status:\"n\",syntax:\"<'border-style'>\"},{name:\"-webkit-border-before-width\",desc:\"The -webkit-border-before-width CSS property sets the width of the individual logical block start border in a single place in the style sheet.\",restriction:\"none\",status:\"n\",syntax:\"<'border-width'>\"},{name:\"-webkit-mask\",desc:\"The mask CSS property alters the visibility of an element by either partially or fully hiding it. This is accomplished by either masking or clipping the image at specific points.\",restriction:\"none\",status:\"n\",syntax:\"[ <mask-reference> || <position> [ / <bg-size> ]? || <repeat-style> || [ <box> | border | padding | content | text ] || [ <box> | border | padding | content ] ]#\"},{name:\"-webkit-mask-attachment\",desc:\"If a -webkit-mask-image is specified, -webkit-mask-attachment determines whether the mask image's position is fixed within the viewport, or scrolls along with its containing block.\",restriction:\"none\",status:\"n\",syntax:\"<attachment>#\",browsers:\"S4,C\"},{name:\"-webkit-mask-composite\",desc:\"The -webkit-mask-composite property specifies the manner in which multiple mask images applied to the same element are composited with one another. Mask images are composited in the opposite order that they are declared with the -webkit-mask-image property.\",restriction:\"none\",status:\"n\",syntax:\"<composite-style>#\"},{name:\"-webkit-mask-position\",desc:\"The mask-position CSS property sets the initial position, relative to the mask position layer defined by mask-origin, for each defined mask image.\",restriction:\"none\",status:\"n\",syntax:\"<position>#\"},{name:\"-webkit-mask-position-x\",desc:\"The -webkit-mask-position-x CSS property sets the initial horizontal position of a mask image.\",restriction:\"none\",status:\"n\",syntax:\"[ <length-percentage> | left | center | right ]#\"},{name:\"-webkit-mask-position-y\",desc:\"The -webkit-mask-position-y CSS property sets the initial vertical position of a mask image.\",restriction:\"none\",status:\"n\",syntax:\"[ <length-percentage> | top | center | bottom ]#\",browsers:\"S4,C1\"},{name:\"-webkit-mask-repeat-x\",desc:\"The -webkit-mask-repeat-x property specifies whether and how a mask image is repeated (tiled) horizontally.\",restriction:\"none\",status:\"n\",syntax:\"repeat | no-repeat | space | round\"},{name:\"-webkit-mask-repeat-y\",desc:\"The -webkit-mask-repeat-y property specifies whether and how a mask image is repeated (tiled) vertically.\",restriction:\"none\",status:\"n\",syntax:\"repeat | no-repeat | space | round\"},{name:\"appearance\",desc:\"Changes the appearance of buttons and other controls to resemble native controls.\",restriction:\"none\",status:\"e\",syntax:\"auto | none\",browsers:\"E12,FF1,S3,C1,O15\"},{name:\"azimuth\",desc:\"In combination with elevation, the azimuth CSS property enables different audio sources to be positioned spatially for aural presentation. This is important in that it provides a natural way to tell several voices apart, as each can be positioned to originate at a different location on the sound stage. Stereo output produce a lateral sound stage, while binaural headphones and multi-speaker setups allow for a fully three-dimensional stage.\",restriction:\"none\",status:\"o\",syntax:\"<angle> | [ [ left-side | far-left | left | center-left | center | center-right | right | far-right | right-side ] || behind ] | leftwards | rightwards\"},{name:\"backdrop-filter\",desc:\"The backdrop-filter CSS property lets you apply graphical effects such as blurring or color shifting to the area behind an element. Because it applies to everything behind the element, to see the effect you must make the element or its background at least partially transparent.\",restriction:\"none\",status:\"e\",syntax:\"none | <filter-function-list>\",browsers:\"E17,S9,C47\"},{name:\"block-overflow\",desc:\"\",restriction:\"none\",status:\"e\",syntax:\"clip | ellipsis | <string>\"},{name:\"box-align\",desc:\"The box-align CSS property specifies how an element aligns its contents across its layout in a perpendicular direction. The effect of the property is only visible if there is extra space in the box.\",restriction:\"none\",status:\"n\",syntax:\"start | center | end | baseline | stretch\"},{name:\"box-direction\",desc:\"The box-direction CSS property specifies whether a box lays out its contents normally (from the top or left edge), or in reverse (from the bottom or right edge).\",restriction:\"none\",status:\"n\",syntax:\"normal | reverse | inherit\",browsers:\"E12,FF,S3,C,O\"},{name:\"box-flex\",desc:\"The -moz-box-flex and -webkit-box-flex CSS properties specify how a -moz-box or -webkit-box grows to fill the box that contains it, in the direction of the containing box's layout.\",restriction:\"none\",status:\"n\",syntax:\"<number>\",browsers:\"E12,FF,S3,C,O\"},{name:\"box-flex-group\",desc:\"The box-flex-group CSS property assigns the flexbox's child elements to a flex group.\",restriction:\"none\",status:\"n\",syntax:\"<integer>\"},{name:\"box-lines\",desc:\"The box-lines CSS property determines whether the box may have a single or multiple lines (rows for horizontally oriented boxes, columns for vertically oriented boxes).\",restriction:\"none\",status:\"n\",syntax:\"single | multiple\"},{name:\"box-ordinal-group\",desc:\"The box-ordinal-group CSS property assigns the flexbox's child elements to an ordinal group.\",restriction:\"none\",status:\"n\",syntax:\"<integer>\"},{name:\"box-orient\",desc:\"The box-orient CSS property specifies whether an element lays out its contents horizontally or vertically.\",restriction:\"none\",status:\"n\",syntax:\"horizontal | vertical | inline-axis | block-axis | inherit\",browsers:\"E12,FF,S3,C,O\"},{name:\"box-pack\",desc:\"The -moz-box-pack and -webkit-box-pack CSS properties specify how a -moz-box or -webkit-box packs its contents in the direction of its layout. The effect of this is only visible if there is extra space in the box.\",restriction:\"none\",status:\"n\",syntax:\"start | center | end | justify\",browsers:\"E12,FF,S3,C,O\"},{name:\"color-adjust\",desc:\"The color-adjust property is a non-standard CSS extension that can be used to force printing of background colors and images in browsers based on the WebKit engine.\",restriction:\"none\",syntax:\"economy | exact\"},{name:\"font-optical-sizing\",desc:\"\",restriction:\"none\",syntax:\"auto | none\",browsers:\"FF61\"},{name:\"font-variation-settings\",desc:\"The font-variation-settings CSS property provides low-level control over OpenType or TrueType font variations, by specifying the four letter axis names of the features you want to vary, along with their variation values.\",restriction:\"none\",syntax:\"normal | [ <string> <number> ]#\",browsers:\"FF61,S11,C62,O49\"},{name:\"gap\",desc:\"The gap CSS property is a shorthand property for row-gap and column-gap specifying the gutters between grid rows and columns.\",restriction:\"none\",syntax:\"<'row-gap'> <'column-gap'>?\",browsers:\"E16,FF61,S10.1,C66,O53\"},{name:\"hanging-punctuation\",desc:\"The hanging-punctuation CSS property specifies whether a punctuation mark should hang at the start or end of a line of text. Hanging punctuation may be placed outside the line box.\",restriction:\"none\",syntax:\"none | [ first || [ force-end | allow-end ] || last ]\"},{name:\"image-resolution\",desc:\"The image-resolution property specifies the intrinsic resolution of all raster images used in or on the element. It affects both content images (e.g. replaced elements and generated content) and decorative images (such as background-image). The intrinsic resolution of an image is used to determine the image’s intrinsic dimensions.\",restriction:\"none\",status:\"e\",syntax:\"[ from-image || <resolution> ] && snap?\"},{name:\"initial-letter\",desc:\"The initial-letter CSS property specifies styling for dropped, raised, and sunken initial letters.\",restriction:\"none\",status:\"e\",syntax:\"normal | [ <number> <integer>? ]\",browsers:\"S9\"},{name:\"initial-letter-align\",desc:\"The initial-letter-align CSS property specifies the alignment of initial letters within a paragraph.\",restriction:\"none\",status:\"e\",syntax:\"[ auto | alphabetic | hanging | ideographic ]\"},{name:\"line-clamp\",desc:\"\",restriction:\"none\",status:\"e\",syntax:\"none | <integer>\"},{name:\"line-height-step\",desc:\"The line-height-step CSS property defines the step units for line box heights. When the step unit is positive, line box heights are rounded up to the closest multiple of the unit. Negative values are invalid.\",restriction:\"none\",status:\"e\",syntax:\"<length>\"},{name:\"mask\",desc:\"The mask CSS property alters the visibility of an element by either partially or fully hiding it. This is accomplished by either masking or clipping the image at specific points.\",restriction:\"none\",syntax:\"<mask-layer>#\",browsers:\"E12,FF,S4,C1,O\"},{name:\"mask-border\",desc:\"The mask-border CSS property lets you create a mask along the edge of an element's border.\\n\\nThis property is a shorthand for mask-border-source, mask-border-slice, mask-border-width, mask-border-outset, mask-border-repeat, and mask-border-mode. As with all shorthand properties, any omitted sub-values will be set to their initial value.\",restriction:\"none\",status:\"e\",syntax:\"<'mask-border-source'> || <'mask-border-slice'> [ / <'mask-border-width'>? [ / <'mask-border-outset'> ]? ]? || <'mask-border-repeat'> || <'mask-border-mode'>\"},{name:\"mask-border-mode\",desc:\"The mask-border-mode CSS property specifies the blending mode used in a mask border.\",restriction:\"none\",status:\"e\",syntax:\"luminance | alpha\"},{name:\"mask-border-outset\",desc:\"The mask-border-outset CSS property specifies the distance by which an element's mask border is set out from its border box.\",restriction:\"none\",status:\"e\",syntax:\"[ <length> | <number> ]{1,4}\"},{name:\"mask-border-repeat\",desc:\"The mask-border-repeat CSS property defines how the edge regions of a source image are adjusted to fit the dimensions of an element's mask border.\",restriction:\"none\",status:\"e\",syntax:\"[ stretch | repeat | round | space ]{1,2}\"},{name:\"mask-border-slice\",desc:\"The mask-border-slice CSS property divides the image specified by mask-border-source into regions. These regions are used to form the components of an element's mask border.\",restriction:\"none\",status:\"e\",syntax:\"<number-percentage>{1,4} fill?\"},{name:\"mask-border-source\",desc:\"The mask-border-source CSS property specifies the source image used to create an element's mask border.\\n\\nThe mask-border-slice property is used to divide the source image into regions, which are then dynamically applied to the final mask border.\",restriction:\"none\",status:\"e\",syntax:\"none | <image>\"},{name:\"mask-border-width\",desc:\"The mask-border-width CSS property specifies the width of an element's mask border.\",restriction:\"none\",status:\"e\",syntax:\"[ <length-percentage> | <number> | auto ]{1,4}\"},{name:\"mask-clip\",desc:\"The mask-clip CSS property determines the area, which is affected by a mask. The painted content of an element must be restricted to this area.\",restriction:\"none\",syntax:\"[ <geometry-box> | no-clip ]#\",browsers:\"FF53,S,C,O\"},{name:\"mask-composite\",desc:\"The mask-composite CSS property represents a compositing operation used on the current mask layer with the mask layers below it.\",restriction:\"none\",syntax:\"<compositing-operator>#\",browsers:\"FF53\"},{name:\"max-lines\",desc:\"\",restriction:\"none\",status:\"e\",syntax:\"none | <integer>\"},{name:\"offset\",desc:\"The offset CSS property is a shorthand property for animating an element along a defined path.\",restriction:\"none\",status:\"e\",syntax:\"[ <'offset-position'>? [ <'offset-path'> [ <'offset-distance'> || <'offset-rotate'> ]? ]? ]! [ / <'offset-anchor'> ]?\",browsers:\"C55\"},{name:\"offset-anchor\",desc:\"Defines an anchor point of the box positioned along the path. The anchor point specifies the point of the box which is to be considered as the point that is moved along the path.\",restriction:\"none\",status:\"e\",syntax:\"auto | <position>\"},{name:\"offset-distance\",desc:\"The offset-distance CSS property specifies a position along an offset-path.\",restriction:\"none\",status:\"e\",syntax:\"<length-percentage>\",browsers:\"C55\"},{name:\"offset-path\",desc:'The offset-path CSS property specifies the offset path where the element gets positioned. The exact element’s position on the offset path is determined by the offset-distance property. An offset path is either a specified path with one or multiple sub-paths or the geometry of a not-styled basic shape. Each shape or path must define an initial position for the computed value of \"0\" for offset-distance and an initial direction which specifies the rotation of the object to the initial position.\\n\\nIn this specification, a direction (or rotation) of 0 degrees is equivalent to the direction of the positive x-axis in the object’s local coordinate system. In other words, a rotation of 0 degree points to the right side of the UA if the object and its ancestors have no transformation applied.',restriction:\"none\",status:\"e\",syntax:\"none | ray( [ <angle> && <size>? && contain? ] ) | <path()> | <url> | [ <basic-shape> || <geometry-box> ]\",browsers:\"C55\"},{name:\"offset-position\",desc:\"Specifies the initial position of the offset path. If position is specified with static, offset-position would be ignored.\",restriction:\"none\",status:\"e\",syntax:\"auto | <position>\"},{name:\"offset-rotate\",desc:\"The offset-rotate CSS property defines the direction of the element while positioning along the offset path.\",restriction:\"none\",status:\"e\",syntax:\"[ auto | reverse ] || <angle>\",browsers:\"C56\"},{name:\"overflow-anchor\",desc:\"\",restriction:\"none\",status:\"e\",syntax:\"auto | none\"},{name:\"overflow-block\",desc:\"\",restriction:\"none\",status:\"e\",syntax:\"<'overflow'>\"},{name:\"overflow-clip-box\",desc:\"The overflow-clip-box CSS property specifies relative to which box the clipping happens when there is an overflow. It is short hand for the overflow-clip-box-inline and overflow-clip-box-block properties.\",restriction:\"none\",status:\"n\",syntax:\"padding-box | content-box\"},{name:\"overflow-inline\",desc:\"\",restriction:\"none\",status:\"e\",syntax:\"<'overflow'>\"},{name:\"overscroll-behavior\",desc:\"The overscroll-behavior CSS property is shorthand for the overscroll-behavior-x and overscroll-behavior-y properties, which allow you to control the browser's scroll overflow behavior — what happens when the boundary of a scrolling area is reached.\",restriction:\"none\",status:\"n\",syntax:\"[ contain | none | auto ]{1,2}\",browsers:\"FF59,C63,O50\"},{name:\"overscroll-behavior-x\",desc:\"The overscroll-behavior-x CSS property is allows you to control the browser's scroll overflow behavior — what happens when the boundary of a scrolling area is reached — in the x axis direction.\",restriction:\"none\",status:\"n\",syntax:\"contain | none | auto\",browsers:\"FF59,C63,O50\"},{name:\"overscroll-behavior-y\",desc:\"The overscroll-behavior-y CSS property is allows you to control the browser's scroll overflow behavior — what happens when the boundary of a scrolling area is reached — in the y axis direction.\",restriction:\"none\",status:\"n\",syntax:\"contain | none | auto\",browsers:\"FF59,C63,O50\"},{name:\"place-content\",desc:\"The place-content CSS shorthand property sets both the align-content and justify-content properties.\",restriction:\"none\",syntax:\"<'align-content'> <'justify-content'>?\",browsers:\"FF,C59,O\"},{name:\"rotate\",desc:\"The rotate CSS property allows you to specify rotation transforms individually and independantly of the transform property. This maps better to typical user interface usage, and saves having to remember the exact order of transform functions to specify in the transform value.\",restriction:\"none\",syntax:\"none | [ x | y | z | <number>{3} ]? && <angle>\",browsers:\"FF60,C\"},{name:\"row-gap\",desc:\"The row-gap CSS property specifies the gutter between grid rows.\",restriction:\"none\",syntax:\"normal | <length-percentage>\",browsers:\"E16,FF61,S10.1,C66,O53\"},{name:\"ruby-merge\",desc:\"This property controls how ruby annotation boxes should be rendered when there are more than one in a ruby container box: whether each pair should be kept separate, the annotations should be collapsed and rendered as a group, or the separation should be determined based on the space available.\",restriction:\"none\",status:\"e\",syntax:\"separate | collapse | auto\"},{name:\"scale\",desc:\"The scale CSS property allows you to specify scale transforms individually and independantly of the transform property. This maps better to typical user interface usage, and saves having to remember the exact order of transform functions to specify in the transform value.\",restriction:\"none\",syntax:\"none | <number>{1,3}\",browsers:\"FF60,C\"},{name:\"scroll-snap-type-x\",desc:\"The scroll-snap-type-x CSS property defines how strictly snap points are enforced on the horizontal axis of the scroll container in case there is one.\\n\\nSpecifying any precise animations or physics used to enforce those snap points is not covered by this property but instead left up to the user agent.\",restriction:\"none\",status:\"n\",syntax:\"none | mandatory | proximity\",browsers:\"FF39,S9\"},{name:\"scroll-snap-type-y\",desc:\"The scroll-snap-type-y CSS property defines how strictly snap points are enforced on the vertical axis of the scroll container in case there is one.\\n\\nSpecifying any precise animations or physics used to enforce those snap points is not covered by this property but instead left up to the user agent.\",restriction:\"none\",status:\"n\",syntax:\"none | mandatory | proximity\",browsers:\"FF39\"},{name:\"text-combine-upright\",desc:\"The text-combine-upright CSS property specifies the combination of multiple characters into the space of a single character. If the combined text is wider than 1em, the user agent must fit the contents within 1em. The resulting composition is treated as a single upright glyph for layout and decoration. This property only has an effect in vertical writing modes.\\n\\nThis is used to produce an effect that is known as tate-chū-yoko (縦中横) in Japanese, or as 直書橫向 in Chinese.\",restriction:\"none\",syntax:\"none | all | [ digits <integer>? ]\",browsers:\"all\"},{name:\"text-decoration-skip\",desc:\"The text-decoration-skip CSS property specifies what parts of the element’s content any text decoration affecting the element must skip over. It controls all text decoration lines drawn by the element and also any text decoration lines drawn by its ancestors.\",restriction:\"none\",status:\"e\",syntax:\"none | [ objects || [ spaces | [ leading-spaces || trailing-spaces ] ] || edges || box-decoration ]\",browsers:\"S8,C57,O44\"},{name:\"text-decoration-skip-ink\",desc:\"The text-decoration-skip-ink CSS property specifies how overlines and underlines are drawn when they pass over glyph ascenders and descenders.\",restriction:\"none\",status:\"e\",syntax:\"auto | none\",browsers:\"C64,O50\"},{name:\"text-emphasis\",desc:\"The text-emphasis CSS property is a shorthand property for setting text-emphasis-style and text-emphasis-color in one declaration. This property will apply the specified emphasis mark to each character of the element's text, except separator characters, like spaces,  and control characters.\",restriction:\"none\",syntax:\"<'text-emphasis-style'> || <'text-emphasis-color'>\",browsers:\"FF46,S,C25,O15\"},{name:\"text-emphasis-color\",desc:\"The text-emphasis-color CSS property defines the color used to draw emphasis marks on text being rendered in the HTML document. This value can also be set and reset using the text-emphasis shorthand.\",restriction:\"none\",syntax:\"<color>\",browsers:\"FF46,S,C25,O15\"},{name:\"text-emphasis-position\",desc:\"The text-emphasis-position CSS property describes where emphasis marks are drawn at. The effect of emphasis marks on the line height is the same as for ruby text: if there isn't enough place, the line height is increased.\",restriction:\"none\",syntax:\"[ over | under ] && [ right | left ]\",browsers:\"FF46,S,C,O\"},{name:\"text-emphasis-style\",desc:\"The text-emphasis-style CSS property defines the type of emphasis used. It can also be set, and reset, using the text-emphasis shorthand.\",restriction:\"none\",syntax:\"none | [ [ filled | open ] || [ dot | circle | double-circle | triangle | sesame ] ] | <string>\",browsers:\"FF46,S,C25,O15\"},{name:\"text-size-adjust\",desc:\"The text-size-adjust CSS property controls the text inflation algorithm used on some smartphones and tablets. Other browsers will ignore this property.\",restriction:\"none\",status:\"e\",syntax:\"none | auto | <percentage>\",browsers:\"E12,C54,O42\"},{name:\"transform-box\",desc:\"The transform-box CSS property defines the layout box to which the transform and transform-origin properties relate.\",restriction:\"none\",syntax:\"border-box | fill-box | view-box\",browsers:\"FF55,C64,O51\"},{name:\"translate\",desc:\"The translate CSS property allows you to specify translation transforms individually and independantly of the transform property. This maps better to typical user interface usage, and saves having to remember the exact order of transform functions to specify in the transform value.\",restriction:\"none\",syntax:\"none | <length-percentage> [ <length-percentage> <length>? ]?\",browsers:\"FF60,C\"},{name:\"speak-as\",desc:\"The speak-as descriptor specifies how a counter symbol constructed with a given @counter-style will be represented in the spoken form. For example, an author can specify a counter symbol to be either spoken as its numerical value or just represented with an audio cue.\",restriction:\"none\",syntax:\"auto | bullets | numbers | words | spell-out | <counter-style-name>\"},{name:\"font-display\",desc:\"The font-display descriptor determines how a font face is displayed based on whether and when it is downloaded and ready to use.\",restriction:\"none\",status:\"e\",syntax:\"[ auto | block | swap | fallback | optional ]\"},{name:\"bleed\",desc:\"The bleed CSS at-rule descriptor, used with the @page at-rule, specifies the extent of the page bleed area outside the page box. This property only has effect if crop marks are enabled using the marks property.\",restriction:\"none\",status:\"e\",syntax:\"auto | <length>\"},{name:\"marks\",desc:\"The marks CSS at-rule descriptor, used with the @page at-rule, adds crop and/or cross marks to the presentation of the document. Crop marks indicate where the page should be cut. Cross marks are used to align sheets.\",restriction:\"none\",status:\"e\",syntax:\"none | [ crop || cross ]\"},{name:\"max-zoom\",desc:\"The max-zoom CSS descriptor sets the maximum zoom factor of a document defined by the @viewport at-rule. The browser will not zoom in any further than this, whether automatically or at the user's request.\\n\\nA zoom factor of 1.0 or 100% corresponds to no zooming. Larger values are zoomed in. Smaller values are zoomed out.\",restriction:\"none\",syntax:\"auto | <number> | <percentage>\"},{name:\"min-zoom\",desc:\"The min-zoom CSS descriptor sets the minimum zoom factor of a document defined by the @viewport at-rule. The browser will not zoom out any further than this, whether automatically or at the user's request.\\n\\nA zoom factor of 1.0 or 100% corresponds to no zooming. Larger values are zoomed in. Smaller values are zoomed out.\",restriction:\"none\",syntax:\"auto | <number> | <percentage>\"},{name:\"orientation\",desc:\"The orientation CSS @media media feature can be used to apply styles based on the orientation of the viewport (or the page box, for paged media).\",restriction:\"none\",syntax:\"auto | portrait | landscape\"},{name:\"user-zoom\",desc:\"The user-zoom CSS descriptor controls whether or not the user can change the zoom factor of a document defined by @viewport.\",restriction:\"none\",syntax:\"zoom | fixed\"}]},_r={100:\"Thin\",200:\"Extra Light (Ultra Light)\",300:\"Light\",400:\"Normal\",500:\"Medium\",600:\"Semi Bold (Demi Bold)\",700:\"Bold\",800:\"Extra Bold (Ultra Bold)\",900:\"Black (Heavy)\",\"self-end\":\"The item is packed flush to the edge of the alignment container of the end side of the item, in the appropriate axis.\",\"self-start\":\"The item is packed flush to the edge of the alignment container of the start side of the item, in the appropriate axis..\",alternate:\"The animation cycle iterations that are odd counts are played in the normal direction, and the animation cycle iterations that are even counts are played in a reverse direction.\",\"alternate-reverse\":\"The animation cycle iterations that are odd counts are played in the reverse direction, and the animation cycle iterations that are even counts are played in a normal direction.\",backwards:\"The beginning property value (as defined in the first @keyframes at-rule) is applied before the animation is displayed, during the period defined by 'animation-delay'.\",forwards:\"The final property value (as defined in the last @keyframes at-rule) is maintained after the animation completes.\",paused:\"A running animation will be paused.\",running:\"Resume playback of a paused animation.\",multiply:\"The source color is multiplied by the destination color and replaces the destination.\",screen:\"Multiplies the complements of the backdrop and source color values, then complements the result.\",overlay:\"Multiplies or screens the colors, depending on the backdrop color value.\",darken:\"Selects the darker of the backdrop and source colors.\",lighten:\"Selects the lighter of the backdrop and source colors.\",\"color-dodge\":\"Brightens the backdrop color to reflect the source color.\",\"color-burn\":\"Darkens the backdrop color to reflect the source color.\",\"hard-light\":\"Multiplies or screens the colors, depending on the source color value.\",\"soft-light\":\"Darkens or lightens the colors, depending on the source color value.\",difference:\"Subtracts the darker of the two constituent colors from the lighter color..\",exclusion:\"Produces an effect similar to that of the Difference mode but lower in contrast.\",hue:\"Creates a color with the hue of the source color and the saturation and luminosity of the backdrop color.\",saturation:\"Creates a color with the saturation of the source color and the hue and luminosity of the backdrop color.\",color:\"Creates a color with the hue and saturation of the source color and the luminosity of the backdrop color.\",luminosity:\"Creates a color with the luminosity of the source color and the hue and saturation of the backdrop color.\",repeat:\"The image is tiled (repeated) to fill the area.\",clone:\"Each box is independently wrapped with the border and padding.\",slice:\"The effect is as though the element were rendered with no breaks present, and then sliced by the breaks afterward.\",inset:\"Changes the drop shadow from an outer shadow (one that shadows the box onto the canvas, as if it were lifted above the canvas) to an inner shadow (one that shadows the canvas onto the box, as if the box were cut out of the canvas and shifted behind it).\",\"border-box\":\"The specified width and height (and respective min/max properties) on this element determine the border box of the element.\",\"content-box\":\"Behavior of width and height as specified by CSS2.1. The specified width and height (and respective min/max properties) apply to the width and height respectively of the content box of the element.\",\"rect()\":\"Specifies offsets from the edges of the border box.\",evenodd:\"Determines the ‘insideness’ of a point on the canvas by drawing a ray from that point to infinity in any direction and counting the number of path segments from the given shape that the ray crosses.\",nonzero:\"Determines the ‘insideness’ of a point on the canvas by drawing a ray from that point to infinity in any direction and then examining the places where a segment of the shape crosses the ray.\",linearRGB:\"Color operations should occur in the linearized RGB color space.\",sRGB:\"Color operations should occur in the sRGB color space.\",balance:\"Balance content equally between columns, if possible.\",size:\"For properties that can have effects on more than just an element and its descendants, those effects don't escape the containing element.\",layout:\"Turns on layout containment for the element.\",paint:\"Turns on paint containment for the element.\",\"attr()\":\"The attr(n) function returns as a string the value of attribute n for the subject of the selector.\",\"counter(name)\":\"Counters are denoted by identifiers (see the 'counter-increment' and 'counter-reset' properties).\",alias:\"Indicates an alias of/shortcut to something is to be created. Often rendered as an arrow with a small curved arrow next to it.\",\"all-scroll\":\"Indicates that the something can be scrolled in any direction. Often rendered as arrows pointing up, down, left, and right with a dot in the middle.\",cell:\"Indicates that a cell or set of cells may be selected. Often rendered as a thick plus-sign with a dot in the middle.\",\"col-resize\":\"Indicates that the item/column can be resized horizontally. Often rendered as arrows pointing left and right with a vertical bar separating them.\",\"context-menu\":\"A context menu is available for the object under the cursor. Often rendered as an arrow with a small menu-like graphic next to it.\",copy:\"Indicates something is to be copied. Often rendered as an arrow with a small plus sign next to it.\",crosshair:\"A simple crosshair (e.g., short line segments resembling a '+' sign). Often used to indicate a two dimensional bitmap selection mode.\",\"e-resize\":\"Indicates that east edge is to be moved.\",\"ew-resize\":\"Indicates a bidirectional east-west resize cursor.\",grab:\"Indicates that something can be grabbed.\",grabbing:\"Indicates that something is being grabbed.\",help:\"Help is available for the object under the cursor. Often rendered as a question mark or a balloon.\",move:\"Indicates something is to be moved.\",\"-moz-grab\":\"Indicates that something can be grabbed.\",\"-moz-grabbing\":\"Indicates that something is being grabbed.\",\"-moz-zoom-in\":\"Indicates that something can be zoomed (magnified) in.\",\"-moz-zoom-out\":\"Indicates that something can be zoomed (magnified) out.\",\"ne-resize\":\"Indicates that movement starts from north-east corner.\",\"nesw-resize\":\"Indicates a bidirectional north-east/south-west cursor.\",\"no-drop\":\"Indicates that the dragged item cannot be dropped at the current cursor location. Often rendered as a hand or pointer with a small circle with a line through it.\",\"not-allowed\":\"Indicates that the requested action will not be carried out. Often rendered as a circle with a line through it.\",\"n-resize\":\"Indicates that north edge is to be moved.\",\"ns-resize\":\"Indicates a bidirectional north-south cursor.\",\"nw-resize\":\"Indicates that movement starts from north-west corner.\",\"nwse-resize\":\"Indicates a bidirectional north-west/south-east cursor.\",pointer:\"The cursor is a pointer that indicates a link.\",progress:\"A progress indicator. The program is performing some processing, but is different from 'wait' in that the user may still interact with the program. Often rendered as a spinning beach ball, or an arrow with a watch or hourglass.\",\"row-resize\":\"Indicates that the item/row can be resized vertically. Often rendered as arrows pointing up and down with a horizontal bar separating them.\",\"se-resize\":\"Indicates that movement starts from south-east corner.\",\"s-resize\":\"Indicates that south edge is to be moved.\",\"sw-resize\":\"Indicates that movement starts from south-west corner.\",\"vertical-text\":\"Indicates vertical-text that may be selected. Often rendered as a horizontal I-beam.\",wait:\"Indicates that the program is busy and the user should wait. Often rendered as a watch or hourglass.\",\"-webkit-grab\":\"Indicates that something can be grabbed.\",\"-webkit-grabbing\":\"Indicates that something is being grabbed.\",\"-webkit-zoom-in\":\"Indicates that something can be zoomed (magnified) in.\",\"-webkit-zoom-out\":\"Indicates that something can be zoomed (magnified) out.\",\"w-resize\":\"Indicates that west edge is to be moved.\",\"zoom-in\":\"Indicates that something can be zoomed (magnified) in.\",\"zoom-out\":\"Indicates that something can be zoomed (magnified) out.\",ltr:\"Left-to-right direction.\",rtl:\"Right-to-left direction.\",block:\"The element generates a block-level box\",flex:\"The element generates a principal flex container box and establishes a flex formatting context.\",flexbox:\"The element lays out its contents using flow layout (block-and-inline layout). Standardized as 'flex'.\",\"flow-root\":\"The element generates a block container box, and lays out its contents using flow layout.\",grid:\"The element generates a principal grid container box, and establishes a grid formatting context.\",\"inline-block\":\"A block box, which itself is flowed as a single inline box, similar to a replaced element. The inside of an inline-block is formatted as a block box, and the box itself is formatted as an inline box.\",\"inline-flex\":\"Inline-level flex container.\",\"inline-flexbox\":\"Inline-level flex container. Standardized as 'inline-flex'\",\"inline-table\":\"Inline-level table wrapper box containing table box.\",\"list-item\":\"One or more block boxes and one marker box.\",\"-moz-box\":\"The element lays out its contents using flow layout (block-and-inline layout). Standardized as 'flex'.\",\"-moz-inline-box\":\"Inline-level flex container. Standardized as 'inline-flex'\",\"-ms-flexbox\":\"The element lays out its contents using flow layout (block-and-inline layout). Standardized as 'flex'.\",\"-ms-grid\":\"The element generates a principal grid container box, and establishes a grid formatting context.\",\"-ms-inline-flexbox\":\"Inline-level flex container. Standardized as 'inline-flex'\",\"-ms-inline-grid\":\"Inline-level grid container.\",\"run-in\":\"The element generates a run-in box. Run-in elements act like inlines or blocks, depending on the surrounding elements.\",table:\"The element generates a principal table wrapper box containing an additionally-generated table box, and establishes a table formatting context.\",\"-webkit-box\":\"The element lays out its contents using flow layout (block-and-inline layout). Standardized as 'flex'.\",\"-webkit-flex\":\"The element lays out its contents using flow layout (block-and-inline layout).\",\"-webkit-inline-box\":\"Inline-level flex container. Standardized as 'inline-flex'\",\"-webkit-inline-flex\":\"Inline-level flex container.\",hide:\"No borders or backgrounds are drawn around/behind empty cells.\",show:\"Borders and backgrounds are drawn around/behind empty cells (like normal cells).\",accumulate:\"If the ancestor container element has a property of new, then all graphics elements within the current container are rendered both on the parent's background image and onto the target.\",new:\"Create a new background image canvas. All children of the current container element can access the background, and they will be rendered onto both the parent's background image canvas in addition to the target device.\",\"blur()\":\"Applies a Gaussian blur to the input image.\",\"brightness()\":\"Applies a linear multiplier to input image, making it appear more or less bright.\",\"contrast()\":\"Adjusts the contrast of the input.\",\"drop-shadow()\":\"Applies a drop shadow effect to the input image.\",\"grayscale()\":\"Converts the input image to grayscale.\",\"hue-rotate()\":\"Applies a hue rotation on the input image. \",\"invert()\":\"Inverts the samples in the input image.\",\"opacity()\":\"Applies transparency to the samples in the input image.\",\"saturate()\":\"Saturates the input image.\",\"sepia()\":\"Converts the input image to sepia.\",\"column-reverse\":\"Same as 'column', except the main-start and main-end directions are swapped.\",\"row-reverse\":\"Same as 'row', except the main-start and main-end directions are swapped.\",\"wrap-reverse\":\"Same as 'wrap', except the cross-start and cross-end directions are swapped.\",\"inline-end\":\"A keyword indicating that the element must float on the end side of its containing block. That is the right side with ltr scripts, and the left side with rtl scripts.\",\"inline-start\":\"A keyword indicating that the element must float on the start side of its containing block. That is the left side with ltr scripts, and the right side with rtl scripts.\",bold:\"Same as 700\",bolder:\"Specifies the weight of the face bolder than the inherited value.\",caption:\"The font used for captioned controls (e.g., buttons, drop-downs, etc.).\",lighter:\"Specifies the weight of the face lighter than the inherited value.\",menu:\"The font used in menus (e.g., dropdown menus and menu lists).\",\"message-box\":\"The font used in dialog boxes.\",\"small-caption\":\"The font used for labeling small controls.\",\"status-bar\":\"The font used in window status bars.\",'\"aalt\"':\"Access All Alternates.\",'\"abvf\"':\"Above-base Forms. Required in Khmer script.\",'\"abvm\"':\"Above-base Mark Positioning. Required in Indic scripts.\",'\"abvs\"':\"Above-base Substitutions. Required in Indic scripts.\",'\"afrc\"':\"Alternative Fractions.\",'\"akhn\"':\"Akhand. Required in most Indic scripts.\",'\"blwf\"':\"Below-base Form. Required in a number of Indic scripts.\",'\"blwm\"':\"Below-base Mark Positioning. Required in Indic scripts.\",'\"blws\"':\"Below-base Substitutions. Required in Indic scripts.\",'\"calt\"':\"Contextual Alternates.\",'\"case\"':\"Case-Sensitive Forms. Applies only to European scripts; particularly prominent in Spanish-language setting.\",'\"ccmp\"':\"Glyph Composition/Decomposition.\",'\"cfar\"':\"Conjunct Form After Ro. Required in Khmer scripts.\",'\"cjct\"':\"Conjunct Forms. Required in Indic scripts that show similarity to Devanagari.\",'\"clig\"':\"Contextual Ligatures.\",'\"cpct\"':\"Centered CJK Punctuation. Used primarily in Chinese fonts.\",'\"cpsp\"':\"Capital Spacing. Should not be used in connecting scripts (e.g. most Arabic).\",'\"cswh\"':\"Contextual Swash.\",'\"curs\"':\"Cursive Positioning. Can be used in any cursive script.\",'\"c2pc\"':\"Petite Capitals From Capitals. Applies only to bicameral scripts.\",'\"dist\"':\"Distances. Required in Indic scripts.\",'\"dnom\"':\"Denominators.\",'\"dtls\"':\"Dotless Forms. Applied to math formula layout.\",'\"expt\"':\"Expert Forms. Applies only to Japanese.\",'\"falt\"':\"Final Glyph on Line Alternates. Can be used in any cursive script.\",'\"fin2\"':\"Terminal Form #2. Used only with the Syriac script.\",'\"fin3\"':\"Terminal Form #3. Used only with the Syriac script.\",'\"fina\"':\"Terminal Forms. Can be used in any alphabetic script.\",'\"flac\"':\"Flattened ascent forms. Applied to math formula layout.\",'\"frac\"':\"Fractions.\",'\"fwid\"':\"Full Widths. Applies to any script which can use monospaced forms.\",'\"half\"':\"Half Forms. Required in Indic scripts that show similarity to Devanagari.\",'\"haln\"':\"Halant Forms. Required in Indic scripts.\",'\"halt\"':\"Alternate Half Widths. Used only in CJKV fonts.\",'\"hist\"':\"Historical Forms.\",'\"hkna\"':\"Horizontal Kana Alternates. Applies only to fonts that support kana (hiragana and katakana).\",'\"hlig\"':\"Historical Ligatures.\",'\"hngl\"':\"Hangul. Korean only.\",'\"hojo\"':\"Hojo Kanji Forms (JIS X 0212-1990 Kanji Forms). Used only with Kanji script.\",'\"hwid\"':\"Half Widths. Generally used only in CJKV fonts.\",'\"init\"':\"Initial Forms. Can be used in any alphabetic script.\",'\"isol\"':\"Isolated Forms. Can be used in any cursive script.\",'\"ital\"':\"Italics. Applies mostly to Latin; note that many non-Latin fonts contain Latin as well.\",'\"jalt\"':\"Justification Alternates. Can be used in any cursive script.\",'\"jp78\"':\"JIS78 Forms. Applies only to Japanese.\",'\"jp83\"':\"JIS83 Forms. Applies only to Japanese.\",'\"jp90\"':\"JIS90 Forms. Applies only to Japanese.\",'\"jp04\"':\"JIS2004 Forms. Applies only to Japanese.\",'\"lfbd\"':\"Left Bounds.\",'\"ljmo\"':\"Leading Jamo Forms. Required for Hangul script when Ancient Hangul writing system is supported.\",'\"locl\"':\"Localized Forms.\",'\"ltra\"':\"Left-to-right glyph alternates.\",'\"ltrm\"':\"Left-to-right mirrored forms.\",'\"mark\"':\"Mark Positioning.\",'\"med2\"':\"Medial Form #2. Used only with the Syriac script.\",'\"medi\"':\"Medial Forms.\",'\"mgrk\"':\"Mathematical Greek.\",'\"mkmk\"':\"Mark to Mark Positioning.\",'\"nalt\"':\"Alternate Annotation Forms.\",'\"nlck\"':\"NLC Kanji Forms. Used only with Kanji script.\",'\"nukt\"':\"Nukta Forms. Required in Indic scripts..\",'\"numr\"':\"Numerators.\",'\"opbd\"':\"Optical Bounds.\",'\"ordn\"':\"Ordinals. Applies mostly to Latin script.\",'\"ornm\"':\"Ornaments.\",'\"palt\"':\"Proportional Alternate Widths. Used mostly in CJKV fonts.\",'\"pcap\"':\"Petite Capitals.\",'\"pkna\"':\"Proportional Kana. Generally used only in Japanese fonts.\",'\"pnum\"':\"Proportional Figures.\",'\"pref\"':\"Pre-base Forms. Required in Khmer and Myanmar (Burmese) scripts and southern Indic scripts that may display a pre-base form of Ra.\",'\"pres\"':\"Pre-base Substitutions. Required in Indic scripts.\",'\"pstf\"':\"Post-base Forms. Required in scripts of south and southeast Asia that have post-base forms for consonants eg: Gurmukhi, Malayalam, Khmer.\",'\"psts\"':\"Post-base Substitutions.\",'\"pwid\"':\"Proportional Widths.\",'\"qwid\"':\"Quarter Widths. Generally used only in CJKV fonts.\",'\"rand\"':\"Randomize.\",'\"rclt\"':\"Required Contextual Alternates. May apply to any script, but is especially important for many styles of Arabic.\",'\"rlig\"':\"Required Ligatures. Applies to Arabic and Syriac. May apply to some other scripts.\",'\"rkrf\"':\"Rakar Forms. Required in Devanagari and Gujarati scripts.\",'\"rphf\"':\"Reph Form. Required in Indic scripts. E.g. Devanagari, Kannada.\",'\"rtbd\"':\"Right Bounds.\",'\"rtla\"':\"Right-to-left alternates.\",'\"rtlm\"':\"Right-to-left mirrored forms.\",'\"ruby\"':\"Ruby Notation Forms. Applies only to Japanese.\",'\"salt\"':\"Stylistic Alternates.\",'\"sinf\"':\"Scientific Inferiors.\",'\"size\"':\"Optical size.\",'\"smpl\"':\"Simplified Forms. Applies only to Chinese and Japanese.\",'\"ssty\"':\"Math script style alternates.\",'\"stch\"':\"Stretching Glyph Decomposition.\",'\"subs\"':\"Subscript.\",'\"sups\"':\"Superscript.\",'\"titl\"':\"Titling.\",'\"tjmo\"':\"Trailing Jamo Forms. Required for Hangul script when Ancient Hangul writing system is supported.\",'\"tnam\"':\"Traditional Name Forms. Applies only to Japanese.\",'\"trad\"':\"Traditional Forms. Applies only to Chinese and Japanese.\",'\"twid\"':\"Third Widths. Generally used only in CJKV fonts.\",'\"unic\"':\"Unicase.\",'\"valt\"':\"Alternate Vertical Metrics. Applies only to scripts with vertical writing modes.\",'\"vatu\"':\"Vattu Variants. Used for Indic scripts. E.g. Devanagari.\",'\"vert\"':\"Vertical Alternates. Applies only to scripts with vertical writing modes.\",'\"vhal\"':\"Alternate Vertical Half Metrics. Used only in CJKV fonts.\",'\"vjmo\"':\"Vowel Jamo Forms. Required for Hangul script when Ancient Hangul writing system is supported.\",'\"vkna\"':\"Vertical Kana Alternates. Applies only to fonts that support kana (hiragana and katakana).\",'\"vkrn\"':\"Vertical Kerning.\",'\"vpal\"':\"Proportional Alternate Vertical Metrics. Used mostly in CJKV fonts.\",'\"vrt2\"':\"Vertical Alternates and Rotation. Applies only to scripts with vertical writing modes.\",'\"zero\"':\"Slashed Zero.\",narrower:\"Indicates a narrower value relative to the width of the parent element.\",wider:\"Indicates a wider value relative to the width of the parent element.\",weight:\"Allow synthetic bold faces.\",\"annotation()\":\"Enables display of alternate annotation forms.\",\"character-variant()\":\"Enables display of specific character variants.\",\"historical-forms\":\"Enables display of historical forms.\",\"ornaments()\":\"Enables replacement of default glyphs with ornaments, if provided in the font.\",\"styleset()\":\"Enables display with stylistic sets.\",\"stylistic()\":\"Enables display of stylistic alternates.\",\"swash()\":\"Enables display of swash glyphs.\",\"all-petite-caps\":\"Enables display of petite capitals for both upper and lowercase letters.\",\"all-small-caps\":\"Enables display of small capitals for both upper and lowercase letters.\",\"petite-caps\":\"Enables display of petite capitals.\",\"titling-caps\":\"Enables display of titling capitals.\",unicase:\"Enables display of mixture of small capitals for uppercase letters with normal lowercase letters.\",\"full-width\":\"Enables rendering of full-width variants.\",jis04:\"Enables rendering of JIS04 forms.\",jis78:\"Enables rendering of JIS78 forms.\",jis83:\"Enables rendering of JIS83 forms.\",jis90:\"Enables rendering of JIS90 forms.\",\"proportional-width\":\"Enables rendering of proportionally-spaced variants.\",simplified:\"Enables rendering of simplified forms.\",traditional:\"Enables rendering of traditional forms.\",\"additional-ligatures\":\"Enables display of additional ligatures.\",\"common-ligatures\":\"Enables display of common ligatures.\",contextual:\"Enables display of contextual alternates.\",\"discretionary-ligatures\":\"Enables display of discretionary ligatures.\",\"historical-ligatures\":\"Enables display of historical ligatures.\",\"no-additional-ligatures\":\"Disables display of additional ligatures.\",\"no-common-ligatures\":\"Disables display of common ligatures.\",\"no-contextual\":\"Disables display of contextual alternates.\",\"no-discretionary-ligatures\":\"Disables display of discretionary ligatures.\",\"no-historical-ligatures\":\"Disables display of historical ligatures.\",\"diagonal-fractions\":\"Enables display of lining diagonal fractions.\",\"lining-nums\":\"Enables display of lining numerals.\",\"oldstyle-nums\":\"Enables display of old-style numerals.\",ordinal:\"Enables display of letter forms used with ordinal numbers.\",\"proportional-nums\":\"Enables display of proportional numerals.\",\"slashed-zero\":\"Enables display of slashed zeros.\",\"stacked-fractions\":\"Enables display of lining stacked fractions.\",\"tabular-nums\":\"Enables display of tabular numerals.\",span:\"Contributes a grid span to the grid item’s placement such that the corresponding edge of the grid item’s grid area is N lines from its opposite edge.\",\"minmax()\":\"Defines a size range greater than or equal to min and less than or equal to max.\",dense:\"If specified, the auto-placement algorithm uses a “dense” packing algorithm, which attempts to fill in holes earlier in the grid if smaller items come up later.\",\"fit-content\":\"Use the fit-content inline size or fit-content block size, as appropriate to the writing mode.\",manual:\"Words are only broken at line breaks where there are characters inside the word that suggest line break opportunities\",flip:\"After rotating by the precededing angle, the image is flipped horizontally. Defaults to 0deg if the angle is ommitted.\",\"from-image\":\"If the image has an orientation specified in its metadata, such as EXIF, this value computes to the angle that the metadata specifies is necessary to correctly orient the image.\",\"crisp-edges\":\"The image must be scaled with an algorithm that preserves contrast and edges in the image, and which does not smooth colors or introduce blur to the image in the process.\",optimizeQuality:\"Deprecated.\",pixelated:\"When scaling the image up, the 'nearest neighbor' or similar algorithm must be used, so that the image appears to be simply composed of very large pixels.\",active:\"The input method editor is initially active; text entry is performed using it unless the user specifically dismisses it.\",disabled:\"The input method editor is disabled and may not be activated by the user.\",inactive:\"The input method editor is initially inactive, but the user may activate it if they wish.\",safe:\"If the size of the item overflows the alignment container, the item is instead aligned as if the alignment mode were start.\",unsafe:\"Regardless of the relative sizes of the item and alignment container, the given alignment value is honored.\",\"space-evenly\":\"The items are evenly distributed within the alignment container along the main axis.\",circle:\"A hollow circle.\",disc:\"A filled circle.\",inside:\"The marker box is outside the principal block box, as described in the section on the ::marker pseudo-element below.\",outside:\"The ::marker pseudo-element is an inline element placed immediately before all ::before pseudo-elements in the principal block box, after which the element's content flows.\",\"symbols()\":\"Allows a counter style to be defined inline.\",\"path()\":\"Defines an SVG path as a string, with optional 'fill-rule' as the first argument.\",\"block-axis\":\"Elements are oriented along the box's axis.\",\"inline-axis\":\"Elements are oriented vertically.\",\"padding-box\":\"The specified width and height (and respective min/max properties) on this element determine the padding box of the element.\",\"line-through\":\"Each line of text has a line through the middle.\",overline:\"Each line of text has a line above it.\",underline:\"Each line of text is underlined.\",dashed:\"Produces a dashed line style.\",dotted:\"Produces a dotted line.\",double:\"Produces a double line.\",solid:\"Produces a solid line.\",wavy:\"Produces a wavy line.\",\"matrix()\":\"Specifies a 2D transformation in the form of a transformation matrix of six values. matrix(a,b,c,d,e,f) is equivalent to applying the transformation matrix [a b c d e f]\",\"matrix3d()\":\"Specifies a 3D transformation as a 4x4 homogeneous matrix of 16 values in column-major order.\",perspective:\"Specifies a perspective projection matrix.\",\"rotate()\":\"Specifies a 2D rotation by the angle specified in the parameter about the origin of the element, as defined by the transform-origin property.\",\"rotate3d()\":\"Specifies a clockwise 3D rotation by the angle specified in last parameter about the [x,y,z] direction vector described by the first 3 parameters.\",\"rotateX('angle')\":\"Specifies a clockwise rotation by the given angle about the X axis.\",\"rotateY('angle')\":\"Specifies a clockwise rotation by the given angle about the Y axis.\",\"rotateZ('angle')\":\"Specifies a clockwise rotation by the given angle about the Z axis.\",\"scale()\":\"Specifies a 2D scale operation by the [sx,sy] scaling vector described by the 2 parameters. If the second parameter is not provided, it is takes a value equal to the first.\",\"scale3d()\":\"Specifies a 3D scale operation by the [sx,sy,sz] scaling vector described by the 3 parameters.\",\"scaleX()\":\"Specifies a scale operation using the [sx,1] scaling vector, where sx is given as the parameter.\",\"scaleY()\":\"Specifies a scale operation using the [sy,1] scaling vector, where sy is given as the parameter.\",\"scaleZ()\":\"Specifies a scale operation using the [1,1,sz] scaling vector, where sz is given as the parameter.\",\"skew()\":\"Specifies a skew transformation along the X and Y axes. The first angle parameter specifies the skew on the X axis. The second angle parameter specifies the skew on the Y axis. If the second parameter is not given then a value of 0 is used for the Y angle (ie: no skew on the Y axis).\",\"skewX()\":\"Specifies a skew transformation along the X axis by the given angle.\",\"skewY()\":\"Specifies a skew transformation along the Y axis by the given angle.\",\"translate()\":\"Specifies a 2D translation by the vector [tx, ty], where tx is the first translation-value parameter and ty is the optional second translation-value parameter.\",\"translate3d()\":\"Specifies a 3D translation by the vector [tx,ty,tz], with tx, ty and tz being the first, second and third translation-value parameters respectively.\",\"translateX()\":\"Specifies a translation by the given amount in the X direction.\",\"translateY()\":\"Specifies a translation by the given amount in the Y direction.\",\"translateZ()\":\"Specifies a translation by the given amount in the Z direction. Note that percentage values are not allowed in the translateZ translation-value, and if present are evaluated as 0.\",false:\"The element does not contain an accelerator key sequence.\",true:\"The element contains an accelerator key sequence.\",bt:\"Bottom-to-top block flow. Layout is horizontal.\",lr:\"Left-to-right direction. The flow orientation is vertical.\",rl:\"Right-to-left direction. The flow orientation is vertical.\",tb:\"Top-to-bottom direction. The flow orientation is horizontal.\",zoom:\"The element is zoomable.\",\"no-limit\":\"There is no limit.\",mode:\"Any of the range of mode values available to the -ms-layout-grid-mode property.\",type:\"Any of the range of type values available to the -ms-layout-grid-type property.\",\"-ms-autohiding-scrollbar\":\"Indicates the element displays auto-hiding scrollbars during mouse interactions and panning indicators during touch and keyboard interactions.\",scrollbar:'Scrollbars are typically narrow strips inserted on one or two edges of an element and which often have arrows to click on and a \"thumb\" to drag up and down (or left and right) to move the contents of the element.',\"ideograph-alpha\":\"Creates 1/4em extra spacing between runs of ideographic letters and non-ideographic letters, such as Latin-based, Cyrillic, Greek, Arabic or Hebrew.\",\"ideograph-numeric\":\"Creates 1/4em extra spacing between runs of ideographic letters and numeric glyphs.\",\"ideograph-parenthesis\":\"Creates extra spacing between normal (non wide) parenthesis and ideographs.\",\"ideograph-space\":\"Extends the width of the space character while surrounded by ideographs.\",punctuation:\"Creates extra non-breaking spacing around punctuation as required by language-specific typographic conventions.\",digits:\"Attempt to typeset horizontally each maximal sequence of consecutive ASCII digits (U+0030–U+0039) that has as many or fewer characters than the specified integer such that it takes up the space of a single character within the vertical line box.\",\"inter-cluster\":\"Justification primarily changes spacing at word separators and at grapheme cluster boundaries in clustered scripts. This value is typically used for Southeast Asian scripts such as Thai.\",\"inter-ideograph\":\"Justification primarily changes spacing at word separators and at inter-graphemic boundaries in scripts that use no word spaces. This value is typically used for CJK languages.\",\"inter-word\":\"Justification primarily changes spacing at word separators. This value is typically used for languages that separate words using spaces, like English or (sometimes) Korean.\",kashida:\"Justification primarily stretches Arabic and related scripts through the use of kashida or other calligraphic elongation.\",clip:\"Clip inline content that overflows. Characters may be only partially rendered.\",ellipsis:\"Render an ellipsis character (U+2026) to represent clipped inline content.\",over:\"The underline is aligned with the 'top' (right in vertical writing) edge of the element's em-box. In this mode, an overline also switches sides.\",under:\"The underline is aligned with the 'bottom' (left in vertical writing) edge of the element's em-box. In this case the underline usually does not cross the descenders. This is sometimes called 'accounting' underline.\",grippers:\"Grippers are always on.\",\"break-all\":\"Lines may break between any two grapheme clusters for non-CJK scripts.\",clear:\"Inline flow content can only wrap on top and bottom of the exclusion and must leave the areas to the start and end edges of the exclusion box empty.\",maximum:\"Inline flow content can wrap on the side of the exclusion with the largest available space for the given line, and must leave the other side of the exclusion empty.\",minimum:\"Inline flow content can flow around the edge of the exclusion with the smallest available space within the flow content’s containing block, and must leave the other edge of the exclusion empty.\",current:\"Indicates that the user agent should target the frame that the element is in.\",root:\"Indicates that the user agent should target the full window.\",\"scale-down\":\"Size the content as if ‘none’ or ‘contain’ were specified, whichever would result in a smaller concrete object size.\",invert:\"Performs a color inversion on the pixels on the screen.\",\"-moz-hidden-unscrollable\":\"Same as the standardized 'clip', except doesn’t establish a block formatting context.\",painted:'The given element can be the target element for pointer events when the pointer is over a \"painted\" area. ',visibleFill:\"The given element can be the target element for pointer events when the ‘visibility’ property is set to visible and when the pointer is over the interior of the element.\",visiblePainted:\"The given element can be the target element for pointer events when the ‘visibility’ property is set to visible and when the pointer is over a ‘painted’ area.\",visibleStroke:\"The given element can be the target element for pointer events when the ‘visibility’ property is set to visible and when the pointer is over the perimeter of the element.\",absolute:\"The box's position (and possibly size) is specified with the 'top', 'right', 'bottom', and 'left' properties. These properties specify offsets with respect to the box's 'containing block'.\",\"-ms-page\":\"The box's position is calculated according to the 'absolute' model.\",relative:\"The box's position is calculated according to the normal flow (this is called the position in normal flow). Then the box is offset relative to its normal position.\",static:\"The box is a normal box, laid out according to the normal flow. The 'top', 'right', 'bottom', and 'left' properties do not apply.\",sticky:\"The box's position is calculated according to the normal flow. Then the box is offset relative to its flow root and containing block and in all cases, including table elements, does not affect the position of any following boxes.\",\"-webkit-sticky\":\"The box's position is calculated according to the normal flow. Then the box is offset relative to its flow root and containing block and in all cases, including table elements, does not affect the position of any following boxes.\",\"distribute-letter\":\"If the width of the ruby text is smaller than that of the base, then the ruby text contents are evenly distributed across the width of the base, with the first and last ruby text glyphs lining up with the corresponding first and last base glyphs. If the width of the ruby text is at least the width of the base, then the letters of the base are evenly distributed across the width of the ruby text.\",\"distribute-space\":\"If the width of the ruby text is smaller than that of the base, then the ruby text contents are evenly distributed across the width of the base, with a certain amount of white space preceding the first and following the last character in the ruby text. That amount of white space is normally equal to half the amount of inter-character space of the ruby text.\",\"line-edge\":\"If the ruby text is not adjacent to a line edge, it is aligned as in 'auto'. If it is adjacent to a line edge, then it is still aligned as in auto, but the side of the ruby text that touches the end of the line is lined up with the corresponding edge of the base.\",after:\"The ruby text appears after the base. This is a relatively rare setting used in ideographic East Asian writing systems, most easily found in educational text.\",before:\"The ruby text appears before the base. This is the most common setting used in ideographic East Asian writing systems.\",\"attr(x)\":\"The value of attribute 'x' is a string value. The string value is evaluated as a <number> to determine the number of ruby base elements to be spanned by the annotation element.\",smooth:\"Scrolls in a smooth fashion using a user-agent-defined timing function and time period.\",\"margin-box\":\"The background is painted within (clipped to) the margin box.\",crispEdges:\"Emphasize the contrast between clean edges of artwork over rendering speed and geometric precision.\",\"format()\":\"Optional hint describing the format of the font resource.\",\"local()\":\"Format-specific string that identifies a locally available copy of a given font.\",butt:\"Indicates that the stroke for each subpath does not extend beyond its two endpoints.\",bevel:\"Indicates that a bevelled corner is to be used to join path segments.\",miter:\"Indicates that a sharp corner is to be used to join path segments.\",additive:\"Represents “sign-value” numbering systems, which, rather than using reusing digits in different positions to change their value, define additional digits with much larger values, so that the value of the number can be obtained by adding all the digits together.\",cyclic:\"Cycles repeatedly through its provided symbols, looping back to the beginning when it reaches the end of the list.\",extends:\"Use the algorithm of another counter style, but alter other aspects.\",numeric:\"interprets the list of counter symbols as digits to a \\\"place-value\\\" numbering system, similar to the default 'decimal' counter style.\",symbolic:\"Cycles repeatedly through its provided symbols, doubling, tripling, etc. the symbols on each successive pass through the list.\",sideways:\"This value is equivalent to 'sideways-right' in 'vertical-rl' writing mode and equivalent to 'sideways-left' in 'vertical-lr' writing mode.\",\"sideways-right\":\"In vertical writing modes, this causes text to be set as if in a horizontal layout, but rotated 90° clockwise.\",upright:\"In vertical writing modes, characters from horizontal-only scripts are rendered upright, i.e. in their standard horizontal orientation.\",optimizeLegibility:\"Indicates that the user agent shall emphasize legibility over rendering speed and geometric precision.\",capitalize:\"Puts the first typographic letter unit of each word in titlecase.\",lowercase:\"Puts all letters in lowercase.\",uppercase:\"Puts all letters in uppercase.\",\"perspective()\":\"Specifies a perspective projection matrix.\",flat:\"All children of this element are rendered flattened into the 2D plane of the element.\",\"preserve-3d\":\"Flattening is not performed, so children maintain their position in 3D space.\",\"bidi-override\":\"Inside the element, reordering is strictly in sequence according to the 'direction' property; the implicit part of the bidirectional algorithm is ignored.\",embed:\"If the element is inline-level, this value opens an additional level of embedding with respect to the bidirectional algorithm. The direction of this embedding level is given by the 'direction' property.\",\"isolate-override\":\"This combines the isolation behavior of 'isolate' with the directional override behavior of 'bidi-override'\",plaintext:\"For the purposes of the Unicode bidirectional algorithm, the base directionality of each bidi paragraph for which the element forms the containing block is determined not by the element's computed 'direction'.\",\"U+26\":\"Ampersand.\",\"U+20-24F, U+2B0-2FF, U+370-4FF, U+1E00-1EFF, U+2000-20CF, U+2100-23FF, U+2500-26FF, U+E000-F8FF, U+FB00–FB4F\":\"WGL4 character set (Pan-European).\",\"U+20-17F, U+2B0-2FF, U+2000-206F, U+20A0-20CF, U+2100-21FF, U+2600-26FF\":\"The Multilingual European Subset No. 1. Latin. Covers ~44 languages.\",\"U+20-2FF, U+370-4FF, U+1E00-20CF, U+2100-23FF, U+2500-26FF, U+FB00-FB4F, U+FFF0-FFFD\":\"The Multilingual European Subset No. 2. Latin, Greek, and Cyrillic. Covers ~128 language.\",\"U+20-4FF, U+530-58F, U+10D0-10FF, U+1E00-23FF, U+2440-245F, U+2500-26FF, U+FB00-FB4F, U+FE20-FE2F, U+FFF0-FFFD\":\"The Multilingual European Subset No. 3. Covers all characters belonging to European scripts.\",\"U+00-7F\":\"Basic Latin (ASCII).\",\"U+80-FF\":\"Latin-1 Supplement. Accented characters for Western European languages, common punctuation characters, multiplication and division signs.\",\"U+100-17F\":\"Latin Extended-A. Accented characters for for Czech, Dutch, Polish, and Turkish.\",\"U+180-24F\":\"Latin Extended-B. Croatian, Slovenian, Romanian, Non-European and historic latin, Khoisan, Pinyin, Livonian, Sinology.\",\"U+1E00-1EFF\":\"Latin Extended Additional. Vietnamese, German captial sharp s, Medievalist, Latin general use.\",\"U+250-2AF\":\"International Phonetic Alphabet Extensions.\",\"U+370-3FF\":\"Greek and Coptic.\",\"U+1F00-1FFF\":\"Greek Extended. Accented characters for polytonic Greek.\",\"U+400-4FF\":\"Cyrillic.\",\"U+500-52F\":\"Cyrillic Supplement. Extra letters for Komi, Khanty, Chukchi, Mordvin, Kurdish, Aleut, Chuvash, Abkhaz, Azerbaijani, and Orok.\",\"U+00-52F, U+1E00-1FFF, U+2200–22FF\":\"Latin, Greek, Cyrillic, some punctuation and symbols.\",\"U+530–58F\":\"Armenian.\",\"U+590–5FF\":\"Hebrew.\",\"U+600–6FF\":\"Arabic.\",\"U+750–77F\":\"Arabic Supplement. Additional letters for African languages, Khowar, Torwali, Burushaski, and early Persian.\",\"U+8A0–8FF\":\"Arabic Extended-A. Additional letters for African languages, European and Central Asian languages, Rohingya, Tamazight, Arwi, and Koranic annotation signs.\",\"U+700–74F\":\"Syriac.\",\"U+900–97F\":\"Devanagari.\",\"U+980–9FF\":\"Bengali.\",\"U+A00–A7F\":\"Gurmukhi.\",\"U+A80–AFF\":\"Gujarati.\",\"U+B00–B7F\":\"Oriya.\",\"U+B80–BFF\":\"Tamil.\",\"U+C00–C7F\":\"Telugu.\",\"U+C80–CFF\":\"Kannada.\",\"U+D00–D7F\":\"Malayalam.\",\"U+D80–DFF\":\"Sinhala.\",\"U+118A0–118FF\":\"Warang Citi.\",\"U+E00–E7F\":\"Thai.\",\"U+1A20–1AAF\":\"Tai Tham.\",\"U+AA80–AADF\":\"Tai Viet.\",\"U+E80–EFF\":\"Lao.\",\"U+F00–FFF\":\"Tibetan.\",\"U+1000–109F\":\"Myanmar (Burmese).\",\"U+10A0–10FF\":\"Georgian.\",\"U+1200–137F\":\"Ethiopic.\",\"U+1380–139F\":\"Ethiopic Supplement. Extra Syllables for Sebatbeit, and Tonal marks\",\"U+2D80–2DDF\":\"Ethiopic Extended. Extra Syllables for Me'en, Blin, and Sebatbeit.\",\"U+AB00–AB2F\":\"Ethiopic Extended-A. Extra characters for Gamo-Gofa-Dawro, Basketo, and Gumuz.\",\"U+1780–17FF\":\"Khmer.\",\"U+1800–18AF\":\"Mongolian.\",\"U+1B80–1BBF\":\"Sundanese.\",\"U+1CC0–1CCF\":\"Sundanese Supplement. Punctuation.\",\"U+4E00–9FD5\":\"CJK (Chinese, Japanese, Korean) Unified Ideographs. Most common ideographs for modern Chinese and Japanese.\",\"U+3400–4DB5\":\"CJK Unified Ideographs Extension A. Rare ideographs.\",\"U+2F00–2FDF\":\"Kangxi Radicals.\",\"U+2E80–2EFF\":\"CJK Radicals Supplement. Alternative forms of Kangxi Radicals.\",\"U+1100–11FF\":\"Hangul Jamo.\",\"U+AC00–D7AF\":\"Hangul Syllables.\",\"U+3040–309F\":\"Hiragana.\",\"U+30A0–30FF\":\"Katakana.\",\"U+A5, U+4E00-9FFF, U+30??, U+FF00-FF9F\":\"Japanese Kanji, Hiragana and Katakana characters plus Yen/Yuan symbol.\",\"U+A4D0–A4FF\":\"Lisu.\",\"U+A000–A48F\":\"Yi Syllables.\",\"U+A490–A4CF\":\"Yi Radicals.\",\"U+2000-206F\":\"General Punctuation.\",\"U+3000–303F\":\"CJK Symbols and Punctuation.\",\"U+2070–209F\":\"Superscripts and Subscripts.\",\"U+20A0–20CF\":\"Currency Symbols.\",\"U+2100–214F\":\"Letterlike Symbols.\",\"U+2150–218F\":\"Number Forms.\",\"U+2190–21FF\":\"Arrows.\",\"U+2200–22FF\":\"Mathematical Operators.\",\"U+2300–23FF\":\"Miscellaneous Technical.\",\"U+E000-F8FF\":\"Private Use Area.\",\"U+FB00–FB4F\":\"Alphabetic Presentation Forms. Ligatures for latin, Armenian, and Hebrew.\",\"U+FB50–FDFF\":\"Arabic Presentation Forms-A. Contextual forms / ligatures for Persian, Urdu, Sindhi, Central Asian languages, etc, Arabic pedagogical symbols, word ligatures.\",\"U+1F600–1F64F\":\"Emoji: Emoticons.\",\"U+2600–26FF\":\"Emoji: Miscellaneous Symbols.\",\"U+1F300–1F5FF\":\"Emoji: Miscellaneous Symbols and Pictographs.\",\"U+1F900–1F9FF\":\"Emoji: Supplemental Symbols and Pictographs.\",\"U+1F680–1F6FF\":\"Emoji: Transport and Map Symbols.\",\"text-bottom\":\"Align the bottom of the box with the after-edge of the parent element's font.\",\"text-top\":\"Align the top of the box with the before-edge of the parent element's font.\",break:\"If the content fits within the CSS Region, then this property has no effect.\",pre:\"Sets 'white-space-collapsing' to 'preserve' and 'text-wrap' to 'none'.\",\"pre-line\":\"Sets 'white-space-collapsing' to 'preserve-breaks' and 'text-wrap' to 'normal'.\",\"pre-wrap\":\"Sets 'white-space-collapsing' to 'preserve' and 'text-wrap' to 'normal'.\",\"scroll-position\":\"Indicates that the author expects to animate or change the scroll position of the element in the near future.\",\"horizontal-tb\":\"Top-to-bottom block flow direction. The writing mode is horizontal.\",\"sideways-lr\":\"Left-to-right block flow direction. The writing mode is vertical, while the typographic mode is horizontal.\",\"sideways-rl\":\"Right-to-left block flow direction. The writing mode is vertical, while the typographic mode is horizontal.\",\"vertical-lr\":\"Left-to-right block flow direction. The writing mode is vertical.\",\"vertical-rl\":\"Right-to-left block flow direction. The writing mode is vertical.\"},Fr=vr(),Er={aliceblue:\"#f0f8ff\",antiquewhite:\"#faebd7\",aqua:\"#00ffff\",aquamarine:\"#7fffd4\",azure:\"#f0ffff\",beige:\"#f5f5dc\",bisque:\"#ffe4c4\",black:\"#000000\",blanchedalmond:\"#ffebcd\",blue:\"#0000ff\",blueviolet:\"#8a2be2\",brown:\"#a52a2a\",burlywood:\"#deb887\",cadetblue:\"#5f9ea0\",chartreuse:\"#7fff00\",chocolate:\"#d2691e\",coral:\"#ff7f50\",cornflowerblue:\"#6495ed\",cornsilk:\"#fff8dc\",crimson:\"#dc143c\",cyan:\"#00ffff\",darkblue:\"#00008b\",darkcyan:\"#008b8b\",darkgoldenrod:\"#b8860b\",darkgray:\"#a9a9a9\",darkgrey:\"#a9a9a9\",darkgreen:\"#006400\",darkkhaki:\"#bdb76b\",darkmagenta:\"#8b008b\",darkolivegreen:\"#556b2f\",darkorange:\"#ff8c00\",darkorchid:\"#9932cc\",darkred:\"#8b0000\",darksalmon:\"#e9967a\",darkseagreen:\"#8fbc8f\",darkslateblue:\"#483d8b\",darkslategray:\"#2f4f4f\",darkslategrey:\"#2f4f4f\",darkturquoise:\"#00ced1\",darkviolet:\"#9400d3\",deeppink:\"#ff1493\",deepskyblue:\"#00bfff\",dimgray:\"#696969\",dimgrey:\"#696969\",dodgerblue:\"#1e90ff\",firebrick:\"#b22222\",floralwhite:\"#fffaf0\",forestgreen:\"#228b22\",fuchsia:\"#ff00ff\",gainsboro:\"#dcdcdc\",ghostwhite:\"#f8f8ff\",gold:\"#ffd700\",goldenrod:\"#daa520\",gray:\"#808080\",grey:\"#808080\",green:\"#008000\",greenyellow:\"#adff2f\",honeydew:\"#f0fff0\",hotpink:\"#ff69b4\",indianred:\"#cd5c5c\",indigo:\"#4b0082\",ivory:\"#fffff0\",khaki:\"#f0e68c\",lavender:\"#e6e6fa\",lavenderblush:\"#fff0f5\",lawngreen:\"#7cfc00\",lemonchiffon:\"#fffacd\",lightblue:\"#add8e6\",lightcoral:\"#f08080\",lightcyan:\"#e0ffff\",lightgoldenrodyellow:\"#fafad2\",lightgray:\"#d3d3d3\",lightgrey:\"#d3d3d3\",lightgreen:\"#90ee90\",lightpink:\"#ffb6c1\",lightsalmon:\"#ffa07a\",lightseagreen:\"#20b2aa\",lightskyblue:\"#87cefa\",lightslategray:\"#778899\",lightslategrey:\"#778899\",lightsteelblue:\"#b0c4de\",lightyellow:\"#ffffe0\",lime:\"#00ff00\",limegreen:\"#32cd32\",linen:\"#faf0e6\",magenta:\"#ff00ff\",maroon:\"#800000\",mediumaquamarine:\"#66cdaa\",mediumblue:\"#0000cd\",mediumorchid:\"#ba55d3\",mediumpurple:\"#9370d8\",mediumseagreen:\"#3cb371\",mediumslateblue:\"#7b68ee\",mediumspringgreen:\"#00fa9a\",mediumturquoise:\"#48d1cc\",mediumvioletred:\"#c71585\",midnightblue:\"#191970\",mintcream:\"#f5fffa\",mistyrose:\"#ffe4e1\",moccasin:\"#ffe4b5\",navajowhite:\"#ffdead\",navy:\"#000080\",oldlace:\"#fdf5e6\",olive:\"#808000\",olivedrab:\"#6b8e23\",orange:\"#ffa500\",orangered:\"#ff4500\",orchid:\"#da70d6\",palegoldenrod:\"#eee8aa\",palegreen:\"#98fb98\",paleturquoise:\"#afeeee\",palevioletred:\"#d87093\",papayawhip:\"#ffefd5\",peachpuff:\"#ffdab9\",peru:\"#cd853f\",pink:\"#ffc0cb\",plum:\"#dda0dd\",powderblue:\"#b0e0e6\",purple:\"#800080\",red:\"#ff0000\",rebeccapurple:\"#663399\",rosybrown:\"#bc8f8f\",royalblue:\"#4169e1\",saddlebrown:\"#8b4513\",salmon:\"#fa8072\",sandybrown:\"#f4a460\",seagreen:\"#2e8b57\",seashell:\"#fff5ee\",sienna:\"#a0522d\",silver:\"#c0c0c0\",skyblue:\"#87ceeb\",slateblue:\"#6a5acd\",slategray:\"#708090\",slategrey:\"#708090\",snow:\"#fffafa\",springgreen:\"#00ff7f\",steelblue:\"#4682b4\",tan:\"#d2b48c\",teal:\"#008080\",thistle:\"#d8bfd8\",tomato:\"#ff6347\",turquoise:\"#40e0d0\",violet:\"#ee82ee\",wheat:\"#f5deb3\",white:\"#ffffff\",whitesmoke:\"#f5f5f5\",yellow:\"#ffff00\",yellowgreen:\"#9acd32\"},Tr={currentColor:\"The value of the 'color' property. The computed value of the 'currentColor' keyword is the computed value of the 'color' property. If the 'currentColor' keyword is set on the 'color' property itself, it is treated as 'color:inherit' at parse time.\",transparent:\"Fully transparent. This keyword can be considered a shorthand for rgba(0,0,0,0) which is its computed value.\"},Ir={bottom:\"Computes to ‘100%’ for the vertical position if one or two values are given, otherwise specifies the bottom edge as the origin for the next offset.\",center:\"Computes to ‘50%’ (‘left 50%’) for the horizontal position if the horizontal position is not otherwise specified, or ‘50%’ (‘top 50%’) for the vertical position if it is.\",left:\"Computes to ‘0%’ for the horizontal position if one or two values are given, otherwise specifies the left edge as the origin for the next offset.\",right:\"Computes to ‘100%’ for the horizontal position if one or two values are given, otherwise specifies the right edge as the origin for the next offset.\",top:\"Computes to ‘0%’ for the vertical position if one or two values are given, otherwise specifies the top edge as the origin for the next offset.\"},Ar={\"no-repeat\":\"Placed once and not repeated in this direction.\",repeat:\"Repeated in this direction as often as needed to cover the background painting area.\",\"repeat-x\":\"Computes to ‘repeat no-repeat’.\",\"repeat-y\":\"Computes to ‘no-repeat repeat’.\",round:\"Repeated as often as will fit within the background positioning area. If it doesn’t fit a whole number of times, it is rescaled so that it does.\",space:\"Repeated as often as will fit within the background positioning area without being clipped and then the images are spaced out to fill the area.\"},Pr={dashed:\"A series of square-ended dashes.\",dotted:\"A series of round dots.\",double:\"Two parallel solid lines with some space between them.\",groove:\"Looks as if it were carved in the canvas.\",hidden:\"Same as ‘none’, but has different behavior in the border conflict resolution rules for border-collapsed tables.\",inset:\"Looks as if the content on the inside of the border is sunken into the canvas.\",none:\"No border. Color and width are ignored.\",outset:\"Looks as if the content on the inside of the border is coming out of the canvas.\",ridge:\"Looks as if it were coming out of the canvas.\",solid:\"A single line segment.\"},Or=[\"medium\",\"thick\",\"thin\"],Nr={\"border-box\":\"The background is painted within (clipped to) the border box.\",\"content-box\":\"The background is painted within (clipped to) the content box.\",\"padding-box\":\"The background is painted within (clipped to) the padding box.\"},Lr={\"margin-box\":\"Uses the margin box as reference box.\",\"fill-box\":\"Uses the object bounding box as reference box.\",\"stroke-box\":\"Uses the stroke bounding box as reference box.\",\"view-box\":\"Uses the nearest SVG viewport as reference box.\"},Rr={initial:\"Represents the value specified as the property’s initial value.\",inherit:\"Represents the computed value of the property on the element’s parent.\",unset:\"Acts as either `inherit` or `initial`, depending on whether the property is inherited or not.\"},zr=[{func:\"rgb($red, $green, $blue)\",desc:Fr(\"css.builtin.rgb\",\"Creates a Color from red, green, and blue values.\")},{func:\"rgba($red, $green, $blue, $alpha)\",desc:Fr(\"css.builtin.rgba\",\"Creates a Color from red, green, blue, and alpha values.\")},{func:\"hsl($hue, $saturation, $lightness)\",desc:Fr(\"css.builtin.hsl\",\"Creates a Color from hue, saturation, and lightness values.\")},{func:\"hsla($hue, $saturation, $lightness, $alpha)\",desc:Fr(\"css.builtin.hsla\",\"Creates a Color from hue, saturation, lightness, and alpha values.\")}],Dr={\"url()\":\"Reference an image file by URL\",\"image()\":\"Provide image fallbacks and annotations.\",\"-webkit-image-set()\":\"Provide multiple resolutions. Remember to use unprefixed image-set() in addition.\",\"image-set()\":\"Provide multiple resolutions of an image and let the UA decide which is most appropriate in a given situation.\",\"-moz-element()\":\"Use an element in the document as an image. Remember to use unprefixed element() in addition.\",\"element()\":\"Use an element in the document as an image.\",\"cross-fade()\":\"Indicates the two images to be combined and how far along in the transition the combination is.\",\"-webkit-gradient()\":\"Deprecated. Use modern linear-gradient() or radial-gradient() instead.\",\"-webkit-linear-gradient()\":\"Linear gradient. Remember to use unprefixed version in addition.\",\"-moz-linear-gradient()\":\"Linear gradient. Remember to use unprefixed version in addition.\",\"-o-linear-gradient()\":\"Linear gradient. Remember to use unprefixed version in addition.\",\"linear-gradient()\":\"A linear gradient is created by specifying a straight gradient line, and then several colors placed along that line.\",\"-webkit-repeating-linear-gradient()\":\"Repeating Linear gradient. Remember to use unprefixed version in addition.\",\"-moz-repeating-linear-gradient()\":\"Repeating Linear gradient. Remember to use unprefixed version in addition.\",\"-o-repeating-linear-gradient()\":\"RepeatingLinear gradient. Remember to use unprefixed version in addition.\",\"repeating-linear-gradient()\":\"Same as linear-gradient, except the color-stops are repeated infinitely in both directions, with their positions shifted by multiples of the difference between the last specified color-stop’s position and the first specified color-stop’s position.\",\"-webkit-radial-gradient()\":\"Radial gradient. Remember to use unprefixed version in addition.\",\"-moz-radial-gradient()\":\"Radial gradient. Remember to use unprefixed version in addition.\",\"radial-gradient()\":\"Colors emerge from a single point and smoothly spread outward in a circular or elliptical shape.\",\"-webkit-repeating-radial-gradient()\":\"Repeating radial gradient. Remember to use unprefixed version in addition.\",\"-moz-repeating-radial-gradient()\":\"Repeating radial gradient. Remember to use unprefixed version in addition.\",\"repeating-radial-gradient()\":\"Same as radial-gradient, except the color-stops are repeated infinitely in both directions, with their positions shifted by multiples of the difference between the last specified color-stop’s position and the first specified color-stop’s position.\"},Ur={ease:\"Equivalent to cubic-bezier(0.25, 0.1, 0.25, 1.0).\",\"ease-in\":\"Equivalent to cubic-bezier(0.42, 0, 1.0, 1.0).\",\"ease-in-out\":\"Equivalent to cubic-bezier(0.42, 0, 0.58, 1.0).\",\"ease-out\":\"Equivalent to cubic-bezier(0, 0, 0.58, 1.0).\",linear:\"Equivalent to cubic-bezier(0.0, 0.0, 1.0, 1.0).\",\"step-end\":\"Equivalent to steps(1, end).\",\"step-start\":\"Equivalent to steps(1, start).\",\"steps()\":\"The first parameter specifies the number of intervals in the function. The second parameter, which is optional, is either the value “start” or “end”.\",\"cubic-bezier()\":\"Specifies a cubic-bezier curve. The four values specify points P1 and P2  of the curve as (x1, y1, x2, y2).\",\"cubic-bezier(0.6, -0.28, 0.735, 0.045)\":\"Ease-in Back. Overshoots.\",\"cubic-bezier(0.68, -0.55, 0.265, 1.55)\":\"Ease-in-out Back. Overshoots.\",\"cubic-bezier(0.175, 0.885, 0.32, 1.275)\":\"Ease-out Back. Overshoots.\",\"cubic-bezier(0.6, 0.04, 0.98, 0.335)\":\"Ease-in Circular. Based on half circle.\",\"cubic-bezier(0.785, 0.135, 0.15, 0.86)\":\"Ease-in-out Circular. Based on half circle.\",\"cubic-bezier(0.075, 0.82, 0.165, 1)\":\"Ease-out Circular. Based on half circle.\",\"cubic-bezier(0.55, 0.055, 0.675, 0.19)\":\"Ease-in Cubic. Based on power of three.\",\"cubic-bezier(0.645, 0.045, 0.355, 1)\":\"Ease-in-out Cubic. Based on power of three.\",\"cubic-bezier(0.215, 0.610, 0.355, 1)\":\"Ease-out Cubic. Based on power of three.\",\"cubic-bezier(0.95, 0.05, 0.795, 0.035)\":\"Ease-in Exponential. Based on two to the power ten.\",\"cubic-bezier(1, 0, 0, 1)\":\"Ease-in-out Exponential. Based on two to the power ten.\",\"cubic-bezier(0.19, 1, 0.22, 1)\":\"Ease-out Exponential. Based on two to the power ten.\",\"cubic-bezier(0.47, 0, 0.745, 0.715)\":\"Ease-in Sine.\",\"cubic-bezier(0.445, 0.05, 0.55, 0.95)\":\"Ease-in-out Sine.\",\"cubic-bezier(0.39, 0.575, 0.565, 1)\":\"Ease-out Sine.\",\"cubic-bezier(0.55, 0.085, 0.68, 0.53)\":\"Ease-in Quadratic. Based on power of two.\",\"cubic-bezier(0.455, 0.03, 0.515, 0.955)\":\"Ease-in-out Quadratic. Based on power of two.\",\"cubic-bezier(0.25, 0.46, 0.45, 0.94)\":\"Ease-out Quadratic. Based on power of two.\",\"cubic-bezier(0.895, 0.03, 0.685, 0.22)\":\"Ease-in Quartic. Based on power of four.\",\"cubic-bezier(0.77, 0, 0.175, 1)\":\"Ease-in-out Quartic. Based on power of four.\",\"cubic-bezier(0.165, 0.84, 0.44, 1)\":\"Ease-out Quartic. Based on power of four.\",\"cubic-bezier(0.755, 0.05, 0.855, 0.06)\":\"Ease-in Quintic. Based on power of five.\",\"cubic-bezier(0.86, 0, 0.07, 1)\":\"Ease-in-out Quintic. Based on power of five.\",\"cubic-bezier(0.23, 1, 0.320, 1)\":\"Ease-out Quintic. Based on power of five.\"},Mr={\"circle()\":\"Defines a circle.\",\"ellipse()\":\"Defines an ellipse.\",\"inset()\":\"Defines an inset rectangle.\",\"polygon()\":\"Defines a polygon.\"},jr={length:[\"em\",\"rem\",\"ex\",\"px\",\"cm\",\"mm\",\"in\",\"pt\",\"pc\",\"ch\",\"vw\",\"vh\",\"vmin\",\"vmax\"],angle:[\"deg\",\"rad\",\"grad\",\"turn\"],time:[\"ms\",\"s\"],frequency:[\"Hz\",\"kHz\"],resolution:[\"dpi\",\"dpcm\",\"dppx\"],percentage:[\"%\",\"fr\"]},Vr=[\"a\",\"abbr\",\"address\",\"area\",\"article\",\"aside\",\"audio\",\"b\",\"base\",\"bdi\",\"bdo\",\"blockquote\",\"body\",\"br\",\"button\",\"canvas\",\"caption\",\"cite\",\"code\",\"col\",\"colgroup\",\"data\",\"datalist\",\"dd\",\"del\",\"details\",\"dfn\",\"dialog\",\"div\",\"dl\",\"dt\",\"em\",\"embed\",\"fieldset\",\"figcaption\",\"figure\",\"footer\",\"form\",\"h1\",\"h2\",\"h3\",\"h4\",\"h5\",\"h6\",\"head\",\"header\",\"hgroup\",\"hr\",\"html\",\"i\",\"iframe\",\"img\",\"input\",\"ins\",\"kbd\",\"keygen\",\"label\",\"legend\",\"li\",\"link\",\"main\",\"map\",\"mark\",\"menu\",\"menuitem\",\"meta\",\"meter\",\"nav\",\"noscript\",\"object\",\"ol\",\"optgroup\",\"option\",\"output\",\"p\",\"param\",\"picture\",\"pre\",\"progress\",\"q\",\"rb\",\"rp\",\"rt\",\"rtc\",\"ruby\",\"s\",\"samp\",\"script\",\"section\",\"select\",\"small\",\"source\",\"span\",\"strong\",\"style\",\"sub\",\"summary\",\"sup\",\"table\",\"tbody\",\"td\",\"template\",\"textarea\",\"tfoot\",\"th\",\"thead\",\"time\",\"title\",\"tr\",\"track\",\"u\",\"ul\",\"let\",\"video\",\"wbr\"],Br=[\"circle\",\"clipPath\",\"cursor\",\"defs\",\"desc\",\"ellipse\",\"feBlend\",\"feColorMatrix\",\"feComponentTransfer\",\"feComposite\",\"feConvolveMatrix\",\"feDiffuseLighting\",\"feDisplacementMap\",\"feDistantLight\",\"feDropShadow\",\"feFlood\",\"feFuncA\",\"feFuncB\",\"feFuncG\",\"feFuncR\",\"feGaussianBlur\",\"feImage\",\"feMerge\",\"feMergeNode\",\"feMorphology\",\"feOffset\",\"fePointLight\",\"feSpecularLighting\",\"feSpotLight\",\"feTile\",\"feTurbulence\",\"filter\",\"foreignObject\",\"g\",\"hatch\",\"hatchpath\",\"image\",\"line\",\"linearGradient\",\"marker\",\"mask\",\"mesh\",\"meshpatch\",\"meshrow\",\"metadata\",\"mpath\",\"path\",\"pattern\",\"polygon\",\"polyline\",\"radialGradient\",\"rect\",\"set\",\"solidcolor\",\"stop\",\"svg\",\"switch\",\"symbol\",\"text\",\"textPath\",\"tspan\",\"use\",\"view\"];var Kr=48,qr=57,Wr=65,$r=97,Gr=102;function Hr(e){return e<Kr?0:e<=qr?e-Kr:(e<$r&&(e+=$r-Wr),e>=$r&&e<=Gr?e-$r+10:0)}function Yr(e){if(\"#\"!==e[0])return null;switch(e.length){case 4:return{red:17*Hr(e.charCodeAt(1))/255,green:17*Hr(e.charCodeAt(2))/255,blue:17*Hr(e.charCodeAt(3))/255,alpha:1};case 5:return{red:17*Hr(e.charCodeAt(1))/255,green:17*Hr(e.charCodeAt(2))/255,blue:17*Hr(e.charCodeAt(3))/255,alpha:17*Hr(e.charCodeAt(4))/255};case 7:return{red:(16*Hr(e.charCodeAt(1))+Hr(e.charCodeAt(2)))/255,green:(16*Hr(e.charCodeAt(3))+Hr(e.charCodeAt(4)))/255,blue:(16*Hr(e.charCodeAt(5))+Hr(e.charCodeAt(6)))/255,alpha:1};case 9:return{red:(16*Hr(e.charCodeAt(1))+Hr(e.charCodeAt(2)))/255,green:(16*Hr(e.charCodeAt(3))+Hr(e.charCodeAt(4)))/255,blue:(16*Hr(e.charCodeAt(5))+Hr(e.charCodeAt(6)))/255,alpha:(16*Hr(e.charCodeAt(7))+Hr(e.charCodeAt(8)))/255}}return null}function Jr(e){if(e.type===cn.HexColorValue)return Yr(e.getText());if(e.type===cn.Function){var t=e,n=t.getName(),r=t.getArguments().getChildren();if(!n||r.length<3||r.length>4)return null;try{var i=4===r.length?Xr(r[3],1):1;if(\"rgb\"===n||\"rgba\"===n)return{red:Xr(r[0],255),green:Xr(r[1],255),blue:Xr(r[2],255),alpha:i};if(\"hsl\"===n||\"hsla\"===n)return function(e,t,n,r){if(void 0===r&&(r=1),0===t)return{red:n,green:n,blue:n,alpha:r};var i=function(e,t,n){for(;n<0;)n+=6;for(;n>=6;)n-=6;return n<1?(t-e)*n+e:n<3?t:n<4?(t-e)*(4-n)+e:e},o=n<=.5?n*(t+1):n+t-n*t,s=2*n-o;return{red:i(s,o,2+(e/=60)),green:i(s,o,e),blue:i(s,o,e-2),alpha:r}}(function(e){var t=e.getText();if(t.match(/^([-+]?[0-9]*\\.?[0-9]+)(deg)?$/))return parseFloat(t)%360;throw new Error}(r[0]),Xr(r[1],100),Xr(r[2],100),i)}catch(e){return null}}else if(e.type===cn.Identifier){if(e.parent&&e.parent.type!==cn.Term)return null;var o=e.parent;if(o.parent&&o.parent.type===cn.BinaryExpression){var s=o.parent;if(s.parent&&s.parent.type===cn.ListEntry&&s.parent.key===s)return null}var a=e.getText().toLowerCase();if(\"none\"===a)return null;var l=Er[a];if(l)return Yr(l)}return null}function Xr(e,t){var n=e.getText().match(/^([-+]?[0-9]*\\.?[0-9]+)(%?)$/);if(n){n[2]&&(t=100);var r=parseFloat(n[1])/t;if(r>=0&&r<=1)return r}throw new Error}function Qr(e){return!!e&&(e=e.toLowerCase(),li().hasOwnProperty(e))}function Zr(e){if(e){e=e.toLowerCase();var t=li()[e];return t&&\"standard\"===t.status}return!1}function ei(e){return e.browsers.count>1}function ti(e){if(!e.description||\"\"===e.description)return null;var t=\"\";e.data&&e.data.status&&(t+=function(e){switch(e){case\"e\":return\"⚠️ Property is experimental. Be cautious when using it.️\\n\\n\";case\"n\":return\"🚨️ Property is nonstandard. Avoid using it.\\n\\n\";case\"o\":return\"🚨️️️ Property is obsolete. Avoid using it.\\n\\n\";default:return\"\"}}(e.data.status)),t+=e.description;var n=ni(e.browsers);return n&&(t+=\"\\n(\"+n+\")\"),e.data&&e.data.syntax&&(t+=\"\\n\\nSyntax: \"+e.data.syntax),t}function ni(e){var t=\"\";if(!e||e.all||0===e.count)return null;for(var n in fi)if(\"string\"==typeof e[n]){t.length>0&&(t+=\", \"),t+=fi[n];var r=e[n];r.length>0&&(t=t+\" \"+r)}return t}function ri(e){var t={all:!1,count:0,onCodeComplete:!1},n=0;if(e)for(var r=0,i=e.split(\",\");r<i.length;r++){var o=i[r];if(\"all\"===(o=o.trim()))t.all=!0,n=Number.MAX_VALUE;else if(\"none\"!==o)for(var s in fi)0===o.indexOf(s)&&(t[s]=o.substring(s.length).trim(),n++)}else t.all=!0,n=Number.MAX_VALUE;return t.count=n,t.onCodeComplete=n>0,t}var ii,oi=function(){function e(e){this.data=e}return Object.defineProperty(e.prototype,\"name\",{get:function(){return this.data.name},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"description\",{get:function(){return this.data.desc||_r[this.data.name]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"browsers\",{get:function(){return this.browserEntry||(this.browserEntry=ri(this.data.browsers)),this.browserEntry},enumerable:!0,configurable:!0}),e}(),si=function(){function e(e){this.data=e}return Object.defineProperty(e.prototype,\"name\",{get:function(){return this.data.name},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"description\",{get:function(){return this.data.desc||_r[this.data.name]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"browsers\",{get:function(){return this.browserEntry||(this.browserEntry=ri(this.data.browsers)),this.browserEntry},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"restrictions\",{get:function(){return this.data.restriction?this.data.restriction.split(\",\").map(function(e){return e.trim()}):[]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"status\",{get:function(){return function(e){switch(e){case\"e\":return\"experimental\";case\"n\":return\"nonstandard\";case\"o\":return\"obsolete\";default:return\"standard\"}}(this.data.status)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"values\",{get:function(){return this.data.values?Array.isArray(this.data.values)?this.data.values.map(function(e){return new oi(e)}):[new oi(this.data.values.value)]:[]},enumerable:!0,configurable:!0}),e}(),ai=Sr.properties;function li(){if(!ii){ii={};for(var e=0;e<ai.length;e++){var t=ai[e];ii[t.name]=new si(t)}}return ii}var ci,hi=Sr.atdirectives;var di,ui=Sr.pseudoelements;var mi,pi=Sr.pseudoclasses;var fi={E:\"Edge\",FF:\"Firefox\",S:\"Safari\",C:\"Chrome\",IE:\"IE\",O:\"Opera\"},gi=function(){function e(e){void 0===e&&(e=new dn),this.keyframeRegex=/^@(\\-(webkit|ms|moz|o)\\-)?keyframes$/i,this.scanner=e,this.token=null,this.prevToken=null}return e.prototype.peekIdent=function(e){return ft.Ident===this.token.type&&e.length===this.token.text.length&&e===this.token.text.toLowerCase()},e.prototype.peekKeyword=function(e){return ft.AtKeyword===this.token.type&&e.length===this.token.text.length&&e===this.token.text.toLowerCase()},e.prototype.peekDelim=function(e){return ft.Delim===this.token.type&&e===this.token.text},e.prototype.peek=function(e){return e===this.token.type},e.prototype.peekRegExp=function(e,t){return e===this.token.type&&t.test(this.token.text)},e.prototype.hasWhitespace=function(){return this.prevToken&&this.prevToken.offset+this.prevToken.len!==this.token.offset},e.prototype.consumeToken=function(){this.prevToken=this.token,this.token=this.scanner.scan()},e.prototype.mark=function(){return{prev:this.prevToken,curr:this.token,pos:this.scanner.pos()}},e.prototype.restoreAtMark=function(e){this.prevToken=e.prev,this.token=e.curr,this.scanner.goBackTo(e.pos)},e.prototype.try=function(e){var t=this.mark(),n=e();return n||(this.restoreAtMark(t),null)},e.prototype.acceptOneKeyword=function(e){if(ft.AtKeyword===this.token.type)for(var t=0,n=e;t<n.length;t++){var r=n[t];if(r.length===this.token.text.length&&r===this.token.text.toLowerCase())return this.consumeToken(),!0}return!1},e.prototype.accept=function(e){return e===this.token.type&&(this.consumeToken(),!0)},e.prototype.acceptIdent=function(e){return!!this.peekIdent(e)&&(this.consumeToken(),!0)},e.prototype.acceptKeyword=function(e){return!!this.peekKeyword(e)&&(this.consumeToken(),!0)},e.prototype.acceptDelim=function(e){return!!this.peekDelim(e)&&(this.consumeToken(),!0)},e.prototype.acceptUnquotedString=function(){var e=this.scanner.pos();this.scanner.goBackTo(this.token.offset);var t=this.scanner.scanUnquotedString();return t?(this.token=t,this.consumeToken(),!0):(this.scanner.goBackTo(e),!1)},e.prototype.resync=function(e,t){for(;;){if(e&&-1!==e.indexOf(this.token.type))return this.consumeToken(),!0;if(t&&-1!==t.indexOf(this.token.type))return!0;if(this.token.type===ft.EOF)return!1;this.token=this.scanner.scan()}},e.prototype.createNode=function(e){return new gn(this.token.offset,this.token.len,e)},e.prototype.create=function(e){var t=Object.create(e.prototype);return e.apply(t,[this.token.offset,this.token.len]),t},e.prototype.finish=function(e,t,n,r){if(!(e instanceof bn)&&(t&&this.markError(e,t,n,r),null!==this.prevToken)){var i=this.prevToken.offset+this.prevToken.len;e.length=i>e.offset?i-e.offset:0}return e},e.prototype.markError=function(e,t,n,r){this.token!==this.lastErrorToken&&(e.addIssue(new br(e,t,fn.Error,null,this.token.offset,this.token.len)),this.lastErrorToken=this.token),(n||r)&&this.resync(n,r)},e.prototype.parseStylesheet=function(e){var t=e.version;return this.internalParse(e.getText(),this._parseStylesheet,function(n,r){if(e.version!==t)throw new Error(\"Underlying model has changed, AST is no longer valid\");return e.getText().substr(n,r)})},e.prototype.internalParse=function(e,t,n){this.scanner.setSource(e),this.token=this.scanner.scan();var r=t.bind(this)();return r&&(r.textProvider=n||function(t,n){return e.substr(t,n)}),r},e.prototype._parseStylesheet=function(){var e=this.create(wn);e.addChild(this._parseCharset());var t=!1;do{var n=!1;do{n=!1;var r=this._parseStylesheetStatement();for(r&&(e.addChild(r),n=!0,t=!1,this.peek(ft.EOF)||!this._needsSemicolonAfter(r)||this.accept(ft.SemiColon)||this.markError(e,Cr.SemiColonExpected));this.accept(ft.SemiColon)||this.accept(ft.CDO)||this.accept(ft.CDC);)n=!0,t=!1}while(n);if(this.peek(ft.EOF))break;t||(this.peek(ft.AtKeyword)?this.markError(e,Cr.UnknownAtRule):this.markError(e,Cr.RuleOrSelectorExpected),t=!0),this.consumeToken()}while(!this.peek(ft.EOF));return this.finish(e)},e.prototype._parseStylesheetStatement=function(){return this.peek(ft.AtKeyword)?this._parseStylesheetAtStatement():this._parseRuleset(!1)},e.prototype._parseStylesheetAtStatement=function(){return this._parseImport()||this._parseMedia()||this._parsePage()||this._parseFontFace()||this._parseKeyframe()||this._parseSupports()||this._parseViewPort()||this._parseNamespace()||this._parseDocument()||this._parseUnknownAtRule()},e.prototype._tryParseRuleset=function(e){var t=this.mark();if(this._parseSelector(e)){for(;this.accept(ft.Comma)&&this._parseSelector(e););if(this.accept(ft.CurlyL))return this.restoreAtMark(t),this._parseRuleset(e)}return this.restoreAtMark(t),null},e.prototype._parseRuleset=function(e){void 0===e&&(e=!1);var t=this.create(kn);if(!t.getSelectors().addChild(this._parseSelector(e)))return null;for(;this.accept(ft.Comma)&&t.getSelectors().addChild(this._parseSelector(e)););return this._parseBody(t,this._parseRuleSetDeclaration.bind(this))},e.prototype._parseRuleSetDeclaration=function(){return this._parseAtApply()||this._tryParseCustomPropertyDeclaration()||this._parseDeclaration()},e.prototype._parseAtApply=function(){if(!this.peekKeyword(\"@apply\"))return null;var e=this.create(_n);return this.consumeToken(),e.setIdentifier(this._parseIdent([hn.Variable]))?this.finish(e):this.finish(e,Cr.IdentifierExpected)},e.prototype._needsSemicolonAfter=function(e){switch(e.type){case cn.Keyframe:case cn.ViewPort:case cn.Media:case cn.Ruleset:case cn.Namespace:case cn.If:case cn.For:case cn.Each:case cn.While:case cn.MixinDeclaration:case cn.FunctionDeclaration:return!1;case cn.VariableDeclaration:case cn.ExtendsReference:case cn.MixinContent:case cn.ReturnStatement:case cn.MediaQuery:case cn.Debug:case cn.Import:case cn.AtApplyRule:case cn.CustomPropertyDeclaration:return!0;case cn.MixinReference:return!e.getContent();case cn.Declaration:return!e.getNestedProperties()}return!1},e.prototype._parseDeclarations=function(e){var t=this.create(vn);if(!this.accept(ft.CurlyL))return null;for(var n=e();t.addChild(n)&&!this.peek(ft.CurlyR);){if(this._needsSemicolonAfter(n)&&!this.accept(ft.SemiColon))return this.finish(t,Cr.SemiColonExpected,[ft.SemiColon,ft.CurlyR]);for(;this.accept(ft.SemiColon););n=e()}return this.accept(ft.CurlyR)?this.finish(t):this.finish(t,Cr.RightCurlyExpected,[ft.CurlyR,ft.SemiColon])},e.prototype._parseBody=function(e,t){return e.setDeclarations(this._parseDeclarations(t))?this.finish(e):this.finish(e,Cr.LeftCurlyExpected,[ft.CurlyR,ft.SemiColon])},e.prototype._parseSelector=function(e){var t=this.create(Cn),n=!1;for(e&&(n=t.addChild(this._parseCombinator()));t.addChild(this._parseSimpleSelector());)n=!0,t.addChild(this._parseCombinator());return n?this.finish(t):null},e.prototype._parseDeclaration=function(e){var t=this.create(In);return t.setProperty(this._parseProperty())?this.accept(ft.Colon)?(t.colonPosition=this.prevToken.offset,t.setValue(this._parseExpr())?(t.addChild(this._parsePrio()),this.peek(ft.SemiColon)&&(t.semicolonPosition=this.token.offset),this.finish(t)):this.finish(t,Cr.PropertyValueExpected)):this.finish(t,Cr.ColonExpected,[ft.Colon],e):null},e.prototype._tryParseCustomPropertyDeclaration=function(){if(!this.peekRegExp(ft.Ident,/^--/))return null;var e=this.create(En);if(!e.setProperty(this._parseProperty()))return null;if(!this.accept(ft.Colon))return this.finish(e,Cr.ColonExpected,[ft.Colon]);e.colonPosition=this.prevToken.offset;var t=this.mark();if(this.peek(ft.CurlyL)){var n=this.create(Tn),r=this._parseDeclarations(this._parseRuleSetDeclaration.bind(this));if(n.setDeclarations(r)&&!r.isErroneous(!0)&&(n.addChild(this._parsePrio()),this.peek(ft.SemiColon)))return this.finish(n),e.setPropertySet(n),e.semicolonPosition=this.token.offset,this.finish(e);this.restoreAtMark(t)}var i=this._parseExpr();return i&&!i.isErroneous(!0)&&(this._parsePrio(),this.peek(ft.SemiColon))?(e.setValue(i),e.semicolonPosition=this.token.offset,this.finish(e)):(this.restoreAtMark(t),e.addChild(this._parseCustomPropertyValue()),e.addChild(this._parsePrio()),this.token.offset===e.colonPosition+1?this.finish(e,Cr.PropertyValueExpected):this.finish(e))},e.prototype._parseCustomPropertyValue=function(){var e=this.create(gn),t=function(){return 0===n&&0===r&&0===i},n=0,r=0,i=0;e:for(;;){switch(this.token.type){case ft.SemiColon:case ft.Exclamation:if(t())break e;break;case ft.CurlyL:n++;break;case ft.CurlyR:if(--n<0){if(0===r&&0===i)break e;return this.finish(e,Cr.LeftCurlyExpected)}break;case ft.ParenthesisL:r++;break;case ft.ParenthesisR:if(--r<0)return this.finish(e,Cr.LeftParenthesisExpected);break;case ft.BracketL:i++;break;case ft.BracketR:if(--i<0)return this.finish(e,Cr.LeftSquareBracketExpected);break;case ft.BadString:break e;case ft.EOF:var o=Cr.RightCurlyExpected;return i>0?o=Cr.RightSquareBracketExpected:r>0&&(o=Cr.RightParenthesisExpected),this.finish(e,o)}this.consumeToken()}return this.finish(e)},e.prototype._tryToParseDeclaration=function(){var e=this.mark();return this._parseProperty()&&this.accept(ft.Colon)?(this.restoreAtMark(e),this._parseDeclaration()):(this.restoreAtMark(e),null)},e.prototype._parseProperty=function(){var e=this.create(An),t=this.mark();return(this.acceptDelim(\"*\")||this.acceptDelim(\"_\"))&&this.hasWhitespace()?(this.restoreAtMark(t),null):e.setIdentifier(this._parsePropertyIdentifier())?this.finish(e):null},e.prototype._parsePropertyIdentifier=function(){return this._parseIdent()},e.prototype._parseCharset=function(){if(!this.peek(ft.Charset))return null;var e=this.create(gn);return this.consumeToken(),this.accept(ft.String)?this.accept(ft.SemiColon)?this.finish(e):this.finish(e,Cr.SemiColonExpected):this.finish(e,Cr.IdentifierExpected)},e.prototype._parseImport=function(){if(!this.peekKeyword(\"@import\"))return null;var e=this.create(Wn);return this.consumeToken(),e.addChild(this._parseURILiteral())||e.addChild(this._parseStringLiteral())?(this.peek(ft.SemiColon)||this.peek(ft.EOF)||e.setMedialist(this._parseMediaQueryList()),this.finish(e)):this.finish(e,Cr.URIOrStringExpected)},e.prototype._parseNamespace=function(){if(!this.peekKeyword(\"@namespace\"))return null;var e=this.create($n);return this.consumeToken(),e.addChild(this._parseURILiteral())||(e.addChild(this._parseIdent()),e.addChild(this._parseURILiteral())||e.addChild(this._parseStringLiteral()))?this.accept(ft.SemiColon)?this.finish(e):this.finish(e,Cr.SemiColonExpected):this.finish(e,Cr.URIExpected,[ft.SemiColon])},e.prototype._parseFontFace=function(){if(!this.peekKeyword(\"@font-face\"))return null;var e=this.create(Vn);return this.consumeToken(),this._parseBody(e,this._parseRuleSetDeclaration.bind(this))},e.prototype._parseViewPort=function(){if(!this.peekKeyword(\"@-ms-viewport\")&&!this.peekKeyword(\"@-o-viewport\")&&!this.peekKeyword(\"@viewport\"))return null;var e=this.create(jn);return this.consumeToken(),this._parseBody(e,this._parseRuleSetDeclaration.bind(this))},e.prototype._parseKeyframe=function(){if(!this.peekRegExp(ft.AtKeyword,this.keyframeRegex))return null;var e=this.create(Kn),t=this.create(gn);return this.consumeToken(),e.setKeyword(this.finish(t)),\"@-ms-keyframes\"===t.getText()&&this.markError(t,Cr.UnknownKeyword),e.setIdentifier(this._parseKeyframeIdent())?this._parseBody(e,this._parseKeyframeSelector.bind(this)):this.finish(e,Cr.IdentifierExpected,[ft.CurlyR])},e.prototype._parseKeyframeIdent=function(){return this._parseIdent([hn.Keyframe])},e.prototype._parseKeyframeSelector=function(){var e=this.create(qn);if(!e.addChild(this._parseIdent())&&!this.accept(ft.Percentage))return null;for(;this.accept(ft.Comma);)if(!e.addChild(this._parseIdent())&&!this.accept(ft.Percentage))return this.finish(e,Cr.PercentageExpected);return this._parseBody(e,this._parseRuleSetDeclaration.bind(this))},e.prototype._tryParseKeyframeSelector=function(){var e=this.create(qn),t=this.mark();if(!e.addChild(this._parseIdent())&&!this.accept(ft.Percentage))return null;for(;this.accept(ft.Comma);)if(!e.addChild(this._parseIdent())&&!this.accept(ft.Percentage))return this.restoreAtMark(t),null;return this.peek(ft.CurlyL)?this._parseBody(e,this._parseRuleSetDeclaration.bind(this)):(this.restoreAtMark(t),null)},e.prototype._parseSupports=function(e){if(void 0===e&&(e=!1),!this.peekKeyword(\"@supports\"))return null;var t=this.create(Hn);return this.consumeToken(),t.addChild(this._parseSupportsCondition()),this._parseBody(t,this._parseSupportsDeclaration.bind(this,e))},e.prototype._parseSupportsDeclaration=function(e){return void 0===e&&(e=!1),e&&(this._tryParseRuleset(e)||this._tryToParseDeclaration())||this._parseStylesheetStatement()},e.prototype._parseSupportsCondition=function(){var e=this.create(Qn);if(this.acceptIdent(\"not\"))e.addChild(this._parseSupportsConditionInParens());else if(e.addChild(this._parseSupportsConditionInParens()),this.peekRegExp(ft.Ident,/^(and|or)$/i))for(var t=this.token.text.toLowerCase();this.acceptIdent(t);)e.addChild(this._parseSupportsConditionInParens());return this.finish(e)},e.prototype._parseSupportsConditionInParens=function(){var e=this.create(Qn);if(this.accept(ft.ParenthesisL))return e.lParent=this.prevToken.offset,e.addChild(this._tryToParseDeclaration())||this._parseSupportsCondition()?this.accept(ft.ParenthesisR)?(e.rParent=this.prevToken.offset,this.finish(e)):this.finish(e,Cr.RightParenthesisExpected,[ft.ParenthesisR],[]):this.finish(e,Cr.ConditionExpected);if(this.peek(ft.Ident)){var t=this.mark();if(this.consumeToken(),!this.hasWhitespace()&&this.accept(ft.ParenthesisL)){for(var n=1;this.token.type!==ft.EOF&&0!==n;)this.token.type===ft.ParenthesisL?n++:this.token.type===ft.ParenthesisR&&n--,this.consumeToken();return this.finish(e)}this.restoreAtMark(t)}return this.finish(e,Cr.LeftParenthesisExpected,[],[ft.ParenthesisL])},e.prototype._parseMediaDeclaration=function(e){return void 0===e&&(e=!1),this._tryParseRuleset(e)||this._tryToParseDeclaration()||this._parseStylesheetStatement()},e.prototype._parseMedia=function(e){if(void 0===e&&(e=!1),!this.peekKeyword(\"@media\"))return null;var t=this.create(Gn);return this.consumeToken(),t.addChild(this._parseMediaQueryList())?this._parseBody(t,this._parseMediaDeclaration.bind(this,e)):this.finish(t,Cr.MediaQueryExpected)},e.prototype._parseMediaQueryList=function(){var e=this.create(Jn);if(!e.addChild(this._parseMediaQuery([ft.CurlyL])))return this.finish(e,Cr.MediaQueryExpected);for(;this.accept(ft.Comma);)if(!e.addChild(this._parseMediaQuery([ft.CurlyL])))return this.finish(e,Cr.MediaQueryExpected);return this.finish(e)},e.prototype._parseMediaQuery=function(e){var t=this.create(Xn),n=!0,r=!1;if(!this.peek(ft.ParenthesisL)){if(this.acceptIdent(\"only\")||this.acceptIdent(\"not\"),!t.addChild(this._parseIdent()))return null;r=!0,n=this.acceptIdent(\"and\")}for(;n;){if(!this.accept(ft.ParenthesisL))return r?this.finish(t,Cr.LeftParenthesisExpected,[],e):null;if(!t.addChild(this._parseMediaFeatureName()))return this.finish(t,Cr.IdentifierExpected,[],e);if(this.accept(ft.Colon)&&!t.addChild(this._parseExpr()))return this.finish(t,Cr.TermExpected,[],e);if(!this.accept(ft.ParenthesisR))return this.finish(t,Cr.RightParenthesisExpected,[],e);n=this.acceptIdent(\"and\")}return this.finish(t)},e.prototype._parseMediaFeatureName=function(){return this._parseIdent()},e.prototype._parseMedium=function(){var e=this.create(gn);return e.addChild(this._parseIdent())?this.finish(e):null},e.prototype._parsePageDeclaration=function(){return this._parsePageMarginBox()||this._parseRuleSetDeclaration()},e.prototype._parsePage=function(){if(!this.peekKeyword(\"@page\"))return null;var e=this.create(Zn);if(this.consumeToken(),e.addChild(this._parsePageSelector()))for(;this.accept(ft.Comma);)if(!e.addChild(this._parsePageSelector()))return this.finish(e,Cr.IdentifierExpected);return this._parseBody(e,this._parsePageDeclaration.bind(this))},e.prototype._parsePageMarginBox=function(){if(!this.peek(ft.AtKeyword))return null;var e=this.create(er);return this.acceptOneKeyword([\"@bottom-center\",\"@bottom-left\",\"@bottom-left-corner\",\"@bottom-right\",\"@bottom-right-corner\",\"@left-bottom\",\"@left-middle\",\"@left-top\",\"@right-bottom\",\"@right-middle\",\"@right-top\",\"@top-center\",\"@top-left\",\"@top-left-corner\",\"@top-right\",\"@top-right-corner\"])||this.markError(e,Cr.UnknownAtRule,[],[ft.CurlyL]),this._parseBody(e,this._parseRuleSetDeclaration.bind(this))},e.prototype._parsePageSelector=function(){if(!this.peek(ft.Ident)&&!this.peek(ft.Colon))return null;var e=this.create(gn);return e.addChild(this._parseIdent()),this.accept(ft.Colon)&&!e.addChild(this._parseIdent())?this.finish(e,Cr.IdentifierExpected):this.finish(e)},e.prototype._parseDocument=function(){if(!this.peekKeyword(\"@-moz-document\"))return null;var e=this.create(Yn);return this.consumeToken(),this.resync([],[ft.CurlyL]),this._parseBody(e,this._parseStylesheetStatement.bind(this))},e.prototype._parseUnknownAtRule=function(){var e=this.create(mr);e.addChild(this._parseUnknownAtRuleName());var t=0,n=0,r=0,i=0;e:for(;;){switch(this.token.type){case ft.SemiColon:if(0===n&&0===r&&0===i)break e;break;case ft.EOF:return n>0?this.finish(e,Cr.RightCurlyExpected):i>0?this.finish(e,Cr.RightSquareBracketExpected):r>0?this.finish(e,Cr.RightParenthesisExpected):this.finish(e);case ft.CurlyL:t++,n++;break;case ft.CurlyR:if(n--,t>0&&0===n){if(this.consumeToken(),i>0)return this.finish(e,Cr.RightSquareBracketExpected);if(r>0)return this.finish(e,Cr.RightParenthesisExpected);break e}if(n<0){if(0===r&&0===i)break e;return this.finish(e,Cr.LeftCurlyExpected)}break;case ft.ParenthesisL:r++;break;case ft.ParenthesisR:if(--r<0)return this.finish(e,Cr.LeftParenthesisExpected);break;case ft.BracketL:i++;break;case ft.BracketR:if(--i<0)return this.finish(e,Cr.LeftSquareBracketExpected)}this.consumeToken()}return e},e.prototype._parseUnknownAtRuleName=function(){var e=this.create(gn);return this.accept(ft.AtKeyword)?this.finish(e):e},e.prototype._parseOperator=function(){if(this.peekDelim(\"/\")||this.peekDelim(\"*\")||this.peekDelim(\"+\")||this.peekDelim(\"-\")||this.peek(ft.Dashmatch)||this.peek(ft.Includes)||this.peek(ft.SubstringOperator)||this.peek(ft.PrefixOperator)||this.peek(ft.SuffixOperator)||this.peekDelim(\"=\")){var e=this.createNode(cn.Operator);return this.consumeToken(),this.finish(e)}return null},e.prototype._parseUnaryOperator=function(){if(!this.peekDelim(\"+\")&&!this.peekDelim(\"-\"))return null;var e=this.create(gn);return this.consumeToken(),this.finish(e)},e.prototype._parseCombinator=function(){if(this.peekDelim(\">\")){var e=this.create(gn);this.consumeToken();var t=this.mark();if(!this.hasWhitespace()&&this.acceptDelim(\">\")){if(!this.hasWhitespace()&&this.acceptDelim(\">\"))return e.type=cn.SelectorCombinatorShadowPiercingDescendant,this.finish(e);this.restoreAtMark(t)}return e.type=cn.SelectorCombinatorParent,this.finish(e)}if(this.peekDelim(\"+\")){e=this.create(gn);return this.consumeToken(),e.type=cn.SelectorCombinatorSibling,this.finish(e)}if(this.peekDelim(\"~\")){e=this.create(gn);return this.consumeToken(),e.type=cn.SelectorCombinatorAllSiblings,this.finish(e)}if(!this.peekDelim(\"/\"))return null;e=this.create(gn);this.consumeToken();t=this.mark();if(!this.hasWhitespace()&&this.acceptIdent(\"deep\")&&!this.hasWhitespace()&&this.acceptDelim(\"/\"))return e.type=cn.SelectorCombinatorShadowPiercingDescendant,this.finish(e);this.restoreAtMark(t)},e.prototype._parseSimpleSelector=function(){var e=this.create(Sn),t=0;for(e.addChild(this._parseElementName())&&t++;(0===t||!this.hasWhitespace())&&e.addChild(this._parseSimpleSelectorBody());)t++;return t>0?this.finish(e):null},e.prototype._parseSimpleSelectorBody=function(){return this._parsePseudo()||this._parseHash()||this._parseClass()||this._parseAttrib()},e.prototype._parseSelectorIdent=function(){return this._parseIdent()},e.prototype._parseHash=function(){if(!this.peek(ft.Hash)&&!this.peekDelim(\"#\"))return null;var e=this.createNode(cn.IdentifierSelector);if(this.acceptDelim(\"#\")){if(this.hasWhitespace()||!e.addChild(this._parseSelectorIdent()))return this.finish(e,Cr.IdentifierExpected)}else this.consumeToken();return this.finish(e)},e.prototype._parseClass=function(){if(!this.peekDelim(\".\"))return null;var e=this.createNode(cn.ClassSelector);return this.consumeToken(),this.hasWhitespace()||!e.addChild(this._parseSelectorIdent())?this.finish(e,Cr.IdentifierExpected):this.finish(e)},e.prototype._parseElementName=function(){var e=this.mark(),t=this.createNode(cn.ElementNameSelector);return t.addChild(this._parseNamespacePrefix()),t.addChild(this._parseSelectorIdent())||this.acceptDelim(\"*\")?this.finish(t):(this.restoreAtMark(e),null)},e.prototype._parseNamespacePrefix=function(){var e=this.mark(),t=this.createNode(cn.NamespacePrefix);return!t.addChild(this._parseIdent())&&this.acceptDelim(\"*\"),this.acceptDelim(\"|\")?this.finish(t):(this.restoreAtMark(e),null)},e.prototype._parseAttrib=function(){if(!this.peek(ft.BracketL))return null;var e=this.create(ir);return this.consumeToken(),e.setNamespacePrefix(this._parseNamespacePrefix()),e.setIdentifier(this._parseIdent())?(e.setOperator(this._parseOperator())&&(e.setValue(this._parseBinaryExpr()),this.acceptIdent(\"i\")),this.accept(ft.BracketR)?this.finish(e):this.finish(e,Cr.RightSquareBracketExpected)):this.finish(e,Cr.IdentifierExpected)},e.prototype._parsePseudo=function(){var e=this,t=this._tryParsePseudoIdentifier();if(t){if(!this.hasWhitespace()&&this.accept(ft.ParenthesisL)){if(t.addChild(this.try(function(){var t=e.create(gn);if(!t.addChild(e._parseSelector(!1)))return null;for(;e.accept(ft.Comma)&&t.addChild(e._parseSelector(!1)););return e.peek(ft.ParenthesisR)?e.finish(t):void 0})||this._parseBinaryExpr()),!this.accept(ft.ParenthesisR))return this.finish(t,Cr.RightParenthesisExpected)}return this.finish(t)}return null},e.prototype._tryParsePseudoIdentifier=function(){if(!this.peek(ft.Colon))return null;var e=this.mark(),t=this.createNode(cn.PseudoSelector);return this.consumeToken(),this.hasWhitespace()?(this.restoreAtMark(e),null):(this.accept(ft.Colon)&&this.hasWhitespace()&&this.markError(t,Cr.IdentifierExpected),t.addChild(this._parseIdent())||this.markError(t,Cr.IdentifierExpected),t)},e.prototype._tryParsePrio=function(){var e=this.mark(),t=this._parsePrio();return t||(this.restoreAtMark(e),null)},e.prototype._parsePrio=function(){if(!this.peek(ft.Exclamation))return null;var e=this.createNode(cn.Prio);return this.accept(ft.Exclamation)&&this.acceptIdent(\"important\")?this.finish(e):null},e.prototype._parseExpr=function(e){void 0===e&&(e=!1);var t=this.create(tr);if(!t.addChild(this._parseBinaryExpr()))return null;for(;;){if(this.peek(ft.Comma)){if(e)return this.finish(t);this.consumeToken()}if(!t.addChild(this._parseBinaryExpr()))break}return this.finish(t)},e.prototype._parseNamedLine=function(){if(!this.peek(ft.BracketL))return null;var e=this.createNode(cn.GridLine);for(this.consumeToken();e.addChild(this._parseIdent()););return this.accept(ft.BracketR)?this.finish(e):this.finish(e,Cr.RightSquareBracketExpected)},e.prototype._parseBinaryExpr=function(e,t){var n=this.create(nr);if(!n.setLeft(e||this._parseTerm()))return null;if(!n.setOperator(t||this._parseOperator()))return this.finish(n);if(!n.setRight(this._parseTerm()))return this.finish(n,Cr.TermExpected);n=this.finish(n);var r=this._parseOperator();return r&&(n=this._parseBinaryExpr(n,r)),this.finish(n)},e.prototype._parseTerm=function(){var e=this.create(rr);return e.setOperator(this._parseUnaryOperator()),e.setExpression(this._parseURILiteral())||e.setExpression(this._parseFunction())||e.setExpression(this._parseIdent())||e.setExpression(this._parseStringLiteral())||e.setExpression(this._parseNumeric())||e.setExpression(this._parseHexColor())||e.setExpression(this._parseOperation())||e.setExpression(this._parseNamedLine())?this.finish(e):null},e.prototype._parseOperation=function(){if(!this.peek(ft.ParenthesisL))return null;var e=this.create(gn);return this.consumeToken(),e.addChild(this._parseExpr()),this.accept(ft.ParenthesisR)?this.finish(e):this.finish(e,Cr.RightParenthesisExpected)},e.prototype._parseNumeric=function(){if(this.peek(ft.Num)||this.peek(ft.Percentage)||this.peek(ft.Resolution)||this.peek(ft.Length)||this.peek(ft.EMS)||this.peek(ft.EXS)||this.peek(ft.Angle)||this.peek(ft.Time)||this.peek(ft.Dimension)||this.peek(ft.Freq)){var e=this.create(sr);return this.consumeToken(),this.finish(e)}return null},e.prototype._parseStringLiteral=function(){if(!this.peek(ft.String)&&!this.peek(ft.BadString))return null;var e=this.createNode(cn.StringLiteral);return this.consumeToken(),this.finish(e)},e.prototype._parseURILiteral=function(){if(!this.peekRegExp(ft.Ident,/^url(-prefix)?$/i))return null;var e=this.mark(),t=this.createNode(cn.URILiteral);return this.accept(ft.Ident),this.hasWhitespace()||!this.peek(ft.ParenthesisL)?(this.restoreAtMark(e),null):(this.scanner.inURL=!0,this.consumeToken(),t.addChild(this._parseURLArgument()),this.scanner.inURL=!1,this.accept(ft.ParenthesisR)?this.finish(t):this.finish(t,Cr.RightParenthesisExpected))},e.prototype._parseURLArgument=function(){var e=this.create(gn);return this.accept(ft.String)||this.accept(ft.BadString)||this.acceptUnquotedString()?this.finish(e):null},e.prototype._parseIdent=function(e){if(!this.peek(ft.Ident))return null;var t=this.create(yn);return e&&(t.referenceTypes=e),t.isCustomProperty=this.peekRegExp(ft.Ident,/^--/),this.consumeToken(),this.finish(t)},e.prototype._parseFunction=function(){var e=this.mark(),t=this.create(Pn);if(!t.setIdentifier(this._parseFunctionIdentifier()))return null;if(this.hasWhitespace()||!this.accept(ft.ParenthesisL))return this.restoreAtMark(e),null;if(t.getArguments().addChild(this._parseFunctionArgument()))for(;this.accept(ft.Comma);)t.getArguments().addChild(this._parseFunctionArgument())||this.markError(t,Cr.ExpressionExpected);return this.accept(ft.ParenthesisR)?this.finish(t):this.finish(t,Cr.RightParenthesisExpected)},e.prototype._parseFunctionIdentifier=function(){if(!this.peek(ft.Ident))return null;var e=this.create(yn);if(e.referenceTypes=[hn.Function],this.acceptIdent(\"progid\")){if(this.accept(ft.Colon))for(;this.accept(ft.Ident)&&this.acceptDelim(\".\"););return this.finish(e)}return this.consumeToken(),this.finish(e)},e.prototype._parseFunctionArgument=function(){var e=this.create(Nn);return e.setValue(this._parseExpr(!0))?this.finish(e):null},e.prototype._parseHexColor=function(){if(this.peekRegExp(ft.Hash,/^#([A-Fa-f0-9]{3}|[A-Fa-f0-9]{4}|[A-Fa-f0-9]{6}|[A-Fa-f0-9]{8})$/g)){var e=this.create(or);return this.consumeToken(),this.finish(e)}return null},e}();var bi,yi,wi,vi,xi,ki,Ci,Si,_i,Fi,Ei,Ti,Ii,Ai,Pi,Oi=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Ni=function(){function e(e,t){this.offset=e,this.length=t,this.symbols=[],this.parent=null,this.children=[]}return e.prototype.addChild=function(e){this.children.push(e),e.setParent(this)},e.prototype.setParent=function(e){this.parent=e},e.prototype.findScope=function(e,t){return void 0===t&&(t=0),this.offset<=e&&this.offset+this.length>e+t||this.offset===e&&this.length===t?this.findInScope(e,t):null},e.prototype.findInScope=function(e,t){void 0===t&&(t=0);var n=e+t,r=function(e,t){var n=0,r=e.length;if(0===r)return 0;for(;n<r;){var i=Math.floor((n+r)/2);t(e[i])?r=i:n=i+1}return n}(this.children,function(e){return e.offset>n});if(0===r)return this;var i=this.children[r-1];return i.offset<=e&&i.offset+i.length>=e+t?i.findInScope(e,t):this},e.prototype.addSymbol=function(e){this.symbols.push(e)},e.prototype.getSymbol=function(e,t){for(var n=0;n<this.symbols.length;n++){var r=this.symbols[n];if(r.name===e&&r.type===t)return r}return null},e.prototype.getSymbols=function(){return this.symbols},e}(),Li=function(e){function t(){return e.call(this,0,Number.MAX_VALUE)||this}return Oi(t,e),t}(Ni),Ri=function(){return function(e,t,n,r){this.name=e,this.value=t,this.node=n,this.type=r}}(),zi=function(){function e(e){this.scope=e}return e.prototype.addSymbol=function(e,t,n,r){if(-1!==e.offset){var i=this.scope.findScope(e.offset,e.length);i&&i.addSymbol(new Ri(t,n,e,r))}},e.prototype.addScope=function(e){if(-1!==e.offset){var t=this.scope.findScope(e.offset,e.length);if(t&&(t.offset!==e.offset||t.length!==e.length)){var n=new Ni(e.offset,e.length);return t.addChild(n),n}return t}return null},e.prototype.addSymbolToChildScope=function(e,t,n,r,i){if(e&&-1!==e.offset){var o=this.addScope(e);o&&o.addSymbol(new Ri(n,r,t,i))}},e.prototype.visitNode=function(e){switch(e.type){case cn.Keyframe:return this.addSymbol(e,e.getName(),void 0,hn.Keyframe),!0;case cn.CustomPropertyDeclaration:return this.visitCustomPropertyDeclarationNode(e);case cn.VariableDeclaration:return this.visitVariableDeclarationNode(e);case cn.Ruleset:return this.visitRuleSet(e);case cn.MixinDeclaration:return this.addSymbol(e,e.getName(),void 0,hn.Mixin),!0;case cn.FunctionDeclaration:return this.addSymbol(e,e.getName(),void 0,hn.Function),!0;case cn.FunctionParameter:return this.visitFunctionParameterNode(e);case cn.Declarations:return this.addScope(e),!0;case cn.For:var t=e,n=t.getDeclarations();return n&&this.addSymbolToChildScope(n,t.variable,t.variable.getName(),void 0,hn.Variable),!0;case cn.Each:var r=e,i=r.getDeclarations();if(i)for(var o=0,s=r.getVariables().getChildren();o<s.length;o++){var a=s[o];this.addSymbolToChildScope(i,a,a.getName(),void 0,hn.Variable)}return!0}return!0},e.prototype.visitRuleSet=function(e){var t=this.scope.findScope(e.offset,e.length);if(t)for(var n=0,r=e.getSelectors().getChildren();n<r.length;n++){var i=r[n];i instanceof Cn&&1===i.getChildren().length&&t.addSymbol(new Ri(i.getChild(0).getText(),void 0,i,hn.Rule))}return!0},e.prototype.visitVariableDeclarationNode=function(e){var t=e.getValue()?e.getValue().getText():void 0;return this.addSymbol(e,e.getName(),t,hn.Variable),!0},e.prototype.visitFunctionParameterNode=function(e){var t=e.getParent().getDeclarations();if(t){var n=e.getDefaultValue(),r=n?n.getText():void 0;this.addSymbolToChildScope(t,e,e.getName(),r,hn.Variable)}return!0},e.prototype.visitCustomPropertyDeclarationNode=function(e){var t=e.getValue()?e.getValue().getText():\"\";return this.addCSSVariable(e.getProperty(),e.getProperty().getName(),t,hn.Variable),!0},e.prototype.addCSSVariable=function(e,t,n,r){-1!==e.offset&&this.scope.addSymbol(new Ri(t,n,e,r))},e}(),Di=function(){function e(e){this.global=new Li,e.acceptVisitor(new zi(this.global))}return e.prototype.findSymbolsAtOffset=function(e,t){for(var n=this.global.findScope(e,0),r=[],i={};n;){for(var o=n.getSymbols(),s=0;s<o.length;s++){var a=o[s];a.type!==t||i[a.name]||(r.push(a),i[a.name]=!0)}n=n.parent}return r},e.prototype.internalFindSymbol=function(e,t){var n=e;if(e.parent instanceof On&&e.parent.getParent()instanceof xn&&(n=e.parent.getParent().getDeclarations()),e.parent instanceof Nn&&e.parent.getParent()instanceof Pn){var r=e.parent.getParent().getIdentifier();if(r){var i=this.internalFindSymbol(r,[hn.Function]);i&&(n=i.node.getDeclarations())}}if(!n)return null;for(var o=e.getText(),s=this.global.findScope(n.offset,n.length);s;){for(var a=0;a<t.length;a++){var l=t[a],c=s.getSymbol(o,l);if(c)return c}s=s.parent}return null},e.prototype.evaluateReferenceTypes=function(e){if(e instanceof yn){var t=e.referenceTypes;if(t)return t;if(e.isCustomProperty)return[hn.Variable];var n=function(e){var t=e.findParent(cn.Declaration);return t&&t.getValue()&&t.getValue().encloses(e)?t:null}(e);if(n){var r=n.getNonPrefixedPropertyName();if((\"animation\"===r||\"animation-name\"===r)&&n.getValue()&&n.getValue().offset===e.offset)return[hn.Keyframe]}}else if(e instanceof cr)return[hn.Variable];return e.findParent(cn.Selector)?[hn.Rule]:e.findParent(cn.ExtendsReference)?[hn.Rule]:null},e.prototype.findSymbolFromNode=function(e){if(!e)return null;for(;e.type===cn.Interpolation;)e=e.getParent();var t=this.evaluateReferenceTypes(e);return t?this.internalFindSymbol(e,t):null},e.prototype.matchesSymbol=function(e,t){if(!e)return!1;for(;e.type===cn.Interpolation;)e=e.getParent();if(t.name.length!==e.length||t.name!==e.getText())return!1;var n=this.evaluateReferenceTypes(e);return!(!n||-1===n.indexOf(t.type))&&this.internalFindSymbol(e,n)===t},e.prototype.findSymbol=function(e,t,n){for(var r=this.global.findScope(n);r;){var i=r.getSymbol(e,t);if(i)return i;r=r.parent}return null},e}();function Ui(e,t){if(e.length<t.length)return!1;for(var n=0;n<t.length;n++)if(e[n]!==t[n])return!1;return!0}function Mi(e,t,n){void 0===n&&(n=4);var r=Math.abs(e.length-t.length);if(r>n)return 0;var i,o,s=[],a=[];for(i=0;i<t.length+1;++i)a.push(0);for(i=0;i<e.length+1;++i)s.push(a);for(i=1;i<e.length+1;++i)for(o=1;o<t.length+1;++o)e[i-1]===t[o-1]?s[i][o]=s[i-1][o-1]+1:s[i][o]=Math.max(s[i-1][o],s[i][o-1]);return s[e.length][t.length]-Math.sqrt(r)}function ji(e,t){return void 0===t&&(t=!0),e?e.length<140?e:e.slice(0,140)+(t?\"…\":\"\"):\"\"}!function(e){e.create=function(e,t){return{line:e,character:t}},e.is=function(e){var t=e;return fo.objectLiteral(t)&&fo.number(t.line)&&fo.number(t.character)}}(bi||(bi={})),function(e){e.create=function(e,t,n,r){if(fo.number(e)&&fo.number(t)&&fo.number(n)&&fo.number(r))return{start:bi.create(e,t),end:bi.create(n,r)};if(bi.is(e)&&bi.is(t))return{start:e,end:t};throw new Error(\"Range#create called with invalid arguments[\"+e+\", \"+t+\", \"+n+\", \"+r+\"]\")},e.is=function(e){var t=e;return fo.objectLiteral(t)&&bi.is(t.start)&&bi.is(t.end)}}(yi||(yi={})),function(e){e.create=function(e,t){return{uri:e,range:t}},e.is=function(e){var t=e;return fo.defined(t)&&yi.is(t.range)&&(fo.string(t.uri)||fo.undefined(t.uri))}}(wi||(wi={})),function(e){e.create=function(e,t,n,r){return{red:e,green:t,blue:n,alpha:r}},e.is=function(e){var t=e;return fo.number(t.red)&&fo.number(t.green)&&fo.number(t.blue)&&fo.number(t.alpha)}}(vi||(vi={})),function(e){e.create=function(e,t){return{range:e,color:t}},e.is=function(e){var t=e;return yi.is(t.range)&&vi.is(t.color)}}(xi||(xi={})),function(e){e.create=function(e,t,n){return{label:e,textEdit:t,additionalTextEdits:n}},e.is=function(e){var t=e;return fo.string(t.label)&&(fo.undefined(t.textEdit)||Ii.is(t))&&(fo.undefined(t.additionalTextEdits)||fo.typedArray(t.additionalTextEdits,Ii.is))}}(ki||(ki={})),function(e){e.Comment=\"comment\",e.Imports=\"imports\",e.Region=\"region\"}(Ci||(Ci={})),function(e){e.create=function(e,t,n,r,i){var o={startLine:e,endLine:t};return fo.defined(n)&&(o.startCharacter=n),fo.defined(r)&&(o.endCharacter=r),fo.defined(i)&&(o.kind=i),o},e.is=function(e){var t=e;return fo.number(t.startLine)&&fo.number(t.startLine)&&(fo.undefined(t.startCharacter)||fo.number(t.startCharacter))&&(fo.undefined(t.endCharacter)||fo.number(t.endCharacter))&&(fo.undefined(t.kind)||fo.string(t.kind))}}(Si||(Si={})),function(e){e.create=function(e,t){return{location:e,message:t}},e.is=function(e){var t=e;return fo.defined(t)&&wi.is(t.location)&&fo.string(t.message)}}(_i||(_i={})),function(e){e.Error=1,e.Warning=2,e.Information=3,e.Hint=4}(Fi||(Fi={})),function(e){e.create=function(e,t,n,r,i,o){var s={range:e,message:t};return fo.defined(n)&&(s.severity=n),fo.defined(r)&&(s.code=r),fo.defined(i)&&(s.source=i),fo.defined(o)&&(s.relatedInformation=o),s},e.is=function(e){var t=e;return fo.defined(t)&&yi.is(t.range)&&fo.string(t.message)&&(fo.number(t.severity)||fo.undefined(t.severity))&&(fo.number(t.code)||fo.string(t.code)||fo.undefined(t.code))&&(fo.string(t.source)||fo.undefined(t.source))&&(fo.undefined(t.relatedInformation)||fo.typedArray(t.relatedInformation,_i.is))}}(Ei||(Ei={})),function(e){e.create=function(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];var i={title:e,command:t};return fo.defined(n)&&n.length>0&&(i.arguments=n),i},e.is=function(e){var t=e;return fo.defined(t)&&fo.string(t.title)&&fo.string(t.command)}}(Ti||(Ti={})),function(e){e.replace=function(e,t){return{range:e,newText:t}},e.insert=function(e,t){return{range:{start:e,end:e},newText:t}},e.del=function(e){return{range:e,newText:\"\"}},e.is=function(e){var t=e;return fo.objectLiteral(t)&&fo.string(t.newText)&&yi.is(t.range)}}(Ii||(Ii={})),function(e){e.create=function(e,t){return{textDocument:e,edits:t}},e.is=function(e){var t=e;return fo.defined(t)&&Bi.is(t.textDocument)&&Array.isArray(t.edits)}}(Ai||(Ai={})),function(e){e.is=function(e){var t=e;return t&&(void 0!==t.changes||void 0!==t.documentChanges)&&(void 0===t.documentChanges||fo.typedArray(t.documentChanges,Ai.is))}}(Pi||(Pi={}));var Vi,Bi,Ki,qi,Wi,$i,Gi,Hi,Yi,Ji,Xi,Qi,Zi,eo,to,no,ro,io=function(){function e(e){this.edits=e}return e.prototype.insert=function(e,t){this.edits.push(Ii.insert(e,t))},e.prototype.replace=function(e,t){this.edits.push(Ii.replace(e,t))},e.prototype.delete=function(e){this.edits.push(Ii.del(e))},e.prototype.add=function(e){this.edits.push(e)},e.prototype.all=function(){return this.edits},e.prototype.clear=function(){this.edits.splice(0,this.edits.length)},e}();!function(){function e(e){var t=this;this._textEditChanges=Object.create(null),e&&(this._workspaceEdit=e,e.documentChanges?e.documentChanges.forEach(function(e){var n=new io(e.edits);t._textEditChanges[e.textDocument.uri]=n}):e.changes&&Object.keys(e.changes).forEach(function(n){var r=new io(e.changes[n]);t._textEditChanges[n]=r}))}Object.defineProperty(e.prototype,\"edit\",{get:function(){return this._workspaceEdit},enumerable:!0,configurable:!0}),e.prototype.getTextEditChange=function(e){if(Bi.is(e)){if(this._workspaceEdit||(this._workspaceEdit={documentChanges:[]}),!this._workspaceEdit.documentChanges)throw new Error(\"Workspace edit is not configured for versioned document changes.\");var t=e;if(!(r=this._textEditChanges[t.uri])){var n={textDocument:t,edits:i=[]};this._workspaceEdit.documentChanges.push(n),r=new io(i),this._textEditChanges[t.uri]=r}return r}if(this._workspaceEdit||(this._workspaceEdit={changes:Object.create(null)}),!this._workspaceEdit.changes)throw new Error(\"Workspace edit is not configured for normal text edit changes.\");var r;if(!(r=this._textEditChanges[e])){var i=[];this._workspaceEdit.changes[e]=i,r=new io(i),this._textEditChanges[e]=r}return r}}();!function(e){e.create=function(e){return{uri:e}},e.is=function(e){var t=e;return fo.defined(t)&&fo.string(t.uri)}}(Vi||(Vi={})),function(e){e.create=function(e,t){return{uri:e,version:t}},e.is=function(e){var t=e;return fo.defined(t)&&fo.string(t.uri)&&fo.number(t.version)}}(Bi||(Bi={})),function(e){e.create=function(e,t,n,r){return{uri:e,languageId:t,version:n,text:r}},e.is=function(e){var t=e;return fo.defined(t)&&fo.string(t.uri)&&fo.string(t.languageId)&&fo.number(t.version)&&fo.string(t.text)}}(Ki||(Ki={})),function(e){e.PlainText=\"plaintext\",e.Markdown=\"markdown\"}(qi||(qi={})),function(e){e.is=function(t){var n=t;return n===e.PlainText||n===e.Markdown}}(qi||(qi={})),function(e){e.is=function(e){var t=e;return fo.objectLiteral(e)&&qi.is(t.kind)&&fo.string(t.value)}}(Wi||(Wi={})),function(e){e.Text=1,e.Method=2,e.Function=3,e.Constructor=4,e.Field=5,e.Variable=6,e.Class=7,e.Interface=8,e.Module=9,e.Property=10,e.Unit=11,e.Value=12,e.Enum=13,e.Keyword=14,e.Snippet=15,e.Color=16,e.File=17,e.Reference=18,e.Folder=19,e.EnumMember=20,e.Constant=21,e.Struct=22,e.Event=23,e.Operator=24,e.TypeParameter=25}($i||($i={})),function(e){e.PlainText=1,e.Snippet=2}(Gi||(Gi={})),function(e){e.create=function(e){return{label:e}}}(Hi||(Hi={})),function(e){e.create=function(e,t){return{items:e||[],isIncomplete:!!t}}}(Yi||(Yi={})),function(e){e.fromPlainText=function(e){return e.replace(/[\\\\`*_{}[\\]()#+\\-.!]/g,\"\\\\$&\")},e.is=function(e){var t=e;return fo.string(t)||fo.objectLiteral(t)&&fo.string(t.language)&&fo.string(t.value)}}(Ji||(Ji={})),function(e){e.is=function(e){var t=e;return fo.objectLiteral(t)&&(Wi.is(t.contents)||Ji.is(t.contents)||fo.typedArray(t.contents,Ji.is))&&(void 0===e.range||yi.is(e.range))}}(Xi||(Xi={})),function(e){e.create=function(e,t){return t?{label:e,documentation:t}:{label:e}}}(Qi||(Qi={})),function(e){e.create=function(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];var i={label:e};return fo.defined(t)&&(i.documentation=t),fo.defined(n)?i.parameters=n:i.parameters=[],i}}(Zi||(Zi={})),function(e){e.Text=1,e.Read=2,e.Write=3}(eo||(eo={})),function(e){e.create=function(e,t){var n={range:e};return fo.number(t)&&(n.kind=t),n}}(to||(to={})),function(e){e.File=1,e.Module=2,e.Namespace=3,e.Package=4,e.Class=5,e.Method=6,e.Property=7,e.Field=8,e.Constructor=9,e.Enum=10,e.Interface=11,e.Function=12,e.Variable=13,e.Constant=14,e.String=15,e.Number=16,e.Boolean=17,e.Array=18,e.Object=19,e.Key=20,e.Null=21,e.EnumMember=22,e.Struct=23,e.Event=24,e.Operator=25,e.TypeParameter=26}(no||(no={})),function(e){e.create=function(e,t,n,r,i){var o={name:e,kind:t,location:{uri:r,range:n}};return i&&(o.containerName=i),o}}(ro||(ro={}));var oo,so,ao,lo,co,ho=function(){return function(){}}();!function(e){e.create=function(e,t,n,r,i,o){var s={name:e,detail:t,kind:n,range:r,selectionRange:i};return void 0!==o&&(s.children=o),s},e.is=function(e){var t=e;return t&&fo.string(t.name)&&fo.string(t.detail)&&fo.number(t.kind)&&yi.is(t.range)&&yi.is(t.selectionRange)&&(void 0===t.deprecated||fo.boolean(t.deprecated))&&(void 0===t.children||Array.isArray(t.children))}}(ho||(ho={})),function(e){e.QuickFix=\"quickfix\",e.Refactor=\"refactor\",e.RefactorExtract=\"refactor.extract\",e.RefactorInline=\"refactor.inline\",e.RefactorRewrite=\"refactor.rewrite\",e.Source=\"source\",e.SourceOrganizeImports=\"source.organizeImports\"}(oo||(oo={})),function(e){e.create=function(e,t){var n={diagnostics:e};return null!=t&&(n.only=t),n},e.is=function(e){var t=e;return fo.defined(t)&&fo.typedArray(t.diagnostics,Ei.is)&&(void 0===t.only||fo.typedArray(t.only,fo.string))}}(so||(so={})),function(e){e.create=function(e,t,n){var r={title:e};return Ti.is(t)?r.command=t:r.edit=t,void 0!==n&&(r.kind=n),r},e.is=function(e){var t=e;return t&&fo.string(t.title)&&(void 0===t.diagnostics||fo.typedArray(t.diagnostics,Ei.is))&&(void 0===t.kind||fo.string(t.kind))&&(void 0!==t.edit||void 0!==t.command)&&(void 0===t.command||Ti.is(t.command))&&(void 0===t.edit||Pi.is(t.edit))}}(ao||(ao={})),function(e){e.create=function(e,t){var n={range:e};return fo.defined(t)&&(n.data=t),n},e.is=function(e){var t=e;return fo.defined(t)&&yi.is(t.range)&&(fo.undefined(t.command)||Ti.is(t.command))}}(lo||(lo={})),function(e){e.create=function(e,t){return{tabSize:e,insertSpaces:t}},e.is=function(e){var t=e;return fo.defined(t)&&fo.number(t.tabSize)&&fo.boolean(t.insertSpaces)}}(co||(co={}));var uo,mo,po=function(){return function(){}}();!function(e){e.create=function(e,t,n){return{range:e,target:t,data:n}},e.is=function(e){var t=e;return fo.defined(t)&&yi.is(t.range)&&(fo.undefined(t.target)||fo.string(t.target))}}(po||(po={})),function(e){e.create=function(e,t,n,r){return new go(e,t,n,r)},e.is=function(e){var t=e;return!!(fo.defined(t)&&fo.string(t.uri)&&(fo.undefined(t.languageId)||fo.string(t.languageId))&&fo.number(t.lineCount)&&fo.func(t.getText)&&fo.func(t.positionAt)&&fo.func(t.offsetAt))},e.applyEdits=function(e,t){for(var n=e.getText(),r=function e(t,n){if(t.length<=1)return t;var r=t.length/2|0,i=t.slice(0,r),o=t.slice(r);e(i,n),e(o,n);for(var s=0,a=0,l=0;s<i.length&&a<o.length;){var c=n(i[s],o[a]);t[l++]=c<=0?i[s++]:o[a++]}for(;s<i.length;)t[l++]=i[s++];for(;a<o.length;)t[l++]=o[a++];return t}(t,function(e,t){var n=e.range.start.line-t.range.start.line;return 0===n?e.range.start.character-t.range.start.character:n}),i=n.length,o=r.length-1;o>=0;o--){var s=r[o],a=e.offsetAt(s.range.start),l=e.offsetAt(s.range.end);if(!(l<=i))throw new Error(\"Ovelapping edit\");n=n.substring(0,a)+s.newText+n.substring(l,n.length),i=a}return n}}(uo||(uo={})),function(e){e.Manual=1,e.AfterDelay=2,e.FocusOut=3}(mo||(mo={}));var fo,go=function(){function e(e,t,n,r){this._uri=e,this._languageId=t,this._version=n,this._content=r,this._lineOffsets=null}return Object.defineProperty(e.prototype,\"uri\",{get:function(){return this._uri},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"languageId\",{get:function(){return this._languageId},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"version\",{get:function(){return this._version},enumerable:!0,configurable:!0}),e.prototype.getText=function(e){if(e){var t=this.offsetAt(e.start),n=this.offsetAt(e.end);return this._content.substring(t,n)}return this._content},e.prototype.update=function(e,t){this._content=e.text,this._version=t,this._lineOffsets=null},e.prototype.getLineOffsets=function(){if(null===this._lineOffsets){for(var e=[],t=this._content,n=!0,r=0;r<t.length;r++){n&&(e.push(r),n=!1);var i=t.charAt(r);n=\"\\r\"===i||\"\\n\"===i,\"\\r\"===i&&r+1<t.length&&\"\\n\"===t.charAt(r+1)&&r++}n&&t.length>0&&e.push(t.length),this._lineOffsets=e}return this._lineOffsets},e.prototype.positionAt=function(e){e=Math.max(Math.min(e,this._content.length),0);var t=this.getLineOffsets(),n=0,r=t.length;if(0===r)return bi.create(0,e);for(;n<r;){var i=Math.floor((n+r)/2);t[i]>e?r=i:n=i+1}var o=n-1;return bi.create(o,e-t[o])},e.prototype.offsetAt=function(e){var t=this.getLineOffsets();if(e.line>=t.length)return this._content.length;if(e.line<0)return 0;var n=t[e.line],r=e.line+1<t.length?t[e.line+1]:this._content.length;return Math.max(Math.min(n+e.character,r),n)},Object.defineProperty(e.prototype,\"lineCount\",{get:function(){return this.getLineOffsets().length},enumerable:!0,configurable:!0}),e}();!function(e){var t=Object.prototype.toString;e.defined=function(e){return void 0!==e},e.undefined=function(e){return void 0===e},e.boolean=function(e){return!0===e||!1===e},e.string=function(e){return\"[object String]\"===t.call(e)},e.number=function(e){return\"[object Number]\"===t.call(e)},e.func=function(e){return\"[object Function]\"===t.call(e)},e.objectLiteral=function(e){return null!==e&&\"object\"==typeof e},e.typedArray=function(e,t){return Array.isArray(e)&&e.every(t)}}(fo||(fo={}));var bo=vr(),yo=Gi.Snippet,wo=function(){function e(e){void 0===e&&(e=null),this.completionParticipants=[],this.valueTypes=[cn.Identifier,cn.Value,cn.StringLiteral,cn.URILiteral,cn.NumericValue,cn.HexColorValue,cn.VariableName,cn.Prio],this.variablePrefix=e}return e.prototype.getSymbolContext=function(){return this.symbolContext||(this.symbolContext=new Di(this.styleSheet)),this.symbolContext},e.prototype.setCompletionParticipants=function(e){this.completionParticipants=e||[]},e.prototype.doComplete=function(e,t,n){this.offset=e.offsetAt(t),this.position=t,this.currentWord=function(e,t){var n=t-1,r=e.getText();for(;n>=0&&-1===' \\t\\n\\r\":{[()]},*>+'.indexOf(r.charAt(n));)n--;return r.substring(n+1,t)}(e,this.offset),this.defaultReplaceRange=yi.create(bi.create(this.position.line,this.position.character-this.currentWord.length),this.position),this.textDocument=e,this.styleSheet=n;try{var r={isIncomplete:!1,items:[]};this.nodePath=pn(this.styleSheet,this.offset);for(var i=this.nodePath.length-1;i>=0;i--){var o=this.nodePath[i];if(o instanceof An)this.getCompletionsForDeclarationProperty(o.getParent(),r);else if(o instanceof tr)o.parent instanceof lr?this.getVariableProposals(null,r):this.getCompletionsForExpression(o,r);else if(o instanceof Sn){var s=o.findParent(cn.ExtendsReference);if(s)this.getCompletionsForExtendsReference(s,o,r);else{var a=o.findParent(cn.Ruleset);this.getCompletionsForSelector(a,a&&a.isNested(),r)}}else if(o instanceof Nn)this.getCompletionsForFunctionArgument(o,o.getParent(),r);else if(o instanceof vn)this.getCompletionsForDeclarations(o,r);else if(o instanceof ar)this.getCompletionsForVariableDeclaration(o,r);else if(o instanceof kn)this.getCompletionsForRuleSet(o,r);else if(o instanceof lr)this.getCompletionsForInterpolation(o,r);else if(o instanceof Mn)this.getCompletionsForFunctionDeclaration(o,r);else if(o instanceof dr)this.getCompletionsForMixinReference(o,r);else if(o instanceof Pn)this.getCompletionsForFunctionArgument(null,o,r);else if(o instanceof Hn)this.getCompletionsForSupports(o,r);else if(o instanceof Qn)this.getCompletionsForSupportsCondition(o,r);else if(o instanceof hr)this.getCompletionsForExtendsReference(o,null,r);else if(o.type===cn.URILiteral)this.getCompletionForUriLiteralValue(o,r);else{if(null!==o.parent)continue;this.getCompletionForTopLevel(r)}if(r.items.length>0||this.offset>o.offset)return this.finalize(r)}return this.getCompletionsForStylesheet(r),0===r.items.length&&this.variablePrefix&&0===this.currentWord.indexOf(this.variablePrefix)&&this.getVariableProposals(null,r),this.finalize(r)}finally{this.position=null,this.currentWord=null,this.textDocument=null,this.styleSheet=null,this.symbolContext=null,this.defaultReplaceRange=null,this.nodePath=null}},e.prototype.finalize=function(e){if(e.items.some(function(e){return!!e.sortText}))for(var t=0,n=e.items;t<n.length;t++){var r=n[t];r.sortText||(r.sortText=\"d\")}return e},e.prototype.findInNodePath=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];for(var n=this.nodePath.length-1;n>=0;n--){var r=this.nodePath[n];if(-1!==e.indexOf(r.type))return r}return null},e.prototype.getCompletionsForDeclarationProperty=function(e,t){return this.getPropertyProposals(e,t)},e.prototype.getPropertyProposals=function(e,t){var n=this,r=li();for(var i in r)if(r.hasOwnProperty(i)){var o=r[i];if(o.browsers.onCodeComplete){var s=void 0,a=void 0,l=!1;e?(s=this.getCompletionRange(e.getProperty()),a=o.name,Co(e.colonPosition)||(a+=\": \",l=!0)):(s=this.getCompletionRange(null),a=o.name+\": \",l=!0);var c={label:o.name,documentation:ti(o),textEdit:Ii.replace(s,a),kind:$i.Property};1===o.restrictions.length&&\"none\"===o.restrictions[0]&&(l=!1),l&&(c.command={title:\"Suggest\",command:\"editor.action.triggerSuggest\"}),Ui(o.name,\"-\")&&(c.sortText=\"x\"),t.items.push(c)}}return this.completionParticipants.forEach(function(e){e.onCssProperty&&e.onCssProperty({propertyName:n.currentWord,range:n.defaultReplaceRange})}),t},e.prototype.getCompletionsForDeclarationValue=function(e,t){for(var n=this,r=e.getFullPropertyName(),i=li()[r],o=e.getValue();o&&o.hasChildren();)o=o.findChildAtOffset(this.offset,!1);if(this.completionParticipants.forEach(function(e){e.onCssPropertyValue&&e.onCssPropertyValue({propertyName:r,propertyValue:n.currentWord,range:n.getCompletionRange(o)})}),i){for(var s=0,a=i.restrictions;s<a.length;s++){switch(a[s]){case\"color\":this.getColorProposals(i,o,t);break;case\"position\":this.getPositionProposals(i,o,t);break;case\"repeat\":this.getRepeatStyleProposals(i,o,t);break;case\"line-style\":this.getLineStyleProposals(i,o,t);break;case\"line-width\":this.getLineWidthProposals(i,o,t);break;case\"geometry-box\":this.getGeometryBoxProposals(i,o,t);break;case\"box\":this.getBoxProposals(i,o,t);break;case\"image\":this.getImageProposals(i,o,t);break;case\"timing-function\":this.getTimingFunctionProposals(i,o,t);break;case\"shape\":this.getBasicShapeProposals(i,o,t)}}this.getValueEnumProposals(i,o,t),this.getCSSWideKeywordProposals(i,o,t),this.getUnitProposals(i,o,t)}else for(var l=0,c=function(e,t){var n=t.getFullPropertyName(),r=new vo;function i(e){return(e instanceof yn||e instanceof sr||e instanceof or)&&r.add(e.getText()),!0}return e.accept(function(e){if(e instanceof In&&e!==t&&(o=e.getFullPropertyName(),n===o)){var r=e.getValue();r&&r.accept(i)}var o;return!0}),r}(this.styleSheet,e).getEntries();l<c.length;l++){var h=c[l];t.items.push({label:h,textEdit:Ii.replace(this.getCompletionRange(o),h),kind:$i.Value})}return this.getVariableProposals(o,t),this.getTermProposals(i,o,t),t},e.prototype.getValueEnumProposals=function(e,t,n){if(e.values)for(var r=0,i=e.values;r<i.length;r++){var o=i[r];if(ei(o)){var s=o.name,a=void 0;if(d=\")\",u=void 0,(u=(h=s).length-d.length)>0?h.lastIndexOf(d)===u:0===u&&h===d){var l=s.lastIndexOf(\"(\");-1!==l&&(s=s.substr(0,l)+\"($1)\",a=yo)}var c={label:o.name,documentation:ti(o),textEdit:Ii.replace(this.getCompletionRange(t),s),kind:$i.Value,insertTextFormat:a};n.items.push(c)}}var h,d,u;return n},e.prototype.getCSSWideKeywordProposals=function(e,t,n){for(var r in Rr)n.items.push({label:r,documentation:Rr[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Value});return n},e.prototype.getCompletionsForInterpolation=function(e,t){return this.offset>=e.offset+2&&this.getVariableProposals(null,t),t},e.prototype.getVariableProposals=function(e,t){for(var n=0,r=this.getSymbolContext().findSymbolsAtOffset(this.offset,hn.Variable);n<r.length;n++){var i=r[n],o=Ui(i.name,\"--\")?\"var(\"+i.name+\")\":i.name,s={label:i.name,documentation:i.value?ji(i.value):i.value,textEdit:Ii.replace(this.getCompletionRange(e),o),kind:$i.Variable,sortText:\"z\"};if(i.node.type===cn.FunctionParameter){var a=i.node.getParent();a.type===cn.MixinDeclaration&&(s.detail=bo(\"completion.argument\",\"argument from '{0}'\",a.getName()))}t.items.push(s)}return t},e.prototype.getVariableProposalsForCSSVarFunction=function(e){for(var t=this.getSymbolContext().findSymbolsAtOffset(this.offset,hn.Variable),n=0,r=t=t.filter(function(e){return Ui(e.name,\"--\")});n<r.length;n++){var i=r[n];e.items.push({label:i.name,documentation:i.value?ji(i.value):i.value,textEdit:Ii.replace(this.getCompletionRange(null),i.name),kind:$i.Variable})}return e},e.prototype.getUnitProposals=function(e,t,n){var r=\"0\";if(this.currentWord.length>0){var i=this.currentWord.match(/^-?\\d[\\.\\d+]*/);i&&(r=i[0],n.isIncomplete=r.length===this.currentWord.length)}else 0===this.currentWord.length&&(n.isIncomplete=!0);t&&t.parent&&t.parent.type===cn.Term&&(t=t.getParent());for(var o=0,s=e.restrictions;o<s.length;o++){var a=s[o],l=jr[a];if(l)for(var c=0,h=l;c<h.length;c++){var d=r+h[c];n.items.push({label:d,textEdit:Ii.replace(this.getCompletionRange(t),d),kind:$i.Unit})}}return n},e.prototype.getCompletionRange=function(e){if(e&&e.offset<=this.offset){var t=-1!==e.end?this.textDocument.positionAt(e.end):this.position;return yi.create(this.textDocument.positionAt(e.offset),t)}return this.defaultReplaceRange},e.prototype.getColorProposals=function(e,t,n){for(var r in Er)n.items.push({label:r,documentation:Er[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Color});for(var r in Tr)n.items.push({label:r,documentation:Tr[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Value});var i=new vo;this.styleSheet.acceptVisitor(new ko(i,this.offset));for(var o=0,s=i.getEntries();o<s.length;o++){r=s[o];n.items.push({label:r,textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Color})}for(var a=function(e){var r=1,i=e.func.replace(/\\[?\\$(\\w+)\\]?/g,function(e,t){return\"${\"+r+++\":\"+t+\"}\"});n.items.push({label:e.func.substr(0,e.func.indexOf(\"(\")),detail:e.func,documentation:e.desc,textEdit:Ii.replace(l.getCompletionRange(t),i),insertTextFormat:yo,kind:$i.Function})},l=this,c=0,h=zr;c<h.length;c++){a(h[c])}return n},e.prototype.getPositionProposals=function(e,t,n){for(var r in Ir)n.items.push({label:r,documentation:Ir[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Value});return n},e.prototype.getRepeatStyleProposals=function(e,t,n){for(var r in Ar)n.items.push({label:r,documentation:Ar[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Value});return n},e.prototype.getLineStyleProposals=function(e,t,n){for(var r in Pr)n.items.push({label:r,documentation:Pr[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Value});return n},e.prototype.getLineWidthProposals=function(e,t,n){for(var r=0,i=Or;r<i.length;r++){var o=i[r];n.items.push({label:o,textEdit:Ii.replace(this.getCompletionRange(t),o),kind:$i.Value})}return n},e.prototype.getGeometryBoxProposals=function(e,t,n){for(var r in Lr)n.items.push({label:r,documentation:Lr[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Value});return n},e.prototype.getBoxProposals=function(e,t,n){for(var r in Nr)n.items.push({label:r,documentation:Nr[r],textEdit:Ii.replace(this.getCompletionRange(t),r),kind:$i.Value});return n},e.prototype.getImageProposals=function(e,t,n){for(var r in Dr){var i=xo(r);n.items.push({label:r,documentation:Dr[r],textEdit:Ii.replace(this.getCompletionRange(t),i),kind:$i.Function,insertTextFormat:r!==i?yo:void 0})}return n},e.prototype.getTimingFunctionProposals=function(e,t,n){for(var r in Ur){var i=xo(r);n.items.push({label:r,documentation:Ur[r],textEdit:Ii.replace(this.getCompletionRange(t),i),kind:$i.Function,insertTextFormat:r!==i?yo:void 0})}return n},e.prototype.getBasicShapeProposals=function(e,t,n){for(var r in Mr){var i=xo(r);n.items.push({label:r,documentation:Mr[r],textEdit:Ii.replace(this.getCompletionRange(t),i),kind:$i.Function,insertTextFormat:r!==i?yo:void 0})}return n},e.prototype.getCompletionsForStylesheet=function(e){var t=this.styleSheet.findFirstChildBeforeOffset(this.offset);return t?t instanceof kn?this.getCompletionsForRuleSet(t,e):t instanceof Hn?this.getCompletionsForSupports(t,e):e:this.getCompletionForTopLevel(e)},e.prototype.getCompletionForTopLevel=function(e){for(var t=0,n=function(){if(!ci){ci=[];for(var e=0;e<hi.length;e++){var t=hi[e];ci.push(new si(t))}}return ci}();t<n.length;t++){var r=n[t];r.browsers.count>0&&e.items.push({label:r.name,textEdit:Ii.replace(this.getCompletionRange(null),r.name),documentation:ti(r),kind:$i.Keyword})}return this.getCompletionsForSelector(null,!1,e),e},e.prototype.getCompletionsForRuleSet=function(e,t){var n=e.getDeclarations();return n&&n.endsWith(\"}\")&&this.offset>=n.end?this.getCompletionForTopLevel(t):!n||this.offset<=n.offset?this.getCompletionsForSelector(e,e.isNested(),t):(e.findParent(cn.Ruleset),this.getCompletionsForDeclarations(e.getDeclarations(),t))},e.prototype.getCompletionsForSelector=function(e,t,n){var r=this,i=this.findInNodePath(cn.PseudoSelector,cn.IdentifierSelector,cn.ClassSelector,cn.ElementNameSelector);!i&&this.offset-this.currentWord.length>0&&\":\"===this.textDocument.getText()[this.offset-this.currentWord.length-1]&&(this.currentWord=\":\"+this.currentWord,this.defaultReplaceRange=yi.create(bi.create(this.position.line,this.position.character-this.currentWord.length),this.position));for(var o=0,s=function(){if(!mi){mi=[];for(var e=0;e<pi.length;e++){var t=pi[e];mi.push(new si(t))}}return mi}();o<s.length;o++){if((m=s[o]).browsers.onCodeComplete){var a=xo(m.name),l={label:m.name,textEdit:Ii.replace(this.getCompletionRange(i),a),documentation:ti(m),kind:$i.Function,insertTextFormat:m.name!==a?yo:void 0};Ui(m.name,\":-\")&&(l.sortText=\"x\"),n.items.push(l)}}for(var c=0,h=function(){if(!di){di=[];for(var e=0;e<ui.length;e++){var t=ui[e];di.push(new si(t))}}return di}();c<h.length;c++){if((m=h[c]).browsers.onCodeComplete){a=xo(m.name),l={label:m.name,textEdit:Ii.replace(this.getCompletionRange(i),a),documentation:ti(m),kind:$i.Function,insertTextFormat:m.name!==a?yo:void 0};Ui(m.name,\"::-\")&&(l.sortText=\"x\"),n.items.push(l)}}if(!t){for(var d=0,u=Vr;d<u.length;d++){var m=u[d];n.items.push({label:m,textEdit:Ii.replace(this.getCompletionRange(i),m),kind:$i.Keyword})}for(var p=0,f=Br;p<f.length;p++){m=f[p];n.items.push({label:m,textEdit:Ii.replace(this.getCompletionRange(i),m),kind:$i.Keyword})}}var g={};g[this.currentWord]=!0;var b=this.styleSheet.getTextProvider();if(this.styleSheet.accept(function(e){if(e.type===cn.SimpleSelector&&e.length>0){var t=b(e.offset,e.length);return\".\"!==t.charAt(0)||g[t]||(g[t]=!0,n.items.push({label:t,textEdit:Ii.replace(r.getCompletionRange(i),t),kind:$i.Keyword})),!1}return!0}),e&&e.isNested()){var y=e.getSelectors().findFirstChildBeforeOffset(this.offset);y&&0===e.getSelectors().getChildren().indexOf(y)&&this.getPropertyProposals(null,n)}return n},e.prototype.getCompletionsForDeclarations=function(e,t){if(!e||this.offset===e.offset)return t;var n=e.findFirstChildBeforeOffset(this.offset);if(!n)return this.getCompletionsForDeclarationProperty(null,t);if(n instanceof Fn){var r=n;if(!Co(r.colonPosition)||this.offset<=r.colonPosition)return this.getCompletionsForDeclarationProperty(r,t);if(Co(r.semicolonPosition)&&r.semicolonPosition<this.offset)return this.offset===r.semicolonPosition+1?t:this.getCompletionsForDeclarationProperty(null,t);if(r instanceof In)return this.getCompletionsForDeclarationValue(r,t)}else n instanceof hr?this.getCompletionsForExtendsReference(n,null,t):this.currentWord&&\"@\"===this.currentWord[0]&&this.getCompletionsForDeclarationProperty(null,t);return t},e.prototype.getCompletionsForVariableDeclaration=function(e,t){return this.offset>e.colonPosition&&this.getVariableProposals(e.getValue(),t),t},e.prototype.getCompletionsForExpression=function(e,t){if(e.getParent()instanceof Nn)return this.getCompletionsForFunctionArgument(e.getParent(),e.getParent().getParent(),t),t;var n=e.findParent(cn.Declaration);if(!n)return this.getTermProposals(null,null,t),t;var r=e.findChildAtOffset(this.offset,!0);return r?r instanceof sr||r instanceof yn?this.getCompletionsForDeclarationValue(n,t):t:this.getCompletionsForDeclarationValue(n,t)},e.prototype.getCompletionsForFunctionArgument=function(e,t,n){return\"var\"===t.getIdentifier().getText()&&(t.getArguments().hasChildren()&&t.getArguments().getChild(0)!==e||this.getVariableProposalsForCSSVarFunction(n)),n},e.prototype.getCompletionsForFunctionDeclaration=function(e,t){var n=e.getDeclarations();return n&&this.offset>n.offset&&this.offset<n.end&&this.getTermProposals(null,null,t),t},e.prototype.getCompletionsForMixinReference=function(e,t){for(var n=0,r=this.getSymbolContext().findSymbolsAtOffset(this.offset,hn.Mixin);n<r.length;n++){var i=r[n];i.node instanceof ur&&t.items.push(this.makeTermProposal(i,i.node.getParameters(),null))}return t},e.prototype.getTermProposals=function(e,t,n){for(var r=0,i=this.getSymbolContext().findSymbolsAtOffset(this.offset,hn.Function);r<i.length;r++){var o=i[r];o.node instanceof Mn&&n.items.push(this.makeTermProposal(o,o.node.getParameters(),t))}return n},e.prototype.makeTermProposal=function(e,t,n){e.node;var r=t.getChildren().map(function(e){return e instanceof On?e.getName():e.getText()}),i=e.name+\"(\"+r.map(function(e,t){return\"${\"+(t+1)+\":\"+e+\"}\"}).join(\", \")+\")\";return{label:e.name,detail:e.name+\"(\"+r.join(\", \")+\")\",textEdit:Ii.replace(this.getCompletionRange(n),i),insertTextFormat:yo,kind:$i.Function,sortText:\"z\"}},e.prototype.getCompletionsForSupportsCondition=function(e,t){var n=e.findFirstChildBeforeOffset(this.offset);if(n){if(n instanceof In)return Co(n.colonPosition||this.offset<=n.colonPosition)?this.getCompletionsForDeclarationValue(n,t):this.getCompletionsForDeclarationProperty(n,t);if(n instanceof Qn)return this.getCompletionsForSupportsCondition(n,t)}return Co(e.lParent)&&this.offset>e.lParent&&(!Co(e.rParent)||this.offset<=e.rParent)?this.getCompletionsForDeclarationProperty(null,t):t},e.prototype.getCompletionsForSupports=function(e,t){var n=e.getDeclarations();if(!n||this.offset<=n.offset){var r=e.findFirstChildBeforeOffset(this.offset);return r instanceof Qn?this.getCompletionsForSupportsCondition(r,t):t}return this.getCompletionForTopLevel(t)},e.prototype.getCompletionsForExtendsReference=function(e,t,n){return n},e.prototype.getCompletionForUriLiteralValue=function(e,t){var n,r,i;if(0===e.getChildren().length){n=\"\",r=this.position;var o=this.textDocument.positionAt(e.offset+\"url(\".length);i=yi.create(o,o)}else{var s=e.getChild(0);n=s.getText(),r=this.position,i=this.getCompletionRange(s)}return this.completionParticipants.forEach(function(e){e.onCssURILiteralValue&&e.onCssURILiteralValue({uriValue:n,position:r,range:i})}),t},e}(),vo=function(){function e(){this.entries={}}return e.prototype.add=function(e){this.entries[e]=!0},e.prototype.getEntries=function(){return Object.keys(this.entries)},e}();function xo(e){return e.replace(/\\(\\)$/,\"($1)\")}var ko=function(){function e(e,t){this.entries=e,this.currentOffset=t}return e.prototype.visitNode=function(e){return(e instanceof or||e instanceof Pn&&function(e){var t=e.getName();return!!t&&/^(rgb|rgba|hsl|hsla)$/gi.test(t)}(e))&&(this.currentOffset<e.offset||e.end<this.currentOffset)&&this.entries.add(e.getText()),!0},e}();function Co(e){return void 0!==e}var So,_o=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Fo=function(){function e(){}return e.prototype.findAttribute=function(e){if(this.attributes)for(var t=0,n=this.attributes;t<n.length;t++){var r=n[t];if(r.name===e)return r.value}return null},e.prototype.addChild=function(t){t instanceof e&&(t.parent=this),this.children||(this.children=[]),this.children.push(t)},e.prototype.append=function(e){if(this.attributes){var t=this.attributes[this.attributes.length-1];t.value=t.value+e}},e.prototype.prepend=function(e){if(this.attributes){var t=this.attributes[0];t.value=e+t.value}},e.prototype.findRoot=function(){for(var e=this;e.parent&&!(e.parent instanceof Eo);)e=e.parent;return e},e.prototype.removeChild=function(e){if(this.children){var t=this.children.indexOf(e);if(-1!==t)return this.children.splice(t,1),!0}return!1},e.prototype.addAttr=function(e,t){this.attributes||(this.attributes=[]);for(var n=0,r=this.attributes;n<r.length;n++){var i=r[n];if(i.name===e)return void(i.value+=\" \"+t)}this.attributes.push({name:e,value:t})},e.prototype.clone=function(t){void 0===t&&(t=!0);var n=new e;if(this.attributes){n.attributes=[];for(var r=0,i=this.attributes;r<i.length;r++){var o=i[r];n.addAttr(o.name,o.value)}}if(t&&this.children){n.children=[];for(var s=0;s<this.children.length;s++)n.addChild(this.children[s].clone())}return n},e.prototype.cloneWithParent=function(){var e=this.clone(!1);!this.parent||this.parent instanceof Eo||this.parent.cloneWithParent().addChild(e);return e},e}(),Eo=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return _o(t,e),t}(Fo),To=function(e){function t(t){var n=e.call(this)||this;return n.addAttr(\"name\",t),n}return _o(t,e),t}(Fo),Io=function(){function e(e){this.quote=e}return e.prototype.print=function(e){return this.result=[],e instanceof Eo?this.doPrint(e.children,0):this.doPrint([e],0),[{language:\"html\",value:this.result.join(\"\\n\")}]},e.prototype.doPrint=function(e,t){for(var n=0,r=e;n<r.length;n++){var i=r[n];this.doPrintElement(i,t),i.children&&this.doPrint(i.children,t+1)}},e.prototype.writeLine=function(e,t){var n=new Array(e+1).join(\"  \");this.result.push(n+t)},e.prototype.doPrintElement=function(e,t){var n=e.findAttribute(\"name\");if(e instanceof To||\"…\"===n)this.writeLine(t,n);else{var r=[\"<\"];if(n?r.push(n):r.push(\"element\"),e.attributes)for(var i=0,o=e.attributes;i<o.length;i++){var s=o[i];if(\"name\"!==s.name){r.push(\" \"),r.push(s.name);var a=s.value;a&&(r.push(\"=\"),r.push(So.ensure(a,this.quote)))}}r.push(\">\"),this.writeLine(t,r.join(\"\"))}},e}();function Ao(e,t){for(var n=new Fo,r=0,i=e.getChildren();r<i.length;r++){var o=i[r];switch(o.type){case cn.SelectorCombinator:if(t){var s=o.getText().split(\"&\");if(1===s.length){n.addAttr(\"name\",s[0]);break}if(n=t.cloneWithParent(),s[0])n.findRoot().prepend(s[0]);for(var a=1;a<s.length;a++){if(a>1){var l=t.cloneWithParent();n.addChild(l.findRoot()),n=l}n.append(s[a])}}break;case cn.SelectorPlaceholder:if(\"@at-root\"===o.getText())return n;case cn.ElementNameSelector:var c=o.getText();n.addAttr(\"name\",\"*\"===c?\"element\":Po(c));break;case cn.ClassSelector:n.addAttr(\"class\",Po(o.getText().substring(1)));break;case cn.IdentifierSelector:n.addAttr(\"id\",Po(o.getText().substring(1)));break;case cn.MixinDeclaration:n.addAttr(\"class\",o.getName());break;case cn.PseudoSelector:n.addAttr(Po(o.getText()),\"\");break;case cn.AttributeSelector:var h=o,d=h.getIdentifier();if(d){var u=h.getValue(),m=h.getOperator(),p=void 0;if(u)switch(Po(m.getText())){case\"|=\":p=So.remove(Po(u.getText()))+\"-…\";break;case\"^=\":p=So.remove(Po(u.getText()))+\"…\";break;case\"$=\":p=\"…\"+So.remove(Po(u.getText()));break;case\"~=\":p=\" … \"+So.remove(Po(u.getText()))+\" … \";break;case\"*=\":p=\"…\"+So.remove(Po(u.getText()))+\"…\";break;default:p=So.remove(Po(u.getText()))}n.addAttr(Po(d.getText()),p)}}}return n}function Po(e){var t=new dn;t.setSource(e);var n=t.scanUnquotedString();return n?n.text:e}function Oo(e){var t=function(e){if(e.matches(\"@at-root\"))return null;var t=new Eo,n=[];if(e.getParent()instanceof kn)for(var r=e.getParent().getParent();r&&!Ro(r);){if(r instanceof kn){if(r.getSelectors().matches(\"@at-root\"))break;n.push(r)}r=r.getParent()}for(var i=new Lo(t),o=n.length-1;o>=0;o--){var s=n[o].getSelectors().getChild(0);s&&i.processSelector(s)}return i.processSelector(e),t}(e);return new Io('\"').print(t)}function No(e){var t=Ao(e);return new Io('\"').print(t)}!function(e){function t(e){var t=e.match(/^['\"](.*)[\"']$/);return t?t[1]:e}e.ensure=function(e,n){return n+t(e)+n},e.remove=t}(So||(So={}));var Lo=function(){function e(e){this.prev=null,this.element=e}return e.prototype.processSelector=function(e){var t=null;if(!(this.element instanceof Eo)&&e.getChildren().some(function(e){return e.hasChildren()&&e.getChild(0).type===cn.SelectorCombinator})){var n=this.element.findRoot();n.parent instanceof Eo&&(t=this.element,this.element=n.parent,this.element.removeChild(n),this.prev=null)}for(var r=0,i=e.getChildren();r<i.length;r++){var o=i[r];if(o instanceof Sn){if(this.prev instanceof Sn){var s=new To(\"…\");this.element.addChild(s),this.element=s}else this.prev&&(this.prev.matches(\"+\")||this.prev.matches(\"~\"))&&(this.element=this.element.parent);this.prev&&this.prev.matches(\"~\")&&(this.element.addChild(Ao(o)),this.element.addChild(new To(\"⋮\")));var a=Ao(o,t),l=a.findRoot();this.element.addChild(l),this.element=a}(o instanceof Sn||o.type===cn.SelectorCombinatorParent||o.type===cn.SelectorCombinatorShadowPiercingDescendant||o.type===cn.SelectorCombinatorSibling||o.type===cn.SelectorCombinatorAllSiblings)&&(this.prev=o)}},e}();function Ro(e){switch(e.type){case cn.MixinDeclaration:case cn.Stylesheet:return!0}return!1}var zo=function(){function e(){}return e.prototype.doHover=function(e,t,n){function r(t){return yi.create(e.positionAt(t.offset),e.positionAt(t.end))}for(var i=pn(n,e.offsetAt(t)),o=0;o<i.length;o++){var s=i[o];if(s instanceof Cn)return{contents:Oo(s),range:r(s)};if(s instanceof Sn)return{contents:No(s),range:r(s)};if(s instanceof In){var a=s.getFullPropertyName(),l=li()[a];if(l){var c=[];l.description&&c.push(Ji.fromPlainText(l.description));var h=ni(l.browsers);if(h&&c.push(Ji.fromPlainText(h)),c.length)return{contents:c,range:r(s)}}}}return null},e}(),Do=vr(),Uo=function(){function e(){}return e.prototype.findDefinition=function(e,t,n){var r=new Di(n),i=mn(n,e.offsetAt(t));if(!i)return null;var o=r.findSymbolFromNode(i);return o?{uri:e.uri,range:Mo(o.node,e)}:null},e.prototype.findReferences=function(e,t,n){return this.findDocumentHighlights(e,t,n).map(function(t){return{uri:e.uri,range:t.range}})},e.prototype.findDocumentHighlights=function(e,t,n){var r=[],i=mn(n,e.offsetAt(t));if(!i||i.type===cn.Stylesheet||i.type===cn.Declarations)return r;var o=new Di(n),s=o.findSymbolFromNode(i),a=i.getText();return n.accept(function(t){if(s){if(o.matchesSymbol(t,s))return r.push({kind:jo(t),range:Mo(t,e)}),!1}else i.type===t.type&&i.length===t.length&&a===t.getText()&&r.push({kind:jo(t),range:Mo(t,e)});return!0}),r},e.prototype.findDocumentSymbols=function(e,t){var n=[];return t.accept(function(t){var r={name:null,kind:no.Class,location:null},i=t;return t instanceof Cn?(r.name=t.getText(),i=t.findParent(cn.Ruleset)):t instanceof ar?(r.name=t.getName(),r.kind=no.Variable):t instanceof ur?(r.name=t.getName(),r.kind=no.Method):t instanceof Mn?(r.name=t.getName(),r.kind=no.Function):t instanceof Kn?r.name=Do(\"literal.keyframes\",\"@keyframes {0}\",t.getName()):t instanceof Vn&&(r.name=Do(\"literal.fontface\",\"@font-face\")),r.name&&(r.location=wi.create(e.uri,Mo(i,e)),n.push(r)),!0}),n},e.prototype.findDocumentColors=function(e,t){var n=[];return t.accept(function(t){var r=function(e,t){var n=Jr(e);if(n){var r=Mo(e,t);return{color:n,range:r}}return null}(t,e);return r&&n.push(r),!0}),n},e.prototype.getColorPresentations=function(e,t,n,r){var i,o=[],s=Math.round(255*n.red),a=Math.round(255*n.green),l=Math.round(255*n.blue);i=1===n.alpha?\"rgb(\"+s+\", \"+a+\", \"+l+\")\":\"rgba(\"+s+\", \"+a+\", \"+l+\", \"+n.alpha+\")\",o.push({label:i,textEdit:Ii.replace(r,i)}),i=1===n.alpha?\"#\"+Vo(s)+Vo(a)+Vo(l):\"#\"+Vo(s)+Vo(a)+Vo(l)+Vo(Math.round(255*n.alpha)),o.push({label:i,textEdit:Ii.replace(r,i)});var c=function(e){var t=e.red,n=e.green,r=e.blue,i=e.alpha,o=Math.max(t,n,r),s=Math.min(t,n,r),a=0,l=0,c=(s+o)/2,h=o-s;if(h>0){switch(l=Math.min(c<=.5?h/(2*c):h/(2-2*c),1),o){case t:a=(n-r)/h+(n<r?6:0);break;case n:a=(r-t)/h+2;break;case r:a=(t-n)/h+4}a*=60,a=Math.round(a)}return{h:a,s:l,l:c,a:i}}(n);return i=1===c.a?\"hsl(\"+c.h+\", \"+Math.round(100*c.s)+\"%, \"+Math.round(100*c.l)+\"%)\":\"hsla(\"+c.h+\", \"+Math.round(100*c.s)+\"%, \"+Math.round(100*c.l)+\"%, \"+c.a+\")\",o.push({label:i,textEdit:Ii.replace(r,i)}),o},e.prototype.doRename=function(e,t,n,r){var i,o=this.findDocumentHighlights(e,t,r).map(function(e){return Ii.replace(e.range,n)});return{changes:(i={},i[e.uri]=o,i)}},e}();function Mo(e,t){return yi.create(t.positionAt(e.offset),t.positionAt(e.end))}function jo(e){if(e.type===cn.Selector)return eo.Write;if(e instanceof yn&&e.parent&&e.parent instanceof An&&e.isCustomProperty)return eo.Write;if(e.parent)switch(e.parent.type){case cn.FunctionDeclaration:case cn.MixinDeclaration:case cn.Keyframe:case cn.VariableDeclaration:case cn.FunctionParameter:return eo.Write}return eo.Read}function Vo(e){var t=e.toString(16);return 2!==t.length?\"0\"+t:t}var Bo=vr(),Ko=fn.Warning,qo=fn.Error,Wo=fn.Ignore,$o=function(){return function(e,t,n){this.id=e,this.message=t,this.defaultValue=n}}(),Go={AllVendorPrefixes:new $o(\"compatibleVendorPrefixes\",Bo(\"rule.vendorprefixes.all\",\"When using a vendor-specific prefix make sure to also include all other vendor-specific properties\"),Wo),IncludeStandardPropertyWhenUsingVendorPrefix:new $o(\"vendorPrefix\",Bo(\"rule.standardvendorprefix.all\",\"When using a vendor-specific prefix also include the standard property\"),Ko),DuplicateDeclarations:new $o(\"duplicateProperties\",Bo(\"rule.duplicateDeclarations\",\"Do not use duplicate style definitions\"),Wo),EmptyRuleSet:new $o(\"emptyRules\",Bo(\"rule.emptyRuleSets\",\"Do not use empty rulesets\"),Ko),ImportStatemement:new $o(\"importStatement\",Bo(\"rule.importDirective\",\"Import statements do not load in parallel\"),Wo),BewareOfBoxModelSize:new $o(\"boxModel\",Bo(\"rule.bewareOfBoxModelSize\",\"Do not use width or height when using padding or border\"),Wo),UniversalSelector:new $o(\"universalSelector\",Bo(\"rule.universalSelector\",\"The universal selector (*) is known to be slow\"),Wo),ZeroWithUnit:new $o(\"zeroUnits\",Bo(\"rule.zeroWidthUnit\",\"No unit for zero needed\"),Wo),RequiredPropertiesForFontFace:new $o(\"fontFaceProperties\",Bo(\"rule.fontFaceProperties\",\"@font-face rule must define 'src' and 'font-family' properties\"),Ko),HexColorLength:new $o(\"hexColorLength\",Bo(\"rule.hexColor\",\"Hex colors must consist of three, four, six or eight hex numbers\"),qo),ArgsInColorFunction:new $o(\"argumentsInColorFunction\",Bo(\"rule.colorFunction\",\"Invalid number of parameters\"),qo),UnknownProperty:new $o(\"unknownProperties\",Bo(\"rule.unknownProperty\",\"Unknown property.\"),Ko),UnknownAtRules:new $o(\"unknownAtRules\",Bo(\"rule.unknownAtRules\",\"Unknown at-rule.\"),Ko),IEStarHack:new $o(\"ieHack\",Bo(\"rule.ieHack\",\"IE hacks are only necessary when supporting IE7 and older\"),Wo),UnknownVendorSpecificProperty:new $o(\"unknownVendorSpecificProperties\",Bo(\"rule.unknownVendorSpecificProperty\",\"Unknown vendor specific property.\"),Wo),PropertyIgnoredDueToDisplay:new $o(\"propertyIgnoredDueToDisplay\",Bo(\"rule.propertyIgnoredDueToDisplay\",\"Property is ignored due to the display.\"),Ko),AvoidImportant:new $o(\"important\",Bo(\"rule.avoidImportant\",\"Avoid using !important. It is an indication that the specificity of the entire CSS has gotten out of control and needs to be refactored.\"),Wo),AvoidFloat:new $o(\"float\",Bo(\"rule.avoidFloat\",\"Avoid using 'float'. Floats lead to fragile CSS that is easy to break if one aspect of the layout changes.\"),Wo),AvoidIdSelector:new $o(\"idSelector\",Bo(\"rule.avoidIdSelector\",\"Selectors should not contain IDs because these rules are too tightly coupled with the HTML.\"),Wo)},Ho=function(){function e(e){void 0===e&&(e={}),this.conf=e}return e.prototype.get=function(e){if(this.conf.hasOwnProperty(e.id)){var t=function(e){switch(e){case\"ignore\":return fn.Ignore;case\"warning\":return fn.Warning;case\"error\":return fn.Error}return null}(this.conf[e.id]);if(t)return t}return e.defaultValue},e}();var Yo=vr(),Jo=function(){function e(){}return e.prototype.doCodeActions=function(e,t,n,r){var i=[];if(n.diagnostics)for(var o=0,s=n.diagnostics;o<s.length;o++){var a=s[o];this.appendFixesForMarker(e,r,a,i)}return i},e.prototype.getFixesForUnknownProperty=function(e,t,n,r){var i=t.getName(),o=[];for(var s in li()){var a=Mi(i,s);a>=i.length/2&&o.push({property:s,score:a})}o.sort(function(e,t){return t.score-e.score});for(var l=3,c=0,h=o;c<h.length;c++){var d=h[c].property,u=Yo(\"css.codeaction.rename\",\"Rename to '{0}'\",d),m=Ii.replace(n.range,d);if(r.push(Ti.create(u,\"_css.applyCodeAction\",e.uri,e.version,[m])),--l<=0)return}},e.prototype.appendFixesForMarker=function(e,t,n,r){if(n.code===Go.UnknownProperty.id)for(var i=e.offsetAt(n.range.start),o=e.offsetAt(n.range.end),s=pn(t,i),a=s.length-1;a>=0;a--){var l=s[a];if(l instanceof In){var c=l.getProperty();if(c&&c.offset===i&&c.end===o)return void this.getFixesForUnknownProperty(e,c,n,r)}}},e}(),Xo=vr(),Qo=function(){return function(e,t){this.name=e,this.node=t}}(),Zo=function(){function e(){this.data={}}return e.prototype.add=function(e,t,n){var r=this.data[e];r||(r={nodes:[],names:[]},this.data[e]=r),r.names.push(t),n&&r.nodes.push(n)},e}(),es=function(){function e(e,t){this.warnings=[],this.settings=t,this.documentText=e.getText(),this.keyframes=new Zo}return e.entries=function(t,n,r,i){var o=new e(n,r);return t.acceptVisitor(o),o.completeValidations(),o.getEntries(i)},e.prototype.fetch=function(e,t){for(var n=[],r=0,i=e;r<i.length;r++){var o=i[r];o.name===t&&n.push(o)}return n},e.prototype.fetchWithValue=function(e,t,n){for(var r=[],i=0,o=e;i<o.length;i++){var s=o[i];if(s.name===t){var a=s.node.getValue();a&&this.findValueInExpression(a,n)&&r.push(s)}}return r},e.prototype.findValueInExpression=function(e,t){var n=!1;return e.accept(function(e){return e.type===cn.Identifier&&e.getText()===t&&(n=!0),!n}),n},e.prototype.getEntries=function(e){return void 0===e&&(e=fn.Warning|fn.Error),this.warnings.filter(function(t){return 0!=(t.getLevel()&e)})},e.prototype.addEntry=function(e,t,n){var r=new br(e,t,this.settings.get(t),n);this.warnings.push(r)},e.prototype.getMissingNames=function(e,t){e=e.slice(0);for(var n=0;n<t.length;n++){var r=e.indexOf(t[n]);-1!==r&&(e[r]=null)}var i=null;for(n=0;n<e.length;n++){var o=e[n];o&&(i=null===i?Xo(\"namelist.single\",\"'{0}'\",o):Xo(\"namelist.concatenated\",\"{0}, '{1}'\",i,o))}return i},e.prototype.visitNode=function(e){switch(e.type){case cn.UnknownAtRule:return this.visitUnknownAtRule(e);case cn.Keyframe:return this.visitKeyframe(e);case cn.FontFace:return this.visitFontFace(e);case cn.Ruleset:return this.visitRuleSet(e);case cn.SimpleSelector:return this.visitSimpleSelector(e);case cn.Function:return this.visitFunction(e);case cn.NumericValue:return this.visitNumericValue(e);case cn.Import:return this.visitImport(e);case cn.HexColorValue:return this.visitHexColorValue(e);case cn.Prio:return this.visitPrio(e)}return!0},e.prototype.completeValidations=function(){this.validateKeyframes()},e.prototype.visitUnknownAtRule=function(e){var t=e.getChild(0);return!!t&&(this.addEntry(t,Go.UnknownAtRules,\"Unknown at rule \"+t.getText()),!0)},e.prototype.visitKeyframe=function(e){var t=e.getKeyword(),n=t.getText();return this.keyframes.add(e.getName(),n,\"@keyframes\"!==n?t:null),!0},e.prototype.validateKeyframes=function(){var e=[\"@-webkit-keyframes\",\"@-moz-keyframes\",\"@-o-keyframes\"];for(var t in this.keyframes.data){var n=this.keyframes.data[t].names,r=-1===n.indexOf(\"@keyframes\");if(r||1!==n.length){var i=this.getMissingNames(e,n);if(i||r)for(var o=0,s=this.keyframes.data[t].nodes;o<s.length;o++){var a=s[o];if(r){var l=Xo(\"keyframes.standardrule.missing\",\"Always define standard rule '@keyframes' when defining keyframes.\");this.addEntry(a,Go.IncludeStandardPropertyWhenUsingVendorPrefix,l)}if(i){l=Xo(\"keyframes.vendorspecific.missing\",\"Always include all vendor specific rules: Missing: {0}\",i);this.addEntry(a,Go.AllVendorPrefixes,l)}}}}return!0},e.prototype.visitSimpleSelector=function(e){var t=this.documentText.charAt(e.offset);return 1===e.length&&\"*\"===t&&this.addEntry(e,Go.UniversalSelector),\"#\"===t&&this.addEntry(e,Go.AvoidIdSelector),!0},e.prototype.visitImport=function(e){return this.addEntry(e,Go.ImportStatemement),!0},e.prototype.visitRuleSet=function(t){var n=t.getDeclarations();if(!n)return!1;n.hasChildren()||this.addEntry(t.getSelectors(),Go.EmptyRuleSet);for(var r=[],i=0,o=n.getChildren();i<o.length;i++){if((z=o[i])instanceof In){var s=z;r.push(new Qo(s.getFullPropertyName().toLowerCase(),s))}}if(0===this.fetch(r,\"box-sizing\").length){var a=this.fetch(r,\"width\");if(a.length>0){for(var l=!1,c=0,h=[\"border\",\"border-left\",\"border-right\",\"padding\",\"padding-left\",\"padding-right\"];c<h.length;c++)for(var d=h[c],u=0,m=this.fetch(r,d);u<m.length;u++){(O=(z=m[u]).node.getValue())&&!O.matches(\"none\")&&(this.addEntry(z.node,Go.BewareOfBoxModelSize),l=!0)}if(l)for(var p=0,f=a;p<f.length;p++){var g=f[p];this.addEntry(g.node,Go.BewareOfBoxModelSize)}}var b=this.fetch(r,\"height\");if(b.length>0){l=!1;for(var y=0,w=[\"border\",\"border-top\",\"border-bottom\",\"padding\",\"padding-top\",\"padding-bottom\"];y<w.length;y++){d=w[y];for(var v=0,x=this.fetch(r,d);v<x.length;v++){(O=(z=x[v]).node.getValue())&&!O.matches(\"none\")&&(this.addEntry(z.node,Go.BewareOfBoxModelSize),l=!0)}}if(l)for(var k=0,C=b;k<C.length;k++){var S=C[k];this.addEntry(S.node,Go.BewareOfBoxModelSize)}}}var _=this.fetchWithValue(r,\"display\",\"inline\");if(_.length>0)for(var F=0,E=[\"width\",\"height\",\"margin-top\",\"margin-bottom\",\"float\"];F<E.length;F++)for(var T=E[F],I=this.fetch(r,T),A=0;A<I.length;A++){var P=I[A].node,O=P.getValue();(\"float\"!==T||O&&!O.matches(\"none\"))&&this.addEntry(P,Go.PropertyIgnoredDueToDisplay,Xo(\"rule.propertyIgnoredDueToDisplayInline\",\"Property is ignored due to the display. With 'display: inline', the width, height, margin-top, margin-bottom, and float properties have no effect.\"))}if((_=this.fetchWithValue(r,\"display\",\"inline-block\")).length>0)for(I=this.fetch(r,\"float\"),A=0;A<I.length;A++){var N=I[A].node;(O=N.getValue())&&!O.matches(\"none\")&&this.addEntry(N,Go.PropertyIgnoredDueToDisplay,Xo(\"rule.propertyIgnoredDueToDisplayInlineBlock\",\"inline-block is ignored due to the float. If 'float' has a value other than 'none', the box is floated and 'display' is treated as 'block'\"))}if((_=this.fetchWithValue(r,\"display\",\"block\")).length>0)for(I=this.fetch(r,\"vertical-align\"),A=0;A<I.length;A++)this.addEntry(I[A].node,Go.PropertyIgnoredDueToDisplay,Xo(\"rule.propertyIgnoredDueToDisplayBlock\",\"Property is ignored due to the display. With 'display: block', vertical-align should not be used.\"));var L=this.fetch(r,\"float\");for(A=0;A<L.length;A++)this.addEntry(L[A].node,Go.AvoidFloat);for(var R=0;R<r.length;R++){var z;if(\"background\"!==(z=r[R]).name)if((O=z.node.getValue())&&\"-\"!==this.documentText.charAt(O.offset)){var D=this.fetch(r,z.name);if(D.length>1)for(var U=0;U<D.length;U++){var M=D[U].node.getValue();M&&\"-\"!==this.documentText.charAt(M.offset)&&D[U]!==z&&this.addEntry(z.node,Go.DuplicateDeclarations)}}}for(var j=new Zo,V=!1,B=0,K=n.getChildren();B<K.length;B++){var q=K[B];if(this.isCSSDeclaration(q)){var W=(s=q).getFullPropertyName(),$=W.charAt(0);if(\"-\"===$){if(\"-\"!==W.charAt(1)){Qr(W)||this.addEntry(s.getProperty(),Go.UnknownVendorSpecificProperty);var G=s.getNonPrefixedPropertyName();j.add(G,W,s.getProperty())}}else\"*\"!==$&&\"_\"!==$||(this.addEntry(s.getProperty(),Go.IEStarHack),W=W.substr(1)),Qr(W)||this.addEntry(s.getProperty(),Go.UnknownProperty,Xo(\"property.unknownproperty.detailed\",\"Unknown property: '{0}'\",W)),j.add(W,W,null)}else V=!0}if(!V)for(var H in j.data){var Y=j.data[H],J=Y.names,X=Zr(H)&&-1===J.indexOf(H);if(X||1!==J.length){for(var Q=[],Z=(R=0,e.prefixes.length);R<Z;R++){var ee=e.prefixes[R];Zr(ee+H)&&Q.push(ee+H)}var te=this.getMissingNames(Q,J);if(te||X)for(var ne=0,re=Y.nodes;ne<re.length;ne++){var ie=re[ne];if(X){var oe=Xo(\"property.standard.missing\",\"Also define the standard property '{0}' for compatibility\",H);this.addEntry(ie,Go.IncludeStandardPropertyWhenUsingVendorPrefix,oe)}if(te){oe=Xo(\"property.vendorspecific.missing\",\"Always include all vendor specific properties: Missing: {0}\",te);this.addEntry(ie,Go.AllVendorPrefixes,oe)}}}}return!0},e.prototype.visitPrio=function(e){return this.addEntry(e,Go.AvoidImportant),!0},e.prototype.visitNumericValue=function(e){var t=e.findParent(cn.Declaration);if(t){var n=t.getValue();if(n&&n.offset===e.offset&&n.length===e.length){var r=e.getValue();if(!r.unit||-1===jr.length.indexOf(r.unit.toLowerCase()))return!0;0===parseFloat(r.value)&&r.unit&&this.addEntry(e,Go.ZeroWithUnit)}}return!0},e.prototype.visitFontFace=function(e){var t=e.getDeclarations();if(t){for(var n=!1,r=!1,i=!1,o=0,s=t.getChildren();o<s.length;o++){var a=s[o];if(this.isCSSDeclaration(a)){var l=a.getProperty().getName().toLowerCase();\"src\"===l&&(n=!0),\"font-family\"===l&&(r=!0)}else i=!0}return i||n&&r||this.addEntry(e,Go.RequiredPropertiesForFontFace),!0}},e.prototype.isCSSDeclaration=function(e){if(e instanceof In){if(!e.getValue())return!1;var t=e.getProperty();return!(!t||t.getIdentifier().containsInterpolation())}return!1},e.prototype.visitHexColorValue=function(e){var t=e.length;return 9!==t&&7!==t&&5!==t&&4!==t&&this.addEntry(e,Go.HexColorLength),!1},e.prototype.visitFunction=function(e){var t=e.getName().toLowerCase(),n=-1,r=0;switch(t){case\"rgb(\":case\"hsl(\":n=3;break;case\"rgba(\":case\"hsla(\":n=4}return-1!==n&&(e.getArguments().accept(function(e){return!(e instanceof nr)||(r+=1,!1)}),r!==n&&this.addEntry(e,Go.ArgsInColorFunction)),!0},e.prefixes=[\"-ms-\",\"-moz-\",\"-o-\",\"-webkit-\"],e}(),ts=function(){function e(){}return e.prototype.configure=function(e){this.settings=e},e.prototype.doValidation=function(e,t,n){if(void 0===n&&(n=this.settings),n&&!1===n.validate)return[];var r=[];return r.push.apply(r,yr.entries(t)),r.push.apply(r,es.entries(t,e,new Ho(n&&n.lint))),r.filter(function(e){return e.getLevel()!==fn.Ignore}).map(function(t){var n=yi.create(e.positionAt(t.getOffset()),e.positionAt(t.getOffset()+t.getLength()));return{code:t.getRule().id,source:e.languageId,message:t.getMessage(),severity:t.getLevel()===fn.Warning?Fi.Warning:Fi.Error,range:n}})},e}(),ns=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),rs=\"/\".charCodeAt(0),is=\"\\n\".charCodeAt(0),os=\"\\r\".charCodeAt(0),ss=\"\\f\".charCodeAt(0),as=\"$\".charCodeAt(0),ls=\"#\".charCodeAt(0),cs=\"{\".charCodeAt(0),hs=\"=\".charCodeAt(0),ds=\"!\".charCodeAt(0),us=\"<\".charCodeAt(0),ms=\">\".charCodeAt(0),ps=\".\".charCodeAt(0),fs=ft.CustomToken,gs=fs++,bs=fs++,ys=(fs++,fs++),ws=fs++,vs=fs++,xs=fs++,ks=fs++,Cs=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ns(t,e),t.prototype.scanNext=function(t){if(this.stream.advanceIfChar(as)){var n=[\"$\"];if(this.ident(n))return this.finishToken(t,gs,n.join(\"\"));this.stream.goBackTo(t)}return this.stream.advanceIfChars([ls,cs])?this.finishToken(t,bs):this.stream.advanceIfChars([hs,hs])?this.finishToken(t,ys):this.stream.advanceIfChars([ds,hs])?this.finishToken(t,ws):this.stream.advanceIfChar(us)?this.stream.advanceIfChar(hs)?this.finishToken(t,xs):this.finishToken(t,ft.Delim):this.stream.advanceIfChar(ms)?this.stream.advanceIfChar(hs)?this.finishToken(t,vs):this.finishToken(t,ft.Delim):this.stream.advanceIfChars([ps,ps,ps])?this.finishToken(t,ks):e.prototype.scanNext.call(this,t)},t.prototype.comment=function(){return!!e.prototype.comment.call(this)||!(this.inURL||!this.stream.advanceIfChars([rs,rs]))&&(this.stream.advanceWhileChar(function(e){switch(e){case is:case os:case ss:return!1;default:return!0}}),!0)},t}(dn),Ss=vr(),_s=function(){return function(e,t){this.id=e,this.message=t}}(),Fs={FromExpected:new _s(\"scss-fromexpected\",Ss(\"expected.from\",\"'from' expected\")),ThroughOrToExpected:new _s(\"scss-throughexpected\",Ss(\"expected.through\",\"'through' or 'to' expected\")),InExpected:new _s(\"scss-fromexpected\",Ss(\"expected.in\",\"'in' expected\"))},Es=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Ts=function(e){function t(){return e.call(this,new Cs)||this}return Es(t,e),t.prototype._parseStylesheetStatement=function(){return this.peek(ft.AtKeyword)?this._parseWarnAndDebug()||this._parseControlStatement()||this._parseMixinDeclaration()||this._parseMixinContent()||this._parseMixinReference()||this._parseFunctionDeclaration()||e.prototype._parseStylesheetAtStatement.call(this):this._parseRuleset(!0)||this._parseVariableDeclaration()},t.prototype._parseImport=function(){if(!this.peekKeyword(\"@import\"))return null;var e=this.create(Wn);if(this.consumeToken(),!e.addChild(this._parseURILiteral())&&!e.addChild(this._parseStringLiteral()))return this.finish(e,Cr.URIOrStringExpected);for(;this.accept(ft.Comma);)if(!e.addChild(this._parseURILiteral())&&!e.addChild(this._parseStringLiteral()))return this.finish(e,Cr.URIOrStringExpected);return this.peek(ft.SemiColon)||this.peek(ft.EOF)||e.setMedialist(this._parseMediaQueryList()),this.finish(e)},t.prototype._parseVariableDeclaration=function(e){if(void 0===e&&(e=[]),!this.peek(gs))return null;var t=this.create(ar);if(!t.setVariable(this._parseVariable()))return null;if(!this.accept(ft.Colon))return this.finish(t,Cr.ColonExpected);if(t.colonPosition=this.prevToken.offset,!t.setValue(this._parseExpr()))return this.finish(t,Cr.VariableValueExpected,[],e);for(;this.accept(ft.Exclamation);){if(!this.peekRegExp(ft.Ident,/^(default|global)$/))return this.finish(t,Cr.UnknownKeyword);this.consumeToken()}return this.peek(ft.SemiColon)&&(t.semicolonPosition=this.token.offset),this.finish(t)},t.prototype._parseMediaFeatureName=function(){return this._parseFunction()||this._parseIdent()||this._parseVariable()},t.prototype._parseKeyframeSelector=function(){return this._tryParseKeyframeSelector()||this._parseControlStatement(this._parseKeyframeSelector.bind(this))||this._parseVariableDeclaration()||this._parseMixinContent()},t.prototype._parseVariable=function(){if(!this.peek(gs))return null;var e=this.create(cr);return this.consumeToken(),e},t.prototype._parseIdent=function(e){var t=this;if(!this.peek(ft.Ident)&&!this.peek(bs)&&!this.peekDelim(\"-\"))return null;var n=this.create(yn);n.referenceTypes=e;for(var r=!1,i=function(){return t.acceptDelim(\"-\")?(!t.hasWhitespace()&&t.acceptDelim(\"-\")&&(n.isCustomProperty=!0),t.hasWhitespace()?null:t._parseInterpolation()):null};(this.accept(ft.Ident)||n.addChild(this._parseInterpolation()||this.try(i)))&&(r=!0,!this.hasWhitespace()&&this.acceptDelim(\"-\"),!this.hasWhitespace()););return r?this.finish(n):null},t.prototype._parseTerm=function(){var t=e.prototype._parseTerm.call(this);return t||((t=this.create(rr)).setExpression(this._parseVariable())||t.setExpression(this._parseSelectorCombinator())||t.setExpression(this._tryParsePrio())?this.finish(t):null)},t.prototype._parseInterpolation=function(){if(this.peek(bs)){var e=this.create(lr);return this.consumeToken(),e.addChild(this._parseExpr())||this._parseSelectorCombinator()?this.accept(ft.CurlyR)?this.finish(e):this.finish(e,Cr.RightCurlyExpected):this.accept(ft.CurlyR)?this.finish(e):this.finish(e,Cr.ExpressionExpected)}return null},t.prototype._parseOperator=function(){if(this.peek(ys)||this.peek(ws)||this.peek(vs)||this.peek(xs)||this.peekDelim(\">\")||this.peekDelim(\"<\")||this.peekIdent(\"and\")||this.peekIdent(\"or\")||this.peekDelim(\"%\")){var t=this.createNode(cn.Operator);return this.consumeToken(),this.finish(t)}return e.prototype._parseOperator.call(this)},t.prototype._parseUnaryOperator=function(){if(this.peekIdent(\"not\")){var t=this.create(gn);return this.consumeToken(),this.finish(t)}return e.prototype._parseUnaryOperator.call(this)},t.prototype._parseRuleSetDeclaration=function(){return this.peek(ft.AtKeyword)?this._parseKeyframe()||this._parseImport()||this._parseMedia(!0)||this._parseFontFace()||this._parseWarnAndDebug()||this._parseControlStatement()||this._parseFunctionDeclaration()||this._parseExtends()||this._parseMixinReference()||this._parseMixinContent()||this._parseMixinDeclaration()||this._parseRuleset(!0)||this._parseSupports(!0):this._parseVariableDeclaration()||this._tryParseRuleset(!0)||e.prototype._parseRuleSetDeclaration.call(this)},t.prototype._parseDeclaration=function(e){var t=this.create(In);if(!t.setProperty(this._parseProperty()))return null;if(!this.accept(ft.Colon))return this.finish(t,Cr.ColonExpected,[ft.Colon],e);t.colonPosition=this.prevToken.offset;var n=!1;if(t.setValue(this._parseExpr())&&(n=!0,t.addChild(this._parsePrio())),this.peek(ft.CurlyL))t.setNestedProperties(this._parseNestedProperties());else if(!n)return this.finish(t,Cr.PropertyValueExpected);return this.peek(ft.SemiColon)&&(t.semicolonPosition=this.token.offset),this.finish(t)},t.prototype._parseNestedProperties=function(){var e=this.create(Bn);return this._parseBody(e,this._parseDeclaration.bind(this))},t.prototype._parseExtends=function(){if(this.peekKeyword(\"@extend\")){var e=this.create(hr);if(this.consumeToken(),!e.getSelectors().addChild(this._parseSimpleSelector()))return this.finish(e,Cr.SelectorExpected);for(;this.accept(ft.Comma);)e.getSelectors().addChild(this._parseSimpleSelector());return this.accept(ft.Exclamation)&&!this.acceptIdent(\"optional\")?this.finish(e,Cr.UnknownKeyword):this.finish(e)}return null},t.prototype._parseSimpleSelectorBody=function(){return this._parseSelectorCombinator()||this._parseSelectorPlaceholder()||e.prototype._parseSimpleSelectorBody.call(this)},t.prototype._parseSelectorCombinator=function(){if(this.peekDelim(\"&\")){var e=this.createNode(cn.SelectorCombinator);for(this.consumeToken();!this.hasWhitespace()&&(this.acceptDelim(\"-\")||this.accept(ft.Num)||this.accept(ft.Dimension)||e.addChild(this._parseIdent())||this.acceptDelim(\"&\")););return this.finish(e)}return null},t.prototype._parseSelectorPlaceholder=function(){if(this.peekDelim(\"%\")){var e=this.createNode(cn.SelectorPlaceholder);return this.consumeToken(),this._parseIdent(),this.finish(e)}if(this.peekKeyword(\"@at-root\")){e=this.createNode(cn.SelectorPlaceholder);return this.consumeToken(),this.finish(e)}return null},t.prototype._parseElementName=function(){var t=this.mark(),n=e.prototype._parseElementName.call(this);return n&&!this.hasWhitespace()&&this.peek(ft.ParenthesisL)?(this.restoreAtMark(t),null):n},t.prototype._tryParsePseudoIdentifier=function(){return this._parseInterpolation()||e.prototype._tryParsePseudoIdentifier.call(this)},t.prototype._parseWarnAndDebug=function(){if(!this.peekKeyword(\"@debug\")&&!this.peekKeyword(\"@warn\")&&!this.peekKeyword(\"@error\"))return null;var e=this.createNode(cn.Debug);return this.consumeToken(),e.addChild(this._parseExpr()),this.finish(e)},t.prototype._parseControlStatement=function(e){return void 0===e&&(e=this._parseRuleSetDeclaration.bind(this)),this.peek(ft.AtKeyword)?this._parseIfStatement(e)||this._parseForStatement(e)||this._parseEachStatement(e)||this._parseWhileStatement(e):null},t.prototype._parseIfStatement=function(e){return this.peekKeyword(\"@if\")?this._internalParseIfStatement(e):null},t.prototype._internalParseIfStatement=function(e){var t=this.create(Ln);if(this.consumeToken(),!t.setExpression(this._parseExpr(!0)))return this.finish(t,Cr.ExpressionExpected);if(this._parseBody(t,e),this.acceptKeyword(\"@else\"))if(this.peekIdent(\"if\"))t.setElseClause(this._internalParseIfStatement(e));else if(this.peek(ft.CurlyL)){var n=this.create(Un);this._parseBody(n,e),t.setElseClause(n)}return this.finish(t)},t.prototype._parseForStatement=function(e){if(!this.peekKeyword(\"@for\"))return null;var t=this.create(Rn);return this.consumeToken(),t.setVariable(this._parseVariable())?this.acceptIdent(\"from\")?t.addChild(this._parseBinaryExpr())?this.acceptIdent(\"to\")||this.acceptIdent(\"through\")?t.addChild(this._parseBinaryExpr())?this._parseBody(t,e):this.finish(t,Cr.ExpressionExpected,[ft.CurlyR]):this.finish(t,Fs.ThroughOrToExpected,[ft.CurlyR]):this.finish(t,Cr.ExpressionExpected,[ft.CurlyR]):this.finish(t,Fs.FromExpected,[ft.CurlyR]):this.finish(t,Cr.VariableNameExpected,[ft.CurlyR])},t.prototype._parseEachStatement=function(e){if(!this.peekKeyword(\"@each\"))return null;var t=this.create(zn);this.consumeToken();var n=t.getVariables();if(!n.addChild(this._parseVariable()))return this.finish(t,Cr.VariableNameExpected,[ft.CurlyR]);for(;this.accept(ft.Comma);)if(!n.addChild(this._parseVariable()))return this.finish(t,Cr.VariableNameExpected,[ft.CurlyR]);return this.finish(n),this.acceptIdent(\"in\")?t.addChild(this._parseExpr())?this._parseBody(t,e):this.finish(t,Cr.ExpressionExpected,[ft.CurlyR]):this.finish(t,Fs.InExpected,[ft.CurlyR])},t.prototype._parseWhileStatement=function(e){if(!this.peekKeyword(\"@while\"))return null;var t=this.create(Dn);return this.consumeToken(),t.addChild(this._parseBinaryExpr())?this._parseBody(t,e):this.finish(t,Cr.ExpressionExpected,[ft.CurlyR])},t.prototype._parseFunctionBodyDeclaration=function(){return this._parseVariableDeclaration()||this._parseReturnStatement()||this._parseWarnAndDebug()||this._parseControlStatement(this._parseFunctionBodyDeclaration.bind(this))},t.prototype._parseFunctionDeclaration=function(){if(!this.peekKeyword(\"@function\"))return null;var e=this.create(Mn);if(this.consumeToken(),!e.setIdentifier(this._parseIdent([hn.Function])))return this.finish(e,Cr.IdentifierExpected,[ft.CurlyR]);if(!this.accept(ft.ParenthesisL))return this.finish(e,Cr.LeftParenthesisExpected,[ft.CurlyR]);if(e.getParameters().addChild(this._parseParameterDeclaration()))for(;this.accept(ft.Comma);)if(!e.getParameters().addChild(this._parseParameterDeclaration()))return this.finish(e,Cr.VariableNameExpected);return this.accept(ft.ParenthesisR)?this._parseBody(e,this._parseFunctionBodyDeclaration.bind(this)):this.finish(e,Cr.RightParenthesisExpected,[ft.CurlyR])},t.prototype._parseReturnStatement=function(){if(!this.peekKeyword(\"@return\"))return null;var e=this.createNode(cn.ReturnStatement);return this.consumeToken(),e.addChild(this._parseExpr())?this.finish(e):this.finish(e,Cr.ExpressionExpected)},t.prototype._parseMixinDeclaration=function(){if(!this.peekKeyword(\"@mixin\"))return null;var e=this.create(ur);if(this.consumeToken(),!e.setIdentifier(this._parseIdent([hn.Mixin])))return this.finish(e,Cr.IdentifierExpected,[ft.CurlyR]);if(this.accept(ft.ParenthesisL)){if(e.getParameters().addChild(this._parseParameterDeclaration()))for(;this.accept(ft.Comma);)if(!e.getParameters().addChild(this._parseParameterDeclaration()))return this.finish(e,Cr.VariableNameExpected);if(!this.accept(ft.ParenthesisR))return this.finish(e,Cr.RightParenthesisExpected,[ft.CurlyR])}return this._parseBody(e,this._parseRuleSetDeclaration.bind(this))},t.prototype._parseParameterDeclaration=function(){var e=this.create(On);return e.setIdentifier(this._parseVariable())?(this.accept(ks),this.accept(ft.Colon)&&!e.setDefaultValue(this._parseExpr(!0))?this.finish(e,Cr.VariableValueExpected,[],[ft.Comma,ft.ParenthesisR]):this.finish(e)):null},t.prototype._parseMixinContent=function(){if(!this.peekKeyword(\"@content\"))return null;var e=this.createNode(cn.MixinContent);return this.consumeToken(),this.finish(e)},t.prototype._parseMixinReference=function(){if(!this.peekKeyword(\"@include\"))return null;var e=this.create(dr);if(this.consumeToken(),!e.setIdentifier(this._parseIdent([hn.Mixin])))return this.finish(e,Cr.IdentifierExpected,[ft.CurlyR]);if(this.accept(ft.ParenthesisL)){if(e.getArguments().addChild(this._parseFunctionArgument()))for(;this.accept(ft.Comma);)if(!e.getArguments().addChild(this._parseFunctionArgument()))return this.finish(e,Cr.ExpressionExpected);if(!this.accept(ft.ParenthesisR))return this.finish(e,Cr.RightParenthesisExpected)}if(this.peek(ft.CurlyL)){var t=this.create(xn);this._parseBody(t,this._parseMixinReferenceBodyStatement.bind(this)),e.setContent(t)}return this.finish(e)},t.prototype._parseMixinReferenceBodyStatement=function(){return this._tryParseKeyframeSelector()||this._parseRuleSetDeclaration()},t.prototype._parseFunctionArgument=function(){var e=this.create(Nn),t=this.mark(),n=this._parseVariable();if(n)if(this.accept(ft.Colon))e.setIdentifier(n);else{if(this.accept(ks))return e.setValue(n),this.finish(e);this.restoreAtMark(t)}return e.setValue(this._parseExpr(!0))?(this.accept(ks),e.addChild(this._parsePrio()),this.finish(e)):null},t.prototype._parseURLArgument=function(){var t=this.mark(),n=e.prototype._parseURLArgument.call(this);if(!n||!this.peek(ft.ParenthesisR)){this.restoreAtMark(t);var r=this.create(gn);return r.addChild(this._parseBinaryExpr()),this.finish(r)}return n},t.prototype._parseOperation=function(){if(!this.peek(ft.ParenthesisL))return null;var e=this.create(gn);for(this.consumeToken();e.addChild(this._parseListElement());)this.accept(ft.Comma);return this.accept(ft.ParenthesisR)?this.finish(e):this.finish(e,Cr.RightParenthesisExpected)},t.prototype._parseListElement=function(){var e=this.create(pr),t=this._parseBinaryExpr();if(!t)return null;if(this.accept(ft.Colon)){if(e.setKey(t),!e.setValue(this._parseBinaryExpr()))return this.finish(e,Cr.ExpressionExpected)}else e.setValue(t);return this.finish(e)},t}(gi),Is=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),As=vr(),Ps=function(e){function t(){return e.call(this,\"$\")||this}return Is(t,e),t.prototype.createReplaceFunction=function(){var e=1;return function(n,r){return\"\\\\\"+r+\": ${\"+e+++\":\"+(t.variableDefaults[r]||\"\")+\"}\"}},t.prototype.createFunctionProposals=function(e,t,n,r){for(var i=0,o=e;i<o.length;i++){var s=o[i],a=s.func.replace(/\\[?(\\$\\w+)\\]?/g,this.createReplaceFunction()),l={label:s.func.substr(0,s.func.indexOf(\"(\")),detail:s.func,documentation:s.desc,textEdit:Ii.replace(this.getCompletionRange(t),a),insertTextFormat:Gi.Snippet,kind:$i.Function};n&&(l.sortText=\"z\"),r.items.push(l)}return r},t.prototype.getCompletionsForSelector=function(n,r,i){return this.createFunctionProposals(t.selectorFuncs,void 0,!0,i),e.prototype.getCompletionsForSelector.call(this,n,r,i)},t.prototype.getTermProposals=function(n,r,i){var o=t.builtInFuncs;return n&&(o=o.filter(function(e){return!e.type||-1!==n.restrictions.indexOf(e.type)})),this.createFunctionProposals(o,r,!0,i),e.prototype.getTermProposals.call(this,n,r,i)},t.prototype.getColorProposals=function(n,r,i){return this.createFunctionProposals(t.colorProposals,r,!1,i),e.prototype.getColorProposals.call(this,n,r,i)},t.prototype.getCompletionsForDeclarationProperty=function(t,n){return this.getCompletionForAtDirectives(n),this.getCompletionsForSelector(null,!0,n),e.prototype.getCompletionsForDeclarationProperty.call(this,t,n)},t.prototype.getCompletionsForExtendsReference=function(e,t,n){for(var r=0,i=this.getSymbolContext().findSymbolsAtOffset(this.offset,hn.Rule);r<i.length;r++){var o=i[r],s={label:o.name,textEdit:Ii.replace(this.getCompletionRange(t),o.name),kind:$i.Function};n.items.push(s)}return n},t.prototype.getCompletionForAtDirectives=function(e){var n;return(n=e.items).push.apply(n,t.scssAtDirectives),e},t.prototype.getCompletionForTopLevel=function(t){return this.getCompletionForAtDirectives(t),e.prototype.getCompletionForTopLevel.call(this,t),t},t.variableDefaults={$red:\"1\",$green:\"2\",$blue:\"3\",$alpha:\"1.0\",$color:\"#000000\",$weight:\"0.5\",$hue:\"0\",$saturation:\"0%\",$lightness:\"0%\",$degrees:\"0\",$amount:\"0\",$string:'\"\"',$substring:'\"s\"',$number:\"0\",$limit:\"1\"},t.colorProposals=[{func:\"red($color)\",desc:As(\"scss.builtin.red\",\"Gets the red component of a color.\")},{func:\"green($color)\",desc:As(\"scss.builtin.green\",\"Gets the green component of a color.\")},{func:\"blue($color)\",desc:As(\"scss.builtin.blue\",\"Gets the blue component of a color.\")},{func:\"mix($color, $color, [$weight])\",desc:As(\"scss.builtin.mix\",\"Mixes two colors together.\")},{func:\"hue($color)\",desc:As(\"scss.builtin.hue\",\"Gets the hue component of a color.\")},{func:\"saturation($color)\",desc:As(\"scss.builtin.saturation\",\"Gets the saturation component of a color.\")},{func:\"lightness($color)\",desc:As(\"scss.builtin.lightness\",\"Gets the lightness component of a color.\")},{func:\"adjust-hue($color, $degrees)\",desc:As(\"scss.builtin.adjust-hue\",\"Changes the hue of a color.\")},{func:\"lighten($color, $amount)\",desc:As(\"scss.builtin.lighten\",\"Makes a color lighter.\")},{func:\"darken($color, $amount)\",desc:As(\"scss.builtin.darken\",\"Makes a color darker.\")},{func:\"saturate($color, $amount)\",desc:As(\"scss.builtin.saturate\",\"Makes a color more saturated.\")},{func:\"desaturate($color, $amount)\",desc:As(\"scss.builtin.desaturate\",\"Makes a color less saturated.\")},{func:\"grayscale($color)\",desc:As(\"scss.builtin.grayscale\",\"Converts a color to grayscale.\")},{func:\"complement($color)\",desc:As(\"scss.builtin.complement\",\"Returns the complement of a color.\")},{func:\"invert($color)\",desc:As(\"scss.builtin.invert\",\"Returns the inverse of a color.\")},{func:\"alpha($color)\",desc:As(\"scss.builtin.alpha\",\"Gets the opacity component of a color.\")},{func:\"opacity($color)\",desc:\"Gets the alpha component (opacity) of a color.\"},{func:\"rgba($color, $alpha)\",desc:As(\"scss.builtin.rgba\",\"Changes the alpha component for a color.\")},{func:\"opacify($color, $amount)\",desc:As(\"scss.builtin.opacify\",\"Makes a color more opaque.\")},{func:\"fade-in($color, $amount)\",desc:As(\"scss.builtin.fade-in\",\"Makes a color more opaque.\")},{func:\"transparentize($color, $amount)\",desc:As(\"scss.builtin.transparentize\",\"Makes a color more transparent.\")},{func:\"fade-out($color, $amount)\",desc:As(\"scss.builtin.fade-out\",\"Makes a color more transparent.\")},{func:\"adjust-color($color, [$red], [$green], [$blue], [$hue], [$saturation], [$lightness], [$alpha])\",desc:As(\"scss.builtin.adjust-color\",\"Increases or decreases one or more components of a color.\")},{func:\"scale-color($color, [$red], [$green], [$blue], [$saturation], [$lightness], [$alpha])\",desc:As(\"scss.builtin.scale-color\",\"Fluidly scales one or more properties of a color.\")},{func:\"change-color($color, [$red], [$green], [$blue], [$hue], [$saturation], [$lightness], [$alpha])\",desc:As(\"scss.builtin.change-color\",\"Changes one or more properties of a color.\")},{func:\"ie-hex-str($color)\",desc:As(\"scss.builtin.ie-hex-str\",\"Converts a color into the format understood by IE filters.\")}],t.selectorFuncs=[{func:\"selector-nest($selectors…)\",desc:As(\"scss.builtin.selector-nest\",\"Nests selector beneath one another like they would be nested in the stylesheet.\")},{func:\"selector-append($selectors…)\",desc:As(\"scss.builtin.selector-append\",\"Appends selectors to one another without spaces in between.\")},{func:\"selector-extend($selector, $extendee, $extender)\",desc:As(\"scss.builtin.selector-extend\",\"Extends $extendee with $extender within $selector.\")},{func:\"selector-replace($selector, $original, $replacement)\",desc:As(\"scss.builtin.selector-replace\",\"Replaces $original with $replacement within $selector.\")},{func:\"selector-unify($selector1, $selector2)\",desc:As(\"scss.builtin.selector-unify\",\"Unifies two selectors to produce a selector that matches elements matched by both.\")},{func:\"is-superselector($super, $sub)\",desc:As(\"scss.builtin.is-superselector\",\"Returns whether $super matches all the elements $sub does, and possibly more.\")},{func:\"simple-selectors($selector)\",desc:As(\"scss.builtin.simple-selectors\",\"Returns the simple selectors that comprise a compound selector.\")},{func:\"selector-parse($selector)\",desc:As(\"scss.builtin.selector-parse\",\"Parses a selector into the format returned by &.\")}],t.builtInFuncs=[{func:\"unquote($string)\",desc:As(\"scss.builtin.unquote\",\"Removes quotes from a string.\")},{func:\"quote($string)\",desc:As(\"scss.builtin.quote\",\"Adds quotes to a string.\")},{func:\"str-length($string)\",desc:As(\"scss.builtin.str-length\",\"Returns the number of characters in a string.\")},{func:\"str-insert($string, $insert, $index)\",desc:As(\"scss.builtin.str-insert\",\"Inserts $insert into $string at $index.\")},{func:\"str-index($string, $substring)\",desc:As(\"scss.builtin.str-index\",\"Returns the index of the first occurance of $substring in $string.\")},{func:\"str-slice($string, $start-at, [$end-at])\",desc:As(\"scss.builtin.str-slice\",\"Extracts a substring from $string.\")},{func:\"to-upper-case($string)\",desc:As(\"scss.builtin.to-upper-case\",\"Converts a string to upper case.\")},{func:\"to-lower-case($string)\",desc:As(\"scss.builtin.to-lower-case\",\"Converts a string to lower case.\")},{func:\"percentage($number)\",desc:As(\"scss.builtin.percentage\",\"Converts a unitless number to a percentage.\"),type:\"percentage\"},{func:\"round($number)\",desc:As(\"scss.builtin.round\",\"Rounds a number to the nearest whole number.\")},{func:\"ceil($number)\",desc:As(\"scss.builtin.ceil\",\"Rounds a number up to the next whole number.\")},{func:\"floor($number)\",desc:As(\"scss.builtin.floor\",\"Rounds a number down to the previous whole number.\")},{func:\"abs($number)\",desc:As(\"scss.builtin.abs\",\"Returns the absolute value of a number.\")},{func:\"min($numbers)\",desc:As(\"scss.builtin.min\",\"Finds the minimum of several numbers.\")},{func:\"max($numbers)\",desc:As(\"scss.builtin.max\",\"Finds the maximum of several numbers.\")},{func:\"random([$limit])\",desc:As(\"scss.builtin.random\",\"Returns a random number.\")},{func:\"length($list)\",desc:As(\"scss.builtin.length\",\"Returns the length of a list.\")},{func:\"nth($list, $n)\",desc:As(\"scss.builtin.nth\",\"Returns a specific item in a list.\")},{func:\"set-nth($list, $n, $value)\",desc:As(\"scss.builtin.set-nth\",\"Replaces the nth item in a list.\")},{func:\"join($list1, $list2, [$separator])\",desc:As(\"scss.builtin.join\",\"Joins together two lists into one.\")},{func:\"append($list1, $val, [$separator])\",desc:As(\"scss.builtin.append\",\"Appends a single value onto the end of a list.\")},{func:\"zip($lists)\",desc:As(\"scss.builtin.zip\",\"Combines several lists into a single multidimensional list.\")},{func:\"index($list, $value)\",desc:As(\"scss.builtin.index\",\"Returns the position of a value within a list.\")},{func:\"list-separator(#list)\",desc:As(\"scss.builtin.list-separator\",\"Returns the separator of a list.\")},{func:\"map-get($map, $key)\",desc:As(\"scss.builtin.map-get\",\"Returns the value in a map associated with a given key.\")},{func:\"map-merge($map1, $map2)\",desc:As(\"scss.builtin.map-merge\",\"Merges two maps together into a new map.\")},{func:\"map-remove($map, $keys)\",desc:As(\"scss.builtin.map-remove\",\"Returns a new map with keys removed.\")},{func:\"map-keys($map)\",desc:As(\"scss.builtin.map-keys\",\"Returns a list of all keys in a map.\")},{func:\"map-values($map)\",desc:As(\"scss.builtin.map-values\",\"Returns a list of all values in a map.\")},{func:\"map-has-key($map, $key)\",desc:As(\"scss.builtin.map-has-key\",\"Returns whether a map has a value associated with a given key.\")},{func:\"keywords($args)\",desc:As(\"scss.builtin.keywords\",\"Returns the keywords passed to a function that takes variable arguments.\")},{func:\"feature-exists($feature)\",desc:As(\"scss.builtin.feature-exists\",\"Returns whether a feature exists in the current Sass runtime.\")},{func:\"variable-exists($name)\",desc:As(\"scss.builtin.variable-exists\",\"Returns whether a variable with the given name exists in the current scope.\")},{func:\"global-variable-exists($name)\",desc:As(\"scss.builtin.global-variable-exists\",\"Returns whether a variable with the given name exists in the global scope.\")},{func:\"function-exists($name)\",desc:As(\"scss.builtin.function-exists\",\"Returns whether a function with the given name exists.\")},{func:\"mixin-exists($name)\",desc:As(\"scss.builtin.mixin-exists\",\"Returns whether a mixin with the given name exists.\")},{func:\"inspect($value)\",desc:As(\"scss.builtin.inspect\",\"Returns the string representation of a value as it would be represented in Sass.\")},{func:\"type-of($value)\",desc:As(\"scss.builtin.type-of\",\"Returns the type of a value.\")},{func:\"unit($number)\",desc:As(\"scss.builtin.unit\",\"Returns the unit(s) associated with a number.\")},{func:\"unitless($number)\",desc:As(\"scss.builtin.unitless\",\"Returns whether a number has units.\")},{func:\"comparable($number1, $number2)\",desc:As(\"scss.builtin.comparable\",\"Returns whether two numbers can be added, subtracted, or compared.\")},{func:\"call($name, $args…)\",desc:As(\"scss.builtin.call\",\"Dynamically calls a Sass function.\")}],t.scssAtDirectives=[{label:\"@extend\",documentation:As(\"scss.builtin.@extend\",\"Inherits the styles of another selector.\"),kind:$i.Keyword},{label:\"@at-root\",documentation:As(\"scss.builtin.@at-root\",\"Causes one or more rules to be emitted at the root of the document.\"),kind:$i.Keyword},{label:\"@debug\",documentation:As(\"scss.builtin.@debug\",\"Prints the value of an expression to the standard error output stream. Useful for debugging complicated Sass files.\"),kind:$i.Keyword},{label:\"@warn\",documentation:As(\"scss.builtin.@warn\",\"Prints the value of an expression to the standard error output stream. Useful for libraries that need to warn users of deprecations or recovering from minor mixin usage mistakes. Warnings can be turned off with the `--quiet` command-line option or the `:quiet` Sass option.\"),kind:$i.Keyword},{label:\"@error\",documentation:As(\"scss.builtin.@error\",\"Throws the value of an expression as a fatal error with stack trace. Useful for validating arguments to mixins and functions.\"),kind:$i.Keyword},{label:\"@if\",documentation:As(\"scss.builtin.@if\",\"Includes the body if the expression does not evaluate to `false` or `null`.\"),insertText:\"@if ${1:expr} {\\n\\t$0\\n}\",insertTextFormat:Gi.Snippet,kind:$i.Keyword},{label:\"@for\",documentation:As(\"scss.builtin.@for\",\"For loop that repeatedly outputs a set of styles for each `$var` in the `from/through` or `from/to` clause.\"),insertText:\"@for \\\\$${1:var} from ${2:start} ${3|to,through|} ${4:end} {\\n\\t$0\\n}\",insertTextFormat:Gi.Snippet,kind:$i.Keyword},{label:\"@each\",documentation:As(\"scss.builtin.@each\",\"Each loop that sets `$var` to each item in the list or map, then outputs the styles it contains using that value of `$var`.\"),insertText:\"@each \\\\$${1:var} in ${2:list} {\\n\\t$0\\n}\",insertTextFormat:Gi.Snippet,kind:$i.Keyword},{label:\"@while\",documentation:As(\"scss.builtin.@while\",\"While loop that takes an expression and repeatedly outputs the nested styles until the statement evaluates to `false`.\"),insertText:\"@while ${1:condition} {\\n\\t$0\\n}\",insertTextFormat:Gi.Snippet,kind:$i.Keyword},{label:\"@mixin\",documentation:As(\"scss.builtin.@mixin\",\"Defines styles that can be re-used throughout the stylesheet with `@include`.\"),insertText:\"@mixin ${1:name} {\\n\\t$0\\n}\",insertTextFormat:Gi.Snippet,kind:$i.Keyword},{label:\"@include\",documentation:As(\"scss.builtin.@include\",\"Includes the styles defined by another mixin into the current rule.\"),kind:$i.Keyword}],t}(wo),Os=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Ns=\"/\".charCodeAt(0),Ls=\"\\n\".charCodeAt(0),Rs=\"\\r\".charCodeAt(0),zs=\"\\f\".charCodeAt(0),Ds=\"`\".charCodeAt(0),Us=\".\".charCodeAt(0),Ms=ft.CustomToken,js=Ms++,Vs=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Os(t,e),t.prototype.scanNext=function(t){var n=this.escapedJavaScript();return null!==n?this.finishToken(t,n):this.stream.advanceIfChars([Us,Us,Us])?this.finishToken(t,js):e.prototype.scanNext.call(this,t)},t.prototype.comment=function(){return!!e.prototype.comment.call(this)||!(this.inURL||!this.stream.advanceIfChars([Ns,Ns]))&&(this.stream.advanceWhileChar(function(e){switch(e){case Ls:case Rs:case zs:return!1;default:return!0}}),!0)},t.prototype.escapedJavaScript=function(){return this.stream.peekChar()===Ds?(this.stream.advance(1),this.stream.advanceWhileChar(function(e){return e!==Ds}),this.stream.advanceIfChar(Ds)?ft.EscapedJavaScript:ft.BadEscapedJavaScript):null},t}(dn),Bs=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Ks=function(e){function t(){return e.call(this,new Vs)||this}return Bs(t,e),t.prototype._parseStylesheetStatement=function(){return this.peek(ft.AtKeyword)?this._parseVariableDeclaration()||this._parsePlugin()||e.prototype._parseStylesheetAtStatement.call(this):this._tryParseMixinDeclaration()||this._tryParseMixinReference(!0)||this._parseRuleset(!0)},t.prototype._parseImport=function(){if(!this.peekKeyword(\"@import\")&&!this.peekKeyword(\"@import-once\"))return null;var e=this.create(Wn);if(this.consumeToken(),this.accept(ft.ParenthesisL)){if(!this.accept(ft.Ident))return this.finish(e,Cr.IdentifierExpected,[ft.SemiColon]);do{if(!this.accept(ft.Comma))break}while(this.accept(ft.Ident));if(!this.accept(ft.ParenthesisR))return this.finish(e,Cr.RightParenthesisExpected,[ft.SemiColon])}return e.addChild(this._parseURILiteral())||e.addChild(this._parseStringLiteral())?(this.peek(ft.SemiColon)||this.peek(ft.EOF)||e.setMedialist(this._parseMediaQueryList()),this.finish(e)):this.finish(e,Cr.URIOrStringExpected,[ft.SemiColon])},t.prototype._parsePlugin=function(){if(!this.peekKeyword(\"@plugin\"))return null;var e=this.createNode(cn.Plugin);return this.consumeToken(),e.addChild(this._parseStringLiteral())?this.accept(ft.SemiColon)?this.finish(e):this.finish(e,Cr.SemiColonExpected):this.finish(e,Cr.StringLiteralExpected)},t.prototype._parseMediaQuery=function(t){var n=e.prototype._parseMediaQuery.call(this,t);if(!n){var r=this.create(Xn);return r.addChild(this._parseVariable())?this.finish(r):null}return n},t.prototype._parseMediaDeclaration=function(e){return void 0===e&&(e=!1),this._tryParseRuleset(e)||this._tryToParseDeclaration()||this._tryParseMixinDeclaration()||this._tryParseMixinReference()||this._parseDetachedRuleSetMixin()||this._parseStylesheetStatement()},t.prototype._parseMediaFeatureName=function(){return this._parseIdent()||this._parseVariable()},t.prototype._parseVariableDeclaration=function(e){void 0===e&&(e=[]);var t=this.create(ar),n=this.mark();return t.setVariable(this._parseVariable())?this.accept(ft.Colon)?(t.colonPosition=this.prevToken.offset,t.setValue(this._parseDetachedRuleSet()||this._parseExpr())?(t.addChild(this._parsePrio()),this.peek(ft.SemiColon)&&(t.semicolonPosition=this.token.offset),this.finish(t)):this.finish(t,Cr.VariableValueExpected,[],e)):(this.restoreAtMark(n),null):null},t.prototype._parseDetachedRuleSet=function(){if(!this.peek(ft.CurlyL))return null;var e=this.create(xn);return this._parseBody(e,this._parseDetachedRuleSetBody.bind(this)),this.finish(e)},t.prototype._parseDetachedRuleSetBody=function(){return this._tryParseKeyframeSelector()||this._tryParseRuleset(!0)||e.prototype._parseRuleSetDeclaration.call(this)},t.prototype._parseVariable=function(){if(!this.peekDelim(\"@\")&&!this.peek(ft.AtKeyword))return null;for(var e=this.create(cr),t=this.mark();this.acceptDelim(\"@\");)if(this.hasWhitespace())return this.restoreAtMark(t),null;return this.accept(ft.AtKeyword)?e:(this.restoreAtMark(t),null)},t.prototype._parseTerm=function(){var t=e.prototype._parseTerm.call(this);return t||((t=this.create(rr)).setExpression(this._parseVariable())||t.setExpression(this._parseEscaped())?this.finish(t):null)},t.prototype._parseEscaped=function(){if(this.peek(ft.EscapedJavaScript)||this.peek(ft.BadEscapedJavaScript)){var e=this.createNode(cn.EscapedValue);return this.consumeToken(),this.finish(e)}if(this.peekDelim(\"~\")){e=this.createNode(cn.EscapedValue);return this.consumeToken(),this.finish(e,this.accept(ft.String)?null:Cr.TermExpected)}return null},t.prototype._parseOperator=function(){var t=this._parseGuardOperator();return t||e.prototype._parseOperator.call(this)},t.prototype._parseGuardOperator=function(){if(this.peekDelim(\">\")){var e=this.createNode(cn.Operator);return this.consumeToken(),this.acceptDelim(\"=\"),e}if(this.peekDelim(\"=\")){e=this.createNode(cn.Operator);return this.consumeToken(),this.acceptDelim(\"<\"),e}if(this.peekDelim(\"<\")){e=this.createNode(cn.Operator);return this.consumeToken(),this.acceptDelim(\"=\"),e}return null},t.prototype._parseRuleSetDeclaration=function(){return this.peek(ft.AtKeyword)?this._parseKeyframe()||this._parseMedia(!0)||this._parseImport()||this._parseSupports(!0)||this._parseDetachedRuleSetMixin()||this._parseVariableDeclaration():this._tryParseMixinDeclaration()||this._tryParseRuleset(!0)||this._tryParseMixinReference()||this._parseExtend()||e.prototype._parseRuleSetDeclaration.call(this)},t.prototype._parseKeyframeIdent=function(){return this._parseIdent([hn.Keyframe])||this._parseVariable()},t.prototype._parseKeyframeSelector=function(){return this._parseDetachedRuleSetMixin()||e.prototype._parseKeyframeSelector.call(this)},t.prototype._parseSimpleSelectorBody=function(){return this._parseSelectorCombinator()||e.prototype._parseSimpleSelectorBody.call(this)},t.prototype._parseSelector=function(e){var t=this.create(Cn),n=!1;for(e&&(n=t.addChild(this._parseCombinator()));t.addChild(this._parseSimpleSelector());){n=!0;var r=this.mark();if(t.addChild(this._parseGuard())&&this.peek(ft.CurlyL))break;this.restoreAtMark(r),t.addChild(this._parseCombinator())}return n?this.finish(t):null},t.prototype._parseSelectorCombinator=function(){if(this.peekDelim(\"&\")){var e=this.createNode(cn.SelectorCombinator);for(this.consumeToken();!this.hasWhitespace()&&(this.acceptDelim(\"-\")||this.accept(ft.Num)||this.accept(ft.Dimension)||e.addChild(this._parseIdent())||this.acceptDelim(\"&\")););return this.finish(e)}return null},t.prototype._parseSelectorIdent=function(){if(!this.peekInterpolatedIdent())return null;var e=this.createNode(cn.SelectorInterpolation);return this._acceptInterpolatedIdent(e)?this.finish(e):null},t.prototype._parsePropertyIdentifier=function(){if(!this.peekInterpolatedIdent())return null;var e=this.create(yn);e.isCustomProperty=this.peekRegExp(ft.Ident,/^--/);var t=this._acceptInterpolatedIdent(e);return t&&!this.hasWhitespace()&&(this.acceptDelim(\"+\"),this.hasWhitespace()||this.acceptIdent(\"_\")),t?this.finish(e):null},t.prototype.peekInterpolatedIdent=function(){return this.peek(ft.Ident)||this.peekDelim(\"@\")||this.peekDelim(\"-\")},t.prototype._acceptInterpolatedIdent=function(e){for(var t=this,n=!1,r=function(){return t.acceptDelim(\"-\")?(!t.hasWhitespace()&&t.acceptDelim(\"-\"),t.hasWhitespace()?null:t._parseInterpolation()):null};(this.accept(ft.Ident)||e.addChild(this._parseInterpolation()||this.try(r)))&&(n=!0,!this.hasWhitespace()&&this.acceptDelim(\"-\"),!this.hasWhitespace()););return n},t.prototype._parseInterpolation=function(){var e=this.mark();if(this.peekDelim(\"@\")){var t=this.createNode(cn.Interpolation);return this.consumeToken(),this.hasWhitespace()||!this.accept(ft.CurlyL)?(this.restoreAtMark(e),null):t.addChild(this._parseIdent())?this.accept(ft.CurlyR)?this.finish(t):this.finish(t,Cr.RightCurlyExpected):this.finish(t,Cr.IdentifierExpected)}return null},t.prototype._tryParseMixinDeclaration=function(){var e=this.mark(),t=this.create(ur);if(!t.setIdentifier(this._parseMixinDeclarationIdentifier())||!this.accept(ft.ParenthesisL))return this.restoreAtMark(e),null;if(t.getParameters().addChild(this._parseMixinParameter()))for(;(this.accept(ft.Comma)||this.accept(ft.SemiColon))&&!this.peek(ft.ParenthesisR);)t.getParameters().addChild(this._parseMixinParameter())||this.markError(t,Cr.IdentifierExpected,[],[ft.ParenthesisR]);return this.accept(ft.ParenthesisR)?(t.setGuard(this._parseGuard()),this.peek(ft.CurlyL)?this._parseBody(t,this._parseMixInBodyDeclaration.bind(this)):(this.restoreAtMark(e),null)):(this.restoreAtMark(e),null)},t.prototype._parseMixInBodyDeclaration=function(){return this._parseFontFace()||this._parseRuleSetDeclaration()},t.prototype._parseMixinDeclarationIdentifier=function(){var e;if(this.peekDelim(\"#\")||this.peekDelim(\".\")){if(e=this.create(yn),this.consumeToken(),this.hasWhitespace()||!e.addChild(this._parseIdent()))return null}else{if(!this.peek(ft.Hash))return null;e=this.create(yn),this.consumeToken()}return e.referenceTypes=[hn.Mixin],this.finish(e)},t.prototype._parsePseudo=function(){if(!this.peek(ft.Colon))return null;var t=this.mark(),n=this.create(hr);return this.consumeToken(),this.acceptIdent(\"extend\")?this._completeExtends(n):(this.restoreAtMark(t),e.prototype._parsePseudo.call(this))},t.prototype._parseExtend=function(){if(!this.peekDelim(\"&\"))return null;var e=this.mark(),t=this.create(hr);return this.consumeToken(),!this.hasWhitespace()&&this.accept(ft.Colon)&&this.acceptIdent(\"extend\")?this._completeExtends(t):(this.restoreAtMark(e),null)},t.prototype._completeExtends=function(e){if(!this.accept(ft.ParenthesisL))return this.finish(e,Cr.LeftParenthesisExpected);var t=e.getSelectors();if(!t.addChild(this._parseSelector(!0)))return this.finish(e,Cr.SelectorExpected);for(;this.accept(ft.Comma);)if(!t.addChild(this._parseSelector(!0)))return this.finish(e,Cr.SelectorExpected);return this.accept(ft.ParenthesisR)?this.finish(e):this.finish(e,Cr.RightParenthesisExpected)},t.prototype._parseDetachedRuleSetMixin=function(){if(!this.peek(ft.AtKeyword))return null;var e=this.mark(),t=this.create(dr);return t.addChild(this._parseVariable())&&this.accept(ft.ParenthesisL)?this.accept(ft.ParenthesisR)?this.finish(t):this.finish(t,Cr.RightParenthesisExpected):(this.restoreAtMark(e),null)},t.prototype._tryParseMixinReference=function(e){void 0===e&&(e=!1);for(var t=this.mark(),n=this.create(dr),r=this._parseMixinDeclarationIdentifier();r;){this.acceptDelim(\">\");var i=this._parseMixinDeclarationIdentifier();if(!i)break;n.getNamespaces().addChild(r),r=i}if(!n.setIdentifier(r))return this.restoreAtMark(t),null;var o=!1;if(!this.hasWhitespace()&&this.accept(ft.ParenthesisL)){if(o=!0,n.getArguments().addChild(this._parseMixinArgument()))for(;(this.accept(ft.Comma)||this.accept(ft.SemiColon))&&!this.peek(ft.ParenthesisR);)if(!n.getArguments().addChild(this._parseMixinArgument()))return this.finish(n,Cr.ExpressionExpected);if(!this.accept(ft.ParenthesisR))return this.finish(n,Cr.RightParenthesisExpected);r.referenceTypes=[hn.Mixin]}else r.referenceTypes=[hn.Mixin,hn.Rule];return n.addChild(this._parsePrio()),o||this.peek(ft.SemiColon)||this.peek(ft.CurlyR)||this.peek(ft.EOF)?this.finish(n):(this.restoreAtMark(t),null)},t.prototype._parseMixinArgument=function(){var e=this.create(Nn),t=this.mark(),n=this._parseVariable();return n&&(this.accept(ft.Colon)?e.setIdentifier(n):this.restoreAtMark(t)),e.setValue(this._parseDetachedRuleSet()||this._parseExpr(!0))?this.finish(e):(this.restoreAtMark(t),null)},t.prototype._parseMixinParameter=function(){var e=this.create(On);if(this.peekKeyword(\"@rest\")){var t=this.create(gn);return this.consumeToken(),this.accept(js)?(e.setIdentifier(this.finish(t)),this.finish(e)):this.finish(e,Cr.DotExpected,[],[ft.Comma,ft.ParenthesisR])}if(this.peek(js)){var n=this.create(gn);return this.consumeToken(),e.setIdentifier(this.finish(n)),this.finish(e)}var r=!1;return e.setIdentifier(this._parseVariable())&&(this.accept(ft.Colon),r=!0),e.setDefaultValue(this._parseExpr(!0))||r?this.finish(e):null},t.prototype._parseGuard=function(){if(!this.peekIdent(\"when\"))return null;var e=this.create(fr);if(this.consumeToken(),e.isNegated=this.acceptIdent(\"not\"),!e.getConditions().addChild(this._parseGuardCondition()))return this.finish(e,Cr.ConditionExpected);for(;this.acceptIdent(\"and\")||this.accept(ft.Comma);)if(!e.getConditions().addChild(this._parseGuardCondition()))return this.finish(e,Cr.ConditionExpected);return this.finish(e)},t.prototype._parseGuardCondition=function(){if(!this.peek(ft.ParenthesisL))return null;var e=this.create(gr);return this.consumeToken(),e.addChild(this._parseExpr()),this.accept(ft.ParenthesisR)?this.finish(e):this.finish(e,Cr.RightParenthesisExpected)},t.prototype._parseFunctionIdentifier=function(){if(this.peekDelim(\"%\")){var t=this.create(yn);return t.referenceTypes=[hn.Function],this.consumeToken(),this.finish(t)}return e.prototype._parseFunctionIdentifier.call(this)},t.prototype._parseURLArgument=function(){var t=this.mark(),n=e.prototype._parseURLArgument.call(this);if(!n||!this.peek(ft.ParenthesisR)){this.restoreAtMark(t);var r=this.create(gn);return r.addChild(this._parseBinaryExpr()),this.finish(r)}return n},t}(gi),qs=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Ws=vr(),$s=function(e){function t(){return e.call(this,\"@\")||this}return qs(t,e),t.prototype.createFunctionProposals=function(e,t,n,r){for(var i=0,o=e;i<o.length;i++){var s=o[i],a={label:s.name,detail:s.example,documentation:s.description,textEdit:Ii.replace(this.getCompletionRange(t),s.name+\"($0)\"),insertTextFormat:Gi.Snippet,kind:$i.Function};n&&(a.sortText=\"z\"),r.items.push(a)}return r},t.prototype.getTermProposals=function(n,r,i){var o=t.builtInProposals;return n&&(o=o.filter(function(e){return!e.type||-1!==n.restrictions.indexOf(e.type)})),this.createFunctionProposals(o,r,!0,i),e.prototype.getTermProposals.call(this,n,r,i)},t.prototype.getColorProposals=function(n,r,i){return this.createFunctionProposals(t.colorProposals,r,!1,i),e.prototype.getColorProposals.call(this,n,r,i)},t.prototype.getCompletionsForDeclarationProperty=function(t,n){return this.getCompletionsForSelector(null,!0,n),e.prototype.getCompletionsForDeclarationProperty.call(this,t,n)},t.builtInProposals=[{name:\"escape\",example:\"escape(@string);\",description:Ws(\"less.builtin.escape\",\"URL encodes a string\")},{name:\"e\",example:\"e(@string);\",description:Ws(\"less.builtin.e\",\"escape string content\")},{name:\"replace\",example:\"replace(@string, @pattern, @replacement[, @flags]);\",description:Ws(\"less.builtin.replace\",\"string replace\")},{name:\"unit\",example:\"unit(@dimension, [@unit: '']);\",description:Ws(\"less.builtin.unit\",\"remove or change the unit of a dimension\")},{name:\"color\",example:\"color(@string);\",description:Ws(\"less.builtin.color\",\"parses a string to a color\"),type:\"color\"},{name:\"convert\",example:\"convert(@value, unit);\",description:Ws(\"less.builtin.convert\",\"converts numbers from one type into another\")},{name:\"data-uri\",example:\"data-uri([mimetype,] url);\",description:Ws(\"less.builtin.data-uri\",\"inlines a resource and falls back to `url()`\"),type:\"url\"},{name:\"length\",example:\"length(@list);\",description:Ws(\"less.builtin.length\",\"returns the number of elements in a value list\")},{name:\"extract\",example:\"extract(@list, index);\",description:Ws(\"less.builtin.extract\",\"returns a value at the specified position in the list\")},{name:\"abs\",description:Ws(\"less.builtin.abs\",\"absolute value of a number\"),example:\"abs(number);\"},{name:\"acos\",description:Ws(\"less.builtin.acos\",\"arccosine - inverse of cosine function\"),example:\"acos(number);\"},{name:\"asin\",description:Ws(\"less.builtin.asin\",\"arcsine - inverse of sine function\"),example:\"asin(number);\"},{name:\"ceil\",example:\"ceil(@number);\",description:Ws(\"less.builtin.ceil\",\"rounds up to an integer\")},{name:\"cos\",description:Ws(\"less.builtin.cos\",\"cosine function\"),example:\"cos(number);\"},{name:\"floor\",description:Ws(\"less.builtin.floor\",\"rounds down to an integer\"),example:\"floor(@number);\"},{name:\"percentage\",description:Ws(\"less.builtin.percentage\",\"converts to a %, e.g. 0.5 > 50%\"),example:\"percentage(@number);\",type:\"percentage\"},{name:\"round\",description:Ws(\"less.builtin.round\",\"rounds a number to a number of places\"),example:\"round(number, [places: 0]);\"},{name:\"sqrt\",description:Ws(\"less.builtin.sqrt\",\"calculates square root of a number\"),example:\"sqrt(number);\"},{name:\"sin\",description:Ws(\"less.builtin.sin\",\"sine function\"),example:\"sin(number);\"},{name:\"tan\",description:Ws(\"less.builtin.tan\",\"tangent function\"),example:\"tan(number);\"},{name:\"atan\",description:Ws(\"less.builtin.atan\",\"arctangent - inverse of tangent function\"),example:\"atan(number);\"},{name:\"pi\",description:Ws(\"less.builtin.pi\",\"returns pi\"),example:\"pi();\"},{name:\"pow\",description:Ws(\"less.builtin.pow\",\"first argument raised to the power of the second argument\"),example:\"pow(@base, @exponent);\"},{name:\"mod\",description:Ws(\"less.builtin.mod\",\"first argument modulus second argument\"),example:\"mod(number, number);\"},{name:\"min\",description:Ws(\"less.builtin.min\",\"returns the lowest of one or more values\"),example:\"min(@x, @y);\"},{name:\"max\",description:Ws(\"less.builtin.max\",\"returns the lowest of one or more values\"),example:\"max(@x, @y);\"}],t.colorProposals=[{name:\"argb\",example:\"argb(@color);\",description:Ws(\"less.builtin.argb\",\"creates a #AARRGGBB\")},{name:\"hsl\",example:\"hsl(@hue, @saturation, @lightness);\",description:Ws(\"less.builtin.hsl\",\"creates a color\")},{name:\"hsla\",example:\"hsla(@hue, @saturation, @lightness, @alpha);\",description:Ws(\"less.builtin.hsla\",\"creates a color\")},{name:\"hsv\",example:\"hsv(@hue, @saturation, @value);\",description:Ws(\"less.builtin.hsv\",\"creates a color\")},{name:\"hsva\",example:\"hsva(@hue, @saturation, @value, @alpha);\",description:Ws(\"less.builtin.hsva\",\"creates a color\")},{name:\"hue\",example:\"hue(@color);\",description:Ws(\"less.builtin.hue\",\"returns the `hue` channel of `@color` in the HSL space\")},{name:\"saturation\",example:\"saturation(@color);\",description:Ws(\"less.builtin.saturation\",\"returns the `saturation` channel of `@color` in the HSL space\")},{name:\"lightness\",example:\"lightness(@color);\",description:Ws(\"less.builtin.lightness\",\"returns the `lightness` channel of `@color` in the HSL space\")},{name:\"hsvhue\",example:\"hsvhue(@color);\",description:Ws(\"less.builtin.hsvhue\",\"returns the `hue` channel of `@color` in the HSV space\")},{name:\"hsvsaturation\",example:\"hsvsaturation(@color);\",description:Ws(\"less.builtin.hsvsaturation\",\"returns the `saturation` channel of `@color` in the HSV space\")},{name:\"hsvvalue\",example:\"hsvvalue(@color);\",description:Ws(\"less.builtin.hsvvalue\",\"returns the `value` channel of `@color` in the HSV space\")},{name:\"red\",example:\"red(@color);\",description:Ws(\"less.builtin.red\",\"returns the `red` channel of `@color`\")},{name:\"green\",example:\"green(@color);\",description:Ws(\"less.builtin.green\",\"returns the `green` channel of `@color`\")},{name:\"blue\",example:\"blue(@color);\",description:Ws(\"less.builtin.blue\",\"returns the `blue` channel of `@color`\")},{name:\"alpha\",example:\"alpha(@color);\",description:Ws(\"less.builtin.alpha\",\"returns the `alpha` channel of `@color`\")},{name:\"luma\",example:\"luma(@color);\",description:Ws(\"less.builtin.luma\",\"returns the `luma` value (perceptual brightness) of `@color`\")},{name:\"saturate\",example:\"saturate(@color, 10%);\",description:Ws(\"less.builtin.saturate\",\"return `@color` 10% points more saturated\")},{name:\"desaturate\",example:\"desaturate(@color, 10%);\",description:Ws(\"less.builtin.desaturate\",\"return `@color` 10% points less saturated\")},{name:\"lighten\",example:\"lighten(@color, 10%);\",description:Ws(\"less.builtin.lighten\",\"return `@color` 10% points lighter\")},{name:\"darken\",example:\"darken(@color, 10%);\",description:Ws(\"less.builtin.darken\",\"return `@color` 10% points darker\")},{name:\"fadein\",example:\"fadein(@color, 10%);\",description:Ws(\"less.builtin.fadein\",\"return `@color` 10% points less transparent\")},{name:\"fadeout\",example:\"fadeout(@color, 10%);\",description:Ws(\"less.builtin.fadeout\",\"return `@color` 10% points more transparent\")},{name:\"fade\",example:\"fade(@color, 50%);\",description:Ws(\"less.builtin.fade\",\"return `@color` with 50% transparency\")},{name:\"spin\",example:\"spin(@color, 10);\",description:Ws(\"less.builtin.spin\",\"return `@color` with a 10 degree larger in hue\")},{name:\"mix\",example:\"mix(@color1, @color2, [@weight: 50%]);\",description:Ws(\"less.builtin.mix\",\"return a mix of `@color1` and `@color2`\")},{name:\"greyscale\",example:\"greyscale(@color);\",description:Ws(\"less.builtin.greyscale\",\"returns a grey, 100% desaturated color\")},{name:\"contrast\",example:\"contrast(@color1, [@darkcolor: black], [@lightcolor: white], [@threshold: 43%]);\",description:Ws(\"less.builtin.contrast\",\"return `@darkcolor` if `@color1 is> 43% luma` otherwise return `@lightcolor`, see notes\")},{name:\"multiply\",example:\"multiply(@color1, @color2);\"},{name:\"screen\",example:\"screen(@color1, @color2);\"},{name:\"overlay\",example:\"overlay(@color1, @color2);\"},{name:\"softlight\",example:\"softlight(@color1, @color2);\"},{name:\"hardlight\",example:\"hardlight(@color1, @color2);\"},{name:\"difference\",example:\"difference(@color1, @color2);\"},{name:\"exclusion\",example:\"exclusion(@color1, @color2);\"},{name:\"average\",example:\"average(@color1, @color2);\"},{name:\"negation\",example:\"negation(@color1, @color2);\"}],t}(wo);function Gs(e,t){return function(e,t){var n=t&&t.rangeLimit||Number.MAX_VALUE,r=e.sort(function(e,t){var n=e.startLine-t.startLine;return 0===n&&(n=e.endLine-t.endLine),n}),i=[],o=-1;return r.forEach(function(e){e.startLine<o&&o<e.endLine||(i.push(e),o=e.endLine)}),i.length<n?i:i.slice(0,n)}(function(e){function t(t){return e.positionAt(t.offset).line}function n(t){return e.positionAt(t.offset+t.len).line}var r=[],i=[],o=function(){switch(e.languageId){case\"scss\":return new Cs;case\"less\":return new Vs;default:return new dn}}();o.ignoreComment=!1,o.setSource(e.getText());var s,a=o.scan(),l=function(){switch(a.type){case ft.CurlyL:case bs:i.push({line:t(a),type:\"brace\",isStart:!0});break;case ft.CurlyR:if(0!==i.length){var l=Hs(i,\"brace\");if(!l)break;var c=n(a);\"brace\"===l.type&&(n(s)!==c&&c--,l.line!==c&&r.push({startLine:l.line,endLine:c,kind:void 0}))}break;case ft.Comment:var h=function(e){return\"#region\"===e?{line:t(a),type:\"comment\",isStart:!0}:{line:n(a),type:\"comment\",isStart:!1}},d=function(t){var n=t.text.match(/^\\s*\\/\\*\\s*(#region|#endregion)\\b\\s*(.*?)\\s*\\*\\//);if(n)return h(n[1]);if(\"scss\"===e.languageId||\"less\"===e.languageId){var r=t.text.match(/^\\s*\\/\\/\\s*(#region|#endregion)\\b\\s*(.*?)\\s*/);if(r)return h(r[1])}return null}(a);if(d)if(d.isStart)i.push(d);else{var l=Hs(i,\"comment\");if(!l)break;\"comment\"===l.type&&l.line!==d.line&&r.push({startLine:l.line,endLine:d.line,kind:\"region\"})}else{var u=function(e,r){var i=t(e),o=n(e);return i!==o?{startLine:i,endLine:o,kind:r}:null}(a,\"comment\");u&&r.push(u)}}s=a,a=o.scan()};for(;a.type!==ft.EOF;)l();return r}(e),t)}function Hs(e,t){if(0===e.length)return null;for(var n=e.length-1;n>=0;n--)if(e[n].type===t&&e[n].isStart)return e.splice(n,1)[0];return null}function Ys(e,t,n,r,i,o){return{configure:o.configure.bind(o),doValidation:o.doValidation.bind(o),parseStylesheet:e.parseStylesheet.bind(e),doComplete:t.doComplete.bind(t),setCompletionParticipants:t.setCompletionParticipants.bind(t),doHover:n.doHover.bind(n),findDefinition:r.findDefinition.bind(r),findReferences:r.findReferences.bind(r),findDocumentHighlights:r.findDocumentHighlights.bind(r),findDocumentSymbols:r.findDocumentSymbols.bind(r),doCodeActions:i.doCodeActions.bind(i),findColorSymbols:function(e,t){return r.findDocumentColors(e,t).map(function(e){return e.range})},findDocumentColors:r.findDocumentColors.bind(r),getColorPresentations:r.getColorPresentations.bind(r),doRename:r.doRename.bind(r),getFoldingRanges:Gs}}var Js=monaco.Promise,Xs=function(){function e(e,t){switch(this._ctx=e,this._languageSettings=t.languageSettings,this._languageId=t.languageId,this._languageId){case\"css\":this._languageService=Ys(new gi,new wo,new zo,new Uo,new Jo,new ts);break;case\"less\":this._languageService=Ys(new Ks,new $s,new zo,new Uo,new Jo,new ts);break;case\"scss\":this._languageService=Ys(new Ts,new Ps,new zo,new Uo,new Jo,new ts);break;default:throw new Error(\"Invalid language id: \"+this._languageId)}this._languageService.configure(this._languageSettings)}return e.prototype.doValidation=function(e){var t=this._getTextDocument(e);if(t){var n=this._languageService.parseStylesheet(t),r=this._languageService.doValidation(t,n);return Js.as(r)}return Js.as([])},e.prototype.doComplete=function(e,t){var n=this._getTextDocument(e),r=this._languageService.parseStylesheet(n),i=this._languageService.doComplete(n,t,r);return Js.as(i)},e.prototype.doHover=function(e,t){var n=this._getTextDocument(e),r=this._languageService.parseStylesheet(n),i=this._languageService.doHover(n,t,r);return Js.as(i)},e.prototype.findDefinition=function(e,t){var n=this._getTextDocument(e),r=this._languageService.parseStylesheet(n),i=this._languageService.findDefinition(n,t,r);return Js.as(i)},e.prototype.findReferences=function(e,t){var n=this._getTextDocument(e),r=this._languageService.parseStylesheet(n),i=this._languageService.findReferences(n,t,r);return Js.as(i)},e.prototype.findDocumentHighlights=function(e,t){var n=this._getTextDocument(e),r=this._languageService.parseStylesheet(n),i=this._languageService.findDocumentHighlights(n,t,r);return Js.as(i)},e.prototype.findDocumentSymbols=function(e){var t=this._getTextDocument(e),n=this._languageService.parseStylesheet(t),r=this._languageService.findDocumentSymbols(t,n);return Js.as(r)},e.prototype.doCodeActions=function(e,t,n){var r=this._getTextDocument(e),i=this._languageService.parseStylesheet(r),o=this._languageService.doCodeActions(r,t,n,i);return Js.as(o)},e.prototype.findDocumentColors=function(e){var t=this._getTextDocument(e),n=this._languageService.parseStylesheet(t),r=this._languageService.findDocumentColors(t,n);return Js.as(r)},e.prototype.getColorPresentations=function(e,t,n){var r=this._getTextDocument(e),i=this._languageService.parseStylesheet(r),o=this._languageService.getColorPresentations(r,i,t,n);return Js.as(o)},e.prototype.provideFoldingRanges=function(e,t){var n=this._getTextDocument(e),r=this._languageService.getFoldingRanges(n,t);return Js.as(r)},e.prototype.doRename=function(e,t,n){var r=this._getTextDocument(e),i=this._languageService.parseStylesheet(r),o=this._languageService.doRename(r,t,n,i);return Js.as(o)},e.prototype._getTextDocument=function(e){for(var t=0,n=this._ctx.getMirrorModels();t<n.length;t++){var r=n[t];if(r.uri.toString()===e)return uo.create(e,this._languageId,r.version,r.getValue())}return null},e}();self.onmessage=function(){bt(function(e,t){return new Xs(e,t)})}}();\n"], { type: "text/javascript" }));

var htmlWorker = URL.createObjectURL(new Blob(["!function(){\"use strict\";var e;!function(){var t=Object.create(null);t[\"WinJS/Core/_WinJS\"]={};var n=function(e,n,r){var i={},o=!1,a=n.map(function(e){return\"exports\"===e?(o=!0,i):t[e]}),s=r.apply({},a);t[e]=o?i:s};n(\"WinJS/Core/_Global\",[],function(){return\"undefined\"!=typeof window?window:\"undefined\"!=typeof self?self:\"undefined\"!=typeof global?global:{}}),n(\"WinJS/Core/_BaseCoreUtils\",[\"WinJS/Core/_Global\"],function(e){var t=!!e.Windows;var n=null;return{hasWinRT:t,markSupportedForProcessing:function(e){return e.supportedForProcessing=!0,e},_setImmediate:function(t){null===n&&(n=e.setImmediate?e.setImmediate.bind(e):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):e.setTimeout.bind(e)),n(t)}}}),n(\"WinJS/Core/_WriteProfilerMark\",[\"WinJS/Core/_Global\"],function(e){return e.msWriteProfilerMark||function(){}}),n(\"WinJS/Core/_Base\",[\"WinJS/Core/_WinJS\",\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_WriteProfilerMark\"],function(e,t,n,r){function i(e,t,n){var r,i,o,a=Object.keys(t),s=Array.isArray(e);for(i=0,o=a.length;i<o;i++){var u=a[i],l=95!==u.charCodeAt(0),c=t[u];!c||\"object\"!=typeof c||void 0===c.value&&\"function\"!=typeof c.get&&\"function\"!=typeof c.set?l?s?e.forEach(function(e){e[u]=c}):e[u]=c:(r=r||{})[u]={value:c,enumerable:l,configurable:!0,writable:!0}:(void 0===c.enumerable&&(c.enumerable=l),n&&c.setName&&\"function\"==typeof c.setName&&c.setName(n+\".\"+u),(r=r||{})[u]=c)}r&&(s?e.forEach(function(e){Object.defineProperties(e,r)}):Object.defineProperties(e,r))}return function(){var n=e;function o(n,r){var i=n||{};if(r){var o=r.split(\".\");i===t&&\"WinJS\"===o[0]&&(i=e,o.splice(0,1));for(var a=0,s=o.length;a<s;a++){var u=o[a];i[u]||Object.defineProperty(i,u,{value:{},writable:!1,enumerable:!0,configurable:!0}),i=i[u]}}return i}function a(e,t,n){var r=o(e,t);return n&&i(r,n,t||\"<ANONYMOUS>\"),r}n.Namespace||(n.Namespace=Object.create(Object.prototype));var s={uninitialized:1,working:2,initialized:3};Object.defineProperties(n.Namespace,{defineWithParent:{value:a,writable:!0,enumerable:!0,configurable:!0},define:{value:function(e,n){return a(t,e,n)},writable:!0,enumerable:!0,configurable:!0},_lazy:{value:function(e){var t,n,i=s.uninitialized;return{setName:function(e){t=e},get:function(){switch(i){case s.initialized:return n;case s.uninitialized:i=s.working;try{r(\"WinJS.Namespace._lazy:\"+t+\",StartTM\"),n=e()}finally{r(\"WinJS.Namespace._lazy:\"+t+\",StopTM\"),i=s.uninitialized}return e=null,i=s.initialized,n;case s.working:throw\"Illegal: reentrancy on initialization\";default:throw\"Illegal\"}},set:function(e){switch(i){case s.working:throw\"Illegal: reentrancy on initialization\";default:i=s.initialized,n=e}},enumerable:!0,configurable:!0}},writable:!0,enumerable:!0,configurable:!0},_moduleDefine:{value:function(e,n,r){var a=[e],s=null;return n&&(s=o(t,n),a.push(s)),i(a,r,n||\"<ANONYMOUS>\"),s},writable:!0,enumerable:!0,configurable:!0}})}(),function(){function t(e,t,r){return e=e||function(){},n.markSupportedForProcessing(e),t&&i(e.prototype,t),r&&i(e,r),e}e.Namespace.define(\"WinJS.Class\",{define:t,derive:function(e,r,o,a){if(e){r=r||function(){};var s=e.prototype;return r.prototype=Object.create(s),n.markSupportedForProcessing(r),Object.defineProperty(r.prototype,\"constructor\",{value:r,writable:!0,configurable:!0,enumerable:!0}),o&&i(r.prototype,o),a&&i(r,a),r}return t(r,o,a)},mix:function(e){var t,n;for(e=e||function(){},t=1,n=arguments.length;t<n;t++)i(e.prototype,arguments[t]);return e}})}(),{Namespace:e.Namespace,Class:e.Class}}),n(\"WinJS/Core/_ErrorFromName\",[\"WinJS/Core/_Base\"],function(e){var t=e.Class.derive(Error,function(e,t){this.name=e,this.message=t||e},{},{supportedForProcessing:!1});return e.Namespace.define(\"WinJS\",{ErrorFromName:t}),t}),n(\"WinJS/Core/_Events\",[\"exports\",\"WinJS/Core/_Base\"],function(e,t){function n(e){var t=\"_on\"+e+\"state\";return{get:function(){var e=this[t];return e&&e.userHandler},set:function(n){var r=this[t];n?(r||(r={wrapper:function(e){return r.userHandler(e)},userHandler:n},Object.defineProperty(this,t,{value:r,enumerable:!1,writable:!0,configurable:!0}),this.addEventListener(e,r.wrapper,!1)),r.userHandler=n):r&&(this.removeEventListener(e,r.wrapper,!1),this[t]=null)},enumerable:!0}}var r=t.Class.define(function(e,t,n){this.detail=t,this.target=n,this.timeStamp=Date.now(),this.type=e},{bubbles:{value:!1,writable:!1},cancelable:{value:!1,writable:!1},currentTarget:{get:function(){return this.target}},defaultPrevented:{get:function(){return this._preventDefaultCalled}},trusted:{value:!1,writable:!1},eventPhase:{value:0,writable:!1},target:null,timeStamp:null,type:null,preventDefault:function(){this._preventDefaultCalled=!0},stopImmediatePropagation:function(){this._stopImmediatePropagationCalled=!0},stopPropagation:function(){}},{supportedForProcessing:!1}),i={_listeners:null,addEventListener:function(e,t,n){n=n||!1,this._listeners=this._listeners||{};for(var r=this._listeners[e]=this._listeners[e]||[],i=0,o=r.length;i<o;i++){var a=r[i];if(a.useCapture===n&&a.listener===t)return}r.push({listener:t,useCapture:n})},dispatchEvent:function(e,t){var n=this._listeners&&this._listeners[e];if(n){for(var i=new r(e,t,this),o=0,a=(n=n.slice(0,n.length)).length;o<a&&!i._stopImmediatePropagationCalled;o++)n[o].listener(i);return i.defaultPrevented||!1}return!1},removeEventListener:function(e,t,n){n=n||!1;var r=this._listeners&&this._listeners[e];if(r)for(var i=0,o=r.length;i<o;i++){var a=r[i];if(a.listener===t&&a.useCapture===n){r.splice(i,1),0===r.length&&delete this._listeners[e];break}}}};t.Namespace._moduleDefine(e,\"WinJS.Utilities\",{_createEventProperty:n,createEventProperties:function(){for(var e={},t=0,r=arguments.length;t<r;t++){var i=arguments[t];e[\"on\"+i]=n(i)}return e},eventMixin:i})}),n(\"WinJS/Core/_Trace\",[\"WinJS/Core/_Global\"],function(e){function t(e){return e}return{_traceAsyncOperationStarting:e.Debug&&e.Debug.msTraceAsyncOperationStarting&&e.Debug.msTraceAsyncOperationStarting.bind(e.Debug)||t,_traceAsyncOperationCompleted:e.Debug&&e.Debug.msTraceAsyncOperationCompleted&&e.Debug.msTraceAsyncOperationCompleted.bind(e.Debug)||t,_traceAsyncCallbackStarting:e.Debug&&e.Debug.msTraceAsyncCallbackStarting&&e.Debug.msTraceAsyncCallbackStarting.bind(e.Debug)||t,_traceAsyncCallbackCompleted:e.Debug&&e.Debug.msTraceAsyncCallbackCompleted&&e.Debug.msTraceAsyncCallbackCompleted.bind(e.Debug)||t}}),n(\"WinJS/Promise/_StateMachine\",[\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_Base\",\"WinJS/Core/_ErrorFromName\",\"WinJS/Core/_Events\",\"WinJS/Core/_Trace\"],function(e,t,n,r,i,o){e.Debug&&(e.Debug.setNonUserCodeExceptions=!0);var a=new(n.Class.mix(n.Class.define(null,{},{supportedForProcessing:!1}),i.eventMixin));a._listeners={};var s=\"error\",u=\"Canceled\",l=!1,c={promise:1,thenPromise:2,errorPromise:4,exceptionPromise:8,completePromise:16};c.all=c.promise|c.thenPromise|c.errorPromise|c.exceptionPromise|c.completePromise;var h,d,f,p,g,m,b,_,v,y,w=1;function C(){}h={name:\"created\",enter:function(e){e._setState(d)},cancel:C,done:C,then:C,_completed:C,_error:C,_notify:C,_progress:C,_setCompleteValue:C,_setErrorValue:C},d={name:\"working\",enter:C,cancel:function(e){e._setState(g)},done:N,then:V,_completed:E,_error:O,_notify:C,_progress:R,_setCompleteValue:W,_setErrorValue:M},f={name:\"waiting\",enter:function(e){var t=e._value;if(t instanceof j&&t._state!==y&&t._state!==_)D(t,{promise:e});else{var n=function(r){t._errorId?e._chainedError(r,t):(q(e,r,S,t,n),e._error(r))};n.handlesOnError=!0,t.then(e._completed.bind(e),n,e._progress.bind(e))}},cancel:function(e){e._setState(p)},done:N,then:V,_completed:E,_error:O,_notify:C,_progress:R,_setCompleteValue:W,_setErrorValue:M},p={name:\"waiting_canceled\",enter:function(e){e._setState(m);var t=e._value;t.cancel&&t.cancel()},cancel:C,done:N,then:V,_completed:E,_error:O,_notify:C,_progress:R,_setCompleteValue:W,_setErrorValue:M},g={name:\"canceled\",enter:function(e){e._setState(m),e._cancelAction()},cancel:C,done:N,then:V,_completed:E,_error:O,_notify:C,_progress:R,_setCompleteValue:W,_setErrorValue:M},m={name:\"canceling\",enter:function(e){var t=new Error(u);t.name=t.message,e._value=t,e._setState(v)},cancel:C,done:C,then:C,_completed:C,_error:C,_notify:C,_progress:C,_setCompleteValue:C,_setErrorValue:C},b={name:\"complete_notify\",enter:function(e){if(e.done=H.prototype.done,e.then=H.prototype.then,e._listeners)for(var t,n=[e];n.length;)(t=n.shift())._state._notify(t,n);e._setState(_)},cancel:C,done:null,then:null,_completed:C,_error:C,_notify:P,_progress:C,_setCompleteValue:C,_setErrorValue:C},_={name:\"success\",enter:function(e){e.done=H.prototype.done,e.then=H.prototype.then,e._cleanupAction()},cancel:C,done:null,then:null,_completed:C,_error:C,_notify:P,_progress:C,_setCompleteValue:C,_setErrorValue:C},v={name:\"error_notify\",enter:function(e){if(e.done=F.prototype.done,e.then=F.prototype.then,e._listeners)for(var t,n=[e];n.length;)(t=n.shift())._state._notify(t,n);e._setState(y)},cancel:C,done:null,then:null,_completed:C,_error:C,_notify:I,_progress:C,_setCompleteValue:C,_setErrorValue:C},y={name:\"error\",enter:function(e){e.done=F.prototype.done,e.then=F.prototype.then,e._cleanupAction()},cancel:C,done:null,then:null,_completed:C,_error:C,_notify:I,_progress:C,_setCompleteValue:C,_setErrorValue:C};var T=n.Class.define(null,{_listeners:null,_nextState:null,_state:null,_value:null,cancel:function(){this._state.cancel(this),this._run()},done:function(e,t,n){this._state.done(this,e,t,n)},then:function e(t,n,r){if(this.then===e)return this._state.then(this,t,n,r);this.then(t,n,r)},_chainedError:function(e,t){var n=this._state._error(this,e,k,t);return this._run(),n},_completed:function(e){var t=this._state._completed(this,e);return this._run(),t},_error:function(e){var t=this._state._error(this,e,A);return this._run(),t},_progress:function(e){this._state._progress(this,e)},_setState:function(e){this._nextState=e},_setCompleteValue:function(e){this._state._setCompleteValue(this,e),this._run()},_setChainedErrorValue:function(e,t){var n=this._state._setErrorValue(this,e,k,t);return this._run(),n},_setExceptionValue:function(e){var t=this._state._setErrorValue(this,e,L);return this._run(),t},_run:function(){for(;this._nextState;)this._state=this._nextState,this._nextState=null,this._state.enter(this)}},{supportedForProcessing:!1});function E(e,t){var n;n=t&&\"object\"==typeof t&&\"function\"==typeof t.then?f:b,e._value=t,e._setState(n)}function x(e,t,n,r,i,o){return{exception:e,error:t,promise:n,handler:o,id:r,parent:i}}function S(e,t,n,r){var i=n._isException;return x(i?t:null,i?null:t,e,n._errorId,n,r)}function k(e,t,n){var r=n._isException,i=n._errorId;return U(e,i,r),x(r?t:null,r?null:t,e,i,n)}function A(e,t){var n=++w;return U(e,n),x(null,t,e,n)}function L(e,t){var n=++w;return U(e,n,!0),x(t,null,e,n)}function N(e,t,n,r){D(e,{c:t,e:n,p:r,asyncOpID:o._traceAsyncOperationStarting(\"WinJS.Promise.done\")})}function O(e,t,n,r){e._value=t,q(e,t,n,r),e._setState(v)}function P(t,n){var r,i,a=t._value,s=t._listeners;if(s)for(t._listeners=null,r=0,i=Array.isArray(s)?s.length:1;r<i;r++){var u=1===i?s:s[r],l=u.c,c=u.promise;if(o._traceAsyncOperationCompleted(u.asyncOpID,e.Debug&&e.Debug.MS_ASYNC_OP_STATUS_SUCCESS),c){o._traceAsyncCallbackStarting(u.asyncOpID);try{c._setCompleteValue(l?l(a):a)}catch(e){c._setExceptionValue(e)}finally{o._traceAsyncCallbackCompleted()}c._state!==f&&c._listeners&&n.push(c)}else H.prototype.done.call(t,l)}}function I(t,n){var r,i,a=t._value,s=t._listeners;if(s)for(t._listeners=null,r=0,i=Array.isArray(s)?s.length:1;r<i;r++){var l=1===i?s:s[r],c=l.e,h=l.promise,d=e.Debug&&(a&&a.name===u?e.Debug.MS_ASYNC_OP_STATUS_CANCELED:e.Debug.MS_ASYNC_OP_STATUS_ERROR);if(o._traceAsyncOperationCompleted(l.asyncOpID,d),h){var p=!1;try{c?(o._traceAsyncCallbackStarting(l.asyncOpID),p=!0,c.handlesOnError||q(h,a,S,t,c),h._setCompleteValue(c(a))):h._setChainedErrorValue(a,t)}catch(e){h._setExceptionValue(e)}finally{p&&o._traceAsyncCallbackCompleted()}h._state!==f&&h._listeners&&n.push(h)}else F.prototype.done.call(t,null,c)}}function q(e,t,n,r,i){if(a._listeners[s]){if(t instanceof Error&&t.message===u)return;a.dispatchEvent(s,n(e,t,r,i))}}function R(e,t){var n,r,i=e._listeners;if(i)for(n=0,r=Array.isArray(i)?i.length:1;n<r;n++){var o=1===r?i:i[n],a=o.p;if(a)try{a(t)}catch(e){}o.c||o.e||!o.promise||o.promise._progress(t)}}function D(e,t){var n=e._listeners;n?(n=Array.isArray(n)?n:[n]).push(t):n=t,e._listeners=n}function U(e,t,n){e._isException=n||!1,e._errorId=t}function M(e,t,n,r){e._value=t,q(e,t,n,r),e._setState(y)}function W(e,t){var n;n=t&&\"object\"==typeof t&&\"function\"==typeof t.then?f:_,e._value=t,e._setState(n)}function V(e,t,n,r){var i=new j(e);return D(e,{promise:i,c:t,e:n,p:r,asyncOpID:o._traceAsyncOperationStarting(\"WinJS.Promise.then\")}),i}var B,j=n.Class.derive(T,function(e){l&&(!0===l||l&c.thenPromise)&&(this._stack=G._getStack()),this._creator=e,this._setState(h),this._run()},{_creator:null,_cancelAction:function(){this._creator&&this._creator.cancel()},_cleanupAction:function(){this._creator=null}},{supportedForProcessing:!1}),F=n.Class.define(function(e){l&&(!0===l||l&c.errorPromise)&&(this._stack=G._getStack()),this._value=e,q(this,e,A)},{cancel:function(){},done:function(e,t){var n=this._value;if(t)try{t.handlesOnError||q(null,n,S,this,t);var r=t(n);return void(r&&\"object\"==typeof r&&\"function\"==typeof r.done&&r.done())}catch(e){n=e}n instanceof Error&&n.message===u||G._doneHandler(n)},then:function(e,t){if(!t)return this;var n,r=this._value;try{t.handlesOnError||q(null,r,S,this,t),n=new H(t(r))}catch(e){n=e===r?this:new K(e)}return n}},{supportedForProcessing:!1}),K=n.Class.derive(F,function(e){l&&(!0===l||l&c.exceptionPromise)&&(this._stack=G._getStack()),this._value=e,q(this,e,L)},{},{supportedForProcessing:!1}),H=n.Class.define(function(e){if(l&&(!0===l||l&c.completePromise)&&(this._stack=G._getStack()),e&&\"object\"==typeof e&&\"function\"==typeof e.then){var t=new j(null);return t._setCompleteValue(e),t}this._value=e},{cancel:function(){},done:function(e){if(e)try{var t=e(this._value);t&&\"object\"==typeof t&&\"function\"==typeof t.done&&t.done()}catch(e){G._doneHandler(e)}},then:function(e){try{var t=e?e(this._value):this._value;return t===this._value?this:new H(t)}catch(e){return new K(e)}}},{supportedForProcessing:!1});var G=n.Class.derive(T,function(e,t){l&&(!0===l||l&c.promise)&&(this._stack=G._getStack()),this._oncancel=t,this._setState(h),this._run();try{e(this._completed.bind(this),this._error.bind(this),this._progress.bind(this))}catch(e){this._setExceptionValue(e)}},{_oncancel:null,_cancelAction:function(){try{if(!this._oncancel)throw new Error(\"Promise did not implement oncancel\");this._oncancel()}catch(e){e.message,e.stack;a.dispatchEvent(\"error\",e)}},_cleanupAction:function(){this._oncancel=null}},{addEventListener:function(e,t,n){a.addEventListener(e,t,n)},any:function(e){return new G(function(t,n){var r=Object.keys(e);0===r.length&&t();var i=0;r.forEach(function(o){G.as(e[o]).then(function(){t({key:o,value:e[o]})},function(a){a instanceof Error&&a.name===u?++i===r.length&&t(G.cancel):n({key:o,value:e[o]})})})},function(){Object.keys(e).forEach(function(t){var n=G.as(e[t]);\"function\"==typeof n.cancel&&n.cancel()})})},as:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then?e:new H(e)},cancel:{get:function(){return B=B||new F(new r(u))}},dispatchEvent:function(e,t){return a.dispatchEvent(e,t)},is:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then},join:function(e){return new G(function(t,n,r){var i=Object.keys(e),o=Array.isArray(e)?[]:{},a=Array.isArray(e)?[]:{},s=0,l=i.length,c=function(e){if(0==--l){var s=Object.keys(o).length;if(0===s)t(a);else{var c=0;i.forEach(function(e){var t=o[e];t instanceof Error&&t.name===u&&c++}),c===s?t(G.cancel):n(o)}}else r({Key:e,Done:!0})};i.forEach(function(t){var n=e[t];void 0===n?s++:G.then(n,function(e){a[t]=e,c(t)},function(e){o[t]=e,c(t)})}),0!==(l-=s)||t(a)},function(){Object.keys(e).forEach(function(t){var n=G.as(e[t]);\"function\"==typeof n.cancel&&n.cancel()})})},removeEventListener:function(e,t,n){a.removeEventListener(e,t,n)},supportedForProcessing:!1,then:function(e,t,n,r){return G.as(e).then(t,n,r)},thenEach:function(e,t,n,r){var i=Array.isArray(e)?[]:{};return Object.keys(e).forEach(function(o){i[o]=G.as(e[o]).then(t,n,r)}),G.join(i)},timeout:function(n,r){var i,o,a=(i=n,new G(function(n){i?o=e.setTimeout(n,i):t._setImmediate(n)},function(){o&&e.clearTimeout(o)}));return r?function(e,t){var n=function(){e.cancel()};return e.then(function(){t.cancel()}),t.then(n,n),t}(a,r):a},wrap:function(e){return new H(e)},wrapError:function(e){return new F(e)},_veryExpensiveTagWithStack:{get:function(){return l},set:function(e){l=e}},_veryExpensiveTagWithStack_tag:c,_getStack:function(){if(e.Debug&&e.Debug.debuggerEnabled)try{throw new Error}catch(e){return e.stack}},_cancelBlocker:function(e,t){if(!G.is(e))return G.wrap(e);var n,r,i=new G(function(e,t){n=e,r=t},function(){n=null,r=null,t&&t()});return e.then(function(e){n&&n(e)},function(e){r&&r(e)}),i}});return Object.defineProperties(G,i.createEventProperties(s)),G._doneHandler=function(e){t._setImmediate(function(){throw e})},{PromiseStateMachine:T,Promise:G,state_created:h}}),n(\"WinJS/Promise\",[\"WinJS/Core/_Base\",\"WinJS/Promise/_StateMachine\"],function(e,t){return e.Namespace.define(\"WinJS\",{Promise:t.Promise}),t.Promise}),(e=t[\"WinJS/Core/_WinJS\"]).TPromise=e.Promise,e.PPromise=e.Promise}();var t=e.Promise,n=e.TPromise,r=(e.PPromise,{});n.addEventListener(\"error\",function(e){var t=e.detail,n=t.id;t.parent?t.handler&&r&&delete r[n]:(r[n]=t,1===Object.keys(r).length&&setTimeout(function(){var e=r;r={},Object.keys(e).forEach(function(t){var n=e[t];n.exception?o(n.exception):n.error&&o(n.error),console.log(\"WARNING: Promise with no error callback:\"+n.id),console.log(n),n.exception&&console.log(n.exception.stack)})},0))});var i=new(function(){function e(){this.listeners=[],this.unexpectedErrorHandler=function(e){setTimeout(function(){if(e.stack)throw new Error(e.message+\"\\n\\n\"+e.stack);throw e},0)}}return e.prototype.emit=function(e){this.listeners.forEach(function(t){t(e)})},e.prototype.onUnexpectedError=function(e){this.unexpectedErrorHandler(e),this.emit(e)},e.prototype.onUnexpectedExternalError=function(e){this.unexpectedErrorHandler(e)},e}());function o(e){var t;(t=e)instanceof Error&&t.name===s&&t.message===s||i.onUnexpectedError(e)}function a(e){return e instanceof Error?{$isError:!0,name:e.name,message:e.message,stack:e.stacktrace||e.stack}:e}var s=\"Canceled\";var u=function(){function e(){this._toDispose=[]}return e.prototype.dispose=function(){this._toDispose=function e(t){for(var n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];return Array.isArray(t)?(t.forEach(function(e){return e&&e.dispose()}),[]):0===n.length?t?(t.dispose(),t):void 0:(e(t),e(n),[])}(this._toDispose)},e.prototype._register=function(e){return this._toDispose.push(e),e},e.None=Object.freeze({dispose:function(){}}),e}(),l=!1,c=\"undefined\"!=typeof process&&void 0!==process.versions&&void 0!==process.versions.electron&&\"renderer\"===process.type;if(\"object\"!=typeof navigator||c){if(\"object\"==typeof process){l=\"win32\"===process.platform,\"darwin\"===process.platform,\"linux\"===process.platform,\"en\";var h=process.env.VSCODE_NLS_CONFIG;if(h)try{var d=JSON.parse(h);d.availableLanguages[\"*\"];d.locale,d._translationsConfigFile}catch(e){}}}else{var f=navigator.userAgent;l=f.indexOf(\"Windows\")>=0,f.indexOf(\"Macintosh\")>=0,f.indexOf(\"Linux\")>=0,navigator.language}var p=l,g=\"object\"==typeof self?self:\"object\"==typeof global?global:{},m=null;function b(e){return null===m&&(m=g.setImmediate?g.setImmediate.bind(g):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):g.setTimeout.bind(g)),m(e)}var _,v,y=(_=function(e,t){return(_=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}_(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),w={done:!0,value:void 0};!function(e){var t={next:function(){return w}};function n(e,t){for(var n=e.next();!n.done;n=e.next())t(n.value)}e.empty=function(){return t},e.fromArray=function(e,t,n){return void 0===t&&(t=0),void 0===n&&(n=e.length),{next:function(){return t>=n?w:{done:!1,value:e[t++]}}}},e.from=function(t){return t?Array.isArray(t)?e.fromArray(t):t:e.empty()},e.map=function(e,t){return{next:function(){var n=e.next();return n.done?w:{done:!1,value:t(n.value)}}}},e.filter=function(e,t){return{next:function(){for(;;){var n=e.next();if(n.done)return w;if(t(n.value))return{done:!1,value:n.value}}}}},e.forEach=n,e.collect=function(e){var t=[];return n(e,function(e){return t.push(e)}),t}}(v||(v={}));!function(e){function t(t,n,r,i){return void 0===n&&(n=0),void 0===r&&(r=t.length),void 0===i&&(i=n-1),e.call(this,t,n,r,i)||this}y(t,e),t.prototype.current=function(){return e.prototype.current.call(this)},t.prototype.previous=function(){return this.index=Math.max(this.index-1,this.start-1),this.current()},t.prototype.first=function(){return this.index=this.start,this.current()},t.prototype.last=function(){return this.index=this.end-1,this.current()},t.prototype.parent=function(){return null}}(function(){function e(e,t,n,r){void 0===t&&(t=0),void 0===n&&(n=e.length),void 0===r&&(r=t-1),this.items=e,this.start=t,this.end=n,this.index=r}return e.prototype.next=function(){return this.index=Math.min(this.index+1,this.end),this.current()},e.prototype.current=function(){return this.index===this.start-1||this.index===this.end?null:this.items[this.index]},e}());var C,T=function(){return function(e){this.element=e}}(),E=function(){function e(){}return e.prototype.isEmpty=function(){return!this._first},e.prototype.unshift=function(e){return this.insert(e,!1)},e.prototype.push=function(e){return this.insert(e,!0)},e.prototype.insert=function(e,t){var n=this,r=new T(e);if(this._first)if(t){var i=this._last;this._last=r,r.prev=i,i.next=r}else{var o=this._first;this._first=r,r.next=o,o.prev=r}else this._first=r,this._last=r;return function(){for(var e=n._first;e instanceof T;){if(e===r){if(e.prev&&e.next){var t=e.prev;t.next=e.next,e.next.prev=t}else e.prev||e.next?e.next?e.prev||(n._first=n._first.next,n._first.prev=void 0):(n._last=n._last.prev,n._last.next=void 0):(n._first=void 0,n._last=void 0);break}e=e.next}}},e.prototype.iterator=function(){var e,t=this._first;return{next:function(){return t?(e?e.value=t.element:e={done:!1,value:t.element},t=t.next,e):w}}},e}();!function(e){var t={dispose:function(){}};e.None=function(){return t}}(C||(C={}));var x=function(){function e(e){void 0===e&&(e=null),this._options=e,this._event=null,this._disposed=!1,this._deliveryQueue=null,this._listeners=null}return Object.defineProperty(e.prototype,\"event\",{get:function(){var t=this;return this._event||(this._event=function(n,r,i){t._listeners||(t._listeners=new E);var o=t._listeners.isEmpty();o&&t._options&&t._options.onFirstListenerAdd&&t._options.onFirstListenerAdd(t);var a,s=t._listeners.push(r?[n,r]:n);return o&&t._options&&t._options.onFirstListenerDidAdd&&t._options.onFirstListenerDidAdd(t),t._options&&t._options.onListenerDidAdd&&t._options.onListenerDidAdd(t,n,r),a={dispose:function(){(a.dispose=e._noop,t._disposed)||(s(),t._options&&t._options.onLastListenerRemove&&(t._listeners&&!t._listeners.isEmpty()||t._options.onLastListenerRemove(t)))}},Array.isArray(i)&&i.push(a),a}),this._event},enumerable:!0,configurable:!0}),e.prototype.fire=function(e){if(this._listeners){this._deliveryQueue||(this._deliveryQueue=[]);for(var t=this._listeners.iterator(),n=t.next();!n.done;n=t.next())this._deliveryQueue.push([n.value,e]);for(;this._deliveryQueue.length>0;){var r=this._deliveryQueue.shift(),i=r[0],a=r[1];try{\"function\"==typeof i?i.call(void 0,a):i[0].call(i[1],a)}catch(n){o(n)}}}},e.prototype.dispose=function(){this._listeners&&(this._listeners=null),this._deliveryQueue&&(this._deliveryQueue.length=0),this._disposed=!0},e._noop=function(){},e}();!function(){function e(){var e=this;this.hasListeners=!1,this.events=[],this.emitter=new x({onFirstListenerAdd:function(){return e.onFirstListenerAdd()},onLastListenerRemove:function(){return e.onLastListenerRemove()}})}Object.defineProperty(e.prototype,\"event\",{get:function(){return this.emitter.event},enumerable:!0,configurable:!0}),e.prototype.add=function(e){var t=this,n={event:e,listener:null};this.events.push(n),this.hasListeners&&this.hook(n);var r;return r=function(e){var t,n=this,r=!1;return function(){return r?t:(r=!0,t=e.apply(n,arguments))}}(function(){t.hasListeners&&t.unhook(n);var e=t.events.indexOf(n);t.events.splice(e,1)}),{dispose:function(){r()}}},e.prototype.onFirstListenerAdd=function(){var e=this;this.hasListeners=!0,this.events.forEach(function(t){return e.hook(t)})},e.prototype.onLastListenerRemove=function(){var e=this;this.hasListeners=!1,this.events.forEach(function(t){return e.unhook(t)})},e.prototype.hook=function(e){var t=this;e.listener=e.event(function(e){return t.emitter.fire(e)})},e.prototype.unhook=function(e){e.listener&&e.listener.dispose(),e.listener=null},e.prototype.dispose=function(){this.emitter.dispose()}}();(function(){function e(e){this._event=e}Object.defineProperty(e.prototype,\"event\",{get:function(){return this._event},enumerable:!0,configurable:!0}),e.prototype.map=function(t){return new e((n=this._event,r=t,function(e,t,i){return void 0===t&&(t=null),n(function(n){return e.call(t,r(n))},null,i)}));var n,r},e.prototype.filter=function(t){return new e((n=this._event,r=t,function(e,t,i){return void 0===t&&(t=null),n(function(n){return r(n)&&e.call(t,n)},null,i)}));var n,r},e.prototype.on=function(e,t,n){return this._event(e,t,n)}})(),function(){function e(){var e=this;this.listening=!1,this.inputEvent=C.None,this.inputEventListener=u.None,this.emitter=new x({onFirstListenerDidAdd:function(){e.listening=!0,e.inputEventListener=e.inputEvent(e.emitter.fire,e.emitter)},onLastListenerRemove:function(){e.listening=!1,e.inputEventListener.dispose()}}),this.event=this.emitter.event}Object.defineProperty(e.prototype,\"input\",{set:function(e){this.inputEvent=e,this.listening&&(this.inputEventListener.dispose(),this.inputEventListener=e(this.emitter.fire,this.emitter))},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.inputEventListener.dispose(),this.emitter.dispose()}}();var S,k=Object.freeze(function(e,t){var n=setTimeout(e.bind(t),0);return{dispose:function(){clearTimeout(n)}}});!function(e){e.isCancellationToken=function(t){return t===e.None||t===e.Cancelled||t instanceof A||!(!t||\"object\"!=typeof t)&&\"boolean\"==typeof t.isCancellationRequested&&\"function\"==typeof t.onCancellationRequested},e.None=Object.freeze({isCancellationRequested:!1,onCancellationRequested:C.None}),e.Cancelled=Object.freeze({isCancellationRequested:!0,onCancellationRequested:k})}(S||(S={}));var A=function(){function e(){this._isCancelled=!1,this._emitter=null}return e.prototype.cancel=function(){this._isCancelled||(this._isCancelled=!0,this._emitter&&(this._emitter.fire(void 0),this.dispose()))},Object.defineProperty(e.prototype,\"isCancellationRequested\",{get:function(){return this._isCancelled},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"onCancellationRequested\",{get:function(){return this._isCancelled?k:(this._emitter||(this._emitter=new x),this._emitter.event)},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this._emitter&&(this._emitter.dispose(),this._emitter=null)},e}(),L=function(){function e(){}return Object.defineProperty(e.prototype,\"token\",{get:function(){return this._token||(this._token=new A),this._token},enumerable:!0,configurable:!0}),e.prototype.cancel=function(){this._token?this._token instanceof A&&this._token.cancel():this._token=S.Cancelled},e.prototype.dispose=function(){this._token?this._token instanceof A&&this._token.dispose():this._token=S.None},e}(),N=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();(function(e){function t(t,n){var r=e.call(this)||this;return r._token=-1,\"function\"==typeof t&&\"number\"==typeof n&&r.setIfNotSet(t,n),r}N(t,e),t.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},t.prototype.cancel=function(){-1!==this._token&&(clearTimeout(this._token),this._token=-1)},t.prototype.cancelAndSet=function(e,t){var n=this;this.cancel(),this._token=setTimeout(function(){n._token=-1,e()},t)},t.prototype.setIfNotSet=function(e,t){var n=this;-1===this._token&&(this._token=setTimeout(function(){n._token=-1,e()},t))}})(u),function(e){function t(){var t=e.call(this)||this;return t._token=-1,t}N(t,e),t.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},t.prototype.cancel=function(){-1!==this._token&&(clearInterval(this._token),this._token=-1)},t.prototype.cancelAndSet=function(e,t){this.cancel(),this._token=setInterval(function(){e()},t)}}(u);!function(){if(\"function\"!=typeof requestIdleCallback||\"function\"!=typeof cancelIdleCallback)Object.freeze({didTimeout:!0,timeRemaining:function(){return 15}})}();var O=function(){function e(e){var n,r;(r=n=e)&&\"function\"==typeof r.then&&\"function\"==typeof n.done?this._winjsPromise=e:this._winjsPromise=new t(function(t,n){var r=!0;e(function(e){r?b(function(){return t(e)}):t(e)},function(e){r?b(function(){return n(e)}):n(e)}),r=!1})}return e.all=function(n){return new e(t.join(n).then(null,function(e){for(var t in e)if(e.hasOwnProperty(t))return e[t]}))},e.race=function(n){return new e(t.any(n).then(function(e){return e.value},function(e){return e.value}))},e.resolve=function(n){return new e(t.wrap(n))},e.reject=function(n){return new e(t.wrapError(n))},e.prototype.then=function(n,r){var i=!0,o=new e(this._winjsPromise.then(n&&function(e){return i?new t(function(t,r){b(function(){var i;try{i=n(e)}catch(e){return void r(e)}t(i)})}):n(e)},r&&function(e){return i?new t(function(t,n){b(function(){var i;try{i=r(e)}catch(e){return void n(e)}t(i)})}):r(e)}));return i=!1,o},e.prototype.catch=function(e){return this.then(null,e)},e}(),P=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),I=self;void 0===I.Promise&&(I.Promise=O);var q=\"$initialize\",R=function(){function e(e){this._workerId=-1,this._handler=e,this._lastSentReq=0,this._pendingReplies=Object.create(null)}return e.prototype.setWorkerId=function(e){this._workerId=e},e.prototype.sendMessage=function(e,t){var n=this,r=String(++this._lastSentReq);return new Promise(function(i,o){n._pendingReplies[r]={resolve:i,reject:o},n._send({vsWorker:n._workerId,req:r,method:e,args:t})})},e.prototype.handleMessage=function(e){var t;try{t=JSON.parse(e)}catch(e){return}t&&t.vsWorker&&(-1!==this._workerId&&t.vsWorker!==this._workerId||this._handleMessage(t))},e.prototype._handleMessage=function(e){var t=this;if(e.seq){var n=e;if(!this._pendingReplies[n.seq])return void console.warn(\"Got reply to unknown seq\");var r=this._pendingReplies[n.seq];if(delete this._pendingReplies[n.seq],n.err){var i=n.err;return n.err.$isError&&((i=new Error).name=n.err.name,i.message=n.err.message,i.stack=n.err.stack),void r.reject(i)}r.resolve(n.res)}else{var o=e,s=o.req;this._handler.handleMessage(o.method,o.args).then(function(e){t._send({vsWorker:t._workerId,seq:s,res:e,err:void 0})},function(e){e.detail instanceof Error&&(e.detail=a(e.detail)),t._send({vsWorker:t._workerId,seq:s,res:void 0,err:a(e)})})}},e.prototype._send=function(e){var t=JSON.stringify(e);this._handler.sendMessage(t)},e}(),D=(function(e){function t(t,n){var r=e.call(this)||this,i=null;r._worker=r._register(t.create(\"vs/base/common/worker/simpleWorker\",function(e){r._protocol.handleMessage(e)},function(e){i&&i(e)})),r._protocol=new R({sendMessage:function(e){r._worker.postMessage(e)},handleMessage:function(e,t){return Promise.resolve(null)}}),r._protocol.setWorkerId(r._worker.getId());var o=null;void 0!==self.require&&\"function\"==typeof self.require.getConfig?o=self.require.getConfig():void 0!==self.requirejs&&(o=self.requirejs.s.contexts._.config),r._onModuleLoaded=r._protocol.sendMessage(q,[r._worker.getId(),n,o]),r._lazyProxy=new Promise(function(e,t){i=t,r._onModuleLoaded.then(function(t){for(var n={},r=0;r<t.length;r++)n[t[r]]=s(t[r],a);e(n)},function(e){t(e),r._onError(\"Worker failed to load \"+n,e)})});var a=function(e,t){return r._request(e,t)},s=function(e,t){return function(){var n=Array.prototype.slice.call(arguments,0);return t(e,n)}};return r}P(t,e),t.prototype.getProxyObject=function(){return this._lazyProxy},t.prototype._request=function(e,t){var n=this;return new Promise(function(r,i){n._onModuleLoaded.then(function(){n._protocol.sendMessage(e,t).then(r,i)},i)})},t.prototype._onError=function(e,t){console.error(e),console.info(t)}}(u),function(){function e(e,t){var n=this;this._requestHandler=t,this._protocol=new R({sendMessage:function(t){e(t)},handleMessage:function(e,t){return n._handleMessage(e,t)}})}return e.prototype.onmessage=function(e){this._protocol.handleMessage(e)},e.prototype._handleMessage=function(e,t){if(e===q)return this.initialize(t[0],t[1],t[2]);if(!this._requestHandler||\"function\"!=typeof this._requestHandler[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._requestHandler[e].apply(this._requestHandler,t))}catch(e){return Promise.reject(e)}},e.prototype.initialize=function(e,t,n){var r=this;if(this._protocol.setWorkerId(e),this._requestHandler){var i=[];for(var o in this._requestHandler)\"function\"==typeof this._requestHandler[o]&&i.push(o);return Promise.resolve(i)}return n&&(void 0!==n.baseUrl&&delete n.baseUrl,void 0!==n.paths&&void 0!==n.paths.vs&&delete n.paths.vs,n.catchError=!0,self.require.config(n)),new Promise(function(e,n){self.require([t],function(){for(var t=[],i=0;i<arguments.length;i++)t[i]=arguments[i];var o=t[0];if(r._requestHandler=o.create(),r._requestHandler){var a=[];for(var s in r._requestHandler)\"function\"==typeof r._requestHandler[s]&&a.push(s);e(a)}else n(new Error(\"No RequestHandler!\"))},n)})},e}());function U(e,t){return function e(t,n,r,i,o){if(i<=r)return;var a=r+(i-r)/2|0;e(t,n,r,a,o);e(t,n,a+1,i,o);if(n(t[a],t[a+1])<=0)return;!function(e,t,n,r,i,o){for(var a=n,s=r+1,u=n;u<=i;u++)o[u]=e[u];for(var u=n;u<=i;u++)a>r?e[u]=o[s++]:s>i?e[u]=o[a++]:t(o[s],o[a])<0?e[u]=o[s++]:e[u]=o[a++]}(t,n,r,a,i,o)}(e,t,0,e.length-1,[]),e}var M=function(){function e(e,t,n,r){this.originalStart=e,this.originalLength=t,this.modifiedStart=n,this.modifiedLength=r}return e.prototype.getOriginalEnd=function(){return this.originalStart+this.originalLength},e.prototype.getModifiedEnd=function(){return this.modifiedStart+this.modifiedLength},e}();function W(e){return{getLength:function(){return e.length},getElementAtIndex:function(t){return e.charCodeAt(t)}}}function V(e,t,n){return new H(W(e),W(t)).ComputeDiff(n)}var B,j=function(){function e(){}return e.Assert=function(e,t){if(!e)throw new Error(t)},e}(),F=function(){function e(){}return e.Copy=function(e,t,n,r,i){for(var o=0;o<i;o++)n[r+o]=e[t+o]},e}(),K=function(){function e(){this.m_changes=[],this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE,this.m_originalCount=0,this.m_modifiedCount=0}return e.prototype.MarkNextChange=function(){(this.m_originalCount>0||this.m_modifiedCount>0)&&this.m_changes.push(new M(this.m_originalStart,this.m_originalCount,this.m_modifiedStart,this.m_modifiedCount)),this.m_originalCount=0,this.m_modifiedCount=0,this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE},e.prototype.AddOriginalElement=function(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_originalCount++},e.prototype.AddModifiedElement=function(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_modifiedCount++},e.prototype.getChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes},e.prototype.getReverseChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes.reverse(),this.m_changes},e}(),H=function(){function e(e,t,n){void 0===n&&(n=null),this.OriginalSequence=e,this.ModifiedSequence=t,this.ContinueProcessingPredicate=n,this.m_forwardHistory=[],this.m_reverseHistory=[]}return e.prototype.ElementsAreEqual=function(e,t){return this.OriginalSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(t)},e.prototype.OriginalElementsAreEqual=function(e,t){return this.OriginalSequence.getElementAtIndex(e)===this.OriginalSequence.getElementAtIndex(t)},e.prototype.ModifiedElementsAreEqual=function(e,t){return this.ModifiedSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(t)},e.prototype.ComputeDiff=function(e){return this._ComputeDiff(0,this.OriginalSequence.getLength()-1,0,this.ModifiedSequence.getLength()-1,e)},e.prototype._ComputeDiff=function(e,t,n,r,i){var o=this.ComputeDiffRecursive(e,t,n,r,[!1]);return i?this.PrettifyChanges(o):o},e.prototype.ComputeDiffRecursive=function(e,t,n,r,i){for(i[0]=!1;e<=t&&n<=r&&this.ElementsAreEqual(e,n);)e++,n++;for(;t>=e&&r>=n&&this.ElementsAreEqual(t,r);)t--,r--;if(e>t||n>r){var o=void 0;return n<=r?(j.Assert(e===t+1,\"originalStart should only be one more than originalEnd\"),o=[new M(e,0,n,r-n+1)]):e<=t?(j.Assert(n===r+1,\"modifiedStart should only be one more than modifiedEnd\"),o=[new M(e,t-e+1,n,0)]):(j.Assert(e===t+1,\"originalStart should only be one more than originalEnd\"),j.Assert(n===r+1,\"modifiedStart should only be one more than modifiedEnd\"),o=[]),o}var a=[0],s=[0],u=this.ComputeRecursionPoint(e,t,n,r,a,s,i),l=a[0],c=s[0];if(null!==u)return u;if(!i[0]){var h=this.ComputeDiffRecursive(e,l,n,c,i),d=[];return d=i[0]?[new M(l+1,t-(l+1)+1,c+1,r-(c+1)+1)]:this.ComputeDiffRecursive(l+1,t,c+1,r,i),this.ConcatenateChanges(h,d)}return[new M(e,t-e+1,n,r-n+1)]},e.prototype.WALKTRACE=function(e,t,n,r,i,o,a,s,u,l,c,h,d,f,p,g,m,b){var _,v,y=null,w=new K,C=t,T=n,E=d[0]-g[0]-r,x=Number.MIN_VALUE,S=this.m_forwardHistory.length-1;do{(v=E+e)===C||v<T&&u[v-1]<u[v+1]?(f=(c=u[v+1])-E-r,c<x&&w.MarkNextChange(),x=c,w.AddModifiedElement(c+1,f),E=v+1-e):(f=(c=u[v-1]+1)-E-r,c<x&&w.MarkNextChange(),x=c-1,w.AddOriginalElement(c,f+1),E=v-1-e),S>=0&&(e=(u=this.m_forwardHistory[S])[0],C=1,T=u.length-1)}while(--S>=-1);if(_=w.getReverseChanges(),b[0]){var k=d[0]+1,A=g[0]+1;if(null!==_&&_.length>0){var L=_[_.length-1];k=Math.max(k,L.getOriginalEnd()),A=Math.max(A,L.getModifiedEnd())}y=[new M(k,h-k+1,A,p-A+1)]}else{w=new K,C=o,T=a,E=d[0]-g[0]-s,x=Number.MAX_VALUE,S=m?this.m_reverseHistory.length-1:this.m_reverseHistory.length-2;do{(v=E+i)===C||v<T&&l[v-1]>=l[v+1]?(f=(c=l[v+1]-1)-E-s,c>x&&w.MarkNextChange(),x=c+1,w.AddOriginalElement(c+1,f+1),E=v+1-i):(f=(c=l[v-1])-E-s,c>x&&w.MarkNextChange(),x=c,w.AddModifiedElement(c+1,f+1),E=v-1-i),S>=0&&(i=(l=this.m_reverseHistory[S])[0],C=1,T=l.length-1)}while(--S>=-1);y=w.getChanges()}return this.ConcatenateChanges(_,y)},e.prototype.ComputeRecursionPoint=function(e,t,n,r,i,o,a){var s,u=0,l=0,c=0,h=0,d=0,f=0;e--,n--,i[0]=0,o[0]=0,this.m_forwardHistory=[],this.m_reverseHistory=[];var p,g,m=t-e+(r-n),b=m+1,_=new Array(b),v=new Array(b),y=r-n,w=t-e,C=e-n,T=t-r,E=(w-y)%2==0;for(_[y]=e,v[w]=t,a[0]=!1,s=1;s<=m/2+1;s++){var x=0,S=0;for(c=this.ClipDiagonalBound(y-s,s,y,b),h=this.ClipDiagonalBound(y+s,s,y,b),p=c;p<=h;p+=2){for(l=(u=p===c||p<h&&_[p-1]<_[p+1]?_[p+1]:_[p-1]+1)-(p-y)-C,g=u;u<t&&l<r&&this.ElementsAreEqual(u+1,l+1);)u++,l++;if(_[p]=u,u+l>x+S&&(x=u,S=l),!E&&Math.abs(p-w)<=s-1&&u>=v[p])return i[0]=u,o[0]=l,g<=v[p]&&s<=1448?this.WALKTRACE(y,c,h,C,w,d,f,T,_,v,u,t,i,l,r,o,E,a):null}var k=(x-e+(S-n)-s)/2;if(null!==this.ContinueProcessingPredicate&&!this.ContinueProcessingPredicate(x,this.OriginalSequence,k))return a[0]=!0,i[0]=x,o[0]=S,k>0&&s<=1448?this.WALKTRACE(y,c,h,C,w,d,f,T,_,v,u,t,i,l,r,o,E,a):[new M(++e,t-e+1,++n,r-n+1)];for(d=this.ClipDiagonalBound(w-s,s,w,b),f=this.ClipDiagonalBound(w+s,s,w,b),p=d;p<=f;p+=2){for(l=(u=p===d||p<f&&v[p-1]>=v[p+1]?v[p+1]-1:v[p-1])-(p-w)-T,g=u;u>e&&l>n&&this.ElementsAreEqual(u,l);)u--,l--;if(v[p]=u,E&&Math.abs(p-y)<=s&&u<=_[p])return i[0]=u,o[0]=l,g>=_[p]&&s<=1448?this.WALKTRACE(y,c,h,C,w,d,f,T,_,v,u,t,i,l,r,o,E,a):null}if(s<=1447){var A=new Array(h-c+2);A[0]=y-c+1,F.Copy(_,c,A,1,h-c+1),this.m_forwardHistory.push(A),(A=new Array(f-d+2))[0]=w-d+1,F.Copy(v,d,A,1,f-d+1),this.m_reverseHistory.push(A)}}return this.WALKTRACE(y,c,h,C,w,d,f,T,_,v,u,t,i,l,r,o,E,a)},e.prototype.PrettifyChanges=function(e){for(var t=0;t<e.length;t++){for(var n=e[t],r=t<e.length-1?e[t+1].originalStart:this.OriginalSequence.getLength(),i=t<e.length-1?e[t+1].modifiedStart:this.ModifiedSequence.getLength(),o=n.originalLength>0,a=n.modifiedLength>0;n.originalStart+n.originalLength<r&&n.modifiedStart+n.modifiedLength<i&&(!o||this.OriginalElementsAreEqual(n.originalStart,n.originalStart+n.originalLength))&&(!a||this.ModifiedElementsAreEqual(n.modifiedStart,n.modifiedStart+n.modifiedLength));)n.originalStart++,n.modifiedStart++;var s=[null];t<e.length-1&&this.ChangesOverlap(e[t],e[t+1],s)&&(e[t]=s[0],e.splice(t+1,1),t--)}for(t=e.length-1;t>=0;t--){n=e[t],r=0,i=0;if(t>0){var u=e[t-1];u.originalLength>0&&(r=u.originalStart+u.originalLength),u.modifiedLength>0&&(i=u.modifiedStart+u.modifiedLength)}o=n.originalLength>0,a=n.modifiedLength>0;for(var l=0,c=this._boundaryScore(n.originalStart,n.originalLength,n.modifiedStart,n.modifiedLength),h=1;;h++){var d=n.originalStart-h,f=n.modifiedStart-h;if(d<r||f<i)break;if(o&&!this.OriginalElementsAreEqual(d,d+n.originalLength))break;if(a&&!this.ModifiedElementsAreEqual(f,f+n.modifiedLength))break;var p=this._boundaryScore(d,n.originalLength,f,n.modifiedLength);p>c&&(c=p,l=h)}n.originalStart-=l,n.modifiedStart-=l}return e},e.prototype._OriginalIsBoundary=function(e){if(e<=0||e>=this.OriginalSequence.getLength()-1)return!0;var t=this.OriginalSequence.getElementAtIndex(e);return\"string\"==typeof t&&/^\\s*$/.test(t)},e.prototype._OriginalRegionIsBoundary=function(e,t){if(this._OriginalIsBoundary(e)||this._OriginalIsBoundary(e-1))return!0;if(t>0){var n=e+t;if(this._OriginalIsBoundary(n-1)||this._OriginalIsBoundary(n))return!0}return!1},e.prototype._ModifiedIsBoundary=function(e){if(e<=0||e>=this.ModifiedSequence.getLength()-1)return!0;var t=this.ModifiedSequence.getElementAtIndex(e);return\"string\"==typeof t&&/^\\s*$/.test(t)},e.prototype._ModifiedRegionIsBoundary=function(e,t){if(this._ModifiedIsBoundary(e)||this._ModifiedIsBoundary(e-1))return!0;if(t>0){var n=e+t;if(this._ModifiedIsBoundary(n-1)||this._ModifiedIsBoundary(n))return!0}return!1},e.prototype._boundaryScore=function(e,t,n,r){return(this._OriginalRegionIsBoundary(e,t)?1:0)+(this._ModifiedRegionIsBoundary(n,r)?1:0)},e.prototype.ConcatenateChanges=function(e,t){var n=[];if(0===e.length||0===t.length)return t.length>0?t:e;if(this.ChangesOverlap(e[e.length-1],t[0],n)){var r=new Array(e.length+t.length-1);return F.Copy(e,0,r,0,e.length-1),r[e.length-1]=n[0],F.Copy(t,1,r,e.length,t.length-1),r}r=new Array(e.length+t.length);return F.Copy(e,0,r,0,e.length),F.Copy(t,0,r,e.length,t.length),r},e.prototype.ChangesOverlap=function(e,t,n){if(j.Assert(e.originalStart<=t.originalStart,\"Left change is not less than or equal to right change\"),j.Assert(e.modifiedStart<=t.modifiedStart,\"Left change is not less than or equal to right change\"),e.originalStart+e.originalLength>=t.originalStart||e.modifiedStart+e.modifiedLength>=t.modifiedStart){var r=e.originalStart,i=e.originalLength,o=e.modifiedStart,a=e.modifiedLength;return e.originalStart+e.originalLength>=t.originalStart&&(i=t.originalStart+t.originalLength-e.originalStart),e.modifiedStart+e.modifiedLength>=t.modifiedStart&&(a=t.modifiedStart+t.modifiedLength-e.modifiedStart),n[0]=new M(r,i,o,a),!0}return n[0]=null,!1},e.prototype.ClipDiagonalBound=function(e,t,n,r){if(e>=0&&e<r)return e;var i=t%2==0;return e<0?i===(n%2==0)?0:1:i===((r-n-1)%2==0)?r-1:r-2},e}(),G=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Y=/^\\w[\\w\\d+.-]*$/,z=/^\\//,J=/^\\/\\//;var $=\"\",Q=\"/\",Z=/^(([^:\\/?#]+?):)?(\\/\\/([^\\/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?/,X=function(){function e(e,t,n,r,i){\"object\"==typeof e?(this.scheme=e.scheme||$,this.authority=e.authority||$,this.path=e.path||$,this.query=e.query||$,this.fragment=e.fragment||$):(this.scheme=e||$,this.authority=t||$,this.path=function(e,t){switch(e){case\"https\":case\"http\":case\"file\":t?t[0]!==Q&&(t=Q+t):t=Q}return t}(this.scheme,n||$),this.query=r||$,this.fragment=i||$,function(e){if(!e.scheme)throw new Error('[UriError]: Scheme is missing: {scheme: \"\", authority: \"'+e.authority+'\", path: \"'+e.path+'\", query: \"'+e.query+'\", fragment: \"'+e.fragment+'\"}');if(e.scheme&&!Y.test(e.scheme))throw new Error(\"[UriError]: Scheme contains illegal characters.\");if(e.path)if(e.authority){if(!z.test(e.path))throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash (\"/\") character')}else if(J.test(e.path))throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters (\"//\")')}(this))}return e.isUri=function(t){return t instanceof e||!!t&&(\"string\"==typeof t.authority&&\"string\"==typeof t.fragment&&\"string\"==typeof t.path&&\"string\"==typeof t.query&&\"string\"==typeof t.scheme)},Object.defineProperty(e.prototype,\"fsPath\",{get:function(){return ie(this)},enumerable:!0,configurable:!0}),e.prototype.with=function(e){if(!e)return this;var t=e.scheme,n=e.authority,r=e.path,i=e.query,o=e.fragment;return void 0===t?t=this.scheme:null===t&&(t=$),void 0===n?n=this.authority:null===n&&(n=$),void 0===r?r=this.path:null===r&&(r=$),void 0===i?i=this.query:null===i&&(i=$),void 0===o?o=this.fragment:null===o&&(o=$),t===this.scheme&&n===this.authority&&r===this.path&&i===this.query&&o===this.fragment?this:new ee(t,n,r,i,o)},e.parse=function(e){var t=Z.exec(e);return t?new ee(t[2]||$,decodeURIComponent(t[4]||$),decodeURIComponent(t[5]||$),decodeURIComponent(t[7]||$),decodeURIComponent(t[9]||$)):new ee($,$,$,$,$)},e.file=function(e){var t=$;if(p&&(e=e.replace(/\\\\/g,Q)),e[0]===Q&&e[1]===Q){var n=e.indexOf(Q,2);-1===n?(t=e.substring(2),e=Q):(t=e.substring(2,n),e=e.substring(n)||Q)}return new ee(\"file\",t,e,$,$)},e.from=function(e){return new ee(e.scheme,e.authority,e.path,e.query,e.fragment)},e.prototype.toString=function(e){return void 0===e&&(e=!1),oe(this,e)},e.prototype.toJSON=function(){return this},e.revive=function(t){if(t){if(t instanceof e)return t;var n=new ee(t);return n._fsPath=t.fsPath,n._formatted=t.external,n}return t},e}(),ee=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._formatted=null,t._fsPath=null,t}return G(t,e),Object.defineProperty(t.prototype,\"fsPath\",{get:function(){return this._fsPath||(this._fsPath=ie(this)),this._fsPath},enumerable:!0,configurable:!0}),t.prototype.toString=function(e){return void 0===e&&(e=!1),e?oe(this,!0):(this._formatted||(this._formatted=oe(this,!1)),this._formatted)},t.prototype.toJSON=function(){var e={$mid:1};return this._fsPath&&(e.fsPath=this._fsPath),this._formatted&&(e.external=this._formatted),this.path&&(e.path=this.path),this.scheme&&(e.scheme=this.scheme),this.authority&&(e.authority=this.authority),this.query&&(e.query=this.query),this.fragment&&(e.fragment=this.fragment),e},t}(X),te=((B={})[58]=\"%3A\",B[47]=\"%2F\",B[63]=\"%3F\",B[35]=\"%23\",B[91]=\"%5B\",B[93]=\"%5D\",B[64]=\"%40\",B[33]=\"%21\",B[36]=\"%24\",B[38]=\"%26\",B[39]=\"%27\",B[40]=\"%28\",B[41]=\"%29\",B[42]=\"%2A\",B[43]=\"%2B\",B[44]=\"%2C\",B[59]=\"%3B\",B[61]=\"%3D\",B[32]=\"%20\",B);function ne(e,t){for(var n=void 0,r=-1,i=0;i<e.length;i++){var o=e.charCodeAt(i);if(o>=97&&o<=122||o>=65&&o<=90||o>=48&&o<=57||45===o||46===o||95===o||126===o||t&&47===o)-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),void 0!==n&&(n+=e.charAt(i));else{void 0===n&&(n=e.substr(0,i));var a=te[o];void 0!==a?(-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),n+=a):-1===r&&(r=i)}}return-1!==r&&(n+=encodeURIComponent(e.substring(r))),void 0!==n?n:e}function re(e){for(var t=void 0,n=0;n<e.length;n++){var r=e.charCodeAt(n);35===r||63===r?(void 0===t&&(t=e.substr(0,n)),t+=te[r]):void 0!==t&&(t+=e[n])}return void 0!==t?t:e}function ie(e){var t;return t=e.authority&&e.path.length>1&&\"file\"===e.scheme?\"//\"+e.authority+e.path:47===e.path.charCodeAt(0)&&(e.path.charCodeAt(1)>=65&&e.path.charCodeAt(1)<=90||e.path.charCodeAt(1)>=97&&e.path.charCodeAt(1)<=122)&&58===e.path.charCodeAt(2)?e.path[1].toLowerCase()+e.path.substr(2):e.path,p&&(t=t.replace(/\\//g,\"\\\\\")),t}function oe(e,t){var n=t?re:ne,r=\"\",i=e.scheme,o=e.authority,a=e.path,s=e.query,u=e.fragment;if(i&&(r+=i,r+=\":\"),(o||\"file\"===i)&&(r+=Q,r+=Q),o){var l=o.indexOf(\"@\");if(-1!==l){var c=o.substr(0,l);o=o.substr(l+1),-1===(l=c.indexOf(\":\"))?r+=n(c,!1):(r+=n(c.substr(0,l),!1),r+=\":\",r+=n(c.substr(l+1),!1)),r+=\"@\"}-1===(l=(o=o.toLowerCase()).indexOf(\":\"))?r+=n(o,!1):(r+=n(o.substr(0,l),!1),r+=o.substr(l))}if(a){if(a.length>=3&&47===a.charCodeAt(0)&&58===a.charCodeAt(2))(h=a.charCodeAt(1))>=65&&h<=90&&(a=\"/\"+String.fromCharCode(h+32)+\":\"+a.substr(3));else if(a.length>=2&&58===a.charCodeAt(1)){var h;(h=a.charCodeAt(0))>=65&&h<=90&&(a=String.fromCharCode(h+32)+\":\"+a.substr(2))}r+=n(a,!0)}return s&&(r+=\"?\",r+=n(s,!1)),u&&(r+=\"#\",r+=t?u:ne(u,!1)),r}var ae=function(){function e(e,t){this.lineNumber=e,this.column=t}return e.prototype.with=function(t,n){return void 0===t&&(t=this.lineNumber),void 0===n&&(n=this.column),t===this.lineNumber&&n===this.column?this:new e(t,n)},e.prototype.delta=function(e,t){return void 0===e&&(e=0),void 0===t&&(t=0),this.with(this.lineNumber+e,this.column+t)},e.prototype.equals=function(t){return e.equals(this,t)},e.equals=function(e,t){return!e&&!t||!!e&&!!t&&e.lineNumber===t.lineNumber&&e.column===t.column},e.prototype.isBefore=function(t){return e.isBefore(this,t)},e.isBefore=function(e,t){return e.lineNumber<t.lineNumber||!(t.lineNumber<e.lineNumber)&&e.column<t.column},e.prototype.isBeforeOrEqual=function(t){return e.isBeforeOrEqual(this,t)},e.isBeforeOrEqual=function(e,t){return e.lineNumber<t.lineNumber||!(t.lineNumber<e.lineNumber)&&e.column<=t.column},e.compare=function(e,t){var n=0|e.lineNumber,r=0|t.lineNumber;return n===r?(0|e.column)-(0|t.column):n-r},e.prototype.clone=function(){return new e(this.lineNumber,this.column)},e.prototype.toString=function(){return\"(\"+this.lineNumber+\",\"+this.column+\")\"},e.lift=function(t){return new e(t.lineNumber,t.column)},e.isIPosition=function(e){return e&&\"number\"==typeof e.lineNumber&&\"number\"==typeof e.column},e}(),se=function(){function e(e,t,n,r){e>n||e===n&&t>r?(this.startLineNumber=n,this.startColumn=r,this.endLineNumber=e,this.endColumn=t):(this.startLineNumber=e,this.startColumn=t,this.endLineNumber=n,this.endColumn=r)}return e.prototype.isEmpty=function(){return e.isEmpty(this)},e.isEmpty=function(e){return e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn},e.prototype.containsPosition=function(t){return e.containsPosition(this,t)},e.containsPosition=function(e,t){return!(t.lineNumber<e.startLineNumber||t.lineNumber>e.endLineNumber)&&(!(t.lineNumber===e.startLineNumber&&t.column<e.startColumn)&&!(t.lineNumber===e.endLineNumber&&t.column>e.endColumn))},e.prototype.containsRange=function(t){return e.containsRange(this,t)},e.containsRange=function(e,t){return!(t.startLineNumber<e.startLineNumber||t.endLineNumber<e.startLineNumber)&&(!(t.startLineNumber>e.endLineNumber||t.endLineNumber>e.endLineNumber)&&(!(t.startLineNumber===e.startLineNumber&&t.startColumn<e.startColumn)&&!(t.endLineNumber===e.endLineNumber&&t.endColumn>e.endColumn)))},e.prototype.plusRange=function(t){return e.plusRange(this,t)},e.plusRange=function(t,n){var r,i,o,a;return n.startLineNumber<t.startLineNumber?(r=n.startLineNumber,i=n.startColumn):n.startLineNumber===t.startLineNumber?(r=n.startLineNumber,i=Math.min(n.startColumn,t.startColumn)):(r=t.startLineNumber,i=t.startColumn),n.endLineNumber>t.endLineNumber?(o=n.endLineNumber,a=n.endColumn):n.endLineNumber===t.endLineNumber?(o=n.endLineNumber,a=Math.max(n.endColumn,t.endColumn)):(o=t.endLineNumber,a=t.endColumn),new e(r,i,o,a)},e.prototype.intersectRanges=function(t){return e.intersectRanges(this,t)},e.intersectRanges=function(t,n){var r=t.startLineNumber,i=t.startColumn,o=t.endLineNumber,a=t.endColumn,s=n.startLineNumber,u=n.startColumn,l=n.endLineNumber,c=n.endColumn;return r<s?(r=s,i=u):r===s&&(i=Math.max(i,u)),o>l?(o=l,a=c):o===l&&(a=Math.min(a,c)),r>o?null:r===o&&i>a?null:new e(r,i,o,a)},e.prototype.equalsRange=function(t){return e.equalsRange(this,t)},e.equalsRange=function(e,t){return!!e&&!!t&&e.startLineNumber===t.startLineNumber&&e.startColumn===t.startColumn&&e.endLineNumber===t.endLineNumber&&e.endColumn===t.endColumn},e.prototype.getEndPosition=function(){return new ae(this.endLineNumber,this.endColumn)},e.prototype.getStartPosition=function(){return new ae(this.startLineNumber,this.startColumn)},e.prototype.toString=function(){return\"[\"+this.startLineNumber+\",\"+this.startColumn+\" -> \"+this.endLineNumber+\",\"+this.endColumn+\"]\"},e.prototype.setEndPosition=function(t,n){return new e(this.startLineNumber,this.startColumn,t,n)},e.prototype.setStartPosition=function(t,n){return new e(t,n,this.endLineNumber,this.endColumn)},e.prototype.collapseToStart=function(){return e.collapseToStart(this)},e.collapseToStart=function(t){return new e(t.startLineNumber,t.startColumn,t.startLineNumber,t.startColumn)},e.fromPositions=function(t,n){return void 0===n&&(n=t),new e(t.lineNumber,t.column,n.lineNumber,n.column)},e.lift=function(t){return t?new e(t.startLineNumber,t.startColumn,t.endLineNumber,t.endColumn):null},e.isIRange=function(e){return e&&\"number\"==typeof e.startLineNumber&&\"number\"==typeof e.startColumn&&\"number\"==typeof e.endLineNumber&&\"number\"==typeof e.endColumn},e.areIntersectingOrTouching=function(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<t.startColumn)&&!(t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<e.startColumn)},e.areIntersecting=function(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<=t.startColumn)&&!(t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<=e.startColumn)},e.compareRangesUsingStarts=function(e,t){if(e&&t){var n=0|e.startLineNumber,r=0|t.startLineNumber;if(n===r){var i=0|e.startColumn,o=0|t.startColumn;if(i===o){var a=0|e.endLineNumber,s=0|t.endLineNumber;return a===s?(0|e.endColumn)-(0|t.endColumn):a-s}return i-o}return n-r}return(e?1:0)-(t?1:0)},e.compareRangesUsingEnds=function(e,t){return e.endLineNumber===t.endLineNumber?e.endColumn===t.endColumn?e.startLineNumber===t.startLineNumber?e.startColumn-t.startColumn:e.startLineNumber-t.startLineNumber:e.endColumn-t.endColumn:e.endLineNumber-t.endLineNumber},e.spansMultipleLines=function(e){return e.endLineNumber>e.startLineNumber},e}();var ue=5e3,le=3;function ce(e,t,n,r){return new H(e,t,n).ComputeDiff(r)}var he=function(){function e(t){for(var n=[],r=[],i=0,o=t.length;i<o;i++)n[i]=e._getFirstNonBlankColumn(t[i],1),r[i]=e._getLastNonBlankColumn(t[i],1);this._lines=t,this._startColumns=n,this._endColumns=r}return e.prototype.getLength=function(){return this._lines.length},e.prototype.getElementAtIndex=function(e){return this._lines[e].substring(this._startColumns[e]-1,this._endColumns[e]-1)},e.prototype.getStartLineNumber=function(e){return e+1},e.prototype.getEndLineNumber=function(e){return e+1},e._getFirstNonBlankColumn=function(e,t){var n=function(e){for(var t=0,n=e.length;t<n;t++){var r=e.charCodeAt(t);if(32!==r&&9!==r)return t}return-1}(e);return-1===n?t:n+1},e._getLastNonBlankColumn=function(e,t){var n=function(e,t){void 0===t&&(t=e.length-1);for(var n=t;n>=0;n--){var r=e.charCodeAt(n);if(32!==r&&9!==r)return n}return-1}(e);return-1===n?t:n+2},e.prototype.getCharSequence=function(e,t,n){for(var r=[],i=[],o=[],a=0,s=t;s<=n;s++)for(var u=this._lines[s],l=e?this._startColumns[s]:1,c=e?this._endColumns[s]:u.length+1,h=l;h<c;h++)r[a]=u.charCodeAt(h-1),i[a]=s+1,o[a]=h,a++;return new de(r,i,o)},e}(),de=function(){function e(e,t,n){this._charCodes=e,this._lineNumbers=t,this._columns=n}return e.prototype.getLength=function(){return this._charCodes.length},e.prototype.getElementAtIndex=function(e){return this._charCodes[e]},e.prototype.getStartLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getStartColumn=function(e){return this._columns[e]},e.prototype.getEndLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getEndColumn=function(e){return this._columns[e]+1},e}(),fe=function(){function e(e,t,n,r,i,o,a,s){this.originalStartLineNumber=e,this.originalStartColumn=t,this.originalEndLineNumber=n,this.originalEndColumn=r,this.modifiedStartLineNumber=i,this.modifiedStartColumn=o,this.modifiedEndLineNumber=a,this.modifiedEndColumn=s}return e.createFromDiffChange=function(t,n,r){var i,o,a,s,u,l,c,h;return 0===t.originalLength?(i=0,o=0,a=0,s=0):(i=n.getStartLineNumber(t.originalStart),o=n.getStartColumn(t.originalStart),a=n.getEndLineNumber(t.originalStart+t.originalLength-1),s=n.getEndColumn(t.originalStart+t.originalLength-1)),0===t.modifiedLength?(u=0,l=0,c=0,h=0):(u=r.getStartLineNumber(t.modifiedStart),l=r.getStartColumn(t.modifiedStart),c=r.getEndLineNumber(t.modifiedStart+t.modifiedLength-1),h=r.getEndColumn(t.modifiedStart+t.modifiedLength-1)),new e(i,o,a,s,u,l,c,h)},e}();var pe=function(){function e(e,t,n,r,i){this.originalStartLineNumber=e,this.originalEndLineNumber=t,this.modifiedStartLineNumber=n,this.modifiedEndLineNumber=r,this.charChanges=i}return e.createFromDiffResult=function(t,n,r,i,o,a,s){var u,l,c,h,d=void 0;if(0===n.originalLength?(u=r.getStartLineNumber(n.originalStart)-1,l=0):(u=r.getStartLineNumber(n.originalStart),l=r.getEndLineNumber(n.originalStart+n.originalLength-1)),0===n.modifiedLength?(c=i.getStartLineNumber(n.modifiedStart)-1,h=0):(c=i.getStartLineNumber(n.modifiedStart),h=i.getEndLineNumber(n.modifiedStart+n.modifiedLength-1)),a&&0!==n.originalLength&&0!==n.modifiedLength&&o()){var f=r.getCharSequence(t,n.originalStart,n.originalStart+n.originalLength-1),p=i.getCharSequence(t,n.modifiedStart,n.modifiedStart+n.modifiedLength-1),g=ce(f,p,o,!0);s&&(g=function(e){if(e.length<=1)return e;for(var t=[e[0]],n=t[0],r=1,i=e.length;r<i;r++){var o=e[r],a=o.originalStart-(n.originalStart+n.originalLength),s=o.modifiedStart-(n.modifiedStart+n.modifiedLength);Math.min(a,s)<le?(n.originalLength=o.originalStart+o.originalLength-n.originalStart,n.modifiedLength=o.modifiedStart+o.modifiedLength-n.modifiedStart):(t.push(o),n=o)}return t}(g)),d=[];for(var m=0,b=g.length;m<b;m++)d.push(fe.createFromDiffChange(g[m],f,p))}return new e(u,l,c,h,d)},e}(),ge=function(){function e(e,t,n){this.shouldComputeCharChanges=n.shouldComputeCharChanges,this.shouldPostProcessCharChanges=n.shouldPostProcessCharChanges,this.shouldIgnoreTrimWhitespace=n.shouldIgnoreTrimWhitespace,this.shouldMakePrettyDiff=n.shouldMakePrettyDiff,this.maximumRunTimeMs=ue,this.originalLines=e,this.modifiedLines=t,this.original=new he(e),this.modified=new he(t)}return e.prototype.computeDiff=function(){if(1===this.original.getLength()&&0===this.original.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:1,modifiedStartLineNumber:1,modifiedEndLineNumber:this.modified.getLength(),charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];if(1===this.modified.getLength()&&0===this.modified.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:this.original.getLength(),modifiedStartLineNumber:1,modifiedEndLineNumber:1,charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];this.computationStartTime=(new Date).getTime();var e=ce(this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldMakePrettyDiff);if(this.shouldIgnoreTrimWhitespace){for(var t=[],n=0,r=e.length;n<r;n++)t.push(pe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,e[n],this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges));return t}for(var i=[],o=0,a=0,s=(n=-1,e.length);n<s;n++){for(var u=n+1<s?e[n+1]:null,l=u?u.originalStart:this.originalLines.length,c=u?u.modifiedStart:this.modifiedLines.length;o<l&&a<c;){var h=this.originalLines[o],d=this.modifiedLines[a];if(h!==d){for(var f=he._getFirstNonBlankColumn(h,1),p=he._getFirstNonBlankColumn(d,1);f>1&&p>1;){if(h.charCodeAt(f-2)!==d.charCodeAt(p-2))break;f--,p--}(f>1||p>1)&&this._pushTrimWhitespaceCharChange(i,o+1,1,f,a+1,1,p);for(var g=he._getLastNonBlankColumn(h,1),m=he._getLastNonBlankColumn(d,1),b=h.length+1,_=d.length+1;g<b&&m<_;){if(h.charCodeAt(g-1)!==h.charCodeAt(m-1))break;g++,m++}(g<b||m<_)&&this._pushTrimWhitespaceCharChange(i,o+1,g,b,a+1,m,_)}o++,a++}u&&(i.push(pe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,u,this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges)),o+=u.originalLength,a+=u.modifiedLength)}return i},e.prototype._pushTrimWhitespaceCharChange=function(e,t,n,r,i,o,a){if(!this._mergeTrimWhitespaceCharChange(e,t,n,r,i,o,a)){var s=void 0;this.shouldComputeCharChanges&&(s=[new fe(t,n,t,r,i,o,i,a)]),e.push(new pe(t,t,i,i,s))}},e.prototype._mergeTrimWhitespaceCharChange=function(e,t,n,r,i,o,a){var s=e.length;if(0===s)return!1;var u=e[s-1];return 0!==u.originalEndLineNumber&&0!==u.modifiedEndLineNumber&&(u.originalEndLineNumber+1===t&&u.modifiedEndLineNumber+1===i&&(u.originalEndLineNumber=t,u.modifiedEndLineNumber=i,this.shouldComputeCharChanges&&u.charChanges.push(new fe(t,n,t,r,i,o,i,a)),!0))},e.prototype._continueProcessingPredicate=function(){return 0===this.maximumRunTimeMs||(new Date).getTime()-this.computationStartTime<this.maximumRunTimeMs},e}(),me=function(){function e(e,t,n){for(var r=new Uint8Array(e*t),i=0,o=e*t;i<o;i++)r[i]=n;this._data=r,this.rows=e,this.cols=t}return e.prototype.get=function(e,t){return this._data[e*this.cols+t]},e.prototype.set=function(e,t,n){this._data[e*this.cols+t]=n},e}();function be(e){return e<0?0:e>255?255:0|e}function _e(e){return e<0?0:e>4294967295?4294967295:0|e}var ve=function(){return function(e,t){this.index=e,this.remainder=t}}(),ye=function(){function e(e){this.values=e,this.prefixSum=new Uint32Array(e.length),this.prefixSumValidIndex=new Int32Array(1),this.prefixSumValidIndex[0]=-1}return e.prototype.getCount=function(){return this.values.length},e.prototype.insertValues=function(e,t){e=_e(e);var n=this.values,r=this.prefixSum,i=t.length;return 0!==i&&(this.values=new Uint32Array(n.length+i),this.values.set(n.subarray(0,e),0),this.values.set(n.subarray(e),e+i),this.values.set(t,e),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSum=new Uint32Array(this.values.length),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.changeValue=function(e,t){return e=_e(e),t=_e(t),this.values[e]!==t&&(this.values[e]=t,e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),!0)},e.prototype.removeValues=function(e,t){e=_e(e),t=_e(t);var n=this.values,r=this.prefixSum;if(e>=n.length)return!1;var i=n.length-e;return t>=i&&(t=i),0!==t&&(this.values=new Uint32Array(n.length-t),this.values.set(n.subarray(0,e),0),this.values.set(n.subarray(e+t),e),this.prefixSum=new Uint32Array(this.values.length),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.getTotalValue=function(){return 0===this.values.length?0:this._getAccumulatedValue(this.values.length-1)},e.prototype.getAccumulatedValue=function(e){return e<0?0:(e=_e(e),this._getAccumulatedValue(e))},e.prototype._getAccumulatedValue=function(e){if(e<=this.prefixSumValidIndex[0])return this.prefixSum[e];var t=this.prefixSumValidIndex[0]+1;0===t&&(this.prefixSum[0]=this.values[0],t++),e>=this.values.length&&(e=this.values.length-1);for(var n=t;n<=e;n++)this.prefixSum[n]=this.prefixSum[n-1]+this.values[n];return this.prefixSumValidIndex[0]=Math.max(this.prefixSumValidIndex[0],e),this.prefixSum[e]},e.prototype.getIndexOf=function(e){e=Math.floor(e),this.getTotalValue();for(var t=0,n=this.values.length-1,r=0,i=0,o=0;t<=n;)if(r=t+(n-t)/2|0,e<(o=(i=this.prefixSum[r])-this.values[r]))n=r-1;else{if(!(e>=i))break;t=r+1}return new ve(r,e-o)},e}(),we=function(){function e(e,t,n,r){this._uri=e,this._lines=t,this._eol=n,this._versionId=r,this._lineStarts=null}return e.prototype.dispose=function(){this._lines.length=0},e.prototype.getText=function(){return this._lines.join(this._eol)},e.prototype.onEvents=function(e){e.eol&&e.eol!==this._eol&&(this._eol=e.eol,this._lineStarts=null);for(var t=e.changes,n=0,r=t.length;n<r;n++){var i=t[n];this._acceptDeleteRange(i.range),this._acceptInsertText(new ae(i.range.startLineNumber,i.range.startColumn),i.text)}this._versionId=e.versionId},e.prototype._ensureLineStarts=function(){if(!this._lineStarts){for(var e=this._eol.length,t=this._lines.length,n=new Uint32Array(t),r=0;r<t;r++)n[r]=this._lines[r].length+e;this._lineStarts=new ye(n)}},e.prototype._setLineText=function(e,t){this._lines[e]=t,this._lineStarts&&this._lineStarts.changeValue(e,this._lines[e].length+this._eol.length)},e.prototype._acceptDeleteRange=function(e){if(e.startLineNumber!==e.endLineNumber)this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.endLineNumber-1].substring(e.endColumn-1)),this._lines.splice(e.startLineNumber,e.endLineNumber-e.startLineNumber),this._lineStarts&&this._lineStarts.removeValues(e.startLineNumber,e.endLineNumber-e.startLineNumber);else{if(e.startColumn===e.endColumn)return;this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.startLineNumber-1].substring(e.endColumn-1))}},e.prototype._acceptInsertText=function(e,t){if(0!==t.length){var n=t.split(/\\r\\n|\\r|\\n/);if(1!==n.length){n[n.length-1]+=this._lines[e.lineNumber-1].substring(e.column-1),this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+n[0]);for(var r=new Uint32Array(n.length-1),i=1;i<n.length;i++)this._lines.splice(e.lineNumber+i-1,0,n[i]),r[i-1]=n[i].length+this._eol.length;this._lineStarts&&this._lineStarts.insertValues(e.lineNumber,r)}else this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+n[0]+this._lines[e.lineNumber-1].substring(e.column-1))}},e}(),Ce=\"`~!@#$%^&*()-=+[{]}\\\\|;:'\\\",.<>/?\";var Te=function(e){void 0===e&&(e=\"\");for(var t=\"(-?\\\\d*\\\\.\\\\d\\\\w*)|([^\",n=0;n<Ce.length;n++)e.indexOf(Ce[n])>=0||(t+=\"\\\\\"+Ce[n]);return t+=\"\\\\s]+)\",new RegExp(t,\"g\")}();var Ee=function(){function e(t){var n=be(t);this._defaultValue=n,this._asciiMap=e._createAsciiMap(n),this._map=new Map}return e._createAsciiMap=function(e){for(var t=new Uint8Array(256),n=0;n<256;n++)t[n]=e;return t},e.prototype.set=function(e,t){var n=be(t);e>=0&&e<256?this._asciiMap[e]=n:this._map.set(e,n)},e.prototype.get=function(e){return e>=0&&e<256?this._asciiMap[e]:this._map.get(e)||this._defaultValue},e}(),xe=function(){function e(e){for(var t=0,n=0,r=0,i=e.length;r<i;r++){var o=e[r],a=o[0];(l=o[1])>t&&(t=l),a>n&&(n=a),(c=o[2])>n&&(n=c)}var s=new me(++n,++t,0);for(r=0,i=e.length;r<i;r++){var u=e[r],l=(a=u[0],u[1]),c=u[2];s.set(a,l,c)}this._states=s,this._maxCharCode=t}return e.prototype.nextState=function(e,t){return t<0||t>=this._maxCharCode?0:this._states.get(e,t)},e}(),Se=null;var ke=null;var Ae=function(){function e(){}return e._createLink=function(e,t,n,r,i){var o=i-1;do{var a=t.charCodeAt(o);if(2!==e.get(a))break;o--}while(o>r);if(r>0){var s=t.charCodeAt(r-1),u=t.charCodeAt(o);(40===s&&41===u||91===s&&93===u||123===s&&125===u)&&o--}return{range:{startLineNumber:n,startColumn:r+1,endLineNumber:n,endColumn:o+2},url:t.substring(r,o+1)}},e.computeLinks=function(t){for(var n=(null===Se&&(Se=new xe([[1,104,2],[1,72,2],[1,102,6],[1,70,6],[2,116,3],[2,84,3],[3,116,4],[3,84,4],[4,112,5],[4,80,5],[5,115,9],[5,83,9],[5,58,10],[6,105,7],[6,73,7],[7,108,8],[7,76,8],[8,101,9],[8,69,9],[9,58,10],[10,47,11],[11,47,12]])),Se),r=function(){if(null===ke){ke=new Ee(0);for(var e=0;e<\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".length;e++)ke.set(\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".charCodeAt(e),1);for(e=0;e<\".,;\".length;e++)ke.set(\".,;\".charCodeAt(e),2)}return ke}(),i=[],o=1,a=t.getLineCount();o<=a;o++){for(var s=t.getLineContent(o),u=s.length,l=0,c=0,h=0,d=1,f=!1,p=!1,g=!1;l<u;){var m=!1,b=s.charCodeAt(l);if(13===d){var _=void 0;switch(b){case 40:f=!0,_=0;break;case 41:_=f?0:1;break;case 91:p=!0,_=0;break;case 93:_=p?0:1;break;case 123:g=!0,_=0;break;case 125:_=g?0:1;break;case 39:_=34===h||96===h?0:1;break;case 34:_=39===h||96===h?0:1;break;case 96:_=39===h||34===h?0:1;break;default:_=r.get(b)}1===_&&(i.push(e._createLink(r,s,o,c,l)),m=!0)}else if(12===d){1===(_=r.get(b))?m=!0:d=13}else 0===(d=n.nextState(d,b))&&(m=!0);m&&(d=1,f=!1,p=!1,g=!1,c=l+1,h=b),l++}13===d&&i.push(e._createLink(r,s,o,c,u))}return i},e}();var Le,Ne=function(){function e(){this._defaultValueSet=[[\"true\",\"false\"],[\"True\",\"False\"],[\"Private\",\"Public\",\"Friend\",\"ReadOnly\",\"Partial\",\"Protected\",\"WriteOnly\"],[\"public\",\"protected\",\"private\"]]}return e.prototype.navigateValueSet=function(e,t,n,r,i){var o;if(e&&t&&(o=this.doNavigateValueSet(t,i)))return{range:e,value:o};if(n&&r&&(o=this.doNavigateValueSet(r,i)))return{range:n,value:o};return null},e.prototype.doNavigateValueSet=function(e,t){var n=this.numberReplace(e,t);return null!==n?n:this.textReplace(e,t)},e.prototype.numberReplace=function(e,t){var n=Math.pow(10,e.length-(e.lastIndexOf(\".\")+1)),r=Number(e),i=parseFloat(e);return isNaN(r)||isNaN(i)||r!==i?null:0!==r||t?(r=Math.floor(r*n),r+=t?n:-n,String(r/n)):null},e.prototype.textReplace=function(e,t){return this.valueSetsReplace(this._defaultValueSet,e,t)},e.prototype.valueSetsReplace=function(e,t,n){for(var r=null,i=0,o=e.length;null===r&&i<o;i++)r=this.valueSetReplace(e[i],t,n);return r},e.prototype.valueSetReplace=function(e,t,n){var r=e.indexOf(t);return r>=0?((r+=n?1:-1)<0?r=e.length-1:r%=e.length,e[r]):null},e.INSTANCE=new e,e}(),Oe=function(){function e(){this._keyCodeToStr=[],this._strToKeyCode=Object.create(null)}return e.prototype.define=function(e,t){this._keyCodeToStr[e]=t,this._strToKeyCode[t.toLowerCase()]=e},e.prototype.keyCodeToStr=function(e){return this._keyCodeToStr[e]},e.prototype.strToKeyCode=function(e){return this._strToKeyCode[e.toLowerCase()]||0},e}(),Pe=new Oe,Ie=new Oe,qe=new Oe;!function(){function e(e,t,n,r){void 0===n&&(n=t),void 0===r&&(r=n),Pe.define(e,t),Ie.define(e,n),qe.define(e,r)}e(0,\"unknown\"),e(1,\"Backspace\"),e(2,\"Tab\"),e(3,\"Enter\"),e(4,\"Shift\"),e(5,\"Ctrl\"),e(6,\"Alt\"),e(7,\"PauseBreak\"),e(8,\"CapsLock\"),e(9,\"Escape\"),e(10,\"Space\"),e(11,\"PageUp\"),e(12,\"PageDown\"),e(13,\"End\"),e(14,\"Home\"),e(15,\"LeftArrow\",\"Left\"),e(16,\"UpArrow\",\"Up\"),e(17,\"RightArrow\",\"Right\"),e(18,\"DownArrow\",\"Down\"),e(19,\"Insert\"),e(20,\"Delete\"),e(21,\"0\"),e(22,\"1\"),e(23,\"2\"),e(24,\"3\"),e(25,\"4\"),e(26,\"5\"),e(27,\"6\"),e(28,\"7\"),e(29,\"8\"),e(30,\"9\"),e(31,\"A\"),e(32,\"B\"),e(33,\"C\"),e(34,\"D\"),e(35,\"E\"),e(36,\"F\"),e(37,\"G\"),e(38,\"H\"),e(39,\"I\"),e(40,\"J\"),e(41,\"K\"),e(42,\"L\"),e(43,\"M\"),e(44,\"N\"),e(45,\"O\"),e(46,\"P\"),e(47,\"Q\"),e(48,\"R\"),e(49,\"S\"),e(50,\"T\"),e(51,\"U\"),e(52,\"V\"),e(53,\"W\"),e(54,\"X\"),e(55,\"Y\"),e(56,\"Z\"),e(57,\"Meta\"),e(58,\"ContextMenu\"),e(59,\"F1\"),e(60,\"F2\"),e(61,\"F3\"),e(62,\"F4\"),e(63,\"F5\"),e(64,\"F6\"),e(65,\"F7\"),e(66,\"F8\"),e(67,\"F9\"),e(68,\"F10\"),e(69,\"F11\"),e(70,\"F12\"),e(71,\"F13\"),e(72,\"F14\"),e(73,\"F15\"),e(74,\"F16\"),e(75,\"F17\"),e(76,\"F18\"),e(77,\"F19\"),e(78,\"NumLock\"),e(79,\"ScrollLock\"),e(80,\";\",\";\",\"OEM_1\"),e(81,\"=\",\"=\",\"OEM_PLUS\"),e(82,\",\",\",\",\"OEM_COMMA\"),e(83,\"-\",\"-\",\"OEM_MINUS\"),e(84,\".\",\".\",\"OEM_PERIOD\"),e(85,\"/\",\"/\",\"OEM_2\"),e(86,\"`\",\"`\",\"OEM_3\"),e(110,\"ABNT_C1\"),e(111,\"ABNT_C2\"),e(87,\"[\",\"[\",\"OEM_4\"),e(88,\"\\\\\",\"\\\\\",\"OEM_5\"),e(89,\"]\",\"]\",\"OEM_6\"),e(90,\"'\",\"'\",\"OEM_7\"),e(91,\"OEM_8\"),e(92,\"OEM_102\"),e(93,\"NumPad0\"),e(94,\"NumPad1\"),e(95,\"NumPad2\"),e(96,\"NumPad3\"),e(97,\"NumPad4\"),e(98,\"NumPad5\"),e(99,\"NumPad6\"),e(100,\"NumPad7\"),e(101,\"NumPad8\"),e(102,\"NumPad9\"),e(103,\"NumPad_Multiply\"),e(104,\"NumPad_Add\"),e(105,\"NumPad_Separator\"),e(106,\"NumPad_Subtract\"),e(107,\"NumPad_Decimal\"),e(108,\"NumPad_Divide\")}(),function(e){e.toString=function(e){return Pe.keyCodeToStr(e)},e.fromString=function(e){return Pe.strToKeyCode(e)},e.toUserSettingsUS=function(e){return Ie.keyCodeToStr(e)},e.toUserSettingsGeneral=function(e){return qe.keyCodeToStr(e)},e.fromUserSettings=function(e){return Ie.strToKeyCode(e)||qe.strToKeyCode(e)}}(Le||(Le={}));var Re,De,Ue,Me,We,Ve,Be,je,Fe,Ke,He,Ge,Ye,ze,Je,$e,Qe,Ze,Xe,et,tt,nt,rt,it,ot,at,st,ut=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),lt=function(e){function t(t,n,r,i){var o=e.call(this,t,n,r,i)||this;return o.selectionStartLineNumber=t,o.selectionStartColumn=n,o.positionLineNumber=r,o.positionColumn=i,o}return ut(t,e),t.prototype.clone=function(){return new t(this.selectionStartLineNumber,this.selectionStartColumn,this.positionLineNumber,this.positionColumn)},t.prototype.toString=function(){return\"[\"+this.selectionStartLineNumber+\",\"+this.selectionStartColumn+\" -> \"+this.positionLineNumber+\",\"+this.positionColumn+\"]\"},t.prototype.equalsSelection=function(e){return t.selectionsEqual(this,e)},t.selectionsEqual=function(e,t){return e.selectionStartLineNumber===t.selectionStartLineNumber&&e.selectionStartColumn===t.selectionStartColumn&&e.positionLineNumber===t.positionLineNumber&&e.positionColumn===t.positionColumn},t.prototype.getDirection=function(){return this.selectionStartLineNumber===this.startLineNumber&&this.selectionStartColumn===this.startColumn?0:1},t.prototype.setEndPosition=function(e,n){return 0===this.getDirection()?new t(this.startLineNumber,this.startColumn,e,n):new t(e,n,this.startLineNumber,this.startColumn)},t.prototype.getPosition=function(){return new ae(this.positionLineNumber,this.positionColumn)},t.prototype.setStartPosition=function(e,n){return 0===this.getDirection()?new t(e,n,this.endLineNumber,this.endColumn):new t(this.endLineNumber,this.endColumn,e,n)},t.fromPositions=function(e,n){return void 0===n&&(n=e),new t(e.lineNumber,e.column,n.lineNumber,n.column)},t.liftSelection=function(e){return new t(e.selectionStartLineNumber,e.selectionStartColumn,e.positionLineNumber,e.positionColumn)},t.selectionsArrEqual=function(e,t){if(e&&!t||!e&&t)return!1;if(!e&&!t)return!0;if(e.length!==t.length)return!1;for(var n=0,r=e.length;n<r;n++)if(!this.selectionsEqual(e[n],t[n]))return!1;return!0},t.isISelection=function(e){return e&&\"number\"==typeof e.selectionStartLineNumber&&\"number\"==typeof e.selectionStartColumn&&\"number\"==typeof e.positionLineNumber&&\"number\"==typeof e.positionColumn},t.createWithDirection=function(e,n,r,i,o){return 0===o?new t(e,n,r,i):new t(r,i,e,n)},t}(se),ct=function(){function e(e,t,n){this.offset=0|e,this.type=t,this.language=n}return e.prototype.toString=function(){return\"(\"+this.offset+\", \"+this.type+\")\"},e}();!function(e){e[e.Unnecessary=1]=\"Unnecessary\"}(Re||(Re={})),function(e){e[e.Hint=1]=\"Hint\",e[e.Info=2]=\"Info\",e[e.Warning=4]=\"Warning\",e[e.Error=8]=\"Error\"}(De||(De={})),function(e){e[e.Unknown=0]=\"Unknown\",e[e.Backspace=1]=\"Backspace\",e[e.Tab=2]=\"Tab\",e[e.Enter=3]=\"Enter\",e[e.Shift=4]=\"Shift\",e[e.Ctrl=5]=\"Ctrl\",e[e.Alt=6]=\"Alt\",e[e.PauseBreak=7]=\"PauseBreak\",e[e.CapsLock=8]=\"CapsLock\",e[e.Escape=9]=\"Escape\",e[e.Space=10]=\"Space\",e[e.PageUp=11]=\"PageUp\",e[e.PageDown=12]=\"PageDown\",e[e.End=13]=\"End\",e[e.Home=14]=\"Home\",e[e.LeftArrow=15]=\"LeftArrow\",e[e.UpArrow=16]=\"UpArrow\",e[e.RightArrow=17]=\"RightArrow\",e[e.DownArrow=18]=\"DownArrow\",e[e.Insert=19]=\"Insert\",e[e.Delete=20]=\"Delete\",e[e.KEY_0=21]=\"KEY_0\",e[e.KEY_1=22]=\"KEY_1\",e[e.KEY_2=23]=\"KEY_2\",e[e.KEY_3=24]=\"KEY_3\",e[e.KEY_4=25]=\"KEY_4\",e[e.KEY_5=26]=\"KEY_5\",e[e.KEY_6=27]=\"KEY_6\",e[e.KEY_7=28]=\"KEY_7\",e[e.KEY_8=29]=\"KEY_8\",e[e.KEY_9=30]=\"KEY_9\",e[e.KEY_A=31]=\"KEY_A\",e[e.KEY_B=32]=\"KEY_B\",e[e.KEY_C=33]=\"KEY_C\",e[e.KEY_D=34]=\"KEY_D\",e[e.KEY_E=35]=\"KEY_E\",e[e.KEY_F=36]=\"KEY_F\",e[e.KEY_G=37]=\"KEY_G\",e[e.KEY_H=38]=\"KEY_H\",e[e.KEY_I=39]=\"KEY_I\",e[e.KEY_J=40]=\"KEY_J\",e[e.KEY_K=41]=\"KEY_K\",e[e.KEY_L=42]=\"KEY_L\",e[e.KEY_M=43]=\"KEY_M\",e[e.KEY_N=44]=\"KEY_N\",e[e.KEY_O=45]=\"KEY_O\",e[e.KEY_P=46]=\"KEY_P\",e[e.KEY_Q=47]=\"KEY_Q\",e[e.KEY_R=48]=\"KEY_R\",e[e.KEY_S=49]=\"KEY_S\",e[e.KEY_T=50]=\"KEY_T\",e[e.KEY_U=51]=\"KEY_U\",e[e.KEY_V=52]=\"KEY_V\",e[e.KEY_W=53]=\"KEY_W\",e[e.KEY_X=54]=\"KEY_X\",e[e.KEY_Y=55]=\"KEY_Y\",e[e.KEY_Z=56]=\"KEY_Z\",e[e.Meta=57]=\"Meta\",e[e.ContextMenu=58]=\"ContextMenu\",e[e.F1=59]=\"F1\",e[e.F2=60]=\"F2\",e[e.F3=61]=\"F3\",e[e.F4=62]=\"F4\",e[e.F5=63]=\"F5\",e[e.F6=64]=\"F6\",e[e.F7=65]=\"F7\",e[e.F8=66]=\"F8\",e[e.F9=67]=\"F9\",e[e.F10=68]=\"F10\",e[e.F11=69]=\"F11\",e[e.F12=70]=\"F12\",e[e.F13=71]=\"F13\",e[e.F14=72]=\"F14\",e[e.F15=73]=\"F15\",e[e.F16=74]=\"F16\",e[e.F17=75]=\"F17\",e[e.F18=76]=\"F18\",e[e.F19=77]=\"F19\",e[e.NumLock=78]=\"NumLock\",e[e.ScrollLock=79]=\"ScrollLock\",e[e.US_SEMICOLON=80]=\"US_SEMICOLON\",e[e.US_EQUAL=81]=\"US_EQUAL\",e[e.US_COMMA=82]=\"US_COMMA\",e[e.US_MINUS=83]=\"US_MINUS\",e[e.US_DOT=84]=\"US_DOT\",e[e.US_SLASH=85]=\"US_SLASH\",e[e.US_BACKTICK=86]=\"US_BACKTICK\",e[e.US_OPEN_SQUARE_BRACKET=87]=\"US_OPEN_SQUARE_BRACKET\",e[e.US_BACKSLASH=88]=\"US_BACKSLASH\",e[e.US_CLOSE_SQUARE_BRACKET=89]=\"US_CLOSE_SQUARE_BRACKET\",e[e.US_QUOTE=90]=\"US_QUOTE\",e[e.OEM_8=91]=\"OEM_8\",e[e.OEM_102=92]=\"OEM_102\",e[e.NUMPAD_0=93]=\"NUMPAD_0\",e[e.NUMPAD_1=94]=\"NUMPAD_1\",e[e.NUMPAD_2=95]=\"NUMPAD_2\",e[e.NUMPAD_3=96]=\"NUMPAD_3\",e[e.NUMPAD_4=97]=\"NUMPAD_4\",e[e.NUMPAD_5=98]=\"NUMPAD_5\",e[e.NUMPAD_6=99]=\"NUMPAD_6\",e[e.NUMPAD_7=100]=\"NUMPAD_7\",e[e.NUMPAD_8=101]=\"NUMPAD_8\",e[e.NUMPAD_9=102]=\"NUMPAD_9\",e[e.NUMPAD_MULTIPLY=103]=\"NUMPAD_MULTIPLY\",e[e.NUMPAD_ADD=104]=\"NUMPAD_ADD\",e[e.NUMPAD_SEPARATOR=105]=\"NUMPAD_SEPARATOR\",e[e.NUMPAD_SUBTRACT=106]=\"NUMPAD_SUBTRACT\",e[e.NUMPAD_DECIMAL=107]=\"NUMPAD_DECIMAL\",e[e.NUMPAD_DIVIDE=108]=\"NUMPAD_DIVIDE\",e[e.KEY_IN_COMPOSITION=109]=\"KEY_IN_COMPOSITION\",e[e.ABNT_C1=110]=\"ABNT_C1\",e[e.ABNT_C2=111]=\"ABNT_C2\",e[e.MAX_VALUE=112]=\"MAX_VALUE\"}(Ue||(Ue={})),function(e){e[e.LTR=0]=\"LTR\",e[e.RTL=1]=\"RTL\"}(Me||(Me={})),function(e){e[e.Auto=1]=\"Auto\",e[e.Hidden=2]=\"Hidden\",e[e.Visible=3]=\"Visible\"}(We||(We={})),function(e){e[e.Left=1]=\"Left\",e[e.Center=2]=\"Center\",e[e.Right=4]=\"Right\",e[e.Full=7]=\"Full\"}(Ve||(Ve={})),function(e){e[e.TextDefined=0]=\"TextDefined\",e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(Be||(Be={})),function(e){e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(je||(je={})),function(e){e[e.LF=0]=\"LF\",e[e.CRLF=1]=\"CRLF\"}(Fe||(Fe={})),function(e){e[e.AlwaysGrowsWhenTypingAtEdges=0]=\"AlwaysGrowsWhenTypingAtEdges\",e[e.NeverGrowsWhenTypingAtEdges=1]=\"NeverGrowsWhenTypingAtEdges\",e[e.GrowsOnlyWhenTypingBefore=2]=\"GrowsOnlyWhenTypingBefore\",e[e.GrowsOnlyWhenTypingAfter=3]=\"GrowsOnlyWhenTypingAfter\"}(Ke||(Ke={})),function(e){e[e.Smooth=0]=\"Smooth\",e[e.Immediate=1]=\"Immediate\"}(He||(He={})),function(e){e[e.NotSet=0]=\"NotSet\",e[e.ContentFlush=1]=\"ContentFlush\",e[e.RecoverFromMarkers=2]=\"RecoverFromMarkers\",e[e.Explicit=3]=\"Explicit\",e[e.Paste=4]=\"Paste\",e[e.Undo=5]=\"Undo\",e[e.Redo=6]=\"Redo\"}(Ge||(Ge={})),function(e){e[e.None=0]=\"None\",e[e.Small=1]=\"Small\",e[e.Large=2]=\"Large\",e[e.SmallBlocks=3]=\"SmallBlocks\",e[e.LargeBlocks=4]=\"LargeBlocks\"}(Ye||(Ye={})),function(e){e[e.None=0]=\"None\",e[e.Same=1]=\"Same\",e[e.Indent=2]=\"Indent\",e[e.DeepIndent=3]=\"DeepIndent\"}(ze||(ze={})),function(e){e[e.Hidden=0]=\"Hidden\",e[e.Blink=1]=\"Blink\",e[e.Smooth=2]=\"Smooth\",e[e.Phase=3]=\"Phase\",e[e.Expand=4]=\"Expand\",e[e.Solid=5]=\"Solid\"}(Je||(Je={})),function(e){e[e.Line=1]=\"Line\",e[e.Block=2]=\"Block\",e[e.Underline=3]=\"Underline\",e[e.LineThin=4]=\"LineThin\",e[e.BlockOutline=5]=\"BlockOutline\",e[e.UnderlineThin=6]=\"UnderlineThin\"}($e||($e={})),function(e){e[e.Off=0]=\"Off\",e[e.On=1]=\"On\",e[e.Relative=2]=\"Relative\",e[e.Interval=3]=\"Interval\",e[e.Custom=4]=\"Custom\"}(Qe||(Qe={})),function(e){e[e.EXACT=0]=\"EXACT\",e[e.ABOVE=1]=\"ABOVE\",e[e.BELOW=2]=\"BELOW\"}(Ze||(Ze={})),function(e){e[e.TOP_RIGHT_CORNER=0]=\"TOP_RIGHT_CORNER\",e[e.BOTTOM_RIGHT_CORNER=1]=\"BOTTOM_RIGHT_CORNER\",e[e.TOP_CENTER=2]=\"TOP_CENTER\"}(Xe||(Xe={})),function(e){e[e.UNKNOWN=0]=\"UNKNOWN\",e[e.TEXTAREA=1]=\"TEXTAREA\",e[e.GUTTER_GLYPH_MARGIN=2]=\"GUTTER_GLYPH_MARGIN\",e[e.GUTTER_LINE_NUMBERS=3]=\"GUTTER_LINE_NUMBERS\",e[e.GUTTER_LINE_DECORATIONS=4]=\"GUTTER_LINE_DECORATIONS\",e[e.GUTTER_VIEW_ZONE=5]=\"GUTTER_VIEW_ZONE\",e[e.CONTENT_TEXT=6]=\"CONTENT_TEXT\",e[e.CONTENT_EMPTY=7]=\"CONTENT_EMPTY\",e[e.CONTENT_VIEW_ZONE=8]=\"CONTENT_VIEW_ZONE\",e[e.CONTENT_WIDGET=9]=\"CONTENT_WIDGET\",e[e.OVERVIEW_RULER=10]=\"OVERVIEW_RULER\",e[e.SCROLLBAR=11]=\"SCROLLBAR\",e[e.OVERLAY_WIDGET=12]=\"OVERLAY_WIDGET\",e[e.OUTSIDE_EDITOR=13]=\"OUTSIDE_EDITOR\"}(et||(et={})),function(e){e[e.None=0]=\"None\",e[e.Indent=1]=\"Indent\",e[e.IndentOutdent=2]=\"IndentOutdent\",e[e.Outdent=3]=\"Outdent\"}(tt||(tt={})),function(e){e[e.Method=0]=\"Method\",e[e.Function=1]=\"Function\",e[e.Constructor=2]=\"Constructor\",e[e.Field=3]=\"Field\",e[e.Variable=4]=\"Variable\",e[e.Class=5]=\"Class\",e[e.Struct=6]=\"Struct\",e[e.Interface=7]=\"Interface\",e[e.Module=8]=\"Module\",e[e.Property=9]=\"Property\",e[e.Event=10]=\"Event\",e[e.Operator=11]=\"Operator\",e[e.Unit=12]=\"Unit\",e[e.Value=13]=\"Value\",e[e.Constant=14]=\"Constant\",e[e.Enum=15]=\"Enum\",e[e.EnumMember=16]=\"EnumMember\",e[e.Keyword=17]=\"Keyword\",e[e.Text=18]=\"Text\",e[e.Color=19]=\"Color\",e[e.File=20]=\"File\",e[e.Reference=21]=\"Reference\",e[e.Customcolor=22]=\"Customcolor\",e[e.Folder=23]=\"Folder\",e[e.TypeParameter=24]=\"TypeParameter\",e[e.Snippet=25]=\"Snippet\"}(nt||(nt={})),function(e){e[e.KeepWhitespace=1]=\"KeepWhitespace\",e[e.InsertAsSnippet=4]=\"InsertAsSnippet\"}(rt||(rt={})),function(e){e[e.Invoke=0]=\"Invoke\",e[e.TriggerCharacter=1]=\"TriggerCharacter\",e[e.TriggerForIncompleteCompletions=2]=\"TriggerForIncompleteCompletions\"}(it||(it={})),function(e){e[e.Invoke=1]=\"Invoke\",e[e.TriggerCharacter=2]=\"TriggerCharacter\",e[e.ContentChange=3]=\"ContentChange\"}(ot||(ot={})),function(e){e[e.Text=0]=\"Text\",e[e.Read=1]=\"Read\",e[e.Write=2]=\"Write\"}(at||(at={})),function(e){e[e.File=0]=\"File\",e[e.Module=1]=\"Module\",e[e.Namespace=2]=\"Namespace\",e[e.Package=3]=\"Package\",e[e.Class=4]=\"Class\",e[e.Method=5]=\"Method\",e[e.Property=6]=\"Property\",e[e.Field=7]=\"Field\",e[e.Constructor=8]=\"Constructor\",e[e.Enum=9]=\"Enum\",e[e.Interface=10]=\"Interface\",e[e.Function=11]=\"Function\",e[e.Variable=12]=\"Variable\",e[e.Constant=13]=\"Constant\",e[e.String=14]=\"String\",e[e.Number=15]=\"Number\",e[e.Boolean=16]=\"Boolean\",e[e.Array=17]=\"Array\",e[e.Object=18]=\"Object\",e[e.Key=19]=\"Key\",e[e.Null=20]=\"Null\",e[e.EnumMember=21]=\"EnumMember\",e[e.Struct=22]=\"Struct\",e[e.Event=23]=\"Event\",e[e.Operator=24]=\"Operator\",e[e.TypeParameter=25]=\"TypeParameter\"}(st||(st={}));var ht=function(){function e(){}return e.chord=function(e,t){return function(e,t){return(e|(65535&t)<<16>>>0)>>>0}(e,t)},e.CtrlCmd=2048,e.Shift=1024,e.Alt=512,e.WinCtrl=256,e}();var dt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ft=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return dt(t,e),Object.defineProperty(t.prototype,\"uri\",{get:function(){return this._uri},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,\"version\",{get:function(){return this._versionId},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,\"eol\",{get:function(){return this._eol},enumerable:!0,configurable:!0}),t.prototype.getValue=function(){return this.getText()},t.prototype.getLinesContent=function(){return this._lines.slice(0)},t.prototype.getLineCount=function(){return this._lines.length},t.prototype.getLineContent=function(e){return this._lines[e-1]},t.prototype.getWordAtPosition=function(e,t){var n=function(e,t,n,r){t.lastIndex=0;var i=t.exec(n);if(!i)return null;var o=i[0].indexOf(\" \")>=0?function(e,t,n,r){var i,o=e-1-r;for(t.lastIndex=0;i=t.exec(n);){var a=i.index||0;if(a>o)return null;if(t.lastIndex>=o)return{word:i[0],startColumn:r+1+a,endColumn:r+1+t.lastIndex}}return null}(e,t,n,r):function(e,t,n,r){var i,o=e-1-r,a=n.lastIndexOf(\" \",o-1)+1,s=n.indexOf(\" \",o);for(-1===s&&(s=n.length),t.lastIndex=a;i=t.exec(n);){var u=i.index||0;if(u<=o&&t.lastIndex>=o)return{word:i[0],startColumn:r+1+u,endColumn:r+1+t.lastIndex}}return null}(e,t,n,r);return t.lastIndex=0,o}(e.column,function(e){var t=Te;if(e&&e instanceof RegExp)if(e.global)t=e;else{var n=\"g\";e.ignoreCase&&(n+=\"i\"),e.multiline&&(n+=\"m\"),t=new RegExp(e.source,n)}return t.lastIndex=0,t}(t),this._lines[e.lineNumber-1],0);return n?new se(e.lineNumber,n.startColumn,e.lineNumber,n.endColumn):null},t.prototype.getWordUntilPosition=function(e,t){var n=this.getWordAtPosition(e,t);return n?{word:this._lines[e.lineNumber-1].substring(n.startColumn-1,e.column-1),startColumn:n.startColumn,endColumn:e.column}:{word:\"\",startColumn:e.column,endColumn:e.column}},t.prototype.createWordIterator=function(e){var t,n,r=this,i=0,o=0,a=[],s=function(){if(o<a.length){var u=n.substring(a[o].start,a[o].end);return o+=1,t?t.value=u:t={done:!1,value:u},t}return i>=r._lines.length?w:(n=r._lines[i],a=r._wordenize(n,e),o=0,i+=1,s())};return{next:s}},t.prototype.getLineWords=function(e,t){for(var n=this._lines[e-1],r=[],i=0,o=this._wordenize(n,t);i<o.length;i++){var a=o[i];r.push({word:n.substring(a.start,a.end),startColumn:a.start+1,endColumn:a.end+1})}return r},t.prototype._wordenize=function(e,t){var n,r=[];for(t.lastIndex=0;(n=t.exec(e))&&0!==n[0].length;)r.push({start:n.index,end:n.index+n[0].length});return r},t.prototype.getValueInRange=function(e){if((e=this._validateRange(e)).startLineNumber===e.endLineNumber)return this._lines[e.startLineNumber-1].substring(e.startColumn-1,e.endColumn-1);var t=this._eol,n=e.startLineNumber-1,r=e.endLineNumber-1,i=[];i.push(this._lines[n].substring(e.startColumn-1));for(var o=n+1;o<r;o++)i.push(this._lines[o]);return i.push(this._lines[r].substring(0,e.endColumn-1)),i.join(t)},t.prototype.offsetAt=function(e){return e=this._validatePosition(e),this._ensureLineStarts(),this._lineStarts.getAccumulatedValue(e.lineNumber-2)+(e.column-1)},t.prototype.positionAt=function(e){e=Math.floor(e),e=Math.max(0,e),this._ensureLineStarts();var t=this._lineStarts.getIndexOf(e),n=this._lines[t.index].length;return{lineNumber:1+t.index,column:1+Math.min(t.remainder,n)}},t.prototype._validateRange=function(e){var t=this._validatePosition({lineNumber:e.startLineNumber,column:e.startColumn}),n=this._validatePosition({lineNumber:e.endLineNumber,column:e.endColumn});return t.lineNumber!==e.startLineNumber||t.column!==e.startColumn||n.lineNumber!==e.endLineNumber||n.column!==e.endColumn?{startLineNumber:t.lineNumber,startColumn:t.column,endLineNumber:n.lineNumber,endColumn:n.column}:e},t.prototype._validatePosition=function(e){if(!ae.isIPosition(e))throw new Error(\"bad position\");var t=e.lineNumber,n=e.column,r=!1;if(t<1)t=1,n=1,r=!0;else if(t>this._lines.length)t=this._lines.length,n=this._lines[t-1].length+1,r=!0;else{var i=this._lines[t-1].length+1;n<1?(n=1,r=!0):n>i&&(n=i,r=!0)}return r?{lineNumber:t,column:n}:e},t}(we),pt=function(e){function t(t){var n=e.call(this,t)||this;return n._models=Object.create(null),n}return dt(t,e),t.prototype.dispose=function(){this._models=Object.create(null)},t.prototype._getModel=function(e){return this._models[e]},t.prototype._getModels=function(){var e=this,t=[];return Object.keys(this._models).forEach(function(n){return t.push(e._models[n])}),t},t.prototype.acceptNewModel=function(e){this._models[e.url]=new ft(X.parse(e.url),e.lines,e.EOL,e.versionId)},t.prototype.acceptModelChanged=function(e,t){this._models[e]&&this._models[e].onEvents(t)},t.prototype.acceptRemovedModel=function(e){this._models[e]&&delete this._models[e]},t}(function(){function e(e){this._foreignModuleFactory=e,this._foreignModule=null}return e.prototype.computeDiff=function(e,t,n){var r=this._getModel(e),i=this._getModel(t);if(!r||!i)return Promise.resolve(null);var o=r.getLinesContent(),a=i.getLinesContent(),s=new ge(o,a,{shouldComputeCharChanges:!0,shouldPostProcessCharChanges:!0,shouldIgnoreTrimWhitespace:n,shouldMakePrettyDiff:!0}).computeDiff(),u=!(s.length>0)&&this._modelsAreIdentical(r,i);return Promise.resolve({identical:u,changes:s})},e.prototype._modelsAreIdentical=function(e,t){var n=e.getLineCount();if(n!==t.getLineCount())return!1;for(var r=1;r<=n;r++){if(e.getLineContent(r)!==t.getLineContent(r))return!1}return!0},e.prototype.computeMoreMinimalEdits=function(t,n){var r=this._getModel(t);if(!r)return Promise.resolve(n);for(var i=[],o=void 0,a=0,s=n=U(n,function(e,t){return e.range&&t.range?se.compareRangesUsingStarts(e.range,t.range):(e.range?0:1)-(t.range?0:1)});a<s.length;a++){var u=s[a],l=u.range,c=u.text,h=u.eol;if(\"number\"==typeof h&&(o=h),l){var d=r.getValueInRange(l);if(d!==(c=c.replace(/\\r\\n|\\n|\\r/g,r.eol)))if(Math.max(c.length,d.length)>e._diffLimit)i.push({range:l,text:c});else for(var f=V(d,c,!1),p=r.offsetAt(se.lift(l).getStartPosition()),g=0,m=f;g<m.length;g++){var b=m[g],_=r.positionAt(p+b.originalStart),v=r.positionAt(p+b.originalStart+b.originalLength),y={text:c.substr(b.modifiedStart,b.modifiedLength),range:{startLineNumber:_.lineNumber,startColumn:_.column,endLineNumber:v.lineNumber,endColumn:v.column}};r.getValueInRange(y.range)!==y.text&&i.push(y)}}}return\"number\"==typeof o&&i.push({eol:o,text:void 0,range:void 0}),Promise.resolve(i)},e.prototype.computeLinks=function(e){var t=this._getModel(e);return t?Promise.resolve(function(e){return e&&\"function\"==typeof e.getLineCount&&\"function\"==typeof e.getLineContent?Ae.computeLinks(e):[]}(t)):Promise.resolve(null)},e.prototype.textualSuggest=function(t,n,r,i){var o=this._getModel(t);if(!o)return Promise.resolve(null);var a=[],s=new RegExp(r,i),u=o.getWordUntilPosition(n,s),l=Object.create(null);l[u.word]=!0;for(var c=o.createWordIterator(s),h=c.next();!h.done&&a.length<=e._suggestionsLimit;h=c.next()){var d=h.value;l[d]||(l[d]=!0,isNaN(Number(d))&&a.push({kind:18,label:d,insertText:d,range:{startLineNumber:n.lineNumber,startColumn:u.startColumn,endLineNumber:n.lineNumber,endColumn:u.endColumn}}))}return Promise.resolve({suggestions:a})},e.prototype.computeWordRanges=function(e,t,n,r){var i=this._getModel(e);if(!i)return Promise.resolve(Object.create(null));for(var o=new RegExp(n,r),a=Object.create(null),s=t.startLineNumber;s<t.endLineNumber;s++)for(var u=0,l=i.getLineWords(s,o);u<l.length;u++){var c=l[u];if(isNaN(Number(c.word))){var h=a[c.word];h||(h=[],a[c.word]=h),h.push({startLineNumber:s,startColumn:c.startColumn,endLineNumber:s,endColumn:c.endColumn})}}return Promise.resolve(a)},e.prototype.navigateValueSet=function(e,t,n,r,i){var o=this._getModel(e);if(!o)return Promise.resolve(null);var a=new RegExp(r,i);t.startColumn===t.endColumn&&(t={startLineNumber:t.startLineNumber,startColumn:t.startColumn,endLineNumber:t.endLineNumber,endColumn:t.endColumn+1});var s=o.getValueInRange(t),u=o.getWordAtPosition({lineNumber:t.startLineNumber,column:t.startColumn},a);if(!u)return Promise.resolve(null);var l=o.getValueInRange(u),c=Ne.INSTANCE.navigateValueSet(t,s,u,l,n);return Promise.resolve(c)},e.prototype.loadForeignModule=function(e,t){var n=this,r={getMirrorModels:function(){return n._getModels()}};if(this._foreignModuleFactory){this._foreignModule=this._foreignModuleFactory(r,t);var i=[];for(var o in this._foreignModule)\"function\"==typeof this._foreignModule[o]&&i.push(o);return Promise.resolve(i)}return Promise.reject(new Error(\"Unexpected usage\"))},e.prototype.fmr=function(e,t){if(!this._foreignModule||\"function\"!=typeof this._foreignModule[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._foreignModule[e].apply(this._foreignModule,t))}catch(e){return Promise.reject(e)}},e._diffLimit=1e4,e._suggestionsLimit=1e4,e}());\"function\"==typeof importScripts&&(g.monaco={editor:void 0,languages:void 0,CancellationTokenSource:L,Emitter:x,KeyCode:Ue,KeyMod:ht,Position:ae,Range:se,Selection:lt,SelectionDirection:Me,MarkerSeverity:De,MarkerTag:Re,Promise:n,Uri:X,Token:ct});var gt,mt,bt=!1;function _t(e){if(!bt){bt=!0;var t=new pt(e),n=new D(function(e){self.postMessage(e)},t);self.onmessage=function(e){n.onmessage(e.data)}}}function vt(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];return function(e,t){return 0===t.length?e:e.replace(/\\{(\\d+)\\}/g,function(e,n){var r=n[0];return void 0!==t[r]?t[r]:e})}(t,n)}function yt(e){return vt}self.onmessage=function(e){bt||_t(null)},function(e){e[e.StartCommentTag=0]=\"StartCommentTag\",e[e.Comment=1]=\"Comment\",e[e.EndCommentTag=2]=\"EndCommentTag\",e[e.StartTagOpen=3]=\"StartTagOpen\",e[e.StartTagClose=4]=\"StartTagClose\",e[e.StartTagSelfClose=5]=\"StartTagSelfClose\",e[e.StartTag=6]=\"StartTag\",e[e.EndTagOpen=7]=\"EndTagOpen\",e[e.EndTagClose=8]=\"EndTagClose\",e[e.EndTag=9]=\"EndTag\",e[e.DelimiterAssign=10]=\"DelimiterAssign\",e[e.AttributeName=11]=\"AttributeName\",e[e.AttributeValue=12]=\"AttributeValue\",e[e.StartDoctypeTag=13]=\"StartDoctypeTag\",e[e.Doctype=14]=\"Doctype\",e[e.EndDoctypeTag=15]=\"EndDoctypeTag\",e[e.Content=16]=\"Content\",e[e.Whitespace=17]=\"Whitespace\",e[e.Unknown=18]=\"Unknown\",e[e.Script=19]=\"Script\",e[e.Styles=20]=\"Styles\",e[e.EOS=21]=\"EOS\"}(gt||(gt={})),function(e){e[e.WithinContent=0]=\"WithinContent\",e[e.AfterOpeningStartTag=1]=\"AfterOpeningStartTag\",e[e.AfterOpeningEndTag=2]=\"AfterOpeningEndTag\",e[e.WithinDoctype=3]=\"WithinDoctype\",e[e.WithinTag=4]=\"WithinTag\",e[e.WithinEndTag=5]=\"WithinEndTag\",e[e.WithinComment=6]=\"WithinComment\",e[e.WithinScriptContent=7]=\"WithinScriptContent\",e[e.WithinStyleContent=8]=\"WithinStyleContent\",e[e.AfterAttributeName=9]=\"AfterAttributeName\",e[e.BeforeAttributeValue=10]=\"BeforeAttributeValue\"}(mt||(mt={}));var wt=yt(),Ct=function(){function e(e,t){this.source=e,this.len=e.length,this.position=t}return e.prototype.eos=function(){return this.len<=this.position},e.prototype.getSource=function(){return this.source},e.prototype.pos=function(){return this.position},e.prototype.goBackTo=function(e){this.position=e},e.prototype.goBack=function(e){this.position-=e},e.prototype.advance=function(e){this.position+=e},e.prototype.goToEnd=function(){this.position=this.source.length},e.prototype.nextChar=function(){return this.source.charCodeAt(this.position++)||0},e.prototype.peekChar=function(e){return void 0===e&&(e=0),this.source.charCodeAt(this.position+e)||0},e.prototype.advanceIfChar=function(e){return e===this.source.charCodeAt(this.position)&&(this.position++,!0)},e.prototype.advanceIfChars=function(e){var t;if(this.position+e.length>this.source.length)return!1;for(t=0;t<e.length;t++)if(this.source.charCodeAt(this.position+t)!==e[t])return!1;return this.advance(t),!0},e.prototype.advanceIfRegExp=function(e){var t=this.source.substr(this.position).match(e);return t?(this.position=this.position+t.index+t[0].length,t[0]):\"\"},e.prototype.advanceUntilRegExp=function(e){var t=this.source.substr(this.position).match(e);return t?(this.position=this.position+t.index,t[0]):(this.goToEnd(),\"\")},e.prototype.advanceUntilChar=function(e){for(;this.position<this.source.length;){if(this.source.charCodeAt(this.position)===e)return!0;this.advance(1)}return!1},e.prototype.advanceUntilChars=function(e){for(;this.position+e.length<=this.source.length;){for(var t=0;t<e.length&&this.source.charCodeAt(this.position+t)===e[t];t++);if(t===e.length)return!0;this.advance(1)}return this.goToEnd(),!1},e.prototype.skipWhitespace=function(){return this.advanceWhileChar(function(e){return e===qt||e===Rt||e===Ot||e===It||e===Pt})>0},e.prototype.advanceWhileChar=function(e){for(var t=this.position;this.position<this.len&&e(this.source.charCodeAt(this.position));)this.position++;return this.position-t},e}(),Tt=\"!\".charCodeAt(0),Et=\"-\".charCodeAt(0),xt=\"<\".charCodeAt(0),St=\">\".charCodeAt(0),kt=\"/\".charCodeAt(0),At=\"=\".charCodeAt(0),Lt='\"'.charCodeAt(0),Nt=\"'\".charCodeAt(0),Ot=\"\\n\".charCodeAt(0),Pt=\"\\r\".charCodeAt(0),It=\"\\f\".charCodeAt(0),qt=\" \".charCodeAt(0),Rt=\"\\t\".charCodeAt(0),Dt={\"text/x-handlebars-template\":!0};function Ut(e,t,n){void 0===t&&(t=0),void 0===n&&(n=mt.WithinContent);var r,i,o,a,s,u=new Ct(e,t),l=n,c=0,h=gt.Unknown;function d(){return u.advanceIfRegExp(/^[_:\\w][_:\\w-.\\d]*/).toLowerCase()}function f(e,t,n){return h=t,c=e,r=n,t}return{scan:function(){var e=u.pos(),t=l,n=function e(){var t,n=u.pos();if(u.eos())return f(n,gt.EOS);switch(l){case mt.WithinComment:return u.advanceIfChars([Et,Et,St])?(l=mt.WithinContent,f(n,gt.EndCommentTag)):(u.advanceUntilChars([Et,Et,St]),f(n,gt.Comment));case mt.WithinDoctype:return u.advanceIfChar(St)?(l=mt.WithinContent,f(n,gt.EndDoctypeTag)):(u.advanceUntilChar(St),f(n,gt.Doctype));case mt.WithinContent:if(u.advanceIfChar(xt)){if(!u.eos()&&u.peekChar()===Tt){if(u.advanceIfChars([Tt,Et,Et]))return l=mt.WithinComment,f(n,gt.StartCommentTag);if(u.advanceIfRegExp(/^!doctype/i))return l=mt.WithinDoctype,f(n,gt.StartDoctypeTag)}return u.advanceIfChar(kt)?(l=mt.AfterOpeningEndTag,f(n,gt.EndTagOpen)):(l=mt.AfterOpeningStartTag,f(n,gt.StartTagOpen))}return u.advanceUntilChar(xt),f(n,gt.Content);case mt.AfterOpeningEndTag:var r=d();return r.length>0?(l=mt.WithinEndTag,f(n,gt.EndTag)):u.skipWhitespace()?f(n,gt.Whitespace,wt(\"error.unexpectedWhitespace\",\"Tag name must directly follow the open bracket.\")):(l=mt.WithinEndTag,u.advanceUntilChar(St),n<u.pos()?f(n,gt.Unknown,wt(\"error.endTagNameExpected\",\"End tag name expected.\")):e());case mt.WithinEndTag:if(u.skipWhitespace())return f(n,gt.Whitespace);if(u.advanceIfChar(St))return l=mt.WithinContent,f(n,gt.EndTagClose);t=wt(\"error.tagNameExpected\",\"Closing bracket expected.\");break;case mt.AfterOpeningStartTag:return o=d(),s=void 0,a=void 0,o.length>0?(i=!1,l=mt.WithinTag,f(n,gt.StartTag)):u.skipWhitespace()?f(n,gt.Whitespace,wt(\"error.unexpectedWhitespace\",\"Tag name must directly follow the open bracket.\")):(l=mt.WithinTag,u.advanceUntilChar(St),n<u.pos()?f(n,gt.Unknown,wt(\"error.startTagNameExpected\",\"Start tag name expected.\")):e());case mt.WithinTag:return u.skipWhitespace()?(i=!0,f(n,gt.Whitespace)):i&&(a=u.advanceIfRegExp(/^[^\\s\"'>\\/=\\x00-\\x0F\\x7F\\x80-\\x9F]*/).toLowerCase()).length>0?(l=mt.AfterAttributeName,i=!1,f(n,gt.AttributeName)):u.advanceIfChars([kt,St])?(l=mt.WithinContent,f(n,gt.StartTagSelfClose)):u.advanceIfChar(St)?(l=\"script\"===o?s&&Dt[s]?mt.WithinContent:mt.WithinScriptContent:\"style\"===o?mt.WithinStyleContent:mt.WithinContent,f(n,gt.StartTagClose)):(u.advance(1),f(n,gt.Unknown,wt(\"error.unexpectedCharacterInTag\",\"Unexpected character in tag.\")));case mt.AfterAttributeName:return u.skipWhitespace()?(i=!0,f(n,gt.Whitespace)):u.advanceIfChar(At)?(l=mt.BeforeAttributeValue,f(n,gt.DelimiterAssign)):(l=mt.WithinTag,e());case mt.BeforeAttributeValue:if(u.skipWhitespace())return f(n,gt.Whitespace);var c=u.advanceIfRegExp(/^[^\\s\"'`=<>\\/]+/);if(c.length>0)return\"type\"===a&&(s=c),l=mt.WithinTag,i=!1,f(n,gt.AttributeValue);var h=u.peekChar();return h===Nt||h===Lt?(u.advance(1),u.advanceUntilChar(h)&&u.advance(1),\"type\"===a&&(s=u.getSource().substring(n+1,u.pos()-1)),l=mt.WithinTag,i=!1,f(n,gt.AttributeValue)):(l=mt.WithinTag,i=!1,e());case mt.WithinScriptContent:for(var p=1;!u.eos();){var g=u.advanceIfRegExp(/<!--|-->|<\\/?script\\s*\\/?>?/i);if(0===g.length)return u.goToEnd(),f(n,gt.Script);if(\"\\x3c!--\"===g)1===p&&(p=2);else if(\"--\\x3e\"===g)p=1;else if(\"/\"!==g[1])2===p&&(p=3);else{if(3!==p){u.goBack(g.length);break}p=2}}return l=mt.WithinContent,n<u.pos()?f(n,gt.Script):e();case mt.WithinStyleContent:return u.advanceUntilRegExp(/<\\/style/i),l=mt.WithinContent,n<u.pos()?f(n,gt.Styles):e()}return u.advance(1),l=mt.WithinContent,f(n,gt.Unknown,t)}();return n!==gt.EOS&&e===u.pos()?(console.log(\"Scanner.scan has not advanced at offset \"+e+\", state before: \"+t+\" after: \"+l),u.advance(1),f(e,gt.Unknown)):n},getTokenType:function(){return h},getTokenOffset:function(){return c},getTokenLength:function(){return u.pos()-c},getTokenEnd:function(){return u.pos()},getTokenText:function(){return u.getSource().substring(c,u.pos())},getScannerState:function(){return l},getTokenError:function(){return r}}}function Mt(e,t){var n=0,r=e.length;if(0===r)return 0;for(;n<r;){var i=Math.floor((n+r)/2);t(e[i])?r=i:n=i+1}return n}function Wt(e,t){if(e.length<t.length)return!1;for(var n=0;n<t.length;n++)if(e[n]!==t[n])return!1;return!0}function Vt(e,t){for(var n=\"\";t>0;)1==(1&t)&&(n+=e),e+=e,t>>>=1;return n}var Bt=\"a\".charCodeAt(0),jt=\"z\".charCodeAt(0),Ft=\"A\".charCodeAt(0),Kt=\"Z\".charCodeAt(0),Ht=\"0\".charCodeAt(0),Gt=\"9\".charCodeAt(0);function Yt(e,t){var n=e.charCodeAt(t);return Bt<=n&&n<=jt||Ft<=n&&n<=Kt||Ht<=n&&n<=Gt}var zt=yt(),Jt=[\"area\",\"base\",\"br\",\"col\",\"embed\",\"hr\",\"img\",\"input\",\"keygen\",\"link\",\"menuitem\",\"meta\",\"param\",\"source\",\"track\",\"wbr\"];function $t(e){return!!e&&function(e,t,n){for(var r=0,i=e.length-1;r<=i;){var o=(r+i)/2|0,a=n(e[o],t);if(a<0)r=o+1;else{if(!(a>0))return o;i=o-1}}return-(r+1)}(Jt,e.toLowerCase(),function(e,t){return e.localeCompare(t)})>=0}var Qt=function(){return function(e,t){void 0===t&&(t=[]),this.label=e,this.attributes=t}}(),Zt={html:new Qt(zt(\"tags.html\",\"The html element represents the root of an HTML document.\"),[\"manifest\"]),head:new Qt(zt(\"tags.head\",\"The head element represents a collection of metadata for the Document.\")),title:new Qt(zt(\"tags.title\",\"The title element represents the document's title or name. Authors should use titles that identify their documents even when they are used out of context, for example in a user's history or bookmarks, or in search results. The document's title is often different from its first heading, since the first heading does not have to stand alone when taken out of context.\")),base:new Qt(zt(\"tags.base\",\"The base element allows authors to specify the document base URL for the purposes of resolving relative URLs, and the name of the default browsing context for the purposes of following hyperlinks. The element does not represent any content beyond this information.\"),[\"href\",\"target\"]),link:new Qt(zt(\"tags.link\",\"The link element allows authors to link their document to other resources.\"),[\"href\",\"crossorigin:xo\",\"rel\",\"media\",\"hreflang\",\"type\",\"sizes\"]),meta:new Qt(zt(\"tags.meta\",\"The meta element represents various kinds of metadata that cannot be expressed using the title, base, link, style, and script elements.\"),[\"name\",\"http-equiv\",\"content\",\"charset\"]),style:new Qt(zt(\"tags.style\",\"The style element allows authors to embed style information in their documents. The style element is one of several inputs to the styling processing model. The element does not represent content for the user.\"),[\"media\",\"nonce\",\"type\",\"scoped:v\"]),body:new Qt(zt(\"tags.body\",\"The body element represents the content of the document.\"),[\"onafterprint\",\"onbeforeprint\",\"onbeforeunload\",\"onhashchange\",\"onlanguagechange\",\"onmessage\",\"onoffline\",\"ononline\",\"onpagehide\",\"onpageshow\",\"onpopstate\",\"onstorage\",\"onunload\"]),article:new Qt(zt(\"tags.article\",\"The article element represents a complete, or self-contained, composition in a document, page, application, or site and that is, in principle, independently distributable or reusable, e.g. in syndication. This could be a forum post, a magazine or newspaper article, a blog entry, a user-submitted comment, an interactive widget or gadget, or any other independent item of content. Each article should be identified, typically by including a heading (h1–h6 element) as a child of the article element.\")),section:new Qt(zt(\"tags.section\",\"The section element represents a generic section of a document or application. A section, in this context, is a thematic grouping of content. Each section should be identified, typically by including a heading ( h1- h6 element) as a child of the section element.\")),nav:new Qt(zt(\"tags.nav\",\"The nav element represents a section of a page that links to other pages or to parts within the page: a section with navigation links.\")),aside:new Qt(zt(\"tags.aside\",\"The aside element represents a section of a page that consists of content that is tangentially related to the content around the aside element, and which could be considered separate from that content. Such sections are often represented as sidebars in printed typography.\")),h1:new Qt(zt(\"tags.h1\",\"The h1 element represents a section heading.\")),h2:new Qt(zt(\"tags.h2\",\"The h2 element represents a section heading.\")),h3:new Qt(zt(\"tags.h3\",\"The h3 element represents a section heading.\")),h4:new Qt(zt(\"tags.h4\",\"The h4 element represents a section heading.\")),h5:new Qt(zt(\"tags.h5\",\"The h5 element represents a section heading.\")),h6:new Qt(zt(\"tags.h6\",\"The h6 element represents a section heading.\")),header:new Qt(zt(\"tags.header\",\"The header element represents introductory content for its nearest ancestor sectioning content or sectioning root element. A header typically contains a group of introductory or navigational aids. When the nearest ancestor sectioning content or sectioning root element is the body element, then it applies to the whole page.\")),footer:new Qt(zt(\"tags.footer\",\"The footer element represents a footer for its nearest ancestor sectioning content or sectioning root element. A footer typically contains information about its section such as who wrote it, links to related documents, copyright data, and the like.\")),address:new Qt(zt(\"tags.address\",\"The address element represents the contact information for its nearest article or body element ancestor. If that is the body element, then the contact information applies to the document as a whole.\")),p:new Qt(zt(\"tags.p\",\"The p element represents a paragraph.\")),hr:new Qt(zt(\"tags.hr\",\"The hr element represents a paragraph-level thematic break, e.g. a scene change in a story, or a transition to another topic within a section of a reference book.\")),pre:new Qt(zt(\"tags.pre\",\"The pre element represents a block of preformatted text, in which structure is represented by typographic conventions rather than by elements.\")),blockquote:new Qt(zt(\"tags.blockquote\",\"The blockquote element represents content that is quoted from another source, optionally with a citation which must be within a footer or cite element, and optionally with in-line changes such as annotations and abbreviations.\"),[\"cite\"]),ol:new Qt(zt(\"tags.ol\",\"The ol element represents a list of items, where the items have been intentionally ordered, such that changing the order would change the meaning of the document.\"),[\"reversed:v\",\"start\",\"type:lt\"]),ul:new Qt(zt(\"tags.ul\",\"The ul element represents a list of items, where the order of the items is not important — that is, where changing the order would not materially change the meaning of the document.\")),li:new Qt(zt(\"tags.li\",\"The li element represents a list item. If its parent element is an ol, ul, or menu element, then the element is an item of the parent element's list, as defined for those elements. Otherwise, the list item has no defined list-related relationship to any other li element.\"),[\"value\"]),dl:new Qt(zt(\"tags.dl\",\"The dl element represents an association list consisting of zero or more name-value groups (a description list). A name-value group consists of one or more names (dt elements) followed by one or more values (dd elements), ignoring any nodes other than dt and dd elements. Within a single dl element, there should not be more than one dt element for each name.\")),dt:new Qt(zt(\"tags.dt\",\"The dt element represents the term, or name, part of a term-description group in a description list (dl element).\")),dd:new Qt(zt(\"tags.dd\",\"The dd element represents the description, definition, or value, part of a term-description group in a description list (dl element).\")),figure:new Qt(zt(\"tags.figure\",\"The figure element represents some flow content, optionally with a caption, that is self-contained (like a complete sentence) and is typically referenced as a single unit from the main flow of the document.\")),figcaption:new Qt(zt(\"tags.figcaption\",\"The figcaption element represents a caption or legend for the rest of the contents of the figcaption element's parent figure element, if any.\")),main:new Qt(zt(\"tags.main\",\"The main element represents the main content of the body of a document or application. The main content area consists of content that is directly related to or expands upon the central topic of a document or central functionality of an application.\")),div:new Qt(zt(\"tags.div\",\"The div element has no special meaning at all. It represents its children. It can be used with the class, lang, and title attributes to mark up semantics common to a group of consecutive elements.\")),a:new Qt(zt(\"tags.a\",\"If the a element has an href attribute, then it represents a hyperlink (a hypertext anchor) labeled by its contents.\"),[\"href\",\"target\",\"download\",\"ping\",\"rel\",\"hreflang\",\"type\"]),em:new Qt(zt(\"tags.em\",\"The em element represents stress emphasis of its contents.\")),strong:new Qt(zt(\"tags.strong\",\"The strong element represents strong importance, seriousness, or urgency for its contents.\")),small:new Qt(zt(\"tags.small\",\"The small element represents side comments such as small print.\")),s:new Qt(zt(\"tags.s\",\"The s element represents contents that are no longer accurate or no longer relevant.\")),cite:new Qt(zt(\"tags.cite\",\"The cite element represents a reference to a creative work. It must include the title of the work or the name of the author(person, people or organization) or an URL reference, or a reference in abbreviated form as per the conventions used for the addition of citation metadata.\")),q:new Qt(zt(\"tags.q\",\"The q element represents some phrasing content quoted from another source.\"),[\"cite\"]),dfn:new Qt(zt(\"tags.dfn\",\"The dfn element represents the defining instance of a term. The paragraph, description list group, or section that is the nearest ancestor of the dfn element must also contain the definition(s) for the term given by the dfn element.\")),abbr:new Qt(zt(\"tags.abbr\",\"The abbr element represents an abbreviation or acronym, optionally with its expansion. The title attribute may be used to provide an expansion of the abbreviation. The attribute, if specified, must contain an expansion of the abbreviation, and nothing else.\")),ruby:new Qt(zt(\"tags.ruby\",\"The ruby element allows one or more spans of phrasing content to be marked with ruby annotations. Ruby annotations are short runs of text presented alongside base text, primarily used in East Asian typography as a guide for pronunciation or to include other annotations. In Japanese, this form of typography is also known as furigana. Ruby text can appear on either side, and sometimes both sides, of the base text, and it is possible to control its position using CSS. A more complete introduction to ruby can be found in the Use Cases & Exploratory Approaches for Ruby Markup document as well as in CSS Ruby Module Level 1. [RUBY-UC] [CSSRUBY]\")),rb:new Qt(zt(\"tags.rb\",\"The rb element marks the base text component of a ruby annotation. When it is the child of a ruby element, it doesn't represent anything itself, but its parent ruby element uses it as part of determining what it represents.\")),rt:new Qt(zt(\"tags.rt\",\"The rt element marks the ruby text component of a ruby annotation. When it is the child of a ruby element or of an rtc element that is itself the child of a ruby element, it doesn't represent anything itself, but its ancestor ruby element uses it as part of determining what it represents.\")),rp:new Qt(zt(\"tags.rp\",\"The rp element is used to provide fallback text to be shown by user agents that don't support ruby annotations. One widespread convention is to provide parentheses around the ruby text component of a ruby annotation.\")),time:new Qt(zt(\"tags.time\",\"The time element represents its contents, along with a machine-readable form of those contents in the datetime attribute. The kind of content is limited to various kinds of dates, times, time-zone offsets, and durations, as described below.\"),[\"datetime\"]),code:new Qt(zt(\"tags.code\",\"The code element represents a fragment of computer code. This could be an XML element name, a file name, a computer program, or any other string that a computer would recognize.\")),var:new Qt(zt(\"tags.var\",\"The var element represents a variable. This could be an actual variable in a mathematical expression or programming context, an identifier representing a constant, a symbol identifying a physical quantity, a function parameter, or just be a term used as a placeholder in prose.\")),samp:new Qt(zt(\"tags.samp\",\"The samp element represents sample or quoted output from another program or computing system.\")),kbd:new Qt(zt(\"tags.kbd\",\"The kbd element represents user input (typically keyboard input, although it may also be used to represent other input, such as voice commands).\")),sub:new Qt(zt(\"tags.sub\",\"The sub element represents a subscript.\")),sup:new Qt(zt(\"tags.sup\",\"The sup element represents a superscript.\")),i:new Qt(zt(\"tags.i\",\"The i element represents a span of text in an alternate voice or mood, or otherwise offset from the normal prose in a manner indicating a different quality of text, such as a taxonomic designation, a technical term, an idiomatic phrase from another language, transliteration, a thought, or a ship name in Western texts.\")),b:new Qt(zt(\"tags.b\",\"The b element represents a span of text to which attention is being drawn for utilitarian purposes without conveying any extra importance and with no implication of an alternate voice or mood, such as key words in a document abstract, product names in a review, actionable words in interactive text-driven software, or an article lede.\")),u:new Qt(zt(\"tags.u\",\"The u element represents a span of text with an unarticulated, though explicitly rendered, non-textual annotation, such as labeling the text as being a proper name in Chinese text (a Chinese proper name mark), or labeling the text as being misspelt.\")),mark:new Qt(zt(\"tags.mark\",\"The mark element represents a run of text in one document marked or highlighted for reference purposes, due to its relevance in another context. When used in a quotation or other block of text referred to from the prose, it indicates a highlight that was not originally present but which has been added to bring the reader's attention to a part of the text that might not have been considered important by the original author when the block was originally written, but which is now under previously unexpected scrutiny. When used in the main prose of a document, it indicates a part of the document that has been highlighted due to its likely relevance to the user's current activity.\")),bdi:new Qt(zt(\"tags.bdi\",\"The bdi element represents a span of text that is to be isolated from its surroundings for the purposes of bidirectional text formatting. [BIDI]\")),bdo:new Qt(zt(\"tags.dbo\",\"The bdo element represents explicit text directionality formatting control for its children. It allows authors to override the Unicode bidirectional algorithm by explicitly specifying a direction override. [BIDI]\")),span:new Qt(zt(\"tags.span\",\"The span element doesn't mean anything on its own, but can be useful when used together with the global attributes, e.g. class, lang, or dir. It represents its children.\")),br:new Qt(zt(\"tags.br\",\"The br element represents a line break.\")),wbr:new Qt(zt(\"tags.wbr\",\"The wbr element represents a line break opportunity.\")),ins:new Qt(zt(\"tags.ins\",\"The ins element represents an addition to the document.\")),del:new Qt(zt(\"tags.del\",\"The del element represents a removal from the document.\"),[\"cite\",\"datetime\"]),picture:new Qt(zt(\"tags.picture\",\"The picture element is a container which provides multiple sources to its contained img element to allow authors to declaratively control or give hints to the user agent about which image resource to use, based on the screen pixel density, viewport size, image format, and other factors. It represents its children.\")),img:new Qt(zt(\"tags.img\",\"An img element represents an image.\"),[\"alt\",\"src\",\"srcset\",\"crossorigin:xo\",\"usemap\",\"ismap:v\",\"width\",\"height\"]),iframe:new Qt(zt(\"tags.iframe\",\"The iframe element represents a nested browsing context.\"),[\"src\",\"srcdoc\",\"name\",\"sandbox:sb\",\"seamless:v\",\"allowfullscreen:v\",\"width\",\"height\"]),embed:new Qt(zt(\"tags.embed\",\"The embed element provides an integration point for an external (typically non-HTML) application or interactive content.\"),[\"src\",\"type\",\"width\",\"height\"]),object:new Qt(zt(\"tags.object\",\"The object element can represent an external resource, which, depending on the type of the resource, will either be treated as an image, as a nested browsing context, or as an external resource to be processed by a plugin.\"),[\"data\",\"type\",\"typemustmatch:v\",\"name\",\"usemap\",\"form\",\"width\",\"height\"]),param:new Qt(zt(\"tags.param\",\"The param element defines parameters for plugins invoked by object elements. It does not represent anything on its own.\"),[\"name\",\"value\"]),video:new Qt(zt(\"tags.video\",\"A video element is used for playing videos or movies, and audio files with captions.\"),[\"src\",\"crossorigin:xo\",\"poster\",\"preload:pl\",\"autoplay:v\",\"mediagroup\",\"loop:v\",\"muted:v\",\"controls:v\",\"width\",\"height\"]),audio:new Qt(zt(\"tags.audio\",\"An audio element represents a sound or audio stream.\"),[\"src\",\"crossorigin:xo\",\"preload:pl\",\"autoplay:v\",\"mediagroup\",\"loop:v\",\"muted:v\",\"controls:v\"]),source:new Qt(zt(\"tags.source\",\"The source element allows authors to specify multiple alternative media resources for media elements. It does not represent anything on its own.\"),[\"src\",\"type\"]),track:new Qt(zt(\"tags.track\",\"The track element allows authors to specify explicit external timed text tracks for media elements. It does not represent anything on its own.\"),[\"default:v\",\"kind:tk\",\"label\",\"src\",\"srclang\"]),map:new Qt(zt(\"tags.map\",\"The map element, in conjunction with an img element and any area element descendants, defines an image map. The element represents its children.\"),[\"name\"]),area:new Qt(zt(\"tags.area\",\"The area element represents either a hyperlink with some text and a corresponding area on an image map, or a dead area on an image map.\"),[\"alt\",\"coords\",\"shape:sh\",\"href\",\"target\",\"download\",\"ping\",\"rel\",\"hreflang\",\"type\"]),table:new Qt(zt(\"tags.table\",\"The table element represents data with more than one dimension, in the form of a table.\"),[\"sortable:v\",\"border\"]),caption:new Qt(zt(\"tags.caption\",\"The caption element represents the title of the table that is its parent, if it has a parent and that is a table element.\")),colgroup:new Qt(zt(\"tags.colgroup\",\"The colgroup element represents a group of one or more columns in the table that is its parent, if it has a parent and that is a table element.\"),[\"span\"]),col:new Qt(zt(\"tags.col\",\"If a col element has a parent and that is a colgroup element that itself has a parent that is a table element, then the col element represents one or more columns in the column group represented by that colgroup.\"),[\"span\"]),tbody:new Qt(zt(\"tags.tbody\",\"The tbody element represents a block of rows that consist of a body of data for the parent table element, if the tbody element has a parent and it is a table.\")),thead:new Qt(zt(\"tags.thead\",\"The thead element represents the block of rows that consist of the column labels (headers) for the parent table element, if the thead element has a parent and it is a table.\")),tfoot:new Qt(zt(\"tags.tfoot\",\"The tfoot element represents the block of rows that consist of the column summaries (footers) for the parent table element, if the tfoot element has a parent and it is a table.\")),tr:new Qt(zt(\"tags.tr\",\"The tr element represents a row of cells in a table.\")),td:new Qt(zt(\"tags.td\",\"The td element represents a data cell in a table.\"),[\"colspan\",\"rowspan\",\"headers\"]),th:new Qt(zt(\"tags.th\",\"The th element represents a header cell in a table.\"),[\"colspan\",\"rowspan\",\"headers\",\"scope:s\",\"sorted\",\"abbr\"]),form:new Qt(zt(\"tags.form\",\"The form element represents a collection of form-associated elements, some of which can represent editable values that can be submitted to a server for processing.\"),[\"accept-charset\",\"action\",\"autocomplete:o\",\"enctype:et\",\"method:m\",\"name\",\"novalidate:v\",\"target\"]),label:new Qt(zt(\"tags.label\",\"The label element represents a caption in a user interface. The caption can be associated with a specific form control, known as the label element's labeled control, either using the for attribute, or by putting the form control inside the label element itself.\"),[\"form\",\"for\"]),input:new Qt(zt(\"tags.input\",\"The input element represents a typed data field, usually with a form control to allow the user to edit the data.\"),[\"accept\",\"alt\",\"autocomplete:inputautocomplete\",\"autofocus:v\",\"checked:v\",\"dirname\",\"disabled:v\",\"form\",\"formaction\",\"formenctype:et\",\"formmethod:fm\",\"formnovalidate:v\",\"formtarget\",\"height\",\"inputmode:im\",\"list\",\"max\",\"maxlength\",\"min\",\"minlength\",\"multiple:v\",\"name\",\"pattern\",\"placeholder\",\"readonly:v\",\"required:v\",\"size\",\"src\",\"step\",\"type:t\",\"value\",\"width\"]),button:new Qt(zt(\"tags.button\",\"The button element represents a button labeled by its contents.\"),[\"autofocus:v\",\"disabled:v\",\"form\",\"formaction\",\"formenctype:et\",\"formmethod:fm\",\"formnovalidate:v\",\"formtarget\",\"name\",\"type:bt\",\"value\"]),select:new Qt(zt(\"tags.select\",\"The select element represents a control for selecting amongst a set of options.\"),[\"autocomplete:inputautocomplete\",\"autofocus:v\",\"disabled:v\",\"form\",\"multiple:v\",\"name\",\"required:v\",\"size\"]),datalist:new Qt(zt(\"tags.datalist\",\"The datalist element represents a set of option elements that represent predefined options for other controls. In the rendering, the datalist element represents nothing and it, along with its children, should be hidden.\")),optgroup:new Qt(zt(\"tags.optgroup\",\"The optgroup element represents a group of option elements with a common label.\"),[\"disabled:v\",\"label\"]),option:new Qt(zt(\"tags.option\",\"The option element represents an option in a select element or as part of a list of suggestions in a datalist element.\"),[\"disabled:v\",\"label\",\"selected:v\",\"value\"]),textarea:new Qt(zt(\"tags.textarea\",\"The textarea element represents a multiline plain text edit control for the element's raw value. The contents of the control represent the control's default value.\"),[\"autocomplete:inputautocomplete\",\"autofocus:v\",\"cols\",\"dirname\",\"disabled:v\",\"form\",\"inputmode:im\",\"maxlength\",\"minlength\",\"name\",\"placeholder\",\"readonly:v\",\"required:v\",\"rows\",\"wrap:w\"]),output:new Qt(zt(\"tags.output\",\"The output element represents the result of a calculation performed by the application, or the result of a user action.\"),[\"for\",\"form\",\"name\"]),progress:new Qt(zt(\"tags.progress\",\"The progress element represents the completion progress of a task. The progress is either indeterminate, indicating that progress is being made but that it is not clear how much more work remains to be done before the task is complete (e.g. because the task is waiting for a remote host to respond), or the progress is a number in the range zero to a maximum, giving the fraction of work that has so far been completed.\"),[\"value\",\"max\"]),meter:new Qt(zt(\"tags.meter\",\"The meter element represents a scalar measurement within a known range, or a fractional value; for example disk usage, the relevance of a query result, or the fraction of a voting population to have selected a particular candidate.\"),[\"value\",\"min\",\"max\",\"low\",\"high\",\"optimum\"]),fieldset:new Qt(zt(\"tags.fieldset\",\"The fieldset element represents a set of form controls optionally grouped under a common name.\"),[\"disabled:v\",\"form\",\"name\"]),legend:new Qt(zt(\"tags.legend\",\"The legend element represents a caption for the rest of the contents of the legend element's parent fieldset element, if any.\")),details:new Qt(zt(\"tags.details\",\"The details element represents a disclosure widget from which the user can obtain additional information or controls.\"),[\"open:v\"]),summary:new Qt(zt(\"tags.summary\",\"The summary element represents a summary, caption, or legend for the rest of the contents of the summary element's parent details element, if any.\")),dialog:new Qt(zt(\"tags.dialog\",\"The dialog element represents a part of an application that a user interacts with to perform a task, for example a dialog box, inspector, or window.\")),script:new Qt(zt(\"tags.script\",\"The script element allows authors to include dynamic script and data blocks in their documents. The element does not represent content for the user.\"),[\"src\",\"type\",\"charset\",\"async:v\",\"defer:v\",\"crossorigin:xo\",\"nonce\"]),noscript:new Qt(zt(\"tags.noscript\",\"The noscript element represents nothing if scripting is enabled, and represents its children if scripting is disabled. It is used to present different markup to user agents that support scripting and those that don't support scripting, by affecting how the document is parsed.\")),template:new Qt(zt(\"tags.template\",\"The template element is used to declare fragments of HTML that can be cloned and inserted in the document by script.\")),canvas:new Qt(zt(\"tags.canvas\",\"The canvas element provides scripts with a resolution-dependent bitmap canvas, which can be used for rendering graphs, game graphics, art, or other visual images on the fly.\"),[\"width\",\"height\"])},Xt={\"ion-checkbox\":new Qt(zt(\"tags.ion.checkbox\",\"The checkbox is no different than the HTML checkbox input, except it's styled differently. The checkbox behaves like any AngularJS checkbox.\"),[\"name\",\"ng-false-value\",\"ng-model\",\"ng-true-value\"]),\"ion-content\":new Qt(zt(\"tags.ion.content\",\"The ionContent directive provides an easy to use content area that can be configured to use Ionic's custom Scroll View, or the built-in overflow scrolling of the browser.\"),[\"delegate-handle\",\"direction:scrolldir\",\"has-bouncing:b\",\"locking:b\",\"on-scroll\",\"on-scroll-complete\",\"overflow-scroll:b\",\"padding:b\",\"scroll:b\",\"scrollbar-x:b\",\"scrollbar-y:b\",\"start-x\",\"start-y\"]),\"ion-delete-button\":new Qt(zt(\"tags.ion.deletebutton\",\"Child of ionItem\"),[]),\"ion-footer-bar\":new Qt(zt(\"tags.ion.footerbar\",'Adds a fixed footer bar below some content. Can also be a subfooter (higher up) if the \"bar-subfooter\" class is applied.'),[\"align-title:align\",\"keyboard-attach:v\"]),\"ion-header-bar\":new Qt(zt(\"tags.ion.headerbar\",'Adds a fixed header bar above some content. Can also be a subheader (lower down) if the \"bar-subheader\" class is applied.'),[\"align-title:align\",\"no-tap-scroll:b\"]),\"ion-infinite-scroll\":new Qt(zt(\"tags.ion.infinitescroll\",\"Child of ionContent or ionScroll. The ionInfiniteScroll directive allows you to call a function whenever the user gets to the bottom of the page or near the bottom of the page.\"),[\"distance\",\"icon\",\"immediate-check:b\",\"on-infinite\",\"spinner\"]),\"ion-input\":new Qt(zt(\"tags.ion.input\",'ionInput is meant for text type inputs only. Ionic uses an actual <input type=\"text\"> HTML element within the component, with Ionic wrapping to better handle the user experience and interactivity.'),[\"type:inputtype\",\"clearInput:v\"]),\"ion-item\":new Qt(zt(\"tags.ion.item\",\"Child of ionList.\"),[]),\"ion-list\":new Qt(zt(\"tags.ion.list\",\"The List is a widely used interface element in almost any mobile app, and can include content ranging from basic text all the way to buttons, toggles, icons, and thumbnails.\"),[\"can-swipe:b\",\"delegate-handle\",\"show-delete:b\",\"show-reorder:b\",\"type:listtype\"]),\"ion-modal-view\":new Qt(zt(\"tags.ion.modalview\",\"The Modal is a content pane that can go over the user's main view temporarily. Usually used for making a choice or editing an item.\"),[]),\"ion-nav-back-button\":new Qt(zt(\"tags.ion.navbackbutton\",\"Child of ionNavBar. Creates a back button inside an ionNavBar. The back button will appear when the user is able to go back in the current navigation stack.\"),[]),\"ion-nav-bar\":new Qt(zt(\"tags.ion.navbar\",\"If you have an ionNavView directive, you can also create an <ion-nav-bar>, which will create a topbar that updates as the application state changes.\"),[\"align-title:align\",\"delegate-handle\",\"no-tap-scroll:b\"]),\"ion-nav-buttons\":new Qt(zt(\"tags.ion.navbuttons\",\"Child of ionNavView. Use ionNavButtons to set the buttons on your ionNavBar from within an ionView.\"),[\"side:navsides\"]),\"ion-nav-title\":new Qt(zt(\"tags.ion.navtitle\",\"Child of ionNavView. The ionNavTitle directive replaces an ionNavBar title text with custom HTML from within an ionView template.\"),[]),\"ion-nav-view\":new Qt(zt(\"tags.ion.navview\",\"The ionNavView directive is used to render templates in your application. Each template is part of a state. States are usually mapped to a url, and are defined programatically using angular-ui-router.\"),[\"name\"]),\"ion-option-button\":new Qt(zt(\"tags.ion.optionbutton\",\"Child of ionItem. Creates an option button inside a list item, that is visible when the item is swiped to the left by the user.\"),[]),\"ion-pane\":new Qt(zt(\"tags.ion.pane\",'A simple container that fits content, with no side effects. Adds the \"pane\" class to the element.'),[]),\"ion-popover-view\":new Qt(zt(\"tags.ion.popoverview\",\"The Popover is a view that floats above an app's content. Popovers provide an easy way to present or gather information from the user.\"),[]),\"ion-radio\":new Qt(zt(\"tags.ion.radio\",\"The radio ionRirective is no different than the HTML radio input, except it's styled differently. The ionRadio behaves like AngularJS radio input.\"),[\"disabled:b\",\"icon\",\"name\",\"ng-disabled:b\",\"ng-model\",\"ng-value\",\"value\"]),\"ion-refresher\":new Qt(zt(\"tags.ion.refresher\",\"Child of ionContent or ionScroll. Allows you to add pull-to-refresh to a scrollView. Place it as the first child of your ionContent or ionScroll element.\"),[\"disable-pulling-rotation:b\",\"on-pulling\",\"on-refresh\",\"pulling-icon\",\"pulling-text\",\"refreshing-icon\",\"spinner\"]),\"ion-reorder-button\":new Qt(zt(\"tags.ion.reorderbutton\",\"Child of ionItem.\"),[\"on-reorder\"]),\"ion-scroll\":new Qt(zt(\"tags.ion.scroll\",\"Creates a scrollable container for all content inside.\"),[\"delegate-handle\",\"direction:scrolldir\",\"has-bouncing:b\",\"locking:b\",\"max-zoom\",\"min-zoom\",\"on-refresh\",\"on-scroll\",\"paging:b\",\"scrollbar-x:b\",\"scrollbar-y:b\",\"zooming:b\"]),\"ion-side-menu\":new Qt(zt(\"tags.ion.sidemenu\",\"Child of ionSideMenus. A container for a side menu, sibling to an ionSideMenuContent directive.\"),[\"is-enabled:b\",\"expose-aside-when\",\"side:navsides\",\"width\"]),\"ion-side-menu-content\":new Qt(zt(\"tags.ion.sidemenucontent\",\"Child of ionSideMenus. A container for the main visible content, sibling to one or more ionSideMenu directives.\"),[\"drag-content:b\",\"edge-drag-threshold\"]),\"ion-side-menus\":new Qt(zt(\"tags.ion.sidemenus\",\"A container element for side menu(s) and the main content. Allows the left and/or right side menu to be toggled by dragging the main content area side to side.\"),[\"delegate-handle\",\"enable-menu-with-back-views:b\"]),\"ion-slide\":new Qt(zt(\"tags.ion.slide\",\"Child of ionSlideBox. Displays a slide inside of a slidebox.\"),[]),\"ion-slide-box\":new Qt(zt(\"tags.ion.slidebox\",\"The Slide Box is a multi-page container where each page can be swiped or dragged between.\"),[\"active-slide\",\"auto-play:b\",\"delegate-handle\",\"does-continue:b\",\"on-slide-changed\",\"pager-click\",\"show-pager:b\",\"slide-interval\"]),\"ion-spinner\":new Qt(zt(\"tags.ion.spinner\",\"The ionSpinner directive provides a variety of animated spinners.\"),[\"icon\"]),\"ion-tab\":new Qt(zt(\"tags.ion.tab\",\"Child of ionTabs. Contains a tab's content. The content only exists while the given tab is selected.\"),[\"badge\",\"badge-style\",\"disabled\",\"hidden\",\"href\",\"icon\",\"icon-off\",\"icon-on\",\"ng-click\",\"on-deselect\",\"on-select\",\"title\"]),\"ion-tabs\":new Qt(zt(\"tags.ion.tabs\",'Powers a multi-tabbed interface with a tab bar and a set of \"pages\" that can be tabbed through.'),[\"delegate-handle\"]),\"ion-title\":new Qt(zt(\"tags.ion.title\",\"ion-title is a component that sets the title of the ionNavbar\"),[]),\"ion-toggle\":new Qt(zt(\"tags.ion.toggle\",\"A toggle is an animated switch which binds a given model to a boolean. Allows dragging of the switch's nub. The toggle behaves like any AngularJS checkbox otherwise.\"),[\"name\",\"ng-false-value\",\"ng-model\",\"ng-true-value\",\"toggle-class\"]),\"ion-view \":new Qt(zt(\"tags.ion.view\",\"Child of ionNavView. A container for view content and any navigational and header bar information.\"),[\"cache-view:b\",\"can-swipe-back:b\",\"hide-back-button:b\",\"hide-nav-bar:b\",\"view-title\"])};function en(e,t){for(var n in t)e(n,t[n].label)}function tn(e,t,n,r){if(r.forEach(function(e){var n=e.split(\":\");t(n[0],n[1])}),e){var i=n[e];if(i){var o=i.attributes;o&&o.forEach(function(e){var n=e.split(\":\");t(n[0],n[1])})}}}function nn(e,t,n,r,i,o,a){var s=t+\":\",u=function(e){e.forEach(function(e){if(e.length>s.length&&Wt(e,s)){var r=e.substr(s.length);if(\"v\"===r)n(t);else{var i=o[r];i&&i.forEach(n)}}})};if(e){var l=r[e];if(l){var c=l.attributes;c&&u(c)}}if(u(i),a){var h=a[e];h&&u(h)}}var rn,on,an,sn,un,ln,cn,hn,dn,fn,pn,gn,mn,bn,_n,vn=function(){function e(e,t,n,r){this.start=e,this.end=t,this.children=n,this.parent=r,this.closed=!1}return Object.defineProperty(e.prototype,\"attributeNames\",{get:function(){return this.attributes?Object.keys(this.attributes):[]},enumerable:!0,configurable:!0}),e.prototype.isSameTag=function(e){return this.tag&&e&&this.tag.length===e.length&&this.tag.toLowerCase()===e},Object.defineProperty(e.prototype,\"firstChild\",{get:function(){return this.children[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"lastChild\",{get:function(){return this.children.length?this.children[this.children.length-1]:void 0},enumerable:!0,configurable:!0}),e.prototype.findNodeBefore=function(e){var t=Mt(this.children,function(t){return e<=t.start})-1;if(t>=0){var n=this.children[t];if(e>n.start){if(e<n.end)return n.findNodeBefore(e);var r=n.lastChild;return r&&r.end===n.end?n.findNodeBefore(e):n}}return this},e.prototype.findNodeAt=function(e){var t=Mt(this.children,function(t){return e<=t.start})-1;if(t>=0){var n=this.children[t];if(e>n.start&&e<=n.end)return n.findNodeAt(e)}return this},e}();!function(e){e.create=function(e,t){return{line:e,character:t}},e.is=function(e){var t=e;return Yn.objectLiteral(t)&&Yn.number(t.line)&&Yn.number(t.character)}}(rn||(rn={})),function(e){e.create=function(e,t,n,r){if(Yn.number(e)&&Yn.number(t)&&Yn.number(n)&&Yn.number(r))return{start:rn.create(e,t),end:rn.create(n,r)};if(rn.is(e)&&rn.is(t))return{start:e,end:t};throw new Error(\"Range#create called with invalid arguments[\"+e+\", \"+t+\", \"+n+\", \"+r+\"]\")},e.is=function(e){var t=e;return Yn.objectLiteral(t)&&rn.is(t.start)&&rn.is(t.end)}}(on||(on={})),function(e){e.create=function(e,t){return{uri:e,range:t}},e.is=function(e){var t=e;return Yn.defined(t)&&on.is(t.range)&&(Yn.string(t.uri)||Yn.undefined(t.uri))}}(an||(an={})),function(e){e.create=function(e,t,n,r){return{red:e,green:t,blue:n,alpha:r}},e.is=function(e){var t=e;return Yn.number(t.red)&&Yn.number(t.green)&&Yn.number(t.blue)&&Yn.number(t.alpha)}}(sn||(sn={})),function(e){e.create=function(e,t){return{range:e,color:t}},e.is=function(e){var t=e;return on.is(t.range)&&sn.is(t.color)}}(un||(un={})),function(e){e.create=function(e,t,n){return{label:e,textEdit:t,additionalTextEdits:n}},e.is=function(e){var t=e;return Yn.string(t.label)&&(Yn.undefined(t.textEdit)||mn.is(t))&&(Yn.undefined(t.additionalTextEdits)||Yn.typedArray(t.additionalTextEdits,mn.is))}}(ln||(ln={})),function(e){e.Comment=\"comment\",e.Imports=\"imports\",e.Region=\"region\"}(cn||(cn={})),function(e){e.create=function(e,t,n,r,i){var o={startLine:e,endLine:t};return Yn.defined(n)&&(o.startCharacter=n),Yn.defined(r)&&(o.endCharacter=r),Yn.defined(i)&&(o.kind=i),o},e.is=function(e){var t=e;return Yn.number(t.startLine)&&Yn.number(t.startLine)&&(Yn.undefined(t.startCharacter)||Yn.number(t.startCharacter))&&(Yn.undefined(t.endCharacter)||Yn.number(t.endCharacter))&&(Yn.undefined(t.kind)||Yn.string(t.kind))}}(hn||(hn={})),function(e){e.create=function(e,t){return{location:e,message:t}},e.is=function(e){var t=e;return Yn.defined(t)&&an.is(t.location)&&Yn.string(t.message)}}(dn||(dn={})),function(e){e.Error=1,e.Warning=2,e.Information=3,e.Hint=4}(fn||(fn={})),function(e){e.create=function(e,t,n,r,i,o){var a={range:e,message:t};return Yn.defined(n)&&(a.severity=n),Yn.defined(r)&&(a.code=r),Yn.defined(i)&&(a.source=i),Yn.defined(o)&&(a.relatedInformation=o),a},e.is=function(e){var t=e;return Yn.defined(t)&&on.is(t.range)&&Yn.string(t.message)&&(Yn.number(t.severity)||Yn.undefined(t.severity))&&(Yn.number(t.code)||Yn.string(t.code)||Yn.undefined(t.code))&&(Yn.string(t.source)||Yn.undefined(t.source))&&(Yn.undefined(t.relatedInformation)||Yn.typedArray(t.relatedInformation,dn.is))}}(pn||(pn={})),function(e){e.create=function(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];var i={title:e,command:t};return Yn.defined(n)&&n.length>0&&(i.arguments=n),i},e.is=function(e){var t=e;return Yn.defined(t)&&Yn.string(t.title)&&Yn.string(t.command)}}(gn||(gn={})),function(e){e.replace=function(e,t){return{range:e,newText:t}},e.insert=function(e,t){return{range:{start:e,end:e},newText:t}},e.del=function(e){return{range:e,newText:\"\"}},e.is=function(e){var t=e;return Yn.objectLiteral(t)&&Yn.string(t.newText)&&on.is(t.range)}}(mn||(mn={})),function(e){e.create=function(e,t){return{textDocument:e,edits:t}},e.is=function(e){var t=e;return Yn.defined(t)&&wn.is(t.textDocument)&&Array.isArray(t.edits)}}(bn||(bn={})),function(e){e.is=function(e){var t=e;return t&&(void 0!==t.changes||void 0!==t.documentChanges)&&(void 0===t.documentChanges||Yn.typedArray(t.documentChanges,bn.is))}}(_n||(_n={}));var yn,wn,Cn,Tn,En,xn,Sn,kn,An,Ln,Nn,On,Pn,In,qn,Rn,Dn,Un=function(){function e(e){this.edits=e}return e.prototype.insert=function(e,t){this.edits.push(mn.insert(e,t))},e.prototype.replace=function(e,t){this.edits.push(mn.replace(e,t))},e.prototype.delete=function(e){this.edits.push(mn.del(e))},e.prototype.add=function(e){this.edits.push(e)},e.prototype.all=function(){return this.edits},e.prototype.clear=function(){this.edits.splice(0,this.edits.length)},e}();!function(){function e(e){var t=this;this._textEditChanges=Object.create(null),e&&(this._workspaceEdit=e,e.documentChanges?e.documentChanges.forEach(function(e){var n=new Un(e.edits);t._textEditChanges[e.textDocument.uri]=n}):e.changes&&Object.keys(e.changes).forEach(function(n){var r=new Un(e.changes[n]);t._textEditChanges[n]=r}))}Object.defineProperty(e.prototype,\"edit\",{get:function(){return this._workspaceEdit},enumerable:!0,configurable:!0}),e.prototype.getTextEditChange=function(e){if(wn.is(e)){if(this._workspaceEdit||(this._workspaceEdit={documentChanges:[]}),!this._workspaceEdit.documentChanges)throw new Error(\"Workspace edit is not configured for versioned document changes.\");var t=e;if(!(r=this._textEditChanges[t.uri])){var n={textDocument:t,edits:i=[]};this._workspaceEdit.documentChanges.push(n),r=new Un(i),this._textEditChanges[t.uri]=r}return r}if(this._workspaceEdit||(this._workspaceEdit={changes:Object.create(null)}),!this._workspaceEdit.changes)throw new Error(\"Workspace edit is not configured for normal text edit changes.\");var r;if(!(r=this._textEditChanges[e])){var i=[];this._workspaceEdit.changes[e]=i,r=new Un(i),this._textEditChanges[e]=r}return r}}();!function(e){e.create=function(e){return{uri:e}},e.is=function(e){var t=e;return Yn.defined(t)&&Yn.string(t.uri)}}(yn||(yn={})),function(e){e.create=function(e,t){return{uri:e,version:t}},e.is=function(e){var t=e;return Yn.defined(t)&&Yn.string(t.uri)&&Yn.number(t.version)}}(wn||(wn={})),function(e){e.create=function(e,t,n,r){return{uri:e,languageId:t,version:n,text:r}},e.is=function(e){var t=e;return Yn.defined(t)&&Yn.string(t.uri)&&Yn.string(t.languageId)&&Yn.number(t.version)&&Yn.string(t.text)}}(Cn||(Cn={})),function(e){e.PlainText=\"plaintext\",e.Markdown=\"markdown\"}(Tn||(Tn={})),function(e){e.is=function(t){var n=t;return n===e.PlainText||n===e.Markdown}}(Tn||(Tn={})),function(e){e.is=function(e){var t=e;return Yn.objectLiteral(e)&&Tn.is(t.kind)&&Yn.string(t.value)}}(En||(En={})),function(e){e.Text=1,e.Method=2,e.Function=3,e.Constructor=4,e.Field=5,e.Variable=6,e.Class=7,e.Interface=8,e.Module=9,e.Property=10,e.Unit=11,e.Value=12,e.Enum=13,e.Keyword=14,e.Snippet=15,e.Color=16,e.File=17,e.Reference=18,e.Folder=19,e.EnumMember=20,e.Constant=21,e.Struct=22,e.Event=23,e.Operator=24,e.TypeParameter=25}(xn||(xn={})),function(e){e.PlainText=1,e.Snippet=2}(Sn||(Sn={})),function(e){e.create=function(e){return{label:e}}}(kn||(kn={})),function(e){e.create=function(e,t){return{items:e||[],isIncomplete:!!t}}}(An||(An={})),function(e){e.fromPlainText=function(e){return e.replace(/[\\\\`*_{}[\\]()#+\\-.!]/g,\"\\\\$&\")},e.is=function(e){var t=e;return Yn.string(t)||Yn.objectLiteral(t)&&Yn.string(t.language)&&Yn.string(t.value)}}(Ln||(Ln={})),function(e){e.is=function(e){var t=e;return Yn.objectLiteral(t)&&(En.is(t.contents)||Ln.is(t.contents)||Yn.typedArray(t.contents,Ln.is))&&(void 0===e.range||on.is(e.range))}}(Nn||(Nn={})),function(e){e.create=function(e,t){return t?{label:e,documentation:t}:{label:e}}}(On||(On={})),function(e){e.create=function(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];var i={label:e};return Yn.defined(t)&&(i.documentation=t),Yn.defined(n)?i.parameters=n:i.parameters=[],i}}(Pn||(Pn={})),function(e){e.Text=1,e.Read=2,e.Write=3}(In||(In={})),function(e){e.create=function(e,t){var n={range:e};return Yn.number(t)&&(n.kind=t),n}}(qn||(qn={})),function(e){e.File=1,e.Module=2,e.Namespace=3,e.Package=4,e.Class=5,e.Method=6,e.Property=7,e.Field=8,e.Constructor=9,e.Enum=10,e.Interface=11,e.Function=12,e.Variable=13,e.Constant=14,e.String=15,e.Number=16,e.Boolean=17,e.Array=18,e.Object=19,e.Key=20,e.Null=21,e.EnumMember=22,e.Struct=23,e.Event=24,e.Operator=25,e.TypeParameter=26}(Rn||(Rn={})),function(e){e.create=function(e,t,n,r,i){var o={name:e,kind:t,location:{uri:r,range:n}};return i&&(o.containerName=i),o}}(Dn||(Dn={}));var Mn,Wn,Vn,Bn,jn,Fn=function(){return function(){}}();!function(e){e.create=function(e,t,n,r,i,o){var a={name:e,detail:t,kind:n,range:r,selectionRange:i};return void 0!==o&&(a.children=o),a},e.is=function(e){var t=e;return t&&Yn.string(t.name)&&Yn.string(t.detail)&&Yn.number(t.kind)&&on.is(t.range)&&on.is(t.selectionRange)&&(void 0===t.deprecated||Yn.boolean(t.deprecated))&&(void 0===t.children||Array.isArray(t.children))}}(Fn||(Fn={})),function(e){e.QuickFix=\"quickfix\",e.Refactor=\"refactor\",e.RefactorExtract=\"refactor.extract\",e.RefactorInline=\"refactor.inline\",e.RefactorRewrite=\"refactor.rewrite\",e.Source=\"source\",e.SourceOrganizeImports=\"source.organizeImports\"}(Mn||(Mn={})),function(e){e.create=function(e,t){var n={diagnostics:e};return null!=t&&(n.only=t),n},e.is=function(e){var t=e;return Yn.defined(t)&&Yn.typedArray(t.diagnostics,pn.is)&&(void 0===t.only||Yn.typedArray(t.only,Yn.string))}}(Wn||(Wn={})),function(e){e.create=function(e,t,n){var r={title:e};return gn.is(t)?r.command=t:r.edit=t,void 0!==n&&(r.kind=n),r},e.is=function(e){var t=e;return t&&Yn.string(t.title)&&(void 0===t.diagnostics||Yn.typedArray(t.diagnostics,pn.is))&&(void 0===t.kind||Yn.string(t.kind))&&(void 0!==t.edit||void 0!==t.command)&&(void 0===t.command||gn.is(t.command))&&(void 0===t.edit||_n.is(t.edit))}}(Vn||(Vn={})),function(e){e.create=function(e,t){var n={range:e};return Yn.defined(t)&&(n.data=t),n},e.is=function(e){var t=e;return Yn.defined(t)&&on.is(t.range)&&(Yn.undefined(t.command)||gn.is(t.command))}}(Bn||(Bn={})),function(e){e.create=function(e,t){return{tabSize:e,insertSpaces:t}},e.is=function(e){var t=e;return Yn.defined(t)&&Yn.number(t.tabSize)&&Yn.boolean(t.insertSpaces)}}(jn||(jn={}));var Kn,Hn,Gn=function(){return function(){}}();!function(e){e.create=function(e,t,n){return{range:e,target:t,data:n}},e.is=function(e){var t=e;return Yn.defined(t)&&on.is(t.range)&&(Yn.undefined(t.target)||Yn.string(t.target))}}(Gn||(Gn={})),function(e){e.create=function(e,t,n,r){return new zn(e,t,n,r)},e.is=function(e){var t=e;return!!(Yn.defined(t)&&Yn.string(t.uri)&&(Yn.undefined(t.languageId)||Yn.string(t.languageId))&&Yn.number(t.lineCount)&&Yn.func(t.getText)&&Yn.func(t.positionAt)&&Yn.func(t.offsetAt))},e.applyEdits=function(e,t){for(var n=e.getText(),r=function e(t,n){if(t.length<=1)return t;var r=t.length/2|0,i=t.slice(0,r),o=t.slice(r);e(i,n),e(o,n);for(var a=0,s=0,u=0;a<i.length&&s<o.length;){var l=n(i[a],o[s]);t[u++]=l<=0?i[a++]:o[s++]}for(;a<i.length;)t[u++]=i[a++];for(;s<o.length;)t[u++]=o[s++];return t}(t,function(e,t){var n=e.range.start.line-t.range.start.line;return 0===n?e.range.start.character-t.range.start.character:n}),i=n.length,o=r.length-1;o>=0;o--){var a=r[o],s=e.offsetAt(a.range.start),u=e.offsetAt(a.range.end);if(!(u<=i))throw new Error(\"Ovelapping edit\");n=n.substring(0,s)+a.newText+n.substring(u,n.length),i=s}return n}}(Kn||(Kn={})),function(e){e.Manual=1,e.AfterDelay=2,e.FocusOut=3}(Hn||(Hn={}));var Yn,zn=function(){function e(e,t,n,r){this._uri=e,this._languageId=t,this._version=n,this._content=r,this._lineOffsets=null}return Object.defineProperty(e.prototype,\"uri\",{get:function(){return this._uri},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"languageId\",{get:function(){return this._languageId},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"version\",{get:function(){return this._version},enumerable:!0,configurable:!0}),e.prototype.getText=function(e){if(e){var t=this.offsetAt(e.start),n=this.offsetAt(e.end);return this._content.substring(t,n)}return this._content},e.prototype.update=function(e,t){this._content=e.text,this._version=t,this._lineOffsets=null},e.prototype.getLineOffsets=function(){if(null===this._lineOffsets){for(var e=[],t=this._content,n=!0,r=0;r<t.length;r++){n&&(e.push(r),n=!1);var i=t.charAt(r);n=\"\\r\"===i||\"\\n\"===i,\"\\r\"===i&&r+1<t.length&&\"\\n\"===t.charAt(r+1)&&r++}n&&t.length>0&&e.push(t.length),this._lineOffsets=e}return this._lineOffsets},e.prototype.positionAt=function(e){e=Math.max(Math.min(e,this._content.length),0);var t=this.getLineOffsets(),n=0,r=t.length;if(0===r)return rn.create(0,e);for(;n<r;){var i=Math.floor((n+r)/2);t[i]>e?r=i:n=i+1}var o=n-1;return rn.create(o,e-t[o])},e.prototype.offsetAt=function(e){var t=this.getLineOffsets();if(e.line>=t.length)return this._content.length;if(e.line<0)return 0;var n=t[e.line],r=e.line+1<t.length?t[e.line+1]:this._content.length;return Math.max(Math.min(n+e.character,r),n)},Object.defineProperty(e.prototype,\"lineCount\",{get:function(){return this.getLineOffsets().length},enumerable:!0,configurable:!0}),e}();!function(e){var t=Object.prototype.toString;e.defined=function(e){return void 0!==e},e.undefined=function(e){return void 0===e},e.boolean=function(e){return!0===e||!1===e},e.string=function(e){return\"[object String]\"===t.call(e)},e.number=function(e){return\"[object Number]\"===t.call(e)},e.func=function(e){return\"[object Function]\"===t.call(e)},e.objectLiteral=function(e){return null!==e&&\"object\"==typeof e},e.typedArray=function(e,t){return Array.isArray(e)&&e.every(t)}}(Yn||(Yn={}));var Jn,$n,Qn,Zn,Xn=[($n=[\"aria-activedescendant\",\"aria-atomic:b\",\"aria-autocomplete:autocomplete\",\"aria-busy:b\",\"aria-checked:tristate\",\"aria-colcount\",\"aria-colindex\",\"aria-colspan\",\"aria-controls\",\"aria-current:current\",\"aria-describedat\",\"aria-describedby\",\"aria-disabled:b\",\"aria-dropeffect:dropeffect\",\"aria-errormessage\",\"aria-expanded:u\",\"aria-flowto\",\"aria-grabbed:u\",\"aria-haspopup:b\",\"aria-hidden:b\",\"aria-invalid:invalid\",\"aria-kbdshortcuts\",\"aria-label\",\"aria-labelledby\",\"aria-level\",\"aria-live:live\",\"aria-modal:b\",\"aria-multiline:b\",\"aria-multiselectable:b\",\"aria-orientation:orientation\",\"aria-owns\",\"aria-placeholder\",\"aria-posinset\",\"aria-pressed:tristate\",\"aria-readonly:b\",\"aria-relevant:relevant\",\"aria-required:b\",\"aria-roledescription\",\"aria-rowcount\",\"aria-rowindex\",\"aria-rowspan\",\"aria-selected:u\",\"aria-setsize\",\"aria-sort:sort\",\"aria-valuemax\",\"aria-valuemin\",\"aria-valuenow\",\"aria-valuetext\",\"accesskey\",\"class\",\"contenteditable:b\",\"contextmenu\",\"dir:d\",\"draggable:b\",\"dropzone\",\"hidden:v\",\"id\",\"itemid\",\"itemprop\",\"itemref\",\"itemscope:v\",\"itemtype\",\"lang\",\"role:roles\",\"spellcheck:b\",\"style\",\"tabindex\",\"title\",\"translate:y\"],Qn=[\"onabort\",\"onblur\",\"oncanplay\",\"oncanplaythrough\",\"onchange\",\"onclick\",\"oncontextmenu\",\"ondblclick\",\"ondrag\",\"ondragend\",\"ondragenter\",\"ondragleave\",\"ondragover\",\"ondragstart\",\"ondrop\",\"ondurationchange\",\"onemptied\",\"onended\",\"onerror\",\"onfocus\",\"onformchange\",\"onforminput\",\"oninput\",\"oninvalid\",\"onkeydown\",\"onkeypress\",\"onkeyup\",\"onload\",\"onloadeddata\",\"onloadedmetadata\",\"onloadstart\",\"onmousedown\",\"onmousemove\",\"onmouseout\",\"onmouseover\",\"onmouseup\",\"onmousewheel\",\"onpause\",\"onplay\",\"onplaying\",\"onprogress\",\"onratechange\",\"onreset\",\"onresize\",\"onreadystatechange\",\"onscroll\",\"onseeked\",\"onseeking\",\"onselect\",\"onshow\",\"onstalled\",\"onsubmit\",\"onsuspend\",\"ontimeupdate\",\"onvolumechange\",\"onwaiting\"],Zn={b:[\"true\",\"false\"],u:[\"true\",\"false\",\"undefined\"],o:[\"on\",\"off\"],y:[\"yes\",\"no\"],w:[\"soft\",\"hard\"],d:[\"ltr\",\"rtl\",\"auto\"],m:[\"GET\",\"POST\",\"dialog\"],fm:[\"GET\",\"POST\"],s:[\"row\",\"col\",\"rowgroup\",\"colgroup\"],t:[\"hidden\",\"text\",\"search\",\"tel\",\"url\",\"email\",\"password\",\"datetime\",\"date\",\"month\",\"week\",\"time\",\"datetime-local\",\"number\",\"range\",\"color\",\"checkbox\",\"radio\",\"file\",\"submit\",\"image\",\"reset\",\"button\"],im:[\"verbatim\",\"latin\",\"latin-name\",\"latin-prose\",\"full-width-latin\",\"kana\",\"kana-name\",\"katakana\",\"numeric\",\"tel\",\"email\",\"url\"],bt:[\"button\",\"submit\",\"reset\",\"menu\"],lt:[\"1\",\"a\",\"A\",\"i\",\"I\"],mt:[\"context\",\"toolbar\"],mit:[\"command\",\"checkbox\",\"radio\"],et:[\"application/x-www-form-urlencoded\",\"multipart/form-data\",\"text/plain\"],tk:[\"subtitles\",\"captions\",\"descriptions\",\"chapters\",\"metadata\"],pl:[\"none\",\"metadata\",\"auto\"],sh:[\"circle\",\"default\",\"poly\",\"rect\"],xo:[\"anonymous\",\"use-credentials\"],sb:[\"allow-forms\",\"allow-modals\",\"allow-pointer-lock\",\"allow-popups\",\"allow-popups-to-escape-sandbox\",\"allow-same-origin\",\"allow-scripts\",\"allow-top-navigation\"],tristate:[\"true\",\"false\",\"mixed\",\"undefined\"],inputautocomplete:[\"additional-name\",\"address-level1\",\"address-level2\",\"address-level3\",\"address-level4\",\"address-line1\",\"address-line2\",\"address-line3\",\"bday\",\"bday-year\",\"bday-day\",\"bday-month\",\"billing\",\"cc-additional-name\",\"cc-csc\",\"cc-exp\",\"cc-exp-month\",\"cc-exp-year\",\"cc-family-name\",\"cc-given-name\",\"cc-name\",\"cc-number\",\"cc-type\",\"country\",\"country-name\",\"current-password\",\"email\",\"family-name\",\"fax\",\"given-name\",\"home\",\"honorific-prefix\",\"honorific-suffix\",\"impp\",\"language\",\"mobile\",\"name\",\"new-password\",\"nickname\",\"organization\",\"organization-title\",\"pager\",\"photo\",\"postal-code\",\"sex\",\"shipping\",\"street-address\",\"tel-area-code\",\"tel\",\"tel-country-code\",\"tel-extension\",\"tel-local\",\"tel-local-prefix\",\"tel-local-suffix\",\"tel-national\",\"transaction-amount\",\"transaction-currency\",\"url\",\"username\",\"work\"],autocomplete:[\"inline\",\"list\",\"both\",\"none\"],current:[\"page\",\"step\",\"location\",\"date\",\"time\",\"true\",\"false\"],dropeffect:[\"copy\",\"move\",\"link\",\"execute\",\"popup\",\"none\"],invalid:[\"grammar\",\"false\",\"spelling\",\"true\"],live:[\"off\",\"polite\",\"assertive\"],orientation:[\"vertical\",\"horizontal\",\"undefined\"],relevant:[\"additions\",\"removals\",\"text\",\"all\",\"additions text\"],sort:[\"ascending\",\"descending\",\"none\",\"other\"],roles:[\"alert\",\"alertdialog\",\"button\",\"checkbox\",\"dialog\",\"gridcell\",\"link\",\"log\",\"marquee\",\"menuitem\",\"menuitemcheckbox\",\"menuitemradio\",\"option\",\"progressbar\",\"radio\",\"scrollbar\",\"searchbox\",\"slider\",\"spinbutton\",\"status\",\"switch\",\"tab\",\"tabpanel\",\"textbox\",\"timer\",\"tooltip\",\"treeitem\",\"combobox\",\"grid\",\"listbox\",\"menu\",\"menubar\",\"radiogroup\",\"tablist\",\"tree\",\"treegrid\",\"application\",\"article\",\"cell\",\"columnheader\",\"definition\",\"directory\",\"document\",\"feed\",\"figure\",\"group\",\"heading\",\"img\",\"list\",\"listitem\",\"math\",\"none\",\"note\",\"presentation\",\"region\",\"row\",\"rowgroup\",\"rowheader\",\"separator\",\"table\",\"term\",\"text\",\"toolbar\",\"banner\",\"complementary\",\"contentinfo\",\"form\",\"main\",\"navigation\",\"region\",\"search\",\"doc-abstract\",\"doc-acknowledgments\",\"doc-afterword\",\"doc-appendix\",\"doc-backlink\",\"doc-biblioentry\",\"doc-bibliography\",\"doc-biblioref\",\"doc-chapter\",\"doc-colophon\",\"doc-conclusion\",\"doc-cover\",\"doc-credit\",\"doc-credits\",\"doc-dedication\",\"doc-endnote\",\"doc-endnotes\",\"doc-epigraph\",\"doc-epilogue\",\"doc-errata\",\"doc-example\",\"doc-footnote\",\"doc-foreword\",\"doc-glossary\",\"doc-glossref\",\"doc-index\",\"doc-introduction\",\"doc-noteref\",\"doc-notice\",\"doc-pagebreak\",\"doc-pagelist\",\"doc-part\",\"doc-preface\",\"doc-prologue\",\"doc-pullquote\",\"doc-qna\",\"doc-subtitle\",\"doc-tip\",\"doc-toc\"]},{getId:function(){return\"html5\"},isApplicable:function(){return!0},collectTags:function(e){return en(e,Zt)},collectAttributes:function(e,t){tn(e,t,Zt,$n),Qn.forEach(function(e){t(e,\"event\")})},collectValues:function(e,t,n){return nn(e,t,n,Zt,$n,Zn)}}),function(){var e={input:[\"ng-model\",\"ng-required\",\"ng-minlength\",\"ng-maxlength\",\"ng-pattern\",\"ng-trim\"],select:[\"ng-model\"],textarea:[\"ng-model\",\"ng-required\",\"ng-minlength\",\"ng-maxlength\",\"ng-pattern\",\"ng-trim\"]},t=[\"ng-app\",\"ng-strict-di\",\"ng-bind\",\"ng-bind-html\",\"ng-bind-template\",\"ng-blur\",\"ng-change\",\"ng-checked\",\"ng-class\",\"ng-class-even\",\"ng-class-odd\",\"ng-click\",\"ng-cloak\",\"ng-controller\",\"ng-copy\",\"ng-csp\",\"ng-cut\",\"ng-dblclick\",\"ng-disabled\",\"ng-focus\",\"ng-form\",\"ng-hide\",\"ng-href\",\"ng-if\",\"ng-include\",\"ng-init\",\"ng-jq\",\"ng-keydown\",\"ng-keypress\",\"ng-keyup\",\"ng-list\",\"ng-model-options\",\"ng-mousedown\",\"ng-mouseenter\",\"ng-mouseleave\",\"ng-mousemove\",\"ng-mouseover\",\"ng-mouseup\",\"ng-non-bindable\",\"ng-open\",\"ng-options\",\"ng-paste\",\"ng-pluralize\",\"ng-readonly\",\"ng-repeat\",\"ng-selected\",\"ng-show\",\"ng-src\",\"ng-srcset\",\"ng-style\",\"ng-submit\",\"ng-switch\",\"ng-transclude\",\"ng-value\"];return{getId:function(){return\"angular1\"},isApplicable:function(e){return\"html\"===e},collectTags:function(e){},collectAttributes:function(n,r){if(n){var i=e[n];i&&i.forEach(function(e){r(e),r(\"data-\"+e)})}t.forEach(function(e){r(e),r(\"data-\"+e)})},collectValues:function(e,t,n){}}}(),function(){var e={a:[\"nav-direction:navdir\",\"nav-transition:trans\"],button:[\"menu-toggle:menusides\"]},t=[\"collection-repeat\",\"force-refresh-images:b\",\"ion-stop-event\",\"item-height\",\"item-render-buffer\",\"item-width\",\"menu-close:v\",\"on-double-tap\",\"on-drag\",\"on-drag-down\",\"on-drag-left\",\"on-drag-right\",\"on-drag-up\",\"on-hold\",\"on-release\",\"on-swipe\",\"on-swipe-down\",\"on-swipe-left\",\"on-swipe-right\",\"on-swipe-up\",\"on-tap\",\"on-touch\"],n={align:[\"center\",\"left\",\"right\"],b:[\"true\",\"false\"],inputtype:[\"email\",\"number\",\"password\",\"search\",\"tel\",\"text\",\"url\"],listtype:[\"card\",\"list-inset\"],menusides:[\"left\",\"right\"],navdir:[\"back\",\"enter\",\"exit\",\"forward\",\"swap\"],navsides:[\"left\",\"primary\",\"right\",\"secondary\"],scrolldir:[\"x\",\"xy\",\"y\"],trans:[\"android\",\"ios\",\"none\"]};return{getId:function(){return\"ionic\"},isApplicable:function(e){return\"html\"===e},collectTags:function(e){return en(e,Xt)},collectAttributes:function(n,r){if(tn(n,r,Xt,t),n){var i=e[n];i&&i.forEach(function(e){var t=e.split(\":\");r(t[0],t[1])})}},collectValues:function(r,i,o){return nn(r,i,o,Xt,t,n,e)}}}(),(Jn={a:[\"asp-action\",\"asp-controller\",\"asp-fragment\",\"asp-host\",\"asp-protocol\",\"asp-route\"],div:[\"asp-validation-summary\"],form:[\"asp-action\",\"asp-controller\",\"asp-anti-forgery\"],input:[\"asp-for\",\"asp-format\"],label:[\"asp-for\"],select:[\"asp-for\",\"asp-items\"],span:[\"asp-validation-for\"]},{getId:function(){return\"razor\"},isApplicable:function(e){return\"razor\"===e},collectTags:function(e){},collectAttributes:function(e,t){if(e){var n=Jn[e];n&&n.forEach(function(e){return t(e)})}},collectValues:function(e,t,n){}})],er={\"Aacute;\":\"Á\",Aacute:\"Á\",\"aacute;\":\"á\",aacute:\"á\",\"Abreve;\":\"Ă\",\"abreve;\":\"ă\",\"ac;\":\"∾\",\"acd;\":\"∿\",\"acE;\":\"∾̳\",\"Acirc;\":\"Â\",Acirc:\"Â\",\"acirc;\":\"â\",acirc:\"â\",\"acute;\":\"´\",acute:\"´\",\"Acy;\":\"А\",\"acy;\":\"а\",\"AElig;\":\"Æ\",AElig:\"Æ\",\"aelig;\":\"æ\",aelig:\"æ\",\"af;\":\"⁡\",\"Afr;\":\"𝔄\",\"afr;\":\"𝔞\",\"Agrave;\":\"À\",Agrave:\"À\",\"agrave;\":\"à\",agrave:\"à\",\"alefsym;\":\"ℵ\",\"aleph;\":\"ℵ\",\"Alpha;\":\"Α\",\"alpha;\":\"α\",\"Amacr;\":\"Ā\",\"amacr;\":\"ā\",\"amalg;\":\"⨿\",\"AMP;\":\"&\",AMP:\"&\",\"amp;\":\"&\",amp:\"&\",\"And;\":\"⩓\",\"and;\":\"∧\",\"andand;\":\"⩕\",\"andd;\":\"⩜\",\"andslope;\":\"⩘\",\"andv;\":\"⩚\",\"ang;\":\"∠\",\"ange;\":\"⦤\",\"angle;\":\"∠\",\"angmsd;\":\"∡\",\"angmsdaa;\":\"⦨\",\"angmsdab;\":\"⦩\",\"angmsdac;\":\"⦪\",\"angmsdad;\":\"⦫\",\"angmsdae;\":\"⦬\",\"angmsdaf;\":\"⦭\",\"angmsdag;\":\"⦮\",\"angmsdah;\":\"⦯\",\"angrt;\":\"∟\",\"angrtvb;\":\"⊾\",\"angrtvbd;\":\"⦝\",\"angsph;\":\"∢\",\"angst;\":\"Å\",\"angzarr;\":\"⍼\",\"Aogon;\":\"Ą\",\"aogon;\":\"ą\",\"Aopf;\":\"𝔸\",\"aopf;\":\"𝕒\",\"ap;\":\"≈\",\"apacir;\":\"⩯\",\"apE;\":\"⩰\",\"ape;\":\"≊\",\"apid;\":\"≋\",\"apos;\":\"'\",\"ApplyFunction;\":\"⁡\",\"approx;\":\"≈\",\"approxeq;\":\"≊\",\"Aring;\":\"Å\",Aring:\"Å\",\"aring;\":\"å\",aring:\"å\",\"Ascr;\":\"𝒜\",\"ascr;\":\"𝒶\",\"Assign;\":\"≔\",\"ast;\":\"*\",\"asymp;\":\"≈\",\"asympeq;\":\"≍\",\"Atilde;\":\"Ã\",Atilde:\"Ã\",\"atilde;\":\"ã\",atilde:\"ã\",\"Auml;\":\"Ä\",Auml:\"Ä\",\"auml;\":\"ä\",auml:\"ä\",\"awconint;\":\"∳\",\"awint;\":\"⨑\",\"backcong;\":\"≌\",\"backepsilon;\":\"϶\",\"backprime;\":\"‵\",\"backsim;\":\"∽\",\"backsimeq;\":\"⋍\",\"Backslash;\":\"∖\",\"Barv;\":\"⫧\",\"barvee;\":\"⊽\",\"Barwed;\":\"⌆\",\"barwed;\":\"⌅\",\"barwedge;\":\"⌅\",\"bbrk;\":\"⎵\",\"bbrktbrk;\":\"⎶\",\"bcong;\":\"≌\",\"Bcy;\":\"Б\",\"bcy;\":\"б\",\"bdquo;\":\"„\",\"becaus;\":\"∵\",\"Because;\":\"∵\",\"because;\":\"∵\",\"bemptyv;\":\"⦰\",\"bepsi;\":\"϶\",\"bernou;\":\"ℬ\",\"Bernoullis;\":\"ℬ\",\"Beta;\":\"Β\",\"beta;\":\"β\",\"beth;\":\"ℶ\",\"between;\":\"≬\",\"Bfr;\":\"𝔅\",\"bfr;\":\"𝔟\",\"bigcap;\":\"⋂\",\"bigcirc;\":\"◯\",\"bigcup;\":\"⋃\",\"bigodot;\":\"⨀\",\"bigoplus;\":\"⨁\",\"bigotimes;\":\"⨂\",\"bigsqcup;\":\"⨆\",\"bigstar;\":\"★\",\"bigtriangledown;\":\"▽\",\"bigtriangleup;\":\"△\",\"biguplus;\":\"⨄\",\"bigvee;\":\"⋁\",\"bigwedge;\":\"⋀\",\"bkarow;\":\"⤍\",\"blacklozenge;\":\"⧫\",\"blacksquare;\":\"▪\",\"blacktriangle;\":\"▴\",\"blacktriangledown;\":\"▾\",\"blacktriangleleft;\":\"◂\",\"blacktriangleright;\":\"▸\",\"blank;\":\"␣\",\"blk12;\":\"▒\",\"blk14;\":\"░\",\"blk34;\":\"▓\",\"block;\":\"█\",\"bne;\":\"=⃥\",\"bnequiv;\":\"≡⃥\",\"bNot;\":\"⫭\",\"bnot;\":\"⌐\",\"Bopf;\":\"𝔹\",\"bopf;\":\"𝕓\",\"bot;\":\"⊥\",\"bottom;\":\"⊥\",\"bowtie;\":\"⋈\",\"boxbox;\":\"⧉\",\"boxDL;\":\"╗\",\"boxDl;\":\"╖\",\"boxdL;\":\"╕\",\"boxdl;\":\"┐\",\"boxDR;\":\"╔\",\"boxDr;\":\"╓\",\"boxdR;\":\"╒\",\"boxdr;\":\"┌\",\"boxH;\":\"═\",\"boxh;\":\"─\",\"boxHD;\":\"╦\",\"boxHd;\":\"╤\",\"boxhD;\":\"╥\",\"boxhd;\":\"┬\",\"boxHU;\":\"╩\",\"boxHu;\":\"╧\",\"boxhU;\":\"╨\",\"boxhu;\":\"┴\",\"boxminus;\":\"⊟\",\"boxplus;\":\"⊞\",\"boxtimes;\":\"⊠\",\"boxUL;\":\"╝\",\"boxUl;\":\"╜\",\"boxuL;\":\"╛\",\"boxul;\":\"┘\",\"boxUR;\":\"╚\",\"boxUr;\":\"╙\",\"boxuR;\":\"╘\",\"boxur;\":\"└\",\"boxV;\":\"║\",\"boxv;\":\"│\",\"boxVH;\":\"╬\",\"boxVh;\":\"╫\",\"boxvH;\":\"╪\",\"boxvh;\":\"┼\",\"boxVL;\":\"╣\",\"boxVl;\":\"╢\",\"boxvL;\":\"╡\",\"boxvl;\":\"┤\",\"boxVR;\":\"╠\",\"boxVr;\":\"╟\",\"boxvR;\":\"╞\",\"boxvr;\":\"├\",\"bprime;\":\"‵\",\"Breve;\":\"˘\",\"breve;\":\"˘\",\"brvbar;\":\"¦\",brvbar:\"¦\",\"Bscr;\":\"ℬ\",\"bscr;\":\"𝒷\",\"bsemi;\":\"⁏\",\"bsim;\":\"∽\",\"bsime;\":\"⋍\",\"bsol;\":\"\\\\\",\"bsolb;\":\"⧅\",\"bsolhsub;\":\"⟈\",\"bull;\":\"•\",\"bullet;\":\"•\",\"bump;\":\"≎\",\"bumpE;\":\"⪮\",\"bumpe;\":\"≏\",\"Bumpeq;\":\"≎\",\"bumpeq;\":\"≏\",\"Cacute;\":\"Ć\",\"cacute;\":\"ć\",\"Cap;\":\"⋒\",\"cap;\":\"∩\",\"capand;\":\"⩄\",\"capbrcup;\":\"⩉\",\"capcap;\":\"⩋\",\"capcup;\":\"⩇\",\"capdot;\":\"⩀\",\"CapitalDifferentialD;\":\"ⅅ\",\"caps;\":\"∩︀\",\"caret;\":\"⁁\",\"caron;\":\"ˇ\",\"Cayleys;\":\"ℭ\",\"ccaps;\":\"⩍\",\"Ccaron;\":\"Č\",\"ccaron;\":\"č\",\"Ccedil;\":\"Ç\",Ccedil:\"Ç\",\"ccedil;\":\"ç\",ccedil:\"ç\",\"Ccirc;\":\"Ĉ\",\"ccirc;\":\"ĉ\",\"Cconint;\":\"∰\",\"ccups;\":\"⩌\",\"ccupssm;\":\"⩐\",\"Cdot;\":\"Ċ\",\"cdot;\":\"ċ\",\"cedil;\":\"¸\",cedil:\"¸\",\"Cedilla;\":\"¸\",\"cemptyv;\":\"⦲\",\"cent;\":\"¢\",cent:\"¢\",\"CenterDot;\":\"·\",\"centerdot;\":\"·\",\"Cfr;\":\"ℭ\",\"cfr;\":\"𝔠\",\"CHcy;\":\"Ч\",\"chcy;\":\"ч\",\"check;\":\"✓\",\"checkmark;\":\"✓\",\"Chi;\":\"Χ\",\"chi;\":\"χ\",\"cir;\":\"○\",\"circ;\":\"ˆ\",\"circeq;\":\"≗\",\"circlearrowleft;\":\"↺\",\"circlearrowright;\":\"↻\",\"circledast;\":\"⊛\",\"circledcirc;\":\"⊚\",\"circleddash;\":\"⊝\",\"CircleDot;\":\"⊙\",\"circledR;\":\"®\",\"circledS;\":\"Ⓢ\",\"CircleMinus;\":\"⊖\",\"CirclePlus;\":\"⊕\",\"CircleTimes;\":\"⊗\",\"cirE;\":\"⧃\",\"cire;\":\"≗\",\"cirfnint;\":\"⨐\",\"cirmid;\":\"⫯\",\"cirscir;\":\"⧂\",\"ClockwiseContourIntegral;\":\"∲\",\"CloseCurlyDoubleQuote;\":\"”\",\"CloseCurlyQuote;\":\"’\",\"clubs;\":\"♣\",\"clubsuit;\":\"♣\",\"Colon;\":\"∷\",\"colon;\":\":\",\"Colone;\":\"⩴\",\"colone;\":\"≔\",\"coloneq;\":\"≔\",\"comma;\":\",\",\"commat;\":\"@\",\"comp;\":\"∁\",\"compfn;\":\"∘\",\"complement;\":\"∁\",\"complexes;\":\"ℂ\",\"cong;\":\"≅\",\"congdot;\":\"⩭\",\"Congruent;\":\"≡\",\"Conint;\":\"∯\",\"conint;\":\"∮\",\"ContourIntegral;\":\"∮\",\"Copf;\":\"ℂ\",\"copf;\":\"𝕔\",\"coprod;\":\"∐\",\"Coproduct;\":\"∐\",\"COPY;\":\"©\",COPY:\"©\",\"copy;\":\"©\",copy:\"©\",\"copysr;\":\"℗\",\"CounterClockwiseContourIntegral;\":\"∳\",\"crarr;\":\"↵\",\"Cross;\":\"⨯\",\"cross;\":\"✗\",\"Cscr;\":\"𝒞\",\"cscr;\":\"𝒸\",\"csub;\":\"⫏\",\"csube;\":\"⫑\",\"csup;\":\"⫐\",\"csupe;\":\"⫒\",\"ctdot;\":\"⋯\",\"cudarrl;\":\"⤸\",\"cudarrr;\":\"⤵\",\"cuepr;\":\"⋞\",\"cuesc;\":\"⋟\",\"cularr;\":\"↶\",\"cularrp;\":\"⤽\",\"Cup;\":\"⋓\",\"cup;\":\"∪\",\"cupbrcap;\":\"⩈\",\"CupCap;\":\"≍\",\"cupcap;\":\"⩆\",\"cupcup;\":\"⩊\",\"cupdot;\":\"⊍\",\"cupor;\":\"⩅\",\"cups;\":\"∪︀\",\"curarr;\":\"↷\",\"curarrm;\":\"⤼\",\"curlyeqprec;\":\"⋞\",\"curlyeqsucc;\":\"⋟\",\"curlyvee;\":\"⋎\",\"curlywedge;\":\"⋏\",\"curren;\":\"¤\",curren:\"¤\",\"curvearrowleft;\":\"↶\",\"curvearrowright;\":\"↷\",\"cuvee;\":\"⋎\",\"cuwed;\":\"⋏\",\"cwconint;\":\"∲\",\"cwint;\":\"∱\",\"cylcty;\":\"⌭\",\"Dagger;\":\"‡\",\"dagger;\":\"†\",\"daleth;\":\"ℸ\",\"Darr;\":\"↡\",\"dArr;\":\"⇓\",\"darr;\":\"↓\",\"dash;\":\"‐\",\"Dashv;\":\"⫤\",\"dashv;\":\"⊣\",\"dbkarow;\":\"⤏\",\"dblac;\":\"˝\",\"Dcaron;\":\"Ď\",\"dcaron;\":\"ď\",\"Dcy;\":\"Д\",\"dcy;\":\"д\",\"DD;\":\"ⅅ\",\"dd;\":\"ⅆ\",\"ddagger;\":\"‡\",\"ddarr;\":\"⇊\",\"DDotrahd;\":\"⤑\",\"ddotseq;\":\"⩷\",\"deg;\":\"°\",deg:\"°\",\"Del;\":\"∇\",\"Delta;\":\"Δ\",\"delta;\":\"δ\",\"demptyv;\":\"⦱\",\"dfisht;\":\"⥿\",\"Dfr;\":\"𝔇\",\"dfr;\":\"𝔡\",\"dHar;\":\"⥥\",\"dharl;\":\"⇃\",\"dharr;\":\"⇂\",\"DiacriticalAcute;\":\"´\",\"DiacriticalDot;\":\"˙\",\"DiacriticalDoubleAcute;\":\"˝\",\"DiacriticalGrave;\":\"`\",\"DiacriticalTilde;\":\"˜\",\"diam;\":\"⋄\",\"Diamond;\":\"⋄\",\"diamond;\":\"⋄\",\"diamondsuit;\":\"♦\",\"diams;\":\"♦\",\"die;\":\"¨\",\"DifferentialD;\":\"ⅆ\",\"digamma;\":\"ϝ\",\"disin;\":\"⋲\",\"div;\":\"÷\",\"divide;\":\"÷\",divide:\"÷\",\"divideontimes;\":\"⋇\",\"divonx;\":\"⋇\",\"DJcy;\":\"Ђ\",\"djcy;\":\"ђ\",\"dlcorn;\":\"⌞\",\"dlcrop;\":\"⌍\",\"dollar;\":\"$\",\"Dopf;\":\"𝔻\",\"dopf;\":\"𝕕\",\"Dot;\":\"¨\",\"dot;\":\"˙\",\"DotDot;\":\"⃜\",\"doteq;\":\"≐\",\"doteqdot;\":\"≑\",\"DotEqual;\":\"≐\",\"dotminus;\":\"∸\",\"dotplus;\":\"∔\",\"dotsquare;\":\"⊡\",\"doublebarwedge;\":\"⌆\",\"DoubleContourIntegral;\":\"∯\",\"DoubleDot;\":\"¨\",\"DoubleDownArrow;\":\"⇓\",\"DoubleLeftArrow;\":\"⇐\",\"DoubleLeftRightArrow;\":\"⇔\",\"DoubleLeftTee;\":\"⫤\",\"DoubleLongLeftArrow;\":\"⟸\",\"DoubleLongLeftRightArrow;\":\"⟺\",\"DoubleLongRightArrow;\":\"⟹\",\"DoubleRightArrow;\":\"⇒\",\"DoubleRightTee;\":\"⊨\",\"DoubleUpArrow;\":\"⇑\",\"DoubleUpDownArrow;\":\"⇕\",\"DoubleVerticalBar;\":\"∥\",\"DownArrow;\":\"↓\",\"Downarrow;\":\"⇓\",\"downarrow;\":\"↓\",\"DownArrowBar;\":\"⤓\",\"DownArrowUpArrow;\":\"⇵\",\"DownBreve;\":\"̑\",\"downdownarrows;\":\"⇊\",\"downharpoonleft;\":\"⇃\",\"downharpoonright;\":\"⇂\",\"DownLeftRightVector;\":\"⥐\",\"DownLeftTeeVector;\":\"⥞\",\"DownLeftVector;\":\"↽\",\"DownLeftVectorBar;\":\"⥖\",\"DownRightTeeVector;\":\"⥟\",\"DownRightVector;\":\"⇁\",\"DownRightVectorBar;\":\"⥗\",\"DownTee;\":\"⊤\",\"DownTeeArrow;\":\"↧\",\"drbkarow;\":\"⤐\",\"drcorn;\":\"⌟\",\"drcrop;\":\"⌌\",\"Dscr;\":\"𝒟\",\"dscr;\":\"𝒹\",\"DScy;\":\"Ѕ\",\"dscy;\":\"ѕ\",\"dsol;\":\"⧶\",\"Dstrok;\":\"Đ\",\"dstrok;\":\"đ\",\"dtdot;\":\"⋱\",\"dtri;\":\"▿\",\"dtrif;\":\"▾\",\"duarr;\":\"⇵\",\"duhar;\":\"⥯\",\"dwangle;\":\"⦦\",\"DZcy;\":\"Џ\",\"dzcy;\":\"џ\",\"dzigrarr;\":\"⟿\",\"Eacute;\":\"É\",Eacute:\"É\",\"eacute;\":\"é\",eacute:\"é\",\"easter;\":\"⩮\",\"Ecaron;\":\"Ě\",\"ecaron;\":\"ě\",\"ecir;\":\"≖\",\"Ecirc;\":\"Ê\",Ecirc:\"Ê\",\"ecirc;\":\"ê\",ecirc:\"ê\",\"ecolon;\":\"≕\",\"Ecy;\":\"Э\",\"ecy;\":\"э\",\"eDDot;\":\"⩷\",\"Edot;\":\"Ė\",\"eDot;\":\"≑\",\"edot;\":\"ė\",\"ee;\":\"ⅇ\",\"efDot;\":\"≒\",\"Efr;\":\"𝔈\",\"efr;\":\"𝔢\",\"eg;\":\"⪚\",\"Egrave;\":\"È\",Egrave:\"È\",\"egrave;\":\"è\",egrave:\"è\",\"egs;\":\"⪖\",\"egsdot;\":\"⪘\",\"el;\":\"⪙\",\"Element;\":\"∈\",\"elinters;\":\"⏧\",\"ell;\":\"ℓ\",\"els;\":\"⪕\",\"elsdot;\":\"⪗\",\"Emacr;\":\"Ē\",\"emacr;\":\"ē\",\"empty;\":\"∅\",\"emptyset;\":\"∅\",\"EmptySmallSquare;\":\"◻\",\"emptyv;\":\"∅\",\"EmptyVerySmallSquare;\":\"▫\",\"emsp;\":\" \",\"emsp13;\":\" \",\"emsp14;\":\" \",\"ENG;\":\"Ŋ\",\"eng;\":\"ŋ\",\"ensp;\":\" \",\"Eogon;\":\"Ę\",\"eogon;\":\"ę\",\"Eopf;\":\"𝔼\",\"eopf;\":\"𝕖\",\"epar;\":\"⋕\",\"eparsl;\":\"⧣\",\"eplus;\":\"⩱\",\"epsi;\":\"ε\",\"Epsilon;\":\"Ε\",\"epsilon;\":\"ε\",\"epsiv;\":\"ϵ\",\"eqcirc;\":\"≖\",\"eqcolon;\":\"≕\",\"eqsim;\":\"≂\",\"eqslantgtr;\":\"⪖\",\"eqslantless;\":\"⪕\",\"Equal;\":\"⩵\",\"equals;\":\"=\",\"EqualTilde;\":\"≂\",\"equest;\":\"≟\",\"Equilibrium;\":\"⇌\",\"equiv;\":\"≡\",\"equivDD;\":\"⩸\",\"eqvparsl;\":\"⧥\",\"erarr;\":\"⥱\",\"erDot;\":\"≓\",\"Escr;\":\"ℰ\",\"escr;\":\"ℯ\",\"esdot;\":\"≐\",\"Esim;\":\"⩳\",\"esim;\":\"≂\",\"Eta;\":\"Η\",\"eta;\":\"η\",\"ETH;\":\"Ð\",ETH:\"Ð\",\"eth;\":\"ð\",eth:\"ð\",\"Euml;\":\"Ë\",Euml:\"Ë\",\"euml;\":\"ë\",euml:\"ë\",\"euro;\":\"€\",\"excl;\":\"!\",\"exist;\":\"∃\",\"Exists;\":\"∃\",\"expectation;\":\"ℰ\",\"ExponentialE;\":\"ⅇ\",\"exponentiale;\":\"ⅇ\",\"fallingdotseq;\":\"≒\",\"Fcy;\":\"Ф\",\"fcy;\":\"ф\",\"female;\":\"♀\",\"ffilig;\":\"ﬃ\",\"fflig;\":\"ﬀ\",\"ffllig;\":\"ﬄ\",\"Ffr;\":\"𝔉\",\"ffr;\":\"𝔣\",\"filig;\":\"ﬁ\",\"FilledSmallSquare;\":\"◼\",\"FilledVerySmallSquare;\":\"▪\",\"fjlig;\":\"fj\",\"flat;\":\"♭\",\"fllig;\":\"ﬂ\",\"fltns;\":\"▱\",\"fnof;\":\"ƒ\",\"Fopf;\":\"𝔽\",\"fopf;\":\"𝕗\",\"ForAll;\":\"∀\",\"forall;\":\"∀\",\"fork;\":\"⋔\",\"forkv;\":\"⫙\",\"Fouriertrf;\":\"ℱ\",\"fpartint;\":\"⨍\",\"frac12;\":\"½\",frac12:\"½\",\"frac13;\":\"⅓\",\"frac14;\":\"¼\",frac14:\"¼\",\"frac15;\":\"⅕\",\"frac16;\":\"⅙\",\"frac18;\":\"⅛\",\"frac23;\":\"⅔\",\"frac25;\":\"⅖\",\"frac34;\":\"¾\",frac34:\"¾\",\"frac35;\":\"⅗\",\"frac38;\":\"⅜\",\"frac45;\":\"⅘\",\"frac56;\":\"⅚\",\"frac58;\":\"⅝\",\"frac78;\":\"⅞\",\"frasl;\":\"⁄\",\"frown;\":\"⌢\",\"Fscr;\":\"ℱ\",\"fscr;\":\"𝒻\",\"gacute;\":\"ǵ\",\"Gamma;\":\"Γ\",\"gamma;\":\"γ\",\"Gammad;\":\"Ϝ\",\"gammad;\":\"ϝ\",\"gap;\":\"⪆\",\"Gbreve;\":\"Ğ\",\"gbreve;\":\"ğ\",\"Gcedil;\":\"Ģ\",\"Gcirc;\":\"Ĝ\",\"gcirc;\":\"ĝ\",\"Gcy;\":\"Г\",\"gcy;\":\"г\",\"Gdot;\":\"Ġ\",\"gdot;\":\"ġ\",\"gE;\":\"≧\",\"ge;\":\"≥\",\"gEl;\":\"⪌\",\"gel;\":\"⋛\",\"geq;\":\"≥\",\"geqq;\":\"≧\",\"geqslant;\":\"⩾\",\"ges;\":\"⩾\",\"gescc;\":\"⪩\",\"gesdot;\":\"⪀\",\"gesdoto;\":\"⪂\",\"gesdotol;\":\"⪄\",\"gesl;\":\"⋛︀\",\"gesles;\":\"⪔\",\"Gfr;\":\"𝔊\",\"gfr;\":\"𝔤\",\"Gg;\":\"⋙\",\"gg;\":\"≫\",\"ggg;\":\"⋙\",\"gimel;\":\"ℷ\",\"GJcy;\":\"Ѓ\",\"gjcy;\":\"ѓ\",\"gl;\":\"≷\",\"gla;\":\"⪥\",\"glE;\":\"⪒\",\"glj;\":\"⪤\",\"gnap;\":\"⪊\",\"gnapprox;\":\"⪊\",\"gnE;\":\"≩\",\"gne;\":\"⪈\",\"gneq;\":\"⪈\",\"gneqq;\":\"≩\",\"gnsim;\":\"⋧\",\"Gopf;\":\"𝔾\",\"gopf;\":\"𝕘\",\"grave;\":\"`\",\"GreaterEqual;\":\"≥\",\"GreaterEqualLess;\":\"⋛\",\"GreaterFullEqual;\":\"≧\",\"GreaterGreater;\":\"⪢\",\"GreaterLess;\":\"≷\",\"GreaterSlantEqual;\":\"⩾\",\"GreaterTilde;\":\"≳\",\"Gscr;\":\"𝒢\",\"gscr;\":\"ℊ\",\"gsim;\":\"≳\",\"gsime;\":\"⪎\",\"gsiml;\":\"⪐\",\"GT;\":\">\",GT:\">\",\"Gt;\":\"≫\",\"gt;\":\">\",gt:\">\",\"gtcc;\":\"⪧\",\"gtcir;\":\"⩺\",\"gtdot;\":\"⋗\",\"gtlPar;\":\"⦕\",\"gtquest;\":\"⩼\",\"gtrapprox;\":\"⪆\",\"gtrarr;\":\"⥸\",\"gtrdot;\":\"⋗\",\"gtreqless;\":\"⋛\",\"gtreqqless;\":\"⪌\",\"gtrless;\":\"≷\",\"gtrsim;\":\"≳\",\"gvertneqq;\":\"≩︀\",\"gvnE;\":\"≩︀\",\"Hacek;\":\"ˇ\",\"hairsp;\":\" \",\"half;\":\"½\",\"hamilt;\":\"ℋ\",\"HARDcy;\":\"Ъ\",\"hardcy;\":\"ъ\",\"hArr;\":\"⇔\",\"harr;\":\"↔\",\"harrcir;\":\"⥈\",\"harrw;\":\"↭\",\"Hat;\":\"^\",\"hbar;\":\"ℏ\",\"Hcirc;\":\"Ĥ\",\"hcirc;\":\"ĥ\",\"hearts;\":\"♥\",\"heartsuit;\":\"♥\",\"hellip;\":\"…\",\"hercon;\":\"⊹\",\"Hfr;\":\"ℌ\",\"hfr;\":\"𝔥\",\"HilbertSpace;\":\"ℋ\",\"hksearow;\":\"⤥\",\"hkswarow;\":\"⤦\",\"hoarr;\":\"⇿\",\"homtht;\":\"∻\",\"hookleftarrow;\":\"↩\",\"hookrightarrow;\":\"↪\",\"Hopf;\":\"ℍ\",\"hopf;\":\"𝕙\",\"horbar;\":\"―\",\"HorizontalLine;\":\"─\",\"Hscr;\":\"ℋ\",\"hscr;\":\"𝒽\",\"hslash;\":\"ℏ\",\"Hstrok;\":\"Ħ\",\"hstrok;\":\"ħ\",\"HumpDownHump;\":\"≎\",\"HumpEqual;\":\"≏\",\"hybull;\":\"⁃\",\"hyphen;\":\"‐\",\"Iacute;\":\"Í\",Iacute:\"Í\",\"iacute;\":\"í\",iacute:\"í\",\"ic;\":\"⁣\",\"Icirc;\":\"Î\",Icirc:\"Î\",\"icirc;\":\"î\",icirc:\"î\",\"Icy;\":\"И\",\"icy;\":\"и\",\"Idot;\":\"İ\",\"IEcy;\":\"Е\",\"iecy;\":\"е\",\"iexcl;\":\"¡\",iexcl:\"¡\",\"iff;\":\"⇔\",\"Ifr;\":\"ℑ\",\"ifr;\":\"𝔦\",\"Igrave;\":\"Ì\",Igrave:\"Ì\",\"igrave;\":\"ì\",igrave:\"ì\",\"ii;\":\"ⅈ\",\"iiiint;\":\"⨌\",\"iiint;\":\"∭\",\"iinfin;\":\"⧜\",\"iiota;\":\"℩\",\"IJlig;\":\"Ĳ\",\"ijlig;\":\"ĳ\",\"Im;\":\"ℑ\",\"Imacr;\":\"Ī\",\"imacr;\":\"ī\",\"image;\":\"ℑ\",\"ImaginaryI;\":\"ⅈ\",\"imagline;\":\"ℐ\",\"imagpart;\":\"ℑ\",\"imath;\":\"ı\",\"imof;\":\"⊷\",\"imped;\":\"Ƶ\",\"Implies;\":\"⇒\",\"in;\":\"∈\",\"incare;\":\"℅\",\"infin;\":\"∞\",\"infintie;\":\"⧝\",\"inodot;\":\"ı\",\"Int;\":\"∬\",\"int;\":\"∫\",\"intcal;\":\"⊺\",\"integers;\":\"ℤ\",\"Integral;\":\"∫\",\"intercal;\":\"⊺\",\"Intersection;\":\"⋂\",\"intlarhk;\":\"⨗\",\"intprod;\":\"⨼\",\"InvisibleComma;\":\"⁣\",\"InvisibleTimes;\":\"⁢\",\"IOcy;\":\"Ё\",\"iocy;\":\"ё\",\"Iogon;\":\"Į\",\"iogon;\":\"į\",\"Iopf;\":\"𝕀\",\"iopf;\":\"𝕚\",\"Iota;\":\"Ι\",\"iota;\":\"ι\",\"iprod;\":\"⨼\",\"iquest;\":\"¿\",iquest:\"¿\",\"Iscr;\":\"ℐ\",\"iscr;\":\"𝒾\",\"isin;\":\"∈\",\"isindot;\":\"⋵\",\"isinE;\":\"⋹\",\"isins;\":\"⋴\",\"isinsv;\":\"⋳\",\"isinv;\":\"∈\",\"it;\":\"⁢\",\"Itilde;\":\"Ĩ\",\"itilde;\":\"ĩ\",\"Iukcy;\":\"І\",\"iukcy;\":\"і\",\"Iuml;\":\"Ï\",Iuml:\"Ï\",\"iuml;\":\"ï\",iuml:\"ï\",\"Jcirc;\":\"Ĵ\",\"jcirc;\":\"ĵ\",\"Jcy;\":\"Й\",\"jcy;\":\"й\",\"Jfr;\":\"𝔍\",\"jfr;\":\"𝔧\",\"jmath;\":\"ȷ\",\"Jopf;\":\"𝕁\",\"jopf;\":\"𝕛\",\"Jscr;\":\"𝒥\",\"jscr;\":\"𝒿\",\"Jsercy;\":\"Ј\",\"jsercy;\":\"ј\",\"Jukcy;\":\"Є\",\"jukcy;\":\"є\",\"Kappa;\":\"Κ\",\"kappa;\":\"κ\",\"kappav;\":\"ϰ\",\"Kcedil;\":\"Ķ\",\"kcedil;\":\"ķ\",\"Kcy;\":\"К\",\"kcy;\":\"к\",\"Kfr;\":\"𝔎\",\"kfr;\":\"𝔨\",\"kgreen;\":\"ĸ\",\"KHcy;\":\"Х\",\"khcy;\":\"х\",\"KJcy;\":\"Ќ\",\"kjcy;\":\"ќ\",\"Kopf;\":\"𝕂\",\"kopf;\":\"𝕜\",\"Kscr;\":\"𝒦\",\"kscr;\":\"𝓀\",\"lAarr;\":\"⇚\",\"Lacute;\":\"Ĺ\",\"lacute;\":\"ĺ\",\"laemptyv;\":\"⦴\",\"lagran;\":\"ℒ\",\"Lambda;\":\"Λ\",\"lambda;\":\"λ\",\"Lang;\":\"⟪\",\"lang;\":\"⟨\",\"langd;\":\"⦑\",\"langle;\":\"⟨\",\"lap;\":\"⪅\",\"Laplacetrf;\":\"ℒ\",\"laquo;\":\"«\",laquo:\"«\",\"Larr;\":\"↞\",\"lArr;\":\"⇐\",\"larr;\":\"←\",\"larrb;\":\"⇤\",\"larrbfs;\":\"⤟\",\"larrfs;\":\"⤝\",\"larrhk;\":\"↩\",\"larrlp;\":\"↫\",\"larrpl;\":\"⤹\",\"larrsim;\":\"⥳\",\"larrtl;\":\"↢\",\"lat;\":\"⪫\",\"lAtail;\":\"⤛\",\"latail;\":\"⤙\",\"late;\":\"⪭\",\"lates;\":\"⪭︀\",\"lBarr;\":\"⤎\",\"lbarr;\":\"⤌\",\"lbbrk;\":\"❲\",\"lbrace;\":\"{\",\"lbrack;\":\"[\",\"lbrke;\":\"⦋\",\"lbrksld;\":\"⦏\",\"lbrkslu;\":\"⦍\",\"Lcaron;\":\"Ľ\",\"lcaron;\":\"ľ\",\"Lcedil;\":\"Ļ\",\"lcedil;\":\"ļ\",\"lceil;\":\"⌈\",\"lcub;\":\"{\",\"Lcy;\":\"Л\",\"lcy;\":\"л\",\"ldca;\":\"⤶\",\"ldquo;\":\"“\",\"ldquor;\":\"„\",\"ldrdhar;\":\"⥧\",\"ldrushar;\":\"⥋\",\"ldsh;\":\"↲\",\"lE;\":\"≦\",\"le;\":\"≤\",\"LeftAngleBracket;\":\"⟨\",\"LeftArrow;\":\"←\",\"Leftarrow;\":\"⇐\",\"leftarrow;\":\"←\",\"LeftArrowBar;\":\"⇤\",\"LeftArrowRightArrow;\":\"⇆\",\"leftarrowtail;\":\"↢\",\"LeftCeiling;\":\"⌈\",\"LeftDoubleBracket;\":\"⟦\",\"LeftDownTeeVector;\":\"⥡\",\"LeftDownVector;\":\"⇃\",\"LeftDownVectorBar;\":\"⥙\",\"LeftFloor;\":\"⌊\",\"leftharpoondown;\":\"↽\",\"leftharpoonup;\":\"↼\",\"leftleftarrows;\":\"⇇\",\"LeftRightArrow;\":\"↔\",\"Leftrightarrow;\":\"⇔\",\"leftrightarrow;\":\"↔\",\"leftrightarrows;\":\"⇆\",\"leftrightharpoons;\":\"⇋\",\"leftrightsquigarrow;\":\"↭\",\"LeftRightVector;\":\"⥎\",\"LeftTee;\":\"⊣\",\"LeftTeeArrow;\":\"↤\",\"LeftTeeVector;\":\"⥚\",\"leftthreetimes;\":\"⋋\",\"LeftTriangle;\":\"⊲\",\"LeftTriangleBar;\":\"⧏\",\"LeftTriangleEqual;\":\"⊴\",\"LeftUpDownVector;\":\"⥑\",\"LeftUpTeeVector;\":\"⥠\",\"LeftUpVector;\":\"↿\",\"LeftUpVectorBar;\":\"⥘\",\"LeftVector;\":\"↼\",\"LeftVectorBar;\":\"⥒\",\"lEg;\":\"⪋\",\"leg;\":\"⋚\",\"leq;\":\"≤\",\"leqq;\":\"≦\",\"leqslant;\":\"⩽\",\"les;\":\"⩽\",\"lescc;\":\"⪨\",\"lesdot;\":\"⩿\",\"lesdoto;\":\"⪁\",\"lesdotor;\":\"⪃\",\"lesg;\":\"⋚︀\",\"lesges;\":\"⪓\",\"lessapprox;\":\"⪅\",\"lessdot;\":\"⋖\",\"lesseqgtr;\":\"⋚\",\"lesseqqgtr;\":\"⪋\",\"LessEqualGreater;\":\"⋚\",\"LessFullEqual;\":\"≦\",\"LessGreater;\":\"≶\",\"lessgtr;\":\"≶\",\"LessLess;\":\"⪡\",\"lesssim;\":\"≲\",\"LessSlantEqual;\":\"⩽\",\"LessTilde;\":\"≲\",\"lfisht;\":\"⥼\",\"lfloor;\":\"⌊\",\"Lfr;\":\"𝔏\",\"lfr;\":\"𝔩\",\"lg;\":\"≶\",\"lgE;\":\"⪑\",\"lHar;\":\"⥢\",\"lhard;\":\"↽\",\"lharu;\":\"↼\",\"lharul;\":\"⥪\",\"lhblk;\":\"▄\",\"LJcy;\":\"Љ\",\"ljcy;\":\"љ\",\"Ll;\":\"⋘\",\"ll;\":\"≪\",\"llarr;\":\"⇇\",\"llcorner;\":\"⌞\",\"Lleftarrow;\":\"⇚\",\"llhard;\":\"⥫\",\"lltri;\":\"◺\",\"Lmidot;\":\"Ŀ\",\"lmidot;\":\"ŀ\",\"lmoust;\":\"⎰\",\"lmoustache;\":\"⎰\",\"lnap;\":\"⪉\",\"lnapprox;\":\"⪉\",\"lnE;\":\"≨\",\"lne;\":\"⪇\",\"lneq;\":\"⪇\",\"lneqq;\":\"≨\",\"lnsim;\":\"⋦\",\"loang;\":\"⟬\",\"loarr;\":\"⇽\",\"lobrk;\":\"⟦\",\"LongLeftArrow;\":\"⟵\",\"Longleftarrow;\":\"⟸\",\"longleftarrow;\":\"⟵\",\"LongLeftRightArrow;\":\"⟷\",\"Longleftrightarrow;\":\"⟺\",\"longleftrightarrow;\":\"⟷\",\"longmapsto;\":\"⟼\",\"LongRightArrow;\":\"⟶\",\"Longrightarrow;\":\"⟹\",\"longrightarrow;\":\"⟶\",\"looparrowleft;\":\"↫\",\"looparrowright;\":\"↬\",\"lopar;\":\"⦅\",\"Lopf;\":\"𝕃\",\"lopf;\":\"𝕝\",\"loplus;\":\"⨭\",\"lotimes;\":\"⨴\",\"lowast;\":\"∗\",\"lowbar;\":\"_\",\"LowerLeftArrow;\":\"↙\",\"LowerRightArrow;\":\"↘\",\"loz;\":\"◊\",\"lozenge;\":\"◊\",\"lozf;\":\"⧫\",\"lpar;\":\"(\",\"lparlt;\":\"⦓\",\"lrarr;\":\"⇆\",\"lrcorner;\":\"⌟\",\"lrhar;\":\"⇋\",\"lrhard;\":\"⥭\",\"lrm;\":\"‎\",\"lrtri;\":\"⊿\",\"lsaquo;\":\"‹\",\"Lscr;\":\"ℒ\",\"lscr;\":\"𝓁\",\"Lsh;\":\"↰\",\"lsh;\":\"↰\",\"lsim;\":\"≲\",\"lsime;\":\"⪍\",\"lsimg;\":\"⪏\",\"lsqb;\":\"[\",\"lsquo;\":\"‘\",\"lsquor;\":\"‚\",\"Lstrok;\":\"Ł\",\"lstrok;\":\"ł\",\"LT;\":\"<\",LT:\"<\",\"Lt;\":\"≪\",\"lt;\":\"<\",lt:\"<\",\"ltcc;\":\"⪦\",\"ltcir;\":\"⩹\",\"ltdot;\":\"⋖\",\"lthree;\":\"⋋\",\"ltimes;\":\"⋉\",\"ltlarr;\":\"⥶\",\"ltquest;\":\"⩻\",\"ltri;\":\"◃\",\"ltrie;\":\"⊴\",\"ltrif;\":\"◂\",\"ltrPar;\":\"⦖\",\"lurdshar;\":\"⥊\",\"luruhar;\":\"⥦\",\"lvertneqq;\":\"≨︀\",\"lvnE;\":\"≨︀\",\"macr;\":\"¯\",macr:\"¯\",\"male;\":\"♂\",\"malt;\":\"✠\",\"maltese;\":\"✠\",\"Map;\":\"⤅\",\"map;\":\"↦\",\"mapsto;\":\"↦\",\"mapstodown;\":\"↧\",\"mapstoleft;\":\"↤\",\"mapstoup;\":\"↥\",\"marker;\":\"▮\",\"mcomma;\":\"⨩\",\"Mcy;\":\"М\",\"mcy;\":\"м\",\"mdash;\":\"—\",\"mDDot;\":\"∺\",\"measuredangle;\":\"∡\",\"MediumSpace;\":\" \",\"Mellintrf;\":\"ℳ\",\"Mfr;\":\"𝔐\",\"mfr;\":\"𝔪\",\"mho;\":\"℧\",\"micro;\":\"µ\",micro:\"µ\",\"mid;\":\"∣\",\"midast;\":\"*\",\"midcir;\":\"⫰\",\"middot;\":\"·\",middot:\"·\",\"minus;\":\"−\",\"minusb;\":\"⊟\",\"minusd;\":\"∸\",\"minusdu;\":\"⨪\",\"MinusPlus;\":\"∓\",\"mlcp;\":\"⫛\",\"mldr;\":\"…\",\"mnplus;\":\"∓\",\"models;\":\"⊧\",\"Mopf;\":\"𝕄\",\"mopf;\":\"𝕞\",\"mp;\":\"∓\",\"Mscr;\":\"ℳ\",\"mscr;\":\"𝓂\",\"mstpos;\":\"∾\",\"Mu;\":\"Μ\",\"mu;\":\"μ\",\"multimap;\":\"⊸\",\"mumap;\":\"⊸\",\"nabla;\":\"∇\",\"Nacute;\":\"Ń\",\"nacute;\":\"ń\",\"nang;\":\"∠⃒\",\"nap;\":\"≉\",\"napE;\":\"⩰̸\",\"napid;\":\"≋̸\",\"napos;\":\"ŉ\",\"napprox;\":\"≉\",\"natur;\":\"♮\",\"natural;\":\"♮\",\"naturals;\":\"ℕ\",\"nbsp;\":\" \",nbsp:\" \",\"nbump;\":\"≎̸\",\"nbumpe;\":\"≏̸\",\"ncap;\":\"⩃\",\"Ncaron;\":\"Ň\",\"ncaron;\":\"ň\",\"Ncedil;\":\"Ņ\",\"ncedil;\":\"ņ\",\"ncong;\":\"≇\",\"ncongdot;\":\"⩭̸\",\"ncup;\":\"⩂\",\"Ncy;\":\"Н\",\"ncy;\":\"н\",\"ndash;\":\"–\",\"ne;\":\"≠\",\"nearhk;\":\"⤤\",\"neArr;\":\"⇗\",\"nearr;\":\"↗\",\"nearrow;\":\"↗\",\"nedot;\":\"≐̸\",\"NegativeMediumSpace;\":\"​\",\"NegativeThickSpace;\":\"​\",\"NegativeThinSpace;\":\"​\",\"NegativeVeryThinSpace;\":\"​\",\"nequiv;\":\"≢\",\"nesear;\":\"⤨\",\"nesim;\":\"≂̸\",\"NestedGreaterGreater;\":\"≫\",\"NestedLessLess;\":\"≪\",\"NewLine;\":\"\\n\",\"nexist;\":\"∄\",\"nexists;\":\"∄\",\"Nfr;\":\"𝔑\",\"nfr;\":\"𝔫\",\"ngE;\":\"≧̸\",\"nge;\":\"≱\",\"ngeq;\":\"≱\",\"ngeqq;\":\"≧̸\",\"ngeqslant;\":\"⩾̸\",\"nges;\":\"⩾̸\",\"nGg;\":\"⋙̸\",\"ngsim;\":\"≵\",\"nGt;\":\"≫⃒\",\"ngt;\":\"≯\",\"ngtr;\":\"≯\",\"nGtv;\":\"≫̸\",\"nhArr;\":\"⇎\",\"nharr;\":\"↮\",\"nhpar;\":\"⫲\",\"ni;\":\"∋\",\"nis;\":\"⋼\",\"nisd;\":\"⋺\",\"niv;\":\"∋\",\"NJcy;\":\"Њ\",\"njcy;\":\"њ\",\"nlArr;\":\"⇍\",\"nlarr;\":\"↚\",\"nldr;\":\"‥\",\"nlE;\":\"≦̸\",\"nle;\":\"≰\",\"nLeftarrow;\":\"⇍\",\"nleftarrow;\":\"↚\",\"nLeftrightarrow;\":\"⇎\",\"nleftrightarrow;\":\"↮\",\"nleq;\":\"≰\",\"nleqq;\":\"≦̸\",\"nleqslant;\":\"⩽̸\",\"nles;\":\"⩽̸\",\"nless;\":\"≮\",\"nLl;\":\"⋘̸\",\"nlsim;\":\"≴\",\"nLt;\":\"≪⃒\",\"nlt;\":\"≮\",\"nltri;\":\"⋪\",\"nltrie;\":\"⋬\",\"nLtv;\":\"≪̸\",\"nmid;\":\"∤\",\"NoBreak;\":\"⁠\",\"NonBreakingSpace;\":\" \",\"Nopf;\":\"ℕ\",\"nopf;\":\"𝕟\",\"Not;\":\"⫬\",\"not;\":\"¬\",not:\"¬\",\"NotCongruent;\":\"≢\",\"NotCupCap;\":\"≭\",\"NotDoubleVerticalBar;\":\"∦\",\"NotElement;\":\"∉\",\"NotEqual;\":\"≠\",\"NotEqualTilde;\":\"≂̸\",\"NotExists;\":\"∄\",\"NotGreater;\":\"≯\",\"NotGreaterEqual;\":\"≱\",\"NotGreaterFullEqual;\":\"≧̸\",\"NotGreaterGreater;\":\"≫̸\",\"NotGreaterLess;\":\"≹\",\"NotGreaterSlantEqual;\":\"⩾̸\",\"NotGreaterTilde;\":\"≵\",\"NotHumpDownHump;\":\"≎̸\",\"NotHumpEqual;\":\"≏̸\",\"notin;\":\"∉\",\"notindot;\":\"⋵̸\",\"notinE;\":\"⋹̸\",\"notinva;\":\"∉\",\"notinvb;\":\"⋷\",\"notinvc;\":\"⋶\",\"NotLeftTriangle;\":\"⋪\",\"NotLeftTriangleBar;\":\"⧏̸\",\"NotLeftTriangleEqual;\":\"⋬\",\"NotLess;\":\"≮\",\"NotLessEqual;\":\"≰\",\"NotLessGreater;\":\"≸\",\"NotLessLess;\":\"≪̸\",\"NotLessSlantEqual;\":\"⩽̸\",\"NotLessTilde;\":\"≴\",\"NotNestedGreaterGreater;\":\"⪢̸\",\"NotNestedLessLess;\":\"⪡̸\",\"notni;\":\"∌\",\"notniva;\":\"∌\",\"notnivb;\":\"⋾\",\"notnivc;\":\"⋽\",\"NotPrecedes;\":\"⊀\",\"NotPrecedesEqual;\":\"⪯̸\",\"NotPrecedesSlantEqual;\":\"⋠\",\"NotReverseElement;\":\"∌\",\"NotRightTriangle;\":\"⋫\",\"NotRightTriangleBar;\":\"⧐̸\",\"NotRightTriangleEqual;\":\"⋭\",\"NotSquareSubset;\":\"⊏̸\",\"NotSquareSubsetEqual;\":\"⋢\",\"NotSquareSuperset;\":\"⊐̸\",\"NotSquareSupersetEqual;\":\"⋣\",\"NotSubset;\":\"⊂⃒\",\"NotSubsetEqual;\":\"⊈\",\"NotSucceeds;\":\"⊁\",\"NotSucceedsEqual;\":\"⪰̸\",\"NotSucceedsSlantEqual;\":\"⋡\",\"NotSucceedsTilde;\":\"≿̸\",\"NotSuperset;\":\"⊃⃒\",\"NotSupersetEqual;\":\"⊉\",\"NotTilde;\":\"≁\",\"NotTildeEqual;\":\"≄\",\"NotTildeFullEqual;\":\"≇\",\"NotTildeTilde;\":\"≉\",\"NotVerticalBar;\":\"∤\",\"npar;\":\"∦\",\"nparallel;\":\"∦\",\"nparsl;\":\"⫽⃥\",\"npart;\":\"∂̸\",\"npolint;\":\"⨔\",\"npr;\":\"⊀\",\"nprcue;\":\"⋠\",\"npre;\":\"⪯̸\",\"nprec;\":\"⊀\",\"npreceq;\":\"⪯̸\",\"nrArr;\":\"⇏\",\"nrarr;\":\"↛\",\"nrarrc;\":\"⤳̸\",\"nrarrw;\":\"↝̸\",\"nRightarrow;\":\"⇏\",\"nrightarrow;\":\"↛\",\"nrtri;\":\"⋫\",\"nrtrie;\":\"⋭\",\"nsc;\":\"⊁\",\"nsccue;\":\"⋡\",\"nsce;\":\"⪰̸\",\"Nscr;\":\"𝒩\",\"nscr;\":\"𝓃\",\"nshortmid;\":\"∤\",\"nshortparallel;\":\"∦\",\"nsim;\":\"≁\",\"nsime;\":\"≄\",\"nsimeq;\":\"≄\",\"nsmid;\":\"∤\",\"nspar;\":\"∦\",\"nsqsube;\":\"⋢\",\"nsqsupe;\":\"⋣\",\"nsub;\":\"⊄\",\"nsubE;\":\"⫅̸\",\"nsube;\":\"⊈\",\"nsubset;\":\"⊂⃒\",\"nsubseteq;\":\"⊈\",\"nsubseteqq;\":\"⫅̸\",\"nsucc;\":\"⊁\",\"nsucceq;\":\"⪰̸\",\"nsup;\":\"⊅\",\"nsupE;\":\"⫆̸\",\"nsupe;\":\"⊉\",\"nsupset;\":\"⊃⃒\",\"nsupseteq;\":\"⊉\",\"nsupseteqq;\":\"⫆̸\",\"ntgl;\":\"≹\",\"Ntilde;\":\"Ñ\",Ntilde:\"Ñ\",\"ntilde;\":\"ñ\",ntilde:\"ñ\",\"ntlg;\":\"≸\",\"ntriangleleft;\":\"⋪\",\"ntrianglelefteq;\":\"⋬\",\"ntriangleright;\":\"⋫\",\"ntrianglerighteq;\":\"⋭\",\"Nu;\":\"Ν\",\"nu;\":\"ν\",\"num;\":\"#\",\"numero;\":\"№\",\"numsp;\":\" \",\"nvap;\":\"≍⃒\",\"nVDash;\":\"⊯\",\"nVdash;\":\"⊮\",\"nvDash;\":\"⊭\",\"nvdash;\":\"⊬\",\"nvge;\":\"≥⃒\",\"nvgt;\":\">⃒\",\"nvHarr;\":\"⤄\",\"nvinfin;\":\"⧞\",\"nvlArr;\":\"⤂\",\"nvle;\":\"≤⃒\",\"nvlt;\":\"<⃒\",\"nvltrie;\":\"⊴⃒\",\"nvrArr;\":\"⤃\",\"nvrtrie;\":\"⊵⃒\",\"nvsim;\":\"∼⃒\",\"nwarhk;\":\"⤣\",\"nwArr;\":\"⇖\",\"nwarr;\":\"↖\",\"nwarrow;\":\"↖\",\"nwnear;\":\"⤧\",\"Oacute;\":\"Ó\",Oacute:\"Ó\",\"oacute;\":\"ó\",oacute:\"ó\",\"oast;\":\"⊛\",\"ocir;\":\"⊚\",\"Ocirc;\":\"Ô\",Ocirc:\"Ô\",\"ocirc;\":\"ô\",ocirc:\"ô\",\"Ocy;\":\"О\",\"ocy;\":\"о\",\"odash;\":\"⊝\",\"Odblac;\":\"Ő\",\"odblac;\":\"ő\",\"odiv;\":\"⨸\",\"odot;\":\"⊙\",\"odsold;\":\"⦼\",\"OElig;\":\"Œ\",\"oelig;\":\"œ\",\"ofcir;\":\"⦿\",\"Ofr;\":\"𝔒\",\"ofr;\":\"𝔬\",\"ogon;\":\"˛\",\"Ograve;\":\"Ò\",Ograve:\"Ò\",\"ograve;\":\"ò\",ograve:\"ò\",\"ogt;\":\"⧁\",\"ohbar;\":\"⦵\",\"ohm;\":\"Ω\",\"oint;\":\"∮\",\"olarr;\":\"↺\",\"olcir;\":\"⦾\",\"olcross;\":\"⦻\",\"oline;\":\"‾\",\"olt;\":\"⧀\",\"Omacr;\":\"Ō\",\"omacr;\":\"ō\",\"Omega;\":\"Ω\",\"omega;\":\"ω\",\"Omicron;\":\"Ο\",\"omicron;\":\"ο\",\"omid;\":\"⦶\",\"ominus;\":\"⊖\",\"Oopf;\":\"𝕆\",\"oopf;\":\"𝕠\",\"opar;\":\"⦷\",\"OpenCurlyDoubleQuote;\":\"“\",\"OpenCurlyQuote;\":\"‘\",\"operp;\":\"⦹\",\"oplus;\":\"⊕\",\"Or;\":\"⩔\",\"or;\":\"∨\",\"orarr;\":\"↻\",\"ord;\":\"⩝\",\"order;\":\"ℴ\",\"orderof;\":\"ℴ\",\"ordf;\":\"ª\",ordf:\"ª\",\"ordm;\":\"º\",ordm:\"º\",\"origof;\":\"⊶\",\"oror;\":\"⩖\",\"orslope;\":\"⩗\",\"orv;\":\"⩛\",\"oS;\":\"Ⓢ\",\"Oscr;\":\"𝒪\",\"oscr;\":\"ℴ\",\"Oslash;\":\"Ø\",Oslash:\"Ø\",\"oslash;\":\"ø\",oslash:\"ø\",\"osol;\":\"⊘\",\"Otilde;\":\"Õ\",Otilde:\"Õ\",\"otilde;\":\"õ\",otilde:\"õ\",\"Otimes;\":\"⨷\",\"otimes;\":\"⊗\",\"otimesas;\":\"⨶\",\"Ouml;\":\"Ö\",Ouml:\"Ö\",\"ouml;\":\"ö\",ouml:\"ö\",\"ovbar;\":\"⌽\",\"OverBar;\":\"‾\",\"OverBrace;\":\"⏞\",\"OverBracket;\":\"⎴\",\"OverParenthesis;\":\"⏜\",\"par;\":\"∥\",\"para;\":\"¶\",para:\"¶\",\"parallel;\":\"∥\",\"parsim;\":\"⫳\",\"parsl;\":\"⫽\",\"part;\":\"∂\",\"PartialD;\":\"∂\",\"Pcy;\":\"П\",\"pcy;\":\"п\",\"percnt;\":\"%\",\"period;\":\".\",\"permil;\":\"‰\",\"perp;\":\"⊥\",\"pertenk;\":\"‱\",\"Pfr;\":\"𝔓\",\"pfr;\":\"𝔭\",\"Phi;\":\"Φ\",\"phi;\":\"φ\",\"phiv;\":\"ϕ\",\"phmmat;\":\"ℳ\",\"phone;\":\"☎\",\"Pi;\":\"Π\",\"pi;\":\"π\",\"pitchfork;\":\"⋔\",\"piv;\":\"ϖ\",\"planck;\":\"ℏ\",\"planckh;\":\"ℎ\",\"plankv;\":\"ℏ\",\"plus;\":\"+\",\"plusacir;\":\"⨣\",\"plusb;\":\"⊞\",\"pluscir;\":\"⨢\",\"plusdo;\":\"∔\",\"plusdu;\":\"⨥\",\"pluse;\":\"⩲\",\"PlusMinus;\":\"±\",\"plusmn;\":\"±\",plusmn:\"±\",\"plussim;\":\"⨦\",\"plustwo;\":\"⨧\",\"pm;\":\"±\",\"Poincareplane;\":\"ℌ\",\"pointint;\":\"⨕\",\"Popf;\":\"ℙ\",\"popf;\":\"𝕡\",\"pound;\":\"£\",pound:\"£\",\"Pr;\":\"⪻\",\"pr;\":\"≺\",\"prap;\":\"⪷\",\"prcue;\":\"≼\",\"prE;\":\"⪳\",\"pre;\":\"⪯\",\"prec;\":\"≺\",\"precapprox;\":\"⪷\",\"preccurlyeq;\":\"≼\",\"Precedes;\":\"≺\",\"PrecedesEqual;\":\"⪯\",\"PrecedesSlantEqual;\":\"≼\",\"PrecedesTilde;\":\"≾\",\"preceq;\":\"⪯\",\"precnapprox;\":\"⪹\",\"precneqq;\":\"⪵\",\"precnsim;\":\"⋨\",\"precsim;\":\"≾\",\"Prime;\":\"″\",\"prime;\":\"′\",\"primes;\":\"ℙ\",\"prnap;\":\"⪹\",\"prnE;\":\"⪵\",\"prnsim;\":\"⋨\",\"prod;\":\"∏\",\"Product;\":\"∏\",\"profalar;\":\"⌮\",\"profline;\":\"⌒\",\"profsurf;\":\"⌓\",\"prop;\":\"∝\",\"Proportion;\":\"∷\",\"Proportional;\":\"∝\",\"propto;\":\"∝\",\"prsim;\":\"≾\",\"prurel;\":\"⊰\",\"Pscr;\":\"𝒫\",\"pscr;\":\"𝓅\",\"Psi;\":\"Ψ\",\"psi;\":\"ψ\",\"puncsp;\":\" \",\"Qfr;\":\"𝔔\",\"qfr;\":\"𝔮\",\"qint;\":\"⨌\",\"Qopf;\":\"ℚ\",\"qopf;\":\"𝕢\",\"qprime;\":\"⁗\",\"Qscr;\":\"𝒬\",\"qscr;\":\"𝓆\",\"quaternions;\":\"ℍ\",\"quatint;\":\"⨖\",\"quest;\":\"?\",\"questeq;\":\"≟\",\"QUOT;\":'\"',QUOT:'\"',\"quot;\":'\"',quot:'\"',\"rAarr;\":\"⇛\",\"race;\":\"∽̱\",\"Racute;\":\"Ŕ\",\"racute;\":\"ŕ\",\"radic;\":\"√\",\"raemptyv;\":\"⦳\",\"Rang;\":\"⟫\",\"rang;\":\"⟩\",\"rangd;\":\"⦒\",\"range;\":\"⦥\",\"rangle;\":\"⟩\",\"raquo;\":\"»\",raquo:\"»\",\"Rarr;\":\"↠\",\"rArr;\":\"⇒\",\"rarr;\":\"→\",\"rarrap;\":\"⥵\",\"rarrb;\":\"⇥\",\"rarrbfs;\":\"⤠\",\"rarrc;\":\"⤳\",\"rarrfs;\":\"⤞\",\"rarrhk;\":\"↪\",\"rarrlp;\":\"↬\",\"rarrpl;\":\"⥅\",\"rarrsim;\":\"⥴\",\"Rarrtl;\":\"⤖\",\"rarrtl;\":\"↣\",\"rarrw;\":\"↝\",\"rAtail;\":\"⤜\",\"ratail;\":\"⤚\",\"ratio;\":\"∶\",\"rationals;\":\"ℚ\",\"RBarr;\":\"⤐\",\"rBarr;\":\"⤏\",\"rbarr;\":\"⤍\",\"rbbrk;\":\"❳\",\"rbrace;\":\"}\",\"rbrack;\":\"]\",\"rbrke;\":\"⦌\",\"rbrksld;\":\"⦎\",\"rbrkslu;\":\"⦐\",\"Rcaron;\":\"Ř\",\"rcaron;\":\"ř\",\"Rcedil;\":\"Ŗ\",\"rcedil;\":\"ŗ\",\"rceil;\":\"⌉\",\"rcub;\":\"}\",\"Rcy;\":\"Р\",\"rcy;\":\"р\",\"rdca;\":\"⤷\",\"rdldhar;\":\"⥩\",\"rdquo;\":\"”\",\"rdquor;\":\"”\",\"rdsh;\":\"↳\",\"Re;\":\"ℜ\",\"real;\":\"ℜ\",\"realine;\":\"ℛ\",\"realpart;\":\"ℜ\",\"reals;\":\"ℝ\",\"rect;\":\"▭\",\"REG;\":\"®\",REG:\"®\",\"reg;\":\"®\",reg:\"®\",\"ReverseElement;\":\"∋\",\"ReverseEquilibrium;\":\"⇋\",\"ReverseUpEquilibrium;\":\"⥯\",\"rfisht;\":\"⥽\",\"rfloor;\":\"⌋\",\"Rfr;\":\"ℜ\",\"rfr;\":\"𝔯\",\"rHar;\":\"⥤\",\"rhard;\":\"⇁\",\"rharu;\":\"⇀\",\"rharul;\":\"⥬\",\"Rho;\":\"Ρ\",\"rho;\":\"ρ\",\"rhov;\":\"ϱ\",\"RightAngleBracket;\":\"⟩\",\"RightArrow;\":\"→\",\"Rightarrow;\":\"⇒\",\"rightarrow;\":\"→\",\"RightArrowBar;\":\"⇥\",\"RightArrowLeftArrow;\":\"⇄\",\"rightarrowtail;\":\"↣\",\"RightCeiling;\":\"⌉\",\"RightDoubleBracket;\":\"⟧\",\"RightDownTeeVector;\":\"⥝\",\"RightDownVector;\":\"⇂\",\"RightDownVectorBar;\":\"⥕\",\"RightFloor;\":\"⌋\",\"rightharpoondown;\":\"⇁\",\"rightharpoonup;\":\"⇀\",\"rightleftarrows;\":\"⇄\",\"rightleftharpoons;\":\"⇌\",\"rightrightarrows;\":\"⇉\",\"rightsquigarrow;\":\"↝\",\"RightTee;\":\"⊢\",\"RightTeeArrow;\":\"↦\",\"RightTeeVector;\":\"⥛\",\"rightthreetimes;\":\"⋌\",\"RightTriangle;\":\"⊳\",\"RightTriangleBar;\":\"⧐\",\"RightTriangleEqual;\":\"⊵\",\"RightUpDownVector;\":\"⥏\",\"RightUpTeeVector;\":\"⥜\",\"RightUpVector;\":\"↾\",\"RightUpVectorBar;\":\"⥔\",\"RightVector;\":\"⇀\",\"RightVectorBar;\":\"⥓\",\"ring;\":\"˚\",\"risingdotseq;\":\"≓\",\"rlarr;\":\"⇄\",\"rlhar;\":\"⇌\",\"rlm;\":\"‏\",\"rmoust;\":\"⎱\",\"rmoustache;\":\"⎱\",\"rnmid;\":\"⫮\",\"roang;\":\"⟭\",\"roarr;\":\"⇾\",\"robrk;\":\"⟧\",\"ropar;\":\"⦆\",\"Ropf;\":\"ℝ\",\"ropf;\":\"𝕣\",\"roplus;\":\"⨮\",\"rotimes;\":\"⨵\",\"RoundImplies;\":\"⥰\",\"rpar;\":\")\",\"rpargt;\":\"⦔\",\"rppolint;\":\"⨒\",\"rrarr;\":\"⇉\",\"Rrightarrow;\":\"⇛\",\"rsaquo;\":\"›\",\"Rscr;\":\"ℛ\",\"rscr;\":\"𝓇\",\"Rsh;\":\"↱\",\"rsh;\":\"↱\",\"rsqb;\":\"]\",\"rsquo;\":\"’\",\"rsquor;\":\"’\",\"rthree;\":\"⋌\",\"rtimes;\":\"⋊\",\"rtri;\":\"▹\",\"rtrie;\":\"⊵\",\"rtrif;\":\"▸\",\"rtriltri;\":\"⧎\",\"RuleDelayed;\":\"⧴\",\"ruluhar;\":\"⥨\",\"rx;\":\"℞\",\"Sacute;\":\"Ś\",\"sacute;\":\"ś\",\"sbquo;\":\"‚\",\"Sc;\":\"⪼\",\"sc;\":\"≻\",\"scap;\":\"⪸\",\"Scaron;\":\"Š\",\"scaron;\":\"š\",\"sccue;\":\"≽\",\"scE;\":\"⪴\",\"sce;\":\"⪰\",\"Scedil;\":\"Ş\",\"scedil;\":\"ş\",\"Scirc;\":\"Ŝ\",\"scirc;\":\"ŝ\",\"scnap;\":\"⪺\",\"scnE;\":\"⪶\",\"scnsim;\":\"⋩\",\"scpolint;\":\"⨓\",\"scsim;\":\"≿\",\"Scy;\":\"С\",\"scy;\":\"с\",\"sdot;\":\"⋅\",\"sdotb;\":\"⊡\",\"sdote;\":\"⩦\",\"searhk;\":\"⤥\",\"seArr;\":\"⇘\",\"searr;\":\"↘\",\"searrow;\":\"↘\",\"sect;\":\"§\",sect:\"§\",\"semi;\":\";\",\"seswar;\":\"⤩\",\"setminus;\":\"∖\",\"setmn;\":\"∖\",\"sext;\":\"✶\",\"Sfr;\":\"𝔖\",\"sfr;\":\"𝔰\",\"sfrown;\":\"⌢\",\"sharp;\":\"♯\",\"SHCHcy;\":\"Щ\",\"shchcy;\":\"щ\",\"SHcy;\":\"Ш\",\"shcy;\":\"ш\",\"ShortDownArrow;\":\"↓\",\"ShortLeftArrow;\":\"←\",\"shortmid;\":\"∣\",\"shortparallel;\":\"∥\",\"ShortRightArrow;\":\"→\",\"ShortUpArrow;\":\"↑\",\"shy;\":\"­\",shy:\"­\",\"Sigma;\":\"Σ\",\"sigma;\":\"σ\",\"sigmaf;\":\"ς\",\"sigmav;\":\"ς\",\"sim;\":\"∼\",\"simdot;\":\"⩪\",\"sime;\":\"≃\",\"simeq;\":\"≃\",\"simg;\":\"⪞\",\"simgE;\":\"⪠\",\"siml;\":\"⪝\",\"simlE;\":\"⪟\",\"simne;\":\"≆\",\"simplus;\":\"⨤\",\"simrarr;\":\"⥲\",\"slarr;\":\"←\",\"SmallCircle;\":\"∘\",\"smallsetminus;\":\"∖\",\"smashp;\":\"⨳\",\"smeparsl;\":\"⧤\",\"smid;\":\"∣\",\"smile;\":\"⌣\",\"smt;\":\"⪪\",\"smte;\":\"⪬\",\"smtes;\":\"⪬︀\",\"SOFTcy;\":\"Ь\",\"softcy;\":\"ь\",\"sol;\":\"/\",\"solb;\":\"⧄\",\"solbar;\":\"⌿\",\"Sopf;\":\"𝕊\",\"sopf;\":\"𝕤\",\"spades;\":\"♠\",\"spadesuit;\":\"♠\",\"spar;\":\"∥\",\"sqcap;\":\"⊓\",\"sqcaps;\":\"⊓︀\",\"sqcup;\":\"⊔\",\"sqcups;\":\"⊔︀\",\"Sqrt;\":\"√\",\"sqsub;\":\"⊏\",\"sqsube;\":\"⊑\",\"sqsubset;\":\"⊏\",\"sqsubseteq;\":\"⊑\",\"sqsup;\":\"⊐\",\"sqsupe;\":\"⊒\",\"sqsupset;\":\"⊐\",\"sqsupseteq;\":\"⊒\",\"squ;\":\"□\",\"Square;\":\"□\",\"square;\":\"□\",\"SquareIntersection;\":\"⊓\",\"SquareSubset;\":\"⊏\",\"SquareSubsetEqual;\":\"⊑\",\"SquareSuperset;\":\"⊐\",\"SquareSupersetEqual;\":\"⊒\",\"SquareUnion;\":\"⊔\",\"squarf;\":\"▪\",\"squf;\":\"▪\",\"srarr;\":\"→\",\"Sscr;\":\"𝒮\",\"sscr;\":\"𝓈\",\"ssetmn;\":\"∖\",\"ssmile;\":\"⌣\",\"sstarf;\":\"⋆\",\"Star;\":\"⋆\",\"star;\":\"☆\",\"starf;\":\"★\",\"straightepsilon;\":\"ϵ\",\"straightphi;\":\"ϕ\",\"strns;\":\"¯\",\"Sub;\":\"⋐\",\"sub;\":\"⊂\",\"subdot;\":\"⪽\",\"subE;\":\"⫅\",\"sube;\":\"⊆\",\"subedot;\":\"⫃\",\"submult;\":\"⫁\",\"subnE;\":\"⫋\",\"subne;\":\"⊊\",\"subplus;\":\"⪿\",\"subrarr;\":\"⥹\",\"Subset;\":\"⋐\",\"subset;\":\"⊂\",\"subseteq;\":\"⊆\",\"subseteqq;\":\"⫅\",\"SubsetEqual;\":\"⊆\",\"subsetneq;\":\"⊊\",\"subsetneqq;\":\"⫋\",\"subsim;\":\"⫇\",\"subsub;\":\"⫕\",\"subsup;\":\"⫓\",\"succ;\":\"≻\",\"succapprox;\":\"⪸\",\"succcurlyeq;\":\"≽\",\"Succeeds;\":\"≻\",\"SucceedsEqual;\":\"⪰\",\"SucceedsSlantEqual;\":\"≽\",\"SucceedsTilde;\":\"≿\",\"succeq;\":\"⪰\",\"succnapprox;\":\"⪺\",\"succneqq;\":\"⪶\",\"succnsim;\":\"⋩\",\"succsim;\":\"≿\",\"SuchThat;\":\"∋\",\"Sum;\":\"∑\",\"sum;\":\"∑\",\"sung;\":\"♪\",\"Sup;\":\"⋑\",\"sup;\":\"⊃\",\"sup1;\":\"¹\",sup1:\"¹\",\"sup2;\":\"²\",sup2:\"²\",\"sup3;\":\"³\",sup3:\"³\",\"supdot;\":\"⪾\",\"supdsub;\":\"⫘\",\"supE;\":\"⫆\",\"supe;\":\"⊇\",\"supedot;\":\"⫄\",\"Superset;\":\"⊃\",\"SupersetEqual;\":\"⊇\",\"suphsol;\":\"⟉\",\"suphsub;\":\"⫗\",\"suplarr;\":\"⥻\",\"supmult;\":\"⫂\",\"supnE;\":\"⫌\",\"supne;\":\"⊋\",\"supplus;\":\"⫀\",\"Supset;\":\"⋑\",\"supset;\":\"⊃\",\"supseteq;\":\"⊇\",\"supseteqq;\":\"⫆\",\"supsetneq;\":\"⊋\",\"supsetneqq;\":\"⫌\",\"supsim;\":\"⫈\",\"supsub;\":\"⫔\",\"supsup;\":\"⫖\",\"swarhk;\":\"⤦\",\"swArr;\":\"⇙\",\"swarr;\":\"↙\",\"swarrow;\":\"↙\",\"swnwar;\":\"⤪\",\"szlig;\":\"ß\",szlig:\"ß\",\"Tab;\":\"\\t\",\"target;\":\"⌖\",\"Tau;\":\"Τ\",\"tau;\":\"τ\",\"tbrk;\":\"⎴\",\"Tcaron;\":\"Ť\",\"tcaron;\":\"ť\",\"Tcedil;\":\"Ţ\",\"tcedil;\":\"ţ\",\"Tcy;\":\"Т\",\"tcy;\":\"т\",\"tdot;\":\"⃛\",\"telrec;\":\"⌕\",\"Tfr;\":\"𝔗\",\"tfr;\":\"𝔱\",\"there4;\":\"∴\",\"Therefore;\":\"∴\",\"therefore;\":\"∴\",\"Theta;\":\"Θ\",\"theta;\":\"θ\",\"thetasym;\":\"ϑ\",\"thetav;\":\"ϑ\",\"thickapprox;\":\"≈\",\"thicksim;\":\"∼\",\"ThickSpace;\":\"  \",\"thinsp;\":\" \",\"ThinSpace;\":\" \",\"thkap;\":\"≈\",\"thksim;\":\"∼\",\"THORN;\":\"Þ\",THORN:\"Þ\",\"thorn;\":\"þ\",thorn:\"þ\",\"Tilde;\":\"∼\",\"tilde;\":\"˜\",\"TildeEqual;\":\"≃\",\"TildeFullEqual;\":\"≅\",\"TildeTilde;\":\"≈\",\"times;\":\"×\",times:\"×\",\"timesb;\":\"⊠\",\"timesbar;\":\"⨱\",\"timesd;\":\"⨰\",\"tint;\":\"∭\",\"toea;\":\"⤨\",\"top;\":\"⊤\",\"topbot;\":\"⌶\",\"topcir;\":\"⫱\",\"Topf;\":\"𝕋\",\"topf;\":\"𝕥\",\"topfork;\":\"⫚\",\"tosa;\":\"⤩\",\"tprime;\":\"‴\",\"TRADE;\":\"™\",\"trade;\":\"™\",\"triangle;\":\"▵\",\"triangledown;\":\"▿\",\"triangleleft;\":\"◃\",\"trianglelefteq;\":\"⊴\",\"triangleq;\":\"≜\",\"triangleright;\":\"▹\",\"trianglerighteq;\":\"⊵\",\"tridot;\":\"◬\",\"trie;\":\"≜\",\"triminus;\":\"⨺\",\"TripleDot;\":\"⃛\",\"triplus;\":\"⨹\",\"trisb;\":\"⧍\",\"tritime;\":\"⨻\",\"trpezium;\":\"⏢\",\"Tscr;\":\"𝒯\",\"tscr;\":\"𝓉\",\"TScy;\":\"Ц\",\"tscy;\":\"ц\",\"TSHcy;\":\"Ћ\",\"tshcy;\":\"ћ\",\"Tstrok;\":\"Ŧ\",\"tstrok;\":\"ŧ\",\"twixt;\":\"≬\",\"twoheadleftarrow;\":\"↞\",\"twoheadrightarrow;\":\"↠\",\"Uacute;\":\"Ú\",Uacute:\"Ú\",\"uacute;\":\"ú\",uacute:\"ú\",\"Uarr;\":\"↟\",\"uArr;\":\"⇑\",\"uarr;\":\"↑\",\"Uarrocir;\":\"⥉\",\"Ubrcy;\":\"Ў\",\"ubrcy;\":\"ў\",\"Ubreve;\":\"Ŭ\",\"ubreve;\":\"ŭ\",\"Ucirc;\":\"Û\",Ucirc:\"Û\",\"ucirc;\":\"û\",ucirc:\"û\",\"Ucy;\":\"У\",\"ucy;\":\"у\",\"udarr;\":\"⇅\",\"Udblac;\":\"Ű\",\"udblac;\":\"ű\",\"udhar;\":\"⥮\",\"ufisht;\":\"⥾\",\"Ufr;\":\"𝔘\",\"ufr;\":\"𝔲\",\"Ugrave;\":\"Ù\",Ugrave:\"Ù\",\"ugrave;\":\"ù\",ugrave:\"ù\",\"uHar;\":\"⥣\",\"uharl;\":\"↿\",\"uharr;\":\"↾\",\"uhblk;\":\"▀\",\"ulcorn;\":\"⌜\",\"ulcorner;\":\"⌜\",\"ulcrop;\":\"⌏\",\"ultri;\":\"◸\",\"Umacr;\":\"Ū\",\"umacr;\":\"ū\",\"uml;\":\"¨\",uml:\"¨\",\"UnderBar;\":\"_\",\"UnderBrace;\":\"⏟\",\"UnderBracket;\":\"⎵\",\"UnderParenthesis;\":\"⏝\",\"Union;\":\"⋃\",\"UnionPlus;\":\"⊎\",\"Uogon;\":\"Ų\",\"uogon;\":\"ų\",\"Uopf;\":\"𝕌\",\"uopf;\":\"𝕦\",\"UpArrow;\":\"↑\",\"Uparrow;\":\"⇑\",\"uparrow;\":\"↑\",\"UpArrowBar;\":\"⤒\",\"UpArrowDownArrow;\":\"⇅\",\"UpDownArrow;\":\"↕\",\"Updownarrow;\":\"⇕\",\"updownarrow;\":\"↕\",\"UpEquilibrium;\":\"⥮\",\"upharpoonleft;\":\"↿\",\"upharpoonright;\":\"↾\",\"uplus;\":\"⊎\",\"UpperLeftArrow;\":\"↖\",\"UpperRightArrow;\":\"↗\",\"Upsi;\":\"ϒ\",\"upsi;\":\"υ\",\"upsih;\":\"ϒ\",\"Upsilon;\":\"Υ\",\"upsilon;\":\"υ\",\"UpTee;\":\"⊥\",\"UpTeeArrow;\":\"↥\",\"upuparrows;\":\"⇈\",\"urcorn;\":\"⌝\",\"urcorner;\":\"⌝\",\"urcrop;\":\"⌎\",\"Uring;\":\"Ů\",\"uring;\":\"ů\",\"urtri;\":\"◹\",\"Uscr;\":\"𝒰\",\"uscr;\":\"𝓊\",\"utdot;\":\"⋰\",\"Utilde;\":\"Ũ\",\"utilde;\":\"ũ\",\"utri;\":\"▵\",\"utrif;\":\"▴\",\"uuarr;\":\"⇈\",\"Uuml;\":\"Ü\",Uuml:\"Ü\",\"uuml;\":\"ü\",uuml:\"ü\",\"uwangle;\":\"⦧\",\"vangrt;\":\"⦜\",\"varepsilon;\":\"ϵ\",\"varkappa;\":\"ϰ\",\"varnothing;\":\"∅\",\"varphi;\":\"ϕ\",\"varpi;\":\"ϖ\",\"varpropto;\":\"∝\",\"vArr;\":\"⇕\",\"varr;\":\"↕\",\"varrho;\":\"ϱ\",\"varsigma;\":\"ς\",\"varsubsetneq;\":\"⊊︀\",\"varsubsetneqq;\":\"⫋︀\",\"varsupsetneq;\":\"⊋︀\",\"varsupsetneqq;\":\"⫌︀\",\"vartheta;\":\"ϑ\",\"vartriangleleft;\":\"⊲\",\"vartriangleright;\":\"⊳\",\"Vbar;\":\"⫫\",\"vBar;\":\"⫨\",\"vBarv;\":\"⫩\",\"Vcy;\":\"В\",\"vcy;\":\"в\",\"VDash;\":\"⊫\",\"Vdash;\":\"⊩\",\"vDash;\":\"⊨\",\"vdash;\":\"⊢\",\"Vdashl;\":\"⫦\",\"Vee;\":\"⋁\",\"vee;\":\"∨\",\"veebar;\":\"⊻\",\"veeeq;\":\"≚\",\"vellip;\":\"⋮\",\"Verbar;\":\"‖\",\"verbar;\":\"|\",\"Vert;\":\"‖\",\"vert;\":\"|\",\"VerticalBar;\":\"∣\",\"VerticalLine;\":\"|\",\"VerticalSeparator;\":\"❘\",\"VerticalTilde;\":\"≀\",\"VeryThinSpace;\":\" \",\"Vfr;\":\"𝔙\",\"vfr;\":\"𝔳\",\"vltri;\":\"⊲\",\"vnsub;\":\"⊂⃒\",\"vnsup;\":\"⊃⃒\",\"Vopf;\":\"𝕍\",\"vopf;\":\"𝕧\",\"vprop;\":\"∝\",\"vrtri;\":\"⊳\",\"Vscr;\":\"𝒱\",\"vscr;\":\"𝓋\",\"vsubnE;\":\"⫋︀\",\"vsubne;\":\"⊊︀\",\"vsupnE;\":\"⫌︀\",\"vsupne;\":\"⊋︀\",\"Vvdash;\":\"⊪\",\"vzigzag;\":\"⦚\",\"Wcirc;\":\"Ŵ\",\"wcirc;\":\"ŵ\",\"wedbar;\":\"⩟\",\"Wedge;\":\"⋀\",\"wedge;\":\"∧\",\"wedgeq;\":\"≙\",\"weierp;\":\"℘\",\"Wfr;\":\"𝔚\",\"wfr;\":\"𝔴\",\"Wopf;\":\"𝕎\",\"wopf;\":\"𝕨\",\"wp;\":\"℘\",\"wr;\":\"≀\",\"wreath;\":\"≀\",\"Wscr;\":\"𝒲\",\"wscr;\":\"𝓌\",\"xcap;\":\"⋂\",\"xcirc;\":\"◯\",\"xcup;\":\"⋃\",\"xdtri;\":\"▽\",\"Xfr;\":\"𝔛\",\"xfr;\":\"𝔵\",\"xhArr;\":\"⟺\",\"xharr;\":\"⟷\",\"Xi;\":\"Ξ\",\"xi;\":\"ξ\",\"xlArr;\":\"⟸\",\"xlarr;\":\"⟵\",\"xmap;\":\"⟼\",\"xnis;\":\"⋻\",\"xodot;\":\"⨀\",\"Xopf;\":\"𝕏\",\"xopf;\":\"𝕩\",\"xoplus;\":\"⨁\",\"xotime;\":\"⨂\",\"xrArr;\":\"⟹\",\"xrarr;\":\"⟶\",\"Xscr;\":\"𝒳\",\"xscr;\":\"𝓍\",\"xsqcup;\":\"⨆\",\"xuplus;\":\"⨄\",\"xutri;\":\"△\",\"xvee;\":\"⋁\",\"xwedge;\":\"⋀\",\"Yacute;\":\"Ý\",Yacute:\"Ý\",\"yacute;\":\"ý\",yacute:\"ý\",\"YAcy;\":\"Я\",\"yacy;\":\"я\",\"Ycirc;\":\"Ŷ\",\"ycirc;\":\"ŷ\",\"Ycy;\":\"Ы\",\"ycy;\":\"ы\",\"yen;\":\"¥\",yen:\"¥\",\"Yfr;\":\"𝔜\",\"yfr;\":\"𝔶\",\"YIcy;\":\"Ї\",\"yicy;\":\"ї\",\"Yopf;\":\"𝕐\",\"yopf;\":\"𝕪\",\"Yscr;\":\"𝒴\",\"yscr;\":\"𝓎\",\"YUcy;\":\"Ю\",\"yucy;\":\"ю\",\"Yuml;\":\"Ÿ\",\"yuml;\":\"ÿ\",yuml:\"ÿ\",\"Zacute;\":\"Ź\",\"zacute;\":\"ź\",\"Zcaron;\":\"Ž\",\"zcaron;\":\"ž\",\"Zcy;\":\"З\",\"zcy;\":\"з\",\"Zdot;\":\"Ż\",\"zdot;\":\"ż\",\"zeetrf;\":\"ℨ\",\"ZeroWidthSpace;\":\"​\",\"Zeta;\":\"Ζ\",\"zeta;\":\"ζ\",\"Zfr;\":\"ℨ\",\"zfr;\":\"𝔷\",\"ZHcy;\":\"Ж\",\"zhcy;\":\"ж\",\"zigrarr;\":\"⇝\",\"Zopf;\":\"ℤ\",\"zopf;\":\"𝕫\",\"Zscr;\":\"𝒵\",\"zscr;\":\"𝓏\",\"zwj;\":\"‍\",\"zwnj;\":\"‌\"},tr=yt(),nr=function(){function e(){this.completionParticipants=[]}return e.prototype.setCompletionParticipants=function(e){this.completionParticipants=e||[]},e.prototype.doComplete=function(e,t,n,r){var i={isIncomplete:!1,items:[]},o=this.completionParticipants,a=Xn.filter(function(t){return t.isApplicable(e.languageId)&&(!r||!1!==r[t.getId()])}),s=e.getText(),u=e.offsetAt(t),l=n.findNodeBefore(u);if(!l)return i;var c,h=Ut(s,l.start),d=\"\";function f(t,n){return void 0===n&&(n=u),t>u&&(t=u),{start:e.positionAt(t),end:e.positionAt(n)}}function p(e,t){var n=f(e,t);return a.forEach(function(e){e.collectTags(function(e,t){i.items.push({label:e,kind:xn.Property,documentation:t,textEdit:mn.replace(n,e),insertTextFormat:Sn.PlainText})})}),i}function g(e){for(var t=e;t>0;){var n=s.charAt(t-1);if(\"\\n\\r\".indexOf(n)>=0)return s.substring(t,e);if(!rr(n))return null;t--}return s.substring(0,e)}function m(e,t,n){void 0===n&&(n=u);var r=f(e,n),o=ir(s,n,mt.WithinEndTag,gt.EndTagClose)?\"\":\">\",c=l;for(t&&(c=c.parent);c;){var h=c.tag;if(h&&(!c.closed||c.endTagStart&&c.endTagStart>u)){var d={label:\"/\"+h,kind:xn.Property,filterText:\"/\"+h+o,textEdit:mn.replace(r,\"/\"+h+o),insertTextFormat:Sn.PlainText},p=g(c.start),m=g(e-1);if(null!==p&&null!==m&&p!==m){var b=p+\"</\"+h+o;d.textEdit=mn.replace(f(e-1-m.length),b),d.filterText=m+\"</\"+h+o}return i.items.push(d),i}c=c.parent}return t?i:(a.forEach(function(e){e.collectTags(function(e,t){i.items.push({label:\"/\"+e,kind:xn.Property,documentation:t,filterText:\"/\"+e+o,textEdit:mn.replace(r,\"/\"+e+o),insertTextFormat:Sn.PlainText})})}),i)}function b(t,n){if(r&&r.hideAutoCompleteProposals)return i;if(!$t(n)){var o=e.positionAt(t);i.items.push({label:\"</\"+n+\">\",kind:xn.Property,filterText:\"</\"+n+\">\",textEdit:mn.insert(o,\"$0</\"+n+\">\"),insertTextFormat:Sn.Snippet})}return i}function _(e,t){return p(e,t),m(e,!0,t),i}function v(e,t){void 0===t&&(t=u);for(var r=u;r<t&&\"<\"!==s[r];)r++;var o=f(e,r),l=ir(s,t,mt.AfterAttributeName,gt.DelimiterAssign)?\"\":'=\"$1\"',c=d.toLowerCase(),h=Object.create(null);return a.forEach(function(e){e.collectAttributes(c,function(e,t){if(!h[e]){h[e]=!0;var n,r=e;\"v\"!==t&&l.length&&(r+=l,t&&(n={title:\"Suggest\",command:\"editor.action.triggerSuggest\"})),i.items.push({label:e,kind:\"handler\"===t?xn.Function:xn.Value,textEdit:mn.replace(o,r),insertTextFormat:Sn.Snippet,command:n})}})}),function(e,t){var r=\"data-\",o={};o[r]=r+'$1=\"$2\"',n&&n.roots.forEach(function(e){return function e(n){n.attributeNames.forEach(function(e){!Wt(e,r)||o[e]||t[e]||(o[e]=e+'=\"$1\"')});n.children.forEach(function(t){return e(t)})}(e)});Object.keys(o).forEach(function(t){return i.items.push({label:t,kind:xn.Value,textEdit:mn.replace(e,o[t]),insertTextFormat:Sn.Snippet})})}(o,h),i}function y(n,r){var l,p,g,m;if(void 0===r&&(r=u),u>n&&u<=r&&(m=s[n],/^[\"']*$/.test(m))){var b=n+1,_=r;r>n&&s[r-1]===s[n]&&_--;var v=function(e,t,n){for(;t>n&&!rr(e[t-1]);)t--;return t}(s,u,b),y=function(e,t,n){for(;t<n&&!rr(e[t]);)t++;return t}(s,u,_);l=f(v,y),g=u>=b&&u<=_?s.substring(b,u):\"\",p=!1}else l=f(n,r),g=s.substring(n,u),p=!0;var w=d.toLowerCase(),C=c.toLowerCase();if(o.length>0)for(var E=f(n,r),x=0,S=o;x<S.length;x++){var k=S[x];k.onHtmlAttributeValue&&k.onHtmlAttributeValue({document:e,position:t,tag:w,attribute:C,value:g,range:E})}h.getTokenText();return a.forEach(function(e){e.collectValues(w,C,function(e){var t=p?'\"'+e+'\"':e;i.items.push({label:e,filterText:t,kind:xn.Unit,textEdit:mn.replace(l,t),insertTextFormat:Sn.PlainText})})}),T(),i}function w(e){return u===h.getTokenEnd()&&(E=h.scan())===e&&h.getTokenOffset()===u?h.getTokenEnd():u}function C(){for(var n=0,r=o;n<r.length;n++){var i=r[n];i.onHtmlContent&&i.onHtmlContent({document:e,position:t})}return T()}function T(){for(var e,n,r,o=u-1,a=t.character;o>=0&&Yt(s,o);)o--,a--;if(o>=0&&\"&\"===s[o]){var l=on.create(rn.create(t.line,a-1),t);for(var c in er)if(n=\";\",r=void 0,(r=(e=c).length-n.length)>0?e.lastIndexOf(n)===r:0===r&&e===n){var h=\"&\"+c;i.items.push({label:h,kind:xn.Keyword,documentation:tr(\"entity.propose\",\"Character entity representing '\"+er[c]+\"'\"),textEdit:mn.replace(l,h),insertTextFormat:Sn.PlainText})}}return i}for(var E=h.scan();E!==gt.EOS&&h.getTokenOffset()<=u;){switch(E){case gt.StartTagOpen:if(h.getTokenEnd()===u){var x=w(gt.StartTag);return _(u,x)}break;case gt.StartTag:if(h.getTokenOffset()<=u&&u<=h.getTokenEnd())return p(h.getTokenOffset(),h.getTokenEnd());d=h.getTokenText();break;case gt.AttributeName:if(h.getTokenOffset()<=u&&u<=h.getTokenEnd())return v(h.getTokenOffset(),h.getTokenEnd());c=h.getTokenText();break;case gt.DelimiterAssign:if(h.getTokenEnd()===u){x=w(gt.AttributeValue);return y(u,x)}break;case gt.AttributeValue:if(h.getTokenOffset()<=u&&u<=h.getTokenEnd())return y(h.getTokenOffset(),h.getTokenEnd());break;case gt.Whitespace:if(u<=h.getTokenEnd())switch(h.getScannerState()){case mt.AfterOpeningStartTag:return _(h.getTokenOffset(),w(gt.StartTag));case mt.WithinTag:case mt.AfterAttributeName:return v(h.getTokenEnd());case mt.BeforeAttributeValue:return y(h.getTokenEnd());case mt.AfterOpeningEndTag:return m(h.getTokenOffset()-1,!1);case mt.WithinContent:return C()}break;case gt.EndTagOpen:if(u<=h.getTokenEnd())return m(h.getTokenOffset()+1,!1,w(gt.EndTag));break;case gt.EndTag:if(u<=h.getTokenEnd())for(var S=h.getTokenOffset()-1;S>=0;){var k=s.charAt(S);if(\"/\"===k)return m(S,!1,h.getTokenEnd());if(!rr(k))break;S--}break;case gt.StartTagClose:if(u<=h.getTokenEnd()&&d)return b(h.getTokenEnd(),d);break;case gt.Content:if(u<=h.getTokenEnd())return C();break;default:if(u<=h.getTokenEnd())return i}E=h.scan()}return i},e.prototype.doTagComplete=function(e,t,n){var r=e.offsetAt(t);if(r<=0)return null;var i=e.getText().charAt(r-1);if(\">\"===i){if((a=n.findNodeBefore(r))&&a.tag&&!$t(a.tag)&&a.start<r&&(!a.endTagStart||a.endTagStart>r))for(var o=(s=Ut(e.getText(),a.start)).scan();o!==gt.EOS&&s.getTokenEnd()<=r;){if(o===gt.StartTagClose&&s.getTokenEnd()===r)return\"$0</\"+a.tag+\">\";o=s.scan()}}else if(\"/\"===i){for(var a=n.findNodeBefore(r);a&&a.closed;)a=a.parent;if(a&&a.tag){var s;for(o=(s=Ut(e.getText(),a.start)).scan();o!==gt.EOS&&s.getTokenEnd()<=r;){if(o===gt.EndTagOpen&&s.getTokenEnd()===r)return a.tag+\">\";o=s.scan()}}}return null},e}();function rr(e){return/^\\s*$/.test(e)}function ir(e,t,n,r){for(var i=Ut(e,t,n),o=i.scan();o===gt.Whitespace;)o=i.scan();return o===r}function or(e,t,n){var r=e.offsetAt(t),i=n.findNodeAt(r);if(!i||!i.tag)return null;var o=Xn.filter(function(t){return t.isApplicable(e.languageId)});function a(e,t,n){e=e.toLowerCase();for(var r=function(r){var i=null;if(r.collectTags(function(r,o){r===e&&(i={contents:[{language:\"html\",value:n?\"<\"+e+\">\":\"</\"+e+\">\"},Ln.fromPlainText(o)],range:t})}),i)return{value:i}},i=0,a=o;i<a.length;i++){var s=r(a[i]);if(\"object\"==typeof s)return s.value}return null}function s(t,n){for(var i=Ut(e.getText(),n),o=i.scan();o!==gt.EOS&&(i.getTokenEnd()<r||i.getTokenEnd()===r&&o!==t);)o=i.scan();return o===t&&r<=i.getTokenEnd()?{start:e.positionAt(i.getTokenOffset()),end:e.positionAt(i.getTokenEnd())}:null}if(i.endTagStart&&r>=i.endTagStart){var u=s(gt.EndTag,i.endTagStart);return u?a(i.tag,u,!1):null}var l=s(gt.StartTag,i.start);return l?a(i.tag,l,!0):null}function ar(e,t){return e}const sr=function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){\"undefined\"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:\"Module\"}),Object.defineProperty(e,\"__esModule\",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&\"object\"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,\"default\",{enumerable:!0,value:e}),2&t&&\"string\"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,\"a\",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p=\"\",n(n.s=8)}([,,function(e,t){e.exports.mergeOpts=function(e,t){var n,r={};for(n in e)n!==t&&(r[n]=e[n]);if(t in e)for(n in e[t])r[n]=e[t][n];return r}},function(e,t){var n=\"ªµºÀ-ÖØ-öø-ˁˆ-ˑˠ-ˤˬˮͰ-ʹͶͷͺ-ͽΆΈ-ΊΌΎ-ΡΣ-ϵϷ-ҁҊ-ԧԱ-Ֆՙա-ևא-תװ-ײؠ-يٮٯٱ-ۓەۥۦۮۯۺ-ۼۿܐܒ-ܯݍ-ޥޱߊ-ߪߴߵߺࠀ-ࠕࠚࠤࠨࡀ-ࡘࢠࢢ-ࢬऄ-हऽॐक़-ॡॱ-ॷॹ-ॿঅ-ঌএঐও-নপ-রলশ-হঽৎড়ঢ়য়-ৡৰৱਅ-ਊਏਐਓ-ਨਪ-ਰਲਲ਼ਵਸ਼ਸਹਖ਼-ੜਫ਼ੲ-ੴઅ-ઍએ-ઑઓ-નપ-રલળવ-હઽૐૠૡଅ-ଌଏଐଓ-ନପ-ରଲଳଵ-ହଽଡ଼ଢ଼ୟ-ୡୱஃஅ-ஊஎ-ஐஒ-கஙசஜஞடணதந-பம-ஹௐఅ-ఌఎ-ఐఒ-నప-ళవ-హఽౘౙౠౡಅ-ಌಎ-ಐಒ-ನಪ-ಳವ-ಹಽೞೠೡೱೲഅ-ഌഎ-ഐഒ-ഺഽൎൠൡൺ-ൿඅ-ඖක-නඳ-රලව-ෆก-ะาำเ-ๆກຂຄງຈຊຍດ-ທນ-ຟມ-ຣລວສຫອ-ະາຳຽເ-ໄໆໜ-ໟༀཀ-ཇཉ-ཬྈ-ྌက-ဪဿၐ-ၕၚ-ၝၡၥၦၮ-ၰၵ-ႁႎႠ-ჅჇჍა-ჺჼ-ቈቊ-ቍቐ-ቖቘቚ-ቝበ-ኈኊ-ኍነ-ኰኲ-ኵኸ-ኾዀዂ-ዅወ-ዖዘ-ጐጒ-ጕጘ-ፚᎀ-ᎏᎠ-Ᏼᐁ-ᙬᙯ-ᙿᚁ-ᚚᚠ-ᛪᛮ-ᛰᜀ-ᜌᜎ-ᜑᜠ-ᜱᝀ-ᝑᝠ-ᝬᝮ-ᝰក-ឳៗៜᠠ-ᡷᢀ-ᢨᢪᢰ-ᣵᤀ-ᤜᥐ-ᥭᥰ-ᥴᦀ-ᦫᧁ-ᧇᨀ-ᨖᨠ-ᩔᪧᬅ-ᬳᭅ-ᭋᮃ-ᮠᮮᮯᮺ-ᯥᰀ-ᰣᱍ-ᱏᱚ-ᱽᳩ-ᳬᳮ-ᳱᳵᳶᴀ-ᶿḀ-ἕἘ-Ἕἠ-ὅὈ-Ὅὐ-ὗὙὛὝὟ-ώᾀ-ᾴᾶ-ᾼιῂ-ῄῆ-ῌῐ-ΐῖ-Ίῠ-Ῥῲ-ῴῶ-ῼⁱⁿₐ-ₜℂℇℊ-ℓℕℙ-ℝℤΩℨK-ℭℯ-ℹℼ-ℿⅅ-ⅉⅎⅠ-ↈⰀ-Ⱞⰰ-ⱞⱠ-ⳤⳫ-ⳮⳲⳳⴀ-ⴥⴧⴭⴰ-ⵧⵯⶀ-ⶖⶠ-ⶦⶨ-ⶮⶰ-ⶶⶸ-ⶾⷀ-ⷆⷈ-ⷎⷐ-ⷖⷘ-ⷞⸯ々-〇〡-〩〱-〵〸-〼ぁ-ゖゝ-ゟァ-ヺー-ヿㄅ-ㄭㄱ-ㆎㆠ-ㆺㇰ-ㇿ㐀-䶵一-鿌ꀀ-ꒌꓐ-ꓽꔀ-ꘌꘐ-ꘟꘪꘫꙀ-ꙮꙿ-ꚗꚠ-ꛯꜗ-ꜟꜢ-ꞈꞋ-ꞎꞐ-ꞓꞠ-Ɦꟸ-ꠁꠃ-ꠅꠇ-ꠊꠌ-ꠢꡀ-ꡳꢂ-ꢳꣲ-ꣷꣻꤊ-ꤥꤰ-ꥆꥠ-ꥼꦄ-ꦲꧏꨀ-ꨨꩀ-ꩂꩄ-ꩋꩠ-ꩶꩺꪀ-ꪯꪱꪵꪶꪹ-ꪽꫀꫂꫛ-ꫝꫠ-ꫪꫲ-ꫴꬁ-ꬆꬉ-ꬎꬑ-ꬖꬠ-ꬦꬨ-ꬮꯀ-ꯢ가-힣ힰ-ퟆퟋ-ퟻ豈-舘並-龎ﬀ-ﬆﬓ-ﬗיִײַ-ﬨשׁ-זּטּ-לּמּנּסּףּפּצּ-ﮱﯓ-ﴽﵐ-ﶏﶒ-ﷇﷰ-ﷻﹰ-ﹴﹶ-ﻼＡ-Ｚａ-ｚｦ-ﾾￂ-ￇￊ-ￏￒ-ￗￚ-ￜ\",r=\"̀-ͯ҃-֑҇-ׇֽֿׁׂׅׄؐ-ؚؠ-ىٲ-ۓۧ-ۨۻ-ۼܰ-݊ࠀ-ࠔࠛ-ࠣࠥ-ࠧࠩ-࠭ࡀ-ࡗࣤ-ࣾऀ-ःऺ-़ा-ॏ॑-ॗॢ-ॣ०-९ঁ-ঃ়া-ৄেৈৗয়-ৠਁ-ਃ਼ਾ-ੂੇੈੋ-੍ੑ੦-ੱੵઁ-ઃ઼ા-ૅે-ૉો-્ૢ-ૣ૦-૯ଁ-ଃ଼ା-ୄେୈୋ-୍ୖୗୟ-ୠ୦-୯ஂா-ூெ-ைொ-்ௗ௦-௯ఁ-ఃె-ైొ-్ౕౖౢ-ౣ౦-౯ಂಃ಼ಾ-ೄೆ-ೈೊ-್ೕೖೢ-ೣ೦-೯ംഃെ-ൈൗൢ-ൣ൦-൯ංඃ්ා-ුූෘ-ෟෲෳิ-ฺเ-ๅ๐-๙ິ-ູ່-ໍ໐-໙༘༙༠-༩༹༵༷ཁ-ཇཱ-྄྆-྇ྍ-ྗྙ-ྼ࿆က-ဩ၀-၉ၧ-ၭၱ-ၴႂ-ႍႏ-ႝ፝-፟ᜎ-ᜐᜠ-ᜰᝀ-ᝐᝲᝳក-ឲ៝០-៩᠋-᠍᠐-᠙ᤠ-ᤫᤰ-᤻ᥑ-ᥭᦰ-ᧀᧈ-ᧉ᧐-᧙ᨀ-ᨕᨠ-ᩓ᩠-᩿᩼-᪉᪐-᪙ᭆ-ᭋ᭐-᭙᭫-᭳᮰-᮹᯦-᯳ᰀ-ᰢ᱀-᱉ᱛ-ᱽ᳐-᳒ᴀ-ᶾḁ-ἕ‌‍‿⁀⁔⃐-⃥⃜⃡-⃰ⶁ-ⶖⷠ-ⷿ〡-〨゙゚Ꙁ-ꙭꙴ-꙽ꚟ꛰-꛱ꟸ-ꠀ꠆ꠋꠣ-ꠧꢀ-ꢁꢴ-꣄꣐-꣙ꣳ-ꣷ꤀-꤉ꤦ-꤭ꤰ-ꥅꦀ-ꦃ꦳-꧀ꨀ-ꨧꩀ-ꩁꩌ-ꩍ꩐-꩙ꩻꫠ-ꫩꫲ-ꫳꯀ-ꯡ꯬꯭꯰-꯹ﬠ-ﬨ︀-️︠-︦︳︴﹍-﹏０-９＿\",i=new RegExp(\"[$@A-Z_a-z\"+n+\"]\"),o=new RegExp(\"[$0-9A-Z_a-z\"+n+r+\"]\");t.identifier=new RegExp(\"[$@A-Z_a-z\"+n+\"][$0-9A-Z_a-z\"+n+r+\"]*\",\"g\"),t.newline=/[\\n\\r\\u2028\\u2029]/,t.lineBreak=new RegExp(\"\\r\\n|\"+t.newline.source),t.allLineBreaks=new RegExp(t.lineBreak.source,\"g\"),t.isIdentifierStart=function(e){return i.test(String.fromCharCode(e))},t.isIdentifierChar=function(e){return o.test(String.fromCharCode(e))}},function(e,t){function n(e){this._character_count=0,this._indent_count=-1,this._items=[];var t=!0;this.set_indent=function(t){this._character_count=e.baseIndentLength+t*e.indent_length,this._indent_count=t},this.get_character_count=function(){return this._character_count},this.is_empty=function(){return t},this.last=function(){return this._empty?null:this._items[this._items.length-1]},this.push=function(e){this._items.push(e),this._character_count+=e.length,t=!1},this.pop=function(){var e=null;return t||(e=this._items.pop(),this._character_count-=e.length,t=0===this._items.length),e},this.remove_indent=function(){this._indent_count>0&&(this._indent_count-=1,this._character_count-=e.indent_length)},this.trim=function(){for(;\" \"===this.last();)this._items.pop(),this._character_count-=1;t=0===this._items.length},this.toString=function(){var t=\"\";return this._empty||(this._indent_count>=0&&(t=e.indent_cache[this._indent_count]),t+=this._items.join(\"\")),t}}e.exports.Output=function(e,t){t=t||\"\",this.indent_cache=[t],this.baseIndentLength=t.length,this.indent_length=e.length,this.raw=!1,this._lines=[],this.baseIndentString=t,this.indent_string=e,this.previous_line=null,this.current_line=null,this.space_before_token=!1,this.add_outputline=function(){this.previous_line=this.current_line,this.current_line=new n(this),this._lines.push(this.current_line)},this.add_outputline(),this.get_line_number=function(){return this._lines.length},this.add_new_line=function(e){return!(1===this.get_line_number()&&this.just_added_newline()||!e&&this.just_added_newline()||(this.raw||this.add_outputline(),0))},this.get_code=function(e,t){var n=this._lines.join(\"\\n\").replace(/[\\r\\n\\t ]+$/,\"\");return e&&(n+=\"\\n\"),\"\\n\"!==t&&(n=n.replace(/[\\n]/g,t)),n},this.set_indent=function(e){if(this._lines.length>1){for(;e>=this.indent_cache.length;)this.indent_cache.push(this.indent_cache[this.indent_cache.length-1]+this.indent_string);return this.current_line.set_indent(e),!0}return this.current_line.set_indent(0),!1},this.add_raw_token=function(e){for(var t=0;t<e.newlines;t++)this.add_outputline();this.current_line.push(e.whitespace_before),this.current_line.push(e.text),this.space_before_token=!1},this.add_token=function(e){this.add_space_before_token(),this.current_line.push(e)},this.add_space_before_token=function(){this.space_before_token&&!this.just_added_newline()&&this.current_line.push(\" \"),this.space_before_token=!1},this.remove_indent=function(e){for(var t=this._lines.length;e<t;)this._lines[e].remove_indent(),e++},this.trim=function(n){for(n=void 0!==n&&n,this.current_line.trim(e,t);n&&this._lines.length>1&&this.current_line.is_empty();)this._lines.pop(),this.current_line=this._lines[this._lines.length-1],this.current_line.trim();this.previous_line=this._lines.length>1?this._lines[this._lines.length-2]:null},this.just_added_newline=function(){return this.current_line.is_empty()},this.just_added_blankline=function(){return!!this.just_added_newline()&&(1===this._lines.length||this._lines[this._lines.length-2].is_empty())}}},,function(e,t){e.exports.InputScanner=function(e){var t=e||\"\",n=t.length,r=0;this.back=function(){r>0&&(r-=1)},this.hasNext=function(){return r<n},this.next=function(){var e=null;return this.hasNext()&&(e=t.charAt(r),r+=1),e},this.peek=function(e){var i=null;return e=e||0,(e+=r)>=0&&e<n&&(i=t.charAt(e)),i},this.test=function(e,i){if(i=i||0,i+=r,e.lastIndex=i,i>=0&&i<n){var o=e.exec(t);return o&&o.index===i}return!1},this.testChar=function(e,t){var n=this.peek(t);return null!==n&&e.test(n)},this.match=function(e){e.lastIndex=r;var n=e.exec(t);return n&&n.index===r?r+=n[0].length:n=null,n},this.readWhile=function(e){var t=\"\",n=this.match(e);return n&&(t=n[0]),t},this.readUntil=function(e){var i,o=r;e.lastIndex=r;var a=e.exec(t);return o=a?a.index:n,i=t.substring(r,o),r=o,i},this.readUntilAfter=function(e){var i,o=r;e.lastIndex=r;var a=e.exec(t);return o=a?a.index+a[0].length:n,i=t.substring(r,o),r=o,i},this.peekUntilAfter=function(e){var t=r,n=this.readUntilAfter(e);return r=t,n},this.lookBack=function(e){var n=r-1;return n>=e.length&&t.substring(n-e.length,n).toLowerCase()===e}}},,function(e,t,n){var r=n(9).Beautifier;e.exports=function(e,t){return new r(e,t).beautify()}},function(e,t,n){var r=n(2).mergeOpts,i=n(3),o=n(4).Output,a=n(6).InputScanner,s=i.lineBreak,u=i.allLineBreaks;e.exports.Beautifier=function(e,t){e=e||\"\";var n=(t=r(t=t||{},\"css\")).indent_size?parseInt(t.indent_size,10):4,i=t.indent_char||\" \",l=void 0!==t.preserve_newlines&&t.preserve_newlines,c=void 0===t.selector_separator_newline||t.selector_separator_newline,h=void 0!==t.end_with_newline&&t.end_with_newline,d=void 0===t.newline_between_rules||t.newline_between_rules,f=void 0!==t.space_around_combinator&&t.space_around_combinator;f=f||void 0!==t.space_around_selector_separator&&t.space_around_selector_separator;var p=t.eol?t.eol:\"auto\";t.indent_with_tabs&&(i=\"\\t\",n=1),\"auto\"===p&&(p=\"\\n\",e&&s.test(e||\"\")&&(p=e.match(s)[0])),p=p.replace(/\\\\r/,\"\\r\").replace(/\\\\n/,\"\\n\"),e=e.replace(u,\"\\n\");var g,m,b=/\\s/,_=/(?:\\s|\\n)+/g,v=/\\/\\*(?:[\\s\\S]*?)((?:\\*\\/)|$)/g,y=/\\/\\/(?:[^\\n\\r\\u2028\\u2029]*)/g,w=0;function C(e){var t=\"\";for(g=m.next();g;){if(t+=g,\"\\\\\"===g)t+=m.next();else if(-1!==e.indexOf(g)||\"\\n\"===g)break;g=m.next()}return t}function T(e){for(var t=b.test(m.peek()),n=!0;b.test(m.peek());)g=m.next(),e&&\"\\n\"===g&&(l||n)&&(n=!1,L.add_new_line(!0));return t}function E(){for(var e=0,t=1,n=m.peek(t);n;){if(\"{\"===n)return!0;if(\"(\"===n)e+=1;else if(\")\"===n){if(0===e)return!1;e-=1}else if(\";\"===n||\"}\"===n)return!1;t++,n=m.peek(t)}return!1}var x=\"\",S=0;if(e&&e.length){for(;\" \"===e.charAt(S)||\"\\t\"===e.charAt(S);)S+=1;x=e.substring(0,S),e=e.substring(S)}var k,A,L,N=new Array(n+1).join(i);function O(e){L.just_added_newline()&&L.set_indent(k),L.add_token(e)}function P(e){e&&(L.space_before_token=!0)}this.beautify=function(){L=new o(N,x),m=new a(e),k=0,A=0,g=null,w=0;for(var t=!1,n=!1,r=!1,i=!1;;){var s=\"\"!==m.readWhile(_);if(!(g=m.next()))break;if(\"/\"===g&&\"*\"===m.peek())L.add_new_line(),m.back(),O(m.readWhile(v)),T(!0),L.add_new_line();else if(\"/\"===g&&\"/\"===m.peek())L.space_before_token=!0,m.back(),O(m.readWhile(y)),T(!0);else if(\"@\"===g)if(P(s),\"{\"===m.peek())O(g+C(\"}\"));else{O(g);var u=m.peekUntilAfter(/[: ,;{}()[\\]\\/='\"]/g);u.match(/[ :]$/)&&(O(u=C(\": \").replace(/\\s$/,\"\")),L.space_before_token=!0),\"extend\"===(u=u.replace(/\\s$/,\"\"))&&(i=!0),u in this.NESTED_AT_RULE?(A+=1,u in this.CONDITIONAL_GROUP_RULE&&(r=!0)):t||0!==w||-1===u.indexOf(\":\")||(n=!0)}else\"#\"===g&&\"{\"===m.peek()?(P(s),O(g+C(\"}\"))):\"{\"===g?m.match(/[\\t\\n ]*}/g)?(L.space_before_token=!0,O(\"{}\"),T(!0),L.add_new_line(),d&&0===k&&!L.just_added_blankline()&&L.add_new_line(!0)):(k++,L.space_before_token=!0,O(g),T(!0),L.add_new_line(),r?(r=!1,t=k>A):t=k>=A):\"}\"===g?(k>0&&k--,L.add_new_line(),O(g),t=!1,n=!1,A&&A--,T(!0),L.add_new_line(),d&&0===k&&!L.just_added_blankline()&&L.add_new_line(!0)):\":\"===g?!t&&!r||m.lookBack(\"&\")||E()||m.lookBack(\"(\")||i?(m.lookBack(\" \")&&(L.space_before_token=!0),\":\"===m.peek()?(g=m.next(),O(\"::\")):O(\":\")):(O(\":\"),n||(n=!0,L.space_before_token=!0)):'\"'===g||\"'\"===g?(P(s),O(g+C(g))):\";\"===g?(n=!1,i=!1,O(g),T(!0),\"/\"!==m.peek()&&L.add_new_line()):\"(\"===g?m.lookBack(\"url\")?(O(g),T(),(g=m.next())&&(\")\"!==g&&'\"'!==g&&\"'\"!==g?O(g+C(\")\")):(m.back(),w++))):(w++,P(s),O(g),T()):\")\"===g?(O(g),w--):\",\"===g?(O(g),T(!0),c&&!n&&w<1?L.add_new_line():L.space_before_token=!0):(\">\"===g||\"+\"===g||\"~\"===g)&&!n&&w<1?f?(L.space_before_token=!0,O(g),L.space_before_token=!0):(O(g),T(),g&&b.test(g)&&(g=\"\")):\"]\"===g?O(g):\"[\"===g?(P(s),O(g)):\"=\"===g?(T(),O(\"=\"),b.test(g)&&(g=\"\")):\"!\"===g?(O(\" \"),O(g)):(P(s),O(g))}return L.get_code(h,p)},this.NESTED_AT_RULE={\"@page\":!0,\"@font-face\":!0,\"@keyframes\":!0,\"@media\":!0,\"@supports\":!0,\"@document\":!0},this.CONDITIONAL_GROUP_RULE={\"@media\":!0,\"@supports\":!0,\"@document\":!0}}}]);var ur=function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){\"undefined\"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:\"Module\"}),Object.defineProperty(e,\"__esModule\",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&\"object\"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,\"default\",{enumerable:!0,value:e}),2&t&&\"string\"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,\"a\",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p=\"\",n(n.s=10)}([,,function(e,t){e.exports.mergeOpts=function(e,t){var n,r={};for(n in e)n!==t&&(r[n]=e[n]);if(t in e)for(n in e[t])r[n]=e[t][n];return r}},function(e,t){var n=\"ªµºÀ-ÖØ-öø-ˁˆ-ˑˠ-ˤˬˮͰ-ʹͶͷͺ-ͽΆΈ-ΊΌΎ-ΡΣ-ϵϷ-ҁҊ-ԧԱ-Ֆՙա-ևא-תװ-ײؠ-يٮٯٱ-ۓەۥۦۮۯۺ-ۼۿܐܒ-ܯݍ-ޥޱߊ-ߪߴߵߺࠀ-ࠕࠚࠤࠨࡀ-ࡘࢠࢢ-ࢬऄ-हऽॐक़-ॡॱ-ॷॹ-ॿঅ-ঌএঐও-নপ-রলশ-হঽৎড়ঢ়য়-ৡৰৱਅ-ਊਏਐਓ-ਨਪ-ਰਲਲ਼ਵਸ਼ਸਹਖ਼-ੜਫ਼ੲ-ੴઅ-ઍએ-ઑઓ-નપ-રલળવ-હઽૐૠૡଅ-ଌଏଐଓ-ନପ-ରଲଳଵ-ହଽଡ଼ଢ଼ୟ-ୡୱஃஅ-ஊஎ-ஐஒ-கஙசஜஞடணதந-பம-ஹௐఅ-ఌఎ-ఐఒ-నప-ళవ-హఽౘౙౠౡಅ-ಌಎ-ಐಒ-ನಪ-ಳವ-ಹಽೞೠೡೱೲഅ-ഌഎ-ഐഒ-ഺഽൎൠൡൺ-ൿඅ-ඖක-නඳ-රලව-ෆก-ะาำเ-ๆກຂຄງຈຊຍດ-ທນ-ຟມ-ຣລວສຫອ-ະາຳຽເ-ໄໆໜ-ໟༀཀ-ཇཉ-ཬྈ-ྌက-ဪဿၐ-ၕၚ-ၝၡၥၦၮ-ၰၵ-ႁႎႠ-ჅჇჍა-ჺჼ-ቈቊ-ቍቐ-ቖቘቚ-ቝበ-ኈኊ-ኍነ-ኰኲ-ኵኸ-ኾዀዂ-ዅወ-ዖዘ-ጐጒ-ጕጘ-ፚᎀ-ᎏᎠ-Ᏼᐁ-ᙬᙯ-ᙿᚁ-ᚚᚠ-ᛪᛮ-ᛰᜀ-ᜌᜎ-ᜑᜠ-ᜱᝀ-ᝑᝠ-ᝬᝮ-ᝰក-ឳៗៜᠠ-ᡷᢀ-ᢨᢪᢰ-ᣵᤀ-ᤜᥐ-ᥭᥰ-ᥴᦀ-ᦫᧁ-ᧇᨀ-ᨖᨠ-ᩔᪧᬅ-ᬳᭅ-ᭋᮃ-ᮠᮮᮯᮺ-ᯥᰀ-ᰣᱍ-ᱏᱚ-ᱽᳩ-ᳬᳮ-ᳱᳵᳶᴀ-ᶿḀ-ἕἘ-Ἕἠ-ὅὈ-Ὅὐ-ὗὙὛὝὟ-ώᾀ-ᾴᾶ-ᾼιῂ-ῄῆ-ῌῐ-ΐῖ-Ίῠ-Ῥῲ-ῴῶ-ῼⁱⁿₐ-ₜℂℇℊ-ℓℕℙ-ℝℤΩℨK-ℭℯ-ℹℼ-ℿⅅ-ⅉⅎⅠ-ↈⰀ-Ⱞⰰ-ⱞⱠ-ⳤⳫ-ⳮⳲⳳⴀ-ⴥⴧⴭⴰ-ⵧⵯⶀ-ⶖⶠ-ⶦⶨ-ⶮⶰ-ⶶⶸ-ⶾⷀ-ⷆⷈ-ⷎⷐ-ⷖⷘ-ⷞⸯ々-〇〡-〩〱-〵〸-〼ぁ-ゖゝ-ゟァ-ヺー-ヿㄅ-ㄭㄱ-ㆎㆠ-ㆺㇰ-ㇿ㐀-䶵一-鿌ꀀ-ꒌꓐ-ꓽꔀ-ꘌꘐ-ꘟꘪꘫꙀ-ꙮꙿ-ꚗꚠ-ꛯꜗ-ꜟꜢ-ꞈꞋ-ꞎꞐ-ꞓꞠ-Ɦꟸ-ꠁꠃ-ꠅꠇ-ꠊꠌ-ꠢꡀ-ꡳꢂ-ꢳꣲ-ꣷꣻꤊ-ꤥꤰ-ꥆꥠ-ꥼꦄ-ꦲꧏꨀ-ꨨꩀ-ꩂꩄ-ꩋꩠ-ꩶꩺꪀ-ꪯꪱꪵꪶꪹ-ꪽꫀꫂꫛ-ꫝꫠ-ꫪꫲ-ꫴꬁ-ꬆꬉ-ꬎꬑ-ꬖꬠ-ꬦꬨ-ꬮꯀ-ꯢ가-힣ힰ-ퟆퟋ-ퟻ豈-舘並-龎ﬀ-ﬆﬓ-ﬗיִײַ-ﬨשׁ-זּטּ-לּמּנּסּףּפּצּ-ﮱﯓ-ﴽﵐ-ﶏﶒ-ﷇﷰ-ﷻﹰ-ﹴﹶ-ﻼＡ-Ｚａ-ｚｦ-ﾾￂ-ￇￊ-ￏￒ-ￗￚ-ￜ\",r=\"̀-ͯ҃-֑҇-ׇֽֿׁׂׅׄؐ-ؚؠ-ىٲ-ۓۧ-ۨۻ-ۼܰ-݊ࠀ-ࠔࠛ-ࠣࠥ-ࠧࠩ-࠭ࡀ-ࡗࣤ-ࣾऀ-ःऺ-़ा-ॏ॑-ॗॢ-ॣ०-९ঁ-ঃ়া-ৄেৈৗয়-ৠਁ-ਃ਼ਾ-ੂੇੈੋ-੍ੑ੦-ੱੵઁ-ઃ઼ા-ૅે-ૉો-્ૢ-ૣ૦-૯ଁ-ଃ଼ା-ୄେୈୋ-୍ୖୗୟ-ୠ୦-୯ஂா-ூெ-ைொ-்ௗ௦-௯ఁ-ఃె-ైొ-్ౕౖౢ-ౣ౦-౯ಂಃ಼ಾ-ೄೆ-ೈೊ-್ೕೖೢ-ೣ೦-೯ംഃെ-ൈൗൢ-ൣ൦-൯ංඃ්ා-ුූෘ-ෟෲෳิ-ฺเ-ๅ๐-๙ິ-ູ່-ໍ໐-໙༘༙༠-༩༹༵༷ཁ-ཇཱ-྄྆-྇ྍ-ྗྙ-ྼ࿆က-ဩ၀-၉ၧ-ၭၱ-ၴႂ-ႍႏ-ႝ፝-፟ᜎ-ᜐᜠ-ᜰᝀ-ᝐᝲᝳក-ឲ៝០-៩᠋-᠍᠐-᠙ᤠ-ᤫᤰ-᤻ᥑ-ᥭᦰ-ᧀᧈ-ᧉ᧐-᧙ᨀ-ᨕᨠ-ᩓ᩠-᩿᩼-᪉᪐-᪙ᭆ-ᭋ᭐-᭙᭫-᭳᮰-᮹᯦-᯳ᰀ-ᰢ᱀-᱉ᱛ-ᱽ᳐-᳒ᴀ-ᶾḁ-ἕ‌‍‿⁀⁔⃐-⃥⃜⃡-⃰ⶁ-ⶖⷠ-ⷿ〡-〨゙゚Ꙁ-ꙭꙴ-꙽ꚟ꛰-꛱ꟸ-ꠀ꠆ꠋꠣ-ꠧꢀ-ꢁꢴ-꣄꣐-꣙ꣳ-ꣷ꤀-꤉ꤦ-꤭ꤰ-ꥅꦀ-ꦃ꦳-꧀ꨀ-ꨧꩀ-ꩁꩌ-ꩍ꩐-꩙ꩻꫠ-ꫩꫲ-ꫳꯀ-ꯡ꯬꯭꯰-꯹ﬠ-ﬨ︀-️︠-︦︳︴﹍-﹏０-９＿\",i=new RegExp(\"[$@A-Z_a-z\"+n+\"]\"),o=new RegExp(\"[$0-9A-Z_a-z\"+n+r+\"]\");t.identifier=new RegExp(\"[$@A-Z_a-z\"+n+\"][$0-9A-Z_a-z\"+n+r+\"]*\",\"g\"),t.newline=/[\\n\\r\\u2028\\u2029]/,t.lineBreak=new RegExp(\"\\r\\n|\"+t.newline.source),t.allLineBreaks=new RegExp(t.lineBreak.source,\"g\"),t.isIdentifierStart=function(e){return i.test(String.fromCharCode(e))},t.isIdentifierChar=function(e){return o.test(String.fromCharCode(e))}},,,function(e,t){e.exports.InputScanner=function(e){var t=e||\"\",n=t.length,r=0;this.back=function(){r>0&&(r-=1)},this.hasNext=function(){return r<n},this.next=function(){var e=null;return this.hasNext()&&(e=t.charAt(r),r+=1),e},this.peek=function(e){var i=null;return e=e||0,(e+=r)>=0&&e<n&&(i=t.charAt(e)),i},this.test=function(e,i){if(i=i||0,i+=r,e.lastIndex=i,i>=0&&i<n){var o=e.exec(t);return o&&o.index===i}return!1},this.testChar=function(e,t){var n=this.peek(t);return null!==n&&e.test(n)},this.match=function(e){e.lastIndex=r;var n=e.exec(t);return n&&n.index===r?r+=n[0].length:n=null,n},this.readWhile=function(e){var t=\"\",n=this.match(e);return n&&(t=n[0]),t},this.readUntil=function(e){var i,o=r;e.lastIndex=r;var a=e.exec(t);return o=a?a.index:n,i=t.substring(r,o),r=o,i},this.readUntilAfter=function(e){var i,o=r;e.lastIndex=r;var a=e.exec(t);return o=a?a.index+a[0].length:n,i=t.substring(r,o),r=o,i},this.peekUntilAfter=function(e){var t=r,n=this.readUntilAfter(e);return r=t,n},this.lookBack=function(e){var n=r-1;return n>=e.length&&t.substring(n-e.length,n).toLowerCase()===e}}},,,,function(e,t,n){var r=n(11).Beautifier;e.exports=function(e,t,n,i){return new r(e,t,n,i).beautify()}},function(e,t,n){var r=n(2).mergeOpts,i=n(3),o=n(6).InputScanner,a=i.lineBreak,s=i.allLineBreaks;function u(e){return e.replace(/\\s+$/g,\"\")}e.exports.Beautifier=function(e,t,n,i){var l,c,h,d,f,p,g,m,b,_,v,y,w,C,T,E,x,S,k,A,L,N,O;function P(){return this.token=\"\",this.current_mode=\"CONTENT\",this.tags={parent:null,tag:\"\",indent_level:0,token:null},this.last_token={text:\"\",type:\"\"},this.token_text=\"\",this.newlines=0,this.indent_content=c,this.indent_body_inner_html=h,this.indent_head_inner_html=d,this.Utils={whitespace:\"\\n\\r\\t \".split(\"\"),single_token:t.void_elements||[\"area\",\"base\",\"br\",\"col\",\"embed\",\"hr\",\"img\",\"input\",\"keygen\",\"link\",\"menuitem\",\"meta\",\"param\",\"source\",\"track\",\"wbr\",\"!doctype\",\"?xml\",\"?php\",\"?=\",\"basefont\",\"isindex\"],extra_liners:N,in_array:function(e,t){for(var n=0;n<t.length;n++)if(e===t[n])return!0;return!1}},this.is_whitespace=function(e){for(var t=0;t<e.length;t++)if(!this.Utils.in_array(e.charAt(t),this.Utils.whitespace))return!1;return!0},this.traverse_whitespace=function(){var e=\"\";if(this.Utils.in_array(this.input.peek(),this.Utils.whitespace)){this.newlines=0;do{e=this.input.next(),y&&\"\\n\"===e&&this.newlines<=w&&(this.newlines+=1)}while(this.Utils.in_array(this.input.peek(),this.Utils.whitespace));return!0}return!1},this.space_or_wrap=function(e){return this.line_char_count>=this.wrap_line_length?(this.print_newline(!1,e),this.print_indentation(e),!0):(this.line_char_count++,e.push(\" \"),!1)},this.get_content=function(){for(var e=\"\",t={parent:this.tags.token,text:\"\",type:\"TK_CONTENT\"},n=[];!(\"<\"===this.input.peek()||C&&this.input.test(/\\{\\{/g));){if(!this.input.hasNext()){n.length||(t.type=\"TK_EOF\");break}this.traverse_whitespace()?this.space_or_wrap(n):(e=this.input.next(),this.line_char_count++,n.push(e))}return t.text=n.join(\"\"),t},this.get_contents_to=function(e){if(!this.input.hasNext())return{text:\"\",type:\"TK_EOF\"};var t=new RegExp(\"</\"+e+\"\\\\s*>\",\"igm\");return{text:this.input.readUntil(t),type:\"TK_\"+e}},this.record_tag=function(e,t){var n={parent:this.tags,tag:e,indent_level:this.indent_level,token:t};this.tags=n},this.retrieve_tag=function(e){for(var t=null,n=this.tags;n&&n.tag!==e;)n=n.parent;return n&&(t=n.token,this.indent_level=n.indent_level,this.tags=n.parent),t},this.indent_to_tag=function(e){for(var t=this.tags;t&&t.tag!==e;)t=t.parent;t&&(this.indent_level=t.indent_level)},this.get_tag=function(){var e,t=\"\",n={parent:this.tags.token,text:\"\",type:\"\",tag_name:\"\",is_inline_tag:!1,is_unformatted:!1,is_content_unformatted:!1,is_opening_tag:!1,is_closing_tag:!1,multiline_content:!1,start_tag_token:null},r=[],i=\"\",o=!1,a=!0,s=!1,u=!1,l=\"\",c=!1,h=this.input.peek(),d=this.input.peek(1),f=this.input.peek(2);\"<\"!==h||\"!\"!==d&&\"?\"!==d&&\"%\"!==d?C&&\"{\"===h&&\"{\"===d&&\"!\"===f?(e=\"{\",l=(t=this.get_comment()).match(/^{{([^\\s}]+)/)[1],r=[t],u=!0):\"<\"===h?(r.push(this.input.next()),e=\"<\",l=this.input.readUntil(/[\\s>{]/g),r.push(l),l=l.toLowerCase(),o=!0):C&&\"{\"===h&&\"{\"===d&&(r.push(this.input.next()),r.push(this.input.next()),\"#\"===f&&r.push(this.input.next()),e=\"{\",l=this.input.readUntil(/[\\s}]/g),r.push(l),l=l.toLowerCase(),o=!1):(e=\"<\",l=(t=this.get_comment()).match(/^<([^\\s>]+)/)[1],r=[t],u=!0),n.is_closing_tag=\"/\"===l.charAt(0),n.tag_name=n.is_closing_tag?l.substr(1):l,n.is_inline_tag=this.Utils.in_array(n.tag_name,b)||\"{\"===e,n.is_unformatted=this.Utils.in_array(l,_),n.is_content_unformatted=this.Utils.in_array(l,v);var p,g=E;if((k||A)&&(g=r.join(\"\").length+1),this.line_char_count+=r.join(\"\").length,!u){for(;this.input.hasNext();){if(t=this.input.next(),C&&\"{\"===e&&r.length>2&&\"}\"===t&&\"}\"===r[r.length-1]){this.line_char_count++,r.push(t);break}if(\"'\"!==t&&'\"'!==t||(t+=this.get_unformatted(t),\"{\"!==e&&(o=!0)),n.is_unformatted)r.push(t),this.line_char_count++;else if(this.Utils.in_array(t,this.Utils.whitespace))o=!0;else{if(\"<\"===e&&(\"=\"===t&&(o=!1),!S||!s||c||\">\"!==t&&\"/\"!==t||this.input.test(/\\/?\\s*>/g,-1)&&(o=!1,c=!0,this.print_newline(!1,r),this.print_indentation(r))),o)if(\"{\"===e)this.line_char_count++,r.push(\" \"),o=!1;else if(r.length&&\"=\"!==r[r.length-1]&&\">\"!==t&&o){var m=this.space_or_wrap(r)&&\"/\"!==t&&!x;if(o=!1,x&&\"/\"!==t){var y=!1;S&&a&&(y=!this.input.test(/\\S*(=\"([^\"]|\\\\\")*\")?\\s*\\/?\\s*>/g,-1)),a&&!y||(this.print_newline(!1,r),this.print_indentation(r),m=!0)}if(m){s=!0;for(var w=0;w<g;w++)this.line_char_count++,r.push(\" \")}if(a)for(var T=0;T<r.length;T++)if(\" \"===r[T]){a=!1;break}}if(C&&\"<\"===e&&\"{\"===t&&\"{\"===this.input.peek()&&(t+=this.get_unformatted(/}}/g),r.length&&\" \"!==r[r.length-1]&&\"<\"!==r[r.length-1]&&(t=\" \"+t),o=!0),this.line_char_count++,r.push(t),\">\"===t)break}}if(!t)return r.length?n.text=r.join(\"\"):n.type=\"TK_EOF\",n}return\"script\"!==l&&\"style\"!==l||(p=r.join(\"\")),r.length>2&&\"/\"===r[r.length-2]||this.Utils.in_array(l,this.Utils.single_token)?(n.type=\"TK_TAG_SINGLE\",n.is_closing_tag=!0):C&&\"{\"===e&&\"else\"===l?(this.indent_to_tag(\"if\"),n.type=\"TK_TAG_HANDLEBARS_ELSE\",this.indent_content=!0,this.traverse_whitespace()):C&&\"{\"===e&&(r.length<2||/[^#\\^\\/]/.test(r[2].charAt(0)))?(n.type=\"TK_TAG_SINGLE\",n.is_closing_tag=!0):n.is_unformatted||n.is_content_unformatted?(i=this.get_unformatted(new RegExp(\"</\"+l+\">\",\"ig\")),r.push(i),n.type=\"TK_TAG_SINGLE\",n.is_closing_tag=!0):\"script\"===l&&(-1===p.search(\"type\")||p.search(\"type\")>-1&&p.search(/\\b(text|application|dojo)\\/(x-)?(javascript|ecmascript|jscript|livescript|(ld\\+)?json|method|aspect)/)>-1)?(this.record_tag(l),n.type=\"TK_TAG_SCRIPT\"):\"style\"===l&&(-1===p.search(\"type\")||p.search(\"type\")>-1&&p.search(\"text/css\")>-1)?(this.record_tag(l),n.type=\"TK_TAG_STYLE\"):\"!\"===l.charAt(0)?(n.type=\"TK_TAG_SINGLE\",this.traverse_whitespace()):(n.is_closing_tag?(n.start_tag_token=this.retrieve_tag(l.substring(1)),n.type=\"TK_TAG_END\"):(this.record_tag(l,n),\"html\"!==l&&(this.indent_content=!0),n.type=\"TK_TAG_START\",n.is_opening_tag=!0),this.traverse_whitespace()&&this.space_or_wrap(r),this.Utils.in_array(l,this.Utils.extra_liners)&&(this.print_newline(!1,this.output),this.output.length&&\"\\n\"!==this.output[this.output.length-2]&&this.print_newline(!0,this.output))),n.text=r.join(\"\"),n},this.get_comment=function(){for(var e=\"\",t=\">\",n=!1,r=this.input.next();r&&((e+=r).charAt(e.length-1)!==t.charAt(t.length-1)||-1===e.indexOf(t));)n||(n=e.length>10,0===e.indexOf(\"<![if\")?(t=\"<![endif]>\",n=!0):0===e.indexOf(\"<![cdata[\")?(t=\"]]>\",n=!0):0===e.indexOf(\"<![\")?(t=\"]>\",n=!0):0===e.indexOf(\"\\x3c!--\")?(t=\"--\\x3e\",n=!0):0===e.indexOf(\"{{!--\")?(t=\"--}}\",n=!0):0===e.indexOf(\"{{!\")?5===e.length&&-1===e.indexOf(\"{{!--\")&&(t=\"}}\",n=!0):0===e.indexOf(\"<?\")?(t=\"?>\",n=!0):0===e.indexOf(\"<%\")&&(t=\"%>\",n=!0)),r=this.input.next();return e},this.get_unformatted=function(e){var t,n=\"\",r=\"\";if('\"'===e||\"'\"===e)for(var i='\"'===e?/\"|{{/g:/'|{{/g;this.input.hasNext()&&(n+=r=this.input.readUntilAfter(i),!r[r.length-1].match(/['\"]/g));)this.input.hasNext()&&(n+=this.input.readUntilAfter(/}}/g));else n=this.input.readUntilAfter(e);return-1!==(t=n.lastIndexOf(\"\\n\"))?this.line_char_count=n.length-t:this.line_char_count+=n.length,n},this.get_token=function(){var e;if(\"TK_TAG_SCRIPT\"===this.last_token.type||\"TK_TAG_STYLE\"===this.last_token.type){var t=this.last_token.type.substr(7);e=this.get_contents_to(t)}else\"CONTENT\"===this.current_mode?\"\"===(e=this.get_content()).text&&(e=this.get_tag()):\"TAG\"===this.current_mode&&(e=this.get_tag());return e},this.get_full_indent=function(e){return(e=this.indent_level+e||0)<1?\"\":Array(e+1).join(this.indent_string)},this.printer=function(e,t,n,r,i){e=(e=e||\"\").replace(/\\r\\n|[\\r\\u2028\\u2029]/g,\"\\n\"),this.input=new o(e),this.output=[],this.indent_character=t,this.indent_string=\"\",this.indent_size=n,this.brace_style=i,this.indent_level=0,this.wrap_line_length=r,this.line_char_count=0;for(var a=0;a<this.indent_size;a++)this.indent_string+=this.indent_character;this.print_newline=function(e,t){t&&t.length&&(e||\"\\n\"!==t[t.length-1])&&(this.line_char_count=0,\"\\n\"!==t[t.length-1]&&(t[t.length-1]=u(t[t.length-1])),t.push(\"\\n\"))},this.print_indentation=function(e){for(var t=0;t<this.indent_level;t++)e.push(this.indent_string),this.line_char_count+=this.indent_string.length},this.print_token=function(e){this.is_whitespace(e)&&!this.output.length||((e||\"\"!==e)&&this.output.length&&\"\\n\"===this.output[this.output.length-1]&&(this.print_indentation(this.output),e=e.replace(/^\\s+/g,\"\")),this.print_token_raw(e))},this.print_token_raw=function(e){this.newlines>0&&(e=u(e)),e&&\"\"!==e&&(e.length>1&&\"\\n\"===e.charAt(e.length-1)?(this.output.push(e.slice(0,-1)),this.print_newline(!1,this.output)):this.output.push(e));for(var t=0;t<this.newlines;t++)this.print_newline(t>0,this.output);this.newlines=0},this.indent=function(){this.indent_level++},this.unindent=function(){this.indent_level>0&&this.indent_level--}},this}e=e||\"\",void 0!==(t=r(t=t||{},\"html\")).wrap_line_length&&0!==parseInt(t.wrap_line_length,10)||void 0===t.max_char||0===parseInt(t.max_char,10)||(t.wrap_line_length=t.max_char),c=void 0!==t.indent_inner_html&&t.indent_inner_html,h=void 0===t.indent_body_inner_html||t.indent_body_inner_html,d=void 0===t.indent_head_inner_html||t.indent_head_inner_html,f=void 0===t.indent_size?4:parseInt(t.indent_size,10),p=void 0===t.indent_char?\" \":t.indent_char,m=void 0===t.brace_style?\"collapse\":t.brace_style,g=0===parseInt(t.wrap_line_length,10)?32786:parseInt(t.wrap_line_length||250,10),b=t.inline||[\"a\",\"abbr\",\"area\",\"audio\",\"b\",\"bdi\",\"bdo\",\"br\",\"button\",\"canvas\",\"cite\",\"code\",\"data\",\"datalist\",\"del\",\"dfn\",\"em\",\"embed\",\"i\",\"iframe\",\"img\",\"input\",\"ins\",\"kbd\",\"keygen\",\"label\",\"map\",\"mark\",\"math\",\"meter\",\"noscript\",\"object\",\"output\",\"progress\",\"q\",\"ruby\",\"s\",\"samp\",\"select\",\"small\",\"span\",\"strong\",\"sub\",\"sup\",\"svg\",\"template\",\"textarea\",\"time\",\"u\",\"var\",\"video\",\"wbr\",\"text\",\"acronym\",\"address\",\"big\",\"dt\",\"ins\",\"strike\",\"tt\"],_=t.unformatted||[],v=t.content_unformatted||[\"pre\",\"textarea\"],y=void 0===t.preserve_newlines||t.preserve_newlines,w=y?isNaN(parseInt(t.max_preserve_newlines,10))?32786:parseInt(t.max_preserve_newlines,10):0,C=void 0!==t.indent_handlebars&&t.indent_handlebars,T=void 0===t.wrap_attributes?\"auto\":t.wrap_attributes,E=isNaN(parseInt(t.wrap_attributes_indent_size,10))?f:parseInt(t.wrap_attributes_indent_size,10),x=\"force\"===T.substr(0,\"force\".length),S=\"force-expand-multiline\"===T,k=\"force-aligned\"===T,A=\"aligned-multiple\"===T,L=void 0!==t.end_with_newline&&t.end_with_newline,N=\"object\"==typeof t.extra_liners&&t.extra_liners?t.extra_liners.concat():\"string\"==typeof t.extra_liners?t.extra_liners.split(\",\"):\"head,body,/html\".split(\",\"),O=t.eol?t.eol:\"auto\",t.indent_with_tabs&&(p=\"\\t\",f=1),\"auto\"===O&&(O=\"\\n\",e&&a.test(e||\"\")&&(O=e.match(a)[0])),O=O.replace(/\\\\r/,\"\\r\").replace(/\\\\n/,\"\\n\"),e=e.replace(s,\"\\n\"),this.beautify=function(){(l=new P).printer(e,p,f,g,m);for(var r=null,o={text:\"\",type:\"\",tag_name:\"\",is_opening_tag:!1,is_closing_tag:!1,is_inline_tag:!1};\"TK_EOF\"!==(r=l.get_token()).type;){switch(r.type){case\"TK_TAG_START\":r.is_inline_tag||\"TK_CONTENT\"===l.last_token.type||(r.parent&&(r.parent.multiline_content=!0),l.print_newline(!1,l.output)),l.print_token(r.text),l.indent_content&&(!l.indent_body_inner_html&&\"body\"===r.tag_name||!l.indent_head_inner_html&&\"head\"===r.tag_name||l.indent(),l.indent_content=!1),o=r,l.current_mode=\"CONTENT\";break;case\"TK_TAG_STYLE\":case\"TK_TAG_SCRIPT\":l.print_newline(!1,l.output),l.print_token(r.text),o=r,l.current_mode=\"CONTENT\";break;case\"TK_TAG_END\":(r.start_tag_token&&r.start_tag_token.multiline_content||!(r.is_inline_tag||o.is_inline_tag||l.last_token===o&&o.is_opening_tag&&r.is_closing_tag&&o.tag_name===r.tag_name||\"TK_CONTENT\"===l.last_token.type))&&l.print_newline(!1,l.output),l.print_token(r.text),o=r,l.current_mode=\"CONTENT\";break;case\"TK_TAG_SINGLE\":\"!--\"===r.tag_name&&l.last_token.is_closing_tag&&-1===r.text.indexOf(\"\\n\")||r.is_inline_tag||r.is_unformatted||l.print_newline(!1,l.output),l.print_token(r.text),o=r,l.current_mode=\"CONTENT\";break;case\"TK_TAG_HANDLEBARS_ELSE\":for(var a=!1,s=l.output.length-1;s>=0&&\"\\n\"!==l.output[s];s--)if(l.output[s].match(/{{#if/)){a=!0;break}a||l.print_newline(!1,l.output),l.print_token(r.text),l.indent_content&&(l.indent(),l.indent_content=!1),o=r,l.current_mode=\"CONTENT\";break;case\"TK_TAG_HANDLEBARS_COMMENT\":l.print_token(r.text),l.current_mode=\"TAG\";break;case\"TK_CONTENT\":if(l.print_token(r.text),l.current_mode=\"TAG\",!r.text)continue;break;case\"TK_STYLE\":case\"TK_SCRIPT\":if(\"\"!==r.text){l.print_newline(!1,l.output);var u,c=r.text,h=1;\"TK_SCRIPT\"===r.type?u=\"function\"==typeof n&&n:\"TK_STYLE\"===r.type&&(u=\"function\"==typeof i&&i),\"keep\"===t.indent_scripts?h=0:\"separate\"===t.indent_scripts&&(h=-l.indent_level);var d=l.get_full_indent(h);if(u){var b=function(){this.eol=\"\\n\"};b.prototype=t;var _=new b;c=u(c.replace(/^\\s*/,d),_)}else{var v=c.match(/^\\s*/)[0].match(/[^\\n\\r]*$/)[0].split(l.indent_string).length-1,y=l.get_full_indent(h-v);c=c.replace(/^\\s*/,d).replace(/\\r\\n|\\r|\\n/g,\"\\n\"+y).replace(/\\s+$/,\"\")}c&&(l.print_token_raw(c),l.print_newline(!0,l.output))}l.current_mode=\"TAG\";break;default:\"\"!==r.text&&l.print_token(r.text)}l.last_token=r}var w=l.output.join(\"\").replace(/[\\r\\n\\t ]+$/,\"\");return L&&(w+=\"\\n\"),\"\\n\"!==O&&(w=w.replace(/[\\n]/g,O)),w}}}]);function lr(e,t,n){var r=e.getText(),i=!0,o=0,a=n.tabSize||4;if(t){for(var s=e.offsetAt(t.start),u=s;u>0&&fr(r,u-1);)u--;0===u||dr(r,u-1)?s=u:u<s&&(s=u+1);for(var l=e.offsetAt(t.end),c=l;c<r.length&&fr(r,c);)c++;if((c===r.length||dr(r,c))&&(l=c),t=on.create(e.positionAt(s),e.positionAt(l)),i=l===r.length,r=r.substring(s,l),0!==s){var h=e.offsetAt(rn.create(t.start.line,0));o=function(e,t,n){var r=t,i=0,o=n.tabSize||4;for(;r<e.length;){var a=e.charAt(r);if(\" \"===a)i++;else{if(\"\\t\"!==a)break;i+=o}r++}return Math.floor(i/o)}(e.getText(),h,n)}}else t=on.create(rn.create(0,0),e.positionAt(r.length));var d=function(e,t){return ur(e,t,ar,sr)}(r,{indent_size:n.insertSpaces?a:1,indent_char:n.insertSpaces?\" \":\"\\t\",wrap_line_length:cr(n,\"wrapLineLength\",120),unformatted:hr(n,\"unformatted\",void 0),content_unformatted:hr(n,\"contentUnformatted\",void 0),indent_inner_html:cr(n,\"indentInnerHtml\",!1),preserve_newlines:cr(n,\"preserveNewLines\",!0),max_preserve_newlines:cr(n,\"maxPreserveNewLines\",32786),indent_handlebars:cr(n,\"indentHandlebars\",!1),end_with_newline:i&&cr(n,\"endWithNewline\",!1),extra_liners:hr(n,\"extraLiners\",void 0),wrap_attributes:cr(n,\"wrapAttributes\",\"auto\"),wrap_attributes_indent_size:cr(n,\"wrapAttributesIndentSize\",void 0),eol:\"\\n\"});if(o>0){var f=n.insertSpaces?Vt(\" \",a*o):Vt(\"\\t\",o);d=d.split(\"\\n\").join(\"\\n\"+f),0===t.start.character&&(d=f+d)}return[{range:t,newText:d}]}function cr(e,t,n){if(e&&e.hasOwnProperty(t)){var r=e[t];if(null!==r)return r}return n}function hr(e,t,n){var r=cr(e,t,null);return\"string\"==typeof r?r.length>0?r.split(\",\").map(function(e){return e.trim().toLowerCase()}):[]:n}function dr(e,t){return-1!==\"\\r\\n\".indexOf(e.charAt(t))}function fr(e,t){return-1!==\" \\t\".indexOf(e.charAt(t))}var pr,gr=function(){var e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();if(\"object\"==typeof process)pr=\"win32\"===process.platform;else if(\"object\"==typeof navigator){var mr=navigator.userAgent;pr=mr.indexOf(\"Windows\")>=0}var br=/^\\w[\\w\\d+.-]*$/,_r=/^\\//,vr=/^\\/\\//;var yr,wr=\"\",Cr=\"/\",Tr=/^(([^:\\/?#]+?):)?(\\/\\/([^\\/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?/,Er=function(){function e(e,t,n,r,i){\"object\"==typeof e?(this.scheme=e.scheme||wr,this.authority=e.authority||wr,this.path=e.path||wr,this.query=e.query||wr,this.fragment=e.fragment||wr):(this.scheme=e||wr,this.authority=t||wr,this.path=function(e,t){switch(e){case\"https\":case\"http\":case\"file\":t?t[0]!==Cr&&(t=Cr+t):t=Cr}return t}(this.scheme,n||wr),this.query=r||wr,this.fragment=i||wr,function(e){if(e.scheme&&!br.test(e.scheme))throw new Error(\"[UriError]: Scheme contains illegal characters.\");if(e.path)if(e.authority){if(!_r.test(e.path))throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash (\"/\") character')}else if(vr.test(e.path))throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters (\"//\")')}(this))}return e.isUri=function(t){return t instanceof e||!!t&&(\"string\"==typeof t.authority&&\"string\"==typeof t.fragment&&\"string\"==typeof t.path&&\"string\"==typeof t.query&&\"string\"==typeof t.scheme)},Object.defineProperty(e.prototype,\"fsPath\",{get:function(){return Lr(this)},enumerable:!0,configurable:!0}),e.prototype.with=function(e){if(!e)return this;var t=e.scheme,n=e.authority,r=e.path,i=e.query,o=e.fragment;return void 0===t?t=this.scheme:null===t&&(t=wr),void 0===n?n=this.authority:null===n&&(n=wr),void 0===r?r=this.path:null===r&&(r=wr),void 0===i?i=this.query:null===i&&(i=wr),void 0===o?o=this.fragment:null===o&&(o=wr),t===this.scheme&&n===this.authority&&r===this.path&&i===this.query&&o===this.fragment?this:new xr(t,n,r,i,o)},e.parse=function(e){var t=Tr.exec(e);return t?new xr(t[2]||wr,decodeURIComponent(t[4]||wr),decodeURIComponent(t[5]||wr),decodeURIComponent(t[7]||wr),decodeURIComponent(t[9]||wr)):new xr(wr,wr,wr,wr,wr)},e.file=function(e){var t=wr;if(pr&&(e=e.replace(/\\\\/g,Cr)),e[0]===Cr&&e[1]===Cr){var n=e.indexOf(Cr,2);-1===n?(t=e.substring(2),e=Cr):(t=e.substring(2,n),e=e.substring(n)||Cr)}return new xr(\"file\",t,e,wr,wr)},e.from=function(e){return new xr(e.scheme,e.authority,e.path,e.query,e.fragment)},e.prototype.toString=function(e){return void 0===e&&(e=!1),Nr(this,e)},e.prototype.toJSON=function(){return this},e.revive=function(t){if(t){if(t instanceof e)return t;var n=new xr(t);return n._fsPath=t.fsPath,n._formatted=t.external,n}return t},e}(),xr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._formatted=null,t._fsPath=null,t}return gr(t,e),Object.defineProperty(t.prototype,\"fsPath\",{get:function(){return this._fsPath||(this._fsPath=Lr(this)),this._fsPath},enumerable:!0,configurable:!0}),t.prototype.toString=function(e){return void 0===e&&(e=!1),e?Nr(this,!0):(this._formatted||(this._formatted=Nr(this,!1)),this._formatted)},t.prototype.toJSON=function(){var e={$mid:1};return this._fsPath&&(e.fsPath=this._fsPath),this._formatted&&(e.external=this._formatted),this.path&&(e.path=this.path),this.scheme&&(e.scheme=this.scheme),this.authority&&(e.authority=this.authority),this.query&&(e.query=this.query),this.fragment&&(e.fragment=this.fragment),e},t}(Er),Sr=((yr={})[58]=\"%3A\",yr[47]=\"%2F\",yr[63]=\"%3F\",yr[35]=\"%23\",yr[91]=\"%5B\",yr[93]=\"%5D\",yr[64]=\"%40\",yr[33]=\"%21\",yr[36]=\"%24\",yr[38]=\"%26\",yr[39]=\"%27\",yr[40]=\"%28\",yr[41]=\"%29\",yr[42]=\"%2A\",yr[43]=\"%2B\",yr[44]=\"%2C\",yr[59]=\"%3B\",yr[61]=\"%3D\",yr[32]=\"%20\",yr);function kr(e,t){for(var n=void 0,r=-1,i=0;i<e.length;i++){var o=e.charCodeAt(i);if(o>=97&&o<=122||o>=65&&o<=90||o>=48&&o<=57||45===o||46===o||95===o||126===o||t&&47===o)-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),void 0!==n&&(n+=e.charAt(i));else{void 0===n&&(n=e.substr(0,i));var a=Sr[o];void 0!==a?(-1!==r&&(n+=encodeURIComponent(e.substring(r,i)),r=-1),n+=a):-1===r&&(r=i)}}return-1!==r&&(n+=encodeURIComponent(e.substring(r))),void 0!==n?n:e}function Ar(e){for(var t=void 0,n=0;n<e.length;n++){var r=e.charCodeAt(n);35===r||63===r?(void 0===t&&(t=e.substr(0,n)),t+=Sr[r]):void 0!==t&&(t+=e[n])}return void 0!==t?t:e}function Lr(e){var t;return t=e.authority&&e.path.length>1&&\"file\"===e.scheme?\"//\"+e.authority+e.path:47===e.path.charCodeAt(0)&&(e.path.charCodeAt(1)>=65&&e.path.charCodeAt(1)<=90||e.path.charCodeAt(1)>=97&&e.path.charCodeAt(1)<=122)&&58===e.path.charCodeAt(2)?e.path[1].toLowerCase()+e.path.substr(2):e.path,pr&&(t=t.replace(/\\//g,\"\\\\\")),t}function Nr(e,t){var n=t?Ar:kr,r=\"\",i=e.scheme,o=e.authority,a=e.path,s=e.query,u=e.fragment;if(i&&(r+=i,r+=\":\"),(o||\"file\"===i)&&(r+=Cr,r+=Cr),o){var l=o.indexOf(\"@\");if(-1!==l){var c=o.substr(0,l);o=o.substr(l+1),-1===(l=c.indexOf(\":\"))?r+=n(c,!1):(r+=n(c.substr(0,l),!1),r+=\":\",r+=n(c.substr(l+1),!1)),r+=\"@\"}-1===(l=(o=o.toLowerCase()).indexOf(\":\"))?r+=n(o,!1):(r+=n(o.substr(0,l),!1),r+=o.substr(l))}if(a){if(a.length>=3&&47===a.charCodeAt(0)&&58===a.charCodeAt(2))(h=a.charCodeAt(1))>=65&&h<=90&&(a=\"/\"+String.fromCharCode(h+32)+\":\"+a.substr(3));else if(a.length>=2&&58===a.charCodeAt(1)){var h;(h=a.charCodeAt(0))>=65&&h<=90&&(a=String.fromCharCode(h+32)+\":\"+a.substr(2))}r+=n(a,!0)}return s&&(r+=\"?\",r+=n(s,!1)),u&&(r+=\"#\",r+=n(u,!1)),r}function Or(e,t){var n=e[0];return n!==e[e.length-1]||\"'\"!==n&&'\"'!==n||(e=e.substr(1,e.length-2)),\"razor\"===t&&\"~\"===e[0]&&(e=e.substr(1)),e}function Pr(e,t,n,r,i,o){var a=Or(n,e.languageId);if(!function(e,t){if(!e.length)return!1;if(\"handlebars\"===t&&/{{.*}}/.test(e))return!1;if(\"razor\"===t&&/@/.test(e))return!1;try{return!!Er.parse(e)}catch(e){return!1}}(a,e.languageId))return null;a.length<n.length&&(r++,i--);var s=function(e,t,n,r){return/^\\s*javascript\\:/i.test(t)||/^\\s*\\#/i.test(t)||/[\\n\\r]/.test(t)?null:(t=t.replace(/^\\s*/g,\"\"),/^https?:\\/\\//i.test(t)||/^file:\\/\\//i.test(t)?t:/^\\/\\//i.test(t)?(Wt(e,\"https://\")?\"https\":\"http\")+\":\"+t.replace(/^\\s*/g,\"\"):n?n.resolveReference(t,r||e):t)}(e.uri,a,t,o);return s&&function(e){try{return Er.parse(e),!0}catch(e){return!1}}(s)?{range:on.create(e.positionAt(r),e.positionAt(i)),target:s}:null}function Ir(e,t){for(var n=[],r=Ut(e.getText(),0),i=r.scan(),o=!1,a=!1,s=void 0;i!==gt.EOS;){switch(i){case gt.StartTag:if(!s)a=\"base\"===r.getTokenText().toLowerCase();break;case gt.AttributeName:var u=r.getTokenText().toLowerCase();o=\"src\"===u||\"href\"===u;break;case gt.AttributeValue:if(o){var l=r.getTokenText();if(!a){var c=Pr(e,t,l,r.getTokenOffset(),r.getTokenEnd(),s);c&&n.push(c)}a&&void 0===s&&(s=Or(l,e.languageId))&&t&&(s=t.resolveReference(s,e.uri)),a=!1,o=!1}}i=r.scan()}return n}function qr(e,t,n){var r=e.offsetAt(t),i=n.findNodeAt(r);if(!i.tag)return[];var o=[],a=Ur(gt.StartTag,e,i.start),s=\"number\"==typeof i.endTagStart&&Ur(gt.EndTag,e,i.endTagStart);return(a&&Dr(a,t)||s&&Dr(s,t))&&(a&&o.push({kind:In.Read,range:a}),s&&o.push({kind:In.Read,range:s})),o}function Rr(e,t){return e.line<t.line||e.line===t.line&&e.character<=t.character}function Dr(e,t){return Rr(e.start,t)&&Rr(t,e.end)}function Ur(e,t,n){for(var r=Ut(t.getText(),n),i=r.scan();i!==gt.EOS&&i!==e;)i=r.scan();return i!==gt.EOS?{start:t.positionAt(r.getTokenOffset()),end:t.positionAt(r.getTokenEnd())}:null}function Mr(e,t){var n=[];return t.roots.forEach(function(t){!function e(t,n,r,i){var o=function(e){var t=e.tag;if(e.attributes){var n=e.attributes.id,r=e.attributes.class;n&&(t+=\"#\"+n.replace(/[\\\"\\']/g,\"\")),r&&(t+=r.replace(/[\\\"\\']/g,\"\").split(/\\s+/).map(function(e){return\".\"+e}).join(\"\"))}return t||\"?\"}(n);var a=an.create(t.uri,on.create(t.positionAt(n.start),t.positionAt(n.end)));var s={name:o,location:a,containerName:r,kind:Rn.Field};i.push(s);n.children.forEach(function(n){e(t,n,o,i)})}(e,t,\"\",n)}),n}function Wr(e,t){var n=Ut(e.getText()),r=n.scan(),i=[],o=[],a=null,s=-1;function u(e){i.push(e),s=e.startLine}for(;r!==gt.EOS;){switch(r){case gt.StartTag:var l=n.getTokenText(),c=e.positionAt(n.getTokenOffset()).line;o.push({startLine:c,tagName:l}),a=l;break;case gt.EndTag:a=n.getTokenText();break;case gt.StartTagClose:if(!a||!$t(a))break;case gt.EndTagClose:case gt.StartTagSelfClose:for(var h=o.length-1;h>=0&&o[h].tagName!==a;)h--;if(h>=0){var d=o[h];o.length=h,(p=e.positionAt(n.getTokenOffset()).line-1)>(c=d.startLine)&&s!==c&&u({startLine:c,endLine:p})}break;case gt.Comment:c=e.positionAt(n.getTokenOffset()).line;var f=n.getTokenText().match(/^\\s*#(region\\b)|(endregion\\b)/);if(f)if(f[1])o.push({startLine:c,tagName:\"\"});else{for(h=o.length-1;h>=0&&o[h].tagName.length;)h--;if(h>=0){var p;d=o[h];o.length=h,(p=c)>(c=d.startLine)&&s!==c&&u({startLine:c,endLine:p,kind:cn.Region})}}else c<(p=e.positionAt(n.getTokenOffset()+n.getTokenLength()).line)&&u({startLine:c,endLine:p,kind:cn.Comment})}r=n.scan()}var g=t&&t.rangeLimit||Number.MAX_VALUE;return i.length>g&&(i=function(e,t){e=e.sort(function(e,t){var n=e.startLine-t.startLine;return 0===n&&(n=e.endLine-t.endLine),n});for(var n=void 0,r=[],i=[],o=[],a=function(e,t){i[e]=t,t<30&&(o[t]=(o[t]||0)+1)},s=0;s<e.length;s++){var u=e[s];if(n){if(u.startLine>n.startLine)if(u.endLine<=n.endLine)r.push(n),n=u,a(s,r.length);else if(u.startLine>n.endLine){do{n=r.pop()}while(n&&u.startLine>n.endLine);n&&r.push(n),n=u,a(s,r.length)}}else n=u,a(s,0)}var l=0,c=0;for(s=0;s<o.length;s++){var h=o[s];if(h){if(h+l>t){c=s;break}l+=h}}var d=[];for(s=0;s<e.length;s++){var f=i[s];\"number\"==typeof f&&(f<c||f===c&&l++<t)&&d.push(e[s])}return d}(i,g)),i}function Vr(){var e=new nr;return{createScanner:Ut,parseHTMLDocument:function(e){return function(e){for(var t=Ut(e),n=new vn(0,e.length,[],void 0),r=n,i=-1,o=null,a=t.scan();a!==gt.EOS;){switch(a){case gt.StartTagOpen:var s=new vn(t.getTokenOffset(),e.length,[],r);r.children.push(s),r=s;break;case gt.StartTag:r.tag=t.getTokenText();break;case gt.StartTagClose:r.end=t.getTokenEnd(),r.tag&&$t(r.tag)&&r.parent&&(r.closed=!0,r=r.parent);break;case gt.EndTagOpen:i=t.getTokenOffset();break;case gt.EndTag:for(var u=t.getTokenText().toLowerCase();!r.isSameTag(u)&&r.parent;)r.end=i,r.closed=!1,r=r.parent;r!==n&&(r.closed=!0,r.endTagStart=i);break;case gt.StartTagSelfClose:r.parent&&(r.closed=!0,r.end=t.getTokenEnd(),r=r.parent);break;case gt.EndTagClose:r.parent&&(r.end=t.getTokenEnd(),r=r.parent);break;case gt.AttributeName:o=t.getTokenText(),(l=r.attributes)||(r.attributes=l={}),l[o]=null;break;case gt.AttributeValue:var l,c=t.getTokenText();(l=r.attributes)&&o&&(l[o]=c,o=null)}a=t.scan()}for(;r.parent;)r.end=e.length,r.closed=!1,r=r.parent;return{roots:n.children,findNodeBefore:n.findNodeBefore.bind(n),findNodeAt:n.findNodeAt.bind(n)}}(e.getText())},doComplete:e.doComplete.bind(e),setCompletionParticipants:e.setCompletionParticipants.bind(e),doHover:or,format:lr,findDocumentHighlights:qr,findDocumentLinks:Ir,findDocumentSymbols:Mr,getFoldingRanges:Wr,doTagComplete:e.doTagComplete.bind(e)}}var Br=monaco.Promise,jr=function(){function e(e,t){this._ctx=e,this._languageSettings=t.languageSettings,this._languageId=t.languageId,this._languageService=Vr()}return e.prototype.doValidation=function(e){return Br.as([])},e.prototype.doComplete=function(e,t){var n=this._getTextDocument(e),r=this._languageService.parseHTMLDocument(n);return Br.as(this._languageService.doComplete(n,t,r,this._languageSettings&&this._languageSettings.suggest))},e.prototype.format=function(e,t,n){var r=this._getTextDocument(e),i=this._languageService.format(r,t,this._languageSettings&&this._languageSettings.format);return Br.as(i)},e.prototype.findDocumentHighlights=function(e,t){var n=this._getTextDocument(e),r=this._languageService.parseHTMLDocument(n),i=this._languageService.findDocumentHighlights(n,t,r);return Br.as(i)},e.prototype.findDocumentLinks=function(e){var t=this._getTextDocument(e),n=this._languageService.findDocumentLinks(t,null);return Br.as(n)},e.prototype.provideFoldingRanges=function(e,t){var n=this._getTextDocument(e),r=this._languageService.getFoldingRanges(n,t);return Br.as(r)},e.prototype._getTextDocument=function(e){for(var t=0,n=this._ctx.getMirrorModels();t<n.length;t++){var r=n[t];if(r.uri.toString()===e)return Kn.create(e,this._languageId,r.version,r.getValue())}return null},e}();self.onmessage=function(){_t(function(e,t){return new jr(e,t)})}}();\n"], { type: "text/javascript" }));

var typescriptWorker = URL.createObjectURL(new Blob(["!function(){\"use strict\";var e;!function(){var n=Object.create(null);n[\"WinJS/Core/_WinJS\"]={};var t=function(e,t,r){var a={},i=!1,o=t.map(function(e){return\"exports\"===e?(i=!0,a):n[e]}),s=r.apply({},o);n[e]=i?a:s};t(\"WinJS/Core/_Global\",[],function(){return\"undefined\"!=typeof window?window:\"undefined\"!=typeof self?self:\"undefined\"!=typeof global?global:{}}),t(\"WinJS/Core/_BaseCoreUtils\",[\"WinJS/Core/_Global\"],function(e){var n=!!e.Windows;var t=null;return{hasWinRT:n,markSupportedForProcessing:function(e){return e.supportedForProcessing=!0,e},_setImmediate:function(n){null===t&&(t=e.setImmediate?e.setImmediate.bind(e):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):e.setTimeout.bind(e)),t(n)}}}),t(\"WinJS/Core/_WriteProfilerMark\",[\"WinJS/Core/_Global\"],function(e){return e.msWriteProfilerMark||function(){}}),t(\"WinJS/Core/_Base\",[\"WinJS/Core/_WinJS\",\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_WriteProfilerMark\"],function(e,n,t,r){function a(e,n,t){var r,a,i,o=Object.keys(n),s=Array.isArray(e);for(a=0,i=o.length;a<i;a++){var l=o[a],c=95!==l.charCodeAt(0),u=n[l];!u||\"object\"!=typeof u||void 0===u.value&&\"function\"!=typeof u.get&&\"function\"!=typeof u.set?c?s?e.forEach(function(e){e[l]=u}):e[l]=u:(r=r||{})[l]={value:u,enumerable:c,configurable:!0,writable:!0}:(void 0===u.enumerable&&(u.enumerable=c),t&&u.setName&&\"function\"==typeof u.setName&&u.setName(t+\".\"+l),(r=r||{})[l]=u)}r&&(s?e.forEach(function(e){Object.defineProperties(e,r)}):Object.defineProperties(e,r))}return function(){var t=e;function i(t,r){var a=t||{};if(r){var i=r.split(\".\");a===n&&\"WinJS\"===i[0]&&(a=e,i.splice(0,1));for(var o=0,s=i.length;o<s;o++){var l=i[o];a[l]||Object.defineProperty(a,l,{value:{},writable:!1,enumerable:!0,configurable:!0}),a=a[l]}}return a}function o(e,n,t){var r=i(e,n);return t&&a(r,t,n||\"<ANONYMOUS>\"),r}t.Namespace||(t.Namespace=Object.create(Object.prototype));var s={uninitialized:1,working:2,initialized:3};Object.defineProperties(t.Namespace,{defineWithParent:{value:o,writable:!0,enumerable:!0,configurable:!0},define:{value:function(e,t){return o(n,e,t)},writable:!0,enumerable:!0,configurable:!0},_lazy:{value:function(e){var n,t,a=s.uninitialized;return{setName:function(e){n=e},get:function(){switch(a){case s.initialized:return t;case s.uninitialized:a=s.working;try{r(\"WinJS.Namespace._lazy:\"+n+\",StartTM\"),t=e()}finally{r(\"WinJS.Namespace._lazy:\"+n+\",StopTM\"),a=s.uninitialized}return e=null,a=s.initialized,t;case s.working:throw\"Illegal: reentrancy on initialization\";default:throw\"Illegal\"}},set:function(e){switch(a){case s.working:throw\"Illegal: reentrancy on initialization\";default:a=s.initialized,t=e}},enumerable:!0,configurable:!0}},writable:!0,enumerable:!0,configurable:!0},_moduleDefine:{value:function(e,t,r){var o=[e],s=null;return t&&(s=i(n,t),o.push(s)),a(o,r,t||\"<ANONYMOUS>\"),s},writable:!0,enumerable:!0,configurable:!0}})}(),function(){function n(e,n,r){return e=e||function(){},t.markSupportedForProcessing(e),n&&a(e.prototype,n),r&&a(e,r),e}e.Namespace.define(\"WinJS.Class\",{define:n,derive:function(e,r,i,o){if(e){r=r||function(){};var s=e.prototype;return r.prototype=Object.create(s),t.markSupportedForProcessing(r),Object.defineProperty(r.prototype,\"constructor\",{value:r,writable:!0,configurable:!0,enumerable:!0}),i&&a(r.prototype,i),o&&a(r,o),r}return n(r,i,o)},mix:function(e){var n,t;for(e=e||function(){},n=1,t=arguments.length;n<t;n++)a(e.prototype,arguments[n]);return e}})}(),{Namespace:e.Namespace,Class:e.Class}}),t(\"WinJS/Core/_ErrorFromName\",[\"WinJS/Core/_Base\"],function(e){var n=e.Class.derive(Error,function(e,n){this.name=e,this.message=n||e},{},{supportedForProcessing:!1});return e.Namespace.define(\"WinJS\",{ErrorFromName:n}),n}),t(\"WinJS/Core/_Events\",[\"exports\",\"WinJS/Core/_Base\"],function(e,n){function t(e){var n=\"_on\"+e+\"state\";return{get:function(){var e=this[n];return e&&e.userHandler},set:function(t){var r=this[n];t?(r||(r={wrapper:function(e){return r.userHandler(e)},userHandler:t},Object.defineProperty(this,n,{value:r,enumerable:!1,writable:!0,configurable:!0}),this.addEventListener(e,r.wrapper,!1)),r.userHandler=t):r&&(this.removeEventListener(e,r.wrapper,!1),this[n]=null)},enumerable:!0}}var r=n.Class.define(function(e,n,t){this.detail=n,this.target=t,this.timeStamp=Date.now(),this.type=e},{bubbles:{value:!1,writable:!1},cancelable:{value:!1,writable:!1},currentTarget:{get:function(){return this.target}},defaultPrevented:{get:function(){return this._preventDefaultCalled}},trusted:{value:!1,writable:!1},eventPhase:{value:0,writable:!1},target:null,timeStamp:null,type:null,preventDefault:function(){this._preventDefaultCalled=!0},stopImmediatePropagation:function(){this._stopImmediatePropagationCalled=!0},stopPropagation:function(){}},{supportedForProcessing:!1}),a={_listeners:null,addEventListener:function(e,n,t){t=t||!1,this._listeners=this._listeners||{};for(var r=this._listeners[e]=this._listeners[e]||[],a=0,i=r.length;a<i;a++){var o=r[a];if(o.useCapture===t&&o.listener===n)return}r.push({listener:n,useCapture:t})},dispatchEvent:function(e,n){var t=this._listeners&&this._listeners[e];if(t){for(var a=new r(e,n,this),i=0,o=(t=t.slice(0,t.length)).length;i<o&&!a._stopImmediatePropagationCalled;i++)t[i].listener(a);return a.defaultPrevented||!1}return!1},removeEventListener:function(e,n,t){t=t||!1;var r=this._listeners&&this._listeners[e];if(r)for(var a=0,i=r.length;a<i;a++){var o=r[a];if(o.listener===n&&o.useCapture===t){r.splice(a,1),0===r.length&&delete this._listeners[e];break}}}};n.Namespace._moduleDefine(e,\"WinJS.Utilities\",{_createEventProperty:t,createEventProperties:function(){for(var e={},n=0,r=arguments.length;n<r;n++){var a=arguments[n];e[\"on\"+a]=t(a)}return e},eventMixin:a})}),t(\"WinJS/Core/_Trace\",[\"WinJS/Core/_Global\"],function(e){function n(e){return e}return{_traceAsyncOperationStarting:e.Debug&&e.Debug.msTraceAsyncOperationStarting&&e.Debug.msTraceAsyncOperationStarting.bind(e.Debug)||n,_traceAsyncOperationCompleted:e.Debug&&e.Debug.msTraceAsyncOperationCompleted&&e.Debug.msTraceAsyncOperationCompleted.bind(e.Debug)||n,_traceAsyncCallbackStarting:e.Debug&&e.Debug.msTraceAsyncCallbackStarting&&e.Debug.msTraceAsyncCallbackStarting.bind(e.Debug)||n,_traceAsyncCallbackCompleted:e.Debug&&e.Debug.msTraceAsyncCallbackCompleted&&e.Debug.msTraceAsyncCallbackCompleted.bind(e.Debug)||n}}),t(\"WinJS/Promise/_StateMachine\",[\"WinJS/Core/_Global\",\"WinJS/Core/_BaseCoreUtils\",\"WinJS/Core/_Base\",\"WinJS/Core/_ErrorFromName\",\"WinJS/Core/_Events\",\"WinJS/Core/_Trace\"],function(e,n,t,r,a,i){e.Debug&&(e.Debug.setNonUserCodeExceptions=!0);var o=new(t.Class.mix(t.Class.define(null,{},{supportedForProcessing:!1}),a.eventMixin));o._listeners={};var s=\"error\",l=\"Canceled\",c=!1,u={promise:1,thenPromise:2,errorPromise:4,exceptionPromise:8,completePromise:16};u.all=u.promise|u.thenPromise|u.errorPromise|u.exceptionPromise|u.completePromise;var d,m,p,f,g,_,v,y,h,b,E=1;function T(){}d={name:\"created\",enter:function(e){e._setState(m)},cancel:T,done:T,then:T,_completed:T,_error:T,_notify:T,_progress:T,_setCompleteValue:T,_setErrorValue:T},m={name:\"working\",enter:T,cancel:function(e){e._setState(g)},done:M,then:K,_completed:L,_error:O,_notify:T,_progress:w,_setCompleteValue:V,_setErrorValue:G},p={name:\"waiting\",enter:function(e){var n=e._value;if(n instanceof H&&n._state!==b&&n._state!==y)P(n,{promise:e});else{var t=function(r){n._errorId?e._chainedError(r,n):(N(e,r,A,n,t),e._error(r))};t.handlesOnError=!0,n.then(e._completed.bind(e),t,e._progress.bind(e))}},cancel:function(e){e._setState(f)},done:M,then:K,_completed:L,_error:O,_notify:T,_progress:w,_setCompleteValue:V,_setErrorValue:G},f={name:\"waiting_canceled\",enter:function(e){e._setState(_);var n=e._value;n.cancel&&n.cancel()},cancel:T,done:M,then:K,_completed:L,_error:O,_notify:T,_progress:w,_setCompleteValue:V,_setErrorValue:G},g={name:\"canceled\",enter:function(e){e._setState(_),e._cancelAction()},cancel:T,done:M,then:K,_completed:L,_error:O,_notify:T,_progress:w,_setCompleteValue:V,_setErrorValue:G},_={name:\"canceling\",enter:function(e){var n=new Error(l);n.name=n.message,e._value=n,e._setState(h)},cancel:T,done:T,then:T,_completed:T,_error:T,_notify:T,_progress:T,_setCompleteValue:T,_setErrorValue:T},v={name:\"complete_notify\",enter:function(e){if(e.done=W.prototype.done,e.then=W.prototype.then,e._listeners)for(var n,t=[e];t.length;)(n=t.shift())._state._notify(n,t);e._setState(y)},cancel:T,done:null,then:null,_completed:T,_error:T,_notify:I,_progress:T,_setCompleteValue:T,_setErrorValue:T},y={name:\"success\",enter:function(e){e.done=W.prototype.done,e.then=W.prototype.then,e._cleanupAction()},cancel:T,done:null,then:null,_completed:T,_error:T,_notify:I,_progress:T,_setCompleteValue:T,_setErrorValue:T},h={name:\"error_notify\",enter:function(e){if(e.done=U.prototype.done,e.then=U.prototype.then,e._listeners)for(var n,t=[e];t.length;)(n=t.shift())._state._notify(n,t);e._setState(b)},cancel:T,done:null,then:null,_completed:T,_error:T,_notify:R,_progress:T,_setCompleteValue:T,_setErrorValue:T},b={name:\"error\",enter:function(e){e.done=U.prototype.done,e.then=U.prototype.then,e._cleanupAction()},cancel:T,done:null,then:null,_completed:T,_error:T,_notify:R,_progress:T,_setCompleteValue:T,_setErrorValue:T};var S=t.Class.define(null,{_listeners:null,_nextState:null,_state:null,_value:null,cancel:function(){this._state.cancel(this),this._run()},done:function(e,n,t){this._state.done(this,e,n,t)},then:function e(n,t,r){if(this.then===e)return this._state.then(this,n,t,r);this.then(n,t,r)},_chainedError:function(e,n){var t=this._state._error(this,e,C,n);return this._run(),t},_completed:function(e){var n=this._state._completed(this,e);return this._run(),n},_error:function(e){var n=this._state._error(this,e,D);return this._run(),n},_progress:function(e){this._state._progress(this,e)},_setState:function(e){this._nextState=e},_setCompleteValue:function(e){this._state._setCompleteValue(this,e),this._run()},_setChainedErrorValue:function(e,n){var t=this._state._setErrorValue(this,e,C,n);return this._run(),t},_setExceptionValue:function(e){var n=this._state._setErrorValue(this,e,k);return this._run(),n},_run:function(){for(;this._nextState;)this._state=this._nextState,this._nextState=null,this._state.enter(this)}},{supportedForProcessing:!1});function L(e,n){var t;t=n&&\"object\"==typeof n&&\"function\"==typeof n.then?p:v,e._value=n,e._setState(t)}function x(e,n,t,r,a,i){return{exception:e,error:n,promise:t,handler:i,id:r,parent:a}}function A(e,n,t,r){var a=t._isException;return x(a?n:null,a?null:n,e,t._errorId,t,r)}function C(e,n,t){var r=t._isException,a=t._errorId;return F(e,a,r),x(r?n:null,r?null:n,e,a,t)}function D(e,n){var t=++E;return F(e,t),x(null,n,e,t)}function k(e,n){var t=++E;return F(e,t,!0),x(n,null,e,t)}function M(e,n,t,r){P(e,{c:n,e:t,p:r,asyncOpID:i._traceAsyncOperationStarting(\"WinJS.Promise.done\")})}function O(e,n,t,r){e._value=n,N(e,n,t,r),e._setState(h)}function I(n,t){var r,a,o=n._value,s=n._listeners;if(s)for(n._listeners=null,r=0,a=Array.isArray(s)?s.length:1;r<a;r++){var l=1===a?s:s[r],c=l.c,u=l.promise;if(i._traceAsyncOperationCompleted(l.asyncOpID,e.Debug&&e.Debug.MS_ASYNC_OP_STATUS_SUCCESS),u){i._traceAsyncCallbackStarting(l.asyncOpID);try{u._setCompleteValue(c?c(o):o)}catch(e){u._setExceptionValue(e)}finally{i._traceAsyncCallbackCompleted()}u._state!==p&&u._listeners&&t.push(u)}else W.prototype.done.call(n,c)}}function R(n,t){var r,a,o=n._value,s=n._listeners;if(s)for(n._listeners=null,r=0,a=Array.isArray(s)?s.length:1;r<a;r++){var c=1===a?s:s[r],u=c.e,d=c.promise,m=e.Debug&&(o&&o.name===l?e.Debug.MS_ASYNC_OP_STATUS_CANCELED:e.Debug.MS_ASYNC_OP_STATUS_ERROR);if(i._traceAsyncOperationCompleted(c.asyncOpID,m),d){var f=!1;try{u?(i._traceAsyncCallbackStarting(c.asyncOpID),f=!0,u.handlesOnError||N(d,o,A,n,u),d._setCompleteValue(u(o))):d._setChainedErrorValue(o,n)}catch(e){d._setExceptionValue(e)}finally{f&&i._traceAsyncCallbackCompleted()}d._state!==p&&d._listeners&&t.push(d)}else U.prototype.done.call(n,null,u)}}function N(e,n,t,r,a){if(o._listeners[s]){if(n instanceof Error&&n.message===l)return;o.dispatchEvent(s,t(e,n,r,a))}}function w(e,n){var t,r,a=e._listeners;if(a)for(t=0,r=Array.isArray(a)?a.length:1;t<r;t++){var i=1===r?a:a[t],o=i.p;if(o)try{o(n)}catch(e){}i.c||i.e||!i.promise||i.promise._progress(n)}}function P(e,n){var t=e._listeners;t?(t=Array.isArray(t)?t:[t]).push(n):t=n,e._listeners=t}function F(e,n,t){e._isException=t||!1,e._errorId=n}function G(e,n,t,r){e._value=n,N(e,n,t,r),e._setState(b)}function V(e,n){var t;t=n&&\"object\"==typeof n&&\"function\"==typeof n.then?p:y,e._value=n,e._setState(t)}function K(e,n,t,r){var a=new H(e);return P(e,{promise:a,c:n,e:t,p:r,asyncOpID:i._traceAsyncOperationStarting(\"WinJS.Promise.then\")}),a}var B,H=t.Class.derive(S,function(e){c&&(!0===c||c&u.thenPromise)&&(this._stack=q._getStack()),this._creator=e,this._setState(d),this._run()},{_creator:null,_cancelAction:function(){this._creator&&this._creator.cancel()},_cleanupAction:function(){this._creator=null}},{supportedForProcessing:!1}),U=t.Class.define(function(e){c&&(!0===c||c&u.errorPromise)&&(this._stack=q._getStack()),this._value=e,N(this,e,D)},{cancel:function(){},done:function(e,n){var t=this._value;if(n)try{n.handlesOnError||N(null,t,A,this,n);var r=n(t);return void(r&&\"object\"==typeof r&&\"function\"==typeof r.done&&r.done())}catch(e){t=e}t instanceof Error&&t.message===l||q._doneHandler(t)},then:function(e,n){if(!n)return this;var t,r=this._value;try{n.handlesOnError||N(null,r,A,this,n),t=new W(n(r))}catch(e){t=e===r?this:new j(e)}return t}},{supportedForProcessing:!1}),j=t.Class.derive(U,function(e){c&&(!0===c||c&u.exceptionPromise)&&(this._stack=q._getStack()),this._value=e,N(this,e,k)},{},{supportedForProcessing:!1}),W=t.Class.define(function(e){if(c&&(!0===c||c&u.completePromise)&&(this._stack=q._getStack()),e&&\"object\"==typeof e&&\"function\"==typeof e.then){var n=new H(null);return n._setCompleteValue(e),n}this._value=e},{cancel:function(){},done:function(e){if(e)try{var n=e(this._value);n&&\"object\"==typeof n&&\"function\"==typeof n.done&&n.done()}catch(e){q._doneHandler(e)}},then:function(e){try{var n=e?e(this._value):this._value;return n===this._value?this:new W(n)}catch(e){return new j(e)}}},{supportedForProcessing:!1});var q=t.Class.derive(S,function(e,n){c&&(!0===c||c&u.promise)&&(this._stack=q._getStack()),this._oncancel=n,this._setState(d),this._run();try{e(this._completed.bind(this),this._error.bind(this),this._progress.bind(this))}catch(e){this._setExceptionValue(e)}},{_oncancel:null,_cancelAction:function(){try{if(!this._oncancel)throw new Error(\"Promise did not implement oncancel\");this._oncancel()}catch(e){e.message,e.stack;o.dispatchEvent(\"error\",e)}},_cleanupAction:function(){this._oncancel=null}},{addEventListener:function(e,n,t){o.addEventListener(e,n,t)},any:function(e){return new q(function(n,t){var r=Object.keys(e);0===r.length&&n();var a=0;r.forEach(function(i){q.as(e[i]).then(function(){n({key:i,value:e[i]})},function(o){o instanceof Error&&o.name===l?++a===r.length&&n(q.cancel):t({key:i,value:e[i]})})})},function(){Object.keys(e).forEach(function(n){var t=q.as(e[n]);\"function\"==typeof t.cancel&&t.cancel()})})},as:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then?e:new W(e)},cancel:{get:function(){return B=B||new U(new r(l))}},dispatchEvent:function(e,n){return o.dispatchEvent(e,n)},is:function(e){return e&&\"object\"==typeof e&&\"function\"==typeof e.then},join:function(e){return new q(function(n,t,r){var a=Object.keys(e),i=Array.isArray(e)?[]:{},o=Array.isArray(e)?[]:{},s=0,c=a.length,u=function(e){if(0==--c){var s=Object.keys(i).length;if(0===s)n(o);else{var u=0;a.forEach(function(e){var n=i[e];n instanceof Error&&n.name===l&&u++}),u===s?n(q.cancel):t(i)}}else r({Key:e,Done:!0})};a.forEach(function(n){var t=e[n];void 0===t?s++:q.then(t,function(e){o[n]=e,u(n)},function(e){i[n]=e,u(n)})}),0!==(c-=s)||n(o)},function(){Object.keys(e).forEach(function(n){var t=q.as(e[n]);\"function\"==typeof t.cancel&&t.cancel()})})},removeEventListener:function(e,n,t){o.removeEventListener(e,n,t)},supportedForProcessing:!1,then:function(e,n,t,r){return q.as(e).then(n,t,r)},thenEach:function(e,n,t,r){var a=Array.isArray(e)?[]:{};return Object.keys(e).forEach(function(i){a[i]=q.as(e[i]).then(n,t,r)}),q.join(a)},timeout:function(t,r){var a,i,o=(a=t,new q(function(t){a?i=e.setTimeout(t,a):n._setImmediate(t)},function(){i&&e.clearTimeout(i)}));return r?function(e,n){var t=function(){e.cancel()};return e.then(function(){n.cancel()}),n.then(t,t),n}(o,r):o},wrap:function(e){return new W(e)},wrapError:function(e){return new U(e)},_veryExpensiveTagWithStack:{get:function(){return c},set:function(e){c=e}},_veryExpensiveTagWithStack_tag:u,_getStack:function(){if(e.Debug&&e.Debug.debuggerEnabled)try{throw new Error}catch(e){return e.stack}},_cancelBlocker:function(e,n){if(!q.is(e))return q.wrap(e);var t,r,a=new q(function(e,n){t=e,r=n},function(){t=null,r=null,n&&n()});return e.then(function(e){t&&t(e)},function(e){r&&r(e)}),a}});return Object.defineProperties(q,a.createEventProperties(s)),q._doneHandler=function(e){n._setImmediate(function(){throw e})},{PromiseStateMachine:S,Promise:q,state_created:d}}),t(\"WinJS/Promise\",[\"WinJS/Core/_Base\",\"WinJS/Promise/_StateMachine\"],function(e,n){return e.Namespace.define(\"WinJS\",{Promise:n.Promise}),n.Promise}),(e=n[\"WinJS/Core/_WinJS\"]).TPromise=e.Promise,e.PPromise=e.Promise}();var n=e.Promise,t=e.TPromise,r=(e.PPromise,{});t.addEventListener(\"error\",function(e){var n=e.detail,t=n.id;n.parent?n.handler&&r&&delete r[t]:(r[t]=n,1===Object.keys(r).length&&setTimeout(function(){var e=r;r={},Object.keys(e).forEach(function(n){var t=e[n];t.exception?i(t.exception):t.error&&i(t.error),console.log(\"WARNING: Promise with no error callback:\"+t.id),console.log(t),t.exception&&console.log(t.exception.stack)})},0))});var a=new(function(){function e(){this.listeners=[],this.unexpectedErrorHandler=function(e){setTimeout(function(){if(e.stack)throw new Error(e.message+\"\\n\\n\"+e.stack);throw e},0)}}return e.prototype.emit=function(e){this.listeners.forEach(function(n){n(e)})},e.prototype.onUnexpectedError=function(e){this.unexpectedErrorHandler(e),this.emit(e)},e.prototype.onUnexpectedExternalError=function(e){this.unexpectedErrorHandler(e)},e}());function i(e){var n;(n=e)instanceof Error&&n.name===s&&n.message===s||a.onUnexpectedError(e)}function o(e){return e instanceof Error?{$isError:!0,name:e.name,message:e.message,stack:e.stacktrace||e.stack}:e}var s=\"Canceled\";var l=function(){function e(){this._toDispose=[]}return e.prototype.dispose=function(){this._toDispose=function e(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];return Array.isArray(n)?(n.forEach(function(e){return e&&e.dispose()}),[]):0===t.length?n?(n.dispose(),n):void 0:(e(n),e(t),[])}(this._toDispose)},e.prototype._register=function(e){return this._toDispose.push(e),e},e.None=Object.freeze({dispose:function(){}}),e}(),c=!1,u=\"undefined\"!=typeof process&&void 0!==process.versions&&void 0!==process.versions.electron&&\"renderer\"===process.type;if(\"object\"!=typeof navigator||u){if(\"object\"==typeof process){c=\"win32\"===process.platform,\"darwin\"===process.platform,\"linux\"===process.platform,\"en\";var d=process.env.VSCODE_NLS_CONFIG;if(d)try{var m=JSON.parse(d);m.availableLanguages[\"*\"];m.locale,m._translationsConfigFile}catch(e){}}}else{var p=navigator.userAgent;c=p.indexOf(\"Windows\")>=0,p.indexOf(\"Macintosh\")>=0,p.indexOf(\"Linux\")>=0,navigator.language}var f=c,g=\"object\"==typeof self?self:\"object\"==typeof global?global:{},_=null;function v(e){return null===_&&(_=g.setImmediate?g.setImmediate.bind(g):\"undefined\"!=typeof process&&\"function\"==typeof process.nextTick?process.nextTick.bind(process):g.setTimeout.bind(g)),_(e)}var y,h,b=(y=function(e,n){return(y=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)n.hasOwnProperty(t)&&(e[t]=n[t])})(e,n)},function(e,n){function t(){this.constructor=e}y(e,n),e.prototype=null===n?Object.create(n):(t.prototype=n.prototype,new t)}),E={done:!0,value:void 0};!function(e){var n={next:function(){return E}};function t(e,n){for(var t=e.next();!t.done;t=e.next())n(t.value)}e.empty=function(){return n},e.fromArray=function(e,n,t){return void 0===n&&(n=0),void 0===t&&(t=e.length),{next:function(){return n>=t?E:{done:!1,value:e[n++]}}}},e.from=function(n){return n?Array.isArray(n)?e.fromArray(n):n:e.empty()},e.map=function(e,n){return{next:function(){var t=e.next();return t.done?E:{done:!1,value:n(t.value)}}}},e.filter=function(e,n){return{next:function(){for(;;){var t=e.next();if(t.done)return E;if(n(t.value))return{done:!1,value:t.value}}}}},e.forEach=t,e.collect=function(e){var n=[];return t(e,function(e){return n.push(e)}),n}}(h||(h={}));!function(e){function n(n,t,r,a){return void 0===t&&(t=0),void 0===r&&(r=n.length),void 0===a&&(a=t-1),e.call(this,n,t,r,a)||this}b(n,e),n.prototype.current=function(){return e.prototype.current.call(this)},n.prototype.previous=function(){return this.index=Math.max(this.index-1,this.start-1),this.current()},n.prototype.first=function(){return this.index=this.start,this.current()},n.prototype.last=function(){return this.index=this.end-1,this.current()},n.prototype.parent=function(){return null}}(function(){function e(e,n,t,r){void 0===n&&(n=0),void 0===t&&(t=e.length),void 0===r&&(r=n-1),this.items=e,this.start=n,this.end=t,this.index=r}return e.prototype.next=function(){return this.index=Math.min(this.index+1,this.end),this.current()},e.prototype.current=function(){return this.index===this.start-1||this.index===this.end?null:this.items[this.index]},e}());var T,S=function(){return function(e){this.element=e}}(),L=function(){function e(){}return e.prototype.isEmpty=function(){return!this._first},e.prototype.unshift=function(e){return this.insert(e,!1)},e.prototype.push=function(e){return this.insert(e,!0)},e.prototype.insert=function(e,n){var t=this,r=new S(e);if(this._first)if(n){var a=this._last;this._last=r,r.prev=a,a.next=r}else{var i=this._first;this._first=r,r.next=i,i.prev=r}else this._first=r,this._last=r;return function(){for(var e=t._first;e instanceof S;){if(e===r){if(e.prev&&e.next){var n=e.prev;n.next=e.next,e.next.prev=n}else e.prev||e.next?e.next?e.prev||(t._first=t._first.next,t._first.prev=void 0):(t._last=t._last.prev,t._last.next=void 0):(t._first=void 0,t._last=void 0);break}e=e.next}}},e.prototype.iterator=function(){var e,n=this._first;return{next:function(){return n?(e?e.value=n.element:e={done:!1,value:n.element},n=n.next,e):E}}},e}();!function(e){var n={dispose:function(){}};e.None=function(){return n}}(T||(T={}));var x=function(){function e(e){void 0===e&&(e=null),this._options=e,this._event=null,this._disposed=!1,this._deliveryQueue=null,this._listeners=null}return Object.defineProperty(e.prototype,\"event\",{get:function(){var n=this;return this._event||(this._event=function(t,r,a){n._listeners||(n._listeners=new L);var i=n._listeners.isEmpty();i&&n._options&&n._options.onFirstListenerAdd&&n._options.onFirstListenerAdd(n);var o,s=n._listeners.push(r?[t,r]:t);return i&&n._options&&n._options.onFirstListenerDidAdd&&n._options.onFirstListenerDidAdd(n),n._options&&n._options.onListenerDidAdd&&n._options.onListenerDidAdd(n,t,r),o={dispose:function(){(o.dispose=e._noop,n._disposed)||(s(),n._options&&n._options.onLastListenerRemove&&(n._listeners&&!n._listeners.isEmpty()||n._options.onLastListenerRemove(n)))}},Array.isArray(a)&&a.push(o),o}),this._event},enumerable:!0,configurable:!0}),e.prototype.fire=function(e){if(this._listeners){this._deliveryQueue||(this._deliveryQueue=[]);for(var n=this._listeners.iterator(),t=n.next();!t.done;t=n.next())this._deliveryQueue.push([t.value,e]);for(;this._deliveryQueue.length>0;){var r=this._deliveryQueue.shift(),a=r[0],o=r[1];try{\"function\"==typeof a?a.call(void 0,o):a[0].call(a[1],o)}catch(t){i(t)}}}},e.prototype.dispose=function(){this._listeners&&(this._listeners=null),this._deliveryQueue&&(this._deliveryQueue.length=0),this._disposed=!0},e._noop=function(){},e}();!function(){function e(){var e=this;this.hasListeners=!1,this.events=[],this.emitter=new x({onFirstListenerAdd:function(){return e.onFirstListenerAdd()},onLastListenerRemove:function(){return e.onLastListenerRemove()}})}Object.defineProperty(e.prototype,\"event\",{get:function(){return this.emitter.event},enumerable:!0,configurable:!0}),e.prototype.add=function(e){var n=this,t={event:e,listener:null};this.events.push(t),this.hasListeners&&this.hook(t);var r;return r=function(e){var n,t=this,r=!1;return function(){return r?n:(r=!0,n=e.apply(t,arguments))}}(function(){n.hasListeners&&n.unhook(t);var e=n.events.indexOf(t);n.events.splice(e,1)}),{dispose:function(){r()}}},e.prototype.onFirstListenerAdd=function(){var e=this;this.hasListeners=!0,this.events.forEach(function(n){return e.hook(n)})},e.prototype.onLastListenerRemove=function(){var e=this;this.hasListeners=!1,this.events.forEach(function(n){return e.unhook(n)})},e.prototype.hook=function(e){var n=this;e.listener=e.event(function(e){return n.emitter.fire(e)})},e.prototype.unhook=function(e){e.listener&&e.listener.dispose(),e.listener=null},e.prototype.dispose=function(){this.emitter.dispose()}}();(function(){function e(e){this._event=e}Object.defineProperty(e.prototype,\"event\",{get:function(){return this._event},enumerable:!0,configurable:!0}),e.prototype.map=function(n){return new e((t=this._event,r=n,function(e,n,a){return void 0===n&&(n=null),t(function(t){return e.call(n,r(t))},null,a)}));var t,r},e.prototype.filter=function(n){return new e((t=this._event,r=n,function(e,n,a){return void 0===n&&(n=null),t(function(t){return r(t)&&e.call(n,t)},null,a)}));var t,r},e.prototype.on=function(e,n,t){return this._event(e,n,t)}})(),function(){function e(){var e=this;this.listening=!1,this.inputEvent=T.None,this.inputEventListener=l.None,this.emitter=new x({onFirstListenerDidAdd:function(){e.listening=!0,e.inputEventListener=e.inputEvent(e.emitter.fire,e.emitter)},onLastListenerRemove:function(){e.listening=!1,e.inputEventListener.dispose()}}),this.event=this.emitter.event}Object.defineProperty(e.prototype,\"input\",{set:function(e){this.inputEvent=e,this.listening&&(this.inputEventListener.dispose(),this.inputEventListener=e(this.emitter.fire,this.emitter))},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.inputEventListener.dispose(),this.emitter.dispose()}}();var A,C=Object.freeze(function(e,n){var t=setTimeout(e.bind(n),0);return{dispose:function(){clearTimeout(t)}}});!function(e){e.isCancellationToken=function(n){return n===e.None||n===e.Cancelled||n instanceof D||!(!n||\"object\"!=typeof n)&&\"boolean\"==typeof n.isCancellationRequested&&\"function\"==typeof n.onCancellationRequested},e.None=Object.freeze({isCancellationRequested:!1,onCancellationRequested:T.None}),e.Cancelled=Object.freeze({isCancellationRequested:!0,onCancellationRequested:C})}(A||(A={}));var D=function(){function e(){this._isCancelled=!1,this._emitter=null}return e.prototype.cancel=function(){this._isCancelled||(this._isCancelled=!0,this._emitter&&(this._emitter.fire(void 0),this.dispose()))},Object.defineProperty(e.prototype,\"isCancellationRequested\",{get:function(){return this._isCancelled},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,\"onCancellationRequested\",{get:function(){return this._isCancelled?C:(this._emitter||(this._emitter=new x),this._emitter.event)},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this._emitter&&(this._emitter.dispose(),this._emitter=null)},e}(),k=function(){function e(){}return Object.defineProperty(e.prototype,\"token\",{get:function(){return this._token||(this._token=new D),this._token},enumerable:!0,configurable:!0}),e.prototype.cancel=function(){this._token?this._token instanceof D&&this._token.cancel():this._token=A.Cancelled},e.prototype.dispose=function(){this._token?this._token instanceof D&&this._token.dispose():this._token=A.None},e}(),M=function(){var e=function(n,t){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)n.hasOwnProperty(t)&&(e[t]=n[t])})(n,t)};return function(n,t){function r(){this.constructor=n}e(n,t),n.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}}();(function(e){function n(n,t){var r=e.call(this)||this;return r._token=-1,\"function\"==typeof n&&\"number\"==typeof t&&r.setIfNotSet(n,t),r}M(n,e),n.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},n.prototype.cancel=function(){-1!==this._token&&(clearTimeout(this._token),this._token=-1)},n.prototype.cancelAndSet=function(e,n){var t=this;this.cancel(),this._token=setTimeout(function(){t._token=-1,e()},n)},n.prototype.setIfNotSet=function(e,n){var t=this;-1===this._token&&(this._token=setTimeout(function(){t._token=-1,e()},n))}})(l),function(e){function n(){var n=e.call(this)||this;return n._token=-1,n}M(n,e),n.prototype.dispose=function(){this.cancel(),e.prototype.dispose.call(this)},n.prototype.cancel=function(){-1!==this._token&&(clearInterval(this._token),this._token=-1)},n.prototype.cancelAndSet=function(e,n){this.cancel(),this._token=setInterval(function(){e()},n)}}(l);!function(){if(\"function\"!=typeof requestIdleCallback||\"function\"!=typeof cancelIdleCallback)Object.freeze({didTimeout:!0,timeRemaining:function(){return 15}})}();var O=function(){function e(e){var t,r;(r=t=e)&&\"function\"==typeof r.then&&\"function\"==typeof t.done?this._winjsPromise=e:this._winjsPromise=new n(function(n,t){var r=!0;e(function(e){r?v(function(){return n(e)}):n(e)},function(e){r?v(function(){return t(e)}):t(e)}),r=!1})}return e.all=function(t){return new e(n.join(t).then(null,function(e){for(var n in e)if(e.hasOwnProperty(n))return e[n]}))},e.race=function(t){return new e(n.any(t).then(function(e){return e.value},function(e){return e.value}))},e.resolve=function(t){return new e(n.wrap(t))},e.reject=function(t){return new e(n.wrapError(t))},e.prototype.then=function(t,r){var a=!0,i=new e(this._winjsPromise.then(t&&function(e){return a?new n(function(n,r){v(function(){var a;try{a=t(e)}catch(e){return void r(e)}n(a)})}):t(e)},r&&function(e){return a?new n(function(n,t){v(function(){var a;try{a=r(e)}catch(e){return void t(e)}n(a)})}):r(e)}));return a=!1,i},e.prototype.catch=function(e){return this.then(null,e)},e}(),I=function(){var e=function(n,t){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)n.hasOwnProperty(t)&&(e[t]=n[t])})(n,t)};return function(n,t){function r(){this.constructor=n}e(n,t),n.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}}(),R=self;void 0===R.Promise&&(R.Promise=O);var N=\"$initialize\",w=function(){function e(e){this._workerId=-1,this._handler=e,this._lastSentReq=0,this._pendingReplies=Object.create(null)}return e.prototype.setWorkerId=function(e){this._workerId=e},e.prototype.sendMessage=function(e,n){var t=this,r=String(++this._lastSentReq);return new Promise(function(a,i){t._pendingReplies[r]={resolve:a,reject:i},t._send({vsWorker:t._workerId,req:r,method:e,args:n})})},e.prototype.handleMessage=function(e){var n;try{n=JSON.parse(e)}catch(e){return}n&&n.vsWorker&&(-1!==this._workerId&&n.vsWorker!==this._workerId||this._handleMessage(n))},e.prototype._handleMessage=function(e){var n=this;if(e.seq){var t=e;if(!this._pendingReplies[t.seq])return void console.warn(\"Got reply to unknown seq\");var r=this._pendingReplies[t.seq];if(delete this._pendingReplies[t.seq],t.err){var a=t.err;return t.err.$isError&&((a=new Error).name=t.err.name,a.message=t.err.message,a.stack=t.err.stack),void r.reject(a)}r.resolve(t.res)}else{var i=e,s=i.req;this._handler.handleMessage(i.method,i.args).then(function(e){n._send({vsWorker:n._workerId,seq:s,res:e,err:void 0})},function(e){e.detail instanceof Error&&(e.detail=o(e.detail)),n._send({vsWorker:n._workerId,seq:s,res:void 0,err:o(e)})})}},e.prototype._send=function(e){var n=JSON.stringify(e);this._handler.sendMessage(n)},e}(),P=(function(e){function n(n,t){var r=e.call(this)||this,a=null;r._worker=r._register(n.create(\"vs/base/common/worker/simpleWorker\",function(e){r._protocol.handleMessage(e)},function(e){a&&a(e)})),r._protocol=new w({sendMessage:function(e){r._worker.postMessage(e)},handleMessage:function(e,n){return Promise.resolve(null)}}),r._protocol.setWorkerId(r._worker.getId());var i=null;void 0!==self.require&&\"function\"==typeof self.require.getConfig?i=self.require.getConfig():void 0!==self.requirejs&&(i=self.requirejs.s.contexts._.config),r._onModuleLoaded=r._protocol.sendMessage(N,[r._worker.getId(),t,i]),r._lazyProxy=new Promise(function(e,n){a=n,r._onModuleLoaded.then(function(n){for(var t={},r=0;r<n.length;r++)t[n[r]]=s(n[r],o);e(t)},function(e){n(e),r._onError(\"Worker failed to load \"+t,e)})});var o=function(e,n){return r._request(e,n)},s=function(e,n){return function(){var t=Array.prototype.slice.call(arguments,0);return n(e,t)}};return r}I(n,e),n.prototype.getProxyObject=function(){return this._lazyProxy},n.prototype._request=function(e,n){var t=this;return new Promise(function(r,a){t._onModuleLoaded.then(function(){t._protocol.sendMessage(e,n).then(r,a)},a)})},n.prototype._onError=function(e,n){console.error(e),console.info(n)}}(l),function(){function e(e,n){var t=this;this._requestHandler=n,this._protocol=new w({sendMessage:function(n){e(n)},handleMessage:function(e,n){return t._handleMessage(e,n)}})}return e.prototype.onmessage=function(e){this._protocol.handleMessage(e)},e.prototype._handleMessage=function(e,n){if(e===N)return this.initialize(n[0],n[1],n[2]);if(!this._requestHandler||\"function\"!=typeof this._requestHandler[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._requestHandler[e].apply(this._requestHandler,n))}catch(e){return Promise.reject(e)}},e.prototype.initialize=function(e,n,t){var r=this;if(this._protocol.setWorkerId(e),this._requestHandler){var a=[];for(var i in this._requestHandler)\"function\"==typeof this._requestHandler[i]&&a.push(i);return Promise.resolve(a)}return t&&(void 0!==t.baseUrl&&delete t.baseUrl,void 0!==t.paths&&void 0!==t.paths.vs&&delete t.paths.vs,t.catchError=!0,self.require.config(t)),new Promise(function(e,t){self.require([n],function(){for(var n=[],a=0;a<arguments.length;a++)n[a]=arguments[a];var i=n[0];if(r._requestHandler=i.create(),r._requestHandler){var o=[];for(var s in r._requestHandler)\"function\"==typeof r._requestHandler[s]&&o.push(s);e(o)}else t(new Error(\"No RequestHandler!\"))},t)})},e}());function F(e,n){return function e(n,t,r,a,i){if(a<=r)return;var o=r+(a-r)/2|0;e(n,t,r,o,i);e(n,t,o+1,a,i);if(t(n[o],n[o+1])<=0)return;!function(e,n,t,r,a,i){for(var o=t,s=r+1,l=t;l<=a;l++)i[l]=e[l];for(var l=t;l<=a;l++)o>r?e[l]=i[s++]:s>a?e[l]=i[o++]:n(i[s],i[o])<0?e[l]=i[s++]:e[l]=i[o++]}(n,t,r,o,a,i)}(e,n,0,e.length-1,[]),e}var G=function(){function e(e,n,t,r){this.originalStart=e,this.originalLength=n,this.modifiedStart=t,this.modifiedLength=r}return e.prototype.getOriginalEnd=function(){return this.originalStart+this.originalLength},e.prototype.getModifiedEnd=function(){return this.modifiedStart+this.modifiedLength},e}();function V(e){return{getLength:function(){return e.length},getElementAtIndex:function(n){return e.charCodeAt(n)}}}function K(e,n,t){return new W(V(e),V(n)).ComputeDiff(t)}var B,H=function(){function e(){}return e.Assert=function(e,n){if(!e)throw new Error(n)},e}(),U=function(){function e(){}return e.Copy=function(e,n,t,r,a){for(var i=0;i<a;i++)t[r+i]=e[n+i]},e}(),j=function(){function e(){this.m_changes=[],this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE,this.m_originalCount=0,this.m_modifiedCount=0}return e.prototype.MarkNextChange=function(){(this.m_originalCount>0||this.m_modifiedCount>0)&&this.m_changes.push(new G(this.m_originalStart,this.m_originalCount,this.m_modifiedStart,this.m_modifiedCount)),this.m_originalCount=0,this.m_modifiedCount=0,this.m_originalStart=Number.MAX_VALUE,this.m_modifiedStart=Number.MAX_VALUE},e.prototype.AddOriginalElement=function(e,n){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,n),this.m_originalCount++},e.prototype.AddModifiedElement=function(e,n){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,n),this.m_modifiedCount++},e.prototype.getChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes},e.prototype.getReverseChanges=function(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes.reverse(),this.m_changes},e}(),W=function(){function e(e,n,t){void 0===t&&(t=null),this.OriginalSequence=e,this.ModifiedSequence=n,this.ContinueProcessingPredicate=t,this.m_forwardHistory=[],this.m_reverseHistory=[]}return e.prototype.ElementsAreEqual=function(e,n){return this.OriginalSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(n)},e.prototype.OriginalElementsAreEqual=function(e,n){return this.OriginalSequence.getElementAtIndex(e)===this.OriginalSequence.getElementAtIndex(n)},e.prototype.ModifiedElementsAreEqual=function(e,n){return this.ModifiedSequence.getElementAtIndex(e)===this.ModifiedSequence.getElementAtIndex(n)},e.prototype.ComputeDiff=function(e){return this._ComputeDiff(0,this.OriginalSequence.getLength()-1,0,this.ModifiedSequence.getLength()-1,e)},e.prototype._ComputeDiff=function(e,n,t,r,a){var i=this.ComputeDiffRecursive(e,n,t,r,[!1]);return a?this.PrettifyChanges(i):i},e.prototype.ComputeDiffRecursive=function(e,n,t,r,a){for(a[0]=!1;e<=n&&t<=r&&this.ElementsAreEqual(e,t);)e++,t++;for(;n>=e&&r>=t&&this.ElementsAreEqual(n,r);)n--,r--;if(e>n||t>r){var i=void 0;return t<=r?(H.Assert(e===n+1,\"originalStart should only be one more than originalEnd\"),i=[new G(e,0,t,r-t+1)]):e<=n?(H.Assert(t===r+1,\"modifiedStart should only be one more than modifiedEnd\"),i=[new G(e,n-e+1,t,0)]):(H.Assert(e===n+1,\"originalStart should only be one more than originalEnd\"),H.Assert(t===r+1,\"modifiedStart should only be one more than modifiedEnd\"),i=[]),i}var o=[0],s=[0],l=this.ComputeRecursionPoint(e,n,t,r,o,s,a),c=o[0],u=s[0];if(null!==l)return l;if(!a[0]){var d=this.ComputeDiffRecursive(e,c,t,u,a),m=[];return m=a[0]?[new G(c+1,n-(c+1)+1,u+1,r-(u+1)+1)]:this.ComputeDiffRecursive(c+1,n,u+1,r,a),this.ConcatenateChanges(d,m)}return[new G(e,n-e+1,t,r-t+1)]},e.prototype.WALKTRACE=function(e,n,t,r,a,i,o,s,l,c,u,d,m,p,f,g,_,v){var y,h,b=null,E=new j,T=n,S=t,L=m[0]-g[0]-r,x=Number.MIN_VALUE,A=this.m_forwardHistory.length-1;do{(h=L+e)===T||h<S&&l[h-1]<l[h+1]?(p=(u=l[h+1])-L-r,u<x&&E.MarkNextChange(),x=u,E.AddModifiedElement(u+1,p),L=h+1-e):(p=(u=l[h-1]+1)-L-r,u<x&&E.MarkNextChange(),x=u-1,E.AddOriginalElement(u,p+1),L=h-1-e),A>=0&&(e=(l=this.m_forwardHistory[A])[0],T=1,S=l.length-1)}while(--A>=-1);if(y=E.getReverseChanges(),v[0]){var C=m[0]+1,D=g[0]+1;if(null!==y&&y.length>0){var k=y[y.length-1];C=Math.max(C,k.getOriginalEnd()),D=Math.max(D,k.getModifiedEnd())}b=[new G(C,d-C+1,D,f-D+1)]}else{E=new j,T=i,S=o,L=m[0]-g[0]-s,x=Number.MAX_VALUE,A=_?this.m_reverseHistory.length-1:this.m_reverseHistory.length-2;do{(h=L+a)===T||h<S&&c[h-1]>=c[h+1]?(p=(u=c[h+1]-1)-L-s,u>x&&E.MarkNextChange(),x=u+1,E.AddOriginalElement(u+1,p+1),L=h+1-a):(p=(u=c[h-1])-L-s,u>x&&E.MarkNextChange(),x=u,E.AddModifiedElement(u+1,p+1),L=h-1-a),A>=0&&(a=(c=this.m_reverseHistory[A])[0],T=1,S=c.length-1)}while(--A>=-1);b=E.getChanges()}return this.ConcatenateChanges(y,b)},e.prototype.ComputeRecursionPoint=function(e,n,t,r,a,i,o){var s,l=0,c=0,u=0,d=0,m=0,p=0;e--,t--,a[0]=0,i[0]=0,this.m_forwardHistory=[],this.m_reverseHistory=[];var f,g,_=n-e+(r-t),v=_+1,y=new Array(v),h=new Array(v),b=r-t,E=n-e,T=e-t,S=n-r,L=(E-b)%2==0;for(y[b]=e,h[E]=n,o[0]=!1,s=1;s<=_/2+1;s++){var x=0,A=0;for(u=this.ClipDiagonalBound(b-s,s,b,v),d=this.ClipDiagonalBound(b+s,s,b,v),f=u;f<=d;f+=2){for(c=(l=f===u||f<d&&y[f-1]<y[f+1]?y[f+1]:y[f-1]+1)-(f-b)-T,g=l;l<n&&c<r&&this.ElementsAreEqual(l+1,c+1);)l++,c++;if(y[f]=l,l+c>x+A&&(x=l,A=c),!L&&Math.abs(f-E)<=s-1&&l>=h[f])return a[0]=l,i[0]=c,g<=h[f]&&s<=1448?this.WALKTRACE(b,u,d,T,E,m,p,S,y,h,l,n,a,c,r,i,L,o):null}var C=(x-e+(A-t)-s)/2;if(null!==this.ContinueProcessingPredicate&&!this.ContinueProcessingPredicate(x,this.OriginalSequence,C))return o[0]=!0,a[0]=x,i[0]=A,C>0&&s<=1448?this.WALKTRACE(b,u,d,T,E,m,p,S,y,h,l,n,a,c,r,i,L,o):[new G(++e,n-e+1,++t,r-t+1)];for(m=this.ClipDiagonalBound(E-s,s,E,v),p=this.ClipDiagonalBound(E+s,s,E,v),f=m;f<=p;f+=2){for(c=(l=f===m||f<p&&h[f-1]>=h[f+1]?h[f+1]-1:h[f-1])-(f-E)-S,g=l;l>e&&c>t&&this.ElementsAreEqual(l,c);)l--,c--;if(h[f]=l,L&&Math.abs(f-b)<=s&&l<=y[f])return a[0]=l,i[0]=c,g>=y[f]&&s<=1448?this.WALKTRACE(b,u,d,T,E,m,p,S,y,h,l,n,a,c,r,i,L,o):null}if(s<=1447){var D=new Array(d-u+2);D[0]=b-u+1,U.Copy(y,u,D,1,d-u+1),this.m_forwardHistory.push(D),(D=new Array(p-m+2))[0]=E-m+1,U.Copy(h,m,D,1,p-m+1),this.m_reverseHistory.push(D)}}return this.WALKTRACE(b,u,d,T,E,m,p,S,y,h,l,n,a,c,r,i,L,o)},e.prototype.PrettifyChanges=function(e){for(var n=0;n<e.length;n++){for(var t=e[n],r=n<e.length-1?e[n+1].originalStart:this.OriginalSequence.getLength(),a=n<e.length-1?e[n+1].modifiedStart:this.ModifiedSequence.getLength(),i=t.originalLength>0,o=t.modifiedLength>0;t.originalStart+t.originalLength<r&&t.modifiedStart+t.modifiedLength<a&&(!i||this.OriginalElementsAreEqual(t.originalStart,t.originalStart+t.originalLength))&&(!o||this.ModifiedElementsAreEqual(t.modifiedStart,t.modifiedStart+t.modifiedLength));)t.originalStart++,t.modifiedStart++;var s=[null];n<e.length-1&&this.ChangesOverlap(e[n],e[n+1],s)&&(e[n]=s[0],e.splice(n+1,1),n--)}for(n=e.length-1;n>=0;n--){t=e[n],r=0,a=0;if(n>0){var l=e[n-1];l.originalLength>0&&(r=l.originalStart+l.originalLength),l.modifiedLength>0&&(a=l.modifiedStart+l.modifiedLength)}i=t.originalLength>0,o=t.modifiedLength>0;for(var c=0,u=this._boundaryScore(t.originalStart,t.originalLength,t.modifiedStart,t.modifiedLength),d=1;;d++){var m=t.originalStart-d,p=t.modifiedStart-d;if(m<r||p<a)break;if(i&&!this.OriginalElementsAreEqual(m,m+t.originalLength))break;if(o&&!this.ModifiedElementsAreEqual(p,p+t.modifiedLength))break;var f=this._boundaryScore(m,t.originalLength,p,t.modifiedLength);f>u&&(u=f,c=d)}t.originalStart-=c,t.modifiedStart-=c}return e},e.prototype._OriginalIsBoundary=function(e){if(e<=0||e>=this.OriginalSequence.getLength()-1)return!0;var n=this.OriginalSequence.getElementAtIndex(e);return\"string\"==typeof n&&/^\\s*$/.test(n)},e.prototype._OriginalRegionIsBoundary=function(e,n){if(this._OriginalIsBoundary(e)||this._OriginalIsBoundary(e-1))return!0;if(n>0){var t=e+n;if(this._OriginalIsBoundary(t-1)||this._OriginalIsBoundary(t))return!0}return!1},e.prototype._ModifiedIsBoundary=function(e){if(e<=0||e>=this.ModifiedSequence.getLength()-1)return!0;var n=this.ModifiedSequence.getElementAtIndex(e);return\"string\"==typeof n&&/^\\s*$/.test(n)},e.prototype._ModifiedRegionIsBoundary=function(e,n){if(this._ModifiedIsBoundary(e)||this._ModifiedIsBoundary(e-1))return!0;if(n>0){var t=e+n;if(this._ModifiedIsBoundary(t-1)||this._ModifiedIsBoundary(t))return!0}return!1},e.prototype._boundaryScore=function(e,n,t,r){return(this._OriginalRegionIsBoundary(e,n)?1:0)+(this._ModifiedRegionIsBoundary(t,r)?1:0)},e.prototype.ConcatenateChanges=function(e,n){var t=[];if(0===e.length||0===n.length)return n.length>0?n:e;if(this.ChangesOverlap(e[e.length-1],n[0],t)){var r=new Array(e.length+n.length-1);return U.Copy(e,0,r,0,e.length-1),r[e.length-1]=t[0],U.Copy(n,1,r,e.length,n.length-1),r}r=new Array(e.length+n.length);return U.Copy(e,0,r,0,e.length),U.Copy(n,0,r,e.length,n.length),r},e.prototype.ChangesOverlap=function(e,n,t){if(H.Assert(e.originalStart<=n.originalStart,\"Left change is not less than or equal to right change\"),H.Assert(e.modifiedStart<=n.modifiedStart,\"Left change is not less than or equal to right change\"),e.originalStart+e.originalLength>=n.originalStart||e.modifiedStart+e.modifiedLength>=n.modifiedStart){var r=e.originalStart,a=e.originalLength,i=e.modifiedStart,o=e.modifiedLength;return e.originalStart+e.originalLength>=n.originalStart&&(a=n.originalStart+n.originalLength-e.originalStart),e.modifiedStart+e.modifiedLength>=n.modifiedStart&&(o=n.modifiedStart+n.modifiedLength-e.modifiedStart),t[0]=new G(r,a,i,o),!0}return t[0]=null,!1},e.prototype.ClipDiagonalBound=function(e,n,t,r){if(e>=0&&e<r)return e;var a=n%2==0;return e<0?a===(t%2==0)?0:1:a===((r-t-1)%2==0)?r-1:r-2},e}(),q=function(){var e=function(n,t){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)n.hasOwnProperty(t)&&(e[t]=n[t])})(n,t)};return function(n,t){function r(){this.constructor=n}e(n,t),n.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}}(),z=/^\\w[\\w\\d+.-]*$/,J=/^\\//,X=/^\\/\\//;var Y=\"\",Q=\"/\",Z=/^(([^:\\/?#]+?):)?(\\/\\/([^\\/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?/,$=function(){function e(e,n,t,r,a){\"object\"==typeof e?(this.scheme=e.scheme||Y,this.authority=e.authority||Y,this.path=e.path||Y,this.query=e.query||Y,this.fragment=e.fragment||Y):(this.scheme=e||Y,this.authority=n||Y,this.path=function(e,n){switch(e){case\"https\":case\"http\":case\"file\":n?n[0]!==Q&&(n=Q+n):n=Q}return n}(this.scheme,t||Y),this.query=r||Y,this.fragment=a||Y,function(e){if(!e.scheme)throw new Error('[UriError]: Scheme is missing: {scheme: \"\", authority: \"'+e.authority+'\", path: \"'+e.path+'\", query: \"'+e.query+'\", fragment: \"'+e.fragment+'\"}');if(e.scheme&&!z.test(e.scheme))throw new Error(\"[UriError]: Scheme contains illegal characters.\");if(e.path)if(e.authority){if(!J.test(e.path))throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash (\"/\") character')}else if(X.test(e.path))throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters (\"//\")')}(this))}return e.isUri=function(n){return n instanceof e||!!n&&(\"string\"==typeof n.authority&&\"string\"==typeof n.fragment&&\"string\"==typeof n.path&&\"string\"==typeof n.query&&\"string\"==typeof n.scheme)},Object.defineProperty(e.prototype,\"fsPath\",{get:function(){return ae(this)},enumerable:!0,configurable:!0}),e.prototype.with=function(e){if(!e)return this;var n=e.scheme,t=e.authority,r=e.path,a=e.query,i=e.fragment;return void 0===n?n=this.scheme:null===n&&(n=Y),void 0===t?t=this.authority:null===t&&(t=Y),void 0===r?r=this.path:null===r&&(r=Y),void 0===a?a=this.query:null===a&&(a=Y),void 0===i?i=this.fragment:null===i&&(i=Y),n===this.scheme&&t===this.authority&&r===this.path&&a===this.query&&i===this.fragment?this:new ee(n,t,r,a,i)},e.parse=function(e){var n=Z.exec(e);return n?new ee(n[2]||Y,decodeURIComponent(n[4]||Y),decodeURIComponent(n[5]||Y),decodeURIComponent(n[7]||Y),decodeURIComponent(n[9]||Y)):new ee(Y,Y,Y,Y,Y)},e.file=function(e){var n=Y;if(f&&(e=e.replace(/\\\\/g,Q)),e[0]===Q&&e[1]===Q){var t=e.indexOf(Q,2);-1===t?(n=e.substring(2),e=Q):(n=e.substring(2,t),e=e.substring(t)||Q)}return new ee(\"file\",n,e,Y,Y)},e.from=function(e){return new ee(e.scheme,e.authority,e.path,e.query,e.fragment)},e.prototype.toString=function(e){return void 0===e&&(e=!1),ie(this,e)},e.prototype.toJSON=function(){return this},e.revive=function(n){if(n){if(n instanceof e)return n;var t=new ee(n);return t._fsPath=n.fsPath,t._formatted=n.external,t}return n},e}(),ee=function(e){function n(){var n=null!==e&&e.apply(this,arguments)||this;return n._formatted=null,n._fsPath=null,n}return q(n,e),Object.defineProperty(n.prototype,\"fsPath\",{get:function(){return this._fsPath||(this._fsPath=ae(this)),this._fsPath},enumerable:!0,configurable:!0}),n.prototype.toString=function(e){return void 0===e&&(e=!1),e?ie(this,!0):(this._formatted||(this._formatted=ie(this,!1)),this._formatted)},n.prototype.toJSON=function(){var e={$mid:1};return this._fsPath&&(e.fsPath=this._fsPath),this._formatted&&(e.external=this._formatted),this.path&&(e.path=this.path),this.scheme&&(e.scheme=this.scheme),this.authority&&(e.authority=this.authority),this.query&&(e.query=this.query),this.fragment&&(e.fragment=this.fragment),e},n}($),ne=((B={})[58]=\"%3A\",B[47]=\"%2F\",B[63]=\"%3F\",B[35]=\"%23\",B[91]=\"%5B\",B[93]=\"%5D\",B[64]=\"%40\",B[33]=\"%21\",B[36]=\"%24\",B[38]=\"%26\",B[39]=\"%27\",B[40]=\"%28\",B[41]=\"%29\",B[42]=\"%2A\",B[43]=\"%2B\",B[44]=\"%2C\",B[59]=\"%3B\",B[61]=\"%3D\",B[32]=\"%20\",B);function te(e,n){for(var t=void 0,r=-1,a=0;a<e.length;a++){var i=e.charCodeAt(a);if(i>=97&&i<=122||i>=65&&i<=90||i>=48&&i<=57||45===i||46===i||95===i||126===i||n&&47===i)-1!==r&&(t+=encodeURIComponent(e.substring(r,a)),r=-1),void 0!==t&&(t+=e.charAt(a));else{void 0===t&&(t=e.substr(0,a));var o=ne[i];void 0!==o?(-1!==r&&(t+=encodeURIComponent(e.substring(r,a)),r=-1),t+=o):-1===r&&(r=a)}}return-1!==r&&(t+=encodeURIComponent(e.substring(r))),void 0!==t?t:e}function re(e){for(var n=void 0,t=0;t<e.length;t++){var r=e.charCodeAt(t);35===r||63===r?(void 0===n&&(n=e.substr(0,t)),n+=ne[r]):void 0!==n&&(n+=e[t])}return void 0!==n?n:e}function ae(e){var n;return n=e.authority&&e.path.length>1&&\"file\"===e.scheme?\"//\"+e.authority+e.path:47===e.path.charCodeAt(0)&&(e.path.charCodeAt(1)>=65&&e.path.charCodeAt(1)<=90||e.path.charCodeAt(1)>=97&&e.path.charCodeAt(1)<=122)&&58===e.path.charCodeAt(2)?e.path[1].toLowerCase()+e.path.substr(2):e.path,f&&(n=n.replace(/\\//g,\"\\\\\")),n}function ie(e,n){var t=n?re:te,r=\"\",a=e.scheme,i=e.authority,o=e.path,s=e.query,l=e.fragment;if(a&&(r+=a,r+=\":\"),(i||\"file\"===a)&&(r+=Q,r+=Q),i){var c=i.indexOf(\"@\");if(-1!==c){var u=i.substr(0,c);i=i.substr(c+1),-1===(c=u.indexOf(\":\"))?r+=t(u,!1):(r+=t(u.substr(0,c),!1),r+=\":\",r+=t(u.substr(c+1),!1)),r+=\"@\"}-1===(c=(i=i.toLowerCase()).indexOf(\":\"))?r+=t(i,!1):(r+=t(i.substr(0,c),!1),r+=i.substr(c))}if(o){if(o.length>=3&&47===o.charCodeAt(0)&&58===o.charCodeAt(2))(d=o.charCodeAt(1))>=65&&d<=90&&(o=\"/\"+String.fromCharCode(d+32)+\":\"+o.substr(3));else if(o.length>=2&&58===o.charCodeAt(1)){var d;(d=o.charCodeAt(0))>=65&&d<=90&&(o=String.fromCharCode(d+32)+\":\"+o.substr(2))}r+=t(o,!0)}return s&&(r+=\"?\",r+=t(s,!1)),l&&(r+=\"#\",r+=n?l:te(l,!1)),r}var oe=function(){function e(e,n){this.lineNumber=e,this.column=n}return e.prototype.with=function(n,t){return void 0===n&&(n=this.lineNumber),void 0===t&&(t=this.column),n===this.lineNumber&&t===this.column?this:new e(n,t)},e.prototype.delta=function(e,n){return void 0===e&&(e=0),void 0===n&&(n=0),this.with(this.lineNumber+e,this.column+n)},e.prototype.equals=function(n){return e.equals(this,n)},e.equals=function(e,n){return!e&&!n||!!e&&!!n&&e.lineNumber===n.lineNumber&&e.column===n.column},e.prototype.isBefore=function(n){return e.isBefore(this,n)},e.isBefore=function(e,n){return e.lineNumber<n.lineNumber||!(n.lineNumber<e.lineNumber)&&e.column<n.column},e.prototype.isBeforeOrEqual=function(n){return e.isBeforeOrEqual(this,n)},e.isBeforeOrEqual=function(e,n){return e.lineNumber<n.lineNumber||!(n.lineNumber<e.lineNumber)&&e.column<=n.column},e.compare=function(e,n){var t=0|e.lineNumber,r=0|n.lineNumber;return t===r?(0|e.column)-(0|n.column):t-r},e.prototype.clone=function(){return new e(this.lineNumber,this.column)},e.prototype.toString=function(){return\"(\"+this.lineNumber+\",\"+this.column+\")\"},e.lift=function(n){return new e(n.lineNumber,n.column)},e.isIPosition=function(e){return e&&\"number\"==typeof e.lineNumber&&\"number\"==typeof e.column},e}(),se=function(){function e(e,n,t,r){e>t||e===t&&n>r?(this.startLineNumber=t,this.startColumn=r,this.endLineNumber=e,this.endColumn=n):(this.startLineNumber=e,this.startColumn=n,this.endLineNumber=t,this.endColumn=r)}return e.prototype.isEmpty=function(){return e.isEmpty(this)},e.isEmpty=function(e){return e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn},e.prototype.containsPosition=function(n){return e.containsPosition(this,n)},e.containsPosition=function(e,n){return!(n.lineNumber<e.startLineNumber||n.lineNumber>e.endLineNumber)&&(!(n.lineNumber===e.startLineNumber&&n.column<e.startColumn)&&!(n.lineNumber===e.endLineNumber&&n.column>e.endColumn))},e.prototype.containsRange=function(n){return e.containsRange(this,n)},e.containsRange=function(e,n){return!(n.startLineNumber<e.startLineNumber||n.endLineNumber<e.startLineNumber)&&(!(n.startLineNumber>e.endLineNumber||n.endLineNumber>e.endLineNumber)&&(!(n.startLineNumber===e.startLineNumber&&n.startColumn<e.startColumn)&&!(n.endLineNumber===e.endLineNumber&&n.endColumn>e.endColumn)))},e.prototype.plusRange=function(n){return e.plusRange(this,n)},e.plusRange=function(n,t){var r,a,i,o;return t.startLineNumber<n.startLineNumber?(r=t.startLineNumber,a=t.startColumn):t.startLineNumber===n.startLineNumber?(r=t.startLineNumber,a=Math.min(t.startColumn,n.startColumn)):(r=n.startLineNumber,a=n.startColumn),t.endLineNumber>n.endLineNumber?(i=t.endLineNumber,o=t.endColumn):t.endLineNumber===n.endLineNumber?(i=t.endLineNumber,o=Math.max(t.endColumn,n.endColumn)):(i=n.endLineNumber,o=n.endColumn),new e(r,a,i,o)},e.prototype.intersectRanges=function(n){return e.intersectRanges(this,n)},e.intersectRanges=function(n,t){var r=n.startLineNumber,a=n.startColumn,i=n.endLineNumber,o=n.endColumn,s=t.startLineNumber,l=t.startColumn,c=t.endLineNumber,u=t.endColumn;return r<s?(r=s,a=l):r===s&&(a=Math.max(a,l)),i>c?(i=c,o=u):i===c&&(o=Math.min(o,u)),r>i?null:r===i&&a>o?null:new e(r,a,i,o)},e.prototype.equalsRange=function(n){return e.equalsRange(this,n)},e.equalsRange=function(e,n){return!!e&&!!n&&e.startLineNumber===n.startLineNumber&&e.startColumn===n.startColumn&&e.endLineNumber===n.endLineNumber&&e.endColumn===n.endColumn},e.prototype.getEndPosition=function(){return new oe(this.endLineNumber,this.endColumn)},e.prototype.getStartPosition=function(){return new oe(this.startLineNumber,this.startColumn)},e.prototype.toString=function(){return\"[\"+this.startLineNumber+\",\"+this.startColumn+\" -> \"+this.endLineNumber+\",\"+this.endColumn+\"]\"},e.prototype.setEndPosition=function(n,t){return new e(this.startLineNumber,this.startColumn,n,t)},e.prototype.setStartPosition=function(n,t){return new e(n,t,this.endLineNumber,this.endColumn)},e.prototype.collapseToStart=function(){return e.collapseToStart(this)},e.collapseToStart=function(n){return new e(n.startLineNumber,n.startColumn,n.startLineNumber,n.startColumn)},e.fromPositions=function(n,t){return void 0===t&&(t=n),new e(n.lineNumber,n.column,t.lineNumber,t.column)},e.lift=function(n){return n?new e(n.startLineNumber,n.startColumn,n.endLineNumber,n.endColumn):null},e.isIRange=function(e){return e&&\"number\"==typeof e.startLineNumber&&\"number\"==typeof e.startColumn&&\"number\"==typeof e.endLineNumber&&\"number\"==typeof e.endColumn},e.areIntersectingOrTouching=function(e,n){return!(e.endLineNumber<n.startLineNumber||e.endLineNumber===n.startLineNumber&&e.endColumn<n.startColumn)&&!(n.endLineNumber<e.startLineNumber||n.endLineNumber===e.startLineNumber&&n.endColumn<e.startColumn)},e.areIntersecting=function(e,n){return!(e.endLineNumber<n.startLineNumber||e.endLineNumber===n.startLineNumber&&e.endColumn<=n.startColumn)&&!(n.endLineNumber<e.startLineNumber||n.endLineNumber===e.startLineNumber&&n.endColumn<=e.startColumn)},e.compareRangesUsingStarts=function(e,n){if(e&&n){var t=0|e.startLineNumber,r=0|n.startLineNumber;if(t===r){var a=0|e.startColumn,i=0|n.startColumn;if(a===i){var o=0|e.endLineNumber,s=0|n.endLineNumber;return o===s?(0|e.endColumn)-(0|n.endColumn):o-s}return a-i}return t-r}return(e?1:0)-(n?1:0)},e.compareRangesUsingEnds=function(e,n){return e.endLineNumber===n.endLineNumber?e.endColumn===n.endColumn?e.startLineNumber===n.startLineNumber?e.startColumn-n.startColumn:e.startLineNumber-n.startLineNumber:e.endColumn-n.endColumn:e.endLineNumber-n.endLineNumber},e.spansMultipleLines=function(e){return e.endLineNumber>e.startLineNumber},e}();var le=5e3,ce=3;function ue(e,n,t,r){return new W(e,n,t).ComputeDiff(r)}var de=function(){function e(n){for(var t=[],r=[],a=0,i=n.length;a<i;a++)t[a]=e._getFirstNonBlankColumn(n[a],1),r[a]=e._getLastNonBlankColumn(n[a],1);this._lines=n,this._startColumns=t,this._endColumns=r}return e.prototype.getLength=function(){return this._lines.length},e.prototype.getElementAtIndex=function(e){return this._lines[e].substring(this._startColumns[e]-1,this._endColumns[e]-1)},e.prototype.getStartLineNumber=function(e){return e+1},e.prototype.getEndLineNumber=function(e){return e+1},e._getFirstNonBlankColumn=function(e,n){var t=function(e){for(var n=0,t=e.length;n<t;n++){var r=e.charCodeAt(n);if(32!==r&&9!==r)return n}return-1}(e);return-1===t?n:t+1},e._getLastNonBlankColumn=function(e,n){var t=function(e,n){void 0===n&&(n=e.length-1);for(var t=n;t>=0;t--){var r=e.charCodeAt(t);if(32!==r&&9!==r)return t}return-1}(e);return-1===t?n:t+2},e.prototype.getCharSequence=function(e,n,t){for(var r=[],a=[],i=[],o=0,s=n;s<=t;s++)for(var l=this._lines[s],c=e?this._startColumns[s]:1,u=e?this._endColumns[s]:l.length+1,d=c;d<u;d++)r[o]=l.charCodeAt(d-1),a[o]=s+1,i[o]=d,o++;return new me(r,a,i)},e}(),me=function(){function e(e,n,t){this._charCodes=e,this._lineNumbers=n,this._columns=t}return e.prototype.getLength=function(){return this._charCodes.length},e.prototype.getElementAtIndex=function(e){return this._charCodes[e]},e.prototype.getStartLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getStartColumn=function(e){return this._columns[e]},e.prototype.getEndLineNumber=function(e){return this._lineNumbers[e]},e.prototype.getEndColumn=function(e){return this._columns[e]+1},e}(),pe=function(){function e(e,n,t,r,a,i,o,s){this.originalStartLineNumber=e,this.originalStartColumn=n,this.originalEndLineNumber=t,this.originalEndColumn=r,this.modifiedStartLineNumber=a,this.modifiedStartColumn=i,this.modifiedEndLineNumber=o,this.modifiedEndColumn=s}return e.createFromDiffChange=function(n,t,r){var a,i,o,s,l,c,u,d;return 0===n.originalLength?(a=0,i=0,o=0,s=0):(a=t.getStartLineNumber(n.originalStart),i=t.getStartColumn(n.originalStart),o=t.getEndLineNumber(n.originalStart+n.originalLength-1),s=t.getEndColumn(n.originalStart+n.originalLength-1)),0===n.modifiedLength?(l=0,c=0,u=0,d=0):(l=r.getStartLineNumber(n.modifiedStart),c=r.getStartColumn(n.modifiedStart),u=r.getEndLineNumber(n.modifiedStart+n.modifiedLength-1),d=r.getEndColumn(n.modifiedStart+n.modifiedLength-1)),new e(a,i,o,s,l,c,u,d)},e}();var fe=function(){function e(e,n,t,r,a){this.originalStartLineNumber=e,this.originalEndLineNumber=n,this.modifiedStartLineNumber=t,this.modifiedEndLineNumber=r,this.charChanges=a}return e.createFromDiffResult=function(n,t,r,a,i,o,s){var l,c,u,d,m=void 0;if(0===t.originalLength?(l=r.getStartLineNumber(t.originalStart)-1,c=0):(l=r.getStartLineNumber(t.originalStart),c=r.getEndLineNumber(t.originalStart+t.originalLength-1)),0===t.modifiedLength?(u=a.getStartLineNumber(t.modifiedStart)-1,d=0):(u=a.getStartLineNumber(t.modifiedStart),d=a.getEndLineNumber(t.modifiedStart+t.modifiedLength-1)),o&&0!==t.originalLength&&0!==t.modifiedLength&&i()){var p=r.getCharSequence(n,t.originalStart,t.originalStart+t.originalLength-1),f=a.getCharSequence(n,t.modifiedStart,t.modifiedStart+t.modifiedLength-1),g=ue(p,f,i,!0);s&&(g=function(e){if(e.length<=1)return e;for(var n=[e[0]],t=n[0],r=1,a=e.length;r<a;r++){var i=e[r],o=i.originalStart-(t.originalStart+t.originalLength),s=i.modifiedStart-(t.modifiedStart+t.modifiedLength);Math.min(o,s)<ce?(t.originalLength=i.originalStart+i.originalLength-t.originalStart,t.modifiedLength=i.modifiedStart+i.modifiedLength-t.modifiedStart):(n.push(i),t=i)}return n}(g)),m=[];for(var _=0,v=g.length;_<v;_++)m.push(pe.createFromDiffChange(g[_],p,f))}return new e(l,c,u,d,m)},e}(),ge=function(){function e(e,n,t){this.shouldComputeCharChanges=t.shouldComputeCharChanges,this.shouldPostProcessCharChanges=t.shouldPostProcessCharChanges,this.shouldIgnoreTrimWhitespace=t.shouldIgnoreTrimWhitespace,this.shouldMakePrettyDiff=t.shouldMakePrettyDiff,this.maximumRunTimeMs=le,this.originalLines=e,this.modifiedLines=n,this.original=new de(e),this.modified=new de(n)}return e.prototype.computeDiff=function(){if(1===this.original.getLength()&&0===this.original.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:1,modifiedStartLineNumber:1,modifiedEndLineNumber:this.modified.getLength(),charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];if(1===this.modified.getLength()&&0===this.modified.getElementAtIndex(0).length)return[{originalStartLineNumber:1,originalEndLineNumber:this.original.getLength(),modifiedStartLineNumber:1,modifiedEndLineNumber:1,charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}];this.computationStartTime=(new Date).getTime();var e=ue(this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldMakePrettyDiff);if(this.shouldIgnoreTrimWhitespace){for(var n=[],t=0,r=e.length;t<r;t++)n.push(fe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,e[t],this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges));return n}for(var a=[],i=0,o=0,s=(t=-1,e.length);t<s;t++){for(var l=t+1<s?e[t+1]:null,c=l?l.originalStart:this.originalLines.length,u=l?l.modifiedStart:this.modifiedLines.length;i<c&&o<u;){var d=this.originalLines[i],m=this.modifiedLines[o];if(d!==m){for(var p=de._getFirstNonBlankColumn(d,1),f=de._getFirstNonBlankColumn(m,1);p>1&&f>1;){if(d.charCodeAt(p-2)!==m.charCodeAt(f-2))break;p--,f--}(p>1||f>1)&&this._pushTrimWhitespaceCharChange(a,i+1,1,p,o+1,1,f);for(var g=de._getLastNonBlankColumn(d,1),_=de._getLastNonBlankColumn(m,1),v=d.length+1,y=m.length+1;g<v&&_<y;){if(d.charCodeAt(g-1)!==d.charCodeAt(_-1))break;g++,_++}(g<v||_<y)&&this._pushTrimWhitespaceCharChange(a,i+1,g,v,o+1,_,y)}i++,o++}l&&(a.push(fe.createFromDiffResult(this.shouldIgnoreTrimWhitespace,l,this.original,this.modified,this._continueProcessingPredicate.bind(this),this.shouldComputeCharChanges,this.shouldPostProcessCharChanges)),i+=l.originalLength,o+=l.modifiedLength)}return a},e.prototype._pushTrimWhitespaceCharChange=function(e,n,t,r,a,i,o){if(!this._mergeTrimWhitespaceCharChange(e,n,t,r,a,i,o)){var s=void 0;this.shouldComputeCharChanges&&(s=[new pe(n,t,n,r,a,i,a,o)]),e.push(new fe(n,n,a,a,s))}},e.prototype._mergeTrimWhitespaceCharChange=function(e,n,t,r,a,i,o){var s=e.length;if(0===s)return!1;var l=e[s-1];return 0!==l.originalEndLineNumber&&0!==l.modifiedEndLineNumber&&(l.originalEndLineNumber+1===n&&l.modifiedEndLineNumber+1===a&&(l.originalEndLineNumber=n,l.modifiedEndLineNumber=a,this.shouldComputeCharChanges&&l.charChanges.push(new pe(n,t,n,r,a,i,a,o)),!0))},e.prototype._continueProcessingPredicate=function(){return 0===this.maximumRunTimeMs||(new Date).getTime()-this.computationStartTime<this.maximumRunTimeMs},e}(),_e=function(){function e(e,n,t){for(var r=new Uint8Array(e*n),a=0,i=e*n;a<i;a++)r[a]=t;this._data=r,this.rows=e,this.cols=n}return e.prototype.get=function(e,n){return this._data[e*this.cols+n]},e.prototype.set=function(e,n,t){this._data[e*this.cols+n]=t},e}();function ve(e){return e<0?0:e>255?255:0|e}function ye(e){return e<0?0:e>4294967295?4294967295:0|e}var he=function(){return function(e,n){this.index=e,this.remainder=n}}(),be=function(){function e(e){this.values=e,this.prefixSum=new Uint32Array(e.length),this.prefixSumValidIndex=new Int32Array(1),this.prefixSumValidIndex[0]=-1}return e.prototype.getCount=function(){return this.values.length},e.prototype.insertValues=function(e,n){e=ye(e);var t=this.values,r=this.prefixSum,a=n.length;return 0!==a&&(this.values=new Uint32Array(t.length+a),this.values.set(t.subarray(0,e),0),this.values.set(t.subarray(e),e+a),this.values.set(n,e),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSum=new Uint32Array(this.values.length),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.changeValue=function(e,n){return e=ye(e),n=ye(n),this.values[e]!==n&&(this.values[e]=n,e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),!0)},e.prototype.removeValues=function(e,n){e=ye(e),n=ye(n);var t=this.values,r=this.prefixSum;if(e>=t.length)return!1;var a=t.length-e;return n>=a&&(n=a),0!==n&&(this.values=new Uint32Array(t.length-n),this.values.set(t.subarray(0,e),0),this.values.set(t.subarray(e+n),e),this.prefixSum=new Uint32Array(this.values.length),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)},e.prototype.getTotalValue=function(){return 0===this.values.length?0:this._getAccumulatedValue(this.values.length-1)},e.prototype.getAccumulatedValue=function(e){return e<0?0:(e=ye(e),this._getAccumulatedValue(e))},e.prototype._getAccumulatedValue=function(e){if(e<=this.prefixSumValidIndex[0])return this.prefixSum[e];var n=this.prefixSumValidIndex[0]+1;0===n&&(this.prefixSum[0]=this.values[0],n++),e>=this.values.length&&(e=this.values.length-1);for(var t=n;t<=e;t++)this.prefixSum[t]=this.prefixSum[t-1]+this.values[t];return this.prefixSumValidIndex[0]=Math.max(this.prefixSumValidIndex[0],e),this.prefixSum[e]},e.prototype.getIndexOf=function(e){e=Math.floor(e),this.getTotalValue();for(var n=0,t=this.values.length-1,r=0,a=0,i=0;n<=t;)if(r=n+(t-n)/2|0,e<(i=(a=this.prefixSum[r])-this.values[r]))t=r-1;else{if(!(e>=a))break;n=r+1}return new he(r,e-i)},e}(),Ee=function(){function e(e,n,t,r){this._uri=e,this._lines=n,this._eol=t,this._versionId=r,this._lineStarts=null}return e.prototype.dispose=function(){this._lines.length=0},e.prototype.getText=function(){return this._lines.join(this._eol)},e.prototype.onEvents=function(e){e.eol&&e.eol!==this._eol&&(this._eol=e.eol,this._lineStarts=null);for(var n=e.changes,t=0,r=n.length;t<r;t++){var a=n[t];this._acceptDeleteRange(a.range),this._acceptInsertText(new oe(a.range.startLineNumber,a.range.startColumn),a.text)}this._versionId=e.versionId},e.prototype._ensureLineStarts=function(){if(!this._lineStarts){for(var e=this._eol.length,n=this._lines.length,t=new Uint32Array(n),r=0;r<n;r++)t[r]=this._lines[r].length+e;this._lineStarts=new be(t)}},e.prototype._setLineText=function(e,n){this._lines[e]=n,this._lineStarts&&this._lineStarts.changeValue(e,this._lines[e].length+this._eol.length)},e.prototype._acceptDeleteRange=function(e){if(e.startLineNumber!==e.endLineNumber)this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.endLineNumber-1].substring(e.endColumn-1)),this._lines.splice(e.startLineNumber,e.endLineNumber-e.startLineNumber),this._lineStarts&&this._lineStarts.removeValues(e.startLineNumber,e.endLineNumber-e.startLineNumber);else{if(e.startColumn===e.endColumn)return;this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.startLineNumber-1].substring(e.endColumn-1))}},e.prototype._acceptInsertText=function(e,n){if(0!==n.length){var t=n.split(/\\r\\n|\\r|\\n/);if(1!==t.length){t[t.length-1]+=this._lines[e.lineNumber-1].substring(e.column-1),this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+t[0]);for(var r=new Uint32Array(t.length-1),a=1;a<t.length;a++)this._lines.splice(e.lineNumber+a-1,0,t[a]),r[a-1]=t[a].length+this._eol.length;this._lineStarts&&this._lineStarts.insertValues(e.lineNumber,r)}else this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+t[0]+this._lines[e.lineNumber-1].substring(e.column-1))}},e}(),Te=\"`~!@#$%^&*()-=+[{]}\\\\|;:'\\\",.<>/?\";var Se=function(e){void 0===e&&(e=\"\");for(var n=\"(-?\\\\d*\\\\.\\\\d\\\\w*)|([^\",t=0;t<Te.length;t++)e.indexOf(Te[t])>=0||(n+=\"\\\\\"+Te[t]);return n+=\"\\\\s]+)\",new RegExp(n,\"g\")}();var Le=function(){function e(n){var t=ve(n);this._defaultValue=t,this._asciiMap=e._createAsciiMap(t),this._map=new Map}return e._createAsciiMap=function(e){for(var n=new Uint8Array(256),t=0;t<256;t++)n[t]=e;return n},e.prototype.set=function(e,n){var t=ve(n);e>=0&&e<256?this._asciiMap[e]=t:this._map.set(e,t)},e.prototype.get=function(e){return e>=0&&e<256?this._asciiMap[e]:this._map.get(e)||this._defaultValue},e}(),xe=function(){function e(e){for(var n=0,t=0,r=0,a=e.length;r<a;r++){var i=e[r],o=i[0];(c=i[1])>n&&(n=c),o>t&&(t=o),(u=i[2])>t&&(t=u)}var s=new _e(++t,++n,0);for(r=0,a=e.length;r<a;r++){var l=e[r],c=(o=l[0],l[1]),u=l[2];s.set(o,c,u)}this._states=s,this._maxCharCode=n}return e.prototype.nextState=function(e,n){return n<0||n>=this._maxCharCode?0:this._states.get(e,n)},e}(),Ae=null;var Ce=null;var De=function(){function e(){}return e._createLink=function(e,n,t,r,a){var i=a-1;do{var o=n.charCodeAt(i);if(2!==e.get(o))break;i--}while(i>r);if(r>0){var s=n.charCodeAt(r-1),l=n.charCodeAt(i);(40===s&&41===l||91===s&&93===l||123===s&&125===l)&&i--}return{range:{startLineNumber:t,startColumn:r+1,endLineNumber:t,endColumn:i+2},url:n.substring(r,i+1)}},e.computeLinks=function(n){for(var t=(null===Ae&&(Ae=new xe([[1,104,2],[1,72,2],[1,102,6],[1,70,6],[2,116,3],[2,84,3],[3,116,4],[3,84,4],[4,112,5],[4,80,5],[5,115,9],[5,83,9],[5,58,10],[6,105,7],[6,73,7],[7,108,8],[7,76,8],[8,101,9],[8,69,9],[9,58,10],[10,47,11],[11,47,12]])),Ae),r=function(){if(null===Ce){Ce=new Le(0);for(var e=0;e<\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".length;e++)Ce.set(\" \\t<>'\\\"、。｡､，．：；？！＠＃＄％＆＊‘“〈《「『【〔（［｛｢｣｝］）〕】』」》〉”’｀～…\".charCodeAt(e),1);for(e=0;e<\".,;\".length;e++)Ce.set(\".,;\".charCodeAt(e),2)}return Ce}(),a=[],i=1,o=n.getLineCount();i<=o;i++){for(var s=n.getLineContent(i),l=s.length,c=0,u=0,d=0,m=1,p=!1,f=!1,g=!1;c<l;){var _=!1,v=s.charCodeAt(c);if(13===m){var y=void 0;switch(v){case 40:p=!0,y=0;break;case 41:y=p?0:1;break;case 91:f=!0,y=0;break;case 93:y=f?0:1;break;case 123:g=!0,y=0;break;case 125:y=g?0:1;break;case 39:y=34===d||96===d?0:1;break;case 34:y=39===d||96===d?0:1;break;case 96:y=39===d||34===d?0:1;break;default:y=r.get(v)}1===y&&(a.push(e._createLink(r,s,i,u,c)),_=!0)}else if(12===m){1===(y=r.get(v))?_=!0:m=13}else 0===(m=t.nextState(m,v))&&(_=!0);_&&(m=1,p=!1,f=!1,g=!1,u=c+1,d=v),c++}13===m&&a.push(e._createLink(r,s,i,u,l))}return a},e}();var ke,Me=function(){function e(){this._defaultValueSet=[[\"true\",\"false\"],[\"True\",\"False\"],[\"Private\",\"Public\",\"Friend\",\"ReadOnly\",\"Partial\",\"Protected\",\"WriteOnly\"],[\"public\",\"protected\",\"private\"]]}return e.prototype.navigateValueSet=function(e,n,t,r,a){var i;if(e&&n&&(i=this.doNavigateValueSet(n,a)))return{range:e,value:i};if(t&&r&&(i=this.doNavigateValueSet(r,a)))return{range:t,value:i};return null},e.prototype.doNavigateValueSet=function(e,n){var t=this.numberReplace(e,n);return null!==t?t:this.textReplace(e,n)},e.prototype.numberReplace=function(e,n){var t=Math.pow(10,e.length-(e.lastIndexOf(\".\")+1)),r=Number(e),a=parseFloat(e);return isNaN(r)||isNaN(a)||r!==a?null:0!==r||n?(r=Math.floor(r*t),r+=n?t:-t,String(r/t)):null},e.prototype.textReplace=function(e,n){return this.valueSetsReplace(this._defaultValueSet,e,n)},e.prototype.valueSetsReplace=function(e,n,t){for(var r=null,a=0,i=e.length;null===r&&a<i;a++)r=this.valueSetReplace(e[a],n,t);return r},e.prototype.valueSetReplace=function(e,n,t){var r=e.indexOf(n);return r>=0?((r+=t?1:-1)<0?r=e.length-1:r%=e.length,e[r]):null},e.INSTANCE=new e,e}(),Oe=function(){function e(){this._keyCodeToStr=[],this._strToKeyCode=Object.create(null)}return e.prototype.define=function(e,n){this._keyCodeToStr[e]=n,this._strToKeyCode[n.toLowerCase()]=e},e.prototype.keyCodeToStr=function(e){return this._keyCodeToStr[e]},e.prototype.strToKeyCode=function(e){return this._strToKeyCode[e.toLowerCase()]||0},e}(),Ie=new Oe,Re=new Oe,Ne=new Oe;!function(){function e(e,n,t,r){void 0===t&&(t=n),void 0===r&&(r=t),Ie.define(e,n),Re.define(e,t),Ne.define(e,r)}e(0,\"unknown\"),e(1,\"Backspace\"),e(2,\"Tab\"),e(3,\"Enter\"),e(4,\"Shift\"),e(5,\"Ctrl\"),e(6,\"Alt\"),e(7,\"PauseBreak\"),e(8,\"CapsLock\"),e(9,\"Escape\"),e(10,\"Space\"),e(11,\"PageUp\"),e(12,\"PageDown\"),e(13,\"End\"),e(14,\"Home\"),e(15,\"LeftArrow\",\"Left\"),e(16,\"UpArrow\",\"Up\"),e(17,\"RightArrow\",\"Right\"),e(18,\"DownArrow\",\"Down\"),e(19,\"Insert\"),e(20,\"Delete\"),e(21,\"0\"),e(22,\"1\"),e(23,\"2\"),e(24,\"3\"),e(25,\"4\"),e(26,\"5\"),e(27,\"6\"),e(28,\"7\"),e(29,\"8\"),e(30,\"9\"),e(31,\"A\"),e(32,\"B\"),e(33,\"C\"),e(34,\"D\"),e(35,\"E\"),e(36,\"F\"),e(37,\"G\"),e(38,\"H\"),e(39,\"I\"),e(40,\"J\"),e(41,\"K\"),e(42,\"L\"),e(43,\"M\"),e(44,\"N\"),e(45,\"O\"),e(46,\"P\"),e(47,\"Q\"),e(48,\"R\"),e(49,\"S\"),e(50,\"T\"),e(51,\"U\"),e(52,\"V\"),e(53,\"W\"),e(54,\"X\"),e(55,\"Y\"),e(56,\"Z\"),e(57,\"Meta\"),e(58,\"ContextMenu\"),e(59,\"F1\"),e(60,\"F2\"),e(61,\"F3\"),e(62,\"F4\"),e(63,\"F5\"),e(64,\"F6\"),e(65,\"F7\"),e(66,\"F8\"),e(67,\"F9\"),e(68,\"F10\"),e(69,\"F11\"),e(70,\"F12\"),e(71,\"F13\"),e(72,\"F14\"),e(73,\"F15\"),e(74,\"F16\"),e(75,\"F17\"),e(76,\"F18\"),e(77,\"F19\"),e(78,\"NumLock\"),e(79,\"ScrollLock\"),e(80,\";\",\";\",\"OEM_1\"),e(81,\"=\",\"=\",\"OEM_PLUS\"),e(82,\",\",\",\",\"OEM_COMMA\"),e(83,\"-\",\"-\",\"OEM_MINUS\"),e(84,\".\",\".\",\"OEM_PERIOD\"),e(85,\"/\",\"/\",\"OEM_2\"),e(86,\"`\",\"`\",\"OEM_3\"),e(110,\"ABNT_C1\"),e(111,\"ABNT_C2\"),e(87,\"[\",\"[\",\"OEM_4\"),e(88,\"\\\\\",\"\\\\\",\"OEM_5\"),e(89,\"]\",\"]\",\"OEM_6\"),e(90,\"'\",\"'\",\"OEM_7\"),e(91,\"OEM_8\"),e(92,\"OEM_102\"),e(93,\"NumPad0\"),e(94,\"NumPad1\"),e(95,\"NumPad2\"),e(96,\"NumPad3\"),e(97,\"NumPad4\"),e(98,\"NumPad5\"),e(99,\"NumPad6\"),e(100,\"NumPad7\"),e(101,\"NumPad8\"),e(102,\"NumPad9\"),e(103,\"NumPad_Multiply\"),e(104,\"NumPad_Add\"),e(105,\"NumPad_Separator\"),e(106,\"NumPad_Subtract\"),e(107,\"NumPad_Decimal\"),e(108,\"NumPad_Divide\")}(),function(e){e.toString=function(e){return Ie.keyCodeToStr(e)},e.fromString=function(e){return Ie.strToKeyCode(e)},e.toUserSettingsUS=function(e){return Re.keyCodeToStr(e)},e.toUserSettingsGeneral=function(e){return Ne.keyCodeToStr(e)},e.fromUserSettings=function(e){return Re.strToKeyCode(e)||Ne.strToKeyCode(e)}}(ke||(ke={}));var we,Pe,Fe,Ge,Ve,Ke,Be,He,Ue,je,We,qe,ze,Je,Xe,Ye,Qe,Ze,$e,en,nn,tn,rn,an,on,sn,ln,cn=function(){var e=function(n,t){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)n.hasOwnProperty(t)&&(e[t]=n[t])})(n,t)};return function(n,t){function r(){this.constructor=n}e(n,t),n.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}}(),un=function(e){function n(n,t,r,a){var i=e.call(this,n,t,r,a)||this;return i.selectionStartLineNumber=n,i.selectionStartColumn=t,i.positionLineNumber=r,i.positionColumn=a,i}return cn(n,e),n.prototype.clone=function(){return new n(this.selectionStartLineNumber,this.selectionStartColumn,this.positionLineNumber,this.positionColumn)},n.prototype.toString=function(){return\"[\"+this.selectionStartLineNumber+\",\"+this.selectionStartColumn+\" -> \"+this.positionLineNumber+\",\"+this.positionColumn+\"]\"},n.prototype.equalsSelection=function(e){return n.selectionsEqual(this,e)},n.selectionsEqual=function(e,n){return e.selectionStartLineNumber===n.selectionStartLineNumber&&e.selectionStartColumn===n.selectionStartColumn&&e.positionLineNumber===n.positionLineNumber&&e.positionColumn===n.positionColumn},n.prototype.getDirection=function(){return this.selectionStartLineNumber===this.startLineNumber&&this.selectionStartColumn===this.startColumn?0:1},n.prototype.setEndPosition=function(e,t){return 0===this.getDirection()?new n(this.startLineNumber,this.startColumn,e,t):new n(e,t,this.startLineNumber,this.startColumn)},n.prototype.getPosition=function(){return new oe(this.positionLineNumber,this.positionColumn)},n.prototype.setStartPosition=function(e,t){return 0===this.getDirection()?new n(e,t,this.endLineNumber,this.endColumn):new n(this.endLineNumber,this.endColumn,e,t)},n.fromPositions=function(e,t){return void 0===t&&(t=e),new n(e.lineNumber,e.column,t.lineNumber,t.column)},n.liftSelection=function(e){return new n(e.selectionStartLineNumber,e.selectionStartColumn,e.positionLineNumber,e.positionColumn)},n.selectionsArrEqual=function(e,n){if(e&&!n||!e&&n)return!1;if(!e&&!n)return!0;if(e.length!==n.length)return!1;for(var t=0,r=e.length;t<r;t++)if(!this.selectionsEqual(e[t],n[t]))return!1;return!0},n.isISelection=function(e){return e&&\"number\"==typeof e.selectionStartLineNumber&&\"number\"==typeof e.selectionStartColumn&&\"number\"==typeof e.positionLineNumber&&\"number\"==typeof e.positionColumn},n.createWithDirection=function(e,t,r,a,i){return 0===i?new n(e,t,r,a):new n(r,a,e,t)},n}(se),dn=function(){function e(e,n,t){this.offset=0|e,this.type=n,this.language=t}return e.prototype.toString=function(){return\"(\"+this.offset+\", \"+this.type+\")\"},e}();!function(e){e[e.Unnecessary=1]=\"Unnecessary\"}(we||(we={})),function(e){e[e.Hint=1]=\"Hint\",e[e.Info=2]=\"Info\",e[e.Warning=4]=\"Warning\",e[e.Error=8]=\"Error\"}(Pe||(Pe={})),function(e){e[e.Unknown=0]=\"Unknown\",e[e.Backspace=1]=\"Backspace\",e[e.Tab=2]=\"Tab\",e[e.Enter=3]=\"Enter\",e[e.Shift=4]=\"Shift\",e[e.Ctrl=5]=\"Ctrl\",e[e.Alt=6]=\"Alt\",e[e.PauseBreak=7]=\"PauseBreak\",e[e.CapsLock=8]=\"CapsLock\",e[e.Escape=9]=\"Escape\",e[e.Space=10]=\"Space\",e[e.PageUp=11]=\"PageUp\",e[e.PageDown=12]=\"PageDown\",e[e.End=13]=\"End\",e[e.Home=14]=\"Home\",e[e.LeftArrow=15]=\"LeftArrow\",e[e.UpArrow=16]=\"UpArrow\",e[e.RightArrow=17]=\"RightArrow\",e[e.DownArrow=18]=\"DownArrow\",e[e.Insert=19]=\"Insert\",e[e.Delete=20]=\"Delete\",e[e.KEY_0=21]=\"KEY_0\",e[e.KEY_1=22]=\"KEY_1\",e[e.KEY_2=23]=\"KEY_2\",e[e.KEY_3=24]=\"KEY_3\",e[e.KEY_4=25]=\"KEY_4\",e[e.KEY_5=26]=\"KEY_5\",e[e.KEY_6=27]=\"KEY_6\",e[e.KEY_7=28]=\"KEY_7\",e[e.KEY_8=29]=\"KEY_8\",e[e.KEY_9=30]=\"KEY_9\",e[e.KEY_A=31]=\"KEY_A\",e[e.KEY_B=32]=\"KEY_B\",e[e.KEY_C=33]=\"KEY_C\",e[e.KEY_D=34]=\"KEY_D\",e[e.KEY_E=35]=\"KEY_E\",e[e.KEY_F=36]=\"KEY_F\",e[e.KEY_G=37]=\"KEY_G\",e[e.KEY_H=38]=\"KEY_H\",e[e.KEY_I=39]=\"KEY_I\",e[e.KEY_J=40]=\"KEY_J\",e[e.KEY_K=41]=\"KEY_K\",e[e.KEY_L=42]=\"KEY_L\",e[e.KEY_M=43]=\"KEY_M\",e[e.KEY_N=44]=\"KEY_N\",e[e.KEY_O=45]=\"KEY_O\",e[e.KEY_P=46]=\"KEY_P\",e[e.KEY_Q=47]=\"KEY_Q\",e[e.KEY_R=48]=\"KEY_R\",e[e.KEY_S=49]=\"KEY_S\",e[e.KEY_T=50]=\"KEY_T\",e[e.KEY_U=51]=\"KEY_U\",e[e.KEY_V=52]=\"KEY_V\",e[e.KEY_W=53]=\"KEY_W\",e[e.KEY_X=54]=\"KEY_X\",e[e.KEY_Y=55]=\"KEY_Y\",e[e.KEY_Z=56]=\"KEY_Z\",e[e.Meta=57]=\"Meta\",e[e.ContextMenu=58]=\"ContextMenu\",e[e.F1=59]=\"F1\",e[e.F2=60]=\"F2\",e[e.F3=61]=\"F3\",e[e.F4=62]=\"F4\",e[e.F5=63]=\"F5\",e[e.F6=64]=\"F6\",e[e.F7=65]=\"F7\",e[e.F8=66]=\"F8\",e[e.F9=67]=\"F9\",e[e.F10=68]=\"F10\",e[e.F11=69]=\"F11\",e[e.F12=70]=\"F12\",e[e.F13=71]=\"F13\",e[e.F14=72]=\"F14\",e[e.F15=73]=\"F15\",e[e.F16=74]=\"F16\",e[e.F17=75]=\"F17\",e[e.F18=76]=\"F18\",e[e.F19=77]=\"F19\",e[e.NumLock=78]=\"NumLock\",e[e.ScrollLock=79]=\"ScrollLock\",e[e.US_SEMICOLON=80]=\"US_SEMICOLON\",e[e.US_EQUAL=81]=\"US_EQUAL\",e[e.US_COMMA=82]=\"US_COMMA\",e[e.US_MINUS=83]=\"US_MINUS\",e[e.US_DOT=84]=\"US_DOT\",e[e.US_SLASH=85]=\"US_SLASH\",e[e.US_BACKTICK=86]=\"US_BACKTICK\",e[e.US_OPEN_SQUARE_BRACKET=87]=\"US_OPEN_SQUARE_BRACKET\",e[e.US_BACKSLASH=88]=\"US_BACKSLASH\",e[e.US_CLOSE_SQUARE_BRACKET=89]=\"US_CLOSE_SQUARE_BRACKET\",e[e.US_QUOTE=90]=\"US_QUOTE\",e[e.OEM_8=91]=\"OEM_8\",e[e.OEM_102=92]=\"OEM_102\",e[e.NUMPAD_0=93]=\"NUMPAD_0\",e[e.NUMPAD_1=94]=\"NUMPAD_1\",e[e.NUMPAD_2=95]=\"NUMPAD_2\",e[e.NUMPAD_3=96]=\"NUMPAD_3\",e[e.NUMPAD_4=97]=\"NUMPAD_4\",e[e.NUMPAD_5=98]=\"NUMPAD_5\",e[e.NUMPAD_6=99]=\"NUMPAD_6\",e[e.NUMPAD_7=100]=\"NUMPAD_7\",e[e.NUMPAD_8=101]=\"NUMPAD_8\",e[e.NUMPAD_9=102]=\"NUMPAD_9\",e[e.NUMPAD_MULTIPLY=103]=\"NUMPAD_MULTIPLY\",e[e.NUMPAD_ADD=104]=\"NUMPAD_ADD\",e[e.NUMPAD_SEPARATOR=105]=\"NUMPAD_SEPARATOR\",e[e.NUMPAD_SUBTRACT=106]=\"NUMPAD_SUBTRACT\",e[e.NUMPAD_DECIMAL=107]=\"NUMPAD_DECIMAL\",e[e.NUMPAD_DIVIDE=108]=\"NUMPAD_DIVIDE\",e[e.KEY_IN_COMPOSITION=109]=\"KEY_IN_COMPOSITION\",e[e.ABNT_C1=110]=\"ABNT_C1\",e[e.ABNT_C2=111]=\"ABNT_C2\",e[e.MAX_VALUE=112]=\"MAX_VALUE\"}(Fe||(Fe={})),function(e){e[e.LTR=0]=\"LTR\",e[e.RTL=1]=\"RTL\"}(Ge||(Ge={})),function(e){e[e.Auto=1]=\"Auto\",e[e.Hidden=2]=\"Hidden\",e[e.Visible=3]=\"Visible\"}(Ve||(Ve={})),function(e){e[e.Left=1]=\"Left\",e[e.Center=2]=\"Center\",e[e.Right=4]=\"Right\",e[e.Full=7]=\"Full\"}(Ke||(Ke={})),function(e){e[e.TextDefined=0]=\"TextDefined\",e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(Be||(Be={})),function(e){e[e.LF=1]=\"LF\",e[e.CRLF=2]=\"CRLF\"}(He||(He={})),function(e){e[e.LF=0]=\"LF\",e[e.CRLF=1]=\"CRLF\"}(Ue||(Ue={})),function(e){e[e.AlwaysGrowsWhenTypingAtEdges=0]=\"AlwaysGrowsWhenTypingAtEdges\",e[e.NeverGrowsWhenTypingAtEdges=1]=\"NeverGrowsWhenTypingAtEdges\",e[e.GrowsOnlyWhenTypingBefore=2]=\"GrowsOnlyWhenTypingBefore\",e[e.GrowsOnlyWhenTypingAfter=3]=\"GrowsOnlyWhenTypingAfter\"}(je||(je={})),function(e){e[e.Smooth=0]=\"Smooth\",e[e.Immediate=1]=\"Immediate\"}(We||(We={})),function(e){e[e.NotSet=0]=\"NotSet\",e[e.ContentFlush=1]=\"ContentFlush\",e[e.RecoverFromMarkers=2]=\"RecoverFromMarkers\",e[e.Explicit=3]=\"Explicit\",e[e.Paste=4]=\"Paste\",e[e.Undo=5]=\"Undo\",e[e.Redo=6]=\"Redo\"}(qe||(qe={})),function(e){e[e.None=0]=\"None\",e[e.Small=1]=\"Small\",e[e.Large=2]=\"Large\",e[e.SmallBlocks=3]=\"SmallBlocks\",e[e.LargeBlocks=4]=\"LargeBlocks\"}(ze||(ze={})),function(e){e[e.None=0]=\"None\",e[e.Same=1]=\"Same\",e[e.Indent=2]=\"Indent\",e[e.DeepIndent=3]=\"DeepIndent\"}(Je||(Je={})),function(e){e[e.Hidden=0]=\"Hidden\",e[e.Blink=1]=\"Blink\",e[e.Smooth=2]=\"Smooth\",e[e.Phase=3]=\"Phase\",e[e.Expand=4]=\"Expand\",e[e.Solid=5]=\"Solid\"}(Xe||(Xe={})),function(e){e[e.Line=1]=\"Line\",e[e.Block=2]=\"Block\",e[e.Underline=3]=\"Underline\",e[e.LineThin=4]=\"LineThin\",e[e.BlockOutline=5]=\"BlockOutline\",e[e.UnderlineThin=6]=\"UnderlineThin\"}(Ye||(Ye={})),function(e){e[e.Off=0]=\"Off\",e[e.On=1]=\"On\",e[e.Relative=2]=\"Relative\",e[e.Interval=3]=\"Interval\",e[e.Custom=4]=\"Custom\"}(Qe||(Qe={})),function(e){e[e.EXACT=0]=\"EXACT\",e[e.ABOVE=1]=\"ABOVE\",e[e.BELOW=2]=\"BELOW\"}(Ze||(Ze={})),function(e){e[e.TOP_RIGHT_CORNER=0]=\"TOP_RIGHT_CORNER\",e[e.BOTTOM_RIGHT_CORNER=1]=\"BOTTOM_RIGHT_CORNER\",e[e.TOP_CENTER=2]=\"TOP_CENTER\"}($e||($e={})),function(e){e[e.UNKNOWN=0]=\"UNKNOWN\",e[e.TEXTAREA=1]=\"TEXTAREA\",e[e.GUTTER_GLYPH_MARGIN=2]=\"GUTTER_GLYPH_MARGIN\",e[e.GUTTER_LINE_NUMBERS=3]=\"GUTTER_LINE_NUMBERS\",e[e.GUTTER_LINE_DECORATIONS=4]=\"GUTTER_LINE_DECORATIONS\",e[e.GUTTER_VIEW_ZONE=5]=\"GUTTER_VIEW_ZONE\",e[e.CONTENT_TEXT=6]=\"CONTENT_TEXT\",e[e.CONTENT_EMPTY=7]=\"CONTENT_EMPTY\",e[e.CONTENT_VIEW_ZONE=8]=\"CONTENT_VIEW_ZONE\",e[e.CONTENT_WIDGET=9]=\"CONTENT_WIDGET\",e[e.OVERVIEW_RULER=10]=\"OVERVIEW_RULER\",e[e.SCROLLBAR=11]=\"SCROLLBAR\",e[e.OVERLAY_WIDGET=12]=\"OVERLAY_WIDGET\",e[e.OUTSIDE_EDITOR=13]=\"OUTSIDE_EDITOR\"}(en||(en={})),function(e){e[e.None=0]=\"None\",e[e.Indent=1]=\"Indent\",e[e.IndentOutdent=2]=\"IndentOutdent\",e[e.Outdent=3]=\"Outdent\"}(nn||(nn={})),function(e){e[e.Method=0]=\"Method\",e[e.Function=1]=\"Function\",e[e.Constructor=2]=\"Constructor\",e[e.Field=3]=\"Field\",e[e.Variable=4]=\"Variable\",e[e.Class=5]=\"Class\",e[e.Struct=6]=\"Struct\",e[e.Interface=7]=\"Interface\",e[e.Module=8]=\"Module\",e[e.Property=9]=\"Property\",e[e.Event=10]=\"Event\",e[e.Operator=11]=\"Operator\",e[e.Unit=12]=\"Unit\",e[e.Value=13]=\"Value\",e[e.Constant=14]=\"Constant\",e[e.Enum=15]=\"Enum\",e[e.EnumMember=16]=\"EnumMember\",e[e.Keyword=17]=\"Keyword\",e[e.Text=18]=\"Text\",e[e.Color=19]=\"Color\",e[e.File=20]=\"File\",e[e.Reference=21]=\"Reference\",e[e.Customcolor=22]=\"Customcolor\",e[e.Folder=23]=\"Folder\",e[e.TypeParameter=24]=\"TypeParameter\",e[e.Snippet=25]=\"Snippet\"}(tn||(tn={})),function(e){e[e.KeepWhitespace=1]=\"KeepWhitespace\",e[e.InsertAsSnippet=4]=\"InsertAsSnippet\"}(rn||(rn={})),function(e){e[e.Invoke=0]=\"Invoke\",e[e.TriggerCharacter=1]=\"TriggerCharacter\",e[e.TriggerForIncompleteCompletions=2]=\"TriggerForIncompleteCompletions\"}(an||(an={})),function(e){e[e.Invoke=1]=\"Invoke\",e[e.TriggerCharacter=2]=\"TriggerCharacter\",e[e.ContentChange=3]=\"ContentChange\"}(on||(on={})),function(e){e[e.Text=0]=\"Text\",e[e.Read=1]=\"Read\",e[e.Write=2]=\"Write\"}(sn||(sn={})),function(e){e[e.File=0]=\"File\",e[e.Module=1]=\"Module\",e[e.Namespace=2]=\"Namespace\",e[e.Package=3]=\"Package\",e[e.Class=4]=\"Class\",e[e.Method=5]=\"Method\",e[e.Property=6]=\"Property\",e[e.Field=7]=\"Field\",e[e.Constructor=8]=\"Constructor\",e[e.Enum=9]=\"Enum\",e[e.Interface=10]=\"Interface\",e[e.Function=11]=\"Function\",e[e.Variable=12]=\"Variable\",e[e.Constant=13]=\"Constant\",e[e.String=14]=\"String\",e[e.Number=15]=\"Number\",e[e.Boolean=16]=\"Boolean\",e[e.Array=17]=\"Array\",e[e.Object=18]=\"Object\",e[e.Key=19]=\"Key\",e[e.Null=20]=\"Null\",e[e.EnumMember=21]=\"EnumMember\",e[e.Struct=22]=\"Struct\",e[e.Event=23]=\"Event\",e[e.Operator=24]=\"Operator\",e[e.TypeParameter=25]=\"TypeParameter\"}(ln||(ln={}));var mn=function(){function e(){}return e.chord=function(e,n){return function(e,n){return(e|(65535&n)<<16>>>0)>>>0}(e,n)},e.CtrlCmd=2048,e.Shift=1024,e.Alt=512,e.WinCtrl=256,e}();var pn=function(){var e=function(n,t){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)n.hasOwnProperty(t)&&(e[t]=n[t])})(n,t)};return function(n,t){function r(){this.constructor=n}e(n,t),n.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}}(),fn=function(e){function n(){return null!==e&&e.apply(this,arguments)||this}return pn(n,e),Object.defineProperty(n.prototype,\"uri\",{get:function(){return this._uri},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,\"version\",{get:function(){return this._versionId},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,\"eol\",{get:function(){return this._eol},enumerable:!0,configurable:!0}),n.prototype.getValue=function(){return this.getText()},n.prototype.getLinesContent=function(){return this._lines.slice(0)},n.prototype.getLineCount=function(){return this._lines.length},n.prototype.getLineContent=function(e){return this._lines[e-1]},n.prototype.getWordAtPosition=function(e,n){var t=function(e,n,t,r){n.lastIndex=0;var a=n.exec(t);if(!a)return null;var i=a[0].indexOf(\" \")>=0?function(e,n,t,r){var a,i=e-1-r;for(n.lastIndex=0;a=n.exec(t);){var o=a.index||0;if(o>i)return null;if(n.lastIndex>=i)return{word:a[0],startColumn:r+1+o,endColumn:r+1+n.lastIndex}}return null}(e,n,t,r):function(e,n,t,r){var a,i=e-1-r,o=t.lastIndexOf(\" \",i-1)+1,s=t.indexOf(\" \",i);for(-1===s&&(s=t.length),n.lastIndex=o;a=n.exec(t);){var l=a.index||0;if(l<=i&&n.lastIndex>=i)return{word:a[0],startColumn:r+1+l,endColumn:r+1+n.lastIndex}}return null}(e,n,t,r);return n.lastIndex=0,i}(e.column,function(e){var n=Se;if(e&&e instanceof RegExp)if(e.global)n=e;else{var t=\"g\";e.ignoreCase&&(t+=\"i\"),e.multiline&&(t+=\"m\"),n=new RegExp(e.source,t)}return n.lastIndex=0,n}(n),this._lines[e.lineNumber-1],0);return t?new se(e.lineNumber,t.startColumn,e.lineNumber,t.endColumn):null},n.prototype.getWordUntilPosition=function(e,n){var t=this.getWordAtPosition(e,n);return t?{word:this._lines[e.lineNumber-1].substring(t.startColumn-1,e.column-1),startColumn:t.startColumn,endColumn:e.column}:{word:\"\",startColumn:e.column,endColumn:e.column}},n.prototype.createWordIterator=function(e){var n,t,r=this,a=0,i=0,o=[],s=function(){if(i<o.length){var l=t.substring(o[i].start,o[i].end);return i+=1,n?n.value=l:n={done:!1,value:l},n}return a>=r._lines.length?E:(t=r._lines[a],o=r._wordenize(t,e),i=0,a+=1,s())};return{next:s}},n.prototype.getLineWords=function(e,n){for(var t=this._lines[e-1],r=[],a=0,i=this._wordenize(t,n);a<i.length;a++){var o=i[a];r.push({word:t.substring(o.start,o.end),startColumn:o.start+1,endColumn:o.end+1})}return r},n.prototype._wordenize=function(e,n){var t,r=[];for(n.lastIndex=0;(t=n.exec(e))&&0!==t[0].length;)r.push({start:t.index,end:t.index+t[0].length});return r},n.prototype.getValueInRange=function(e){if((e=this._validateRange(e)).startLineNumber===e.endLineNumber)return this._lines[e.startLineNumber-1].substring(e.startColumn-1,e.endColumn-1);var n=this._eol,t=e.startLineNumber-1,r=e.endLineNumber-1,a=[];a.push(this._lines[t].substring(e.startColumn-1));for(var i=t+1;i<r;i++)a.push(this._lines[i]);return a.push(this._lines[r].substring(0,e.endColumn-1)),a.join(n)},n.prototype.offsetAt=function(e){return e=this._validatePosition(e),this._ensureLineStarts(),this._lineStarts.getAccumulatedValue(e.lineNumber-2)+(e.column-1)},n.prototype.positionAt=function(e){e=Math.floor(e),e=Math.max(0,e),this._ensureLineStarts();var n=this._lineStarts.getIndexOf(e),t=this._lines[n.index].length;return{lineNumber:1+n.index,column:1+Math.min(n.remainder,t)}},n.prototype._validateRange=function(e){var n=this._validatePosition({lineNumber:e.startLineNumber,column:e.startColumn}),t=this._validatePosition({lineNumber:e.endLineNumber,column:e.endColumn});return n.lineNumber!==e.startLineNumber||n.column!==e.startColumn||t.lineNumber!==e.endLineNumber||t.column!==e.endColumn?{startLineNumber:n.lineNumber,startColumn:n.column,endLineNumber:t.lineNumber,endColumn:t.column}:e},n.prototype._validatePosition=function(e){if(!oe.isIPosition(e))throw new Error(\"bad position\");var n=e.lineNumber,t=e.column,r=!1;if(n<1)n=1,t=1,r=!0;else if(n>this._lines.length)n=this._lines.length,t=this._lines[n-1].length+1,r=!0;else{var a=this._lines[n-1].length+1;t<1?(t=1,r=!0):t>a&&(t=a,r=!0)}return r?{lineNumber:n,column:t}:e},n}(Ee),gn=function(e){function n(n){var t=e.call(this,n)||this;return t._models=Object.create(null),t}return pn(n,e),n.prototype.dispose=function(){this._models=Object.create(null)},n.prototype._getModel=function(e){return this._models[e]},n.prototype._getModels=function(){var e=this,n=[];return Object.keys(this._models).forEach(function(t){return n.push(e._models[t])}),n},n.prototype.acceptNewModel=function(e){this._models[e.url]=new fn($.parse(e.url),e.lines,e.EOL,e.versionId)},n.prototype.acceptModelChanged=function(e,n){this._models[e]&&this._models[e].onEvents(n)},n.prototype.acceptRemovedModel=function(e){this._models[e]&&delete this._models[e]},n}(function(){function e(e){this._foreignModuleFactory=e,this._foreignModule=null}return e.prototype.computeDiff=function(e,n,t){var r=this._getModel(e),a=this._getModel(n);if(!r||!a)return Promise.resolve(null);var i=r.getLinesContent(),o=a.getLinesContent(),s=new ge(i,o,{shouldComputeCharChanges:!0,shouldPostProcessCharChanges:!0,shouldIgnoreTrimWhitespace:t,shouldMakePrettyDiff:!0}).computeDiff(),l=!(s.length>0)&&this._modelsAreIdentical(r,a);return Promise.resolve({identical:l,changes:s})},e.prototype._modelsAreIdentical=function(e,n){var t=e.getLineCount();if(t!==n.getLineCount())return!1;for(var r=1;r<=t;r++){if(e.getLineContent(r)!==n.getLineContent(r))return!1}return!0},e.prototype.computeMoreMinimalEdits=function(n,t){var r=this._getModel(n);if(!r)return Promise.resolve(t);for(var a=[],i=void 0,o=0,s=t=F(t,function(e,n){return e.range&&n.range?se.compareRangesUsingStarts(e.range,n.range):(e.range?0:1)-(n.range?0:1)});o<s.length;o++){var l=s[o],c=l.range,u=l.text,d=l.eol;if(\"number\"==typeof d&&(i=d),c){var m=r.getValueInRange(c);if(m!==(u=u.replace(/\\r\\n|\\n|\\r/g,r.eol)))if(Math.max(u.length,m.length)>e._diffLimit)a.push({range:c,text:u});else for(var p=K(m,u,!1),f=r.offsetAt(se.lift(c).getStartPosition()),g=0,_=p;g<_.length;g++){var v=_[g],y=r.positionAt(f+v.originalStart),h=r.positionAt(f+v.originalStart+v.originalLength),b={text:u.substr(v.modifiedStart,v.modifiedLength),range:{startLineNumber:y.lineNumber,startColumn:y.column,endLineNumber:h.lineNumber,endColumn:h.column}};r.getValueInRange(b.range)!==b.text&&a.push(b)}}}return\"number\"==typeof i&&a.push({eol:i,text:void 0,range:void 0}),Promise.resolve(a)},e.prototype.computeLinks=function(e){var n=this._getModel(e);return n?Promise.resolve(function(e){return e&&\"function\"==typeof e.getLineCount&&\"function\"==typeof e.getLineContent?De.computeLinks(e):[]}(n)):Promise.resolve(null)},e.prototype.textualSuggest=function(n,t,r,a){var i=this._getModel(n);if(!i)return Promise.resolve(null);var o=[],s=new RegExp(r,a),l=i.getWordUntilPosition(t,s),c=Object.create(null);c[l.word]=!0;for(var u=i.createWordIterator(s),d=u.next();!d.done&&o.length<=e._suggestionsLimit;d=u.next()){var m=d.value;c[m]||(c[m]=!0,isNaN(Number(m))&&o.push({kind:18,label:m,insertText:m,range:{startLineNumber:t.lineNumber,startColumn:l.startColumn,endLineNumber:t.lineNumber,endColumn:l.endColumn}}))}return Promise.resolve({suggestions:o})},e.prototype.computeWordRanges=function(e,n,t,r){var a=this._getModel(e);if(!a)return Promise.resolve(Object.create(null));for(var i=new RegExp(t,r),o=Object.create(null),s=n.startLineNumber;s<n.endLineNumber;s++)for(var l=0,c=a.getLineWords(s,i);l<c.length;l++){var u=c[l];if(isNaN(Number(u.word))){var d=o[u.word];d||(d=[],o[u.word]=d),d.push({startLineNumber:s,startColumn:u.startColumn,endLineNumber:s,endColumn:u.endColumn})}}return Promise.resolve(o)},e.prototype.navigateValueSet=function(e,n,t,r,a){var i=this._getModel(e);if(!i)return Promise.resolve(null);var o=new RegExp(r,a);n.startColumn===n.endColumn&&(n={startLineNumber:n.startLineNumber,startColumn:n.startColumn,endLineNumber:n.endLineNumber,endColumn:n.endColumn+1});var s=i.getValueInRange(n),l=i.getWordAtPosition({lineNumber:n.startLineNumber,column:n.startColumn},o);if(!l)return Promise.resolve(null);var c=i.getValueInRange(l),u=Me.INSTANCE.navigateValueSet(n,s,l,c,t);return Promise.resolve(u)},e.prototype.loadForeignModule=function(e,n){var t=this,r={getMirrorModels:function(){return t._getModels()}};if(this._foreignModuleFactory){this._foreignModule=this._foreignModuleFactory(r,n);var a=[];for(var i in this._foreignModule)\"function\"==typeof this._foreignModule[i]&&a.push(i);return Promise.resolve(a)}return Promise.reject(new Error(\"Unexpected usage\"))},e.prototype.fmr=function(e,n){if(!this._foreignModule||\"function\"!=typeof this._foreignModule[e])return Promise.reject(new Error(\"Missing requestHandler or method: \"+e));try{return Promise.resolve(this._foreignModule[e].apply(this._foreignModule,n))}catch(e){return Promise.reject(e)}},e._diffLimit=1e4,e._suggestionsLimit=1e4,e}());\"function\"==typeof importScripts&&(g.monaco={editor:void 0,languages:void 0,CancellationTokenSource:k,Emitter:x,KeyCode:Fe,KeyMod:mn,Position:oe,Range:se,Selection:un,SelectionDirection:Ge,MarkerSeverity:Pe,MarkerTag:we,Promise:t,Uri:$,Token:dn});var _n=!1;function vn(e){if(!_n){_n=!0;var n=new gn(e),t=new P(function(e){self.postMessage(e)},n);self.onmessage=function(e){t.onmessage(e.data)}}}self.onmessage=function(e){_n||vn(null)};var yn=function(){return(yn=Object.assign||function(e){for(var n,t=1,r=arguments.length;t<r;t++)for(var a in n=arguments[t])Object.prototype.hasOwnProperty.call(n,a)&&(e[a]=n[a]);return e}).apply(this,arguments)},hn=function(e,n){var t,r,a,i,o={label:0,sent:function(){if(1&a[0])throw a[1];return a[1]},trys:[],ops:[]};return i={next:s(0),throw:s(1),return:s(2)},\"function\"==typeof Symbol&&(i[Symbol.iterator]=function(){return this}),i;function s(i){return function(s){return function(i){if(t)throw new TypeError(\"Generator is already executing.\");for(;o;)try{if(t=1,r&&(a=2&i[0]?r.return:i[0]?r.throw||((a=r.return)&&a.call(r),0):r.next)&&!(a=a.call(r,i[1])).done)return a;switch(r=0,a&&(i=[2&i[0],a.value]),i[0]){case 0:case 1:a=i;break;case 4:return o.label++,{value:i[1],done:!1};case 5:o.label++,r=i[1],i=[0];continue;case 7:i=o.ops.pop(),o.trys.pop();continue;default:if(!(a=(a=o.trys).length>0&&a[a.length-1])&&(6===i[0]||2===i[0])){o=0;continue}if(3===i[0]&&(!a||i[1]>a[0]&&i[1]<a[3])){o.label=i[1];break}if(6===i[0]&&o.label<a[1]){o.label=a[1],a=i;break}if(a&&o.label<a[2]){o.label=a[2],o.ops.push(i);break}a[2]&&o.ops.pop(),o.trys.pop();continue}i=n.call(e,o)}catch(e){i=[6,e],r=0}finally{t=a=0}if(5&i[0])throw i[1];return{value:i[0]?i[1]:void 0,done:!0}}([i,s])}}},bn=function(e,n){return Object.defineProperty?Object.defineProperty(e,\"raw\",{value:n}):e.raw=n,e},En=function(){var e=function(n,t){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)n.hasOwnProperty(t)&&(e[t]=n[t])})(n,t)};return function(n,t){function r(){this.constructor=n}e(n,t),n.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}}();!function(e){e.versionMajorMinor=\"3.1\",e.version=e.versionMajorMinor+\".6\"}(Tn||(Tn={})),function(e){!function(e){e[e.LessThan=-1]=\"LessThan\",e[e.EqualTo=0]=\"EqualTo\",e[e.GreaterThan=1]=\"GreaterThan\"}(e.Comparison||(e.Comparison={}))}(Tn||(Tn={})),function(e){function n(){var e=Object.create(null);return e.__=void 0,delete e.__,e}function t(){return new e.MapCtr}var r;function a(e,n,t){if(void 0===t&&(t=D),e)for(var r=0,a=e;r<a.length;r++){if(t(a[r],n))return!0}return!1}function i(e,n){var t;if(e){t=[];for(var r=0;r<e.length;r++)t.push(n(e[r],r))}return t}function o(e,n){if(e){if(!n)return e.length>0;for(var t=0,r=e;t<r.length;t++){if(n(r[t]))return!0}}return!1}function s(e,n){return n<0?e.length+n:n}function l(e,n,t,r){if(void 0===n||0===n.length)return e;if(void 0===e)return n.slice(t,r);t=void 0===t?0:s(n,t),r=void 0===r?n.length:s(n,r);for(var a=t;a<r&&a<n.length;a++)void 0!==n[a]&&e.push(n[a]);return e}function c(e,n,t){return!a(e,n,t)&&(e.push(n),!0)}function u(e,n,t){n.sort(function(n,r){return t(e[n],e[r])||M(n,r)})}function d(e,n){return e.slice().sort(n)}function m(e){var n=0;return{next:function(){return n===e.length?{value:void 0,done:!0}:{value:e[++n-1],done:!1}}}}function p(e,n,t,r,a){if(!e||0===e.length)return-1;for(var i=a||0,o=e.length-1,s=t(n);i<=o;){var l=i+(o-i>>1);switch(r(t(e[l]),s)){case-1:i=l+1;break;case 0:return l;case 1:o=l-1}}return~i}function f(e,n,t,r,a){if(e&&e.length>0){var i=e.length;if(i>0){var o=void 0===r||r<0?0:r,s=void 0===a||o+a>i-1?i-1:o+a,l=void 0;for(arguments.length<=2?(l=e[o],o++):l=t;o<=s;)l=n(l,e[o],o),o++;return l}}return t}e.emptyArray=[],e.createMap=t,e.createMapFromEntries=function(e){for(var n=t(),r=0,a=e;r<a.length;r++){var i=a[r],o=i[0],s=i[1];n.set(o,s)}return n},e.createMapFromTemplate=function(n){var t=new e.MapCtr;for(var r in n)_.call(n,r)&&t.set(r,n[r]);return t},e.MapCtr=\"undefined\"!=typeof Map&&\"entries\"in Map.prototype?Map:(r=function(){function e(e,n){this.index=0,this.data=e,this.selector=n,this.keys=Object.keys(e)}return e.prototype.next=function(){var e=this.index;return e<this.keys.length?(this.index++,{value:this.selector(this.data,this.keys[e]),done:!1}):{value:void 0,done:!0}},e}(),function(){function e(){this.data=n(),this.size=0}return e.prototype.get=function(e){return this.data[e]},e.prototype.set=function(e,n){return this.has(e)||this.size++,this.data[e]=n,this},e.prototype.has=function(e){return e in this.data},e.prototype.delete=function(e){return!!this.has(e)&&(this.size--,delete this.data[e],!0)},e.prototype.clear=function(){this.data=n(),this.size=0},e.prototype.keys=function(){return new r(this.data,function(e,n){return n})},e.prototype.values=function(){return new r(this.data,function(e,n){return e[n]})},e.prototype.entries=function(){return new r(this.data,function(e,n){return[n,e[n]]})},e.prototype.forEach=function(e){for(var n in this.data)e(this.data[n],n)},e}()),e.length=function(e){return e?e.length:0},e.forEach=function(e,n){if(e)for(var t=0;t<e.length;t++){var r=n(e[t],t);if(r)return r}},e.firstDefined=function(e,n){if(void 0!==e)for(var t=0;t<e.length;t++){var r=n(e[t],t);if(void 0!==r)return r}},e.firstDefinedIterator=function(e,n){for(;;){var t=e.next(),r=t.value;if(t.done)return;var a=n(r);if(void 0!==a)return a}},e.zipWith=function(e,n,t){var r=[];g.assertEqual(e.length,n.length);for(var a=0;a<e.length;a++)r.push(t(e[a],n[a],a));return r},e.zipToIterator=function(e,n){g.assertEqual(e.length,n.length);var t=0;return{next:function(){return t===e.length?{value:void 0,done:!0}:{value:[e[++t-1],n[t-1]],done:!1}}}},e.zipToMap=function(e,n){g.assert(e.length===n.length);for(var r=t(),a=0;a<e.length;++a)r.set(e[a],n[a]);return r},e.every=function(e,n){if(e)for(var t=0;t<e.length;t++)if(!n(e[t],t))return!1;return!0},e.find=function(e,n){for(var t=0;t<e.length;t++){var r=e[t];if(n(r,t))return r}},e.findLast=function(e,n){for(var t=e.length-1;t>=0;t--){var r=e[t];if(n(r,t))return r}},e.findIndex=function(e,n,t){for(var r=t||0;r<e.length;r++)if(n(e[r],r))return r;return-1},e.findLastIndex=function(e,n,t){for(var r=void 0===t?e.length-1:t;r>=0;r--)if(n(e[r],r))return r;return-1},e.findMap=function(e,n){for(var t=0;t<e.length;t++){var r=n(e[t],t);if(r)return r}return g.fail()},e.contains=a,e.arraysEqual=function(e,n,t){return void 0===t&&(t=D),e.length===n.length&&e.every(function(e,r){return t(e,n[r])})},e.indexOfAnyCharCode=function(e,n,t){for(var r=t||0;r<e.length;r++)if(a(n,e.charCodeAt(r)))return r;return-1},e.countWhere=function(e,n){var t=0;if(e)for(var r=0;r<e.length;r++)n(e[r],r)&&t++;return t},e.filter=function(e,n){if(e){for(var t=e.length,r=0;r<t&&n(e[r]);)r++;if(r<t){var a=e.slice(0,r);for(r++;r<t;){var i=e[r];n(i)&&a.push(i),r++}return a}}return e},e.filterMutate=function(e,n){for(var t=0,r=0;r<e.length;r++)n(e[r],r,e)&&(e[t]=e[r],t++);e.length=t},e.clear=function(e){e.length=0},e.map=i,e.mapIterator=function(e,n){return{next:function(){var t=e.next();return t.done?t:{value:n(t.value),done:!1}}}},e.sameMap=function(e,n){if(e)for(var t=0;t<e.length;t++){var r=e[t],a=n(r,t);if(r!==a){var i=e.slice(0,t);for(i.push(a),t++;t<e.length;t++)i.push(n(e[t],t));return i}}return e},e.flatten=function(e){var n;if(e){n=[];for(var t=0,r=e;t<r.length;t++){var a=r[t];a&&(S(a)?l(n,a):n.push(a))}}return n},e.flatMap=function(e,n){var t;if(e){t=[];for(var r=0;r<e.length;r++){var a=n(e[r],r);a&&(S(a)?l(t,a):t.push(a))}}return t},e.flatMapIterator=function(n,t){var r=n.next();if(r.done)return e.emptyIterator;var a=i(r.value);return{next:function(){for(;;){var e=a.next();if(!e.done)return e;var t=n.next();if(t.done)return t;a=i(t.value)}}};function i(n){var r=t(n);return void 0===r?e.emptyIterator:S(r)?m(r):r}},e.sameFlatMap=function(e,n){var t;if(e)for(var r=0;r<e.length;r++){var a=e[r],i=n(a,r);(t||a!==i||S(i))&&(t||(t=e.slice(0,r)),S(i)?l(t,i):t.push(i))}return t||e},e.mapAllOrFail=function(e,n){for(var t=[],r=0;r<e.length;r++){var a=n(e[r],r);if(void 0===a)return;t.push(a)}return t},e.mapDefined=function(e,n){var t=[];if(e)for(var r=0;r<e.length;r++){var a=n(e[r],r);void 0!==a&&t.push(a)}return t},e.mapDefinedIterator=function(e,n){return{next:function(){for(;;){var t=e.next();if(t.done)return t;var r=n(t.value);if(void 0!==r)return{value:r,done:!1}}}}},e.emptyIterator={next:function(){return{value:void 0,done:!0}}},e.singleIterator=function(e){var n=!1;return{next:function(){var t=n;return n=!0,t?{value:void 0,done:!0}:{value:e,done:!1}}}},e.spanMap=function(e,n,t){var r;if(e){r=[];for(var a=e.length,i=void 0,o=void 0,s=0,l=0;s<a;){for(;l<a;){if(o=n(e[l],l),0===l)i=o;else if(o!==i)break;l++}if(s<l){var c=t(e.slice(s,l),i,s,l);c&&r.push(c),s=l}i=o,l++}}return r},e.mapEntries=function(e,n){if(e){var r=t();return e.forEach(function(e,t){var a=n(t,e),i=a[0],o=a[1];r.set(i,o)}),r}},e.some=o,e.getRangesWhere=function(e,n,t){for(var r,a=0;a<e.length;a++)n(e[a])?r=void 0===r?a:r:void 0!==r&&(t(r,a),r=void 0);void 0!==r&&t(r,e.length)},e.concatenate=function(e,n){return o(n)?o(e)?e.concat(n):n:e},e.deduplicate=function(e,n,t){return 0===e.length?[]:1===e.length?e.slice():t?function(e,n,t){var r=e.map(function(e,n){return n});u(e,r,t);for(var a=e[r[0]],i=[r[0]],o=1;o<r.length;o++){var s=r[o],l=e[s];n(a,l)||(i.push(s),a=l)}return i.sort(),i.map(function(n){return e[n]})}(e,n,t):function(e,n){for(var t=[],r=0,a=e;r<a.length;r++)c(t,a[r],n);return t}(e,n)},e.insertSorted=function(e,n,t){if(0!==e.length){var r=p(e,n,x,t);r<0&&e.splice(~r,0,n)}else e.push(n)},e.sortAndDeduplicate=function(e,n,t){return function(e,n){if(0===e.length)return[];for(var t=e[0],r=[t],a=1;a<e.length;a++){var i=e[a];switch(n(i,t)){case!0:case 0:continue;case-1:return g.fail(\"Array is unsorted.\")}r.push(t=i)}return r}(d(e,n),t||n)},e.arrayIsEqualTo=function(e,n,t){if(void 0===t&&(t=D),!e||!n)return e===n;if(e.length!==n.length)return!1;for(var r=0;r<e.length;r++)if(!t(e[r],n[r],r))return!1;return!0},e.compact=function(e){var n;if(e)for(var t=0;t<e.length;t++){var r=e[t];!n&&r||(n||(n=e.slice(0,t)),r&&n.push(r))}return n||e},e.relativeComplement=function(e,n,t){if(!n||!e||0===n.length||0===e.length)return n;var r=[];e:for(var a=0,i=0;i<n.length;i++){i>0&&g.assertGreaterThanOrEqual(t(n[i],n[i-1]),0);n:for(var o=a;a<e.length;a++)switch(a>o&&g.assertGreaterThanOrEqual(t(e[a],e[a-1]),0),t(n[i],e[a])){case-1:r.push(n[i]);continue e;case 0:continue e;case 1:continue n}}return r},e.sum=function(e,n){for(var t=0,r=0,a=e;r<a.length;r++)t+=a[r][n];return t},e.append=function(e,n){return void 0===n?e:void 0===e?[n]:(e.push(n),e)},e.addRange=l,e.pushIfUnique=c,e.appendIfUnique=function(e,n,t){return e?(c(e,n,t),e):[n]},e.sort=d,e.arrayIterator=m,e.stableSort=function(e,n){var t=e.map(function(e,n){return n});return u(e,t,n),t.map(function(n){return e[n]})},e.rangeEquals=function(e,n,t,r){for(;t<r;){if(e[t]!==n[t])return!1;t++}return!0},e.elementAt=function(e,n){if(e&&(n=s(e,n))<e.length)return e[n]},e.firstOrUndefined=function(e){return 0===e.length?void 0:e[0]},e.first=function(e){return g.assert(0!==e.length),e[0]},e.lastOrUndefined=function(e){return 0===e.length?void 0:e[e.length-1]},e.last=function(e){return g.assert(0!==e.length),e[e.length-1]},e.singleOrUndefined=function(e){return e&&1===e.length?e[0]:void 0},e.singleOrMany=function(e){return e&&1===e.length?e[0]:e},e.replaceElement=function(e,n,t){var r=e.slice(0);return r[n]=t,r},e.binarySearch=p,e.reduceLeft=f;var g,_=Object.prototype.hasOwnProperty;function v(e,n){return _.call(e,n)}function y(e,n){for(var t,r=[],a=e.next(),i=a.value,o=a.done;!o;i=(t=e.next()).value,o=t.done,t)r.push(n?n(i):i);return r}function h(e,n,t){void 0===t&&(t=x);for(var r=b(),a=0,i=e;a<i.length;a++){var o=i[a];r.add(n(o),t(o))}return r}function b(){var e=t();return e.add=E,e.remove=T,e}function E(e,n){var t=this.get(e);return t?t.push(n):this.set(e,t=[n]),t}function T(e,n){var t=this.get(e);t&&(B(t,n),t.length||this.delete(e))}function S(e){return Array.isArray?Array.isArray(e):e instanceof Array}function L(e){}function x(e){return e}function A(e){return e.toLowerCase()}function C(e,n,t,r,a){if(a){for(var i=[],o=0;o<arguments.length;o++)i[o]=arguments[o];return function(e){return f(i,function(e,n){return n(e)},e)}}return r?function(a){return r(t(n(e(a))))}:t?function(r){return t(n(e(r)))}:n?function(t){return n(e(t))}:e?function(n){return e(n)}:function(e){return e}}function D(e,n){return e===n}function k(e,n){return e===n?0:void 0===e?-1:void 0===n?1:e<n?-1:1}function M(e,n){return k(e,n)}function O(e,n){return e===n?0:void 0===e?-1:void 0===n?1:(e=e.toUpperCase())<(n=n.toUpperCase())?-1:e>n?1:0}function I(e,n){return k(e,n)}e.hasProperty=v,e.getProperty=function(e,n){return _.call(e,n)?e[n]:void 0},e.getOwnKeys=function(e){var n=[];for(var t in e)_.call(e,t)&&n.push(t);return n},e.getOwnValues=function(e){var n=[];for(var t in e)_.call(e,t)&&n.push(e[t]);return n},e.arrayFrom=y,e.assign=function(e){for(var n=[],t=1;t<arguments.length;t++)n[t-1]=arguments[t];for(var r=0,a=n;r<a.length;r++){var i=a[r];for(var o in i)v(i,o)&&(e[o]=i[o])}return e},e.equalOwnProperties=function(e,n,t){if(void 0===t&&(t=D),e===n)return!0;if(!e||!n)return!1;for(var r in e)if(_.call(e,r)){if(!_.call(n,r))return!1;if(!t(e[r],n[r]))return!1}for(var r in n)if(_.call(n,r)&&!_.call(e,r))return!1;return!0},e.arrayToMap=function(e,n,r){void 0===r&&(r=x);for(var a=t(),i=0,o=e;i<o.length;i++){var s=o[i],l=n(s);void 0!==l&&a.set(l,r(s))}return a},e.arrayToNumericMap=function(e,n,t){void 0===t&&(t=x);for(var r=[],a=0,i=e;a<i.length;a++){var o=i[a];r[n(o)]=t(o)}return r},e.arrayToMultiMap=h,e.group=function(e,n){return y(h(e,n).values())},e.clone=function(e){var n={};for(var t in e)_.call(e,t)&&(n[t]=e[t]);return n},e.extend=function(e,n){var t={};for(var r in n)_.call(n,r)&&(t[r]=n[r]);for(var r in e)_.call(e,r)&&(t[r]=e[r]);return t},e.createMultiMap=b,e.isArray=S,e.toArray=function(e){return S(e)?e:[e]},e.isString=function(e){return\"string\"==typeof e},e.isNumber=function(e){return\"number\"==typeof e},e.tryCast=function(e,n){return void 0!==e&&n(e)?e:void 0},e.cast=function(e,n){return void 0!==e&&n(e)?e:g.fail(\"Invalid cast. The supplied value \"+e+\" did not pass the test '\"+g.getFunctionName(n)+\"'.\")},e.noop=L,e.returnFalse=function(){return!1},e.returnTrue=function(){return!0},e.identity=x,e.toLowerCase=A,e.notImplemented=function(){throw new Error(\"Not implemented\")},e.memoize=function(e){var n;return function(){return e&&(n=e(),e=void 0),n}},e.chain=function(e,n,t,r,a){if(a){for(var o=[],s=0;s<arguments.length;s++)o[s]=arguments[s];return function(e){return C.apply(void 0,i(o,function(n){return n(e)}))}}return r?function(a){return C(e(a),n(a),t(a),r(a))}:t?function(r){return C(e(r),n(r),t(r))}:n?function(t){return C(e(t),n(t))}:e?function(n){return C(e(n))}:function(e){return function(e){return e}}},e.compose=C,function(e){e[e.None=0]=\"None\",e[e.Normal=1]=\"Normal\",e[e.Aggressive=2]=\"Aggressive\",e[e.VeryAggressive=3]=\"VeryAggressive\"}(e.AssertionLevel||(e.AssertionLevel={})),function(e){function n(e,t){var r=new Error(e?\"Debug Failure. \"+e:\"Debug Failure.\");throw Error.captureStackTrace&&Error.captureStackTrace(r,t||n),r}function t(e,t){return null==e?n(t):e}e.currentAssertionLevel=0,e.isDebugging=!1,e.shouldAssert=function(n){return e.currentAssertionLevel>=n},e.assert=function e(t,r,a,i){t||(a&&(r+=\"\\r\\nVerbose Debug Information: \"+(\"string\"==typeof a?a:a())),n(r?\"False expression: \"+r:\"False expression.\",i||e))},e.assertEqual=function(e,t,r,a){e!==t&&n(\"Expected \"+e+\" === \"+t+\". \"+(r?a?r+\" \"+a:r:\"\"))},e.assertLessThan=function(e,t,r){e>=t&&n(\"Expected \"+e+\" < \"+t+\". \"+(r||\"\"))},e.assertLessThanOrEqual=function(e,t){e>t&&n(\"Expected \"+e+\" <= \"+t)},e.assertGreaterThanOrEqual=function(e,t){e<t&&n(\"Expected \"+e+\" >= \"+t)},e.fail=n,e.assertDefined=t,e.assertEachDefined=function(e,n){for(var r=0,a=e;r<a.length;r++)t(a[r],n);return e},e.assertNever=function e(t,r,a){return n(r||\"Illegal value: \"+t,a||e)},e.getFunctionName=function(e){if(\"function\"!=typeof e)return\"\";if(e.hasOwnProperty(\"name\"))return e.name;var n=Function.prototype.toString.call(e),t=/^function\\s+([\\w\\$]+)\\s*\\(/.exec(n);return t?t[1]:\"\"}}(g=e.Debug||(e.Debug={})),e.equateValues=D,e.equateStringsCaseInsensitive=function(e,n){return e===n||void 0!==e&&void 0!==n&&e.toUpperCase()===n.toUpperCase()},e.equateStringsCaseSensitive=function(e,n){return D(e,n)},e.compareValues=M,e.min=function(e,n,t){return-1===t(e,n)?e:n},e.compareStringsCaseInsensitive=O,e.compareStringsCaseSensitive=I,e.getStringComparer=function(e){return e?O:I};var R,N,w=function(){var e,n,t=function(){if(\"object\"==typeof Intl&&\"function\"==typeof Intl.Collator)return a;if(\"function\"==typeof String.prototype.localeCompare&&\"function\"==typeof String.prototype.toLocaleUpperCase&&\"a\".localeCompare(\"B\")<0)return i;return o}();return function(r){return void 0===r?e||(e=t(r)):\"en-US\"===r?n||(n=t(r)):t(r)};function r(e,n,t){if(e===n)return 0;if(void 0===e)return-1;if(void 0===n)return 1;var r=t(e,n);return r<0?-1:r>0?1:0}function a(e){var n=new Intl.Collator(e,{usage:\"sort\",sensitivity:\"variant\"}).compare;return function(e,t){return r(e,t,n)}}function i(e){return void 0!==e?o():function(e,t){return r(e,t,n)};function n(e,n){return e.localeCompare(n)}}function o(){return function(n,t){return r(n,t,e)};function e(e,t){return n(e.toUpperCase(),t.toUpperCase())||n(e,t)}function n(e,n){return e<n?-1:e>n?1:0}}}();function P(e,n,t){for(var r=new Array(n.length+1),a=new Array(n.length+1),i=t+1,o=0;o<=n.length;o++)r[o]=o;for(o=1;o<=e.length;o++){var s=e.charCodeAt(o-1),l=o>t?o-t:1,c=n.length>t+o?t+o:n.length;a[0]=o;for(var u=o,d=1;d<l;d++)a[d]=i;for(d=l;d<=c;d++){var m=s===n.charCodeAt(d-1)?r[d-1]:Math.min(r[d]+1,a[d-1]+1,r[d-1]+2);a[d]=m,u=Math.min(u,m)}for(d=c+1;d<=n.length;d++)a[d]=i;if(u>t)return;var p=r;r=a,a=p}var f=r[n.length];return f>t?void 0:f}function F(e,n){var t=e.length-n.length;return t>=0&&e.indexOf(n,t)===t}function G(e,n){return e.length>n.length&&F(e,n)}function V(e,n){for(var t=n;t<e.length-1;t++)e[t]=e[t+1];e.pop()}function K(e,n){e[n]=e[e.length-1],e.pop()}function B(e,n){return function(e,n){for(var t=0;t<e.length;t++)if(n(e[t]))return K(e,t),!0;return!1}(e,function(e){return e===n})}function H(e,n){return 0===e.lastIndexOf(n,0)}function U(e,n){var t=e.prefix,r=e.suffix;return n.length>=t.length+r.length&&H(n,t)&&F(n,r)}e.getUILocale=function(){return N},e.setUILocale=function(e){N!==e&&(N=e,R=void 0)},e.compareStringsCaseSensitiveUI=function(e,n){return(R||(R=w(N)))(e,n)},e.compareProperties=function(e,n,t,r){return e===n?0:void 0===e?-1:void 0===n?1:r(e[t],n[t])},e.compareBooleans=function(e,n){return M(e?1:0,n?1:0)},e.getSpellingSuggestion=function(e,n,t){for(var r,a=Math.min(2,Math.floor(.34*e.length)),i=Math.floor(.4*e.length)+1,o=!1,s=e.toLowerCase(),l=0,c=n;l<c.length;l++){var u=c[l],d=t(u);if(void 0!==d&&Math.abs(d.length-s.length)<=a){var m=d.toLowerCase();if(m===s){if(d===e)continue;return u}if(o)continue;if(d.length<3)continue;var p=P(s,m,i-1);if(void 0===p)continue;p<3?(o=!0,r=u):(g.assert(p<i),i=p,r=u)}}return r},e.endsWith=F,e.removeSuffix=function(e,n){return F(e,n)?e.slice(0,e.length-n.length):e},e.tryRemoveSuffix=function(e,n){return F(e,n)?e.slice(0,e.length-n.length):void 0},e.stringContains=function(e,n){return-1!==e.indexOf(n)},e.fileExtensionIs=G,e.fileExtensionIsOneOf=function(e,n){for(var t=0,r=n;t<r.length;t++)if(G(e,r[t]))return!0;return!1},e.removeMinAndVersionNumbers=function(e){var n=/[.-]((min)|(\\d+(\\.\\d+)*))$/;return e.replace(n,\"\").replace(n,\"\")},e.orderedRemoveItem=function(e,n){for(var t=0;t<e.length;t++)if(e[t]===n)return V(e,t),!0;return!1},e.orderedRemoveItemAt=V,e.unorderedRemoveItemAt=K,e.unorderedRemoveItem=B,e.createGetCanonicalFileName=function(e){return e?x:A},e.patternText=function(e){return e.prefix+\"*\"+e.suffix},e.matchedText=function(e,n){return g.assert(U(e,n)),n.substring(e.prefix.length,n.length-e.suffix.length)},e.findBestPatternMatch=function(e,n,t){for(var r,a=-1,i=0,o=e;i<o.length;i++){var s=o[i],l=n(s);U(l,t)&&l.prefix.length>a&&(a=l.prefix.length,r=s)}return r},e.startsWith=H,e.removePrefix=function(e,n){return H(e,n)?e.substr(n.length):e},e.tryRemovePrefix=function(e,n,t){return void 0===t&&(t=x),H(t(e),t(n))?e.substring(n.length):void 0},e.and=function(e,n){return function(t){return e(t)&&n(t)}},e.or=function(e,n){return function(t){return e(t)||n(t)}},e.assertType=function(e){},e.singleElementArray=function(e){return void 0===e?void 0:[e]},e.enumerateInsertsAndDeletes=function(e,n,t,r,a,i){i=i||L;for(var o=0,s=0,l=e.length,c=n.length;o<l&&s<c;){var u=e[o],d=n[s],m=t(u,d);-1===m?(r(u),o++):1===m?(a(d),s++):(i(d,u),o++,s++)}for(;o<l;)r(e[o++]);for(;s<c;)a(n[s++])},e.fill=function(e,n){return new Array(e).fill(0).map(function(e,t){return n(t)})}}(Tn||(Tn={})),function(e){e.timestamp=\"undefined\"!=typeof performance&&performance.now?function(){return performance.now()}:Date.now?Date.now:function(){return+new Date}}(Tn||(Tn={})),function(e){!function(n){var t,r,a,i=\"function\"==typeof onProfilerEvent&&!0===onProfilerEvent.profiler?onProfilerEvent:function(){},o=!1,s=0;n.mark=function(n){o&&(r.set(n,e.timestamp()),t.set(n,(t.get(n)||0)+1),i(n))},n.measure=function(n,t,i){if(o){var l=i&&r.get(i)||e.timestamp(),c=t&&r.get(t)||s;a.set(n,(a.get(n)||0)+(l-c))}},n.getCount=function(e){return t&&t.get(e)||0},n.getDuration=function(e){return a&&a.get(e)||0},n.forEachMeasure=function(e){a.forEach(function(n,t){e(t,n)})},n.enable=function(){t=e.createMap(),r=e.createMap(),a=e.createMap(),o=!0,s=e.timestamp()},n.disable=function(){o=!1}}(e.performance||(e.performance={}))}(Tn||(Tn={})),function(e){var n=/^(0|[1-9]\\d*)(?:\\.(0|[1-9]\\d*)(?:\\.(0|[1-9]\\d*)(?:\\-([a-z0-9-.]+))?(?:\\+([a-z0-9-.]+))?)?)?$/i,t=/^(?:0|[1-9]\\d*|[a-z-][a-z0-9-]*)(?:\\.(?:0|[1-9]\\d*|[a-z-][a-z0-9-]*))*$/i,r=/^[a-z0-9-]+(?:\\.[a-z0-9-]+)*$/i,a=/^(0|[1-9]\\d*)$/,i=function(){function n(n,a,i,s,l){if(void 0===a&&(a=0),void 0===i&&(i=0),void 0===s&&(s=\"\"),void 0===l&&(l=\"\"),\"string\"==typeof n){var c=e.Debug.assertDefined(o(n),\"Invalid version\");n=c.major,a=c.minor,i=c.patch,s=c.prerelease,l=c.build}e.Debug.assert(n>=0,\"Invalid argument: major\"),e.Debug.assert(a>=0,\"Invalid argument: minor\"),e.Debug.assert(i>=0,\"Invalid argument: patch\"),e.Debug.assert(!s||t.test(s),\"Invalid argument: prerelease\"),e.Debug.assert(!l||r.test(l),\"Invalid argument: build\"),this.major=n,this.minor=a,this.patch=i,this.prerelease=s?s.split(\".\"):e.emptyArray,this.build=l?l.split(\".\"):e.emptyArray}return n.tryParse=function(e){var t=o(e);if(t)return new n(t.major,t.minor,t.patch,t.prerelease,t.build)},n.prototype.compareTo=function(n){return this===n?0:void 0===n?1:e.compareValues(this.major,n.major)||e.compareValues(this.minor,n.minor)||e.compareValues(this.patch,n.patch)||function(n,t){if(n===t)return 0;if(0===n.length)return 0===t.length?0:1;if(0===t.length)return-1;for(var r=Math.min(n.length,t.length),i=0;i<r;i++){var o=n[i],s=t[i];if(o!==s){var l=a.test(o),c=a.test(s);if(l||c){if(l!==c)return l?-1:1;var u=e.compareValues(+o,+s);if(u)return u}else{var u=e.compareStringsCaseSensitive(o,s);if(u)return u}}}return e.compareValues(n.length,t.length)}(this.prerelease,n.prerelease)},n.prototype.increment=function(t){switch(t){case\"major\":return new n(this.major+1,0,0);case\"minor\":return new n(this.major,this.minor+1,0);case\"patch\":return new n(this.major,this.minor,this.patch+1);default:return e.Debug.assertNever(t)}},n.prototype.toString=function(){var n=this.major+\".\"+this.minor+\".\"+this.patch;return e.some(this.prerelease)&&(n+=\"-\"+this.prerelease.join(\".\")),e.some(this.build)&&(n+=\"+\"+this.build.join(\".\")),n},n.zero=new n(0,0,0),n}();function o(e){var a=n.exec(e);if(a){var i=a[1],o=a[2],s=void 0===o?\"0\":o,l=a[3],c=void 0===l?\"0\":l,u=a[4],d=void 0===u?\"\":u,m=a[5],p=void 0===m?\"\":m;if((!d||t.test(d))&&(!p||r.test(p)))return{major:parseInt(i,10),minor:parseInt(s,10),patch:parseInt(c,10),prerelease:d,build:p}}}e.Version=i;var s=function(){function n(n){this._alternatives=n?e.Debug.assertDefined(p(n),\"Invalid range spec.\"):e.emptyArray}return n.tryParse=function(e){var t=p(e);if(t){var r=new n(\"\");return r._alternatives=t,r}},n.prototype.test=function(e){return\"string\"==typeof e&&(e=new i(e)),function(e,n){if(0===n.length)return!0;for(var t=0,r=n;t<r.length;t++){var a=r[t];if(h(e,a))return!0}return!1}(e,this._alternatives)},n.prototype.toString=function(){return n=this._alternatives,e.map(n,E).join(\" || \")||\"*\";var n},n}();e.VersionRange=s;var l=/\\s*\\|\\|\\s*/g,c=/\\s+/g,u=/^([xX*0]|[1-9]\\d*)(?:\\.([xX*0]|[1-9]\\d*)(?:\\.([xX*0]|[1-9]\\d*)(?:-([a-z0-9-.]+))?(?:\\+([a-z0-9-.]+))?)?)?$/i,d=/^\\s*([a-z0-9-+.*]+)\\s+-\\s+([a-z0-9-+.*]+)\\s*$/i,m=/^\\s*(~|\\^|<|<=|>|>=|=)?\\s*([a-z0-9-+.*]+)$/i;function p(e){for(var n=[],t=0,r=e.trim().split(l);t<r.length;t++){var a=r[t];if(a){var i=[],o=d.exec(a);if(o){if(!g(o[1],o[2],i))return}else for(var s=0,u=a.split(c);s<u.length;s++){var p=u[s],f=m.exec(p);if(!f||!_(f[1],f[2],i))return}n.push(i)}}return n}function f(e){var n=u.exec(e);if(n){var t=n[1],r=n[2],a=void 0===r?\"*\":r,o=n[3],s=void 0===o?\"*\":o,l=n[4],c=n[5];return{version:new i(v(t)?0:parseInt(t,10),v(t)||v(a)?0:parseInt(a,10),v(t)||v(a)||v(s)?0:parseInt(s,10),l,c),major:t,minor:a,patch:s}}}function g(e,n,t){var r=f(e);if(!r)return!1;var a=f(n);return!!a&&(v(r.major)||t.push(y(\">=\",r.version)),v(a.major)||t.push(v(a.minor)?y(\"<\",a.version.increment(\"major\")):v(a.patch)?y(\"<\",a.version.increment(\"minor\")):y(\"<=\",a.version)),!0)}function _(e,n,t){var r=f(n);if(!r)return!1;var a=r.version,o=r.major,s=r.minor,l=r.patch;if(v(o))\"<\"!==e&&\">\"!==e||t.push(y(\"<\",i.zero));else switch(e){case\"~\":t.push(y(\">=\",a)),t.push(y(\"<\",a.increment(v(s)?\"major\":\"minor\")));break;case\"^\":t.push(y(\">=\",a)),t.push(y(\"<\",a.increment(a.major>0||v(s)?\"major\":a.minor>0||v(l)?\"minor\":\"patch\")));break;case\"<\":case\">=\":t.push(y(e,a));break;case\"<=\":case\">\":t.push(v(s)?y(\"<=\"===e?\"<\":\">=\",a.increment(\"major\")):v(l)?y(\"<=\"===e?\"<\":\">=\",a.increment(\"minor\")):y(e,a));break;case\"=\":case void 0:v(s)||v(l)?(t.push(y(\">=\",a)),t.push(y(\"<\",a.increment(v(s)?\"major\":\"minor\")))):t.push(y(\"=\",a));break;default:return!1}return!0}function v(e){return\"*\"===e||\"x\"===e||\"X\"===e}function y(e,n){return{operator:e,operand:n}}function h(e,n){for(var t=0,r=n;t<r.length;t++){var a=r[t];if(!b(e,a.operator,a.operand))return!1}return!0}function b(n,t,r){var a=n.compareTo(r);switch(t){case\"<\":return a<0;case\"<=\":return a<=0;case\">\":return a>0;case\">=\":return a>=0;case\"=\":return 0===a;default:return e.Debug.assertNever(t)}}function E(n){return e.map(n,T).join(\" \")}function T(e){return\"\"+e.operator+e.operand}}(Tn||(Tn={})),function(e){!function(e){e[e.Unknown=0]=\"Unknown\",e[e.EndOfFileToken=1]=\"EndOfFileToken\",e[e.SingleLineCommentTrivia=2]=\"SingleLineCommentTrivia\",e[e.MultiLineCommentTrivia=3]=\"MultiLineCommentTrivia\",e[e.NewLineTrivia=4]=\"NewLineTrivia\",e[e.WhitespaceTrivia=5]=\"WhitespaceTrivia\",e[e.ShebangTrivia=6]=\"ShebangTrivia\",e[e.ConflictMarkerTrivia=7]=\"ConflictMarkerTrivia\",e[e.NumericLiteral=8]=\"NumericLiteral\",e[e.StringLiteral=9]=\"StringLiteral\",e[e.JsxText=10]=\"JsxText\",e[e.JsxTextAllWhiteSpaces=11]=\"JsxTextAllWhiteSpaces\",e[e.RegularExpressionLiteral=12]=\"RegularExpressionLiteral\",e[e.NoSubstitutionTemplateLiteral=13]=\"NoSubstitutionTemplateLiteral\",e[e.TemplateHead=14]=\"TemplateHead\",e[e.TemplateMiddle=15]=\"TemplateMiddle\",e[e.TemplateTail=16]=\"TemplateTail\",e[e.OpenBraceToken=17]=\"OpenBraceToken\",e[e.CloseBraceToken=18]=\"CloseBraceToken\",e[e.OpenParenToken=19]=\"OpenParenToken\",e[e.CloseParenToken=20]=\"CloseParenToken\",e[e.OpenBracketToken=21]=\"OpenBracketToken\",e[e.CloseBracketToken=22]=\"CloseBracketToken\",e[e.DotToken=23]=\"DotToken\",e[e.DotDotDotToken=24]=\"DotDotDotToken\",e[e.SemicolonToken=25]=\"SemicolonToken\",e[e.CommaToken=26]=\"CommaToken\",e[e.LessThanToken=27]=\"LessThanToken\",e[e.LessThanSlashToken=28]=\"LessThanSlashToken\",e[e.GreaterThanToken=29]=\"GreaterThanToken\",e[e.LessThanEqualsToken=30]=\"LessThanEqualsToken\",e[e.GreaterThanEqualsToken=31]=\"GreaterThanEqualsToken\",e[e.EqualsEqualsToken=32]=\"EqualsEqualsToken\",e[e.ExclamationEqualsToken=33]=\"ExclamationEqualsToken\",e[e.EqualsEqualsEqualsToken=34]=\"EqualsEqualsEqualsToken\",e[e.ExclamationEqualsEqualsToken=35]=\"ExclamationEqualsEqualsToken\",e[e.EqualsGreaterThanToken=36]=\"EqualsGreaterThanToken\",e[e.PlusToken=37]=\"PlusToken\",e[e.MinusToken=38]=\"MinusToken\",e[e.AsteriskToken=39]=\"AsteriskToken\",e[e.AsteriskAsteriskToken=40]=\"AsteriskAsteriskToken\",e[e.SlashToken=41]=\"SlashToken\",e[e.PercentToken=42]=\"PercentToken\",e[e.PlusPlusToken=43]=\"PlusPlusToken\",e[e.MinusMinusToken=44]=\"MinusMinusToken\",e[e.LessThanLessThanToken=45]=\"LessThanLessThanToken\",e[e.GreaterThanGreaterThanToken=46]=\"GreaterThanGreaterThanToken\",e[e.GreaterThanGreaterThanGreaterThanToken=47]=\"GreaterThanGreaterThanGreaterThanToken\",e[e.AmpersandToken=48]=\"AmpersandToken\",e[e.BarToken=49]=\"BarToken\",e[e.CaretToken=50]=\"CaretToken\",e[e.ExclamationToken=51]=\"ExclamationToken\",e[e.TildeToken=52]=\"TildeToken\",e[e.AmpersandAmpersandToken=53]=\"AmpersandAmpersandToken\",e[e.BarBarToken=54]=\"BarBarToken\",e[e.QuestionToken=55]=\"QuestionToken\",e[e.ColonToken=56]=\"ColonToken\",e[e.AtToken=57]=\"AtToken\",e[e.EqualsToken=58]=\"EqualsToken\",e[e.PlusEqualsToken=59]=\"PlusEqualsToken\",e[e.MinusEqualsToken=60]=\"MinusEqualsToken\",e[e.AsteriskEqualsToken=61]=\"AsteriskEqualsToken\",e[e.AsteriskAsteriskEqualsToken=62]=\"AsteriskAsteriskEqualsToken\",e[e.SlashEqualsToken=63]=\"SlashEqualsToken\",e[e.PercentEqualsToken=64]=\"PercentEqualsToken\",e[e.LessThanLessThanEqualsToken=65]=\"LessThanLessThanEqualsToken\",e[e.GreaterThanGreaterThanEqualsToken=66]=\"GreaterThanGreaterThanEqualsToken\",e[e.GreaterThanGreaterThanGreaterThanEqualsToken=67]=\"GreaterThanGreaterThanGreaterThanEqualsToken\",e[e.AmpersandEqualsToken=68]=\"AmpersandEqualsToken\",e[e.BarEqualsToken=69]=\"BarEqualsToken\",e[e.CaretEqualsToken=70]=\"CaretEqualsToken\",e[e.Identifier=71]=\"Identifier\",e[e.BreakKeyword=72]=\"BreakKeyword\",e[e.CaseKeyword=73]=\"CaseKeyword\",e[e.CatchKeyword=74]=\"CatchKeyword\",e[e.ClassKeyword=75]=\"ClassKeyword\",e[e.ConstKeyword=76]=\"ConstKeyword\",e[e.ContinueKeyword=77]=\"ContinueKeyword\",e[e.DebuggerKeyword=78]=\"DebuggerKeyword\",e[e.DefaultKeyword=79]=\"DefaultKeyword\",e[e.DeleteKeyword=80]=\"DeleteKeyword\",e[e.DoKeyword=81]=\"DoKeyword\",e[e.ElseKeyword=82]=\"ElseKeyword\",e[e.EnumKeyword=83]=\"EnumKeyword\",e[e.ExportKeyword=84]=\"ExportKeyword\",e[e.ExtendsKeyword=85]=\"ExtendsKeyword\",e[e.FalseKeyword=86]=\"FalseKeyword\",e[e.FinallyKeyword=87]=\"FinallyKeyword\",e[e.ForKeyword=88]=\"ForKeyword\",e[e.FunctionKeyword=89]=\"FunctionKeyword\",e[e.IfKeyword=90]=\"IfKeyword\",e[e.ImportKeyword=91]=\"ImportKeyword\",e[e.InKeyword=92]=\"InKeyword\",e[e.InstanceOfKeyword=93]=\"InstanceOfKeyword\",e[e.NewKeyword=94]=\"NewKeyword\",e[e.NullKeyword=95]=\"NullKeyword\",e[e.ReturnKeyword=96]=\"ReturnKeyword\",e[e.SuperKeyword=97]=\"SuperKeyword\",e[e.SwitchKeyword=98]=\"SwitchKeyword\",e[e.ThisKeyword=99]=\"ThisKeyword\",e[e.ThrowKeyword=100]=\"ThrowKeyword\",e[e.TrueKeyword=101]=\"TrueKeyword\",e[e.TryKeyword=102]=\"TryKeyword\",e[e.TypeOfKeyword=103]=\"TypeOfKeyword\",e[e.VarKeyword=104]=\"VarKeyword\",e[e.VoidKeyword=105]=\"VoidKeyword\",e[e.WhileKeyword=106]=\"WhileKeyword\",e[e.WithKeyword=107]=\"WithKeyword\",e[e.ImplementsKeyword=108]=\"ImplementsKeyword\",e[e.InterfaceKeyword=109]=\"InterfaceKeyword\",e[e.LetKeyword=110]=\"LetKeyword\",e[e.PackageKeyword=111]=\"PackageKeyword\",e[e.PrivateKeyword=112]=\"PrivateKeyword\",e[e.ProtectedKeyword=113]=\"ProtectedKeyword\",e[e.PublicKeyword=114]=\"PublicKeyword\",e[e.StaticKeyword=115]=\"StaticKeyword\",e[e.YieldKeyword=116]=\"YieldKeyword\",e[e.AbstractKeyword=117]=\"AbstractKeyword\",e[e.AsKeyword=118]=\"AsKeyword\",e[e.AnyKeyword=119]=\"AnyKeyword\",e[e.AsyncKeyword=120]=\"AsyncKeyword\",e[e.AwaitKeyword=121]=\"AwaitKeyword\",e[e.BooleanKeyword=122]=\"BooleanKeyword\",e[e.ConstructorKeyword=123]=\"ConstructorKeyword\",e[e.DeclareKeyword=124]=\"DeclareKeyword\",e[e.GetKeyword=125]=\"GetKeyword\",e[e.InferKeyword=126]=\"InferKeyword\",e[e.IsKeyword=127]=\"IsKeyword\",e[e.KeyOfKeyword=128]=\"KeyOfKeyword\",e[e.ModuleKeyword=129]=\"ModuleKeyword\",e[e.NamespaceKeyword=130]=\"NamespaceKeyword\",e[e.NeverKeyword=131]=\"NeverKeyword\",e[e.ReadonlyKeyword=132]=\"ReadonlyKeyword\",e[e.RequireKeyword=133]=\"RequireKeyword\",e[e.NumberKeyword=134]=\"NumberKeyword\",e[e.ObjectKeyword=135]=\"ObjectKeyword\",e[e.SetKeyword=136]=\"SetKeyword\",e[e.StringKeyword=137]=\"StringKeyword\",e[e.SymbolKeyword=138]=\"SymbolKeyword\",e[e.TypeKeyword=139]=\"TypeKeyword\",e[e.UndefinedKeyword=140]=\"UndefinedKeyword\",e[e.UniqueKeyword=141]=\"UniqueKeyword\",e[e.UnknownKeyword=142]=\"UnknownKeyword\",e[e.FromKeyword=143]=\"FromKeyword\",e[e.GlobalKeyword=144]=\"GlobalKeyword\",e[e.OfKeyword=145]=\"OfKeyword\",e[e.QualifiedName=146]=\"QualifiedName\",e[e.ComputedPropertyName=147]=\"ComputedPropertyName\",e[e.TypeParameter=148]=\"TypeParameter\",e[e.Parameter=149]=\"Parameter\",e[e.Decorator=150]=\"Decorator\",e[e.PropertySignature=151]=\"PropertySignature\",e[e.PropertyDeclaration=152]=\"PropertyDeclaration\",e[e.MethodSignature=153]=\"MethodSignature\",e[e.MethodDeclaration=154]=\"MethodDeclaration\",e[e.Constructor=155]=\"Constructor\",e[e.GetAccessor=156]=\"GetAccessor\",e[e.SetAccessor=157]=\"SetAccessor\",e[e.CallSignature=158]=\"CallSignature\",e[e.ConstructSignature=159]=\"ConstructSignature\",e[e.IndexSignature=160]=\"IndexSignature\",e[e.TypePredicate=161]=\"TypePredicate\",e[e.TypeReference=162]=\"TypeReference\",e[e.FunctionType=163]=\"FunctionType\",e[e.ConstructorType=164]=\"ConstructorType\",e[e.TypeQuery=165]=\"TypeQuery\",e[e.TypeLiteral=166]=\"TypeLiteral\",e[e.ArrayType=167]=\"ArrayType\",e[e.TupleType=168]=\"TupleType\",e[e.OptionalType=169]=\"OptionalType\",e[e.RestType=170]=\"RestType\",e[e.UnionType=171]=\"UnionType\",e[e.IntersectionType=172]=\"IntersectionType\",e[e.ConditionalType=173]=\"ConditionalType\",e[e.InferType=174]=\"InferType\",e[e.ParenthesizedType=175]=\"ParenthesizedType\",e[e.ThisType=176]=\"ThisType\",e[e.TypeOperator=177]=\"TypeOperator\",e[e.IndexedAccessType=178]=\"IndexedAccessType\",e[e.MappedType=179]=\"MappedType\",e[e.LiteralType=180]=\"LiteralType\",e[e.ImportType=181]=\"ImportType\",e[e.ObjectBindingPattern=182]=\"ObjectBindingPattern\",e[e.ArrayBindingPattern=183]=\"ArrayBindingPattern\",e[e.BindingElement=184]=\"BindingElement\",e[e.ArrayLiteralExpression=185]=\"ArrayLiteralExpression\",e[e.ObjectLiteralExpression=186]=\"ObjectLiteralExpression\",e[e.PropertyAccessExpression=187]=\"PropertyAccessExpression\",e[e.ElementAccessExpression=188]=\"ElementAccessExpression\",e[e.CallExpression=189]=\"CallExpression\",e[e.NewExpression=190]=\"NewExpression\",e[e.TaggedTemplateExpression=191]=\"TaggedTemplateExpression\",e[e.TypeAssertionExpression=192]=\"TypeAssertionExpression\",e[e.ParenthesizedExpression=193]=\"ParenthesizedExpression\",e[e.FunctionExpression=194]=\"FunctionExpression\",e[e.ArrowFunction=195]=\"ArrowFunction\",e[e.DeleteExpression=196]=\"DeleteExpression\",e[e.TypeOfExpression=197]=\"TypeOfExpression\",e[e.VoidExpression=198]=\"VoidExpression\",e[e.AwaitExpression=199]=\"AwaitExpression\",e[e.PrefixUnaryExpression=200]=\"PrefixUnaryExpression\",e[e.PostfixUnaryExpression=201]=\"PostfixUnaryExpression\",e[e.BinaryExpression=202]=\"BinaryExpression\",e[e.ConditionalExpression=203]=\"ConditionalExpression\",e[e.TemplateExpression=204]=\"TemplateExpression\",e[e.YieldExpression=205]=\"YieldExpression\",e[e.SpreadElement=206]=\"SpreadElement\",e[e.ClassExpression=207]=\"ClassExpression\",e[e.OmittedExpression=208]=\"OmittedExpression\",e[e.ExpressionWithTypeArguments=209]=\"ExpressionWithTypeArguments\",e[e.AsExpression=210]=\"AsExpression\",e[e.NonNullExpression=211]=\"NonNullExpression\",e[e.MetaProperty=212]=\"MetaProperty\",e[e.SyntheticExpression=213]=\"SyntheticExpression\",e[e.TemplateSpan=214]=\"TemplateSpan\",e[e.SemicolonClassElement=215]=\"SemicolonClassElement\",e[e.Block=216]=\"Block\",e[e.VariableStatement=217]=\"VariableStatement\",e[e.EmptyStatement=218]=\"EmptyStatement\",e[e.ExpressionStatement=219]=\"ExpressionStatement\",e[e.IfStatement=220]=\"IfStatement\",e[e.DoStatement=221]=\"DoStatement\",e[e.WhileStatement=222]=\"WhileStatement\",e[e.ForStatement=223]=\"ForStatement\",e[e.ForInStatement=224]=\"ForInStatement\",e[e.ForOfStatement=225]=\"ForOfStatement\",e[e.ContinueStatement=226]=\"ContinueStatement\",e[e.BreakStatement=227]=\"BreakStatement\",e[e.ReturnStatement=228]=\"ReturnStatement\",e[e.WithStatement=229]=\"WithStatement\",e[e.SwitchStatement=230]=\"SwitchStatement\",e[e.LabeledStatement=231]=\"LabeledStatement\",e[e.ThrowStatement=232]=\"ThrowStatement\",e[e.TryStatement=233]=\"TryStatement\",e[e.DebuggerStatement=234]=\"DebuggerStatement\",e[e.VariableDeclaration=235]=\"VariableDeclaration\",e[e.VariableDeclarationList=236]=\"VariableDeclarationList\",e[e.FunctionDeclaration=237]=\"FunctionDeclaration\",e[e.ClassDeclaration=238]=\"ClassDeclaration\",e[e.InterfaceDeclaration=239]=\"InterfaceDeclaration\",e[e.TypeAliasDeclaration=240]=\"TypeAliasDeclaration\",e[e.EnumDeclaration=241]=\"EnumDeclaration\",e[e.ModuleDeclaration=242]=\"ModuleDeclaration\",e[e.ModuleBlock=243]=\"ModuleBlock\",e[e.CaseBlock=244]=\"CaseBlock\",e[e.NamespaceExportDeclaration=245]=\"NamespaceExportDeclaration\",e[e.ImportEqualsDeclaration=246]=\"ImportEqualsDeclaration\",e[e.ImportDeclaration=247]=\"ImportDeclaration\",e[e.ImportClause=248]=\"ImportClause\",e[e.NamespaceImport=249]=\"NamespaceImport\",e[e.NamedImports=250]=\"NamedImports\",e[e.ImportSpecifier=251]=\"ImportSpecifier\",e[e.ExportAssignment=252]=\"ExportAssignment\",e[e.ExportDeclaration=253]=\"ExportDeclaration\",e[e.NamedExports=254]=\"NamedExports\",e[e.ExportSpecifier=255]=\"ExportSpecifier\",e[e.MissingDeclaration=256]=\"MissingDeclaration\",e[e.ExternalModuleReference=257]=\"ExternalModuleReference\",e[e.JsxElement=258]=\"JsxElement\",e[e.JsxSelfClosingElement=259]=\"JsxSelfClosingElement\",e[e.JsxOpeningElement=260]=\"JsxOpeningElement\",e[e.JsxClosingElement=261]=\"JsxClosingElement\",e[e.JsxFragment=262]=\"JsxFragment\",e[e.JsxOpeningFragment=263]=\"JsxOpeningFragment\",e[e.JsxClosingFragment=264]=\"JsxClosingFragment\",e[e.JsxAttribute=265]=\"JsxAttribute\",e[e.JsxAttributes=266]=\"JsxAttributes\",e[e.JsxSpreadAttribute=267]=\"JsxSpreadAttribute\",e[e.JsxExpression=268]=\"JsxExpression\",e[e.CaseClause=269]=\"CaseClause\",e[e.DefaultClause=270]=\"DefaultClause\",e[e.HeritageClause=271]=\"HeritageClause\",e[e.CatchClause=272]=\"CatchClause\",e[e.PropertyAssignment=273]=\"PropertyAssignment\",e[e.ShorthandPropertyAssignment=274]=\"ShorthandPropertyAssignment\",e[e.SpreadAssignment=275]=\"SpreadAssignment\",e[e.EnumMember=276]=\"EnumMember\",e[e.SourceFile=277]=\"SourceFile\",e[e.Bundle=278]=\"Bundle\",e[e.UnparsedSource=279]=\"UnparsedSource\",e[e.InputFiles=280]=\"InputFiles\",e[e.JSDocTypeExpression=281]=\"JSDocTypeExpression\",e[e.JSDocAllType=282]=\"JSDocAllType\",e[e.JSDocUnknownType=283]=\"JSDocUnknownType\",e[e.JSDocNullableType=284]=\"JSDocNullableType\",e[e.JSDocNonNullableType=285]=\"JSDocNonNullableType\",e[e.JSDocOptionalType=286]=\"JSDocOptionalType\",e[e.JSDocFunctionType=287]=\"JSDocFunctionType\",e[e.JSDocVariadicType=288]=\"JSDocVariadicType\",e[e.JSDocComment=289]=\"JSDocComment\",e[e.JSDocTypeLiteral=290]=\"JSDocTypeLiteral\",e[e.JSDocSignature=291]=\"JSDocSignature\",e[e.JSDocTag=292]=\"JSDocTag\",e[e.JSDocAugmentsTag=293]=\"JSDocAugmentsTag\",e[e.JSDocClassTag=294]=\"JSDocClassTag\",e[e.JSDocCallbackTag=295]=\"JSDocCallbackTag\",e[e.JSDocEnumTag=296]=\"JSDocEnumTag\",e[e.JSDocParameterTag=297]=\"JSDocParameterTag\",e[e.JSDocReturnTag=298]=\"JSDocReturnTag\",e[e.JSDocThisTag=299]=\"JSDocThisTag\",e[e.JSDocTypeTag=300]=\"JSDocTypeTag\",e[e.JSDocTemplateTag=301]=\"JSDocTemplateTag\",e[e.JSDocTypedefTag=302]=\"JSDocTypedefTag\",e[e.JSDocPropertyTag=303]=\"JSDocPropertyTag\",e[e.SyntaxList=304]=\"SyntaxList\",e[e.NotEmittedStatement=305]=\"NotEmittedStatement\",e[e.PartiallyEmittedExpression=306]=\"PartiallyEmittedExpression\",e[e.CommaListExpression=307]=\"CommaListExpression\",e[e.MergeDeclarationMarker=308]=\"MergeDeclarationMarker\",e[e.EndOfDeclarationMarker=309]=\"EndOfDeclarationMarker\",e[e.Count=310]=\"Count\",e[e.FirstAssignment=58]=\"FirstAssignment\",e[e.LastAssignment=70]=\"LastAssignment\",e[e.FirstCompoundAssignment=59]=\"FirstCompoundAssignment\",e[e.LastCompoundAssignment=70]=\"LastCompoundAssignment\",e[e.FirstReservedWord=72]=\"FirstReservedWord\",e[e.LastReservedWord=107]=\"LastReservedWord\",e[e.FirstKeyword=72]=\"FirstKeyword\",e[e.LastKeyword=145]=\"LastKeyword\",e[e.FirstFutureReservedWord=108]=\"FirstFutureReservedWord\",e[e.LastFutureReservedWord=116]=\"LastFutureReservedWord\",e[e.FirstTypeNode=161]=\"FirstTypeNode\",e[e.LastTypeNode=181]=\"LastTypeNode\",e[e.FirstPunctuation=17]=\"FirstPunctuation\",e[e.LastPunctuation=70]=\"LastPunctuation\",e[e.FirstToken=0]=\"FirstToken\",e[e.LastToken=145]=\"LastToken\",e[e.FirstTriviaToken=2]=\"FirstTriviaToken\",e[e.LastTriviaToken=7]=\"LastTriviaToken\",e[e.FirstLiteralToken=8]=\"FirstLiteralToken\",e[e.LastLiteralToken=13]=\"LastLiteralToken\",e[e.FirstTemplateToken=13]=\"FirstTemplateToken\",e[e.LastTemplateToken=16]=\"LastTemplateToken\",e[e.FirstBinaryOperator=27]=\"FirstBinaryOperator\",e[e.LastBinaryOperator=70]=\"LastBinaryOperator\",e[e.FirstNode=146]=\"FirstNode\",e[e.FirstJSDocNode=281]=\"FirstJSDocNode\",e[e.LastJSDocNode=303]=\"LastJSDocNode\",e[e.FirstJSDocTagNode=292]=\"FirstJSDocTagNode\",e[e.LastJSDocTagNode=303]=\"LastJSDocTagNode\",e[e.FirstContextualKeyword=117]=\"FirstContextualKeyword\",e[e.LastContextualKeyword=145]=\"LastContextualKeyword\"}(e.SyntaxKind||(e.SyntaxKind={})),function(e){e[e.None=0]=\"None\",e[e.Let=1]=\"Let\",e[e.Const=2]=\"Const\",e[e.NestedNamespace=4]=\"NestedNamespace\",e[e.Synthesized=8]=\"Synthesized\",e[e.Namespace=16]=\"Namespace\",e[e.ExportContext=32]=\"ExportContext\",e[e.ContainsThis=64]=\"ContainsThis\",e[e.HasImplicitReturn=128]=\"HasImplicitReturn\",e[e.HasExplicitReturn=256]=\"HasExplicitReturn\",e[e.GlobalAugmentation=512]=\"GlobalAugmentation\",e[e.HasAsyncFunctions=1024]=\"HasAsyncFunctions\",e[e.DisallowInContext=2048]=\"DisallowInContext\",e[e.YieldContext=4096]=\"YieldContext\",e[e.DecoratorContext=8192]=\"DecoratorContext\",e[e.AwaitContext=16384]=\"AwaitContext\",e[e.ThisNodeHasError=32768]=\"ThisNodeHasError\",e[e.JavaScriptFile=65536]=\"JavaScriptFile\",e[e.ThisNodeOrAnySubNodesHasError=131072]=\"ThisNodeOrAnySubNodesHasError\",e[e.HasAggregatedChildData=262144]=\"HasAggregatedChildData\",e[e.PossiblyContainsDynamicImport=524288]=\"PossiblyContainsDynamicImport\",e[e.PossiblyContainsImportMeta=1048576]=\"PossiblyContainsImportMeta\",e[e.JSDoc=2097152]=\"JSDoc\",e[e.Ambient=4194304]=\"Ambient\",e[e.InWithStatement=8388608]=\"InWithStatement\",e[e.JsonFile=16777216]=\"JsonFile\",e[e.BlockScoped=3]=\"BlockScoped\",e[e.ReachabilityCheckFlags=384]=\"ReachabilityCheckFlags\",e[e.ReachabilityAndEmitFlags=1408]=\"ReachabilityAndEmitFlags\",e[e.ContextFlags=12679168]=\"ContextFlags\",e[e.TypeExcludesFlags=20480]=\"TypeExcludesFlags\",e[e.PermanentlySetIncrementalFlags=1572864]=\"PermanentlySetIncrementalFlags\"}(e.NodeFlags||(e.NodeFlags={})),function(e){e[e.None=0]=\"None\",e[e.Export=1]=\"Export\",e[e.Ambient=2]=\"Ambient\",e[e.Public=4]=\"Public\",e[e.Private=8]=\"Private\",e[e.Protected=16]=\"Protected\",e[e.Static=32]=\"Static\",e[e.Readonly=64]=\"Readonly\",e[e.Abstract=128]=\"Abstract\",e[e.Async=256]=\"Async\",e[e.Default=512]=\"Default\",e[e.Const=2048]=\"Const\",e[e.HasComputedFlags=536870912]=\"HasComputedFlags\",e[e.AccessibilityModifier=28]=\"AccessibilityModifier\",e[e.ParameterPropertyModifier=92]=\"ParameterPropertyModifier\",e[e.NonPublicAccessibilityModifier=24]=\"NonPublicAccessibilityModifier\",e[e.TypeScriptModifier=2270]=\"TypeScriptModifier\",e[e.ExportDefault=513]=\"ExportDefault\",e[e.All=3071]=\"All\"}(e.ModifierFlags||(e.ModifierFlags={})),function(e){e[e.None=0]=\"None\",e[e.IntrinsicNamedElement=1]=\"IntrinsicNamedElement\",e[e.IntrinsicIndexedElement=2]=\"IntrinsicIndexedElement\",e[e.IntrinsicElement=3]=\"IntrinsicElement\"}(e.JsxFlags||(e.JsxFlags={})),function(e){e[e.Succeeded=1]=\"Succeeded\",e[e.Failed=2]=\"Failed\",e[e.FailedAndReported=3]=\"FailedAndReported\"}(e.RelationComparisonResult||(e.RelationComparisonResult={})),function(e){e[e.None=0]=\"None\",e[e.Auto=1]=\"Auto\",e[e.Loop=2]=\"Loop\",e[e.Unique=3]=\"Unique\",e[e.Node=4]=\"Node\",e[e.KindMask=7]=\"KindMask\",e[e.ReservedInNestedScopes=8]=\"ReservedInNestedScopes\",e[e.Optimistic=16]=\"Optimistic\",e[e.FileLevel=32]=\"FileLevel\"}(e.GeneratedIdentifierFlags||(e.GeneratedIdentifierFlags={})),function(e){e[e.None=0]=\"None\",e[e.PrecedingLineBreak=1]=\"PrecedingLineBreak\",e[e.PrecedingJSDocComment=2]=\"PrecedingJSDocComment\",e[e.Unterminated=4]=\"Unterminated\",e[e.ExtendedUnicodeEscape=8]=\"ExtendedUnicodeEscape\",e[e.Scientific=16]=\"Scientific\",e[e.Octal=32]=\"Octal\",e[e.HexSpecifier=64]=\"HexSpecifier\",e[e.BinarySpecifier=128]=\"BinarySpecifier\",e[e.OctalSpecifier=256]=\"OctalSpecifier\",e[e.ContainsSeparator=512]=\"ContainsSeparator\",e[e.BinaryOrOctalSpecifier=384]=\"BinaryOrOctalSpecifier\",e[e.NumericLiteralFlags=1008]=\"NumericLiteralFlags\"}(e.TokenFlags||(e.TokenFlags={})),function(e){e[e.Unreachable=1]=\"Unreachable\",e[e.Start=2]=\"Start\",e[e.BranchLabel=4]=\"BranchLabel\",e[e.LoopLabel=8]=\"LoopLabel\",e[e.Assignment=16]=\"Assignment\",e[e.TrueCondition=32]=\"TrueCondition\",e[e.FalseCondition=64]=\"FalseCondition\",e[e.SwitchClause=128]=\"SwitchClause\",e[e.ArrayMutation=256]=\"ArrayMutation\",e[e.Referenced=512]=\"Referenced\",e[e.Shared=1024]=\"Shared\",e[e.PreFinally=2048]=\"PreFinally\",e[e.AfterFinally=4096]=\"AfterFinally\",e[e.Label=12]=\"Label\",e[e.Condition=96]=\"Condition\"}(e.FlowFlags||(e.FlowFlags={}));var n,t=function(){return function(){}}();e.OperationCanceledException=t,function(e){e[e.Not=0]=\"Not\",e[e.SafeModules=1]=\"SafeModules\",e[e.Completely=2]=\"Completely\"}(e.StructureIsReused||(e.StructureIsReused={})),function(e){e[e.Success=0]=\"Success\",e[e.DiagnosticsPresent_OutputsSkipped=1]=\"DiagnosticsPresent_OutputsSkipped\",e[e.DiagnosticsPresent_OutputsGenerated=2]=\"DiagnosticsPresent_OutputsGenerated\"}(e.ExitStatus||(e.ExitStatus={})),function(e){e[e.None=0]=\"None\",e[e.Literal=1]=\"Literal\",e[e.Subtype=2]=\"Subtype\"}(e.UnionReduction||(e.UnionReduction={})),function(e){e[e.None=0]=\"None\",e[e.NoTruncation=1]=\"NoTruncation\",e[e.WriteArrayAsGenericType=2]=\"WriteArrayAsGenericType\",e[e.GenerateNamesForShadowedTypeParams=4]=\"GenerateNamesForShadowedTypeParams\",e[e.UseStructuralFallback=8]=\"UseStructuralFallback\",e[e.ForbidIndexedAccessSymbolReferences=16]=\"ForbidIndexedAccessSymbolReferences\",e[e.WriteTypeArgumentsOfSignature=32]=\"WriteTypeArgumentsOfSignature\",e[e.UseFullyQualifiedType=64]=\"UseFullyQualifiedType\",e[e.UseOnlyExternalAliasing=128]=\"UseOnlyExternalAliasing\",e[e.SuppressAnyReturnType=256]=\"SuppressAnyReturnType\",e[e.WriteTypeParametersInQualifiedName=512]=\"WriteTypeParametersInQualifiedName\",e[e.MultilineObjectLiterals=1024]=\"MultilineObjectLiterals\",e[e.WriteClassExpressionAsTypeLiteral=2048]=\"WriteClassExpressionAsTypeLiteral\",e[e.UseTypeOfFunction=4096]=\"UseTypeOfFunction\",e[e.OmitParameterModifiers=8192]=\"OmitParameterModifiers\",e[e.UseAliasDefinedOutsideCurrentScope=16384]=\"UseAliasDefinedOutsideCurrentScope\",e[e.AllowThisInObjectLiteral=32768]=\"AllowThisInObjectLiteral\",e[e.AllowQualifedNameInPlaceOfIdentifier=65536]=\"AllowQualifedNameInPlaceOfIdentifier\",e[e.AllowAnonymousIdentifier=131072]=\"AllowAnonymousIdentifier\",e[e.AllowEmptyUnionOrIntersection=262144]=\"AllowEmptyUnionOrIntersection\",e[e.AllowEmptyTuple=524288]=\"AllowEmptyTuple\",e[e.AllowUniqueESSymbolType=1048576]=\"AllowUniqueESSymbolType\",e[e.AllowEmptyIndexInfoType=2097152]=\"AllowEmptyIndexInfoType\",e[e.IgnoreErrors=3112960]=\"IgnoreErrors\",e[e.InObjectTypeLiteral=4194304]=\"InObjectTypeLiteral\",e[e.InTypeAlias=8388608]=\"InTypeAlias\",e[e.InInitialEntityName=16777216]=\"InInitialEntityName\",e[e.InReverseMappedType=33554432]=\"InReverseMappedType\"}(e.NodeBuilderFlags||(e.NodeBuilderFlags={})),function(e){e[e.None=0]=\"None\",e[e.NoTruncation=1]=\"NoTruncation\",e[e.WriteArrayAsGenericType=2]=\"WriteArrayAsGenericType\",e[e.UseStructuralFallback=8]=\"UseStructuralFallback\",e[e.WriteTypeArgumentsOfSignature=32]=\"WriteTypeArgumentsOfSignature\",e[e.UseFullyQualifiedType=64]=\"UseFullyQualifiedType\",e[e.SuppressAnyReturnType=256]=\"SuppressAnyReturnType\",e[e.MultilineObjectLiterals=1024]=\"MultilineObjectLiterals\",e[e.WriteClassExpressionAsTypeLiteral=2048]=\"WriteClassExpressionAsTypeLiteral\",e[e.UseTypeOfFunction=4096]=\"UseTypeOfFunction\",e[e.OmitParameterModifiers=8192]=\"OmitParameterModifiers\",e[e.UseAliasDefinedOutsideCurrentScope=16384]=\"UseAliasDefinedOutsideCurrentScope\",e[e.AllowUniqueESSymbolType=1048576]=\"AllowUniqueESSymbolType\",e[e.AddUndefined=131072]=\"AddUndefined\",e[e.WriteArrowStyleSignature=262144]=\"WriteArrowStyleSignature\",e[e.InArrayType=524288]=\"InArrayType\",e[e.InElementType=2097152]=\"InElementType\",e[e.InFirstTypeArgument=4194304]=\"InFirstTypeArgument\",e[e.InTypeAlias=8388608]=\"InTypeAlias\",e[e.WriteOwnNameForAnyLike=0]=\"WriteOwnNameForAnyLike\",e[e.NodeBuilderFlagsMask=9469291]=\"NodeBuilderFlagsMask\"}(e.TypeFormatFlags||(e.TypeFormatFlags={})),function(e){e[e.None=0]=\"None\",e[e.WriteTypeParametersOrArguments=1]=\"WriteTypeParametersOrArguments\",e[e.UseOnlyExternalAliasing=2]=\"UseOnlyExternalAliasing\",e[e.AllowAnyNodeKind=4]=\"AllowAnyNodeKind\",e[e.UseAliasDefinedOutsideCurrentScope=8]=\"UseAliasDefinedOutsideCurrentScope\"}(e.SymbolFormatFlags||(e.SymbolFormatFlags={})),function(e){e[e.Accessible=0]=\"Accessible\",e[e.NotAccessible=1]=\"NotAccessible\",e[e.CannotBeNamed=2]=\"CannotBeNamed\"}(e.SymbolAccessibility||(e.SymbolAccessibility={})),function(e){e[e.UnionOrIntersection=0]=\"UnionOrIntersection\",e[e.Spread=1]=\"Spread\"}(e.SyntheticSymbolKind||(e.SyntheticSymbolKind={})),function(e){e[e.This=0]=\"This\",e[e.Identifier=1]=\"Identifier\"}(e.TypePredicateKind||(e.TypePredicateKind={})),function(e){e[e.Unknown=0]=\"Unknown\",e[e.TypeWithConstructSignatureAndValue=1]=\"TypeWithConstructSignatureAndValue\",e[e.VoidNullableOrNeverType=2]=\"VoidNullableOrNeverType\",e[e.NumberLikeType=3]=\"NumberLikeType\",e[e.StringLikeType=4]=\"StringLikeType\",e[e.BooleanType=5]=\"BooleanType\",e[e.ArrayLikeType=6]=\"ArrayLikeType\",e[e.ESSymbolType=7]=\"ESSymbolType\",e[e.Promise=8]=\"Promise\",e[e.TypeWithCallSignature=9]=\"TypeWithCallSignature\",e[e.ObjectType=10]=\"ObjectType\"}(e.TypeReferenceSerializationKind||(e.TypeReferenceSerializationKind={})),function(e){e[e.None=0]=\"None\",e[e.FunctionScopedVariable=1]=\"FunctionScopedVariable\",e[e.BlockScopedVariable=2]=\"BlockScopedVariable\",e[e.Property=4]=\"Property\",e[e.EnumMember=8]=\"EnumMember\",e[e.Function=16]=\"Function\",e[e.Class=32]=\"Class\",e[e.Interface=64]=\"Interface\",e[e.ConstEnum=128]=\"ConstEnum\",e[e.RegularEnum=256]=\"RegularEnum\",e[e.ValueModule=512]=\"ValueModule\",e[e.NamespaceModule=1024]=\"NamespaceModule\",e[e.TypeLiteral=2048]=\"TypeLiteral\",e[e.ObjectLiteral=4096]=\"ObjectLiteral\",e[e.Method=8192]=\"Method\",e[e.Constructor=16384]=\"Constructor\",e[e.GetAccessor=32768]=\"GetAccessor\",e[e.SetAccessor=65536]=\"SetAccessor\",e[e.Signature=131072]=\"Signature\",e[e.TypeParameter=262144]=\"TypeParameter\",e[e.TypeAlias=524288]=\"TypeAlias\",e[e.ExportValue=1048576]=\"ExportValue\",e[e.Alias=2097152]=\"Alias\",e[e.Prototype=4194304]=\"Prototype\",e[e.ExportStar=8388608]=\"ExportStar\",e[e.Optional=16777216]=\"Optional\",e[e.Transient=33554432]=\"Transient\",e[e.Assignment=67108864]=\"Assignment\",e[e.ModuleExports=134217728]=\"ModuleExports\",e[e.All=67108863]=\"All\",e[e.Enum=384]=\"Enum\",e[e.Variable=3]=\"Variable\",e[e.Value=67220415]=\"Value\",e[e.Type=67897832]=\"Type\",e[e.Namespace=1920]=\"Namespace\",e[e.Module=1536]=\"Module\",e[e.Accessor=98304]=\"Accessor\",e[e.FunctionScopedVariableExcludes=67220414]=\"FunctionScopedVariableExcludes\",e[e.BlockScopedVariableExcludes=67220415]=\"BlockScopedVariableExcludes\",e[e.ParameterExcludes=67220415]=\"ParameterExcludes\",e[e.PropertyExcludes=0]=\"PropertyExcludes\",e[e.EnumMemberExcludes=68008959]=\"EnumMemberExcludes\",e[e.FunctionExcludes=67219887]=\"FunctionExcludes\",e[e.ClassExcludes=68008383]=\"ClassExcludes\",e[e.InterfaceExcludes=67897736]=\"InterfaceExcludes\",e[e.RegularEnumExcludes=68008191]=\"RegularEnumExcludes\",e[e.ConstEnumExcludes=68008831]=\"ConstEnumExcludes\",e[e.ValueModuleExcludes=110735]=\"ValueModuleExcludes\",e[e.NamespaceModuleExcludes=0]=\"NamespaceModuleExcludes\",e[e.MethodExcludes=67212223]=\"MethodExcludes\",e[e.GetAccessorExcludes=67154879]=\"GetAccessorExcludes\",e[e.SetAccessorExcludes=67187647]=\"SetAccessorExcludes\",e[e.TypeParameterExcludes=67635688]=\"TypeParameterExcludes\",e[e.TypeAliasExcludes=67897832]=\"TypeAliasExcludes\",e[e.AliasExcludes=2097152]=\"AliasExcludes\",e[e.ModuleMember=2623475]=\"ModuleMember\",e[e.ExportHasLocal=944]=\"ExportHasLocal\",e[e.BlockScoped=418]=\"BlockScoped\",e[e.PropertyOrAccessor=98308]=\"PropertyOrAccessor\",e[e.ClassMember=106500]=\"ClassMember\",e[e.Classifiable=2885600]=\"Classifiable\",e[e.LateBindingContainer=6240]=\"LateBindingContainer\"}(e.SymbolFlags||(e.SymbolFlags={})),function(e){e[e.Numeric=0]=\"Numeric\",e[e.Literal=1]=\"Literal\"}(e.EnumKind||(e.EnumKind={})),function(e){e[e.Instantiated=1]=\"Instantiated\",e[e.SyntheticProperty=2]=\"SyntheticProperty\",e[e.SyntheticMethod=4]=\"SyntheticMethod\",e[e.Readonly=8]=\"Readonly\",e[e.Partial=16]=\"Partial\",e[e.HasNonUniformType=32]=\"HasNonUniformType\",e[e.ContainsPublic=64]=\"ContainsPublic\",e[e.ContainsProtected=128]=\"ContainsProtected\",e[e.ContainsPrivate=256]=\"ContainsPrivate\",e[e.ContainsStatic=512]=\"ContainsStatic\",e[e.Late=1024]=\"Late\",e[e.ReverseMapped=2048]=\"ReverseMapped\",e[e.OptionalParameter=4096]=\"OptionalParameter\",e[e.RestParameter=8192]=\"RestParameter\",e[e.Synthetic=6]=\"Synthetic\"}(e.CheckFlags||(e.CheckFlags={})),function(e){e.Call=\"__call\",e.Constructor=\"__constructor\",e.New=\"__new\",e.Index=\"__index\",e.ExportStar=\"__export\",e.Global=\"__global\",e.Missing=\"__missing\",e.Type=\"__type\",e.Object=\"__object\",e.JSXAttributes=\"__jsxAttributes\",e.Class=\"__class\",e.Function=\"__function\",e.Computed=\"__computed\",e.Resolving=\"__resolving__\",e.ExportEquals=\"export=\",e.Default=\"default\",e.This=\"this\"}(e.InternalSymbolName||(e.InternalSymbolName={})),function(e){e[e.TypeChecked=1]=\"TypeChecked\",e[e.LexicalThis=2]=\"LexicalThis\",e[e.CaptureThis=4]=\"CaptureThis\",e[e.CaptureNewTarget=8]=\"CaptureNewTarget\",e[e.SuperInstance=256]=\"SuperInstance\",e[e.SuperStatic=512]=\"SuperStatic\",e[e.ContextChecked=1024]=\"ContextChecked\",e[e.AsyncMethodWithSuper=2048]=\"AsyncMethodWithSuper\",e[e.AsyncMethodWithSuperBinding=4096]=\"AsyncMethodWithSuperBinding\",e[e.CaptureArguments=8192]=\"CaptureArguments\",e[e.EnumValuesComputed=16384]=\"EnumValuesComputed\",e[e.LexicalModuleMergesWithClass=32768]=\"LexicalModuleMergesWithClass\",e[e.LoopWithCapturedBlockScopedBinding=65536]=\"LoopWithCapturedBlockScopedBinding\",e[e.CapturedBlockScopedBinding=131072]=\"CapturedBlockScopedBinding\",e[e.BlockScopedBindingInLoop=262144]=\"BlockScopedBindingInLoop\",e[e.ClassWithBodyScopedClassBinding=524288]=\"ClassWithBodyScopedClassBinding\",e[e.BodyScopedClassBinding=1048576]=\"BodyScopedClassBinding\",e[e.NeedsLoopOutParameter=2097152]=\"NeedsLoopOutParameter\",e[e.AssignmentsMarked=4194304]=\"AssignmentsMarked\",e[e.ClassWithConstructorReference=8388608]=\"ClassWithConstructorReference\",e[e.ConstructorReferenceInClass=16777216]=\"ConstructorReferenceInClass\"}(e.NodeCheckFlags||(e.NodeCheckFlags={})),function(e){e[e.Any=1]=\"Any\",e[e.Unknown=2]=\"Unknown\",e[e.String=4]=\"String\",e[e.Number=8]=\"Number\",e[e.Boolean=16]=\"Boolean\",e[e.Enum=32]=\"Enum\",e[e.StringLiteral=64]=\"StringLiteral\",e[e.NumberLiteral=128]=\"NumberLiteral\",e[e.BooleanLiteral=256]=\"BooleanLiteral\",e[e.EnumLiteral=512]=\"EnumLiteral\",e[e.ESSymbol=1024]=\"ESSymbol\",e[e.UniqueESSymbol=2048]=\"UniqueESSymbol\",e[e.Void=4096]=\"Void\",e[e.Undefined=8192]=\"Undefined\",e[e.Null=16384]=\"Null\",e[e.Never=32768]=\"Never\",e[e.TypeParameter=65536]=\"TypeParameter\",e[e.Object=131072]=\"Object\",e[e.Union=262144]=\"Union\",e[e.Intersection=524288]=\"Intersection\",e[e.Index=1048576]=\"Index\",e[e.IndexedAccess=2097152]=\"IndexedAccess\",e[e.Conditional=4194304]=\"Conditional\",e[e.Substitution=8388608]=\"Substitution\",e[e.NonPrimitive=16777216]=\"NonPrimitive\",e[e.FreshLiteral=33554432]=\"FreshLiteral\",e[e.UnionOfPrimitiveTypes=67108864]=\"UnionOfPrimitiveTypes\",e[e.ContainsWideningType=134217728]=\"ContainsWideningType\",e[e.ContainsObjectLiteral=268435456]=\"ContainsObjectLiteral\",e[e.ContainsAnyFunctionType=536870912]=\"ContainsAnyFunctionType\",e[e.AnyOrUnknown=3]=\"AnyOrUnknown\",e[e.Nullable=24576]=\"Nullable\",e[e.Literal=448]=\"Literal\",e[e.Unit=27072]=\"Unit\",e[e.StringOrNumberLiteral=192]=\"StringOrNumberLiteral\",e[e.StringOrNumberLiteralOrUnique=2240]=\"StringOrNumberLiteralOrUnique\",e[e.DefinitelyFalsy=29120]=\"DefinitelyFalsy\",e[e.PossiblyFalsy=29148]=\"PossiblyFalsy\",e[e.Intrinsic=16839967]=\"Intrinsic\",e[e.Primitive=32764]=\"Primitive\",e[e.StringLike=68]=\"StringLike\",e[e.NumberLike=168]=\"NumberLike\",e[e.BooleanLike=272]=\"BooleanLike\",e[e.EnumLike=544]=\"EnumLike\",e[e.ESSymbolLike=3072]=\"ESSymbolLike\",e[e.VoidLike=12288]=\"VoidLike\",e[e.DisjointDomains=16809468]=\"DisjointDomains\",e[e.UnionOrIntersection=786432]=\"UnionOrIntersection\",e[e.StructuredType=917504]=\"StructuredType\",e[e.TypeVariable=2162688]=\"TypeVariable\",e[e.InstantiableNonPrimitive=14745600]=\"InstantiableNonPrimitive\",e[e.InstantiablePrimitive=1048576]=\"InstantiablePrimitive\",e[e.Instantiable=15794176]=\"Instantiable\",e[e.StructuredOrInstantiable=16711680]=\"StructuredOrInstantiable\",e[e.Narrowable=33492479]=\"Narrowable\",e[e.NotUnionOrUnit=16909315]=\"NotUnionOrUnit\",e[e.NotPrimitiveUnion=16748579]=\"NotPrimitiveUnion\",e[e.RequiresWidening=402653184]=\"RequiresWidening\",e[e.PropagatingFlags=939524096]=\"PropagatingFlags\",e[e.NonWideningType=134217728]=\"NonWideningType\",e[e.Wildcard=268435456]=\"Wildcard\",e[e.EmptyObject=536870912]=\"EmptyObject\",e[e.ConstructionFlags=939524096]=\"ConstructionFlags\",e[e.GenericMappedType=134217728]=\"GenericMappedType\"}(e.TypeFlags||(e.TypeFlags={})),function(e){e[e.Class=1]=\"Class\",e[e.Interface=2]=\"Interface\",e[e.Reference=4]=\"Reference\",e[e.Tuple=8]=\"Tuple\",e[e.Anonymous=16]=\"Anonymous\",e[e.Mapped=32]=\"Mapped\",e[e.Instantiated=64]=\"Instantiated\",e[e.ObjectLiteral=128]=\"ObjectLiteral\",e[e.EvolvingArray=256]=\"EvolvingArray\",e[e.ObjectLiteralPatternWithComputedProperties=512]=\"ObjectLiteralPatternWithComputedProperties\",e[e.ContainsSpread=1024]=\"ContainsSpread\",e[e.ReverseMapped=2048]=\"ReverseMapped\",e[e.JsxAttributes=4096]=\"JsxAttributes\",e[e.MarkerType=8192]=\"MarkerType\",e[e.JSLiteral=16384]=\"JSLiteral\",e[e.ClassOrInterface=3]=\"ClassOrInterface\"}(e.ObjectFlags||(e.ObjectFlags={})),function(e){e[e.Invariant=0]=\"Invariant\",e[e.Covariant=1]=\"Covariant\",e[e.Contravariant=2]=\"Contravariant\",e[e.Bivariant=3]=\"Bivariant\",e[e.Independent=4]=\"Independent\"}(e.Variance||(e.Variance={})),function(e){e[e.Call=0]=\"Call\",e[e.Construct=1]=\"Construct\"}(e.SignatureKind||(e.SignatureKind={})),function(e){e[e.String=0]=\"String\",e[e.Number=1]=\"Number\"}(e.IndexKind||(e.IndexKind={})),function(e){e[e.NakedTypeVariable=1]=\"NakedTypeVariable\",e[e.HomomorphicMappedType=2]=\"HomomorphicMappedType\",e[e.MappedTypeConstraint=4]=\"MappedTypeConstraint\",e[e.ReturnType=8]=\"ReturnType\",e[e.LiteralKeyof=16]=\"LiteralKeyof\",e[e.NoConstraints=32]=\"NoConstraints\",e[e.AlwaysStrict=64]=\"AlwaysStrict\",e[e.PriorityImpliesCombination=28]=\"PriorityImpliesCombination\"}(e.InferencePriority||(e.InferencePriority={})),function(e){e[e.None=0]=\"None\",e[e.NoDefault=1]=\"NoDefault\",e[e.AnyDefault=2]=\"AnyDefault\"}(e.InferenceFlags||(e.InferenceFlags={})),function(e){e[e.False=0]=\"False\",e[e.Maybe=1]=\"Maybe\",e[e.True=-1]=\"True\"}(e.Ternary||(e.Ternary={})),function(e){e[e.None=0]=\"None\",e[e.ExportsProperty=1]=\"ExportsProperty\",e[e.ModuleExports=2]=\"ModuleExports\",e[e.PrototypeProperty=3]=\"PrototypeProperty\",e[e.ThisProperty=4]=\"ThisProperty\",e[e.Property=5]=\"Property\",e[e.Prototype=6]=\"Prototype\"}(e.AssignmentDeclarationKind||(e.AssignmentDeclarationKind={})),function(e){e[e.Warning=0]=\"Warning\",e[e.Error=1]=\"Error\",e[e.Suggestion=2]=\"Suggestion\",e[e.Message=3]=\"Message\"}(n=e.DiagnosticCategory||(e.DiagnosticCategory={})),e.diagnosticCategoryName=function(e,t){void 0===t&&(t=!0);var r=n[e.category];return t?r.toLowerCase():r},function(e){e[e.Classic=1]=\"Classic\",e[e.NodeJs=2]=\"NodeJs\"}(e.ModuleResolutionKind||(e.ModuleResolutionKind={})),function(e){e[e.None=0]=\"None\",e[e.CommonJS=1]=\"CommonJS\",e[e.AMD=2]=\"AMD\",e[e.UMD=3]=\"UMD\",e[e.System=4]=\"System\",e[e.ES2015=5]=\"ES2015\",e[e.ESNext=6]=\"ESNext\"}(e.ModuleKind||(e.ModuleKind={})),function(e){e[e.None=0]=\"None\",e[e.Preserve=1]=\"Preserve\",e[e.React=2]=\"React\",e[e.ReactNative=3]=\"ReactNative\"}(e.JsxEmit||(e.JsxEmit={})),function(e){e[e.CarriageReturnLineFeed=0]=\"CarriageReturnLineFeed\",e[e.LineFeed=1]=\"LineFeed\"}(e.NewLineKind||(e.NewLineKind={})),function(e){e[e.Unknown=0]=\"Unknown\",e[e.JS=1]=\"JS\",e[e.JSX=2]=\"JSX\",e[e.TS=3]=\"TS\",e[e.TSX=4]=\"TSX\",e[e.External=5]=\"External\",e[e.JSON=6]=\"JSON\",e[e.Deferred=7]=\"Deferred\"}(e.ScriptKind||(e.ScriptKind={})),function(e){e[e.ES3=0]=\"ES3\",e[e.ES5=1]=\"ES5\",e[e.ES2015=2]=\"ES2015\",e[e.ES2016=3]=\"ES2016\",e[e.ES2017=4]=\"ES2017\",e[e.ES2018=5]=\"ES2018\",e[e.ESNext=6]=\"ESNext\",e[e.JSON=100]=\"JSON\",e[e.Latest=6]=\"Latest\"}(e.ScriptTarget||(e.ScriptTarget={})),function(e){e[e.Standard=0]=\"Standard\",e[e.JSX=1]=\"JSX\"}(e.LanguageVariant||(e.LanguageVariant={})),function(e){e[e.None=0]=\"None\",e[e.Recursive=1]=\"Recursive\"}(e.WatchDirectoryFlags||(e.WatchDirectoryFlags={})),function(e){e[e.nullCharacter=0]=\"nullCharacter\",e[e.maxAsciiCharacter=127]=\"maxAsciiCharacter\",e[e.lineFeed=10]=\"lineFeed\",e[e.carriageReturn=13]=\"carriageReturn\",e[e.lineSeparator=8232]=\"lineSeparator\",e[e.paragraphSeparator=8233]=\"paragraphSeparator\",e[e.nextLine=133]=\"nextLine\",e[e.space=32]=\"space\",e[e.nonBreakingSpace=160]=\"nonBreakingSpace\",e[e.enQuad=8192]=\"enQuad\",e[e.emQuad=8193]=\"emQuad\",e[e.enSpace=8194]=\"enSpace\",e[e.emSpace=8195]=\"emSpace\",e[e.threePerEmSpace=8196]=\"threePerEmSpace\",e[e.fourPerEmSpace=8197]=\"fourPerEmSpace\",e[e.sixPerEmSpace=8198]=\"sixPerEmSpace\",e[e.figureSpace=8199]=\"figureSpace\",e[e.punctuationSpace=8200]=\"punctuationSpace\",e[e.thinSpace=8201]=\"thinSpace\",e[e.hairSpace=8202]=\"hairSpace\",e[e.zeroWidthSpace=8203]=\"zeroWidthSpace\",e[e.narrowNoBreakSpace=8239]=\"narrowNoBreakSpace\",e[e.ideographicSpace=12288]=\"ideographicSpace\",e[e.mathematicalSpace=8287]=\"mathematicalSpace\",e[e.ogham=5760]=\"ogham\",e[e._=95]=\"_\",e[e.$=36]=\"$\",e[e._0=48]=\"_0\",e[e._1=49]=\"_1\",e[e._2=50]=\"_2\",e[e._3=51]=\"_3\",e[e._4=52]=\"_4\",e[e._5=53]=\"_5\",e[e._6=54]=\"_6\",e[e._7=55]=\"_7\",e[e._8=56]=\"_8\",e[e._9=57]=\"_9\",e[e.a=97]=\"a\",e[e.b=98]=\"b\",e[e.c=99]=\"c\",e[e.d=100]=\"d\",e[e.e=101]=\"e\",e[e.f=102]=\"f\",e[e.g=103]=\"g\",e[e.h=104]=\"h\",e[e.i=105]=\"i\",e[e.j=106]=\"j\",e[e.k=107]=\"k\",e[e.l=108]=\"l\",e[e.m=109]=\"m\",e[e.n=110]=\"n\",e[e.o=111]=\"o\",e[e.p=112]=\"p\",e[e.q=113]=\"q\",e[e.r=114]=\"r\",e[e.s=115]=\"s\",e[e.t=116]=\"t\",e[e.u=117]=\"u\",e[e.v=118]=\"v\",e[e.w=119]=\"w\",e[e.x=120]=\"x\",e[e.y=121]=\"y\",e[e.z=122]=\"z\",e[e.A=65]=\"A\",e[e.B=66]=\"B\",e[e.C=67]=\"C\",e[e.D=68]=\"D\",e[e.E=69]=\"E\",e[e.F=70]=\"F\",e[e.G=71]=\"G\",e[e.H=72]=\"H\",e[e.I=73]=\"I\",e[e.J=74]=\"J\",e[e.K=75]=\"K\",e[e.L=76]=\"L\",e[e.M=77]=\"M\",e[e.N=78]=\"N\",e[e.O=79]=\"O\",e[e.P=80]=\"P\",e[e.Q=81]=\"Q\",e[e.R=82]=\"R\",e[e.S=83]=\"S\",e[e.T=84]=\"T\",e[e.U=85]=\"U\",e[e.V=86]=\"V\",e[e.W=87]=\"W\",e[e.X=88]=\"X\",e[e.Y=89]=\"Y\",e[e.Z=90]=\"Z\",e[e.ampersand=38]=\"ampersand\",e[e.asterisk=42]=\"asterisk\",e[e.at=64]=\"at\",e[e.backslash=92]=\"backslash\",e[e.backtick=96]=\"backtick\",e[e.bar=124]=\"bar\",e[e.caret=94]=\"caret\",e[e.closeBrace=125]=\"closeBrace\",e[e.closeBracket=93]=\"closeBracket\",e[e.closeParen=41]=\"closeParen\",e[e.colon=58]=\"colon\",e[e.comma=44]=\"comma\",e[e.dot=46]=\"dot\",e[e.doubleQuote=34]=\"doubleQuote\",e[e.equals=61]=\"equals\",e[e.exclamation=33]=\"exclamation\",e[e.greaterThan=62]=\"greaterThan\",e[e.hash=35]=\"hash\",e[e.lessThan=60]=\"lessThan\",e[e.minus=45]=\"minus\",e[e.openBrace=123]=\"openBrace\",e[e.openBracket=91]=\"openBracket\",e[e.openParen=40]=\"openParen\",e[e.percent=37]=\"percent\",e[e.plus=43]=\"plus\",e[e.question=63]=\"question\",e[e.semicolon=59]=\"semicolon\",e[e.singleQuote=39]=\"singleQuote\",e[e.slash=47]=\"slash\",e[e.tilde=126]=\"tilde\",e[e.backspace=8]=\"backspace\",e[e.formFeed=12]=\"formFeed\",e[e.byteOrderMark=65279]=\"byteOrderMark\",e[e.tab=9]=\"tab\",e[e.verticalTab=11]=\"verticalTab\"}(e.CharacterCodes||(e.CharacterCodes={})),function(e){e.Ts=\".ts\",e.Tsx=\".tsx\",e.Dts=\".d.ts\",e.Js=\".js\",e.Jsx=\".jsx\",e.Json=\".json\"}(e.Extension||(e.Extension={})),function(e){e[e.None=0]=\"None\",e[e.TypeScript=1]=\"TypeScript\",e[e.ContainsTypeScript=2]=\"ContainsTypeScript\",e[e.ContainsJsx=4]=\"ContainsJsx\",e[e.ContainsESNext=8]=\"ContainsESNext\",e[e.ContainsES2017=16]=\"ContainsES2017\",e[e.ContainsES2016=32]=\"ContainsES2016\",e[e.ES2015=64]=\"ES2015\",e[e.ContainsES2015=128]=\"ContainsES2015\",e[e.Generator=256]=\"Generator\",e[e.ContainsGenerator=512]=\"ContainsGenerator\",e[e.DestructuringAssignment=1024]=\"DestructuringAssignment\",e[e.ContainsDestructuringAssignment=2048]=\"ContainsDestructuringAssignment\",e[e.ContainsDecorators=4096]=\"ContainsDecorators\",e[e.ContainsPropertyInitializer=8192]=\"ContainsPropertyInitializer\",e[e.ContainsLexicalThis=16384]=\"ContainsLexicalThis\",e[e.ContainsCapturedLexicalThis=32768]=\"ContainsCapturedLexicalThis\",e[e.ContainsLexicalThisInComputedPropertyName=65536]=\"ContainsLexicalThisInComputedPropertyName\",e[e.ContainsDefaultValueAssignments=131072]=\"ContainsDefaultValueAssignments\",e[e.ContainsParameterPropertyAssignments=262144]=\"ContainsParameterPropertyAssignments\",e[e.ContainsSpread=524288]=\"ContainsSpread\",e[e.ContainsObjectSpread=1048576]=\"ContainsObjectSpread\",e[e.ContainsRest=524288]=\"ContainsRest\",e[e.ContainsObjectRest=1048576]=\"ContainsObjectRest\",e[e.ContainsComputedPropertyName=2097152]=\"ContainsComputedPropertyName\",e[e.ContainsBlockScopedBinding=4194304]=\"ContainsBlockScopedBinding\",e[e.ContainsBindingPattern=8388608]=\"ContainsBindingPattern\",e[e.ContainsYield=16777216]=\"ContainsYield\",e[e.ContainsHoistedDeclarationOrCompletion=33554432]=\"ContainsHoistedDeclarationOrCompletion\",e[e.ContainsDynamicImport=67108864]=\"ContainsDynamicImport\",e[e.Super=134217728]=\"Super\",e[e.ContainsSuper=268435456]=\"ContainsSuper\",e[e.HasComputedFlags=536870912]=\"HasComputedFlags\",e[e.AssertTypeScript=3]=\"AssertTypeScript\",e[e.AssertJsx=4]=\"AssertJsx\",e[e.AssertESNext=8]=\"AssertESNext\",e[e.AssertES2017=16]=\"AssertES2017\",e[e.AssertES2016=32]=\"AssertES2016\",e[e.AssertES2015=192]=\"AssertES2015\",e[e.AssertGenerator=768]=\"AssertGenerator\",e[e.AssertDestructuringAssignment=3072]=\"AssertDestructuringAssignment\",e[e.OuterExpressionExcludes=536872257]=\"OuterExpressionExcludes\",e[e.PropertyAccessExcludes=671089985]=\"PropertyAccessExcludes\",e[e.NodeExcludes=939525441]=\"NodeExcludes\",e[e.ArrowFunctionExcludes=1003902273]=\"ArrowFunctionExcludes\",e[e.FunctionExcludes=1003935041]=\"FunctionExcludes\",e[e.ConstructorExcludes=1003668801]=\"ConstructorExcludes\",e[e.MethodOrAccessorExcludes=1003668801]=\"MethodOrAccessorExcludes\",e[e.ClassExcludes=942011713]=\"ClassExcludes\",e[e.ModuleExcludes=977327425]=\"ModuleExcludes\",e[e.TypeExcludes=-3]=\"TypeExcludes\",e[e.ObjectLiteralExcludes=942740801]=\"ObjectLiteralExcludes\",e[e.ArrayLiteralOrCallOrNewExcludes=940049729]=\"ArrayLiteralOrCallOrNewExcludes\",e[e.VariableDeclarationListExcludes=948962625]=\"VariableDeclarationListExcludes\",e[e.ParameterExcludes=939525441]=\"ParameterExcludes\",e[e.CatchClauseExcludes=940574017]=\"CatchClauseExcludes\",e[e.BindingPatternExcludes=940049729]=\"BindingPatternExcludes\",e[e.TypeScriptClassSyntaxMask=274432]=\"TypeScriptClassSyntaxMask\",e[e.ES2015FunctionSyntaxMask=163840]=\"ES2015FunctionSyntaxMask\"}(e.TransformFlags||(e.TransformFlags={})),function(e){e[e.None=0]=\"None\",e[e.SingleLine=1]=\"SingleLine\",e[e.AdviseOnEmitNode=2]=\"AdviseOnEmitNode\",e[e.NoSubstitution=4]=\"NoSubstitution\",e[e.CapturesThis=8]=\"CapturesThis\",e[e.NoLeadingSourceMap=16]=\"NoLeadingSourceMap\",e[e.NoTrailingSourceMap=32]=\"NoTrailingSourceMap\",e[e.NoSourceMap=48]=\"NoSourceMap\",e[e.NoNestedSourceMaps=64]=\"NoNestedSourceMaps\",e[e.NoTokenLeadingSourceMaps=128]=\"NoTokenLeadingSourceMaps\",e[e.NoTokenTrailingSourceMaps=256]=\"NoTokenTrailingSourceMaps\",e[e.NoTokenSourceMaps=384]=\"NoTokenSourceMaps\",e[e.NoLeadingComments=512]=\"NoLeadingComments\",e[e.NoTrailingComments=1024]=\"NoTrailingComments\",e[e.NoComments=1536]=\"NoComments\",e[e.NoNestedComments=2048]=\"NoNestedComments\",e[e.HelperName=4096]=\"HelperName\",e[e.ExportName=8192]=\"ExportName\",e[e.LocalName=16384]=\"LocalName\",e[e.InternalName=32768]=\"InternalName\",e[e.Indented=65536]=\"Indented\",e[e.NoIndentation=131072]=\"NoIndentation\",e[e.AsyncFunctionBody=262144]=\"AsyncFunctionBody\",e[e.ReuseTempVariableScope=524288]=\"ReuseTempVariableScope\",e[e.CustomPrologue=1048576]=\"CustomPrologue\",e[e.NoHoisting=2097152]=\"NoHoisting\",e[e.HasEndOfDeclarationMarker=4194304]=\"HasEndOfDeclarationMarker\",e[e.Iterator=8388608]=\"Iterator\",e[e.NoAsciiEscaping=16777216]=\"NoAsciiEscaping\",e[e.TypeScriptClassWrapper=33554432]=\"TypeScriptClassWrapper\",e[e.NeverApplyImportHelper=67108864]=\"NeverApplyImportHelper\"}(e.EmitFlags||(e.EmitFlags={})),function(e){e[e.Extends=1]=\"Extends\",e[e.Assign=2]=\"Assign\",e[e.Rest=4]=\"Rest\",e[e.Decorate=8]=\"Decorate\",e[e.Metadata=16]=\"Metadata\",e[e.Param=32]=\"Param\",e[e.Awaiter=64]=\"Awaiter\",e[e.Generator=128]=\"Generator\",e[e.Values=256]=\"Values\",e[e.Read=512]=\"Read\",e[e.Spread=1024]=\"Spread\",e[e.Await=2048]=\"Await\",e[e.AsyncGenerator=4096]=\"AsyncGenerator\",e[e.AsyncDelegator=8192]=\"AsyncDelegator\",e[e.AsyncValues=16384]=\"AsyncValues\",e[e.ExportStar=32768]=\"ExportStar\",e[e.MakeTemplateObject=65536]=\"MakeTemplateObject\",e[e.FirstEmitHelper=1]=\"FirstEmitHelper\",e[e.LastEmitHelper=65536]=\"LastEmitHelper\",e[e.ForOfIncludes=256]=\"ForOfIncludes\",e[e.ForAwaitOfIncludes=16384]=\"ForAwaitOfIncludes\",e[e.AsyncGeneratorIncludes=6144]=\"AsyncGeneratorIncludes\",e[e.AsyncDelegatorIncludes=26624]=\"AsyncDelegatorIncludes\",e[e.SpreadIncludes=1536]=\"SpreadIncludes\"}(e.ExternalEmitHelpers||(e.ExternalEmitHelpers={})),function(e){e[e.SourceFile=0]=\"SourceFile\",e[e.Expression=1]=\"Expression\",e[e.IdentifierName=2]=\"IdentifierName\",e[e.MappedTypeParameter=3]=\"MappedTypeParameter\",e[e.Unspecified=4]=\"Unspecified\"}(e.EmitHint||(e.EmitHint={})),function(e){e[e.None=0]=\"None\",e[e.SingleLine=0]=\"SingleLine\",e[e.MultiLine=1]=\"MultiLine\",e[e.PreserveLines=2]=\"PreserveLines\",e[e.LinesMask=3]=\"LinesMask\",e[e.NotDelimited=0]=\"NotDelimited\",e[e.BarDelimited=4]=\"BarDelimited\",e[e.AmpersandDelimited=8]=\"AmpersandDelimited\",e[e.CommaDelimited=16]=\"CommaDelimited\",e[e.DelimitersMask=28]=\"DelimitersMask\",e[e.AllowTrailingComma=32]=\"AllowTrailingComma\",e[e.Indented=64]=\"Indented\",e[e.SpaceBetweenBraces=128]=\"SpaceBetweenBraces\",e[e.SpaceBetweenSiblings=256]=\"SpaceBetweenSiblings\",e[e.Braces=512]=\"Braces\",e[e.Parenthesis=1024]=\"Parenthesis\",e[e.AngleBrackets=2048]=\"AngleBrackets\",e[e.SquareBrackets=4096]=\"SquareBrackets\",e[e.BracketsMask=7680]=\"BracketsMask\",e[e.OptionalIfUndefined=8192]=\"OptionalIfUndefined\",e[e.OptionalIfEmpty=16384]=\"OptionalIfEmpty\",e[e.Optional=24576]=\"Optional\",e[e.PreferNewLine=32768]=\"PreferNewLine\",e[e.NoTrailingNewLine=65536]=\"NoTrailingNewLine\",e[e.NoInterveningComments=131072]=\"NoInterveningComments\",e[e.NoSpaceIfEmpty=262144]=\"NoSpaceIfEmpty\",e[e.SingleElement=524288]=\"SingleElement\",e[e.Modifiers=131328]=\"Modifiers\",e[e.HeritageClauses=256]=\"HeritageClauses\",e[e.SingleLineTypeLiteralMembers=384]=\"SingleLineTypeLiteralMembers\",e[e.MultiLineTypeLiteralMembers=16449]=\"MultiLineTypeLiteralMembers\",e[e.TupleTypeElements=272]=\"TupleTypeElements\",e[e.UnionTypeConstituents=260]=\"UnionTypeConstituents\",e[e.IntersectionTypeConstituents=264]=\"IntersectionTypeConstituents\",e[e.ObjectBindingPatternElements=262576]=\"ObjectBindingPatternElements\",e[e.ArrayBindingPatternElements=262448]=\"ArrayBindingPatternElements\",e[e.ObjectLiteralExpressionProperties=263122]=\"ObjectLiteralExpressionProperties\",e[e.ArrayLiteralExpressionElements=4466]=\"ArrayLiteralExpressionElements\",e[e.CommaListElements=272]=\"CommaListElements\",e[e.CallExpressionArguments=1296]=\"CallExpressionArguments\",e[e.NewExpressionArguments=9488]=\"NewExpressionArguments\",e[e.TemplateExpressionSpans=131072]=\"TemplateExpressionSpans\",e[e.SingleLineBlockStatements=384]=\"SingleLineBlockStatements\",e[e.MultiLineBlockStatements=65]=\"MultiLineBlockStatements\",e[e.VariableDeclarationList=272]=\"VariableDeclarationList\",e[e.SingleLineFunctionBodyStatements=384]=\"SingleLineFunctionBodyStatements\",e[e.MultiLineFunctionBodyStatements=1]=\"MultiLineFunctionBodyStatements\",e[e.ClassHeritageClauses=0]=\"ClassHeritageClauses\",e[e.ClassMembers=65]=\"ClassMembers\",e[e.InterfaceMembers=65]=\"InterfaceMembers\",e[e.EnumMembers=81]=\"EnumMembers\",e[e.CaseBlockClauses=65]=\"CaseBlockClauses\",e[e.NamedImportsOrExportsElements=262576]=\"NamedImportsOrExportsElements\",e[e.JsxElementOrFragmentChildren=131072]=\"JsxElementOrFragmentChildren\",e[e.JsxElementAttributes=131328]=\"JsxElementAttributes\",e[e.CaseOrDefaultClauseStatements=81985]=\"CaseOrDefaultClauseStatements\",e[e.HeritageClauseTypes=272]=\"HeritageClauseTypes\",e[e.SourceFileStatements=65537]=\"SourceFileStatements\",e[e.Decorators=24577]=\"Decorators\",e[e.TypeArguments=26896]=\"TypeArguments\",e[e.TypeParameters=26896]=\"TypeParameters\",e[e.Parameters=1296]=\"Parameters\",e[e.IndexSignatureParameters=4432]=\"IndexSignatureParameters\"}(e.ListFormat||(e.ListFormat={})),function(e){e[e.None=0]=\"None\",e[e.TripleSlashXML=1]=\"TripleSlashXML\",e[e.SingleLine=2]=\"SingleLine\",e[e.MultiLine=4]=\"MultiLine\",e[e.All=7]=\"All\",e[e.Default=7]=\"Default\"}(e.PragmaKindFlags||(e.PragmaKindFlags={})),e.commentPragmas={reference:{args:[{name:\"types\",optional:!0,captureSpan:!0},{name:\"lib\",optional:!0,captureSpan:!0},{name:\"path\",optional:!0,captureSpan:!0},{name:\"no-default-lib\",optional:!0}],kind:1},\"amd-dependency\":{args:[{name:\"path\"},{name:\"name\",optional:!0}],kind:1},\"amd-module\":{args:[{name:\"name\"}],kind:1},\"ts-check\":{kind:2},\"ts-nocheck\":{kind:2},jsx:{args:[{name:\"factory\"}],kind:4}}}(Tn||(Tn={})),function(e){var n,t;e.setStackTraceLimit=function(){Error.stackTraceLimit<100&&(Error.stackTraceLimit=100)},function(e){e[e.Created=0]=\"Created\",e[e.Changed=1]=\"Changed\",e[e.Deleted=2]=\"Deleted\"}(n=e.FileWatcherEventKind||(e.FileWatcherEventKind={})),function(e){e[e.High=2e3]=\"High\",e[e.Medium=500]=\"Medium\",e[e.Low=250]=\"Low\"}(t=e.PollingInterval||(e.PollingInterval={}));var r,a,i=[r=250,a=2*r,4*a];function o(e){var n;return(n={})[t.Low]=e.Low,n[t.Medium]=e.Medium,n[t.High]=e.High,n}e.watchFileUsingPriorityPollingInterval=function(e,n,t,r){return e.watchFile(n,t,function(e){return i[e]}(r))},e.missingFileModifiedTime=new Date(0);var s={Low:32,Medium:64,High:256},l=o(s);function c(n){if(n.getEnvironmentVariable){var r=function(e,n){var t=a(e);if(t)return r(\"Low\"),r(\"Medium\"),r(\"High\"),!0;return!1;function r(e){n[e]=t[e]||n[e]}}(\"TSC_WATCH_POLLINGINTERVAL\",t);l=i(\"TSC_WATCH_POLLINGCHUNKSIZE\",s)||l,e.unchangedPollThresholds=i(\"TSC_WATCH_UNCHANGEDPOLLTHRESHOLDS\",s)||e.unchangedPollThresholds}function a(e){var t;return r(\"Low\"),r(\"Medium\"),r(\"High\"),t;function r(r){var a=function(e,t){return n.getEnvironmentVariable(e+\"_\"+t.toUpperCase())}(e,r);a&&((t||(t={}))[r]=Number(a))}}function i(e,n){var t=a(e);return(r||t)&&o(t?yn({},n,t):n)}}function u(e,n){var t=e.mtime.getTime(),r=n.getTime();return t!==r&&(e.mtime=n,e.callback(e.fileName,d(t,r)),!0)}function d(e,t){return 0===e?n.Created:0===t?n.Deleted:n.Changed}e.unchangedPollThresholds=o(s),e.setCustomPollingValues=c,e.createDynamicPriorityPollingWatchFile=function(n){var r=[],a=[],i=c(t.Low),o=c(t.Medium),s=c(t.High);return function(n,t,a){var i={fileName:n,callback:t,unchangedPolls:0,mtime:y(n)};return r.push(i),g(i,a),{close:function(){i.isClosed=!0,e.unorderedRemoveItem(r,i)}}};function c(e){var n=[];return n.pollingInterval=e,n.pollIndex=0,n.pollScheduled=!1,n}function d(n){n.pollIndex=p(n,n.pollingInterval,n.pollIndex,l[n.pollingInterval]),n.length?v(n.pollingInterval):(e.Debug.assert(0===n.pollIndex),n.pollScheduled=!1)}function m(e){p(a,t.Low,0,a.length),d(e),!e.pollScheduled&&a.length&&v(t.Low)}function p(n,r,i,o){for(var s=n.length,l=i,c=0;c<o&&s>0;++i===n.length&&(l<i&&(n.length=l),i=0,l=0),s--){var d=n[i];if(d)if(d.isClosed)n[i]=void 0;else{c++;var m=u(d,y(d.fileName));d.isClosed?n[i]=void 0:m?(d.unchangedPolls=0,n!==a&&(n[i]=void 0,p=d,a.push(p),_(t.Low))):d.unchangedPolls!==e.unchangedPollThresholds[r]?d.unchangedPolls++:n===a?(d.unchangedPolls=1,n[i]=void 0,g(d,t.Low)):r!==t.High&&(d.unchangedPolls++,n[i]=void 0,g(d,r===t.Low?t.Medium:t.High)),n[i]&&(l<i&&(n[l]=d,n[i]=void 0),l++)}}var p;return i}function f(e){switch(e){case t.Low:return i;case t.Medium:return o;case t.High:return s}}function g(e,n){f(n).push(e),_(n)}function _(e){f(e).pollScheduled||v(e)}function v(e){f(e).pollScheduled=n.setTimeout(e===t.Low?m:d,e,f(e))}function y(t){return n.getModifiedTime(t)||e.missingFileModifiedTime}},e.onWatchedFileStat=u,e.getFileWatcherEventKind=d,e.createRecursiveDirectoryWatcher=function(n){var t=e.createMap(),r=e.createMultiMap(),a=e.getStringComparer(!n.useCaseSensitiveFileNames),i=e.createGetCanonicalFileName(n.useCaseSensitiveFileNames);return o;function o(a,o){var l=i(a),c=t.get(l);return c?c.refCount++:(c={watcher:n.watchDirectory(a,function(n){r.forEach(function(t,r){(r===l||e.startsWith(l,r)&&l[r.length]===e.directorySeparator)&&t.forEach(function(e){return e(n)})}),s(a,l)}),refCount:1,childWatches:e.emptyArray},t.set(l,c),s(a,l)),o&&r.add(l,o),{dirName:a,close:function(){var n=e.Debug.assertDefined(t.get(l));o&&r.remove(l,o),n.refCount--,n.refCount||(t.delete(l),e.closeFileWatcherOf(n),n.childWatches.forEach(e.closeFileWatcher))}}}function s(r,i){var s=t.get(i);s&&(s.childWatches=function(t,r){var i;return e.enumerateInsertsAndDeletes(n.directoryExists(t)?e.mapDefined(n.getAccessibleSortedChildDirectories(t),function(r){var i=e.getNormalizedAbsolutePath(r,t);return 0===a(i,e.normalizePath(n.realpath(i)))?i:void 0}):e.emptyArray,r,function(e,n){return a(e,n.dirName)},function(e){s(o(e))},e.closeFileWatcher,s),i||e.emptyArray;function s(e){(i||(i=[])).push(e)}}(r,s.childWatches))}},e.getNodeMajorVersion=function(){if(\"undefined\"!=typeof process){var e=process.version;if(e){var n=e.indexOf(\".\");if(-1!==n)return parseInt(e.substring(1,n))}}},e.sys=void 0,e.sys&&e.sys.getEnvironmentVariable&&(c(e.sys),e.Debug.currentAssertionLevel=/^development$/i.test(e.sys.getEnvironmentVariable(\"NODE_ENV\"))?1:0),e.sys&&e.sys.debugMode&&(e.Debug.isDebugging=!0)}(Tn||(Tn={})),function(e){function n(e,n,t,r,a){return{code:e,category:n,key:t,message:r,reportsUnnecessary:a}}e.Diagnostics={Unterminated_string_literal:n(1002,e.DiagnosticCategory.Error,\"Unterminated_string_literal_1002\",\"Unterminated string literal.\"),Identifier_expected:n(1003,e.DiagnosticCategory.Error,\"Identifier_expected_1003\",\"Identifier expected.\"),_0_expected:n(1005,e.DiagnosticCategory.Error,\"_0_expected_1005\",\"'{0}' expected.\"),A_file_cannot_have_a_reference_to_itself:n(1006,e.DiagnosticCategory.Error,\"A_file_cannot_have_a_reference_to_itself_1006\",\"A file cannot have a reference to itself.\"),Trailing_comma_not_allowed:n(1009,e.DiagnosticCategory.Error,\"Trailing_comma_not_allowed_1009\",\"Trailing comma not allowed.\"),Asterisk_Slash_expected:n(1010,e.DiagnosticCategory.Error,\"Asterisk_Slash_expected_1010\",\"'*/' expected.\"),An_element_access_expression_should_take_an_argument:n(1011,e.DiagnosticCategory.Error,\"An_element_access_expression_should_take_an_argument_1011\",\"An element access expression should take an argument.\"),Unexpected_token:n(1012,e.DiagnosticCategory.Error,\"Unexpected_token_1012\",\"Unexpected token.\"),A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma:n(1013,e.DiagnosticCategory.Error,\"A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma_1013\",\"A rest parameter or binding pattern may not have a trailing comma.\"),A_rest_parameter_must_be_last_in_a_parameter_list:n(1014,e.DiagnosticCategory.Error,\"A_rest_parameter_must_be_last_in_a_parameter_list_1014\",\"A rest parameter must be last in a parameter list.\"),Parameter_cannot_have_question_mark_and_initializer:n(1015,e.DiagnosticCategory.Error,\"Parameter_cannot_have_question_mark_and_initializer_1015\",\"Parameter cannot have question mark and initializer.\"),A_required_parameter_cannot_follow_an_optional_parameter:n(1016,e.DiagnosticCategory.Error,\"A_required_parameter_cannot_follow_an_optional_parameter_1016\",\"A required parameter cannot follow an optional parameter.\"),An_index_signature_cannot_have_a_rest_parameter:n(1017,e.DiagnosticCategory.Error,\"An_index_signature_cannot_have_a_rest_parameter_1017\",\"An index signature cannot have a rest parameter.\"),An_index_signature_parameter_cannot_have_an_accessibility_modifier:n(1018,e.DiagnosticCategory.Error,\"An_index_signature_parameter_cannot_have_an_accessibility_modifier_1018\",\"An index signature parameter cannot have an accessibility modifier.\"),An_index_signature_parameter_cannot_have_a_question_mark:n(1019,e.DiagnosticCategory.Error,\"An_index_signature_parameter_cannot_have_a_question_mark_1019\",\"An index signature parameter cannot have a question mark.\"),An_index_signature_parameter_cannot_have_an_initializer:n(1020,e.DiagnosticCategory.Error,\"An_index_signature_parameter_cannot_have_an_initializer_1020\",\"An index signature parameter cannot have an initializer.\"),An_index_signature_must_have_a_type_annotation:n(1021,e.DiagnosticCategory.Error,\"An_index_signature_must_have_a_type_annotation_1021\",\"An index signature must have a type annotation.\"),An_index_signature_parameter_must_have_a_type_annotation:n(1022,e.DiagnosticCategory.Error,\"An_index_signature_parameter_must_have_a_type_annotation_1022\",\"An index signature parameter must have a type annotation.\"),An_index_signature_parameter_type_must_be_string_or_number:n(1023,e.DiagnosticCategory.Error,\"An_index_signature_parameter_type_must_be_string_or_number_1023\",\"An index signature parameter type must be 'string' or 'number'.\"),readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature:n(1024,e.DiagnosticCategory.Error,\"readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature_1024\",\"'readonly' modifier can only appear on a property declaration or index signature.\"),Accessibility_modifier_already_seen:n(1028,e.DiagnosticCategory.Error,\"Accessibility_modifier_already_seen_1028\",\"Accessibility modifier already seen.\"),_0_modifier_must_precede_1_modifier:n(1029,e.DiagnosticCategory.Error,\"_0_modifier_must_precede_1_modifier_1029\",\"'{0}' modifier must precede '{1}' modifier.\"),_0_modifier_already_seen:n(1030,e.DiagnosticCategory.Error,\"_0_modifier_already_seen_1030\",\"'{0}' modifier already seen.\"),_0_modifier_cannot_appear_on_a_class_element:n(1031,e.DiagnosticCategory.Error,\"_0_modifier_cannot_appear_on_a_class_element_1031\",\"'{0}' modifier cannot appear on a class element.\"),super_must_be_followed_by_an_argument_list_or_member_access:n(1034,e.DiagnosticCategory.Error,\"super_must_be_followed_by_an_argument_list_or_member_access_1034\",\"'super' must be followed by an argument list or member access.\"),Only_ambient_modules_can_use_quoted_names:n(1035,e.DiagnosticCategory.Error,\"Only_ambient_modules_can_use_quoted_names_1035\",\"Only ambient modules can use quoted names.\"),Statements_are_not_allowed_in_ambient_contexts:n(1036,e.DiagnosticCategory.Error,\"Statements_are_not_allowed_in_ambient_contexts_1036\",\"Statements are not allowed in ambient contexts.\"),A_declare_modifier_cannot_be_used_in_an_already_ambient_context:n(1038,e.DiagnosticCategory.Error,\"A_declare_modifier_cannot_be_used_in_an_already_ambient_context_1038\",\"A 'declare' modifier cannot be used in an already ambient context.\"),Initializers_are_not_allowed_in_ambient_contexts:n(1039,e.DiagnosticCategory.Error,\"Initializers_are_not_allowed_in_ambient_contexts_1039\",\"Initializers are not allowed in ambient contexts.\"),_0_modifier_cannot_be_used_in_an_ambient_context:n(1040,e.DiagnosticCategory.Error,\"_0_modifier_cannot_be_used_in_an_ambient_context_1040\",\"'{0}' modifier cannot be used in an ambient context.\"),_0_modifier_cannot_be_used_with_a_class_declaration:n(1041,e.DiagnosticCategory.Error,\"_0_modifier_cannot_be_used_with_a_class_declaration_1041\",\"'{0}' modifier cannot be used with a class declaration.\"),_0_modifier_cannot_be_used_here:n(1042,e.DiagnosticCategory.Error,\"_0_modifier_cannot_be_used_here_1042\",\"'{0}' modifier cannot be used here.\"),_0_modifier_cannot_appear_on_a_data_property:n(1043,e.DiagnosticCategory.Error,\"_0_modifier_cannot_appear_on_a_data_property_1043\",\"'{0}' modifier cannot appear on a data property.\"),_0_modifier_cannot_appear_on_a_module_or_namespace_element:n(1044,e.DiagnosticCategory.Error,\"_0_modifier_cannot_appear_on_a_module_or_namespace_element_1044\",\"'{0}' modifier cannot appear on a module or namespace element.\"),A_0_modifier_cannot_be_used_with_an_interface_declaration:n(1045,e.DiagnosticCategory.Error,\"A_0_modifier_cannot_be_used_with_an_interface_declaration_1045\",\"A '{0}' modifier cannot be used with an interface declaration.\"),A_declare_modifier_is_required_for_a_top_level_declaration_in_a_d_ts_file:n(1046,e.DiagnosticCategory.Error,\"A_declare_modifier_is_required_for_a_top_level_declaration_in_a_d_ts_file_1046\",\"A 'declare' modifier is required for a top level declaration in a .d.ts file.\"),A_rest_parameter_cannot_be_optional:n(1047,e.DiagnosticCategory.Error,\"A_rest_parameter_cannot_be_optional_1047\",\"A rest parameter cannot be optional.\"),A_rest_parameter_cannot_have_an_initializer:n(1048,e.DiagnosticCategory.Error,\"A_rest_parameter_cannot_have_an_initializer_1048\",\"A rest parameter cannot have an initializer.\"),A_set_accessor_must_have_exactly_one_parameter:n(1049,e.DiagnosticCategory.Error,\"A_set_accessor_must_have_exactly_one_parameter_1049\",\"A 'set' accessor must have exactly one parameter.\"),A_set_accessor_cannot_have_an_optional_parameter:n(1051,e.DiagnosticCategory.Error,\"A_set_accessor_cannot_have_an_optional_parameter_1051\",\"A 'set' accessor cannot have an optional parameter.\"),A_set_accessor_parameter_cannot_have_an_initializer:n(1052,e.DiagnosticCategory.Error,\"A_set_accessor_parameter_cannot_have_an_initializer_1052\",\"A 'set' accessor parameter cannot have an initializer.\"),A_set_accessor_cannot_have_rest_parameter:n(1053,e.DiagnosticCategory.Error,\"A_set_accessor_cannot_have_rest_parameter_1053\",\"A 'set' accessor cannot have rest parameter.\"),A_get_accessor_cannot_have_parameters:n(1054,e.DiagnosticCategory.Error,\"A_get_accessor_cannot_have_parameters_1054\",\"A 'get' accessor cannot have parameters.\"),Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value:n(1055,e.DiagnosticCategory.Error,\"Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Prom_1055\",\"Type '{0}' is not a valid async function return type in ES5/ES3 because it does not refer to a Promise-compatible constructor value.\"),Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher:n(1056,e.DiagnosticCategory.Error,\"Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher_1056\",\"Accessors are only available when targeting ECMAScript 5 and higher.\"),An_async_function_or_method_must_have_a_valid_awaitable_return_type:n(1057,e.DiagnosticCategory.Error,\"An_async_function_or_method_must_have_a_valid_awaitable_return_type_1057\",\"An async function or method must have a valid awaitable return type.\"),The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1058,e.DiagnosticCategory.Error,\"The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_t_1058\",\"The return type of an async function must either be a valid promise or must not contain a callable 'then' member.\"),A_promise_must_have_a_then_method:n(1059,e.DiagnosticCategory.Error,\"A_promise_must_have_a_then_method_1059\",\"A promise must have a 'then' method.\"),The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback:n(1060,e.DiagnosticCategory.Error,\"The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback_1060\",\"The first parameter of the 'then' method of a promise must be a callback.\"),Enum_member_must_have_initializer:n(1061,e.DiagnosticCategory.Error,\"Enum_member_must_have_initializer_1061\",\"Enum member must have initializer.\"),Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method:n(1062,e.DiagnosticCategory.Error,\"Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method_1062\",\"Type is referenced directly or indirectly in the fulfillment callback of its own 'then' method.\"),An_export_assignment_cannot_be_used_in_a_namespace:n(1063,e.DiagnosticCategory.Error,\"An_export_assignment_cannot_be_used_in_a_namespace_1063\",\"An export assignment cannot be used in a namespace.\"),The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type:n(1064,e.DiagnosticCategory.Error,\"The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type_1064\",\"The return type of an async function or method must be the global Promise<T> type.\"),In_ambient_enum_declarations_member_initializer_must_be_constant_expression:n(1066,e.DiagnosticCategory.Error,\"In_ambient_enum_declarations_member_initializer_must_be_constant_expression_1066\",\"In ambient enum declarations member initializer must be constant expression.\"),Unexpected_token_A_constructor_method_accessor_or_property_was_expected:n(1068,e.DiagnosticCategory.Error,\"Unexpected_token_A_constructor_method_accessor_or_property_was_expected_1068\",\"Unexpected token. A constructor, method, accessor, or property was expected.\"),Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces:n(1069,e.DiagnosticCategory.Error,\"Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces_1069\",\"Unexpected token. A type parameter name was expected without curly braces.\"),_0_modifier_cannot_appear_on_a_type_member:n(1070,e.DiagnosticCategory.Error,\"_0_modifier_cannot_appear_on_a_type_member_1070\",\"'{0}' modifier cannot appear on a type member.\"),_0_modifier_cannot_appear_on_an_index_signature:n(1071,e.DiagnosticCategory.Error,\"_0_modifier_cannot_appear_on_an_index_signature_1071\",\"'{0}' modifier cannot appear on an index signature.\"),A_0_modifier_cannot_be_used_with_an_import_declaration:n(1079,e.DiagnosticCategory.Error,\"A_0_modifier_cannot_be_used_with_an_import_declaration_1079\",\"A '{0}' modifier cannot be used with an import declaration.\"),Invalid_reference_directive_syntax:n(1084,e.DiagnosticCategory.Error,\"Invalid_reference_directive_syntax_1084\",\"Invalid 'reference' directive syntax.\"),Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0:n(1085,e.DiagnosticCategory.Error,\"Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0_1085\",\"Octal literals are not available when targeting ECMAScript 5 and higher. Use the syntax '{0}'.\"),An_accessor_cannot_be_declared_in_an_ambient_context:n(1086,e.DiagnosticCategory.Error,\"An_accessor_cannot_be_declared_in_an_ambient_context_1086\",\"An accessor cannot be declared in an ambient context.\"),_0_modifier_cannot_appear_on_a_constructor_declaration:n(1089,e.DiagnosticCategory.Error,\"_0_modifier_cannot_appear_on_a_constructor_declaration_1089\",\"'{0}' modifier cannot appear on a constructor declaration.\"),_0_modifier_cannot_appear_on_a_parameter:n(1090,e.DiagnosticCategory.Error,\"_0_modifier_cannot_appear_on_a_parameter_1090\",\"'{0}' modifier cannot appear on a parameter.\"),Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement:n(1091,e.DiagnosticCategory.Error,\"Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement_1091\",\"Only a single variable declaration is allowed in a 'for...in' statement.\"),Type_parameters_cannot_appear_on_a_constructor_declaration:n(1092,e.DiagnosticCategory.Error,\"Type_parameters_cannot_appear_on_a_constructor_declaration_1092\",\"Type parameters cannot appear on a constructor declaration.\"),Type_annotation_cannot_appear_on_a_constructor_declaration:n(1093,e.DiagnosticCategory.Error,\"Type_annotation_cannot_appear_on_a_constructor_declaration_1093\",\"Type annotation cannot appear on a constructor declaration.\"),An_accessor_cannot_have_type_parameters:n(1094,e.DiagnosticCategory.Error,\"An_accessor_cannot_have_type_parameters_1094\",\"An accessor cannot have type parameters.\"),A_set_accessor_cannot_have_a_return_type_annotation:n(1095,e.DiagnosticCategory.Error,\"A_set_accessor_cannot_have_a_return_type_annotation_1095\",\"A 'set' accessor cannot have a return type annotation.\"),An_index_signature_must_have_exactly_one_parameter:n(1096,e.DiagnosticCategory.Error,\"An_index_signature_must_have_exactly_one_parameter_1096\",\"An index signature must have exactly one parameter.\"),_0_list_cannot_be_empty:n(1097,e.DiagnosticCategory.Error,\"_0_list_cannot_be_empty_1097\",\"'{0}' list cannot be empty.\"),Type_parameter_list_cannot_be_empty:n(1098,e.DiagnosticCategory.Error,\"Type_parameter_list_cannot_be_empty_1098\",\"Type parameter list cannot be empty.\"),Type_argument_list_cannot_be_empty:n(1099,e.DiagnosticCategory.Error,\"Type_argument_list_cannot_be_empty_1099\",\"Type argument list cannot be empty.\"),Invalid_use_of_0_in_strict_mode:n(1100,e.DiagnosticCategory.Error,\"Invalid_use_of_0_in_strict_mode_1100\",\"Invalid use of '{0}' in strict mode.\"),with_statements_are_not_allowed_in_strict_mode:n(1101,e.DiagnosticCategory.Error,\"with_statements_are_not_allowed_in_strict_mode_1101\",\"'with' statements are not allowed in strict mode.\"),delete_cannot_be_called_on_an_identifier_in_strict_mode:n(1102,e.DiagnosticCategory.Error,\"delete_cannot_be_called_on_an_identifier_in_strict_mode_1102\",\"'delete' cannot be called on an identifier in strict mode.\"),A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator:n(1103,e.DiagnosticCategory.Error,\"A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator_1103\",\"A 'for-await-of' statement is only allowed within an async function or async generator.\"),A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement:n(1104,e.DiagnosticCategory.Error,\"A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement_1104\",\"A 'continue' statement can only be used within an enclosing iteration statement.\"),A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement:n(1105,e.DiagnosticCategory.Error,\"A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement_1105\",\"A 'break' statement can only be used within an enclosing iteration or switch statement.\"),Jump_target_cannot_cross_function_boundary:n(1107,e.DiagnosticCategory.Error,\"Jump_target_cannot_cross_function_boundary_1107\",\"Jump target cannot cross function boundary.\"),A_return_statement_can_only_be_used_within_a_function_body:n(1108,e.DiagnosticCategory.Error,\"A_return_statement_can_only_be_used_within_a_function_body_1108\",\"A 'return' statement can only be used within a function body.\"),Expression_expected:n(1109,e.DiagnosticCategory.Error,\"Expression_expected_1109\",\"Expression expected.\"),Type_expected:n(1110,e.DiagnosticCategory.Error,\"Type_expected_1110\",\"Type expected.\"),A_default_clause_cannot_appear_more_than_once_in_a_switch_statement:n(1113,e.DiagnosticCategory.Error,\"A_default_clause_cannot_appear_more_than_once_in_a_switch_statement_1113\",\"A 'default' clause cannot appear more than once in a 'switch' statement.\"),Duplicate_label_0:n(1114,e.DiagnosticCategory.Error,\"Duplicate_label_0_1114\",\"Duplicate label '{0}'.\"),A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement:n(1115,e.DiagnosticCategory.Error,\"A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement_1115\",\"A 'continue' statement can only jump to a label of an enclosing iteration statement.\"),A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement:n(1116,e.DiagnosticCategory.Error,\"A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement_1116\",\"A 'break' statement can only jump to a label of an enclosing statement.\"),An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode:n(1117,e.DiagnosticCategory.Error,\"An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode_1117\",\"An object literal cannot have multiple properties with the same name in strict mode.\"),An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name:n(1118,e.DiagnosticCategory.Error,\"An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name_1118\",\"An object literal cannot have multiple get/set accessors with the same name.\"),An_object_literal_cannot_have_property_and_accessor_with_the_same_name:n(1119,e.DiagnosticCategory.Error,\"An_object_literal_cannot_have_property_and_accessor_with_the_same_name_1119\",\"An object literal cannot have property and accessor with the same name.\"),An_export_assignment_cannot_have_modifiers:n(1120,e.DiagnosticCategory.Error,\"An_export_assignment_cannot_have_modifiers_1120\",\"An export assignment cannot have modifiers.\"),Octal_literals_are_not_allowed_in_strict_mode:n(1121,e.DiagnosticCategory.Error,\"Octal_literals_are_not_allowed_in_strict_mode_1121\",\"Octal literals are not allowed in strict mode.\"),Variable_declaration_list_cannot_be_empty:n(1123,e.DiagnosticCategory.Error,\"Variable_declaration_list_cannot_be_empty_1123\",\"Variable declaration list cannot be empty.\"),Digit_expected:n(1124,e.DiagnosticCategory.Error,\"Digit_expected_1124\",\"Digit expected.\"),Hexadecimal_digit_expected:n(1125,e.DiagnosticCategory.Error,\"Hexadecimal_digit_expected_1125\",\"Hexadecimal digit expected.\"),Unexpected_end_of_text:n(1126,e.DiagnosticCategory.Error,\"Unexpected_end_of_text_1126\",\"Unexpected end of text.\"),Invalid_character:n(1127,e.DiagnosticCategory.Error,\"Invalid_character_1127\",\"Invalid character.\"),Declaration_or_statement_expected:n(1128,e.DiagnosticCategory.Error,\"Declaration_or_statement_expected_1128\",\"Declaration or statement expected.\"),Statement_expected:n(1129,e.DiagnosticCategory.Error,\"Statement_expected_1129\",\"Statement expected.\"),case_or_default_expected:n(1130,e.DiagnosticCategory.Error,\"case_or_default_expected_1130\",\"'case' or 'default' expected.\"),Property_or_signature_expected:n(1131,e.DiagnosticCategory.Error,\"Property_or_signature_expected_1131\",\"Property or signature expected.\"),Enum_member_expected:n(1132,e.DiagnosticCategory.Error,\"Enum_member_expected_1132\",\"Enum member expected.\"),Variable_declaration_expected:n(1134,e.DiagnosticCategory.Error,\"Variable_declaration_expected_1134\",\"Variable declaration expected.\"),Argument_expression_expected:n(1135,e.DiagnosticCategory.Error,\"Argument_expression_expected_1135\",\"Argument expression expected.\"),Property_assignment_expected:n(1136,e.DiagnosticCategory.Error,\"Property_assignment_expected_1136\",\"Property assignment expected.\"),Expression_or_comma_expected:n(1137,e.DiagnosticCategory.Error,\"Expression_or_comma_expected_1137\",\"Expression or comma expected.\"),Parameter_declaration_expected:n(1138,e.DiagnosticCategory.Error,\"Parameter_declaration_expected_1138\",\"Parameter declaration expected.\"),Type_parameter_declaration_expected:n(1139,e.DiagnosticCategory.Error,\"Type_parameter_declaration_expected_1139\",\"Type parameter declaration expected.\"),Type_argument_expected:n(1140,e.DiagnosticCategory.Error,\"Type_argument_expected_1140\",\"Type argument expected.\"),String_literal_expected:n(1141,e.DiagnosticCategory.Error,\"String_literal_expected_1141\",\"String literal expected.\"),Line_break_not_permitted_here:n(1142,e.DiagnosticCategory.Error,\"Line_break_not_permitted_here_1142\",\"Line break not permitted here.\"),or_expected:n(1144,e.DiagnosticCategory.Error,\"or_expected_1144\",\"'{' or ';' expected.\"),Declaration_expected:n(1146,e.DiagnosticCategory.Error,\"Declaration_expected_1146\",\"Declaration expected.\"),Import_declarations_in_a_namespace_cannot_reference_a_module:n(1147,e.DiagnosticCategory.Error,\"Import_declarations_in_a_namespace_cannot_reference_a_module_1147\",\"Import declarations in a namespace cannot reference a module.\"),Cannot_use_imports_exports_or_module_augmentations_when_module_is_none:n(1148,e.DiagnosticCategory.Error,\"Cannot_use_imports_exports_or_module_augmentations_when_module_is_none_1148\",\"Cannot use imports, exports, or module augmentations when '--module' is 'none'.\"),File_name_0_differs_from_already_included_file_name_1_only_in_casing:n(1149,e.DiagnosticCategory.Error,\"File_name_0_differs_from_already_included_file_name_1_only_in_casing_1149\",\"File name '{0}' differs from already included file name '{1}' only in casing.\"),new_T_cannot_be_used_to_create_an_array_Use_new_Array_T_instead:n(1150,e.DiagnosticCategory.Error,\"new_T_cannot_be_used_to_create_an_array_Use_new_Array_T_instead_1150\",\"'new T[]' cannot be used to create an array. Use 'new Array<T>()' instead.\"),const_declarations_must_be_initialized:n(1155,e.DiagnosticCategory.Error,\"const_declarations_must_be_initialized_1155\",\"'const' declarations must be initialized.\"),const_declarations_can_only_be_declared_inside_a_block:n(1156,e.DiagnosticCategory.Error,\"const_declarations_can_only_be_declared_inside_a_block_1156\",\"'const' declarations can only be declared inside a block.\"),let_declarations_can_only_be_declared_inside_a_block:n(1157,e.DiagnosticCategory.Error,\"let_declarations_can_only_be_declared_inside_a_block_1157\",\"'let' declarations can only be declared inside a block.\"),Unterminated_template_literal:n(1160,e.DiagnosticCategory.Error,\"Unterminated_template_literal_1160\",\"Unterminated template literal.\"),Unterminated_regular_expression_literal:n(1161,e.DiagnosticCategory.Error,\"Unterminated_regular_expression_literal_1161\",\"Unterminated regular expression literal.\"),An_object_member_cannot_be_declared_optional:n(1162,e.DiagnosticCategory.Error,\"An_object_member_cannot_be_declared_optional_1162\",\"An object member cannot be declared optional.\"),A_yield_expression_is_only_allowed_in_a_generator_body:n(1163,e.DiagnosticCategory.Error,\"A_yield_expression_is_only_allowed_in_a_generator_body_1163\",\"A 'yield' expression is only allowed in a generator body.\"),Computed_property_names_are_not_allowed_in_enums:n(1164,e.DiagnosticCategory.Error,\"Computed_property_names_are_not_allowed_in_enums_1164\",\"Computed property names are not allowed in enums.\"),A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1165,e.DiagnosticCategory.Error,\"A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_t_1165\",\"A computed property name in an ambient context must refer to an expression whose type is a literal type or a 'unique symbol' type.\"),A_computed_property_name_in_a_class_property_declaration_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1166,e.DiagnosticCategory.Error,\"A_computed_property_name_in_a_class_property_declaration_must_refer_to_an_expression_whose_type_is_a_1166\",\"A computed property name in a class property declaration must refer to an expression whose type is a literal type or a 'unique symbol' type.\"),A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1168,e.DiagnosticCategory.Error,\"A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_ty_1168\",\"A computed property name in a method overload must refer to an expression whose type is a literal type or a 'unique symbol' type.\"),A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1169,e.DiagnosticCategory.Error,\"A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_1169\",\"A computed property name in an interface must refer to an expression whose type is a literal type or a 'unique symbol' type.\"),A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1170,e.DiagnosticCategory.Error,\"A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type__1170\",\"A computed property name in a type literal must refer to an expression whose type is a literal type or a 'unique symbol' type.\"),A_comma_expression_is_not_allowed_in_a_computed_property_name:n(1171,e.DiagnosticCategory.Error,\"A_comma_expression_is_not_allowed_in_a_computed_property_name_1171\",\"A comma expression is not allowed in a computed property name.\"),extends_clause_already_seen:n(1172,e.DiagnosticCategory.Error,\"extends_clause_already_seen_1172\",\"'extends' clause already seen.\"),extends_clause_must_precede_implements_clause:n(1173,e.DiagnosticCategory.Error,\"extends_clause_must_precede_implements_clause_1173\",\"'extends' clause must precede 'implements' clause.\"),Classes_can_only_extend_a_single_class:n(1174,e.DiagnosticCategory.Error,\"Classes_can_only_extend_a_single_class_1174\",\"Classes can only extend a single class.\"),implements_clause_already_seen:n(1175,e.DiagnosticCategory.Error,\"implements_clause_already_seen_1175\",\"'implements' clause already seen.\"),Interface_declaration_cannot_have_implements_clause:n(1176,e.DiagnosticCategory.Error,\"Interface_declaration_cannot_have_implements_clause_1176\",\"Interface declaration cannot have 'implements' clause.\"),Binary_digit_expected:n(1177,e.DiagnosticCategory.Error,\"Binary_digit_expected_1177\",\"Binary digit expected.\"),Octal_digit_expected:n(1178,e.DiagnosticCategory.Error,\"Octal_digit_expected_1178\",\"Octal digit expected.\"),Unexpected_token_expected:n(1179,e.DiagnosticCategory.Error,\"Unexpected_token_expected_1179\",\"Unexpected token. '{' expected.\"),Property_destructuring_pattern_expected:n(1180,e.DiagnosticCategory.Error,\"Property_destructuring_pattern_expected_1180\",\"Property destructuring pattern expected.\"),Array_element_destructuring_pattern_expected:n(1181,e.DiagnosticCategory.Error,\"Array_element_destructuring_pattern_expected_1181\",\"Array element destructuring pattern expected.\"),A_destructuring_declaration_must_have_an_initializer:n(1182,e.DiagnosticCategory.Error,\"A_destructuring_declaration_must_have_an_initializer_1182\",\"A destructuring declaration must have an initializer.\"),An_implementation_cannot_be_declared_in_ambient_contexts:n(1183,e.DiagnosticCategory.Error,\"An_implementation_cannot_be_declared_in_ambient_contexts_1183\",\"An implementation cannot be declared in ambient contexts.\"),Modifiers_cannot_appear_here:n(1184,e.DiagnosticCategory.Error,\"Modifiers_cannot_appear_here_1184\",\"Modifiers cannot appear here.\"),Merge_conflict_marker_encountered:n(1185,e.DiagnosticCategory.Error,\"Merge_conflict_marker_encountered_1185\",\"Merge conflict marker encountered.\"),A_rest_element_cannot_have_an_initializer:n(1186,e.DiagnosticCategory.Error,\"A_rest_element_cannot_have_an_initializer_1186\",\"A rest element cannot have an initializer.\"),A_parameter_property_may_not_be_declared_using_a_binding_pattern:n(1187,e.DiagnosticCategory.Error,\"A_parameter_property_may_not_be_declared_using_a_binding_pattern_1187\",\"A parameter property may not be declared using a binding pattern.\"),Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement:n(1188,e.DiagnosticCategory.Error,\"Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement_1188\",\"Only a single variable declaration is allowed in a 'for...of' statement.\"),The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer:n(1189,e.DiagnosticCategory.Error,\"The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer_1189\",\"The variable declaration of a 'for...in' statement cannot have an initializer.\"),The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer:n(1190,e.DiagnosticCategory.Error,\"The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer_1190\",\"The variable declaration of a 'for...of' statement cannot have an initializer.\"),An_import_declaration_cannot_have_modifiers:n(1191,e.DiagnosticCategory.Error,\"An_import_declaration_cannot_have_modifiers_1191\",\"An import declaration cannot have modifiers.\"),Module_0_has_no_default_export:n(1192,e.DiagnosticCategory.Error,\"Module_0_has_no_default_export_1192\",\"Module '{0}' has no default export.\"),An_export_declaration_cannot_have_modifiers:n(1193,e.DiagnosticCategory.Error,\"An_export_declaration_cannot_have_modifiers_1193\",\"An export declaration cannot have modifiers.\"),Export_declarations_are_not_permitted_in_a_namespace:n(1194,e.DiagnosticCategory.Error,\"Export_declarations_are_not_permitted_in_a_namespace_1194\",\"Export declarations are not permitted in a namespace.\"),Catch_clause_variable_cannot_have_a_type_annotation:n(1196,e.DiagnosticCategory.Error,\"Catch_clause_variable_cannot_have_a_type_annotation_1196\",\"Catch clause variable cannot have a type annotation.\"),Catch_clause_variable_cannot_have_an_initializer:n(1197,e.DiagnosticCategory.Error,\"Catch_clause_variable_cannot_have_an_initializer_1197\",\"Catch clause variable cannot have an initializer.\"),An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive:n(1198,e.DiagnosticCategory.Error,\"An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive_1198\",\"An extended Unicode escape value must be between 0x0 and 0x10FFFF inclusive.\"),Unterminated_Unicode_escape_sequence:n(1199,e.DiagnosticCategory.Error,\"Unterminated_Unicode_escape_sequence_1199\",\"Unterminated Unicode escape sequence.\"),Line_terminator_not_permitted_before_arrow:n(1200,e.DiagnosticCategory.Error,\"Line_terminator_not_permitted_before_arrow_1200\",\"Line terminator not permitted before arrow.\"),Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_ns_from_mod_import_a_from_mod_import_d_from_mod_or_another_module_format_instead:n(1202,e.DiagnosticCategory.Error,\"Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_1202\",\"Import assignment cannot be used when targeting ECMAScript modules. Consider using 'import * as ns from \\\"mod\\\"', 'import {a} from \\\"mod\\\"', 'import d from \\\"mod\\\"', or another module format instead.\"),Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or_another_module_format_instead:n(1203,e.DiagnosticCategory.Error,\"Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or__1203\",\"Export assignment cannot be used when targeting ECMAScript modules. Consider using 'export default' or another module format instead.\"),Cannot_re_export_a_type_when_the_isolatedModules_flag_is_provided:n(1205,e.DiagnosticCategory.Error,\"Cannot_re_export_a_type_when_the_isolatedModules_flag_is_provided_1205\",\"Cannot re-export a type when the '--isolatedModules' flag is provided.\"),Decorators_are_not_valid_here:n(1206,e.DiagnosticCategory.Error,\"Decorators_are_not_valid_here_1206\",\"Decorators are not valid here.\"),Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name:n(1207,e.DiagnosticCategory.Error,\"Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name_1207\",\"Decorators cannot be applied to multiple get/set accessors of the same name.\"),Cannot_compile_namespaces_when_the_isolatedModules_flag_is_provided:n(1208,e.DiagnosticCategory.Error,\"Cannot_compile_namespaces_when_the_isolatedModules_flag_is_provided_1208\",\"Cannot compile namespaces when the '--isolatedModules' flag is provided.\"),Ambient_const_enums_are_not_allowed_when_the_isolatedModules_flag_is_provided:n(1209,e.DiagnosticCategory.Error,\"Ambient_const_enums_are_not_allowed_when_the_isolatedModules_flag_is_provided_1209\",\"Ambient const enums are not allowed when the '--isolatedModules' flag is provided.\"),Invalid_use_of_0_Class_definitions_are_automatically_in_strict_mode:n(1210,e.DiagnosticCategory.Error,\"Invalid_use_of_0_Class_definitions_are_automatically_in_strict_mode_1210\",\"Invalid use of '{0}'. Class definitions are automatically in strict mode.\"),A_class_declaration_without_the_default_modifier_must_have_a_name:n(1211,e.DiagnosticCategory.Error,\"A_class_declaration_without_the_default_modifier_must_have_a_name_1211\",\"A class declaration without the 'default' modifier must have a name.\"),Identifier_expected_0_is_a_reserved_word_in_strict_mode:n(1212,e.DiagnosticCategory.Error,\"Identifier_expected_0_is_a_reserved_word_in_strict_mode_1212\",\"Identifier expected. '{0}' is a reserved word in strict mode.\"),Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_strict_mode:n(1213,e.DiagnosticCategory.Error,\"Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_stric_1213\",\"Identifier expected. '{0}' is a reserved word in strict mode. Class definitions are automatically in strict mode.\"),Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode:n(1214,e.DiagnosticCategory.Error,\"Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode_1214\",\"Identifier expected. '{0}' is a reserved word in strict mode. Modules are automatically in strict mode.\"),Invalid_use_of_0_Modules_are_automatically_in_strict_mode:n(1215,e.DiagnosticCategory.Error,\"Invalid_use_of_0_Modules_are_automatically_in_strict_mode_1215\",\"Invalid use of '{0}'. Modules are automatically in strict mode.\"),Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules:n(1216,e.DiagnosticCategory.Error,\"Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules_1216\",\"Identifier expected. '__esModule' is reserved as an exported marker when transforming ECMAScript modules.\"),Export_assignment_is_not_supported_when_module_flag_is_system:n(1218,e.DiagnosticCategory.Error,\"Export_assignment_is_not_supported_when_module_flag_is_system_1218\",\"Export assignment is not supported when '--module' flag is 'system'.\"),Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_to_remove_this_warning:n(1219,e.DiagnosticCategory.Error,\"Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_t_1219\",\"Experimental support for decorators is a feature that is subject to change in a future release. Set the 'experimentalDecorators' option to remove this warning.\"),Generators_are_only_available_when_targeting_ECMAScript_2015_or_higher:n(1220,e.DiagnosticCategory.Error,\"Generators_are_only_available_when_targeting_ECMAScript_2015_or_higher_1220\",\"Generators are only available when targeting ECMAScript 2015 or higher.\"),Generators_are_not_allowed_in_an_ambient_context:n(1221,e.DiagnosticCategory.Error,\"Generators_are_not_allowed_in_an_ambient_context_1221\",\"Generators are not allowed in an ambient context.\"),An_overload_signature_cannot_be_declared_as_a_generator:n(1222,e.DiagnosticCategory.Error,\"An_overload_signature_cannot_be_declared_as_a_generator_1222\",\"An overload signature cannot be declared as a generator.\"),_0_tag_already_specified:n(1223,e.DiagnosticCategory.Error,\"_0_tag_already_specified_1223\",\"'{0}' tag already specified.\"),Signature_0_must_be_a_type_predicate:n(1224,e.DiagnosticCategory.Error,\"Signature_0_must_be_a_type_predicate_1224\",\"Signature '{0}' must be a type predicate.\"),Cannot_find_parameter_0:n(1225,e.DiagnosticCategory.Error,\"Cannot_find_parameter_0_1225\",\"Cannot find parameter '{0}'.\"),Type_predicate_0_is_not_assignable_to_1:n(1226,e.DiagnosticCategory.Error,\"Type_predicate_0_is_not_assignable_to_1_1226\",\"Type predicate '{0}' is not assignable to '{1}'.\"),Parameter_0_is_not_in_the_same_position_as_parameter_1:n(1227,e.DiagnosticCategory.Error,\"Parameter_0_is_not_in_the_same_position_as_parameter_1_1227\",\"Parameter '{0}' is not in the same position as parameter '{1}'.\"),A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods:n(1228,e.DiagnosticCategory.Error,\"A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods_1228\",\"A type predicate is only allowed in return type position for functions and methods.\"),A_type_predicate_cannot_reference_a_rest_parameter:n(1229,e.DiagnosticCategory.Error,\"A_type_predicate_cannot_reference_a_rest_parameter_1229\",\"A type predicate cannot reference a rest parameter.\"),A_type_predicate_cannot_reference_element_0_in_a_binding_pattern:n(1230,e.DiagnosticCategory.Error,\"A_type_predicate_cannot_reference_element_0_in_a_binding_pattern_1230\",\"A type predicate cannot reference element '{0}' in a binding pattern.\"),An_export_assignment_can_only_be_used_in_a_module:n(1231,e.DiagnosticCategory.Error,\"An_export_assignment_can_only_be_used_in_a_module_1231\",\"An export assignment can only be used in a module.\"),An_import_declaration_can_only_be_used_in_a_namespace_or_module:n(1232,e.DiagnosticCategory.Error,\"An_import_declaration_can_only_be_used_in_a_namespace_or_module_1232\",\"An import declaration can only be used in a namespace or module.\"),An_export_declaration_can_only_be_used_in_a_module:n(1233,e.DiagnosticCategory.Error,\"An_export_declaration_can_only_be_used_in_a_module_1233\",\"An export declaration can only be used in a module.\"),An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file:n(1234,e.DiagnosticCategory.Error,\"An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file_1234\",\"An ambient module declaration is only allowed at the top level in a file.\"),A_namespace_declaration_is_only_allowed_in_a_namespace_or_module:n(1235,e.DiagnosticCategory.Error,\"A_namespace_declaration_is_only_allowed_in_a_namespace_or_module_1235\",\"A namespace declaration is only allowed in a namespace or module.\"),The_return_type_of_a_property_decorator_function_must_be_either_void_or_any:n(1236,e.DiagnosticCategory.Error,\"The_return_type_of_a_property_decorator_function_must_be_either_void_or_any_1236\",\"The return type of a property decorator function must be either 'void' or 'any'.\"),The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any:n(1237,e.DiagnosticCategory.Error,\"The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any_1237\",\"The return type of a parameter decorator function must be either 'void' or 'any'.\"),Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression:n(1238,e.DiagnosticCategory.Error,\"Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression_1238\",\"Unable to resolve signature of class decorator when called as an expression.\"),Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression:n(1239,e.DiagnosticCategory.Error,\"Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression_1239\",\"Unable to resolve signature of parameter decorator when called as an expression.\"),Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression:n(1240,e.DiagnosticCategory.Error,\"Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression_1240\",\"Unable to resolve signature of property decorator when called as an expression.\"),Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression:n(1241,e.DiagnosticCategory.Error,\"Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression_1241\",\"Unable to resolve signature of method decorator when called as an expression.\"),abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration:n(1242,e.DiagnosticCategory.Error,\"abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration_1242\",\"'abstract' modifier can only appear on a class, method, or property declaration.\"),_0_modifier_cannot_be_used_with_1_modifier:n(1243,e.DiagnosticCategory.Error,\"_0_modifier_cannot_be_used_with_1_modifier_1243\",\"'{0}' modifier cannot be used with '{1}' modifier.\"),Abstract_methods_can_only_appear_within_an_abstract_class:n(1244,e.DiagnosticCategory.Error,\"Abstract_methods_can_only_appear_within_an_abstract_class_1244\",\"Abstract methods can only appear within an abstract class.\"),Method_0_cannot_have_an_implementation_because_it_is_marked_abstract:n(1245,e.DiagnosticCategory.Error,\"Method_0_cannot_have_an_implementation_because_it_is_marked_abstract_1245\",\"Method '{0}' cannot have an implementation because it is marked abstract.\"),An_interface_property_cannot_have_an_initializer:n(1246,e.DiagnosticCategory.Error,\"An_interface_property_cannot_have_an_initializer_1246\",\"An interface property cannot have an initializer.\"),A_type_literal_property_cannot_have_an_initializer:n(1247,e.DiagnosticCategory.Error,\"A_type_literal_property_cannot_have_an_initializer_1247\",\"A type literal property cannot have an initializer.\"),A_class_member_cannot_have_the_0_keyword:n(1248,e.DiagnosticCategory.Error,\"A_class_member_cannot_have_the_0_keyword_1248\",\"A class member cannot have the '{0}' keyword.\"),A_decorator_can_only_decorate_a_method_implementation_not_an_overload:n(1249,e.DiagnosticCategory.Error,\"A_decorator_can_only_decorate_a_method_implementation_not_an_overload_1249\",\"A decorator can only decorate a method implementation, not an overload.\"),Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5:n(1250,e.DiagnosticCategory.Error,\"Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_1250\",\"Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'.\"),Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_definitions_are_automatically_in_strict_mode:n(1251,e.DiagnosticCategory.Error,\"Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_d_1251\",\"Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'. Class definitions are automatically in strict mode.\"),Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_are_automatically_in_strict_mode:n(1252,e.DiagnosticCategory.Error,\"Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_1252\",\"Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'. Modules are automatically in strict mode.\"),_0_tag_cannot_be_used_independently_as_a_top_level_JSDoc_tag:n(1253,e.DiagnosticCategory.Error,\"_0_tag_cannot_be_used_independently_as_a_top_level_JSDoc_tag_1253\",\"'{0}' tag cannot be used independently as a top level JSDoc tag.\"),A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_reference:n(1254,e.DiagnosticCategory.Error,\"A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_refere_1254\",\"A 'const' initializer in an ambient context must be a string or numeric literal or literal enum reference.\"),A_definite_assignment_assertion_is_not_permitted_in_this_context:n(1255,e.DiagnosticCategory.Error,\"A_definite_assignment_assertion_is_not_permitted_in_this_context_1255\",\"A definite assignment assertion '!' is not permitted in this context.\"),A_rest_element_must_be_last_in_a_tuple_type:n(1256,e.DiagnosticCategory.Error,\"A_rest_element_must_be_last_in_a_tuple_type_1256\",\"A rest element must be last in a tuple type.\"),A_required_element_cannot_follow_an_optional_element:n(1257,e.DiagnosticCategory.Error,\"A_required_element_cannot_follow_an_optional_element_1257\",\"A required element cannot follow an optional element.\"),with_statements_are_not_allowed_in_an_async_function_block:n(1300,e.DiagnosticCategory.Error,\"with_statements_are_not_allowed_in_an_async_function_block_1300\",\"'with' statements are not allowed in an async function block.\"),await_expression_is_only_allowed_within_an_async_function:n(1308,e.DiagnosticCategory.Error,\"await_expression_is_only_allowed_within_an_async_function_1308\",\"'await' expression is only allowed within an async function.\"),can_only_be_used_in_an_object_literal_property_inside_a_destructuring_assignment:n(1312,e.DiagnosticCategory.Error,\"can_only_be_used_in_an_object_literal_property_inside_a_destructuring_assignment_1312\",\"'=' can only be used in an object literal property inside a destructuring assignment.\"),The_body_of_an_if_statement_cannot_be_the_empty_statement:n(1313,e.DiagnosticCategory.Error,\"The_body_of_an_if_statement_cannot_be_the_empty_statement_1313\",\"The body of an 'if' statement cannot be the empty statement.\"),Global_module_exports_may_only_appear_in_module_files:n(1314,e.DiagnosticCategory.Error,\"Global_module_exports_may_only_appear_in_module_files_1314\",\"Global module exports may only appear in module files.\"),Global_module_exports_may_only_appear_in_declaration_files:n(1315,e.DiagnosticCategory.Error,\"Global_module_exports_may_only_appear_in_declaration_files_1315\",\"Global module exports may only appear in declaration files.\"),Global_module_exports_may_only_appear_at_top_level:n(1316,e.DiagnosticCategory.Error,\"Global_module_exports_may_only_appear_at_top_level_1316\",\"Global module exports may only appear at top level.\"),A_parameter_property_cannot_be_declared_using_a_rest_parameter:n(1317,e.DiagnosticCategory.Error,\"A_parameter_property_cannot_be_declared_using_a_rest_parameter_1317\",\"A parameter property cannot be declared using a rest parameter.\"),An_abstract_accessor_cannot_have_an_implementation:n(1318,e.DiagnosticCategory.Error,\"An_abstract_accessor_cannot_have_an_implementation_1318\",\"An abstract accessor cannot have an implementation.\"),A_default_export_can_only_be_used_in_an_ECMAScript_style_module:n(1319,e.DiagnosticCategory.Error,\"A_default_export_can_only_be_used_in_an_ECMAScript_style_module_1319\",\"A default export can only be used in an ECMAScript-style module.\"),Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1320,e.DiagnosticCategory.Error,\"Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member_1320\",\"Type of 'await' operand must either be a valid promise or must not contain a callable 'then' member.\"),Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1321,e.DiagnosticCategory.Error,\"Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_cal_1321\",\"Type of 'yield' operand in an async generator must either be a valid promise or must not contain a callable 'then' member.\"),Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1322,e.DiagnosticCategory.Error,\"Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_con_1322\",\"Type of iterated elements of a 'yield*' operand must either be a valid promise or must not contain a callable 'then' member.\"),Dynamic_import_is_only_supported_when_module_flag_is_commonjs_or_esNext:n(1323,e.DiagnosticCategory.Error,\"Dynamic_import_is_only_supported_when_module_flag_is_commonjs_or_esNext_1323\",\"Dynamic import is only supported when '--module' flag is 'commonjs' or 'esNext'.\"),Dynamic_import_must_have_one_specifier_as_an_argument:n(1324,e.DiagnosticCategory.Error,\"Dynamic_import_must_have_one_specifier_as_an_argument_1324\",\"Dynamic import must have one specifier as an argument.\"),Specifier_of_dynamic_import_cannot_be_spread_element:n(1325,e.DiagnosticCategory.Error,\"Specifier_of_dynamic_import_cannot_be_spread_element_1325\",\"Specifier of dynamic import cannot be spread element.\"),Dynamic_import_cannot_have_type_arguments:n(1326,e.DiagnosticCategory.Error,\"Dynamic_import_cannot_have_type_arguments_1326\",\"Dynamic import cannot have type arguments\"),String_literal_with_double_quotes_expected:n(1327,e.DiagnosticCategory.Error,\"String_literal_with_double_quotes_expected_1327\",\"String literal with double quotes expected.\"),Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_literal:n(1328,e.DiagnosticCategory.Error,\"Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_li_1328\",\"Property value can only be string literal, numeric literal, 'true', 'false', 'null', object literal or array literal.\"),_0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0:n(1329,e.DiagnosticCategory.Error,\"_0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write__1329\",\"'{0}' accepts too few arguments to be used as a decorator here. Did you mean to call it first and write '@{0}()'?\"),A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly:n(1330,e.DiagnosticCategory.Error,\"A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly_1330\",\"A property of an interface or type literal whose type is a 'unique symbol' type must be 'readonly'.\"),A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly:n(1331,e.DiagnosticCategory.Error,\"A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly_1331\",\"A property of a class whose type is a 'unique symbol' type must be both 'static' and 'readonly'.\"),A_variable_whose_type_is_a_unique_symbol_type_must_be_const:n(1332,e.DiagnosticCategory.Error,\"A_variable_whose_type_is_a_unique_symbol_type_must_be_const_1332\",\"A variable whose type is a 'unique symbol' type must be 'const'.\"),unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name:n(1333,e.DiagnosticCategory.Error,\"unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name_1333\",\"'unique symbol' types may not be used on a variable declaration with a binding name.\"),unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement:n(1334,e.DiagnosticCategory.Error,\"unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement_1334\",\"'unique symbol' types are only allowed on variables in a variable statement.\"),unique_symbol_types_are_not_allowed_here:n(1335,e.DiagnosticCategory.Error,\"unique_symbol_types_are_not_allowed_here_1335\",\"'unique symbol' types are not allowed here.\"),An_index_signature_parameter_type_cannot_be_a_type_alias_Consider_writing_0_Colon_1_Colon_2_instead:n(1336,e.DiagnosticCategory.Error,\"An_index_signature_parameter_type_cannot_be_a_type_alias_Consider_writing_0_Colon_1_Colon_2_instead_1336\",\"An index signature parameter type cannot be a type alias. Consider writing '[{0}: {1}]: {2}' instead.\"),An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead:n(1337,e.DiagnosticCategory.Error,\"An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead_1337\",\"An index signature parameter type cannot be a union type. Consider using a mapped object type instead.\"),infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type:n(1338,e.DiagnosticCategory.Error,\"infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type_1338\",\"'infer' declarations are only permitted in the 'extends' clause of a conditional type.\"),Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here:n(1339,e.DiagnosticCategory.Error,\"Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here_1339\",\"Module '{0}' does not refer to a value, but is used as a value here.\"),Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0:n(1340,e.DiagnosticCategory.Error,\"Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0_1340\",\"Module '{0}' does not refer to a type, but is used as a type here. Did you mean 'typeof import('{0}')'?\"),Type_arguments_cannot_be_used_here:n(1342,e.DiagnosticCategory.Error,\"Type_arguments_cannot_be_used_here_1342\",\"Type arguments cannot be used here.\"),The_import_meta_meta_property_is_only_allowed_using_ESNext_for_the_target_and_module_compiler_options:n(1343,e.DiagnosticCategory.Error,\"The_import_meta_meta_property_is_only_allowed_using_ESNext_for_the_target_and_module_compiler_option_1343\",\"The 'import.meta' meta-property is only allowed using 'ESNext' for the 'target' and 'module' compiler options.\"),A_label_is_not_allowed_here:n(1344,e.DiagnosticCategory.Error,\"A_label_is_not_allowed_here_1344\",\"'A label is not allowed here.\"),An_expression_of_type_void_cannot_be_tested_for_truthiness:n(1345,e.DiagnosticCategory.Error,\"An_expression_of_type_void_cannot_be_tested_for_truthiness_1345\",\"An expression of type 'void' cannot be tested for truthiness\"),This_parameter_is_not_allowed_with_use_strict_directive:n(1346,e.DiagnosticCategory.Error,\"This_parameter_is_not_allowed_with_use_strict_directive_1346\",\"This parameter is not allowed with 'use strict' directive.\"),use_strict_directive_cannot_be_used_with_non_simple_parameter_list:n(1347,e.DiagnosticCategory.Error,\"use_strict_directive_cannot_be_used_with_non_simple_parameter_list_1347\",\"'use strict' directive cannot be used with non-simple parameter list.\"),Non_simple_parameter_declared_here:n(1348,e.DiagnosticCategory.Error,\"Non_simple_parameter_declared_here_1348\",\"Non-simple parameter declared here.\"),use_strict_directive_used_here:n(1349,e.DiagnosticCategory.Error,\"use_strict_directive_used_here_1349\",\"'use strict' directive used here.\"),Duplicate_identifier_0:n(2300,e.DiagnosticCategory.Error,\"Duplicate_identifier_0_2300\",\"Duplicate identifier '{0}'.\"),Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor:n(2301,e.DiagnosticCategory.Error,\"Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor_2301\",\"Initializer of instance member variable '{0}' cannot reference identifier '{1}' declared in the constructor.\"),Static_members_cannot_reference_class_type_parameters:n(2302,e.DiagnosticCategory.Error,\"Static_members_cannot_reference_class_type_parameters_2302\",\"Static members cannot reference class type parameters.\"),Circular_definition_of_import_alias_0:n(2303,e.DiagnosticCategory.Error,\"Circular_definition_of_import_alias_0_2303\",\"Circular definition of import alias '{0}'.\"),Cannot_find_name_0:n(2304,e.DiagnosticCategory.Error,\"Cannot_find_name_0_2304\",\"Cannot find name '{0}'.\"),Module_0_has_no_exported_member_1:n(2305,e.DiagnosticCategory.Error,\"Module_0_has_no_exported_member_1_2305\",\"Module '{0}' has no exported member '{1}'.\"),File_0_is_not_a_module:n(2306,e.DiagnosticCategory.Error,\"File_0_is_not_a_module_2306\",\"File '{0}' is not a module.\"),Cannot_find_module_0:n(2307,e.DiagnosticCategory.Error,\"Cannot_find_module_0_2307\",\"Cannot find module '{0}'.\"),Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambiguity:n(2308,e.DiagnosticCategory.Error,\"Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambig_2308\",\"Module {0} has already exported a member named '{1}'. Consider explicitly re-exporting to resolve the ambiguity.\"),An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements:n(2309,e.DiagnosticCategory.Error,\"An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements_2309\",\"An export assignment cannot be used in a module with other exported elements.\"),Type_0_recursively_references_itself_as_a_base_type:n(2310,e.DiagnosticCategory.Error,\"Type_0_recursively_references_itself_as_a_base_type_2310\",\"Type '{0}' recursively references itself as a base type.\"),A_class_may_only_extend_another_class:n(2311,e.DiagnosticCategory.Error,\"A_class_may_only_extend_another_class_2311\",\"A class may only extend another class.\"),An_interface_may_only_extend_a_class_or_another_interface:n(2312,e.DiagnosticCategory.Error,\"An_interface_may_only_extend_a_class_or_another_interface_2312\",\"An interface may only extend a class or another interface.\"),Type_parameter_0_has_a_circular_constraint:n(2313,e.DiagnosticCategory.Error,\"Type_parameter_0_has_a_circular_constraint_2313\",\"Type parameter '{0}' has a circular constraint.\"),Generic_type_0_requires_1_type_argument_s:n(2314,e.DiagnosticCategory.Error,\"Generic_type_0_requires_1_type_argument_s_2314\",\"Generic type '{0}' requires {1} type argument(s).\"),Type_0_is_not_generic:n(2315,e.DiagnosticCategory.Error,\"Type_0_is_not_generic_2315\",\"Type '{0}' is not generic.\"),Global_type_0_must_be_a_class_or_interface_type:n(2316,e.DiagnosticCategory.Error,\"Global_type_0_must_be_a_class_or_interface_type_2316\",\"Global type '{0}' must be a class or interface type.\"),Global_type_0_must_have_1_type_parameter_s:n(2317,e.DiagnosticCategory.Error,\"Global_type_0_must_have_1_type_parameter_s_2317\",\"Global type '{0}' must have {1} type parameter(s).\"),Cannot_find_global_type_0:n(2318,e.DiagnosticCategory.Error,\"Cannot_find_global_type_0_2318\",\"Cannot find global type '{0}'.\"),Named_property_0_of_types_1_and_2_are_not_identical:n(2319,e.DiagnosticCategory.Error,\"Named_property_0_of_types_1_and_2_are_not_identical_2319\",\"Named property '{0}' of types '{1}' and '{2}' are not identical.\"),Interface_0_cannot_simultaneously_extend_types_1_and_2:n(2320,e.DiagnosticCategory.Error,\"Interface_0_cannot_simultaneously_extend_types_1_and_2_2320\",\"Interface '{0}' cannot simultaneously extend types '{1}' and '{2}'.\"),Excessive_stack_depth_comparing_types_0_and_1:n(2321,e.DiagnosticCategory.Error,\"Excessive_stack_depth_comparing_types_0_and_1_2321\",\"Excessive stack depth comparing types '{0}' and '{1}'.\"),Type_0_is_not_assignable_to_type_1:n(2322,e.DiagnosticCategory.Error,\"Type_0_is_not_assignable_to_type_1_2322\",\"Type '{0}' is not assignable to type '{1}'.\"),Cannot_redeclare_exported_variable_0:n(2323,e.DiagnosticCategory.Error,\"Cannot_redeclare_exported_variable_0_2323\",\"Cannot redeclare exported variable '{0}'.\"),Property_0_is_missing_in_type_1:n(2324,e.DiagnosticCategory.Error,\"Property_0_is_missing_in_type_1_2324\",\"Property '{0}' is missing in type '{1}'.\"),Property_0_is_private_in_type_1_but_not_in_type_2:n(2325,e.DiagnosticCategory.Error,\"Property_0_is_private_in_type_1_but_not_in_type_2_2325\",\"Property '{0}' is private in type '{1}' but not in type '{2}'.\"),Types_of_property_0_are_incompatible:n(2326,e.DiagnosticCategory.Error,\"Types_of_property_0_are_incompatible_2326\",\"Types of property '{0}' are incompatible.\"),Property_0_is_optional_in_type_1_but_required_in_type_2:n(2327,e.DiagnosticCategory.Error,\"Property_0_is_optional_in_type_1_but_required_in_type_2_2327\",\"Property '{0}' is optional in type '{1}' but required in type '{2}'.\"),Types_of_parameters_0_and_1_are_incompatible:n(2328,e.DiagnosticCategory.Error,\"Types_of_parameters_0_and_1_are_incompatible_2328\",\"Types of parameters '{0}' and '{1}' are incompatible.\"),Index_signature_is_missing_in_type_0:n(2329,e.DiagnosticCategory.Error,\"Index_signature_is_missing_in_type_0_2329\",\"Index signature is missing in type '{0}'.\"),Index_signatures_are_incompatible:n(2330,e.DiagnosticCategory.Error,\"Index_signatures_are_incompatible_2330\",\"Index signatures are incompatible.\"),this_cannot_be_referenced_in_a_module_or_namespace_body:n(2331,e.DiagnosticCategory.Error,\"this_cannot_be_referenced_in_a_module_or_namespace_body_2331\",\"'this' cannot be referenced in a module or namespace body.\"),this_cannot_be_referenced_in_current_location:n(2332,e.DiagnosticCategory.Error,\"this_cannot_be_referenced_in_current_location_2332\",\"'this' cannot be referenced in current location.\"),this_cannot_be_referenced_in_constructor_arguments:n(2333,e.DiagnosticCategory.Error,\"this_cannot_be_referenced_in_constructor_arguments_2333\",\"'this' cannot be referenced in constructor arguments.\"),this_cannot_be_referenced_in_a_static_property_initializer:n(2334,e.DiagnosticCategory.Error,\"this_cannot_be_referenced_in_a_static_property_initializer_2334\",\"'this' cannot be referenced in a static property initializer.\"),super_can_only_be_referenced_in_a_derived_class:n(2335,e.DiagnosticCategory.Error,\"super_can_only_be_referenced_in_a_derived_class_2335\",\"'super' can only be referenced in a derived class.\"),super_cannot_be_referenced_in_constructor_arguments:n(2336,e.DiagnosticCategory.Error,\"super_cannot_be_referenced_in_constructor_arguments_2336\",\"'super' cannot be referenced in constructor arguments.\"),Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors:n(2337,e.DiagnosticCategory.Error,\"Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors_2337\",\"Super calls are not permitted outside constructors or in nested functions inside constructors.\"),super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_derived_class:n(2338,e.DiagnosticCategory.Error,\"super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_der_2338\",\"'super' property access is permitted only in a constructor, member function, or member accessor of a derived class.\"),Property_0_does_not_exist_on_type_1:n(2339,e.DiagnosticCategory.Error,\"Property_0_does_not_exist_on_type_1_2339\",\"Property '{0}' does not exist on type '{1}'.\"),Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword:n(2340,e.DiagnosticCategory.Error,\"Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword_2340\",\"Only public and protected methods of the base class are accessible via the 'super' keyword.\"),Property_0_is_private_and_only_accessible_within_class_1:n(2341,e.DiagnosticCategory.Error,\"Property_0_is_private_and_only_accessible_within_class_1_2341\",\"Property '{0}' is private and only accessible within class '{1}'.\"),An_index_expression_argument_must_be_of_type_string_number_symbol_or_any:n(2342,e.DiagnosticCategory.Error,\"An_index_expression_argument_must_be_of_type_string_number_symbol_or_any_2342\",\"An index expression argument must be of type 'string', 'number', 'symbol', or 'any'.\"),This_syntax_requires_an_imported_helper_named_1_but_module_0_has_no_exported_member_1:n(2343,e.DiagnosticCategory.Error,\"This_syntax_requires_an_imported_helper_named_1_but_module_0_has_no_exported_member_1_2343\",\"This syntax requires an imported helper named '{1}', but module '{0}' has no exported member '{1}'.\"),Type_0_does_not_satisfy_the_constraint_1:n(2344,e.DiagnosticCategory.Error,\"Type_0_does_not_satisfy_the_constraint_1_2344\",\"Type '{0}' does not satisfy the constraint '{1}'.\"),Argument_of_type_0_is_not_assignable_to_parameter_of_type_1:n(2345,e.DiagnosticCategory.Error,\"Argument_of_type_0_is_not_assignable_to_parameter_of_type_1_2345\",\"Argument of type '{0}' is not assignable to parameter of type '{1}'.\"),Call_target_does_not_contain_any_signatures:n(2346,e.DiagnosticCategory.Error,\"Call_target_does_not_contain_any_signatures_2346\",\"Call target does not contain any signatures.\"),Untyped_function_calls_may_not_accept_type_arguments:n(2347,e.DiagnosticCategory.Error,\"Untyped_function_calls_may_not_accept_type_arguments_2347\",\"Untyped function calls may not accept type arguments.\"),Value_of_type_0_is_not_callable_Did_you_mean_to_include_new:n(2348,e.DiagnosticCategory.Error,\"Value_of_type_0_is_not_callable_Did_you_mean_to_include_new_2348\",\"Value of type '{0}' is not callable. Did you mean to include 'new'?\"),Cannot_invoke_an_expression_whose_type_lacks_a_call_signature_Type_0_has_no_compatible_call_signatures:n(2349,e.DiagnosticCategory.Error,\"Cannot_invoke_an_expression_whose_type_lacks_a_call_signature_Type_0_has_no_compatible_call_signatur_2349\",\"Cannot invoke an expression whose type lacks a call signature. Type '{0}' has no compatible call signatures.\"),Only_a_void_function_can_be_called_with_the_new_keyword:n(2350,e.DiagnosticCategory.Error,\"Only_a_void_function_can_be_called_with_the_new_keyword_2350\",\"Only a void function can be called with the 'new' keyword.\"),Cannot_use_new_with_an_expression_whose_type_lacks_a_call_or_construct_signature:n(2351,e.DiagnosticCategory.Error,\"Cannot_use_new_with_an_expression_whose_type_lacks_a_call_or_construct_signature_2351\",\"Cannot use 'new' with an expression whose type lacks a call or construct signature.\"),Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first:n(2352,e.DiagnosticCategory.Error,\"Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the__2352\",\"Conversion of type '{0}' to type '{1}' may be a mistake because neither type sufficiently overlaps with the other. If this was intentional, convert the expression to 'unknown' first.\"),Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1:n(2353,e.DiagnosticCategory.Error,\"Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1_2353\",\"Object literal may only specify known properties, and '{0}' does not exist in type '{1}'.\"),This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found:n(2354,e.DiagnosticCategory.Error,\"This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found_2354\",\"This syntax requires an imported helper but module '{0}' cannot be found.\"),A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value:n(2355,e.DiagnosticCategory.Error,\"A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value_2355\",\"A function whose declared type is neither 'void' nor 'any' must return a value.\"),An_arithmetic_operand_must_be_of_type_any_number_or_an_enum_type:n(2356,e.DiagnosticCategory.Error,\"An_arithmetic_operand_must_be_of_type_any_number_or_an_enum_type_2356\",\"An arithmetic operand must be of type 'any', 'number' or an enum type.\"),The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access:n(2357,e.DiagnosticCategory.Error,\"The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access_2357\",\"The operand of an increment or decrement operator must be a variable or a property access.\"),The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_parameter:n(2358,e.DiagnosticCategory.Error,\"The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_paramete_2358\",\"The left-hand side of an 'instanceof' expression must be of type 'any', an object type or a type parameter.\"),The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_Function_interface_type:n(2359,e.DiagnosticCategory.Error,\"The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_F_2359\",\"The right-hand side of an 'instanceof' expression must be of type 'any' or of a type assignable to the 'Function' interface type.\"),The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol:n(2360,e.DiagnosticCategory.Error,\"The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol_2360\",\"The left-hand side of an 'in' expression must be of type 'any', 'string', 'number', or 'symbol'.\"),The_right_hand_side_of_an_in_expression_must_be_of_type_any_an_object_type_or_a_type_parameter:n(2361,e.DiagnosticCategory.Error,\"The_right_hand_side_of_an_in_expression_must_be_of_type_any_an_object_type_or_a_type_parameter_2361\",\"The right-hand side of an 'in' expression must be of type 'any', an object type or a type parameter.\"),The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_or_an_enum_type:n(2362,e.DiagnosticCategory.Error,\"The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_or_an_enum_type_2362\",\"The left-hand side of an arithmetic operation must be of type 'any', 'number' or an enum type.\"),The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_or_an_enum_type:n(2363,e.DiagnosticCategory.Error,\"The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_or_an_enum_type_2363\",\"The right-hand side of an arithmetic operation must be of type 'any', 'number' or an enum type.\"),The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access:n(2364,e.DiagnosticCategory.Error,\"The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access_2364\",\"The left-hand side of an assignment expression must be a variable or a property access.\"),Operator_0_cannot_be_applied_to_types_1_and_2:n(2365,e.DiagnosticCategory.Error,\"Operator_0_cannot_be_applied_to_types_1_and_2_2365\",\"Operator '{0}' cannot be applied to types '{1}' and '{2}'.\"),Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined:n(2366,e.DiagnosticCategory.Error,\"Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined_2366\",\"Function lacks ending return statement and return type does not include 'undefined'.\"),This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap:n(2367,e.DiagnosticCategory.Error,\"This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap_2367\",\"This condition will always return '{0}' since the types '{1}' and '{2}' have no overlap.\"),Type_parameter_name_cannot_be_0:n(2368,e.DiagnosticCategory.Error,\"Type_parameter_name_cannot_be_0_2368\",\"Type parameter name cannot be '{0}'.\"),A_parameter_property_is_only_allowed_in_a_constructor_implementation:n(2369,e.DiagnosticCategory.Error,\"A_parameter_property_is_only_allowed_in_a_constructor_implementation_2369\",\"A parameter property is only allowed in a constructor implementation.\"),A_rest_parameter_must_be_of_an_array_type:n(2370,e.DiagnosticCategory.Error,\"A_rest_parameter_must_be_of_an_array_type_2370\",\"A rest parameter must be of an array type.\"),A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation:n(2371,e.DiagnosticCategory.Error,\"A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation_2371\",\"A parameter initializer is only allowed in a function or constructor implementation.\"),Parameter_0_cannot_be_referenced_in_its_initializer:n(2372,e.DiagnosticCategory.Error,\"Parameter_0_cannot_be_referenced_in_its_initializer_2372\",\"Parameter '{0}' cannot be referenced in its initializer.\"),Initializer_of_parameter_0_cannot_reference_identifier_1_declared_after_it:n(2373,e.DiagnosticCategory.Error,\"Initializer_of_parameter_0_cannot_reference_identifier_1_declared_after_it_2373\",\"Initializer of parameter '{0}' cannot reference identifier '{1}' declared after it.\"),Duplicate_string_index_signature:n(2374,e.DiagnosticCategory.Error,\"Duplicate_string_index_signature_2374\",\"Duplicate string index signature.\"),Duplicate_number_index_signature:n(2375,e.DiagnosticCategory.Error,\"Duplicate_number_index_signature_2375\",\"Duplicate number index signature.\"),A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_properties_or_has_parameter_properties:n(2376,e.DiagnosticCategory.Error,\"A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_proper_2376\",\"A 'super' call must be the first statement in the constructor when a class contains initialized properties or has parameter properties.\"),Constructors_for_derived_classes_must_contain_a_super_call:n(2377,e.DiagnosticCategory.Error,\"Constructors_for_derived_classes_must_contain_a_super_call_2377\",\"Constructors for derived classes must contain a 'super' call.\"),A_get_accessor_must_return_a_value:n(2378,e.DiagnosticCategory.Error,\"A_get_accessor_must_return_a_value_2378\",\"A 'get' accessor must return a value.\"),Getter_and_setter_accessors_do_not_agree_in_visibility:n(2379,e.DiagnosticCategory.Error,\"Getter_and_setter_accessors_do_not_agree_in_visibility_2379\",\"Getter and setter accessors do not agree in visibility.\"),get_and_set_accessor_must_have_the_same_type:n(2380,e.DiagnosticCategory.Error,\"get_and_set_accessor_must_have_the_same_type_2380\",\"'get' and 'set' accessor must have the same type.\"),A_signature_with_an_implementation_cannot_use_a_string_literal_type:n(2381,e.DiagnosticCategory.Error,\"A_signature_with_an_implementation_cannot_use_a_string_literal_type_2381\",\"A signature with an implementation cannot use a string literal type.\"),Specialized_overload_signature_is_not_assignable_to_any_non_specialized_signature:n(2382,e.DiagnosticCategory.Error,\"Specialized_overload_signature_is_not_assignable_to_any_non_specialized_signature_2382\",\"Specialized overload signature is not assignable to any non-specialized signature.\"),Overload_signatures_must_all_be_exported_or_non_exported:n(2383,e.DiagnosticCategory.Error,\"Overload_signatures_must_all_be_exported_or_non_exported_2383\",\"Overload signatures must all be exported or non-exported.\"),Overload_signatures_must_all_be_ambient_or_non_ambient:n(2384,e.DiagnosticCategory.Error,\"Overload_signatures_must_all_be_ambient_or_non_ambient_2384\",\"Overload signatures must all be ambient or non-ambient.\"),Overload_signatures_must_all_be_public_private_or_protected:n(2385,e.DiagnosticCategory.Error,\"Overload_signatures_must_all_be_public_private_or_protected_2385\",\"Overload signatures must all be public, private or protected.\"),Overload_signatures_must_all_be_optional_or_required:n(2386,e.DiagnosticCategory.Error,\"Overload_signatures_must_all_be_optional_or_required_2386\",\"Overload signatures must all be optional or required.\"),Function_overload_must_be_static:n(2387,e.DiagnosticCategory.Error,\"Function_overload_must_be_static_2387\",\"Function overload must be static.\"),Function_overload_must_not_be_static:n(2388,e.DiagnosticCategory.Error,\"Function_overload_must_not_be_static_2388\",\"Function overload must not be static.\"),Function_implementation_name_must_be_0:n(2389,e.DiagnosticCategory.Error,\"Function_implementation_name_must_be_0_2389\",\"Function implementation name must be '{0}'.\"),Constructor_implementation_is_missing:n(2390,e.DiagnosticCategory.Error,\"Constructor_implementation_is_missing_2390\",\"Constructor implementation is missing.\"),Function_implementation_is_missing_or_not_immediately_following_the_declaration:n(2391,e.DiagnosticCategory.Error,\"Function_implementation_is_missing_or_not_immediately_following_the_declaration_2391\",\"Function implementation is missing or not immediately following the declaration.\"),Multiple_constructor_implementations_are_not_allowed:n(2392,e.DiagnosticCategory.Error,\"Multiple_constructor_implementations_are_not_allowed_2392\",\"Multiple constructor implementations are not allowed.\"),Duplicate_function_implementation:n(2393,e.DiagnosticCategory.Error,\"Duplicate_function_implementation_2393\",\"Duplicate function implementation.\"),Overload_signature_is_not_compatible_with_function_implementation:n(2394,e.DiagnosticCategory.Error,\"Overload_signature_is_not_compatible_with_function_implementation_2394\",\"Overload signature is not compatible with function implementation.\"),Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local:n(2395,e.DiagnosticCategory.Error,\"Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local_2395\",\"Individual declarations in merged declaration '{0}' must be all exported or all local.\"),Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters:n(2396,e.DiagnosticCategory.Error,\"Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters_2396\",\"Duplicate identifier 'arguments'. Compiler uses 'arguments' to initialize rest parameters.\"),Declaration_name_conflicts_with_built_in_global_identifier_0:n(2397,e.DiagnosticCategory.Error,\"Declaration_name_conflicts_with_built_in_global_identifier_0_2397\",\"Declaration name conflicts with built-in global identifier '{0}'.\"),Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference:n(2399,e.DiagnosticCategory.Error,\"Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference_2399\",\"Duplicate identifier '_this'. Compiler uses variable declaration '_this' to capture 'this' reference.\"),Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference:n(2400,e.DiagnosticCategory.Error,\"Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference_2400\",\"Expression resolves to variable declaration '_this' that compiler uses to capture 'this' reference.\"),Duplicate_identifier_super_Compiler_uses_super_to_capture_base_class_reference:n(2401,e.DiagnosticCategory.Error,\"Duplicate_identifier_super_Compiler_uses_super_to_capture_base_class_reference_2401\",\"Duplicate identifier '_super'. Compiler uses '_super' to capture base class reference.\"),Expression_resolves_to_super_that_compiler_uses_to_capture_base_class_reference:n(2402,e.DiagnosticCategory.Error,\"Expression_resolves_to_super_that_compiler_uses_to_capture_base_class_reference_2402\",\"Expression resolves to '_super' that compiler uses to capture base class reference.\"),Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_type_2:n(2403,e.DiagnosticCategory.Error,\"Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_t_2403\",\"Subsequent variable declarations must have the same type.  Variable '{0}' must be of type '{1}', but here has type '{2}'.\"),The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation:n(2404,e.DiagnosticCategory.Error,\"The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation_2404\",\"The left-hand side of a 'for...in' statement cannot use a type annotation.\"),The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any:n(2405,e.DiagnosticCategory.Error,\"The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any_2405\",\"The left-hand side of a 'for...in' statement must be of type 'string' or 'any'.\"),The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access:n(2406,e.DiagnosticCategory.Error,\"The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access_2406\",\"The left-hand side of a 'for...in' statement must be a variable or a property access.\"),The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_here_has_type_0:n(2407,e.DiagnosticCategory.Error,\"The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_2407\",\"The right-hand side of a 'for...in' statement must be of type 'any', an object type or a type parameter, but here has type '{0}'.\"),Setters_cannot_return_a_value:n(2408,e.DiagnosticCategory.Error,\"Setters_cannot_return_a_value_2408\",\"Setters cannot return a value.\"),Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class:n(2409,e.DiagnosticCategory.Error,\"Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class_2409\",\"Return type of constructor signature must be assignable to the instance type of the class.\"),The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any:n(2410,e.DiagnosticCategory.Error,\"The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any_2410\",\"The 'with' statement is not supported. All symbols in a 'with' block will have type 'any'.\"),Property_0_of_type_1_is_not_assignable_to_string_index_type_2:n(2411,e.DiagnosticCategory.Error,\"Property_0_of_type_1_is_not_assignable_to_string_index_type_2_2411\",\"Property '{0}' of type '{1}' is not assignable to string index type '{2}'.\"),Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2:n(2412,e.DiagnosticCategory.Error,\"Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2_2412\",\"Property '{0}' of type '{1}' is not assignable to numeric index type '{2}'.\"),Numeric_index_type_0_is_not_assignable_to_string_index_type_1:n(2413,e.DiagnosticCategory.Error,\"Numeric_index_type_0_is_not_assignable_to_string_index_type_1_2413\",\"Numeric index type '{0}' is not assignable to string index type '{1}'.\"),Class_name_cannot_be_0:n(2414,e.DiagnosticCategory.Error,\"Class_name_cannot_be_0_2414\",\"Class name cannot be '{0}'.\"),Class_0_incorrectly_extends_base_class_1:n(2415,e.DiagnosticCategory.Error,\"Class_0_incorrectly_extends_base_class_1_2415\",\"Class '{0}' incorrectly extends base class '{1}'.\"),Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2:n(2416,e.DiagnosticCategory.Error,\"Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2_2416\",\"Property '{0}' in type '{1}' is not assignable to the same property in base type '{2}'.\"),Class_static_side_0_incorrectly_extends_base_class_static_side_1:n(2417,e.DiagnosticCategory.Error,\"Class_static_side_0_incorrectly_extends_base_class_static_side_1_2417\",\"Class static side '{0}' incorrectly extends base class static side '{1}'.\"),Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1:n(2418,e.DiagnosticCategory.Error,\"Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1_2418\",\"Type of computed property's value is '{0}', which is not assignable to type '{1}'.\"),Class_0_incorrectly_implements_interface_1:n(2420,e.DiagnosticCategory.Error,\"Class_0_incorrectly_implements_interface_1_2420\",\"Class '{0}' incorrectly implements interface '{1}'.\"),A_class_may_only_implement_another_class_or_interface:n(2422,e.DiagnosticCategory.Error,\"A_class_may_only_implement_another_class_or_interface_2422\",\"A class may only implement another class or interface.\"),Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_accessor:n(2423,e.DiagnosticCategory.Error,\"Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_access_2423\",\"Class '{0}' defines instance member function '{1}', but extended class '{2}' defines it as instance member accessor.\"),Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_property:n(2424,e.DiagnosticCategory.Error,\"Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_proper_2424\",\"Class '{0}' defines instance member function '{1}', but extended class '{2}' defines it as instance member property.\"),Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_function:n(2425,e.DiagnosticCategory.Error,\"Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_functi_2425\",\"Class '{0}' defines instance member property '{1}', but extended class '{2}' defines it as instance member function.\"),Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_function:n(2426,e.DiagnosticCategory.Error,\"Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_functi_2426\",\"Class '{0}' defines instance member accessor '{1}', but extended class '{2}' defines it as instance member function.\"),Interface_name_cannot_be_0:n(2427,e.DiagnosticCategory.Error,\"Interface_name_cannot_be_0_2427\",\"Interface name cannot be '{0}'.\"),All_declarations_of_0_must_have_identical_type_parameters:n(2428,e.DiagnosticCategory.Error,\"All_declarations_of_0_must_have_identical_type_parameters_2428\",\"All declarations of '{0}' must have identical type parameters.\"),Interface_0_incorrectly_extends_interface_1:n(2430,e.DiagnosticCategory.Error,\"Interface_0_incorrectly_extends_interface_1_2430\",\"Interface '{0}' incorrectly extends interface '{1}'.\"),Enum_name_cannot_be_0:n(2431,e.DiagnosticCategory.Error,\"Enum_name_cannot_be_0_2431\",\"Enum name cannot be '{0}'.\"),In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enum_element:n(2432,e.DiagnosticCategory.Error,\"In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enu_2432\",\"In an enum with multiple declarations, only one declaration can omit an initializer for its first enum element.\"),A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merged:n(2433,e.DiagnosticCategory.Error,\"A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merg_2433\",\"A namespace declaration cannot be in a different file from a class or function with which it is merged.\"),A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged:n(2434,e.DiagnosticCategory.Error,\"A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged_2434\",\"A namespace declaration cannot be located prior to a class or function with which it is merged.\"),Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces:n(2435,e.DiagnosticCategory.Error,\"Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces_2435\",\"Ambient modules cannot be nested in other modules or namespaces.\"),Ambient_module_declaration_cannot_specify_relative_module_name:n(2436,e.DiagnosticCategory.Error,\"Ambient_module_declaration_cannot_specify_relative_module_name_2436\",\"Ambient module declaration cannot specify relative module name.\"),Module_0_is_hidden_by_a_local_declaration_with_the_same_name:n(2437,e.DiagnosticCategory.Error,\"Module_0_is_hidden_by_a_local_declaration_with_the_same_name_2437\",\"Module '{0}' is hidden by a local declaration with the same name.\"),Import_name_cannot_be_0:n(2438,e.DiagnosticCategory.Error,\"Import_name_cannot_be_0_2438\",\"Import name cannot be '{0}'.\"),Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relative_module_name:n(2439,e.DiagnosticCategory.Error,\"Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relati_2439\",\"Import or export declaration in an ambient module declaration cannot reference module through relative module name.\"),Import_declaration_conflicts_with_local_declaration_of_0:n(2440,e.DiagnosticCategory.Error,\"Import_declaration_conflicts_with_local_declaration_of_0_2440\",\"Import declaration conflicts with local declaration of '{0}'.\"),Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module:n(2441,e.DiagnosticCategory.Error,\"Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_2441\",\"Duplicate identifier '{0}'. Compiler reserves name '{1}' in top level scope of a module.\"),Types_have_separate_declarations_of_a_private_property_0:n(2442,e.DiagnosticCategory.Error,\"Types_have_separate_declarations_of_a_private_property_0_2442\",\"Types have separate declarations of a private property '{0}'.\"),Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2:n(2443,e.DiagnosticCategory.Error,\"Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2_2443\",\"Property '{0}' is protected but type '{1}' is not a class derived from '{2}'.\"),Property_0_is_protected_in_type_1_but_public_in_type_2:n(2444,e.DiagnosticCategory.Error,\"Property_0_is_protected_in_type_1_but_public_in_type_2_2444\",\"Property '{0}' is protected in type '{1}' but public in type '{2}'.\"),Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses:n(2445,e.DiagnosticCategory.Error,\"Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses_2445\",\"Property '{0}' is protected and only accessible within class '{1}' and its subclasses.\"),Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1:n(2446,e.DiagnosticCategory.Error,\"Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1_2446\",\"Property '{0}' is protected and only accessible through an instance of class '{1}'.\"),The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead:n(2447,e.DiagnosticCategory.Error,\"The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead_2447\",\"The '{0}' operator is not allowed for boolean types. Consider using '{1}' instead.\"),Block_scoped_variable_0_used_before_its_declaration:n(2448,e.DiagnosticCategory.Error,\"Block_scoped_variable_0_used_before_its_declaration_2448\",\"Block-scoped variable '{0}' used before its declaration.\"),Class_0_used_before_its_declaration:n(2449,e.DiagnosticCategory.Error,\"Class_0_used_before_its_declaration_2449\",\"Class '{0}' used before its declaration.\"),Enum_0_used_before_its_declaration:n(2450,e.DiagnosticCategory.Error,\"Enum_0_used_before_its_declaration_2450\",\"Enum '{0}' used before its declaration.\"),Cannot_redeclare_block_scoped_variable_0:n(2451,e.DiagnosticCategory.Error,\"Cannot_redeclare_block_scoped_variable_0_2451\",\"Cannot redeclare block-scoped variable '{0}'.\"),An_enum_member_cannot_have_a_numeric_name:n(2452,e.DiagnosticCategory.Error,\"An_enum_member_cannot_have_a_numeric_name_2452\",\"An enum member cannot have a numeric name.\"),The_type_argument_for_type_parameter_0_cannot_be_inferred_from_the_usage_Consider_specifying_the_type_arguments_explicitly:n(2453,e.DiagnosticCategory.Error,\"The_type_argument_for_type_parameter_0_cannot_be_inferred_from_the_usage_Consider_specifying_the_typ_2453\",\"The type argument for type parameter '{0}' cannot be inferred from the usage. Consider specifying the type arguments explicitly.\"),Variable_0_is_used_before_being_assigned:n(2454,e.DiagnosticCategory.Error,\"Variable_0_is_used_before_being_assigned_2454\",\"Variable '{0}' is used before being assigned.\"),Type_argument_candidate_1_is_not_a_valid_type_argument_because_it_is_not_a_supertype_of_candidate_0:n(2455,e.DiagnosticCategory.Error,\"Type_argument_candidate_1_is_not_a_valid_type_argument_because_it_is_not_a_supertype_of_candidate_0_2455\",\"Type argument candidate '{1}' is not a valid type argument because it is not a supertype of candidate '{0}'.\"),Type_alias_0_circularly_references_itself:n(2456,e.DiagnosticCategory.Error,\"Type_alias_0_circularly_references_itself_2456\",\"Type alias '{0}' circularly references itself.\"),Type_alias_name_cannot_be_0:n(2457,e.DiagnosticCategory.Error,\"Type_alias_name_cannot_be_0_2457\",\"Type alias name cannot be '{0}'.\"),An_AMD_module_cannot_have_multiple_name_assignments:n(2458,e.DiagnosticCategory.Error,\"An_AMD_module_cannot_have_multiple_name_assignments_2458\",\"An AMD module cannot have multiple name assignments.\"),Type_0_has_no_property_1_and_no_string_index_signature:n(2459,e.DiagnosticCategory.Error,\"Type_0_has_no_property_1_and_no_string_index_signature_2459\",\"Type '{0}' has no property '{1}' and no string index signature.\"),Type_0_has_no_property_1:n(2460,e.DiagnosticCategory.Error,\"Type_0_has_no_property_1_2460\",\"Type '{0}' has no property '{1}'.\"),Type_0_is_not_an_array_type:n(2461,e.DiagnosticCategory.Error,\"Type_0_is_not_an_array_type_2461\",\"Type '{0}' is not an array type.\"),A_rest_element_must_be_last_in_a_destructuring_pattern:n(2462,e.DiagnosticCategory.Error,\"A_rest_element_must_be_last_in_a_destructuring_pattern_2462\",\"A rest element must be last in a destructuring pattern.\"),A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature:n(2463,e.DiagnosticCategory.Error,\"A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature_2463\",\"A binding pattern parameter cannot be optional in an implementation signature.\"),A_computed_property_name_must_be_of_type_string_number_symbol_or_any:n(2464,e.DiagnosticCategory.Error,\"A_computed_property_name_must_be_of_type_string_number_symbol_or_any_2464\",\"A computed property name must be of type 'string', 'number', 'symbol', or 'any'.\"),this_cannot_be_referenced_in_a_computed_property_name:n(2465,e.DiagnosticCategory.Error,\"this_cannot_be_referenced_in_a_computed_property_name_2465\",\"'this' cannot be referenced in a computed property name.\"),super_cannot_be_referenced_in_a_computed_property_name:n(2466,e.DiagnosticCategory.Error,\"super_cannot_be_referenced_in_a_computed_property_name_2466\",\"'super' cannot be referenced in a computed property name.\"),A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type:n(2467,e.DiagnosticCategory.Error,\"A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type_2467\",\"A computed property name cannot reference a type parameter from its containing type.\"),Cannot_find_global_value_0:n(2468,e.DiagnosticCategory.Error,\"Cannot_find_global_value_0_2468\",\"Cannot find global value '{0}'.\"),The_0_operator_cannot_be_applied_to_type_symbol:n(2469,e.DiagnosticCategory.Error,\"The_0_operator_cannot_be_applied_to_type_symbol_2469\",\"The '{0}' operator cannot be applied to type 'symbol'.\"),Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object:n(2470,e.DiagnosticCategory.Error,\"Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object_2470\",\"'Symbol' reference does not refer to the global Symbol constructor object.\"),A_computed_property_name_of_the_form_0_must_be_of_type_symbol:n(2471,e.DiagnosticCategory.Error,\"A_computed_property_name_of_the_form_0_must_be_of_type_symbol_2471\",\"A computed property name of the form '{0}' must be of type 'symbol'.\"),Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher:n(2472,e.DiagnosticCategory.Error,\"Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher_2472\",\"Spread operator in 'new' expressions is only available when targeting ECMAScript 5 and higher.\"),Enum_declarations_must_all_be_const_or_non_const:n(2473,e.DiagnosticCategory.Error,\"Enum_declarations_must_all_be_const_or_non_const_2473\",\"Enum declarations must all be const or non-const.\"),In_const_enum_declarations_member_initializer_must_be_constant_expression:n(2474,e.DiagnosticCategory.Error,\"In_const_enum_declarations_member_initializer_must_be_constant_expression_2474\",\"In 'const' enum declarations member initializer must be constant expression.\"),const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_import_declaration_or_export_assignment_or_type_query:n(2475,e.DiagnosticCategory.Error,\"const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_im_2475\",\"'const' enums can only be used in property or index access expressions or the right hand side of an import declaration or export assignment or type query.\"),A_const_enum_member_can_only_be_accessed_using_a_string_literal:n(2476,e.DiagnosticCategory.Error,\"A_const_enum_member_can_only_be_accessed_using_a_string_literal_2476\",\"A const enum member can only be accessed using a string literal.\"),const_enum_member_initializer_was_evaluated_to_a_non_finite_value:n(2477,e.DiagnosticCategory.Error,\"const_enum_member_initializer_was_evaluated_to_a_non_finite_value_2477\",\"'const' enum member initializer was evaluated to a non-finite value.\"),const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN:n(2478,e.DiagnosticCategory.Error,\"const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN_2478\",\"'const' enum member initializer was evaluated to disallowed value 'NaN'.\"),Property_0_does_not_exist_on_const_enum_1:n(2479,e.DiagnosticCategory.Error,\"Property_0_does_not_exist_on_const_enum_1_2479\",\"Property '{0}' does not exist on 'const' enum '{1}'.\"),let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations:n(2480,e.DiagnosticCategory.Error,\"let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations_2480\",\"'let' is not allowed to be used as a name in 'let' or 'const' declarations.\"),Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1:n(2481,e.DiagnosticCategory.Error,\"Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1_2481\",\"Cannot initialize outer scoped variable '{0}' in the same scope as block scoped declaration '{1}'.\"),The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation:n(2483,e.DiagnosticCategory.Error,\"The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation_2483\",\"The left-hand side of a 'for...of' statement cannot use a type annotation.\"),Export_declaration_conflicts_with_exported_declaration_of_0:n(2484,e.DiagnosticCategory.Error,\"Export_declaration_conflicts_with_exported_declaration_of_0_2484\",\"Export declaration conflicts with exported declaration of '{0}'.\"),The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access:n(2487,e.DiagnosticCategory.Error,\"The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access_2487\",\"The left-hand side of a 'for...of' statement must be a variable or a property access.\"),Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator:n(2488,e.DiagnosticCategory.Error,\"Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator_2488\",\"Type '{0}' must have a '[Symbol.iterator]()' method that returns an iterator.\"),An_iterator_must_have_a_next_method:n(2489,e.DiagnosticCategory.Error,\"An_iterator_must_have_a_next_method_2489\",\"An iterator must have a 'next()' method.\"),The_type_returned_by_the_next_method_of_an_iterator_must_have_a_value_property:n(2490,e.DiagnosticCategory.Error,\"The_type_returned_by_the_next_method_of_an_iterator_must_have_a_value_property_2490\",\"The type returned by the 'next()' method of an iterator must have a 'value' property.\"),The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern:n(2491,e.DiagnosticCategory.Error,\"The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern_2491\",\"The left-hand side of a 'for...in' statement cannot be a destructuring pattern.\"),Cannot_redeclare_identifier_0_in_catch_clause:n(2492,e.DiagnosticCategory.Error,\"Cannot_redeclare_identifier_0_in_catch_clause_2492\",\"Cannot redeclare identifier '{0}' in catch clause.\"),Tuple_type_0_with_length_1_cannot_be_assigned_to_tuple_with_length_2:n(2493,e.DiagnosticCategory.Error,\"Tuple_type_0_with_length_1_cannot_be_assigned_to_tuple_with_length_2_2493\",\"Tuple type '{0}' with length '{1}' cannot be assigned to tuple with length '{2}'.\"),Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher:n(2494,e.DiagnosticCategory.Error,\"Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher_2494\",\"Using a string in a 'for...of' statement is only supported in ECMAScript 5 and higher.\"),Type_0_is_not_an_array_type_or_a_string_type:n(2495,e.DiagnosticCategory.Error,\"Type_0_is_not_an_array_type_or_a_string_type_2495\",\"Type '{0}' is not an array type or a string type.\"),The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_standard_function_expression:n(2496,e.DiagnosticCategory.Error,\"The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_stand_2496\",\"The 'arguments' object cannot be referenced in an arrow function in ES3 and ES5. Consider using a standard function expression.\"),Module_0_resolves_to_a_non_module_entity_and_cannot_be_imported_using_this_construct:n(2497,e.DiagnosticCategory.Error,\"Module_0_resolves_to_a_non_module_entity_and_cannot_be_imported_using_this_construct_2497\",\"Module '{0}' resolves to a non-module entity and cannot be imported using this construct.\"),Module_0_uses_export_and_cannot_be_used_with_export_Asterisk:n(2498,e.DiagnosticCategory.Error,\"Module_0_uses_export_and_cannot_be_used_with_export_Asterisk_2498\",\"Module '{0}' uses 'export =' and cannot be used with 'export *'.\"),An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments:n(2499,e.DiagnosticCategory.Error,\"An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments_2499\",\"An interface can only extend an identifier/qualified-name with optional type arguments.\"),A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments:n(2500,e.DiagnosticCategory.Error,\"A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments_2500\",\"A class can only implement an identifier/qualified-name with optional type arguments.\"),A_rest_element_cannot_contain_a_binding_pattern:n(2501,e.DiagnosticCategory.Error,\"A_rest_element_cannot_contain_a_binding_pattern_2501\",\"A rest element cannot contain a binding pattern.\"),_0_is_referenced_directly_or_indirectly_in_its_own_type_annotation:n(2502,e.DiagnosticCategory.Error,\"_0_is_referenced_directly_or_indirectly_in_its_own_type_annotation_2502\",\"'{0}' is referenced directly or indirectly in its own type annotation.\"),Cannot_find_namespace_0:n(2503,e.DiagnosticCategory.Error,\"Cannot_find_namespace_0_2503\",\"Cannot find namespace '{0}'.\"),Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator:n(2504,e.DiagnosticCategory.Error,\"Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator_2504\",\"Type '{0}' must have a '[Symbol.asyncIterator]()' method that returns an async iterator.\"),A_generator_cannot_have_a_void_type_annotation:n(2505,e.DiagnosticCategory.Error,\"A_generator_cannot_have_a_void_type_annotation_2505\",\"A generator cannot have a 'void' type annotation.\"),_0_is_referenced_directly_or_indirectly_in_its_own_base_expression:n(2506,e.DiagnosticCategory.Error,\"_0_is_referenced_directly_or_indirectly_in_its_own_base_expression_2506\",\"'{0}' is referenced directly or indirectly in its own base expression.\"),Type_0_is_not_a_constructor_function_type:n(2507,e.DiagnosticCategory.Error,\"Type_0_is_not_a_constructor_function_type_2507\",\"Type '{0}' is not a constructor function type.\"),No_base_constructor_has_the_specified_number_of_type_arguments:n(2508,e.DiagnosticCategory.Error,\"No_base_constructor_has_the_specified_number_of_type_arguments_2508\",\"No base constructor has the specified number of type arguments.\"),Base_constructor_return_type_0_is_not_a_class_or_interface_type:n(2509,e.DiagnosticCategory.Error,\"Base_constructor_return_type_0_is_not_a_class_or_interface_type_2509\",\"Base constructor return type '{0}' is not a class or interface type.\"),Base_constructors_must_all_have_the_same_return_type:n(2510,e.DiagnosticCategory.Error,\"Base_constructors_must_all_have_the_same_return_type_2510\",\"Base constructors must all have the same return type.\"),Cannot_create_an_instance_of_an_abstract_class:n(2511,e.DiagnosticCategory.Error,\"Cannot_create_an_instance_of_an_abstract_class_2511\",\"Cannot create an instance of an abstract class.\"),Overload_signatures_must_all_be_abstract_or_non_abstract:n(2512,e.DiagnosticCategory.Error,\"Overload_signatures_must_all_be_abstract_or_non_abstract_2512\",\"Overload signatures must all be abstract or non-abstract.\"),Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression:n(2513,e.DiagnosticCategory.Error,\"Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression_2513\",\"Abstract method '{0}' in class '{1}' cannot be accessed via super expression.\"),Classes_containing_abstract_methods_must_be_marked_abstract:n(2514,e.DiagnosticCategory.Error,\"Classes_containing_abstract_methods_must_be_marked_abstract_2514\",\"Classes containing abstract methods must be marked abstract.\"),Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2:n(2515,e.DiagnosticCategory.Error,\"Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2_2515\",\"Non-abstract class '{0}' does not implement inherited abstract member '{1}' from class '{2}'.\"),All_declarations_of_an_abstract_method_must_be_consecutive:n(2516,e.DiagnosticCategory.Error,\"All_declarations_of_an_abstract_method_must_be_consecutive_2516\",\"All declarations of an abstract method must be consecutive.\"),Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type:n(2517,e.DiagnosticCategory.Error,\"Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type_2517\",\"Cannot assign an abstract constructor type to a non-abstract constructor type.\"),A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard:n(2518,e.DiagnosticCategory.Error,\"A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard_2518\",\"A 'this'-based type guard is not compatible with a parameter-based type guard.\"),An_async_iterator_must_have_a_next_method:n(2519,e.DiagnosticCategory.Error,\"An_async_iterator_must_have_a_next_method_2519\",\"An async iterator must have a 'next()' method.\"),Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions:n(2520,e.DiagnosticCategory.Error,\"Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions_2520\",\"Duplicate identifier '{0}'. Compiler uses declaration '{1}' to support async functions.\"),Expression_resolves_to_variable_declaration_0_that_compiler_uses_to_support_async_functions:n(2521,e.DiagnosticCategory.Error,\"Expression_resolves_to_variable_declaration_0_that_compiler_uses_to_support_async_functions_2521\",\"Expression resolves to variable declaration '{0}' that compiler uses to support async functions.\"),The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_using_a_standard_function_or_method:n(2522,e.DiagnosticCategory.Error,\"The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_usi_2522\",\"The 'arguments' object cannot be referenced in an async function or method in ES3 and ES5. Consider using a standard function or method.\"),yield_expressions_cannot_be_used_in_a_parameter_initializer:n(2523,e.DiagnosticCategory.Error,\"yield_expressions_cannot_be_used_in_a_parameter_initializer_2523\",\"'yield' expressions cannot be used in a parameter initializer.\"),await_expressions_cannot_be_used_in_a_parameter_initializer:n(2524,e.DiagnosticCategory.Error,\"await_expressions_cannot_be_used_in_a_parameter_initializer_2524\",\"'await' expressions cannot be used in a parameter initializer.\"),Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value:n(2525,e.DiagnosticCategory.Error,\"Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value_2525\",\"Initializer provides no value for this binding element and the binding element has no default value.\"),A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface:n(2526,e.DiagnosticCategory.Error,\"A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface_2526\",\"A 'this' type is available only in a non-static member of a class or interface.\"),The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary:n(2527,e.DiagnosticCategory.Error,\"The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary_2527\",\"The inferred type of '{0}' references an inaccessible '{1}' type. A type annotation is necessary.\"),A_module_cannot_have_multiple_default_exports:n(2528,e.DiagnosticCategory.Error,\"A_module_cannot_have_multiple_default_exports_2528\",\"A module cannot have multiple default exports.\"),Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_functions:n(2529,e.DiagnosticCategory.Error,\"Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_func_2529\",\"Duplicate identifier '{0}'. Compiler reserves name '{1}' in top level scope of a module containing async functions.\"),Property_0_is_incompatible_with_index_signature:n(2530,e.DiagnosticCategory.Error,\"Property_0_is_incompatible_with_index_signature_2530\",\"Property '{0}' is incompatible with index signature.\"),Object_is_possibly_null:n(2531,e.DiagnosticCategory.Error,\"Object_is_possibly_null_2531\",\"Object is possibly 'null'.\"),Object_is_possibly_undefined:n(2532,e.DiagnosticCategory.Error,\"Object_is_possibly_undefined_2532\",\"Object is possibly 'undefined'.\"),Object_is_possibly_null_or_undefined:n(2533,e.DiagnosticCategory.Error,\"Object_is_possibly_null_or_undefined_2533\",\"Object is possibly 'null' or 'undefined'.\"),A_function_returning_never_cannot_have_a_reachable_end_point:n(2534,e.DiagnosticCategory.Error,\"A_function_returning_never_cannot_have_a_reachable_end_point_2534\",\"A function returning 'never' cannot have a reachable end point.\"),Enum_type_0_has_members_with_initializers_that_are_not_literals:n(2535,e.DiagnosticCategory.Error,\"Enum_type_0_has_members_with_initializers_that_are_not_literals_2535\",\"Enum type '{0}' has members with initializers that are not literals.\"),Type_0_cannot_be_used_to_index_type_1:n(2536,e.DiagnosticCategory.Error,\"Type_0_cannot_be_used_to_index_type_1_2536\",\"Type '{0}' cannot be used to index type '{1}'.\"),Type_0_has_no_matching_index_signature_for_type_1:n(2537,e.DiagnosticCategory.Error,\"Type_0_has_no_matching_index_signature_for_type_1_2537\",\"Type '{0}' has no matching index signature for type '{1}'.\"),Type_0_cannot_be_used_as_an_index_type:n(2538,e.DiagnosticCategory.Error,\"Type_0_cannot_be_used_as_an_index_type_2538\",\"Type '{0}' cannot be used as an index type.\"),Cannot_assign_to_0_because_it_is_not_a_variable:n(2539,e.DiagnosticCategory.Error,\"Cannot_assign_to_0_because_it_is_not_a_variable_2539\",\"Cannot assign to '{0}' because it is not a variable.\"),Cannot_assign_to_0_because_it_is_a_constant_or_a_read_only_property:n(2540,e.DiagnosticCategory.Error,\"Cannot_assign_to_0_because_it_is_a_constant_or_a_read_only_property_2540\",\"Cannot assign to '{0}' because it is a constant or a read-only property.\"),The_target_of_an_assignment_must_be_a_variable_or_a_property_access:n(2541,e.DiagnosticCategory.Error,\"The_target_of_an_assignment_must_be_a_variable_or_a_property_access_2541\",\"The target of an assignment must be a variable or a property access.\"),Index_signature_in_type_0_only_permits_reading:n(2542,e.DiagnosticCategory.Error,\"Index_signature_in_type_0_only_permits_reading_2542\",\"Index signature in type '{0}' only permits reading.\"),Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_meta_property_reference:n(2543,e.DiagnosticCategory.Error,\"Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_me_2543\",\"Duplicate identifier '_newTarget'. Compiler uses variable declaration '_newTarget' to capture 'new.target' meta-property reference.\"),Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta_property_reference:n(2544,e.DiagnosticCategory.Error,\"Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta__2544\",\"Expression resolves to variable declaration '_newTarget' that compiler uses to capture 'new.target' meta-property reference.\"),A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any:n(2545,e.DiagnosticCategory.Error,\"A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any_2545\",\"A mixin class must have a constructor with a single rest parameter of type 'any[]'.\"),Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1:n(2546,e.DiagnosticCategory.Error,\"Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1_2546\",\"Property '{0}' has conflicting declarations and is inaccessible in type '{1}'.\"),The_type_returned_by_the_next_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_property:n(2547,e.DiagnosticCategory.Error,\"The_type_returned_by_the_next_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value__2547\",\"The type returned by the 'next()' method of an async iterator must be a promise for a type with a 'value' property.\"),Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator:n(2548,e.DiagnosticCategory.Error,\"Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator_2548\",\"Type '{0}' is not an array type or does not have a '[Symbol.iterator]()' method that returns an iterator.\"),Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator:n(2549,e.DiagnosticCategory.Error,\"Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns__2549\",\"Type '{0}' is not an array type or a string type or does not have a '[Symbol.iterator]()' method that returns an iterator.\"),Property_0_does_not_exist_on_type_1_Did_you_mean_2:n(2551,e.DiagnosticCategory.Error,\"Property_0_does_not_exist_on_type_1_Did_you_mean_2_2551\",\"Property '{0}' does not exist on type '{1}'. Did you mean '{2}'?\"),Cannot_find_name_0_Did_you_mean_1:n(2552,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Did_you_mean_1_2552\",\"Cannot find name '{0}'. Did you mean '{1}'?\"),Computed_values_are_not_permitted_in_an_enum_with_string_valued_members:n(2553,e.DiagnosticCategory.Error,\"Computed_values_are_not_permitted_in_an_enum_with_string_valued_members_2553\",\"Computed values are not permitted in an enum with string valued members.\"),Expected_0_arguments_but_got_1:n(2554,e.DiagnosticCategory.Error,\"Expected_0_arguments_but_got_1_2554\",\"Expected {0} arguments, but got {1}.\"),Expected_at_least_0_arguments_but_got_1:n(2555,e.DiagnosticCategory.Error,\"Expected_at_least_0_arguments_but_got_1_2555\",\"Expected at least {0} arguments, but got {1}.\"),Expected_0_arguments_but_got_1_or_more:n(2556,e.DiagnosticCategory.Error,\"Expected_0_arguments_but_got_1_or_more_2556\",\"Expected {0} arguments, but got {1} or more.\"),Expected_at_least_0_arguments_but_got_1_or_more:n(2557,e.DiagnosticCategory.Error,\"Expected_at_least_0_arguments_but_got_1_or_more_2557\",\"Expected at least {0} arguments, but got {1} or more.\"),Expected_0_type_arguments_but_got_1:n(2558,e.DiagnosticCategory.Error,\"Expected_0_type_arguments_but_got_1_2558\",\"Expected {0} type arguments, but got {1}.\"),Type_0_has_no_properties_in_common_with_type_1:n(2559,e.DiagnosticCategory.Error,\"Type_0_has_no_properties_in_common_with_type_1_2559\",\"Type '{0}' has no properties in common with type '{1}'.\"),Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it:n(2560,e.DiagnosticCategory.Error,\"Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it_2560\",\"Value of type '{0}' has no properties in common with type '{1}'. Did you mean to call it?\"),Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_write_2:n(2561,e.DiagnosticCategory.Error,\"Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_writ_2561\",\"Object literal may only specify known properties, but '{0}' does not exist in type '{1}'. Did you mean to write '{2}'?\"),Base_class_expressions_cannot_reference_class_type_parameters:n(2562,e.DiagnosticCategory.Error,\"Base_class_expressions_cannot_reference_class_type_parameters_2562\",\"Base class expressions cannot reference class type parameters.\"),The_containing_function_or_module_body_is_too_large_for_control_flow_analysis:n(2563,e.DiagnosticCategory.Error,\"The_containing_function_or_module_body_is_too_large_for_control_flow_analysis_2563\",\"The containing function or module body is too large for control flow analysis.\"),Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor:n(2564,e.DiagnosticCategory.Error,\"Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor_2564\",\"Property '{0}' has no initializer and is not definitely assigned in the constructor.\"),Property_0_is_used_before_being_assigned:n(2565,e.DiagnosticCategory.Error,\"Property_0_is_used_before_being_assigned_2565\",\"Property '{0}' is used before being assigned.\"),A_rest_element_cannot_have_a_property_name:n(2566,e.DiagnosticCategory.Error,\"A_rest_element_cannot_have_a_property_name_2566\",\"A rest element cannot have a property name.\"),Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations:n(2567,e.DiagnosticCategory.Error,\"Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations_2567\",\"Enum declarations can only merge with namespace or other enum declarations.\"),Type_0_is_not_an_array_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators:n(2568,e.DiagnosticCategory.Error,\"Type_0_is_not_an_array_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators_2568\",\"Type '{0}' is not an array type. Use compiler option '--downlevelIteration' to allow iterating of iterators.\"),Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators:n(2569,e.DiagnosticCategory.Error,\"Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterati_2569\",\"Type '{0}' is not an array type or a string type. Use compiler option '--downlevelIteration' to allow iterating of iterators.\"),Property_0_does_not_exist_on_type_1_Did_you_forget_to_use_await:n(2570,e.DiagnosticCategory.Error,\"Property_0_does_not_exist_on_type_1_Did_you_forget_to_use_await_2570\",\"Property '{0}' does not exist on type '{1}'. Did you forget to use 'await'?\"),Object_is_of_type_unknown:n(2571,e.DiagnosticCategory.Error,\"Object_is_of_type_unknown_2571\",\"Object is of type 'unknown'.\"),Rest_signatures_are_incompatible:n(2572,e.DiagnosticCategory.Error,\"Rest_signatures_are_incompatible_2572\",\"Rest signatures are incompatible.\"),Property_0_is_incompatible_with_rest_element_type:n(2573,e.DiagnosticCategory.Error,\"Property_0_is_incompatible_with_rest_element_type_2573\",\"Property '{0}' is incompatible with rest element type.\"),A_rest_element_type_must_be_an_array_type:n(2574,e.DiagnosticCategory.Error,\"A_rest_element_type_must_be_an_array_type_2574\",\"A rest element type must be an array type.\"),No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments:n(2575,e.DiagnosticCategory.Error,\"No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments_2575\",\"No overload expects {0} arguments, but overloads do exist that expect either {1} or {2} arguments.\"),Property_0_is_a_static_member_of_type_1:n(2576,e.DiagnosticCategory.Error,\"Property_0_is_a_static_member_of_type_1_2576\",\"Property '{0}' is a static member of type '{1}'\"),Return_type_annotation_circularly_references_itself:n(2577,e.DiagnosticCategory.Error,\"Return_type_annotation_circularly_references_itself_2577\",\"Return type annotation circularly references itself.\"),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode:n(2580,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode_2580\",\"Cannot find name '{0}'. Do you need to install type definitions for node? Try `npm i @types/node`.\"),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery:n(2581,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery_2581\",\"Cannot find name '{0}'. Do you need to install type definitions for jQuery? Try `npm i @types/jquery`.\"),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashjest_or_npm_i_types_Slashmocha:n(2582,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashje_2582\",\"Cannot find name '{0}'. Do you need to install type definitions for a test runner? Try `npm i @types/jest` or `npm i @types/mocha`.\"),Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later:n(2583,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2583\",\"Cannot find name '{0}'. Do you need to change your target library? Try changing the `lib` compiler option to es2015 or later.\"),Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_include_dom:n(2584,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2584\",\"Cannot find name '{0}'. Do you need to change your target library? Try changing the `lib` compiler option to include 'dom'.\"),_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later:n(2585,e.DiagnosticCategory.Error,\"_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_2585\",\"'{0}' only refers to a type, but is being used as a value here. Do you need to change your target library? Try changing the `lib` compiler option to es2015 or later.\"),Enum_type_0_circularly_references_itself:n(2586,e.DiagnosticCategory.Error,\"Enum_type_0_circularly_references_itself_2586\",\"Enum type '{0}' circularly references itself.\"),JSDoc_type_0_circularly_references_itself:n(2587,e.DiagnosticCategory.Error,\"JSDoc_type_0_circularly_references_itself_2587\",\"JSDoc type '{0}' circularly references itself.\"),JSX_element_attributes_type_0_may_not_be_a_union_type:n(2600,e.DiagnosticCategory.Error,\"JSX_element_attributes_type_0_may_not_be_a_union_type_2600\",\"JSX element attributes type '{0}' may not be a union type.\"),The_return_type_of_a_JSX_element_constructor_must_return_an_object_type:n(2601,e.DiagnosticCategory.Error,\"The_return_type_of_a_JSX_element_constructor_must_return_an_object_type_2601\",\"The return type of a JSX element constructor must return an object type.\"),JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist:n(2602,e.DiagnosticCategory.Error,\"JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist_2602\",\"JSX element implicitly has type 'any' because the global type 'JSX.Element' does not exist.\"),Property_0_in_type_1_is_not_assignable_to_type_2:n(2603,e.DiagnosticCategory.Error,\"Property_0_in_type_1_is_not_assignable_to_type_2_2603\",\"Property '{0}' in type '{1}' is not assignable to type '{2}'.\"),JSX_element_type_0_does_not_have_any_construct_or_call_signatures:n(2604,e.DiagnosticCategory.Error,\"JSX_element_type_0_does_not_have_any_construct_or_call_signatures_2604\",\"JSX element type '{0}' does not have any construct or call signatures.\"),JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements:n(2605,e.DiagnosticCategory.Error,\"JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements_2605\",\"JSX element type '{0}' is not a constructor function for JSX elements.\"),Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property:n(2606,e.DiagnosticCategory.Error,\"Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property_2606\",\"Property '{0}' of JSX spread attribute is not assignable to target property.\"),JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property:n(2607,e.DiagnosticCategory.Error,\"JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property_2607\",\"JSX element class does not support attributes because it does not have a '{0}' property.\"),The_global_type_JSX_0_may_not_have_more_than_one_property:n(2608,e.DiagnosticCategory.Error,\"The_global_type_JSX_0_may_not_have_more_than_one_property_2608\",\"The global type 'JSX.{0}' may not have more than one property.\"),JSX_spread_child_must_be_an_array_type:n(2609,e.DiagnosticCategory.Error,\"JSX_spread_child_must_be_an_array_type_2609\",\"JSX spread child must be an array type.\"),Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity:n(2649,e.DiagnosticCategory.Error,\"Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity_2649\",\"Cannot augment module '{0}' with value exports because it resolves to a non-module entity.\"),A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_members_defined_in_other_enums:n(2651,e.DiagnosticCategory.Error,\"A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_memb_2651\",\"A member initializer in a enum declaration cannot reference members declared after it, including members defined in other enums.\"),Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_default_0_declaration_instead:n(2652,e.DiagnosticCategory.Error,\"Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_d_2652\",\"Merged declaration '{0}' cannot include a default export declaration. Consider adding a separate 'export default {0}' declaration instead.\"),Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1:n(2653,e.DiagnosticCategory.Error,\"Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1_2653\",\"Non-abstract class expression does not implement inherited abstract member '{0}' from class '{1}'.\"),Exported_external_package_typings_file_cannot_contain_tripleslash_references_Please_contact_the_package_author_to_update_the_package_definition:n(2654,e.DiagnosticCategory.Error,\"Exported_external_package_typings_file_cannot_contain_tripleslash_references_Please_contact_the_pack_2654\",\"Exported external package typings file cannot contain tripleslash references. Please contact the package author to update the package definition.\"),Exported_external_package_typings_file_0_is_not_a_module_Please_contact_the_package_author_to_update_the_package_definition:n(2656,e.DiagnosticCategory.Error,\"Exported_external_package_typings_file_0_is_not_a_module_Please_contact_the_package_author_to_update_2656\",\"Exported external package typings file '{0}' is not a module. Please contact the package author to update the package definition.\"),JSX_expressions_must_have_one_parent_element:n(2657,e.DiagnosticCategory.Error,\"JSX_expressions_must_have_one_parent_element_2657\",\"JSX expressions must have one parent element.\"),Type_0_provides_no_match_for_the_signature_1:n(2658,e.DiagnosticCategory.Error,\"Type_0_provides_no_match_for_the_signature_1_2658\",\"Type '{0}' provides no match for the signature '{1}'.\"),super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_higher:n(2659,e.DiagnosticCategory.Error,\"super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_highe_2659\",\"'super' is only allowed in members of object literal expressions when option 'target' is 'ES2015' or higher.\"),super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions:n(2660,e.DiagnosticCategory.Error,\"super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions_2660\",\"'super' can only be referenced in members of derived classes or object literal expressions.\"),Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module:n(2661,e.DiagnosticCategory.Error,\"Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module_2661\",\"Cannot export '{0}'. Only local declarations can be exported from a module.\"),Cannot_find_name_0_Did_you_mean_the_static_member_1_0:n(2662,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Did_you_mean_the_static_member_1_0_2662\",\"Cannot find name '{0}'. Did you mean the static member '{1}.{0}'?\"),Cannot_find_name_0_Did_you_mean_the_instance_member_this_0:n(2663,e.DiagnosticCategory.Error,\"Cannot_find_name_0_Did_you_mean_the_instance_member_this_0_2663\",\"Cannot find name '{0}'. Did you mean the instance member 'this.{0}'?\"),Invalid_module_name_in_augmentation_module_0_cannot_be_found:n(2664,e.DiagnosticCategory.Error,\"Invalid_module_name_in_augmentation_module_0_cannot_be_found_2664\",\"Invalid module name in augmentation, module '{0}' cannot be found.\"),Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augmented:n(2665,e.DiagnosticCategory.Error,\"Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augm_2665\",\"Invalid module name in augmentation. Module '{0}' resolves to an untyped module at '{1}', which cannot be augmented.\"),Exports_and_export_assignments_are_not_permitted_in_module_augmentations:n(2666,e.DiagnosticCategory.Error,\"Exports_and_export_assignments_are_not_permitted_in_module_augmentations_2666\",\"Exports and export assignments are not permitted in module augmentations.\"),Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_module:n(2667,e.DiagnosticCategory.Error,\"Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_mod_2667\",\"Imports are not permitted in module augmentations. Consider moving them to the enclosing external module.\"),export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always_visible:n(2668,e.DiagnosticCategory.Error,\"export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always__2668\",\"'export' modifier cannot be applied to ambient modules and module augmentations since they are always visible.\"),Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations:n(2669,e.DiagnosticCategory.Error,\"Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_2669\",\"Augmentations for the global scope can only be directly nested in external modules or ambient module declarations.\"),Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambient_context:n(2670,e.DiagnosticCategory.Error,\"Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambien_2670\",\"Augmentations for the global scope should have 'declare' modifier unless they appear in already ambient context.\"),Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity:n(2671,e.DiagnosticCategory.Error,\"Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity_2671\",\"Cannot augment module '{0}' because it resolves to a non-module entity.\"),Cannot_assign_a_0_constructor_type_to_a_1_constructor_type:n(2672,e.DiagnosticCategory.Error,\"Cannot_assign_a_0_constructor_type_to_a_1_constructor_type_2672\",\"Cannot assign a '{0}' constructor type to a '{1}' constructor type.\"),Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration:n(2673,e.DiagnosticCategory.Error,\"Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration_2673\",\"Constructor of class '{0}' is private and only accessible within the class declaration.\"),Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration:n(2674,e.DiagnosticCategory.Error,\"Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration_2674\",\"Constructor of class '{0}' is protected and only accessible within the class declaration.\"),Cannot_extend_a_class_0_Class_constructor_is_marked_as_private:n(2675,e.DiagnosticCategory.Error,\"Cannot_extend_a_class_0_Class_constructor_is_marked_as_private_2675\",\"Cannot extend a class '{0}'. Class constructor is marked as private.\"),Accessors_must_both_be_abstract_or_non_abstract:n(2676,e.DiagnosticCategory.Error,\"Accessors_must_both_be_abstract_or_non_abstract_2676\",\"Accessors must both be abstract or non-abstract.\"),A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type:n(2677,e.DiagnosticCategory.Error,\"A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type_2677\",\"A type predicate's type must be assignable to its parameter's type.\"),Type_0_is_not_comparable_to_type_1:n(2678,e.DiagnosticCategory.Error,\"Type_0_is_not_comparable_to_type_1_2678\",\"Type '{0}' is not comparable to type '{1}'.\"),A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void:n(2679,e.DiagnosticCategory.Error,\"A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void_2679\",\"A function that is called with the 'new' keyword cannot have a 'this' type that is 'void'.\"),A_0_parameter_must_be_the_first_parameter:n(2680,e.DiagnosticCategory.Error,\"A_0_parameter_must_be_the_first_parameter_2680\",\"A '{0}' parameter must be the first parameter.\"),A_constructor_cannot_have_a_this_parameter:n(2681,e.DiagnosticCategory.Error,\"A_constructor_cannot_have_a_this_parameter_2681\",\"A constructor cannot have a 'this' parameter.\"),get_and_set_accessor_must_have_the_same_this_type:n(2682,e.DiagnosticCategory.Error,\"get_and_set_accessor_must_have_the_same_this_type_2682\",\"'get' and 'set' accessor must have the same 'this' type.\"),this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation:n(2683,e.DiagnosticCategory.Error,\"this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_2683\",\"'this' implicitly has type 'any' because it does not have a type annotation.\"),The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1:n(2684,e.DiagnosticCategory.Error,\"The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1_2684\",\"The 'this' context of type '{0}' is not assignable to method's 'this' of type '{1}'.\"),The_this_types_of_each_signature_are_incompatible:n(2685,e.DiagnosticCategory.Error,\"The_this_types_of_each_signature_are_incompatible_2685\",\"The 'this' types of each signature are incompatible.\"),_0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead:n(2686,e.DiagnosticCategory.Error,\"_0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead_2686\",\"'{0}' refers to a UMD global, but the current file is a module. Consider adding an import instead.\"),All_declarations_of_0_must_have_identical_modifiers:n(2687,e.DiagnosticCategory.Error,\"All_declarations_of_0_must_have_identical_modifiers_2687\",\"All declarations of '{0}' must have identical modifiers.\"),Cannot_find_type_definition_file_for_0:n(2688,e.DiagnosticCategory.Error,\"Cannot_find_type_definition_file_for_0_2688\",\"Cannot find type definition file for '{0}'.\"),Cannot_extend_an_interface_0_Did_you_mean_implements:n(2689,e.DiagnosticCategory.Error,\"Cannot_extend_an_interface_0_Did_you_mean_implements_2689\",\"Cannot extend an interface '{0}'. Did you mean 'implements'?\"),An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead:n(2691,e.DiagnosticCategory.Error,\"An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead_2691\",\"An import path cannot end with a '{0}' extension. Consider importing '{1}' instead.\"),_0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible:n(2692,e.DiagnosticCategory.Error,\"_0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible_2692\",\"'{0}' is a primitive, but '{1}' is a wrapper object. Prefer using '{0}' when possible.\"),_0_only_refers_to_a_type_but_is_being_used_as_a_value_here:n(2693,e.DiagnosticCategory.Error,\"_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_2693\",\"'{0}' only refers to a type, but is being used as a value here.\"),Namespace_0_has_no_exported_member_1:n(2694,e.DiagnosticCategory.Error,\"Namespace_0_has_no_exported_member_1_2694\",\"Namespace '{0}' has no exported member '{1}'.\"),Left_side_of_comma_operator_is_unused_and_has_no_side_effects:n(2695,e.DiagnosticCategory.Error,\"Left_side_of_comma_operator_is_unused_and_has_no_side_effects_2695\",\"Left side of comma operator is unused and has no side effects.\",!0),The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead:n(2696,e.DiagnosticCategory.Error,\"The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead_2696\",\"The 'Object' type is assignable to very few other types. Did you mean to use the 'any' type instead?\"),An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option:n(2697,e.DiagnosticCategory.Error,\"An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_in_2697\",\"An async function or method must return a 'Promise'. Make sure you have a declaration for 'Promise' or include 'ES2015' in your `--lib` option.\"),Spread_types_may_only_be_created_from_object_types:n(2698,e.DiagnosticCategory.Error,\"Spread_types_may_only_be_created_from_object_types_2698\",\"Spread types may only be created from object types.\"),Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1:n(2699,e.DiagnosticCategory.Error,\"Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1_2699\",\"Static property '{0}' conflicts with built-in property 'Function.{0}' of constructor function '{1}'.\"),Rest_types_may_only_be_created_from_object_types:n(2700,e.DiagnosticCategory.Error,\"Rest_types_may_only_be_created_from_object_types_2700\",\"Rest types may only be created from object types.\"),The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access:n(2701,e.DiagnosticCategory.Error,\"The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access_2701\",\"The target of an object rest assignment must be a variable or a property access.\"),_0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here:n(2702,e.DiagnosticCategory.Error,\"_0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here_2702\",\"'{0}' only refers to a type, but is being used as a namespace here.\"),The_operand_of_a_delete_operator_must_be_a_property_reference:n(2703,e.DiagnosticCategory.Error,\"The_operand_of_a_delete_operator_must_be_a_property_reference_2703\",\"The operand of a delete operator must be a property reference.\"),The_operand_of_a_delete_operator_cannot_be_a_read_only_property:n(2704,e.DiagnosticCategory.Error,\"The_operand_of_a_delete_operator_cannot_be_a_read_only_property_2704\",\"The operand of a delete operator cannot be a read-only property.\"),An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option:n(2705,e.DiagnosticCategory.Error,\"An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_de_2705\",\"An async function or method in ES5/ES3 requires the 'Promise' constructor.  Make sure you have a declaration for the 'Promise' constructor or include 'ES2015' in your `--lib` option.\"),Required_type_parameters_may_not_follow_optional_type_parameters:n(2706,e.DiagnosticCategory.Error,\"Required_type_parameters_may_not_follow_optional_type_parameters_2706\",\"Required type parameters may not follow optional type parameters.\"),Generic_type_0_requires_between_1_and_2_type_arguments:n(2707,e.DiagnosticCategory.Error,\"Generic_type_0_requires_between_1_and_2_type_arguments_2707\",\"Generic type '{0}' requires between {1} and {2} type arguments.\"),Cannot_use_namespace_0_as_a_value:n(2708,e.DiagnosticCategory.Error,\"Cannot_use_namespace_0_as_a_value_2708\",\"Cannot use namespace '{0}' as a value.\"),Cannot_use_namespace_0_as_a_type:n(2709,e.DiagnosticCategory.Error,\"Cannot_use_namespace_0_as_a_type_2709\",\"Cannot use namespace '{0}' as a type.\"),_0_are_specified_twice_The_attribute_named_0_will_be_overwritten:n(2710,e.DiagnosticCategory.Error,\"_0_are_specified_twice_The_attribute_named_0_will_be_overwritten_2710\",\"'{0}' are specified twice. The attribute named '{0}' will be overwritten.\"),A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option:n(2711,e.DiagnosticCategory.Error,\"A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES20_2711\",\"A dynamic import call returns a 'Promise'. Make sure you have a declaration for 'Promise' or include 'ES2015' in your `--lib` option.\"),A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option:n(2712,e.DiagnosticCategory.Error,\"A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declarat_2712\",\"A dynamic import call in ES5/ES3 requires the 'Promise' constructor.  Make sure you have a declaration for the 'Promise' constructor or include 'ES2015' in your `--lib` option.\"),Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1:n(2713,e.DiagnosticCategory.Error,\"Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_p_2713\",\"Cannot access '{0}.{1}' because '{0}' is a type, but not a namespace. Did you mean to retrieve the type of the property '{1}' in '{0}' with '{0}[\\\"{1}\\\"]'?\"),The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context:n(2714,e.DiagnosticCategory.Error,\"The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context_2714\",\"The expression of an export assignment must be an identifier or qualified name in an ambient context.\"),Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor:n(2715,e.DiagnosticCategory.Error,\"Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor_2715\",\"Abstract property '{0}' in class '{1}' cannot be accessed in the constructor.\"),Type_parameter_0_has_a_circular_default:n(2716,e.DiagnosticCategory.Error,\"Type_parameter_0_has_a_circular_default_2716\",\"Type parameter '{0}' has a circular default.\"),Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_type_2:n(2717,e.DiagnosticCategory.Error,\"Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_t_2717\",\"Subsequent property declarations must have the same type.  Property '{0}' must be of type '{1}', but here has type '{2}'.\"),Duplicate_declaration_0:n(2718,e.DiagnosticCategory.Error,\"Duplicate_declaration_0_2718\",\"Duplicate declaration '{0}'.\"),Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated:n(2719,e.DiagnosticCategory.Error,\"Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated_2719\",\"Type '{0}' is not assignable to type '{1}'. Two different types with this name exist, but they are unrelated.\"),Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass:n(2720,e.DiagnosticCategory.Error,\"Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclas_2720\",\"Class '{0}' incorrectly implements class '{1}'. Did you mean to extend '{1}' and inherit its members as a subclass?\"),Cannot_invoke_an_object_which_is_possibly_null:n(2721,e.DiagnosticCategory.Error,\"Cannot_invoke_an_object_which_is_possibly_null_2721\",\"Cannot invoke an object which is possibly 'null'.\"),Cannot_invoke_an_object_which_is_possibly_undefined:n(2722,e.DiagnosticCategory.Error,\"Cannot_invoke_an_object_which_is_possibly_undefined_2722\",\"Cannot invoke an object which is possibly 'undefined'.\"),Cannot_invoke_an_object_which_is_possibly_null_or_undefined:n(2723,e.DiagnosticCategory.Error,\"Cannot_invoke_an_object_which_is_possibly_null_or_undefined_2723\",\"Cannot invoke an object which is possibly 'null' or 'undefined'.\"),Module_0_has_no_exported_member_1_Did_you_mean_2:n(2724,e.DiagnosticCategory.Error,\"Module_0_has_no_exported_member_1_Did_you_mean_2_2724\",\"Module '{0}' has no exported member '{1}'. Did you mean '{2}'?\"),Class_name_cannot_be_Object_when_targeting_ES5_with_module_0:n(2725,e.DiagnosticCategory.Error,\"Class_name_cannot_be_Object_when_targeting_ES5_with_module_0_2725\",\"Class name cannot be 'Object' when targeting ES5 with module {0}.\"),Cannot_find_lib_definition_for_0:n(2726,e.DiagnosticCategory.Error,\"Cannot_find_lib_definition_for_0_2726\",\"Cannot find lib definition for '{0}'.\"),Cannot_find_lib_definition_for_0_Did_you_mean_1:n(2727,e.DiagnosticCategory.Error,\"Cannot_find_lib_definition_for_0_Did_you_mean_1_2727\",\"Cannot find lib definition for '{0}'. Did you mean '{1}'?\"),_0_is_declared_here:n(2728,e.DiagnosticCategory.Message,\"_0_is_declared_here_2728\",\"'{0}' is declared here.\"),Property_0_is_used_before_its_initialization:n(2729,e.DiagnosticCategory.Error,\"Property_0_is_used_before_its_initialization_2729\",\"Property '{0}' is used before its initialization.\"),An_arrow_function_cannot_have_a_this_parameter:n(2730,e.DiagnosticCategory.Error,\"An_arrow_function_cannot_have_a_this_parameter_2730\",\"An arrow function cannot have a 'this' parameter.\"),Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_in_String:n(2731,e.DiagnosticCategory.Error,\"Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_i_2731\",\"Implicit conversion of a 'symbol' to a 'string' will fail at runtime. Consider wrapping this expression in 'String(...)'.\"),Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension:n(2732,e.DiagnosticCategory.Error,\"Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension_2732\",\"Cannot find module '{0}'. Consider using '--resolveJsonModule' to import module with '.json' extension\"),Index_0_is_out_of_bounds_in_tuple_of_length_1:n(2733,e.DiagnosticCategory.Error,\"Index_0_is_out_of_bounds_in_tuple_of_length_1_2733\",\"Index '{0}' is out-of-bounds in tuple of length {1}.\"),It_is_highly_likely_that_you_are_missing_a_semicolon:n(2734,e.DiagnosticCategory.Error,\"It_is_highly_likely_that_you_are_missing_a_semicolon_2734\",\"It is highly likely that you are missing a semicolon.\"),Import_declaration_0_is_using_private_name_1:n(4e3,e.DiagnosticCategory.Error,\"Import_declaration_0_is_using_private_name_1_4000\",\"Import declaration '{0}' is using private name '{1}'.\"),Type_parameter_0_of_exported_class_has_or_is_using_private_name_1:n(4002,e.DiagnosticCategory.Error,\"Type_parameter_0_of_exported_class_has_or_is_using_private_name_1_4002\",\"Type parameter '{0}' of exported class has or is using private name '{1}'.\"),Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1:n(4004,e.DiagnosticCategory.Error,\"Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1_4004\",\"Type parameter '{0}' of exported interface has or is using private name '{1}'.\"),Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1:n(4006,e.DiagnosticCategory.Error,\"Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1_4006\",\"Type parameter '{0}' of constructor signature from exported interface has or is using private name '{1}'.\"),Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1:n(4008,e.DiagnosticCategory.Error,\"Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1_4008\",\"Type parameter '{0}' of call signature from exported interface has or is using private name '{1}'.\"),Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:n(4010,e.DiagnosticCategory.Error,\"Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1_4010\",\"Type parameter '{0}' of public static method from exported class has or is using private name '{1}'.\"),Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:n(4012,e.DiagnosticCategory.Error,\"Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1_4012\",\"Type parameter '{0}' of public method from exported class has or is using private name '{1}'.\"),Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1:n(4014,e.DiagnosticCategory.Error,\"Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1_4014\",\"Type parameter '{0}' of method from exported interface has or is using private name '{1}'.\"),Type_parameter_0_of_exported_function_has_or_is_using_private_name_1:n(4016,e.DiagnosticCategory.Error,\"Type_parameter_0_of_exported_function_has_or_is_using_private_name_1_4016\",\"Type parameter '{0}' of exported function has or is using private name '{1}'.\"),Implements_clause_of_exported_class_0_has_or_is_using_private_name_1:n(4019,e.DiagnosticCategory.Error,\"Implements_clause_of_exported_class_0_has_or_is_using_private_name_1_4019\",\"Implements clause of exported class '{0}' has or is using private name '{1}'.\"),extends_clause_of_exported_class_0_has_or_is_using_private_name_1:n(4020,e.DiagnosticCategory.Error,\"extends_clause_of_exported_class_0_has_or_is_using_private_name_1_4020\",\"'extends' clause of exported class '{0}' has or is using private name '{1}'.\"),extends_clause_of_exported_interface_0_has_or_is_using_private_name_1:n(4022,e.DiagnosticCategory.Error,\"extends_clause_of_exported_interface_0_has_or_is_using_private_name_1_4022\",\"'extends' clause of exported interface '{0}' has or is using private name '{1}'.\"),Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4023,e.DiagnosticCategory.Error,\"Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4023\",\"Exported variable '{0}' has or is using name '{1}' from external module {2} but cannot be named.\"),Exported_variable_0_has_or_is_using_name_1_from_private_module_2:n(4024,e.DiagnosticCategory.Error,\"Exported_variable_0_has_or_is_using_name_1_from_private_module_2_4024\",\"Exported variable '{0}' has or is using name '{1}' from private module '{2}'.\"),Exported_variable_0_has_or_is_using_private_name_1:n(4025,e.DiagnosticCategory.Error,\"Exported_variable_0_has_or_is_using_private_name_1_4025\",\"Exported variable '{0}' has or is using private name '{1}'.\"),Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4026,e.DiagnosticCategory.Error,\"Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot__4026\",\"Public static property '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4027,e.DiagnosticCategory.Error,\"Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4027\",\"Public static property '{0}' of exported class has or is using name '{1}' from private module '{2}'.\"),Public_static_property_0_of_exported_class_has_or_is_using_private_name_1:n(4028,e.DiagnosticCategory.Error,\"Public_static_property_0_of_exported_class_has_or_is_using_private_name_1_4028\",\"Public static property '{0}' of exported class has or is using private name '{1}'.\"),Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4029,e.DiagnosticCategory.Error,\"Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_name_4029\",\"Public property '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4030,e.DiagnosticCategory.Error,\"Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4030\",\"Public property '{0}' of exported class has or is using name '{1}' from private module '{2}'.\"),Public_property_0_of_exported_class_has_or_is_using_private_name_1:n(4031,e.DiagnosticCategory.Error,\"Public_property_0_of_exported_class_has_or_is_using_private_name_1_4031\",\"Public property '{0}' of exported class has or is using private name '{1}'.\"),Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4032,e.DiagnosticCategory.Error,\"Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2_4032\",\"Property '{0}' of exported interface has or is using name '{1}' from private module '{2}'.\"),Property_0_of_exported_interface_has_or_is_using_private_name_1:n(4033,e.DiagnosticCategory.Error,\"Property_0_of_exported_interface_has_or_is_using_private_name_1_4033\",\"Property '{0}' of exported interface has or is using private name '{1}'.\"),Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4034,e.DiagnosticCategory.Error,\"Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_mod_4034\",\"Parameter type of public static setter '{0}' from exported class has or is using name '{1}' from private module '{2}'.\"),Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1:n(4035,e.DiagnosticCategory.Error,\"Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1_4035\",\"Parameter type of public static setter '{0}' from exported class has or is using private name '{1}'.\"),Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4036,e.DiagnosticCategory.Error,\"Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2_4036\",\"Parameter type of public setter '{0}' from exported class has or is using name '{1}' from private module '{2}'.\"),Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1:n(4037,e.DiagnosticCategory.Error,\"Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1_4037\",\"Parameter type of public setter '{0}' from exported class has or is using private name '{1}'.\"),Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4038,e.DiagnosticCategory.Error,\"Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_modul_4038\",\"Return type of public static getter '{0}' from exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4039,e.DiagnosticCategory.Error,\"Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_4039\",\"Return type of public static getter '{0}' from exported class has or is using name '{1}' from private module '{2}'.\"),Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1:n(4040,e.DiagnosticCategory.Error,\"Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1_4040\",\"Return type of public static getter '{0}' from exported class has or is using private name '{1}'.\"),Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4041,e.DiagnosticCategory.Error,\"Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_4041\",\"Return type of public getter '{0}' from exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4042,e.DiagnosticCategory.Error,\"Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2_4042\",\"Return type of public getter '{0}' from exported class has or is using name '{1}' from private module '{2}'.\"),Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1:n(4043,e.DiagnosticCategory.Error,\"Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1_4043\",\"Return type of public getter '{0}' from exported class has or is using private name '{1}'.\"),Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4044,e.DiagnosticCategory.Error,\"Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_mod_4044\",\"Return type of constructor signature from exported interface has or is using name '{0}' from private module '{1}'.\"),Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0:n(4045,e.DiagnosticCategory.Error,\"Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0_4045\",\"Return type of constructor signature from exported interface has or is using private name '{0}'.\"),Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4046,e.DiagnosticCategory.Error,\"Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4046\",\"Return type of call signature from exported interface has or is using name '{0}' from private module '{1}'.\"),Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0:n(4047,e.DiagnosticCategory.Error,\"Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0_4047\",\"Return type of call signature from exported interface has or is using private name '{0}'.\"),Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4048,e.DiagnosticCategory.Error,\"Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4048\",\"Return type of index signature from exported interface has or is using name '{0}' from private module '{1}'.\"),Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0:n(4049,e.DiagnosticCategory.Error,\"Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0_4049\",\"Return type of index signature from exported interface has or is using private name '{0}'.\"),Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:n(4050,e.DiagnosticCategory.Error,\"Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module__4050\",\"Return type of public static method from exported class has or is using name '{0}' from external module {1} but cannot be named.\"),Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:n(4051,e.DiagnosticCategory.Error,\"Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1_4051\",\"Return type of public static method from exported class has or is using name '{0}' from private module '{1}'.\"),Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0:n(4052,e.DiagnosticCategory.Error,\"Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0_4052\",\"Return type of public static method from exported class has or is using private name '{0}'.\"),Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:n(4053,e.DiagnosticCategory.Error,\"Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_c_4053\",\"Return type of public method from exported class has or is using name '{0}' from external module {1} but cannot be named.\"),Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:n(4054,e.DiagnosticCategory.Error,\"Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1_4054\",\"Return type of public method from exported class has or is using name '{0}' from private module '{1}'.\"),Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0:n(4055,e.DiagnosticCategory.Error,\"Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0_4055\",\"Return type of public method from exported class has or is using private name '{0}'.\"),Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4056,e.DiagnosticCategory.Error,\"Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4056\",\"Return type of method from exported interface has or is using name '{0}' from private module '{1}'.\"),Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0:n(4057,e.DiagnosticCategory.Error,\"Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0_4057\",\"Return type of method from exported interface has or is using private name '{0}'.\"),Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:n(4058,e.DiagnosticCategory.Error,\"Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named_4058\",\"Return type of exported function has or is using name '{0}' from external module {1} but cannot be named.\"),Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1:n(4059,e.DiagnosticCategory.Error,\"Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1_4059\",\"Return type of exported function has or is using name '{0}' from private module '{1}'.\"),Return_type_of_exported_function_has_or_is_using_private_name_0:n(4060,e.DiagnosticCategory.Error,\"Return_type_of_exported_function_has_or_is_using_private_name_0_4060\",\"Return type of exported function has or is using private name '{0}'.\"),Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4061,e.DiagnosticCategory.Error,\"Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_can_4061\",\"Parameter '{0}' of constructor from exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4062,e.DiagnosticCategory.Error,\"Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2_4062\",\"Parameter '{0}' of constructor from exported class has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1:n(4063,e.DiagnosticCategory.Error,\"Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1_4063\",\"Parameter '{0}' of constructor from exported class has or is using private name '{1}'.\"),Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4064,e.DiagnosticCategory.Error,\"Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_mod_4064\",\"Parameter '{0}' of constructor signature from exported interface has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1:n(4065,e.DiagnosticCategory.Error,\"Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1_4065\",\"Parameter '{0}' of constructor signature from exported interface has or is using private name '{1}'.\"),Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4066,e.DiagnosticCategory.Error,\"Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4066\",\"Parameter '{0}' of call signature from exported interface has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1:n(4067,e.DiagnosticCategory.Error,\"Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1_4067\",\"Parameter '{0}' of call signature from exported interface has or is using private name '{1}'.\"),Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4068,e.DiagnosticCategory.Error,\"Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module__4068\",\"Parameter '{0}' of public static method from exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4069,e.DiagnosticCategory.Error,\"Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2_4069\",\"Parameter '{0}' of public static method from exported class has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:n(4070,e.DiagnosticCategory.Error,\"Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1_4070\",\"Parameter '{0}' of public static method from exported class has or is using private name '{1}'.\"),Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4071,e.DiagnosticCategory.Error,\"Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_c_4071\",\"Parameter '{0}' of public method from exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4072,e.DiagnosticCategory.Error,\"Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2_4072\",\"Parameter '{0}' of public method from exported class has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:n(4073,e.DiagnosticCategory.Error,\"Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1_4073\",\"Parameter '{0}' of public method from exported class has or is using private name '{1}'.\"),Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4074,e.DiagnosticCategory.Error,\"Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4074\",\"Parameter '{0}' of method from exported interface has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1:n(4075,e.DiagnosticCategory.Error,\"Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1_4075\",\"Parameter '{0}' of method from exported interface has or is using private name '{1}'.\"),Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4076,e.DiagnosticCategory.Error,\"Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4076\",\"Parameter '{0}' of exported function has or is using name '{1}' from external module {2} but cannot be named.\"),Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2:n(4077,e.DiagnosticCategory.Error,\"Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2_4077\",\"Parameter '{0}' of exported function has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_exported_function_has_or_is_using_private_name_1:n(4078,e.DiagnosticCategory.Error,\"Parameter_0_of_exported_function_has_or_is_using_private_name_1_4078\",\"Parameter '{0}' of exported function has or is using private name '{1}'.\"),Exported_type_alias_0_has_or_is_using_private_name_1:n(4081,e.DiagnosticCategory.Error,\"Exported_type_alias_0_has_or_is_using_private_name_1_4081\",\"Exported type alias '{0}' has or is using private name '{1}'.\"),Default_export_of_the_module_has_or_is_using_private_name_0:n(4082,e.DiagnosticCategory.Error,\"Default_export_of_the_module_has_or_is_using_private_name_0_4082\",\"Default export of the module has or is using private name '{0}'.\"),Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1:n(4083,e.DiagnosticCategory.Error,\"Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1_4083\",\"Type parameter '{0}' of exported type alias has or is using private name '{1}'.\"),Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_library_to_resolve_the_conflict:n(4090,e.DiagnosticCategory.Error,\"Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_librar_4090\",\"Conflicting definitions for '{0}' found at '{1}' and '{2}'. Consider installing a specific version of this library to resolve the conflict.\"),Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4091,e.DiagnosticCategory.Error,\"Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4091\",\"Parameter '{0}' of index signature from exported interface has or is using name '{1}' from private module '{2}'.\"),Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1:n(4092,e.DiagnosticCategory.Error,\"Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1_4092\",\"Parameter '{0}' of index signature from exported interface has or is using private name '{1}'.\"),Property_0_of_exported_class_expression_may_not_be_private_or_protected:n(4094,e.DiagnosticCategory.Error,\"Property_0_of_exported_class_expression_may_not_be_private_or_protected_4094\",\"Property '{0}' of exported class expression may not be private or protected.\"),Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4095,e.DiagnosticCategory.Error,\"Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_4095\",\"Public static method '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4096,e.DiagnosticCategory.Error,\"Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4096\",\"Public static method '{0}' of exported class has or is using name '{1}' from private module '{2}'.\"),Public_static_method_0_of_exported_class_has_or_is_using_private_name_1:n(4097,e.DiagnosticCategory.Error,\"Public_static_method_0_of_exported_class_has_or_is_using_private_name_1_4097\",\"Public static method '{0}' of exported class has or is using private name '{1}'.\"),Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4098,e.DiagnosticCategory.Error,\"Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4098\",\"Public method '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named.\"),Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4099,e.DiagnosticCategory.Error,\"Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4099\",\"Public method '{0}' of exported class has or is using name '{1}' from private module '{2}'.\"),Public_method_0_of_exported_class_has_or_is_using_private_name_1:n(4100,e.DiagnosticCategory.Error,\"Public_method_0_of_exported_class_has_or_is_using_private_name_1_4100\",\"Public method '{0}' of exported class has or is using private name '{1}'.\"),Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4101,e.DiagnosticCategory.Error,\"Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2_4101\",\"Method '{0}' of exported interface has or is using name '{1}' from private module '{2}'.\"),Method_0_of_exported_interface_has_or_is_using_private_name_1:n(4102,e.DiagnosticCategory.Error,\"Method_0_of_exported_interface_has_or_is_using_private_name_1_4102\",\"Method '{0}' of exported interface has or is using private name '{1}'.\"),The_current_host_does_not_support_the_0_option:n(5001,e.DiagnosticCategory.Error,\"The_current_host_does_not_support_the_0_option_5001\",\"The current host does not support the '{0}' option.\"),Cannot_find_the_common_subdirectory_path_for_the_input_files:n(5009,e.DiagnosticCategory.Error,\"Cannot_find_the_common_subdirectory_path_for_the_input_files_5009\",\"Cannot find the common subdirectory path for the input files.\"),File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0:n(5010,e.DiagnosticCategory.Error,\"File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0_5010\",\"File specification cannot end in a recursive directory wildcard ('**'): '{0}'.\"),Cannot_read_file_0_Colon_1:n(5012,e.DiagnosticCategory.Error,\"Cannot_read_file_0_Colon_1_5012\",\"Cannot read file '{0}': {1}.\"),Failed_to_parse_file_0_Colon_1:n(5014,e.DiagnosticCategory.Error,\"Failed_to_parse_file_0_Colon_1_5014\",\"Failed to parse file '{0}': {1}.\"),Unknown_compiler_option_0:n(5023,e.DiagnosticCategory.Error,\"Unknown_compiler_option_0_5023\",\"Unknown compiler option '{0}'.\"),Compiler_option_0_requires_a_value_of_type_1:n(5024,e.DiagnosticCategory.Error,\"Compiler_option_0_requires_a_value_of_type_1_5024\",\"Compiler option '{0}' requires a value of type {1}.\"),Could_not_write_file_0_Colon_1:n(5033,e.DiagnosticCategory.Error,\"Could_not_write_file_0_Colon_1_5033\",\"Could not write file '{0}': {1}.\"),Option_project_cannot_be_mixed_with_source_files_on_a_command_line:n(5042,e.DiagnosticCategory.Error,\"Option_project_cannot_be_mixed_with_source_files_on_a_command_line_5042\",\"Option 'project' cannot be mixed with source files on a command line.\"),Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES2015_or_higher:n(5047,e.DiagnosticCategory.Error,\"Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES_5047\",\"Option 'isolatedModules' can only be used when either option '--module' is provided or option 'target' is 'ES2015' or higher.\"),Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided:n(5051,e.DiagnosticCategory.Error,\"Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided_5051\",\"Option '{0} can only be used when either option '--inlineSourceMap' or option '--sourceMap' is provided.\"),Option_0_cannot_be_specified_without_specifying_option_1:n(5052,e.DiagnosticCategory.Error,\"Option_0_cannot_be_specified_without_specifying_option_1_5052\",\"Option '{0}' cannot be specified without specifying option '{1}'.\"),Option_0_cannot_be_specified_with_option_1:n(5053,e.DiagnosticCategory.Error,\"Option_0_cannot_be_specified_with_option_1_5053\",\"Option '{0}' cannot be specified with option '{1}'.\"),A_tsconfig_json_file_is_already_defined_at_Colon_0:n(5054,e.DiagnosticCategory.Error,\"A_tsconfig_json_file_is_already_defined_at_Colon_0_5054\",\"A 'tsconfig.json' file is already defined at: '{0}'.\"),Cannot_write_file_0_because_it_would_overwrite_input_file:n(5055,e.DiagnosticCategory.Error,\"Cannot_write_file_0_because_it_would_overwrite_input_file_5055\",\"Cannot write file '{0}' because it would overwrite input file.\"),Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files:n(5056,e.DiagnosticCategory.Error,\"Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files_5056\",\"Cannot write file '{0}' because it would be overwritten by multiple input files.\"),Cannot_find_a_tsconfig_json_file_at_the_specified_directory_Colon_0:n(5057,e.DiagnosticCategory.Error,\"Cannot_find_a_tsconfig_json_file_at_the_specified_directory_Colon_0_5057\",\"Cannot find a tsconfig.json file at the specified directory: '{0}'.\"),The_specified_path_does_not_exist_Colon_0:n(5058,e.DiagnosticCategory.Error,\"The_specified_path_does_not_exist_Colon_0_5058\",\"The specified path does not exist: '{0}'.\"),Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier:n(5059,e.DiagnosticCategory.Error,\"Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier_5059\",\"Invalid value for '--reactNamespace'. '{0}' is not a valid identifier.\"),Option_paths_cannot_be_used_without_specifying_baseUrl_option:n(5060,e.DiagnosticCategory.Error,\"Option_paths_cannot_be_used_without_specifying_baseUrl_option_5060\",\"Option 'paths' cannot be used without specifying '--baseUrl' option.\"),Pattern_0_can_have_at_most_one_Asterisk_character:n(5061,e.DiagnosticCategory.Error,\"Pattern_0_can_have_at_most_one_Asterisk_character_5061\",\"Pattern '{0}' can have at most one '*' character.\"),Substitution_0_in_pattern_1_in_can_have_at_most_one_Asterisk_character:n(5062,e.DiagnosticCategory.Error,\"Substitution_0_in_pattern_1_in_can_have_at_most_one_Asterisk_character_5062\",\"Substitution '{0}' in pattern '{1}' in can have at most one '*' character.\"),Substitutions_for_pattern_0_should_be_an_array:n(5063,e.DiagnosticCategory.Error,\"Substitutions_for_pattern_0_should_be_an_array_5063\",\"Substitutions for pattern '{0}' should be an array.\"),Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2:n(5064,e.DiagnosticCategory.Error,\"Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2_5064\",\"Substitution '{0}' for pattern '{1}' has incorrect type, expected 'string', got '{2}'.\"),File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0:n(5065,e.DiagnosticCategory.Error,\"File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildca_5065\",\"File specification cannot contain a parent directory ('..') that appears after a recursive directory wildcard ('**'): '{0}'.\"),Substitutions_for_pattern_0_shouldn_t_be_an_empty_array:n(5066,e.DiagnosticCategory.Error,\"Substitutions_for_pattern_0_shouldn_t_be_an_empty_array_5066\",\"Substitutions for pattern '{0}' shouldn't be an empty array.\"),Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name:n(5067,e.DiagnosticCategory.Error,\"Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name_5067\",\"Invalid value for 'jsxFactory'. '{0}' is not a valid identifier or qualified-name.\"),Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript_files_Learn_more_at_https_Colon_Slash_Slashaka_ms_Slashtsconfig:n(5068,e.DiagnosticCategory.Error,\"Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript__5068\",\"Adding a tsconfig.json file will help organize projects that contain both TypeScript and JavaScript files. Learn more at https://aka.ms/tsconfig.\"),Option_0_cannot_be_specified_without_specifying_option_1_or_option_2:n(5069,e.DiagnosticCategory.Error,\"Option_0_cannot_be_specified_without_specifying_option_1_or_option_2_5069\",\"Option '{0}' cannot be specified without specifying option '{1}' or option '{2}'.\"),Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy:n(5070,e.DiagnosticCategory.Error,\"Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy_5070\",\"Option '--resolveJsonModule' cannot be specified without 'node' module resolution strategy.\"),Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_esNext:n(5071,e.DiagnosticCategory.Error,\"Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_5071\",\"Option '--resolveJsonModule' can only be specified when module code generation is 'commonjs', 'amd', 'es2015' or 'esNext'.\"),Unknown_build_option_0:n(5072,e.DiagnosticCategory.Error,\"Unknown_build_option_0_5072\",\"Unknown build option '{0}'.\"),Build_option_0_requires_a_value_of_type_1:n(5073,e.DiagnosticCategory.Error,\"Build_option_0_requires_a_value_of_type_1_5073\",\"Build option '{0}' requires a value of type {1}.\"),Generates_a_sourcemap_for_each_corresponding_d_ts_file:n(6e3,e.DiagnosticCategory.Message,\"Generates_a_sourcemap_for_each_corresponding_d_ts_file_6000\",\"Generates a sourcemap for each corresponding '.d.ts' file.\"),Concatenate_and_emit_output_to_single_file:n(6001,e.DiagnosticCategory.Message,\"Concatenate_and_emit_output_to_single_file_6001\",\"Concatenate and emit output to single file.\"),Generates_corresponding_d_ts_file:n(6002,e.DiagnosticCategory.Message,\"Generates_corresponding_d_ts_file_6002\",\"Generates corresponding '.d.ts' file.\"),Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations:n(6003,e.DiagnosticCategory.Message,\"Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations_6003\",\"Specify the location where debugger should locate map files instead of generated locations.\"),Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations:n(6004,e.DiagnosticCategory.Message,\"Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations_6004\",\"Specify the location where debugger should locate TypeScript files instead of source locations.\"),Watch_input_files:n(6005,e.DiagnosticCategory.Message,\"Watch_input_files_6005\",\"Watch input files.\"),Redirect_output_structure_to_the_directory:n(6006,e.DiagnosticCategory.Message,\"Redirect_output_structure_to_the_directory_6006\",\"Redirect output structure to the directory.\"),Do_not_erase_const_enum_declarations_in_generated_code:n(6007,e.DiagnosticCategory.Message,\"Do_not_erase_const_enum_declarations_in_generated_code_6007\",\"Do not erase const enum declarations in generated code.\"),Do_not_emit_outputs_if_any_errors_were_reported:n(6008,e.DiagnosticCategory.Message,\"Do_not_emit_outputs_if_any_errors_were_reported_6008\",\"Do not emit outputs if any errors were reported.\"),Do_not_emit_comments_to_output:n(6009,e.DiagnosticCategory.Message,\"Do_not_emit_comments_to_output_6009\",\"Do not emit comments to output.\"),Do_not_emit_outputs:n(6010,e.DiagnosticCategory.Message,\"Do_not_emit_outputs_6010\",\"Do not emit outputs.\"),Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typechecking:n(6011,e.DiagnosticCategory.Message,\"Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typech_6011\",\"Allow default imports from modules with no default export. This does not affect code emit, just typechecking.\"),Skip_type_checking_of_declaration_files:n(6012,e.DiagnosticCategory.Message,\"Skip_type_checking_of_declaration_files_6012\",\"Skip type checking of declaration files.\"),Do_not_resolve_the_real_path_of_symlinks:n(6013,e.DiagnosticCategory.Message,\"Do_not_resolve_the_real_path_of_symlinks_6013\",\"Do not resolve the real path of symlinks.\"),Only_emit_d_ts_declaration_files:n(6014,e.DiagnosticCategory.Message,\"Only_emit_d_ts_declaration_files_6014\",\"Only emit '.d.ts' declaration files.\"),Specify_ECMAScript_target_version_Colon_ES3_default_ES5_ES2015_ES2016_ES2017_ES2018_or_ESNEXT:n(6015,e.DiagnosticCategory.Message,\"Specify_ECMAScript_target_version_Colon_ES3_default_ES5_ES2015_ES2016_ES2017_ES2018_or_ESNEXT_6015\",\"Specify ECMAScript target version: 'ES3' (default), 'ES5', 'ES2015', 'ES2016', 'ES2017','ES2018' or 'ESNEXT'.\"),Specify_module_code_generation_Colon_none_commonjs_amd_system_umd_es2015_or_ESNext:n(6016,e.DiagnosticCategory.Message,\"Specify_module_code_generation_Colon_none_commonjs_amd_system_umd_es2015_or_ESNext_6016\",\"Specify module code generation: 'none', 'commonjs', 'amd', 'system', 'umd', 'es2015', or 'ESNext'.\"),Print_this_message:n(6017,e.DiagnosticCategory.Message,\"Print_this_message_6017\",\"Print this message.\"),Print_the_compiler_s_version:n(6019,e.DiagnosticCategory.Message,\"Print_the_compiler_s_version_6019\",\"Print the compiler's version.\"),Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json:n(6020,e.DiagnosticCategory.Message,\"Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json_6020\",\"Compile the project given the path to its configuration file, or to a folder with a 'tsconfig.json'.\"),Syntax_Colon_0:n(6023,e.DiagnosticCategory.Message,\"Syntax_Colon_0_6023\",\"Syntax: {0}\"),options:n(6024,e.DiagnosticCategory.Message,\"options_6024\",\"options\"),file:n(6025,e.DiagnosticCategory.Message,\"file_6025\",\"file\"),Examples_Colon_0:n(6026,e.DiagnosticCategory.Message,\"Examples_Colon_0_6026\",\"Examples: {0}\"),Options_Colon:n(6027,e.DiagnosticCategory.Message,\"Options_Colon_6027\",\"Options:\"),Version_0:n(6029,e.DiagnosticCategory.Message,\"Version_0_6029\",\"Version {0}\"),Insert_command_line_options_and_files_from_a_file:n(6030,e.DiagnosticCategory.Message,\"Insert_command_line_options_and_files_from_a_file_6030\",\"Insert command line options and files from a file.\"),Starting_compilation_in_watch_mode:n(6031,e.DiagnosticCategory.Message,\"Starting_compilation_in_watch_mode_6031\",\"Starting compilation in watch mode...\"),File_change_detected_Starting_incremental_compilation:n(6032,e.DiagnosticCategory.Message,\"File_change_detected_Starting_incremental_compilation_6032\",\"File change detected. Starting incremental compilation...\"),KIND:n(6034,e.DiagnosticCategory.Message,\"KIND_6034\",\"KIND\"),FILE:n(6035,e.DiagnosticCategory.Message,\"FILE_6035\",\"FILE\"),VERSION:n(6036,e.DiagnosticCategory.Message,\"VERSION_6036\",\"VERSION\"),LOCATION:n(6037,e.DiagnosticCategory.Message,\"LOCATION_6037\",\"LOCATION\"),DIRECTORY:n(6038,e.DiagnosticCategory.Message,\"DIRECTORY_6038\",\"DIRECTORY\"),STRATEGY:n(6039,e.DiagnosticCategory.Message,\"STRATEGY_6039\",\"STRATEGY\"),FILE_OR_DIRECTORY:n(6040,e.DiagnosticCategory.Message,\"FILE_OR_DIRECTORY_6040\",\"FILE OR DIRECTORY\"),Generates_corresponding_map_file:n(6043,e.DiagnosticCategory.Message,\"Generates_corresponding_map_file_6043\",\"Generates corresponding '.map' file.\"),Compiler_option_0_expects_an_argument:n(6044,e.DiagnosticCategory.Error,\"Compiler_option_0_expects_an_argument_6044\",\"Compiler option '{0}' expects an argument.\"),Unterminated_quoted_string_in_response_file_0:n(6045,e.DiagnosticCategory.Error,\"Unterminated_quoted_string_in_response_file_0_6045\",\"Unterminated quoted string in response file '{0}'.\"),Argument_for_0_option_must_be_Colon_1:n(6046,e.DiagnosticCategory.Error,\"Argument_for_0_option_must_be_Colon_1_6046\",\"Argument for '{0}' option must be: {1}.\"),Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1:n(6048,e.DiagnosticCategory.Error,\"Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1_6048\",\"Locale must be of the form <language> or <language>-<territory>. For example '{0}' or '{1}'.\"),Unsupported_locale_0:n(6049,e.DiagnosticCategory.Error,\"Unsupported_locale_0_6049\",\"Unsupported locale '{0}'.\"),Unable_to_open_file_0:n(6050,e.DiagnosticCategory.Error,\"Unable_to_open_file_0_6050\",\"Unable to open file '{0}'.\"),Corrupted_locale_file_0:n(6051,e.DiagnosticCategory.Error,\"Corrupted_locale_file_0_6051\",\"Corrupted locale file {0}.\"),Raise_error_on_expressions_and_declarations_with_an_implied_any_type:n(6052,e.DiagnosticCategory.Message,\"Raise_error_on_expressions_and_declarations_with_an_implied_any_type_6052\",\"Raise error on expressions and declarations with an implied 'any' type.\"),File_0_not_found:n(6053,e.DiagnosticCategory.Error,\"File_0_not_found_6053\",\"File '{0}' not found.\"),File_0_has_unsupported_extension_The_only_supported_extensions_are_1:n(6054,e.DiagnosticCategory.Error,\"File_0_has_unsupported_extension_The_only_supported_extensions_are_1_6054\",\"File '{0}' has unsupported extension. The only supported extensions are {1}.\"),Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures:n(6055,e.DiagnosticCategory.Message,\"Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures_6055\",\"Suppress noImplicitAny errors for indexing objects lacking index signatures.\"),Do_not_emit_declarations_for_code_that_has_an_internal_annotation:n(6056,e.DiagnosticCategory.Message,\"Do_not_emit_declarations_for_code_that_has_an_internal_annotation_6056\",\"Do not emit declarations for code that has an '@internal' annotation.\"),Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir:n(6058,e.DiagnosticCategory.Message,\"Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir_6058\",\"Specify the root directory of input files. Use to control the output directory structure with --outDir.\"),File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files:n(6059,e.DiagnosticCategory.Error,\"File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files_6059\",\"File '{0}' is not under 'rootDir' '{1}'. 'rootDir' is expected to contain all source files.\"),Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix:n(6060,e.DiagnosticCategory.Message,\"Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix_6060\",\"Specify the end of line sequence to be used when emitting files: 'CRLF' (dos) or 'LF' (unix).\"),NEWLINE:n(6061,e.DiagnosticCategory.Message,\"NEWLINE_6061\",\"NEWLINE\"),Option_0_can_only_be_specified_in_tsconfig_json_file:n(6064,e.DiagnosticCategory.Error,\"Option_0_can_only_be_specified_in_tsconfig_json_file_6064\",\"Option '{0}' can only be specified in 'tsconfig.json' file.\"),Enables_experimental_support_for_ES7_decorators:n(6065,e.DiagnosticCategory.Message,\"Enables_experimental_support_for_ES7_decorators_6065\",\"Enables experimental support for ES7 decorators.\"),Enables_experimental_support_for_emitting_type_metadata_for_decorators:n(6066,e.DiagnosticCategory.Message,\"Enables_experimental_support_for_emitting_type_metadata_for_decorators_6066\",\"Enables experimental support for emitting type metadata for decorators.\"),Enables_experimental_support_for_ES7_async_functions:n(6068,e.DiagnosticCategory.Message,\"Enables_experimental_support_for_ES7_async_functions_6068\",\"Enables experimental support for ES7 async functions.\"),Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6:n(6069,e.DiagnosticCategory.Message,\"Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6_6069\",\"Specify module resolution strategy: 'node' (Node.js) or 'classic' (TypeScript pre-1.6).\"),Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file:n(6070,e.DiagnosticCategory.Message,\"Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file_6070\",\"Initializes a TypeScript project and creates a tsconfig.json file.\"),Successfully_created_a_tsconfig_json_file:n(6071,e.DiagnosticCategory.Message,\"Successfully_created_a_tsconfig_json_file_6071\",\"Successfully created a tsconfig.json file.\"),Suppress_excess_property_checks_for_object_literals:n(6072,e.DiagnosticCategory.Message,\"Suppress_excess_property_checks_for_object_literals_6072\",\"Suppress excess property checks for object literals.\"),Stylize_errors_and_messages_using_color_and_context_experimental:n(6073,e.DiagnosticCategory.Message,\"Stylize_errors_and_messages_using_color_and_context_experimental_6073\",\"Stylize errors and messages using color and context (experimental).\"),Do_not_report_errors_on_unused_labels:n(6074,e.DiagnosticCategory.Message,\"Do_not_report_errors_on_unused_labels_6074\",\"Do not report errors on unused labels.\"),Report_error_when_not_all_code_paths_in_function_return_a_value:n(6075,e.DiagnosticCategory.Message,\"Report_error_when_not_all_code_paths_in_function_return_a_value_6075\",\"Report error when not all code paths in function return a value.\"),Report_errors_for_fallthrough_cases_in_switch_statement:n(6076,e.DiagnosticCategory.Message,\"Report_errors_for_fallthrough_cases_in_switch_statement_6076\",\"Report errors for fallthrough cases in switch statement.\"),Do_not_report_errors_on_unreachable_code:n(6077,e.DiagnosticCategory.Message,\"Do_not_report_errors_on_unreachable_code_6077\",\"Do not report errors on unreachable code.\"),Disallow_inconsistently_cased_references_to_the_same_file:n(6078,e.DiagnosticCategory.Message,\"Disallow_inconsistently_cased_references_to_the_same_file_6078\",\"Disallow inconsistently-cased references to the same file.\"),Specify_library_files_to_be_included_in_the_compilation:n(6079,e.DiagnosticCategory.Message,\"Specify_library_files_to_be_included_in_the_compilation_6079\",\"Specify library files to be included in the compilation.\"),Specify_JSX_code_generation_Colon_preserve_react_native_or_react:n(6080,e.DiagnosticCategory.Message,\"Specify_JSX_code_generation_Colon_preserve_react_native_or_react_6080\",\"Specify JSX code generation: 'preserve', 'react-native', or 'react'.\"),File_0_has_an_unsupported_extension_so_skipping_it:n(6081,e.DiagnosticCategory.Message,\"File_0_has_an_unsupported_extension_so_skipping_it_6081\",\"File '{0}' has an unsupported extension, so skipping it.\"),Only_amd_and_system_modules_are_supported_alongside_0:n(6082,e.DiagnosticCategory.Error,\"Only_amd_and_system_modules_are_supported_alongside_0_6082\",\"Only 'amd' and 'system' modules are supported alongside --{0}.\"),Base_directory_to_resolve_non_absolute_module_names:n(6083,e.DiagnosticCategory.Message,\"Base_directory_to_resolve_non_absolute_module_names_6083\",\"Base directory to resolve non-absolute module names.\"),Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react_JSX_emit:n(6084,e.DiagnosticCategory.Message,\"Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react__6084\",\"[Deprecated] Use '--jsxFactory' instead. Specify the object invoked for createElement when targeting 'react' JSX emit\"),Enable_tracing_of_the_name_resolution_process:n(6085,e.DiagnosticCategory.Message,\"Enable_tracing_of_the_name_resolution_process_6085\",\"Enable tracing of the name resolution process.\"),Resolving_module_0_from_1:n(6086,e.DiagnosticCategory.Message,\"Resolving_module_0_from_1_6086\",\"======== Resolving module '{0}' from '{1}'. ========\"),Explicitly_specified_module_resolution_kind_Colon_0:n(6087,e.DiagnosticCategory.Message,\"Explicitly_specified_module_resolution_kind_Colon_0_6087\",\"Explicitly specified module resolution kind: '{0}'.\"),Module_resolution_kind_is_not_specified_using_0:n(6088,e.DiagnosticCategory.Message,\"Module_resolution_kind_is_not_specified_using_0_6088\",\"Module resolution kind is not specified, using '{0}'.\"),Module_name_0_was_successfully_resolved_to_1:n(6089,e.DiagnosticCategory.Message,\"Module_name_0_was_successfully_resolved_to_1_6089\",\"======== Module name '{0}' was successfully resolved to '{1}'. ========\"),Module_name_0_was_not_resolved:n(6090,e.DiagnosticCategory.Message,\"Module_name_0_was_not_resolved_6090\",\"======== Module name '{0}' was not resolved. ========\"),paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0:n(6091,e.DiagnosticCategory.Message,\"paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0_6091\",\"'paths' option is specified, looking for a pattern to match module name '{0}'.\"),Module_name_0_matched_pattern_1:n(6092,e.DiagnosticCategory.Message,\"Module_name_0_matched_pattern_1_6092\",\"Module name '{0}', matched pattern '{1}'.\"),Trying_substitution_0_candidate_module_location_Colon_1:n(6093,e.DiagnosticCategory.Message,\"Trying_substitution_0_candidate_module_location_Colon_1_6093\",\"Trying substitution '{0}', candidate module location: '{1}'.\"),Resolving_module_name_0_relative_to_base_url_1_2:n(6094,e.DiagnosticCategory.Message,\"Resolving_module_name_0_relative_to_base_url_1_2_6094\",\"Resolving module name '{0}' relative to base url '{1}' - '{2}'.\"),Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1:n(6095,e.DiagnosticCategory.Message,\"Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1_6095\",\"Loading module as file / folder, candidate module location '{0}', target file type '{1}'.\"),File_0_does_not_exist:n(6096,e.DiagnosticCategory.Message,\"File_0_does_not_exist_6096\",\"File '{0}' does not exist.\"),File_0_exist_use_it_as_a_name_resolution_result:n(6097,e.DiagnosticCategory.Message,\"File_0_exist_use_it_as_a_name_resolution_result_6097\",\"File '{0}' exist - use it as a name resolution result.\"),Loading_module_0_from_node_modules_folder_target_file_type_1:n(6098,e.DiagnosticCategory.Message,\"Loading_module_0_from_node_modules_folder_target_file_type_1_6098\",\"Loading module '{0}' from 'node_modules' folder, target file type '{1}'.\"),Found_package_json_at_0:n(6099,e.DiagnosticCategory.Message,\"Found_package_json_at_0_6099\",\"Found 'package.json' at '{0}'.\"),package_json_does_not_have_a_0_field:n(6100,e.DiagnosticCategory.Message,\"package_json_does_not_have_a_0_field_6100\",\"'package.json' does not have a '{0}' field.\"),package_json_has_0_field_1_that_references_2:n(6101,e.DiagnosticCategory.Message,\"package_json_has_0_field_1_that_references_2_6101\",\"'package.json' has '{0}' field '{1}' that references '{2}'.\"),Allow_javascript_files_to_be_compiled:n(6102,e.DiagnosticCategory.Message,\"Allow_javascript_files_to_be_compiled_6102\",\"Allow javascript files to be compiled.\"),Option_0_should_have_array_of_strings_as_a_value:n(6103,e.DiagnosticCategory.Error,\"Option_0_should_have_array_of_strings_as_a_value_6103\",\"Option '{0}' should have array of strings as a value.\"),Checking_if_0_is_the_longest_matching_prefix_for_1_2:n(6104,e.DiagnosticCategory.Message,\"Checking_if_0_is_the_longest_matching_prefix_for_1_2_6104\",\"Checking if '{0}' is the longest matching prefix for '{1}' - '{2}'.\"),Expected_type_of_0_field_in_package_json_to_be_1_got_2:n(6105,e.DiagnosticCategory.Message,\"Expected_type_of_0_field_in_package_json_to_be_1_got_2_6105\",\"Expected type of '{0}' field in 'package.json' to be '{1}', got '{2}'.\"),baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1:n(6106,e.DiagnosticCategory.Message,\"baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1_6106\",\"'baseUrl' option is set to '{0}', using this value to resolve non-relative module name '{1}'.\"),rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0:n(6107,e.DiagnosticCategory.Message,\"rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0_6107\",\"'rootDirs' option is set, using it to resolve relative module name '{0}'.\"),Longest_matching_prefix_for_0_is_1:n(6108,e.DiagnosticCategory.Message,\"Longest_matching_prefix_for_0_is_1_6108\",\"Longest matching prefix for '{0}' is '{1}'.\"),Loading_0_from_the_root_dir_1_candidate_location_2:n(6109,e.DiagnosticCategory.Message,\"Loading_0_from_the_root_dir_1_candidate_location_2_6109\",\"Loading '{0}' from the root dir '{1}', candidate location '{2}'.\"),Trying_other_entries_in_rootDirs:n(6110,e.DiagnosticCategory.Message,\"Trying_other_entries_in_rootDirs_6110\",\"Trying other entries in 'rootDirs'.\"),Module_resolution_using_rootDirs_has_failed:n(6111,e.DiagnosticCategory.Message,\"Module_resolution_using_rootDirs_has_failed_6111\",\"Module resolution using 'rootDirs' has failed.\"),Do_not_emit_use_strict_directives_in_module_output:n(6112,e.DiagnosticCategory.Message,\"Do_not_emit_use_strict_directives_in_module_output_6112\",\"Do not emit 'use strict' directives in module output.\"),Enable_strict_null_checks:n(6113,e.DiagnosticCategory.Message,\"Enable_strict_null_checks_6113\",\"Enable strict null checks.\"),Unknown_option_excludes_Did_you_mean_exclude:n(6114,e.DiagnosticCategory.Error,\"Unknown_option_excludes_Did_you_mean_exclude_6114\",\"Unknown option 'excludes'. Did you mean 'exclude'?\"),Raise_error_on_this_expressions_with_an_implied_any_type:n(6115,e.DiagnosticCategory.Message,\"Raise_error_on_this_expressions_with_an_implied_any_type_6115\",\"Raise error on 'this' expressions with an implied 'any' type.\"),Resolving_type_reference_directive_0_containing_file_1_root_directory_2:n(6116,e.DiagnosticCategory.Message,\"Resolving_type_reference_directive_0_containing_file_1_root_directory_2_6116\",\"======== Resolving type reference directive '{0}', containing file '{1}', root directory '{2}'. ========\"),Resolving_using_primary_search_paths:n(6117,e.DiagnosticCategory.Message,\"Resolving_using_primary_search_paths_6117\",\"Resolving using primary search paths...\"),Resolving_from_node_modules_folder:n(6118,e.DiagnosticCategory.Message,\"Resolving_from_node_modules_folder_6118\",\"Resolving from node_modules folder...\"),Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2:n(6119,e.DiagnosticCategory.Message,\"Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2_6119\",\"======== Type reference directive '{0}' was successfully resolved to '{1}', primary: {2}. ========\"),Type_reference_directive_0_was_not_resolved:n(6120,e.DiagnosticCategory.Message,\"Type_reference_directive_0_was_not_resolved_6120\",\"======== Type reference directive '{0}' was not resolved. ========\"),Resolving_with_primary_search_path_0:n(6121,e.DiagnosticCategory.Message,\"Resolving_with_primary_search_path_0_6121\",\"Resolving with primary search path '{0}'.\"),Root_directory_cannot_be_determined_skipping_primary_search_paths:n(6122,e.DiagnosticCategory.Message,\"Root_directory_cannot_be_determined_skipping_primary_search_paths_6122\",\"Root directory cannot be determined, skipping primary search paths.\"),Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set:n(6123,e.DiagnosticCategory.Message,\"Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set_6123\",\"======== Resolving type reference directive '{0}', containing file '{1}', root directory not set. ========\"),Type_declaration_files_to_be_included_in_compilation:n(6124,e.DiagnosticCategory.Message,\"Type_declaration_files_to_be_included_in_compilation_6124\",\"Type declaration files to be included in compilation.\"),Looking_up_in_node_modules_folder_initial_location_0:n(6125,e.DiagnosticCategory.Message,\"Looking_up_in_node_modules_folder_initial_location_0_6125\",\"Looking up in 'node_modules' folder, initial location '{0}'.\"),Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_modules_folder:n(6126,e.DiagnosticCategory.Message,\"Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_mod_6126\",\"Containing file is not specified and root directory cannot be determined, skipping lookup in 'node_modules' folder.\"),Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1:n(6127,e.DiagnosticCategory.Message,\"Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1_6127\",\"======== Resolving type reference directive '{0}', containing file not set, root directory '{1}'. ========\"),Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set:n(6128,e.DiagnosticCategory.Message,\"Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set_6128\",\"======== Resolving type reference directive '{0}', containing file not set, root directory not set. ========\"),Resolving_real_path_for_0_result_1:n(6130,e.DiagnosticCategory.Message,\"Resolving_real_path_for_0_result_1_6130\",\"Resolving real path for '{0}', result '{1}'.\"),Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system:n(6131,e.DiagnosticCategory.Error,\"Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system_6131\",\"Cannot compile modules using option '{0}' unless the '--module' flag is 'amd' or 'system'.\"),File_name_0_has_a_1_extension_stripping_it:n(6132,e.DiagnosticCategory.Message,\"File_name_0_has_a_1_extension_stripping_it_6132\",\"File name '{0}' has a '{1}' extension - stripping it.\"),_0_is_declared_but_its_value_is_never_read:n(6133,e.DiagnosticCategory.Error,\"_0_is_declared_but_its_value_is_never_read_6133\",\"'{0}' is declared but its value is never read.\",!0),Report_errors_on_unused_locals:n(6134,e.DiagnosticCategory.Message,\"Report_errors_on_unused_locals_6134\",\"Report errors on unused locals.\"),Report_errors_on_unused_parameters:n(6135,e.DiagnosticCategory.Message,\"Report_errors_on_unused_parameters_6135\",\"Report errors on unused parameters.\"),The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files:n(6136,e.DiagnosticCategory.Message,\"The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files_6136\",\"The maximum dependency depth to search under node_modules and load JavaScript files.\"),Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1:n(6137,e.DiagnosticCategory.Error,\"Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1_6137\",\"Cannot import type declaration files. Consider importing '{0}' instead of '{1}'.\"),Property_0_is_declared_but_its_value_is_never_read:n(6138,e.DiagnosticCategory.Error,\"Property_0_is_declared_but_its_value_is_never_read_6138\",\"Property '{0}' is declared but its value is never read.\",!0),Import_emit_helpers_from_tslib:n(6139,e.DiagnosticCategory.Message,\"Import_emit_helpers_from_tslib_6139\",\"Import emit helpers from 'tslib'.\"),Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using_cache_location_2:n(6140,e.DiagnosticCategory.Error,\"Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using__6140\",\"Auto discovery for typings is enabled in project '{0}'. Running extra resolution pass for module '{1}' using cache location '{2}'.\"),Parse_in_strict_mode_and_emit_use_strict_for_each_source_file:n(6141,e.DiagnosticCategory.Message,\"Parse_in_strict_mode_and_emit_use_strict_for_each_source_file_6141\",'Parse in strict mode and emit \"use strict\" for each source file.'),Module_0_was_resolved_to_1_but_jsx_is_not_set:n(6142,e.DiagnosticCategory.Error,\"Module_0_was_resolved_to_1_but_jsx_is_not_set_6142\",\"Module '{0}' was resolved to '{1}', but '--jsx' is not set.\"),Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1:n(6144,e.DiagnosticCategory.Message,\"Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1_6144\",\"Module '{0}' was resolved as locally declared ambient module in file '{1}'.\"),Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified:n(6145,e.DiagnosticCategory.Message,\"Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified_6145\",\"Module '{0}' was resolved as ambient module declared in '{1}' since this file was not modified.\"),Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h:n(6146,e.DiagnosticCategory.Message,\"Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h_6146\",\"Specify the JSX factory function to use when targeting 'react' JSX emit, e.g. 'React.createElement' or 'h'.\"),Resolution_for_module_0_was_found_in_cache_from_location_1:n(6147,e.DiagnosticCategory.Message,\"Resolution_for_module_0_was_found_in_cache_from_location_1_6147\",\"Resolution for module '{0}' was found in cache from location '{1}'.\"),Directory_0_does_not_exist_skipping_all_lookups_in_it:n(6148,e.DiagnosticCategory.Message,\"Directory_0_does_not_exist_skipping_all_lookups_in_it_6148\",\"Directory '{0}' does not exist, skipping all lookups in it.\"),Show_diagnostic_information:n(6149,e.DiagnosticCategory.Message,\"Show_diagnostic_information_6149\",\"Show diagnostic information.\"),Show_verbose_diagnostic_information:n(6150,e.DiagnosticCategory.Message,\"Show_verbose_diagnostic_information_6150\",\"Show verbose diagnostic information.\"),Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file:n(6151,e.DiagnosticCategory.Message,\"Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file_6151\",\"Emit a single file with source maps instead of having a separate file.\"),Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap_to_be_set:n(6152,e.DiagnosticCategory.Message,\"Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap__6152\",\"Emit the source alongside the sourcemaps within a single file; requires '--inlineSourceMap' or '--sourceMap' to be set.\"),Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule:n(6153,e.DiagnosticCategory.Message,\"Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule_6153\",\"Transpile each file as a separate module (similar to 'ts.transpileModule').\"),Print_names_of_generated_files_part_of_the_compilation:n(6154,e.DiagnosticCategory.Message,\"Print_names_of_generated_files_part_of_the_compilation_6154\",\"Print names of generated files part of the compilation.\"),Print_names_of_files_part_of_the_compilation:n(6155,e.DiagnosticCategory.Message,\"Print_names_of_files_part_of_the_compilation_6155\",\"Print names of files part of the compilation.\"),The_locale_used_when_displaying_messages_to_the_user_e_g_en_us:n(6156,e.DiagnosticCategory.Message,\"The_locale_used_when_displaying_messages_to_the_user_e_g_en_us_6156\",\"The locale used when displaying messages to the user (e.g. 'en-us')\"),Do_not_generate_custom_helper_functions_like_extends_in_compiled_output:n(6157,e.DiagnosticCategory.Message,\"Do_not_generate_custom_helper_functions_like_extends_in_compiled_output_6157\",\"Do not generate custom helper functions like '__extends' in compiled output.\"),Do_not_include_the_default_library_file_lib_d_ts:n(6158,e.DiagnosticCategory.Message,\"Do_not_include_the_default_library_file_lib_d_ts_6158\",\"Do not include the default library file (lib.d.ts).\"),Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files:n(6159,e.DiagnosticCategory.Message,\"Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files_6159\",\"Do not add triple-slash references or imported modules to the list of compiled files.\"),Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files:n(6160,e.DiagnosticCategory.Message,\"Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files_6160\",\"[Deprecated] Use '--skipLibCheck' instead. Skip type checking of default library declaration files.\"),List_of_folders_to_include_type_definitions_from:n(6161,e.DiagnosticCategory.Message,\"List_of_folders_to_include_type_definitions_from_6161\",\"List of folders to include type definitions from.\"),Disable_size_limitations_on_JavaScript_projects:n(6162,e.DiagnosticCategory.Message,\"Disable_size_limitations_on_JavaScript_projects_6162\",\"Disable size limitations on JavaScript projects.\"),The_character_set_of_the_input_files:n(6163,e.DiagnosticCategory.Message,\"The_character_set_of_the_input_files_6163\",\"The character set of the input files.\"),Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files:n(6164,e.DiagnosticCategory.Message,\"Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files_6164\",\"Emit a UTF-8 Byte Order Mark (BOM) in the beginning of output files.\"),Do_not_truncate_error_messages:n(6165,e.DiagnosticCategory.Message,\"Do_not_truncate_error_messages_6165\",\"Do not truncate error messages.\"),Output_directory_for_generated_declaration_files:n(6166,e.DiagnosticCategory.Message,\"Output_directory_for_generated_declaration_files_6166\",\"Output directory for generated declaration files.\"),A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl:n(6167,e.DiagnosticCategory.Message,\"A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl_6167\",\"A series of entries which re-map imports to lookup locations relative to the 'baseUrl'.\"),List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime:n(6168,e.DiagnosticCategory.Message,\"List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime_6168\",\"List of root folders whose combined content represents the structure of the project at runtime.\"),Show_all_compiler_options:n(6169,e.DiagnosticCategory.Message,\"Show_all_compiler_options_6169\",\"Show all compiler options.\"),Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file:n(6170,e.DiagnosticCategory.Message,\"Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file_6170\",\"[Deprecated] Use '--outFile' instead. Concatenate and emit output to single file\"),Command_line_Options:n(6171,e.DiagnosticCategory.Message,\"Command_line_Options_6171\",\"Command-line Options\"),Basic_Options:n(6172,e.DiagnosticCategory.Message,\"Basic_Options_6172\",\"Basic Options\"),Strict_Type_Checking_Options:n(6173,e.DiagnosticCategory.Message,\"Strict_Type_Checking_Options_6173\",\"Strict Type-Checking Options\"),Module_Resolution_Options:n(6174,e.DiagnosticCategory.Message,\"Module_Resolution_Options_6174\",\"Module Resolution Options\"),Source_Map_Options:n(6175,e.DiagnosticCategory.Message,\"Source_Map_Options_6175\",\"Source Map Options\"),Additional_Checks:n(6176,e.DiagnosticCategory.Message,\"Additional_Checks_6176\",\"Additional Checks\"),Experimental_Options:n(6177,e.DiagnosticCategory.Message,\"Experimental_Options_6177\",\"Experimental Options\"),Advanced_Options:n(6178,e.DiagnosticCategory.Message,\"Advanced_Options_6178\",\"Advanced Options\"),Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3:n(6179,e.DiagnosticCategory.Message,\"Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3_6179\",\"Provide full support for iterables in 'for-of', spread, and destructuring when targeting 'ES5' or 'ES3'.\"),Enable_all_strict_type_checking_options:n(6180,e.DiagnosticCategory.Message,\"Enable_all_strict_type_checking_options_6180\",\"Enable all strict type-checking options.\"),List_of_language_service_plugins:n(6181,e.DiagnosticCategory.Message,\"List_of_language_service_plugins_6181\",\"List of language service plugins.\"),Scoped_package_detected_looking_in_0:n(6182,e.DiagnosticCategory.Message,\"Scoped_package_detected_looking_in_0_6182\",\"Scoped package detected, looking in '{0}'\"),Reusing_resolution_of_module_0_to_file_1_from_old_program:n(6183,e.DiagnosticCategory.Message,\"Reusing_resolution_of_module_0_to_file_1_from_old_program_6183\",\"Reusing resolution of module '{0}' to file '{1}' from old program.\"),Reusing_module_resolutions_originating_in_0_since_resolutions_are_unchanged_from_old_program:n(6184,e.DiagnosticCategory.Message,\"Reusing_module_resolutions_originating_in_0_since_resolutions_are_unchanged_from_old_program_6184\",\"Reusing module resolutions originating in '{0}' since resolutions are unchanged from old program.\"),Disable_strict_checking_of_generic_signatures_in_function_types:n(6185,e.DiagnosticCategory.Message,\"Disable_strict_checking_of_generic_signatures_in_function_types_6185\",\"Disable strict checking of generic signatures in function types.\"),Enable_strict_checking_of_function_types:n(6186,e.DiagnosticCategory.Message,\"Enable_strict_checking_of_function_types_6186\",\"Enable strict checking of function types.\"),Enable_strict_checking_of_property_initialization_in_classes:n(6187,e.DiagnosticCategory.Message,\"Enable_strict_checking_of_property_initialization_in_classes_6187\",\"Enable strict checking of property initialization in classes.\"),Numeric_separators_are_not_allowed_here:n(6188,e.DiagnosticCategory.Error,\"Numeric_separators_are_not_allowed_here_6188\",\"Numeric separators are not allowed here.\"),Multiple_consecutive_numeric_separators_are_not_permitted:n(6189,e.DiagnosticCategory.Error,\"Multiple_consecutive_numeric_separators_are_not_permitted_6189\",\"Multiple consecutive numeric separators are not permitted.\"),Found_package_json_at_0_Package_ID_is_1:n(6190,e.DiagnosticCategory.Message,\"Found_package_json_at_0_Package_ID_is_1_6190\",\"Found 'package.json' at '{0}'. Package ID is '{1}'.\"),Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen:n(6191,e.DiagnosticCategory.Message,\"Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen_6191\",\"Whether to keep outdated console output in watch mode instead of clearing the screen.\"),All_imports_in_import_declaration_are_unused:n(6192,e.DiagnosticCategory.Error,\"All_imports_in_import_declaration_are_unused_6192\",\"All imports in import declaration are unused.\",!0),Found_1_error_Watching_for_file_changes:n(6193,e.DiagnosticCategory.Message,\"Found_1_error_Watching_for_file_changes_6193\",\"Found 1 error. Watching for file changes.\"),Found_0_errors_Watching_for_file_changes:n(6194,e.DiagnosticCategory.Message,\"Found_0_errors_Watching_for_file_changes_6194\",\"Found {0} errors. Watching for file changes.\"),Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols:n(6195,e.DiagnosticCategory.Message,\"Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols_6195\",\"Resolve 'keyof' to string valued property names only (no numbers or symbols).\"),_0_is_declared_but_never_used:n(6196,e.DiagnosticCategory.Error,\"_0_is_declared_but_never_used_6196\",\"'{0}' is declared but never used.\",!0),Include_modules_imported_with_json_extension:n(6197,e.DiagnosticCategory.Message,\"Include_modules_imported_with_json_extension_6197\",\"Include modules imported with '.json' extension\"),All_destructured_elements_are_unused:n(6198,e.DiagnosticCategory.Error,\"All_destructured_elements_are_unused_6198\",\"All destructured elements are unused.\",!0),All_variables_are_unused:n(6199,e.DiagnosticCategory.Error,\"All_variables_are_unused_6199\",\"All variables are unused.\",!0),Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0:n(6200,e.DiagnosticCategory.Error,\"Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0_6200\",\"Definitions of the following identifiers conflict with those in another file: {0}\"),Conflicts_are_in_this_file:n(6201,e.DiagnosticCategory.Message,\"Conflicts_are_in_this_file_6201\",\"Conflicts are in this file.\"),_0_was_also_declared_here:n(6203,e.DiagnosticCategory.Message,\"_0_was_also_declared_here_6203\",\"'{0}' was also declared here.\"),and_here:n(6204,e.DiagnosticCategory.Message,\"and_here_6204\",\"and here.\"),All_type_parameters_are_unused:n(6205,e.DiagnosticCategory.Error,\"All_type_parameters_are_unused_6205\",\"All type parameters are unused\"),package_json_has_a_typesVersions_field_with_version_specific_path_mappings:n(6206,e.DiagnosticCategory.Message,\"package_json_has_a_typesVersions_field_with_version_specific_path_mappings_6206\",\"'package.json' has a 'typesVersions' field with version-specific path mappings.\"),package_json_does_not_have_a_typesVersions_entry_that_matches_version_0:n(6207,e.DiagnosticCategory.Message,\"package_json_does_not_have_a_typesVersions_entry_that_matches_version_0_6207\",\"'package.json' does not have a 'typesVersions' entry that matches version '{0}'.\"),package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2:n(6208,e.DiagnosticCategory.Message,\"package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_ma_6208\",\"'package.json' has a 'typesVersions' entry '{0}' that matches compiler version '{1}', looking for a pattern to match module name '{2}'.\"),package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range:n(6209,e.DiagnosticCategory.Message,\"package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range_6209\",\"'package.json' has a 'typesVersions' entry '{0}' that is not a valid semver range.\"),An_argument_for_0_was_not_provided:n(6210,e.DiagnosticCategory.Message,\"An_argument_for_0_was_not_provided_6210\",\"An argument for '{0}' was not provided.\"),An_argument_matching_this_binding_pattern_was_not_provided:n(6211,e.DiagnosticCategory.Message,\"An_argument_matching_this_binding_pattern_was_not_provided_6211\",\"An argument matching this binding pattern was not provided.\"),Did_you_mean_to_call_this_expression:n(6212,e.DiagnosticCategory.Message,\"Did_you_mean_to_call_this_expression_6212\",\"Did you mean to call this expression?\"),Did_you_mean_to_use_new_with_this_expression:n(6213,e.DiagnosticCategory.Message,\"Did_you_mean_to_use_new_with_this_expression_6213\",\"Did you mean to use 'new' with this expression?\"),Projects_to_reference:n(6300,e.DiagnosticCategory.Message,\"Projects_to_reference_6300\",\"Projects to reference\"),Enable_project_compilation:n(6302,e.DiagnosticCategory.Message,\"Enable_project_compilation_6302\",\"Enable project compilation\"),Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0:n(6202,e.DiagnosticCategory.Error,\"Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0_6202\",\"Project references may not form a circular graph. Cycle detected: {0}\"),Composite_projects_may_not_disable_declaration_emit:n(6304,e.DiagnosticCategory.Error,\"Composite_projects_may_not_disable_declaration_emit_6304\",\"Composite projects may not disable declaration emit.\"),Output_file_0_has_not_been_built_from_source_file_1:n(6305,e.DiagnosticCategory.Error,\"Output_file_0_has_not_been_built_from_source_file_1_6305\",\"Output file '{0}' has not been built from source file '{1}'.\"),Referenced_project_0_must_have_setting_composite_Colon_true:n(6306,e.DiagnosticCategory.Error,\"Referenced_project_0_must_have_setting_composite_Colon_true_6306\",\"Referenced project '{0}' must have setting \\\"composite\\\": true.\"),File_0_is_not_in_project_file_list_Projects_must_list_all_files_or_use_an_include_pattern:n(6307,e.DiagnosticCategory.Error,\"File_0_is_not_in_project_file_list_Projects_must_list_all_files_or_use_an_include_pattern_6307\",\"File '{0}' is not in project file list. Projects must list all files or use an 'include' pattern.\"),Cannot_prepend_project_0_because_it_does_not_have_outFile_set:n(6308,e.DiagnosticCategory.Error,\"Cannot_prepend_project_0_because_it_does_not_have_outFile_set_6308\",\"Cannot prepend project '{0}' because it does not have 'outFile' set\"),Output_file_0_from_project_1_does_not_exist:n(6309,e.DiagnosticCategory.Error,\"Output_file_0_from_project_1_does_not_exist_6309\",\"Output file '{0}' from project '{1}' does not exist\"),Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2:n(6350,e.DiagnosticCategory.Message,\"Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2_6350\",\"Project '{0}' is out of date because oldest output '{1}' is older than newest input '{2}'\"),Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2:n(6351,e.DiagnosticCategory.Message,\"Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2_6351\",\"Project '{0}' is up to date because newest input '{1}' is older than oldest output '{2}'\"),Project_0_is_out_of_date_because_output_file_1_does_not_exist:n(6352,e.DiagnosticCategory.Message,\"Project_0_is_out_of_date_because_output_file_1_does_not_exist_6352\",\"Project '{0}' is out of date because output file '{1}' does not exist\"),Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date:n(6353,e.DiagnosticCategory.Message,\"Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date_6353\",\"Project '{0}' is out of date because its dependency '{1}' is out of date\"),Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies:n(6354,e.DiagnosticCategory.Message,\"Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies_6354\",\"Project '{0}' is up to date with .d.ts files from its dependencies\"),Projects_in_this_build_Colon_0:n(6355,e.DiagnosticCategory.Message,\"Projects_in_this_build_Colon_0_6355\",\"Projects in this build: {0}\"),A_non_dry_build_would_delete_the_following_files_Colon_0:n(6356,e.DiagnosticCategory.Message,\"A_non_dry_build_would_delete_the_following_files_Colon_0_6356\",\"A non-dry build would delete the following files: {0}\"),A_non_dry_build_would_build_project_0:n(6357,e.DiagnosticCategory.Message,\"A_non_dry_build_would_build_project_0_6357\",\"A non-dry build would build project '{0}'\"),Building_project_0:n(6358,e.DiagnosticCategory.Message,\"Building_project_0_6358\",\"Building project '{0}'...\"),Updating_output_timestamps_of_project_0:n(6359,e.DiagnosticCategory.Message,\"Updating_output_timestamps_of_project_0_6359\",\"Updating output timestamps of project '{0}'...\"),delete_this_Project_0_is_up_to_date_because_it_was_previously_built:n(6360,e.DiagnosticCategory.Message,\"delete_this_Project_0_is_up_to_date_because_it_was_previously_built_6360\",\"delete this - Project '{0}' is up to date because it was previously built\"),Project_0_is_up_to_date:n(6361,e.DiagnosticCategory.Message,\"Project_0_is_up_to_date_6361\",\"Project '{0}' is up to date\"),Skipping_build_of_project_0_because_its_dependency_1_has_errors:n(6362,e.DiagnosticCategory.Message,\"Skipping_build_of_project_0_because_its_dependency_1_has_errors_6362\",\"Skipping build of project '{0}' because its dependency '{1}' has errors\"),Project_0_can_t_be_built_because_its_dependency_1_has_errors:n(6363,e.DiagnosticCategory.Message,\"Project_0_can_t_be_built_because_its_dependency_1_has_errors_6363\",\"Project '{0}' can't be built because its dependency '{1}' has errors\"),Build_one_or_more_projects_and_their_dependencies_if_out_of_date:n(6364,e.DiagnosticCategory.Message,\"Build_one_or_more_projects_and_their_dependencies_if_out_of_date_6364\",\"Build one or more projects and their dependencies, if out of date\"),Delete_the_outputs_of_all_projects:n(6365,e.DiagnosticCategory.Message,\"Delete_the_outputs_of_all_projects_6365\",\"Delete the outputs of all projects\"),Enable_verbose_logging:n(6366,e.DiagnosticCategory.Message,\"Enable_verbose_logging_6366\",\"Enable verbose logging\"),Show_what_would_be_built_or_deleted_if_specified_with_clean:n(6367,e.DiagnosticCategory.Message,\"Show_what_would_be_built_or_deleted_if_specified_with_clean_6367\",\"Show what would be built (or deleted, if specified with '--clean')\"),Build_all_projects_including_those_that_appear_to_be_up_to_date:n(6368,e.DiagnosticCategory.Message,\"Build_all_projects_including_those_that_appear_to_be_up_to_date_6368\",\"Build all projects, including those that appear to be up to date\"),Option_build_must_be_the_first_command_line_argument:n(6369,e.DiagnosticCategory.Error,\"Option_build_must_be_the_first_command_line_argument_6369\",\"Option '--build' must be the first command line argument.\"),Options_0_and_1_cannot_be_combined:n(6370,e.DiagnosticCategory.Error,\"Options_0_and_1_cannot_be_combined_6370\",\"Options '{0}' and '{1}' cannot be combined.\"),The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1:n(6500,e.DiagnosticCategory.Message,\"The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1_6500\",\"The expected type comes from property '{0}' which is declared here on type '{1}'\"),The_expected_type_comes_from_this_index_signature:n(6501,e.DiagnosticCategory.Message,\"The_expected_type_comes_from_this_index_signature_6501\",\"The expected type comes from this index signature.\"),The_expected_type_comes_from_the_return_type_of_this_signature:n(6502,e.DiagnosticCategory.Message,\"The_expected_type_comes_from_the_return_type_of_this_signature_6502\",\"The expected type comes from the return type of this signature.\"),Variable_0_implicitly_has_an_1_type:n(7005,e.DiagnosticCategory.Error,\"Variable_0_implicitly_has_an_1_type_7005\",\"Variable '{0}' implicitly has an '{1}' type.\"),Parameter_0_implicitly_has_an_1_type:n(7006,e.DiagnosticCategory.Error,\"Parameter_0_implicitly_has_an_1_type_7006\",\"Parameter '{0}' implicitly has an '{1}' type.\"),Member_0_implicitly_has_an_1_type:n(7008,e.DiagnosticCategory.Error,\"Member_0_implicitly_has_an_1_type_7008\",\"Member '{0}' implicitly has an '{1}' type.\"),new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type:n(7009,e.DiagnosticCategory.Error,\"new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type_7009\",\"'new' expression, whose target lacks a construct signature, implicitly has an 'any' type.\"),_0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type:n(7010,e.DiagnosticCategory.Error,\"_0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type_7010\",\"'{0}', which lacks return-type annotation, implicitly has an '{1}' return type.\"),Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type:n(7011,e.DiagnosticCategory.Error,\"Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type_7011\",\"Function expression, which lacks return-type annotation, implicitly has an '{0}' return type.\"),Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type:n(7013,e.DiagnosticCategory.Error,\"Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type_7013\",\"Construct signature, which lacks return-type annotation, implicitly has an 'any' return type.\"),Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number:n(7015,e.DiagnosticCategory.Error,\"Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number_7015\",\"Element implicitly has an 'any' type because index expression is not of type 'number'.\"),Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type:n(7016,e.DiagnosticCategory.Error,\"Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type_7016\",\"Could not find a declaration file for module '{0}'. '{1}' implicitly has an 'any' type.\"),Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature:n(7017,e.DiagnosticCategory.Error,\"Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_7017\",\"Element implicitly has an 'any' type because type '{0}' has no index signature.\"),Object_literal_s_property_0_implicitly_has_an_1_type:n(7018,e.DiagnosticCategory.Error,\"Object_literal_s_property_0_implicitly_has_an_1_type_7018\",\"Object literal's property '{0}' implicitly has an '{1}' type.\"),Rest_parameter_0_implicitly_has_an_any_type:n(7019,e.DiagnosticCategory.Error,\"Rest_parameter_0_implicitly_has_an_any_type_7019\",\"Rest parameter '{0}' implicitly has an 'any[]' type.\"),Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type:n(7020,e.DiagnosticCategory.Error,\"Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type_7020\",\"Call signature, which lacks return-type annotation, implicitly has an 'any' return type.\"),_0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or_indirectly_in_its_own_initializer:n(7022,e.DiagnosticCategory.Error,\"_0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or__7022\",\"'{0}' implicitly has type 'any' because it does not have a type annotation and is referenced directly or indirectly in its own initializer.\"),_0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions:n(7023,e.DiagnosticCategory.Error,\"_0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_reference_7023\",\"'{0}' implicitly has return type 'any' because it does not have a return type annotation and is referenced directly or indirectly in one of its return expressions.\"),Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions:n(7024,e.DiagnosticCategory.Error,\"Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_ref_7024\",\"Function implicitly has return type 'any' because it does not have a return type annotation and is referenced directly or indirectly in one of its return expressions.\"),Generator_implicitly_has_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_return_type:n(7025,e.DiagnosticCategory.Error,\"Generator_implicitly_has_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_return_typ_7025\",\"Generator implicitly has type '{0}' because it does not yield any values. Consider supplying a return type.\"),JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists:n(7026,e.DiagnosticCategory.Error,\"JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists_7026\",\"JSX element implicitly has type 'any' because no interface 'JSX.{0}' exists.\"),Unreachable_code_detected:n(7027,e.DiagnosticCategory.Error,\"Unreachable_code_detected_7027\",\"Unreachable code detected.\",!0),Unused_label:n(7028,e.DiagnosticCategory.Error,\"Unused_label_7028\",\"Unused label.\",!0),Fallthrough_case_in_switch:n(7029,e.DiagnosticCategory.Error,\"Fallthrough_case_in_switch_7029\",\"Fallthrough case in switch.\"),Not_all_code_paths_return_a_value:n(7030,e.DiagnosticCategory.Error,\"Not_all_code_paths_return_a_value_7030\",\"Not all code paths return a value.\"),Binding_element_0_implicitly_has_an_1_type:n(7031,e.DiagnosticCategory.Error,\"Binding_element_0_implicitly_has_an_1_type_7031\",\"Binding element '{0}' implicitly has an '{1}' type.\"),Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation:n(7032,e.DiagnosticCategory.Error,\"Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation_7032\",\"Property '{0}' implicitly has type 'any', because its set accessor lacks a parameter type annotation.\"),Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation:n(7033,e.DiagnosticCategory.Error,\"Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation_7033\",\"Property '{0}' implicitly has type 'any', because its get accessor lacks a return type annotation.\"),Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined:n(7034,e.DiagnosticCategory.Error,\"Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined_7034\",\"Variable '{0}' implicitly has type '{1}' in some locations where its type cannot be determined.\"),Try_npm_install_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_module_0:n(7035,e.DiagnosticCategory.Error,\"Try_npm_install_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_mod_7035\",\"Try `npm install @types/{1}` if it exists or add a new declaration (.d.ts) file containing `declare module '{0}';`\"),Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0:n(7036,e.DiagnosticCategory.Error,\"Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0_7036\",\"Dynamic import's specifier must be of type 'string', but here has type '{0}'.\"),Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for_all_imports_Implies_allowSyntheticDefaultImports:n(7037,e.DiagnosticCategory.Message,\"Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for__7037\",\"Enables emit interoperability between CommonJS and ES Modules via creation of namespace objects for all imports. Implies 'allowSyntheticDefaultImports'.\"),Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead:n(7038,e.DiagnosticCategory.Message,\"Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cau_7038\",\"Type originates at this import. A namespace-style import cannot be called or constructed, and will cause a failure at runtime. Consider using a default import or import require here instead.\"),Mapped_object_type_implicitly_has_an_any_template_type:n(7039,e.DiagnosticCategory.Error,\"Mapped_object_type_implicitly_has_an_any_template_type_7039\",\"Mapped object type implicitly has an 'any' template type.\"),If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_Slash_Slashgithub_com_SlashDefinitelyTyped_SlashDefinitelyTyped_Slashtree_Slashmaster_Slashtypes_Slash_1:n(7040,e.DiagnosticCategory.Error,\"If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_S_7040\",\"If the '{0}' package actually exposes this module, consider sending a pull request to amend 'https://github.com/DefinitelyTyped/DefinitelyTyped/tree/master/types/{1}`\"),The_containing_arrow_function_captures_the_global_value_of_this_which_implicitly_has_type_any:n(7041,e.DiagnosticCategory.Error,\"The_containing_arrow_function_captures_the_global_value_of_this_which_implicitly_has_type_any_7041\",\"The containing arrow function captures the global value of 'this' which implicitly has type 'any'.\"),Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used:n(7042,e.DiagnosticCategory.Error,\"Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used_7042\",\"Module '{0}' was resolved to '{1}', but '--resolveJsonModule' is not used.\"),You_cannot_rename_this_element:n(8e3,e.DiagnosticCategory.Error,\"You_cannot_rename_this_element_8000\",\"You cannot rename this element.\"),You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library:n(8001,e.DiagnosticCategory.Error,\"You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library_8001\",\"You cannot rename elements that are defined in the standard TypeScript library.\"),import_can_only_be_used_in_a_ts_file:n(8002,e.DiagnosticCategory.Error,\"import_can_only_be_used_in_a_ts_file_8002\",\"'import ... =' can only be used in a .ts file.\"),export_can_only_be_used_in_a_ts_file:n(8003,e.DiagnosticCategory.Error,\"export_can_only_be_used_in_a_ts_file_8003\",\"'export=' can only be used in a .ts file.\"),type_parameter_declarations_can_only_be_used_in_a_ts_file:n(8004,e.DiagnosticCategory.Error,\"type_parameter_declarations_can_only_be_used_in_a_ts_file_8004\",\"'type parameter declarations' can only be used in a .ts file.\"),implements_clauses_can_only_be_used_in_a_ts_file:n(8005,e.DiagnosticCategory.Error,\"implements_clauses_can_only_be_used_in_a_ts_file_8005\",\"'implements clauses' can only be used in a .ts file.\"),interface_declarations_can_only_be_used_in_a_ts_file:n(8006,e.DiagnosticCategory.Error,\"interface_declarations_can_only_be_used_in_a_ts_file_8006\",\"'interface declarations' can only be used in a .ts file.\"),module_declarations_can_only_be_used_in_a_ts_file:n(8007,e.DiagnosticCategory.Error,\"module_declarations_can_only_be_used_in_a_ts_file_8007\",\"'module declarations' can only be used in a .ts file.\"),type_aliases_can_only_be_used_in_a_ts_file:n(8008,e.DiagnosticCategory.Error,\"type_aliases_can_only_be_used_in_a_ts_file_8008\",\"'type aliases' can only be used in a .ts file.\"),_0_can_only_be_used_in_a_ts_file:n(8009,e.DiagnosticCategory.Error,\"_0_can_only_be_used_in_a_ts_file_8009\",\"'{0}' can only be used in a .ts file.\"),types_can_only_be_used_in_a_ts_file:n(8010,e.DiagnosticCategory.Error,\"types_can_only_be_used_in_a_ts_file_8010\",\"'types' can only be used in a .ts file.\"),type_arguments_can_only_be_used_in_a_ts_file:n(8011,e.DiagnosticCategory.Error,\"type_arguments_can_only_be_used_in_a_ts_file_8011\",\"'type arguments' can only be used in a .ts file.\"),parameter_modifiers_can_only_be_used_in_a_ts_file:n(8012,e.DiagnosticCategory.Error,\"parameter_modifiers_can_only_be_used_in_a_ts_file_8012\",\"'parameter modifiers' can only be used in a .ts file.\"),non_null_assertions_can_only_be_used_in_a_ts_file:n(8013,e.DiagnosticCategory.Error,\"non_null_assertions_can_only_be_used_in_a_ts_file_8013\",\"'non-null assertions' can only be used in a .ts file.\"),enum_declarations_can_only_be_used_in_a_ts_file:n(8015,e.DiagnosticCategory.Error,\"enum_declarations_can_only_be_used_in_a_ts_file_8015\",\"'enum declarations' can only be used in a .ts file.\"),type_assertion_expressions_can_only_be_used_in_a_ts_file:n(8016,e.DiagnosticCategory.Error,\"type_assertion_expressions_can_only_be_used_in_a_ts_file_8016\",\"'type assertion expressions' can only be used in a .ts file.\"),Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0:n(8017,e.DiagnosticCategory.Error,\"Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0_8017\",\"Octal literal types must use ES2015 syntax. Use the syntax '{0}'.\"),Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0:n(8018,e.DiagnosticCategory.Error,\"Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0_8018\",\"Octal literals are not allowed in enums members initializer. Use the syntax '{0}'.\"),Report_errors_in_js_files:n(8019,e.DiagnosticCategory.Message,\"Report_errors_in_js_files_8019\",\"Report errors in .js files.\"),JSDoc_types_can_only_be_used_inside_documentation_comments:n(8020,e.DiagnosticCategory.Error,\"JSDoc_types_can_only_be_used_inside_documentation_comments_8020\",\"JSDoc types can only be used inside documentation comments.\"),JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags:n(8021,e.DiagnosticCategory.Error,\"JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags_8021\",\"JSDoc '@typedef' tag should either have a type annotation or be followed by '@property' or '@member' tags.\"),JSDoc_0_is_not_attached_to_a_class:n(8022,e.DiagnosticCategory.Error,\"JSDoc_0_is_not_attached_to_a_class_8022\",\"JSDoc '@{0}' is not attached to a class.\"),JSDoc_0_1_does_not_match_the_extends_2_clause:n(8023,e.DiagnosticCategory.Error,\"JSDoc_0_1_does_not_match_the_extends_2_clause_8023\",\"JSDoc '@{0} {1}' does not match the 'extends {2}' clause.\"),JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name:n(8024,e.DiagnosticCategory.Error,\"JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_8024\",\"JSDoc '@param' tag has name '{0}', but there is no parameter with that name.\"),Class_declarations_cannot_have_more_than_one_augments_or_extends_tag:n(8025,e.DiagnosticCategory.Error,\"Class_declarations_cannot_have_more_than_one_augments_or_extends_tag_8025\",\"Class declarations cannot have more than one `@augments` or `@extends` tag.\"),Expected_0_type_arguments_provide_these_with_an_extends_tag:n(8026,e.DiagnosticCategory.Error,\"Expected_0_type_arguments_provide_these_with_an_extends_tag_8026\",\"Expected {0} type arguments; provide these with an '@extends' tag.\"),Expected_0_1_type_arguments_provide_these_with_an_extends_tag:n(8027,e.DiagnosticCategory.Error,\"Expected_0_1_type_arguments_provide_these_with_an_extends_tag_8027\",\"Expected {0}-{1} type arguments; provide these with an '@extends' tag.\"),JSDoc_may_only_appear_in_the_last_parameter_of_a_signature:n(8028,e.DiagnosticCategory.Error,\"JSDoc_may_only_appear_in_the_last_parameter_of_a_signature_8028\",\"JSDoc '...' may only appear in the last parameter of a signature.\"),JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_had_an_array_type:n(8029,e.DiagnosticCategory.Error,\"JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_h_8029\",\"JSDoc '@param' tag has name '{0}', but there is no parameter with that name. It would match 'arguments' if it had an array type.\"),The_type_of_a_function_declaration_must_match_the_function_s_signature:n(8030,e.DiagnosticCategory.Error,\"The_type_of_a_function_declaration_must_match_the_function_s_signature_8030\",\"The type of a function declaration must match the function's signature.\"),You_cannot_rename_a_module_via_a_global_import:n(8031,e.DiagnosticCategory.Error,\"You_cannot_rename_a_module_via_a_global_import_8031\",\"You cannot rename a module via a global import.\"),Only_identifiers_Slashqualified_names_with_optional_type_arguments_are_currently_supported_in_a_class_extends_clause:n(9002,e.DiagnosticCategory.Error,\"Only_identifiers_Slashqualified_names_with_optional_type_arguments_are_currently_supported_in_a_clas_9002\",\"Only identifiers/qualified-names with optional type arguments are currently supported in a class 'extends' clause.\"),class_expressions_are_not_currently_supported:n(9003,e.DiagnosticCategory.Error,\"class_expressions_are_not_currently_supported_9003\",\"'class' expressions are not currently supported.\"),Language_service_is_disabled:n(9004,e.DiagnosticCategory.Error,\"Language_service_is_disabled_9004\",\"Language service is disabled.\"),JSX_attributes_must_only_be_assigned_a_non_empty_expression:n(17e3,e.DiagnosticCategory.Error,\"JSX_attributes_must_only_be_assigned_a_non_empty_expression_17000\",\"JSX attributes must only be assigned a non-empty 'expression'.\"),JSX_elements_cannot_have_multiple_attributes_with_the_same_name:n(17001,e.DiagnosticCategory.Error,\"JSX_elements_cannot_have_multiple_attributes_with_the_same_name_17001\",\"JSX elements cannot have multiple attributes with the same name.\"),Expected_corresponding_JSX_closing_tag_for_0:n(17002,e.DiagnosticCategory.Error,\"Expected_corresponding_JSX_closing_tag_for_0_17002\",\"Expected corresponding JSX closing tag for '{0}'.\"),JSX_attribute_expected:n(17003,e.DiagnosticCategory.Error,\"JSX_attribute_expected_17003\",\"JSX attribute expected.\"),Cannot_use_JSX_unless_the_jsx_flag_is_provided:n(17004,e.DiagnosticCategory.Error,\"Cannot_use_JSX_unless_the_jsx_flag_is_provided_17004\",\"Cannot use JSX unless the '--jsx' flag is provided.\"),A_constructor_cannot_contain_a_super_call_when_its_class_extends_null:n(17005,e.DiagnosticCategory.Error,\"A_constructor_cannot_contain_a_super_call_when_its_class_extends_null_17005\",\"A constructor cannot contain a 'super' call when its class extends 'null'.\"),An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses:n(17006,e.DiagnosticCategory.Error,\"An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_ex_17006\",\"An unary expression with the '{0}' operator is not allowed in the left-hand side of an exponentiation expression. Consider enclosing the expression in parentheses.\"),A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses:n(17007,e.DiagnosticCategory.Error,\"A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Con_17007\",\"A type assertion expression is not allowed in the left-hand side of an exponentiation expression. Consider enclosing the expression in parentheses.\"),JSX_element_0_has_no_corresponding_closing_tag:n(17008,e.DiagnosticCategory.Error,\"JSX_element_0_has_no_corresponding_closing_tag_17008\",\"JSX element '{0}' has no corresponding closing tag.\"),super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class:n(17009,e.DiagnosticCategory.Error,\"super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class_17009\",\"'super' must be called before accessing 'this' in the constructor of a derived class.\"),Unknown_type_acquisition_option_0:n(17010,e.DiagnosticCategory.Error,\"Unknown_type_acquisition_option_0_17010\",\"Unknown type acquisition option '{0}'.\"),super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class:n(17011,e.DiagnosticCategory.Error,\"super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class_17011\",\"'super' must be called before accessing a property of 'super' in the constructor of a derived class.\"),_0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2:n(17012,e.DiagnosticCategory.Error,\"_0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2_17012\",\"'{0}' is not a valid meta-property for keyword '{1}'. Did you mean '{2}'?\"),Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constructor:n(17013,e.DiagnosticCategory.Error,\"Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constru_17013\",\"Meta-property '{0}' is only allowed in the body of a function declaration, function expression, or constructor.\"),JSX_fragment_has_no_corresponding_closing_tag:n(17014,e.DiagnosticCategory.Error,\"JSX_fragment_has_no_corresponding_closing_tag_17014\",\"JSX fragment has no corresponding closing tag.\"),Expected_corresponding_closing_tag_for_JSX_fragment:n(17015,e.DiagnosticCategory.Error,\"Expected_corresponding_closing_tag_for_JSX_fragment_17015\",\"Expected corresponding closing tag for JSX fragment.\"),JSX_fragment_is_not_supported_when_using_jsxFactory:n(17016,e.DiagnosticCategory.Error,\"JSX_fragment_is_not_supported_when_using_jsxFactory_17016\",\"JSX fragment is not supported when using --jsxFactory\"),JSX_fragment_is_not_supported_when_using_an_inline_JSX_factory_pragma:n(17017,e.DiagnosticCategory.Error,\"JSX_fragment_is_not_supported_when_using_an_inline_JSX_factory_pragma_17017\",\"JSX fragment is not supported when using an inline JSX factory pragma\"),Circularity_detected_while_resolving_configuration_Colon_0:n(18e3,e.DiagnosticCategory.Error,\"Circularity_detected_while_resolving_configuration_Colon_0_18000\",\"Circularity detected while resolving configuration: {0}\"),A_path_in_an_extends_option_must_be_relative_or_rooted_but_0_is_not:n(18001,e.DiagnosticCategory.Error,\"A_path_in_an_extends_option_must_be_relative_or_rooted_but_0_is_not_18001\",\"A path in an 'extends' option must be relative or rooted, but '{0}' is not.\"),The_files_list_in_config_file_0_is_empty:n(18002,e.DiagnosticCategory.Error,\"The_files_list_in_config_file_0_is_empty_18002\",\"The 'files' list in config file '{0}' is empty.\"),No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2:n(18003,e.DiagnosticCategory.Error,\"No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2_18003\",\"No inputs were found in config file '{0}'. Specified 'include' paths were '{1}' and 'exclude' paths were '{2}'.\"),File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module:n(80001,e.DiagnosticCategory.Suggestion,\"File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module_80001\",\"File is a CommonJS module; it may be converted to an ES6 module.\"),This_constructor_function_may_be_converted_to_a_class_declaration:n(80002,e.DiagnosticCategory.Suggestion,\"This_constructor_function_may_be_converted_to_a_class_declaration_80002\",\"This constructor function may be converted to a class declaration.\"),Import_may_be_converted_to_a_default_import:n(80003,e.DiagnosticCategory.Suggestion,\"Import_may_be_converted_to_a_default_import_80003\",\"Import may be converted to a default import.\"),JSDoc_types_may_be_moved_to_TypeScript_types:n(80004,e.DiagnosticCategory.Suggestion,\"JSDoc_types_may_be_moved_to_TypeScript_types_80004\",\"JSDoc types may be moved to TypeScript types.\"),require_call_may_be_converted_to_an_import:n(80005,e.DiagnosticCategory.Suggestion,\"require_call_may_be_converted_to_an_import_80005\",\"'require' call may be converted to an import.\"),This_may_be_converted_to_an_async_function:n(80006,e.DiagnosticCategory.Suggestion,\"This_may_be_converted_to_an_async_function_80006\",\"This may be converted to an async function.\"),Add_missing_super_call:n(90001,e.DiagnosticCategory.Message,\"Add_missing_super_call_90001\",\"Add missing 'super()' call\"),Make_super_call_the_first_statement_in_the_constructor:n(90002,e.DiagnosticCategory.Message,\"Make_super_call_the_first_statement_in_the_constructor_90002\",\"Make 'super()' call the first statement in the constructor\"),Change_extends_to_implements:n(90003,e.DiagnosticCategory.Message,\"Change_extends_to_implements_90003\",\"Change 'extends' to 'implements'\"),Remove_declaration_for_Colon_0:n(90004,e.DiagnosticCategory.Message,\"Remove_declaration_for_Colon_0_90004\",\"Remove declaration for: '{0}'\"),Remove_import_from_0:n(90005,e.DiagnosticCategory.Message,\"Remove_import_from_0_90005\",\"Remove import from '{0}'\"),Implement_interface_0:n(90006,e.DiagnosticCategory.Message,\"Implement_interface_0_90006\",\"Implement interface '{0}'\"),Implement_inherited_abstract_class:n(90007,e.DiagnosticCategory.Message,\"Implement_inherited_abstract_class_90007\",\"Implement inherited abstract class\"),Add_0_to_unresolved_variable:n(90008,e.DiagnosticCategory.Message,\"Add_0_to_unresolved_variable_90008\",\"Add '{0}.' to unresolved variable\"),Remove_destructuring:n(90009,e.DiagnosticCategory.Message,\"Remove_destructuring_90009\",\"Remove destructuring\"),Remove_variable_statement:n(90010,e.DiagnosticCategory.Message,\"Remove_variable_statement_90010\",\"Remove variable statement\"),Remove_template_tag:n(90011,e.DiagnosticCategory.Message,\"Remove_template_tag_90011\",\"Remove template tag\"),Remove_type_parameters:n(90012,e.DiagnosticCategory.Message,\"Remove_type_parameters_90012\",\"Remove type parameters\"),Import_0_from_module_1:n(90013,e.DiagnosticCategory.Message,\"Import_0_from_module_1_90013\",\"Import '{0}' from module \\\"{1}\\\"\"),Change_0_to_1:n(90014,e.DiagnosticCategory.Message,\"Change_0_to_1_90014\",\"Change '{0}' to '{1}'\"),Add_0_to_existing_import_declaration_from_1:n(90015,e.DiagnosticCategory.Message,\"Add_0_to_existing_import_declaration_from_1_90015\",\"Add '{0}' to existing import declaration from \\\"{1}\\\"\"),Declare_property_0:n(90016,e.DiagnosticCategory.Message,\"Declare_property_0_90016\",\"Declare property '{0}'\"),Add_index_signature_for_property_0:n(90017,e.DiagnosticCategory.Message,\"Add_index_signature_for_property_0_90017\",\"Add index signature for property '{0}'\"),Disable_checking_for_this_file:n(90018,e.DiagnosticCategory.Message,\"Disable_checking_for_this_file_90018\",\"Disable checking for this file\"),Ignore_this_error_message:n(90019,e.DiagnosticCategory.Message,\"Ignore_this_error_message_90019\",\"Ignore this error message\"),Initialize_property_0_in_the_constructor:n(90020,e.DiagnosticCategory.Message,\"Initialize_property_0_in_the_constructor_90020\",\"Initialize property '{0}' in the constructor\"),Initialize_static_property_0:n(90021,e.DiagnosticCategory.Message,\"Initialize_static_property_0_90021\",\"Initialize static property '{0}'\"),Change_spelling_to_0:n(90022,e.DiagnosticCategory.Message,\"Change_spelling_to_0_90022\",\"Change spelling to '{0}'\"),Declare_method_0:n(90023,e.DiagnosticCategory.Message,\"Declare_method_0_90023\",\"Declare method '{0}'\"),Declare_static_method_0:n(90024,e.DiagnosticCategory.Message,\"Declare_static_method_0_90024\",\"Declare static method '{0}'\"),Prefix_0_with_an_underscore:n(90025,e.DiagnosticCategory.Message,\"Prefix_0_with_an_underscore_90025\",\"Prefix '{0}' with an underscore\"),Rewrite_as_the_indexed_access_type_0:n(90026,e.DiagnosticCategory.Message,\"Rewrite_as_the_indexed_access_type_0_90026\",\"Rewrite as the indexed access type '{0}'\"),Declare_static_property_0:n(90027,e.DiagnosticCategory.Message,\"Declare_static_property_0_90027\",\"Declare static property '{0}'\"),Call_decorator_expression:n(90028,e.DiagnosticCategory.Message,\"Call_decorator_expression_90028\",\"Call decorator expression\"),Add_async_modifier_to_containing_function:n(90029,e.DiagnosticCategory.Message,\"Add_async_modifier_to_containing_function_90029\",\"Add async modifier to containing function\"),Replace_infer_0_with_unknown:n(90030,e.DiagnosticCategory.Message,\"Replace_infer_0_with_unknown_90030\",\"Replace 'infer {0}' with 'unknown'\"),Replace_all_unused_infer_with_unknown:n(90031,e.DiagnosticCategory.Message,\"Replace_all_unused_infer_with_unknown_90031\",\"Replace all unused 'infer' with 'unknown'\"),Import_default_0_from_module_1:n(90032,e.DiagnosticCategory.Message,\"Import_default_0_from_module_1_90032\",\"Import default '{0}' from module \\\"{1}\\\"\"),Add_default_import_0_to_existing_import_declaration_from_1:n(90033,e.DiagnosticCategory.Message,\"Add_default_import_0_to_existing_import_declaration_from_1_90033\",\"Add default import '{0}' to existing import declaration from \\\"{1}\\\"\"),Convert_function_to_an_ES2015_class:n(95001,e.DiagnosticCategory.Message,\"Convert_function_to_an_ES2015_class_95001\",\"Convert function to an ES2015 class\"),Convert_function_0_to_class:n(95002,e.DiagnosticCategory.Message,\"Convert_function_0_to_class_95002\",\"Convert function '{0}' to class\"),Extract_to_0_in_1:n(95004,e.DiagnosticCategory.Message,\"Extract_to_0_in_1_95004\",\"Extract to {0} in {1}\"),Extract_function:n(95005,e.DiagnosticCategory.Message,\"Extract_function_95005\",\"Extract function\"),Extract_constant:n(95006,e.DiagnosticCategory.Message,\"Extract_constant_95006\",\"Extract constant\"),Extract_to_0_in_enclosing_scope:n(95007,e.DiagnosticCategory.Message,\"Extract_to_0_in_enclosing_scope_95007\",\"Extract to {0} in enclosing scope\"),Extract_to_0_in_1_scope:n(95008,e.DiagnosticCategory.Message,\"Extract_to_0_in_1_scope_95008\",\"Extract to {0} in {1} scope\"),Annotate_with_type_from_JSDoc:n(95009,e.DiagnosticCategory.Message,\"Annotate_with_type_from_JSDoc_95009\",\"Annotate with type from JSDoc\"),Annotate_with_types_from_JSDoc:n(95010,e.DiagnosticCategory.Message,\"Annotate_with_types_from_JSDoc_95010\",\"Annotate with types from JSDoc\"),Infer_type_of_0_from_usage:n(95011,e.DiagnosticCategory.Message,\"Infer_type_of_0_from_usage_95011\",\"Infer type of '{0}' from usage\"),Infer_parameter_types_from_usage:n(95012,e.DiagnosticCategory.Message,\"Infer_parameter_types_from_usage_95012\",\"Infer parameter types from usage\"),Convert_to_default_import:n(95013,e.DiagnosticCategory.Message,\"Convert_to_default_import_95013\",\"Convert to default import\"),Install_0:n(95014,e.DiagnosticCategory.Message,\"Install_0_95014\",\"Install '{0}'\"),Replace_import_with_0:n(95015,e.DiagnosticCategory.Message,\"Replace_import_with_0_95015\",\"Replace import with '{0}'.\"),Use_synthetic_default_member:n(95016,e.DiagnosticCategory.Message,\"Use_synthetic_default_member_95016\",\"Use synthetic 'default' member.\"),Convert_to_ES6_module:n(95017,e.DiagnosticCategory.Message,\"Convert_to_ES6_module_95017\",\"Convert to ES6 module\"),Add_undefined_type_to_property_0:n(95018,e.DiagnosticCategory.Message,\"Add_undefined_type_to_property_0_95018\",\"Add 'undefined' type to property '{0}'\"),Add_initializer_to_property_0:n(95019,e.DiagnosticCategory.Message,\"Add_initializer_to_property_0_95019\",\"Add initializer to property '{0}'\"),Add_definite_assignment_assertion_to_property_0:n(95020,e.DiagnosticCategory.Message,\"Add_definite_assignment_assertion_to_property_0_95020\",\"Add definite assignment assertion to property '{0}'\"),Add_all_missing_members:n(95022,e.DiagnosticCategory.Message,\"Add_all_missing_members_95022\",\"Add all missing members\"),Infer_all_types_from_usage:n(95023,e.DiagnosticCategory.Message,\"Infer_all_types_from_usage_95023\",\"Infer all types from usage\"),Delete_all_unused_declarations:n(95024,e.DiagnosticCategory.Message,\"Delete_all_unused_declarations_95024\",\"Delete all unused declarations\"),Prefix_all_unused_declarations_with_where_possible:n(95025,e.DiagnosticCategory.Message,\"Prefix_all_unused_declarations_with_where_possible_95025\",\"Prefix all unused declarations with '_' where possible\"),Fix_all_detected_spelling_errors:n(95026,e.DiagnosticCategory.Message,\"Fix_all_detected_spelling_errors_95026\",\"Fix all detected spelling errors\"),Add_initializers_to_all_uninitialized_properties:n(95027,e.DiagnosticCategory.Message,\"Add_initializers_to_all_uninitialized_properties_95027\",\"Add initializers to all uninitialized properties\"),Add_definite_assignment_assertions_to_all_uninitialized_properties:n(95028,e.DiagnosticCategory.Message,\"Add_definite_assignment_assertions_to_all_uninitialized_properties_95028\",\"Add definite assignment assertions to all uninitialized properties\"),Add_undefined_type_to_all_uninitialized_properties:n(95029,e.DiagnosticCategory.Message,\"Add_undefined_type_to_all_uninitialized_properties_95029\",\"Add undefined type to all uninitialized properties\"),Change_all_jsdoc_style_types_to_TypeScript:n(95030,e.DiagnosticCategory.Message,\"Change_all_jsdoc_style_types_to_TypeScript_95030\",\"Change all jsdoc-style types to TypeScript\"),Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types:n(95031,e.DiagnosticCategory.Message,\"Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types_95031\",\"Change all jsdoc-style types to TypeScript (and add '| undefined' to nullable types)\"),Implement_all_unimplemented_interfaces:n(95032,e.DiagnosticCategory.Message,\"Implement_all_unimplemented_interfaces_95032\",\"Implement all unimplemented interfaces\"),Install_all_missing_types_packages:n(95033,e.DiagnosticCategory.Message,\"Install_all_missing_types_packages_95033\",\"Install all missing types packages\"),Rewrite_all_as_indexed_access_types:n(95034,e.DiagnosticCategory.Message,\"Rewrite_all_as_indexed_access_types_95034\",\"Rewrite all as indexed access types\"),Convert_all_to_default_imports:n(95035,e.DiagnosticCategory.Message,\"Convert_all_to_default_imports_95035\",\"Convert all to default imports\"),Make_all_super_calls_the_first_statement_in_their_constructor:n(95036,e.DiagnosticCategory.Message,\"Make_all_super_calls_the_first_statement_in_their_constructor_95036\",\"Make all 'super()' calls the first statement in their constructor\"),Add_qualifier_to_all_unresolved_variables_matching_a_member_name:n(95037,e.DiagnosticCategory.Message,\"Add_qualifier_to_all_unresolved_variables_matching_a_member_name_95037\",\"Add qualifier to all unresolved variables matching a member name\"),Change_all_extended_interfaces_to_implements:n(95038,e.DiagnosticCategory.Message,\"Change_all_extended_interfaces_to_implements_95038\",\"Change all extended interfaces to 'implements'\"),Add_all_missing_super_calls:n(95039,e.DiagnosticCategory.Message,\"Add_all_missing_super_calls_95039\",\"Add all missing super calls\"),Implement_all_inherited_abstract_classes:n(95040,e.DiagnosticCategory.Message,\"Implement_all_inherited_abstract_classes_95040\",\"Implement all inherited abstract classes\"),Add_all_missing_async_modifiers:n(95041,e.DiagnosticCategory.Message,\"Add_all_missing_async_modifiers_95041\",\"Add all missing 'async' modifiers\"),Add_ts_ignore_to_all_error_messages:n(95042,e.DiagnosticCategory.Message,\"Add_ts_ignore_to_all_error_messages_95042\",\"Add '@ts-ignore' to all error messages\"),Annotate_everything_with_types_from_JSDoc:n(95043,e.DiagnosticCategory.Message,\"Annotate_everything_with_types_from_JSDoc_95043\",\"Annotate everything with types from JSDoc\"),Add_to_all_uncalled_decorators:n(95044,e.DiagnosticCategory.Message,\"Add_to_all_uncalled_decorators_95044\",\"Add '()' to all uncalled decorators\"),Convert_all_constructor_functions_to_classes:n(95045,e.DiagnosticCategory.Message,\"Convert_all_constructor_functions_to_classes_95045\",\"Convert all constructor functions to classes\"),Generate_get_and_set_accessors:n(95046,e.DiagnosticCategory.Message,\"Generate_get_and_set_accessors_95046\",\"Generate 'get' and 'set' accessors\"),Convert_require_to_import:n(95047,e.DiagnosticCategory.Message,\"Convert_require_to_import_95047\",\"Convert 'require' to 'import'\"),Convert_all_require_to_import:n(95048,e.DiagnosticCategory.Message,\"Convert_all_require_to_import_95048\",\"Convert all 'require' to 'import'\"),Move_to_a_new_file:n(95049,e.DiagnosticCategory.Message,\"Move_to_a_new_file_95049\",\"Move to a new file\"),Remove_unreachable_code:n(95050,e.DiagnosticCategory.Message,\"Remove_unreachable_code_95050\",\"Remove unreachable code\"),Remove_all_unreachable_code:n(95051,e.DiagnosticCategory.Message,\"Remove_all_unreachable_code_95051\",\"Remove all unreachable code\"),Add_missing_typeof:n(95052,e.DiagnosticCategory.Message,\"Add_missing_typeof_95052\",\"Add missing 'typeof'\"),Remove_unused_label:n(95053,e.DiagnosticCategory.Message,\"Remove_unused_label_95053\",\"Remove unused label\"),Remove_all_unused_labels:n(95054,e.DiagnosticCategory.Message,\"Remove_all_unused_labels_95054\",\"Remove all unused labels\"),Convert_0_to_mapped_object_type:n(95055,e.DiagnosticCategory.Message,\"Convert_0_to_mapped_object_type_95055\",\"Convert '{0}' to mapped object type\"),Convert_namespace_import_to_named_imports:n(95056,e.DiagnosticCategory.Message,\"Convert_namespace_import_to_named_imports_95056\",\"Convert namespace import to named imports\"),Convert_named_imports_to_namespace_import:n(95057,e.DiagnosticCategory.Message,\"Convert_named_imports_to_namespace_import_95057\",\"Convert named imports to namespace import\"),Add_or_remove_braces_in_an_arrow_function:n(95058,e.DiagnosticCategory.Message,\"Add_or_remove_braces_in_an_arrow_function_95058\",\"Add or remove braces in an arrow function\"),Add_braces_to_arrow_function:n(95059,e.DiagnosticCategory.Message,\"Add_braces_to_arrow_function_95059\",\"Add braces to arrow function\"),Remove_braces_from_arrow_function:n(95060,e.DiagnosticCategory.Message,\"Remove_braces_from_arrow_function_95060\",\"Remove braces from arrow function\"),Convert_default_export_to_named_export:n(95061,e.DiagnosticCategory.Message,\"Convert_default_export_to_named_export_95061\",\"Convert default export to named export\"),Convert_named_export_to_default_export:n(95062,e.DiagnosticCategory.Message,\"Convert_named_export_to_default_export_95062\",\"Convert named export to default export\"),Add_missing_enum_member_0:n(95063,e.DiagnosticCategory.Message,\"Add_missing_enum_member_0_95063\",\"Add missing enum member '{0}'\"),Add_all_missing_imports:n(95064,e.DiagnosticCategory.Message,\"Add_all_missing_imports_95064\",\"Add all missing imports\"),Convert_to_async_function:n(95065,e.DiagnosticCategory.Message,\"Convert_to_async_function_95065\",\"Convert to async function\"),Convert_all_to_async_functions:n(95066,e.DiagnosticCategory.Message,\"Convert_all_to_async_functions_95066\",\"Convert all to async functions\"),Generate_types_for_0:n(95067,e.DiagnosticCategory.Message,\"Generate_types_for_0_95067\",\"Generate types for '{0}'\"),Generate_types_for_all_packages_without_types:n(95068,e.DiagnosticCategory.Message,\"Generate_types_for_all_packages_without_types_95068\",\"Generate types for all packages without types\")}}(Tn||(Tn={})),function(e){var n;function t(e){return e>=71}e.tokenIsIdentifierOrKeyword=t,e.tokenIsIdentifierOrKeywordOrGreaterThan=function(e){return 29===e||t(e)};var r=((n={abstract:117,any:119,as:118,boolean:122,break:72,case:73,catch:74,class:75,continue:77,const:76}).constructor=123,n.debugger=78,n.declare=124,n.default=79,n.delete=80,n.do=81,n.else=82,n.enum=83,n.export=84,n.extends=85,n.false=86,n.finally=87,n.for=88,n.from=143,n.function=89,n.get=125,n.if=90,n.implements=108,n.import=91,n.in=92,n.infer=126,n.instanceof=93,n.interface=109,n.is=127,n.keyof=128,n.let=110,n.module=129,n.namespace=130,n.never=131,n.new=94,n.null=95,n.number=134,n.object=135,n.package=111,n.private=112,n.protected=113,n.public=114,n.readonly=132,n.require=133,n.global=144,n.return=96,n.set=136,n.static=115,n.string=137,n.super=97,n.switch=98,n.symbol=138,n.this=99,n.throw=100,n.true=101,n.try=102,n.type=139,n.typeof=103,n.undefined=140,n.unique=141,n.unknown=142,n.var=104,n.void=105,n.while=106,n.with=107,n.yield=116,n.async=120,n.await=121,n.of=145,n),a=e.createMapFromTemplate(r),i=e.createMapFromTemplate(yn({},r,{\"{\":17,\"}\":18,\"(\":19,\")\":20,\"[\":21,\"]\":22,\".\":23,\"...\":24,\";\":25,\",\":26,\"<\":27,\">\":29,\"<=\":30,\">=\":31,\"==\":32,\"!=\":33,\"===\":34,\"!==\":35,\"=>\":36,\"+\":37,\"-\":38,\"**\":40,\"*\":39,\"/\":41,\"%\":42,\"++\":43,\"--\":44,\"<<\":45,\"</\":28,\">>\":46,\">>>\":47,\"&\":48,\"|\":49,\"^\":50,\"!\":51,\"~\":52,\"&&\":53,\"||\":54,\"?\":55,\":\":56,\"=\":58,\"+=\":59,\"-=\":60,\"*=\":61,\"**=\":62,\"/=\":63,\"%=\":64,\"<<=\":65,\">>=\":66,\">>>=\":67,\"&=\":68,\"|=\":69,\"^=\":70,\"@\":57})),o=[170,170,181,181,186,186,192,214,216,246,248,543,546,563,592,685,688,696,699,705,720,721,736,740,750,750,890,890,902,902,904,906,908,908,910,929,931,974,976,983,986,1011,1024,1153,1164,1220,1223,1224,1227,1228,1232,1269,1272,1273,1329,1366,1369,1369,1377,1415,1488,1514,1520,1522,1569,1594,1600,1610,1649,1747,1749,1749,1765,1766,1786,1788,1808,1808,1810,1836,1920,1957,2309,2361,2365,2365,2384,2384,2392,2401,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2524,2525,2527,2529,2544,2545,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2649,2652,2654,2654,2674,2676,2693,2699,2701,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2749,2749,2768,2768,2784,2784,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2870,2873,2877,2877,2908,2909,2911,2913,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,2997,2999,3001,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3168,3169,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3294,3294,3296,3297,3333,3340,3342,3344,3346,3368,3370,3385,3424,3425,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3585,3632,3634,3635,3648,3654,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3760,3762,3763,3773,3773,3776,3780,3782,3782,3804,3805,3840,3840,3904,3911,3913,3946,3976,3979,4096,4129,4131,4135,4137,4138,4176,4181,4256,4293,4304,4342,4352,4441,4447,4514,4520,4601,4608,4614,4616,4678,4680,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4742,4744,4744,4746,4749,4752,4782,4784,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4814,4816,4822,4824,4846,4848,4878,4880,4880,4882,4885,4888,4894,4896,4934,4936,4954,5024,5108,5121,5740,5743,5750,5761,5786,5792,5866,6016,6067,6176,6263,6272,6312,7680,7835,7840,7929,7936,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8319,8319,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8497,8499,8505,8544,8579,12293,12295,12321,12329,12337,12341,12344,12346,12353,12436,12445,12446,12449,12538,12540,12542,12549,12588,12593,12686,12704,12727,13312,19893,19968,40869,40960,42124,44032,55203,63744,64045,64256,64262,64275,64279,64285,64285,64287,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65136,65138,65140,65140,65142,65276,65313,65338,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500],s=[170,170,181,181,186,186,192,214,216,246,248,543,546,563,592,685,688,696,699,705,720,721,736,740,750,750,768,846,864,866,890,890,902,902,904,906,908,908,910,929,931,974,976,983,986,1011,1024,1153,1155,1158,1164,1220,1223,1224,1227,1228,1232,1269,1272,1273,1329,1366,1369,1369,1377,1415,1425,1441,1443,1465,1467,1469,1471,1471,1473,1474,1476,1476,1488,1514,1520,1522,1569,1594,1600,1621,1632,1641,1648,1747,1749,1756,1759,1768,1770,1773,1776,1788,1808,1836,1840,1866,1920,1968,2305,2307,2309,2361,2364,2381,2384,2388,2392,2403,2406,2415,2433,2435,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2492,2492,2494,2500,2503,2504,2507,2509,2519,2519,2524,2525,2527,2531,2534,2545,2562,2562,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2620,2620,2622,2626,2631,2632,2635,2637,2649,2652,2654,2654,2662,2676,2689,2691,2693,2699,2701,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2748,2757,2759,2761,2763,2765,2768,2768,2784,2784,2790,2799,2817,2819,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2870,2873,2876,2883,2887,2888,2891,2893,2902,2903,2908,2909,2911,2913,2918,2927,2946,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,2997,2999,3001,3006,3010,3014,3016,3018,3021,3031,3031,3047,3055,3073,3075,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3134,3140,3142,3144,3146,3149,3157,3158,3168,3169,3174,3183,3202,3203,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3262,3268,3270,3272,3274,3277,3285,3286,3294,3294,3296,3297,3302,3311,3330,3331,3333,3340,3342,3344,3346,3368,3370,3385,3390,3395,3398,3400,3402,3405,3415,3415,3424,3425,3430,3439,3458,3459,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3530,3530,3535,3540,3542,3542,3544,3551,3570,3571,3585,3642,3648,3662,3664,3673,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3769,3771,3773,3776,3780,3782,3782,3784,3789,3792,3801,3804,3805,3840,3840,3864,3865,3872,3881,3893,3893,3895,3895,3897,3897,3902,3911,3913,3946,3953,3972,3974,3979,3984,3991,3993,4028,4038,4038,4096,4129,4131,4135,4137,4138,4140,4146,4150,4153,4160,4169,4176,4185,4256,4293,4304,4342,4352,4441,4447,4514,4520,4601,4608,4614,4616,4678,4680,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4742,4744,4744,4746,4749,4752,4782,4784,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4814,4816,4822,4824,4846,4848,4878,4880,4880,4882,4885,4888,4894,4896,4934,4936,4954,4969,4977,5024,5108,5121,5740,5743,5750,5761,5786,5792,5866,6016,6099,6112,6121,6160,6169,6176,6263,6272,6313,7680,7835,7840,7929,7936,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8255,8256,8319,8319,8400,8412,8417,8417,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8497,8499,8505,8544,8579,12293,12295,12321,12335,12337,12341,12344,12346,12353,12436,12441,12442,12445,12446,12449,12542,12549,12588,12593,12686,12704,12727,13312,19893,19968,40869,40960,42124,44032,55203,63744,64045,64256,64262,64275,64279,64285,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65056,65059,65075,65076,65101,65103,65136,65138,65140,65140,65142,65276,65296,65305,65313,65338,65343,65343,65345,65370,65381,65470,65474,65479,65482,65487,65490,65495,65498,65500],l=[170,170,181,181,186,186,192,214,216,246,248,705,710,721,736,740,748,748,750,750,880,884,886,887,890,893,902,902,904,906,908,908,910,929,931,1013,1015,1153,1162,1319,1329,1366,1369,1369,1377,1415,1488,1514,1520,1522,1568,1610,1646,1647,1649,1747,1749,1749,1765,1766,1774,1775,1786,1788,1791,1791,1808,1808,1810,1839,1869,1957,1969,1969,1994,2026,2036,2037,2042,2042,2048,2069,2074,2074,2084,2084,2088,2088,2112,2136,2208,2208,2210,2220,2308,2361,2365,2365,2384,2384,2392,2401,2417,2423,2425,2431,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2493,2493,2510,2510,2524,2525,2527,2529,2544,2545,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2649,2652,2654,2654,2674,2676,2693,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2749,2749,2768,2768,2784,2785,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2869,2873,2877,2877,2908,2909,2911,2913,2929,2929,2947,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,3001,3024,3024,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3133,3133,3160,3161,3168,3169,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3261,3261,3294,3294,3296,3297,3313,3314,3333,3340,3342,3344,3346,3386,3389,3389,3406,3406,3424,3425,3450,3455,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3585,3632,3634,3635,3648,3654,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3760,3762,3763,3773,3773,3776,3780,3782,3782,3804,3807,3840,3840,3904,3911,3913,3948,3976,3980,4096,4138,4159,4159,4176,4181,4186,4189,4193,4193,4197,4198,4206,4208,4213,4225,4238,4238,4256,4293,4295,4295,4301,4301,4304,4346,4348,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4744,4746,4749,4752,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4822,4824,4880,4882,4885,4888,4954,4992,5007,5024,5108,5121,5740,5743,5759,5761,5786,5792,5866,5870,5872,5888,5900,5902,5905,5920,5937,5952,5969,5984,5996,5998,6e3,6016,6067,6103,6103,6108,6108,6176,6263,6272,6312,6314,6314,6320,6389,6400,6428,6480,6509,6512,6516,6528,6571,6593,6599,6656,6678,6688,6740,6823,6823,6917,6963,6981,6987,7043,7072,7086,7087,7098,7141,7168,7203,7245,7247,7258,7293,7401,7404,7406,7409,7413,7414,7424,7615,7680,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8305,8305,8319,8319,8336,8348,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8505,8508,8511,8517,8521,8526,8526,8544,8584,11264,11310,11312,11358,11360,11492,11499,11502,11506,11507,11520,11557,11559,11559,11565,11565,11568,11623,11631,11631,11648,11670,11680,11686,11688,11694,11696,11702,11704,11710,11712,11718,11720,11726,11728,11734,11736,11742,11823,11823,12293,12295,12321,12329,12337,12341,12344,12348,12353,12438,12445,12447,12449,12538,12540,12543,12549,12589,12593,12686,12704,12730,12784,12799,13312,19893,19968,40908,40960,42124,42192,42237,42240,42508,42512,42527,42538,42539,42560,42606,42623,42647,42656,42735,42775,42783,42786,42888,42891,42894,42896,42899,42912,42922,43e3,43009,43011,43013,43015,43018,43020,43042,43072,43123,43138,43187,43250,43255,43259,43259,43274,43301,43312,43334,43360,43388,43396,43442,43471,43471,43520,43560,43584,43586,43588,43595,43616,43638,43642,43642,43648,43695,43697,43697,43701,43702,43705,43709,43712,43712,43714,43714,43739,43741,43744,43754,43762,43764,43777,43782,43785,43790,43793,43798,43808,43814,43816,43822,43968,44002,44032,55203,55216,55238,55243,55291,63744,64109,64112,64217,64256,64262,64275,64279,64285,64285,64287,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65136,65140,65142,65276,65313,65338,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500],c=[170,170,181,181,186,186,192,214,216,246,248,705,710,721,736,740,748,748,750,750,768,884,886,887,890,893,902,902,904,906,908,908,910,929,931,1013,1015,1153,1155,1159,1162,1319,1329,1366,1369,1369,1377,1415,1425,1469,1471,1471,1473,1474,1476,1477,1479,1479,1488,1514,1520,1522,1552,1562,1568,1641,1646,1747,1749,1756,1759,1768,1770,1788,1791,1791,1808,1866,1869,1969,1984,2037,2042,2042,2048,2093,2112,2139,2208,2208,2210,2220,2276,2302,2304,2403,2406,2415,2417,2423,2425,2431,2433,2435,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2492,2500,2503,2504,2507,2510,2519,2519,2524,2525,2527,2531,2534,2545,2561,2563,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2620,2620,2622,2626,2631,2632,2635,2637,2641,2641,2649,2652,2654,2654,2662,2677,2689,2691,2693,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2748,2757,2759,2761,2763,2765,2768,2768,2784,2787,2790,2799,2817,2819,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2869,2873,2876,2884,2887,2888,2891,2893,2902,2903,2908,2909,2911,2915,2918,2927,2929,2929,2946,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,3001,3006,3010,3014,3016,3018,3021,3024,3024,3031,3031,3046,3055,3073,3075,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3133,3140,3142,3144,3146,3149,3157,3158,3160,3161,3168,3171,3174,3183,3202,3203,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3260,3268,3270,3272,3274,3277,3285,3286,3294,3294,3296,3299,3302,3311,3313,3314,3330,3331,3333,3340,3342,3344,3346,3386,3389,3396,3398,3400,3402,3406,3415,3415,3424,3427,3430,3439,3450,3455,3458,3459,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3530,3530,3535,3540,3542,3542,3544,3551,3570,3571,3585,3642,3648,3662,3664,3673,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3769,3771,3773,3776,3780,3782,3782,3784,3789,3792,3801,3804,3807,3840,3840,3864,3865,3872,3881,3893,3893,3895,3895,3897,3897,3902,3911,3913,3948,3953,3972,3974,3991,3993,4028,4038,4038,4096,4169,4176,4253,4256,4293,4295,4295,4301,4301,4304,4346,4348,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4744,4746,4749,4752,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4822,4824,4880,4882,4885,4888,4954,4957,4959,4992,5007,5024,5108,5121,5740,5743,5759,5761,5786,5792,5866,5870,5872,5888,5900,5902,5908,5920,5940,5952,5971,5984,5996,5998,6e3,6002,6003,6016,6099,6103,6103,6108,6109,6112,6121,6155,6157,6160,6169,6176,6263,6272,6314,6320,6389,6400,6428,6432,6443,6448,6459,6470,6509,6512,6516,6528,6571,6576,6601,6608,6617,6656,6683,6688,6750,6752,6780,6783,6793,6800,6809,6823,6823,6912,6987,6992,7001,7019,7027,7040,7155,7168,7223,7232,7241,7245,7293,7376,7378,7380,7414,7424,7654,7676,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8204,8205,8255,8256,8276,8276,8305,8305,8319,8319,8336,8348,8400,8412,8417,8417,8421,8432,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8505,8508,8511,8517,8521,8526,8526,8544,8584,11264,11310,11312,11358,11360,11492,11499,11507,11520,11557,11559,11559,11565,11565,11568,11623,11631,11631,11647,11670,11680,11686,11688,11694,11696,11702,11704,11710,11712,11718,11720,11726,11728,11734,11736,11742,11744,11775,11823,11823,12293,12295,12321,12335,12337,12341,12344,12348,12353,12438,12441,12442,12445,12447,12449,12538,12540,12543,12549,12589,12593,12686,12704,12730,12784,12799,13312,19893,19968,40908,40960,42124,42192,42237,42240,42508,42512,42539,42560,42607,42612,42621,42623,42647,42655,42737,42775,42783,42786,42888,42891,42894,42896,42899,42912,42922,43e3,43047,43072,43123,43136,43204,43216,43225,43232,43255,43259,43259,43264,43309,43312,43347,43360,43388,43392,43456,43471,43481,43520,43574,43584,43597,43600,43609,43616,43638,43642,43643,43648,43714,43739,43741,43744,43759,43762,43766,43777,43782,43785,43790,43793,43798,43808,43814,43816,43822,43968,44010,44012,44013,44016,44025,44032,55203,55216,55238,55243,55291,63744,64109,64112,64217,64256,64262,64275,64279,64285,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65024,65039,65056,65062,65075,65076,65101,65103,65136,65140,65142,65276,65296,65305,65313,65338,65343,65343,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500];function u(e,n){if(e<n[0])return!1;for(var t,r=0,a=n.length;r+1<a;){if(t=r+(a-r)/2,n[t-=t%2]<=e&&e<=n[t+1])return!0;e<n[t]?a=t:r=t+2}return!1}function d(e,n){return u(e,n>=1?l:o)}e.isUnicodeIdentifierStart=d;var m,p=(m=[],i.forEach(function(e,n){m[e]=n}),m);function f(e){for(var n=new Array,t=0,r=0;t<e.length;){var a=e.charCodeAt(t);switch(t++,a){case 13:10===e.charCodeAt(t)&&t++;case 10:n.push(r),r=t;break;default:a>127&&b(a)&&(n.push(r),r=t)}}return n.push(r),n}function g(n,t,r,a){(t<0||t>=n.length)&&e.Debug.fail(\"Bad line number. Line: \"+t+\", lineStarts.length: \"+n.length+\" , line map is correct? \"+(void 0!==a?e.arraysEqual(n,f(a)):\"unknown\"));var i=n[t]+r;return t<n.length-1?e.Debug.assert(i<n[t+1]):void 0!==a&&e.Debug.assert(i<=a.length),i}function _(e){return e.lineMap||(e.lineMap=f(e.text))}function v(n,t){var r=e.binarySearch(n,t,e.identity,e.compareValues);return r<0&&(r=~r-1,e.Debug.assert(-1!==r,\"position cannot precede the beginning of the file\")),{line:r,character:t-n[r]}}function y(e){return h(e)||b(e)}function h(e){return 32===e||9===e||11===e||12===e||160===e||133===e||5760===e||e>=8192&&e<=8203||8239===e||8287===e||12288===e||65279===e}function b(e){return 10===e||13===e||8232===e||8233===e}function E(e){return e>=48&&e<=57}function T(e){return e>=48&&e<=55}e.tokenToString=function(e){return p[e]},e.stringToToken=function(e){return i.get(e)},e.computeLineStarts=f,e.getPositionOfLineAndCharacter=function(e,n,t){return g(_(e),n,t,e.text)},e.computePositionOfLineAndCharacter=g,e.getLineStarts=_,e.computeLineAndCharacterOfPosition=v,e.getLineAndCharacterOfPosition=function(e,n){return v(_(e),n)},e.isWhiteSpaceLike=y,e.isWhiteSpaceSingleLine=h,e.isLineBreak=b,e.isOctalDigit=T,e.couldStartTrivia=function(e,n){var t=e.charCodeAt(n);switch(t){case 13:case 10:case 9:case 11:case 12:case 32:case 47:case 60:case 124:case 61:case 62:return!0;case 35:return 0===n;default:return t>127}},e.skipTrivia=function(n,t,r,a){if(void 0===a&&(a=!1),e.positionIsSynthesized(t))return t;for(;;){var i=n.charCodeAt(t);switch(i){case 13:10===n.charCodeAt(t+1)&&t++;case 10:if(t++,r)return t;continue;case 9:case 11:case 12:case 32:t++;continue;case 47:if(a)break;if(47===n.charCodeAt(t+1)){for(t+=2;t<n.length&&!b(n.charCodeAt(t));)t++;continue}if(42===n.charCodeAt(t+1)){for(t+=2;t<n.length;){if(42===n.charCodeAt(t)&&47===n.charCodeAt(t+1)){t+=2;break}t++}continue}break;case 60:case 124:case 61:case 62:if(L(n,t)){t=x(n,t);continue}break;case 35:if(0===t&&C(n,t)){t=D(n,t);continue}break;default:if(i>127&&y(i)){t++;continue}}return t}};var S=\"<<<<<<<\".length;function L(n,t){if(e.Debug.assert(t>=0),0===t||b(n.charCodeAt(t-1))){var r=n.charCodeAt(t);if(t+S<n.length){for(var a=0;a<S;a++)if(n.charCodeAt(t+a)!==r)return!1;return 61===r||32===n.charCodeAt(t+S)}}return!1}function x(n,t,r){r&&r(e.Diagnostics.Merge_conflict_marker_encountered,t,S);var a=n.charCodeAt(t),i=n.length;if(60===a||62===a)for(;t<i&&!b(n.charCodeAt(t));)t++;else for(e.Debug.assert(124===a||61===a);t<i;){var o=n.charCodeAt(t);if((61===o||62===o)&&o!==a&&L(n,t))break;t++}return t}var A=/^#!.*/;function C(n,t){return e.Debug.assert(0===t),A.test(n)}function D(e,n){return n+=A.exec(e)[0].length}function k(e,n,t,r,a,i,o){var s,l,c,u,d=!1,m=r||0===t,p=o;e:for(;t>=0&&t<n.length;){var f=n.charCodeAt(t);switch(f){case 13:10===n.charCodeAt(t+1)&&t++;case 10:if(t++,r)break e;m=!0,d&&(u=!0);continue;case 9:case 11:case 12:case 32:t++;continue;case 47:var g=n.charCodeAt(t+1),_=!1;if(47===g||42===g){var v=47===g?2:3,h=t;if(t+=2,47===g)for(;t<n.length;){if(b(n.charCodeAt(t))){_=!0;break}t++}else for(;t<n.length;){if(42===n.charCodeAt(t)&&47===n.charCodeAt(t+1)){t+=2;break}t++}if(m){if(d&&(p=a(s,l,c,u,i,p),!e&&p))return p;s=h,l=t,c=v,u=_,d=!0}continue}break e;default:if(f>127&&y(f)){d&&b(f)&&(u=!0),t++;continue}break e}}return d&&(p=a(s,l,c,u,i,p)),p}function M(e,n,t,r,a){return k(!0,e,n,!1,t,r,a)}function O(e,n,t,r,a){return k(!0,e,n,!0,t,r,a)}function I(e,n,t,r,a,i){return i||(i=[]),i.push({kind:t,pos:e,end:n,hasTrailingNewLine:r}),i}function R(e,n){return e>=65&&e<=90||e>=97&&e<=122||36===e||95===e||e>127&&d(e,n)}function N(e,n){return e>=65&&e<=90||e>=97&&e<=122||e>=48&&e<=57||36===e||95===e||e>127&&function(e,n){return u(e,n>=1?c:s)}(e,n)}e.forEachLeadingCommentRange=function(e,n,t,r){return k(!1,e,n,!1,t,r)},e.forEachTrailingCommentRange=function(e,n,t,r){return k(!1,e,n,!0,t,r)},e.reduceEachLeadingCommentRange=M,e.reduceEachTrailingCommentRange=O,e.getLeadingCommentRanges=function(e,n){return M(e,n,I,void 0,void 0)},e.getTrailingCommentRanges=function(e,n){return O(e,n,I,void 0,void 0)},e.getShebang=function(e){var n=A.exec(e);if(n)return n[0]},e.isIdentifierStart=R,e.isIdentifierPart=N,e.isIdentifierText=function(e,n){if(!R(e.charCodeAt(0),n))return!1;for(var t=1;t<e.length;t++)if(!N(e.charCodeAt(t),n))return!1;return!0},e.createScanner=function(n,r,i,o,s,l,c){void 0===i&&(i=0);var u,d,m,p,f,g,_,v=o,S=0;return J(v,l,c),{getStartPos:function(){return m},getTextPos:function(){return u},getToken:function(){return f},getTokenPos:function(){return p},getTokenText:function(){return v.substring(p,u)},getTokenValue:function(){return g},hasExtendedUnicodeEscape:function(){return 0!=(8&_)},hasPrecedingLineBreak:function(){return 0!=(1&_)},isIdentifier:function(){return 71===f||f>107},isReservedWord:function(){return f>=72&&f<=107},isUnterminated:function(){return 0!=(4&_)},getTokenFlags:function(){return _},reScanGreaterToken:function(){if(29===f){if(62===v.charCodeAt(u))return 62===v.charCodeAt(u+1)?61===v.charCodeAt(u+2)?(u+=3,f=67):(u+=2,f=47):61===v.charCodeAt(u+1)?(u+=2,f=66):(u++,f=46);if(61===v.charCodeAt(u))return u++,f=31}return f},reScanSlashToken:function(){if(41===f||63===f){for(var t=p+1,r=!1,a=!1;;){if(t>=d){_|=4,A(e.Diagnostics.Unterminated_regular_expression_literal);break}var i=v.charCodeAt(t);if(b(i)){_|=4,A(e.Diagnostics.Unterminated_regular_expression_literal);break}if(r)r=!1;else{if(47===i&&!a){t++;break}91===i?a=!0:92===i?r=!0:93===i&&(a=!1)}t++}for(;t<d&&N(v.charCodeAt(t),n);)t++;u=t,g=v.substring(p,u),f=12}return f},reScanTemplateToken:function(){return e.Debug.assert(18===f,\"'reScanTemplateToken' should only be called on a '}'\"),u=p,f=G()},scanJsxIdentifier:function(){if(t(f)){for(var e=u;u<d;){var r=v.charCodeAt(u);if(45!==r&&(e===u?!R(r,n):!N(r,n)))break;u++}g+=v.substring(e,u)}return f},scanJsxAttributeValue:function(){switch(m=u,v.charCodeAt(u)){case 34:case 39:return g=F(!0),f=9;default:return W()}},reScanJsxToken:function(){return u=p=m,f=q()},scanJsxToken:q,scanJSDocToken:function(){if(m=p=u,_=0,u>=d)return f=1;var e=v.charCodeAt(u);switch(u++,e){case 9:case 11:case 12:case 32:for(;u<d&&h(v.charCodeAt(u));)u++;return f=5;case 64:return f=57;case 10:case 13:return _|=1,f=4;case 42:return f=39;case 123:return f=17;case 125:return f=18;case 91:return f=21;case 93:return f=22;case 60:return f=27;case 61:return f=58;case 44:return f=26;case 46:return f=23;case 96:for(;u<d&&96!==v.charCodeAt(u);)u++;return g=v.substring(p+1,u),u++,f=13}if(R(e,6)){for(;N(v.charCodeAt(u),6)&&u<d;)u++;return g=v.substring(p,u),f=U()}return f=0},scan:W,getText:function(){return v},setText:J,setScriptTarget:function(e){n=e},setLanguageVariant:function(e){i=e},setOnError:function(e){s=e},setTextPos:X,setInJSDocType:function(e){S+=e?1:-1},tryScan:function(e){return z(e,!1)},lookAhead:function(e){return z(e,!0)},scanRange:function(e,n,t){var r=d,a=u,i=m,o=p,s=f,l=g,c=_;J(v,e,n);var y=t();return d=r,u=a,m=i,p=o,f=s,g=l,_=c,y}};function A(e,n,t){if(void 0===n&&(n=u),s){var r=u;u=n,s(e,t||0),u=r}}function k(){for(var n=u,t=!1,r=!1,a=\"\";;){var i=v.charCodeAt(u);if(95!==i){if(!E(i))break;t=!0,r=!1,u++}else _|=512,t?(t=!1,r=!0,a+=v.substring(n,u)):A(r?e.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted:e.Diagnostics.Numeric_separators_are_not_allowed_here,u,1),n=++u}return 95===v.charCodeAt(u-1)&&A(e.Diagnostics.Numeric_separators_are_not_allowed_here,u-1,1),a+v.substring(n,u)}function M(){var n,t,r=u,a=k();46===v.charCodeAt(u)&&(u++,n=k());var i=u;if(69===v.charCodeAt(u)||101===v.charCodeAt(u)){u++,_|=16,43!==v.charCodeAt(u)&&45!==v.charCodeAt(u)||u++;var o=u,s=k();s?(t=v.substring(i,o)+s,i=u):A(e.Diagnostics.Digit_expected)}if(512&_){var l=a;return n&&(l+=\".\"+n),t&&(l+=t),\"\"+ +l}return\"\"+ +v.substring(r,i)}function O(){for(var e=u;T(v.charCodeAt(u));)u++;return+v.substring(e,u)}function I(e,n){return P(e,!1,n)}function w(e,n){return P(e,!0,n)}function P(n,t,r){for(var a=0,i=0,o=!1,s=!1;a<n||t;){var l=v.charCodeAt(u);if(r&&95===l)_|=512,o?(o=!1,s=!0):A(s?e.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted:e.Diagnostics.Numeric_separators_are_not_allowed_here,u,1),u++;else{if(o=r,l>=48&&l<=57)i=16*i+l-48;else if(l>=65&&l<=70)i=16*i+l-65+10;else{if(!(l>=97&&l<=102))break;i=16*i+l-97+10}u++,a++,s=!1}}return a<n&&(i=-1),95===v.charCodeAt(u-1)&&A(e.Diagnostics.Numeric_separators_are_not_allowed_here,u-1,1),i}function F(n){void 0===n&&(n=!1);for(var t=v.charCodeAt(u),r=\"\",a=++u;;){if(u>=d){r+=v.substring(a,u),_|=4,A(e.Diagnostics.Unterminated_string_literal);break}var i=v.charCodeAt(u);if(i===t){r+=v.substring(a,u),u++;break}if(92!==i||n){if(b(i)&&!n){r+=v.substring(a,u),_|=4,A(e.Diagnostics.Unterminated_string_literal);break}u++}else r+=v.substring(a,u),r+=V(),a=u}return r}function G(){for(var n,t=96===v.charCodeAt(u),r=++u,a=\"\";;){if(u>=d){a+=v.substring(r,u),_|=4,A(e.Diagnostics.Unterminated_template_literal),n=t?13:16;break}var i=v.charCodeAt(u);if(96===i){a+=v.substring(r,u),u++,n=t?13:16;break}if(36===i&&u+1<d&&123===v.charCodeAt(u+1)){a+=v.substring(r,u),u+=2,n=t?14:15;break}92!==i?13!==i?u++:(a+=v.substring(r,u),++u<d&&10===v.charCodeAt(u)&&u++,a+=\"\\n\",r=u):(a+=v.substring(r,u),a+=V(),r=u)}return e.Debug.assert(void 0!==n),g=a,n}function V(){if(++u>=d)return A(e.Diagnostics.Unexpected_end_of_text),\"\";var n,t,r=v.charCodeAt(u);switch(u++,r){case 48:return\"\\0\";case 98:return\"\\b\";case 116:return\"\\t\";case 110:return\"\\n\";case 118:return\"\\v\";case 102:return\"\\f\";case 114:return\"\\r\";case 39:return\"'\";case 34:return'\"';case 117:return u<d&&123===v.charCodeAt(u)?(_|=8,u++,n=w(1,!1),t=!1,n<0?(A(e.Diagnostics.Hexadecimal_digit_expected),t=!0):n>1114111&&(A(e.Diagnostics.An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive),t=!0),u>=d?(A(e.Diagnostics.Unexpected_end_of_text),t=!0):125===v.charCodeAt(u)?u++:(A(e.Diagnostics.Unterminated_Unicode_escape_sequence),t=!0),t?\"\":function(n){if(e.Debug.assert(0<=n&&n<=1114111),n<=65535)return String.fromCharCode(n);var t=Math.floor((n-65536)/1024)+55296,r=(n-65536)%1024+56320;return String.fromCharCode(t,r)}(n)):K(4);case 120:return K(2);case 13:u<d&&10===v.charCodeAt(u)&&u++;case 10:case 8232:case 8233:return\"\";default:return String.fromCharCode(r)}}function K(n){var t=I(n,!1);return t>=0?String.fromCharCode(t):(A(e.Diagnostics.Hexadecimal_digit_expected),\"\")}function B(){if(u+5<d&&117===v.charCodeAt(u+1)){var e=u;u+=2;var n=I(4,!1);return u=e,n}return-1}function H(){for(var e=\"\",t=u;u<d;){var r=v.charCodeAt(u);if(N(r,n))u++;else{if(92!==r)break;if(!((r=B())>=0&&N(r,n)))break;e+=v.substring(t,u),e+=String.fromCharCode(r),t=u+=6}}return e+=v.substring(t,u)}function U(){var e=g.length;if(e>=2&&e<=11){var n=g.charCodeAt(0);if(n>=97&&n<=122){var t=a.get(g);if(void 0!==t)return f=t}}return f=71}function j(n){e.Debug.assert(2===n||8===n,\"Expected either base 2 or base 8\");for(var t=0,r=0,a=!1,i=!1;;){var o=v.charCodeAt(u);if(95!==o){a=!0;var s=o-48;if(!E(o)||s>=n)break;t=t*n+s,u++,r++,i=!1}else _|=512,a?(a=!1,i=!0):A(i?e.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted:e.Diagnostics.Numeric_separators_are_not_allowed_here,u,1),u++}return 0===r?-1:95===v.charCodeAt(u-1)?(A(e.Diagnostics.Numeric_separators_are_not_allowed_here,u-1,1),t):t}function W(){m=u,_=0;for(var t=!1;;){if(p=u,u>=d)return f=1;var a=v.charCodeAt(u);if(35===a&&0===u&&C(v,u)){if(u=D(v,u),r)continue;return f=6}switch(a){case 10:case 13:if(_|=1,r){u++;continue}return 13===a&&u+1<d&&10===v.charCodeAt(u+1)?u+=2:u++,f=4;case 9:case 11:case 12:case 32:case 160:case 5760:case 8192:case 8193:case 8194:case 8195:case 8196:case 8197:case 8198:case 8199:case 8200:case 8201:case 8202:case 8203:case 8239:case 8287:case 12288:case 65279:if(r){u++;continue}for(;u<d&&h(v.charCodeAt(u));)u++;return f=5;case 33:return 61===v.charCodeAt(u+1)?61===v.charCodeAt(u+2)?(u+=3,f=35):(u+=2,f=33):(u++,f=51);case 34:case 39:return g=F(),f=9;case 96:return f=G();case 37:return 61===v.charCodeAt(u+1)?(u+=2,f=64):(u++,f=42);case 38:return 38===v.charCodeAt(u+1)?(u+=2,f=53):61===v.charCodeAt(u+1)?(u+=2,f=68):(u++,f=48);case 40:return u++,f=19;case 41:return u++,f=20;case 42:if(61===v.charCodeAt(u+1))return u+=2,f=61;if(42===v.charCodeAt(u+1))return 61===v.charCodeAt(u+2)?(u+=3,f=62):(u+=2,f=40);if(u++,S&&!t&&1&_){t=!0;continue}return f=39;case 43:return 43===v.charCodeAt(u+1)?(u+=2,f=43):61===v.charCodeAt(u+1)?(u+=2,f=59):(u++,f=37);case 44:return u++,f=26;case 45:return 45===v.charCodeAt(u+1)?(u+=2,f=44):61===v.charCodeAt(u+1)?(u+=2,f=60):(u++,f=38);case 46:return E(v.charCodeAt(u+1))?(g=M(),f=8):46===v.charCodeAt(u+1)&&46===v.charCodeAt(u+2)?(u+=3,f=24):(u++,f=23);case 47:if(47===v.charCodeAt(u+1)){for(u+=2;u<d&&!b(v.charCodeAt(u));)u++;if(r)continue;return f=2}if(42===v.charCodeAt(u+1)){u+=2,42===v.charCodeAt(u)&&47!==v.charCodeAt(u+1)&&(_|=2);for(var o=!1;u<d;){var s=v.charCodeAt(u);if(42===s&&47===v.charCodeAt(u+1)){u+=2,o=!0;break}b(s)&&(_|=1),u++}if(o||A(e.Diagnostics.Asterisk_Slash_expected),r)continue;return o||(_|=4),f=3}return 61===v.charCodeAt(u+1)?(u+=2,f=63):(u++,f=41);case 48:var l;if(u+2<d&&(88===v.charCodeAt(u+1)||120===v.charCodeAt(u+1)))return u+=2,(l=w(1,!0))<0&&(A(e.Diagnostics.Hexadecimal_digit_expected),l=0),g=\"\"+l,_|=64,f=8;if(u+2<d&&(66===v.charCodeAt(u+1)||98===v.charCodeAt(u+1)))return u+=2,(l=j(2))<0&&(A(e.Diagnostics.Binary_digit_expected),l=0),g=\"\"+l,_|=128,f=8;if(u+2<d&&(79===v.charCodeAt(u+1)||111===v.charCodeAt(u+1)))return u+=2,(l=j(8))<0&&(A(e.Diagnostics.Octal_digit_expected),l=0),g=\"\"+l,_|=256,f=8;if(u+1<d&&T(v.charCodeAt(u+1)))return g=\"\"+O(),_|=32,f=8;case 49:case 50:case 51:case 52:case 53:case 54:case 55:case 56:case 57:return g=M(),f=8;case 58:return u++,f=56;case 59:return u++,f=25;case 60:if(L(v,u)){if(u=x(v,u,A),r)continue;return f=7}return 60===v.charCodeAt(u+1)?61===v.charCodeAt(u+2)?(u+=3,f=65):(u+=2,f=45):61===v.charCodeAt(u+1)?(u+=2,f=30):1===i&&47===v.charCodeAt(u+1)&&42!==v.charCodeAt(u+2)?(u+=2,f=28):(u++,f=27);case 61:if(L(v,u)){if(u=x(v,u,A),r)continue;return f=7}return 61===v.charCodeAt(u+1)?61===v.charCodeAt(u+2)?(u+=3,f=34):(u+=2,f=32):62===v.charCodeAt(u+1)?(u+=2,f=36):(u++,f=58);case 62:if(L(v,u)){if(u=x(v,u,A),r)continue;return f=7}return u++,f=29;case 63:return u++,f=55;case 91:return u++,f=21;case 93:return u++,f=22;case 94:return 61===v.charCodeAt(u+1)?(u+=2,f=70):(u++,f=50);case 123:return u++,f=17;case 124:if(L(v,u)){if(u=x(v,u,A),r)continue;return f=7}return 124===v.charCodeAt(u+1)?(u+=2,f=54):61===v.charCodeAt(u+1)?(u+=2,f=69):(u++,f=49);case 125:return u++,f=18;case 126:return u++,f=52;case 64:return u++,f=57;case 92:var c=B();return c>=0&&R(c,n)?(u+=6,g=String.fromCharCode(c)+H(),f=U()):(A(e.Diagnostics.Invalid_character),u++,f=0);default:if(R(a,n)){for(u++;u<d&&N(a=v.charCodeAt(u),n);)u++;return g=v.substring(p,u),92===a&&(g+=H()),f=U()}if(h(a)){u++;continue}if(b(a)){_|=1,u++;continue}return A(e.Diagnostics.Invalid_character),u++,f=0}}}function q(){if(m=p=u,u>=d)return f=1;var e=v.charCodeAt(u);if(60===e)return 47===v.charCodeAt(u+1)?(u+=2,f=28):(u++,f=27);if(123===e)return u++,f=17;for(var n=0;u<d&&123!==(e=v.charCodeAt(u));){if(60===e){if(L(v,u))return u=x(v,u,A),f=7;break}b(e)&&0===n?n=-1:y(e)||(n=u),u++}return-1===n?11:10}function z(e,n){var t=u,r=m,a=p,i=f,o=g,s=_,l=e();return l&&!n||(u=t,m=r,p=a,f=i,g=o,_=s),l}function J(e,n,t){v=e||\"\",d=void 0===t?v.length:n+t,X(n||0)}function X(n){e.Debug.assert(n>=0),u=n,m=n,p=n,f=0,g=void 0,_=0}}}(Tn||(Tn={})),function(e){e.isExternalModuleNameRelative=function(n){return e.pathIsRelative(n)||e.isRootedDiskPath(n)},e.sortAndDeduplicateDiagnostics=function(n){return e.sortAndDeduplicate(n,e.compareDiagnostics)}}(Tn||(Tn={})),function(e){e.resolvingEmptyArray=[],e.emptyMap=e.createMap(),e.emptyUnderscoreEscapedMap=e.emptyMap,e.externalHelpersModuleNameText=\"tslib\",e.defaultMaximumTruncationLength=160,e.getDeclarationOfKind=function(e,n){var t=e.declarations;if(t)for(var r=0,a=t;r<a.length;r++){var i=a[r];if(i.kind===n)return i}},e.createUnderscoreEscapedMap=function(){return new e.MapCtr},e.hasEntries=function(e){return!!e&&!!e.size},e.createSymbolTable=function(n){var t=e.createMap();if(n)for(var r=0,a=n;r<a.length;r++){var i=a[r];t.set(i.escapedName,i)}return t};var n,t,r=(n=\"\",{getText:function(){return n},write:t=function(e){return n+=e},rawWrite:t,writeTextOfNode:t,writeKeyword:t,writeOperator:t,writePunctuation:t,writeSpace:t,writeStringLiteral:t,writeLiteral:t,writeParameter:t,writeProperty:t,writeSymbol:t,getTextPos:function(){return n.length},getLine:function(){return 0},getColumn:function(){return 0},getIndent:function(){return 0},isAtStartOfLine:function(){return!1},writeLine:function(){return n+=\" \"},increaseIndent:e.noop,decreaseIndent:e.noop,clear:function(){return n=\"\"},trackSymbol:e.noop,reportInaccessibleThisError:e.noop,reportInaccessibleUniqueSymbolError:e.noop,reportPrivateInBaseOfClassExpression:e.noop});function a(n,t,r){return r(e.isRootedDiskPath(n)?e.normalizePath(n):e.getNormalizedAbsolutePath(n,t))}function i(e,n){for(;e;){var t=n(e);if(\"quit\"===t)return;if(t)return e;e=e.parent}}function o(e,n){e.forEach(function(e,t){n.set(t,e)})}function s(e){return e.end-e.pos}function l(n){return function(n){if(!(262144&n.flags)){var t=0!=(32768&n.flags)||e.forEachChild(n,l);t&&(n.flags|=131072),n.flags|=262144}}(n),0!=(131072&n.flags)}function c(e){for(;e&&277!==e.kind;)e=e.parent;return e}function u(n,t){e.Debug.assert(n>=0);var r=e.getLineStarts(t),a=n,i=t.text;if(a+1===r.length)return i.length-1;var o=r[a],s=r[a+1]-1;for(e.Debug.assert(e.isLineBreak(i.charCodeAt(s)));o<=s&&e.isLineBreak(i.charCodeAt(s));)s--;return s}function d(e){return void 0===e||e.pos===e.end&&e.pos>=0&&1!==e.kind}function m(e){return!d(e)}function p(e,n){return 42===e.charCodeAt(n+1)&&33===e.charCodeAt(n+2)}function f(n,t,r){return d(n)?n.pos:e.isJSDocNode(n)?e.skipTrivia((t||c(n)).text,n.pos,!1,!0):r&&e.hasJSDocNodes(n)?f(n.jsDoc[0]):304===n.kind&&n._children.length>0?f(n._children[0],t,r):e.skipTrivia((t||c(n)).text,n.pos)}function g(e,n,t){return void 0===t&&(t=!1),_(e.text,n,t)}function _(n,t,r){if(void 0===r&&(r=!1),d(t))return\"\";var a=n.substring(r?t.pos:e.skipTrivia(n,t.pos),t.end);return function e(n){return 281===n.kind||n.parent&&e(n.parent)}(t)&&(a=a.replace(/(^|\\r?\\n|\\r)\\s*\\*\\s*/g,\"$1\")),a}function v(e,n){return void 0===n&&(n=!1),g(c(e),e,n)}function y(e){return e.pos}function h(e){var n=e.emitNode;return n&&n.flags||0}function b(e){var n=Ge(e);return 235===n.kind&&272===n.parent.kind}function E(n){return e.isModuleDeclaration(n)&&(9===n.name.kind||T(n))}function T(e){return!!(512&e.flags)}function S(e){return E(e)&&L(e)}function L(n){switch(n.parent.kind){case 277:return e.isExternalModule(n.parent);case 243:return E(n.parent.parent)&&e.isSourceFile(n.parent.parent.parent)&&!e.isExternalModule(n.parent.parent.parent)}return!1}function x(n,t){switch(n.kind){case 277:case 244:case 272:case 242:case 223:case 224:case 225:case 155:case 154:case 156:case 157:case 237:case 194:case 195:return!0;case 216:return!e.isFunctionLike(t)}return!1}function A(n){switch(n.kind){case 158:case 159:case 153:case 160:case 163:case 164:case 287:case 238:case 207:case 239:case 240:case 301:case 237:case 154:case 155:case 156:case 157:case 194:case 195:return!0;default:return e.assertType(n),!1}}function C(e){switch(e.kind){case 247:case 246:return!0;default:return!1}}function D(e){return e&&0!==s(e)?v(e):\"(Missing)\"}function k(n){switch(n.kind){case 71:return n.escapedText;case 9:case 8:return e.escapeLeadingUnderscores(n.text);case 147:return Oe(n.expression)?e.escapeLeadingUnderscores(n.expression.text):void 0;default:return e.Debug.assertNever(n)}}function M(n,t,r,a,i,o,s){var l=I(n,t);return e.createFileDiagnostic(n,l.start,l.length,r,a,i,o,s)}function O(n,t){var r=e.createScanner(n.languageVersion,!0,n.languageVariant,n.text,void 0,t);r.scan();var a=r.getTokenPos();return e.createTextSpanFromBounds(a,r.getTextPos())}function I(n,t){var r=t;switch(t.kind){case 277:var a=e.skipTrivia(n.text,0,!1);return a===n.text.length?e.createTextSpan(0,0):O(n,a);case 235:case 184:case 238:case 207:case 239:case 242:case 241:case 276:case 237:case 194:case 154:case 156:case 157:case 240:case 152:case 151:r=t.name;break;case 195:return function(n,t){var r=e.skipTrivia(n.text,t.pos);if(t.body&&216===t.body.kind){var a=e.getLineAndCharacterOfPosition(n,t.body.pos).line;if(a<e.getLineAndCharacterOfPosition(n,t.body.end).line)return e.createTextSpan(r,u(a,n)-r+1)}return e.createTextSpanFromBounds(r,t.end)}(n,t)}if(void 0===r)return O(n,t.pos);var i=d(r),o=i?r.pos:e.skipTrivia(n.text,r.pos);return i?(e.Debug.assert(o===r.pos,\"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809\"),e.Debug.assert(o===r.end,\"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809\")):(e.Debug.assert(o>=r.pos,\"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809\"),e.Debug.assert(o<=r.end,\"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809\")),e.createTextSpanFromBounds(o,r.end)}function R(n){return!!(2&e.getCombinedNodeFlags(n))}function N(n){return e.isImportTypeNode(n)&&e.isLiteralTypeNode(n.argument)&&e.isStringLiteral(n.argument.literal)}function w(e){return 219===e.kind&&9===e.expression.kind}e.toPath=a,e.changesAffectModuleResolution=function(n,t){return n.configFilePath!==t.configFilePath||e.moduleResolutionOptionDeclarations.some(function(r){return!e.isJsonEqual(e.getCompilerOptionValue(n,r),e.getCompilerOptionValue(t,r))})},e.findAncestor=i,e.forEachAncestor=function(n,t){for(;;){var r=t(n);if(\"quit\"===r)return;if(void 0!==r)return r;if(e.isSourceFile(n))return;n=n.parent}},e.forEachEntry=function(e,n){for(var t,r=e.entries(),a=r.next(),i=a.value,o=a.done;!o;i=(t=r.next()).value,o=t.done,t){var s=i[0],l=n(i[1],s);if(l)return l}},e.forEachKey=function(e,n){for(var t,r=e.keys(),a=r.next(),i=a.value,o=a.done;!o;i=(t=r.next()).value,o=t.done,t){var s=n(i);if(s)return s}},e.copyEntries=o,e.arrayToSet=function(n,t){return e.arrayToMap(n,t||function(e){return e},function(){return!0})},e.cloneMap=function(n){var t=e.createMap();return o(n,t),t},e.usingSingleLineStringWriter=function(e){var n=r.getText();try{return e(r),r.getText()}finally{r.clear(),r.writeKeyword(n)}},e.getFullWidth=s,e.getResolvedModule=function(e,n){return e&&e.resolvedModules&&e.resolvedModules.get(n)},e.setResolvedModule=function(n,t,r){n.resolvedModules||(n.resolvedModules=e.createMap()),n.resolvedModules.set(t,r)},e.setResolvedTypeReferenceDirective=function(n,t,r){n.resolvedTypeReferenceDirectiveNames||(n.resolvedTypeReferenceDirectiveNames=e.createMap()),n.resolvedTypeReferenceDirectiveNames.set(t,r)},e.projectReferenceIsEqualTo=function(e,n){return e.path===n.path&&!e.prepend==!n.prepend&&!e.circular==!n.circular},e.moduleResolutionIsEqualTo=function(e,n){return e.isExternalLibraryImport===n.isExternalLibraryImport&&e.extension===n.extension&&e.resolvedFileName===n.resolvedFileName&&e.originalPath===n.originalPath&&(t=e.packageId,r=n.packageId,t===r||!!t&&!!r&&t.name===r.name&&t.subModuleName===r.subModuleName&&t.version===r.version);var t,r},e.packageIdToString=function(e){var n=e.name,t=e.subModuleName;return(t?n+\"/\"+t:n)+\"@\"+e.version},e.typeDirectiveIsEqualTo=function(e,n){return e.resolvedFileName===n.resolvedFileName&&e.primary===n.primary},e.hasChangesInResolutions=function(n,t,r,a){e.Debug.assert(n.length===t.length);for(var i=0;i<n.length;i++){var o=t[i],s=r&&r.get(n[i]);if(s?!o||!a(s,o):o)return!0}return!1},e.containsParseError=l,e.getSourceFileOfNode=c,e.isStatementWithLocals=function(e){switch(e.kind){case 216:case 244:case 223:case 224:case 225:return!0}return!1},e.getStartPositionOfLine=function(n,t){return e.Debug.assert(n>=0),e.getLineStarts(t)[n]},e.nodePosToString=function(n){var t=c(n),r=e.getLineAndCharacterOfPosition(t,n.pos);return t.fileName+\"(\"+(r.line+1)+\",\"+(r.character+1)+\")\"},e.getEndLinePosition=u,e.isFileLevelUniqueName=function(e,n,t){return!(t&&t(n)||e.identifiers.has(n))},e.nodeIsMissing=d,e.nodeIsPresent=m,e.addStatementsAfterPrologue=function(e,n){if(void 0===n||0===n.length)return e;for(var t=0;t<e.length&&w(e[t]);++t);return e.splice.apply(e,[t,0].concat(n)),e},e.isRecognizedTripleSlashComment=function(n,t,r){if(47===n.charCodeAt(t+1)&&t+2<r&&47===n.charCodeAt(t+2)){var a=n.substring(t,r);return!!(a.match(e.fullTripleSlashReferencePathRegEx)||a.match(e.fullTripleSlashAMDReferencePathRegEx)||a.match(P)||a.match(F))}return!1},e.isPinnedComment=p,e.getTokenPosOfNode=f,e.getNonDecoratorTokenPosOfNode=function(n,t){return d(n)||!n.decorators?f(n,t):e.skipTrivia((t||c(n)).text,n.decorators.end)},e.getSourceTextOfNodeFromSourceFile=g,e.getTextOfNodeFromSourceText=_,e.getTextOfNode=v,e.indexOfNode=function(n,t){return e.binarySearch(n,t,y,e.compareValues)},e.getEmitFlags=h,e.getLiteralText=function(n,t,r){if(!Ve(n)&&n.parent&&!(e.isNumericLiteral(n)&&512&n.numericLiteralFlags))return g(t,n);var a=r||16777216&h(n)?Je:Ze;switch(n.kind){case 9:return n.singleQuote?\"'\"+a(n.text,39)+\"'\":'\"'+a(n.text,34)+'\"';case 13:return\"`\"+a(n.text,96)+\"`\";case 14:return\"`\"+a(n.text,96)+\"${\";case 15:return\"}\"+a(n.text,96)+\"${\";case 16:return\"}\"+a(n.text,96)+\"`\";case 8:case 12:return n.text}return e.Debug.fail(\"Literal kind '\"+n.kind+\"' not accounted for.\")},e.getTextOfConstantValue=function(n){return e.isString(n)?'\"'+Ze(n)+'\"':\"\"+n},e.makeIdentifierFromModuleName=function(n){return e.getBaseFileName(n).replace(/^(\\d)/,\"_$1\").replace(/\\W/g,\"_\")},e.isBlockOrCatchScoped=function(n){return 0!=(3&e.getCombinedNodeFlags(n))||b(n)},e.isCatchClauseVariableDeclarationOrBindingElement=b,e.isAmbientModule=E,e.isModuleWithStringLiteralName=function(n){return e.isModuleDeclaration(n)&&9===n.name.kind},e.isNonGlobalAmbientModule=function(n){return e.isModuleDeclaration(n)&&e.isStringLiteral(n.name)},e.isEffectiveModuleDeclaration=function(n){return e.isModuleDeclaration(n)||e.isIdentifier(n)},e.isShorthandAmbientModuleSymbol=function(e){return(n=e.valueDeclaration)&&242===n.kind&&!n.body;var n},e.isBlockScopedContainerTopLevel=function(n){return 277===n.kind||242===n.kind||e.isFunctionLike(n)},e.isGlobalScopeAugmentation=T,e.isExternalModuleAugmentation=S,e.isModuleAugmentationExternal=L,e.getNonAugmentationDeclaration=function(n){return e.find(n.declarations,function(n){return!(S(n)||e.isModuleDeclaration(n)&&T(n))})},e.isEffectiveExternalModule=function(n,t){return e.isExternalModule(n)||t.isolatedModules||e.getEmitModuleKind(t)===e.ModuleKind.CommonJS&&!!n.commonJsModuleIndicator},e.isBlockScope=x,e.isDeclarationWithTypeParameters=function(n){switch(n.kind){case 295:case 302:case 291:return!0;default:return e.assertType(n),A(n)}},e.isDeclarationWithTypeParameterChildren=A,e.isAnyImportSyntax=C,e.isLateVisibilityPaintedStatement=function(e){switch(e.kind){case 247:case 246:case 217:case 238:case 237:case 242:case 240:case 239:case 241:return!0;default:return!1}},e.isAnyImportOrReExport=function(n){return C(n)||e.isExportDeclaration(n)},e.getEnclosingBlockScopeContainer=function(e){return i(e.parent,function(e){return x(e,e.parent)})},e.declarationNameToString=D,e.getNameFromIndexInfo=function(e){return e.declaration?D(e.declaration.parameters[0].name):void 0},e.getTextOfPropertyName=k,e.entityNameToString=function n(t){switch(t.kind){case 71:return 0===s(t)?e.idText(t):v(t);case 146:return n(t.left)+\".\"+n(t.right);case 187:return n(t.expression)+\".\"+n(t.name);default:throw e.Debug.assertNever(t)}},e.createDiagnosticForNode=function(e,n,t,r,a,i){return M(c(e),e,n,t,r,a,i)},e.createDiagnosticForNodeArray=function(n,t,r,a,i,o,s){var l=e.skipTrivia(n.text,t.pos);return e.createFileDiagnostic(n,l,t.end-l,r,a,i,o,s)},e.createDiagnosticForNodeInSourceFile=M,e.createDiagnosticForNodeFromMessageChain=function(e,n,t){var r=c(e),a=I(r,e);return{file:r,start:a.start,length:a.length,code:n.code,category:n.category,messageText:n.next?n:n.messageText,relatedInformation:t}},e.getSpanOfTokenAtPosition=O,e.getErrorSpanForNode=I,e.isExternalOrCommonJsModule=function(e){return void 0!==(e.externalModuleIndicator||e.commonJsModuleIndicator)},e.isJsonSourceFile=function(e){return 6===e.scriptKind},e.isEnumConst=function(n){return!!(2048&e.getCombinedModifierFlags(n))},e.isDeclarationReadonly=function(n){return!(!(64&e.getCombinedModifierFlags(n))||e.isParameterPropertyDeclaration(n))},e.isVarConst=R,e.isLet=function(n){return!!(1&e.getCombinedNodeFlags(n))},e.isSuperCall=function(e){return 189===e.kind&&97===e.expression.kind},e.isImportCall=function(e){return 189===e.kind&&91===e.expression.kind},e.isLiteralImportTypeNode=N,e.isPrologueDirective=w,e.getLeadingCommentRangesOfNode=function(n,t){return 10!==n.kind?e.getLeadingCommentRanges(t.text,n.pos):void 0},e.getJSDocCommentRanges=function(n,t){var r=149===n.kind||148===n.kind||194===n.kind||195===n.kind||193===n.kind?e.concatenate(e.getTrailingCommentRanges(t,n.pos),e.getLeadingCommentRanges(t,n.pos)):e.getLeadingCommentRanges(t,n.pos);return e.filter(r,function(e){return 42===t.charCodeAt(e.pos+1)&&42===t.charCodeAt(e.pos+2)&&47!==t.charCodeAt(e.pos+3)})},e.fullTripleSlashReferencePathRegEx=/^(\\/\\/\\/\\s*<reference\\s+path\\s*=\\s*)('|\")(.+?)\\2.*?\\/>/;var P=/^(\\/\\/\\/\\s*<reference\\s+types\\s*=\\s*)('|\")(.+?)\\2.*?\\/>/;e.fullTripleSlashAMDReferencePathRegEx=/^(\\/\\/\\/\\s*<amd-dependency\\s+path\\s*=\\s*)('|\")(.+?)\\2.*?\\/>/;var F=/^(\\/\\/\\/\\s*<reference\\s+no-default-lib\\s*=\\s*)('|\")(.+?)\\2\\s*\\/>/;function G(n){if(161<=n.kind&&n.kind<=181)return!0;switch(n.kind){case 119:case 142:case 134:case 137:case 122:case 138:case 140:case 131:return!0;case 105:return 198!==n.parent.kind;case 209:return!On(n);case 148:return 179===n.parent.kind||174===n.parent.kind;case 71:146===n.parent.kind&&n.parent.right===n?n=n.parent:187===n.parent.kind&&n.parent.name===n&&(n=n.parent),e.Debug.assert(71===n.kind||146===n.kind||187===n.kind,\"'node' was expected to be a qualified name, identifier or property access in 'isPartOfTypeNode'.\");case 146:case 187:case 99:var t=n.parent;if(165===t.kind)return!1;if(181===t.kind)return!t.isTypeOf;if(161<=t.kind&&t.kind<=181)return!0;switch(t.kind){case 209:return!On(t);case 148:case 301:return n===t.constraint;case 152:case 151:case 149:case 235:return n===t.type;case 237:case 194:case 195:case 155:case 154:case 153:case 156:case 157:return n===t.type;case 158:case 159:case 160:case 192:return n===t.type;case 189:case 190:return e.contains(t.typeArguments,n);case 191:return!1}}return!1}function V(e){if(e)switch(e.kind){case 184:case 276:case 149:case 273:case 152:case 151:case 274:case 235:return!0}return!1}function K(e){return 236===e.parent.kind&&217===e.parent.parent.kind}function B(e,n,t){return e.properties.filter(function(e){if(273===e.kind){var r=k(e.name);return n===r||!!t&&t===r}return!1})}function H(n){if(n&&n.statements.length){var t=n.statements[0].expression;return e.tryCast(t,e.isObjectLiteralExpression)}}function U(n,t){var r=H(n);return r?B(r,t):e.emptyArray}function j(n,t){for(e.Debug.assert(277!==n.kind);;){if(!(n=n.parent))return e.Debug.fail();switch(n.kind){case 147:if(e.isClassLike(n.parent.parent))return n;n=n.parent;break;case 150:149===n.parent.kind&&e.isClassElement(n.parent.parent)?n=n.parent.parent:e.isClassElement(n.parent)&&(n=n.parent);break;case 195:if(!t)continue;case 237:case 194:case 242:case 152:case 151:case 154:case 153:case 155:case 156:case 157:case 158:case 159:case 160:case 241:case 277:return n}}}function W(e,n,t){switch(e.kind){case 238:return!0;case 152:return 238===n.kind;case 156:case 157:case 154:return void 0!==e.body&&238===n.kind;case 149:return void 0!==n.body&&(155===n.kind||154===n.kind||157===n.kind)&&238===t.kind}return!1}function q(e,n,t){return void 0!==e.decorators&&W(e,n,t)}function z(e,n,t){return q(e,n,t)||J(e,n)}function J(n,t){switch(n.kind){case 238:return e.some(n.members,function(e){return z(e,n,t)});case 154:case 157:return e.some(n.parameters,function(e){return q(e,n,t)});default:return!1}}function X(e){var n=e.parent;return(260===n.kind||259===n.kind||261===n.kind)&&n.tagName===e}function Y(e){switch(e.kind){case 97:case 95:case 101:case 86:case 12:case 185:case 186:case 187:case 188:case 189:case 190:case 191:case 210:case 192:case 211:case 193:case 194:case 207:case 195:case 198:case 196:case 197:case 200:case 201:case 202:case 203:case 206:case 204:case 13:case 208:case 258:case 259:case 262:case 205:case 199:case 212:return!0;case 146:for(;146===e.parent.kind;)e=e.parent;return 165===e.parent.kind||X(e);case 71:if(165===e.parent.kind||X(e))return!0;case 8:case 9:case 99:return Q(e);default:return!1}}function Q(e){var n=e.parent;switch(n.kind){case 235:case 149:case 152:case 151:case 276:case 273:case 184:return n.initializer===e;case 219:case 220:case 221:case 222:case 228:case 229:case 230:case 269:case 232:return n.expression===e;case 223:var t=n;return t.initializer===e&&236!==t.initializer.kind||t.condition===e||t.incrementor===e;case 224:case 225:var r=n;return r.initializer===e&&236!==r.initializer.kind||r.expression===e;case 192:case 210:case 214:case 147:return e===n.expression;case 150:case 268:case 267:case 275:return!0;case 209:return n.expression===e&&On(n);case 274:return n.objectAssignmentInitializer===e;default:return Y(n)}}function Z(e){return 246===e.kind&&257===e.moduleReference.kind}function $(e){return ee(e)}function ee(e){return!!e&&!!(65536&e.flags)}function ne(n){return ee(n)&&n.initializer&&e.isBinaryExpression(n.initializer)&&54===n.initializer.operatorToken.kind&&n.name&&In(n.name)&&re(n.name,n.initializer.left)?n.initializer.right:n.initializer}function te(n,t){if(e.isCallExpression(n)){var r=Ee(n.expression);return 194===r.kind||195===r.kind?n:void 0}return 194===n.kind||207===n.kind||195===n.kind?n:e.isObjectLiteralExpression(n)&&(0===n.properties.length||t)?n:void 0}function re(n,t){return e.isIdentifier(n)&&e.isIdentifier(t)?n.escapedText===t.escapedText:e.isIdentifier(n)&&e.isPropertyAccessExpression(t)?(99===t.expression.kind||e.isIdentifier(t.expression)&&(\"window\"===t.expression.escapedText||\"self\"===t.expression.escapedText||\"global\"===t.expression.escapedText))&&re(n,t.name):!(!e.isPropertyAccessExpression(n)||!e.isPropertyAccessExpression(t))&&(n.name.escapedText===t.name.escapedText&&re(n.expression,t.expression))}function ae(n){var t=function(n){if(58!==n.operatorToken.kind||!e.isPropertyAccessExpression(n.left))return 0;var t=n.left;if(In(t.expression)&&\"prototype\"===t.name.escapedText&&e.isObjectLiteralExpression(oe(n)))return 6;return ie(t)}(n);return 5===t||ee(n)?t:0}function ie(n){if(99===n.expression.kind)return 4;if(e.isIdentifier(n.expression)&&\"module\"===n.expression.escapedText&&\"exports\"===n.name.escapedText)return 2;if(In(n.expression)){if(Nn(n.expression))return 3;for(var t=n;e.isPropertyAccessExpression(t.expression);)t=t.expression;e.Debug.assert(e.isIdentifier(t.expression));var r=t.expression;return\"exports\"===r.escapedText||\"module\"===r.escapedText&&\"exports\"===t.name.escapedText?1:5}return 0}function oe(n){for(;e.isBinaryExpression(n.right);)n=n.right;return n.right}function se(n){switch(n.parent.kind){case 247:case 253:return n.parent;case 257:return n.parent.parent;case 189:return n.parent;case 180:return e.Debug.assert(e.isStringLiteral(n)),e.tryCast(n.parent.parent,e.isImportTypeNode);default:return}}function le(e){return 302===e.kind||295===e.kind}function ce(n){return e.isExpressionStatement(n)&&e.isBinaryExpression(n.expression)&&0!==ae(n.expression)&&e.isBinaryExpression(n.expression.right)&&54===n.expression.right.operatorToken.kind?n.expression.right.right:void 0}function ue(e){switch(e.kind){case 217:var n=de(e);return n&&n.initializer;case 152:case 273:return e.initializer}}function de(n){return e.isVariableStatement(n)?e.firstOrUndefined(n.declarationList.declarations):void 0}function me(n){return e.isModuleDeclaration(n)&&n.body&&242===n.body.kind?n.body:void 0}function pe(n){var t=n.parent;return 273===t.kind||152===t.kind||219===t.kind&&187===n.kind||me(t)||e.isBinaryExpression(n)&&58===n.operatorToken.kind?t:t.parent&&(de(t.parent)===n||e.isBinaryExpression(t)&&58===t.operatorToken.kind)?t.parent:t.parent&&t.parent.parent&&(de(t.parent.parent)||ue(t.parent.parent)===n||ce(t.parent.parent))?t.parent.parent:void 0}function fe(e){return ge(_e(e))}function ge(n){var t,r=ce(n)||(t=n,e.isExpressionStatement(t)&&t.expression&&e.isBinaryExpression(t.expression)&&58===t.expression.operatorToken.kind?t.expression.right:void 0)||ue(n)||de(n)||me(n)||n;return r&&e.isFunctionLike(r)?r:void 0}function _e(n){return e.Debug.assertDefined(i(n.parent,e.isJSDoc)).parent}function ve(n){var t=e.isJSDocParameterTag(n)?n.typeExpression&&n.typeExpression.type:n.type;return void 0!==n.dotDotDotToken||!!t&&288===t.kind}function ye(e){for(var n=e.parent;;){switch(n.kind){case 202:var t=n.operatorToken.kind;return Dn(t)&&n.left===e?58===t?1:2:0;case 200:case 201:var r=n.operator;return 43===r||44===r?2:0;case 224:case 225:return n.initializer===e?1:0;case 193:case 185:case 206:case 211:e=n;break;case 274:if(n.name!==e)return 0;e=n.parent;break;case 273:if(n.name===e)return 0;e=n.parent;break;default:return 0}n=e.parent}}function he(e,n){for(;e&&e.kind===n;)e=e.parent;return e}function be(e){return he(e,193)}function Ee(e){for(;193===e.kind;)e=e.expression;return e}function Te(n){var t=e.isExportAssignment(n)?n.expression:n.right;return In(t)||e.isClassExpression(t)}function Se(n){if(ee(n)){var t=e.getJSDocAugmentsTag(n);if(t)return t.class}return Le(n)}function Le(e){var n=Ce(e.heritageClauses,85);return n&&n.types.length>0?n.types[0]:void 0}function xe(e){var n=Ce(e.heritageClauses,108);return n?n.types:void 0}function Ae(e){var n=Ce(e.heritageClauses,85);return n?n.types:void 0}function Ce(e,n){if(e)for(var t=0,r=e;t<r.length;t++){var a=r[t];if(a.token===n)return a}}function De(e){return 72<=e&&e<=145}function ke(e){return 117<=e&&e<=145}function Me(e){return De(e)&&!ke(e)}function Oe(n){return e.isStringLiteralLike(n)||e.isNumericLiteral(n)}function Ie(n){var t=e.getNameOfDeclaration(n);return!!t&&Re(t)}function Re(e){return 147===e.kind&&!Oe(e.expression)&&!Ne(e.expression)}function Ne(n){return e.isPropertyAccessExpression(n)&&Fe(n.expression)}function we(n){switch(n.kind){case 71:return n.escapedText;case 9:case 8:return e.escapeLeadingUnderscores(n.text);case 147:var t=n.expression;return Ne(t)?Pe(e.idText(t.name)):Oe(t)?e.escapeLeadingUnderscores(t.text):void 0;default:return e.Debug.assertNever(n)}}function Pe(e){return\"__@\"+e}function Fe(e){return 71===e.kind&&\"Symbol\"===e.escapedText}function Ge(e){for(;184===e.kind;)e=e.parent.parent;return e}function Ve(n){return e.positionIsSynthesized(n.pos)||e.positionIsSynthesized(n.end)}function Ke(e,n,t){switch(e){case 190:return t?0:1;case 200:case 197:case 198:case 196:case 199:case 203:case 205:return 1;case 202:switch(n){case 40:case 58:case 59:case 60:case 62:case 61:case 63:case 64:case 65:case 66:case 67:case 68:case 70:case 69:return 1}}return 0}function Be(e){return 202===e.kind?e.operatorToken.kind:200===e.kind||201===e.kind?e.operator:e.kind}function He(e,n,t){switch(e){case 307:return 0;case 206:return 1;case 205:return 2;case 203:return 4;case 202:switch(n){case 26:return 0;case 58:case 59:case 60:case 62:case 61:case 63:case 64:case 65:case 66:case 67:case 68:case 70:case 69:return 3;default:return Ue(n)}case 200:case 197:case 198:case 196:case 199:return 16;case 201:return 17;case 189:return 18;case 190:return t?19:18;case 191:case 187:case 188:return 19;case 99:case 97:case 71:case 95:case 101:case 86:case 8:case 9:case 185:case 186:case 194:case 195:case 207:case 258:case 259:case 262:case 12:case 13:case 204:case 193:case 208:return 20;default:return-1}}function Ue(e){switch(e){case 54:return 5;case 53:return 6;case 49:return 7;case 50:return 8;case 48:return 9;case 32:case 33:case 34:case 35:return 10;case 27:case 29:case 30:case 31:case 93:case 92:case 118:return 11;case 45:case 46:case 47:return 12;case 37:case 38:return 13;case 39:case 41:case 42:return 14;case 40:return 15}return-1}e.isPartOfTypeNode=G,e.isChildOfNodeWithKind=function(e,n){for(;e;){if(e.kind===n)return!0;e=e.parent}return!1},e.forEachReturnStatement=function(n,t){return function n(r){switch(r.kind){case 228:return t(r);case 244:case 216:case 220:case 221:case 222:case 223:case 224:case 225:case 229:case 230:case 269:case 270:case 231:case 233:case 272:return e.forEachChild(r,n)}}(n)},e.forEachYieldExpression=function(n,t){return function n(r){switch(r.kind){case 205:t(r);var a=r.expression;return void(a&&n(a));case 241:case 239:case 242:case 240:case 238:case 207:return;default:if(e.isFunctionLike(r)){if(r.name&&147===r.name.kind)return void n(r.name.expression)}else G(r)||e.forEachChild(r,n)}}(n)},e.getRestParameterElementType=function(n){return n&&167===n.kind?n.elementType:n&&162===n.kind?e.singleOrUndefined(n.typeArguments):void 0},e.getMembersOfDeclaration=function(e){switch(e.kind){case 239:case 238:case 207:case 166:return e.members;case 186:return e.properties}},e.isVariableLike=V,e.isVariableLikeOrAccessor=function(n){return V(n)||e.isAccessor(n)},e.isVariableDeclarationInVariableStatement=K,e.isValidESSymbolDeclaration=function(n){return e.isVariableDeclaration(n)?R(n)&&e.isIdentifier(n.name)&&K(n):e.isPropertyDeclaration(n)?Sn(n)&&Tn(n):e.isPropertySignature(n)&&Sn(n)},e.introducesArgumentsExoticObject=function(e){switch(e.kind){case 154:case 153:case 155:case 156:case 157:case 237:case 194:return!0}return!1},e.unwrapInnermostStatementOfLabel=function(e,n){for(;;){if(n&&n(e),231!==e.statement.kind)return e.statement;e=e.statement}},e.isFunctionBlock=function(n){return n&&216===n.kind&&e.isFunctionLike(n.parent)},e.isObjectLiteralMethod=function(e){return e&&154===e.kind&&186===e.parent.kind},e.isObjectLiteralOrClassExpressionMethod=function(e){return 154===e.kind&&(186===e.parent.kind||207===e.parent.kind)},e.isIdentifierTypePredicate=function(e){return e&&1===e.kind},e.isThisTypePredicate=function(e){return e&&0===e.kind},e.getPropertyAssignment=B,e.getTsConfigObjectLiteralExpression=H,e.getTsConfigPropArrayElementValue=function(n,t,r){return e.firstDefined(U(n,t),function(n){return e.isArrayLiteralExpression(n.initializer)?e.find(n.initializer.elements,function(n){return e.isStringLiteral(n)&&n.text===r}):void 0})},e.getTsConfigPropArray=U,e.getContainingFunction=function(n){return i(n.parent,e.isFunctionLike)},e.getContainingClass=function(n){return i(n.parent,e.isClassLike)},e.getThisContainer=j,e.getNewTargetContainer=function(e){var n=j(e,!1);if(n)switch(n.kind){case 155:case 237:case 194:return n}},e.getSuperContainer=function(n,t){for(;;){if(!(n=n.parent))return n;switch(n.kind){case 147:n=n.parent;break;case 237:case 194:case 195:if(!t)continue;case 152:case 151:case 154:case 153:case 155:case 156:case 157:return n;case 150:149===n.parent.kind&&e.isClassElement(n.parent.parent)?n=n.parent.parent:e.isClassElement(n.parent)&&(n=n.parent)}}},e.getImmediatelyInvokedFunctionExpression=function(e){if(194===e.kind||195===e.kind){for(var n=e,t=e.parent;193===t.kind;)n=t,t=t.parent;if(189===t.kind&&t.expression===n)return t}},e.isSuperProperty=function(e){var n=e.kind;return(187===n||188===n)&&97===e.expression.kind},e.isThisProperty=function(e){var n=e.kind;return(187===n||188===n)&&99===e.expression.kind},e.getEntityNameFromTypeNode=function(e){switch(e.kind){case 162:return e.typeName;case 209:return In(e.expression)?e.expression:void 0;case 71:case 146:return e}},e.getInvokedExpression=function(e){switch(e.kind){case 191:return e.tag;case 260:case 259:return e.tagName;default:return e.expression}},e.nodeCanBeDecorated=W,e.nodeIsDecorated=q,e.nodeOrChildIsDecorated=z,e.childIsDecorated=J,e.isJSXTagName=X,e.isExpressionNode=Y,e.isInExpressionContext=Q,e.isExternalModuleImportEqualsDeclaration=Z,e.getExternalModuleImportEqualsDeclarationExpression=function(n){return e.Debug.assert(Z(n)),n.moduleReference.expression},e.isInternalModuleImportEqualsDeclaration=function(e){return 246===e.kind&&257!==e.moduleReference.kind},e.isSourceFileJS=$,e.isSourceFileNotJS=function(e){return!ee(e)},e.isInJSFile=ee,e.isInJsonFile=function(e){return!!e&&!!(16777216&e.flags)},e.isInJSDoc=function(e){return!!e&&!!(2097152&e.flags)},e.isJSDocIndexSignature=function(n){return e.isTypeReferenceNode(n)&&e.isIdentifier(n.typeName)&&\"Object\"===n.typeName.escapedText&&n.typeArguments&&2===n.typeArguments.length&&(137===n.typeArguments[0].kind||134===n.typeArguments[0].kind)},e.isRequireCall=function(n,t){if(189!==n.kind)return!1;var r=n,a=r.expression,i=r.arguments;if(71!==a.kind||\"require\"!==a.escapedText)return!1;if(1!==i.length)return!1;var o=i[0];return!t||e.isStringLiteralLike(o)},e.isSingleOrDoubleQuote=function(e){return 39===e||34===e},e.isStringDoubleQuoted=function(e,n){return 34===g(n,e).charCodeAt(0)},e.getDeclarationOfExpando=function(n){if(n.parent){var t,r;if(e.isVariableDeclaration(n.parent)&&n.parent.initializer===n){if(!ee(n)&&!R(n.parent))return;t=n.parent.name,r=n.parent}else if(e.isBinaryExpression(n.parent)&&58===n.parent.operatorToken.kind&&n.parent.right===n)r=t=n.parent.left;else if(e.isBinaryExpression(n.parent)&&54===n.parent.operatorToken.kind&&(e.isVariableDeclaration(n.parent.parent)&&n.parent.parent.initializer===n.parent?(t=n.parent.parent.name,r=n.parent.parent):e.isBinaryExpression(n.parent.parent)&&58===n.parent.parent.operatorToken.kind&&n.parent.parent.right===n.parent&&(r=t=n.parent.parent.left),!t||!In(t)||!re(t,n.parent.left)))return;if(t&&te(n,Nn(t)))return r}},e.isAssignmentDeclaration=function(n){return e.isBinaryExpression(n)||e.isPropertyAccessExpression(n)||e.isIdentifier(n)},e.getEffectiveInitializer=ne,e.getDeclaredExpandoInitializer=function(e){var n=ne(e);return n&&te(n,Nn(e.name))},e.getAssignedExpandoInitializer=function(n){if(n&&n.parent&&e.isBinaryExpression(n.parent)&&58===n.parent.operatorToken.kind){var t=Nn(n.parent.left);return te(n.parent.right,t)||function(n,t,r){var a=e.isBinaryExpression(t)&&54===t.operatorToken.kind&&te(t.right,r);if(a&&re(n,t.left))return a}(n.parent.left,n.parent.right,t)}},e.getExpandoInitializer=te,e.isDefaultedExpandoInitializer=function(n){var t=e.isVariableDeclaration(n.parent)?n.parent.name:e.isBinaryExpression(n.parent)&&58===n.parent.operatorToken.kind?n.parent.left:void 0;return t&&te(n.right,Nn(t))&&In(t)&&re(t,n.left)},e.getNameOfExpando=function(n){if(e.isBinaryExpression(n.parent)){var t=54===n.parent.operatorToken.kind&&e.isBinaryExpression(n.parent.parent)?n.parent.parent:n.parent;if(58===t.operatorToken.kind&&e.isIdentifier(t.left))return t.left}else if(e.isVariableDeclaration(n.parent))return n.parent.name},e.getRightMostAssignedExpression=function(e){for(;Mn(e,!0);)e=e.right;return e},e.isExportsIdentifier=function(n){return e.isIdentifier(n)&&\"exports\"===n.escapedText},e.isModuleExportsPropertyAccessExpression=function(n){return e.isPropertyAccessExpression(n)&&e.isIdentifier(n.expression)&&\"module\"===n.expression.escapedText&&\"exports\"===n.name.escapedText},e.getAssignmentDeclarationKind=ae,e.getAssignmentDeclarationPropertyAccessKind=ie,e.getInitializerOfBinaryExpression=oe,e.isPrototypePropertyAssignment=function(n){return e.isBinaryExpression(n)&&3===ae(n)},e.isSpecialPropertyDeclaration=function(n){return ee(n)&&n.parent&&219===n.parent.kind&&!!e.getJSDocTypeTag(n.parent)},e.isFunctionSymbol=function(n){if(!n||!n.valueDeclaration)return!1;var t=n.valueDeclaration;return 237===t.kind||e.isVariableDeclaration(t)&&t.initializer&&e.isFunctionLike(t.initializer)},e.importFromModuleSpecifier=function(n){return se(n)||e.Debug.fail(e.Debug.showSyntaxKind(n.parent))},e.tryGetImportFromModuleSpecifier=se,e.getExternalModuleName=function(n){switch(n.kind){case 247:case 253:return n.moduleSpecifier;case 246:return 257===n.moduleReference.kind?n.moduleReference.expression:void 0;case 181:return N(n)?n.argument.literal:void 0;default:return e.Debug.assertNever(n)}},e.getNamespaceDeclarationNode=function(n){switch(n.kind){case 247:return n.importClause&&e.tryCast(n.importClause.namedBindings,e.isNamespaceImport);case 246:return n;case 253:return;default:return e.Debug.assertNever(n)}},e.isDefaultImport=function(e){return 247===e.kind&&!!e.importClause&&!!e.importClause.name},e.hasQuestionToken=function(e){if(e)switch(e.kind){case 149:case 154:case 153:case 274:case 273:case 152:case 151:return void 0!==e.questionToken}return!1},e.isJSDocConstructSignature=function(e){return 287===e.kind&&e.parameters.length>0&&e.parameters[0].name&&\"new\"===e.parameters[0].name.escapedText},e.isJSDocTypeAlias=le,e.isTypeAlias=function(n){return le(n)||e.isTypeAliasDeclaration(n)},e.getJSDocCommentsAndTags=function(n){var t;V(n)&&e.hasInitializer(n)&&e.hasJSDocNodes(n.initializer)&&(t=e.addRange(t,n.initializer.jsDoc));for(var r=n;r&&r.parent;){if(e.hasJSDocNodes(r)&&(t=e.addRange(t,r.jsDoc)),149===r.kind){t=e.addRange(t,e.getJSDocParameterTags(r));break}if(148===r.kind){t=e.addRange(t,e.getJSDocTypeParameterTags(r));break}r=pe(r)}return t||e.emptyArray},e.getParameterSymbolFromJSDoc=function(n){if(n.symbol)return n.symbol;if(e.isIdentifier(n.name)){var t=n.name.escapedText,r=fe(n);if(r){var a=e.find(r.parameters,function(e){return 71===e.name.kind&&e.name.escapedText===t});return a&&a.symbol}}},e.getHostSignatureFromJSDoc=fe,e.getHostSignatureFromJSDocHost=ge,e.getJSDocHost=_e,e.getTypeParameterFromJsDoc=function(n){var t=n.name.escapedText,r=n.parent.parent.parent.typeParameters;return e.find(r,function(e){return e.name.escapedText===t})},e.hasRestParameter=function(n){var t=e.lastOrUndefined(n.parameters);return!!t&&ve(t)},e.isRestParameter=ve,function(e){e[e.None=0]=\"None\",e[e.Definite=1]=\"Definite\",e[e.Compound=2]=\"Compound\"}(e.AssignmentKind||(e.AssignmentKind={})),e.getAssignmentTargetKind=ye,e.isAssignmentTarget=function(e){return 0!==ye(e)},e.isNodeWithPossibleHoistedDeclaration=function(e){switch(e.kind){case 216:case 217:case 229:case 220:case 230:case 244:case 269:case 270:case 231:case 223:case 224:case 225:case 221:case 222:case 233:case 272:return!0}return!1},e.isValueSignatureDeclaration=function(n){return e.isFunctionExpression(n)||e.isArrowFunction(n)||e.isMethodOrAccessor(n)||e.isFunctionDeclaration(n)||e.isConstructorDeclaration(n)},e.walkUpParenthesizedTypes=function(e){return he(e,175)},e.walkUpParenthesizedExpressions=be,e.skipParentheses=Ee,e.isDeleteTarget=function(e){return(187===e.kind||188===e.kind)&&(e=be(e.parent))&&196===e.kind},e.isNodeDescendantOf=function(e,n){for(;e;){if(e===n)return!0;e=e.parent}return!1},e.isDeclarationName=function(n){return!e.isSourceFile(n)&&!e.isBindingPattern(n)&&e.isDeclaration(n.parent)&&n.parent.name===n},e.getDeclarationFromName=function(n){var t=n.parent;switch(n.kind){case 9:case 8:if(e.isComputedPropertyName(t))return t.parent;case 71:if(e.isDeclaration(t))return t.name===n?t:void 0;if(e.isQualifiedName(t)){var r=t.parent;return e.isJSDocParameterTag(r)&&r.name===t?r:void 0}var a=t.parent;return e.isBinaryExpression(a)&&0!==ae(a)&&(a.left.symbol||a.symbol)&&e.getNameOfDeclaration(a)===n?a:void 0;default:return}},e.isLiteralComputedPropertyDeclarationName=function(n){return(9===n.kind||8===n.kind)&&147===n.parent.kind&&e.isDeclaration(n.parent.parent)},e.isIdentifierName=function(e){var n=e.parent;switch(n.kind){case 152:case 151:case 154:case 153:case 156:case 157:case 276:case 273:case 187:return n.name===e;case 146:if(n.right===e){for(;146===n.kind;)n=n.parent;return 165===n.kind||162===n.kind}return!1;case 184:case 251:return n.propertyName===e;case 255:case 265:return!0}return!1},e.isAliasSymbolDeclaration=function(n){return 246===n.kind||245===n.kind||248===n.kind&&!!n.name||249===n.kind||251===n.kind||255===n.kind||252===n.kind&&Te(n)||e.isBinaryExpression(n)&&2===ae(n)},e.exportAssignmentIsAlias=Te,e.getEffectiveBaseTypeNode=Se,e.getClassExtendsHeritageElement=Le,e.getClassImplementsHeritageClauseElements=xe,e.getAllSuperTypeNodes=function(n){return e.isInterfaceDeclaration(n)?Ae(n)||e.emptyArray:e.isClassLike(n)&&e.concatenate(e.singleElementArray(Se(n)),xe(n))||e.emptyArray},e.getInterfaceBaseTypeNodes=Ae,e.getHeritageClause=Ce,e.tryResolveScriptReference=function(n,t,r){if(!n.getCompilerOptions().noResolve){var a=e.isRootedDiskPath(r.fileName)?r.fileName:e.combinePaths(e.getDirectoryPath(t.fileName),r.fileName);return n.getSourceFile(a)}},e.getAncestor=function(e,n){for(;e;){if(e.kind===n)return e;e=e.parent}},e.isKeyword=De,e.isContextualKeyword=ke,e.isNonContextualKeyword=Me,e.isStringANonContextualKeyword=function(n){var t=e.stringToToken(n);return void 0!==t&&Me(t)},e.isTrivia=function(e){return 2<=e&&e<=7},function(e){e[e.Normal=0]=\"Normal\",e[e.Generator=1]=\"Generator\",e[e.Async=2]=\"Async\",e[e.Invalid=4]=\"Invalid\",e[e.AsyncGenerator=3]=\"AsyncGenerator\"}(e.FunctionFlags||(e.FunctionFlags={})),e.getFunctionFlags=function(e){if(!e)return 4;var n=0;switch(e.kind){case 237:case 194:case 154:e.asteriskToken&&(n|=1);case 195:En(e,256)&&(n|=2)}return e.body||(n|=4),n},e.isAsyncFunction=function(e){switch(e.kind){case 237:case 194:case 195:case 154:return void 0!==e.body&&void 0===e.asteriskToken&&En(e,256)}return!1},e.isStringOrNumericLiteralLike=Oe,e.hasDynamicName=Ie,e.isDynamicName=Re,e.isWellKnownSymbolSyntactically=Ne,e.getPropertyNameForPropertyNameNode=we,e.isPropertyNameLiteral=function(e){switch(e.kind){case 71:case 9:case 13:case 8:return!0;default:return!1}},e.getTextOfIdentifierOrLiteral=function(n){return 71===n.kind?e.idText(n):n.text},e.getEscapedTextOfIdentifierOrLiteral=function(n){return 71===n.kind?n.escapedText:e.escapeLeadingUnderscores(n.text)},e.getPropertyNameForKnownSymbolName=Pe,e.isKnownSymbol=function(n){return e.startsWith(n.escapedName,\"__@\")},e.isESSymbolIdentifier=Fe,e.isPushOrUnshiftIdentifier=function(e){return\"push\"===e.escapedText||\"unshift\"===e.escapedText},e.isParameterDeclaration=function(e){return 149===Ge(e).kind},e.getRootDeclaration=Ge,e.nodeStartsNewLexicalEnvironment=function(e){var n=e.kind;return 155===n||194===n||237===n||195===n||154===n||156===n||157===n||242===n||277===n},e.nodeIsSynthesized=Ve,e.getOriginalSourceFile=function(n){return e.getParseTreeNode(n,e.isSourceFile)||n},function(e){e[e.Left=0]=\"Left\",e[e.Right=1]=\"Right\"}(e.Associativity||(e.Associativity={})),e.getExpressionAssociativity=function(e){var n=Be(e),t=190===e.kind&&void 0!==e.arguments;return Ke(e.kind,n,t)},e.getOperatorAssociativity=Ke,e.getExpressionPrecedence=function(e){var n=Be(e),t=190===e.kind&&void 0!==e.arguments;return He(e.kind,n,t)},e.getOperator=Be,e.getOperatorPrecedence=He,e.getBinaryOperatorPrecedence=Ue,e.createDiagnosticCollection=function(){var n=[],t=[],r=e.createMap(),a=!1;return{add:function(i){var o;i.file?(o=r.get(i.file.fileName))||(o=[],r.set(i.file.fileName,o),e.insertSorted(t,i.file.fileName,e.compareStringsCaseSensitive)):(a&&(a=!1,n=n.slice()),o=n),e.insertSorted(o,i,e.compareDiagnostics)},lookup:function(t){var a;if(a=t.file?r.get(t.file.fileName):n){var i=e.binarySearch(a,t,e.identity,e.compareDiagnosticsSkipRelatedInformation);return i>=0?a[i]:void 0}},getGlobalDiagnostics:function(){return a=!0,n},getDiagnostics:function(a){if(a)return r.get(a)||[];var i=e.flatMap(t,function(e){return r.get(e)});return n.length?(i.unshift.apply(i,n),i):i},reattachFileDiagnostics:function(n){e.forEach(r.get(n.fileName),function(e){return e.file=n})}}};var je=/[\\\\\\\"\\u0000-\\u001f\\t\\v\\f\\b\\r\\n\\u2028\\u2029\\u0085]/g,We=/[\\\\\\'\\u0000-\\u001f\\t\\v\\f\\b\\r\\n\\u2028\\u2029\\u0085]/g,qe=/[\\\\\\`\\u0000-\\u001f\\t\\v\\f\\b\\r\\n\\u2028\\u2029\\u0085]/g,ze=e.createMapFromTemplate({\"\\t\":\"\\\\t\",\"\\v\":\"\\\\v\",\"\\f\":\"\\\\f\",\"\\b\":\"\\\\b\",\"\\r\":\"\\\\r\",\"\\n\":\"\\\\n\",\"\\\\\":\"\\\\\\\\\",'\"':'\\\\\"',\"'\":\"\\\\'\",\"`\":\"\\\\`\",\"\\u2028\":\"\\\\u2028\",\"\\u2029\":\"\\\\u2029\",\"\":\"\\\\u0085\"});function Je(e,n){var t=96===n?qe:39===n?We:je;return e.replace(t,Xe)}function Xe(e,n,t){if(0===e.charCodeAt(0)){var r=t.charCodeAt(n+e.length);return r>=48&&r<=57?\"\\\\x00\":\"\\\\0\"}return ze.get(e)||Ye(e.charCodeAt(0))}function Ye(e){return\"\\\\u\"+(\"0000\"+e.toString(16).toUpperCase()).slice(-4)}e.escapeString=Je,e.isIntrinsicJsxName=function(e){var n=e.charCodeAt(0);return n>=97&&n<=122||e.indexOf(\"-\")>-1};var Qe=/[^\\u0000-\\u007F]/g;function Ze(e,n){return e=Je(e,n),Qe.test(e)?e.replace(Qe,function(e){return Ye(e.charCodeAt(0))}):e}e.escapeNonAsciiString=Ze;var $e=[\"\",\"    \"];function en(e){return void 0===$e[e]&&($e[e]=en(e-1)+$e[1]),$e[e]}function nn(){return $e[1].length}function tn(e,n,t){return n.moduleName||rn(e,n.fileName,t&&t.fileName)}function rn(n,t,r){var i=function(e){return n.getCanonicalFileName(e)},o=a(r?e.getDirectoryPath(r):n.getCommonSourceDirectory(),n.getCurrentDirectory(),i),s=e.getNormalizedAbsolutePath(t,n.getCurrentDirectory()),l=e.getRelativePathToDirectoryOrUrl(o,s,o,i,!1),c=e.removeFileExtension(l);return r?e.ensurePathIsNonModuleName(c):c}function an(n,t,r,a,i){var o=t.declarationDir||t.outDir,s=o?ln(n,o,r,a,i):n;return e.removeFileExtension(s)+\".d.ts\"}function on(e,n,t){return!(n.noEmitForJsFiles&&$(e)||e.isDeclarationFile||t(e))}function sn(e,n,t){return ln(e,t,n.getCurrentDirectory(),n.getCommonSourceDirectory(),function(e){return n.getCanonicalFileName(e)})}function ln(n,t,r,a,i){var o=e.getNormalizedAbsolutePath(n,r);return o=0===i(o).indexOf(i(a))?o.substring(a.length):o,e.combinePaths(t,o)}function cn(n,t){return e.getLineAndCharacterOfPosition(n,t).line}function un(n,t){return e.computeLineAndCharacterOfPosition(n,t).line}function dn(e){if(e&&e.parameters.length>0){var n=2===e.parameters.length&&mn(e.parameters[0]);return e.parameters[n?1:0]}}function mn(e){return pn(e.name)}function pn(e){return!!e&&71===e.kind&&fn(e)}function fn(e){return 99===e.originalKeywordKind}function gn(n){var t=n.type;return t||!ee(n)?t:e.isJSDocPropertyLikeTag(n)?n.typeExpression&&n.typeExpression.type:e.getJSDocType(n)}function _n(e,n,t,r){vn(e,n,t.pos,r)}function vn(e,n,t,r){r&&r.length&&t!==r[0].pos&&un(e,t)!==un(e,r[0].pos)&&n.writeLine()}function yn(e,n,t,r,a,i,o,s){if(r&&r.length>0){a&&t.write(\" \");for(var l=!1,c=0,u=r;c<u.length;c++){var d=u[c];l&&(t.write(\" \"),l=!1),s(e,n,t,d.pos,d.end,o),d.hasTrailingNewLine?t.writeLine():l=!0}l&&i&&t.write(\" \")}}function hn(e,n,t,r,a,i){var o=Math.min(n,i-1),s=e.substring(a,o).replace(/^\\s+|\\s+$/g,\"\");s?(t.write(s),o!==n&&t.writeLine()):t.writeLiteral(r)}function bn(n,t,r){for(var a=0;t<r&&e.isWhiteSpaceSingleLine(n.charCodeAt(t));t++)9===n.charCodeAt(t)?a+=nn()-a%nn():a++;return a}function En(e,n){return!!Ln(e,n)}function Tn(e){return En(e,32)}function Sn(e){return En(e,64)}function Ln(e,n){return xn(e)&n}function xn(e){if(536870912&e.modifierFlagsCache)return-536870913&e.modifierFlagsCache;var n=An(e);return e.modifierFlagsCache=536870912|n,n}function An(e){var n=0;if(e.modifiers)for(var t=0,r=e.modifiers;t<r.length;t++){n|=Cn(r[t].kind)}return(4&e.flags||71===e.kind&&e.isInJSDocNamespace)&&(n|=1),n}function Cn(e){switch(e){case 115:return 32;case 114:return 4;case 113:return 16;case 112:return 8;case 117:return 128;case 84:return 1;case 124:return 2;case 76:return 2048;case 79:return 512;case 120:return 256;case 132:return 64}return 0}function Dn(e){return e>=58&&e<=70}function kn(n){if(e.isExpressionWithTypeArguments(n)&&85===n.parent.token&&e.isClassLike(n.parent.parent))return n.parent.parent}function Mn(n,t){return e.isBinaryExpression(n)&&(t?58===n.operatorToken.kind:Dn(n.operatorToken.kind))&&e.isLeftHandSideExpression(n.left)}function On(e){return void 0!==kn(e)}function In(e){return 71===e.kind||Rn(e)}function Rn(n){return e.isPropertyAccessExpression(n)&&In(n.expression)}function Nn(n){return e.isPropertyAccessExpression(n)&&\"prototype\"===n.name.escapedText}e.getIndentString=en,e.getIndentSize=nn,e.createTextWriter=function(n){var t,r,a,i,o;function s(n){var r=e.computeLineStarts(n);r.length>1?(i=i+r.length-1,o=t.length-n.length+e.last(r),a=o-t.length==0):a=!1}function l(e){e&&e.length&&(a&&(e=en(r)+e,a=!1),t+=e,s(e))}function c(){t=\"\",r=0,a=!0,i=0,o=0}return c(),{write:l,rawWrite:function(e){void 0!==e&&(t+=e,s(e))},writeTextOfNode:function(e,n){var t=_(e,n);l(t),s(t)},writeLiteral:function(e){e&&e.length&&l(e)},writeLine:function(){a||(i++,o=(t+=n).length,a=!0)},increaseIndent:function(){r++},decreaseIndent:function(){r--},getIndent:function(){return r},getTextPos:function(){return t.length},getLine:function(){return i},getColumn:function(){return a?r*nn():t.length-o},getText:function(){return t},isAtStartOfLine:function(){return a},clear:c,reportInaccessibleThisError:e.noop,reportPrivateInBaseOfClassExpression:e.noop,reportInaccessibleUniqueSymbolError:e.noop,trackSymbol:e.noop,writeKeyword:l,writeOperator:l,writeParameter:l,writeProperty:l,writePunctuation:l,writeSpace:l,writeStringLiteral:l,writeSymbol:l}},e.getResolvedExternalModuleName=tn,e.getExternalModuleNameFromDeclaration=function(e,n,t){var r=n.getExternalModuleFileFromDeclaration(t);if(r&&!r.isDeclarationFile)return tn(e,r)},e.getExternalModuleNameFromPath=rn,e.getOwnEmitOutputFilePath=function(n,t,r){var a=t.getCompilerOptions();return(a.outDir?e.removeFileExtension(sn(n,t,a.outDir)):e.removeFileExtension(n))+r},e.getDeclarationEmitOutputFilePath=function(e,n){return an(e,n.getCompilerOptions(),n.getCurrentDirectory(),n.getCommonSourceDirectory(),function(e){return n.getCanonicalFileName(e)})},e.getDeclarationEmitOutputFilePathWorker=an,e.getSourceFilesToEmit=function(n,t){var r=n.getCompilerOptions(),a=function(e){return n.isSourceFileFromExternalLibrary(e)};if(r.outFile||r.out){var i=e.getEmitModuleKind(r),o=i===e.ModuleKind.AMD||i===e.ModuleKind.System;return e.filter(n.getSourceFiles(),function(n){return(o||!e.isExternalModule(n))&&on(n,r,a)})}var s=void 0===t?n.getSourceFiles():[t];return e.filter(s,function(e){return on(e,r,a)})},e.sourceFileMayBeEmitted=on,e.getSourceFilePathInNewDir=sn,e.getSourceFilePathInNewDirWorker=ln,e.writeFile=function(n,t,r,a,i,o){n.writeFile(r,a,i,function(n){t.add(e.createCompilerDiagnostic(e.Diagnostics.Could_not_write_file_0_Colon_1,r,n))},o)},e.getLineOfLocalPosition=cn,e.getLineOfLocalPositionFromLineMap=un,e.getFirstConstructorWithBody=function(n){return e.find(n.members,function(n){return e.isConstructorDeclaration(n)&&m(n.body)})},e.getSetAccessorTypeAnnotationNode=function(e){var n=dn(e);return n&&n.type},e.getThisParameter=function(n){if(n.parameters.length&&!e.isJSDocSignature(n)){var t=n.parameters[0];if(mn(t))return t}},e.parameterIsThisKeyword=mn,e.isThisIdentifier=pn,e.identifierIsThisKeyword=fn,e.getAllAccessorDeclarations=function(n,t){var r,a,i,o;return Ie(t)?(r=t,156===t.kind?i=t:157===t.kind?o=t:e.Debug.fail(\"Accessor has wrong kind\")):e.forEach(n,function(n){e.isAccessor(n)&&En(n,32)===En(t,32)&&we(n.name)===we(t.name)&&(r?a||(a=n):r=n,156!==n.kind||i||(i=n),157!==n.kind||o||(o=n))}),{firstAccessor:r,secondAccessor:a,getAccessor:i,setAccessor:o}},e.getEffectiveTypeAnnotationNode=gn,e.getTypeAnnotationNode=function(e){return e.type},e.getEffectiveReturnTypeNode=function(n){return e.isJSDocSignature(n)?n.type&&n.type.typeExpression&&n.type.typeExpression.type:n.type||(ee(n)?e.getJSDocReturnType(n):void 0)},e.getJSDocTypeParameterDeclarations=function(n){return e.flatMap(e.getJSDocTags(n),function(n){return function(n){return e.isJSDocTemplateTag(n)&&!(289===n.parent.kind&&n.parent.tags.some(le))}(n)?n.typeParameters:void 0})},e.getEffectiveSetAccessorTypeAnnotationNode=function(e){var n=dn(e);return n&&gn(n)},e.emitNewLineBeforeLeadingComments=_n,e.emitNewLineBeforeLeadingCommentsOfPosition=vn,e.emitNewLineBeforeLeadingCommentOfPosition=function(e,n,t,r){t!==r&&un(e,t)!==un(e,r)&&n.writeLine()},e.emitComments=yn,e.emitDetachedComments=function(n,t,r,a,i,o,s){var l,c;if(s?0===i.pos&&(l=e.filter(e.getLeadingCommentRanges(n,i.pos),function(e){return p(n,e.pos)})):l=e.getLeadingCommentRanges(n,i.pos),l){for(var u=[],d=void 0,m=0,f=l;m<f.length;m++){var g=f[m];if(d){var _=un(t,d.end);if(un(t,g.pos)>=_+2)break}u.push(g),d=g}u.length&&(_=un(t,e.last(u).end),un(t,e.skipTrivia(n,i.pos))>=_+2&&(_n(t,r,i,l),yn(n,t,r,u,!1,!0,o,a),c={nodePos:i.pos,detachedCommentEndPos:e.last(u).end}))}return c},e.writeCommentRange=function(n,t,r,a,i,o){if(42===n.charCodeAt(a+1))for(var s=e.computeLineAndCharacterOfPosition(t,a),l=t.length,c=void 0,u=a,d=s.line;u<i;d++){var m=d+1===l?n.length+1:t[d+1];if(u!==a){void 0===c&&(c=bn(n,t[s.line],a));var p=r.getIndent()*nn()-c+bn(n,u,m);if(p>0){var f=p%nn(),g=en((p-f)/nn());for(r.rawWrite(g);f;)r.rawWrite(\" \"),f--}else r.rawWrite(\"\")}hn(n,i,r,o,u,m),u=m}else r.write(n.substring(a,i))},e.hasModifiers=function(e){return 0!==xn(e)},e.hasModifier=En,e.hasStaticModifier=Tn,e.hasReadonlyModifier=Sn,e.getSelectedModifierFlags=Ln,e.getModifierFlags=xn,e.getModifierFlagsNoCache=An,e.modifierToFlag=Cn,e.isLogicalOperator=function(e){return 54===e||53===e||51===e},e.isAssignmentOperator=Dn,e.tryGetClassExtendingExpressionWithTypeArguments=kn,e.isAssignmentExpression=Mn,e.isDestructuringAssignment=function(e){if(Mn(e,!0)){var n=e.left.kind;return 186===n||185===n}return!1},e.isExpressionWithTypeArgumentsInClassExtendsClause=On,e.isExpressionWithTypeArgumentsInClassImplementsClause=function(n){return 209===n.kind&&In(n.expression)&&n.parent&&108===n.parent.token&&n.parent.parent&&e.isClassLike(n.parent.parent)},e.isEntityNameExpression=In,e.isPropertyAccessEntityNameExpression=Rn,e.isPrototypeAccess=Nn,e.isRightSideOfQualifiedNameOrPropertyAccess=function(e){return 146===e.parent.kind&&e.parent.right===e||187===e.parent.kind&&e.parent.name===e},e.isEmptyObjectLiteral=function(e){return 186===e.kind&&0===e.properties.length},e.isEmptyArrayLiteral=function(e){return 185===e.kind&&0===e.elements.length},e.getLocalSymbolForExportDefault=function(n){return function(n){return n&&e.length(n.declarations)>0&&En(n.declarations[0],512)}(n)?n.declarations[0].localSymbol:void 0},e.tryExtractTSExtension=function(n){return e.find(e.supportedTSExtensionsForExtractExtension,function(t){return e.fileExtensionIs(n,t)})};var wn=\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=\";function Pn(n){for(var t,r,a,i,o=\"\",s=function(n){for(var t=[],r=n.length,a=0;a<r;a++){var i=n.charCodeAt(a);i<128?t.push(i):i<2048?(t.push(i>>6|192),t.push(63&i|128)):i<65536?(t.push(i>>12|224),t.push(i>>6&63|128),t.push(63&i|128)):i<131072?(t.push(i>>18|240),t.push(i>>12&63|128),t.push(i>>6&63|128),t.push(63&i|128)):e.Debug.assert(!1,\"Unexpected code point\")}return t}(n),l=0,c=s.length;l<c;)t=s[l]>>2,r=(3&s[l])<<4|s[l+1]>>4,a=(15&s[l+1])<<2|s[l+2]>>6,i=63&s[l+2],l+1>=c?a=i=64:l+2>=c&&(i=64),o+=wn.charAt(t)+wn.charAt(r)+wn.charAt(a)+wn.charAt(i),l+=3;return o}e.convertToBase64=Pn,e.base64encode=function(e,n){return e&&e.base64encode?e.base64encode(n):Pn(n)},e.base64decode=function(e,n){if(e&&e.base64decode)return e.base64decode(n);for(var t=n.length,r=[],a=0;a<t&&n.charCodeAt(a)!==wn.charCodeAt(64);){var i=wn.indexOf(n[a]),o=wn.indexOf(n[a+1]),s=wn.indexOf(n[a+2]),l=wn.indexOf(n[a+3]),c=(63&i)<<2|o>>4&3,u=(15&o)<<4|s>>2&15,d=(3&s)<<6|63&l;0===u&&0!==s?r.push(c):0===d&&0!==l?r.push(c,u):r.push(c,u,d),a+=4}return function(e){for(var n=\"\",t=0,r=e.length;t<r;){var a=e[t];if(a<128)n+=String.fromCharCode(a),t++;else if(192==(192&a)){for(var i=63&a,o=e[++t];128==(192&o);)i=i<<6|63&o,o=e[++t];n+=String.fromCharCode(i)}else n+=String.fromCharCode(a),t++}return n}(r)},e.readJson=function(n,t){try{var r=t.readFile(n);if(!r)return{};var a=e.parseConfigFileTextToJson(n,r);return a.error?{}:a.config}catch(e){return{}}},e.directoryProbablyExists=function(e,n){return!n.directoryExists||n.directoryExists(e)};var Fn,Gn=\"\\r\\n\",Vn=\"\\n\";function Kn(n,t,r){void 0===n&&(n=0);var a=function(n){var t=[];for(var r in n){var a=n[r];\"number\"==typeof a&&t.push([a,r])}return e.stableSort(t,function(n,t){return e.compareValues(n[0],t[0])})}(t);if(0===n)return a.length>0&&0===a[0][0]?a[0][1]:\"0\";if(r){for(var i=\"\",o=n,s=a.length-1;s>=0&&0!==o;s--){var l=a[s],c=l[0],u=l[1];0!==c&&(o&c)===c&&(o&=~c,i=u+(i?\", \":\"\")+i)}if(0===o)return i}else for(var d=0,m=a;d<m.length;d++){var p=m[d];c=p[0],u=p[1];if(c===n)return u}return n.toString()}function Bn(n,t){return void 0===t&&(t=n),e.Debug.assert(t>=n||-1===t),{pos:n,end:t}}function Hn(e,n){return Bn(n,e.end)}function Un(e){return e.decorators&&e.decorators.length>0?Hn(e,e.decorators.end):e}function jn(e,n,t){return Wn(qn(e,t),n.end,t)}function Wn(e,n,t){return e===n||cn(t,e)===cn(t,n)}function qn(n,t){return e.positionIsSynthesized(n.pos)?-1:e.skipTrivia(t.text,n.pos)}function zn(e){return void 0!==e.initializer}function Jn(e){return 33554432&e.flags?e.checkFlags:0}function Xn(n){var t=n.parent;if(!t)return 0;switch(t.kind){case 193:return Xn(t);case 201:case 200:var r=t.operator;return 43===r||44===r?l():0;case 202:var a=t,i=a.left,o=a.operatorToken;return i===n&&Dn(o.kind)?58===o.kind?1:l():0;case 187:return t.name!==n?0:Xn(t);case 273:var s=Xn(t.parent);return n===t.name?function(n){switch(n){case 0:return 1;case 1:return 0;case 2:return 2;default:return e.Debug.assertNever(n)}}(s):s;case 274:return n===t.objectAssignmentInitializer?0:Xn(t.parent);case 185:return Xn(t);default:return 0}function l(){return t.parent&&219===function(e){for(;193===e.kind;)e=e.parent;return e}(t.parent).kind?1:2}}function Yn(n,t){for(;;){var r=t(n);if(void 0!==r)return r;var a=e.getDirectoryPath(n);if(a===n)return;n=a}}function Qn(e){if(32&e.flags){var n=Zn(e);return!!n&&En(n,128)}return!1}function Zn(n){return e.find(n.declarations,e.isClassLike)}function $n(e){return 131072&e.flags?e.objectFlags:0}e.getNewLineCharacter=function(n,t){switch(n.newLine){case 0:return Gn;case 1:return Vn}return t?t():e.sys?e.sys.newLine:Gn},e.formatSyntaxKind=function(n){return Kn(n,e.SyntaxKind,!1)},e.formatModifierFlags=function(n){return Kn(n,e.ModifierFlags,!0)},e.formatTransformFlags=function(n){return Kn(n,e.TransformFlags,!0)},e.formatEmitFlags=function(n){return Kn(n,e.EmitFlags,!0)},e.formatSymbolFlags=function(n){return Kn(n,e.SymbolFlags,!0)},e.formatTypeFlags=function(n){return Kn(n,e.TypeFlags,!0)},e.formatObjectFlags=function(n){return Kn(n,e.ObjectFlags,!0)},e.createRange=Bn,e.moveRangeEnd=function(e,n){return Bn(e.pos,n)},e.moveRangePos=Hn,e.moveRangePastDecorators=Un,e.moveRangePastModifiers=function(e){return e.modifiers&&e.modifiers.length>0?Hn(e,e.modifiers.end):Un(e)},e.isCollapsedRange=function(e){return e.pos===e.end},e.createTokenRange=function(n,t){return Bn(n,n+e.tokenToString(t).length)},e.rangeIsOnSingleLine=function(e,n){return jn(e,e,n)},e.rangeStartPositionsAreOnSameLine=function(e,n,t){return Wn(qn(e,t),qn(n,t),t)},e.rangeEndPositionsAreOnSameLine=function(e,n,t){return Wn(e.end,n.end,t)},e.rangeStartIsOnSameLineAsRangeEnd=jn,e.rangeEndIsOnSameLineAsRangeStart=function(e,n,t){return Wn(e.end,qn(n,t),t)},e.positionsAreOnSameLine=Wn,e.getStartPositionOfRange=qn,e.isDeclarationNameOfEnumOrNamespace=function(n){var t=e.getParseTreeNode(n);if(t)switch(t.parent.kind){case 241:case 242:return t===t.parent.name}return!1},e.getInitializedVariables=function(n){return e.filter(n.declarations,zn)},e.isWatchSet=function(e){return e.watch&&e.hasOwnProperty(\"watch\")},e.closeFileWatcher=function(e){e.close()},e.getCheckFlags=Jn,e.getDeclarationModifierFlagsFromSymbol=function(n){if(n.valueDeclaration){var t=e.getCombinedModifierFlags(n.valueDeclaration);return n.parent&&32&n.parent.flags?t:-29&t}if(6&Jn(n)){var r=n.checkFlags;return(256&r?8:64&r?4:16)|(512&r?32:0)}return 4194304&n.flags?36:0},e.skipAlias=function(e,n){return 2097152&e.flags?n.getAliasedSymbol(e):e},e.getCombinedLocalAndExportSymbolFlags=function(e){return e.exportSymbol?e.exportSymbol.flags|e.flags:e.flags},e.isWriteOnlyAccess=function(e){return 1===Xn(e)},e.isWriteAccess=function(e){return 0!==Xn(e)},function(e){e[e.Read=0]=\"Read\",e[e.Write=1]=\"Write\",e[e.ReadWrite=2]=\"ReadWrite\"}(Fn||(Fn={})),e.compareDataObjects=function e(n,t){if(!n||!t||Object.keys(n).length!==Object.keys(t).length)return!1;for(var r in n)if(\"object\"==typeof n[r]){if(!e(n[r],t[r]))return!1}else if(\"function\"!=typeof n[r]&&n[r]!==t[r])return!1;return!0},e.clearMap=function(e,n){e.forEach(n),e.clear()},e.mutateMap=function(e,n,t){var r=t.createNewValue,a=t.onDeleteValue,i=t.onExistingValue;e.forEach(function(t,r){var o=n.get(r);void 0===o?(e.delete(r),a(t,r)):i&&i(t,o,r)}),n.forEach(function(n,t){e.has(t)||e.set(t,r(t,n))})},e.forEachAncestorDirectory=Yn,e.isAbstractConstructorType=function(e){return!!(16&$n(e))&&!!e.symbol&&Qn(e.symbol)},e.isAbstractConstructorSymbol=Qn,e.getClassLikeDeclarationOfSymbol=Zn,e.getObjectFlags=$n,e.typeHasCallOrConstructSignatures=function(e,n){return 0!==n.getSignaturesOfType(e,0).length||0!==n.getSignaturesOfType(e,1).length},e.forSomeAncestorDirectory=function(e,n){return!!Yn(e,function(e){return!!n(e)||void 0})},e.isUMDExportSymbol=function(n){return!!n&&!!n.declarations&&!!n.declarations[0]&&e.isNamespaceExportDeclaration(n.declarations[0])},e.showModuleSpecifier=function(n){var t=n.moduleSpecifier;return e.isStringLiteral(t)?t.text:v(t)},e.getLastChild=function(n){var t;return e.forEachChild(n,function(e){m(e)&&(t=e)},function(e){for(var n=e.length-1;n>=0;n--)if(m(e[n])){t=e[n];break}}),t},e.addToSeen=function(e,n,t){return void 0===t&&(t=!0),n=String(n),!e.has(n)&&(e.set(n,t),!0)},e.isObjectTypeDeclaration=function(n){return e.isClassLike(n)||e.isInterfaceDeclaration(n)||e.isTypeLiteralNode(n)}}(Tn||(Tn={})),function(e){function n(e){return e.start+e.length}function t(e){return 0===e.length}function r(e,n){var t=i(e,n);return t&&0===t.length?void 0:t}function a(e,n,t,r){return t<=e+n&&t+r>=e}function i(e,t){var r=Math.max(e.start,t.start),a=Math.min(n(e),n(t));return r<=a?s(r,a):void 0}function o(e,n){if(e<0)throw new Error(\"start < 0\");if(n<0)throw new Error(\"length < 0\");return{start:e,length:n}}function s(e,n){return o(e,n-e)}function l(e,n){if(n<0)throw new Error(\"newLength < 0\");return{span:e,newLength:n}}function c(n){return!!e.isBindingPattern(n)&&e.every(n.elements,u)}function u(n){return!!e.isOmittedExpression(n)||c(n.name)}function d(n){for(var t=n.parent;e.isBindingElement(t.parent);)t=t.parent.parent;return t.parent}function m(n,t){e.isBindingElement(n)&&(n=d(n));var r=t(n);return 235===n.kind&&(n=n.parent),n&&236===n.kind&&(r|=t(n),n=n.parent),n&&217===n.kind&&(r|=t(n)),r}function p(e,n){if(e)for(;void 0!==e.original;)e=e.original;return!n||n(e)?e:void 0}function f(e){return 0==(8&e.flags)}function g(e){var n=e;return n.length>=3&&95===n.charCodeAt(0)&&95===n.charCodeAt(1)&&95===n.charCodeAt(2)?n.substr(1):n}function _(n){var t=h(n);return t&&e.isIdentifier(t)?t:void 0}function v(n){return n.name||function(n){var t=n.parent.parent;if(t){if(e.isDeclaration(t))return _(t);switch(t.kind){case 217:if(t.declarationList&&t.declarationList.declarations[0])return _(t.declarationList.declarations[0]);break;case 219:var r=t.expression;switch(r.kind){case 187:return r.name;case 188:var a=r.argumentExpression;if(e.isIdentifier(a))return a}break;case 193:return _(t.expression);case 231:if(e.isDeclaration(t.statement)||e.isExpression(t.statement))return _(t.statement)}}}(n)}function y(n){switch(n.kind){case 71:return n;case 303:case 297:var t=n.name;if(146===t.kind)return t.right;break;case 202:var r=n;switch(e.getAssignmentDeclarationKind(r)){case 1:case 4:case 5:case 3:return r.left.name;default:return}case 302:return v(n);case 252:var a=n.expression;return e.isIdentifier(a)?a:void 0}return n.name}function h(n){if(void 0!==n)return y(n)||(e.isFunctionExpression(n)||e.isClassExpression(n)?function(n){if(!n.parent)return;if(e.isPropertyAssignment(n.parent)||e.isBindingElement(n.parent))return n.parent.name;if(e.isBinaryExpression(n.parent)&&n===n.parent.right){if(e.isIdentifier(n.parent.left))return n.parent.left;if(e.isPropertyAccessExpression(n.parent.left))return n.parent.left.name}}(n):void 0)}function b(n){if(n.name){if(e.isIdentifier(n.name)){var t=n.name.escapedText;return L(n.parent).filter(function(n){return e.isJSDocParameterTag(n)&&e.isIdentifier(n.name)&&n.name.escapedText===t})}var r=n.parent.parameters.indexOf(n);e.Debug.assert(r>-1,\"Parameters should always be in their parents' parameter list\");var a=L(n.parent).filter(e.isJSDocParameterTag);if(r<a.length)return[a[r]]}return e.emptyArray}function E(n){return x(n,e.isJSDocReturnTag)}function T(n){var t=x(n,e.isJSDocTypeTag);if(t&&t.typeExpression&&t.typeExpression.type)return t}function S(n){var t=x(n,e.isJSDocTypeTag);return!t&&e.isParameter(n)&&(t=e.find(b(n),function(e){return!!e.typeExpression})),t&&t.typeExpression&&t.typeExpression.type}function L(n){var t=n.jsDocCache;if(void 0===t){var r=e.getJSDocCommentsAndTags(n);e.Debug.assert(r.length<2||r[0]!==r[1]),n.jsDocCache=t=e.flatMap(r,function(n){return e.isJSDoc(n)?n.tags:n})}return t}function x(n,t){return e.find(L(n),t)}e.getDefaultLibFileName=function(e){switch(e.target){case 6:return\"lib.esnext.full.d.ts\";case 5:return\"lib.es2018.full.d.ts\";case 4:return\"lib.es2017.full.d.ts\";case 3:return\"lib.es2016.full.d.ts\";case 2:return\"lib.es6.d.ts\";default:return\"lib.d.ts\"}},e.textSpanEnd=n,e.textSpanIsEmpty=t,e.textSpanContainsPosition=function(e,t){return t>=e.start&&t<n(e)},e.textRangeContainsPositionInclusive=function(e,n){return n>=e.pos&&n<=e.end},e.textSpanContainsTextSpan=function(e,t){return t.start>=e.start&&n(t)<=n(e)},e.textSpanOverlapsWith=function(e,n){return void 0!==r(e,n)},e.textSpanOverlap=r,e.textSpanIntersectsWithTextSpan=function(e,n){return a(e.start,e.length,n.start,n.length)},e.textSpanIntersectsWith=function(e,n,t){return a(e.start,e.length,n,t)},e.decodedTextSpanIntersectsWith=a,e.textSpanIntersectsWithPosition=function(e,t){return t<=n(e)&&t>=e.start},e.textSpanIntersection=i,e.createTextSpan=o,e.createTextSpanFromBounds=s,e.textChangeRangeNewSpan=function(e){return o(e.span.start,e.newLength)},e.textChangeRangeIsUnchanged=function(e){return t(e.span)&&0===e.newLength},e.createTextChangeRange=l,e.unchangedTextChangeRange=l(o(0,0),0),e.collapseTextChangeRangesAcrossMultipleVersions=function(t){if(0===t.length)return e.unchangedTextChangeRange;if(1===t.length)return t[0];for(var r=t[0],a=r.span.start,i=n(r.span),o=a+r.newLength,c=1;c<t.length;c++){var u=t[c],d=a,m=i,p=o,f=u.span.start,g=n(u.span),_=f+u.newLength;a=Math.min(d,f),i=Math.max(m,m+(g-p)),o=Math.max(_,_+(p-g))}return l(s(a,i),o-a)},e.getTypeParameterOwner=function(n){if(n&&148===n.kind)for(var t=n;t;t=t.parent)if(e.isFunctionLike(t)||e.isClassLike(t)||239===t.kind)return t},e.isParameterPropertyDeclaration=function(n){return e.hasModifier(n,92)&&155===n.parent.kind},e.isEmptyBindingPattern=c,e.isEmptyBindingElement=u,e.walkUpBindingElementsAndPatterns=d,e.getCombinedModifierFlags=function(n){return m(n,e.getModifierFlags)},e.getCombinedNodeFlags=function(e){return m(e,function(e){return e.flags})},e.validateLocaleAndSetLanguage=function(n,t,r){var a=/^([a-z]+)([_\\-]([a-z]+))?$/.exec(n.toLowerCase());if(a){var i=a[1];o(i,a[3],r)||o(i,void 0,r),e.setUILocale(n)}else r&&r.push(e.createCompilerDiagnostic(e.Diagnostics.Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1,\"en\",\"ja-jp\"));function o(n,r,a){var i=e.normalizePath(t.getExecutingFilePath()),o=e.getDirectoryPath(i),s=e.combinePaths(o,n);if(r&&(s=s+\"-\"+r),s=t.resolvePath(e.combinePaths(s,\"diagnosticMessages.generated.json\")),!t.fileExists(s))return!1;var l=\"\";try{l=t.readFile(s)}catch(n){return a&&a.push(e.createCompilerDiagnostic(e.Diagnostics.Unable_to_open_file_0,s)),!1}try{e.localizedDiagnosticMessages=JSON.parse(l)}catch(n){return a&&a.push(e.createCompilerDiagnostic(e.Diagnostics.Corrupted_locale_file_0,s)),!1}return!0}},e.getOriginalNode=p,e.isParseTreeNode=f,e.getParseTreeNode=function(e,n){return void 0===e||f(e)?e:!f(e=p(e))||n&&!n(e)?void 0:e},e.escapeLeadingUnderscores=function(e){return e.length>=2&&95===e.charCodeAt(0)&&95===e.charCodeAt(1)?\"_\"+e:e},e.unescapeLeadingUnderscores=g,e.idText=function(e){return g(e.escapedText)},e.symbolName=function(e){return g(e.escapedName)},e.getNameOfJSDocTypedef=v,e.isNamedDeclaration=function(e){return!!e.name},e.getNonAssignedNameOfDeclaration=y,e.getNameOfDeclaration=h,e.getJSDocParameterTags=b,e.getJSDocTypeParameterTags=function(n){var t=n.name.escapedText;return L(n.parent).filter(function(n){return e.isJSDocTemplateTag(n)&&n.typeParameters.some(function(e){return e.name.escapedText===t})})},e.hasJSDocParameterTags=function(n){return!!x(n,e.isJSDocParameterTag)},e.getJSDocAugmentsTag=function(n){return x(n,e.isJSDocAugmentsTag)},e.getJSDocClassTag=function(n){return x(n,e.isJSDocClassTag)},e.getJSDocEnumTag=function(n){return x(n,e.isJSDocEnumTag)},e.getJSDocThisTag=function(n){return x(n,e.isJSDocThisTag)},e.getJSDocReturnTag=E,e.getJSDocTemplateTag=function(n){return x(n,e.isJSDocTemplateTag)},e.getJSDocTypeTag=T,e.getJSDocType=S,e.getJSDocReturnType=function(n){var t=E(n);if(t&&t.typeExpression)return t.typeExpression.type;var r=T(n);if(r&&r.typeExpression){var a=r.typeExpression.type;if(e.isTypeLiteralNode(a)){var i=e.find(a.members,e.isCallSignatureDeclaration);return i&&i.type}if(e.isFunctionTypeNode(a))return a.type}},e.getJSDocTags=L,e.getAllJSDocTagsOfKind=function(e,n){return L(e).filter(function(e){return e.kind===n})},e.getEffectiveTypeParameterDeclarations=function(n){if(e.isJSDocSignature(n))return e.emptyArray;if(e.isJSDocTypeAlias(n))return e.Debug.assert(289===n.parent.kind),e.flatMap(n.parent.tags,function(n){return e.isJSDocTemplateTag(n)?n.typeParameters:void 0});if(n.typeParameters)return n.typeParameters;if(e.isInJSFile(n)){var t=e.getJSDocTypeParameterDeclarations(n);if(t.length)return t;var r=S(n);if(r&&e.isFunctionTypeNode(r)&&r.typeParameters)return r.typeParameters}return e.emptyArray},e.getEffectiveConstraintOfTypeParameter=function(n){return n.constraint?n.constraint:e.isJSDocTemplateTag(n.parent)&&n===n.parent.typeParameters[0]?n.parent.constraint:void 0}}(Tn||(Tn={})),function(e){e.isNumericLiteral=function(e){return 8===e.kind},e.isStringLiteral=function(e){return 9===e.kind},e.isJsxText=function(e){return 10===e.kind},e.isRegularExpressionLiteral=function(e){return 12===e.kind},e.isNoSubstitutionTemplateLiteral=function(e){return 13===e.kind},e.isTemplateHead=function(e){return 14===e.kind},e.isTemplateMiddle=function(e){return 15===e.kind},e.isTemplateTail=function(e){return 16===e.kind},e.isIdentifier=function(e){return 71===e.kind},e.isQualifiedName=function(e){return 146===e.kind},e.isComputedPropertyName=function(e){return 147===e.kind},e.isTypeParameterDeclaration=function(e){return 148===e.kind},e.isParameter=function(e){return 149===e.kind},e.isDecorator=function(e){return 150===e.kind},e.isPropertySignature=function(e){return 151===e.kind},e.isPropertyDeclaration=function(e){return 152===e.kind},e.isMethodSignature=function(e){return 153===e.kind},e.isMethodDeclaration=function(e){return 154===e.kind},e.isConstructorDeclaration=function(e){return 155===e.kind},e.isGetAccessorDeclaration=function(e){return 156===e.kind},e.isSetAccessorDeclaration=function(e){return 157===e.kind},e.isCallSignatureDeclaration=function(e){return 158===e.kind},e.isConstructSignatureDeclaration=function(e){return 159===e.kind},e.isIndexSignatureDeclaration=function(e){return 160===e.kind},e.isGetOrSetAccessorDeclaration=function(e){return 157===e.kind||156===e.kind},e.isTypePredicateNode=function(e){return 161===e.kind},e.isTypeReferenceNode=function(e){return 162===e.kind},e.isFunctionTypeNode=function(e){return 163===e.kind},e.isConstructorTypeNode=function(e){return 164===e.kind},e.isTypeQueryNode=function(e){return 165===e.kind},e.isTypeLiteralNode=function(e){return 166===e.kind},e.isArrayTypeNode=function(e){return 167===e.kind},e.isTupleTypeNode=function(e){return 168===e.kind},e.isUnionTypeNode=function(e){return 171===e.kind},e.isIntersectionTypeNode=function(e){return 172===e.kind},e.isConditionalTypeNode=function(e){return 173===e.kind},e.isInferTypeNode=function(e){return 174===e.kind},e.isParenthesizedTypeNode=function(e){return 175===e.kind},e.isThisTypeNode=function(e){return 176===e.kind},e.isTypeOperatorNode=function(e){return 177===e.kind},e.isIndexedAccessTypeNode=function(e){return 178===e.kind},e.isMappedTypeNode=function(e){return 179===e.kind},e.isLiteralTypeNode=function(e){return 180===e.kind},e.isImportTypeNode=function(e){return 181===e.kind},e.isObjectBindingPattern=function(e){return 182===e.kind},e.isArrayBindingPattern=function(e){return 183===e.kind},e.isBindingElement=function(e){return 184===e.kind},e.isArrayLiteralExpression=function(e){return 185===e.kind},e.isObjectLiteralExpression=function(e){return 186===e.kind},e.isPropertyAccessExpression=function(e){return 187===e.kind},e.isElementAccessExpression=function(e){return 188===e.kind},e.isCallExpression=function(e){return 189===e.kind},e.isNewExpression=function(e){return 190===e.kind},e.isTaggedTemplateExpression=function(e){return 191===e.kind},e.isTypeAssertion=function(e){return 192===e.kind},e.isParenthesizedExpression=function(e){return 193===e.kind},e.skipPartiallyEmittedExpressions=function(e){for(;306===e.kind;)e=e.expression;return e},e.isFunctionExpression=function(e){return 194===e.kind},e.isArrowFunction=function(e){return 195===e.kind},e.isDeleteExpression=function(e){return 196===e.kind},e.isTypeOfExpression=function(e){return 197===e.kind},e.isVoidExpression=function(e){return 198===e.kind},e.isAwaitExpression=function(e){return 199===e.kind},e.isPrefixUnaryExpression=function(e){return 200===e.kind},e.isPostfixUnaryExpression=function(e){return 201===e.kind},e.isBinaryExpression=function(e){return 202===e.kind},e.isConditionalExpression=function(e){return 203===e.kind},e.isTemplateExpression=function(e){return 204===e.kind},e.isYieldExpression=function(e){return 205===e.kind},e.isSpreadElement=function(e){return 206===e.kind},e.isClassExpression=function(e){return 207===e.kind},e.isOmittedExpression=function(e){return 208===e.kind},e.isExpressionWithTypeArguments=function(e){return 209===e.kind},e.isAsExpression=function(e){return 210===e.kind},e.isNonNullExpression=function(e){return 211===e.kind},e.isMetaProperty=function(e){return 212===e.kind},e.isTemplateSpan=function(e){return 214===e.kind},e.isSemicolonClassElement=function(e){return 215===e.kind},e.isBlock=function(e){return 216===e.kind},e.isVariableStatement=function(e){return 217===e.kind},e.isEmptyStatement=function(e){return 218===e.kind},e.isExpressionStatement=function(e){return 219===e.kind},e.isIfStatement=function(e){return 220===e.kind},e.isDoStatement=function(e){return 221===e.kind},e.isWhileStatement=function(e){return 222===e.kind},e.isForStatement=function(e){return 223===e.kind},e.isForInStatement=function(e){return 224===e.kind},e.isForOfStatement=function(e){return 225===e.kind},e.isContinueStatement=function(e){return 226===e.kind},e.isBreakStatement=function(e){return 227===e.kind},e.isBreakOrContinueStatement=function(e){return 227===e.kind||226===e.kind},e.isReturnStatement=function(e){return 228===e.kind},e.isWithStatement=function(e){return 229===e.kind},e.isSwitchStatement=function(e){return 230===e.kind},e.isLabeledStatement=function(e){return 231===e.kind},e.isThrowStatement=function(e){return 232===e.kind},e.isTryStatement=function(e){return 233===e.kind},e.isDebuggerStatement=function(e){return 234===e.kind},e.isVariableDeclaration=function(e){return 235===e.kind},e.isVariableDeclarationList=function(e){return 236===e.kind},e.isFunctionDeclaration=function(e){return 237===e.kind},e.isClassDeclaration=function(e){return 238===e.kind},e.isInterfaceDeclaration=function(e){return 239===e.kind},e.isTypeAliasDeclaration=function(e){return 240===e.kind},e.isEnumDeclaration=function(e){return 241===e.kind},e.isModuleDeclaration=function(e){return 242===e.kind},e.isModuleBlock=function(e){return 243===e.kind},e.isCaseBlock=function(e){return 244===e.kind},e.isNamespaceExportDeclaration=function(e){return 245===e.kind},e.isImportEqualsDeclaration=function(e){return 246===e.kind},e.isImportDeclaration=function(e){return 247===e.kind},e.isImportClause=function(e){return 248===e.kind},e.isNamespaceImport=function(e){return 249===e.kind},e.isNamedImports=function(e){return 250===e.kind},e.isImportSpecifier=function(e){return 251===e.kind},e.isExportAssignment=function(e){return 252===e.kind},e.isExportDeclaration=function(e){return 253===e.kind},e.isNamedExports=function(e){return 254===e.kind},e.isExportSpecifier=function(e){return 255===e.kind},e.isMissingDeclaration=function(e){return 256===e.kind},e.isExternalModuleReference=function(e){return 257===e.kind},e.isJsxElement=function(e){return 258===e.kind},e.isJsxSelfClosingElement=function(e){return 259===e.kind},e.isJsxOpeningElement=function(e){return 260===e.kind},e.isJsxClosingElement=function(e){return 261===e.kind},e.isJsxFragment=function(e){return 262===e.kind},e.isJsxOpeningFragment=function(e){return 263===e.kind},e.isJsxClosingFragment=function(e){return 264===e.kind},e.isJsxAttribute=function(e){return 265===e.kind},e.isJsxAttributes=function(e){return 266===e.kind},e.isJsxSpreadAttribute=function(e){return 267===e.kind},e.isJsxExpression=function(e){return 268===e.kind},e.isCaseClause=function(e){return 269===e.kind},e.isDefaultClause=function(e){return 270===e.kind},e.isHeritageClause=function(e){return 271===e.kind},e.isCatchClause=function(e){return 272===e.kind},e.isPropertyAssignment=function(e){return 273===e.kind},e.isShorthandPropertyAssignment=function(e){return 274===e.kind},e.isSpreadAssignment=function(e){return 275===e.kind},e.isEnumMember=function(e){return 276===e.kind},e.isSourceFile=function(e){return 277===e.kind},e.isBundle=function(e){return 278===e.kind},e.isUnparsedSource=function(e){return 279===e.kind},e.isJSDocTypeExpression=function(e){return 281===e.kind},e.isJSDocAllType=function(e){return 282===e.kind},e.isJSDocUnknownType=function(e){return 283===e.kind},e.isJSDocNullableType=function(e){return 284===e.kind},e.isJSDocNonNullableType=function(e){return 285===e.kind},e.isJSDocOptionalType=function(e){return 286===e.kind},e.isJSDocFunctionType=function(e){return 287===e.kind},e.isJSDocVariadicType=function(e){return 288===e.kind},e.isJSDoc=function(e){return 289===e.kind},e.isJSDocAugmentsTag=function(e){return 293===e.kind},e.isJSDocClassTag=function(e){return 294===e.kind},e.isJSDocEnumTag=function(e){return 296===e.kind},e.isJSDocThisTag=function(e){return 299===e.kind},e.isJSDocParameterTag=function(e){return 297===e.kind},e.isJSDocReturnTag=function(e){return 298===e.kind},e.isJSDocTypeTag=function(e){return 300===e.kind},e.isJSDocTemplateTag=function(e){return 301===e.kind},e.isJSDocTypedefTag=function(e){return 302===e.kind},e.isJSDocPropertyTag=function(e){return 303===e.kind},e.isJSDocPropertyLikeTag=function(e){return 303===e.kind||297===e.kind},e.isJSDocTypeLiteral=function(e){return 290===e.kind},e.isJSDocCallbackTag=function(e){return 295===e.kind},e.isJSDocSignature=function(e){return 291===e.kind}}(Tn||(Tn={})),function(e){function n(e){return e>=146}function t(e){return 8<=e&&e<=13}function r(e){return 13<=e&&e<=16}function a(e){switch(e){case 117:case 120:case 76:case 124:case 79:case 84:case 114:case 112:case 113:case 132:case 115:return!0}return!1}function i(n){return!!(92&e.modifierToFlag(n))}function o(e){return e&&l(e.kind)}function s(e){switch(e){case 237:case 154:case 155:case 156:case 157:case 194:case 195:return!0;default:return!1}}function l(e){switch(e){case 153:case 158:case 291:case 159:case 160:case 163:case 287:case 164:return!0;default:return s(e)}}function c(e){var n=e.kind;return 155===n||152===n||154===n||156===n||157===n||160===n||215===n}function u(e){var n=e.kind;return 159===n||158===n||151===n||153===n||160===n}function d(e){var n=e.kind;return 273===n||274===n||275===n||154===n||156===n||157===n}function m(e){switch(e.kind){case 182:case 186:return!0}return!1}function p(e){switch(e.kind){case 183:case 185:return!0}return!1}function f(e){switch(e){case 187:case 188:case 190:case 189:case 258:case 259:case 262:case 191:case 185:case 193:case 186:case 207:case 194:case 71:case 12:case 8:case 9:case 13:case 204:case 86:case 95:case 99:case 101:case 97:case 211:case 212:case 91:return!0;default:return!1}}function g(e){switch(e){case 200:case 201:case 196:case 197:case 198:case 199:case 192:return!0;default:return f(e)}}function _(n){return function(e){switch(e){case 203:case 205:case 195:case 202:case 206:case 210:case 208:case 307:case 306:return!0;default:return g(e)}}(e.skipPartiallyEmittedExpressions(n).kind)}function v(e){return 306===e.kind}function y(e){return 305===e.kind}function h(e){return 237===e||256===e||238===e||239===e||240===e||241===e||242===e||247===e||246===e||253===e||252===e||245===e}function b(e){return 227===e||226===e||234===e||221===e||219===e||218===e||224===e||225===e||223===e||220===e||231===e||228===e||230===e||232===e||233===e||217===e||222===e||229===e||305===e||309===e||308===e}function E(e){return e.kind>=292&&e.kind<=303}function T(e){return!!e.initializer}e.isSyntaxList=function(e){return 304===e.kind},e.isNode=function(e){return n(e.kind)},e.isNodeKind=n,e.isToken=function(e){return e.kind>=0&&e.kind<=145},e.isNodeArray=function(e){return e.hasOwnProperty(\"pos\")&&e.hasOwnProperty(\"end\")},e.isLiteralKind=t,e.isLiteralExpression=function(e){return t(e.kind)},e.isTemplateLiteralKind=r,e.isTemplateLiteralToken=function(e){return r(e.kind)},e.isTemplateMiddleOrTemplateTail=function(e){var n=e.kind;return 15===n||16===n},e.isStringTextContainingNode=function(e){return 9===e.kind||r(e.kind)},e.isGeneratedIdentifier=function(n){return e.isIdentifier(n)&&(7&n.autoGenerateFlags)>0},e.isModifierKind=a,e.isParameterPropertyModifier=i,e.isClassMemberModifier=function(e){return i(e)||115===e},e.isModifier=function(e){return a(e.kind)},e.isEntityName=function(e){var n=e.kind;return 146===n||71===n},e.isPropertyName=function(e){var n=e.kind;return 71===n||9===n||8===n||147===n},e.isBindingName=function(e){var n=e.kind;return 71===n||182===n||183===n},e.isFunctionLike=o,e.isFunctionLikeDeclaration=function(e){return e&&s(e.kind)},e.isFunctionLikeKind=l,e.isFunctionOrModuleBlock=function(n){return e.isSourceFile(n)||e.isModuleBlock(n)||e.isBlock(n)&&o(n.parent)},e.isClassElement=c,e.isClassLike=function(e){return e&&(238===e.kind||207===e.kind)},e.isAccessor=function(e){return e&&(156===e.kind||157===e.kind)},e.isMethodOrAccessor=function(e){switch(e.kind){case 154:case 156:case 157:return!0;default:return!1}},e.isTypeElement=u,e.isClassOrTypeElement=function(e){return u(e)||c(e)},e.isObjectLiteralElementLike=d,e.isTypeNode=function(e){return(n=e.kind)>=161&&n<=181||119===n||142===n||134===n||135===n||122===n||137===n||138===n||99===n||105===n||140===n||95===n||131===n||209===n||282===n||283===n||284===n||285===n||286===n||287===n||288===n;var n},e.isFunctionOrConstructorTypeNode=function(e){switch(e.kind){case 163:case 164:return!0}return!1},e.isBindingPattern=function(e){if(e){var n=e.kind;return 183===n||182===n}return!1},e.isAssignmentPattern=function(e){var n=e.kind;return 185===n||186===n},e.isArrayBindingElement=function(e){var n=e.kind;return 184===n||208===n},e.isDeclarationBindingElement=function(e){switch(e.kind){case 235:case 149:case 184:return!0}return!1},e.isBindingOrAssignmentPattern=function(e){return m(e)||p(e)},e.isObjectBindingOrAssignmentPattern=m,e.isArrayBindingOrAssignmentPattern=p,e.isPropertyAccessOrQualifiedNameOrImportTypeNode=function(e){var n=e.kind;return 187===n||146===n||181===n},e.isPropertyAccessOrQualifiedName=function(e){var n=e.kind;return 187===n||146===n},e.isCallLikeExpression=function(e){switch(e.kind){case 260:case 259:case 189:case 190:case 191:case 150:return!0;default:return!1}},e.isCallOrNewExpression=function(e){return 189===e.kind||190===e.kind},e.isTemplateLiteral=function(e){var n=e.kind;return 204===n||13===n},e.isLeftHandSideExpression=function(n){return f(e.skipPartiallyEmittedExpressions(n).kind)},e.isUnaryExpression=function(n){return g(e.skipPartiallyEmittedExpressions(n).kind)},e.isUnaryExpressionWithWrite=function(e){switch(e.kind){case 201:return!0;case 200:return 43===e.operator||44===e.operator;default:return!1}},e.isExpression=_,e.isAssertionExpression=function(e){var n=e.kind;return 192===n||210===n},e.isPartiallyEmittedExpression=v,e.isNotEmittedStatement=y,e.isNotEmittedOrPartiallyEmittedNode=function(e){return y(e)||v(e)},e.isIterationStatement=function e(n,t){switch(n.kind){case 223:case 224:case 225:case 221:case 222:return!0;case 231:return t&&e(n.statement,t)}return!1},e.isForInOrOfStatement=function(e){return 224===e.kind||225===e.kind},e.isConciseBody=function(n){return e.isBlock(n)||_(n)},e.isFunctionBody=function(n){return e.isBlock(n)},e.isForInitializer=function(n){return e.isVariableDeclarationList(n)||_(n)},e.isModuleBody=function(e){var n=e.kind;return 243===n||242===n||71===n},e.isNamespaceBody=function(e){var n=e.kind;return 243===n||242===n},e.isJSDocNamespaceBody=function(e){var n=e.kind;return 71===n||242===n},e.isNamedImportBindings=function(e){var n=e.kind;return 250===n||249===n},e.isModuleOrEnumDeclaration=function(e){return 242===e.kind||241===e.kind},e.isDeclaration=function(n){return 148===n.kind?301!==n.parent.kind||e.isInJSFile(n):195===(t=n.kind)||184===t||238===t||207===t||155===t||241===t||276===t||255===t||237===t||194===t||156===t||248===t||246===t||251===t||239===t||265===t||154===t||153===t||242===t||245===t||249===t||149===t||273===t||152===t||151===t||157===t||274===t||240===t||148===t||235===t||302===t||295===t||303===t;var t},e.isDeclarationStatement=function(e){return h(e.kind)},e.isStatementButNotDeclaration=function(e){return b(e.kind)},e.isStatement=function(n){var t=n.kind;return b(t)||h(t)||function(n){return 216===n.kind&&((void 0===n.parent||233!==n.parent.kind&&272!==n.parent.kind)&&!e.isFunctionBlock(n))}(n)},e.isModuleReference=function(e){var n=e.kind;return 257===n||146===n||71===n},e.isJsxTagNameExpression=function(e){var n=e.kind;return 99===n||71===n||187===n},e.isJsxChild=function(e){var n=e.kind;return 258===n||268===n||259===n||10===n||262===n},e.isJsxAttributeLike=function(e){var n=e.kind;return 265===n||267===n},e.isStringLiteralOrJsxExpression=function(e){var n=e.kind;return 9===n||268===n},e.isJsxOpeningLikeElement=function(e){var n=e.kind;return 260===n||259===n},e.isCaseOrDefaultClause=function(e){var n=e.kind;return 269===n||270===n},e.isJSDocNode=function(e){return e.kind>=281&&e.kind<=303},e.isJSDocCommentContainingNode=function(n){return 289===n.kind||E(n)||e.isJSDocTypeLiteral(n)||e.isJSDocSignature(n)},e.isJSDocTag=E,e.isSetAccessor=function(e){return 157===e.kind},e.isGetAccessor=function(e){return 156===e.kind},e.hasJSDocNodes=function(e){var n=e.jsDoc;return!!n&&n.length>0},e.hasType=function(e){return!!e.type},e.hasInitializer=T,e.hasOnlyExpressionInitializer=function(n){return T(n)&&!e.isForStatement(n)&&!e.isForInStatement(n)&&!e.isForOfStatement(n)&&!e.isJsxAttribute(n)},e.isObjectLiteralElement=function(e){return 265===e.kind||267===e.kind||d(e)},e.isTypeReferenceType=function(e){return 162===e.kind||209===e.kind};var S=1073741823;e.guessIndentation=function(n){for(var t=S,r=0,a=n;r<a.length;r++){var i=a[r];if(i.length){for(var o=0;o<i.length&&o<t&&e.isWhiteSpaceLike(i.charCodeAt(o));o++);if(o<t&&(t=o),0===t)return 0}}return t===S?void 0:t},e.isStringLiteralLike=function(e){return 9===e.kind||13===e.kind}}(Tn||(Tn={})),function(e){function n(e,n){this.flags=e,this.escapedName=n,this.declarations=void 0,this.valueDeclaration=void 0,this.id=void 0,this.mergeId=void 0,this.parent=void 0}function t(n,t){this.flags=t,e.Debug.isDebugging&&(this.checker=n)}function r(){}function a(e,n,t){this.pos=n,this.end=t,this.kind=e,this.id=0,this.flags=0,this.modifierFlagsCache=0,this.transformFlags=0,this.parent=void 0,this.original=void 0}function i(e,n,t){this.fileName=e,this.text=n,this.skipTrivia=t||function(e){return e}}function o(n,t,r){return void 0===r&&(r=0),n.replace(/{(\\d+)}/g,function(n,a){return e.Debug.assertDefined(t[+a+r])})}function s(n){return e.localizedDiagnosticMessages&&e.localizedDiagnosticMessages[n.key]||n.message}function l(e){return e.file?e.file.path:void 0}function c(n,t){return u(n,t)||function(n,t){if(!n.relatedInformation&&!t.relatedInformation)return 0;if(n.relatedInformation&&t.relatedInformation)return e.compareValues(n.relatedInformation.length,t.relatedInformation.length)||e.forEach(n.relatedInformation,function(e,n){var r=t.relatedInformation[n];return c(e,r)})||0;return n.relatedInformation?-1:1}(n,t)||0}function u(n,t){return e.compareStringsCaseSensitive(l(n),l(t))||e.compareValues(n.start,t.start)||e.compareValues(n.length,t.length)||e.compareValues(n.code,t.code)||function(n,t){var r=n,a=t;for(;r&&a;){var i=e.isString(r)?r:r.messageText,o=e.isString(a)?a:a.messageText,s=e.compareStringsCaseSensitive(i,o);if(s)return s;r=e.isString(r)?void 0:r.next,a=e.isString(a)?void 0:a.next}if(!r&&!a)return 0;return r?1:-1}(n.messageText,t.messageText)||0}function d(e){return e.target||0}function m(n){return\"number\"==typeof n.module?n.module:d(n)>=2?e.ModuleKind.ES2015:e.ModuleKind.CommonJS}function p(e){return!(!e.declaration&&!e.composite)}function f(e,n){return void 0===e[n]?!!e.strict:!!e[n]}function g(e,n){return n.strictFlag?f(e,n.name):e[n.name]}e.isNamedImportsOrExports=function(e){return 250===e.kind||254===e.kind},e.objectAllocator={getNodeConstructor:function(){return a},getTokenConstructor:function(){return a},getIdentifierConstructor:function(){return a},getSourceFileConstructor:function(){return a},getSymbolConstructor:function(){return n},getTypeConstructor:function(){return t},getSignatureConstructor:function(){return r},getSourceMapSourceConstructor:function(){return i}},e.formatStringFromArgs=o,e.getLocaleSpecificMessage=s,e.createFileDiagnostic=function(n,t,r,a){e.Debug.assertGreaterThanOrEqual(t,0),e.Debug.assertGreaterThanOrEqual(r,0),n&&(e.Debug.assertLessThanOrEqual(t,n.text.length),e.Debug.assertLessThanOrEqual(t+r,n.text.length));var i=s(a);return arguments.length>4&&(i=o(i,arguments,4)),{file:n,start:t,length:r,messageText:i,category:a.category,code:a.code,reportsUnnecessary:a.reportsUnnecessary}},e.formatMessage=function(e,n){var t=s(n);return arguments.length>2&&(t=o(t,arguments,2)),t},e.createCompilerDiagnostic=function(e){var n=s(e);return arguments.length>1&&(n=o(n,arguments,1)),{file:void 0,start:void 0,length:void 0,messageText:n,category:e.category,code:e.code,reportsUnnecessary:e.reportsUnnecessary}},e.createCompilerDiagnosticFromMessageChain=function(e){return{file:void 0,start:void 0,length:void 0,code:e.code,category:e.category,messageText:e.next?e:e.messageText}},e.chainDiagnosticMessages=function(e,n){var t=s(n);return arguments.length>2&&(t=o(t,arguments,2)),{messageText:t,category:n.category,code:n.code,next:e}},e.concatenateDiagnosticMessageChains=function(e,n){for(var t=e;t.next;)t=t.next;return t.next=n,e},e.compareDiagnostics=c,e.compareDiagnosticsSkipRelatedInformation=u,e.getEmitScriptTarget=d,e.getEmitModuleKind=m,e.getEmitModuleResolutionKind=function(n){var t=n.moduleResolution;return void 0===t&&(t=m(n)===e.ModuleKind.CommonJS?e.ModuleResolutionKind.NodeJs:e.ModuleResolutionKind.Classic),t},e.hasJsonModuleEmitEnabled=function(n){switch(m(n)){case e.ModuleKind.CommonJS:case e.ModuleKind.AMD:case e.ModuleKind.ES2015:case e.ModuleKind.ESNext:return!0;default:return!1}},e.unreachableCodeIsError=function(e){return!1===e.allowUnreachableCode},e.unusedLabelIsError=function(e){return!1===e.allowUnusedLabels},e.getAreDeclarationMapsEnabled=function(e){return!(!p(e)||!e.declarationMap)},e.getAllowSyntheticDefaultImports=function(n){var t=m(n);return void 0!==n.allowSyntheticDefaultImports?n.allowSyntheticDefaultImports:n.esModuleInterop||t===e.ModuleKind.System},e.getEmitDeclarations=p,e.getStrictOptionValue=f,e.compilerOptionsAffectSemanticDiagnostics=function(n,t){return t!==n&&e.semanticDiagnosticsOptionDeclarations.some(function(r){return!e.isJsonEqual(g(t,r),g(n,r))})},e.getCompilerOptionValue=g,e.hasZeroOrOneAsteriskCharacter=function(e){for(var n=!1,t=0;t<e.length;t++)if(42===e.charCodeAt(t)){if(n)return!1;n=!0}return!0},e.directorySeparator=\"/\";var _=\"\\\\\",v=\"://\",y=/\\\\/g;function h(n){return n.replace(y,e.directorySeparator)}function b(e){return e>=97&&e<=122||e>=65&&e<=90}function E(n){if(!n)return 0;var t=n.charCodeAt(0);if(47===t||92===t){if(n.charCodeAt(1)!==t)return 1;var r=n.indexOf(47===t?e.directorySeparator:_,2);return r<0?n.length:r+1}if(b(t)&&58===n.charCodeAt(1)){var a=n.charCodeAt(2);if(47===a||92===a)return 3;if(2===n.length)return 2}var i=n.indexOf(v);if(-1!==i){var o=i+v.length,s=n.indexOf(e.directorySeparator,o);if(-1!==s){var l=n.slice(0,i),c=n.slice(o,s);if(\"file\"===l&&(\"\"===c||\"localhost\"===c)&&b(n.charCodeAt(s+1))){var u=function(e,n){var t=e.charCodeAt(n);if(58===t)return n+1;if(37===t&&51===e.charCodeAt(n+1)){var r=e.charCodeAt(n+2);if(97===r||65===r)return n+3}return-1}(n,s+2);if(-1!==u){if(47===n.charCodeAt(u))return~(u+1);if(u===n.length)return~u}}return~(s+1)}return~n.length}return 0}function T(e){var n=E(e);return n<0?~n:n}function S(e){return E(e)>0}function L(n,t){return void 0===t&&(t=\"\"),function(n,t){var r=n.substring(0,t),a=n.substring(t).split(e.directorySeparator);return a.length&&!e.lastOrUndefined(a)&&a.pop(),[r].concat(a)}(n=e.combinePaths(t,n),T(n))}function x(n){if(!e.some(n))return[];for(var t=[n[0]],r=1;r<n.length;r++){var a=n[r];if(a&&\".\"!==a){if(\"..\"===a)if(t.length>1){if(\"..\"!==t[t.length-1]){t.pop();continue}}else if(t[0])continue;t.push(a)}}return t}function A(e,n){return x(L(e,n))}function C(n){return 0===n.length?\"\":(n[0]&&e.ensureTrailingDirectorySeparator(n[0]))+n.slice(1).join(e.directorySeparator)}e.normalizeSlashes=h,e.getRootLength=T,e.normalizePath=function(n){return e.resolvePath(n)},e.normalizePathAndParts=function(n){var t=x(L(n=h(n))),r=t[0],a=t.slice(1);if(a.length){var i=r+a.join(e.directorySeparator);return{path:e.hasTrailingDirectorySeparator(n)?e.ensureTrailingDirectorySeparator(i):i,parts:a}}return{path:r,parts:a}},e.getDirectoryPath=function(n){var t=T(n=h(n));return t===n.length?n:(n=e.removeTrailingDirectorySeparator(n)).slice(0,Math.max(t,n.lastIndexOf(e.directorySeparator)))},e.startsWithDirectory=function(n,t,r){var a=r(n),i=r(t);return e.startsWith(a,i+\"/\")||e.startsWith(a,i+\"\\\\\")},e.isUrl=function(e){return E(e)<0},e.pathIsRelative=function(e){return/^\\.\\.?($|[\\\\\\/])/.test(e)},e.isRootedDiskPath=S,e.isDiskPathRoot=function(e){var n=E(e);return n>0&&n===e.length},e.convertToRelativePath=function(n,t,r){return S(n)?e.getRelativePathToDirectoryOrUrl(t,n,t,r,!1):n},e.getPathComponents=L,e.reducePathComponents=x,e.getNormalizedPathComponents=A,e.getNormalizedAbsolutePath=function(e,n){return C(A(e,n))},e.getPathFromPathComponents=C}(Tn||(Tn={})),function(e){function n(n,t,r,a){var i,o=e.reducePathComponents(e.getPathComponents(n)),s=e.reducePathComponents(e.getPathComponents(t));for(i=0;i<o.length&&i<s.length;i++){var l=a(o[i]),c=a(s[i]);if(!(0===i?e.equateStringsCaseInsensitive:r)(l,c))break}if(0===i)return s;for(var u=s.slice(i),d=[];i<o.length;i++)d.push(\"..\");return[\"\"].concat(d,u)}function t(t,r,a){I.assert(e.getRootLength(t)>0==e.getRootLength(r)>0,\"Paths must either both be absolute or both be relative\");var i=\"function\"==typeof a?a:e.identity,o=n(t,r,\"boolean\"==typeof a&&a?e.equateStringsCaseInsensitive:e.equateStringsCaseSensitive,i);return e.getPathFromPathComponents(o)}function r(n){return 0!==e.getRootLength(n)||e.pathIsRelative(n)?n:\"./\"+n}function a(n,t,r){if(n=e.normalizeSlashes(n),e.getRootLength(n)===n.length)return\"\";var a=(n=l(n)).slice(Math.max(e.getRootLength(n),n.lastIndexOf(e.directorySeparator)+1)),i=void 0!==t&&void 0!==r?K(a,t,r):void 0;return i?a.slice(0,a.length-i.length):a}function i(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];n&&(n=e.normalizeSlashes(n));for(var a=0,i=t;a<i.length;a++){var o=i[a];o&&(o=e.normalizeSlashes(o),n=n&&0===e.getRootLength(o)?c(n)+o:o)}return n}function o(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];var a=e.some(t)?i.apply(void 0,[n].concat(t)):e.normalizeSlashes(n),o=e.getPathFromPathComponents(e.reducePathComponents(e.getPathComponents(a)));return o&&s(a)?c(o):o}function s(e){if(0===e.length)return!1;var n=e.charCodeAt(e.length-1);return 47===n||92===n}function l(e){return s(e)?e.substr(0,e.length-1):e}function c(n){return s(n)?n:n+e.directorySeparator}function u(n,t,r){if(n===t)return 0;if(void 0===n)return-1;if(void 0===t)return 1;for(var a=e.reducePathComponents(e.getPathComponents(n)),i=e.reducePathComponents(e.getPathComponents(t)),o=Math.min(a.length,i.length),s=0;s<o;s++){var l=(0===s?e.compareStringsCaseInsensitive:r)(a[s],i[s]);if(0!==l)return l}return e.compareValues(a.length,i.length)}function d(n,t,r,a){if(\"string\"==typeof r?(n=i(r,n),t=i(r,t)):\"boolean\"==typeof r&&(a=r),void 0===n||void 0===t)return!1;if(n===t)return!0;var o=e.reducePathComponents(e.getPathComponents(n)),s=e.reducePathComponents(e.getPathComponents(t));if(s.length<o.length)return!1;for(var l=a?e.equateStringsCaseInsensitive:e.equateStringsCaseSensitive,c=0;c<o.length;c++){if(!(0===c?e.equateStringsCaseInsensitive:l)(o[c],s[c]))return!1}return!0}e.getPathComponentsRelativeTo=n,e.getRelativePathFromFile=function(n,a,i){return r(t(e.getDirectoryPath(n),a,i))},e.getRelativePathFromDirectory=t,e.getRelativePathToDirectoryOrUrl=function(t,r,a,i,s){var l=n(o(a,t),o(a,r),e.equateStringsCaseSensitive,i),c=l[0];if(s&&e.isRootedDiskPath(c)){var u=c.charAt(0)===e.directorySeparator?\"file://\":\"file:///\";l[0]=u+c}return e.getPathFromPathComponents(l)},e.ensurePathIsNonModuleName=r,e.getBaseFileName=a,e.combinePaths=i,e.resolvePath=o,e.hasTrailingDirectorySeparator=s,e.removeTrailingDirectorySeparator=l,e.ensureTrailingDirectorySeparator=c,e.comparePathsCaseSensitive=function(n,t){return u(n,t,e.compareStringsCaseSensitive)},e.comparePathsCaseInsensitive=function(n,t){return u(n,t,e.compareStringsCaseInsensitive)},e.comparePaths=function(n,t,r,a){return\"string\"==typeof r?(n=i(r,n),t=i(r,t)):\"boolean\"==typeof r&&(a=r),u(n,t,e.getStringComparer(a))},e.containsPath=d,e.tryRemoveDirectoryPrefix=function(n,t,r){var a,i,o=e.tryRemovePrefix(n,t,r);return void 0===o?void 0:47===(i=(a=o).charCodeAt(0))||92===i?a.slice(1):void 0};var m=/[^\\w\\s\\/]/g;function p(e){return\"\\\\\"+e}e.regExpEscape=function(e){return e.replace(m,p)};var f=[42,63];function g(n){return e.stringContains(a(n),\".\")}e.hasExtension=g,e.commonPackageFolders=[\"node_modules\",\"bower_components\",\"jspm_packages\"];var _=\"(?!(\"+e.commonPackageFolders.join(\"|\")+\")(/|$))\",v={singleAsteriskRegexFragment:\"([^./]|(\\\\.(?!min\\\\.js$))?)*\",doubleAsteriskRegexFragment:\"(/\"+_+\"[^/.][^/]*)*?\",replaceWildcardCharacter:function(e){return L(e,v.singleAsteriskRegexFragment)}},y={singleAsteriskRegexFragment:\"[^/]*\",doubleAsteriskRegexFragment:\"(/\"+_+\"[^/.][^/]*)*?\",replaceWildcardCharacter:function(e){return L(e,y.singleAsteriskRegexFragment)}},h={singleAsteriskRegexFragment:\"[^/]*\",doubleAsteriskRegexFragment:\"(/.+?)?\",replaceWildcardCharacter:function(e){return L(e,h.singleAsteriskRegexFragment)}},b={files:v,directories:y,exclude:h};function E(e,n,t){var r=T(e,n,t);if(r&&r.length)return\"^(\"+r.map(function(e){return\"(\"+e+\")\"}).join(\"|\")+\")\"+(\"exclude\"===t?\"($|/)\":\"$\")}function T(n,t,r){if(void 0!==n&&0!==n.length)return e.flatMap(n,function(n){return n&&function(n,t,r,a){var i=a.singleAsteriskRegexFragment,o=a.doubleAsteriskRegexFragment,s=a.replaceWildcardCharacter,c=\"\",u=!1,d=e.getNormalizedPathComponents(n,t),p=e.last(d);if(\"exclude\"!==r&&\"**\"===p)return;d[0]=l(d[0]),S(p)&&d.push(\"**\",\"*\");for(var f=0,g=0,v=d;g<v.length;g++){var y=v[g];if(\"**\"===y)c+=o;else if(\"directories\"===r&&(c+=\"(\",f++),u&&(c+=e.directorySeparator),\"exclude\"!==r){var h=\"\";42===y.charCodeAt(0)?(h+=\"([^./]\"+i+\")?\",y=y.substr(1)):63===y.charCodeAt(0)&&(h+=\"[^./]\",y=y.substr(1)),(h+=y.replace(m,s))!==y&&(c+=_),c+=h}else c+=y.replace(m,s);u=!0}for(;f>0;)c+=\")?\",f--;return c}(n,t,r,b[r])})}function S(e){return!/[.*?]/.test(e)}function L(e,n){return\"*\"===e?n:\"?\"===e?\"[^/]\":\"\\\\\"+e}function x(n,t,r,a,o){n=e.normalizePath(n);var s=i(o=e.normalizePath(o),n);return{includeFilePatterns:e.map(T(r,s,\"files\"),function(e){return\"^\"+e+\"$\"}),includeFilePattern:E(r,s,\"files\"),includeDirectoryPattern:E(r,s,\"directories\"),excludePattern:E(t,s,\"exclude\"),basePaths:function(n,t,r){var a=[n];if(t){for(var o=[],s=0,l=t;s<l.length;s++){var c=l[s],u=e.isRootedDiskPath(c)?c:e.normalizePath(i(n,c));o.push(C(u))}o.sort(e.getStringComparer(!r));for(var m=function(t){e.every(a,function(e){return!d(e,t,n,!r)})&&a.push(t)},p=0,f=o;p<f.length;p++){var g=f[p];m(g)}}return a}(n,r,a)}}function A(e,n){return new RegExp(e,n?\"\":\"i\")}function C(n){var t=e.indexOfAnyCharCode(n,f);return t<0?g(n)?l(e.getDirectoryPath(n)):n:n.substring(0,n.lastIndexOf(e.directorySeparator,t))}function D(e){switch(e.substr(e.lastIndexOf(\".\")).toLowerCase()){case\".js\":return 1;case\".jsx\":return 2;case\".ts\":return 3;case\".tsx\":return 4;case\".json\":return 6;default:return 0}}e.getRegularExpressionForWildcard=E,e.isImplicitGlob=S,e.getFileMatcherPatterns=x,e.getRegexFromPattern=A,e.matchFiles=function(n,t,r,a,o,s,l,c){n=e.normalizePath(n),s=e.normalizePath(s);for(var u=x(n,r,a,o,s),d=u.includeFilePatterns&&u.includeFilePatterns.map(function(e){return A(e,o)}),m=u.includeDirectoryPattern&&A(u.includeDirectoryPattern,o),p=u.excludePattern&&A(u.excludePattern,o),f=d?d.map(function(){return[]}):[[]],g=0,_=u.basePaths;g<_.length;g++){var v=_[g];y(v,i(s,v),l)}return e.flatten(f);function y(n,r,a){for(var o=c(n),s=o.files,l=o.directories,u=function(a){var o=i(n,a),s=i(r,a);if(t&&!e.fileExtensionIsOneOf(o,t))return\"continue\";if(p&&p.test(s))return\"continue\";if(d){var l=e.findIndex(d,function(e){return e.test(s)});-1!==l&&f[l].push(o)}else f[0].push(o)},g=0,_=e.sort(s,e.compareStringsCaseSensitive);g<_.length;g++)u(b=_[g]);if(void 0===a||0!=--a)for(var v=0,h=e.sort(l,e.compareStringsCaseSensitive);v<h.length;v++){var b=h[v],E=i(n,b),T=i(r,b);m&&!m.test(T)||p&&p.test(T)||y(E,T,a)}}},e.ensureScriptKind=function(e,n){return n||D(e)||3},e.getScriptKindFromFileName=D,e.supportedTSExtensions=[\".ts\",\".tsx\",\".d.ts\"],e.supportedTSExtensionsForExtractExtension=[\".d.ts\",\".ts\",\".tsx\"],e.supportedJSExtensions=[\".js\",\".jsx\"],e.supportedJSAndJsonExtensions=[\".js\",\".jsx\",\".json\"];var k=e.supportedTSExtensions.concat(e.supportedJSExtensions);function M(n,t){var r=n&&n.allowJs;if(!t||0===t.length)return r?k:e.supportedTSExtensions;var a=(r?k:e.supportedTSExtensions).concat(e.mapDefined(t,function(e){return 7===e.scriptKind||r&&(1===(n=e.scriptKind)||2===n)?e.extension:void 0;var n}));return e.deduplicate(a,e.equateStringsCaseSensitive,e.compareStringsCaseSensitive)}function O(e,n){return e<2?0:e<n.length?2:n.length}e.getSupportedExtensions=M,e.hasJSFileExtension=function(n){return e.some(e.supportedJSExtensions,function(t){return e.fileExtensionIs(n,t)})},e.hasJSOrJsonFileExtension=function(n){return e.supportedJSAndJsonExtensions.some(function(t){return e.fileExtensionIs(n,t)})},e.hasTSFileExtension=function(n){return e.some(e.supportedTSExtensions,function(t){return e.fileExtensionIs(n,t)})},e.isSupportedSourceFileName=function(n,t,r){if(!n)return!1;for(var a=0,i=M(t,r);a<i.length;a++){var o=i[a];if(e.fileExtensionIs(n,o))return!0}return!1},function(e){e[e.TypeScriptFiles=0]=\"TypeScriptFiles\",e[e.DeclarationAndJavaScriptFiles=2]=\"DeclarationAndJavaScriptFiles\",e[e.Highest=0]=\"Highest\",e[e.Lowest=2]=\"Lowest\"}(e.ExtensionPriority||(e.ExtensionPriority={})),e.getExtensionPriority=function(n,t){for(var r=t.length-1;r>=0;r--)if(e.fileExtensionIs(n,t[r]))return O(r,t);return 0},e.adjustExtensionPriority=O,e.getNextLowestExtensionPriority=function(e,n){return e<2?2:n.length};var I,R=[\".d.ts\",\".ts\",\".js\",\".tsx\",\".jsx\",\".json\"];function N(n,t){return e.fileExtensionIs(n,t)?w(n,t):void 0}function w(e,n){return e.substring(0,e.length-n.length)}function P(n,t,r,a){var i=void 0!==r&&void 0!==a?K(n,r,a):K(n);return i?n.slice(0,n.length-i.length)+(e.startsWith(t,\".\")?t:\".\"+t):n}function F(n){I.assert(e.hasZeroOrOneAsteriskCharacter(n));var t=n.indexOf(\"*\");return-1===t?void 0:{prefix:n.substr(0,t),suffix:n.substr(t+1)}}function G(e){return\".ts\"===e||\".tsx\"===e||\".d.ts\"===e}function V(n){return e.find(R,function(t){return e.fileExtensionIs(n,t)})}function K(n,t,r){if(t)return function(n,t,r){\"string\"==typeof t&&(t=[t]);for(var a=0,i=t;a<i.length;a++){var o=i[a];if(e.startsWith(o,\".\")||(o=\".\"+o),n.length>=o.length&&\".\"===n.charAt(n.length-o.length)){var s=n.slice(n.length-o.length);if(r(s,o))return s}}return\"\"}(n,t,r?e.equateStringsCaseInsensitive:e.equateStringsCaseSensitive);var i=a(n),o=i.lastIndexOf(\".\");return o>=0?i.substring(o):\"\"}e.removeFileExtension=function(e){for(var n=0,t=R;n<t.length;n++){var r=N(e,t[n]);if(void 0!==r)return r}return e},e.tryRemoveExtension=N,e.removeExtension=w,e.changeExtension=function(e,n){return P(e,n,R,!1)},e.changeAnyExtension=P,function(n){function t(n){var t=e.SyntaxKind;return t?t[n.kind]:n.kind.toString()}n.showSymbol=function(n){var r=e.SymbolFlags;return\"{ flags: \"+(r?function(e,n){for(var t=[],r=0;r<=30;r++){var a=1<<r;e&a&&t.push(n[a])}return t.join(\"|\")}(n.flags,r):n.flags)+\"; declarations: \"+e.map(n.declarations,t)+\" }\"},n.showSyntaxKind=t}(I=e.Debug||(e.Debug={})),e.tryParsePattern=F,e.positionIsSynthesized=function(e){return!(e>=0)},e.extensionIsTS=G,e.resolutionExtensionIsTSOrJson=function(e){return G(e)||\".json\"===e},e.extensionFromPath=function(e){var n=V(e);return void 0!==n?n:I.fail(\"File \"+e+\" has unknown extension.\")},e.isAnySupportedFileExtension=function(e){return void 0!==V(e)},e.tryGetExtensionFromPath=V,e.getAnyExtensionFromPath=K,e.isCheckJsEnabledForFile=function(e,n){return e.checkJsDirective?e.checkJsDirective.enabled:n.checkJs},e.emptyFileSystemEntries={files:e.emptyArray,directories:e.emptyArray},e.matchPatternOrExact=function(n,t){for(var r=[],a=0,i=n;a<i.length;a++){var o=i[a],s=F(o);if(s)r.push(s);else if(o===t)return o}return e.findBestPatternMatch(r,function(e){return e},t)},e.sliceAfter=function(e,n){var t=e.indexOf(n);return I.assert(-1!==t),e.slice(t)},e.minAndMax=function(e,n){I.assert(0!==e.length);for(var t=n(e[0]),r=t,a=1;a<e.length;a++){var i=n(e[a]);i<t?t=i:i>r&&(r=i)}return{min:t,max:r}};var B=function(){function n(){this.map=e.createMap()}return n.prototype.add=function(n){this.map.set(String(e.getNodeId(n)),n)},n.prototype.tryAdd=function(e){return!this.has(e)&&(this.add(e),!0)},n.prototype.has=function(n){return this.map.has(String(e.getNodeId(n)))},n.prototype.forEach=function(e){this.map.forEach(e)},n.prototype.some=function(n){return e.forEachEntry(this.map,n)||!1},n}();e.NodeSet=B;var H=function(){function n(){this.map=e.createMap()}return n.prototype.get=function(n){var t=this.map.get(String(e.getNodeId(n)));return t&&t.value},n.prototype.getOrUpdate=function(e,n){var t=this.get(e);if(t)return t;var r=n();return this.set(e,r),r},n.prototype.set=function(n,t){this.map.set(String(e.getNodeId(n)),{node:n,value:t})},n.prototype.has=function(n){return this.map.has(String(e.getNodeId(n)))},n.prototype.forEach=function(e){this.map.forEach(function(n){var t=n.node,r=n.value;return e(r,t)})},n}();e.NodeMap=H,e.rangeOfNode=function(n){return{pos:e.getTokenPosOfNode(n),end:n.end}},e.rangeOfTypeParameters=function(e){return{pos:e.pos-1,end:e.end+1}},e.skipTypeChecking=function(e,n){return n.skipLibCheck&&e.isDeclarationFile||n.skipDefaultLibCheck&&e.hasNoDefaultLib},e.isJsonEqual=function n(t,r){return t===r||\"object\"==typeof t&&null!==t&&\"object\"==typeof r&&null!==r&&e.equalOwnProperties(t,r,n)}}(Tn||(Tn={})),function(e){var n,t,r,a,i,o,s;function l(e,n){return n&&e(n)}function c(e,n,t){if(t){if(n)return n(t);for(var r=0,a=t;r<a.length;r++){var i=e(a[r]);if(i)return i}}}function u(e,n){return 42===e.charCodeAt(n+1)&&42===e.charCodeAt(n+2)&&47!==e.charCodeAt(n+3)}function d(n,t,r){if(n&&!(n.kind<=145))switch(n.kind){case 146:return l(t,n.left)||l(t,n.right);case 148:return l(t,n.name)||l(t,n.constraint)||l(t,n.default)||l(t,n.expression);case 274:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||l(t,n.questionToken)||l(t,n.exclamationToken)||l(t,n.equalsToken)||l(t,n.objectAssignmentInitializer);case 275:return l(t,n.expression);case 149:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.dotDotDotToken)||l(t,n.name)||l(t,n.questionToken)||l(t,n.type)||l(t,n.initializer);case 152:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||l(t,n.questionToken)||l(t,n.exclamationToken)||l(t,n.type)||l(t,n.initializer);case 151:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||l(t,n.questionToken)||l(t,n.type)||l(t,n.initializer);case 273:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||l(t,n.questionToken)||l(t,n.initializer);case 235:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||l(t,n.exclamationToken)||l(t,n.type)||l(t,n.initializer);case 184:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.dotDotDotToken)||l(t,n.propertyName)||l(t,n.name)||l(t,n.initializer);case 163:case 164:case 158:case 159:case 160:return c(t,r,n.decorators)||c(t,r,n.modifiers)||c(t,r,n.typeParameters)||c(t,r,n.parameters)||l(t,n.type);case 154:case 153:case 155:case 156:case 157:case 194:case 237:case 195:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.asteriskToken)||l(t,n.name)||l(t,n.questionToken)||l(t,n.exclamationToken)||c(t,r,n.typeParameters)||c(t,r,n.parameters)||l(t,n.type)||l(t,n.equalsGreaterThanToken)||l(t,n.body);case 162:return l(t,n.typeName)||c(t,r,n.typeArguments);case 161:return l(t,n.parameterName)||l(t,n.type);case 165:return l(t,n.exprName);case 166:return c(t,r,n.members);case 167:return l(t,n.elementType);case 168:return c(t,r,n.elementTypes);case 171:case 172:return c(t,r,n.types);case 173:return l(t,n.checkType)||l(t,n.extendsType)||l(t,n.trueType)||l(t,n.falseType);case 174:return l(t,n.typeParameter);case 181:return l(t,n.argument)||l(t,n.qualifier)||c(t,r,n.typeArguments);case 175:case 177:return l(t,n.type);case 178:return l(t,n.objectType)||l(t,n.indexType);case 179:return l(t,n.readonlyToken)||l(t,n.typeParameter)||l(t,n.questionToken)||l(t,n.type);case 180:return l(t,n.literal);case 182:case 183:case 185:return c(t,r,n.elements);case 186:return c(t,r,n.properties);case 187:return l(t,n.expression)||l(t,n.name);case 188:return l(t,n.expression)||l(t,n.argumentExpression);case 189:case 190:return l(t,n.expression)||c(t,r,n.typeArguments)||c(t,r,n.arguments);case 191:return l(t,n.tag)||c(t,r,n.typeArguments)||l(t,n.template);case 192:return l(t,n.type)||l(t,n.expression);case 193:case 196:case 197:case 198:return l(t,n.expression);case 200:return l(t,n.operand);case 205:return l(t,n.asteriskToken)||l(t,n.expression);case 199:return l(t,n.expression);case 201:return l(t,n.operand);case 202:return l(t,n.left)||l(t,n.operatorToken)||l(t,n.right);case 210:return l(t,n.expression)||l(t,n.type);case 211:return l(t,n.expression);case 212:return l(t,n.name);case 203:return l(t,n.condition)||l(t,n.questionToken)||l(t,n.whenTrue)||l(t,n.colonToken)||l(t,n.whenFalse);case 206:return l(t,n.expression);case 216:case 243:return c(t,r,n.statements);case 277:return c(t,r,n.statements)||l(t,n.endOfFileToken);case 217:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.declarationList);case 236:return c(t,r,n.declarations);case 219:return l(t,n.expression);case 220:return l(t,n.expression)||l(t,n.thenStatement)||l(t,n.elseStatement);case 221:return l(t,n.statement)||l(t,n.expression);case 222:return l(t,n.expression)||l(t,n.statement);case 223:return l(t,n.initializer)||l(t,n.condition)||l(t,n.incrementor)||l(t,n.statement);case 224:return l(t,n.initializer)||l(t,n.expression)||l(t,n.statement);case 225:return l(t,n.awaitModifier)||l(t,n.initializer)||l(t,n.expression)||l(t,n.statement);case 226:case 227:return l(t,n.label);case 228:return l(t,n.expression);case 229:return l(t,n.expression)||l(t,n.statement);case 230:return l(t,n.expression)||l(t,n.caseBlock);case 244:return c(t,r,n.clauses);case 269:return l(t,n.expression)||c(t,r,n.statements);case 270:return c(t,r,n.statements);case 231:return l(t,n.label)||l(t,n.statement);case 232:return l(t,n.expression);case 233:return l(t,n.tryBlock)||l(t,n.catchClause)||l(t,n.finallyBlock);case 272:return l(t,n.variableDeclaration)||l(t,n.block);case 150:return l(t,n.expression);case 238:case 207:case 239:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||c(t,r,n.typeParameters)||c(t,r,n.heritageClauses)||c(t,r,n.members);case 240:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||c(t,r,n.typeParameters)||l(t,n.type);case 241:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||c(t,r,n.members);case 276:return l(t,n.name)||l(t,n.initializer);case 242:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||l(t,n.body);case 246:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.name)||l(t,n.moduleReference);case 247:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.importClause)||l(t,n.moduleSpecifier);case 248:return l(t,n.name)||l(t,n.namedBindings);case 245:case 249:return l(t,n.name);case 250:case 254:return c(t,r,n.elements);case 253:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.exportClause)||l(t,n.moduleSpecifier);case 251:case 255:return l(t,n.propertyName)||l(t,n.name);case 252:return c(t,r,n.decorators)||c(t,r,n.modifiers)||l(t,n.expression);case 204:return l(t,n.head)||c(t,r,n.templateSpans);case 214:return l(t,n.expression)||l(t,n.literal);case 147:return l(t,n.expression);case 271:return c(t,r,n.types);case 209:return l(t,n.expression)||c(t,r,n.typeArguments);case 257:return l(t,n.expression);case 256:return c(t,r,n.decorators);case 307:return c(t,r,n.elements);case 258:return l(t,n.openingElement)||c(t,r,n.children)||l(t,n.closingElement);case 262:return l(t,n.openingFragment)||c(t,r,n.children)||l(t,n.closingFragment);case 259:case 260:return l(t,n.tagName)||c(t,r,n.typeArguments)||l(t,n.attributes);case 266:return c(t,r,n.properties);case 265:return l(t,n.name)||l(t,n.initializer);case 267:return l(t,n.expression);case 268:return l(t,n.dotDotDotToken)||l(t,n.expression);case 261:return l(t,n.tagName);case 169:case 170:case 281:case 285:case 284:case 286:case 288:return l(t,n.type);case 287:return c(t,r,n.parameters)||l(t,n.type);case 289:return c(t,r,n.tags);case 297:case 303:return n.isNameFirst?l(t,n.name)||l(t,n.typeExpression):l(t,n.typeExpression)||l(t,n.name);case 298:case 300:return l(t,n.typeExpression);case 293:return l(t,n.class);case 301:return l(t,n.constraint)||c(t,r,n.typeParameters);case 302:return n.typeExpression&&281===n.typeExpression.kind?l(t,n.typeExpression)||l(t,n.fullName):l(t,n.fullName)||l(t,n.typeExpression);case 295:return l(t,n.fullName)||l(t,n.typeExpression);case 299:case 296:return l(t,n.typeExpression);case 291:return c(t,r,n.decorators)||c(t,r,n.modifiers)||e.forEach(n.typeParameters,t)||e.forEach(n.parameters,t)||l(t,n.type);case 290:if(n.jsDocPropertyTags)for(var a=0,i=n.jsDocPropertyTags;a<i.length;a++){l(t,i[a])}return;case 306:return l(t,n.expression)}}function m(n){return e.fileExtensionIs(n,\".d.ts\")}function p(n,t){for(var r=e.createScanner(n.languageVersion,!1,0,t),a=[];;){var i=r.scan();if(!e.isTrivia(i))break;var o={kind:r.getToken(),pos:r.getTokenPos(),end:r.getTextPos()};h(a,o,t.substring(o.pos,o.end))}n.pragmas=e.createMap();for(var s=0,l=a;s<l.length;s++){var c=l[s];if(n.pragmas.has(c.name)){var u=n.pragmas.get(c.name);u instanceof Array?u.push(c.args):n.pragmas.set(c.name,[u,c.args])}else n.pragmas.set(c.name,c.args)}}function f(n,t){n.checkJsDirective=void 0,n.referencedFiles=[],n.typeReferenceDirectives=[],n.libReferenceDirectives=[],n.amdDependencies=[],n.hasNoDefaultLib=!1,n.pragmas.forEach(function(r,a){switch(a){case\"reference\":var i=n.referencedFiles,o=n.typeReferenceDirectives,s=n.libReferenceDirectives;e.forEach(e.toArray(r),function(r){r.arguments[\"no-default-lib\"]?n.hasNoDefaultLib=!0:r.arguments.types?o.push({pos:r.arguments.types.pos,end:r.arguments.types.end,fileName:r.arguments.types.value}):r.arguments.lib?s.push({pos:r.arguments.lib.pos,end:r.arguments.lib.end,fileName:r.arguments.lib.value}):r.arguments.path?i.push({pos:r.arguments.path.pos,end:r.arguments.path.end,fileName:r.arguments.path.value}):t(r.range.pos,r.range.end-r.range.pos,e.Diagnostics.Invalid_reference_directive_syntax)});break;case\"amd-dependency\":n.amdDependencies=e.map(e.toArray(r),function(e){return{name:e.arguments.name,path:e.arguments.path}});break;case\"amd-module\":if(r instanceof Array)for(var l=0,c=r;l<c.length;l++){var u=c[l];n.moduleName&&t(u.range.pos,u.range.end-u.range.pos,e.Diagnostics.An_AMD_module_cannot_have_multiple_name_assignments),n.moduleName=u.arguments.name}else n.moduleName=r.arguments.name;break;case\"ts-nocheck\":case\"ts-check\":e.forEach(e.toArray(r),function(e){(!n.checkJsDirective||e.range.pos>n.checkJsDirective.pos)&&(n.checkJsDirective={enabled:\"ts-check\"===a,end:e.range.end,pos:e.range.pos})});break;case\"jsx\":return;default:e.Debug.fail(\"Unhandled pragma kind\")}})}!function(e){e[e.None=0]=\"None\",e[e.Yield=1]=\"Yield\",e[e.Await=2]=\"Await\",e[e.Type=4]=\"Type\",e[e.IgnoreMissingOpenBrace=16]=\"IgnoreMissingOpenBrace\",e[e.JSDoc=32]=\"JSDoc\"}(n||(n={})),e.createNode=function(n,o,s){return 277===n?new(i||(i=e.objectAllocator.getSourceFileConstructor()))(n,o,s):71===n?new(a||(a=e.objectAllocator.getIdentifierConstructor()))(n,o,s):e.isNodeKind(n)?new(t||(t=e.objectAllocator.getNodeConstructor()))(n,o,s):new(r||(r=e.objectAllocator.getTokenConstructor()))(n,o,s)},e.isJSDocLikeText=u,e.forEachChild=d,e.createSourceFile=function(n,t,r,a,i){var s;return void 0===a&&(a=!1),e.performance.mark(\"beforeParse\"),s=100===r?o.parseSourceFile(n,t,r,void 0,a,6):o.parseSourceFile(n,t,r,void 0,a,i),e.performance.mark(\"afterParse\"),e.performance.measure(\"Parse\",\"beforeParse\",\"afterParse\"),s},e.parseIsolatedEntityName=function(e,n){return o.parseIsolatedEntityName(e,n)},e.parseJsonText=function(e,n){return o.parseJsonText(e,n)},e.isExternalModule=function(e){return void 0!==e.externalModuleIndicator},e.updateSourceFile=function(e,n,t,r){void 0===r&&(r=!1);var a=s.updateSourceFile(e,n,t,r);return a.flags|=1572864&e.flags,a},e.parseIsolatedJSDocComment=function(e,n,t){var r=o.JSDocParser.parseIsolatedJSDocComment(e,n,t);return r&&r.jsDoc&&o.fixupParentReferences(r.jsDoc),r},e.parseJSDocTypeExpressionForTests=function(e,n,t){return o.JSDocParser.parseJSDocTypeExpressionForTests(e,n,t)},function(n){var t,r,a,i,o,s,l,c,g,_,v,y,h,b,T,S,L,x=e.createScanner(6,!0),A=10240,C=!1;function D(n,t,r,a,i){void 0===r&&(r=2),M(t,r,a,6),(o=w(n,2,6,!1)).flags=b,re();var l=ne();if(1===te())o.statements=be([],l,l),o.endOfFileToken=ge();else{var c=ye(219);switch(te()){case 21:c.expression=kt();break;case 101:case 86:case 95:c.expression=ge();break;case 38:le(function(){return 8===re()&&56!==re()})?c.expression=ct():c.expression=Ot();break;case 8:case 9:if(le(function(){return 56!==re()})){c.expression=tn();break}default:c.expression=Ot()}Ee(c),o.statements=be([c],l),o.endOfFileToken=fe(1,e.Diagnostics.Unexpected_token)}i&&N(o),o.parseDiagnostics=s;var u=o;return O(),u}function k(e){return 4===e||2===e||1===e||6===e?1:0}function M(n,o,c,u){switch(t=e.objectAllocator.getNodeConstructor(),r=e.objectAllocator.getTokenConstructor(),a=e.objectAllocator.getIdentifierConstructor(),i=e.objectAllocator.getSourceFileConstructor(),g=n,l=c,s=[],h=0,v=e.createMap(),y=0,_=0,u){case 1:case 2:b=65536;break;case 6:b=16842752;break;default:b=0}C=!1,x.setText(g),x.setOnError(ee),x.setScriptTarget(o),x.setLanguageVariant(k(u))}function O(){x.setText(\"\"),x.setOnError(void 0),s=void 0,o=void 0,v=void 0,l=void 0,g=void 0}function I(n,t,r,a){var i=m(n);return i&&(b|=4194304),(o=w(n,t,a,i)).flags=b,re(),p(o,g),f(o,function(n,t,r){s.push(e.createFileDiagnostic(o,n,t,r))}),o.statements=je(0,Jt),e.Debug.assert(1===te()),o.endOfFileToken=R(ge()),function(n){n.externalModuleIndicator=e.forEach(n.statements,wr)||function(e){return 1048576&e.flags?Pr(e):void 0}(n)}(o),o.nodeCount=_,o.identifierCount=y,o.identifiers=v,o.parseDiagnostics=s,r&&N(o),o}function R(n){e.Debug.assert(!n.jsDoc);var t=e.mapDefined(e.getJSDocCommentRanges(n,o.text),function(e){return L.parseJSDocComment(n,e.pos,e.end-e.pos)});return t.length&&(n.jsDoc=t),n}function N(n){var t=n;return void d(n,function n(r){if(r.parent!==t){r.parent=t;var a=t;if(t=r,d(r,n),e.hasJSDocNodes(r))for(var i=0,o=r.jsDoc;i<o.length;i++){var s=o[i];s.parent=r,t=s,d(s,n)}t=a}})}function w(n,t,r,a){var o=new i(277,0,g.length);return _++,o.text=g,o.bindDiagnostics=[],o.bindSuggestionDiagnostics=void 0,o.languageVersion=t,o.fileName=e.normalizePath(n),o.languageVariant=k(r),o.isDeclarationFile=a,o.scriptKind=r,o}function P(e,n){e?b|=n:b&=~n}function F(e){P(e,2048)}function G(e){P(e,4096)}function V(e){P(e,8192)}function K(e){P(e,16384)}function B(e,n){var t=e&b;if(t){P(!1,t);var r=n();return P(!0,t),r}return n()}function H(e,n){var t=e&~b;if(t){P(!0,t);var r=n();return P(!1,t),r}return n()}function U(e){return B(2048,e)}function j(e){return H(16384,e)}function W(e){return 0!=(b&e)}function q(){return W(4096)}function z(){return W(2048)}function J(){return W(8192)}function X(){return W(16384)}function Y(e,n){Z(x.getTokenPos(),x.getTextPos(),e,n)}function Q(n,t,r,a){var i=e.lastOrUndefined(s);i&&n===i.start||s.push(e.createFileDiagnostic(o,n,t,r,a)),C=!0}function Z(e,n,t,r){Q(e,n-e,t,r)}function $(e,n,t){Z(e.pos,e.end,n,t)}function ee(e,n){Q(x.getTextPos(),n,e)}function ne(){return x.getStartPos()}function te(){return c}function re(){return c=x.scan()}function ae(){return c=x.reScanGreaterToken()}function ie(){return c=x.scanJsxIdentifier()}function oe(){return c=x.scanJsxToken()}function se(n,t){var r=c,a=s.length,i=C,o=b,l=t?x.lookAhead(n):x.tryScan(n);return e.Debug.assert(o===b),l&&!t||(c=r,s.length=a,C=i),l}function le(e){return se(e,!0)}function ce(e){return se(e,!1)}function ue(){return 71===te()||(116!==te()||!q())&&((121!==te()||!X())&&te()>107)}function de(n,t,r){return void 0===r&&(r=!0),te()===n?(r&&re(),!0):(t?Y(t):Y(e.Diagnostics._0_expected,e.tokenToString(n)),!1)}function me(e){return te()===e&&(re(),!0)}function pe(e){if(te()===e)return ge()}function fe(n,t,r){return pe(n)||Te(n,!1,t||e.Diagnostics._0_expected,r||e.tokenToString(n))}function ge(){var e=ye(te());return re(),Ee(e)}function _e(){return 25===te()||(18===te()||1===te()||x.hasPrecedingLineBreak())}function ve(){return _e()?(25===te()&&re(),!0):de(25)}function ye(n,i){_++;var o=i>=0?i:x.getStartPos();return e.isNodeKind(n)||0===n?new t(n,o,o):71===n?new a(n,o,o):new r(n,o,o)}function he(e,n){var t=ye(e,n);return 2&x.getTokenFlags()&&R(t),t}function be(e,n,t){var r=e.length,a=r>=1&&r<=4?e.slice():e;return a.pos=n,a.end=void 0===t?x.getStartPos():t,a}function Ee(e,n){return e.end=void 0===n?x.getStartPos():n,b&&(e.flags|=b),C&&(C=!1,e.flags|=32768),e}function Te(n,t,r,a){t?Q(x.getStartPos(),0,r,a):r&&Y(r,a);var i=ye(n);return 71===n?i.escapedText=\"\":(e.isLiteralKind(n)||e.isTemplateLiteralKind(n))&&(i.text=\"\"),Ee(i)}function Se(e){var n=v.get(e);return void 0===n&&v.set(e,n=e),n}function Le(n,t){if(y++,n){var r=ye(71);return 71!==te()&&(r.originalKeywordKind=te()),r.escapedText=e.escapeLeadingUnderscores(Se(x.getTokenValue())),re(),Ee(r)}return Te(71,1===te(),t||e.Diagnostics.Identifier_expected)}function xe(e){return Le(ue(),e)}function Ae(n){return Le(e.tokenIsIdentifierOrKeyword(te()),n)}function Ce(){return e.tokenIsIdentifierOrKeyword(te())||9===te()||8===te()}function De(e){if(9===te()||8===te()){var n=tn();return n.text=Se(n.text),n}return e&&21===te()?function(){var e=ye(147);return de(21),e.expression=U(Xn),de(22),Ee(e)}():Ae()}function ke(){return De(!0)}function Me(e){return te()===e&&ce(Ie)}function Oe(){return re(),!x.hasPrecedingLineBreak()&&Re()}function Ie(){switch(te()){case 76:return 83===re();case 84:return re(),79===te()?le(Ne):39!==te()&&118!==te()&&17!==te()&&Re();case 79:return Ne();case 115:case 125:case 136:return re(),Re();default:return Oe()}}function Re(){return 21===te()||17===te()||39===te()||24===te()||Ce()}function Ne(){return re(),75===te()||89===te()||109===te()||117===te()&&le(Bt)||120===te()&&le(Ht)}function we(n,t){if(qe(n))return!0;switch(n){case 0:case 1:case 3:return!(25===te()&&t)&&qt();case 2:return 73===te()||79===te();case 4:return le(bn);case 5:return le(pr)||25===te()&&!t;case 6:return 21===te()||Ce();case 12:return 21===te()||39===te()||24===te()||Ce();case 18:return Ce();case 9:return 21===te()||24===te()||Ce();case 7:return 17===te()?le(Pe):t?ue()&&!Ke():zn()&&!Ke();case 8:return tr();case 10:return 26===te()||24===te()||tr();case 19:return ue();case 15:if(26===te())return!0;case 11:return 24===te()||Jn();case 16:return mn(!1);case 17:return mn(!0);case 20:case 21:return 26===te()||wn();case 22:return Lr();case 23:return e.tokenIsIdentifierOrKeyword(te());case 13:return e.tokenIsIdentifierOrKeyword(te())||17===te();case 14:return!0}return e.Debug.fail(\"Non-exhaustive case in 'isListElement'.\")}function Pe(){if(e.Debug.assert(17===te()),18===re()){var n=re();return 26===n||17===n||85===n||108===n}return!0}function Fe(){return re(),ue()}function Ge(){return re(),e.tokenIsIdentifierOrKeyword(te())}function Ve(){return re(),e.tokenIsIdentifierOrKeywordOrGreaterThan(te())}function Ke(){return(108===te()||85===te())&&le(Be)}function Be(){return re(),Jn()}function He(){return re(),wn()}function Ue(e){if(1===te())return!0;switch(e){case 1:case 2:case 4:case 5:case 6:case 12:case 9:case 23:return 18===te();case 3:return 18===te()||73===te()||79===te();case 7:return 17===te()||85===te()||108===te();case 8:return function(){if(_e())return!0;if(it(te()))return!0;if(36===te())return!0;return!1}();case 19:return 29===te()||19===te()||17===te()||85===te()||108===te();case 11:return 20===te()||25===te();case 15:case 21:case 10:return 22===te();case 17:case 16:case 18:return 20===te()||22===te();case 20:return 26!==te();case 22:return 17===te()||18===te();case 13:return 29===te()||41===te();case 14:return 27===te()&&le(kr);default:return!1}}function je(e,n){var t=h;h|=1<<e;for(var r=[],a=ne();!Ue(e);)if(we(e,!1)){var i=We(e,n);r.push(i)}else if(ze(e))break;return h=t,be(r,a)}function We(e,n){var t=qe(e);return t?function(e){return x.setTextPos(e.end),re(),e}(t):n()}function qe(n){if(!C&&l){var t=l.currentNode(x.getStartPos());if(!e.nodeIsMissing(t))if(!t.intersectsChange)if(!e.containsParseError(t))if((12679168&t.flags)===b&&function(e,n){switch(n){case 5:return function(e){if(e)switch(e.kind){case 155:case 160:case 156:case 157:case 152:case 215:return!0;case 154:var n=e,t=71===n.name.kind&&123===n.name.originalKeywordKind;return!t}return!1}(e);case 2:return function(e){if(e)switch(e.kind){case 269:case 270:return!0}return!1}(e);case 0:case 1:case 3:return function(e){if(e)switch(e.kind){case 237:case 217:case 216:case 220:case 219:case 232:case 228:case 230:case 227:case 226:case 224:case 225:case 223:case 222:case 229:case 218:case 233:case 231:case 221:case 234:case 247:case 246:case 253:case 252:case 242:case 238:case 239:case 241:case 240:return!0}return!1}(e);case 6:return function(e){return 276===e.kind}(e);case 4:return function(e){if(e)switch(e.kind){case 159:case 153:case 160:case 151:case 158:return!0}return!1}(e);case 8:return function(e){if(235!==e.kind)return!1;return void 0===e.initializer}(e);case 17:case 16:return function(e){if(149!==e.kind)return!1;return void 0===e.initializer}(e);case 18:return!1}return!1}(t,n))return t.jsDocCache&&(t.jsDocCache=void 0),t}}function ze(n){return Y(function(n){switch(n){case 0:case 1:return e.Diagnostics.Declaration_or_statement_expected;case 2:return e.Diagnostics.case_or_default_expected;case 3:return e.Diagnostics.Statement_expected;case 18:case 4:return e.Diagnostics.Property_or_signature_expected;case 5:return e.Diagnostics.Unexpected_token_A_constructor_method_accessor_or_property_was_expected;case 6:return e.Diagnostics.Enum_member_expected;case 7:return e.Diagnostics.Expression_expected;case 8:return e.Diagnostics.Variable_declaration_expected;case 9:return e.Diagnostics.Property_destructuring_pattern_expected;case 10:return e.Diagnostics.Array_element_destructuring_pattern_expected;case 11:return e.Diagnostics.Argument_expression_expected;case 12:return e.Diagnostics.Property_assignment_expected;case 15:return e.Diagnostics.Expression_or_comma_expected;case 17:case 16:return e.Diagnostics.Parameter_declaration_expected;case 19:return e.Diagnostics.Type_parameter_declaration_expected;case 20:return e.Diagnostics.Type_argument_expected;case 21:return e.Diagnostics.Type_expected;case 22:return e.Diagnostics.Unexpected_token_expected;case 23:case 13:case 14:return e.Diagnostics.Identifier_expected;default:return}}(n)),!!function(){for(var e=0;e<24;e++)if(h&1<<e&&(we(e,!0)||Ue(e)))return!0;return!1}()||(re(),!1)}function Je(e,n,t){var r=h;h|=1<<e;for(var a=[],i=ne(),o=-1;;)if(we(e,!1)){var s=x.getStartPos();if(a.push(We(e,n)),o=x.getTokenPos(),me(26))continue;if(o=-1,Ue(e))break;de(26),t&&25===te()&&!x.hasPrecedingLineBreak()&&re(),s===x.getStartPos()&&re()}else{if(Ue(e))break;if(ze(e))break}h=r;var l=be(a,i);return o>=0&&(l.hasTrailingComma=!0),l}function Xe(){var e=be([],ne());return e.isMissingList=!0,e}function Ye(e,n,t,r){if(de(t)){var a=Je(e,n);return de(r),a}return Xe()}function Qe(e,n){for(var t=e?Ae(n):xe(n),r=x.getStartPos();me(23);){if(27===te()){t.jsdocDotPos=r;break}r=x.getStartPos(),t=Ze(t,$e(e))}return t}function Ze(e,n){var t=ye(146,e.pos);return t.left=e,t.right=n,Ee(t)}function $e(n){if(x.hasPrecedingLineBreak()&&e.tokenIsIdentifierOrKeyword(te())&&le(Kt))return Te(71,!0,e.Diagnostics.Identifier_expected);return n?Ae():xe()}function en(){var n,t=ye(204);t.head=(n=rn(te()),e.Debug.assert(14===n.kind,\"Template head has wrong token kind\"),n),e.Debug.assert(14===t.head.kind,\"Template head has wrong token kind\");var r=[],a=ne();do{r.push(nn())}while(15===e.last(r).literal.kind);return t.templateSpans=be(r,a),Ee(t)}function nn(){var n,t,r=ye(214);return r.expression=U(Xn),18===te()?(c=x.reScanTemplateToken(),t=rn(te()),e.Debug.assert(15===t.kind||16===t.kind,\"Template fragment has wrong token kind\"),n=t):n=fe(16,e.Diagnostics._0_expected,e.tokenToString(18)),r.literal=n,Ee(r)}function tn(){return rn(te())}function rn(e){var n=ye(e),t=x.getTokenValue();return n.text=t,x.hasExtendedUnicodeEscape()&&(n.hasExtendedUnicodeEscape=!0),x.isUnterminated()&&(n.isUnterminated=!0),8===n.kind&&(n.numericLiteralFlags=1008&x.getTokenFlags()),re(),Ee(n),n}function an(){var n=ye(162);return n.typeName=Qe(!0,e.Diagnostics.Type_expected),x.hasPrecedingLineBreak()||27!==te()||(n.typeArguments=Ye(20,jn,27,29)),Ee(n)}function on(e){var n=ye(282);return e?Fn(286,n):(re(),Ee(n))}function sn(){var e=ye(149);return 99!==te()&&94!==te()||(e.name=Ae(),de(56)),e.type=ln(),Ee(e)}function ln(){x.setInJSDocType(!0);var e=pe(24),n=Hn();if(x.setInJSDocType(!1),e){var t=ye(288,e.pos);t.type=n,n=Ee(t)}return 58===te()?Fn(286,n):n}function cn(){var e=ye(148);return e.name=xe(),me(85)&&(wn()||!Jn()?e.constraint=jn():e.expression=ut()),me(58)&&(e.default=jn()),Ee(e)}function un(){if(27===te())return Ye(19,cn,27,29)}function dn(){if(me(56))return jn()}function mn(n){return 24===te()||tr()||e.isModifierKind(te())||57===te()||wn(!n)}function pn(){var n=he(149);return 99===te()?(n.name=Le(!0),n.type=dn(),Ee(n)):(n.decorators=fr(),n.modifiers=gr(),n.dotDotDotToken=pe(24),n.name=rr(),0===e.getFullWidth(n.name)&&!e.hasModifiers(n)&&e.isModifierKind(te())&&re(),n.questionToken=pe(55),n.type=dn(),n.initializer=Yn(),Ee(n))}function fn(n,t,r){32&t||(r.typeParameters=un());var a=function(e,n){if(!de(19))return e.parameters=Xe(),!1;var t=q(),r=X();return G(!!(1&n)),K(!!(2&n)),e.parameters=32&n?Je(17,sn):Je(16,pn),G(t),K(r),de(20)}(r,t);return(!function(n,t){if(36===n)return de(n),!0;if(me(56))return!0;if(t&&36===te())return Y(e.Diagnostics._0_expected,e.tokenToString(56)),re(),!0;return!1}(n,!!(4&t))||(r.type=Hn(),!function n(t){switch(t.kind){case 162:return e.nodeIsMissing(t.typeName);case 163:case 164:var r=t,a=r.parameters,i=r.type;return!!a.isMissingList||n(i);case 175:return n(t.type);default:return!1}}(r.type)))&&a}function gn(){me(26)||ve()}function _n(e){var n=he(e);return 159===e&&de(94),fn(56,4,n),gn(),Ee(n)}function vn(){return 21===te()&&le(yn)}function yn(){if(re(),24===te()||22===te())return!0;if(e.isModifierKind(te())){if(re(),ue())return!0}else{if(!ue())return!1;re()}return 56===te()||26===te()||55===te()&&(re(),56===te()||26===te()||22===te())}function hn(e){return e.kind=160,e.parameters=Ye(16,pn,21,22),e.type=qn(),gn(),Ee(e)}function bn(){if(19===te()||27===te())return!0;for(var n=!1;e.isModifierKind(te());)n=!0,re();return 21===te()||(Ce()&&(n=!0,re()),!!n&&(19===te()||27===te()||55===te()||56===te()||26===te()||_e()))}function En(){if(19===te()||27===te())return _n(158);if(94===te()&&le(Tn))return _n(159);var e=he(0);return e.modifiers=gr(),vn()?hn(e):function(e){return e.name=ke(),e.questionToken=pe(55),19===te()||27===te()?(e.kind=153,fn(56,4,e)):(e.kind=151,e.type=qn(),58===te()&&(e.initializer=Yn())),gn(),Ee(e)}(e)}function Tn(){return re(),19===te()||27===te()}function Sn(){return 23===re()}function Ln(){switch(re()){case 19:case 27:case 23:return!0}return!1}function xn(){var e;return de(17)?(e=je(4,En),de(18)):e=Xe(),e}function An(){return re(),37===te()||38===te()?132===re():(132===te()&&re(),21===te()&&Fe()&&92===re())}function Cn(){var e=ye(179);return de(17),132!==te()&&37!==te()&&38!==te()||(e.readonlyToken=ge(),132!==e.readonlyToken.kind&&fe(132)),de(21),e.typeParameter=function(){var e=ye(148);return e.name=xe(),de(92),e.constraint=jn(),Ee(e)}(),de(22),55!==te()&&37!==te()&&38!==te()||(e.questionToken=ge(),55!==e.questionToken.kind&&fe(55)),e.type=qn(),ve(),de(18),Ee(e)}function Dn(){var e=ne();if(me(24)){var n=ye(170,e);return n.type=jn(),Ee(n)}var t=jn();return 2097152&b||284!==t.kind||t.pos!==t.type.pos||(t.kind=169),t}function kn(){var e=ge();return 23===te()?void 0:e}function Mn(e){var n,t=ye(180);e&&((n=ye(200)).operator=38,re());var r=101===te()||86===te()?ge():rn(te());return e&&(n.operand=r,Ee(n),r=n),t.literal=r,Ee(t)}function On(){return re(),91===te()}function In(){o.flags|=524288;var n=ye(181);return me(103)&&(n.isTypeOf=!0),de(91),de(19),n.argument=jn(),de(20),me(23)&&(n.qualifier=Qe(!0,e.Diagnostics.Type_expected)),n.typeArguments=Sr(),Ee(n)}function Rn(){return 8===re()}function Nn(){switch(te()){case 119:case 142:case 137:case 134:case 138:case 122:case 140:case 131:case 135:return ce(kn)||an();case 39:return on(!1);case 61:return on(!0);case 55:return r=x.getStartPos(),re(),26===te()||18===te()||20===te()||29===te()||58===te()||49===te()?Ee(t=ye(283,r)):((t=ye(284,r)).type=jn(),Ee(t));case 89:return function(){if(le(Dr)){var e=he(287);return re(),fn(56,36,e),Ee(e)}var n=ye(162);return n.typeName=Ae(),Ee(n)}();case 51:return function(){var e=ye(285);return re(),e.type=Nn(),Ee(e)}();case 13:case 9:case 8:case 101:case 86:return Mn();case 38:return le(Rn)?Mn(!0):an();case 105:case 95:return ge();case 99:var e=(n=ye(176),re(),Ee(n));return 127!==te()||x.hasPrecedingLineBreak()?e:function(e){re();var n=ye(161,e.pos);return n.parameterName=e,n.type=jn(),Ee(n)}(e);case 103:return le(On)?In():function(){var e=ye(165);return de(103),e.exprName=Qe(!0),Ee(e)}();case 17:return le(An)?Cn():function(){var e=ye(166);return e.members=xn(),Ee(e)}();case 21:return function(){var e=ye(168);return e.elementTypes=Ye(21,Dn,21,22),Ee(e)}();case 19:return function(){var e=ye(175);return de(19),e.type=jn(),de(20),Ee(e)}();case 91:return In();default:return an()}var n,t,r}function wn(e){switch(te()){case 119:case 142:case 137:case 134:case 122:case 138:case 141:case 105:case 140:case 95:case 99:case 103:case 131:case 17:case 21:case 27:case 49:case 48:case 94:case 9:case 8:case 101:case 86:case 135:case 39:case 55:case 51:case 24:case 126:case 91:return!0;case 89:return!e;case 38:return!e&&le(Rn);case 19:return!e&&le(Pn);default:return ue()}}function Pn(){return re(),20===te()||mn(!1)||wn()}function Fn(e,n){re();var t=ye(e,n.pos);return t.type=n,Ee(t)}function Gn(){var e=te();switch(e){case 128:case 141:return function(e){var n=ye(177);return de(e),n.operator=e,n.type=Gn(),Ee(n)}(e);case 126:return function(){var e=ye(174);de(126);var n=ye(148);return n.name=xe(),e.typeParameter=Ee(n),Ee(e)}()}return function(){for(var e=Nn();!x.hasPrecedingLineBreak();)switch(te()){case 51:e=Fn(285,e);break;case 55:if(!(2097152&b)&&le(He))return e;e=Fn(284,e);break;case 21:var n;de(21),wn()?((n=ye(178,e.pos)).objectType=e,n.indexType=jn(),de(22),e=Ee(n)):((n=ye(167,e.pos)).elementType=e,de(22),e=Ee(n));break;default:return e}return e}()}function Vn(e,n,t){me(t);var r=n();if(te()===t){for(var a=[r];me(t);)a.push(n());var i=ye(e,r.pos);i.types=be(a,r.pos),r=Ee(i)}return r}function Kn(){return Vn(172,Gn,48)}function Bn(){if(re(),20===te()||24===te())return!0;if(function(){if(e.isModifierKind(te())&&gr(),ue()||99===te())return re(),!0;if(21===te()||17===te()){var n=s.length;return rr(),n===s.length}return!1}()){if(56===te()||26===te()||55===te()||58===te())return!0;if(20===te()&&(re(),36===te()))return!0}return!1}function Hn(){var e=ue()&&ce(Un),n=jn();if(e){var t=ye(161,e.pos);return t.parameterName=e,t.type=n,Ee(t)}return n}function Un(){var e=xe();if(127===te()&&!x.hasPrecedingLineBreak())return re(),e}function jn(){return B(20480,Wn)}function Wn(e){if(27===te()||19===te()&&le(Bn)||94===te())return function(){var e=ne(),n=he(me(94)?164:163,e);return fn(36,4,n),Ee(n)}();var n=Vn(171,Kn,49);if(!e&&!x.hasPrecedingLineBreak()&&me(85)){var t=ye(173,n.pos);return t.checkType=n,t.extendsType=Wn(!0),de(55),t.trueType=Wn(),de(56),t.falseType=Wn(),Ee(t)}return n}function qn(){return me(56)?jn():void 0}function zn(){switch(te()){case 99:case 97:case 95:case 101:case 86:case 8:case 9:case 13:case 14:case 19:case 21:case 17:case 89:case 75:case 94:case 41:case 63:case 71:return!0;case 91:return le(Ln);default:return ue()}}function Jn(){if(zn())return!0;switch(te()){case 37:case 38:case 52:case 51:case 80:case 103:case 105:case 43:case 44:case 27:case 121:case 116:return!0;default:return!!function(){if(z()&&92===te())return!1;return e.getBinaryOperatorPrecedence(te())>0}()||ue()}}function Xn(){var e=J();e&&V(!1);for(var n,t=Qn();n=pe(26);)t=st(t,n,Qn());return e&&V(!0),t}function Yn(){return me(58)?Qn():void 0}function Qn(){if(function(){if(116===te())return!!q()||le(Ut);return!1}())return n=ye(205),re(),x.hasPrecedingLineBreak()||39!==te()&&!Jn()?Ee(n):(n.asteriskToken=pe(39),n.expression=Qn(),Ee(n));var n,t=function(){var n=function(){if(19===te()||27===te()||120===te())return le($n);if(36===te())return 1;return 0}();if(0===n)return;var t=1===n?tt(!0):ce(et);if(!t)return;var r=e.hasModifier(t,256),a=te();return t.equalsGreaterThanToken=fe(36),t.body=36===a||17===a?rt(r):xe(),Ee(t)}()||function(){if(120===te()&&1===le(nt)){var e=_r(),n=at(0);return Zn(n,e)}return}();if(t)return t;var r=at(0);return 71===r.kind&&36===te()?Zn(r):e.isLeftHandSideExpression(r)&&e.isAssignmentOperator(ae())?st(r,ge(),Qn()):function(n){var t=pe(55);if(!t)return n;var r=ye(203,n.pos);return r.condition=n,r.questionToken=t,r.whenTrue=B(A,Qn),r.colonToken=fe(56),r.whenFalse=e.nodeIsPresent(r.colonToken)?Qn():Te(71,!1,e.Diagnostics._0_expected,e.tokenToString(56)),Ee(r)}(r)}function Zn(n,t){var r;e.Debug.assert(36===te(),\"parseSimpleArrowFunctionExpression should only have been called if we had a =>\"),t?(r=ye(195,t.pos)).modifiers=t:r=ye(195,n.pos);var a=ye(149,n.pos);return a.name=n,Ee(a),r.parameters=be([a],a.pos,a.end),r.equalsGreaterThanToken=fe(36),r.body=rt(!!t),R(Ee(r))}function $n(){if(120===te()){if(re(),x.hasPrecedingLineBreak())return 0;if(19!==te()&&27!==te())return 0}var n=te(),t=re();if(19===n){if(20===t)switch(re()){case 36:case 56:case 17:return 1;default:return 0}if(21===t||17===t)return 2;if(24===t)return 1;if(e.isModifierKind(t)&&120!==t&&le(Fe))return 1;if(!ue()&&99!==t)return 0;switch(re()){case 56:return 1;case 55:return re(),56===te()||26===te()||58===te()||20===te()?1:0;case 26:case 58:case 20:return 2}return 0}return e.Debug.assert(27===n),ue()?1===o.languageVariant?le(function(){var e=re();if(85===e)switch(re()){case 58:case 29:return!1;default:return!0}else if(26===e)return!0;return!1})?1:0:2:0}function et(){return tt(!1)}function nt(){if(120===te()){if(re(),x.hasPrecedingLineBreak()||36===te())return 0;var e=at(0);if(!x.hasPrecedingLineBreak()&&71===e.kind&&36===te())return 1}return 0}function tt(n){var t=he(195);if(t.modifiers=_r(),(fn(56,e.hasModifier(t,256)?2:0,t)||n)&&(n||36===te()||17===te()))return t}function rt(e){return 17===te()?wt(e?2:0):25===te()||89===te()||75===te()||!qt()||17!==te()&&89!==te()&&75!==te()&&57!==te()&&Jn()?e?j(Qn):B(16384,Qn):wt(16|(e?2:0))}function at(e){return ot(e,ut())}function it(e){return 92===e||145===e}function ot(n,t){for(;;){ae();var r=e.getBinaryOperatorPrecedence(te());if(!(40===te()?r>=n:r>n))break;if(92===te()&&z())break;if(118===te()){if(x.hasPrecedingLineBreak())break;re(),t=lt(t,jn())}else t=st(t,ge(),at(r))}return t}function st(e,n,t){var r=ye(202,e.pos);return r.left=e,r.operatorToken=n,r.right=t,Ee(r)}function lt(e,n){var t=ye(210,e.pos);return t.expression=e,t.type=n,Ee(t)}function ct(){var e=ye(200);return e.operator=te(),re(),e.operand=dt(),Ee(e)}function ut(){if(function(){switch(te()){case 37:case 38:case 52:case 51:case 80:case 103:case 105:case 121:return!1;case 27:if(1!==o.languageVariant)return!1;default:return!0}}()){var n=mt();return 40===te()?ot(e.getBinaryOperatorPrecedence(te()),n):n}var t=te(),r=dt();if(40===te()){var a=e.skipTrivia(g,r.pos),i=r.end;192===r.kind?Z(a,i,e.Diagnostics.A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses):Z(a,i,e.Diagnostics.An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses,e.tokenToString(t))}return r}function dt(){switch(te()){case 37:case 38:case 52:case 51:return ct();case 80:return e=ye(196),re(),e.expression=dt(),Ee(e);case 103:return function(){var e=ye(197);return re(),e.expression=dt(),Ee(e)}();case 105:return function(){var e=ye(198);return re(),e.expression=dt(),Ee(e)}();case 27:return function(){var e=ye(192);return de(27),e.type=jn(),de(29),e.expression=dt(),Ee(e)}();case 121:if(121===te()&&(X()||le(Ut)))return function(){var e=ye(199);return re(),e.expression=dt(),Ee(e)}();default:return mt()}var e}function mt(){if(43===te()||44===te())return(n=ye(200)).operator=te(),re(),n.operand=pt(),Ee(n);if(1===o.languageVariant&&27===te()&&le(Ve))return gt(!0);var n,t=pt();return e.Debug.assert(e.isLeftHandSideExpression(t)),43!==te()&&44!==te()||x.hasPrecedingLineBreak()?t:((n=ye(201,t.pos)).operand=t,n.operator=te(),re(),Ee(n))}function pt(){var n;if(91===te())if(le(Tn))o.flags|=524288,n=ge();else if(le(Sn)){var t=x.getStartPos();re(),re();var r=ye(212,t);r.keywordToken=91,r.name=Ae(),n=Ee(r),o.flags|=1048576}else n=ft();else n=97===te()?function(){var n=ge();if(19===te()||23===te()||21===te())return n;var t=ye(187,n.pos);return t.expression=n,fe(23,e.Diagnostics.super_must_be_followed_by_an_argument_list_or_member_access),t.name=$e(!0),Ee(t)}():ft();return function(e){for(;;)if(e=Et(e),27!==te()){if(19!==te())return e;var n=ye(189,e.pos);n.expression=e,n.arguments=Lt(),e=Ee(n)}else{var t=ce(xt);if(!t)return e;if(Tt()){e=St(e,t);continue}var n=ye(189,e.pos);n.expression=e,n.typeArguments=t,n.arguments=Lt(),e=Ee(n)}}(n)}function ft(){return Et(At())}function gt(n){var t,r=function(e){var n=x.getStartPos();if(de(27),29===te()){var t=ye(263,n);return oe(),Ee(t)}var r,a=yt(),i=Sr(),o=(s=ye(266),s.properties=je(13,bt),Ee(s));var s;29===te()?(r=ye(260,n),oe()):(de(41),e?de(29):(de(29,void 0,!1),oe()),r=ye(259,n));return r.tagName=a,r.typeArguments=i,r.attributes=o,Ee(r)}(n);if(260===r.kind)(a=ye(258,r.pos)).openingElement=r,a.children=vt(a.openingElement),a.closingElement=function(e){var n=ye(261);de(28),n.tagName=yt(),e?de(29):(de(29,void 0,!1),oe());return Ee(n)}(n),E(a.openingElement.tagName,a.closingElement.tagName)||$(a.closingElement,e.Diagnostics.Expected_corresponding_JSX_closing_tag_for_0,e.getTextOfNodeFromSourceText(g,a.openingElement.tagName)),t=Ee(a);else if(263===r.kind){var a;(a=ye(262,r.pos)).openingFragment=r,a.children=vt(a.openingFragment),a.closingFragment=function(n){var t=ye(264);de(28),e.tokenIsIdentifierOrKeyword(te())&&$(yt(),e.Diagnostics.Expected_corresponding_closing_tag_for_JSX_fragment);n?de(29):(de(29,void 0,!1),oe());return Ee(t)}(n),t=Ee(a)}else e.Debug.assert(259===r.kind),t=r;if(n&&27===te()){var i=ce(function(){return gt(!0)});if(i){Y(e.Diagnostics.JSX_expressions_must_have_one_parent_element);var o=ye(202,t.pos);return o.end=i.end,o.left=t,o.right=i,o.operatorToken=Te(26,!1,void 0),o.operatorToken.pos=o.operatorToken.end=o.right.pos,o}}return t}function _t(n,t){switch(t){case 1:return void(e.isJsxOpeningFragment(n)?$(n,e.Diagnostics.JSX_fragment_has_no_corresponding_closing_tag):$(n.tagName,e.Diagnostics.JSX_element_0_has_no_corresponding_closing_tag,e.getTextOfNodeFromSourceText(g,n.tagName)));case 28:case 7:return;case 10:case 11:return(r=ye(10)).containsOnlyWhiteSpaces=11===c,c=x.scanJsxToken(),Ee(r);case 17:return ht(!1);case 27:return gt(!1);default:return e.Debug.assertNever(t)}var r}function vt(e){var n=[],t=ne(),r=h;for(h|=16384;;){var a=_t(e,c=x.reScanJsxToken());if(!a)break;n.push(a)}return h=r,be(n,t)}function yt(){ie();for(var e=99===te()?ge():Ae();me(23);){var n=ye(187,e.pos);n.expression=e,n.name=$e(!0),e=Ee(n)}return e}function ht(e){var n=ye(268);if(de(17))return 18!==te()&&(n.dotDotDotToken=pe(24),n.expression=Qn()),e?de(18):(de(18,void 0,!1),oe()),Ee(n)}function bt(){if(17===te())return function(){var e=ye(267);return de(17),de(24),e.expression=Xn(),de(18),Ee(e)}();ie();var e=ye(265);if(e.name=Ae(),58===te())switch(c=x.scanJsxAttributeValue()){case 9:e.initializer=tn();break;default:e.initializer=ht(!0)}return Ee(e)}function Et(n){for(;;){if(pe(23)){var t=ye(187,n.pos);t.expression=n,t.name=$e(!0),n=Ee(t)}else if(51!==te()||x.hasPrecedingLineBreak())if(J()||!me(21)){if(!Tt())return n;n=St(n,void 0)}else{var r=ye(188,n.pos);if(r.expression=n,22===te())r.argumentExpression=Te(71,!0,e.Diagnostics.An_element_access_expression_should_take_an_argument);else{var a=U(Xn);e.isStringOrNumericLiteralLike(a)&&(a.text=Se(a.text)),r.argumentExpression=a}de(22),n=Ee(r)}else{re();var i=ye(211,n.pos);i.expression=n,n=Ee(i)}}}function Tt(){return 13===te()||14===te()}function St(e,n){var t=ye(191,e.pos);return t.tag=e,t.typeArguments=n,t.template=13===te()?tn():en(),Ee(t)}function Lt(){de(19);var e=Je(11,Dt);return de(20),e}function xt(){if(me(27)){var e=Je(20,jn);if(de(29))return e&&function(){switch(te()){case 19:case 13:case 14:case 23:case 20:case 22:case 56:case 25:case 55:case 32:case 34:case 33:case 35:case 53:case 54:case 50:case 48:case 49:case 18:case 1:return!0;case 26:case 17:default:return!1}}()?e:void 0}}function At(){switch(te()){case 8:case 9:case 13:return tn();case 99:case 97:case 95:case 101:case 86:return ge();case 19:return n=he(193),de(19),n.expression=U(Xn),de(20),Ee(n);case 21:return kt();case 17:return Ot();case 120:if(!le(Ht))break;return It();case 75:return hr(he(0),207);case 89:return It();case 94:return function(){var n=x.getStartPos();if(de(94),me(23)){var t=ye(212,n);return t.keywordToken=94,t.name=Ae(),Ee(t)}var r,a=At();for(;;){a=Et(a),r=ce(xt),Tt()&&(e.Debug.assert(!!r,\"Expected a type argument list; all plain tagged template starts should be consumed in 'parseMemberExpressionRest'\"),a=St(a,r),r=void 0);break}var i=ye(190,n);i.expression=a,i.typeArguments=r,(i.typeArguments||19===te())&&(i.arguments=Lt());return Ee(i)}();case 41:case 63:if(12===(c=x.reScanSlashToken()))return tn();break;case 14:return en()}var n;return xe(e.Diagnostics.Expression_expected)}function Ct(){return 24===te()?(e=ye(206),de(24),e.expression=Qn(),Ee(e)):26===te()?ye(208):Qn();var e}function Dt(){return B(A,Ct)}function kt(){var e=ye(185);return de(21),x.hasPrecedingLineBreak()&&(e.multiLine=!0),e.elements=Je(15,Ct),de(22),Ee(e)}function Mt(){var e=he(0);if(pe(24))return e.kind=275,e.expression=Qn(),Ee(e);if(e.decorators=fr(),e.modifiers=gr(),Me(125))return mr(e,156);if(Me(136))return mr(e,157);var n=pe(39),t=ue();if(e.name=ke(),e.questionToken=pe(55),e.exclamationToken=pe(51),n||19===te()||27===te())return ur(e,n);if(t&&(26===te()||18===te()||58===te())){e.kind=274;var r=pe(58);r&&(e.equalsToken=r,e.objectAssignmentInitializer=U(Qn))}else e.kind=273,de(56),e.initializer=U(Qn);return Ee(e)}function Ot(){var e=ye(186);return de(17),x.hasPrecedingLineBreak()&&(e.multiLine=!0),e.properties=Je(12,Mt,!0),de(18),Ee(e)}function It(){var n=J();n&&V(!1);var t=he(194);t.modifiers=gr(),de(89),t.asteriskToken=pe(39);var r=t.asteriskToken?1:0,a=e.hasModifier(t,256)?2:0;return t.name=r&&a?H(20480,Rt):r?function(e){return H(4096,e)}(Rt):a?j(Rt):Rt(),fn(56,r|a,t),t.body=wt(r|a),n&&V(!0),Ee(t)}function Rt(){return ue()?xe():void 0}function Nt(e,n){var t=ye(216);return de(17,n)||e?(x.hasPrecedingLineBreak()&&(t.multiLine=!0),t.statements=je(1,Jt),de(18)):t.statements=Xe(),Ee(t)}function wt(e,n){var t=q();G(!!(1&e));var r=X();K(!!(2&e));var a=J();a&&V(!1);var i=Nt(!!(16&e),n);return a&&V(!0),G(t),K(r),i}function Pt(){var e=ne();de(88);var n,t,r=pe(121);if(de(19),25!==te()&&(n=104===te()||110===te()||76===te()?or(!0):H(2048,Xn)),r?de(145):me(145)){var a=ye(225,e);a.awaitModifier=r,a.initializer=n,a.expression=U(Qn),de(20),t=a}else if(me(92)){var i=ye(224,e);i.initializer=n,i.expression=U(Xn),de(20),t=i}else{var o=ye(223,e);o.initializer=n,de(25),25!==te()&&20!==te()&&(o.condition=U(Xn)),de(25),20!==te()&&(o.incrementor=U(Xn)),de(20),t=o}return t.statement=Jt(),Ee(t)}function Ft(e){var n=ye(e);return de(227===e?72:77),_e()||(n.label=xe()),ve(),Ee(n)}function Gt(){return 73===te()?(e=ye(269),de(73),e.expression=U(Xn),de(56),e.statements=je(3,Jt),Ee(e)):function(){var e=ye(270);return de(79),de(56),e.statements=je(3,Jt),Ee(e)}();var e}function Vt(){var e=ye(233);return de(102),e.tryBlock=Nt(!1),e.catchClause=74===te()?function(){var e=ye(272);de(74),me(19)?(e.variableDeclaration=ir(),de(20)):e.variableDeclaration=void 0;return e.block=Nt(!1),Ee(e)}():void 0,e.catchClause&&87!==te()||(de(87),e.finallyBlock=Nt(!1)),Ee(e)}function Kt(){return re(),e.tokenIsIdentifierOrKeyword(te())&&!x.hasPrecedingLineBreak()}function Bt(){return re(),75===te()&&!x.hasPrecedingLineBreak()}function Ht(){return re(),89===te()&&!x.hasPrecedingLineBreak()}function Ut(){return re(),(e.tokenIsIdentifierOrKeyword(te())||8===te()||9===te())&&!x.hasPrecedingLineBreak()}function jt(){for(;;)switch(te()){case 104:case 110:case 76:case 89:case 75:case 83:return!0;case 109:case 139:return re(),!x.hasPrecedingLineBreak()&&ue();case 129:case 130:return Zt();case 117:case 120:case 124:case 112:case 113:case 114:case 132:if(re(),x.hasPrecedingLineBreak())return!1;continue;case 144:return re(),17===te()||71===te()||84===te();case 91:return re(),9===te()||39===te()||17===te()||e.tokenIsIdentifierOrKeyword(te());case 84:if(re(),58===te()||39===te()||17===te()||79===te()||118===te())return!0;continue;case 115:re();continue;default:return!1}}function Wt(){return le(jt)}function qt(){switch(te()){case 57:case 25:case 17:case 104:case 110:case 89:case 75:case 83:case 90:case 81:case 106:case 88:case 77:case 72:case 96:case 107:case 98:case 100:case 102:case 78:case 74:case 87:return!0;case 91:return Wt()||le(Ln);case 76:case 84:return Wt();case 120:case 124:case 109:case 129:case 130:case 139:case 144:return!0;case 114:case 112:case 113:case 115:case 132:return Wt()||!le(Kt);default:return Jn()}}function zt(){return re(),ue()||17===te()||21===te()}function Jt(){switch(te()){case 25:return e=ye(218),de(25),Ee(e);case 17:return Nt(!1);case 104:return lr(he(235));case 110:if(le(zt))return lr(he(235));break;case 89:return cr(he(237));case 75:return yr(he(238));case 90:return function(){var e=ye(220);return de(90),de(19),e.expression=U(Xn),de(20),e.thenStatement=Jt(),e.elseStatement=me(82)?Jt():void 0,Ee(e)}();case 81:return function(){var e=ye(221);return de(81),e.statement=Jt(),de(106),de(19),e.expression=U(Xn),de(20),me(25),Ee(e)}();case 106:return function(){var e=ye(222);return de(106),de(19),e.expression=U(Xn),de(20),e.statement=Jt(),Ee(e)}();case 88:return Pt();case 77:return Ft(226);case 72:return Ft(227);case 96:return function(){var e=ye(228);return de(96),_e()||(e.expression=U(Xn)),ve(),Ee(e)}();case 107:return function(){var e=ye(229);return de(107),de(19),e.expression=U(Xn),de(20),e.statement=H(8388608,Jt),Ee(e)}();case 98:return function(){var e=ye(230);de(98),de(19),e.expression=U(Xn),de(20);var n=ye(244);return de(17),n.clauses=je(2,Gt),de(18),e.caseBlock=Ee(n),Ee(e)}();case 100:return function(){var e=ye(232);return de(100),e.expression=x.hasPrecedingLineBreak()?void 0:U(Xn),ve(),Ee(e)}();case 102:case 74:case 87:return Vt();case 78:return function(){var e=ye(234);return de(78),ve(),Ee(e)}();case 57:return Yt();case 120:case 109:case 139:case 129:case 130:case 124:case 76:case 83:case 84:case 91:case 112:case 113:case 114:case 117:case 115:case 132:case 144:if(Wt())return Yt()}var e;return function(){var e=he(0),n=U(Xn);return 71===n.kind&&me(56)?(e.kind=231,e.label=n,e.statement=Jt()):(e.kind=219,e.expression=n,ve()),Ee(e)}()}function Xt(e){return 124===e.kind}function Yt(){var n=he(0);if(n.decorators=fr(),n.modifiers=gr(),e.some(n.modifiers,Xt)){for(var t=0,r=n.modifiers;t<r.length;t++){r[t].flags|=4194304}return H(4194304,function(){return Qt(n)})}return Qt(n)}function Qt(n){switch(te()){case 104:case 110:case 76:return lr(n);case 89:return cr(n);case 75:return yr(n);case 109:return function(e){return e.kind=239,de(109),e.name=xe(),e.typeParameters=un(),e.heritageClauses=br(),e.members=xn(),Ee(e)}(n);case 139:return function(e){return e.kind=240,de(139),e.name=xe(),e.typeParameters=un(),de(58),e.type=jn(),ve(),Ee(e)}(n);case 83:return function(e){e.kind=241,de(83),e.name=xe(),de(17)?(e.members=Je(6,xr),de(18)):e.members=Xe();return Ee(e)}(n);case 144:case 129:case 130:return function(e){var n=0;if(144===te())return Cr(e);if(me(130))n|=16;else if(de(129),9===te())return Cr(e);return function e(n,t){n.kind=242;var r=16&t;n.flags|=t;n.name=xe();n.body=me(23)?e(ye(0),4|r):Ar();return Ee(n)}(e,n)}(n);case 91:return function(e){de(91);var n,t=x.getStartPos();if(ue()&&(n=xe(),26!==te()&&143!==te()))return function(e,n){return e.kind=246,e.name=n,de(58),e.moduleReference=function(){return 133===te()&&le(Dr)?(e=ye(257),de(133),de(19),e.expression=Mr(),de(20),Ee(e)):Qe(!1);var e}(),ve(),Ee(e)}(e,n);e.kind=247,(n||39===te()||17===te())&&(e.importClause=function(e,n){var t=ye(248,n);e&&(t.name=e);t.name&&!me(26)||(t.namedBindings=39===te()?(r=ye(249),de(39),de(118),r.name=xe(),Ee(r)):Or(250));var r;return Ee(t)}(n,t),de(143));return e.moduleSpecifier=Mr(),ve(),Ee(e)}(n);case 84:switch(re(),te()){case 79:case 58:return function(e){e.kind=252,me(58)?e.isExportEquals=!0:de(79);return e.expression=Qn(),ve(),Ee(e)}(n);case 118:return function(e){return e.kind=245,de(118),de(130),e.name=xe(),ve(),Ee(e)}(n);default:return function(e){e.kind=253,me(39)?(de(143),e.moduleSpecifier=Mr()):(e.exportClause=Or(254),(143===te()||9===te()&&!x.hasPrecedingLineBreak())&&(de(143),e.moduleSpecifier=Mr()));return ve(),Ee(e)}(n)}default:if(n.decorators||n.modifiers){var t=Te(256,!0,e.Diagnostics.Declaration_expected);return t.pos=n.pos,t.decorators=n.decorators,t.modifiers=n.modifiers,Ee(t)}return}}function Zt(){return re(),!x.hasPrecedingLineBreak()&&(ue()||9===te())}function $t(e,n){if(17===te()||!_e())return wt(e,n);ve()}function er(){if(26===te())return ye(208);var e=ye(184);return e.dotDotDotToken=pe(24),e.name=rr(),e.initializer=Yn(),Ee(e)}function nr(){var e=ye(184);e.dotDotDotToken=pe(24);var n=ue(),t=ke();return n&&56!==te()?e.name=t:(de(56),e.propertyName=t,e.name=rr()),e.initializer=Yn(),Ee(e)}function tr(){return 17===te()||21===te()||ue()}function rr(){return 21===te()?(e=ye(183),de(21),e.elements=Je(10,er),de(22),Ee(e)):17===te()?function(){var e=ye(182);return de(17),e.elements=Je(9,nr),de(18),Ee(e)}():xe();var e}function ar(){return ir(!0)}function ir(e){var n=ye(235);return n.name=rr(),e&&71===n.name.kind&&51===te()&&!x.hasPrecedingLineBreak()&&(n.exclamationToken=ge()),n.type=qn(),it(te())||(n.initializer=Yn()),Ee(n)}function or(n){var t=ye(236);switch(te()){case 104:break;case 110:t.flags|=1;break;case 76:t.flags|=2;break;default:e.Debug.fail()}if(re(),145===te()&&le(sr))t.declarations=Xe();else{var r=z();F(n),t.declarations=Je(8,n?ir:ar),F(r)}return Ee(t)}function sr(){return Fe()&&20===re()}function lr(e){return e.kind=217,e.declarationList=or(!1),ve(),Ee(e)}function cr(n){n.kind=237,de(89),n.asteriskToken=pe(39),n.name=e.hasModifier(n,512)?Rt():xe();var t=n.asteriskToken?1:0,r=e.hasModifier(n,256)?2:0;return fn(56,t|r,n),n.body=$t(t|r,e.Diagnostics.or_expected),Ee(n)}function ur(n,t,r){n.kind=154,n.asteriskToken=t;var a=t?1:0,i=e.hasModifier(n,256)?2:0;return fn(56,a|i,n),n.body=$t(a|i,r),Ee(n)}function dr(n){return n.kind=152,n.questionToken||51!==te()||x.hasPrecedingLineBreak()||(n.exclamationToken=ge()),n.type=qn(),n.initializer=e.hasModifier(n,32)?U(Yn):B(6144,Yn),ve(),Ee(n)}function mr(e,n){return e.kind=n,e.name=ke(),fn(56,0,e),e.body=$t(0),Ee(e)}function pr(){var n;if(57===te())return!0;for(;e.isModifierKind(te());){if(n=te(),e.isClassMemberModifier(n))return!0;re()}if(39===te())return!0;if(Ce()&&(n=te(),re()),21===te())return!0;if(void 0!==n){if(!e.isKeyword(n)||136===n||125===n)return!0;switch(te()){case 19:case 27:case 51:case 56:case 58:case 55:return!0;default:return _e()}}return!1}function fr(){for(var e,n=ne();;){var t=ne();if(!me(57))break;var r=ye(150,t);r.expression=H(8192,pt),Ee(r),(e||(e=[])).push(r)}return e&&be(e,n)}function gr(n){for(var t,r=ne();;){var a=x.getStartPos(),i=te();if(76===te()&&n){if(!ce(Oe))break}else if(!e.isModifierKind(te())||!ce(Ie))break;var o=Ee(ye(i,a));(t||(t=[])).push(o)}return t&&be(t,r)}function _r(){var e;if(120===te()){var n=x.getStartPos(),t=te();re(),e=be([Ee(ye(t,n))],n)}return e}function vr(){if(25===te()){var n=ye(215);return re(),Ee(n)}var t=he(0);return t.decorators=fr(),t.modifiers=gr(!0),Me(125)?mr(t,156):Me(136)?mr(t,157):123===te()?function(n){return n.kind=155,de(123),fn(56,0,n),n.body=$t(0,e.Diagnostics.or_expected),Ee(n)}(t):vn()?hn(t):e.tokenIsIdentifierOrKeyword(te())||9===te()||8===te()||39===te()||21===te()?function(n){var t=pe(39);return n.name=ke(),n.questionToken=pe(55),t||19===te()||27===te()?ur(n,t,e.Diagnostics.or_expected):dr(n)}(t):t.decorators||t.modifiers?(t.name=Te(71,!0,e.Diagnostics.Declaration_expected),dr(t)):e.Debug.fail(\"Should not have attempted to parse class member declaration.\")}function yr(e){return hr(e,238)}function hr(e,n){return e.kind=n,de(75),e.name=!ue()||108===te()&&le(Ge)?void 0:xe(),e.typeParameters=un(),e.heritageClauses=br(),de(17)?(e.members=je(5,vr),de(18)):e.members=Xe(),Ee(e)}function br(){if(Lr())return je(22,Er)}function Er(){var n=te();e.Debug.assert(85===n||108===n);var t=ye(271);return t.token=n,re(),t.types=Je(7,Tr),Ee(t)}function Tr(){var e=ye(209);return e.expression=pt(),e.typeArguments=Sr(),Ee(e)}function Sr(){return 27===te()?Ye(20,jn,27,29):void 0}function Lr(){return 85===te()||108===te()}function xr(){var e=he(276);return e.name=ke(),e.initializer=U(Yn),Ee(e)}function Ar(){var e=ye(243);return de(17)?(e.statements=je(1,Jt),de(18)):e.statements=Xe(),Ee(e)}function Cr(e){return e.kind=242,144===te()?(e.name=xe(),e.flags|=512):(e.name=tn(),e.name.text=Se(e.name.text)),17===te()?e.body=Ar():ve(),Ee(e)}function Dr(){return 19===re()}function kr(){return 41===re()}function Mr(){if(9===te()){var e=tn();return e.text=Se(e.text),e}return Xn()}function Or(e){var n=ye(e);return n.elements=Ye(23,250===e?Rr:Ir,17,18),Ee(n)}function Ir(){return Nr(255)}function Rr(){return Nr(251)}function Nr(n){var t=ye(n),r=e.isKeyword(te())&&!ue(),a=x.getTokenPos(),i=x.getTextPos(),o=Ae();return 118===te()?(t.propertyName=o,de(118),r=e.isKeyword(te())&&!ue(),a=x.getTokenPos(),i=x.getTextPos(),t.name=Ae()):t.name=o,251===n&&r&&Z(a,i,e.Diagnostics.Identifier_expected),Ee(t)}function wr(n){return e.hasModifier(n,1)||246===n.kind&&257===n.moduleReference.kind||247===n.kind||252===n.kind||253===n.kind?n:void 0}function Pr(n){return function(n){return e.isMetaProperty(n)&&91===n.keywordToken&&\"meta\"===n.name.escapedText}(n)?n:d(n,Pr)}n.parseSourceFile=function(n,t,r,a,i,o){if(void 0===i&&(i=!1),6===(o=e.ensureScriptKind(n,o))){var s=D(n,t,r,a,i);return e.convertToObjectWorker(s,s.parseDiagnostics,!1,void 0,void 0),s.referencedFiles=e.emptyArray,s.typeReferenceDirectives=e.emptyArray,s.libReferenceDirectives=e.emptyArray,s.amdDependencies=e.emptyArray,s.hasNoDefaultLib=!1,s.pragmas=e.emptyMap,s}M(t,r,a,o);var l=I(n,r,i,o);return O(),l},n.parseIsolatedEntityName=function(e,n){M(e,n,void 0,1),re();var t=Qe(!0),r=1===te()&&!s.length;return O(),r?t:void 0},n.parseJsonText=D,n.fixupParentReferences=N,function(e){e[e.SourceElements=0]=\"SourceElements\",e[e.BlockStatements=1]=\"BlockStatements\",e[e.SwitchClauses=2]=\"SwitchClauses\",e[e.SwitchClauseStatements=3]=\"SwitchClauseStatements\",e[e.TypeMembers=4]=\"TypeMembers\",e[e.ClassMembers=5]=\"ClassMembers\",e[e.EnumMembers=6]=\"EnumMembers\",e[e.HeritageClauseElement=7]=\"HeritageClauseElement\",e[e.VariableDeclarations=8]=\"VariableDeclarations\",e[e.ObjectBindingElements=9]=\"ObjectBindingElements\",e[e.ArrayBindingElements=10]=\"ArrayBindingElements\",e[e.ArgumentExpressions=11]=\"ArgumentExpressions\",e[e.ObjectLiteralMembers=12]=\"ObjectLiteralMembers\",e[e.JsxAttributes=13]=\"JsxAttributes\",e[e.JsxChildren=14]=\"JsxChildren\",e[e.ArrayLiteralMembers=15]=\"ArrayLiteralMembers\",e[e.Parameters=16]=\"Parameters\",e[e.JSDocParameters=17]=\"JSDocParameters\",e[e.RestProperties=18]=\"RestProperties\",e[e.TypeParameters=19]=\"TypeParameters\",e[e.TypeArguments=20]=\"TypeArguments\",e[e.TupleElementTypes=21]=\"TupleElementTypes\",e[e.HeritageClauses=22]=\"HeritageClauses\",e[e.ImportOrExportSpecifiers=23]=\"ImportOrExportSpecifiers\",e[e.Count=24]=\"Count\"}(T||(T={})),function(e){e[e.False=0]=\"False\",e[e.True=1]=\"True\",e[e.Unknown=2]=\"Unknown\"}(S||(S={})),function(n){function t(e){var n=ye(281),t=(e?me:de)(17);return n.type=H(2097152,ln),e&&!t||de(18),N(n),Ee(n)}var r,a;function i(n,r){void 0===n&&(n=0);var a=g,i=void 0===r?a.length:n+r;if(r=i-n,e.Debug.assert(n>=0),e.Debug.assert(n<=i),e.Debug.assert(i<=a.length),u(a,n)){var o,s,l,d=[];return x.scanRange(n+3,r-5,function(){var e,t,r=1,c=n-Math.max(a.lastIndexOf(\"\\n\",n),0)+4;function u(n){e||(e=c),d.push(n),c+=n.length}for(R();N(5););N(4)&&(r=0,c=0);e:for(;;){switch(te()){case 57:0===r||1===r?(p(d),b(y(c)),r=0,e=void 0,c++):u(x.getTokenText());break;case 4:d.push(x.getTokenText()),r=0,c=0;break;case 39:var f=x.getTokenText();1===r||2===r?(r=2,u(f)):(r=1,c+=f.length);break;case 71:u(x.getTokenText()),r=2;break;case 5:var g=x.getTokenText();2===r?d.push(g):void 0!==e&&c+g.length>e&&d.push(g.slice(e-c-1)),c+=g.length;break;case 1:break e;default:r=2,u(x.getTokenText())}R()}return m(d),p(d),(t=ye(289,n)).tags=o&&be(o,s,l),t.comment=d.length?d.join(\"\"):void 0,Ee(t,i)})}function m(e){for(;e.length&&(\"\\n\"===e[0]||\"\\r\"===e[0]);)e.shift()}function p(e){for(;e.length&&\"\"===e[e.length-1].trim();)e.pop()}function f(){for(;;){if(R(),1===te())return!0;if(5!==te()&&4!==te())return!1}}function _(){if(5!==te()&&4!==te()||!le(f))for(;5===te()||4===te();)R()}function v(){if(5!==te()&&4!==te()||!le(f))for(var e=x.hasPrecedingLineBreak();e&&39===te()||5===te()||4===te();)4===te()?e=!0:39===te()&&(e=!1),R()}function y(n){e.Debug.assert(57===te());var r=ye(57,x.getTokenPos());r.end=x.getTextPos(),R();var a,i=w(void 0);switch(v(),i.escapedText){case\"augments\":case\"extends\":a=function(e,n){var t=ye(293,e.pos);return t.atToken=e,t.tagName=n,t.class=function(){var e=me(17),n=ye(209);n.expression=function(){for(var e=w();me(23);){var n=ye(187,e.pos);n.expression=e,n.name=w(),e=Ee(n)}return e}(),n.typeArguments=Sr();var t=Ee(n);return e&&de(18),t}(),Ee(t)}(r,i);break;case\"class\":case\"constructor\":a=function(e,n){var t=ye(294,e.pos);return t.atToken=e,t.tagName=n,Ee(t)}(r,i);break;case\"this\":a=function(e,n){var r=ye(299,e.pos);return r.atToken=e,r.tagName=n,r.typeExpression=t(!0),_(),Ee(r)}(r,i);break;case\"enum\":a=function(e,n){var r=ye(296,e.pos);return r.atToken=e,r.tagName=n,r.typeExpression=t(!0),_(),Ee(r)}(r,i);break;case\"arg\":case\"argument\":case\"param\":return L(r,i,2,n);case\"return\":case\"returns\":a=function(n,t){e.forEach(o,function(e){return 298===e.kind})&&Z(t.pos,x.getTokenPos(),e.Diagnostics._0_tag_already_specified,t.escapedText);var r=ye(298,n.pos);return r.atToken=n,r.tagName=t,r.typeExpression=E(),Ee(r)}(r,i);break;case\"template\":a=function(n,r){var a;17===te()&&(a=t());var i=[],o=ne();do{_();var s=ye(148);s.name=w(e.Diagnostics.Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces),Ee(s),_(),i.push(s)}while(N(26));var l=ye(301,n.pos);return l.atToken=n,l.tagName=r,l.constraint=a,l.typeParameters=be(i,o),Ee(l),l}(r,i);break;case\"type\":a=A(r,i);break;case\"typedef\":a=function(n,t,r){var a=E();v();var i,o=ye(302,n.pos);if(o.atToken=n,o.tagName=t,o.fullName=C(),o.name=D(o.fullName),_(),o.comment=h(r),o.typeExpression=a,!a||S(a.type)){for(var s=void 0,l=void 0,c=void 0,u=n.pos;s=ce(function(){return M(r)});)if(l||(l=ye(290,u)),300===s.kind){if(c)break;c=s}else l.jsDocPropertyTags=e.append(l.jsDocPropertyTags,s);l&&(a&&167===a.type.kind&&(l.isArrayType=!0),o.typeExpression=c&&c.typeExpression&&!S(c.typeExpression.type)?c.typeExpression:Ee(l),i=o.typeExpression.end)}return Ee(o,i||void 0!==o.comment?x.getStartPos():(o.fullName||o.typeExpression||o.tagName).end)}(r,i,n);break;case\"callback\":a=function(n,t,r){var a,i=ye(295,n.pos);i.atToken=n,i.tagName=t,i.fullName=C(),i.name=D(i.fullName),_(),i.comment=h(r);var o=ye(291,x.getStartPos());o.parameters=[];for(;a=ce(function(){return O(4,r)});)o.parameters=e.append(o.parameters,a);var s=ce(function(){if(N(57)){var e=y(r);if(e&&298===e.kind)return e}});s&&(o.type=s);return i.typeExpression=Ee(o),Ee(i)}(r,i,n);break;default:a=function(e,n){var t=ye(292,e.pos);return t.atToken=e,t.tagName=n,Ee(t)}(r,i)}return a.comment||(a.comment=h(n+a.end-a.pos)),a}function h(n){var t,r=[],a=0;function i(e){t||(t=n),r.push(e),n+=e.length}var o=te();e:for(;;){switch(o){case 4:a>=1&&(a=0,r.push(x.getTokenText())),n=0;break;case 57:x.setTextPos(x.getTextPos()-1);case 1:break e;case 5:if(2===a)i(x.getTokenText());else{var s=x.getTokenText();void 0!==t&&n+s.length>t&&r.push(s.slice(t-n-1)),n+=s.length}break;case 17:a=2,le(function(){return 57===R()&&e.tokenIsIdentifierOrKeyword(R())&&\"link\"===x.getTokenText()})&&(i(x.getTokenText()),R(),i(x.getTokenText()),R()),i(x.getTokenText());break;case 39:if(0===a){a=1,n+=1;break}default:a=2,i(x.getTokenText())}o=R()}return m(r),p(r),0===r.length?void 0:r.join(\"\")}function b(e){e&&(o?o.push(e):(o=[e],s=e.pos),l=e.end)}function E(){return v(),17===te()?t():void 0}function T(){if(13===te())return{name:Le(!0),isBracketed:!1};var e=me(21),n=function(){var e=w();me(21)&&de(22);for(;me(23);){var n=w();me(21)&&de(22),e=Ze(e,n)}return e}();return e&&(_(),pe(58)&&Xn(),de(22)),{name:n,isBracketed:e}}function S(n){switch(n.kind){case 135:return!0;case 167:return S(n.elementType);default:return e.isTypeReferenceNode(n)&&e.isIdentifier(n.typeName)&&\"Object\"===n.typeName.escapedText}}function L(n,t,r,a){var i=E(),o=!i;v();var s=T(),l=s.name,c=s.isBracketed;_(),o&&(i=E());var u=ye(1===r?303:297,n.pos),d=h(a+x.getStartPos()-n.pos),m=4!==r&&function(n,t,r,a){if(n&&S(n.type)){for(var i=ye(281,x.getTokenPos()),o=void 0,s=void 0,l=x.getStartPos(),c=void 0;o=ce(function(){return O(r,a,t)});)297!==o.kind&&303!==o.kind||(c=e.append(c,o));if(c)return(s=ye(290,l)).jsDocPropertyTags=c,167===n.type.kind&&(s.isArrayType=!0),i.type=Ee(s),Ee(i)}}(i,l,r,a);return m&&(i=m,o=!0),u.atToken=n,u.tagName=t,u.typeExpression=i,u.name=l,u.isNameFirst=o,u.isBracketed=c,u.comment=d,Ee(u)}function A(n,r){e.forEach(o,function(e){return 300===e.kind})&&Z(r.pos,x.getTokenPos(),e.Diagnostics._0_tag_already_specified,r.escapedText);var a=ye(300,n.pos);return a.atToken=n,a.tagName=r,a.typeExpression=t(!0),Ee(a)}function C(n){var t=x.getTokenPos();if(e.tokenIsIdentifierOrKeyword(te())){var r=w();if(me(23)){var a=ye(242,t);return n&&(a.flags|=4),a.name=r,a.body=C(!0),Ee(a)}return n&&(r.isInJSDocNamespace=!0),r}}function D(n){if(n)for(var t=n;;){if(e.isIdentifier(t)||!t.body)return e.isIdentifier(t)?t:t.name;t=t.body}}function k(n,t){for(;!e.isIdentifier(n)||!e.isIdentifier(t);){if(e.isIdentifier(n)||e.isIdentifier(t)||n.right.escapedText!==t.right.escapedText)return!1;n=n.left,t=t.left}return n.escapedText===t.escapedText}function M(e){return O(1,e)}function O(n,t,r){for(var a=!0,i=!1;;)switch(R()){case 57:if(a){var o=I(n,t);return!(o&&(297===o.kind||303===o.kind)&&4!==n&&r&&(e.isIdentifier(o.name)||!k(r,o.name.left)))&&o}i=!1;break;case 4:a=!0,i=!1;break;case 39:i&&(a=!1),i=!0;break;case 71:a=!1;break;case 1:return!1}}function I(n,t){e.Debug.assert(57===te());var r=ye(57);r.end=x.getTextPos(),R();var a,i=w();switch(_(),i.escapedText){case\"type\":return 1===n&&A(r,i);case\"prop\":case\"property\":a=1;break;case\"arg\":case\"argument\":case\"param\":a=6;break;default:return!1}return!!(n&a)&&L(r,i,n,t)}function R(){return c=x.scanJSDocToken()}function N(e){return te()===e&&(R(),!0)}function w(n){if(!e.tokenIsIdentifierOrKeyword(te()))return Te(71,!n,n||e.Diagnostics.Identifier_expected);var t=x.getTokenPos(),r=x.getTextPos(),a=ye(71,t);return a.escapedText=e.escapeLeadingUnderscores(x.getTokenText()),Ee(a,r),R(),a}}n.parseJSDocTypeExpressionForTests=function(e,n,r){M(e,6,void 0,1),o=w(\"file.js\",6,1,!1),x.setText(e,n,r),c=x.scan();var a=t(),i=s;return O(),a?{jsDocTypeExpression:a,diagnostics:i}:void 0},n.parseJSDocTypeExpression=t,n.parseIsolatedJSDocComment=function(e,n,t){M(e,6,void 0,1),o={languageVariant:0,text:e};var r=i(n,t),a=s;return O(),r?{jsDoc:r,diagnostics:a}:void 0},n.parseJSDocComment=function(e,n,t){var r,a=c,l=s.length,u=C,d=i(n,t);return d&&(d.parent=e),65536&b&&(o.jsDocDiagnostics||(o.jsDocDiagnostics=[]),(r=o.jsDocDiagnostics).push.apply(r,s)),c=a,s.length=l,C=u,d},function(e){e[e.BeginningOfLine=0]=\"BeginningOfLine\",e[e.SawAsterisk=1]=\"SawAsterisk\",e[e.SavingComments=2]=\"SavingComments\"}(r||(r={})),function(e){e[e.Property=1]=\"Property\",e[e.Parameter=2]=\"Parameter\",e[e.CallbackParameter=4]=\"CallbackParameter\"}(a||(a={})),n.parseJSDocCommentWorker=i}(L=n.JSDocParser||(n.JSDocParser={}))}(o||(o={})),function(n){function t(n,t,a,o,s,l){return void(t?u(n):c(n));function c(n){var t=\"\";if(l&&r(n)&&(t=o.substring(n.pos,n.end)),n._children&&(n._children=void 0),n.pos+=a,n.end+=a,l&&r(n)&&e.Debug.assert(t===s.substring(n.pos,n.end)),d(n,c,u),e.hasJSDocNodes(n))for(var m=0,p=n.jsDoc;m<p.length;m++){c(p[m])}i(n,l)}function u(e){e._children=void 0,e.pos+=a,e.end+=a;for(var n=0,t=e;n<t.length;n++){c(t[n])}}}function r(e){switch(e.kind){case 9:case 8:case 71:return!0}return!1}function a(n,t,r,a,i){e.Debug.assert(n.end>=t,\"Adjusting an element that was entirely before the change range\"),e.Debug.assert(n.pos<=r,\"Adjusting an element that was entirely after the change range\"),e.Debug.assert(n.pos<=n.end),n.pos=Math.min(n.pos,a),n.end>=r?n.end+=i:n.end=Math.min(n.end,a),e.Debug.assert(n.pos<=n.end),n.parent&&(e.Debug.assert(n.pos>=n.parent.pos),e.Debug.assert(n.end<=n.parent.end))}function i(n,t){if(t){var r=n.pos,a=function(n){e.Debug.assert(n.pos>=r),r=n.end};if(e.hasJSDocNodes(n))for(var i=0,o=n.jsDoc;i<o.length;i++){a(o[i])}d(n,a),e.Debug.assert(r<=n.end)}}function s(n,t){var r,a=n;if(d(n,function n(i){if(e.nodeIsMissing(i))return;if(!(i.pos<=t))return e.Debug.assert(i.pos>t),!0;if(i.pos>=a.pos&&(a=i),t<i.end)return d(i,n),!0;e.Debug.assert(i.end<=t),r=i}),r){var i=function(n){for(;;){var t=e.getLastChild(n);if(!t)return n;n=t}}(r);i.pos>a.pos&&(a=i)}return a}function l(n,t,r,a){var i=n.text;if(r&&(e.Debug.assert(i.length-r.span.length+r.newLength===t.length),a||e.Debug.shouldAssert(3))){var o=i.substr(0,r.span.start),s=t.substr(0,r.span.start);e.Debug.assert(o===s);var l=i.substring(e.textSpanEnd(r.span),i.length),c=t.substring(e.textSpanEnd(e.textChangeRangeNewSpan(r)),t.length);e.Debug.assert(l===c)}}var c;n.updateSourceFile=function(n,r,c,u){if(l(n,r,c,u=u||e.Debug.shouldAssert(2)),e.textChangeRangeIsUnchanged(c))return n;if(0===n.statements.length)return o.parseSourceFile(n.fileName,r,n.languageVersion,void 0,!0,n.scriptKind);var m=n;e.Debug.assert(!m.hasBeenIncrementallyParsed),m.hasBeenIncrementallyParsed=!0;var p=n.text,f=function(n){var t=n.statements,r=0;e.Debug.assert(r<t.length);var a=t[r],i=-1;return{currentNode:function(o){return o!==i&&(a&&a.end===o&&r<t.length-1&&(a=t[++r]),a&&a.pos===o||function(e){return t=void 0,r=-1,a=void 0,void d(n,i,o);function i(n){return e>=n.pos&&e<n.end&&(d(n,i,o),!0)}function o(n){if(e>=n.pos&&e<n.end)for(var s=0;s<n.length;s++){var l=n[s];if(l){if(l.pos===e)return t=n,r=s,a=l,!0;if(l.pos<e&&e<l.end)return d(l,i,o),!0}}return!1}}(o)),i=o,e.Debug.assert(!a||a.pos===o),a}}}(n),g=function(n,t){for(var r=t.span.start,a=0;r>0&&a<=1;a++){var i=s(n,r);e.Debug.assert(i.pos<=r);var o=i.pos;r=Math.max(0,o-1)}var l=e.createTextSpanFromBounds(r,e.textSpanEnd(t.span)),c=t.newLength+(t.span.start-r);return e.createTextChangeRange(l,c)}(n,c);l(n,r,g,u),e.Debug.assert(g.span.start<=c.span.start),e.Debug.assert(e.textSpanEnd(g.span)===e.textSpanEnd(c.span)),e.Debug.assert(e.textSpanEnd(e.textChangeRangeNewSpan(g))===e.textSpanEnd(e.textChangeRangeNewSpan(c)));var _=e.textChangeRangeNewSpan(g).length-g.span.length;return function(n,r,o,s,l,c,u,m){return void p(n);function p(n){if(e.Debug.assert(n.pos<=n.end),n.pos>o)t(n,!1,l,c,u,m);else{var g=n.end;if(g>=r){if(n.intersectsChange=!0,n._children=void 0,a(n,r,o,s,l),d(n,p,f),e.hasJSDocNodes(n))for(var _=0,v=n.jsDoc;_<v.length;_++){var y=v[_];p(y)}i(n,m)}else e.Debug.assert(g<r)}}function f(n){if(e.Debug.assert(n.pos<=n.end),n.pos>o)t(n,!0,l,c,u,m);else{var i=n.end;if(i>=r){n.intersectsChange=!0,n._children=void 0,a(n,r,o,s,l);for(var d=0,f=n;d<f.length;d++){var g=f[d];p(g)}}else e.Debug.assert(i<r)}}}(m,g.span.start,e.textSpanEnd(g.span),e.textSpanEnd(e.textChangeRangeNewSpan(g)),_,p,r,u),o.parseSourceFile(n.fileName,r,n.languageVersion,f,!0,n.scriptKind)},function(e){e[e.Value=-1]=\"Value\"}(c||(c={}))}(s||(s={})),e.isDeclarationFileName=m,e.processCommentPragmas=p,e.processPragmasIntoFields=f;var g=e.createMap();function _(e){if(g.has(e))return g.get(e);var n=new RegExp(\"(\\\\s\"+e+\"\\\\s*=\\\\s*)('|\\\")(.+?)\\\\2\",\"im\");return g.set(e,n),n}var v=/^\\/\\/\\/\\s*<(\\S+)\\s.*?\\/>/im,y=/^\\/\\/\\/?\\s*@(\\S+)\\s*(.*)\\s*$/im;function h(n,t,r){var a=2===t.kind&&v.exec(r);if(a){var i=a[1].toLowerCase(),o=e.commentPragmas[i];if(!(o&&1&o.kind))return;if(o.args){for(var s={},l=0,c=o.args;l<c.length;l++){var u=c[l],d=_(u.name).exec(r);if(!d&&!u.optional)return;if(d)if(u.captureSpan){var m=t.pos+d.index+d[1].length+d[2].length;s[u.name]={value:d[3],pos:m,end:m+d[3].length}}else s[u.name]=d[3]}n.push({name:i,args:{arguments:s,range:t}})}else n.push({name:i,args:{arguments:{},range:t}})}else{var p=2===t.kind&&y.exec(r);if(p)return b(n,t,2,p);if(3===t.kind)for(var f=/\\s*@(\\S+)\\s*(.*)\\s*$/gim,g=void 0;g=f.exec(r);)b(n,t,4,g)}}function b(n,t,r,a){if(a){var i=a[1].toLowerCase(),o=e.commentPragmas[i];if(o&&o.kind&r){var s=function(n,t){if(!t)return{};if(!n.args)return{};for(var r=t.split(/\\s+/),a={},i=0;i<n.args.length;i++){var o=n.args[i];if(!r[i]&&!o.optional)return\"fail\";if(o.captureSpan)return e.Debug.fail(\"Capture spans not yet implemented for non-xml pragmas\");a[o.name]=r[i]}return a}(o,a[2]);\"fail\"!==s&&n.push({name:i,args:{arguments:s,range:t}})}}}function E(e,n){return e.kind===n.kind&&(71===e.kind?e.escapedText===n.escapedText:99===e.kind||e.name.escapedText===n.name.escapedText&&E(e.expression,n.expression))}e.tagNamesAreEquivalent=E}(Tn||(Tn={})),function(e){e.compileOnSaveCommandLineOption={name:\"compileOnSave\",type:\"boolean\"};var n,t,r=[[\"es5\",\"lib.es5.d.ts\"],[\"es6\",\"lib.es2015.d.ts\"],[\"es2015\",\"lib.es2015.d.ts\"],[\"es7\",\"lib.es2016.d.ts\"],[\"es2016\",\"lib.es2016.d.ts\"],[\"es2017\",\"lib.es2017.d.ts\"],[\"es2018\",\"lib.es2018.d.ts\"],[\"esnext\",\"lib.esnext.d.ts\"],[\"dom\",\"lib.dom.d.ts\"],[\"dom.iterable\",\"lib.dom.iterable.d.ts\"],[\"webworker\",\"lib.webworker.d.ts\"],[\"webworker.importscripts\",\"lib.webworker.importscripts.d.ts\"],[\"scripthost\",\"lib.scripthost.d.ts\"],[\"es2015.core\",\"lib.es2015.core.d.ts\"],[\"es2015.collection\",\"lib.es2015.collection.d.ts\"],[\"es2015.generator\",\"lib.es2015.generator.d.ts\"],[\"es2015.iterable\",\"lib.es2015.iterable.d.ts\"],[\"es2015.promise\",\"lib.es2015.promise.d.ts\"],[\"es2015.proxy\",\"lib.es2015.proxy.d.ts\"],[\"es2015.reflect\",\"lib.es2015.reflect.d.ts\"],[\"es2015.symbol\",\"lib.es2015.symbol.d.ts\"],[\"es2015.symbol.wellknown\",\"lib.es2015.symbol.wellknown.d.ts\"],[\"es2016.array.include\",\"lib.es2016.array.include.d.ts\"],[\"es2017.object\",\"lib.es2017.object.d.ts\"],[\"es2017.sharedmemory\",\"lib.es2017.sharedmemory.d.ts\"],[\"es2017.string\",\"lib.es2017.string.d.ts\"],[\"es2017.intl\",\"lib.es2017.intl.d.ts\"],[\"es2017.typedarrays\",\"lib.es2017.typedarrays.d.ts\"],[\"es2018.intl\",\"lib.es2018.intl.d.ts\"],[\"es2018.promise\",\"lib.es2018.promise.d.ts\"],[\"es2018.regexp\",\"lib.es2018.regexp.d.ts\"],[\"esnext.array\",\"lib.esnext.array.d.ts\"],[\"esnext.symbol\",\"lib.esnext.symbol.d.ts\"],[\"esnext.asynciterable\",\"lib.esnext.asynciterable.d.ts\"],[\"esnext.intl\",\"lib.esnext.intl.d.ts\"]];function a(e){return e&&void 0!==e.enableAutoDiscovery&&void 0===e.enable?{enable:e.enableAutoDiscovery,include:e.include||[],exclude:e.exclude||[]}:e}function i(){return n||(n=o(e.optionDeclarations))}function o(n){var t=e.createMap(),r=e.createMap();return e.forEach(n,function(e){t.set(e.name.toLowerCase(),e),e.shortName&&r.set(e.shortName,e.name)}),{optionNameMap:t,shortOptionNames:r}}function s(n){return l(n,e.createCompilerDiagnostic)}function l(n,t){var r=e.arrayFrom(n.type.keys()).map(function(e){return\"'\"+e+\"'\"}).join(\", \");return t(e.Diagnostics.Argument_for_0_option_must_be_Colon_1,\"--\"+n.name,r)}function c(e,n,t){return V(e,K(n||\"\"),t)}function u(n,t,r){if(void 0===t&&(t=\"\"),t=K(t),!e.startsWith(t,\"-\")){if(\"\"===t)return[];var a=t.split(\",\");switch(n.element.type){case\"number\":return e.map(a,parseInt);case\"string\":return e.map(a,function(e){return e||\"\"});default:return e.mapDefined(a,function(e){return c(n.element,e,r)})}}}function d(n,t,r,a){var i=t[0],o=t[1],s={},l=[],d=[];return m(r),{options:s,fileNames:l,errors:d};function m(t){for(var r=0;r<t.length;){var a=t[r];if(r++,64===a.charCodeAt(0))f(a.slice(1));else if(45===a.charCodeAt(0)){var m=p(n,a.slice(45===a.charCodeAt(1)?2:1),!0);if(m)if(m.isTSConfigOnly)d.push(e.createCompilerDiagnostic(e.Diagnostics.Option_0_can_only_be_specified_in_tsconfig_json_file,m.name));else switch(t[r]||\"boolean\"===m.type||d.push(e.createCompilerDiagnostic(o,m.name)),m.type){case\"number\":s[m.name]=parseInt(t[r]),r++;break;case\"boolean\":var g=t[r];s[m.name]=\"false\"!==g,\"false\"!==g&&\"true\"!==g||r++;break;case\"string\":s[m.name]=t[r]||\"\",r++;break;case\"list\":var _=u(m,t[r],d);s[m.name]=_||[],_&&r++;break;default:s[m.name]=c(m,t[r],d),r++}else d.push(e.createCompilerDiagnostic(i,a))}else l.push(a)}}function f(n){var t=a?a(n):e.sys.readFile(n);if(t){for(var r=[],i=0;;){for(;i<t.length&&t.charCodeAt(i)<=32;)i++;if(i>=t.length)break;var o=i;if(34===t.charCodeAt(o)){for(i++;i<t.length&&34!==t.charCodeAt(i);)i++;i<t.length?(r.push(t.substring(o+1,i)),i++):d.push(e.createCompilerDiagnostic(e.Diagnostics.Unterminated_quoted_string_in_response_file_0,n))}else{for(;t.charCodeAt(i)>32;)i++;r.push(t.substring(o,i))}}m(r)}else d.push(e.createCompilerDiagnostic(e.Diagnostics.File_0_not_found,n))}}function m(e,n){return p(i,e,n)}function p(e,n,t){void 0===t&&(t=!1),n=n.toLowerCase();var r=e(),a=r.optionNameMap,i=r.shortOptionNames;if(t){var o=i.get(n);void 0!==o&&(n=o)}return a.get(n)}function f(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];return e.createCompilerDiagnostic.apply(void 0,arguments).messageText}function g(n,t){var r=e.parseJsonText(n,t);return{config:h(r,r.parseDiagnostics),error:r.parseDiagnostics.length?r.parseDiagnostics[0]:void 0}}function _(n,t){var r=v(n,t);return e.isString(r)?e.parseJsonText(n,r):{parseDiagnostics:[r]}}function v(n,t){var r;try{r=t(n)}catch(t){return e.createCompilerDiagnostic(e.Diagnostics.Cannot_read_file_0_Colon_1,n,t.message)}return void 0===r?e.createCompilerDiagnostic(e.Diagnostics.The_specified_path_does_not_exist_Colon_0,n):r}function y(n){return e.arrayToMap(n,function(e){return e.name})}function h(e,n){return b(e,n,!0,void 0,void 0)}function b(n,t,r,a,i){return n.statements.length?c(n.statements[0].expression,a):r?{}:void 0;function o(e){return a&&a.elementOptions===e}function s(a,s,l,d){for(var m=r?{}:void 0,p=0,f=a.properties;p<f.length;p++){var g=f[p];if(273===g.kind){g.questionToken&&t.push(e.createDiagnosticForNodeInSourceFile(n,g.questionToken,e.Diagnostics._0_can_only_be_used_in_a_ts_file,\"?\")),u(g.name)||t.push(e.createDiagnosticForNodeInSourceFile(n,g.name,e.Diagnostics.String_literal_with_double_quotes_expected));var _=e.getTextOfPropertyName(g.name),v=_&&e.unescapeLeadingUnderscores(_),y=v&&s?s.get(v):void 0;v&&l&&!y&&t.push(e.createDiagnosticForNodeInSourceFile(n,g.name,l,v));var h=c(g.initializer,y);if(void 0!==v&&(r&&(m[v]=h),i&&(d||o(s)))){var b=T(y,h);d?b&&i.onSetValidOptionKeyValueInParent(d,y,h):o(s)&&(b?i.onSetValidOptionKeyValueInRoot(v,g.name,h,g.initializer):y||i.onSetUnknownOptionKeyValueInRoot(v,g.name,h,g.initializer))}}else t.push(e.createDiagnosticForNodeInSourceFile(n,g,e.Diagnostics.Property_assignment_expected))}return m}function c(a,i){switch(a.kind){case 101:return _(i&&\"boolean\"!==i.type),!0;case 86:return _(i&&\"boolean\"!==i.type),!1;case 95:return _(i&&\"extends\"===i.name),null;case 9:u(a)||t.push(e.createDiagnosticForNodeInSourceFile(n,a,e.Diagnostics.String_literal_with_double_quotes_expected)),_(i&&e.isString(i.type)&&\"string\"!==i.type);var o=a.text;if(i&&!e.isString(i.type)){var d=i;d.type.has(o.toLowerCase())||t.push(l(d,function(t,r,i){return e.createDiagnosticForNodeInSourceFile(n,a,t,r,i)}))}return o;case 8:return _(i&&\"number\"!==i.type),Number(a.text);case 200:if(38!==a.operator||8!==a.operand.kind)break;return _(i&&\"number\"!==i.type),-Number(a.operand.text);case 186:_(i&&\"object\"!==i.type);var m=a;if(i){var p=i;return s(m,p.elementOptions,p.extraKeyDiagnosticMessage,p.name)}return s(m,void 0,void 0,void 0);case 185:return _(i&&\"list\"!==i.type),f=a.elements,g=i&&i.element,(r?f.map:f.forEach).call(f,function(e){return c(e,g)})}var f,g;return void(i?_(!0):t.push(e.createDiagnosticForNodeInSourceFile(n,a,e.Diagnostics.Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_literal)));function _(r){r&&t.push(e.createDiagnosticForNodeInSourceFile(n,a,e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,i.name,E(i)))}}function u(t){return e.isStringLiteral(t)&&e.isStringDoubleQuoted(t,n)}}function E(n){return\"list\"===n.type?\"Array\":e.isString(n.type)?n.type:\"string\"}function T(n,t){return!!n&&(!!x(t)||(\"list\"===n.type?e.isArray(t):typeof t===(e.isString(n.type)?n.type:\"string\")))}function S(e,n,t,r,a,i,o){return C(void 0,e,n,t,r,a,i,o)}function L(e,n){n&&Object.defineProperty(e,\"configFile\",{enumerable:!1,writable:!1,value:n})}function x(e){return null==e}function A(n,t){return e.getDirectoryPath(e.getNormalizedAbsolutePath(n,t))}function C(n,t,r,a,i,o,s,l){void 0===i&&(i={}),void 0===s&&(s=[]),void 0===l&&(l=[]),e.Debug.assert(void 0===n&&void 0!==t||void 0!==n&&void 0===t);var c,u=[],d=M(n,t,r,a,o,s,u),m=d.raw,p=e.extend(i,d.options||{});p.configFilePath=o&&e.normalizeSlashes(o),L(p,t);var f=function(){var n,i,d;if(e.hasProperty(m,\"files\")&&!x(m.files))if(e.isArray(m.files)){n=m.files;var f=e.hasProperty(m,\"references\")&&!x(m.references),g=!f||0===m.references.length;if(0===n.length&&g)if(t){var _=o||\"tsconfig.json\",v=e.Diagnostics.The_files_list_in_config_file_0_is_empty,h=e.firstDefined(e.getTsConfigPropArray(t,\"files\"),function(e){return e.initializer}),b=h?e.createDiagnosticForNodeInSourceFile(t,h,v,_):e.createCompilerDiagnostic(v,_);u.push(b)}else y(e.Diagnostics.The_files_list_in_config_file_0_is_empty,o||\"tsconfig.json\")}else y(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,\"files\",\"Array\");e.hasProperty(m,\"include\")&&!x(m.include)&&(e.isArray(m.include)?i=m.include:y(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,\"include\",\"Array\"));if(e.hasProperty(m,\"exclude\")&&!x(m.exclude))e.isArray(m.exclude)?d=m.exclude:y(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,\"exclude\",\"Array\");else if(m.compilerOptions){var E=m.compilerOptions.outDir,T=m.compilerOptions.declarationDir;(E||T)&&(d=[E,T].filter(function(e){return!!e}))}void 0===n&&void 0===i&&(i=[\"**/*\"]);var S=function(n,t,r,a,i,o,s,l,c){var u,d;a=e.normalizePath(a),t&&(u=q(t,s,!1,c,\"include\"));r&&(d=q(r,s,!0,c,\"exclude\"));var m=function(n,t,r,a){var i=e.getRegularExpressionForWildcard(t,r,\"exclude\"),o=i&&new RegExp(i,a?\"\":\"i\"),s={};if(void 0!==n){for(var l=[],c=0,u=n;c<u.length;c++){var d=u[c],m=e.normalizePath(e.combinePaths(r,d));if(!o||!o.test(m)){var p=z(m,a);if(p){var f=p.key,g=p.flags,_=s[f];(void 0===_||_<g)&&(s[f]=g,1===g&&l.push(f))}}}for(var f in s)if(e.hasProperty(s,f))for(var v=0,y=l;v<y.length;v++){var h=y[v];f!==h&&e.containsPath(h,f,r,!a)&&delete s[f]}}return s}(u,d,a,o.useCaseSensitiveFileNames);return W({filesSpecs:n,includeSpecs:t,excludeSpecs:r,validatedIncludeSpecs:u,validatedExcludeSpecs:d,wildcardDirectories:m},a,i,o,l)}(n,i,d,o?A(o,a):a,p,r,u,l,t);0!==S.fileNames.length||e.hasProperty(m,\"files\")||0!==s.length||e.hasProperty(m,\"references\")||u.push(D(S.spec,o));if(e.hasProperty(m,\"references\")&&!x(m.references))if(e.isArray(m.references))for(var L=0,C=m.references;L<C.length;L++){var k=C[L];\"string\"!=typeof k.path?y(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,\"reference.path\",\"string\"):(c||(c=[])).push({path:e.getNormalizedAbsolutePath(k.path,a),originalPath:k.path,prepend:k.prepend,circular:k.circular})}else y(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,\"references\",\"Array\");return S}(),g=f.fileNames,_=f.wildcardDirectories,v=f.spec;return{options:p,fileNames:g,projectReferences:c,typeAcquisition:d.typeAcquisition||N(),raw:m,errors:u,wildcardDirectories:_,compileOnSave:!!m.compileOnSave,configFileSpecs:v};function y(n,r,a){t||u.push(e.createCompilerDiagnostic(n,r,a))}}function D(n,t){var r=n.includeSpecs,a=n.excludeSpecs;return e.createCompilerDiagnostic(e.Diagnostics.No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2,t||\"tsconfig.json\",JSON.stringify(r||[]),JSON.stringify(a||[]))}function k(e){return!!e.options}function M(n,r,a,i,o,s,l){i=e.normalizeSlashes(i);var c=e.getNormalizedAbsolutePath(o||\"\",i);if(s.indexOf(c)>=0)return l.push(e.createCompilerDiagnostic(e.Diagnostics.Circularity_detected_while_resolving_configuration_Colon_0,s.concat([c]).join(\" -> \"))),{raw:n||h(r,l)};var u=n?function(n,t,r,a,i){e.hasProperty(n,\"excludes\")&&i.push(e.createCompilerDiagnostic(e.Diagnostics.Unknown_option_excludes_Did_you_mean_exclude));var o,s=R(n.compilerOptions,r,i,a),l=w(n.typeAcquisition||n.typingOptions,r,i,a);if(n.compileOnSave=function(n,t,r){if(!e.hasProperty(n,e.compileOnSaveCommandLineOption.name))return!1;var a=F(e.compileOnSaveCommandLineOption,n.compileOnSave,t,r);return\"boolean\"==typeof a&&a}(n,r,i),n.extends)if(e.isString(n.extends)){var c=a?A(a,r):r;o=O(n.extends,t,c,i,e.createCompilerDiagnostic)}else i.push(e.createCompilerDiagnostic(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,\"extends\",\"string\"));return{raw:n,options:s,typeAcquisition:l,extendedConfigPath:o}}(n,a,i,o,l):function(n,r,a,i,o){var s,l,c,u=I(i),d={onSetValidOptionKeyValueInParent:function(n,t,r){e.Debug.assert(\"compilerOptions\"===n||\"typeAcquisition\"===n||\"typingOptions\"===n);var o=\"compilerOptions\"===n?u:\"typeAcquisition\"===n?s||(s=N(i)):l||(l=N(i));o[t.name]=function n(t,r,a){if(x(a))return;if(\"list\"===t.type){var i=t;return i.element.isFilePath||!e.isString(i.element.type)?e.filter(e.map(a,function(e){return n(i.element,r,e)}),function(e){return!!e}):a}if(!e.isString(t.type))return t.type.get(e.isString(a)?a.toLowerCase():a);return G(t,r,a)}(t,a,r)},onSetValidOptionKeyValueInRoot:function(t,s,l,u){switch(t){case\"extends\":var d=i?A(i,a):a;return void(c=O(l,r,d,o,function(t,r){return e.createDiagnosticForNodeInSourceFile(n,u,t,r)}))}},onSetUnknownOptionKeyValueInRoot:function(t,r,a,i){\"excludes\"===t&&o.push(e.createDiagnosticForNodeInSourceFile(n,r,e.Diagnostics.Unknown_option_excludes_Did_you_mean_exclude))}},m=b(n,o,!0,(void 0===t&&(t={name:void 0,type:\"object\",elementOptions:y([{name:\"compilerOptions\",type:\"object\",elementOptions:y(e.optionDeclarations),extraKeyDiagnosticMessage:e.Diagnostics.Unknown_compiler_option_0},{name:\"typingOptions\",type:\"object\",elementOptions:y(e.typeAcquisitionDeclarations),extraKeyDiagnosticMessage:e.Diagnostics.Unknown_type_acquisition_option_0},{name:\"typeAcquisition\",type:\"object\",elementOptions:y(e.typeAcquisitionDeclarations),extraKeyDiagnosticMessage:e.Diagnostics.Unknown_type_acquisition_option_0},{name:\"extends\",type:\"string\"},{name:\"references\",type:\"list\",element:{name:\"references\",type:\"object\"}},{name:\"files\",type:\"list\",element:{name:\"files\",type:\"string\"}},{name:\"include\",type:\"list\",element:{name:\"include\",type:\"string\"}},{name:\"exclude\",type:\"list\",element:{name:\"exclude\",type:\"string\"}},e.compileOnSaveCommandLineOption])}),t),d);s||(s=l?void 0!==l.enableAutoDiscovery?{enable:l.enableAutoDiscovery,include:l.include,exclude:l.exclude}:l:N(i));return{raw:m,options:u,typeAcquisition:s,extendedConfigPath:c}}(r,a,i,o,l);if(u.extendedConfigPath){s=s.concat([c]);var d=function(n,t,r,a,i,o){var s,l=_(t,function(e){return r.readFile(e)});n&&(n.extendedSourceFiles=[l.fileName]);if(l.parseDiagnostics.length)return void o.push.apply(o,l.parseDiagnostics);var c=e.getDirectoryPath(t),u=M(void 0,l,r,c,e.getBaseFileName(t),i,o);n&&l.extendedSourceFiles&&(s=n.extendedSourceFiles).push.apply(s,l.extendedSourceFiles);if(k(u)){var d=e.convertToRelativePath(c,a,e.identity),m=function(n){return e.isRootedDiskPath(n)?n:e.combinePaths(d,n)},p=function(n){f[n]&&(f[n]=e.map(f[n],m))},f=u.raw;p(\"include\"),p(\"exclude\"),p(\"files\")}return u}(r,u.extendedConfigPath,a,i,s,l);if(d&&k(d)){var m=d.raw,p=u.raw,f=function(e){var n=p[e]||m[e];n&&(p[e]=n)};f(\"include\"),f(\"exclude\"),f(\"files\"),void 0===p.compileOnSave&&(p.compileOnSave=m.compileOnSave),u.options=e.assign({},d.options,u.options)}}return u}function O(n,t,r,a,i){if(n=e.normalizeSlashes(n),e.isRootedDiskPath(n)||e.startsWith(n,\"./\")||e.startsWith(n,\"../\")){var o=e.getNormalizedAbsolutePath(n,r);if(t.fileExists(o)||e.endsWith(o,\".json\")||(o+=\".json\",t.fileExists(o)))return o;a.push(i(e.Diagnostics.File_0_does_not_exist,n))}else a.push(i(e.Diagnostics.A_path_in_an_extends_option_must_be_relative_or_rooted_but_0_is_not,n))}function I(n){return n&&\"jsconfig.json\"===e.getBaseFileName(n)?{allowJs:!0,maxNodeModuleJsDepth:2,allowSyntheticDefaultImports:!0,skipLibCheck:!0,noEmit:!0}:{}}function R(n,t,r,a){var i=I(a);return P(e.optionDeclarations,n,t,i,e.Diagnostics.Unknown_compiler_option_0,r),a&&(i.configFilePath=e.normalizeSlashes(a)),i}function N(n){return{enable:!!n&&\"jsconfig.json\"===e.getBaseFileName(n),include:[],exclude:[]}}function w(n,t,r,i){var o=N(i),s=a(n);return P(e.typeAcquisitionDeclarations,s,t,o,e.Diagnostics.Unknown_type_acquisition_option_0,r),o}function P(n,t,r,a,i,o){if(t){var s=y(n);for(var l in t){var c=s.get(l);c?a[c.name]=F(c,t[l],r,o):o.push(e.createCompilerDiagnostic(i,l))}}}function F(n,t,r,a){if(T(n,t)){var i=n.type;return\"list\"===i&&e.isArray(t)?function(n,t,r,a){return e.filter(e.map(t,function(e){return F(n.element,e,r,a)}),function(e){return!!e})}(n,t,r,a):e.isString(i)?G(n,r,t):V(n,t,a)}a.push(e.createCompilerDiagnostic(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,n.name,E(n)))}function G(n,t,r){return n.isFilePath&&\"\"===(r=e.normalizePath(e.combinePaths(t,r)))&&(r=\".\"),r}function V(e,n,t){if(!x(n)){var r=n.toLowerCase(),a=e.type.get(r);if(void 0!==a)return a;t.push(s(e))}}function K(e){return\"function\"==typeof e.trim?e.trim():e.replace(/^[\\s]+|[\\s]+$/g,\"\")}e.libs=r.map(function(e){return e[0]}),e.libMap=e.createMapFromEntries(r),e.commonOptionsWithBuild=[{name:\"help\",shortName:\"h\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Print_this_message},{name:\"help\",shortName:\"?\",type:\"boolean\"},{name:\"watch\",shortName:\"w\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Watch_input_files},{name:\"preserveWatchOutput\",type:\"boolean\",showInSimplifiedHelpView:!1,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen},{name:\"listFiles\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Print_names_of_files_part_of_the_compilation},{name:\"listEmittedFiles\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Print_names_of_generated_files_part_of_the_compilation},{name:\"traceResolution\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Enable_tracing_of_the_name_resolution_process}],e.optionDeclarations=e.commonOptionsWithBuild.concat([{name:\"all\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Show_all_compiler_options},{name:\"version\",shortName:\"v\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Print_the_compiler_s_version},{name:\"init\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file},{name:\"project\",shortName:\"p\",type:\"string\",isFilePath:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,paramType:e.Diagnostics.FILE_OR_DIRECTORY,description:e.Diagnostics.Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json},{name:\"build\",type:\"boolean\",shortName:\"b\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Build_one_or_more_projects_and_their_dependencies_if_out_of_date},{name:\"pretty\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Stylize_errors_and_messages_using_color_and_context_experimental},{name:\"target\",shortName:\"t\",type:e.createMapFromTemplate({es3:0,es5:1,es6:2,es2015:2,es2016:3,es2017:4,es2018:5,esnext:6}),affectsSourceFile:!0,affectsModuleResolution:!0,paramType:e.Diagnostics.VERSION,showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Specify_ECMAScript_target_version_Colon_ES3_default_ES5_ES2015_ES2016_ES2017_ES2018_or_ESNEXT},{name:\"module\",shortName:\"m\",type:e.createMapFromTemplate({none:e.ModuleKind.None,commonjs:e.ModuleKind.CommonJS,amd:e.ModuleKind.AMD,system:e.ModuleKind.System,umd:e.ModuleKind.UMD,es6:e.ModuleKind.ES2015,es2015:e.ModuleKind.ES2015,esnext:e.ModuleKind.ESNext}),affectsModuleResolution:!0,paramType:e.Diagnostics.KIND,showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Specify_module_code_generation_Colon_none_commonjs_amd_system_umd_es2015_or_ESNext},{name:\"lib\",type:\"list\",element:{name:\"lib\",type:e.libMap},affectsModuleResolution:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Specify_library_files_to_be_included_in_the_compilation},{name:\"allowJs\",type:\"boolean\",affectsModuleResolution:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Allow_javascript_files_to_be_compiled},{name:\"checkJs\",type:\"boolean\",category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Report_errors_in_js_files},{name:\"jsx\",type:e.createMapFromTemplate({preserve:1,\"react-native\":3,react:2}),affectsSourceFile:!0,paramType:e.Diagnostics.KIND,showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Specify_JSX_code_generation_Colon_preserve_react_native_or_react},{name:\"declaration\",shortName:\"d\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Generates_corresponding_d_ts_file},{name:\"declarationMap\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Generates_a_sourcemap_for_each_corresponding_d_ts_file},{name:\"emitDeclarationOnly\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Only_emit_d_ts_declaration_files},{name:\"sourceMap\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Generates_corresponding_map_file},{name:\"outFile\",type:\"string\",isFilePath:!0,paramType:e.Diagnostics.FILE,showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Concatenate_and_emit_output_to_single_file},{name:\"outDir\",type:\"string\",isFilePath:!0,paramType:e.Diagnostics.DIRECTORY,showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Redirect_output_structure_to_the_directory},{name:\"rootDir\",type:\"string\",isFilePath:!0,paramType:e.Diagnostics.LOCATION,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir},{name:\"composite\",type:\"boolean\",isTSConfigOnly:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Enable_project_compilation},{name:\"removeComments\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Do_not_emit_comments_to_output},{name:\"noEmit\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Do_not_emit_outputs},{name:\"importHelpers\",type:\"boolean\",category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Import_emit_helpers_from_tslib},{name:\"downlevelIteration\",type:\"boolean\",category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3},{name:\"isolatedModules\",type:\"boolean\",category:e.Diagnostics.Basic_Options,description:e.Diagnostics.Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule},{name:\"strict\",type:\"boolean\",showInSimplifiedHelpView:!0,category:e.Diagnostics.Strict_Type_Checking_Options,description:e.Diagnostics.Enable_all_strict_type_checking_options},{name:\"noImplicitAny\",type:\"boolean\",affectsSemanticDiagnostics:!0,strictFlag:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Strict_Type_Checking_Options,description:e.Diagnostics.Raise_error_on_expressions_and_declarations_with_an_implied_any_type},{name:\"strictNullChecks\",type:\"boolean\",affectsSemanticDiagnostics:!0,strictFlag:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Strict_Type_Checking_Options,description:e.Diagnostics.Enable_strict_null_checks},{name:\"strictFunctionTypes\",type:\"boolean\",affectsSemanticDiagnostics:!0,strictFlag:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Strict_Type_Checking_Options,description:e.Diagnostics.Enable_strict_checking_of_function_types},{name:\"strictPropertyInitialization\",type:\"boolean\",affectsSemanticDiagnostics:!0,strictFlag:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Strict_Type_Checking_Options,description:e.Diagnostics.Enable_strict_checking_of_property_initialization_in_classes},{name:\"noImplicitThis\",type:\"boolean\",affectsSemanticDiagnostics:!0,strictFlag:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Strict_Type_Checking_Options,description:e.Diagnostics.Raise_error_on_this_expressions_with_an_implied_any_type},{name:\"alwaysStrict\",type:\"boolean\",affectsSourceFile:!0,strictFlag:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Strict_Type_Checking_Options,description:e.Diagnostics.Parse_in_strict_mode_and_emit_use_strict_for_each_source_file},{name:\"noUnusedLocals\",type:\"boolean\",affectsSemanticDiagnostics:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Additional_Checks,description:e.Diagnostics.Report_errors_on_unused_locals},{name:\"noUnusedParameters\",type:\"boolean\",affectsSemanticDiagnostics:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Additional_Checks,description:e.Diagnostics.Report_errors_on_unused_parameters},{name:\"noImplicitReturns\",type:\"boolean\",affectsSemanticDiagnostics:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Additional_Checks,description:e.Diagnostics.Report_error_when_not_all_code_paths_in_function_return_a_value},{name:\"noFallthroughCasesInSwitch\",type:\"boolean\",affectsBindDiagnostics:!0,affectsSemanticDiagnostics:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Additional_Checks,description:e.Diagnostics.Report_errors_for_fallthrough_cases_in_switch_statement},{name:\"moduleResolution\",type:e.createMapFromTemplate({node:e.ModuleResolutionKind.NodeJs,classic:e.ModuleResolutionKind.Classic}),affectsModuleResolution:!0,paramType:e.Diagnostics.STRATEGY,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6},{name:\"baseUrl\",type:\"string\",affectsModuleResolution:!0,isFilePath:!0,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.Base_directory_to_resolve_non_absolute_module_names},{name:\"paths\",type:\"object\",affectsModuleResolution:!0,isTSConfigOnly:!0,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl},{name:\"rootDirs\",type:\"list\",isTSConfigOnly:!0,element:{name:\"rootDirs\",type:\"string\",isFilePath:!0},affectsModuleResolution:!0,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime},{name:\"typeRoots\",type:\"list\",element:{name:\"typeRoots\",type:\"string\",isFilePath:!0},affectsModuleResolution:!0,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.List_of_folders_to_include_type_definitions_from},{name:\"types\",type:\"list\",element:{name:\"types\",type:\"string\"},affectsModuleResolution:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.Type_declaration_files_to_be_included_in_compilation},{name:\"allowSyntheticDefaultImports\",type:\"boolean\",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typechecking},{name:\"esModuleInterop\",type:\"boolean\",affectsSemanticDiagnostics:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for_all_imports_Implies_allowSyntheticDefaultImports},{name:\"preserveSymlinks\",type:\"boolean\",category:e.Diagnostics.Module_Resolution_Options,description:e.Diagnostics.Do_not_resolve_the_real_path_of_symlinks},{name:\"sourceRoot\",type:\"string\",paramType:e.Diagnostics.LOCATION,category:e.Diagnostics.Source_Map_Options,description:e.Diagnostics.Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations},{name:\"mapRoot\",type:\"string\",paramType:e.Diagnostics.LOCATION,category:e.Diagnostics.Source_Map_Options,description:e.Diagnostics.Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations},{name:\"inlineSourceMap\",type:\"boolean\",category:e.Diagnostics.Source_Map_Options,description:e.Diagnostics.Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file},{name:\"inlineSources\",type:\"boolean\",category:e.Diagnostics.Source_Map_Options,description:e.Diagnostics.Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap_to_be_set},{name:\"experimentalDecorators\",type:\"boolean\",category:e.Diagnostics.Experimental_Options,description:e.Diagnostics.Enables_experimental_support_for_ES7_decorators},{name:\"emitDecoratorMetadata\",type:\"boolean\",category:e.Diagnostics.Experimental_Options,description:e.Diagnostics.Enables_experimental_support_for_emitting_type_metadata_for_decorators},{name:\"jsxFactory\",type:\"string\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h},{name:\"diagnostics\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Show_diagnostic_information},{name:\"extendedDiagnostics\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Show_verbose_diagnostic_information},{name:\"resolveJsonModule\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Include_modules_imported_with_json_extension},{name:\"out\",type:\"string\",isFilePath:!1,category:e.Diagnostics.Advanced_Options,paramType:e.Diagnostics.FILE,description:e.Diagnostics.Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file},{name:\"reactNamespace\",type:\"string\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react_JSX_emit},{name:\"skipDefaultLibCheck\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files},{name:\"charset\",type:\"string\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.The_character_set_of_the_input_files},{name:\"emitBOM\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files},{name:\"locale\",type:\"string\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.The_locale_used_when_displaying_messages_to_the_user_e_g_en_us},{name:\"newLine\",type:e.createMapFromTemplate({crlf:0,lf:1}),paramType:e.Diagnostics.NEWLINE,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix},{name:\"noErrorTruncation\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_truncate_error_messages},{name:\"noLib\",type:\"boolean\",affectsModuleResolution:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_include_the_default_library_file_lib_d_ts},{name:\"noResolve\",type:\"boolean\",affectsModuleResolution:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files},{name:\"stripInternal\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_emit_declarations_for_code_that_has_an_internal_annotation},{name:\"disableSizeLimit\",type:\"boolean\",affectsSourceFile:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Disable_size_limitations_on_JavaScript_projects},{name:\"noImplicitUseStrict\",type:\"boolean\",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_emit_use_strict_directives_in_module_output},{name:\"noEmitHelpers\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_generate_custom_helper_functions_like_extends_in_compiled_output},{name:\"noEmitOnError\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_emit_outputs_if_any_errors_were_reported},{name:\"preserveConstEnums\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_erase_const_enum_declarations_in_generated_code},{name:\"declarationDir\",type:\"string\",isFilePath:!0,paramType:e.Diagnostics.DIRECTORY,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Output_directory_for_generated_declaration_files},{name:\"skipLibCheck\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Skip_type_checking_of_declaration_files},{name:\"allowUnusedLabels\",type:\"boolean\",affectsBindDiagnostics:!0,affectsSemanticDiagnostics:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_report_errors_on_unused_labels},{name:\"allowUnreachableCode\",type:\"boolean\",affectsBindDiagnostics:!0,affectsSemanticDiagnostics:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Do_not_report_errors_on_unreachable_code},{name:\"suppressExcessPropertyErrors\",type:\"boolean\",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Suppress_excess_property_checks_for_object_literals},{name:\"suppressImplicitAnyIndexErrors\",type:\"boolean\",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures},{name:\"forceConsistentCasingInFileNames\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Disallow_inconsistently_cased_references_to_the_same_file},{name:\"maxNodeModuleJsDepth\",type:\"number\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files},{name:\"noStrictGenericChecks\",type:\"boolean\",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Disable_strict_checking_of_generic_signatures_in_function_types},{name:\"keyofStringsOnly\",type:\"boolean\",category:e.Diagnostics.Advanced_Options,description:e.Diagnostics.Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols},{name:\"plugins\",type:\"list\",isTSConfigOnly:!0,element:{name:\"plugin\",type:\"object\"},description:e.Diagnostics.List_of_language_service_plugins}]),e.semanticDiagnosticsOptionDeclarations=e.optionDeclarations.filter(function(e){return!!e.affectsSemanticDiagnostics}),e.moduleResolutionOptionDeclarations=e.optionDeclarations.filter(function(e){return!!e.affectsModuleResolution}),e.sourceFileAffectingCompilerOptions=e.optionDeclarations.filter(function(e){return!!e.affectsSourceFile||!!e.affectsModuleResolution||!!e.affectsBindDiagnostics}),e.buildOpts=e.commonOptionsWithBuild.concat([{name:\"verbose\",shortName:\"v\",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Enable_verbose_logging,type:\"boolean\"},{name:\"dry\",shortName:\"d\",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Show_what_would_be_built_or_deleted_if_specified_with_clean,type:\"boolean\"},{name:\"force\",shortName:\"f\",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Build_all_projects_including_those_that_appear_to_be_up_to_date,type:\"boolean\"},{name:\"clean\",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Delete_the_outputs_of_all_projects,type:\"boolean\"}]),e.typeAcquisitionDeclarations=[{name:\"enableAutoDiscovery\",type:\"boolean\"},{name:\"enable\",type:\"boolean\"},{name:\"include\",type:\"list\",element:{name:\"include\",type:\"string\"}},{name:\"exclude\",type:\"list\",element:{name:\"exclude\",type:\"string\"}}],e.defaultInitCompilerOptions={module:e.ModuleKind.CommonJS,target:1,strict:!0,esModuleInterop:!0},e.convertEnableAutoDiscoveryToEnable=a,e.createOptionNameMap=o,e.createCompilerDiagnosticForInvalidCustomType=s,e.parseCustomTypeOption=c,e.parseListTypeOption=u,e.parseCommandLine=function(n,t){return d(i,[e.Diagnostics.Unknown_compiler_option_0,e.Diagnostics.Compiler_option_0_expects_an_argument],n,t)},e.getOptionFromName=m,e.parseBuildCommand=function(n){var t,r=d(function(){return t||(t=o(e.buildOpts))},[e.Diagnostics.Unknown_build_option_0,e.Diagnostics.Build_option_0_requires_a_value_of_type_1],n),a=r.options,i=r.fileNames,s=r.errors,l=a;return 0===i.length&&i.push(\".\"),l.clean&&l.force&&s.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,\"clean\",\"force\")),l.clean&&l.verbose&&s.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,\"clean\",\"verbose\")),l.clean&&l.watch&&s.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,\"clean\",\"watch\")),l.watch&&l.dry&&s.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,\"watch\",\"dry\")),{buildOptions:l,projects:i,errors:s}},e.printVersion=function(){e.sys.write(f(e.Diagnostics.Version_0,e.version)+e.sys.newLine)},e.printHelp=function(n,t){void 0===t&&(t=\"\");var r=[],a=f(e.Diagnostics.Syntax_Colon_0,\"\").length,i=f(e.Diagnostics.Examples_Colon_0,\"\").length,o=Math.max(a,i),s=M(o-a);s+=\"tsc \"+t+\"[\"+f(e.Diagnostics.options)+\"] [\"+f(e.Diagnostics.file)+\"...]\",r.push(f(e.Diagnostics.Syntax_Colon_0,s)),r.push(e.sys.newLine+e.sys.newLine);var l=M(o);r.push(f(e.Diagnostics.Examples_Colon_0,M(o-i)+\"tsc hello.ts\")+e.sys.newLine),r.push(l+\"tsc --outFile file.js file.ts\"+e.sys.newLine),r.push(l+\"tsc @args.txt\"+e.sys.newLine),r.push(l+\"tsc --build tsconfig.json\"+e.sys.newLine),r.push(e.sys.newLine),r.push(f(e.Diagnostics.Options_Colon)+e.sys.newLine),o=0;for(var c=[],u=[],d=e.createMap(),m=0,p=n;m<p.length;m++){var g=p[m];if(g.description){var _=\" \";g.shortName&&(_+=\"-\"+g.shortName,_+=k(g),_+=\", \"),_+=\"--\"+g.name,_+=k(g),c.push(_);var v=void 0;if(\"lib\"===g.name){v=f(g.description);var y=g.element.type;d.set(v,e.arrayFrom(y.keys()).map(function(e){return\"'\"+e+\"'\"}))}else v=f(g.description);u.push(v),o=Math.max(_.length,o)}}var h=\" @<\"+f(e.Diagnostics.file)+\">\";c.push(h),u.push(f(e.Diagnostics.Insert_command_line_options_and_files_from_a_file)),o=Math.max(h.length,o);for(var b=0;b<c.length;b++){var E=c[b],T=(v=u[b],d.get(v));if(r.push(E+M(o-E.length+2)+v+e.sys.newLine),T){r.push(M(o+4));for(var S=0,L=T;S<L.length;S++){var x=L[S];r.push(x+\" \")}r.push(e.sys.newLine)}}for(var A=0,C=r;A<C.length;A++){var D=C[A];e.sys.write(D)}return;function k(e){return void 0!==e.paramType?\" \"+f(e.paramType):\"\"}function M(e){return Array(e+1).join(\" \")}},e.getParsedCommandLineOfConfigFile=function(n,t,r){var a;try{a=r.readFile(n)}catch(t){var i=e.createCompilerDiagnostic(e.Diagnostics.Cannot_read_file_0_Colon_1,n,t.message);return void r.onUnRecoverableConfigFileDiagnostic(i)}if(a){var o=e.parseJsonText(n,a),s=r.getCurrentDirectory();return S(o,r,e.getNormalizedAbsolutePath(e.getDirectoryPath(n),s),t,e.getNormalizedAbsolutePath(n,s))}i=e.createCompilerDiagnostic(e.Diagnostics.File_0_not_found,n),r.onUnRecoverableConfigFileDiagnostic(i)},e.readConfigFile=function(n,t){var r=v(n,t);return e.isString(r)?g(n,r):{config:{},error:r}},e.parseConfigFileTextToJson=g,e.readJsonConfigFile=_,e.convertToObject=h,e.convertToObjectWorker=b,e.generateTSConfig=function(n,t,r){var a=function(n){var t=e.createMap(),r=i().optionNameMap,a=function(a){if(e.hasProperty(n,a)){if(r.has(a)&&r.get(a).category===e.Diagnostics.Command_line_Options)return\"continue\";var i=n[a],s=r.get(a.toLowerCase());if(s){var l=function e(n){return\"string\"===n.type||\"number\"===n.type||\"boolean\"===n.type?void 0:\"list\"===n.type?e(n.element):n.type}(s);l?\"list\"===s.type?t.set(a,i.map(function(e){return o(e,l)})):t.set(a,o(i,l)):t.set(a,i)}}};for(var s in n)a(s);return t}(e.extend(n,e.defaultInitCompilerOptions));return function(){for(var n=e.createMultiMap(),i=0,o=e.optionDeclarations;i<o.length;i++){var u=o[i],d=u.category;c(u)&&n.add(e.getLocaleSpecificMessage(d),u)}var m=0,p=0,f=[],g=[];n.forEach(function(n,t){0!==f.length&&(f.push(\"\"),g.push(\"\")),f.push(\"/* \"+t+\" */\"),g.push(\"\");for(var r=0,i=n;r<i.length;r++){var o=i[r],l=void 0;l=a.has(o.name)?'\"'+o.name+'\": '+JSON.stringify(a.get(o.name))+((p+=1)===a.size?\"\":\",\"):'// \"'+o.name+'\": '+JSON.stringify(s(o))+\",\",f.push(l),g.push(\"/* \"+(o.description&&e.getLocaleSpecificMessage(o.description)||o.name)+\" */\"),m=Math.max(l.length,m)}});var _=l(2),v=[];v.push(\"{\"),v.push(_+'\"compilerOptions\": {');for(var y=0;y<f.length;y++){var h=f[y],b=g[y];v.push(h&&\"\"+_+_+h+(b&&l(m-h.length+2)+b))}if(t.length){v.push(_+\"},\"),v.push(_+'\"files\": [');for(var y=0;y<t.length;y++)v.push(\"\"+_+_+JSON.stringify(t[y])+(y===t.length-1?\"\":\",\"));v.push(_+\"]\")}else v.push(_+\"}\");return v.push(\"}\"),v.join(r)}();function o(n,t){return e.forEachEntry(t,function(e,t){if(e===n)return t})}function s(e){switch(e.type){case\"number\":return 1;case\"boolean\":return!0;case\"string\":return e.isFilePath?\"./\":\"\";case\"list\":return[];case\"object\":return{};default:return e.type.keys().next().value}}function l(e){return Array(e+1).join(\" \")}function c(n){var t=n.category,r=n.name;return void 0!==t&&t!==e.Diagnostics.Command_line_Options&&(t!==e.Diagnostics.Advanced_Options||a.has(r))}},e.parseJsonConfigFileContent=function(e,n,t,r,a,i,o){return C(e,void 0,n,t,r,a,i,o)},e.parseJsonSourceFileConfigFileContent=S,e.setConfigFileInOptions=L,e.isErrorNoInputFiles=function(n){return n.code===e.Diagnostics.No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2.code},e.getErrorForNoInputFiles=D,e.convertCompilerOptionsFromJson=function(e,n,t){var r=[];return{options:R(e,n,r,t),errors:r}},e.convertTypeAcquisitionFromJson=function(e,n,t){var r=[];return{options:w(e,n,r,t),errors:r}};var B=/(^|\\/)\\*\\*\\/?$/,H=/(^|\\/)\\*\\*\\/(.*\\/)?\\.\\.($|\\/)/,U=/\\/[^\\/]*?[*?][^\\/]*\\//,j=/^[^*?]*(?=\\/[^\\/]*[*?])/;function W(n,t,r,a,i){void 0===i&&(i=[]),t=e.normalizePath(t);var o=a.useCaseSensitiveFileNames?e.identity:e.toLowerCase,s=e.createMap(),l=e.createMap(),c=n.filesSpecs,u=n.validatedIncludeSpecs,d=n.validatedExcludeSpecs,m=n.wildcardDirectories,p=e.getSupportedExtensions(r,i);if(c)for(var f=0,g=c;f<g.length;f++){var _=g[f],v=e.getNormalizedAbsolutePath(_,t);s.set(o(v),v)}if(u&&u.length>0)for(var y=0,h=a.readDirectory(t,p,d,u,void 0);y<h.length;y++){if(!J(v=h[y],s,l,p,o)){X(v,l,p,o);var b=o(v);s.has(b)||l.has(b)||l.set(b,v)}}var E=e.arrayFrom(s.values()),T=e.arrayFrom(l.values());return{fileNames:E.concat(T),wildcardDirectories:m,spec:n}}function q(n,t,r,a,i){return n.filter(function(n){var o=function(n,t){if(!t&&B.test(n))return e.Diagnostics.File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0;if(H.test(n))return e.Diagnostics.File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0}(n,r);return void 0!==o&&t.push(function(n,t){var r=e.getTsConfigPropArrayElementValue(a,i,t);return r?e.createDiagnosticForNodeInSourceFile(a,r,n,t):e.createCompilerDiagnostic(n,t)}(o,n)),void 0===o})}function z(n,t){var r=j.exec(n);return r?{key:t?r[0]:r[0].toLowerCase(),flags:U.test(n)?1:0}:e.isImplicitGlob(n)?{key:n,flags:1}:void 0}function J(n,t,r,a,i){for(var o=e.getExtensionPriority(n,a),s=e.adjustExtensionPriority(o,a),l=0;l<s;l++){var c=a[l],u=i(e.changeExtension(n,c));if(t.has(u)||r.has(u))return!0}return!1}function X(n,t,r,a){for(var i=e.getExtensionPriority(n,r),o=e.getNextLowestExtensionPriority(i,r);o<r.length;o++){var s=r[o],l=a(e.changeExtension(n,s));t.delete(l)}}function Y(n,t){switch(t.type){case\"object\":case\"string\":return\"\";case\"number\":return\"number\"==typeof n?n:\"\";case\"boolean\":return\"boolean\"==typeof n?n:\"\";case\"list\":var r=t.element;return e.isArray(n)?n.map(function(e){return Y(e,r)}):\"\";default:return e.forEachEntry(t.type,function(e,t){if(e===n)return t})}}e.getFileNamesFromConfigSpecs=W,e.convertCompilerOptionsForTelemetry=function(e){var n={};for(var t in e)if(e.hasOwnProperty(t)){var r=m(t);void 0!==r&&(n[t]=Y(e[t],r))}return n}}(Tn||(Tn={})),function(e){function n(n){n.trace(e.formatMessage.apply(void 0,arguments))}function t(e,n){return!!e.traceResolution&&void 0!==n.trace}function r(e,n){return n&&{path:n.path,extension:n.ext,packageId:e}}function a(e){return r(void 0,e)}function i(n){if(n)return e.Debug.assert(void 0===n.packageId),{path:n.path,ext:n.extension}}var o,s;function l(n){if(n)return e.Debug.assert(e.extensionIsTS(n.extension)),{fileName:n.path,packageId:n.packageId}}function c(e,n,t){return{resolvedModule:e&&{resolvedFileName:e.path,originalPath:!0===e.originalPath?void 0:e.originalPath,extension:e.extension,isExternalLibraryImport:n,packageId:e.packageId},failedLookupLocations:t}}function u(t,r,a,i){if(e.hasProperty(t,r)){var o=t[r];if(typeof o===a&&null!==o)return o;i.traceEnabled&&n(i.host,e.Diagnostics.Expected_type_of_0_field_in_package_json_to_be_1_got_2,r,a,null===o?\"null\":typeof o)}else i.traceEnabled&&n(i.host,e.Diagnostics.package_json_does_not_have_a_0_field,r)}function d(t,r,a,i){var o=u(t,r,\"string\",i);if(void 0!==o){var s=e.normalizePath(e.combinePaths(a,o));return i.traceEnabled&&n(i.host,e.Diagnostics.package_json_has_0_field_1_that_references_2,r,o,s),s}}function m(e,n,t){return d(e,\"typings\",n,t)||d(e,\"types\",n,t)}function p(e,n,t){return d(e,\"main\",n,t)}function f(t,r){var a=function(t,r){var a=u(t,\"typesVersions\",\"object\",r);if(void 0!==a)return r.traceEnabled&&n(r.host,e.Diagnostics.package_json_has_a_typesVersions_field_with_version_specific_path_mappings),a}(t,r);if(void 0!==a){if(r.traceEnabled)for(var i in a)e.hasProperty(a,i)&&!e.VersionRange.tryParse(i)&&n(r.host,e.Diagnostics.package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range,i);var o=g(a);if(o){var s=o.version,l=o.paths;if(\"object\"==typeof l)return o;r.traceEnabled&&n(r.host,e.Diagnostics.Expected_type_of_0_field_in_package_json_to_be_1_got_2,\"typesVersions['\"+s+\"']\",\"object\",typeof l)}else r.traceEnabled&&n(r.host,e.Diagnostics.package_json_does_not_have_a_typesVersions_entry_that_matches_version_0,e.versionMajorMinor)}}function g(n){for(var t in s||(s=new e.Version(e.version)),n)if(e.hasProperty(n,t)){var r=e.VersionRange.tryParse(t);if(void 0!==r&&r.test(s))return{version:t,paths:n[t]}}}function _(n,t){return n.typeRoots?n.typeRoots:(n.configFilePath?r=e.getDirectoryPath(n.configFilePath):t.getCurrentDirectory&&(r=t.getCurrentDirectory()),void 0!==r?function(n,t){if(!t.directoryExists)return[e.combinePaths(n,v)];var r;return e.forEachAncestorDirectory(e.normalizePath(n),function(n){var a=e.combinePaths(n,v);t.directoryExists(a)&&(r||(r=[])).push(a)}),r}(r,t):void 0);var r}e.trace=n,e.isTraceEnabled=t,function(e){e[e.TypeScript=0]=\"TypeScript\",e[e.JavaScript=1]=\"JavaScript\",e[e.Json=2]=\"Json\",e[e.DtsOnly=3]=\"DtsOnly\"}(o||(o={})),e.getPackageJsonTypesVersionsPaths=g,e.getEffectiveTypeRoots=_;var v=e.combinePaths(\"node_modules\",\"@types\");function y(n,t,r,a){return{getOrCreateCacheForDirectory:function(t){var i=e.toPath(t,r,a),o=n.get(i);o||(o=e.createMap(),n.set(i,o));return o},getOrCreateCacheForModuleName:function(n){e.Debug.assert(!e.isExternalModuleNameRelative(n));var i=t.get(n);i||(i=function(){var n=e.createMap();return{get:function(t){return n.get(e.toPath(t,r,a))},set:function(t,i){var o=e.toPath(t,r,a);if(n.has(o))return;n.set(o,i);var s=i.resolvedModule&&(i.resolvedModule.originalPath||i.resolvedModule.resolvedFileName),l=s&&function(n,t){var i=e.toPath(e.getDirectoryPath(t),r,a),o=0,s=Math.min(n.length,i.length);for(;o<s&&n.charCodeAt(o)===i.charCodeAt(o);)o++;if(o===n.length&&(i.length===o||i[o]===e.directorySeparator))return n;var l=e.getRootLength(n);if(o<l)return;var c=n.lastIndexOf(e.directorySeparator,o-1);if(-1===c)return;return n.substr(0,Math.max(c,l))}(o,s),c=o;for(;c!==l;){var u=e.getDirectoryPath(c);if(u===c||n.has(u))break;n.set(u,i),c=u}}}}(),t.set(n,i));return i}}}function h(t,r,a,i,o){return e.isExternalModuleNameRelative(r)?function(t,r,a,i,o){if(!o.compilerOptions.rootDirs)return;o.traceEnabled&&n(o.host,e.Diagnostics.rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0,r);for(var s,l,c=e.normalizePath(e.combinePaths(a,r)),u=0,d=o.compilerOptions.rootDirs;u<d.length;u++){var m=d[u],p=e.normalizePath(m);e.endsWith(p,e.directorySeparator)||(p+=e.directorySeparator);var f=e.startsWith(c,p)&&(void 0===l||l.length<p.length);o.traceEnabled&&n(o.host,e.Diagnostics.Checking_if_0_is_the_longest_matching_prefix_for_1_2,p,c,f),f&&(l=p,s=m)}if(l){o.traceEnabled&&n(o.host,e.Diagnostics.Longest_matching_prefix_for_0_is_1,c,l);var g=c.substr(l.length);o.traceEnabled&&n(o.host,e.Diagnostics.Loading_0_from_the_root_dir_1_candidate_location_2,g,l,c);var _=i(t,c,!e.directoryProbablyExists(a,o.host),o);if(_)return _;o.traceEnabled&&n(o.host,e.Diagnostics.Trying_other_entries_in_rootDirs);for(var v=0,y=o.compilerOptions.rootDirs;v<y.length;v++){var m=y[v];if(m!==s){var h=e.combinePaths(e.normalizePath(m),g);o.traceEnabled&&n(o.host,e.Diagnostics.Loading_0_from_the_root_dir_1_candidate_location_2,g,m,h);var b=e.getDirectoryPath(h),E=i(t,h,!e.directoryProbablyExists(b,o.host),o);if(E)return E}}o.traceEnabled&&n(o.host,e.Diagnostics.Module_resolution_using_rootDirs_has_failed)}return}(t,r,a,i,o):function(t,r,a,i){var o=i.compilerOptions,s=o.baseUrl,l=o.paths;if(!s)return;i.traceEnabled&&n(i.host,e.Diagnostics.baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1,s,r);if(l){i.traceEnabled&&n(i.host,e.Diagnostics.paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0,r);var c=V(t,r,s,l,a,!1,i);if(c)return c.value}var u=e.normalizePath(e.combinePaths(s,r));i.traceEnabled&&n(i.host,e.Diagnostics.Resolving_module_name_0_relative_to_base_url_1_2,r,s,u);return a(t,u,!e.directoryProbablyExists(e.getDirectoryPath(u),i.host),i)}(t,r,i,o)}function b(n,t,r){return T(n,t,{moduleResolution:e.ModuleResolutionKind.NodeJs,allowJs:!0},r,void 0,!0)}function E(n,t,r,a,i){return T(n,e.getDirectoryPath(t),r,a,i,!1)}function T(r,a,i,s,l,u){var d=t(i,s),m=[],p={compilerOptions:i,host:s,traceEnabled:d,failedLookupLocations:m},f=u?_(o.JavaScript):_(o.TypeScript)||_(o.JavaScript)||(i.resolveJsonModule?_(o.Json):void 0);if(f&&f.value){var g=f.value;return c(g.resolved,g.isExternalLibraryImport,m)}return{resolvedModule:void 0,failedLookupLocations:m};function _(t){var c=h(t,r,a,function(e,n,t,r){return L(e,n,t,r,!0)},p);if(c)return W({resolved:c,isExternalLibraryImport:e.stringContains(c.path,e.nodeModulesPathPart)});if(e.isExternalModuleNameRelative(r)){var u=e.normalizePathAndParts(e.combinePaths(a,r)),m=u.path,f=u.parts,g=L(t,m,!1,p,!0);return g&&W({resolved:g,isExternalLibraryImport:e.contains(f,\"node_modules\")})}d&&n(s,e.Diagnostics.Loading_module_0_from_node_modules_folder_target_file_type_1,r,o[t]);var _=w(t,r,a,p,l);if(_){var v=_.value;if(!i.preserveSymlinks&&v&&!v.originalPath){var y=S(v.path,s,d),b=y===v.path?void 0:v.path;v=yn({},v,{path:y,originalPath:b})}return{value:v&&{resolved:v,isExternalLibraryImport:!0}}}}}function S(t,r,a){if(!r.realpath)return t;var i=e.normalizePath(r.realpath(t));return a&&n(r,e.Diagnostics.Resolving_real_path_for_0_result_1,t,i),e.Debug.assert(r.fileExists(i),t+\" linked to nonexistent file \"+i),i}function L(t,a,i,s,l){if(s.traceEnabled&&n(s.host,e.Diagnostics.Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1,a,o[t]),!e.hasTrailingDirectorySeparator(a)){if(!i){var c=e.getDirectoryPath(a);e.directoryProbablyExists(c,s.host)||(s.traceEnabled&&n(s.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,c),i=!0)}var u=D(t,a,i,s);if(u){var d=l?function(n){var t=e.normalizePath(n.path),r=t.lastIndexOf(e.nodeModulesPathPart);if(-1===r)return;var a=r+e.nodeModulesPathPart.length,i=x(t,a);64===t.charCodeAt(a)&&(i=x(t,i));var o=t.slice(0,i),s=e.removeExtension(t.slice(i+1),n.ext)+\".d.ts\";return{packageDirectory:o,subModuleName:s}}(u):void 0,m=d&&R(d.packageDirectory,d.subModuleName,!1,s);return r(m&&m.packageId,u)}}i||(e.directoryProbablyExists(a,s.host)||(s.traceEnabled&&n(s.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,a),i=!0));return O(t,a,i,s,l)}function x(n,t){var r=n.indexOf(e.directorySeparator,t+1);return-1===r?t:r}function A(n){return\"\"===n?\"index.d.ts\":e.endsWith(n,\".d.ts\")?n:\"index\"===n||e.endsWith(n,\"/index\")?n+\".d.ts\":n+\"/index.d.ts\"}function C(e,n,t,r){return a(D(e,n,t,r))}function D(t,r,a,i){if(t===o.Json){var s=e.tryRemoveExtension(r,\".json\");return void 0===s?void 0:k(s,t,a,i)}var l=k(r,t,a,i);if(l)return l;if(e.hasJSFileExtension(r)){var c=e.removeFileExtension(r);if(i.traceEnabled){var u=r.substring(c.length);n(i.host,e.Diagnostics.File_name_0_has_a_1_extension_stripping_it,r,u)}return k(c,t,a,i)}}function k(n,t,r,a){if(!r){var i=e.getDirectoryPath(n);i&&(r=!e.directoryProbablyExists(i,a.host))}switch(t){case o.DtsOnly:return s(\".d.ts\");case o.TypeScript:return s(\".ts\")||s(\".tsx\")||s(\".d.ts\");case o.JavaScript:return s(\".js\")||s(\".jsx\");case o.Json:return s(\".json\")}function s(e){var t=M(n+e,r,a);return void 0===t?void 0:{path:t,ext:e}}}function M(t,r,a){if(!r){if(a.host.fileExists(t))return a.traceEnabled&&n(a.host,e.Diagnostics.File_0_exist_use_it_as_a_name_resolution_result,t),t;a.traceEnabled&&n(a.host,e.Diagnostics.File_0_does_not_exist,t)}a.failedLookupLocations.push(t)}function O(e,n,t,a,i){void 0===i&&(i=!0);var o=i?R(n,\"\",t,a):void 0,s=o&&o.packageId,l=o&&o.packageJsonContent;return r(s,I(e,n,t,a,l,l&&f(l,a)))}function I(t,r,s,l,c,u){var d=c&&function(t,r,s,l,c){var u=s!==o.JavaScript&&s!==o.Json?m(t,l,c):p(t,l,c);if(!u){if(s!==o.TypeScript)return;if(!(u=p(t,l,c)))return}var d=function(t,r,i,s){var l=M(r,i,s);if(l){var c=function(n,t){var r=e.tryGetExtensionFromPath(t);return void 0!==r&&function(e,n){switch(e){case o.JavaScript:return\".js\"===n||\".jsx\"===n;case o.Json:return\".json\"===n;case o.TypeScript:return\".ts\"===n||\".tsx\"===n||\".d.ts\"===n;case o.DtsOnly:return\".d.ts\"===n}}(n,r)?{path:t,ext:r}:void 0}(t,l);if(c)return a(c);s.traceEnabled&&n(s.host,e.Diagnostics.File_0_has_an_unsupported_extension_so_skipping_it,l)}var u=t===o.DtsOnly?o.TypeScript:t;return L(u,r,i,s,!1)},f=!e.directoryProbablyExists(e.getDirectoryPath(u),c.host);if(r&&e.containsPath(l,u)){var g=e.getRelativePathFromDirectory(l,u,!1);c.traceEnabled&&n(c.host,e.Diagnostics.package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2,r.version,e.version,g);var _=V(s,g,l,r.paths,d,f,c);if(_)return i(_.value)}return i(d(s,u,f,c))}(c,u,t,r,l);if(d)return d;var f=!s&&e.directoryProbablyExists(r,l.host);return D(t,e.combinePaths(r,\"index\"),!f,l)}function R(t,r,a,i){var o=i.host,s=i.traceEnabled,l=!a&&e.directoryProbablyExists(t,o),c=e.combinePaths(t,\"package.json\");if(l&&o.fileExists(c)){var u=e.readJson(c,o);if(\"\"===r){var d=m(u,t,i);if(\"string\"==typeof d)r=A(d.substring(t.length+1));else{var g=p(u,t,i);if(\"string\"==typeof g&&g.length>t.length){var _=g.substring(t.length+1);r=(e.forEach(e.supportedJSExtensions,function(n){return e.tryRemoveExtension(_,n)})||_)+\".d.ts\"}else r=\"index.d.ts\"}}e.endsWith(r,\".d.ts\")||(r=A(r));var v=f(u,i),y=\"string\"==typeof u.name&&\"string\"==typeof u.version?{name:u.name,subModuleName:r,version:u.version}:void 0;return s&&(y?n(o,e.Diagnostics.Found_package_json_at_0_Package_ID_is_1,c,e.packageIdToString(y)):n(o,e.Diagnostics.Found_package_json_at_0,c)),{packageJsonContent:u,packageId:y,versionPaths:v}}l&&s&&n(o,e.Diagnostics.File_0_does_not_exist,c),i.failedLookupLocations.push(c)}function N(n){var t=n.indexOf(e.directorySeparator);return\"@\"===n[0]&&(t=n.indexOf(e.directorySeparator,t+1)),-1===t?{packageName:n,rest:\"\"}:{packageName:n.slice(0,t),rest:n.slice(t+1)}}function w(e,n,t,r,a){return P(e,n,t,r,!1,a)}function P(n,t,r,a,i,o){var s=o&&o.getOrCreateCacheForModuleName(t);return e.forEachAncestorDirectory(e.normalizeSlashes(r),function(r){if(\"node_modules\"!==e.getBaseFileName(r)){var o=U(s,t,r,a);return o||W(F(n,t,r,a,i))}})}function F(t,r,a,i,s){var l=e.combinePaths(a,\"node_modules\"),c=e.directoryProbablyExists(l,i.host);!c&&i.traceEnabled&&n(i.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,l);var u=s?void 0:G(t,r,l,c,i);if(u)return u;if(t!==o.JavaScript&&t!==o.Json){var d=e.combinePaths(l,\"@types\"),m=c;return c&&!e.directoryProbablyExists(d,i.host)&&(i.traceEnabled&&n(i.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,d),m=!1),G(o.DtsOnly,function(t,r){var a=B(t);r.traceEnabled&&a!==t&&n(r.host,e.Diagnostics.Scoped_package_detected_looking_in_0,a);return a}(r,i),d,m,i)}}function G(t,i,o,s,l){var c,u,d,m=e.normalizePath(e.combinePaths(o,i)),p=R(m,\"\",!s,l);if(p){c=p.packageJsonContent,u=p.packageId,d=p.versionPaths;var f=D(t,m,!s,l);if(f)return a(f);var g=I(t,m,!s,l,c,d);return r(u,g)}var _=function(e,n,t,a){var i=D(e,n,t,a)||I(e,n,t,a,c,d);return r(u,i)},v=N(i),y=v.packageName,h=v.rest;if(\"\"!==h){var b=e.combinePaths(o,y),E=R(b,h,!s,l);if(E&&(u=E.packageId,d=E.versionPaths),d){l.traceEnabled&&n(l.host,e.Diagnostics.package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2,d.version,e.version,h);var T=s&&e.directoryProbablyExists(b,l.host),S=V(t,h,b,d.paths,_,!T,l);if(S)return S.value}}return _(t,m,!s,l)}function V(t,r,i,o,s,l,c){var u=e.matchPatternOrExact(e.getOwnKeys(o),r);if(u){var d=e.isString(u)?void 0:e.matchedText(u,r),m=e.isString(u)?u:e.patternText(u);return c.traceEnabled&&n(c.host,e.Diagnostics.Module_name_0_matched_pattern_1,r,m),{value:e.forEach(o[m],function(r){var o=d?r.replace(\"*\",d):r,u=e.normalizePath(e.combinePaths(i,o));c.traceEnabled&&n(c.host,e.Diagnostics.Trying_substitution_0_candidate_module_location_Colon_1,r,o);var m=e.tryGetExtensionFromPath(u);if(void 0!==m){var p=M(u,l,c);if(void 0!==p)return a({path:p,ext:m})}return s(t,u,l||!e.directoryProbablyExists(e.getDirectoryPath(u),c.host),c)})}}}e.resolveTypeReferenceDirective=function(r,a,i,s){var c=t(i,s),u=[],d={compilerOptions:i,host:s,traceEnabled:c,failedLookupLocations:u},m=_(i,s);c&&(void 0===a?void 0===m?n(s,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set,r):n(s,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1,r,m):void 0===m?n(s,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set,r,a):n(s,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_1_root_directory_2,r,a,m));var p,f=function(){if(m&&m.length)return c&&n(s,e.Diagnostics.Resolving_with_primary_search_path_0,m.join(\", \")),e.forEach(m,function(t){var a=e.combinePaths(t,r),i=e.getDirectoryPath(a),u=e.directoryProbablyExists(i,s);return!u&&c&&n(s,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,i),l(O(o.DtsOnly,a,!u,d))});c&&n(s,e.Diagnostics.Root_directory_cannot_be_determined_skipping_primary_search_paths)}(),g=!0;return f||(f=function(){var t=a&&e.getDirectoryPath(a);if(void 0!==t){c&&n(s,e.Diagnostics.Looking_up_in_node_modules_folder_initial_location_0,t);var i=w(o.DtsOnly,r,t,d,void 0),u=l(i&&i.value);return!u&&c&&n(s,e.Diagnostics.Type_reference_directive_0_was_not_resolved,r),u}c&&n(s,e.Diagnostics.Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_modules_folder)}(),g=!1),f&&(i.preserveSymlinks||(f=yn({},f,{fileName:S(f.fileName,s,c)})),c&&n(s,e.Diagnostics.Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2,r,f.fileName,g),p={primary:g,resolvedFileName:f.fileName,packageId:f.packageId}),{resolvedTypeReferenceDirective:p,failedLookupLocations:u}},e.getAutomaticTypeDirectiveNames=function(n,t){if(n.types)return n.types;var r=[];if(t.directoryExists&&t.getDirectories){var a=_(n,t);if(a)for(var i=0,o=a;i<o.length;i++){var s=o[i];if(t.directoryExists(s))for(var l=0,c=t.getDirectories(s);l<c.length;l++){var u=c[l],d=e.normalizePath(u),m=e.combinePaths(s,d,\"package.json\");t.fileExists(m)&&null===e.readJson(m,t).typings||r.push(e.getBaseFileName(d))}}}return r},e.createModuleResolutionCache=function(n,t){return y(e.createMap(),e.createMap(),n,t)},e.createModuleResolutionCacheWithMaps=y,e.resolveModuleNameFromCache=function(n,t,r){var a=e.getDirectoryPath(t),i=r&&r.getOrCreateCacheForDirectory(a);return i&&i.get(n)},e.resolveModuleName=function(r,a,i,o,s){var l=t(i,o);l&&n(o,e.Diagnostics.Resolving_module_0_from_1,r,a);var c=e.getDirectoryPath(a),u=s&&s.getOrCreateCacheForDirectory(c),d=u&&u.get(r);if(d)l&&n(o,e.Diagnostics.Resolution_for_module_0_was_found_in_cache_from_location_1,r,c);else{var m=i.moduleResolution;switch(void 0===m?(m=e.getEmitModuleKind(i)===e.ModuleKind.CommonJS?e.ModuleResolutionKind.NodeJs:e.ModuleResolutionKind.Classic,l&&n(o,e.Diagnostics.Module_resolution_kind_is_not_specified_using_0,e.ModuleResolutionKind[m])):l&&n(o,e.Diagnostics.Explicitly_specified_module_resolution_kind_Colon_0,e.ModuleResolutionKind[m]),m){case e.ModuleResolutionKind.NodeJs:d=E(r,a,i,o,s);break;case e.ModuleResolutionKind.Classic:d=j(r,a,i,o,s);break;default:return e.Debug.fail(\"Unexpected moduleResolution: \"+m)}u&&(u.set(r,d),e.isExternalModuleNameRelative(r)||s.getOrCreateCacheForModuleName(r).set(c,d))}return l&&(d.resolvedModule?n(o,e.Diagnostics.Module_name_0_was_successfully_resolved_to_1,r,d.resolvedModule.resolvedFileName):n(o,e.Diagnostics.Module_name_0_was_not_resolved,r)),d},e.resolveJSModule=function(e,n,t){var r=b(e,n,t),a=r.resolvedModule,i=r.failedLookupLocations;if(!a)throw new Error(\"Could not resolve JS module '\"+e+\"' starting at '\"+n+\"'. Looked in: \"+i.join(\", \"));return a.resolvedFileName},e.tryResolveJSModule=function(e,n,t){var r=b(e,n,t).resolvedModule;return r&&r.resolvedFileName},e.nodeModuleNameResolver=E,e.nodeModulesPathPart=\"/node_modules/\",e.parsePackageName=N;var K=\"__\";function B(n){if(e.startsWith(n,\"@\")){var t=n.replace(e.directorySeparator,K);if(t!==n)return t.slice(1)}return n}function H(n){return e.stringContains(n,K)?\"@\"+n.replace(K,e.directorySeparator):n}function U(t,r,a,i){var o,s=t&&t.get(a);if(s)return i.traceEnabled&&n(i.host,e.Diagnostics.Resolution_for_module_0_was_found_in_cache_from_location_1,r,a),(o=i.failedLookupLocations).push.apply(o,s.failedLookupLocations),{value:s.resolvedModule&&{path:s.resolvedModule.resolvedFileName,originalPath:s.resolvedModule.originalPath||!0,extension:s.resolvedModule.extension,packageId:s.resolvedModule.packageId}}}function j(n,r,a,i,s){var l=[],u={compilerOptions:a,host:i,traceEnabled:t(a,i),failedLookupLocations:l},d=e.getDirectoryPath(r),m=p(o.TypeScript)||p(o.JavaScript);return c(m&&m.value,!1,l);function p(t){var r=h(t,n,d,C,u);if(r)return{value:r};if(e.isExternalModuleNameRelative(n)){var a=e.normalizePath(e.combinePaths(d,n));return W(C(t,a,!1,u))}var i=s&&s.getOrCreateCacheForModuleName(n),l=e.forEachAncestorDirectory(d,function(r){var a=U(i,n,r,u);if(a)return a;var o=e.normalizePath(e.combinePaths(r,n));return W(C(t,o,!1,u))});return l||(t===o.TypeScript?function(e,n,t){return P(o.DtsOnly,e,n,t,!0,void 0)}(n,d,u):void 0)}}function W(e){return void 0!==e?{value:e}:void 0}e.getTypesPackageName=function(e){return\"@types/\"+B(e)},e.mangleScopedPackageName=B,e.getPackageNameFromTypesPackageName=function(n){var t=e.removePrefix(n,\"@types/\");return t!==n?H(t):n},e.unmangleScopedPackageName=H,e.classicNameResolver=j,e.loadModuleFromGlobalCache=function(r,a,i,s,l){var u=t(i,s);u&&n(s,e.Diagnostics.Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using_cache_location_2,a,r,l);var d=[],m={compilerOptions:i,host:s,traceEnabled:u,failedLookupLocations:d};return c(F(o.DtsOnly,r,l,m,!1),!0,d)}}(Tn||(Tn={})),function(e){var n;function t(n){return n.body?function n(r){switch(r.kind){case 239:case 240:return 0;case 241:if(e.isEnumConst(r))return 2;break;case 247:case 246:if(!e.hasModifier(r,1))return 0;break;case 243:var a=0;return e.forEachChild(r,function(t){var r=n(t);switch(r){case 0:return;case 2:return void(a=2);case 1:return a=1,!0;default:e.Debug.assertNever(r)}}),a;case 242:return t(r);case 71:if(r.isInJSDocNamespace)return 0}return 1}(n.body):1}!function(e){e[e.NonInstantiated=0]=\"NonInstantiated\",e[e.Instantiated=1]=\"Instantiated\",e[e.ConstEnumOnly=2]=\"ConstEnumOnly\"}(e.ModuleInstanceState||(e.ModuleInstanceState={})),e.getModuleInstanceState=t,function(e){e[e.None=0]=\"None\",e[e.IsContainer=1]=\"IsContainer\",e[e.IsBlockScopedContainer=2]=\"IsBlockScopedContainer\",e[e.IsControlFlowContainer=4]=\"IsControlFlowContainer\",e[e.IsFunctionLike=8]=\"IsFunctionLike\",e[e.IsFunctionExpression=16]=\"IsFunctionExpression\",e[e.HasLocals=32]=\"HasLocals\",e[e.IsInterface=64]=\"IsInterface\",e[e.IsObjectLiteralOrClassExpressionMethod=128]=\"IsObjectLiteralOrClassExpressionMethod\"}(n||(n={}));var r=function(){var n,r,d,m,p,f,g,_,v,y,h,b,E,T,S,L,x,A,C,D,k,M,O,I,R=0,N={flags:1},w={flags:1},P=0;function F(t,r,a,i,o){return e.createDiagnosticForNodeInSourceFile(e.getSourceFileOfNode(t)||n,t,r,a,i,o)}return function(t,a){n=t,r=a,d=e.getEmitScriptTarget(r),k=function(n,t){return!(!e.getStrictOptionValue(t,\"alwaysStrict\")||n.isDeclarationFile)||!!n.externalModuleIndicator}(n,a),O=e.createUnderscoreEscapedMap(),R=0,I=n.isDeclarationFile,M=e.objectAllocator.getSymbolConstructor(),n.locals||(De(n),n.symbolCount=R,n.classifiableNames=O,function(){if(!v)return;for(var t=p,r=_,a=g,i=m,o=h,s=0,l=v;s<l.length;s++){var c=l[s],u=e.getJSDocHost(c);p=e.findAncestor(u.parent,function(e){return!!(1&ge(e))})||n,g=e.getEnclosingBlockScopeContainer(u)||n,h={flags:2},m=c,De(c.typeExpression),c.fullName&&71!==c.fullName.kind?De(c.fullName):(m=c.parent,Ee(c,524288,67897832))}p=t,_=r,g=a,m=i,h=o}());n=void 0,r=void 0,d=void 0,m=void 0,p=void 0,f=void 0,g=void 0,_=void 0,v=void 0,y=!1,h=void 0,b=void 0,E=void 0,T=void 0,S=void 0,L=void 0,A=void 0,C=!1,D=0,P=0};function G(e,n){return R++,new M(e,n)}function V(n,t,r){n.flags|=r,t.symbol=n,n.declarations=e.append(n.declarations,t),1955&r&&!n.exports&&(n.exports=e.createSymbolTable()),6240&r&&!n.members&&(n.members=e.createSymbolTable()),67220415&r&&K(n,t)}function K(n,t){var r=n.valueDeclaration;(!r||e.isAssignmentDeclaration(r)&&!e.isAssignmentDeclaration(t)||r.kind!==t.kind&&e.isEffectiveModuleDeclaration(r))&&(n.valueDeclaration=t)}function B(n){if(252===n.kind)return n.isExportEquals?\"export=\":\"default\";var t=e.getNameOfDeclaration(n);if(t){if(e.isAmbientModule(n)){var r=e.getTextOfIdentifierOrLiteral(t);return e.isGlobalScopeAugmentation(n)?\"__global\":'\"'+r+'\"'}if(147===t.kind){var a=t.expression;return e.isStringOrNumericLiteralLike(a)?e.escapeLeadingUnderscores(a.text):(e.Debug.assert(e.isWellKnownSymbolSyntactically(a)),e.getPropertyNameForKnownSymbolName(e.idText(a.name)))}return e.isPropertyNameLiteral(t)?e.getEscapedTextOfIdentifierOrLiteral(t):void 0}switch(n.kind){case 155:return\"__constructor\";case 163:case 158:case 291:return\"__call\";case 164:case 159:return\"__new\";case 160:return\"__index\";case 253:return\"__export\";case 277:return\"export=\";case 202:if(2===e.getAssignmentDeclarationKind(n))return\"export=\";e.Debug.fail(\"Unknown binary declaration kind\");break;case 287:return e.isJSDocConstructSignature(n)?\"__new\":\"__call\";case 149:e.Debug.assert(287===n.parent.kind,\"Impossible parameter parent kind\",function(){return\"parent is: \"+(e.SyntaxKind?e.SyntaxKind[n.parent.kind]:n.parent.kind)+\", expected JSDocFunctionType\"});var i=n.parent,o=i.parameters.indexOf(n);return\"arg\"+o}}function H(t,r,a,i,o,s){e.Debug.assert(!e.hasDynamicName(a));var l,c=e.hasModifier(a,512),u=c&&r?\"default\":B(a);if(void 0===u)l=G(0,\"__missing\");else if(l=t.get(u),2885600&i&&O.set(u,!0),l){if(s&&!l.isReplaceableByMethod)return l;if(l.flags&o)if(l.isReplaceableByMethod)t.set(u,l=G(0,u));else if(!(3&i&&67108864&l.flags)){e.isNamedDeclaration(a)&&(a.name.parent=a);var d=2&l.flags?e.Diagnostics.Cannot_redeclare_block_scoped_variable_0:e.Diagnostics.Duplicate_identifier_0,m=!0;(384&l.flags||384&i)&&(d=e.Diagnostics.Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations,m=!1),l.declarations&&l.declarations.length&&(c?(d=e.Diagnostics.A_module_cannot_have_multiple_default_exports,m=!1):l.declarations&&l.declarations.length&&252===a.kind&&!a.isExportEquals&&(d=e.Diagnostics.A_module_cannot_have_multiple_default_exports,m=!1));var p=function(t){var r;n.bindDiagnostics.push(F(e.getNameOfDeclaration(t)||t,d,m?(r=t,e.isNamedDeclaration(r)?e.declarationNameToString(r.name):e.unescapeLeadingUnderscores(e.Debug.assertDefined(B(r)))):void 0))};e.forEach(l.declarations,p),p(a),l=G(0,u)}}else t.set(u,l=G(0,u)),s&&(l.isReplaceableByMethod=!0);return V(l,a,i),l.parent?e.Debug.assert(l.parent===r,\"Existing symbol parent should match new one\"):l.parent=r,l}function U(n,t,r){var a=1&e.getCombinedModifierFlags(n);if(2097152&t)return 255===n.kind||246===n.kind&&a?H(p.symbol.exports,p.symbol,n,t,r):H(p.locals,void 0,n,t,r);if(e.isJSDocTypeAlias(n)&&e.Debug.assert(e.isInJSFile(n)),!e.isAmbientModule(n)&&(a||32&p.flags)||e.isJSDocTypeAlias(n)){if(e.hasModifier(n,512)&&!B(n))return H(p.symbol.exports,p.symbol,n,t,r);var i=67220415&t?1048576:0,o=H(p.locals,void 0,n,i,r);return o.exportSymbol=H(p.symbol.exports,p.symbol,n,t,r),n.localSymbol=o,o}return H(p.locals,void 0,n,t,r)}function j(e){if(I)J(e);else if(536870912&e.transformFlags)I=!0,J(e),I=!1,P|=e.transformFlags&~c(e.kind);else{var n=P;P=0,J(e),P=n|l(e,P)}}function W(e){q(e,function(e){return 237===e.kind?De(e):void 0}),q(e,function(e){return 237!==e.kind?De(e):void 0})}function q(n,t){if(void 0===t&&(t=De),void 0!==n)if(I)e.forEach(n,t);else{var r=P;P=0;for(var a=0,i=0,o=n;i<o.length;i++){var s=o[i];t(s),a|=-536870913&s.transformFlags}n.transformFlags=536870912|a,P|=r}}function z(n){e.forEachChild(n,De,q)}function J(n){if(function(n){if(!(1&h.flags))return!1;if(h===N){var i=e.isStatementButNotDeclaration(n)&&218!==n.kind||238===n.kind||242===n.kind&&function(e){var n=t(e);return 1===n||2===n&&!!r.preserveConstEnums}(n);if(i&&(h=w,!r.allowUnreachableCode)){var o=e.unreachableCodeIsError(r)&&!(4194304&n.flags)&&(!e.isVariableStatement(n)||!!(3&e.getCombinedNodeFlags(n.declarationList))||n.declarationList.declarations.some(function(e){return!!e.initializer}));!function(n,t){if(e.isStatement(n)&&a(n)&&e.isBlock(n.parent)){var r=n.parent.statements,i=e.sliceAfter(r,n);e.getRangesWhere(i,a,function(e,n){return t(i[e],i[n-1])})}else t(n,n)}(n,function(n,t){return Ce(o,n,t,e.Diagnostics.Unreachable_code_detected)})}}return!0}(n))return z(n),void ke(n);switch(n.kind){case 222:!function(e){var n=ee(),t=$(),r=$();te(n,h),h=n,ce(e.expression,t,r),h=se(t),ue(e.statement,r,n),te(n,h),h=se(r)}(n);break;case 221:!function(n){var t=ee(),r=231===n.parent.kind?e.lastOrUndefined(A):void 0,a=r?r.continueTarget:$(),i=r?r.breakTarget:$();te(t,h),h=t,ue(n.statement,i,a),te(a,h),h=se(a),ce(n.expression,t,i),h=se(i)}(n);break;case 223:!function(e){var n=ee(),t=$(),r=$();De(e.initializer),te(n,h),h=n,ce(e.condition,t,r),h=se(t),ue(e.statement,r,n),De(e.incrementor),te(n,h),h=se(r)}(n);break;case 224:case 225:!function(e){var n=ee(),t=$();te(n,h),h=n,225===e.kind&&De(e.awaitModifier);De(e.expression),te(t,h),De(e.initializer),236!==e.initializer.kind&&pe(e.initializer);ue(e.statement,t,n),te(n,h),h=se(t)}(n);break;case 220:!function(e){var n=$(),t=$(),r=$();ce(e.expression,n,t),h=se(n),De(e.thenStatement),te(r,h),h=se(t),De(e.elseStatement),te(r,h),h=se(r)}(n);break;case 228:case 232:!function(e){De(e.expression),228===e.kind&&(C=!0,T&&te(T,h));h=N}(n);break;case 227:case 226:!function(e){if(De(e.label),e.label){var n=function(e){if(A)for(var n=0,t=A;n<t.length;n++){var r=t[n];if(r.name===e)return r}return}(e.label.escapedText);n&&(n.referenced=!0,de(e,n.breakTarget,n.continueTarget))}else de(e,b,E)}(n);break;case 233:!function(e){var n=$(),t=h;De(e.tryBlock),te(n,h);var r=h,a=N;e.catchClause&&(h=t,De(e.catchClause),te(n,h),a=h);if(e.finallyBlock){var i={flags:2048,antecedent:t,lock:{}};if(te(n,i),h=se(n),De(e.finallyBlock),1&h.flags||1&r.flags&&1&a.flags&&(h=r===w||a===w?w:N),!(1&h.flags)){var o={flags:4096,antecedent:h};i.lock=o,h=o}}else h=se(n)}(n);break;case 230:!function(n){var t=$();De(n.expression);var r=b,a=x;b=t,x=h,De(n.caseBlock),te(t,h);var i=e.forEach(n.caseBlock.clauses,function(e){return 270===e.kind});n.possiblyExhaustive=!i&&!t.antecedents,i||te(t,ae(x,n,0,0));b=r,x=a,h=se(t)}(n);break;case 244:!function(n){var t=P;P=0;for(var a=n.clauses,i=N,o=0;o<a.length;o++){for(var s=o;!a[o].statements.length&&o+1<a.length;)De(a[o]),o++;var l=$();te(l,ae(x,n.parent,s,o+1)),te(l,ae(x,n.parent,s,o+1)),te(l,i),h=se(l);var c=a[o];De(c),i=h,1&h.flags||o===a.length-1||!r.noFallthroughCasesInSwitch||Ae(c,e.Diagnostics.Fallthrough_case_in_switch)}a.transformFlags=536870912|P,P|=t}(n);break;case 269:!function(e){var n=h;h=x,De(e.expression),h=n,q(e.statements)}(n);break;case 231:!function(n){var t=ee(),a=$();De(n.label),te(t,h);var i=function(e,n,t){var r={name:e,breakTarget:n,continueTarget:t,referenced:!1};return(A||(A=[])).push(r),r}(n.label.escapedText,a,t);De(n.statement),A.pop(),i.referenced||r.allowUnusedLabels||function(e,n,t){Ce(e,n,n,t)}(e.unusedLabelIsError(r),n.label,e.Diagnostics.Unused_label);n.statement&&221===n.statement.kind||(te(a,h),h=se(a))}(n);break;case 200:!function(e){if(51===e.operator){var n=S;S=L,L=n,z(e),L=S,S=n}else z(e),43!==e.operator&&44!==e.operator||pe(e.operand)}(n);break;case 201:!function(e){z(e),(43===e.operator||44===e.operator)&&pe(e.operand)}(n);break;case 202:!function(n){var t=n.operatorToken.kind;if(53===t||54===t)if(function(e){for(;193===e.parent.kind||200===e.parent.kind&&51===e.parent.operator;)e=e.parent;return!function(e){var n=e.parent;switch(n.kind){case 220:case 222:case 221:return n.expression===e;case 223:case 203:return n.condition===e}return!1}(e)&&!le(e.parent)}(n)){var r=$();fe(n,r,r),h=se(r)}else fe(n,S,L);else if(z(n),e.isAssignmentOperator(t)&&!e.isAssignmentTarget(n)&&(pe(n.left),58===t&&188===n.left.kind)){var a=n.left;Z(a.expression)&&(h=oe(h,n))}}(n);break;case 196:!function(e){z(e),187===e.expression.kind&&pe(e.expression)}(n);break;case 203:!function(e){var n=$(),t=$(),r=$();ce(e.condition,n,t),h=se(n),De(e.questionToken),De(e.whenTrue),te(r,h),h=se(t),De(e.colonToken),De(e.whenFalse),te(r,h),h=se(r)}(n);break;case 235:!function(n){z(n),(n.initializer||e.isForInOrOfStatement(n.parent.parent))&&function n(t){var r=e.isOmittedExpression(t)?void 0:t.name;if(e.isBindingPattern(r))for(var a=0,i=r.elements;a<i.length;a++){var o=i[a];n(o)}else h=ie(h,t)}(n)}(n);break;case 189:!function(n){var t=n.expression;for(;193===t.kind;)t=t.expression;194===t.kind||195===t.kind?(q(n.typeArguments),q(n.arguments),De(n.expression)):z(n);if(187===n.expression.kind){var r=n.expression;Z(r.expression)&&e.isPushOrUnshiftIdentifier(r.name)&&(h=oe(h,n))}}(n);break;case 302:case 295:!function(e){e.fullName&&u(e,e.fullName)}(n);break;case 277:W(n.statements),De(n.endOfFileToken);break;case 216:case 243:W(n.statements);break;default:z(n)}ke(n)}function X(n){switch(n.kind){case 71:case 99:case 187:case 188:return Y(n);case 189:return function(e){if(e.arguments)for(var n=0,t=e.arguments;n<t.length;n++){var r=t[n];if(Y(r))return!0}if(187===e.expression.kind&&Y(e.expression.expression))return!0;return!1}(n);case 193:return X(n.expression);case 202:return function(n){switch(n.operatorToken.kind){case 58:return Y(n.left);case 32:case 33:case 34:case 35:return Z(n.left)||Z(n.right)||Q(n.right,n.left)||Q(n.left,n.right);case 93:return Z(n.left);case 92:return t=n.left,r=n.right,e.isStringLiteralLike(t)&&X(r);case 26:return X(n.right)}var t,r;return!1}(n);case 200:return 51===n.operator&&X(n.operand);case 197:return X(n.expression)}return!1}function Y(n){return 71===n.kind||99===n.kind||97===n.kind||e.isPropertyAccessExpression(n)&&Y(n.expression)||e.isElementAccessExpression(n)&&n.argumentExpression&&(e.isStringLiteral(n.argumentExpression)||e.isNumericLiteral(n.argumentExpression))&&Y(n.expression)}function Q(n,t){return e.isTypeOfExpression(n)&&Z(n.expression)&&e.isStringLiteralLike(t)}function Z(e){switch(e.kind){case 193:return Z(e.expression);case 202:switch(e.operatorToken.kind){case 58:return Z(e.left);case 26:return Z(e.right)}}return Y(e)}function $(){return{flags:4,antecedents:void 0}}function ee(){return{flags:8,antecedents:void 0}}function ne(e){e.flags|=512&e.flags?1024:512}function te(n,t){1&t.flags||e.contains(n.antecedents,t)||((n.antecedents||(n.antecedents=[])).push(t),ne(t))}function re(e,n,t){return 1&n.flags?n:t?101===t.kind&&64&e||86===t.kind&&32&e?N:X(t)?(ne(n),{flags:e,expression:t,antecedent:n}):n:32&e?n:N}function ae(e,n,t,r){return X(n.expression)?(ne(e),{flags:128,switchStatement:n,clauseStart:t,clauseEnd:r,antecedent:e}):e}function ie(e,n){return ne(e),{flags:16,antecedent:e,node:n}}function oe(e,n){ne(e);var t={flags:256,antecedent:e,node:n};return t}function se(e){var n=e.antecedents;return n?1===n.length?n[0]:e:N}function le(e){for(;;)if(193===e.kind)e=e.expression;else{if(200!==e.kind||51!==e.operator)return 202===e.kind&&(53===e.operatorToken.kind||54===e.operatorToken.kind);e=e.operand}}function ce(e,n,t){var r=S,a=L;S=n,L=t,De(e),S=r,L=a,e&&le(e)||(te(n,re(32,h,e)),te(t,re(64,h,e)))}function ue(e,n,t){var r=b,a=E;b=n,E=t,De(e),b=r,E=a}function de(e,n,t){var r=227===e.kind?n:t;r&&(te(r,h),h=N)}function me(e){202===e.kind&&58===e.operatorToken.kind?pe(e.left):pe(e)}function pe(e){if(Y(e))h=ie(h,e);else if(185===e.kind)for(var n=0,t=e.elements;n<t.length;n++){var r=t[n];206===r.kind?pe(r.expression):me(r)}else if(186===e.kind)for(var a=0,i=e.properties;a<i.length;a++){var o=i[a];273===o.kind?me(o.initializer):274===o.kind?pe(o.name):275===o.kind&&pe(o.expression)}}function fe(e,n,t){var r=$();53===e.operatorToken.kind?ce(e.left,r,t):ce(e.left,n,r),h=se(r),De(e.operatorToken),ce(e.right,n,t)}function ge(n){switch(n.kind){case 207:case 238:case 241:case 186:case 166:case 290:case 266:return 1;case 239:return 65;case 242:case 240:case 179:return 33;case 277:return 37;case 154:if(e.isObjectLiteralOrClassExpressionMethod(n))return 173;case 155:case 237:case 153:case 156:case 157:case 158:case 291:case 287:case 163:case 159:case 160:case 164:return 45;case 194:case 195:return 61;case 243:return 4;case 152:return n.initializer?4:0;case 272:case 223:case 224:case 225:case 244:return 2;case 216:return e.isFunctionLike(n.parent)?0:2}return 0}function _e(e){_&&(_.nextContainer=e),_=e}function ve(t,r,a){switch(p.kind){case 242:return U(t,r,a);case 277:return function(t,r,a){return e.isExternalModule(n)?U(t,r,a):H(n.locals,void 0,t,r,a)}(t,r,a);case 207:case 238:return function(n,t,r){return e.hasModifier(n,32)?H(p.symbol.exports,p.symbol,n,t,r):H(p.symbol.members,p.symbol,n,t,r)}(t,r,a);case 241:return H(p.symbol.exports,p.symbol,t,r,a);case 166:case 290:case 186:case 239:case 266:return H(p.symbol.members,p.symbol,t,r,a);case 163:case 164:case 158:case 159:case 291:case 160:case 154:case 153:case 155:case 156:case 157:case 237:case 194:case 195:case 287:case 302:case 295:case 240:case 179:return H(p.locals,void 0,t,r,a)}}function ye(e){4194304&e.flags&&!function(e){var n=277===e.kind?e:e.body;if(n&&(277===n.kind||243===n.kind))for(var t=0,r=n.statements;t<r.length;t++){var a=r[t];if(253===a.kind||252===a.kind)return!0}return!1}(e)?e.flags|=32:e.flags&=-33}function he(e){var n=t(e),r=0!==n;return ve(e,r?512:1024,r?110735:0),n}function be(e,n,t){var r=G(n,t);return 106508&n&&(r.parent=p.symbol),V(r,e,n),r}function Ee(n,t,r){switch(g.kind){case 242:U(n,t,r);break;case 277:if(e.isExternalOrCommonJsModule(p)){U(n,t,r);break}default:g.locals||(g.locals=e.createSymbolTable(),_e(g)),H(g.locals,void 0,n,t,r)}}function Te(t){!(k&&t.originalKeywordKind>=108&&t.originalKeywordKind<=116)||e.isIdentifierName(t)||4194304&t.flags||n.parseDiagnostics.length||n.bindDiagnostics.push(F(t,function(t){if(e.getContainingClass(t))return e.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_strict_mode;if(n.externalModuleIndicator)return e.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode;return e.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode}(t),e.declarationNameToString(t)))}function Se(t,r){if(r&&71===r.kind){var a=r;if(o=a,e.isIdentifier(o)&&(\"eval\"===o.escapedText||\"arguments\"===o.escapedText)){var i=e.getErrorSpanForNode(n,r);n.bindDiagnostics.push(e.createFileDiagnostic(n,i.start,i.length,function(t){if(e.getContainingClass(t))return e.Diagnostics.Invalid_use_of_0_Class_definitions_are_automatically_in_strict_mode;if(n.externalModuleIndicator)return e.Diagnostics.Invalid_use_of_0_Modules_are_automatically_in_strict_mode;return e.Diagnostics.Invalid_use_of_0_in_strict_mode}(t),e.idText(a)))}}var o}function Le(e){k&&Se(e,e.name)}function xe(t){if(d<2&&277!==g.kind&&242!==g.kind&&!e.isFunctionLike(g)){var r=e.getErrorSpanForNode(n,t);n.bindDiagnostics.push(e.createFileDiagnostic(n,r.start,r.length,function(t){if(e.getContainingClass(t))return e.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_definitions_are_automatically_in_strict_mode;if(n.externalModuleIndicator)return e.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_are_automatically_in_strict_mode;return e.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5}(t)))}}function Ae(t,r,a,i,o){var s=e.getSpanOfTokenAtPosition(n,t.pos);n.bindDiagnostics.push(e.createFileDiagnostic(n,s.start,s.length,r,a,i,o))}function Ce(t,r,a,i){!function(t,r,a){var i=e.createFileDiagnostic(n,r.pos,r.end-r.pos,a);t?n.bindDiagnostics.push(i):n.bindSuggestionDiagnostics=e.append(n.bindSuggestionDiagnostics,yn({},i,{category:e.DiagnosticCategory.Suggestion}))}(t,{pos:e.getTokenPosOfNode(r,n),end:a.end},i)}function De(t){if(t){t.parent=m;var a=k;if(function(t){switch(t.kind){case 71:if(t.isInJSDocNamespace){for(var a=t.parent;a&&!e.isJSDocTypeAlias(a);)a=a.parent;Ee(a,524288,67897832);break}case 99:return h&&(e.isExpression(t)||274===m.kind)&&(t.flowNode=h),Te(t);case 187:case 188:h&&Y(t)&&(t.flowNode=h),e.isSpecialPropertyDeclaration(t)&&function(n){99===n.expression.kind?Ne(n):e.isPropertyAccessEntityNameExpression(n)&&277===n.parent.parent.kind&&(e.isPrototypeAccess(n.expression)?we(n,n.parent):Pe(n))}(t),e.isInJSFile(t)&&n.commonJsModuleIndicator&&e.isModuleExportsPropertyAccessExpression(t)&&!s(g,\"module\")&&H(n.locals,void 0,t.expression,134217729,67220414);break;case 202:var l=e.getAssignmentDeclarationKind(t);switch(l){case 1:Re(t);break;case 2:!function(t){if(!Ie(t))return;var r=e.getRightMostAssignedExpression(t.right);if(e.isEmptyObjectLiteral(r)||p===n&&i(n,r))return;var a=e.exportAssignmentIsAlias(t)?2097152:1049092;H(n.symbol.exports,n.symbol,t,a,0)}(t);break;case 3:we(t.left,t);break;case 6:!function(e){e.left.parent=e,e.right.parent=e;var n=e.left;Fe(n.expression,n,!1)}(t);break;case 4:Ne(t);break;case 5:!function(t){var r=t.left,a=Ge(r.expression);if(!e.isInJSFile(t)&&!e.isFunctionSymbol(a))return;t.left.parent=t,t.right.parent=t,e.isIdentifier(r.expression)&&p===n&&o(n,r.expression)?Re(t):Pe(r)}(t);break;case 0:break;default:e.Debug.fail(\"Unknown special property assignment kind\")}return function(n){k&&e.isLeftHandSideExpression(n.left)&&e.isAssignmentOperator(n.operatorToken.kind)&&Se(n,n.left)}(t);case 272:return function(e){k&&e.variableDeclaration&&Se(e,e.variableDeclaration.name)}(t);case 196:return function(t){if(k&&71===t.expression.kind){var r=e.getErrorSpanForNode(n,t.expression);n.bindDiagnostics.push(e.createFileDiagnostic(n,r.start,r.length,e.Diagnostics.delete_cannot_be_called_on_an_identifier_in_strict_mode))}}(t);case 8:return function(t){k&&32&t.numericLiteralFlags&&n.bindDiagnostics.push(F(t,e.Diagnostics.Octal_literals_are_not_allowed_in_strict_mode))}(t);case 201:return function(e){k&&Se(e,e.operand)}(t);case 200:return function(e){k&&(43!==e.operator&&44!==e.operator||Se(e,e.operand))}(t);case 229:return function(n){k&&Ae(n,e.Diagnostics.with_statements_are_not_allowed_in_strict_mode)}(t);case 231:return function(n){k&&r.target>=2&&(e.isDeclarationStatement(n.statement)||e.isVariableStatement(n.statement))&&Ae(n.label,e.Diagnostics.A_label_is_not_allowed_here)}(t);case 176:return void(y=!0);case 161:break;case 148:return function(n){if(e.isJSDocTemplateTag(n.parent)){var t=e.find(n.parent.parent.tags,e.isJSDocTypeAlias)||e.getHostSignatureFromJSDoc(n.parent);t?(t.locals||(t.locals=e.createSymbolTable()),H(t.locals,void 0,n,262144,67635688)):ve(n,262144,67635688)}else if(174===n.parent.kind){var r=function(n){var t=e.findAncestor(n,function(n){return n.parent&&e.isConditionalTypeNode(n.parent)&&n.parent.extendsType===n});return t&&t.parent}(n.parent);r?(r.locals||(r.locals=e.createSymbolTable()),H(r.locals,void 0,n,262144,67635688)):be(n,262144,B(n))}else ve(n,262144,67635688)}(t);case 149:return Be(t);case 235:return Ke(t);case 184:return t.flowNode=h,Ke(t);case 152:case 151:return function(e){return He(e,4|(e.questionToken?16777216:0),0)}(t);case 273:case 274:return He(t,4,0);case 276:return He(t,8,68008959);case 158:case 159:case 160:return ve(t,131072,0);case 154:case 153:return He(t,8192|(t.questionToken?16777216:0),e.isObjectLiteralMethod(t)?0:67212223);case 237:return function(t){n.isDeclarationFile||4194304&t.flags||e.isAsyncFunction(t)&&(D|=1024);Le(t),k?(xe(t),Ee(t,16,67219887)):ve(t,16,67219887)}(t);case 155:return ve(t,16384,0);case 156:return He(t,32768,67154879);case 157:return He(t,65536,67187647);case 163:case 287:case 291:case 164:return function(n){var t=G(131072,B(n));V(t,n,131072);var r=G(2048,\"__type\");V(r,n,2048),r.members=e.createSymbolTable(),r.members.set(t.escapedName,t)}(t);case 166:case 290:case 179:return function(e){return be(e,2048,\"__type\")}(t);case 186:return function(t){var r;if(function(e){e[e.Property=1]=\"Property\",e[e.Accessor=2]=\"Accessor\"}(r||(r={})),k)for(var a=e.createUnderscoreEscapedMap(),i=0,o=t.properties;i<o.length;i++){var s=o[i];if(275!==s.kind&&71===s.name.kind){var l=s.name,c=273===s.kind||274===s.kind||154===s.kind?1:2,u=a.get(l.escapedText);if(u){if(1===c&&1===u){var d=e.getErrorSpanForNode(n,l);n.bindDiagnostics.push(e.createFileDiagnostic(n,d.start,d.length,e.Diagnostics.An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode))}}else a.set(l.escapedText,c)}}return be(t,4096,\"__object\")}(t);case 194:case 195:return function(t){n.isDeclarationFile||4194304&t.flags||e.isAsyncFunction(t)&&(D|=1024);h&&(t.flowNode=h);Le(t);var r=t.name?t.name.escapedText:\"__function\";return be(t,16,r)}(t);case 189:e.isInJSFile(t)&&function(t){!n.commonJsModuleIndicator&&e.isRequireCall(t,!1)&&Ie(t)}(t);break;case 207:case 238:return k=!0,function(t){if(238===t.kind)Ee(t,32,68008383);else{var r=t.name?t.name.escapedText:\"__class\";be(t,32,r),t.name&&O.set(t.name.escapedText,!0)}var a=t.symbol,i=G(4194308,\"prototype\"),o=a.exports.get(i.escapedName);o&&(t.name&&(t.name.parent=t),n.bindDiagnostics.push(F(o.declarations[0],e.Diagnostics.Duplicate_identifier_0,e.symbolName(i))));a.exports.set(i.escapedName,i),i.parent=a}(t);case 239:return Ee(t,64,67897736);case 240:return Ee(t,524288,67897832);case 241:return function(n){return e.isEnumConst(n)?Ee(n,128,68008831):Ee(n,256,68008191)}(t);case 242:return function(t){if(ye(t),e.isAmbientModule(t))if(e.hasModifier(t,1)&&Ae(t,e.Diagnostics.export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always_visible),e.isModuleAugmentationExternal(t))he(t);else{var r=void 0;if(9===t.name.kind){var a=t.name.text;e.hasZeroOrOneAsteriskCharacter(a)?r=e.tryParsePattern(a):Ae(t.name,e.Diagnostics.Pattern_0_can_have_at_most_one_Asterisk_character,a)}var i=ve(t,512,110735);n.patternAmbientModules=e.append(n.patternAmbientModules,r&&{pattern:r,symbol:i})}else{var o=he(t);if(0!==o){var i=t.symbol;i.constEnumOnlyModule=!(304&i.flags)&&2===o&&!1!==i.constEnumOnlyModule}}}(t);case 266:return function(e){return be(e,4096,\"__jsxAttributes\")}(t);case 265:return function(e,n,t){return ve(e,n,t)}(t,4,0);case 246:case 249:case 251:case 255:return ve(t,2097152,2097152);case 245:return function(t){t.modifiers&&t.modifiers.length&&n.bindDiagnostics.push(F(t,e.Diagnostics.Modifiers_cannot_appear_here));if(277!==t.parent.kind)return void n.bindDiagnostics.push(F(t,e.Diagnostics.Global_module_exports_may_only_appear_at_top_level));var r=t.parent;if(!e.isExternalModule(r))return void n.bindDiagnostics.push(F(t,e.Diagnostics.Global_module_exports_may_only_appear_in_module_files));if(!r.isDeclarationFile)return void n.bindDiagnostics.push(F(t,e.Diagnostics.Global_module_exports_may_only_appear_in_declaration_files));n.symbol.globalExports=n.symbol.globalExports||e.createSymbolTable(),H(n.symbol.globalExports,n.symbol,t,2097152,2097152)}(t);case 248:return function(e){e.name&&ve(e,2097152,2097152)}(t);case 253:return function(e){p.symbol&&p.symbol.exports?e.exportClause||H(p.symbol.exports,p.symbol,e,8388608,0):be(e,8388608,B(e))}(t);case 252:return function(n){if(p.symbol&&p.symbol.exports){var t=e.exportAssignmentIsAlias(n)?2097152:4,r=H(p.symbol.exports,p.symbol,n,t,67108863);n.isExportEquals&&K(r,n)}else be(n,2097152,B(n))}(t);case 277:return Me(t.statements),function(){if(ye(n),e.isExternalModule(n))Oe();else if(e.isJsonSourceFile(n)){Oe();var t=n.symbol;H(n.symbol.exports,n.symbol,n,4,67108863),n.symbol=t}}();case 216:if(!e.isFunctionLike(t.parent))return;case 243:return Me(t.statements);case 297:if(291===t.parent.kind)return Be(t);if(290!==t.parent.kind)break;case 303:var c=t,u=c.isBracketed||c.typeExpression&&286===c.typeExpression.type.kind?16777220:4;return ve(c,u,0);case 302:case 295:(v||(v=[])).push(t)}}(t),t.kind>145){var c=m;m=t;var u=ge(t);0===u?j(t):function(n,t){var r=p,a=f,i=g;1&t?(195!==n.kind&&(f=p),p=g=n,32&t&&(p.locals=e.createSymbolTable()),_e(p)):2&t&&((g=n).locals=void 0);if(4&t){var o=h,s=b,l=E,c=T,u=A,d=C,m=16&t&&!e.hasModifier(n,256)&&!n.asteriskToken&&!!e.getImmediatelyInvokedFunctionExpression(n);m||(h={flags:2},144&t&&(h.container=n)),T=m||155===n.kind?$():void 0,b=void 0,E=void 0,A=void 0,C=!1,j(n),n.flags&=-1409,!(1&h.flags)&&8&t&&e.nodeIsPresent(n.body)&&(n.flags|=128,C&&(n.flags|=256)),277===n.kind&&(n.flags|=D),T&&(te(T,h),h=se(T),155===n.kind&&(n.returnFlowNode=h)),m||(h=o),b=s,E=l,T=c,A=u,C=d}else 64&t?(y=!1,j(n),n.flags=y?64|n.flags:-65&n.flags):j(n);p=r,f=a,g=i}(t,u),m=c}else if(!I&&0==(536870912&t.transformFlags)){P|=l(t,0);var c=m;1===t.kind&&(m=t),ke(t),m=c}k=a}}function ke(n){if(e.hasJSDocNodes(n))if(e.isInJSFile(n))for(var t=0,r=n.jsDoc;t<r.length;t++){var a=r[t];De(a)}else for(var i=0,o=n.jsDoc;i<o.length;i++){var a=o[i];u(n,a)}}function Me(t){if(!k)for(var r=0,a=t;r<a.length;r++){var i=a[r];if(!e.isPrologueDirective(i))return;if(o=i,s=void 0,'\"use strict\"'===(s=e.getSourceTextOfNodeFromSourceFile(n,o.expression))||\"'use strict'\"===s)return void(k=!0)}var o,s}function Oe(){be(n,512,'\"'+e.removeFileExtension(n.fileName)+'\"')}function Ie(e){return!n.externalModuleIndicator&&(n.commonJsModuleIndicator||(n.commonJsModuleIndicator=e,Oe()),!0)}function Re(n){if(Ie(n)){var t=n.left,r=Ve(t.expression,void 0,function(e,n){return n&&V(n,e,67110400),n});if(r){var a=e.isClassExpression(n.right)?1048612:1048580;H(r.exports,r,t,a,0)}}}function Ne(n){e.Debug.assert(e.isInJSFile(n));var t=e.getThisContainer(n,!1);switch(t.kind){case 237:case 194:var r=t.symbol;if(e.isBinaryExpression(t.parent)&&58===t.parent.operatorToken.kind){var a=t.parent.left;e.isPropertyAccessEntityNameExpression(a)&&e.isPrototypeAccess(a.expression)&&(r=Ge(a.expression.expression,f))}r&&(r.members=r.members||e.createSymbolTable(),H(r.members,r,n,4,0));break;case 155:case 152:case 154:case 156:case 157:var i=t.parent,o=e.hasModifier(t,32)?i.symbol.exports:i.symbol.members;H(o,i.symbol,n,4,0,!0);break;case 277:break;default:e.Debug.fail(e.Debug.showSyntaxKind(t))}}function we(e,n){var t=e.expression,r=t.expression;e.parent=n,r.parent=t,t.parent=e,Fe(r,e,!0)}function Pe(e){e.expression.parent=e,Fe(e.expression,e,!1)}function Fe(t,r,a){var i=Ge(t),o=e.isBinaryExpression(r.parent)?277===function(n){for(;e.isBinaryExpression(n.parent);)n=n.parent;return n.parent}(r.parent).parent.kind:277===r.parent.parent.kind;if(o&&!a&&(!i||!(1920&i.flags))){i=Ve(r.expression,i,function(t,r,a){if(r)return V(r,t,67110400),r;var i=a?a.exports:n.jsGlobalAugmentations||(n.jsGlobalAugmentations=e.createSymbolTable());return H(i,a,t,67110400,110735)})}if(i&&function(n){if(1072&n.flags)return!0;var t=n.valueDeclaration,r=t?e.isVariableDeclaration(t)?t.initializer:e.isBinaryExpression(t)?t.right:e.isPropertyAccessExpression(t)&&e.isBinaryExpression(t.parent)?t.parent.right:void 0:void 0;if(r=r&&e.getRightMostAssignedExpression(r)){var a=e.isPrototypeAccess(e.isVariableDeclaration(t)?t.name:e.isBinaryExpression(t)?t.left:t);return!!e.getExpandoInitializer(e.isBinaryExpression(r)&&54===r.operatorToken.kind?r.right:r,a)}return!1}(i)){var s=a?i.members||(i.members=e.createSymbolTable()):i.exports||(i.exports=e.createSymbolTable()),l=e.isFunctionLikeDeclaration(e.getAssignedExpandoInitializer(r)),c=l?8192:4,u=l?67212223:0;H(s,i,r,67108864|c,-67108865&u)}}function Ge(n,t){if(void 0===t&&(t=p),e.isIdentifier(n))return s(t,n.escapedText);var r=Ge(n.expression);return r&&r.exports&&r.exports.get(n.name.escapedText)}function Ve(t,r,a){if(i(n,t))return n.symbol;if(e.isIdentifier(t))return a(t,Ge(t),r);var o=Ve(t.expression,r,a);return o&&o.exports?a(t.name,o.exports.get(t.name.escapedText),o):e.Debug.fail()}function Ke(n){if(k&&Se(n,n.name),!e.isBindingPattern(n.name)){var t=e.isInJSFile(n)&&!!e.getJSDocEnumTag(n),r=t?256:0,a=t?68008191:0;e.isBlockOrCatchScoped(n)?Ee(n,2|r,67220415|a):e.isParameterDeclaration(n)?ve(n,1,67220415):ve(n,1|r,67220414|a)}}function Be(n){if((297!==n.kind||291===p.kind)&&(!k||4194304&n.flags||Se(n,n.name),e.isBindingPattern(n.name)?be(n,1,\"__\"+n.parent.parameters.indexOf(n)):ve(n,1,67220415),e.isParameterPropertyDeclaration(n))){var t=n.parent.parent;H(t.symbol.members,t.symbol,n,4|(n.questionToken?16777216:0),0)}}function He(t,r,a){return n.isDeclarationFile||4194304&t.flags||!e.isAsyncFunction(t)||(D|=1024),h&&e.isObjectLiteralOrClassExpressionMethod(t)&&(t.flowNode=h),e.hasDynamicName(t)?be(t,r,\"__computed\"):ve(t,r,a)}}();function a(n){return!(e.isFunctionDeclaration(n)||function(n){switch(n.kind){case 239:case 240:return!0;case 242:return 1!==t(n);case 241:return e.hasModifier(n,2048);default:return!1}}(n)||e.isEnumDeclaration(n)||e.isVariableStatement(n)&&!(3&e.getCombinedNodeFlags(n))&&n.declarationList.declarations.some(function(e){return!e.initializer}))}function i(n,t){return e.isExportsIdentifier(t)||e.isModuleExportsPropertyAccessExpression(t)||e.isIdentifier(t)&&o(n,t)}function o(n,t){var r=s(n,t.escapedText);return!!r&&!!r.valueDeclaration&&e.isVariableDeclaration(r.valueDeclaration)&&!!r.valueDeclaration.initializer&&function n(t,r){return i(t,r)||e.isAssignmentExpression(r,!0)&&(n(t,r.left)||n(t,r.right))}(n,r.valueDeclaration.initializer)}function s(n,t){var r=n.locals&&n.locals.get(t);return r?r.exportSymbol||r:e.isSourceFile(n)&&n.jsGlobalAugmentations&&n.jsGlobalAugmentations.has(t)?n.jsGlobalAugmentations.get(t):n.symbol&&n.symbol.exports&&n.symbol.exports.get(t)}function l(n,t){var r=n.kind;switch(r){case 189:return function(e,n){var t=n,r=e.expression;e.typeArguments&&(t|=3);(524288&n||402653184&r.transformFlags)&&(t|=192,268435456&r.transformFlags&&(t|=16384));91===r.kind&&(t|=67108864,16384&n&&(t|=32768));return e.transformFlags=536870912|t,-940049730&t}(n,t);case 190:return function(e,n){var t=n;e.typeArguments&&(t|=3);524288&n&&(t|=192);return e.transformFlags=536870912|t,-940049730&t}(n,t);case 242:return function(n,t){var r=3;0==(2&e.getModifierFlags(n))&&(r|=t);return n.transformFlags=536870912|r,-977327426&r}(n,t);case 193:return function(e,n){var t=n,r=e.expression,a=r.kind,i=r.transformFlags;210!==a&&192!==a||(t|=3);1024&i&&(t|=1024);return e.transformFlags=536870912|t,-536872258&t}(n,t);case 202:return function(e,n){var t=n,r=e.operatorToken.kind,a=e.left.kind;58===r&&186===a?t|=3272:58===r&&185===a?t|=3264:40!==r&&62!==r||(t|=32);return e.transformFlags=536870912|t,-939525442&t}(n,t);case 219:return function(e,n){var t=n;1024&e.expression.transformFlags&&(t|=192);return e.transformFlags=536870912|t,-939525442&t}(n,t);case 149:return function(n,t){var r=t,a=n.name,i=n.initializer,o=n.dotDotDotToken;(n.questionToken||n.type||4096&t||e.isThisIdentifier(a))&&(r|=3);e.hasModifier(n,92)&&(r|=262147);1048576&t&&(r|=8);(8388608&t||i||o)&&(r|=131264);return n.transformFlags=536870912|r,-939525442&r}(n,t);case 195:return function(n,t){var r=192|t;(e.hasModifier(n,2270)||n.typeParameters||n.type)&&(r|=3);e.hasModifier(n,256)&&(r|=16);1048576&t&&(r|=8);16384&t&&(r|=32768);return n.transformFlags=536870912|r,-1003902274&r}(n,t);case 194:return function(n,t){var r=t;(e.hasModifier(n,2270)||n.typeParameters||n.type)&&(r|=3);e.hasModifier(n,256)&&(r|=n.asteriskToken?8:16);1048576&t&&(r|=8);163840&t&&(r|=192);n.asteriskToken&&(r|=768);return n.transformFlags=536870912|r,-1003935042&r}(n,t);case 237:return function(n,t){var r,a=e.getModifierFlags(n);!n.body||2&a?r=3:(r=33554432|t,(2270&a||n.typeParameters||n.type)&&(r|=3),256&a&&(r|=n.asteriskToken?8:16),1048576&t&&(r|=8),163840&t&&(r|=192),n.asteriskToken&&(r|=768));return n.transformFlags=536870912|r,-1003935042&r}(n,t);case 235:return function(e,n){var t=n;t|=8388800,1048576&n&&(t|=8);e.type&&(t|=3);return e.transformFlags=536870912|t,-939525442&t}(n,t);case 236:return function(e,n){var t=33554432|n;8388608&n&&(t|=192);3&e.flags&&(t|=4194496);return e.transformFlags=536870912|t,-948962626&t}(n,t);case 217:return function(n,t){var r,a=n.declarationList.transformFlags;e.hasModifier(n,2)?r=3:(r=t,8388608&a&&(r|=192));return n.transformFlags=536870912|r,-939525442&r}(n,t);case 231:return function(n,t){var r=t;4194304&t&&e.isIterationStatement(n,!0)&&(r|=192);return n.transformFlags=536870912|r,-939525442&r}(n,t);case 238:return function(n,t){var r;e.hasModifier(n,2)?r=3:(r=192|t,(274432&t||n.typeParameters)&&(r|=3),65536&t&&(r|=16384));return n.transformFlags=536870912|r,-942011714&r}(n,t);case 207:return function(e,n){var t=192|n;(274432&n||e.typeParameters)&&(t|=3);65536&n&&(t|=16384);return e.transformFlags=536870912|t,-942011714&t}(n,t);case 271:return function(n,t){var r=t;switch(n.token){case 85:r|=192;break;case 108:r|=3;break;default:e.Debug.fail(\"Unexpected token for heritage clause\")}return n.transformFlags=536870912|r,-939525442&r}(n,t);case 272:return function(n,t){var r=t;n.variableDeclaration?e.isBindingPattern(n.variableDeclaration.name)&&(r|=192):r|=8;return n.transformFlags=536870912|r,-940574018&r}(n,t);case 209:return function(e,n){var t=192|n;e.typeArguments&&(t|=3);return e.transformFlags=536870912|t,-939525442&t}(n,t);case 155:return function(n,t){var r=t;!e.hasModifier(n,2270)&&n.body||(r|=3);1048576&t&&(r|=8);return n.transformFlags=536870912|r,-1003668802&r}(n,t);case 152:return function(n,t){var r=3|t;(n.initializer||e.isComputedPropertyName(n.name))&&(r|=8192);return n.transformFlags=536870912|r,-939525442&r}(n,t);case 154:return function(n,t){var r=192|t;(n.decorators||e.hasModifier(n,2270)||n.typeParameters||n.type||n.name&&e.isComputedPropertyName(n.name)||!n.body)&&(r|=3);1048576&t&&(r|=8);e.hasModifier(n,256)&&(r|=n.asteriskToken?8:16);n.asteriskToken&&(r|=768);return n.transformFlags=536870912|r,-1003668802&r}(n,t);case 156:case 157:return function(n,t){var r=t;(n.decorators||e.hasModifier(n,2270)||n.type||n.name&&e.isComputedPropertyName(n.name)||!n.body)&&(r|=3);1048576&t&&(r|=8);return n.transformFlags=536870912|r,-1003668802&r}(n,t);case 246:return function(n,t){var r=t;e.isExternalModuleImportEqualsDeclaration(n)||(r|=3);return n.transformFlags=536870912|r,-939525442&r}(n,t);case 187:return function(e,n){var t=n;134217728&t&&(t^=134217728,t|=268435456);return e.transformFlags=536870912|t,-671089986&t}(n,t);case 188:return function(e,n){var t=n;134217728&e.expression.transformFlags&&(t&=-134217729,t|=268435456);return e.transformFlags=536870912|t,-671089986&t}(n,t);default:return function(e,n,t){var r=t,a=939525441;switch(n){case 120:case 199:r|=24;break;case 192:case 210:case 306:r|=3,a=536872257;break;case 114:case 112:case 113:case 117:case 124:case 76:case 241:case 276:case 211:case 132:r|=3;break;case 258:case 259:case 260:case 10:case 261:case 262:case 263:case 264:case 265:case 266:case 267:case 268:r|=4;break;case 13:case 14:case 15:case 16:case 204:case 191:case 274:case 115:case 212:r|=192;break;case 9:e.hasExtendedUnicodeEscape&&(r|=192);break;case 8:384&e.numericLiteralFlags&&(r|=192);break;case 225:e.awaitModifier&&(r|=8),r|=192;break;case 205:r|=16777416;break;case 119:case 134:case 131:case 135:case 137:case 122:case 138:case 105:case 148:case 151:case 153:case 158:case 159:case 160:case 161:case 162:case 163:case 164:case 165:case 166:case 167:case 168:case 169:case 170:case 171:case 172:case 173:case 174:case 175:case 239:case 240:case 176:case 177:case 178:case 179:case 180:case 245:r=3,a=-3;break;case 147:r|=2097152,16384&t&&(r|=65536);break;case 206:r|=524480;break;case 275:r|=1048584;break;case 97:r|=134217920,a=536872257;break;case 99:r|=16384;break;case 182:r|=8388800,524288&t&&(r|=1048584),a=940049729;break;case 183:r|=8388800,a=940049729;break;case 184:r|=192,e.dotDotDotToken&&(r|=524288);break;case 150:r|=4099;break;case 186:a=942740801,2097152&t&&(r|=192),65536&t&&(r|=16384),1048576&t&&(r|=8);break;case 185:case 190:a=940049729,524288&t&&(r|=192);break;case 221:case 222:case 223:case 224:4194304&t&&(r|=192);break;case 277:32768&t&&(r|=192);break;case 228:r|=33554440;break;case 226:case 227:r|=33554432}return e.transformFlags=536870912|r,r&~a}(n,r,t)}}function c(e){if(e>=161&&e<=181)return-3;switch(e){case 189:case 190:case 185:return 940049729;case 242:return 977327425;case 149:return 939525441;case 195:return 1003902273;case 194:case 237:return 1003935041;case 236:return 948962625;case 238:case 207:return 942011713;case 155:return 1003668801;case 154:case 156:case 157:return 1003668801;case 119:case 134:case 131:case 137:case 135:case 122:case 138:case 105:case 148:case 151:case 153:case 158:case 159:case 160:case 239:case 240:return-3;case 186:return 942740801;case 272:return 940574017;case 182:case 183:return 940049729;case 192:case 210:case 306:case 193:case 97:return 536872257;case 187:case 188:return 671089985;default:return 939525441}}function u(n,t){t.parent=n,e.forEachChild(t,function(e){return u(t,e)})}e.bindSourceFile=function(n,t){e.performance.mark(\"beforeBind\"),r(n,t),e.performance.mark(\"afterBind\"),e.performance.measure(\"Bind\",\"beforeBind\",\"afterBind\")},e.isExportsOrModuleExportsOrAlias=i,e.computeTransformFlagsForNode=l,e.getTransformFlagsSubtreeExclusions=c}(Tn||(Tn={})),function(e){e.createGetSymbolWalker=function(n,t,r,a,i,o,s,l,c,u){return function(d){void 0===d&&(d=function(){return!0});var m=[],p=[];return{walkType:function(n){try{return f(n),{visitedTypes:e.getOwnValues(m),visitedSymbols:e.getOwnValues(p)}}finally{e.clear(m),e.clear(p)}},walkSymbol:function(n){try{return v(n),{visitedTypes:e.getOwnValues(m),visitedSymbols:e.getOwnValues(p)}}finally{e.clear(m),e.clear(p)}}};function f(n){if(n&&!m[n.id]){m[n.id]=n;var t=v(n.symbol);if(!t){if(131072&n.flags){var r=n,i=r.objectFlags;4&i&&function(n){f(n.target),e.forEach(n.typeArguments,f)}(n),32&i&&function(e){f(e.typeParameter),f(e.constraintType),f(e.templateType),f(e.modifiersType)}(n),3&i&&(_(o=n),e.forEach(o.typeParameters,f),e.forEach(a(o),f),f(o.thisType)),24&i&&_(r)}var o;65536&n.flags&&function(e){f(c(e))}(n),786432&n.flags&&function(n){e.forEach(n.types,f)}(n),1048576&n.flags&&function(e){f(e.type)}(n),2097152&n.flags&&function(e){f(e.objectType),f(e.indexType),f(e.constraint)}(n)}}}function g(a){var i=t(a);i&&f(i.type),e.forEach(a.typeParameters,f);for(var o=0,s=a.parameters;o<s.length;o++){var l=s[o];v(l)}f(n(a)),f(r(a))}function _(e){var n=l(e,0);f(n);var t=l(e,1);f(t);for(var r=i(e),a=0,o=r.callSignatures;a<o.length;a++){var s=o[a];g(s)}for(var c=0,u=r.constructSignatures;c<u.length;c++){var s=u[c];g(s)}for(var d=0,m=r.properties;d<m.length;d++){var p=m[d];v(p)}}function v(n){if(!n)return!1;var t=e.getSymbolId(n);if(p[t])return!1;if(p[t]=n,!d(n))return!0;var r=o(n);return f(r),n.exports&&n.exports.forEach(v),e.forEach(n.declarations,function(e){if(e.type&&165===e.type.kind){var n=e.type,t=s(u(n.exprName));v(t)}}),!1}}}}(Tn||(Tn={})),function(e){var n,t=/^\".+\"$/,r=1,a=1,i=1,o=1;function s(e){return e.id||(e.id=a,a++),e.id}function l(e){return e.id||(e.id=r,r++),e.id}function c(n,t){var r=e.getModuleInstanceState(n);return 1===r||t&&2===r}function u(n){switch(n.parent.kind){case 251:case 255:return e.isIdentifier(n);default:return e.isDeclarationName(n)}}function d(e){switch(e.kind){case 248:case 246:case 249:case 251:return!0;case 71:return 251===e.parent.kind;default:return!1}}e.getNodeId=s,e.getSymbolId=l,e.isInstantiatedModule=c,e.createTypeChecker=function(r,a){var m,p,f,g=e.memoize(function(){var n=e.createMap();return r.getSourceFiles().forEach(function(t){t.resolvedModules&&e.forEachEntry(t.resolvedModules,function(e){e&&e.packageId&&n.set(e.packageId.name,!0)})}),n}),_=e.objectAllocator.getSymbolConstructor(),v=e.objectAllocator.getTypeConstructor(),y=e.objectAllocator.getSignatureConstructor(),h=0,b=0,E=0,T=0,S=0,L=e.createSymbolTable(),x=e.identity,A=r.getCompilerOptions(),C=e.getEmitScriptTarget(A),D=e.getEmitModuleKind(A),k=e.getAllowSyntheticDefaultImports(A),M=e.getStrictOptionValue(A,\"strictNullChecks\"),O=e.getStrictOptionValue(A,\"strictFunctionTypes\"),I=e.getStrictOptionValue(A,\"strictPropertyInitialization\"),R=e.getStrictOptionValue(A,\"noImplicitAny\"),N=e.getStrictOptionValue(A,\"noImplicitThis\"),w=!!A.keyofStringsOnly,P=function(){var n,t=r.getResolvedTypeReferenceDirectives();return t&&(n=e.createMap(),t.forEach(function(e,t){if(e&&e.resolvedFileName){var a=r.getSourceFile(e.resolvedFileName);n.set(a.path,t)}})),{getReferencedExportContainer:Ov,getReferencedImportDeclaration:Iv,getReferencedDeclarationWithCollidingName:Nv,isDeclarationWithCollidingName:wv,isValueAliasDeclaration:function(n){return!(n=e.getParseTreeNode(n))||Pv(n)},hasGlobalName:$v,isReferencedAliasDeclaration:function(n,t){return!(n=e.getParseTreeNode(n))||function n(t,r){if(e.isAliasSymbolDeclaration(t)){var a=Er(t);if(a&&It(a).referenced)return!0;var i=It(a).target;if(i&&1&e.getModifierFlags(t)&&67220415&i.flags&&(A.preserveConstEnums||!Vv(i)))return!0}return!!r&&!!e.forEachChild(t,function(e){return n(e,r)})}(n,t)},getNodeCheckFlags:function(n){return(n=e.getParseTreeNode(n))?Wv(n):0},isTopLevelValueImportEqualsWithEntityName:Fv,isDeclarationVisible:ra,isImplementationOfOverload:Kv,isRequiredInitializedParameter:Bv,isOptionalUninitializedParameterProperty:Hv,isExpandoFunctionDeclaration:Uv,getPropertiesOfContainerFunction:jv,createTypeOfDeclaration:Yv,createReturnTypeOfSignatureDeclaration:Qv,createTypeOfExpression:Zv,createLiteralConstValue:ry,isSymbolAccessible:Hr,isEntityNameVisible:zr,getConstantValue:function(n){var t=e.getParseTreeNode(n,zv);return t?Jv(t):void 0},collectLinkedAliases:aa,getReferencedValueDeclaration:ny,getTypeReferenceSerializationKind:Xv,isOptionalParameter:ko,moduleExportsSomeValue:Mv,isArgumentsLocalBinding:kv,getExternalModuleFileFromDeclaration:ay,getTypeReferenceDirectivesForEntityName:function(e){if(n){var t=67899368;(71===e.kind&&td(e)||187===e.kind&&!function(e){return e.parent&&209===e.parent.kind&&e.parent.parent&&271===e.parent.parent.kind}(e))&&(t=68268991);var r=ir(e,t,!0);return r&&r!==Q?a(r,t):void 0}},getTypeReferenceDirectivesForSymbol:a,isLiteralConstDeclaration:ty,isLateBound:function(n){var t=e.getParseTreeNode(n,e.isDeclaration),r=t&&Er(t);return!!(r&&1024&e.getCheckFlags(r))},getJsxFactoryEntity:function(n){return n?(yt(n),e.getSourceFileOfNode(n).localJsxFactory||Zn):Zn},getAllAccessorDeclarations:function(n){var t=157===(n=e.getParseTreeNode(n,e.isGetOrSetAccessorDeclaration)).kind?156:157,r=e.getDeclarationOfKind(Er(n),t),a=r&&r.pos<n.pos?r:n,i=r&&r.pos<n.pos?n:r,o=157===n.kind?n:r,s=156===n.kind?n:r;return{firstAccessor:a,secondAccessor:i,setAccessor:o,getAccessor:s}},getSymbolOfExternalModuleSpecifier:function(e){return lr(e,e,void 0)}};function a(t,r){if(n&&function(t){if(!t.declarations)return!1;for(var r=t;;){var a=Tr(r);if(!a)break;r=a}if(r.valueDeclaration&&277===r.valueDeclaration.kind&&512&r.flags)return!1;for(var i=0,o=t.declarations;i<o.length;i++){var s=o[i],l=e.getSourceFileOfNode(s);if(n.has(l.path))return!0}return!1}(t)){for(var a,i=0,o=t.declarations;i<o.length;i++){var s=o[i];if(s.symbol&&s.symbol.flags&r){var l=e.getSourceFileOfNode(s),c=n.get(l.path);if(!c)return;(a||(a=[])).push(c)}}return a}}}(),F=function(){return{typeToTypeNode:function(e,t,r,a){return n(t,r,a,function(n){return i(e,n)})},indexInfoToIndexSignatureDeclaration:function(e,t,r,a,i){return n(r,a,i,function(n){return c(e,t,n)})},signatureToSignatureDeclaration:function(e,t,r,a,i){return n(r,a,i,function(n){return u(e,t,n)})},symbolToEntityName:function(e,t,r,a,i){return n(r,a,i,function(n){return E(e,n,t,!1)})},symbolToExpression:function(t,r,a,i,o){return n(a,i,o,function(n){return function(n,t,r){var a=v(n,t,r);return function n(r,a){var i=h(r,a,t),o=r[a];0===a&&(t.flags|=16777216);var s=ta(o,t);0===a&&(t.flags^=16777216);var l=s.charCodeAt(0),c=e.isIdentifierStart(l,C);if(0===a||c){var u=e.setEmitFlags(e.createIdentifier(s,i),16777216);return u.symbol=o,a>0?e.createPropertyAccess(n(r,a-1),u):u}91===l&&(s=s.substring(1,s.length-1),l=s.charCodeAt(0));var d=void 0;return e.isSingleOrDoubleQuote(l)?(d=e.createLiteral(s.substring(1,s.length-1).replace(/\\\\./g,function(e){return e.substring(1)}))).singleQuote=39===l:\"\"+ +s===s&&(d=e.createLiteral(+s)),d||((d=e.setEmitFlags(e.createIdentifier(s,i),16777216)).symbol=o),e.createElementAccess(n(r,a-1),d)}(a,a.length-1)}(t,n,r)})},symbolToTypeParameterDeclarations:function(e,t,r,a){return n(t,r,a,function(n){return y(e,n)})},symbolToParameterDeclaration:function(e,t,r,a){return n(t,r,a,function(n){return g(e,n)})},typeParameterToDeclaration:function(e,t,r,a){return n(t,r,a,function(n){return f(e,n)})}};function n(n,t,r,a){e.Debug.assert(void 0===n||0==(8&n.flags));var i={enclosingDeclaration:n,flags:t||0,tracker:r&&r.trackSymbol?r:{trackSymbol:e.noop},encounteredError:!1,visitedSymbols:void 0,inferTypeParameters:void 0,approximateLength:0},o=a(i);return i.encounteredError?void 0:o}function a(n){return n.truncating?n.truncating:n.truncating=!(1&n.flags)&&n.approximateLength>e.defaultMaximumTruncationLength}function i(n,t){m&&m.throwIfCancellationRequested&&m.throwIfCancellationRequested();var r=8388608&t.flags;if(t.flags&=-8388609,n){if(1&n.flags)return t.approximateLength+=3,e.createKeywordTypeNode(119);if(2&n.flags)return e.createKeywordTypeNode(142);if(4&n.flags)return t.approximateLength+=6,e.createKeywordTypeNode(137);if(8&n.flags)return t.approximateLength+=6,e.createKeywordTypeNode(134);if(16&n.flags)return t.approximateLength+=7,e.createKeywordTypeNode(122);if(512&n.flags&&!(262144&n.flags)){var f=Tr(n.symbol),g=E(f,t,67897832,!1),_=si(f)===n?g:e.createQualifiedName(g,e.symbolName(n.symbol));return t.approximateLength+=e.symbolName(n.symbol).length,e.createTypeReferenceNode(_,void 0)}if(544&n.flags){var v=E(n.symbol,t,67897832,!1);return t.approximateLength+=e.symbolName(n.symbol).length,e.createTypeReferenceNode(v,void 0)}if(64&n.flags)return t.approximateLength+=n.value.length+2,e.createLiteralTypeNode(e.setEmitFlags(e.createLiteral(n.value),16777216));if(128&n.flags)return t.approximateLength+=(\"\"+n.value).length,e.createLiteralTypeNode(e.createLiteral(n.value));if(256&n.flags)return t.approximateLength+=n.intrinsicName.length,\"true\"===n.intrinsicName?e.createTrue():e.createFalse();if(2048&n.flags){if(!(1048576&t.flags)){if(Br(n.symbol,t.enclosingDeclaration))return t.approximateLength+=6,b(n.symbol,t,67220415);t.tracker.reportInaccessibleUniqueSymbolError&&t.tracker.reportInaccessibleUniqueSymbolError()}return t.approximateLength+=13,e.createTypeOperatorNode(141,e.createKeywordTypeNode(138))}if(4096&n.flags)return t.approximateLength+=4,e.createKeywordTypeNode(105);if(8192&n.flags)return t.approximateLength+=9,e.createKeywordTypeNode(140);if(16384&n.flags)return t.approximateLength+=4,e.createKeywordTypeNode(95);if(32768&n.flags)return t.approximateLength+=5,e.createKeywordTypeNode(131);if(1024&n.flags)return t.approximateLength+=6,e.createKeywordTypeNode(138);if(16777216&n.flags)return t.approximateLength+=6,e.createKeywordTypeNode(135);if(65536&n.flags&&n.isThisType)return 4194304&t.flags&&(t.encounteredError||32768&t.flags||(t.encounteredError=!0),t.tracker.reportInaccessibleThisError&&t.tracker.reportInaccessibleThisError()),t.approximateLength+=4,e.createThis();var y=e.getObjectFlags(n);if(4&y)return e.Debug.assert(!!(131072&n.flags)),function(n){var r=n.typeArguments||e.emptyArray;if(n.target===Ge){if(2&t.flags){var a=i(r[0],t);return e.createTypeReferenceNode(\"Array\",[a])}var o=i(r[0],t);return e.createArrayTypeNode(o)}if(8&n.target.objectFlags){if(r.length>0){var l=cs(n),c=s(r.slice(0,l),t),u=n.target.hasRestElement;if(c){for(var d=n.target.minLength;d<Math.min(l,c.length);d++)c[d]=u&&d===l-1?e.createRestTypeNode(e.createArrayTypeNode(c[d])):e.createOptionalTypeNode(c[d]);return e.createTupleTypeNode(c)}}return t.encounteredError||524288&t.flags?e.createTupleTypeNode([]):void(t.encounteredError=!0)}if(2048&t.flags&&n.symbol.valueDeclaration&&e.isClassLike(n.symbol.valueDeclaration)&&!Br(n.symbol,t.enclosingDeclaration))return R(n);var m=n.target.outerTypeParameters,d=0,p=void 0;if(m)for(var f=m.length;d<f;){var g=d,_=as(m[d]);do{d++}while(d<f&&as(m[d])===_);if(!e.rangeEquals(m,r,g,d)){var v=s(r.slice(g,d),t),y=t.flags;t.flags|=16;var h=b(_,t,67897832,v);t.flags=y,p=p?w(p,h):h}}var E=void 0;if(r.length>0){var T=(n.target.typeParameters||e.emptyArray).length;E=s(r.slice(d,T),t)}var S=t.flags;t.flags|=16;var L=b(n.symbol,t,67897832,E);return t.flags=S,p?w(p,L):L}(n);if(65536&n.flags||3&y){if(65536&n.flags&&e.contains(t.inferTypeParameters,n))return t.approximateLength+=e.symbolName(n.symbol).length+6,e.createInferTypeNode(p(n,t,void 0));if(4&t.flags&&65536&n.flags&&e.length(n.symbol.declarations)&&e.isTypeParameterDeclaration(n.symbol.declarations[0])&&d(n,t)&&!Kr(n.symbol,t.enclosingDeclaration)){var v=n.symbol.declarations[0].name;return t.approximateLength+=e.idText(v).length,e.createTypeReferenceNode(e.getGeneratedNameForNode(v,24),void 0)}return n.symbol?b(n.symbol,t,67897832):e.createTypeReferenceNode(e.createIdentifier(\"?\"),void 0)}if(!r&&n.aliasSymbol&&(16384&t.flags||Kr(n.aliasSymbol,t.enclosingDeclaration))){var h=s(n.aliasTypeArguments,t);return!Ir(n.aliasSymbol.escapedName)||32&n.aliasSymbol.flags?b(n.aliasSymbol,t,67897832,h):e.createTypeReferenceNode(e.createIdentifier(\"\"),h)}if(!(786432&n.flags)){if(48&y)return e.Debug.assert(!!(131072&n.flags)),R(n);if(1048576&n.flags){var T=n.type;t.approximateLength+=6;var S=i(T,t);return e.createTypeOperatorNode(S)}if(2097152&n.flags){var L=i(n.objectType,t),S=i(n.indexType,t);return t.approximateLength+=2,e.createIndexedAccessTypeNode(L,S)}if(4194304&n.flags){var x=i(n.checkType,t),A=t.inferTypeParameters;t.inferTypeParameters=n.root.inferTypeParameters;var C=i(n.extendsType,t);t.inferTypeParameters=A;var D=i(Ll(n),t),k=i(xl(n),t);return t.approximateLength+=15,e.createConditionalTypeNode(x,C,D,k)}return 8388608&n.flags?i(n.typeVariable,t):e.Debug.fail(\"Should be unreachable.\")}var M=262144&n.flags?function(e){for(var n=[],t=0,r=0;r<e.length;r++){var a=e[r];if(t|=a.flags,!(24576&a.flags)){if(768&a.flags){var i=256&a.flags?fe:ri(a);if(262144&i.flags){var o=i.types.length;if(r+o<=e.length&&Vl(e[r+o-1])===Vl(i.types[o-1])){n.push(i),r+=o-1;continue}}}n.push(a)}}return 16384&t&&n.push(oe),8192&t&&n.push(ae),n||e}(n.types):n.types,O=s(M,t,!0);if(O&&O.length>0){var I=e.createUnionOrIntersectionTypeNode(262144&n.flags?171:172,O);return I}t.encounteredError||262144&t.flags||(t.encounteredError=!0)}else t.encounteredError=!0;function R(n){var r,a=n.symbol;if(a){var i=16&e.getObjectFlags(n)&&n.symbol&&32&n.symbol.flags;if(r=(i?\"+\":\"\")+l(a),gf(a.valueDeclaration)){var o=n===hf(a)?67897832:67220415;return b(a,t,o)}if(32&a.flags&&!wa(a)&&!(207===a.valueDeclaration.kind&&2048&t.flags)||896&a.flags||function(){var n=!!(8192&a.flags)&&e.some(a.declarations,function(n){return e.hasModifier(n,32)}),i=!!(16&a.flags)&&(a.parent||e.forEach(a.declarations,function(e){return 277===e.parent.kind||243===e.parent.kind}));if(n||i)return(!!(4096&t.flags)||t.visitedSymbols&&t.visitedSymbols.has(r))&&(!(8&t.flags)||Br(a,t.enclosingDeclaration))}())return b(a,t,67220415);if(t.visitedSymbols&&t.visitedSymbols.has(r)){var s=function(n){if(n.symbol&&2048&n.symbol.flags){var t=e.findAncestor(n.symbol.declarations[0].parent,function(e){return 175!==e.kind});if(240===t.kind)return Er(t)}}(n);return s?b(s,t,67897832):(t.approximateLength+=3,e.createKeywordTypeNode(119))}t.visitedSymbols||(t.visitedSymbols=e.createMap()),t.visitedSymbols.set(r,!0);var c=N(n);return t.visitedSymbols.delete(r),c}return N(n)}function N(n){if(Ji(n))return function(n){e.Debug.assert(!!(131072&n.flags));var r,a=n.declaration.readonlyToken?e.createToken(n.declaration.readonlyToken.kind):void 0,o=n.declaration.questionToken?e.createToken(n.declaration.questionToken.kind):void 0;r=Ui(n)?e.createTypeOperatorNode(i(ji(n),t)):i(Ki(n),t);var s=p(Vi(n),t,r),l=i(Bi(n),t),c=e.createMappedTypeNode(a,s,o,l);return t.approximateLength+=10,e.setEmitFlags(c,1)}(n);var r=Xi(n);if(!r.properties.length&&!r.stringIndexInfo&&!r.numberIndexInfo){if(!r.callSignatures.length&&!r.constructSignatures.length)return t.approximateLength+=2,e.setEmitFlags(e.createTypeLiteralNode(void 0),1);if(1===r.callSignatures.length&&!r.constructSignatures.length){var s=r.callSignatures[0],l=u(s,163,t);return l}if(1===r.constructSignatures.length&&!r.callSignatures.length){var s=r.constructSignatures[0],l=u(s,164,t);return l}}var d=t.flags;t.flags|=4194304;var m=function(n){if(a(t))return[e.createPropertySignature(void 0,\"...\",void 0,void 0,void 0)];for(var r=[],i=0,s=n.callSignatures;i<s.length;i++){var l=s[i];r.push(u(l,158,t))}for(var d=0,m=n.constructSignatures;d<m.length;d++){var l=m[d];r.push(u(l,159,t))}if(n.stringIndexInfo){var p=2048&n.objectFlags?ns($,n.stringIndexInfo.isReadonly,n.stringIndexInfo.declaration):n.stringIndexInfo;r.push(c(p,0,t))}n.numberIndexInfo&&r.push(c(n.numberIndexInfo,1,t));var f=n.properties;if(!f)return r;for(var g=0,_=0,v=f;_<v.length;_++){var y=v[_];if(g++,2048&t.flags){if(4194304&y.flags)continue;24&e.getDeclarationModifierFlagsFromSymbol(y)&&t.tracker.reportPrivateInBaseOfClassExpression&&t.tracker.reportPrivateInBaseOfClassExpression(e.unescapeLeadingUnderscores(y.escapedName))}if(a(t)&&g+2<f.length-1){r.push(e.createPropertySignature(void 0,\"... \"+(f.length-g)+\" more ...\",void 0,void 0,void 0)),o(f[f.length-1],t,r);break}o(y,t,r)}return r.length?r:void 0}(r);t.flags=d;var f=e.createTypeLiteralNode(m);return t.approximateLength+=2,e.setEmitFlags(f,1024&t.flags?0:1)}function w(n,t){if(e.isImportTypeNode(n)){var r=n.typeArguments;n.qualifier&&((e.isIdentifier(n.qualifier)?n.qualifier:n.qualifier.right).typeArguments=r),n.typeArguments=t.typeArguments;for(var a=P(t),i=0,o=a;i<o.length;i++){var s=o[i];n.qualifier=n.qualifier?e.createQualifiedName(n.qualifier,s):s}return n}var r=n.typeArguments;(e.isIdentifier(n.typeName)?n.typeName:n.typeName.right).typeArguments=r,n.typeArguments=t.typeArguments;for(var a=P(t),l=0,c=a;l<c.length;l++){var s=c[l];n.typeName=e.createQualifiedName(n.typeName,s)}return n}function P(n){for(var t=n.typeName,r=[];!e.isIdentifier(t);)r.unshift(t.right),t=t.left;return r.unshift(t),r}}function o(n,t,r){var a=2048&e.getCheckFlags(n)&&33554432&t.flags?$:Ga(n),o=t.enclosingDeclaration;if(t.enclosingDeclaration=void 0,t.tracker.trackSymbol&&1024&e.getCheckFlags(n)){var s=e.first(n.declarations);hi(s)&&_(s.name,o,t)}var l=E(n,t,67220415,!0);t.approximateLength+=e.symbolName(n).length+1,t.enclosingDeclaration=o;var c=16777216&n.flags?e.createToken(55):void 0;if(8208&n.flags&&!Yi(a).length)for(var d=ho(a,0),m=0,p=d;m<p.length;m++){var f=p[m],g=u(f,153,t);g.name=l,g.questionToken=c,n.valueDeclaration&&e.setCommentRange(g,n.valueDeclaration),r.push(g)}else{var v=t.flags;t.flags|=2048&e.getCheckFlags(n)?33554432:0;var y=a?i(a,t):e.createKeywordTypeNode(119);t.flags=v;var h=Zf(n)?[e.createToken(132)]:void 0;h&&(t.approximateLength+=9);var b=e.createPropertySignature(h,l,c,y,void 0);n.valueDeclaration&&e.setCommentRange(b,n.valueDeclaration),r.push(b)}}function s(n,t,r){if(e.some(n)){if(a(t)){if(!r)return[e.createTypeReferenceNode(\"...\",void 0)];if(n.length>2)return[i(n[0],t),e.createTypeReferenceNode(\"... \"+(n.length-2)+\" more ...\",void 0),i(n[n.length-1],t)]}for(var o=[],s=0,l=0,c=n;l<c.length;l++){var u=c[l];if(s++,a(t)&&s+2<n.length-1){o.push(e.createTypeReferenceNode(\"... \"+(n.length-s)+\" more ...\",void 0));var d=i(n[n.length-1],t);d&&o.push(d);break}t.approximateLength+=2;var m=i(u,t);m&&o.push(m)}return o}}function c(n,t,r){var a=e.getNameFromIndexInfo(n)||\"x\",o=e.createKeywordTypeNode(0===t?137:134),s=e.createParameter(void 0,void 0,void 0,a,void 0,o,void 0),l=i(n.type||$,r);return n.type||2097152&r.flags||(r.encounteredError=!0),r.approximateLength+=a.length+4,e.createIndexSignature(void 0,n.isReadonly?[e.createToken(132)]:void 0,[s],l)}function u(n,t,r){var a,o;32&r.flags&&n.target&&n.mapper&&n.target.typeParameters?o=n.target.typeParameters.map(function(e){return i(lc(e,n.mapper),r)}):a=n.typeParameters&&n.typeParameters.map(function(e){return f(e,r)});var s,l=function(n){if(n.hasRestParameter){var t=n.parameters.length-1,r=n.parameters[t],a=Ga(r);if(pu(a)){var i=a.typeArguments||e.emptyArray,o=a.target.minLength,s=a.target.hasRestElement?i.length-1:-1,l=e.map(i,function(e,r){var a=kf(n,t+r),i=r===s?8192:r>=o?4096:0,l=Tt(1,a,i);return l.type=r===s?Ws(e):e,l});return e.concatenate(n.parameters.slice(0,t),l)}}return n.parameters}(n).map(function(e){return g(e,r,155===t)});if(n.thisParameter){var c=g(n.thisParameter,r);l.unshift(c)}var u=Bo(n);if(u){var d=1===u.kind?e.setEmitFlags(e.createIdentifier(u.parameterName),16777216):e.createThisTypeNode(),m=i(u.type,r);s=e.createTypePredicateNode(d,m)}else{var p=Ho(n);s=p&&i(p,r)}return 256&r.flags?s&&119===s.kind&&(s=void 0):s||(s=e.createKeywordTypeNode(119)),r.approximateLength+=3,e.createSignatureDeclaration(t,a,l,s,o)}function d(e,n){return!!Ft(n.enclosingDeclaration,e.symbol.escapedName,67897832,void 0,e.symbol.escapedName,!1)}function p(n,t,r){var a=t.flags;t.flags&=-513;var o=4&t.flags&&n.symbol.declarations[0]&&e.isTypeParameterDeclaration(n.symbol.declarations[0])&&d(n,t),s=o?e.getGeneratedNameForNode(n.symbol.declarations[0].name,24):E(n.symbol,t,67897832,!0),l=uo(n),c=l&&i(l,t);return t.flags=a,e.createTypeParameterDeclaration(s,r,c)}function f(e,n,t){void 0===t&&(t=no(e));var r=t&&i(t,n);return p(e,n,r)}function g(n,t,r){var a=e.getDeclarationOfKind(n,149);a||St(n)||(a=e.getDeclarationOfKind(n,297));var o=Ga(n);a&&Bv(a)&&(o=Eu(o));var s=i(o,t),l=!(8192&t.flags)&&r&&a&&a.modifiers?a.modifiers.map(e.getSynthesizedClone):void 0,c=a&&e.isRestParameter(a)||8192&e.getCheckFlags(n),u=c?e.createToken(24):void 0,d=a&&a.name?71===a.name.kind?e.setEmitFlags(e.getSynthesizedClone(a.name),16777216):146===a.name.kind?e.setEmitFlags(e.getSynthesizedClone(a.name.right),16777216):function n(r){t.tracker.trackSymbol&&e.isComputedPropertyName(r)&&vi(r)&&_(r,t.enclosingDeclaration,t);var a=e.visitEachChild(r,n,e.nullTransformationContext,void 0,n),i=e.nodeIsSynthesized(a)?a:e.getSynthesizedClone(a);return 184===i.kind&&(i.initializer=void 0),e.setEmitFlags(i,16777217)}(a.name):e.symbolName(n),m=a&&ko(a)||4096&e.getCheckFlags(n),p=m?e.createToken(55):void 0,f=e.createParameter(void 0,l,u,d,p,s,void 0);return t.approximateLength+=e.symbolName(n).length+3,f}function _(e,n,t){if(t.tracker.trackSymbol){var r=ev(e.expression),a=Ft(r,r.escapedText,68268991,void 0,void 0,!0);a&&t.tracker.trackSymbol(a,n,67220415)}}function v(n,t,r,a){var i;t.tracker.trackSymbol(n,t.enclosingDeclaration,r);var o=262144&n.flags;return!o&&(t.enclosingDeclaration||64&t.flags)?(i=e.Debug.assertDefined(function n(r,i,o){var s=Gr(r,t.enclosingDeclaration,i,!!(128&t.flags));if(!s||Vr(s[0],t.enclosingDeclaration,1===s.length?i:Fr(i))){var l=Sr(s?s[0]:r,t.enclosingDeclaration);if(e.length(l))for(var c=0,u=l;c<u.length;c++){var d=u[c],m=n(d,Fr(i),!1);if(m){s=m.concat(s||[Lr(d,r)||r]);break}}}if(s)return s;if(o||!(6144&r.flags)){if(!o&&!a&&e.forEach(r.declarations,Wr))return;return[r]}}(n,r,!0)),e.Debug.assert(i&&i.length>0)):i=[n],i}function y(n,t){var r,a=q_(n);return 524384&a.flags&&(r=e.createNodeArray(e.map(Wa(n),function(e){return f(e,t)}))),r}function h(n,t,r){e.Debug.assert(n&&0<=t&&t<n.length);var a,i=n[t];if(512&r.flags&&t<n.length-1){var o=i,l=n[t+1];if(1&e.getCheckFlags(l)){var c=function(n){return e.concatenate(ja(n),Wa(n))}(2097152&o.flags?er(o):o);a=s(e.map(c,l.mapper),r)}else a=y(i,r)}return a}function b(n,a,i,o){var s=v(n,a,i,!(16384&a.flags)),l=67220415===i;if(e.some(s[0].declarations,Wr)){var c=s.length>1?b(s,s.length-1,1):void 0,u=o||h(s,0,a),d=function(n,a){var i=e.getDeclarationOfKind(n,277);if(i&&void 0!==i.moduleName)return i.moduleName;if(!i){if(a.tracker.trackReferencedAmbientModule){var o=e.filter(n.declarations,e.isAmbientModule);if(e.length(o))for(var s=0,l=o;s<l.length;s++){var c=l[s];a.tracker.trackReferencedAmbientModule(c,n)}}if(t.test(n.escapedName))return n.escapedName.substring(1,n.escapedName.length-1)}if(!a.enclosingDeclaration||!a.tracker.moduleResolverHost)return t.test(n.escapedName)?n.escapedName.substring(1,n.escapedName.length-1):e.getSourceFileOfNode(e.getNonAugmentationDeclaration(n)).fileName;var u=e.getSourceFileOfNode(e.getOriginalNode(a.enclosingDeclaration)),d=It(n),m=d.specifierCache&&d.specifierCache.get(u.path);if(!m){var p=A.out||A.outFile,f=a.tracker.moduleResolverHost,g=p?yn({},A,{baseUrl:f.getCommonSourceDirectory()}):A;m=e.first(e.moduleSpecifiers.getModuleSpecifiers(n,g,u,f,r.getSourceFiles(),{importModuleSpecifierPreference:p?\"non-relative\":\"relative\"},r.redirectTargetsMap)),d.specifierCache=d.specifierCache||e.createMap(),d.specifierCache.set(u.path,m)}return m}(s[0],a),m=e.createLiteralTypeNode(e.createLiteral(d));if(a.tracker.trackExternalModuleSymbolOfImportTypeNode&&a.tracker.trackExternalModuleSymbolOfImportTypeNode(s[0]),a.approximateLength+=d.length+10,!c||e.isEntityName(c)){if(c){var p=e.isIdentifier(c)?c:c.right;p.typeArguments=void 0}return e.createImportTypeNode(m,c,u,l)}var f=function n(t){return e.isIndexedAccessTypeNode(t.objectType)?n(t.objectType):t}(c),g=f.objectType.typeName;return e.createIndexedAccessTypeNode(e.createImportTypeNode(m,g,u,l),f.indexType)}var _=b(s,s.length-1,0);if(e.isIndexedAccessTypeNode(_))return _;if(l)return e.createTypeQueryNode(_);var p=e.isIdentifier(_)?_:_.right,y=p.typeArguments;return p.typeArguments=void 0,e.createTypeReferenceNode(_,y);function b(n,t,r){var i=t===n.length-1?o:h(n,t,a),s=n[t];0===t&&(a.flags|=16777216);var l=ta(s,a);a.approximateLength+=l.length+1,0===t&&(a.flags^=16777216);var c=n[t-1];if(!(16&a.flags)&&c&&Li(c)&&Li(c).get(s.escapedName)===s){var u=b(n,t-1,r);return e.isIndexedAccessTypeNode(u)?e.createIndexedAccessTypeNode(u,e.createLiteralTypeNode(e.createLiteral(l))):e.createIndexedAccessTypeNode(e.createTypeReferenceNode(u,i),e.createLiteralTypeNode(e.createLiteral(l)))}var d=e.setEmitFlags(e.createIdentifier(l,i),16777216);if(d.symbol=s,t>r){var u=b(n,t-1,r);return e.isEntityName(u)?e.createQualifiedName(u,d):e.Debug.fail(\"Impossible construct - an export of an indexed access cannot be reachable\")}return d}}function E(n,t,r,a){var i=v(n,t,r);return!a||1===i.length||t.encounteredError||65536&t.flags||(t.encounteredError=!0),function n(r,a){var i=h(r,a,t),o=r[a];0===a&&(t.flags|=16777216);var s=ta(o,t);0===a&&(t.flags^=16777216);var l=e.setEmitFlags(e.createIdentifier(s,i),16777216);return l.symbol=o,a>0?e.createQualifiedName(n(r,a-1),l):l}(i,i.length-1)}}(),G=Tt(4,\"undefined\");G.declarations=[];var V,K=Tt(4,\"arguments\"),B=Tt(4,\"require\"),H={getNodeCount:function(){return e.sum(r.getSourceFiles(),\"nodeCount\")},getIdentifierCount:function(){return e.sum(r.getSourceFiles(),\"identifierCount\")},getSymbolCount:function(){return e.sum(r.getSourceFiles(),\"symbolCount\")+b},getTypeCount:function(){return h},isUndefinedSymbol:function(e){return e===G},isArgumentsSymbol:function(e){return e===K},isUnknownSymbol:function(e){return e===Q},getMergedSymbol:br,getDiagnostics:gv,getGlobalDiagnostics:function(){return _v(),Xn.getGlobalDiagnostics()},getTypeOfSymbolAtLocation:function(n,t){return(t=e.getParseTreeNode(t))?function(n,t){if(n=n.exportSymbol||n,71===t.kind&&(e.isRightSideOfQualifiedNameOrPropertyAccess(t)&&(t=t.parent),e.isExpressionNode(t)&&!e.isAssignmentTarget(t))){var r=Tg(t);if(xr(Rt(t).resolvedSymbol)===n)return r}return Ga(n)}(n,t):te},getSymbolsOfParameterPropertyDeclaration:function(n,t){var r=e.getParseTreeNode(n,e.isParameter);return void 0===r?e.Debug.fail(\"Cannot get symbols of a synthetic parameter that cannot be resolved to a parse-tree node.\"):function(n,t){var r=n.parent,a=n.parent.parent,i=wt(r.locals,t,67220415),o=wt(Li(a.symbol),t,67220415);return i&&o?[i,o]:e.Debug.fail(\"There should exist two symbols, one as property declaration and one as parameter declaration\")}(r,e.escapeLeadingUnderscores(t))},getDeclaredTypeOfSymbol:si,getPropertiesOfType:$i,getPropertyOfType:function(n,t){return vo(n,e.escapeLeadingUnderscores(t))},getTypeOfPropertyOfType:function(n,t){return ua(n,e.escapeLeadingUnderscores(t))},getIndexInfoOfType:To,getSignaturesOfType:ho,getIndexTypeOfType:So,getBaseTypes:Za,getBaseTypeOfLiteralType:cu,getWidenedType:Mu,getTypeFromTypeNode:function(n){var t=e.getParseTreeNode(n,e.isTypeNode);return t?Ul(t):te},getParameterType:Mf,getPromisedTypeOfPromise:qg,getReturnTypeOfSignature:Ho,getNullableType:bu,getNonNullableType:Tu,typeToTypeNode:F.typeToTypeNode,indexInfoToIndexSignatureDeclaration:F.indexInfoToIndexSignatureDeclaration,signatureToSignatureDeclaration:F.signatureToSignatureDeclaration,symbolToEntityName:F.symbolToEntityName,symbolToExpression:F.symbolToExpression,symbolToTypeParameterDeclarations:F.symbolToTypeParameterDeclarations,symbolToParameterDeclaration:F.symbolToParameterDeclaration,typeParameterToDeclaration:F.typeParameterToDeclaration,getSymbolsInScope:function(n,t){return(n=e.getParseTreeNode(n))?function(n,t){if(8388608&n.flags)return[];var r=e.createSymbolTable(),a=!1;return function(){for(;n;){switch(n.locals&&!Nt(n)&&o(n.locals,t),n.kind){case 242:o(Er(n).exports,2623475&t);break;case 241:o(Er(n).exports,8&t);break;case 207:var r=n.name;r&&i(n.symbol,t);case 238:case 239:a||o(Li(Er(n)),67897832&t);break;case 194:var s=n.name;s&&i(n.symbol,t)}e.introducesArgumentsExoticObject(n)&&i(K,t),a=e.hasModifier(n,32),n=n.parent}o(Sn,t)}(),r.delete(\"this\"),Ao(r);function i(n,t){if(e.getCombinedLocalAndExportSymbolFlags(n)&t){var a=n.escapedName;r.has(a)||r.set(a,n)}}function o(e,n){n&&e.forEach(function(e){i(e,n)})}}(n,t):[]},getSymbolAtLocation:function(n){return(n=e.getParseTreeNode(n))?Sv(n):void 0},getShorthandAssignmentValueSymbol:function(n){return(n=e.getParseTreeNode(n))?function(e){if(e&&274===e.kind)return ir(e.name,69317567)}(n):void 0},getExportSpecifierLocalTargetSymbol:function(n){var t=e.getParseTreeNode(n,e.isExportSpecifier);return t?function(e){return e.parent.parent.moduleSpecifier?Xt(e.parent.parent,e):ir(e.propertyName||e.name,70107135)}(t):void 0},getExportSymbolOfSymbol:function(e){return br(e.exportSymbol||e)},getTypeAtLocation:function(n){return(n=e.getParseTreeNode(n))?Lv(n):te},getPropertySymbolOfDestructuringAssignment:function(n){var t=e.getParseTreeNode(n,e.isIdentifier);return t?function(n){var t=function n(t){if(e.Debug.assert(186===t.kind||185===t.kind),225===t.parent.kind){var r=M_(t.parent.expression,t.parent.awaitModifier);return cg(t,r||te)}if(202===t.parent.kind){var r=Tg(t.parent.right);return cg(t,r||te)}if(273===t.parent.kind){var a=n(t.parent.parent);return sg(a||te,t.parent)}e.Debug.assert(185===t.parent.kind);var i=n(t.parent),o=O_(i||te,t.parent,!1,!1)||te;return lg(t.parent,i,t.parent.elements.indexOf(t),o||te)}(n.parent.parent);return t&&vo(t,n.escapedText)}(t):void 0},signatureToString:function(n,t,r,a){return Xr(n,e.getParseTreeNode(t),r,a)},typeToString:function(n,t,r){return Yr(n,e.getParseTreeNode(t),r)},symbolToString:function(n,t,r,a){return Jr(n,e.getParseTreeNode(t),r,a)},typePredicateToString:function(n,t,r){return Zr(n,e.getParseTreeNode(t),r)},writeSignature:function(n,t,r,a,i){return Xr(n,e.getParseTreeNode(t),r,a,i)},writeType:function(n,t,r,a){return Yr(n,e.getParseTreeNode(t),r,a)},writeSymbol:function(n,t,r,a,i){return Jr(n,e.getParseTreeNode(t),r,a,i)},writeTypePredicate:function(n,t,r,a){return Zr(n,e.getParseTreeNode(t),r,a)},getAugmentedPropertiesOfType:Cv,getRootSymbols:function n(t){var r=function(n){if(6&e.getCheckFlags(n))return e.mapDefined(It(n).containingType.types,function(e){return vo(e,n.escapedName)});if(33554432&n.flags){var t=n,r=t.leftSpread,a=t.rightSpread,i=t.syntheticOrigin;return r?[r,a]:i?[i]:e.singleElementArray(function(e){for(var n,t=e;t=It(t).target;)n=t;return n}(n))}}(t);return r?e.flatMap(r,n):[t]},getContextualType:function(n){var t=e.getParseTreeNode(n,e.isExpression);return t?Am(t):void 0},getContextualTypeForObjectLiteralElement:function(n){var t=e.getParseTreeNode(n,e.isObjectLiteralElementLike);return t?bm(t):void 0},getContextualTypeForArgumentAtIndex:function(n,t){var r=e.getParseTreeNode(n,e.isCallLikeExpression);return r&&_m(r,t)},getContextualTypeForJsxAttribute:function(n){var t=e.getParseTreeNode(n,e.isJsxAttributeLike);return t&&Sm(t)},isContextSensitive:dc,getFullyQualifiedName:ar,getResolvedSignature:function(e,n,t){return U(e,n,t,!1)},getResolvedSignatureForSignatureHelp:function(e,n,t){return U(e,n,t,!0)},getConstantValue:function(n){var t=e.getParseTreeNode(n,zv);return t?Jv(t):void 0},isValidPropertyAccess:function(n,t){var r=e.getParseTreeNode(n,e.isPropertyAccessOrQualifiedNameOrImportTypeNode);return!!r&&function(e,n){switch(e.kind){case 187:return Mp(e,97===e.expression.kind,n,Mu(Lg(e.expression)));case 146:return Mp(e,!1,n,Mu(Lg(e.left)));case 181:return Mp(e,!1,n,Ul(e))}}(r,e.escapeLeadingUnderscores(t))},isValidPropertyAccessForCompletions:function(n,t,r){var a=e.getParseTreeNode(n,e.isPropertyAccessExpression);return!!a&&function(n,t,r){return Mp(n,181!==n.kind&&97===n.expression.kind,r.escapedName,t)&&(!(8192&r.flags)||(i=ho(Tu(ua(a=t,r.escapedName)),0),e.Debug.assert(0!==i.length),i.some(function(e){var n=Vo(e);return!n||bc(a,function(e,n,t){if(!e.typeParameters)return n;var r=wu(e.typeParameters,e,0);return Wu(r.inferences,t,n),lc(n,Jo(e,$u(r)))}(e,n,a))})));var a,i}(a,t,r)},getSignatureFromDeclaration:function(n){var t=e.getParseTreeNode(n,e.isFunctionLike);return t?wo(t):void 0},isImplementationOfOverload:function(n){var t=e.getParseTreeNode(n,e.isFunctionLike);return t?Kv(t):void 0},getImmediateAliasedSymbol:function(n){e.Debug.assert(0!=(2097152&n.flags),\"Should only get Alias here.\");var t=It(n);if(!t.immediateTarget){var r=Wt(n);if(!r)return e.Debug.fail();t.immediateTarget=Qt(r,!0)}return t.immediateTarget},getAliasedSymbol:er,getEmitResolver:function(e,n){return gv(e,n),P},getExportsOfModule:fr,getExportsAndPropertiesOfModule:function(n){var t=fr(n),r=dr(n);return r!==n&&e.addRange(t,$i(Ga(r))),t},getSymbolWalker:e.createGetSymbolWalker(function(e){return jo(e)||$},Bo,Ho,Za,Xi,Ga,nd,Eo,no,ev),getAmbientModules:function(){return Ne||(Ne=[],Sn.forEach(function(e,n){t.test(n)&&Ne.push(e)})),Ne},getAllAttributesTypeFromJsxOpeningLikeElement:function(n){var t=e.getParseTreeNode(n,e.isJsxOpeningLikeElement);return t?function(e){return qm(e.tagName)?op(e):sp(e,!0)}(t):void 0},getJsxIntrinsicTagNamesAt:function(t){var r=Ym(n.IntrinsicElements,t);return r?$i(r):e.emptyArray},isOptionalParameter:function(n){var t=e.getParseTreeNode(n,e.isParameter);return!!t&&ko(t)},tryGetMemberInModuleExports:function(n,t){return gr(e.escapeLeadingUnderscores(n),t)},tryGetMemberInModuleExportsAndProperties:function(n,t){return function(e,n){var t=gr(e,n);if(t)return t;var r=dr(n);if(r!==n){var a=Ga(r);return 32764&a.flags?void 0:vo(a,e)}}(e.escapeLeadingUnderscores(n),t)},tryFindAmbientModuleWithoutAugmentations:function(e){return Do(e,!1)},getApparentType:po,getUnionType:tl,createAnonymousType:wr,createSignature:Di,createSymbol:Tt,createIndexInfo:ns,getAnyType:function(){return $},getStringType:function(){return le},getNumberType:function(){return ce},createPromiseType:Bf,createArrayType:Ws,getBooleanType:function(){return fe},getFalseType:function(e){return e?ue:de},getTrueType:function(e){return e?me:pe},getVoidType:function(){return _e},getUndefinedType:function(){return ae},getNullType:function(){return oe},getESSymbolType:function(){return ge},getNeverType:function(){return ve},isSymbolAccessible:Hr,isArrayLikeType:tu,isTypeInvalidDueToUnionDiscriminant:function(n,t){return t.properties.some(function(t){var r=t.name&&e.getTextOfPropertyName(t.name),a=void 0===r?void 0:ua(n,r);return!!a&&!!(448&a.flags)&&!_c(Lv(t),a)})},getAllPossiblePropertiesOfTypes:function(n){var t=tl(n);if(!(262144&t.flags))return Cv(t);for(var r=e.createSymbolTable(),a=0,i=n;a<i.length;a++)for(var o=i[a],s=0,l=Cv(o);s<l.length;s++){var c=l[s].escapedName;if(!r.has(c)){var u=fo(t,c);u&&r.set(c,u)}}return e.arrayFrom(r.values())},getSuggestionForNonexistentProperty:function(e,n){return xp(e,n)},getSuggestionForNonexistentSymbol:function(n,t,r){return function(n,t,r){var a=Ap(n,t,r);return a&&e.symbolName(a)}(n,e.escapeLeadingUnderscores(t),r)},getSuggestionForNonexistentExport:function(n,t){return(r=Cp(n,t))&&e.symbolName(r);var r},getBaseConstraintOfType:io,getDefaultFromTypeParameter:function(e){return e&&65536&e.flags?uo(e):void 0},resolveName:function(n,t,r,a){return Ft(t,e.escapeLeadingUnderscores(n),r,void 0,void 0,!1,a)},getJsxNamespace:function(n){return e.unescapeLeadingUnderscores(yt(n))},getAccessibleSymbolChain:Gr,getTypePredicateOfSignature:Bo,resolveExternalModuleSymbol:dr,tryGetThisTypeAt:function(n){return(n=e.getParseTreeNode(n))&&om(n)},getTypeArgumentConstraint:function(n){var t=e.getParseTreeNode(n,e.isTypeNode);return t&&function(n){var t=e.tryCast(n.parent,e.isTypeReferenceType);if(t){var r=Fg(t),a=no(r[t.typeArguments.indexOf(n)]);return a&&lc(a,Jl(r,wg(t,r)))}}(t)},getSuggestionDiagnostics:function(n,t){if(e.skipTypeChecking(n,A))return e.emptyArray;var r;try{return m=t,mv(n),e.Debug.assert(!!(1&Rt(n).flags)),r=e.addRange(r,Yn.get(n.fileName)),n.isDeclarationFile||pv(0)&&pv(1)||r_(fv(n),function(n,t,a){e.containsParseError(n)||pv(t)||(r||(r=[])).push(yn({},a,{category:e.DiagnosticCategory.Suggestion}))}),r||e.emptyArray}finally{m=void 0}},runWithCancellationToken:function(e,n){try{return m=e,n(H)}finally{m=void 0}},getLocalTypeParametersOfClassOrInterfaceOrTypeAlias:Wa};function U(n,t,r,a){var i=e.getParseTreeNode(n,e.isCallLikeExpression);V=r;var o=i?ff(i,t,a):void 0;return V=void 0,o}var j=e.createMap(),W=e.createMap(),q=e.createMap(),z=e.createMap(),J=e.createMap(),X=[],Y=e.createMap(),Q=Tt(4,\"unknown\"),Z=Tt(0,\"__resolving__\"),$=kr(1,\"any\"),ee=kr(1,\"any\"),ne=kr(1,\"any\"),te=kr(1,\"error\"),re=kr(2,\"unknown\"),ae=kr(8192,\"undefined\"),ie=M?ae:kr(134225920,\"undefined\"),oe=kr(16384,\"null\"),se=M?oe:kr(134234112,\"null\"),le=kr(4,\"string\"),ce=kr(8,\"number\"),ue=kr(256,\"false\"),de=kr(256,\"false\"),me=kr(256,\"true\"),pe=kr(256,\"true\");ue.flags|=33554432,me.flags|=33554432,me.regularType=pe,pe.freshType=me,ue.regularType=de,de.freshType=ue;var fe=Mr([de,pe]);Mr([de,me]),Mr([ue,pe]),Mr([ue,me]);var ge=kr(1024,\"symbol\"),_e=kr(4096,\"void\"),ve=kr(32768,\"never\"),ye=kr(32768,\"never\"),he=kr(32768,\"never\"),be=kr(16777216,\"object\"),Ee=tl([le,ce,ge]),Te=w?le:Ee,Se=wr(void 0,L,e.emptyArray,e.emptyArray,void 0,void 0),Le=Tt(2048,\"__type\");Le.members=e.createSymbolTable();var xe=wr(Le,L,e.emptyArray,e.emptyArray,void 0,void 0),Ae=wr(void 0,L,e.emptyArray,e.emptyArray,void 0,void 0);Ae.instantiations=e.createMap();var Ce=wr(void 0,L,e.emptyArray,e.emptyArray,void 0,void 0);Ce.flags|=536870912;var De=wr(void 0,L,e.emptyArray,e.emptyArray,void 0,void 0),ke=wr(void 0,L,e.emptyArray,e.emptyArray,void 0,void 0),Me=wr(void 0,L,e.emptyArray,e.emptyArray,void 0,void 0),Oe=Dr(65536),Ie=Dr(65536);Ie.constraint=Oe;var Re,Ne,we,Pe,Fe,Ge,Ve,Ke,Be,He,Ue,je,We,qe,ze,Je,Xe,Ye,Qe,Ze,$e,en,nn,tn,rn,an,on,sn,ln,cn,un,dn,mn,pn=Dr(65536),fn=Oo(\"<<unresolved>>\",0,$),gn=Di(void 0,void 0,void 0,e.emptyArray,$,void 0,0,!1,!1),_n=Di(void 0,void 0,void 0,e.emptyArray,te,void 0,0,!1,!1),vn=Di(void 0,void 0,void 0,e.emptyArray,$,void 0,0,!1,!1),bn=Di(void 0,void 0,void 0,e.emptyArray,ye,void 0,0,!1,!1),En=[vn],Tn=ns(le,!0),Sn=e.createSymbolTable(),Ln=e.createMap(),xn=e.createMap(),An=0,Cn=0,Dn=0,kn=!1,Mn=Kl(\"\"),On=Kl(0),In=[],Rn=[],Nn=[],wn=0,Pn=10,Fn=[],Gn=[],Vn=[],Kn=[],Bn=[],Hn=[],Un=[],jn=[],Wn=[],qn=[],zn=[],Jn=[],Xn=e.createDiagnosticCollection(),Yn=e.createMultiMap();!function(e){e[e.None=0]=\"None\",e[e.TypeofEQString=1]=\"TypeofEQString\",e[e.TypeofEQNumber=2]=\"TypeofEQNumber\",e[e.TypeofEQBoolean=4]=\"TypeofEQBoolean\",e[e.TypeofEQSymbol=8]=\"TypeofEQSymbol\",e[e.TypeofEQObject=16]=\"TypeofEQObject\",e[e.TypeofEQFunction=32]=\"TypeofEQFunction\",e[e.TypeofEQHostObject=64]=\"TypeofEQHostObject\",e[e.TypeofNEString=128]=\"TypeofNEString\",e[e.TypeofNENumber=256]=\"TypeofNENumber\",e[e.TypeofNEBoolean=512]=\"TypeofNEBoolean\",e[e.TypeofNESymbol=1024]=\"TypeofNESymbol\",e[e.TypeofNEObject=2048]=\"TypeofNEObject\",e[e.TypeofNEFunction=4096]=\"TypeofNEFunction\",e[e.TypeofNEHostObject=8192]=\"TypeofNEHostObject\",e[e.EQUndefined=16384]=\"EQUndefined\",e[e.EQNull=32768]=\"EQNull\",e[e.EQUndefinedOrNull=65536]=\"EQUndefinedOrNull\",e[e.NEUndefined=131072]=\"NEUndefined\",e[e.NENull=262144]=\"NENull\",e[e.NEUndefinedOrNull=524288]=\"NEUndefinedOrNull\",e[e.Truthy=1048576]=\"Truthy\",e[e.Falsy=2097152]=\"Falsy\",e[e.All=4194303]=\"All\",e[e.BaseStringStrictFacts=933633]=\"BaseStringStrictFacts\",e[e.BaseStringFacts=3145473]=\"BaseStringFacts\",e[e.StringStrictFacts=4079361]=\"StringStrictFacts\",e[e.StringFacts=4194049]=\"StringFacts\",e[e.EmptyStringStrictFacts=3030785]=\"EmptyStringStrictFacts\",e[e.EmptyStringFacts=3145473]=\"EmptyStringFacts\",e[e.NonEmptyStringStrictFacts=1982209]=\"NonEmptyStringStrictFacts\",e[e.NonEmptyStringFacts=4194049]=\"NonEmptyStringFacts\",e[e.BaseNumberStrictFacts=933506]=\"BaseNumberStrictFacts\",e[e.BaseNumberFacts=3145346]=\"BaseNumberFacts\",e[e.NumberStrictFacts=4079234]=\"NumberStrictFacts\",e[e.NumberFacts=4193922]=\"NumberFacts\",e[e.ZeroStrictFacts=3030658]=\"ZeroStrictFacts\",e[e.ZeroFacts=3145346]=\"ZeroFacts\",e[e.NonZeroStrictFacts=1982082]=\"NonZeroStrictFacts\",e[e.NonZeroFacts=4193922]=\"NonZeroFacts\",e[e.BaseBooleanStrictFacts=933252]=\"BaseBooleanStrictFacts\",e[e.BaseBooleanFacts=3145092]=\"BaseBooleanFacts\",e[e.BooleanStrictFacts=4078980]=\"BooleanStrictFacts\",e[e.BooleanFacts=4193668]=\"BooleanFacts\",e[e.FalseStrictFacts=3030404]=\"FalseStrictFacts\",e[e.FalseFacts=3145092]=\"FalseFacts\",e[e.TrueStrictFacts=1981828]=\"TrueStrictFacts\",e[e.TrueFacts=4193668]=\"TrueFacts\",e[e.SymbolStrictFacts=1981320]=\"SymbolStrictFacts\",e[e.SymbolFacts=4193160]=\"SymbolFacts\",e[e.ObjectStrictFacts=1972176]=\"ObjectStrictFacts\",e[e.ObjectFacts=4184016]=\"ObjectFacts\",e[e.FunctionStrictFacts=1970144]=\"FunctionStrictFacts\",e[e.FunctionFacts=4181984]=\"FunctionFacts\",e[e.UndefinedFacts=2457472]=\"UndefinedFacts\",e[e.NullFacts=2340752]=\"NullFacts\",e[e.EmptyObjectStrictFacts=4079615]=\"EmptyObjectStrictFacts\",e[e.EmptyObjectFacts=4194303]=\"EmptyObjectFacts\"}(mn||(mn={}));var Qn,Zn,$n,et,nt,tt,rt,at,it,ot=e.createMapFromTemplate({string:1,number:2,boolean:4,symbol:8,undefined:16384,object:16,function:32}),st=e.createMapFromTemplate({string:128,number:256,boolean:512,symbol:1024,undefined:131072,object:2048,function:4096}),lt=e.createMapFromTemplate({string:le,number:ce,boolean:fe,symbol:ge,undefined:ae}),ct=tl(e.arrayFrom(ot.keys(),Kl)),ut=e.createMap(),dt=e.createMap(),mt=e.createMap(),pt=e.createMap(),ft=e.createMap(),gt=e.createMap();!function(e){e[e.Type=0]=\"Type\",e[e.ResolvedBaseConstructorType=1]=\"ResolvedBaseConstructorType\",e[e.DeclaredType=2]=\"DeclaredType\",e[e.ResolvedReturnType=3]=\"ResolvedReturnType\",e[e.ImmediateBaseConstraint=4]=\"ImmediateBaseConstraint\",e[e.EnumTagType=5]=\"EnumTagType\",e[e.JSDocTypeReference=6]=\"JSDocTypeReference\"}($n||($n={})),function(e){e[e.Normal=0]=\"Normal\",e[e.SkipContextSensitive=1]=\"SkipContextSensitive\",e[e.Inferential=2]=\"Inferential\",e[e.Contextual=3]=\"Contextual\"}(et||(et={})),function(e){e[e.None=0]=\"None\",e[e.Bivariant=1]=\"Bivariant\",e[e.Strict=2]=\"Strict\"}(nt||(nt={})),function(e){e[e.IncludeReadonly=1]=\"IncludeReadonly\",e[e.ExcludeReadonly=2]=\"ExcludeReadonly\",e[e.IncludeOptional=4]=\"IncludeOptional\",e[e.ExcludeOptional=8]=\"ExcludeOptional\"}(tt||(tt={})),function(e){e[e.None=0]=\"None\",e[e.Source=1]=\"Source\",e[e.Target=2]=\"Target\",e[e.Both=3]=\"Both\"}(rt||(rt={})),function(e){e.resolvedExports=\"resolvedExports\",e.resolvedMembers=\"resolvedMembers\"}(at||(at={})),function(e){e[e.Local=0]=\"Local\",e[e.Parameter=1]=\"Parameter\"}(it||(it={}));var _t=e.createSymbolTable();_t.set(G.escapedName,G);var vt=e.and(sv,function(n){return!e.isAccessor(n)});return function(){for(var n=0,t=r.getSourceFiles();n<t.length;n++){var a=t[n];e.bindSourceFile(a,A)}var i;Re=e.createMap();for(var o=0,s=r.getSourceFiles();o<s.length;o++){var a=s[o];if(!a.redirectInfo&&(e.isExternalOrCommonJsModule(a)||Mt(Sn,a.locals),a.jsGlobalAugmentations&&Mt(Sn,a.jsGlobalAugmentations),a.patternAmbientModules&&a.patternAmbientModules.length&&(we=e.concatenate(we,a.patternAmbientModules)),a.moduleAugmentations.length&&(i||(i=[])).push(a.moduleAugmentations),a.symbol&&a.symbol.globalExports)){var l=a.symbol.globalExports;l.forEach(function(e,n){Sn.has(n)||Sn.set(n,e)})}}if(i)for(var c=0,u=i;c<u.length;c++)for(var d=u[c],m=0,p=d;m<p.length;m++){var f=p[m];e.isGlobalScopeAugmentation(f.parent)&&Ot(f)}if(function(n,t,r){t.forEach(function(t,a){var i=n.get(a);i?e.forEach(i.declarations,function(n,t){return function(r){return Xn.add(e.createDiagnosticForNode(r,t,n))}}(e.unescapeLeadingUnderscores(a),r)):n.set(a,t)})}(Sn,_t,e.Diagnostics.Declaration_name_conflicts_with_built_in_global_identifier_0),It(G).type=ie,It(K).type=Ds(\"IArguments\",0,!0),It(Q).type=te,Ge=Ds(\"Array\",1,!0),Pe=Ds(\"Object\",0,!0),Fe=Ds(\"Function\",0,!0),Ke=Ds(\"String\",0,!0),Be=Ds(\"Number\",0,!0),He=Ds(\"Boolean\",0,!0),Ue=Ds(\"RegExp\",0,!0),We=Ws($),(qe=Ws(ee))===Se&&(qe=wr(void 0,L,e.emptyArray,e.emptyArray,void 0,void 0)),Ve=Vs(\"ReadonlyArray\",1),ze=Ve?Ks(Ve,[$]):We,je=Vs(\"ThisType\",1),i)for(var g=0,_=i;g<_.length;g++)for(var d=_[g],v=0,y=d;v<y.length;v++){var f=y[v];e.isGlobalScopeAugmentation(f.parent)||Ot(f)}function h(n,t){n.forEach(function(n,r){var a=t.get(r),i=n.blockScoped?e.Diagnostics.Cannot_redeclare_block_scoped_variable_0:e.Diagnostics.Duplicate_identifier_0;n.instances.forEach(function(e){kt(e,i,r,a.instances[0])})})}Re.forEach(function(n){var t=n.firstFile,r=n.secondFile,a=n.firstFileInstances,i=n.secondFileInstances,o=e.arrayFrom(a.keys());if(o.length<8)return h(a,i),void h(i,a);var s=o.join(\", \");Xn.add(ht(e.createDiagnosticForNode(t,e.Diagnostics.Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0,s),e.createDiagnosticForNode(r,e.Diagnostics.Conflicts_are_in_this_file))),Xn.add(ht(e.createDiagnosticForNode(r,e.Diagnostics.Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0,s),e.createDiagnosticForNode(t,e.Diagnostics.Conflicts_are_in_this_file)))}),Re=void 0}(),H;function yt(n){if(n){var t=e.getSourceFileOfNode(n);if(t){if(t.localJsxNamespace)return t.localJsxNamespace;var r=t.pragmas.get(\"jsx\");if(r){var a=e.isArray(r)?r[0]:r;if(t.localJsxFactory=e.parseIsolatedEntityName(a.arguments.factory,C),t.localJsxFactory)return t.localJsxNamespace=ev(t.localJsxFactory).escapedText}}}return Qn||(Qn=\"React\",A.jsxFactory?(Zn=e.parseIsolatedEntityName(A.jsxFactory,C))&&(Qn=ev(Zn).escapedText):A.reactNamespace&&(Qn=e.escapeLeadingUnderscores(A.reactNamespace))),Qn}function ht(e){for(var n,t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];return e.relatedInformation||(e.relatedInformation=[]),(n=e.relatedInformation).push.apply(n,t),e}function bt(n,t,r,a,i,o){var s=n?e.createDiagnosticForNode(n,t,r,a,i,o):e.createCompilerDiagnostic(t,r,a,i,o);return Xn.add(s),s}function Et(n,t,r,a,i,o,s){!function(n,t){n?Xn.add(t):Yn.add(t.file.fileName,yn({},t,{category:e.DiagnosticCategory.Suggestion}))}(n,\"message\"in r?e.createDiagnosticForNode(t,r,a,i,o,s):e.createDiagnosticForNodeFromMessageChain(t,r))}function Tt(e,n,t){b++;var r=new _(33554432|e,n);return r.checkFlags=t||0,r}function St(e){return 0!=(33554432&e.flags)}function Lt(e){var n=0;return 2&e&&(n|=67220415),1&e&&(n|=67220414),4&e&&(n|=0),8&e&&(n|=68008959),16&e&&(n|=67219887),32&e&&(n|=68008383),64&e&&(n|=67897736),256&e&&(n|=68008191),128&e&&(n|=68008831),512&e&&(n|=110735),8192&e&&(n|=67212223),32768&e&&(n|=67154879),65536&e&&(n|=67187647),262144&e&&(n|=67635688),524288&e&&(n|=67897832),2097152&e&&(n|=2097152),n}function xt(e,n){n.mergeId||(n.mergeId=i,i++),Fn[n.mergeId]=e}function At(n){var t=Tt(n.flags,n.escapedName);return t.declarations=n.declarations?n.declarations.slice():[],t.parent=n.parent,n.valueDeclaration&&(t.valueDeclaration=n.valueDeclaration),n.constEnumOnlyModule&&(t.constEnumOnlyModule=!0),n.members&&(t.members=e.cloneMap(n.members)),n.exports&&(t.exports=e.cloneMap(n.exports)),xt(t,n),t}function Ct(n,t){if(!(n.flags&Lt(t.flags))||67108864&(t.flags|n.flags))e.Debug.assert(t!==n),33554432&n.flags||(n=At(n)),512&t.flags&&512&n.flags&&n.constEnumOnlyModule&&!t.constEnumOnlyModule&&(n.constEnumOnlyModule=!1),n.flags|=t.flags,t.valueDeclaration&&(!n.valueDeclaration||e.isAssignmentDeclaration(n.valueDeclaration)&&!e.isAssignmentDeclaration(t.valueDeclaration)||e.isEffectiveModuleDeclaration(n.valueDeclaration)&&!e.isEffectiveModuleDeclaration(t.valueDeclaration))&&(n.valueDeclaration=t.valueDeclaration),e.addRange(n.declarations,t.declarations),t.members&&(n.members||(n.members=e.createSymbolTable()),Mt(n.members,t.members)),t.exports&&(n.exports||(n.exports=e.createSymbolTable()),Mt(n.exports,t.exports)),xt(n,t);else if(1024&n.flags)bt(e.getNameOfDeclaration(t.declarations[0]),e.Diagnostics.Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity,Jr(n));else{var r=!!(384&n.flags||384&t.flags),a=!!(2&n.flags||2&t.flags),i=r?e.Diagnostics.Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations:a?e.Diagnostics.Cannot_redeclare_block_scoped_variable_0:e.Diagnostics.Duplicate_identifier_0,o=t.declarations&&e.getSourceFileOfNode(t.declarations[0]),s=n.declarations&&e.getSourceFileOfNode(n.declarations[0]);if(o&&s&&Re&&!r&&o!==s){var l=-1===e.comparePaths(o.path,s.path)?o:s,c=l===o?s:o,u=l.path+\"|\"+c.path,d=Re.get(u)||{firstFile:l,secondFile:c,firstFileInstances:e.createMap(),secondFileInstances:e.createMap()},m=Jr(t),p=d.firstFileInstances.get(m)||{instances:[],blockScoped:a},f=d.secondFileInstances.get(m)||{instances:[],blockScoped:a};return e.forEach(t.declarations,function(n){var t=(e.getExpandoInitializer(n,!1)?e.getNameOfExpando(n):e.getNameOfDeclaration(n))||n;(o===l?p:f).instances.push(t)}),e.forEach(n.declarations,function(n){var t=(e.getExpandoInitializer(n,!1)?e.getNameOfExpando(n):e.getNameOfDeclaration(n))||n;(s===l?p:f).instances.push(t)}),d.firstFileInstances.set(m,p),d.secondFileInstances.set(m,f),Re.set(u,d),n}var g=Jr(t);Dt(t,i,g,n),Dt(n,i,g,t)}return n}function Dt(n,t,r,a){e.forEach(n.declarations,function(n){kt((e.getExpandoInitializer(n,!1)?e.getNameOfExpando(n):e.getNameOfDeclaration(n))||n,t,r,a.declarations&&a.declarations[0])})}function kt(n,t,r,a){var i=function(n,t,r,a,i,o){var s=n?e.createDiagnosticForNode(n,t,r,a,i,o):e.createCompilerDiagnostic(t,r,a,i,o),l=Xn.lookup(s);return l||(Xn.add(s),s)}(n,t,r);a&&e.length(i.relatedInformation)<5&&ht(i,e.length(i.relatedInformation)?e.createDiagnosticForNode(a,e.Diagnostics.and_here):e.createDiagnosticForNode(a,e.Diagnostics._0_was_also_declared_here,r))}function Mt(e,n){n.forEach(function(n,t){e.set(t,e.has(t)?Ct(e.get(t),n):n)})}function Ot(n){var t=n.parent;if(t.symbol.declarations[0]===t)if(e.isGlobalScopeAugmentation(t))Mt(Sn,t.symbol.exports);else{var r=lr(n,n,4194304&n.parent.parent.flags?void 0:e.Diagnostics.Invalid_module_name_in_augmentation_module_0_cannot_be_found,!0);if(!r)return;1920&(r=dr(r)).flags?r=Ct(r,t.symbol):bt(n,e.Diagnostics.Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity,n.text)}else e.Debug.assert(t.symbol.declarations.length>1)}function It(e){if(33554432&e.flags)return e;var n=l(e);return Gn[n]||(Gn[n]={})}function Rt(e){var n=s(e);return Vn[n]||(Vn[n]={flags:0})}function Nt(n){return 277===n.kind&&!e.isExternalOrCommonJsModule(n)}function wt(n,t,r){if(r){var a=n.get(t);if(a){if(e.Debug.assert(0==(1&e.getCheckFlags(a)),\"Should never get an instantiated symbol here.\"),a.flags&r)return a;if(2097152&a.flags){var i=er(a);if(i===Q||i.flags&r)return a}}}}function Pt(n,t){var a=e.getSourceFileOfNode(n),i=e.getSourceFileOfNode(t);if(a!==i){if(D&&(a.externalModuleIndicator||i.externalModuleIndicator)||!A.outFile&&!A.out||td(t)||4194304&n.flags)return!0;if(c(t,n))return!0;var o=r.getSourceFiles();return o.indexOf(a)<=o.indexOf(i)}if(n.pos<=t.pos){if(184===n.kind){var s=e.getAncestor(t,184);return s?e.findAncestor(s,e.isBindingElement)!==e.findAncestor(n,e.isBindingElement)||n.pos<s.pos:Pt(e.getAncestor(n,235),t)}return 235===n.kind?!function(n,t){var r=e.getEnclosingBlockScopeContainer(n);switch(n.parent.parent.kind){case 217:case 223:case 225:if(Ut(t,n,r))return!0}var a=n.parent.parent;return e.isForInOrOfStatement(a)&&Ut(t,a.expression,r)}(n,t):!e.isClassDeclaration(n)||!e.findAncestor(t,function(t){return e.isComputedPropertyName(t)&&t.parent.parent===n})}if(255===t.parent.kind||252===t.parent.kind&&t.parent.isExportEquals)return!0;if(252===t.kind&&t.isExportEquals)return!0;var l=e.getEnclosingBlockScopeContainer(n);return!!(2097152&t.flags)||td(t)||c(t,n,l);function c(n,t,r){return!!e.findAncestor(n,function(a){if(a===r)return\"quit\";if(e.isFunctionLike(a))return!0;if(a.parent&&152===a.parent.kind&&a.parent.initializer===a)if(e.hasModifier(a.parent,32)){if(154===t.kind)return!0}else if(152!==t.kind||e.hasModifier(t,32)||e.getContainingClass(n)!==e.getContainingClass(t))return!0;return!1})}}function Ft(e,n,t,r,a,i,o,s){return void 0===o&&(o=!1),Gt(e,n,t,r,a,i,o,wt,s)}function Gt(n,t,r,a,i,o,s,l,c){var u,d,m,p,f,g=n,_=n,v=!1;e:for(;n;){if(n.locals&&!Nt(n)&&(u=l(n.locals,t,r))){var y=!0;if(e.isFunctionLike(n)&&d&&d!==n.body?(r&u.flags&67897832&&289!==d.kind&&(y=!!(262144&u.flags)&&(d===n.type||149===d.kind||148===d.kind)),67220415&r&&1&u.flags&&(y=149===d.kind||d===n.type&&!!e.findAncestor(u.valueDeclaration,e.isParameter))):173===n.kind&&(y=d===n.trueType),y)break e;u=void 0}switch(n.kind){case 277:if(!e.isExternalOrCommonJsModule(n))break;v=!0;case 242:var h=Er(n).exports;if(277===n.kind||e.isAmbientModule(n)){if(u=h.get(\"default\")){var b=e.getLocalSymbolForExportDefault(u);if(b&&u.flags&r&&b.escapedName===t)break e;u=void 0}var E=h.get(t);if(E&&2097152===E.flags&&e.getDeclarationOfKind(E,255))break}if(\"default\"!==t&&(u=l(h,t,2623475&r))){if(!e.isSourceFile(n)||!n.commonJsModuleIndicator||u.declarations.some(e.isJSDocTypeAlias))break e;u=void 0}break;case 241:if(u=l(Er(n).exports,t,8&r))break e;break;case 152:case 151:if(e.isClassLike(n.parent)&&!e.hasModifier(n,32)){var T=Cr(n.parent);T&&T.locals&&l(T.locals,t,67220415&r)&&(p=n)}break;case 238:case 207:case 239:if(u=l(Er(n).members||L,t,67897832&r)){if(!Bt(u,n)){u=void 0;break}if(d&&e.hasModifier(d,32))return void bt(_,e.Diagnostics.Static_members_cannot_reference_class_type_parameters);break e}if(207===n.kind&&32&r){var S=n.name;if(S&&t===S.escapedText){u=n.symbol;break e}}break;case 209:if(d===n.expression&&85===n.parent.token){var x=n.parent.parent;if(e.isClassLike(x)&&(u=l(Er(x).members,t,67897832&r)))return void(a&&bt(_,e.Diagnostics.Base_class_expressions_cannot_reference_class_type_parameters))}break;case 147:if(f=n.parent.parent,(e.isClassLike(f)||239===f.kind)&&(u=l(Er(f).members,t,67897832&r)))return void bt(_,e.Diagnostics.A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type);break;case 154:case 153:case 155:case 156:case 157:case 237:case 195:if(3&r&&\"arguments\"===t){u=K;break e}break;case 194:if(3&r&&\"arguments\"===t){u=K;break e}if(16&r){var C=n.name;if(C&&t===C.escapedText){u=n.symbol;break e}}break;case 150:n.parent&&149===n.parent.kind&&(n=n.parent),n.parent&&e.isClassElement(n.parent)&&(n=n.parent);break;case 302:case 295:n=e.getJSDocHost(n)}Vt(n)&&(m=n),d=n,n=n.parent}if(!o||!u||m&&u===m.symbol||(u.isReferenced|=r),!u){if(d&&(e.Debug.assert(277===d.kind),d.commonJsModuleIndicator&&\"exports\"===t&&r&d.symbol.flags))return d.symbol;s||(u=l(Sn,t,r))}if(!u&&g&&e.isInJSFile(g)&&g.parent&&e.isRequireCall(g.parent,!1))return B;if(u){if(a){if(p){var D=p.name;return void bt(_,e.Diagnostics.Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor,e.declarationNameToString(D),Kt(i))}if(_&&(2&r||(32&r||384&r)&&67220415==(67220415&r))){var k=xr(u);(2&k.flags||32&k.flags||384&k.flags)&&function(n,t){e.Debug.assert(!!(2&n.flags||32&n.flags||384&n.flags));var r=e.find(n.declarations,function(n){return e.isBlockOrCatchScoped(n)||e.isClassLike(n)||241===n.kind||e.isInJSFile(n)&&!!e.getJSDocEnumTag(n)});if(void 0===r)return e.Debug.fail(\"Declaration to checkResolvedBlockScopedVariable is undefined\");if(!(4194304&r.flags||Pt(r,t))){var a=void 0,i=e.declarationNameToString(e.getNameOfDeclaration(r));2&n.flags?a=bt(t,e.Diagnostics.Block_scoped_variable_0_used_before_its_declaration,i):32&n.flags?a=bt(t,e.Diagnostics.Class_0_used_before_its_declaration,i):256&n.flags?a=bt(t,e.Diagnostics.Enum_0_used_before_its_declaration,i):(e.Debug.assert(!!(128&n.flags)),A.preserveConstEnums&&(a=bt(t,e.Diagnostics.Class_0_used_before_its_declaration,i))),a&&ht(a,e.createDiagnosticForNode(r,e.Diagnostics._0_is_declared_here,i))}}(k,_)}if(u&&v&&67220415==(67220415&r)&&!(2097152&g.flags)){var M=u.declarations;M&&1===M.length&&245===M[0].kind&&bt(_,e.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead,e.unescapeLeadingUnderscores(t))}}return u}if(a&&(!_||!(function(n,t,r){if(!e.isIdentifier(n)||n.escapedText!==t||yv(n)||td(n))return!1;for(var a=e.getThisContainer(n,!1),i=a;i;){if(e.isClassLike(i.parent)){var o=Er(i.parent);if(!o)break;var s=Ga(o);if(vo(s,t))return bt(n,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0,Kt(r),Jr(o)),!0;if(i===a&&!e.hasModifier(i,32)){var l=si(o).thisType;if(vo(l,t))return bt(n,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0,Kt(r)),!0}}i=i.parent}return!1}(_,t,i)||Ht(_)||function(n,t,r){var a=1920|(e.isInJSFile(n)?67220415:0);if(r===a){var i=$t(Ft(n,t,67897832&~a,void 0,void 0,!1)),o=n.parent;if(i){if(e.isQualifiedName(o)){e.Debug.assert(o.left===n,\"Should only be resolving left side of qualified name as a namespace\");var s=o.right.escapedText,l=vo(si(i),s);if(l)return bt(o,e.Diagnostics.Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1,e.unescapeLeadingUnderscores(t),e.unescapeLeadingUnderscores(s)),!0}return bt(n,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here,e.unescapeLeadingUnderscores(t)),!0}}return!1}(_,t,r)||function(n,t,r){if(67220415&r){if(\"any\"===t||\"string\"===t||\"number\"===t||\"boolean\"===t||\"never\"===t)return bt(n,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here,e.unescapeLeadingUnderscores(t)),!0;var a=$t(Ft(n,t,788544,void 0,void 0,!1));if(a&&!(1024&a.flags)){var i=\"Promise\"===t||\"Symbol\"===t?e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later:e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here;return bt(n,i,e.unescapeLeadingUnderscores(t)),!0}}return!1}(_,t,r)||function(n,t,r){if(111127&r){var a=$t(Ft(n,t,1024,void 0,void 0,!1));if(a)return bt(n,e.Diagnostics.Cannot_use_namespace_0_as_a_value,e.unescapeLeadingUnderscores(t)),!0}else if(788544&r){var a=$t(Ft(n,t,1536,void 0,void 0,!1));if(a)return bt(n,e.Diagnostics.Cannot_use_namespace_0_as_a_type,e.unescapeLeadingUnderscores(t)),!0}return!1}(_,t,r)))){var O=void 0;if(c&&wn<Pn&&(O=Ap(g,t,r))){var I=Jr(O),R=bt(_,c,Kt(i),I);O.valueDeclaration&&ht(R,e.createDiagnosticForNode(O.valueDeclaration,e.Diagnostics._0_is_declared_here,I))}O||bt(_,a,Kt(i)),wn++}}function Vt(e){switch(e.kind){case 237:case 238:case 239:case 241:case 240:case 242:return!0;default:return!1}}function Kt(n){return e.isString(n)?e.unescapeLeadingUnderscores(n):e.declarationNameToString(n)}function Bt(n,t){for(var r=0,a=n.declarations;r<a.length;r++){var i=a[r];if(148===i.kind&&(e.isJSDocTemplateTag(i.parent)?e.getJSDocHost(i.parent):i.parent)===t)return!(e.isJSDocTemplateTag(i.parent)&&e.find(i.parent.parent.tags,e.isJSDocTypeAlias))}return!1}function Ht(n){var t=function n(t){switch(t.kind){case 71:case 187:return t.parent?n(t.parent):void 0;case 209:if(e.isEntityNameExpression(t.expression))return t.expression;default:return}}(n);return!(!t||!ir(t,64,!0)||(bt(n,e.Diagnostics.Cannot_extend_an_interface_0_Did_you_mean_implements,e.getTextOfNode(t)),0))}function Ut(n,t,r){return!!t&&!!e.findAncestor(n,function(n){return n===r||e.isFunctionLike(n)?\"quit\":n===t})}function jt(e){switch(e.kind){case 246:return e;case 248:return e.parent;case 249:return e.parent.parent;case 251:return e.parent.parent.parent;default:return}}function Wt(n){return e.find(n.declarations,e.isAliasSymbolDeclaration)}function qt(e,n,t){var r=e.exports.get(\"export=\");return r?vo(Ga(r),n):$t(e.exports.get(n),t)}function zt(n){return e.isExportAssignment(n)&&!n.isExportEquals||e.hasModifier(n,512)||e.isExportSpecifier(n)}function Jt(n,t,r){if(!k)return!1;if(!n||n.isDeclarationFile){var a=qt(t,\"default\",!0);return!(a&&e.some(a.declarations,zt)||qt(t,e.escapeLeadingUnderscores(\"__esModule\"),r))}return e.isSourceFileJS(n)?!n.externalModuleIndicator&&!qt(t,e.escapeLeadingUnderscores(\"__esModule\"),r):pr(t)}function Xt(n,t,r){void 0===r&&(r=!1);var a=sr(n,n.moduleSpecifier),i=mr(a,n.moduleSpecifier,r);if(i){var o=t.propertyName||t.name;if(o.escapedText){if(e.isShorthandAmbientModuleSymbol(a))return a;var s=void 0;s=a&&a.exports&&a.exports.get(\"export=\")?vo(Ga(i),o.escapedText):function(e,n){if(3&e.flags){var t=e.valueDeclaration.type;if(t)return $t(vo(Ul(t),n))}}(i,o.escapedText),s=$t(s,r);var l=function(e,n,t){if(1536&e.flags)return $t(_r(e).get(n),t)}(i,o.escapedText,r);!l&&k&&\"default\"===o.escapedText&&(l=dr(a,r)||$t(a,r));var c=l&&s&&l!==s?function(n,t){if(n===Q&&t===Q)return Q;if(67899368&n.flags)return n;var r=Tt(n.flags|t.flags,n.escapedName);return r.declarations=e.deduplicate(e.concatenate(n.declarations,t.declarations),e.equateValues),r.parent=n.parent||t.parent,n.valueDeclaration&&(r.valueDeclaration=n.valueDeclaration),t.members&&(r.members=t.members),n.exports&&(r.exports=n.exports),r}(s,l):l||s;if(!c){var u=ar(a),d=e.declarationNameToString(o),m=Cp(o,i);if(void 0!==m){var p=Jr(m),f=bt(o,e.Diagnostics.Module_0_has_no_exported_member_1_Did_you_mean_2,u,d,p);m.valueDeclaration&&ht(f,e.createDiagnosticForNode(m.valueDeclaration,e.Diagnostics._0_is_declared_here,p))}else bt(o,e.Diagnostics.Module_0_has_no_exported_member_1,u,d)}return c}}}function Yt(e,n,t){return e.parent.parent.moduleSpecifier?Xt(e.parent.parent,e,t):ir(e.propertyName||e.name,n,!1,t)}function Qt(n,t){switch(void 0===t&&(t=!1),n.kind){case 246:return function(n,t){return 257===n.moduleReference.kind?dr(sr(n,e.getExternalModuleImportEqualsDeclarationExpression(n))):rr(n.moduleReference,t)}(n,t);case 248:return function(n,t){var r=sr(n,n.parent.moduleSpecifier);if(r){var a=void 0;a=e.isShorthandAmbientModuleSymbol(r)?r:qt(r,\"default\",t);var i=Jt(e.find(r.declarations,e.isSourceFile),r,t);if(a||i){if(i)return dr(r,t)||$t(r,t)}else bt(n.name,e.Diagnostics.Module_0_has_no_default_export,Jr(r));return a}}(n,t);case 249:return function(e,n){var t=e.parent.parent.moduleSpecifier;return mr(sr(e,t),t,n)}(n,t);case 251:return function(e,n){return Xt(e.parent.parent.parent,e,n)}(n,t);case 255:return Yt(n,68009983,t);case 252:case 202:return function(n,t){var r=e.isExportAssignment(n)?n.expression:n.right;if(e.isClassExpression(r))return Lg(r).symbol;var a=ir(r,68009983,!0,t);return a||(Lg(r),Rt(r).resolvedSymbol)}(n,t);case 245:return function(e,n){return dr(e.parent.symbol,n)}(n,t);default:return e.Debug.fail()}}function Zt(e,n){return void 0===n&&(n=68009983),!(!e||2097152!=(e.flags&(2097152|n))&&!(2097152&e.flags&&67108864&e.flags))}function $t(e,n){return!n&&Zt(e)?er(e):e}function er(n){e.Debug.assert(0!=(2097152&n.flags),\"Should only get Alias here.\");var t=It(n);if(t.target)t.target===Z&&(t.target=Q);else{t.target=Z;var r=Wt(n);if(!r)return e.Debug.fail();var a=Qt(r);t.target===Z?t.target=a||Q:bt(r,e.Diagnostics.Circular_definition_of_import_alias_0,Jr(n))}return t.target}function nr(e){var n=Er(e),t=er(n);t&&(t===Q||67220415&t.flags&&!Vv(t))&&tr(n)}function tr(n){var t=It(n);if(!t.referenced){t.referenced=!0;var r=Wt(n);if(!r)return e.Debug.fail();252===r.kind?gg(r.expression):255===r.kind?gg(r.propertyName||r.name):e.isInternalModuleImportEqualsDeclaration(r)&&gg(r.moduleReference)}}function rr(n,t){return 71===n.kind&&e.isRightSideOfQualifiedNameOrPropertyAccess(n)&&(n=n.parent),71===n.kind||146===n.parent.kind?ir(n,1920,!1,t):(e.Debug.assert(246===n.parent.kind),ir(n,68009983,!1,t))}function ar(e){return e.parent?ar(e.parent)+\".\"+Jr(e):Jr(e)}function ir(n,t,r,a,i){if(!e.nodeIsMissing(n)){var o,s=1920|(e.isInJSFile(n)?67220415&t:0);if(71===n.kind){var l=t===s?e.Diagnostics.Cannot_find_namespace_0:ed(ev(n).escapedText),c=e.isInJSFile(n)?function(n,t){if(bs(n.parent)){var r=function(n){if(!e.findAncestor(n,function(n){return e.isJSDocNode(n)||2097152&n.flags?e.isJSDocTypeAlias(n):\"quit\"})){var t=e.getJSDocHost(n);if(e.isExpressionStatement(t)&&e.isBinaryExpression(t.expression)&&3===e.getAssignmentDeclarationKind(t.expression)){var r=Er(t.expression.left);if(r)return or(r)}if((e.isObjectLiteralMethod(t)||e.isPropertyAssignment(t))&&e.isBinaryExpression(t.parent.parent)&&6===e.getAssignmentDeclarationKind(t.parent.parent)){var r=Er(t.parent.parent.left);if(r)return or(r)}var a=e.getHostSignatureFromJSDocHost(t);if(a){var r=Er(a);return r&&r.valueDeclaration}}}(n.parent);if(r)return Ft(r,n.escapedText,t,void 0,n,!0)}}(n,t):void 0;if(!(o=Ft(i||n,n.escapedText,t,r||c?void 0:l,n,!0)))return c}else{if(146!==n.kind&&187!==n.kind)throw e.Debug.assertNever(n,\"Unknown entity name kind.\");var u=146===n.kind?n.left:n.expression,d=146===n.kind?n.right:n.name,m=ir(u,s,r,!1,i);if(!m||e.nodeIsMissing(d))return;if(m===Q)return m;if(e.isInJSFile(n)&&m.valueDeclaration&&e.isVariableDeclaration(m.valueDeclaration)&&m.valueDeclaration.initializer&&xf(m.valueDeclaration.initializer)){var p=m.valueDeclaration.initializer.arguments[0],f=sr(p,p);if(f){var g=dr(f);g&&(m=g)}}if(!(o=wt(_r(m),d.escapedText,t)))return void(r||bt(d,e.Diagnostics.Namespace_0_has_no_exported_member_1,ar(m),e.declarationNameToString(d)))}return e.Debug.assert(0==(1&e.getCheckFlags(o)),\"Should never get an instantiated symbol here.\"),o.flags&t||a?o:er(o)}}function or(n){var t=n.parent.valueDeclaration;return(e.isAssignmentDeclaration(t)?e.getAssignedExpandoInitializer(t):e.hasOnlyExpressionInitializer(t)?e.getDeclaredExpandoInitializer(t):void 0)||t}function sr(n,t){return lr(n,t,e.Diagnostics.Cannot_find_module_0)}function lr(n,t,r,a){return void 0===a&&(a=!1),e.isStringLiteralLike(t)?cr(n,t.text,r,t,a):void 0}function cr(n,t,a,i,o){if(void 0===o&&(o=!1),void 0!==t){e.startsWith(t,\"@types/\")&&bt(i,e.Diagnostics.Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1,e.removePrefix(t,\"@types/\"),t);var s=Do(t,!0);if(s)return s;var l=e.getSourceFileOfNode(n),c=e.getResolvedModule(l,t),u=c&&e.getResolutionDiagnostic(A,c),d=c&&!u&&r.getSourceFile(c.resolvedFileName);if(d)return d.symbol?(c.isExternalLibraryImport&&!e.extensionIsTS(c.extension)&&ur(!1,i,c,t),br(d.symbol)):void(a&&bt(i,e.Diagnostics.File_0_is_not_a_module,d.fileName));if(we){var m=e.findBestPatternMatch(we,function(e){return e.pattern},t);if(m)return br(m.symbol)}if(c&&!e.resolutionExtensionIsTSOrJson(c.extension)&&void 0===u||u===e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type)o?bt(i,e.Diagnostics.Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augmented,t,c.resolvedFileName):ur(R&&!!a,i,c,t);else if(a){if(e.pathIsRelative(t)){var p=e.getSourceFileOfNode(n),f=p.redirectedReferences;if(f)for(var g=e.getNormalizedAbsolutePath(t,e.getDirectoryPath(p.fileName)),_=0,v=[\".ts\",\".tsx\"];_<v.length;_++){var y=g+v[_];if(f.indexOf(y)>=0)return void bt(i,e.Diagnostics.Output_file_0_has_not_been_built_from_source_file_1,t,y)}}if(u)bt(i,u,t,c.resolvedFileName);else{var h=e.tryExtractTSExtension(t);h?bt(i,e.Diagnostics.An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead,h,e.removeExtension(t,h)):!A.resolveJsonModule&&e.fileExtensionIs(t,\".json\")&&e.getEmitModuleResolutionKind(A)===e.ModuleResolutionKind.NodeJs&&e.hasJsonModuleEmitEnabled(A)?bt(i,e.Diagnostics.Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension,t):bt(i,a,t)}}}}function ur(n,t,r,a){var i,o=r.packageId,s=r.resolvedFileName,l=!e.isExternalModuleNameRelative(a)&&o?(i=o.name,g().has(e.getTypesPackageName(i))?e.chainDiagnosticMessages(void 0,e.Diagnostics.If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_Slash_Slashgithub_com_SlashDefinitelyTyped_SlashDefinitelyTyped_Slashtree_Slashmaster_Slashtypes_Slash_1,o.name,e.mangleScopedPackageName(o.name)):e.chainDiagnosticMessages(void 0,e.Diagnostics.Try_npm_install_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_module_0,a,e.mangleScopedPackageName(o.name))):void 0;Et(n,t,e.chainDiagnosticMessages(l,e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type,a,s))}function dr(n,t){return n&&br(function(n,t){if(!n||n===Q||1===t.exports.size)return n;var r=At(n);return void 0===r.exports&&(r.flags=512|r.flags,r.exports=e.createSymbolTable()),t.exports.forEach(function(e,n){\"export=\"!==n&&r.exports.set(n,r.exports.has(n)?Ct(r.exports.get(n),e):e)}),r}($t(n.exports.get(\"export=\"),t),n))||n}function mr(n,t,r){var a=dr(n,r);if(!r&&a){if(!(1539&a.flags||e.getDeclarationOfKind(a,277)))return bt(t,e.Diagnostics.Module_0_resolves_to_a_non_module_entity_and_cannot_be_imported_using_this_construct,Jr(n)),a;if(A.esModuleInterop){var i=t.parent;if(e.isImportDeclaration(i)&&e.getNamespaceDeclarationNode(i)||e.isImportCall(i)){var o=Ga(a),s=yo(o,0);if(s&&s.length||(s=yo(o,1)),s&&s.length){var l=Lf(o,a,n),c=Tt(a.flags,a.escapedName);c.declarations=a.declarations?a.declarations.slice():[],c.parent=a.parent,c.target=a,c.originatingImport=i,a.valueDeclaration&&(c.valueDeclaration=a.valueDeclaration),a.constEnumOnlyModule&&(c.constEnumOnlyModule=!0),a.members&&(c.members=e.cloneMap(a.members)),a.exports&&(c.exports=e.cloneMap(a.exports));var u=Xi(l);return c.type=wr(c,u.members,e.emptyArray,e.emptyArray,u.stringIndexInfo,u.numberIndexInfo),c}}}}return a}function pr(e){return void 0!==e.exports.get(\"export=\")}function fr(e){return Ao(vr(e))}function gr(e,n){var t=vr(n);if(t)return t.get(e)}function _r(e){return 32&e.flags?Si(e,\"resolvedExports\"):1536&e.flags?vr(e):e.exports||L}function vr(e){var n=It(e);return n.resolvedExports||(n.resolvedExports=hr(e))}function yr(n,t,r,a){t&&t.forEach(function(t,i){if(\"default\"!==i){var o=n.get(i);if(o){if(r&&a&&o&&$t(o)!==$t(t)){var s=r.get(i);s.exportsWithDuplicate?s.exportsWithDuplicate.push(a):s.exportsWithDuplicate=[a]}}else n.set(i,t),r&&a&&r.set(i,{specifierText:e.getTextOfNode(a.moduleSpecifier)})}})}function hr(n){var t=[];return function n(r){if(r&&r.exports&&e.pushIfUnique(t,r)){var a=e.cloneMap(r.exports),i=r.exports.get(\"__export\");if(i){for(var o=e.createSymbolTable(),s=e.createMap(),l=0,c=i.declarations;l<c.length;l++){var u=c[l],d=sr(u,u.moduleSpecifier),m=n(d);yr(o,m,s,u)}s.forEach(function(n,t){var r=n.exportsWithDuplicate;if(\"export=\"!==t&&r&&r.length&&!a.has(t))for(var i=0,o=r;i<o.length;i++){var l=o[i];Xn.add(e.createDiagnosticForNode(l,e.Diagnostics.Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambiguity,s.get(t).specifierText,e.unescapeLeadingUnderscores(t)))}}),yr(a,o)}return a}}(n=dr(n))||L}function br(e){var n;return e&&e.mergeId&&(n=Fn[e.mergeId])?n:e}function Er(e){return br(e.symbol&&xi(e.symbol))}function Tr(e){return br(e.parent&&xi(e.parent))}function Sr(n,t){var r=Tr(n);if(r){var a=e.mapDefined(r.declarations,function(e){var n=Ur(e);return $t(n&&n.exports&&n.exports.get(\"export=\"))===$t(r)?n:void 0});return t&&Gr(r,t,1920,!1)?e.concatenate([r],a):e.append(a,r)}var i=e.mapDefined(n.declarations,function(n){return!e.isAmbientModule(n)&&n.parent&&Wr(n.parent)?Er(n.parent):void 0});if(e.length(i))return e.mapDefined(i,function(e){return Lr(e,n)?e:void 0})}function Lr(n,t){if(n===Tr(t))return t;var r=_r(n),a=r.get(t.escapedName);return a&&i(a)?a:e.forEachEntry(r,function(e){if(i(e))return e});function i(e){if(e===t||$t(e)===t||$t(e)===$t(t))return e}}function xr(e){return br(e&&0!=(1048576&e.flags)?e.exportSymbol:e)}function Ar(e){return!!(67220415&e.flags||2097152&e.flags&&67220415&er(e).flags)}function Cr(n){for(var t=0,r=n.members;t<r.length;t++){var a=r[t];if(155===a.kind&&e.nodeIsPresent(a.body))return a}}function Dr(e){var n=new v(H,e);return h++,n.id=h,n}function kr(e,n){var t=Dr(e);return t.intrinsicName=n,t}function Mr(e){var n=tl(e);return n.flags|=16,n.intrinsicName=\"boolean\",n}function Or(e,n){var t=Dr(131072);return t.objectFlags=e,t.symbol=n,t.members=void 0,t.properties=void 0,t.callSignatures=void 0,t.constructSignatures=void 0,t.stringIndexInfo=void 0,t.numberIndexInfo=void 0,t}function Ir(e){return 95===e.charCodeAt(0)&&95===e.charCodeAt(1)&&95!==e.charCodeAt(2)&&64!==e.charCodeAt(2)}function Rr(n){var t;return n.forEach(function(e,n){!Ir(n)&&Ar(e)&&(t||(t=[])).push(e)}),t||e.emptyArray}function Nr(n,t,r,a,i,o){return n.members=t,n.properties=t===L?e.emptyArray:Rr(t),n.callSignatures=r,n.constructSignatures=a,n.stringIndexInfo=i,n.numberIndexInfo=o,n}function wr(e,n,t,r,a,i){return Nr(Or(16,e),n,t,r,a,i)}function Pr(n,t){for(var r,a=n;a;a=a.parent){if(a.locals&&!Nt(a)&&(r=t(a.locals)))return r;switch(a.kind){case 277:if(!e.isExternalOrCommonJsModule(a))break;case 242:if(r=t(Er(a).exports))return r}}return t(Sn)}function Fr(e){return 67220415===e?67220415:1920}function Gr(n,t,r,a,i){if(void 0===i&&(i=e.createMap()),n&&!function(e){if(e.declarations&&e.declarations.length){for(var n=0,t=e.declarations;n<t.length;n++){var r=t[n];switch(r.kind){case 152:case 154:case 156:case 157:continue;default:return!1}}return!0}return!1}(n)){var o=\"\"+l(n),s=i.get(o);return s||i.set(o,s=[]),Pr(t,c)}function c(i,o){if(e.pushIfUnique(s,i)){var l=function(i,o){return d(i.get(n.escapedName),void 0,o)?[n]:e.forEachEntry(i,function(i){if(2097152&i.flags&&\"export=\"!==i.escapedName&&\"default\"!==i.escapedName&&!(e.isUMDExportSymbol(i)&&t&&e.isExternalModule(e.getSourceFileOfNode(t)))&&(!a||e.some(i.declarations,e.isExternalModuleImportEqualsDeclaration))&&(o||!e.getDeclarationOfKind(i,255))){var s=er(i);if(d(i,s,o))return[i];var l=_r(s),m=l&&c(l,!0);if(m&&u(i,Fr(r)))return[i].concat(m)}if(i.escapedName===n.escapedName&&i.exportSymbol&&d(br(i.exportSymbol),void 0,o))return[n]})}(i,o);return s.pop(),l}}function u(e,n){return!Vr(e,t,n)||!!Gr(e.parent,t,Fr(n),a,i)}function d(t,a,i){return n===(a||t)&&!e.some(t.declarations,Wr)&&(i||u(t,r))}}function Vr(n,t,r){var a=!1;return Pr(t,function(t){var i=br(t.get(n.escapedName));return!!i&&(i===n||!!((i=2097152&i.flags&&!e.getDeclarationOfKind(i,255)?er(i):i).flags&r)&&(a=!0,!0))}),a}function Kr(e,n){return 0===Hr(e,n,67897832,!1).accessibility}function Br(e,n){return 0===Hr(e,n,67220415,!1).accessibility}function Hr(n,t,r,a){if(n&&t){var i=function n(t,r,a,i,o){if(e.length(t)){for(var s,l=0,c=t;l<c.length;l++){var u=c[l],d=Gr(u,r,i,!1);if(d){s=u;var m=qr(d[0],o);if(m)return m}else if(e.some(u.declarations,Wr))return{accessibility:0};var p=Sr(u,r),f=e.first(u.declarations);!e.length(p)&&67220415&i&&f&&e.isObjectLiteralExpression(f)&&f.parent&&e.isVariableDeclaration(f.parent)&&f===f.parent.initializer&&(p=[Er(f.parent)]);var g=n(p,r,a,a===u?Fr(i):i,o);if(g)return g}return s?{accessibility:1,errorSymbolName:Jr(a,r,i),errorModuleName:s!==a?Jr(s,r,1920):void 0}:void 0}}([n],t,n,r,a);if(i)return i;var o=e.forEach(n.declarations,Ur);return o&&o!==Ur(t)?{accessibility:2,errorSymbolName:Jr(n,t,r),errorModuleName:Jr(o)}:{accessibility:1,errorSymbolName:Jr(n,t,r)}}return{accessibility:0}}function Ur(n){var t=e.findAncestor(n,jr);return t&&Er(t)}function jr(n){return e.isAmbientModule(n)||277===n.kind&&e.isExternalOrCommonJsModule(n)}function Wr(n){return e.isModuleWithStringLiteralName(n)||277===n.kind&&e.isExternalOrCommonJsModule(n)}function qr(n,t){var r;if(e.every(n.declarations,function(n){if(!ra(n)){var t=jt(n);return t&&!e.hasModifier(t,1)&&ra(t.parent)?a(n,t):e.isVariableDeclaration(n)&&e.isVariableStatement(n.parent.parent)&&!e.hasModifier(n.parent.parent,1)&&ra(n.parent.parent.parent)?a(n,n.parent.parent):!(!e.isLateVisibilityPaintedStatement(n)||e.hasModifier(n,1)||!ra(n.parent))&&a(n,n)}return!0}))return{accessibility:0,aliasesToMakeVisible:r};function a(n,a){return t&&(Rt(n).isVisible=!0,r=e.appendIfUnique(r,a)),!0}}function zr(n,t){var r;r=165===n.parent.kind||e.isExpressionWithTypeArgumentsInClassExtendsClause(n.parent)||147===n.parent.kind?68268991:146===n.kind||187===n.kind||246===n.parent.kind?1920:67897832;var a=ev(n),i=Ft(t,a.escapedText,r,void 0,void 0,!1);return i&&qr(i,!0)||{accessibility:1,errorSymbolName:e.getTextOfNode(a),errorNode:a}}function Jr(n,t,r,a,i){void 0===a&&(a=4);var o=3112960;2&a&&(o|=128),1&a&&(o|=512),8&a&&(o|=16384);var s=4&a?F.symbolToExpression:F.symbolToEntityName;return i?l(i).getText():e.usingSingleLineStringWriter(l);function l(a){var i=s(n,r,t,o),l=e.createPrinter({removeComments:!0}),c=t&&e.getSourceFileOfNode(t);return l.writeNode(4,i,c,a),a}}function Xr(n,t,r,a,i){return void 0===r&&(r=0),i?o(i).getText():e.usingSingleLineStringWriter(o);function o(i){var o;o=262144&r?1===a?164:163:1===a?159:158;var s=F.signatureToSignatureDeclaration(n,o,t,3113472|Qr(r)),l=e.createPrinter({removeComments:!0,omitTrailingSemicolon:!0}),c=t&&e.getSourceFileOfNode(t);return l.writeNode(4,s,c,i),i}}function Yr(n,t,r,a){void 0===r&&(r=1064960),void 0===a&&(a=e.createTextWriter(\"\"));var i=A.noErrorTruncation||1&r,o=F.typeToTypeNode(n,t,3112960|Qr(r)|(i?1:0),a);if(void 0===o)return e.Debug.fail(\"should always get typenode\");var s=e.createPrinter({removeComments:!0}),l=t&&e.getSourceFileOfNode(t);s.writeNode(4,o,l,a);var c=a.getText(),u=i?void 0:2*e.defaultMaximumTruncationLength;return u&&c&&c.length>=u?c.substr(0,u-\"...\".length)+\"...\":c}function Qr(e){return void 0===e&&(e=0),9469291&e}function Zr(n,t,r,a){return void 0===r&&(r=16384),a?i(a).getText():e.usingSingleLineStringWriter(i);function i(a){var i=e.createTypePredicateNode(1===n.kind?e.createIdentifier(n.parameterName):e.createThisTypeNode(),F.typeToTypeNode(n.type,t,3113472|Qr(r))),o=e.createPrinter({removeComments:!0}),s=t&&e.getSourceFileOfNode(t);return o.writeNode(4,i,s,a),a}}function $r(e){return 8===e?\"private\":16===e?\"protected\":\"public\"}function ea(n){return n&&n.parent&&243===n.parent.kind&&e.isExternalModuleAugmentation(n.parent.parent)}function na(n){return 277===n.kind||e.isAmbientModule(n)}function ta(n,t){if(t&&\"default\"===n.escapedName&&!(16384&t.flags)&&(!(16777216&t.flags)||!n.declarations||t.enclosingDeclaration&&e.findAncestor(n.declarations[0],na)!==e.findAncestor(t.enclosingDeclaration,na)))return\"default\";if(n.declarations&&n.declarations.length){var r=n.declarations[0],a=e.getNameOfDeclaration(r);if(a)return e.declarationNameToString(a);if(r.parent&&235===r.parent.kind)return e.declarationNameToString(r.parent.name);switch(r.kind){case 207:case 194:case 195:return!t||t.encounteredError||131072&t.flags||(t.encounteredError=!0),207===r.kind?\"(Anonymous class)\":\"(Anonymous function)\"}}var i=n.nameType;if(i){if(64&i.flags&&!e.isIdentifierText(i.value,A.target))return'\"'+e.escapeString(i.value,34)+'\"';if(i&&2048&i.flags)return\"[\"+ta(i.symbol,t)+\"]\"}return e.symbolName(n)}function ra(n){if(n){var t=Rt(n);return void 0===t.isVisible&&(t.isVisible=!!function(){switch(n.kind){case 295:case 302:return!!(n.parent&&n.parent.parent&&n.parent.parent.parent&&e.isSourceFile(n.parent.parent.parent));case 184:return ra(n.parent.parent);case 235:if(e.isBindingPattern(n.name)&&!n.name.elements.length)return!1;case 242:case 238:case 239:case 240:case 237:case 241:case 246:if(e.isExternalModuleAugmentation(n))return!0;var t=ca(n);return 1&e.getCombinedModifierFlags(n)||246!==n.kind&&277!==t.kind&&4194304&t.flags?ra(t):Nt(t);case 152:case 151:case 156:case 157:case 154:case 153:if(e.hasModifier(n,24))return!1;case 155:case 159:case 158:case 160:case 149:case 243:case 163:case 164:case 166:case 162:case 167:case 168:case 171:case 172:case 175:return ra(n.parent);case 248:case 249:case 251:return!1;case 148:case 277:case 245:return!0;case 252:default:return!1}}()),t.isVisible}return!1}function aa(n,t){var r,a;return n.parent&&252===n.parent.kind?r=Ft(n,n.escapedText,70107135,void 0,n,!1):255===n.parent.kind&&(r=Yt(n.parent,70107135)),r&&function n(r){e.forEach(r,function(r){var i=jt(r)||r;if(t?Rt(r).isVisible=!0:(a=a||[],e.pushIfUnique(a,i)),e.isInternalModuleImportEqualsDeclaration(r)){var o=r.moduleReference,s=ev(o),l=Ft(r,s.escapedText,68009983,void 0,void 0,!1);l&&n(l.declarations)}})}(r.declarations),a}function ia(e,n){var t=oa(e,n);if(t>=0){for(var r=In.length,a=t;a<r;a++)Rn[a]=!1;return!1}return In.push(e),Rn.push(!0),Nn.push(n),!0}function oa(e,n){for(var t=In.length-1;t>=0;t--){if(sa(In[t],Nn[t]))return-1;if(In[t]===e&&Nn[t]===n)return t}return-1}function sa(n,t){switch(t){case 0:return!!It(n).type;case 5:return!!Rt(n).resolvedEnumType;case 2:return!!It(n).declaredType;case 1:return!!n.resolvedBaseConstructorType;case 3:return!!n.resolvedReturnType;case 4:return!!n.immediateBaseConstraint;case 6:return!!It(n).resolvedJSDocType}return e.Debug.assertNever(t)}function la(){return In.pop(),Nn.pop(),Rn.pop()}function ca(n){return e.findAncestor(e.getRootDeclaration(n),function(e){switch(e.kind){case 235:case 236:case 251:case 250:case 249:case 248:return!1;default:return!0}}).parent}function ua(e,n){var t=vo(e,n);return t?Ga(t):void 0}function da(e){return e&&0!=(1&e.flags)}function ma(e){var n=Er(e);return n&&It(n).type||ha(e,!1)}function pa(n){return 147===n.kind&&!e.isStringOrNumericLiteralLike(n.expression)}function fa(n,t,r){if(32768&(n=kd(n,function(e){return!(24576&e.flags)})).flags)return Se;if(262144&n.flags)return Md(n,function(e){return fa(e,t,r)});for(var a=e.createSymbolTable(),i=e.createUnderscoreEscapedMap(),o=0,s=t;o<s.length;o++){var l=s[o];i.set(e.getTextOfPropertyName(l),!0)}for(var c=0,u=$i(n);c<u.length;c++){var d=u[c];i.has(d.escapedName)||24&e.getDeclarationModifierFlagsFromSymbol(d)||!Nl(d)||a.set(d.escapedName,wl(d))}var m=To(n,0),p=To(n,1);return wr(r,a,e.emptyArray,e.emptyArray,m,p)}function ga(n){var t,r=n.parent,a=ma(r.parent);if(a===te)return te;if(!a)return n.initializer?vg(n):a;if(da(a))return a;if(182===r.kind)if(n.dotDotDotToken){if(2&a.flags||!jm(a))return bt(n,e.Diagnostics.Rest_types_may_only_be_created_from_object_types),te;for(var i=[],o=0,s=r.elements;o<s.length;o++){var l=s[o];l.dotDotDotToken||i.push(l.propertyName||l.name)}t=fa(a,i,n.symbol)}else{var c=n.propertyName||n.name,u=vi(c),d=e.isComputedPropertyName(c)&&e.isWellKnownSymbolSyntactically(c.expression);if(!u&&!d&&pa(c)){var m=Lg(c.expression);if(rg(m,3072))return R&&bt(n,e.Diagnostics.Type_0_cannot_be_used_to_index_type_1,Yr(m),Yr(a)),$;var p=rg(m,168)&&So(a,1)||So(a,0);return p||!R||A.suppressImplicitAnyIndexErrors||(So(a,1)?bt(n,e.Diagnostics.Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number):bt(n,e.Diagnostics.Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature,Yr(a))),p||$}var f=u?Bm(c):void 0,g=u?Ei(f):d?e.getPropertyNameForKnownSymbolName(e.idText(c.expression.name)):e.getTextOfPropertyName(c);if(M&&4194304&n.flags&&e.isParameterDeclaration(n)&&(a=Tu(a)),u&&f&&!vo(a,g)&&rg(f,3072))return R&&bt(n,e.Diagnostics.Type_0_cannot_be_used_to_index_type_1,Yr(f),Yr(a)),$;var _=Qd(ua(a,g),n.name);if(!(t=_&&jd(n,_)||Km(g)&&So(a,1)||So(a,0)))return bt(c,e.Diagnostics.Type_0_has_no_property_1_and_no_string_index_signature,Yr(a),e.declarationNameToString(c)),te}else{var v=O_(a,r,!1,!1),y=r.elements.indexOf(n);if(n.dotDotDotToken)t=pu(a)?Fm((a.typeArguments||e.emptyArray).slice(y,cs(a))):Ws(v);else{var h=r.elements.indexOf(n);if(!(t=au(a)?iu(a,h)||n.initializer&&vg(n):v))return pu(a)?bt(n,e.Diagnostics.Tuple_type_0_with_length_1_cannot_be_assigned_to_tuple_with_length_2,Yr(a),cs(a),r.elements.length):bt(n,e.Diagnostics.Type_0_has_no_property_1,Yr(a),\"\"+h),te}}return!M||!n.initializer||8192&vu(gg(n.initializer))||(t=md(t,131072)),n.initializer&&!e.getEffectiveTypeAnnotationNode(e.walkUpBindingElementsAndPatterns(n))?tl([t,gg(n.initializer)],2):t}function _a(n){var t=e.getJSDocType(n);if(t)return Ul(t)}function va(n){var t=e.skipParentheses(n);return 185===t.kind&&0===t.elements.length}function ya(e,n){return void 0===n&&(n=!0),M&&n?Eu(e):e}function ha(n,t){if(e.isVariableDeclaration(n)&&224===n.parent.parent.kind){var r=ml(vp(n.parent.parent.expression));return 1114112&r.flags?pl(r):le}if(e.isVariableDeclaration(n)&&225===n.parent.parent.kind){var a=n.parent.parent;return M_(a.expression,a.awaitModifier)||$}if(e.isBindingPattern(n.parent))return ga(n);var i,o,s=t&&(e.isParameter(n)&&Co(n)||!e.isBindingElement(n)&&!e.isVariableDeclaration(n)&&!!n.questionToken),l=ka(n);if(l)return ya(l,s);if((R||e.isInJSFile(n))&&235===n.kind&&!e.isBindingPattern(n.name)&&!(1&e.getCombinedModifierFlags(n))&&!(4194304&n.flags)){if(!(2&e.getCombinedNodeFlags(n)||n.initializer&&(i=n.initializer,o=e.skipParentheses(i),95!==o.kind&&(71!==o.kind||nd(o)!==G))))return ee;if(n.initializer&&va(n.initializer))return qe}if(149===n.kind){var c,u=n.parent;if(157===u.kind&&!bi(u)){var d=e.getDeclarationOfKind(Er(n.parent),156);if(d){var m=wo(d),p=Ty(u);return p&&n===p?(e.Debug.assert(!p.type),Ga(m.thisParameter)):Ho(m)}}if(e.isInJSFile(n)){var f=e.getJSDocType(u);if(f&&e.isFunctionTypeNode(f))return Mf(wo(f),u.parameters.indexOf(n))}if(c=\"this\"===n.symbol.escapedName?dm(u):mm(n))return ya(c,s)}else if(e.isInJSFile(n)){var g=Ea(n,Er(n),e.getDeclaredExpandoInitializer(n));if(g)return g}return n.initializer?ya(c=vg(n),s):e.isJsxAttribute(n)?me:e.isBindingPattern(n.name)?Aa(n.name,!1,!0):void 0}function ba(n,t){var r=e.getAssignedExpandoInitializer(n.valueDeclaration);if(r){var a=e.getJSDocTypeTag(r);return a&&a.typeExpression?Ul(a.typeExpression):Ea(n.valueDeclaration,n,r)||uu(gg(r))}for(var i,o,s=!1,l=!1,c=0,u=n.declarations;c<u.length;c++){var d=u[c],m=e.isBinaryExpression(d)?d:e.isPropertyAccessExpression(d)?e.isBinaryExpression(d.parent)?d.parent:d:void 0;if(!m)return te;var p=e.isPropertyAccessExpression(m)?e.getAssignmentDeclarationPropertyAccessKind(m):e.getAssignmentDeclarationKind(m);4===p&&(La(m)?s=!0:l=!0),(i=Ta(i,m,0,d))||(o||(o=[])).push(e.isBinaryExpression(m)?Sa(n,t,m,p):ve)}var f=i;if(!f){var g=s?function(n,t){return e.Debug.assert(n.length===t.length),n.filter(function(n,r){var a=t[r],i=e.isBinaryExpression(a)?a:e.isBinaryExpression(a.parent)?a.parent:void 0;return i&&La(i)})}(o,n.declarations):void 0;if(l){var _=function(n){var t=e.forEach(n.declarations,function(n){var t=e.getThisContainer(n,!1).parent;return e.isClassLike(t)&&t});if(t){var r=si(Er(t)),a=r&&Za(r)[0];if(a)return ua(a,n.escapedName)}}(n);_&&((g||(g=[])).push(_),s=!0)}f=tl(e.some(g,function(e){return!!(-134242305&e.flags)})?g:o,2)}var v=Mu(ya(f,l&&!s));return kd(v,function(e){return!!(-24577&e.flags)})===ve?(R&&Iu(n.valueDeclaration,$),$):v}function Ea(n,t,r){if(e.isInJSFile(n)&&r&&e.isObjectLiteralExpression(r)&&!r.properties.length){for(var a=e.createSymbolTable();e.isBinaryExpression(n)||e.isPropertyAccessExpression(n);){var i=Er(n);i&&e.hasEntries(i.exports)&&Mt(a,i.exports),n=e.isBinaryExpression(n)?n.parent:n.parent.parent}var o=Er(n);o&&e.hasEntries(o.exports)&&Mt(a,o.exports);var s=wr(t,a,e.emptyArray,e.emptyArray,void 0,void 0);return s.objectFlags|=16384,s}}function Ta(n,t,r,a){var i=e.getJSDocType(t.parent);if(i){var o=Mu(Ul(i));if(!n)return o;n===te||o===te||_c(n,o)||S_(n,a,o)}return n}function Sa(n,t,r,a){var i=t?Ga(t):uu(gg(r.right));if(131072&i.flags&&2===a&&\"export=\"===n.escapedName){var o=Xi(i),s=e.createSymbolTable();e.copyEntries(o.members,s),t&&!t.exports&&(t.exports=e.createSymbolTable()),(t||n).exports.forEach(function(e,n){if(s.has(n)){var t=o.members.get(n),r=Tt(e.flags|t.flags,n);r.type=tl([Ga(e),Ga(t)]),s.set(n,r)}else s.set(n,e)});var l=wr(o.symbol,s,o.callSignatures,o.constructSignatures,o.stringIndexInfo,o.numberIndexInfo);return l.objectFlags|=16384&e.getObjectFlags(i),l}return ru(i)?(R&&Iu(r,We),We):i}function La(n){var t=e.getThisContainer(n,!1);return 155===t.kind||237===t.kind||194===t.kind&&!e.isPrototypePropertyAssignment(t.parent)}function xa(n,t,r){return n.initializer?ya(vg(n)):e.isBindingPattern(n.name)?Aa(n.name,t,r):(r&&R&&!Da(n)&&Iu(n,$),$)}function Aa(n,t,r){return void 0===t&&(t=!1),void 0===r&&(r=!1),182===n.kind?function(n,t,r){var a,i=e.createSymbolTable(),o=128;e.forEach(n.elements,function(n){var s=n.propertyName||n.name;if(pa(s))o|=512;else if(n.dotDotDotToken)a=ns($,!1);else{var l=e.getTextOfPropertyName(s),c=Tt(4|(n.initializer?16777216:0),l);c.type=xa(n,t,r),c.bindingElement=n,i.set(c.escapedName,c)}});var s=wr(void 0,i,e.emptyArray,e.emptyArray,a,void 0);return s.flags|=268435456,s.objectFlags|=o,t&&(s.pattern=n),s}(n,t,r):function(n,t,r){var a=n.elements,i=e.lastOrUndefined(a),o=!(!i||184!==i.kind||!i.dotDotDotToken);if(0===a.length||1===a.length&&o)return C>=2?Us($):We;var s=Js(e.map(a,function(n){return e.isOmittedExpression(n)?$:xa(n,t,r)}),e.findLastIndex(a,function(n){return!e.isOmittedExpression(n)&&!wm(n)},a.length-(o?2:1))+1,o);return t&&((s=ls(s)).pattern=n),s}(n,t,r)}function Ca(n,t){return function(n,t,r){return n?(r&&Ru(t,n),2048&n.flags&&(e.isBindingElement(t)||!t.type)&&n.symbol!==Er(t)&&(n=ge),Mu(n)):(n=e.isParameter(t)&&t.dotDotDotToken?We:$,r&&R&&(Da(t)||Iu(t,n)),n)}(ha(n,!0),n,t)}function Da(n){var t=e.getRootDeclaration(n);return Bg(149===t.kind?t.parent:t)}function ka(n){var t=e.getEffectiveTypeAnnotationNode(n);if(t)return Ul(t)}function Ma(n){var t=It(n);return t.type||(t.type=function(n){if(4194304&n.flags)return(t=si(Tr(n))).typeParameters?ss(t,e.map(t.typeParameters,function(e){return $})):t;var t;if(n===B)return $;if(134217728&n.flags){var r=Er(e.getSourceFileOfNode(n.valueDeclaration)),a=e.createSymbolTable();return a.set(\"exports\",r),wr(n,a,e.emptyArray,e.emptyArray,void 0,void 0)}var i,o=n.valueDeclaration;if(e.isCatchClauseVariableDeclarationOrBindingElement(o))return $;if(e.isSourceFile(o)){var s=e.cast(o,e.isJsonSourceFile);if(!s.statements.length)return Se;var l=uu(Lg(s.statements[0].expression));return 131072&l.flags?xu(l):l}if(252===o.kind)return Lg(o.expression);if(!ia(n,0))return te;if(e.isInJSFile(o)&&(e.isBinaryExpression(o)||e.isPropertyAccessExpression(o)&&e.isBinaryExpression(o.parent)))i=ba(n);else if(e.isJSDocPropertyLikeTag(o)||e.isPropertyAccessExpression(o)||e.isIdentifier(o)||e.isClassDeclaration(o)||e.isFunctionDeclaration(o)||e.isMethodDeclaration(o)&&!e.isObjectLiteralMethod(o)||e.isMethodSignature(o)){if(9136&n.flags)return Pa(n);i=e.isBinaryExpression(o.parent)?ba(n):ka(o)||$}else if(e.isPropertyAssignment(o))i=ka(o)||hg(o);else if(e.isJsxAttribute(o))i=ka(o)||zm(o);else if(e.isShorthandPropertyAssignment(o))i=ka(o)||yg(o.name,0);else if(e.isObjectLiteralMethod(o))i=ka(o)||bg(o,0);else{if(!(e.isParameter(o)||e.isPropertyDeclaration(o)||e.isPropertySignature(o)||e.isVariableDeclaration(o)||e.isBindingElement(o)))return e.Debug.fail(\"Unhandled declaration kind! \"+e.Debug.showSyntaxKind(o)+\" for \"+e.Debug.showSymbol(n));i=Ca(o,!0)}return la()||(i=Fa(n)),i}(n))}function Oa(n){if(n)return 156===n.kind?e.getEffectiveReturnTypeNode(n):e.getEffectiveSetAccessorTypeAnnotationNode(n)}function Ia(e){var n=Oa(e);return n&&Ul(n)}function Ra(e){return Vo(wo(e))}function Na(n){var t=It(n);return t.type||(t.type=function(n){var t,r=e.getDeclarationOfKind(n,156),a=e.getDeclarationOfKind(n,157);if(r&&e.isInJSFile(r)){var i=_a(r);if(i)return i}if(!ia(n,0))return te;var o=Ia(r);if(o)t=o;else{var s=Ia(a);s?t=s:r&&r.body?t=Uf(r):(R&&(a?bt(a,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation,Jr(n)):(e.Debug.assert(!!r,\"there must existed getter as we are current checking either setter or getter in this function\"),bt(r,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation,Jr(n)))),t=$)}if(!la()&&(t=$,R)){var l=e.getDeclarationOfKind(n,156);bt(l,e.Diagnostics._0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions,Jr(n))}return t}(n))}function wa(e){var n=Qa(ei(e));return 2162688&n.flags?n:void 0}function Pa(n){var t=It(n),r=t;if(!t.type){var a=e.getDeclarationOfExpando(n.valueDeclaration);if(a){var i=Er(a);i&&(e.hasEntries(i.exports)||e.hasEntries(i.members))&&(t=n=At(n),e.hasEntries(i.exports)&&(n.exports=n.exports||e.createSymbolTable(),Mt(n.exports,i.exports)),e.hasEntries(i.members)&&(n.members=n.members||e.createSymbolTable(),Mt(n.members,i.members)))}r.type=t.type=function(n){var t=n.valueDeclaration;if(1536&n.flags&&e.isShorthandAmbientModuleSymbol(n))return $;if(202===t.kind||187===t.kind&&202===t.parent.kind)return ba(n);if(512&n.flags&&t&&e.isSourceFile(t)&&t.commonJsModuleIndicator){var r=dr(n);if(r!==n){if(!ia(n,0))return te;var a=br(n.exports.get(\"export=\")),i=ba(a,a===r?void 0:r);return la()?i:Fa(n)}}var o=Or(16,n);if(32&n.flags){var s=wa(n);return s?ll([o,s]):o}return M&&16777216&n.flags?Eu(o):o}(n)}return t.type}function Fa(n){return e.getEffectiveTypeAnnotationNode(n.valueDeclaration)?(bt(n.valueDeclaration,e.Diagnostics._0_is_referenced_directly_or_indirectly_in_its_own_type_annotation,Jr(n)),te):(R&&bt(n.valueDeclaration,e.Diagnostics._0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or_indirectly_in_its_own_initializer,Jr(n)),$)}function Ga(n){return 1&e.getCheckFlags(n)?function(e){var n=It(e);if(!n.type){if(!ia(e,0))return n.type=te;var t=lc(Ga(n.target),n.mapper);la()||(t=Fa(e)),n.type=t}return n.type}(n):2048&e.getCheckFlags(n)?function(e){return Bu(e.propertyType,e.mappedType)}(n):7&n.flags?Ma(n):9136&n.flags?Pa(n):8&n.flags?function(e){var n=It(e);return n.type||(n.type=ii(e))}(n):98304&n.flags?Na(n):2097152&n.flags?function(e){var n=It(e);if(!n.type){var t=er(e);n.type=67220415&t.flags?Ga(t):te}return n.type}(n):te}function Va(n,t){return void 0!==n&&void 0!==t&&0!=(4&e.getObjectFlags(n))&&n.target===t}function Ka(n){return 4&e.getObjectFlags(n)?n.target:n}function Ba(n,t){return function n(r){if(7&e.getObjectFlags(r)){var a=Ka(r);return a===t||e.some(Za(a),n)}return!!(524288&r.flags)&&e.some(r.types,n)}(n)}function Ha(n,t){for(var r=0,a=t;r<a.length;r++){var i=a[r];n=e.appendIfUnique(n,oi(Er(i)))}return n}function Ua(n,t){for(;;){if(!(n=n.parent))return;switch(n.kind){case 238:case 207:case 239:case 158:case 159:case 153:case 163:case 164:case 287:case 237:case 154:case 194:case 195:case 240:case 301:case 302:case 295:case 179:case 173:var r=Ua(n,t);if(179===n.kind)return e.append(r,oi(Er(n.typeParameter)));if(173===n.kind)return e.concatenate(r,Al(n));var a=Ha(r,e.getEffectiveTypeParameterDeclarations(n)),i=t&&(238===n.kind||207===n.kind||239===n.kind)&&ei(Er(n)).thisType;return i?e.append(a,i):a}}}function ja(n){return Ua(32&n.flags?n.valueDeclaration:e.getDeclarationOfKind(n,239))}function Wa(n){for(var t,r=0,a=n.declarations;r<a.length;r++){var i=a[r];if(239===i.kind||238===i.kind||207===i.kind||e.isTypeAlias(i)){var o=i;t=Ha(t,e.getEffectiveTypeParameterDeclarations(o))}}return t}function qa(e){var n=ho(e,1);if(1===n.length){var t=n[0];return!t.typeParameters&&1===t.parameters.length&&t.hasRestParameter&&Df(t.parameters[0])===We}return!1}function za(e){if($a(e)&&ho(e,1).length>0)return!0;if(2162688&e.flags){var n=io(e);return!!n&&$a(n)&&qa(n)}return _f(e)}function Ja(n){return e.getEffectiveBaseTypeNode(n.symbol.valueDeclaration)}function Xa(n,t,r){var a=e.length(t),i=e.isInJSFile(r);return _f(n)&&!a?ho(n,0):e.filter(ho(n,1),function(n){return(i||a>=Ro(n.typeParameters))&&a<=e.length(n.typeParameters)})}function Ya(n,t,r){var a=Xa(n,t,r),i=e.map(t,Ul);return e.sameMap(a,function(n){return e.some(n.typeParameters)?Wo(n,i,e.isInJSFile(r)):n})}function Qa(n){if(!n.resolvedBaseConstructorType){var t=n.symbol.valueDeclaration,r=e.getEffectiveBaseTypeNode(t),a=Ja(n);if(!a)return n.resolvedBaseConstructorType=ae;if(!ia(n,1))return te;var i=Lg(a.expression);if(r&&a!==r&&(e.Debug.assert(!r.typeArguments),Lg(r.expression)),655360&i.flags&&Xi(i),!la())return bt(n.symbol.valueDeclaration,e.Diagnostics._0_is_referenced_directly_or_indirectly_in_its_own_base_expression,Jr(n.symbol)),n.resolvedBaseConstructorType=te;if(!(1&i.flags||i===se||za(i)))return bt(a.expression,e.Diagnostics.Type_0_is_not_a_constructor_function_type,Yr(i)),n.resolvedBaseConstructorType=te;n.resolvedBaseConstructorType=i}return n.resolvedBaseConstructorType}function Za(n){return n.resolvedBaseTypes||(8&n.objectFlags?n.resolvedBaseTypes=[Ws(tl(n.typeParameters||e.emptyArray))]:96&n.symbol.flags?(32&n.symbol.flags&&function(n){n.resolvedBaseTypes=e.resolvingEmptyArray;var t=po(Qa(n));if(!(655361&t.flags))return n.resolvedBaseTypes=e.emptyArray;var r,a=Ja(n),i=Ss(a),o=t&&t.symbol?si(t.symbol):void 0;if(t.symbol&&32&t.symbol.flags&&function(e){var n=e.outerTypeParameters;if(n){var t=n.length-1,r=e.typeArguments;return n[t].symbol!==r[t].symbol}return!0}(o))r=us(a,t.symbol,i);else if(1&t.flags)r=t;else if(_f(t)&&!a.typeArguments)r=vf(t.symbol)||$;else{var s=Ya(t,a.typeArguments,a);if(!s.length)return bt(a.expression,e.Diagnostics.No_base_constructor_has_the_specified_number_of_type_arguments),n.resolvedBaseTypes=e.emptyArray;r=Ho(s[0])}r===te?n.resolvedBaseTypes=e.emptyArray:$a(r)?n===r||Ba(r,n)?(bt(n.symbol.valueDeclaration,e.Diagnostics.Type_0_recursively_references_itself_as_a_base_type,Yr(n,void 0,2)),n.resolvedBaseTypes=e.emptyArray):(n.resolvedBaseTypes===e.resolvingEmptyArray&&(n.members=void 0),n.resolvedBaseTypes=[r]):(bt(a.expression,e.Diagnostics.Base_constructor_return_type_0_is_not_a_class_or_interface_type,Yr(r)),n.resolvedBaseTypes=e.emptyArray)}(n),64&n.symbol.flags&&function(n){n.resolvedBaseTypes=n.resolvedBaseTypes||e.emptyArray;for(var t=0,r=n.symbol.declarations;t<r.length;t++){var a=r[t];if(239===a.kind&&e.getInterfaceBaseTypeNodes(a))for(var i=0,o=e.getInterfaceBaseTypeNodes(a);i<o.length;i++){var s=o[i],l=Ul(s);l!==te&&($a(l)?n===l||Ba(l,n)?bt(a,e.Diagnostics.Type_0_recursively_references_itself_as_a_base_type,Yr(n,void 0,2)):n.resolvedBaseTypes===e.emptyArray?n.resolvedBaseTypes=[l]:n.resolvedBaseTypes.push(l):bt(s,e.Diagnostics.An_interface_may_only_extend_a_class_or_another_interface))}}}(n)):e.Debug.fail(\"type must be class or interface\")),n.resolvedBaseTypes}function $a(n){return!!(16908289&n.flags)&&!Ji(n)||!!(524288&n.flags)&&e.every(n.types,$a)}function ei(n){var t=It(n);if(!t.declaredType){var r=32&n.flags?1:2,a=t.declaredType=Or(r,n),i=ja(n),o=Wa(n);(i||o||1===r||!function(n){for(var t=0,r=n.declarations;t<r.length;t++){var a=r[t];if(239===a.kind){if(64&a.flags)return!1;var i=e.getInterfaceBaseTypeNodes(a);if(i)for(var o=0,s=i;o<s.length;o++){var l=s[o];if(e.isEntityNameExpression(l.expression)){var c=ir(l.expression,67897832,!0);if(!c||!(64&c.flags)||ei(c).thisType)return!1}}}}return!0}(n))&&(a.objectFlags|=4,a.typeParameters=e.concatenate(i,o),a.outerTypeParameters=i,a.localTypeParameters=o,a.instantiations=e.createMap(),a.instantiations.set(is(a.typeParameters),a),a.target=a,a.typeArguments=a.typeParameters,a.thisType=Dr(65536),a.thisType.isThisType=!0,a.thisType.symbol=n,a.thisType.constraint=a)}return t.declaredType}function ni(n){var t=n.initializer;if(!t)return!(4194304&n.flags);switch(t.kind){case 9:case 8:return!0;case 200:return 38===t.operator&&8===t.operand.kind;case 71:return e.nodeIsMissing(t)||!!Er(n.parent).exports.get(t.escapedText);case 202:return function e(n){return 9===n.kind||202===n.kind&&e(n.left)&&e(n.right)}(t);default:return!1}}function ti(e){var n=It(e);if(void 0!==n.enumKind)return n.enumKind;for(var t=!1,r=0,a=e.declarations;r<a.length;r++){var i=a[r];if(241===i.kind)for(var o=0,s=i.members;o<s.length;o++){var l=s[o];if(l.initializer&&9===l.initializer.kind)return n.enumKind=1;ni(l)||(t=!0)}}return n.enumKind=t?0:1}function ri(e){return 512&e.flags&&!(262144&e.flags)?si(Tr(e.symbol)):e}function ai(e){var n=It(e);if(n.declaredType)return n.declaredType;if(1===ti(e)){E++;for(var t=[],r=0,a=e.declarations;r<a.length;r++){var i=a[r];if(241===i.kind)for(var o=0,s=i.members;o<s.length;o++){var l=s[o],c=Gl(Kl(qv(l),E,Er(l)));It(Er(l)).declaredType=c,t.push(Vl(c))}}if(t.length){var u=tl(t,1,e,void 0);return 262144&u.flags&&(u.flags|=512,u.symbol=e),n.declaredType=u}}var d=Dr(32);return d.symbol=e,n.declaredType=d}function ii(e){var n=It(e);if(!n.declaredType){var t=ai(Tr(e));n.declaredType||(n.declaredType=t)}return n.declaredType}function oi(e){var n=It(e);if(!n.declaredType){var t=Dr(65536);t.symbol=e,n.declaredType=t}return n.declaredType}function si(e){return li(e)||te}function li(n){return 96&n.flags?ei(n):524288&n.flags?function(n){var t=It(n);if(!t.declaredType){if(!ia(n,2))return te;var r=e.find(n.declarations,function(n){return e.isJSDocTypeAlias(n)||240===n.kind}),a=e.isJSDocTypeAlias(r)?r.typeExpression:r.type,i=a?Ul(a):te;if(la()){var o=Wa(n);o&&(t.typeParameters=o,t.instantiations=e.createMap(),t.instantiations.set(is(o),i))}else i=te,bt(r.name,e.Diagnostics.Type_alias_0_circularly_references_itself,Jr(n));t.declaredType=i}return t.declaredType}(n):262144&n.flags?oi(n):384&n.flags?ai(n):8&n.flags?ii(n):2097152&n.flags?function(e){var n=It(e);return n.declaredType||(n.declaredType=si(er(e))),n.declaredType}(n):void 0}function ci(e){switch(e.kind){case 119:case 142:case 137:case 134:case 122:case 138:case 135:case 105:case 140:case 95:case 131:case 180:return!0;case 167:return ci(e.elementType);case 162:return!e.typeArguments||e.typeArguments.every(ci)}return!1}function ui(n){var t=e.getEffectiveConstraintOfTypeParameter(n);return!t||ci(t)}function di(n){var t=e.getEffectiveTypeAnnotationNode(n);return t?ci(t):!e.hasInitializer(n)}function mi(n){if(n.declarations&&1===n.declarations.length){var t=n.declarations[0];if(t)switch(t.kind){case 152:case 151:return di(t);case 154:case 153:case 155:return r=t,a=e.getEffectiveReturnTypeNode(r),i=e.getEffectiveTypeParameterDeclarations(r),(155===r.kind||!!a&&ci(a))&&r.parameters.every(di)&&i.every(ui)}}var r,a,i;return!1}function pi(n,t,r){for(var a=e.createSymbolTable(),i=0,o=n;i<o.length;i++){var s=o[i];a.set(s.escapedName,r&&mi(s)?s:nc(s,t))}return a}function fi(e,n){for(var t=0,r=n;t<r.length;t++){var a=r[t];e.has(a.escapedName)||e.set(a.escapedName,a)}}function gi(n){if(!n.declaredProperties){var t=n.symbol,r=Li(t);n.declaredProperties=Rr(r),n.declaredCallSignatures=e.emptyArray,n.declaredConstructSignatures=e.emptyArray,n.declaredCallSignatures=Go(r.get(\"__call\")),n.declaredConstructSignatures=Go(r.get(\"__new\")),n.declaredStringIndexInfo=ts(t,0),n.declaredNumberIndexInfo=ts(t,1)}return n}function _i(e){return!!(2240&e.flags)}function vi(n){return e.isComputedPropertyName(n)&&e.isEntityNameExpression(n.expression)&&_i(Bm(n))}function yi(e){return 95===e.charCodeAt(0)&&95===e.charCodeAt(1)&&64===e.charCodeAt(2)}function hi(n){var t=e.getNameOfDeclaration(n);return!!t&&vi(t)}function bi(n){return e.hasDynamicName(n)&&!hi(n)}function Ei(n){return 2048&n.flags?\"__@\"+n.symbol.escapedName+\"@\"+l(n.symbol):192&n.flags?e.escapeLeadingUnderscores(\"\"+n.value):e.Debug.fail()}function Ti(n,t,r,a){e.Debug.assert(!!a.symbol,\"The member is expected to have a symbol.\");var i=Rt(a);if(!i.resolvedSymbol){i.resolvedSymbol=a.symbol;var o=Bm(a.name);if(_i(o)){var s=Ei(o),l=a.symbol.flags,c=r.get(s);c||r.set(s,c=Tt(0,s,1024));var u=t&&t.get(s);if(c.flags&Lt(l)||u){var d=u?e.concatenate(u.declarations,c.declarations):c.declarations,m=e.declarationNameToString(a.name);e.forEach(d,function(n){return bt(e.getNameOfDeclaration(n)||n,e.Diagnostics.Duplicate_declaration_0,m)}),bt(a.name||a,e.Diagnostics.Duplicate_declaration_0,m),c=Tt(0,s,1024)}return c.nameType=o,function(n,t,r){e.Debug.assert(!!(1024&e.getCheckFlags(n)),\"Expected a late-bound symbol.\"),n.flags|=r,It(t.symbol).lateSymbol=n,n.declarations?n.declarations.push(t):n.declarations=[t],67220415&r&&(n.valueDeclaration&&n.valueDeclaration.kind===t.kind||(n.valueDeclaration=t))}(c,a,l),c.parent?e.Debug.assert(c.parent===n,\"Existing symbol parent should match new one\"):c.parent=n,i.resolvedSymbol=c}}return i.resolvedSymbol}function Si(n,t){var r=It(n);if(!r[t]){var a=\"resolvedExports\"===t,i=a?1536&n.flags?hr(n):n.exports:n.members;r[t]=i||L;for(var o=e.createSymbolTable(),s=0,l=n.declarations;s<l.length;s++){var c=l[s],u=e.getMembersOfDeclaration(c);if(u)for(var d=0,m=u;d<m.length;d++){var p=m[d];a===e.hasStaticModifier(p)&&hi(p)&&Ti(n,i,o,p)}}r[t]=function(n,t){if(!e.hasEntries(n))return t;if(!e.hasEntries(t))return n;var r=e.createSymbolTable();return Mt(r,n),Mt(r,t),r}(i,o)||L}return r[t]}function Li(e){return 6240&e.flags?Si(e,\"resolvedMembers\"):e.members||L}function xi(n){if(106500&n.flags&&\"__computed\"===n.escapedName){var t=It(n);return!t.lateSymbol&&e.some(n.declarations,hi)&&(e.some(n.declarations,e.hasStaticModifier)?_r(n.parent):Li(n.parent)),t.lateSymbol||(t.lateSymbol=n)}return n}function Ai(n,t,r){if(4&e.getObjectFlags(n)){var a=n.target,i=n.typeArguments;if(e.length(a.typeParameters)===e.length(i)){var o=ss(a,e.concatenate(i,[t||a.thisType]));return r?po(o):o}}else if(524288&n.flags)return ll(e.map(n.types,function(e){return Ai(e,t,r)}));return r?po(n):n}function Ci(n,t,r,a){var i,o,s,l,c,u;e.rangeEquals(r,a,0,r.length)?(i=x,o=t.symbol?Li(t.symbol):e.createSymbolTable(t.declaredProperties),s=t.declaredCallSignatures,l=t.declaredConstructSignatures,c=t.declaredStringIndexInfo,u=t.declaredNumberIndexInfo):(i=Jl(r,a),o=pi(t.declaredProperties,i,1===r.length),s=ql(t.declaredCallSignatures,i),l=ql(t.declaredConstructSignatures,i),c=uc(t.declaredStringIndexInfo,i),u=uc(t.declaredNumberIndexInfo,i));var d=Za(t);if(d.length){t.symbol&&o===Li(t.symbol)&&(o=e.createSymbolTable(t.declaredProperties)),Nr(n,o,s,l,c,u);for(var m=e.lastOrUndefined(a),p=0,f=d;p<f.length;p++){var g=f[p],_=m?Ai(lc(g,i),m):g;fi(o,$i(_)),s=e.concatenate(s,ho(_,0)),l=e.concatenate(l,ho(_,1)),c||(c=_===$?ns($,!1):To(_,0)),u=u||To(_,1)}}Nr(n,o,s,l,c,u)}function Di(e,n,t,r,a,i,o,s,l){var c=new y(H);return c.declaration=e,c.typeParameters=n,c.parameters=r,c.thisParameter=t,c.resolvedReturnType=a,c.resolvedTypePredicate=i,c.minArgumentCount=o,c.hasRestParameter=s,c.hasLiteralTypes=l,c.target=void 0,c.mapper=void 0,c}function ki(e){return Di(e.declaration,e.typeParameters,e.thisParameter,e.parameters,void 0,void 0,e.minArgumentCount,e.hasRestParameter,e.hasLiteralTypes)}function Mi(e,n,t,r,a){for(var i=0,o=e;i<o.length;i++){var s=o[i];if(Zc(s,n,t,r,a,vc))return s}}function Oi(n,t,r){if(t.typeParameters){if(r>0)return;for(var a=1;a<n.length;a++)if(!Mi(n[a],t,!1,!1,!1))return;return[t]}var i;for(a=0;a<n.length;a++){var o=a===r?t:Mi(n[a],t,!0,!0,!0);if(!o)return;i=e.appendIfUnique(i,o)}return i}function Ii(n,t){for(var r,a=e.map(n,function(e){return ho(e,t)}),i=0;i<a.length;i++)for(var o=0,s=a[i];o<s.length;o++){var l=s[o];if(!r||!Mi(r,l,!1,!0,!0)){var c=Oi(a,l,i);if(c){var u=l;if(c.length>1){var d=l.thisParameter;if(e.forEach(c,function(e){return e.thisParameter})){var m=tl(e.map(c,function(e){return e.thisParameter?Ga(e.thisParameter):$}),2);d=Lu(l.thisParameter,m)}(u=ki(l)).thisParameter=d,u.unionSignatures=c}(r||(r=[])).push(u)}}}return r||e.emptyArray}function Ri(e,n){for(var t=[],r=!1,a=0,i=e;a<i.length;a++){var o=To(i[a],n);if(!o)return;t.push(o.type),r=r||o.isReadonly}return ns(tl(t,2),r)}function Ni(e,n){return e?n?ll([e,n]):e:n}function wi(e,n){return e?n?ns(ll([e.type,n.type]),e.isReadonly&&n.isReadonly):e:n}function Pi(e,n){return e&&n&&ns(tl([e.type,n.type]),e.isReadonly||n.isReadonly)}function Fi(n){for(var t,r,a=e.emptyArray,i=e.emptyArray,o=n.types,s=e.countWhere(o,qa),l=function(l){var c=n.types[l];if(0===s||s===o.length&&0===l||!qa(c)){var u=ho(c,1);u.length&&s>0&&(u=e.map(u,function(e){var n=ki(e);return n.resolvedReturnType=function(e,n,t){for(var r=[],a=0;a<n.length;a++)a===t?r.push(e):qa(n[a])&&r.push(Ho(ho(n[a],1)[0]));return ll(r)}(Ho(e),o,l),n})),i=e.concatenate(i,u)}a=e.concatenate(a,ho(c,0)),t=wi(t,To(c,0)),r=wi(r,To(c,1))},c=0;c<o.length;c++)l(c);Nr(n,L,a,i,t,r)}function Gi(n){var t=n.symbol;if(n.target)Nr(n,L,e.emptyArray,e.emptyArray,void 0,void 0),Nr(n,i=pi(Yi(n.target),n.mapper,!1),r=ql(ho(n.target,0),n.mapper),a=ql(ho(n.target,1),n.mapper),o=uc(To(n.target,0),n.mapper),l=uc(To(n.target,1),n.mapper));else if(2048&t.flags){Nr(n,L,e.emptyArray,e.emptyArray,void 0,void 0);var r=Go((i=Li(t)).get(\"__call\")),a=Go(i.get(\"__new\"));Nr(n,i,r,a,o=ts(t,0),l=ts(t,1))}else{var i=L,o=void 0;if(t.exports&&(i=_r(t)),Nr(n,i,e.emptyArray,e.emptyArray,void 0,void 0),32&t.flags){var s=Qa(c=ei(t));2818048&s.flags?fi(i=e.createSymbolTable(Rr(i)),$i(s)):s===$&&(o=ns($,!1))}var l=384&t.flags?Tn:void 0;if(Nr(n,i,e.emptyArray,e.emptyArray,o,l),8208&t.flags&&(n.callSignatures=Go(t)),32&t.flags){var c=ei(t);(a=Go(t.members.get(\"__constructor\"))).length||(a=function(n){var t=ho(Qa(n),1);if(0===t.length)return[Di(void 0,n.localTypeParameters,void 0,e.emptyArray,n,void 0,0,!1,!1)];for(var r=Ja(n),a=e.isInJSFile(r),i=Ss(r),o=e.length(i),s=[],l=0,c=t;l<c.length;l++){var u=c[l],d=Ro(u.typeParameters),m=e.length(u.typeParameters);if(a||o>=d&&o<=m){var p=m?zo(u,No(i,u.typeParameters,d,a)):ki(u);p.typeParameters=n.localTypeParameters,p.resolvedReturnType=n,s.push(p)}}return s}(c)),n.constructSignatures=a}}}function Vi(e){return e.typeParameter||(e.typeParameter=oi(Er(e.declaration.typeParameter)))}function Ki(e){return e.constraintType||(e.constraintType=lc(no(Vi(e)),e.mapper||x)||te)}function Bi(e){return e.templateType||(e.templateType=e.declaration.type?lc(ya(Ul(e.declaration.type),!!(4&Wi(e))),e.mapper||x):te)}function Hi(n){return e.getEffectiveConstraintOfTypeParameter(n.declaration.typeParameter)}function Ui(e){var n=Hi(e);return 177===n.kind&&128===n.operator}function ji(e){if(!e.modifiersType)if(Ui(e))e.modifiersType=lc(Ul(Hi(e).type),e.mapper||x);else{var n=Ki(El(e.declaration)),t=n&&65536&n.flags?no(n):n;e.modifiersType=t&&1048576&t.flags?lc(t.type,e.mapper||x):Se}return e.modifiersType}function Wi(e){var n=e.declaration;return(n.readonlyToken?38===n.readonlyToken.kind?2:1:0)|(n.questionToken?38===n.questionToken.kind?8:4:0)}function qi(e){var n=Wi(e);return 8&n?-1:4&n?1:0}function zi(e){var n=qi(e),t=ji(e);return n||(Ji(t)?qi(t):0)}function Ji(n){return!!(32&e.getObjectFlags(n))&&_l(Ki(n))}function Xi(n){return n.members||(131072&n.flags?4&n.objectFlags?function(n){var t=gi(n.target),r=e.concatenate(t.typeParameters,[t.thisType]);Ci(n,t,r,n.typeArguments&&n.typeArguments.length===r.length?n.typeArguments:e.concatenate(n.typeArguments,[n]))}(n):3&n.objectFlags?function(n){Ci(n,gi(n),e.emptyArray,e.emptyArray)}(n):2048&n.objectFlags?function(n){for(var t=To(n.source,0),r=Wi(n.mappedType),a=!(1&r),i=4&r?0:16777216,o=t&&ns(Bu(t.type,n.mappedType),a&&t.isReadonly),s=e.createSymbolTable(),l=0,c=$i(n.source);l<c.length;l++){var u=c[l],d=2048|(a&&Zf(u)?8:0),m=Tt(4|u.flags&i,u.escapedName,d);m.declarations=u.declarations,m.nameType=u.nameType,m.propertyType=Ga(u),m.mappedType=n.mappedType,s.set(u.escapedName,m)}Nr(n,s,e.emptyArray,e.emptyArray,o,void 0)}(n):16&n.objectFlags?Gi(n):32&n.objectFlags&&function(n){var t,r,a=e.createSymbolTable();Nr(n,L,e.emptyArray,e.emptyArray,void 0,void 0);var i=Vi(n),o=Ki(n),s=Bi(n.target||n),l=po(ji(n)),c=Wi(n),u=w?64:2240;if(Ui(n)){for(var d=0,m=$i(l);d<m.length;d++)f(ul(m[d],u));(1&l.flags||To(l,0))&&f(le),!w&&To(l,1)&&f(ce)}else{var p=14745600&o.flags?po(o):o;Dd(1048576&p.flags?ml(po(p.type)):p,f)}function f(e){var o=Yl(n.mapper,Jl([i],[e])),u=lc(s,o);if(2240&e.flags){var d=Ei(e),m=vo(l,d),p=!!(4&c||!(8&c)&&m&&16777216&m.flags),f=Tt(4|(p?16777216:0),d,1&c||!(2&c)&&m&&Zf(m)?8:0);f.type=M&&p&&!bc(ae,u)?Eu(u):M&&!p&&m&&16777216&m.flags?md(u,131072):u,m&&(f.syntheticOrigin=m,f.declarations=m.declarations),f.nameType=e,a.set(d,f)}else 5&e.flags?t=ns(u,!!(1&c)):8&e.flags&&(r=ns(u,!!(1&c)))}Nr(n,a,e.emptyArray,e.emptyArray,t,r)}(n):262144&n.flags?function(e){var n=Ii(e.types,0),t=Ii(e.types,1),r=Ri(e.types,0),a=Ri(e.types,1);Nr(e,L,n,t,r,a)}(n):524288&n.flags&&Fi(n)),n}function Yi(n){return 131072&n.flags?Xi(n).properties:e.emptyArray}function Qi(e,n){if(131072&e.flags){var t=Xi(e).members.get(n);if(t&&Ar(t))return t}}function Zi(n){if(!n.resolvedProperties){for(var t=e.createSymbolTable(),r=0,a=n.types;r<a.length;r++){for(var i=0,o=$i(a[r]);i<o.length;i++){var s=o[i];if(!t.has(s.escapedName)){var l=_o(n,s.escapedName);l&&t.set(s.escapedName,l)}}if(262144&n.flags)break}n.resolvedProperties=Rr(t)}return n.resolvedProperties}function $i(e){return 786432&(e=po(e)).flags?Zi(e):Yi(e)}function eo(e){return 65536&e.flags?no(e):2097152&e.flags?function(e){var n=eo(e.objectType)||e.objectType;if(n!==e.objectType){var t=hl(n,e.indexType,void 0,te);if(t&&t!==te)return t}var r=io(e);return r&&r!==e?r:void 0}(e):4194304&e.flags?ao(e):io(e)}function no(e){return so(e)?rs(e):void 0}function to(e){if(!e.resolvedDefaultConstraint){var n=e.root.trueType,t=8388608&n.flags?n.substitute:n;e.resolvedDefaultConstraint=tl([lc(t,e.combinedMapper||e.mapper),xl(e)])}return e.resolvedDefaultConstraint}function ro(e){if(e.root.isDistributive){var n=vl(e.checkType),t=n===e.checkType?eo(n):n;if(t){var r=sc(e,Yl(zl(e.root.checkType,t),e.mapper));if(!(32768&r.flags))return r}}}function ao(e){return ro(e)||to(e)}function io(e){if(15532032&e.flags){var n=lo(e);return n!==De&&n!==ke?n:void 0}return 1048576&e.flags?Te:void 0}function oo(e){return io(e)||e}function so(e){return lo(e)!==ke}function lo(e){var n=!1;return e.resolvedBaseConstraint||(e.resolvedBaseConstraint=Ai(t(e),e));function t(e){if(!e.immediateBaseConstraint){if(!ia(e,4))return ke;if(50===S)return n=!0,e.immediateBaseConstraint=De;S++;var t=function(e){if(65536&e.flags){var n=rs(e);return e.isThisType||!n?n:r(n)}if(786432&e.flags){for(var t=e.types,a=[],i=0,o=t;i<o.length;i++){var s=o[i],l=r(s);l&&a.push(l)}return 262144&e.flags&&a.length===t.length?tl(a):524288&e.flags&&a.length?ll(a):void 0}if(1048576&e.flags)return Te;if(2097152&e.flags){var c=r(e.objectType),u=r(e.indexType),d=c&&u?hl(c,u,void 0,te):void 0;return d&&d!==te?r(d):void 0}if(4194304&e.flags){var n=ao(e);return n&&r(n)}return 8388608&e.flags?r(e.substitute):e}(vl(e));S--,la()&&!n||(t=ke),e.immediateBaseConstraint=t||De}return e.immediateBaseConstraint}function r(e){var n=t(e);return n!==De&&n!==ke?n:void 0}}function co(n){if(n.default)n.default===Me&&(n.default=ke);else if(n.target){var t=co(n.target);n.default=t?lc(t,n.mapper):De}else{n.default=Me;var r=n.symbol&&e.forEach(n.symbol.declarations,function(n){return e.isTypeParameterDeclaration(n)&&n.default}),a=r?Ul(r):De;n.default===Me&&(n.default=a)}return n.default}function uo(e){var n=co(e);return n!==De&&n!==ke?n:void 0}function mo(e){return e.resolvedApparentType||(e.resolvedApparentType=function(e){var n=ac(e);if(n){var t=no(n);if(t&&(eu(t)||nu(t)||pu(t))){var r=zl(n,t);return lc(e,Yl(r,e.mapper))}}return e}(e))}function po(n){var t=15794176&n.flags?io(n)||Se:n;return 32&e.getObjectFlags(t)?mo(t):524288&t.flags?function(e){return e.resolvedApparentType||(e.resolvedApparentType=Ai(e,e,!0))}(t):68&t.flags?Ke:168&t.flags?Be:272&t.flags?He:3072&t.flags?Ms(C>=2):16777216&t.flags?Se:1048576&t.flags?Te:t}function fo(n,t){for(var r,a,i=262144&n.flags,o=i?24:0,s=i?0:16777216,l=4,c=0,u=0,d=n.types;u<d.length;u++)if((L=po(d[u]))!==te){var m=(S=vo(L,t))?e.getDeclarationModifierFlagsFromSymbol(S):0;if(!S||m&o){if(i){var p=!yi(t)&&(Km(t)&&To(L,1)||To(L,0));p?(c|=p.isReadonly?8:0,a=e.append(a,p.type)):c|=16}}else s&=S.flags,r=e.appendIfUnique(r,S),c|=(Zf(S)?8:0)|(24&m?0:64)|(16&m?128:0)|(8&m?256:0)|(32&m?512:0),fp(S)||(l=2)}if(r){if(!(1!==r.length||16&c||a))return r[0];for(var f,g,_,v,y=[],h=!0,b=!1,E=0,T=r;E<T.length;E++){var S=T[E];v?S.valueDeclaration!==v&&(b=!0):v=S.valueDeclaration,f=e.addRange(f,S.declarations);var L=Ga(S);h?(g=L,_=S.nameType,h=!1):L!==g&&(c|=32),y.push(L)}e.addRange(y,a);var x=Tt(4|s,t,l|c);return x.containingType=n,!b&&v&&(x.valueDeclaration=v),x.declarations=f,x.nameType=_,x.type=i?tl(y):ll(y),x}}function go(n,t){var r=n.propertyCache||(n.propertyCache=e.createSymbolTable()),a=r.get(t);return a||(a=fo(n,t))&&r.set(t,a),a}function _o(n,t){var r=go(n,t);return!r||16&e.getCheckFlags(r)?void 0:r}function vo(e,n){if(131072&(e=po(e)).flags){var t=Xi(e),r=t.members.get(n);if(r&&Ar(r))return r;if(t===Ce||t.callSignatures.length||t.constructSignatures.length){var a=Qi(Fe,n);if(a)return a}return Qi(Pe,n)}if(786432&e.flags)return _o(e,n)}function yo(n,t){if(917504&n.flags){var r=Xi(n);return 0===t?r.callSignatures:r.constructSignatures}return e.emptyArray}function ho(e,n){return yo(po(e),n)}function bo(e,n){if(917504&e.flags){var t=Xi(e);return 0===n?t.stringIndexInfo:t.numberIndexInfo}}function Eo(e,n){var t=bo(e,n);return t&&t.type}function To(e,n){return bo(po(e),n)}function So(e,n){return Eo(po(e),n)}function Lo(e,n){if(Su(e)){for(var t=[],r=0,a=$i(e);r<a.length;r++){var i=a[r];(0===n||Km(i.escapedName))&&t.push(Ga(i))}if(t.length)return tl(t,2)}}function xo(n){for(var t,r=0,a=e.getEffectiveTypeParameterDeclarations(n);r<a.length;r++){var i=a[r];t=e.appendIfUnique(t,oi(i.symbol))}return t}function Ao(e){var n=[];return e.forEach(function(e,t){Ir(t)||n.push(e)}),n}function Co(n){return e.isInJSFile(n)&&(n.type&&286===n.type.kind||e.getJSDocParameterTags(n).some(function(e){var n=e.isBracketed,t=e.typeExpression;return n||!!t&&286===t.type.kind}))}function Do(n,t){if(!e.isExternalModuleNameRelative(n)){var r=wt(Sn,'\"'+n+'\"',512);return r&&t?br(r):r}}function ko(n){if(e.hasQuestionToken(n)||Mo(n)||Co(n))return!0;if(n.initializer){var t=wo(n.parent),r=n.parent.parameters.indexOf(n);return e.Debug.assert(r>=0),r>=Nf(t)}var a=e.getImmediatelyInvokedFunctionExpression(n.parent);return!!a&&!n.type&&!n.dotDotDotToken&&n.parent.parameters.indexOf(n)>=a.arguments.length}function Mo(n){if(!e.isJSDocParameterTag(n))return!1;var t=n.isBracketed,r=n.typeExpression;return t||!!r&&286===r.type.kind}function Oo(e,n,t){return{kind:1,parameterName:e,parameterIndex:n,type:t}}function Io(e){return{kind:0,type:e}}function Ro(n){var t,r=0;if(n)for(var a=0;a<n.length;a++)(t=n[a]).symbol&&e.forEach(t.symbol.declarations,function(n){return e.isTypeParameterDeclaration(n)&&n.default})||(r=a+1);return r}function No(n,t,r,a){var i=e.length(t);if(i){var o=e.length(n);if(a||o>=r&&o<=i){n||(n=[]);for(var s=o;s<i;s++)n[s]=Zu(a);for(s=o;s<i;s++){var l=Jl(t,n),c=uo(t[s]);a&&c&&_c(c,Se)&&(c=$),n[s]=c?lc(c,l):Zu(a)}n.length=t.length}}return n}function wo(n){var t,r=Rt(n);if(!r.resolvedSignature){for(var a=[],i=!1,o=0,s=void 0,l=!1,c=e.getImmediatelyInvokedFunctionExpression(n),u=e.isJSDocConstructSignature(n),d=!c&&e.isInJSFile(n)&&e.isValueSignatureDeclaration(n)&&!e.hasJSDocParameterTags(n)&&!e.getJSDocType(n),m=u?1:0;m<n.parameters.length;m++){var p=n.parameters[m],f=p.symbol,g=e.isJSDocParameterTag(p)?p.typeExpression&&p.typeExpression.type:p.type;f&&4&f.flags&&!e.isBindingPattern(p.name)&&(f=Ft(p,f.escapedName,67220415,void 0,void 0,!1)),0===m&&\"this\"===f.escapedName?(l=!0,s=p.symbol):a.push(f),g&&180===g.kind&&(i=!0),Mo(p)||p.initializer||p.questionToken||p.dotDotDotToken||c&&a.length>c.arguments.length&&!g||d||Co(p)||(o=a.length)}if(!(156!==n.kind&&157!==n.kind||bi(n)||l&&s)){var _=156===n.kind?157:156,v=e.getDeclarationOfKind(Er(n),_);v&&(s=(t=Ty(v))&&t.symbol)}var y=155===n.kind?ei(br(n.parent.symbol)):void 0,h=y?y.localTypeParameters:xo(n),b=e.hasRestParameter(n)||e.isInJSFile(n)&&function(n,t){if(e.isJSDocSignature(n)||!Fo(n))return!1;var r=e.lastOrUndefined(n.parameters),a=r?e.getJSDocParameterTags(r):e.getJSDocTags(n).filter(e.isJSDocParameterTag),i=e.firstDefined(a,function(n){return n.typeExpression&&e.isJSDocVariadicType(n.typeExpression.type)?n.typeExpression.type:void 0}),o=Tt(3,\"args\",8192);return o.type=i?Ws(Ul(i.type)):We,i&&t.pop(),t.push(o),!0}(n,a);r.resolvedSignature=Di(n,h,s,a,void 0,void 0,o,b,i)}return r.resolvedSignature}function Po(n){var t=e.isInJSFile(n)?e.getJSDocTypeTag(n):void 0,r=t&&t.typeExpression&&Bp(Ul(t.typeExpression));return r&&Xo(r)}function Fo(n){var t=Rt(n);return void 0===t.containsArgumentsReference&&(8192&t.flags?t.containsArgumentsReference=!0:t.containsArgumentsReference=function n(t){if(!t)return!1;switch(t.kind){case 71:return\"arguments\"===t.escapedText&&e.isExpressionNode(t);case 152:case 154:case 156:case 157:return 147===t.name.kind&&n(t.name);default:return!e.nodeStartsNewLexicalEnvironment(t)&&!e.isPartOfTypeNode(t)&&!!e.forEachChild(t,n)}}(n.body)),t.containsArgumentsReference}function Go(n){if(!n)return e.emptyArray;for(var t=[],r=0;r<n.declarations.length;r++){var a=n.declarations[r];if(e.isFunctionLike(a)){if(r>0&&a.body){var i=n.declarations[r-1];if(a.parent===i.parent&&a.kind===i.kind&&a.pos===i.end)continue}t.push(wo(a))}}return t}function Vo(e){if(e.thisParameter)return Ga(e.thisParameter)}function Ko(e){return void 0!==Bo(e)}function Bo(n){if(!n.resolvedTypePredicate){if(n.target){var t=Bo(n.target);n.resolvedTypePredicate=t?(o=t,s=n.mapper,e.isIdentifierTypePredicate(o)?{kind:1,parameterName:o.parameterName,parameterIndex:o.parameterIndex,type:lc(o.type,s)}:{kind:0,type:lc(o.type,s)}):fn}else if(n.unionSignatures)n.resolvedTypePredicate=function(n){for(var t,r=[],a=0,i=n;a<i.length;a++){var o=i[a],s=Bo(o);if(s){if(t){if(!rl(t,s))return}else t=s;r.push(s.type)}}if(t){var l=tl(r);return e.isIdentifierTypePredicate(t)?Oo(t.parameterName,t.parameterIndex,l):Io(l)}}(n.unionSignatures)||fn;else{var r=n.declaration&&e.getEffectiveReturnTypeNode(n.declaration),a=void 0;if(!r&&e.isInJSFile(n.declaration)){var i=Po(n.declaration);i&&n!==i&&(a=Bo(i))}n.resolvedTypePredicate=r&&e.isTypePredicateNode(r)?function(e,n){var t=e.parameterName,r=Ul(e.type);return 71===t.kind?Oo(t.escapedText,function(e,n){for(var t=0;t<e.length;t++){var r=e[t];if(71===r.name.kind&&r.name.escapedText===n.escapedText)return t}return-1}(n.parameters,t),r):Io(r)}(r,n.declaration):a||fn}e.Debug.assert(!!n.resolvedTypePredicate)}var o,s;return n.resolvedTypePredicate===fn?void 0:n.resolvedTypePredicate}function Ho(n){if(!n.resolvedReturnType){if(!ia(n,3))return te;var t=n.target?lc(Ho(n.target),n.mapper):n.unionSignatures?tl(e.map(n.unionSignatures,Ho),2):Uo(n.declaration)||(e.nodeIsMissing(n.declaration.body)?$:Uf(n.declaration));if(!la()){if(n.declaration){var r=e.getEffectiveReturnTypeNode(n.declaration);if(r)bt(r,e.Diagnostics.Return_type_annotation_circularly_references_itself);else if(R){var a=n.declaration,i=e.getNameOfDeclaration(a);i?bt(i,e.Diagnostics._0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions,e.declarationNameToString(i)):bt(a,e.Diagnostics.Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions)}}t=$}n.resolvedReturnType=t}return n.resolvedReturnType}function Uo(n){if(155===n.kind)return ei(br(n.parent.symbol));if(e.isJSDocConstructSignature(n))return Ul(n.parameters[0].type);var t,r=e.getEffectiveReturnTypeNode(n);if(r)return Ul(r);if(156===n.kind&&!bi(n)){var a=e.isInJSFile(n)&&_a(n);if(a)return a;var i=Ia(e.getDeclarationOfKind(Er(n),157));if(i)return i}return(t=Po(n))&&Ho(t)}function jo(e){if(e.hasRestParameter){var n=Ga(e.parameters[e.parameters.length-1]),t=pu(n)?fu(n):n;return t&&So(t,1)}}function Wo(e,n,t){return qo(e,No(n,e.typeParameters,Ro(e.typeParameters),t))}function qo(n,t){var r=n.instantiations||(n.instantiations=e.createMap()),a=is(t),i=r.get(a);return i||r.set(a,i=zo(n,t)),i}function zo(e,n){return ec(e,Jo(e,n),!0)}function Jo(e,n){return Jl(e.typeParameters,n)}function Xo(e){return e.typeParameters?e.erasedSignatureCache||(e.erasedSignatureCache=function(e){return ec(e,Xl(e.typeParameters),!0)}(e)):e}function Yo(n){return n.typeParameters?n.canonicalSignatureCache||(n.canonicalSignatureCache=function(n){return Wo(n,e.map(n.typeParameters,function(e){return e.target&&!no(e.target)?e.target:e}),e.isInJSFile(n.declaration))}(n)):n}function Qo(n){var t=n.typeParameters;if(t){var r=Xl(t);return ec(n,Jl(t,e.map(t,function(e){return lc(io(e),r)||Se})),!0)}return n}function Zo(n){if(!n.isolatedSignatureType){var t=155===n.declaration.kind||159===n.declaration.kind,r=Or(16);r.members=L,r.properties=e.emptyArray,r.callSignatures=t?e.emptyArray:[n],r.constructSignatures=t?[n]:e.emptyArray,n.isolatedSignatureType=r}return n.isolatedSignatureType}function $o(e){return e.members.get(\"__index\")}function es(n,t){var r=1===t?134:137,a=$o(n);if(a)for(var i=0,o=a.declarations;i<o.length;i++){var s=o[i],l=e.cast(s,e.isIndexSignatureDeclaration);if(1===l.parameters.length){var c=l.parameters[0];if(c.type&&c.type.kind===r)return l}}}function ns(e,n,t){return{type:e,isReadonly:n,declaration:t}}function ts(n,t){var r=es(n,t);if(r)return ns(r.type?Ul(r.type):$,e.hasModifier(r,64),r)}function rs(n){if(!n.constraint)if(n.target){var t=no(n.target);n.constraint=t?lc(t,n.mapper):De}else{var r=(i=(a=n).symbol&&e.getDeclarationOfKind(a.symbol,148))&&e.getEffectiveConstraintOfTypeParameter(i);n.constraint=r?Ul(r):function(n){var t;if(n.symbol)for(var r=0,a=n.symbol.declarations;r<a.length;r++){var i=a[r];if(174===i.parent.kind){var o=i.parent.parent;if(162===o.kind){var s=o,l=Fg(s);if(l){var c=s.typeArguments.indexOf(i.parent);if(c<l.length){var u=no(l[c]);if(u){var d=lc(u,Jl(l,wg(s,l)));d!==n&&(t=e.append(t,d))}}}}else 149===o.kind&&o.dotDotDotToken&&(t=e.append(t,Ws(re)))}}return t&&ll(t)}(n)||De}var a,i;return n.constraint===De?void 0:n.constraint}function as(n){var t=e.getDeclarationOfKind(n.symbol,148),r=e.isJSDocTemplateTag(t.parent)?e.getHostSignatureFromJSDoc(t.parent):t.parent;return r&&Er(r)}function is(e){var n=\"\";if(e)for(var t=e.length,r=0;r<t;){for(var a=e[r].id,i=1;r+i<t&&e[r+i].id===a+i;)i++;n.length&&(n+=\",\"),n+=a,i>1&&(n+=\":\"+i),r+=i}return n}function os(e,n){for(var t=0,r=0,a=e;r<a.length;r++){var i=a[r];i.flags&n||(t|=i.flags)}return 939524096&t}function ss(e,n){var t=is(n),r=e.instantiations.get(t);return r||(r=Or(4,e.symbol),e.instantiations.set(t,r),r.flags|=n?os(n,0):0,r.target=e,r.typeArguments=n),r}function ls(e){var n=Dr(e.flags);return n.symbol=e.symbol,n.objectFlags=e.objectFlags,n.target=e.target,n.typeArguments=e.typeArguments,n}function cs(n){return e.length(n.target.typeParameters)}function us(n,t,r){var a=si(br(t)),i=a.localTypeParameters;if(i){var o=e.length(n.typeArguments),s=Ro(i),l=e.isInJSFile(n);if((R||!l)&&(o<s||o>i.length)){var c=l&&293!==n.parent.kind;if(bt(n,s===i.length?c?e.Diagnostics.Expected_0_type_arguments_provide_these_with_an_extends_tag:e.Diagnostics.Generic_type_0_requires_1_type_argument_s:c?e.Diagnostics.Expected_0_1_type_arguments_provide_these_with_an_extends_tag:e.Diagnostics.Generic_type_0_requires_between_1_and_2_type_arguments,Yr(a,void 0,2),s,i.length),!l)return te}return ss(a,e.concatenate(a.outerTypeParameters,No(r,i,s,l)))}return Es(n,t)?a:te}function ds(n,t){var r=si(n),a=It(n),i=a.typeParameters,o=is(t),s=a.instantiations.get(o);return s||a.instantiations.set(o,s=lc(r,Jl(i,No(t,i,Ro(i),e.isInJSFile(n.valueDeclaration))))),s}function ms(n){switch(n.kind){case 162:return n.typeName;case 209:var t=n.expression;if(e.isEntityNameExpression(t))return t}}function ps(e,n){return e&&ir(e,n)||Q}function fs(n,t){var r=Ss(n);if(t===Q)return te;var a=_s(n,t,r);if(a)return a;var i=e.isInJSFile(n)&&t.valueDeclaration&&e.getJSDocEnumTag(t.valueDeclaration);if(i){var o=Rt(i);if(!ia(i,5))return te;var s=i.typeExpression?Ul(i.typeExpression):te;return la()||(s=te,bt(n,e.Diagnostics.Enum_type_0_circularly_references_itself,Jr(t))),o.resolvedEnumType=s}var l=li(t);if(l)return Es(n,t)?65536&l.flags?hs(l,n):Vl(l):te;if(!(67220415&t.flags&&bs(n)))return te;var c=gs(n,t,r);return c||(ps(ms(n),67897832),Ga(t))}function gs(n,t,r){if(!ia(t,6))return te;var a=yf(t),i=Ga(t),o=i.symbol&&i.symbol!==t&&!bf(i)&&_s(n,i.symbol,r);if(!la())return It(t).resolvedJSDocType=te,bt(n,e.Diagnostics.JSDoc_type_0_circularly_references_itself,Jr(t)),te;if(o||a){var s=o&&a?ll([a,o]):o||a;return It(t).resolvedJSDocType=s}}function _s(n,t,r){if(96&t.flags){if(t.valueDeclaration&&e.isBinaryExpression(t.valueDeclaration.parent)){var a=gs(n,t,r);if(a)return a}return us(n,t,r)}return 524288&t.flags?function(n,t,r){var a=si(t),i=It(t).typeParameters;if(i){var o=e.length(n.typeArguments),s=Ro(i);return o<s||o>i.length?(bt(n,s===i.length?e.Diagnostics.Generic_type_0_requires_1_type_argument_s:e.Diagnostics.Generic_type_0_requires_between_1_and_2_type_arguments,Jr(t),s,i.length),te):ds(t,r)}return Es(n,t)?a:te}(n,t,r):16&t.flags&&bs(n)&&(t.members||e.getJSDocClassTag(t.valueDeclaration))?hf(t):void 0}function vs(e){return 168===e.kind&&1===e.elementTypes.length}function ys(e,n,t){return vs(n)&&vs(t)?ys(e,n.elementTypes[0],t.elementTypes[0]):Tl(Ul(n))===e?Ul(t):void 0}function hs(n,t){for(var r;t&&!e.isStatement(t)&&289!==t.kind;){var a=t.parent;if(173===a.kind&&t===a.trueType){var i=ys(n,a.checkType,a.extendsType);i&&(r=e.append(r,i))}t=a}return r?function(e,n){var t=Dr(8388608);return t.typeVariable=e,t.substitute=n,t}(n,ll(e.append(r,n))):n}function bs(e){return!!(2097152&e.flags)&&(162===e.kind||181===e.kind)}function Es(n,t){return!n.typeArguments||(bt(n,e.Diagnostics.Type_0_is_not_generic,t?Jr(t):n.typeName?e.declarationNameToString(n.typeName):\"(anonymous)\"),!1)}function Ts(n){var t=Rt(n);if(!t.resolvedType){var r=void 0,a=void 0,i=67897832;bs(n)&&(a=function(n){if(e.isIdentifier(n.typeName)){var t=n.typeArguments;switch(n.typeName.escapedText){case\"String\":return Es(n),le;case\"Number\":return Es(n),ce;case\"Boolean\":return Es(n),fe;case\"Void\":return Es(n),_e;case\"Undefined\":return Es(n),ae;case\"Null\":return Es(n),oe;case\"Function\":case\"function\":return Es(n),Fe;case\"Array\":case\"array\":return t&&t.length?void 0:We;case\"Promise\":case\"promise\":return t&&t.length?void 0:Bf($);case\"Object\":if(t&&2===t.length){if(e.isJSDocIndexSignature(n)){var r=Ul(t[0]),a=ns(Ul(t[1]),!1);return wr(void 0,L,e.emptyArray,e.emptyArray,r===le?a:void 0,r===ce?a:void 0)}return $}return Es(n),$}}}(n),i|=67220415),a||(a=fs(n,r=ps(ms(n),i))),t.resolvedSymbol=r,t.resolvedType=a}return t.resolvedType}function Ss(n){return e.map(n.typeArguments,Ul)}function Ls(e){var n=Rt(e);return n.resolvedType||(n.resolvedType=Vl(Mu(Lg(e.exprName)))),n.resolvedType}function xs(n,t){function r(e){for(var n=0,t=e.declarations;n<t.length;n++){var r=t[n];switch(r.kind){case 238:case 239:case 241:return r}}}if(!n)return t?Ae:Se;var a=si(n);return 131072&a.flags?e.length(a.typeParameters)!==t?(bt(r(n),e.Diagnostics.Global_type_0_must_have_1_type_parameter_s,e.symbolName(n),t),t?Ae:Se):a:(bt(r(n),e.Diagnostics.Global_type_0_must_be_a_class_or_interface_type,e.symbolName(n)),t?Ae:Se)}function As(n,t){return Cs(n,67220415,t?e.Diagnostics.Cannot_find_global_value_0:void 0)}function Cs(e,n,t){return Ft(void 0,e,n,t,e,!1)}function Ds(n,t,r){var a=function(n,t){return Cs(n,67897832,t?e.Diagnostics.Cannot_find_global_type_0:void 0)}(n,r);return a||r?xs(a,t):void 0}function ks(e){return Xe||(Xe=As(\"Symbol\",e))}function Ms(e){return Ye||(Ye=Ds(\"Symbol\",0,e))||Se}function Os(e){return Ze||(Ze=Ds(\"Promise\",1,e))||Ae}function Is(e){return $e||($e=As(\"Promise\",e))}function Rs(e){return an||(an=Ds(\"AsyncIterable\",1,e))||Ae}function Ns(e){return on||(on=Ds(\"AsyncIterator\",1,e))||Ae}function ws(e){return sn||(sn=Ds(\"AsyncIterableIterator\",1,e))||Ae}function Ps(e){return nn||(nn=Ds(\"Iterable\",1,e))||Ae}function Fs(e){return tn||(tn=Ds(\"Iterator\",1,e))||Ae}function Gs(e){return rn||(rn=Ds(\"IterableIterator\",1,e))||Ae}function Vs(e,n){void 0===n&&(n=0);var t=Cs(e,67897832,void 0);return t&&xs(t,n)}function Ks(e,n){return e!==Ae?ss(e,n):Se}function Bs(e){return Ks(Qe||(Qe=Ds(\"TypedPropertyDescriptor\",1,!0))||Ae,[e])}function Hs(e){return Ks(ws(!0),[e])}function Us(e){return Ks(Ps(!0),[e])}function js(e){return Ks(Gs(!0),[e])}function Ws(e){return Ks(Ge,[e])}function qs(e){return Ks(Ve,[e])}function zs(n,t,r,a){var i=n+(r?\"+\":\",\")+t+(a&&a.length?\",\"+a.join(\",\"):\"\"),o=j.get(i);return o||j.set(i,o=function(n,t,r,a){var i,o=[],s=r?n-1:n;if(n){i=new Array(n);for(var l=0;l<n;l++){var c=i[l]=Dr(65536);if(l<s){var u=Tt(4|(l>=t?16777216:0),\"\"+l);u.type=c,o.push(u)}}}var d=[];for(l=t;l<=s;l++)d.push(Kl(l));var m=Tt(4,\"length\");m.type=r?ce:tl(d),o.push(m);var p=Or(12);return p.typeParameters=i,p.outerTypeParameters=void 0,p.localTypeParameters=i,p.instantiations=e.createMap(),p.instantiations.set(is(p.typeParameters),p),p.target=p,p.typeArguments=p.typeParameters,p.thisType=Dr(65536),p.thisType.isThisType=!0,p.thisType.constraint=p,p.declaredProperties=o,p.declaredCallSignatures=e.emptyArray,p.declaredConstructSignatures=e.emptyArray,p.declaredStringIndexInfo=void 0,p.declaredNumberIndexInfo=void 0,p.minLength=t,p.hasRestElement=r,p.associatedNames=a,p}(n,t,r,a)),o}function Js(e,n,t,r){void 0===n&&(n=e.length),void 0===t&&(t=!1);var a=e.length;if(1===a&&t)return Ws(e[0]);var i=zs(a,n,a>0&&t,r);return e.length?ss(i,e):i}function Xs(e){return e.id}function Ys(n,t){return e.binarySearch(n,t,Xs,e.compareValues)>=0}function Qs(n,t){var r=e.binarySearch(n,t,Xs,e.compareValues);return r<0&&(n.splice(~r,0,t),!0)}function Zs(n,t,r){var a=r.flags;if(262144&a)return $s(n,t,r.types);if(!(32768&a||524288&a&&function(e){for(var n=0,t=0,r=e.types;t<r.length;t++){var a=r[t];if(27072&a.flags&&27072&n)return!0;if(24576&(n|=a.flags)&&16908288&n||16777216&n&&32252&n||68&n&&16809400&n||168&n&&16809300&n||3072&n&&16806396&n||12288&n&&16797180&n)return!0}return!1}(r)))if(t|=-939524097&a,3&a)r===ne&&(t|=268435456);else if(!M&&24576&a)134217728&a||(t|=134217728);else{var i=n.length,o=i&&r.id>n[i-1].id?~i:e.binarySearch(n,r,Xs,e.compareValues);o<0&&n.splice(~o,0,r)}return t}function $s(e,n,t){for(var r=0,a=t;r<a.length;r++)n=Zs(e,n,a[r]);return n}function el(n,t){for(var r=0,a=t;r<a.length;r++){var i=a[r];if(n!==i&&hc(n,i)&&(!(1&e.getObjectFlags(Ka(n)))||!(1&e.getObjectFlags(Ka(i)))||Ec(n,i)))return!0}return!1}function nl(n){if(0!==n.length&&!function(e){var n=e[0];if(512&n.flags){for(var t=Tr(n.symbol),r=1;r<e.length;r++){var a=e[r];if(!(512&a.flags)||t!==Tr(a.symbol))return!1}return!0}return!1}(n))for(var t=n.length;t>0;)el(n[--t],n)&&e.orderedRemoveItemAt(n,t)}function tl(n,t,r,a){if(void 0===t&&(t=1),0===n.length)return ve;if(1===n.length)return n[0];var i=[],o=$s(i,0,n);if(3&o)return 1&o?268435456&o?ne:$:re;switch(t){case 1:2240&o|256&&function(n,t){for(var r=n.length;r>0;){var a=n[--r];(64&a.flags&&4&t||128&a.flags&&8&t||2048&a.flags&&1024&t||448&a.flags&&33554432&a.flags&&Ys(n,a.regularType))&&e.orderedRemoveItemAt(n,r)}}(i,o);break;case 2:nl(i)}return 0===i.length?16384&o?134217728&o?oe:se:8192&o?134217728&o?ae:ie:ve:al(i,16748579&o?0:67108864,r,a)}function rl(n,t){return e.isIdentifierTypePredicate(n)?e.isIdentifierTypePredicate(t)&&n.parameterIndex===t.parameterIndex:!e.isIdentifierTypePredicate(t)}function al(e,n,t,r){if(0===e.length)return ve;if(1===e.length)return e[0];var a=is(e),i=W.get(a);return i||(i=Dr(262144|os(e,24576)|n),W.set(a,i),i.types=e,i.aliasSymbol=t,i.aliasTypeArguments=r),i}function il(n,t,r){var a=r.flags;return 524288&a?ol(n,t,r.types):(16&e.getObjectFlags(r)&&Nc(r)?t|=536870912:(t|=-939524097&a,3&a?r===ne&&(t|=268435456):!M&&24576&a||e.contains(n,r)||n.push(r)),t)}function ol(e,n,t){for(var r=0,a=t;r<a.length;r++)n=il(e,n,Vl(a[r]));return n}function sl(e,n){for(var t=0,r=e;t<r.length;t++){var a=r[t];if(!Ys(a.types,n)){var i=64&n.flags?le:128&n.flags?ce:2048&n.flags?ge:void 0;if(!i||!Ys(a.types,i))return!1}}return!0}function ll(n,t,r){var a=[],i=ol(a,0,n);if(32768&i)return ve;if(1&i)return 268435456&i?ne:$;if(!M&&24576&i)return 8192&i?ae:oe;if((4&i&&64&i||8&i&&128&i||1024&i&&2048&i)&&function(n,t){for(var r=n.length;r>0;){var a=n[--r];(4&a.flags&&64&t||8&a.flags&&128&t||1024&a.flags&&2048&t)&&e.orderedRemoveItemAt(n,r)}}(a,i),536870912&i&&!(131072&i)&&a.push(Se),0===a.length)return re;if(1===a.length)return a[0];if(262144&i){if(67108864&i&&function(n){for(var t,r=e.findIndex(n,function(e){return 0!=(67108864&e.flags)}),a=r+1;a<n.length;)67108864&(d=n[a]).flags?((t||(t=[n[r]])).push(d),e.orderedRemoveItemAt(n,a)):a++;if(!t)return!1;for(var i=[],o=[],s=0,l=t;s<l.length;s++)for(var c=0,u=l[s].types;c<u.length;c++){var d;Qs(i,d=u[c])&&sl(t,d)&&Qs(o,d)}return n[r]=al(o,67108864),!0}(a))return ll(a,t,r);var o=e.findIndex(a,function(e){return 0!=(262144&e.flags)}),s=a[o];return tl(e.map(s.types,function(n){return ll(e.replaceElement(a,o,n))}),1,t,r)}var l=is(a),c=q.get(l);return c||(c=Dr(524288|os(a,24576)),q.set(l,c),c.types=a,c.aliasSymbol=t,c.aliasTypeArguments=r),c}function cl(e,n){var t=Dr(1048576);return t.type=e,t.stringsOnly=n,t}function ul(n,t){if(!(24&e.getDeclarationModifierFlagsFromSymbol(n))){var r=xi(n).nameType;if(!r&&!e.isKnownSymbol(n)){var a=n.valueDeclaration&&e.getNameOfDeclaration(n.valueDeclaration);r=a&&e.isNumericLiteral(a)?Kl(+a.text):a&&147===a.kind&&e.isNumericLiteral(a.expression)?Kl(+a.expression.text):Kl(e.symbolName(n))}if(r&&r.flags&t)return r}return ve}function dl(n,t){return tl(e.map($i(n),function(e){return ul(e,t)}))}function ml(n,t){return void 0===t&&(t=w),262144&n.flags?ll(e.map(n.types,function(e){return ml(e,t)})):524288&n.flags?tl(e.map(n.types,function(e){return ml(e,t)})):tg(n,14745600)?function(e,n){return n?e.resolvedStringIndexType||(e.resolvedStringIndexType=cl(e,!0)):e.resolvedIndexType||(e.resolvedIndexType=cl(e,!1))}(n,t):32&e.getObjectFlags(n)?Ki(n):n===ne?ne:1&n.flags?Te:t?To(n,0)?le:dl(n,64):To(n,0)?tl([le,ce,dl(n,2048)]):function(e){var n=To(e,1);return n!==Tn?n:void 0}(n)?tl([ce,dl(n,2112)]):dl(n,2240)}function pl(n){if(w)return n;var t=un||(un=Cs(\"Extract\",524288,e.Diagnostics.Cannot_find_global_type_0));return t?ds(t,[n,le]):le}function fl(n){return!R&&(!!(16384&e.getObjectFlags(n))||(262144&n.flags?e.every(n.types,fl):524288&n.flags?e.some(n.types,fl):!!(15794176&n.flags)&&fl(lo(n))))}function gl(n,t,r,a,i){var o=r&&188===r.kind?r:void 0,s=_i(t)?Ei(t):o&&Rp(o.argumentExpression,t,!1)?e.getPropertyNameForKnownSymbolName(e.idText(o.argumentExpression.name)):void 0;if(void 0!==s){var l=vo(n,s);if(l){if(o){if(kp(l,o,99===o.expression.kind),e.isAssignmentTarget(o)&&($f(o,l)||eg(o)))return bt(o.argumentExpression,e.Diagnostics.Cannot_assign_to_0_because_it_is_a_constant_or_a_read_only_property,Jr(l)),i;a&&(Rt(r).resolvedSymbol=l)}var c=Ga(l);return o?jd(o,c):c}if(pu(n)){var u=fu(n);if(u&&Km(s)&&+s>=0)return u}}if(!(24576&t.flags)&&rg(t,3308)){if(32769&n.flags)return n;var d=rg(t,168)&&To(n,1)||To(n,0)||void 0;if(d)return r&&!rg(t,12)?bt(p=188===r.kind?r.argumentExpression:r.indexType,e.Diagnostics.Type_0_cannot_be_used_as_an_index_type,Yr(t)):o&&d.isReadonly&&(e.isAssignmentTarget(o)||e.isDeleteTarget(o))&&bt(o,e.Diagnostics.Index_signature_in_type_0_only_permits_reading,Yr(n)),d.type;if(32768&t.flags)return ve;if(fl(n))return $;if(o&&!ig(n)){if(R&&!A.suppressImplicitAnyIndexErrors)if(void 0!==s&&Sp(s,n))bt(o,e.Diagnostics.Property_0_is_a_static_member_of_type_1,s,Yr(n));else if(So(n,1))bt(o.argumentExpression,e.Diagnostics.Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number);else{var m=void 0;void 0!==s&&(m=xp(s,n))?void 0!==m&&bt(o.argumentExpression,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2,s,Yr(n),m):bt(o,e.Diagnostics.Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature,Yr(n))}return i}}if(fl(n))return $;if(r){var p=188===r.kind?r.argumentExpression:r.indexType;192&t.flags?bt(p,e.Diagnostics.Property_0_does_not_exist_on_type_1,\"\"+t.value,Yr(n)):12&t.flags?bt(p,e.Diagnostics.Type_0_has_no_matching_index_signature_for_type_1,Yr(n),Yr(t)):bt(p,e.Diagnostics.Type_0_cannot_be_used_as_an_index_type,Yr(t))}return da(t)?t:i}function _l(e){return tg(e,15794176)}function vl(n){return 2097152&n.flags?function(n){if(n.simplified)return n.simplified===ke?n:n.simplified;n.simplified=ke;var t=vl(n.objectType),r=vl(n.indexType);if(262144&r.flags)return n.simplified=Md(r,function(e){return vl(hl(t,e))});if(!(15794176&r.flags)){if(262144&t.flags)return n.simplified=Md(t,function(e){return vl(hl(e,r))});if(524288&t.flags)return n.simplified=ll(e.map(t.types,function(e){return vl(hl(e,r))}))}if(Ji(t))return n.simplified=yl(t,n);if(65536&t.flags){var a=no(t);if(a&&Ji(a))return n.simplified=yl(a,n)}return n.simplified=n}(n):n}function yl(e,n){var t=Jl([Vi(e)],[n.indexType]),r=Yl(e.mapper,t);return lc(Bi(e),r)}function hl(e,n,t,r){if(void 0===r&&(r=t?te:re),e===ne||n===ne)return ne;if(_l(n)||(!t||188!==t.kind)&&function(e){return tg(e,148963328)}(e)){if(3&e.flags)return e;var a=e.id+\",\"+n.id,i=J.get(a);return i||J.set(a,i=function(e,n){var t=Dr(2097152);return t.objectType=e,t.indexType=n,t}(e,n)),i}var o=po(e);if(262144&n.flags&&!(16&n.flags)){for(var s=[],l=!1,c=0,u=n.types;c<u.length;c++){var d=gl(o,u[c],t,!1,r);if(d===r){if(!t)return r;l=!0}s.push(d)}return l?r:tl(s)}return gl(o,n,t,!0,r)}function bl(e){var n=Rt(e);if(!n.resolvedType){var t=Ul(e.objectType),r=Ul(e.indexType),a=hl(t,r,e);n.resolvedType=2097152&a.flags&&a.objectType===t&&a.indexType===r?hs(a,e):a}return n.resolvedType}function El(e){var n=Rt(e);if(!n.resolvedType){var t=Or(32,e.symbol);t.declaration=e,t.aliasSymbol=Ol(e),t.aliasTypeArguments=Il(t.aliasSymbol),n.resolvedType=t,Ki(t)}return n.resolvedType}function Tl(e){return 8388608&e.flags?e.typeVariable:e}function Sl(e,n){var t=lc(e.checkType,n),r=lc(e.extendsType,n);if(t===ne||r===ne)return ne;var a,i=e.isDistributive&&tg(t,15794176);if(e.inferTypeParameters){var o=wu(e.inferTypeParameters,void 0,0);i||Wu(o.inferences,t,r,96),a=Yl(n,o)}if(!i){if(3&r.flags)return lc(e.trueType,n);if(1&t.flags)return tl([lc(e.trueType,a||n),lc(e.falseType,n)]);var s=a?lc(e.extendsType,a):r;if(!bc(cc(t),cc(s)))return lc(e.falseType,n);if(Vc(t,s,mt,void 0))return lc(e.trueType,a||n)}var l=Tl(t),c=Dr(4194304);return c.root=e,c.checkType=l,c.extendsType=r,c.mapper=n,c.combinedMapper=a,c.aliasSymbol=e.aliasSymbol,c.aliasTypeArguments=Wl(e.aliasTypeArguments,n),c}function Ll(e){return e.resolvedTrueType||(e.resolvedTrueType=lc(e.root.trueType,e.mapper))}function xl(e){return e.resolvedFalseType||(e.resolvedFalseType=lc(e.root.falseType,e.mapper))}function Al(n){var t;return n.locals&&n.locals.forEach(function(n){262144&n.flags&&(t=e.append(t,si(n)))}),t}function Cl(n){var t=Rt(n);if(!t.resolvedType){var r=Ul(n.checkType),a=Ol(n),i=Il(a),o=Ua(n,!0),s=i?o:e.filter(o,function(e){return function(e,n){if(rc(e,n))return!0;for(;n;){if(173===n.kind&&rc(e,n.extendsType))return!0;n=n.parent}return!1}(e,n)}),l={node:n,checkType:r,extendsType:Ul(n.extendsType),trueType:Ul(n.trueType),falseType:Ul(n.falseType),isDistributive:!!(65536&r.flags),inferTypeParameters:Al(n),outerTypeParameters:s,instantiations:void 0,aliasSymbol:a,aliasTypeArguments:i};t.resolvedType=Sl(l,void 0),s&&(l.instantiations=e.createMap(),l.instantiations.set(is(s),t.resolvedType))}return t.resolvedType}function Dl(n){var t=Rt(n);if(!t.resolvedType){if(n.isTypeOf&&n.typeArguments)return bt(n,e.Diagnostics.Type_arguments_cannot_be_used_here),t.resolvedSymbol=Q,t.resolvedType=te;if(!e.isLiteralImportTypeNode(n))return bt(n.argument,e.Diagnostics.String_literal_expected),t.resolvedSymbol=Q,t.resolvedType=te;var r=n.isTypeOf?67220415:2097152&n.flags?68008959:67897832,a=sr(n,n.argument.literal);if(!a)return t.resolvedSymbol=Q,t.resolvedType=te;var i=dr(a,!1);if(e.nodeIsMissing(n.qualifier))i.flags&r?kl(n,t,i,r):(bt(n,67220415===r?e.Diagnostics.Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here:e.Diagnostics.Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0,n.argument.literal.text),t.resolvedSymbol=Q,t.resolvedType=te);else{for(var o=function n(t){return e.isIdentifier(t)?[t]:e.append(n(t.left),t.right)}(n.qualifier),s=i,l=void 0;l=o.shift();){var c=o.length?1920:r,u=wt(_r(br($t(s))),l.escapedText,c);if(!u)return bt(l,e.Diagnostics.Namespace_0_has_no_exported_member_1,ar(s),e.declarationNameToString(l)),t.resolvedType=te;Rt(l).resolvedSymbol=u,Rt(l.parent).resolvedSymbol=u,s=u}kl(n,t,s,r)}}return t.resolvedType}function kl(e,n,t,r){var a=$t(t);return n.resolvedSymbol=a,n.resolvedType=67220415===r?Ga(t):fs(e,a)}function Ml(n){var t=Rt(n);if(!t.resolvedType){var r=Ol(n);if(0!==Li(n.symbol).size||r){var a=Or(16,n.symbol);a.aliasSymbol=r,a.aliasTypeArguments=Il(r),e.isJSDocTypeLiteral(n)&&n.isArrayType&&(a=Ws(a)),t.resolvedType=a}else t.resolvedType=xe}return t.resolvedType}function Ol(n){return e.isTypeAlias(n.parent)?Er(n.parent):void 0}function Il(e){return e?Wa(e):void 0}function Rl(n,t,r,a,i){if(1&n.flags||1&t.flags)return $;if(2&n.flags||2&t.flags)return re;if(32768&n.flags)return t;if(32768&t.flags)return n;if(262144&n.flags)return Md(n,function(e){return Rl(e,t,r,a,i)});if(262144&t.flags)return Md(t,function(e){return Rl(n,e,r,a,i)});if(17826812&t.flags)return n;var o,s,l=e.createSymbolTable(),c=e.createUnderscoreEscapedMap();n===Se?(o=To(t,0),s=To(t,1)):(o=Pi(To(n,0),To(t,0)),s=Pi(To(n,1),To(t,1)));for(var u=0,d=$i(t);u<d.length;u++){var m=d[u];24&e.getDeclarationModifierFlagsFromSymbol(m)?c.set(m.escapedName,!0):Nl(m)&&l.set(m.escapedName,wl(m))}for(var p=0,f=$i(n);p<f.length;p++){var g=f[p];if(!c.has(g.escapedName)&&Nl(g))if(l.has(g.escapedName)){var _=Ga(m=l.get(g.escapedName));if(16777216&m.flags){var v=e.concatenate(g.declarations,m.declarations),y=Tt(4|16777216&g.flags,g.escapedName);y.type=tl([Ga(g),md(_,131072)]),y.leftSpread=g,y.rightSpread=m,y.declarations=v,y.nameType=g.nameType,l.set(g.escapedName,y)}}else l.set(g.escapedName,wl(g))}var h=wr(r,l,e.emptyArray,e.emptyArray,Pl(o),Pl(s));return h.flags|=268435456|a,h.objectFlags|=1152|i,h}function Nl(n){return 40960&n.flags?!n.declarations.some(function(n){return e.isClassLike(n.parent)}):!(65536&n.flags)}function wl(e){if(!Zf(e))return e;var n=Tt(4|16777216&e.flags,e.escapedName);return n.type=Ga(e),n.declarations=e.declarations,n.nameType=e.nameType,n.syntheticOrigin=e,n}function Pl(e){return e&&e.isReadonly?ns(e.type,!1,e.declaration):e}function Fl(e,n,t){var r=Dr(e);return r.symbol=t,r.value=n,r}function Gl(e){if(448&e.flags&&!(33554432&e.flags)){if(!e.freshType){var n=Fl(33554432|e.flags,e.value,e.symbol);n.regularType=e,e.freshType=n}return e.freshType}return e}function Vl(n){return 448&n.flags&&33554432&n.flags?n.regularType:262144&n.flags?tl(e.sameMap(n.types,Vl)):n}function Kl(e,n,t){var r=\"number\"==typeof e?\"#\":\"@\",a=n?n+r+e:r+e,i=z.get(a);if(!i){var o=(\"number\"==typeof e?128:64)|(n?512:0);z.set(a,i=Fl(o,e,t))}return i}function Bl(n){if(e.isValidESSymbolDeclaration(n)){var t=Er(n),r=It(t);return r.uniqueESSymbolType||(r.uniqueESSymbolType=function(e){var n=Dr(2048);return n.symbol=e,n}(t))}return ge}function Hl(n){var t=Rt(n);return t.resolvedType||(t.resolvedType=function(n){var t=e.getThisContainer(n,!1),r=t&&t.parent;return!r||!e.isClassLike(r)&&239!==r.kind||e.hasModifier(t,32)||155===t.kind&&!e.isNodeDescendantOf(n,t.body)?(bt(n,e.Diagnostics.A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface),te):ei(Er(r)).thisType}(n)),t.resolvedType}function Ul(n){switch(n.kind){case 119:case 282:case 283:return $;case 142:return re;case 137:return le;case 134:return ce;case 122:return fe;case 138:return ge;case 105:return _e;case 140:return ae;case 95:return oe;case 131:return ve;case 135:return 65536&n.flags?$:be;case 176:case 99:return Hl(n);case 180:return function(e){var n=Rt(e);return n.resolvedType||(n.resolvedType=Vl(Lg(e.literal))),n.resolvedType}(n);case 162:return Ts(n);case 161:return fe;case 209:return Ts(n);case 165:return Ls(n);case 167:return function(e){var n=Rt(e);return n.resolvedType||(n.resolvedType=Ws(Ul(e.elementType))),n.resolvedType}(n);case 168:return function(n){var t=Rt(n);if(!t.resolvedType){var r=e.lastOrUndefined(n.elementTypes),a=r&&170===r.kind?r:void 0,i=e.findLastIndex(n.elementTypes,function(e){return 169!==e.kind&&e!==a})+1,o=e.map(n.elementTypes,function(e){var n=Ul(e);return e===a&&So(n,1)||n});t.resolvedType=Js(o,i,!!a)}return t.resolvedType}(n);case 169:return function(e){var n=Ul(e.type);return M?Eu(n):n}(n);case 171:return function(n){var t=Rt(n);if(!t.resolvedType){var r=Ol(n);t.resolvedType=tl(e.map(n.types,Ul),1,r,Il(r))}return t.resolvedType}(n);case 172:return function(n){var t=Rt(n);if(!t.resolvedType){var r=Ol(n);t.resolvedType=ll(e.map(n.types,Ul),r,Il(r))}return t.resolvedType}(n);case 284:return function(e){var n=Ul(e.type);return M?bu(n,16384):n}(n);case 286:return ya(Ul(n.type));case 175:case 170:case 285:case 281:return Ul(n.type);case 288:return function(n){var t=Ul(n.type),r=n.parent,a=n.parent.parent;if(e.isJSDocTypeExpression(n.parent)&&e.isJSDocParameterTag(a)){var i=e.getHostSignatureFromJSDoc(a);if(i){var o=e.lastOrUndefined(i.parameters),s=e.getParameterSymbolFromJSDoc(a);if(!o||s&&o.symbol===s&&e.isRestParameter(o))return Ws(t)}}return e.isParameter(r)&&e.isJSDocFunctionType(r.parent)?Ws(t):ya(t)}(n);case 163:case 164:case 166:case 290:case 287:case 291:return Ml(n);case 177:return function(n){var t=Rt(n);if(!t.resolvedType)switch(n.operator){case 128:t.resolvedType=ml(Ul(n.type));break;case 141:t.resolvedType=138===n.type.kind?Bl(e.walkUpParenthesizedTypes(n.parent)):te}return t.resolvedType}(n);case 178:return bl(n);case 179:return El(n);case 173:return Cl(n);case 174:return function(e){var n=Rt(e);return n.resolvedType||(n.resolvedType=oi(Er(e.typeParameter))),n.resolvedType}(n);case 181:return Dl(n);case 71:case 146:var t=Sv(n);return t?si(t):te;default:return te}}function jl(e,n,t){if(e&&e.length)for(var r=0;r<e.length;r++){var a=e[r],i=t(a,n);if(a!==i){var o=0===r?[]:e.slice(0,r);for(o.push(i),r++;r<e.length;r++)o.push(t(e[r],n));return o}}return e}function Wl(e,n){return jl(e,n,lc)}function ql(e,n){return jl(e,n,ec)}function zl(e,n){return function(t){return t===e?n:t}}function Jl(n,t){return e.Debug.assert(void 0===t||n.length===t.length),1===n.length?zl(n[0],t?t[0]:$):2===n.length?(r=n[0],a=t?t[0]:$,i=n[1],o=t?t[1]:$,function(e){return e===r?a:e===i?o:e}):function(e,n){return function(t){for(var r=0;r<e.length;r++)if(t===e[r])return n?n[r]:$;return t}}(n,t);var r,a,i,o}function Xl(e){return Jl(e,void 0)}function Yl(e,n){return e?n?function(t){return lc(e(t),n)}:e:n}function Ql(e,n,t){return function(r){return r===e?n:t(r)}}function Zl(e){return 65536&e.flags?ne:e}function $l(e){var n=Dr(65536);return n.symbol=e.symbol,n.target=e,n}function ec(n,t,r){var a;if(n.typeParameters&&!r){a=e.map(n.typeParameters,$l),t=Yl(Jl(n.typeParameters,a),t);for(var i=0,o=a;i<o.length;i++)o[i].mapper=t}var s=Di(n.declaration,a,n.thisParameter&&nc(n.thisParameter,t),jl(n.parameters,t,nc),void 0,void 0,n.minArgumentCount,n.hasRestParameter,n.hasLiteralTypes);return s.target=n,s.mapper=t,s}function nc(n,t){var r=It(n);if(r.type&&!tg(r.type,15925248))return n;1&e.getCheckFlags(n)&&(n=r.target,t=Yl(r.mapper,t));var a=Tt(n.flags,n.escapedName,1|13312&e.getCheckFlags(n));return a.declarations=n.declarations,a.parent=n.parent,a.target=n,a.mapper=t,n.valueDeclaration&&(a.valueDeclaration=n.valueDeclaration),n.nameType&&(a.nameType=n.nameType),a}function tc(n,t){var r=64&n.objectFlags?n.target:n,a=r.symbol,i=It(a),o=i.outerTypeParameters;if(!o){var s=a.declarations[0];if(e.isInJSFile(s)){var l=e.findAncestor(s,e.isJSDocParameterTag);if(l){var c=e.getParameterSymbolFromJSDoc(l);c&&(s=c.valueDeclaration)}}var u=Ua(s,!0);if(gf(s)){var d=xo(s);u=e.addRange(u,d)}o=u||e.emptyArray,o=2048&a.flags&&!r.aliasTypeArguments?e.filter(o,function(e){return rc(e,s)}):o,i.outerTypeParameters=o,o.length&&(i.instantiations=e.createMap(),i.instantiations.set(is(o),r))}if(o.length){var m=64&n.objectFlags?Yl(n.mapper,t):t,p=e.map(o,m),f=is(p),g=i.instantiations.get(f);if(!g){var _=Jl(o,p);g=32&r.objectFlags?function(n,t){var r=ac(n);if(r){var a=lc(r,t);if(r!==a)return Md(a,function(a){if(15400963&a.flags&&a!==ne){var i=Ql(r,a,t);return eu(a)?Ws(ic(n,ce,!0,i)):nu(a)?qs(ic(n,ce,!0,i)):pu(a)?function(n,t,r){var a=n.target.minLength,i=e.map(n.typeArguments||e.emptyArray,function(e,n){return ic(t,Kl(\"\"+n),n>=a,r)}),o=Wi(t),s=4&o?0:8&o?cs(n)-(n.target.hasRestElement?1:0):a;return Js(i,s,n.target.hasRestElement,n.target.associatedNames)}(a,n,i):oc(n,i)}return a})}return oc(n,t)}(r,_):oc(r,_),i.instantiations.set(f,g)}return g}return n}function rc(n,t){if(n.symbol&&n.symbol.declarations&&1===n.symbol.declarations.length){var r=n.symbol.declarations[0].parent;if(e.findAncestor(t,function(e){return 216===e.kind?\"quit\":e===r}))return!!e.forEachChild(t,function t(r){switch(r.kind){case 176:return!!n.isThisType;case 71:return!n.isThisType&&e.isPartOfTypeNode(r)&&function(e){return!(146===e.kind||162===e.parent.kind&&e.parent.typeArguments&&e===e.parent.typeName)}(r)&&Ul(r)===n;case 165:return!0}return!!e.forEachChild(r,t)})}return!0}function ac(e){var n=Ki(e);if(1048576&n.flags){var t=n.type;if(65536&t.flags)return t}}function ic(e,n,t,r){var a=Yl(r,Jl([Vi(e)],[n])),i=lc(Bi(e.target||e),a),o=Wi(e);return M&&4&o&&!bc(ae,i)?Eu(i):M&&8&o&&t?md(i,131072):i}function oc(e,n){var t=Or(64|e.objectFlags,e.symbol);return 32&e.objectFlags&&(t.declaration=e.declaration),t.target=e,t.mapper=n,t.aliasSymbol=e.aliasSymbol,t.aliasTypeArguments=Wl(e.aliasTypeArguments,n),t}function sc(n,t){var r=n.root;if(r.outerTypeParameters){var a=e.map(r.outerTypeParameters,t),i=is(a),o=r.instantiations.get(i);return o||(o=function(e,n){if(e.isDistributive){var t=e.checkType,r=n(t);if(t!==r&&294912&r.flags)return Md(r,function(r){return Sl(e,Ql(t,r,n))})}return Sl(e,n)}(r,Jl(r.outerTypeParameters,a)),r.instantiations.set(i,o)),o}return n}function lc(e,n){if(!e||!n||n===x)return e;if(50===T)return te;T++;var t=function(e,n){var t=e.flags;if(65536&t)return n(e);if(131072&t){var r=e.objectFlags;if(16&r)return e.symbol&&14384&e.symbol.flags&&e.symbol.declarations?tc(e,n):e;if(32&r)return tc(e,n);if(4&r){var a=e.typeArguments,i=Wl(a,n);return i!==a?ss(e.target,i):e}return e}if(262144&t&&!(32764&t)){var o=e.types,s=Wl(o,n);return s!==o?tl(s,1,e.aliasSymbol,Wl(e.aliasTypeArguments,n)):e}if(524288&t){var o=e.types,s=Wl(o,n);return s!==o?ll(s,e.aliasSymbol,Wl(e.aliasTypeArguments,n)):e}return 1048576&t?ml(lc(e.type,n)):2097152&t?hl(lc(e.objectType,n),lc(e.indexType,n)):4194304&t?sc(e,Yl(e.mapper,n)):8388608&t?lc(e.typeVariable,n):e}(e,n);return T--,t}function cc(e){return 65535&e.flags?e:e.wildcardInstantiation||(e.wildcardInstantiation=lc(e,Zl))}function uc(e,n){return e&&ns(lc(e.type,n),e.isReadonly,e.declaration)}function dc(n){switch(e.Debug.assert(154!==n.kind||e.isObjectLiteralMethod(n)),n.kind){case 194:case 195:case 154:return mc(n);case 186:return e.some(n.properties,dc);case 185:return e.some(n.elements,dc);case 203:return dc(n.whenTrue)||dc(n.whenFalse);case 202:return 54===n.operatorToken.kind&&(dc(n.left)||dc(n.right));case 273:return dc(n.initializer);case 193:return dc(n.expression);case 266:return e.some(n.properties,dc);case 265:var t=n.initializer;return!!t&&dc(t);case 268:var r=n.expression;return!!r&&dc(r)}return!1}function mc(n){if(n.typeParameters)return!1;if(e.some(n.parameters,function(n){return!e.getEffectiveTypeAnnotationNode(n)}))return!0;if(195!==n.kind){var t=e.firstOrUndefined(n.parameters);if(!t||!e.parameterIsThisKeyword(t))return!0}return pc(n)}function pc(e){var n=e.body;return 216!==n.kind&&dc(n)}function fc(n){return(e.isInJSFile(n)&&e.isFunctionDeclaration(n)||Om(n)||e.isObjectLiteralMethod(n))&&mc(n)}function gc(n){if(131072&n.flags){var t=Xi(n);if(t.constructSignatures.length||t.callSignatures.length){var r=Or(16,n.symbol);return r.members=t.members,r.properties=t.properties,r.callSignatures=e.emptyArray,r.constructSignatures=e.emptyArray,r}}else if(524288&n.flags)return ll(e.map(n.types,gc));return n}function _c(e,n){return Fc(e,n,ft)}function vc(e,n){return Fc(e,n,ft)?-1:0}function yc(e,n){return Fc(e,n,dt)?-1:0}function hc(e,n){return Fc(e,n,ut)}function bc(e,n){return Fc(e,n,dt)}function Ec(n,t){return 262144&n.flags?e.every(n.types,function(e){return Ec(e,t)}):262144&t.flags?e.some(t.types,function(e){return Ec(n,e)}):14745600&n.flags?Ec(io(n)||Se,t):t===Pe||t===Fe?hc(n,t):Ba(n,Ka(t))}function Tc(e,n){return Fc(e,n,pt)}function Sc(e,n){return Tc(e,n)||Tc(n,e)}function Lc(e,n,t,r,a,i){return Vc(e,n,dt,t,r,a,i)}function xc(e,n,t,r,a,i){return Ac(e,n,dt,t,r,a,i)}function Ac(e,n,t,r,a,i,o){return!!Fc(e,n,t)||(!r||!Dc(a,e,n,t,i))&&Vc(e,n,t,r,i,o)}function Cc(n){return!!(4194304&n.flags||524288&n.flags&&e.some(n.types,Cc))}function Dc(n,t,r,a,i){if(!n||Cc(r))return!1;if(!Vc(t,r,a,void 0)&&function(n,t,r,a,i){for(var o=ho(t,0),s=ho(t,1),l=0,c=[s,o];l<c.length;l++){var u=c[l];if(e.some(u,function(e){var n=Ho(e);return!(32769&n.flags)&&Vc(n,r,a,void 0)})){var d={};Lc(t,r,n,i,void 0,d);var m=d.error;return ht(m,e.createDiagnosticForNode(n,u===s?e.Diagnostics.Did_you_mean_to_use_new_with_this_expression:e.Diagnostics.Did_you_mean_to_call_this_expression)),!0}}return!1}(n,t,r,a,i))return!0;switch(n.kind){case 268:case 193:return Dc(n.expression,t,r,a,i);case 202:switch(n.operatorToken.kind){case 58:case 26:return Dc(n.right,t,r,a,i)}break;case 186:return function(n,t,r,a){return kc(function(n){var t,r,a,i;return hn(this,function(o){switch(o.label){case 0:if(!e.length(n.properties))return[2];t=0,r=n.properties,o.label=1;case 1:if(!(t<r.length))return[3,8];if(a=r[t],e.isSpreadAssignment(a))return[3,7];if(!(i=ul(Er(a),2240))||32768&i.flags)return[3,7];switch(a.kind){case 157:case 156:case 154:case 274:return[3,2];case 273:return[3,4]}return[3,6];case 2:return[4,{errorNode:a.name,innerExpression:void 0,nameType:i}];case 3:return o.sent(),[3,7];case 4:return[4,{errorNode:a.name,innerExpression:a.initializer,nameType:i,errorMessage:pa(a.name)?e.Diagnostics.Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1:void 0}];case 5:return o.sent(),[3,7];case 6:e.Debug.assertNever(a),o.label=7;case 7:return t++,[3,1];case 8:return[2]}})}(n),t,r,a)}(n,t,r,a);case 185:return function(e,n,t,r){if(au(n))return kc(Mc(e,t),n,t,r);var a=Pm(e,3,!0);return!!au(a)&&kc(Mc(e,t),a,t,r)}(n,t,r,a);case 266:return function(n,t,r,a){return kc(function(n){var t,r,a;return hn(this,function(i){switch(i.label){case 0:if(!e.length(n.properties))return[2];t=0,r=n.properties,i.label=1;case 1:return t<r.length?(a=r[t],e.isJsxSpreadAttribute(a)?[3,3]:[4,{errorNode:a.name,innerExpression:a.initializer,nameType:Kl(e.idText(a.name))}]):[3,4];case 2:i.sent(),i.label=3;case 3:return t++,[3,1];case 4:return[2]}})}(n),t,r,a)}(n,t,r,a);case 195:return function(n,t,r,a){if(e.isBlock(n.body))return!1;if(e.some(n.parameters,e.hasType))return!1;var i=Bp(t);if(!i)return!1;var o=ho(r,0);if(!e.length(o))return!1;var s=n.body,l=Ho(i),c=tl(e.map(o,Ho));if(!Vc(l,c,a,void 0)){var u=s&&Dc(s,l,c,a,void 0);if(u)return u;var d={};if(Vc(l,c,a,s,void 0,void 0,d),d.error)return r.symbol&&e.length(r.symbol.declarations)&&ht(d.error,e.createDiagnosticForNode(r.symbol.declarations[0],e.Diagnostics.The_expected_type_comes_from_the_return_type_of_this_signature)),!0}return!1}(n,t,r,a)}return!1}function kc(n,t,r,a){for(var i=!1,o=n.next();!o.done;o=n.next()){var s=o.value,l=s.errorNode,c=s.innerExpression,u=s.nameType,d=s.errorMessage,m=hl(t,u,void 0,te),p=hl(r,u,void 0,te);if(m!==te&&p!==te&&!Vc(m,p,a,void 0))if(c&&Dc(c,m,p,a,void 0))i=!0;else{var f={},g=c?yg(c,0,m):m;if(Vc(g,p,a,l,d,void 0,f)&&g!==m&&Vc(m,p,a,l,d,void 0,f),f.error){var _=f.error,v=_i(u)?Ei(u):void 0,y=void 0!==v?vo(r,v):void 0,h=!1;if(!y){var b=rg(u,168)&&To(r,1)||To(r,0)||void 0;b&&b.declaration&&!e.getSourceFileOfNode(b.declaration).hasNoDefaultLib&&(h=!0,ht(_,e.createDiagnosticForNode(b.declaration,e.Diagnostics.The_expected_type_comes_from_this_index_signature)))}if(!h&&(y&&e.length(y.declarations)||r.symbol&&e.length(r.symbol.declarations))){var E=y&&e.length(y.declarations)?y.declarations[0]:r.symbol.declarations[0];e.getSourceFileOfNode(E).hasNoDefaultLib||ht(_,e.createDiagnosticForNode(E,e.Diagnostics.The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1,!v||2048&u.flags?Yr(u):e.unescapeLeadingUnderscores(v),Yr(r)))}}i=!0}}return i}function Mc(n,t){var r,a,i,o;return hn(this,function(s){switch(s.label){case 0:if(!(r=e.length(n.elements)))return[2];a=0,s.label=1;case 1:return a<r?au(t)&&!vo(t,\"\"+a)?[3,3]:(i=n.elements[a],e.isOmittedExpression(i)?[3,3]:(o=Kl(a),[4,{errorNode:i,innerExpression:i,nameType:o}])):[3,4];case 2:s.sent(),s.label=3;case 3:return a++,[3,1];case 4:return[2]}})}function Oc(e,n,t,r,a){return Vc(e,n,pt,t,r,a)}function Ic(n,t,r,a,i,o,s){if(n===t)return-1;var l=Rf(t);if(!wf(t)&&Nf(n)>l)return 0;n.typeParameters&&n.typeParameters!==t.typeParameters&&(n=Hp(n,t=Yo(t),void 0,s));var c=Rf(n),u=Ff(n),d=Ff(t);if(u&&d&&c!==l)return 0;var m=t.declaration?t.declaration.kind:0,p=!r&&O&&154!==m&&153!==m&&155!==m,f=-1,g=Vo(n);if(g&&g!==_e){var _=Vo(t);if(_){if(!(b=!p&&s(g,_,!1)||s(_,g,i)))return i&&o(e.Diagnostics.The_this_types_of_each_signature_are_incompatible),0;f&=b}}for(var v=u||d?Math.min(c,l):Math.max(c,l),y=u||d?v-1:-1,h=0;h<v;h++){var b,E=h===y?If(n,h):Mf(n,h),T=h===y?If(t,h):Mf(t,h),S=r?void 0:Bp(Tu(E)),L=r?void 0:Bp(Tu(T));if(!(b=S&&L&&!Ko(S)&&!Ko(L)&&(24576&vu(E))==(24576&vu(T))?Ic(L,S,p?2:1,!1,i,o,s):!r&&!p&&s(E,T,!1)||s(T,E,i)))return i&&o(e.Diagnostics.Types_of_parameters_0_and_1_are_incompatible,e.unescapeLeadingUnderscores(kf(n,h)),e.unescapeLeadingUnderscores(kf(t,h))),0;f&=b}if(!a){var x=t.declaration&&gf(t.declaration)?vf(t.declaration.symbol):Ho(t);if(x===_e)return f;var A=n.declaration&&gf(n.declaration)?vf(n.declaration.symbol):Ho(n),C=Bo(t);if(C){var D=Bo(n);if(D)f&=function(n,t,r,a,i,o,s){if(n.kind!==t.kind)return i&&(o(e.Diagnostics.A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard),o(e.Diagnostics.Type_predicate_0_is_not_assignable_to_1,Zr(n),Zr(t))),0;if(1===n.kind){var l=t,c=n.parameterIndex-(e.getThisParameter(r)?1:0),u=l.parameterIndex-(e.getThisParameter(a)?1:0);if(c!==u)return i&&(o(e.Diagnostics.Parameter_0_is_not_in_the_same_position_as_parameter_1,n.parameterName,l.parameterName),o(e.Diagnostics.Type_predicate_0_is_not_assignable_to_1,Zr(n),Zr(t))),0}var d=s(n.type,t.type,i);return 0===d&&i&&o(e.Diagnostics.Type_predicate_0_is_not_assignable_to_1,Zr(n),Zr(t)),d}(D,C,n.declaration,t.declaration,i,o,s);else if(e.isIdentifierTypePredicate(C))return i&&o(e.Diagnostics.Signature_0_must_be_a_type_predicate,Xr(n)),0}else f&=1===r&&s(x,A,!1)||s(A,x,i)}return f}function Rc(e,n){var t=Xo(e),r=Xo(n),a=Ho(t),i=Ho(r);return!(i!==_e&&!Fc(i,a,dt)&&!Fc(a,i,dt))&&0!==Ic(t,r,0,!0,!1,void 0,yc)}function Nc(n){return 131072&n.flags?0===(t=Xi(n)).properties.length&&0===t.callSignatures.length&&0===t.constructSignatures.length&&!t.stringIndexInfo&&!t.numberIndexInfo:!!(16777216&n.flags)||(262144&n.flags?e.some(n.types,Nc):!!(524288&n.flags)&&e.every(n.types,Nc));var t}function wc(n,t,r){if(n===t)return!0;var a=l(n)+\",\"+l(t),i=gt.get(a);if(void 0!==i&&(2!==i||!r))return 1===i;if(!(n.escapedName===t.escapedName&&256&n.flags&&256&t.flags))return gt.set(a,3),!1;for(var o=Ga(t),s=0,c=$i(Ga(n));s<c.length;s++){var u=c[s];if(8&u.flags){var d=vo(o,u.escapedName);if(!(d&&8&d.flags))return r?(r(e.Diagnostics.Property_0_is_missing_in_type_1,e.symbolName(u),Yr(si(t),void 0,64)),gt.set(a,3)):gt.set(a,2),!1}}return gt.set(a,1),!0}function Pc(e,n,t,r){var a=e.flags,i=n.flags;if(3&i||32768&a||e===ne)return!0;if(32768&i)return!1;if(68&a&&4&i)return!0;if(64&a&&512&a&&64&i&&!(512&i)&&e.value===n.value)return!0;if(168&a&&8&i)return!0;if(128&a&&512&a&&128&i&&!(512&i)&&e.value===n.value)return!0;if(272&a&&16&i)return!0;if(3072&a&&1024&i)return!0;if(32&a&&32&i&&wc(e.symbol,n.symbol,r))return!0;if(512&a&&512&i){if(262144&a&&262144&i&&wc(e.symbol,n.symbol,r))return!0;if(448&a&&448&i&&e.value===n.value&&wc(Tr(e.symbol),Tr(n.symbol),r))return!0}if(8192&a&&(!M||12288&i))return!0;if(16384&a&&(!M||16384&i))return!0;if(131072&a&&16777216&i)return!0;if(2048&a||2048&i)return!1;if(t===dt||t===mt||t===pt){if(1&a)return!0;if(136&a&&!(512&a)&&(32&i||128&i&&512&i))return!0}return!1}function Fc(e,n,t){if(448&e.flags&&33554432&e.flags&&(e=e.regularType),448&n.flags&&33554432&n.flags&&(n=n.regularType),e===n||t===pt&&!(32768&n.flags)&&Pc(n,e,t)||t!==ft&&Pc(e,n,t))return!0;if(131072&e.flags&&131072&n.flags){var r=t.get(qc(e,n,t));if(void 0!==r)return 1===r}return!!(16711680&e.flags||16711680&n.flags)&&Vc(e,n,t,void 0)}function Gc(n,t,r){return 4096&e.getObjectFlags(n)&&!(Wm(t.escapedName)||r)}function Vc(t,r,a,i,o,s,l){var c,u,d,m,p=0,f=0,g=0,_=!1,v=!1;e.Debug.assert(a!==ft||!i,\"no error reporting in identity checking\");var y=C(t,r,!!i,o);if(_)bt(i,e.Diagnostics.Excessive_stack_depth_comparing_types_0_and_1,Yr(t),Yr(r));else if(c){if(s){var h=s();h&&(c=e.concatenateDiagnosticMessageChains(h,c))}var b=void 0;if(o&&i&&!y&&t.symbol){var E=It(t.symbol);if(E.originatingImport&&!e.isImportCall(E.originatingImport)&&Vc(Ga(E.target),r,a,void 0)){var T=e.createDiagnosticForNode(E.originatingImport,e.Diagnostics.Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead);b=e.append(b,T)}}var S=e.createDiagnosticForNodeFromMessageChain(i,c,b);l&&(l.error=S),Xn.add(S)}return 0!==y;function L(n,t,r,a){e.Debug.assert(!!i),c=e.chainDiagnosticMessages(c,n,t,r,a)}function x(n,t,r){var i=Yr(t),o=Yr(r);i===o&&(i=Yr(t,void 0,64),o=Yr(r,void 0,64)),n||(n=a===pt?e.Diagnostics.Type_0_is_not_comparable_to_type_1:i===o?e.Diagnostics.Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated:e.Diagnostics.Type_0_is_not_assignable_to_type_1),L(n,i,o)}function C(t,r,o,s){if(void 0===o&&(o=!1),448&t.flags&&33554432&t.flags&&(t=t.regularType),448&r.flags&&33554432&r.flags&&(r=r.regularType),8388608&t.flags&&(t=a===mt?t.typeVariable:t.substitute),8388608&r.flags&&(r=r.typeVariable),2097152&t.flags&&(t=vl(t)),2097152&r.flags&&(r=vl(r)),262144&r.flags&&131072&t.flags&&r.types.length<=3&&tg(r,24576)){var l=Od(r,-24577);294912&l.flags||(r=l)}if(t===r)return-1;if(a===ft)return function(e,n){var t,r=e.flags&n.flags;return 131072&r||2097152&r||4194304&r||1048576&r||8388608&r?I(e,n,!1):786432&r&&(t=D(e,n))&&(t&=D(n,e))?t:0}(t,r);if(a===pt&&!(32768&r.flags)&&Pc(r,t,a)||Pc(t,r,a,o?L:void 0))return-1;if(Ju(t)&&33554432&t.flags){var u=262144&r.flags?M(t,r):void 0;if(function n(t,r,o,s){if(!R&&16384&e.getObjectFlags(r))return!1;if(tg(r,131072)&&!(512&e.getObjectFlags(r))){var l=!!(4096&e.getObjectFlags(t));if((a===dt||a===mt||a===pt)&&(Cd(Pe,r)||!l&&Nc(r)))return!1;if(o)return n(t,o,void 0,s);for(var c=function(n){if(!function(e,n){return e.valueDeclaration&&n.valueDeclaration&&e.valueDeclaration.parent!==n.valueDeclaration}(n,t.symbol)&&!mp(r,n.escapedName,l)){if(s){if(!i)return{value:e.Debug.fail()};if(e.isJsxAttributes(i)||e.isJsxOpeningLikeElement(i))L(e.Diagnostics.Property_0_does_not_exist_on_type_1,Jr(n),Yr(r));else{var a=t.symbol&&e.firstOrUndefined(t.symbol.declarations),o=void 0;if(n.valueDeclaration&&e.findAncestor(n.valueDeclaration,function(e){return e===a})){var c=n.valueDeclaration;e.Debug.assertNode(c,e.isObjectLiteralElementLike),i=c;var u=c.name;e.isIdentifier(u)&&(o=xp(u,r))}void 0!==o?L(e.Diagnostics.Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_write_2,Jr(n),Yr(r),o):L(e.Diagnostics.Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1,Jr(n),Yr(r))}}return{value:!0}}},u=0,d=Yi(t);u<d.length;u++){var m=d[u],p=c(m);if(\"object\"==typeof p)return p.value}}return!1}(t,r,u,o))return o&&x(s,t,r),0;(function(e){if(!(786432&e.flags))return!1;for(var n=!1,t=0,r=e.types;t<r.length;t++)if(!(24576&r[t].flags)){if(n)return!0;n=!0}return!1})(r)&&!u&&(t=xu(t))}if(a!==pt&&!v&&688124&t.flags&&t!==Pe&&655360&r.flags&&Kc(r)&&($i(t).length>0||Dv(t))&&!function(n,t){for(var r=!!(4096&e.getObjectFlags(n)),a=0,i=$i(n);a<i.length;a++){var o=i[a];if(mp(t,o.escapedName,r))return!0}return!1}(t,r)){if(o){var d=ho(t,0),m=ho(t,1);d.length>0&&C(Ho(d[0]),r,!1)||m.length>0&&C(Ho(m[0]),r,!1)?L(e.Diagnostics.Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it,Yr(t),Yr(r)):L(e.Diagnostics.Type_0_has_no_properties_in_common_with_type_1,Yr(t),Yr(r))}return 0}var p=0,f=c,g=v;if(v=!1,262144&t.flags?p=a===pt?O(t,r,o&&!(32764&t.flags)):function(e,n,t){for(var r=-1,a=e.types,i=0,o=a;i<o.length;i++){var s=o[i],l=C(s,n,t);if(!l)return 0;r&=l}return r}(t,r,o&&!(32764&t.flags)):(262144&r.flags?p=k(t,r,o&&!(32764&t.flags)&&!(32764&r.flags)):524288&r.flags?(v=!0,p=function(e,n,t){for(var r=-1,a=n.types,i=0,o=a;i<o.length;i++){var s=o[i],l=C(e,s,t);if(!l)return 0;r&=l}return r}(t,r,o)):524288&t.flags&&(p=O(t,r,!1)),!p&&(16711680&t.flags||16711680&r.flags)&&(p=I(t,r,o))&&(c=f)),!p&&524288&t.flags){var _=function(n,t){for(var r,a=!1,i=0,o=n.types;i<o.length;i++)if(15794176&(u=o[i]).flags){for(var s=eo(u);s&&5308416&s.flags;)s=eo(s);if(s){if(!(262144&s.flags))return;r=e.append(r,s)}}else 16809468&u.flags&&(a=!0);if(r&&(t||a)){if(a)for(var l=0,c=n.types;l<c.length;l++){var u;16809468&(u=c[l]).flags&&(r=e.append(r,u))}return ll(r)}}(t,!!(262144&r.flags));_&&(p=C(_,r,o))&&(c=f)}if(v=g,!p&&o){if(131072&t.flags&&32764&r.flags)!function(n,t){var r=Yr(n),a=Yr(t);(Ke===n&&le===t||Be===n&&ce===t||He===n&&fe===t||Ms(!1)===n&&ge===t)&&L(e.Diagnostics._0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible,a,r)}(t,r);else if(t.symbol&&131072&t.flags&&Pe===t)L(e.Diagnostics.The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead);else if(4096&e.getObjectFlags(t)&&524288&r.flags){var y=r.types,h=Ym(n.IntrinsicAttributes,i),b=Ym(n.IntrinsicClassAttributes,i);if(h!==te&&b!==te&&(e.contains(y,h)||e.contains(y,b)))return p}x(s,t,r)}return p}function D(e,n){for(var t=-1,r=0,a=e.types;r<a.length;r++){var i=k(a[r],n,!1);if(!i)return 0;t&=i}return t}function k(n,t,r){var a=t.types;if(262144&t.flags&&Ys(a,n))return-1;for(var i=0,o=a;i<o.length;i++){var s=C(n,o[i],!1);if(s)return s}return r&&C(n,M(n,t)||function(n,t){var r=e.getObjectFlags(n);if(20&r&&262144&t.flags)return e.find(t.types,function(t){if(131072&t.flags){var a=r&e.getObjectFlags(t);if(4&a)return n.target===t.target;if(16&a)return!!n.aliasSymbol&&n.aliasSymbol===t.aliasSymbol}return!1})}(n,t)||function(n,t){if(128&e.getObjectFlags(n)&&Dd(t,tu))return e.find(t.types,function(e){return!tu(e)})}(n,t)||function(n,t){var r=0;if(ho(n,r).length>0||ho(n,r=1).length>0)return e.find(t.types,function(e){return ho(e,r).length>0})}(n,t)||a[a.length-1],!0),0}function M(e,n){var t,r=Yi(e);if(r){var a=function(e,n){for(var t,r=0,a=e;r<a.length;r++){var i=a[r];if(ld(n,i.escapedName)){if(t){t.push(i);continue}t=[i]}}return t}(r,n);if(a)for(var i=0,o=a;i<o.length;i++)for(var s=o[i],l=Ga(s),c=0,u=n.types;c<u.length;c++){var d=u[c],m=ua(d,s.escapedName);if(m&&C(l,m)){if(d===t)continue;if(t)return;t=d}}}return t}function O(e,n,t){var r=e.types;if(262144&e.flags&&Ys(r,n))return-1;for(var a=r.length,i=0;i<a;i++){var o=C(r[i],n,t&&i===a-1);if(o)return o}return 0}function I(n,t,r){if(_)return 0;var i=qc(n,t,a),o=a.get(i);if(void 0!==o&&(!r||2!==o))return 1===o?-1:0;if(u){for(var s=0;s<p;s++)if(i===u[s])return 1;if(100===f)return _=!0,0}else u=[],d=[],m=[];var l=p;u[p]=i,p++,d[f]=n,m[f]=t,f++;var v=g;1&g||!Yc(n,d,f)||(g|=1),2&g||!Yc(t,m,f)||(g|=2);var y=3!==g?function(n,t,r){var i,o,s=n.flags&t.flags;if(a===ft&&!(131072&s)){if(1048576&s)return C(n.type,t.type,!1);var l=0;return 2097152&s&&(l=C(n.objectType,t.objectType,!1))&&(l&=C(n.indexType,t.indexType,!1))?l:4194304&s&&n.root.isDistributive===t.root.isDistributive&&(l=C(n.checkType,t.checkType,!1))&&(l&=C(n.extendsType,t.extendsType,!1))&&(l&=C(Ll(n),Ll(t),!1))&&(l&=C(xl(n),xl(t),!1))?l:8388608&s?C(n.substitute,t.substitute,!1):0}var u,d=c;if(65536&t.flags){if(32&e.getObjectFlags(n)&&Ki(n)===ml(t)&&!(4&Wi(n))){var m=Bi(n),p=hl(t,Vi(n));if(i=C(m,p,r))return i}}else if(1048576&t.flags){if(1048576&n.flags&&(i=C(t.type,n.type,!1)))return i;if(a!==mt){var f=vl(t.type),g=f!==t.type?f:eo(t.type);if(g&&-1===C(n,ml(g,t.stringsOnly),r))return-1}}else if(2097152&t.flags){if(a!==ft){var g=io(t);if(g&&g!==t&&(i=C(n,g,r)))return i}}else if(Ji(t)){var _=Bi(t),v=Wi(t);if(!(8&v)){if(2097152&_.flags&&_.objectType===n&&_.indexType===Vi(t))return-1;if(!Ji(n)&&Ki(t)===ml(n)){var p=hl(n,Vi(t)),m=Bi(t);if(i=C(p,m,r))return i}}}if(2162688&n.flags){if(2097152&n.flags&&2097152&t.flags&&((i=C(n.objectType,t.objectType,r))&&(i&=C(n.indexType,t.indexType,r)),i))return c=d,i;var g=(u=n,a===mt?void 0:eo(u));if(!g||65536&n.flags&&3&g.flags){if(i=C(Se,Od(t,-16777217)))return c=d,i}else{var y=Ai(g,n);if(i=C(y,t,r))return c=d,i}}else if(1048576&n.flags){if(i=C(Te,t,r))return c=d,i}else if(4194304&n.flags){if(4194304&t.flags){if(_c(n.extendsType,t.extendsType)&&(C(n.checkType,t.checkType)||C(t.checkType,n.checkType))&&((i=C(Ll(n),Ll(t),r))&&(i&=C(xl(n),xl(t),r)),i))return c=d,i}else if(a!==mt){var h=ro(n);if(h&&(i=C(h,t,r)))return c=d,i;var b=to(n);if(b&&(i=C(b,t,r)))return c=d,i}}else{if(function(n){return!!(32&e.getObjectFlags(n)&&4&Wi(n))}(t)&&!Ji(n)&&Nc(n))return-1;if(Ji(t))return Ji(n)&&(i=function(e,n,t){var r;if((a===pt||(a===ft?Wi(e)===Wi(n):zi(e)<=zi(n)))&&(r=C(Ki(n),Ki(e),t))){var i=Jl([Vi(e)],[Vi(n)]);return r&C(lc(Bi(e),i),Bi(n),t)}return 0}(n,t,r))?(c=d,i):0;if(a===mt&&Ji(n))return 0;var E=!!(32764&n.flags);if(a!==ft&&(n=po(n)),4&e.getObjectFlags(n)&&4&e.getObjectFlags(t)&&n.target===t.target&&!(8192&e.getObjectFlags(n)||8192&e.getObjectFlags(t))){var T=Hc(n.target);if(i=function(n,t,r,i){var o=n.typeArguments||e.emptyArray,s=t.typeArguments||e.emptyArray;if(o.length!==s.length&&a===ft)return 0;for(var l=o.length<=s.length?o.length:s.length,c=-1,u=0;u<l;u++){var d=u<r.length?r[u]:1;if(4!==d){var m=o[u],p=s[u],f=-1;if(1===d?f=C(m,p,i):2===d?f=C(p,m,i):3===d?(f=C(p,m,!1))||(f=C(m,p,i)):(f=C(m,p,i))&&(f&=C(p,m,i)),!f)return 0;c&=f}}return c}(n,t,T,r))return i;if(T!==e.emptyArray&&!function(e,n){for(var t=0;t<n.length;t++)if(1===n[t]&&4096&e.typeArguments[t].flags)return!0;return!1}(t,T)){if(!r||!e.some(T,function(e){return 0===e}))return 0;o=c,c=d}}else if(pu(n)&&(eu(t)||nu(t))||eu(n)&&nu(t))return C(So(n,1)||$,So(t,1)||$,r);if(655360&n.flags&&131072&t.flags){var S=r&&c===d&&!E;if((i=function(n,t,r){if(a===ft)return function(e,n){if(!(131072&e.flags&&131072&n.flags))return 0;var t=Yi(e),r=Yi(n);if(t.length!==r.length)return 0;for(var a=-1,i=0,o=t;i<o.length;i++){var s=o[i],l=Qi(n,s.escapedName);if(!l)return 0;var c=Qc(s,l,C);if(!c)return 0;a&=c}return a}(n,t);var i=a===ut&&!Ju(n)&&!ru(n)&&!pu(n),o=Hu(n,t,i);if(o)return r&&L(e.Diagnostics.Property_0_is_missing_in_type_1,Jr(o),Yr(n)),0;if(Ju(t))for(var s=0,l=$i(n);s<l.length;s++){var c=l[s];if(!Qi(t,c.escapedName)){var u=Ga(c);if(u!==ae&&u!==ie)return r&&L(e.Diagnostics.Property_0_does_not_exist_on_type_1,Jr(c),Yr(t)),0}}var d=-1;if(pu(t)){var m=fu(t);if(m){if(!pu(n))return 0;var p=fu(n);if(p&&!C(p,m,r))return r&&L(e.Diagnostics.Rest_signatures_are_incompatible),0;for(var f=cs(t)-1,g=cs(n)-(p?1:0),_=f;_<g;_++){var v=C(n.typeArguments[_],m,r);if(!v)return r&&L(e.Diagnostics.Property_0_is_incompatible_with_rest_element_type,\"\"+_),0;d&=v}}}for(var y=Yi(t),h=0,b=y;h<b.length;h++){var E=b[h];if(!(4194304&E.flags)){var c=vo(n,E.escapedName);if(c&&c!==E){if(Gc(n,c,Ga(E)))continue;var T=e.getDeclarationModifierFlagsFromSymbol(c),S=e.getDeclarationModifierFlagsFromSymbol(E);if(8&T||8&S){var x=c.valueDeclaration!==E.valueDeclaration;if(256&e.getCheckFlags(c)&&x)return r&&L(e.Diagnostics.Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1,Jr(c),Yr(n)),0;if(x)return r&&(8&T&&8&S?L(e.Diagnostics.Types_have_separate_declarations_of_a_private_property_0,Jr(E)):L(e.Diagnostics.Property_0_is_private_in_type_1_but_not_in_type_2,Jr(E),Yr(8&T?n:t),Yr(8&T?t:n))),0}else if(16&S){if(!Xc(c,E))return r&&L(e.Diagnostics.Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2,Jr(E),Yr(Jc(c)||n),Yr(Jc(E)||t)),0}else if(16&T)return r&&L(e.Diagnostics.Property_0_is_protected_in_type_1_but_public_in_type_2,Jr(E),Yr(n),Yr(t)),0;var v=C(Ga(c),Ga(E),r);if(!v)return r&&L(e.Diagnostics.Types_of_property_0_are_incompatible,Jr(E)),0;if(d&=v,a!==pt&&16777216&c.flags&&!(16777216&E.flags))return r&&L(e.Diagnostics.Property_0_is_optional_in_type_1_but_required_in_type_2,Jr(E),Yr(n),Yr(t)),0}}}return d}(n,t,S))&&(i&=N(n,t,0,S))&&(i&=N(n,t,1,S))&&(i&=F(n,t,0,E,S))&&(i&=F(n,t,1,E,S)),i){if(!o)return c=d,i;c=o}}}return 0}(n,t,r):1;if(g=v,f--,y){if(-1===y||0===f){for(s=l;s<p;s++)a.set(u[s],1);p=l}}else a.set(i,r?3:2),p=l;return y}function N(n,t,r,i){if(a===ft)return function(e,n,t){var r=ho(e,t),a=ho(n,t);if(r.length!==a.length)return 0;for(var i=-1,o=0;o<r.length;o++){var s=Zc(r[o],a[o],!1,!1,!1,C);if(!s)return 0;i&=s}return i}(n,t,r);if(t===Ce||n===Ce)return-1;var o=n.symbol&&gf(n.symbol.valueDeclaration),s=t.symbol&&gf(t.symbol.valueDeclaration),l=ho(n,o&&1===r?0:r),u=ho(t,s&&1===r?0:r);if(1===r&&l.length&&u.length){if(e.isAbstractConstructorType(n)&&!e.isAbstractConstructorType(t))return i&&L(e.Diagnostics.Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type),0;if(!function(n,t,r){if(!n.declaration||!t.declaration)return!0;var a=e.getSelectedModifierFlags(n.declaration,24),i=e.getSelectedModifierFlags(t.declaration,24);return 8===i||(16===i&&8!==a||(16!==i&&!a||(r&&L(e.Diagnostics.Cannot_assign_a_0_constructor_type_to_a_1_constructor_type,$r(a),$r(i)),!1)))}(l[0],u[0],i))return 0}var d=-1,m=c;if(64&e.getObjectFlags(n)&&64&e.getObjectFlags(t)&&n.symbol===t.symbol)for(var p=0;p<u.length;p++){if(!(E=w(l[p],u[p],!0,i)))return 0;d&=E}else if(1===l.length&&1===u.length){var f=a===pt||!!A.noStrictGenericChecks;d=w(l[0],u[0],f,i)}else e:for(var g=0,_=u;g<_.length;g++){for(var v=_[g],y=i,h=0,b=l;h<b.length;h++){var E;if(E=w(b[h],v,!0,y)){d&=E,c=m;continue e}y=!1}return y&&L(e.Diagnostics.Type_0_provides_no_match_for_the_signature_1,Yr(n),Xr(v,void 0,void 0,r)),0}return d}function w(e,n,t,r){return Ic(t?Xo(e):e,t?Xo(n):n,0,!1,r,L,C)}function P(n,t,r){var a=C(n.type,t.type,r);return!a&&r&&L(e.Diagnostics.Index_signatures_are_incompatible),a}function F(n,t,r,i,o){if(a===ft)return function(e,n,t){var r=To(n,t),a=To(e,t);return a||r?a&&r&&a.isReadonly===r.isReadonly?C(a.type,r.type):0:-1}(n,t,r);var s=To(t,r);if(!s||3&s.type.flags&&!i)return-1;var l=To(n,r)||1===r&&To(n,0);if(l)return P(l,s,o);if(Ji(n))return 0===r&&C(Bi(n),s.type,o);if(Su(n)){var c=-1;if(0===r){var u=To(n,1);u&&(c=P(u,s,o))}return c&&(c&=function(n,t,r,a){for(var i=-1,o=0,s=Yi(n);o<s.length;o++){var l=s[o];if(!Gc(n,l,void 0)&&!(l.nameType&&2048&l.nameType.flags)&&(0===r||Km(l.escapedName))){var c=C(Ga(l),t,a);if(!c)return a&&L(e.Diagnostics.Property_0_is_incompatible_with_index_signature,Jr(l)),0;i&=c}}return i}(n,s.type,r,o)),c}return o&&L(e.Diagnostics.Index_signature_is_missing_in_type_0,Yr(n)),0}}function Kc(n){if(131072&n.flags){var t=Xi(n);return 0===t.callSignatures.length&&0===t.constructSignatures.length&&!t.stringIndexInfo&&!t.numberIndexInfo&&t.properties.length>0&&e.every(t.properties,function(e){return!!(16777216&e.flags)})}return!!(524288&n.flags)&&e.every(n.types,Kc)}function Bc(n,t,r){var a=ss(n,e.map(n.typeParameters,function(e){return e===t?r:e}));return a.objectFlags|=8192,a}function Hc(n){if(!O)return e.emptyArray;var t=n.typeParameters||e.emptyArray,r=n.variances;if(!r){if(n===Ge||n===Ve)r=[1];else{n.variances=e.emptyArray,r=[];for(var a=0,i=t;a<i.length;a++){var o=i[a],s=Bc(n,o,Oe),l=Bc(n,o,Ie),c=(bc(l,s)?1:0)|(bc(s,l)?2:0);3===c&&bc(Bc(n,o,pn),s)&&(c=4),r.push(c)}}n.variances=r}return r}function Uc(e){return 65536&e.flags&&!no(e)}function jc(n){return!!(4&e.getObjectFlags(n))&&e.some(n.typeArguments,function(e){return Uc(e)||jc(e)})}function Wc(e,n,t){void 0===t&&(t=0);for(var r=\"\"+e.target.id,a=0,i=e.typeArguments;a<i.length;a++){var o=i[a];if(Uc(o)){var s=n.indexOf(o);s<0&&(s=n.length,n.push(o)),r+=\"=\"+s}else t<4&&jc(o)?r+=\"<\"+Wc(o,n,t+1)+\">\":r+=\"-\"+o.id}return r}function qc(e,n,t){if(t===ft&&e.id>n.id){var r=e;e=n,n=r}if(jc(e)&&jc(n)){var a=[];return Wc(e,a)+\",\"+Wc(n,a)}return e.id+\",\"+n.id}function zc(n,t){if(!(6&e.getCheckFlags(n)))return t(n);for(var r=0,a=n.containingType.types;r<a.length;r++){var i=vo(a[r],n.escapedName),o=i&&zc(i,t);if(o)return o}}function Jc(e){return e.parent&&32&e.parent.flags?si(Tr(e)):void 0}function Xc(n,t){return!zc(t,function(t){return!!(16&e.getDeclarationModifierFlagsFromSymbol(t))&&(r=n,a=Jc(t),!zc(r,function(e){var n=Jc(e);return!!n&&Ba(n,a)}));var r,a})}function Yc(e,n,t){if(t>=5&&131072&e.flags){var r=e.symbol;if(r)for(var a=0,i=0;i<t;i++){var o=n[i];if(131072&o.flags&&o.symbol===r&&++a>=5)return!0}}return!1}function Qc(n,t,r){if(n===t)return-1;var a=24&e.getDeclarationModifierFlagsFromSymbol(n);if(a!==(24&e.getDeclarationModifierFlagsFromSymbol(t)))return 0;if(a){if(q_(n)!==q_(t))return 0}else if((16777216&n.flags)!=(16777216&t.flags))return 0;return Zf(n)!==Zf(t)?0:r(Ga(n),Ga(t))}function Zc(n,t,r,a,i,o){if(n===t)return-1;if(!function(e,n,t){var r=Rf(e),a=Rf(n),i=Nf(e),o=Nf(n),s=wf(e),l=wf(n);if(r===a&&i===o&&s===l)return!0;var c=s?1:0,u=l?1:0;return!!(t&&i<=o&&(c>u||c===u&&r>=a))}(n,t,r))return 0;if(e.length(n.typeParameters)!==e.length(t.typeParameters))return 0;n=Xo(n),t=Xo(t);var s=-1;if(!a){var l=Vo(n);if(l){var c=Vo(t);if(c){if(!(m=o(l,c)))return 0;s&=m}}}for(var u=Rf(t),d=0;d<u;d++){var m;if(!(m=o(Mf(n,d),Mf(t,d))))return 0;s&=m}if(!i){var p=Bo(n),f=Bo(t);s&=void 0!==p||void 0!==f?function(e,n,t){return void 0!==e&&void 0!==n&&rl(e,n)?t(e.type,n.type):0}(p,f,o):o(Ho(n),Ho(t))}return s}function $c(n){return function(e){for(var n,t=0,r=e;t<r.length;t++){var a=r[t],i=cu(a);if(n||(n=i),i===a||i!==n)return!1}return!0}(n)?tl(n):e.reduceLeft(n,function(e,n){return hc(e,n)?n:e})}function eu(n){return!!(4&e.getObjectFlags(n))&&n.target===Ge}function nu(n){return!!(4&e.getObjectFlags(n))&&n.target===Ve}function tu(n){return 4&e.getObjectFlags(n)&&(n.target===Ge||n.target===Ve)||!(24576&n.flags)&&bc(n,ze)}function ru(e){var n=eu(e)?e.typeArguments[0]:void 0;return n===ie||n===he}function au(e){return pu(e)||!!vo(e,\"0\")}function iu(e,n){return pu(e)?n<gu(e)?e.typeArguments[n]:fu(e):ua(e,\"\"+n)}function ou(e){return!(59840&e.flags)}function su(e){return!!(27072&e.flags)}function lu(n){return!!(16&n.flags)||(262144&n.flags?!!(512&n.flags)||e.every(n.types,su):su(n))}function cu(n){return 512&n.flags?ri(n):64&n.flags?le:128&n.flags?ce:256&n.flags?fe:262144&n.flags?tl(e.sameMap(n.types,cu)):n}function uu(n){return 512&n.flags&&33554432&n.flags?ri(n):64&n.flags&&33554432&n.flags?le:128&n.flags&&33554432&n.flags?ce:256&n.flags&&33554432&n.flags?fe:262144&n.flags?tl(e.sameMap(n.types,uu)):n}function du(n){return 2048&n.flags?ge:262144&n.flags?tl(e.sameMap(n.types,du)):n}function mu(n,t){return function n(t,r){if(r){if(786432&r.flags){var a=r.types;return e.some(a,function(e){return n(t,e)})}if(14745600&r.flags){var i=io(r)||Se;return tg(i,4)&&tg(t,64)||tg(i,8)&&tg(t,128)||tg(i,1024)&&tg(t,2048)||n(t,i)}return!!(1048640&r.flags&&tg(t,64)||128&r.flags&&tg(t,128)||256&r.flags&&tg(t,256)||2048&r.flags&&tg(t,2048))}return!1}(n,t)||(n=du(uu(n))),n}function pu(n){return!!(4&e.getObjectFlags(n)&&8&n.target.objectFlags)}function fu(e){return e.target.hasRestElement?e.typeArguments[e.target.typeParameters.length-1]:void 0}function gu(e){return cs(e)-(e.target.hasRestElement?1:0)}function _u(e){for(var n=0,t=0,r=e;t<r.length;t++)n|=vu(r[t]);return n}function vu(e){return 262144&e.flags?_u(e.types):64&e.flags?\"\"===e.value?64:0:128&e.flags?0===e.value?128:0:256&e.flags?e===ue||e===de?256:0:29148&e.flags}function yu(e){return 29120&vu(e)?kd(e,function(e){return!(29120&vu(e))}):e}function hu(e){return 4&e.flags?Mn:8&e.flags?On:e===de||e===ue||28672&e.flags||64&e.flags&&\"\"===e.value||128&e.flags&&0===e.value?e:ve}function bu(e,n){var t=n&~e.flags&24576;return 0===t?e:tl(8192===t?[e,ae]:16384===t?[e,oe]:[e,ae,oe])}function Eu(n){return e.Debug.assert(M),8192&n.flags?n:tl([n,ae])}function Tu(e){return M?function(e){return Je||(Je=Cs(\"NonNullable\",524288,void 0)||Q),Je!==Q?ds(Je,[e]):md(e,524288)}(e):e}function Su(e){return e.symbol&&0!=(6656&e.symbol.flags)&&!Dv(e)}function Lu(e,n){var t=Tt(e.flags,e.escapedName);return t.declarations=e.declarations,t.parent=e.parent,t.type=n,t.target=e,e.valueDeclaration&&(t.valueDeclaration=e.valueDeclaration),e.nameType&&(t.nameType=e.nameType),t}function xu(n){if(!(Ju(n)&&33554432&n.flags))return n;var t=n.regularType;if(t)return t;var r=n,a=function(n,t){for(var r=e.createSymbolTable(),a=0,i=Yi(n);a<i.length;a++){var o=i[a],s=Ga(o),l=t(s);r.set(o.escapedName,l===s?o:Lu(o,l))}return r}(n,xu),i=wr(r.symbol,a,r.callSignatures,r.constructSignatures,r.stringIndexInfo,r.numberIndexInfo);return i.flags=-33554433&r.flags,i.objectFlags|=128|16384&e.getObjectFlags(r),n.regularType=i,i}function Au(e,n,t){return{parent:e,propertyName:n,siblings:t,resolvedProperties:void 0}}function Cu(n){if(!n.resolvedProperties){for(var t=e.createMap(),r=0,a=function e(n){if(!n.siblings){for(var t=[],r=0,a=e(n.parent);r<a.length;r++){var i=a[r];if(Ju(i)){var o=Qi(i,n.propertyName);o&&Dd(Ga(o),function(e){t.push(e)})}}n.siblings=t}return n.siblings}(n);r<a.length;r++){var i=a[r];if(Ju(i)&&!(1024&e.getObjectFlags(i)))for(var o=0,s=$i(i);o<s.length;o++){var l=s[o];t.set(l.escapedName,l)}}n.resolvedProperties=e.arrayFrom(t.values())}return n.resolvedProperties}function Du(e,n){if(!(4&e.flags))return e;var t=Ga(e),r=Ou(t,n&&Au(n,e.escapedName,void 0));return r===t?e:Lu(e,r)}function ku(e){var n=Y.get(e.escapedName);if(n)return n;var t=Lu(e,ae);return t.flags|=16777216,Y.set(e.escapedName,t),t}function Mu(e){return Ou(e,void 0)}function Ou(n,t){if(402653184&n.flags){if(24576&n.flags)return $;if(Ju(n))return function(n,t){for(var r=e.createSymbolTable(),a=0,i=Yi(n);a<i.length;a++){var o=i[a];r.set(o.escapedName,Du(o,t))}if(t)for(var s=0,l=Cu(t);s<l.length;s++)o=l[s],r.has(o.escapedName)||r.set(o.escapedName,ku(o));var c=To(n,0),u=To(n,1),d=wr(n.symbol,r,e.emptyArray,e.emptyArray,c&&ns(Mu(c.type),c.isReadonly),u&&ns(Mu(u.type),u.isReadonly));return d.objectFlags|=16384&e.getObjectFlags(n),d}(n,t);if(262144&n.flags){var r=t||Au(void 0,void 0,n.types),a=e.sameMap(n.types,function(e){return 24576&e.flags?e:Ou(e,r)});return tl(a,e.some(a,Nc)?2:1)}if(eu(n)||pu(n))return ss(n.target,e.sameMap(n.typeArguments,Mu))}return n}function Iu(n,t){var r,a=Yr(Mu(t));switch(n.kind){case 202:case 152:case 151:r=e.Diagnostics.Member_0_implicitly_has_an_1_type;break;case 149:r=n.dotDotDotToken?e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type:e.Diagnostics.Parameter_0_implicitly_has_an_1_type;break;case 184:r=e.Diagnostics.Binding_element_0_implicitly_has_an_1_type;break;case 237:case 154:case 153:case 156:case 157:case 194:case 195:if(!n.name)return void bt(n,e.Diagnostics.Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type,a);r=e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type;break;case 179:return void bt(n,e.Diagnostics.Mapped_object_type_implicitly_has_an_any_template_type);default:r=e.Diagnostics.Variable_0_implicitly_has_an_1_type}bt(n,r,e.declarationNameToString(e.getNameOfDeclaration(n)),a)}function Ru(n,t){a&&R&&134217728&t.flags&&(function n(t){var r=!1;if(134217728&t.flags){if(262144&t.flags)if(e.some(t.types,Nc))r=!0;else for(var a=0,i=t.types;a<i.length;a++)n(u=i[a])&&(r=!0);if(eu(t)||pu(t))for(var o=0,s=t.typeArguments;o<s.length;o++)n(u=s[o])&&(r=!0);if(Ju(t))for(var l=0,c=Yi(t);l<c.length;l++){var u,d=c[l];134217728&(u=Ga(d)).flags&&(n(u)||bt(d.valueDeclaration,e.Diagnostics.Object_literal_s_property_0_implicitly_has_an_1_type,Jr(d),Yr(Mu(u))),r=!0)}}return r}(t)||Iu(n,t))}function Nu(e,n,t){var r=Rf(e),a=Rf(n),i=Pf(e),o=Pf(n),s=o?Math.min(a-1,r):i?a:Math.min(r,a),l=Vo(e);if(l){var c=Vo(n);c&&t(l,c)}for(var u=0;u<s;u++)t(Mf(e,u),Mf(n,u));o&&t(If(e,s),o)}function wu(e,n,t,r,a){var i=a?a.map(Fu):e.map(Pu),o=function(e){for(var n=0;n<i.length;n++)if(e===i[n].typeParameter)return i[n].isFixed=!0,Qu(o,n);return e};return o.typeParameters=e,o.signature=n,o.inferences=i,o.flags=t,o.compareTypes=r||yc,o}function Pu(e){return{typeParameter:e,candidates:void 0,contraCandidates:void 0,inferredType:void 0,priority:void 0,topLevel:!0,isFixed:!1}}function Fu(e){return{typeParameter:e.typeParameter,candidates:e.candidates&&e.candidates.slice(),contraCandidates:e.contraCandidates&&e.contraCandidates.slice(),inferredType:e.inferredType,priority:e.priority,topLevel:e.topLevel,isFixed:e.isFixed}}function Gu(n){var t=e.getObjectFlags(n);return!!(15794176&n.flags||4&t&&e.forEach(n.typeArguments,Gu)||16&t&&n.symbol&&10288&n.symbol.flags||32&t||786432&n.flags&&function(n){return void 0===n.couldContainTypeVariables&&(n.couldContainTypeVariables=e.some(n.types,Gu)),n.couldContainTypeVariables}(n))}function Vu(n,t){return n===t||!!(786432&n.flags)&&e.some(n.types,function(e){return Vu(e,t)})}function Ku(n,t){var r=n.id+\",\"+t.id;if(Ln.has(r))return Ln.get(r);Ln.set(r,void 0);var a=function(n,t){var r=$i(n);if(0!==r.length||To(n,0)){for(var a=0,i=r;a<i.length;a++){var o=i[a];if(536870912&Ga(o).flags)return}if(eu(n))return Ws(Bu(n.typeArguments[0],t));if(nu(n))return qs(Bu(n.typeArguments[0],t));if(pu(n)){var s=e.map(n.typeArguments||e.emptyArray,function(e){return Bu(e,t)}),l=4&Wi(t)?cs(n)-(n.target.hasRestElement?1:0):n.target.minLength;return Js(s,l,n.target.hasRestElement,n.target.associatedNames)}var c=Or(2064,void 0);return c.source=n,c.mappedType=t,c}}(n,t);return Ln.set(r,a),a}function Bu(e,n){var t=hl(Ki(n).type,Vi(n)),r=Bi(n),a=Pu(t);return Wu([a],e,r),ju(a)}function Hu(e,n,t){for(var r=0,a=524288&n.flags?Zi(n):Yi(n);r<a.length;r++){var i=a[r];if((t||!(16777216&i.flags))&&!vo(e,i.escapedName))return i}}function Uu(e,n){return pu(e)&&pu(n)&&function(e,n){return n.target.minLength>e.target.minLength||!fu(n)&&(!!fu(e)||gu(n)<gu(e))}(e,n)||!!Hu(e,n,!1)&&!!Hu(n,e,!1)}function ju(e){return e.candidates?tl(e.candidates,2):e.contraCandidates?ll(e.contraCandidates):Se}function Wu(n,t,r,a){var i,o;void 0===a&&(a=0);var s,l=!1;function c(n,t){if(Gu(t)){if(n===ne){var p=s;return s=n,c(t,t),void(s=p)}if(n.aliasSymbol&&n.aliasTypeArguments&&n.aliasSymbol===t.aliasSymbol)for(var f=n.aliasTypeArguments,g=t.aliasTypeArguments,_=0;_<f.length;_++)c(f[_],g[_]);else{if(262144&n.flags&&262144&t.flags&&!(512&n.flags&&512&t.flags)||524288&n.flags&&524288&t.flags){if(n===t){for(var v=0,y=n.types;v<y.length;v++)c(R=y[v],R);return}for(var h=void 0,b=0,E=n.types;b<E.length;b++)if(qu(R=E[b],t.types))(h||(h=[])).push(R),c(R,R);else if(192&R.flags){var T=cu(R);qu(T,t.types)&&(h||(h=[])).push(R,T)}h&&(n=zu(n,h),t=zu(t,h))}if(2162688&t.flags){if(536870912&n.flags||n===ye)return;var S=d(t);if(S){if(!S.isFixed){if((void 0===S.priority||a<S.priority)&&(S.candidates=void 0,S.contraCandidates=void 0,S.priority=a),a===S.priority){var L=s||n;l?S.contraCandidates=e.append(S.contraCandidates,L):S.candidates=e.append(S.candidates,L)}8&a||!(65536&t.flags)||Vu(r,t)||(S.topLevel=!1)}return}}if(4&e.getObjectFlags(n)&&4&e.getObjectFlags(t)&&n.target===t.target){f=n.typeArguments||e.emptyArray,g=t.typeArguments||e.emptyArray;var x=f.length<g.length?f.length:g.length,A=Hc(n.target);for(_=0;_<x;_++)_<A.length&&2===A[_]?u(f[_],g[_]):c(f[_],g[_])}else if(1048576&n.flags&&1048576&t.flags)l=!l,c(n.type,t.type),l=!l;else if((lu(n)||4&n.flags)&&1048576&t.flags){var C=function(n){var t=e.createSymbolTable();Dd(n,function(n){if(64&n.flags){var r=e.escapeLeadingUnderscores(n.value),a=Tt(4,r);a.type=$,n.symbol&&(a.declarations=n.symbol.declarations,a.valueDeclaration=n.symbol.valueDeclaration),t.set(r,a)}});var r=4&n.flags?ns(Se,!1):void 0;return wr(void 0,t,e.emptyArray,e.emptyArray,r,void 0)}(n);l=!l;var D=a;a|=16,c(C,t.type),a=D,l=!l}else if(2097152&n.flags&&2097152&t.flags)c(n.objectType,t.objectType),c(n.indexType,t.indexType);else if(4194304&n.flags&&4194304&t.flags)c(n.checkType,t.checkType),c(n.extendsType,t.extendsType),c(Ll(n),Ll(t)),c(xl(n),xl(t));else if(4194304&t.flags)c(n,tl([Ll(t),xl(t)]));else if(786432&t.flags){for(var k=0,M=void 0,O=0,I=g=t.types;O<I.length;O++){var R;d(R=I[O])?(M=R,k++):c(n,R)}1===k&&(D=a,a|=1,c(n,M),a=D)}else if(262144&n.flags)for(var N=0,w=f=n.types;N<w.length;N++)c(w[N],t);else{if(!(32&a&&16318464&n.flags)){var P=po(n);if(P!==n&&!(655360&P.flags))return c(P,t);n=P}if(655360&n.flags){var F=n.id+\",\"+t.id;if(o&&o.get(F))return;(o||(o=e.createMap())).set(F,!0);var G=131072&t.flags&&!(16&e.getObjectFlags(t)&&t.symbol&&32&t.symbol.flags)?t.symbol:void 0;if(G){if(e.contains(i,G))return;(i||(i=[])).push(G),m(n,t),i.pop()}else m(n,t)}}}}}function u(e,n){O||64&a?(l=!l,c(e,n),l=!l):c(e,n)}function d(e){if(2162688&e.flags)for(var t=0,r=n;t<r.length;t++){var a=r[t];if(e===a.typeParameter)return a}}function m(n,t){if(Ji(n)&&Ji(t)&&(c(Ki(n),Ki(t)),c(Bi(n),Bi(t))),32&e.getObjectFlags(t)){var r=Ki(t);if(1048576&r.flags){var i=d(r.type);if(i&&!i.isFixed){var o=Ku(n,t);if(o){var s=a;a|=2,c(o,i.typeParameter),a=s}}return}if(65536&r.flags)return s=a,a|=4,c(ml(n),r),a=s,void c(tl(e.map($i(n),Ga)),Bi(t))}Uu(n,t)||(function(e,n){if(pu(e)){if(pu(n)){for(var t=gu(e),r=gu(n),a=fu(e),i=fu(n),o=r<t||a?r:t,s=0;s<o;s++)c(s<t?e.typeArguments[s]:a,n.typeArguments[s]);if(i){var l=o<t?e.typeArguments.slice(o,t):[];a&&l.push(a),l.length&&c(tl(l),i)}return}if(eu(n))return void g(e,n)}for(var u=Yi(n),d=0,m=u;d<m.length;d++){var p=m[d],f=vo(e,p.escapedName);f&&c(Ga(f),Ga(p))}}(n,t),p(n,t,0),p(n,t,1),g(n,t))}function p(e,n,t){for(var r=ho(e,t),a=ho(n,t),i=r.length,o=a.length,s=i<o?i:o,l=!!(536870912&e.flags),c=0;c<s;c++)f(Qo(r[i-s+c]),Qo(a[o-s+c]),l)}function f(e,n,t){t||Nu(e,n,u);var r=Bo(e),a=Bo(n);r&&a&&r.kind===a.kind?c(r.type,a.type):c(Ho(e),Ho(n))}function g(e,n){var t=So(n,0);t&&(r=So(e,0)||Lo(e,0))&&c(r,t);var r,a=So(n,1);a&&(r=So(e,1)||So(e,0)||Lo(e,1))&&c(r,a)}c(t,r)}function qu(e,n){for(var t=0,r=n;t<r.length;t++)if(_c(r[t],e))return!0;return!1}function zu(e,n){for(var t=[],r=0,a=e.types;r<a.length;r++){var i=a[r];qu(i,n)||t.push(i)}return 262144&e.flags?tl(t):ll(t)}function Ju(n){return!!(128&e.getObjectFlags(n))}function Xu(n){return 28&n.priority?ll(n.contraCandidates):(t=n.contraCandidates,e.reduceLeft(t,function(e,n){return hc(n,e)?n:e}));var t}function Yu(n,t){var r,a,i=function(n){if(n.length>1){var t=e.filter(n,Ju);if(t.length){var r=Mu(tl(t,2));return e.concatenate(e.filter(n,function(e){return!Ju(e)}),[r])}}return n}(n.candidates),o=(r=n.typeParameter,!!(a=no(r))&&tg(4194304&a.flags?to(a):a,1081340)),s=!o&&n.topLevel&&(n.isFixed||!Vu(Ho(t),n.typeParameter)),l=o?e.sameMap(i,Vl):s?e.sameMap(i,uu):i;return Mu(28&n.priority?tl(l,2):function(n){if(!M)return $c(n);var t=e.filter(n,function(e){return!(24576&e.flags)});return t.length?bu($c(t),24576&_u(n)):tl(n,2)}(l))}function Qu(e,n){var t=e.inferences[n],r=t.inferredType;if(!r){var a=e.signature;if(a)if(t.contraCandidates&&(!t.candidates||1===t.candidates.length&&32768&t.candidates[0].flags)&&(t.candidates=[Xu(t)],t.contraCandidates=void 0),t.candidates)r=Yu(t,a);else if(1&e.flags)r=ye;else{var i=uo(t.typeParameter);r=i?lc(i,Yl(function(e,n){return function(t){return e.indexOf(t)>=n?Se:t}}(e.signature.typeParameters,n),e)):Zu(!!(2&e.flags))}else r=ju(t);t.inferredType=r;var o=no(t.typeParameter);if(o){var s=lc(o,e);e.compareTypes(r,Ai(s,r))||(t.inferredType=r=s)}}return r}function Zu(e){return e?$:Se}function $u(e){for(var n=[],t=0;t<e.inferences.length;t++)n.push(Qu(e,t));return n}function ed(n){switch(n){case\"document\":case\"console\":return e.Diagnostics.Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_include_dom;case\"$\":return e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery;case\"describe\":case\"suite\":case\"it\":case\"test\":return e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashjest_or_npm_i_types_Slashmocha;case\"process\":case\"require\":case\"Buffer\":case\"module\":return e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode;case\"Map\":case\"Set\":case\"Promise\":case\"Symbol\":case\"WeakMap\":case\"WeakSet\":case\"Iterator\":case\"AsyncIterator\":return e.Diagnostics.Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later;default:return e.Diagnostics.Cannot_find_name_0}}function nd(n){var t=Rt(n);return t.resolvedSymbol||(t.resolvedSymbol=!e.nodeIsMissing(n)&&Ft(n,n.escapedText,68268991,ed(n.escapedText),n,!e.isWriteOnlyAccess(n),!1,e.Diagnostics.Cannot_find_name_0_Did_you_mean_1)||Q),t.resolvedSymbol}function td(n){return!!e.findAncestor(n,function(e){return 165===e.kind||71!==e.kind&&146!==e.kind&&\"quit\"})}function rd(n){var t=n.parent;if(182!==t.kind)return\"\"+t.elements.indexOf(n);var r=n.propertyName||n.name;switch(r.kind){case 71:return e.idText(r);case 147:return e.isStringOrNumericLiteralLike(r.expression)?r.expression.text:void 0;case 9:case 8:return r.text;default:e.Debug.fail(\"Unexpected name kind for binding element name\")}}function ad(n,t){switch(n.kind){case 71:return 71===t.kind&&nd(n)===nd(t)||(235===t.kind||184===t.kind)&&xr(nd(n))===Er(t);case 99:return 99===t.kind;case 97:return 97===t.kind;case 187:case 188:return(e.isPropertyAccessExpression(t)||e.isElementAccessExpression(t))&&id(n)===id(t)&&ad(n.expression,t.expression);case 184:if(187!==t.kind)return!1;var r=t;if(r.name.escapedText!==rd(n))return!1;if(184===n.parent.parent.kind&&ad(n.parent.parent,r.expression))return!0;if(235===n.parent.parent.kind){var a=n.parent.parent.initializer;return!!a&&ad(a,r.expression)}}return!1}function id(n){return e.isPropertyAccessExpression(n)?n.name.escapedText:e.isStringLiteral(n.argumentExpression)||e.isNumericLiteral(n.argumentExpression)?e.escapeLeadingUnderscores(n.argumentExpression.text):void 0}function od(e,n){for(;187===e.kind;)if(ad(e=e.expression,n))return!0;return!1}function sd(e,n){return 187===n.kind&&od(e,n.expression)&&ld(function e(n){if(71===n.kind)return Ga(nd(n));if(187===n.kind){var t=e(n.expression);return t&&ua(t,n.name.escapedText)}}(n.expression),n.name.escapedText)}function ld(n,t){if(n&&262144&n.flags){var r=go(n,t);if(r&&2&e.getCheckFlags(r))return void 0===r.isDiscriminantProperty&&(r.isDiscriminantProperty=!!(32&r.checkFlags)&&lu(Ga(r))),r.isDiscriminantProperty}return!1}function cd(e,n){return ad(e,n)||od(e,n)}function ud(e,n){if(e!==n){if(32768&n.flags)return n;var t=kd(e,function(e){return function(e,n){if(!(262144&e.flags))return bc(e,n);for(var t=0,r=e.types;t<r.length;t++)if(bc(r[t],n))return!0;return!1}(n,e)});if(33554432&n.flags&&256&n.flags&&(t=Md(t,Gl)),bc(n,t))return t}return e}function dd(n){var t=n.flags;if(4&t)return M?4079361:4194049;if(64&t){var r=\"\"===n.value;return M?r?3030785:1982209:r?3145473:4194049}if(40&t)return M?4079234:4193922;if(128&t){var a=0===n.value;return M?a?3030658:1982082:a?3145346:4193922}return 16&t?M?4078980:4193668:272&t?M?n===ue||n===de?3030404:1981828:n===ue||n===de?3145092:4193668:131072&t?16&e.getObjectFlags(n)&&Nc(n)?M?4079615:4194303:function(e){var n=Xi(e);return!!(n.callSignatures.length||n.constructSignatures.length||n.members.get(\"bind\")&&hc(e,Fe))}(n)?M?1970144:4181984:M?1972176:4184016:12288&t?2457472:16384&t?2340752:3072&t?M?1981320:4193160:16777216&t?M?1972176:4184016:15794176&t?dd(io(n)||Se):786432&t?function(e){for(var n=0,t=0,r=e;t<r.length;t++)n|=dd(r[t]);return n}(n.types):4194303}function md(e,n){return kd(e,function(e){return 0!=(dd(e)&n)})}function pd(e,n){if(n){var t=Tg(n);return tl([md(e,131072),t])}return e}function fd(n,t){var r=e.getTextOfPropertyName(t);return Qd(ua(n,r),t)||Km(r)&&So(n,1)||So(n,0)||te}function gd(e,n){return au(e)&&iu(e,n)||O_(e,void 0,!1,!1)||te}function _d(e){return Ws(O_(e,void 0,!1,!1)||te)}function vd(e){return 202===e.parent.kind&&e.parent.left===e||225===e.parent.kind&&e.parent.initializer===e}function yd(e){return fd(hd(e.parent),e.name)}function hd(e){var n=e.parent;switch(n.kind){case 224:return le;case 225:return M_(n.expression,n.awaitModifier)||te;case 202:return function(e){return 185===e.parent.kind&&vd(e.parent)||273===e.parent.kind&&vd(e.parent.parent)?pd(hd(e),e.right):Tg(e.right)}(n);case 196:return ae;case 185:return function(e,n){return gd(hd(e),e.elements.indexOf(n))}(n,e);case 206:return function(e){return _d(hd(e.parent))}(n);case 273:return yd(n);case 274:return function(e){return pd(yd(e),e.objectAssignmentInitializer)}(n)}return te}function bd(e){return e.initializer?function(e){return Rt(e).resolvedType||Tg(e)}(e.initializer):224===e.parent.parent.kind?le:225===e.parent.parent.kind&&M_(e.parent.parent.expression,e.parent.parent.awaitModifier)||te}function Ed(e){return 235===e.kind?bd(e):function(e){var n=e.parent,t=Ed(n.parent);return pd(182===n.kind?fd(t,e.propertyName||e.name):e.dotDotDotToken?_d(t):gd(t,n.elements.indexOf(e)),e.initializer)}(e)}function Td(e,n){return Qd(235===e.kind||184===e.kind?Ed(e):hd(e),n)}function Sd(e){switch(e.kind){case 193:return Sd(e.expression);case 202:switch(e.operatorToken.kind){case 58:return Sd(e.left);case 26:return Sd(e.right)}}return e}function Ld(e){return 269===e.kind?Vl(Tg(e.expression)):ve}function xd(e){var n=Rt(e);if(!n.switchTypes){n.switchTypes=[];for(var t=0,r=e.caseBlock.clauses;t<r.length;t++){var a=r[t];n.switchTypes.push(Ld(a))}}return n.switchTypes}function Ad(n){for(var t=[],r=0,a=n.caseBlock.clauses;r<a.length;r++){var i=a[r];if(269===i.kind){if(9===i.expression.kind){t.push(i.expression.text);continue}return e.emptyArray}t.push(void 0)}return t}function Cd(e,n){return e===n||262144&n.flags&&function(e,n){if(262144&e.flags){for(var t=0,r=e.types;t<r.length;t++){var a=r[t];if(!Ys(n.types,a))return!1}return!0}return!!(512&e.flags&&ri(e)===n)||Ys(n.types,e)}(e,n)}function Dd(n,t){return 262144&n.flags?e.forEach(n.types,t):t(n)}function kd(n,t){if(262144&n.flags){var r=n.types,a=e.filter(r,t);return a===r?n:al(a,67108864&n.flags)}return t(n)?n:ve}function Md(e,n,t){if(32768&e.flags)return e;if(!(262144&e.flags))return n(e);for(var r,a,i=0,o=e.types;i<o.length;i++){var s=n(o[i]);s&&(r?a?a.push(s):a=[r,s]:r=s)}return a?tl(a,t?0:1):r}function Od(e,n){return kd(e,function(e){return 0!=(e.flags&n)})}function Id(e,n){return Cd(le,e)&&tg(n,64)||Cd(ce,e)&&tg(n,128)?Md(e,function(e){return 4&e.flags?Od(n,68):8&e.flags?Od(n,136):e}):e}function Rd(e){return 0===e.flags}function Nd(e){return 0===e.flags?e.type:e}function wd(e,n){return n?{flags:0,type:e}:e}function Pd(e){return X[e.id]||(X[e.id]=function(e){var n=Or(256);return n.elementType=e,n}(e))}function Fd(e,n){var t=cu(Sg(n));return Cd(t,e.elementType)?e:Pd(tl([e.elementType,t]))}function Gd(e){return e.finalArrayType||(e.finalArrayType=32768&(n=e.elementType).flags?qe:Ws(262144&n.flags?tl(n.types,2):n));var n}function Vd(n){return 256&e.getObjectFlags(n)?Gd(n):n}function Kd(n){return 256&e.getObjectFlags(n)?n.elementType:ve}function Bd(n,t){return function(n){for(var t=!1,r=0,a=n;r<a.length;r++){var i=a[r];if(!(32768&i.flags)){if(!(256&e.getObjectFlags(i)))return!1;t=!0}}return t}(n)?Pd(tl(e.map(n,Kd))):tl(e.sameMap(n,Vd),t)}function Hd(n){var t=function e(n){var t=n.parent;return 193===t.kind||202===t.kind&&58===t.operatorToken.kind&&t.left===n||202===t.kind&&26===t.operatorToken.kind&&t.right===n?e(t):n}(n),r=t.parent,a=187===r.kind&&(\"length\"===r.name.escapedText||189===r.parent.kind&&e.isPushOrUnshiftIdentifier(r.name)),i=188===r.kind&&r.expression===t&&202===r.parent.kind&&58===r.parent.operatorToken.kind&&r.parent.left===r&&!e.isAssignmentTarget(r.parent)&&rg(Tg(r.argumentExpression),168);return a||i}function Ud(n){var t=Rt(n);return void 0===t.maybeTypePredicate&&(t.maybeTypePredicate=function(n){if(97!==n.expression.kind){var t=vp(n.expression);if(t!==ye){var r=po(t);return r!==te&&e.some(ho(r,0),Ko)}}return!1}(n)),t.maybeTypePredicate}function jd(n,t,r,a,i){var s;void 0===r&&(r=t);var c=0;if(kn)return te;if(!n.flowNode||!i&&!(33492479&t.flags))return t;var u=Dn,d=Nd(p(n.flowNode));Dn=u;var m=256&e.getObjectFlags(d)&&Hd(n)?We:Vd(d);return n.parent&&211===n.parent.kind&&32768&md(m,524288).flags?t:m;function p(i){if(2e3===c)return kn=!0,o=n,s=e.findAncestor(o,e.isFunctionOrModuleBlock),l=e.getSourceFileOfNode(o),d=e.getSpanOfTokenAtPosition(l,s.statements.pos),Xn.add(e.createFileDiagnostic(l,d.start,d.length,e.Diagnostics.The_containing_function_or_module_body_is_too_large_for_control_flow_analysis)),te;var o,s,l,d;for(c++;;){var m=i.flags;if(1024&m)for(var b=u;b<Dn;b++)if(jn[b]===i)return c--,Wn[b];var E=void 0;if(4096&m)i.locked=!0,E=p(i.antecedent),i.locked=!1;else{if(2048&m){i=i.antecedent;continue}if(16&m){if(!(E=f(i))){i=i.antecedent;continue}}else if(96&m)E=_(i);else if(128&m)E=v(i);else if(12&m){if(1===i.antecedents.length){i=i.antecedents[0];continue}E=4&m?y(i):h(i)}else if(256&m){if(!(E=g(i))){i=i.antecedent;continue}}else if(2&m){var T=i.container;if(T&&T!==a&&187!==n.kind&&188!==n.kind&&99!==n.kind){i=T.flowNode;continue}E=r}else E=E_(t)}return 1024&m&&(jn[Dn]=i,Wn[Dn]=E,Dn++),c--,E}}function f(r){var a=r.node;if(ad(n,a)){if(2===e.getAssignmentTargetKind(a)){var i=p(r.antecedent);return wd(cu(Nd(i)),Rd(i))}if(t===ee||t===qe){if(function(e){return 235===e.kind&&e.initializer&&va(e.initializer)||184!==e.kind&&202===e.parent.kind&&va(e.parent.right)}(a))return Pd(ve);var o=cu(Td(a,n));return bc(o,t)?o:We}return 262144&t.flags?ud(t,Td(a,n)):t}if(od(n,a)){if(e.isVariableDeclaration(a)&&(e.isInJSFile(a)||e.isVarConst(a))){var s=e.getDeclaredExpandoInitializer(a);if(s&&(194===s.kind||195===s.kind))return p(r.antecedent)}return t}}function g(r){if(t===ee||t===qe){var a=r.node,i=189===a.kind?a.expression.expression:a.left.expression;if(ad(n,Sd(i))){var o=p(r.antecedent),s=Nd(o);if(256&e.getObjectFlags(s)){var l=s;if(189===a.kind)for(var c=0,u=a.arguments;c<u.length;c++)l=Fd(l,u[c]);else rg(Sg(a.left.argumentExpression),168)&&(l=Fd(l,a.right));return l===s?o:wd(l,Rd(o))}return o}}}function _(e){var n=p(e.antecedent),t=Nd(n);if(32768&t.flags)return n;var r=0!=(32&e.flags),a=Vd(t),i=O(a,e.expression,r);if(i===a)return n;var o=Rd(n);return wd(o&&32768&i.flags?ye:i,o)}function v(t){var r=p(t.antecedent),a=Nd(r),i=t.switchStatement.expression;return ad(n,i)?a=C(a,t.switchStatement,t.clauseStart,t.clauseEnd):b(i,a)?a=E(a,i,function(e){return C(e,t.switchStatement,t.clauseStart,t.clauseEnd)}):197===i.kind&&ad(n,i.expression)&&(a=function(n,t,r,a){var i=Ad(t);if(!i.length)return n;var o,s,l=e.findIndex(i,function(e){return void 0===e}),c=r===a||l>=r&&l<a;if(l>-1){var u=i.filter(function(e){return void 0!==e}),d=l<r?r-1:r,m=l<a?a-1:a;o=u.slice(d,m),s=Wf(d,m,u,c)}else o=i.slice(r,a),s=Wf(r,a,i,c);if(!(c||262144&n.flags)){var p=md(tl(o.map(function(e){return lt.get(e)||ve})),s);if(262144&p.flags&&(p=ud(p,io(n)||n)),!(32768&p.flags)){if(hc(p,n))return p;if(15794176&n.flags){var f=io(n)||$;if(hc(p,f))return ll([n,p])}}}return c?kd(n,function(e){return(dd(e)&s)===s}):md(n,s)}(a,t.switchStatement,t.clauseStart,t.clauseEnd)),wd(a,Rd(r))}function y(n){for(var a=[],i=!1,o=!1,s=0,l=n.antecedents;s<l.length;s++){var c=l[s];if(!(2048&c.flags&&c.lock.locked)){var u=p(c),d=Nd(u);if(d===t&&t===r)return d;e.pushIfUnique(a,d),Cd(d,t)||(i=!0),Rd(u)&&(o=!0)}}return wd(Bd(a,i?2:1),o)}function h(r){var a=function(e){return e.id||(e.id=o,o++),e.id}(r),i=Kn[a]||(Kn[a]=e.createMap());if(!s&&!(s=function n(t){if(71===t.kind){var r=nd(t);return r!==Q?(Xd(t)?\"@\":\"\")+l(r):void 0}if(99===t.kind)return\"0\";if(187===t.kind)return(i=n(t.expression))&&i+\".\"+e.idText(t.name);if(184===t.kind){var a=t.parent.parent,i=184===a.kind?n(a):a.initializer&&n(a.initializer),o=rd(t);return i&&o&&i+\".\"+o}}(n)))return t;var c=i.get(s);if(c)return c;for(var u=An;u<Cn;u++)if(Bn[u]===r&&Hn[u]===s&&Un[u].length)return wd(Bd(Un[u],1),!0);var d,m=[],f=!1;Bn[Cn]=r,Hn[Cn]=s,Un[Cn]=m;for(var g=0,_=r.antecedents;g<_.length;g++){var v=_[g];Cn++;var y=p(v);Cn--,d||(d=y);var h=Nd(y),b=i.get(s);if(b)return b;if(e.pushIfUnique(m,h),Cd(h,t)||(f=!0),h===t)break}var E=Bd(m,f?2:1);return Rd(d)?wd(E,!0):(i.set(s,E),E)}function b(e,t){if(!(262144&t.flags)||187!==e.kind&&188!==e.kind)return!1;var r=e,a=id(r);return!!a&&ad(n,r.expression)&&ld(t,a)}function E(e,n,t){var r=id(n);if(!r)return e;var a=ua(e,r),i=a&&t(a);return a===i?e:kd(e,function(e){return Tc(ua(e,r),i)})}function T(e,r,a){return ad(n,r)?md(e,a?1048576:2097152):b(r,t)?E(e,r,function(e){return md(e,a?1048576:2097152)}):sd(n,r)?t:e}function S(n,t,r){if(393216&n.flags||65536&n.flags&&n.isThisType){var a=e.escapeLeadingUnderscores(t.text);return kd(n,function(e){return function(e,n,t){if(To(e,0))return!0;var r=vo(e,n);return r?!!(16777216&r.flags)||t:!t}(e,a,r)})}return n}function L(r,a,i){switch(a.operatorToken.kind){case 58:return T(r,a.left,i);case 32:case 33:case 34:case 35:var o=a.operatorToken.kind,s=Sd(a.left),l=Sd(a.right);if(197===s.kind&&e.isStringLiteralLike(l))return A(r,s,o,l,i);if(197===l.kind&&e.isStringLiteralLike(s))return A(r,l,o,s,i);if(ad(n,s))return x(r,o,l,i);if(ad(n,l))return x(r,o,s,i);if(b(s,t))return E(r,s,function(e){return x(e,o,l,i)});if(b(l,t))return E(r,l,function(e){return x(e,o,s,i)});if(sd(n,s)||sd(n,l))return t;break;case 93:return function(r,a,i){var o=Sd(a.left);if(!ad(n,o))return od(n,o)?t:r;var s,l=Tg(a.right);if(!hc(l,Fe))return r;var c=vo(l,\"prototype\");if(c){var u=Ga(c);da(u)||(s=u)}if(da(r)&&(s===Pe||s===Fe))return r;if(!s){var d=void 0;2&e.getObjectFlags(l)?d=gi(l).declaredConstructSignatures:16&e.getObjectFlags(l)&&(d=ho(l,1)),d&&d.length&&(s=tl(e.map(d,function(e){return Ho(Xo(e))})))}return s?D(r,s,i,Ec):r}(r,a,i);case 92:var c=Sd(a.right);if(e.isStringLiteralLike(a.left)&&ad(n,c))return S(r,a.left,i);break;case 26:return O(r,a.right,i)}return r}function x(e,n,t,r){if(1&e.flags)return e;33!==n&&35!==n||(r=!r);var a=Tg(t);if(24576&a.flags)return M?md(e,32===n||33===n?r?65536:524288:16384&a.flags?r?32768:262144:r?16384:131072):e;if(16909315&e.flags)return e;if(r){var i=kd(e,function(e){return Sc(e,a)});return 32768&i.flags?e:Id(i,a)}if(su(a)){var o=Vl(a);return kd(e,function(e){return Vl(e)!==o})}return e}function A(e,r,a,i,o){var s=Sd(r.expression);if(!ad(n,s))return od(n,s)?t:e;if(33!==a&&35!==a||(o=!o),1&e.flags&&\"function\"===i.text)return e;var l=o?ot.get(i.text)||64:st.get(i.text)||8192;return md(o?Md(e,function(e){if(2&e.flags&&\"object\"===i.text)return tl([be,oe]);var n=\"function\"===i.text?Fe:lt.get(i.text);if(n){if(hc(e,n))return e;if(hc(n,e))return n;if(15794176&e.flags){var t=io(e)||$;if(hc(n,t))return ll([e,n])}}return e}):e,l)}function C(n,t,r,a){var i=xd(t);if(!i.length)return n;var o=i.slice(r,a),s=r===a||e.contains(o,ve),l=tl(o),c=32768&l.flags?ve:Id(kd(n,function(e){return Sc(l,e)}),l);if(!s)return c;var u=kd(n,function(n){return!(su(n)&&e.contains(i,Vl(n)))});return 32768&c.flags?u:tl([c,u])}function D(e,n,t,r){if(!t)return kd(e,function(e){return!r(e,n)});if(262144&e.flags){var a=kd(e,function(e){return r(e,n)});if(!(32768&a.flags))return a}return hc(n,e)?n:bc(e,n)?e:bc(n,e)?n:ll([e,n])}function k(r,a,i){if(!function(e,n){if(e.arguments)for(var t=0,r=e.arguments;t<r.length;t++)if(cd(n,r[t]))return!0;return!(187!==e.expression.kind||!cd(n,e.expression.expression))}(a,n)||!Ud(a))return r;var o=ff(a),s=Bo(o);if(!s)return r;if(da(r)&&(s.type===Pe||s.type===Fe))return r;if(e.isIdentifierTypePredicate(s)){var l=a.arguments[s.parameterIndex-(o.thisParameter?1:0)];if(l){if(ad(n,l))return D(r,s.type,i,hc);if(od(n,l))return t}}else{var c=e.skipParentheses(a.expression);if(188===c.kind||187===c.kind){var u=c,d=e.skipParentheses(u.expression);if(ad(n,d))return D(r,s.type,i,hc);if(od(n,d))return t}}return r}function O(e,n,t){switch(n.kind){case 71:case 99:case 97:case 187:case 188:return T(e,n,t);case 189:return k(e,n,t);case 193:return O(e,n.expression,t);case 202:return L(e,n,t);case 200:if(51===n.operator)return O(e,n.operand,!t)}return e}}function Wd(n){return e.findAncestor(n.parent,function(n){return e.isFunctionLike(n)&&!e.getImmediatelyInvokedFunctionExpression(n)||243===n.kind||277===n.kind||152===n.kind})}function qd(n){var t,r=e.getRootDeclaration(n.valueDeclaration).parent,a=Rt(r);return 4194304&a.flags||(a.flags|=4194304,t=r,e.findAncestor(t.parent,function(n){return e.isFunctionLike(n)&&!!(4194304&Rt(n).flags)})||zd(r)),n.isAssigned||!1}function zd(n){if(71===n.kind){if(e.isAssignmentTarget(n)){var t=nd(n);t.valueDeclaration&&149===e.getRootDeclaration(t.valueDeclaration).kind&&(t.isAssigned=!0)}}else e.forEachChild(n,zd)}function Jd(e){return 3&e.flags&&0!=(2&pp(e))&&Ga(e)!==qe}function Xd(e){var n=e.parent;return 187===n.kind||189===n.kind&&n.expression===e||188===n.kind&&n.expression===e||184===n.kind&&n.name===e&&!!n.initializer}function Yd(e){return 14745600&e.flags&&tg(io(e)||Se,24576)}function Qd(e,n){return e&&Xd(n)&&Dd(e,Yd)?Md(Mu(e),oo):e}function Zd(e,n){!Zt(e,67220415)||td(n)||Vv(er(e))||tr(e)}function $d(n){var t=nd(n);if(t===Q)return te;if(t===K){var r=e.getContainingFunction(n);return C<2&&(195===r.kind?bt(n,e.Diagnostics.The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_standard_function_expression):e.hasModifier(r,256)&&bt(n,e.Diagnostics.The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_using_a_standard_function_or_method)),Rt(r).flags|=8192,Ga(t)}n.parent&&e.isPropertyAccessExpression(n.parent)&&n.parent.expression===n||Zd(t,n);var a=xr(t),i=a.valueDeclaration;if(32&a.flags)if(238===i.kind&&e.nodeIsDecorated(i))for(r=e.getContainingClass(n);void 0!==r;){if(r===i&&r.name!==n){Rt(i).flags|=8388608,Rt(n).flags|=16777216;break}r=e.getContainingClass(r)}else if(207===i.kind)for(r=e.getThisContainer(n,!1);277!==r.kind;){if(r.parent===i){152===r.kind&&e.hasModifier(r,32)&&(Rt(i).flags|=8388608,Rt(n).flags|=16777216);break}r=e.getThisContainer(r,!1)}!function(n,t){if(!(C>=2||0==(34&t.flags)||272===t.valueDeclaration.parent.kind)){for(var r=e.getEnclosingBlockScopeContainer(t.valueDeclaration),a=function(n,t){return!!e.findAncestor(n,function(n){return n===t?\"quit\":e.isFunctionLike(n)})}(n.parent,r),i=r,o=!1;i&&!e.nodeStartsNewLexicalEnvironment(i);){if(e.isIterationStatement(i,!1)){o=!0;break}i=i.parent}o&&(a&&(Rt(i).flags|=65536),223===r.kind&&e.getAncestor(t.valueDeclaration,236).parent===r&&function(n,t){for(var r=n;193===r.parent.kind;)r=r.parent;var a=!1;if(e.isAssignmentTarget(r))a=!0;else if(200===r.parent.kind||201===r.parent.kind){var i=r.parent;a=43===i.operator||44===i.operator}return!!a&&!!e.findAncestor(r,function(e){return e===t?\"quit\":e===t.statement})}(n,r)&&(Rt(t.valueDeclaration).flags|=2097152),Rt(t.valueDeclaration).flags|=262144),a&&(Rt(t.valueDeclaration).flags|=131072)}}(n,t);var o=Qd(Ga(a),n),s=e.getAssignmentTargetKind(n);if(s){if(!(3&a.flags||e.isInJSFile(n)&&512&a.flags))return bt(n,e.Diagnostics.Cannot_assign_to_0_because_it_is_not_a_variable,Jr(t)),te;if(Zf(a))return bt(n,e.Diagnostics.Cannot_assign_to_0_because_it_is_a_constant_or_a_read_only_property,Jr(t)),te}var l=2097152&a.flags;if(3&a.flags){if(1===s)return o}else{if(!l)return o;i=e.find(t.declarations,d)}if(!i)return o;for(var c=149===e.getRootDeclaration(i).kind,u=Wd(i),m=Wd(n),p=m!==u,f=n.parent&&n.parent.parent&&e.isSpreadAssignment(n.parent)&&vd(n.parent.parent),g=134217728&t.flags;m!==u&&(194===m.kind||195===m.kind||e.isObjectLiteralOrClassExpressionMethod(m))&&(Jd(a)||c&&!qd(a));)m=Wd(m);var _=c||l||p||f||g||o!==ee&&o!==qe&&(!M||0!=(3&o.flags)||td(n)||255===n.parent.kind)||211===n.parent.kind||235===i.kind&&i.exclamationToken||4194304&i.flags,v=jd(n,o,_?c?function(e,n){return M&&149===n.kind&&n.initializer&&8192&vu(e)&&!(8192&vu(Lg(n.initializer)))?md(e,131072):e}(o,i):o:o===ee||o===qe?ae:Eu(o),m,!_);if(o===ee||o===qe){if(v===ee||v===qe)return R&&(bt(e.getNameOfDeclaration(i),e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined,Jr(t),Yr(v)),bt(n,e.Diagnostics.Variable_0_implicitly_has_an_1_type,Jr(t),Yr(v))),E_(v)}else if(!_&&!(8192&vu(o))&&8192&vu(v))return bt(n,e.Diagnostics.Variable_0_is_used_before_being_assigned,Jr(t)),o;return s?cu(v):v}function em(e,n){Rt(e).flags|=2,152===n.kind||155===n.kind?Rt(n.parent).flags|=4:Rt(n).flags|=4}function nm(n){return e.isSuperCall(n)?n:e.isFunctionLike(n)?void 0:e.forEachChild(n,nm)}function tm(e){var n=Rt(e);return void 0===n.hasSuperCall&&(n.superCall=nm(e.body),n.hasSuperCall=!!n.superCall),n.superCall}function rm(e){return Qa(si(Er(e)))===se}function am(n,t,r){var a=t.parent;if(e.getEffectiveBaseTypeNode(a)&&!rm(a)){var i=tm(t);(!i||i.end>n.pos)&&bt(n,r)}}function im(n){var t=e.getThisContainer(n,!0),r=!1;switch(155===t.kind&&am(n,t,e.Diagnostics.super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class),195===t.kind&&(t=e.getThisContainer(t,!1),r=!0),t.kind){case 242:bt(n,e.Diagnostics.this_cannot_be_referenced_in_a_module_or_namespace_body);break;case 241:bt(n,e.Diagnostics.this_cannot_be_referenced_in_current_location);break;case 155:sm(n,t)&&bt(n,e.Diagnostics.this_cannot_be_referenced_in_constructor_arguments);break;case 152:case 151:e.hasModifier(t,32)&&bt(n,e.Diagnostics.this_cannot_be_referenced_in_a_static_property_initializer);break;case 147:bt(n,e.Diagnostics.this_cannot_be_referenced_in_a_computed_property_name)}r&&C<2&&em(n,t);var a=om(n,t);return!a&&N&&bt(n,r&&277===t.kind?e.Diagnostics.The_containing_arrow_function_captures_the_global_value_of_this_which_implicitly_has_type_any:e.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation),a||$}function om(n,t){void 0===t&&(t=e.getThisContainer(n,!1));var r=e.isInJSFile(n);if(e.isFunctionLike(t)&&(!pm(n)||e.getThisParameter(t))){var a=function(n){return 194===n.kind&&e.isBinaryExpression(n.parent)&&3===e.getAssignmentDeclarationKind(n.parent)?n.parent.left.expression.expression:154===n.kind&&186===n.parent.kind&&e.isBinaryExpression(n.parent.parent)&&6===e.getAssignmentDeclarationKind(n.parent.parent)?n.parent.parent.left.expression:194===n.kind&&273===n.parent.kind&&186===n.parent.parent.kind&&e.isBinaryExpression(n.parent.parent.parent)&&6===e.getAssignmentDeclarationKind(n.parent.parent.parent)?n.parent.parent.parent.left.expression:void 0}(t);if(r&&a){var i=Lg(a).symbol;if(i&&i.members&&16&i.flags&&(o=vf(i)))return jd(n,o)}else if(r&&(194===t.kind||237===t.kind)&&e.getJSDocClassTag(t)){var o;if(o=vf(t.symbol))return jd(n,o)}var s=Ra(t)||dm(t);if(s)return jd(n,s)}if(e.isClassLike(t.parent)){var l,c=Er(t.parent);return jd(n,l=e.hasModifier(t,32)?Ga(c):si(c).thisType)}if(r&&(l=function(n){var t=e.getJSDocType(n);if(t&&287===t.kind){var r=t;if(r.parameters.length>0&&r.parameters[0].name&&\"this\"===r.parameters[0].name.escapedText)return Ul(r.parameters[0].type)}var a=e.getJSDocThisTag(n);if(a&&a.typeExpression)return Ul(a.typeExpression)}(t))&&l!==te)return jd(n,l)}function sm(n,t){return!!e.findAncestor(n,function(e){return e===t?\"quit\":149===e.kind})}function lm(n){var t=189===n.parent.kind&&n.parent.expression===n,r=e.getSuperContainer(n,!0),a=!1;if(!t)for(;r&&195===r.kind;)r=e.getSuperContainer(r,!0),a=C<2;var i=0;if(!function(n){return!!n&&(t?155===n.kind:!(!e.isClassLike(n.parent)&&186!==n.parent.kind)&&(e.hasModifier(n,32)?154===n.kind||153===n.kind||156===n.kind||157===n.kind:154===n.kind||153===n.kind||156===n.kind||157===n.kind||152===n.kind||151===n.kind||155===n.kind))}(r)){var o=e.findAncestor(n,function(e){return e===r?\"quit\":147===e.kind});return o&&147===o.kind?bt(n,e.Diagnostics.super_cannot_be_referenced_in_a_computed_property_name):t?bt(n,e.Diagnostics.Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors):r&&r.parent&&(e.isClassLike(r.parent)||186===r.parent.kind)?bt(n,e.Diagnostics.super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_derived_class):bt(n,e.Diagnostics.super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions),te}if(t||155!==r.kind||am(n,r,e.Diagnostics.super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class),i=e.hasModifier(r,32)||t?512:256,Rt(n).flags|=i,154===r.kind&&e.hasModifier(r,256)&&(e.isSuperProperty(n.parent)&&e.isAssignmentTarget(n.parent)?Rt(r).flags|=4096:Rt(r).flags|=2048),a&&em(n.parent,r),186===r.parent.kind)return C<2?(bt(n,e.Diagnostics.super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_higher),te):$;var s=r.parent;if(!e.getEffectiveBaseTypeNode(s))return bt(n,e.Diagnostics.super_can_only_be_referenced_in_a_derived_class),te;var l=si(Er(s)),c=l&&Za(l)[0];return c?155===r.kind&&sm(n,r)?(bt(n,e.Diagnostics.super_cannot_be_referenced_in_constructor_arguments),te):512===i?Qa(l):Ai(c,l.thisType):te}function cm(n){return 4&e.getObjectFlags(n)&&n.target===je?n.typeArguments[0]:void 0}function um(n){return Md(n,function(n){return 524288&n.flags?e.forEach(n.types,cm):cm(n)})}function dm(n){if(195!==n.kind){if(fc(n)){var t=Nm(n);if(t){var r=t.thisParameter;if(r)return Ga(r)}}var a=e.isInJSFile(n);if(N||a){var i=function(e){return 154!==e.kind&&156!==e.kind&&157!==e.kind||186!==e.parent.kind?194===e.kind&&273===e.parent.kind?e.parent.parent:void 0:e.parent}(n);if(i){for(var o=xm(i),s=i,l=o;l;){var c=um(l);if(c)return lc(c,Cm(i));if(273!==s.parent.kind)break;l=xm(s=s.parent.parent)}return o?Tu(o):gg(i)}var u=n.parent;if(202===u.kind&&58===u.operatorToken.kind){var d=u.left;if(187===d.kind||188===d.kind){var m=d.expression;if(a&&e.isIdentifier(m)){var p=e.getSourceFileOfNode(u);if(p.commonJsModuleIndicator&&nd(m)===p.symbol)return}return gg(m)}}}}}function mm(n){var t=n.parent;if(fc(t)){var r=e.getImmediatelyInvokedFunctionExpression(t);if(r&&r.arguments){var a=Yp(r),i=t.parameters.indexOf(n);if(n.dotDotDotToken)return Wp(a,i,a.length,$,void 0);var o=Rt(r),s=o.resolvedSignature;o.resolvedSignature=gn;var l=i<a.length?uu(Lg(a[i])):n.initializer?void 0:ie;return o.resolvedSignature=s,l}var c=Nm(t);if(c){var u=t.parameters.indexOf(n)-(e.getThisParameter(t)?1:0);return n.dotDotDotToken&&e.lastOrUndefined(t.parameters)===n?If(c,u):Of(c,u)}}}function pm(n){for(var t=!1;n.parent&&!e.isFunctionLike(n.parent);){if(e.isParameter(n.parent)&&(t||n.parent.initializer===n))return!0;e.isBindingElement(n.parent)&&n.parent.initializer===n&&(t=!0),n=n.parent}return!1}function fm(e){var n=Uo(e);if(n)return n;var t=Im(e);return t&&!function(e){return!e.resolvedReturnType&&oa(e,3)>=0}(t)?Ho(t):void 0}function gm(e,n){var t=Yp(e).indexOf(n);return-1===t?void 0:_m(e,t)}function _m(e,n){return Mf(Rt(e).resolvedSignature===vn?vn:ff(e),n)}function vm(n){var t=n.parent,r=t.left,a=t.operatorToken,i=t.right;switch(a.kind){case 58:if(n!==i)return;var o=function(n){var t=e.getAssignmentDeclarationKind(n);switch(t){case 0:return!0;case 5:case 1:case 6:case 3:if(n.left.symbol){var r=n.left.symbol.valueDeclaration;if(!r)return!1;var a=n.left,i=e.getEffectiveTypeAnnotationNode(r);if(i)return Ul(i);if(e.isIdentifier(a.expression)){var o=a.expression,s=Ft(o,o.escapedText,67220415,void 0,o.escapedText,!0);if(s){var l=e.getEffectiveTypeAnnotationNode(s.valueDeclaration);if(l){var c=ym(Ul(l),a.name.escapedText);return c||!1}return!1}}return!e.isInJSFile(r)}return!0;case 2:case 4:if(!n.symbol)return!0;if(n.symbol.valueDeclaration){var l=e.getEffectiveTypeAnnotationNode(n.symbol.valueDeclaration);if(l){var c=Ul(l);if(c)return c}}if(2===t)return!1;var u=n.left;if(!e.isObjectLiteralMethod(e.getThisContainer(u.expression,!1)))return!1;var d=im(u.expression);return d&&ym(d,u.name.escapedText)||!1;default:return e.Debug.assertNever(t)}}(t);if(!o)return;return!0===o?Tg(r):o;case 54:var s=Am(t);return s||n!==i||e.isDefaultedExpandoInitializer(t)?s:Tg(r);case 53:case 26:return n===i?Am(t):void 0;default:return}}function ym(e,n){return Md(e,function(e){if(917504&e.flags){var t=vo(e,n);if(t)return Ga(t);if(pu(e)){var r=fu(e);if(r&&Km(n)&&+n>=0)return r}}},!0)}function hm(e,n){return Md(e,function(e){return Eo(e,n)},!0)}function bm(e){var n=xm(e.parent);if(n){if(!bi(e)){var t=ym(n,Er(e).escapedName);if(t)return t}return Gm(e.name)&&hm(n,1)||hm(n,0)}}function Em(e,n){return e&&(ym(e,\"\"+n)||hm(e,1)||I_(e,void 0,!1,!1,!1))}function Tm(n){var t=n.parent;return e.isJsxAttributeLike(t)?Am(n):e.isJsxElement(t)?function(e){var n=xm(e.openingElement.tagName),t=tp(ep(e));return n&&!da(n)&&t&&\"\"!==t?ym(n,t):void 0}(t):void 0}function Sm(n){if(e.isJsxAttribute(n)){var t=xm(n.parent);if(!t||da(t))return;return ym(t,n.name.escapedText)}return Am(n.parent)}function Lm(e){switch(e.kind){case 9:case 8:case 13:case 101:case 86:case 95:case 71:return!0;case 187:case 193:return Lm(e.expression)}return!1}function xm(n){var t,r=Am(n);if(!((r=r&&Md(r,po))&&262144&r.flags&&e.isObjectLiteralExpression(n)))return r;e:for(var a=0,i=n.properties;a<i.length;a++){var o=i[a];if(o.symbol&&273===o.kind&&Lm(o.initializer)&&ld(r,o.symbol.escapedName))for(var s=Lg(o.initializer),l=0,c=r.types;l<c.length;l++){var u=c[l],d=ua(u,o.symbol.escapedName);if(d&&bc(s,d)){if(t){if(u===t)continue;t=void 0;break e}t=u}}}return t||r}function Am(t){if(!(8388608&t.flags)){if(t.contextualType)return t.contextualType;var r=t.parent;switch(r.kind){case 235:case 149:case 152:case 151:case 184:return function(n){var t=n.parent;if(e.hasInitializer(t)&&n===t.initializer){var r=e.getEffectiveTypeAnnotationNode(t);if(r)return Ul(r);if(149===t.kind){var a=mm(t);if(a)return a}if(e.isBindingPattern(t.name))return Aa(t.name,!0,!1);if(e.isBindingPattern(t.parent)){var i=t.parent.parent,o=t.propertyName||t.name;if(184!==i.kind){var s=e.getEffectiveTypeAnnotationNode(i);if(s&&!e.isBindingPattern(o)){var l=e.getTextOfPropertyName(o);if(l)return ua(Ul(s),l)}}}}}(t);case 195:case 228:return function(n){var t=e.getContainingFunction(n);if(t){var r=e.getFunctionFlags(t);if(1&r)return;var a=fm(t);return 2&r?a&&Wg(a):a}}(t);case 205:return function(n){var t=e.getContainingFunction(n);if(t){var r=e.getFunctionFlags(t),a=fm(t);if(a)return n.asteriskToken?a:P_(a,0!=(2&r))}}(r);case 189:case 190:return gm(r,t);case 192:case 210:return Ul(r.type);case 202:return vm(t);case 273:case 274:return bm(r);case 275:return xm(r.parent);case 185:var a=r;return Em(xm(a),e.indexOfNode(a.elements,t));case 203:return function(e){var n=e.parent;return e===n.whenTrue||e===n.whenFalse?Am(n):void 0}(t);case 214:return e.Debug.assert(204===r.parent.kind),function(e,n){if(191===e.parent.kind)return gm(e.parent,n)}(r.parent,t);case 193:var i=e.isInJSFile(r)?e.getJSDocTypeTag(r):void 0;return i?Ul(i.typeExpression.type):Am(r);case 268:return Tm(r);case 265:case 267:return Sm(r);case 260:case 259:return function(t){if(e.isJsxOpeningElement(t)&&t.parent.contextualType)return t.parent.contextualType;if(qm(t.tagName))return op(t);var r=Lg(t.tagName);if(da(r))return $;var a=e.isInJSFile(t);return Md(r,function(r){return function(t,r,a){if(4&t.flags)return $;if(64&t.flags){var i=Ym(n.IntrinsicElements,a);if(i!==te){var o=t.value,s=vo(i,e.escapeLeadingUnderscores(o));if(s)return Ga(s);var l=So(i,0);if(l)return l}return $}var c=ho(t,1),u=!0;if(0===c.length&&(c=ho(t,0),u=!1,0===c.length))return te;var d=Rt(a);d.resolvedSignatures||(d.resolvedSignatures=e.createMap());var m=\"\"+Xs(t),p=d.resolvedSignatures.get(m);return p&&p!==En?c=p:p||(d.resolvedSignatures.set(m,En),d.resolvedSignatures.set(m,c=Zm(a,c))),tl(e.map(c,u?function(e){return km(e,r,a,!1)}:function(e){return function(e,t){var r=Vf(e,Se);r=Dm(t,ep(t),r);var a=Ym(n.IntrinsicAttributes,t);return a!==te&&(r=Ni(a,r)),r}(e,a)}),0)}(r,a,t)})}(r)}}}function Cm(n){var t=e.findAncestor(n,function(e){return!!e.contextualMapper});return t?t.contextualMapper:x}function Dm(t,r,a){var i,o=(i=r)&&wt(i.exports,n.LibraryManagedAttributes,67897832);if(o){var s=si(o);if(e.length(s.typeParameters)>=2)return ss(s,l=No([gg(t.tagName),a],s.typeParameters,2,e.isInJSFile(t)));if(e.length(s.aliasTypeArguments)>=2){var l=No([gg(t.tagName),a],s.aliasTypeArguments,2,e.isInJSFile(t));return ds(s.aliasSymbol,l)}}return a}function km(t,r,a,i){var o,s=ep(a),l=(o=s,np(n.ElementAttributesPropertyNameContainer,o)),c=void 0===l?Vf(t,Se):\"\"===l?Ho(t):function(e,n){var t=Ho(e);return da(t)?t:ua(t,n)}(t,l);if(!c)return i&&l&&e.length(a.attributes.properties)&&bt(a,e.Diagnostics.JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property,e.unescapeLeadingUnderscores(l)),Se;if(da(c=Dm(a,s,c)))return c;var u=c,d=Ym(n.IntrinsicClassAttributes,a);if(d!==te){var m=Wa(d.symbol),p=Ho(t);u=Ni(m?ss(d,No([p],m,Ro(m),r)):d,u)}var f=Ym(n.IntrinsicAttributes,a);return f!==te&&(u=Ni(f,u)),u}function Mm(n,t){var r=ho(n,0);if(1===r.length){var a=r[0];if(!function(n,t){for(var r=0;r<t.parameters.length;r++){var a=t.parameters[r];if(a.initializer||a.questionToken||a.dotDotDotToken||Co(a))break}return t.parameters.length&&e.parameterIsThisKeyword(t.parameters[0])&&r--,!wf(n)&&Rf(n)<r}(a,t))return a}}function Om(e){return 194===e.kind||195===e.kind}function Im(n){return Om(n)||e.isObjectLiteralMethod(n)?Nm(n):void 0}function Rm(n){return e.isObjectLiteralMethod(n)?function(n){if(e.Debug.assert(e.isObjectLiteralMethod(n)),!(8388608&n.flags))return bm(n)}(n):xm(n)}function Nm(n){e.Debug.assert(154!==n.kind||e.isObjectLiteralMethod(n));var t=Po(n);if(t)return t;var r=Rm(n);if(r){if(!(262144&r.flags))return Mm(r,n);for(var a,i,o=0,s=r.types;o<s.length;o++){var l=Mm(s[o],n);if(l)if(a){if(!Zc(a[0],l,!1,!0,!0,vc))return;a.push(l)}else a=[l]}return a&&((i=ki(a[0])).unionSignatures=a),i}}function wm(e){return 184===e.kind&&!!e.initializer||202===e.kind&&58===e.operatorToken.kind}function Pm(n,t,r){for(var a=n.elements,i=a.length,o=!1,s=[],l=e.isAssignmentTarget(n),c=xm(n),u=0;u<i;u++){var d=a[u];if(l&&206===d.kind){var m=Lg(d.expression,t,r),p=So(m,1)||I_(m,void 0,!1,!1,!1);p&&s.push(p)}else{var f=yg(d,t,Em(c,u),r);s.push(f)}u<i-1&&206===d.kind&&(o=!0)}if(!o){var g=i>0&&206===a[i-1].kind,_=i-(g?1:0);if(l&&_>0)return(f=ls(Js(s,_,g))).pattern=n,f;if(c&&function(n){return!!(262144&n.flags?e.forEach(n.types,au):au(n))}(c)){var v=c.pattern;if(!g&&v&&(183===v.kind||185===v.kind))for(var y=v.elements,h=i;h<y.length;h++)wm(d=y[h])?s.push(c.typeArguments[h]):(h<y.length-1||!(184===d.kind&&d.dotDotDotToken||206===d.kind))&&(208!==d.kind&&bt(d,e.Diagnostics.Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value),s.push(M?he:ie));return Js(s,_,g)}if(r)return Js(s,_,g)}return Fm(s,2)}function Fm(e,n){return void 0===n&&(n=1),Ws(e.length?tl(e,n):M?he:ie)}function Gm(e){switch(e.kind){case 147:return function(e){return rg(Bm(e),168)}(e);case 71:return Km(e.escapedText);case 8:case 9:return Km(e.text);default:return!1}}function Vm(e){return\"Infinity\"===e||\"-Infinity\"===e||\"NaN\"===e}function Km(e){return(+e).toString()===e}function Bm(n){var t=Rt(n.expression);return t.resolvedType||(t.resolvedType=Lg(n.expression),24576&t.resolvedType.flags||!rg(t.resolvedType,3308)&&!bc(t.resolvedType,Ee)?bt(n,e.Diagnostics.A_computed_property_name_must_be_of_type_string_number_symbol_or_any):Rp(n.expression,t.resolvedType,!0)),t.resolvedType}function Hm(e,n,t,r){for(var a=[],i=0;i<t.length;i++)(0===r||Gm(e[i+n].name))&&a.push(Ga(t[i]));return ns(a.length?tl(a,2):ae,!1)}function Um(n,t){var r,a=e.isAssignmentTarget(n);!function(n,t){var r;!function(e){e[e.Property=1]=\"Property\",e[e.GetAccessor=2]=\"GetAccessor\",e[e.SetAccessor=4]=\"SetAccessor\",e[e.GetOrSetAccessor=6]=\"GetOrSetAccessor\"}(r||(r={}));for(var a=e.createUnderscoreEscapedMap(),i=0,o=n.properties;i<o.length;i++){var s=o[i];if(275!==s.kind){var l=s.name;if(147===l.kind&&vy(l),274===s.kind&&!t&&s.objectAssignmentInitializer)return Oy(s.equalsToken,e.Diagnostics.can_only_be_used_in_an_object_literal_property_inside_a_destructuring_assignment);if(s.modifiers)for(var c=0,u=s.modifiers;c<u.length;c++){var d=u[c];120===d.kind&&154===s.kind||Oy(d,e.Diagnostics._0_modifier_cannot_be_used_here,e.getTextOfNode(d))}var m=void 0;switch(s.kind){case 274:by(s.exclamationToken,e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context);case 273:hy(s.questionToken,e.Diagnostics.An_object_member_cannot_be_declared_optional),8===l.kind&&Ry(l);case 154:m=1;break;case 156:m=2;break;case 157:m=4;break;default:throw e.Debug.assertNever(s,\"Unexpected syntax kind:\"+s.kind)}var p=e.getPropertyNameForPropertyNameNode(l);if(void 0!==p){var f=a.get(p);if(f)if(1===m&&1===f)Oy(l,e.Diagnostics.Duplicate_identifier_0,e.getTextOfNode(l));else{if(!(6&m&&6&f))return Oy(l,e.Diagnostics.An_object_literal_cannot_have_property_and_accessor_with_the_same_name);if(6===f||m===f)return Oy(l,e.Diagnostics.An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name);a.set(p,m|f)}else a.set(p,m)}}}}(n,a);var i=[],o=Se,s=33554432,l=xm(n),c=l&&l.pattern&&(182===l.pattern.kind||186===l.pattern.kind),u=e.isInJSFile(n)&&!e.isInJsonFile(n),d=e.getJSDocEnumTag(n),m=!l&&u&&!d,p=0,f=!1,g=!1,_=!1;r=e.createSymbolTable();for(var v=0,y=0;y<n.properties.length;y++){var h=n.properties[y],b=Er(h),E=h.name&&147===h.name.kind&&!e.isWellKnownSymbolSyntactically(h.name.expression)?Bm(h.name):void 0;if(273===h.kind||274===h.kind||e.isObjectLiteralMethod(h)){var T=273===h.kind?hg(h,t):274===h.kind?yg(h.name,t):bg(h,t);if(u){var S=_a(h);S?(Lc(T,S,h),T=S):d&&d.typeExpression&&Lc(T,Ul(d.typeExpression),h)}p|=T.flags;var L=E&&2240&E.flags?E:void 0,x=L?Tt(4|b.flags,Ei(L),1024):Tt(4|b.flags,b.escapedName);if(L&&(x.nameType=L),a)(273===h.kind&&wm(h.initializer)||274===h.kind&&h.objectAssignmentInitializer)&&(x.flags|=16777216);else if(c&&!(512&e.getObjectFlags(l))){var D=vo(l,b.escapedName);D?x.flags|=16777216&D.flags:A.suppressExcessPropertyErrors||To(l,0)||bt(h.name,e.Diagnostics.Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1,Jr(b),Yr(l))}x.declarations=b.declarations,x.parent=b.parent,b.valueDeclaration&&(x.valueDeclaration=b.valueDeclaration),x.type=T,x.target=b,b=x}else{if(275===h.kind){if(C<2&&iy(h,2),i.length>0&&(o=Rl(o,O(),n.symbol,s,0),i=[],r=e.createSymbolTable(),g=!1,_=!1,p=0),!jm(T=Lg(h.expression)))return bt(h,e.Diagnostics.Spread_types_may_only_be_created_from_object_types),te;o=Rl(o,T,n.symbol,s,0),v=y+1;continue}e.Debug.assert(156===h.kind||157===h.kind),uv(h)}!E||2240&E.flags?r.set(b.escapedName,b):bc(E,Ee)&&(bc(E,ce)?_=!0:g=!0,a&&(f=!0)),i.push(b)}if(c)for(var k=0,M=$i(l);k<M.length;k++)x=M[k],r.get(x.escapedName)||o&&vo(o,x.escapedName)||(16777216&x.flags||bt(x.valueDeclaration||x.bindingElement,e.Diagnostics.Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value),r.set(x.escapedName,x),i.push(x));return o!==Se?(i.length>0&&(o=Rl(o,O(),n.symbol,s,0)),o):O();function O(){var t=g?Hm(n.properties,v,i,0):void 0,o=_?Hm(n.properties,v,i,1):void 0,l=wr(n.symbol,r,e.emptyArray,e.emptyArray,t,o),c=A.suppressExcessPropertyErrors?0:33554432;return l.flags|=268435456|c|939524096&p,l.objectFlags|=128,m&&(l.objectFlags|=16384),f&&(l.objectFlags|=512),a&&(l.pattern=n),24576&l.flags||(s|=939524096&l.flags),l}}function jm(n){return!!(16777219&n.flags||29120&vu(n)&&jm(yu(n))||131072&n.flags&&!Ji(n)||786432&n.flags&&e.every(n.types,jm))}function Wm(n){return!e.stringContains(n,\"-\")}function qm(n){return 71===n.kind&&e.isIntrinsicJsxName(n.escapedText)}function zm(e,n){return e.initializer?yg(e.initializer,n):me}function Jm(e,n){for(var t=[],r=0,a=e.children;r<a.length;r++){var i=a[r];10===i.kind?i.containsOnlyWhiteSpaces||t.push(le):t.push(yg(i,n))}return t}function Xm(n,t){return function(n,t){for(var r,a=n.attributes,i=e.createSymbolTable(),o=Se,s=!1,l=!1,c=0,u=tp(ep(n)),d=0,m=a.properties;d<m.length;d++){var p=m[d],f=p.symbol;if(e.isJsxAttribute(p)){var g=zm(p,t);c|=939524096&g.flags;var _=Tt(33554436|f.flags,f.escapedName);_.declarations=f.declarations,_.parent=f.parent,f.valueDeclaration&&(_.valueDeclaration=f.valueDeclaration),_.type=g,_.target=f,i.set(_.escapedName,_),p.name.escapedText===u&&(l=!0)}else e.Debug.assert(267===p.kind),i.size>0&&(o=Rl(o,E(),a.symbol,c,4096),i=e.createSymbolTable()),da(g=gg(p.expression,t))&&(s=!0),jm(g)?o=Rl(o,g,n.symbol,c,4096):r=r?ll([r,g]):g}s||i.size>0&&(o=Rl(o,E(),a.symbol,c,4096));var v=258===n.parent.kind?n.parent:void 0;if(v&&v.openingElement===n&&v.children.length>0){var y=Jm(v,t);if(!s&&u&&\"\"!==u){l&&bt(a,e.Diagnostics._0_are_specified_twice_The_attribute_named_0_will_be_overwritten,e.unescapeLeadingUnderscores(u));var h=Tt(33554436,u);h.type=1===y.length?y[0]:Ws(tl(y));var b=e.createSymbolTable();b.set(u,h),o=Rl(o,wr(a.symbol,b,e.emptyArray,e.emptyArray,void 0,void 0),a.symbol,c,4096)}}return s?$:r&&o!==Se?ll([r,o]):r||(o===Se?E():o);function E(){var n=wr(a.symbol,i,e.emptyArray,e.emptyArray,void 0,void 0);return n.flags|=c|=268435456,n.objectFlags|=4224,n}}(n.parent,t)}function Ym(e,n){var t=ep(n),r=t&&_r(t),a=r&&wt(r,e,67897832);return a?si(a):te}function Qm(t){var r=Rt(t);if(!r.resolvedSymbol){var a=Ym(n.IntrinsicElements,t);if(a!==te){if(!e.isIdentifier(t.tagName))return e.Debug.fail();var i=vo(a,t.tagName.escapedText);return i?(r.jsxFlags|=1,r.resolvedSymbol=i):So(a,0)?(r.jsxFlags|=2,r.resolvedSymbol=a.symbol):(bt(t,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.idText(t.tagName),\"JSX.\"+n.IntrinsicElements),r.resolvedSymbol=Q)}return R&&bt(t,e.Diagnostics.JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists,e.unescapeLeadingUnderscores(n.IntrinsicElements)),r.resolvedSymbol=Q}return r.resolvedSymbol}function Zm(n,t){for(var r,a=[],i=!!n.typeArguments,o=0,s=t;o<s.length;o++){var l=s[o];if(l.typeParameters){var c=e.isInJSFile(n),u=$m(l,n,c,!1);if(u)i=!1,a.push(u);else{n.typeArguments&&Kp(l,n.typeArguments)&&(r=l);var d=Up(l,n,wu(l.typeParameters,l,c?2:0));a.push(Wo(l,d,c))}}else a.push(l)}return n.typeArguments&&i&&(r?qp(r,n.typeArguments,!0):0!==n.typeArguments.length&&Xn.add($p(n,t,n.typeArguments))),a}function $m(e,n,t,r){if(void 0===r&&(r=!1),n.typeArguments&&Kp(e,n.typeArguments)){var a=qp(e,n.typeArguments,r);if(a)return Wo(e,a,t)}}function ep(e){var t=e&&Rt(e);if(t&&t.jsxNamespace)return t.jsxNamespace;if(!t||!1!==t.jsxNamespace){var r=yt(e),a=Ft(e,r,1920,void 0,r,!1);if(a){var i=wt(_r($t(a)),n.JSX,1920);if(i)return t&&(t.jsxNamespace=i),i;t&&(t.jsxNamespace=!1)}}return Cs(n.JSX,1920,void 0)}function np(n,t){var r=t&&wt(t.exports,n,67897832),a=r&&si(r),i=a&&$i(a);if(i){if(0===i.length)return\"\";if(1===i.length)return i[0].escapedName;i.length>1&&bt(r.declarations[0],e.Diagnostics.The_global_type_JSX_0_may_not_have_more_than_one_property,e.unescapeLeadingUnderscores(n))}}function tp(e){return np(n.ElementChildrenAttributeNameContainer,e)}function rp(e){if(e){if(524288&e.flags){for(var n=[],t=0,r=e.types;t<r.length;t++){var a=r[t];n.push(po(a))}return ll(n)}return po(e)}}function ap(n,t,r){var a=Rt(n);a.resolvedSignatures||(a.resolvedSignatures=e.createMap());var i=\"\"+Xs(t);if(!a.resolvedSignatures.get(i)||a.resolvedSignatures.get(i)!==En){if(a.resolvedSignatures.get(i))return a.resolvedSignatures.get(i);a.resolvedSignatures.set(i,En);var o=ho(t,1);if(0!==o.length||0!==(o=ho(t,0)).length){var s=Zm(n,o);return a.resolvedSignatures.set(i,s),s}r&&bt(n.tagName,e.Diagnostics.JSX_element_type_0_does_not_have_any_construct_or_call_signatures,e.getTextOfNode(n.tagName))}}function ip(t,r,a,i){if(262144&a.flags)return tl(a.types.map(function(e){return ip(t,r,e,i)}),2);if(da(a))return a;if(4&a.flags)return $;if(64&a.flags){var o=Ym(n.IntrinsicElements,t);if(o!==te){var s=a.value,l=vo(o,e.escapeLeadingUnderscores(s));if(l)return Ga(l);var c=So(o,0);if(c)return c;bt(t,e.Diagnostics.Property_0_does_not_exist_on_type_1,s,\"JSX.\"+n.IntrinsicElements)}return $}var u=ap(t,a,!0);if(!e.length(u))return te;var d=tl(u.map(Ho),2),m=r?function(t,r,a,i){if(e.Debug.assert(!(262144&r.flags)),!i||!bc(a,i)){var o=up(t);if(o){var s=[];mf(t,r,s,!1);for(var l=void 0,c=void 0,u=0,d=s;u<d.length;u++){var m=d[u],p=Ho(m),f=p&&(0===m.parameters.length?Se:Ga(m.parameters[0]));if(f=rp(f),p&&bc(p,o)){for(var g=!0,_=0,v=t.attributes.properties;_<v.length;_++){var y=v[_];if(e.isJsxAttribute(y)&&Wm(y.name.escapedText)&&!vo(f,y.name.escapedText)){g=!1;break}}g&&(l=Ni(l,f)),c=Ni(c,f)}}l||(l=c);var h=Ym(n.IntrinsicAttributes,t);return h!==te&&(l=Ni(h,l)),l}}}(t,a,d,i):function(t,r,a,i){if(e.Debug.assert(!(262144&r.flags)),!i||!bc(a,i)){var o=up(t);if(o){var s=mf(t,r,void 0,!1);if(s!==_n){var l=s&&Ho(s),c=l&&(0===s.parameters.length?Se:Ga(s.parameters[0]));if(c=rp(c),l&&bc(l,o)){var u=Ym(n.IntrinsicAttributes,t);return u!==te&&(c=Ni(u,c)),c}}}}}(t,a,d,i);if(m)return m;i&&Vc(d,i,dt,t,e.Diagnostics.JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements);var p=e.isInJSFile(t);return tl(u.map(function(e){return km(e,p,t,!0)}))}function op(n){e.Debug.assert(qm(n.tagName));var t=Rt(n);if(!t.resolvedJsxElementAttributesType){var r=Qm(n);return 1&t.jsxFlags?t.resolvedJsxElementAttributesType=Ga(r):2&t.jsxFlags?t.resolvedJsxElementAttributesType=ts(r,0).type:t.resolvedJsxElementAttributesType=te}return t.resolvedJsxElementAttributesType}function sp(e,t){return ip(e,t,Lg(e.tagName),(r=e,(a=Ym(n.ElementClass,r))===te?void 0:a));var r,a}function lp(e){var n;return vo(qm((n=e.parent.parent).tagName)?op(n):sp(n,!1),e.name.escapedText)||Q}function cp(e){return Ym(n.Element,e)}function up(e){var n=cp(e);if(n)return tl([n,oe])}function dp(n,t){var r,a=e.isJsxOpeningLikeElement(n);a&&function(n){py(n,n.typeArguments);for(var t=e.createUnderscoreEscapedMap(),r=0,a=n.attributes.properties;r<a.length;r++){var i=a[r];if(267!==i.kind){var o=i.name,s=i.initializer;if(t.get(o.escapedText))return Oy(o,e.Diagnostics.JSX_elements_cannot_have_multiple_attributes_with_the_same_name);if(t.set(o.escapedText,!0),s&&268===s.kind&&!s.expression)return Oy(s,e.Diagnostics.JSX_attributes_must_only_be_assigned_a_non_empty_expression)}}}(n),r=n,0===(A.jsx||0)&&bt(r,e.Diagnostics.Cannot_use_JSX_unless_the_jsx_flag_is_provided),void 0===cp(r)&&R&&bt(r,e.Diagnostics.JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist);var i=Xn&&2===A.jsx?e.Diagnostics.Cannot_find_name_0:void 0,o=yt(n),s=Ft(a?n.tagName:n,o,67220415,i,o,!0);s&&(s.isReferenced=67108863,2097152&s.flags&&!Vv(er(s))&&tr(s)),a?function(n,t){var r=qm(n.tagName)?op(n):sp(n,!1),a=gg(n.attributes,t),i=bc(a,r);if(!i||da(a)||da(r))i||xc(a,r,n.tagName,n.attributes);else for(var o=0,s=n.attributes.properties;o<s.length;o++){var l=s[o];if(e.isJsxAttribute(l)){var c=l.name,u=Wm(e.idText(c))||!!vo(r,c.escapedText);if(u&&!mp(r,c.escapedText,!0)){bt(l,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.idText(c),Yr(r));break}}}}(n,t):Jm(n.parent)}function mp(e,n,t){if(131072&e.flags){var r=Xi(e);if(r.stringIndexInfo||r.numberIndexInfo&&Km(n)||Qi(e,n)||t&&!Wm(n))return!0}else if(786432&e.flags){for(var a=0,i=e.types;a<i.length;a++)if(mp(i[a],n,t))return!0}else if(4194304&e.flags)return mp(e.root.trueType,n,t)||mp(e.root.falseType,n,t);return!1}function pp(n){return n.valueDeclaration?e.getCombinedNodeFlags(n.valueDeclaration):0}function fp(n){if(8192&n.flags||4&e.getCheckFlags(n))return!0;if(e.isInJSFile(n.valueDeclaration)){var t=n.valueDeclaration.parent;return t&&e.isBinaryExpression(t)&&3===e.getAssignmentDeclarationKind(t)}}function gp(n,t,r,a){var i,o=e.getDeclarationModifierFlagsFromSymbol(a),s=146===n.kind?n.right:181===n.kind?n:n.name;if(256&e.getCheckFlags(a))return bt(s,e.Diagnostics.Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1,Jr(a),Yr(r)),!1;if(t){if(C<2&&_p(a))return bt(s,e.Diagnostics.Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword),!1;if(128&o)return bt(s,e.Diagnostics.Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression,Jr(a),Yr(Jc(a))),!1}if(128&o&&e.isThisProperty(n)&&_p(a)&&(i=e.getClassLikeDeclarationOfSymbol(Tr(a)))&&function(n){return!!e.findAncestor(n,function(n){return!!(e.isConstructorDeclaration(n)&&e.nodeIsPresent(n.body)||e.isPropertyDeclaration(n))||!(!e.isClassLike(n)&&!e.isFunctionLikeDeclaration(n))&&\"quit\"})}(n))return bt(s,e.Diagnostics.Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor,Jr(a),e.getTextOfIdentifierOrLiteral(i.name)),!1;if(!(24&o))return!0;if(8&o)return!!bv(n,i=e.getClassLikeDeclarationOfSymbol(Tr(a)))||(bt(s,e.Diagnostics.Property_0_is_private_and_only_accessible_within_class_1,Jr(a),Yr(Jc(a))),!1);if(t)return!0;var l=hv(n,function(n){var t=si(Er(n));return function(n,t){return zc(t,function(t){return!!(16&e.getDeclarationModifierFlagsFromSymbol(t))&&!Ba(n,Jc(t))})?void 0:n}(t,a)?t:void 0});if(!l){var c=void 0;if(32&o||!(c=function(n){var t=e.getThisContainer(n,!1);return t&&e.isFunctionLike(t)?e.getThisParameter(t):void 0}(n))||!c.type)return bt(s,e.Diagnostics.Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses,Jr(a),Yr(Jc(a)||r)),!1;var u=Ul(c.type);l=65536&u.flags?no(u):u}return!!(32&o)||(65536&r.flags&&(r=r.isThisType?no(r):io(r)),!(!r||!Ba(r,l))||(bt(s,e.Diagnostics.Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1,Jr(a),Yr(l)),!1))}function _p(e){return!!zc(e,function(e){return!(8192&e.flags)})}function vp(e,n,t,r){return yp(Lg(e),e,n,t,r)}function yp(n,t,r,a,i){if(2&n.flags)return bt(t,e.Diagnostics.Object_is_of_type_unknown),te;var o=24576&(M?vu(n):n.flags);if(o){bt(t,8192&o?16384&o?i||e.Diagnostics.Object_is_possibly_null_or_undefined:a||e.Diagnostics.Object_is_possibly_undefined:r||e.Diagnostics.Object_is_possibly_null);var s=Tu(n);return 57344&s.flags?te:s}return n}function hp(e){return Ep(e,e.expression,e.name)}function bp(e){return Ep(e,e.left,e.right)}function Ep(n,t,r){var a,i=vp(t),o=Rt(t).resolvedSymbol,s=po(Mu(i));if(da(s)||s===ye)return e.isIdentifier(t)&&o&&Zd(o,n),s;var l=e.getAssignmentTargetKind(n),c=vo(s,r.escapedText);if(!e.isIdentifier(t)||!o||c&&Vv(c)||Zd(o,n),c){if(function(n,t,r){var a,i=n.valueDeclaration;if(i){var o=e.idText(r);!function(n){return!!e.findAncestor(n,function(n){switch(n.kind){case 152:return!0;case 273:return!1;default:return!e.isExpressionNode(n)&&\"quit\"}})}(t)||Pt(i,r)||function(e){if(!(32&e.parent.flags))return!1;for(var n=Ga(e.parent);;){if(!(n=n.symbol&&Tp(n)))return!1;var t=vo(n,e.escapedName);if(t&&t.valueDeclaration)return!0}}(n)?238!==i.kind||162===t.parent.kind||4194304&i.flags||Pt(i,r)||(a=bt(r,e.Diagnostics.Class_0_used_before_its_declaration,o)):a=bt(r,e.Diagnostics.Property_0_is_used_before_its_initialization,o),a&&ht(a,e.createDiagnosticForNode(i,e.Diagnostics._0_is_declared_here,o))}}(c,n,r),kp(c,n,99===t.kind),Rt(n).resolvedSymbol=c,gp(n,97===t.kind,s,c),l&&($f(n,c)||eg(n)))return bt(r,e.Diagnostics.Cannot_assign_to_0_because_it_is_a_constant_or_a_read_only_property,e.idText(r)),te;a=Qd(Ga(c),n)}else{var u=To(s,0);if(!u||!u.type)return fl(i)?$:(r.escapedText&&!Ht(n)&&function(n,t){var r,a;if(262144&t.flags&&!(32764&t.flags))for(var i=0,o=t.types;i<o.length;i++){var s=o[i];if(!vo(s,n.escapedText)){r=e.chainDiagnosticMessages(r,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.declarationNameToString(n),Yr(s));break}}if(Sp(n.escapedText,t))r=e.chainDiagnosticMessages(r,e.Diagnostics.Property_0_is_a_static_member_of_type_1,e.declarationNameToString(n),Yr(t));else{var l=qg(t);if(l&&vo(l,n.escapedText))r=e.chainDiagnosticMessages(r,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_forget_to_use_await,e.declarationNameToString(n),Yr(t));else{var c=Lp(n,t);if(void 0!==c){var u=e.symbolName(c);r=e.chainDiagnosticMessages(r,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2,e.declarationNameToString(n),Yr(t),u),a=c.valueDeclaration&&e.createDiagnosticForNode(c.valueDeclaration,e.Diagnostics._0_is_declared_here,u)}else r=e.chainDiagnosticMessages(r,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.declarationNameToString(n),Yr(t))}}var d=e.createDiagnosticForNodeFromMessageChain(n,r);a&&ht(d,a),Xn.add(d)}(r,65536&i.flags&&i.isThisType?s:i),te);u.isReadonly&&(e.isAssignmentTarget(n)||e.isDeleteTarget(n))&&bt(n,e.Diagnostics.Index_signature_in_type_0_only_permits_reading,Yr(s)),a=u.type}if(187!==n.kind||1===l||c&&!(98311&c.flags)&&!(8192&c.flags&&262144&a.flags))return a;var d=!1;if(M&&I&&99===t.kind){var m=c&&c.valueDeclaration;if(m&&z_(m)){var p=Wd(n);155===p.kind&&p.parent===m.parent&&(d=!0)}}else M&&c&&c.valueDeclaration&&e.isPropertyAccessExpression(c.valueDeclaration)&&e.getAssignmentDeclarationPropertyAccessKind(c.valueDeclaration)&&Wd(n)===Wd(c.valueDeclaration)&&(d=!0);var f=jd(n,a,d?Eu(a):a);return d&&!(8192&vu(a))&&8192&vu(f)?(bt(r,e.Diagnostics.Property_0_is_used_before_being_assigned,Jr(c)),a):l?cu(f):f}function Tp(e){var n=Za(e);if(0!==n.length)return ll(n)}function Sp(n,t){var r=t.symbol&&vo(Ga(t.symbol),n);return void 0!==r&&r.valueDeclaration&&e.hasModifier(r.valueDeclaration,32)}function Lp(n,t){return Dp(e.isString(n)?n:e.idText(n),$i(t),67220415)}function xp(n,t){var r=Lp(n,t);return r&&e.symbolName(r)}function Ap(n,t,r){return e.Debug.assert(void 0!==t,\"outername should always be defined\"),Gt(n,t,r,void 0,t,!1,!1,function(n,r,a){return e.Debug.assertEqual(t,r,\"name should equal outerName\"),wt(n,r,a)||Dp(e.unescapeLeadingUnderscores(r),e.arrayFrom(n.values()),a)})}function Cp(n,t){return t.exports&&Dp(e.idText(n),fr(t),2623475)}function Dp(n,t,r){return e.getSpellingSuggestion(n,t,function(n){var t=e.symbolName(n);return!e.startsWith(t,'\"')&&n.flags&r?t:void 0})}function kp(n,t,r){if(n&&106500&n.flags&&n.valueDeclaration&&e.hasModifier(n.valueDeclaration,8)&&(!t||!e.isWriteOnlyAccess(t)||65536&n.flags&&!(32768&n.flags))){if(r){var a=e.findAncestor(t,e.isFunctionLikeDeclaration);if(a&&a.symbol===n)return}(1&e.getCheckFlags(n)?It(n).target:n).isReferenced=67108863}}function Mp(n,t,r,a){if(a===te||da(a))return!0;var i=vo(a,r);return i?gp(n,t,a,i):e.isInJSFile(n)&&0!=(262144&a.flags)&&a.types.some(function(e){return Mp(n,t,r,e)})}function Op(n){var t=n.initializer;if(236===t.kind){var r=t.declarations[0];if(r&&!e.isBindingPattern(r.name))return Er(r)}else if(71===t.kind)return nd(t)}function Ip(n){var t=vp(n.expression),r=n.argumentExpression;if(!r){var a,i=e.getSourceFileOfNode(n);return 190===n.parent.kind&&n.parent.expression===n?My(i,a=e.skipTrivia(i.text,n.expression.end),n.end-a,e.Diagnostics.new_T_cannot_be_used_to_create_an_array_Use_new_Array_T_instead):My(i,a=n.end-\"]\".length,n.end-a,e.Diagnostics.Expression_expected),te}var o=Lg(r);if(t===te||t===ye)return t;if(ig(t)&&9!==r.kind)return bt(r,e.Diagnostics.A_const_enum_member_can_only_be_accessed_using_a_string_literal),te;if(pu(t)&&!t.target.hasRestElement&&e.isNumericLiteral(r)){var s=+r.text,l=e.length(t.target.typeParameters);s>=l&&bt(r,e.Diagnostics.Index_0_is_out_of_bounds_in_tuple_of_length_1,s,l)}return Vg(hl(t,function(n){var t,r=e.skipParentheses(n);if(71===r.kind){var a=nd(r);if(3&a.flags)for(var i=n,o=n.parent;o;){if(224===o.kind&&i===o.statement&&Op(o)===a&&So(t=Tg(o.expression),1)&&!So(t,0))return!0;i=o,o=o.parent}}return!1}(r)?ce:o,n),n)}function Rp(n,t,r){if(t===te)return!1;if(!e.isWellKnownSymbolSyntactically(n))return!1;if(0==(3072&t.flags))return r&&bt(n,e.Diagnostics.A_computed_property_name_of_the_form_0_must_be_of_type_symbol,e.getTextOfNode(n)),!1;var a=n.expression,i=nd(a);if(!i)return!1;var o=ks(!0);return!(!o||i!==o&&(r&&bt(a,e.Diagnostics.Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object),1))}function Np(n){return e.isCallOrNewExpression(n)}function wp(n){return Np(n)&&e.forEach(n.typeArguments,lv),191===n.kind?Lg(n.template):150!==n.kind&&e.forEach(n.arguments,function(e){Lg(e)}),gn}function Pp(e){return wp(e),_n}function Fp(e){return!!e&&(206===e.kind||213===e.kind&&e.isSpread)}function Gp(n){return e.findIndex(n,Fp)}function Vp(n,t,r,a){if(void 0===a&&(a=!1),e.isJsxOpeningLikeElement(n))return!0;var i,o=!1;if(191===n.kind)if(i=t.length,204===n.template.kind){var s=e.last(n.template.templateSpans);o=e.nodeIsMissing(s.literal)||!!s.literal.isUnterminated}else{var l=n.template;e.Debug.assert(13===l.kind),o=!!l.isUnterminated}else if(150===n.kind)i=Qp(n,r);else{if(!n.arguments)return e.Debug.assert(190===n.kind),0===Nf(r);i=a?t.length+1:t.length,o=n.arguments.end===n.end;var c=Gp(t);if(c>=0)return c>=Nf(r)&&(wf(r)||c<Rf(r))}if(!wf(r)&&i>Rf(r))return!1;var u=i>=Nf(r);return o||u}function Kp(n,t){var r=e.length(n.typeParameters),a=Ro(n.typeParameters);return!t||t.length>=a&&t.length<=r}function Bp(e){if(131072&e.flags){var n=Xi(e);if(1===n.callSignatures.length&&0===n.constructSignatures.length&&0===n.properties.length&&!n.stringIndexInfo&&!n.numberIndexInfo)return n.callSignatures[0]}}function Hp(n,t,r,a){var i=wu(n.typeParameters,n,0,a);return Nu(r?ec(t,r):t,n,function(e,n){Wu(i.inferences,e,n)}),r||Wu(i.inferences,Ho(t),Ho(n),8),Wo(n,$u(i),e.isInJSFile(t.declaration))}function Up(e,n,t){var r=Mf(e,0),a=fg(n.attributes,r,x);Wu(t.inferences,a,r);var i=Mf(e,0),o=fg(n.attributes,i,t);return Wu(t.inferences,o,i),$u(t)}function jp(e,n,t,r,a){for(var i=0,o=a.inferences;i<o.length;i++){var s=o[i];s.isFixed||(s.inferredType=void 0)}if(150!==e.kind){var l=Am(e);if(l){var c=lc(l,function(e){return e&&function(e){return!!e.typeParameters}(e)?wu(e.typeParameters,e.signature,1|e.flags,e.compareTypes,e.inferences):e}(Cm(e))),u=Bp(c),d=u&&u.typeParameters?Zo(qo(u,u.typeParameters)):c,m=Ho(n);Wu(a.inferences,d,m,8)}}var p=Vo(n);if(p){var f=Jp(e),g=f?Lg(f):_e;Wu(a.inferences,g,p)}for(var _=Ff(n),v=_?Math.min(Rf(n)-1,t.length):t.length,y=0;y<v;y++){var h=t[y];if(208!==h.kind){var b=Mf(n,y),E=fg(h,b,r&&void 0!==r[y]?x:a);Wu(a.inferences,E,b)}}if(_){var T=Wp(t,v,t.length,_,a);Wu(a.inferences,T,_)}return $u(a)}function Wp(n,t,r,a,i){if(t>=r-1){var o=n[r-1];if(Fp(o))return 213===o.kind?Ws(o.type):Dd(s=fg(o.expression,a,i),function(e){return!(15794177&e.flags||eu(e)||pu(e))})?Ws(So(s,1)||te):s}for(var s,l=So(a,1)||$,c=tg(l,1081340),u=[],d=-1,m=t;m<r;m++){var p=fg(n[m],l,i);d<0&&Fp(n[m])&&(d=m-t),u.push(c?Vl(p):uu(p))}return d<0?Js(u):Js(e.append(u.slice(0,d),tl(u.slice(d))),d,!0)}function qp(n,t,r,a){for(var i,o=e.isInJSFile(n.declaration),s=n.typeParameters,l=No(e.map(t,Ul),s,Ro(s),o),c=0;c<t.length;c++){e.Debug.assert(void 0!==s[c],\"Should not call checkTypeArguments with too many type arguments\");var u=no(s[c]);if(u){var d=r&&a?function(){return e.chainDiagnosticMessages(void 0,e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1)}:void 0,m=a||e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1;i||(i=Jl(s,l));var p=l[c];if(!Lc(p,Ai(lc(u,i),p),r?t[c]:void 0,m,d))return}}return l}function zp(n,t,r,a,i,o){if(e.isJsxOpeningLikeElement(n))return function(n,t,r){if(n.attributes.end===n.end)return!0;for(var a=e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1,i=Mf(t,0),o=fg(n.attributes,i,void 0),s=0,l=$i(o);s<l.length;s++){var c=l[s];if(!vo(i,c.escapedName)&&Wm(c.escapedName))return!1}return Vc(o,i,r,void 0,a)}(n,r,a);var s=Vo(r);if(s&&s!==_e&&190!==n.kind){var l=Jp(n);if(!Vc(l?Lg(l):_e,s,a,o?l||n:void 0,e.Diagnostics.The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1))return!1}for(var c=e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1,u=Ff(r),d=u?Math.min(Rf(r)-1,t.length):t.length,m=0;m<d;m++){var p=t[m];if(208!==p.kind){var f=Mf(r,m),g=fg(p,f,i&&i[m]?x:void 0);if(!Ac(i?xu(g):g,f,a,o?p:void 0,p,c))return!1}}return!u||Vc(Wp(t,d,t.length,u,void 0),u,a,o?d<t.length?t[d]:n:void 0,c)}function Jp(n){if(189===n.kind){var t=e.skipOuterExpressions(n.expression);if(187===t.kind||188===t.kind)return t.expression}}function Xp(n,t,r){var a=e.createNode(213,n.pos,n.end);return a.parent=n,a.type=t,a.isSpread=r||!1,a}function Yp(n){if(191===n.kind){var t=n.template,r=[Xp(t,ln||(ln=Ds(\"TemplateStringsArray\",0,!0))||Se)];return 204===t.kind&&e.forEach(t.templateSpans,function(e){r.push(e.expression)}),r}if(150===n.kind)return function(n){var t=n.parent,r=n.expression;switch(t.kind){case 238:case 207:return[Xp(r,Ga(Er(t)))];case 149:var a=t.parent;return[Xp(r,155===t.parent.kind?Ga(Er(a)):te),Xp(r,$),Xp(r,ce)];case 152:case 154:case 156:case 157:var i=152!==t.kind&&0!==C;return[Xp(r,function(n){var t=Er(n.parent);return e.hasModifier(n,32)?Ga(t):si(t)}(t)),Xp(r,function(n){var t=n.name;switch(t.kind){case 71:return Kl(e.idText(t));case 8:case 9:return Kl(t.text);case 147:var r=Bm(t);return rg(r,3072)?r:le;default:return e.Debug.fail(\"Unsupported property name.\"),te}}(t)),Xp(r,i?Bs(Lv(t)):$)]}return e.Debug.fail()}(n);if(e.isJsxOpeningLikeElement(n))return n.attributes.properties.length>0?[n.attributes]:e.emptyArray;var a=n.arguments||e.emptyArray,i=a.length;if(i&&Fp(a[i-1])&&Gp(a)===i-1){var o=a[i-1],s=gg(o.expression);if(pu(s)){var l=s.typeArguments||e.emptyArray,c=s.target.hasRestElement?l.length-1:-1,u=e.map(l,function(e,n){return Xp(o,e,n===c)});return e.concatenate(a.slice(0,i-1),u)}}return a}function Qp(n,t){switch(n.parent.kind){case 238:case 207:return 1;case 152:return 2;case 154:case 156:case 157:return 0===C||t.parameters.length<=2?2:3;case 149:return 3;default:return e.Debug.fail()}}function Zp(n,t,r){for(var a,i=Number.POSITIVE_INFINITY,o=Number.NEGATIVE_INFINITY,s=Number.NEGATIVE_INFINITY,l=Number.POSITIVE_INFINITY,c=r.length,u=0,d=t;u<d.length;u++){var m=d[u],p=Nf(m),f=Rf(m);p<c&&p>s&&(s=p),c<f&&f<l&&(l=f),p<i&&(i=p,a=m),o=Math.max(o,f)}var g,_=e.some(t,wf),v=_?i:i<o?i+\"-\"+o:i,y=Gp(r)>-1;if(c<=o&&y&&c--,a&&Nf(a)>c&&a.declaration){var h=a.declaration.parameters[a.thisParameter?c+1:c];h&&(g=e.createDiagnosticForNode(h,e.isBindingPattern(h.name)?e.Diagnostics.An_argument_matching_this_binding_pattern_was_not_provided:e.Diagnostics.An_argument_for_0_was_not_provided,h.name?e.isBindingPattern(h.name)?void 0:e.idText(ev(h.name)):c))}if(_||y){var b=_&&y?e.Diagnostics.Expected_at_least_0_arguments_but_got_1_or_more:_?e.Diagnostics.Expected_at_least_0_arguments_but_got_1:e.Diagnostics.Expected_0_arguments_but_got_1_or_more,E=e.createDiagnosticForNode(n,b,v,c);return g?ht(E,g):E}if(i<c&&c<o)return e.createDiagnosticForNode(n,e.Diagnostics.No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments,c,s,l);var T=e.createDiagnosticForNode(n,e.Diagnostics.Expected_0_arguments_but_got_1,v,c);return g?ht(T,g):T}function $p(n,t,r){for(var a=1/0,i=-1/0,o=0,s=t;o<s.length;o++){var l=s[o];a=Math.min(a,Ro(l.typeParameters)),i=Math.max(i,e.length(l.typeParameters))}var c=a===i?a:a+\"-\"+i;return e.createDiagnosticForNodeArray(e.getSourceFileOfNode(n),r,e.Diagnostics.Expected_0_type_arguments_but_got_1,c,r.length)}function ef(n,t,r,i,o){var s,l=191===n.kind,c=150===n.kind,u=e.isJsxOpeningLikeElement(n);c||(s=n.typeArguments,(l||u||97!==n.expression.kind)&&e.forEach(s,lv));var d=r||[];if(function(n,t){var r,a,i,o,s=0,l=-1;e.Debug.assert(!t.length);for(var c=0,u=n;c<u.length;c++){var d=u[c],m=d.declaration&&Er(d.declaration),p=d.declaration&&d.declaration.parent;a&&m!==a?(i=s=t.length,r=p):r&&p===r?i+=1:(r=p,i=s),a=m,d.hasLiteralTypes?(o=++l,s++):o=i,t.splice(o,0,d)}}(t,d),!d.length)return Xn.add(e.createDiagnosticForNode(n,e.Diagnostics.Call_target_does_not_contain_any_signatures)),Pp(n);var m,p,f,g,_=Yp(n),v=1===d.length&&!d[0].typeParameters,y=c||v?void 0:nf(_),h=i&&189===n.kind&&n.arguments.hasTrailingComma;if(d.length>1&&(g=E(d,ut,h)),g||(g=E(d,dt,h)),g)return g;if(m){if(u)return m;zp(n,_,m,dt,void 0,!0)}else if(p)Xn.add(Zp(n,[p],_));else if(f)qp(f,n.typeArguments,!0,o);else{var b=e.filter(t,function(e){return Kp(e,s)});0===b.length?Xn.add($p(n,t,s)):c?o&&Xn.add(e.createDiagnosticForNode(n,o)):Xn.add(Zp(n,b,_))}return a||!_?Pp(n):function(n,t,r,a){return e.Debug.assert(t.length>0),a||1===t.length||t.some(function(e){return!!e.typeParameters})?function(n,t,r){var a=function(e,n){for(var t=-1,r=-1,a=0;a<e.length;a++){var i=e[a],o=Rf(i);if(wf(i)||o>=n)return a;o>r&&(r=o,t=a)}return t}(t,void 0===V?r.length:V),i=t[a],o=i.typeParameters;if(!o)return i;var s=Np(n)?n.typeArguments:void 0,l=s?zo(i,function(e,n,t){for(var r=e.map(Lv);r.length>n.length;)r.pop();for(;r.length<n.length;)r.push(no(n[r.length])||Zu(t));return r}(s,o,e.isInJSFile(n))):function(n,t,r,a){var i=wu(t,r,e.isInJSFile(n)?2:0),o=jp(n,r,a,nf(a),i);return zo(r,o)}(n,o,i,r);return t[a]=l,l}(n,t,r):function(n){var t,r=e.mapDefined(n,function(e){return e.thisParameter});r.length&&(t=rf(r,r.map(Df)));for(var a=e.minAndMax(n,tf),i=a.min,o=a.max,s=[],l=function(t){var r=e.mapDefined(n,function(n){var r=n.parameters,a=n.hasRestParameter;return a?t<r.length-1?r[t]:e.last(r):t<r.length?r[t]:void 0});e.Debug.assert(0!==r.length),s.push(rf(r,e.mapDefined(n,function(e){return Of(e,t)})))},c=0;c<o;c++)l(c);var u=e.mapDefined(n,function(n){return n.hasRestParameter?e.last(n.parameters):void 0}),d=0!==u.length;if(d){var m=Ws(tl(e.mapDefined(n,jo),2));s.push(af(u,m))}return Di(n[0].declaration,void 0,t,s,ll(n.map(Ho)),void 0,i,d,n.some(function(e){return e.hasLiteralTypes}))}(t)}(n,d,_,!!r);function E(t,r,a){if(void 0===a&&(a=!1),m=void 0,p=void 0,f=void 0,v){var i=t[0];if(s||!Vp(n,_,i,a))return;return zp(n,_,i,r,y,!1)?i:void(m=i)}for(var o=0;o<t.length;o++)if(Kp(i=t[o],s)&&Vp(n,_,i,a)){var l=void 0,c=void 0;if(i.typeParameters){var u=void 0;if(s){if(!(u=qp(i,s,!1))){f=i;continue}}else c=wu(i.typeParameters,i,e.isInJSFile(n)?2:0),u=jp(n,i,_,y,c);if(l=Wo(i,u,e.isInJSFile(i.declaration)),Ff(i)&&!Vp(n,_,l,a)){p=l;continue}}else l=i;if(zp(n,_,l,r,y,!1)){if(y&&(y=void 0,c&&(l=Wo(i,u=jp(n,i,_,y,c),e.isInJSFile(i.declaration))),!zp(n,_,l,r,y,!1))){m=l;continue}return t[o]=l,l}m=l}}}function nf(e){for(var n,t=0;t<e.length;t++)dc(e[t])&&(n||(n=new Array(e.length)),n[t]=!0);return n}function tf(e){var n=e.parameters.length;return e.hasRestParameter?n-1:n}function rf(e,n){return af(e,tl(n,2))}function af(n,t){return Lu(e.first(n),t)}function of(e,n,t,r){return da(e)||da(n)&&65536&e.flags||!t&&!r&&!(294912&n.flags)&&bc(e,Fe)}function sf(n,t,r){if(n.arguments&&C<1){var a=Gp(n.arguments);a>=0&&bt(n.arguments[a],e.Diagnostics.Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher)}var i=vp(n.expression);if(i===ye)return bn;if((i=po(i))===te)return Pp(n);if(da(i))return n.typeArguments&&bt(n,e.Diagnostics.Untyped_function_calls_may_not_accept_type_arguments),wp(n);var o=ho(i,1);if(o.length){if(!function(n,t){if(!t||!t.declaration)return!0;var r=t.declaration,a=e.getSelectedModifierFlags(r,24);if(!a)return!0;var i=e.getClassLikeDeclarationOfSymbol(r.parent.symbol),o=si(r.parent.symbol);if(!bv(n,i)){var s=e.getContainingClass(n);if(s&&16&a){var l=Lv(s);if(function n(t,r){var a=Za(r);if(!e.length(a))return!1;var i=a[0];if(524288&i.flags){for(var o=i.types,s=e.countWhere(o,qa),l=0,c=0,u=i.types;c<u.length;c++){var d=u[c];if(l++,(0===s||s===o.length&&0===l||!qa(d))&&3&e.getObjectFlags(d)){if(d.symbol===t)return!0;if(n(t,d))return!0}}return!1}return i.symbol===t||n(t,i)}(r.parent.symbol,l))return!0}return 8&a&&bt(n,e.Diagnostics.Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration,Yr(o)),16&a&&bt(n,e.Diagnostics.Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration,Yr(o)),!1}return!0}(n,o[0]))return Pp(n);var s=i.symbol&&e.getClassLikeDeclarationOfSymbol(i.symbol);return s&&e.hasModifier(s,128)?(bt(n,e.Diagnostics.Cannot_create_an_instance_of_an_abstract_class),Pp(n)):ef(n,o,t,r)}var l=ho(i,0);if(l.length){var c=ef(n,l,t,r);return R||(c.declaration&&!gf(c.declaration)&&Ho(c)!==_e&&bt(n,e.Diagnostics.Only_a_void_function_can_be_called_with_the_new_keyword),Vo(c)===_e&&bt(n,e.Diagnostics.A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void)),c}return lf(n,i,1),Pp(n)}function lf(n,t,r,a){var i=bt(n,0===r?e.Diagnostics.Cannot_invoke_an_expression_whose_type_lacks_a_call_signature_Type_0_has_no_compatible_call_signatures:e.Diagnostics.Cannot_use_new_with_an_expression_whose_type_lacks_a_call_or_construct_signature,Yr(t));cf(t,r,a?ht(i,a):i)}function cf(n,t,r){if(n.symbol){var a=It(n.symbol).originatingImport;if(a&&!e.isImportCall(a)){var i=ho(Ga(It(n.symbol).target),t);if(!i||!i.length)return;ht(r,e.createDiagnosticForNode(a,e.Diagnostics.Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead))}}}function uf(n){switch(n.parent.kind){case 238:case 207:return e.Diagnostics.Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression;case 149:return e.Diagnostics.Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression;case 152:return e.Diagnostics.Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression;case 154:case 156:case 157:return e.Diagnostics.Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression;default:return e.Debug.fail()}}function df(n,t,r){var a=Lg(n.expression),i=po(a);if(i===te)return Pp(n);var o,s,l=ho(i,0),c=ho(i,1);if(of(a,i,l.length,c.length))return wp(n);if(o=n,(s=l).length&&e.every(s,function(e){return 0===e.minArgumentCount&&!e.hasRestParameter&&e.parameters.length<Qp(o,e)})){var u=e.getTextOfNode(n.expression,!1);return bt(n,e.Diagnostics._0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0,u),Pp(n)}var d=uf(n);if(!l.length){var m=e.chainDiagnosticMessages(void 0,e.Diagnostics.Cannot_invoke_an_expression_whose_type_lacks_a_call_signature_Type_0_has_no_compatible_call_signatures,Yr(i));m=e.chainDiagnosticMessages(m,d);var p=e.createDiagnosticForNodeFromMessageChain(n,m);return Xn.add(p),cf(i,0,p),Pp(n)}return ef(n,l,t,r,d)}function mf(n,t,r,a){e.Debug.assert(!(262144&t.flags));var i=t&&ho(t,0);if(i&&i.length>0)return ef(n,i,r,a)}function pf(n,t,r){switch(n.kind){case 189:return function(n,t,r){if(97===n.expression.kind){var a=lm(n.expression);if(da(a)){for(var i=0,o=n.arguments;i<o.length;i++)Lg(o[i]);return gn}if(a!==te){var s=e.getEffectiveBaseTypeNode(e.getContainingClass(n));if(s)return ef(n,Ya(a,s.typeArguments,s),t,r)}return wp(n)}var l=vp(n.expression,e.Diagnostics.Cannot_invoke_an_object_which_is_possibly_null,e.Diagnostics.Cannot_invoke_an_object_which_is_possibly_undefined,e.Diagnostics.Cannot_invoke_an_object_which_is_possibly_null_or_undefined);if(l===ye)return bn;var c=po(l);if(c===te)return Pp(n);var u=ho(c,0),d=ho(c,1);if(of(l,c,u.length,d.length))return l!==te&&n.typeArguments&&bt(n,e.Diagnostics.Untyped_function_calls_may_not_accept_type_arguments),wp(n);if(!u.length){if(d.length)bt(n,e.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new,Yr(l));else{var m=void 0;if(1===n.arguments.length&&_g(e.first(n.arguments))){var p=e.getSourceFileOfNode(n).text;e.isLineBreak(p.charCodeAt(e.skipTrivia(p,n.expression.end,!0)-1))&&(m=e.createDiagnosticForNode(n.expression,e.Diagnostics.It_is_highly_likely_that_you_are_missing_a_semicolon))}lf(n,c,0,m)}return Pp(n)}return u.some(function(n){return e.isInJSFile(n.declaration)&&!!e.getJSDocClassTag(n.declaration)})?(bt(n,e.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new,Yr(l)),Pp(n)):ef(n,u,t,r)}(n,t,r);case 190:return sf(n,t,r);case 191:return function(e,n,t){var r=Lg(e.tag),a=po(r);if(a===te)return Pp(e);var i=ho(a,0),o=ho(a,1);return of(r,a,i.length,o.length)?wp(e):i.length?ef(e,i,n,t):(lf(e,a,0),Pp(e))}(n,t,r);case 150:return df(n,t,r);case 260:case 259:return Dd(Lg(n.tagName),function(a){var i=mf(n,a,t,r);if(i&&i!==_n)return i;var o=ap(n,a);return t&&e.length(o)&&t.push.apply(t,o),e.length(o)?o[0]:_n})||_n}throw e.Debug.assertNever(n,\"Branch in 'resolveSignature' should be unreachable.\")}function ff(e,n,t){void 0===t&&(t=!1);var r=Rt(e),a=r.resolvedSignature;if(a&&a!==vn&&!n)return a;r.resolvedSignature=vn;var i=pf(e,n,t);return r.resolvedSignature=An===Cn?i:a,i}function gf(n){if(!n||!e.isInJSFile(n))return!1;var t=e.isFunctionDeclaration(n)||e.isFunctionExpression(n)?n:e.isVariableDeclaration(n)&&n.initializer&&e.isFunctionExpression(n.initializer)?n.initializer:void 0;if(t){if(e.getJSDocClassTag(n))return!0;var r=Er(t);return!!r&&void 0!==r.members}return!1}function _f(e){if(131072&e.flags){var n=Xi(e);return 1===n.callSignatures.length&&gf(n.callSignatures[0].declaration)}return!1}function vf(e){var n;gf(e.valueDeclaration)&&(n=hf(e));var t=yf(e),r=Ga(e);return r.symbol&&!bf(r)&&gf(r.symbol.valueDeclaration)&&(n=hf(r.symbol)),t&&n?ll([n,t]):t||n}function yf(n){var t=n.valueDeclaration,r=t&&t.parent&&(e.isFunctionDeclaration(t)&&Er(t)||e.isBinaryExpression(t.parent)&&Er(t.parent.left)||e.isVariableDeclaration(t.parent)&&Er(t.parent)),a=r&&r.exports&&r.exports.get(\"prototype\"),i=a&&a.valueDeclaration&&function(n){if(!n.parent)return!1;for(var t=n.parent;t&&187===t.kind;)t=t.parent;if(t&&e.isBinaryExpression(t)&&e.isPrototypeAccess(t.left)&&58===t.operatorToken.kind){var r=e.getInitializerOfBinaryExpression(t);return e.isObjectLiteralExpression(r)&&r}}(a.valueDeclaration);return i?Lg(i):void 0}function hf(n){var t=It(n);return t.inferredClassType||(t.inferredClassType=wr(n,Li(n)||L,e.emptyArray,e.emptyArray,void 0,void 0)),t.inferredClassType}function bf(n){return n.symbol&&16&e.getObjectFlags(n)&&It(n.symbol).inferredClassType===n}function Ef(n){py(n,n.typeArguments)||fy(n.arguments);var t=ff(n);if(97===n.expression.kind)return _e;if(190===n.kind){var r=t.declaration;if(r&&155!==r.kind&&159!==r.kind&&164!==r.kind&&!e.isJSDocConstructSignature(r)){var a=Lg(n.expression).symbol;a||71!==n.expression.kind||(a=nd(n.expression));var i=a&&vf(a);return i?t.target?lc(i,t.mapper):i:(R&&bt(n,e.Diagnostics.new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type),$)}}if(e.isInJSFile(n)&&xf(n))return function(e){var n=sr(e,e);if(n){var t=dr(n);if(t)return Ga(t)}return $}(n.arguments[0]);var o,s=Ho(t);if(3072&s.flags&&Tf(n))return Bl(e.walkUpParenthesizedExpressions(n.parent));if(e.isInJSFile(n)){var l=e.getDeclarationOfExpando(n);if(l){var c=Er(l);c&&e.hasEntries(c.exports)&&((o=wr(c,c.exports,e.emptyArray,e.emptyArray,void 0,void 0)).objectFlags|=16384)}}return o?ll([s,o]):s}function Tf(n){if(!e.isCallExpression(n))return!1;var t=n.expression;if(e.isPropertyAccessExpression(t)&&\"for\"===t.name.escapedText&&(t=t.expression),!e.isIdentifier(t)||\"Symbol\"!==t.escapedText)return!1;var r=ks(!1);return!!r&&r===Ft(t,\"Symbol\",67220415,void 0,void 0,!1)}function Sf(n){if(fy(n.arguments)||function(n){if(D===e.ModuleKind.ES2015)return Oy(n,e.Diagnostics.Dynamic_import_is_only_supported_when_module_flag_is_commonjs_or_esNext);if(n.typeArguments)return Oy(n,e.Diagnostics.Dynamic_import_cannot_have_type_arguments);var t=n.arguments;1!==t.length?Oy(n,e.Diagnostics.Dynamic_import_must_have_one_specifier_as_an_argument):e.isSpreadElement(t[0])&&Oy(t[0],e.Diagnostics.Specifier_of_dynamic_import_cannot_be_spread_element)}(n),0===n.arguments.length)return Hf(n,$);for(var t=n.arguments[0],r=gg(t),a=1;a<n.arguments.length;++a)gg(n.arguments[a]);(8192&r.flags||16384&r.flags||!bc(r,le))&&bt(t,e.Diagnostics.Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0,Yr(r));var i=sr(n,t);if(i){var o=mr(i,t,!0);if(o)return Hf(n,Lf(Ga(o),o,i))}return Hf(n,$)}function Lf(n,t,r){if(k&&n&&n!==te){var a=n;if(!a.syntheticType)if(Jt(e.find(r.declarations,e.isSourceFile),r,!1)){var i=e.createSymbolTable(),o=Tt(2097152,\"default\");o.target=$t(t),i.set(\"default\",o);var s=Tt(2048,\"__type\"),l=wr(s,i,e.emptyArray,e.emptyArray,void 0,void 0);s.type=l,a.syntheticType=jm(n)?Rl(n,l,s,0,0):l}else a.syntheticType=n;return a.syntheticType}return n}function xf(n){if(!e.isRequireCall(n,!0))return!1;if(!e.isIdentifier(n.expression))return e.Debug.fail();var t=Ft(n.expression,n.expression.escapedText,67220415,void 0,void 0,!0);if(t===B)return!0;if(2097152&t.flags)return!1;var r=16&t.flags?237:3&t.flags?235:0;if(0!==r){var a=e.getDeclarationOfKind(t,r);return!!a&&!!(4194304&a.flags)}return!1}function Af(n,t,r,i){var o=xu(cu(Lg(r,i)));lv(t);var s=Ul(t);return a&&s!==te&&(Tc(s,Mu(o))||Oc(o,s,n,e.Diagnostics.Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first)),s}function Cf(n){return function(n){var t=n.name.escapedText;switch(n.keywordToken){case 94:if(\"target\"!==t)return Oy(n.name,e.Diagnostics._0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2,n.name.escapedText,e.tokenToString(n.keywordToken),\"target\");break;case 91:\"meta\"!==t&&Oy(n.name,e.Diagnostics._0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2,n.name.escapedText,e.tokenToString(n.keywordToken),\"meta\")}}(n),94===n.keywordToken?function(n){var t=e.getNewTargetContainer(n);if(t){if(155===t.kind){var r=Er(t.parent);return Ga(r)}var r=Er(t);return Ga(r)}return bt(n,e.Diagnostics.Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constructor,\"new.target\"),te}(n):91===n.keywordToken?function(n){(C<6||D<e.ModuleKind.ESNext)&&bt(n,e.Diagnostics.The_import_meta_meta_property_is_only_allowed_using_ESNext_for_the_target_and_module_compiler_options);var t=e.getSourceFileOfNode(n);return e.Debug.assert(!!(1048576&t.flags),\"Containing file is missing import meta node flag.\"),e.Debug.assert(!!t.externalModuleIndicator,\"Containing file should be a module.\"),\"meta\"===n.name.escapedText?cn||(cn=Ds(\"ImportMeta\",0,!0))||Se:te}(n):e.Debug.assertNever(n.keywordToken)}function Df(n){var t=Ga(n);if(M){var r=n.valueDeclaration;if(r&&e.hasInitializer(r))return Eu(t)}return t}function kf(e,n){var t=e.parameters.length-(e.hasRestParameter?1:0);if(n<t)return e.parameters[n].escapedName;var r=e.parameters[t]||Q,a=Ga(r);if(pu(a)){var i=a.target.associatedNames,o=n-t;return i?i[o]:r.escapedName+\"_\"+o}return r.escapedName}function Mf(e,n){return Of(e,n)||$}function Of(e,n){var t=e.parameters.length-(e.hasRestParameter?1:0);if(n<t)return Df(e.parameters[n]);if(e.hasRestParameter){var r=Ga(e.parameters[t]);return pu(r)?n-t<gu(r)?r.typeArguments[n-t]:fu(r):So(r,1)}}function If(e,n){var t=Rf(e),r=Pf(e);if(r&&n===t-1)return r;for(var a=r?Math.min(n,t-1):n,i=[],o=[],s=a;s<t;s++)i.push(Mf(e,s)),o.push(kf(e,s));var l=Nf(e);return Js(i,l<a?0:l-a,!!r,o)}function Rf(n){var t=n.parameters.length;if(n.hasRestParameter){var r=Ga(n.parameters[t-1]);if(pu(r))return t+(r.typeArguments||e.emptyArray).length-1}return t}function Nf(e){if(e.hasRestParameter){var n=Ga(e.parameters[e.parameters.length-1]);if(pu(n)){var t=n.target.minLength;if(t>0)return e.parameters.length-1+t}}return e.minArgumentCount}function wf(e){if(e.hasRestParameter){var n=Ga(e.parameters[e.parameters.length-1]);return!pu(n)||n.target.hasRestElement}return!1}function Pf(e){if(e.hasRestParameter){var n=Ga(e.parameters[e.parameters.length-1]);return pu(n)?function(e){var n=fu(e);return n&&Ws(n)}(n):n}}function Ff(e){var n=Pf(e);return!n||eu(n)||da(n)?void 0:n}function Gf(e){return Vf(e,ve)}function Vf(e,n){return e.parameters.length>0?Mf(e,0):n}function Kf(n,t){var r=It(n);if(!r.type){r.type=t;var a=n.valueDeclaration;71!==a.name.kind&&(r.type===Se&&(r.type=Aa(a.name)),function n(t){for(var r=0,a=t.elements;r<a.length;r++){var i=a[r];e.isOmittedExpression(i)||(71===i.name.kind?It(Er(i)).type=ga(i):n(i.name))}}(a.name))}}function Bf(e){var n=Os(!0);return n!==Ae?ss(n,[e=Jg(e)||Se]):Se}function Hf(n,t){var r=Bf(t);return r===Se?(bt(n,e.isImportCall(n)?e.Diagnostics.A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option:e.Diagnostics.An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option),te):(Is(!0)||bt(n,e.isImportCall(n)?e.Diagnostics.A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option:e.Diagnostics.An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option),r)}function Uf(n,t){if(!n.body)return te;var r,a=e.getFunctionFlags(n);if(216!==n.body.kind)r=gg(n.body,t),2&a&&(r=zg(r,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member));else{var i=function(n,t){var r=e.getFunctionFlags(n),a=[],i=zf(n),o=!1;if(e.forEachReturnStatement(n.body,function(s){var l=s.expression;if(l){var c=gg(l,t);2&r&&(c=zg(c,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member)),32768&c.flags&&(o=!0),e.pushIfUnique(a,c)}else i=!0}),0!==a.length||i||!o&&!function(e){switch(e.kind){case 194:case 195:return!0;case 154:return 186===e.parent.kind;default:return!1}}(n))return!(M&&a.length&&i)||gf(n)&&a.some(function(e){return e.symbol===n.symbol})||e.pushIfUnique(a,ae),a}(n,t);if(1&a){if(!(i=e.concatenate(function(n,t){var r=[],a=0!=(2&e.getFunctionFlags(n));return e.forEachYieldExpression(n.body,function(n){e.pushIfUnique(r,jf(n,a,t))}),r}(n,t),i))||0===i.length){var o=2&a?Hs($):js($);return R&&bt(n.asteriskToken,e.Diagnostics.Generator_implicitly_has_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_return_type,Yr(o)),o}}else{if(!i)return 2&a?Hf(n,ve):ve;if(0===i.length)return 2&a?Hf(n,_e):_e}r=tl(i,2)}var s=Im(n);if(s||Ru(n,r),su(r)){var l=s?s===wo(n)?r:Ho(s):void 0;if(l)switch(3&a){case 3:l=P_(l,!0);break;case 1:l=P_(l,!1);break;case 2:l=qg(l)}r=mu(r,l)}var c=Mu(r);switch(3&a){case 3:return Hs(c);case 1:return js(c);case 2:return Bf(c);default:return c}}function jf(n,t,r){var a=n.expression||n,i=n.expression?Lg(n.expression,r):ie,o=n.asteriskToken?O_(i,a,!1,t):i;return t?Jg(o,a,n.asteriskToken?e.Diagnostics.Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:e.Diagnostics.Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member):o}function Wf(e,n,t,r){var a=0;if(r){for(var i=n;i<t.length;i++)a|=st.get(t[i])||8192;for(i=e;i<n;i++)a&=~(st.get(t[i])||0);for(i=0;i<e;i++)a|=st.get(t[i])||8192}else{for(i=e;i<n;i++)a|=ot.get(t[i])||64;for(i=0;i<e;i++)a&=~(ot.get(t[i])||0)}return a}function qf(n){if(!n.possiblyExhaustive)return!1;if(197===n.expression.kind){var t=Tg(n.expression.expression),r=Wf(0,0,Ad(n),!0);return!!(32768&kd(io(t)||t,function(e){return(dd(e)&r)===r}).flags)}var a=Tg(n.expression);if(!lu(a))return!1;var i,o,s=xd(n);return!(!s.length||e.some(s,ou))&&(i=Md(a,Vl),o=s,262144&i.flags?!e.forEach(i.types,function(n){return!e.contains(o,n)}):e.contains(o,i))}function zf(n){return!!(128&n.flags)&&!e.some(n.body.statements,function(e){return 230===e.kind&&qf(e)})}function Jf(n,t){if(a&&(!t||!tg(t,4097))&&153!==n.kind&&!e.nodeIsMissing(n.body)&&216===n.body.kind&&zf(n)){var r=256&n.flags;if(t&&32768&t.flags)bt(e.getEffectiveReturnTypeNode(n),e.Diagnostics.A_function_returning_never_cannot_have_a_reachable_end_point);else if(t&&!r)bt(e.getEffectiveReturnTypeNode(n),e.Diagnostics.A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value);else if(t&&M&&!bc(ae,t))bt(e.getEffectiveReturnTypeNode(n),e.Diagnostics.Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined);else if(A.noImplicitReturns){if(!t){if(!r)return;if(G_(n,Ho(wo(n))))return}bt(e.getEffectiveReturnTypeNode(n)||n,e.Diagnostics.Not_all_code_paths_return_a_value)}}}function Xf(n,t){if(e.Debug.assert(154!==n.kind||e.isObjectLiteralMethod(n)),1===t&&dc(n)){if(!e.getEffectiveReturnTypeNode(n)&&pc(n)){var r=Rt(n);if(r.contextFreeType)return r.contextFreeType;var a=Uf(n,t),i=Di(void 0,void 0,void 0,e.emptyArray,a,void 0,0,!1,!1),o=wr(n.symbol,L,[i],e.emptyArray,void 0,void 0);return o.flags|=536870912,r.contextFreeType=o}return Ce}my(n)||194!==n.kind||yy(n);var s=Rt(n),l=Ga(br(n.symbol));if(da(l))return l;if(!(1024&s.flags)){var c=Nm(n);if(!(1024&s.flags)){if(s.flags|=1024,c){var u=ho(l,0)[0];if(dc(n)){var d=Cm(n);2===t&&function(n,t,r){for(var a=n.parameters.length-(n.hasRestParameter?1:0),i=0;i<a;i++){var o=n.parameters[i].valueDeclaration;if(o.type){var s=e.getEffectiveTypeAnnotationNode(o);s&&Wu(r.inferences,Ul(s),Mf(t,i))}}}(u,c,d),function(n,t){n.typeParameters=t.typeParameters,t.thisParameter&&(!(i=n.thisParameter)||i.valueDeclaration&&!i.valueDeclaration.type)&&(i||(n.thisParameter=Lu(t.thisParameter,void 0)),Kf(n.thisParameter,Ga(t.thisParameter)));for(var r=n.parameters.length-(n.hasRestParameter?1:0),a=0;a<r;a++){var i=n.parameters[a];e.getEffectiveTypeAnnotationNode(i.valueDeclaration)||Kf(i,Mf(t,a))}n.hasRestParameter&&(!St(i=e.last(n.parameters))&&e.getEffectiveTypeAnnotationNode(i.valueDeclaration)||Kf(i,If(t,r)))}(u,d===x?c:ec(c,d))}Uo(n)||u.resolvedReturnType||(a=Uf(n,t),u.resolvedReturnType||(u.resolvedReturnType=a))}Dg(n),uv(n)}}return l}function Yf(e,n){var t=Uo(e);return t&&2==(3&n)?Jg(t)||te:t}function Qf(e,n,t){return!!rg(n,168)||(bt(e,t),!1)}function Zf(n){return!!(8&e.getCheckFlags(n)||4&n.flags&&64&e.getDeclarationModifierFlagsFromSymbol(n)||3&n.flags&&2&pp(n)||98304&n.flags&&!(65536&n.flags)||8&n.flags)}function $f(n,t){if(Zf(t)){if(4&t.flags&&(187===n.kind||188===n.kind)&&99===n.expression.kind){var r=e.getContainingFunction(n);return!r||155!==r.kind||!t.valueDeclaration||!(r.parent===t.valueDeclaration.parent||r===t.valueDeclaration.parent)}return!0}return!1}function eg(n){if(187===n.kind||188===n.kind){var t=e.skipParentheses(n.expression);if(71===t.kind){var r=Rt(t).resolvedSymbol;if(2097152&r.flags){var a=Wt(r);return!!a&&249===a.kind}}}return!1}function ng(n,t){var r=e.skipOuterExpressions(n,3);return 71===r.kind||187===r.kind||188===r.kind||(bt(n,t),!1)}function tg(e,n){if(e.flags&n&-134217729||134217728&n&&Ji(e))return!0;if(786432&e.flags)for(var t=0,r=e.types;t<r.length;t++)if(tg(r[t],n))return!0;return!1}function rg(e,n,t){return!!(e.flags&n)||!(t&&28675&e.flags)&&(!!(168&n)&&bc(e,ce)||!!(68&n)&&bc(e,le)||!!(272&n)&&bc(e,fe)||!!(4096&n)&&bc(e,_e)||!!(32768&n)&&bc(e,ve)||!!(16384&n)&&bc(e,oe)||!!(8192&n)&&bc(e,ae)||!!(1024&n)&&bc(e,ge)||!!(16777216&n)&&bc(e,be))}function ag(n,t,r){return 262144&n.flags?e.every(n.types,function(e){return ag(e,t,r)}):rg(n,t,r)}function ig(n){return!!(16&e.getObjectFlags(n))&&!!n.symbol&&og(n.symbol)}function og(e){return 0!=(128&e.flags)}function sg(n,t,r,a){if(void 0===a&&(a=!1),273===t.kind||274===t.kind){var i=t.name;if(147===i.kind&&Bm(i),pa(i))return;if(l=function(n,t,r,a){if(da(n))return n;var i,o=e.getTextOfPropertyName(t);if(o){var s=vo(n,o);s&&(kp(s,r,a),gp(r,!1,n,s),i=Ga(s)),i=i||(Km(o)?So(n,1):void 0)}return i||So(n,0)}(n,i,t,a))return cg(274===t.kind?t:t.initializer,l);bt(i,e.Diagnostics.Type_0_has_no_property_1_and_no_string_index_signature,Yr(n),e.declarationNameToString(i))}else{if(275===t.kind){C<6&&iy(t,4);var o=[];if(r)for(var s=0;s<r.length-1;s++)o.push(r[s].name);var l=fa(n,o,n.symbol);return cy(r,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma),cg(t.expression,l)}bt(t,e.Diagnostics.Property_assignment_expected)}}function lg(n,t,r,a,i){var o=n.elements,s=o[r];if(208!==s.kind)if(206!==s.kind){var l=\"\"+r;if(c=da(t)?t:au(t)?iu(t,r):a)return cg(s,c,i);Lg(s),pu(t)?bt(s,e.Diagnostics.Tuple_type_0_with_length_1_cannot_be_assigned_to_tuple_with_length_2,Yr(t),cs(t),o.length):bt(s,e.Diagnostics.Type_0_has_no_property_1,Yr(t),l)}else if(r<o.length-1)bt(s,e.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern);else{var c,u=s.expression;if(202!==u.kind||58!==u.operatorToken.kind)return cy(n.elements,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma),cg(u,c=pu(t)?Fm((t.typeArguments||e.emptyArray).slice(r,cs(t))):Ws(a),i);bt(u.operatorToken,e.Diagnostics.A_rest_element_cannot_have_an_initializer)}}function cg(n,t,r,a){var i;if(274===n.kind){var o=n;o.objectAssignmentInitializer&&(!M||8192&vu(Lg(o.objectAssignmentInitializer))||(t=md(t,131072)),mg(o.name,o.equalsToken,o.objectAssignmentInitializer,r)),i=n.name}else i=n;return 202===i.kind&&58===i.operatorToken.kind&&(dg(i,r),i=i.left),186===i.kind?function(e,n,t){var r=e.properties;if(M&&0===r.length)return yp(n,e);for(var a=0,i=r;a<i.length;a++)sg(n,i[a],r,t);return n}(i,t,a):185===i.kind?function(e,n,t){var r=e.elements;C<2&&A.downlevelIteration&&iy(e,512);for(var a=O_(n,e,!1,!1)||te,i=0;i<r.length;i++)lg(e,n,i,a,t);return n}(i,t,r):function(n,t,r){var a=Lg(n,r),i=275===n.parent.kind?e.Diagnostics.The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access:e.Diagnostics.The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access;return ng(n,i)&&xc(t,a,n,n),t}(i,t,r)}function ug(e,n){return 0!=(24576&n.flags)||Tc(e,n)}function dg(n,t){return e.isInJSFile(n)&&e.getAssignedExpandoInitializer(n)?Lg(n.right,t):mg(n.left,n.operatorToken,n.right,t,n)}function mg(n,t,r,i,o){var s,l=t.kind;if(58===l&&(186===n.kind||185===n.kind))return cg(n,Lg(r,i),i,99===r.kind);s=53===l||54===l?C_(n,i):Lg(n,i);var c,u,d=Lg(r,i);switch(l){case 39:case 40:case 61:case 62:case 41:case 63:case 42:case 64:case 38:case 60:case 45:case 65:case 46:case 66:case 47:case 67:case 49:case 69:case 50:case 70:case 48:case 68:if(s===ye||d===ye)return ye;s=yp(s,n),d=yp(d,r);var m=void 0;if(272&s.flags&&272&d.flags&&void 0!==(m=function(e){switch(e){case 49:case 69:return 54;case 50:case 70:return 35;case 48:case 68:return 53;default:return}}(t.kind)))bt(o||t,e.Diagnostics.The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead,e.tokenToString(t.kind),e.tokenToString(m));else{var p=Qf(n,s,e.Diagnostics.The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_or_an_enum_type),f=Qf(r,d,e.Diagnostics.The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_or_an_enum_type);p&&f&&b(ce)}return ce;case 37:case 59:if(s===ye||d===ye)return ye;rg(s,68)||rg(d,68)||(s=yp(s,n),d=yp(d,r));var g=void 0;return rg(s,168,!0)&&rg(d,168,!0)?g=ce:rg(s,68,!0)||rg(d,68,!0)?g=le:(da(s)||da(d))&&(g=s===te||d===te?te:$),g&&!h(l)?g:g?(59===l&&b(g),g):(E(),$);case 27:case 29:case 30:case 31:return h(l)&&(s=cu(yp(s,n)),d=cu(yp(d,r)),Tc(s,d)||Tc(d,s)||E()),fe;case 32:case 33:case 34:case 35:var _=lu(s),v=lu(d);return _&&v||(s=_?cu(s):s,d=v?cu(d):d),ug(s,d)||ug(d,s)||E(),fe;case 93:return function(n,t,r,a){return r===ye||a===ye?ye:(!da(r)&&ag(r,32764)&&bt(n,e.Diagnostics.The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_parameter),da(a)||Dv(a)||hc(a,Fe)||bt(t,e.Diagnostics.The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_Function_interface_type),fe)}(n,r,s,d);case 92:return function(n,t,r,a){return r===ye||a===ye?ye:(r=yp(r,n),a=yp(a,t),Tc(r,le)||rg(r,3240)||bt(n,e.Diagnostics.The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol),rg(a,31522816)||bt(t,e.Diagnostics.The_right_hand_side_of_an_in_expression_must_be_of_type_any_an_object_type_or_a_type_parameter),fe)}(n,r,s,d);case 53:return 1048576&dd(s)?tl([(u=M?s:cu(d),Md(u,hu)),d]):s;case 54:return 2097152&dd(s)?tl([yu(s),d],2):s;case 58:var y=e.isBinaryExpression(n.parent)?e.getAssignmentDeclarationKind(n.parent):0;return function(n,t){if(2===n)for(var r=Lg(t,i),a=0,o=Yi(r);a<o.length;a++){var s=o[a],l=Ga(s);if(l.symbol&&32&l.symbol.flags){var c=s.escapedName,u=Ft(s.valueDeclaration,c,67897832,void 0,c,!1);if(u&&u.declarations.some(e.isJSDocTypedefTag))return Oy(u.declarations[0],e.Diagnostics.Duplicate_identifier_0,e.unescapeLeadingUnderscores(c)),Oy(s.valueDeclaration,e.Diagnostics.Duplicate_identifier_0,e.unescapeLeadingUnderscores(c))}}}(y,r),function(t){switch(t){case 2:return!0;case 1:case 5:case 6:case 3:case 4:var a=Er(n),i=e.getAssignedExpandoInitializer(r);return i&&e.isObjectLiteralExpression(i)&&a&&e.hasEntries(a.exports);default:return!1}}(y)?s:(b(d),xu(d));case 26:return!A.allowUnreachableCode&&function n(t){switch((t=e.skipParentheses(t)).kind){case 71:case 9:case 12:case 191:case 204:case 13:case 8:case 101:case 86:case 95:case 140:case 194:case 207:case 195:case 185:case 186:case 197:case 211:case 259:case 258:return!0;case 203:return n(t.whenTrue)&&n(t.whenFalse);case 202:return!e.isAssignmentOperator(t.operatorToken.kind)&&n(t.left)&&n(t.right);case 200:case 201:switch(t.operator){case 51:case 37:case 38:case 52:return!0}return!1;case 198:case 192:case 210:default:return!1}}(n)&&(71!==(c=r).kind||\"eval\"!==c.escapedText)&&bt(n,e.Diagnostics.Left_side_of_comma_operator_is_unused_and_has_no_side_effects),d;default:return e.Debug.fail()}function h(t){var a=tg(s,3072)?n:tg(d,3072)?r:void 0;return!a||(bt(a,e.Diagnostics.The_0_operator_cannot_be_applied_to_type_symbol,e.tokenToString(t)),!1)}function b(t){a&&e.isAssignmentOperator(l)&&(!ng(n,e.Diagnostics.The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access)||e.isIdentifier(n)&&\"exports\"===e.unescapeLeadingUnderscores(n.escapedText)||xc(t,s,n,r))}function E(){var n=Yr(s),r=Yr(d),a=o||t;(function(n,r,a){switch(t.kind){case 34:case 32:return bt(n,e.Diagnostics.This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap,\"false\",r,a);case 35:case 33:return bt(n,e.Diagnostics.This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap,\"true\",r,a)}})(a,n,r)||bt(a,e.Diagnostics.Operator_0_cannot_be_applied_to_types_1_and_2,e.tokenToString(t.kind),n,r)}}function pg(n){a&&(4096&n.flags&&!function(n){for(var t=n,r=n.parent;r;){if(e.isFunctionLike(r)&&t===r.body)return!1;if(e.isClassLike(t))return!0;t=r,r=r.parent}return!1}(n)||ky(n,e.Diagnostics.A_yield_expression_is_only_allowed_in_a_generator_body),pm(n)&&bt(n,e.Diagnostics.yield_expressions_cannot_be_used_in_a_parameter_initializer));var t=e.getContainingFunction(n);if(!t)return $;var r=e.getFunctionFlags(t);if(!(1&r))return $;n.asteriskToken&&(3==(3&r)&&C<6&&iy(n,26624),1==(3&r)&&C<2&&A.downlevelIteration&&iy(n,256));var i=0!=(2&r),o=jf(n,i),s=Uo(t);return s&&xc(o,P_(s,i)||$,n.expression||n,n.expression),$}function fg(n,t,r){var a=function(n){return 266!==n.kind||e.isJsxSelfClosingElement(n.parent)?n:n.parent.parent}(n),i=a.contextualType,o=a.contextualMapper;a.contextualType=t,a.contextualMapper=r;var s=Lg(n,r===x?1:r?2:3);return a.contextualType=i,a.contextualMapper=o,s}function gg(e,n){var t=Rt(e);if(!t.resolvedType){if(n)return Lg(e,n);var r=An;An=Cn,t.resolvedType=Lg(e,n),An=r}return t.resolvedType}function _g(n){return 192===(n=e.skipParentheses(n)).kind||210===n.kind}function vg(n){var t=e.getEffectiveInitializer(n),r=Tg(t,!0),a=2&e.getCombinedNodeFlags(n)||e.isDeclarationReadonly(n)||_g(t)?r:uu(r);if(e.isInJSFile(n)){if(24576&a.flags)return R&&Iu(n,$),$;if(ru(a))return R&&Iu(n,We),We}return a}function yg(e,n,t,r){2===arguments.length&&(t=Am(e));var a=Lg(e,n,r);return _g(e)?a:mu(a,t)}function hg(e,n){return 147===e.name.kind&&Bm(e.name),yg(e.initializer,n)}function bg(e,n){return Ly(e),147===e.name.kind&&Bm(e.name),Eg(e,Xf(e,n),n)}function Eg(e,n,t){if(2===t){var r=Bp(n);if(r&&r.typeParameters){var a=xm(e);if(a){var i=Bp(Tu(a));if(i&&!i.typeParameters)return Zo(Hp(r,i,Cm(e)))}}}return n}function Tg(n,t){var r=e.skipParentheses(n);if(189!==r.kind||97===r.expression.kind||e.isRequireCall(r,!0)||Tf(r)){if(192===r.kind||210===r.kind)return Ul(r.type)}else{var a=Bp(vp(r.expression));if(a&&!a.typeParameters)return Ho(a)}return t?gg(n):Lg(n)}function Sg(e){var n=Rt(e);if(n.contextFreeType)return n.contextFreeType;var t=e.contextualType;e.contextualType=$;var r=n.contextFreeType=Lg(e,1);return e.contextualType=t,r}function Lg(n,t,r){var i;return ig(i=146===n.kind?bp(n):Eg(n,function(n,t,r){switch(n.kind){case 71:return $d(n);case 99:return im(n);case 97:return lm(n);case 95:return se;case 13:case 9:return Gl(Kl(n.text));case 8:return Ry(n),Gl(Kl(+n.text));case 101:return me;case 86:return ue;case 204:return function(n){return e.forEach(n.templateSpans,function(n){tg(Lg(n.expression),3072)&&bt(n.expression,e.Diagnostics.Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_in_String)}),le}(n);case 12:return Ue;case 185:return Pm(n,t,r);case 186:return Um(n,t);case 187:return hp(n);case 188:return Ip(n);case 189:if(91===n.expression.kind)return Sf(n);case 190:return Ef(n);case 191:return function(e){return py(e,e.typeArguments),C<2&&iy(e,65536),Ho(ff(e))}(n);case 193:return function(n,t){var r=e.isInJSFile(n)?e.getJSDocTypeTag(n):void 0;return r?Af(r,r.typeExpression.type,n.expression,t):Lg(n.expression,t)}(n,t);case 207:return function(e){return j_(e),uv(e),Ga(Er(e))}(n);case 194:case 195:return Xf(n,t);case 197:return function(e){return Lg(e.expression),ct}(n);case 192:case 210:return function(e){return Af(e,e.type,e.expression)}(n);case 211:return function(e){return Tu(Lg(e.expression))}(n);case 212:return Cf(n);case 196:return function(n){Lg(n.expression);var t=e.skipParentheses(n.expression);if(187!==t.kind&&188!==t.kind)return bt(t,e.Diagnostics.The_operand_of_a_delete_operator_must_be_a_property_reference),fe;var r=xr(Rt(t).resolvedSymbol);return r&&Zf(r)&&bt(t,e.Diagnostics.The_operand_of_a_delete_operator_cannot_be_a_read_only_property),fe}(n);case 198:return function(e){return Lg(e.expression),ie}(n);case 199:return function(n){return a&&(16384&n.flags||ky(n,e.Diagnostics.await_expression_is_only_allowed_within_an_async_function),pm(n)&&bt(n,e.Diagnostics.await_expressions_cannot_be_used_in_a_parameter_initializer)),zg(Lg(n.expression),n,e.Diagnostics.Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member)}(n);case 200:return function(n){var t=Lg(n.operand);if(t===ye)return ye;if(8===n.operand.kind){if(38===n.operator)return Gl(Kl(-n.operand.text));if(37===n.operator)return Gl(Kl(+n.operand.text))}switch(n.operator){case 37:case 38:case 52:return yp(t,n.operand),tg(t,3072)&&bt(n.operand,e.Diagnostics.The_0_operator_cannot_be_applied_to_type_symbol,e.tokenToString(n.operator)),ce;case 51:C_(n.operand);var r=3145728&dd(t);return 1048576===r?ue:2097152===r?me:fe;case 43:case 44:return Qf(n.operand,yp(t,n.operand),e.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_or_an_enum_type)&&ng(n.operand,e.Diagnostics.The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access),ce}return te}(n);case 201:return function(n){var t=Lg(n.operand);return t===ye?ye:(Qf(n.operand,yp(t,n.operand),e.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_or_an_enum_type)&&ng(n.operand,e.Diagnostics.The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access),ce)}(n);case 202:return dg(n,t);case 203:return function(e,n){return C_(e.condition),tl([Lg(e.whenTrue,n),Lg(e.whenFalse,n)],2)}(n,t);case 206:return function(e,n){return C<2&&A.downlevelIteration&&iy(e,1536),O_(Lg(e.expression,n),e.expression,!1,!1)}(n,t);case 208:return ie;case 205:return pg(n);case 213:return n.type;case 268:return function(n,t){if(n.expression){var r=Lg(n.expression,t);return n.dotDotDotToken&&r!==$&&!eu(r)&&bt(n,e.Diagnostics.JSX_spread_child_must_be_an_array_type),r}return te}(n,t);case 258:case 259:return function(e,n){return uv(e),cp(e)||$}(n);case 262:return function(n,t){return dp(n.openingFragment,t),2===A.jsx&&(A.jsxFactory||e.getSourceFileOfNode(n).pragmas.has(\"jsx\"))&&bt(n,A.jsxFactory?e.Diagnostics.JSX_fragment_is_not_supported_when_using_jsxFactory:e.Diagnostics.JSX_fragment_is_not_supported_when_using_an_inline_JSX_factory_pragma),cp(n)||$}(n,t);case 266:return Xm(n,t);case 260:e.Debug.fail(\"Shouldn't ever directly check a JsxOpeningElement\")}return te}(n,t,r),t))&&(187===n.parent.kind&&n.parent.expression===n||188===n.parent.kind&&n.parent.expression===n||(71===n.kind||146===n.kind)&&Ev(n)||165===n.parent.kind&&n.parent.exprName===n||bt(n,e.Diagnostics.const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_import_declaration_or_export_assignment_or_type_query)),i}function xg(n){n.expression&&ky(n.expression,e.Diagnostics.Type_expected),lv(n.constraint),lv(n.default);var t=oi(Er(n));so(t)||bt(e.getEffectiveConstraintOfTypeParameter(n),e.Diagnostics.Type_parameter_0_has_a_circular_constraint,Yr(t)),function(e){return co(e)!==ke}(t)||bt(n.default,e.Diagnostics.Type_parameter_0_has_a_circular_default,Yr(t));var r=no(t),i=uo(t);r&&i&&Lc(i,Ai(r,i),n.default,e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1),a&&B_(n.name,e.Diagnostics.Type_parameter_name_cannot_be_0)}function Ag(n){sy(n),T_(n);var t=e.getContainingFunction(n);e.hasModifier(n,92)&&(155===t.kind&&e.nodeIsPresent(t.body)||bt(n,e.Diagnostics.A_parameter_property_is_only_allowed_in_a_constructor_implementation)),n.questionToken&&e.isBindingPattern(n.name)&&t.body&&bt(n,e.Diagnostics.A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature),n.name&&e.isIdentifier(n.name)&&(\"this\"===n.name.escapedText||\"new\"===n.name.escapedText)&&(0!==t.parameters.indexOf(n)&&bt(n,e.Diagnostics.A_0_parameter_must_be_the_first_parameter,n.name.escapedText),155!==t.kind&&159!==t.kind&&164!==t.kind||bt(n,e.Diagnostics.A_constructor_cannot_have_a_this_parameter),195===t.kind&&bt(n,e.Diagnostics.An_arrow_function_cannot_have_a_this_parameter)),!n.dotDotDotToken||e.isBindingPattern(n.name)||bc(Ga(n.symbol),We)||bt(n,e.Diagnostics.A_rest_parameter_must_be_of_an_array_type)}function Cg(n,t,r){for(var a=0,i=n.elements;a<i.length;a++){var o=i[a];if(!e.isOmittedExpression(o)){var s=o.name;if(71===s.kind&&s.escapedText===r)return bt(t,e.Diagnostics.A_type_predicate_cannot_reference_element_0_in_a_binding_pattern,r),!0;if((183===s.kind||182===s.kind)&&Cg(s,t,r))return!0}}}function Dg(n){160===n.kind?function(n){sy(n)||function(n){var t=n.parameters[0];if(1!==n.parameters.length)return Oy(t?t.name:n,e.Diagnostics.An_index_signature_must_have_exactly_one_parameter);if(t.dotDotDotToken)return Oy(t.dotDotDotToken,e.Diagnostics.An_index_signature_cannot_have_a_rest_parameter);if(e.hasModifiers(t))return Oy(t.name,e.Diagnostics.An_index_signature_parameter_cannot_have_an_accessibility_modifier);if(t.questionToken)return Oy(t.questionToken,e.Diagnostics.An_index_signature_parameter_cannot_have_a_question_mark);if(t.initializer)return Oy(t.name,e.Diagnostics.An_index_signature_parameter_cannot_have_an_initializer);if(!t.type)return Oy(t.name,e.Diagnostics.An_index_signature_parameter_must_have_a_type_annotation);if(137!==t.type.kind&&134!==t.type.kind){var r=Ul(t.type);return 4&r.flags||8&r.flags?Oy(t.name,e.Diagnostics.An_index_signature_parameter_type_cannot_be_a_type_alias_Consider_writing_0_Colon_1_Colon_2_instead,e.getTextOfNode(t.name),Yr(r),Yr(Ul(n.type))):262144&r.flags&&ag(r,64,!0)?Oy(t.name,e.Diagnostics.An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead):Oy(t.name,e.Diagnostics.An_index_signature_parameter_type_must_be_string_or_number)}n.type||Oy(n,e.Diagnostics.An_index_signature_must_have_a_type_annotation)}(n)}(n):163!==n.kind&&237!==n.kind&&164!==n.kind&&158!==n.kind&&155!==n.kind&&159!==n.kind||my(n);var t=e.getFunctionFlags(n);if(4&t||(3==(3&t)&&C<6&&iy(n,6144),2==(3&t)&&C<4&&iy(n,64),0!=(3&t)&&C<2&&iy(n,128)),H_(n.typeParameters),e.forEach(n.parameters,Ag),n.type&&lv(n.type),a){!function(n){C>=2||A.noEmit||!e.hasRestParameter(n)||4194304&n.flags||e.nodeIsMissing(n.body)||e.forEach(n.parameters,function(n){n.name&&!e.isBindingPattern(n.name)&&n.name.escapedText===K.escapedName&&bt(n,e.Diagnostics.Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters)})}(n);var r=e.getEffectiveReturnTypeNode(n);if(R&&!r)switch(n.kind){case 159:bt(n,e.Diagnostics.Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type);break;case 158:bt(n,e.Diagnostics.Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type)}if(r){var i=e.getFunctionFlags(n);if(1==(5&i)){var o=Ul(r);if(o===_e)bt(r,e.Diagnostics.A_generator_cannot_have_a_void_type_annotation);else{var s=P_(o,0!=(2&i))||$;Lc(2&i?Hs(s):js(s),o,r)}}else 2==(3&i)&&function(n,t){var r,a=Ul(t);if(C>=2){if(a===te)return;var i=Os(!0);if(i!==Ae&&!Va(a,i))return void bt(t,e.Diagnostics.The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type)}else{if(function(n){Yg(n&&e.getEntityNameFromTypeNode(n))}(t),a===te)return;var o=e.getEntityNameFromTypeNode(t);if(void 0===o)return void bt(t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value,Yr(a));var s=ir(o,67220415,!0),l=s?Ga(s):te;if(l===te)return void(71===o.kind&&\"Promise\"===o.escapedText&&Ka(a)===Os(!1)?bt(t,e.Diagnostics.An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option):bt(t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value,e.entityNameToString(o)));var c=(r=!0,en||(en=Ds(\"PromiseConstructorLike\",0,r))||Se);if(c===Se)return void bt(t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value,e.entityNameToString(o));if(!Lc(l,c,t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value))return;var u=o&&ev(o),d=wt(n.locals,u.escapedText,67220415);if(d)return void bt(d.valueDeclaration,e.Diagnostics.Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions,e.idText(u),e.entityNameToString(o))}zg(a,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member)}(n,r)}160!==n.kind&&287!==n.kind&&t_(n)}}function kg(n){for(var t=e.createMap(),r=0,a=n.members;r<a.length;r++){var i=a[r];if(151===i.kind){var o=void 0,s=i.name;switch(s.kind){case 9:case 8:o=s.text;break;case 71:o=e.idText(s);break;default:continue}t.get(o)?(bt(e.getNameOfDeclaration(i.symbol.valueDeclaration),e.Diagnostics.Duplicate_identifier_0,o),bt(i.name,e.Diagnostics.Duplicate_identifier_0,o)):t.set(o,!0)}}}function Mg(n){if(239===n.kind){var t=Er(n);if(t.declarations.length>0&&t.declarations[0]!==n)return}var r=$o(Er(n));if(r)for(var a=!1,i=!1,o=0,s=r.declarations;o<s.length;o++){var l=s[o];if(1===l.parameters.length&&l.parameters[0].type)switch(l.parameters[0].type.kind){case 137:i?bt(l,e.Diagnostics.Duplicate_string_index_signature):i=!0;break;case 134:a?bt(l,e.Diagnostics.Duplicate_number_index_signature):a=!0}}}function Og(n){sy(n)||function(n){if(e.isClassLike(n.parent)){if(Sy(n.name,e.Diagnostics.A_computed_property_name_in_a_class_property_declaration_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type))return!0}else if(239===n.parent.kind){if(Sy(n.name,e.Diagnostics.A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type))return!0;if(n.initializer)return Oy(n.initializer,e.Diagnostics.An_interface_property_cannot_have_an_initializer)}else if(166===n.parent.kind){if(Sy(n.name,e.Diagnostics.A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type))return!0;if(n.initializer)return Oy(n.initializer,e.Diagnostics.A_type_literal_property_cannot_have_an_initializer)}if(4194304&n.flags&&Ay(n),e.isPropertyDeclaration(n)&&n.exclamationToken&&(!e.isClassLike(n.parent)||!n.type||n.initializer||4194304&n.flags||e.hasModifier(n,160)))return Oy(n.exclamationToken,e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context)}(n)||vy(n.name),T_(n)}function Ig(n){Dg(n),function(n){var t=e.isInJSFile(n)?e.getJSDocTypeParameterDeclarations(n):void 0,r=n.typeParameters||t&&e.firstOrUndefined(t);if(r){var a=r.pos===r.end?r.pos:e.skipTrivia(e.getSourceFileOfNode(n).text,r.pos);return My(n,a,r.end-a,e.Diagnostics.Type_parameters_cannot_appear_on_a_constructor_declaration)}}(n)||function(n){var t=e.getEffectiveReturnTypeNode(n);t&&Oy(t,e.Diagnostics.Type_annotation_cannot_appear_on_a_constructor_declaration)}(n),lv(n.body);var t=Er(n);if(n===e.getDeclarationOfKind(t,n.kind)&&Ug(t),!e.nodeIsMissing(n.body)&&a){var r=n.parent;if(e.getEffectiveBaseTypeNode(r)){em(n.parent,r);var i=rm(r),o=tm(n);if(o){if(i&&bt(o,e.Diagnostics.A_constructor_cannot_contain_a_super_call_when_its_class_extends_null),e.some(n.parent.members,function(n){return 152===n.kind&&!e.hasModifier(n,32)&&!!n.initializer})||e.some(n.parameters,function(n){return e.hasModifier(n,92)})){for(var s=void 0,l=0,c=n.body.statements;l<c.length;l++){var u=c[l];if(219===u.kind&&e.isSuperCall(u.expression)){s=u;break}if(!e.isPrologueDirective(u))break}s||bt(n,e.Diagnostics.A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_properties_or_has_parameter_properties)}}else i||bt(n,e.Diagnostics.Constructors_for_derived_classes_must_contain_a_super_call)}}}function Rg(n){if(a){if(my(n)||function(n){var t=n.kind;if(C<1)return Oy(n.name,e.Diagnostics.Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher);if(4194304&n.flags)return Oy(n.name,e.Diagnostics.An_accessor_cannot_be_declared_in_an_ambient_context);if(void 0===n.body&&!e.hasModifier(n,128))return My(n,n.end-1,\";\".length,e.Diagnostics._0_expected,\"{\");if(n.body&&e.hasModifier(n,128))return Oy(n,e.Diagnostics.An_abstract_accessor_cannot_have_an_implementation);if(n.typeParameters)return Oy(n.name,e.Diagnostics.An_accessor_cannot_have_type_parameters);if(!function(e){return Ty(e)||e.parameters.length===(156===e.kind?0:1)}(n))return Oy(n.name,156===t?e.Diagnostics.A_get_accessor_cannot_have_parameters:e.Diagnostics.A_set_accessor_must_have_exactly_one_parameter);if(157===t){if(n.type)return Oy(n.name,e.Diagnostics.A_set_accessor_cannot_have_a_return_type_annotation);var r=n.parameters[0];if(r.dotDotDotToken)return Oy(r.dotDotDotToken,e.Diagnostics.A_set_accessor_cannot_have_rest_parameter);if(r.questionToken)return Oy(r.questionToken,e.Diagnostics.A_set_accessor_cannot_have_an_optional_parameter);if(r.initializer)return Oy(n.name,e.Diagnostics.A_set_accessor_parameter_cannot_have_an_initializer)}return!1}(n)||vy(n.name),$g(n),Dg(n),156===n.kind&&!(4194304&n.flags)&&e.nodeIsPresent(n.body)&&128&n.flags&&(256&n.flags||bt(n.name,e.Diagnostics.A_get_accessor_must_return_a_value)),147===n.name.kind&&Bm(n.name),!bi(n)){var t=156===n.kind?157:156,r=e.getDeclarationOfKind(Er(n),t);if(r){var i=e.getModifierFlags(n),o=e.getModifierFlags(r);(28&i)!=(28&o)&&bt(n.name,e.Diagnostics.Getter_and_setter_accessors_do_not_agree_in_visibility),(128&i)!=(128&o)&&bt(n.name,e.Diagnostics.Accessors_must_both_be_abstract_or_non_abstract),Ng(n,r,Ia,e.Diagnostics.get_and_set_accessor_must_have_the_same_type),Ng(n,r,Ra,e.Diagnostics.get_and_set_accessor_must_have_the_same_this_type)}}var s=Na(Er(n));156===n.kind&&Jf(n,s)}lv(n.body)}function Ng(e,n,t,r){var a=t(e),i=t(n);a&&i&&!_c(a,i)&&bt(e,r)}function wg(n,t){return No(e.map(n.typeArguments,Ul),t,Ro(t),e.isInJSFile(n))}function Pg(n,t){for(var r,a,i=!0,o=0;o<t.length;o++){var s=no(t[o]);s&&(r||(a=Jl(t,r=wg(n,t))),i=i&&Lc(r[o],lc(s,a),n.typeArguments[o],e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1))}return i}function Fg(n){var t=Ts(n);if(t!==te){var r=Rt(n).resolvedSymbol;if(r)return 524288&r.flags&&It(r).typeParameters||(4&e.getObjectFlags(t)?t.target.localTypeParameters:void 0)}}function Gg(n){py(n,n.typeArguments),162!==n.kind||void 0===n.typeName.jsdocDotPos||e.isInJSFile(n)||e.isInJSDoc(n)||My(n,n.typeName.jsdocDotPos,1,e.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments);var t=Ts(n);if(t!==te){if(n.typeArguments&&(e.forEach(n.typeArguments,lv),a)){var r=Fg(n);r&&Pg(n,r)}32&t.flags&&8&Rt(n).resolvedSymbol.flags&&bt(n,e.Diagnostics.Enum_type_0_has_members_with_initializers_that_are_not_literals,Yr(t))}}function Vg(n,t){if(!(2097152&n.flags))return n;var r=n.objectType,a=n.indexType;return bc(a,ml(r,!1))?(188===t.kind&&e.isAssignmentTarget(t)&&32&e.getObjectFlags(r)&&1&Wi(r)&&bt(t,e.Diagnostics.Index_signature_in_type_0_only_permits_reading,Yr(r)),n):To(po(r),1)&&rg(a,168)?n:(bt(t,e.Diagnostics.Type_0_cannot_be_used_to_index_type_1,Yr(a),Yr(r)),n)}function Kg(n){!function(n){if(141===n.operator){if(138!==n.type.kind)return Oy(n.type,e.Diagnostics._0_expected,e.tokenToString(138));var t=e.walkUpParenthesizedTypes(n.parent);switch(t.kind){case 235:var r=t;if(71!==r.name.kind)return Oy(n,e.Diagnostics.unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name);if(!e.isVariableDeclarationInVariableStatement(r))return Oy(n,e.Diagnostics.unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement);if(!(2&r.parent.flags))return Oy(t.name,e.Diagnostics.A_variable_whose_type_is_a_unique_symbol_type_must_be_const);break;case 152:if(!e.hasModifier(t,32)||!e.hasModifier(t,64))return Oy(t.name,e.Diagnostics.A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly);break;case 151:if(!e.hasModifier(t,64))return Oy(t.name,e.Diagnostics.A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly);break;default:Oy(n,e.Diagnostics.unique_symbol_types_are_not_allowed_here)}}}(n),lv(n.type)}function Bg(n){return e.hasModifier(n,8)&&!!(4194304&n.flags)}function Hg(n,t){var r=e.getCombinedModifierFlags(n);return 239!==n.parent.kind&&238!==n.parent.kind&&207!==n.parent.kind&&4194304&n.flags&&(2&r||e.isModuleBlock(n.parent)&&e.isModuleDeclaration(n.parent.parent)&&e.isGlobalScopeAugmentation(n.parent.parent)||(r|=1),r|=2),r&t}function Ug(n){if(a){for(var t,r,i,o=0,s=155,l=!1,c=!0,u=!1,d=n.declarations,m=0!=(16384&n.flags),p=!1,f=!1,g=0,_=d;g<_.length;g++){var v=_[g],y=4194304&v.flags,h=239===v.parent.kind||166===v.parent.kind||y;if(h&&(i=void 0),237===v.kind||154===v.kind||153===v.kind||155===v.kind){var b=Hg(v,155);o|=b,s&=b,l=l||e.hasQuestionToken(v),c=c&&e.hasQuestionToken(v),e.nodeIsPresent(v.body)&&t?m?f=!0:p=!0:i&&i.parent===v.parent&&i.end!==v.pos&&C(i),e.nodeIsPresent(v.body)?t||(t=v):u=!0,i=v,h||(r=v)}}if(f&&e.forEach(d,function(n){bt(n,e.Diagnostics.Multiple_constructor_implementations_are_not_allowed)}),p&&e.forEach(d,function(n){bt(e.getNameOfDeclaration(n),e.Diagnostics.Duplicate_function_implementation)}),!r||r.body||e.hasModifier(r,128)||r.questionToken||C(r),u&&(function(n,t,r,a,i){if(0!=(a^i)){var o=Hg(A(n,t),r);e.forEach(n,function(n){var t=Hg(n,r)^o;1&t?bt(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_exported_or_non_exported):2&t?bt(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_ambient_or_non_ambient):24&t?bt(e.getNameOfDeclaration(n)||n,e.Diagnostics.Overload_signatures_must_all_be_public_private_or_protected):128&t&&bt(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_abstract_or_non_abstract)})}}(d,t,155,o,s),function(n,t,r,a){if(r!==a){var i=e.hasQuestionToken(A(n,t));e.forEach(n,function(n){e.hasQuestionToken(n)!==i&&bt(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_optional_or_required)})}}(d,t,l,c),t))for(var E=Go(n),T=wo(t),S=0,L=E;S<L.length;S++){var x=L[S];if(!Rc(T,x)){bt(x.declaration,e.Diagnostics.Overload_signature_is_not_compatible_with_function_implementation);break}}}function A(e,n){return void 0!==n&&n.parent===e[0].parent?n:e[0]}function C(n){if(!n.name||!e.nodeIsMissing(n.name)){var t=!1,r=e.forEachChild(n.parent,function(e){if(t)return e;t=e===n});if(r&&r.pos===n.end&&r.kind===n.kind){var a=r.name||r,i=r.name;if(n.name&&i&&(e.isComputedPropertyName(n.name)&&e.isComputedPropertyName(i)||!e.isComputedPropertyName(n.name)&&!e.isComputedPropertyName(i)&&e.getEscapedTextOfIdentifierOrLiteral(n.name)===e.getEscapedTextOfIdentifierOrLiteral(i)))return void(154!==n.kind&&153!==n.kind||e.hasModifier(n,32)===e.hasModifier(r,32)||bt(a,e.hasModifier(n,32)?e.Diagnostics.Function_overload_must_be_static:e.Diagnostics.Function_overload_must_not_be_static));if(e.nodeIsPresent(r.body))return void bt(a,e.Diagnostics.Function_implementation_name_must_be_0,e.declarationNameToString(n.name))}var o=n.name||n;m?bt(o,e.Diagnostics.Constructor_implementation_is_missing):e.hasModifier(n,128)?bt(o,e.Diagnostics.All_declarations_of_an_abstract_method_must_be_consecutive):bt(o,e.Diagnostics.Function_implementation_is_missing_or_not_immediately_following_the_declaration)}}}function jg(n){if(a){var t=n.localSymbol;if((t||(t=Er(n)).exportSymbol)&&e.getDeclarationOfKind(t,n.kind)===n){for(var r=0,i=0,o=0,s=0,l=t.declarations;s<l.length;s++){var c=v(g=l[s]),u=Hg(g,513);1&u?512&u?o|=c:r|=c:i|=c}var d=r&i,m=o&(r|i);if(d||m)for(var p=0,f=t.declarations;p<f.length;p++){c=v(g=f[p]);var g,_=e.getNameOfDeclaration(g);c&m?bt(_,e.Diagnostics.Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_default_0_declaration_instead,e.declarationNameToString(_)):c&d&&bt(_,e.Diagnostics.Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local,e.declarationNameToString(_))}}}function v(n){var t=n;switch(t.kind){case 239:case 240:case 302:case 295:return 2;case 242:return e.isAmbientModule(t)||0!==e.getModuleInstanceState(t)?5:4;case 238:case 241:return 3;case 277:return 7;case 252:if(!e.isEntityNameExpression(t.expression))return 1;t=t.expression;case 246:case 249:case 248:var r=0,a=er(Er(t));return e.forEach(a.declarations,function(e){r|=v(e)}),r;case 235:case 184:case 237:case 251:return 1;default:return e.Debug.fail(e.Debug.showSyntaxKind(t))}}}function Wg(e,n,t){var r=qg(e,n);return r&&Jg(r,n,t)}function qg(n,t){if(!da(n)){var r=n;if(r.promisedTypeOfPromise)return r.promisedTypeOfPromise;if(Va(n,Os(!1)))return r.promisedTypeOfPromise=n.typeArguments[0];var a=ua(n,\"then\");if(!da(a)){var i=a?ho(a,0):e.emptyArray;if(0!==i.length){var o=md(tl(e.map(i,Gf)),524288);if(!da(o)){var s=ho(o,0);if(0!==s.length)return r.promisedTypeOfPromise=tl(e.map(s,Gf),2);t&&bt(t,e.Diagnostics.The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback)}}else t&&bt(t,e.Diagnostics.A_promise_must_have_a_then_method)}}}function zg(e,n,t){return Jg(e,n,t)||te}function Jg(n,t,r){var a=n;if(a.awaitedTypeOfType)return a.awaitedTypeOfType;if(da(n))return a.awaitedTypeOfType=n;if(262144&n.flags){for(var i=void 0,o=0,s=n.types;o<s.length;o++){var l=s[o];i=e.append(i,Jg(l,t,r))}if(!i)return;return a.awaitedTypeOfType=tl(i)}var c=qg(n);if(c){if(n.id===c.id||Jn.indexOf(c.id)>=0)return void(t&&bt(t,e.Diagnostics.Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method));Jn.push(n.id);var u=Jg(c,t,r);if(Jn.pop(),!u)return;return a.awaitedTypeOfType=u}var d=ua(n,\"then\");if(!(d&&ho(d,0).length>0))return a.awaitedTypeOfType=n;if(t){if(!r)return e.Debug.fail();bt(t,r)}}function Xg(n){var t=Ho(ff(n));if(!(1&t.flags)){var r,a,i=uf(n);switch(n.parent.kind){case 238:r=tl([Ga(Er(n.parent)),_e]);break;case 149:r=_e,a=e.chainDiagnosticMessages(void 0,e.Diagnostics.The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any);break;case 152:r=_e,a=e.chainDiagnosticMessages(void 0,e.Diagnostics.The_return_type_of_a_property_decorator_function_must_be_either_void_or_any);break;case 154:case 156:case 157:r=tl([Bs(Lv(n.parent)),_e]);break;default:return e.Debug.fail()}Lc(t,r,n,i,function(){return a})}}function Yg(e){if(e){var n=ev(e),t=2097152|(71===e.kind?67897832:1920),r=Ft(n,n.escapedText,t,void 0,void 0,!0);r&&2097152&r.flags&&Ar(r)&&!Vv(er(r))&&tr(r)}}function Qg(n){var t=function n(t){if(t)switch(t.kind){case 172:case 171:for(var r=void 0,a=0,i=t.types;a<i.length;a++){for(var o=i[a];175===o.kind;)o=o.type;if(131!==o.kind&&(M||95!==o.kind&&140!==o.kind)){var s=n(o);if(!s)return;if(r){if(!e.isIdentifier(r)||!e.isIdentifier(s)||r.escapedText!==s.escapedText)return}else r=s}}return r;case 175:return n(t.type);case 162:return t.typeName}}(n);t&&e.isEntityName(t)&&Yg(t)}function Zg(n){var t=e.getEffectiveTypeAnnotationNode(n);return e.isRestParameter(n)?e.getRestParameterElementType(t):t}function $g(n){if(n.decorators&&e.nodeCanBeDecorated(n,n.parent,n.parent.parent)){A.experimentalDecorators||bt(n,e.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_to_remove_this_warning);var t=n.decorators[0];if(iy(t,8),149===n.kind&&iy(t,32),A.emitDecoratorMetadata)switch(iy(t,16),n.kind){case 238:var r=e.getFirstConstructorWithBody(n);if(r)for(var a=0,i=r.parameters;a<i.length;a++)Qg(Zg(i[a]));break;case 156:case 157:var o=156===n.kind?157:156,s=e.getDeclarationOfKind(Er(n),o);Qg(Oa(n)||s&&Oa(s));break;case 154:for(var l=0,c=n.parameters;l<c.length;l++)Qg(Zg(c[l]));Qg(e.getEffectiveReturnTypeNode(n));break;case 152:Qg(e.getEffectiveTypeAnnotationNode(n));break;case 149:Qg(Zg(n));for(var u=0,d=n.parent.parameters;u<d.length;u++)Qg(Zg(d[u]))}e.forEach(n.decorators,Xg)}}function e_(e){switch(e.kind){case 71:return e;case 187:return e.name;default:return}}function n_(n){$g(n),Dg(n);var t=e.getFunctionFlags(n);if(n.name&&147===n.name.kind&&Bm(n.name),!bi(n)){var r=Er(n),i=n.localSymbol||r,o=e.find(i.declarations,function(e){return e.kind===n.kind&&!(65536&e.flags)});n===o&&Ug(i),r.parent&&e.getDeclarationOfKind(r,n.kind)===n&&Ug(r)}var s=153===n.kind?void 0:n.body;if(lv(s),0==(1&t)){var l=Yf(n,t);Jf(n,l)}if(a&&!e.getEffectiveReturnTypeNode(n)&&(R&&e.nodeIsMissing(s)&&!Bg(n)&&Iu(n,$),1&t&&e.nodeIsPresent(s)&&Ho(wo(n))),e.isInJSFile(n)){var c=e.getJSDocTypeTag(n);c&&c.typeExpression&&!Mm(Ul(c.typeExpression),n)&&bt(c,e.Diagnostics.The_type_of_a_function_declaration_must_match_the_function_s_signature)}}function t_(n){if(a){var t=e.getSourceFileOfNode(n),r=xn.get(t.path);r||(r=[],xn.set(t.path,r)),r.push(n)}}function r_(n,t){for(var r=0,a=n;r<a.length;r++){var i=a[r];switch(i.kind){case 238:case 207:o_(i,t),s_(i,t);break;case 239:s_(i,t);break;case 277:case 242:case 216:case 244:case 223:case 224:case 225:d_(i,t);break;case 155:case 194:case 237:case 195:case 154:case 156:case 157:i.body&&d_(i,t),s_(i,t);break;case 153:case 158:case 159:case 163:case 164:case 240:case 174:s_(i,t);break;default:e.Debug.assertNever(i,\"Node should not have been registered for unused identifiers check\")}}}function a_(n,t,r){var a=e.getNameOfDeclaration(n)||n,i=vv(n)?e.Diagnostics._0_is_declared_but_never_used:e.Diagnostics._0_is_declared_but_its_value_is_never_read;r(n,0,e.createDiagnosticForNode(a,i,t))}function i_(n){return e.isIdentifier(n)&&95===e.idText(n).charCodeAt(0)}function o_(n,t){if(!(4194304&n.flags))for(var r=0,a=n.members;r<a.length;r++){var i=a[r];switch(i.kind){case 154:case 152:case 156:case 157:if(157===i.kind&&32768&i.symbol.flags)break;var o=Er(i);!o.isReferenced&&e.hasModifier(i,8)&&t(i,0,e.createDiagnosticForNode(i.name,e.Diagnostics._0_is_declared_but_its_value_is_never_read,Jr(o)));break;case 155:for(var s=0,l=i.parameters;s<l.length;s++){var c=l[s];!c.symbol.isReferenced&&e.hasModifier(c,8)&&t(c,0,e.createDiagnosticForNode(c.name,e.Diagnostics.Property_0_is_declared_but_its_value_is_never_read,e.symbolName(c.symbol)))}break;case 160:case 215:break;default:e.Debug.fail()}}}function s_(n,t){if(!(4194304&n.flags||174!==n.kind&&e.last(Er(n).declarations)!==n))if(174===n.kind)l_(s=n.typeParameter)&&t(n,1,e.createDiagnosticForNode(n,e.Diagnostics._0_is_declared_but_its_value_is_never_read,e.idText(s.name)));else for(var r=e.getEffectiveTypeParameterDeclarations(n),a=new e.NodeSet,i=0,o=r;i<o.length;i++){var s;if(l_(s=o[i])){var l=e.idText(s.name),c=s.parent;if(174!==c.kind&&c.typeParameters.every(l_)){if(a.tryAdd(c)){var u=e.isJSDocTemplateTag(c)?e.rangeOfNode(c):e.rangeOfTypeParameters(c.typeParameters),d=1===r.length,m=d?e.Diagnostics._0_is_declared_but_its_value_is_never_read:e.Diagnostics.All_type_parameters_are_unused,p=d?l:void 0;t(s,1,e.createFileDiagnostic(e.getSourceFileOfNode(c),u.pos,u.end-u.pos,m,p))}}else t(s,1,e.createDiagnosticForNode(s,e.Diagnostics._0_is_declared_but_its_value_is_never_read,l))}}}function l_(e){return!(262144&br(e.symbol).isReferenced||i_(e.name))}function c_(e,n,t,r){var a=String(r(n)),i=e.get(a);i?i[1].push(t):e.set(a,[n,[t]])}function u_(n){return e.tryCast(e.getRootDeclaration(n),e.isParameter)}function d_(n,t){if(!(4194304&n.flags)){var r=e.createMap(),a=e.createMap(),i=e.createMap();n.locals.forEach(function(n){var o;if(262144&n.flags?3&n.flags&&!(3&n.isReferenced):!n.isReferenced&&!n.exportSymbol)for(var l=0,c=n.declarations;l<c.length;l++){var u=c[l];if(!(e.isAmbientModule(u)||(e.isVariableDeclaration(u)&&e.isForInOrOfStatement(u.parent.parent)||p_(u))&&i_(u.name)))if(p_(u))c_(r,248===(o=u).kind?o:249===o.kind?o.parent:o.parent.parent,u,s);else if(e.isBindingElement(u)&&e.isObjectBindingPattern(u.parent))u!==e.last(u.parent.elements)&&e.last(u.parent.elements).dotDotDotToken||c_(a,u.parent,u,s);else if(e.isVariableDeclaration(u))c_(i,u.parent,u,s);else{var d=n.valueDeclaration&&u_(n.valueDeclaration),m=n.valueDeclaration&&e.getNameOfDeclaration(n.valueDeclaration);d&&m?e.isParameterPropertyDeclaration(d)||e.parameterIsThisKeyword(d)||i_(m)||t(d,1,e.createDiagnosticForNode(m,e.Diagnostics._0_is_declared_but_its_value_is_never_read,e.symbolName(n))):a_(u,e.symbolName(n),t)}}}),r.forEach(function(n){var r=n[0],a=n[1],i=r.parent;if((r.name?1:0)+(r.namedBindings?249===r.namedBindings.kind?1:r.namedBindings.elements.length:0)===a.length)t(i,0,1===a.length?e.createDiagnosticForNode(i,e.Diagnostics._0_is_declared_but_its_value_is_never_read,e.idText(e.first(a).name)):e.createDiagnosticForNode(i,e.Diagnostics.All_imports_in_import_declaration_are_unused));else for(var o=0,s=a;o<s.length;o++){var l=s[o];a_(l,e.idText(l.name),t)}}),a.forEach(function(n){var r=n[0],a=n[1],o=u_(r.parent)?1:0;if(r.elements.length===a.length)1===a.length&&235===r.parent.kind&&236===r.parent.parent.kind?c_(i,r.parent.parent,r.parent,s):t(r,o,1===a.length?e.createDiagnosticForNode(r,e.Diagnostics._0_is_declared_but_its_value_is_never_read,m_(e.first(a).name)):e.createDiagnosticForNode(r,e.Diagnostics.All_destructured_elements_are_unused));else for(var l=0,c=a;l<c.length;l++){var u=c[l];t(u,o,e.createDiagnosticForNode(u,e.Diagnostics._0_is_declared_but_its_value_is_never_read,m_(u.name)))}}),i.forEach(function(n){var r=n[0],a=n[1];if(r.declarations.length===a.length)t(r,0,1===a.length?e.createDiagnosticForNode(e.first(a).name,e.Diagnostics._0_is_declared_but_its_value_is_never_read,m_(e.first(a).name)):e.createDiagnosticForNode(217===r.parent.kind?r.parent:r,e.Diagnostics.All_variables_are_unused));else for(var i=0,o=a;i<o.length;i++){var s=o[i];t(s,0,e.createDiagnosticForNode(s,e.Diagnostics._0_is_declared_but_its_value_is_never_read,m_(s.name)))}})}}function m_(n){switch(n.kind){case 71:return e.idText(n);case 183:case 182:return m_(e.cast(e.first(n.elements),e.isBindingElement).name);default:return e.Debug.assertNever(n)}}function p_(e){return 248===e.kind||251===e.kind||249===e.kind}function f_(n){if(216===n.kind&&Iy(n),e.isFunctionOrModuleBlock(n)){var t=kn;e.forEach(n.statements,lv),kn=t}else e.forEach(n.statements,lv);n.locals&&t_(n)}function g_(n,t,r){if(!t||t.escapedText!==r)return!1;if(152===n.kind||151===n.kind||154===n.kind||153===n.kind||156===n.kind||157===n.kind)return!1;if(4194304&n.flags)return!1;var a=e.getRootDeclaration(n);return 149!==a.kind||!e.nodeIsMissing(a.parent.body)}function __(n){e.findAncestor(n,function(t){return!!(4&Wv(t))&&(71!==n.kind?bt(e.getNameOfDeclaration(n),e.Diagnostics.Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference):bt(n,e.Diagnostics.Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference),!0)})}function v_(n){e.findAncestor(n,function(t){return!!(8&Wv(t))&&(71!==n.kind?bt(e.getNameOfDeclaration(n),e.Diagnostics.Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_meta_property_reference):bt(n,e.Diagnostics.Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta_property_reference),!0)})}function y_(n,t){if(!(D>=e.ModuleKind.ES2015||A.noEmit)&&(g_(n,t,\"require\")||g_(n,t,\"exports\"))&&(!e.isModuleDeclaration(n)||1===e.getModuleInstanceState(n))){var r=ca(n);277===r.kind&&e.isExternalOrCommonJsModule(r)&&bt(t,e.Diagnostics.Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module,e.declarationNameToString(t),e.declarationNameToString(t))}}function h_(n,t){if(!(C>=4||A.noEmit)&&g_(n,t,\"Promise\")&&(!e.isModuleDeclaration(n)||1===e.getModuleInstanceState(n))){var r=ca(n);277===r.kind&&e.isExternalOrCommonJsModule(r)&&1024&r.flags&&bt(t,e.Diagnostics.Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_functions,e.declarationNameToString(t),e.declarationNameToString(t))}}function b_(n){if(149===e.getRootDeclaration(n).kind){var t=e.getContainingFunction(n);!function r(a){if(!e.isTypeNode(a)&&!e.isDeclarationName(a)){if(187===a.kind)return r(a.expression);if(71!==a.kind)return e.forEachChild(a,r);var i=Ft(a,a.escapedText,69317567,void 0,void 0,!1);if(i&&i!==Q&&i.valueDeclaration)if(i.valueDeclaration!==n){var o=e.getEnclosingBlockScopeContainer(i.valueDeclaration);if(o===t){if(149===i.valueDeclaration.kind||184===i.valueDeclaration.kind){if(i.valueDeclaration.pos<n.pos)return;if(e.findAncestor(a,function(t){return t===n.initializer?\"quit\":e.isFunctionLike(t.parent)||152===t.parent.kind&&!e.hasModifier(t.parent,32)&&e.isClassLike(t.parent.parent)}))return}bt(a,e.Diagnostics.Initializer_of_parameter_0_cannot_reference_identifier_1_declared_after_it,e.declarationNameToString(n.name),e.declarationNameToString(a))}}else bt(a,e.Diagnostics.Parameter_0_cannot_be_referenced_in_its_initializer,e.declarationNameToString(n.name))}}(n.initializer)}}function E_(e){return e===ee?$:e===qe?We:e}function T_(n){if($g(n),e.isBindingElement(n)||lv(n.type),n.name){if(147===n.name.kind&&(Bm(n.name),n.initializer&&gg(n.initializer)),184===n.kind){182===n.parent.kind&&C<6&&iy(n,4),n.propertyName&&147===n.propertyName.kind&&Bm(n.propertyName);var t=n.parent.parent,r=ma(t),a=n.propertyName||n.name;if(!e.isBindingPattern(a)){var i=e.getTextOfPropertyName(a);if(i){var o=vo(r,i);kp(o,void 0,!1),t.initializer&&o&&gp(t,97===t.initializer.kind,r,o)}}}if(e.isBindingPattern(n.name)&&(183===n.name.kind&&C<2&&A.downlevelIteration&&iy(n,512),e.forEach(n.name.elements,lv)),n.initializer&&149===e.getRootDeclaration(n).kind&&e.nodeIsMissing(e.getContainingFunction(n).body))bt(n,e.Diagnostics.A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation);else if(e.isBindingPattern(n.name)){if(n.initializer&&224!==n.parent.parent.kind){var s=gg(n.initializer);M&&0===n.name.elements.length?yp(s,n):xc(s,Ca(n),n,n.initializer),b_(n)}}else{var l=Er(n),c=E_(Ga(l));if(n===l.valueDeclaration){var u=e.getEffectiveInitializer(n);u&&(e.isInJSFile(n)&&e.isObjectLiteralExpression(u)&&(0===u.properties.length||e.isPrototypeAccess(n.name))&&e.hasEntries(l.exports)||224===n.parent.parent.kind||(xc(gg(u),c,n,u,void 0),b_(n)))}else{var d=E_(Ca(n));c===te||d===te||_c(c,d)||67108864&l.flags||S_(c,n,d),n.initializer&&xc(gg(n.initializer),d,n,n.initializer,void 0),m=n,p=l.valueDeclaration,149===m.kind&&235===p.kind||235===m.kind&&149===p.kind||e.hasQuestionToken(m)===e.hasQuestionToken(p)&&e.getSelectedModifierFlags(m,504)===e.getSelectedModifierFlags(p,504)||(bt(e.getNameOfDeclaration(l.valueDeclaration),e.Diagnostics.All_declarations_of_0_must_have_identical_modifiers,e.declarationNameToString(n.name)),bt(n.name,e.Diagnostics.All_declarations_of_0_must_have_identical_modifiers,e.declarationNameToString(n.name)))}152!==n.kind&&151!==n.kind&&(jg(n),235!==n.kind&&184!==n.kind||function(n){if(0==(3&e.getCombinedNodeFlags(n))&&!e.isParameterDeclaration(n)&&(235!==n.kind||n.initializer)){var t=Er(n);if(1&t.flags){if(!e.isIdentifier(n.name))return e.Debug.fail();var r=Ft(n,n.name.escapedText,3,void 0,void 0,!1);if(r&&r!==t&&2&r.flags&&3&pp(r)){var a=e.getAncestor(r.valueDeclaration,236),i=217===a.parent.kind&&a.parent.parent?a.parent.parent:void 0;if(!i||!(216===i.kind&&e.isFunctionLike(i.parent)||243===i.kind||242===i.kind||277===i.kind)){var o=Jr(r);bt(n,e.Diagnostics.Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1,o,o)}}}}}(n),y_(n,n.name),h_(n,n.name))}}var m,p}function S_(n,t,r){var a=e.getNameOfDeclaration(t);bt(a,152===t.kind||151===t.kind?e.Diagnostics.Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_type_2:e.Diagnostics.Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_type_2,e.declarationNameToString(a),Yr(n),Yr(r))}function L_(n){return function(n){if(224!==n.parent.parent.kind&&225!==n.parent.parent.kind)if(4194304&n.flags)Ay(n);else if(!n.initializer){if(e.isBindingPattern(n.name)&&!e.isBindingPattern(n.parent))return Oy(n,e.Diagnostics.A_destructuring_declaration_must_have_an_initializer);if(e.isVarConst(n))return Oy(n,e.Diagnostics.const_declarations_must_be_initialized)}if(n.exclamationToken&&(217!==n.parent.parent.kind||!n.type||n.initializer||4194304&n.flags))return Oy(n.exclamationToken,e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context);A.module===e.ModuleKind.ES2015||A.module===e.ModuleKind.ESNext||A.module===e.ModuleKind.System||A.noEmit||4194304&n.parent.parent.flags||!e.hasModifier(n.parent.parent,1)||function n(t){if(71===t.kind){if(\"__esModule\"===e.idText(t))return Oy(t,e.Diagnostics.Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules)}else for(var r=t.elements,a=0,i=r;a<i.length;a++){var o=i[a];if(!e.isOmittedExpression(o))return n(o.name)}return!1}(n.name),(e.isLet(n)||e.isVarConst(n))&&function n(t){if(71===t.kind){if(110===t.originalKeywordKind)return Oy(t,e.Diagnostics.let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations)}else for(var r=t.elements,a=0,i=r;a<i.length;a++){var o=i[a];e.isOmittedExpression(o)||n(o.name)}return!1}(n.name)}(n),T_(n)}function x_(n){return function(n){if(n.dotDotDotToken){var t=n.parent.elements;if(n!==e.last(t))return Oy(n,e.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern);if(cy(t,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma),n.propertyName)return Oy(n.name,e.Diagnostics.A_rest_element_cannot_have_a_property_name);n.initializer&&My(n,n.initializer.pos-1,1,e.Diagnostics.A_rest_element_cannot_have_an_initializer)}}(n),T_(n)}function A_(n){sy(n)||Cy(n.declarationList)||function(n){if(!function e(n){switch(n.kind){case 220:case 221:case 222:case 229:case 223:case 224:case 225:return!1;case 231:return e(n.parent)}return!0}(n.parent)){if(e.isLet(n.declarationList))return Oy(n,e.Diagnostics.let_declarations_can_only_be_declared_inside_a_block);e.isVarConst(n.declarationList)&&Oy(n,e.Diagnostics.const_declarations_can_only_be_declared_inside_a_block)}}(n),e.forEach(n.declarationList.declarations,lv)}function C_(n,t){var r=Lg(n,t);return 4096&r.flags&&bt(n,e.Diagnostics.An_expression_of_type_void_cannot_be_tested_for_truthiness),r}function D_(n){Ey(n);var t,r=vp(n.expression);if(236===n.initializer.kind){var a=n.initializer.declarations[0];a&&e.isBindingPattern(a.name)&&bt(a.name,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern),k_(n)}else{var i=n.initializer,o=Lg(i);185===i.kind||186===i.kind?bt(i,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern):bc(32768&(t=pl(ml(r))).flags?le:t,o)?ng(i,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access):bt(i,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any)}r!==ve&&rg(r,31522816)||bt(n.expression,e.Diagnostics.The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_here_has_type_0,Yr(r)),lv(n.statement),n.locals&&t_(n)}function k_(e){var n=e.initializer;n.declarations.length>=1&&L_(n.declarations[0])}function M_(e,n){return O_(vp(e),e,!0,void 0!==n)}function O_(e,n,t,r){return da(e)?e:I_(e,n,t,r,!0)||$}function I_(n,t,r,a,i){if(n!==ve){var o=C>=2,s=!o&&A.downlevelIteration;if(o||s||a){var l=R_(n,o?t:void 0,a,!0,i);if(l||o)return l}var c=n,u=!1,d=!1;if(r){if(262144&c.flags){var m=n.types,p=e.filter(m,function(e){return!(68&e.flags)});p!==m&&(c=tl(p,2))}else 68&c.flags&&(c=ve);if((d=c!==n)&&(C<1&&t&&(bt(t,e.Diagnostics.Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher),u=!0),32768&c.flags))return le}if(!tu(c)){if(t&&!u){var f=!!R_(n,void 0,a,!0,i);bt(t,!r||d?s?e.Diagnostics.Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator:f?e.Diagnostics.Type_0_is_not_an_array_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators:e.Diagnostics.Type_0_is_not_an_array_type:s?e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator:f?e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators:e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type,Yr(c))}return d?le:void 0}var g=So(c,1);return d&&g?68&g.flags?le:tl([g,le],2):g}N_(t,n,a)}function R_(n,t,r,a,i){if(!da(n))return Md(n,function(n){var o=n;if(r){if(o.iteratedTypeOfAsyncIterable)return o.iteratedTypeOfAsyncIterable;if(Va(n,Rs(!1))||Va(n,ws(!1)))return o.iteratedTypeOfAsyncIterable=n.typeArguments[0]}if(a){if(o.iteratedTypeOfIterable)return o.iteratedTypeOfIterable;if(Va(n,Ps(!1))||Va(n,Gs(!1)))return o.iteratedTypeOfIterable=n.typeArguments[0]}var s=r&&ua(n,e.getPropertyNameForKnownSymbolName(\"asyncIterator\")),l=s||(a?ua(n,e.getPropertyNameForKnownSymbolName(\"iterator\")):void 0);if(!da(l)){var c=l?ho(l,0):void 0;if(e.some(c)){var u=w_(tl(e.map(c,Ho),2),t,!!s);return i&&t&&u&&Lc(n,s?function(e){return Ks(Rs(!0),[e])}(u):Us(u),t),s?o.iteratedTypeOfAsyncIterable=u:o.iteratedTypeOfIterable=u}t&&(N_(t,n,r),t=void 0)}})}function N_(n,t,r){bt(n,r?e.Diagnostics.Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator:e.Diagnostics.Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator,Yr(t))}function w_(n,t,r){if(!da(n)){var a=n;if(r?a.iteratedTypeOfAsyncIterator:a.iteratedTypeOfIterator)return r?a.iteratedTypeOfAsyncIterator:a.iteratedTypeOfIterator;if(Va(n,(r?Ns:Fs)(!1)))return r?a.iteratedTypeOfAsyncIterator=n.typeArguments[0]:a.iteratedTypeOfIterator=n.typeArguments[0];var i=ua(n,\"next\");if(!da(i)){var o=i?ho(i,0):e.emptyArray;if(0!==o.length){var s=tl(e.map(o,Ho),2);if(!(da(s)||r&&da(s=Wg(s,t,e.Diagnostics.The_type_returned_by_the_next_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_property)))){var l=s&&ua(s,\"value\");if(l)return r?a.iteratedTypeOfAsyncIterator=l:a.iteratedTypeOfIterator=l;t&&bt(t,r?e.Diagnostics.The_type_returned_by_the_next_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_property:e.Diagnostics.The_type_returned_by_the_next_method_of_an_iterator_must_have_a_value_property)}}else t&&bt(t,r?e.Diagnostics.An_async_iterator_must_have_a_next_method:e.Diagnostics.An_iterator_must_have_a_next_method)}}}function P_(e,n){if(!da(e))return R_(e,void 0,n,!n,!1)||w_(e,void 0,n)}function F_(n){Iy(n)||function(n){for(var t=n;t;){if(e.isFunctionLike(t))return Oy(n,e.Diagnostics.Jump_target_cannot_cross_function_boundary);switch(t.kind){case 231:if(n.label&&t.label.escapedText===n.label.escapedText){var r=226===n.kind&&!e.isIterationStatement(t.statement,!0);return!!r&&Oy(n,e.Diagnostics.A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement)}break;case 230:if(227===n.kind&&!n.label)return!1;break;default:if(e.isIterationStatement(t,!1)&&!n.label)return!1}t=t.parent}if(n.label){var a=227===n.kind?e.Diagnostics.A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement:e.Diagnostics.A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement;return Oy(n,a)}var a=227===n.kind?e.Diagnostics.A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement:e.Diagnostics.A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement;Oy(n,a)}(n)}function G_(n,t){var r=2==(3&e.getFunctionFlags(n))?qg(t):t;return!!r&&tg(r,4099)}function V_(n){Iy(n)||void 0===n.expression&&function(n,t,r,a,i){var o=e.getSourceFileOfNode(n);if(!Dy(o)){var s=e.getSpanOfTokenAtPosition(o,n.pos);Xn.add(e.createFileDiagnostic(o,e.textSpanEnd(s),0,t,r,a,i))}}(n,e.Diagnostics.Line_break_not_permitted_here),n.expression&&Lg(n.expression)}function K_(n){var t,r=es(n.symbol,1),a=es(n.symbol,0),i=So(n,0),o=So(n,1);if(i||o){e.forEach(Yi(n),function(e){var t=Ga(e);p(e,t,n,a,i,0),p(e,t,n,r,o,1)});var s=n.symbol.valueDeclaration;if(1&e.getObjectFlags(n)&&e.isClassLike(s))for(var l=0,c=s.members;l<c.length;l++){var u=c[l];if(!e.hasModifier(u,32)&&bi(u)){var d=Er(u),m=Ga(d);p(d,m,n,a,i,0),p(d,m,n,r,o,1)}}}function p(n,t,r,a,i,o){if(i&&!e.isKnownSymbol(n)){var s,l=n.valueDeclaration,c=l&&e.getNameOfDeclaration(l);if(1!==o||(c?Gm(c):Km(n.escapedName)))l&&c&&(202===l.kind||147===c.kind||n.parent===r.symbol)?s=l:a?s=a:2&e.getObjectFlags(r)&&(s=e.forEach(Za(r),function(e){return Qi(e,n.escapedName)&&So(e,o)})?void 0:r.symbol.declarations[0]),s&&!bc(t,i)&&bt(s,0===o?e.Diagnostics.Property_0_of_type_1_is_not_assignable_to_string_index_type_2:e.Diagnostics.Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2,Jr(n),Yr(t),Yr(i))}}i&&o&&!(t=r||a)&&2&e.getObjectFlags(n)&&(t=e.forEach(Za(n),function(e){return So(e,0)&&So(e,1)})?void 0:n.symbol.declarations[0]),t&&!bc(o,i)&&bt(t,e.Diagnostics.Numeric_index_type_0_is_not_assignable_to_string_index_type_1,Yr(o),Yr(i))}function B_(e,n){switch(e.escapedText){case\"any\":case\"unknown\":case\"number\":case\"boolean\":case\"string\":case\"symbol\":case\"void\":case\"object\":bt(e,n,e.escapedText)}}function H_(n){if(n)for(var t=!1,r=0;r<n.length;r++){var i=n[r];if(xg(i),a){i.default?t=!0:t&&bt(i,e.Diagnostics.Required_type_parameters_may_not_follow_optional_type_parameters);for(var o=0;o<r;o++)n[o].symbol===i.symbol&&bt(i.name,e.Diagnostics.Duplicate_identifier_0,e.declarationNameToString(i.name))}}}function U_(n){if(1!==n.declarations.length){var t=It(n);if(!t.typeParametersChecked){t.typeParametersChecked=!0;var r=function(n){return e.filter(n.declarations,function(e){return 238===e.kind||239===e.kind})}(n);if(r.length<=1)return;if(!function(n,t){for(var r=e.length(t),a=Ro(t),i=0,o=n;i<o.length;i++){var s=o[i],l=e.getEffectiveTypeParameterDeclarations(s),c=l.length;if(c<a||c>r)return!1;for(var u=0;u<c;u++){var d=l[u],m=t[u];if(d.name.escapedText!==m.symbol.escapedName)return!1;var p=e.getEffectiveConstraintOfTypeParameter(d),f=p&&Ul(p),g=no(m);if(f&&(!g||!_c(f,g)))return!1;var _=d.default&&Ul(d.default),v=uo(m);if(_&&v&&!_c(_,v))return!1}}return!0}(r,si(n).localTypeParameters))for(var a=Jr(n),i=0,o=r;i<o.length;i++)bt(o[i].name,e.Diagnostics.All_declarations_of_0_must_have_identical_type_parameters,a)}}}function j_(n){var t;!function(n){var t=e.getSourceFileOfNode(n);(function(n){var t=!1,r=!1;if(!sy(n)&&n.heritageClauses)for(var a=0,i=n.heritageClauses;a<i.length;a++){var o=i[a];if(85===o.token){if(t)return ky(o,e.Diagnostics.extends_clause_already_seen);if(r)return ky(o,e.Diagnostics.extends_clause_must_precede_implements_clause);if(o.types.length>1)return ky(o.types[1],e.Diagnostics.Classes_can_only_extend_a_single_class);t=!0}else{if(e.Debug.assert(108===o.token),r)return ky(o,e.Diagnostics.implements_clause_already_seen);r=!0}gy(o)}})(n)||uy(n.typeParameters,t)}(n),$g(n),n.name&&(B_(n.name,e.Diagnostics.Class_name_cannot_be_0),y_(n,n.name),h_(n,n.name),4194304&n.flags||(t=n.name,1===C&&\"Object\"===t.escapedText&&D!==e.ModuleKind.ES2015&&D!==e.ModuleKind.ESNext&&bt(t,e.Diagnostics.Class_name_cannot_be_Object_when_targeting_ES5_with_module_0,e.ModuleKind[D]))),H_(e.getEffectiveTypeParameterDeclarations(n)),jg(n);var r=Er(n),i=si(r),o=Ai(i),s=Ga(r);U_(r),function(n){var t;!function(e){e[e.Getter=1]=\"Getter\",e[e.Setter=2]=\"Setter\",e[e.Method=4]=\"Method\",e[e.Property=3]=\"Property\"}(t||(t={}));for(var r=e.createUnderscoreEscapedMap(),a=e.createUnderscoreEscapedMap(),i=0,o=n.members;i<o.length;i++){var s=o[i];if(155===s.kind)for(var l=0,c=s.parameters;l<c.length;l++){var u=c[l];e.isParameterPropertyDeclaration(u)&&!e.isBindingPattern(u.name)&&f(r,u.name,u.name.escapedText,3)}else{var d=e.hasModifier(s,32)?a:r,m=s.name,p=m&&e.getPropertyNameForPropertyNameNode(m);if(m&&p)switch(s.kind){case 156:f(d,m,p,1);break;case 157:f(d,m,p,2);break;case 152:f(d,m,p,3);break;case 154:f(d,m,p,4)}}}function f(n,t,r,a){var i=n.get(r);i?4&i?4!==a&&bt(t,e.Diagnostics.Duplicate_identifier_0,e.getTextOfNode(t)):i&a?bt(t,e.Diagnostics.Duplicate_identifier_0,e.getTextOfNode(t)):n.set(r,i|a):n.set(r,a)}}(n),4194304&n.flags||function(n){for(var t=0,r=n.members;t<r.length;t++){var a=r[t],i=a.name;if(e.hasModifier(a,32)&&i){var o=e.getPropertyNameForPropertyNameNode(i);switch(o){case\"name\":case\"length\":case\"caller\":case\"arguments\":case\"prototype\":bt(i,e.Diagnostics.Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1,o,ta(Er(n)))}}}}(n);var l=e.getEffectiveBaseTypeNode(n);if(l){C<2&&iy(l.parent,1);var c=Za(i);if(c.length&&a){var u=c[0],d=Qa(i),m=po(d);!function(n,t){var r=ho(n,1);if(r.length){var a=r[0].declaration;if(a&&e.hasModifier(a,8)){var i=e.getClassLikeDeclarationOfSymbol(n.symbol);bv(t,i)||bt(t,e.Diagnostics.Cannot_extend_a_class_0_Class_constructor_is_marked_as_private,ar(n.symbol))}}}(m,l),lv(l.expression);var p=e.getClassExtendsHeritageElement(n);if(p&&p!==l&&Lg(p.expression),e.some(l.typeArguments)){e.forEach(l.typeArguments,lv);for(var f=0,g=Xa(m,l.typeArguments,l);f<g.length&&Pg(l,g[f].typeParameters);f++);}if(Lc(o,T=Ai(u,i.thisType),void 0)||W_(n,o,T,e.Diagnostics.Class_0_incorrectly_extends_base_class_1),Lc(s,gc(m),n.name||n,e.Diagnostics.Class_static_side_0_incorrectly_extends_base_class_static_side_1),2162688&d.flags&&!qa(s)&&bt(n.name||n,e.Diagnostics.A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any),!(m.symbol&&32&m.symbol.flags||2162688&d.flags)){var _=Ya(m,l.typeArguments,l);e.forEach(_,function(e){return!gf(e.declaration)&&Ho(e)!==u})&&bt(l.expression,e.Diagnostics.Base_constructors_must_all_have_the_same_return_type)}!function(n,t){for(var r=$i(t),a=0,i=r;a<i.length;a++){var o=i[a],s=q_(o);if(!(4194304&s.flags)){var l=q_(Qi(n,s.escapedName)),c=e.getDeclarationModifierFlagsFromSymbol(s);if(e.Debug.assert(!!l,\"derived should point to something, even if it is the base class' declaration.\"),l)if(l===s){var u=e.getClassLikeDeclarationOfSymbol(n.symbol);!(128&c)||u&&e.hasModifier(u,128)||(207===u.kind?bt(u,e.Diagnostics.Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1,Jr(o),Yr(t)):bt(u,e.Diagnostics.Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2,Yr(n),Jr(o),Yr(t)))}else{var d=e.getDeclarationModifierFlagsFromSymbol(l);if(8&c||8&d)continue;if(fp(s)||98308&s.flags&&98308&l.flags)continue;var m=void 0;m=fp(s)?98304&l.flags?e.Diagnostics.Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_accessor:e.Diagnostics.Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_property:98304&s.flags?e.Diagnostics.Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_function:e.Diagnostics.Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_function,bt(e.getNameOfDeclaration(l.valueDeclaration)||l.valueDeclaration,m,Yr(t),Jr(s),Yr(n))}}}}(i,u)}}var v=e.getClassImplementsHeritageClauseElements(n);if(v)for(var y=0,h=v;y<h.length;y++){var b=h[y];if(e.isEntityNameExpression(b.expression)||bt(b.expression,e.Diagnostics.A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments),Gg(b),a){var E=Ul(b);if(E!==te)if($a(E)){var T,S=E.symbol&&32&E.symbol.flags?e.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass:e.Diagnostics.Class_0_incorrectly_implements_interface_1;Lc(o,T=Ai(E,i.thisType),void 0)||W_(n,o,T,S)}else bt(b,e.Diagnostics.A_class_may_only_implement_another_class_or_interface)}}a&&(K_(i),Mg(n),function(n){if(M&&I&&!(4194304&n.flags))for(var t=Cr(n),r=0,a=n.members;r<a.length;r++){var i=a[r];if(z_(i)){var o=i.name;if(e.isIdentifier(o)){var s=Ga(Er(i));3&s.flags||8192&vu(s)||t&&J_(o,s,t)||bt(i.name,e.Diagnostics.Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor,e.declarationNameToString(o))}}}}(n))}function W_(n,t,r,a){for(var i=!1,o=function(n){if(e.hasStaticModifier(n))return\"continue\";var a=n.name&&Sv(n.name)||Sv(n);if(a){var o=vo(t,a.escapedName),s=vo(r,a.escapedName);o&&s&&(Lc(Ga(o),Ga(s),n.name||n,void 0,function(){return e.chainDiagnosticMessages(void 0,e.Diagnostics.Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2,Jr(a),Yr(t),Yr(r))})||(i=!0))}},s=0,l=n.members;s<l.length;s++)o(l[s]);i||Lc(t,r,n.name||n,a)}function q_(n){return 1&e.getCheckFlags(n)?n.target:n}function z_(n){return 152===n.kind&&!e.hasModifier(n,160)&&!n.exclamationToken&&!n.initializer}function J_(n,t,r){var a=e.createPropertyAccess(e.createThis(),n);return a.expression.parent=a,a.parent=r,a.flowNode=r.returnFlowNode,!(8192&vu(jd(a,t,Eu(t))))}function X_(n){if(sy(n)||function(n){var t=!1;if(n.heritageClauses)for(var r=0,a=n.heritageClauses;r<a.length;r++){var i=a[r];if(85!==i.token)return e.Debug.assert(108===i.token),ky(i,e.Diagnostics.Interface_declaration_cannot_have_implements_clause);if(t)return ky(i,e.Diagnostics.extends_clause_already_seen);t=!0,gy(i)}}(n),H_(n.typeParameters),a){B_(n.name,e.Diagnostics.Interface_name_cannot_be_0),jg(n);var t=Er(n);if(U_(t),n===e.getDeclarationOfKind(t,239)){var r=si(t),i=Ai(r);if(function(n,t){var r=Za(n);if(r.length<2)return!0;var a=e.createUnderscoreEscapedMap();e.forEach(gi(n).declaredProperties,function(e){a.set(e.escapedName,{prop:e,containingType:n})});for(var i=!0,o=0,s=r;o<s.length;o++)for(var l=s[o],c=0,u=$i(Ai(l,n.thisType));c<u.length;c++){var d=u[c],m=a.get(d.escapedName);if(m){if(m.containingType!==n&&0===Qc(m.prop,d,vc)){i=!1;var p=Yr(m.containingType),f=Yr(l),g=e.chainDiagnosticMessages(void 0,e.Diagnostics.Named_property_0_of_types_1_and_2_are_not_identical,Jr(d),p,f);g=e.chainDiagnosticMessages(g,e.Diagnostics.Interface_0_cannot_simultaneously_extend_types_1_and_2,Yr(n),p,f),Xn.add(e.createDiagnosticForNodeFromMessageChain(t,g))}}else a.set(d.escapedName,{prop:d,containingType:l})}return i}(r,n.name)){for(var o=0,s=Za(r);o<s.length;o++)Lc(i,Ai(s[o],r.thisType),n.name,e.Diagnostics.Interface_0_incorrectly_extends_interface_1);K_(r)}}kg(n)}e.forEach(e.getInterfaceBaseTypeNodes(n),function(n){e.isEntityNameExpression(n.expression)||bt(n.expression,e.Diagnostics.An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments),Gg(n)}),e.forEach(n.members,lv),a&&(Mg(n),t_(n))}function Y_(e){var n=Rt(e);if(!(16384&n.flags)){n.flags|=16384;for(var t=0,r=0,a=e.members;r<a.length;r++){var i=a[r],o=Q_(i,t);Rt(i).enumMemberValue=o,t=\"number\"==typeof o?o+1:void 0}}}function Q_(n,t){if(pa(n.name))bt(n.name,e.Diagnostics.Computed_property_names_are_not_allowed_in_enums);else{var r=e.getTextOfPropertyName(n.name);Km(r)&&!Vm(r)&&bt(n.name,e.Diagnostics.An_enum_member_cannot_have_a_numeric_name)}return n.initializer?function(n){var t=ti(Er(n.parent)),r=e.isEnumConst(n.parent),a=n.initializer,i=1!==t||ni(n)?function t(r){switch(r.kind){case 200:var a=t(r.operand);if(\"number\"==typeof a)switch(r.operator){case 37:return a;case 38:return-a;case 52:return~a}break;case 202:var i=t(r.left),s=t(r.right);if(\"number\"==typeof i&&\"number\"==typeof s)switch(r.operatorToken.kind){case 49:return i|s;case 48:return i&s;case 46:return i>>s;case 47:return i>>>s;case 45:return i<<s;case 50:return i^s;case 39:return i*s;case 41:return i/s;case 37:return i+s;case 38:return i-s;case 42:return i%s;case 40:return Math.pow(i,s)}else if(\"string\"==typeof i&&\"string\"==typeof s&&37===r.operatorToken.kind)return i+s;break;case 9:return r.text;case 8:return Ry(r),+r.text;case 193:return t(r.expression);case 71:var l=r;return Vm(l.escapedText)?+l.escapedText:e.nodeIsMissing(r)?0:o(r,Er(n.parent),l.escapedText);case 188:case 187:var c=r;if(function e(n){return 71===n.kind||187===n.kind&&e(n.expression)||188===n.kind&&e(n.expression)&&9===n.argumentExpression.kind}(c)){var u=Tg(c.expression);if(u.symbol&&384&u.symbol.flags){var d=void 0;if(187===c.kind)d=c.name.escapedText;else{var m=c.argumentExpression;e.Debug.assert(e.isLiteralExpression(m)),d=e.escapeLeadingUnderscores(m.text)}return o(r,u.symbol,d)}}}}(a):void 0;if(void 0!==i)r&&\"number\"==typeof i&&!isFinite(i)&&bt(a,isNaN(i)?e.Diagnostics.const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN:e.Diagnostics.const_enum_member_initializer_was_evaluated_to_a_non_finite_value);else{if(1===t)return bt(a,e.Diagnostics.Computed_values_are_not_permitted_in_an_enum_with_string_valued_members),0;r?bt(a,e.Diagnostics.In_const_enum_declarations_member_initializer_must_be_constant_expression):4194304&n.parent.flags?bt(a,e.Diagnostics.In_ambient_enum_declarations_member_initializer_must_be_constant_expression):Lc(Lg(a),si(Er(n.parent)),a,void 0)}return i;function o(t,r,a){var i=r.exports.get(a);if(i){var o=i.valueDeclaration;if(o!==n)return Pt(o,n)?qv(o):(bt(t,e.Diagnostics.A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_members_defined_in_other_enums),0)}}}(n):4194304&n.parent.flags&&!e.isEnumConst(n.parent)?void 0:void 0!==t?t:void bt(n.name,e.Diagnostics.Enum_member_must_have_initializer)}function Z_(n){if(a){var t=e.isGlobalScopeAugmentation(n),r=4194304&n.flags;t&&!r&&bt(n.name,e.Diagnostics.Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambient_context);var i=e.isAmbientModule(n);if(av(n,i?e.Diagnostics.An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file:e.Diagnostics.A_namespace_declaration_is_only_allowed_in_a_namespace_or_module))return;sy(n)||r||9!==n.name.kind||Oy(n.name,e.Diagnostics.Only_ambient_modules_can_use_quoted_names),e.isIdentifier(n.name)&&(y_(n,n.name),h_(n,n.name)),jg(n);var o=Er(n);if(512&o.flags&&o.declarations.length>1&&!r&&c(n,!!A.preserveConstEnums||!!A.isolatedModules)){var s=function(n){for(var t=0,r=n.declarations;t<r.length;t++){var a=r[t];if((238===a.kind||237===a.kind&&e.nodeIsPresent(a.body))&&!(4194304&a.flags))return a}}(o);s&&(e.getSourceFileOfNode(n)!==e.getSourceFileOfNode(s)?bt(n.name,e.Diagnostics.A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merged):n.pos<s.pos&&bt(n.name,e.Diagnostics.A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged));var l=e.getDeclarationOfKind(o,238);l&&(m=n,p=l,f=e.getEnclosingBlockScopeContainer(m),g=e.getEnclosingBlockScopeContainer(p),Nt(f)?Nt(g):!Nt(g)&&f===g)&&(Rt(n).flags|=32768)}if(i)if(e.isExternalModuleAugmentation(n)){if((t||33554432&Er(n).flags)&&n.body)for(var u=0,d=n.body.statements;u<d.length;u++)$_(d[u],t)}else Nt(n.parent)?t?bt(n.name,e.Diagnostics.Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations):e.isExternalModuleNameRelative(e.getTextOfIdentifierOrLiteral(n.name))&&bt(n.name,e.Diagnostics.Ambient_module_declaration_cannot_specify_relative_module_name):bt(n.name,t?e.Diagnostics.Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations:e.Diagnostics.Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces)}var m,p,f,g;n.body&&(lv(n.body),e.isGlobalScopeAugmentation(n)||t_(n))}function $_(n,t){switch(n.kind){case 217:for(var r=0,a=n.declarationList.declarations;r<a.length;r++)$_(a[r],t);break;case 252:case 253:ky(n,e.Diagnostics.Exports_and_export_assignments_are_not_permitted_in_module_augmentations);break;case 246:case 247:ky(n,e.Diagnostics.Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_module);break;case 184:case 235:var i=n.name;if(e.isBindingPattern(i)){for(var o=0,s=i.elements;o<s.length;o++)$_(s[o],t);break}case 238:case 241:case 237:case 239:case 242:case 240:if(t)return;var l=Er(n);if(l){var c=!(33554432&l.flags);c||(c=e.isExternalModuleAugmentation(l.parent.declarations[0]))}}}function ev(e){switch(e.kind){case 71:return e;case 146:do{e=e.left}while(71!==e.kind);return e;case 187:do{e=e.expression}while(71!==e.kind);return e}}function nv(n){var t=e.getExternalModuleName(n);if(!t||e.nodeIsMissing(t))return!1;if(!e.isStringLiteral(t))return bt(t,e.Diagnostics.String_literal_expected),!1;var r=243===n.parent.kind&&e.isAmbientModule(n.parent.parent);return 277===n.parent.kind||r?!(r&&e.isExternalModuleNameRelative(t.text)&&!ea(n)&&(bt(n,e.Diagnostics.Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relative_module_name),1)):(bt(t,253===n.kind?e.Diagnostics.Export_declarations_are_not_permitted_in_a_namespace:e.Diagnostics.Import_declarations_in_a_namespace_cannot_reference_a_module),!1)}function tv(n){var t=Er(n),r=er(t);if(r!==Q){var a=(68268991&t.flags?67220415:0)|(67897832&t.flags?67897832:0)|(1920&t.flags?1920:0);r.flags&a&&bt(n,255===n.kind?e.Diagnostics.Export_declaration_conflicts_with_exported_declaration_of_0:e.Diagnostics.Import_declaration_conflicts_with_local_declaration_of_0,Jr(t)),!A.isolatedModules||255!==n.kind||67220415&r.flags||4194304&n.flags||bt(n,e.Diagnostics.Cannot_re_export_a_type_when_the_isolatedModules_flag_is_provided)}}function rv(e){y_(e,e.name),h_(e,e.name),tv(e)}function av(e,n){var t=277===e.parent.kind||243===e.parent.kind||242===e.parent.kind;return t||ky(e,n),!t}function iv(n){if(tv(n),e.getEmitDeclarations(A)&&aa(n.propertyName||n.name,!0),!n.parent.parent.moduleSpecifier){var t=n.propertyName||n.name,r=Ft(t,t.escapedText,70107135,void 0,void 0,!0);r&&(r===G||Nt(ca(r.declarations[0])))?bt(t,e.Diagnostics.Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module,e.idText(t)):nr(n)}}function ov(n){var t=Er(n),r=It(t);if(!r.exportsChecked){var a=t.exports.get(\"export=\");if(a&&function(n){return e.forEachEntry(n.exports,function(e,n){return\"export=\"!==n})}(t)){var i=Wt(a)||a.valueDeclaration;ea(i)||e.isInJSFile(i)||bt(i,e.Diagnostics.An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements)}var o=vr(t);o&&o.forEach(function(n,t){var r=n.declarations,a=n.flags;if(\"__export\"!==t&&!(1984&a)){var i=e.countWhere(r,vt);if(!(524288&a&&i<=2)&&i>1)for(var o=0,s=r;o<s.length;o++){var l=s[o];sv(l)&&Xn.add(e.createDiagnosticForNode(l,e.Diagnostics.Cannot_redeclare_exported_variable_0,e.unescapeLeadingUnderscores(t)))}}}),r.exportsChecked=!0}}function sv(e){return 237!==e.kind&&154!==e.kind||!!e.body}function lv(n){if(n){e.isInJSFile(n)&&e.forEach(n.jsDoc,function(n){var t=n.tags;return e.forEach(t,lv)});var t=n.kind;if(m)switch(t){case 242:case 238:case 239:case 237:m.throwIfCancellationRequested()}switch(t){case 148:return xg(n);case 149:return Ag(n);case 152:case 151:return Og(n);case 163:case 164:case 158:case 159:case 160:return Dg(n);case 154:case 153:return function(n){Ly(n)||vy(n.name),n_(n),e.hasModifier(n,128)&&154===n.kind&&n.body&&bt(n,e.Diagnostics.Method_0_cannot_have_an_implementation_because_it_is_marked_abstract,e.declarationNameToString(n.name))}(n);case 155:return Ig(n);case 156:case 157:return Rg(n);case 162:return Gg(n);case 161:return function(n){var t=function(e){switch(e.parent.kind){case 195:case 158:case 237:case 194:case 163:case 154:case 153:var n=e.parent;if(e===n.type)return n}}(n);if(t){var r=Bo(wo(t));if(r){lv(n.type);var a=n.parameterName;if(e.isThisTypePredicate(r))Hl(a);else if(r.parameterIndex>=0)t.parameters[r.parameterIndex].dotDotDotToken?bt(a,e.Diagnostics.A_type_predicate_cannot_reference_a_rest_parameter):Lc(r.type,Lv(t.parameters[r.parameterIndex]),n.type,void 0,function(){return e.chainDiagnosticMessages(void 0,e.Diagnostics.A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type)});else if(a){for(var i=!1,o=0,s=t.parameters;o<s.length;o++){var l=s[o].name;if(e.isBindingPattern(l)&&Cg(l,a,r.parameterName)){i=!0;break}}i||bt(n.parameterName,e.Diagnostics.Cannot_find_parameter_0,r.parameterName)}}}else bt(n,e.Diagnostics.A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods)}(n);case 165:return function(e){Ls(e)}(n);case 166:return function(n){e.forEach(n.members,lv),a&&(K_(Ml(n)),Mg(n),kg(n))}(n);case 167:return function(e){lv(e.elementType)}(n);case 168:return function(n){for(var t=n.elementTypes,r=!1,a=0;a<t.length;a++){var i=t[a];if(170===i.kind){if(a!==t.length-1){Oy(i,e.Diagnostics.A_rest_element_must_be_last_in_a_tuple_type);break}eu(Ul(i))||bt(i,e.Diagnostics.A_rest_element_type_must_be_an_array_type)}else if(169===i.kind)r=!0;else if(r){Oy(i,e.Diagnostics.A_required_element_cannot_follow_an_optional_element);break}}cy(n.elementTypes),e.forEach(n.elementTypes,lv)}(n);case 171:case 172:return function(n){e.forEach(n.types,lv)}(n);case 175:case 169:case 170:return lv(n.type);case 176:return function(e){Hl(e)}(n);case 177:return Kg(n);case 173:return function(n){e.forEachChild(n,lv)}(n);case 174:return function(n){e.findAncestor(n,function(e){return e.parent&&173===e.parent.kind&&e.parent.extendsType===e})||Oy(n,e.Diagnostics.infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type),lv(n.typeParameter),t_(n)}(n);case 181:return function(e){lv(e.argument),Ul(e)}(n);case 293:return function(n){var t=e.getJSDocHost(n);if(e.isClassDeclaration(t)||e.isClassExpression(t)){var r=e.getJSDocTags(t).filter(e.isJSDocAugmentsTag);e.Debug.assert(r.length>0),r.length>1&&bt(r[1],e.Diagnostics.Class_declarations_cannot_have_more_than_one_augments_or_extends_tag);var a=e_(n.class.expression),i=e.getClassExtendsHeritageElement(t);if(i){var o=e_(i.expression);o&&a.escapedText!==o.escapedText&&bt(a,e.Diagnostics.JSDoc_0_1_does_not_match_the_extends_2_clause,e.idText(n.tagName),e.idText(a),e.idText(o))}}else bt(t,e.Diagnostics.JSDoc_0_is_not_attached_to_a_class,e.idText(n.tagName))}(n);case 302:case 295:return function(n){n.typeExpression||bt(n.name,e.Diagnostics.JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags),n.name&&B_(n.name,e.Diagnostics.Type_alias_name_cannot_be_0),lv(n.typeExpression)}(n);case 301:return function(e){lv(e.constraint);for(var n=0,t=e.typeParameters;n<t.length;n++)lv(t[n])}(n);case 300:return function(e){lv(e.typeExpression)}(n);case 297:return function(n){if(lv(n.typeExpression),!e.getParameterSymbolFromJSDoc(n)){var t=e.getHostSignatureFromJSDoc(n);if(t){var r=e.getJSDocTags(t).filter(e.isJSDocParameterTag).indexOf(n);if(r>-1&&r<t.parameters.length&&e.isBindingPattern(t.parameters[r].name))return;Fo(t)?e.findLast(e.getJSDocTags(t),e.isJSDocParameterTag)===n&&n.typeExpression&&n.typeExpression.type&&!eu(Ul(n.typeExpression.type))&&bt(n.name,e.Diagnostics.JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_had_an_array_type,e.idText(146===n.name.kind?n.name.right:n.name)):bt(n.name,e.Diagnostics.JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name,e.idText(146===n.name.kind?n.name.right:n.name))}}}(n);case 287:Dg(n);case 285:case 284:case 282:case 283:case 290:return cv(n),void e.forEachChild(n,lv);case 288:return void function(n){cv(n),lv(n.type);var t=n.parent;if(e.isParameter(t)&&e.isJSDocFunctionType(t.parent))e.last(t.parent.parameters)!==t&&bt(n,e.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list);else{e.isJSDocTypeExpression(t)||bt(n,e.Diagnostics.JSDoc_may_only_appear_in_the_last_parameter_of_a_signature);var r=n.parent.parent;if(e.isJSDocParameterTag(r)){var a=e.getParameterSymbolFromJSDoc(r);if(a){var i=e.getHostSignatureFromJSDoc(r);i&&e.last(i.parameters).symbol===a||bt(n,e.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list)}}else bt(n,e.Diagnostics.JSDoc_may_only_appear_in_the_last_parameter_of_a_signature)}}(n);case 281:return lv(n.type);case 178:return function(e){lv(e.objectType),lv(e.indexType),Vg(bl(e),e)}(n);case 179:return function(n){lv(n.typeParameter),lv(n.type),R&&!n.type&&Iu(n,$),Lc(Ki(El(n)),Te,e.getEffectiveConstraintOfTypeParameter(n.typeParameter))}(n);case 237:return function(e){a&&(n_(e),yy(e),y_(e,e.name),h_(e,e.name))}(n);case 216:case 243:return f_(n);case 217:return A_(n);case 219:return function(e){Iy(e),Lg(e.expression)}(n);case 220:return function(n){Iy(n),C_(n.expression),lv(n.thenStatement),218===n.thenStatement.kind&&bt(n.thenStatement,e.Diagnostics.The_body_of_an_if_statement_cannot_be_the_empty_statement),lv(n.elseStatement)}(n);case 221:return function(e){Iy(e),lv(e.statement),C_(e.expression)}(n);case 222:return function(e){Iy(e),C_(e.expression),lv(e.statement)}(n);case 223:return function(n){Iy(n)||n.initializer&&236===n.initializer.kind&&Cy(n.initializer),n.initializer&&(236===n.initializer.kind?e.forEach(n.initializer.declarations,L_):Lg(n.initializer)),n.condition&&C_(n.condition),n.incrementor&&Lg(n.incrementor),lv(n.statement),n.locals&&t_(n)}(n);case 224:return D_(n);case 225:return function(n){if(Ey(n),n.awaitModifier?2==(6&e.getFunctionFlags(e.getContainingFunction(n)))&&C<6&&iy(n,16384):A.downlevelIteration&&C<2&&iy(n,256),236===n.initializer.kind)k_(n);else{var t=n.initializer,r=M_(n.expression,n.awaitModifier);if(185===t.kind||186===t.kind)cg(t,r||te);else{var a=Lg(t);ng(t,e.Diagnostics.The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access),r&&xc(r,a,t,n.expression)}}lv(n.statement),n.locals&&t_(n)}(n);case 226:case 227:return F_(n);case 228:return function(n){if(!Iy(n)){var t=e.getContainingFunction(n);if(t){var r=Ho(wo(t)),a=e.getFunctionFlags(t),i=1&a;if(M||n.expression||32768&r.flags){var o=n.expression?gg(n.expression):ae;if(i)return;if(157===t.kind)n.expression&&bt(n,e.Diagnostics.Setters_cannot_return_a_value);else if(155===t.kind)n.expression&&!xc(o,r,n,n.expression)&&bt(n,e.Diagnostics.Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class);else if(Uo(t))if(2&a){var s=qg(r),l=zg(o,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);s&&Lc(l,s,n)}else xc(o,r,n,n.expression)}else 155===t.kind||!A.noImplicitReturns||G_(t,r)||i||bt(n,e.Diagnostics.Not_all_code_paths_return_a_value)}else ky(n,e.Diagnostics.A_return_statement_can_only_be_used_within_a_function_body)}}(n);case 229:return function(n){Iy(n)||16384&n.flags&&ky(n,e.Diagnostics.with_statements_are_not_allowed_in_an_async_function_block),Lg(n.expression);var t=e.getSourceFileOfNode(n);if(!Dy(t)){var r=e.getSpanOfTokenAtPosition(t,n.pos).start;My(t,r,n.statement.pos-r,e.Diagnostics.The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any)}}(n);case 230:return function(n){var t;Iy(n);var r=!1,i=Lg(n.expression),o=lu(i);e.forEach(n.caseBlock.clauses,function(s){if(270===s.kind&&!r)if(void 0===t)t=s;else{var l=e.getSourceFileOfNode(n),c=e.skipTrivia(l.text,s.pos);My(l,c,(s.statements.length>0?s.statements[0].pos:s.end)-c,e.Diagnostics.A_default_clause_cannot_appear_more_than_once_in_a_switch_statement),r=!0}if(a&&269===s.kind){var u=Lg(s.expression),d=lu(u),m=i;d&&o||(u=d?cu(u):u,m=cu(i)),ug(m,u)||Oc(u,m,s.expression,void 0)}e.forEach(s.statements,lv)}),n.caseBlock.locals&&t_(n.caseBlock)}(n);case 231:return function(n){Iy(n)||e.findAncestor(n.parent,function(t){return e.isFunctionLike(t)?\"quit\":231===t.kind&&t.label.escapedText===n.label.escapedText&&(Oy(n.label,e.Diagnostics.Duplicate_label_0,e.getTextOfNode(n.label)),!0)}),lv(n.statement)}(n);case 232:return V_(n);case 233:return function(n){Iy(n),f_(n.tryBlock);var t=n.catchClause;if(t){if(t.variableDeclaration)if(t.variableDeclaration.type)ky(t.variableDeclaration.type,e.Diagnostics.Catch_clause_variable_cannot_have_a_type_annotation);else if(t.variableDeclaration.initializer)ky(t.variableDeclaration.initializer,e.Diagnostics.Catch_clause_variable_cannot_have_an_initializer);else{var r=t.block.locals;r&&e.forEachKey(t.locals,function(n){var t=r.get(n);t&&0!=(2&t.flags)&&Oy(t.valueDeclaration,e.Diagnostics.Cannot_redeclare_identifier_0_in_catch_clause,n)})}f_(t.block)}n.finallyBlock&&f_(n.finallyBlock)}(n);case 235:return L_(n);case 184:return x_(n);case 238:return function(n){n.name||e.hasModifier(n,512)||ky(n,e.Diagnostics.A_class_declaration_without_the_default_modifier_must_have_a_name),j_(n),e.forEach(n.members,lv),t_(n)}(n);case 239:return X_(n);case 240:return function(n){sy(n),B_(n.name,e.Diagnostics.Type_alias_name_cannot_be_0),H_(n.typeParameters),lv(n.type),t_(n)}(n);case 241:return function(n){if(a){sy(n),B_(n.name,e.Diagnostics.Enum_name_cannot_be_0),y_(n,n.name),h_(n,n.name),jg(n),Y_(n);var t=e.isEnumConst(n);A.isolatedModules&&t&&4194304&n.flags&&bt(n.name,e.Diagnostics.Ambient_const_enums_are_not_allowed_when_the_isolatedModules_flag_is_provided);var r=Er(n);if(n===e.getDeclarationOfKind(r,n.kind)){r.declarations.length>1&&e.forEach(r.declarations,function(n){e.isEnumDeclaration(n)&&e.isEnumConst(n)!==t&&bt(e.getNameOfDeclaration(n),e.Diagnostics.Enum_declarations_must_all_be_const_or_non_const)});var i=!1;e.forEach(r.declarations,function(n){if(241!==n.kind)return!1;var t=n;if(!t.members.length)return!1;var r=t.members[0];r.initializer||(i?bt(r.name,e.Diagnostics.In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enum_element):i=!0)})}}}(n);case 242:return Z_(n);case 247:return function(n){if(!av(n,e.Diagnostics.An_import_declaration_can_only_be_used_in_a_namespace_or_module)&&(!sy(n)&&e.hasModifiers(n)&&ky(n,e.Diagnostics.An_import_declaration_cannot_have_modifiers),nv(n))){var t=n.importClause;t&&(t.name&&rv(t),t.namedBindings&&(249===t.namedBindings.kind?rv(t.namedBindings):sr(n,n.moduleSpecifier)&&e.forEach(t.namedBindings.elements,rv)))}}(n);case 246:return function(n){if(!av(n,e.Diagnostics.An_import_declaration_can_only_be_used_in_a_namespace_or_module)&&(sy(n),e.isInternalModuleImportEqualsDeclaration(n)||nv(n)))if(rv(n),e.hasModifier(n,1)&&nr(n),257!==n.moduleReference.kind){var t=er(Er(n));if(t!==Q){if(67220415&t.flags){var r=ev(n.moduleReference);1920&ir(r,67221439).flags||bt(r,e.Diagnostics.Module_0_is_hidden_by_a_local_declaration_with_the_same_name,e.declarationNameToString(r))}67897832&t.flags&&B_(n.name,e.Diagnostics.Import_name_cannot_be_0)}}else D>=e.ModuleKind.ES2015&&!(4194304&n.flags)&&Oy(n,e.Diagnostics.Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_ns_from_mod_import_a_from_mod_import_d_from_mod_or_another_module_format_instead)}(n);case 253:return function(n){if(!av(n,e.Diagnostics.An_export_declaration_can_only_be_used_in_a_module)&&(!sy(n)&&e.hasModifiers(n)&&ky(n,e.Diagnostics.An_export_declaration_cannot_have_modifiers),!n.moduleSpecifier||nv(n)))if(n.exportClause){e.forEach(n.exportClause.elements,iv);var t=243===n.parent.kind&&e.isAmbientModule(n.parent.parent),r=!t&&243===n.parent.kind&&!n.moduleSpecifier&&4194304&n.flags;277===n.parent.kind||t||r||bt(n,e.Diagnostics.Export_declarations_are_not_permitted_in_a_namespace)}else{var a=sr(n,n.moduleSpecifier);a&&pr(a)&&bt(n.moduleSpecifier,e.Diagnostics.Module_0_uses_export_and_cannot_be_used_with_export_Asterisk,Jr(a)),D!==e.ModuleKind.System&&D!==e.ModuleKind.ES2015&&D!==e.ModuleKind.ESNext&&iy(n,32768)}}(n);case 252:return function(n){if(!av(n,e.Diagnostics.An_export_assignment_can_only_be_used_in_a_module)){var t=277===n.parent.kind?n.parent:n.parent.parent;242!==t.kind||e.isAmbientModule(t)?(!sy(n)&&e.hasModifiers(n)&&ky(n,e.Diagnostics.An_export_assignment_cannot_have_modifiers),71===n.expression.kind?(nr(n),e.getEmitDeclarations(A)&&aa(n.expression,!0)):gg(n.expression),ov(t),4194304&n.flags&&!e.isEntityNameExpression(n.expression)&&Oy(n.expression,e.Diagnostics.The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context),!n.isExportEquals||4194304&n.flags||(D>=e.ModuleKind.ES2015?Oy(n,e.Diagnostics.Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or_another_module_format_instead):D===e.ModuleKind.System&&Oy(n,e.Diagnostics.Export_assignment_is_not_supported_when_module_flag_is_system))):n.isExportEquals?bt(n,e.Diagnostics.An_export_assignment_cannot_be_used_in_a_namespace):bt(n,e.Diagnostics.A_default_export_can_only_be_used_in_an_ECMAScript_style_module)}}(n);case 218:case 234:return void Iy(n);case 256:return function(e){$g(e)}(n)}}}function cv(n){e.isInJSFile(n)||Oy(n,e.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments)}function uv(e){if(dn){var n=\"\"+s(e);dn.set(n,e)}}function dv(){dn.forEach(function(n){switch(n.kind){case 194:case 195:case 154:case 153:!function(n){e.Debug.assert(154!==n.kind||e.isObjectLiteralMethod(n));var t=e.getFunctionFlags(n),r=Yf(n,t);if(0==(1&t)&&Jf(n,r),n.body)if(e.getEffectiveReturnTypeNode(n)||Ho(wo(n)),216===n.body.kind)lv(n.body);else{var a=Lg(n.body);r&&xc(2==(3&t)?zg(a,n.body,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member):a,r,n.body,n.body)}}(n);break;case 156:case 157:Rg(n);break;case 207:!function(n){e.forEach(n.members,lv),t_(n)}(n);break;case 259:!function(e){dp(e,0)}(n);break;case 258:!function(e){dp(e.openingElement,0),qm(e.closingElement.tagName)?Qm(e.closingElement):Lg(e.closingElement.tagName)}(n)}})}function mv(n){e.performance.mark(\"beforeCheck\"),function(n){var t=Rt(n);if(!(1&t.flags)){if(e.skipTypeChecking(n,A))return;!function(n){4194304&n.flags&&function(n){for(var t=0,r=n.statements;t<r.length;t++){var a=r[t];if((e.isDeclaration(a)||217===a.kind)&&(239!==(i=a).kind&&240!==i.kind&&247!==i.kind&&246!==i.kind&&253!==i.kind&&252!==i.kind&&245!==i.kind&&!e.hasModifier(i,515)&&ky(i,e.Diagnostics.A_declare_modifier_is_required_for_a_top_level_declaration_in_a_d_ts_file)))return!0}var i}(n)}(n),e.clear(qn),e.clear(zn),dn=e.createMap(),e.forEach(n.statements,lv),dv(),e.isExternalOrCommonJsModule(n)&&t_(n),n.isDeclarationFile||!A.noUnusedLocals&&!A.noUnusedParameters||r_(fv(n),function(n,t,r){!e.containsParseError(n)&&pv(t)&&Xn.add(r)}),dn=void 0,e.isExternalOrCommonJsModule(n)&&ov(n),qn.length&&(e.forEach(qn,__),e.clear(qn)),zn.length&&(e.forEach(zn,v_),e.clear(zn)),t.flags|=1}}(n),e.performance.mark(\"afterCheck\"),e.performance.measure(\"Check\",\"beforeCheck\",\"afterCheck\")}function pv(n){switch(n){case 0:return!!A.noUnusedLocals;case 1:return!!A.noUnusedParameters;default:return e.Debug.assertNever(n)}}function fv(n){return xn.get(n.path)||e.emptyArray}function gv(n,t){try{return m=t,function(n){if(_v(),n){var t=Xn.getGlobalDiagnostics(),a=t.length;mv(n);var i=Xn.getDiagnostics(n.fileName),o=Xn.getGlobalDiagnostics();if(o!==t){var s=e.relativeComplement(t,o,e.compareDiagnostics);return e.concatenate(s,i)}return 0===a&&o.length>0?e.concatenate(o,i):i}return e.forEach(r.getSourceFiles(),mv),Xn.getDiagnostics()}(n)}finally{m=void 0}}function _v(){if(!a)throw new Error(\"Trying to get diagnostics from a type checker that does not produce them.\")}function vv(e){switch(e.kind){case 148:case 238:case 239:case 240:case 241:return!0;default:return!1}}function yv(e){for(;146===e.parent.kind;)e=e.parent;return 162===e.parent.kind}function hv(n,t){for(var r;(n=e.getContainingClass(n))&&!(r=t(n)););return r}function bv(e,n){return!!hv(e,function(e){return e===n})}function Ev(e){return void 0!==function(e){for(;146===e.parent.kind;)e=e.parent;return 246===e.parent.kind?e.parent.moduleReference===e?e.parent:void 0:252===e.parent.kind&&e.parent.expression===e?e.parent:void 0}(e)}function Tv(n){if(e.isDeclarationName(n))return Er(n.parent);if(e.isInJSFile(n)&&187===n.parent.kind&&n.parent===n.parent.parent.left){var t=function(n){switch(e.getAssignmentDeclarationKind(n.parent.parent)){case 1:case 3:return Er(n.parent);case 4:case 2:case 5:return Er(n.parent.parent)}}(n);if(t)return t}if(252===n.parent.kind&&e.isEntityNameExpression(n)){var r=ir(n,70107135,!0);if(r&&r!==Q)return r}else if(!e.isPropertyAccessExpression(n)&&Ev(n)){var a=e.getAncestor(n,246);return e.Debug.assert(void 0!==a),rr(n,!0)}if(!e.isPropertyAccessExpression(n)){var i=function(n){for(var t=n.parent;e.isQualifiedName(t);)n=t,t=t.parent;if(t&&181===t.kind&&t.qualifier===n)return t}(n);if(i){Ul(i);var o=Rt(n).resolvedSymbol;return o===Q?void 0:o}}for(;e.isRightSideOfQualifiedNameOrPropertyAccess(n);)n=n.parent;if(function(e){for(;187===e.parent.kind;)e=e.parent;return 209===e.parent.kind}(n)){var s=0;209===n.parent.kind?(s=67897832,e.isExpressionWithTypeArgumentsInClassExtendsClause(n.parent)&&(s|=67220415)):s=1920,s|=2097152;var l=e.isEntityNameExpression(n)?ir(n,s):void 0;if(l)return l}if(297===n.parent.kind)return e.getParameterSymbolFromJSDoc(n.parent);if(148===n.parent.kind&&301===n.parent.parent.kind){e.Debug.assert(!e.isInJSFile(n));var c=e.getTypeParameterFromJsDoc(n.parent);return c&&c.symbol}if(e.isExpressionNode(n)){if(e.nodeIsMissing(n))return;if(71===n.kind){if(e.isJSXTagName(n)&&qm(n)){var u=Qm(n.parent);return u===Q?void 0:u}return ir(n,67220415,!1,!0)}if(187===n.kind||146===n.kind){var d=Rt(n);return d.resolvedSymbol?d.resolvedSymbol:(187===n.kind?hp(n):bp(n),d.resolvedSymbol)}}else{if(yv(n))return ir(n,s=162===n.parent.kind?67897832:1920,!1,!0);if(265===n.parent.kind)return lp(n.parent)}return 161===n.parent.kind?ir(n,1):void 0}function Sv(n){if(277===n.kind)return e.isExternalModule(n)?br(n.symbol):void 0;var t=n.parent,r=t.parent;if(!(8388608&n.flags)){if(u(n))return Er(t);if(e.isLiteralComputedPropertyDeclarationName(n))return Er(t.parent);if(71===n.kind){if(Ev(n))return Tv(n);if(184===t.kind&&182===r.kind&&n===t.propertyName){var a=vo(Lv(r),n.escapedText);if(a)return a}}switch(n.kind){case 71:case 187:case 146:return Tv(n);case 99:var i=e.getThisContainer(n,!1);if(e.isFunctionLike(i)){var o=wo(i);if(o.thisParameter)return o.thisParameter}if(e.isInExpressionContext(n))return Lg(n).symbol;case 176:return Hl(n).symbol;case 97:return Lg(n).symbol;case 123:var s=n.parent;return s&&155===s.kind?s.parent.symbol:void 0;case 9:case 13:if(e.isExternalModuleImportEqualsDeclaration(n.parent.parent)&&e.getExternalModuleImportEqualsDeclarationExpression(n.parent.parent)===n||(247===n.parent.kind||253===n.parent.kind)&&n.parent.moduleSpecifier===n||e.isInJSFile(n)&&e.isRequireCall(n.parent,!1)||e.isImportCall(n.parent)||e.isLiteralTypeNode(n.parent)&&e.isLiteralImportTypeNode(n.parent.parent)&&n.parent.parent.argument===n.parent)return sr(n,n);case 8:var l=e.isElementAccessExpression(t)?t.argumentExpression===n?Tg(t.expression):void 0:e.isLiteralTypeNode(t)&&e.isIndexedAccessTypeNode(r)?Ul(r.objectType):void 0;return l&&vo(l,e.escapeLeadingUnderscores(n.text));case 79:case 89:case 36:case 75:return Er(n.parent);case 181:return e.isLiteralImportTypeNode(n)?Sv(n.argument.literal):void 0;default:return}}}function Lv(n){if(8388608&n.flags)return te;if(e.isPartOfTypeNode(n)){var t=Ul(n);return e.isExpressionWithTypeArgumentsInClassImplementsClause(n)?Ai(t,xv(n).thisType):t}if(e.isExpressionNode(n))return Av(n);if(e.isExpressionWithTypeArgumentsInClassExtendsClause(n)){var r=xv(n),a=e.firstOrUndefined(Za(r));return a?Ai(a,r.thisType):te}var i,o;if(vv(n))return si(o=Er(n));if(71===(i=n).kind&&vv(i.parent)&&i.parent.name===i)return(o=Sv(n))?si(o):te;if(e.isDeclaration(n))return Ga(o=Er(n));if(u(n))return(o=Sv(n))?Ga(o):te;if(e.isBindingPattern(n))return ha(n.parent,!0)||te;if(Ev(n)&&(o=Sv(n))){var s=si(o);return s!==te?s:Ga(o)}return te}function xv(e){return ei(Er(e.parent.parent))}function Av(n){return e.isRightSideOfQualifiedNameOrPropertyAccess(n)&&(n=n.parent),Vl(Tg(n))}function Cv(n){n=po(n);var t=e.createSymbolTable($i(n));return Dv(n)&&e.forEach($i(Fe),function(e){t.has(e.escapedName)||t.set(e.escapedName,e)}),Rr(t)}function Dv(n){return e.typeHasCallOrConstructSignatures(n,H)}function kv(n){if(!e.isGeneratedIdentifier(n)){var t=e.getParseTreeNode(n,e.isIdentifier);if(t)return!(187===t.parent.kind&&t.parent.name===t)&&ey(t)===K}return!1}function Mv(n){var t=sr(n.parent,n);if(!t||e.isShorthandAmbientModuleSymbol(t))return!0;var r=pr(t),a=It(t=dr(t));return void 0===a.exportsSomeValue&&(a.exportsSomeValue=r?!!(67220415&t.flags):e.forEachEntry(vr(t),function(e){return(e=$t(e))&&!!(67220415&e.flags)})),a.exportsSomeValue}function Ov(n,t){var r=e.getParseTreeNode(n,e.isIdentifier);if(r){var a=ey(r,function(n){return e.isModuleOrEnumDeclaration(n.parent)&&n===n.parent.name}(r));if(a){if(1048576&a.flags){var i=br(a.exportSymbol);if(!t&&944&i.flags&&!(3&i.flags))return;a=i}var o=Tr(a);if(o){if(512&o.flags&&277===o.valueDeclaration.kind){var s=o.valueDeclaration;return s!==e.getSourceFileOfNode(r)?void 0:s}return e.findAncestor(r.parent,function(n){return e.isModuleOrEnumDeclaration(n)&&Er(n)===o})}}}}function Iv(n){var t=e.getParseTreeNode(n,e.isIdentifier);if(t){var r=ey(t);if(Zt(r,67220415))return Wt(r)}}function Rv(n){if(418&n.flags){var t=It(n);if(void 0===t.isDeclarationWithCollidingName){var r=e.getEnclosingBlockScopeContainer(n.valueDeclaration);if(e.isStatementWithLocals(r)){var a=Rt(n.valueDeclaration);if(Ft(r.parent,n.escapedName,67220415,void 0,void 0,!1))t.isDeclarationWithCollidingName=!0;else if(131072&a.flags){var i=262144&a.flags,o=e.isIterationStatement(r,!1),s=216===r.kind&&e.isIterationStatement(r.parent,!1);t.isDeclarationWithCollidingName=!(e.isBlockScopedContainerTopLevel(r)||i&&(o||s))}else t.isDeclarationWithCollidingName=!1}}return t.isDeclarationWithCollidingName}return!1}function Nv(n){if(!e.isGeneratedIdentifier(n)){var t=e.getParseTreeNode(n,e.isIdentifier);if(t){var r=ey(t);if(r&&Rv(r))return r.valueDeclaration}}}function wv(n){var t=e.getParseTreeNode(n,e.isDeclaration);if(t){var r=Er(t);if(r)return Rv(r)}return!1}function Pv(n){switch(n.kind){case 246:case 248:case 249:case 251:case 255:return Gv(Er(n)||Q);case 253:var t=n.exportClause;return!!t&&e.some(t.elements,Pv);case 252:return!n.expression||71!==n.expression.kind||Gv(Er(n)||Q)}return!1}function Fv(n){var t=e.getParseTreeNode(n,e.isImportEqualsDeclaration);return!(void 0===t||277!==t.parent.kind||!e.isInternalModuleImportEqualsDeclaration(t))&&Gv(Er(t))&&t.moduleReference&&!e.nodeIsMissing(t.moduleReference)}function Gv(e){var n=er(e);return n===Q||!!(67220415&n.flags)&&(A.preserveConstEnums||!Vv(n))}function Vv(e){return og(e)||!!e.constEnumOnlyModule}function Kv(n){if(e.nodeIsPresent(n.body)){if(e.isGetAccessor(n)||e.isSetAccessor(n))return!1;var t=Go(Er(n));return t.length>1||1===t.length&&t[0].declaration!==n}return!1}function Bv(n){return!(!M||ko(n)||e.isJSDocParameterTag(n)||!n.initializer||e.hasModifier(n,92))}function Hv(n){return M&&ko(n)&&!n.initializer&&e.hasModifier(n,92)}function Uv(n){var t=e.getParseTreeNode(n,e.isFunctionDeclaration);if(!t)return!1;var r=Er(t);return!!(r&&16&r.flags)&&!!e.forEachEntry(_r(r),function(n){return 67220415&n.flags&&e.isPropertyAccessExpression(n.valueDeclaration)})}function jv(n){var t=e.getParseTreeNode(n,e.isFunctionDeclaration);if(!t)return e.emptyArray;var r=Er(t);return r&&$i(Ga(r))||e.emptyArray}function Wv(e){return Rt(e).flags||0}function qv(e){return Y_(e.parent),Rt(e).enumMemberValue}function zv(e){switch(e.kind){case 276:case 187:case 188:return!0}return!1}function Jv(n){if(276===n.kind)return qv(n);var t=Rt(n).resolvedSymbol;if(t&&8&t.flags){var r=t.valueDeclaration;if(e.isEnumConst(r.parent))return qv(r)}}function Xv(n,t){var r=e.getParseTreeNode(n,e.isEntityName);if(!r)return e.TypeReferenceSerializationKind.Unknown;if(t&&!(t=e.getParseTreeNode(t)))return e.TypeReferenceSerializationKind.Unknown;var a=ir(r,67220415,!0,!1,t),i=ir(r,67897832,!0,!1,t);if(a&&a===i){var o=Is(!1);if(o&&a===o)return e.TypeReferenceSerializationKind.Promise;var s=Ga(a);if(s&&za(s))return e.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue}if(!i)return e.TypeReferenceSerializationKind.Unknown;var l=si(i);return l===te?e.TypeReferenceSerializationKind.Unknown:3&l.flags?e.TypeReferenceSerializationKind.ObjectType:rg(l,61440)?e.TypeReferenceSerializationKind.VoidNullableOrNeverType:rg(l,272)?e.TypeReferenceSerializationKind.BooleanType:rg(l,168)?e.TypeReferenceSerializationKind.NumberLikeType:rg(l,68)?e.TypeReferenceSerializationKind.StringLikeType:pu(l)?e.TypeReferenceSerializationKind.ArrayLikeType:rg(l,3072)?e.TypeReferenceSerializationKind.ESSymbolType:function(e){return!!(131072&e.flags)&&ho(e,0).length>0}(l)?e.TypeReferenceSerializationKind.TypeWithCallSignature:eu(l)?e.TypeReferenceSerializationKind.ArrayLikeType:e.TypeReferenceSerializationKind.ObjectType}function Yv(n,t,r,a,i){var o=e.getParseTreeNode(n,e.isVariableLikeOrAccessor);if(!o)return e.createToken(119);var s=Er(o),l=!s||133120&s.flags?te:uu(Ga(s));return 2048&l.flags&&l.symbol===s&&(r|=1048576),i&&(l=Eu(l)),F.typeToTypeNode(l,t,1024|r,a)}function Qv(n,t,r,a){var i=e.getParseTreeNode(n,e.isFunctionLike);if(!i)return e.createToken(119);var o=wo(i);return F.typeToTypeNode(Ho(o),t,1024|r,a)}function Zv(n,t,r,a){var i=e.getParseTreeNode(n,e.isExpression);if(!i)return e.createToken(119);var o=Mu(Av(i));return F.typeToTypeNode(o,t,1024|r,a)}function $v(n){return Sn.has(e.escapeLeadingUnderscores(n))}function ey(n,t){var r=Rt(n).resolvedSymbol;if(r)return r;var a=n;if(t){var i=n.parent;e.isDeclaration(i)&&n===i.name&&(a=ca(i))}return Ft(a,n.escapedText,70366143,void 0,void 0,!0)}function ny(n){if(!e.isGeneratedIdentifier(n)){var t=e.getParseTreeNode(n,e.isIdentifier);if(t){var r=ey(t);if(r)return xr(r).valueDeclaration}}}function ty(n){if(e.isDeclarationReadonly(n)||e.isVariableDeclaration(n)&&e.isVarConst(n)){var t=Ga(Er(n));return!!(448&t.flags&&33554432&t.flags)}return!1}function ry(n){return function(n,t){return(512&n.flags?F.symbolToExpression(n.symbol,67220415,t):n===me?e.createTrue():n===ue&&e.createFalse())||e.createLiteral(n.value)}(Ga(Er(n)),n)}function ay(n){var t=242===n.kind?e.tryCast(n.name,e.isStringLiteral):e.getExternalModuleName(n),r=lr(t,t,void 0);if(r)return e.getDeclarationOfKind(r,277)}function iy(n,t){if((p&t)!==t&&A.importHelpers){var r=e.getSourceFileOfNode(n);if(e.isEffectiveExternalModule(r,A)&&!(4194304&n.flags)){var a=(l=r,c=n,f||(f=cr(l,e.externalHelpersModuleNameText,e.Diagnostics.This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found,c)||Q),f);if(a!==Q)for(var i=t&~p,o=1;o<=65536;o<<=1)if(i&o){var s=oy(o);wt(a.exports,e.escapeLeadingUnderscores(s),67220415)||bt(n,e.Diagnostics.This_syntax_requires_an_imported_helper_named_1_but_module_0_has_no_exported_member_1,e.externalHelpersModuleNameText,s)}p|=t}}var l,c}function oy(n){switch(n){case 1:return\"__extends\";case 2:return\"__assign\";case 4:return\"__rest\";case 8:return\"__decorate\";case 16:return\"__metadata\";case 32:return\"__param\";case 64:return\"__awaiter\";case 128:return\"__generator\";case 256:return\"__values\";case 512:return\"__read\";case 1024:return\"__spread\";case 2048:return\"__await\";case 4096:return\"__asyncGenerator\";case 8192:return\"__asyncDelegator\";case 16384:return\"__asyncValues\";case 32768:return\"__exportStar\";case 65536:return\"__makeTemplateObject\";default:return e.Debug.fail(\"Unrecognized helper\")}}function sy(n){return function(n){if(!n.decorators)return!1;if(!e.nodeCanBeDecorated(n,n.parent,n.parent.parent))return 154!==n.kind||e.nodeIsPresent(n.body)?ky(n,e.Diagnostics.Decorators_are_not_valid_here):ky(n,e.Diagnostics.A_decorator_can_only_decorate_a_method_implementation_not_an_overload);if(156===n.kind||157===n.kind){var t=e.getAllAccessorDeclarations(n.parent.members,n);if(t.firstAccessor.decorators&&n===t.secondAccessor)return ky(n,e.Diagnostics.Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name)}return!1}(n)||function(n){var t,r,a,i,o=function(n){return!!n.modifiers&&(function(n){switch(n.kind){case 156:case 157:case 155:case 152:case 151:case 154:case 153:case 160:case 242:case 247:case 246:case 253:case 252:case 194:case 195:case 149:return!1;default:if(243===n.parent.kind||277===n.parent.kind)return!1;switch(n.kind){case 237:return ly(n,120);case 238:return ly(n,117);case 239:case 217:case 240:return!0;case 241:return ly(n,76);default:return e.Debug.fail(),!1}}}(n)?ky(n,e.Diagnostics.Modifiers_cannot_appear_here):void 0)}(n);if(void 0!==o)return o;for(var s=0,l=0,c=n.modifiers;l<c.length;l++){var u=c[l];if(132!==u.kind){if(151===n.kind||153===n.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_type_member,e.tokenToString(u.kind));if(160===n.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_an_index_signature,e.tokenToString(u.kind))}switch(u.kind){case 76:if(241!==n.kind)return Oy(n,e.Diagnostics.A_class_member_cannot_have_the_0_keyword,e.tokenToString(76));break;case 114:case 113:case 112:var d=$r(e.modifierToFlag(u.kind));if(28&s)return Oy(u,e.Diagnostics.Accessibility_modifier_already_seen);if(32&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,d,\"static\");if(64&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,d,\"readonly\");if(256&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,d,\"async\");if(243===n.parent.kind||277===n.parent.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_module_or_namespace_element,d);if(128&s)return 112===u.kind?Oy(u,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,d,\"abstract\"):Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,d,\"abstract\");s|=e.modifierToFlag(u.kind);break;case 115:if(32&s)return Oy(u,e.Diagnostics._0_modifier_already_seen,\"static\");if(64&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,\"static\",\"readonly\");if(256&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,\"static\",\"async\");if(243===n.parent.kind||277===n.parent.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_module_or_namespace_element,\"static\");if(149===n.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,\"static\");if(128&s)return Oy(u,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,\"static\",\"abstract\");s|=32,t=u;break;case 132:if(64&s)return Oy(u,e.Diagnostics._0_modifier_already_seen,\"readonly\");if(152!==n.kind&&151!==n.kind&&160!==n.kind&&149!==n.kind)return Oy(u,e.Diagnostics.readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature);s|=64,i=u;break;case 84:if(1&s)return Oy(u,e.Diagnostics._0_modifier_already_seen,\"export\");if(2&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,\"export\",\"declare\");if(128&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,\"export\",\"abstract\");if(256&s)return Oy(u,e.Diagnostics._0_modifier_must_precede_1_modifier,\"export\",\"async\");if(238===n.parent.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_class_element,\"export\");if(149===n.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,\"export\");s|=1;break;case 79:var m=277===n.parent.kind?n.parent:n.parent.parent;if(242===m.kind&&!e.isAmbientModule(m))return Oy(u,e.Diagnostics.A_default_export_can_only_be_used_in_an_ECMAScript_style_module);s|=512;break;case 124:if(2&s)return Oy(u,e.Diagnostics._0_modifier_already_seen,\"declare\");if(256&s)return Oy(u,e.Diagnostics._0_modifier_cannot_be_used_in_an_ambient_context,\"async\");if(238===n.parent.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_class_element,\"declare\");if(149===n.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,\"declare\");if(4194304&n.parent.flags&&243===n.parent.kind)return Oy(u,e.Diagnostics.A_declare_modifier_cannot_be_used_in_an_already_ambient_context);s|=2,r=u;break;case 117:if(128&s)return Oy(u,e.Diagnostics._0_modifier_already_seen,\"abstract\");if(238!==n.kind){if(154!==n.kind&&152!==n.kind&&156!==n.kind&&157!==n.kind)return Oy(u,e.Diagnostics.abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration);if(238!==n.parent.kind||!e.hasModifier(n.parent,128))return Oy(u,e.Diagnostics.Abstract_methods_can_only_appear_within_an_abstract_class);if(32&s)return Oy(u,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,\"static\",\"abstract\");if(8&s)return Oy(u,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,\"private\",\"abstract\")}s|=128;break;case 120:if(256&s)return Oy(u,e.Diagnostics._0_modifier_already_seen,\"async\");if(2&s||4194304&n.parent.flags)return Oy(u,e.Diagnostics._0_modifier_cannot_be_used_in_an_ambient_context,\"async\");if(149===n.kind)return Oy(u,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,\"async\");s|=256,a=u}}return 155===n.kind?32&s?Oy(t,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,\"static\"):128&s?Oy(t,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,\"abstract\"):256&s?Oy(a,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,\"async\"):!!(64&s)&&Oy(i,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,\"readonly\"):(247===n.kind||246===n.kind)&&2&s?Oy(r,e.Diagnostics.A_0_modifier_cannot_be_used_with_an_import_declaration,\"declare\"):149===n.kind&&92&s&&e.isBindingPattern(n.name)?Oy(n,e.Diagnostics.A_parameter_property_may_not_be_declared_using_a_binding_pattern):149===n.kind&&92&s&&n.dotDotDotToken?Oy(n,e.Diagnostics.A_parameter_property_cannot_be_declared_using_a_rest_parameter):!!(256&s)&&function(n,t){switch(n.kind){case 154:case 237:case 194:case 195:return!1}return Oy(t,e.Diagnostics._0_modifier_cannot_be_used_here,\"async\")}(n,a)}(n)}function ly(e,n){return e.modifiers.length>1||e.modifiers[0].kind!==n}function cy(n,t){return void 0===t&&(t=e.Diagnostics.Trailing_comma_not_allowed),!(!n||!n.hasTrailingComma)&&My(n[0],n.end-\",\".length,\",\".length,t)}function uy(n,t){if(n&&0===n.length){var r=n.pos-\"<\".length;return My(t,r,e.skipTrivia(t.text,n.end)+\">\".length-r,e.Diagnostics.Type_parameter_list_cannot_be_empty)}return!1}function dy(n){if(C>=3){var t=n.body&&e.isBlock(n.body)&&e.findUseStrictPrologue(n.body.statements);if(t){var r=(i=n.parameters,e.filter(i,function(n){return!!n.initializer||e.isBindingPattern(n.name)||e.isRestParameter(n)}));if(e.length(r)){e.forEach(r,function(n){ht(bt(n,e.Diagnostics.This_parameter_is_not_allowed_with_use_strict_directive),e.createDiagnosticForNode(t,e.Diagnostics.use_strict_directive_used_here))});var a=r.map(function(n,t){return 0===t?e.createDiagnosticForNode(n,e.Diagnostics.Non_simple_parameter_declared_here):e.createDiagnosticForNode(n,e.Diagnostics.and_here)});return ht.apply(void 0,[bt(t,e.Diagnostics.use_strict_directive_cannot_be_used_with_non_simple_parameter_list)].concat(a)),!0}}}var i;return!1}function my(n){var t=e.getSourceFileOfNode(n);return sy(n)||uy(n.typeParameters,t)||function(n){for(var t=!1,r=n.length,a=0;a<r;a++){var i=n[a];if(i.dotDotDotToken){if(a!==r-1)return Oy(i.dotDotDotToken,e.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list);if(4194304&i.flags||cy(n,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma),e.isBindingPattern(i.name))return Oy(i.name,e.Diagnostics.A_rest_element_cannot_contain_a_binding_pattern);if(i.questionToken)return Oy(i.questionToken,e.Diagnostics.A_rest_parameter_cannot_be_optional);if(i.initializer)return Oy(i.name,e.Diagnostics.A_rest_parameter_cannot_have_an_initializer)}else if(i.questionToken){if(t=!0,i.initializer)return Oy(i.name,e.Diagnostics.Parameter_cannot_have_question_mark_and_initializer)}else if(t&&!i.initializer)return Oy(i.name,e.Diagnostics.A_required_parameter_cannot_follow_an_optional_parameter)}}(n.parameters)||function(n,t){if(!e.isArrowFunction(n))return!1;var r=n.equalsGreaterThanToken,a=e.getLineAndCharacterOfPosition(t,r.pos).line,i=e.getLineAndCharacterOfPosition(t,r.end).line;return a!==i&&Oy(r,e.Diagnostics.Line_terminator_not_permitted_before_arrow)}(n,t)||e.isFunctionLikeDeclaration(n)&&dy(n)}function py(n,t){return cy(t)||function(n,t){if(t&&0===t.length){var r=e.getSourceFileOfNode(n),a=t.pos-\"<\".length;return My(r,a,e.skipTrivia(r.text,t.end)+\">\".length-a,e.Diagnostics.Type_argument_list_cannot_be_empty)}return!1}(n,t)}function fy(n){return function(n){if(n)for(var t=0,r=n;t<r.length;t++){var a=r[t];if(208===a.kind)return My(a,a.pos,0,e.Diagnostics.Argument_expression_expected)}return!1}(n)}function gy(n){var t=n.types;if(cy(t))return!0;if(t&&0===t.length){var r=e.tokenToString(n.token);return My(n,t.pos,0,e.Diagnostics._0_list_cannot_be_empty,r)}return e.some(t,_y)}function _y(e){return py(e,e.typeArguments)}function vy(n){if(147!==n.kind)return!1;var t=n;return 202===t.expression.kind&&26===t.expression.operatorToken.kind&&Oy(t.expression,e.Diagnostics.A_comma_expression_is_not_allowed_in_a_computed_property_name)}function yy(n){if(n.asteriskToken){if(e.Debug.assert(237===n.kind||194===n.kind||154===n.kind),4194304&n.flags)return Oy(n.asteriskToken,e.Diagnostics.Generators_are_not_allowed_in_an_ambient_context);if(!n.body)return Oy(n.asteriskToken,e.Diagnostics.An_overload_signature_cannot_be_declared_as_a_generator)}}function hy(e,n){return!!e&&Oy(e,n)}function by(e,n){return!!e&&Oy(e,n)}function Ey(n){if(Iy(n))return!0;if(225===n.kind&&n.awaitModifier&&0==(16384&n.flags))return Oy(n.awaitModifier,e.Diagnostics.A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator);if(236===n.initializer.kind){var t=n.initializer;if(!Cy(t)){var r=t.declarations;if(!r.length)return!1;if(r.length>1){var a=224===n.kind?e.Diagnostics.Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement:e.Diagnostics.Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement;return ky(t.declarations[1],a)}var i=r[0];if(i.initializer){var a=224===n.kind?e.Diagnostics.The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer:e.Diagnostics.The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer;return Oy(i.name,a)}if(i.type)return Oy(i,a=224===n.kind?e.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation:e.Diagnostics.The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation)}}return!1}function Ty(n){if(n.parameters.length===(156===n.kind?1:2))return e.getThisParameter(n)}function Sy(n,t){if(function(n){return e.isDynamicName(n)&&!vi(n)}(n))return Oy(n,t)}function Ly(n){if(my(n))return!0;if(154===n.kind){if(186===n.parent.kind){if(n.modifiers&&(1!==n.modifiers.length||120!==e.first(n.modifiers).kind))return ky(n,e.Diagnostics.Modifiers_cannot_appear_here);if(hy(n.questionToken,e.Diagnostics.An_object_member_cannot_be_declared_optional))return!0;if(by(n.exclamationToken,e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context))return!0;if(void 0===n.body)return My(n,n.end-1,\";\".length,e.Diagnostics._0_expected,\"{\")}if(yy(n))return!0}if(e.isClassLike(n.parent)){if(4194304&n.flags)return Sy(n.name,e.Diagnostics.A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);if(154===n.kind&&!n.body)return Sy(n.name,e.Diagnostics.A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type)}else{if(239===n.parent.kind)return Sy(n.name,e.Diagnostics.A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);if(166===n.parent.kind)return Sy(n.name,e.Diagnostics.A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type)}}function xy(e){return 9===e.kind||8===e.kind||200===e.kind&&38===e.operator&&8===e.operand.kind}function Ay(n){if(n.initializer){var t=!(xy(n.initializer)||function(n){if((e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n)&&xy(n.argumentExpression))&&e.isEntityNameExpression(n.expression))return!!(512&gg(n).flags)}(n.initializer)||101===n.initializer.kind||86===n.initializer.kind),r=e.isDeclarationReadonly(n)||e.isVariableDeclaration(n)&&e.isVarConst(n);if(!r||n.type)return Oy(n.initializer,e.Diagnostics.Initializers_are_not_allowed_in_ambient_contexts);if(t)return Oy(n.initializer,e.Diagnostics.A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_reference);if(!r||t)return Oy(n.initializer,e.Diagnostics.Initializers_are_not_allowed_in_ambient_contexts)}}function Cy(n){var t=n.declarations;return!!cy(n.declarations)||!n.declarations.length&&My(n,t.pos,t.end-t.pos,e.Diagnostics.Variable_declaration_list_cannot_be_empty)}function Dy(e){return e.parseDiagnostics.length>0}function ky(n,t,r,a,i){var o=e.getSourceFileOfNode(n);if(!Dy(o)){var s=e.getSpanOfTokenAtPosition(o,n.pos);return Xn.add(e.createFileDiagnostic(o,s.start,s.length,t,r,a,i)),!0}return!1}function My(n,t,r,a,i,o,s){var l=e.getSourceFileOfNode(n);return!Dy(l)&&(Xn.add(e.createFileDiagnostic(l,t,r,a,i,o,s)),!0)}function Oy(n,t,r,a,i){return!Dy(e.getSourceFileOfNode(n))&&(Xn.add(e.createDiagnosticForNode(n,t,r,a,i)),!0)}function Iy(n){if(4194304&n.flags){if(e.isAccessor(n.parent))return Rt(n).hasReportedStatementInAmbientContext=!0;if(!Rt(n).hasReportedStatementInAmbientContext&&e.isFunctionLike(n.parent))return Rt(n).hasReportedStatementInAmbientContext=ky(n,e.Diagnostics.An_implementation_cannot_be_declared_in_ambient_contexts);if(216===n.parent.kind||243===n.parent.kind||277===n.parent.kind){var t=Rt(n.parent);if(!t.hasReportedStatementInAmbientContext)return t.hasReportedStatementInAmbientContext=ky(n,e.Diagnostics.Statements_are_not_allowed_in_ambient_contexts)}}return!1}function Ry(n){if(32&n.numericLiteralFlags){var t=void 0;if(C>=1?t=e.Diagnostics.Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0:e.isChildOfNodeWithKind(n,180)?t=e.Diagnostics.Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0:e.isChildOfNodeWithKind(n,276)&&(t=e.Diagnostics.Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0),t){var r=e.isPrefixUnaryExpression(n.parent)&&38===n.parent.operator,a=(r?\"-\":\"\")+\"0o\"+n.text;return Oy(r?n.parent:n,t,a)}}return!1}},function(e){e.JSX=\"JSX\",e.IntrinsicElements=\"IntrinsicElements\",e.ElementClass=\"ElementClass\",e.ElementAttributesPropertyNameContainer=\"ElementAttributesProperty\",e.ElementChildrenAttributeNameContainer=\"ElementChildrenAttribute\",e.Element=\"Element\",e.IntrinsicAttributes=\"IntrinsicAttributes\",e.IntrinsicClassAttributes=\"IntrinsicClassAttributes\",e.LibraryManagedAttributes=\"LibraryManagedAttributes\"}(n||(n={}))}(Tn||(Tn={})),function(e){function n(n){var t=e.createNode(n,-1,-1);return t.flags|=8,t}function t(n,t){return n!==t&&(Un(n,t),Fn(n,t),e.aggregateTransformFlags(n)),n}function r(n,t){if(n&&n!==e.emptyArray){if(e.isNodeArray(n))return n}else n=[];var r=n;return r.pos=-1,r.end=-1,r.hasTrailingComma=t,r}function a(e){if(void 0===e)return e;var t=n(e.kind);for(var r in t.flags|=e.flags,Un(t,e),e)!t.hasOwnProperty(r)&&e.hasOwnProperty(r)&&(t[r]=e[r]);return t}function i(n,t){if(\"number\"==typeof n)return o(n+\"\");if(\"boolean\"==typeof n)return n?p():f();if(e.isString(n)){var r=s(n);return t&&(r.singleQuote=!0),r}return a=n,(i=s(e.getTextOfIdentifierOrLiteral(a))).textSourceNode=a,i;var a,i}function o(e){var t=n(8);return t.text=e,t.numericLiteralFlags=0,t}function s(e){var t=n(9);return t.text=e,t}function l(t,a){var i=n(71);return i.escapedText=e.escapeLeadingUnderscores(t),i.originalKeywordKind=t?e.stringToToken(t):0,i.autoGenerateFlags=0,i.autoGenerateId=0,a&&(i.typeArguments=r(a)),i}e.updateNode=t,e.createNodeArray=r,e.getSynthesizedClone=a,e.createLiteral=i,e.createNumericLiteral=o,e.createStringLiteral=s,e.createRegularExpressionLiteral=function(e){var t=n(12);return t.text=e,t},e.createIdentifier=l,e.updateIdentifier=function(n,r){return n.typeArguments!==r?t(l(e.idText(n),r),n):n};var c,u=0;function d(e){var n=l(e);return n.autoGenerateFlags=19,n.autoGenerateId=u,u++,n}function m(e){return n(e)}function p(){return n(101)}function f(){return n(86)}function g(e){return m(e)}function _(e,t){var r=n(146);return r.left=e,r.right=Nn(t),r}function v(t){var r=n(147);return r.expression=function(n){return e.isCommaSequence(n)?oe(n):n}(t),r}function y(e,t,r){var a=n(148);return a.name=Nn(e),a.constraint=t,a.default=r,a}function h(t,r,a,i,o,s,l){var c=n(149);return c.decorators=wn(t),c.modifiers=wn(r),c.dotDotDotToken=a,c.name=Nn(i),c.questionToken=o,c.type=s,c.initializer=l?e.parenthesizeExpressionForList(l):void 0,c}function b(t){var r=n(150);return r.expression=e.parenthesizeForAccess(t),r}function E(e,t,r,a,i){var o=n(151);return o.modifiers=wn(e),o.name=Nn(t),o.questionToken=r,o.type=a,o.initializer=i,o}function T(e,t,r,a,i,o){var s=n(152);return s.decorators=wn(e),s.modifiers=wn(t),s.name=Nn(r),s.questionToken=void 0!==a&&55===a.kind?a:void 0,s.exclamationToken=void 0!==a&&51===a.kind?a:void 0,s.type=i,s.initializer=o,s}function S(e,n,t,r,a){var i=k(153,e,n,t);return i.name=Nn(r),i.questionToken=a,i}function L(e,t,a,i,o,s,l,c,u){var d=n(154);return d.decorators=wn(e),d.modifiers=wn(t),d.asteriskToken=a,d.name=Nn(i),d.questionToken=o,d.typeParameters=wn(s),d.parameters=r(l),d.type=c,d.body=u,d}function x(e,t,a,i){var o=n(155);return o.decorators=wn(e),o.modifiers=wn(t),o.typeParameters=void 0,o.parameters=r(a),o.type=void 0,o.body=i,o}function A(e,t,a,i,o,s){var l=n(156);return l.decorators=wn(e),l.modifiers=wn(t),l.name=Nn(a),l.typeParameters=void 0,l.parameters=r(i),l.type=o,l.body=s,l}function C(e,t,a,i,o){var s=n(157);return s.decorators=wn(e),s.modifiers=wn(t),s.name=Nn(a),s.typeParameters=void 0,s.parameters=r(i),s.body=o,s}function D(e,t,a,i){var o=n(160);return o.decorators=wn(e),o.modifiers=wn(t),o.parameters=r(a),o.type=i,o}function k(e,t,r,a,i){var o=n(e);return o.typeParameters=wn(t),o.parameters=wn(r),o.type=a,o.typeArguments=wn(i),o}function M(e,n,r,a){return e.typeParameters!==n||e.parameters!==r||e.type!==a?t(k(e.kind,n,r,a),e):e}function O(e,t){var r=n(161);return r.parameterName=Nn(e),r.type=t,r}function I(t,r){var a=n(162);return a.typeName=Nn(t),a.typeArguments=r&&e.parenthesizeTypeParameters(r),a}function R(e){var t=n(165);return t.exprName=e,t}function N(e){var t=n(166);return t.members=r(e),t}function w(t){var r=n(167);return r.elementType=e.parenthesizeArrayTypeMember(t),r}function P(e){var t=n(168);return t.elementTypes=r(e),t}function F(t){var r=n(169);return r.type=e.parenthesizeArrayTypeMember(t),r}function G(e){var t=n(170);return t.type=e,t}function V(t,r){var a=n(t);return a.types=e.parenthesizeElementTypeMembers(r),a}function K(e,n){return e.types!==n?t(V(e.kind,n),e):e}function B(t,r,a,i){var o=n(173);return o.checkType=e.parenthesizeConditionalTypeMember(t),o.extendsType=e.parenthesizeConditionalTypeMember(r),o.trueType=a,o.falseType=i,o}function H(e){var t=n(174);return t.typeParameter=e,t}function U(e,t,r,a){var i=n(181);return i.argument=e,i.qualifier=t,i.typeArguments=wn(r),i.isTypeOf=a,i}function j(e){var t=n(175);return t.type=e,t}function W(t,r){var a=n(177);return a.operator=\"number\"==typeof t?t:128,a.type=e.parenthesizeElementTypeMember(\"number\"==typeof t?r:t),a}function q(t,r){var a=n(178);return a.objectType=e.parenthesizeElementTypeMember(t),a.indexType=r,a}function z(e,t,r,a){var i=n(179);return i.readonlyToken=e,i.typeParameter=t,i.questionToken=r,i.type=a,i}function J(e){var t=n(180);return t.literal=e,t}function X(e){var t=n(182);return t.elements=r(e),t}function Y(e){var t=n(183);return t.elements=r(e),t}function Q(e,t,r,a){var i=n(184);return i.dotDotDotToken=e,i.propertyName=Nn(t),i.name=Nn(r),i.initializer=a,i}function Z(t,a){var i=n(185);return i.elements=e.parenthesizeListElements(r(t)),a&&(i.multiLine=!0),i}function $(e,t){var a=n(186);return a.properties=r(e),t&&(a.multiLine=!0),a}function ee(t,r){var a=n(187);return a.expression=e.parenthesizeForAccess(t),a.name=Nn(r),Gn(a,131072),a}function ne(t,r){var a,o=n(188);return o.expression=e.parenthesizeForAccess(t),o.argumentExpression=(a=r,e.isString(a)||\"number\"==typeof a?i(a):a),o}function te(t,a,i){var o=n(189);return o.expression=e.parenthesizeForAccess(t),o.typeArguments=wn(a),o.arguments=e.parenthesizeListElements(r(i)),o}function re(t,a,i){var o=n(190);return o.expression=e.parenthesizeForNew(t),o.typeArguments=wn(a),o.arguments=i?e.parenthesizeListElements(r(i)):void 0,o}function ae(t,r,a){var i=n(191);return i.tag=e.parenthesizeForAccess(t),a?(i.typeArguments=wn(r),i.template=a):(i.typeArguments=void 0,i.template=r),i}function ie(t,r){var a=n(192);return a.type=t,a.expression=e.parenthesizePrefixOperand(r),a}function oe(e){var t=n(193);return t.expression=e,t}function se(e,t,a,i,o,s,l){var c=n(194);return c.modifiers=wn(e),c.asteriskToken=t,c.name=Nn(a),c.typeParameters=wn(i),c.parameters=r(o),c.type=s,c.body=l,c}function le(t,a,i,o,s,l){var c=n(195);return c.modifiers=wn(t),c.typeParameters=wn(a),c.parameters=r(i),c.type=o,c.equalsGreaterThanToken=s||m(36),c.body=e.parenthesizeConciseBody(l),c}function ce(t){var r=n(196);return r.expression=e.parenthesizePrefixOperand(t),r}function ue(t){var r=n(197);return r.expression=e.parenthesizePrefixOperand(t),r}function de(t){var r=n(198);return r.expression=e.parenthesizePrefixOperand(t),r}function me(t){var r=n(199);return r.expression=e.parenthesizePrefixOperand(t),r}function pe(t,r){var a=n(200);return a.operator=t,a.operand=e.parenthesizePrefixOperand(r),a}function fe(t,r){var a=n(201);return a.operand=e.parenthesizePostfixOperand(t),a.operator=r,a}function ge(t,r,a){var i,o=n(202),s=\"number\"==typeof(i=r)?m(i):i,l=s.kind;return o.left=e.parenthesizeBinaryOperand(l,t,!0,void 0),o.operatorToken=s,o.right=e.parenthesizeBinaryOperand(l,a,!1,o.left),o}function _e(t,r,a,i,o){var s=n(203);return s.condition=e.parenthesizeForConditionalHead(t),s.questionToken=o?r:m(55),s.whenTrue=e.parenthesizeSubexpressionOfConditionalExpression(o?a:r),s.colonToken=o?i:m(56),s.whenFalse=e.parenthesizeSubexpressionOfConditionalExpression(o||a),s}function ve(e,t){var a=n(204);return a.head=e,a.templateSpans=r(t),a}function ye(e,t){var r=n(205);return r.asteriskToken=e&&39===e.kind?e:void 0,r.expression=e&&39!==e.kind?e:t,r}function he(t){var r=n(206);return r.expression=e.parenthesizeExpressionForList(t),r}function be(e,t,a,i,o){var s=n(207);return s.decorators=void 0,s.modifiers=wn(e),s.name=Nn(t),s.typeParameters=wn(a),s.heritageClauses=wn(i),s.members=r(o),s}function Ee(t,r){var a=n(209);return a.expression=e.parenthesizeForAccess(r),a.typeArguments=wn(t),a}function Te(e,t){var r=n(210);return r.expression=e,r.type=t,r}function Se(t){var r=n(211);return r.expression=e.parenthesizeForAccess(t),r}function Le(e,t){var r=n(212);return r.keywordToken=e,r.name=t,r}function xe(e,t){var r=n(214);return r.expression=e,r.literal=t,r}function Ae(e,t){var a=n(216);return a.statements=r(e),t&&(a.multiLine=t),a}function Ce(t,r){var a=n(217);return a.decorators=void 0,a.modifiers=wn(t),a.declarationList=e.isArray(r)?We(r):r,a}function De(t){var r=n(219);return r.expression=e.parenthesizeExpressionForExpressionStatement(t),r}function ke(e,n){return e.expression!==n?t(De(n),e):e}function Me(e,t,r){var a=n(220);return a.expression=e,a.thenStatement=t,a.elseStatement=r,a}function Oe(e,t){var r=n(221);return r.statement=e,r.expression=t,r}function Ie(e,t){var r=n(222);return r.expression=e,r.statement=t,r}function Re(e,t,r,a){var i=n(223);return i.initializer=e,i.condition=t,i.incrementor=r,i.statement=a,i}function Ne(e,t,r){var a=n(224);return a.initializer=e,a.expression=t,a.statement=r,a}function we(e,t,r,a){var i=n(225);return i.awaitModifier=e,i.initializer=t,i.expression=r,i.statement=a,i}function Pe(e){var t=n(226);return t.label=Nn(e),t}function Fe(e){var t=n(227);return t.label=Nn(e),t}function Ge(e){var t=n(228);return t.expression=e,t}function Ve(e,t){var r=n(229);return r.expression=e,r.statement=t,r}function Ke(t,r){var a=n(230);return a.expression=e.parenthesizeExpressionForList(t),a.caseBlock=r,a}function Be(e,t){var r=n(231);return r.label=Nn(e),r.statement=t,r}function He(e){var t=n(232);return t.expression=e,t}function Ue(e,t,r){var a=n(233);return a.tryBlock=e,a.catchClause=t,a.finallyBlock=r,a}function je(t,r,a){var i=n(235);return i.name=Nn(t),i.type=r,i.initializer=void 0!==a?e.parenthesizeExpressionForList(a):void 0,i}function We(e,t){void 0===t&&(t=0);var a=n(236);return a.flags|=3&t,a.declarations=r(e),a}function qe(e,t,a,i,o,s,l,c){var u=n(237);return u.decorators=wn(e),u.modifiers=wn(t),u.asteriskToken=a,u.name=Nn(i),u.typeParameters=wn(o),u.parameters=r(s),u.type=l,u.body=c,u}function ze(e,t,a,i,o,s){var l=n(238);return l.decorators=wn(e),l.modifiers=wn(t),l.name=Nn(a),l.typeParameters=wn(i),l.heritageClauses=wn(o),l.members=r(s),l}function Je(e,t,a,i,o,s){var l=n(239);return l.decorators=wn(e),l.modifiers=wn(t),l.name=Nn(a),l.typeParameters=wn(i),l.heritageClauses=wn(o),l.members=r(s),l}function Xe(e,t,r,a,i){var o=n(240);return o.decorators=wn(e),o.modifiers=wn(t),o.name=Nn(r),o.typeParameters=wn(a),o.type=i,o}function Ye(e,t,a,i){var o=n(241);return o.decorators=wn(e),o.modifiers=wn(t),o.name=Nn(a),o.members=r(i),o}function Qe(e,t,r,a,i){void 0===i&&(i=0);var o=n(242);return o.flags|=532&i,o.decorators=wn(e),o.modifiers=wn(t),o.name=r,o.body=a,o}function Ze(e){var t=n(243);return t.statements=r(e),t}function $e(e){var t=n(244);return t.clauses=r(e),t}function en(e){var t=n(245);return t.name=Nn(e),t}function nn(e,t,r,a){var i=n(246);return i.decorators=wn(e),i.modifiers=wn(t),i.name=Nn(r),i.moduleReference=a,i}function tn(e,t,r,a){var i=n(247);return i.decorators=wn(e),i.modifiers=wn(t),i.importClause=r,i.moduleSpecifier=a,i}function rn(e,t){var r=n(248);return r.name=e,r.namedBindings=t,r}function an(e){var t=n(249);return t.name=e,t}function on(e){var t=n(250);return t.elements=r(e),t}function sn(e,t){var r=n(251);return r.propertyName=e,r.name=t,r}function ln(t,r,a,i){var o=n(252);return o.decorators=wn(t),o.modifiers=wn(r),o.isExportEquals=a,o.expression=a?e.parenthesizeBinaryOperand(58,i,!1,void 0):e.parenthesizeDefaultExpression(i),o}function cn(e,t,r,a){var i=n(253);return i.decorators=wn(e),i.modifiers=wn(t),i.exportClause=r,i.moduleSpecifier=a,i}function un(e){var t=n(254);return t.elements=r(e),t}function dn(e,t){var r=n(255);return r.propertyName=Nn(e),r.name=Nn(t),r}function mn(e){var t=n(257);return t.expression=e,t}function pn(e,t,a){var i=n(258);return i.openingElement=e,i.children=r(t),i.closingElement=a,i}function fn(e,t,r){var a=n(259);return a.tagName=e,a.typeArguments=wn(t),a.attributes=r,a}function gn(e,t,r){var a=n(260);return a.tagName=e,a.typeArguments=wn(t),a.attributes=r,a}function _n(e){var t=n(261);return t.tagName=e,t}function vn(e,t,a){var i=n(262);return i.openingFragment=e,i.children=r(t),i.closingFragment=a,i}function yn(e,t){var r=n(265);return r.name=e,r.initializer=t,r}function hn(e){var t=n(266);return t.properties=r(e),t}function bn(e){var t=n(267);return t.expression=e,t}function En(e,t){var r=n(268);return r.dotDotDotToken=e,r.expression=t,r}function Tn(t,a){var i=n(269);return i.expression=e.parenthesizeExpressionForList(t),i.statements=r(a),i}function Sn(e){var t=n(270);return t.statements=r(e),t}function Ln(e,t){var a=n(271);return a.token=e,a.types=r(t),a}function xn(t,r){var a=n(272);return a.variableDeclaration=e.isString(t)?je(t):t,a.block=r,a}function An(t,r){var a=n(273);return a.name=Nn(t),a.questionToken=void 0,a.initializer=e.parenthesizeExpressionForList(r),a}function Cn(t,r){var a=n(274);return a.name=Nn(t),a.objectAssignmentInitializer=void 0!==r?e.parenthesizeExpressionForList(r):void 0,a}function Dn(t){var r=n(275);return r.expression=void 0!==t?e.parenthesizeExpressionForList(t):void 0,r}function kn(t,r){var a=n(276);return a.name=Nn(t),a.initializer=r&&e.parenthesizeExpressionForList(r),a}function Mn(e,t){var r=n(306);return r.expression=e,r.original=t,Fn(r,t),r}function On(n){if(e.nodeIsSynthesized(n)&&!e.isParseTreeNode(n)&&!n.original&&!n.emitNode&&!n.id){if(307===n.kind)return n.elements;if(e.isBinaryExpression(n)&&26===n.operatorToken.kind)return[n.left,n.right]}return n}function In(t){var a=n(307);return a.elements=r(e.sameFlatMap(t,On)),a}function Rn(n,t){void 0===t&&(t=e.emptyArray);var r=e.createNode(278);return r.prepends=t,r.sourceFiles=n,r}function Nn(n){return e.isString(n)?l(n):n}function wn(e){return e?r(e):void 0}function Pn(n){if(!n.emitNode){if(e.isParseTreeNode(n)){if(277===n.kind)return n.emitNode={annotatedNodes:[n]};Pn(e.getSourceFileOfNode(n)).annotatedNodes.push(n)}n.emitNode={}}return n.emitNode}function Fn(e,n){return n&&(e.pos=n.pos,e.end=n.end),e}function Gn(e,n){return Pn(e).flags=n,e}function Vn(e){var n=e.emitNode;return n&&n.leadingComments}function Kn(e,n){return Pn(e).leadingComments=n,e}function Bn(e){var n=e.emitNode;return n&&n.trailingComments}function Hn(e,n){return Pn(e).trailingComments=n,e}function Un(n,t){if(n.original=t,t){var r=t.emitNode;r&&(n.emitNode=function(n,t){var r=n.flags,a=n.leadingComments,i=n.trailingComments,o=n.commentRange,s=n.sourceMapRange,l=n.tokenSourceMapRanges,c=n.constantValue,u=n.helpers,d=n.startsOnNewLine;t||(t={});a&&(t.leadingComments=e.addRange(a.slice(),t.leadingComments));i&&(t.trailingComments=e.addRange(i.slice(),t.trailingComments));r&&(t.flags=r);o&&(t.commentRange=o);s&&(t.sourceMapRange=s);l&&(t.tokenSourceMapRanges=function(e,n){n||(n=[]);for(var t in e)n[t]=e[t];return n}(l,t.tokenSourceMapRanges));void 0!==c&&(t.constantValue=c);u&&(t.helpers=e.addRange(t.helpers,u));void 0!==d&&(t.startsOnNewLine=d);return t}(r,n.emitNode))}return n}e.createTempVariable=function(e,n){var t=l(\"\");return t.autoGenerateFlags=1,t.autoGenerateId=u,u++,e&&e(t),n&&(t.autoGenerateFlags|=8),t},e.createLoopVariable=function(){var e=l(\"\");return e.autoGenerateFlags=2,e.autoGenerateId=u,u++,e},e.createUniqueName=function(e){var n=l(e);return n.autoGenerateFlags=3,n.autoGenerateId=u,u++,n},e.createOptimisticUniqueName=d,e.createFileLevelUniqueName=function(e){var n=d(e);return n.autoGenerateFlags|=32,n},e.getGeneratedNameForNode=function(n,t){var r=l(n&&e.isIdentifier(n)?e.idText(n):\"\");return r.autoGenerateFlags=4|t,r.autoGenerateId=u,r.original=n,u++,r},e.createToken=m,e.createSuper=function(){return n(97)},e.createThis=function(){return n(99)},e.createNull=function(){return n(95)},e.createTrue=p,e.createFalse=f,e.createModifier=g,e.createModifiersFromModifierFlags=function(e){var n=[];return 1&e&&n.push(g(84)),2&e&&n.push(g(124)),512&e&&n.push(g(79)),2048&e&&n.push(g(76)),4&e&&n.push(g(114)),8&e&&n.push(g(112)),16&e&&n.push(g(113)),128&e&&n.push(g(117)),32&e&&n.push(g(115)),64&e&&n.push(g(132)),256&e&&n.push(g(120)),n},e.createQualifiedName=_,e.updateQualifiedName=function(e,n,r){return e.left!==n||e.right!==r?t(_(n,r),e):e},e.createComputedPropertyName=v,e.updateComputedPropertyName=function(e,n){return e.expression!==n?t(v(n),e):e},e.createTypeParameterDeclaration=y,e.updateTypeParameterDeclaration=function(e,n,r,a){return e.name!==n||e.constraint!==r||e.default!==a?t(y(n,r,a),e):e},e.createParameter=h,e.updateParameter=function(e,n,r,a,i,o,s,l){return e.decorators!==n||e.modifiers!==r||e.dotDotDotToken!==a||e.name!==i||e.questionToken!==o||e.type!==s||e.initializer!==l?t(h(n,r,a,i,o,s,l),e):e},e.createDecorator=b,e.updateDecorator=function(e,n){return e.expression!==n?t(b(n),e):e},e.createPropertySignature=E,e.updatePropertySignature=function(e,n,r,a,i,o){return e.modifiers!==n||e.name!==r||e.questionToken!==a||e.type!==i||e.initializer!==o?t(E(n,r,a,i,o),e):e},e.createProperty=T,e.updateProperty=function(e,n,r,a,i,o,s){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.questionToken!==(void 0!==i&&55===i.kind?i:void 0)||e.exclamationToken!==(void 0!==i&&51===i.kind?i:void 0)||e.type!==o||e.initializer!==s?t(T(n,r,a,i,o,s),e):e},e.createMethodSignature=S,e.updateMethodSignature=function(e,n,r,a,i,o){return e.typeParameters!==n||e.parameters!==r||e.type!==a||e.name!==i||e.questionToken!==o?t(S(n,r,a,i,o),e):e},e.createMethod=L,e.updateMethod=function(e,n,r,a,i,o,s,l,c,u){return e.decorators!==n||e.modifiers!==r||e.asteriskToken!==a||e.name!==i||e.questionToken!==o||e.typeParameters!==s||e.parameters!==l||e.type!==c||e.body!==u?t(L(n,r,a,i,o,s,l,c,u),e):e},e.createConstructor=x,e.updateConstructor=function(e,n,r,a,i){return e.decorators!==n||e.modifiers!==r||e.parameters!==a||e.body!==i?t(x(n,r,a,i),e):e},e.createGetAccessor=A,e.updateGetAccessor=function(e,n,r,a,i,o,s){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.parameters!==i||e.type!==o||e.body!==s?t(A(n,r,a,i,o,s),e):e},e.createSetAccessor=C,e.updateSetAccessor=function(e,n,r,a,i,o){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.parameters!==i||e.body!==o?t(C(n,r,a,i,o),e):e},e.createCallSignature=function(e,n,t){return k(158,e,n,t)},e.updateCallSignature=function(e,n,t,r){return M(e,n,t,r)},e.createConstructSignature=function(e,n,t){return k(159,e,n,t)},e.updateConstructSignature=function(e,n,t,r){return M(e,n,t,r)},e.createIndexSignature=D,e.updateIndexSignature=function(e,n,r,a,i){return e.parameters!==a||e.type!==i||e.decorators!==n||e.modifiers!==r?t(D(n,r,a,i),e):e},e.createSignatureDeclaration=k,e.createKeywordTypeNode=function(e){return n(e)},e.createTypePredicateNode=O,e.updateTypePredicateNode=function(e,n,r){return e.parameterName!==n||e.type!==r?t(O(n,r),e):e},e.createTypeReferenceNode=I,e.updateTypeReferenceNode=function(e,n,r){return e.typeName!==n||e.typeArguments!==r?t(I(n,r),e):e},e.createFunctionTypeNode=function(e,n,t){return k(163,e,n,t)},e.updateFunctionTypeNode=function(e,n,t,r){return M(e,n,t,r)},e.createConstructorTypeNode=function(e,n,t){return k(164,e,n,t)},e.updateConstructorTypeNode=function(e,n,t,r){return M(e,n,t,r)},e.createTypeQueryNode=R,e.updateTypeQueryNode=function(e,n){return e.exprName!==n?t(R(n),e):e},e.createTypeLiteralNode=N,e.updateTypeLiteralNode=function(e,n){return e.members!==n?t(N(n),e):e},e.createArrayTypeNode=w,e.updateArrayTypeNode=function(e,n){return e.elementType!==n?t(w(n),e):e},e.createTupleTypeNode=P,e.updateTupleTypeNode=function(e,n){return e.elementTypes!==n?t(P(n),e):e},e.createOptionalTypeNode=F,e.updateOptionalTypeNode=function(e,n){return e.type!==n?t(F(n),e):e},e.createRestTypeNode=G,e.updateRestTypeNode=function(e,n){return e.type!==n?t(G(n),e):e},e.createUnionTypeNode=function(e){return V(171,e)},e.updateUnionTypeNode=function(e,n){return K(e,n)},e.createIntersectionTypeNode=function(e){return V(172,e)},e.updateIntersectionTypeNode=function(e,n){return K(e,n)},e.createUnionOrIntersectionTypeNode=V,e.createConditionalTypeNode=B,e.updateConditionalTypeNode=function(e,n,r,a,i){return e.checkType!==n||e.extendsType!==r||e.trueType!==a||e.falseType!==i?t(B(n,r,a,i),e):e},e.createInferTypeNode=H,e.updateInferTypeNode=function(e,n){return e.typeParameter!==n?t(H(n),e):e},e.createImportTypeNode=U,e.updateImportTypeNode=function(e,n,r,a,i){return e.argument!==n||e.qualifier!==r||e.typeArguments!==a||e.isTypeOf!==i?t(U(n,r,a,i),e):e},e.createParenthesizedType=j,e.updateParenthesizedType=function(e,n){return e.type!==n?t(j(n),e):e},e.createThisTypeNode=function(){return n(176)},e.createTypeOperatorNode=W,e.updateTypeOperatorNode=function(e,n){return e.type!==n?t(W(e.operator,n),e):e},e.createIndexedAccessTypeNode=q,e.updateIndexedAccessTypeNode=function(e,n,r){return e.objectType!==n||e.indexType!==r?t(q(n,r),e):e},e.createMappedTypeNode=z,e.updateMappedTypeNode=function(e,n,r,a,i){return e.readonlyToken!==n||e.typeParameter!==r||e.questionToken!==a||e.type!==i?t(z(n,r,a,i),e):e},e.createLiteralTypeNode=J,e.updateLiteralTypeNode=function(e,n){return e.literal!==n?t(J(n),e):e},e.createObjectBindingPattern=X,e.updateObjectBindingPattern=function(e,n){return e.elements!==n?t(X(n),e):e},e.createArrayBindingPattern=Y,e.updateArrayBindingPattern=function(e,n){return e.elements!==n?t(Y(n),e):e},e.createBindingElement=Q,e.updateBindingElement=function(e,n,r,a,i){return e.propertyName!==r||e.dotDotDotToken!==n||e.name!==a||e.initializer!==i?t(Q(n,r,a,i),e):e},e.createArrayLiteral=Z,e.updateArrayLiteral=function(e,n){return e.elements!==n?t(Z(n,e.multiLine),e):e},e.createObjectLiteral=$,e.updateObjectLiteral=function(e,n){return e.properties!==n?t($(n,e.multiLine),e):e},e.createPropertyAccess=ee,e.updatePropertyAccess=function(n,r,a){return n.expression!==r||n.name!==a?t(Gn(ee(r,a),e.getEmitFlags(n)),n):n},e.createElementAccess=ne,e.updateElementAccess=function(e,n,r){return e.expression!==n||e.argumentExpression!==r?t(ne(n,r),e):e},e.createCall=te,e.updateCall=function(e,n,r,a){return e.expression!==n||e.typeArguments!==r||e.arguments!==a?t(te(n,r,a),e):e},e.createNew=re,e.updateNew=function(e,n,r,a){return e.expression!==n||e.typeArguments!==r||e.arguments!==a?t(re(n,r,a),e):e},e.createTaggedTemplate=ae,e.updateTaggedTemplate=function(e,n,r,a){return e.tag!==n||(a?e.typeArguments!==r||e.template!==a:void 0!==e.typeArguments||e.template!==r)?t(ae(n,r,a),e):e},e.createTypeAssertion=ie,e.updateTypeAssertion=function(e,n,r){return e.type!==n||e.expression!==r?t(ie(n,r),e):e},e.createParen=oe,e.updateParen=function(e,n){return e.expression!==n?t(oe(n),e):e},e.createFunctionExpression=se,e.updateFunctionExpression=function(e,n,r,a,i,o,s,l){return e.name!==a||e.modifiers!==n||e.asteriskToken!==r||e.typeParameters!==i||e.parameters!==o||e.type!==s||e.body!==l?t(se(n,r,a,i,o,s,l),e):e},e.createArrowFunction=le,e.updateArrowFunction=function(e,n,r,a,i,o,s){return e.modifiers!==n||e.typeParameters!==r||e.parameters!==a||e.type!==i||e.equalsGreaterThanToken!==o||e.body!==s?t(le(n,r,a,i,o,s),e):e},e.createDelete=ce,e.updateDelete=function(e,n){return e.expression!==n?t(ce(n),e):e},e.createTypeOf=ue,e.updateTypeOf=function(e,n){return e.expression!==n?t(ue(n),e):e},e.createVoid=de,e.updateVoid=function(e,n){return e.expression!==n?t(de(n),e):e},e.createAwait=me,e.updateAwait=function(e,n){return e.expression!==n?t(me(n),e):e},e.createPrefix=pe,e.updatePrefix=function(e,n){return e.operand!==n?t(pe(e.operator,n),e):e},e.createPostfix=fe,e.updatePostfix=function(e,n){return e.operand!==n?t(fe(n,e.operator),e):e},e.createBinary=ge,e.updateBinary=function(e,n,r,a){return e.left!==n||e.right!==r?t(ge(n,a||e.operatorToken,r),e):e},e.createConditional=_e,e.updateConditional=function(e,n,r,a,i,o){return e.condition!==n||e.questionToken!==r||e.whenTrue!==a||e.colonToken!==i||e.whenFalse!==o?t(_e(n,r,a,i,o),e):e},e.createTemplateExpression=ve,e.updateTemplateExpression=function(e,n,r){return e.head!==n||e.templateSpans!==r?t(ve(n,r),e):e},e.createTemplateHead=function(e){var t=n(14);return t.text=e,t},e.createTemplateMiddle=function(e){var t=n(15);return t.text=e,t},e.createTemplateTail=function(e){var t=n(16);return t.text=e,t},e.createNoSubstitutionTemplateLiteral=function(e){var t=n(13);return t.text=e,t},e.createYield=ye,e.updateYield=function(e,n,r){return e.expression!==r||e.asteriskToken!==n?t(ye(n,r),e):e},e.createSpread=he,e.updateSpread=function(e,n){return e.expression!==n?t(he(n),e):e},e.createClassExpression=be,e.updateClassExpression=function(e,n,r,a,i,o){return e.modifiers!==n||e.name!==r||e.typeParameters!==a||e.heritageClauses!==i||e.members!==o?t(be(n,r,a,i,o),e):e},e.createOmittedExpression=function(){return n(208)},e.createExpressionWithTypeArguments=Ee,e.updateExpressionWithTypeArguments=function(e,n,r){return e.typeArguments!==n||e.expression!==r?t(Ee(n,r),e):e},e.createAsExpression=Te,e.updateAsExpression=function(e,n,r){return e.expression!==n||e.type!==r?t(Te(n,r),e):e},e.createNonNullExpression=Se,e.updateNonNullExpression=function(e,n){return e.expression!==n?t(Se(n),e):e},e.createMetaProperty=Le,e.updateMetaProperty=function(e,n){return e.name!==n?t(Le(e.keywordToken,n),e):e},e.createTemplateSpan=xe,e.updateTemplateSpan=function(e,n,r){return e.expression!==n||e.literal!==r?t(xe(n,r),e):e},e.createSemicolonClassElement=function(){return n(215)},e.createBlock=Ae,e.updateBlock=function(e,n){return e.statements!==n?t(Ae(n,e.multiLine),e):e},e.createVariableStatement=Ce,e.updateVariableStatement=function(e,n,r){return e.modifiers!==n||e.declarationList!==r?t(Ce(n,r),e):e},e.createEmptyStatement=function(){return n(218)},e.createExpressionStatement=De,e.updateExpressionStatement=ke,e.createStatement=De,e.updateStatement=ke,e.createIf=Me,e.updateIf=function(e,n,r,a){return e.expression!==n||e.thenStatement!==r||e.elseStatement!==a?t(Me(n,r,a),e):e},e.createDo=Oe,e.updateDo=function(e,n,r){return e.statement!==n||e.expression!==r?t(Oe(n,r),e):e},e.createWhile=Ie,e.updateWhile=function(e,n,r){return e.expression!==n||e.statement!==r?t(Ie(n,r),e):e},e.createFor=Re,e.updateFor=function(e,n,r,a,i){return e.initializer!==n||e.condition!==r||e.incrementor!==a||e.statement!==i?t(Re(n,r,a,i),e):e},e.createForIn=Ne,e.updateForIn=function(e,n,r,a){return e.initializer!==n||e.expression!==r||e.statement!==a?t(Ne(n,r,a),e):e},e.createForOf=we,e.updateForOf=function(e,n,r,a,i){return e.awaitModifier!==n||e.initializer!==r||e.expression!==a||e.statement!==i?t(we(n,r,a,i),e):e},e.createContinue=Pe,e.updateContinue=function(e,n){return e.label!==n?t(Pe(n),e):e},e.createBreak=Fe,e.updateBreak=function(e,n){return e.label!==n?t(Fe(n),e):e},e.createReturn=Ge,e.updateReturn=function(e,n){return e.expression!==n?t(Ge(n),e):e},e.createWith=Ve,e.updateWith=function(e,n,r){return e.expression!==n||e.statement!==r?t(Ve(n,r),e):e},e.createSwitch=Ke,e.updateSwitch=function(e,n,r){return e.expression!==n||e.caseBlock!==r?t(Ke(n,r),e):e},e.createLabel=Be,e.updateLabel=function(e,n,r){return e.label!==n||e.statement!==r?t(Be(n,r),e):e},e.createThrow=He,e.updateThrow=function(e,n){return e.expression!==n?t(He(n),e):e},e.createTry=Ue,e.updateTry=function(e,n,r,a){return e.tryBlock!==n||e.catchClause!==r||e.finallyBlock!==a?t(Ue(n,r,a),e):e},e.createDebuggerStatement=function(){return n(234)},e.createVariableDeclaration=je,e.updateVariableDeclaration=function(e,n,r,a){return e.name!==n||e.type!==r||e.initializer!==a?t(je(n,r,a),e):e},e.createVariableDeclarationList=We,e.updateVariableDeclarationList=function(e,n){return e.declarations!==n?t(We(n,e.flags),e):e},e.createFunctionDeclaration=qe,e.updateFunctionDeclaration=function(e,n,r,a,i,o,s,l,c){return e.decorators!==n||e.modifiers!==r||e.asteriskToken!==a||e.name!==i||e.typeParameters!==o||e.parameters!==s||e.type!==l||e.body!==c?t(qe(n,r,a,i,o,s,l,c),e):e},e.createClassDeclaration=ze,e.updateClassDeclaration=function(e,n,r,a,i,o,s){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.typeParameters!==i||e.heritageClauses!==o||e.members!==s?t(ze(n,r,a,i,o,s),e):e},e.createInterfaceDeclaration=Je,e.updateInterfaceDeclaration=function(e,n,r,a,i,o,s){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.typeParameters!==i||e.heritageClauses!==o||e.members!==s?t(Je(n,r,a,i,o,s),e):e},e.createTypeAliasDeclaration=Xe,e.updateTypeAliasDeclaration=function(e,n,r,a,i,o){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.typeParameters!==i||e.type!==o?t(Xe(n,r,a,i,o),e):e},e.createEnumDeclaration=Ye,e.updateEnumDeclaration=function(e,n,r,a,i){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.members!==i?t(Ye(n,r,a,i),e):e},e.createModuleDeclaration=Qe,e.updateModuleDeclaration=function(e,n,r,a,i){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.body!==i?t(Qe(n,r,a,i,e.flags),e):e},e.createModuleBlock=Ze,e.updateModuleBlock=function(e,n){return e.statements!==n?t(Ze(n),e):e},e.createCaseBlock=$e,e.updateCaseBlock=function(e,n){return e.clauses!==n?t($e(n),e):e},e.createNamespaceExportDeclaration=en,e.updateNamespaceExportDeclaration=function(e,n){return e.name!==n?t(en(n),e):e},e.createImportEqualsDeclaration=nn,e.updateImportEqualsDeclaration=function(e,n,r,a,i){return e.decorators!==n||e.modifiers!==r||e.name!==a||e.moduleReference!==i?t(nn(n,r,a,i),e):e},e.createImportDeclaration=tn,e.updateImportDeclaration=function(e,n,r,a,i){return e.decorators!==n||e.modifiers!==r||e.importClause!==a||e.moduleSpecifier!==i?t(tn(n,r,a,i),e):e},e.createImportClause=rn,e.updateImportClause=function(e,n,r){return e.name!==n||e.namedBindings!==r?t(rn(n,r),e):e},e.createNamespaceImport=an,e.updateNamespaceImport=function(e,n){return e.name!==n?t(an(n),e):e},e.createNamedImports=on,e.updateNamedImports=function(e,n){return e.elements!==n?t(on(n),e):e},e.createImportSpecifier=sn,e.updateImportSpecifier=function(e,n,r){return e.propertyName!==n||e.name!==r?t(sn(n,r),e):e},e.createExportAssignment=ln,e.updateExportAssignment=function(e,n,r,a){return e.decorators!==n||e.modifiers!==r||e.expression!==a?t(ln(n,r,e.isExportEquals,a),e):e},e.createExportDeclaration=cn,e.updateExportDeclaration=function(e,n,r,a,i){return e.decorators!==n||e.modifiers!==r||e.exportClause!==a||e.moduleSpecifier!==i?t(cn(n,r,a,i),e):e},e.createNamedExports=un,e.updateNamedExports=function(e,n){return e.elements!==n?t(un(n),e):e},e.createExportSpecifier=dn,e.updateExportSpecifier=function(e,n,r){return e.propertyName!==n||e.name!==r?t(dn(n,r),e):e},e.createExternalModuleReference=mn,e.updateExternalModuleReference=function(e,n){return e.expression!==n?t(mn(n),e):e},e.createJsxElement=pn,e.updateJsxElement=function(e,n,r,a){return e.openingElement!==n||e.children!==r||e.closingElement!==a?t(pn(n,r,a),e):e},e.createJsxSelfClosingElement=fn,e.updateJsxSelfClosingElement=function(e,n,r,a){return e.tagName!==n||e.typeArguments!==r||e.attributes!==a?t(fn(n,r,a),e):e},e.createJsxOpeningElement=gn,e.updateJsxOpeningElement=function(e,n,r,a){return e.tagName!==n||e.typeArguments!==r||e.attributes!==a?t(gn(n,r,a),e):e},e.createJsxClosingElement=_n,e.updateJsxClosingElement=function(e,n){return e.tagName!==n?t(_n(n),e):e},e.createJsxFragment=vn,e.updateJsxFragment=function(e,n,r,a){return e.openingFragment!==n||e.children!==r||e.closingFragment!==a?t(vn(n,r,a),e):e},e.createJsxAttribute=yn,e.updateJsxAttribute=function(e,n,r){return e.name!==n||e.initializer!==r?t(yn(n,r),e):e},e.createJsxAttributes=hn,e.updateJsxAttributes=function(e,n){return e.properties!==n?t(hn(n),e):e},e.createJsxSpreadAttribute=bn,e.updateJsxSpreadAttribute=function(e,n){return e.expression!==n?t(bn(n),e):e},e.createJsxExpression=En,e.updateJsxExpression=function(e,n){return e.expression!==n?t(En(e.dotDotDotToken,n),e):e},e.createCaseClause=Tn,e.updateCaseClause=function(e,n,r){return e.expression!==n||e.statements!==r?t(Tn(n,r),e):e},e.createDefaultClause=Sn,e.updateDefaultClause=function(e,n){return e.statements!==n?t(Sn(n),e):e},e.createHeritageClause=Ln,e.updateHeritageClause=function(e,n){return e.types!==n?t(Ln(e.token,n),e):e},e.createCatchClause=xn,e.updateCatchClause=function(e,n,r){return e.variableDeclaration!==n||e.block!==r?t(xn(n,r),e):e},e.createPropertyAssignment=An,e.updatePropertyAssignment=function(e,n,r){return e.name!==n||e.initializer!==r?t(An(n,r),e):e},e.createShorthandPropertyAssignment=Cn,e.updateShorthandPropertyAssignment=function(e,n,r){return e.name!==n||e.objectAssignmentInitializer!==r?t(Cn(n,r),e):e},e.createSpreadAssignment=Dn,e.updateSpreadAssignment=function(e,n){return e.expression!==n?t(Dn(n),e):e},e.createEnumMember=kn,e.updateEnumMember=function(e,n,r){return e.name!==n||e.initializer!==r?t(kn(n,r),e):e},e.updateSourceFileNode=function(e,a,i,o,s,l,c){if(e.statements!==a||void 0!==i&&e.isDeclarationFile!==i||void 0!==o&&e.referencedFiles!==o||void 0!==s&&e.typeReferenceDirectives!==s||void 0!==c&&e.libReferenceDirectives!==c||void 0!==l&&e.hasNoDefaultLib!==l){var u=n(277);return u.flags|=e.flags,u.statements=r(a),u.endOfFileToken=e.endOfFileToken,u.fileName=e.fileName,u.path=e.path,u.text=e.text,u.isDeclarationFile=void 0===i?e.isDeclarationFile:i,u.referencedFiles=void 0===o?e.referencedFiles:o,u.typeReferenceDirectives=void 0===s?e.typeReferenceDirectives:s,u.hasNoDefaultLib=void 0===l?e.hasNoDefaultLib:l,u.libReferenceDirectives=void 0===c?e.libReferenceDirectives:c,void 0!==e.amdDependencies&&(u.amdDependencies=e.amdDependencies),void 0!==e.moduleName&&(u.moduleName=e.moduleName),void 0!==e.languageVariant&&(u.languageVariant=e.languageVariant),void 0!==e.renamedDependencies&&(u.renamedDependencies=e.renamedDependencies),void 0!==e.languageVersion&&(u.languageVersion=e.languageVersion),void 0!==e.scriptKind&&(u.scriptKind=e.scriptKind),void 0!==e.externalModuleIndicator&&(u.externalModuleIndicator=e.externalModuleIndicator),void 0!==e.commonJsModuleIndicator&&(u.commonJsModuleIndicator=e.commonJsModuleIndicator),void 0!==e.identifiers&&(u.identifiers=e.identifiers),void 0!==e.nodeCount&&(u.nodeCount=e.nodeCount),void 0!==e.identifierCount&&(u.identifierCount=e.identifierCount),void 0!==e.symbolCount&&(u.symbolCount=e.symbolCount),void 0!==e.parseDiagnostics&&(u.parseDiagnostics=e.parseDiagnostics),void 0!==e.bindDiagnostics&&(u.bindDiagnostics=e.bindDiagnostics),void 0!==e.bindSuggestionDiagnostics&&(u.bindSuggestionDiagnostics=e.bindSuggestionDiagnostics),void 0!==e.lineMap&&(u.lineMap=e.lineMap),void 0!==e.classifiableNames&&(u.classifiableNames=e.classifiableNames),void 0!==e.resolvedModules&&(u.resolvedModules=e.resolvedModules),void 0!==e.resolvedTypeReferenceDirectiveNames&&(u.resolvedTypeReferenceDirectiveNames=e.resolvedTypeReferenceDirectiveNames),void 0!==e.imports&&(u.imports=e.imports),void 0!==e.moduleAugmentations&&(u.moduleAugmentations=e.moduleAugmentations),void 0!==e.pragmas&&(u.pragmas=e.pragmas),void 0!==e.localJsxFactory&&(u.localJsxFactory=e.localJsxFactory),void 0!==e.localJsxNamespace&&(u.localJsxNamespace=e.localJsxNamespace),t(u,e)}return e},e.getMutableClone=function(e){var n=a(e);return n.pos=e.pos,n.end=e.end,n.parent=e.parent,n},e.createNotEmittedStatement=function(e){var t=n(305);return t.original=e,Fn(t,e),t},e.createEndOfDeclarationMarker=function(e){var t=n(309);return t.emitNode={},t.original=e,t},e.createMergeDeclarationMarker=function(e){var t=n(308);return t.emitNode={},t.original=e,t},e.createPartiallyEmittedExpression=Mn,e.updatePartiallyEmittedExpression=function(e,n){return e.expression!==n?t(Mn(n,e.original),e):e},e.createCommaList=In,e.updateCommaList=function(e,n){return e.elements!==n?t(In(n),e):e},e.createBundle=Rn,e.createUnparsedSourceFile=function(n,t,r){var a=e.createNode(279);return a.text=n,a.sourceMapPath=t,a.sourceMapText=r,a},e.createInputFiles=function(n,t,r,a,i,o){var s=e.createNode(280);return s.javascriptText=n,s.javascriptMapPath=r,s.javascriptMapText=a,s.declarationText=t,s.declarationMapPath=i,s.declarationMapText=o,s},e.updateBundle=function(n,t,r){return void 0===r&&(r=e.emptyArray),n.sourceFiles!==t||n.prepends!==r?Rn(t,r):n},e.createImmediatelyInvokedFunctionExpression=function(e,n,t){return te(se(void 0,void 0,void 0,void 0,n?[n]:[],void 0,Ae(e,!0)),void 0,t?[t]:[])},e.createImmediatelyInvokedArrowFunction=function(e,n,t){return te(le(void 0,void 0,n?[n]:[],void 0,void 0,Ae(e,!0)),void 0,t?[t]:[])},e.createComma=function(e,n){return ge(e,26,n)},e.createLessThan=function(e,n){return ge(e,27,n)},e.createAssignment=function(e,n){return ge(e,58,n)},e.createStrictEquality=function(e,n){return ge(e,34,n)},e.createStrictInequality=function(e,n){return ge(e,35,n)},e.createAdd=function(e,n){return ge(e,37,n)},e.createSubtract=function(e,n){return ge(e,38,n)},e.createPostfixIncrement=function(e){return fe(e,43)},e.createLogicalAnd=function(e,n){return ge(e,53,n)},e.createLogicalOr=function(e,n){return ge(e,54,n)},e.createLogicalNot=function(e){return pe(51,e)},e.createVoidZero=function(){return de(i(0))},e.createExportDefault=function(e){return ln(void 0,void 0,!1,e)},e.createExternalModuleExport=function(e){return cn(void 0,void 0,un([dn(void 0,e)]))},e.disposeEmitNodes=function(n){var t=(n=e.getSourceFileOfNode(e.getParseTreeNode(n)))&&n.emitNode,r=t&&t.annotatedNodes;if(r)for(var a=0,i=r;a<i.length;a++)i[a].emitNode=void 0},e.getOrCreateEmitNode=Pn,e.setTextRange=Fn,e.setEmitFlags=Gn,e.addEmitFlags=function(e,n){var t=Pn(e);return t.flags=t.flags|n,e},e.getSourceMapRange=function(e){var n=e.emitNode;return n&&n.sourceMapRange||e},e.setSourceMapRange=function(e,n){return Pn(e).sourceMapRange=n,e},e.createSourceMapSource=function(n,t,r){return new(c||(c=e.objectAllocator.getSourceMapSourceConstructor()))(n,t,r)},e.getTokenSourceMapRange=function(e,n){var t=e.emitNode,r=t&&t.tokenSourceMapRanges;return r&&r[n]},e.setTokenSourceMapRange=function(e,n,t){var r=Pn(e);return(r.tokenSourceMapRanges||(r.tokenSourceMapRanges=[]))[n]=t,e},e.getStartsOnNewLine=function(e){var n=e.emitNode;return n&&n.startsOnNewLine},e.setStartsOnNewLine=function(e,n){return Pn(e).startsOnNewLine=n,e},e.getCommentRange=function(e){var n=e.emitNode;return n&&n.commentRange||e},e.setCommentRange=function(e,n){return Pn(e).commentRange=n,e},e.getSyntheticLeadingComments=Vn,e.setSyntheticLeadingComments=Kn,e.addSyntheticLeadingComment=function(n,t,r,a){return Kn(n,e.append(Vn(n),{kind:t,pos:-1,end:-1,hasTrailingNewLine:a,text:r}))},e.getSyntheticTrailingComments=Bn,e.setSyntheticTrailingComments=Hn,e.addSyntheticTrailingComment=function(n,t,r,a){return Hn(n,e.append(Bn(n),{kind:t,pos:-1,end:-1,hasTrailingNewLine:a,text:r}))},e.moveSyntheticComments=function(e,n){Kn(e,Vn(n)),Hn(e,Bn(n));var t=Pn(n);return t.leadingComments=void 0,t.trailingComments=void 0,e},e.getConstantValue=function(e){var n=e.emitNode;return n&&n.constantValue},e.setConstantValue=function(e,n){return Pn(e).constantValue=n,e},e.addEmitHelper=function(n,t){var r=Pn(n);return r.helpers=e.append(r.helpers,t),n},e.addEmitHelpers=function(n,t){if(e.some(t))for(var r=Pn(n),a=0,i=t;a<i.length;a++){var o=i[a];r.helpers=e.appendIfUnique(r.helpers,o)}return n},e.removeEmitHelper=function(n,t){var r=n.emitNode;if(r){var a=r.helpers;if(a)return e.orderedRemoveItem(a,t)}return!1},e.getEmitHelpers=function(e){var n=e.emitNode;return n&&n.helpers},e.moveEmitHelpers=function(n,t,r){var a=n.emitNode,i=a&&a.helpers;if(e.some(i)){for(var o=Pn(t),s=0,l=0;l<i.length;l++){var c=i[l];r(c)?(s++,o.helpers=e.appendIfUnique(o.helpers,c)):s>0&&(i[l-s]=c)}s>0&&(i.length-=s)}},e.compareEmitHelpers=function(n,t){return n===t?0:n.priority===t.priority?0:void 0===n.priority?1:void 0===t.priority?-1:e.compareValues(n.priority,t.priority)},e.setOriginalNode=Un}(Tn||(Tn={})),function(e){function n(n,t,r){if(e.isComputedPropertyName(t))return e.setTextRange(e.createElementAccess(n,t.expression),r);var a=e.setTextRange(e.isIdentifier(t)?e.createPropertyAccess(n,t):e.createElementAccess(n,t),t);return e.getOrCreateEmitNode(a).flags|=64,a}function t(n,t){var r=e.createIdentifier(n||\"React\");return r.flags&=-9,r.parent=e.getParseTreeNode(t),r}function r(n,r,a){return n?function n(r,a){if(e.isQualifiedName(r)){var i=n(r.left,a),o=e.createIdentifier(e.idText(r.right));return o.escapedText=r.right.escapedText,e.createPropertyAccess(i,o)}return t(e.idText(r),a)}(n,a):e.createPropertyAccess(t(r,a),\"createElement\")}function a(n){return e.setEmitFlags(e.createIdentifier(n),4098)}e.nullTransformationContext={enableEmitNotification:e.noop,enableSubstitution:e.noop,endLexicalEnvironment:function(){},getCompilerOptions:e.notImplemented,getEmitHost:e.notImplemented,getEmitResolver:e.notImplemented,hoistFunctionDeclaration:e.noop,hoistVariableDeclaration:e.noop,isEmitNotificationEnabled:e.notImplemented,isSubstitutionEnabled:e.notImplemented,onEmitNode:e.noop,onSubstituteNode:e.notImplemented,readEmitHelpers:e.notImplemented,requestEmitHelper:e.noop,resumeLexicalEnvironment:e.noop,startLexicalEnvironment:e.noop,suspendLexicalEnvironment:e.noop,addDiagnostic:e.noop},e.createTypeCheck=function(n,t){return\"undefined\"===t?e.createStrictEquality(n,e.createVoidZero()):e.createStrictEquality(e.createTypeOf(n),e.createLiteral(t))},e.createMemberAccessForPropertyName=n,e.createFunctionCall=function(n,t,r,a){return e.setTextRange(e.createCall(e.createPropertyAccess(n,\"call\"),void 0,[t].concat(r)),a)},e.createFunctionApply=function(n,t,r,a){return e.setTextRange(e.createCall(e.createPropertyAccess(n,\"apply\"),void 0,[t,r]),a)},e.createArraySlice=function(n,t){var r=[];return void 0!==t&&r.push(\"number\"==typeof t?e.createLiteral(t):t),e.createCall(e.createPropertyAccess(n,\"slice\"),void 0,r)},e.createArrayConcat=function(n,t){return e.createCall(e.createPropertyAccess(n,\"concat\"),void 0,t)},e.createMathPow=function(n,t,r){return e.setTextRange(e.createCall(e.createPropertyAccess(e.createIdentifier(\"Math\"),\"pow\"),void 0,[n,t]),r)},e.createExpressionForJsxElement=function(n,t,a,i,o,s,l){var c=[a];if(i&&c.push(i),o&&o.length>0)if(i||c.push(e.createNull()),o.length>1)for(var u=0,d=o;u<d.length;u++){var m=d[u];D(m),c.push(m)}else c.push(o[0]);return e.setTextRange(e.createCall(r(n,t,s),void 0,c),l)},e.createExpressionForJsxFragment=function(n,a,i,o,s){var l=[e.createPropertyAccess(t(a,o),\"Fragment\")];if(l.push(e.createNull()),i&&i.length>0)if(i.length>1)for(var c=0,u=i;c<u.length;c++){var d=u[c];D(d),l.push(d)}else l.push(i[0]);return e.setTextRange(e.createCall(r(n,a,o),void 0,l),s)},e.getHelperName=a;var i={name:\"typescript:values\",scoped:!1,text:'\\n            var __values = (this && this.__values) || function (o) {\\n                var m = typeof Symbol === \"function\" && o[Symbol.iterator], i = 0;\\n                if (m) return m.call(o);\\n                return {\\n                    next: function () {\\n                        if (o && i >= o.length) o = void 0;\\n                        return { value: o && o[i++], done: !o };\\n                    }\\n                };\\n            };'};e.createValuesHelper=function(n,t,r){return n.requestEmitHelper(i),e.setTextRange(e.createCall(a(\"__values\"),void 0,[t]),r)};var o={name:\"typescript:read\",scoped:!1,text:'\\n            var __read = (this && this.__read) || function (o, n) {\\n                var m = typeof Symbol === \"function\" && o[Symbol.iterator];\\n                if (!m) return o;\\n                var i = m.call(o), r, ar = [], e;\\n                try {\\n                    while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);\\n                }\\n                catch (error) { e = { error: error }; }\\n                finally {\\n                    try {\\n                        if (r && !r.done && (m = i[\"return\"])) m.call(i);\\n                    }\\n                    finally { if (e) throw e.error; }\\n                }\\n                return ar;\\n            };'};e.createReadHelper=function(n,t,r,i){return n.requestEmitHelper(o),e.setTextRange(e.createCall(a(\"__read\"),void 0,void 0!==r?[t,e.createLiteral(r)]:[t]),i)};var s={name:\"typescript:spread\",scoped:!1,text:\"\\n            var __spread = (this && this.__spread) || function () {\\n                for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));\\n                return ar;\\n            };\"};function l(n,t){var r=e.skipParentheses(n);switch(r.kind){case 71:return t;case 99:case 8:case 9:return!1;case 185:return 0!==r.elements.length;case 186:return r.properties.length>0;default:return!0}}function c(n){return e.isIdentifier(n)?e.createLiteral(n):e.isComputedPropertyName(n)?e.getMutableClone(n.expression):e.getMutableClone(n)}function u(e,n,t){return d(e,n,t,8192)}function d(n,t,r,a){void 0===a&&(a=0);var i=e.getNameOfDeclaration(n);if(i&&e.isIdentifier(i)&&!e.isGeneratedIdentifier(i)){var o=e.getMutableClone(i);return a|=e.getEmitFlags(i),r||(a|=48),t||(a|=1536),a&&e.setEmitFlags(o,a),o}return e.getGeneratedNameForNode(n)}function m(n,t,r,a){var i=e.createPropertyAccess(n,e.nodeIsSynthesized(t)?t:e.getSynthesizedClone(t));e.setTextRange(i,t);var o=0;return a||(o|=48),r||(o|=1536),o&&e.setEmitFlags(i,o),i}function p(n){return e.isStringLiteral(n.expression)&&\"use strict\"===n.expression.text}function f(n,t,r){e.Debug.assert(0===n.length,\"Prologue directives should be at the first statement in the target statements array\");for(var a=!1,i=0,o=t.length;i<o;){var s=t[i];if(!e.isPrologueDirective(s))break;p(s)&&(a=!0),n.push(s),i++}return r&&!a&&n.push(D(e.createStatement(e.createLiteral(\"use strict\")))),i}function g(n,t,r,a){for(var i=t.length;void 0!==r&&r<i;){var o=t[r];if(!(1048576&e.getEmitFlags(o)))break;e.append(n,a?e.visitNode(o,a,e.isStatement):o),r++}return r}function _(n){for(var t=0,r=n;t<r.length;t++){var a=r[t];if(!e.isPrologueDirective(a))break;if(p(a))return a}}function v(n){if(n=e.skipPartiallyEmittedExpressions(n),e.isLiteralKind(n.kind))return n.kind;if(202===n.kind&&37===n.operatorToken.kind){if(void 0!==n.cachedLiteralKind)return n.cachedLiteralKind;var t=v(n.left),r=e.isLiteralKind(t)&&t===v(n.right)?t:0;return n.cachedLiteralKind=r,r}return 0}function y(n){var t=e.skipPartiallyEmittedExpressions(n);return e.isLeftHandSideExpression(t)&&(190!==t.kind||t.arguments)?n:e.setTextRange(e.createParen(n),n)}function h(n){var t=e.skipPartiallyEmittedExpressions(n);return e.getExpressionPrecedence(t)>e.getOperatorPrecedence(202,26)?n:e.setTextRange(e.createParen(n),n)}function b(n){return 173===n.kind?e.createParenthesizedType(n):n}function E(n){switch(n.kind){case 171:case 172:case 163:case 164:return e.createParenthesizedType(n)}return b(n)}function T(e,n){for(;;){switch(e.kind){case 201:e=e.operand;continue;case 202:e=e.left;continue;case 203:e=e.condition;continue;case 191:e=e.tag;continue;case 189:if(n)return e;case 210:case 188:case 187:case 211:case 306:e=e.expression;continue}return e}}function S(e){return 202===e.kind&&26===e.operatorToken.kind||307===e.kind}function L(e,n){switch(void 0===n&&(n=7),e.kind){case 193:return 0!=(1&n);case 192:case 210:case 211:return 0!=(2&n);case 306:return 0!=(4&n)}return!1}function x(n,t){var r;void 0===t&&(t=7);do{r=n,1&t&&(n=e.skipParentheses(n)),2&t&&(n=A(n)),4&t&&(n=e.skipPartiallyEmittedExpressions(n))}while(r!==n);return n}function A(n){for(;e.isAssertionExpression(n)||211===n.kind;)n=n.expression;return n}function C(n,t,r){return void 0===r&&(r=7),n&&L(n,r)&&(!(193===(a=n).kind&&e.nodeIsSynthesized(a)&&e.nodeIsSynthesized(e.getSourceMapRange(a))&&e.nodeIsSynthesized(e.getCommentRange(a)))||e.some(e.getSyntheticLeadingComments(a))||e.some(e.getSyntheticTrailingComments(a)))?function(n,t){switch(n.kind){case 193:return e.updateParen(n,t);case 192:return e.updateTypeAssertion(n,n.type,t);case 210:return e.updateAsExpression(n,t,n.type);case 211:return e.updateNonNullExpression(n,t);case 306:return e.updatePartiallyEmittedExpression(n,t)}}(n,C(n.expression,t)):t;var a}function D(n){return e.setStartsOnNewLine(n,!0)}function k(n){var t=e.getOriginalNode(n,e.isSourceFile),r=t&&t.emitNode;return r&&r.externalHelpersModuleName}function M(n,t,r){if(n)return n.moduleName?e.createLiteral(n.moduleName):n.isDeclarationFile||!r.out&&!r.outFile?void 0:e.createLiteral(e.getExternalModuleNameFromPath(t,n.fileName))}function O(n){if(e.isDeclarationBindingElement(n))return n.name;if(!e.isObjectLiteralElementLike(n))return e.isAssignmentExpression(n,!0)?O(n.left):e.isSpreadElement(n)?O(n.expression):n;switch(n.kind){case 273:return O(n.initializer);case 274:return n.name;case 275:return O(n.expression)}}function I(e){var n=e.kind;return 9===n||8===n}function R(n){if(e.isBindingElement(n)){if(n.dotDotDotToken)return e.Debug.assertNode(n.name,e.isIdentifier),e.setOriginalNode(e.setTextRange(e.createSpread(n.name),n),n);var t=G(n.name);return n.initializer?e.setOriginalNode(e.setTextRange(e.createAssignment(t,n.initializer),n),n):t}return e.Debug.assertNode(n,e.isExpression),n}function N(n){if(e.isBindingElement(n)){if(n.dotDotDotToken)return e.Debug.assertNode(n.name,e.isIdentifier),e.setOriginalNode(e.setTextRange(e.createSpreadAssignment(n.name),n),n);if(n.propertyName){var t=G(n.name);return e.setOriginalNode(e.setTextRange(e.createPropertyAssignment(n.propertyName,n.initializer?e.createAssignment(t,n.initializer):t),n),n)}return e.Debug.assertNode(n.name,e.isIdentifier),e.setOriginalNode(e.setTextRange(e.createShorthandPropertyAssignment(n.name,n.initializer),n),n)}return e.Debug.assertNode(n,e.isObjectLiteralElementLike),n}function w(e){switch(e.kind){case 183:case 185:return F(e);case 182:case 186:return P(e)}}function P(n){return e.isObjectBindingPattern(n)?e.setOriginalNode(e.setTextRange(e.createObjectLiteral(e.map(n.elements,N)),n),n):(e.Debug.assertNode(n,e.isObjectLiteralExpression),n)}function F(n){return e.isArrayBindingPattern(n)?e.setOriginalNode(e.setTextRange(e.createArrayLiteral(e.map(n.elements,R)),n),n):(e.Debug.assertNode(n,e.isArrayLiteralExpression),n)}function G(n){return e.isBindingPattern(n)?w(n):(e.Debug.assertNode(n,e.isExpression),n)}e.createSpreadHelper=function(n,t,r){return n.requestEmitHelper(o),n.requestEmitHelper(s),e.setTextRange(e.createCall(a(\"__spread\"),void 0,t),r)},e.createForOfBindingStatement=function(n,t){if(e.isVariableDeclarationList(n)){var r=e.first(n.declarations),a=e.updateVariableDeclaration(r,r.name,void 0,t);return e.setTextRange(e.createVariableStatement(void 0,e.updateVariableDeclarationList(n,[a])),n)}var i=e.setTextRange(e.createAssignment(n,t),n);return e.setTextRange(e.createStatement(i),n)},e.insertLeadingStatement=function(n,t){return e.isBlock(n)?e.updateBlock(n,e.setTextRange(e.createNodeArray([t].concat(n.statements)),n.statements)):e.createBlock(e.createNodeArray([n,t]),!0)},e.restoreEnclosingLabel=function n(t,r,a){if(!r)return t;var i=e.updateLabel(r,r.label,231===r.statement.kind?n(t,r.statement):t);return a&&a(r),i},e.createCallBinding=function(n,t,r,a){void 0===a&&(a=!1);var i,o,s=x(n,7);if(e.isSuperProperty(s))i=e.createThis(),o=s;else if(97===s.kind)i=e.createThis(),o=r<2?e.setTextRange(e.createIdentifier(\"_super\"),s):s;else if(4096&e.getEmitFlags(s))i=e.createVoidZero(),o=y(s);else switch(s.kind){case 187:l(s.expression,a)?(i=e.createTempVariable(t),o=e.createPropertyAccess(e.setTextRange(e.createAssignment(i,s.expression),s.expression),s.name),e.setTextRange(o,s)):(i=s.expression,o=s);break;case 188:l(s.expression,a)?(i=e.createTempVariable(t),o=e.createElementAccess(e.setTextRange(e.createAssignment(i,s.expression),s.expression),s.argumentExpression),e.setTextRange(o,s)):(i=s.expression,o=s);break;default:i=e.createVoidZero(),o=y(n)}return{target:o,thisArg:i}},e.inlineExpressions=function(n){return n.length>10?e.createCommaList(n):e.reduceLeft(n,e.createComma)},e.createExpressionFromEntityName=function n(t){if(e.isQualifiedName(t)){var r=n(t.left),a=e.getMutableClone(t.right);return e.setTextRange(e.createPropertyAccess(r,a),t)}return e.getMutableClone(t)},e.createExpressionForPropertyName=c,e.createExpressionForObjectLiteralElementLike=function(t,r,a){switch(r.kind){case 156:case 157:return function(n,t,r,a){var i=e.getAllAccessorDeclarations(n,t),o=i.firstAccessor,s=i.getAccessor,l=i.setAccessor;if(t===o){var u=[];if(s){var d=e.createFunctionExpression(s.modifiers,void 0,void 0,void 0,s.parameters,void 0,s.body);e.setTextRange(d,s),e.setOriginalNode(d,s);var m=e.createPropertyAssignment(\"get\",d);u.push(m)}if(l){var p=e.createFunctionExpression(l.modifiers,void 0,void 0,void 0,l.parameters,void 0,l.body);e.setTextRange(p,l),e.setOriginalNode(p,l);var f=e.createPropertyAssignment(\"set\",p);u.push(f)}u.push(e.createPropertyAssignment(\"enumerable\",e.createTrue())),u.push(e.createPropertyAssignment(\"configurable\",e.createTrue()));var g=e.setTextRange(e.createCall(e.createPropertyAccess(e.createIdentifier(\"Object\"),\"defineProperty\"),void 0,[r,c(t.name),e.createObjectLiteral(u,a)]),o);return e.aggregateTransformFlags(g)}}(t.properties,r,a,!!t.multiLine);case 273:return function(t,r){return e.aggregateTransformFlags(e.setOriginalNode(e.setTextRange(e.createAssignment(n(r,t.name,t.name),t.initializer),t),t))}(r,a);case 274:return function(t,r){return e.aggregateTransformFlags(e.setOriginalNode(e.setTextRange(e.createAssignment(n(r,t.name,t.name),e.getSynthesizedClone(t.name)),t),t))}(r,a);case 154:return function(t,r){return e.aggregateTransformFlags(e.setOriginalNode(e.setTextRange(e.createAssignment(n(r,t.name,t.name),e.setOriginalNode(e.setTextRange(e.createFunctionExpression(t.modifiers,t.asteriskToken,void 0,void 0,t.parameters,void 0,t.body),t),t)),t),t))}(r,a)}},e.getInternalName=function(e,n,t){return d(e,n,t,49152)},e.isInternalName=function(n){return 0!=(32768&e.getEmitFlags(n))},e.getLocalName=function(e,n,t){return d(e,n,t,16384)},e.isLocalName=function(n){return 0!=(16384&e.getEmitFlags(n))},e.getExportName=u,e.isExportName=function(n){return 0!=(8192&e.getEmitFlags(n))},e.getDeclarationName=function(e,n,t){return d(e,n,t)},e.getExternalModuleOrNamespaceExportName=function(n,t,r,a){return n&&e.hasModifier(t,1)?m(n,d(t),r,a):u(t,r,a)},e.getNamespaceMemberName=m,e.convertToFunctionBody=function(n,t){return e.isBlock(n)?n:e.setTextRange(e.createBlock([e.setTextRange(e.createReturn(n),n)],t),n)},e.convertFunctionDeclarationToExpression=function(n){if(!n.body)return e.Debug.fail();var t=e.createFunctionExpression(n.modifiers,n.asteriskToken,n.name,n.typeParameters,n.parameters,n.type,n.body);return e.setOriginalNode(t,n),e.setTextRange(t,n),e.getStartsOnNewLine(n)&&e.setStartsOnNewLine(t,!0),e.aggregateTransformFlags(t),t},e.addPrologue=function(e,n,t,r){return g(e,n,f(e,n,t),r)},e.addStandardPrologue=f,e.addCustomPrologue=g,e.findUseStrictPrologue=_,e.startsWithUseStrict=function(n){var t=e.firstOrUndefined(n);return void 0!==t&&e.isPrologueDirective(t)&&p(t)},e.ensureUseStrict=function(n){return _(n)?n:e.setTextRange(e.createNodeArray([D(e.createStatement(e.createLiteral(\"use strict\")))].concat(n)),n)},e.parenthesizeBinaryOperand=function(n,t,r,a){return 193===e.skipPartiallyEmittedExpressions(t).kind?t:function(n,t,r,a){var i=e.getOperatorPrecedence(202,n),o=e.getOperatorAssociativity(202,n),s=e.skipPartiallyEmittedExpressions(t);if(!r&&195===t.kind&&i>4)return!0;var l=e.getExpressionPrecedence(s);switch(e.compareValues(l,i)){case-1:return!(!r&&1===o&&205===t.kind);case 1:return!1;case 0:if(r)return 1===o;if(e.isBinaryExpression(s)&&s.operatorToken.kind===n){if(function(e){return 39===e||49===e||48===e||50===e}(n))return!1;if(37===n){var c=a?v(a):0;if(e.isLiteralKind(c)&&c===v(s))return!1}}var u=e.getExpressionAssociativity(s);return 0===u}}(n,t,r,a)?e.createParen(t):t},e.parenthesizeForConditionalHead=function(n){var t=e.getOperatorPrecedence(203,55),r=e.skipPartiallyEmittedExpressions(n),a=e.getExpressionPrecedence(r);return-1===e.compareValues(a,t)?e.createParen(n):n},e.parenthesizeSubexpressionOfConditionalExpression=function(n){return S(e.skipPartiallyEmittedExpressions(n))?e.createParen(n):n},e.parenthesizeDefaultExpression=function(n){var t=e.skipPartiallyEmittedExpressions(n),r=S(t);if(!r)switch(T(t,!1).kind){case 207:case 194:r=!0}return r?e.createParen(n):n},e.parenthesizeForNew=function(n){var t=T(n,!0);switch(t.kind){case 189:return e.createParen(n);case 190:return t.arguments?n:e.createParen(n)}return y(n)},e.parenthesizeForAccess=y,e.parenthesizePostfixOperand=function(n){return e.isLeftHandSideExpression(n)?n:e.setTextRange(e.createParen(n),n)},e.parenthesizePrefixOperand=function(n){return e.isUnaryExpression(n)?n:e.setTextRange(e.createParen(n),n)},e.parenthesizeListElements=function(n){for(var t,r=0;r<n.length;r++){var a=h(n[r]);void 0===t&&a===n[r]||(void 0===t&&(t=n.slice(0,r)),t.push(a))}return void 0!==t?e.setTextRange(e.createNodeArray(t,n.hasTrailingComma),n):n},e.parenthesizeExpressionForList=h,e.parenthesizeExpressionForExpressionStatement=function(n){var t=e.skipPartiallyEmittedExpressions(n);if(e.isCallExpression(t)){var r=t.expression,a=e.skipPartiallyEmittedExpressions(r).kind;if(194===a||195===a){var i=e.getMutableClone(t);return i.expression=e.setTextRange(e.createParen(r),r),C(n,i,4)}}var o=T(t,!1).kind;return 186===o||194===o?e.setTextRange(e.createParen(n),n):n},e.parenthesizeConditionalTypeMember=b,e.parenthesizeElementTypeMember=E,e.parenthesizeArrayTypeMember=function(n){switch(n.kind){case 165:case 177:case 174:return e.createParenthesizedType(n)}return E(n)},e.parenthesizeElementTypeMembers=function(n){return e.createNodeArray(e.sameMap(n,E))},e.parenthesizeTypeParameters=function(n){if(e.some(n)){for(var t=[],r=0;r<n.length;++r){var a=n[r];t.push(0===r&&e.isFunctionOrConstructorTypeNode(a)&&a.typeParameters?e.createParenthesizedType(a):a)}return e.createNodeArray(t)}},e.parenthesizeConciseBody=function(n){return e.isBlock(n)||!S(n)&&186!==T(n,!1).kind?n:e.setTextRange(e.createParen(n),n)},e.isCommaSequence=S,function(e){e[e.Parentheses=1]=\"Parentheses\",e[e.Assertions=2]=\"Assertions\",e[e.PartiallyEmittedExpressions=4]=\"PartiallyEmittedExpressions\",e[e.All=7]=\"All\"}(e.OuterExpressionKinds||(e.OuterExpressionKinds={})),e.isOuterExpression=L,e.skipOuterExpressions=x,e.skipAssertions=A,e.recreateOuterExpressions=C,e.startOnNewLine=D,e.getExternalHelpersModuleName=k,e.getOrCreateExternalHelpersModuleNameIfNeeded=function(n,t,r,a){if(t.importHelpers&&e.isEffectiveExternalModule(n,t)){var i=k(n);if(i)return i;var o=e.getEmitModuleKind(t),s=(r||t.esModuleInterop&&a)&&o!==e.ModuleKind.System&&o!==e.ModuleKind.ES2015&&o!==e.ModuleKind.ESNext;if(!s){var l=e.getEmitHelpers(n);if(l)for(var c=0,u=l;c<u.length;c++)if(!u[c].scoped){s=!0;break}}if(s){var d=e.getOriginalNode(n,e.isSourceFile),m=e.getOrCreateEmitNode(d);return m.externalHelpersModuleName||(m.externalHelpersModuleName=e.createUniqueName(e.externalHelpersModuleNameText))}}},e.getLocalNameForExternalImport=function(n,t){var r=e.getNamespaceDeclarationNode(n);if(r&&!e.isDefaultImport(n)){var a=r.name;return e.isGeneratedIdentifier(a)?a:e.createIdentifier(e.getSourceTextOfNodeFromSourceFile(t,a)||e.idText(a))}return 247===n.kind&&n.importClause?e.getGeneratedNameForNode(n):253===n.kind&&n.moduleSpecifier?e.getGeneratedNameForNode(n):void 0},e.getExternalModuleNameLiteral=function(n,t,r,a,i){var o=e.getExternalModuleName(n);if(9===o.kind)return function(e,n,t,r){return M(t.getExternalModuleFileFromDeclaration(e),n,r)}(n,r,a,i)||function(n,t){var r=t.renamedDependencies&&t.renamedDependencies.get(n.text);return r&&e.createLiteral(r)}(o,t)||e.getSynthesizedClone(o)},e.tryGetModuleNameFromFile=M,e.getInitializerOfBindingOrAssignmentElement=function n(t){if(e.isDeclarationBindingElement(t))return t.initializer;if(e.isPropertyAssignment(t)){var r=t.initializer;return e.isAssignmentExpression(r,!0)?r.right:void 0}return e.isShorthandPropertyAssignment(t)?t.objectAssignmentInitializer:e.isAssignmentExpression(t,!0)?t.right:e.isSpreadElement(t)?n(t.expression):void 0},e.getTargetOfBindingOrAssignmentElement=O,e.getRestIndicatorOfBindingOrAssignmentElement=function(e){switch(e.kind){case 149:case 184:return e.dotDotDotToken;case 206:case 275:return e}},e.getPropertyNameOfBindingOrAssignmentElement=function(n){switch(n.kind){case 184:if(n.propertyName){var t=n.propertyName;return e.isComputedPropertyName(t)&&I(t.expression)?t.expression:t}break;case 273:if(n.name)return t=n.name,e.isComputedPropertyName(t)&&I(t.expression)?t.expression:t;break;case 275:return n.name}var r=O(n);if(r&&e.isPropertyName(r))return e.isComputedPropertyName(r)&&I(r.expression)?r.expression:r;e.Debug.fail(\"Invalid property name for binding element.\")},e.getElementsOfBindingOrAssignmentPattern=function(e){switch(e.kind){case 182:case 183:case 185:return e.elements;case 186:return e.properties}},e.convertToArrayAssignmentElement=R,e.convertToObjectAssignmentElement=N,e.convertToAssignmentPattern=w,e.convertToObjectAssignmentPattern=P,e.convertToArrayAssignmentPattern=F,e.convertToAssignmentElementTarget=G}(Tn||(Tn={})),function(e){var n=e.or(e.isTypeNode,e.isTypeParameterDeclaration);function t(n,t,r,a){if(void 0===n||void 0===t)return n;e.aggregateTransformFlags(n);var i,o=t(n);return o===n?n:void 0!==o?(i=e.isArray(o)?(a||function(n){return e.Debug.assert(n.length<=1,\"Too many nodes written to output.\"),e.singleOrUndefined(n)})(o):o,e.Debug.assertNode(i,r),e.aggregateTransformFlags(i),i):void 0}function r(n,t,r,a,i){if(void 0===n||void 0===t)return n;var o,s=n.length;(void 0===a||a<0)&&(a=0),(void 0===i||i>s-a)&&(i=s-a),(a>0||i<s)&&(o=e.createNodeArray([],n.hasTrailingComma&&a+i===s));for(var l=0;l<i;l++){var c=n[l+a];e.aggregateTransformFlags(c);var u=void 0!==c?t(c):void 0;if((void 0!==o||void 0===u||u!==c)&&(void 0===o&&(o=e.createNodeArray(n.slice(0,l),n.hasTrailingComma),e.setTextRange(o,n)),u))if(e.isArray(u))for(var d=0,m=u;d<m.length;d++){var p=m[d];e.Debug.assertNode(p,r),e.aggregateTransformFlags(p),o.push(p)}else e.Debug.assertNode(u,r),e.aggregateTransformFlags(u),o.push(u)}return o||n}function a(n,t,a,i,o){a.startLexicalEnvironment(),n=r(n,t,e.isStatement,i),o&&!e.startsWithUseStrict(n)&&(n=e.setTextRange(e.createNodeArray([e.createExpressionStatement(e.createLiteral(\"use strict\"))].concat(n)),n));var s=a.endLexicalEnvironment();return e.setTextRange(e.createNodeArray(e.concatenate(s,n)),n)}function i(n,t,a,i){void 0===i&&(i=r),a.startLexicalEnvironment();var o=i(n,t,e.isParameterDeclaration);return a.suspendLexicalEnvironment(),o}function o(n,r,a){a.resumeLexicalEnvironment();var i=t(n,r,e.isConciseBody),o=a.endLexicalEnvironment();if(e.some(o)){var s=e.convertToFunctionBody(i),l=e.mergeLexicalEnvironment(s.statements,o);return e.updateBlock(s,l)}return i}e.visitNode=t,e.visitNodes=r,e.visitLexicalEnvironment=a,e.visitParameterList=i,e.visitFunctionBody=o,e.visitEachChild=function(s,l,c,u,d){if(void 0===u&&(u=r),void 0!==s){var m=s.kind;if(m>0&&m<=145||176===m)return s;switch(m){case 71:return e.updateIdentifier(s,u(s.typeArguments,l,n));case 146:return e.updateQualifiedName(s,t(s.left,l,e.isEntityName),t(s.right,l,e.isIdentifier));case 147:return e.updateComputedPropertyName(s,t(s.expression,l,e.isExpression));case 148:return e.updateTypeParameterDeclaration(s,t(s.name,l,e.isIdentifier),t(s.constraint,l,e.isTypeNode),t(s.default,l,e.isTypeNode));case 149:return e.updateParameter(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.dotDotDotToken,d,e.isToken),t(s.name,l,e.isBindingName),t(s.questionToken,d,e.isToken),t(s.type,l,e.isTypeNode),t(s.initializer,l,e.isExpression));case 150:return e.updateDecorator(s,t(s.expression,l,e.isExpression));case 151:return e.updatePropertySignature(s,u(s.modifiers,l,e.isToken),t(s.name,l,e.isPropertyName),t(s.questionToken,d,e.isToken),t(s.type,l,e.isTypeNode),t(s.initializer,l,e.isExpression));case 152:return e.updateProperty(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isPropertyName),t(s.questionToken,d,e.isToken),t(s.type,l,e.isTypeNode),t(s.initializer,l,e.isExpression));case 153:return e.updateMethodSignature(s,u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.parameters,l,e.isParameterDeclaration),t(s.type,l,e.isTypeNode),t(s.name,l,e.isPropertyName),t(s.questionToken,d,e.isToken));case 154:return e.updateMethod(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.asteriskToken,d,e.isToken),t(s.name,l,e.isPropertyName),t(s.questionToken,d,e.isToken),u(s.typeParameters,l,e.isTypeParameterDeclaration),i(s.parameters,l,c,u),t(s.type,l,e.isTypeNode),o(s.body,l,c));case 155:return e.updateConstructor(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),i(s.parameters,l,c,u),o(s.body,l,c));case 156:return e.updateGetAccessor(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isPropertyName),i(s.parameters,l,c,u),t(s.type,l,e.isTypeNode),o(s.body,l,c));case 157:return e.updateSetAccessor(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isPropertyName),i(s.parameters,l,c,u),o(s.body,l,c));case 158:return e.updateCallSignature(s,u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.parameters,l,e.isParameterDeclaration),t(s.type,l,e.isTypeNode));case 159:return e.updateConstructSignature(s,u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.parameters,l,e.isParameterDeclaration),t(s.type,l,e.isTypeNode));case 160:return e.updateIndexSignature(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),u(s.parameters,l,e.isParameterDeclaration),t(s.type,l,e.isTypeNode));case 161:return e.updateTypePredicateNode(s,t(s.parameterName,l),t(s.type,l,e.isTypeNode));case 162:return e.updateTypeReferenceNode(s,t(s.typeName,l,e.isEntityName),u(s.typeArguments,l,e.isTypeNode));case 163:return e.updateFunctionTypeNode(s,u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.parameters,l,e.isParameterDeclaration),t(s.type,l,e.isTypeNode));case 164:return e.updateConstructorTypeNode(s,u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.parameters,l,e.isParameterDeclaration),t(s.type,l,e.isTypeNode));case 165:return e.updateTypeQueryNode(s,t(s.exprName,l,e.isEntityName));case 166:return e.updateTypeLiteralNode(s,u(s.members,l,e.isTypeElement));case 167:return e.updateArrayTypeNode(s,t(s.elementType,l,e.isTypeNode));case 168:return e.updateTupleTypeNode(s,u(s.elementTypes,l,e.isTypeNode));case 169:return e.updateOptionalTypeNode(s,t(s.type,l,e.isTypeNode));case 170:return e.updateRestTypeNode(s,t(s.type,l,e.isTypeNode));case 171:return e.updateUnionTypeNode(s,u(s.types,l,e.isTypeNode));case 172:return e.updateIntersectionTypeNode(s,u(s.types,l,e.isTypeNode));case 173:return e.updateConditionalTypeNode(s,t(s.checkType,l,e.isTypeNode),t(s.extendsType,l,e.isTypeNode),t(s.trueType,l,e.isTypeNode),t(s.falseType,l,e.isTypeNode));case 174:return e.updateInferTypeNode(s,t(s.typeParameter,l,e.isTypeParameterDeclaration));case 181:return e.updateImportTypeNode(s,t(s.argument,l,e.isTypeNode),t(s.qualifier,l,e.isEntityName),r(s.typeArguments,l,e.isTypeNode),s.isTypeOf);case 175:return e.updateParenthesizedType(s,t(s.type,l,e.isTypeNode));case 177:return e.updateTypeOperatorNode(s,t(s.type,l,e.isTypeNode));case 178:return e.updateIndexedAccessTypeNode(s,t(s.objectType,l,e.isTypeNode),t(s.indexType,l,e.isTypeNode));case 179:return e.updateMappedTypeNode(s,t(s.readonlyToken,d,e.isToken),t(s.typeParameter,l,e.isTypeParameterDeclaration),t(s.questionToken,d,e.isToken),t(s.type,l,e.isTypeNode));case 180:return e.updateLiteralTypeNode(s,t(s.literal,l,e.isExpression));case 182:return e.updateObjectBindingPattern(s,u(s.elements,l,e.isBindingElement));case 183:return e.updateArrayBindingPattern(s,u(s.elements,l,e.isArrayBindingElement));case 184:return e.updateBindingElement(s,t(s.dotDotDotToken,d,e.isToken),t(s.propertyName,l,e.isPropertyName),t(s.name,l,e.isBindingName),t(s.initializer,l,e.isExpression));case 185:return e.updateArrayLiteral(s,u(s.elements,l,e.isExpression));case 186:return e.updateObjectLiteral(s,u(s.properties,l,e.isObjectLiteralElementLike));case 187:return e.updatePropertyAccess(s,t(s.expression,l,e.isExpression),t(s.name,l,e.isIdentifier));case 188:return e.updateElementAccess(s,t(s.expression,l,e.isExpression),t(s.argumentExpression,l,e.isExpression));case 189:return e.updateCall(s,t(s.expression,l,e.isExpression),u(s.typeArguments,l,e.isTypeNode),u(s.arguments,l,e.isExpression));case 190:return e.updateNew(s,t(s.expression,l,e.isExpression),u(s.typeArguments,l,e.isTypeNode),u(s.arguments,l,e.isExpression));case 191:return e.updateTaggedTemplate(s,t(s.tag,l,e.isExpression),r(s.typeArguments,l,e.isExpression),t(s.template,l,e.isTemplateLiteral));case 192:return e.updateTypeAssertion(s,t(s.type,l,e.isTypeNode),t(s.expression,l,e.isExpression));case 193:return e.updateParen(s,t(s.expression,l,e.isExpression));case 194:return e.updateFunctionExpression(s,u(s.modifiers,l,e.isModifier),t(s.asteriskToken,d,e.isToken),t(s.name,l,e.isIdentifier),u(s.typeParameters,l,e.isTypeParameterDeclaration),i(s.parameters,l,c,u),t(s.type,l,e.isTypeNode),o(s.body,l,c));case 195:return e.updateArrowFunction(s,u(s.modifiers,l,e.isModifier),u(s.typeParameters,l,e.isTypeParameterDeclaration),i(s.parameters,l,c,u),t(s.type,l,e.isTypeNode),t(s.equalsGreaterThanToken,l,e.isToken),o(s.body,l,c));case 196:return e.updateDelete(s,t(s.expression,l,e.isExpression));case 197:return e.updateTypeOf(s,t(s.expression,l,e.isExpression));case 198:return e.updateVoid(s,t(s.expression,l,e.isExpression));case 199:return e.updateAwait(s,t(s.expression,l,e.isExpression));case 200:return e.updatePrefix(s,t(s.operand,l,e.isExpression));case 201:return e.updatePostfix(s,t(s.operand,l,e.isExpression));case 202:return e.updateBinary(s,t(s.left,l,e.isExpression),t(s.right,l,e.isExpression),t(s.operatorToken,l,e.isToken));case 203:return e.updateConditional(s,t(s.condition,l,e.isExpression),t(s.questionToken,l,e.isToken),t(s.whenTrue,l,e.isExpression),t(s.colonToken,l,e.isToken),t(s.whenFalse,l,e.isExpression));case 204:return e.updateTemplateExpression(s,t(s.head,l,e.isTemplateHead),u(s.templateSpans,l,e.isTemplateSpan));case 205:return e.updateYield(s,t(s.asteriskToken,d,e.isToken),t(s.expression,l,e.isExpression));case 206:return e.updateSpread(s,t(s.expression,l,e.isExpression));case 207:return e.updateClassExpression(s,u(s.modifiers,l,e.isModifier),t(s.name,l,e.isIdentifier),u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.heritageClauses,l,e.isHeritageClause),u(s.members,l,e.isClassElement));case 209:return e.updateExpressionWithTypeArguments(s,u(s.typeArguments,l,e.isTypeNode),t(s.expression,l,e.isExpression));case 210:return e.updateAsExpression(s,t(s.expression,l,e.isExpression),t(s.type,l,e.isTypeNode));case 211:return e.updateNonNullExpression(s,t(s.expression,l,e.isExpression));case 212:return e.updateMetaProperty(s,t(s.name,l,e.isIdentifier));case 214:return e.updateTemplateSpan(s,t(s.expression,l,e.isExpression),t(s.literal,l,e.isTemplateMiddleOrTemplateTail));case 216:return e.updateBlock(s,u(s.statements,l,e.isStatement));case 217:return e.updateVariableStatement(s,u(s.modifiers,l,e.isModifier),t(s.declarationList,l,e.isVariableDeclarationList));case 219:return e.updateExpressionStatement(s,t(s.expression,l,e.isExpression));case 220:return e.updateIf(s,t(s.expression,l,e.isExpression),t(s.thenStatement,l,e.isStatement,e.liftToBlock),t(s.elseStatement,l,e.isStatement,e.liftToBlock));case 221:return e.updateDo(s,t(s.statement,l,e.isStatement,e.liftToBlock),t(s.expression,l,e.isExpression));case 222:return e.updateWhile(s,t(s.expression,l,e.isExpression),t(s.statement,l,e.isStatement,e.liftToBlock));case 223:return e.updateFor(s,t(s.initializer,l,e.isForInitializer),t(s.condition,l,e.isExpression),t(s.incrementor,l,e.isExpression),t(s.statement,l,e.isStatement,e.liftToBlock));case 224:return e.updateForIn(s,t(s.initializer,l,e.isForInitializer),t(s.expression,l,e.isExpression),t(s.statement,l,e.isStatement,e.liftToBlock));case 225:return e.updateForOf(s,t(s.awaitModifier,l,e.isToken),t(s.initializer,l,e.isForInitializer),t(s.expression,l,e.isExpression),t(s.statement,l,e.isStatement,e.liftToBlock));case 226:return e.updateContinue(s,t(s.label,l,e.isIdentifier));case 227:return e.updateBreak(s,t(s.label,l,e.isIdentifier));case 228:return e.updateReturn(s,t(s.expression,l,e.isExpression));case 229:return e.updateWith(s,t(s.expression,l,e.isExpression),t(s.statement,l,e.isStatement,e.liftToBlock));case 230:return e.updateSwitch(s,t(s.expression,l,e.isExpression),t(s.caseBlock,l,e.isCaseBlock));case 231:return e.updateLabel(s,t(s.label,l,e.isIdentifier),t(s.statement,l,e.isStatement,e.liftToBlock));case 232:return e.updateThrow(s,t(s.expression,l,e.isExpression));case 233:return e.updateTry(s,t(s.tryBlock,l,e.isBlock),t(s.catchClause,l,e.isCatchClause),t(s.finallyBlock,l,e.isBlock));case 235:return e.updateVariableDeclaration(s,t(s.name,l,e.isBindingName),t(s.type,l,e.isTypeNode),t(s.initializer,l,e.isExpression));case 236:return e.updateVariableDeclarationList(s,u(s.declarations,l,e.isVariableDeclaration));case 237:return e.updateFunctionDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.asteriskToken,d,e.isToken),t(s.name,l,e.isIdentifier),u(s.typeParameters,l,e.isTypeParameterDeclaration),i(s.parameters,l,c,u),t(s.type,l,e.isTypeNode),o(s.body,l,c));case 238:return e.updateClassDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isIdentifier),u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.heritageClauses,l,e.isHeritageClause),u(s.members,l,e.isClassElement));case 239:return e.updateInterfaceDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isIdentifier),u(s.typeParameters,l,e.isTypeParameterDeclaration),u(s.heritageClauses,l,e.isHeritageClause),u(s.members,l,e.isTypeElement));case 240:return e.updateTypeAliasDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isIdentifier),u(s.typeParameters,l,e.isTypeParameterDeclaration),t(s.type,l,e.isTypeNode));case 241:return e.updateEnumDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isIdentifier),u(s.members,l,e.isEnumMember));case 242:return e.updateModuleDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isIdentifier),t(s.body,l,e.isModuleBody));case 243:return e.updateModuleBlock(s,u(s.statements,l,e.isStatement));case 244:return e.updateCaseBlock(s,u(s.clauses,l,e.isCaseOrDefaultClause));case 245:return e.updateNamespaceExportDeclaration(s,t(s.name,l,e.isIdentifier));case 246:return e.updateImportEqualsDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.name,l,e.isIdentifier),t(s.moduleReference,l,e.isModuleReference));case 247:return e.updateImportDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.importClause,l,e.isImportClause),t(s.moduleSpecifier,l,e.isExpression));case 248:return e.updateImportClause(s,t(s.name,l,e.isIdentifier),t(s.namedBindings,l,e.isNamedImportBindings));case 249:return e.updateNamespaceImport(s,t(s.name,l,e.isIdentifier));case 250:return e.updateNamedImports(s,u(s.elements,l,e.isImportSpecifier));case 251:return e.updateImportSpecifier(s,t(s.propertyName,l,e.isIdentifier),t(s.name,l,e.isIdentifier));case 252:return e.updateExportAssignment(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.expression,l,e.isExpression));case 253:return e.updateExportDeclaration(s,u(s.decorators,l,e.isDecorator),u(s.modifiers,l,e.isModifier),t(s.exportClause,l,e.isNamedExports),t(s.moduleSpecifier,l,e.isExpression));case 254:return e.updateNamedExports(s,u(s.elements,l,e.isExportSpecifier));case 255:return e.updateExportSpecifier(s,t(s.propertyName,l,e.isIdentifier),t(s.name,l,e.isIdentifier));case 257:return e.updateExternalModuleReference(s,t(s.expression,l,e.isExpression));case 258:return e.updateJsxElement(s,t(s.openingElement,l,e.isJsxOpeningElement),u(s.children,l,e.isJsxChild),t(s.closingElement,l,e.isJsxClosingElement));case 259:return e.updateJsxSelfClosingElement(s,t(s.tagName,l,e.isJsxTagNameExpression),u(s.typeArguments,l,e.isTypeNode),t(s.attributes,l,e.isJsxAttributes));case 260:return e.updateJsxOpeningElement(s,t(s.tagName,l,e.isJsxTagNameExpression),u(s.typeArguments,l,e.isTypeNode),t(s.attributes,l,e.isJsxAttributes));case 261:return e.updateJsxClosingElement(s,t(s.tagName,l,e.isJsxTagNameExpression));case 262:return e.updateJsxFragment(s,t(s.openingFragment,l,e.isJsxOpeningFragment),u(s.children,l,e.isJsxChild),t(s.closingFragment,l,e.isJsxClosingFragment));case 265:return e.updateJsxAttribute(s,t(s.name,l,e.isIdentifier),t(s.initializer,l,e.isStringLiteralOrJsxExpression));case 266:return e.updateJsxAttributes(s,u(s.properties,l,e.isJsxAttributeLike));case 267:return e.updateJsxSpreadAttribute(s,t(s.expression,l,e.isExpression));case 268:return e.updateJsxExpression(s,t(s.expression,l,e.isExpression));case 269:return e.updateCaseClause(s,t(s.expression,l,e.isExpression),u(s.statements,l,e.isStatement));case 270:return e.updateDefaultClause(s,u(s.statements,l,e.isStatement));case 271:return e.updateHeritageClause(s,u(s.types,l,e.isExpressionWithTypeArguments));case 272:return e.updateCatchClause(s,t(s.variableDeclaration,l,e.isVariableDeclaration),t(s.block,l,e.isBlock));case 273:return e.updatePropertyAssignment(s,t(s.name,l,e.isPropertyName),t(s.initializer,l,e.isExpression));case 274:return e.updateShorthandPropertyAssignment(s,t(s.name,l,e.isIdentifier),t(s.objectAssignmentInitializer,l,e.isExpression));case 275:return e.updateSpreadAssignment(s,t(s.expression,l,e.isExpression));case 276:return e.updateEnumMember(s,t(s.name,l,e.isPropertyName),t(s.initializer,l,e.isExpression));case 277:return e.updateSourceFileNode(s,a(s.statements,l,c));case 306:return e.updatePartiallyEmittedExpression(s,t(s.expression,l,e.isExpression));case 307:return e.updateCommaList(s,u(s.elements,l,e.isExpression));default:return s}}}}(Tn||(Tn={})),function(e){function n(e,n,t){return e?n(t,e):t}function t(e,n,t){return e?n(t,e):t}function r(r,a,i,o){if(void 0===r)return a;var s=o?t:e.reduceLeft,l=o||i,c=r.kind;if(c>0&&c<=145)return a;if(c>=161&&c<=180)return a;var u=a;switch(r.kind){case 215:case 218:case 208:case 234:case 305:break;case 146:u=n(r.left,i,u),u=n(r.right,i,u);break;case 147:u=n(r.expression,i,u);break;case 149:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=n(r.type,i,u),u=n(r.initializer,i,u);break;case 150:u=n(r.expression,i,u);break;case 151:u=s(r.modifiers,l,u),u=n(r.name,i,u),u=n(r.questionToken,i,u),u=n(r.type,i,u),u=n(r.initializer,i,u);break;case 152:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=n(r.type,i,u),u=n(r.initializer,i,u);break;case 154:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.typeParameters,l,u),u=s(r.parameters,l,u),u=n(r.type,i,u),u=n(r.body,i,u);break;case 155:u=s(r.modifiers,l,u),u=s(r.parameters,l,u),u=n(r.body,i,u);break;case 156:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.parameters,l,u),u=n(r.type,i,u),u=n(r.body,i,u);break;case 157:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.parameters,l,u),u=n(r.body,i,u);break;case 182:case 183:u=s(r.elements,l,u);break;case 184:u=n(r.propertyName,i,u),u=n(r.name,i,u),u=n(r.initializer,i,u);break;case 185:u=s(r.elements,l,u);break;case 186:u=s(r.properties,l,u);break;case 187:u=n(r.expression,i,u),u=n(r.name,i,u);break;case 188:u=n(r.expression,i,u),u=n(r.argumentExpression,i,u);break;case 189:case 190:u=n(r.expression,i,u),u=s(r.typeArguments,l,u),u=s(r.arguments,l,u);break;case 191:u=n(r.tag,i,u),u=n(r.template,i,u);break;case 192:u=n(r.type,i,u),u=n(r.expression,i,u);break;case 194:u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.typeParameters,l,u),u=s(r.parameters,l,u),u=n(r.type,i,u),u=n(r.body,i,u);break;case 195:u=s(r.modifiers,l,u),u=s(r.typeParameters,l,u),u=s(r.parameters,l,u),u=n(r.type,i,u),u=n(r.body,i,u);break;case 193:case 196:case 197:case 198:case 199:case 205:case 206:case 211:u=n(r.expression,i,u);break;case 200:case 201:u=n(r.operand,i,u);break;case 202:u=n(r.left,i,u),u=n(r.right,i,u);break;case 203:u=n(r.condition,i,u),u=n(r.whenTrue,i,u),u=n(r.whenFalse,i,u);break;case 204:u=n(r.head,i,u),u=s(r.templateSpans,l,u);break;case 207:u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.typeParameters,l,u),u=s(r.heritageClauses,l,u),u=s(r.members,l,u);break;case 209:u=n(r.expression,i,u),u=s(r.typeArguments,l,u);break;case 210:u=n(r.expression,i,u),u=n(r.type,i,u);break;case 214:u=n(r.expression,i,u),u=n(r.literal,i,u);break;case 216:u=s(r.statements,l,u);break;case 217:u=s(r.modifiers,l,u),u=n(r.declarationList,i,u);break;case 219:u=n(r.expression,i,u);break;case 220:u=n(r.expression,i,u),u=n(r.thenStatement,i,u),u=n(r.elseStatement,i,u);break;case 221:u=n(r.statement,i,u),u=n(r.expression,i,u);break;case 222:case 229:u=n(r.expression,i,u),u=n(r.statement,i,u);break;case 223:u=n(r.initializer,i,u),u=n(r.condition,i,u),u=n(r.incrementor,i,u),u=n(r.statement,i,u);break;case 224:case 225:u=n(r.initializer,i,u),u=n(r.expression,i,u),u=n(r.statement,i,u);break;case 228:case 232:u=n(r.expression,i,u);break;case 230:u=n(r.expression,i,u),u=n(r.caseBlock,i,u);break;case 231:u=n(r.label,i,u),u=n(r.statement,i,u);break;case 233:u=n(r.tryBlock,i,u),u=n(r.catchClause,i,u),u=n(r.finallyBlock,i,u);break;case 235:u=n(r.name,i,u),u=n(r.type,i,u),u=n(r.initializer,i,u);break;case 236:u=s(r.declarations,l,u);break;case 237:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.typeParameters,l,u),u=s(r.parameters,l,u),u=n(r.type,i,u),u=n(r.body,i,u);break;case 238:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.typeParameters,l,u),u=s(r.heritageClauses,l,u),u=s(r.members,l,u);break;case 241:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=s(r.members,l,u);break;case 242:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=n(r.body,i,u);break;case 243:u=s(r.statements,l,u);break;case 244:u=s(r.clauses,l,u);break;case 246:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.name,i,u),u=n(r.moduleReference,i,u);break;case 247:u=s(r.decorators,l,u),u=s(r.modifiers,l,u),u=n(r.importClause,i,u),u=n(r.moduleSpecifier,i,u);break;case 248:u=n(r.name,i,u),u=n(r.namedBindings,i,u);break;case 249:u=n(r.name,i,u);break;case 250:case 254:u=s(r.elements,l,u);break;case 251:case 255:u=n(r.propertyName,i,u),u=n(r.name,i,u);break;case 252:u=e.reduceLeft(r.decorators,i,u),u=e.reduceLeft(r.modifiers,i,u),u=n(r.expression,i,u);break;case 253:u=e.reduceLeft(r.decorators,i,u),u=e.reduceLeft(r.modifiers,i,u),u=n(r.exportClause,i,u),u=n(r.moduleSpecifier,i,u);break;case 257:u=n(r.expression,i,u);break;case 258:u=n(r.openingElement,i,u),u=e.reduceLeft(r.children,i,u),u=n(r.closingElement,i,u);break;case 262:u=n(r.openingFragment,i,u),u=e.reduceLeft(r.children,i,u),u=n(r.closingFragment,i,u);break;case 259:case 260:u=n(r.tagName,i,u),u=n(r.attributes,i,u);break;case 266:u=s(r.properties,l,u);break;case 261:u=n(r.tagName,i,u);break;case 265:u=n(r.name,i,u),u=n(r.initializer,i,u);break;case 267:case 268:u=n(r.expression,i,u);break;case 269:u=n(r.expression,i,u);case 270:u=s(r.statements,l,u);break;case 271:u=s(r.types,l,u);break;case 272:u=n(r.variableDeclaration,i,u),u=n(r.block,i,u);break;case 273:u=n(r.name,i,u),u=n(r.initializer,i,u);break;case 274:u=n(r.name,i,u),u=n(r.objectAssignmentInitializer,i,u);break;case 275:u=n(r.expression,i,u);break;case 276:u=n(r.name,i,u),u=n(r.initializer,i,u);break;case 277:u=s(r.statements,l,u);break;case 306:u=n(r.expression,i,u);break;case 307:u=s(r.elements,l,u)}return u}function a(n){if(void 0===n)return 0;if(536870912&n.transformFlags)return n.transformFlags&~e.getTransformFlagsSubtreeExclusions(n.kind);var t=function(n){if(e.hasModifier(n,2)||e.isTypeNode(n)&&209!==n.kind)return 0;return r(n,0,i,o)}(n);return e.computeTransformFlagsForNode(n,t)}function i(e,n){return e|a(n)}function o(e,n){return e|function(e){if(void 0===e)return 0;for(var n=0,t=0,r=0,i=e;r<i.length;r++){var o=i[r];n|=a(o),t|=-536870913&o.transformFlags}return e.transformFlags=536870912|t,n}(n)}var s;e.reduceEachChild=r,e.mergeLexicalEnvironment=function(n,t){return e.some(t)?e.isNodeArray(n)?e.setTextRange(e.createNodeArray(e.addStatementsAfterPrologue(n.slice(),t)),n):e.addStatementsAfterPrologue(n,t):n},e.liftToBlock=function(n){return s.assert(e.every(n,e.isStatement),\"Cannot lift nodes to a Block.\"),e.singleOrUndefined(n)||e.createBlock(n)},e.aggregateTransformFlags=function(e){return a(e),e},function(n){var t=!1;n.failBadSyntaxKind=function t(r,a){return n.fail((a||\"Unexpected node.\")+\"\\r\\nNode \"+e.formatSyntaxKind(r.kind)+\" was unexpected.\",t)},n.assertEachNode=n.shouldAssert(1)?function(t,r,a){return n.assert(void 0===r||e.every(t,r),a||\"Unexpected node.\",function(){return\"Node array did not pass test '\"+n.getFunctionName(r)+\"'.\"},n.assertEachNode)}:e.noop,n.assertNode=n.shouldAssert(1)?function(t,r,a){return n.assert(void 0===r||r(t),a||\"Unexpected node.\",function(){return\"Node \"+e.formatSyntaxKind(t.kind)+\" did not pass test '\"+n.getFunctionName(r)+\"'.\"},n.assertNode)}:e.noop,n.assertOptionalNode=n.shouldAssert(1)?function(t,r,a){return n.assert(void 0===r||void 0===t||r(t),a||\"Unexpected node.\",function(){return\"Node \"+e.formatSyntaxKind(t.kind)+\" did not pass test '\"+n.getFunctionName(r)+\"'.\"},n.assertOptionalNode)}:e.noop,n.assertOptionalToken=n.shouldAssert(1)?function(t,r,a){return n.assert(void 0===r||void 0===t||t.kind===r,a||\"Unexpected node.\",function(){return\"Node \"+e.formatSyntaxKind(t.kind)+\" was not a '\"+e.formatSyntaxKind(r)+\"' token.\"},n.assertOptionalToken)}:e.noop,n.assertMissingNode=n.shouldAssert(1)?function(t,r){return n.assert(void 0===t,r||\"Unexpected node.\",function(){return\"Node \"+e.formatSyntaxKind(t.kind)+\" was unexpected'.\"},n.assertMissingNode)}:e.noop,n.enableDebugInfo=function(){if(!t){Object.defineProperties(e.objectAllocator.getSymbolConstructor().prototype,{__debugFlags:{get:function(){return e.formatSymbolFlags(this.flags)}}}),Object.defineProperties(e.objectAllocator.getTypeConstructor().prototype,{__debugFlags:{get:function(){return e.formatTypeFlags(this.flags)}},__debugObjectFlags:{get:function(){return 131072&this.flags?e.formatObjectFlags(this.objectFlags):\"\"}},__debugTypeToString:{value:function(){return this.checker.typeToString(this)}}});for(var n=0,r=[e.objectAllocator.getNodeConstructor(),e.objectAllocator.getIdentifierConstructor(),e.objectAllocator.getTokenConstructor(),e.objectAllocator.getSourceFileConstructor()];n<r.length;n++){var a=r[n];a.prototype.hasOwnProperty(\"__debugKind\")||Object.defineProperties(a.prototype,{__debugKind:{get:function(){return e.formatSyntaxKind(this.kind)}},__debugModifierFlags:{get:function(){return e.formatModifierFlags(e.getModifierFlagsNoCache(this))}},__debugTransformFlags:{get:function(){return e.formatTransformFlags(this.transformFlags)}},__debugEmitFlags:{get:function(){return e.formatEmitFlags(e.getEmitFlags(this))}},__debugGetText:{value:function(n){if(e.nodeIsSynthesized(this))return\"\";var t=e.getParseTreeNode(this),r=t&&e.getSourceFileOfNode(t);return r?e.getSourceTextOfNodeFromSourceFile(r,t,n):\"\"}}})}t=!0}}}(s=e.Debug||(e.Debug={}))}(Tn||(Tn={})),function(e){e.createSourceFileLikeCache=function(n){var t=e.createMap();return{get:function(r){if(t.has(r))return t.get(r);if(n.fileExists&&n.readFile&&n.fileExists(r)){var a={text:n.readFile(r),lineMap:void 0,getLineAndCharacterOfPosition:function(n){return e.computeLineAndCharacterOfPosition(e.getLineStarts(this),n)}};return t.set(r,a),a}}}}}(Tn||(Tn={})),function(e){!function(n){function t(e){var n={encodedText:e.mappings,currentNameIndex:void 0,sourceMapNamesLength:e.names?e.names.length:void 0,currentEmittedColumn:0,currentEmittedLine:0,currentSourceColumn:0,currentSourceLine:0,currentSourceIndex:0,decodingIndex:0};function t(){return{emittedColumn:n.currentEmittedColumn,emittedLine:n.currentEmittedLine,sourceColumn:n.currentSourceColumn,sourceIndex:n.currentSourceIndex,sourceLine:n.currentSourceLine,nameIndex:n.currentNameIndex}}return{get decodingIndex(){return n.decodingIndex},get error(){return n.error},get lastSpan(){return t()},next:function(){return function(e){return e.decodingIndex===e.encodedText.length}(n)||n.error?{done:!0,value:void 0}:function(e){for(;e.decodingIndex<e.encodedText.length;){var n=e.encodedText.charCodeAt(e.decodingIndex);if(59!==n){if(44!==n)return e.currentEmittedColumn+=a(),!t(e.currentEmittedColumn<0,\"Invalid emittedColumn found\")&&(!t(r(e.encodedText,e.decodingIndex),\"Unsupported Error Format: No entries after emitted column\")&&(e.currentSourceIndex+=a(),!t(e.currentSourceIndex<0,\"Invalid sourceIndex found\")&&(!t(r(e.encodedText,e.decodingIndex),\"Unsupported Error Format: No entries after sourceIndex\")&&(e.currentSourceLine+=a(),!t(e.currentSourceLine<0,\"Invalid sourceLine found\")&&(!t(r(e.encodedText,e.decodingIndex),\"Unsupported Error Format: No entries after emitted Line\")&&(e.currentSourceColumn+=a(),!t(e.currentSourceColumn<0,\"Invalid sourceLine found\")&&(r(e.encodedText,e.decodingIndex)||(void 0===e.currentNameIndex&&(e.currentNameIndex=0),e.currentNameIndex+=a()),!t(!r(e.encodedText,e.decodingIndex),\"Unsupported Error Format: There are more entries after \"+(void 0===e.currentNameIndex?\"sourceColumn\":\"nameIndex\")))))))));e.decodingIndex++}else e.currentEmittedLine++,e.currentEmittedColumn=0,e.decodingIndex++}return t(!0,\"No encoded entry found\"),!1;function t(n,t){return!!e.error||(n&&(e.error=t),n)}function a(){for(var n,r=!0,a=0,i=0;r;e.decodingIndex++){if(t(e.decodingIndex>=e.encodedText.length,\"Error in decoding base64VLQFormatDecode, past the mapping string\"))return;var o=(n=e.encodedText.charAt(e.decodingIndex),\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/\".indexOf(n));r=0!=(32&o),i|=(31&o)<<a,a+=5}return 0==(1&i)?i>>=1:i=-(i>>=1),i}}(n)?{done:!1,value:t()}:{done:!0,value:void 0}}}}function r(e,n){return n===e.length||44===e.charCodeAt(n)||59===e.charCodeAt(n)}n.identitySourceMapper={getOriginalPosition:e.identity,getGeneratedPosition:e.identity},n.decode=function(n,r,a,i,o){void 0===o&&(o=e.createSourceFileLikeCache(n));var s,l,c,u=e.getDirectoryPath(r),d=a.sourceRoot?e.getNormalizedAbsolutePath(a.sourceRoot,u):u;return{getOriginalPosition:function(t){var r=l||(l=p().slice().sort(g));if(!e.length(r))return t;var a=e.binarySearch(r,{emittedPosition:t.position},e.identity,g);return a<0&&r.length>0&&(a=~a),{fileName:e.toPath(r[a].sourcePath,d,n.getCanonicalFileName),position:r[a].sourcePosition}},getGeneratedPosition:function(t){var r=c||(c=p().slice().sort(f));if(!e.length(r))return t;var i=e.binarySearch(r,{sourcePath:t.fileName,sourcePosition:t.position},e.identity,f);return i<0&&r.length>0&&(i=~i),r[i]&&0===e.comparePaths(t.fileName,r[i].sourcePath,d)?{fileName:e.toPath(a.file,d,n.getCanonicalFileName),position:r[i].emittedPosition}:t}};function m(t,r,a,s){var l=function(t,r){var a=e.toPath(t,r,n.getCanonicalFileName),s=i&&i.getSourceFile(a);return s&&s.resolvedPath===a?s:o.get(a)}(t,r);return l?e.getPositionOfLineAndCharacter(l,a,s):-1}function p(){return s||(s=function(n,r,a){var i=t(n),o=e.arrayFrom(i,r);return i.error?(a&&a.log&&a.log(\"Encountered error while decoding sourcemap: \"+i.error),[]):o}(a,_,n))}function f(n,t){return e.comparePaths(n.sourcePath,t.sourcePath,d)||e.compareValues(n.sourcePosition,t.sourcePosition)}function g(n,t){return e.compareValues(n.emittedPosition,t.emittedPosition)}function _(e){var n=a.sources[e.sourceIndex];return{emittedPosition:m(a.file,u,e.emittedLine,e.emittedColumn),sourcePosition:m(n,d,e.sourceLine,e.sourceColumn),sourcePath:n}}},n.decodeMappings=t}(e.sourcemaps||(e.sourcemaps={}))}(Tn||(Tn={})),function(e){function n(n){return(n=e.getOriginalNode(n))?e.getNodeId(n):0}function t(e){return void 0!==e.propertyName&&\"default\"===e.propertyName.escapedText}function r(n){if(e.getNamespaceDeclarationNode(n))return!0;var r=n.importClause&&n.importClause.namedBindings;if(!r)return!1;if(!e.isNamedImports(r))return!1;for(var a=0,i=0,o=r.elements;i<o.length;i++){t(o[i])&&a++}return a>0&&a!==r.elements.length||!!(r.elements.length-a)&&e.isDefaultImport(n)}function a(n){return!r(n)&&(e.isDefaultImport(n)||!!n.importClause&&e.isNamedImports(n.importClause.namedBindings)&&function(n){return!!n&&!!e.isNamedImports(n)&&e.some(n.elements,t)}(n.importClause.namedBindings))}function i(n,t,r){if(e.isBindingPattern(n.name))for(var a=0,o=n.name.elements;a<o.length;a++){var s=o[a];e.isOmittedExpression(s)||(r=i(s,t,r))}else if(!e.isGeneratedIdentifier(n.name)){var l=e.idText(n.name);t.get(l)||(t.set(l,!0),r=e.append(r,n.name))}return r}function o(e,n,t){var r=e[n];return r?r.push(t):e[n]=r=[t],r}e.getOriginalNodeId=n,e.chainBundle=function(n){return function(t){return 277===t.kind?n(t):function(t){return e.createBundle(e.map(t.sourceFiles,n),t.prepends)}(t)}},e.getImportNeedsImportStarHelper=r,e.getImportNeedsImportDefaultHelper=a,e.collectExternalModuleInfo=function(t,s,l){for(var c,u,d=[],m=e.createMultiMap(),p=[],f=e.createMap(),g=!1,_=!1,v=!1,y=0,h=t.statements;y<h.length;y++){var b=h[y];switch(b.kind){case 247:d.push(b),v=v||r(b)||a(b);break;case 246:257===b.moduleReference.kind&&d.push(b);break;case 253:if(b.moduleSpecifier)b.exportClause?d.push(b):(d.push(b),_=!0);else for(var E=0,T=b.exportClause.elements;E<T.length;E++){var S=T[E];if(!f.get(e.idText(S.name))){var L=S.propertyName||S.name;m.add(e.idText(L),S),(C=s.getReferencedImportDeclaration(L)||s.getReferencedValueDeclaration(L))&&o(p,n(C),S.name),f.set(e.idText(S.name),!0),c=e.append(c,S.name)}}break;case 252:b.isExportEquals&&!u&&(u=b);break;case 217:if(e.hasModifier(b,1))for(var x=0,A=b.declarationList.declarations;x<A.length;x++){var C;c=i(C=A[x],f,c)}break;case 237:e.hasModifier(b,1)&&(e.hasModifier(b,512)?g||(o(p,n(b),e.getDeclarationName(b)),g=!0):(L=b.name,f.get(e.idText(L))||(o(p,n(b),L),f.set(e.idText(L),!0),c=e.append(c,L))));break;case 238:e.hasModifier(b,1)&&(e.hasModifier(b,512)?g||(o(p,n(b),e.getDeclarationName(b)),g=!0):(L=b.name)&&!f.get(e.idText(L))&&(o(p,n(b),L),f.set(e.idText(L),!0),c=e.append(c,L)))}}var D=e.getOrCreateExternalHelpersModuleNameIfNeeded(t,l,_,v),k=D&&e.createImportDeclaration(void 0,void 0,e.createImportClause(void 0,e.createNamespaceImport(D)),e.createLiteral(e.externalHelpersModuleNameText));return k&&(e.addEmitFlags(k,67108864),d.unshift(k)),{externalImports:d,exportSpecifiers:m,exportEquals:u,hasExportStarsToExportValues:_,exportedBindings:p,exportedNames:c,externalHelpersImportDeclaration:k}},e.isSimpleCopiableExpression=function(n){return e.isStringLiteralLike(n)||8===n.kind||e.isKeyword(n.kind)||e.isIdentifier(n)},e.helperString=function(e){for(var n=[],t=1;t<arguments.length;t++)n[t-1]=arguments[t];return function(t){for(var r=\"\",a=0;a<n.length;a++)r+=e[a],r+=t(n[a]);return r+=e[e.length-1]}}}(Tn||(Tn={})),function(e){function n(t,r){var a=e.getTargetOfBindingOrAssignmentElement(t);return e.isBindingOrAssignmentPattern(a)?function(t,r){for(var a=e.getElementsOfBindingOrAssignmentPattern(t),i=0,o=a;i<o.length;i++){var s=o[i];if(n(s,r))return!0}return!1}(a,r):!!e.isIdentifier(a)&&a.escapedText===r}function t(n,i,o,s,l){if(!l){var c=e.visitNode(e.getInitializerOfBindingOrAssignmentElement(i),n.visitor,e.isExpression);c?o=o?function(n,t,r,i){return t=a(n,t,!0,i),e.createConditional(e.createTypeCheck(t,\"undefined\"),r,t)}(n,o,c,s):c:o||(o=e.createVoidZero())}var u=e.getTargetOfBindingOrAssignmentElement(i);e.isObjectBindingOrAssignmentPattern(u)?function(n,i,o,s,l){var c,u,d=e.getElementsOfBindingOrAssignmentPattern(o),p=d.length;if(1!==p){var f=!e.isDeclarationBindingElement(i)||0!==p;s=a(n,s,f,l)}for(var g=0;g<p;g++){var _=d[g];if(e.getRestIndicatorOfBindingOrAssignmentElement(_)){if(g===p-1){c&&(n.emitBindingOrAssignment(n.createObjectBindingOrAssignmentPattern(c),s,l,o),c=void 0);var v=m(n.context,s,d,u,o);t(n,_,v,_)}}else{var y=e.getPropertyNameOfBindingOrAssignmentElement(_);if(!(n.level>=1)||1572864&_.transformFlags||1572864&e.getTargetOfBindingOrAssignmentElement(_).transformFlags||e.isComputedPropertyName(y)){c&&(n.emitBindingOrAssignment(n.createObjectBindingOrAssignmentPattern(c),s,l,o),c=void 0);var v=r(n,s,y);e.isComputedPropertyName(y)&&(u=e.append(u,v.argumentExpression)),t(n,_,v,_)}else c=e.append(c,_)}}c&&n.emitBindingOrAssignment(n.createObjectBindingOrAssignmentPattern(c),s,l,o)}(n,i,u,o,s):e.isArrayBindingOrAssignmentPattern(u)?function(n,r,i,o,s){var l,c,u=e.getElementsOfBindingOrAssignmentPattern(i),d=u.length;if(n.level<1&&n.downlevelIteration)o=a(n,e.createReadHelper(n.context,o,d>0&&e.getRestIndicatorOfBindingOrAssignmentElement(u[d-1])?void 0:d,s),!1,s);else if(1!==d&&(n.level<1||0===d)||e.every(u,e.isOmittedExpression)){var m=!e.isDeclarationBindingElement(r)||0!==d;o=a(n,o,m,s)}for(var p=0;p<d;p++){var f=u[p];if(n.level>=1)if(1048576&f.transformFlags){var g=e.createTempVariable(void 0);n.hoistTempVariables&&n.context.hoistVariableDeclaration(g),c=e.append(c,[g,f]),l=e.append(l,n.createArrayBindingOrAssignmentElement(g))}else l=e.append(l,f);else{if(e.isOmittedExpression(f))continue;if(e.getRestIndicatorOfBindingOrAssignmentElement(f)){if(p===d-1){var _=e.createArraySlice(o,p);t(n,f,_,f)}}else{var _=e.createElementAccess(o,p);t(n,f,_,f)}}}l&&n.emitBindingOrAssignment(n.createArrayBindingOrAssignmentPattern(l),o,s,i);if(c)for(var v=0,y=c;v<y.length;v++){var h=y[v],b=h[0],f=h[1];t(n,f,b,f)}}(n,i,u,o,s):n.emitBindingOrAssignment(u,o,s,i)}function r(n,t,r){if(e.isComputedPropertyName(r)){var i=a(n,e.visitNode(r.expression,n.visitor),!1,r);return e.createElementAccess(t,i)}if(e.isStringOrNumericLiteralLike(r))return(i=e.getSynthesizedClone(r)).text=i.text,e.createElementAccess(t,i);var o=e.createIdentifier(e.idText(r));return e.createPropertyAccess(t,o)}function a(n,t,r,a){if(e.isIdentifier(t)&&r)return t;var i=e.createTempVariable(void 0);return n.hoistTempVariables?(n.context.hoistVariableDeclaration(i),n.emitExpression(e.setTextRange(e.createAssignment(i,t),a))):n.emitBindingOrAssignment(i,t,a,void 0),i}function i(n){return e.Debug.assertEachNode(n,e.isArrayBindingElement),e.createArrayBindingPattern(n)}function o(n){return e.createArrayLiteral(e.map(n,e.convertToArrayAssignmentElement))}function s(n){return e.Debug.assertEachNode(n,e.isBindingElement),e.createObjectBindingPattern(n)}function l(n){return e.createObjectLiteral(e.map(n,e.convertToObjectAssignmentElement))}function c(n){return e.createBindingElement(void 0,void 0,n)}function u(e){return e}!function(e){e[e.All=0]=\"All\",e[e.ObjectRest=1]=\"ObjectRest\"}(e.FlattenLevel||(e.FlattenLevel={})),e.flattenDestructuringAssignment=function(r,i,s,c,d,m){var p,f,g=r;if(e.isDestructuringAssignment(r))for(p=r.right;e.isEmptyArrayLiteral(r.left)||e.isEmptyObjectLiteral(r.left);){if(!e.isDestructuringAssignment(p))return e.visitNode(p,i,e.isExpression);g=r=p,p=r.right}var _={context:s,level:c,downlevelIteration:!!s.getCompilerOptions().downlevelIteration,hoistTempVariables:!0,emitExpression:v,emitBindingOrAssignment:function(n,t,r,a){e.Debug.assertNode(n,m?e.isIdentifier:e.isExpression);var o=m?m(n,t,r):e.setTextRange(e.createAssignment(e.visitNode(n,i,e.isExpression),t),r);o.original=a,v(o)},createArrayBindingOrAssignmentPattern:o,createObjectBindingOrAssignmentPattern:l,createArrayBindingOrAssignmentElement:u,visitor:i};if(p&&(p=e.visitNode(p,i,e.isExpression),e.isIdentifier(p)&&n(r,p.escapedText)?p=a(_,p,!1,g):d?p=a(_,p,!0,g):e.nodeIsSynthesized(r)&&(g=p)),t(_,r,p,g,e.isDestructuringAssignment(r)),p&&d){if(!e.some(f))return p;f.push(p)}return e.aggregateTransformFlags(e.inlineExpressions(f))||e.createOmittedExpression();function v(n){e.setEmitFlags(n,64),e.aggregateTransformFlags(n),f=e.append(f,n)}},e.flattenDestructuringBinding=function(r,o,l,u,d,m,p){var f;void 0===m&&(m=!1);var g=[],_=[],v={context:l,level:u,downlevelIteration:!!l.getCompilerOptions().downlevelIteration,hoistTempVariables:m,emitExpression:function(n){f=e.append(f,n)},emitBindingOrAssignment:M,createArrayBindingOrAssignmentPattern:i,createObjectBindingOrAssignmentPattern:s,createArrayBindingOrAssignmentElement:c,visitor:o};if(e.isVariableDeclaration(r)){var y=e.getInitializerOfBindingOrAssignmentElement(r);y&&e.isIdentifier(y)&&n(r,y.escapedText)&&(y=a(v,y,!1,y),r=e.updateVariableDeclaration(r,r.name,r.type,y))}if(t(v,r,d,r,p),f){var h=e.createTempVariable(void 0);if(m){var b=e.inlineExpressions(f);f=void 0,M(h,b,void 0,void 0)}else{l.hoistVariableDeclaration(h);var E=e.last(g);E.pendingExpressions=e.append(E.pendingExpressions,e.createAssignment(h,E.value)),e.addRange(E.pendingExpressions,f),E.value=h}}for(var T=0,S=g;T<S.length;T++){var L=S[T],x=L.pendingExpressions,A=L.name,C=(b=L.value,L.location),D=L.original,k=e.createVariableDeclaration(A,void 0,x?e.inlineExpressions(e.append(x,b)):b);k.original=D,e.setTextRange(k,C),e.isIdentifier(A)&&e.setEmitFlags(k,64),e.aggregateTransformFlags(k),_.push(k)}return _;function M(n,t,r,a){e.Debug.assertNode(n,e.isBindingName),f&&(t=e.inlineExpressions(e.append(f,t)),f=void 0),g.push({pendingExpressions:f,name:n,value:t,location:r,original:a})}};var d={name:\"typescript:rest\",scoped:!1,text:'\\n            var __rest = (this && this.__rest) || function (s, e) {\\n                var t = {};\\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)\\n                    t[p] = s[p];\\n                if (s != null && typeof Object.getOwnPropertySymbols === \"function\")\\n                    for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)\\n                        t[p[i]] = s[p[i]];\\n                return t;\\n            };'};function m(n,t,r,a,i){n.requestEmitHelper(d);for(var o=[],s=0,l=0;l<r.length-1;l++){var c=e.getPropertyNameOfBindingOrAssignmentElement(r[l]);if(c)if(e.isComputedPropertyName(c)){var u=a[s];s++,o.push(e.createConditional(e.createTypeCheck(u,\"symbol\"),u,e.createAdd(u,e.createLiteral(\"\"))))}else o.push(e.createLiteral(c))}return e.createCall(e.getHelperName(\"__rest\"),void 0,[t,e.setTextRange(e.createArrayLiteral(o),i)])}}(Tn||(Tn={})),function(e){var n,t;function r(n,t,r,i,o,s){var l=[];return l.push(e.createArrayLiteral(t,!0)),l.push(r),i&&(l.push(i),o&&l.push(o)),n.requestEmitHelper(a),e.setTextRange(e.createCall(e.getHelperName(\"__decorate\"),void 0,l),s)}!function(e){e[e.ClassAliases=1]=\"ClassAliases\",e[e.NamespaceExports=2]=\"NamespaceExports\",e[e.NonQualifiedEnumMembers=8]=\"NonQualifiedEnumMembers\"}(n||(n={})),function(e){e[e.None=0]=\"None\",e[e.HasStaticInitializedProperties=1]=\"HasStaticInitializedProperties\",e[e.HasConstructorDecorators=2]=\"HasConstructorDecorators\",e[e.HasMemberDecorators=4]=\"HasMemberDecorators\",e[e.IsExportOfNamespace=8]=\"IsExportOfNamespace\",e[e.IsNamedExternalExport=16]=\"IsNamedExternalExport\",e[e.IsDefaultExternalExport=32]=\"IsDefaultExternalExport\",e[e.IsDerivedClass=64]=\"IsDerivedClass\",e[e.UseImmediatelyInvokedFunctionExpression=128]=\"UseImmediatelyInvokedFunctionExpression\",e[e.HasAnyDecorators=6]=\"HasAnyDecorators\",e[e.NeedsName=5]=\"NeedsName\",e[e.MayNeedImmediatelyInvokedFunctionExpression=7]=\"MayNeedImmediatelyInvokedFunctionExpression\",e[e.IsExported=56]=\"IsExported\"}(t||(t={})),e.transformTypeScript=function(n){var t,a,o,l,c,u,d,m,p,f,g=n.startLexicalEnvironment,_=n.resumeLexicalEnvironment,v=n.endLexicalEnvironment,y=n.hoistVariableDeclaration,h=n.getEmitResolver(),b=n.getCompilerOptions(),E=e.getStrictOptionValue(b,\"strictNullChecks\"),T=e.getEmitScriptTarget(b),S=e.getEmitModuleKind(b),L=n.onEmitNode,x=n.onSubstituteNode;return n.onEmitNode=function(n,r,a){var i=p,o=t;e.isSourceFile(r)&&(t=r),2&d&&function(n){return 242===e.getOriginalNode(n).kind}(r)&&(p|=2),8&d&&function(n){return 241===e.getOriginalNode(n).kind}(r)&&(p|=8),L(n,r,a),p=i,t=o},n.onSubstituteNode=function(n,t){return t=x(n,t),1===n?function(n){switch(n.kind){case 71:return function(n){return function(n){if(1&d&&16777216&h.getNodeCheckFlags(n)){var t=h.getReferencedValueDeclaration(n);if(t){var r=m[t.id];if(r){var a=e.getSynthesizedClone(r);return e.setSourceMapRange(a,n),e.setCommentRange(a,n),a}}}}(n)||Ge(n)||n}(n);case 187:case 188:return function(e){return Ve(e)}(n)}return n}(t):e.isShorthandPropertyAssignment(t)?function(n){if(2&d){var t=n.name,r=Ge(t);if(r){if(n.objectAssignmentInitializer){var a=e.createAssignment(r,n.objectAssignmentInitializer);return e.setTextRange(e.createPropertyAssignment(t,a),n)}return e.setTextRange(e.createPropertyAssignment(t,r),n)}}return n}(t):t},n.enableSubstitution(187),n.enableSubstitution(188),function(n){return 278===n.kind?function(n){return e.createBundle(n.sourceFiles.map(A),e.mapDefined(n.prepends,function(n){return 280===n.kind?e.createUnparsedSourceFile(n.javascriptText,n.javascriptMapPath,n.javascriptMapText):n}))}(n):A(n)};function A(r){if(r.isDeclarationFile)return r;t=r;var a=C(r,G);return e.addEmitHelpers(a,n.readEmitHelpers()),t=void 0,a}function C(n,t){var r=l,a=c,i=u;!function(n){switch(n.kind){case 277:case 244:case 243:case 216:l=n,c=void 0,u=void 0;break;case 238:case 237:if(e.hasModifier(n,2))break;n.name?ye(n):e.Debug.assert(238===n.kind||e.hasModifier(n,512)),e.isClassDeclaration(n)&&(c=n)}}(n);var o=t(n);return l!==r&&(u=i),l=r,c=a,o}function D(e){return C(e,k)}function k(t){return 1&t.transformFlags?F(t):2&t.transformFlags?e.visitEachChild(t,D,n):t}function M(e){return C(e,O)}function O(t){switch(t.kind){case 247:case 246:case 252:case 253:return function(t){if(e.getParseTreeNode(t)!==t)return 2&t.transformFlags?e.visitEachChild(t,D,n):t;switch(t.kind){case 247:return function(n){if(!n.importClause)return n;var t=e.visitNode(n.importClause,Te,e.isImportClause);return t?e.updateImportDeclaration(n,void 0,void 0,t,n.moduleSpecifier):void 0}(t);case 246:return Ce(t);case 252:return function(t){return h.isValueAliasDeclaration(t)?e.visitEachChild(t,D,n):void 0}(t);case 253:return function(n){if(!n.exportClause)return b.isolatedModules||h.moduleExportsSomeValue(n.moduleSpecifier)?n:void 0;if(h.isValueAliasDeclaration(n)){var t=e.visitNode(n.exportClause,xe,e.isNamedExports);return t?e.updateExportDeclaration(n,void 0,void 0,t,n.moduleSpecifier):void 0}}(t);default:e.Debug.fail(\"Unhandled ellided statement\")}}(t);default:return k(t)}}function I(e){return C(e,R)}function R(t){if(253!==t.kind&&247!==t.kind&&248!==t.kind&&(246!==t.kind||257!==t.moduleReference.kind))return 1&t.transformFlags||e.hasModifier(t,1)?F(t):2&t.transformFlags?e.visitEachChild(t,D,n):t}function N(e){return C(e,w)}function w(n){switch(n.kind){case 155:return;case 152:case 160:case 156:case 157:case 154:return k(n);case 215:return n;default:return e.Debug.failBadSyntaxKind(n)}}function P(n){if(!(2270&e.modifierToFlag(n.kind)||a&&84===n.kind))return n}function F(i){if(e.hasModifier(i,2)&&e.isStatement(i))return e.createNotEmittedStatement(i);switch(i.kind){case 84:case 79:return a?void 0:i;case 114:case 112:case 113:case 117:case 76:case 124:case 132:case 167:case 168:case 169:case 170:case 166:case 161:case 148:case 119:case 142:case 122:case 137:case 134:case 131:case 105:case 138:case 164:case 163:case 165:case 162:case 171:case 172:case 173:case 175:case 176:case 177:case 178:case 179:case 180:case 160:case 150:case 240:return;case 152:return function(n){var t=de(n.name,e.some(n.decorators)||!!n.initializer,!0);t&&!ce(t)&&(f||(f=[])).push(t)}(i);case 245:return;case 155:return function(t){if(pe(t))return e.updateConstructor(t,e.visitNodes(t.decorators,D,e.isDecorator),e.visitNodes(t.modifiers,D,e.isModifier),e.visitParameterList(t.parameters,D,n),e.visitFunctionBody(t.body,D,n))}(i);case 239:return e.createNotEmittedStatement(i);case 238:return function(a){var i=f;f=void 0;var o=U(a,!0),s=function(n,t){var r=0;e.some(t)&&(r|=1);var a=e.getEffectiveBaseTypeNode(n);return a&&95!==e.skipOuterExpressions(a.expression).kind&&(r|=64),function(n){if(n.decorators&&n.decorators.length>0)return!0;var t=e.getFirstConstructorWithBody(n);return!!t&&e.forEach(t.parameters,V)}(n)&&(r|=2),e.childIsDecorated(n)&&(r|=4),De(n)?r|=8:function(n){return ke(n)&&e.hasModifier(n,512)}(n)?r|=32:Me(n)&&(r|=16),T<=1&&7&r&&(r|=128),r}(a,o);128&s&&n.startLexicalEnvironment();var l=a.name||(5&s?e.getGeneratedNameForNode(a):void 0),c=2&s?function(n,t,r){var a=e.moveRangePastDecorators(n),i=function(n){if(8388608&h.getNodeCheckFlags(n)){Fe();var t=e.createUniqueName(n.name&&!e.isGeneratedIdentifier(n.name)?e.idText(n.name):\"default\");return m[e.getOriginalNodeId(n)]=t,y(t),t}}(n),o=e.getLocalName(n,!1,!0),s=e.visitNodes(n.heritageClauses,D,e.isHeritageClause),l=K(n,0!=(64&r)),c=e.createClassExpression(void 0,t,void 0,s,l);e.setOriginalNode(c,n),e.setTextRange(c,a);var u=e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(o,void 0,i?e.createAssignment(i,c):c)],1));return e.setOriginalNode(u,n),e.setTextRange(u,a),e.setCommentRange(u,n),u}(a,l,s):function(n,t,r){var a=128&r?void 0:e.visitNodes(n.modifiers,P,e.isModifier),i=e.createClassDeclaration(void 0,a,t,void 0,e.visitNodes(n.heritageClauses,D,e.isHeritageClause),K(n,0!=(64&r))),o=e.getEmitFlags(n);return 1&r&&(o|=32),e.setTextRange(i,n),e.setOriginalNode(i,n),e.setEmitFlags(i,o),i}(a,l,s),u=[c];if(e.some(f)&&u.push(e.createExpressionStatement(e.inlineExpressions(f))),f=i,1&s&&z(u,o,128&s?e.getInternalName(a):e.getLocalName(a)),ee(u,a,!1),ee(u,a,!0),function(t,a){var i=function(t){var a=function(n){var t=n.decorators,r=Q(e.getFirstConstructorWithBody(n));if(t||r)return{decorators:t,parameters:r}}(t),i=$(t,t,a);if(i){var o=m&&m[e.getOriginalNodeId(t)],s=e.getLocalName(t,!1,!0),l=r(n,i,s),c=e.createAssignment(s,o?e.createAssignment(o,l):l);return e.setEmitFlags(c,1536),e.setSourceMapRange(c,e.moveRangePastDecorators(t)),c}}(a);i&&t.push(e.setOriginalNode(e.createExpressionStatement(i),a))}(u,a),128&s){var d=e.createTokenRange(e.skipTrivia(t.text,a.members.end),18),p=e.getInternalName(a),g=e.createPartiallyEmittedExpression(p);g.end=d.end,e.setEmitFlags(g,1536);var _=e.createReturn(g);_.pos=d.pos,e.setEmitFlags(_,1920),u.push(_),e.addStatementsAfterPrologue(u,n.endLexicalEnvironment());var v=e.createImmediatelyInvokedArrowFunction(u);e.setEmitFlags(v,33554432);var b=e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(e.getLocalName(a,!1,!1),void 0,v)]));e.setOriginalNode(b,a),e.setCommentRange(b,a),e.setSourceMapRange(b,e.moveRangePastDecorators(a)),e.startOnNewLine(b),u=[b]}return 8&s?Ie(u,a):(128&s||2&s)&&(32&s?u.push(e.createExportDefault(e.getLocalName(a,!1,!0))):16&s&&u.push(e.createExternalModuleExport(e.getLocalName(a,!1,!0)))),u.length>1&&(u.push(e.createEndOfDeclarationMarker(a)),e.setEmitFlags(c,4194304|e.getEmitFlags(c))),e.singleOrMany(u)}(i);case 207:return function(n){var t=f;f=void 0;var r=U(n,!0),a=e.visitNodes(n.heritageClauses,D,e.isHeritageClause),i=K(n,e.some(a,function(e){return 85===e.token})),o=e.createClassExpression(void 0,n.name,void 0,a,i);if(e.setOriginalNode(o,n),e.setTextRange(o,n),e.some(r)||e.some(f)){var s=[],l=8388608&h.getNodeCheckFlags(n),c=e.createTempVariable(y,!!l);if(l){Fe();var u=e.getSynthesizedClone(c);u.autoGenerateFlags&=-9,m[e.getOriginalNodeId(n)]=u}return e.setEmitFlags(o,65536|e.getEmitFlags(o)),s.push(e.startOnNewLine(e.createAssignment(c,o))),e.addRange(s,e.map(f,e.startOnNewLine)),f=t,e.addRange(s,function(n,t){for(var r=[],a=0,i=n;a<i.length;a++){var o=i[a],s=J(o,t);e.startOnNewLine(s),e.setSourceMapRange(s,e.moveRangePastModifiers(o)),e.setCommentRange(s,o),e.setOriginalNode(s,o),r.push(s)}return r}(r,c)),s.push(e.startOnNewLine(c)),e.inlineExpressions(s)}return f=t,o}(i);case 271:return function(n){if(85===n.token){var t=e.visitNodes(n.types,D,e.isExpressionWithTypeArguments,0,1);return e.setTextRange(e.createHeritageClause(85,t),n)}}(i);case 209:return function(n){return e.updateExpressionWithTypeArguments(n,void 0,e.visitNode(n.expression,D,e.isLeftHandSideExpression))}(i);case 154:return function(t){if(pe(t)){var r=e.updateMethod(t,void 0,e.visitNodes(t.modifiers,P,e.isModifier),t.asteriskToken,me(t),void 0,void 0,e.visitParameterList(t.parameters,D,n),void 0,e.visitFunctionBody(t.body,D,n));return r!==t&&(e.setCommentRange(r,t),e.setSourceMapRange(r,e.moveRangePastDecorators(t))),r}}(i);case 156:return function(t){if(fe(t)){var r=e.updateGetAccessor(t,void 0,e.visitNodes(t.modifiers,P,e.isModifier),me(t),e.visitParameterList(t.parameters,D,n),void 0,e.visitFunctionBody(t.body,D,n)||e.createBlock([]));return r!==t&&(e.setCommentRange(r,t),e.setSourceMapRange(r,e.moveRangePastDecorators(t))),r}}(i);case 157:return function(t){if(fe(t)){var r=e.updateSetAccessor(t,void 0,e.visitNodes(t.modifiers,P,e.isModifier),me(t),e.visitParameterList(t.parameters,D,n),e.visitFunctionBody(t.body,D,n)||e.createBlock([]));return r!==t&&(e.setCommentRange(r,t),e.setSourceMapRange(r,e.moveRangePastDecorators(t))),r}}(i);case 237:return function(t){if(!pe(t))return e.createNotEmittedStatement(t);var r=e.updateFunctionDeclaration(t,void 0,e.visitNodes(t.modifiers,P,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,D,n),void 0,e.visitFunctionBody(t.body,D,n)||e.createBlock([]));if(De(t)){var a=[r];return Ie(a,t),a}return r}(i);case 194:return function(t){return pe(t)?e.updateFunctionExpression(t,e.visitNodes(t.modifiers,P,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,D,n),void 0,e.visitFunctionBody(t.body,D,n)||e.createBlock([])):e.createOmittedExpression()}(i);case 195:return function(t){return e.updateArrowFunction(t,e.visitNodes(t.modifiers,P,e.isModifier),void 0,e.visitParameterList(t.parameters,D,n),void 0,t.equalsGreaterThanToken,e.visitFunctionBody(t.body,D,n))}(i);case 149:return function(n){if(!e.parameterIsThisKeyword(n)){var t=e.createParameter(void 0,void 0,n.dotDotDotToken,e.visitNode(n.name,D,e.isBindingName),void 0,void 0,e.visitNode(n.initializer,D,e.isExpression));return e.setOriginalNode(t,n),e.setTextRange(t,e.moveRangePastModifiers(n)),e.setCommentRange(t,n),e.setSourceMapRange(t,e.moveRangePastModifiers(n)),e.setEmitFlags(t.name,32),t}}(i);case 193:return function(r){var a=e.skipOuterExpressions(r.expression,-3);if(e.isAssertionExpression(a)){var i=e.visitNode(r.expression,D,e.isExpression);return e.length(e.getLeadingCommentRangesOfNode(i,t))?e.updateParen(r,i):e.createPartiallyEmittedExpression(i,r)}return e.visitEachChild(r,D,n)}(i);case 192:case 210:return function(n){var t=e.visitNode(n.expression,D,e.isExpression);return e.createPartiallyEmittedExpression(t,n)}(i);case 189:return function(n){return e.updateCall(n,e.visitNode(n.expression,D,e.isExpression),void 0,e.visitNodes(n.arguments,D,e.isExpression))}(i);case 190:return function(n){return e.updateNew(n,e.visitNode(n.expression,D,e.isExpression),void 0,e.visitNodes(n.arguments,D,e.isExpression))}(i);case 191:return function(n){return e.updateTaggedTemplate(n,e.visitNode(n.tag,D,e.isExpression),void 0,e.visitNode(n.template,D,e.isExpression))}(i);case 211:return function(n){var t=e.visitNode(n.expression,D,e.isLeftHandSideExpression);return e.createPartiallyEmittedExpression(t,n)}(i);case 241:return function(n){if(function(n){return!e.isEnumConst(n)||b.preserveConstEnums||b.isolatedModules}(n)){var r=[],a=2,i=be(r,n);i&&(S===e.ModuleKind.System&&l===t||(a|=512));var s=we(n),c=Pe(n),u=e.hasModifier(n,1)?e.getExternalModuleOrNamespaceExportName(o,n,!1,!0):e.getLocalName(n,!1,!0),d=e.createLogicalOr(u,e.createAssignment(u,e.createObjectLiteral()));if(ve(n)){var m=e.getLocalName(n,!1,!0);d=e.createAssignment(m,d)}var p=e.createExpressionStatement(e.createCall(e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,s)],void 0,function(n,t){var r=o;o=t;var a=[];g();var i=e.map(n.members,_e);return e.addStatementsAfterPrologue(a,v()),e.addRange(a,i),o=r,e.createBlock(e.setTextRange(e.createNodeArray(a),n.members),!0)}(n,c)),void 0,[d]));return e.setOriginalNode(p,n),i&&(e.setSyntheticLeadingComments(p,void 0),e.setSyntheticTrailingComments(p,void 0)),e.setTextRange(p,n),e.addEmitFlags(p,a),r.push(p),r.push(e.createEndOfDeclarationMarker(n)),r}}(i);case 217:return function(t){if(De(t)){var r=e.getInitializedVariables(t.declarationList);if(0===r.length)return;return e.setTextRange(e.createExpressionStatement(e.inlineExpressions(e.map(r,ge))),t)}return e.visitEachChild(t,D,n)}(i);case 235:return function(n){return e.updateVariableDeclaration(n,e.visitNode(n.name,D,e.isBindingName),void 0,e.visitNode(n.initializer,D,e.isExpression))}(i);case 242:return Ee(i);case 246:return Ce(i);default:return e.Debug.failBadSyntaxKind(i)}}function G(t){var r=e.getStrictOptionValue(b,\"alwaysStrict\")&&!(e.isExternalModule(t)&&S>=e.ModuleKind.ES2015)&&!e.isJsonSourceFile(t);return e.updateSourceFileNode(t,e.visitLexicalEnvironment(t.statements,M,n,0,r))}function V(e){return void 0!==e.decorators&&e.decorators.length>0}function K(t,r){var a=[],i=function(t,r){var a=e.forEach(t.members,W),i=262144&t.transformFlags,o=e.getFirstConstructorWithBody(t);if(!a&&!i)return e.visitEachChild(o,D,n);var s=function(t){return e.visitParameterList(t&&t.parameters,D,n)||[]}(o),l=function(n,t,r){var a=[],i=0;if(_(),t){i=function(n,t){if(n.body){var r=n.body.statements,a=e.addPrologue(t,r,!1,D);if(a===r.length)return a;var i=r[a];return 219===i.kind&&e.isSuperCall(i.expression)?(t.push(e.visitNode(i,D,e.isStatement)),a+1):a}return 0}(t,a);var o=function(n){return e.filter(n.parameters,B)}(t);e.addRange(a,e.map(o,H))}else r&&a.push(e.createExpressionStatement(e.createCall(e.createSuper(),void 0,[e.createSpread(e.createIdentifier(\"arguments\"))])));var s=U(n,!1);return z(a,s,e.createThis()),t&&e.addRange(a,e.visitNodes(t.body.statements,D,e.isStatement,i)),a=e.mergeLexicalEnvironment(a,v()),e.setTextRange(e.createBlock(e.setTextRange(e.createNodeArray(a),t?t.body.statements:n.members),!0),t?t.body:void 0)}(t,o,r);return e.startOnNewLine(e.setOriginalNode(e.setTextRange(e.createConstructor(void 0,void 0,s,l),o||t),o))}(t,r);return i&&a.push(i),e.addRange(a,e.visitNodes(t.members,N,e.isClassElement)),e.setTextRange(e.createNodeArray(a),t.members)}function B(n){return e.hasModifier(n,92)&&e.isIdentifier(n.name)}function H(n){e.Debug.assert(e.isIdentifier(n.name));var t=n.name,r=e.getMutableClone(t);e.setEmitFlags(r,1584);var a=e.getMutableClone(t);return e.setEmitFlags(a,1536),e.startOnNewLine(e.setEmitFlags(e.setTextRange(e.createExpressionStatement(e.createAssignment(e.setTextRange(e.createPropertyAccess(e.createThis(),r),n.name),a)),e.moveRangePos(n,-1)),1536))}function U(n,t){return e.filter(n.members,t?j:W)}function j(e){return q(e,!0)}function W(e){return q(e,!1)}function q(n,t){return 152===n.kind&&t===e.hasModifier(n,32)&&void 0!==n.initializer}function z(n,t,r){for(var a=0,i=t;a<i.length;a++){var o=i[a],s=e.createExpressionStatement(J(o,r));e.setSourceMapRange(s,e.moveRangePastModifiers(o)),e.setCommentRange(s,o),e.setOriginalNode(s,o),n.push(s)}}function J(n,t){var r=e.isComputedPropertyName(n.name)&&!ce(n.name.expression)?e.updateComputedPropertyName(n.name,e.getGeneratedNameForNode(n.name)):n.name,a=e.visitNode(n.initializer,D,e.isExpression),i=e.createMemberAccessForPropertyName(t,r,r);return e.createAssignment(i,a)}function X(n,t){return e.filter(n.members,t?function(e){return Y(e,!0,n)}:function(e){return Y(e,!1,n)})}function Y(n,t,r){return e.nodeOrChildIsDecorated(n,r)&&t===e.hasModifier(n,32)}function Q(e){var n;if(e)for(var t=e.parameters,r=0;r<t.length;r++){var a=t[r];(n||a.decorators)&&(n||(n=new Array(t.length)),n[r]=a.decorators)}return n}function Z(n,t){switch(t.kind){case 156:case 157:return function(n,t){if(t.body){var r=e.getAllAccessorDeclarations(n.members,t),a=r.firstAccessor,i=r.secondAccessor,o=r.setAccessor,s=a.decorators?a:i&&i.decorators?i:void 0;if(s&&t===s){var l=s.decorators,c=Q(o);if(l||c)return{decorators:l,parameters:c}}}}(n,t);case 154:return function(e){if(e.body){var n=e.decorators,t=Q(e);if(n||t)return{decorators:n,parameters:t}}}(t);case 152:return function(e){var n=e.decorators;if(n)return{decorators:n}}(t);default:return}}function $(t,r,a){if(a){var o=[];return e.addRange(o,e.map(a.decorators,te)),e.addRange(o,e.flatMap(a.parameters,re)),function(t,r,a){!function(t,r,a){b.emitDecoratorMetadata&&(function(e){var n=e.kind;return 154===n||156===n||157===n||152===n}(t)&&a.push(i(n,\"design:type\",ae(t))),function(n){switch(n.kind){case 238:case 207:return void 0!==e.getFirstConstructorWithBody(n);case 154:case 156:case 157:return!0}return!1}(t)&&a.push(i(n,\"design:paramtypes\",function(n,t){var r=e.isClassLike(n)?e.getFirstConstructorWithBody(n):e.isFunctionLike(n)&&e.nodeIsPresent(n.body)?n:void 0,a=[];if(r)for(var i=function(n,t){if(t&&156===n.kind){var r=e.getAllAccessorDeclarations(t.members,n).setAccessor;if(r)return r.parameters}return n.parameters}(r,t),o=i.length,s=0;s<o;s++){var l=i[s];0===s&&e.isIdentifier(l.name)&&\"this\"===l.name.escapedText||(l.dotDotDotToken?a.push(ie(e.getRestParameterElementType(l.type))):a.push(ae(l)))}return e.createArrayLiteral(a)}(t,r))),function(e){return 154===e.kind}(t)&&a.push(i(n,\"design:returntype\",function(n){return e.isFunctionLike(n)&&n.type?ie(n.type):e.isAsyncFunction(n)?e.createIdentifier(\"Promise\"):e.createVoidZero()}(t))))}(t,r,a)}(t,r,o),o}}function ee(n,t,r){e.addRange(n,e.map(function(e,n){for(var t,r=X(e,n),a=0,i=r;a<i.length;a++){var o=i[a],s=ne(e,o);s&&(t?t.push(s):t=[s])}return t}(t,r),Oe))}function ne(t,a){var i=$(a,t,Z(t,a));if(i){var o=function(n,t){return e.hasModifier(t,32)?e.getDeclarationName(n):function(n){return e.createPropertyAccess(e.getDeclarationName(n),\"prototype\")}(n)}(t,a),s=ue(a,!0),l=T>0?152===a.kind?e.createVoidZero():e.createNull():void 0,c=r(n,i,o,s,l,e.moveRangePastDecorators(a));return e.setEmitFlags(c,1536),c}}function te(n){return e.visitNode(n.expression,D,e.isExpression)}function re(t,r){var a;if(t){a=[];for(var i=0,o=t;i<o.length;i++){var l=o[i],c=s(n,te(l),r,l.expression);e.setEmitFlags(c,1536),a.push(c)}}return a}function ae(n){switch(n.kind){case 152:case 149:return ie(n.type);case 157:case 156:return ie(function(n){var t=h.getAllAccessorDeclarations(n);return t.setAccessor&&e.getSetAccessorTypeAnnotationNode(t.setAccessor)||t.getAccessor&&e.getEffectiveReturnTypeNode(t.getAccessor)}(n));case 238:case 207:case 154:return e.createIdentifier(\"Function\");default:return e.createVoidZero()}}function ie(n){if(void 0===n)return e.createIdentifier(\"Object\");switch(n.kind){case 105:case 140:case 95:case 131:return e.createVoidZero();case 175:return ie(n.type);case 163:case 164:return e.createIdentifier(\"Function\");case 167:case 168:return e.createIdentifier(\"Array\");case 161:case 122:return e.createIdentifier(\"Boolean\");case 137:return e.createIdentifier(\"String\");case 135:return e.createIdentifier(\"Object\");case 180:switch(n.literal.kind){case 9:return e.createIdentifier(\"String\");case 8:return e.createIdentifier(\"Number\");case 101:case 86:return e.createIdentifier(\"Boolean\");default:return e.Debug.failBadSyntaxKind(n.literal)}case 134:return e.createIdentifier(\"Number\");case 138:return T<2?le():e.createIdentifier(\"Symbol\");case 162:return function(n){var t=h.getTypeReferenceSerializationKind(n.typeName,c||l);switch(t){case e.TypeReferenceSerializationKind.Unknown:var r=function n(t){if(71===t.kind){var r=se(t);return oe(r,r)}if(71===t.left.kind)return oe(se(t.left),se(t));var a=n(t.left),i=e.createTempVariable(y);return e.createLogicalAnd(e.createLogicalAnd(a.left,e.createStrictInequality(e.createAssignment(i,a.right),e.createVoidZero())),e.createPropertyAccess(i,t.right))}(n.typeName),a=e.createTempVariable(y);return e.createConditional(e.createTypeCheck(e.createAssignment(a,r),\"function\"),a,e.createIdentifier(\"Object\"));case e.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue:return se(n.typeName);case e.TypeReferenceSerializationKind.VoidNullableOrNeverType:return e.createVoidZero();case e.TypeReferenceSerializationKind.BooleanType:return e.createIdentifier(\"Boolean\");case e.TypeReferenceSerializationKind.NumberLikeType:return e.createIdentifier(\"Number\");case e.TypeReferenceSerializationKind.StringLikeType:return e.createIdentifier(\"String\");case e.TypeReferenceSerializationKind.ArrayLikeType:return e.createIdentifier(\"Array\");case e.TypeReferenceSerializationKind.ESSymbolType:return T<2?le():e.createIdentifier(\"Symbol\");case e.TypeReferenceSerializationKind.TypeWithCallSignature:return e.createIdentifier(\"Function\");case e.TypeReferenceSerializationKind.Promise:return e.createIdentifier(\"Promise\");case e.TypeReferenceSerializationKind.ObjectType:return e.createIdentifier(\"Object\");default:return e.Debug.assertNever(t)}}(n);case 172:case 171:return function(n){for(var t,r=0,a=n.types;r<a.length;r++){for(var i=a[r];175===i.kind;)i=i.type;if(131!==i.kind&&(E||95!==i.kind&&140!==i.kind)){var o=ie(i);if(e.isIdentifier(o)&&\"Object\"===o.escapedText)return o;if(t){if(!e.isIdentifier(t)||!e.isIdentifier(o)||t.escapedText!==o.escapedText)return e.createIdentifier(\"Object\")}else t=o}}return t||e.createVoidZero()}(n);case 165:case 177:case 178:case 179:case 166:case 119:case 142:case 176:case 181:break;default:return e.Debug.failBadSyntaxKind(n)}return e.createIdentifier(\"Object\")}function oe(n,t){return e.createLogicalAnd(e.createStrictInequality(e.createTypeOf(n),e.createLiteral(\"undefined\")),t)}function se(n){switch(n.kind){case 71:var t=e.getMutableClone(n);return t.flags&=-9,t.original=void 0,t.parent=e.getParseTreeNode(l),t;case 146:return function(n){return e.createPropertyAccess(se(n.left),n.right)}(n)}}function le(){return e.createConditional(e.createTypeCheck(e.createIdentifier(\"Symbol\"),\"function\"),e.createIdentifier(\"Symbol\"),e.createIdentifier(\"Object\"))}function ce(n){return!e.isIdentifier(n)&&e.isSimpleCopiableExpression(n)||e.isWellKnownSymbolSyntactically(n)}function ue(n,t){var r=n.name;return e.isComputedPropertyName(r)?t&&!ce(r.expression)?e.getGeneratedNameForNode(r):r.expression:e.isIdentifier(r)?e.createLiteral(e.idText(r)):e.getSynthesizedClone(r)}function de(n,t,r){if(e.isComputedPropertyName(n)){var a=e.visitNode(n.expression,D,e.isExpression),i=e.skipPartiallyEmittedExpressions(a),o=ce(i);if(!o&&t){var s=e.getGeneratedNameForNode(n);return y(s),e.createAssignment(s,a)}return r&&(o||e.isIdentifier(i))?void 0:a}}function me(n){var t=n.name,r=de(t,e.some(n.decorators),!1);return r?(e.some(f)&&(r=e.inlineExpressions(f.concat([r])),f.length=0),e.updateComputedPropertyName(t,r)):t}function pe(n){return!e.nodeIsMissing(n.body)}function fe(n){return!(e.nodeIsMissing(n.body)&&e.hasModifier(n,128))}function ge(t){var r=t.name;return e.isBindingPattern(r)?e.flattenDestructuringAssignment(t,D,n,0,!1,Re):e.setTextRange(e.createAssignment(Ne(r),e.visitNode(t.initializer,D,e.isExpression)),t)}function _e(t){var r=ue(t,!1),a=function(t){var r=h.getConstantValue(t);return void 0!==r?e.createLiteral(r):(0==(8&d)&&(d|=8,n.enableSubstitution(71)),t.initializer?e.visitNode(t.initializer,D,e.isExpression):e.createVoidZero())}(t),i=e.createAssignment(e.createElementAccess(o,r),a),s=9===a.kind?i:e.createAssignment(e.createElementAccess(o,i),r);return e.setTextRange(e.createExpressionStatement(e.setTextRange(s,t)),t)}function ve(n){return De(n)||ke(n)&&S!==e.ModuleKind.ES2015&&S!==e.ModuleKind.ESNext&&S!==e.ModuleKind.System}function ye(n){u||(u=e.createUnderscoreEscapedMap());var t=he(n);u.has(t)||u.set(t,n)}function he(n){return e.Debug.assertNode(n.name,e.isIdentifier),n.name.escapedText}function be(n,t){var r=e.createVariableStatement(e.visitNodes(t.modifiers,P,e.isModifier),e.createVariableDeclarationList([e.createVariableDeclaration(e.getLocalName(t,!1,!0))],277===l.kind?0:1));if(e.setOriginalNode(r,t),ye(t),function(e){if(u){var n=he(e);return u.get(n)===e}return!0}(t))return 241===t.kind?e.setSourceMapRange(r.declarationList,t):e.setSourceMapRange(r,t),e.setCommentRange(r,t),e.addEmitFlags(r,4195328),n.push(r),!0;var a=e.createMergeDeclarationMarker(r);return e.setEmitFlags(a,4195840),n.push(a),!1}function Ee(r){if(!function(n){return e.isInstantiatedModule(n,!!b.preserveConstEnums||!!b.isolatedModules)}(r))return e.createNotEmittedStatement(r);e.Debug.assertNode(r.name,e.isIdentifier,\"A TypeScript namespace should have an Identifier name.\"),0==(2&d)&&(d|=2,n.enableSubstitution(71),n.enableSubstitution(274),n.enableEmitNotification(242));var i=[],s=2,c=be(i,r);c&&(S===e.ModuleKind.System&&l===t||(s|=512));var m=we(r),p=Pe(r),f=e.hasModifier(r,1)?e.getExternalModuleOrNamespaceExportName(o,r,!1,!0):e.getLocalName(r,!1,!0),_=e.createLogicalOr(f,e.createAssignment(f,e.createObjectLiteral()));if(ve(r)){var y=e.getLocalName(r,!1,!0);_=e.createAssignment(y,_)}var h=e.createExpressionStatement(e.createCall(e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,m)],void 0,function(n,t){var r=o,i=a,s=u;o=t,a=n,u=void 0;var l,c,d=[];g();var m=n.body;if(243===m.kind)C(m,function(n){return e.addRange(d,e.visitNodes(n.statements,I,e.isStatement))}),l=m.statements,c=m;else{var p=Ee(m);p&&(e.isArray(p)?e.addRange(d,p):d.push(p));var f=function e(n){if(242===n.body.kind){var t=e(n.body);return t||n.body}}(n).body;l=e.moveRangePos(f.statements,-1)}e.addStatementsAfterPrologue(d,v()),o=r,a=i,u=s;var _=e.createBlock(e.setTextRange(e.createNodeArray(d),l),!0);return e.setTextRange(_,c),243!==m.kind&&e.setEmitFlags(_,1536|e.getEmitFlags(_)),_}(r,p)),void 0,[_]));return e.setOriginalNode(h,r),c&&(e.setSyntheticLeadingComments(h,void 0),e.setSyntheticTrailingComments(h,void 0)),e.setTextRange(h,r),e.addEmitFlags(h,s),i.push(h),i.push(e.createEndOfDeclarationMarker(r)),i}function Te(n){var t=h.isReferencedAliasDeclaration(n)?n.name:void 0,r=e.visitNode(n.namedBindings,Se,e.isNamedImportBindings);return t||r?e.updateImportClause(n,t,r):void 0}function Se(n){if(249===n.kind)return h.isReferencedAliasDeclaration(n)?n:void 0;var t=e.visitNodes(n.elements,Le,e.isImportSpecifier);return e.some(t)?e.updateNamedImports(n,t):void 0}function Le(e){return h.isReferencedAliasDeclaration(e)?e:void 0}function xe(n){var t=e.visitNodes(n.elements,Ae,e.isExportSpecifier);return e.some(t)?e.updateNamedExports(n,t):void 0}function Ae(e){return h.isValueAliasDeclaration(e)?e:void 0}function Ce(r){if(e.isExternalModuleImportEqualsDeclaration(r))return h.isReferencedAliasDeclaration(r)?e.visitEachChild(r,D,n):void 0;if(function(n){return h.isReferencedAliasDeclaration(n)||!e.isExternalModule(t)&&h.isTopLevelValueImportEqualsWithEntityName(n)}(r)){var a,i,s,l=e.createExpressionFromEntityName(r.moduleReference);return e.setEmitFlags(l,3584),Me(r)||!De(r)?e.setOriginalNode(e.setTextRange(e.createVariableStatement(e.visitNodes(r.modifiers,P,e.isModifier),e.createVariableDeclarationList([e.setOriginalNode(e.createVariableDeclaration(r.name,void 0,l),r)])),r),r):e.setOriginalNode((a=r.name,i=l,s=r,e.setTextRange(e.createExpressionStatement(e.createAssignment(e.getNamespaceMemberName(o,a,!1,!0),i)),s)),r)}}function De(n){return void 0!==a&&e.hasModifier(n,1)}function ke(n){return void 0===a&&e.hasModifier(n,1)}function Me(n){return ke(n)&&!e.hasModifier(n,512)}function Oe(n){return e.createExpressionStatement(n)}function Ie(n,t){var r=e.createAssignment(e.getExternalModuleOrNamespaceExportName(o,t,!1,!0),e.getLocalName(t));e.setSourceMapRange(r,e.createRange(t.name?t.name.pos:t.pos,t.end));var a=e.createExpressionStatement(r);e.setSourceMapRange(a,e.createRange(-1,t.end)),n.push(a)}function Re(n,t,r){return e.setTextRange(e.createAssignment(Ne(n),t),r)}function Ne(n){return e.getNamespaceMemberName(o,n,!1,!0)}function we(n){var t=e.getGeneratedNameForNode(n);return e.setSourceMapRange(t,n.name),t}function Pe(n){return e.getGeneratedNameForNode(n)}function Fe(){0==(1&d)&&(d|=1,n.enableSubstitution(71),m=[])}function Ge(n){if(d&p&&!e.isGeneratedIdentifier(n)&&!e.isLocalName(n)){var t=h.getReferencedExportContainer(n,!1);if(t&&277!==t.kind&&(2&p&&242===t.kind||8&p&&241===t.kind))return e.setTextRange(e.createPropertyAccess(e.getGeneratedNameForNode(t),n),n)}}function Ve(n){var t=function(n){if(!b.isolatedModules)return e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n)?h.getConstantValue(n):void 0}(n);if(void 0!==t){e.setConstantValue(n,t);var r=e.createLiteral(t);if(!b.removeComments){var a=e.isPropertyAccessExpression(n)?e.declarationNameToString(n.name):e.getTextOfNode(n.argumentExpression);e.addSyntheticTrailingComment(r,3,\" \"+a+\" \")}return r}return n}};var a={name:\"typescript:decorate\",scoped:!1,priority:2,text:'\\n            var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {\\n                var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;\\n                if (typeof Reflect === \"object\" && typeof Reflect.decorate === \"function\") r = Reflect.decorate(decorators, target, key, desc);\\n                else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;\\n                return c > 3 && r && Object.defineProperty(target, key, r), r;\\n            };'};function i(n,t,r){return n.requestEmitHelper(o),e.createCall(e.getHelperName(\"__metadata\"),void 0,[e.createLiteral(t),r])}var o={name:\"typescript:metadata\",scoped:!1,priority:3,text:'\\n            var __metadata = (this && this.__metadata) || function (k, v) {\\n                if (typeof Reflect === \"object\" && typeof Reflect.metadata === \"function\") return Reflect.metadata(k, v);\\n            };'};function s(n,t,r,a){return n.requestEmitHelper(l),e.setTextRange(e.createCall(e.getHelperName(\"__param\"),void 0,[e.createLiteral(r),t]),a)}var l={name:\"typescript:param\",scoped:!1,priority:4,text:\"\\n            var __param = (this && this.__param) || function (paramIndex, decorator) {\\n                return function (target, key) { decorator(target, key, paramIndex); }\\n            };\"}}(Tn||(Tn={})),function(e){var n;!function(e){e[e.AsyncMethodsWithSuper=1]=\"AsyncMethodsWithSuper\"}(n||(n={})),e.transformES2017=function(n){var t,a,i=n.resumeLexicalEnvironment,o=n.endLexicalEnvironment,s=n.hoistVariableDeclaration,l=n.getEmitResolver(),c=n.getCompilerOptions(),u=e.getEmitScriptTarget(c),d=0,m=n.onEmitNode,p=n.onSubstituteNode;return n.onEmitNode=function(e,n,r){if(1&t&&function(e){var n=e.kind;return 238===n||155===n||154===n||156===n||157===n}(n)){var a=6144&l.getNodeCheckFlags(n);if(a!==d){var i=d;return d=a,m(e,n,r),void(d=i)}}m(e,n,r)},n.onSubstituteNode=function(n,t){return t=p(n,t),1===n&&d?function(n){switch(n.kind){case 187:return x(n);case 188:return A(n);case 189:return function(n){var t=n.expression;if(e.isSuperProperty(t)){var r=e.isPropertyAccessExpression(t)?x(t):A(t);return e.createCall(e.createPropertyAccess(r,\"call\"),void 0,[e.createThis()].concat(n.arguments))}return n}(n)}return n}(t):t},e.chainBundle(function(t){if(t.isDeclarationFile)return t;var r=e.visitEachChild(t,f,n);return e.addEmitHelpers(r,n.readEmitHelpers()),r});function f(t){if(0==(16&t.transformFlags))return t;switch(t.kind){case 120:return;case 199:return function(n){return e.setOriginalNode(e.setTextRange(e.createYield(void 0,e.visitNode(n.expression,f,e.isExpression)),n),n)}(t);case 154:return function(t){return e.updateMethod(t,void 0,e.visitNodes(t.modifiers,f,e.isModifier),t.asteriskToken,t.name,void 0,void 0,e.visitParameterList(t.parameters,f,n),void 0,2&e.getFunctionFlags(t)?T(t):e.visitFunctionBody(t.body,f,n))}(t);case 237:return function(t){return e.updateFunctionDeclaration(t,void 0,e.visitNodes(t.modifiers,f,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,f,n),void 0,2&e.getFunctionFlags(t)?T(t):e.visitFunctionBody(t.body,f,n))}(t);case 194:return function(t){return e.updateFunctionExpression(t,e.visitNodes(t.modifiers,f,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,f,n),void 0,2&e.getFunctionFlags(t)?T(t):e.visitFunctionBody(t.body,f,n))}(t);case 195:return function(t){return e.updateArrowFunction(t,e.visitNodes(t.modifiers,f,e.isModifier),void 0,e.visitParameterList(t.parameters,f,n),void 0,t.equalsGreaterThanToken,2&e.getFunctionFlags(t)?T(t):e.visitFunctionBody(t.body,f,n))}(t);default:return e.visitEachChild(t,f,n)}}function g(t){if(e.isNodeWithPossibleHoistedDeclaration(t))switch(t.kind){case 217:return function(t){if(v(t.declarationList)){var r=y(t.declarationList,!1);return r?e.createExpressionStatement(r):void 0}return e.visitEachChild(t,f,n)}(t);case 223:return function(n){var t=n.initializer;return e.updateFor(n,v(t)?y(t,!1):e.visitNode(n.initializer,f,e.isForInitializer),e.visitNode(n.condition,f,e.isExpression),e.visitNode(n.incrementor,f,e.isExpression),e.visitNode(n.statement,g,e.isStatement,e.liftToBlock))}(t);case 224:return function(n){return e.updateForIn(n,v(n.initializer)?y(n.initializer,!0):e.visitNode(n.initializer,f,e.isForInitializer),e.visitNode(n.expression,f,e.isExpression),e.visitNode(n.statement,g,e.isStatement,e.liftToBlock))}(t);case 225:return function(n){return e.updateForOf(n,e.visitNode(n.awaitModifier,f,e.isToken),v(n.initializer)?y(n.initializer,!0):e.visitNode(n.initializer,f,e.isForInitializer),e.visitNode(n.expression,f,e.isExpression),e.visitNode(n.statement,g,e.isStatement,e.liftToBlock))}(t);case 272:return function(t){var r,i=e.createUnderscoreEscapedMap();if(_(t.variableDeclaration,i),i.forEach(function(n,t){a.has(t)&&(r||(r=e.cloneMap(a)),r.delete(t))}),r){var o=a;a=r;var s=e.visitEachChild(t,g,n);return a=o,s}return e.visitEachChild(t,g,n)}(t);case 216:case 230:case 244:case 269:case 270:case 233:case 221:case 222:case 220:case 229:case 231:return e.visitEachChild(t,g,n);default:return e.Debug.assertNever(t,\"Unhandled node.\")}return f(t)}function _(n,t){var r=n.name;if(e.isIdentifier(r))t.set(r.escapedText,!0);else for(var a=0,i=r.elements;a<i.length;a++){var o=i[a];e.isOmittedExpression(o)||_(o,t)}}function v(n){return!!n&&e.isVariableDeclarationList(n)&&!(3&n.flags)&&n.declarations.some(E)}function y(n,t){!function(n){e.forEach(n.declarations,h)}(n);var r=e.getInitializedVariables(n);return 0===r.length?t?e.visitNode(e.convertToAssignmentElementTarget(n.declarations[0].name),f,e.isExpression):void 0:e.inlineExpressions(e.map(r,b))}function h(n){var t=n.name;if(e.isIdentifier(t))s(t);else for(var r=0,a=t.elements;r<a.length;r++){var i=a[r];e.isOmittedExpression(i)||h(i)}}function b(n){var t=e.setSourceMapRange(e.createAssignment(e.convertToAssignmentElementTarget(n.name),n.initializer),n);return e.visitNode(t,f,e.isExpression)}function E(n){var t=n.name;if(e.isIdentifier(t))return a.has(t.escapedText);for(var r=0,i=t.elements;r<i.length;r++){var o=i[r];if(!e.isOmittedExpression(o)&&E(o))return!0}return!1}function T(t){i();var s,c=e.getOriginalNode(t,e.isFunctionLike).type,d=u<2?function(n){var t=n&&e.getEntityNameFromTypeNode(n);if(t&&e.isEntityName(t)){var r=l.getTypeReferenceSerializationKind(t);if(r===e.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue||r===e.TypeReferenceSerializationKind.Unknown)return t}}(c):void 0,m=195===t.kind,p=0!=(8192&l.getNodeCheckFlags(t)),g=a;a=e.createUnderscoreEscapedMap();for(var v=0,y=t.parameters;v<y.length;v++)_(y[v],a);if(m){var h=r(n,p,d,S(t.body)),b=o();e.some(b)?(x=e.convertToFunctionBody(h),s=e.updateBlock(x,e.setTextRange(e.createNodeArray(e.concatenate(b,x.statements)),x.statements))):s=h}else{var E=[],T=e.addPrologue(E,t.body.statements,!1,f);E.push(e.createReturn(r(n,p,d,S(t.body,T)))),e.addStatementsAfterPrologue(E,o());var x=e.createBlock(E,!0);e.setTextRange(x,t.body),u>=2&&(4096&l.getNodeCheckFlags(t)?(L(),e.addEmitHelper(x,e.advancedAsyncSuperHelper)):2048&l.getNodeCheckFlags(t)&&(L(),e.addEmitHelper(x,e.asyncSuperHelper))),s=x}return a=g,s}function S(n,t){return e.isBlock(n)?e.updateBlock(n,e.visitNodes(n.statements,g,e.isStatement,t)):e.convertToFunctionBody(e.visitNode(n,g,e.isConciseBody))}function L(){0==(1&t)&&(t|=1,n.enableSubstitution(189),n.enableSubstitution(187),n.enableSubstitution(188),n.enableEmitNotification(238),n.enableEmitNotification(154),n.enableEmitNotification(156),n.enableEmitNotification(157),n.enableEmitNotification(155))}function x(n){return 97===n.expression.kind?C(e.createLiteral(e.idText(n.name)),n):n}function A(e){return 97===e.expression.kind?C(e.argumentExpression,e):e}function C(n,t){return 4096&d?e.setTextRange(e.createPropertyAccess(e.createCall(e.createFileLevelUniqueName(\"_super\"),void 0,[n]),\"value\"),t):e.setTextRange(e.createCall(e.createFileLevelUniqueName(\"_super\"),void 0,[n]),t)}};var t={name:\"typescript:awaiter\",scoped:!1,priority:5,text:'\\n            var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {\\n                return new (P || (P = Promise))(function (resolve, reject) {\\n                    function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }\\n                    function rejected(value) { try { step(generator[\"throw\"](value)); } catch (e) { reject(e); } }\\n                    function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }\\n                    step((generator = generator.apply(thisArg, _arguments || [])).next());\\n                });\\n            };'};function r(n,r,a,i){n.requestEmitHelper(t);var o=e.createFunctionExpression(void 0,e.createToken(39),void 0,void 0,[],void 0,i);return(o.emitNode||(o.emitNode={})).flags|=786432,e.createCall(e.getHelperName(\"__awaiter\"),void 0,[e.createThis(),r?e.createIdentifier(\"arguments\"):e.createVoidZero(),a?e.createExpressionFromEntityName(a):e.createVoidZero(),o])}e.asyncSuperHelper={name:\"typescript:async-super\",scoped:!0,text:e.helperString(bn([\"\\n            const \",\" = name => super[name];\"],[\"\\n            const \",\" = name => super[name];\"]),\"_super\")},e.advancedAsyncSuperHelper={name:\"typescript:advanced-async-super\",scoped:!0,text:e.helperString(bn([\"\\n            const \",\" = (function (geti, seti) {\\n                const cache = Object.create(null);\\n                return name => cache[name] || (cache[name] = { get value() { return geti(name); }, set value(v) { seti(name, v); } });\\n            })(name => super[name], (name, value) => super[name] = value);\"],[\"\\n            const \",\" = (function (geti, seti) {\\n                const cache = Object.create(null);\\n                return name => cache[name] || (cache[name] = { get value() { return geti(name); }, set value(v) { seti(name, v); } });\\n            })(name => super[name], (name, value) => super[name] = value);\"]),\"_super\")}}(Tn||(Tn={})),function(e){var n;!function(e){e[e.AsyncMethodsWithSuper=1]=\"AsyncMethodsWithSuper\"}(n||(n={})),e.transformESNext=function(n){var t=n.resumeLexicalEnvironment,l=n.endLexicalEnvironment,u=n.hoistVariableDeclaration,d=n.getEmitResolver(),m=n.getCompilerOptions(),p=e.getEmitScriptTarget(m),f=n.onEmitNode;n.onEmitNode=function(e,n,t){if(1&g&&function(e){var n=e.kind;return 238===n||155===n||154===n||156===n||157===n}(n)){var r=6144&d.getNodeCheckFlags(n);if(r!==y){var a=y;return y=r,f(e,n,t),void(y=a)}}f(e,n,t)};var g,_,v=n.onSubstituteNode;n.onSubstituteNode=function(n,t){return t=v(n,t),1===n&&y?function(n){switch(n.kind){case 187:return k(n);case 188:return M(n);case 189:return function(n){var t=n.expression;if(e.isSuperProperty(t)){var r=e.isPropertyAccessExpression(t)?k(t):M(t);return e.createCall(e.createPropertyAccess(r,\"call\"),void 0,[e.createThis()].concat(n.arguments))}return n}(n)}return n}(t):t};var y=0;return e.chainBundle(function(t){if(t.isDeclarationFile)return t;var r=e.visitEachChild(t,h,n);return e.addEmitHelpers(r,n.readEmitHelpers()),r});function h(e){return T(e,!1)}function b(e){return T(e,!0)}function E(e){if(120!==e.kind)return e}function T(t,o){if(0==(8&t.transformFlags))return t;switch(t.kind){case 199:return function(t){return 2&_&&1&_?e.setOriginalNode(e.setTextRange(e.createYield(i(n,e.visitNode(t.expression,h,e.isExpression))),t),t):e.visitEachChild(t,h,n)}(t);case 205:return function(t){if(2&_&&1&_){if(t.asteriskToken){var r=e.visitNode(t.expression,h,e.isExpression);return e.setOriginalNode(e.setTextRange(e.createYield(i(n,e.updateYield(t,t.asteriskToken,function(n,t,r){return n.requestEmitHelper(a),n.requestEmitHelper(s),e.setTextRange(e.createCall(e.getHelperName(\"__asyncDelegator\"),void 0,[t]),r)}(n,c(n,r,r),r)))),t),t)}return e.setOriginalNode(e.setTextRange(e.createYield(L(t.expression?e.visitNode(t.expression,h,e.isExpression):e.createVoidZero())),t),t)}return e.visitEachChild(t,h,n)}(t);case 228:return function(t){return 2&_&&1&_?e.updateReturn(t,L(t.expression?e.visitNode(t.expression,h,e.isExpression):e.createVoidZero())):e.visitEachChild(t,h,n)}(t);case 231:return function(t){if(2&_){var r=e.unwrapInnermostStatementOfLabel(t);return 225===r.kind&&r.awaitModifier?S(r,t):e.restoreEnclosingLabel(e.visitEachChild(r,h,n),t)}return e.visitEachChild(t,h,n)}(t);case 186:return function(t){if(1048576&t.transformFlags){var a=function(n){for(var t,r=[],a=0,i=n;a<i.length;a++){var o=i[a];if(275===o.kind){t&&(r.push(e.createObjectLiteral(t)),t=void 0);var s=o.expression;r.push(e.visitNode(s,h,e.isExpression))}else t=e.append(t,273===o.kind?e.createPropertyAssignment(o.name,e.visitNode(o.initializer,h,e.isExpression)):e.visitNode(o,h,e.isObjectLiteralElementLike))}return t&&r.push(e.createObjectLiteral(t)),r}(t.properties);return a.length&&186!==a[0].kind&&a.unshift(e.createObjectLiteral()),r(n,a)}return e.visitEachChild(t,h,n)}(t);case 202:return function(t,r){return e.isDestructuringAssignment(t)&&1048576&t.left.transformFlags?e.flattenDestructuringAssignment(t,h,n,1,!r):26===t.operatorToken.kind?e.updateBinary(t,e.visitNode(t.left,b,e.isExpression),e.visitNode(t.right,r?b:h,e.isExpression)):e.visitEachChild(t,h,n)}(t,o);case 235:return function(t){return e.isBindingPattern(t.name)&&1048576&t.name.transformFlags?e.flattenDestructuringBinding(t,h,n,1):e.visitEachChild(t,h,n)}(t);case 225:return S(t,void 0);case 223:return function(n){return e.updateFor(n,e.visitNode(n.initializer,b,e.isForInitializer),e.visitNode(n.condition,h,e.isExpression),e.visitNode(n.incrementor,h,e.isExpression),e.visitNode(n.statement,h,e.isStatement))}(t);case 198:return function(t){return e.visitEachChild(t,b,n)}(t);case 155:return function(t){var r=_;_=0;var a=e.updateConstructor(t,void 0,t.modifiers,e.visitParameterList(t.parameters,h,n),A(t));return _=r,a}(t);case 154:return function(t){var r=_;_=e.getFunctionFlags(t);var a=e.updateMethod(t,void 0,1&_?e.visitNodes(t.modifiers,E,e.isModifier):t.modifiers,2&_?void 0:t.asteriskToken,e.visitNode(t.name,h,e.isPropertyName),e.visitNode(void 0,h,e.isToken),void 0,e.visitParameterList(t.parameters,h,n),void 0,2&_&&1&_?x(t):A(t));return _=r,a}(t);case 156:return function(t){var r=_;_=0;var a=e.updateGetAccessor(t,void 0,t.modifiers,e.visitNode(t.name,h,e.isPropertyName),e.visitParameterList(t.parameters,h,n),void 0,A(t));return _=r,a}(t);case 157:return function(t){var r=_;_=0;var a=e.updateSetAccessor(t,void 0,t.modifiers,e.visitNode(t.name,h,e.isPropertyName),e.visitParameterList(t.parameters,h,n),A(t));return _=r,a}(t);case 237:return function(t){var r=_;_=e.getFunctionFlags(t);var a=e.updateFunctionDeclaration(t,void 0,1&_?e.visitNodes(t.modifiers,E,e.isModifier):t.modifiers,2&_?void 0:t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,h,n),void 0,2&_&&1&_?x(t):A(t));return _=r,a}(t);case 194:return function(t){var r=_;_=e.getFunctionFlags(t);var a=e.updateFunctionExpression(t,1&_?e.visitNodes(t.modifiers,E,e.isModifier):t.modifiers,2&_?void 0:t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,h,n),void 0,2&_&&1&_?x(t):A(t));return _=r,a}(t);case 195:return function(t){var r=_;_=e.getFunctionFlags(t);var a=e.updateArrowFunction(t,t.modifiers,void 0,e.visitParameterList(t.parameters,h,n),void 0,t.equalsGreaterThanToken,A(t));return _=r,a}(t);case 149:return function(t){return 1048576&t.transformFlags?e.updateParameter(t,void 0,void 0,t.dotDotDotToken,e.getGeneratedNameForNode(t),void 0,void 0,e.visitNode(t.initializer,h,e.isExpression)):e.visitEachChild(t,h,n)}(t);case 219:return function(t){return e.visitEachChild(t,b,n)}(t);case 193:return function(t,r){return e.visitEachChild(t,r?b:h,n)}(t,o);case 272:return function(t){return t.variableDeclaration?e.visitEachChild(t,h,n):e.updateCatchClause(t,e.createVariableDeclaration(e.createTempVariable(void 0)),e.visitNode(t.block,h,e.isBlock))}(t);default:return e.visitEachChild(t,h,n)}}function S(t,r){return 1048576&t.initializer.transformFlags&&(t=function(n){var t=e.skipParentheses(n.initializer);if(e.isVariableDeclarationList(t)||e.isAssignmentPattern(t)){var r=void 0,a=void 0,i=e.createTempVariable(void 0),o=[e.createForOfBindingStatement(t,i)];return e.isBlock(n.statement)?(e.addRange(o,n.statement.statements),r=n.statement,a=n.statement.statements):n.statement&&(e.append(o,n.statement),r=n.statement,a=n.statement),e.updateForOf(n,n.awaitModifier,e.setTextRange(e.createVariableDeclarationList([e.setTextRange(e.createVariableDeclaration(i),n.initializer)],1),n.initializer),n.expression,e.setTextRange(e.createBlock(e.setTextRange(e.createNodeArray(o),a),!0),r))}return n}(t)),t.awaitModifier?function(t,r){var a=e.visitNode(t.expression,h,e.isExpression),i=e.isIdentifier(a)?e.getGeneratedNameForNode(a):e.createTempVariable(void 0),o=e.isIdentifier(a)?e.getGeneratedNameForNode(i):e.createTempVariable(void 0),s=e.createUniqueName(\"e\"),l=e.getGeneratedNameForNode(s),d=e.createTempVariable(void 0),m=c(n,a,t.expression),p=e.createCall(e.createPropertyAccess(i,\"next\"),void 0,[]),f=e.createPropertyAccess(o,\"done\"),g=e.createPropertyAccess(o,\"value\"),_=e.createFunctionCall(d,i,[]);u(s),u(d);var v=e.setEmitFlags(e.setTextRange(e.createFor(e.setEmitFlags(e.setTextRange(e.createVariableDeclarationList([e.setTextRange(e.createVariableDeclaration(i,void 0,m),t.expression),e.createVariableDeclaration(o)]),t.expression),2097152),e.createComma(e.createAssignment(o,L(p)),e.createLogicalNot(f)),void 0,function(n,t){var r,a,i=e.createForOfBindingStatement(n.initializer,t),o=[e.visitNode(i,h,e.isStatement)],s=e.visitNode(n.statement,h,e.isStatement);return e.isBlock(s)?(e.addRange(o,s.statements),r=s,a=s.statements):o.push(s),e.setEmitFlags(e.setTextRange(e.createBlock(e.setTextRange(e.createNodeArray(o),a),!0),r),432)}(t,g)),t),256);return e.createTry(e.createBlock([e.restoreEnclosingLabel(v,r)]),e.createCatchClause(e.createVariableDeclaration(l),e.setEmitFlags(e.createBlock([e.createExpressionStatement(e.createAssignment(s,e.createObjectLiteral([e.createPropertyAssignment(\"error\",l)])))]),1)),e.createBlock([e.createTry(e.createBlock([e.setEmitFlags(e.createIf(e.createLogicalAnd(e.createLogicalAnd(o,e.createLogicalNot(f)),e.createAssignment(d,e.createPropertyAccess(i,\"return\"))),e.createExpressionStatement(L(_))),1)]),void 0,e.setEmitFlags(e.createBlock([e.setEmitFlags(e.createIf(s,e.createThrow(e.createPropertyAccess(s,\"error\"))),1)]),1))]))}(t,r):e.restoreEnclosingLabel(e.visitEachChild(t,h,n),r)}function L(t){return 1&_?e.createYield(void 0,i(n,t)):e.createAwait(t)}function x(r){t();var i=[],s=e.addPrologue(i,r.body.statements,!1,h);C(i,r),i.push(e.createReturn(function(n,t){return n.requestEmitHelper(a),n.requestEmitHelper(o),(t.emitNode||(t.emitNode={})).flags|=262144,e.createCall(e.getHelperName(\"__asyncGenerator\"),void 0,[e.createThis(),e.createIdentifier(\"arguments\"),t])}(n,e.createFunctionExpression(void 0,e.createToken(39),r.name&&e.getGeneratedNameForNode(r.name),void 0,[],void 0,e.updateBlock(r.body,e.visitLexicalEnvironment(r.body.statements,h,n,s)))))),e.addStatementsAfterPrologue(i,l());var c=e.updateBlock(r.body,i);return p>=2&&(4096&d.getNodeCheckFlags(r)?(D(),e.addEmitHelper(c,e.advancedAsyncSuperHelper)):2048&d.getNodeCheckFlags(r)&&(D(),e.addEmitHelper(c,e.asyncSuperHelper))),c}function A(n){t();var r=0,a=[],i=e.visitNode(n.body,h,e.isConciseBody);e.isBlock(i)&&(r=e.addPrologue(a,i.statements,!1,h)),e.addRange(a,C(void 0,n));var o=l();if(r>0||e.some(a)||e.some(o)){var s=e.convertToFunctionBody(i,!0);return e.addStatementsAfterPrologue(a,o),e.addRange(a,s.statements.slice(r)),e.updateBlock(s,e.setTextRange(e.createNodeArray(a),s.statements))}return i}function C(t,r){for(var a=0,i=r.parameters;a<i.length;a++){var o=i[a];if(1048576&o.transformFlags){var s=e.getGeneratedNameForNode(o),l=e.flattenDestructuringBinding(o,h,n,1,s,!1,!0);if(e.some(l)){var c=e.createVariableStatement(void 0,e.createVariableDeclarationList(l));e.setEmitFlags(c,1048576),t=e.append(t,c)}}}return t}function D(){0==(1&g)&&(g|=1,n.enableSubstitution(189),n.enableSubstitution(187),n.enableSubstitution(188),n.enableEmitNotification(238),n.enableEmitNotification(154),n.enableEmitNotification(156),n.enableEmitNotification(157),n.enableEmitNotification(155))}function k(n){return 97===n.expression.kind?O(e.createLiteral(e.idText(n.name)),n):n}function M(e){return 97===e.expression.kind?O(e.argumentExpression,e):e}function O(n,t){return 4096&y?e.setTextRange(e.createPropertyAccess(e.createCall(e.createIdentifier(\"_super\"),void 0,[n]),\"value\"),t):e.setTextRange(e.createCall(e.createIdentifier(\"_super\"),void 0,[n]),t)}};var t={name:\"typescript:assign\",scoped:!1,priority:1,text:\"\\n            var __assign = (this && this.__assign) || function () {\\n                __assign = Object.assign || function(t) {\\n                    for (var s, i = 1, n = arguments.length; i < n; i++) {\\n                        s = arguments[i];\\n                        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))\\n                            t[p] = s[p];\\n                    }\\n                    return t;\\n                };\\n                return __assign.apply(this, arguments);\\n            };\"};function r(n,r){return n.getCompilerOptions().target>=2?e.createCall(e.createPropertyAccess(e.createIdentifier(\"Object\"),\"assign\"),void 0,r):(n.requestEmitHelper(t),e.createCall(e.getHelperName(\"__assign\"),void 0,r))}e.createAssignHelper=r;var a={name:\"typescript:await\",scoped:!1,text:\"\\n            var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }\"};function i(n,t){return n.requestEmitHelper(a),e.createCall(e.getHelperName(\"__await\"),void 0,[t])}var o={name:\"typescript:asyncGenerator\",scoped:!1,text:'\\n            var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {\\n                if (!Symbol.asyncIterator) throw new TypeError(\"Symbol.asyncIterator is not defined.\");\\n                var g = generator.apply(thisArg, _arguments || []), i, q = [];\\n                return i = {}, verb(\"next\"), verb(\"throw\"), verb(\"return\"), i[Symbol.asyncIterator] = function () { return this; }, i;\\n                function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }\\n                function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }\\n                function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }\\n                function fulfill(value) { resume(\"next\", value); }\\n                function reject(value) { resume(\"throw\", value); }\\n                function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }\\n            };'};var s={name:\"typescript:asyncDelegator\",scoped:!1,text:'\\n            var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {\\n                var i, p;\\n                return i = {}, verb(\"next\"), verb(\"throw\", function (e) { throw e; }), verb(\"return\"), i[Symbol.iterator] = function () { return this; }, i;\\n                function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === \"return\" } : f ? f(v) : v; } : f; }\\n            };'};var l={name:\"typescript:asyncValues\",scoped:!1,text:'\\n            var __asyncValues = (this && this.__asyncValues) || function (o) {\\n                if (!Symbol.asyncIterator) throw new TypeError(\"Symbol.asyncIterator is not defined.\");\\n                var m = o[Symbol.asyncIterator], i;\\n                return m ? m.call(o) : (o = typeof __values === \"function\" ? __values(o) : o[Symbol.iterator](), i = {}, verb(\"next\"), verb(\"throw\"), verb(\"return\"), i[Symbol.asyncIterator] = function () { return this; }, i);\\n                function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }\\n                function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }\\n            };'};function c(n,t,r){return n.requestEmitHelper(l),e.setTextRange(e.createCall(e.getHelperName(\"__asyncValues\"),void 0,[t]),r)}}(Tn||(Tn={})),function(e){e.transformJsx=function(t){var r,a=t.getCompilerOptions();return e.chainBundle(function(n){if(n.isDeclarationFile)return n;r=n;var a=e.visitEachChild(n,i,t);return e.addEmitHelpers(a,t.readEmitHelpers()),a});function i(n){return 4&n.transformFlags?function(n){switch(n.kind){case 258:return s(n,!1);case 259:return l(n,!1);case 262:return c(n,!1);case 268:return g(n);default:return e.visitEachChild(n,i,t)}}(n):n}function o(n){switch(n.kind){case 10:return function(n){var t=function(n){for(var t,r=0,a=-1,i=0;i<n.length;i++){var o=n.charCodeAt(i);e.isLineBreak(o)?(-1!==r&&-1!==a&&(t=p(t,n.substr(r,a-r+1))),r=-1):e.isWhiteSpaceSingleLine(o)||(a=i,-1===r&&(r=i))}return-1!==r?p(t,n.substr(r)):t}(e.getTextOfNode(n,!0));return void 0===t?void 0:e.createLiteral(t)}(n);case 268:return g(n);case 258:return s(n,!0);case 259:return l(n,!0);case 262:return c(n,!0);default:return e.Debug.failBadSyntaxKind(n)}}function s(e,n){return u(e.openingElement,e.children,n,e)}function l(e,n){return u(e,void 0,n,e)}function c(n,i){return function(n,i,s,l){var c=e.createExpressionForJsxFragment(t.getEmitResolver().getJsxFactoryEntity(r),a.reactNamespace,e.mapDefined(i,o),n,l);return s&&e.startOnNewLine(c),c}(n.openingFragment,n.children,i,n)}function u(n,i,s,l){var c,u=function n(t){if(258===t.kind)return n(t.openingElement);var r=t.tagName;return e.isIdentifier(r)&&e.isIntrinsicJsxName(r.escapedText)?e.createLiteral(e.idText(r)):e.createExpressionFromEntityName(r)}(n),p=n.attributes.properties;if(0===p.length)c=e.createNull();else{var f=e.flatten(e.spanMap(p,e.isJsxSpreadAttribute,function(n,t){return t?e.map(n,d):e.createObjectLiteral(e.map(n,m))}));e.isJsxSpreadAttribute(p[0])&&f.unshift(e.createObjectLiteral()),(c=e.singleOrUndefined(f))||(c=e.createAssignHelper(t,f))}var g=e.createExpressionForJsxElement(t.getEmitResolver().getJsxFactoryEntity(r),a.reactNamespace,u,c,e.mapDefined(i,o),n,l);return s&&e.startOnNewLine(g),g}function d(n){return e.visitNode(n.expression,i,e.isExpression)}function m(n){var t=function(n){var t=n.name,r=e.idText(t);return/^[A-Za-z_]\\w*$/.test(r)?t:e.createLiteral(r)}(n),a=function(n){if(void 0===n)return e.createTrue();if(9===n.kind){var t=e.createLiteral((a=n.text,((i=f(a))===a?void 0:i)||n.text));return t.singleQuote=void 0!==n.singleQuote?n.singleQuote:!e.isStringDoubleQuoted(n,r),e.setTextRange(t,n)}return 268===n.kind?void 0===n.expression?e.createTrue():g(n):e.Debug.failBadSyntaxKind(n);var a,i}(n.initializer);return e.createPropertyAssignment(t,a)}function p(e,n){var t=f(n);return void 0===e?t:e+\" \"+t}function f(e){return e.replace(/&((#((\\d+)|x([\\da-fA-F]+)))|(\\w+));/g,function(e,t,r,a,i,o,s){if(i)return String.fromCharCode(parseInt(i,10));if(o)return String.fromCharCode(parseInt(o,16));var l=n.get(s);return l?String.fromCharCode(l):e})}function g(n){return e.visitNode(n.expression,i,e.isExpression)}};var n=e.createMapFromTemplate({quot:34,amp:38,apos:39,lt:60,gt:62,nbsp:160,iexcl:161,cent:162,pound:163,curren:164,yen:165,brvbar:166,sect:167,uml:168,copy:169,ordf:170,laquo:171,not:172,shy:173,reg:174,macr:175,deg:176,plusmn:177,sup2:178,sup3:179,acute:180,micro:181,para:182,middot:183,cedil:184,sup1:185,ordm:186,raquo:187,frac14:188,frac12:189,frac34:190,iquest:191,Agrave:192,Aacute:193,Acirc:194,Atilde:195,Auml:196,Aring:197,AElig:198,Ccedil:199,Egrave:200,Eacute:201,Ecirc:202,Euml:203,Igrave:204,Iacute:205,Icirc:206,Iuml:207,ETH:208,Ntilde:209,Ograve:210,Oacute:211,Ocirc:212,Otilde:213,Ouml:214,times:215,Oslash:216,Ugrave:217,Uacute:218,Ucirc:219,Uuml:220,Yacute:221,THORN:222,szlig:223,agrave:224,aacute:225,acirc:226,atilde:227,auml:228,aring:229,aelig:230,ccedil:231,egrave:232,eacute:233,ecirc:234,euml:235,igrave:236,iacute:237,icirc:238,iuml:239,eth:240,ntilde:241,ograve:242,oacute:243,ocirc:244,otilde:245,ouml:246,divide:247,oslash:248,ugrave:249,uacute:250,ucirc:251,uuml:252,yacute:253,thorn:254,yuml:255,OElig:338,oelig:339,Scaron:352,scaron:353,Yuml:376,fnof:402,circ:710,tilde:732,Alpha:913,Beta:914,Gamma:915,Delta:916,Epsilon:917,Zeta:918,Eta:919,Theta:920,Iota:921,Kappa:922,Lambda:923,Mu:924,Nu:925,Xi:926,Omicron:927,Pi:928,Rho:929,Sigma:931,Tau:932,Upsilon:933,Phi:934,Chi:935,Psi:936,Omega:937,alpha:945,beta:946,gamma:947,delta:948,epsilon:949,zeta:950,eta:951,theta:952,iota:953,kappa:954,lambda:955,mu:956,nu:957,xi:958,omicron:959,pi:960,rho:961,sigmaf:962,sigma:963,tau:964,upsilon:965,phi:966,chi:967,psi:968,omega:969,thetasym:977,upsih:978,piv:982,ensp:8194,emsp:8195,thinsp:8201,zwnj:8204,zwj:8205,lrm:8206,rlm:8207,ndash:8211,mdash:8212,lsquo:8216,rsquo:8217,sbquo:8218,ldquo:8220,rdquo:8221,bdquo:8222,dagger:8224,Dagger:8225,bull:8226,hellip:8230,permil:8240,prime:8242,Prime:8243,lsaquo:8249,rsaquo:8250,oline:8254,frasl:8260,euro:8364,image:8465,weierp:8472,real:8476,trade:8482,alefsym:8501,larr:8592,uarr:8593,rarr:8594,darr:8595,harr:8596,crarr:8629,lArr:8656,uArr:8657,rArr:8658,dArr:8659,hArr:8660,forall:8704,part:8706,exist:8707,empty:8709,nabla:8711,isin:8712,notin:8713,ni:8715,prod:8719,sum:8721,minus:8722,lowast:8727,radic:8730,prop:8733,infin:8734,ang:8736,and:8743,or:8744,cap:8745,cup:8746,int:8747,there4:8756,sim:8764,cong:8773,asymp:8776,ne:8800,equiv:8801,le:8804,ge:8805,sub:8834,sup:8835,nsub:8836,sube:8838,supe:8839,oplus:8853,otimes:8855,perp:8869,sdot:8901,lceil:8968,rceil:8969,lfloor:8970,rfloor:8971,lang:9001,rang:9002,loz:9674,spades:9824,clubs:9827,hearts:9829,diams:9830})}(Tn||(Tn={})),function(e){e.transformES2016=function(n){var t=n.hoistVariableDeclaration;return e.chainBundle(function(t){return t.isDeclarationFile?t:e.visitEachChild(t,r,n)});function r(a){if(0==(32&a.transformFlags))return a;switch(a.kind){case 202:return function(a){switch(a.operatorToken.kind){case 62:return function(n){var a,i,o=e.visitNode(n.left,r,e.isExpression),s=e.visitNode(n.right,r,e.isExpression);if(e.isElementAccessExpression(o)){var l=e.createTempVariable(t),c=e.createTempVariable(t);a=e.setTextRange(e.createElementAccess(e.setTextRange(e.createAssignment(l,o.expression),o.expression),e.setTextRange(e.createAssignment(c,o.argumentExpression),o.argumentExpression)),o),i=e.setTextRange(e.createElementAccess(l,c),o)}else if(e.isPropertyAccessExpression(o)){var l=e.createTempVariable(t);a=e.setTextRange(e.createPropertyAccess(e.setTextRange(e.createAssignment(l,o.expression),o.expression),o.name),o),i=e.setTextRange(e.createPropertyAccess(l,o.name),o)}else a=o,i=o;return e.setTextRange(e.createAssignment(a,e.createMathPow(i,s,n)),n)}(a);case 40:return function(n){var t=e.visitNode(n.left,r,e.isExpression),a=e.visitNode(n.right,r,e.isExpression);return e.createMathPow(t,a,n)}(a);default:return e.visitEachChild(a,r,n)}}(a);default:return e.visitEachChild(a,r,n)}}}}(Tn||(Tn={})),function(e){var n,t,r,a,i;!function(e){e[e.CapturedThis=1]=\"CapturedThis\",e[e.BlockScopedBindings=2]=\"BlockScopedBindings\"}(n||(n={})),function(e){e[e.ToOriginal=0]=\"ToOriginal\",e[e.ToOutParameter=1]=\"ToOutParameter\"}(t||(t={})),function(e){e[e.Break=2]=\"Break\",e[e.Continue=4]=\"Continue\",e[e.Return=8]=\"Return\"}(r||(r={})),function(e){e[e.NoReplacement=0]=\"NoReplacement\",e[e.ReplaceSuperCapture=1]=\"ReplaceSuperCapture\",e[e.ReplaceWithReturn=2]=\"ReplaceWithReturn\"}(a||(a={})),function(e){e[e.None=0]=\"None\",e[e.Function=1]=\"Function\",e[e.ArrowFunction=2]=\"ArrowFunction\",e[e.AsyncFunctionBody=4]=\"AsyncFunctionBody\",e[e.NonStaticClassElement=8]=\"NonStaticClassElement\",e[e.CapturesThis=16]=\"CapturesThis\",e[e.ExportedVariableStatement=32]=\"ExportedVariableStatement\",e[e.TopLevel=64]=\"TopLevel\",e[e.Block=128]=\"Block\",e[e.IterationStatement=256]=\"IterationStatement\",e[e.IterationStatementBlock=512]=\"IterationStatementBlock\",e[e.ForStatement=1024]=\"ForStatement\",e[e.ForInOrForOfStatement=2048]=\"ForInOrForOfStatement\",e[e.ConstructorWithCapturedSuper=4096]=\"ConstructorWithCapturedSuper\",e[e.ComputedPropertyName=8192]=\"ComputedPropertyName\",e[e.AncestorFactsMask=16383]=\"AncestorFactsMask\",e[e.BlockScopeIncludes=0]=\"BlockScopeIncludes\",e[e.BlockScopeExcludes=4032]=\"BlockScopeExcludes\",e[e.SourceFileIncludes=64]=\"SourceFileIncludes\",e[e.SourceFileExcludes=3968]=\"SourceFileExcludes\",e[e.FunctionIncludes=65]=\"FunctionIncludes\",e[e.FunctionExcludes=16286]=\"FunctionExcludes\",e[e.AsyncFunctionBodyIncludes=69]=\"AsyncFunctionBodyIncludes\",e[e.AsyncFunctionBodyExcludes=16278]=\"AsyncFunctionBodyExcludes\",e[e.ArrowFunctionIncludes=66]=\"ArrowFunctionIncludes\",e[e.ArrowFunctionExcludes=16256]=\"ArrowFunctionExcludes\",e[e.ConstructorIncludes=73]=\"ConstructorIncludes\",e[e.ConstructorExcludes=16278]=\"ConstructorExcludes\",e[e.DoOrWhileStatementIncludes=256]=\"DoOrWhileStatementIncludes\",e[e.DoOrWhileStatementExcludes=0]=\"DoOrWhileStatementExcludes\",e[e.ForStatementIncludes=1280]=\"ForStatementIncludes\",e[e.ForStatementExcludes=3008]=\"ForStatementExcludes\",e[e.ForInOrForOfStatementIncludes=2304]=\"ForInOrForOfStatementIncludes\",e[e.ForInOrForOfStatementExcludes=1984]=\"ForInOrForOfStatementExcludes\",e[e.BlockIncludes=128]=\"BlockIncludes\",e[e.BlockExcludes=3904]=\"BlockExcludes\",e[e.IterationStatementBlockIncludes=512]=\"IterationStatementBlockIncludes\",e[e.IterationStatementBlockExcludes=4032]=\"IterationStatementBlockExcludes\",e[e.ComputedPropertyNameIncludes=8192]=\"ComputedPropertyNameIncludes\",e[e.ComputedPropertyNameExcludes=0]=\"ComputedPropertyNameExcludes\",e[e.NewTarget=16384]=\"NewTarget\",e[e.NewTargetInComputedPropertyName=32768]=\"NewTargetInComputedPropertyName\",e[e.SubtreeFactsMask=-16384]=\"SubtreeFactsMask\",e[e.PropagateNewTargetMask=49152]=\"PropagateNewTargetMask\"}(i||(i={})),e.transformES2015=function(n){var t,r,a,i,l,c,u=n.startLexicalEnvironment,d=n.resumeLexicalEnvironment,m=n.endLexicalEnvironment,p=n.hoistVariableDeclaration,f=n.getCompilerOptions(),g=n.getEmitResolver(),_=n.onSubstituteNode,v=n.onEmitNode;return n.onEmitNode=function(n,t,r){if(1&c&&e.isFunctionLike(t)){var a=y(16286,8&e.getEmitFlags(t)?81:65);return v(n,t,r),void h(a,0,0)}v(n,t,r)},n.onSubstituteNode=function(n,t){return t=_(n,t),1===n?function(n){switch(n.kind){case 71:return function(n){if(2&c&&!e.isInternalName(n)){var t=g.getReferencedDeclarationWithCollidingName(n);if(t&&(!e.isClassLike(t)||!function(n,t){var r=e.getParseTreeNode(t);if(!r||r===n||r.end<=n.pos||r.pos>=n.end)return!1;for(var a=e.getEnclosingBlockScopeContainer(n);r;){if(r===a||r===n)return!1;if(e.isClassElement(r)&&r.parent===n)return!0;r=r.parent}return!1}(t,n)))return e.setTextRange(e.getGeneratedNameForNode(e.getNameOfDeclaration(t)),n)}return n}(n);case 99:return function(n){return 1&c&&16&a?e.setTextRange(e.createFileLevelUniqueName(\"_this\"),n):n}(n)}return n}(t):e.isIdentifier(t)?function(n){if(2&c&&!e.isInternalName(n)){var t=e.getParseTreeNode(n,e.isIdentifier);if(t&&function(e){switch(e.parent.kind){case 184:case 238:case 241:case 235:return e.parent.name===e&&g.isDeclarationWithCollidingName(e.parent)}return!1}(t))return e.setTextRange(e.getGeneratedNameForNode(t),n)}return n}(t):t},e.chainBundle(function(o){if(o.isDeclarationFile)return o;t=o,r=o.text;var s=function(n){var t=y(3968,64),r=[];u();var a=e.addStandardPrologue(r,n.statements,!1);return R(r,n),a=e.addCustomPrologue(r,n.statements,a,T),e.addRange(r,e.visitNodes(n.statements,T,e.isStatement,a)),i&&r.push(e.createVariableStatement(void 0,e.createVariableDeclarationList(i))),e.addStatementsAfterPrologue(r,m()),h(t,0,0),e.updateSourceFileNode(n,e.setTextRange(e.createNodeArray(r),n.statements))}(o);return e.addEmitHelpers(s,n.readEmitHelpers()),t=void 0,r=void 0,i=void 0,a=0,s});function y(e,n){var t=a;return a=16383&(a&~e|n),t}function h(e,n,t){a=-16384&(a&~n|t)|e}function b(e){return 0!=(4096&a)&&228===e.kind&&!e.expression}function E(n){return 0!=(128&n.transformFlags)||void 0!==l||4096&a&&(e.isStatement(n)||216===n.kind)||e.isIterationStatement(n,!1)&&ie(n)||0!=(33554432&e.getEmitFlags(n))}function T(r){return E(r)?function(r){switch(r.kind){case 115:return;case 238:return function(n){var t=e.createVariableDeclaration(e.getLocalName(n,!0),void 0,A(n));e.setOriginalNode(t,n);var r=[],a=e.createVariableStatement(void 0,e.createVariableDeclarationList([t]));if(e.setOriginalNode(a,n),e.setTextRange(a,n),e.startOnNewLine(a),r.push(a),e.hasModifier(n,1)){var i=e.hasModifier(n,512)?e.createExportDefault(e.getLocalName(n)):e.createExternalModuleExport(e.getLocalName(n));e.setOriginalNode(i,a),r.push(i)}var o=e.getEmitFlags(n);return 0==(4194304&o)&&(r.push(e.createEndOfDeclarationMarker(n)),e.setEmitFlags(a,4194304|o)),e.singleOrMany(r)}(r);case 207:return function(e){return A(e)}(r);case 149:return function(n){return n.dotDotDotToken?void 0:e.isBindingPattern(n.name)?e.setOriginalNode(e.setTextRange(e.createParameter(void 0,void 0,void 0,e.getGeneratedNameForNode(n),void 0,void 0,void 0),n),n):n.initializer?e.setOriginalNode(e.setTextRange(e.createParameter(void 0,void 0,void 0,n.name,void 0,void 0,void 0),n),n):n}(r);case 237:return function(t){var r=l;l=void 0;var i=y(16286,65),o=e.visitParameterList(t.parameters,T,n),s=64&t.transformFlags?B(t):H(t),c=16384&a?e.getLocalName(t):t.name;return h(i,49152,0),l=r,e.updateFunctionDeclaration(t,void 0,e.visitNodes(t.modifiers,T,e.isModifier),t.asteriskToken,c,void 0,o,void 0,s)}(r);case 195:return function(t){16384&t.transformFlags&&Le();var r=l;l=void 0;var a=y(16256,66),i=e.createFunctionExpression(void 0,void 0,void 0,void 0,e.visitParameterList(t.parameters,T,n),void 0,B(t));return e.setTextRange(i,t),e.setOriginalNode(i,t),e.setEmitFlags(i,8),h(a,0,0),l=r,i}(r);case 194:return function(t){var r=262144&e.getEmitFlags(t)?y(16278,69):y(16286,65),i=l;l=void 0;var o=e.visitParameterList(t.parameters,T,n),s=64&t.transformFlags?B(t):H(t),c=16384&a?e.getLocalName(t):t.name;return h(r,49152,0),l=i,e.updateFunctionExpression(t,void 0,t.asteriskToken,c,void 0,o,void 0,s)}(r);case 235:return z(r);case 71:return function(n){return l?e.isGeneratedIdentifier(n)?n:\"arguments\"===n.escapedText&&g.isArgumentsLocalBinding(n)?l.argumentsName||(l.argumentsName=e.createUniqueName(\"arguments\")):n:n}(r);case 236:return function(t){if(64&t.transformFlags){3&t.flags&&Se();var r=e.flatMap(t.declarations,1&t.flags?q:z),a=e.createVariableDeclarationList(r);return e.setOriginalNode(a,t),e.setTextRange(a,t),e.setCommentRange(a,t),8388608&t.transformFlags&&(e.isBindingPattern(t.declarations[0].name)||e.isBindingPattern(e.last(t.declarations).name))&&e.setSourceMapRange(a,function(n){for(var t=-1,r=-1,a=0,i=n;a<i.length;a++){var o=i[a];t=-1===t?o.pos:-1===o.pos?t:Math.min(t,o.pos),r=Math.max(r,o.end)}return e.createRange(t,r)}(r)),a}return e.visitEachChild(t,T,n)}(r);case 230:return function(t){if(void 0!==l){var r=l.allowedNonLabeledJumps;l.allowedNonLabeledJumps|=2;var a=e.visitEachChild(t,T,n);return l.allowedNonLabeledJumps=r,a}return e.visitEachChild(t,T,n)}(r);case 244:return function(t){var r=y(4032,0),a=e.visitEachChild(t,T,n);return h(r,0,0),a}(r);case 216:return U(r,!1);case 227:case 226:return function(t){if(l){var r=227===t.kind?2:4,a=t.label&&l.labels&&l.labels.get(e.idText(t.label))||!t.label&&l.allowedNonLabeledJumps&r;if(!a){var i=void 0,o=t.label;o?227===t.kind?(i=\"break-\"+o.escapedText,ce(l,!0,e.idText(o),i)):(i=\"continue-\"+o.escapedText,ce(l,!1,e.idText(o),i)):227===t.kind?(l.nonLocalJumps|=2,i=\"break\"):(l.nonLocalJumps|=4,i=\"continue\");var s=e.createLiteral(i);if(l.loopOutParameters.length){for(var c=l.loopOutParameters,u=void 0,d=0;d<c.length;d++){var m=se(c[d],1);u=0===d?m:e.createBinary(u,26,m)}s=e.createBinary(u,26,s)}return e.createReturn(s)}}return e.visitEachChild(t,T,n)}(r);case 231:return function(n){l&&!l.labels&&(l.labels=e.createMap());var t=e.unwrapInnermostStatementOfLabel(n,l&&J);return e.isIterationStatement(t,!1)?function(e,n){switch(e.kind){case 221:case 222:return Q(e,n);case 223:return Z(e,n);case 224:return $(e,n);case 225:return ee(e,n)}}(t,n):e.restoreEnclosingLabel(e.visitNode(t,T,e.isStatement,e.liftToBlock),n,l&&X)}(r);case 221:case 222:return Q(r,void 0);case 223:return Z(r,void 0);case 224:return $(r,void 0);case 225:return ee(r,void 0);case 219:return function(t){switch(t.expression.kind){case 193:return e.updateExpressionStatement(t,j(t.expression,!1));case 202:return e.updateExpressionStatement(t,W(t.expression,!1))}return e.visitEachChild(t,T,n)}(r);case 186:return function(t){for(var r=t.properties,i=r.length,o=i,s=i,l=0;l<i;l++){var c=r[l];if(16777216&c.transformFlags&&4&a&&l<s&&(s=l),147===c.name.kind){o=l;break}}if(o!==i){s<o&&(o=s);var u=e.createTempVariable(p),d=[],m=e.createAssignment(u,e.setEmitFlags(e.createObjectLiteral(e.visitNodes(r,T,e.isObjectLiteralElementLike,0,o),t.multiLine),65536));return t.multiLine&&e.startOnNewLine(m),d.push(m),function(n,t,r,a){for(var i=t.properties,o=i.length,s=a;s<o;s++){var l=i[s];switch(l.kind){case 156:case 157:var c=e.getAllAccessorDeclarations(t.properties,l);l===c.firstAccessor&&n.push(V(r,c,t,!!t.multiLine));break;case 154:n.push(fe(l,r,t,t.multiLine));break;case 273:n.push(me(l,r,t.multiLine));break;case 274:n.push(pe(l,r,t.multiLine));break;default:e.Debug.failBadSyntaxKind(t)}}}(d,t,u,o),d.push(t.multiLine?e.startOnNewLine(e.getMutableClone(u)):u),e.inlineExpressions(d)}return e.visitEachChild(t,T,n)}(r);case 272:return function(t){var r,a,i,o,s=y(4032,0);if(e.Debug.assert(!!t.variableDeclaration,\"Catch clause variable should always be present when downleveling ES2015.\"),e.isBindingPattern(t.variableDeclaration.name)){var l=e.createTempVariable(void 0),c=e.createVariableDeclaration(l);e.setTextRange(c,t.variableDeclaration);var u=e.flattenDestructuringBinding(t.variableDeclaration,T,n,0,l),d=e.createVariableDeclarationList(u);e.setTextRange(d,t.variableDeclaration);var m=e.createVariableStatement(void 0,d);r=e.updateCatchClause(t,c,(a=t.block,i=m,o=e.visitNodes(a.statements,T,e.isStatement),e.updateBlock(a,[i].concat(o))))}else r=e.visitEachChild(t,T,n);return h(s,0,0),r}(r);case 274:return function(n){return e.setTextRange(e.createPropertyAssignment(n.name,e.getSynthesizedClone(n.name)),n)}(r);case 147:return function(t){var r=y(0,8192),i=e.visitEachChild(t,T,n);return h(r,49152,49152&a?32768:0),i}(r);case 185:return function(t){return 64&t.transformFlags?_e(t.elements,!0,!!t.multiLine,!!t.elements.hasTrailingComma):e.visitEachChild(t,T,n)}(r);case 189:return function(n){return 33554432&e.getEmitFlags(n)?function(n){var t=e.cast(e.cast(e.skipOuterExpressions(n.expression),e.isArrowFunction).body,e.isBlock),r=function(n){return e.isVariableStatement(n)&&!!e.first(n.declarationList.declarations).initializer},a=e.visitNodes(t.statements,T,e.isStatement),i=e.filter(a,r),o=e.filter(a,function(e){return!r(e)}),s=e.cast(e.first(i),e.isVariableStatement).declarationList.declarations[0],l=e.skipOuterExpressions(s.initializer),c=e.tryCast(l,e.isAssignmentExpression),u=e.cast(c?e.skipOuterExpressions(c.right):l,e.isCallExpression),d=e.cast(e.skipOuterExpressions(u.expression),e.isFunctionExpression),m=d.body.statements,p=0,f=-1,g=[];if(c){var _=e.tryCast(m[p],e.isExpressionStatement);_&&(g.push(_),p++),g.push(m[p]),p++,g.push(e.createExpressionStatement(e.createAssignment(c.left,e.cast(s.name,e.isIdentifier))))}for(;!e.isReturnStatement(e.elementAt(m,f));)f--;return e.addRange(g,m,p,f),f<-1&&e.addRange(g,m,f+1),e.addRange(g,o),e.addRange(g,i,1),e.recreateOuterExpressions(n.expression,e.recreateOuterExpressions(s.initializer,e.recreateOuterExpressions(c&&c.right,e.updateCall(u,e.recreateOuterExpressions(u.expression,e.updateFunctionExpression(d,void 0,void 0,void 0,void 0,d.parameters,void 0,e.updateBlock(d.body,g))),void 0,u.arguments))))}(n):64&n.transformFlags?ge(n,!0):e.updateCall(n,e.visitNode(n.expression,L,e.isExpression),void 0,e.visitNodes(n.arguments,T,e.isExpression))}(r);case 190:return function(t){if(524288&t.transformFlags){var r=e.createCallBinding(e.createPropertyAccess(t.expression,\"bind\"),p),a=r.target,i=r.thisArg;return e.createNew(e.createFunctionApply(e.visitNode(a,T,e.isExpression),i,_e(e.createNodeArray([e.createVoidZero()].concat(t.arguments)),!1,!1,!1)),void 0,[])}return e.visitEachChild(t,T,n)}(r);case 193:return j(r,!0);case 202:return W(r,!0);case 13:case 14:case 15:case 16:return function(n){return e.setTextRange(e.createLiteral(n.text),n)}(r);case 9:return function(n){return n.hasExtendedUnicodeEscape?e.setTextRange(e.createLiteral(n.text),n):n}(r);case 8:return function(n){return 384&n.numericLiteralFlags?e.setTextRange(e.createNumericLiteral(n.text),n):n}(r);case 191:return function(r){var a=e.visitNode(r.tag,T,e.isExpression),o=[void 0],l=[],c=[],u=r.template;if(e.isNoSubstitutionTemplateLiteral(u))l.push(e.createLiteral(u.text)),c.push(Ee(u));else{l.push(e.createLiteral(u.head.text)),c.push(Ee(u.head));for(var d=0,m=u.templateSpans;d<m.length;d++){var p=m[d];l.push(e.createLiteral(p.literal.text)),c.push(Ee(p.literal)),o.push(e.visitNode(p.expression,T,e.isExpression))}}var f,g=function(n,t,r){return n.requestEmitHelper(s),e.createCall(e.getHelperName(\"__makeTemplateObject\"),void 0,[t,r])}(n,e.createArrayLiteral(l),e.createArrayLiteral(c));if(e.isExternalModule(t)){var _=e.createUniqueName(\"templateObject\");f=_,i=e.append(i,e.createVariableDeclaration(f)),o[0]=e.createLogicalOr(_,e.createAssignment(_,g))}else o[0]=g;return e.createCall(a,void 0,o)}(r);case 204:return function(n){var t=[];(function(n,t){(function(n){return e.Debug.assert(0!==n.templateSpans.length),0!==n.head.text.length||0===n.templateSpans[0].literal.text.length})(t)&&n.push(e.createLiteral(t.head.text))})(t,n),function(n,t){for(var r=0,a=t.templateSpans;r<a.length;r++){var i=a[r];n.push(e.visitNode(i.expression,T,e.isExpression)),0!==i.literal.text.length&&n.push(e.createLiteral(i.literal.text))}}(t,n);var r=e.reduceLeft(t,e.createAdd);return e.nodeIsSynthesized(r)&&(r.pos=n.pos,r.end=n.end),r}(r);case 205:return function(t){return e.visitEachChild(t,T,n)}(r);case 206:return function(n){return e.visitNode(n.expression,T,e.isExpression)}(r);case 97:return Te(!1);case 99:return function(n){return l?2&a?(l.containsLexicalThis=!0,n):l.thisName||(l.thisName=e.createUniqueName(\"this\")):n}(r);case 212:return function(n){return 94===n.keywordToken&&\"target\"===n.name.escapedText?(a|=8192&a?32768:16384,e.createFileLevelUniqueName(\"_newTarget\")):n}(r);case 154:return function(n){e.Debug.assert(!e.isComputedPropertyName(n.name));var t=K(n,e.moveRangePos(n,-1),void 0,void 0);return e.setEmitFlags(t,512|e.getEmitFlags(t)),e.setTextRange(e.createPropertyAssignment(n.name,t),n)}(r);case 156:case 157:return function(t){e.Debug.assert(!e.isComputedPropertyName(t.name));var r=l;l=void 0;var a,i=y(16286,65),o=e.visitParameterList(t.parameters,T,n),s=32896&t.transformFlags?B(t):H(t);return a=156===t.kind?e.updateGetAccessor(t,t.decorators,t.modifiers,t.name,o,t.type,s):e.updateSetAccessor(t,t.decorators,t.modifiers,t.name,o,s),h(i,49152,0),l=r,a}(r);case 217:return function(t){var r,a=y(0,e.hasModifier(t,1)?32:0);if(l&&0==(3&t.declarationList.flags)){for(var i=void 0,o=0,s=t.declarationList.declarations;o<s.length;o++){var c=s[o];if(oe(l,c),c.initializer){var u=void 0;e.isBindingPattern(c.name)?u=e.flattenDestructuringAssignment(c,T,n,0):(u=e.createBinary(c.name,58,e.visitNode(c.initializer,T,e.isExpression)),e.setTextRange(u,c)),i=e.append(i,u)}}r=i?e.setTextRange(e.createExpressionStatement(e.inlineExpressions(i)),t):void 0}else r=e.visitEachChild(t,T,n);return h(a,0,0),r}(r);case 228:return function(t){return l?(l.nonLocalJumps|=8,b(t)&&(t=x(t)),e.createReturn(e.createObjectLiteral([e.createPropertyAssignment(e.createIdentifier(\"value\"),t.expression?e.visitNode(t.expression,T,e.isExpression):e.createVoidZero())]))):b(t)?x(t):e.visitEachChild(t,T,n)}(r);default:return e.visitEachChild(r,T,n)}}(r):r}function S(e){return E(e)?U(e,!0):e}function L(e){return 97===e.kind?Te(!0):T(e)}function x(n){return e.setOriginalNode(e.createReturn(e.createFileLevelUniqueName(\"_this\")),n)}function A(t){t.name&&Se();var i=e.getEffectiveBaseTypeNode(t),s=e.createFunctionExpression(void 0,void 0,void 0,void 0,i?[e.createParameter(void 0,void 0,void 0,e.createFileLevelUniqueName(\"_super\"))]:[],void 0,function(t,i){var s=[];u(),function(t,r,a){a&&t.push(e.setTextRange(e.createExpressionStatement(function(n,t){return n.requestEmitHelper(o),e.createCall(e.getHelperName(\"__extends\"),void 0,[t,e.createFileLevelUniqueName(\"_super\")])}(n,e.getInternalName(r))),a))}(s,t,i),function(t,r,i){var o=l;l=void 0;var s=y(16278,73),c=e.getFirstConstructorWithBody(r),u=function(n,t){if(!n||!t)return!1;if(e.some(n.parameters))return!1;var r=e.firstOrUndefined(n.body.statements);if(!r||!e.nodeIsSynthesized(r)||219!==r.kind)return!1;var a=r.expression;if(!e.nodeIsSynthesized(a)||189!==a.kind)return!1;var i=a.expression;if(!e.nodeIsSynthesized(i)||97!==i.kind)return!1;var o=e.singleOrUndefined(a.arguments);if(!o||!e.nodeIsSynthesized(o)||206!==o.kind)return!1;var s=o.expression;return e.isIdentifier(s)&&\"arguments\"===s.escapedText}(c,void 0!==i),p=e.createFunctionDeclaration(void 0,void 0,void 0,e.getInternalName(r),void 0,function(t,r){return e.visitParameterList(t&&!r?t.parameters:void 0,T,n)||[]}(c,u),void 0,function(n,t,r,i){var o=[];d();var s=-1;i?s=0:n&&(s=e.addStandardPrologue(o,n.body.statements,!1)),n&&(k(o,n),I(o,n,i),i||(s=e.addCustomPrologue(o,n.body.statements,s,T)),e.Debug.assert(s>=0,\"statementOffset not initialized correctly!\"));var l=!!r&&95!==e.skipOuterExpressions(r.expression).kind,c=function(n,t,r,a,i){if(!r)return t&&R(n,t),0;if(!t)return n.push(e.createReturn(D())),2;if(a)return N(n,t,D()),Le(),1;var o,s,l,c=t.body.statements;if(i<c.length&&219===(o=c[i]).kind&&e.isSuperCall(o.expression)&&(l=o.expression,s=ge(l,!1)),s&&i===c.length-1&&!(49152&t.transformFlags)){var u=e.createReturn(s);return 202===s.kind&&189===s.left.kind||e.Debug.fail(\"Assumed generated super call would have form 'super.call(...) || this'.\"),e.setCommentRange(u,e.getCommentRange(e.setEmitFlags(s.left,1536))),n.push(u),2}return N(n,t,s||C()),s?1:0}(o,n,l,i,s);1!==c&&2!==c||s++,n&&(1===c&&(a|=4096),e.addRange(o,e.visitNodes(n.body.statements,T,e.isStatement,s))),!l||2===c||n&&function n(t){if(228===t.kind)return!0;if(220===t.kind){var r=t;if(r.elseStatement)return n(r.thenStatement)&&n(r.elseStatement)}else if(216===t.kind){var a=e.lastOrUndefined(t.statements);if(a&&n(a))return!0}return!1}(n.body)||o.push(e.createReturn(e.createFileLevelUniqueName(\"_this\"))),e.addStatementsAfterPrologue(o,m()),n&&w(o,n,!1);var u=e.createBlock(e.setTextRange(e.createNodeArray(o),n?n.body.statements:t.members),!0);return e.setTextRange(u,n?n.body:t),n||e.setEmitFlags(u,1536),u}(c,r,i,u));e.setTextRange(p,c||r),i&&e.setEmitFlags(p,8),t.push(p),h(s,49152,0),l=o}(s,t,i),function(n,t){for(var r=0,a=t.members;r<a.length;r++){var i=a[r];switch(i.kind){case 215:n.push(P(i));break;case 154:n.push(F(xe(t,i),i,t));break;case 156:case 157:var o=e.getAllAccessorDeclarations(t.members,i);i===o.firstAccessor&&n.push(G(xe(t,i),o,t));break;case 155:break;default:e.Debug.failBadSyntaxKind(t)}}}(s,t);var c=e.createTokenRange(e.skipTrivia(r,t.members.end),18),p=e.getInternalName(t),f=e.createPartiallyEmittedExpression(p);f.end=c.end,e.setEmitFlags(f,1536);var g=e.createReturn(f);g.pos=c.pos,e.setEmitFlags(g,1920),s.push(g),e.addStatementsAfterPrologue(s,m());var _=e.createBlock(e.setTextRange(e.createNodeArray(s),t.members),!0);return e.setEmitFlags(_,1536),_}(t,i));e.setEmitFlags(s,65536&e.getEmitFlags(t)|524288);var c=e.createPartiallyEmittedExpression(s);c.end=t.end,e.setEmitFlags(c,1536);var p=e.createPartiallyEmittedExpression(c);p.end=e.skipTrivia(r,t.pos),e.setEmitFlags(p,1536);var f=e.createParen(e.createCall(p,void 0,i?[e.visitNode(i.expression,T,e.isExpression)]:[]));return e.addSyntheticLeadingComment(f,3,\"* @class \"),f}function C(){return e.setEmitFlags(e.createThis(),4)}function D(){return e.createLogicalOr(e.createLogicalAnd(e.createStrictInequality(e.createFileLevelUniqueName(\"_super\"),e.createNull()),e.createFunctionApply(e.createFileLevelUniqueName(\"_super\"),C(),e.createIdentifier(\"arguments\"))),C())}function k(n,t){if(function(e){return 0!=(131072&e.transformFlags)}(t))for(var r=0,a=t.parameters;r<a.length;r++){var i=a[r],o=i.name,s=i.initializer;i.dotDotDotToken||(e.isBindingPattern(o)?M(n,i,o,s):s&&O(n,i,o,s))}}function M(t,r,a,i){var o=e.getGeneratedNameForNode(r);a.elements.length>0?t.push(e.setEmitFlags(e.createVariableStatement(void 0,e.createVariableDeclarationList(e.flattenDestructuringBinding(r,T,n,0,o))),1048576)):i&&t.push(e.setEmitFlags(e.createExpressionStatement(e.createAssignment(o,e.visitNode(i,T,e.isExpression))),1048576))}function O(n,t,r,a){a=e.visitNode(a,T,e.isExpression);var i=e.createIf(e.createTypeCheck(e.getSynthesizedClone(r),\"undefined\"),e.setEmitFlags(e.setTextRange(e.createBlock([e.createExpressionStatement(e.setEmitFlags(e.setTextRange(e.createAssignment(e.setEmitFlags(e.getMutableClone(r),48),e.setEmitFlags(a,1584|e.getEmitFlags(a))),t),1536))]),t),1953));e.startOnNewLine(i),e.setTextRange(i,t),e.setEmitFlags(i,1050528),n.push(i)}function I(n,t,r){var a=e.lastOrUndefined(t.parameters);if(function(e,n){return e&&e.dotDotDotToken&&71===e.name.kind&&!n}(a,r)){var i=e.getMutableClone(a.name);e.setEmitFlags(i,48);var o=e.getSynthesizedClone(a.name),s=t.parameters.length-1,l=e.createLoopVariable();n.push(e.setEmitFlags(e.setTextRange(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(i,void 0,e.createArrayLiteral([]))])),a),1048576));var c=e.createFor(e.setTextRange(e.createVariableDeclarationList([e.createVariableDeclaration(l,void 0,e.createLiteral(s))]),a),e.setTextRange(e.createLessThan(l,e.createPropertyAccess(e.createIdentifier(\"arguments\"),\"length\")),a),e.setTextRange(e.createPostfixIncrement(l),a),e.createBlock([e.startOnNewLine(e.setTextRange(e.createExpressionStatement(e.createAssignment(e.createElementAccess(o,0===s?l:e.createSubtract(l,e.createLiteral(s))),e.createElementAccess(e.createIdentifier(\"arguments\"),l))),a))]));e.setEmitFlags(c,1048576),e.startOnNewLine(c),n.push(c)}}function R(n,t){32768&t.transformFlags&&195!==t.kind&&N(n,t,e.createThis())}function N(n,t,r){Le();var a=e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(e.createFileLevelUniqueName(\"_this\"),void 0,r)]));e.setEmitFlags(a,1050112),e.setSourceMapRange(a,t),n.push(a)}function w(n,t,r){if(16384&a){var i=void 0;switch(t.kind){case 195:return n;case 154:case 156:case 157:i=e.createVoidZero();break;case 155:i=e.createPropertyAccess(e.setEmitFlags(e.createThis(),4),\"constructor\");break;case 237:case 194:i=e.createConditional(e.createLogicalAnd(e.setEmitFlags(e.createThis(),4),e.createBinary(e.setEmitFlags(e.createThis(),4),93,e.getLocalName(t))),e.createPropertyAccess(e.setEmitFlags(e.createThis(),4),\"constructor\"),e.createVoidZero());break;default:return e.Debug.failBadSyntaxKind(t)}var o=e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(e.createFileLevelUniqueName(\"_newTarget\"),void 0,i)]));if(r)return[o].concat(n);n.unshift(o)}return n}function P(n){return e.setTextRange(e.createEmptyStatement(),n)}function F(n,t,r){var i=y(0,0),o=e.getCommentRange(t),s=e.getSourceMapRange(t),l=e.createMemberAccessForPropertyName(n,e.visitNode(t.name,T,e.isPropertyName),t.name),c=K(t,t,void 0,r);e.setEmitFlags(c,1536),e.setSourceMapRange(c,s);var u=e.setTextRange(e.createExpressionStatement(e.createAssignment(l,c)),t);return e.setOriginalNode(u,t),e.setCommentRange(u,o),e.setEmitFlags(u,48),h(i,49152,49152&a?16384:0),u}function G(n,t,r){var a=e.createExpressionStatement(V(n,t,r,!1));return e.setEmitFlags(a,1536),e.setSourceMapRange(a,e.getSourceMapRange(t.firstAccessor)),a}function V(n,t,r,i){var o=t.firstAccessor,s=t.getAccessor,l=t.setAccessor,c=y(0,0),u=e.getMutableClone(n);e.setEmitFlags(u,1568),e.setSourceMapRange(u,o.name);var d=e.createExpressionForPropertyName(e.visitNode(o.name,T,e.isPropertyName));e.setEmitFlags(d,1552),e.setSourceMapRange(d,o.name);var m=[];if(s){var p=K(s,void 0,void 0,r);e.setSourceMapRange(p,e.getSourceMapRange(s)),e.setEmitFlags(p,512);var f=e.createPropertyAssignment(\"get\",p);e.setCommentRange(f,e.getCommentRange(s)),m.push(f)}if(l){var g=K(l,void 0,void 0,r);e.setSourceMapRange(g,e.getSourceMapRange(l)),e.setEmitFlags(g,512);var _=e.createPropertyAssignment(\"set\",g);e.setCommentRange(_,e.getCommentRange(l)),m.push(_)}m.push(e.createPropertyAssignment(\"enumerable\",e.createTrue()),e.createPropertyAssignment(\"configurable\",e.createTrue()));var v=e.createCall(e.createPropertyAccess(e.createIdentifier(\"Object\"),\"defineProperty\"),void 0,[u,d,e.createObjectLiteral(m,!0)]);return i&&e.startOnNewLine(v),h(c,49152,49152&a?16384:0),v}function K(t,r,i,o){var s=l;l=void 0;var c=o&&e.isClassLike(o)&&!e.hasModifier(t,32)?y(16286,73):y(16286,65),u=e.visitParameterList(t.parameters,T,n),d=B(t);return 16384&a&&!i&&(237===t.kind||194===t.kind)&&(i=e.getGeneratedNameForNode(t)),h(c,49152,0),l=s,e.setOriginalNode(e.setTextRange(e.createFunctionExpression(void 0,t.asteriskToken,i,void 0,u,void 0,d),r),t)}function B(r){var a,i,o,s=!1,l=!1,c=[],u=[],m=r.body;if(d(),e.isBlock(m)&&(o=e.addStandardPrologue(c,m.statements,!1)),R(c,r),k(c,r),I(c,r,!1),e.isBlock(m))o=e.addCustomPrologue(c,m.statements,o,T),a=m.statements,e.addRange(u,e.visitNodes(m.statements,T,e.isStatement,o)),!s&&m.multiLine&&(s=!0);else{e.Debug.assert(195===r.kind),a=e.moveRangeEnd(m,-1);var p=r.equalsGreaterThanToken;e.nodeIsSynthesized(p)||e.nodeIsSynthesized(m)||(e.rangeEndIsOnSameLineAsRangeStart(p,m,t)?l=!0:s=!0);var f=e.visitNode(m,T,e.isExpression),g=e.createReturn(f);e.setTextRange(g,m),e.moveSyntheticComments(g,m),e.setEmitFlags(g,1440),u.push(g),i=m}var _=n.endLexicalEnvironment();e.addStatementsAfterPrologue(u,_),w(u,r,!1),(e.some(c)||e.some(_))&&(s=!0);var v=e.createBlock(e.setTextRange(e.createNodeArray(c.concat(u)),a),s);return e.setTextRange(v,r.body),!s&&l&&e.setEmitFlags(v,1),i&&e.setTokenSourceMapRange(v,18,i),e.setOriginalNode(v,r.body),v}function H(t){var r=e.visitFunctionBody(t.body,S,n);return e.updateBlock(r,e.setTextRange(e.createNodeArray(w(r.statements,t,!0)),r.statements))}function U(t,r){if(r)return e.visitEachChild(t,T,n);var i=256&a?y(4032,512):y(3904,128),o=e.visitEachChild(t,T,n);return h(i,0,0),o}function j(t,r){if(!r)switch(t.expression.kind){case 193:return e.updateParen(t,j(t.expression,!1));case 202:return e.updateParen(t,W(t.expression,!1))}return e.visitEachChild(t,T,n)}function W(t,r){return e.isDestructuringAssignment(t)?e.flattenDestructuringAssignment(t,T,n,0,r):e.visitEachChild(t,T,n)}function q(t){var r=t.name;if(e.isBindingPattern(r))return z(t);if(!t.initializer&&function(e){var n=g.getNodeCheckFlags(e),t=131072&n,r=262144&n;return!(0!=(64&a)||t&&r&&0!=(512&a))&&0==(2048&a)&&(!g.isDeclarationWithCollidingName(e)||r&&!t&&0==(3072&a))}(t)){var i=e.getMutableClone(t);return i.initializer=e.createVoidZero(),i}return e.visitEachChild(t,T,n)}function z(t){var r,a=y(32,0);return r=e.isBindingPattern(t.name)?e.flattenDestructuringBinding(t,T,n,0,void 0,0!=(32&a)):e.visitEachChild(t,T,n),h(a,0,0),r}function J(n){l.labels.set(e.idText(n.label),!0)}function X(n){l.labels.set(e.idText(n.label),!1)}function Y(t,r,i,o,s){var c=y(t,r),d=function(t,r,i){if(!ie(t)){var o=void 0;l&&(o=l.allowedNonLabeledJumps,l.allowedNonLabeledJumps=6);var s=i?i(t,r,void 0):e.restoreEnclosingLabel(e.visitEachChild(t,T,n),r,l&&X);return l&&(l.allowedNonLabeledJumps=o),s}var c,d=e.createUniqueName(\"_loop\");switch(t.kind){case 223:case 224:case 225:var p=t.initializer;p&&236===p.kind&&(c=p)}var f=[],g=[];if(c&&3&e.getCombinedNodeFlags(c))for(var _=0,v=c.declarations;_<v.length;_++){var y=v[_];de(y,f,g)}var h=l;l={loopOutParameters:g},h&&(h.argumentsName&&(l.argumentsName=h.argumentsName),h.thisName&&(l.thisName=h.thisName),h.hoistedLocalVariables&&(l.hoistedLocalVariables=h.hoistedLocalVariables)),u();var b=e.visitNode(t.statement,T,e.isStatement,e.liftToBlock),E=m(),S=l;if(l=h,g.length||E){var L=e.isBlock(b)?b.statements.slice():[b];g.length&&le(g,1,L),e.addStatementsAfterPrologue(L,E),b=e.createBlock(L,!0)}e.isBlock(b)?b.multiLine=!0:b=e.createBlock([b],!0);var x=0!=(16777216&t.statement.transformFlags),A=x&&0!=(4&a),C=0;S.containsLexicalThis&&(C|=8),A&&(C|=262144);var D,k=[e.createVariableStatement(void 0,e.setEmitFlags(e.createVariableDeclarationList([e.createVariableDeclaration(d,void 0,e.setEmitFlags(e.createFunctionExpression(void 0,x?e.createToken(39):void 0,void 0,void 0,f,void 0,b),C))]),2097152))];if(S.argumentsName&&(h?h.argumentsName=S.argumentsName:(D||(D=[])).push(e.createVariableDeclaration(S.argumentsName,void 0,e.createIdentifier(\"arguments\")))),S.thisName&&(h?h.thisName=S.thisName:(D||(D=[])).push(e.createVariableDeclaration(S.thisName,void 0,e.createIdentifier(\"this\")))),S.hoistedLocalVariables)if(h)h.hoistedLocalVariables=S.hoistedLocalVariables;else{D||(D=[]);for(var M=0,O=S.hoistedLocalVariables;M<O.length;M++){var I=O[M];D.push(e.createVariableDeclaration(I))}}if(g.length){D||(D=[]);for(var R=0,N=g;R<N.length;R++){var w=N[R];D.push(e.createVariableDeclaration(w.outParamName))}}D&&k.push(e.createVariableStatement(void 0,e.createVariableDeclarationList(D)));var P,F=function(n,t,r,a){var i=l,o=[],s=!(-5&r.nonLocalJumps||r.labeledNonLocalBreaks||r.labeledNonLocalContinues),c=e.createCall(n,void 0,e.map(t,function(e){return e.name})),u=a?e.createYield(e.createToken(39),e.setEmitFlags(c,8388608)):c;if(s)o.push(e.createExpressionStatement(u)),le(r.loopOutParameters,0,o);else{var d=e.createUniqueName(\"state\"),m=e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(d,void 0,u)]));if(o.push(m),le(r.loopOutParameters,0,o),8&r.nonLocalJumps){var p=void 0;i?(i.nonLocalJumps|=8,p=e.createReturn(d)):p=e.createReturn(e.createPropertyAccess(d,\"value\")),o.push(e.createIf(e.createBinary(e.createTypeOf(d),34,e.createLiteral(\"object\")),p))}if(2&r.nonLocalJumps&&o.push(e.createIf(e.createBinary(d,34,e.createLiteral(\"break\")),e.createBreak())),r.labeledNonLocalBreaks||r.labeledNonLocalContinues){var f=[];ue(r.labeledNonLocalBreaks,!0,d,i,f),ue(r.labeledNonLocalContinues,!1,d,i,f),o.push(e.createSwitch(d,e.createCaseBlock(f)))}}return o}(d,f,S,x);if(i)P=i(t,r,F);else{var G=e.getMutableClone(t);G.statement=void 0,(G=e.visitEachChild(G,T,n)).statement=e.createBlock(F,!0),G.transformFlags=0,e.aggregateTransformFlags(G),P=e.restoreEnclosingLabel(G,r,l&&X)}return k.push(P),k}(i,o,s);return h(c,0,0),d}function Q(e,n){return Y(0,256,e,n)}function Z(e,n){return Y(3008,1280,e,n)}function $(e,n){return Y(1984,2304,e,n)}function ee(e,n){return Y(1984,2304,e,n,f.downlevelIteration?ae:re)}function ne(t,r,a){var i=[],o=t.initializer;if(e.isVariableDeclarationList(o)){3&t.initializer.flags&&Se();var s=e.firstOrUndefined(o.declarations);if(s&&e.isBindingPattern(s.name)){var l=e.flattenDestructuringBinding(s,T,n,0,r),c=e.setTextRange(e.createVariableDeclarationList(l),t.initializer);e.setOriginalNode(c,t.initializer),e.setSourceMapRange(c,e.createRange(l[0].pos,e.last(l).end)),i.push(e.createVariableStatement(void 0,c))}else i.push(e.setTextRange(e.createVariableStatement(void 0,e.setOriginalNode(e.setTextRange(e.createVariableDeclarationList([e.createVariableDeclaration(s?s.name:e.createTempVariable(void 0),void 0,r)]),e.moveRangePos(o,-1)),o)),e.moveRangeEnd(o,-1)))}else{var u=e.createAssignment(o,r);e.isDestructuringAssignment(u)?(e.aggregateTransformFlags(u),i.push(e.createExpressionStatement(W(u,!1)))):(u.end=o.end,i.push(e.setTextRange(e.createExpressionStatement(e.visitNode(u,T,e.isExpression)),e.moveRangeEnd(o,-1))))}if(a)return te(e.addRange(i,a));var d=e.visitNode(t.statement,T,e.isStatement,e.liftToBlock);return e.isBlock(d)?e.updateBlock(d,e.setTextRange(e.createNodeArray(e.concatenate(i,d.statements)),d.statements)):(i.push(d),te(i))}function te(n){return e.setEmitFlags(e.createBlock(e.createNodeArray(n),!0),432)}function re(n,t,r){var a=e.visitNode(n.expression,T,e.isExpression),i=e.createLoopVariable(),o=e.isIdentifier(a)?e.getGeneratedNameForNode(a):e.createTempVariable(void 0);e.setEmitFlags(a,48|e.getEmitFlags(a));var s=e.setTextRange(e.createFor(e.setEmitFlags(e.setTextRange(e.createVariableDeclarationList([e.setTextRange(e.createVariableDeclaration(i,void 0,e.createLiteral(0)),e.moveRangePos(n.expression,-1)),e.setTextRange(e.createVariableDeclaration(o,void 0,a),n.expression)]),n.expression),2097152),e.setTextRange(e.createLessThan(i,e.createPropertyAccess(o,\"length\")),n.expression),e.setTextRange(e.createPostfixIncrement(i),n.expression),ne(n,e.createElementAccess(o,i),r)),n);return e.setEmitFlags(s,256),e.setTextRange(s,n),e.restoreEnclosingLabel(s,t,l&&X)}function ae(t,r,a){var i=e.visitNode(t.expression,T,e.isExpression),o=e.isIdentifier(i)?e.getGeneratedNameForNode(i):e.createTempVariable(void 0),s=e.isIdentifier(i)?e.getGeneratedNameForNode(o):e.createTempVariable(void 0),c=e.createUniqueName(\"e\"),u=e.getGeneratedNameForNode(c),d=e.createTempVariable(void 0),m=e.createValuesHelper(n,i,t.expression),f=e.createCall(e.createPropertyAccess(o,\"next\"),void 0,[]);p(c),p(d);var g=e.setEmitFlags(e.setTextRange(e.createFor(e.setEmitFlags(e.setTextRange(e.createVariableDeclarationList([e.setTextRange(e.createVariableDeclaration(o,void 0,m),t.expression),e.createVariableDeclaration(s,void 0,f)]),t.expression),2097152),e.createLogicalNot(e.createPropertyAccess(s,\"done\")),e.createAssignment(s,f),ne(t,e.createPropertyAccess(s,\"value\"),a)),t),256);return e.createTry(e.createBlock([e.restoreEnclosingLabel(g,r,l&&X)]),e.createCatchClause(e.createVariableDeclaration(u),e.setEmitFlags(e.createBlock([e.createExpressionStatement(e.createAssignment(c,e.createObjectLiteral([e.createPropertyAssignment(\"error\",u)])))]),1)),e.createBlock([e.createTry(e.createBlock([e.setEmitFlags(e.createIf(e.createLogicalAnd(e.createLogicalAnd(s,e.createLogicalNot(e.createPropertyAccess(s,\"done\"))),e.createAssignment(d,e.createPropertyAccess(o,\"return\"))),e.createExpressionStatement(e.createFunctionCall(d,o,[]))),1)]),void 0,e.setEmitFlags(e.createBlock([e.setEmitFlags(e.createIf(c,e.createThrow(e.createPropertyAccess(c,\"error\"))),1)]),1))]))}function ie(e){return 0!=(65536&g.getNodeCheckFlags(e))}function oe(n,t){n.hoistedLocalVariables||(n.hoistedLocalVariables=[]),function t(r){if(71===r.kind)n.hoistedLocalVariables.push(r);else for(var a=0,i=r.elements;a<i.length;a++){var o=i[a];e.isOmittedExpression(o)||t(o.name)}}(t.name)}function se(n,t){var r=0===t?n.outParamName:n.originalName,a=0===t?n.originalName:n.outParamName;return e.createBinary(a,58,r)}function le(n,t,r){for(var a=0,i=n;a<i.length;a++){var o=i[a];r.push(e.createExpressionStatement(se(o,t)))}}function ce(n,t,r,a){t?(n.labeledNonLocalBreaks||(n.labeledNonLocalBreaks=e.createMap()),n.labeledNonLocalBreaks.set(r,a)):(n.labeledNonLocalContinues||(n.labeledNonLocalContinues=e.createMap()),n.labeledNonLocalContinues.set(r,a))}function ue(n,t,r,a,i){n&&n.forEach(function(n,o){var s=[];if(!a||a.labels&&a.labels.get(o)){var l=e.createIdentifier(o);s.push(t?e.createBreak(l):e.createContinue(l))}else ce(a,t,o,n),s.push(e.createReturn(r));i.push(e.createCaseClause(e.createLiteral(n),s))})}function de(n,t,r){var a=n.name;if(e.isBindingPattern(a))for(var i=0,o=a.elements;i<o.length;i++){var s=o[i];e.isOmittedExpression(s)||de(s,t,r)}else if(t.push(e.createParameter(void 0,void 0,void 0,a)),2097152&g.getNodeCheckFlags(n)){var l=e.createUniqueName(\"out_\"+e.idText(a));r.push({originalName:a,outParamName:l})}}function me(n,t,r){var a=e.createAssignment(e.createMemberAccessForPropertyName(t,e.visitNode(n.name,T,e.isPropertyName)),e.visitNode(n.initializer,T,e.isExpression));return e.setTextRange(a,n),r&&e.startOnNewLine(a),a}function pe(n,t,r){var a=e.createAssignment(e.createMemberAccessForPropertyName(t,e.visitNode(n.name,T,e.isPropertyName)),e.getSynthesizedClone(n.name));return e.setTextRange(a,n),r&&e.startOnNewLine(a),a}function fe(n,t,r,i){var o=y(0,0),s=e.createAssignment(e.createMemberAccessForPropertyName(t,e.visitNode(n.name,T,e.isPropertyName)),K(n,n,void 0,r));return e.setTextRange(s,n),i&&e.startOnNewLine(s),h(o,49152,49152&a?16384:0),s}function ge(t,r){if(524288&t.transformFlags||97===t.expression.kind||e.isSuperProperty(e.skipOuterExpressions(t.expression))){var a=e.createCallBinding(t.expression,p),i=a.target,o=a.thisArg;97===t.expression.kind&&e.setEmitFlags(o,4);var s=void 0;if(s=524288&t.transformFlags?e.createFunctionApply(e.visitNode(i,L,e.isExpression),e.visitNode(o,T,e.isExpression),_e(t.arguments,!1,!1,!1)):e.createFunctionCall(e.visitNode(i,L,e.isExpression),e.visitNode(o,T,e.isExpression),e.visitNodes(t.arguments,T,e.isExpression),t),97===t.expression.kind){var l=e.createThis();e.setEmitFlags(l,4);var c=e.createLogicalOr(s,l);s=r?e.createAssignment(e.createFileLevelUniqueName(\"_this\"),c):c}return e.setOriginalNode(s,t)}return e.visitEachChild(t,T,n)}function _e(t,r,a,i){var o=t.length,s=e.flatten(e.spanMap(t,ve,function(e,n,t,r){return n(e,a,i&&r===o)}));if(f.downlevelIteration){if(1===s.length){var l=s[0];if(e.isCallExpression(l)&&e.isIdentifier(l.expression)&&4096&e.getEmitFlags(l.expression)&&\"___spread\"===l.expression.escapedText)return s[0]}return e.createSpreadHelper(n,s)}if(1===s.length){var c=t[0];return r&&e.isSpreadElement(c)&&185!==c.expression.kind?e.createArraySlice(s[0]):s[0]}return e.createArrayConcat(s.shift(),s)}function ve(n){return e.isSpreadElement(n)?ye:he}function ye(n){return e.map(n,be)}function he(n,t,r){return e.createArrayLiteral(e.visitNodes(e.createNodeArray(n,r),T,e.isExpression),t)}function be(n){return e.visitNode(n.expression,T,e.isExpression)}function Ee(n){var r=e.getSourceTextOfNodeFromSourceFile(t,n),a=13===n.kind||16===n.kind;return r=(r=r.substring(1,r.length-(a?1:2))).replace(/\\r\\n?/g,\"\\n\"),e.setTextRange(e.createLiteral(r),n)}function Te(n){return 8&a&&!n?e.createPropertyAccess(e.createFileLevelUniqueName(\"_super\"),\"prototype\"):e.createFileLevelUniqueName(\"_super\")}function Se(){0==(2&c)&&(c|=2,n.enableSubstitution(71))}function Le(){0==(1&c)&&(c|=1,n.enableSubstitution(99),n.enableEmitNotification(155),n.enableEmitNotification(154),n.enableEmitNotification(156),n.enableEmitNotification(157),n.enableEmitNotification(195),n.enableEmitNotification(194),n.enableEmitNotification(237))}function xe(n,t){return e.hasModifier(t,32)?e.getInternalName(n):e.createPropertyAccess(e.getInternalName(n),\"prototype\")}};var o={name:\"typescript:extends\",scoped:!1,priority:0,text:\"\\n            var __extends = (this && this.__extends) || (function () {\\n                var extendStatics = function (d, b) {\\n                    extendStatics = Object.setPrototypeOf ||\\n                        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\\n                        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\\n                    return extendStatics(d, b);\\n                }\\n\\n                return function (d, b) {\\n                    extendStatics(d, b);\\n                    function __() { this.constructor = d; }\\n                    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\\n                };\\n            })();\"},s={name:\"typescript:makeTemplateObject\",scoped:!1,priority:0,text:'\\n            var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {\\n                if (Object.defineProperty) { Object.defineProperty(cooked, \"raw\", { value: raw }); } else { cooked.raw = raw; }\\n                return cooked;\\n            };'}}(Tn||(Tn={})),function(e){e.transformES5=function(n){var t,r,a=n.getCompilerOptions();1!==a.jsx&&3!==a.jsx||(t=n.onEmitNode,n.onEmitNode=function(n,a,i){switch(a.kind){case 260:case 261:case 259:var o=a.tagName;r[e.getOriginalNodeId(o)]=!0}t(n,a,i)},n.enableEmitNotification(260),n.enableEmitNotification(261),n.enableEmitNotification(259),r=[]);var i=n.onSubstituteNode;return n.onSubstituteNode=function(n,t){return t.id&&r&&r[t.id]?i(n,t):(t=i(n,t),e.isPropertyAccessExpression(t)?function(n){var t=o(n.name);return t?e.setTextRange(e.createElementAccess(n.expression,t),n):n}(t):e.isPropertyAssignment(t)?function(n){var t=e.isIdentifier(n.name)&&o(n.name);return t?e.updatePropertyAssignment(n,t,n.initializer):n}(t):t)},n.enableSubstitution(187),n.enableSubstitution(273),e.chainBundle(function(e){return e});function o(n){var t=n.originalKeywordKind||(e.nodeIsSynthesized(n)?e.stringToToken(e.idText(n)):void 0);if(void 0!==t&&t>=72&&t<=107)return e.setTextRange(e.createLiteral(n),n)}}}(Tn||(Tn={})),function(e){var n,t,r,a,i;!function(e){e[e.Nop=0]=\"Nop\",e[e.Statement=1]=\"Statement\",e[e.Assign=2]=\"Assign\",e[e.Break=3]=\"Break\",e[e.BreakWhenTrue=4]=\"BreakWhenTrue\",e[e.BreakWhenFalse=5]=\"BreakWhenFalse\",e[e.Yield=6]=\"Yield\",e[e.YieldStar=7]=\"YieldStar\",e[e.Return=8]=\"Return\",e[e.Throw=9]=\"Throw\",e[e.Endfinally=10]=\"Endfinally\"}(n||(n={})),function(e){e[e.Open=0]=\"Open\",e[e.Close=1]=\"Close\"}(t||(t={})),function(e){e[e.Exception=0]=\"Exception\",e[e.With=1]=\"With\",e[e.Switch=2]=\"Switch\",e[e.Loop=3]=\"Loop\",e[e.Labeled=4]=\"Labeled\"}(r||(r={})),function(e){e[e.Try=0]=\"Try\",e[e.Catch=1]=\"Catch\",e[e.Finally=2]=\"Finally\",e[e.Done=3]=\"Done\"}(a||(a={})),function(e){e[e.Next=0]=\"Next\",e[e.Throw=1]=\"Throw\",e[e.Return=2]=\"Return\",e[e.Break=3]=\"Break\",e[e.Yield=4]=\"Yield\",e[e.YieldStar=5]=\"YieldStar\",e[e.Catch=6]=\"Catch\",e[e.Endfinally=7]=\"Endfinally\"}(i||(i={})),e.transformGenerators=function(n){var t,r,a,i,s,l,c,u,d,m,p=n.resumeLexicalEnvironment,f=n.endLexicalEnvironment,g=n.hoistFunctionDeclaration,_=n.hoistVariableDeclaration,v=n.getCompilerOptions(),y=e.getEmitScriptTarget(v),h=n.getEmitResolver(),b=n.onSubstituteNode;n.onSubstituteNode=function(n,a){return a=b(n,a),1===n?function(n){return e.isIdentifier(n)?function(n){if(!e.isGeneratedIdentifier(n)&&t&&t.has(e.idText(n))){var a=e.getOriginalNode(n);if(e.isIdentifier(a)&&a.parent){var i=h.getReferencedValueDeclaration(a);if(i){var o=r[e.getOriginalNodeId(i)];if(o){var s=e.getMutableClone(o);return e.setSourceMapRange(s,n),e.setCommentRange(s,n),s}}}}return n}(n):n}(a):a};var E,T,S,L,x,A,C,D,k,M,O,I,R=1,N=0,w=0;return e.chainBundle(function(t){if(t.isDeclarationFile||0==(512&t.transformFlags))return t;var r=e.visitEachChild(t,P,n);return e.addEmitHelpers(r,n.readEmitHelpers()),r});function P(t){var r=t.transformFlags;return i?function(t){switch(t.kind){case 221:case 222:return function(t){return i?(re(),t=e.visitEachChild(t,P,n),ie(),t):e.visitEachChild(t,P,n)}(t);case 230:return function(t){return i&&$({kind:2,isScript:!0,breakLabel:-1}),t=e.visitEachChild(t,P,n),i&&oe(),t}(t);case 231:return function(t){return i&&$({kind:4,isScript:!0,labelText:e.idText(t.label),breakLabel:-1}),t=e.visitEachChild(t,P,n),i&&se(),t}(t);default:return F(t)}}(t):a?F(t):256&r?function(n){switch(n.kind){case 237:return G(n);case 194:return V(n);default:return e.Debug.failBadSyntaxKind(n)}}(t):512&r?e.visitEachChild(t,P,n):t}function F(t){switch(t.kind){case 237:return G(t);case 194:return V(t);case 156:case 157:return function(t){var r=a,o=i;return a=!1,i=!1,t=e.visitEachChild(t,P,n),a=r,i=o,t}(t);case 217:return function(n){if(16777216&n.transformFlags)W(n.declarationList);else{if(1048576&e.getEmitFlags(n))return n;for(var t=0,r=n.declarationList.declarations;t<r.length;t++){var a=r[t];_(a.name)}var i=e.getInitializedVariables(n.declarationList);if(0!==i.length)return e.setSourceMapRange(e.createExpressionStatement(e.inlineExpressions(e.map(i,q))),n)}}(t);case 223:return function(t){i&&re();var r=t.initializer;if(r&&e.isVariableDeclarationList(r)){for(var a=0,o=r.declarations;a<o.length;a++){var s=o[a];_(s.name)}var l=e.getInitializedVariables(r);t=e.updateFor(t,l.length>0?e.inlineExpressions(e.map(l,q)):void 0,e.visitNode(t.condition,P,e.isExpression),e.visitNode(t.incrementor,P,e.isExpression),e.visitNode(t.statement,P,e.isStatement,e.liftToBlock))}else t=e.visitEachChild(t,P,n);return i&&ie(),t}(t);case 224:return function(t){i&&re();var r=t.initializer;if(e.isVariableDeclarationList(r)){for(var a=0,o=r.declarations;a<o.length;a++){var s=o[a];_(s.name)}t=e.updateForIn(t,r.declarations[0].name,e.visitNode(t.expression,P,e.isExpression),e.visitNode(t.statement,P,e.isStatement,e.liftToBlock))}else t=e.visitEachChild(t,P,n);return i&&ie(),t}(t);case 227:return function(t){if(i){var r=me(t.label&&e.idText(t.label));if(r>0)return _e(r,t)}return e.visitEachChild(t,P,n)}(t);case 226:return function(t){if(i){var r=pe(t.label&&e.idText(t.label));if(r>0)return _e(r,t)}return e.visitEachChild(t,P,n)}(t);case 228:return function(n){return t=e.visitNode(n.expression,P,e.isExpression),r=n,e.setTextRange(e.createReturn(e.createArrayLiteral(t?[ge(2),t]:[ge(2)])),r);var t,r}(t);default:return 16777216&t.transformFlags?function(t){switch(t.kind){case 202:return function(t){var r=e.getExpressionAssociativity(t);switch(r){case 0:return function(t){if(z(t.right)){if(e.isLogicalOperator(t.operatorToken.kind))return function(n){var t=Q(),r=Y();return he(r,e.visitNode(n.left,P,e.isExpression),n.left),53===n.operatorToken.kind?Te(t,r,n.left):Ee(t,r,n.left),he(r,e.visitNode(n.right,P,e.isExpression),n.right),Z(t),r}(t);if(26===t.operatorToken.kind)return function(n){var t=[];return r(n.left),r(n.right),e.inlineExpressions(t);function r(n){e.isBinaryExpression(n)&&26===n.operatorToken.kind?(r(n.left),r(n.right)):(z(n)&&t.length>0&&(Se(1,[e.createExpressionStatement(e.inlineExpressions(t))]),t=[]),t.push(e.visitNode(n,P,e.isExpression)))}}(t);var r=e.getMutableClone(t);return r.left=X(e.visitNode(t.left,P,e.isExpression)),r.right=e.visitNode(t.right,P,e.isExpression),r}return e.visitEachChild(t,P,n)}(t);case 1:return function(t){var r,a=t.left,i=t.right;if(z(i)){var o=void 0;switch(a.kind){case 187:o=e.updatePropertyAccess(a,X(e.visitNode(a.expression,P,e.isLeftHandSideExpression)),a.name);break;case 188:o=e.updateElementAccess(a,X(e.visitNode(a.expression,P,e.isLeftHandSideExpression)),X(e.visitNode(a.argumentExpression,P,e.isExpression)));break;default:o=e.visitNode(a,P,e.isExpression)}var s=t.operatorToken.kind;return(r=s)>=59&&r<=70?e.setTextRange(e.createAssignment(o,e.setTextRange(e.createBinary(X(o),function(e){switch(e){case 59:return 37;case 60:return 38;case 61:return 39;case 62:return 40;case 63:return 41;case 64:return 42;case 65:return 45;case 66:return 46;case 67:return 47;case 68:return 48;case 69:return 49;case 70:return 50}}(s),e.visitNode(i,P,e.isExpression)),t)),t):e.updateBinary(t,o,e.visitNode(i,P,e.isExpression))}return e.visitEachChild(t,P,n)}(t);default:return e.Debug.assertNever(r)}}(t);case 203:return function(t){if(z(t.whenTrue)||z(t.whenFalse)){var r=Q(),a=Q(),i=Y();return Te(r,e.visitNode(t.condition,P,e.isExpression),t.condition),he(i,e.visitNode(t.whenTrue,P,e.isExpression),t.whenTrue),be(a),Z(r),he(i,e.visitNode(t.whenFalse,P,e.isExpression),t.whenFalse),Z(a),i}return e.visitEachChild(t,P,n)}(t);case 205:return function(t){var r,a=Q(),i=e.visitNode(t.expression,P,e.isExpression);if(t.asteriskToken){var o=0==(8388608&e.getEmitFlags(t.expression))?e.createValuesHelper(n,i,t):i;!function(e,n){Se(7,[e],n)}(o,t)}else!function(e,n){Se(6,[e],n)}(i,t);return Z(a),r=t,e.setTextRange(e.createCall(e.createPropertyAccess(L,\"sent\"),void 0,[]),r)}(t);case 185:return function(e){return B(e.elements,void 0,void 0,e.multiLine)}(t);case 186:return function(n){var t=n.properties,r=n.multiLine,a=J(t),i=Y();he(i,e.createObjectLiteral(e.visitNodes(t,P,e.isObjectLiteralElementLike,0,a),r));var o=e.reduceLeft(t,function(t,a){z(a)&&t.length>0&&(ye(e.createExpressionStatement(e.inlineExpressions(t))),t=[]);var o=e.createExpressionForObjectLiteralElementLike(n,a,i),s=e.visitNode(o,P,e.isExpression);return s&&(r&&e.startOnNewLine(s),t.push(s)),t},[],a);return o.push(r?e.startOnNewLine(e.getMutableClone(i)):i),e.inlineExpressions(o)}(t);case 188:return function(t){if(z(t.argumentExpression)){var r=e.getMutableClone(t);return r.expression=X(e.visitNode(t.expression,P,e.isLeftHandSideExpression)),r.argumentExpression=e.visitNode(t.argumentExpression,P,e.isExpression),r}return e.visitEachChild(t,P,n)}(t);case 189:return function(t){if(!e.isImportCall(t)&&e.forEach(t.arguments,z)){var r=e.createCallBinding(t.expression,_,y,!0),a=r.target,i=r.thisArg;return e.setOriginalNode(e.createFunctionApply(X(e.visitNode(a,P,e.isLeftHandSideExpression)),i,B(t.arguments),t),t)}return e.visitEachChild(t,P,n)}(t);case 190:return function(t){if(e.forEach(t.arguments,z)){var r=e.createCallBinding(e.createPropertyAccess(t.expression,\"bind\"),_),a=r.target,i=r.thisArg;return e.setOriginalNode(e.setTextRange(e.createNew(e.createFunctionApply(X(e.visitNode(a,P,e.isExpression)),i,B(t.arguments,e.createVoidZero())),void 0,[]),t),t)}return e.visitEachChild(t,P,n)}(t);default:return e.visitEachChild(t,P,n)}}(t):33554944&t.transformFlags?e.visitEachChild(t,P,n):t}}function G(t){if(t.asteriskToken)t=e.setOriginalNode(e.setTextRange(e.createFunctionDeclaration(void 0,t.modifiers,void 0,t.name,void 0,e.visitParameterList(t.parameters,P,n),void 0,K(t.body)),t),t);else{var r=a,o=i;a=!1,i=!1,t=e.visitEachChild(t,P,n),a=r,i=o}return a?void g(t):t}function V(t){if(t.asteriskToken)t=e.setOriginalNode(e.setTextRange(e.createFunctionExpression(void 0,void 0,t.name,void 0,e.visitParameterList(t.parameters,P,n),void 0,K(t.body)),t),t);else{var r=a,o=i;a=!1,i=!1,t=e.visitEachChild(t,P,n),a=r,i=o}return t}function K(n){var t=[],r=a,o=i,g=s,_=l,v=c,y=u,h=d,b=m,x=R,A=E,C=T,D=S,k=L;a=!0,i=!1,s=void 0,l=void 0,c=void 0,u=void 0,d=void 0,m=void 0,R=1,E=void 0,T=void 0,S=void 0,L=e.createTempVariable(void 0),p();var M=e.addPrologue(t,n.statements,!1,P);H(n.statements,M);var O=Le();return e.addStatementsAfterPrologue(t,f()),t.push(e.createReturn(O)),a=r,i=o,s=g,l=_,c=v,u=y,d=h,m=b,R=x,E=A,T=C,S=D,L=k,e.setTextRange(e.createBlock(t,n.multiLine),n)}function B(n,t,r,a){var i,o=J(n);if(o>0){i=Y();var s=e.visitNodes(n,P,e.isExpression,0,o);he(i,e.createArrayLiteral(t?[t].concat(s):s)),t=void 0}var l=e.reduceLeft(n,function(n,r){if(z(r)&&n.length>0){var o=void 0!==i;i||(i=Y()),he(i,o?e.createArrayConcat(i,[e.createArrayLiteral(n,a)]):e.createArrayLiteral(t?[t].concat(n):n,a)),t=void 0,n=[]}return n.push(e.visitNode(r,P,e.isExpression)),n},[],o);return i?e.createArrayConcat(i,[e.createArrayLiteral(l,a)]):e.setTextRange(e.createArrayLiteral(t?[t].concat(l):l,a),r)}function H(e,n){void 0===n&&(n=0);for(var t=e.length,r=n;r<t;r++)j(e[r])}function U(n){e.isBlock(n)?H(n.statements):j(n)}function j(a){var o=i;i||(i=z(a)),function(a){switch(a.kind){case 216:return function(n){z(n)?H(n.statements):ye(e.visitNode(n,P,e.isStatement))}(a);case 219:return function(n){ye(e.visitNode(n,P,e.isStatement))}(a);case 220:return function(n){if(z(n))if(z(n.thenStatement)||z(n.elseStatement)){var t=Q(),r=n.elseStatement?Q():void 0;Te(n.elseStatement?r:t,e.visitNode(n.expression,P,e.isExpression),n.expression),U(n.thenStatement),n.elseStatement&&(be(t),Z(r),U(n.elseStatement)),Z(t)}else ye(e.visitNode(n,P,e.isStatement));else ye(e.visitNode(n,P,e.isStatement))}(a);case 221:return function(n){if(z(n)){var t=Q(),r=Q();ae(t),Z(r),U(n.statement),Z(t),Ee(r,e.visitNode(n.expression,P,e.isExpression)),ie()}else ye(e.visitNode(n,P,e.isStatement))}(a);case 222:return function(n){if(z(n)){var t=Q(),r=ae(t);Z(t),Te(r,e.visitNode(n.expression,P,e.isExpression)),U(n.statement),be(t),ie()}else ye(e.visitNode(n,P,e.isStatement))}(a);case 223:return function(n){if(z(n)){var t=Q(),r=Q(),a=ae(r);if(n.initializer){var i=n.initializer;e.isVariableDeclarationList(i)?W(i):ye(e.setTextRange(e.createExpressionStatement(e.visitNode(i,P,e.isExpression)),i))}Z(t),n.condition&&Te(a,e.visitNode(n.condition,P,e.isExpression)),U(n.statement),Z(r),n.incrementor&&ye(e.setTextRange(e.createExpressionStatement(e.visitNode(n.incrementor,P,e.isExpression)),n.incrementor)),be(t),ie()}else ye(e.visitNode(n,P,e.isStatement))}(a);case 224:return function(n){if(z(n)){var t=Y(),r=Y(),a=e.createLoopVariable(),i=n.initializer;_(a),he(t,e.createArrayLiteral()),ye(e.createForIn(r,e.visitNode(n.expression,P,e.isExpression),e.createExpressionStatement(e.createCall(e.createPropertyAccess(t,\"push\"),void 0,[r])))),he(a,e.createLiteral(0));var o=Q(),s=Q(),l=ae(s);Z(o),Te(l,e.createLessThan(a,e.createPropertyAccess(t,\"length\")));var c=void 0;if(e.isVariableDeclarationList(i)){for(var u=0,d=i.declarations;u<d.length;u++){var m=d[u];_(m.name)}c=e.getSynthesizedClone(i.declarations[0].name)}else c=e.visitNode(i,P,e.isExpression),e.Debug.assert(e.isLeftHandSideExpression(c));he(c,e.createElementAccess(t,a)),U(n.statement),Z(s),ye(e.createExpressionStatement(e.createPostfixIncrement(a))),be(o),ie()}else ye(e.visitNode(n,P,e.isStatement))}(a);case 226:return function(n){var t=pe(n.label?e.idText(n.label):void 0);t>0?be(t,n):ye(n)}(a);case 227:return function(n){var t=me(n.label?e.idText(n.label):void 0);t>0?be(t,n):ye(n)}(a);case 228:return function(n){Se(8,[e.visitNode(n.expression,P,e.isExpression)],n)}(a);case 229:return function(n){var t,r,a;z(n)?(t=X(e.visitNode(n.expression,P,e.isExpression)),r=Q(),a=Q(),Z(r),$({kind:1,expression:t,startLabel:r,endLabel:a}),U(n.statement),e.Debug.assert(1===te()),Z(ee().endLabel)):ye(e.visitNode(n,P,e.isStatement))}(a);case 230:return function(n){if(z(n.caseBlock)){for(var t=n.caseBlock,r=t.clauses.length,a=($({kind:2,isScript:!1,breakLabel:p=Q()}),p),i=X(e.visitNode(n.expression,P,e.isExpression)),o=[],s=-1,l=0;l<r;l++){var c=t.clauses[l];o.push(Q()),270===c.kind&&-1===s&&(s=l)}for(var u=0,d=[];u<r;){for(var m=0,l=u;l<r;l++){var c=t.clauses[l];if(269===c.kind){if(z(c.expression)&&d.length>0)break;d.push(e.createCaseClause(e.visitNode(c.expression,P,e.isExpression),[_e(o[l],c.expression)]))}else m++}d.length&&(ye(e.createSwitch(i,e.createCaseBlock(d))),u+=d.length,d=[]),m>0&&(u+=m,m=0)}be(s>=0?o[s]:a);for(var l=0;l<r;l++)Z(o[l]),H(t.clauses[l].statements);oe()}else ye(e.visitNode(n,P,e.isStatement));var p}(a);case 231:return function(n){var t,r;z(n)?(t=e.idText(n.label),r=Q(),$({kind:4,isScript:!1,labelText:t,breakLabel:r}),U(n.statement),se()):ye(e.visitNode(n,P,e.isStatement))}(a);case 232:return function(n){Se(9,[e.visitNode(n.expression,P,e.isExpression)],n)}(a);case 233:return function(a){var i,o;z(a)?(i=Q(),o=Q(),Z(i),$({kind:0,state:0,startLabel:i,endLabel:o}),ve(),U(a.tryBlock),a.catchClause&&(function(a){var i;if(e.Debug.assert(0===te()),e.isGeneratedIdentifier(a.name))i=a.name,_(a.name);else{var o=e.idText(a.name);i=Y(o),t||(t=e.createMap(),r=[],n.enableSubstitution(71)),t.set(o,!0),r[e.getOriginalNodeId(a)]=i}var s=ne();e.Debug.assert(s.state<1),be(s.endLabel);var l=Q();Z(l),s.state=1,s.catchVariable=i,s.catchLabel=l,he(i,e.createCall(e.createPropertyAccess(L,\"sent\"),void 0,[])),ve()}(a.catchClause.variableDeclaration),U(a.catchClause.block)),a.finallyBlock&&(function(){e.Debug.assert(0===te());var n=ne();e.Debug.assert(n.state<2),be(n.endLabel);var t=Q();Z(t),n.state=2,n.finallyLabel=t}(),U(a.finallyBlock)),function(){e.Debug.assert(0===te());var n=ee();n.state<2?be(n.endLabel):Se(10),Z(n.endLabel),ve(),n.state=3}()):ye(e.visitEachChild(a,P,n))}(a);default:ye(e.visitNode(a,P,e.isStatement))}}(a),i=o}function W(n){for(var t=0,r=n.declarations;t<r.length;t++){var a=r[t],i=e.getSynthesizedClone(a.name);e.setCommentRange(i,a.name),_(i)}for(var o=e.getInitializedVariables(n),s=o.length,l=0,c=[];l<s;){for(var u=l;u<s&&!(z((a=o[u]).initializer)&&c.length>0);u++)c.push(q(a));c.length&&(ye(e.createExpressionStatement(e.inlineExpressions(c))),l+=c.length,c=[])}}function q(n){return e.setSourceMapRange(e.createAssignment(e.setSourceMapRange(e.getSynthesizedClone(n.name),n.name),e.visitNode(n.initializer,P,e.isExpression)),n)}function z(e){return!!e&&0!=(16777216&e.transformFlags)}function J(e){for(var n=e.length,t=0;t<n;t++)if(z(e[t]))return t;return-1}function X(n){var t;return e.isGeneratedIdentifier(n)||4096&e.getEmitFlags(n)?n:(he(t=e.createTempVariable(_),n,n),t)}function Y(n){var t=n?e.createUniqueName(n):e.createTempVariable(void 0);return _(t),t}function Q(){d||(d=[]);var e=R;return R++,d[e]=-1,e}function Z(n){e.Debug.assert(void 0!==d,\"No labels were defined.\"),d[n]=E?E.length:0}function $(e){s||(s=[],c=[],l=[],u=[]);var n=c.length;return c[n]=0,l[n]=E?E.length:0,s[n]=e,u.push(e),n}function ee(){var n=ne();if(void 0===n)return e.Debug.fail(\"beginBlock was never called.\");var t=c.length;return c[t]=1,l[t]=E?E.length:0,s[t]=n,u.pop(),n}function ne(){return e.lastOrUndefined(u)}function te(){var e=ne();return e&&e.kind}function re(){$({kind:3,isScript:!0,breakLabel:-1,continueLabel:-1})}function ae(e){var n=Q();return $({kind:3,isScript:!1,breakLabel:n,continueLabel:e}),n}function ie(){e.Debug.assert(3===te());var n=ee(),t=n.breakLabel;n.isScript||Z(t)}function oe(){e.Debug.assert(2===te());var n=ee(),t=n.breakLabel;n.isScript||Z(t)}function se(){e.Debug.assert(4===te());var n=ee();n.isScript||Z(n.breakLabel)}function le(e){return 2===e.kind||3===e.kind}function ce(e){return 4===e.kind}function ue(e){return 3===e.kind}function de(e,n){for(var t=n;t>=0;t--){var r=u[t];if(!ce(r))break;if(r.labelText===e)return!0}return!1}function me(e){if(u)if(e)for(var n=u.length-1;n>=0;n--){if(ce(t=u[n])&&t.labelText===e)return t.breakLabel;if(le(t)&&de(e,n-1))return t.breakLabel}else for(n=u.length-1;n>=0;n--){var t;if(le(t=u[n]))return t.breakLabel}return 0}function pe(e){if(u)if(e){for(var n=u.length-1;n>=0;n--)if(ue(t=u[n])&&de(e,n-1))return t.continueLabel}else for(n=u.length-1;n>=0;n--){var t;if(ue(t=u[n]))return t.continueLabel}return 0}function fe(n){if(void 0!==n&&n>0){void 0===m&&(m=[]);var t=e.createLiteral(-1);return void 0===m[n]?m[n]=[t]:m[n].push(t),t}return e.createOmittedExpression()}function ge(n){var t=e.createLiteral(n);return e.addSyntheticTrailingComment(t,3,function(e){switch(e){case 2:return\"return\";case 3:return\"break\";case 4:return\"yield\";case 5:return\"yield*\";case 7:return\"endfinally\";default:return}}(n)),t}function _e(n,t){return e.Debug.assertLessThan(0,n,\"Invalid label\"),e.setTextRange(e.createReturn(e.createArrayLiteral([ge(3),fe(n)])),t)}function ve(){Se(0)}function ye(e){e?Se(1,[e]):ve()}function he(e,n,t){Se(2,[e,n],t)}function be(e,n){Se(3,[e],n)}function Ee(e,n,t){Se(4,[e,n],t)}function Te(e,n,t){Se(5,[e,n],t)}function Se(e,n,t){void 0===E&&(E=[],T=[],S=[]),void 0===d&&Z(Q());var r=E.length;E[r]=e,T[r]=n,S[r]=t}function Le(){N=0,w=0,x=void 0,A=!1,C=!1,D=void 0,k=void 0,M=void 0,O=void 0,I=void 0;var t=function(){if(E){for(var n=0;n<E.length;n++)De(n);xe(E.length)}else xe(0);if(D){var t=e.createPropertyAccess(L,\"label\"),r=e.createSwitch(t,e.createCaseBlock(D));return[e.startOnNewLine(r)]}return k||[]}();return function(n,t){return n.requestEmitHelper(o),e.createCall(e.getHelperName(\"__generator\"),void 0,[e.createThis(),t])}(n,e.setEmitFlags(e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,L)],void 0,e.createBlock(t,t.length>0)),524288))}function xe(e){(function(e){if(!C)return!0;if(!d||!m)return!1;for(var n=0;n<d.length;n++)if(d[n]===e&&m[n])return!0;return!1})(e)&&(Ce(e),I=void 0,Me(void 0,void 0)),k&&D&&Ae(!1),function(){if(void 0!==m&&void 0!==x)for(var e=0;e<x.length;e++){var n=x[e];if(void 0!==n)for(var t=0,r=n;t<r.length;t++){var a=r[t],i=m[a];if(void 0!==i)for(var o=0,s=i;o<s.length;o++){var l=s[o];l.text=String(e)}}}}()}function Ae(n){if(D||(D=[]),k){if(I)for(var t=I.length-1;t>=0;t--){var r=I[t];k=[e.createWith(r.expression,e.createBlock(k))]}if(O){var a=O.startLabel,i=O.catchLabel,o=O.finallyLabel,s=O.endLabel;k.unshift(e.createExpressionStatement(e.createCall(e.createPropertyAccess(e.createPropertyAccess(L,\"trys\"),\"push\"),void 0,[e.createArrayLiteral([fe(a),fe(i),fe(o),fe(s)])]))),O=void 0}n&&k.push(e.createExpressionStatement(e.createAssignment(e.createPropertyAccess(L,\"label\"),e.createLiteral(w+1))))}D.push(e.createCaseClause(e.createLiteral(w),k||[])),k=void 0}function Ce(e){if(d)for(var n=0;n<d.length;n++)d[n]===e&&(k&&(Ae(!A),A=!1,C=!1,w++),void 0===x&&(x=[]),void 0===x[w]?x[w]=[n]:x[w].push(n))}function De(n){if(Ce(n),function(e){if(s)for(;N<c.length&&l[N]<=e;N++){var n=s[N],t=c[N];switch(n.kind){case 0:0===t?(M||(M=[]),k||(k=[]),M.push(O),O=n):1===t&&(O=M.pop());break;case 1:0===t?(I||(I=[]),I.push(n)):1===t&&I.pop()}}}(n),!A){A=!1,C=!1;var t=E[n];if(0!==t){if(10===t)return A=!0,void ke(e.createReturn(e.createArrayLiteral([ge(7)])));var r=T[n];if(1===t)return ke(r[0]);var a,i,o,u=S[n];switch(t){case 2:return a=r[0],i=r[1],o=u,void ke(e.setTextRange(e.createExpressionStatement(e.createAssignment(a,i)),o));case 3:return function(n,t){A=!0,ke(e.setEmitFlags(e.setTextRange(e.createReturn(e.createArrayLiteral([ge(3),fe(n)])),t),384))}(r[0],u);case 4:return function(n,t,r){ke(e.setEmitFlags(e.createIf(t,e.setEmitFlags(e.setTextRange(e.createReturn(e.createArrayLiteral([ge(3),fe(n)])),r),384)),1))}(r[0],r[1],u);case 5:return function(n,t,r){ke(e.setEmitFlags(e.createIf(e.createLogicalNot(t),e.setEmitFlags(e.setTextRange(e.createReturn(e.createArrayLiteral([ge(3),fe(n)])),r),384)),1))}(r[0],r[1],u);case 6:return function(n,t){A=!0,ke(e.setEmitFlags(e.setTextRange(e.createReturn(e.createArrayLiteral(n?[ge(4),n]:[ge(4)])),t),384))}(r[0],u);case 7:return function(n,t){A=!0,ke(e.setEmitFlags(e.setTextRange(e.createReturn(e.createArrayLiteral([ge(5),n])),t),384))}(r[0],u);case 8:return Me(r[0],u);case 9:return function(n,t){A=!0,C=!0,ke(e.setTextRange(e.createThrow(n),t))}(r[0],u)}}}}function ke(e){e&&(k?k.push(e):k=[e])}function Me(n,t){A=!0,C=!0,ke(e.setEmitFlags(e.setTextRange(e.createReturn(e.createArrayLiteral(n?[ge(2),n]:[ge(2)])),t),384))}};var o={name:\"typescript:generator\",scoped:!1,priority:6,text:'\\n            var __generator = (this && this.__generator) || function (thisArg, body) {\\n                var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;\\n                return g = { next: verb(0), \"throw\": verb(1), \"return\": verb(2) }, typeof Symbol === \"function\" && (g[Symbol.iterator] = function() { return this; }), g;\\n                function verb(n) { return function (v) { return step([n, v]); }; }\\n                function step(op) {\\n                    if (f) throw new TypeError(\"Generator is already executing.\");\\n                    while (_) try {\\n                        if (f = 1, y && (t = op[0] & 2 ? y[\"return\"] : op[0] ? y[\"throw\"] || ((t = y[\"return\"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;\\n                        if (y = 0, t) op = [op[0] & 2, t.value];\\n                        switch (op[0]) {\\n                            case 0: case 1: t = op; break;\\n                            case 4: _.label++; return { value: op[1], done: false };\\n                            case 5: _.label++; y = op[1]; op = [0]; continue;\\n                            case 7: op = _.ops.pop(); _.trys.pop(); continue;\\n                            default:\\n                                if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }\\n                                if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }\\n                                if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }\\n                                if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }\\n                                if (t[2]) _.ops.pop();\\n                                _.trys.pop(); continue;\\n                        }\\n                        op = body.call(thisArg, _);\\n                    } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }\\n                    if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };\\n                }\\n            };'}}(Tn||(Tn={})),function(e){e.transformModule=function(i){var o=i.startLexicalEnvironment,s=i.endLexicalEnvironment,l=i.hoistVariableDeclaration,c=i.getCompilerOptions(),u=i.getEmitResolver(),d=i.getEmitHost(),m=e.getEmitScriptTarget(c),p=e.getEmitModuleKind(c),f=i.onSubstituteNode,g=i.onEmitNode;i.onSubstituteNode=function(n,t){return(t=f(n,t)).id&&y[t.id]?t:1===n?function(n){switch(n.kind){case 71:return X(n);case 202:return function(n){if(e.isAssignmentOperator(n.operatorToken.kind)&&e.isIdentifier(n.left)&&!e.isGeneratedIdentifier(n.left)&&!e.isLocalName(n.left)&&!e.isDeclarationNameOfEnumOrNamespace(n.left)){var t=Y(n.left);if(t){for(var r=n,a=0,i=t;a<i.length;a++){var o=i[a];y[e.getNodeId(r)]=!0,r=z(o,r,n)}return r}}return n}(n);case 201:case 200:return function(n){if((43===n.operator||44===n.operator)&&e.isIdentifier(n.operand)&&!e.isGeneratedIdentifier(n.operand)&&!e.isLocalName(n.operand)&&!e.isDeclarationNameOfEnumOrNamespace(n.operand)){var t=Y(n.operand);if(t){for(var r=201===n.kind?e.setTextRange(e.createBinary(n.operand,e.createToken(43===n.operator?59:60),e.createLiteral(1)),n):n,a=0,i=t;a<i.length;a++){var o=i[a];y[e.getNodeId(r)]=!0,r=z(o,r)}return r}}return n}(n)}return n}(t):e.isShorthandPropertyAssignment(t)?function(n){var t=n.name,r=X(t);if(r!==t){if(n.objectAssignmentInitializer){var a=e.createAssignment(r,n.objectAssignmentInitializer);return e.setTextRange(e.createPropertyAssignment(t,a),n)}return e.setTextRange(e.createPropertyAssignment(t,r),n)}return n}(t):t},i.onEmitNode=function(n,t,r){277===t.kind?(_=t,v=b[e.getOriginalNodeId(_)],y=[],g(n,t,r),_=void 0,v=void 0,y=void 0):g(n,t,r)},i.enableSubstitution(71),i.enableSubstitution(202),i.enableSubstitution(200),i.enableSubstitution(201),i.enableSubstitution(274),i.enableEmitNotification(277);var _,v,y,h,b=[],E=[];return e.chainBundle(function(n){if(n.isDeclarationFile||!(e.isEffectiveExternalModule(n,c)||67108864&n.transformFlags||e.isJsonSourceFile(n)&&e.hasJsonModuleEmitEnabled(c)&&(c.out||c.outFile)))return n;_=n,v=e.collectExternalModuleInfo(n,u,c),b[e.getOriginalNodeId(n)]=v;var t=function(n){switch(n){case e.ModuleKind.AMD:return L;case e.ModuleKind.UMD:return x;default:return S}}(p)(n);return _=void 0,v=void 0,h=!1,e.aggregateTransformFlags(t)});function T(){return!(v.exportEquals||!e.isExternalModule(_))}function S(t){o();var r=[],a=e.getStrictOptionValue(c,\"alwaysStrict\")||!c.noImplicitUseStrict&&e.isExternalModule(_),l=e.addPrologue(r,t.statements,a,M);T()&&e.append(r,q()),e.append(r,e.visitNode(v.externalHelpersImportDeclaration,M,e.isStatement)),e.addRange(r,e.visitNodes(t.statements,M,e.isStatement,l)),k(r,!1),e.addStatementsAfterPrologue(r,s());var u=e.updateSourceFileNode(t,e.setTextRange(e.createNodeArray(r),t.statements));return v.hasExportStarsToExportValues&&!c.importHelpers&&e.addEmitHelper(u,n),e.addEmitHelpers(u,i.readEmitHelpers()),u}function L(n){var t=e.createIdentifier(\"define\"),r=e.tryGetModuleNameFromFile(n,d,c),a=e.isJsonSourceFile(n)&&n,o=A(n,!0),s=o.aliasedModuleNames,l=o.unaliasedModuleNames,u=o.importAliasNames,m=e.updateSourceFileNode(n,e.setTextRange(e.createNodeArray([e.createExpressionStatement(e.createCall(t,void 0,(r?[r]:[]).concat([e.createArrayLiteral(a?e.emptyArray:[e.createLiteral(\"require\"),e.createLiteral(\"exports\")].concat(s,l)),a?a.statements.length?a.statements[0].expression:e.createObjectLiteral():e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,\"require\"),e.createParameter(void 0,void 0,void 0,\"exports\")].concat(u),void 0,D(n))])))]),n.statements));return e.addEmitHelpers(m,i.readEmitHelpers()),m}function x(n){var t=A(n,!1),r=t.aliasedModuleNames,a=t.unaliasedModuleNames,o=t.importAliasNames,s=e.tryGetModuleNameFromFile(n,d,c),l=e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,\"factory\")],void 0,e.setTextRange(e.createBlock([e.createIf(e.createLogicalAnd(e.createTypeCheck(e.createIdentifier(\"module\"),\"object\"),e.createTypeCheck(e.createPropertyAccess(e.createIdentifier(\"module\"),\"exports\"),\"object\")),e.createBlock([e.createVariableStatement(void 0,[e.createVariableDeclaration(\"v\",void 0,e.createCall(e.createIdentifier(\"factory\"),void 0,[e.createIdentifier(\"require\"),e.createIdentifier(\"exports\")]))]),e.setEmitFlags(e.createIf(e.createStrictInequality(e.createIdentifier(\"v\"),e.createIdentifier(\"undefined\")),e.createExpressionStatement(e.createAssignment(e.createPropertyAccess(e.createIdentifier(\"module\"),\"exports\"),e.createIdentifier(\"v\")))),1)]),e.createIf(e.createLogicalAnd(e.createTypeCheck(e.createIdentifier(\"define\"),\"function\"),e.createPropertyAccess(e.createIdentifier(\"define\"),\"amd\")),e.createBlock([e.createExpressionStatement(e.createCall(e.createIdentifier(\"define\"),void 0,(s?[s]:[]).concat([e.createArrayLiteral([e.createLiteral(\"require\"),e.createLiteral(\"exports\")].concat(r,a)),e.createIdentifier(\"factory\")])))])))],!0),void 0)),u=e.updateSourceFileNode(n,e.setTextRange(e.createNodeArray([e.createExpressionStatement(e.createCall(l,void 0,[e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,\"require\"),e.createParameter(void 0,void 0,void 0,\"exports\")].concat(o),void 0,D(n))]))]),n.statements));return e.addEmitHelpers(u,i.readEmitHelpers()),u}function A(n,t){for(var r=[],a=[],i=[],o=0,s=n.amdDependencies;o<s.length;o++){var l=s[o];l.name?(r.push(e.createLiteral(l.path)),i.push(e.createParameter(void 0,void 0,void 0,l.name))):a.push(e.createLiteral(l.path))}for(var m=0,p=v.externalImports;m<p.length;m++){var f=p[m],g=e.getExternalModuleNameLiteral(f,_,d,u,c),y=e.getLocalNameForExternalImport(f,_);g&&(t&&y?(e.setEmitFlags(y,4),r.push(g),i.push(e.createParameter(void 0,void 0,void 0,y))):a.push(g))}return{aliasedModuleNames:r,unaliasedModuleNames:a,importAliasNames:i}}function C(n){if(!e.isImportEqualsDeclaration(n)&&!e.isExportDeclaration(n)&&e.getExternalModuleNameLiteral(n,_,d,u,c)){var t=e.getLocalNameForExternalImport(n,_),r=N(n,t);if(r!==t)return e.createExpressionStatement(e.createAssignment(t,r))}}function D(r){o();var a=[],i=e.addPrologue(a,r.statements,!c.noImplicitUseStrict,M);T()&&e.append(a,q()),e.append(a,e.visitNode(v.externalHelpersImportDeclaration,M,e.isStatement)),p===e.ModuleKind.AMD&&e.addRange(a,e.mapDefined(v.externalImports,C)),e.addRange(a,e.visitNodes(r.statements,M,e.isStatement,i)),k(a,!0),e.addStatementsAfterPrologue(a,s());var l=e.createBlock(a,!0);return v.hasExportStarsToExportValues&&!c.importHelpers&&e.addEmitHelper(l,n),h&&e.addEmitHelper(l,t),l}function k(n,t){if(v.exportEquals){var r=e.visitNode(v.exportEquals.expression,O);if(r)if(t){var a=e.createReturn(r);e.setTextRange(a,v.exportEquals),e.setEmitFlags(a,1920),n.push(a)}else a=e.createExpressionStatement(e.createAssignment(e.createPropertyAccess(e.createIdentifier(\"module\"),\"exports\"),r)),e.setTextRange(a,v.exportEquals),e.setEmitFlags(a,1536),n.push(a)}}function M(n){switch(n.kind){case 247:return function(n){var t,r=e.getNamespaceDeclarationNode(n);if(p!==e.ModuleKind.AMD){if(!n.importClause)return e.setOriginalNode(e.setTextRange(e.createExpressionStatement(w(n)),n),n);var a=[];r&&!e.isDefaultImport(n)?a.push(e.createVariableDeclaration(e.getSynthesizedClone(r.name),void 0,N(n,w(n)))):(a.push(e.createVariableDeclaration(e.getGeneratedNameForNode(n),void 0,N(n,w(n)))),r&&e.isDefaultImport(n)&&a.push(e.createVariableDeclaration(e.getSynthesizedClone(r.name),void 0,e.getGeneratedNameForNode(n)))),t=e.append(t,e.setOriginalNode(e.setTextRange(e.createVariableStatement(void 0,e.createVariableDeclarationList(a,m>=2?2:0)),n),n))}else r&&e.isDefaultImport(n)&&(t=e.append(t,e.createVariableStatement(void 0,e.createVariableDeclarationList([e.setOriginalNode(e.setTextRange(e.createVariableDeclaration(e.getSynthesizedClone(r.name),void 0,e.getGeneratedNameForNode(n)),n),n)],m>=2?2:0))));if(G(n)){var i=e.getOriginalNodeId(n);E[i]=V(E[i],n)}else t=V(t,n);return e.singleOrMany(t)}(n);case 246:return function(n){var t;if(e.Debug.assert(e.isExternalModuleImportEqualsDeclaration(n),\"import= for internal module references should be handled in an earlier transformer.\"),p!==e.ModuleKind.AMD?t=e.hasModifier(n,1)?e.append(t,e.setOriginalNode(e.setTextRange(e.createExpressionStatement(z(n.name,w(n))),n),n)):e.append(t,e.setOriginalNode(e.setTextRange(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(e.getSynthesizedClone(n.name),void 0,w(n))],m>=2?2:0)),n),n)):e.hasModifier(n,1)&&(t=e.append(t,e.setOriginalNode(e.setTextRange(e.createExpressionStatement(z(e.getExportName(n),e.getLocalName(n))),n),n))),G(n)){var r=e.getOriginalNodeId(n);E[r]=K(E[r],n)}else t=K(t,n);return e.singleOrMany(t)}(n);case 253:return function(n){if(n.moduleSpecifier){var t=e.getGeneratedNameForNode(n);if(n.exportClause){var r=[];p!==e.ModuleKind.AMD&&r.push(e.setOriginalNode(e.setTextRange(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(t,void 0,w(n))])),n),n));for(var a=0,o=n.exportClause.elements;a<o.length;a++){var s=o[a],l=e.createPropertyAccess(t,s.propertyName||s.name);r.push(e.setOriginalNode(e.setTextRange(e.createExpressionStatement(z(e.getExportName(s),l)),s),s))}return e.singleOrMany(r)}return e.setOriginalNode(e.setTextRange(e.createExpressionStatement(function(n,t){return n.getCompilerOptions().importHelpers?e.createCall(e.getHelperName(\"__exportStar\"),void 0,[t,e.createIdentifier(\"exports\")]):e.createCall(e.createIdentifier(\"__export\"),void 0,[t])}(i,p!==e.ModuleKind.AMD?w(n):t)),n),n)}}(n);case 252:return function(n){if(!n.isExportEquals){var t,r=n.original;if(r&&G(r)){var a=e.getOriginalNodeId(n);E[a]=W(E[a],e.createIdentifier(\"default\"),e.visitNode(n.expression,O),n,!0)}else t=W(t,e.createIdentifier(\"default\"),e.visitNode(n.expression,O),n,!0);return e.singleOrMany(t)}}(n);case 217:return function(n){var t,r,a;if(e.hasModifier(n,1)){for(var o=void 0,s=0,l=n.declarationList.declarations;s<l.length;s++){var c=l[s];e.isIdentifier(c.name)&&e.isLocalName(c.name)?(o||(o=e.visitNodes(n.modifiers,J,e.isModifier)),r=e.append(r,c)):c.initializer&&(a=e.append(a,F(c)))}r&&(t=e.append(t,e.updateVariableStatement(n,o,e.updateVariableDeclarationList(n.declarationList,r)))),a&&(t=e.append(t,e.setOriginalNode(e.setTextRange(e.createExpressionStatement(e.inlineExpressions(a)),n),n)))}else t=e.append(t,e.visitEachChild(n,O,i));if(G(n)){var u=e.getOriginalNodeId(n);E[u]=B(E[u],n)}else t=B(t,n);return e.singleOrMany(t)}(n);case 237:return function(n){var t;if(t=e.hasModifier(n,1)?e.append(t,e.setOriginalNode(e.setTextRange(e.createFunctionDeclaration(void 0,e.visitNodes(n.modifiers,J,e.isModifier),n.asteriskToken,e.getDeclarationName(n,!0,!0),void 0,e.visitNodes(n.parameters,O),void 0,e.visitEachChild(n.body,O,i)),n),n)):e.append(t,e.visitEachChild(n,O,i)),G(n)){var r=e.getOriginalNodeId(n);E[r]=U(E[r],n)}else t=U(t,n);return e.singleOrMany(t)}(n);case 238:return function(n){var t;if(t=e.hasModifier(n,1)?e.append(t,e.setOriginalNode(e.setTextRange(e.createClassDeclaration(void 0,e.visitNodes(n.modifiers,J,e.isModifier),e.getDeclarationName(n,!0,!0),void 0,e.visitNodes(n.heritageClauses,O),e.visitNodes(n.members,O)),n),n)):e.append(t,e.visitEachChild(n,O,i)),G(n)){var r=e.getOriginalNodeId(n);E[r]=U(E[r],n)}else t=U(t,n);return e.singleOrMany(t)}(n);case 308:return function(n){if(G(n)&&217===n.original.kind){var t=e.getOriginalNodeId(n);E[t]=B(E[t],n.original)}return n}(n);case 309:return function(n){var t=e.getOriginalNodeId(n),r=E[t];return r?(delete E[t],e.append(r,n)):n}(n);default:return e.visitEachChild(n,O,i)}}function O(n){return 67108864&n.transformFlags||2048&n.transformFlags?e.isImportCall(n)?function(n){var t=e.visitNode(e.firstOrUndefined(n.arguments),O),r=!!(16384&n.transformFlags);switch(c.module){case e.ModuleKind.AMD:return I(t,r);case e.ModuleKind.UMD:return function(n,t){if(h=!0,e.isSimpleCopiableExpression(n)){var r=e.isGeneratedIdentifier(n)?n:e.isStringLiteral(n)?e.createLiteral(n):e.setEmitFlags(e.setTextRange(e.getSynthesizedClone(n),n),1536);return e.createConditional(e.createIdentifier(\"__syncRequire\"),R(n,t),I(r,t))}var a=e.createTempVariable(l);return e.createComma(e.createAssignment(a,n),e.createConditional(e.createIdentifier(\"__syncRequire\"),R(a,t),I(a,t)))}(t,r);case e.ModuleKind.CommonJS:default:return R(t,r)}}(n):1024&n.transformFlags&&e.isBinaryExpression(n)?function(n){return function n(t){if(e.isObjectLiteralExpression(t))for(var r=0,a=t.properties;r<a.length;r++){var i=a[r];switch(i.kind){case 273:if(n(i.initializer))return!0;break;case 274:if(n(i.name))return!0;break;case 275:if(n(i.expression))return!0;break;case 154:case 156:case 157:return!1;default:e.Debug.assertNever(i,\"Unhandled object member kind\")}}else if(e.isArrayLiteralExpression(t))for(var o=0,s=t.elements;o<s.length;o++){var i=s[o];if(e.isSpreadElement(i)){if(n(i.expression))return!0}else if(n(i))return!0}else if(e.isIdentifier(t))return e.length(Y(t))>(e.isExportName(t)?1:0);return!1}(n.left)?e.flattenDestructuringAssignment(n,O,i,0,!1,P):e.visitEachChild(n,O,i)}(n):e.visitEachChild(n,O,i):n}function I(n,t){var a,o=e.createUniqueName(\"resolve\"),s=e.createUniqueName(\"reject\"),l=[e.createParameter(void 0,void 0,void 0,o),e.createParameter(void 0,void 0,void 0,s)],u=e.createBlock([e.createExpressionStatement(e.createCall(e.createIdentifier(\"require\"),void 0,[e.createArrayLiteral([n||e.createOmittedExpression()]),o,s]))]);m>=2?a=e.createArrowFunction(void 0,void 0,l,void 0,void 0,u):(a=e.createFunctionExpression(void 0,void 0,void 0,void 0,l,void 0,u),t&&e.setEmitFlags(a,8));var d=e.createNew(e.createIdentifier(\"Promise\"),void 0,[a]);return c.esModuleInterop?(i.requestEmitHelper(r),e.createCall(e.createPropertyAccess(d,e.createIdentifier(\"then\")),void 0,[e.getHelperName(\"__importStar\")])):d}function R(n,t){var a,o=e.createCall(e.createPropertyAccess(e.createIdentifier(\"Promise\"),\"resolve\"),void 0,[]),s=e.createCall(e.createIdentifier(\"require\"),void 0,n?[n]:[]);return c.esModuleInterop&&(i.requestEmitHelper(r),s=e.createCall(e.getHelperName(\"__importStar\"),void 0,[s])),m>=2?a=e.createArrowFunction(void 0,void 0,[],void 0,void 0,s):(a=e.createFunctionExpression(void 0,void 0,void 0,void 0,[],void 0,e.createBlock([e.createReturn(s)])),t&&e.setEmitFlags(a,8)),e.createCall(e.createPropertyAccess(o,\"then\"),void 0,[a])}function N(n,t){return!c.esModuleInterop||67108864&e.getEmitFlags(n)?t:e.getImportNeedsImportStarHelper(n)?(i.requestEmitHelper(r),e.createCall(e.getHelperName(\"__importStar\"),void 0,[t])):e.getImportNeedsImportDefaultHelper(n)?(i.requestEmitHelper(a),e.createCall(e.getHelperName(\"__importDefault\"),void 0,[t])):t}function w(n){var t=e.getExternalModuleNameLiteral(n,_,d,u,c),r=[];return t&&r.push(t),e.createCall(e.createIdentifier(\"require\"),void 0,r)}function P(n,t,r){var a=Y(n);if(a){for(var i=e.isExportName(n)?t:e.createAssignment(n,t),o=0,s=a;o<s.length;o++){var l=s[o];e.setEmitFlags(i,4),i=z(l,i,r)}return i}return e.createAssignment(n,t)}function F(n){return e.isBindingPattern(n.name)?e.flattenDestructuringAssignment(e.visitNode(n,O),void 0,i,0,!1,P):e.createAssignment(e.setTextRange(e.createPropertyAccess(e.createIdentifier(\"exports\"),n.name),n.name),e.visitNode(n.initializer,O))}function G(n){return 0!=(4194304&e.getEmitFlags(n))}function V(e,n){if(v.exportEquals)return e;var t=n.importClause;if(!t)return e;t.name&&(e=j(e,t));var r=t.namedBindings;if(r)switch(r.kind){case 249:e=j(e,r);break;case 250:for(var a=0,i=r.elements;a<i.length;a++)e=j(e,i[a])}return e}function K(e,n){return v.exportEquals?e:j(e,n)}function B(e,n){if(v.exportEquals)return e;for(var t=0,r=n.declarationList.declarations;t<r.length;t++)e=H(e,r[t]);return e}function H(n,t){if(v.exportEquals)return n;if(e.isBindingPattern(t.name))for(var r=0,a=t.name.elements;r<a.length;r++){var i=a[r];e.isOmittedExpression(i)||(n=H(n,i))}else e.isGeneratedIdentifier(t.name)||(n=j(n,t));return n}function U(n,t){return v.exportEquals?n:(e.hasModifier(t,1)&&(n=W(n,e.hasModifier(t,512)?e.createIdentifier(\"default\"):e.getDeclarationName(t),e.getLocalName(t),t)),t.name&&(n=j(n,t)),n)}function j(n,t){var r=e.getDeclarationName(t),a=v.exportSpecifiers.get(e.idText(r));if(a)for(var i=0,o=a;i<o.length;i++){var s=o[i];n=W(n,s.name,r,s.name)}return n}function W(n,t,r,a,i){return n=e.append(n,function(n,t,r,a){var i=e.setTextRange(e.createExpressionStatement(z(n,t)),r);return e.startOnNewLine(i),a||e.setEmitFlags(i,1536),i}(t,r,a,i))}function q(){var n;return n=0===m?e.createExpressionStatement(z(e.createIdentifier(\"__esModule\"),e.createLiteral(!0))):e.createExpressionStatement(e.createCall(e.createPropertyAccess(e.createIdentifier(\"Object\"),\"defineProperty\"),void 0,[e.createIdentifier(\"exports\"),e.createLiteral(\"__esModule\"),e.createObjectLiteral([e.createPropertyAssignment(\"value\",e.createLiteral(!0))])])),e.setEmitFlags(n,1048576),n}function z(n,t,r){return e.setTextRange(e.createAssignment(e.createPropertyAccess(e.createIdentifier(\"exports\"),e.getSynthesizedClone(n)),t),r)}function J(e){switch(e.kind){case 84:case 79:return}return e}function X(n){if(4096&e.getEmitFlags(n)){var t=e.getExternalHelpersModuleName(_);return t?e.createPropertyAccess(t,n):n}if(!e.isGeneratedIdentifier(n)&&!e.isLocalName(n)){var r=u.getReferencedExportContainer(n,e.isExportName(n));if(r&&277===r.kind)return e.setTextRange(e.createPropertyAccess(e.createIdentifier(\"exports\"),e.getSynthesizedClone(n)),n);var a=u.getReferencedImportDeclaration(n);if(a){if(e.isImportClause(a))return e.setTextRange(e.createPropertyAccess(e.getGeneratedNameForNode(a.parent),e.createIdentifier(\"default\")),n);if(e.isImportSpecifier(a)){var i=a.propertyName||a.name;return e.setTextRange(e.createPropertyAccess(e.getGeneratedNameForNode(a.parent.parent.parent),e.getSynthesizedClone(i)),n)}}}return n}function Y(n){if(!e.isGeneratedIdentifier(n)){var t=u.getReferencedImportDeclaration(n)||u.getReferencedValueDeclaration(n);if(t)return v&&v.exportedBindings[e.getOriginalNodeId(t)]}}};var n={name:\"typescript:export-star\",scoped:!0,text:\"\\n            function __export(m) {\\n                for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];\\n            }\"};var t={name:\"typescript:dynamicimport-sync-require\",scoped:!0,text:'\\n            var __syncRequire = typeof module === \"object\" && typeof module.exports === \"object\";'},r={name:\"typescript:commonjsimportstar\",scoped:!1,text:'\\nvar __importStar = (this && this.__importStar) || function (mod) {\\n    if (mod && mod.__esModule) return mod;\\n    var result = {};\\n    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];\\n    result[\"default\"] = mod;\\n    return result;\\n};'},a={name:\"typescript:commonjsimportdefault\",scoped:!1,text:'\\nvar __importDefault = (this && this.__importDefault) || function (mod) {\\n    return (mod && mod.__esModule) ? mod : { \"default\": mod };\\n};'}}(Tn||(Tn={})),function(e){e.transformSystemModule=function(n){var t=n.startLexicalEnvironment,r=n.endLexicalEnvironment,a=n.hoistVariableDeclaration,i=n.getCompilerOptions(),o=n.getEmitResolver(),s=n.getEmitHost(),l=n.onSubstituteNode,c=n.onEmitNode;n.onSubstituteNode=function(n,t){return function(e){return _&&e.id&&_[e.id]}(t=l(n,t))?t:1===n?function(n){switch(n.kind){case 71:return function(n){if(4096&e.getEmitFlags(n)){var t=e.getExternalHelpersModuleName(u);return t?e.createPropertyAccess(t,n):n}if(!e.isGeneratedIdentifier(n)&&!e.isLocalName(n)){var r=o.getReferencedImportDeclaration(n);if(r){if(e.isImportClause(r))return e.setTextRange(e.createPropertyAccess(e.getGeneratedNameForNode(r.parent),e.createIdentifier(\"default\")),n);if(e.isImportSpecifier(r))return e.setTextRange(e.createPropertyAccess(e.getGeneratedNameForNode(r.parent.parent.parent),e.getSynthesizedClone(r.propertyName||r.name)),n)}}return n}(n);case 202:return function(n){if(e.isAssignmentOperator(n.operatorToken.kind)&&e.isIdentifier(n.left)&&!e.isGeneratedIdentifier(n.left)&&!e.isLocalName(n.left)&&!e.isDeclarationNameOfEnumOrNamespace(n.left)){var t=j(n.left);if(t){for(var r=n,a=0,i=t;a<i.length;a++){var o=i[a];r=G(o,W(r))}return r}}return n}(n);case 200:case 201:return function(n){if((43===n.operator||44===n.operator)&&e.isIdentifier(n.operand)&&!e.isGeneratedIdentifier(n.operand)&&!e.isLocalName(n.operand)&&!e.isDeclarationNameOfEnumOrNamespace(n.operand)){var t=j(n.operand);if(t){for(var r=201===n.kind?e.setTextRange(e.createPrefix(n.operator,n.operand),n):n,a=0,i=t;a<i.length;a++){var o=i[a];r=G(o,W(r))}return 201===n.kind&&(r=43===n.operator?e.createSubtract(W(r),e.createLiteral(1)):e.createAdd(W(r),e.createLiteral(1))),r}}return n}(n)}return n}(t):4===n?function(n){switch(n.kind){case 274:return function(n){var t=n.name;if(!e.isGeneratedIdentifier(t)&&!e.isLocalName(t)){var r=o.getReferencedImportDeclaration(t);if(r){if(e.isImportClause(r))return e.setTextRange(e.createPropertyAssignment(e.getSynthesizedClone(t),e.createPropertyAccess(e.getGeneratedNameForNode(r.parent),e.createIdentifier(\"default\"))),n);if(e.isImportSpecifier(r))return e.setTextRange(e.createPropertyAssignment(e.getSynthesizedClone(t),e.createPropertyAccess(e.getGeneratedNameForNode(r.parent.parent.parent),e.getSynthesizedClone(r.propertyName||r.name))),n)}}return n}(n)}return n}(t):t},n.onEmitNode=function(n,t,r){if(277===t.kind){var a=e.getOriginalNodeId(t);u=t,d=v[a],m=h[a],(_=b[a])&&delete b[a],c(n,t,r),u=void 0,d=void 0,m=void 0,_=void 0}else c(n,t,r)},n.enableSubstitution(71),n.enableSubstitution(274),n.enableSubstitution(202),n.enableSubstitution(200),n.enableSubstitution(201),n.enableEmitNotification(277);var u,d,m,p,f,g,_,v=[],y=[],h=[],b=[];return e.chainBundle(function(n){if(n.isDeclarationFile||!(e.isEffectiveExternalModule(n,i)||67108864&n.transformFlags))return n;var a=e.getOriginalNodeId(n);u=n,g=n,d=v[a]=e.collectExternalModuleInfo(n,o,i),m=e.createUniqueName(\"exports\"),h[a]=m,p=e.createUniqueName(\"context\");var l=function(n){for(var t=e.createMap(),r=[],a=0,l=n;a<l.length;a++){var c=l[a],d=e.getExternalModuleNameLiteral(c,u,s,o,i);if(d){var m=d.text,p=t.get(m);void 0!==p?r[p].externalImports.push(c):(t.set(m,r.length),r.push({name:d,externalImports:[c]}))}}return r}(d.externalImports),c=function(n,a){var o=[];t();var s=e.getStrictOptionValue(i,\"alwaysStrict\")||!i.noImplicitUseStrict&&e.isExternalModule(u),l=e.addPrologue(o,n.statements,s,T);o.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(\"__moduleName\",void 0,e.createLogicalAnd(p,e.createPropertyAccess(p,\"id\")))]))),e.visitNode(d.externalHelpersImportDeclaration,T,e.isStatement);var c=e.visitNodes(n.statements,T,e.isStatement,l);e.addRange(o,f),e.addStatementsAfterPrologue(o,r());var g=function(n){if(d.hasExportStarsToExportValues){if(!d.exportedNames&&0===d.exportSpecifiers.size){for(var t=!1,r=0,a=d.externalImports;r<a.length;r++){var i=a[r];if(253===i.kind&&i.exportClause){t=!0;break}}if(!t){var o=E(void 0);return n.push(o),o.name}}var s=[];if(d.exportedNames)for(var l=0,c=d.exportedNames;l<c.length;l++){var u=c[l];\"default\"!==u.escapedText&&s.push(e.createPropertyAssignment(e.createLiteral(u),e.createTrue()))}for(var m=0,p=d.externalImports;m<p.length;m++){var i=p[m];if(253===i.kind&&i.exportClause)for(var f=0,g=i.exportClause.elements;f<g.length;f++){var _=g[f];s.push(e.createPropertyAssignment(e.createLiteral(e.idText(_.name||_.propertyName)),e.createTrue()))}}var v=e.createUniqueName(\"exportedNames\");n.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(v,void 0,e.createObjectLiteral(s,!0))])));var y=E(v);return n.push(y),y.name}}(o),_=e.createObjectLiteral([e.createPropertyAssignment(\"setters\",function(n,t){for(var r=[],a=0,i=t;a<i.length;a++){for(var o=i[a],s=e.forEach(o.externalImports,function(n){return e.getLocalNameForExternalImport(n,u)}),l=s?e.getGeneratedNameForNode(s):e.createUniqueName(\"\"),c=[],d=0,p=o.externalImports;d<p.length;d++){var f=p[d],g=e.getLocalNameForExternalImport(f,u);switch(f.kind){case 247:if(!f.importClause)break;case 246:e.Debug.assert(void 0!==g),c.push(e.createExpressionStatement(e.createAssignment(g,l)));break;case 253:if(e.Debug.assert(void 0!==g),f.exportClause){for(var _=[],v=0,y=f.exportClause.elements;v<y.length;v++){var h=y[v];_.push(e.createPropertyAssignment(e.createLiteral(e.idText(h.name)),e.createElementAccess(l,e.createLiteral(e.idText(h.propertyName||h.name)))))}c.push(e.createExpressionStatement(e.createCall(m,void 0,[e.createObjectLiteral(_,!0)])))}else c.push(e.createExpressionStatement(e.createCall(n,void 0,[l])))}}r.push(e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,l)],void 0,e.createBlock(c,!0)))}return e.createArrayLiteral(r,!0)}(g,a)),e.createPropertyAssignment(\"execute\",e.createFunctionExpression(void 0,void 0,void 0,void 0,[],void 0,e.createBlock(c,!0)))]);return _.multiLine=!0,o.push(e.createReturn(_)),e.createBlock(o,!0)}(n,l),y=e.createFunctionExpression(void 0,void 0,void 0,void 0,[e.createParameter(void 0,void 0,void 0,m),e.createParameter(void 0,void 0,void 0,p)],void 0,c),S=e.tryGetModuleNameFromFile(n,s,i),L=e.createArrayLiteral(e.map(l,function(e){return e.name})),x=e.setEmitFlags(e.updateSourceFileNode(n,e.setTextRange(e.createNodeArray([e.createExpressionStatement(e.createCall(e.createPropertyAccess(e.createIdentifier(\"System\"),\"register\"),void 0,S?[S,L,y]:[L,y]))]),n.statements)),1024);return i.outFile||i.out||e.moveEmitHelpers(x,c,function(e){return!e.scoped}),_&&(b[a]=_,_=void 0),u=void 0,d=void 0,m=void 0,p=void 0,f=void 0,g=void 0,e.aggregateTransformFlags(x)});function E(n){var t=e.createUniqueName(\"exportStar\"),r=e.createIdentifier(\"m\"),a=e.createIdentifier(\"n\"),i=e.createIdentifier(\"exports\"),o=e.createStrictInequality(a,e.createLiteral(\"default\"));return n&&(o=e.createLogicalAnd(o,e.createLogicalNot(e.createCall(e.createPropertyAccess(n,\"hasOwnProperty\"),void 0,[a])))),e.createFunctionDeclaration(void 0,void 0,void 0,t,void 0,[e.createParameter(void 0,void 0,void 0,r)],void 0,e.createBlock([e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(i,void 0,e.createObjectLiteral([]))])),e.createForIn(e.createVariableDeclarationList([e.createVariableDeclaration(a,void 0)]),r,e.createBlock([e.setEmitFlags(e.createIf(o,e.createExpressionStatement(e.createAssignment(e.createElementAccess(i,a),e.createElementAccess(r,a)))),1)])),e.createExpressionStatement(e.createCall(m,void 0,[i]))],!0))}function T(n){switch(n.kind){case 247:return function(n){var t;if(n.importClause&&a(e.getLocalNameForExternalImport(n,u)),k(n)){var r=e.getOriginalNodeId(n);y[r]=M(y[r],n)}else t=M(t,n);return e.singleOrMany(t)}(n);case 246:return function(n){var t;if(e.Debug.assert(e.isExternalModuleImportEqualsDeclaration(n),\"import= for internal module references should be handled in an earlier transformer.\"),a(e.getLocalNameForExternalImport(n,u)),k(n)){var r=e.getOriginalNodeId(n);y[r]=O(y[r],n)}else t=O(t,n);return e.singleOrMany(t)}(n);case 253:return;case 252:return function(n){if(!n.isExportEquals){var t=e.visitNode(n.expression,B,e.isExpression),r=n.original;if(!r||!k(r))return F(e.createIdentifier(\"default\"),t,!0);var a=e.getOriginalNodeId(n);y[a]=P(y[a],e.createIdentifier(\"default\"),t,!0)}}(n);default:return V(n)}}function S(n){if(e.isBindingPattern(n.name))for(var t=0,r=n.name.elements;t<r.length;t++){var i=r[t];e.isOmittedExpression(i)||S(i)}else a(e.getSynthesizedClone(n.name))}function L(n){return 0==(2097152&e.getEmitFlags(n))&&(277===g.kind||0==(3&e.getOriginalNode(n).flags))}function x(t,r){var a=r?A:C;return e.isBindingPattern(t.name)?e.flattenDestructuringAssignment(t,B,n,0,!1,a):t.initializer?a(t.name,e.visitNode(t.initializer,B,e.isExpression)):t.name}function A(e,n,t){return D(e,n,t,!0)}function C(e,n,t){return D(e,n,t,!1)}function D(n,t,r,i){return a(e.getSynthesizedClone(n)),i?G(n,W(e.setTextRange(e.createAssignment(n,t),r))):W(e.setTextRange(e.createAssignment(n,t),r))}function k(n){return 0!=(4194304&e.getEmitFlags(n))}function M(e,n){if(d.exportEquals)return e;var t=n.importClause;if(!t)return e;t.name&&(e=w(e,t));var r=t.namedBindings;if(r)switch(r.kind){case 249:e=w(e,r);break;case 250:for(var a=0,i=r.elements;a<i.length;a++)e=w(e,i[a])}return e}function O(e,n){return d.exportEquals?e:w(e,n)}function I(e,n,t){if(d.exportEquals)return e;for(var r=0,a=n.declarationList.declarations;r<a.length;r++){var i=a[r];(i.initializer||t)&&(e=R(e,i,t))}return e}function R(n,t,r){if(d.exportEquals)return n;if(e.isBindingPattern(t.name))for(var a=0,i=t.name.elements;a<i.length;a++){var o=i[a];e.isOmittedExpression(o)||(n=R(n,o,r))}else if(!e.isGeneratedIdentifier(t.name)){var s=void 0;r&&(n=P(n,t.name,e.getLocalName(t)),s=e.idText(t.name)),n=w(n,t,s)}return n}function N(n,t){if(d.exportEquals)return n;var r;if(e.hasModifier(t,1)){var a=e.hasModifier(t,512)?e.createLiteral(\"default\"):t.name;n=P(n,a,e.getLocalName(t)),r=e.getTextOfIdentifierOrLiteral(a)}return t.name&&(n=w(n,t,r)),n}function w(n,t,r){if(d.exportEquals)return n;var a=e.getDeclarationName(t),i=d.exportSpecifiers.get(e.idText(a));if(i)for(var o=0,s=i;o<s.length;o++){var l=s[o];l.name.escapedText!==r&&(n=P(n,l.name,a))}return n}function P(n,t,r,a){return n=e.append(n,F(t,r,a))}function F(n,t,r){var a=e.createExpressionStatement(G(n,t));return e.startOnNewLine(a),r||e.setEmitFlags(a,1536),a}function G(n,t){var r=e.isIdentifier(n)?e.createLiteral(n):n;return e.setEmitFlags(t,1536|e.getEmitFlags(t)),e.setCommentRange(e.createCall(m,void 0,[r,t]),t)}function V(t){switch(t.kind){case 217:return function(n){if(!L(n.declarationList))return e.visitNode(n,B,e.isStatement);for(var t,r,a=e.hasModifier(n,1),i=k(n),o=0,s=n.declarationList.declarations;o<s.length;o++){var l=s[o];l.initializer?t=e.append(t,x(l,a&&!i)):S(l)}if(t&&(r=e.append(r,e.setTextRange(e.createExpressionStatement(e.inlineExpressions(t)),n))),i){var c=e.getOriginalNodeId(n);y[c]=I(y[c],n,a)}else r=I(r,n,!1);return e.singleOrMany(r)}(t);case 237:return function(t){if(f=e.hasModifier(t,1)?e.append(f,e.updateFunctionDeclaration(t,t.decorators,e.visitNodes(t.modifiers,U,e.isModifier),t.asteriskToken,e.getDeclarationName(t,!0,!0),void 0,e.visitNodes(t.parameters,B,e.isParameterDeclaration),void 0,e.visitNode(t.body,B,e.isBlock))):e.append(f,e.visitEachChild(t,B,n)),k(t)){var r=e.getOriginalNodeId(t);y[r]=N(y[r],t)}else f=N(f,t)}(t);case 238:return function(n){var t,r=e.getLocalName(n);if(a(r),t=e.append(t,e.setTextRange(e.createExpressionStatement(e.createAssignment(r,e.setTextRange(e.createClassExpression(void 0,n.name,void 0,e.visitNodes(n.heritageClauses,B,e.isHeritageClause),e.visitNodes(n.members,B,e.isClassElement)),n))),n)),k(n)){var i=e.getOriginalNodeId(n);y[i]=N(y[i],n)}else t=N(t,n);return e.singleOrMany(t)}(t);case 223:return function(n){var t=g;return g=n,n=e.updateFor(n,n.initializer&&K(n.initializer),e.visitNode(n.condition,B,e.isExpression),e.visitNode(n.incrementor,B,e.isExpression),e.visitNode(n.statement,V,e.isStatement)),g=t,n}(t);case 224:return function(n){var t=g;return g=n,n=e.updateForIn(n,K(n.initializer),e.visitNode(n.expression,B,e.isExpression),e.visitNode(n.statement,V,e.isStatement,e.liftToBlock)),g=t,n}(t);case 225:return function(n){var t=g;return g=n,n=e.updateForOf(n,n.awaitModifier,K(n.initializer),e.visitNode(n.expression,B,e.isExpression),e.visitNode(n.statement,V,e.isStatement,e.liftToBlock)),g=t,n}(t);case 221:return function(n){return e.updateDo(n,e.visitNode(n.statement,V,e.isStatement,e.liftToBlock),e.visitNode(n.expression,B,e.isExpression))}(t);case 222:return function(n){return e.updateWhile(n,e.visitNode(n.expression,B,e.isExpression),e.visitNode(n.statement,V,e.isStatement,e.liftToBlock))}(t);case 231:return function(n){return e.updateLabel(n,n.label,e.visitNode(n.statement,V,e.isStatement,e.liftToBlock))}(t);case 229:return function(n){return e.updateWith(n,e.visitNode(n.expression,B,e.isExpression),e.visitNode(n.statement,V,e.isStatement,e.liftToBlock))}(t);case 230:return function(n){return e.updateSwitch(n,e.visitNode(n.expression,B,e.isExpression),e.visitNode(n.caseBlock,V,e.isCaseBlock))}(t);case 244:return function(n){var t=g;return g=n,n=e.updateCaseBlock(n,e.visitNodes(n.clauses,V,e.isCaseOrDefaultClause)),g=t,n}(t);case 269:return function(n){return e.updateCaseClause(n,e.visitNode(n.expression,B,e.isExpression),e.visitNodes(n.statements,V,e.isStatement))}(t);case 270:case 233:return function(t){return e.visitEachChild(t,V,n)}(t);case 272:return function(n){var t=g;return g=n,n=e.updateCatchClause(n,n.variableDeclaration,e.visitNode(n.block,V,e.isBlock)),g=t,n}(t);case 216:return function(t){var r=g;return g=t,t=e.visitEachChild(t,V,n),g=r,t}(t);case 308:return function(n){if(k(n)&&217===n.original.kind){var t=e.getOriginalNodeId(n),r=e.hasModifier(n.original,1);y[t]=I(y[t],n.original,r)}return n}(t);case 309:return function(n){var t=e.getOriginalNodeId(n),r=y[t];if(r)return delete y[t],e.append(r,n);var a=e.getOriginalNode(n);return e.isModuleOrEnumDeclaration(a)?e.append(w(r,a),n):n}(t);default:return B(t)}}function K(t){if(function(n){return e.isVariableDeclarationList(n)&&L(n)}(t)){for(var r=void 0,a=0,i=t.declarations;a<i.length;a++){var o=i[a];r=e.append(r,x(o,!1)),o.initializer||S(o)}return r?e.inlineExpressions(r):e.createOmittedExpression()}return e.visitEachChild(t,V,n)}function B(t){return 1024&t.transformFlags&&202===t.kind?function(t){return H(t.left)?e.flattenDestructuringAssignment(t,B,n,0,!0):e.visitEachChild(t,B,n)}(t):e.isImportCall(t)?function(n){return e.createCall(e.createPropertyAccess(p,e.createIdentifier(\"import\")),void 0,e.some(n.arguments)?[e.visitNode(n.arguments[0],B)]:[])}(t):2048&t.transformFlags||67108864&t.transformFlags?e.visitEachChild(t,B,n):t}function H(n){if(e.isAssignmentExpression(n,!0))return H(n.left);if(e.isSpreadElement(n))return H(n.expression);if(e.isObjectLiteralExpression(n))return e.some(n.properties,H);if(e.isArrayLiteralExpression(n))return e.some(n.elements,H);if(e.isShorthandPropertyAssignment(n))return H(n.name);if(e.isPropertyAssignment(n))return H(n.initializer);if(e.isIdentifier(n)){var t=o.getReferencedExportContainer(n);return void 0!==t&&277===t.kind}return!1}function U(e){switch(e.kind){case 84:case 79:return}return e}function j(n){var t;if(!e.isGeneratedIdentifier(n)){var r=o.getReferencedImportDeclaration(n)||o.getReferencedValueDeclaration(n);if(r){var a=o.getReferencedExportContainer(n,!1);a&&277===a.kind&&(t=e.append(t,e.getDeclarationName(r))),t=e.addRange(t,d&&d.exportedBindings[e.getOriginalNodeId(r)])}}return t}function W(n){return void 0===_&&(_=[]),_[e.getNodeId(n)]=!0,n}}}(Tn||(Tn={})),function(e){e.transformES2015Module=function(n){var t,r=n.getCompilerOptions(),a=n.onEmitNode,i=n.onSubstituteNode;return n.onEmitNode=function(n,r,i){e.isSourceFile(r)?(t=r,a(n,r,i),t=void 0):a(n,r,i)},n.onSubstituteNode=function(n,r){return r=i(n,r),e.isIdentifier(r)&&1===n?function(n){if(4096&e.getEmitFlags(n)){var r=e.getExternalHelpersModuleName(t);if(r)return e.createPropertyAccess(r,n)}return n}(r):r},n.enableEmitNotification(277),n.enableSubstitution(71),e.chainBundle(function(t){if(t.isDeclarationFile)return t;if(e.isExternalModule(t)||r.isolatedModules){var a=e.getOrCreateExternalHelpersModuleNameIfNeeded(t,r);if(a){var i=[],s=e.addPrologue(i,t.statements),l=e.createImportDeclaration(void 0,void 0,e.createImportClause(void 0,e.createNamespaceImport(a)),e.createLiteral(e.externalHelpersModuleNameText));return e.addEmitFlags(l,67108864),e.append(i,l),e.addRange(i,e.visitNodes(t.statements,o,e.isStatement,s)),e.updateSourceFileNode(t,e.setTextRange(e.createNodeArray(i),t.statements))}return e.visitEachChild(t,o,n)}return t});function o(e){switch(e.kind){case 246:return;case 252:return function(e){return e.isExportEquals?void 0:e}(e)}return e}}}(Tn||(Tn={})),function(e){function n(n){return e.isVariableDeclaration(n)||e.isPropertyDeclaration(n)||e.isPropertySignature(n)||e.isBindingElement(n)||e.isConstructorDeclaration(n)?t:e.isSetAccessor(n)||e.isGetAccessor(n)?function(t){var r;r=157===n.kind?e.hasModifier(n,32)?t.errorModuleName?e.Diagnostics.Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1:e.hasModifier(n,32)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1:t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1;return{diagnosticMessage:r,errorNode:n.name,typeName:n.name}}:e.isConstructSignatureDeclaration(n)||e.isCallSignatureDeclaration(n)||e.isMethodDeclaration(n)||e.isMethodSignature(n)||e.isFunctionDeclaration(n)||e.isIndexSignatureDeclaration(n)?function(t){var r;switch(n.kind){case 159:r=t.errorModuleName?e.Diagnostics.Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0;break;case 158:r=t.errorModuleName?e.Diagnostics.Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0;break;case 160:r=t.errorModuleName?e.Diagnostics.Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0;break;case 154:case 153:r=e.hasModifier(n,32)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:e.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0:238===n.parent.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:e.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0:t.errorModuleName?e.Diagnostics.Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0;break;case 237:r=t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:e.Diagnostics.Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_exported_function_has_or_is_using_private_name_0;break;default:return e.Debug.fail(\"This is unknown kind for signature: \"+n.kind)}return{diagnosticMessage:r,errorNode:n.name||n}}:e.isParameter(n)?e.isParameterPropertyDeclaration(n)&&e.hasModifier(n.parent,8)?t:function(t){var r=function(t){switch(n.parent.kind){case 155:return t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1;case 159:case 164:return t.errorModuleName?e.Diagnostics.Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1;case 158:return t.errorModuleName?e.Diagnostics.Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1;case 160:return t.errorModuleName?e.Diagnostics.Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1;case 154:case 153:return e.hasModifier(n.parent,32)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:238===n.parent.parent.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1;case 237:case 163:return t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_private_name_1;default:return e.Debug.fail(\"Unknown parent for parameter: \"+e.SyntaxKind[n.parent.kind])}}(t);return void 0!==r?{diagnosticMessage:r,errorNode:n,typeName:n.name}:void 0}:e.isTypeParameterDeclaration(n)?function(){var t;switch(n.parent.kind){case 238:t=e.Diagnostics.Type_parameter_0_of_exported_class_has_or_is_using_private_name_1;break;case 239:t=e.Diagnostics.Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1;break;case 159:t=e.Diagnostics.Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1;break;case 158:t=e.Diagnostics.Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1;break;case 154:case 153:t=e.hasModifier(n.parent,32)?e.Diagnostics.Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:238===n.parent.parent.kind?e.Diagnostics.Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:e.Diagnostics.Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1;break;case 237:t=e.Diagnostics.Type_parameter_0_of_exported_function_has_or_is_using_private_name_1;break;case 240:t=e.Diagnostics.Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1;break;default:return e.Debug.fail(\"This is unknown parent for type parameter: \"+n.parent.kind)}return{diagnosticMessage:t,errorNode:n,typeName:n.name}}:e.isExpressionWithTypeArguments(n)?function(){var t;t=238===n.parent.parent.kind?108===n.parent.token?e.Diagnostics.Implements_clause_of_exported_class_0_has_or_is_using_private_name_1:e.Diagnostics.extends_clause_of_exported_class_0_has_or_is_using_private_name_1:e.Diagnostics.extends_clause_of_exported_interface_0_has_or_is_using_private_name_1;return{diagnosticMessage:t,errorNode:n,typeName:e.getNameOfDeclaration(n.parent.parent)}}:e.isImportEqualsDeclaration(n)?function(){return{diagnosticMessage:e.Diagnostics.Import_declaration_0_is_using_private_name_1,errorNode:n,typeName:n.name}}:e.isTypeAliasDeclaration(n)?function(){return{diagnosticMessage:e.Diagnostics.Exported_type_alias_0_has_or_is_using_private_name_1,errorNode:n.type,typeName:n.name}}:e.Debug.assertNever(n,\"Attempted to set a declaration diagnostic context for unhandled node kind: \"+e.SyntaxKind[n.kind]);function t(t){var r=function(t){return 235===n.kind||184===n.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Exported_variable_0_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Exported_variable_0_has_or_is_using_private_name_1:152===n.kind||151===n.kind||149===n.kind&&e.hasModifier(n.parent,8)?e.hasModifier(n,32)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_private_name_1:238===n.parent.kind||149===n.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_private_name_1:void 0}(t);return void 0!==r?{diagnosticMessage:r,errorNode:n,typeName:n.name}:void 0}}e.canProduceDiagnostics=function(n){return e.isVariableDeclaration(n)||e.isPropertyDeclaration(n)||e.isPropertySignature(n)||e.isBindingElement(n)||e.isSetAccessor(n)||e.isGetAccessor(n)||e.isConstructSignatureDeclaration(n)||e.isCallSignatureDeclaration(n)||e.isMethodDeclaration(n)||e.isMethodSignature(n)||e.isFunctionDeclaration(n)||e.isParameter(n)||e.isTypeParameterDeclaration(n)||e.isExpressionWithTypeArguments(n)||e.isImportEqualsDeclaration(n)||e.isTypeAliasDeclaration(n)||e.isConstructorDeclaration(n)||e.isIndexSignatureDeclaration(n)},e.createGetSymbolAccessibilityDiagnosticForNodeName=function(t){return e.isSetAccessor(t)||e.isGetAccessor(t)?function(n){var r=function(n){return e.hasModifier(t,32)?n.errorModuleName?2===n.accessibility?e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_private_name_1:238===t.parent.kind?n.errorModuleName?2===n.accessibility?e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_private_name_1:n.errorModuleName?e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_private_name_1}(n);return void 0!==r?{diagnosticMessage:r,errorNode:t,typeName:t.name}:void 0}:e.isMethodSignature(t)||e.isMethodDeclaration(t)?function(n){var r=function(n){return e.hasModifier(t,32)?n.errorModuleName?2===n.accessibility?e.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_private_name_1:238===t.parent.kind?n.errorModuleName?2===n.accessibility?e.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_private_name_1:n.errorModuleName?e.Diagnostics.Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Method_0_of_exported_interface_has_or_is_using_private_name_1}(n);return void 0!==r?{diagnosticMessage:r,errorNode:t,typeName:t.name}:void 0}:n(t)},e.createGetSymbolAccessibilityDiagnosticForNode=n}(Tn||(Tn={})),function(e){e.getDeclarationDiagnostics=function(n,r,a){if(a&&e.isSourceFileJS(a))return[];var i=n.getCompilerOptions();return e.transformNodes(r,n,i,a?[a]:e.filter(n.getSourceFiles(),e.isSourceFileNotJS),[t],!1).diagnostics};var n=531469;function t(t){var o,s,l,c,u,d,m,p,f,g,_=function(){return e.Debug.fail(\"Diagnostic emitted without context\")},v=_,y=!0,h=!1,b=!1,E=!1,T=!1,S=t.getEmitHost(),L={trackSymbol:function(e,n,t){if(262144&e.flags)return;O(x.isSymbolAccessible(e,n,t,!0)),M(x.getTypeReferenceDirectivesForSymbol(e,t))},reportInaccessibleThisError:function(){m&&t.addDiagnostic(e.createDiagnosticForNode(m,e.Diagnostics.The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary,e.declarationNameToString(m),\"this\"))},reportInaccessibleUniqueSymbolError:function(){m&&t.addDiagnostic(e.createDiagnosticForNode(m,e.Diagnostics.The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary,e.declarationNameToString(m),\"unique symbol\"))},reportPrivateInBaseOfClassExpression:function(n){m&&t.addDiagnostic(e.createDiagnosticForNode(m,e.Diagnostics.Property_0_of_exported_class_expression_may_not_be_private_or_protected,n))},moduleResolverHost:S,trackReferencedAmbientModule:function(n,t){var r=x.getTypeReferenceDirectivesForSymbol(t,67108863);if(e.length(r))return M(r);var a=e.getSourceFileOfNode(n);f.set(\"\"+e.getOriginalNodeId(a),a)},trackExternalModuleSymbolOfImportTypeNode:function(e){h||(d||(d=[])).push(e)}},x=t.getEmitResolver(),A=t.getCompilerOptions(),C=e.getNewLineCharacter(A),D=A.noResolve,k=A.stripInternal;return function(n){if(277===n.kind&&(n.isDeclarationFile||e.isSourceFileJS(n)))return n;if(278===n.kind){h=!0,f=e.createMap(),g=e.createMap();var r=!1,a=e.createBundle(e.map(n.sourceFiles,function(n){if(!n.isDeclarationFile&&!e.isSourceFileJS(n)){if(r=r||n.hasNoDefaultLib,p=n,o=n,l=void 0,u=!1,c=e.createMap(),v=_,E=!1,T=!1,I(n,f),R(n,g),e.isExternalModule(n)){b=!1,y=!1;var a=e.visitNodes(n.statements,Y),i=e.updateSourceFileNode(n,[e.createModuleDeclaration([],[e.createModifier(124)],e.createLiteral(e.getResolvedExternalModuleName(t.getEmitHost(),n)),e.createModuleBlock(e.setTextRange(e.createNodeArray(q(a)),n.statements)))],!0,[],[],!1,[]);return i}y=!0;var s=e.visitNodes(n.statements,Y);return e.updateSourceFileNode(n,q(s),!0,[],[],!1,[])}}),e.mapDefined(n.prepends,function(n){if(280===n.kind)return e.createUnparsedSourceFile(n.declarationText,n.declarationMapPath,n.declarationMapText)}));a.syntheticFileReferences=[],a.syntheticTypeReferences=N(),a.syntheticLibReferences=O(),a.hasNoDefaultLib=r;var i=e.getDirectoryPath(e.normalizeSlashes(e.getOutputPathsFor(n,S,!0).declarationFilePath)),m=P(a.syntheticFileReferences,i);return f.forEach(m),a}y=!0,E=!1,T=!1,o=n,p=n,v=_,h=!1,b=!1,u=!1,l=void 0,c=e.createMap(),s=void 0,f=I(p,e.createMap()),g=R(p,e.createMap());var L=[],x=e.getDirectoryPath(e.normalizeSlashes(e.getOutputPathsFor(n,S,!0).declarationFilePath)),A=P(L,x),C=e.visitNodes(n.statements,Y),D=e.setTextRange(e.createNodeArray(q(C)),n.statements);f.forEach(A);var k=e.filter(D,e.isAnyImportSyntax);e.isExternalModule(n)&&(!b||E&&!T)&&(D=e.setTextRange(e.createNodeArray(D.concat([e.createExportDeclaration(void 0,void 0,e.createNamedExports([]),void 0)])),D));var M=e.updateSourceFileNode(n,D,!0,L,N(),n.hasNoDefaultLib,O());return M.exportedModulesFromDeclarationEmit=d,M;function O(){return e.map(e.arrayFrom(g.keys()),function(e){return{fileName:e,pos:-1,end:-1}})}function N(){return s?e.mapDefined(e.arrayFrom(s.keys()),w):[]}function w(n){if(k)for(var t=0,r=k;t<r.length;t++){var a=r[t];if(e.isImportEqualsDeclaration(a)&&e.isExternalModuleReference(a.moduleReference)){var i=a.moduleReference.expression;if(e.isStringLiteralLike(i)&&i.text===n)return}else if(e.isImportDeclaration(a)&&e.isStringLiteral(a.moduleSpecifier)&&a.moduleSpecifier.text===n)return}return{fileName:n,pos:-1,end:-1}}function P(t,r){return function(a){var i;if(a.isDeclarationFile)i=a.fileName;else{if(h&&e.contains(n.sourceFiles,a))return;var o=e.getOutputPathsFor(a,S,!0);i=o.declarationFilePath||o.jsFilePath}if(i){var s=e.getRelativePathToDirectoryOrUrl(r,i,S.getCurrentDirectory(),S.getCanonicalFileName,!1);if(e.startsWith(s,\"./\")&&e.hasExtension(s)&&(s=s.substring(2)),e.startsWith(s,\"node_modules/\")||-1!==s.indexOf(\"/node_modules/\"))return;t.push({pos:-1,end:-1,fileName:s})}}}};function M(n){if(n){s=s||e.createMap();for(var t=0,r=n;t<r.length;t++){var a=r[t];s.set(a,!0)}}}function O(n){if(0===n.accessibility){if(n&&n.aliasesToMakeVisible)if(l)for(var r=0,a=n.aliasesToMakeVisible;r<a.length;r++){var i=a[r];e.pushIfUnique(l,i)}else l=n.aliasesToMakeVisible}else{var o=v(n);o&&(o.typeName?t.addDiagnostic(e.createDiagnosticForNode(n.errorNode||o.errorNode,o.diagnosticMessage,e.getTextOfNode(o.typeName),n.errorSymbolName,n.errorModuleName)):t.addDiagnostic(e.createDiagnosticForNode(n.errorNode||o.errorNode,o.diagnosticMessage,n.errorSymbolName,n.errorModuleName)))}}function I(n,t){return D||e.isSourceFileJS(n)?t:(e.forEach(n.referencedFiles,function(r){var a=e.tryResolveScriptReference(S,n,r);a&&t.set(\"\"+e.getOriginalNodeId(a),a)}),t)}function R(n,t){return e.forEach(n.libReferenceDirectives,function(e){S.getLibFileFromReference(e)&&t.set(e.fileName.toLocaleLowerCase(),!0)}),t}function N(n,t){var a;u||(a=v,v=e.createGetSymbolAccessibilityDiagnosticForNode(n));var i=e.updateParameter(n,void 0,r(n,t),n.dotDotDotToken,function n(t){return 71===t.kind?t:183===t.kind?e.updateArrayBindingPattern(t,e.visitNodes(t.elements,r)):e.updateObjectBindingPattern(t,e.visitNodes(t.elements,r));function r(t){return 208===t.kind?t:e.updateBindingElement(t,t.dotDotDotToken,t.propertyName,n(t.name),w(t)?t.initializer:void 0)}}(n.name),x.isOptionalParameter(n)?n.questionToken||e.createToken(55):void 0,F(n,n.type,!0),P(n));return u||(v=a),i}function w(n){return function(n){switch(n.kind){case 152:case 151:return!e.hasModifier(n,8);case 149:case 235:return!0}return!1}(n)&&x.isLiteralConstDeclaration(e.getParseTreeNode(n))}function P(n){if(w(n))return x.createLiteralConstValue(e.getParseTreeNode(n))}function F(t,r,a){if((a||!e.hasModifier(t,8))&&!w(t)){var i,s=149===t.kind&&(x.isRequiredInitializedParameter(t)||x.isOptionalUninitializedParameterProperty(t));return r&&!s?e.visitNode(r,X):e.getParseTreeNode(t)?157===t.kind?e.createKeywordTypeNode(119):(m=t.name,u||(i=v,v=e.createGetSymbolAccessibilityDiagnosticForNode(t)),235===t.kind||184===t.kind?l(x.createTypeOfDeclaration(t,o,n,L)):149===t.kind||152===t.kind||151===t.kind?t.initializer?l(x.createTypeOfDeclaration(t,o,n,L,s)||x.createTypeOfExpression(t.initializer,o,n,L)):l(x.createTypeOfDeclaration(t,o,n,L,s)):l(x.createReturnTypeOfSignatureDeclaration(t,o,n,L))):r?e.visitNode(r,X):e.createKeywordTypeNode(119)}function l(n){return m=void 0,u||(v=i),n||e.createKeywordTypeNode(119)}}function G(n){switch((n=e.getParseTreeNode(n)).kind){case 237:case 242:case 239:case 238:case 240:case 241:return!x.isDeclarationVisible(n);case 235:return!V(n);case 246:case 247:case 253:case 252:return!1}return!1}function V(n){return!e.isOmittedExpression(n)&&(e.isBindingPattern(n.name)?e.some(n.name.elements,V):x.isDeclarationVisible(n))}function K(n,t,r){if(!e.hasModifier(n,8)){var a=e.map(t,function(e){return N(e,r)});if(a)return e.createNodeArray(a,t.hasTrailingComma)}}function B(n,t){return e.hasModifier(n,8)?void 0:e.visitNodes(t,X)}function H(n){return e.isSourceFile(n)||e.isTypeAliasDeclaration(n)||e.isModuleDeclaration(n)||e.isClassDeclaration(n)||e.isInterfaceDeclaration(n)||e.isFunctionLike(n)||e.isIndexSignatureDeclaration(n)||e.isMappedTypeNode(n)}function U(e,n){O(x.isEntityNameVisible(e,n)),M(x.getTypeReferenceDirectivesForEntityName(e))}function j(n,t){return e.hasJSDocNodes(n)&&e.hasJSDocNodes(t)&&(n.jsDoc=t.jsDoc),e.setCommentRange(n,e.getCommentRange(t))}function W(n,r){if(r){if(b=b||242!==n.kind&&181!==n.kind,e.isStringLiteralLike(r))if(h){var a=e.getExternalModuleNameFromDeclaration(t.getEmitHost(),x,n);if(a)return e.createLiteral(a)}else{var i=x.getSymbolOfExternalModuleSpecifier(r);i&&(d||(d=[])).push(i)}return r}}function q(n){for(;e.length(l);){var t=l.shift();if(!e.isLateVisibilityPaintedStatement(t))return e.Debug.fail(\"Late replaced statement was found which is not handled by the declaration transformer!: \"+(e.SyntaxKind?e.SyntaxKind[t.kind]:t.kind));var r=Q(t,!0);c.set(\"\"+e.getOriginalNodeId(t),r)}return e.visitNodes(n,function(n){if(e.isLateVisibilityPaintedStatement(n)){var t=\"\"+e.getOriginalNodeId(n);if(c.has(t)){var r=c.get(t);return c.delete(t),r&&e.isSourceFile(n.parent)&&((e.isArray(r)?e.some(r,J):J(r))&&(E=!0),(e.isArray(r)?e.some(r,z):z(r))&&(b=!0)),r}}return n})}function z(n){return e.isAnyImportOrReExport(n)||e.isExportAssignment(n)||e.hasModifier(n,1)}function J(n){return!(e.isAnyImportOrReExport(n)||e.isExportAssignment(n)||e.hasModifier(n,1)||e.isAmbientModule(n))}function X(n){if(!ee(n)){if(e.isDeclaration(n)){if(G(n))return;if(e.hasDynamicName(n)&&!x.isLateBound(e.getParseTreeNode(n)))return}if(!(e.isFunctionLike(n)&&x.isImplementationOfOverload(n)||e.isSemicolonClassElement(n))){var r;H(n)&&(r=o,o=n);var a=v;if((e.isMethodDeclaration(n)||e.isMethodSignature(n))&&e.hasModifier(n,8)){if(n.symbol&&n.symbol.declarations&&n.symbol.declarations[0]!==n)return;return E(e.createProperty(void 0,te(n),n.name,void 0,void 0,void 0))}var i=e.canProduceDiagnostics(n);i&&!u&&(v=e.createGetSymbolAccessibilityDiagnosticForNode(n)),e.isTypeQueryNode(n)&&U(n.exprName,o);var s=u,l=(166===n.kind||179===n.kind)&&240!==n.parent.kind;if(l&&(u=!0),function(e){switch(e.kind){case 159:case 155:case 154:case 156:case 157:case 152:case 151:case 153:case 158:case 160:case 235:case 148:case 209:case 162:case 173:case 163:case 164:case 181:return!0}return!1}(n))switch(n.kind){case 209:(e.isEntityName(n.expression)||e.isEntityNameExpression(n.expression))&&U(n.expression,o);var c=e.visitEachChild(n,X,t);return E(e.updateExpressionWithTypeArguments(c,e.parenthesizeTypeParameters(c.typeArguments),c.expression));case 162:U(n.typeName,o);c=e.visitEachChild(n,X,t);return E(e.updateTypeReferenceNode(c,c.typeName,e.parenthesizeTypeParameters(c.typeArguments)));case 159:return E(e.updateConstructSignature(n,B(n,n.typeParameters),K(n,n.parameters),F(n,n.type)));case 155:var d=e.hasModifier(n,8),p=e.createSignatureDeclaration(155,d?void 0:B(n,n.typeParameters),d?void 0:K(n,n.parameters,0),void 0);return p.modifiers=e.createNodeArray(te(n)),E(p);case 154:var f=e.createSignatureDeclaration(153,B(n,n.typeParameters),K(n,n.parameters),F(n,n.type));return f.name=n.name,f.modifiers=e.createNodeArray(te(n)),f.questionToken=n.questionToken,E(f);case 156:case 157:return E(re(n));case 152:return E(e.updateProperty(n,void 0,te(n),n.name,n.questionToken,e.hasModifier(n,8)?void 0:F(n,n.type),P(n)));case 151:return E(e.updatePropertySignature(n,te(n),n.name,n.questionToken,e.hasModifier(n,8)?void 0:F(n,n.type),P(n)));case 153:return E(e.updateMethodSignature(n,B(n,n.typeParameters),K(n,n.parameters),F(n,n.type),n.name,n.questionToken));case 158:return E(e.updateCallSignature(n,B(n,n.typeParameters),K(n,n.parameters),F(n,n.type)));case 160:return E(e.updateIndexSignature(n,void 0,te(n),K(n,n.parameters),e.visitNode(n.type,X)||e.createKeywordTypeNode(119)));case 235:return e.isBindingPattern(n.name)?Z(n.name):(l=!0,u=!0,E(e.updateVariableDeclaration(n,n.name,F(n,n.type),P(n))));case 148:return function(n){return 154===n.parent.kind&&e.hasModifier(n.parent,8)}(n)&&(n.default||n.constraint)?E(e.updateTypeParameterDeclaration(n,n.name,void 0,void 0)):E(e.visitEachChild(n,X,t));case 173:var g=e.visitNode(n.checkType,X),_=e.visitNode(n.extendsType,X),y=o;o=n.trueType;var h=e.visitNode(n.trueType,X);o=y;var b=e.visitNode(n.falseType,X);return E(e.updateConditionalTypeNode(n,g,_,h,b));case 163:return E(e.updateFunctionTypeNode(n,e.visitNodes(n.typeParameters,X),K(n,n.parameters),e.visitNode(n.type,X)));case 164:return E(e.updateConstructorTypeNode(n,e.visitNodes(n.typeParameters,X),K(n,n.parameters),e.visitNode(n.type,X)));case 181:return e.isLiteralImportTypeNode(n)?E(e.updateImportTypeNode(n,e.updateLiteralTypeNode(n.argument,W(n,n.argument.literal)),n.qualifier,e.visitNodes(n.typeArguments,X,e.isTypeNode),n.isTypeOf)):E(n);default:e.Debug.assertNever(n,\"Attempted to process unhandled node kind: \"+e.SyntaxKind[n.kind])}return E(e.visitEachChild(n,X,t))}}function E(t){return t&&i&&e.hasDynamicName(n)&&function(n){var t;u||(t=v,v=e.createGetSymbolAccessibilityDiagnosticForNodeName(n));m=n.name,e.Debug.assert(x.isLateBound(e.getParseTreeNode(n))),U(n.name.expression,o),u||(v=t);m=void 0}(n),H(n)&&(o=r),i&&!u&&(v=a),l&&(u=s),t===n?t:t&&e.setOriginalNode(j(t,n),n)}}function Y(t){if(function(e){switch(e.kind){case 237:case 242:case 246:case 239:case 238:case 240:case 241:case 217:case 247:case 253:case 252:return!0}return!1}(t)&&!ee(t)){switch(t.kind){case 253:return e.isSourceFile(t.parent)&&(b=!0,T=!0),e.updateExportDeclaration(t,void 0,t.modifiers,t.exportClause,W(t,t.moduleSpecifier));case 252:if(e.isSourceFile(t.parent)&&(b=!0,T=!0),71===t.expression.kind)return t;var r=e.createOptimisticUniqueName(\"_default\");v=function(){return{diagnosticMessage:e.Diagnostics.Default_export_of_the_module_has_or_is_using_private_name_0,errorNode:t}};var a=e.createVariableDeclaration(r,x.createTypeOfExpression(t.expression,t,n,L),void 0);return[e.createVariableStatement(y?[e.createModifier(124)]:[],e.createVariableDeclarationList([a],2)),e.updateExportAssignment(t,t.decorators,t.modifiers,r)]}var i=Q(t);return c.set(\"\"+e.getOriginalNodeId(t),i),t}}function Q(t,r){if(!ee(t)){switch(t.kind){case 246:return function(n){if(x.isDeclarationVisible(n)){if(257===n.moduleReference.kind){var t=e.getExternalModuleImportEqualsDeclarationExpression(n);return e.updateImportEqualsDeclaration(n,void 0,n.modifiers,n.name,e.updateExternalModuleReference(n.moduleReference,W(n,t)))}var r=v;return v=e.createGetSymbolAccessibilityDiagnosticForNode(n),U(n.moduleReference,o),v=r,n}}(t);case 247:return function(n){if(!n.importClause)return e.updateImportDeclaration(n,void 0,n.modifiers,n.importClause,W(n,n.moduleSpecifier));var t=n.importClause&&n.importClause.name&&x.isDeclarationVisible(n.importClause)?n.importClause.name:void 0;if(!n.importClause.namedBindings)return t&&e.updateImportDeclaration(n,void 0,n.modifiers,e.updateImportClause(n.importClause,t,void 0),W(n,n.moduleSpecifier));if(249===n.importClause.namedBindings.kind){var r=x.isDeclarationVisible(n.importClause.namedBindings)?n.importClause.namedBindings:void 0;return t||r?e.updateImportDeclaration(n,void 0,n.modifiers,e.updateImportClause(n.importClause,t,r),W(n,n.moduleSpecifier)):void 0}var a=e.mapDefined(n.importClause.namedBindings.elements,function(e){return x.isDeclarationVisible(e)?e:void 0});return a&&a.length||t?e.updateImportDeclaration(n,void 0,n.modifiers,e.updateImportClause(n.importClause,t,a&&a.length?e.updateNamedImports(n.importClause.namedBindings,a):void 0),W(n,n.moduleSpecifier)):void 0}(t)}if(!(e.isDeclaration(t)&&G(t)||e.isFunctionLike(t)&&x.isImplementationOfOverload(t))){var a;H(t)&&(a=o,o=t);var i=e.canProduceDiagnostics(t),s=v;i&&(v=e.createGetSymbolAccessibilityDiagnosticForNode(t));var l=y;switch(t.kind){case 240:return I(e.updateTypeAliasDeclaration(t,void 0,te(t,r),t.name,e.visitNodes(t.typeParameters,X,e.isTypeParameterDeclaration),e.visitNode(t.type,X,e.isTypeNode)));case 239:return I(e.updateInterfaceDeclaration(t,void 0,te(t,r),t.name,B(t,t.typeParameters),ae(t.heritageClauses),e.visitNodes(t.members,X)));case 237:var u=I(e.updateFunctionDeclaration(t,void 0,te(t,r),void 0,t.name,B(t,t.typeParameters),K(t,t.parameters),F(t,t.type),void 0));if(u&&x.isExpandoFunctionDeclaration(t)){var d=e.mapDefined(x.getPropertiesOfContainerFunction(t),function(t){if(e.isPropertyAccessExpression(t.valueDeclaration)){var r=x.createTypeOfDeclaration(t.valueDeclaration,o,n,L),a=e.createVariableDeclaration(e.unescapeLeadingUnderscores(t.escapedName),r,void 0);return e.createVariableStatement(void 0,e.createVariableDeclarationList([a]))}});return[u,e.createModuleDeclaration(void 0,te(t,r),t.name,e.createModuleBlock(d),16)]}return u;case 242:y=!1;var m=t.body;if(m&&243===m.kind){var p=e.visitNodes(m.statements,Y),f=e.updateModuleBlock(m,q(p));y=l;var g=te(t,r);return I(e.updateModuleDeclaration(t,void 0,g,e.isExternalModuleAugmentation(t)?W(t,t.name):t.name,f))}y=l;g=te(t,r);y=!1,e.visitNode(m,Y);var _=\"\"+e.getOriginalNodeId(m);f=c.get(_);return c.delete(_),I(e.updateModuleDeclaration(t,void 0,g,t.name,f));case 238:var h=e.createNodeArray(te(t,r)),b=B(t,t.typeParameters),E=e.getFirstConstructorWithBody(t),T=void 0;if(E){var S=v;T=e.compact(e.flatMap(E.parameters,function(n){if(e.hasModifier(n,92))return v=e.createGetSymbolAccessibilityDiagnosticForNode(n),71===n.name.kind?j(e.createProperty(void 0,te(n),n.name,n.questionToken,F(n,n.type),P(n)),n):function t(r){var a;for(var i=0,o=r.elements;i<o.length;i++){var s=o[i];e.isOmittedExpression(s)||(e.isBindingPattern(s.name)&&(a=e.concatenate(a,t(s.name))),(a=a||[]).push(e.createProperty(void 0,te(n),s.name,void 0,F(s,void 0),void 0)))}return a}(n.name)})),v=S}var A=e.createNodeArray(e.concatenate(T,e.visitNodes(t.members,X))),C=e.getEffectiveBaseTypeNode(t);if(C&&!e.isEntityNameExpression(C.expression)&&95!==C.expression.kind){var D=e.createOptimisticUniqueName(e.unescapeLeadingUnderscores(t.name.escapedText)+\"_base\");v=function(){return{diagnosticMessage:e.Diagnostics.extends_clause_of_exported_class_0_has_or_is_using_private_name_1,errorNode:C,typeName:t.name}};var k=e.createVariableDeclaration(D,x.createTypeOfExpression(C.expression,t,n,L),void 0),M=e.createVariableStatement(y?[e.createModifier(124)]:[],e.createVariableDeclarationList([k],2)),O=e.createNodeArray(e.map(t.heritageClauses,function(n){if(85===n.token){var t=v;v=e.createGetSymbolAccessibilityDiagnosticForNode(n.types[0]);var r=e.updateHeritageClause(n,e.map(n.types,function(n){return e.updateExpressionWithTypeArguments(n,e.visitNodes(n.typeArguments,X),D)}));return v=t,r}return e.updateHeritageClause(n,e.visitNodes(e.createNodeArray(e.filter(n.types,function(n){return e.isEntityNameExpression(n.expression)||95===n.expression.kind})),X))}));return[M,I(e.updateClassDeclaration(t,void 0,h,t.name,b,O,A))]}O=ae(t.heritageClauses);return I(e.updateClassDeclaration(t,void 0,h,t.name,b,O,A));case 217:return I(function(n,t){if(!e.forEach(n.declarationList.declarations,V))return;var r=e.visitNodes(n.declarationList.declarations,X);if(!e.length(r))return;return e.updateVariableStatement(n,e.createNodeArray(te(n,t)),e.updateVariableDeclarationList(n.declarationList,r))}(t,r));case 241:return I(e.updateEnumDeclaration(t,void 0,e.createNodeArray(te(t,r)),t.name,e.createNodeArray(e.mapDefined(t.members,function(n){if(!ee(n)){var t=x.getConstantValue(n);return j(e.updateEnumMember(n,n.name,void 0!==t?e.createLiteral(t):void 0),n)}}))))}return e.Debug.assertNever(t,\"Unhandled top-level node in declaration emit: \"+e.SyntaxKind[t.kind])}}function I(n){return H(t)&&(o=a),i&&(v=s),242===t.kind&&(y=l),n===t?n:n&&e.setOriginalNode(j(n,t),t)}}function Z(n){return e.flatten(e.mapDefined(n.elements,function(n){return function(n){if(208===n.kind)return;if(n.name){if(!V(n))return;return e.isBindingPattern(n.name)?Z(n.name):e.createVariableDeclaration(n.name,F(n,void 0),void 0)}}(n)}))}function $(n){var t=p.text.substring(n.pos,n.end);return e.stringContains(t,\"@internal\")}function ee(n){if(k&&n){var t=e.getLeadingCommentRangesOfNode(e.getParseTreeNode(n),p);if(e.forEach(t,$))return!0}return!1}function ne(n){return e.isExportAssignment(n)||e.isExportDeclaration(n)}function te(n,t){var r=e.getModifierFlags(n),i=function(n,t){var r=2811,i=y&&!function(e){if(239===e.kind)return!0;return!1}(n)?2:0,o=277===n.parent.kind;(!o||h&&o&&e.isExternalModule(n.parent))&&(r^=2|(t||h&&o||function(n){return!!e.isModuleBlock(n)&&e.some(n.statements,ne)}(n.parent)?0:1),i=0);return a(n,r,i)}(n,t);return r===i?n.modifiers:e.createModifiersFromModifierFlags(i)}function re(n){var t=x.getAllAccessorDeclarations(n);if(n.kind===t.firstAccessor.kind){var a=i(n);!a&&t.secondAccessor&&(a=i(t.secondAccessor),v=e.createGetSymbolAccessibilityDiagnosticForNode(t.secondAccessor));var o=e.createProperty(void 0,r(n,void 0,t.setAccessor?0:64),n.name,n.questionToken,F(n,a),void 0),s=t.secondAccessor&&e.getLeadingCommentRangesOfNode(t.secondAccessor,p);if(s)for(var l=function(n){if(3===n.kind){var t=p.text.slice(n.pos+2,n.end-2),r=t.split(/\\r\\n?|\\n/g);if(r.length>1){var a=r.slice(1),i=e.guessIndentation(a);t=[r[0]].concat(e.map(a,function(e){return e.slice(i)})).join(C)}e.addSyntheticLeadingComment(o,n.kind,t,n.hasTrailingNewLine)}},c=0,u=s;c<u.length;c++){l(u[c])}return o}}function ae(n){return e.createNodeArray(e.filter(e.map(n,function(n){return e.updateHeritageClause(n,e.visitNodes(e.createNodeArray(e.filter(n.types,function(t){return e.isEntityNameExpression(t.expression)||85===n.token&&95===t.expression.kind})),X))}),function(e){return e.types&&!!e.types.length}))}}function r(n,t,r){return e.createModifiersFromModifierFlags(a(n,t,r))}function a(n,t,r){void 0===t&&(t=3067),void 0===r&&(r=0);var a=e.getModifierFlags(n)&t|r;return 512&a&&!(1&a)&&(a^=1),512&a&&2&a&&(a^=2),a}function i(e){if(e)return 156===e.kind?e.type:e.parameters.length>0?e.parameters[0].type:void 0}e.transformDeclarations=t}(Tn||(Tn={})),function(e){var n,t;!function(e){e[e.Uninitialized=0]=\"Uninitialized\",e[e.Initialized=1]=\"Initialized\",e[e.Completed=2]=\"Completed\",e[e.Disposed=3]=\"Disposed\"}(n||(n={})),function(e){e[e.Substitution=1]=\"Substitution\",e[e.EmitNotifications=2]=\"EmitNotifications\"}(t||(t={})),e.getTransformers=function(n,t){var r=n.jsx,a=e.getEmitScriptTarget(n),i=e.getEmitModuleKind(n),o=[];return e.addRange(o,t&&t.before),o.push(e.transformTypeScript),2===r&&o.push(e.transformJsx),a<6&&o.push(e.transformESNext),a<4&&o.push(e.transformES2017),a<3&&o.push(e.transformES2016),a<2&&(o.push(e.transformES2015),o.push(e.transformGenerators)),o.push(function(n){switch(n){case e.ModuleKind.ESNext:case e.ModuleKind.ES2015:return e.transformES2015Module;case e.ModuleKind.System:return e.transformSystemModule;default:return e.transformModule}}(i)),a<1&&o.push(e.transformES5),e.addRange(o,t&&t.after),o},e.transformNodes=function(n,t,r,a,i,o){for(var s,l,c,u=new Array(310),d=[],m=[],p=0,f=!1,g=function(e,n){return n},_=function(e,n,t){return t(e,n)},v=0,y=[],h={getCompilerOptions:function(){return r},getEmitResolver:function(){return n},getEmitHost:function(){return t},startLexicalEnvironment:function(){e.Debug.assert(v>0,\"Cannot modify the lexical environment during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the lexical environment after transformation has completed.\"),e.Debug.assert(!f,\"Lexical environment is suspended.\"),d[p]=s,m[p]=l,p++,s=void 0,l=void 0},suspendLexicalEnvironment:function(){e.Debug.assert(v>0,\"Cannot modify the lexical environment during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the lexical environment after transformation has completed.\"),e.Debug.assert(!f,\"Lexical environment is already suspended.\"),f=!0},resumeLexicalEnvironment:function(){e.Debug.assert(v>0,\"Cannot modify the lexical environment during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the lexical environment after transformation has completed.\"),e.Debug.assert(f,\"Lexical environment is not suspended.\"),f=!1},endLexicalEnvironment:function(){var n;if(e.Debug.assert(v>0,\"Cannot modify the lexical environment during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the lexical environment after transformation has completed.\"),e.Debug.assert(!f,\"Lexical environment is suspended.\"),(s||l)&&(l&&(n=l.slice()),s)){var t=e.createVariableStatement(void 0,e.createVariableDeclarationList(s));n?n.push(t):n=[t]}return s=d[--p],l=m[p],0===p&&(d=[],m=[]),n},hoistVariableDeclaration:function(n){e.Debug.assert(v>0,\"Cannot modify the lexical environment during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the lexical environment after transformation has completed.\");var t=e.setEmitFlags(e.createVariableDeclaration(n),64);s?s.push(t):s=[t]},hoistFunctionDeclaration:function(n){e.Debug.assert(v>0,\"Cannot modify the lexical environment during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the lexical environment after transformation has completed.\"),l?l.push(n):l=[n]},requestEmitHelper:function(n){e.Debug.assert(v>0,\"Cannot modify the transformation context during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the transformation context after transformation has completed.\"),e.Debug.assert(!n.scoped,\"Cannot request a scoped emit helper.\"),c=e.append(c,n)},readEmitHelpers:function(){e.Debug.assert(v>0,\"Cannot modify the transformation context during initialization.\"),e.Debug.assert(v<2,\"Cannot modify the transformation context after transformation has completed.\");var n=c;return c=void 0,n},enableSubstitution:function(n){e.Debug.assert(v<2,\"Cannot modify the transformation context after transformation has completed.\"),u[n]|=1},enableEmitNotification:function(n){e.Debug.assert(v<2,\"Cannot modify the transformation context after transformation has completed.\"),u[n]|=2},isSubstitutionEnabled:x,isEmitNotificationEnabled:A,get onSubstituteNode(){return g},set onSubstituteNode(n){e.Debug.assert(v<1,\"Cannot modify transformation hooks after initialization has completed.\"),e.Debug.assert(void 0!==n,\"Value must not be 'undefined'\"),g=n},get onEmitNode(){return _},set onEmitNode(n){e.Debug.assert(v<1,\"Cannot modify transformation hooks after initialization has completed.\"),e.Debug.assert(void 0!==n,\"Value must not be 'undefined'\"),_=n},addDiagnostic:function(e){y.push(e)}},b=0,E=a;b<E.length;b++){var T=E[b];e.disposeEmitNodes(e.getSourceFileOfNode(e.getParseTreeNode(T)))}e.performance.mark(\"beforeTransform\");var S=e.chain.apply(void 0,i)(h);v=1;var L=e.map(a,o?S:function(n){return!n||e.isSourceFile(n)&&n.isDeclarationFile?n:S(n)});return v=2,e.performance.mark(\"afterTransform\"),e.performance.measure(\"transformTime\",\"beforeTransform\",\"afterTransform\"),{transformed:L,substituteNode:function(n,t){return e.Debug.assert(v<3,\"Cannot substitute a node after the result is disposed.\"),t&&x(t)&&g(n,t)||t},emitNodeWithNotification:function(n,t,r){e.Debug.assert(v<3,\"Cannot invoke TransformationResult callbacks after the result is disposed.\"),t&&(A(t)?_(n,t,r):r(n,t))},dispose:function(){if(v<3){for(var n=0,t=a;n<t.length;n++){var r=t[n];e.disposeEmitNodes(e.getSourceFileOfNode(e.getParseTreeNode(r)))}s=void 0,d=void 0,l=void 0,m=void 0,g=void 0,_=void 0,c=void 0,v=3}},diagnostics:y};function x(n){return 0!=(1&u[n.kind])&&0==(4&e.getEmitFlags(n))}function A(n){return 0!=(2&u[n.kind])||0!=(2&e.getEmitFlags(n))}}}(Tn||(Tn={})),function(e){var n={emittedLine:0,emittedColumn:0,sourceLine:0,sourceColumn:0,sourceIndex:0};e.createSourceMapWriter=function(t,r,i){void 0===i&&(i=t.getCompilerOptions());var o,s,l,c,u,d,m,p,f,g=i.extendedDiagnostics,_=!(i.sourceMap||i.inlineSourceMap);return{initialize:function(r,a,g,v){_||e.fileExtensionIs(r,\".json\")||(p&&y(),f=v,o=void 0,s=void 0,c=-1,u=void 0,d=n,m=0,(p={sourceMapFilePath:a,jsSourceMappingURL:i.inlineSourceMap?void 0:e.getBaseFileName(e.normalizeSlashes(a)),sourceMapFile:e.getBaseFileName(e.normalizeSlashes(r)),sourceMapSourceRoot:i.sourceRoot||\"\",sourceMapSources:[],inputSourceFileNames:[],sourceMapNames:[],sourceMapMappings:\"\",sourceMapSourcesContent:i.inlineSources?[]:void 0}).sourceMapSourceRoot=e.normalizeSlashes(p.sourceMapSourceRoot),p.sourceMapSourceRoot.length&&47!==p.sourceMapSourceRoot.charCodeAt(p.sourceMapSourceRoot.length-1)&&(p.sourceMapSourceRoot+=e.directorySeparator),i.mapRoot?(l=e.normalizeSlashes(i.mapRoot),277===g.kind&&(l=e.getDirectoryPath(e.getSourceFilePathInNewDir(g.fileName,t,l))),e.isRootedDiskPath(l)||e.isUrl(l)?p.jsSourceMappingURL=e.combinePaths(l,p.jsSourceMappingURL):(l=e.combinePaths(t.getCommonSourceDirectory(),l),p.jsSourceMappingURL=e.getRelativePathToDirectoryOrUrl(e.getDirectoryPath(e.normalizePath(r)),e.combinePaths(l,p.jsSourceMappingURL),t.getCurrentDirectory(),t.getCanonicalFileName,!0))):l=e.getDirectoryPath(e.normalizePath(r)))},reset:y,setSourceFile:T,emitPos:b,emitNodeWithSourceMap:function(n,a,s){var c,d;if(_||e.isInJsonFile(a))return s(n,a);if(a){if(e.isUnparsedSource(a)&&void 0!==a.sourceMapText){var m=a.sourceMapText,f=void 0;try{f=JSON.parse(m)}catch(e){}if(!f||\"object\"!=typeof(d=f)||!d.mappings||\"string\"!=typeof d.mappings||!d.sources)return s(n,a);for(var g=r.getLine(),y=r.getColumn(),E=f,L=i.sourceRoot?t.getCommonSourceDirectory():l,x=e.createMap(),A=e.createMap(),C=e.sourcemaps.decodeMappings(E),D=C.next(),k=D.value,M=D.done;!M;c=C.next(),k=c.value,M=c.done,c){var O=\"\"+k.sourceIndex;if(!x.has(O)){var I=E.sources[k.sourceIndex],R=E.sourceRoot?e.combinePaths(E.sourceRoot,I):I,N=e.combinePaths(e.getDirectoryPath(a.sourceMapPath),R),w=e.getRelativePathToDirectoryOrUrl(L,N,t.getCurrentDirectory(),t.getCanonicalFileName,!0);x.set(O,w),A.set(O,e.getNormalizedAbsolutePath(w,L))}var P=x.get(O),F=A.get(O);S(F,E.sourcesContent?E.sourcesContent[k.sourceIndex]:null,P);var G=p.sourceMapSources.indexOf(P);h(),u=yn({},k,{emittedLine:k.emittedLine+g,emittedColumn:0===k.emittedLine?k.emittedColumn+y:k.emittedColumn,sourceIndex:G})}return s(n,a)}var V=a.emitNode,K=V&&V.flags||0,B=V&&V.sourceMapRange,H=B||a,U=H.pos,j=H.end,W=B&&B.source,q=o;W===q&&(W=void 0),W&&T(W),305!==a.kind&&0==(16&K)&&U>=0&&b(v(U)),W&&T(q),64&K?(_=!0,s(n,a),_=!1):s(n,a),W&&T(W),305!==a.kind&&0==(32&K)&&j>=0&&b(j),W&&T(q)}},emitTokenWithSourceMap:function(n,t,r,a,i){if(_||e.isInJsonFile(n))return i(t,r,a);var o=n&&n.emitNode,s=o&&o.flags||0,l=o&&o.tokenSourceMapRanges&&o.tokenSourceMapRanges[t];return a=v(l?l.pos:a),0==(128&s)&&a>=0&&b(a),a=i(t,r,a),l&&(a=l.end),0==(256&s)&&a>=0&&b(a),a},getText:L,getSourceMappingURL:function(){if(!_&&!E(o)){if(i.inlineSourceMap){var n=e.base64encode(e.sys,L());return p.jsSourceMappingURL=\"data:application/json;base64,\"+n}return p.jsSourceMappingURL}}};function v(n){return o.skipTrivia?o.skipTrivia(n):e.skipTrivia(s,n)}function y(){_||(f&&f.push(p),o=void 0,l=void 0,c=void 0,u=void 0,d=void 0,m=void 0,p=void 0,f=void 0)}function h(){if(u&&u!==d){e.Debug.assert(u.emittedColumn>=0,\"lastEncodedSourceMapSpan.emittedColumn was negative\"),e.Debug.assert(u.sourceIndex>=0,\"lastEncodedSourceMapSpan.sourceIndex was negative\"),e.Debug.assert(u.sourceLine>=0,\"lastEncodedSourceMapSpan.sourceLine was negative\"),e.Debug.assert(u.sourceColumn>=0,\"lastEncodedSourceMapSpan.sourceColumn was negative\");var n=d.emittedColumn;if(d.emittedLine===u.emittedLine)p.sourceMapMappings&&(p.sourceMapMappings+=\",\");else{for(var t=d.emittedLine;t<u.emittedLine;t++)p.sourceMapMappings+=\";\";n=0}p.sourceMapMappings+=a(u.emittedColumn-n),p.sourceMapMappings+=a(u.sourceIndex-d.sourceIndex),p.sourceMapMappings+=a(u.sourceLine-d.sourceLine),p.sourceMapMappings+=a(u.sourceColumn-d.sourceColumn),u.nameIndex>=0&&(e.Debug.assert(!1,\"We do not support name index right now, Make sure to update updateLastEncodedAndRecordedSpans when we start using this\"),p.sourceMapMappings+=a(u.nameIndex-m),m=u.nameIndex),d=u}}function b(n){if(!(_||e.positionIsSynthesized(n)||E(o))){g&&e.performance.mark(\"beforeSourcemap\");var t=e.getLineAndCharacterOfPosition(o,n),a=r.getLine(),i=r.getColumn();!u||u.emittedLine!==a||u.emittedColumn!==i||u.sourceIndex===c&&(u.sourceLine>t.line||u.sourceLine===t.line&&u.sourceColumn>t.character)?(h(),u={emittedLine:a,emittedColumn:i,sourceLine:t.line,sourceColumn:t.character,sourceIndex:c}):(u.sourceLine=t.line,u.sourceColumn=t.character,u.sourceIndex=c),g&&(e.performance.mark(\"afterSourcemap\"),e.performance.measure(\"Source Map\",\"beforeSourcemap\",\"afterSourcemap\"))}}function E(n){return e.fileExtensionIs(n.fileName,\".json\")}function T(e){_||(s=(o=e).text,E(e)||S(e.fileName,e.text))}function S(n,r,a){if(!a){var o=i.sourceRoot?t.getCommonSourceDirectory():l;a=e.getRelativePathToDirectoryOrUrl(o,n,t.getCurrentDirectory(),t.getCanonicalFileName,!0)}-1===(c=p.sourceMapSources.indexOf(a))&&(c=p.sourceMapSources.length,p.sourceMapSources.push(a),p.inputSourceFileNames.push(n),i.inlineSources&&p.sourceMapSourcesContent.push(r))}function L(){if(!_&&!E(o))return h(),JSON.stringify({version:3,file:p.sourceMapFile,sourceRoot:p.sourceMapSourceRoot,sources:p.sourceMapSources,names:p.sourceMapNames,mappings:p.sourceMapMappings,sourcesContent:p.sourceMapSourcesContent})}};var t=\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/\";function r(e){if(e<64)return t.charAt(e);throw TypeError(e+\": not a 64 based value\")}function a(e){e<0?e=1+(-e<<1):e<<=1;var n=\"\";do{var t=31&e;(e>>=5)>0&&(t|=32),n+=r(t)}while(e>0);return n}}(Tn||(Tn={})),function(e){e.createCommentWriter=function(n,t){var r,a,i,o,s,l=n.extendedDiagnostics,c=e.getNewLineCharacter(n),u=-1,d=-1,m=-1,p=!1,f=!!n.removeComments;return{reset:function(){a=void 0,i=void 0,o=void 0,s=void 0},setWriter:function(e){r=e},setSourceFile:function(n){i=(a=n).text,o=e.getLineStarts(a),s=void 0},emitNodeWithComments:function(n,t,r){if(f)r(n,t);else if(t){p=!1;var a=t.emitNode,i=a&&a.flags||0,o=a&&a.commentRange||t,s=o.pos,c=o.end;if(s<0&&c<0||s===c)g(n,t,a,i,r);else{l&&e.performance.mark(\"preEmitNodeWithComment\");var _=305!==t.kind,v=s<0||0!=(512&i)||10===t.kind,y=c<0||0!=(1024&i)||10===t.kind;v||h(s,_);var b=u,E=d,T=m;(!v||s>=0&&0!=(512&i))&&(u=s),(!y||c>=0&&0!=(1024&i))&&(d=c,236===t.kind&&(m=c)),l&&e.performance.measure(\"commentTime\",\"preEmitNodeWithComment\"),g(n,t,a,i,r),l&&e.performance.mark(\"postEmitNodeWithComment\"),u=b,d=E,m=T,!y&&_&&function(e){A(e,S)}(c),l&&e.performance.measure(\"commentTime\",\"postEmitNodeWithComment\")}}},emitBodyWithDetachedComments:function(n,t,a){l&&e.performance.mark(\"preEmitBodyWithDetachedComments\");var u,d,m=t.pos,g=t.end,_=e.getEmitFlags(n),v=f||g<0||0!=(1024&_);m<0||0!=(512&_)||(u=t,(d=e.emitDetachedComments(i,o,r,C,u,c,f))&&(s?s.push(d):s=[d])),l&&e.performance.measure(\"commentTime\",\"preEmitBodyWithDetachedComments\"),2048&_&&!f?(f=!0,a(n),f=!1):a(n),l&&e.performance.mark(\"beginEmitBodyWithDetachedCommetns\"),v||(h(t.end,!0),p&&!r.isAtStartOfLine()&&r.writeLine()),l&&e.performance.measure(\"commentTime\",\"beginEmitBodyWithDetachedCommetns\")},emitTrailingCommentsOfPosition:function(n,t){f||(l&&e.performance.mark(\"beforeEmitTrailingCommentsOfPosition\"),A(n,t?S:L),l&&e.performance.measure(\"commentTime\",\"beforeEmitTrailingCommentsOfPosition\"))},emitLeadingCommentsOfPosition:function(e){f||-1===e||h(e,!0)}};function g(n,t,r,a,i){var o=r&&r.leadingComments;e.some(o)&&(l&&e.performance.mark(\"preEmitNodeWithSynthesizedComments\"),e.forEach(o,_),l&&e.performance.measure(\"commentTime\",\"preEmitNodeWithSynthesizedComments\")),function(e,n,t,r){2048&t?(f=!0,r(e,n),f=!1):r(e,n)}(n,t,a,i);var s=r&&r.trailingComments;e.some(s)&&(l&&e.performance.mark(\"postEmitNodeWithSynthesizedComments\"),e.forEach(s,v),l&&e.performance.measure(\"commentTime\",\"postEmitNodeWithSynthesizedComments\"))}function _(e){2===e.kind&&r.writeLine(),y(e),e.hasTrailingNewLine||2===e.kind?r.writeLine():r.write(\" \")}function v(e){r.isAtStartOfLine()||r.write(\" \"),y(e),e.hasTrailingNewLine&&r.writeLine()}function y(n){var t=function(e){return 3===e.kind?\"/*\"+e.text+\"*/\":\"//\"+e.text}(n),a=3===n.kind?e.computeLineStarts(t):void 0;e.writeCommentRange(t,a,r,0,t.length,c)}function h(e,n){p=!1,n?x(e,T):0===e&&x(e,b)}function b(n,t,r,a,o){(function(n,t){return e.isRecognizedTripleSlashComment(i,n,t)})(n,t)&&T(n,t,r,a,o)}function E(t,r){return!n.onlyPrintJsDocStyle||e.isJSDocLikeText(t,r)||e.isPinnedComment(t,r)}function T(n,a,s,l,u){E(i,n)&&(p||(e.emitNewLineBeforeLeadingCommentOfPosition(o,r,u,n),p=!0),t&&t(n),e.writeCommentRange(i,o,r,n,a,c),t&&t(a),l?r.writeLine():3===s&&r.write(\" \"))}function S(n,a,s,l){E(i,n)&&(r.isAtStartOfLine()||r.write(\" \"),t&&t(n),e.writeCommentRange(i,o,r,n,a,c),t&&t(a),l&&r.writeLine())}function L(n,a,s,l){t&&t(n),e.writeCommentRange(i,o,r,n,a,c),t&&t(a),l?r.writeLine():r.write(\" \")}function x(n,t){-1!==u&&n===u||(function(n){return void 0!==s&&e.last(s).nodePos===n}(n)?function(n){var t=e.last(s).detachedCommentEndPos;s.length-1?s.pop():s=void 0,e.forEachLeadingCommentRange(i,t,n,t)}(t):e.forEachLeadingCommentRange(i,n,t,n))}function A(n,t){(-1===d||n!==d&&n!==m)&&e.forEachTrailingCommentRange(i,n,t)}function C(n,r,a,o,s,l){E(i,o)&&(t&&t(o),e.writeCommentRange(n,r,a,o,s,l),t&&t(s))}}}(Tn||(Tn={})),function(e){var n,t,r=\".tsbundleinfo\",a=function(){var e=[];return e[512]=[\"{\",\"}\"],e[1024]=[\"(\",\")\"],e[2048]=[\"<\",\">\"],e[4096]=[\"[\",\"]\"],e}();function i(n,t,r,a){void 0===a&&(a=!1);var i=e.isArray(r)?r:e.getSourceFilesToEmit(n,r),s=n.getCompilerOptions();if(s.outFile||s.out){if(i.length){var l=e.createBundle(i,n.getPrependNodes());if(d=t(o(l,n,a),l))return d}}else for(var c=0,u=i;c<u.length;c++){var d,m=u[c];if(d=t(o(m,n,a),m))return d}}function o(n,t,a){var i=t.getCompilerOptions();if(278===n.kind)return{jsFilePath:c=i.outFile||i.out,sourceMapFilePath:u=s(c,i),declarationFilePath:o=a||e.getEmitDeclarations(i)?e.removeFileExtension(c)+\".d.ts\":void 0,declarationMapPath:e.getAreDeclarationMapsEnabled(i)?o+\".map\":void 0,bundleInfoPath:i.references&&c?e.removeFileExtension(c)+r:void 0};var o,c=e.getOwnEmitOutputFilePath(n.fileName,t,l(n,i)),u=e.isJsonSourceFile(n)?void 0:s(c,i),d=e.isSourceFileJS(n);return{jsFilePath:c,sourceMapFilePath:u,declarationFilePath:o=!a&&!e.getEmitDeclarations(i)||d?void 0:e.getDeclarationEmitOutputFilePath(n.fileName,t),declarationMapPath:e.getAreDeclarationMapsEnabled(i)?o+\".map\":void 0,bundleInfoPath:void 0}}function s(e,n){return n.sourceMap&&!n.inlineSourceMap?e+\".map\":void 0}function l(n,t){if(e.isJsonSourceFile(n))return\".json\";if(1===t.jsx)if(e.isSourceFileJS(n)){if(e.fileExtensionIs(n.fileName,\".jsx\"))return\".jsx\"}else if(1===n.languageVariant)return\".jsx\";return\".js\"}function c(n,t){void 0===n&&(n={}),void 0===t&&(t={});var r,i,o,s,l,c,u,d,m,p,f=t.hasGlobalName,g=t.onEmitSourceMapOfNode,_=t.onEmitSourceMapOfToken,v=t.onEmitSourceMapOfPosition,y=t.onEmitNode,h=t.onSetSourceFile,b=t.substituteNode,E=t.onBeforeEmitNodeArray,T=t.onAfterEmitNodeArray,S=t.onBeforeEmitToken,L=t.onAfterEmitToken,x=e.getNewLineCharacter(n),A=e.createCommentWriter(n,v),C=A.emitNodeWithComments,D=A.emitBodyWithDetachedComments,k=A.emitTrailingCommentsOfPosition,M=A.emitLeadingCommentsOfPosition,O=function(e){I(),m.write(e)},I=e.noop,R=Qe,N=!1;n.omitTrailingSemicolon&&(I=function(){N&&(Qe(),N=!1)},R=function(){N=!0});var w,P={pos:-1,end:-1},F=e.getEmitModuleKind(n),G=e.createMap();return Y(),{printNode:function(n,t,r){switch(n){case 0:e.Debug.assert(e.isSourceFile(t),\"Expected a SourceFile node.\");break;case 2:e.Debug.assert(e.isIdentifier(t),\"Expected an Identifier node.\");break;case 1:e.Debug.assert(e.isExpression(t),\"Expected an Expression node.\")}switch(t.kind){case 277:return K(t);case 278:return V(t);case 279:return function(e,n){var t=m;X(n),z(4,e,void 0),Y(),m=t}(t,W()),q()}return B(n,t,r,W()),q()},printList:function(e,n,t){return H(e,n,t,W()),q()},printFile:K,printBundle:V,writeNode:B,writeList:H,writeFile:j,writeBundle:U};function V(e){return U(e,W()),q()}function K(e){return j(e,W()),q()}function B(e,n,t,r){var a=m;X(r),z(e,n,t),Y(),m=a}function H(e,n,t,r){var a=m;X(r),t&&J(t),We(P,n,e),Y(),m=a}function U(e,n,t){w=!1;var r,a=m;X(n),Ie(e),Oe(e),le(e),De(!!(r=e).hasNoDefaultLib,r.syntheticFileReferences||[],r.syntheticTypeReferences||[],r.syntheticLibReferences||[]);for(var i=0,o=e.prepends;i<o.length;i++){z(4,o[i],void 0),rn()}t&&(t.originalOffset=m.getTextPos());for(var s=0,l=e.sourceFiles;s<l.length;s++){var c=l[s];z(0,c,c)}Y(),m=a}function j(e,n){w=!0;var t=m;X(n),Ie(e),Oe(e),z(0,e,e),Y(),m=t}function W(){return p||(p=e.createTextWriter(x))}function q(){var e=p.getText();return p.clear(),e}function z(e,n,t){t&&J(t),ee(0,e)(e,n)}function J(e){r=e,A.setSourceFile(e),h&&h(e)}function X(e){m=e,A.setWriter(e)}function Y(){i=[],o=[],s=e.createMap(),l=[],c=0,u=[],A.reset(),X(void 0)}function Q(e){e&&ee(0,4)(4,e)}function Z(e){e&&ee(0,2)(2,e)}function $(e){e&&ee(0,1)(1,e)}function ee(n,t){switch(n){case 0:if(y)return te;case 1:return C&&0!==t?re:ae;case 2:if(g&&0!==t)return ie;case 3:return oe;default:return e.Debug.assertNever(n,\"Unexpected value for PipelinePhase: \"+n)}}function ne(e,n){return ee(e+1,n)}function te(n,t){e.Debug.assertDefined(y)(n,t,ne(0,n))}function re(n,t){e.Debug.assertDefined(C),e.Debug.assert(0!==n),C(n,se(n,t),ne(1,n))}function ae(e,n){ne(1,e)(e,se(e,n))}function ie(n,t){e.Debug.assert(0!==n),e.Debug.assertDefined(g)(n,t,oe)}function oe(t,a){if(0===t)return function(n){rn();var t=n.statements;if(D){var r=0===t.length||!e.isPrologueDirective(t[0])||e.nodeIsSynthesized(t[0]);if(r)return void D(n,t,ke)}ke(n)}(e.cast(a,e.isSourceFile));if(2===t)return ue(e.cast(a,e.isIdentifier));if(3===t)return function(e){Q(e.name),nn(),Ze(\"in\"),nn(),Q(e.constraint)}(e.cast(a,e.isTypeParameterDeclaration));if(4===t){if(e.isKeyword(a.kind))return ln(a,Ze);switch(a.kind){case 14:case 15:case 16:return ce(a);case 279:return o=a,void m.rawWrite(o.text);case 71:return ue(a);case 146:return function(e){(function(e){71===e.kind?$(e):Q(e)})(e.left),Ye(\".\"),Q(e.right)}(a);case 147:return function(e){Ye(\"[\"),$(e.expression),Ye(\"]\")}(a);case 148:return function(e){Q(e.name),e.constraint&&(nn(),Ze(\"extends\"),nn(),Q(e.constraint));e.default&&(nn(),$e(\"=\"),nn(),Q(e.default))}(a);case 149:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),Q(e.dotDotDotToken),Re(e.name,en),Q(e.questionToken),e.parent&&287===e.parent.kind&&!e.name?Q(e.type):we(e.type);Pe(e.initializer,e.type?e.type.end:e.questionToken?e.questionToken.end:e.name?e.name.end:e.modifiers?e.modifiers.end:e.decorators?e.decorators.end:e.pos,e)}(a);case 150:return i=a,Ye(\"@\"),void $(i.expression);case 151:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),Re(e.name,tn),Q(e.questionToken),we(e.type),R()}(a);case 152:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),Q(e.name),Q(e.questionToken),Q(e.exclamationToken),we(e.type),Pe(e.initializer,e.type?e.type.end:e.questionToken?e.questionToken.end:e.name.end,e),R()}(a);case 153:return function(e){Sn(e),Ke(e,e.decorators),Ne(e,e.modifiers),Q(e.name),Q(e.questionToken),He(e,e.typeParameters),Ue(e,e.parameters),we(e.type),R(),Ln(e)}(a);case 154:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),Q(e.asteriskToken),Q(e.name),Q(e.questionToken),ye(e,he)}(a);case 155:return function(e){Ne(e,e.modifiers),Ze(\"constructor\"),ye(e,he)}(a);case 156:case 157:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),Ze(156===e.kind?\"get\":\"set\"),nn(),Q(e.name),ye(e,he)}(a);case 158:return function(e){Sn(e),Ke(e,e.decorators),Ne(e,e.modifiers),He(e,e.typeParameters),Ue(e,e.parameters),we(e.type),R(),Ln(e)}(a);case 159:return function(e){Sn(e),Ke(e,e.decorators),Ne(e,e.modifiers),Ze(\"new\"),nn(),He(e,e.typeParameters),Ue(e,e.parameters),we(e.type),R(),Ln(e)}(a);case 160:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),n=e,t=e.parameters,We(n,t,4432),we(e.type),R();var n,t}(a);case 161:return function(e){Q(e.parameterName),nn(),Ze(\"is\"),nn(),Q(e.type)}(a);case 162:return function(e){Q(e.typeName),Be(e,e.typeArguments)}(a);case 163:return function(e){Sn(e),He(e,e.typeParameters),je(e,e.parameters),nn(),Ye(\"=>\"),nn(),Q(e.type),Ln(e)}(a);case 287:return function(e){O(\"function\"),Ue(e,e.parameters),O(\":\"),Q(e.type)}(a);case 164:return function(e){Sn(e),Ze(\"new\"),nn(),He(e,e.typeParameters),Ue(e,e.parameters),nn(),Ye(\"=>\"),nn(),Q(e.type),Ln(e)}(a);case 165:return function(e){Ze(\"typeof\"),nn(),Q(e.exprName)}(a);case 166:return function(n){Ye(\"{\");var t=1&e.getEmitFlags(n)?384:16449;We(n,n.members,262144|t),Ye(\"}\")}(a);case 167:return function(e){Q(e.elementType),Ye(\"[\"),Ye(\"]\")}(a);case 168:return function(e){Ye(\"[\"),We(e,e.elementTypes,272),Ye(\"]\")}(a);case 169:return function(e){Q(e.type),O(\"?\")}(a);case 171:return function(e){We(e,e.types,260)}(a);case 172:return function(e){We(e,e.types,264)}(a);case 173:return function(e){Q(e.checkType),nn(),Ze(\"extends\"),nn(),Q(e.extendsType),nn(),Ye(\"?\"),nn(),Q(e.trueType),nn(),Ye(\":\"),nn(),Q(e.falseType)}(a);case 174:return function(e){Ze(\"infer\"),nn(),Q(e.typeParameter)}(a);case 175:return function(e){Ye(\"(\"),Q(e.type),Ye(\")\")}(a);case 209:return function(e){$(e.expression),Be(e,e.typeArguments)}(a);case 176:return void Ze(\"this\");case 177:return function(e){cn(e.operator,Ze),nn(),Q(e.type)}(a);case 178:return function(e){Q(e.objectType),Ye(\"[\"),Q(e.indexType),Ye(\"]\")}(a);case 179:return function(n){var t=e.getEmitFlags(n);Ye(\"{\"),1&t?nn():(rn(),an());n.readonlyToken&&(Q(n.readonlyToken),132!==n.readonlyToken.kind&&Ze(\"readonly\"),nn());Ye(\"[\"),ee(0,3)(3,n.typeParameter),Ye(\"]\"),n.questionToken&&(Q(n.questionToken),55!==n.questionToken.kind&&Ye(\"?\"));Ye(\":\"),nn(),Q(n.type),R(),1&t?nn():(rn(),on());Ye(\"}\")}(a);case 180:return function(e){$(e.literal)}(a);case 181:return function(e){e.isTypeOf&&(Ze(\"typeof\"),nn());Ze(\"import\"),Ye(\"(\"),Q(e.argument),Ye(\")\"),e.qualifier&&(Ye(\".\"),Q(e.qualifier));Be(e,e.typeArguments)}(a);case 282:return void O(\"*\");case 283:return void O(\"?\");case 284:return function(e){O(\"?\"),Q(e.type)}(a);case 285:return function(e){O(\"!\"),Q(e.type)}(a);case 286:return function(e){Q(e.type),O(\"=\")}(a);case 170:case 288:return function(e){O(\"...\"),Q(e.type)}(a);case 182:return function(e){Ye(\"{\"),We(e,e.elements,262576),Ye(\"}\")}(a);case 183:return function(e){Ye(\"[\"),We(e,e.elements,262448),Ye(\"]\")}(a);case 184:return function(e){Q(e.dotDotDotToken),e.propertyName&&(Q(e.propertyName),Ye(\":\"),nn());Q(e.name),Pe(e.initializer,e.name.end,e)}(a);case 214:return function(e){$(e.expression),Q(e.literal)}(a);case 215:return void R();case 216:return function(e){me(e,!e.multiLine&&hn(e))}(a);case 217:return function(e){Ne(e,e.modifiers),Q(e.declarationList),R()}(a);case 218:return void R();case 219:return function(n){$(n.expression),(!e.isJsonSourceFile(r)||e.nodeIsSynthesized(n.expression))&&R()}(a);case 220:return function(e){var n=ge(90,e.pos,Ze,e);nn(),ge(19,n,Ye,e),$(e.expression),ge(20,e.expression.end,Ye,e),Ve(e,e.thenStatement),e.elseStatement&&(un(e),ge(82,e.thenStatement.end,Ze,e),220===e.elseStatement.kind?(nn(),Q(e.elseStatement)):Ve(e,e.elseStatement))}(a);case 221:return function(n){ge(81,n.pos,Ze,n),Ve(n,n.statement),e.isBlock(n.statement)?nn():un(n);pe(n,n.statement.end),Ye(\";\")}(a);case 222:return function(e){pe(e,e.pos),Ve(e,e.statement)}(a);case 223:return function(e){var n=ge(88,e.pos,Ze,e);nn();var t=ge(19,n,Ye,e);fe(e.initializer),t=ge(25,e.initializer?e.initializer.end:t,R,e),Ge(e.condition),t=ge(25,e.condition?e.condition.end:t,R,e),Ge(e.incrementor),ge(20,e.incrementor?e.incrementor.end:t,Ye,e),Ve(e,e.statement)}(a);case 224:return function(e){var n=ge(88,e.pos,Ze,e);nn(),ge(19,n,Ye,e),fe(e.initializer),nn(),ge(92,e.initializer.end,Ze,e),nn(),$(e.expression),ge(20,e.expression.end,Ye,e),Ve(e,e.statement)}(a);case 225:return function(e){var n=ge(88,e.pos,Ze,e);nn(),function(e){e&&(Q(e),nn())}(e.awaitModifier),ge(19,n,Ye,e),fe(e.initializer),nn(),ge(145,e.initializer.end,Ze,e),nn(),$(e.expression),ge(20,e.expression.end,Ye,e),Ve(e,e.statement)}(a);case 226:return function(e){ge(77,e.pos,Ze,e),Fe(e.label),R()}(a);case 227:return function(e){ge(72,e.pos,Ze,e),Fe(e.label),R()}(a);case 228:return function(e){ge(96,e.pos,Ze,e),Ge(e.expression),R()}(a);case 229:return function(e){var n=ge(107,e.pos,Ze,e);nn(),ge(19,n,Ye,e),$(e.expression),ge(20,e.expression.end,Ye,e),Ve(e,e.statement)}(a);case 230:return function(e){var n=ge(98,e.pos,Ze,e);nn(),ge(19,n,Ye,e),$(e.expression),ge(20,e.expression.end,Ye,e),nn(),Q(e.caseBlock)}(a);case 231:return function(e){Q(e.label),ge(56,e.label.end,Ye,e),nn(),Q(e.statement)}(a);case 232:return function(e){ge(100,e.pos,Ze,e),Ge(e.expression),R()}(a);case 233:return function(e){ge(102,e.pos,Ze,e),nn(),Q(e.tryBlock),e.catchClause&&(un(e),Q(e.catchClause));e.finallyBlock&&(un(e),ge(87,(e.catchClause||e.tryBlock).end,Ze,e),nn(),Q(e.finallyBlock))}(a);case 234:return function(e){sn(78,e.pos,Ze),R()}(a);case 235:return function(e){Q(e.name),we(e.type),Pe(e.initializer,e.type?e.type.end:e.name.end,e)}(a);case 236:return function(n){Ze(e.isLet(n)?\"let\":e.isVarConst(n)?\"const\":\"var\"),nn(),We(n,n.declarations,272)}(a);case 237:return function(e){_e(e)}(a);case 238:return function(e){Se(e)}(a);case 239:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),Ze(\"interface\"),nn(),Q(e.name),He(e,e.typeParameters),We(e,e.heritageClauses,256),nn(),Ye(\"{\"),We(e,e.members,65),Ye(\"}\")}(a);case 240:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),Ze(\"type\"),nn(),Q(e.name),He(e,e.typeParameters),nn(),Ye(\"=\"),nn(),Q(e.type),R()}(a);case 241:return function(e){Ne(e,e.modifiers),Ze(\"enum\"),nn(),Q(e.name),nn(),Ye(\"{\"),We(e,e.members,81),Ye(\"}\")}(a);case 242:return function(e){Ne(e,e.modifiers),512&~e.flags&&(Ze(16&e.flags?\"namespace\":\"module\"),nn());Q(e.name);var n=e.body;if(!n)return R();for(;242===n.kind;)Ye(\".\"),Q(n.name),n=n.body;nn(),Q(n)}(a);case 243:return function(n){Sn(n),e.forEach(n.statements,An),me(n,hn(n)),Ln(n)}(a);case 244:return function(e){ge(17,e.pos,Ye,e),We(e,e.clauses,65),ge(18,e.clauses.end,Ye,e,!0)}(a);case 245:return function(e){var n=ge(84,e.pos,Ze,e);nn(),n=ge(118,n,Ze,e),nn(),n=ge(130,n,Ze,e),nn(),Q(e.name),R()}(a);case 246:return function(e){Ne(e,e.modifiers),ge(91,e.modifiers?e.modifiers.end:e.pos,Ze,e),nn(),Q(e.name),nn(),ge(58,e.name.end,Ye,e),nn(),function(e){71===e.kind?$(e):Q(e)}(e.moduleReference),R()}(a);case 247:return function(e){Ne(e,e.modifiers),ge(91,e.modifiers?e.modifiers.end:e.pos,Ze,e),nn(),e.importClause&&(Q(e.importClause),nn(),ge(143,e.importClause.end,Ze,e),nn());$(e.moduleSpecifier),R()}(a);case 248:return function(e){Q(e.name),e.name&&e.namedBindings&&(ge(26,e.name.end,Ye,e),nn());Q(e.namedBindings)}(a);case 249:return function(e){var n=ge(39,e.pos,Ye,e);nn(),ge(118,n,Ze,e),nn(),Q(e.name)}(a);case 250:return function(e){Le(e)}(a);case 251:return function(e){xe(e)}(a);case 252:return function(e){var n=ge(84,e.pos,Ze,e);nn(),e.isExportEquals?ge(58,n,$e,e):ge(79,n,Ze,e);nn(),$(e.expression),R()}(a);case 253:return function(e){var n=ge(84,e.pos,Ze,e);nn(),e.exportClause?Q(e.exportClause):n=ge(39,n,Ye,e);if(e.moduleSpecifier){nn();var t=e.exportClause?e.exportClause.end:n;ge(143,t,Ze,e),nn(),$(e.moduleSpecifier)}R()}(a);case 254:return function(e){Le(e)}(a);case 255:return function(e){xe(e)}(a);case 256:return;case 257:return function(e){Ze(\"require\"),Ye(\"(\"),$(e.expression),Ye(\")\")}(a);case 10:return function(e){I(),m.writeLiteral(En(e,!0))}(a);case 260:case 263:return function(n){Ye(\"<\"),e.isJsxOpeningElement(n)&&(Ae(n.tagName),n.attributes.properties&&n.attributes.properties.length>0&&nn(),Q(n.attributes));Ye(\">\")}(a);case 261:case 264:return function(n){Ye(\"</\"),e.isJsxClosingElement(n)&&Ae(n.tagName);Ye(\">\")}(a);case 265:return function(e){Q(e.name),function(e,n,t,r){t&&(n(e),r(t))}(\"=\",Ye,e.initializer,Q)}(a);case 266:return function(e){We(e,e.properties,131328)}(a);case 267:return function(e){Ye(\"{...\"),$(e.expression),Ye(\"}\")}(a);case 268:return function(e){e.expression&&(Ye(\"{\"),Q(e.dotDotDotToken),$(e.expression),Ye(\"}\"))}(a);case 269:return function(e){ge(73,e.pos,Ze,e),nn(),$(e.expression),Ce(e,e.statements,e.expression.end)}(a);case 270:return function(e){var n=ge(79,e.pos,Ze,e);Ce(e,e.statements,n)}(a);case 271:return function(e){nn(),cn(e.token,Ze),nn(),We(e,e.types,272)}(a);case 272:return function(e){var n=ge(74,e.pos,Ze,e);nn(),e.variableDeclaration&&(ge(19,n,Ye,e),Q(e.variableDeclaration),ge(20,e.variableDeclaration.end,Ye,e),nn());Q(e.block)}(a);case 273:return function(n){Q(n.name),Ye(\":\"),nn();var t=n.initializer;if(k&&0==(512&e.getEmitFlags(t))){var r=e.getCommentRange(t);k(r.pos)}$(t)}(a);case 274:return function(e){Q(e.name),e.objectAssignmentInitializer&&(nn(),Ye(\"=\"),nn(),$(e.objectAssignmentInitializer))}(a);case 275:return function(e){e.expression&&(Ye(\"...\"),$(e.expression))}(a);case 276:return function(e){Q(e.name),Pe(e.initializer,e.name.end,e)}(a)}if(e.isExpression(a))t=1,a=se(1,a);else if(e.isToken(a))return ln(a,Ye)}var i,o;if(1===t)switch(a.kind){case 8:return function(e){ce(e)}(a);case 9:case 12:case 13:return ce(a);case 71:return ue(a);case 86:case 95:case 97:case 101:case 99:case 91:return void ln(a,Ze);case 185:return function(e){var n=e.elements,t=e.multiLine?32768:0;qe(e,n,4466|t)}(a);case 186:return function(n){e.forEach(n.properties,Cn);var t=65536&e.getEmitFlags(n);t&&an();var a=n.multiLine?32768:0,i=r.languageVersion>=1&&!e.isJsonSourceFile(r)?32:0;We(n,n.properties,263122|i|a),t&&on()}(a);case 187:return function(t){var a=!1,i=!1;if(!(131072&e.getEmitFlags(t))){var o=t.expression.end,s=e.skipTrivia(r.text,t.expression.end)+1,l=e.createToken(23);l.pos=o,l.end=s,a=yn(t,t.expression,l),i=yn(t,l,t.name)}$(t.expression),mn(a),!a&&function(t){if(t=e.skipPartiallyEmittedExpressions(t),e.isNumericLiteral(t)){var r=Tn(t,!0);return!t.numericLiteralFlags&&!e.stringContains(r,e.tokenToString(23))}if(e.isPropertyAccessExpression(t)||e.isElementAccessExpression(t)){var a=e.getConstantValue(t);return\"number\"==typeof a&&isFinite(a)&&Math.floor(a)===a&&n.removeComments}}(t.expression)&&Ye(\".\");ge(23,t.expression.end,Ye,t),mn(i),Q(t.name),pn(a,i)}(a);case 188:return function(e){$(e.expression),ge(21,e.expression.end,Ye,e),$(e.argumentExpression),ge(22,e.argumentExpression.end,Ye,e)}(a);case 189:return function(e){$(e.expression),Be(e,e.typeArguments),qe(e,e.arguments,1296)}(a);case 190:return function(e){ge(94,e.pos,Ze,e),nn(),$(e.expression),Be(e,e.typeArguments),qe(e,e.arguments,9488)}(a);case 191:return function(e){$(e.tag),Be(e,e.typeArguments),nn(),$(e.template)}(a);case 192:return function(e){Ye(\"<\"),Q(e.type),Ye(\">\"),$(e.expression)}(a);case 193:return function(e){var n=ge(19,e.pos,Ye,e);$(e.expression),ge(20,e.expression?e.expression.end:n,Ye,e)}(a);case 194:return function(e){Dn(e.name),_e(e)}(a);case 195:return function(e){Ke(e,e.decorators),Ne(e,e.modifiers),ye(e,de)}(a);case 196:return function(e){ge(80,e.pos,Ze,e),nn(),$(e.expression)}(a);case 197:return function(e){ge(103,e.pos,Ze,e),nn(),$(e.expression)}(a);case 198:return function(e){ge(105,e.pos,Ze,e),nn(),$(e.expression)}(a);case 199:return function(e){ge(121,e.pos,Ze,e),nn(),$(e.expression)}(a);case 200:return function(e){cn(e.operator,$e),function(e){var n=e.operand;return 200===n.kind&&(37===e.operator&&(37===n.operator||43===n.operator)||38===e.operator&&(38===n.operator||44===n.operator))}(e)&&nn();$(e.operand)}(a);case 201:return function(e){$(e.operand),cn(e.operator,$e)}(a);case 202:return function(e){var n=26!==e.operatorToken.kind,t=yn(e,e.left,e.operatorToken),r=yn(e,e.operatorToken,e.right);$(e.left),mn(t,n?\" \":void 0),M(e.operatorToken.pos),ln(e.operatorToken,$e),k(e.operatorToken.end,!0),mn(r,\" \"),$(e.right),pn(t,r)}(a);case 203:return function(e){var n=yn(e,e.condition,e.questionToken),t=yn(e,e.questionToken,e.whenTrue),r=yn(e,e.whenTrue,e.colonToken),a=yn(e,e.colonToken,e.whenFalse);$(e.condition),mn(n,\" \"),Q(e.questionToken),mn(t,\" \"),$(e.whenTrue),pn(n,t),mn(r,\" \"),Q(e.colonToken),mn(a,\" \"),$(e.whenFalse),pn(r,a)}(a);case 204:return function(e){Q(e.head),We(e,e.templateSpans,131072)}(a);case 205:return function(e){ge(116,e.pos,Ze,e),Q(e.asteriskToken),Ge(e.expression)}(a);case 206:return function(e){Ye(\"...\"),$(e.expression)}(a);case 207:return function(e){Dn(e.name),Se(e)}(a);case 208:return;case 210:return function(e){$(e.expression),e.type&&(nn(),Ze(\"as\"),nn(),Q(e.type))}(a);case 211:return function(e){$(e.expression),$e(\"!\")}(a);case 212:return function(e){sn(e.keywordToken,e.pos,Ye),Ye(\".\"),Q(e.name)}(a);case 258:return function(e){Q(e.openingElement),We(e,e.children,131072),Q(e.closingElement)}(a);case 259:return function(e){Ye(\"<\"),Ae(e.tagName),nn(),Q(e.attributes),Ye(\"/>\")}(a);case 262:return function(e){Q(e.openingFragment),We(e,e.children,131072),Q(e.closingFragment)}(a);case 306:return function(e){$(e.expression)}(a);case 307:return function(e){qe(e,e.elements,272)}(a)}}function se(e,n){return n&&b&&b(e,n)||n}function le(t){var a=!1,i=278===t.kind?t:void 0;if(!i||F!==e.ModuleKind.None){for(var o=i?i.sourceFiles.length:1,s=0;s<o;s++){var l=i?i.sourceFiles[s]:t,c=e.isSourceFile(l)?l:r,u=n.noEmitHelpers||void 0!==e.getExternalHelpersModuleName(c),d=e.isSourceFile(l)&&!w,m=e.getEmitHelpers(l);if(m)for(var p=0,f=e.stableSort(m,e.compareEmitHelpers);p<f.length;p++){var g=f[p];if(g.scoped){if(i)continue}else{if(u)continue;if(d){if(G.get(g.name))continue;G.set(g.name,!0)}}\"string\"==typeof g.text?dn(g.text):dn(g.text(wn)),a=!0}}return a}}function ce(t){var r,a=Tn(t,n.neverAsciiEscape);!n.sourceMap&&!n.inlineSourceMap||9!==t.kind&&!e.isTemplateLiteralKind(t.kind)?function(e){I(),m.writeStringLiteral(e)}(a):(r=a,I(),m.writeLiteral(r))}function ue(e){(e.symbol?Xe:O)(En(e,!1),e.symbol),We(e,e.typeArguments,26896)}function de(e){He(e,e.typeParameters),je(e,e.parameters),we(e.type),nn(),Q(e.equalsGreaterThanToken)}function me(n,t){ge(17,n.pos,Ye,n);var r=t||1&e.getEmitFlags(n)?384:65;We(n,n.statements,r),ge(18,n.statements.end,Ye,n,!!(1&r))}function pe(e,n){var t=ge(106,n,Ze,e);nn(),ge(19,t,Ye,e),$(e.expression),ge(20,e.expression.end,Ye,e)}function fe(e){void 0!==e&&(236===e.kind?Q(e):$(e))}function ge(n,t,a,i,o){var s=e.getParseTreeNode(i),l=s&&s.kind===i.kind,c=t;if(l&&(t=e.skipTrivia(r.text,t)),M&&l&&i.pos!==c){var u=o&&!e.positionsAreOnSameLine(c,t,r);u&&an(),M(c),u&&on()}return t=cn(n,a,t),k&&l&&i.end!==t&&k(t,!0),t}function _e(e){Ke(e,e.decorators),Ne(e,e.modifiers),Ze(\"function\"),Q(e.asteriskToken),nn(),Z(e.name),ye(e,he)}function ve(e,n){be(n)}function ye(n,t){var r=n.body;if(r)if(e.isBlock(r)){var a=65536&e.getEmitFlags(n);a&&an(),Sn(n),e.forEach(n.parameters,An),An(n.body),t(n),y?y(4,r,ve):be(r),Ln(n),a&&on()}else t(n),nn(),$(r);else t(n),R()}function he(e){He(e,e.typeParameters),Ue(e,e.parameters),we(e.type)}function be(n){nn(),Ye(\"{\"),an();var t=function(n){if(1&e.getEmitFlags(n))return!0;if(n.multiLine)return!1;if(!e.nodeIsSynthesized(n)&&!e.rangeIsOnSingleLine(n,r))return!1;if(fn(n,n.statements,2)||_n(n,n.statements,2))return!1;for(var t,a=0,i=n.statements;a<i.length;a++){var o=i[a];if(gn(t,o,2))return!1;t=o}return!0}(n)?Ee:Te;D?D(n,n.statements,t):t(n),on(),sn(18,n.statements.end,Ye,n)}function Ee(e){Te(e,!0)}function Te(e,n){var t=Me(e.statements,!0),r=m.getTextPos();le(e),0===t&&r===m.getTextPos()&&n?(on(),We(e,e.statements,384),an()):We(e,e.statements,1,t)}function Se(n){e.forEach(n.members,Cn),Ke(n,n.decorators),Ne(n,n.modifiers),Ze(\"class\"),n.name&&(nn(),Z(n.name));var t=65536&e.getEmitFlags(n);t&&an(),He(n,n.typeParameters),We(n,n.heritageClauses,0),nn(),Ye(\"{\"),We(n,n.members,65),Ye(\"}\"),t&&on()}function Le(e){Ye(\"{\"),We(e,e.elements,262576),Ye(\"}\")}function xe(e){e.propertyName&&(Q(e.propertyName),nn(),ge(118,e.propertyName.end,Ze,e),nn()),Q(e.name)}function Ae(e){71===e.kind?$(e):Q(e)}function Ce(n,t,a){var i=81985;1===t.length&&(e.nodeIsSynthesized(n)||e.nodeIsSynthesized(t[0])||e.rangeStartPositionsAreOnSameLine(n,t[0],r))?(sn(56,a,Ye,n),nn(),i&=-66):ge(56,a,Ye,n),We(n,t,i)}function De(e,n,t,a){if(e&&(O('/// <reference no-default-lib=\"true\"/>'),rn()),r&&r.moduleName&&(O('/// <amd-module name=\"'+r.moduleName+'\" />'),rn()),r&&r.amdDependencies)for(var i=0,o=r.amdDependencies;i<o.length;i++){var s=o[i];s.name?O('/// <amd-dependency name=\"'+s.name+'\" path=\"'+s.path+'\" />'):O('/// <amd-dependency path=\"'+s.path+'\" />'),rn()}for(var l=0,c=n;l<c.length;l++){var u=c[l];O('/// <reference path=\"'+u.fileName+'\" />'),rn()}for(var d=0,m=t;d<m.length;d++){u=m[d];O('/// <reference types=\"'+u.fileName+'\" />'),rn()}for(var p=0,f=a;p<f.length;p++){u=f[p];O('/// <reference lib=\"'+u.fileName+'\" />'),rn()}}function ke(n){var t=n.statements;Sn(n),e.forEach(n.statements,An),le(n);var r=e.findIndex(t,function(n){return!e.isPrologueDirective(n)});!function(e){e.isDeclarationFile&&De(e.hasNoDefaultLib,e.referencedFiles,e.typeReferenceDirectives,e.libReferenceDirectives)}(n),We(n,t,1,-1===r?t.length:r),Ln(n)}function Me(n,t,r){for(var a=0;a<n.length;a++){var i=n[a];if(!e.isPrologueDirective(i))return a;(!r||!r.has(i.expression.text))&&((t||a>0)&&rn(),Q(i),r&&r.set(i.expression.text,!0))}return n.length}function Oe(n){if(e.isSourceFile(n))J(n),Me(n.statements);else for(var t=e.createMap(),r=0,a=n.sourceFiles;r<a.length;r++){var i=a[r];J(i),Me(i.statements,!0,t)}}function Ie(n){if(e.isSourceFile(n)){var t=e.getShebang(n.text);if(t)return O(t),rn(),!0}else for(var r=0,a=n.sourceFiles;r<a.length;r++){if(Ie(a[r]))break}}function Re(e,n){if(e){var t=O;O=n,Q(e),O=t}}function Ne(e,n){n&&n.length&&(We(e,n,131328),nn())}function we(e){e&&(Ye(\":\"),nn(),Q(e))}function Pe(e,n,t){e&&(nn(),ge(58,n,$e,t),nn(),$(e))}function Fe(e){e&&(nn(),Q(e))}function Ge(e){e&&(nn(),$(e))}function Ve(n,t){e.isBlock(t)||1&e.getEmitFlags(n)?(nn(),Q(t)):(rn(),an(),Q(t),on())}function Ke(e,n){We(e,n,24577)}function Be(e,n){We(e,n,26896)}function He(n,t){if(e.isFunctionLike(n)&&n.typeArguments)return Be(n,n.typeArguments);We(n,t,26896)}function Ue(e,n){We(e,n,1296)}function je(n,t){!function(n,t){var r=e.singleOrUndefined(t);return r&&r.pos===n.pos&&e.isArrowFunction(n)&&!n.type&&!e.some(n.decorators)&&!e.some(n.modifiers)&&!e.some(n.typeParameters)&&!e.some(r.decorators)&&!e.some(r.modifiers)&&!r.dotDotDotToken&&!r.questionToken&&!r.type&&!r.initializer&&e.isIdentifier(r.name)}(n,t)?Ue(n,t):We(n,t,272)}function We(e,n,t,r,a){Je(Q,e,n,t,r,a)}function qe(e,n,t,r,a){Je($,e,n,t,r,a)}function ze(e){switch(28&e){case 0:break;case 16:Ye(\",\");break;case 4:nn(),Ye(\"|\");break;case 8:nn(),Ye(\"&\")}}function Je(n,t,r,i,o,s){void 0===o&&(o=0),void 0===s&&(s=r?r.length-o:0);var l=void 0===r;if(!(l&&8192&i)){var c=void 0===r||o>=r.length||0===s;if(c&&16384&i)return E&&E(r),void(T&&T(r));if(7680&i&&(Ye(function(e){return a[7680&e][0]}(i)),c&&!l&&k(r.pos,!0)),E&&E(r),c)1&i?rn():128&i&&!(262144&i)&&nn();else{var u=0==(131072&i),d=u;fn(t,r,i)?(rn(),d=!1):128&i&&nn(),64&i&&an();for(var m=void 0,p=!1,f=0;f<s;f++){var g=r[o+f];if(m&&(28&i&&m.end!==t.end&&M(m.end),ze(i),gn(m,g,i)?(0==(67&i)&&(an(),p=!0),rn(),d=!1):m&&256&i&&nn()),d){if(k){var _=e.getCommentRange(g);k(_.pos)}}else d=u;n(g),p&&(on(),p=!1),m=g}var v=32&i&&r.hasTrailingComma;16&i&&v&&Ye(\",\"),m&&28&i&&m.end!==t.end&&!(1024&e.getEmitFlags(m))&&M(m.end),64&i&&on(),_n(t,r,i)?rn():128&i&&nn()}T&&T(r),7680&i&&(c&&!l&&M(r.end),Ye(function(e){return a[7680&e][1]}(i)))}}function Xe(e,n){I(),m.writeSymbol(e,n)}function Ye(e){I(),m.writePunctuation(e)}function Qe(){m.writePunctuation(\";\")}function Ze(e){I(),m.writeKeyword(e)}function $e(e){I(),m.writeOperator(e)}function en(e){I(),m.writeParameter(e)}function nn(){I(),m.writeSpace(\" \")}function tn(e){I(),m.writeProperty(e)}function rn(){I(),m.writeLine()}function an(){I(),m.increaseIndent()}function on(){I(),m.decreaseIndent()}function sn(e,n,t,r){return _?_(r,e,t,n,cn):cn(e,t,n)}function ln(n,t){S&&S(n),t(e.tokenToString(n.kind)),L&&L(n)}function cn(n,t,r){var a=e.tokenToString(n);return t(a),r<0?r:r+a.length}function un(n){1&e.getEmitFlags(n)?nn():rn()}function dn(n){for(var t=n.split(/\\r\\n?|\\n/g),r=e.guessIndentation(t),a=0,i=t;a<i.length;a++){var o=i[a],s=r?o.slice(r):o;s.length&&(rn(),O(s),rn())}}function mn(e,n){e?(an(),rn()):n&&O(n)}function pn(e,n){e&&on(),n&&on()}function fn(n,t,a){if(1&a)return!0;if(2&a){if(32768&a)return!0;var i=t[0];return void 0===i?!e.rangeIsOnSingleLine(n,r):e.positionIsSynthesized(n.pos)||e.nodeIsSynthesized(i)?vn(i,a):!e.rangeStartPositionsAreOnSameLine(n,i,r)}return!1}function gn(n,t,a){return!!(1&a)||(2&a?void 0!==n&&void 0!==t&&(e.nodeIsSynthesized(n)||e.nodeIsSynthesized(t)?vn(n,a)||vn(t,a):!e.rangeEndIsOnSameLineAsRangeStart(n,t,r)):e.getStartsOnNewLine(t))}function _n(n,t,a){if(1&a)return 0==(65536&a);if(2&a){if(32768&a)return!0;var i=e.lastOrUndefined(t);return void 0===i?!e.rangeIsOnSingleLine(n,r):e.positionIsSynthesized(n.pos)||e.nodeIsSynthesized(i)?vn(i,a):!e.rangeEndPositionsAreOnSameLine(n,i,r)}return!1}function vn(n,t){if(e.nodeIsSynthesized(n)){var r=e.getStartsOnNewLine(n);return void 0===r?0!=(32768&t):r}return 0!=(32768&t)}function yn(n,t,a){return n=bn(n),t=bn(t),a=bn(a),!!e.getStartsOnNewLine(a)||!(e.nodeIsSynthesized(n)||e.nodeIsSynthesized(t)||e.nodeIsSynthesized(a)||e.rangeEndIsOnSameLineAsRangeStart(t,a,r))}function hn(n){return 0===n.statements.length&&e.rangeEndIsOnSameLineAsRangeStart(n,n,r)}function bn(n){for(;193===n.kind&&e.nodeIsSynthesized(n);)n=n.expression;return n}function En(n,t){return e.isGeneratedIdentifier(n)?kn(n):e.isIdentifier(n)&&(e.nodeIsSynthesized(n)||!n.parent||!r||n.parent&&r&&e.getSourceFileOfNode(n)!==e.getOriginalNode(r))?e.idText(n):9===n.kind&&n.textSourceNode?En(n.textSourceNode,t):!e.isLiteralExpression(n)||!e.nodeIsSynthesized(n)&&n.parent?e.getSourceTextOfNodeFromSourceFile(r,n,t):n.text}function Tn(n,t){if(9===n.kind&&n.textSourceNode){var a=n.textSourceNode;return e.isIdentifier(a)?t||16777216&e.getEmitFlags(n)?'\"'+e.escapeString(En(a))+'\"':'\"'+e.escapeNonAsciiString(En(a))+'\"':Tn(a,t)}return e.getLiteralText(n,r,t)}function Sn(n){n&&524288&e.getEmitFlags(n)||(l.push(c),c=0,u.push(d))}function Ln(n){n&&524288&e.getEmitFlags(n)||(c=l.pop(),d=u.pop())}function xn(n){d&&d!==e.lastOrUndefined(u)||(d=e.createMap()),d.set(n,!0)}function An(n){if(n)switch(n.kind){case 216:e.forEach(n.statements,An);break;case 231:case 229:case 221:case 222:An(n.statement);break;case 220:An(n.thenStatement),An(n.elseStatement);break;case 223:case 225:case 224:An(n.initializer),An(n.statement);break;case 230:An(n.caseBlock);break;case 244:e.forEach(n.clauses,An);break;case 269:case 270:e.forEach(n.statements,An);break;case 233:An(n.tryBlock),An(n.catchClause),An(n.finallyBlock);break;case 272:An(n.variableDeclaration),An(n.block);break;case 217:An(n.declarationList);break;case 236:e.forEach(n.declarations,An);break;case 235:case 149:case 184:case 238:Dn(n.name);break;case 237:Dn(n.name),524288&e.getEmitFlags(n)&&(e.forEach(n.parameters,An),An(n.body));break;case 182:case 183:e.forEach(n.elements,An);break;case 247:An(n.importClause);break;case 248:Dn(n.name),An(n.namedBindings);break;case 249:Dn(n.name);break;case 250:e.forEach(n.elements,An);break;case 251:Dn(n.propertyName||n.name)}}function Cn(e){if(e)switch(e.kind){case 273:case 274:case 152:case 154:case 156:case 157:Dn(e.name)}}function Dn(n){n&&(e.isGeneratedIdentifier(n)?kn(n):e.isBindingPattern(n)&&An(n))}function kn(n){if(4==(7&n.autoGenerateFlags))return Mn(function(n){var t=n.autoGenerateId,r=n,a=r.original;for(;a&&(r=a,!(e.isIdentifier(r)&&4&r.autoGenerateFlags&&r.autoGenerateId!==t));)a=r.original;return r}(n),n.autoGenerateFlags);var t=n.autoGenerateId;return o[t]||(o[t]=function(n){switch(7&n.autoGenerateFlags){case 1:return Rn(0,!!(8&n.autoGenerateFlags));case 2:return Rn(268435456,!!(8&n.autoGenerateFlags));case 3:return Nn(e.idText(n),32&n.autoGenerateFlags?In:On,!!(16&n.autoGenerateFlags),!!(8&n.autoGenerateFlags))}return e.Debug.fail(\"Unsupported GeneratedIdentifierKind.\")}(n))}function Mn(n,t){var r=e.getNodeId(n);return i[r]||(i[r]=function(n,t){switch(n.kind){case 71:return Nn(En(n),On,!!(16&t),!!(8&t));case 242:case 241:return function(n){var t=En(n.name);return function(n,t){for(var r=t;e.isNodeDescendantOf(r,t);r=r.nextContainer)if(r.locals){var a=r.locals.get(e.escapeLeadingUnderscores(n));if(a&&70366143&a.flags)return!1}return!0}(t,n)?t:Nn(t)}(n);case 247:case 253:return function(n){var t=e.getExternalModuleName(n);return Nn(e.isStringLiteral(t)?e.makeIdentifierFromModuleName(t.text):\"module\")}(n);case 237:case 238:case 252:return Nn(\"default\");case 207:return Nn(\"class\");case 154:case 156:case 157:return function(n){if(e.isIdentifier(n.name))return Mn(n.name);return Rn(0)}(n);default:return Rn(0)}}(n,t))}function On(e){return In(e)&&!s.has(e)&&!(d&&d.has(e))}function In(n){return!r||e.isFileLevelUniqueName(r,n,f)}function Rn(e,n){if(e&&!(c&e)&&On(t=268435456===e?\"_i\":\"_n\"))return c|=e,n&&xn(t),t;for(;;){var t,r=268435455&c;if(c++,8!==r&&13!==r)if(On(t=r<26?\"_\"+String.fromCharCode(97+r):\"_\"+(r-26)))return n&&xn(t),t}}function Nn(e,n,t,r){if(void 0===n&&(n=On),t&&n(e))return r?xn(e):s.set(e,!0),e;95!==e.charCodeAt(e.length-1)&&(e+=\"_\");for(var a=1;;){var i=e+a;if(n(i))return r?xn(i):s.set(i,!0),i;a++}}function wn(e){return Nn(e,In,!0)}}e.forEachEmittedFile=i,e.getOutputPathsFor=o,e.getOutputExtension=l,e.emitFiles=function(n,t,r,a,o,s){var l,u=t.getCompilerOptions(),d=u.sourceMap||u.inlineSourceMap||e.getAreDeclarationMapsEnabled(u)?[]:void 0,m=u.listEmittedFiles?[]:void 0,p=e.createDiagnosticCollection(),f=t.getNewLine(),g=e.createTextWriter(f),_=e.createSourceMapWriter(t,g),v=e.createSourceMapWriter(t,g,{sourceMap:u.declarationMap,sourceRoot:u.sourceRoot,mapRoot:u.mapRoot,extendedDiagnostics:u.extendedDiagnostics}),y={originalOffset:-1,totalLength:-1},h=!1;return e.performance.mark(\"beforePrint\"),i(t,function(r,i){var d=r.jsFilePath,f=r.sourceMapFilePath,g=r.declarationFilePath,y=r.declarationMapPath,L=r.bundleInfoPath;(function(r,i,s,l){if(t.isEmitBlocked(i)||u.noEmit||u.emitDeclarationOnly)h=!0;else if(!a){var d=e.transformNodes(n,t,u,[r],o,!1),m=c(yn({},u,{noEmitHelpers:u.noEmitHelpers}),{hasGlobalName:n.hasGlobalName,onEmitNode:d.emitNodeWithNotification,substituteNode:d.substituteNode,onEmitSourceMapOfNode:_.emitNodeWithSourceMap,onEmitSourceMapOfToken:_.emitTokenWithSourceMap,onEmitSourceMapOfPosition:_.emitPos,onSetSourceFile:T});e.Debug.assert(1===d.transformed.length,\"Should only see one output from the transform\"),E(i,s,d.transformed[0],l,m,_),d.dispose()}})(i,d,f,L),function(r,i,o){if(i&&!e.isInJSFile(r)){var d=e.isSourceFile(r)?[r]:r.sourceFiles,m=e.filter(d,e.isSourceFileNotJS),f=u.outFile||u.out?[e.createBundle(m,e.isSourceFile(r)?void 0:r.prepends)]:m;a&&!e.getEmitDeclarations(u)&&m.forEach(b);var g=e.transformNodes(n,t,u,f,e.concatenate([e.transformDeclarations],s),!1);if(e.length(g.diagnostics))for(var _=0,y=g.diagnostics;_<y.length;_++){var T=y[_];p.add(T)}var L=c(yn({},u,{onlyPrintJsDocStyle:!0,noEmitHelpers:!0}),{hasGlobalName:n.hasGlobalName,onEmitSourceMapOfNode:v.emitNodeWithSourceMap,onEmitSourceMapOfToken:v.emitTokenWithSourceMap,onEmitSourceMapOfPosition:v.emitPos,onSetSourceFile:S,onEmitNode:g.emitNodeWithNotification,substituteNode:g.substituteNode}),x=!!g.diagnostics&&!!g.diagnostics.length||!!t.isEmitBlocked(i)||!!u.noEmit;if(h=h||x,(!x||a)&&(e.Debug.assert(1===g.transformed.length,\"Should only see one output from the decl transform\"),E(i,o,g.transformed[0],void 0,L,v),a&&277===g.transformed[0].kind)){var A=g.transformed[0];l=A.exportedModulesFromDeclarationEmit}g.dispose()}}(i,g,y),!h&&m&&(a||m.push(d),f&&m.push(f),g&&m.push(g),L&&m.push(L))},e.getSourceFilesToEmit(t,r),a),e.performance.measure(\"printTime\",\"beforePrint\"),{emitSkipped:h,diagnostics:p.getDiagnostics(),emittedFiles:m,sourceMaps:d,exportedModulesFromDeclarationEmit:l};function b(t){e.isExportAssignment(t)?71===t.expression.kind&&n.collectLinkedAliases(t.expression,!0):e.isExportSpecifier(t)?n.collectLinkedAliases(t.propertyName||t.name,!0):e.forEachChild(t,b)}function E(n,r,a,i,o,s){var l=278===a.kind?a:void 0,c=277===a.kind?a:void 0,m=l?l.sourceFiles:[c];s.initialize(n,r||\"\",a,d),l?o.writeBundle(l,g,y):o.writeFile(c,g),g.writeLine();var f=s.getSourceMappingURL();f&&g.write(\"//# sourceMappingURL=\"+f),r&&e.writeFile(t,p,r,s.getText(),!1,m),e.writeFile(t,p,n,g.getText(),!!u.emitBOM,m),i&&(y.totalLength=g.getTextPos(),e.writeFile(t,p,i,JSON.stringify(y,void 0,2),!1)),s.reset(),g.clear(),y={originalOffset:-1,totalLength:-1}}function T(e){_.setSourceFile(e)}function S(e){v.setSourceFile(e)}},function(e){e[e.Notification=0]=\"Notification\",e[e.Comments=1]=\"Comments\",e[e.SourceMaps=2]=\"SourceMaps\",e[e.Emit=3]=\"Emit\"}(n||(n={})),e.createPrinter=c,function(e){e[e.Auto=0]=\"Auto\",e[e.CountMask=268435455]=\"CountMask\",e[e._i=268435456]=\"_i\"}(t||(t={}))}(Tn||(Tn={})),function(e){var n;function t(e,n,t,r){return e.watchFile(n,t,r)}function r(e,n,t,r){return e.watchDirectory(n,t,0!=(1&r))}function a(e,t){switch(e){case n.None:return t;case n.TriggerOnly:return s;case n.Verbose:return t===r?o:i}}function i(e,n,t,r,a,i,o,c,u,d,m){u(d+\":: Added:: \"+l(n,r,i,o,m));var p=s(e,n,t,r,a,i,o,c,u,d,m);return{close:function(){u(d+\":: Close:: \"+l(n,r,i,o,m)),p.close()}}}function o(n,t,r,a,i,o,c,u,d,m,p){var f=m+\":: Added:: \"+l(t,a,o,c,p);d(f);var g=e.timestamp(),_=s(n,t,r,a,i,o,c,u,d,m,p),v=e.timestamp()-g;return d(\"Elapsed:: \"+v+\"ms \"+f),{close:function(){var n=m+\":: Close:: \"+l(t,a,o,c,p);d(n);var r=e.timestamp();_.close();var i=e.timestamp()-r;d(\"Elapsed:: \"+i+\"ms \"+n)}}}function s(n,t,r,a,i,o,s,c,u,d,m){return c(n,t,function(n,c){var p=d+\":: Triggered with \"+n+\" \"+(void 0!==c?c:\"\")+\":: \"+l(t,a,o,s,m);u(p);var f=e.timestamp();r(n,c,i);var g=e.timestamp()-f;u(\"Elapsed:: \"+g+\"ms \"+p)},a)}function l(e,n,t,r,a){return\"WatchInfo: \"+e+\" \"+n+\" \"+(a?a(t,r):t)}function c(e){e.watcher.close()}e.createCachedDirectoryStructureHost=function(n,t,r){if(n.getDirectories&&n.readDirectory){var a=e.createMap(),i=e.createGetCanonicalFileName(r);return{useCaseSensitiveFileNames:r,fileExists:function(e){var t=l(o(e));return t&&m(t.files,c(e))||n.fileExists(e)},readFile:function(e,t){return n.readFile(e,t)},directoryExists:n.directoryExists&&function(t){var r=o(t);return a.has(e.ensureTrailingDirectorySeparator(r))||n.directoryExists(t)},getDirectories:function(e){var t=o(e),r=u(e,t);return r?r.directories.slice():n.getDirectories(e)},readDirectory:function(a,i,s,l,c){var d=o(a),m=u(a,d);return m?e.matchFiles(a,i,s,l,r,t,c,function(n){var t=o(n);return t===d?m:u(n,t)||e.emptyFileSystemEntries}):n.readDirectory(a,i,s,l,c)},createDirectory:n.createDirectory&&function(e){var t=l(o(e)),r=c(e);t&&p(t.directories,r,!0),n.createDirectory(e)},writeFile:n.writeFile&&function(e,t,r){var a=l(o(e));return a&&f(a,c(e),!0),n.writeFile(e,t,r)},addOrDeleteFileOrDirectory:function(e,t){if(s(t))g();else{var r=l(t);if(r){if(n.directoryExists){var a=c(e),i={fileExists:n.fileExists(t),directoryExists:n.directoryExists(t)};return i.directoryExists||m(r.directories,a)?g():f(r,a,i.fileExists),i}g()}}},addOrDeleteFile:function(n,t,r){if(r!==e.FileWatcherEventKind.Changed){var a=l(t);a&&f(a,c(n),r===e.FileWatcherEventKind.Created)}},clearCache:g}}function o(n){return e.toPath(n,t,i)}function s(n){return a.get(e.ensureTrailingDirectorySeparator(n))}function l(n){return s(e.getDirectoryPath(n))}function c(n){return e.getBaseFileName(e.normalizePath(n))}function u(t,r){var i=s(r=e.ensureTrailingDirectorySeparator(r));if(i)return i;try{return function(t,r){var i={files:e.map(n.readDirectory(t,void 0,void 0,[\"*.*\"]),c)||[],directories:n.getDirectories(t)||[]};return a.set(e.ensureTrailingDirectorySeparator(r),i),i}(t,r)}catch(n){return void e.Debug.assert(!a.has(e.ensureTrailingDirectorySeparator(r)))}}function d(e,n){return i(e)===i(n)}function m(n,t){return e.some(n,function(e){return d(e,t)})}function p(n,t,r){if(m(n,t)){if(!r)return e.filterMutate(n,function(e){return!d(e,t)})}else if(r)return n.push(t)}function f(e,n,t){p(e.files,n,t)}function g(){a.clear()}},function(e){e[e.None=0]=\"None\",e[e.Partial=1]=\"Partial\",e[e.Full=2]=\"Full\"}(e.ConfigFileProgramReloadLevel||(e.ConfigFileProgramReloadLevel={})),e.updateMissingFilePathsWatch=function(n,t,r){var a=n.getMissingFilePaths(),i=e.arrayToSet(a);e.mutateMap(t,i,{createNewValue:r,onDeleteValue:e.closeFileWatcher})},e.updateWatchingWildcardDirectories=function(n,t,r){function a(e,n){return{watcher:r(e,n),flags:n}}e.mutateMap(n,t,{createNewValue:a,onDeleteValue:c,onExistingValue:function(e,t,r){e.flags!==t&&(e.watcher.close(),n.set(r,a(r,t)))}})},e.isEmittedFileOfProgram=function(e,n){return!!e&&e.isEmittedFile(n)},function(e){e[e.None=0]=\"None\",e[e.TriggerOnly=1]=\"TriggerOnly\",e[e.Verbose=2]=\"Verbose\"}(n=e.WatchLogLevel||(e.WatchLogLevel={})),e.getWatchFactory=function(e,i,o){return function(e,t,r,i,o){var s=a(e,i),l=e===n.None?function(e,n,t,r,a){return i(e,n,function(e,n){return t(e,n,a)},r)}:s,c=a(e,o);return{watchFile:function(e,n,a,o,l,c){return s(e,n,a,o,void 0,l,c,i,t,\"FileWatcher\",r)},watchFilePath:function(e,n,a,o,s,c,u){return l(e,n,a,o,s,c,u,i,t,\"FileWatcher\",r)},watchDirectory:function(e,n,a,i,s,l){return c(e,n,a,i,void 0,s,l,o,t,\"DirectoryWatcher\",r)}}}(e,i,o,t,r)},e.closeFileWatcherOf=c}(Tn||(Tn={})),function(e){var n,t=/(^\\s*$)|(^\\s*\\/\\/\\/?\\s*(@ts-ignore)?)/;function r(n,t){var r=e.getDirectoryPath(t),a=e.isRootedDiskPath(n)?n:e.combinePaths(r,n);return e.normalizePath(a)}function a(n,t,r){var a;return e.forEach(n,function(n){var i=e.getNormalizedPathComponents(n,t);if(i.pop(),a){for(var o=Math.min(a.length,i.length),s=0;s<o;s++)if(r(a[s])!==r(i[s])){if(0===s)return!0;a.length=s;break}i.length<a.length&&(a.length=i.length)}else a=i})?\"\":a?e.getPathFromPathComponents(a):t}function i(e,n){return o(e,n)}function o(n,t,r){void 0===r&&(r=e.sys);var a,i=e.createMap();function o(n){n.length>e.getRootLength(n)&&!function(e){return!!i.has(e)||!!r.directoryExists(e)&&(i.set(e,!0),!0)}(n)&&(o(e.getDirectoryPath(n)),r.createDirectory(n))}function s(){return e.getDirectoryPath(e.normalizePath(r.getExecutingFilePath()))}var l=e.getNewLineCharacter(n,function(){return r.newLine}),c=r.realpath&&function(e){return r.realpath(e)};return{getSourceFile:function(a,i,o){var s;try{e.performance.mark(\"beforeIORead\"),s=r.readFile(a,n.charset),e.performance.mark(\"afterIORead\"),e.performance.measure(\"I/O Read\",\"beforeIORead\",\"afterIORead\")}catch(e){o&&o(e.message),s=\"\"}return void 0!==s?e.createSourceFile(a,s,i,t):void 0},getDefaultLibLocation:s,getDefaultLibFileName:function(n){return e.combinePaths(s(),e.getDefaultLibFileName(n))},writeFile:function(t,i,s,l){try{e.performance.mark(\"beforeIOWrite\"),o(e.getDirectoryPath(e.normalizePath(t))),e.isWatchSet(n)&&r.createHash&&r.getModifiedTime?function(n,t,i){a||(a=e.createMap());var o=r.createHash(t),s=r.getModifiedTime(n);if(s){var l=a.get(n);if(l&&l.byteOrderMark===i&&l.hash===o&&l.mtime.getTime()===s.getTime())return}r.writeFile(n,t,i);var c=r.getModifiedTime(n)||e.missingFileModifiedTime;a.set(n,{hash:o,byteOrderMark:i,mtime:c})}(t,i,s):r.writeFile(t,i,s),e.performance.mark(\"afterIOWrite\"),e.performance.measure(\"I/O Write\",\"beforeIOWrite\",\"afterIOWrite\")}catch(e){l&&l(e.message)}},getCurrentDirectory:e.memoize(function(){return r.getCurrentDirectory()}),useCaseSensitiveFileNames:function(){return r.useCaseSensitiveFileNames},getCanonicalFileName:function(e){return r.useCaseSensitiveFileNames?e:e.toLowerCase()},getNewLine:function(){return l},fileExists:function(e){return r.fileExists(e)},readFile:function(e){return r.readFile(e)},trace:function(e){return r.write(e+l)},directoryExists:function(e){return r.directoryExists(e)},getEnvironmentVariable:function(e){return r.getEnvironmentVariable?r.getEnvironmentVariable(e):\"\"},getDirectories:function(e){return r.getDirectories(e)},realpath:c,readDirectory:function(e,n,t,a,i){return r.readDirectory(e,n,t,a,i)}}}function s(n,t){var r=e.diagnosticCategoryName(n)+\" TS\"+n.code+\": \"+h(n.messageText,t.getNewLine())+t.getNewLine();if(n.file){var a=e.getLineAndCharacterOfPosition(n.file,n.start),i=a.line,o=a.character,s=n.file.fileName;return e.convertToRelativePath(s,t.getCurrentDirectory(),function(e){return t.getCanonicalFileName(e)})+\"(\"+(i+1)+\",\"+(o+1)+\"): \"+r}return r}e.findConfigFile=function(n,t,r){return void 0===r&&(r=\"tsconfig.json\"),e.forEachAncestorDirectory(n,function(n){var a=e.combinePaths(n,r);return t(a)?a:void 0})},e.resolveTripleslashReference=r,e.computeCommonSourceDirectoryOfFilenames=a,e.createCompilerHost=i,e.createCompilerHostWorker=o,e.getPreEmitDiagnostics=function(n,t,r){var a=n.getConfigFileParsingDiagnostics().concat(n.getOptionsDiagnostics(r),n.getSyntacticDiagnostics(t,r),n.getGlobalDiagnostics(r),n.getSemanticDiagnostics(t,r));return e.getEmitDeclarations(n.getCompilerOptions())&&e.addRange(a,n.getDeclarationDiagnostics(t,r)),e.sortAndDeduplicateDiagnostics(a)},e.formatDiagnostics=function(e,n){for(var t=\"\",r=0,a=e;r<a.length;r++)t+=s(a[r],n);return t},e.formatDiagnostic=s,function(e){e.Grey=\"\u001b[90m\",e.Red=\"\u001b[91m\",e.Yellow=\"\u001b[93m\",e.Blue=\"\u001b[94m\",e.Cyan=\"\u001b[96m\"}(n=e.ForegroundColorEscapeSequences||(e.ForegroundColorEscapeSequences={}));var l=\"\u001b[7m\",c=\" \",u=\"\u001b[0m\",d=\"...\",m=\"  \",p=\"    \";function f(t){switch(t){case e.DiagnosticCategory.Error:return n.Red;case e.DiagnosticCategory.Warning:return n.Yellow;case e.DiagnosticCategory.Suggestion:return e.Debug.fail(\"Should never get an Info diagnostic on the command line.\");case e.DiagnosticCategory.Message:return n.Blue}}function g(e,n){return n+e+u}function _(e,n){for(;e.length<n;)e=\" \"+e;return e}function v(n,t,r,a,i,o){var s=e.getLineAndCharacterOfPosition(n,t),m=s.line,p=s.character,f=e.getLineAndCharacterOfPosition(n,t+r),v=f.line,y=f.character,h=e.getLineAndCharacterOfPosition(n,n.text.length).line,b=v-m>=4,E=(v+1+\"\").length;b&&(E=Math.max(d.length,E));for(var T=\"\",S=m;S<=v;S++){T+=o.getNewLine(),b&&m+1<S&&S<v-1&&(T+=a+g(_(d,E),l)+c+o.getNewLine(),S=v-1);var L=e.getPositionOfLineAndCharacter(n,S,0),x=S<h?e.getPositionOfLineAndCharacter(n,S+1,0):n.text.length,A=n.text.slice(L,x);if(A=(A=A.replace(/\\s+$/g,\"\")).replace(\"\\t\",\" \"),T+=a+g(_(S+1+\"\",E),l)+c,T+=A+o.getNewLine(),T+=a+g(_(\"\",E),l)+c,T+=i,S===m){var C=S===v?y:void 0;T+=A.slice(0,p).replace(/\\S/g,\" \"),T+=A.slice(p,C).replace(/./g,\"~\")}else T+=S===v?A.slice(0,y).replace(/./g,\"~\"):A.replace(/./g,\"~\");T+=u}return T}function y(t,r,a,i){void 0===i&&(i=g);var o=e.getLineAndCharacterOfPosition(t,r),s=o.line,l=o.character,c=\"\";return c+=i(a?e.convertToRelativePath(t.fileName,a.getCurrentDirectory(),function(e){return a.getCanonicalFileName(e)}):t.fileName,n.Cyan),c+=\":\",c+=i(\"\"+(s+1),n.Yellow),c+=\":\",c+=i(\"\"+(l+1),n.Yellow)}function h(n,t){if(e.isString(n))return n;for(var r=n,a=\"\",i=0;r;){if(i){a+=t;for(var o=0;o<i;o++)a+=\"  \"}a+=r.messageText,i++,r=r.next}return a}function b(n,t,r){if(0===n.length)return[];for(var a=[],i=e.createMap(),o=0,s=n;o<s.length;o++){var l=s[o],c=void 0;i.has(l)?c=i.get(l):i.set(l,c=r(l,t)),a.push(c)}return a}function E(n){return{fileExists:function(e){return n.fileExists(e)},readDirectory:function(t,r,a,i,o){return e.Debug.assertDefined(n.readDirectory,\"'CompilerHost.readDirectory' must be implemented to correctly process 'projectReferences'\"),n.readDirectory(t,r,a,i,o)},readFile:function(e){return n.readFile(e)},useCaseSensitiveFileNames:n.useCaseSensitiveFileNames(),getCurrentDirectory:function(){return n.getCurrentDirectory()},onUnRecoverableConfigFileDiagnostic:function(){}}}function T(n,t){var r=t||n;return e.resolveConfigFileProjectName(r.path)}function S(e){return e.declaration?\"declaration\":\"composite\"}function L(n,t){switch(t.extension){case\".ts\":case\".d.ts\":return;case\".tsx\":return r();case\".jsx\":return r()||a();case\".js\":return a();case\".json\":return n.resolveJsonModule?void 0:e.Diagnostics.Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used}function r(){return n.jsx?void 0:e.Diagnostics.Module_0_was_resolved_to_1_but_jsx_is_not_set}function a(){return n.allowJs||!e.getStrictOptionValue(n,\"noImplicitAny\")?void 0:e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type}}function x(e){for(var n=e.imports,t=e.moduleAugmentations,r=n.map(function(e){return e.text}),a=0,i=t;a<i.length;a++){var o=i[a];9===o.kind&&r.push(o.text)}return r}e.formatColorAndReset=g,e.formatLocation=y,e.formatDiagnosticsWithColorAndContext=function(t,r){for(var a=\"\",i=0,o=t;i<o.length;i++){var s=o[i];if(s.file&&(a+=y(d=s.file,_=s.start,r),a+=\" - \"),a+=g(e.diagnosticCategoryName(s),f(s.category)),a+=g(\" TS\"+s.code+\": \",n.Grey),a+=h(s.messageText,r.getNewLine()),s.file&&(a+=r.getNewLine(),a+=v(s.file,s.start,s.length,\"\",f(s.category),r),s.relatedInformation)){a+=r.getNewLine();for(var l=0,c=s.relatedInformation;l<c.length;l++){var u=c[l],d=u.file,_=u.start,b=u.length,E=u.messageText;d&&(a+=r.getNewLine(),a+=m+y(d,_,r),a+=v(d,_,b,p,n.Cyan,r)),a+=r.getNewLine(),a+=p+h(E,r.getNewLine())}}a+=r.getNewLine()}return a},e.flattenDiagnosticMessageText=h,e.isProgramUptoDate=function(n,t,r,a,i,o,s,l){if(!n||s)return!1;if(n.getRootFileNames().length!==t.length)return!1;if(!e.arrayIsEqualTo(n.getProjectReferences(),l,function(t,r,a){if(!e.projectReferenceIsEqualTo(t,r))return!1;var o=n.getResolvedProjectReferences()[a];return o?u(o.sourceFile):!i(T(t))}))return!1;if(n.getSourceFiles().some(function(e){return!u(e)||o(e.resolvedPath)}))return!1;if(n.getMissingFilePaths().some(i))return!1;var c=n.getCompilerOptions();return!(!e.compareDataObjects(c,r)||c.configFile&&r.configFile&&c.configFile.text!==r.configFile.text);function u(e){return e.version===a(e.resolvedPath)}},e.getConfigFileParsingDiagnostics=function(e){return e.options.configFile?e.options.configFile.parseDiagnostics.concat(e.errors):e.errors},e.createProgram=function(n,o,s,l,c){var u,d,m,p,f,g,_,v,y=e.isArray(n)?function(e,n,t,r,a){return{rootNames:e,options:n,host:t,oldProgram:r,configFileParsingDiagnostics:a}}(n,o,s,l,c):n,h=y.rootNames,A=y.options,C=y.configFileParsingDiagnostics,D=y.projectReferences,k=y.oldProgram,M=e.createMap(),O={},I={},R=e.createMap(),N=e.createDiagnosticCollection(),w=\"number\"==typeof A.maxNodeModuleJsDepth?A.maxNodeModuleJsDepth:0,P=0,F=e.createMap(),G=e.createMap();e.performance.mark(\"beforeProgram\");var V,K,B,H,U,j=y.host||i(A),W=E(j),q=A.noLib,z=e.memoize(function(){return j.getDefaultLibFileName(A)}),J=j.getDefaultLibLocation?j.getDefaultLibLocation():e.getDirectoryPath(z()),X=e.createDiagnosticCollection(),Y=j.getCurrentDirectory(),Q=e.getSupportedExtensions(A),Z=A.resolveJsonModule?Q.concat([\".json\"]):void 0,$=e.createMap(),ee=j.hasInvalidatedResolution||e.returnFalse;if(j.resolveModuleNames)H=function(n,t,r){return j.resolveModuleNames(e.Debug.assertEachDefined(n),t,r).map(function(n){if(!n||void 0!==n.extension)return n;var t=e.clone(n);return t.extension=e.extensionFromPath(n.resolvedFileName),t})};else{B=e.createModuleResolutionCache(Y,function(e){return j.getCanonicalFileName(e)});var ne=function(n,t){return e.resolveModuleName(n,t,A,j,B).resolvedModule};H=function(n,t){return b(e.Debug.assertEachDefined(n),t,ne)}}if(j.resolveTypeReferenceDirectives)U=function(n,t){return j.resolveTypeReferenceDirectives(e.Debug.assertEachDefined(n),t)};else{var te=function(n,t){return e.resolveTypeReferenceDirective(n,t,A,j).resolvedTypeReferenceDirective};U=function(n,t){return b(e.Debug.assertEachDefined(n),t,te)}}var re,ae,ie=e.createMap(),oe=e.createMap(),se=e.createMultiMap(),le=e.createMap(),ce=j.useCaseSensitiveFileNames()?e.createMap():void 0,ue=D?[]:void 0,de=function(n,t){if(!n)return!1;var r=n.getCompilerOptions();return!!e.sourceFileAffectingCompilerOptions.some(function(n){return!e.isJsonEqual(e.getCompilerOptionValue(r,n),e.getCompilerOptionValue(t,n))})}(k,A),me=function(){if(!k)return 0;var n=k.getCompilerOptions();if(e.changesAffectModuleResolution(n,A))return k.structureIsReused=0;e.Debug.assert(!(3&k.structureIsReused));var t=k.getRootFileNames();if(!e.arrayIsEqualTo(t,h))return k.structureIsReused=0;if(!e.arrayIsEqualTo(A.types,n.types))return k.structureIsReused=0;var r=k.getProjectReferences();if(!e.arrayIsEqualTo(r,D,e.projectReferenceIsEqualTo))return k.structureIsReused=0;var a=k.getResolvedProjectReferences();if(D){e.Debug.assert(!!a);for(var i=0;i<D.length;i++){var o=a[i],s=mn(D[i]);if(o){if(!s||s.sourceFile!==o.sourceFile)return k.structureIsReused=0}else if(void 0!==s)return k.structureIsReused=0}}else e.Debug.assert(!a);var l=[],c=[],u=[];if(k.structureIsReused=2,k.getMissingFilePaths().some(function(e){return j.fileExists(e)}))return k.structureIsReused=0;var d,m=k.getSourceFiles();!function(e){e[e.Exists=0]=\"Exists\",e[e.Modified=1]=\"Modified\"}(d||(d={}));for(var f=e.createMap(),g=0,_=m;g<_.length;g++){var v=_[g],y=j.getSourceFileByPath?j.getSourceFileByPath(v.fileName,v.resolvedPath,A.target,void 0,de):j.getSourceFile(v.fileName,A.target,void 0,de);if(!y)return k.structureIsReused=0;e.Debug.assert(!y.redirectInfo,\"Host should not return a redirect source file from `getSourceFile`\");var b=void 0;if(v.redirectInfo){if(y!==v.redirectInfo.unredirected)return k.structureIsReused=0;b=!1,y=v}else if(k.redirectTargetsMap.has(v.path)){if(y!==v)return k.structureIsReused=0;b=!1}else b=y!==v;y.path=v.path,y.originalFileName=v.originalFileName,y.resolvedPath=v.resolvedPath,y.fileName=v.fileName,c.push(y.path);var E=k.sourceFileToPackageName.get(v.path);if(void 0!==E){var T=f.get(E),S=b?1:0;if(void 0!==T&&1===S||1===T)return k.structureIsReused=0;f.set(E,S)}if(b){if(!e.arrayIsEqualTo(v.libReferenceDirectives,y.libReferenceDirectives,Xe))return k.structureIsReused=0;v.hasNoDefaultLib!==y.hasNoDefaultLib&&(k.structureIsReused=1),e.arrayIsEqualTo(v.referencedFiles,y.referencedFiles,Xe)||(k.structureIsReused=1),Qe(y),e.arrayIsEqualTo(v.imports,y.imports,Ye)||(k.structureIsReused=1),e.arrayIsEqualTo(v.moduleAugmentations,y.moduleAugmentations,Ye)||(k.structureIsReused=1),(1572864&v.flags)!=(1572864&y.flags)&&(k.structureIsReused=1),e.arrayIsEqualTo(v.typeReferenceDirectives,y.typeReferenceDirectives,Xe)||(k.structureIsReused=1),u.push({oldFile:v,newFile:y})}else ee(v.path)&&(k.structureIsReused=1,u.push({oldFile:v,newFile:y}));l.push(y)}if(2!==k.structureIsReused)return k.structureIsReused;for(var L=u.map(function(e){return e.oldFile}),C=0,O=m;C<O.length;C++){var I=O[C];if(!e.contains(L,I))for(var w=0,P=I.ambientModuleNames;w<P.length;w++){var F=P[w];M.set(F,I.fileName)}}for(var V=0,K=u;V<K.length;V++){var B=K[V],v=B.oldFile,y=B.newFile,W=e.getNormalizedAbsolutePath(y.originalFileName,Y);if(H){var q=x(y),z=ke(q,W,y),J=e.hasChangesInResolutions(q,z,v.resolvedModules,e.moduleResolutionIsEqualTo);J?(k.structureIsReused=1,y.resolvedModules=e.zipToMap(q,z)):y.resolvedModules=v.resolvedModules}if(U){var X=e.map(y.typeReferenceDirectives,function(e){return e.fileName.toLocaleLowerCase()}),z=U(X,W),J=e.hasChangesInResolutions(X,z,v.resolvedTypeReferenceDirectiveNames,e.typeDirectiveIsEqualTo);J?(k.structureIsReused=1,y.resolvedTypeReferenceDirectiveNames=e.zipToMap(X,z)):y.resolvedTypeReferenceDirectiveNames=v.resolvedTypeReferenceDirectiveNames}}if(2!==k.structureIsReused)return k.structureIsReused;if(j.hasChangedAutomaticTypeDirectiveNames)return k.structureIsReused=1;re=k.getMissingFilePaths();for(var i=0;i<l.length;i++)le.set(c[i],l[i]),k.isSourceFileFromExternalLibrary(k.getSourceFileByPath(c[i]))&&G.set(c[i],!0);p=l,N=k.getFileProcessingDiagnostics();for(var Q=0,Z=u;Q<Z.length;Q++){var $=Z[Q];N.reattachFileDiagnostics($.newFile)}return R=k.getResolvedTypeReferenceDirectives(),(ue=k.getResolvedProjectReferences())&&ue.forEach(function(e){e&&pn(e.commandLine)}),oe=k.sourceFileToPackageName,se=k.redirectTargetsMap,k.structureIsReused=2}();if(2!==me){if(d=[],m=[],D)for(var pe=0,fe=D;pe<fe.length;pe++){var ge=mn(fe[pe]);if(ue.push(ge),ge){var _e=ge.commandLine.options.outFile||ge.commandLine.options.out;_e&&$e(e.changeExtension(_e,\".d.ts\"),!1,!1,void 0),pn(ge.commandLine)}}e.forEach(h,function(e){return Je(e,!1,!1)});var ve=e.getAutomaticTypeDirectiveNames(A,j);if(ve.length)for(var ye=A.configFilePath?e.getDirectoryPath(A.configFilePath):j.getCurrentDirectory(),he=e.combinePaths(ye,\"__inferred type names__.ts\"),be=U(ve,he),Ee=0;Ee<ve.length;Ee++)on(ve[Ee],be[Ee]);if(!q){var Te=z();!A.lib&&Te?Je(Te,!0,!1):e.forEach(A.lib,function(n){Je(e.combinePaths(J,n),!0,!1)})}re=e.arrayFrom(le.keys(),function(e){return e}).filter(function(e){return!le.get(e)}),p=e.stableSort(d,function(n,t){return e.compareValues(Ae(n),Ae(t))}).concat(m),d=void 0,m=void 0}if(e.Debug.assert(!!re),k&&j.onReleaseOldSourceFile)for(var Se=0,Le=k.getSourceFiles();Se<Le.length;Se++){var xe=Le[Se];Pe(xe.path)&&!de||j.onReleaseOldSourceFile(xe,k.getCompilerOptions())}return k=void 0,u={getRootFileNames:function(){return h},getSourceFile:Pe,getSourceFileByPath:Fe,getSourceFiles:function(){return p},getMissingFilePaths:function(){return re},getCompilerOptions:function(){return A},getSyntacticDiagnostics:function(e,n){return Ge(e,Ve,n)},getOptionsDiagnostics:function(){return e.sortAndDeduplicateDiagnostics(e.concatenate(N.getGlobalDiagnostics(),e.concatenate(X.getGlobalDiagnostics(),A.configFile?X.getDiagnostics(A.configFile.fileName):[])))},getGlobalDiagnostics:function(){return e.sortAndDeduplicateDiagnostics(Re().getGlobalDiagnostics().slice())},getSemanticDiagnostics:function(e,n){return Ge(e,Be,n)},getSuggestionDiagnostics:function(e,n){return Ke(function(){return Re().getSuggestionDiagnostics(e,n)})},getDeclarationDiagnostics:function(e,n){var t=u.getCompilerOptions();return!e||t.out||t.outFile?je(e,n):Ge(e,ze,n)},getTypeChecker:Ne,getClassifiableNames:function(){if(!v){Ne(),v=e.createUnderscoreEscapedMap();for(var n=0,t=p;n<t.length;n++){var r=t[n];e.copyEntries(r.classifiableNames,v)}}return v},getDiagnosticsProducingTypeChecker:Re,getCommonSourceDirectory:De,emit:function(n,t,r,a,i){return Ke(function(){return function(n,t,r,a,i,o){var s=[];if(!i){if(A.noEmit)return{diagnostics:s,sourceMaps:void 0,emittedFiles:void 0,emitSkipped:!0};if(A.noEmitOnError){var l=n.getOptionsDiagnostics(a).concat(n.getSyntacticDiagnostics(t,a),n.getGlobalDiagnostics(a),n.getSemanticDiagnostics(t,a));if(0===l.length&&e.getEmitDeclarations(n.getCompilerOptions())&&(s=n.getDeclarationDiagnostics(void 0,a)),l.length>0||s.length>0)return{diagnostics:e.concatenate(l,s),sourceMaps:void 0,emittedFiles:void 0,emitSkipped:!0}}}var c=Re().getEmitResolver(A.outFile||A.out?void 0:t,a);e.performance.mark(\"beforeEmit\");var u=i?[]:e.getTransformers(A,o),d=e.emitFiles(c,Me(r),t,i,u,o&&o.afterDeclarations);return e.performance.mark(\"afterEmit\"),e.performance.measure(\"Emit\",\"beforeEmit\",\"afterEmit\"),d}(u,n,t,r,a,i)})},getCurrentDirectory:function(){return Y},getNodeCount:function(){return Re().getNodeCount()},getIdentifierCount:function(){return Re().getIdentifierCount()},getSymbolCount:function(){return Re().getSymbolCount()},getTypeCount:function(){return Re().getTypeCount()},getFileProcessingDiagnostics:function(){return N},getResolvedTypeReferenceDirectives:function(){return R},isSourceFileFromExternalLibrary:Ie,isSourceFileDefaultLibrary:function(n){if(n.hasNoDefaultLib)return!0;if(!A.noLib)return!1;var t=j.useCaseSensitiveFileNames()?e.equateStringsCaseSensitive:e.equateStringsCaseInsensitive;return A.lib?e.some(A.lib,function(r){return t(n.fileName,e.combinePaths(J,r))}):t(n.fileName,z())},dropDiagnosticsProducingTypeChecker:function(){g=void 0},getSourceFileFromReference:function(e,n){return Ze(r(n.fileName,e.fileName),function(e){return le.get(Ce(e))})},getLibFileFromReference:function(n){var t=n.fileName.toLocaleLowerCase(),r=e.libMap.get(t);if(r)return Pe(e.combinePaths(J,r))},sourceFileToPackageName:oe,redirectTargetsMap:se,isEmittedFile:function(n){if(A.noEmit)return!1;var t=Ce(n);if(Fe(t))return!1;var r=A.outFile||A.out;if(r)return xn(t,r)||xn(t,e.removeFileExtension(r)+\".d.ts\");if(A.declarationDir&&e.containsPath(A.declarationDir,t,Y,!j.useCaseSensitiveFileNames()))return!0;if(A.outDir)return e.containsPath(A.outDir,t,Y,!j.useCaseSensitiveFileNames());if(e.fileExtensionIsOneOf(t,e.supportedJSExtensions)||e.fileExtensionIs(t,\".d.ts\")){var a=e.removeFileExtension(t);return!!Fe(a+\".ts\")||!!Fe(a+\".tsx\")}return!1},getConfigFileParsingDiagnostics:function(){return C||e.emptyArray},getResolvedModuleWithFailedLookupLocationsFromCache:function(n,t){return B&&e.resolveModuleNameFromCache(n,t,B)},getProjectReferences:function(){return D},getResolvedProjectReferences:function(){return ue},getProjectReferenceRedirect:tn},function(){if(A.strictPropertyInitialization&&!e.getStrictOptionValue(A,\"strictNullChecks\")&&vn(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,\"strictPropertyInitialization\",\"strictNullChecks\"),A.isolatedModules&&(e.getEmitDeclarations(A)&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,S(A),\"isolatedModules\"),A.noEmitOnError&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"noEmitOnError\",\"isolatedModules\"),A.out&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"out\",\"isolatedModules\"),A.outFile&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"outFile\",\"isolatedModules\")),A.inlineSourceMap&&(A.sourceMap&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"sourceMap\",\"inlineSourceMap\"),A.mapRoot&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"mapRoot\",\"inlineSourceMap\")),A.paths&&void 0===A.baseUrl&&vn(e.Diagnostics.Option_paths_cannot_be_used_without_specifying_baseUrl_option,\"paths\"),A.composite&&!1===A.declaration&&vn(e.Diagnostics.Composite_projects_may_not_disable_declaration_emit,\"declaration\"),D)for(var n=0;n<D.length;n++){var t=D[n],r=ue[n]&&ue[n].commandLine.options;if(void 0!==r){if(r.composite||bn(n,e.Diagnostics.Referenced_project_0_must_have_setting_composite_Colon_true,t.path),t.prepend){var a=r.outFile||r.out;a?j.fileExists(a)||bn(n,e.Diagnostics.Output_file_0_from_project_1_does_not_exist,a,t.path):bn(n,e.Diagnostics.Cannot_prepend_project_0_because_it_does_not_have_outFile_set,t.path)}}else bn(n,e.Diagnostics.File_0_does_not_exist,t.path)}if(A.composite){var i=p.filter(function(e){return!e.isDeclarationFile});if(h.length<i.length)for(var o=h.map(function(n){return e.normalizePath(n).toLowerCase()}),s=0,l=i.map(function(n){return e.normalizePath(n.path).toLowerCase()});s<l.length;s++){var c=l[s];-1===o.indexOf(c)&&X.add(e.createCompilerDiagnostic(e.Diagnostics.File_0_is_not_in_project_file_list_Projects_must_list_all_files_or_use_an_include_pattern,c))}}if(A.paths)for(var u in A.paths)if(e.hasProperty(A.paths,u))if(e.hasZeroOrOneAsteriskCharacter(u)||gn(!0,u,e.Diagnostics.Pattern_0_can_have_at_most_one_Asterisk_character,u),e.isArray(A.paths[u])){var d=A.paths[u].length;0===d&&gn(!1,u,e.Diagnostics.Substitutions_for_pattern_0_shouldn_t_be_an_empty_array,u);for(var n=0;n<d;n++){var m=A.paths[u][n],f=typeof m;\"string\"===f?e.hasZeroOrOneAsteriskCharacter(m)||fn(u,n,e.Diagnostics.Substitution_0_in_pattern_1_in_can_have_at_most_one_Asterisk_character,m,u):fn(u,n,e.Diagnostics.Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2,m,u,f)}}else gn(!1,u,e.Diagnostics.Substitutions_for_pattern_0_should_be_an_array,u);A.sourceMap||A.inlineSourceMap||(A.inlineSources&&vn(e.Diagnostics.Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided,\"inlineSources\"),A.sourceRoot&&vn(e.Diagnostics.Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided,\"sourceRoot\")),A.out&&A.outFile&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"out\",\"outFile\"),!A.mapRoot||A.sourceMap||A.declarationMap||vn(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,\"mapRoot\",\"sourceMap\",\"declarationMap\"),A.declarationDir&&(e.getEmitDeclarations(A)||vn(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,\"declarationDir\",\"declaration\",\"composite\"),(A.out||A.outFile)&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"declarationDir\",A.out?\"out\":\"outFile\")),A.declarationMap&&!e.getEmitDeclarations(A)&&vn(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,\"declarationMap\",\"declaration\",\"composite\"),A.lib&&A.noLib&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"lib\",\"noLib\"),A.noImplicitUseStrict&&e.getStrictOptionValue(A,\"alwaysStrict\")&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"noImplicitUseStrict\",\"alwaysStrict\");var g=A.target||0,_=A.outFile||A.out,v=e.forEach(p,function(n){return e.isExternalModule(n)&&!n.isDeclarationFile?n:void 0});if(A.isolatedModules){A.module===e.ModuleKind.None&&g<2&&vn(e.Diagnostics.Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES2015_or_higher,\"isolatedModules\",\"target\");var y=e.forEach(p,function(n){return e.isExternalModule(n)||n.isDeclarationFile?void 0:n});if(y){var b=e.getErrorSpanForNode(y,y);X.add(e.createFileDiagnostic(y,b.start,b.length,e.Diagnostics.Cannot_compile_namespaces_when_the_isolatedModules_flag_is_provided))}}else if(v&&g<2&&A.module===e.ModuleKind.None){var b=e.getErrorSpanForNode(v,v.externalModuleIndicator);X.add(e.createFileDiagnostic(v,b.start,b.length,e.Diagnostics.Cannot_use_imports_exports_or_module_augmentations_when_module_is_none))}if(_)if(A.module&&A.module!==e.ModuleKind.AMD&&A.module!==e.ModuleKind.System)vn(e.Diagnostics.Only_amd_and_system_modules_are_supported_alongside_0,A.out?\"out\":\"outFile\",\"module\");else if(void 0===A.module&&v){var b=e.getErrorSpanForNode(v,v.externalModuleIndicator);X.add(e.createFileDiagnostic(v,b.start,b.length,e.Diagnostics.Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system,A.out?\"out\":\"outFile\"))}if(A.resolveJsonModule&&(e.getEmitModuleResolutionKind(A)!==e.ModuleResolutionKind.NodeJs?vn(e.Diagnostics.Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy,\"resolveJsonModule\"):e.hasJsonModuleEmitEnabled(A)||vn(e.Diagnostics.Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_esNext,\"resolveJsonModule\",\"module\")),A.outDir||A.sourceRoot||A.mapRoot){var E=De();A.outDir&&\"\"===E&&e.forEach(p,function(n){return e.getRootLength(n.fileName)>1})&&vn(e.Diagnostics.Cannot_find_the_common_subdirectory_path_for_the_input_files,\"outDir\")}if(!A.noEmit&&A.allowJs&&e.getEmitDeclarations(A)&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"allowJs\",S(A)),A.checkJs&&!A.allowJs&&X.add(e.createCompilerDiagnostic(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,\"checkJs\",\"allowJs\")),A.emitDeclarationOnly&&(e.getEmitDeclarations(A)||vn(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,\"emitDeclarationOnly\",\"declaration\",\"composite\"),A.noEmit&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"emitDeclarationOnly\",\"noEmit\")),A.emitDecoratorMetadata&&!A.experimentalDecorators&&vn(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,\"emitDecoratorMetadata\",\"experimentalDecorators\"),A.jsxFactory?(A.reactNamespace&&vn(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,\"reactNamespace\",\"jsxFactory\"),e.parseIsolatedEntityName(A.jsxFactory,g)||hn(\"jsxFactory\",e.Diagnostics.Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name,A.jsxFactory)):A.reactNamespace&&!e.isIdentifierText(A.reactNamespace,g)&&hn(\"reactNamespace\",e.Diagnostics.Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier,A.reactNamespace),!A.noEmit&&!A.suppressOutputPathCheck){var T=Me(),L=e.createMap();e.forEachEmittedFile(T,function(e){A.emitDeclarationOnly||x(e.jsFilePath,L),x(e.declarationFilePath,L)})}function x(n,t){if(n){var r,a=Ce(n);le.has(a)&&(A.configFilePath||(r=e.chainDiagnosticMessages(void 0,e.Diagnostics.Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript_files_Learn_more_at_https_Colon_Slash_Slashaka_ms_Slashtsconfig)),r=e.chainDiagnosticMessages(r,e.Diagnostics.Cannot_write_file_0_because_it_would_overwrite_input_file,n),Ln(n,e.createCompilerDiagnosticFromMessageChain(r)));var i=j.useCaseSensitiveFileNames()?a:a.toLocaleLowerCase();t.has(i)?Ln(n,e.createCompilerDiagnostic(e.Diagnostics.Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files,n)):t.set(i,!0)}}}(),e.performance.mark(\"afterProgram\"),e.performance.measure(\"Program\",\"beforeProgram\",\"afterProgram\"),u;function Ae(n){if(e.containsPath(J,n.fileName,!1)){var t=e.getBaseFileName(n.fileName);if(\"lib.d.ts\"===t||\"lib.es6.d.ts\"===t)return 0;var r=e.removeSuffix(e.removePrefix(t,\"lib.\"),\".d.ts\"),a=e.libs.indexOf(r);if(-1!==a)return a+1}return e.libs.length+2}function Ce(n){return e.toPath(n,Y,cn)}function De(){if(void 0===f){var n=e.filter(p,function(n){return e.sourceFileMayBeEmitted(n,A,Ie)});A.rootDir&&dn(n,A.rootDir)?f=e.getNormalizedAbsolutePath(A.rootDir,Y):A.composite&&A.configFilePath?dn(n,f=e.getDirectoryPath(e.normalizeSlashes(A.configFilePath))):(t=n,f=a(e.mapDefined(t,function(e){return e.isDeclarationFile?void 0:e.fileName}),Y,cn)),f&&f[f.length-1]!==e.directorySeparator&&(f+=e.directorySeparator)}var t;return f}function ke(n,t,r){if(0===me&&!r.ambientModuleNames.length)return H(n,t);var a,i,o,s=k&&k.getSourceFile(t);if(s!==r&&r.resolvedModules){for(var l=[],c=0,u=n;c<u.length;c++){var d=u[c],m=r.resolvedModules.get(d);l.push(m)}return l}for(var p={},f=0;f<n.length;f++){if(d=n[f],r===s&&!ee(s.path)){var g=s&&s.resolvedModules.get(d);if(g){e.isTraceEnabled(A,j)&&e.trace(j,e.Diagnostics.Reusing_resolution_of_module_0_to_file_1_from_old_program,d,t),(i||(i=new Array(n.length)))[f]=g,(o||(o=[])).push(d);continue}}var _=!1;e.contains(r.ambientModuleNames,d)?(_=!0,e.isTraceEnabled(A,j)&&e.trace(j,e.Diagnostics.Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1,d,t)):_=h(d),_?(i||(i=new Array(n.length)))[f]=p:(a||(a=[])).push(d)}var v=a&&a.length?H(a,t,o):e.emptyArray;if(!i)return e.Debug.assert(v.length===n.length),v;var y=0;for(f=0;f<i.length;f++)i[f]?i[f]===p&&(i[f]=void 0):(i[f]=v[y],y++);return e.Debug.assert(y===v.length),i;function h(n){var t=e.getResolvedModule(s,n),r=t&&k.getSourceFile(t.resolvedFileName);if(t&&r&&!r.externalModuleIndicator)return!1;var a=M.get(n);return!!a&&(e.isTraceEnabled(A,j)&&e.trace(j,e.Diagnostics.Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified,n,a),!0)}}function Me(n){return yn({getPrependNodes:Oe,getCanonicalFileName:cn,getCommonSourceDirectory:u.getCommonSourceDirectory,getCompilerOptions:u.getCompilerOptions,getCurrentDirectory:function(){return Y},getNewLine:function(){return j.getNewLine()},getSourceFile:u.getSourceFile,getSourceFileByPath:u.getSourceFileByPath,getSourceFiles:u.getSourceFiles,getLibFileFromReference:u.getLibFileFromReference,isSourceFileFromExternalLibrary:Ie,writeFile:n||function(e,n,t,r,a){return j.writeFile(e,n,t,r,a)},isEmitBlocked:we,readFile:function(e){return j.readFile(e)},fileExists:function(n){var t=Ce(n);return!!Fe(t)||!e.contains(re,t)&&j.fileExists(n)}},j.directoryExists?{directoryExists:function(e){return j.directoryExists(e)}}:{},{useCaseSensitiveFileNames:function(){return j.useCaseSensitiveFileNames()}})}function Oe(){if(!D)return e.emptyArray;for(var n=[],t=0;t<D.length;t++){var r=D[t],a=ue[t].commandLine;if(r.prepend&&a&&a.options){var i=a.options.outFile||a.options.out;if(!i)continue;var o=e.changeExtension(i,\".d.ts\"),s=j.readFile(i)||\"/* Input file \"+i+\" was missing */\\r\\n\",l=i+\".map\",c=j.readFile(l),u=j.readFile(o)||\"/* Input file \"+o+\" was missing */\\r\\n\",d=o+\".map\",m=j.readFile(d),p=e.createInputFiles(s,u,c&&l,c,m&&d,m);n.push(p)}}return n}function Ie(e){return!!G.get(e.path)}function Re(){return g||(g=e.createTypeChecker(u,!0))}function Ne(){return _||(_=e.createTypeChecker(u,!1))}function we(e){return $.has(Ce(e))}function Pe(e){return Fe(Ce(e))}function Fe(e){return le.get(e)}function Ge(n,t,r){return n?t(n,r):e.sortAndDeduplicateDiagnostics(e.flatMap(u.getSourceFiles(),function(e){return r&&r.throwIfCancellationRequested(),t(e,r)}))}function Ve(n){return e.isSourceFileJS(n)?(n.additionalSyntacticDiagnostics||(n.additionalSyntacticDiagnostics=function(n){return Ke(function(){var t=[],r=n;return a(n),t;function a(n){switch(r.kind){case 149:case 152:if(r.questionToken===n)return void t.push(s(n,e.Diagnostics._0_can_only_be_used_in_a_ts_file,\"?\"));case 154:case 153:case 155:case 156:case 157:case 194:case 237:case 195:case 235:if(r.type===n)return void t.push(s(n,e.Diagnostics.types_can_only_be_used_in_a_ts_file))}switch(n.kind){case 246:return void t.push(s(n,e.Diagnostics.import_can_only_be_used_in_a_ts_file));case 252:if(n.isExportEquals)return void t.push(s(n,e.Diagnostics.export_can_only_be_used_in_a_ts_file));break;case 271:var o=n;if(108===o.token)return void t.push(s(n,e.Diagnostics.implements_clauses_can_only_be_used_in_a_ts_file));break;case 239:return void t.push(s(n,e.Diagnostics.interface_declarations_can_only_be_used_in_a_ts_file));case 242:return void t.push(s(n,e.Diagnostics.module_declarations_can_only_be_used_in_a_ts_file));case 240:return void t.push(s(n,e.Diagnostics.type_aliases_can_only_be_used_in_a_ts_file));case 241:return void t.push(s(n,e.Diagnostics.enum_declarations_can_only_be_used_in_a_ts_file));case 211:return void t.push(s(n,e.Diagnostics.non_null_assertions_can_only_be_used_in_a_ts_file));case 210:return void t.push(s(n.type,e.Diagnostics.type_assertion_expressions_can_only_be_used_in_a_ts_file));case 192:e.Debug.fail()}var l=r;r=n,e.forEachChild(n,a,i),r=l}function i(n){switch(r.decorators!==n||A.experimentalDecorators||t.push(s(r,e.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_to_remove_this_warning)),r.kind){case 238:case 154:case 153:case 155:case 156:case 157:case 194:case 237:case 195:if(n===r.typeParameters)return void t.push(o(n,e.Diagnostics.type_parameter_declarations_can_only_be_used_in_a_ts_file));case 217:if(n===r.modifiers)return function(n,r){for(var a=0,i=n;a<i.length;a++){var o=i[a];switch(o.kind){case 76:if(r)continue;case 114:case 112:case 113:case 132:case 124:case 117:t.push(s(o,e.Diagnostics._0_can_only_be_used_in_a_ts_file,e.tokenToString(o.kind)))}}}(n,217===r.kind);break;case 152:if(n===r.modifiers){for(var i=0,l=n;i<l.length;i++){var c=l[i];115!==c.kind&&t.push(s(c,e.Diagnostics._0_can_only_be_used_in_a_ts_file,e.tokenToString(c.kind)))}return}break;case 149:if(n===r.modifiers)return void t.push(o(n,e.Diagnostics.parameter_modifiers_can_only_be_used_in_a_ts_file));break;case 189:case 190:case 209:case 259:case 260:if(n===r.typeArguments)return void t.push(o(n,e.Diagnostics.type_arguments_can_only_be_used_in_a_ts_file))}for(var u=0,d=n;u<d.length;u++){var m=d[u];a(m)}}function o(t,r,a,i,o){var s=t.pos;return e.createFileDiagnostic(n,s,t.end-s,r,a,i,o)}function s(t,r,a,i,o){return e.createDiagnosticForNodeInSourceFile(n,t,r,a,i,o)}})}(n)),e.concatenate(n.additionalSyntacticDiagnostics,n.parseDiagnostics)):n.parseDiagnostics}function Ke(n){try{return n()}catch(n){throw n instanceof e.OperationCanceledException&&(_=void 0,g=void 0),n}}function Be(e,n){return qe(e,n,O,He)}function He(n,t){return Ke(function(){if(e.skipTypeChecking(n,A))return e.emptyArray;var r=Re();e.Debug.assert(!!n.bindDiagnostics);for(var a,i=e.isCheckJsEnabledForFile(n,A),o=3===n.scriptKind||4===n.scriptKind||5===n.scriptKind||i||7===n.scriptKind,s=0,l=[o?n.bindDiagnostics:e.emptyArray,o?r.getDiagnostics(n,t):e.emptyArray,N.getDiagnostics(n.fileName),X.getDiagnostics(n.fileName),i?n.jsDocDiagnostics:void 0];s<l.length;s++){var c=l[s];if(c)for(var u=0,d=c;u<d.length;u++){var m=d[u];Ue(m)&&(a=e.append(a,m))}}return a})}function Ue(n){var r=n.file,a=n.start;if(r)for(var i=e.getLineStarts(r),o=e.computeLineAndCharacterOfPosition(i,a).line;o>0;){var s=r.text.slice(i[o-1],i[o]),l=t.exec(s);if(!l)return!0;if(l[3])return!1;o--}return!0}function je(e,n){return qe(e,n,I,We)}function We(n,t){return Ke(function(){var r=Re().getEmitResolver(n,t);return e.getDeclarationDiagnostics(Me(e.noop),r,n)})}function qe(n,t,r,a){var i=n?r.perFile&&r.perFile.get(n.path):r.allDiagnostics;if(i)return i;var o=a(n,t)||e.emptyArray;return n?(r.perFile||(r.perFile=e.createMap()),r.perFile.set(n.path,o)):r.allDiagnostics=o,o}function ze(e,n){return e.isDeclarationFile?[]:je(e,n)}function Je(n,t,r){$e(e.normalizePath(n),t,r,void 0)}function Xe(e,n){return e.fileName===n.fileName}function Ye(e,n){return 71===e.kind?71===n.kind&&e.escapedText===n.escapedText:9===n.kind&&e.text===n.text}function Qe(n){if(!n.imports){var t,r,a,i=e.isSourceFileJS(n),o=e.isExternalModule(n);if(A.importHelpers&&(A.isolatedModules||o)&&!n.isDeclarationFile){var s=e.createLiteral(e.externalHelpersModuleNameText),l=e.createImportDeclaration(void 0,void 0,void 0,s);e.addEmitFlags(l,67108864),s.parent=l,l.parent=n,t=[s]}for(var c=0,u=n.statements;c<u.length;c++){var d=u[c];m(d,!1),(524288&n.flags||i)&&p(d)}return(524288&n.flags||i)&&p(n.endOfFileToken),n.imports=t||e.emptyArray,n.moduleAugmentations=r||e.emptyArray,void(n.ambientModuleNames=a||e.emptyArray)}function m(i,s){if(e.isAnyImportOrReExport(i)){var l=e.getExternalModuleName(i);!(l&&e.isStringLiteral(l)&&l.text)||s&&e.isExternalModuleNameRelative(l.text)||(t=e.append(t,l))}else if(e.isModuleDeclaration(i)&&e.isAmbientModule(i)&&(s||e.hasModifier(i,2)||n.isDeclarationFile)){var c=e.getTextOfIdentifierOrLiteral(i.name);if(o||s&&!e.isExternalModuleNameRelative(c))(r||(r=[])).push(i.name);else if(!s){n.isDeclarationFile&&(a||(a=[])).push(c);var u=i.body;if(u)for(var d=0,p=u.statements;d<p.length;d++)m(p[d],!0)}}}function p(n){e.isRequireCall(n,!0)?t=e.append(t,n.arguments[0]):e.isImportCall(n)&&1===n.arguments.length&&e.isStringLiteralLike(n.arguments[0])?t=e.append(t,n.arguments[0]):e.isLiteralImportTypeNode(n)&&(t=e.append(t,n.argument.literal)),f(n),e.hasJSDocNodes(n)&&e.forEach(n.jsDoc,f)}function f(n){e.forEachChild(n,p)}}function Ze(n,t,r,a){if(e.hasExtension(n)){if(!A.allowNonTsExtensions&&!e.forEach(Z||Q,function(t){return e.fileExtensionIs(j.getCanonicalFileName(n),t)}))return void(r&&r(e.Diagnostics.File_0_has_unsupported_extension_The_only_supported_extensions_are_1,n,\"'\"+Q.join(\"', '\")+\"'\"));var i=t(n);if(r)if(i)a&&j.getCanonicalFileName(n)===j.getCanonicalFileName(a.fileName)&&r(e.Diagnostics.A_file_cannot_have_a_reference_to_itself);else{var o=tn(n);o?r(e.Diagnostics.Output_file_0_has_not_been_built_from_source_file_1,o,n):r(e.Diagnostics.File_0_not_found,n)}return i}var s=A.allowNonTsExtensions&&t(n);if(s)return s;if(!r||!A.allowNonTsExtensions){var l=e.forEach(Q,function(e){return t(n+e)});return r&&!l&&r(e.Diagnostics.File_0_not_found,n+\".ts\"),l}r(e.Diagnostics.File_0_not_found,n)}function $e(n,t,r,a,i,o,s){Ze(n,function(e){return nn(e,Ce(e),t,r,i,o,s,a)},function(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];N.add(void 0!==i&&void 0!==s&&void 0!==o?e.createFileDiagnostic.apply(void 0,[i,o,s-o,n].concat(t)):e.createCompilerDiagnostic.apply(void 0,[n].concat(t)))},i)}function en(n,t,r,a,i){void 0!==r&&void 0!==a&&void 0!==i?N.add(e.createFileDiagnostic(r,a,i-a,e.Diagnostics.File_name_0_differs_from_already_included_file_name_1_only_in_casing,n,t)):N.add(e.createCompilerDiagnostic(e.Diagnostics.File_name_0_differs_from_already_included_file_name_1_only_in_casing,n,t))}function nn(n,t,r,a,i,o,s,l){var c,u=n;if(le.has(t)){var p=le.get(t);if(p&&A.forceConsistentCasingInFileNames){var f=n,g=p.fileName;Ce(g)!==Ce(f)&&(f=tn(n)||n),e.getNormalizedAbsolutePath(g,Y)!==e.getNormalizedAbsolutePath(f,Y)&&en(f,g,i,o,s)}return p&&G.get(p.path)&&0===P?(G.set(p.path,!1),A.noResolve||(rn(p,r),an(p)),sn(p),F.set(p.path,!1),un(p)):p&&F.get(p.path)&&P<w&&(F.set(p.path,!1),un(p)),p}if(i){var _=tn(n);_&&((i.redirectedReferences||(i.redirectedReferences=[])).push(n),n=_,c=Ce(_))}var v=j.getSourceFile(n,A.target,function(t){void 0!==i&&void 0!==o&&void 0!==s?N.add(e.createFileDiagnostic(i,o,s-o,e.Diagnostics.Cannot_read_file_0_Colon_1,n,t)):N.add(e.createCompilerDiagnostic(e.Diagnostics.Cannot_read_file_0_Colon_1,n,t))},de);if(l){var y=e.packageIdToString(l),h=ie.get(y);if(h){var b=function(e,n,t,r){var a=Object.create(e);return a.fileName=t,a.path=r,a.redirectInfo={redirectTarget:e,unredirected:n},Object.defineProperties(a,{id:{get:function(){return this.redirectInfo.redirectTarget.id},set:function(e){this.redirectInfo.redirectTarget.id=e}},symbol:{get:function(){return this.redirectInfo.redirectTarget.symbol},set:function(e){this.redirectInfo.redirectTarget.symbol=e}}}),a}(h,v,n,t);return se.add(h.path,n),le.set(t,b),oe.set(t,l.name),m.push(b),b}v&&(ie.set(y,v),oe.set(t,l.name))}if(le.set(t,v),c&&le.set(c,v),v){if(G.set(t,P>0),v.path=t,v.resolvedPath=Ce(n),v.originalFileName=u,j.useCaseSensitiveFileNames()){var E=t.toLowerCase(),T=ce.get(E);T?en(n,T.fileName,i,o,s):ce.set(E,v)}q=q||v.hasNoDefaultLib&&!a,A.noResolve||(rn(v,r),an(v)),sn(v),un(v),r?d.push(v):m.push(v)}return v}function tn(n){if(ae&&!e.fileExtensionIs(n,\".d.ts\")&&e.fileExtensionIsOneOf(n,e.supportedTSExtensions))return e.forEach(ae,function(t){if(e.contains(t.fileNames,n,xn)){var r=t.options.outFile||t.options.out;return r?e.changeExtension(r,\".d.ts\"):e.getOutputDeclarationFileName(n,t)}})}function rn(n,t){e.forEach(n.referencedFiles,function(e){$e(r(e.fileName,n.originalFileName),t,!1,void 0,n,e.pos,e.end)})}function an(n){var t=e.map(n.typeReferenceDirectives,function(e){return e.fileName.toLocaleLowerCase()});if(t)for(var r=U(t,n.originalFileName),a=0;a<t.length;a++){var i=n.typeReferenceDirectives[a],o=r[a],s=i.fileName.toLocaleLowerCase();e.setResolvedTypeReferenceDirective(n,s,o),on(s,o,n,i.pos,i.end)}}function on(n,t,r,a,i){var o=R.get(n);if(!o||!o.primary){var s=!0;t?t.primary?$e(t.resolvedFileName,!1,!1,t.packageId,r,a,i):o?(t.resolvedFileName!==o.resolvedFileName&&j.readFile(t.resolvedFileName)!==Pe(o.resolvedFileName).text&&N.add(ln(r,a,i,e.Diagnostics.Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_library_to_resolve_the_conflict,n,t.resolvedFileName,o.resolvedFileName)),s=!1):$e(t.resolvedFileName,!1,!1,t.packageId,r,a,i):N.add(ln(r,a,i,e.Diagnostics.Cannot_find_type_definition_file_for_0,n)),s&&R.set(n,t)}}function sn(n){e.forEach(n.libReferenceDirectives,function(t){var r=t.fileName.toLocaleLowerCase(),a=e.libMap.get(r);if(a)Je(e.combinePaths(J,a),!0,!0);else{var i=e.removeSuffix(e.removePrefix(r,\"lib.\"),\".d.ts\"),o=e.getSpellingSuggestion(i,e.libs,e.identity),s=o?e.Diagnostics.Cannot_find_lib_definition_for_0_Did_you_mean_1:e.Diagnostics.Cannot_find_lib_definition_for_0;N.add(ln(n,t.pos,t.end,s,r,o))}})}function ln(n,t,r,a){for(var i=[],o=4;o<arguments.length;o++)i[o-4]=arguments[o];return void 0===n||void 0===t||void 0===r?e.createCompilerDiagnostic.apply(void 0,[a].concat(i)):e.createFileDiagnostic.apply(void 0,[n,t,r-t,a].concat(i))}function cn(e){return j.getCanonicalFileName(e)}function un(n){if(Qe(n),n.imports.length||n.moduleAugmentations.length){var t=x(n),r=ke(t,e.getNormalizedAbsolutePath(n.originalFileName,Y),n);e.Debug.assert(r.length===t.length);for(var a=0;a<t.length;a++){var i=r[a];if(e.setResolvedModule(n,t[a],i),i){var o=i.isExternalLibraryImport,s=!e.resolutionExtensionIsTSOrJson(i.extension),l=o&&s,c=i.resolvedFileName;o&&P++;var u=l&&P>w,d=c&&!L(A,i)&&!A.noResolve&&a<n.imports.length&&!u&&!(s&&!A.allowJs)&&(e.isInJSFile(n.imports[a])||!(2097152&n.imports[a].flags));u?F.set(n.path,!0):d&&nn(c,Ce(c),!1,!1,n,e.skipTrivia(n.text,n.imports[a].pos),n.imports[a].end,i.packageId),o&&P--}}}else n.resolvedModules=void 0}function dn(n,t){for(var r=!0,a=j.getCanonicalFileName(e.getNormalizedAbsolutePath(t,Y)),i=0,o=n;i<o.length;i++){var s=o[i];s.isDeclarationFile||0!==j.getCanonicalFileName(e.getNormalizedAbsolutePath(s.fileName,Y)).indexOf(a)&&(X.add(e.createCompilerDiagnostic(e.Diagnostics.File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files,s.fileName,t)),r=!1)}return r}function mn(n){var t=T(n),r=e.getNormalizedAbsolutePath(e.getDirectoryPath(t),j.getCurrentDirectory()),a=j.getSourceFile(t,100);if(void 0!==a)return a.path=Ce(t),{commandLine:e.parseJsonSourceFileConfigFileContent(a,W,r,void 0,t),sourceFile:a}}function pn(e){(ae||(ae=[])).push(e)}function fn(n,t,r,a,i,o){for(var s=!0,l=0,c=_n();l<c.length;l++){var u=c[l];if(e.isObjectLiteralExpression(u.initializer))for(var d=0,m=e.getPropertyAssignment(u.initializer,n);d<m.length;d++){var p=m[d].initializer;e.isArrayLiteralExpression(p)&&p.elements.length>t&&(X.add(e.createDiagnosticForNodeInSourceFile(A.configFile,p.elements[t],r,a,i,o)),s=!1)}}s&&X.add(e.createCompilerDiagnostic(r,a,i,o))}function gn(n,t,r,a){for(var i=!0,o=0,s=_n();o<s.length;o++){var l=s[o];e.isObjectLiteralExpression(l.initializer)&&Sn(l.initializer,n,t,void 0,r,a)&&(i=!1)}i&&X.add(e.createCompilerDiagnostic(r,a))}function _n(){return function(n){var t=Tn();if(t)return e.getPropertyAssignment(t,n)}(\"paths\")||e.emptyArray}function vn(e,n,t,r){En(!0,n,t,e,n,t,r)}function hn(e,n,t){En(!1,e,void 0,n,t)}function bn(n,t,r,a){var i=function(){if(void 0===K&&(K=null,A.configFile))for(var n=e.getTsConfigObjectLiteralExpression(A.configFile),t=0,r=e.getPropertyAssignment(n,\"references\");t<r.length;t++){var a=r[t];if(e.isArrayLiteralExpression(a.initializer)){K=a.initializer;break}}return K}();i&&i.elements.length>n?X.add(e.createDiagnosticForNodeInSourceFile(A.configFile,i.elements[n],t,r,a)):X.add(e.createCompilerDiagnostic(t,r,a))}function En(n,t,r,a,i,o,s){var l=Tn();(!l||!Sn(l,n,t,r,a,i,o,s))&&X.add(e.createCompilerDiagnostic(a,i,o,s))}function Tn(){if(void 0===V){V=null;var n=e.getTsConfigObjectLiteralExpression(A.configFile);if(n)for(var t=0,r=e.getPropertyAssignment(n,\"compilerOptions\");t<r.length;t++){var a=r[t];if(e.isObjectLiteralExpression(a.initializer)){V=a.initializer;break}}}return V}function Sn(n,t,r,a,i,o,s,l){for(var c=e.getPropertyAssignment(n,r,a),u=0,d=c;u<d.length;u++){var m=d[u];X.add(e.createDiagnosticForNodeInSourceFile(A.configFile,t?m.name:m.initializer,i,o,s,l))}return!!c.length}function Ln(e,n){$.set(Ce(e),!0),X.add(n)}function xn(n,t){return 0===e.comparePaths(n,t,Y,!j.useCaseSensitiveFileNames())}},e.parseConfigHostFromCompilerHost=E,e.resolveProjectReferencePath=T,e.getResolutionDiagnostic=L}(Tn||(Tn={})),function(e){e.getFileEmitOutput=function(e,n,t,r,a){var i=[],o=e.emit(n,function(e,n,t){i.push({name:e,writeByteOrderMark:t,text:n})},r,t,a);return{outputFiles:i,emitSkipped:o.emitSkipped,exportedModulesFromDeclarationEmit:o.exportedModulesFromDeclarationEmit}}}(Tn||(Tn={})),function(e){!function(n){function t(n){if(n.declarations&&n.declarations[0]){var t=e.getSourceFileOfNode(n.declarations[0]);return t&&t.resolvedPath}}function r(e,n){var r=e.getSymbolAtLocation(n);return r&&t(r)}function a(n,t,r,a){return e.toPath(n.getProjectReferenceRedirect(t)||t,r,a)}function i(n,t,i){var o;if(t.imports&&t.imports.length>0)for(var s=n.getTypeChecker(),l=0,c=t.imports;l<c.length;l++){var u=r(s,c[l]);u&&g(u)}var d=e.getDirectoryPath(t.path);if(t.referencedFiles&&t.referencedFiles.length>0)for(var m=0,p=t.referencedFiles;m<p.length;m++){var f=p[m];g(a(n,f.fileName,d,i))}return t.resolvedTypeReferenceDirectiveNames&&t.resolvedTypeReferenceDirectiveNames.forEach(function(e){if(e){var t=e.resolvedFileName;g(a(n,t,d,i))}}),o;function g(n){o||(o=e.createMap()),o.set(n,!0)}}function o(e,n){return n&&!n.referencedMap==!e}function s(e,n){n.forEach(function(n,t){e.fileInfos.get(t).signature=n,e.hasCalledUpdateShapeSignature.set(t,!0)})}function l(n,r,a,i,o,s,l){if(e.Debug.assert(!!a),e.Debug.assert(!l||!!n.exportedModulesMap,\"Compute visible to outside map only if visibleToOutsideReferencedMap present in the state\"),n.hasCalledUpdateShapeSignature.has(a.path)||i.has(a.path))return!1;var c=n.fileInfos.get(a.path);if(!c)return e.Debug.fail();var u,d=c.signature;if(a.isDeclarationFile)u=a.version;else{var m=e.getFileEmitOutput(r,a,!0,o);m.outputFiles&&m.outputFiles.length>0?(u=s(m.outputFiles[0].text),l&&u!==d&&function(n,r,a){if(!r)return void a.set(n.path,!1);var i;r.forEach(function(n){var r;(r=t(n))&&(i||(i=e.createMap()),i.set(r,!0))}),a.set(n.path,i||!1)}(a,m.exportedModulesFromDeclarationEmit,l)):u=d}return i.set(a.path,u),!d||u!==d}function c(n,t){if(!n.allFileNames){var r=t.getSourceFiles();n.allFileNames=r===e.emptyArray?e.emptyArray:r.map(function(e){return e.fileName})}return n.allFileNames}function u(n,t){return e.arrayFrom(e.mapDefinedIterator(n.referencedMap.entries(),function(e){var n=e[0];return e[1].has(t)?n:void 0}))}function d(n){for(var t=0,r=n.statements;t<r.length;t++){var a=r[t];if(!e.isModuleWithStringLiteralName(a))return!1}return!0}function m(n,t,r){if(n.allFilesExcludingDefaultLibraryFile)return n.allFilesExcludingDefaultLibraryFile;var a;l(r);for(var i=0,o=t.getSourceFiles();i<o.length;i++){var s=o[i];s!==r&&l(s)}return n.allFilesExcludingDefaultLibraryFile=a||e.emptyArray,n.allFilesExcludingDefaultLibraryFile;function l(e){t.isSourceFileDefaultLibrary(e)||(a||(a=[])).push(e)}}n.canReuseOldState=o,n.create=function(n,t,r){for(var a=e.createMap(),s=n.getCompilerOptions().module!==e.ModuleKind.None?e.createMap():void 0,l=s?e.createMap():void 0,c=e.createMap(),u=o(s,r),d=0,m=n.getSourceFiles();d<m.length;d++){var p=m[d],f=p.version,g=u?r.fileInfos.get(p.path):void 0;if(s){var _=i(n,p,t);if(_&&s.set(p.path,_),u){var v=r.exportedModulesMap.get(p.path);v&&l.set(p.path,v)}}a.set(p.path,{version:f,signature:g&&g.signature})}return{fileInfos:a,referencedMap:s,exportedModulesMap:l,hasCalledUpdateShapeSignature:c,allFilesExcludingDefaultLibraryFile:void 0,allFileNames:void 0}},n.getFilesAffectedBy=function(n,t,r,a,i,o,c){var p=o||e.createMap(),f=t.getSourceFileByPath(r);if(!f)return e.emptyArray;if(!l(n,t,f,p,a,i,c))return[f];var g=(n.referencedMap?function(n,t,r,a,i,o,s){if(!e.isExternalModule(r)&&!d(r))return m(n,t,r);var c=t.getCompilerOptions();if(c&&(c.isolatedModules||c.out||c.outFile))return[r];var p=e.createMap();p.set(r.path,r);for(var f=u(n,r.path);f.length>0;){var g=f.pop();if(!p.has(g)){var _=t.getSourceFileByPath(g);p.set(g,_),_&&l(n,t,_,a,i,o,s)&&f.push.apply(f,u(n,g))}}return e.arrayFrom(e.mapDefinedIterator(p.values(),function(e){return e}))}:function(e,n,t){var r=n.getCompilerOptions();return r&&(r.out||r.outFile)?[t]:m(e,n,t)})(n,t,f,p,a,i,c);return o||s(n,p),g},n.updateSignaturesFromCache=s,n.updateExportedFilesMapFromCache=function(n,t){t&&(e.Debug.assert(!!n.exportedModulesMap),t.forEach(function(e,t){e?n.exportedModulesMap.set(t,e):n.exportedModulesMap.delete(t)}))},n.getAllDependencies=function(n,t,r){var a,i=t.getCompilerOptions();if(i.outFile||i.out)return c(n,t);if(!n.referencedMap||!e.isExternalModule(r)&&!d(r))return c(n,t);for(var o=e.createMap(),s=[r.path];s.length;){var l=s.pop();if(!o.has(l)){o.set(l,!0);var u=n.referencedMap.get(l);if(u)for(var m=u.keys(),p=m.next(),f=p.value,g=p.done;!g;f=(a=m.next()).value,g=a.done,a)s.push(f)}}return e.arrayFrom(e.mapDefinedIterator(o.keys(),function(e){var n=t.getSourceFileByPath(e);return n?n.fileName:e}))}}(e.BuilderState||(e.BuilderState={}))}(Tn||(Tn={})),function(e){function n(n,t,r){var a=e.BuilderState.create(n,t,r);a.program=n;var i=n.getCompilerOptions();i.outFile||i.out||(a.semanticDiagnosticsPerFile=e.createMap()),a.changedFilesSet=e.createMap();var o=e.BuilderState.canReuseOldState(a.referencedMap,r),s=o&&r.semanticDiagnosticsPerFile&&!!a.semanticDiagnosticsPerFile&&!e.compilerOptionsAffectSemanticDiagnostics(i,r.program.getCompilerOptions());o&&(r.currentChangedFilePath||e.Debug.assert(!(r.affectedFiles||r.currentAffectedFilesSignatures&&r.currentAffectedFilesSignatures.size),\"Cannot reuse if only few affected files of currentChangedFile were iterated\"),s&&e.Debug.assert(!e.forEachKey(r.changedFilesSet,function(e){return r.semanticDiagnosticsPerFile.has(e)}),\"Semantic diagnostics shouldnt be available for changed files\"),e.copyEntries(r.changedFilesSet,a.changedFilesSet));var l=a.referencedMap,c=o?r.referencedMap:void 0;return a.fileInfos.forEach(function(n,t){var i,u,d,m;if(!o||!(i=r.fileInfos.get(t))||i.version!==n.version||(d=u=l&&l.get(t),m=c&&c.get(t),d!==m&&(void 0===d||void 0===m||d.size!==m.size||e.forEachKey(d,function(e){return!m.has(e)})))||u&&e.forEachKey(u,function(e){return!a.fileInfos.has(e)&&r.fileInfos.has(e)}))a.changedFilesSet.set(t,!0);else if(s){var p=r.semanticDiagnosticsPerFile.get(t);p&&(a.semanticDiagnosticsPerFile.set(t,p),a.semanticDiagnosticsFromOldState||(a.semanticDiagnosticsFromOldState=e.createMap()),a.semanticDiagnosticsFromOldState.set(t,!0))}}),a}function t(n,t){e.Debug.assert(!t||!n.affectedFiles||n.affectedFiles[n.affectedFilesIndex-1]!==t||!n.semanticDiagnosticsPerFile.has(t.path))}function r(n,t,r){for(;;){var i=n.affectedFiles;if(i){for(var o=n.seenAffectedFiles,s=n.affectedFilesIndex;s<i.length;){var l=i[s];if(!o.has(l.path))return n.affectedFilesIndex=s,a(n,l),l;o.set(l.path,!0),s++}n.changedFilesSet.delete(n.currentChangedFilePath),n.currentChangedFilePath=void 0,e.BuilderState.updateSignaturesFromCache(n,n.currentAffectedFilesSignatures),n.currentAffectedFilesSignatures.clear(),e.BuilderState.updateExportedFilesMapFromCache(n,n.currentAffectedFilesExportedModulesMap),n.affectedFiles=void 0}var c=n.changedFilesSet.keys().next();if(c.done)return;var u=n.program.getCompilerOptions();if(u.outFile||u.out)return e.Debug.assert(!n.semanticDiagnosticsPerFile),n.program;n.currentAffectedFilesSignatures=n.currentAffectedFilesSignatures||e.createMap(),n.exportedModulesMap&&(n.currentAffectedFilesExportedModulesMap=n.currentAffectedFilesExportedModulesMap||e.createMap()),n.affectedFiles=e.BuilderState.getFilesAffectedBy(n,n.program,c.value,t,r,n.currentAffectedFilesSignatures,n.currentAffectedFilesExportedModulesMap),n.currentChangedFilePath=c.value,n.affectedFilesIndex=0,n.seenAffectedFiles=n.seenAffectedFiles||e.createMap()}}function a(n,t){o(n,t.path)||n.exportedModulesMap&&1!==n.affectedFiles.length&&n.changedFilesSet.has(t.path)&&(e.Debug.assert(!!n.currentAffectedFilesExportedModulesMap),e.forEachEntry(n.currentAffectedFilesExportedModulesMap,function(e,r){return e&&e.has(t.path)&&i(n,r)})||e.forEachEntry(n.exportedModulesMap,function(e,r){return!n.currentAffectedFilesExportedModulesMap.has(r)&&e.has(t.path)&&i(n,r)}))}function i(n,t){return e.forEachEntry(n.referencedMap,function(e,r){return e.has(t)&&o(n,r)})}function o(e,n){return!!e.semanticDiagnosticsFromOldState&&(e.semanticDiagnosticsFromOldState.delete(n),e.semanticDiagnosticsPerFile.delete(n),!e.semanticDiagnosticsFromOldState.size)}function s(e,n){n===e.program?e.changedFilesSet.clear():(e.seenAffectedFiles.set(n.path,!0),e.affectedFilesIndex++)}function l(e,n,t){return s(e,t),{result:n,affected:t}}function c(e,n,t){var r=n.path,a=e.semanticDiagnosticsPerFile.get(r);if(a)return a;var i=e.program.getSemanticDiagnostics(n,t);return e.semanticDiagnosticsPerFile.set(r,i),i}var u;!function(e){e[e.SemanticDiagnosticsBuilderProgram=0]=\"SemanticDiagnosticsBuilderProgram\",e[e.EmitAndSemanticDiagnosticsBuilderProgram=1]=\"EmitAndSemanticDiagnosticsBuilderProgram\"}(u=e.BuilderProgramKind||(e.BuilderProgramKind={})),e.getBuilderCreationParameters=function(n,t,r,a,i,o){var s,l,c;return void 0===n?(e.Debug.assert(void 0===t),s=r,c=a,e.Debug.assert(!!c),l=c.getProgram()):e.isArray(n)?(c=a,l=e.createProgram({rootNames:n,options:t,host:r,oldProgram:c&&c.getProgram(),configFileParsingDiagnostics:i,projectReferences:o}),s=r):(l=n,s=t,c=r,i=a),{host:s,newProgram:l,oldProgram:c,configFileParsingDiagnostics:i||e.emptyArray}},e.createBuilderProgram=function(a,i){var o=i.newProgram,d=i.host,m=i.oldProgram,p=i.configFileParsingDiagnostics,f=m&&m.getState();if(f&&o===f.program&&p===o.getConfigFileParsingDiagnostics())return o=void 0,f=void 0,m;var g=e.createGetCanonicalFileName(d.useCaseSensitiveFileNames()),_=d.createHash||e.identity,v=n(o,g,f);o=void 0,m=void 0,f=void 0;var y={getState:function(){return v},getProgram:function(){return v.program},getCompilerOptions:function(){return v.program.getCompilerOptions()},getSourceFile:function(e){return v.program.getSourceFile(e)},getSourceFiles:function(){return v.program.getSourceFiles()},getOptionsDiagnostics:function(e){return v.program.getOptionsDiagnostics(e)},getGlobalDiagnostics:function(e){return v.program.getGlobalDiagnostics(e)},getConfigFileParsingDiagnostics:function(){return p||v.program.getConfigFileParsingDiagnostics()},getSyntacticDiagnostics:function(e,n){return v.program.getSyntacticDiagnostics(e,n)},getSemanticDiagnostics:function(n,i){t(v,n);var o,l=v.program.getCompilerOptions();if(l.outFile||l.out)return e.Debug.assert(!v.semanticDiagnosticsPerFile),v.program.getSemanticDiagnostics(n,i);if(n)return c(v,n,i);if(a===u.SemanticDiagnosticsBuilderProgram)for(var d=void 0;d=r(v,i,_);)s(v,d);for(var m=0,p=v.program.getSourceFiles();m<p.length;m++){var f=p[m];o=e.addRange(o,c(v,f,i))}return o||e.emptyArray},emit:function(n,r,i,o,s){if(a===u.EmitAndSemanticDiagnosticsBuilderProgram&&(t(v,n),!n)){for(var l=[],c=!1,m=void 0,p=[],f=void 0;f=h(r,i,o,s);)c=c||f.result.emitSkipped,m=e.addRange(m,f.result.diagnostics),p=e.addRange(p,f.result.emittedFiles),l=e.addRange(l,f.result.sourceMaps);return{emitSkipped:c,diagnostics:m||e.emptyArray,emittedFiles:p,sourceMaps:l}}return v.program.emit(n,r||d.writeFile,i,o,s)},getAllDependencies:function(n){return e.BuilderState.getAllDependencies(v,v.program,n)},getCurrentDirectory:function(){return v.program.getCurrentDirectory()}};return a===u.SemanticDiagnosticsBuilderProgram?y.getSemanticDiagnosticsOfNextAffectedFile=function(e,n){for(;;){var t=r(v,e,_);if(!t)return;if(t===v.program)return l(v,v.program.getSemanticDiagnostics(void 0,e),t);if(!n||!n(t))return l(v,c(v,t,e),t);s(v,t)}}:a===u.EmitAndSemanticDiagnosticsBuilderProgram?y.emitNextAffectedFile=h:e.notImplemented(),y;function h(e,n,t,a){var i=r(v,n,_);if(i)return l(v,v.program.emit(i===v.program?void 0:i,e||d.writeFile,n,t,a),i)}}}(Tn||(Tn={})),function(e){e.createSemanticDiagnosticsBuilderProgram=function(n,t,r,a,i,o){return e.createBuilderProgram(e.BuilderProgramKind.SemanticDiagnosticsBuilderProgram,e.getBuilderCreationParameters(n,t,r,a,i,o))},e.createEmitAndSemanticDiagnosticsBuilderProgram=function(n,t,r,a,i,o){return e.createBuilderProgram(e.BuilderProgramKind.EmitAndSemanticDiagnosticsBuilderProgram,e.getBuilderCreationParameters(n,t,r,a,i,o))},e.createAbstractBuilder=function(n,t,r,a,i,o){var s=e.getBuilderCreationParameters(n,t,r,a,i,o).newProgram;return{getProgram:function(){return s},getState:e.notImplemented,getCompilerOptions:e.notImplemented,getSourceFile:e.notImplemented,getSourceFiles:e.notImplemented,getOptionsDiagnostics:e.notImplemented,getGlobalDiagnostics:e.notImplemented,getConfigFileParsingDiagnostics:e.notImplemented,getSyntacticDiagnostics:e.notImplemented,getSemanticDiagnostics:e.notImplemented,emit:e.notImplemented,getAllDependencies:e.notImplemented,getCurrentDirectory:e.notImplemented}}}(Tn||(Tn={})),function(e){e.maxNumberOfFilesToIterateForInvalidation=256,e.createResolutionCache=function(n,t,r){var a,i,o,s=!1,l=e.createMultiMap(),c=e.memoize(function(){return n.getCurrentDirectory()}),u=n.getCachedDirectoryStructureHost(),d=e.createMap(),m=e.createMap(),p=e.createMap(),f=e.createModuleResolutionCacheWithMaps(m,p,c(),n.getCanonicalFileName),g=e.createMap(),_=e.createMap(),v=[\".ts\",\".tsx\",\".js\",\".jsx\",\".json\"],y=e.createMap(),h=e.createMap(),b=t&&e.removeTrailingDirectorySeparator(e.getNormalizedAbsolutePath(t,c())),E=b&&n.toPath(b),T=e.createMap();return{startRecordingFilesWithChangedResolutions:function(){a=[]},finishRecordingFilesWithChangedResolutions:function(){var e=a;return a=void 0,e},startCachingPerDirectoryResolution:C,finishCachingPerDirectoryResolution:function(){s=!1,o=void 0,C(),h.forEach(function(e,n){0===e.refCount&&(h.delete(n),e.watcher.close())})},resolveModuleNames:function(e,n,t){return k(e,n,d,m,D,S,t,r)},getResolvedModuleWithFailedLookupLocationsFromCache:function(e,t){var r=d.get(n.toPath(t));return r&&r.get(e)},resolveTypeReferenceDirectives:function(n,t){return k(n,t,g,_,e.resolveTypeReferenceDirective,L,void 0,!1)},removeResolutionsOfFile:W,invalidateResolutionOfFile:function(e){W(e),z(function(t,r){var a=r(t);return!!a&&n.toPath(a.resolvedFileName)===e})},setFilesWithInvalidatedNonRelativeUnresolvedImports:function(n){e.Debug.assert(o===n||void 0===o),o=n},createHasInvalidatedResolution:function(n){if(s||n)return i=void 0,e.returnTrue;var t=i;return i=void 0,function(e){return!!t&&t.has(e)||A(e)}},updateTypeRootsWatch:function(){var t=n.getCompilationSettings();if(t.types)X();else{var r=e.getEffectiveTypeRoots(t,{directoryExists:Q,getCurrentDirectory:c});r?e.mutateMap(T,e.arrayToMap(r,function(e){return n.toPath(e)}),{createNewValue:Y,onDeleteValue:e.closeFileWatcher}):X()}},closeTypeRootsWatch:X,clear:function(){e.clearMap(h,e.closeFileWatcherOf),y.clear(),l.clear(),X(),d.clear(),g.clear(),s=!1,C()}};function S(e){return e.resolvedModule}function L(e){return e.resolvedTypeReferenceDirective}function x(n,t){return!(void 0===n||t.length<=n.length)&&e.startsWith(t,n)&&t[n.length]===e.directorySeparator}function A(e){if(!o)return!1;var n=o.get(e);return!!n&&!!n.length}function C(){m.clear(),p.clear(),_.clear(),l.forEach(K),l.clear()}function D(t,r,a,i){var o=e.resolveModuleName(t,r,a,i,f);if(!n.getGlobalCache)return o;var s=n.getGlobalCache();if(!(void 0===s||e.isExternalModuleNameRelative(t)||o.resolvedModule&&e.extensionIsTS(o.resolvedModule.extension))){var l=e.loadModuleFromGlobalCache(t,n.projectName,a,i,s),c=l.resolvedModule,u=l.failedLookupLocations;if(c)return{resolvedModule:c,failedLookupLocations:e.addRange(o.failedLookupLocations,u)}}return o}function k(t,r,i,o,l,c,u,d){var m=n.toPath(r),p=i.get(m)||i.set(m,e.createMap()).get(m),f=e.getDirectoryPath(m),g=o.get(f);g||(g=e.createMap(),o.set(f,g));for(var _=[],v=n.getCompilationSettings(),y=d&&A(m),h=e.createMap(),b=0,E=t;b<E.length;b++){var T=E[b],S=p.get(T);if(!h.has(T)&&s||!S||S.isInvalidated||y&&!e.isExternalModuleNameRelative(T)&&!c(S)){var L=S,x=g.get(T);x?S=x:(S=l(T,r,v,n),g.set(T,S)),p.set(T,S),F(T,S),L&&H(L),d&&a&&!C(L,S)&&(a.push(m),d=!1)}e.Debug.assert(void 0!==S&&!S.isInvalidated),h.set(T,!0),_.push(c(S))}return p.forEach(function(n,t){h.has(t)||e.contains(u,t)||(H(n),p.delete(t))}),_;function C(e,n){if(e===n)return!0;if(!e||!n)return!1;var t=c(e),r=c(n);return t===r||!(!t||!r)&&t.resolvedFileName===r.resolvedFileName}}function M(n){return e.endsWith(n,\"/node_modules\")}function O(n){return e.endsWith(n,\"/node_modules/@types\")}function I(n){var t=e.getRootLength(n);if(n.length===t)return!1;var r=n.indexOf(e.directorySeparator,t);if(-1===r)return!1;if(47!==n.charCodeAt(0)&&-1===n.substr(t,r).search(/users/i))return!0;for(var a=r+1,i=2;i>0;i--)if(0===(a=n.indexOf(e.directorySeparator,a)+1))return!1;return!0}function R(e,n){return I(n)||(e.ignore=!0),e}function N(n,t){if(x(E,t)){n=e.isRootedDiskPath(n)?n:e.getNormalizedAbsolutePath(n,c()),e.Debug.assert(n.length===t.length,\"FailedLookup: \"+n+\" failedLookupLocationPath: \"+t);var r=t.indexOf(e.directorySeparator,E.length+1);return-1!==r?{dir:n.substr(0,r),dirPath:t.substr(0,r)}:{dir:b,dirPath:E,nonRecursive:!1}}return w(e.getDirectoryPath(e.getNormalizedAbsolutePath(n,c())),e.getDirectoryPath(t))}function w(n,t){for(;e.stringContains(t,e.nodeModulesPathPart);)n=e.getDirectoryPath(n),t=e.getDirectoryPath(t);if(M(t))return R({dir:n,dirPath:t},e.getDirectoryPath(t));var r,a,i=!0;if(void 0!==E)for(;!x(t,E);){var o=e.getDirectoryPath(t);if(o===t)break;i=!1,r=t,a=n,t=o,n=e.getDirectoryPath(n)}return R({dir:a||n,dirPath:r||t,nonRecursive:i},t)}function P(n){return e.fileExtensionIsOneOf(n,v)}function F(n,t){t.failedLookupLocations&&t.failedLookupLocations.length&&(t.refCount?t.refCount++:(t.refCount=1,e.isExternalModuleNameRelative(n)?G(t):l.add(n,t)))}function G(t){e.Debug.assert(!!t.refCount);for(var r=!1,a=0,i=t.failedLookupLocations;a<i.length;a++){var o=i[a],s=n.toPath(o),l=N(o,s),c=l.dir,u=l.dirPath,d=l.nonRecursive;if(!l.ignore){if(!P(s)){var m=y.get(s)||0;y.set(s,m+1)}u===E?(e.Debug.assert(!d),r=!0):B(c,u,d)}}r&&B(b,E,!0)}function V(e){e.refCount=void 0}function K(e,t){var r=n.getCurrentProgram(),a=r&&r.getTypeChecker().tryFindAmbientModuleWithoutAugmentations(t)?V:G;e.forEach(a)}function B(t,r,a){var i=h.get(r);i?(e.Debug.assert(!!a==!!i.nonRecursive),i.refCount++):h.set(r,{watcher:function(e,t,r){return n.watchDirectoryOfFailedLookupLocation(e,function(e){var r=n.toPath(e);u&&u.addOrDeleteFileOrDirectory(e,r),!s&&J(r,t===r)&&n.onInvalidatedResolution()},r?0:1)}(t,r,a),refCount:1,nonRecursive:a})}function H(t){if(t.refCount&&(t.refCount--,!t.refCount)){for(var r=!1,a=0,i=t.failedLookupLocations;a<i.length;a++){var o=i[a],s=n.toPath(o),l=N(o,s),c=l.dirPath;if(!l.ignore){var u=y.get(s);u&&(1===u?y.delete(s):(e.Debug.assert(u>1),y.set(s,u-1))),c===E?r=!0:U(c)}}r&&U(E)}}function U(e){h.get(e).refCount--}function j(e,n){var t=e.get(n);t&&(t.forEach(H),e.delete(n))}function W(e){j(d,e),j(g,e)}function q(n,t,r){var a=e.createMap();n.forEach(function(n,o){var s=e.getDirectoryPath(o),l=a.get(s);l||(l=e.createMap(),a.set(s,l)),n.forEach(function(n,a){l.has(a)||(l.set(a,!0),!n.isInvalidated&&t(n,r)&&(n.isInvalidated=!0,(i||(i=e.createMap())).set(o,!0)))})})}function z(t){var r;r=n.maxNumberOfFilesToIterateForInvalidation||e.maxNumberOfFilesToIterateForInvalidation,d.size>r||g.size>r?s=!0:(q(d,t,S),q(g,t,L))}function J(t,r){var a;if(r)a=function(e){return x(t,n.toPath(e))};else{var o=e.getDirectoryPath(t);if(O(t)||M(t)||O(o)||M(o))a=function(r){return n.toPath(r)===t||e.startsWith(n.toPath(r),t)};else{if(!P(t)&&!y.has(t))return!1;if(e.isEmittedFileOfProgram(n.getCurrentProgram(),t))return!1;a=function(e){return n.toPath(e)===t}}}var l=i&&i.size;return z(function(n){return e.some(n.failedLookupLocations,a)}),s||i&&i.size!==l}function X(){e.clearMap(T,e.closeFileWatcher)}function Y(e,t){return n.watchTypeRootsDirectory(t,function(r){var a=n.toPath(r);u&&u.addOrDeleteFileOrDirectory(r,a),n.onChangedAutomaticTypeDirectiveNames();var i=function(e,n){if(!s){if(x(E,n))return E;var t=w(e,n),r=t.dirPath;return!t.ignore&&h.has(r)?r:void 0}}(t,e);i&&J(a,i===a)&&n.onInvalidatedResolution()},1)}function Q(t){var r=e.getDirectoryPath(e.getDirectoryPath(t)),a=n.toPath(r);return a===E||I(a)}}}(Tn||(Tn={})),function(e){!function(n){var t,r;function a(n,t,r){var a=n.importModuleSpecifierPreference,i=n.importModuleSpecifierEnding;return{relativePreference:\"relative\"===a?0:\"non-relative\"===a?1:2,ending:function(){switch(i){case\"minimal\":return 0;case\"index\":return 1;case\"js\":return 2;default:return n=r.imports,e.firstDefined(n,function(n){var t=n.text;return e.pathIsRelative(t)?e.hasJSOrJsonFileExtension(t):void 0})?2:e.getEmitModuleResolutionKind(t)!==e.ModuleResolutionKind.NodeJs?1:0}var n}()}}function i(n,t,r,a,i,l,c){var u=o(t,a),d=m(i,t,r,u.getCanonicalFileName,a,l);return e.firstDefined(d,function(e){return f(e,u,a,n)})||s(r,u,n,c)}function o(n,t){return{getCanonicalFileName:e.createGetCanonicalFileName(!t.useCaseSensitiveFileNames||t.useCaseSensitiveFileNames()),sourceDirectory:e.getDirectoryPath(n)}}function s(n,t,r,a){var i=t.getCanonicalFileName,o=t.sourceDirectory,s=a.ending,c=a.relativePreference,u=r.baseUrl,d=r.paths,m=r.rootDirs,f=m&&function(n,t,r,a){var i=g(t,n,a);if(void 0===i)return;var o=g(r,n,a),s=void 0!==o?e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(o,i,a)):i;return e.removeFileExtension(s)}(m,n,o,i)||_(e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(o,n,i)),s,r);if(!u||0===c)return f;var h=v(n,u,i);if(!h)return f;var b=_(h,s,r),E=d&&p(e.removeFileExtension(h),b,d),T=void 0===E?b:E;return 1===c?T:(2!==c&&e.Debug.assertNever(c),y(T)||l(f)<l(T)?f:T)}function l(n){for(var t=0,r=e.startsWith(n,\"./\")?2:0;r<n.length;r++)47===n.charCodeAt(r)&&t++;return t}function c(e,n,t){return t(e)===t(n)}function u(n,t){return\"node_modules\"===t(n)||e.startsWith(n,\"@\")}function d(n,t,r,a){for(var i=e.getPathComponents(e.toPath(n,r,a)),o=e.getPathComponents(e.toPath(t,r,a));!u(i[i.length-2],a)&&!u(o[o.length-2],a)&&c(i[i.length-1],o[o.length-1],a);)i.pop(),o.pop();return[e.getPathFromPathComponents(i),e.getPathFromPathComponents(o)]}function m(n,t,r,a,i,o){var s=o.get(r),l=s?s.concat([r]):[r],c=i.getCurrentDirectory?i.getCurrentDirectory():\"\",u=l.map(function(n){return e.getNormalizedAbsolutePath(n,c)}),m=function(n,t,r){for(var a=e.createMap(),i=0,o=e.mapDefined(n,function(n){return n.resolvedModules&&e.firstDefinedIterator(n.resolvedModules.values(),function(e){return e&&e.originalPath&&e.resolvedFileName!==e.originalPath?[e.resolvedFileName,e.originalPath]:void 0})});i<o.length;i++){var s=o[i],l=d(s[0],s[1],r,t),c=l[0],u=l[1];a.set(u,c)}return a}(n,a,c),p=[],f=!i.useCaseSensitiveFileNames||i.useCaseSensitiveFileNames()?e.compareStringsCaseSensitive:e.compareStringsCaseInsensitive;return m.forEach(function(n,r){if(!e.startsWithDirectory(t,n,a)){var o=u.find(function(e){return 0===f(e.slice(0,n.length+1),n+\"/\")});if(void 0!==o){var s=e.getRelativePathFromDirectory(n,o,a),l=e.resolvePath(r,s);i.fileExists&&!i.fileExists(l)||p.push(l)}}}),p.push.apply(p,u),p}function p(n,t,r){for(var a in r)for(var i=0,o=r[a];i<o.length;i++){var s=o[i],l=e.removeFileExtension(e.normalizePath(s)),c=l.indexOf(\"*\");if(-1!==c){var u=l.substr(0,c),d=l.substr(c+1);if(t.length>=u.length+d.length&&e.startsWith(t,u)&&e.endsWith(t,d)||!d&&t===e.removeTrailingDirectorySeparator(u)){var m=t.substr(u.length,t.length-d.length);return a.replace(\"*\",m)}}else if(l===t||l===n)return a}}function f(n,t,r,a){var i=t.getCanonicalFileName,o=t.sourceDirectory,s=function(n){var t,r=0,a=0,i=0,o=0;!function(e){e[e.BeforeNodeModules=0]=\"BeforeNodeModules\",e[e.NodeModules=1]=\"NodeModules\",e[e.Scope=2]=\"Scope\",e[e.PackageContent=3]=\"PackageContent\"}(t||(t={}));var s=0,l=0,c=0;for(;l>=0;)switch(s=l,l=n.indexOf(\"/\",s+1),c){case 0:n.indexOf(e.nodeModulesPathPart,s)===s&&(r=s,a=l,c=1);break;case 1:case 2:1===c&&\"@\"===n.charAt(s+1)?c=2:(i=l,c=3);break;case 3:c=n.indexOf(e.nodeModulesPathPart,s)===s?1:3}return o=s,c>1?{topLevelNodeModulesIndex:r,topLevelPackageNameIndex:a,packageRootIndex:i,fileNameIndex:o}:void 0}(n);if(s){var l=n.substring(0,s.packageRootIndex),c=e.combinePaths(l,\"package.json\"),u=r.fileExists(c)?JSON.parse(r.readFile(c)):void 0,d=u&&u.typesVersions?e.getPackageJsonTypesVersionsPaths(u.typesVersions):void 0;if(d){var m=n.slice(s.packageRootIndex+1),f=p(e.removeFileExtension(m),_(m,0,a),d.paths);void 0!==f&&(n=e.combinePaths(n.slice(0,s.packageRootIndex),f))}var g=function(n){if(u){var t=u.typings||u.types||u.main;if(t){var a=e.toPath(t,l,i);if(e.removeFileExtension(a)===e.removeFileExtension(i(n)))return l}}var o=e.removeFileExtension(n);if(\"/index\"===i(o.substring(s.fileNameIndex))&&!function(n,t){for(var r=e.getSupportedExtensions({allowJs:!0},[{extension:\"node\",isMixedContent:!1},{extension:\"json\",isMixedContent:!1,scriptKind:6}]),a=0,i=r;a<i.length;a++){var o=i[a],s=t+o;if(n.fileExists(s))return s}}(r,o.substring(0,s.fileNameIndex)))return o.substring(0,s.fileNameIndex);return o}(n);if(e.startsWith(o,i(g.substring(0,s.topLevelNodeModulesIndex)))){var v=g.substring(s.topLevelPackageNameIndex+1),y=e.getPackageNameFromTypesPackageName(v);return e.getEmitModuleResolutionKind(a)!==e.ModuleResolutionKind.NodeJs&&y===v?void 0:y}}}function g(n,t,r){return e.firstDefined(t,function(e){var t=v(n,e,r);return y(t)?void 0:t})}function _(n,t,r){if(e.fileExtensionIs(n,\".json\"))return n;var a=e.removeFileExtension(n);switch(t){case 0:return e.removeSuffix(a,\"/index\");case 1:return a;case 2:return a+function(n,t){var r=e.extensionFromPath(n);switch(r){case\".ts\":case\".d.ts\":return\".js\";case\".tsx\":return 1===t.jsx?\".jsx\":\".js\";case\".js\":case\".jsx\":case\".json\":return r;default:return e.Debug.assertNever(r)}}(n,r);default:return e.Debug.assertNever(t)}}function v(n,t,r){var a=e.getRelativePathToDirectoryOrUrl(t,n,t,r,!1);return e.isRootedDiskPath(a)?void 0:a}function y(n){return e.startsWith(n,\"..\")}!function(e){e[e.Relative=0]=\"Relative\",e[e.NonRelative=1]=\"NonRelative\",e[e.Auto=2]=\"Auto\"}(t||(t={})),function(e){e[e.Minimal=0]=\"Minimal\",e[e.Index=1]=\"Index\",e[e.JsExtension=2]=\"JsExtension\"}(r||(r={})),n.updateModuleSpecifier=function(n,t,r,a,o,s,l){var c=i(n,t,r,a,o,s,function(n,t){return{relativePreference:e.isExternalModuleNameRelative(t)?0:1,ending:e.hasJSOrJsonFileExtension(t)?2:e.getEmitModuleResolutionKind(n)!==e.ModuleResolutionKind.NodeJs||e.endsWith(t,\"index\")?1:0}}(n,l));if(c!==l)return c},n.getModuleSpecifier=function(e,n,t,r,o,s,l,c){return void 0===l&&(l={}),i(e,t,r,o,s,c,a(l,e,n))},n.getModuleSpecifiers=function(n,t,r,i,l,c,u){var d=function(n){var t=e.find(n.declarations,function(n){return e.isNonGlobalAmbientModule(n)&&(!e.isExternalModuleAugmentation(n)||!e.isExternalModuleNameRelative(e.getTextOfIdentifierOrLiteral(n.name)))});if(t)return t.name.text}(n);if(d)return[d];var p=o(r.path,i),g=e.getSourceFileOfNode(n.valueDeclaration||e.getNonAugmentationDeclaration(n)),_=m(l,r.path,g.fileName,p.getCanonicalFileName,i,u),v=a(c,t,r),y=e.mapDefined(_,function(e){return f(e,p,i,t)});return y.length?y:_.map(function(e){return s(e,p,t,v)})}}(e.moduleSpecifiers||(e.moduleSpecifiers={}))}(Tn||(Tn={})),function(e){var n=e.sys?{getCurrentDirectory:function(){return e.sys.getCurrentDirectory()},getNewLine:function(){return e.sys.newLine},getCanonicalFileName:e.createGetCanonicalFileName(e.sys.useCaseSensitiveFileNames)}:void 0;function t(t,r){var a=t===e.sys?n:{getCurrentDirectory:function(){return t.getCurrentDirectory()},getNewLine:function(){return t.newLine},getCanonicalFileName:e.createGetCanonicalFileName(t.useCaseSensitiveFileNames)};if(!r)return function(n){return t.write(e.formatDiagnostic(n,a))};var i=new Array(1);return function(n){i[0]=n,t.write(e.formatDiagnosticsWithColorAndContext(i,a)+a.getNewLine()),i[0]=void 0}}function r(n,t,r){return!(!n.clearScreen||r.preserveWatchOutput||r.extendedDiagnostics||r.diagnostics||!e.contains(e.screenStartingMessageCodes,t.code))&&(n.clearScreen(),!0)}function a(n,t){return t?function(t,a,i){r(n,t,i);var o=\"[\"+e.formatColorAndReset((new Date).toLocaleTimeString(),e.ForegroundColorEscapeSequences.Grey)+\"] \";o+=\"\"+e.flattenDiagnosticMessageText(t.messageText,n.newLine)+(a+a),n.write(o)}:function(t,a,i){var o=\"\";r(n,t,i)||(o+=a),o+=(new Date).toLocaleTimeString()+\" - \",o+=\"\"+e.flattenDiagnosticMessageText(t.messageText,n.newLine)+function(n,t){return e.contains(e.screenStartingMessageCodes,n.code)?t+t:t}(t,a),n.write(o)}}function i(n,t,r,a,i){var o=n.getConfigFileParsingDiagnostics().slice(),s=o.length;e.addRange(o,n.getSyntacticDiagnostics());var l=!1;o.length===s&&(e.addRange(o,n.getOptionsDiagnostics()),e.addRange(o,n.getGlobalDiagnostics()),o.length===s&&(l=!0));var c=n.emit(void 0,i),u=c.emittedFiles,d=c.emitSkipped,m=c.diagnostics;if(e.addRange(o,m),l&&e.addRange(o,n.getSemanticDiagnostics()),e.sortAndDeduplicateDiagnostics(o).forEach(t),r){var p=n.getCurrentDirectory();e.forEach(u,function(n){var t=e.getNormalizedAbsolutePath(n,p);r(\"TSFILE: \"+t)}),n.getCompilerOptions().listFiles&&e.forEach(n.getSourceFiles(),function(e){r(e.fileName)})}return a&&a(o.filter(function(n){return n.category===e.DiagnosticCategory.Error}).length),d&&o.length>0?e.ExitStatus.DiagnosticsPresent_OutputsSkipped:o.length>0?e.ExitStatus.DiagnosticsPresent_OutputsGenerated:e.ExitStatus.Success}e.createDiagnosticReporter=t,e.screenStartingMessageCodes=[e.Diagnostics.Starting_compilation_in_watch_mode.code,e.Diagnostics.File_change_detected_Starting_incremental_compilation.code],e.createWatchStatusReporter=a,e.parseConfigFileWithSystem=function(n,t,r,a){var i=r;i.onUnRecoverableConfigFileDiagnostic=function(n){return c(e.sys,a,n)};var o=e.getParsedCommandLineOfConfigFile(n,t,i);return i.onUnRecoverableConfigFileDiagnostic=void 0,o},e.emitFilesAndReportErrors=i;var o={close:e.noop};function s(n,t){return void 0===n&&(n=e.sys),{onWatchStatusChange:t||a(n),watchFile:n.watchFile?function(e,t,r){return n.watchFile(e,t,r)}:function(){return o},watchDirectory:n.watchDirectory?function(e,t,r){return n.watchDirectory(e,t,r)}:function(){return o},setTimeout:n.setTimeout?function(e,t){for(var r,a=[],i=2;i<arguments.length;i++)a[i-2]=arguments[i];return(r=n.setTimeout).call.apply(r,[n,e,t].concat(a))}:e.noop,clearTimeout:n.clearTimeout?function(e){return n.clearTimeout(e)}:e.noop}}function l(n,t,r,a){void 0===n&&(n=e.sys),t||(t=e.createEmitAndSemanticDiagnosticsBuilderProgram);var o=function(e){return n.write(e+n.newLine)},l=s(n,a),c=l.onWatchStatusChange,u=l.watchFile,d=l.watchDirectory,m=l.setTimeout,p=l.clearTimeout;return{useCaseSensitiveFileNames:function(){return n.useCaseSensitiveFileNames},getNewLine:function(){return n.newLine},getCurrentDirectory:function(){return n.getCurrentDirectory()},getDefaultLibLocation:f,getDefaultLibFileName:function(n){return e.combinePaths(f(),e.getDefaultLibFileName(n))},fileExists:function(e){return n.fileExists(e)},readFile:function(e,t){return n.readFile(e,t)},directoryExists:function(e){return n.directoryExists(e)},getDirectories:function(e){return n.getDirectories(e)},readDirectory:function(e,t,r,a,i){return n.readDirectory(e,t,r,a,i)},realpath:n.realpath&&function(e){return n.realpath(e)},getEnvironmentVariable:n.getEnvironmentVariable&&function(e){return n.getEnvironmentVariable(e)},watchFile:u,watchDirectory:d,setTimeout:m,clearTimeout:p,trace:function(e){return n.write(e)},onWatchStatusChange:c,createDirectory:function(e){return n.createDirectory(e)},writeFile:function(e,t,r){return n.writeFile(e,t,r)},onCachedDirectoryStructureHostCreate:function(e){return e||n},createHash:n.createHash&&function(e){return n.createHash(e)},createProgram:t,afterProgramCreate:function(t){var a=t.getCompilerOptions(),s=e.getNewLineCharacter(a,function(){return n.newLine});i(t,r,o,function(n){c(1===n?e.createCompilerDiagnostic(e.Diagnostics.Found_1_error_Watching_for_file_changes,n):e.createCompilerDiagnostic(e.Diagnostics.Found_0_errors_Watching_for_file_changes,n,n),s,a)})}};function f(){return e.getDirectoryPath(e.normalizePath(n.getExecutingFilePath()))}}function c(n,t,r){t(r),n.exit(e.ExitStatus.DiagnosticsPresent_OutputsSkipped)}e.createWatchHost=s,e.createWatchCompilerHostOfConfigFile=function(e,n,r,a,i,o){var s=i||t(r),u=l(r,a,s,o);return u.onUnRecoverableConfigFileDiagnostic=function(e){return c(r,s,e)},u.configFileName=e,u.optionsToExtend=n,u},e.createWatchCompilerHostOfFilesAndCompilerOptions=function(e,n,r,a,i,o,s){var c=l(r,a,i||t(r),o);return c.rootFiles=e,c.options=n,c.projectReferences=s,c}}(Tn||(Tn={})),function(e){e.createWatchCompilerHost=function(n,t,r,a,i,o,s){return e.isArray(n)?e.createWatchCompilerHostOfFilesAndCompilerOptions(n,t,r,a,i,o,s):e.createWatchCompilerHostOfConfigFile(n,t,r,a,i,o)};var n=1;e.createWatchProgram=function(t){var r,a,i,o,s,l,c,u,d=e.createMap(),m=!1,p=!1,f=t.useCaseSensitiveFileNames(),g=t.getCurrentDirectory(),_=function(){return g},v=function(e,n){return t.readFile(e,n)},y=t.configFileName,h=t.optionsToExtend,b=void 0===h?{}:h,E=t.createProgram,T=t.rootFiles,S=t.options,L=t.projectReferences,x=!1,A=void 0===y?void 0:e.createCachedDirectoryStructureHost(t,g,f);A&&t.onCachedDirectoryStructureHostCreate&&t.onCachedDirectoryStructureHostCreate(A);var C=A||t,D={useCaseSensitiveFileNames:f,readDirectory:function(e,n,t,r,a){return C.readDirectory(e,n,t,r,a)},fileExists:function(e){return t.fileExists(e)},readFile:v,getCurrentDirectory:_,onUnRecoverableConfigFileDiagnostic:t.onUnRecoverableConfigFileDiagnostic},k=j();y&&t.configFileParsingResult&&(te(t.configFileParsingResult),k=j()),Z(e.Diagnostics.Starting_compilation_in_watch_mode),y&&!t.configFileParsingResult&&(k=e.getNewLineCharacter(b,function(){return t.getNewLine()}),e.Debug.assert(!T),ne(),k=j());var M=t.trace&&function(e){t.trace(e+k)},O=M?S.extendedDiagnostics?e.WatchLogLevel.Verbose:S.diagnostics?e.WatchLogLevel.TriggerOnly:e.WatchLogLevel.None:e.WatchLogLevel.None,I=O!==e.WatchLogLevel.None?M:e.noop,R=e.getWatchFactory(O,I),N=R.watchFile,w=R.watchFilePath,P=R.watchDirectory,F=e.createGetCanonicalFileName(f);I(\"Current directory: \"+g+\" CaseSensitiveFileNames: \"+f),y&&N(t,y,function(){e.Debug.assert(!!y),a=e.ConfigFileProgramReloadLevel.Full,$()},e.PollingInterval.High,\"Config file\");var G={getSourceFile:function(e,n,t,r){return X(e,W(e),n,t,r)},getSourceFileByPath:X,getDefaultLibLocation:t.getDefaultLibLocation&&function(){return t.getDefaultLibLocation()},getDefaultLibFileName:function(e){return t.getDefaultLibFileName(e)},writeFile:function(n,r,a,i){try{e.performance.mark(\"beforeIOWrite\"),function n(r){if(r.length>e.getRootLength(r)&&!t.directoryExists(r)){var a=e.getDirectoryPath(r);n(a),t.createDirectory(r)}}(e.getDirectoryPath(e.normalizePath(n))),t.writeFile(n,r,a),e.performance.mark(\"afterIOWrite\"),e.performance.measure(\"I/O Write\",\"beforeIOWrite\",\"afterIOWrite\")}catch(e){i&&i(e.message)}},getCurrentDirectory:_,useCaseSensitiveFileNames:function(){return f},getCanonicalFileName:F,getNewLine:function(){return k},fileExists:J,readFile:v,trace:M,directoryExists:C.directoryExists&&function(e){return C.directoryExists(e)},getDirectories:C.getDirectories&&function(e){return C.getDirectories(e)},realpath:t.realpath&&function(e){return t.realpath(e)},getEnvironmentVariable:t.getEnvironmentVariable?function(e){return t.getEnvironmentVariable(e)}:function(){return\"\"},onReleaseOldSourceFile:function(e,n){var t=d.get(e.path);t&&(q(t)?(l||(l=[])).push(e.path):t.sourceFile===e&&(t.fileWatcher&&t.fileWatcher.close(),d.delete(e.path),V.removeResolutionsOfFile(e.path)))},createHash:t.createHash&&function(e){return t.createHash(e)},toPath:W,getCompilationSettings:function(){return S},watchDirectoryOfFailedLookupLocation:function(e,n,r){return P(t,e,n,r,\"Failed Lookup Locations\")},watchTypeRootsDirectory:function(e,n,r){return P(t,e,n,r,\"Type roots\")},getCachedDirectoryStructureHost:function(){return A},onInvalidatedResolution:$,onChangedAutomaticTypeDirectiveNames:function(){p=!0,$()},maxNumberOfFilesToIterateForInvalidation:t.maxNumberOfFilesToIterateForInvalidation,getCurrentProgram:H,writeLog:I,readDirectory:function(e,n,t,r,a){return C.readDirectory(e,n,t,r,a)}},V=e.createResolutionCache(G,y?e.getDirectoryPath(e.getNormalizedAbsolutePath(y,g)):g,!1);G.resolveModuleNames=t.resolveModuleNames?function(e,n,r){return t.resolveModuleNames(e,n,r)}:function(e,n,t){return V.resolveModuleNames(e,n,t)},G.resolveTypeReferenceDirectives=t.resolveTypeReferenceDirectives?function(e,n){return t.resolveTypeReferenceDirectives(e,n)}:function(e,n){return V.resolveTypeReferenceDirectives(e,n)};var K=!!t.resolveModuleNames||!!t.resolveTypeReferenceDirectives;return U(),se(),y?{getCurrentProgram:B,getProgram:U}:{getCurrentProgram:B,getProgram:U,updateRootFileNames:function(n){e.Debug.assert(!y,\"Cannot update root file names with config file watch mode\"),T=n,$()}};function B(){return r}function H(){return r&&r.getProgram()}function U(){I(\"Synchronizing program\");var n=H();m&&(k=j(),n&&e.changesAffectModuleResolution(n.getCompilerOptions(),S)&&V.clear());var a=V.createHasInvalidatedResolution(K);return e.isProgramUptoDate(H(),T,S,Q,J,a,p,L)?x&&(r=E(void 0,void 0,G,r,u,L),x=!1):function(n,t){O!==e.WatchLogLevel.None&&(I(\"CreatingProgramWith::\"),I(\"  roots: \"+JSON.stringify(T)),I(\"  options: \"+JSON.stringify(S)));var a=m||!n;if(m=!1,x=!1,V.startCachingPerDirectoryResolution(),G.hasInvalidatedResolution=t,G.hasChangedAutomaticTypeDirectiveNames=p,r=E(T,S,G,r,u,L),V.finishCachingPerDirectoryResolution(),e.updateMissingFilePathsWatch(r.getProgram(),i||(i=e.createMap()),ie),a&&V.updateTypeRootsWatch(),l){for(var o=0,s=l;o<s.length;o++){var c=s[o];i.has(c)||d.delete(c)}l=void 0}}(n,a),t.afterProgramCreate&&t.afterProgramCreate(r),r}function j(){return e.getNewLineCharacter(S||b,function(){return t.getNewLine()})}function W(n){return e.toPath(n,g,F)}function q(e){return\"number\"==typeof e}function z(e){return!!e.sourceFile}function J(e){var n=W(e);return!!q(d.get(n))||C.fileExists(e)}function X(r,a,i,o,s){var l=d.get(a);if(!q(l)){if(!l||s||!z(l)||l.version.toString()!==l.sourceFile.version){var c=function(){var n;try{e.performance.mark(\"beforeIORead\"),n=t.readFile(r,S.charset),e.performance.mark(\"afterIORead\"),e.performance.measure(\"I/O Read\",\"beforeIORead\",\"afterIORead\")}catch(e){o&&o(e.message)}return void 0!==n?e.createSourceFile(r,n,i):void 0}();if(l)s&&l.version++,c?(l.sourceFile=c,c.version=l.version.toString(),l.fileWatcher||(l.fileWatcher=w(t,r,re,e.PollingInterval.Low,a,\"Source file\"))):(z(l)&&l.fileWatcher.close(),d.set(a,l.version));else if(c){c.version=n.toString();var u=w(t,r,re,e.PollingInterval.Low,a,\"Source file\");d.set(a,{sourceFile:c,version:n,fileWatcher:u})}else d.set(a,n);return c}return l.sourceFile}}function Y(e){var n=d.get(e);void 0!==n&&(q(n)?d.set(e,{version:Number(n)+1}):n.version++)}function Q(e){var n=d.get(e);return!n||q(n)?void 0:n.version.toString()}function Z(n){t.onWatchStatusChange&&t.onWatchStatusChange(e.createCompilerDiagnostic(n),k,S||b)}function $(){t.setTimeout&&t.clearTimeout&&(s&&t.clearTimeout(s),I(\"Scheduling update\"),s=t.setTimeout(ee,250))}function ee(){switch(s=void 0,Z(e.Diagnostics.File_change_detected_Starting_incremental_compilation),a){case e.ConfigFileProgramReloadLevel.Partial:return function(){I(\"Reloading new file names and options\");var n=e.getFileNamesFromConfigSpecs(c,e.getDirectoryPath(y),S,D);n.fileNames.length?(u=e.filter(u,function(n){return!e.isErrorNoInputFiles(n)}),x=!0):c.filesSpecs||e.some(u,e.isErrorNoInputFiles)||(u=u.concat(e.getErrorForNoInputFiles(c,y)),x=!0),T=n.fileNames,U()}();case e.ConfigFileProgramReloadLevel.Full:return I(\"Reloading config file: \"+y),a=e.ConfigFileProgramReloadLevel.None,A&&A.clearCache(),ne(),m=!0,U(),void se();default:return void U()}}function ne(){te(e.getParsedCommandLineOfConfigFile(y,b,D))}function te(n){T=n.fileNames,S=n.options,c=n.configFileSpecs,L=n.projectReferences,u=e.getConfigFileParsingDiagnostics(n),x=!0}function re(n,t,r){ae(n,r,t),t===e.FileWatcherEventKind.Deleted&&d.get(r)&&V.invalidateResolutionOfFile(r),Y(r),$()}function ae(e,n,t){A&&A.addOrDeleteFile(e,n,t)}function ie(n){return w(t,n,oe,e.PollingInterval.Medium,n,\"Missing file\")}function oe(n,t,r){ae(n,r,t),t===e.FileWatcherEventKind.Created&&i.has(r)&&(i.get(r).close(),i.delete(r),Y(r),$())}function se(){c?e.updateWatchingWildcardDirectories(o||(o=e.createMap()),e.createMapFromTemplate(c.wildcardDirectories),le):o&&e.clearMap(o,e.closeFileWatcherOf)}function le(n,r){return P(t,n,function(t){e.Debug.assert(!!y);var r=W(t);A&&A.addOrDeleteFileOrDirectory(t,r),Y(r),r===n||!e.hasExtension(r)||e.isSupportedSourceFileName(t,S)?a!==e.ConfigFileProgramReloadLevel.Full&&(a=e.ConfigFileProgramReloadLevel.Partial,$()):I(\"Project: \"+y+\" Detected file add/remove of non supported extension: \"+t)},r,\"Wild card directories\")}}}(Tn||(Tn={})),function(e){var n,t,r=new Date(-864e13),a=new Date(864e13);function i(n){var t=e.createMap();return{setValue:function(e,r){t.set(n(e),r)},getValue:function(e){return t.get(n(e))},removeKey:function(e){t.delete(n(e))},forEach:function(e){t.forEach(e)},hasKey:function(e){return t.has(n(e))},getSize:function(){return t.size},clear:function(){t.clear()}}}function o(e,n,t){var r,a=e.getValue(n);return a||(r=t(),e.setValue(n,r)),a||r}function s(n,t){return o(n,t,e.createMap)}function l(n,t){var r=e.getRelativePathFromDirectory(u(t.options,t.options.configFilePath),n,!0),a=e.resolvePath(t.options.declarationDir||t.options.outDir||e.getDirectoryPath(t.options.configFilePath),r);return e.changeExtension(a,\".d.ts\")}function c(n,t){if(t.options.outFile||t.options.out||e.fileExtensionIs(n,\".d.ts\"))return e.emptyArray;var r=[],a=function(n,t){var r=e.getRelativePathFromDirectory(u(t.options,t.options.configFilePath),n,!0),a=e.resolvePath(t.options.outDir||e.getDirectoryPath(t.options.configFilePath),r),i=e.fileExtensionIs(n,\".json\")?\".json\":e.fileExtensionIs(n,\".tsx\")&&1===t.options.jsx?\".jsx\":\".js\";return e.changeExtension(a,i)}(n,t);if(r.push(a),t.options.sourceMap&&r.push(a+\".map\"),e.getEmitDeclarations(t.options)&&!e.fileExtensionIs(n,\".json\")){var i=l(n,t);r.push(i),t.options.declarationMap&&r.push(i+\".map\")}return r}function u(n,t){return n.rootDir||e.getDirectoryPath(t)}function d(e,n){return n>e?n:e}function m(n){return e.fileExtensionIs(n,\".d.ts\")}function p(n,t){return function(r){var a=t?\"[\"+e.formatColorAndReset((new Date).toLocaleTimeString(),e.ForegroundColorEscapeSequences.Grey)+\"] \":(new Date).toLocaleTimeString()+\" - \";a+=\"\"+e.flattenDiagnosticMessageText(r.messageText,n.newLine)+(n.newLine+n.newLine),n.write(a)}}function f(n,t,r){void 0===n&&(n=e.sys);var a=e.createCompilerHostWorker({},void 0,n);return a.getModifiedTime=n.getModifiedTime?function(e){return n.getModifiedTime(e)}:function(){},a.setModifiedTime=n.setModifiedTime?function(e,t){return n.setModifiedTime(e,t)}:e.noop,a.deleteFile=n.deleteFile?function(e){return n.deleteFile(e)}:e.noop,a.reportDiagnostic=t||e.createDiagnosticReporter(n),a.reportSolutionBuilderStatus=r||p(n),a}function g(n){var t={};return e.commonOptionsWithBuild.forEach(function(e){t[e.name]=n[e.name]}),t}function _(n){return e.fileExtensionIs(n,\".json\")?n:e.combinePaths(n,\"tsconfig.json\")}function v(n){if(n.options.outFile||n.options.out)return function(n){var t=n.options.outFile||n.options.out;if(!t)return e.Debug.fail(\"outFile must be set\");var r=[];if(r.push(t),n.options.sourceMap&&r.push(t+\".map\"),e.getEmitDeclarations(n.options)){var a=e.changeExtension(t,\".d.ts\");r.push(a),n.options.declarationMap&&r.push(a+\".map\")}return r}(n);for(var t=[],r=0,a=n.fileNames;r<a.length;r++){var i=a[r];t.push.apply(t,c(i,n))}return t}function y(n,r,a,i){switch(r.type){case t.OutOfDateWithSelf:return i(e.Diagnostics.Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2,a(n),a(r.outOfDateOutputFileName),a(r.newerInputFileName));case t.OutOfDateWithUpstream:return i(e.Diagnostics.Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2,a(n),a(r.outOfDateOutputFileName),a(r.newerProjectName));case t.OutputMissing:return i(e.Diagnostics.Project_0_is_out_of_date_because_output_file_1_does_not_exist,a(n),a(r.missingOutputFileName));case t.UpToDate:if(void 0!==r.newestInputFileTime)return i(e.Diagnostics.Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2,a(n),a(r.newestInputFileName||\"\"),a(r.oldestOutputFileName||\"\"));break;case t.UpToDateWithUpstreamTypes:return i(e.Diagnostics.Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies,a(n));case t.UpstreamOutOfDate:return i(e.Diagnostics.Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date,a(n),a(r.upstreamProjectName));case t.UpstreamBlocked:return i(e.Diagnostics.Project_0_can_t_be_built_because_its_dependency_1_has_errors,a(n),a(r.upstreamProjectName));case t.Unbuildable:return i(e.Diagnostics.Failed_to_parse_file_0_Colon_1,a(n),r.reason);case t.ContainerOnly:case t.ComputingUpstream:break;default:e.assertType(r)}}!function(e){e[e.None=0]=\"None\",e[e.Success=1]=\"Success\",e[e.DeclarationOutputUnchanged=2]=\"DeclarationOutputUnchanged\",e[e.ConfigFileErrors=4]=\"ConfigFileErrors\",e[e.SyntaxErrors=8]=\"SyntaxErrors\",e[e.TypeErrors=16]=\"TypeErrors\",e[e.DeclarationEmitErrors=32]=\"DeclarationEmitErrors\",e[e.EmitErrors=64]=\"EmitErrors\",e[e.AnyErrors=124]=\"AnyErrors\"}(n||(n={})),function(e){e[e.Unbuildable=0]=\"Unbuildable\",e[e.UpToDate=1]=\"UpToDate\",e[e.UpToDateWithUpstreamTypes=2]=\"UpToDateWithUpstreamTypes\",e[e.OutputMissing=3]=\"OutputMissing\",e[e.OutOfDateWithSelf=4]=\"OutOfDateWithSelf\",e[e.OutOfDateWithUpstream=5]=\"OutOfDateWithUpstream\",e[e.UpstreamOutOfDate=6]=\"UpstreamOutOfDate\",e[e.UpstreamBlocked=7]=\"UpstreamBlocked\",e[e.ComputingUpstream=8]=\"ComputingUpstream\",e[e.ContainerOnly=9]=\"ContainerOnly\"}(t=e.UpToDateStatusType||(e.UpToDateStatusType={})),e.getOutputDeclarationFileName=l,e.createBuilderStatusReporter=p,e.createSolutionBuilderHost=f,e.createSolutionBuilderWithWatchHost=function(n,t,r,a){void 0===n&&(n=e.sys);var i=f(n,t,r),o=e.createWatchHost(n,a);return i.onWatchStatusChange=o.onWatchStatusChange,i.watchFile=o.watchFile,i.watchDirectory=o.watchDirectory,i.setTimeout=o.setTimeout,i.clearTimeout=o.clearTimeout,i},e.createSolutionBuilder=function(l,c,u){var p,f,h=l,b=l.getCurrentDirectory(),E=e.createGetCanonicalFileName(l.useCaseSensitiveFileNames()),T=e.parseConfigHostFromCompilerHost(l),S=u,L=g(S),x=i(V),A=i(V),C=i(V),D=e.createMap(),k=function(e){return l.trace&&l.trace(e)},M=i(V),O=i(V),I=i(V),R=[],N=0,w=!1,P=i(V),F=i(V),G=i(V);return{buildAllProjects:function(){S.watch&&H(e.Diagnostics.Starting_compilation_in_watch_mode);var r=X();!function(n){S.verbose&&B(e.Diagnostics.Projects_in_this_build_Colon_0,n.buildQueue.map(function(e){return\"\\r\\n    * \"+le(e)}).join(\"\"))}(r);for(var a=!1,i=0,o=r.buildQueue;i<o.length;i++){var s=o[i],l=K(s);if(void 0===l){ie(s),a=!0;break}var c=l.errors,d=Y(l);ce(s,d);var m=l.options.configFilePath;if(d.type!==t.UpToDate||S.force)if(d.type!==t.UpToDateWithUpstreamTypes||S.force)if(d.type!==t.UpstreamBlocked)if(d.type!==t.ContainerOnly){var p=te(s);a=a||!!(p&n.AnyErrors)}else oe(s,c);else oe(s,c),S.verbose&&B(e.Diagnostics.Skipping_build_of_project_0_because_its_dependency_1_has_errors,m,d.upstreamProjectName);else oe(s,c),re(l);else oe(s,c),u.dry&&B(e.Diagnostics.Project_0_is_up_to_date,m)}return ne(),a?e.ExitStatus.DiagnosticsPresent_OutputsSkipped:e.ExitStatus.Success},getUpToDateStatusOfFile:function(e){return Y(K(e))},cleanAllProjects:function(){var n=function(){for(var e=X(),n=[],t=0,r=e.buildQueue;t<r.length;t++){var a=r[t],i=K(a);if(void 0!==i)for(var o=v(i),s=0,c=o;s<c.length;s++){var u=c[s];l.fileExists(u)&&n.push(u)}else ie(a)}return n}();if(S.dry)return B(e.Diagnostics.A_non_dry_build_would_delete_the_following_files_Colon_0,n.map(function(e){return\"\\r\\n * \"+e}).join(\"\")),e.ExitStatus.Success;for(var t=0,r=n;t<r.length;t++){var a=r[t];l.deleteFile(a)}return e.ExitStatus.Success},resetBuildContext:function(n){void 0===n&&(n=u),L=g(S=n),x.clear(),A.clear(),C.clear(),D.clear(),p=void 0,M.clear(),O.clear(),I.clear(),R.length=0,N=0,f&&(clearTimeout(f),f=void 0),w=!1,e.clearMap(P,function(n){return e.clearMap(n,e.closeFileWatcherOf)}),e.clearMap(F,function(n){return e.clearMap(n,e.closeFileWatcher)}),e.clearMap(G,e.closeFileWatcher)},getBuildGraph:J,invalidateProject:function(e,n){Q(ae(e),n)},buildInvalidatedProject:ee,resolveProjectName:ae,startWatching:function(){for(var e=X(),n=0,t=e.buildQueue;n<t.length;n++){var r=t[n];U(r);var a=K(r);a&&(j(r,a),W(r,a))}}};function V(n){return e.toPath(n,b,E)}function K(n){var t,r=x.getValue(n);if(r)return r.options?r:void 0;T.onUnRecoverableConfigFileDiagnostic=function(e){return t=e};var a=e.getParsedCommandLineOfConfigFile(n,L,T);return T.onUnRecoverableConfigFileDiagnostic=e.noop,x.setValue(n,a||t),a}function B(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];l.reportSolutionBuilderStatus(e.createCompilerDiagnostic.apply(void 0,[n].concat(t)))}function H(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];h.onWatchStatusChange&&h.onWatchStatusChange(e.createCompilerDiagnostic.apply(void 0,[n].concat(t)),l.getNewLine(),L)}function U(n){S.watch&&!G.hasKey(n)&&G.setValue(n,h.watchFile(n,function(){z(n,e.ConfigFileProgramReloadLevel.Full)}))}function j(n,t){S.watch&&e.updateWatchingWildcardDirectories(s(P,n),e.createMapFromTemplate(t.configFileSpecs.wildcardDirectories),function(r,a){return h.watchDirectory(r,function(a){var i=V(a);i!==V(r)&&e.hasExtension(i)&&!e.isSupportedSourceFileName(a,t.options)||function(n,t){if(t.options.noEmit)return!1;if(!e.fileExtensionIs(n,\".d.ts\")&&(e.fileExtensionIs(n,\".ts\")||e.fileExtensionIs(n,\".tsx\")))return!1;var r=t.options.outFile||t.options.out;return!(!r||!q(n,r)&&!q(n,e.removeFileExtension(r)+\".d.ts\"))||(!(!t.options.declarationDir||!e.containsPath(t.options.declarationDir,n,b,!l.useCaseSensitiveFileNames()))||(!(!t.options.outDir||!e.containsPath(t.options.outDir,n,b,!l.useCaseSensitiveFileNames()))||!e.forEach(t.fileNames,function(e){return q(n,e)})))}(a,t)||z(n,e.ConfigFileProgramReloadLevel.Partial)},!!(1&a))})}function W(n,t){S.watch&&e.mutateMap(s(F,n),e.arrayToMap(t.fileNames,V),{createNewValue:function(t,r){return h.watchFile(r,function(){z(n,e.ConfigFileProgramReloadLevel.None)})},onDeleteValue:e.closeFileWatcher})}function q(n,t){return 0===e.comparePaths(n,t,b,!l.useCaseSensitiveFileNames())}function z(e,n){w=!0,Q(e,n),$()}function J(n){return function(n){for(var t=i(V),r=i(V),a=[],s=[],l=i(V),c=0,u=n;c<u.length;c++){var d=u[c];m(d)}return{buildQueue:s,referencingProjectsMap:l};function m(n,c){if(!r.hasKey(n))if(t.hasKey(n))c||B(e.Diagnostics.Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0,a.join(\"\\r\\n\"));else{t.setValue(n,!0),a.push(n);var u=K(n);if(u&&u.projectReferences)for(var d=0,p=u.projectReferences;d<p.length;d++){var f=p[d],g=ae(f.path);m(g,c||f.circular);var _=o(l,g,function(){return i(V)});_.setValue(n,!!f.prepend)}a.pop(),r.setValue(n,!0),s.push(n)}}}(function(e){return e.map(ae)}(n))}function X(){return p||(p=J(c))}function Y(n){if(void 0===n)return{type:t.Unbuildable,reason:\"File deleted mid-build\"};var i=C.getValue(n.options.configFilePath);if(void 0!==i)return i;var o=function(n){for(var i=void 0,o=r,s=0,c=n.fileNames;s<c.length;s++){var u=c[s];if(!l.fileExists(u))return{type:t.Unbuildable,reason:u+\" does not exist\"};var p=l.getModifiedTime(u)||e.missingFileModifiedTime;p>o&&(i=u,o=p)}var f=v(n);if(0===f.length)return{type:t.ContainerOnly};for(var g,_=\"(none)\",y=a,h=\"(none)\",b=r,E=r,T=!1,S=0,L=f;S<L.length;S++){var x=L[S];if(!l.fileExists(x)){g=x;break}var D=l.getModifiedTime(x)||e.missingFileModifiedTime;if(D<y&&(y=D,_=x),D<o){T=!0;break}if(D>b&&(b=D,h=x),m(x)){var k=A.getValue(x);if(void 0!==k)E=d(k,E);else{var M=l.getModifiedTime(x)||e.missingFileModifiedTime;E=d(E,M)}}}var O,I=!1,R=!1;if(n.projectReferences){C.setValue(n.options.configFilePath,{type:t.ComputingUpstream});for(var N=0,w=n.projectReferences;N<w.length;N++){var P=w[N];R=R||!!P.prepend;var F=e.resolveProjectReferencePath(P),G=Y(K(F));if(G.type!==t.ComputingUpstream){if(G.type===t.Unbuildable)return{type:t.UpstreamBlocked,upstreamProjectName:P.path};if(G.type!==t.UpToDate)return{type:t.UpstreamOutOfDate,upstreamProjectName:P.path};if(!(G.newestInputFileTime&&G.newestInputFileTime<=y)){if(!(G.newestDeclarationFileContentChangedTime&&G.newestDeclarationFileContentChangedTime<=y))return e.Debug.assert(void 0!==_,\"Should have an oldest output filename here\"),{type:t.OutOfDateWithUpstream,outOfDateOutputFileName:_,newerProjectName:P.path};I=!0,O=P.path}}}}return void 0!==g?{type:t.OutputMissing,missingOutputFileName:g}:T?{type:t.OutOfDateWithSelf,outOfDateOutputFileName:_,newerInputFileName:i}:R&&I?{type:t.OutOfDateWithUpstream,outOfDateOutputFileName:_,newerProjectName:O}:{type:I?t.UpToDateWithUpstreamTypes:t.UpToDate,newestDeclarationFileContentChangedTime:E,newestInputFileTime:o,newestOutputFileTime:b,newestInputFileName:i,newestOutputFileName:h,oldestOutputFileName:_}}(n);return C.setValue(n.options.configFilePath,o),o}function Q(n,t){t===e.ConfigFileProgramReloadLevel.Full&&(x.removeKey(n),p=void 0),C.removeKey(n),S.watch&&M.removeKey(n),Z(n,t)}function Z(n,t){var r=O.getValue(n);void 0===r?(O.setValue(n,t||e.ConfigFileProgramReloadLevel.None),R.push(n)):r<(t||e.ConfigFileProgramReloadLevel.None)&&O.setValue(n,t||e.ConfigFileProgramReloadLevel.None)}function $(){h.setTimeout&&h.clearTimeout&&(f&&h.clearTimeout(f),f=h.setTimeout(ee,250))}function ee(){f=void 0,w&&(w=!1,I.clear(),H(e.Diagnostics.File_change_detected_Starting_incremental_compilation));var r=function(){if(N<R.length){var e=R[N];N++;var n=O.getValue(e);return O.removeKey(e),O.getSize()||(R.length=0,N=0),{project:e,reloadLevel:n}}}();r&&(function(r,a){var i=K(r);if(i){if(a===e.ConfigFileProgramReloadLevel.Full)U(r),j(r,i),W(r,i);else if(a===e.ConfigFileProgramReloadLevel.Partial){var o=e.getFileNamesFromConfigSpecs(i.configFileSpecs,e.getDirectoryPath(r),i.options,T);0!==o.fileNames.length?e.filterMutate(i.errors,function(n){return!e.isErrorNoInputFiles(n)}):i.configFileSpecs.filesSpecs||e.some(i.errors,e.isErrorNoInputFiles)||i.errors.push(e.getErrorForNoInputFiles(i.configFileSpecs,r)),i.fileNames=o.fileNames,W(r,i)}var s=Y(i);if(ce(r,s),s.type!==t.UpstreamBlocked){var l=te(r),c=X(),u=c.referencingProjectsMap.getValue(r);if(u)for(var d=0,m=c.buildQueue;d<m.length;d++){var p=m[d],f=u.getValue(p);!f&&(void 0===f||l&n.DeclarationOutputUnchanged)||Z(p)}}else S.verbose&&B(e.Diagnostics.Skipping_build_of_project_0_because_its_dependency_1_has_errors,r,s.upstreamProjectName)}else ie(r)}(r.project,r.reloadLevel),O.getSize()?S.watch&&!f&&$():ne())}function ne(){if(S.watch){X().buildQueue.forEach(function(n){I.hasKey(n)||se(M.getValue(n)||e.emptyArray)});var n=0;M.forEach(function(t){return n+=t.filter(function(n){return n.category===e.DiagnosticCategory.Error}).length}),H(1===n?e.Diagnostics.Found_1_error_Watching_for_file_changes:e.Diagnostics.Found_0_errors_Watching_for_file_changes,n)}}function te(i){if(S.dry)return B(e.Diagnostics.A_non_dry_build_would_build_project_0,i),n.Success;S.verbose&&B(e.Diagnostics.Building_project_0,i);var o=n.None;o|=n.DeclarationOutputUnchanged;var s=K(i);if(!s)return o|=n.ConfigFileErrors,ie(i),C.setValue(i,{type:t.Unbuildable,reason:\"Config file errors\"}),o;if(0===s.fileNames.length)return n.None;var c={projectReferences:s.projectReferences,host:l,rootNames:s.fileNames,options:s.options,configFileParsingDiagnostics:s.errors},u=e.createProgram(c),p=u.getOptionsDiagnostics().concat(u.getConfigFileParsingDiagnostics(),u.getSyntacticDiagnostics());if(p.length)return b(p,n.SyntaxErrors,\"Syntactic\");if(e.getEmitDeclarations(u.getCompilerOptions())){var f=u.getDeclarationDiagnostics();if(f.length)return b(f,n.DeclarationEmitErrors,\"Declaration file\")}var g=u.getSemanticDiagnostics();if(g.length)return b(g,n.TypeErrors,\"Semantic\");var _,v=r,y=!1;if(e.emitFilesAndReportErrors(u,function(e){return(_||(_=[])).push(e)},k,void 0,function(t,r,a,i){var s;!y&&m(t)&&(l.fileExists(t)&&l.readFile(t)===r?s=l.getModifiedTime(t):(o&=~n.DeclarationOutputUnchanged,y=!0)),l.writeFile(t,r,a,i,e.emptyArray),void 0!==s&&(v=d(s,v),A.setValue(t,s))}),_)return b(_,n.EmitErrors,\"Emit\");var h={type:t.UpToDate,newestDeclarationFileContentChangedTime:y?a:v};return C.setValue(i,h),o;function b(e,n,r){return o|=n,oe(i,e),C.setValue(i,{type:t.Unbuildable,reason:r+\" errors\"}),o}}function re(n){if(S.dry)return B(e.Diagnostics.A_non_dry_build_would_build_project_0,n.options.configFilePath);S.verbose&&B(e.Diagnostics.Updating_output_timestamps_of_project_0,n.options.configFilePath);for(var a=new Date,i=v(n),o=r,s=0,c=i;s<c.length;s++){var u=c[s];m(u)&&(o=d(o,l.getModifiedTime(u)||e.missingFileModifiedTime)),l.setModifiedTime(u,a)}C.setValue(n.options.configFilePath,{type:t.UpToDate,newestDeclarationFileContentChangedTime:o})}function ae(n){return _(e.resolvePath(l.getCurrentDirectory(),n))}function ie(e){oe(e,[x.getValue(e)])}function oe(e,n){se(n),S.watch&&(I.setValue(e,!0),M.setValue(e,n))}function se(e){e.forEach(function(e){return l.reportDiagnostic(e)})}function le(n){return e.convertToRelativePath(n,l.getCurrentDirectory(),function(e){return l.getCanonicalFileName(e)})}function ce(e,n){if(S.verbose)return y(e,n,le,B)}},e.resolveConfigFileProjectName=_,e.getAllProjectOutputs=v,e.formatUpToDateStatus=y}(Tn||(Tn={})),function(e){function n(e,n){return t(e,n,(r=new Set,a=[],function(e,n,t,i){if(r.has(e)||a.length>4)return i(r.has(e),a);r.add(e),a.push(n);var o=t();return a.pop(),r.delete(e),o}));var r,a}function t(n,a,i){return i(a,n,function(){if(\"function\"==typeof a)return function(n,r,a){var i=function(n,r){var a=n.prototype;return\"object\"!=typeof a||null===a?e.emptyArray:e.mapDefined(o(a),function(e){var n=e.key,a=e.value;return\"constructor\"===n?void 0:t(n,a,r)})}(n,a),l=e.flatMap(o(n),function(e){var n=e.key,r=e.value;return t(n,r,a)}),c=e.cast(Function.prototype.toString.call(n),e.isString),u=e.stringContains(c,\"{ [native code] }\")?function(n){return e.tryCast(s(n,\"length\"),e.isNumber)||0}(n):c;return{kind:2,name:r,source:u,namespaceMembers:l,prototypeMembers:i}}(a,n,i);if(\"object\"==typeof a){var u=function(n,a,i){return e.isArray(a)?{name:n,kind:1,inner:a.length&&t(\"element\",e.first(a),i)||c(n)}:e.forEachEntry(r(),function(e,t){return a instanceof e?{kind:0,name:n,typeName:t}:void 0})}(n,a,i);if(void 0!==u)return u;var d=o(a);return{kind:3,name:n,members:e.flatMap(d,function(e){return t(e.key,e.value,i)})}}return{kind:0,name:n,typeName:l(a)?\"any\":typeof a}},function(e,t){return c(n,\" \"+(e?\"Circular reference\":\"Too-deep object hierarchy\")+\" from \"+t.join(\".\"))})}!function(e){e[e.Const=0]=\"Const\",e[e.Array=1]=\"Array\",e[e.FunctionOrClass=2]=\"FunctionOrClass\",e[e.Object=3]=\"Object\"}(e.ValueKind||(e.ValueKind={})),e.inspectModule=function(t){return n(e.removeFileExtension(e.getBaseFileName(t)),function(e){try{return}catch(e){return}}())},e.inspectValue=n;var r=e.memoize(function(){for(var n=e.createMap(),t=0,r=o(global);t<r.length;t++){var a=r[t],i=a.key,s=a.value;\"function\"==typeof s&&\"object\"==typeof s.prototype&&s!==Object&&n.set(i,s)}return n});var a=new Set([\"arguments\",\"caller\",\"constructor\",\"eval\",\"super_\"]),i=new Set(Object.getOwnPropertyNames(e.noop));function o(n){for(var t=e.createMap(),r=[],o=n;!l(o)&&o!==Object.prototype&&o!==Function.prototype;){for(var c=0,d=Object.getOwnPropertyNames(o);c<d.length;c++){var m=d[c];if(!u(m)&&!a.has(m)&&(\"function\"!=typeof n||!i.has(m))&&e.addToSeen(t,m)){var p=s(o,m);\"toString\"===m&&\"function\"==typeof p&&0===p.length||r.push({key:m,value:p})}}o=Object.getPrototypeOf(o)}return r.sort(function(n,t){return e.compareStringsCaseSensitive(n.key,t.key)})}function s(e,n){var t=Object.getOwnPropertyDescriptor(e,n);return t&&t.value}function l(e){return null==e}function c(e,n){return{kind:0,name:e,typeName:\"any\",comment:n}}function u(e){return e.startsWith(\"_\")}e.isJsPrivate=u}(Tn||(Tn={})),function(e){!function(n){n.ActionSet=\"action::set\",n.ActionInvalidate=\"action::invalidate\",n.ActionPackageInstalled=\"action::packageInstalled\",n.ActionValueInspected=\"action::valueInspected\",n.EventTypesRegistry=\"event::typesRegistry\",n.EventBeginInstallTypes=\"event::beginInstallTypes\",n.EventEndInstallTypes=\"event::endInstallTypes\",n.EventInitializationFailed=\"event::initializationFailed\",function(e){e.GlobalCacheLocation=\"--globalTypingsCacheLocation\",e.LogFile=\"--logFile\",e.EnableTelemetry=\"--enableTelemetry\",e.TypingSafeListLocation=\"--typingSafeListLocation\",e.TypesMapLocation=\"--typesMapLocation\",e.NpmLocation=\"--npmLocation\"}(n.Arguments||(n.Arguments={})),n.hasArgument=function(n){return e.sys.args.indexOf(n)>=0},n.findArgument=function(n){var t=e.sys.args.indexOf(n);return t>=0&&t<e.sys.args.length-1?e.sys.args[t+1]:void 0},n.nowString=function(){var e=new Date;return e.getHours()+\":\"+e.getMinutes()+\":\"+e.getSeconds()+\".\"+e.getMilliseconds()}}(e.server||(e.server={}))}(Tn||(Tn={})),function(e){!function(n){function t(n,t){return new e.Version(e.getProperty(t,\"ts\"+e.versionMajorMinor)||e.getProperty(t,\"latest\")).compareTo(n.version)<=0}n.isTypingUpToDate=t,n.nodeCoreModuleList=[\"assert\",\"async_hooks\",\"buffer\",\"child_process\",\"cluster\",\"console\",\"constants\",\"crypto\",\"dgram\",\"dns\",\"domain\",\"events\",\"fs\",\"http\",\"https\",\"http2\",\"inspector\",\"net\",\"os\",\"path\",\"perf_hooks\",\"process\",\"punycode\",\"querystring\",\"readline\",\"repl\",\"stream\",\"string_decoder\",\"timers\",\"tls\",\"tty\",\"url\",\"util\",\"v8\",\"vm\",\"zlib\"],n.nodeCoreModules=e.arrayToSet(n.nodeCoreModuleList),n.loadSafeList=function(n,t){var r=e.readConfigFile(t,function(e){return n.readFile(e)});return e.createMapFromTemplate(r.config)},n.loadTypesMap=function(n,t){var r=e.readConfigFile(t,function(e){return n.readFile(e)});if(r.config)return e.createMapFromTemplate(r.config.simpleMap)},n.discoverTypings=function(r,a,i,o,s,l,c,u,d){if(!c||!c.enable)return{cachedTypingPaths:[],newTypingNames:[],filesToWatch:[]};var m=e.createMap();i=e.mapDefined(i,function(n){var t=e.normalizePath(n);if(e.hasJSFileExtension(t))return t});var p=[];c.include&&S(c.include,\"Explicitly included types\");var f=c.exclude||[],g=e.arrayToSet(i,e.getDirectoryPath);g.set(o,!0),g.forEach(function(n,t){L(e.combinePaths(t,\"package.json\"),p),L(e.combinePaths(t,\"bower.json\"),p),x(e.combinePaths(t,\"bower_components\"),p),x(e.combinePaths(t,\"node_modules\"),p)}),function(n){var t=e.mapDefined(n,function(n){if(e.hasJSFileExtension(n)){var t=e.removeFileExtension(e.getBaseFileName(n.toLowerCase())),r=e.removeMinAndVersionNumbers(t);return s.get(r)}});t.length&&S(t,\"Inferred typings from file names\"),e.some(n,function(n){return e.fileExtensionIs(n,\".jsx\")})&&(a&&a(\"Inferred 'react' typings due to presence of '.jsx' extension\"),T(\"react\"))}(i),u&&S(e.deduplicate(u.map(function(e){return n.nodeCoreModules.has(e)?\"node\":e}),e.equateStringsCaseSensitive,e.compareStringsCaseSensitive),\"Inferred typings from unresolved imports\"),l.forEach(function(e,n){var r=d.get(n);m.has(n)&&void 0===m.get(n)&&void 0!==r&&t(e,r)&&m.set(n,e.typingLocation)});for(var _=0,v=f;_<v.length;_++){var y=v[_];m.delete(y)&&a&&a(\"Typing for \"+y+\" is in exclude list, will be ignored.\")}var h=[],b=[];m.forEach(function(e,n){void 0!==e?b.push(e):h.push(n)});var E={cachedTypingPaths:b,newTypingNames:h,filesToWatch:p};return a&&a(\"Result: \"+JSON.stringify(E)),E;function T(e){m.has(e)||m.set(e,void 0)}function S(n,t){a&&a(t+\": \"+JSON.stringify(n)),e.forEach(n,T)}function L(n,t){if(r.fileExists(n)){t.push(n);var a=e.readConfigFile(n,function(e){return r.readFile(e)}).config;S(e.flatMap([a.dependencies,a.devDependencies,a.optionalDependencies,a.peerDependencies],e.getOwnKeys),\"Typing names in '\"+n+\"' dependencies\")}}function x(n,t){if(t.push(n),r.directoryExists(n)){var i=r.readDirectory(n,[\".json\"],void 0,void 0,2);a&&a(\"Searching for typing names in \"+n+\"; all files: \"+JSON.stringify(i));for(var o=[],s=0,l=i;s<l.length;s++){var c=l[s],u=e.normalizePath(c),d=e.getBaseFileName(u);if(\"package.json\"===d||\"bower.json\"===d){var p=e.readConfigFile(u,function(e){return r.readFile(e)}).config;if((\"package.json\"!==d||!p._requiredBy||0!==e.filter(p._requiredBy,function(e){return\"#\"===e[0]||\"/\"===e}).length)&&p.name){var f=p.types||p.typings;if(f){var g=e.getNormalizedAbsolutePath(f,e.getDirectoryPath(u));a&&a(\"    Package '\"+p.name+\"' provides its own types.\"),m.set(p.name,g)}else o.push(p.name)}}}S(o,\"    Found package names\")}}},function(e){e[e.Ok=0]=\"Ok\",e[e.ScopedPackagesNotSupported=1]=\"ScopedPackagesNotSupported\",e[e.EmptyName=2]=\"EmptyName\",e[e.NameTooLong=3]=\"NameTooLong\",e[e.NameStartsWithDot=4]=\"NameStartsWithDot\",e[e.NameStartsWithUnderscore=5]=\"NameStartsWithUnderscore\",e[e.NameContainsNonURISafeCharacters=6]=\"NameContainsNonURISafeCharacters\"}(n.PackageNameValidationResult||(n.PackageNameValidationResult={}));var r=214;n.validatePackageName=function(e){return e?e.length>r?3:46===e.charCodeAt(0)?4:95===e.charCodeAt(0)?5:/^@[^\\/]+\\/[^\\/]+$/.test(e)?1:encodeURIComponent(e)!==e?6:0:2},n.renderPackageNameValidationFailure=function(n,t){switch(n){case 2:return\"Package name '\"+t+\"' cannot be empty\";case 3:return\"Package name '\"+t+\"' should be less than \"+r+\" characters\";case 4:return\"Package name '\"+t+\"' cannot start with '.'\";case 5:return\"Package name '\"+t+\"' cannot start with '_'\";case 1:return\"Package '\"+t+\"' is scoped and currently is not supported\";case 6:return\"Package name '\"+t+\"' contains non URI safe characters\";case 0:return e.Debug.fail();default:throw e.Debug.assertNever(n)}}}(e.JsTyping||(e.JsTyping={}))}(Tn||(Tn={})),function(e){!function(e){var n=function(){function e(e){this.text=e}return e.prototype.getText=function(e,n){return 0===e&&n===this.text.length?this.text:this.text.substring(e,n)},e.prototype.getLength=function(){return this.text.length},e.prototype.getChangeRange=function(){},e}();e.fromString=function(e){return new n(e)}}(e.ScriptSnapshot||(e.ScriptSnapshot={})),e.emptyOptions={};var n,t=function(){return function(){}}();e.TextChange=t,function(e){e.none=\"none\",e.definition=\"definition\",e.reference=\"reference\",e.writtenReference=\"writtenReference\"}(e.HighlightSpanKind||(e.HighlightSpanKind={})),function(e){e[e.None=0]=\"None\",e[e.Block=1]=\"Block\",e[e.Smart=2]=\"Smart\"}(n=e.IndentStyle||(e.IndentStyle={})),e.testFormatSettings={baseIndentSize:0,indentSize:4,tabSize:4,newLineCharacter:\"\\n\",convertTabsToSpaces:!0,indentStyle:n.Smart,insertSpaceAfterCommaDelimiter:!0,insertSpaceAfterSemicolonInForStatements:!0,insertSpaceBeforeAndAfterBinaryOperators:!0,insertSpaceAfterConstructor:!1,insertSpaceAfterKeywordsInControlFlowStatements:!0,insertSpaceAfterFunctionKeywordForAnonymousFunctions:!1,insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis:!1,insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets:!1,insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces:!0,insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces:!1,insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces:!1,insertSpaceAfterTypeAssertion:!1,placeOpenBraceOnNewLineForFunctions:!1,placeOpenBraceOnNewLineForControlBlocks:!1,insertSpaceBeforeTypeAnnotation:!1},function(e){e[e.aliasName=0]=\"aliasName\",e[e.className=1]=\"className\",e[e.enumName=2]=\"enumName\",e[e.fieldName=3]=\"fieldName\",e[e.interfaceName=4]=\"interfaceName\",e[e.keyword=5]=\"keyword\",e[e.lineBreak=6]=\"lineBreak\",e[e.numericLiteral=7]=\"numericLiteral\",e[e.stringLiteral=8]=\"stringLiteral\",e[e.localName=9]=\"localName\",e[e.methodName=10]=\"methodName\",e[e.moduleName=11]=\"moduleName\",e[e.operator=12]=\"operator\",e[e.parameterName=13]=\"parameterName\",e[e.propertyName=14]=\"propertyName\",e[e.punctuation=15]=\"punctuation\",e[e.space=16]=\"space\",e[e.text=17]=\"text\",e[e.typeParameterName=18]=\"typeParameterName\",e[e.enumMemberName=19]=\"enumMemberName\",e[e.functionName=20]=\"functionName\",e[e.regularExpressionLiteral=21]=\"regularExpressionLiteral\"}(e.SymbolDisplayPartKind||(e.SymbolDisplayPartKind={})),function(e){e.Comment=\"comment\",e.Region=\"region\",e.Code=\"code\",e.Imports=\"imports\"}(e.OutliningSpanKind||(e.OutliningSpanKind={})),function(e){e[e.JavaScript=0]=\"JavaScript\",e[e.SourceMap=1]=\"SourceMap\",e[e.Declaration=2]=\"Declaration\"}(e.OutputFileType||(e.OutputFileType={})),function(e){e[e.None=0]=\"None\",e[e.InMultiLineCommentTrivia=1]=\"InMultiLineCommentTrivia\",e[e.InSingleQuoteStringLiteral=2]=\"InSingleQuoteStringLiteral\",e[e.InDoubleQuoteStringLiteral=3]=\"InDoubleQuoteStringLiteral\",e[e.InTemplateHeadOrNoSubstitutionTemplate=4]=\"InTemplateHeadOrNoSubstitutionTemplate\",e[e.InTemplateMiddleOrTail=5]=\"InTemplateMiddleOrTail\",e[e.InTemplateSubstitutionPosition=6]=\"InTemplateSubstitutionPosition\"}(e.EndOfLineState||(e.EndOfLineState={})),function(e){e[e.Punctuation=0]=\"Punctuation\",e[e.Keyword=1]=\"Keyword\",e[e.Operator=2]=\"Operator\",e[e.Comment=3]=\"Comment\",e[e.Whitespace=4]=\"Whitespace\",e[e.Identifier=5]=\"Identifier\",e[e.NumberLiteral=6]=\"NumberLiteral\",e[e.StringLiteral=7]=\"StringLiteral\",e[e.RegExpLiteral=8]=\"RegExpLiteral\"}(e.TokenClass||(e.TokenClass={})),function(e){e.unknown=\"\",e.warning=\"warning\",e.keyword=\"keyword\",e.scriptElement=\"script\",e.moduleElement=\"module\",e.classElement=\"class\",e.localClassElement=\"local class\",e.interfaceElement=\"interface\",e.typeElement=\"type\",e.enumElement=\"enum\",e.enumMemberElement=\"enum member\",e.variableElement=\"var\",e.localVariableElement=\"local var\",e.functionElement=\"function\",e.localFunctionElement=\"local function\",e.memberFunctionElement=\"method\",e.memberGetAccessorElement=\"getter\",e.memberSetAccessorElement=\"setter\",e.memberVariableElement=\"property\",e.constructorImplementationElement=\"constructor\",e.callSignatureElement=\"call\",e.indexSignatureElement=\"index\",e.constructSignatureElement=\"construct\",e.parameterElement=\"parameter\",e.typeParameterElement=\"type parameter\",e.primitiveType=\"primitive type\",e.label=\"label\",e.alias=\"alias\",e.constElement=\"const\",e.letElement=\"let\",e.directory=\"directory\",e.externalModuleName=\"external module name\",e.jsxAttribute=\"JSX attribute\",e.string=\"string\"}(e.ScriptElementKind||(e.ScriptElementKind={})),function(e){e.none=\"\",e.publicMemberModifier=\"public\",e.privateMemberModifier=\"private\",e.protectedMemberModifier=\"protected\",e.exportedModifier=\"export\",e.ambientModifier=\"declare\",e.staticModifier=\"static\",e.abstractModifier=\"abstract\",e.optionalModifier=\"optional\"}(e.ScriptElementKindModifier||(e.ScriptElementKindModifier={})),function(e){e.comment=\"comment\",e.identifier=\"identifier\",e.keyword=\"keyword\",e.numericLiteral=\"number\",e.operator=\"operator\",e.stringLiteral=\"string\",e.whiteSpace=\"whitespace\",e.text=\"text\",e.punctuation=\"punctuation\",e.className=\"class name\",e.enumName=\"enum name\",e.interfaceName=\"interface name\",e.moduleName=\"module name\",e.typeParameterName=\"type parameter name\",e.typeAliasName=\"type alias name\",e.parameterName=\"parameter name\",e.docCommentTagName=\"doc comment tag name\",e.jsxOpenTagName=\"jsx open tag name\",e.jsxCloseTagName=\"jsx close tag name\",e.jsxSelfClosingTagName=\"jsx self closing tag name\",e.jsxAttribute=\"jsx attribute\",e.jsxText=\"jsx text\",e.jsxAttributeStringLiteralValue=\"jsx attribute string literal value\"}(e.ClassificationTypeNames||(e.ClassificationTypeNames={})),function(e){e[e.comment=1]=\"comment\",e[e.identifier=2]=\"identifier\",e[e.keyword=3]=\"keyword\",e[e.numericLiteral=4]=\"numericLiteral\",e[e.operator=5]=\"operator\",e[e.stringLiteral=6]=\"stringLiteral\",e[e.regularExpressionLiteral=7]=\"regularExpressionLiteral\",e[e.whiteSpace=8]=\"whiteSpace\",e[e.text=9]=\"text\",e[e.punctuation=10]=\"punctuation\",e[e.className=11]=\"className\",e[e.enumName=12]=\"enumName\",e[e.interfaceName=13]=\"interfaceName\",e[e.moduleName=14]=\"moduleName\",e[e.typeParameterName=15]=\"typeParameterName\",e[e.typeAliasName=16]=\"typeAliasName\",e[e.parameterName=17]=\"parameterName\",e[e.docCommentTagName=18]=\"docCommentTagName\",e[e.jsxOpenTagName=19]=\"jsxOpenTagName\",e[e.jsxCloseTagName=20]=\"jsxCloseTagName\",e[e.jsxSelfClosingTagName=21]=\"jsxSelfClosingTagName\",e[e.jsxAttribute=22]=\"jsxAttribute\",e[e.jsxText=23]=\"jsxText\",e[e.jsxAttributeStringLiteralValue=24]=\"jsxAttributeStringLiteralValue\"}(e.ClassificationType||(e.ClassificationType={}))}(Tn||(Tn={})),function(e){function n(n){switch(n.kind){case 235:return e.isInJSFile(n)&&e.getJSDocEnumTag(n)?7:1;case 149:case 184:case 152:case 151:case 273:case 274:case 154:case 153:case 155:case 156:case 157:case 237:case 194:case 195:case 272:case 265:return 1;case 148:case 239:case 240:case 166:return 2;case 302:return void 0===n.name?3:2;case 276:case 238:return 3;case 242:return e.isAmbientModule(n)?5:1===e.getModuleInstanceState(n)?5:4;case 241:case 250:case 251:case 246:case 247:case 252:case 253:return 7;case 277:return 5}return 7}function t(n){for(;146===n.parent.kind;)n=n.parent;return e.isInternalModuleImportEqualsDeclaration(n.parent)&&n.parent.moduleReference===n}function r(e,n){var t=a(e);return!!t&&!!t.parent&&n(t.parent)&&t.parent.expression===t}function a(e){return s(e)?e.parent:e}function i(n){return 71===n.kind&&e.isBreakOrContinueStatement(n.parent)&&n.parent.label===n}function o(n){return 71===n.kind&&e.isLabeledStatement(n.parent)&&n.parent.label===n}function s(e){return e&&e.parent&&187===e.parent.kind&&e.parent.name===e}e.scanner=e.createScanner(6,!0),function(e){e[e.None=0]=\"None\",e[e.Value=1]=\"Value\",e[e.Type=2]=\"Type\",e[e.Namespace=4]=\"Namespace\",e[e.All=7]=\"All\"}(e.SemanticMeaning||(e.SemanticMeaning={})),e.getMeaningFromDeclaration=n,e.getMeaningFromLocation=function(r){return 277===r.kind?1:252===r.parent.kind||257===r.parent.kind?7:t(r)?function(n){var t=146===n.kind?n:e.isQualifiedName(n.parent)&&n.parent.right===n?n.parent:void 0;return t&&246===t.parent.kind?7:4}(r):e.isDeclarationName(r)?n(r.parent):function(n){switch(e.isRightSideOfQualifiedNameOrPropertyAccess(n)&&(n=n.parent),n.kind){case 99:return!e.isExpressionNode(n);case 176:return!0}switch(n.parent.kind){case 162:return!0;case 181:return!n.parent.isTypeOf;case 209:return!e.isExpressionWithTypeArgumentsInClassExtendsClause(n.parent)}return!1}(r)?2:function(e){return function(e){var n=e,t=!0;if(146===n.parent.kind){for(;n.parent&&146===n.parent.kind;)n=n.parent;t=n.right===e}return 162===n.parent.kind&&!t}(e)||function(e){var n=e,t=!0;if(187===n.parent.kind){for(;n.parent&&187===n.parent.kind;)n=n.parent;t=n.name===e}if(!t&&209===n.parent.kind&&271===n.parent.parent.kind){var r=n.parent.parent.parent;return 238===r.kind&&108===n.parent.parent.token||239===r.kind&&85===n.parent.parent.token}return!1}(e)}(r)?4:e.isTypeParameterDeclaration(r.parent)?(e.Debug.assert(e.isJSDocTemplateTag(r.parent.parent)),2):e.isLiteralTypeNode(r.parent)?3:1},e.isInRightSideOfInternalImportEqualsDeclaration=t,e.isCallExpressionTarget=function(n){return r(n,e.isCallExpression)},e.isNewExpressionTarget=function(n){return r(n,e.isNewExpression)},e.isCallOrNewExpressionTarget=function(n){return r(n,e.isCallOrNewExpression)},e.climbPastPropertyAccess=a,e.getTargetLabel=function(e,n){for(;e;){if(231===e.kind&&e.label.escapedText===n)return e.label;e=e.parent}},e.hasPropertyAccessExpressionWithName=function(n,t){return!!e.isPropertyAccessExpression(n.expression)&&n.expression.name.text===t},e.isJumpStatementTarget=i,e.isLabelOfLabeledStatement=o,e.isLabelName=function(e){return o(e)||i(e)},e.isRightSideOfQualifiedName=function(e){return 146===e.parent.kind&&e.parent.right===e},e.isRightSideOfPropertyAccess=s,e.isNameOfModuleDeclaration=function(e){return 242===e.parent.kind&&e.parent.name===e},e.isNameOfFunctionDeclaration=function(n){return 71===n.kind&&e.isFunctionLike(n.parent)&&n.parent.name===n},e.isLiteralNameOfPropertyDeclarationOrIndexAccess=function(n){switch(n.parent.kind){case 152:case 151:case 273:case 276:case 154:case 153:case 156:case 157:case 242:return e.getNameOfDeclaration(n.parent)===n;case 188:return n.parent.argumentExpression===n;case 147:return!0;case 180:return 178===n.parent.parent.kind;default:return!1}},e.isExpressionOfExternalModuleImportEqualsDeclaration=function(n){return e.isExternalModuleImportEqualsDeclaration(n.parent.parent)&&e.getExternalModuleImportEqualsDeclarationExpression(n.parent.parent)===n},e.getContainerNode=function(n){for(e.isJSDocTypeAlias(n)&&(n=n.parent.parent);;){if(!(n=n.parent))return;switch(n.kind){case 277:case 154:case 153:case 237:case 194:case 156:case 157:case 238:case 239:case 241:case 242:return n}}},e.getNodeKind=function n(t){switch(t.kind){case 277:return e.isExternalModule(t)?\"module\":\"script\";case 242:return\"module\";case 238:case 207:return\"class\";case 239:return\"interface\";case 240:case 295:case 302:return\"type\";case 241:return\"enum\";case 235:return o(t);case 184:return o(e.getRootDeclaration(t));case 195:case 237:case 194:return\"function\";case 156:return\"getter\";case 157:return\"setter\";case 154:case 153:return\"method\";case 152:case 151:return\"property\";case 160:return\"index\";case 159:return\"construct\";case 158:return\"call\";case 155:return\"constructor\";case 148:return\"type parameter\";case 276:return\"enum member\";case 149:return e.hasModifier(t,92)?\"property\":\"parameter\";case 246:case 251:case 255:case 249:return\"alias\";case 202:var r=e.getAssignmentDeclarationKind(t),a=t.right;switch(r){case 0:return\"\";case 1:case 2:var i=n(a);return\"\"===i?\"const\":i;case 3:return e.isFunctionExpression(a)?\"method\":\"property\";case 4:return\"property\";case 5:return e.isFunctionExpression(a)?\"method\":\"property\";case 6:return\"local class\";default:return e.assertType(r),\"\"}case 71:return e.isImportClause(t.parent)?\"alias\":\"\";default:return\"\"}function o(n){return e.isVarConst(n)?\"const\":e.isLet(n)?\"let\":\"var\"}},e.isThis=function(n){switch(n.kind){case 99:return!0;case 71:return e.identifierIsThisKeyword(n)&&149===n.parent.kind;default:return!1}};var l=/^\\/\\/\\/\\s*</;function c(e,n){return d(e.pos,e.end,n)}function u(e,n){return e.pos<n&&n<e.end}function d(e,n,t){return e<=t.pos&&n>=t.end}function m(e,n,t,r){return Math.max(e,t)<Math.min(n,r)}function p(n,t,r){var a=n.getChildren(r);if(a.length){var i=e.last(a);if(i.kind===t)return!0;if(25===i.kind&&1!==a.length)return a[a.length-2].kind===t}return!1}function f(e,n,t){return!!g(e,n,t)}function g(n,t,r){return e.find(n.getChildren(r),function(e){return e.kind===t})}function _(n){var t=e.find(n.parent.getChildren(),function(t){return e.isSyntaxList(t)&&c(t,n)});return e.Debug.assert(!t||e.contains(t.getChildren(),n)),t}function v(e,n,t){return h(e,n,!1,t,!1)}function y(e,n){return h(e,n,!0,void 0,!1)}function h(e,n,t,r,a){var i=e;e:for(;;){for(var o=0,s=i.getChildren(e);o<s.length;o++){var l=s[o];if((t?l.getFullStart():l.getStart(e,!0))>n)break;var c=l.getEnd();if(n<c||n===c&&(1===l.kind||a)){i=l;continue e}if(r&&c===n){var u=b(n,e,l);if(u&&r(u))return u}}return i}}function b(n,t,r,a){var i=function i(o){if(E(o)&&1!==o.kind)return o;var s=o.getChildren(t);for(var l=0;l<s.length;l++){var c=s[l];if(n<c.end){var u=c.getStart(t,!a),d=u>=n||!k(c,t)||L(c);if(d){var m=S(s,l,t);return m&&T(m,t)}return i(c)}}e.Debug.assert(void 0!==r||277===o.kind||1===o.kind||e.isJSDocCommentContainingNode(o));var p=S(s,s.length,t);return p&&T(p,t)}(r||t);return e.Debug.assert(!(i&&L(i))),i}function E(n){return e.isToken(n)&&!L(n)}function T(e,n){if(E(e))return e;var t=e.getChildren(n),r=S(t,t.length,n);return r&&T(r,n)}function S(n,t,r){for(var a=t-1;a>=0;a--){if(L(n[a]))e.Debug.assert(a>0,\"`JsxText` tokens should not be the first child of `JsxElement | JsxSelfClosingElement`\");else if(k(n[a],r))return n[a]}}function L(n){return e.isJsxText(n)&&n.containsOnlyWhiteSpaces}function x(e,n,t){for(var r=e.kind,a=0;;){var i=b(e.getFullStart(),t);if(!i)return;if((e=i).kind===n){if(0===a)return e;a--}else e.kind===r&&a++}}function A(n,t,r){var a=r.getTypeAtLocation(n);return(e.isNewExpression(n.parent)?a.getConstructSignatures():a.getCallSignatures()).filter(function(e){return!!e.typeParameters&&e.typeParameters.length>=t})}function C(n,t){for(var r=n,a=0,i=0;r;){switch(r.kind){case 27:if(!(r=b(r.getFullStart(),t))||!e.isIdentifier(r))return;if(!a)return e.isDeclarationName(r)?void 0:{called:r,nTypeArguments:i};a--;break;case 47:a=3;break;case 46:a=2;break;case 29:a++;break;case 18:if(!(r=x(r,17,t)))return;break;case 20:if(!(r=x(r,19,t)))return;break;case 22:if(!(r=x(r,21,t)))return;break;case 26:i++;break;case 36:case 71:case 9:case 8:case 101:case 86:case 103:case 85:case 128:case 23:case 49:case 55:case 56:break;default:if(e.isTypeNode(r))break;return}r=b(r.getFullStart(),t)}}function D(n,t,r){return e.formatting.getRangeOfEnclosingComment(n,t,void 0,r)}function k(e,n){return 1===e.kind?!!e.jsDoc:0!==e.getWidth(n)}function M(e,n,t){var r=D(e,n,void 0);return!!r&&t===l.test(e.text.substring(r.pos,r.end))}function O(e,n){return{span:e,newText:n}}function I(e){return!!e.useCaseSensitiveFileNames&&e.useCaseSensitiveFileNames()}function R(n,t,r,a){return e.createImportDeclaration(void 0,void 0,n||t?e.createImportClause(n,t&&t.length?e.createNamedImports(t):void 0):void 0,\"string\"==typeof r?N(r,a):r)}function N(n,t){return e.createLiteral(n,0===t)}function w(n,t){return e.isStringDoubleQuoted(n,t)?1:0}function P(n){return\"default\"!==n.escapedName?n.escapedName:e.firstDefined(n.declarations,function(n){var t=e.getNameOfDeclaration(n);return t&&71===t.kind?t.escapedText:void 0})}function F(n,t,r,a){var i=e.createMap();return function n(o){if(!(96&o.flags&&e.addToSeen(i,e.getSymbolId(o))))return;return e.firstDefined(o.declarations,function(i){return e.firstDefined(e.getAllSuperTypeNodes(i),function(i){var o=r.getTypeAtLocation(i),s=o&&o.symbol&&r.getPropertyOfType(o,t);return o&&s&&(e.firstDefined(r.getRootSymbols(s),a)||n(o.symbol))})})}(n)}function G(n,t,r){return e.textSpanContainsPosition(n,t.getStart(r))&&t.getEnd()<=e.textSpanEnd(n)}function V(e,n){return!!e&&!!n&&e.start===n.start&&e.length===n.length}e.getLineStartPositionForPosition=function(n,t){return e.getLineStarts(t)[t.getLineAndCharacterOfPosition(n).line]},e.rangeContainsRange=c,e.rangeContainsRangeExclusive=function(e,n){return u(e,n.pos)&&u(e,n.end)},e.rangeContainsPosition=function(e,n){return e.pos<=n&&n<=e.end},e.rangeContainsPositionExclusive=u,e.startEndContainsRange=d,e.rangeContainsStartEnd=function(e,n,t){return e.pos<=n&&e.end>=t},e.rangeOverlapsWithStartEnd=function(e,n,t){return m(e.pos,e.end,n,t)},e.nodeOverlapsWithStartEnd=function(e,n,t,r){return m(e.getStart(n),e.end,t,r)},e.startEndOverlapsWithStartEnd=m,e.positionBelongsToNode=function(n,t,r){return e.Debug.assert(n.pos<=t),t<n.end||!function n(t,r){if(void 0===t||e.nodeIsMissing(t))return!1;switch(t.kind){case 238:case 239:case 241:case 186:case 182:case 166:case 216:case 243:case 244:case 250:case 254:return p(t,18,r);case 272:return n(t.block,r);case 190:if(!t.arguments)return!0;case 189:case 193:case 175:return p(t,20,r);case 163:case 164:return n(t.type,r);case 155:case 156:case 157:case 237:case 194:case 154:case 153:case 159:case 158:case 195:return t.body?n(t.body,r):t.type?n(t.type,r):f(t,20,r);case 242:return!!t.body&&n(t.body,r);case 220:return t.elseStatement?n(t.elseStatement,r):n(t.thenStatement,r);case 219:return n(t.expression,r)||f(t,25,r);case 185:case 183:case 188:case 147:case 168:return p(t,22,r);case 160:return t.type?n(t.type,r):f(t,22,r);case 269:case 270:return!1;case 223:case 224:case 225:case 222:return n(t.statement,r);case 221:return f(t,106,r)?p(t,20,r):n(t.statement,r);case 165:return n(t.exprName,r);case 197:case 196:case 198:case 205:case 206:var a=t;return n(a.expression,r);case 191:return n(t.template,r);case 204:var i=e.lastOrUndefined(t.templateSpans);return n(i,r);case 214:return e.nodeIsPresent(t.literal);case 253:case 247:return e.nodeIsPresent(t.moduleSpecifier);case 200:return n(t.operand,r);case 202:return n(t.right,r);case 203:return n(t.whenFalse,r);default:return!0}}(n,r)},e.findListItemInfo=function(n){var t=_(n);if(t){var r=t.getChildren();return{listItemIndex:e.indexOfNode(r,n),list:t}}},e.hasChildOfKind=f,e.findChildOfKind=g,e.findContainingList=_,e.getTouchingPropertyName=function(n,t){return v(n,t,function(n){return e.isPropertyNameLiteral(n)||e.isKeyword(n.kind)})},e.getTouchingToken=v,e.getTokenAtPosition=y,e.findTokenOnLeftOfPosition=function(n,t){var r=y(n,t);return e.isToken(r)&&t>r.getStart(n)&&t<r.getEnd()?r:b(t,n)},e.findNextToken=function(n,t,r){return function t(a){return e.isToken(a)&&a.pos===n.end?a:e.firstDefined(a.getChildren(),function(e){var a=e.pos<=n.pos&&e.end>n.end||e.pos===n.end;return a&&k(e,r)?t(e):void 0})}(t)},e.findPrecedingToken=b,e.isInString=function(n,t,r){if(void 0===r&&(r=b(t,n)),r&&e.isStringTextContainingNode(r)){var a=r.getStart(n),i=r.getEnd();if(a<t&&t<i)return!0;if(t===i)return!!r.isUnterminated}return!1},e.isInsideJsxElementOrAttribute=function(e,n){var t=y(e,n);return!!t&&(10===t.kind||27===t.kind&&10===t.parent.kind||27===t.kind&&268===t.parent.kind||!(!t||18!==t.kind||268!==t.parent.kind)||27===t.kind&&261===t.parent.kind)},e.isInTemplateString=function(n,t){var r=y(n,t);return e.isTemplateLiteralKind(r.kind)&&t>r.getStart(n)},e.findPrecedingMatchingToken=x,e.isPossiblyTypeArgumentPosition=function n(t,r,a){var i=C(t,r);return void 0!==i&&(e.isPartOfTypeNode(i.called)||0!==A(i.called,i.nTypeArguments,a).length||n(i.called,r,a))},e.getPossibleGenericSignatures=A,e.getPossibleTypeArgumentsInfo=C,e.isInComment=D,e.hasDocComment=function(n,t){var r=y(n,t);return!!e.findAncestor(r,e.isJSDoc)},e.getNodeModifiers=function(n){var t=e.isDeclaration(n)?e.getCombinedModifierFlags(n):0,r=[];return 8&t&&r.push(\"private\"),16&t&&r.push(\"protected\"),4&t&&r.push(\"public\"),32&t&&r.push(\"static\"),128&t&&r.push(\"abstract\"),1&t&&r.push(\"export\"),4194304&n.flags&&r.push(\"declare\"),r.length>0?r.join(\",\"):\"\"},e.getTypeArgumentOrTypeParameterList=function(n){return 162===n.kind||189===n.kind?n.typeArguments:e.isFunctionLike(n)||238===n.kind||239===n.kind?n.typeParameters:void 0},e.isComment=function(e){return 2===e||3===e},e.isStringOrRegularExpressionOrTemplateLiteral=function(n){return!(9!==n&&12!==n&&!e.isTemplateLiteralKind(n))},e.isPunctuation=function(e){return 17<=e&&e<=70},e.isInsideTemplateLiteral=function(n,t,r){return e.isTemplateLiteralKind(n.kind)&&n.getStart(r)<t&&t<n.end||!!n.isUnterminated&&t===n.end},e.isAccessibilityModifier=function(e){switch(e){case 114:case 112:case 113:return!0}return!1},e.cloneCompilerOptions=function(n){var t=e.clone(n);return e.setConfigFileInOptions(t,n&&n.configFile),t},e.isArrayLiteralOrObjectLiteralDestructuringPattern=function e(n){if(185===n.kind||186===n.kind){if(202===n.parent.kind&&n.parent.left===n&&58===n.parent.operatorToken.kind)return!0;if(225===n.parent.kind&&n.parent.initializer===n)return!0;if(e(273===n.parent.kind?n.parent.parent:n.parent))return!0}return!1},e.isInReferenceComment=function(e,n){return M(e,n,!0)},e.isInNonReferenceComment=function(e,n){return M(e,n,!1)},e.createTextSpanFromNode=function(n,t){return e.createTextSpanFromBounds(n.getStart(t),n.getEnd())},e.createTextRangeFromNode=function(n,t){return e.createRange(n.getStart(t),n.end)},e.createTextSpanFromRange=function(n){return e.createTextSpanFromBounds(n.pos,n.end)},e.createTextRangeFromSpan=function(n){return e.createRange(n.start,n.start+n.length)},e.createTextChangeFromStartLength=function(n,t,r){return O(e.createTextSpan(n,t),r)},e.createTextChange=O,e.typeKeywords=[119,122,86,128,131,95,134,135,137,138,101,105,140,141,142],e.isTypeKeyword=function(n){return e.contains(e.typeKeywords,n)},e.isExternalModuleSymbol=function(e){return!!(1536&e.flags)&&34===e.name.charCodeAt(0)},e.nodeSeenTracker=function(){var n=[];return function(t){var r=e.getNodeId(t);return!n[r]&&(n[r]=!0)}},e.getSnapshotText=function(e){return e.getText(0,e.getLength())},e.repeatString=function(e,n){for(var t=\"\",r=0;r<n;r++)t+=e;return t},e.skipConstraint=function(e){return e.isTypeParameter()&&e.getConstraint()||e},e.getNameFromPropertyName=function(n){return 147===n.kind?e.isStringOrNumericLiteralLike(n.expression)?n.expression.text:void 0:e.getTextOfIdentifierOrLiteral(n)},e.programContainsEs6Modules=function(e){return e.getSourceFiles().some(function(n){return!n.isDeclarationFile&&!e.isSourceFileFromExternalLibrary(n)&&!!n.externalModuleIndicator})},e.compilerOptionsIndicateEs6Modules=function(e){return!!e.module||e.target>=2||!!e.noEmit},e.hostUsesCaseSensitiveFileNames=I,e.hostGetCanonicalFileName=function(n){return e.createGetCanonicalFileName(I(n))},e.makeImportIfNecessary=function(e,n,t,r){return e||n&&n.length?R(e,n,t,r):void 0},e.makeImport=R,e.makeStringLiteral=N,function(e){e[e.Single=0]=\"Single\",e[e.Double=1]=\"Double\"}(e.QuotePreference||(e.QuotePreference={})),e.quotePreferenceFromString=w,e.getQuotePreference=function(n,t){if(t.quotePreference)return\"single\"===t.quotePreference?0:1;var r=n.imports&&e.find(n.imports,e.isStringLiteral);return r?w(r,n):1},e.getQuoteFromPreference=function(n){switch(n){case 0:return\"'\";case 1:return'\"';default:return e.Debug.assertNever(n)}},e.symbolNameNoDefault=function(n){var t=P(n);return void 0===t?void 0:e.unescapeLeadingUnderscores(t)},e.symbolEscapedNameNoDefault=P,e.getPropertySymbolFromBindingElement=function(n,t){var r=n.getTypeAtLocation(t.parent),a=r&&n.getPropertyOfType(r,t.name.text);return a&&98304&a.flags?(e.Debug.assert(!!(33554432&a.flags)),a.target):a},e.getPropertySymbolsFromBaseTypes=F,e.isMemberSymbolInBaseType=function(e,n){return F(e.parent,e.name,n,function(e){return!0})||!1},e.getParentNodeInSpan=function(n,t,r){if(n)for(;n.parent;){if(e.isSourceFile(n.parent)||!G(r,n.parent,t))return n;n=n.parent}},e.findModifier=function(n,t){return n.modifiers&&e.find(n.modifiers,function(e){return e.kind===t})},e.insertImport=function(n,t,r){var a=e.findLast(t.statements,e.isAnyImportSyntax);a?n.insertNodeAfter(t,a,r):n.insertNodeAtTopOfFile(t,r,!0)},e.textSpansEqual=V,e.documentSpansEqual=function(e,n){return e.fileName===n.fileName&&V(e.textSpan,n.textSpan)}}(Tn||(Tn={})),function(e){function n(e){return e.declarations&&e.declarations.length>0&&149===e.declarations[0].kind}e.isFirstDeclarationOfSymbolParameter=n;var t=function(){var n,t,i,o,s=10*e.defaultMaximumTruncationLength;m();var c=function(n){return d(n,e.SymbolDisplayPartKind.text)};return{displayParts:function(){var t=n.length&&n[n.length-1].text;return o>s&&t&&\"...\"!==t&&(e.isWhiteSpaceLike(t.charCodeAt(t.length-1))||n.push(a(\" \",e.SymbolDisplayPartKind.space)),n.push(a(\"...\",e.SymbolDisplayPartKind.punctuation))),n},writeKeyword:function(n){return d(n,e.SymbolDisplayPartKind.keyword)},writeOperator:function(n){return d(n,e.SymbolDisplayPartKind.operator)},writePunctuation:function(n){return d(n,e.SymbolDisplayPartKind.punctuation)},writeSpace:function(n){return d(n,e.SymbolDisplayPartKind.space)},writeStringLiteral:function(n){return d(n,e.SymbolDisplayPartKind.stringLiteral)},writeParameter:function(n){return d(n,e.SymbolDisplayPartKind.parameterName)},writeProperty:function(n){return d(n,e.SymbolDisplayPartKind.propertyName)},writeLiteral:function(n){return d(n,e.SymbolDisplayPartKind.stringLiteral)},writeSymbol:function(e,t){if(o>s)return;u(),o+=e.length,n.push(r(e,t))},writeLine:function(){if(o>s)return;o+=1,n.push(l()),t=!0},write:c,writeTextOfNode:c,getText:function(){return\"\"},getTextPos:function(){return 0},getColumn:function(){return 0},getLine:function(){return 0},isAtStartOfLine:function(){return!1},rawWrite:e.notImplemented,getIndent:function(){return i},increaseIndent:function(){i++},decreaseIndent:function(){i--},clear:m,trackSymbol:e.noop,reportInaccessibleThisError:e.noop,reportInaccessibleUniqueSymbolError:e.noop,reportPrivateInBaseOfClassExpression:e.noop};function u(){if(!(o>s)&&t){var r=e.getIndentString(i);r&&(o+=r.length,n.push(a(r,e.SymbolDisplayPartKind.space))),t=!1}}function d(e,t){o>s||(u(),o+=e.length,n.push(a(e,t)))}function m(){n=[],t=!0,i=0,o=0}}();function r(t,r){return a(t,function(t){var r=t.flags;if(3&r)return n(t)?e.SymbolDisplayPartKind.parameterName:e.SymbolDisplayPartKind.localName;if(4&r)return e.SymbolDisplayPartKind.propertyName;if(32768&r)return e.SymbolDisplayPartKind.propertyName;if(65536&r)return e.SymbolDisplayPartKind.propertyName;if(8&r)return e.SymbolDisplayPartKind.enumMemberName;if(16&r)return e.SymbolDisplayPartKind.functionName;if(32&r)return e.SymbolDisplayPartKind.className;if(64&r)return e.SymbolDisplayPartKind.interfaceName;if(384&r)return e.SymbolDisplayPartKind.enumName;if(1536&r)return e.SymbolDisplayPartKind.moduleName;if(8192&r)return e.SymbolDisplayPartKind.methodName;if(262144&r)return e.SymbolDisplayPartKind.typeParameterName;if(524288&r)return e.SymbolDisplayPartKind.aliasName;if(2097152&r)return e.SymbolDisplayPartKind.aliasName;return e.SymbolDisplayPartKind.text}(r))}function a(n,t){return{text:n,kind:e.SymbolDisplayPartKind[t]}}function i(n){return a(e.tokenToString(n),e.SymbolDisplayPartKind.keyword)}function o(n){return a(n,e.SymbolDisplayPartKind.text)}e.symbolPart=r,e.displayPart=a,e.spacePart=function(){return a(\" \",e.SymbolDisplayPartKind.space)},e.keywordPart=i,e.punctuationPart=function(n){return a(e.tokenToString(n),e.SymbolDisplayPartKind.punctuation)},e.operatorPart=function(n){return a(e.tokenToString(n),e.SymbolDisplayPartKind.operator)},e.textOrKeywordPart=function(n){var t=e.stringToToken(n);return void 0===t?o(n):i(t)},e.textPart=o;var s=\"\\r\\n\";function l(){return a(\"\\n\",e.SymbolDisplayPartKind.lineBreak)}function c(e){try{return e(t),t.displayParts()}finally{t.clear()}}function u(n){return e.isSingleOrDoubleQuote(n.charCodeAt(0))}function d(n,t){return e.ensureScriptKind(n,t&&t.getScriptKind&&t.getScriptKind(n))}function m(e,n){void 0===n&&(n=!0);var t=e&&f(e);return t&&!n&&g(t),t}function p(n,t,r,a,i){var o;if(void 0===t&&(t=!0),e.isIdentifier(n)&&r&&a){var s=a.getSymbolAtLocation(n),l=s&&r.get(String(e.getSymbolId(s)));l&&(o=e.createIdentifier(l.text))}return o||(o=f(n,r,a,i)),o&&!t&&g(o),i&&o&&i(n,o),o}function f(n,t,r,a){var i=t||r||a?e.visitEachChild(n,function(e){return p(e,!0,t,r,a)},e.nullTransformationContext):e.visitEachChild(n,m,e.nullTransformationContext);if(i===n){var o=e.getSynthesizedClone(n);return e.isStringLiteral(o)?o.textSourceNode=n:e.isNumericLiteral(o)&&(o.numericLiteralFlags=n.numericLiteralFlags),e.setTextRange(o,n)}return i.parent=void 0,i}function g(e){_(e),v(e)}function _(e){y(e,512,h)}function v(n){y(n,1024,e.getLastChild)}function y(n,t,r){e.addEmitFlags(n,t);var a=r(n);a&&y(a,t,r)}function h(e){return e.forEachChild(function(e){return e})}function b(n,t){if(e.startsWith(n,t))return 0;var r=n.indexOf(\" \"+t);return-1===r&&(r=n.indexOf(\".\"+t)),-1===r&&(r=n.indexOf('\"'+t)),-1===r?-1:r+1}e.getNewLineOrDefaultFromHost=function(e,n){return n&&n.newLineCharacter||e.getNewLine&&e.getNewLine()||s},e.lineBreakPart=l,e.mapToDisplayParts=c,e.typeToDisplayParts=function(e,n,t,r){return void 0===r&&(r=0),c(function(a){e.writeType(n,t,17408|r,a)})},e.symbolToDisplayParts=function(e,n,t,r,a){return void 0===a&&(a=0),c(function(i){e.writeSymbol(n,t,r,8|a,i)})},e.signatureToDisplayParts=function(e,n,t,r){return void 0===r&&(r=0),r|=25632,c(function(a){e.writeSignature(n,t,r,void 0,a)})},e.isImportOrExportSpecifierName=function(e){return!!e.parent&&(251===e.parent.kind||255===e.parent.kind)&&e.parent.propertyName===e},e.stripQuotes=function(e){var n=e.length;return n>=2&&e.charCodeAt(0)===e.charCodeAt(n-1)&&u(e)?e.substring(1,n-1):e},e.startsWithQuote=u,e.scriptKindIs=function(n,t){for(var r=[],a=2;a<arguments.length;a++)r[a-2]=arguments[a];var i=d(n,t);return e.some(r,function(e){return e===i})},e.getScriptKind=d,e.getUniqueSymbolId=function(n,t){return e.getSymbolId(e.skipAlias(n,t))},e.getFirstNonSpaceCharacterPosition=function(n,t){for(;e.isWhiteSpaceLike(n.charCodeAt(t));)t+=1;return t},e.getSynthesizedDeepClone=m,e.getSynthesizedDeepCloneWithRenames=p,e.getSynthesizedDeepClones=function(n,t){return void 0===t&&(t=!0),n&&e.createNodeArray(n.map(function(e){return m(e,t)}),n.hasTrailingComma)},e.suppressLeadingAndTrailingTrivia=g,e.suppressLeadingTrivia=_,e.suppressTrailingTrivia=v,e.getUniqueName=function(n,t){for(var r=n,a=1;!e.isFileLevelUniqueName(t,r);a++)r=n+\"_\"+a;return r},e.getRenameLocation=function(n,t,r,a){for(var i=0,o=-1,s=0,l=n;s<l.length;s++){var c=l[s],u=c.fileName,d=c.textChanges;e.Debug.assert(u===t);for(var m=0,p=d;m<p.length;m++){var f=p[m],g=f.span,_=f.newText,v=b(_,r);if(-1!==v&&(o=g.start+i+v,!a))return o;i+=_.length-g.length}}return e.Debug.assert(a),e.Debug.assert(o>=0),o},e.copyComments=function(n,t,r,a,i){e.forEachLeadingCommentRange(r.text,n.pos,function(n,o,s,l){3===s?(n+=2,o-=2):n+=2,e.addSyntheticLeadingComment(t,a||s,r.text.slice(n,o),void 0!==i?i:l)})}}(Tn||(Tn={})),function(e){e.createClassifier=function(){var o=e.createScanner(6,!1);function s(a,s,l){var c=0,u=0,d=[],m=function(n){switch(n){case 3:return{prefix:'\"\\\\\\n'};case 2:return{prefix:\"'\\\\\\n\"};case 1:return{prefix:\"/*\\n\"};case 4:return{prefix:\"`\\n\"};case 5:return{prefix:\"}\\n\",pushTemplate:!0};case 6:return{prefix:\"\",pushTemplate:!0};case 0:return{prefix:\"\"};default:return e.Debug.assertNever(n)}}(s),p=m.prefix,f=m.pushTemplate;a=p+a;var g=p.length;f&&d.push(14),o.setText(a);var _=0,v=[],y=0;do{c=o.scan(),e.isTrivia(c)||(E(),u=c);var h=o.getTextPos();if(r(o.getTokenPos(),h,g,i(c),v),h>=a.length){var b=t(o,c,e.lastOrUndefined(d));void 0!==b&&(_=b)}}while(1!==c);function E(){switch(c){case 41:case 63:n[u]||12!==o.reScanSlashToken()||(c=12);break;case 27:71===u&&y++;break;case 29:y>0&&y--;break;case 119:case 137:case 134:case 122:case 138:y>0&&!l&&(c=71);break;case 14:d.push(c);break;case 17:d.length>0&&d.push(c);break;case 18:if(d.length>0){var t=e.lastOrUndefined(d);14===t?16===(c=o.reScanTemplateToken())?d.pop():e.Debug.assertEqual(c,15,\"Should have been a template middle.\"):(e.Debug.assertEqual(t,17,\"Should have been an open brace\"),d.pop())}break;default:if(!e.isKeyword(c))break;23===u?c=71:e.isKeyword(u)&&e.isKeyword(c)&&!function(n,t){if(!e.isAccessibilityModifier(n))return!0;switch(t){case 125:case 136:case 123:case 115:return!0;default:return!1}}(u,c)&&(c=71)}}return{endOfLineState:_,spans:v}}return{getClassificationsForLine:function(n,t,r){return function(n,t){for(var r=[],i=n.spans,o=0,s=0;s<i.length;s+=3){var l=i[s],c=i[s+1],u=i[s+2];if(o>=0){var d=l-o;d>0&&r.push({length:d,classification:e.TokenClass.Whitespace})}r.push({length:c,classification:a(u)}),o=l+c}var m=t.length-o;return m>0&&r.push({length:m,classification:e.TokenClass.Whitespace}),{entries:r,finalLexState:n.endOfLineState}}(s(n,t,r),n)},getEncodedLexicalClassifications:s}};var n=e.arrayToNumericMap([71,9,8,12,99,43,44,20,22,18,101,86],function(e){return e},function(){return!0});function t(n,t,r){switch(t){case 9:if(!n.isUnterminated())return;for(var a=n.getTokenText(),i=a.length-1,o=0;92===a.charCodeAt(i-o);)o++;if(0==(1&o))return;return 34===a.charCodeAt(0)?3:2;case 3:return n.isUnterminated()?1:void 0;default:if(e.isTemplateLiteralKind(t)){if(!n.isUnterminated())return;switch(t){case 16:return 5;case 13:return 4;default:return e.Debug.fail(\"Only 'NoSubstitutionTemplateLiteral's and 'TemplateTail's can be unterminated; got SyntaxKind #\"+t)}}return 14===r?6:void 0}}function r(e,n,t,r,a){if(8!==r){0===e&&t>0&&(e+=t);var i=n-e;i>0&&a.push(e-t,i,r)}}function a(n){switch(n){case 1:return e.TokenClass.Comment;case 3:return e.TokenClass.Keyword;case 4:return e.TokenClass.NumberLiteral;case 5:return e.TokenClass.Operator;case 6:return e.TokenClass.StringLiteral;case 8:return e.TokenClass.Whitespace;case 10:return e.TokenClass.Punctuation;case 2:case 11:case 12:case 13:case 14:case 15:case 16:case 9:case 17:return e.TokenClass.Identifier;default:return}}function i(n){if(e.isKeyword(n))return 3;if(function(e){switch(e){case 39:case 41:case 42:case 37:case 38:case 45:case 46:case 47:case 27:case 29:case 30:case 31:case 93:case 92:case 118:case 32:case 33:case 34:case 35:case 48:case 50:case 49:case 53:case 54:case 69:case 68:case 70:case 65:case 66:case 67:case 59:case 60:case 61:case 63:case 64:case 58:case 26:return!0;default:return!1}}(n)||function(e){switch(e){case 37:case 38:case 52:case 51:case 43:case 44:return!0;default:return!1}}(n))return 5;if(n>=17&&n<=70)return 10;switch(n){case 8:return 4;case 9:return 6;case 12:return 7;case 7:case 3:case 2:return 1;case 5:case 4:return 8;case 71:default:return e.isTemplateLiteralKind(n)?6:2}}function o(e,n){switch(n){case 242:case 238:case 239:case 237:e.throwIfCancellationRequested()}}function s(n,t,r,a,i){var s=[];return r.forEachChild(function l(c){if(c&&e.textSpanIntersectsWith(i,c.pos,c.getFullWidth())){if(o(t,c.kind),e.isIdentifier(c)&&!e.nodeIsMissing(c)&&a.has(c.escapedText)){var u=n.getSymbolAtLocation(c),d=u&&function n(t,r,a){var i=t.getFlags();return 0==(2885600&i)?void 0:32&i?11:384&i?12:524288&i?16:1536&i?4&r||1&r&&function(n){return e.some(n.declarations,function(n){return e.isModuleDeclaration(n)&&1===e.getModuleInstanceState(n)})}(t)?14:void 0:2097152&i?n(a.getAliasedSymbol(t),r,a):2&r?64&i?13:262144&i?15:void 0:void 0}(u,e.getMeaningFromLocation(c),n);d&&function(e,n,t){s.push(e),s.push(n-e),s.push(t)}(c.getStart(r),c.getEnd(),d)}c.forEachChild(l)}}),{spans:s,endOfLineState:0}}function l(e){switch(e){case 1:return\"comment\";case 2:return\"identifier\";case 3:return\"keyword\";case 4:return\"number\";case 5:return\"operator\";case 6:return\"string\";case 8:return\"whitespace\";case 9:return\"text\";case 10:return\"punctuation\";case 11:return\"class name\";case 12:return\"enum name\";case 13:return\"interface name\";case 14:return\"module name\";case 15:return\"type parameter name\";case 16:return\"type alias name\";case 17:return\"parameter name\";case 18:return\"doc comment tag name\";case 19:return\"jsx open tag name\";case 20:return\"jsx close tag name\";case 21:return\"jsx self closing tag name\";case 22:return\"jsx attribute\";case 23:return\"jsx text\";case 24:return\"jsx attribute string literal value\";default:return}}function c(n){e.Debug.assert(n.spans.length%3==0);for(var t=n.spans,r=[],a=0;a<t.length;a+=3)r.push({textSpan:e.createTextSpan(t[a],t[a+1]),classificationType:l(t[a+2])});return r}function u(n,t,r){var a=r.start,i=r.length,s=e.createScanner(6,!1,t.languageVariant,t.text),l=e.createScanner(6,!1,t.languageVariant,t.text),c=[];return y(t),{spans:c,endOfLineState:0};function u(e,n,t){c.push(e),c.push(n),c.push(t)}function d(n,r,a,i){if(3===r){var o=e.parseIsolatedJSDocComment(t.text,a,i);if(o&&o.jsDoc)return o.jsDoc.parent=n,void function(e){var n=e.pos;if(e.tags)for(var t=0,r=e.tags;t<r.length;t++){var a=r[t];switch(a.pos!==n&&m(n,a.pos-n),u(a.atToken.pos,a.atToken.end-a.atToken.pos,10),u(a.tagName.pos,a.tagName.end-a.tagName.pos,18),n=a.tagName.end,a.kind){case 297:i(a);break;case 301:p(a),n=a.end;break;case 300:case 298:y(a.typeExpression),n=a.end}}n!==e.end&&m(n,e.end-n);return;function i(e){e.isNameFirst&&(m(n,e.name.pos-n),u(e.name.pos,e.name.end-e.name.pos,17),n=e.name.end),e.typeExpression&&(m(n,e.typeExpression.pos-n),y(e.typeExpression),n=e.typeExpression.end),e.isNameFirst||(m(n,e.name.pos-n),u(e.name.pos,e.name.end-e.name.pos,17),n=e.name.end)}}(o.jsDoc)}m(a,i)}function m(e,n){u(e,n,1)}function p(e){for(var n=0,t=e.getChildren();n<t.length;n++){y(t[n])}}function f(n,t,r){var a;for(a=t;a<r&&!e.isLineBreak(n.charCodeAt(a));a++);for(u(t,a-t,1),l.setTextPos(a);l.getTextPos()<r;)g()}function g(){var e=l.getTextPos(),n=l.scan(),t=l.getTextPos(),r=v(n);r&&u(e,t-e,r)}function _(n){if(e.isJSDoc(n))return!0;if(e.nodeIsMissing(n))return!0;var r=function(e){switch(e.parent&&e.parent.kind){case 260:if(e.parent.tagName===e)return 19;break;case 261:if(e.parent.tagName===e)return 20;break;case 259:if(e.parent.tagName===e)return 21;break;case 265:if(e.parent.name===e)return 22}return}(n);if(!e.isToken(n)&&10!==n.kind&&void 0===r)return!1;var a=10===n.kind?n.pos:function(n){for(s.setTextPos(n.pos);;){var r=s.getTextPos();if(!e.couldStartTrivia(t.text,r))return r;var a=s.scan(),i=s.getTextPos(),o=i-r;if(!e.isTrivia(a))return r;switch(a){case 4:case 5:continue;case 2:case 3:d(n,a,r,o),s.setTextPos(i);continue;case 7:var l=t.text,c=l.charCodeAt(r);if(60===c||62===c){u(r,o,1);continue}e.Debug.assert(124===c||61===c),f(l,r,i);break;case 6:break;default:e.Debug.assertNever(a)}}}(n),i=n.end-a;if(e.Debug.assert(i>=0),i>0){var o=r||v(n.kind,n);o&&u(a,i,o)}return!0}function v(n,t){if(e.isKeyword(n))return 3;if((27===n||29===n)&&t&&e.getTypeArgumentOrTypeParameterList(t.parent))return 10;if(e.isPunctuation(n)){if(t){var r=t.parent;if(58===n&&(235===r.kind||152===r.kind||149===r.kind||265===r.kind))return 5;if(202===r.kind||200===r.kind||201===r.kind||203===r.kind)return 5}return 10}if(8===n)return 4;if(9===n)return 265===t.parent.kind?24:6;if(12===n)return 6;if(e.isTemplateLiteralKind(n))return 6;if(10===n)return 23;if(71===n){if(t)switch(t.parent.kind){case 238:return t.parent.name===t?11:void 0;case 148:return t.parent.name===t?15:void 0;case 239:return t.parent.name===t?13:void 0;case 241:return t.parent.name===t?12:void 0;case 242:return t.parent.name===t?14:void 0;case 149:return t.parent.name===t?e.isThisIdentifier(t)?3:17:void 0}return 2}}function y(r){if(r&&e.decodedTextSpanIntersectsWith(a,i,r.pos,r.getFullWidth())){o(n,r.kind);for(var s=0,l=r.getChildren(t);s<l.length;s++){var c=l[s];_(c)||y(c)}}}}e.getSemanticClassifications=function(e,n,t,r,a){return c(s(e,n,t,r,a))},e.getEncodedSemanticClassifications=s,e.getSyntacticClassifications=function(e,n,t){return c(u(e,n,t))},e.getEncodedSyntacticClassifications=u}(Tn||(Tn={})),function(e){!function(n){!function(n){function t(e,n){return{name:e,kind:n}}function r(n,t,r){var a=function(n,t){var r=Math.max(n.lastIndexOf(e.directorySeparator),n.lastIndexOf(\"\\\\\")),a=-1!==r?r+1:0,i=n.length-a;return 0===i||e.isIdentifierText(n.substr(a,i),6)?void 0:e.createTextSpan(t+a,i)}(n,t);return r.map(function(e){return{name:e.name,kind:e.kind,span:a}})}function a(n){var t=e.getSupportedExtensions(n);return n.resolveJsonModule&&e.getEmitModuleResolutionKind(n)===e.ModuleResolutionKind.NodeJs?t.concat(\".json\"):t}function i(n,r,a,i,s,l,c){void 0===c&&(c=[]),void 0===n&&(n=\"\"),n=e.normalizeSlashes(n),e.hasTrailingDirectorySeparator(n)||(n=e.getDirectoryPath(n)),\"\"===n&&(n=\".\"+e.directorySeparator),n=e.ensureTrailingDirectorySeparator(n);var u=e.resolvePath(r,n),d=e.hasTrailingDirectorySeparator(u)?u:e.getDirectoryPath(u),_=!(s.useCaseSensitiveFileNames&&s.useCaseSensitiveFileNames());if(g(s,d)){var v=p(s,d,a,void 0,[\"./*\"]);if(v){for(var y=e.createMap(),h=0,b=v;h<b.length;h++){var E=b[h];if(E=e.normalizePath(E),!l||0!==e.comparePaths(E,l,r,_)){var T=i||e.fileExtensionIs(E,\".json\")?e.getBaseFileName(E):e.removeFileExtension(e.getBaseFileName(E));y.has(T)||y.set(T,!0)}}e.forEachKey(y,function(e){c.push(t(e,\"script\"))})}var S=m(s,d);if(S)for(var L=0,x=S;L<x.length;L++){var A=x[L],C=e.getBaseFileName(e.normalizePath(A));\"@types\"!==C&&c.push(t(C,\"directory\"))}var D=function(n,t){var r;return e.forEachAncestorDirectory(n,function(n){return\"node_modules\"===n||!!(r=e.findConfigFile(n,function(e){return f(t,e)},\"package.json\"))||void 0}),r}(d,s);if(D){var k=e.readJson(D,s).typesVersions;if(\"object\"==typeof k){var M=e.getPackageJsonTypesVersionsPaths(k),O=M&&M.paths,I=u.slice(e.ensureTrailingDirectorySeparator(d).length);O&&o(c,I,d,a,O,s)}}}return c}function o(n,r,a,i,o,l){for(var c in o)if(e.hasProperty(o,c)){var u=o[c];if(u)for(var d=function(e,r){n.some(function(n){return n.name===e})||n.push(t(e,r))},m=0,p=s(c,u,r,a,i,l);m<p.length;m++){var f=p[m];d(f.name,f.kind)}}}function s(n,t,r,a,i,o){if(!e.endsWith(n,\"*\"))return e.stringContains(n,\"*\")?e.emptyArray:c(n);var s=n.slice(0,n.length-1),l=e.tryRemovePrefix(r,s);return void 0===l?c(s):e.flatMap(t,function(n){return function(n,t,r,a,i){if(!i.readDirectory)return;var o=e.hasZeroOrOneAsteriskCharacter(r)?e.tryParsePattern(r):void 0;if(!o)return;var s=e.resolvePath(o.prefix),l=e.hasTrailingDirectorySeparator(o.prefix)?s:e.getDirectoryPath(s),c=e.hasTrailingDirectorySeparator(o.prefix)?\"\":e.getBaseFileName(s),u=v(n),d=u?e.hasTrailingDirectorySeparator(n)?n:e.getDirectoryPath(n):void 0,f=u?e.combinePaths(l,c+d):l,g=e.normalizePath(o.suffix),_=e.normalizePath(e.combinePaths(t,f)),y=u?_:e.ensureTrailingDirectorySeparator(_)+c,h=p(i,_,a,void 0,[g?\"**/*\":\"./*\"]).map(function(e){return{name:e,kind:\"script\"}}),b=m(i,_).map(function(n){return e.combinePaths(_,n)}).map(function(e){return{name:e,kind:\"directory\"}});return e.mapDefined(e.concatenate(h,b),function(n){var t,r,a,i,o=n.name,s=n.kind,l=e.normalizePath(o),c=(t=l,r=y,a=g,e.startsWith(t,r)&&e.endsWith(t,a)?t.slice(r.length,t.length-a.length):void 0);return void 0!==c?{name:(i=e.removeFileExtension(c),i[0]===e.directorySeparator?i.slice(1):i),kind:s}:void 0})}(l,a,n,i,o)});function c(n){return e.startsWith(n,r)?[{name:n,kind:\"directory\"}]:e.emptyArray}}function l(n,r,a,i){void 0===i&&(i=[]);var o=e.createMap();if(r.types)for(var s=0,l=r.types;s<l.length;s++){var u=l[s];b(e.unmangleScopedPackageName(u))}else if(n.getDirectories){var d=void 0;try{d=e.getEffectiveTypeRoots(r,n)}catch(e){}if(d)for(var p=0,f=d;p<f.length;p++){h(f[p])}for(var _=0,v=c(a,n);_<v.length;_++){var y=v[_];h(e.combinePaths(e.getDirectoryPath(y),\"node_modules/@types\"))}}return i;function h(t){if(e.Debug.assert(!!n.getDirectories),g(n,t)){var r=m(n,t);if(r)for(var a=0,i=r;a<i.length;a++){var o=i[a];o=e.normalizePath(o);var s=e.getBaseFileName(o);b(e.unmangleScopedPackageName(s))}}}function b(e){o.has(e)||(i.push(t(e,\"external module name\")),o.set(e,!0))}}function c(n,t){var r=[];return e.forEachAncestorDirectory(n,function(n){var a=e.findConfigFile(n,function(e){return f(t,e)},\"package.json\");if(!a)return!0;r.push(a)}),r}n.getStringLiteralCompletionsFromModuleNames=function(n,s,u,m,p){return r(s.text,s.getStart(n)+1,function(n,r,s,u,m){var p=e.normalizeSlashes(r.text),f=n.path,_=e.getDirectoryPath(f);if(function(e){if(e&&e.length>=2&&46===e.charCodeAt(0)){var n=e.length>=3&&46===e.charCodeAt(1)?2:1,t=e.charCodeAt(n);return 47===t||92===t}return!1}(p)||e.isRootedDiskPath(p)){var y=a(s);return s.rootDirs?function(n,t,r,a,o,s,l,c){for(var u=s.project||l.getCurrentDirectory(),d=!(l.useCaseSensitiveFileNames&&l.useCaseSensitiveFileNames()),m=function(n,t,r,a){n=n.map(function(n){return e.normalizePath(e.isRootedDiskPath(n)?n:e.combinePaths(t,n))});var i=e.firstDefined(n,function(n){return e.containsPath(n,r,t,a)?r.substr(n.length):void 0});return e.deduplicate(n.map(function(n){return e.combinePaths(n,i)}),e.equateStringsCaseSensitive,e.compareStringsCaseSensitive)}(n,u,r,d),p=[],f=0,g=m;f<g.length;f++){var _=g[f];i(t,_,a,o,l,c,p)}return p}(s.rootDirs,p,_,y,!1,s,u,f):i(p,_,y,!1,u,f)}return function(n,r,s,u,m){var p=s.baseUrl,f=s.paths,_=[],y=a(s);if(p){var h=s.project||u.getCurrentDirectory(),b=e.normalizePath(e.combinePaths(h,p));i(n,b,y,!1,u,void 0,_),f&&o(_,n,b,y,f,u)}for(var E=v(n)?e.hasTrailingDirectorySeparator(n)?n:e.getDirectoryPath(n):void 0,T=0,S=function(n,t,r){var a=r.getAmbientModules().map(function(n){return e.stripQuotes(n.name)}).filter(function(t){return e.startsWith(t,n)});if(void 0!==t){var i=e.ensureTrailingDirectorySeparator(t);return a.map(function(n){return e.removePrefix(n,i)})}return a}(n,E,m);T<S.length;T++){var L=S[T];_.push(t(L,\"external module name\"))}if(l(u,s,r,_),e.getEmitModuleResolutionKind(s)===e.ModuleResolutionKind.NodeJs){var x=!1;if(void 0===E)for(var A=function(e){_.some(function(n){return n.name===e})||(x=!0,_.push(t(e,\"external module name\")))},C=0,D=function(n,t){if(!n.readFile||!n.fileExists)return e.emptyArray;for(var r=[],a=0,i=c(t,n);a<i.length;a++)for(var o=i[a],s=e.readJson(o,n),l=0,u=d;l<u.length;l++){var m=u[l],p=s[m];if(p)for(var f in p)p.hasOwnProperty(f)&&!e.startsWith(f,\"@types/\")&&r.push(f)}return r}(u,r);C<D.length;C++){var k=D[C];A(k)}x||e.forEachAncestorDirectory(r,function(t){var r=e.combinePaths(t,\"node_modules\");g(u,r)&&i(n,r,y,!1,u,void 0,_)})}return _}(p,_,s,u,m)}(n,s,u,m,p))},n.getTripleSlashReferenceCompletion=function(n,t,a,o){var s=e.getTokenAtPosition(n,t),c=e.getLeadingCommentRanges(n.text,s.pos),d=c&&e.find(c,function(e){return t>=e.pos&&t<=e.end});if(d){var m=n.text.slice(d.pos,t),p=u.exec(m);if(p){var f=p[1],g=p[2],_=p[3],v=e.getDirectoryPath(n.path),y=\"path\"===g?i(_,v,e.getSupportedExtensions(a),!0,o,n.path):\"types\"===g?l(o,a,v):void 0;return y&&r(_,d.pos+f.length,y)}}};var u=/^(\\/\\/\\/\\s*<reference\\s+(path|types)\\s*=\\s*(?:'|\"))([^\\3\"]*)$/,d=[\"dependencies\",\"devDependencies\",\"peerDependencies\",\"optionalDependencies\"];function m(e,n){return _(e,e.getDirectories,n)||[]}function p(n,t,r,a,i){return _(n,n.readDirectory,t,r,a,i)||e.emptyArray}function f(e,n){return _(e,e.fileExists,n)}function g(n,t){try{return e.directoryProbablyExists(t,n)}catch(e){}return!1}function _(e,n){for(var t=[],r=2;r<arguments.length;r++)t[r-2]=arguments[r];try{return n&&n.apply(e,t)}catch(e){}}function v(n){return e.stringContains(n,e.directorySeparator)}}(n.PathCompletions||(n.PathCompletions={}))}(e.Completions||(e.Completions={}))}(Tn||(Tn={})),function(e){!function(n){var t,r,a;function i(e){return 2===e.kind||3===e.kind}function o(e){return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:!0,entries:e.map(function(e){return{name:e.name,kind:e.kind,kindModifiers:\"\",sortText:\"0\",replacementSpan:e.span}})}}function s(e){return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:!1,entries:e}}function l(n,t){return e.isSourceFileJS(n)&&!e.isCheckJsEnabledForFile(n,t)}!function(e){e[e.ThisType=0]=\"ThisType\",e[e.SymbolMemberNoExport=1]=\"SymbolMemberNoExport\",e[e.SymbolMemberExport=2]=\"SymbolMemberExport\",e[e.Export=3]=\"Export\"}(t||(t={})),function(e){e[e.None=0]=\"None\",e[e.All=1]=\"All\",e[e.ClassElementKeywords=2]=\"ClassElementKeywords\",e[e.InterfaceElementKeywords=3]=\"InterfaceElementKeywords\",e[e.ConstructorParameterKeywords=4]=\"ConstructorParameterKeywords\",e[e.FunctionLikeBodyKeywords=5]=\"FunctionLikeBodyKeywords\",e[e.TypeKeywords=6]=\"TypeKeywords\"}(r||(r={})),function(e){e[e.Continue=0]=\"Continue\",e[e.Success=1]=\"Success\",e[e.Fail=2]=\"Fail\"}(a||(a={})),n.getCompletionsAtPosition=function(t,r,a,i,c,u,d){var m=r.getTypeChecker(),f=r.getCompilerOptions();if(e.isInReferenceComment(i,c)){var g=n.PathCompletions.getTripleSlashReferenceCompletion(i,c,f,t);return g&&o(g)}var _=e.findPrecedingToken(c,i);if(!d||_&&function(n,t,r,a){switch(t){case\".\":case\"@\":return!0;case'\"':case\"'\":case\"`\":return function(e){switch(e.kind){case 9:case 13:case 204:case 191:return!0;default:return!1}}(r)&&a===r.getStart(n)+1;case\"<\":return 27===r.kind&&(!e.isBinaryExpression(r.parent)||B(r.parent));case\"/\":return e.isStringLiteralLike(r)?!!e.tryGetImportFromModuleSpecifier(r):41===r.kind&&e.isJsxClosingElement(r.parent);default:return e.Debug.assertNever(t)}}(i,d,_,c)){if(e.isInString(i,c,_))return _&&e.isStringLiteralLike(_)?function(n,t,r,a,i){if(void 0!==n)switch(n.kind){case 0:return o(n.paths);case 1:var s=[];return y(n.symbols,s,t,t,r,6,a,4,i),{isGlobalCompletion:!1,isMemberCompletion:!0,isNewIdentifierLocation:n.hasIndexSignature,entries:s};case 2:var s=n.types.map(function(e){return{name:e.value,kindModifiers:\"\",kind:\"string\",sortText:\"0\"}});return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:n.isNewIdentifier,entries:s};default:return e.Debug.assertNever(n)}}(h(i,_,c,m,f,t),i,m,a,u):void 0;if(_&&e.isBreakOrContinueStatement(_.parent)&&(72===_.kind||77===_.kind||71===_.kind))return function(n){var t=function(n){for(var t=[],r=e.createMap(),a=n;a&&!e.isFunctionLike(a);){if(e.isLabeledStatement(a)){var i=a.label.text;r.has(i)||(r.set(i,!0),t.push({name:i,kindModifiers:\"\",kind:\"label\",sortText:\"0\"}))}a=a.parent}return t}(n);if(t.length)return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:!1,entries:t}}(_.parent);var v=M(r,a,i,l(i,f),c,u,void 0);if(v)switch(v.kind){case 0:return function(n,t,r,a,i,o){var s=i.symbols,c=i.completionKind,u=i.isInSnippetScope,d=i.isNewIdentifierLocation,m=i.location,f=i.propertyAccessToConvert,g=i.keywordFilters,_=i.literals,v=i.symbolToOriginInfoMap,h=i.recommendedCompletion,b=i.isJsxInitializer;if(m&&m.parent&&e.isJsxClosingElement(m.parent)){var E=m.parent.parent.openingElement.tagName,T=!!e.findChildOfKind(m.parent,29,n),S={name:E.getFullText(n)+(T?\"\":\">\"),kind:\"class\",kindModifiers:void 0,sortText:\"0\"};return{isGlobalCompletion:!1,isMemberCompletion:!0,isNewIdentifierLocation:!1,entries:[S]}}var L=[];if(l(n,r)){var x=y(s,L,m,n,t,r.target,a,c,o,f,b,h,v);!function(n,t,r,a,i){e.getNameTable(n).forEach(function(n,o){if(n!==t){var s=e.unescapeLeadingUnderscores(o);e.addToSeen(r,s)&&e.isIdentifierText(s,a)&&i.push({name:s,kind:\"warning\",kindModifiers:\"\",sortText:\"1\"})}})}(n,m.pos,x,r.target,L)}else{if((!s||0===s.length)&&0===g)return;y(s,L,m,n,t,r.target,a,c,o,f,b,h,v)}if(0!==g)for(var A=e.arrayToSet(L,function(e){return e.name}),C=0,D=function(n){return I[n]||(I[n]=R().filter(function(t){var r=e.stringToToken(t.name);switch(n){case 0:return!1;case 1:return 120===r||!e.isContextualKeyword(r)&&!w(r)||124===r||129===r||e.isTypeKeyword(r)&&140!==r;case 2:return w(r);case 3:return N(r);case 4:return e.isParameterPropertyModifier(r);case 5:return function(n){return 120===n||!e.isContextualKeyword(n)&&!w(n)}(r);case 6:return e.isTypeKeyword(r);default:return e.Debug.assertNever(n)}}))}(g);C<D.length;C++){var k=D[C];A.has(k.name)||L.push(k)}for(var M=0,O=_;M<O.length;M++){var P=O[M];L.push(p(P))}return{isGlobalCompletion:u,isMemberCompletion:function(e){switch(e){case 0:case 3:case 2:return!0;default:return!1}}(c),isNewIdentifierLocation:d,entries:L}}(i,m,f,a,v,u);case 1:return s(e.JsDoc.getJSDocTagNameCompletions());case 2:return s(e.JsDoc.getJSDocTagCompletions());case 3:return s(e.JsDoc.getJSDocParameterNameCompletions(v.tag));default:return e.Debug.assertNever(v)}}};var c,u,d,m=JSON.stringify;function p(e){return{name:m(e),kind:\"string\",kindModifiers:\"\",sortText:\"0\"}}function f(n,t,r,a,o,s,l,c,u,d,m){var p=O(n,o,l,s);if(p){var f,y,h=p.name,b=p.needsConvertPropertyAccess;if(l&&0===l.kind)f=b?\"this[\"+g(h,m)+\"]\":\"this.\"+h;else if((l&&function(e){return 2===e.kind||1===e.kind}(l)||b)&&u){f=b?\"[\"+g(h,m)+\"]\":\"[\"+h+\"]\";var E=e.findChildOfKind(u,23,r),T=e.startsWith(h,u.name.text)?u.name.end:E.end;y=e.createTextSpanFromBounds(E.getStart(r),T)}if(d&&(void 0===f&&(f=h),f=\"{\"+f+\"}\",\"boolean\"!=typeof d&&(y=e.createTextSpanFromNode(d,r))),void 0===f||m.includeCompletionsWithInsertText)return{name:h,kind:e.SymbolDisplay.getSymbolKind(a,n,t),kindModifiers:e.SymbolDisplay.getSymbolModifiers(n),sortText:\"0\",source:v(l),hasAction:_(!!l&&i(l)),isRecommended:_(function(e,n,t){return e===n||!!(1048576&e.flags)&&t.getExportSymbolOfSymbol(e)===n}(n,c,a)),insertText:f,replacementSpan:y}}}function g(n,t){if(/^\\d+$/.test(n))return n;var r=JSON.stringify(n);switch(t.quotePreference){case void 0:case\"double\":return r;case\"single\":return\"'\"+e.stripQuotes(r).replace(\"'\",\"\\\\'\").replace('\\\\\"','\"')+\"'\";default:return e.Debug.assertNever(t.quotePreference)}}function _(e){return!!e||void 0}function v(n){return n&&i(n)?e.stripQuotes(n.moduleSymbol.name):void 0}function y(n,t,r,a,i,o,s,l,c,u,d,m,p){for(var g=e.timestamp(),_=e.createMap(),v=0,y=n;v<y.length;v++){var h=y[v],b=p?p[e.getSymbolId(h)]:void 0,E=f(h,r,a,i,o,l,b,m,u,d,c);if(E){var T=E.name;_.has(T)||(b||void 0===h.parent&&!e.some(h.declarations,function(e){return e.getSourceFile()===r.getSourceFile()})||_.set(T,!0),t.push(E))}}return s(\"getCompletionsAtPosition: getCompletionEntriesFromSymbols: \"+(e.timestamp()-g)),_}function h(t,r,a,i,o,s){var l,c,u=r.parent;switch(u.kind){case 180:switch(u.parent.kind){case 162:return{kind:2,types:E(i.getTypeArgumentConstraint(u)),isNewIdentifier:!1};case 178:return b(i.getTypeFromTypeNode(u.parent.objectType));case 181:return{kind:0,paths:n.PathCompletions.getStringLiteralCompletionsFromModuleNames(t,r,o,s,i)};case 171:if(!e.isTypeReferenceNode(u.parent.parent))return;var d=(l=u.parent,c=u,e.mapDefined(l.types,function(n){return n!==c&&e.isLiteralTypeNode(n)&&e.isStringLiteral(n.literal)?n.literal.text:void 0}));return{kind:2,types:E(i.getTypeArgumentConstraint(u.parent)).filter(function(n){return!e.contains(d,n.value)}),isNewIdentifier:!1};default:return}case 273:return e.isObjectLiteralExpression(u.parent)&&u.name===r?b(i.getContextualType(u.parent)):_();case 188:var m=u,p=m.expression,f=m.argumentExpression;return r===f?b(i.getTypeAtLocation(p)):void 0;case 189:case 190:if(!e.isRequireCall(u,!1)&&!e.isImportCall(u)){var g=e.SignatureHelp.getArgumentInfoForCompletions(r,a,t);return g?function(n,t){var r=!1,a=e.createMap(),i=[];return t.getResolvedSignature(n.invocation,i,n.argumentCount),{kind:2,types:e.flatMap(i,function(e){if(e.hasRestParameter||!(n.argumentCount>e.parameters.length)){var i=t.getParameterType(e,n.argumentIndex);return r=r||!!(4&i.flags),E(i,a)}}),isNewIdentifier:r}}(g,i):_()}case 247:case 253:case 257:return{kind:0,paths:n.PathCompletions.getStringLiteralCompletionsFromModuleNames(t,r,o,s,i)};default:return _()}function _(){return{kind:2,types:E(C(r,i)),isNewIdentifier:!1}}}function b(e){return e&&{kind:1,symbols:e.getApparentProperties(),hasIndexSignature:K(e)}}function E(n,t){return void 0===t&&(t=e.createMap()),n?(n=e.skipConstraint(n)).isUnion()?e.flatMap(n.types,function(e){return E(e,t)}):!n.isStringLiteral()||512&n.flags||!e.addToSeen(t,n.value)?e.emptyArray:[n]:e.emptyArray}function T(n,t,r,a,i){var o=n.getCompilerOptions(),s=M(n,t,r,l(r,o),a,{includeCompletionsForModuleExports:!0,includeCompletionsWithInsertText:!0},i);if(!s)return{type:\"none\"};if(0!==s.kind)return{type:\"request\",request:s};var c=s.symbols,u=s.literals,d=s.location,p=s.completionKind,f=s.symbolToOriginInfoMap,g=s.previousToken,_=s.isJsxInitializer,y=e.find(u,function(e){return m(e)===i.name});return void 0!==y?{type:\"literal\",literal:y}:e.firstDefined(c,function(n){var t=f[e.getSymbolId(n)],r=O(n,o.target,t,p);return r&&r.name===i.name&&v(t)===i.source?{type:\"symbol\",symbol:n,location:d,symbolToOriginInfoMap:f,previousToken:g,isJsxInitializer:_}:void 0})||{type:\"none\"}}function S(n,t,r){return t&&i(t)&&t.isDefaultExport&&\"default\"===n.escapedName?e.firstDefined(n.declarations,function(n){return e.isExportAssignment(n)&&e.isIdentifier(n.expression)?n.expression.text:void 0})||e.codefix.moduleSymbolToValidIdentifier(t.moduleSymbol,r):n.name}function L(n,t,r){return A(n,\"\",t,[e.displayPart(n,r)])}function x(n,t,r,a,i,o,s){var l=t.runWithCancellationToken(i,function(t){return e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(t,n,r,a,a,7)}),c=l.displayParts,u=l.documentation,d=l.symbolKind,m=l.tags;return A(n.name,e.SymbolDisplay.getSymbolModifiers(n),d,c,u,m,o,s)}function A(e,n,t,r,a,i,o,s){return{name:e,kindModifiers:n,kind:t,displayParts:r,documentation:a,tags:i,codeActions:o,source:s}}function C(e,n){var t=e.parent;switch(t.kind){case 190:return n.getContextualType(t);case 202:var r=t,a=r.left,i=r.operatorToken,o=r.right;return F(i.kind)?n.getTypeAtLocation(e===o?a:o):n.getContextualType(e);case 269:return t.expression===e?D(t,n):void 0;default:return n.getContextualType(e)}}function D(e,n){return n.getTypeAtLocation(e.parent.parent.expression)}function k(n,t,r){var a=r.getAccessibleSymbolChain(n,t,67108863,!1);return a?e.first(a):n.parent&&(function(e){return e.declarations.some(function(e){return 277===e.kind})}(n.parent)?n:k(n.parent,t,r))}function M(n,t,r,a,i,o,s){var l=n.getTypeChecker(),c=e.timestamp(),u=e.getTokenAtPosition(r,i);t(\"getCompletionData: Get current token: \"+(e.timestamp()-c)),c=e.timestamp();var d=e.isInComment(r,i,u);t(\"getCompletionData: Is inside comment: \"+(e.timestamp()-c));var m=!1,p=!1;if(d){if(e.hasDocComment(r,i)){if(64===r.text.charCodeAt(i-1))return{kind:1};var f=e.getLineStartPositionForPosition(i,r);if(!r.text.substring(f,i).match(/[^\\*|\\s|(\\/\\*\\*)]/))return{kind:2}}var g=function(n,t){var r=e.findAncestor(n,e.isJSDoc);return r&&r.tags&&(e.rangeContainsPosition(r,t)?e.findLast(r.tags,function(e){return e.pos<t}):void 0)}(u,i);if(g){if(g.tagName.pos<=i&&i<=g.tagName.end)return{kind:1};if(function(e){switch(e.kind){case 297:case 303:case 298:case 300:case 302:return!0;default:return!1}}(g)&&g.typeExpression&&281===g.typeExpression.kind&&((u=e.getTokenAtPosition(r,i))&&(e.isDeclarationName(u)||303===u.parent.kind&&u.parent.name===u)||(m=ie(g.typeExpression))),e.isJSDocParameterTag(g)&&(e.nodeIsMissing(g.name)||g.name.pos<=i&&i<=g.name.end))return{kind:3,tag:g}}if(!m)return void t(\"Returning an empty list because completion was inside a regular comment or plain text part of a JsDoc comment.\")}c=e.timestamp();var _=e.findPrecedingToken(i,r,void 0);t(\"getCompletionData: Get previous token 1: \"+(e.timestamp()-c));var v=_;if(v&&i<=v.end&&(e.isIdentifier(v)||e.isKeyword(v.kind))){var y=e.timestamp();v=e.findPrecedingToken(v.getFullStart(),r,void 0),t(\"getCompletionData: Get previous token 2: \"+(e.timestamp()-y))}var h,b=u,E=!1,T=!1,L=!1,x=!1,A=e.getTouchingPropertyName(r,i);if(v){if(function(n){var r=e.timestamp(),a=function(n){return(e.isRegularExpressionLiteral(n)||e.isStringTextContainingNode(n))&&(e.rangeContainsPositionExclusive(e.createTextRangeFromSpan(e.createTextSpanFromNode(n)),i)||i===n.end&&(!!n.isUnterminated||e.isRegularExpressionLiteral(n)))}(n)||function(n){var t=n.parent,r=t.kind;switch(n.kind){case 26:return 235===r||236===r||217===r||241===r||ae(r)||239===r||183===r||240===r||e.isClassLike(t)&&!!t.typeParameters&&t.typeParameters.end>=n.pos;case 23:return 183===r;case 56:return 184===r;case 21:return 183===r;case 19:return 272===r||ae(r);case 17:return 241===r;case 27:return 238===r||207===r||239===r||240===r||e.isFunctionLikeKind(r);case 115:return 152===r&&!e.isClassLike(t.parent);case 24:return 149===r||!!t.parent&&183===t.parent.kind;case 114:case 112:case 113:return 149===r&&!e.isConstructorDeclaration(t.parent);case 118:return 251===r||255===r||249===r;case 125:case 136:return!V(n);case 75:case 83:case 109:case 89:case 104:case 91:case 110:case 76:case 116:case 139:return!0;case 39:return e.isFunctionLike(n.parent)&&!e.isMethodDeclaration(n.parent)}if(w(P(n))&&V(n))return!1;if(re(n)&&(!e.isIdentifier(n)||e.isParameterPropertyModifier(P(n))||ie(n)))return!1;switch(P(n)){case 117:case 120:case 75:case 76:case 124:case 83:case 89:case 109:case 110:case 112:case 113:case 114:case 115:case 104:case 116:return!0}return e.isDeclarationName(n)&&!e.isJsxAttribute(n.parent)&&!(e.isClassLike(n.parent)&&(n!==_||i>_.end))}(n)||function(e){if(8===e.kind){var n=e.getFullText();return\".\"===n.charAt(n.length-1)}return!1}(n)||function(e){if(10===e.kind)return!0;if(29===e.kind&&e.parent){if(260===e.parent.kind)return!0;if(261===e.parent.kind||259===e.parent.kind)return!!e.parent.parent&&258===e.parent.parent.kind}return!1}(n);return t(\"getCompletionsAtPosition: isCompletionListBlocker: \"+(e.timestamp()-r)),a}(v))return void t(\"Returning an empty list because completion was requested in an invalid position.\");var M=v.parent;if(23===v.kind)switch(E=!0,M.kind){case 187:b=(h=M).expression;break;case 146:b=M.left;break;case 242:b=M.name;break;case 181:case 212:b=M;break;default:return}else if(1===r.languageVariant){if(M&&187===M.kind&&(v=M,M=M.parent),u.parent===A)switch(u.kind){case 29:258!==u.parent.kind&&260!==u.parent.kind||(A=u);break;case 41:259===u.parent.kind&&(A=u)}switch(M.kind){case 261:41===v.kind&&(L=!0,A=v);break;case 202:if(!B(M))break;case 259:case 258:case 260:27===v.kind&&(T=!0,A=v);break;case 265:switch(_.kind){case 58:x=!0;break;case 71:M!==_.parent&&!M.initializer&&e.findChildOfKind(M,58,r)&&(x=_)}}}}var O=e.timestamp(),I=5,R=!1,H=0,U=[],j=[];if(E)!function(){I=2;var n=e.isLiteralImportTypeNode(b),t=m||n&&!b.isTypeOf||e.isPartOfTypeNode(b.parent),a=e.isInRightSideOfInternalImportEqualsDeclaration(b)||!t&&e.isPossiblyTypeArgumentPosition(v,r,l);if(e.isEntityName(b)||n){var i=e.isModuleDeclaration(b.parent);i&&(R=!0);var o=l.getSymbolAtLocation(b);if(o&&1920&(o=e.skipAlias(o,l)).flags){for(var s=e.Debug.assertEachDefined(l.getExportsOfModule(o),\"getExportsOfModule() should all be defined\"),c=function(e){return l.isValidPropertyAccess(n?b:b.parent,e.name)},u=function(e){return ne(e)},d=i?function(e){return!!(1920&e.flags)&&!e.declarations.every(function(e){return e.parent===b.parent})}:a?function(e){return u(e)||c(e)}:t?u:c,p=0,f=s;p<f.length;p++){var g=f[p];d(g)&&U.push(g)}return void(!t&&o.declarations.some(function(e){return 277!==e.kind&&242!==e.kind&&241!==e.kind})&&Q(l.getTypeOfSymbolAtLocation(o,b)))}}if(e.isMetaProperty(b)&&(94===b.keywordToken||91===b.keywordToken)){var _=94===b.keywordToken?\"target\":\"meta\";return void U.push(l.createSymbol(4,e.escapeLeadingUnderscores(_)))}t||Q(l.getTypeAtLocation(b))}();else if(T){var W=e.Debug.assertEachDefined(l.getJsxIntrinsicTagNamesAt(A),\"getJsxIntrinsicTagNames() should all be defined\");$(),U=W.concat(U),I=3,H=0}else if(L){var q=v.parent.parent.openingElement.tagName,z=l.getSymbolAtLocation(q);z&&(U=[z]),I=3,H=0}else if(!$())return;t(\"getCompletionData: Semantic work: \"+(e.timestamp()-O));var J=_&&function(n,t,r,a){var i=n.parent;switch(n.kind){case 71:return C(n,a);case 58:switch(i.kind){case 235:return a.getContextualType(i.initializer);case 202:return a.getTypeAtLocation(i.left);case 265:return a.getContextualTypeForJsxAttribute(i);default:return}case 94:return a.getContextualType(i);case 73:return D(e.cast(i,e.isCaseClause),a);case 17:return e.isJsxExpression(i)&&258!==i.parent.kind?a.getContextualTypeForJsxAttribute(i.parent):void 0;default:var o=e.SignatureHelp.getArgumentInfoForCompletions(n,t,r);return o?a.getContextualTypeForArgumentAtIndex(o.invocation,o.argumentIndex+(26===n.kind?1:0)):F(n.kind)&&e.isBinaryExpression(i)&&F(i.operatorToken.kind)?a.getTypeAtLocation(i.left):a.getContextualType(n)}}(_,i,r,l),X=e.mapDefined(J&&(J.isUnion()?J.types:[J]),function(e){return e.isLiteral()?e.value:void 0}),Y=_&&J&&function(n,t,r){return e.firstDefined(t&&(t.isUnion()?t.types:[t]),function(t){var a=t&&t.symbol;return a&&424&a.flags&&!e.isAbstractConstructorSymbol(a)?k(a,n,r):void 0})}(_,J,l);return{kind:0,symbols:U,completionKind:I,isInSnippetScope:p,propertyAccessToConvert:h,isNewIdentifierLocation:R,location:A,keywordFilters:H,literals:X,symbolToOriginInfoMap:j,recommendedCompletion:Y,previousToken:_,isJsxInitializer:x};function Q(e){if(R=!!e.getStringIndexType(),a)U.push.apply(U,G(e,l));else for(var n=0,t=e.getApparentProperties();n<t.length;n++){var r=t[n];l.isValidPropertyAccessForCompletions(181===b.kind?b:b.parent,e,r)&&Z(r)}}function Z(n){var t=e.firstDefined(n.declarations,function(n){return e.tryCast(e.getNameOfDeclaration(n),e.isComputedPropertyName)});if(t){var r=function n(t){return e.isIdentifier(t)?t:e.isPropertyAccessExpression(t)?n(t.expression):void 0}(t.expression),a=r&&l.getSymbolAtLocation(r),i=a&&k(a,v,l);if(i&&!j[e.getSymbolId(i)]){U.push(i);var o=i.parent;j[e.getSymbolId(i)]=o&&e.isExternalModuleSymbol(o)?{kind:2,moduleSymbol:o,isDefaultExport:!1}:{kind:1}}}else U.push(n)}function $(){var t,c;return 1===(function(){var n,t,r=function(n){if(n){var t=n.parent;switch(n.kind){case 17:case 26:if(e.isObjectLiteralExpression(t)||e.isObjectBindingPattern(t))return t;break;case 39:return e.isMethodDeclaration(t)?e.tryCast(t.parent,e.isObjectLiteralExpression):void 0}}return}(v);if(!r)return 0;if(I=0,186===r.kind){var a=l.getContextualType(r);if(!a)return 2;R=K(a),c=r,u=l,n=(s=a).isUnion()?u.getAllPossiblePropertiesOfTypes(s.types.filter(function(n){return!(32764&n.flags||u.isArrayLikeType(n)||e.typeHasCallOrConstructSignatures(n,u)||u.isTypeInvalidDueToUnionDiscriminant(n,c))})):s.getApparentProperties(),t=r.properties}else{e.Debug.assert(182===r.kind),R=!1;var i=e.getRootDeclaration(r.parent);if(!e.isVariableLike(i))return e.Debug.fail(\"Root declaration is not variable-like.\");var o=e.hasInitializer(i)||e.hasType(i)||225===i.parent.parent.kind;if(o||149!==i.kind||(e.isExpression(i.parent)?o=!!l.getContextualType(i.parent):154!==i.parent.kind&&157!==i.parent.kind||(o=e.isExpression(i.parent.parent)&&!!l.getContextualType(i.parent.parent))),o){var a=l.getTypeAtLocation(r);if(!a)return 2;n=l.getPropertiesOfType(a).filter(function(n){return!(24&e.getDeclarationModifierFlagsFromSymbol(n))}),t=r.elements}}var s,c,u;n&&n.length>0&&(U=function(n,t){if(0===t.length)return n;for(var r=e.createUnderscoreEscapedMap(),a=0,i=t;a<i.length;a++){var o=i[a];if((273===o.kind||274===o.kind||184===o.kind||154===o.kind||156===o.kind||157===o.kind)&&!ie(o)){var s=void 0;if(e.isBindingElement(o)&&o.propertyName)71===o.propertyName.kind&&(s=o.propertyName.escapedText);else{var l=e.getNameOfDeclaration(o);s=l&&e.isPropertyNameLiteral(l)?e.getEscapedTextOfIdentifierOrLiteral(l):void 0}r.set(s,!0)}}return n.filter(function(e){return!r.get(e.escapedName)})}(n,e.Debug.assertDefined(t)));return 1}()||function(){var n=!v||17!==v.kind&&26!==v.kind?void 0:e.tryCast(v.parent,e.isNamedImportsOrExports);if(!n)return 0;var t=(250===n.kind?n.parent.parent:n.parent).moduleSpecifier,r=l.getSymbolAtLocation(t);if(!r)return 2;I=3,R=!1;var a=l.getExportsAndPropertiesOfModule(r),i=e.arrayToSet(n.elements,function(e){return ie(e)?void 0:(e.propertyName||e.name).escapedText});return U=a.filter(function(e){return\"default\"!==e.escapedName&&!i.get(e.escapedName)}),1}()||(function(n){if(n){var t=n.parent;switch(n.kind){case 19:case 26:return e.isConstructorDeclaration(n.parent)?n.parent:void 0;default:if(re(n))return t.parent}}}(v)?(I=5,R=!0,H=4,1):0)||function(){var n=function(n,t,r){switch(r.kind){case 304:return e.tryCast(r.parent,e.isObjectTypeDeclaration);case 1:var a=e.tryCast(e.lastOrUndefined(e.cast(r.parent,e.isSourceFile).statements),e.isObjectTypeDeclaration);if(a&&!e.findChildOfKind(a,18,n))return a}if(!t)return;switch(t.kind){case 25:case 18:return V(r)&&r.parent.name===r?r.parent.parent:e.tryCast(r,e.isObjectTypeDeclaration);case 17:case 26:return e.tryCast(t.parent,e.isObjectTypeDeclaration);default:if(!V(t))return;var i=e.isClassLike(t.parent.parent)?w:N;return i(t.kind)||39===t.kind||e.isIdentifier(t)&&i(e.stringToToken(t.text))?t.parent.parent:void 0}}(r,v,A);if(!n)return 0;if(I=3,R=!0,H=39===v.kind?0:e.isClassLike(n)?2:3,!e.isClassLike(n))return 1;var t=v.parent,a=e.isClassElement(t)?e.getModifierFlags(t):0;if(71===v.kind&&!ie(v))switch(v.getText()){case\"private\":a|=8;break;case\"static\":a|=32}if(!(8&a)){var i=e.flatMap(e.getAllSuperTypeNodes(n),function(e){var t=l.getTypeAtLocation(e);return t&&l.getPropertiesOfType(32&a?l.getTypeOfSymbolAtLocation(t.symbol,n):t)});U=function(n,t,r){for(var a=e.createUnderscoreEscapedMap(),i=0,o=t;i<o.length;i++){var s=o[i];if((152===s.kind||154===s.kind||156===s.kind||157===s.kind)&&(!ie(s)&&!e.hasModifier(s,8)&&e.hasModifier(s,32)===!!(32&r))){var l=e.getPropertyNameForPropertyNameNode(s.name);l&&a.set(l,!0)}}return n.filter(function(n){return!(a.has(n.escapedName)||!n.declarations||8&e.getDeclarationModifierFlagsFromSymbol(n))})}(i,n.members,a)}return 1}()||(t=function(e){if(e){var n=e.parent;switch(e.kind){case 28:case 41:case 71:case 187:case 266:case 265:case 267:if(n&&(259===n.kind||260===n.kind))return n;if(265===n.kind)return n.parent.parent;break;case 9:if(n&&(265===n.kind||267===n.kind))return n.parent.parent;break;case 18:if(n&&268===n.kind&&n.parent&&265===n.parent.kind)return n.parent.parent.parent;if(n&&267===n.kind)return n.parent.parent}}return}(v),(c=t&&l.getAllAttributesTypeFromJsxOpeningLikeElement(t))?(U=function(n,t){for(var r=e.createUnderscoreEscapedMap(),a=0,i=t;a<i.length;a++){var o=i[a];ie(o)||265===o.kind&&r.set(o.name.escapedText,!0)}return n.filter(function(e){return!r.get(e.escapedName)})}(l.getPropertiesOfType(c),t.attributes.properties),I=3,R=!1,1):0)||(function(){H=function(n){if(n){var t,r=e.findAncestor(n.parent,function(n){return e.isClassLike(n)?\"quit\":!(!e.isFunctionLikeDeclaration(n)||t!==n.body)||(t=n,!1)});return r&&r}}(v)?5:1,I=1,R=function(e){if(e){var n=e.parent.kind;switch(P(e)){case 26:return 189===n||155===n||190===n||185===n||202===n||163===n;case 19:return 189===n||155===n||190===n||193===n||175===n;case 21:return 185===n||160===n||147===n;case 129:case 130:return!0;case 23:return 242===n;case 17:return 238===n;case 58:return 235===n||202===n;case 14:return 204===n;case 15:return 214===n;case 114:case 112:case 113:return 152===n}}return!1}(v),_!==v&&e.Debug.assert(!!_,\"Expected 'contextToken' to be defined when different from 'previousToken'.\");var t=_!==v?_.getStart():i,c=function(n,t,r){var a=n;for(;a&&!e.positionBelongsToNode(a,t,r);)a=a.parent;return a}(v,t,r)||r;p=function(n){switch(n.kind){case 277:case 204:case 268:case 216:return!0;default:return e.isStatement(n)}}(c);if(U=e.Debug.assertEachDefined(l.getSymbolsInScope(c,70107135),\"getSymbolsInScope() should all be defined\"),o.includeCompletionsWithInsertText&&277!==c.kind){var u=l.tryGetThisTypeAt(c);if(u)for(var d=0,f=G(u,l);d<f.length;d++){var g=f[d];j[e.getSymbolId(g)]={kind:0},U.push(g)}}o.includeCompletionsForModuleExports&&(r.externalModuleIndicator||!r.commonJsModuleIndicator&&!a&&(e.programContainsEs6Modules(n)||e.compilerOptionsIndicateEs6Modules(n.getCompilerOptions())))&&function(t,a,i){var o=a.toLowerCase(),c=e.createMap();e.codefix.forEachExternalModuleToImportFrom(l,r,n.getSourceFiles(),function(n){if(!s||!s.source||e.stripQuotes(n.name)===s.source){var r=l.resolveExternalModuleSymbol(n);if(e.addToSeen(c,e.getSymbolId(r))){r!==n&&e.some(r.declarations,function(e){return!!e.getSourceFile().externalModuleIndicator})&&(t.push(r),j[e.getSymbolId(r)]={kind:3,moduleSymbol:n,isDefaultExport:!1});for(var a=0,u=l.getExportsOfModule(n);a<u.length;a++){var d=u[a];if(l.getMergedSymbol(d.parent)===r&&!e.some(d.declarations,function(n){return e.isExportSpecifier(n)&&!!n.parent.parent.moduleSpecifier})){var m=\"default\"===d.name;m&&(d=e.getLocalSymbolForExportDefault(d)||d);var p={kind:3,moduleSymbol:n,isDefaultExport:m};(s||te(S(d,p,i).toLowerCase(),o))&&(t.push(d),j[e.getSymbolId(d)]=p)}}}}})}(U,_&&e.isIdentifier(_)?_.text:\"\",n.getCompilerOptions().target);!function(n){var t=m||!ee(v)&&(e.isPartOfTypeNode(A)||function(n){if(n){var t=n.parent.kind;switch(n.kind){case 56:return 152===t||151===t||149===t||235===t||e.isFunctionLikeKind(t);case 58:return 240===t;case 118:return 210===t}}return!1}(v)),a=t||!ee(v)&&e.isPossiblyTypeArgumentPosition(v,r,l);t&&(H=6);e.filterMutate(n,function(n){if(!e.isSourceFile(A)){if(e.isExportAssignment(A.parent))return!0;if(n=e.skipAlias(n,l),e.isInRightSideOfInternalImportEqualsDeclaration(A))return!!(1920&n.flags);if(a){var r=ne(n);if(r||t)return r}}return!!(67220415&e.getCombinedLocalAndExportSymbolFlags(n))})}(U)}(),1))}function ee(n){return n&&103===n.kind&&(165===n.parent.kind||e.isTypeOfExpression(n.parent))}function ne(n){return n=n.exportSymbol||n,!!(67897832&(n=e.skipAlias(n,l)).flags)||!!(1536&n.flags)&&l.getExportsOfModule(n).some(ne)}function te(e,n){if(0===n.length)return!0;for(var t=0,r=0;r<e.length;r++)if(e.charCodeAt(r)===n.charCodeAt(t)&&++t===n.length)return!0;return!1}function re(n){return!!n.parent&&e.isParameter(n.parent)&&e.isConstructorDeclaration(n.parent.parent)&&(e.isParameterPropertyModifier(n.kind)||e.isDeclarationName(n))}function ae(n){return e.isFunctionLikeKind(n)&&155!==n}function ie(e){return e.getStart(r)<=i&&i<=e.getEnd()}}function O(n,t,r,a){var i=S(n,r,t);if(!(void 0===i||1536&n.flags&&e.startsWithQuote(i)||e.isKnownSymbol(n))){var o={name:i,needsConvertPropertyAccess:!1};if(e.isIdentifierText(i,t))return o;switch(a){case 3:return;case 0:return{name:JSON.stringify(i),needsConvertPropertyAccess:!1};case 2:case 1:return 32===i.charCodeAt(0)?void 0:{name:i,needsConvertPropertyAccess:!0};case 5:case 4:return o;default:e.Debug.assertNever(a)}}}!function(e){e[e.Paths=0]=\"Paths\",e[e.Properties=1]=\"Properties\",e[e.Types=2]=\"Types\"}(c||(c={})),n.getCompletionEntryDetails=function(n,t,r,a,o,s,l,c,u){var d=n.getTypeChecker(),p=n.getCompilerOptions(),f=o.name,g=e.findPrecedingToken(a,r);if(e.isInString(r,a,g)){var _=g&&e.isStringLiteralLike(g)?h(r,g,a,d,p,s):void 0;return _&&function(n,t,r,a,i,o){switch(r.kind){case 0:var s=e.find(r.paths,function(e){return e.name===n});return s&&A(n,\"\",s.kind,[e.textPart(n)]);case 1:var s=e.find(r.symbols,function(e){return e.name===n});return s&&x(s,i,a,t,o);case 2:return e.find(r.types,function(e){return e.value===n})?A(n,\"\",\"type\",[e.textPart(n)]):void 0;default:return e.Debug.assertNever(r)}}(f,g,_,r,d,u)}var v=T(n,t,r,a,o);switch(v.type){case\"request\":var y=v.request;switch(y.kind){case 1:return e.JsDoc.getJSDocTagNameCompletionDetails(f);case 2:return e.JsDoc.getJSDocTagCompletionDetails(f);case 3:return e.JsDoc.getJSDocParameterNameCompletionDetails(f);default:return e.Debug.assertNever(y)}case\"symbol\":var b=v.symbol,E=v.location,C=function(n,t,r,a,o,s,l,c,u,d,m){var p=n[e.getSymbolId(t)];if(!p||!i(p))return{codeActions:void 0,sourceDisplay:void 0};var f=p.moduleSymbol,g=a.getMergedSymbol(e.skipAlias(t.exportSymbol||t,a)),_=e.codefix.getImportCompletionAction(g,f,l,S(t,p,s.target),o,r,d,u&&e.isIdentifier(u)?u.getStart(l):c,m),v=_.moduleSpecifier,y=_.codeAction;return{sourceDisplay:[e.textPart(v)],codeActions:[y]}}(v.symbolToOriginInfoMap,b,n,d,s,p,r,a,v.previousToken,l,c);return x(b,d,r,E,u,C.codeActions,C.sourceDisplay);case\"literal\":var D=v.literal;return L(m(D),\"string\",\"string\"==typeof D?e.SymbolDisplayPartKind.stringLiteral:e.SymbolDisplayPartKind.numericLiteral);case\"none\":return R().some(function(e){return e.name===f})?L(f,\"keyword\",e.SymbolDisplayPartKind.keyword):void 0;default:e.Debug.assertNever(v)}},n.getCompletionEntrySymbol=function(e,n,t,r,a){var i=T(e,n,t,r,a);return\"symbol\"===i.type?i.symbol:void 0},function(e){e[e.Data=0]=\"Data\",e[e.JsDocTagName=1]=\"JsDocTagName\",e[e.JsDocTag=2]=\"JsDocTag\",e[e.JsDocParameterName=3]=\"JsDocParameterName\"}(u||(u={})),function(e){e[e.ObjectPropertyDeclaration=0]=\"ObjectPropertyDeclaration\",e[e.Global=1]=\"Global\",e[e.PropertyAccess=2]=\"PropertyAccess\",e[e.MemberLike=3]=\"MemberLike\",e[e.String=4]=\"String\",e[e.None=5]=\"None\"}(d||(d={}));var I=[],R=e.memoize(function(){for(var n=[],t=72;t<=145;t++)n.push({name:e.tokenToString(t),kind:\"keyword\",kindModifiers:\"\",sortText:\"0\"});return n});function N(e){return 132===e}function w(n){switch(n){case 117:case 123:case 125:case 136:case 120:return!0;default:return e.isClassMemberModifier(n)}}function P(n){return e.isIdentifier(n)?n.originalKeywordKind||0:n.kind}function F(e){switch(e){case 34:case 32:case 35:case 33:return!0;default:return!1}}function G(n,t){return n.isUnion()?e.Debug.assertEachDefined(t.getAllPossiblePropertiesOfTypes(n.types),\"getAllPossiblePropertiesOfTypes() should all be defined\"):e.Debug.assertEachDefined(n.getApparentProperties(),\"getApparentProperties() should all be defined\")}function V(n){return n.parent&&e.isClassOrTypeElement(n.parent)&&e.isObjectTypeDeclaration(n.parent.parent)}function K(e){return!!e.getStringIndexType()||!!e.getNumberIndexType()}function B(n){var t=n.left;return e.nodeIsMissing(t)}}(e.Completions||(e.Completions={}))}(Tn||(Tn={})),function(e){!function(n){function t(n,t){return{fileName:t.fileName,textSpan:e.createTextSpanFromNode(n,t),kind:\"none\"}}function r(n){return e.isThrowStatement(n)?[n]:e.isTryStatement(n)?e.concatenate(n.catchClause?r(n.catchClause):n.tryBlock&&r(n.tryBlock),n.finallyBlock&&r(n.finallyBlock)):e.isFunctionLike(n)?void 0:i(n,r)}function a(n){return e.isBreakOrContinueStatement(n)?[n]:e.isFunctionLike(n)?void 0:i(n,a)}function i(n,t){var r=[];return n.forEachChild(function(n){var a=t(n);void 0!==a&&r.push.apply(r,e.toArray(a))}),r}function o(e,n){var t=s(n);return!!t&&t===e}function s(n){return e.findAncestor(n,function(t){switch(t.kind){case 230:if(226===n.kind)return!1;case 223:case 224:case 225:case 222:case 221:return!n.label||function(n,t){return!!e.findAncestor(n.parent,function(n){return e.isLabeledStatement(n)?n.label.escapedText===t:\"quit\"})}(t,n.label.escapedText);default:return e.isFunctionLike(t)&&\"quit\"}})}function l(n,t){for(var r=[],a=2;a<arguments.length;a++)r[a-2]=arguments[a];return!(!t||!e.contains(r,t.kind))&&(n.push(t),!0)}function c(n){var t=[];if(l(t,n.getFirstToken(),88,106,81)&&221===n.kind)for(var r=n.getChildren(),i=r.length-1;i>=0&&!l(t,r[i],106);i--);return e.forEach(a(n.statement),function(e){o(n,e)&&l(t,e.getFirstToken(),72,77)}),t}function u(e){var n=s(e);if(n)switch(n.kind){case 223:case 224:case 225:case 221:case 222:return c(n);case 230:return d(n)}}function d(n){var t=[];return l(t,n.getFirstToken(),98),e.forEach(n.caseBlock.clauses,function(r){l(t,r.getFirstToken(),73,79),e.forEach(a(r),function(e){o(n,e)&&l(t,e.getFirstToken(),72)})}),t}function m(n,t){var r=[];(l(r,n.getFirstToken(),102),n.catchClause&&l(r,n.catchClause.getFirstToken(),74),n.finallyBlock)&&l(r,e.findChildOfKind(n,87,t),87);return r}function p(n,t){var a=function(n){for(var t=n;t.parent;){var r=t.parent;if(e.isFunctionBlock(r)||277===r.kind)return r;if(e.isTryStatement(r)&&r.tryBlock===t&&r.catchClause)return t;t=r}}(n);if(a){var i=[];return e.forEach(r(a),function(n){i.push(e.findChildOfKind(n,100,t))}),e.isFunctionBlock(a)&&e.forEachReturnStatement(a,function(n){i.push(e.findChildOfKind(n,96,t))}),i}}function f(n,t){var a=e.getContainingFunction(n);if(a){var i=[];return e.forEachReturnStatement(e.cast(a.body,e.isBlock),function(n){i.push(e.findChildOfKind(n,96,t))}),e.forEach(r(a.body),function(n){i.push(e.findChildOfKind(n,100,t))}),i}}function g(n){var t=e.getContainingFunction(n);if(t){var r=[];return t.modifiers&&t.modifiers.forEach(function(e){l(r,e,120)}),e.forEachChild(t,function(n){_(n,function(n){e.isAwaitExpression(n)&&l(r,n.getFirstToken(),121)})}),r}}function _(n,t){t(n),e.isFunctionLike(n)||e.isClassLike(n)||e.isInterfaceDeclaration(n)||e.isModuleDeclaration(n)||e.isTypeAliasDeclaration(n)||e.isTypeNode(n)||e.forEachChild(n,function(e){return _(e,t)})}n.getDocumentHighlights=function(n,r,a,i,o){var s=e.getTouchingPropertyName(a,i);if(s.parent&&(e.isJsxOpeningElement(s.parent)&&s.parent.tagName===s||e.isJsxClosingElement(s.parent))){var v=s.parent.parent,y=[v.openingElement,v.closingElement].map(function(e){return t(e.tagName,a)});return[{fileName:a.fileName,highlightSpans:y}]}return function(n,t,r,a,i){var o=e.arrayToSet(i,function(e){return e.fileName}),s=e.FindAllReferences.getReferenceEntriesForNode(n,t,r,i,a,void 0,o);if(s){var l=e.arrayToMultiMap(s.map(e.FindAllReferences.toHighlightSpan),function(e){return e.fileName},function(e){return e.span});return e.arrayFrom(l.entries(),function(n){var t=n[0],a=n[1];if(!o.has(t)){e.Debug.assert(r.redirectTargetsMap.has(t));var s=r.getSourceFile(t),l=e.find(i,function(e){return!!e.redirectInfo&&e.redirectInfo.redirectTarget===s});t=l.fileName,e.Debug.assert(o.has(t))}return{fileName:t,highlightSpans:a}})}}(i,s,n,r,o)||function(n,r){var a=function(n,r){switch(n.kind){case 90:case 82:return e.isIfStatement(n.parent)?function(n,r){for(var a=function(n,t){for(var r=[];e.isIfStatement(n.parent)&&n.parent.elseStatement===n;)n=n.parent;for(;;){var a=n.getChildren(t);l(r,a[0],90);for(var i=a.length-1;i>=0&&!l(r,a[i],82);i--);if(!n.elseStatement||!e.isIfStatement(n.elseStatement))break;n=n.elseStatement}return r}(n,r),i=[],o=0;o<a.length;o++){if(82===a[o].kind&&o<a.length-1){for(var s=a[o],c=a[o+1],u=!0,d=c.getStart(r)-1;d>=s.end;d--)if(!e.isWhiteSpaceSingleLine(r.text.charCodeAt(d))){u=!1;break}if(u){i.push({fileName:r.fileName,textSpan:e.createTextSpanFromBounds(s.getStart(),c.end),kind:\"reference\"}),o++;continue}}i.push(t(a[o],r))}return i}(n.parent,r):void 0;case 96:return v(n.parent,e.isReturnStatement,f);case 100:return v(n.parent,e.isThrowStatement,p);case 102:case 74:case 87:var a=74===n.kind?n.parent.parent:n.parent;return v(a,e.isTryStatement,m);case 98:return v(n.parent,e.isSwitchStatement,d);case 73:case 79:return v(n.parent.parent.parent,e.isSwitchStatement,d);case 72:case 77:return v(n.parent,e.isBreakOrContinueStatement,u);case 88:case 106:case 81:return v(n.parent,function(n){return e.isIterationStatement(n,!0)},c);case 123:return s(e.isConstructorDeclaration,[123]);case 125:case 136:return s(e.isAccessor,[125,136]);case 121:return v(n.parent,e.isAwaitExpression,g);case 120:return y(g(n));case 116:return y(function(n){var t=e.getContainingFunction(n);if(t){var r=[];return e.forEachChild(t,function(n){_(n,function(n){e.isYieldExpression(n)&&l(r,n.getFirstToken(),116)})}),r}}(n));default:return e.isModifierKind(n.kind)&&(e.isDeclaration(n.parent)||e.isVariableStatement(n.parent))?y((i=n.kind,o=n.parent,e.mapDefined(function(n,t){var r=n.parent;switch(r.kind){case 243:case 277:case 216:case 269:case 270:return 128&t&&e.isClassDeclaration(n)?n.members.concat([n]):r.statements;case 155:case 154:case 237:return r.parameters.concat(e.isClassLike(r.parent)?r.parent.members:[]);case 238:case 207:var a=r.members;if(28&t){var i=e.find(r.members,e.isConstructorDeclaration);if(i)return a.concat(i.parameters)}else if(128&t)return a.concat([r]);return a;default:e.Debug.assertNever(r,\"Invalid container kind.\")}}(o,e.modifierToFlag(i)),function(n){return e.findModifier(n,i)}))):void 0}var i,o;function s(t,a){return v(n.parent,t,function(n){return e.mapDefined(n.symbol.declarations,function(n){return t(n)?e.find(n.getChildren(r),function(n){return e.contains(a,n.kind)}):void 0})})}function v(e,n,t){return n(e)?y(t(e,r)):void 0}function y(e){return e&&e.map(function(e){return t(e,r)})}}(n,r);return a&&[{fileName:r.fileName,highlightSpans:a}]}(s,a)}}(e.DocumentHighlights||(e.DocumentHighlights={}))}(Tn||(Tn={})),function(e){function n(n,r,a){void 0===r&&(r=\"\");var i=e.createMap(),o=e.createGetCanonicalFileName(!!n);function s(n,t){var r=i.get(n);return!r&&t&&i.set(n,r=e.createMap()),r}function l(e,n,t,r,a,i,o){return u(e,n,t,r,a,i,!0,o)}function c(e,n,t,r,a,i,o){return u(e,n,t,r,a,i,!1,o)}function u(n,t,r,i,o,l,c,u){var d=s(i,!0),m=d.get(t),p=6===u?100:r.target||1;!m&&a&&((f=a.getDocument(i,t))&&(e.Debug.assert(c),m={sourceFile:f,languageServiceRefCount:0},d.set(t,m)));if(m)m.sourceFile.version!==l&&(m.sourceFile=e.updateLanguageServiceSourceFile(m.sourceFile,o,l,o.getChangeRange(m.sourceFile.scriptSnapshot)),a&&a.setDocument(i,t,m.sourceFile)),c&&m.languageServiceRefCount++;else{var f=e.createLanguageServiceSourceFile(n,o,p,l,!1,u);a&&a.setDocument(i,t,f),m={sourceFile:f,languageServiceRefCount:1},d.set(t,m)}return e.Debug.assert(0!==m.languageServiceRefCount),m.sourceFile}function d(n,t){var r=s(t,!1);e.Debug.assert(void 0!==r);var a=r.get(n);a.languageServiceRefCount--,e.Debug.assert(a.languageServiceRefCount>=0),0===a.languageServiceRefCount&&r.delete(n)}return{acquireDocument:function(n,a,i,s,c){return l(n,e.toPath(n,r,o),a,t(a),i,s,c)},acquireDocumentWithKey:l,updateDocument:function(n,a,i,s,l){return c(n,e.toPath(n,r,o),a,t(a),i,s,l)},updateDocumentWithKey:c,releaseDocument:function(n,a){return d(e.toPath(n,r,o),t(a))},releaseDocumentWithKey:d,getLanguageServiceRefCounts:function(n){return e.arrayFrom(i.entries(),function(e){var t=e[0],r=e[1].get(n);return[t,r&&r.languageServiceRefCount]})},reportStats:function(){var n=e.arrayFrom(i.keys()).filter(function(e){return e&&\"_\"===e.charAt(0)}).map(function(e){var n=i.get(e),t=[];return n.forEach(function(e,n){t.push({name:n,refCount:e.languageServiceRefCount})}),t.sort(function(e,n){return n.refCount-e.refCount}),{bucket:e,sourceFiles:t}});return JSON.stringify(n,void 0,2)},getKeyForCompilationSettings:t}}function t(n){return e.sourceFileAffectingCompilerOptions.map(function(t){return e.getCompilerOptionValue(n,t)}).join(\"|\")}e.createDocumentRegistry=function(e,t){return n(e,t)},e.createDocumentRegistryInternal=n}(Tn||(Tn={})),function(e){!function(n){function t(n,t){return e.forEach(277===n.kind?n.statements:n.body.statements,function(n){return t(n)||s(n)&&e.forEach(n.body&&n.body.statements,t)})}function r(n,r){if(n.externalModuleIndicator||void 0!==n.imports)for(var a=0,i=n.imports;a<i.length;a++){var o=i[a];r(e.importFromModuleSpecifier(o),o)}else t(n,function(n){switch(n.kind){case 253:case 247:(t=n).moduleSpecifier&&e.isStringLiteral(t.moduleSpecifier)&&r(t,t.moduleSpecifier);break;case 246:var t;l(t=n)&&r(t,t.moduleReference.expression)}})}function a(n,t,r){var a=n.parent;if(a){var i=r.getMergedSymbol(a);return e.isExternalModuleSymbol(i)?{exportingModuleSymbol:i,exportKind:t}:void 0}}function i(e,n){return n.getMergedSymbol(o(e).symbol)}function o(n){if(189===n.kind)return n.getSourceFile();var t=n.parent;return 277===t.kind?t:(e.Debug.assert(243===t.kind),e.cast(t.parent,s))}function s(e){return 242===e.kind&&9===e.name.kind}function l(e){return 257===e.moduleReference.kind&&9===e.moduleReference.expression.kind}n.createImportTracker=function(n,a,s,c){var u=function(n,t,a){for(var i=e.createMap(),o=0,s=n;o<s.length;o++){var l=s[o];a&&a.throwIfCancellationRequested(),r(l,function(n,r){var a=t.getSymbolAtLocation(r);if(a){var o=e.getSymbolId(a).toString(),s=i.get(o);s||i.set(o,s=[]),s.push(n)}})}return i}(n,s,c);return function(r,d,m){var p=function(n,r,a,s,l,c){var u=s.exportingModuleSymbol,d=s.exportKind,m=e.nodeSeenTracker(),p=e.nodeSeenTracker(),f=[],g=!!u.globalExports,_=g?void 0:[];return function n(t){var r=h(t);if(r)for(var a=0,s=r;a<s.length;a++){var u=s[a];if(m(u))switch(c&&c.throwIfCancellationRequested(),u.kind){case 189:if(!g){var p=u.parent;if(2===d&&235===p.kind){var _=p.name;if(71===_.kind){f.push(_);break}}y(u.getSourceFile())}break;case 71:break;case 246:v(u,u.name,e.hasModifier(u,1),!1);break;case 247:f.push(u);var b=u.importClause&&u.importClause.namedBindings;b&&249===b.kind?v(u,b.name,!1,!0):!g&&e.isDefaultImport(u)&&y(o(u));break;case 253:u.exportClause?f.push(u):n(i(u,l));break;case 181:f.push(u);break;default:e.Debug.assertNever(u,\"Unexpected import kind: \"+e.Debug.showSyntaxKind(u))}}}(u),{directImports:f,indirectUsers:function(){if(g)return n;for(var t=0,a=u.declarations;t<a.length;t++){var i=a[t];e.isExternalModuleAugmentation(i)&&r.has(i.getSourceFile().fileName)&&y(i)}return _.map(e.getSourceFileOfNode)}()};function v(n,r,a,i){if(2===d)i||f.push(n);else if(!g){var s=o(n);e.Debug.assert(277===s.kind||242===s.kind),a||function(n,r,a){var i=a.getSymbolAtLocation(r);return!!t(n,function(n){if(e.isExportDeclaration(n)){var t=n.exportClause,r=n.moduleSpecifier;return!r&&t&&t.elements.some(function(e){return a.getExportSpecifierLocalTargetSymbol(e)===i})}})}(s,r,l)?function n(t){if(y(t)){var r=l.getMergedSymbol(t.symbol);e.Debug.assert(!!(1536&r.flags));var a=h(r);if(a)for(var i=0,s=a;i<s.length;i++){var c=s[i];n(o(c))}}}(s):y(s)}}function y(n){e.Debug.assert(!g);var t=p(n);return t&&_.push(n),t}function h(n){return a.get(e.getSymbolId(n).toString())}}(n,a,u,d,s,c),f=p.directImports,g=p.indirectUsers;return yn({indirectUsers:g},function(n,t,r,a,i){var o=[],s=[];function c(e,n){o.push([e,n])}if(n)for(var u=0,d=n;u<d.length;u++){var m=d[u];p(m)}return{importSearches:o,singleReferences:s};function p(n){if(246!==n.kind)if(71!==n.kind)if(181!==n.kind){if(9===n.moduleSpecifier.kind)if(253!==n.kind){var o=n.importClause||{name:void 0,namedBindings:void 0},u=o.name,d=o.namedBindings;if(d)switch(d.kind){case 249:f(d.name);break;case 250:0!==r&&1!==r||g(d);break;default:e.Debug.assertNever(d)}if(u&&(1===r||2===r)&&(!i||u.escapedText===e.symbolEscapedNameNoDefault(t))){var m=a.getSymbolAtLocation(u);c(u,m)}}else g(n.exportClause)}else n.qualifier?e.isIdentifier(n.qualifier)&&n.qualifier.escapedText===e.symbolName(t)&&s.push(n.qualifier):2===r&&s.push(n.argument.literal);else f(n);else l(n)&&f(n.name)}function f(e){2!==r||i&&!_(e.escapedText)||c(e,a.getSymbolAtLocation(e))}function g(e){if(e)for(var n=0,r=e.elements;n<r.length;n++){var o=r[n],l=o.name,u=o.propertyName;if(_((u||l).escapedText))if(u)s.push(u),i&&l.escapedText!==t.escapedName||c(l,a.getSymbolAtLocation(l));else{var d=255===o.kind&&o.propertyName?a.getExportSpecifierLocalTargetSymbol(o):a.getSymbolAtLocation(l);c(l,d)}}}function _(e){return e===t.escapedName||0!==r&&\"default\"===e}}(f,r,d.exportKind,s,m))}},function(e){e[e.Named=0]=\"Named\",e[e.Default=1]=\"Default\",e[e.ExportEquals=2]=\"ExportEquals\"}(n.ExportKind||(n.ExportKind={})),function(e){e[e.Import=0]=\"Import\",e[e.Export=1]=\"Export\"}(n.ImportExport||(n.ImportExport={})),n.findModuleReferences=function(e,n,t){for(var a=[],i=e.getTypeChecker(),o=0,s=n;o<s.length;o++){var l=s[o],c=t.valueDeclaration;if(277===c.kind){for(var u=0,d=l.referencedFiles;u<d.length;u++){var m=d[u];e.getSourceFileFromReference(l,m)===c&&a.push({kind:\"reference\",referencingFile:l,ref:m})}for(var p=0,f=l.typeReferenceDirectives;p<f.length;p++){m=f[p];var g=e.getResolvedTypeReferenceDirectives().get(m.fileName);void 0!==g&&g.resolvedFileName===c.fileName&&a.push({kind:\"reference\",referencingFile:l,ref:m})}}r(l,function(e,n){i.getSymbolAtLocation(n)===t&&a.push({kind:\"import\",literal:n})})}return a},n.getImportOrExportSymbol=function(n,t,r,i){return i?o():o()||function(){var a=function(n){var t=n.parent;switch(t.kind){case 246:return t.name===n&&l(t)?{isNamedImport:!1}:void 0;case 251:return t.propertyName?void 0:{isNamedImport:!0};case 248:case 249:return e.Debug.assert(t.name===n),{isNamedImport:!1};default:return}}(n);if(a){var i=r.getImmediateAliasedSymbol(t);if(i){\"export=\"===(i=function(n,t){if(n.declarations)for(var r=0,a=n.declarations;r<a.length;r++){var i=a[r];if(e.isExportSpecifier(i)&&!i.propertyName&&!i.parent.parent.moduleSpecifier)return t.getExportSpecifierLocalTargetSymbol(i)}return n}(i,r)).escapedName&&(i=function(n,t){if(2097152&n.flags)return e.Debug.assertDefined(t.getImmediateAliasedSymbol(n));var r=n.valueDeclaration;return e.isExportAssignment(r)?e.Debug.assertDefined(r.expression.symbol):e.isBinaryExpression(r)?e.Debug.assertDefined(r.right.symbol):e.isSourceFile(r)?e.Debug.assertDefined(r.symbol):e.Debug.fail()}(i,r));var o=e.symbolEscapedNameNoDefault(i);return void 0===o||\"default\"===o||o===t.escapedName?yn({kind:0,symbol:i},a):void 0}}}();function o(){var a=n.parent,o=a.parent;if(t.exportSymbol)return 187===a.kind?t.declarations.some(function(e){return e===a})&&e.isBinaryExpression(o)?d(o,!1):void 0:s(t.exportSymbol,c(a));var l=function(e,n){if(235===e.kind){var t=e;return t.name!==n?void 0:272===t.parent.kind?void 0:217===t.parent.parent.kind?t.parent.parent:void 0}return e}(a,n);if(l&&e.hasModifier(l,1)){if(e.isImportEqualsDeclaration(l)&&l.moduleReference===n){if(i)return;return{kind:0,symbol:r.getSymbolAtLocation(l.name),isNamedImport:!1}}return s(t,c(l))}if(e.isExportAssignment(a))return u(a);if(e.isExportAssignment(o))return u(o);if(e.isBinaryExpression(a))return d(a,!0);if(e.isBinaryExpression(o))return d(o,!0);if(e.isJSDocTypedefTag(a))return s(t,0);function u(n){var r=e.Debug.assertDefined(n.symbol.parent,\"Expected export symbol to have a parent\"),a=n.isExportEquals?2:1;return{kind:1,symbol:t,exportInfo:{exportingModuleSymbol:r,exportKind:a}}}function d(n,a){var i;switch(e.getAssignmentDeclarationKind(n)){case 1:i=0;break;case 2:i=2;break;default:return}var o=a?r.getSymbolAtLocation(e.cast(n.left,e.isPropertyAccessExpression).name):t;return!o||1536&r.getMergedSymbol(o.parent).flags||e.Debug.fail(\"Special property assignment kind does not have a module as its parent. Assignment is \"+e.Debug.showSymbol(o)+\", parent is \"+e.Debug.showSymbol(o.parent)),o&&s(o,i)}}function s(e,n){var t=a(e,n,r);return t&&{kind:1,symbol:e,exportInfo:t}}function c(n){return e.hasModifier(n,512)?1:0}},n.getExportInfo=a}(e.FindAllReferences||(e.FindAllReferences={}))}(Tn||(Tn={})),function(e){!function(n){function t(e,n){return{type:\"node\",node:e.name||e,isInString:n}}function r(t,r,i,o,s,l,c){return void 0===l&&(l={}),void 0===c&&(c=e.arrayToSet(o,function(e){return e.fileName})),a(n.Core.getReferencedSymbolsForNode(t,r,i,o,s,l,c))}function a(n){return n&&e.flatMap(n,function(e){return e.references})}function i(t,r,a){var i=n.Core.getIntersectingMeaningFromDeclarations(a,t),o=e.firstOrUndefined(t.declarations)||a,s=e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(r,t,o.getSourceFile(),o,o,i);return{displayParts:s.displayParts,kind:s.symbolKind}}function o(n){if(\"span\"===n.type)return{textSpan:n.textSpan,fileName:n.fileName,isWriteAccess:!1,isDefinition:!1};var t=n.node,r=n.isInString,a=t.getSourceFile();return{fileName:a.fileName,textSpan:s(t,a),isWriteAccess:l(t),isDefinition:79===t.kind||!!e.getDeclarationFromName(t)||e.isLiteralComputedPropertyDeclarationName(t),isInString:r}}function s(n,t){var r=n.getStart(t),a=n.getEnd();return 9===n.kind&&(r+=1,a-=1),e.createTextSpanFromBounds(r,a)}function l(n){var t=e.getDeclarationFromName(n);return!!t&&function(n){if(4194304&n.flags)return!0;switch(n.kind){case 202:case 184:case 238:case 207:case 79:case 241:case 276:case 255:case 248:case 246:case 251:case 239:case 295:case 302:case 265:case 242:case 245:case 249:case 149:case 274:case 240:case 148:return!0;case 273:return!e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent);case 237:case 194:case 155:case 154:case 156:case 157:return!!n.body;case 235:case 152:return!!n.initializer||e.isCatchClause(n.parent);case 153:case 151:case 303:case 297:return!1;default:return e.Debug.failBadSyntaxKind(n)}}(t)||79===n.kind||e.isWriteAccess(n)}!function(e){e[e.Symbol=0]=\"Symbol\",e[e.Label=1]=\"Label\",e[e.Keyword=2]=\"Keyword\",e[e.This=3]=\"This\",e[e.String=4]=\"String\"}(n.DefinitionKind||(n.DefinitionKind={})),n.nodeEntry=t,n.findReferencedSymbols=function(t,r,a,l,c){var u=e.getTouchingPropertyName(l,c),d=n.Core.getReferencedSymbolsForNode(c,u,t,a,r),m=t.getTypeChecker();return d&&d.length?e.mapDefined(d,function(n){var t=n.definition,a=n.references;return t&&{definition:m.runWithCancellationToken(r,function(n){return function(n,t,r){var a=function(){switch(n.type){case 0:var a=n.symbol,o=i(a,t,r),s=o.displayParts,l=o.kind,c=s.map(function(e){return e.text}).join(\"\");return{node:a.declarations?e.getNameOfDeclaration(e.first(a.declarations))||e.first(a.declarations):r,name:c,kind:l,displayParts:s};case 1:var u=n.node;return{node:u,name:u.text,kind:\"label\",displayParts:[e.displayPart(u.text,e.SymbolDisplayPartKind.text)]};case 2:var d=n.node,m=e.tokenToString(d.kind);return{node:d,name:m,kind:\"keyword\",displayParts:[{text:m,kind:\"keyword\"}]};case 3:var p=n.node,a=t.getSymbolAtLocation(p),f=a&&e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(t,a,p.getSourceFile(),e.getContainerNode(p),p).displayParts||[e.textPart(\"this\")];return{node:p,name:\"this\",kind:\"var\",displayParts:f};case 4:var g=n.node;return{node:g,name:g.text,kind:\"var\",displayParts:[e.displayPart(e.getTextOfNode(g),e.SymbolDisplayPartKind.stringLiteral)]};default:return e.Debug.assertNever(n)}}(),o=a.node,l=a.name,c=a.kind,u=a.displayParts,d=o.getSourceFile();return{containerKind:\"\",containerName:\"\",fileName:d.fileName,kind:c,name:l,textSpan:s(e.isComputedPropertyName(o)?o.expression:o,d),displayParts:u}}(t,n,u)}),references:a.map(o)}}):void 0},n.getImplementationsAtPosition=function(a,o,l,c,u){var d=function(a,i,o,s,l){if(277!==s.kind){var c=a.getTypeChecker();if(274===s.parent.kind){var u=[];return n.Core.getReferenceEntriesForShorthandPropertyAssignment(s,c,function(e){return u.push(t(e))}),u}if(97===s.kind||e.isSuperProperty(s.parent)){var d=c.getSymbolAtLocation(s);return d.valueDeclaration&&[t(d.valueDeclaration)]}return r(l,s,a,o,i,{implementations:!0})}}(a,o,l,e.getTouchingPropertyName(c,u),u),m=a.getTypeChecker();return e.map(d,function(n){return function(n,t){if(\"node\"===n.type){var r=n.node,a=r.getSourceFile();return yn({textSpan:s(r,a),fileName:a.fileName},function(n,t){var r=t.getSymbolAtLocation(e.isDeclaration(n)&&n.name?n.name:n);return r?i(r,t,n):186===n.kind?{kind:\"interface\",displayParts:[e.punctuationPart(19),e.textPart(\"object literal\"),e.punctuationPart(20)]}:207===n.kind?{kind:\"local class\",displayParts:[e.punctuationPart(19),e.textPart(\"anonymous local class\"),e.punctuationPart(20)]}:{kind:e.getNodeKind(n),displayParts:[]}}(r,t))}var o=n.textSpan,l=n.fileName;return{textSpan:o,fileName:l,kind:\"\",displayParts:[]}}(n,m)})},n.findReferencedEntries=function(t,r,i,s,l,c){return e.map(a(n.Core.getReferencedSymbolsForNode(l,s,t,i,r,c)),o)},n.getReferenceEntriesForNode=r,n.toHighlightSpan=function(e){if(\"span\"===e.type)return{fileName:e.fileName,span:{textSpan:e.textSpan,kind:\"reference\"}};var n=e.node,t=e.isInString,r=n.getSourceFile(),a=l(n),i={textSpan:s(n,r),kind:a?\"writtenReference\":\"reference\",isInString:t};return{fileName:r.fileName,span:i}}}(e.FindAllReferences||(e.FindAllReferences={}))}(Tn||(Tn={})),function(e){!function(n){!function(t){function r(t,r,a,i,o){e.Debug.assert(!!r.valueDeclaration);for(var s=e.mapDefined(n.findModuleReferences(t,i,r),function(n){if(\"import\"===n.kind){var t=n.literal.parent;if(e.isLiteralTypeNode(t)){var r=e.cast(t.parent,e.isImportTypeNode);if(a&&!r.qualifier)return}return{type:\"node\",node:n.literal}}return{type:\"span\",fileName:n.referencingFile.fileName,textSpan:e.createTextSpanFromRange(n.ref)}}),l=0,c=r.declarations;l<c.length;l++){var u=c[l];switch(u.kind){case 277:break;case 242:o.has(u.getSourceFile().fileName)&&s.push({type:\"node\",node:u.name});break;default:e.Debug.fail(\"Expected a module symbol to be declared by a SourceFile or ModuleDeclaration.\")}}return s.length?[{definition:{type:0,symbol:r},references:s}]:e.emptyArray}var a;t.getReferencedSymbolsForNode=function(t,a,s,l,d,p,g){if(void 0===p&&(p={}),void 0===g&&(g=e.arrayToSet(l,function(e){return e.fileName})),e.isSourceFile(a)){var v=e.GoToDefinition.getReferenceAtPosition(a,t,s),y=v&&s.getTypeChecker().getMergedSymbol(v.file.symbol);return y&&r(s,y,!1,l,g)}if(!p.implementations){var S=function(t,r,a){if(e.isTypeKeyword(t.kind))return function(t,r,a){var i=e.flatMap(t,function(t){return a.throwIfCancellationRequested(),e.mapDefined(m(t,e.tokenToString(r),t),function(e){return e.kind===r?n.nodeEntry(e):void 0})});return i.length?[{definition:{type:2,node:i[0].node},references:i}]:void 0}(r,t.kind,a);if(e.isJumpStatementTarget(t)){var i=e.getTargetLabel(t.parent,t.text);return i&&f(i.parent,i)}return e.isLabelOfLabeledStatement(t)?f(t.parent,t):e.isThis(t)?function(t,r,a){var i=e.getThisContainer(t,!1),o=32;switch(i.kind){case 154:case 153:if(e.isObjectLiteralMethod(i))break;case 152:case 151:case 155:case 156:case 157:o&=e.getModifierFlags(i),i=i.parent;break;case 277:if(e.isExternalModule(i))return;case 237:case 194:break;default:return}var s=e.flatMap(277===i.kind?r:[i.getSourceFile()],function(n){return a.throwIfCancellationRequested(),m(n,\"this\",e.isSourceFile(i)?n:i).filter(function(n){if(!e.isThis(n))return!1;var t=e.getThisContainer(n,!1);switch(i.kind){case 194:case 237:return i.symbol===t.symbol;case 154:case 153:return e.isObjectLiteralMethod(i)&&i.symbol===t.symbol;case 207:case 238:return t.parent&&i.symbol===t.parent.symbol&&(32&e.getModifierFlags(t))===o;case 277:return 277===t.kind&&!e.isExternalModule(t)}})}).map(function(e){return n.nodeEntry(e)});return[{definition:{type:3,node:e.firstDefined(s,function(n){return e.isParameter(n.node.parent)?n.node:void 0})||t},references:s}]}(t,r,a):97===t.kind?function(t){var r=e.getSuperContainer(t,!1);if(r){var a=32;switch(r.kind){case 152:case 151:case 154:case 153:case 155:case 156:case 157:a&=e.getModifierFlags(r),r=r.parent;break;default:return}var i=r.getSourceFile(),o=e.mapDefined(m(i,\"super\",r),function(t){if(97===t.kind){var i=e.getSuperContainer(t,!1);return i&&(32&e.getModifierFlags(i))===a&&i.parent.symbol===r.symbol?n.nodeEntry(t):void 0}});return[{definition:{type:0,symbol:r.symbol},references:o}]}}(t):void 0}(a,l,d);if(S)return S}var L=s.getTypeChecker(),x=L.getSymbolAtLocation(a);if(!x)return!p.implementations&&e.isStringLiteral(a)?function(t,r,a){var i=e.flatMap(r,function(r){return a.throwIfCancellationRequested(),e.mapDefined(m(r,t.text),function(r){return e.isStringLiteral(r)&&r.text===t.text?n.nodeEntry(r,!0):void 0})});return[{definition:{type:4,node:t},references:i}]}(a,l,d):void 0;var A=e.emptyArray,C=function(n){return 1536&n.flags?e.find(n.declarations,e.isSourceFile):void 0}(x),D=a;if(C){var k=x.exports.get(\"export=\");if(A=r(s,x,!!k,l,g),!k||!g.has(C.fileName))return A;x=e.skipAlias(k,L),D=void 0}return e.concatenate(A,function(n,t,r,a,s,l,d){n=t&&function(n,t,r){var a=t.parent;return e.isExportSpecifier(a)?h(t,n,a,r):e.isImportSpecifier(a)&&a.propertyName===t?r.getImmediateAliasedSymbol(n):e.firstDefined(n.declarations,function(a){return a.parent||e.Debug.fail(\"Unexpected symbol at \"+e.Debug.showSyntaxKind(t)+\": \"+e.Debug.showSymbol(n)),e.isTypeLiteralNode(a.parent)&&e.isUnionTypeNode(a.parent.parent)?r.getPropertyOfType(r.getTypeFromTypeNode(a.parent.parent),n.name):void 0})}(n,t,s)||n;var m=t?T(t,n):7,p=[],f=new i(r,a,t?function(n){switch(n.kind){case 123:return 1;case 71:if(e.isClassLike(n.parent))return e.Debug.assert(n.parent.name===n),2;default:return 0}}(t):0,s,l,m,d,p);if(t&&79===t.kind)b(t,n,f),o(t,n,{exportingModuleSymbol:e.Debug.assertDefined(n.parent,\"Expected export symbol to have a parent\"),exportKind:1},f);else{var g=f.createSearch(t,n,void 0,{allSearchSymbols:t?function(e,n,t,r){var a=[];return E(e,n,t,function(e,n,t){a.push(t||n||e)},function(){return!r}),a}(n,t,s,!!d.implementations):[n]}),v=function(n){var t=n.declarations,r=n.flags,a=n.parent,i=n.valueDeclaration;if(i&&(194===i.kind||207===i.kind))return i;if(t){if(8196&r){var o=e.find(t,function(n){return e.hasModifier(n,8)});return o?e.getAncestor(o,238):void 0}if(!u(n)){var s,l=a&&!(262144&n.flags);if(!l||e.isExternalModuleSymbol(a)&&!a.globalExports){for(var c=0,d=t;c<d.length;c++){var m=d[c],p=e.getContainerNode(m);if(s&&s!==p)return;if(!p||277===p.kind&&!e.isExternalOrCommonJsModule(p))return;s=p}return l?s.getSourceFile():s}}}}(n);if(v)_(v,v.getSourceFile(),g,f,!(e.isSourceFile(v)&&!e.contains(r,v)));else for(var y=0,S=f.sourceFiles;y<S.length;y++){var L=S[y];f.cancellationToken.throwIfCancellationRequested(),c(L,g,f)}}return p}(x,D,l,g,L,d,p))},function(e){e[e.None=0]=\"None\",e[e.Constructor=1]=\"Constructor\",e[e.Class=2]=\"Class\"}(a||(a={}));var i=function(){function t(n,t,r,a,i,o,s,l){this.sourceFiles=n,this.sourceFilesSet=t,this.specialSearchKind=r,this.checker=a,this.cancellationToken=i,this.searchMeaning=o,this.options=s,this.result=l,this.inheritsFromCache=e.createMap(),this.markSeenContainingTypeReference=e.nodeSeenTracker(),this.markSeenReExportRHS=e.nodeSeenTracker(),this.symbolIdToReferences=[],this.sourceFileToSeenSymbols=[]}return t.prototype.includesSourceFile=function(e){return this.sourceFilesSet.has(e.fileName)},t.prototype.getImportSearches=function(e,t){return this.importTracker||(this.importTracker=n.createImportTracker(this.sourceFiles,this.sourceFilesSet,this.checker,this.cancellationToken)),this.importTracker(e,t,!!this.options.isForRename)},t.prototype.createSearch=function(n,t,r,a){void 0===a&&(a={});var i=a.text,o=void 0===i?e.stripQuotes(e.unescapeLeadingUnderscores((e.getLocalSymbolForExportDefault(t)||t).escapedName)):i,s=a.allSearchSymbols,l=void 0===s?[t]:s;return{symbol:t,comingFrom:r,text:o,escapedText:e.escapeLeadingUnderscores(o),parents:this.options.implementations&&n?function(n,t,r){var a=e.isRightSideOfPropertyAccess(n)?n.parent:void 0,i=a&&r.getTypeAtLocation(a.expression),o=e.mapDefined(i&&(i.isUnionOrIntersection()?i.types:i.symbol===t.parent?void 0:[i]),function(e){return e.symbol&&96&e.symbol.flags?e.symbol:void 0});return 0===o.length?void 0:o}(n,t,this.checker):void 0,allSearchSymbols:l,includes:function(n){return e.contains(l,n)}}},t.prototype.referenceAdder=function(t){var r=e.getSymbolId(t),a=this.symbolIdToReferences[r];return a||(a=this.symbolIdToReferences[r]=[],this.result.push({definition:{type:0,symbol:t},references:a})),function(e){return a.push(n.nodeEntry(e))}},t.prototype.addStringOrCommentReference=function(e,n){this.result.push({definition:void 0,references:[{type:\"span\",fileName:e,textSpan:n}]})},t.prototype.markSearchedSymbols=function(n,t){for(var r=e.getNodeId(n),a=this.sourceFileToSeenSymbols[r]||(this.sourceFileToSeenSymbols[r]=e.createMap()),i=!1,o=0,s=t;o<s.length;o++){var l=s[o];i=e.addToSeen(a,e.getSymbolId(l))||i}return i},t}();function o(e,n,t,r){var a=r.getImportSearches(n,t),i=a.importSearches,o=a.singleReferences,l=a.indirectUsers;if(o.length)for(var u=r.referenceAdder(n),d=0,m=o;d<m.length;d++){var p=m[d];s(p,r)&&u(p)}for(var f=0,_=i;f<_.length;f++){var v=_[f],y=v[0],h=v[1];g(y.getSourceFile(),r.createSearch(y,h,1),r)}if(l.length){var b=void 0;switch(t.exportKind){case 0:b=r.createSearch(e,n,1);break;case 1:b=r.options.isForRename?void 0:r.createSearch(e,n,1,{text:\"default\"})}if(b)for(var E=0,T=l;E<T.length;E++){c(T[E],b,r)}}}function s(n,t){return!!v(n,t)&&(!t.options.isForRename||!!e.isIdentifier(n)&&!((e.isExportSpecifier(n.parent)||e.isImportSpecifier(n.parent))&&\"default\"===n.escapedText))}function l(e,n){for(var t=0,r=e.declarations;t<r.length;t++){var a=r[t],i=a.getSourceFile();g(i,n.createSearch(a,e,0),n,n.includesSourceFile(i))}}function c(n,t,r){void 0!==e.getNameTable(n).get(t.escapedText)&&g(n,t,r)}function u(n){var t=e.getDeclarationOfKind(n,184);if(t&&182===t.parent.kind&&e.isIdentifier(t.name)&&!t.propertyName)return t}function d(n,t,r,a){var i=t.getSymbolAtLocation(n);if(i)for(var o=0,s=m(r,i.name);o<s.length;o++){var l=s[o];if(e.isIdentifier(l)&&l!==n&&l.escapedText===n.escapedText){var c=t.getSymbolAtLocation(l);if(c===i||t.getShorthandAssignmentValueSymbol(l.parent)===i||e.isExportSpecifier(l.parent)&&h(l,c,l.parent,t)===i){var u=a(l);if(u)return u}}}}function m(n,t,r){return void 0===r&&(r=n),p(n,t,r).map(function(t){return e.getTouchingPropertyName(n,t)})}function p(n,t,r){void 0===r&&(r=n);var a=[];if(!t||!t.length)return a;for(var i=n.text,o=i.length,s=t.length,l=i.indexOf(t,r.pos);l>=0&&!(l>r.end);){var c=l+s;0!==l&&e.isIdentifierPart(i.charCodeAt(l-1),6)||c!==o&&e.isIdentifierPart(i.charCodeAt(c),6)||a.push(l),l=i.indexOf(t,l+s+1)}return a}function f(t,r){var a=t.getSourceFile(),i=r.text,o=e.mapDefined(m(a,i,t),function(t){return t===r||e.isJumpStatementTarget(t)&&e.getTargetLabel(t,i)===r?n.nodeEntry(t):void 0});return[{definition:{type:1,node:r},references:o}]}function g(e,n,t,r){return void 0===r&&(r=!0),t.cancellationToken.throwIfCancellationRequested(),_(e,e,n,t,r)}function _(e,n,t,r,a){if(r.markSearchedSymbols(n,t.allSearchSymbols))for(var i=0,o=p(n,t.text,e);i<o.length;i++){y(n,o[i],t,r,a)}}function v(n,t){return!!(e.getMeaningFromLocation(n)&t.searchMeaning)}function y(t,r,a,i,s){var c=e.getTouchingPropertyName(t,r);if(function(n,t){switch(n.kind){case 71:return n.text.length===t.length;case 9:var r=n;return(e.isLiteralNameOfPropertyDeclarationOrIndexAccess(r)||e.isNameOfModuleDeclaration(n)||e.isExpressionOfExternalModuleImportEqualsDeclaration(n))&&r.text.length===t.length;case 8:return e.isLiteralNameOfPropertyDeclarationOrIndexAccess(n)&&n.text.length===t.length;case 79:return\"default\".length===t.length;default:return!1}}(c,a.text)){if(v(c,i)){var u=i.checker.getSymbolAtLocation(c);if(u){var d=c.parent;if(!e.isImportSpecifier(d)||d.propertyName!==c){if(e.isExportSpecifier(d))return e.Debug.assert(71===c.kind),void function(t,r,a,i,s,c){var u=a.parent,d=a.propertyName,m=a.name,p=u.parent,f=h(t,r,a,s.checker);if(!i.includes(f))return;d?t===d?(p.moduleSpecifier||y(),c&&!s.options.isForRename&&s.markSeenReExportRHS(m)&&b(m,r,s)):s.markSeenReExportRHS(t)&&y():s.options.isForRename&&\"default\"===m.escapedText||y();if(t!==d||!s.options.isForRename){var g=79===t.originalKeywordKind?1:0,_=n.getExportInfo(r,g,s.checker);if(!_)return e.Debug.fail();o(t,r,_,s)}if(1!==i.comingFrom&&p.moduleSpecifier&&!d){var v=s.checker.getExportSpecifierLocalTargetSymbol(a);v&&l(v,s)}function y(){c&&b(t,f,s)}}(c,u,d,a,i,s);var m=function(n,t,r,a){var i=a.checker;return E(t,r,i,function(t,r,a){return n.includes(a||r||t)?!r||6&e.getCheckFlags(t)?t:r:void 0},function(t){return!(n.parents&&!n.parents.some(function(n){return function n(t,r,a,i){if(t===r)return!0;var o=e.getSymbolId(t)+\",\"+e.getSymbolId(r);var s=a.get(o);if(void 0!==s)return s;a.set(o,!1);var l=t.declarations.some(function(t){return e.getAllSuperTypeNodes(t).some(function(e){var t=i.getTypeAtLocation(e);return!!t&&!!t.symbol&&n(t.symbol,r,a,i)})});a.set(o,l);return l}(t.parent,n,a.inheritsFromCache,i)}))})}(a,u,c,i);if(m){switch(i.specialSearchKind){case 0:s&&b(c,m,i);break;case 1:!function(n,t,r,a){e.isNewExpressionTarget(n)&&b(n,r.symbol,a);var i=function(){return a.referenceAdder(r.symbol)};if(e.isClassLike(n.parent))e.Debug.assert(79===n.kind||n.parent.name===n),function(n,t,r){for(var a=0,i=n.members.get(\"__constructor\").declarations;a<i.length;a++){var o=i[a],s=e.findChildOfKind(o,123,t);e.Debug.assert(155===o.kind&&!!s),r(s)}n.exports.forEach(function(n){var t=n.valueDeclaration;if(t&&154===t.kind){var a=t.body;a&&L(a,99,function(n){e.isNewExpressionTarget(n)&&r(n)})}})}(r.symbol,t,i());else{var o=(s=n,e.tryGetClassExtendingExpressionWithTypeArguments(e.climbPastPropertyAccess(s).parent));o&&function(n,t){var r=n.symbol.members.get(\"__constructor\");if(!r)return;for(var a=0,i=r.declarations;a<i.length;a++){var o=i[a];e.Debug.assert(155===o.kind);var s=o.body;s&&L(s,97,function(n){e.isCallExpressionTarget(n)&&t(n)})}}(o,i())}var s}(c,t,a,i);break;case 2:!function(n,t,r){b(n,t.symbol,r);var a=n.parent;if(r.options.isForRename||!e.isClassLike(a))return;e.Debug.assert(a.name===n);for(var i=r.referenceAdder(t.symbol),o=0,s=a.members;o<s.length;o++){var l=s[o];e.isMethodOrAccessor(l)&&e.hasModifier(l,32)&&(l.body&&l.body.forEachChild(function n(t){99===t.kind?i(t):e.isFunctionLike(t)||e.isClassLike(t)||t.forEachChild(n)}))}}(c,a,i);break;default:e.Debug.assertNever(i.specialSearchKind)}!function(e,t,r,a){var i=n.getImportOrExportSymbol(e,t,a.checker,1===r.comingFrom);if(!i)return;var s=i.symbol;0===i.kind?a.options.isForRename&&!i.isNamedImport||l(s,a):o(e,s,i.exportInfo,a)}(c,u,a,i)}else!function(n,t,r){var a=n.flags,i=n.valueDeclaration,o=r.checker.getShorthandAssignmentValueSymbol(i),s=i&&e.getNameOfDeclaration(i);33554432&a||!s||!t.includes(o)||b(s,o,r)}(u,a,i)}}}}else!i.options.implementations&&(i.options.findInStrings&&e.isInString(t,r)||i.options.findInComments&&e.isInNonReferenceComment(t,r))&&i.addStringOrCommentReference(t.fileName,e.createTextSpan(r,a.text.length))}function h(n,t,r,a){return function(n,t){var r=t.parent,a=t.propertyName,i=t.name;return e.Debug.assert(a===n||i===n),a?a===n:!r.parent.moduleSpecifier}(n,r)&&a.getExportSpecifierLocalTargetSymbol(r)||t}function b(n,t,r){var a=r.referenceAdder(t);r.options.implementations?function(n,t,r){if(e.isDeclarationName(n)&&(a=n.parent,4194304&a.flags||(e.isVariableLike(a)?e.hasInitializer(a):e.isFunctionLikeDeclaration(a)?a.body:e.isClassLike(a)||e.isModuleOrEnumDeclaration(a))))return void t(n);var a;if(71!==n.kind)return;274===n.parent.kind&&S(n,r.checker,t);var i=function n(t){return e.isIdentifier(t)||e.isPropertyAccessExpression(t)?n(t.parent):e.isExpressionWithTypeArguments(t)?e.tryCast(t.parent.parent,e.isClassLike):void 0}(n);if(i)return void t(i);var o=e.findAncestor(n,function(n){return!e.isQualifiedName(n.parent)&&!e.isTypeNode(n.parent)&&!e.isTypeElement(n.parent)}),s=o.parent;if(e.hasType(s)&&s.type===o&&r.markSeenContainingTypeReference(s))if(e.hasInitializer(s))c(s.initializer);else if(e.isFunctionLike(s)&&s.body){var l=s.body;216===l.kind?e.forEachReturnStatement(l,function(e){e.expression&&c(e.expression)}):c(l)}else e.isAssertionExpression(s)&&c(s.expression);function c(e){(function e(n){switch(n.kind){case 193:return e(n.expression);case 195:case 194:case 186:case 207:case 185:return!0;default:return!1}})(e)&&t(e)}}(n,a,r):a(n)}function E(n,t,r,a,i){var o=e.getContainingObjectLiteralElement(t);if(o){var s=r.getContextualType(o.parent),l=s&&e.firstDefined(e.getPropertySymbolsFromContextualType(o,r,s,!0),v);if(l)return l;var c=function(n,t){return e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent.parent)?t.getPropertySymbolOfDestructuringAssignment(n):void 0}(t,r),d=c&&a(c);if(d)return d;var m=r.getShorthandAssignmentValueSymbol(t.parent),p=m&&a(m);if(p)return p}var f=v(n);if(f)return f;if(n.valueDeclaration&&e.isParameterPropertyDeclaration(n.valueDeclaration)){var g=r.getSymbolsOfParameterPropertyDeclaration(e.cast(n.valueDeclaration,e.isParameter),n.name);return e.Debug.assert(2===g.length&&!!(1&g[0].flags)&&!!(4&g[1].flags)),v(1&n.flags?g[1]:g[0])}var _=function(n,t){var r=u(n);return r&&e.getPropertySymbolFromBindingElement(t,r)}(n,r);return _&&v(_);function v(n){return e.firstDefined(r.getRootSymbols(n),function(t){return a(n,t)||(t.parent&&96&t.parent.flags&&i(t)?e.getPropertySymbolsFromBaseTypes(t.parent,t.name,r,function(e){return a(n,t,e)}):void 0)})}}function T(n,t){var r=e.getMeaningFromLocation(n),a=t.declarations;if(a){var i=void 0;do{i=r;for(var o=0,s=a;o<s.length;o++){var l=s[o],c=e.getMeaningFromDeclaration(l);c&r&&(r|=c)}}while(r!==i)}return r}function S(n,t,r){var a=t.getSymbolAtLocation(n),i=t.getShorthandAssignmentValueSymbol(a.valueDeclaration);if(i)for(var o=0,s=i.getDeclarations();o<s.length;o++){var l=s[o];1&e.getMeaningFromDeclaration(l)&&r(l)}}function L(n,t,r){e.forEachChild(n,function(e){e.kind===t&&r(e),L(e,t,r)})}t.eachExportReference=function(t,r,a,i,o,s,l,c){for(var u=n.createImportTracker(t,e.arrayToSet(t,function(e){return e.fileName}),r,a)(i,{exportKind:l?1:0,exportingModuleSymbol:o},!1),d=u.importSearches,p=u.indirectUsers,f=0,g=d;f<g.length;f++)c(g[f][0]);for(var _=0,v=p;_<v.length;_++)for(var y=0,h=m(v[_],l?\"default\":s);y<h.length;y++){var b=h[y];e.isIdentifier(b)&&r.getSymbolAtLocation(b)===i&&c(b)}},t.isSymbolReferencedInFile=function(e,n,t){return d(e,n,t,function(){return!0})||!1},t.eachSymbolReferenceInFile=d,t.eachSignatureCall=function(n,t,r,a){if(n.name&&e.isIdentifier(n.name))for(var i=e.Debug.assertDefined(r.getSymbolAtLocation(n.name)),o=0,s=t;o<s.length;o++)for(var l=0,c=m(s[o],i.name);l<c.length;l++){var u=c[l];if(e.isIdentifier(u)&&u!==n.name&&u.escapedText===n.name.escapedText){var d=e.climbPastPropertyAccess(u),p=d.parent;if(e.isCallExpression(p)&&p.expression===d){var f=r.getSymbolAtLocation(u);f&&r.getRootSymbols(f).some(function(e){return e===i})&&a(p)}}}},t.getIntersectingMeaningFromDeclarations=T,t.getReferenceEntriesForShorthandPropertyAssignment=S}(n.Core||(n.Core={}))}(e.FindAllReferences||(e.FindAllReferences={}))}(Tn||(Tn={})),function(e){function n(n,r,a,i){var o=a(n);return function(n){var s=i&&i.tryGetOriginalLocation({fileName:n,position:0}),l=function(n){if(a(n)===o)return r;var t=e.tryRemoveDirectoryPrefix(n,o,a);return void 0===t?void 0:r+\"/\"+t}(s?s.fileName:n);return s?void 0===l?void 0:function(n,r,a,i){var o=e.getRelativePathFromFile(n,r,i);return t(e.getDirectoryPath(a),o)}(s.fileName,l,n,a):l}}function t(n,t){return e.ensurePathIsNonModuleName(function(n,t){return e.normalizePath(e.combinePaths(n,t))}(n,t))}function r(n,t,r){return a(function(e){var n=t(e);return void 0!==n&&r.fileExists(n)?{newFileName:n,updated:!0}:void 0})||a(function(e){var n=t(e);return r.fileExists(e)?void 0!==n?{newFileName:n,updated:!0}:{newFileName:e,updated:!1}:void 0});function a(t){return n&&(n.resolvedModule&&t(n.resolvedModule.resolvedFileName)||e.firstDefined(n.failedLookupLocations,t))}}function a(n,t){return e.createRange(n.getStart(t)+1,n.end-1)}function i(n,t){if(e.isObjectLiteralExpression(n))for(var r=0,a=n.properties;r<a.length;r++){var i=a[r];e.isPropertyAssignment(i)&&e.isStringLiteral(i.name)&&t(i,i.name.text)}}e.getEditsForFileRename=function(o,s,l,c,u,d,m){var p=e.hostUsesCaseSensitiveFileNames(c),f=e.createGetCanonicalFileName(p),g=n(s,l,f,m),_=n(l,s,f,m);return e.textChanges.ChangeTracker.with({host:c,formatContext:u},function(n){!function(n,r,o,s,l,c){var u=n.getCompilerOptions().configFile;if(u){var d=e.getDirectoryPath(u.fileName),m=e.getTsConfigObjectLiteralExpression(u);m&&i(m,function(n,t){switch(t){case\"files\":case\"include\":case\"exclude\":var a=p(n);if(!a&&\"include\"===t&&e.isArrayLiteralExpression(n.initializer)){var o=e.mapDefined(n.initializer.elements,function(n){return e.isStringLiteral(n)?n.text:void 0}),m=e.getFileMatcherPatterns(d,[],o,c,l);e.getRegexFromPattern(e.Debug.assertDefined(m.includeFilePattern),c).test(s)||r.insertNodeAfter(u,e.last(n.initializer.elements),e.createStringLiteral(g(s)))}break;case\"compilerOptions\":i(n.initializer,function(n,t){var r=e.getOptionFromName(t);r&&(r.isFilePath||\"list\"===r.type&&r.element.isFilePath)?p(n):\"paths\"===t&&i(n.initializer,function(n){if(e.isArrayLiteralExpression(n.initializer))for(var t=0,r=n.initializer.elements;t<r.length;t++){var a=r[t];f(a)}})})}})}function p(n){for(var t=e.isArrayLiteralExpression(n.initializer)?n.initializer.elements:[n.initializer],r=!1,a=0,i=t;a<i.length;a++){var o=i[a];r=f(o)||r}return r}function f(n){if(!e.isStringLiteral(n))return!1;var i=t(d,n.text),s=o(i);return void 0!==s&&(r.replaceRangeWithText(u,a(n,u),g(s)),!0)}function g(n){return e.getRelativePathFromDirectory(d,n,!c)}}(o,n,g,l,c.getCurrentDirectory(),p),function(n,i,o,s,l,c){for(var u=n.getSourceFiles(),d=function(d){var m=o(d.path),p=void 0!==m?m:d.path,f=e.getDirectoryPath(p),g=s(d.fileName),_=g||d.fileName,v=e.getDirectoryPath(_),y=void 0!==m||void 0!==g;!function(n,t,r,i){for(var o=0,s=n.referencedFiles||e.emptyArray;o<s.length;o++){var l=s[o],c=r(l.fileName);void 0!==c&&c!==n.text.slice(l.pos,l.end)&&t.replaceRangeWithText(n,l,c)}for(var u=0,d=n.imports;u<d.length;u++){var m=d[u],c=i(m);void 0!==c&&c!==m.text&&t.replaceRangeWithText(n,a(m,n),c)}}(d,i,function(n){if(e.pathIsRelative(n)){var r=t(v,n),a=o(r);return void 0===a?void 0:e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(f,a,c))}},function(t){var a=n.getTypeChecker().getSymbolAtLocation(t);if(!a||!a.declarations.some(function(n){return e.isAmbientModule(n)})){var i=void 0!==g?r(e.resolveModuleName(t.text,_,n.getCompilerOptions(),l),o,l):function(n,t,a,i,o,s){if(n){var l=e.find(n.declarations,e.isSourceFile).fileName,c=s(l);return void 0===c?{newFileName:l,updated:!1}:{newFileName:c,updated:!0}}var u=o.resolveModuleNames?o.getResolvedModuleWithFailedLookupLocationsFromCache&&o.getResolvedModuleWithFailedLookupLocationsFromCache(t.text,a.fileName):i.getResolvedModuleWithFailedLookupLocationsFromCache(t.text,a.fileName);return r(u,s,o)}(a,t,d,n,l,o);return void 0!==i&&(i.updated||y&&e.pathIsRelative(t.text))?e.moduleSpecifiers.updateModuleSpecifier(n.getCompilerOptions(),p,i.newFileName,l,u,n.redirectTargetsMap,t.text):void 0}})},m=0,p=u;m<p.length;m++){var f=p[m];d(f)}}(o,n,g,_,c,f)})},e.getPathUpdater=n}(Tn||(Tn={})),function(e){!function(n){function t(n,t,a){var c,u,d=r(t,a,n);if(d)return[(c=d.fileName,u=d.file.fileName,{fileName:u,textSpan:e.createTextSpanFromBounds(0,0),kind:\"script\",name:c,containerName:void 0,containerKind:void 0})];var m=e.getTouchingPropertyName(t,a);if(m!==t){var p=m.parent;if(e.isJumpStatementTarget(m)){var f=e.getTargetLabel(m.parent,m.text);return f?[s(f,\"label\",m.text,void 0)]:void 0}var g=n.getTypeChecker(),_=function(e,n){var t=n.getSymbolAtLocation(e);if(t&&2097152&t.flags&&function(e,n){if(71!==e.kind)return!1;if(e.parent===n)return!0;switch(n.kind){case 248:case 246:return!0;case 251:return 250===n.parent.kind;default:return!1}}(e,t.declarations[0])){var r=n.getAliasedSymbol(t);if(r.declarations)return r}return t}(m,g);if(!_)return function(n,t){if(!e.isPropertyAccessExpression(n.parent)||n.parent.name!==n)return;var r=t.getTypeAtLocation(n.parent.expression);return e.mapDefined(r.isUnionOrIntersection()?r.types:[r],function(e){var n=t.getIndexInfoOfType(e,0);return n&&n.declaration&&l(t,n.declaration)})}(m,g);var v=function(n,t){var r=function(n){var t=function n(t){return e.isRightSideOfPropertyAccess(t)?n(t.parent):t}(n),r=t.parent;return r&&e.isCallLikeExpression(r)&&e.getInvokedExpression(r)===t?r:void 0}(t),a=r&&n.getResolvedSignature(r);return e.tryCast(a&&a.declaration,function(n){return e.isFunctionLike(n)&&!e.isFunctionTypeNode(n)})}(g,m);if(v&&(!e.isJsxOpeningLikeElement(m.parent)||!function(e){switch(e.kind){case 155:case 164:case 159:return!0;default:return!1}}(v))){var y=l(g,v);if(g.getRootSymbols(_).some(function(n){return function(n,t){return n===t.symbol||n===t.symbol.parent||!e.isCallLikeExpression(t.parent)&&n===t.parent.symbol}(n,v)})||_.declarations&&_.declarations.some(function(n){return e.isVariableDeclaration(n)&&!!n.initializer&&e.isRequireCall(n.initializer,!1)}))return[y];var h=i(g,_,m)||e.emptyArray;return 97===m.kind?[y].concat(h):h.concat([y])}if(274===m.parent.kind){var b=g.getShorthandAssignmentValueSymbol(_.valueDeclaration);return b?b.declarations.map(function(e){return o(e,g,b,m)}):[]}if(e.isPropertyName(m)&&e.isBindingElement(p)&&e.isObjectBindingPattern(p.parent)&&m===(p.propertyName||p.name)){var E=e.getNameFromPropertyName(m),T=g.getTypeAtLocation(p.parent);return void 0===E?e.emptyArray:e.flatMap(T.isUnion()?T.types:[T],function(e){var n=e.getProperty(E);return n&&i(g,n,m)})}var S=e.getContainingObjectLiteralElement(m);if(S){var L=S&&g.getContextualType(S.parent);if(L)return e.flatMap(e.getPropertySymbolsFromContextualType(S,g,L,!1),function(e){return i(g,e,m)})}return i(g,_,m)}}function r(n,t,r){var a=c(n.referencedFiles,t);if(a)return(s=e.tryResolveScriptReference(r,n,a))&&{fileName:a.fileName,file:s};var i=c(n.typeReferenceDirectives,t);if(i){var o=r.getResolvedTypeReferenceDirectives().get(i.fileName);return(s=o&&r.getSourceFile(o.resolvedFileName))&&{fileName:i.fileName,file:s}}var s,l=c(n.libReferenceDirectives,t);return l?(s=r.getLibFileFromReference(l))&&{fileName:l.fileName,file:s}:void 0}function a(n,t,r){return e.flatMap(!n.isUnion()||32&n.flags?[n]:n.types,function(e){return e.symbol&&i(t,e.symbol,r)})}function i(n,t,r){return function(){if(32&t.flags&&(e.isNewExpressionTarget(r)||123===r.kind)){var n=e.find(t.declarations,e.isClassLike)||e.Debug.fail(\"Expected declaration to have at least one class-like declaration\");return a(n.members,!0)}}()||(e.isCallOrNewExpressionTarget(r)||e.isNameOfFunctionDeclaration(r)?a(t.declarations,!1):void 0)||e.map(t.declarations,function(e){return o(e,n,t,r)});function a(a,i){if(a){var s=a.filter(i?e.isConstructorDeclaration:e.isFunctionLike);return s.length?[o(e.find(s,function(e){return!!e.body})||e.last(s),n,t,r)]:void 0}}}function o(n,t,r,a){var i=t.symbolToString(r);return s(n,e.SymbolDisplay.getSymbolKind(t,r,a),i,r.parent?t.symbolToString(r.parent,a):\"\")}function s(n,t,r,a){var i=e.getNameOfDeclaration(n)||n,o=i.getSourceFile();return{fileName:o.fileName,textSpan:e.createTextSpanFromNode(i,o),kind:t,name:r,containerKind:void 0,containerName:a}}function l(e,n){return o(n,e,n.symbol,n)}function c(n,t){return e.find(n,function(n){return e.textRangeContainsPositionInclusive(n,t)})}n.getDefinitionAtPosition=t,n.getReferenceAtPosition=r,n.getTypeDefinitionAtPosition=function(n,t,r){var i=e.getTouchingPropertyName(t,r);if(i!==t){var o=n.getSymbolAtLocation(i);if(o){var s=n.getTypeOfSymbolAtLocation(o,i),l=function(n,t,r){if(t.symbol===n||n.valueDeclaration&&t.symbol&&e.isVariableDeclaration(n.valueDeclaration)&&n.valueDeclaration.initializer===t.symbol.valueDeclaration){var a=t.getCallSignatures();if(1===a.length)return r.getReturnTypeOfSignature(e.first(a))}}(o,s,n),c=l&&a(l,n,i);return c&&0!==c.length?c:a(s,n,i)}}},n.getDefinitionAndBoundSpan=function(n,r,a){var i=t(n,r,a);if(i&&0!==i.length){var o=c(r.referencedFiles,a)||c(r.typeReferenceDirectives,a)||c(r.libReferenceDirectives,a);if(o)return{definitions:i,textSpan:e.createTextSpanFromRange(o)};var s=e.getTouchingPropertyName(r,a);return{definitions:i,textSpan:e.createTextSpan(s.getStart(),s.getWidth())}}},n.findReferenceInPosition=c}(e.GoToDefinition||(e.GoToDefinition={}))}(Tn||(Tn={})),function(e){!function(n){var t,r,a=[\"abstract\",\"access\",\"alias\",\"argument\",\"async\",\"augments\",\"author\",\"borrows\",\"callback\",\"class\",\"classdesc\",\"constant\",\"constructor\",\"constructs\",\"copyright\",\"default\",\"deprecated\",\"description\",\"emits\",\"enum\",\"event\",\"example\",\"exports\",\"extends\",\"external\",\"field\",\"file\",\"fileoverview\",\"fires\",\"function\",\"generator\",\"global\",\"hideconstructor\",\"host\",\"ignore\",\"implements\",\"inheritdoc\",\"inner\",\"instance\",\"interface\",\"kind\",\"lends\",\"license\",\"listens\",\"member\",\"memberof\",\"method\",\"mixes\",\"module\",\"name\",\"namespace\",\"override\",\"package\",\"param\",\"private\",\"property\",\"protected\",\"public\",\"readonly\",\"requires\",\"returns\",\"see\",\"since\",\"static\",\"summary\",\"template\",\"this\",\"throws\",\"todo\",\"tutorial\",\"type\",\"typedef\",\"var\",\"variation\",\"version\",\"virtual\",\"yields\"];function i(e){var n=e.comment;switch(e.kind){case 293:return r(e.class);case 301:return a(e.typeParameters.map(function(e){return e.getText()}).join(\", \"));case 300:return r(e.typeExpression);case 302:case 295:case 303:case 297:var t=e.name;return t?r(t):n;default:return n}function r(e){return a(e.getText())}function a(e){return void 0===n?e:e+\" \"+n}}function o(e,n){if(e)for(var t=0;t<e.length;t++)if(e.indexOf(e[t])===t){var r=n(e[t],t);if(r)return r}}function s(n){return{name:n,kind:\"\",kindModifiers:\"\",displayParts:[e.textPart(n)],documentation:e.emptyArray,tags:void 0,codeActions:void 0}}function l(n){switch(n.kind){case 237:case 194:case 154:case 155:case 153:return{commentOwner:n,parameters:n.parameters};case 273:return l(n.initializer);case 238:case 239:case 151:case 241:case 276:case 240:return{commentOwner:n};case 217:var t=n.declarationList.declarations;return{commentOwner:n,parameters:1===t.length&&t[0].initializer?function(n){for(;193===n.kind;)n=n.expression;switch(n.kind){case 194:case 195:return n.parameters;case 207:var t=e.find(n.members,e.isConstructorDeclaration);return t?t.parameters:e.emptyArray}return e.emptyArray}(t[0].initializer):void 0};case 277:return\"quit\";case 242:return 242===n.parent.kind?void 0:{commentOwner:n};case 202:var r=n;return 0===e.getAssignmentDeclarationKind(r)?\"quit\":{commentOwner:n,parameters:e.isFunctionLike(r.right)?r.right.parameters:e.emptyArray}}}n.getJsDocCommentsFromDeclarations=function(n){var t=[];return o(n,function(n){for(var r=0,a=function(n){switch(n.kind){case 297:case 303:return[n];case 295:case 302:return[n,n.parent];default:return e.getJSDocCommentsAndTags(n)}}(n);r<a.length;r++){var i=a[r].comment;void 0!==i&&(t.length&&t.push(e.lineBreakPart()),t.push(e.textPart(i)))}}),t},n.getJsDocTagsFromDeclarations=function(n){var t=[];return o(n,function(n){for(var r=0,a=e.getJSDocTags(n);r<a.length;r++){var o=a[r];t.push({name:o.tagName.text,text:i(o)})}}),t},n.getJSDocTagNameCompletions=function(){return t||(t=e.map(a,function(e){return{name:e,kind:\"keyword\",kindModifiers:\"\",sortText:\"0\"}}))},n.getJSDocTagNameCompletionDetails=s,n.getJSDocTagCompletions=function(){return r||(r=e.map(a,function(e){return{name:\"@\"+e,kind:\"keyword\",kindModifiers:\"\",sortText:\"0\"}}))},n.getJSDocTagCompletionDetails=s,n.getJSDocParameterNameCompletions=function(n){if(!e.isIdentifier(n.name))return e.emptyArray;var t=n.name.text,r=n.parent,a=r.parent;return e.isFunctionLike(a)?e.mapDefined(a.parameters,function(a){if(e.isIdentifier(a.name)){var i=a.name.text;if(!r.tags.some(function(t){return t!==n&&e.isJSDocParameterTag(t)&&e.isIdentifier(t.name)&&t.name.escapedText===i})&&(void 0===t||e.startsWith(i,t)))return{name:i,kind:\"parameter\",kindModifiers:\"\",sortText:\"0\"}}}):[]},n.getJSDocParameterNameCompletionDetails=function(n){return{name:n,kind:\"parameter\",kindModifiers:\"\",displayParts:[e.textPart(n)],documentation:e.emptyArray,tags:void 0,codeActions:void 0}},n.getDocCommentTemplateAtPosition=function(n,t,r){var a=e.getTokenAtPosition(t,r),i=e.findAncestor(a,e.isJSDoc);if(!i||void 0===i.comment&&!e.length(i.tags)){var o=a.getStart(t);if(i||!(o<r)){var s=function(n){return e.forEachAncestor(n,l)}(a);if(s){var c=s.commentOwner,u=s.parameters;if(!(c.getStart(t)<r)){if(!u||0===u.length)return{newText:\"/** */\",caretOffset:3};var d=function(n,t){for(var r=n.text,a=e.getLineStartPositionForPosition(t,n),i=a;i<=t&&e.isWhiteSpaceSingleLine(r.charCodeAt(i));i++);return r.slice(a,i)}(t,r),m=\"/**\"+n+d+\" * \";return{newText:m+n+function(e,n,t,r){return e.map(function(e,a){var i=e.name,o=e.dotDotDotToken,s=71===i.kind?i.text:\"param\"+a,l=n?o?\"{...any} \":\"{any} \":\"\";return t+\" * @param \"+l+s+r}).join(\"\")}(u,e.hasJSFileExtension(t.fileName),d,n)+d+\" */\"+(o===r?n+d:\"\"),caretOffset:m.length}}}}}}}(e.JsDoc||(e.JsDoc={}))}(Tn||(Tn={})),function(e){!function(n){function t(e,n){switch(e.kind){case 248:case 251:case 246:var t=n.getSymbolAtLocation(e.name),r=n.getAliasedSymbol(t);return t.escapedName!==r.escapedName;default:return!0}}function r(n,t){var r=e.getNameOfDeclaration(n);return!!r&&(i(r,t)||147===r.kind&&a(r.expression,t))}function a(n,t){return i(n,t)||e.isPropertyAccessExpression(n)&&(t.push(n.name.text),!0)&&a(n.expression,t)}function i(n,t){return e.isPropertyNameLiteral(n)&&(t.push(e.getTextOfIdentifierOrLiteral(n)),!0)}function o(n){var t=[],i=e.getNameOfDeclaration(n);if(i&&147===i.kind&&!a(i.expression,t))return e.emptyArray;t.shift();for(var o=e.getContainerNode(n);o;){if(!r(o,t))return e.emptyArray;o=e.getContainerNode(o)}return t.reverse()}function s(n,t){return e.compareValues(n.matchKind,t.matchKind)||e.compareStringsCaseSensitiveUI(n.name,t.name)}function l(n){var t=n.declaration,r=e.getContainerNode(t),a=r&&e.getNameOfDeclaration(r);return{name:n.name,kind:e.getNodeKind(t),kindModifiers:e.getNodeModifiers(t),matchKind:e.PatternMatchKind[n.matchKind],isCaseSensitive:n.isCaseSensitive,fileName:n.fileName,textSpan:e.createTextSpanFromNode(t),containerName:a?a.text:\"\",containerKind:a?e.getNodeKind(r):\"\"}}n.getNavigateToItems=function(n,r,a,i,c,u){var d=e.createPatternMatcher(i);if(!d)return e.emptyArray;for(var m=[],p=function(e){if(a.throwIfCancellationRequested(),u&&e.isDeclarationFile)return\"continue\";e.getNamedDeclarations().forEach(function(n,a){!function(e,n,r,a,i,s){var l=e.getMatchForLastSegmentOfPattern(n);if(l)for(var c=0,u=r;c<u.length;c++){var d=u[c];if(t(d,a))if(e.patternContainsDots){var m=e.getFullMatch(o(d),n);m&&s.push({name:n,fileName:i,matchKind:m.kind,isCaseSensitive:m.isCaseSensitive,declaration:d})}else s.push({name:n,fileName:i,matchKind:l.kind,isCaseSensitive:l.isCaseSensitive,declaration:d})}}(d,a,n,r,e.fileName,m)})},f=0,g=n;f<g.length;f++)p(g[f]);return m.sort(s),(void 0===c?m:m.slice(0,c)).map(l)}}(e.NavigateTo||(e.NavigateTo={}))}(Tn||(Tn={})),function(e){!function(n){var t,r,a,i=/\\s+/g,o=[],s=[];function l(){r=void 0,t=void 0,o=[],a=void 0,s=[]}function c(e){return e.getText(r)}function u(e){return e.node.kind}function d(e,n){e.children?e.children.push(n):e.children=[n]}function m(n){e.Debug.assert(!o.length);var t={node:n,name:void 0,additionalNodes:void 0,parent:void 0,children:void 0,indent:0};a=t;for(var r=0,i=n.statements;r<i.length;r++){y(i[r])}return _(),e.Debug.assert(!a&&!o.length),t}function p(e){d(a,f(e))}function f(n){return{node:n,name:e.isDeclaration(n)||e.isExpression(n)?e.getNameOfDeclaration(n):void 0,additionalNodes:void 0,parent:a,children:void 0,indent:a.indent+1}}function g(e){var n=f(e);d(a,n),o.push(a),a=n}function _(){a.children&&(h(a.children,a),T(a.children)),a=o.pop()}function v(e,n){g(e),y(n),_()}function y(n){if(t.throwIfCancellationRequested(),n&&!e.isToken(n))switch(n.kind){case 155:var r=n;v(r,r.body);for(var a=0,i=r.parameters;a<i.length;a++){var o=i[a];e.isParameterPropertyDeclaration(o)&&p(o)}break;case 154:case 156:case 157:case 153:e.hasDynamicName(n)||v(n,n.body);break;case 152:case 151:e.hasDynamicName(n)||p(n);break;case 248:var s=n;s.name&&p(s.name);var l=s.namedBindings;if(l)if(249===l.kind)p(l);else for(var c=0,u=l.elements;c<u.length;c++){p(u[c])}break;case 184:case 235:var d=n,m=d.name,f=d.initializer;e.isBindingPattern(m)?y(m):f&&function(e){switch(e.kind){case 195:case 194:case 207:return!0;default:return!1}}(f)?f.name?y(f):(g(n),e.forEachChild(f,y),_()):v(n,f);break;case 195:case 237:case 194:v(n,n.body);break;case 241:g(n);for(var h=0,b=n.members;h<b.length;h++){M(S=b[h])||p(S)}_();break;case 238:case 207:case 239:g(n);for(var E=0,T=n.members;E<T.length;E++){var S;y(S=T[E])}_();break;case 242:v(n,function n(t){return t.body&&e.isModuleDeclaration(t.body)?n(t.body):t}(n).body);break;case 255:case 246:case 160:case 158:case 159:case 240:p(n);break;case 202:var L=e.getAssignmentDeclarationKind(n);switch(L){case 1:case 2:case 3:case 6:return void v(n,n.right);case 4:case 5:case 0:break;default:e.Debug.assertNever(L)}default:e.hasJSDocNodes(n)&&e.forEach(n.jsDoc,function(n){e.forEach(n.tags,function(n){e.isJSDocTypeAlias(n)&&p(n)})}),e.forEachChild(n,y)}}function h(n,t){var r=e.createMap();e.filterMutate(n,function(n){var a=e.getNameOfDeclaration(n.node),i=a&&c(a);if(!i)return!0;var o=r.get(i);if(!o)return r.set(i,n),!0;if(o instanceof Array){for(var s=0,l=o;s<l.length;s++){var u;if(b(u=l[s],n,t))return!1}return o.push(n),!0}return!b(u=o,n,t)&&(r.set(i,[u,n]),!0)})}function b(n,t,r){return!!function(n,t,r){if(n.kind!==t.kind||n.parent!==t.parent&&(!E(n,r)||!E(t,r)))return!1;switch(n.kind){case 152:case 154:case 156:case 157:return e.hasModifier(n,32)===e.hasModifier(t,32);case 242:return function e(n,t){return n.body.kind===t.body.kind&&(242!==n.body.kind||e(n.body,t.body))}(n,t);default:return!0}}(n.node,t.node,r)&&(function(n,t){var r;n.additionalNodes=n.additionalNodes||[],n.additionalNodes.push(t.node),t.additionalNodes&&(r=n.additionalNodes).push.apply(r,t.additionalNodes);n.children=e.concatenate(n.children,t.children),n.children&&(h(n.children,n),T(n.children))}(n,t),!0)}function E(n,t){var r=e.isModuleBlock(n.parent)?n.parent.parent:n.parent;return r===t.node||e.contains(t.additionalNodes,r)}function T(e){e.sort(S)}function S(n,t){return e.compareStringsCaseSensitiveUI(L(n.node),L(t.node))||e.compareValues(u(n),u(t))}function L(n){if(242===n.kind)return k(n);var t=e.getNameOfDeclaration(n);if(t&&e.isPropertyName(t))return e.unescapeLeadingUnderscores(e.getPropertyNameForPropertyNameNode(t));switch(n.kind){case 194:case 195:case 207:return R(n);default:return}}function x(n,t){if(242===n.kind)return k(n);if(t){var r=c(t);if(r.length>0)return r}switch(n.kind){case 277:var a=n;return e.isExternalModule(a)?'\"'+e.escapeString(e.getBaseFileName(e.removeFileExtension(e.normalizePath(a.fileName))))+'\"':\"<global>\";case 195:case 237:case 194:case 238:case 207:return 512&e.getModifierFlags(n)?\"default\":R(n);case 155:return\"constructor\";case 159:return\"new()\";case 158:return\"()\";case 160:return\"[]\";default:return\"<unknown>\"}}function A(n){return{text:x(n.node,n.name),kind:e.getNodeKind(n.node),kindModifiers:I(n.node),spans:D(n),nameSpan:n.name&&O(n.name),childItems:e.map(n.children,A)}}function C(n){return{text:x(n.node,n.name),kind:e.getNodeKind(n.node),kindModifiers:I(n.node),spans:D(n),childItems:e.map(n.children,function(n){return{text:x(n.node,n.name),kind:e.getNodeKind(n.node),kindModifiers:e.getNodeModifiers(n.node),spans:D(n),childItems:s,indent:0,bolded:!1,grayed:!1}})||s,indent:n.indent,bolded:!1,grayed:!1}}function D(e){var n=[O(e.node)];if(e.additionalNodes)for(var t=0,r=e.additionalNodes;t<r.length;t++){var a=r[t];n.push(O(a))}return n}function k(n){if(e.isAmbientModule(n))return e.getTextOfNode(n.name);var t=[];for(t.push(e.getTextOfIdentifierOrLiteral(n.name));n.body&&242===n.body.kind;)n=n.body,t.push(e.getTextOfIdentifierOrLiteral(n.name));return t.join(\".\")}function M(e){return!e.name||147===e.name.kind}function O(n){return 277===n.kind?e.createTextSpanFromRange(n):e.createTextSpanFromNode(n,r)}function I(n){return n.parent&&235===n.parent.kind&&(n=n.parent),e.getNodeModifiers(n)}function R(n){return n.name&&e.getFullWidth(n.name)>0?e.declarationNameToString(n.name):235===n.parent.kind?e.declarationNameToString(n.parent.name):202===n.parent.kind&&58===n.parent.operatorToken.kind?c(n.parent.left).replace(i,\"\"):273===n.parent.kind&&n.parent.name?c(n.parent.name):512&e.getModifierFlags(n)?\"default\":e.isClassLike(n)?\"<class>\":\"<function>\"}n.getNavigationBarItems=function(n,a){t=a,r=n;try{return e.map((i=m(n),o=[],function n(t){if(function(n){switch(u(n)){case 238:case 207:case 241:case 239:case 242:case 277:case 240:case 302:case 295:return!0;case 155:case 154:case 156:case 157:case 235:return t(n);case 195:case 237:case 194:return function(e){if(!e.node.body)return!1;switch(u(e.parent)){case 243:case 277:case 154:case 155:return!0;default:return t(e)}}(n);default:return!1}function t(n){return e.some(n.children,function(e){var n=u(e);return 235!==n&&184!==n})}}(t)&&(o.push(t),t.children))for(var r=0,a=t.children;r<a.length;r++){var i=a[r];n(i)}}(i),o),C)}finally{l()}var i,o},n.getNavigationTree=function(e,n){t=n,r=e;try{return A(m(e))}finally{l()}}}(e.NavigationBar||(e.NavigationBar={}))}(Tn||(Tn={})),function(e){!function(n){function t(n){var r=n.body;return r&&!e.isIdentifier(r)?e.isModuleBlock(r)?r:t(r):void 0}function r(n){return void 0!==n&&e.isStringLiteralLike(n)?n.text:void 0}function a(n){if(0===n.length)return n;var t=function(n){for(var t,r=[],a=[],i=[],o=0,s=n;o<s.length;o++){var l=s[o];if(void 0!==l.importClause){var c=l.importClause,u=c.name,d=c.namedBindings;u&&r.push(l),d&&(e.isNamespaceImport(d)?a.push(l):i.push(l))}else t=t||l}return{importWithoutClause:t,defaultImports:r,namespaceImports:a,namedImports:i}}(n),r=t.importWithoutClause,a=t.defaultImports,i=t.namespaceImports,l=t.namedImports,u=[];if(r&&u.push(r),1===a.length&&1===i.length&&0===l.length){var d=a[0];return u.push(o(d,d.importClause.name,i[0].importClause.namedBindings)),u}for(var m,p=0,f=e.stableSort(i,function(e,n){return c(e.importClause.namedBindings.name,n.importClause.namedBindings.name)});p<f.length;p++){var g=f[p];u.push(o(g,void 0,g.importClause.namedBindings))}if(0===a.length&&0===l.length)return u;var _=[];if(1===a.length)m=a[0].importClause.name;else for(var v=0,y=a;v<y.length;v++){d=y[v];_.push(e.createImportSpecifier(e.createIdentifier(\"default\"),d.importClause.name))}_.push.apply(_,e.flatMap(l,function(e){return e.importClause.namedBindings.elements}));var h=s(_),b=a.length>0?a[0]:l[0],E=0===h.length?m?void 0:e.createNamedImports(e.emptyArray):0===l.length?e.createNamedImports(h):e.updateNamedImports(l[0].importClause.namedBindings,h);return u.push(o(b,m,E)),u}function i(n){if(0===n.length)return n;var t=function(e){for(var n,t=[],r=0,a=e;r<a.length;r++){var i=a[r];void 0===i.exportClause?n=n||i:t.push(i)}return{exportWithoutClause:n,namedExports:t}}(n),r=t.exportWithoutClause,a=t.namedExports,i=[];if(r&&i.push(r),0===a.length)return i;var o=[];o.push.apply(o,e.flatMap(a,function(e){return e.exportClause.elements}));var l=s(o),c=a[0];return i.push(e.updateExportDeclaration(c,c.decorators,c.modifiers,e.updateNamedExports(c.exportClause,l),c.moduleSpecifier)),i}function o(n,t,r){return e.updateImportDeclaration(n,n.decorators,n.modifiers,e.updateImportClause(n.importClause,t,r),n.moduleSpecifier)}function s(n){return e.stableSort(n,function(e,n){return c(e.propertyName||e.name,n.propertyName||n.name)||c(e.name,n.name)})}function l(n,t){var a=r(n),i=r(t);return e.compareBooleans(void 0===a,void 0===i)||e.compareBooleans(e.isExternalModuleNameRelative(a),e.isExternalModuleNameRelative(i))||e.compareStringsCaseInsensitive(a,i)}function c(n,t){return e.compareStringsCaseInsensitive(n.text,t.text)}n.organizeImports=function(n,s,c,u,d){var m=e.textChanges.ChangeTracker.fromContext({host:c,formatContext:s}),p=function(t){return a(function(n,t,r){for(var a=r.getTypeChecker(),i=a.getJsxNamespace(),s=!!(4&t.transformFlags),l=[],c=0,u=n;c<u.length;c++){var d=u[c],m=d.importClause;if(m){var p=m.name,f=m.namedBindings;if(p&&!_(p)&&(p=void 0),f)if(e.isNamespaceImport(f))_(f.name)||(f=void 0);else{var g=f.elements.filter(function(e){return _(e.name)});g.length<f.elements.length&&(f=g.length?e.updateNamedImports(f,g):void 0)}(p||f)&&l.push(o(d,p,f))}else l.push(d)}return l;function _(n){return s&&n.text===i||e.FindAllReferences.Core.isSymbolReferencedInFile(n,a,t)}}(t,n,u))};v(n.statements.filter(e.isImportDeclaration),p),v(n.statements.filter(e.isExportDeclaration),i);for(var f=0,g=n.statements.filter(e.isAmbientModule);f<g.length;f++){var _=t(g[f]);v(_.statements.filter(e.isImportDeclaration),p),v(_.statements.filter(e.isExportDeclaration),i)}return m.getChanges();function v(t,a){if(0!==e.length(t)){e.suppressLeadingTrivia(t[0]);var i=e.group(t,function(e){return r(e.moduleSpecifier)}),o=e.stableSort(i,function(e,n){return l(e[0].moduleSpecifier,n[0].moduleSpecifier)}),u=e.flatMap(o,function(e){return r(e[0].moduleSpecifier)?a(e):e});0===u.length?m.delete(n,t[0]):m.replaceNodeWithNodes(n,t[0],u,{useNonAdjustedStartPosition:!0,useNonAdjustedEndPosition:!1,suffix:e.getNewLineOrDefaultFromHost(c,s.options)});for(var d=1;d<t.length;d++)m.delete(n,t[d])}}},n.coalesceImports=a,n.coalesceExports=i,n.compareModuleSpecifiers=l}(e.OrganizeImports||(e.OrganizeImports={}))}(Tn||(Tn={})),function(e){!function(n){n.collectElements=function(n,t){var s=[];return function(n,t,r){for(var s=40,l=0,c=n.statements,u=c.length;l<u;){for(;l<u&&!e.isAnyImportSyntax(c[l]);)p(c[l]),l++;if(l===u)break;for(var d=l;l<u&&e.isAnyImportSyntax(c[l]);)a(c[l],n,t,r),l++;var m=l-1;m!==d&&r.push(i(e.findChildOfKind(c[d],91,n).getStart(n),c[m].getEnd(),\"imports\"))}function p(l){if(0!==s){t.throwIfCancellationRequested(),e.isDeclaration(l)&&a(l,n,t,r);var c=function(n,t){switch(n.kind){case 216:if(e.isFunctionBlock(n))return u(n.parent,195!==n.parent.kind);switch(n.parent.kind){case 221:case 224:case 225:case 223:case 220:case 222:case 229:case 272:return u(n.parent);case 233:var r=n.parent;if(r.tryBlock===n)return u(n.parent);if(r.finallyBlock===n)return u(e.findChildOfKind(r,87,t));default:return o(e.createTextSpanFromNode(n,t),\"code\")}case 243:return u(n.parent);case 238:case 239:case 241:case 244:return u(n);case 186:return c(n);case 185:return c(n,21);case 258:return a=n,s=e.createTextSpanFromBounds(a.openingElement.getStart(t),a.closingElement.getEnd()),l=a.openingElement.tagName.getText(t),o(s,\"code\",s,!1,\"<\"+l+\">...</\"+l+\">\");case 259:case 260:return function(e){if(0!==e.properties.length)return i(e.getStart(t),e.getEnd(),\"code\")}(n.attributes)}var a,s,l;function c(n,t){return void 0===t&&(t=17),u(n,!1,!e.isArrayLiteralExpression(n.parent),t)}function u(r,a,i,s){void 0===a&&(a=!1),void 0===i&&(i=!0),void 0===s&&(s=17);var l=e.findChildOfKind(n,s,t),c=17===s?18:22,u=e.findChildOfKind(n,c,t);if(l&&u){var d=e.createTextSpanFromBounds(i?l.getFullStart():l.getStart(t),u.getEnd());return o(d,\"code\",e.createTextSpanFromNode(r,t),a)}}}(l,n);c&&r.push(c),s--,e.isIfStatement(l)&&l.elseStatement&&e.isIfStatement(l.elseStatement)?(p(l.expression),p(l.thenStatement),s++,p(l.elseStatement),s--):l.forEachChild(p),s++}}}(n,t,s),function(n,t){for(var a=[],i=n.getLineStarts(),s=0;s<i.length;s++){var l=i[s],c=s+1===i.length?n.getEnd():i[s+1]-1,u=n.text.substring(l,c),d=r(u);if(d&&!e.isInComment(n,l))if(d[1]){var m=a.pop();m&&(m.textSpan.length=c-m.textSpan.start,m.hintSpan.length=c-m.textSpan.start,t.push(m))}else{var p=e.createTextSpanFromBounds(n.text.indexOf(\"//\",l),c);a.push(o(p,\"region\",p,!1,d[2]||\"#region\"))}}}(n,s),s.sort(function(e,n){return e.textSpan.start-n.textSpan.start})};var t=/^\\s*\\/\\/\\s*#(end)?region(?:\\s+(.*))?(?:\\r)?$/;function r(e){return t.exec(e)}function a(n,t,a,o){var s=e.getLeadingCommentRangesOfNode(n,t);if(s){for(var l=-1,c=-1,u=0,d=t.getFullText(),m=0,p=s;m<p.length;m++){var f=p[m],g=f.kind,_=f.pos,v=f.end;switch(a.throwIfCancellationRequested(),g){case 2:if(r(d.slice(_,v))){y(),u=0;break}0===u&&(l=_),c=v,u++;break;case 3:y(),o.push(i(_,v,\"comment\")),u=0;break;default:e.Debug.assertNever(g)}}y()}function y(){u>1&&o.push(i(l,c,\"comment\"))}}function i(n,t,r){return o(e.createTextSpanFromBounds(n,t),r)}function o(e,n,t,r,a){return void 0===t&&(t=e),void 0===r&&(r=!1),void 0===a&&(a=\"...\"),{textSpan:e,kind:n,hintSpan:t,bannerText:a,autoCollapse:r}}}(e.OutliningElementsCollector||(e.OutliningElementsCollector={}))}(Tn||(Tn={})),function(e){var n;function t(e,n){return{kind:e,isCaseSensitive:n}}function r(e,n){var t=n.get(e);return t||n.set(e,t=v(e)),t}function a(a,i,o){var s=function(e,n){for(var t=e.length-n.length,r=function(t){if(L(n,function(n,r){return m(e.charCodeAt(r+t))===n}))return{value:t}},a=0;a<=t;a++){var i=r(a);if(\"object\"==typeof i)return i.value}return-1}(a,i.textLowerCase);if(0===s)return t(i.text.length===a.length?n.exact:n.prefix,e.startsWith(a,i.text));if(i.isLowerCase){if(-1===s)return;for(var d=0,p=r(a,o);d<p.length;d++){var f=p[d];if(l(a,f,i.text,!0))return t(n.substring,l(a,f,i.text,!1))}if(i.text.length<a.length&&u(a.charCodeAt(s)))return t(n.substring,!1)}else{if(a.indexOf(i.text)>0)return t(n.substring,!0);if(i.characterSpans.length>0){var g=r(a,o),_=!!c(a,g,i,!1)||!c(a,g,i,!0)&&void 0;if(void 0!==_)return t(n.camelCase,_)}}}function i(e,n,t){if(L(n.totalTextChunk.text,function(e){return 32!==e&&42!==e})){var r=a(e,n.totalTextChunk,t);if(r)return r}for(var i,s=0,l=n.subWordTextChunks;s<l.length;s++){i=o(i,a(e,l[s],t))}return i}function o(n,t){return e.min(n,t,s)}function s(n,t){return void 0===n?1:void 0===t?-1:e.compareValues(n.kind,t.kind)||e.compareBooleans(!n.isCaseSensitive,!t.isCaseSensitive)}function l(e,n,t,r,a){return void 0===a&&(a={start:0,length:t.length}),a.length<=n.length&&S(0,a.length,function(i){return function(e,n,t){return t?m(e)===m(n):e===n}(t.charCodeAt(a.start+i),e.charCodeAt(n.start+i),r)})}function c(n,t,r,a){for(var i=r.characterSpans,o=0,s=0;;){if(s===i.length)return!0;if(o===t.length)return!1;for(var c=t[o],d=!1;s<i.length;s++){var m=i[s];if(d&&(!u(r.text.charCodeAt(i[s-1].start))||!u(r.text.charCodeAt(i[s].start))))break;if(!l(n,c,r.text,a,m))break;d=!0,c=e.createTextSpan(c.start+m.length,c.length-m.length)}o++}}function u(n){if(n>=65&&n<=90)return!0;if(n<127||!e.isUnicodeIdentifierStart(n,6))return!1;var t=String.fromCharCode(n);return t===t.toUpperCase()}function d(n){if(n>=97&&n<=122)return!0;if(n<127||!e.isUnicodeIdentifierStart(n,6))return!1;var t=String.fromCharCode(n);return t===t.toLowerCase()}function m(e){return e>=65&&e<=90?e-65+97:e<127?e:String.fromCharCode(e).toLowerCase().charCodeAt(0)}function p(e){return e>=48&&e<=57}function f(e){return u(e)||d(e)||p(e)||95===e||36===e}function g(e){var n=e.toLowerCase();return{text:e,textLowerCase:n,isLowerCase:e===n,characterSpans:_(e)}}function _(e){return y(e,!1)}function v(e){return y(e,!0)}function y(n,t){for(var r=[],a=0,i=1;i<n.length;i++){var o=p(n.charCodeAt(i-1)),s=p(n.charCodeAt(i)),l=T(n,t,i),c=t&&E(n,i,a);(h(n.charCodeAt(i-1))||h(n.charCodeAt(i))||o!==s||l||c)&&(b(n,a,i)||r.push(e.createTextSpan(a,i-a)),a=i)}return b(n,a,n.length)||r.push(e.createTextSpan(a,n.length-a)),r}function h(e){switch(e){case 33:case 34:case 35:case 37:case 38:case 39:case 40:case 41:case 42:case 44:case 45:case 46:case 47:case 58:case 59:case 63:case 64:case 91:case 92:case 93:case 95:case 123:case 125:return!0}return!1}function b(e,n,t){return L(e,function(e){return h(e)&&95!==e},n,t)}function E(e,n,t){return n!==t&&n+1<e.length&&u(e.charCodeAt(n))&&d(e.charCodeAt(n+1))&&L(e,u,t,n)}function T(e,n,t){var r=u(e.charCodeAt(t-1));return u(e.charCodeAt(t))&&(!n||!r)}function S(e,n,t){for(var r=e;r<n;r++)if(!t(r))return!1;return!0}function L(e,n,t,r){return void 0===t&&(t=0),void 0===r&&(r=e.length),S(t,r,function(t){return n(e.charCodeAt(t),t)})}!function(e){e[e.exact=0]=\"exact\",e[e.prefix=1]=\"prefix\",e[e.substring=2]=\"substring\",e[e.camelCase=3]=\"camelCase\"}(n=e.PatternMatchKind||(e.PatternMatchKind={})),e.createPatternMatcher=function(n){var t=e.createMap(),r=n.trim().split(\".\").map(function(e){return{totalTextChunk:g(n=e.trim()),subWordTextChunks:function(e){for(var n=[],t=0,r=0,a=0;a<e.length;a++){var i=e.charCodeAt(a);f(i)?(0===r&&(t=a),r++):r>0&&(n.push(g(e.substr(t,r))),r=0)}return r>0&&n.push(g(e.substr(t,r))),n}(n)};var n});if(!r.some(function(e){return!e.subWordTextChunks.length}))return{getFullMatch:function(n,a){return function(n,t,r,a){var s;if(i(t,e.last(r),a)&&!(r.length-1>n.length)){for(var l=r.length-2,c=n.length-1;l>=0;l-=1,c-=1)s=o(s,i(n[c],r[l],a));return s}}(n,a,r,t)},getMatchForLastSegmentOfPattern:function(n){return i(n,e.last(r),t)},patternContainsDots:r.length>1}},e.breakIntoCharacterSpans=_,e.breakIntoWordSpans=v}(Tn||(Tn={})),function(e){e.preProcessFile=function(n,t,r){void 0===t&&(t=!0),void 0===r&&(r=!1);var a,i,o,s={languageVersion:1,pragmas:void 0,checkJsDirective:void 0,referencedFiles:[],typeReferenceDirectives:[],libReferenceDirectives:[],amdDependencies:[],hasNoDefaultLib:void 0,moduleName:void 0},l=[],c=0,u=!1;function d(){return i=o,17===(o=e.scanner.scan())?c++:18===o&&c--,o}function m(){var n=e.scanner.getTokenValue(),t=e.scanner.getTokenPos();return{fileName:n,pos:t,end:t+n.length}}function p(){l.push(m()),f()}function f(){0===c&&(u=!0)}function g(){var n=e.scanner.getToken();return 124===n&&(129===(n=d())&&9===(n=d())&&(a||(a=[]),a.push({ref:m(),depth:c})),!0)}function _(){if(23===i)return!1;var n=e.scanner.getToken();if(91===n){if(19===(n=d())){if(9===(n=d()))return p(),!0}else{if(9===n)return p(),!0;if(71===n||e.isKeyword(n))if(143===(n=d())){if(9===(n=d()))return p(),!0}else if(58===n){if(y(!0))return!0}else{if(26!==n)return!0;n=d()}if(17===n){for(n=d();18!==n&&1!==n;)n=d();18===n&&143===(n=d())&&9===(n=d())&&p()}else 39===n&&118===(n=d())&&(71===(n=d())||e.isKeyword(n))&&143===(n=d())&&9===(n=d())&&p()}return!0}return!1}function v(){var n=e.scanner.getToken();if(84===n){if(f(),17===(n=d())){for(n=d();18!==n&&1!==n;)n=d();18===n&&143===(n=d())&&9===(n=d())&&p()}else if(39===n)143===(n=d())&&9===(n=d())&&p();else if(91===n&&(71===(n=d())||e.isKeyword(n))&&58===(n=d())&&y(!0))return!0;return!0}return!1}function y(n){var t=n?d():e.scanner.getToken();return 133===t&&(19===(t=d())&&9===(t=d())&&p(),!0)}function h(){var n=e.scanner.getToken();if(71===n&&\"define\"===e.scanner.getTokenValue()){if(19!==(n=d()))return!0;if(9===(n=d())){if(26!==(n=d()))return!0;n=d()}if(21!==n)return!0;for(n=d();22!==n&&1!==n;)9===n&&p(),n=d();return!0}return!1}if(t&&function(){for(e.scanner.setText(n),d();1!==e.scanner.getToken();)g()||_()||v()||r&&(y(!1)||h())||d();e.scanner.setText(void 0)}(),e.processCommentPragmas(s,n),e.processPragmasIntoFields(s,e.noop),u){if(a)for(var b=0,E=a;b<E.length;b++){var T=E[b];l.push(T.ref)}return{referencedFiles:s.referencedFiles,typeReferenceDirectives:s.typeReferenceDirectives,libReferenceDirectives:s.libReferenceDirectives,importedFiles:l,isLibFile:!!s.hasNoDefaultLib,ambientExternalModules:void 0}}var S=void 0;if(a)for(var L=0,x=a;L<x.length;L++)0===(T=x[L]).depth?(S||(S=[]),S.push(T.ref.fileName)):l.push(T.ref);return{referencedFiles:s.referencedFiles,typeReferenceDirectives:s.typeReferenceDirectives,libReferenceDirectives:s.libReferenceDirectives,importedFiles:l,isLibFile:!!s.hasNoDefaultLib,ambientExternalModules:S}}}(Tn||(Tn={})),function(e){!function(n){function t(n){return{canRename:!1,localizedErrorMessage:e.getLocaleSpecificMessage(n)}}n.getRenameInfo=function(n,r,a){var i=e.getTouchingPropertyName(r,a);return(i&&function(n){switch(n.kind){case 71:case 9:case 99:return!0;case 8:return e.isLiteralNameOfPropertyDeclarationOrIndexAccess(n);default:return!1}}(i)?function(n,r,a,i){var o=r.getSymbolAtLocation(n);if(o){var s=o.declarations;if(s&&0!==s.length){if(s.some(i))return t(e.Diagnostics.You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library);if(!(e.isIdentifier(n)&&79===n.originalKeywordKind&&1536&o.parent.flags)){if(e.isStringLiteralLike(n)&&e.tryGetImportFromModuleSpecifier(n))return function(n,r,a){if(!e.isExternalModuleNameRelative(n.text))return t(e.Diagnostics.You_cannot_rename_a_module_via_a_global_import);var i=e.find(a.declarations,e.isSourceFile);if(i){var o=n.text.endsWith(\"/index\")||n.text.endsWith(\"/index.js\")?void 0:e.tryRemoveSuffix(e.removeFileExtension(i.fileName),\"/index\"),s=void 0===o?i.fileName:o,l=void 0===o?\"module\":\"directory\",c=n.text.lastIndexOf(\"/\")+1,u=e.createTextSpan(n.getStart(r)+1+c,n.text.length-c);return{canRename:!0,fileToRename:s,kind:l,displayName:s,fullDisplayName:s,kindModifiers:\"\",triggerSpan:u}}}(n,a,o);var l=e.SymbolDisplay.getSymbolKind(r,o,n),c=e.isImportOrExportSpecifierName(n)||e.isStringOrNumericLiteralLike(n)&&147===n.parent.kind?e.stripQuotes(e.getTextOfIdentifierOrLiteral(n)):void 0,u=c||r.symbolToString(o),d=c||r.getFullyQualifiedName(o);return function(n,t,r,a,i,o){return{canRename:!0,fileToRename:void 0,kind:r,displayName:n,fullDisplayName:t,kindModifiers:a,triggerSpan:function(n,t){var r=n.getStart(t),a=n.getWidth(t);return 9===n.kind&&(r+=1,a-=2),e.createTextSpan(r,a)}(i,o)}}(u,d,l,e.SymbolDisplay.getSymbolModifiers(o),n,a)}}}}(i,n.getTypeChecker(),r,function(e){return n.isSourceFileDefaultLibrary(e.getSourceFile())}):void 0)||t(e.Diagnostics.You_cannot_rename_this_element)}}(e.Rename||(e.Rename={}))}(Tn||(Tn={})),function(e){!function(n){var t,r;function a(n,t,r){var a=e.Debug.assertDefined(e.findPrecedingToken(n.getFullStart(),t,n.parent,!0));return e.rangeContainsRange(r,a)}function i(n,t){var r=function(n,t){if(27===n.kind||19===n.kind)return{list:function(n,t,r){var a=n.getChildren(r),i=a.indexOf(t);return e.Debug.assert(i>=0&&a.length>i+1),a[i+1]}(n.parent,n,t),argumentIndex:0};var r=e.findContainingList(n);return r&&{list:r,argumentIndex:function(e,n){for(var t=0,r=0,a=e.getChildren();r<a.length;r++){var i=a[r];if(i===n)break;26!==i.kind&&t++}return t}(r,n)}}(n,t);if(r){var a=r.list,i=r.argumentIndex,o=function(n){var t=n.getChildren(),r=e.countWhere(t,function(e){return 26!==e.kind});t.length>0&&26===e.last(t).kind&&r++;return r}(a);return 0!==i&&e.Debug.assertLessThan(i,o),{list:a,argumentIndex:i,argumentCount:o,argumentsSpan:function(n,t){var r=n.getFullStart(),a=e.skipTrivia(t.text,n.getEnd(),!1);return e.createTextSpan(r,a-r)}(a,t)}}}function o(n,t,r){var a=n.parent;if(e.isCallOrNewExpression(a)){var o=a,s=i(n,r);if(!s)return;var c=s.list,u=s.argumentIndex,d=s.argumentCount,m=s.argumentsSpan;return{isTypeParameterList:!!a.typeArguments&&a.typeArguments.pos===c.pos,invocation:{kind:0,node:o},argumentsSpan:m,argumentIndex:u,argumentCount:d}}if(e.isNoSubstitutionTemplateLiteral(n)&&e.isTaggedTemplateExpression(a))return e.isInsideTemplateLiteral(n,t,r)?l(a,0,r):void 0;if(e.isTemplateHead(n)&&191===a.parent.kind){var p=a,f=p.parent;return e.Debug.assert(204===p.kind),l(f,u=e.isInsideTemplateLiteral(n,t,r)?0:1,r)}if(e.isTemplateSpan(a)&&e.isTaggedTemplateExpression(a.parent.parent)){var g=a;f=a.parent.parent;if(e.isTemplateTail(n)&&!e.isInsideTemplateLiteral(n,t,r))return;return l(f,u=function(n,t,r,a){if(e.Debug.assert(r>=t.getStart(),\"Assumed 'position' could not occur before node.\"),e.isTemplateLiteralToken(t))return e.isInsideTemplateLiteral(t,r,a)?0:n+2;return n+1}(g.parent.templateSpans.indexOf(g),n,t,r),r)}if(e.isJsxOpeningLikeElement(a)){var _=a.attributes.pos,v=e.skipTrivia(r.text,a.attributes.end,!1);return{isTypeParameterList:!1,invocation:{kind:0,node:a},argumentsSpan:e.createTextSpan(_,v-_),argumentIndex:0,argumentCount:1}}var y=e.getPossibleTypeArgumentsInfo(n,r);if(y){var h=y.called,b=y.nTypeArguments;return{isTypeParameterList:!0,invocation:o={kind:1,called:h},argumentsSpan:m=e.createTextSpanFromBounds(h.getStart(r),n.end),argumentIndex:b,argumentCount:b+1}}}function s(n){return e.isBinaryExpression(n.left)?s(n.left)+1:2}function l(n,t,r){var a=e.isNoSubstitutionTemplateLiteral(n.template)?1:n.template.templateSpans.length+1;return 0!==t&&e.Debug.assertLessThan(t,a),{isTypeParameterList:!1,invocation:{kind:0,node:n},argumentsSpan:function(n,t){var r=n.template,a=r.getStart(),i=r.getEnd();if(204===r.kind){var o=e.last(r.templateSpans);0===o.literal.getFullWidth()&&(i=e.skipTrivia(t.text,i,!1))}return e.createTextSpan(a,i-a)}(n,r),argumentIndex:t,argumentCount:a}}function c(n){return 0===n.kind?e.getInvokedExpression(n.node):n.called}function u(e){return 0===e.kind?e.node:1===e.kind?e.called:e.node}!function(e){e[e.Call=0]=\"Call\",e[e.TypeArgs=1]=\"TypeArgs\",e[e.Contextual=2]=\"Contextual\"}(t||(t={})),n.getSignatureHelpItems=function(n,t,r,l,d){var g=n.getTypeChecker(),_=e.findTokenOnLeftOfPosition(t,r);if(_){var v=!!l&&\"characterTyped\"===l.kind;if(!v||!e.isInString(t,r,_)&&!e.isInComment(t,r)){var y=function(n,t,r,a){for(var l=function(n){e.Debug.assert(e.rangeContainsRange(n.parent,n),\"Not a subspan\",function(){return\"Child: \"+e.Debug.showSyntaxKind(n)+\", parent: \"+e.Debug.showSyntaxKind(n.parent)});var l=function(n,t,r,a){return function(n,t,r,a){var o=function(n,t,r){if(19===n.kind||26===n.kind){var a=n.parent;switch(a.kind){case 193:case 154:case 194:case 195:var o=i(n,t);if(!o)return;var l=o.argumentIndex,c=o.argumentCount,u=o.argumentsSpan,d=e.isMethodDeclaration(a)?r.getContextualTypeForObjectLiteralElement(a):r.getContextualType(a);return d&&{contextualType:d,argumentIndex:l,argumentCount:c,argumentsSpan:u};case 202:var m=function n(t){return e.isBinaryExpression(t.parent)?n(t.parent):t}(a),p=r.getContextualType(m),f=19===n.kind?0:s(a)-1,g=s(m);return p&&{contextualType:p,argumentIndex:f,argumentCount:g,argumentsSpan:e.createTextSpanFromNode(a)};default:return}}}(n,r,a);if(o){var l,c=o.contextualType,u=o.argumentIndex,d=o.argumentCount,m=o.argumentsSpan,p=c.getCallSignatures();return 1!==p.length?void 0:{isTypeParameterList:!1,invocation:{kind:2,signature:e.first(p),node:n,symbol:(l=c.symbol,\"__type\"===l.name&&e.firstDefined(l.declarations,function(n){return e.isFunctionTypeNode(n)?n.parent.symbol:void 0})||l)},argumentsSpan:m,argumentIndex:u,argumentCount:d}}}(n,0,r,a)||o(n,t,r)}(n,t,r,a);if(l)return{value:l}},c=n;!e.isBlock(c)&&!e.isSourceFile(c);c=c.parent){var u=l(c);if(\"object\"==typeof u)return u.value}}(_,r,t,g);if(y){d.throwIfCancellationRequested();var h=function(n,t,r,i,o){var s=n.invocation,l=n.argumentCount;switch(s.kind){case 0:if(o&&!function(n,t,r){if(!e.isCallOrNewExpression(t))return!1;var i=t.getChildren(r);switch(n.kind){case 19:return e.contains(i,n);case 26:var o=e.findContainingList(n);return!!o&&e.contains(i,o);case 27:return a(n,r,t.expression);default:return!1}}(i,s.node,r))return;var c=[],u=t.getResolvedSignatureForSignatureHelp(s.node,c,l);return 0===c.length?void 0:{kind:0,candidates:c,resolvedSignature:u};case 1:var d=s.called;if(o&&!a(i,r,e.isIdentifier(d)?d.parent:d))return;var c=e.getPossibleGenericSignatures(d,l,t);if(0!==c.length)return{kind:0,candidates:c,resolvedSignature:e.first(c)};var m=t.getSymbolAtLocation(d);return m&&{kind:1,symbol:m};case 2:return{kind:0,candidates:[s.signature],resolvedSignature:s.signature};default:return e.Debug.assertNever(s)}}(y,g,t,_,v);return d.throwIfCancellationRequested(),h?g.runWithCancellationToken(d,function(n){return 0===h.kind?m(h.candidates,h.resolvedSignature,y,t,n):function(n,t,r,a){var i=t.argumentCount,o=t.argumentsSpan,s=t.invocation,l=t.argumentIndex,c=a.getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(n);return c?{items:[function(n,t,r,a,i){var o=e.symbolToDisplayParts(r,n),s=e.createPrinter({removeComments:!0}),l=t.map(function(e){return f(e,r,a,i,s)}),c=n.getDocumentationComment(r),u=n.getJsDocTags();return{isVariadic:!1,prefixDisplayParts:o.concat([e.punctuationPart(27)]),suffixDisplayParts:[e.punctuationPart(29)],separatorDisplayParts:p,parameters:l,documentation:c,tags:u}}(n,c,a,u(s),r)],applicableSpan:o,selectedItemIndex:0,argumentIndex:l,argumentCount:i}:void 0}(h.symbol,y,t,n)}):e.isSourceFileJS(t)?function(n,t,r){if(2!==n.invocation.kind){var a=c(n.invocation),i=e.isIdentifier(a)?a.text:e.isPropertyAccessExpression(a)?a.name.text:void 0,o=t.getTypeChecker();return void 0===i?void 0:e.firstDefined(t.getSourceFiles(),function(t){return e.firstDefined(t.getNamedDeclarations().get(i),function(e){var a=e.symbol&&o.getTypeOfSymbolAtLocation(e.symbol,e),i=a&&a.getCallSignatures();if(i&&i.length)return o.runWithCancellationToken(r,function(e){return m(i,i[0],n,t,e)})})})}}(y,n,d):void 0}}}},function(e){e[e.Candidate=0]=\"Candidate\",e[e.Type=1]=\"Type\"}(r||(r={})),n.getArgumentInfoForCompletions=function(e,n,t){var r=o(e,n,t);return!r||r.isTypeParameterList||0!==r.invocation.kind?void 0:{invocation:r.invocation.node,argumentCount:r.argumentCount,argumentIndex:r.argumentIndex}};var d=3137536;function m(n,t,r,a,i){var o=r.isTypeParameterList,s=r.argumentCount,l=r.argumentsSpan,m=r.invocation,g=r.argumentIndex,_=u(m),v=2===m.kind?m.symbol:i.getSymbolAtLocation(c(m)),y=v?e.symbolToDisplayParts(i,v,void 0,void 0):e.emptyArray,h=n.map(function(n){return function(n,t,r,a,i,o){var s=(r?function(n,t,r,a){var i=(n.target||n).typeParameters,o=e.createPrinter({removeComments:!0}),s=(i||e.emptyArray).map(function(e){return f(e,t,r,a,o)}),l=e.mapToDisplayParts(function(i){var s=n.thisParameter?[t.symbolToParameterDeclaration(n.thisParameter,r,d)]:[],l=e.createNodeArray(s.concat(n.parameters.map(function(e){return t.symbolToParameterDeclaration(e,r,d)})));o.writeList(1296,l,a,i)});return{isVariadic:!1,parameters:s,prefix:[e.punctuationPart(27)],suffix:[e.punctuationPart(29)].concat(l)}}:function(n,t,r,a){var i=n.hasRestParameter,o=e.createPrinter({removeComments:!0}),s=e.mapToDisplayParts(function(i){if(n.typeParameters&&n.typeParameters.length){var s=e.createNodeArray(n.typeParameters.map(function(e){return t.typeParameterToDeclaration(e,r)}));o.writeList(26896,s,a,i)}}),l=n.parameters.map(function(n){return function(n,t,r,a,i){var o=e.mapToDisplayParts(function(e){var o=t.symbolToParameterDeclaration(n,r,d);i.writeNode(4,o,a,e)}),s=t.isOptionalParameter(n.valueDeclaration);return{name:n.name,documentation:n.getDocumentationComment(t),displayParts:o,isOptional:s}}(n,t,r,a,o)});return{isVariadic:i,parameters:l,prefix:s.concat([e.punctuationPart(19)]),suffix:[e.punctuationPart(20)]}})(n,a,i,o),l=s.isVariadic,c=s.parameters,u=s.prefix,m=s.suffix,g=t.concat(u),_=m.concat(function(n,t,r){return e.mapToDisplayParts(function(e){e.writePunctuation(\":\"),e.writeSpace(\" \");var a=r.getTypePredicateOfSignature(n);a?r.writeTypePredicate(a,t,void 0,e):r.writeType(r.getReturnTypeOfSignature(n),t,void 0,e)})}(n,i,a)),v=n.getDocumentationComment(a),y=n.getJsDocTags();return{isVariadic:l,prefixDisplayParts:g,suffixDisplayParts:_,separatorDisplayParts:p,parameters:c,documentation:v,tags:y}}(n,y,o,i,_,a)});0!==g&&e.Debug.assertLessThan(g,s);var b=n.indexOf(t);return e.Debug.assert(-1!==b),{items:h,applicableSpan:l,selectedItemIndex:b,argumentIndex:g,argumentCount:s}}var p=[e.punctuationPart(26),e.spacePart()];function f(n,t,r,a,i){var o=e.mapToDisplayParts(function(e){var o=t.typeParameterToDeclaration(n,r);i.writeNode(4,o,a,e)});return{name:n.symbol.name,documentation:n.symbol.getDocumentationComment(t),displayParts:o,isOptional:!1}}}(e.SignatureHelp||(e.SignatureHelp={}))}(Tn||(Tn={})),function(e){var n=/^\\/\\/[@#] source[M]appingURL=(.+)\\s*$/,t=/^\\s*(\\/\\/[@#] .*)?$/,r=/^data:(?:application\\/json(?:;charset=[uU][tT][fF]-8);base64,([A-Za-z0-9+\\/=]+)$)?/;e.getSourceMapper=function(a,i,o,s,l){var c;return{tryGetOriginalLocation:function n(t){if(e.isDeclarationFileName(t.fileName)){var r=m(t.fileName);if(r){var a=d(t.fileName,r).getOriginalPosition(t);return a===t?void 0:n(a)||a}}},tryGetGeneratedLocation:function(n){var t=l(),r=e.getDeclarationEmitOutputFilePathWorker(n.fileName,t.getCompilerOptions(),i,t.getCommonSourceDirectory(),a);if(void 0!==r){var o=m(r);if(o){var s=d(r,o).getGeneratedPosition(n);return s===n?void 0:s}}},toLineColumnOffset:function(n,t){var r=e.toPath(n,i,a);return(l().getSourceFile(r)||c.get(r)).getLineAndCharacterOfPosition(t)},clearCache:function(){c=e.createSourceFileLikeCache(s)}};function u(n,t,r){var i;try{i=JSON.parse(t)}catch(e){}return i&&i.sources&&i.file&&i.mappings?n.sourceMapper=e.sourcemaps.decode({readFile:function(e){return s.readFile(e)},fileExists:function(e){return s.fileExists(e)},getCanonicalFileName:a,log:o},r,i,l(),c):n.sourceMapper=e.sourcemaps.identitySourceMapper}function d(o,l){if(!s.readFile||!s.fileExists)return l.sourceMapper=e.sourcemaps.identitySourceMapper;if(l.sourceMapper)return l.sourceMapper;var d=function(r){var o=c.get(e.toPath(r,i,a));if(o)for(var s=e.getLineStarts(o),l=s.length-1;l>=0;l--){var u=o.text.substring(s[l],s[l+1]),d=n.exec(u);if(d)return d[1];if(!u.match(t))break}}(o);if(d){var m=r.exec(d);if(m){if(m[1]){var p=m[1];return u(l,e.base64decode(e.sys,p),o)}d=void 0}}var f=[];d&&f.push(d),f.push(o+\".map\");for(var g=0,_=f;g<_.length;g++){var v=_[g],y=e.toPath(v,e.getDirectoryPath(o),a);if(s.fileExists(y))return u(l,s.readFile(y),y)}return l.sourceMapper=e.sourcemaps.identitySourceMapper}function m(n){return l().getSourceFile(n)||c.get(e.toPath(n,i,a))}}}(Tn||(Tn={})),function(e){function n(n){switch(n.kind){case 247:var t=n.importClause,r=n.moduleSpecifier;return t&&!t.name&&t.namedBindings&&249===t.namedBindings.kind&&e.isStringLiteral(r)?t.namedBindings.name:void 0;case 246:return n.name;default:return}}function t(n){var t=[];function i(n){e.isFunctionLike(n)||(e.isReturnStatement(n)&&n.expression&&function(n){if(!r(n)||!n.arguments.every(a))return!1;var t=n.expression;for(;r(t)||e.isPropertyAccessExpression(t);){if(e.isCallExpression(t)&&!t.arguments.every(a))return!1;t=t.expression}return!0}(n.expression)&&t.push(n),e.forEachChild(n,i))}return e.isFunctionLike(n)?e.forEachChild(n,i):i(n),t}function r(n){return e.isCallExpression(n)&&(e.hasPropertyAccessExpressionWithName(n,\"then\")||e.hasPropertyAccessExpressionWithName(n,\"catch\"))}function a(e){switch(e.kind){case 95:case 71:case 237:case 194:case 195:return!0;default:return!1}}e.computeSuggestionDiagnostics=function(r,a,i){a.getSemanticDiagnostics(r,i);var o,s=[],l=a.getTypeChecker();r.commonJsModuleIndicator&&(e.programContainsEs6Modules(a)||e.compilerOptionsIndicateEs6Modules(a.getCompilerOptions()))&&function(n){return n.statements.some(function(n){switch(n.kind){case 217:return n.declarationList.declarations.some(function(n){return e.isRequireCall(function n(t){return e.isPropertyAccessExpression(t)?n(t.expression):t}(n.initializer),!0)});case 219:var t=n.expression;if(!e.isBinaryExpression(t))return e.isRequireCall(t,!0);var r=e.getAssignmentDeclarationKind(t);return 1===r||2===r;default:return!1}})}(r)&&s.push(e.createDiagnosticForNode((o=r.commonJsModuleIndicator,e.isBinaryExpression(o)?o.left:o),e.Diagnostics.File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module));var c=e.isSourceFileJS(r);if(function n(a){if(c)switch(a.kind){case 194:var i=e.getDeclarationOfExpando(a);if(i){var o=i.symbol;if(o&&(o.exports&&o.exports.size||o.members&&o.members.size)){s.push(e.createDiagnosticForNode(e.isVariableDeclaration(a.parent)?a.parent.name:a,e.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration));break}}case 237:var u=a.symbol;u.members&&u.members.size>0&&s.push(e.createDiagnosticForNode(e.isVariableDeclaration(a.parent)?a.parent.name:a,e.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration))}else{if(e.isVariableStatement(a)&&a.parent===r&&2&a.declarationList.flags&&1===a.declarationList.declarations.length){var d=a.declarationList.declarations[0].initializer;d&&e.isRequireCall(d,!0)&&s.push(e.createDiagnosticForNode(d,e.Diagnostics.require_call_may_be_converted_to_an_import))}e.codefix.parameterShouldGetTypeFromJSDoc(a)&&s.push(e.createDiagnosticForNode(a.name||a,e.Diagnostics.JSDoc_types_may_be_moved_to_TypeScript_types))}e.isFunctionLikeDeclaration(a)&&function(n,r,a){if(!e.isAsyncFunction(n)&&n.body){var i=r.getTypeAtLocation(n),o=r.getSignaturesOfType(i,0),s=o.length?r.getReturnTypeOfSignature(o[0]):void 0;s&&r.getPromisedTypeOfPromise(s)&&t(n).length>0&&a.push(e.createDiagnosticForNode(!n.name&&e.isVariableDeclaration(n.parent)&&e.isIdentifier(n.parent.name)?n.parent.name:n,e.Diagnostics.This_may_be_converted_to_an_async_function))}}(a,l,s),a.forEachChild(n)}(r),e.getAllowSyntheticDefaultImports(a.getCompilerOptions()))for(var u=0,d=r.imports;u<d.length;u++){var m=d[u],p=n(e.importFromModuleSpecifier(m));if(p){var f=e.getResolvedModule(r,m.text),g=f&&a.getSourceFile(f.resolvedFileName);g&&g.externalModuleIndicator&&e.isExportAssignment(g.externalModuleIndicator)&&g.externalModuleIndicator.isExportEquals&&s.push(e.createDiagnosticForNode(p,e.Diagnostics.Import_may_be_converted_to_a_default_import))}}return e.addRange(s,r.bindSuggestionDiagnostics),e.addRange(s,a.getSuggestionDiagnostics(r,i)),s.sort(function(e,n){return e.start-n.start})},e.getReturnStatementsWithPromiseHandlers=t}(Tn||(Tn={})),function(e){!function(n){function t(n,t,a){var i=r(n,t,a);if(\"\"!==i)return i;var o=e.getCombinedLocalAndExportSymbolFlags(t);return 32&o?e.getDeclarationOfKind(t,207)?\"local class\":\"class\":384&o?\"enum\":524288&o?\"type\":64&o?\"interface\":262144&o?\"type parameter\":262144&o?\"type parameter\":8&o?\"enum member\":2097152&o?\"alias\":1536&o?\"module\":i}function r(n,t,r){var i=n.getRootSymbols(t);if(1===i.length&&8192&e.first(i).flags&&0!==n.getTypeOfSymbolAtLocation(t,r).getNonNullableType().getCallSignatures().length)return\"method\";if(n.isUndefinedSymbol(t))return\"var\";if(n.isArgumentsSymbol(t))return\"local var\";if(99===r.kind&&e.isExpression(r))return\"parameter\";var o=e.getCombinedLocalAndExportSymbolFlags(t);if(3&o)return e.isFirstDeclarationOfSymbolParameter(t)?\"parameter\":t.valueDeclaration&&e.isVarConst(t.valueDeclaration)?\"const\":e.forEach(t.declarations,e.isLet)?\"let\":a(t)?\"local var\":\"var\";if(16&o)return a(t)?\"local function\":\"function\";if(32768&o)return\"getter\";if(65536&o)return\"setter\";if(8192&o)return\"method\";if(16384&o)return\"constructor\";if(4&o){if(33554432&o&&6&t.checkFlags){var s=e.forEach(n.getRootSymbols(t),function(n){var t=n.getFlags();if(98311&t)return\"property\";e.Debug.assert(!!(8208&t))});return s||(n.getTypeOfSymbolAtLocation(t,r).getCallSignatures().length?\"method\":\"property\")}switch(r.parent&&r.parent.kind){case 260:case 258:case 259:return 71===r.kind?\"property\":\"JSX attribute\";case 265:return\"JSX attribute\";default:return\"property\"}}return\"\"}function a(n){return!n.parent&&e.forEach(n.declarations,function(n){if(194===n.kind)return!0;if(235!==n.kind&&237!==n.kind)return!1;for(var t=n.parent;!e.isFunctionBlock(t);t=t.parent)if(277===t.kind||243===t.kind)return!1;return!0})}n.getSymbolKind=t,n.getSymbolModifiers=function(n){var t=n&&n.declarations&&n.declarations.length>0?e.getNodeModifiers(n.declarations[0]):\"\",r=n&&16777216&n.flags?\"optional\":\"\";return t&&r?t+\",\"+r:t||r},n.getSymbolDisplayPartsDocumentationAndSymbolKind=function n(a,i,o,s,l,c,u){void 0===c&&(c=e.getMeaningFromLocation(l));var d,m,p,f,g,_,v=[],y=e.getCombinedLocalAndExportSymbolFlags(i),h=1&c?r(a,i,l):\"\",b=!1,E=99===l.kind&&e.isInExpressionContext(l);if(99===l.kind&&!E)return{displayParts:[e.keywordPart(99)],documentation:[],symbolKind:\"primitive type\",tags:void 0};if(\"\"!==h||32&y||2097152&y){\"getter\"!==h&&\"setter\"!==h||(h=\"property\");var T=void 0;if(p=E?a.getTypeAtLocation(l):a.getTypeOfSymbolAtLocation(i.exportSymbol||i,l),l.parent&&187===l.parent.kind){var S=l.parent.name;(S===l||S&&0===S.getFullWidth())&&(l=l.parent)}var L=void 0;if(e.isCallOrNewExpression(l)?L=l:e.isCallExpressionTarget(l)||e.isNewExpressionTarget(l)?L=l.parent:l.parent&&e.isJsxOpeningLikeElement(l.parent)&&e.isFunctionLike(i.valueDeclaration)&&(L=l.parent),L){T=a.getResolvedSignature(L,[]);var x=190===L.kind||e.isCallExpression(L)&&97===L.expression.kind,A=x?p.getConstructSignatures():p.getCallSignatures();if(e.contains(A,T.target)||e.contains(A,T)||(T=A.length?A[0]:void 0),T){switch(x&&32&y?(h=\"constructor\",z(p.symbol,h)):2097152&y?(J(h=\"alias\"),v.push(e.spacePart()),x&&(v.push(e.keywordPart(94)),v.push(e.spacePart())),q(i)):z(i,h),h){case\"JSX attribute\":case\"property\":case\"var\":case\"const\":case\"let\":case\"parameter\":case\"local var\":v.push(e.punctuationPart(56)),v.push(e.spacePart()),16&e.getObjectFlags(p)||!p.symbol||(e.addRange(v,e.symbolToDisplayParts(a,p.symbol,s,void 0,5)),v.push(e.lineBreakPart())),x&&(v.push(e.keywordPart(94)),v.push(e.spacePart())),X(T,A,262144);break;default:X(T,A)}b=!0}}else if(e.isNameOfFunctionDeclaration(l)&&!(98304&y)||123===l.kind&&155===l.parent.kind){var C=l.parent;e.find(i.declarations,function(e){return e===(123===l.kind?C.parent:C)})&&(A=155===C.kind?p.getNonNullableType().getConstructSignatures():p.getNonNullableType().getCallSignatures(),T=a.isImplementationOfOverload(C)?A[0]:a.getSignatureFromDeclaration(C),155===C.kind?(h=\"constructor\",z(p.symbol,h)):z(158!==C.kind||2048&p.symbol.flags||4096&p.symbol.flags?i:p.symbol,h),X(T,A),b=!0)}}if(32&y&&!b&&!E&&(j(),e.getDeclarationOfKind(i,207)?J(\"local class\"):v.push(e.keywordPart(75)),v.push(e.spacePart()),q(i),Y(i,o)),64&y&&2&c&&(U(),v.push(e.keywordPart(109)),v.push(e.spacePart()),q(i),Y(i,o)),524288&y&&2&c&&(U(),v.push(e.keywordPart(139)),v.push(e.spacePart()),q(i),Y(i,o),v.push(e.spacePart()),v.push(e.operatorPart(58)),v.push(e.spacePart()),e.addRange(v,e.typeToDisplayParts(a,a.getDeclaredTypeOfSymbol(i),s,8388608))),384&y&&(U(),e.some(i.declarations,function(n){return e.isEnumDeclaration(n)&&e.isEnumConst(n)})&&(v.push(e.keywordPart(76)),v.push(e.spacePart())),v.push(e.keywordPart(83)),v.push(e.spacePart()),q(i)),1536&y){U();var D=(K=e.getDeclarationOfKind(i,242))&&K.name&&71===K.name.kind;v.push(e.keywordPart(D?130:129)),v.push(e.spacePart()),q(i)}if(262144&y&&2&c)if(U(),v.push(e.punctuationPart(19)),v.push(e.textPart(\"type parameter\")),v.push(e.punctuationPart(20)),v.push(e.spacePart()),q(i),i.parent)W(),q(i.parent,s),Y(i.parent,s);else{var k=e.getDeclarationOfKind(i,148);if(void 0===k)return e.Debug.fail();(K=k.parent)&&(e.isFunctionLikeKind(K.kind)?(W(),T=a.getSignatureFromDeclaration(K),159===K.kind?(v.push(e.keywordPart(94)),v.push(e.spacePart())):158!==K.kind&&K.name&&q(K.symbol),e.addRange(v,e.signatureToDisplayParts(a,T,o,32))):240===K.kind&&(W(),v.push(e.keywordPart(139)),v.push(e.spacePart()),q(K.symbol),Y(K.symbol,o)))}if(8&y&&(h=\"enum member\",z(i,\"enum member\"),276===(K=i.declarations[0]).kind)){var M=a.getConstantValue(K);void 0!==M&&(v.push(e.spacePart()),v.push(e.operatorPart(58)),v.push(e.spacePart()),v.push(e.displayPart(e.getTextOfConstantValue(M),\"number\"==typeof M?e.SymbolDisplayPartKind.numericLiteral:e.SymbolDisplayPartKind.stringLiteral)))}if(2097152&y){if(U(),!b){var O=a.getAliasedSymbol(i);if(O!==i&&O.declarations&&O.declarations.length>0){var I=O.declarations[0],R=e.getNameOfDeclaration(I);if(R){var N=e.isModuleWithStringLiteralName(I)&&e.hasModifier(I,2),w=\"default\"!==i.name&&!N,P=n(a,O,e.getSourceFileOfNode(I),I,R,c,w?i:O);v.push.apply(v,P.displayParts),v.push(e.lineBreakPart()),g=P.documentation,_=P.tags}}}switch(i.declarations[0].kind){case 245:v.push(e.keywordPart(84)),v.push(e.spacePart()),v.push(e.keywordPart(130));break;case 252:v.push(e.keywordPart(84)),v.push(e.spacePart()),v.push(e.keywordPart(i.declarations[0].isExportEquals?58:79));break;case 255:v.push(e.keywordPart(84));break;default:v.push(e.keywordPart(91))}v.push(e.spacePart()),q(i),e.forEach(i.declarations,function(n){if(246===n.kind){var t=n;if(e.isExternalModuleImportEqualsDeclaration(t))v.push(e.spacePart()),v.push(e.operatorPart(58)),v.push(e.spacePart()),v.push(e.keywordPart(133)),v.push(e.punctuationPart(19)),v.push(e.displayPart(e.getTextOfNode(e.getExternalModuleImportEqualsDeclarationExpression(t)),e.SymbolDisplayPartKind.stringLiteral)),v.push(e.punctuationPart(20));else{var r=a.getSymbolAtLocation(t.moduleReference);r&&(v.push(e.spacePart()),v.push(e.operatorPart(58)),v.push(e.spacePart()),q(r,s))}return!0}})}if(!b)if(\"\"!==h){if(p)if(E?(U(),v.push(e.keywordPart(99))):z(i,h),\"property\"===h||\"JSX attribute\"===h||3&y||\"local var\"===h||E)if(v.push(e.punctuationPart(56)),v.push(e.spacePart()),p.symbol&&262144&p.symbol.flags){var F=e.mapToDisplayParts(function(n){var t=a.typeParameterToDeclaration(p,s);H().writeNode(4,t,e.getSourceFileOfNode(e.getParseTreeNode(s)),n)});e.addRange(v,F)}else e.addRange(v,e.typeToDisplayParts(a,p,s));else(16&y||8192&y||16384&y||131072&y||98304&y||\"method\"===h)&&(A=p.getNonNullableType().getCallSignatures()).length&&X(A[0],A)}else h=t(a,i,l);if(!d&&(d=i.getDocumentationComment(a),m=i.getJsDocTags(),0===d.length&&4&y&&i.parent&&e.forEach(i.parent.declarations,function(e){return 277===e.kind})))for(var G=0,V=i.declarations;G<V.length;G++){var K;if((K=V[G]).parent&&202===K.parent.kind){var B=a.getSymbolAtLocation(K.parent.right);if(B&&(d=B.getDocumentationComment(a),m=B.getJsDocTags(),d.length>0))break}}return 0===d.length&&g&&(d=g),0===m.length&&_&&(m=_),{displayParts:v,documentation:d,symbolKind:h,tags:0===m.length?void 0:m};function H(){return f||(f=e.createPrinter({removeComments:!0})),f}function U(){v.length&&v.push(e.lineBreakPart()),j()}function j(){u&&(J(\"alias\"),v.push(e.spacePart()))}function W(){v.push(e.spacePart()),v.push(e.keywordPart(92)),v.push(e.spacePart())}function q(n,t){u&&n===i&&(n=u);var r=e.symbolToDisplayParts(a,n,t||o,void 0,7);e.addRange(v,r),16777216&i.flags&&v.push(e.punctuationPart(55))}function z(n,t){U(),t&&(J(t),n&&!e.some(n.declarations,function(n){return e.isArrowFunction(n)||(e.isFunctionExpression(n)||e.isClassExpression(n))&&!n.name})&&(v.push(e.spacePart()),q(n)))}function J(n){switch(n){case\"var\":case\"function\":case\"let\":case\"const\":case\"constructor\":return void v.push(e.textOrKeywordPart(n));default:return v.push(e.punctuationPart(19)),v.push(e.textOrKeywordPart(n)),void v.push(e.punctuationPart(20))}}function X(n,t,r){void 0===r&&(r=0),e.addRange(v,e.signatureToDisplayParts(a,n,s,32|r)),t.length>1&&(v.push(e.spacePart()),v.push(e.punctuationPart(19)),v.push(e.operatorPart(37)),v.push(e.displayPart((t.length-1).toString(),e.SymbolDisplayPartKind.numericLiteral)),v.push(e.spacePart()),v.push(e.textPart(2===t.length?\"overload\":\"overloads\")),v.push(e.punctuationPart(20)));var i=n.getDocumentationComment(a);d=0===i.length?void 0:i,m=n.getJsDocTags()}function Y(n,t){var r=e.mapToDisplayParts(function(r){var i=a.symbolToTypeParameterDeclarations(n,t);H().writeList(26896,i,e.getSourceFileOfNode(e.getParseTreeNode(t)),r)});e.addRange(v,r)}}}(e.SymbolDisplay||(e.SymbolDisplay={}))}(Tn||(Tn={})),function(e){function n(n,t){var a=[],i=t.compilerOptions?r(t.compilerOptions,a):e.getDefaultCompilerOptions();i.isolatedModules=!0,i.suppressOutputPathCheck=!0,i.allowNonTsExtensions=!0,i.noLib=!0,i.lib=void 0,i.types=void 0,i.noEmit=void 0,i.noEmitOnError=void 0,i.paths=void 0,i.rootDirs=void 0,i.declaration=void 0,i.composite=void 0,i.declarationDir=void 0,i.out=void 0,i.outFile=void 0,i.noResolve=!0;var o=t.fileName||(i.jsx?\"module.tsx\":\"module.ts\"),s=e.createSourceFile(o,n,i.target);t.moduleName&&(s.moduleName=t.moduleName),t.renamedDependencies&&(s.renamedDependencies=e.createMapFromTemplate(t.renamedDependencies));var l,c,u=e.getNewLineCharacter(i),d={getSourceFile:function(n){return n===e.normalizePath(o)?s:void 0},writeFile:function(n,t){e.fileExtensionIs(n,\".map\")?(e.Debug.assertEqual(c,void 0,\"Unexpected multiple source map outputs, file:\",n),c=t):(e.Debug.assertEqual(l,void 0,\"Unexpected multiple outputs, file:\",n),l=t)},getDefaultLibFileName:function(){return\"lib.d.ts\"},useCaseSensitiveFileNames:function(){return!1},getCanonicalFileName:function(e){return e},getCurrentDirectory:function(){return\"\"},getNewLine:function(){return u},fileExists:function(e){return e===o},readFile:function(){return\"\"},directoryExists:function(){return!0},getDirectories:function(){return[]}},m=e.createProgram([o],i,d);return t.reportDiagnostics&&(e.addRange(a,m.getSyntacticDiagnostics(s)),e.addRange(a,m.getOptionsDiagnostics())),m.emit(void 0,void 0,void 0,void 0,t.transformers),void 0===l?e.Debug.fail(\"Output generation failed\"):{outputText:l,diagnostics:a,sourceMapText:c}}var t;function r(n,r){t=t||e.filter(e.optionDeclarations,function(n){return\"object\"==typeof n.type&&!e.forEachEntry(n.type,function(e){return\"number\"!=typeof e})}),n=e.cloneCompilerOptions(n);for(var a=function(t){if(!e.hasProperty(n,t.name))return\"continue\";var a=n[t.name];e.isString(a)?n[t.name]=e.parseCustomTypeOption(t,a,r):e.forEachEntry(t.type,function(e){return e===a})||r.push(e.createCompilerDiagnosticForInvalidCustomType(t))},i=0,o=t;i<o.length;i++){a(o[i])}return n}e.transpileModule=n,e.transpile=function(t,r,a,i,o){var s=n(t,{compilerOptions:r,fileName:a,reportDiagnostics:!!i,moduleName:o});return e.addRange(i,s.diagnostics),s.outputText},e.fixupCompilerOptions=r}(Tn||(Tn={})),function(e){!function(n){!function(e){e[e.FormatDocument=0]=\"FormatDocument\",e[e.FormatSelection=1]=\"FormatSelection\",e[e.FormatOnEnter=2]=\"FormatOnEnter\",e[e.FormatOnSemicolon=3]=\"FormatOnSemicolon\",e[e.FormatOnOpeningCurlyBrace=4]=\"FormatOnOpeningCurlyBrace\",e[e.FormatOnClosingCurlyBrace=5]=\"FormatOnClosingCurlyBrace\"}(n.FormattingRequestKind||(n.FormattingRequestKind={}));var t=function(){function n(e,n,t){this.sourceFile=e,this.formattingRequestKind=n,this.options=t}return n.prototype.updateContext=function(n,t,r,a,i){e.Debug.assert(void 0!==n,\"currentTokenSpan is null\"),e.Debug.assert(void 0!==t,\"currentTokenParent is null\"),e.Debug.assert(void 0!==r,\"nextTokenSpan is null\"),e.Debug.assert(void 0!==a,\"nextTokenParent is null\"),e.Debug.assert(void 0!==i,\"commonParent is null\"),this.currentTokenSpan=n,this.currentTokenParent=t,this.nextTokenSpan=r,this.nextTokenParent=a,this.contextNode=i,this.contextNodeAllOnSameLine=void 0,this.nextNodeAllOnSameLine=void 0,this.tokensAreOnSameLine=void 0,this.contextNodeBlockIsOnOneLine=void 0,this.nextNodeBlockIsOnOneLine=void 0},n.prototype.ContextNodeAllOnSameLine=function(){return void 0===this.contextNodeAllOnSameLine&&(this.contextNodeAllOnSameLine=this.NodeIsOnOneLine(this.contextNode)),this.contextNodeAllOnSameLine},n.prototype.NextNodeAllOnSameLine=function(){return void 0===this.nextNodeAllOnSameLine&&(this.nextNodeAllOnSameLine=this.NodeIsOnOneLine(this.nextTokenParent)),this.nextNodeAllOnSameLine},n.prototype.TokensAreOnSameLine=function(){if(void 0===this.tokensAreOnSameLine){var e=this.sourceFile.getLineAndCharacterOfPosition(this.currentTokenSpan.pos).line,n=this.sourceFile.getLineAndCharacterOfPosition(this.nextTokenSpan.pos).line;this.tokensAreOnSameLine=e===n}return this.tokensAreOnSameLine},n.prototype.ContextNodeBlockIsOnOneLine=function(){return void 0===this.contextNodeBlockIsOnOneLine&&(this.contextNodeBlockIsOnOneLine=this.BlockIsOnOneLine(this.contextNode)),this.contextNodeBlockIsOnOneLine},n.prototype.NextNodeBlockIsOnOneLine=function(){return void 0===this.nextNodeBlockIsOnOneLine&&(this.nextNodeBlockIsOnOneLine=this.BlockIsOnOneLine(this.nextTokenParent)),this.nextNodeBlockIsOnOneLine},n.prototype.NodeIsOnOneLine=function(e){return this.sourceFile.getLineAndCharacterOfPosition(e.getStart(this.sourceFile)).line===this.sourceFile.getLineAndCharacterOfPosition(e.getEnd()).line},n.prototype.BlockIsOnOneLine=function(n){var t=e.findChildOfKind(n,17,this.sourceFile),r=e.findChildOfKind(n,18,this.sourceFile);return!(!t||!r)&&this.sourceFile.getLineAndCharacterOfPosition(t.getEnd()).line===this.sourceFile.getLineAndCharacterOfPosition(r.getStart(this.sourceFile)).line},n}();n.FormattingContext=t}(e.formatting||(e.formatting={}))}(Tn||(Tn={})),function(e){!function(n){var t,r=e.createScanner(6,!1,0),a=e.createScanner(6,!1,1);!function(e){e[e.Scan=0]=\"Scan\",e[e.RescanGreaterThanToken=1]=\"RescanGreaterThanToken\",e[e.RescanSlashToken=2]=\"RescanSlashToken\",e[e.RescanTemplateToken=3]=\"RescanTemplateToken\",e[e.RescanJsxIdentifier=4]=\"RescanJsxIdentifier\",e[e.RescanJsxText=5]=\"RescanJsxText\"}(t||(t={})),n.getFormattingScanner=function(n,t,i,o,s){var l=1===t?a:r;l.setText(n),l.setTextPos(i);var c,u,d,m,p,f=!0,g=s({advance:function(){p=void 0,l.getStartPos()!==i?f=!!u&&4===e.last(u).kind:l.scan(),c=void 0,u=void 0;for(var n=l.getStartPos();n<o;){var t=l.getToken();if(!e.isTrivia(t))break;l.scan();var r={pos:n,end:l.getStartPos(),kind:t};n=l.getStartPos(),c=e.append(c,r)}d=l.getStartPos()},readTokenInfo:function(n){e.Debug.assert(_());var t=function(e){switch(e.kind){case 31:case 66:case 67:case 47:case 46:return!0}return!1}(n)?1:12===n.kind?2:function(e){return 15===e.kind||16===e.kind}(n)?3:function(n){if(n.parent)switch(n.parent.kind){case 265:case 260:case 261:case 259:return e.isKeyword(n.kind)||71===n.kind}return!1}(n)?4:10===n.kind?5:0;if(p&&t===m)return v(p,n);l.getStartPos()!==d&&(e.Debug.assert(void 0!==p),l.setTextPos(d),l.scan());var r=function(n,t){var r,a=l.getToken();switch(m=0,t){case 1:if(29===a){m=1;var i=l.reScanGreaterToken();return e.Debug.assert(n.kind===i),i}break;case 2:if(41===(r=a)||63===r){m=2;var i=l.reScanSlashToken();return e.Debug.assert(n.kind===i),i}break;case 3:if(18===a)return m=3,l.reScanTemplateToken();break;case 4:return m=4,l.scanJsxIdentifier();case 5:return m=5,l.reScanJsxToken();case 0:break;default:e.Debug.assertNever(t)}return a}(n,t),a={pos:l.getStartPos(),end:l.getTextPos(),kind:r};for(u&&(u=void 0);l.getStartPos()<o&&(r=l.scan(),e.isTrivia(r));){var i={pos:l.getStartPos(),end:l.getTextPos(),kind:r};if(u||(u=[]),u.push(i),4===r){l.scan();break}}return v(p={leadingTrivia:c,trailingTrivia:u,token:a},n)},isOnToken:_,getCurrentLeadingTrivia:function(){return c},lastTrailingTriviaWasNewLine:function(){return f},skipToEndOf:function(e){l.setTextPos(e.end),d=l.getStartPos(),m=void 0,p=void 0,f=!1,c=void 0,u=void 0}});return p=void 0,l.setText(void 0),g;function _(){var n=p?p.token.kind:l.getToken();return(p?p.token.pos:l.getStartPos())<o&&1!==n&&!e.isTrivia(n)}function v(n,t){return e.isToken(t)&&n.token.kind!==t.kind&&(n.token.kind=t.kind),n}}}(e.formatting||(e.formatting={}))}(Tn||(Tn={})),function(e){!function(n){n.anyContext=e.emptyArray,function(e){e[e.Ignore=1]=\"Ignore\",e[e.Space=2]=\"Space\",e[e.NewLine=4]=\"NewLine\",e[e.Delete=8]=\"Delete\"}(n.RuleAction||(n.RuleAction={})),function(e){e[e.None=0]=\"None\",e[e.CanDeleteNewLines=1]=\"CanDeleteNewLines\"}(n.RuleFlags||(n.RuleFlags={}))}(e.formatting||(e.formatting={}))}(Tn||(Tn={})),function(e){!function(n){function t(e,n,t,r,i,o){return void 0===o&&(o=0),{leftTokenRange:a(n),rightTokenRange:a(t),rule:{debugName:e,context:r,action:i,flags:o}}}function r(e){return{tokens:e,isSpecific:!0}}function a(n){return\"number\"==typeof n?r([n]):e.isArray(n)?r(n):n}function i(n,t,a){void 0===a&&(a=[]);for(var i=[],o=n;o<=t;o++)e.contains(a,o)||i.push(o);return r(i)}function o(e){return function(n){return n.options&&n.options.hasOwnProperty(e)&&!!n.options[e]}}function s(e){return function(n){return n.options&&n.options.hasOwnProperty(e)&&!n.options[e]}}function l(e){return function(n){return!n.options||!n.options.hasOwnProperty(e)||!n.options[e]}}function c(e){return function(n){return!n.options||!n.options.hasOwnProperty(e)||!n.options[e]||n.TokensAreOnSameLine()}}function u(e){return function(n){return!n.options||!n.options.hasOwnProperty(e)||!!n.options[e]}}function d(e){return 223===e.contextNode.kind}function m(e){return!d(e)}function p(e){switch(e.contextNode.kind){case 202:case 203:case 173:case 210:case 255:case 251:case 161:case 171:case 172:return!0;case 184:case 240:case 246:case 235:case 149:case 276:case 152:case 151:return 58===e.currentTokenSpan.kind||58===e.nextTokenSpan.kind;case 224:case 148:return 92===e.currentTokenSpan.kind||92===e.nextTokenSpan.kind;case 225:return 145===e.currentTokenSpan.kind||145===e.nextTokenSpan.kind}return!1}function f(e){return!p(e)}function g(e){return!_(e)}function _(n){var t=n.contextNode.kind;return 152===t||151===t||149===t||235===t||e.isFunctionLikeKind(t)}function v(e){return 203===e.contextNode.kind||173===e.contextNode.kind}function y(e){return e.TokensAreOnSameLine()||S(e)}function h(e){return 182===e.contextNode.kind||179===e.contextNode.kind||function(e){return T(e)&&(e.ContextNodeAllOnSameLine()||e.ContextNodeBlockIsOnOneLine())}(e)}function b(e){return S(e)&&!(e.NextNodeAllOnSameLine()||e.NextNodeBlockIsOnOneLine())}function E(e){return T(e)&&!(e.ContextNodeAllOnSameLine()||e.ContextNodeBlockIsOnOneLine())}function T(e){return L(e.contextNode)}function S(e){return L(e.nextTokenParent)}function L(e){if(k(e))return!0;switch(e.kind){case 216:case 244:case 186:case 243:return!0}return!1}function x(e){switch(e.contextNode.kind){case 237:case 154:case 153:case 156:case 157:case 158:case 194:case 155:case 195:case 239:return!0}return!1}function A(e){return!x(e)}function C(e){return 237===e.contextNode.kind||194===e.contextNode.kind}function D(e){return k(e.contextNode)}function k(e){switch(e.kind){case 238:case 207:case 239:case 241:case 166:case 242:case 253:case 254:case 247:case 250:return!0}return!1}function M(e){switch(e.currentTokenParent.kind){case 238:case 242:case 241:case 272:case 243:case 230:return!0;case 216:var n=e.currentTokenParent.parent;if(!n||195!==n.kind&&194!==n.kind)return!0}return!1}function O(e){switch(e.contextNode.kind){case 220:case 230:case 223:case 224:case 225:case 222:case 233:case 221:case 229:case 272:return!0;default:return!1}}function I(e){return 186===e.contextNode.kind}function R(e){return function(e){return 189===e.contextNode.kind}(e)||function(e){return 190===e.contextNode.kind}(e)}function N(e){return 26!==e.currentTokenSpan.kind}function w(e){return 22!==e.nextTokenSpan.kind}function P(e){return 195===e.contextNode.kind}function F(e){return 181===e.contextNode.kind}function G(e){return e.TokensAreOnSameLine()&&10!==e.contextNode.kind}function V(e){return 258!==e.contextNode.kind&&262!==e.contextNode.kind}function K(e){return 268===e.contextNode.kind||267===e.contextNode.kind}function B(e){return 265===e.nextTokenParent.kind}function H(e){return 265===e.contextNode.kind}function U(e){return 259===e.contextNode.kind}function j(e){return!x(e)&&!S(e)}function W(e){return e.TokensAreOnSameLine()&&!!e.contextNode.decorators&&q(e.currentTokenParent)&&!q(e.nextTokenParent)}function q(n){for(;e.isExpressionNode(n);)n=n.parent;return 150===n.kind}function z(e){return 236===e.currentTokenParent.kind&&e.currentTokenParent.getStart(e.sourceFile)===e.currentTokenSpan.pos}function J(e){return 2!==e.formattingRequestKind}function X(e){return 242===e.contextNode.kind}function Y(e){return 166===e.contextNode.kind}function Q(e){return 159===e.contextNode.kind}function Z(e,n){if(27!==e.kind&&29!==e.kind)return!1;switch(n.kind){case 162:case 192:case 240:case 238:case 207:case 239:case 237:case 194:case 195:case 154:case 153:case 158:case 159:case 189:case 190:case 209:return!0;default:return!1}}function $(e){return Z(e.currentTokenSpan,e.currentTokenParent)||Z(e.nextTokenSpan,e.nextTokenParent)}function ee(e){return 192===e.contextNode.kind}function ne(e){return 105===e.currentTokenSpan.kind&&198===e.currentTokenParent.kind}function te(e){return 205===e.contextNode.kind&&void 0!==e.contextNode.expression}function re(e){return 211===e.contextNode.kind}n.getAllRules=function(){for(var a=[],T=0;T<=145;T++)a.push(T);function L(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return{tokens:a.filter(function(n){return!e.some(function(e){return e===n})}),isSpecific:!1}}var k={tokens:a,isSpecific:!1},q=r(a.concat([3])),Z=i(72,145),ae=i(27,70),ie=[92,93,145,118,127],oe=[71].concat(e.typeKeywords),se=q,le=r([71,3,75,84,91]),ce=r([20,3,81,102,87,82]),ue=[t(\"IgnoreBeforeComment\",k,[2,3],n.anyContext,1),t(\"IgnoreAfterLineComment\",2,k,n.anyContext,1),t(\"NotSpaceBeforeColon\",k,56,[G,f,g],8),t(\"SpaceAfterColon\",56,k,[G,f],2),t(\"NoSpaceBeforeQuestionMark\",k,55,[G,f],8),t(\"SpaceAfterQuestionMarkInConditionalOperator\",55,k,[G,v],2),t(\"NoSpaceAfterQuestionMark\",55,k,[G],8),t(\"NoSpaceBeforeDot\",k,23,[G],8),t(\"NoSpaceAfterDot\",23,k,[G],8),t(\"NoSpaceBetweenImportParenInImportType\",91,19,[G,F],8),t(\"NoSpaceAfterUnaryPrefixOperator\",[43,44,52,51],[8,71,19,21,17,99,94],[G,f],8),t(\"NoSpaceAfterUnaryPreincrementOperator\",43,[71,19,99,94],[G],8),t(\"NoSpaceAfterUnaryPredecrementOperator\",44,[71,19,99,94],[G],8),t(\"NoSpaceBeforeUnaryPostincrementOperator\",[71,20,22,94],43,[G],8),t(\"NoSpaceBeforeUnaryPostdecrementOperator\",[71,20,22,94],44,[G],8),t(\"SpaceAfterPostincrementWhenFollowedByAdd\",43,37,[G,p],2),t(\"SpaceAfterAddWhenFollowedByUnaryPlus\",37,37,[G,p],2),t(\"SpaceAfterAddWhenFollowedByPreincrement\",37,43,[G,p],2),t(\"SpaceAfterPostdecrementWhenFollowedBySubtract\",44,38,[G,p],2),t(\"SpaceAfterSubtractWhenFollowedByUnaryMinus\",38,38,[G,p],2),t(\"SpaceAfterSubtractWhenFollowedByPredecrement\",38,44,[G,p],2),t(\"NoSpaceAfterCloseBrace\",18,[26,25],[G],8),t(\"NewLineBeforeCloseBraceInBlockContext\",q,18,[E],4),t(\"SpaceAfterCloseBrace\",18,L(20),[G,M],2),t(\"SpaceBetweenCloseBraceAndElse\",18,82,[G],2),t(\"SpaceBetweenCloseBraceAndWhile\",18,106,[G],2),t(\"NoSpaceBetweenEmptyBraceBrackets\",17,18,[G,I],8),t(\"SpaceAfterConditionalClosingParen\",20,21,[O],2),t(\"NoSpaceBetweenFunctionKeywordAndStar\",89,39,[C],8),t(\"SpaceAfterStarInGeneratorDeclaration\",39,[71,19],[C],2),t(\"SpaceAfterFunctionInFuncDecl\",89,k,[x],2),t(\"NewLineAfterOpenBraceInBlockContext\",17,k,[E],4),t(\"SpaceAfterGetSetInMember\",[125,136],71,[x],2),t(\"NoSpaceBetweenYieldKeywordAndStar\",116,39,[G,te],8),t(\"SpaceBetweenYieldOrYieldStarAndOperand\",[116,39],k,[G,te],2),t(\"NoSpaceBetweenReturnAndSemicolon\",96,25,[G],8),t(\"SpaceAfterCertainKeywords\",[104,100,94,80,96,103,121],k,[G],2),t(\"SpaceAfterLetConstInVariableDeclaration\",[110,76],k,[G,z],2),t(\"NoSpaceBeforeOpenParenInFuncCall\",k,19,[G,R,N],8),t(\"SpaceBeforeBinaryKeywordOperator\",k,ie,[G,p],2),t(\"SpaceAfterBinaryKeywordOperator\",ie,k,[G,p],2),t(\"SpaceAfterVoidOperator\",105,k,[G,ne],2),t(\"SpaceBetweenAsyncAndOpenParen\",120,19,[P,G],2),t(\"SpaceBetweenAsyncAndFunctionKeyword\",120,89,[G],2),t(\"NoSpaceBetweenTagAndTemplateString\",[71,20],[13,14],[G],8),t(\"SpaceBeforeJsxAttribute\",k,71,[B,G],2),t(\"SpaceBeforeSlashInJsxOpeningElement\",k,41,[U,G],2),t(\"NoSpaceBeforeGreaterThanTokenInJsxOpeningElement\",41,29,[U,G],8),t(\"NoSpaceBeforeEqualInJsxAttribute\",k,58,[H,G],8),t(\"NoSpaceAfterEqualInJsxAttribute\",58,k,[H,G],8),t(\"NoSpaceAfterModuleImport\",[129,133],19,[G],8),t(\"SpaceAfterCertainTypeScriptKeywords\",[117,75,124,79,83,84,85,125,108,91,109,129,130,112,114,113,132,136,115,139,143,128,126],k,[G],2),t(\"SpaceBeforeCertainTypeScriptKeywords\",k,[85,108,143],[G],2),t(\"SpaceAfterModuleName\",9,17,[X],2),t(\"SpaceBeforeArrow\",k,36,[G],2),t(\"SpaceAfterArrow\",36,k,[G],2),t(\"NoSpaceAfterEllipsis\",24,71,[G],8),t(\"NoSpaceAfterOptionalParameters\",55,[20,26],[G,f],8),t(\"NoSpaceBetweenEmptyInterfaceBraceBrackets\",17,18,[G,Y],8),t(\"NoSpaceBeforeOpenAngularBracket\",oe,27,[G,$],8),t(\"NoSpaceBetweenCloseParenAndAngularBracket\",20,27,[G,$],8),t(\"NoSpaceAfterOpenAngularBracket\",27,k,[G,$],8),t(\"NoSpaceBeforeCloseAngularBracket\",k,29,[G,$],8),t(\"NoSpaceAfterCloseAngularBracket\",29,[19,21,29,26],[G,$,A],8),t(\"SpaceBeforeAt\",[20,71],57,[G],2),t(\"NoSpaceAfterAt\",57,k,[G],8),t(\"SpaceAfterDecorator\",k,[117,71,84,79,75,115,114,112,113,125,136,21,39],[W],2),t(\"NoSpaceBeforeNonNullAssertionOperator\",k,51,[G,re],8),t(\"NoSpaceAfterNewKeywordOnConstructorSignature\",94,19,[G,Q],8)],de=[t(\"SpaceAfterConstructor\",123,19,[o(\"insertSpaceAfterConstructor\"),G],2),t(\"NoSpaceAfterConstructor\",123,19,[l(\"insertSpaceAfterConstructor\"),G],8),t(\"SpaceAfterComma\",26,k,[o(\"insertSpaceAfterCommaDelimiter\"),G,V,w],2),t(\"NoSpaceAfterComma\",26,k,[l(\"insertSpaceAfterCommaDelimiter\"),G,V],8),t(\"SpaceAfterAnonymousFunctionKeyword\",89,19,[o(\"insertSpaceAfterFunctionKeywordForAnonymousFunctions\"),x],2),t(\"NoSpaceAfterAnonymousFunctionKeyword\",89,19,[l(\"insertSpaceAfterFunctionKeywordForAnonymousFunctions\"),x],8),t(\"SpaceAfterKeywordInControl\",Z,19,[o(\"insertSpaceAfterKeywordsInControlFlowStatements\"),O],2),t(\"NoSpaceAfterKeywordInControl\",Z,19,[l(\"insertSpaceAfterKeywordsInControlFlowStatements\"),O],8),t(\"SpaceAfterOpenParen\",19,k,[o(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis\"),G],2),t(\"SpaceBeforeCloseParen\",k,20,[o(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis\"),G],2),t(\"SpaceBetweenOpenParens\",19,19,[o(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis\"),G],2),t(\"NoSpaceBetweenParens\",19,20,[G],8),t(\"NoSpaceAfterOpenParen\",19,k,[l(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis\"),G],8),t(\"NoSpaceBeforeCloseParen\",k,20,[l(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis\"),G],8),t(\"SpaceAfterOpenBracket\",21,k,[o(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets\"),G],2),t(\"SpaceBeforeCloseBracket\",k,22,[o(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets\"),G],2),t(\"NoSpaceBetweenBrackets\",21,22,[G],8),t(\"NoSpaceAfterOpenBracket\",21,k,[l(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets\"),G],8),t(\"NoSpaceBeforeCloseBracket\",k,22,[l(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets\"),G],8),t(\"SpaceAfterOpenBrace\",17,k,[u(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces\"),h],2),t(\"SpaceBeforeCloseBrace\",k,18,[u(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces\"),h],2),t(\"NoSpaceBetweenEmptyBraceBrackets\",17,18,[G,I],8),t(\"NoSpaceAfterOpenBrace\",17,k,[s(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces\"),G],8),t(\"NoSpaceBeforeCloseBrace\",k,18,[s(\"insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces\"),G],8),t(\"SpaceAfterTemplateHeadAndMiddle\",[14,15],k,[o(\"insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces\"),G],2),t(\"SpaceBeforeTemplateMiddleAndTail\",k,[15,16],[o(\"insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces\"),G],2),t(\"NoSpaceAfterTemplateHeadAndMiddle\",[14,15],k,[l(\"insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces\"),G],8),t(\"NoSpaceBeforeTemplateMiddleAndTail\",k,[15,16],[l(\"insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces\"),G],8),t(\"SpaceAfterOpenBraceInJsxExpression\",17,k,[o(\"insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces\"),G,K],2),t(\"SpaceBeforeCloseBraceInJsxExpression\",k,18,[o(\"insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces\"),G,K],2),t(\"NoSpaceAfterOpenBraceInJsxExpression\",17,k,[l(\"insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces\"),G,K],8),t(\"NoSpaceBeforeCloseBraceInJsxExpression\",k,18,[l(\"insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces\"),G,K],8),t(\"SpaceAfterSemicolonInFor\",25,k,[o(\"insertSpaceAfterSemicolonInForStatements\"),G,d],2),t(\"NoSpaceAfterSemicolonInFor\",25,k,[l(\"insertSpaceAfterSemicolonInForStatements\"),G,d],8),t(\"SpaceBeforeBinaryOperator\",k,ae,[o(\"insertSpaceBeforeAndAfterBinaryOperators\"),G,p],2),t(\"SpaceAfterBinaryOperator\",ae,k,[o(\"insertSpaceBeforeAndAfterBinaryOperators\"),G,p],2),t(\"NoSpaceBeforeBinaryOperator\",k,ae,[l(\"insertSpaceBeforeAndAfterBinaryOperators\"),G,p],8),t(\"NoSpaceAfterBinaryOperator\",ae,k,[l(\"insertSpaceBeforeAndAfterBinaryOperators\"),G,p],8),t(\"SpaceBeforeOpenParenInFuncDecl\",k,19,[o(\"insertSpaceBeforeFunctionParenthesis\"),G,x],2),t(\"NoSpaceBeforeOpenParenInFuncDecl\",k,19,[l(\"insertSpaceBeforeFunctionParenthesis\"),G,x],8),t(\"NewLineBeforeOpenBraceInControl\",ce,17,[o(\"placeOpenBraceOnNewLineForControlBlocks\"),O,b],4,1),t(\"NewLineBeforeOpenBraceInFunction\",se,17,[o(\"placeOpenBraceOnNewLineForFunctions\"),x,b],4,1),t(\"NewLineBeforeOpenBraceInTypeScriptDeclWithBlock\",le,17,[o(\"placeOpenBraceOnNewLineForFunctions\"),D,b],4,1),t(\"SpaceAfterTypeAssertion\",29,k,[o(\"insertSpaceAfterTypeAssertion\"),G,ee],2),t(\"NoSpaceAfterTypeAssertion\",29,k,[l(\"insertSpaceAfterTypeAssertion\"),G,ee],8),t(\"SpaceBeforeTypeAnnotation\",k,56,[o(\"insertSpaceBeforeTypeAnnotation\"),G,_],2),t(\"NoSpaceBeforeTypeAnnotation\",k,56,[l(\"insertSpaceBeforeTypeAnnotation\"),G,_],8)],me=[t(\"NoSpaceBeforeSemicolon\",k,25,[G],8),t(\"SpaceBeforeOpenBraceInControl\",ce,17,[c(\"placeOpenBraceOnNewLineForControlBlocks\"),O,J,y],2,1),t(\"SpaceBeforeOpenBraceInFunction\",se,17,[c(\"placeOpenBraceOnNewLineForFunctions\"),x,S,J,y],2,1),t(\"SpaceBeforeOpenBraceInTypeScriptDeclWithBlock\",le,17,[c(\"placeOpenBraceOnNewLineForFunctions\"),D,J,y],2,1),t(\"NoSpaceBeforeComma\",k,26,[G],8),t(\"NoSpaceBeforeOpenBracket\",L(120,73),21,[G],8),t(\"NoSpaceAfterCloseBracket\",22,k,[G,j],8),t(\"SpaceAfterSemicolon\",25,k,[G],2),t(\"SpaceBetweenForAndAwaitKeyword\",88,121,[G],2),t(\"SpaceBetweenStatements\",[20,81,82,73],k,[G,V,m],2),t(\"SpaceAfterTryFinally\",[102,87],17,[G],2)];return ue.concat(de,me)}}(e.formatting||(e.formatting={}))}(Tn||(Tn={})),function(e){!function(n){var t;function r(n,t){return e.Debug.assert(n<=145&&t<=145,\"Must compute formatting context from tokens\"),n*s+t}n.getFormatContext=function(a){return{options:a,getRule:(void 0===t&&(i=n.getAllRules(),o=function(e){for(var n=new Array(s*s),t=new Array(n.length),a=0,i=e;a<i.length;a++)for(var o=i[a],c=o.leftTokenRange.isSpecific&&o.rightTokenRange.isSpecific,u=0,d=o.leftTokenRange.tokens;u<d.length;u++)for(var m=d[u],p=0,f=o.rightTokenRange.tokens;p<f.length;p++){var g=f[p],_=r(m,g),v=n[_];void 0===v&&(v=n[_]=[]),l(v,o.rule,c,t,_)}return n}(i),t=function(n){var t=o[r(n.currentTokenSpan.kind,n.nextTokenSpan.kind)];return t&&e.find(t,function(t){return e.every(t.context,function(e){return e(n)})})}),t)};var i,o};var a,i=5,o=31,s=146;function l(t,r,s,l,c){var u,d,m,p=1===r.action?s?a.IgnoreRulesSpecific:a.IgnoreRulesAny:r.context!==n.anyContext?s?a.ContextRulesSpecific:a.ContextRulesAny:s?a.NoContextRulesSpecific:a.NoContextRulesAny,f=l[c]||0;t.splice(function(e,n){for(var t=0,r=0;r<=n;r+=i)t+=e&o,e>>=i;return t}(f,p),0,r),l[c]=(m=1+((u=f)>>(d=p)&o),e.Debug.assert((m&o)===m,\"Adding more rules into the sub-bucket than allowed. Maximum allowed is 32 rules.\"),u&~(o<<d)|m<<d)}!function(e){e[e.IgnoreRulesSpecific=0]=\"IgnoreRulesSpecific\",e[e.IgnoreRulesAny=1*i]=\"IgnoreRulesAny\",e[e.ContextRulesSpecific=2*i]=\"ContextRulesSpecific\",e[e.ContextRulesAny=3*i]=\"ContextRulesAny\",e[e.NoContextRulesSpecific=4*i]=\"NoContextRulesSpecific\",e[e.NoContextRulesAny=5*i]=\"NoContextRulesAny\"}(a||(a={}))}(e.formatting||(e.formatting={}))}(Tn||(Tn={})),function(e){!function(n){var t,r,a,i,o;function s(n,t,r){var a=e.findPrecedingToken(n,r);return a&&a.kind===t&&n===a.getEnd()?a:void 0}function l(e){for(var n=e;n&&n.parent&&n.parent.end===e.end&&!c(n.parent,n);)n=n.parent;return n}function c(n,t){switch(n.kind){case 238:case 239:return e.rangeContainsRange(n.members,t);case 242:var r=n.body;return!!r&&243===r.kind&&e.rangeContainsRange(r.statements,t);case 277:case 216:case 243:return e.rangeContainsRange(n.statements,t);case 272:return e.rangeContainsRange(n.block.statements,t)}return!1}function u(n,t,r,a){return n?d({pos:e.getLineStartPositionForPosition(n.getStart(t),t),end:n.end},t,r,a):[]}function d(t,r,a,i){var o=function(n,t){return function r(a){var i=e.forEachChild(a,function(r){return e.startEndContainsRange(r.getStart(t),r.end,n)&&r});if(i){var o=r(i);if(o)return o}return a}(t)}(t,r);return n.getFormattingScanner(r.text,r.languageVariant,function(n,t,r){var a=n.getStart(r);if(a===t.pos&&n.end===t.end)return a;var i=e.findPrecedingToken(t.pos,r);return i?i.end>=t.pos?n.pos:i.end:n.pos}(o,t,r),t.end,function(s){return m(t,o,n.SmartIndenter.getIndentationForNode(o,t,r,a.options),function(e,t,r){for(var a,i=-1;e;){var o=r.getLineAndCharacterOfPosition(e.getStart(r)).line;if(-1!==i&&o!==i)break;if(n.SmartIndenter.shouldIndentChildNode(t,e,a,r))return t.indentSize;i=o,a=e,e=e.parent}return 0}(o,a.options,r),s,a,i,function(n,t){if(!n.length)return i;var r=n.filter(function(n){return e.rangeOverlapsWithStartEnd(t,n.start,n.start+n.length)}).sort(function(e,n){return e.start-n.start});if(!r.length)return i;var a=0;return function(n){for(;;){if(a>=r.length)return!1;var t=r[a];if(n.end<=t.start)return!1;if(e.startEndOverlapsWithStartEnd(n.pos,n.end,t.start,t.start+t.length))return!0;a++}};function i(){return!1}}(r.parseDiagnostics,t),r)})}function m(t,r,a,i,o,s,l,c,u){var d,m,f,g,_,v=s.options,y=s.getRule,h=new n.FormattingContext(u,l,v),b=[];if(o.advance(),o.isOnToken()){var E=u.getLineAndCharacterOfPosition(r.getStart(u)).line,T=E;r.decorators&&(T=u.getLineAndCharacterOfPosition(e.getNonDecoratorTokenPosOfNode(r,u)).line),function r(a,i,s,l,m,p){if(!e.rangeOverlapsWithStartEnd(t,a.getStart(u),a.getEnd()))return;var f=x(a,s,m,p);var y=i;e.forEachChild(a,function(e){b(e,-1,a,f,s,l,!1)},function(n){!function(n,t,r,i){e.Debug.assert(e.isNodeArray(n));var s=function(e,n){switch(e.kind){case 155:case 237:case 194:case 154:case 153:case 195:if(e.typeParameters===n)return 27;if(e.parameters===n)return 19;break;case 189:case 190:if(e.typeArguments===n)return 27;if(e.arguments===n)return 19;break;case 162:if(e.typeArguments===n)return 27}return 0}(t,n),l=i,c=r;if(0!==s)for(;o.isOnToken();){var d=o.readTokenInfo(t);if(d.token.end>n.pos)break;if(d.token.kind===s){c=u.getLineAndCharacterOfPosition(d.token.pos).line;var m=L(d.token,c,-1,t,i,r);l=x(t,r,m.indentation,m.delta),E(d,t,l,t)}else E(d,t,i,t)}for(var p=-1,f=0;f<n.length;f++){var g=n[f];p=b(g,p,a,l,c,c,!0,0===f)}var _=function(e){switch(e){case 19:return 20;case 27:return 29}return 0}(s);if(0!==_&&o.isOnToken()){var d=o.readTokenInfo(t);26===d.token.kind&&e.isCallLikeExpression(t)&&(o.advance(),d=o.readTokenInfo(t)),d.token.kind===_&&e.rangeContainsRange(t,d.token)&&E(d,t,l,t)}}(n,a,s,f)});for(;o.isOnToken();){var h=o.readTokenInfo(a);if(h.token.end>a.end)break;E(h,a,f,a)}function b(i,s,l,c,d,m,p,f){var g=i.getStart(u),_=u.getLineAndCharacterOfPosition(g).line,h=_;i.decorators&&(h=u.getLineAndCharacterOfPosition(e.getNonDecoratorTokenPosOfNode(i,u)).line);var b=-1;if(p&&e.rangeContainsRange(t,l)&&-1!==(b=function(t,r,a,i,o){if(e.rangeOverlapsWithStartEnd(i,t,r)||e.rangeContainsStartEnd(i,t,r)){if(-1!==o)return o}else{var s=u.getLineAndCharacterOfPosition(t).line,l=e.getLineStartPositionForPosition(t,u),c=n.SmartIndenter.findFirstNonWhitespaceColumn(l,t,u,v);if(s!==a||t===c){var d=n.SmartIndenter.getBaseIndentation(v);return d>c?d:c}}return-1}(g,i.end,d,t,s))&&(s=b),!e.rangeOverlapsWithStartEnd(t,i.pos,i.end))return i.end<t.pos&&o.skipToEndOf(i),s;if(0===i.getFullWidth())return s;for(;o.isOnToken();){var T=o.readTokenInfo(a);if(T.token.end>g)break;E(T,a,c,a)}if(!o.isOnToken())return s;if(e.isToken(i)&&10!==i.kind){var T=o.readTokenInfo(i);return e.Debug.assert(T.token.end===i.end,\"Token end is child end\"),E(T,a,c,i),s}var S=150===i.kind?_:m,x=L(i,_,b,a,c,S);if(r(i,y,_,h,x.indentation,x.delta),10===i.kind){var A={pos:i.getStart(),end:i.getEnd()};k(A,x.indentation,!0,!1)}return y=a,f&&185===l.kind&&-1===s&&(s=x.indentation),s}function E(n,r,a,i){e.Debug.assert(e.rangeContainsRange(r,n.token));var s=o.lastTrailingTriviaWasNewLine(),l=!1;n.leadingTrivia&&A(n.leadingTrivia,r,y,a);var m=0,p=e.rangeContainsRange(t,n.token),f=u.getLineAndCharacterOfPosition(n.token.pos);if(p){var v=c(n.token),h=d;if(m=C(n.token,f,r,y,a),!v)if(0===m){var b=h&&u.getLineAndCharacterOfPosition(h.end).line;l=s&&f.line!==b}else l=1===m}if(n.trailingTrivia&&A(n.trailingTrivia,r,y,a),l){var E=p&&!c(n.token)?a.getIndentationForToken(f.line,n.token.kind,i):-1,T=!0;if(n.leadingTrivia)for(var S=a.getIndentationForComment(n.token.kind,E,i),L=0,x=n.leadingTrivia;L<x.length;L++){var M=x[L],O=e.rangeContainsRange(t,M);switch(M.kind){case 3:O&&k(M,S,!T),T=!1;break;case 2:T&&O&&D(M.pos,S,!1),T=!1;break;case 4:T=!0}}-1!==E&&T&&(D(n.token.pos,E,1===m),g=f.line,_=E)}o.advance(),y=r}}(r,r,E,T,a,i)}if(!o.isOnToken()){var S=o.getCurrentLeadingTrivia();S&&(A(S,r,r,void 0),function(){var e=d?d.end:t.pos,n=u.getLineAndCharacterOfPosition(e).line,r=u.getLineAndCharacterOfPosition(t.end).line;M(n,r+1,d)}())}return b;function L(e,t,r,a,i,o){var s=n.SmartIndenter.shouldIndentChildNode(v,e)?v.indentSize:0;return o===t?{indentation:t===g?_:i.getIndentation(),delta:Math.min(v.indentSize,i.getDelta(e)+s)}:-1===r?19===e.kind&&t===g?{indentation:_,delta:i.getDelta(e)}:n.SmartIndenter.childStartsOnTheSameLineWithElseInIfStatement(a,e,t,u)?{indentation:i.getIndentation(),delta:s}:{indentation:i.getIndentation()+i.getDelta(e),delta:s}:{indentation:r,delta:s}}function x(t,r,a,i){return{getIndentationForComment:function(e,n,t){switch(e){case 18:case 22:case 20:return a+o(t)}return-1!==n?n:a},getIndentationForToken:function(n,i,s){return function(n,a,i){switch(a){case 17:case 18:case 19:case 20:case 82:case 106:case 57:return!1;case 41:case 29:switch(i.kind){case 260:case 261:case 259:return!1}break;case 21:case 22:if(179!==i.kind)return!1}return r!==n&&!(t.decorators&&a===function(n){if(n.modifiers&&n.modifiers.length)return n.modifiers[0].kind;switch(n.kind){case 238:return 75;case 239:return 109;case 237:return 89;case 241:return 241;case 156:return 125;case 157:return 136;case 154:if(n.asteriskToken)return 39;case 152:case 149:var t=e.getNameOfDeclaration(n);if(t)return t.kind}}(t))}(n,i,s)?a+o(s):a},getIndentation:function(){return a},getDelta:o,recomputeIndentation:function(e){t.parent&&n.SmartIndenter.shouldIndentChildNode(v,t.parent,t,u)&&(a+=e?v.indentSize:-v.indentSize,i=n.SmartIndenter.shouldIndentChildNode(v,t)?v.indentSize:0)}};function o(e){return n.SmartIndenter.nodeWillIndentChild(v,t,e,u,!0)?i:0}}function A(n,r,a,i){for(var o=0,s=n;o<s.length;o++){var l=s[o];if(e.isComment(l.kind)&&e.rangeContainsRange(t,l))C(l,u.getLineAndCharacterOfPosition(l.pos),r,a,i)}}function C(n,r,a,i,o){var s=0;c(n)||(d?s=function(n,t,r,a,i,o,s,l){h.updateContext(a,o,n,r,s);var c,d=y(h),m=0;if(d){switch(m=function(e,n,t,r,a){var i=a!==t;switch(e.action){case 1:return 0;case 8:if(n.end!==r.pos)return I(n.end,r.pos-n.end),i?2:0;break;case 4:if(1!==e.flags&&t!==a)return 0;var o=a-t;if(1!==o)return R(n.end,r.pos-n.end,v.newLineCharacter),i?0:1;break;case 2:if(1!==e.flags&&t!==a)return 0;var s=r.pos-n.end;if(1!==s||32!==u.text.charCodeAt(n.end))return R(n.end,r.pos-n.end,\" \"),i?2:0}return 0}(d,a,i,n,t)){case 2:r.getStart(u)===n.pos&&l.recomputeIndentation(!1);break;case 1:r.getStart(u)===n.pos&&l.recomputeIndentation(!0);break;default:e.Debug.assert(0===m)}c=!(8&d.action)&&1!==d.flags}else c=!0;t!==i&&c&&M(i,t,a);return m}(n,r.line,a,d,f,m,i,o):M(u.getLineAndCharacterOfPosition(t.pos).line,r.line));return d=n,m=a,f=r.line,s}function D(n,t,r){var a=p(t,v);if(r)R(n,0,a);else{var i=u.getLineAndCharacterOfPosition(n),o=e.getStartPositionOfLine(i.line,u);(t!==function(e,n){for(var t=0,r=0;r<n;r++)9===u.text.charCodeAt(e+r)?t+=v.tabSize-t%v.tabSize:t++;return t}(o,i.character)||function(e,n){return e!==u.text.substr(n,e.length)}(a,o))&&R(o,i.character,a)}}function k(t,r,a,i){void 0===i&&(i=!0);var o=u.getLineAndCharacterOfPosition(t.pos).line,s=u.getLineAndCharacterOfPosition(t.end).line;if(o!==s){for(var l=[],c=t.pos,d=o;d<s;d++){var m=e.getEndLinePosition(d,u);l.push({pos:c,end:m}),c=e.getStartPositionOfLine(d+1,u)}if(i&&l.push({pos:c,end:t.end}),0!==l.length){var f=e.getStartPositionOfLine(o,u),g=n.SmartIndenter.findFirstNonWhitespaceCharacterAndColumn(f,l[0].pos,u,v);if(r!==g.column){var _=0;a&&(_=1,o++);for(var y=r-g.column,h=_;h<l.length;h++,o++){var b=e.getStartPositionOfLine(o,u),E=0===h?g:n.SmartIndenter.findFirstNonWhitespaceCharacterAndColumn(l[h].pos,l[h].end,u,v),T=E.column+y;if(T>0){var S=p(T,v);R(b,E.character,S)}else I(b,E.character)}}}}else a||D(t.pos,r,!1)}function M(n,t,r){for(var a=n;a<t;a++){var i=e.getStartPositionOfLine(a,u),o=e.getEndLinePosition(a,u);if(!(r&&(e.isComment(r.kind)||e.isStringOrRegularExpressionOrTemplateLiteral(r.kind))&&r.pos<=o&&r.end>o)){var s=O(i,o);-1!==s&&(e.Debug.assert(s===i||!e.isWhiteSpaceSingleLine(u.text.charCodeAt(s-1))),I(s,o+1-s))}}}function O(n,t){for(var r=t;r>=n&&e.isWhiteSpaceSingleLine(u.text.charCodeAt(r));)r--;return r!==t?r+1:-1}function I(n,t){t&&b.push(e.createTextChangeFromStartLength(n,t,\"\"))}function R(n,t,r){(t||r)&&b.push(e.createTextChangeFromStartLength(n,t,r))}}function p(n,t){if((!a||a.tabSize!==t.tabSize||a.indentSize!==t.indentSize)&&(a={tabSize:t.tabSize,indentSize:t.indentSize},i=o=void 0),t.convertTabsToSpaces){var r=void 0,s=Math.floor(n/t.indentSize),l=n%t.indentSize;return o||(o=[]),void 0===o[s]?(r=e.repeatString(\" \",t.indentSize*s),o[s]=r):r=o[s],l?r+e.repeatString(\" \",l):r}var c=Math.floor(n/t.tabSize),u=n-c*t.tabSize,d=void 0;return i||(i=[]),void 0===i[c]?i[c]=d=e.repeatString(\"\\t\",c):d=i[c],u?d+e.repeatString(\" \",u):d}!function(e){e[e.Unknown=-1]=\"Unknown\"}(t||(t={})),n.formatOnEnter=function(n,t,r){var a=t.getLineAndCharacterOfPosition(n).line;if(0===a)return[];for(var i=e.getEndLinePosition(a,t);e.isWhiteSpaceSingleLine(t.text.charCodeAt(i));)i--;return e.isLineBreak(t.text.charCodeAt(i))&&i--,d({pos:e.getStartPositionOfLine(a-1,t),end:i+1},t,r,2)},n.formatOnSemicolon=function(e,n,t){return u(l(s(e,25,n)),n,t,3)},n.formatOnOpeningCurly=function(n,t,r){var a=s(n,17,t);if(!a)return[];var i=l(a.parent);return d({pos:e.getLineStartPositionForPosition(i.getStart(t),t),end:n},t,r,4)},n.formatOnClosingCurly=function(e,n,t){return u(l(s(e,18,n)),n,t,5)},n.formatDocument=function(e,n){return d({pos:0,end:e.text.length},e,n,0)},n.formatSelection=function(n,t,r,a){return d({pos:e.getLineStartPositionForPosition(n,r),end:t},r,a,1)},n.formatNodeGivenIndentation=function(e,t,r,a,i,o){var s={pos:0,end:t.text.length};return n.getFormattingScanner(t.text,r,s.pos,s.end,function(n){return m(s,e,a,i,n,o,1,function(e){return!1},t)})},function(e){e[e.None=0]=\"None\",e[e.LineAdded=1]=\"LineAdded\",e[e.LineRemoved=2]=\"LineRemoved\"}(r||(r={})),n.getRangeOfEnclosingComment=function(n,t,r,a){void 0===a&&(a=e.getTokenAtPosition(n,t));var i=e.findAncestor(a,e.isJSDoc);if(i&&(a=i.parent),!(a.getStart(n)<=t&&t<a.getEnd())){var o=(r=null===r?void 0:void 0===r?e.findPrecedingToken(t,n):r)&&e.getTrailingCommentRanges(n.text,r.end),s=e.getLeadingCommentRangesOfNode(a,n),l=e.concatenate(o,s);return l&&e.find(l,function(r){return e.rangeContainsPositionExclusive(r,t)||t===r.end&&(2===r.kind||t===n.getFullWidth())})}},n.getIndentationString=p}(e.formatting||(e.formatting={}))}(Tn||(Tn={})),function(e){!function(n){!function(t){var r,a;function i(e){return e.baseIndentSize||0}function o(e,n,t,r,a,o,c){for(var u=e.parent;u;){var p=!0;if(t){var f=e.getStart(a);p=f<t.pos||f>t.end}if(p)if(-1!==(h=g(e,a,c)))return h+r;var v=s(u,e,a),y=v.line===n.line||m(u,e,n.line,a);if(p){var h;if(-1!==(h=l(e,u,n,y,a,c)))return h+r;if(-1!==(h=_(e,a,c)))return h+r}T(c,u,e,a,o)&&!y&&(r+=c.indentSize);var b=d(u,e,n.line,a);u=(e=u).parent,n=b?a.getLineAndCharacterOfPosition(e.getStart(a)):v}return r+i(c)}function s(e,n,t){var r=f(n,t),a=r?r.pos:e.getStart(t);return t.getLineAndCharacterOfPosition(a)}function l(n,t,r,a,i,o){return(e.isDeclaration(n)||e.isStatementButNotDeclaration(n))&&(277===t.kind||!a)?y(r,i,o):-1}function c(n,t,r,a){var i=e.findNextToken(n,t,a);return i?17===i.kind?1:18===i.kind&&r===u(i,a).line?2:0:0}function u(e,n){return n.getLineAndCharacterOfPosition(e.getStart(n))}function d(n,t,r,a){if(!e.isCallExpression(n)||!e.contains(n.arguments,t))return!1;var i=n.expression.getEnd();return e.getLineAndCharacterOfPosition(a,i).line===r}function m(n,t,r,a){if(220===n.kind&&n.elseStatement===t){var i=e.findChildOfKind(n,82,a);return e.Debug.assert(void 0!==i),u(i,a).line===r}return!1}function p(n,t,r){return n&&e.rangeContainsStartEnd(n,t,r)?n:void 0}function f(e,n){if(e.parent){var t=e.end;switch(e.parent.kind){case 162:return p(e.parent.typeArguments,e.getStart(n),t);case 186:return e.parent.properties;case 185:return e.parent.elements;case 237:case 194:case 195:case 154:case 153:case 158:case 155:case 164:case 159:var r=e.getStart(n);return p(e.parent.typeParameters,r,t)||p(e.parent.parameters,r,t);case 238:case 207:case 239:case 240:case 301:return p(e.parent.typeParameters,e.getStart(n),t);case 190:case 189:r=e.getStart(n);return p(e.parent.typeArguments,r,t)||p(e.parent.arguments,r,t);case 236:return p(e.parent.declarations,e.getStart(n),t);case 250:case 254:return p(e.parent.elements,e.getStart(n),t);case 182:case 183:return p(e.parent.elements,e.getStart(n),t)}}}function g(e,n,t){var r=f(e,n);if(r){var a=r.indexOf(e);if(-1!==a)return v(r,a,n,t)}return-1}function _(n,t,r){if(20===n.kind)return-1;if(n.parent&&e.isCallOrNewExpression(n.parent)&&n.parent.expression!==n){var a=n.parent.expression,i=function(e){for(;;)switch(e.kind){case 189:case 190:case 187:case 188:e=e.expression;break;default:return e}}(a);if(a===i)return-1;var o=t.getLineAndCharacterOfPosition(a.end),s=t.getLineAndCharacterOfPosition(i.end);return o.line===s.line?-1:y(o,t,r)}return-1}function v(n,t,r,a){e.Debug.assert(t>=0&&t<n.length);for(var i=u(n[t],r),o=t-1;o>=0;o--)if(26!==n[o].kind){if(r.getLineAndCharacterOfPosition(n[o].end).line!==i.line)return y(i,r,a);i=u(n[o],r)}return-1}function y(e,n,t){var r=n.getPositionOfLineAndCharacter(e.line,0);return b(r,r+e.character,n,t)}function h(n,t,r,a){for(var i=0,o=0,s=n;s<t;s++){var l=r.text.charCodeAt(s);if(!e.isWhiteSpaceSingleLine(l))break;9===l?o+=a.tabSize+o%a.tabSize:o++,i++}return{column:o,character:i}}function b(e,n,t,r){return h(e,n,t,r).column}function E(n,t,r,a,i){var o=r?r.kind:0;switch(t.kind){case 219:case 238:case 207:case 239:case 241:case 240:case 185:case 216:case 243:case 186:case 166:case 179:case 168:case 244:case 270:case 269:case 193:case 187:case 189:case 190:case 217:case 252:case 228:case 203:case 183:case 182:case 260:case 263:case 259:case 268:case 153:case 158:case 159:case 149:case 163:case 164:case 175:case 191:case 199:case 254:case 250:case 255:case 251:case 152:return!0;case 235:case 273:return!(!n.indentMultiLineObjectLiteralBeginningOnBlankLine&&a&&186===o)||function(n,t){var r=e.skipTrivia(n.text,t.pos),a=n.getLineAndCharacterOfPosition(r).line,i=n.getLineAndCharacterOfPosition(t.end).line;return a===i}(a,r);case 221:case 222:case 224:case 225:case 223:case 220:case 237:case 194:case 154:case 195:case 155:case 156:case 157:return 216!==o;case 253:return 254!==o;case 247:return 248!==o||!!r.namedBindings&&250!==r.namedBindings.kind;case 258:return 261!==o;case 262:return 264!==o;case 172:case 171:if(166===o)return!1}return i}function T(e,n,t,r,a){return void 0===a&&(a=!1),E(e,n,t,r,!1)&&!(a&&t&&function(e,n){switch(e){case 228:case 232:case 226:case 227:return 216!==n.kind;default:return!1}}(t.kind,n))}!function(e){e[e.Unknown=-1]=\"Unknown\"}(r||(r={})),t.getIndentation=function(t,r,a,s){if(void 0===s&&(s=!1),t>r.text.length)return i(a);if(a.indentStyle===e.IndentStyle.None)return 0;var l=e.findPrecedingToken(t,r,void 0,!0),d=n.getRangeOfEnclosingComment(r,t,l||null);if(d&&3===d.kind)return function(n,t,r,a){var i=e.getLineAndCharacterOfPosition(n,t).line-1,o=e.getLineAndCharacterOfPosition(n,a.pos).line;if(e.Debug.assert(o>=0),i<=o)return b(e.getStartPositionOfLine(o,n),t,n,r);var s=e.getStartPositionOfLine(i,n),l=h(s,t,n,r),c=l.column,u=l.character;return 0===c?c:42===n.text.charCodeAt(s+u)?c-1:c}(r,t,a,d);if(!l)return i(a);if(e.isStringOrRegularExpressionOrTemplateLiteral(l.kind)&&l.getStart(r)<=t&&t<l.end)return 0;var m=r.getLineAndCharacterOfPosition(t).line;if(a.indentStyle===e.IndentStyle.Block)return function(n,t,r){for(var a=t;a>0;){var i=n.text.charCodeAt(a);if(!e.isWhiteSpaceLike(i))break;a--}return b(e.getLineStartPositionForPosition(a,n),a,n,r)}(r,t,a);if(26===l.kind&&202!==l.parent.kind){var p=function(n,t,r){var a=e.findListItemInfo(n);return a&&a.listItemIndex>0?v(a.list.getChildren(),a.listItemIndex-1,t,r):-1}(l,r,a);if(-1!==p)return p}return function(n,t,r,a,s,l){for(var d,m=r;m;){if(e.positionBelongsToNode(m,t,n)&&T(l,m,d,n,!0)){var p=u(m,n),f=c(r,m,a,n),v=0!==f?s&&2===f?l.indentSize:0:a!==p.line?l.indentSize:0;return o(m,p,void 0,v,n,!0,l)}var y=g(m,n,l);if(-1!==y)return y;if(-1!==(y=_(m,n,l)))return y+l.indentSize;d=m,m=m.parent}return i(l)}(r,t,l,m,s,a)},t.getIndentationForNode=function(e,n,t,r){var a=t.getLineAndCharacterOfPosition(e.getStart(t));return o(e,a,n,0,t,!1,r)},t.getBaseIndentation=i,function(e){e[e.Unknown=0]=\"Unknown\",e[e.OpenBrace=1]=\"OpenBrace\",e[e.CloseBrace=2]=\"CloseBrace\"}(a||(a={})),t.isArgumentAndStartLineOverlapsExpressionBeingCalled=d,t.childStartsOnTheSameLineWithElseInIfStatement=m,t.getContainingList=f,t.findFirstNonWhitespaceCharacterAndColumn=h,t.findFirstNonWhitespaceColumn=b,t.nodeWillIndentChild=E,t.shouldIndentChildNode=T}(n.SmartIndenter||(n.SmartIndenter={}))}(e.formatting||(e.formatting={}))}(Tn||(Tn={})),function(e){!function(n){function t(n){var t=n.__pos;return e.Debug.assert(\"number\"==typeof t),t}function r(n,t){e.Debug.assert(\"number\"==typeof t),n.__pos=t}function a(n){var t=n.__end;return e.Debug.assert(\"number\"==typeof t),t}function i(n,t){e.Debug.assert(\"number\"==typeof t),n.__end=t}var o,s;function l(n,t){return e.skipTrivia(n,t,!1,!0)}function c(e,n,t,r){return{pos:u(e,n,r,o.Start),end:d(e,t,r)}}function u(n,t,r,a){if(r.useNonAdjustedStartPosition)return t.getStart(n);var i=t.getFullStart(),s=t.getStart(n);if(i===s)return s;var c=e.getLineStartPositionForPosition(i,n);if(e.getLineStartPositionForPosition(s,n)===c)return a===o.Start?s:i;var u=i>0?1:0,d=e.getStartPositionOfLine(e.getLineOfLocalPosition(n,c)+u,n);return d=l(n.text,d),e.getStartPositionOfLine(e.getLineOfLocalPosition(n,d),n)}function d(n,t,r){var a=t.end;if(r.useNonAdjustedEndPosition||e.isExpression(t))return a;var i=e.skipTrivia(n.text,a,!0);return i!==a&&e.isLineBreak(n.text.charCodeAt(i-1))?i:a}function m(e,n){return!!n&&!!e.parent&&(26===n.kind||25===n.kind&&186===e.parent.kind)}!function(e){e[e.FullStart=0]=\"FullStart\",e[e.Start=1]=\"Start\"}(o=n.Position||(n.Position={})),n.useNonAdjustedPositions={useNonAdjustedStartPosition:!0,useNonAdjustedEndPosition:!0},function(e){e[e.Remove=0]=\"Remove\",e[e.ReplaceWithSingleNode=1]=\"ReplaceWithSingleNode\",e[e.ReplaceWithMultipleNodes=2]=\"ReplaceWithMultipleNodes\",e[e.Text=3]=\"Text\"}(s||(s={}));var p,f=function(){function t(n,t){this.newLineCharacter=n,this.formatContext=t,this.changes=[],this.newFiles=[],this.classesWithNodesInsertedAtStart=e.createMap(),this.deletedNodes=[]}return t.fromContext=function(n){return new t(e.getNewLineOrDefaultFromHost(n.host,n.formatContext.options),n.formatContext)},t.with=function(e,n){var r=t.fromContext(e);return n(r),r.getChanges()},t.prototype.deleteRange=function(e,n){return this.changes.push({kind:s.Remove,sourceFile:e,range:n}),this},t.prototype.delete=function(e,n){this.deletedNodes.push({sourceFile:e,node:n})},t.prototype.deleteModifier=function(n,t){this.deleteRange(n,{pos:t.getStart(n),end:e.skipTrivia(n.text,t.end,!0)})},t.prototype.deleteNodeRange=function(e,n,t,r){void 0===r&&(r={});var a=u(e,n,r,o.FullStart),i=d(e,t,r);return this.deleteRange(e,{pos:a,end:i}),this},t.prototype.deleteNodeRangeExcludingEnd=function(e,n,t,r){void 0===r&&(r={});var a=u(e,n,r,o.FullStart),i=void 0===t?e.text.length:u(e,t,r,o.FullStart);this.deleteRange(e,{pos:a,end:i})},t.prototype.replaceRange=function(e,n,t,r){return void 0===r&&(r={}),this.changes.push({kind:s.ReplaceWithSingleNode,sourceFile:e,range:n,options:r,node:t}),this},t.prototype.replaceNode=function(e,t,r,a){return void 0===a&&(a=n.useNonAdjustedPositions),this.replaceRange(e,c(e,t,t,a),r,a)},t.prototype.replaceNodeRange=function(e,t,r,a,i){void 0===i&&(i=n.useNonAdjustedPositions),this.replaceRange(e,c(e,t,r,i),a,i)},t.prototype.replaceRangeWithNodes=function(e,n,t,r){return void 0===r&&(r={}),this.changes.push({kind:s.ReplaceWithMultipleNodes,sourceFile:e,range:n,options:r,nodes:t}),this},t.prototype.replaceNodeWithNodes=function(e,t,r,a){return void 0===a&&(a=n.useNonAdjustedPositions),this.replaceRangeWithNodes(e,c(e,t,t,a),r,a)},t.prototype.replaceNodeRangeWithNodes=function(e,t,r,a,i){return void 0===i&&(i=n.useNonAdjustedPositions),this.replaceRangeWithNodes(e,c(e,t,r,i),a,i)},t.prototype.nextCommaToken=function(n,t){var r=e.findNextToken(t,t.parent,n);return r&&26===r.kind?r:void 0},t.prototype.replacePropertyAssignment=function(e,n,t){var r=this.nextCommaToken(e,n)?\"\":\",\"+this.newLineCharacter;return this.replaceNode(e,n,t,{suffix:r})},t.prototype.insertNodeAt=function(n,t,r,a){void 0===a&&(a={}),this.replaceRange(n,e.createRange(t),r,a)},t.prototype.insertNodesAt=function(e,n,t,r){void 0===r&&(r={}),this.changes.push({kind:s.ReplaceWithMultipleNodes,sourceFile:e,options:r,nodes:t,range:{pos:n,end:n}})},t.prototype.insertNodeAtTopOfFile=function(n,t,r){var a=function(n){var t=n.text,r=e.getShebang(t),a=0;void 0!==r&&(a=r.length,c());var i=e.getLeadingCommentRanges(t,a);if(!i)return a;i.length&&3===i[0].kind&&e.isPinnedComment(t,i[0].pos)&&(a=i[0].end,c(),i=i.slice(1));for(var o=0,s=i;o<s.length;o++){var l=s[o];if(2!==l.kind||!e.isRecognizedTripleSlashComment(t,l.pos,l.end))break;a=l.end,c()}return a;function c(){if(a<t.length){var n=t.charCodeAt(a);e.isLineBreak(n)&&++a<t.length&&13===n&&10===t.charCodeAt(a)&&a++}}}(n);this.insertNodeAt(n,a,t,{prefix:0===a?void 0:this.newLineCharacter,suffix:(e.isLineBreak(n.text.charCodeAt(a))?\"\":this.newLineCharacter)+(r?this.newLineCharacter:\"\")})},t.prototype.insertNodeBefore=function(e,n,t,r){void 0===r&&(r=!1),this.insertNodeAt(e,u(e,n,{},o.Start),t,this.getOptionsForInsertNodeBefore(n,r))},t.prototype.insertModifierBefore=function(n,t,r){var a=r.getStart(n);this.replaceRange(n,{pos:a,end:a},e.createToken(t),{suffix:\" \"})},t.prototype.insertLastModifierBefore=function(n,t,r){if(r.modifiers){var a=r.modifiers.end;this.replaceRange(n,{pos:a,end:a},e.createToken(t),{prefix:\" \"})}else this.insertModifierBefore(n,t,r)},t.prototype.insertCommentBeforeLine=function(n,t,r,a){var i=e.getStartPositionOfLine(t,n),o=e.getFirstNonSpaceCharacterPosition(n.text,i),s=S(n,o),l=e.getTouchingToken(n,s?o:r),c=n.text.slice(i,o),u=(s?\"\":this.newLineCharacter)+\"//\"+a+this.newLineCharacter+c;this.insertText(n,l.getStart(n),u)},t.prototype.replaceRangeWithText=function(e,n,t){this.changes.push({kind:s.Text,sourceFile:e,range:n,text:t})},t.prototype.insertText=function(n,t,r){this.replaceRangeWithText(n,e.createRange(t),r)},t.prototype.tryInsertTypeAnnotation=function(n,t,r){var a;if(e.isFunctionLike(t)){if(!(a=e.findChildOfKind(t,20,n))){if(!e.isArrowFunction(t))return;a=e.first(t.parameters)}}else a=235!==t.kind&&t.questionToken?t.questionToken:t.name;this.insertNodeAt(n,a.end,r,{prefix:\": \"})},t.prototype.insertTypeParameters=function(n,t,r){var a=(e.findChildOfKind(t,19,n)||e.first(t.parameters)).getStart(n);this.insertNodesAt(n,a,r,{prefix:\"<\",suffix:\">\"})},t.prototype.getOptionsForInsertNodeBefore=function(n,t){return e.isStatement(n)||e.isClassElement(n)?{suffix:t?this.newLineCharacter+this.newLineCharacter:this.newLineCharacter}:e.isVariableDeclaration(n)?{suffix:\", \"}:e.isParameter(n)?{}:e.isStringLiteral(n)&&e.isImportDeclaration(n.parent)||e.isNamedImports(n)?{suffix:\", \"}:e.Debug.failBadSyntaxKind(n)},t.prototype.insertNodeAtConstructorStart=function(n,t,r){var a=e.firstOrUndefined(t.body.statements);a&&t.body.multiLine?this.insertNodeBefore(n,a,r):this.replaceConstructorBody(n,t,[r].concat(t.body.statements))},t.prototype.insertNodeAtConstructorEnd=function(n,t,r){var a=e.lastOrUndefined(t.body.statements);a&&t.body.multiLine?this.insertNodeAfter(n,a,r):this.replaceConstructorBody(n,t,t.body.statements.concat([r]))},t.prototype.replaceConstructorBody=function(n,t,r){this.replaceNode(n,t.body,e.createBlock(r,!0))},t.prototype.insertNodeAtEndOfScope=function(n,t,r){var a=u(n,t.getLastToken(),{},o.Start);this.replaceRange(n,{pos:a,end:a},r,{prefix:e.isLineBreak(n.text.charCodeAt(t.getLastToken().pos))?this.newLineCharacter:this.newLineCharacter+this.newLineCharacter,suffix:this.newLineCharacter})},t.prototype.insertNodeAtClassStart=function(e,n,t){this.insertNodeAtStartWorker(e,n,t)},t.prototype.insertNodeAtObjectStart=function(e,n,t){this.insertNodeAtStartWorker(e,n,t)},t.prototype.insertNodeAtStartWorker=function(n,t,r){var a=t.getStart(n),i=e.formatting.SmartIndenter.findFirstNonWhitespaceColumn(e.getLineStartPositionForPosition(a,n),a,n,this.formatContext.options)+this.formatContext.options.indentSize;this.insertNodeAt(n,v(t).pos,r,yn({indentation:i},this.getInsertNodeAtStartPrefixSuffix(n,t)))},t.prototype.getInsertNodeAtStartPrefixSuffix=function(n,t){var r=e.isObjectLiteralExpression(t)?\",\":\"\";if(0===v(t).length){if(e.addToSeen(this.classesWithNodesInsertedAtStart,e.getNodeId(t),{node:t,sourceFile:n})){var a=e.positionsAreOnSameLine.apply(void 0,_(t,n).concat([n]));return{prefix:this.newLineCharacter,suffix:r+(a?this.newLineCharacter:\"\")}}return{prefix:\"\",suffix:r+this.newLineCharacter}}return{prefix:this.newLineCharacter,suffix:r}},t.prototype.insertNodeAfterComma=function(e,n,t){var r=this.insertNodeAfterWorker(e,this.nextCommaToken(e,n)||n,t);this.insertNodeAt(e,r,t,this.getInsertNodeAfterOptions(e,n))},t.prototype.insertNodeAfter=function(e,n,t){var r=this.insertNodeAfterWorker(e,n,t);this.insertNodeAt(e,r,t,this.getInsertNodeAfterOptions(e,n))},t.prototype.insertNodeAtEndOfList=function(e,n,t){this.insertNodeAt(e,n.end,t,{prefix:\", \"})},t.prototype.insertNodesAfter=function(n,t,r){var a=this.insertNodeAfterWorker(n,t,e.first(r));this.insertNodesAt(n,a,r,this.getInsertNodeAfterOptions(n,t))},t.prototype.insertNodeAfterWorker=function(n,t,r){var a,i;return a=t,i=r,((e.isPropertySignature(a)||e.isPropertyDeclaration(a))&&e.isClassOrTypeElement(i)&&147===i.name.kind||e.isStatementButNotDeclaration(a)&&e.isStatementButNotDeclaration(i))&&59!==n.text.charCodeAt(t.end-1)&&this.replaceRange(n,e.createRange(t.end),e.createToken(25)),d(n,t,{})},t.prototype.getInsertNodeAfterOptions=function(n,t){var r=this.getInsertNodeAfterOptionsWorker(t);return yn({},r,{prefix:t.end===n.end&&e.isStatement(t)?r.prefix?\"\\n\"+r.prefix:\"\\n\":r.prefix})},t.prototype.getInsertNodeAfterOptionsWorker=function(n){switch(n.kind){case 238:case 242:return{prefix:this.newLineCharacter,suffix:this.newLineCharacter};case 235:case 9:case 71:return{prefix:\", \"};case 273:return{suffix:\",\"+this.newLineCharacter};case 84:return{prefix:\" \"};case 149:return{};default:return e.Debug.assert(e.isStatement(n)||e.isClassOrTypeElement(n)),{suffix:this.newLineCharacter}}},t.prototype.insertName=function(n,t,r){if(e.Debug.assert(!t.name),195===t.kind){var a=e.findChildOfKind(t,36,n),i=e.findChildOfKind(t,19,n);i?(this.insertNodesAt(n,i.getStart(n),[e.createToken(89),e.createIdentifier(r)],{joiner:\" \"}),L(this,n,a)):(this.insertText(n,e.first(t.parameters).getStart(n),\"function \"+r+\"(\"),this.replaceRange(n,a,e.createToken(20))),216!==t.body.kind&&(this.insertNodesAt(n,t.body.getStart(n),[e.createToken(17),e.createToken(96)],{joiner:\" \",suffix:\" \"}),this.insertNodesAt(n,t.body.end,[e.createToken(25),e.createToken(18)],{joiner:\" \"}))}else{var o=e.findChildOfKind(t,194===t.kind?89:75,n).end;this.insertNodeAt(n,o,e.createIdentifier(r),{prefix:\" \"})}},t.prototype.insertExportModifier=function(e,n){this.insertText(e,n.getStart(e),\"export \")},t.prototype.insertNodeInListAfter=function(n,t,r,a){if(void 0===a&&(a=e.formatting.SmartIndenter.getContainingList(t,n)),!a)return e.Debug.fail(\"node is not a list element\"),this;var i=e.indexOfNode(a,t);if(i<0)return this;var o=t.getEnd();if(i!==a.length-1){var s=e.getTokenAtPosition(n,t.end);if(s&&m(t,s)){var c=e.getLineAndCharacterOfPosition(n,l(n.text,a[i+1].getFullStart())),u=e.getLineAndCharacterOfPosition(n,s.end),d=void 0,p=void 0;u.line===c.line?(p=s.end,d=function(e){for(var n=\"\",t=0;t<e;t++)n+=\" \";return n}(c.character-u.character)):p=e.getStartPositionOfLine(c.line,n);var f=\"\"+e.tokenToString(s.kind)+n.text.substring(s.end,a[i+1].getStart(n));this.replaceRange(n,e.createRange(p,a[i+1].getStart(n)),r,{prefix:d,suffix:f})}}else{var g=t.getStart(n),_=e.getLineStartPositionForPosition(g,n),v=void 0,y=!1;if(1===a.length)v=26;else{var h=e.findPrecedingToken(t.pos,n);v=m(t,h)?h.kind:26,y=e.getLineStartPositionForPosition(a[i-1].getStart(n),n)!==_}if(function(n,t){for(var r=t;r<n.length;){var a=n.charCodeAt(r);if(!e.isWhiteSpaceSingleLine(a))return 47===a;r++}return!1}(n.text,t.end)&&(y=!0),y){this.replaceRange(n,e.createRange(o),e.createToken(v));var b=e.formatting.SmartIndenter.findFirstNonWhitespaceColumn(_,g,n,this.formatContext.options),E=e.skipTrivia(n.text,o,!0,!1);E!==o&&e.isLineBreak(n.text.charCodeAt(E-1))&&E--,this.replaceRange(n,e.createRange(E),r,{indentation:b,prefix:this.newLineCharacter})}else this.replaceRange(n,e.createRange(o),r,{prefix:e.tokenToString(v)+\" \"})}return this},t.prototype.finishClassesWithNodesInsertedAtStart=function(){var n=this;this.classesWithNodesInsertedAtStart.forEach(function(t){var r=t.node,a=t.sourceFile,i=_(r,a),o=i[0],s=i[1];e.positionsAreOnSameLine(o,s,a)&&o!==s-1&&n.deleteRange(a,e.createRange(o,s-1))})},t.prototype.finishDeleteDeclarations=function(){for(var n=this,t=new e.NodeSet,r=function(n,r){a.deletedNodes.some(function(t){return t.sourceFile===n&&e.rangeContainsRangeExclusive(t.node,r)})||(e.isArray(r)?a.deleteRange(n,e.rangeOfTypeParameters(r)):E.deleteDeclaration(a,t,n,r))},a=this,i=0,o=this.deletedNodes;i<o.length;i++){var s=o[i];r(s.sourceFile,s.node)}t.forEach(function(r){var a=r.getSourceFile(),i=e.formatting.SmartIndenter.getContainingList(r,a);if(r===e.last(i)){var o=e.findLastIndex(i,function(e){return!t.has(e)},i.length-2);-1!==o&&n.deleteRange(a,{pos:i[o].end,end:g(a,i[o+1])})}})},t.prototype.getChanges=function(e){this.finishDeleteDeclarations(),this.finishClassesWithNodesInsertedAtStart();for(var n=p.getTextChangesFromChanges(this.changes,this.newLineCharacter,this.formatContext,e),t=0,r=this.newFiles;t<r.length;t++){var a=r[t],i=a.oldFile,o=a.fileName,s=a.statements;n.push(p.newFileChanges(i,o,s,this.newLineCharacter,this.formatContext))}return n},t.prototype.createNewFile=function(e,n,t){this.newFiles.push({oldFile:e,fileName:n,statements:t})},t}();function g(n,t){return e.skipTrivia(n.text,u(n,t,{},o.FullStart),!1,!0)}function _(n,t){return[e.findChildOfKind(n,17,t).end,e.findChildOfKind(n,18,t).end]}function v(n){return e.isObjectLiteralExpression(n)?n.properties:n.members}function y(n,t){for(var r=t.length-1;r>=0;r--){var a=t[r],i=a.span,o=a.newText;n=\"\"+n.substring(0,i.start)+o+n.substring(e.textSpanEnd(i))}return n}function h(n){var r=e.visitEachChild(n,h,e.nullTransformationContext,b,h),i=e.nodeIsSynthesized(r)?r:Object.create(r);return i.pos=t(n),i.end=a(n),i}function b(n,r,i,o,s){var l=e.visitNodes(n,r,i,o,s);if(!l)return l;var c=l===n?e.createNodeArray(l.slice(0)):l;return c.pos=t(n),c.end=a(n),c}n.ChangeTracker=f,n.getNewFileText=function(e,n,t,r){return p.newFileChangesWorker(void 0,n,e,t,r)},function(n){function t(n,t,a,i,o){var s=a.map(function(e){return r(e,n,i).text}).join(i),l=e.createSourceFile(\"any file name\",s,6,!0,t);return y(s,e.formatting.formatDocument(l,o))}function r(n,t,r){var a=new T(r),i=\"\\n\"===r?1:0;return e.createPrinter({newLine:i,neverAsciiEscape:!0},a).writeNode(4,n,t,a),{text:a.getText(),node:h(n)}}n.getTextChangesFromChanges=function(n,t,a,i){return e.group(n,function(e){return e.sourceFile.path}).map(function(n){for(var o=n[0].sourceFile,l=e.stableSort(n,function(e,n){return e.range.pos-n.range.pos||e.range.end-n.range.end}),c=function(n){e.Debug.assert(l[n].range.end<=l[n+1].range.pos,\"Changes overlap\",function(){return JSON.stringify(l[n].range)+\" and \"+JSON.stringify(l[n+1].range)})},u=0;u<l.length-1;u++)c(u);var d=l.map(function(n){return e.createTextChange(e.createTextSpanFromRange(n.range),function(n,t,a,i,o){if(n.kind===s.Remove)return\"\";if(n.kind===s.Text)return n.text;var l=n.options,c=void 0===l?{}:l,u=n.range.pos,d=function(n){return function(n,t,a,i,o,s,l){var c=i.indentation,u=i.prefix,d=i.delta,m=r(n,t,o),p=m.node,f=m.text;l&&l(p,f);var g=s.options,_=void 0!==c?c:e.formatting.SmartIndenter.getIndentation(a,t,g,u===o||e.getLineStartPositionForPosition(a,t)===a);void 0===d&&(d=e.formatting.SmartIndenter.shouldIndentChildNode(s.options,n)&&g.indentSize||0);var v={text:f,getLineAndCharacterOfPosition:function(n){return e.getLineAndCharacterOfPosition(this,n)}},h=e.formatting.formatNodeGivenIndentation(p,v,t.languageVariant,_,d,s);return y(f,h)}(n,t,u,c,a,i,o)},m=n.kind===s.ReplaceWithMultipleNodes?n.nodes.map(function(n){return e.removeSuffix(d(n),a)}).join(n.options.joiner||a):d(n.node),p=c.preserveLeadingWhitespace||void 0!==c.indentation||e.getLineStartPositionForPosition(u,t)===u?m:m.replace(/^\\s+/,\"\");return(c.prefix||\"\")+p+(c.suffix||\"\")}(n,o,t,a,i))});return{fileName:o.fileName,textChanges:d}})},n.newFileChanges=function(n,r,a,i,o){var s=t(n,e.getScriptKindFromFileName(r),a,i,o);return{fileName:r,textChanges:[e.createTextChange(e.createTextSpan(0,0),s)],isNewFile:!0}},n.newFileChangesWorker=t}(p||(p={})),n.applyChanges=y;var E,T=function(){function n(n){var t=this;this.lastNonTriviaPosition=0,this.writer=e.createTextWriter(n),this.onEmitNode=function(e,n,a){n&&r(n,t.lastNonTriviaPosition),a(e,n),n&&i(n,t.lastNonTriviaPosition)},this.onBeforeEmitNodeArray=function(e){e&&r(e,t.lastNonTriviaPosition)},this.onAfterEmitNodeArray=function(e){e&&i(e,t.lastNonTriviaPosition)},this.onBeforeEmitToken=function(e){e&&r(e,t.lastNonTriviaPosition)},this.onAfterEmitToken=function(e){e&&i(e,t.lastNonTriviaPosition)}}return n.prototype.setLastNonTriviaPosition=function(n,t){if(t||!function(n){return e.skipTrivia(n,0)===n.length}(n)){this.lastNonTriviaPosition=this.writer.getTextPos();for(var r=0;e.isWhiteSpaceLike(n.charCodeAt(n.length-r-1));)r++;this.lastNonTriviaPosition-=r}},n.prototype.write=function(e){this.writer.write(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeKeyword=function(e){this.writer.writeKeyword(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeOperator=function(e){this.writer.writeOperator(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writePunctuation=function(e){this.writer.writePunctuation(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeParameter=function(e){this.writer.writeParameter(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeProperty=function(e){this.writer.writeProperty(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeSpace=function(e){this.writer.writeSpace(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeStringLiteral=function(e){this.writer.writeStringLiteral(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeSymbol=function(e,n){this.writer.writeSymbol(e,n),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeTextOfNode=function(e,n){this.writer.writeTextOfNode(e,n)},n.prototype.writeLine=function(){this.writer.writeLine()},n.prototype.increaseIndent=function(){this.writer.increaseIndent()},n.prototype.decreaseIndent=function(){this.writer.decreaseIndent()},n.prototype.getText=function(){return this.writer.getText()},n.prototype.rawWrite=function(e){this.writer.rawWrite(e),this.setLastNonTriviaPosition(e,!1)},n.prototype.writeLiteral=function(e){this.writer.writeLiteral(e),this.setLastNonTriviaPosition(e,!0)},n.prototype.getTextPos=function(){return this.writer.getTextPos()},n.prototype.getLine=function(){return this.writer.getLine()},n.prototype.getColumn=function(){return this.writer.getColumn()},n.prototype.getIndent=function(){return this.writer.getIndent()},n.prototype.isAtStartOfLine=function(){return this.writer.isAtStartOfLine()},n.prototype.clear=function(){this.writer.clear(),this.lastNonTriviaPosition=0},n}();function S(n,t){return!e.isInComment(n,t)&&!e.isInString(n,t)&&!e.isInTemplateString(n,t)}function L(e,n,t,r){void 0===r&&(r={});var a=u(n,t,r,o.FullStart),i=d(n,t,r);e.deleteRange(n,{pos:a,end:i})}function x(n,t,r,a){var i=e.Debug.assertDefined(e.formatting.SmartIndenter.getContainingList(a,r)),o=e.indexOfNode(i,a);e.Debug.assert(-1!==o),1!==i.length?(e.Debug.assert(!t.has(a),\"Deleting a node twice\"),t.add(a),n.deleteRange(r,{pos:g(r,a),end:o===i.length-1?d(r,a,{}):g(r,i[o+1])})):L(n,r,a)}n.isValidLocationToAddComment=S,function(n){function t(n,t,r){if(r.parent.name){var a=e.Debug.assertDefined(e.getTokenAtPosition(t,r.pos-1));n.deleteRange(t,{pos:a.getStart(t),end:r.end})}else{L(n,t,e.getAncestor(r,247))}}n.deleteDeclaration=function(n,r,a,i){switch(i.kind){case 149:var o=i.parent;if(e.isArrowFunction(o)&&1===o.parameters.length){var s=e.updateArrowFunction(o,o.modifiers,o.typeParameters,void 0,o.type,o.equalsGreaterThanToken,o.body);e.suppressLeadingAndTrailingTrivia(s),n.replaceNode(a,o,s)}else x(n,r,a,i);break;case 247:L(n,a,i,i===a.imports[0].parent?{useNonAdjustedStartPosition:!0,useNonAdjustedEndPosition:!1}:void 0);break;case 184:var l=i.parent;183===l.kind&&i!==e.last(l.elements)?L(n,a,i):x(n,r,a,i);break;case 235:!function(n,t,r,a){var i=a.parent;if(272!==i.kind)if(1===i.declarations.length){var o=i.parent;switch(o.kind){case 225:case 224:n.replaceNode(r,a,e.createObjectLiteral());break;case 223:L(n,r,i);break;case 217:L(n,r,o);break;default:e.Debug.assertNever(o)}}else x(n,t,r,a);else n.deleteNodeRange(r,e.findChildOfKind(i,19,r),e.findChildOfKind(i,20,r))}(n,r,a,i);break;case 148:x(n,r,a,i);break;case 251:var c=i.parent;1===c.elements.length?t(n,a,c):x(n,r,a,i);break;case 249:t(n,a,i);break;default:e.isImportClause(i.parent)&&i.parent.name===i?function(n,t,r){if(r.namedBindings){var a=r.name.getStart(t),i=e.getTokenAtPosition(t,r.name.end);if(i&&26===i.kind){var o=e.skipTrivia(t.text,i.end,!1,!0);n.deleteRange(t,{pos:a,end:o})}else L(n,t,r.name)}else L(n,t,r.parent)}(n,a,i.parent):e.isCallLikeExpression(i.parent)?x(n,r,a,i):L(n,a,i,25===i.kind?{useNonAdjustedEndPosition:!0}:void 0)}}}(E||(E={})),n.deleteNode=L}(e.textChanges||(e.textChanges={}))}(Tn||(Tn={})),function(e){!function(n){var t=e.createMultiMap(),r=e.createMap();function a(n){return e.isArray(n)?e.formatStringFromArgs(e.getLocaleSpecificMessage(n[0]),n.slice(1)):e.getLocaleSpecificMessage(n)}function i(e,n,t,r,a,i){return{fixName:e,description:n,changes:t,fixId:r,fixAllDescription:a,commands:i?[i]:void 0}}function o(e,n){return{changes:e,commands:n}}function s(n,t,r){for(var a=n.program,i=n.sourceFile,o=n.cancellationToken,s=0,l=a.getSemanticDiagnostics(i,o).concat(e.computeSuggestionDiagnostics(i,a,o));s<l.length;s++){var c=l[s];e.contains(t,c.code)&&r(c)}}n.createCodeFixActionNoFixId=function(e,n,t){return i(e,a(t),n,void 0,void 0)},n.createCodeFixAction=function(e,n,t,r,o,s){return i(e,a(t),n,r,a(o),s)},n.registerCodeFix=function(n){for(var a=0,i=n.errorCodes;a<i.length;a++){var o=i[a];t.add(String(o),n)}if(n.fixIds)for(var s=0,l=n.fixIds;s<l.length;s++){var c=l[s];e.Debug.assert(!r.has(c)),r.set(c,n)}},n.getSupportedErrorCodes=function(){return e.arrayFrom(t.keys())},n.getFixes=function(n){return e.flatMap(t.get(String(n.errorCode))||e.emptyArray,function(e){return e.getCodeActions(n)})},n.getAllFixes=function(n){return r.get(e.cast(n.fixId,e.isString)).getAllCodeActions(n)},n.createCombinedCodeActions=o,n.createFileTextChanges=function(e,n){return{fileName:e,textChanges:n}},n.codeFixAll=function(n,t,r){var a=[];return o(e.textChanges.ChangeTracker.with(n,function(e){return s(n,t,function(n){return r(e,n,a)})}),0===a.length?void 0:a)},n.eachDiagnostic=s}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=e.createMap();n.registerRefactor=function(e,n){t.set(e,n)},n.getApplicableRefactors=function(n){return e.arrayFrom(e.flatMapIterator(t.values(),function(e){return n.cancellationToken&&n.cancellationToken.isCancellationRequested()?void 0:e.getAvailableActions(n)}))},n.getEditsForRefactor=function(e,n,r){var a=t.get(n);return a&&a.getEditsForAction(e,r)}}(e.refactor||(e.refactor={})),e.getRefactorContextSpan=function(n){var t=n.startPosition,r=n.endPosition;return e.createTextSpanFromBounds(t,void 0===r?t:r)}}(Tn||(Tn={})),function(e){!function(n){var t=\"addMissingInvocationForDecorator\",r=[e.Diagnostics._0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0.code];function a(n,t,r){var a=e.getTokenAtPosition(t,r),i=e.findAncestor(a,e.isDecorator);e.Debug.assert(!!i,\"Expected position to be owned by a decorator.\");var o=e.createCall(i.expression,void 0,void 0);n.replaceNode(t,i.expression,o)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=e.textChanges.ChangeTracker.with(r,function(e){return a(e,r.sourceFile,r.span.start)});return[n.createCodeFixAction(t,i,e.Diagnostics.Call_decorator_expression,t,e.Diagnostics.Add_to_all_uncalled_decorators)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return a(e,n.file,n.start)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"annotateWithTypeFromJSDoc\",r=[e.Diagnostics.JSDoc_types_may_be_moved_to_TypeScript_types.code];function a(n,t){var r=e.getTokenAtPosition(n,t);return e.tryCast(e.isParameter(r.parent)?r.parent.parent:r.parent,i)}function i(n){return function(n){return e.isFunctionLikeDeclaration(n)||235===n.kind||151===n.kind||152===n.kind}(n)&&o(n)}function o(n){return e.isFunctionLikeDeclaration(n)?n.parameters.some(o)||!n.type&&!!e.getJSDocReturnType(n):!n.type&&!!e.getJSDocType(n)}function s(n,t,r){if(e.isFunctionLikeDeclaration(r)&&(e.getJSDocReturnType(r)||r.parameters.some(function(n){return!!e.getJSDocType(n)}))){if(!r.typeParameters){var a=e.getJSDocTypeParameterDeclarations(r);a.length&&n.insertTypeParameters(t,r,a)}var i=e.isArrowFunction(r)&&!e.findChildOfKind(r,19,t);i&&n.insertNodeBefore(t,e.first(r.parameters),e.createToken(19));for(var o=0,s=r.parameters;o<s.length;o++){var c=s[o];if(!c.type){var u=e.getJSDocType(c);u&&n.tryInsertTypeAnnotation(t,c,l(u))}}if(i&&n.insertNodeAfter(t,e.last(r.parameters),e.createToken(20)),!r.type){var d=e.getJSDocReturnType(r);d&&n.tryInsertTypeAnnotation(t,r,l(d))}}else{var m=e.Debug.assertDefined(e.getJSDocType(r));e.Debug.assert(!r.type),n.tryInsertTypeAnnotation(t,r,l(m))}}function l(n){switch(n.kind){case 282:case 283:return e.createTypeReferenceNode(\"any\",e.emptyArray);case 286:return function(n){return e.createUnionTypeNode([e.visitNode(n.type,l),e.createTypeReferenceNode(\"undefined\",e.emptyArray)])}(n);case 285:return l(n.type);case 284:return function(n){return e.createUnionTypeNode([e.visitNode(n.type,l),e.createTypeReferenceNode(\"null\",e.emptyArray)])}(n);case 288:return function(n){return e.createArrayTypeNode(e.visitNode(n.type,l))}(n);case 287:return function(n){return e.createFunctionTypeNode(e.emptyArray,n.parameters.map(c),n.type)}(n);case 162:return function(n){var t=n.typeName,r=n.typeArguments;if(e.isIdentifier(n.typeName)){if(e.isJSDocIndexSignature(n))return function(n){var t=e.createParameter(void 0,void 0,void 0,134===n.typeArguments[0].kind?\"n\":\"s\",void 0,e.createTypeReferenceNode(134===n.typeArguments[0].kind?\"number\":\"string\",[]),void 0),r=e.createTypeLiteralNode([e.createIndexSignature(void 0,void 0,[t],n.typeArguments[1])]);return e.setEmitFlags(r,1),r}(n);var a=n.typeName.text;switch(n.typeName.text){case\"String\":case\"Boolean\":case\"Object\":case\"Number\":a=a.toLowerCase();break;case\"array\":case\"date\":case\"promise\":a=a[0].toUpperCase()+a.slice(1)}t=e.createIdentifier(a),r=\"Array\"!==a&&\"Promise\"!==a||n.typeArguments?e.visitNodes(n.typeArguments,l):e.createNodeArray([e.createTypeReferenceNode(\"any\",e.emptyArray)])}return e.createTypeReferenceNode(t,r)}(n);default:var t=e.visitEachChild(n,l,void 0);return e.setEmitFlags(t,1),t}}function c(n){var t=n.parent.parameters.indexOf(n),r=288===n.type.kind&&t===n.parent.parameters.length-1,a=n.name||(r?\"rest\":\"arg\"+t),i=r?e.createToken(24):n.dotDotDotToken;return e.createParameter(n.decorators,n.modifiers,i,a,n.questionToken,e.visitNode(n.type,l),n.initializer)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=a(r.sourceFile,r.span.start);if(i){var o=e.textChanges.ChangeTracker.with(r,function(e){return s(e,r.sourceFile,i)});return[n.createCodeFixAction(t,o,e.Diagnostics.Annotate_with_type_from_JSDoc,t,e.Diagnostics.Annotate_everything_with_types_from_JSDoc)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=a(n.file,n.start);t&&s(e,n.file,t)})}}),n.parameterShouldGetTypeFromJSDoc=i}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"convertFunctionToEs6Class\",r=[e.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration.code];function a(n,t,r,a){var o=a.getSymbolAtLocation(e.getTokenAtPosition(t,r));if(o&&19&o.flags){var s,l,c=o.valueDeclaration;switch(c.kind){case 237:s=c,n.delete(t,c),l=function(n){var t=u(o);n.body&&t.unshift(e.createConstructor(void 0,void 0,n.parameters,n.body));var r=i(n,84);return e.createClassDeclaration(void 0,r,n.name,void 0,void 0,t)}(c);break;case 235:s=c.parent.parent,l=function(n){var t=n.initializer;if(!t||194!==t.kind)return;if(71!==n.name.kind)return;var r=u(n.symbol);t.body&&r.unshift(e.createConstructor(void 0,void 0,t.parameters,t.body));var a=i(s,84);return e.createClassDeclaration(void 0,a,n.name,void 0,void 0,r)}(c),1===c.parent.declarations.length?(e.copyComments(s,l,t),n.delete(t,s)):n.delete(t,c)}l&&(e.copyComments(c,l,t),n.insertNodeAfter(t,s,l))}function u(r){var a=[];return r.members&&r.members.forEach(function(e){var n=o(e,void 0);n&&a.push(n)}),r.exports&&r.exports.forEach(function(n){var t=o(n,[e.createToken(115)]);t&&a.push(t)}),a;function o(r,a){if(8192&r.flags){var o=r.valueDeclaration,s=o.parent;if(l=s.right,e.isFunctionLike(l)){var l,c=s.parent&&219===s.parent.kind?s.parent:s;if(n.delete(t,c),!s.right)return e.createProperty([],a,r.name,void 0,void 0,void 0);switch(s.right.kind){case 194:var u=s.right,d=e.concatenate(a,i(u,120)),m=e.createMethod(void 0,d,void 0,o.name,void 0,void 0,u.parameters,void 0,u.body);return e.copyComments(s,m,t),m;case 195:var p=s.right,f=p.body,g=void 0;g=216===f.kind?f:e.createBlock([e.createReturn(f)]);d=e.concatenate(a,i(p,120)),m=e.createMethod(void 0,d,void 0,o.name,void 0,void 0,p.parameters,void 0,g);return e.copyComments(s,m,t),m;default:if(e.isSourceFileJS(t))return;var _=e.createProperty(void 0,a,o.name,void 0,void 0,s.right);return e.copyComments(s.parent,_,t),_}}}}}}function i(n,t){return e.filter(n.modifiers,function(e){return e.kind===t})}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=e.textChanges.ChangeTracker.with(r,function(e){return a(e,r.sourceFile,r.span.start,r.program.getTypeChecker())});return[n.createCodeFixAction(t,i,e.Diagnostics.Convert_function_to_an_ES2015_class,t,e.Diagnostics.Convert_all_constructor_functions_to_classes)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return a(n,t.file,t.start,e.program.getTypeChecker())})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"convertToAsyncFunction\",r=[e.Diagnostics.This_may_be_converted_to_an_async_function.code],a=!0;function i(n,t,r,a,i){var u,d=e.getTokenAtPosition(t,r);if(u=e.isIdentifier(d)&&e.isVariableDeclaration(d.parent)&&d.parent.initializer&&e.isFunctionLikeDeclaration(d.parent.initializer)?d.parent.initializer:e.tryCast(e.getContainingFunction(e.getTokenAtPosition(t,r)),e.isFunctionLikeDeclaration)){var m=e.createMap(),p=e.createMap(),g=[],_=e.isInJSFile(u),v=function(n,t){if(!n.body)return e.createMap();var r=e.createMap();return e.forEachChild(n.body,function n(a){o(a,t,\"then\")?(r.set(e.getNodeId(a).toString(),!0),e.forEach(a.arguments,n)):o(a,t,\"catch\")?(r.set(e.getNodeId(a).toString(),!0),e.forEachChild(a,n)):o(a,t)?r.set(e.getNodeId(a).toString(),!0):e.forEachChild(a,n)}),r}(u,a),y=function(n,t,r,a,i,o,c){var u=e.createMap(),d=e.createMap();return e.forEachChild(n,function n(o){if(e.isIdentifier(o)){var m=t.getSymbolAtLocation(o),p=m&&function(e,n){return e.valueDeclaration&&e.valueDeclaration.getSourceFile()===n}(m,a.sourceFile);if(m&&p){var g=f(t.getTypeAtLocation(o),t),_=e.getSymbolId(m).toString();if(!g||e.isFunctionLikeDeclaration(o.parent)||r.has(_)){if(o.parent&&(e.isParameter(o.parent)||e.isVariableDeclaration(o.parent))){var v=o.text,y=d.get(v);if(y&&y.some(function(e){return e!==m})){var h=l(o,d);u.set(_,h.identifier),r.set(_,h),c.push({identifier:h.identifier,symbol:m}),s(d,v,m)}else{var b=e.getSynthesizedDeepClone(o);u.set(_,b),r.set(_,{identifier:b,types:[],numberOfAssignmentsOriginal:c.filter(function(e){return e.identifier.text===o.text}).length}),(e.isParameter(o.parent)&&function(n){var r=n.parent;if(e.isCallExpression(r)||e.isIdentifier(r)&&!i.get(e.getNodeId(r).toString())){var a=t.getTypeAtLocation(r),o=a&&t.getPromisedTypeOfPromise(a);return!!o}return!1}(o.parent.parent)||e.isVariableDeclaration(o.parent))&&(c.push({identifier:b,symbol:m}),s(d,v,m))}}}else{var E=e.firstOrUndefined(g.parameters),T=E&&e.isParameter(E.valueDeclaration)&&e.tryCast(E.valueDeclaration.name,e.isIdentifier)||e.createOptimisticUniqueName(\"result\"),S=l(T,d);r.set(_,S),c.push({identifier:S.identifier,symbol:m}),s(d,T.text,m)}}}else e.forEachChild(o,n)}),e.getSynthesizedDeepCloneWithRenames(n,!0,u,t,function(n,a){if(e.isIdentifier(n)){var s=t.getSymbolAtLocation(n),l=s&&e.getSymbolId(s).toString(),c=s&&r.get(l);if(c){var u=t.getTypeAtLocation(n);o.set(e.getNodeId(a).toString(),u)}}var d=i.get(e.getNodeId(n).toString());void 0!==d&&(i.delete(e.getNodeId(n).toString()),i.set(e.getNodeId(a).toString(),d))})}(u,a,m,i,v,p,g),h=function(e){var n=[];return e.forEach(function(e){0===e.numberOfAssignmentsOriginal&&n.push(e.identifier)}),n}(m),b=e.getReturnStatementsWithPromiseHandlers(y),E={checker:a,synthNamesMap:m,allVarNames:g,setOfExpressionsToReturn:v,constIdentifiers:h,originalTypeMap:p,isInJSFile:_};if(b.length){n.insertLastModifierBefore(t,120,u);for(var T=function(r){e.forEachChild(r,function a(i){e.isCallExpression(i)?function(e,r){var a=c(e,E,e);n.replaceNodeWithNodes(t,r,a)}(i,r):e.isFunctionLike(i)||e.forEachChild(i,a)})},S=0,L=b;S<L.length;S++){T(L[S])}}}}function o(n,t,r){var a=(r?e.isCallExpression(n):e.isExpression(n))&&(!r||e.hasPropertyAccessExpressionWithName(n,r))&&t.getTypeAtLocation(n);return!(!a||!t.getPromisedTypeOfPromise(a))}function s(e,n,t){e.has(n)?e.get(n).push(t):e.set(n,[t])}function l(n,t){var r=(t.get(n.text)||e.emptyArray).length;return{identifier:0===r?n:e.createIdentifier(n.text+\"_\"+r),types:[],numberOfAssignmentsOriginal:0}}function c(n,t,r,i){if(!n)return e.emptyArray;var o=e.isIdentifier(n)&&t.originalTypeMap.get(e.getNodeId(n).toString())||t.checker.getTypeAtLocation(n);return e.isCallExpression(n)&&e.hasPropertyAccessExpressionWithName(n,\"then\")&&o&&t.checker.getPromisedTypeOfPromise(o)?function(n,t,r,a){var i=n.arguments,o=i[0],s=i[1];if(!o)return c(n.expression,t,r);var l=_(o,t),u=p(o,a,l,n,t);if(s){var d=_(s,t),m=e.createBlock(c(n.expression,t,n,l).concat(u)),f=p(s,a,d,n,t),g=d?d.identifier.text:\"e\",v=e.createCatchClause(g,e.createBlock(f));return[e.createTry(m,v,void 0)]}return c(n.expression,t,n,l).concat(u)}(n,t,r,i):e.isCallExpression(n)&&e.hasPropertyAccessExpressionWithName(n,\"catch\")&&o&&t.checker.getPromisedTypeOfPromise(o)?function(n,t,r){var a=n.arguments[0],i=_(a,t),o=t.setOfExpressionsToReturn.get(e.getNodeId(n).toString());r&&!o&&(r.numberOfAssignmentsOriginal=2,t.synthNamesMap.forEach(function(e,n){if(e.identifier.text===r.identifier.text){var a=u(r);t.synthNamesMap.set(n,a)}}),t.constIdentifiers.some(function(e){return e.text===r.identifier.text})&&t.constIdentifiers.push(u(r).identifier));var s,l=e.createBlock(c(n.expression,t,n,r)),d=p(a,r,i,n,t),m=i?i.identifier.text:\"e\",f=e.createCatchClause(m,e.createBlock(d));if(r&&!o){var g=r.types,v=t.checker.getUnionType(g,2),y=t.isInJSFile?void 0:t.checker.typeToTypeNode(v),h=[e.createVariableDeclaration(e.getSynthesizedDeepClone(r.identifier),y)];s=e.createVariableStatement(void 0,e.createVariableDeclarationList(h,1))}var b=e.createTry(l,f,void 0);return s?[s,b]:[b]}(n,t,i):e.isPropertyAccessExpression(n)?c(n.expression,t,r,i):o&&t.checker.getPromisedTypeOfPromise(o)?function(n,t,r){var a=t.setOfExpressionsToReturn.get(e.getNodeId(n).toString()),i=n.original?n.original.parent:n.parent;if(r&&!a&&(!i||e.isPropertyAccessExpression(i)))return m(r,e.createAwait(n),t);if(!r&&!a&&(!i||e.isPropertyAccessExpression(i)))return[e.createStatement(e.createAwait(n))];return[e.createReturn(e.getSynthesizedDeepClone(n))]}(n,t,i):(a=!1,e.emptyArray)}function u(n){return{identifier:e.createOptimisticUniqueName(n.identifier.text),types:[],numberOfAssignmentsOriginal:0}}function d(e,n){return n.some(function(n){return n.text===e.text})?2:1}function m(n,t,r){return n&&0!==n.identifier.text.length?n.types.length<n.numberOfAssignmentsOriginal?[e.createStatement(e.createAssignment(e.getSynthesizedDeepClone(n.identifier),t))]:[e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(e.getSynthesizedDeepClone(n.identifier),void 0,t)],d(n.identifier,r.constIdentifiers)))]:[e.createStatement(t)]}function p(n,t,r,i,s){var l=s.setOfExpressionsToReturn.get(e.getNodeId(i).toString());switch(n.kind){case 95:break;case 71:if(!r)break;var c=e.createCall(e.getSynthesizedDeepClone(n),void 0,r?[r.identifier]:e.emptyArray);if(l)return[e.createReturn(c)];var u=s.originalTypeMap.get(e.getNodeId(n).toString())||s.checker.getTypeAtLocation(n),p=s.checker.getSignaturesOfType(u,0);if(!p.length){a=!1;break}var _=p[0].getReturnType(),v=m(t,e.createAwait(c),s);return t&&t.types.push(_),v;case 194:case 195:var y=n.body;if(e.isBlock(y)){for(var h=[],b=!1,E=0,T=y.statements;E<T.length;E++){var S=T[E];e.isReturnStatement(S)&&(b=!0),e.getReturnStatementsWithPromiseHandlers(S).length?h=h.concat(g(s,[S],t)):h.push(S)}return l?h.map(function(n){return e.getSynthesizedDeepClone(n)}):function(n,t,r,a){for(var i=[],s=0,l=n;s<l.length;s++){var c=l[s];if(e.isReturnStatement(c)){if(c.expression){var u=o(c.expression,r.checker)?e.createAwait(c.expression):c.expression;void 0===t?i.push(e.createExpressionStatement(u)):i.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(t,void 0,u)],d(t,r.constIdentifiers))))}}else i.push(e.getSynthesizedDeepClone(c))}a||void 0===t||i.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(t,void 0,e.createIdentifier(\"undefined\"))],d(t,r.constIdentifiers))));return i}(h,void 0===t?void 0:t.identifier,s,b)}var L=g(s,e.getReturnStatementsWithPromiseHandlers(e.createReturn(y)),t);if(L.length>0)return L;if(l)return[e.createReturn(e.getSynthesizedDeepClone(y))];var x=f(s.checker.getTypeAtLocation(n),s.checker).getReturnType(),A=e.getSynthesizedDeepClone(y),C=m(t,s.checker.getPromisedTypeOfPromise(x)?e.createAwait(A):A,s);return t&&t.types.push(x),C;default:a=!1}return e.emptyArray}function f(n,t){var r=t.getSignaturesOfType(n,0);return e.lastOrUndefined(r)}function g(n,t,r){for(var a=[],i=0,o=t;i<o.length;i++){var s=o[i];e.forEachChild(s,function t(i){if(e.isCallExpression(i)){var o=c(i,n,i,r);if((a=a.concat(o)).length>0)return}else e.isFunctionLike(i)||e.forEachChild(i,t)})}return a}function _(n,t){var r,a=0,i=[];e.isFunctionLikeDeclaration(n)?n.parameters.length>0&&(r=o(n.parameters[0].name)):e.isIdentifier(n)&&(r=o(n));if(r&&\"undefined\"!==r.identifier.text)return r;function o(n){var r,o=function(e){return e.symbol?e.symbol:t.checker.getSymbolAtLocation(e)}((r=n).original?r.original:r);return o&&t.synthNamesMap.get(e.getSymbolId(o).toString())||{identifier:n,types:i,numberOfAssignmentsOriginal:a}}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){a=!0;var o=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start,r.program.getTypeChecker(),r)});return a?[n.createCodeFixAction(t,o,e.Diagnostics.Convert_to_async_function,t,e.Diagnostics.Convert_all_to_async_functions)]:[]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return i(n,t.file,t.start,e.program.getTypeChecker(),e)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){function t(n,t,r,a){for(var i=0,o=n.imports;i<o.length;i++){var s=o[i],l=e.getResolvedModule(n,s.text);if(l&&l.resolvedFileName===t.fileName){var c=e.importFromModuleSpecifier(s);switch(c.kind){case 246:r.replaceNode(n,c,e.makeImport(c.name,void 0,s,a));break;case 189:e.isRequireCall(c,!1)&&r.replaceNode(n,c,e.createPropertyAccess(e.getSynthesizedDeepClone(c),\"default\"))}}}}function r(n,t){n.forEachChild(function r(a){if(e.isPropertyAccessExpression(a)&&e.isExportsOrModuleExportsOrAlias(n,a.expression)){var i=a.parent;t(a,e.isBinaryExpression(i)&&i.left===a&&58===i.operatorToken.kind)}a.forEachChild(r)})}function a(t,r,a,p,f,g,_,v){switch(r.kind){case 217:return function(t,r,a,i,o,l,m){var p=r.declarationList,f=!1,g=e.flatMap(p.declarations,function(r){var g=r.name,_=r.initializer;if(_){if(e.isExportsOrModuleExportsOrAlias(t,_))return f=!0,[];if(e.isRequireCall(_,!0))return f=!0,function(t,r,a,i,o,l,c,m){switch(r.kind){case 182:var p=e.mapAllOrFail(r.elements,function(n){return n.dotDotDotToken||n.initializer||n.propertyName&&!e.isIdentifier(n.propertyName)||!e.isIdentifier(n.name)?void 0:u(n.propertyName&&n.propertyName.text,n.name.text)});if(p)return[e.makeImport(void 0,p,a,m)];case 183:var f=s(n.moduleSpecifierToValidIdentifier(a.text,c),l);return[e.makeImport(e.createIdentifier(f),void 0,a,m),d(void 0,e.getSynthesizedDeepClone(r),e.createIdentifier(f))];case 71:return function(n,t,r,a,i,o,l){for(var c=i.getSymbolAtLocation(t),u=e.createMap(),d=!1,m=0,p=o.original.get(t.text);m<p.length;m++){var f=p[m];if(i.getSymbolAtLocation(f)===c&&f!==t){var g=f.parent;if(e.isPropertyAccessExpression(g)){var _=g.expression,v=g.name.text;e.Debug.assert(_===f);var y=u.get(v);void 0===y&&(y=s(v,o),u.set(v,y)),a.replaceNode(n,g,e.createIdentifier(y))}else d=!0}}var h=0===u.size?void 0:e.arrayFrom(e.mapIterator(u.entries(),function(n){var t=n[0],r=n[1];return e.createImportSpecifier(t===r?void 0:e.createIdentifier(t),e.createIdentifier(r))}));h||(d=!0);return[e.makeImport(d?e.getSynthesizedDeepClone(t):void 0,h,r,l)]}(t,r,a,i,o,l,m);default:return e.Debug.assertNever(r)}}(t,g,_.arguments[0],a,i,o,l,m);if(e.isPropertyAccessExpression(_)&&e.isRequireCall(_.expression,!0))return f=!0,function(n,t,r,a,i){switch(n.kind){case 182:case 183:var o=s(t,a);return[c(o,t,r,i),d(void 0,n,e.createIdentifier(o))];case 71:return[c(n.text,t,r,i)];default:return e.Debug.assertNever(n)}}(g,_.name.text,_.expression.arguments[0],o,m)}return e.createVariableStatement(void 0,e.createVariableDeclarationList([r],p.flags))});f&&a.replaceNodeWithNodes(t,r,g)}(t,r,p,a,f,g,v),!1;case 219:var y=r.expression;switch(y.kind){case 189:return e.isRequireCall(y,!0)&&p.replaceNode(t,r,e.makeImport(void 0,void 0,y.arguments[0],v)),!1;case 202:return 58===y.operatorToken.kind&&function(n,t,r,a,s){var c=r.left,u=r.right;if(!e.isPropertyAccessExpression(c))return!1;if(e.isExportsOrModuleExportsOrAlias(n,c)){if(!e.isExportsOrModuleExportsOrAlias(n,u)){var p=e.isObjectLiteralExpression(u)?(f=u,(g=e.mapAllOrFail(f.properties,function(n){switch(n.kind){case 156:case 157:case 274:case 275:return;case 273:return e.isIdentifier(n.name)?function(n,t){var r=[e.createToken(84)];switch(t.kind){case 194:var a=t.name;if(a&&a.text!==n)return i();case 195:return l(n,r,t);case 207:return function(n,t,r){return e.createClassDeclaration(e.getSynthesizedDeepClones(r.decorators),e.concatenate(t,e.getSynthesizedDeepClones(r.modifiers)),n,e.getSynthesizedDeepClones(r.typeParameters),e.getSynthesizedDeepClones(r.heritageClauses),e.getSynthesizedDeepClones(r.members))}(n,r,t);default:return i()}function i(){return d(r,e.createIdentifier(n),t)}}(n.name.text,n.initializer):void 0;case 154:return e.isIdentifier(n.name)?l(n.name.text,[e.createToken(84)],n):void 0;default:e.Debug.assertNever(n)}}))&&[g,!1]):e.isRequireCall(u,!0)?function(n,t){var r=n.text,a=t.getSymbolAtLocation(n),s=a?a.exports:e.emptyUnderscoreEscapedMap;return s.has(\"export=\")?[[o(r)],!0]:s.has(\"default\")?s.size>1?[[i(r),o(r)],!0]:[[o(r)],!0]:[[i(r)],!1]}(u.arguments[0],t):void 0;return p?(a.replaceNodeWithNodes(n,r.parent,p[0]),p[1]):(a.replaceRangeWithText(n,e.createRange(c.getStart(n),u.pos),\"export default\"),!0)}a.delete(n,r.parent)}else e.isExportsOrModuleExportsOrAlias(n,c.expression)&&function(n,t,r,a){var i=t.left.name.text,o=a.get(i);if(void 0!==o){var s=[d(void 0,o,t.right),m([e.createExportSpecifier(o,i)])];r.replaceNodeWithNodes(n,t.parent,s)}else!function(n,t,r){var a=n.left,i=n.right,o=n.parent,s=a.name.text;if(!(e.isFunctionExpression(i)||e.isArrowFunction(i)||e.isClassExpression(i))||i.name&&i.name.text!==s)r.replaceNodeRangeWithNodes(t,a.expression,e.findChildOfKind(a,23,t),[e.createToken(84),e.createToken(76)],{joiner:\" \",suffix:\" \"});else{r.replaceRange(t,{pos:a.getStart(t),end:i.getStart(t)},e.createToken(84),{suffix:\" \"}),i.name||r.insertName(t,i,s);var l=e.findChildOfKind(o,25,t);l&&r.delete(t,l)}}(t,n,r)}(n,r,a,s);var f,g;return!1}(t,a,y,p,_)}default:return!1}}function i(e){return m(void 0,e)}function o(n){return m([e.createExportSpecifier(void 0,\"default\")],n)}function s(e,n){for(;n.original.has(e)||n.additional.has(e);)e=\"_\"+e;return n.additional.set(e,!0),e}function l(n,t,r){return e.createFunctionDeclaration(e.getSynthesizedDeepClones(r.decorators),e.concatenate(t,e.getSynthesizedDeepClones(r.modifiers)),e.getSynthesizedDeepClone(r.asteriskToken),n,e.getSynthesizedDeepClones(r.typeParameters),e.getSynthesizedDeepClones(r.parameters),e.getSynthesizedDeepClone(r.type),e.convertToFunctionBody(e.getSynthesizedDeepClone(r.body)))}function c(n,t,r,a){return\"default\"===t?e.makeImport(e.createIdentifier(n),void 0,r,a):e.makeImport(void 0,[u(t,n)],r,a)}function u(n,t){return e.createImportSpecifier(void 0!==n&&n!==t?e.createIdentifier(n):void 0,e.createIdentifier(t))}function d(n,t,r){return e.createVariableStatement(n,e.createVariableDeclarationList([e.createVariableDeclaration(t,void 0,r)],2))}function m(n,t){return e.createExportDeclaration(void 0,void 0,n&&e.createNamedExports(n),void 0===t?void 0:e.createLiteral(t))}n.registerCodeFix({errorCodes:[e.Diagnostics.File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module.code],getCodeActions:function(i){var o=i.sourceFile,l=i.program,c=i.preferences,u=e.textChanges.ChangeTracker.with(i,function(n){if(function(n,t,i,o,l){var c={original:(d=n,m=e.createMultiMap(),function n(t,r){e.isIdentifier(t)&&function(e){var n=e.parent;switch(n.kind){case 187:return n.name!==e;case 184:case 251:return n.propertyName!==e;default:return!0}}(t)&&r(t),t.forEachChild(function(e){return n(e,r)})}(d,function(e){return m.add(e.text,e)}),m),additional:e.createMap()},u=function(n,t,a){var i=e.createMap();return r(n,function(n){var r=n.name,o=r.text,l=r.originalKeywordKind;!i.has(o)&&(void 0!==l&&e.isNonContextualKeyword(l)||t.resolveName(n.name.text,n,67220415,!0))&&i.set(o,s(\"_\"+o,a))}),i}(n,t,c);var d,m;!function(n,t,a){r(n,function(r,i){if(!i){var o=r.name.text;a.replaceNode(n,r,e.createIdentifier(t.get(o)||o))}})}(n,u,i);for(var p=!1,f=0,g=n.statements;f<g.length;f++){var _=g[f],v=a(n,_,t,i,c,o,u,l);p=p||v}return p}(o,l.getTypeChecker(),n,l.getCompilerOptions().target,e.getQuotePreference(o,c)))for(var i=0,u=l.getSourceFiles();i<u.length;i++){var d=u[i];t(d,o,n,e.getQuotePreference(d,c))}});return[n.createCodeFixActionNoFixId(\"convertToEs6Module\",u,e.Diagnostics.Convert_to_ES6_module)]}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"correctQualifiedNameToIndexedAccessType\",r=[e.Diagnostics.Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1.code];function a(n,t){var r=e.findAncestor(e.getTokenAtPosition(n,t),e.isQualifiedName);return e.Debug.assert(!!r,\"Expected position to be owned by a qualified name.\"),e.isIdentifier(r.left)?r:void 0}function i(n,t,r){var a=r.right.text,i=e.createIndexedAccessTypeNode(e.createTypeReferenceNode(r.left,void 0),e.createLiteralTypeNode(e.createLiteral(a)));n.replaceNode(t,r,i)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=a(r.sourceFile,r.span.start);if(o){var s=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,o)}),l=o.left.text+'[\"'+o.right.text+'\"]';return[n.createCodeFixAction(t,s,[e.Diagnostics.Rewrite_as_the_indexed_access_type_0,l],t,e.Diagnostics.Rewrite_all_as_indexed_access_types)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=a(n.file,n.start);t&&i(e,n.file,t)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=[e.Diagnostics.Class_0_incorrectly_implements_interface_1.code,e.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass.code],r=\"fixClassIncorrectlyImplementsInterface\";function a(n,t){return e.Debug.assertDefined(e.getContainingClass(e.getTokenAtPosition(n,t)))}function i(n){return!(8&e.getModifierFlags(n.valueDeclaration))}function o(t,r,a,o,s,l){var c=function(n,t){var r=e.getEffectiveBaseTypeNode(n);if(!r)return e.createSymbolTable();var a=t.getTypeAtLocation(r),o=t.getPropertiesOfType(a);return e.createSymbolTable(o.filter(i))}(o,t),u=t.getTypeAtLocation(r),d=t.getPropertiesOfType(u).filter(e.and(i,function(e){return!c.has(e.escapedName)})),m=t.getTypeAtLocation(o);function p(e,n){var r=t.getIndexInfoOfType(e,n);r&&s.insertNodeAtClassStart(a,o,t.indexInfoToIndexSignatureDeclaration(r,n,o))}m.getNumberIndexType()||p(u,1),m.getStringIndexType()||p(u,0),n.createMissingMemberNodes(o,d,t,l,function(e){return s.insertNodeAtClassStart(a,o,e)})}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var i=t.program,s=t.sourceFile,l=t.span,c=a(s,l.start),u=i.getTypeChecker();return e.mapDefined(e.getClassImplementsHeritageClauseElements(c),function(a){var i=e.textChanges.ChangeTracker.with(t,function(e){return o(u,a,s,c,e,t.preferences)});return 0===i.length?void 0:n.createCodeFixAction(r,i,[e.Diagnostics.Implement_interface_0,a.getText(s)],r,e.Diagnostics.Implement_all_unimplemented_interfaces)})},fixIds:[r],getAllCodeActions:function(r){var i=e.createMap();return n.codeFixAll(r,t,function(n,t){var s=a(t.file,t.start);if(e.addToSeen(i,e.getNodeId(s)))for(var l=0,c=e.getClassImplementsHeritageClauseElements(s);l<c.length;l++){var u=c[l];o(r.program.getTypeChecker(),u,t.file,s,n,r.preferences)}})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){n.importFixId=\"fixMissingImport\";var t,r,a=[e.Diagnostics.Cannot_find_name_0.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_1.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,e.Diagnostics.Cannot_find_namespace_0.code,e.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead.code,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here.code];function i(n,t){return!(67220415&e.skipAlias(n,t).flags)}function o(n,t,r,a,i,o,c){var u=a.getTypeChecker(),d=e.flatMap(n,function(n){return function(n,t,r){var a=n.moduleSymbol,i=n.importKind;return n.exportedSymbolIsTypeOnly&&e.isSourceFileJS(r)?e.emptyArray:e.mapDefined(r.imports,function(n){var r=e.importFromModuleSpecifier(n);return 247!==r.kind&&246!==r.kind||t.getSymbolAtLocation(n)!==a?void 0:{declaration:r,importKind:i}})}(n,u,i)}),m=void 0===r?void 0:function(n,t,r,a){return e.firstDefined(n,function(n){var i=n.declaration,o=function(n){if(247===n.kind){var t=n.importClause&&e.isImportClause(n.importClause)&&n.importClause.namedBindings;return t&&249===t.kind?t.name:void 0}return n.name}(i);if(o){var s=a.getAliasedSymbol(a.getSymbolAtLocation(o));if(s&&s.exports.has(e.escapeLeadingUnderscores(t)))return{kind:0,namespacePrefix:o.text,position:r}}})}(d,t,r,u),p=function(n){return e.firstDefined(n,function(e){var n=e.declaration,t=e.importKind;if(247===n.kind){var r=n.importClause;if(r){var a=r.name,i=r.namedBindings;return 1===t&&!a||0===t&&(!i||250===i.kind)?{kind:2,importClause:r,importKind:t}:void 0}}})}(d),f=p?[p]:function(n,t,r,a,i,o,c){var u=e.firstDefined(t,l);return u?[u]:s(r,a,i,n,o,c)}(n,d,a,i,r,o,c);return(m?[m]:e.emptyArray).concat(f)}function s(n,t,r,a,i,o){var s=e.isSourceFileJS(t);return e.flatMap(a,function(a){var l=a.moduleSymbol,c=a.importKind,u=a.exportedSymbolIsTypeOnly;return e.moduleSpecifiers.getModuleSpecifiers(l,n.getCompilerOptions(),t,i,n.getSourceFiles(),o,n.redirectTargetsMap).map(function(n){return u&&s?{kind:1,moduleSpecifier:n,position:e.Debug.assertDefined(r)}:{kind:3,moduleSpecifier:n,importKind:c}})}).sort(function(e,n){return e.moduleSpecifier.length-n.moduleSpecifier.length})}function l(n){var t=n.declaration,r=n.importKind,a=247===t.kind?t.moduleSpecifier:257===t.moduleReference.kind?t.moduleReference.expression:void 0;return a&&e.isStringLiteral(a)?{kind:3,moduleSpecifier:a.text,importKind:r}:void 0}function c(n,t,r){var a=e.getTokenAtPosition(n.sourceFile,r),s=t===e.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead.code?function(n,t){var r=n.sourceFile,a=n.program,i=n.host,s=n.preferences,l=a.getTypeChecker(),c=function(n,t){var r=e.isIdentifier(n)?t.getSymbolAtLocation(n):void 0;if(e.isUMDExportSymbol(r))return r;var a=n.parent;return e.isJsxOpeningLikeElement(a)&&a.tagName===n||e.isJsxOpeningFragment(a)?e.tryCast(t.resolveName(t.getJsxNamespace(a),e.isJsxOpeningLikeElement(a)?n:a,67220415,!1),e.isUMDExportSymbol):void 0}(t,l);if(!c)return;var u=l.getAliasedSymbol(c),d=c.name;return{fixes:o([{moduleSymbol:u,importKind:function(n){if(e.getAllowSyntheticDefaultImports(n))return 1;var t=e.getEmitModuleKind(n);switch(t){case e.ModuleKind.AMD:case e.ModuleKind.CommonJS:case e.ModuleKind.UMD:return 3;case e.ModuleKind.System:case e.ModuleKind.ES2015:case e.ModuleKind.ESNext:case e.ModuleKind.None:return 2;default:return e.Debug.assertNever(t)}}(a.getCompilerOptions()),exportedSymbolIsTypeOnly:!1}],d,e.isIdentifier(t)?t.getStart(r):void 0,a,r,i,s),symbolName:d}}(n,a):e.isIdentifier(a)?function(n,t){var r=n.sourceFile,a=n.program,s=n.cancellationToken,l=n.host,c=n.preferences,d=a.getTypeChecker(),m=e.isJsxOpeningLikeElement(t.parent)&&t.parent.tagName===t&&(e.isIntrinsicJsxName(t.text)||d.resolveName(t.text,t,67108863,!1))?d.getJsxNamespace():t.text;return e.Debug.assert(\"default\"!==m),{fixes:e.arrayFrom(e.flatMapIterator(function(n,t,r,a,o,s){var l=e.createMultiMap();function c(n,t,r){l.add(e.getUniqueSymbolId(t,o).toString(),{moduleSymbol:n,importKind:r,exportedSymbolIsTypeOnly:i(t,o)})}return y(o,a,s.getSourceFiles(),function(e){r.throwIfCancellationRequested();var a=u(e,o,s.getCompilerOptions());a&&a.name===n&&v(a.symbolForMeaning,t)&&c(e,a.symbol,a.kind);var i=o.tryGetMemberInModuleExportsAndProperties(n,e);i&&v(i,t)&&c(e,i,0)}),l}(m,e.getMeaningFromLocation(t),s,r,d,a).entries(),function(e){return e[0],o(e[1],m,t.getStart(r),a,r,l,c)})),symbolName:m}}(n,a):void 0;return s&&yn({},s,{fixes:e.sort(s.fixes,function(e,n){return e.kind-n.kind})})}function u(n,t,r){var a=function(e,n){var t=n.tryGetMemberInModuleExports(\"default\",e);if(t)return{symbol:t,kind:1};var r=n.resolveExternalModuleSymbol(e);return r===e?void 0:{symbol:r,kind:3}}(n,t);if(a){var i=a.symbol,o=a.kind,s=function n(t,r,a,i){var o=e.getLocalSymbolForExportDefault(t);if(o)return{symbolForMeaning:o,name:o.name};var s=(l=t,l.declarations&&e.firstDefined(l.declarations,function(n){if(e.isExportAssignment(n)){if(e.isIdentifier(n.expression))return n.expression.text}else if(e.isExportSpecifier(n))return e.Debug.assert(\"default\"===n.name.text),n.propertyName&&n.propertyName.text}));var l;if(void 0!==s)return{symbolForMeaning:t,name:s};if(2097152&t.flags){var c=a.getImmediateAliasedSymbol(t);return c&&n(c,e.Debug.assertDefined(c.parent),a,i)}return{symbolForMeaning:t,name:b(r,i.target)}}(i,n,t,r);return s&&yn({symbol:i,kind:o},s)}}function d(t,r,a,i,o){var s,l=e.textChanges.ChangeTracker.with(t,function(n){s=function(n,t,r,a,i){switch(a.kind){case 0:return p(n,t,a),[e.Diagnostics.Change_0_to_1,r,a.namespacePrefix+\".\"+r];case 1:return f(n,t,a,i),[e.Diagnostics.Change_0_to_1,r,g(a.moduleSpecifier,i)+r];case 2:var o=a.importClause,s=a.importKind;m(n,t,o,1===s?r:void 0,0===s?[r]:e.emptyArray);var l=e.stripQuotes(o.parent.moduleSpecifier.getText());return[1===s?e.Diagnostics.Add_default_import_0_to_existing_import_declaration_from_1:e.Diagnostics.Add_0_to_existing_import_declaration_from_1,r,l];case 3:var s=a.importKind,c=a.moduleSpecifier;return _(n,t,c,i,1===s?{defaultImport:r,namedImports:e.emptyArray,namespaceLikeImport:void 0}:0===s?{defaultImport:void 0,namedImports:[r],namespaceLikeImport:void 0}:{defaultImport:void 0,namedImports:e.emptyArray,namespaceLikeImport:{importKind:s,name:r}}),[1===s?e.Diagnostics.Import_default_0_from_module_1:e.Diagnostics.Import_0_from_module_1,r,c];default:return e.Debug.assertNever(a)}}(n,r,a,i,o)});return n.createCodeFixAction(\"import\",l,s,n.importFixId,e.Diagnostics.Add_all_missing_imports)}function m(n,t,r,a,i){if(a&&(e.Debug.assert(!r.name),n.insertNodeAt(t,r.getStart(t),e.createIdentifier(a),{suffix:\", \"})),i.length){var o=i.map(function(n){return e.createImportSpecifier(void 0,e.createIdentifier(n))});if(r.namedBindings&&e.cast(r.namedBindings,e.isNamedImports).elements.length)for(var s=0,l=o;s<l.length;s++){var c=l[s];n.insertNodeInListAfter(t,e.last(e.cast(r.namedBindings,e.isNamedImports).elements),c)}else if(o.length){var u=e.createNamedImports(o);r.namedBindings?n.replaceNode(t,r.namedBindings,u):n.insertNodeAfter(t,e.Debug.assertDefined(r.name),u)}}}function p(e,n,t){var r=t.namespacePrefix,a=t.position;e.insertText(n,a,r+\".\")}function f(e,n,t,r){var a=t.moduleSpecifier,i=t.position;e.insertText(n,i,g(a,r))}function g(n,t){var r=e.getQuoteFromPreference(t);return\"import(\"+r+n+r+\").\"}function _(n,t,r,a,i){var o=i.defaultImport,s=i.namedImports,l=i.namespaceLikeImport,c=e.makeStringLiteral(r,a);(void 0!==o||s.length)&&e.insertImport(n,t,e.makeImport(void 0===o?void 0:e.createIdentifier(o),s.map(function(n){return e.createImportSpecifier(void 0,e.createIdentifier(n))}),r,a)),l&&e.insertImport(n,t,3===l.importKind?e.createImportEqualsDeclaration(void 0,void 0,e.createIdentifier(l.name),e.createExternalModuleReference(c)):e.createImportDeclaration(void 0,void 0,e.createImportClause(void 0,e.createNamespaceImport(e.createIdentifier(l.name))),c))}function v(n,t){var r=n.declarations;return e.some(r,function(n){return!!(e.getMeaningFromDeclaration(n)&t)})}function y(n,t,r,a){h(n,r,function(n,r){var i,o,s;(void 0===r||r!==t&&(i=t.fileName,o=r.fileName,void 0===(s=e.forEachAncestorDirectory(o,function(n){return\"node_modules\"===e.getBaseFileName(n)?n:void 0}))||e.startsWith(i,e.getDirectoryPath(s))))&&a(n)})}function h(n,t,r){for(var a=0,i=n.getAmbientModules();a<i.length;a++){r(i[a],void 0)}for(var o=0,s=t;o<s.length;o++){var l=s[o];e.isExternalOrCommonJsModule(l)&&r(n.getMergedSymbol(l.symbol),l)}}function b(n,t){return E(e.removeFileExtension(e.stripQuotes(n.name)),t)}function E(n,t){var r=e.getBaseFileName(e.removeSuffix(n,\"/index\")),a=\"\",i=!0,o=r.charCodeAt(0);e.isIdentifierStart(o,t)?a+=String.fromCharCode(o):i=!1;for(var s=1;s<r.length;s++){var l=r.charCodeAt(s),c=e.isIdentifierPart(l,t);if(c){var u=String.fromCharCode(l);i||(u=u.toUpperCase()),a+=u}i=c}return e.isStringANonContextualKeyword(a)?\"_\"+a:a||\"_\"}n.registerCodeFix({errorCodes:a,getCodeActions:function(n){var t=n.errorCode,r=n.preferences,a=n.sourceFile,i=n.span,o=c(n,t,i.start);if(o){var s=o.fixes,l=o.symbolName,u=e.getQuotePreference(a,r);return s.map(function(e){return d(n,a,l,e,u)})}},fixIds:[n.importFixId],getAllCodeActions:function(t){var r=t.sourceFile,i=t.preferences,o=[],s=[],l=e.createMap(),u=e.createMap();return n.eachDiagnostic(t,a,function(n){var r=c(t,n.code,n.start);if(r&&r.fixes.length){var a=r.fixes,i=r.symbolName,d=e.first(a);switch(d.kind){case 0:o.push(d);break;case 1:s.push(d);break;case 2:var m=d.importClause,p=d.importKind,f=String(e.getNodeId(m));(g=l.get(f))||l.set(f,g={importClause:m,defaultImport:void 0,namedImports:[]}),0===p?e.pushIfUnique(g.namedImports,i):(e.Debug.assert(void 0===g.defaultImport||g.defaultImport===i),g.defaultImport=i);break;case 3:var g,_=d.moduleSpecifier;p=d.importKind;switch((g=u.get(_))||u.set(_,g={defaultImport:void 0,namedImports:[],namespaceLikeImport:void 0}),p){case 1:e.Debug.assert(void 0===g.defaultImport||g.defaultImport===i),g.defaultImport=i;break;case 0:e.pushIfUnique(g.namedImports,i);break;case 3:case 2:e.Debug.assert(void 0===g.namespaceLikeImport||g.namespaceLikeImport.name===i),g.namespaceLikeImport={importKind:p,name:i}}break;default:e.Debug.assertNever(d)}}}),n.createCombinedCodeActions(e.textChanges.ChangeTracker.with(t,function(n){for(var t=e.getQuotePreference(r,i),a=0,c=o;a<c.length;a++){var d=c[a];p(n,r,d)}for(var g=0,v=s;g<v.length;g++){d=v[g];f(n,r,d,t)}l.forEach(function(e){var t=e.importClause,a=e.defaultImport,i=e.namedImports;m(n,r,t,a,i)}),u.forEach(function(e,a){_(n,r,a,t,e)})}))}}),function(e){e[e.UseNamespace=0]=\"UseNamespace\",e[e.ImportType=1]=\"ImportType\",e[e.AddToExisting=2]=\"AddToExisting\",e[e.AddNew=3]=\"AddNew\"}(t||(t={})),function(e){e[e.Named=0]=\"Named\",e[e.Default=1]=\"Default\",e[e.Namespace=2]=\"Namespace\",e[e.Equals=3]=\"Equals\"}(r||(r={})),n.getImportCompletionAction=function(n,t,r,a,l,c,m,p,f){var g=function(n,t,r,a,o,s,l){var c=[];return h(s,l,function(l,d){if(!d||l===t||!e.startsWith(a.fileName,e.getDirectoryPath(d.fileName))){var m=u(l,s,o);m&&m.name===r&&e.skipAlias(m.symbol,s)===n&&c.push({moduleSymbol:l,importKind:m.kind,exportedSymbolIsTypeOnly:i(m.symbol,s)});for(var p=0,f=s.getExportsOfModule(l);p<f.length;p++){var g=f[p];g.name===r&&e.skipAlias(g,s)===n&&c.push({moduleSymbol:l,importKind:0,exportedSymbolIsTypeOnly:i(g,s)})}}}),c}(n,t,a,r,c.getCompilerOptions(),c.getTypeChecker(),c.getSourceFiles());e.Debug.assert(g.some(function(e){return e.moduleSymbol===t}));var _,v,y,b,E=e.first(s(c,r,p,g,l,f)).moduleSpecifier,T=e.first(o(g,a,p,c,r,l,f));return{moduleSpecifier:E,codeAction:(_=d({host:l,formatContext:m},r,a,T,e.getQuotePreference(r,f)),v=_.description,y=_.changes,b=_.commands,{description:v,changes:y,commands:b})}},n.forEachExternalModuleToImportFrom=y,n.moduleSymbolToValidIdentifier=b,n.moduleSpecifierToValidIdentifier=E}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=[e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_1.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,e.Diagnostics.Module_0_has_no_exported_member_1_Did_you_mean_2.code];function r(n,t,r){var a,i=e.getTokenAtPosition(n,t),o=r.program.getTypeChecker();if(e.isPropertyAccessExpression(i.parent)&&i.parent.name===i){e.Debug.assert(71===i.kind);var s=o.getTypeAtLocation(i.parent.expression);a=o.getSuggestionForNonexistentProperty(i,s)}else if(e.isImportSpecifier(i.parent)&&i.parent.name===i){e.Debug.assert(71===i.kind);var l=function(n,t,r){if(!r||!e.isStringLiteralLike(r.moduleSpecifier))return;var a=e.getResolvedModule(n,r.moduleSpecifier.text);return a?t.program.getSourceFile(a.resolvedFileName):void 0}(n,r,e.findAncestor(i,e.isImportDeclaration));l&&l.symbol&&(a=o.getSuggestionForNonexistentExport(i,l.symbol))}else{var c=e.getMeaningFromLocation(i),u=e.getTextOfNode(i);e.Debug.assert(void 0!==u,\"name should be defined\"),a=o.getSuggestionForNonexistentSymbol(i,u,function(e){var n=0;4&e&&(n|=1920);2&e&&(n|=67897832);1&e&&(n|=67220415);return n}(c))}return void 0===a?void 0:{node:i,suggestion:a}}function a(n,t,r,a,i){!e.isIdentifierText(a,i)&&e.isPropertyAccessExpression(r.parent)?n.replaceNode(t,r.parent,e.createElementAccess(r.parent.expression,e.createLiteral(a))):n.replaceNode(t,r,e.createIdentifier(a))}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var i=t.sourceFile,o=r(i,t.span.start,t);if(o){var s=o.node,l=o.suggestion,c=t.host.getCompilationSettings().target,u=e.textChanges.ChangeTracker.with(t,function(e){return a(e,i,s,l,c)});return[n.createCodeFixAction(\"spelling\",u,[e.Diagnostics.Change_spelling_to_0,l],\"fixSpelling\",e.Diagnostics.Fix_all_detected_spelling_errors)]}},fixIds:[\"fixSpelling\"],getAllCodeActions:function(e){return n.codeFixAll(e,t,function(n,t){var i=r(t.file,t.start,e),o=e.host.getCompilationSettings().target;i&&a(n,e.sourceFile,i.node,i.suggestion,o)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t,r=\"addMissingMember\",a=[e.Diagnostics.Property_0_does_not_exist_on_type_1.code,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2.code],i=\"addMissingMember\";function o(n,t,r){var a=e.getTokenAtPosition(n,t);if(e.isIdentifier(a)){var i=a.parent;if(e.isPropertyAccessExpression(i)){var o=e.skipConstraint(r.getTypeAtLocation(i.expression)),s=o.symbol;if(s&&s.declarations){var l=e.find(s.declarations,e.isClassLike)||e.find(s.declarations,e.isInterfaceDeclaration);if(l){var c=(o.target||o)!==r.getDeclaredTypeOfSymbol(s),u=l.getSourceFile();return{kind:1,token:a,parentDeclaration:l,makeStatic:c,declSourceFile:u,inJs:e.isSourceFileJS(u),call:e.tryCast(i.parent,e.isCallExpression)}}var d=e.find(s.declarations,e.isEnumDeclaration);return d?{kind:0,token:a,parentDeclaration:d}:void 0}}}}function s(n,t,r,a,i){if(i){if(207===r.kind)return;var o=r.name.getText(),s=l(e.createIdentifier(o),a);n.insertNodeAfter(t,r,s)}else{var c=e.getFirstConstructorWithBody(r);if(!c)return;var u=l(e.createThis(),a);n.insertNodeAtConstructorEnd(t,c,u)}}function l(n,t){return e.createStatement(e.createAssignment(e.createPropertyAccess(n,t),e.createIdentifier(\"undefined\")))}function c(n,t,r){var a;if(202===r.parent.parent.kind){var i=r.parent.parent,o=r.parent===i.left?i.right:i.left,s=n.getWidenedType(n.getBaseTypeOfLiteralType(n.getTypeAtLocation(o)));a=n.typeToTypeNode(s,t)}return a||e.createKeywordTypeNode(119)}function u(n,t,r,a,i,o){var s=e.createProperty(void 0,o?[e.createToken(115)]:void 0,a,void 0,i,void 0),l=function(n){for(var t,r=0,a=n.members;r<a.length;r++){var i=a[r];if(!e.isPropertyDeclaration(i))break;t=i}return t}(r);l?n.insertNodeAfter(t,l,s):n.insertNodeAtClassStart(t,r,s)}function d(t,r,a,i,o,s,l,c,u){var d=n.createMethodFromCallExpression(t,s,o.text,c,l,u,!e.isInterfaceDeclaration(i)),m=e.getAncestor(s,154);m&&m.parent===i?r.insertNodeAfter(a,m,d):r.insertNodeAtClassStart(a,i,d)}function m(n,t,r,a){var i=e.some(a.members,function(e){var n=t.getTypeAtLocation(e);return!!(n&&68&n.flags)}),o=e.createEnumMember(r,i?e.createStringLiteral(r.text):void 0);n.replaceNode(a.getSourceFile(),a,e.updateEnumDeclaration(a,a.decorators,a.modifiers,a.name,e.concatenate(a.members,e.singleElementArray(o))))}n.registerCodeFix({errorCodes:a,getCodeActions:function(t){var a=o(t.sourceFile,t.span.start,t.program.getTypeChecker());if(a){if(0===a.kind){var l=a.token,p=a.parentDeclaration,f=e.textChanges.ChangeTracker.with(t,function(e){return m(e,t.program.getTypeChecker(),l,p)});return[n.createCodeFixAction(r,f,[e.Diagnostics.Add_missing_enum_member_0,l.text],i,e.Diagnostics.Add_all_missing_members)]}var g=a.parentDeclaration,_=a.declSourceFile,v=a.inJs,y=a.makeStatic,h=a.token,b=a.call,E=b&&function(t,a,o,s,l,c,u,m){var p=e.textChanges.ChangeTracker.with(t,function(e){return d(t,e,a,o,s,l,c,u,m)});return n.createCodeFixAction(r,p,[c?e.Diagnostics.Declare_static_method_0:e.Diagnostics.Declare_method_0,s.text],i,e.Diagnostics.Add_all_missing_members)}(t,_,g,h,b,y,v,t.preferences),T=v&&!e.isInterfaceDeclaration(g)?e.singleElementArray(function(t,a,o,l,c){var u=e.textChanges.ChangeTracker.with(t,function(e){return s(e,a,o,l,c)});return 0===u.length?void 0:n.createCodeFixAction(r,u,[c?e.Diagnostics.Initialize_static_property_0:e.Diagnostics.Initialize_property_0_in_the_constructor,l],i,e.Diagnostics.Add_all_missing_members)}(t,_,g,h.text,y)):function(t,a,o,s,l){var d=c(t.program.getTypeChecker(),o,s),m=function(t,a,o,s,l,c){var d=e.textChanges.ChangeTracker.with(t,function(e){return u(e,a,o,l,c,s)});return n.createCodeFixAction(r,d,[s?e.Diagnostics.Declare_static_property_0:e.Diagnostics.Declare_property_0,l],i,e.Diagnostics.Add_all_missing_members)}(t,a,o,l,s.text,d);return l?[m]:[m,function(t,a,i,o,s){var l=e.createKeywordTypeNode(137),c=e.createParameter(void 0,void 0,void 0,\"x\",void 0,l,void 0),u=e.createIndexSignature(void 0,void 0,[c],s),d=e.textChanges.ChangeTracker.with(t,function(e){return e.insertNodeAtClassStart(a,i,u)});return n.createCodeFixActionNoFixId(r,d,[e.Diagnostics.Add_index_signature_for_property_0,o])}(t,a,o,s.text,d)]}(t,_,g,h,y);return e.concatenate(e.singleElementArray(E),T)}},fixIds:[i],getAllCodeActions:function(t){var r=t.program,i=t.preferences,l=r.getTypeChecker(),p=e.createMap(),f=new e.NodeMap;return n.createCombinedCodeActions(e.textChanges.ChangeTracker.with(t,function(g){n.eachDiagnostic(t,a,function(n){var t=o(n.file,n.start,l);if(t&&e.addToSeen(p,e.getNodeId(t.parentDeclaration)+\"#\"+t.token.text))if(0===t.kind){var r=t.token,a=t.parentDeclaration;m(g,l,r,a)}else{a=t.parentDeclaration;var i=t.token,s=f.getOrUpdate(a,function(){return[]});s.some(function(e){return e.token.text===i.text})||s.push(t)}}),f.forEach(function(n,a){for(var o=function(n,t){var r=[];for(;n;){var a=e.getClassExtendsHeritageElement(n),i=a&&t.getSymbolAtLocation(a.expression),o=i&&e.find(i.declarations,e.isClassLike);o&&r.push(o),n=o}return r}(a,l),m=function(n){if(o.some(function(e){var t=f.get(e);return!!t&&t.some(function(e){return e.token.text===n.token.text})}))return\"continue\";var a=n.parentDeclaration,l=n.declSourceFile,m=n.inJs,p=n.makeStatic,_=n.token,v=n.call;if(v)d(t,g,l,a,_,v,p,m,i);else if(m&&!e.isInterfaceDeclaration(a))s(g,l,a,_.text,p);else{var y=c(r.getTypeChecker(),a,_);u(g,l,a,_.text,y,p)}},p=0,_=n;p<_.length;p++){m(_[p])}})}))}}),function(e){e[e.Enum=0]=\"Enum\",e[e.ClassOrInterface=1]=\"ClassOrInterface\"}(t||(t={}))}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"fixCannotFindModule\",r=\"generateTypes\",a=e.Diagnostics.Cannot_find_module_0.code,i=[a,e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type.code];function o(n,t,r){var a=r.sourceFile.fileName,i=e.tryResolveJSModule(t,e.getDirectoryPath(a),r.host);if(void 0!==i){var o=e.resolvePath(e.getDirectoryPath(r.program.getCompilerOptions().configFile.fileName),n,t+\".d.ts\");if(!r.host.fileExists(o))return{type:\"generate types\",file:a,fileToGenerateTypesFor:i,outputFileName:o}}}function s(n,t){var r=t.program.getCompilerOptions().configFile;if(r){var a=e.getTsConfigObjectLiteralExpression(r);if(a){var i=f(a,\"compilerOptions\");if(!i){var o=e.createObjectLiteral([c(),m()]);return n.insertNodeAtObjectStart(r,a,p(\"compilerOptions\",o)),u}var s=i.initializer;if(!e.isObjectLiteralExpression(s))return u;var g=function(n,t,r){var a=f(r,\"baseUrl\");return a?e.isStringLiteral(a.initializer)?a.initializer.text:l:(n.insertNodeAtObjectStart(t,r,c()),l)}(n,r,s),_=function(n,t,r){var a=f(r,\"paths\");if(!a||!e.isObjectLiteralExpression(a.initializer))return n.insertNodeAtObjectStart(t,r,m()),u;var i=e.firstDefined(a.initializer.properties,function(n){return e.isPropertyAssignment(n)&&e.isStringLiteral(n.name)&&\"*\"===n.name.text&&e.isArrayLiteralExpression(n.initializer)?e.firstDefined(n.initializer.elements,function(n){return e.isStringLiteral(n)?e.tryRemoveSuffix(n.text,\"/*\"):void 0}):void 0});return i||(n.insertNodeAtObjectStart(t,a.initializer,d()),u)}(n,r,s);return e.combinePaths(g,_)}}}n.registerCodeFix({errorCodes:i,getCodeActions:function(a){var i=a.host,l=a.sourceFile,c=_(l,a.span.start);if(void 0!==c){var u=v(c,i,a.errorCode);return void 0===u?e.singleElementArray(function(a,i){var l,c=e.textChanges.ChangeTracker.with(a,function(e){var n=s(e,a);l=void 0===n?void 0:o(n,i,a)});return l&&n.createCodeFixAction(t,c,[e.Diagnostics.Generate_types_for_0,i],r,e.Diagnostics.Generate_types_for_all_packages_without_types,l)}(a,c)):[n.createCodeFixAction(t,[],[e.Diagnostics.Install_0,u],\"installTypesPackage\",e.Diagnostics.Install_all_missing_types_packages,g(l.fileName,u))]}},fixIds:[\"installTypesPackage\",r],getAllCodeActions:function(t){var a=null;return n.codeFixAll(t,i,function(n,i,l){var c=_(i.file,i.start);if(void 0!==c)switch(t.fixId){case\"installTypesPackage\":var u=v(c,t.host,i.code);u&&l.push(g(i.file.fileName,u));break;case r:var d=null!==a?a:a=s(n,t),m=void 0===d?void 0:o(d,c,t);m&&l.push(m);break;default:e.Debug.fail(\"Bad fixId: \"+t.fixId)}})}});var l=\".\";function c(){return p(\"baseUrl\",e.createStringLiteral(l))}var u=\"types\";function d(){return p(\"*\",e.createArrayLiteral([e.createStringLiteral(u+\"/*\")]))}function m(){return p(\"paths\",e.createObjectLiteral([d()]))}function p(n,t){return e.createPropertyAssignment(e.createStringLiteral(n),t)}function f(n,t){return e.find(n.properties,function(n){return e.isPropertyAssignment(n)&&!!n.name&&e.isStringLiteral(n.name)&&n.name.text===t})}function g(e,n){return{type:\"install package\",file:e,packageName:n}}function _(n,t){var r=e.cast(e.getTokenAtPosition(n,t),e.isStringLiteral).text,a=e.parsePackageName(r).packageName;return e.isExternalModuleNameRelative(a)?void 0:a}function v(n,t,r){return r===a?e.JsTyping.nodeCoreModules.has(n)?\"@types/node\":void 0:t.isKnownTypesPackageName(n)?e.getTypesPackageName(n):void 0}}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=[e.Diagnostics.Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2.code,e.Diagnostics.Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1.code],r=\"fixClassDoesntImplementInheritedAbstractMember\";function a(n,t){var r=e.getTokenAtPosition(n,t);return e.cast(r.parent,e.isClassLike)}function i(t,r,a,i,s){var l=e.getEffectiveBaseTypeNode(t),c=a.getTypeAtLocation(l),u=a.getPropertiesOfType(c).filter(o);n.createMissingMemberNodes(t,u,a,s,function(e){return i.insertNodeAtClassStart(r,t,e)})}function o(n){var t=e.getModifierFlags(e.first(n.getDeclarations()));return!(8&t||!(128&t))}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var o=t.program,s=t.sourceFile,l=t.span,c=e.textChanges.ChangeTracker.with(t,function(e){return i(a(s,l.start),s,o.getTypeChecker(),e,t.preferences)});return 0===c.length?void 0:[n.createCodeFixAction(r,c,e.Diagnostics.Implement_inherited_abstract_class,r,e.Diagnostics.Implement_all_inherited_abstract_classes)]},fixIds:[r],getAllCodeActions:function(r){var o=e.createMap();return n.codeFixAll(r,t,function(n,t){var s=a(t.file,t.start);e.addToSeen(o,e.getNodeId(s))&&i(s,r.sourceFile,r.program.getTypeChecker(),n,r.preferences)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"classSuperMustPrecedeThisAccess\",r=[e.Diagnostics.super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class.code];function a(e,n,t,r){e.insertNodeAtConstructorStart(n,t,r),e.delete(n,r)}function i(n,t){var r=e.getTokenAtPosition(n,t);if(99===r.kind){var a=e.getContainingFunction(r),i=o(a.body);return i&&!i.expression.arguments.some(function(n){return e.isPropertyAccessExpression(n)&&n.expression===r})?{constructor:a,superCall:i}:void 0}}function o(n){return e.isExpressionStatement(n)&&e.isSuperCall(n.expression)?n:e.isFunctionLike(n)?void 0:e.forEachChild(n,o)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,l=i(o,s.start);if(l){var c=l.constructor,u=l.superCall,d=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,c,u)});return[n.createCodeFixAction(t,d,e.Diagnostics.Make_super_call_the_first_statement_in_the_constructor,t,e.Diagnostics.Make_all_super_calls_the_first_statement_in_their_constructor)]}},fixIds:[t],getAllCodeActions:function(t){var o=t.sourceFile,s=e.createMap();return n.codeFixAll(t,r,function(n,t){var r=i(t.file,t.start);if(r){var l=r.constructor,c=r.superCall;e.addToSeen(s,e.getNodeId(l.parent))&&a(n,o,l,c)}})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"constructorForDerivedNeedSuperCall\",r=[e.Diagnostics.Constructors_for_derived_classes_must_contain_a_super_call.code];function a(n,t){var r=e.getTokenAtPosition(n,t);return e.Debug.assert(123===r.kind),r.parent}function i(n,t,r){var a=e.createStatement(e.createCall(e.createSuper(),void 0,e.emptyArray));n.insertNodeAtConstructorStart(t,r,a)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,l=a(o,s.start),c=e.textChanges.ChangeTracker.with(r,function(e){return i(e,o,l)});return[n.createCodeFixAction(t,c,e.Diagnostics.Add_missing_super_call,t,e.Diagnostics.Add_all_missing_super_calls)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return i(n,e.sourceFile,a(t.file,t.start))})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"extendsInterfaceBecomesImplements\",r=[e.Diagnostics.Cannot_extend_an_interface_0_Did_you_mean_implements.code];function a(n,t){var r=e.getTokenAtPosition(n,t),a=e.getContainingClass(r).heritageClauses,i=a[0].getFirstToken();return 85===i.kind?{extendsToken:i,heritageClauses:a}:void 0}function i(n,t,r,a){if(n.replaceNode(t,r,e.createToken(108)),2===a.length&&85===a[0].token&&108===a[1].token){var i=a[1].getFirstToken(),o=i.getFullStart();n.replaceRange(t,{pos:o,end:o},e.createToken(26));for(var s=t.text,l=i.end;l<s.length&&e.isWhiteSpaceSingleLine(s.charCodeAt(l));)l++;n.deleteRange(t,{pos:i.getStart(),end:l})}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=a(o,r.span.start);if(s){var l=s.extendsToken,c=s.heritageClauses,u=e.textChanges.ChangeTracker.with(r,function(e){return i(e,o,l,c)});return[n.createCodeFixAction(t,u,e.Diagnostics.Change_extends_to_implements,t,e.Diagnostics.Change_all_extended_interfaces_to_implements)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=a(n.file,n.start);t&&i(e,n.file,t.extendsToken,t.heritageClauses)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"forgottenThisPropertyAccess\",r=e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,a=[e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,r];function i(n,t,a){var i=e.getTokenAtPosition(n,t);if(e.isIdentifier(i))return{node:i,className:a===r?e.getContainingClass(i).name.text:void 0}}function o(n,t,r){var a=r.node,i=r.className;e.suppressLeadingAndTrailingTrivia(a),n.replaceNode(t,a,e.createPropertyAccess(i?e.createIdentifier(i):e.createThis(),a))}n.registerCodeFix({errorCodes:a,getCodeActions:function(r){var a=r.sourceFile,s=i(a,r.span.start,r.errorCode);if(s){var l=e.textChanges.ChangeTracker.with(r,function(e){return o(e,a,s)});return[n.createCodeFixAction(t,l,[e.Diagnostics.Add_0_to_unresolved_variable,s.className||\"this\"],t,e.Diagnostics.Add_qualifier_to_all_unresolved_variables_matching_a_member_name)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,a,function(n,t){var r=i(t.file,t.start,t.code);r&&o(n,e.sourceFile,r)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"unusedIdentifier\",r=\"unusedIdentifier_prefix\",a=\"unusedIdentifier_delete\",i=\"unusedIdentifier_infer\",o=[e.Diagnostics._0_is_declared_but_its_value_is_never_read.code,e.Diagnostics._0_is_declared_but_never_used.code,e.Diagnostics.Property_0_is_declared_but_its_value_is_never_read.code,e.Diagnostics.All_imports_in_import_declaration_are_unused.code,e.Diagnostics.All_destructured_elements_are_unused.code,e.Diagnostics.All_variables_are_unused.code,e.Diagnostics.All_type_parameters_are_unused.code];function s(n,t,r){n.replaceNode(t,r.parent,e.createKeywordTypeNode(142))}function l(r,i){return n.createCodeFixAction(t,r,i,a,e.Diagnostics.Delete_all_unused_declarations)}function c(n,t,r){n.delete(t,e.Debug.assertDefined(e.cast(r.parent,e.isDeclarationWithTypeParameterChildren).typeParameters))}function u(n){return 91===n.kind?e.tryCast(n.parent,e.isImportDeclaration):void 0}function d(n,t,r,a,i,o){if(17!==n.kind||!e.isObjectBindingPattern(n.parent))return!1;var s=n.parent.parent;return 149===s.kind?g(t,r,s,a,i,o):t.delete(r,s),!0}function m(n,t,r){var a=e.tryCast(t.parent,e.isVariableDeclarationList);return!(!a||a.getChildren(n)[0]!==t)&&(r.delete(n,217===a.parent.kind?a.parent:a),!0)}function p(n,t,r,a){t!==e.Diagnostics.Property_0_is_declared_but_its_value_is_never_read.code&&(126===a.kind&&(a=e.cast(a.parent,e.isInferTypeNode).typeParameter.name),e.isIdentifier(a)&&function(e){switch(e.parent.kind){case 149:case 148:return!0;case 235:var n=e.parent;switch(n.parent.parent.kind){case 225:case 224:return!0}}return!1}(a)&&n.replaceNode(r,a,e.createIdentifier(\"_\"+a.text)))}function f(n,t,r,a,i,o){!function(n,t,r,a,i,o){var s=n.parent;e.isParameter(s)?g(t,r,s,a,i,o):t.delete(r,e.isImportClause(s)?n:e.isComputedPropertyName(s)?s.parent:s)}(t,r,n,a,i,o),e.isIdentifier(t)&&function(n,t,r,a){e.FindAllReferences.Core.eachSymbolReferenceInFile(r,a,t,function(e){187===e.parent.kind&&(e=e.parent),202===e.parent.kind&&219===e.parent.parent.kind&&n.delete(t,e.parent.parent)})}(r,n,t,a)}function g(n,t,r,a,i,o){(function(n,t,r){var a=n.parent;switch(a.kind){case 154:var i=t.getSymbolAtLocation(a.name);if(e.isMemberSymbolInBaseType(i,t))return!1;case 155:case 237:return!0;case 194:case 195:var o=a.parameters,s=o.indexOf(n);return e.Debug.assert(-1!==s),r?o.slice(s+1).every(function(e){return 71===e.name.kind&&!e.symbol.isReferenced}):s===o.length-1;case 157:return!1;default:return e.Debug.failBadSyntaxKind(a)}})(r,a,o)&&(n.delete(t,r),function(n,t,r,a,i){e.FindAllReferences.Core.eachSignatureCall(r.parent,a,i,function(e){var a=r.parent.parameters.indexOf(r);e.arguments.length>a&&n.delete(t,e.arguments[a])})}(n,t,r,i,a))}n.registerCodeFix({errorCodes:o,getCodeActions:function(a){var o=a.errorCode,g=a.sourceFile,_=a.program,v=_.getTypeChecker(),y=_.getSourceFiles(),h=e.getTokenAtPosition(g,a.span.start);if(e.isJSDocTemplateTag(h))return[l(e.textChanges.ChangeTracker.with(a,function(e){return e.delete(g,h)}),e.Diagnostics.Remove_template_tag)];if(27===h.kind)return[l(L=e.textChanges.ChangeTracker.with(a,function(e){return c(e,g,h)}),e.Diagnostics.Remove_type_parameters)];var b=u(h);if(b)return[l(L=e.textChanges.ChangeTracker.with(a,function(e){return e.delete(g,b)}),[e.Diagnostics.Remove_import_from_0,e.showModuleSpecifier(b)])];var E=e.textChanges.ChangeTracker.with(a,function(e){return d(h,e,g,v,y,!1)});if(E.length)return[l(E,e.Diagnostics.Remove_destructuring)];var T=e.textChanges.ChangeTracker.with(a,function(e){return m(g,h,e)});if(T.length)return[l(T,e.Diagnostics.Remove_variable_statement)];var S=[];if(126===h.kind){var L=e.textChanges.ChangeTracker.with(a,function(e){return s(e,g,h)}),x=e.cast(h.parent,e.isInferTypeNode).typeParameter.name.text;S.push(n.createCodeFixAction(t,L,[e.Diagnostics.Replace_infer_0_with_unknown,x],i,e.Diagnostics.Replace_all_unused_infer_with_unknown))}else{var A=e.textChanges.ChangeTracker.with(a,function(e){return f(g,h,e,v,y,!1)});if(A.length){x=e.isComputedPropertyName(h.parent)?h.parent:h;S.push(l(A,[e.Diagnostics.Remove_declaration_for_Colon_0,x.getText(g)]))}}var C=e.textChanges.ChangeTracker.with(a,function(e){return p(e,o,g,h)});return C.length&&S.push(n.createCodeFixAction(t,C,[e.Diagnostics.Prefix_0_with_an_underscore,h.getText(g)],r,e.Diagnostics.Prefix_all_unused_declarations_with_where_possible)),S},fixIds:[r,a,i],getAllCodeActions:function(t){var l=t.sourceFile,g=t.program,_=g.getTypeChecker(),v=g.getSourceFiles();return n.codeFixAll(t,o,function(n,o){var g=e.getTokenAtPosition(l,o.start);switch(t.fixId){case r:p(n,o.code,l,g);break;case a:if(126===g.kind)break;var y=u(g);y?n.delete(l,y):e.isJSDocTemplateTag(g)?n.delete(l,g):27===g.kind?c(n,l,g):d(g,n,l,_,v,!0)||m(l,g,n)||f(l,g,n,_,v,!0);break;case i:126===g.kind&&s(n,l,g);break;default:e.Debug.fail(JSON.stringify(t.fixId))}})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"fixUnreachableCode\",r=[e.Diagnostics.Unreachable_code_detected.code];function a(n,t,r,a){var i=e.getTokenAtPosition(t,r),o=e.findAncestor(i,e.isStatement);e.Debug.assert(o.getStart(t)===i.getStart(t));var s=(e.isBlock(o.parent)?o.parent:o).parent;if(!e.isBlock(o.parent)||o===e.first(o.parent.statements))switch(s.kind){case 220:if(s.elseStatement){if(e.isBlock(o.parent))break;return void n.replaceNode(t,o,e.createBlock(e.emptyArray))}case 222:case 223:return void n.delete(t,s)}if(e.isBlock(o.parent)){var l=r+a,c=e.Debug.assertDefined(function(e,n){for(var t,r=0,a=e;r<a.length;r++){var i=a[r];if(!n(i))break;t=i}return t}(e.sliceAfter(o.parent.statements,o),function(e){return e.pos<l}));n.deleteNodeRange(t,o,c)}else n.delete(t,o)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=e.textChanges.ChangeTracker.with(r,function(e){return a(e,r.sourceFile,r.span.start,r.span.length)});return[n.createCodeFixAction(t,i,e.Diagnostics.Remove_unreachable_code,t,e.Diagnostics.Remove_all_unreachable_code)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return a(e,n.file,n.start,n.length)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"fixUnusedLabel\",r=[e.Diagnostics.Unused_label.code];function a(n,t,r){var a=e.getTokenAtPosition(t,r),i=e.cast(a.parent,e.isLabeledStatement),o=a.getStart(t),s=i.statement.getStart(t),l=e.positionsAreOnSameLine(o,s,t)?s:e.skipTrivia(t.text,e.findChildOfKind(i,56,t).end,!0);n.deleteRange(t,{pos:o,end:l})}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=e.textChanges.ChangeTracker.with(r,function(e){return a(e,r.sourceFile,r.span.start)});return[n.createCodeFixAction(t,i,e.Diagnostics.Remove_unused_label,t,e.Diagnostics.Remove_all_unused_labels)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return a(e,n.file,n.start)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=[e.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments.code];function r(e,n,t,r,a){e.replaceNode(n,t,a.typeToTypeNode(r,t))}function a(n,t,r){var a=e.findAncestor(e.getTokenAtPosition(n,t),i),o=a&&a.type;return o&&{typeNode:o,type:r.getTypeFromTypeNode(o)}}function i(e){switch(e.kind){case 210:case 158:case 159:case 237:case 156:case 160:case 179:case 154:case 153:case 149:case 152:case 151:case 157:case 240:case 192:case 235:return!0;default:return!1}}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var i=t.sourceFile,o=t.program.getTypeChecker(),s=a(i,t.span.start,o);if(s){var l=s.typeNode,c=s.type,u=l.getText(i),d=[m(c,\"fixJSDocTypes_plain\",e.Diagnostics.Change_all_jsdoc_style_types_to_TypeScript)];return 284===l.kind&&d.push(m(o.getNullableType(c,8192),\"fixJSDocTypes_nullable\",e.Diagnostics.Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types)),d}function m(a,s,c){var d=e.textChanges.ChangeTracker.with(t,function(e){return r(e,i,l,a,o)});return n.createCodeFixAction(\"jdocTypes\",d,[e.Diagnostics.Change_0_to_1,u,o.typeToString(a)],s,c)}},fixIds:[\"fixJSDocTypes_plain\",\"fixJSDocTypes_nullable\"],getAllCodeActions:function(e){var i=e.fixId,o=e.program,s=e.sourceFile,l=o.getTypeChecker();return n.codeFixAll(e,t,function(e,n){var t=a(n.file,n.start,l);if(t){var o=t.typeNode,c=t.type,u=284===o.kind&&\"fixJSDocTypes_nullable\"===i?l.getNullableType(c,8192):c;r(e,s,o,u,l)}})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"fixAwaitInSyncFunction\",r=[e.Diagnostics.await_expression_is_only_allowed_within_an_async_function.code,e.Diagnostics.A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator.code];function a(n,t){var r=e.getTokenAtPosition(n,t),a=e.getContainingFunction(r);if(a){var i,o;switch(a.kind){case 154:i=a.name;break;case 237:case 194:i=e.findChildOfKind(a,89,n);break;case 195:i=e.findChildOfKind(a,19,n)||e.first(a.parameters);break;default:return}return i&&{insertBefore:i,returnType:(o=a,o.type?o.type:e.isVariableDeclaration(o.parent)&&o.parent.type&&e.isFunctionTypeNode(o.parent.type)?o.parent.type.type:void 0)}}}function i(n,t,r){var a=r.insertBefore,i=r.returnType;if(i){var o=e.getEntityNameFromTypeNode(i);o&&71===o.kind&&\"Promise\"===o.text||n.replaceNode(t,i,e.createTypeReferenceNode(\"Promise\",e.createNodeArray([i])))}n.insertModifierBefore(t,120,a)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,l=a(o,s.start);if(l){var c=e.textChanges.ChangeTracker.with(r,function(e){return i(e,o,l)});return[n.createCodeFixAction(t,c,e.Diagnostics.Add_async_modifier_to_containing_function,t,e.Diagnostics.Add_all_missing_async_modifiers)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){var r=a(t.file,t.start);r&&i(n,e.sourceFile,r)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"disableJsDiagnostics\",r=e.mapDefined(Object.keys(e.Diagnostics),function(n){var t=e.Diagnostics[n];return t.category===e.DiagnosticCategory.Error?t.code:void 0});function a(n,t,r,a){var i=e.getLineAndCharacterOfPosition(t,r).line;a&&!e.addToSeen(a,i)||n.insertCommentBeforeLine(t,i,r,\" @ts-ignore\")}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=r.sourceFile,o=r.program,s=r.span,l=r.host,c=r.formatContext;if(e.isInJSFile(i)&&e.isCheckJsEnabledForFile(i,o.getCompilerOptions())){var u=[n.createCodeFixActionNoFixId(\"disableJsDiagnostics\",[n.createFileTextChanges(i.fileName,[e.createTextChange(i.checkJsDirective?e.createTextSpanFromBounds(i.checkJsDirective.pos,i.checkJsDirective.end):e.createTextSpan(0,0),\"// @ts-nocheck\"+e.getNewLineOrDefaultFromHost(l,c.options))])],e.Diagnostics.Disable_checking_for_this_file)];return e.textChanges.isValidLocationToAddComment(i,s.start)&&u.unshift(n.createCodeFixAction(\"disableJsDiagnostics\",e.textChanges.ChangeTracker.with(r,function(e){return a(e,i,s.start)}),e.Diagnostics.Ignore_this_error_message,t,e.Diagnostics.Add_ts_ignore_to_all_error_messages)),u}},fixIds:[t],getAllCodeActions:function(t){var i=e.createMap();return n.codeFixAll(t,r,function(n,t){e.textChanges.isValidLocationToAddComment(t.file,t.start)&&a(n,t.file,t.start,i)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){function t(n,t,i,o,s){var l=n.getDeclarations();if(l&&l.length){var c=l[0],u=e.getSynthesizedDeepClone(e.getNameOfDeclaration(c),!1),d=function(n){if(4&n)return e.createToken(114);if(16&n)return e.createToken(113);return}(e.getModifierFlags(c)),m=d?e.createNodeArray([d]):void 0,p=i.getWidenedType(i.getTypeOfSymbolAtLocation(n,t)),f=!!(16777216&n.flags);switch(c.kind){case 156:case 157:case 151:case 152:var g=i.typeToTypeNode(p,t);s(e.createProperty(void 0,m,u,f?e.createToken(55):void 0,g,void 0));break;case 153:case 154:var _=i.getSignaturesOfType(p,0);if(!e.some(_))break;if(1===l.length){e.Debug.assert(1===_.length),h(_[0],m,u,a(o));break}for(var v=0,y=_;v<y.length;v++){h(y[v],e.getSynthesizedDeepClones(m,!1),e.getSynthesizedDeepClone(u,!1))}if(l.length>_.length)h(i.getSignatureFromDeclaration(l[l.length-1]),m,u,a(o));else e.Debug.assert(l.length===_.length),s(function(n,t,i,o,s){for(var l=n[0],c=n[0].minArgumentCount,u=!1,d=0,m=n;d<m.length;d++){var p=m[d];c=Math.min(p.minArgumentCount,c),p.hasRestParameter&&(u=!0),p.parameters.length>=l.parameters.length&&(!p.hasRestParameter||l.hasRestParameter)&&(l=p)}var f=l.parameters.length-(l.hasRestParameter?1:0),g=l.parameters.map(function(e){return e.name}),_=r(f,g,void 0,c,!1);if(u){var v=e.createArrayTypeNode(e.createKeywordTypeNode(119)),y=e.createParameter(void 0,void 0,e.createToken(24),g[f]||\"rest\",f>=c?e.createToken(55):void 0,v,void 0);_.push(y)}return function(n,t,r,i,o,s,l){return e.createMethod(void 0,n,void 0,t,r?e.createToken(55):void 0,i,o,s,a(l))}(o,t,i,void 0,_,void 0,s)}(_,u,f,m,o))}}function h(n,r,a,o){var l=function(n,t,r,a,i,o,s){var l=n.signatureToSignatureDeclaration(t,154,r,256);if(!l)return;return l.decorators=void 0,l.modifiers=a,l.name=i,l.questionToken=o?e.createToken(55):void 0,l.body=s,l}(i,n,t,r,a,f,o);l&&s(l)}}function r(n,t,r,a,i){for(var o=[],s=0;s<n;s++){var l=e.createParameter(void 0,void 0,void 0,t&&t[s]||\"arg\"+s,void 0!==a&&s>=a?e.createToken(55):void 0,i?void 0:r&&r[s]||e.createKeywordTypeNode(119),void 0);o.push(l)}return o}function a(n){return e.createBlock([e.createThrow(e.createNew(e.createIdentifier(\"Error\"),void 0,[e.createLiteral(\"Method not implemented.\",\"single\"===n.quotePreference)]))],!0)}n.createMissingMemberNodes=function(e,n,r,a,i){for(var o=e.symbol.members,s=0,l=n;s<l.length;s++){var c=l[s];o.has(c.escapedName)||t(c,e,r,a,i)}},n.createMethodFromCallExpression=function(n,t,i,o,s,l,c){var u=t.typeArguments,d=t.arguments,m=t.parent,p=n.program.getTypeChecker(),f=e.map(d,function(e){return p.typeToTypeNode(p.getBaseTypeOfLiteralType(p.getTypeAtLocation(e)))}),g=e.map(d,function(n){return e.isIdentifier(n)?n.text:e.isPropertyAccessExpression(n)?n.name.text:void 0}),_=p.getContextualType(t),v=o?void 0:_&&p.typeToTypeNode(_,t)||e.createKeywordTypeNode(119);return e.createMethod(void 0,s?[e.createToken(115)]:void 0,e.isYieldExpression(m)?e.createToken(39):void 0,i,void 0,o?void 0:e.map(u,function(n,t){return e.createTypeParameterDeclaration(84+u.length-1<=90?String.fromCharCode(84+t):\"T\"+t)}),r(d.length,g,f,void 0,o),v,c?a(l):void 0)}}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t,r=\"inferFromUsage\",a=[e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code,e.Diagnostics.Variable_0_implicitly_has_an_1_type.code,e.Diagnostics.Parameter_0_implicitly_has_an_1_type.code,e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code,e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code,e.Diagnostics.Member_0_implicitly_has_an_1_type.code];function i(n,r,a,i,d,m,p){if(e.isParameterPropertyModifier(a.kind)||71===a.kind||24===a.kind){var f=a.parent;switch(i){case e.Diagnostics.Member_0_implicitly_has_an_1_type.code:case e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code:return e.isVariableDeclaration(f)&&p(f)||e.isPropertyDeclaration(f)||e.isPropertySignature(f)?(o(n,r,f,d,m),f):void 0;case e.Diagnostics.Variable_0_implicitly_has_an_1_type.code:var g=d.getTypeChecker().getSymbolAtLocation(a);return g&&g.valueDeclaration&&e.isVariableDeclaration(g.valueDeclaration)&&p(g.valueDeclaration)?(o(n,r,g.valueDeclaration,d,m),g.valueDeclaration):void 0}var _=e.getContainingFunction(a);if(void 0!==_)switch(i){case e.Diagnostics.Parameter_0_implicitly_has_an_1_type.code:if(e.isSetAccessor(_))return s(n,r,_,d,m),_;case e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code:if(p(_)){var v=e.cast(f,e.isParameter);return function(n,r,a,i,o,s){if(!e.isIdentifier(r.name)||!function(e){switch(e.kind){case 237:case 154:case 155:return!0;case 194:return!!e.name}return!1}(a))return;var d=function(n,r,a,i){switch(n.kind){case 155:case 194:case 237:case 154:var o=155===n.kind,s=o?e.findChildOfKind(n,123,r):n.name;if(s)return t.inferTypeForParametersFromReferences(c(s,a,i),n,a.getTypeChecker(),i)}}(a,i,o,s)||a.parameters.map(function(n){return e.isIdentifier(n.name)?u(n.name,o,s):void 0});if(!d||a.parameters.length!==d.length)return;e.zipWith(a.parameters,d,function(e,t){e.type||e.initializer||l(n,i,e,t,o)})}(n,v,_,r,d,m),v}return;case e.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code:case e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code:return e.isGetAccessor(_)&&e.isIdentifier(_.name)?(l(n,r,_,u(_.name,d,m),d),_):void 0;case e.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code:return e.isSetAccessor(_)?(s(n,r,_,d,m),_):void 0;default:return e.Debug.fail(String(i))}}}function o(n,t,r,a,i){e.isIdentifier(r.name)&&l(n,t,r,u(r.name,a,i),a)}function s(n,t,r,a,i){var o=e.firstOrUndefined(r.parameters);o&&e.isIdentifier(r.name)&&e.isIdentifier(o.name)&&l(n,t,o,u(r.name,a,i)||u(o.name,a,i),a)}function l(e,n,t,r,a){var i=r&&function(e,n,t){var r=!0,a=function(){r=!1},i=t.typeToTypeNode(e,n,void 0,{trackSymbol:function(e,n,a){r=r&&0===t.isSymbolAccessible(e,n,a,!1).accessibility},reportInaccessibleThisError:a,reportPrivateInBaseOfClassExpression:a,reportInaccessibleUniqueSymbolError:a});return r?i:void 0}(r,t,a.getTypeChecker());i&&e.tryInsertTypeAnnotation(n,t,i)}function c(n,t,r){return e.mapDefined(e.FindAllReferences.getReferenceEntriesForNode(-1,n,t,t.getSourceFiles(),r),function(n){return\"node\"===n.type?e.tryCast(n.node,e.isIdentifier):void 0})}function u(e,n,r){return t.inferTypeFromReferences(c(e,n,r),n.getTypeChecker(),r)}n.registerCodeFix({errorCodes:a,getCodeActions:function(t){var a=t.sourceFile,o=t.program,s=t.span.start,l=t.errorCode,c=t.cancellationToken;if(!e.isSourceFileJS(a)){var u,d=e.getTokenAtPosition(a,s),m=e.textChanges.ChangeTracker.with(t,function(n){u=i(n,a,d,l,o,c,e.returnTrue)}),p=u&&e.getNameOfDeclaration(u);return p&&0!==m.length?[n.createCodeFixAction(r,m,[function(n,t){switch(n){case e.Diagnostics.Parameter_0_implicitly_has_an_1_type.code:return e.isSetAccessor(e.getContainingFunction(t))?e.Diagnostics.Infer_type_of_0_from_usage:e.Diagnostics.Infer_parameter_types_from_usage;case e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code:return e.Diagnostics.Infer_parameter_types_from_usage;default:return e.Diagnostics.Infer_type_of_0_from_usage}}(l,d),p.getText(a)],r,e.Diagnostics.Infer_all_types_from_usage)]:void 0}},fixIds:[r],getAllCodeActions:function(t){var r=t.sourceFile,o=t.program,s=t.cancellationToken,l=e.nodeSeenTracker();return n.codeFixAll(t,a,function(n,t){i(n,r,e.getTokenAtPosition(t.file,t.start),t.code,o,s,l)})}}),function(n){function t(n,a,i){for(;e.isRightSideOfQualifiedNameOrPropertyAccess(n);)n=n.parent;switch(n.parent.kind){case 201:i.isNumber=!0;break;case 200:!function(e,n){switch(e.operator){case 43:case 44:case 38:case 52:n.isNumber=!0;break;case 37:n.isNumberOrString=!0}}(n.parent,i);break;case 202:!function(n,t,r,a){switch(t.operatorToken.kind){case 40:case 39:case 41:case 42:case 45:case 46:case 47:case 48:case 49:case 50:case 60:case 62:case 61:case 63:case 64:case 68:case 69:case 70:case 65:case 67:case 66:case 38:case 27:case 30:case 29:case 31:var i=r.getTypeAtLocation(t.left===n?t.right:t.left);544&i.flags?s(a,i):a.isNumber=!0;break;case 59:case 37:var o=r.getTypeAtLocation(t.left===n?t.right:t.left);544&o.flags?s(a,o):168&o.flags?a.isNumber=!0:68&o.flags?a.isString=!0:a.isNumberOrString=!0;break;case 58:case 32:case 34:case 35:case 33:s(a,r.getTypeAtLocation(t.left===n?t.right:t.left));break;case 92:n===t.left&&(a.isString=!0);break;case 54:n!==t.left||235!==n.parent.parent.kind&&!e.isAssignmentExpression(n.parent.parent,!0)||s(a,r.getTypeAtLocation(t.right))}}(n,n.parent,a,i);break;case 269:case 270:!function(e,n,t){s(t,n.getTypeAtLocation(e.parent.parent.expression))}(n.parent,a,i);break;case 189:case 190:n.parent.expression===n?function(e,n,r){var a={argumentTypes:[],returnType:{}};if(e.arguments)for(var i=0,o=e.arguments;i<o.length;i++){var s=o[i];a.argumentTypes.push(n.getTypeAtLocation(s))}t(e,n,a.returnType),189===e.kind?(r.callContexts||(r.callContexts=[])).push(a):(r.constructContexts||(r.constructContexts=[])).push(a)}(n.parent,a,i):r(n,a,i);break;case 187:!function(n,r,a){var i=e.escapeLeadingUnderscores(n.name.text);a.properties||(a.properties=e.createUnderscoreEscapedMap());var o=a.properties.get(i)||{};t(n,r,o),a.properties.set(i,o)}(n.parent,a,i);break;case 188:!function(e,n,r,a){if(n===e.argumentExpression)return void(a.isNumberOrString=!0);var i=r.getTypeAtLocation(e.argumentExpression),o={};t(e,r,o),168&i.flags?a.numberIndexContext=o:a.stringIndexContext=o}(n.parent,n,a,i);break;case 235:var o=n.parent,l=o.name,c=o.initializer;if(n===l){c&&s(i,a.getTypeAtLocation(c));break}default:return r(n,a,i)}}function r(n,t,r){e.isExpressionNode(n)&&s(r,t.getContextualType(n))}function a(n,t){if(n.isNumberOrString&&!n.isNumber&&!n.isString)return t.getUnionType([t.getNumberType(),t.getStringType()]);if(n.isNumber)return t.getNumberType();if(n.isString)return t.getStringType();if(n.candidateTypes)return t.getWidenedType(t.getUnionType(n.candidateTypes.map(function(e){return t.getBaseTypeOfLiteralType(e)}),2));if(n.properties&&l(n.properties.get(\"then\"))){var r=i(0,n.properties.get(\"then\").callContexts,!1,t).getCallSignatures().map(function(e){return e.getReturnType()});return t.createPromiseType(r.length?t.getUnionType(r,2):t.getAnyType())}if(n.properties&&l(n.properties.get(\"push\")))return t.createArrayType(i(0,n.properties.get(\"push\").callContexts,!1,t));if(n.numberIndexContext)return t.createArrayType(y(n.numberIndexContext));if(n.properties||n.callContexts||n.constructContexts||n.stringIndexContext){var s=e.createUnderscoreEscapedMap(),c=[],u=[],d=void 0;if(n.properties&&n.properties.forEach(function(e,n){var r=t.createSymbol(4,n);r.type=y(e),s.set(n,r)}),n.callContexts)for(var m=0,p=n.callContexts;m<p.length;m++){var f=p[m];c.push(o(f,t))}if(n.constructContexts)for(var g=0,_=n.constructContexts;g<_.length;g++){var v=_[g];u.push(o(v,t))}return n.stringIndexContext&&(d=t.createIndexInfo(y(n.stringIndexContext),!1)),t.createAnonymousType(void 0,s,c,u,d,void 0)}function y(e){return a(e,t)||t.getAnyType()}}function i(n,t,r,a){var i=[];if(t)for(var o=0,s=t;o<s.length;o++){var l=s[o];l.argumentTypes.length>n&&(r?i=e.concatenate(i,e.map(l.argumentTypes.slice(n),function(e){return a.getBaseTypeOfLiteralType(e)})):i.push(a.getBaseTypeOfLiteralType(l.argumentTypes[n])))}if(i.length){var c=a.getWidenedType(a.getUnionType(i,2));return r?a.createArrayType(c):c}}function o(n,t){for(var r=[],i=0;i<n.argumentTypes.length;i++){var o=t.createSymbol(1,e.escapeLeadingUnderscores(\"arg\"+i));o.type=t.getWidenedType(t.getBaseTypeOfLiteralType(n.argumentTypes[i])),r.push(o)}var s=a(n.returnType,t)||t.getVoidType();return t.createSignature(void 0,void 0,void 0,r,s,void 0,n.argumentTypes.length,!1,!1)}function s(e,n){!n||1&n.flags||32768&n.flags||(e.candidateTypes||(e.candidateTypes=[])).push(n)}function l(e){return!!e&&!!e.callContexts}n.inferTypeFromReferences=function(e,n,r){for(var i={},o=0,s=e;o<s.length;o++){var l=s[o];r.throwIfCancellationRequested(),t(l,n,i)}return a(i,n)},n.inferTypeForParametersFromReferences=function(n,r,a,i){if(0!==n.length&&r.parameters){for(var o={},s=0,l=n;s<l.length;s++){var c=l[s];i.throwIfCancellationRequested(),t(c,a,o)}var u=155===r.kind?o.constructContexts:o.callContexts;return u&&r.parameters.map(function(n,t){for(var r=[],i=e.isRestParameter(n),o=0,s=u;o<s.length;o++){var l=s[o];if(!(l.argumentTypes.length<=t))if(i)for(var c=t;c<l.argumentTypes.length;c++)r.push(a.getBaseTypeOfLiteralType(l.argumentTypes[c]));else r.push(a.getBaseTypeOfLiteralType(l.argumentTypes[t]))}if(r.length){var d=a.getWidenedType(a.getUnionType(r,2));return i?a.createArrayType(d):d}})}}}(t||(t={}))}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"invalidImportSyntax\";function r(r,a,i,o){var s=e.textChanges.ChangeTracker.with(r,function(e){return e.replaceNode(a,i,o)});return n.createCodeFixActionNoFixId(t,s,[e.Diagnostics.Replace_import_with_0,s[0].textChanges[0].newText])}function a(a,i){var o=a.program.getTypeChecker().getTypeAtLocation(i);if(!o.symbol||!o.symbol.originatingImport)return[];var s=[],l=o.symbol.originatingImport;if(e.isImportCall(l)||e.addRange(s,function(n,t){var a=e.getSourceFileOfNode(t),i=e.getNamespaceDeclarationNode(t),o=n.program.getCompilerOptions(),s=[];return s.push(r(n,a,t,e.makeImport(i.name,void 0,t.moduleSpecifier,e.getQuotePreference(a,n.preferences)))),e.getEmitModuleKind(o)===e.ModuleKind.CommonJS&&s.push(r(n,a,t,e.createImportEqualsDeclaration(void 0,void 0,i.name,e.createExternalModuleReference(t.moduleSpecifier)))),s}(a,l)),e.isExpression(i)&&(!e.isNamedDeclaration(i.parent)||i.parent.name!==i)){var c=a.sourceFile,u=e.textChanges.ChangeTracker.with(a,function(n){return n.replaceNode(c,i,e.createPropertyAccess(i,\"default\"),{})});s.push(n.createCodeFixActionNoFixId(t,u,e.Diagnostics.Use_synthetic_default_member))}return s}n.registerCodeFix({errorCodes:[e.Diagnostics.Cannot_invoke_an_expression_whose_type_lacks_a_call_signature_Type_0_has_no_compatible_call_signatures.code,e.Diagnostics.Cannot_use_new_with_an_expression_whose_type_lacks_a_call_or_construct_signature.code],getCodeActions:function(n){var t=n.sourceFile,r=e.Diagnostics.Cannot_invoke_an_expression_whose_type_lacks_a_call_signature_Type_0_has_no_compatible_call_signatures.code===n.errorCode?189:190,i=e.findAncestor(e.getTokenAtPosition(t,n.span.start),function(e){return e.kind===r&&e.getStart()===n.span.start&&e.getEnd()===n.span.start+n.span.length});if(!i)return[];var o=i.expression;return a(n,o)}}),n.registerCodeFix({errorCodes:[e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code,e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1.code,e.Diagnostics.Type_0_is_not_assignable_to_type_1.code,e.Diagnostics.Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated.code,e.Diagnostics.Type_predicate_0_is_not_assignable_to_1.code,e.Diagnostics.Property_0_of_type_1_is_not_assignable_to_string_index_type_2.code,e.Diagnostics.Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2.code,e.Diagnostics.Numeric_index_type_0_is_not_assignable_to_string_index_type_1.code,e.Diagnostics.Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2.code,e.Diagnostics.Property_0_in_type_1_is_not_assignable_to_type_2.code,e.Diagnostics.Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property.code,e.Diagnostics.The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1.code],getCodeActions:function(n){var t=n.sourceFile,r=e.findAncestor(e.getTokenAtPosition(t,n.span.start),function(e){return e.getStart()===n.span.start&&e.getEnd()===n.span.start+n.span.length});if(!r)return[];return a(n,r)}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"strictClassInitialization\",r=\"addMissingPropertyDefiniteAssignmentAssertions\",a=\"addMissingPropertyUndefinedType\",i=\"addMissingPropertyInitializer\",o=[e.Diagnostics.Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor.code];function s(n,t){var r=e.getTokenAtPosition(n,t);return e.isIdentifier(r)?e.cast(r.parent,e.isPropertyDeclaration):void 0}function l(n,t,r){var a=e.updateProperty(r,r.decorators,r.modifiers,r.name,e.createToken(51),r.type,r.initializer);n.replaceNode(t,r,a)}function c(n,t,r){var a=e.createKeywordTypeNode(140),i=r.type,o=e.isUnionTypeNode(i)?i.types.concat(a):[i,a];n.replaceNode(t,i,e.createUnionTypeNode(o))}function u(n,t,r,a){var i=e.updateProperty(r,r.decorators,r.modifiers,r.name,r.questionToken,r.type,a);n.replaceNode(t,r,i)}function d(n,t){return function n(t,r){if(256&r.flags)return r===t.getFalseType()||r===t.getFalseType(!0)?e.createFalse():e.createTrue();if(r.isLiteral())return e.createLiteral(r.value);if(r.isUnion())return e.firstDefined(r.types,function(e){return n(t,e)});if(r.isClass()){var a=e.getClassLikeDeclarationOfSymbol(r.symbol);if(!a||e.hasModifier(a,128))return;var i=e.getFirstConstructorWithBody(a);if(i&&i.parameters.length)return;return e.createNew(e.createIdentifier(r.symbol.name),void 0,void 0)}if(t.isArrayLikeType(r))return e.createArrayLiteral();return}(n,n.getTypeFromTypeNode(t.type))}n.registerCodeFix({errorCodes:o,getCodeActions:function(o){var m=s(o.sourceFile,o.span.start);if(m){var p=[function(r,i){var o=e.textChanges.ChangeTracker.with(r,function(e){return c(e,r.sourceFile,i)});return n.createCodeFixAction(t,o,[e.Diagnostics.Add_undefined_type_to_property_0,i.name.getText()],a,e.Diagnostics.Add_undefined_type_to_all_uninitialized_properties)}(o,m),function(a,i){var o=e.textChanges.ChangeTracker.with(a,function(e){return l(e,a.sourceFile,i)});return n.createCodeFixAction(t,o,[e.Diagnostics.Add_definite_assignment_assertion_to_property_0,i.getText()],r,e.Diagnostics.Add_definite_assignment_assertions_to_all_uninitialized_properties)}(o,m)];return e.append(p,function(r,a){var o=d(r.program.getTypeChecker(),a);if(!o)return;var s=e.textChanges.ChangeTracker.with(r,function(e){return u(e,r.sourceFile,a,o)});return n.createCodeFixAction(t,s,[e.Diagnostics.Add_initializer_to_property_0,a.name.getText()],i,e.Diagnostics.Add_initializers_to_all_uninitialized_properties)}(o,m)),p}},fixIds:[r,a,i],getAllCodeActions:function(t){return n.codeFixAll(t,o,function(n,o){var m=s(o.file,o.start);if(m)switch(t.fixId){case r:l(n,o.file,m);break;case a:c(n,o.file,m);break;case i:var p=d(t.program.getTypeChecker(),m);if(!p)return;u(n,o.file,m,p);break;default:e.Debug.fail(JSON.stringify(t.fixId))}})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){function n(n,t){return e.textChanges.getNewFileText(a(n,0),3,\"\\n\",e.formatting.getFormatContext(t))}var t;function r(e){return a(e,2)}function a(n,t){var c=\"default\"===n.name,d=c?\"_default\":n.name;if(!p(d)||c&&1!==t)return e.emptyArray;var v=c&&2===n.kind?[e.createModifier(84),e.createModifier(79)]:0===t?[e.createModifier(124)]:1===t?[e.createModifier(84)]:void 0,y=function(){return 0===t?[i(n.name,!0)]:e.emptyArray},h=function(){return c?[i(\"_default\",!1)]:e.emptyArray};switch(n.kind){case 2:return y().concat(function(n,t,r){var i=r.source,c=r.prototypeMembers,d=r.namespaceMembers,v=u(i),y=void 0===v?{parameters:e.emptyArray,returnType:g()}:l(v),h=y.parameters,b=y.returnType,E=\"object\"==typeof v?function(n){var t=[];return m(n,function(n){if(e.isAssignmentExpression(n,!0)&&e.isPropertyAccessExpression(n.left)&&99===n.left.expression.kind){var r=n.left.name.text;e.isJsPrivate(r)||t.push(e.createProperty(void 0,void 0,r,void 0,g(),void 0))}}),t}(v):e.emptyArray,T=0!==E.length||0!==c.length||void 0===v||\"number\"!=typeof v&&155===v.kind?[]:void 0,S=e.flatMap(d,function(n){if(p(n.name)){if(T)switch(n.kind){case 3:if(n.members.some(function(e){return 2===e.kind}))break;case 1:case 0:return void T.push(f(e.createProperty(void 0,[e.createModifier(115)],n.name,void 0,s(n),void 0),0===n.kind?n.comment:void 0));case 2:if(!n.namespaceMembers.length){var t=o(n,[e.createModifier(115)]);if(t)return void T.push(t)}}return a(n,2)}});return[T?e.createClassDeclaration(void 0,n,t,void 0,void 0,T.concat(h.length?[e.createConstructor(void 0,void 0,h,void 0)]:e.emptyArray,E,e.mapDefined(c,function(e){return 2===e.kind?o(e):void 0}))):e.createFunctionDeclaration(void 0,n,void 0,t,void 0,h,b,void 0)].concat(0===S.length?e.emptyArray:[_(n&&n.map(function(n){return e.getSynthesizedDeepClone(n)}),t,S)])}(v,d,n));case 3:var b=n.members;if(0===t)return e.flatMap(b,function(e){return a(e,1)});if(b.some(function(e){return 2===e.kind}))return h().concat([_(v,d,e.flatMap(b,r))]);case 0:case 1:var E=0===n.kind?n.comment:void 0,T=e.createVariableStatement(v,e.createVariableDeclarationList([e.createVariableDeclaration(d,s(n))],2));return y().concat(h(),[f(T,E)]);default:return e.Debug.assertNever(n)}}function i(n,t){return e.createExportAssignment(void 0,void 0,t,e.createIdentifier(n))}function o(n,t){var r=n.name,a=n.source;if(p(r)){var i=u(a);if(void 0!==i&&(\"number\"==typeof i||155!==i.kind)){var o=l(i);return o&&e.createMethod(void 0,t,void 0,r,void 0,void 0,o.parameters,o.returnType,void 0)}}}function s(n){switch(n.kind){case 0:return e.createTypeReferenceNode(n.typeName,void 0);case 1:return e.createArrayTypeNode(s(n.inner));case 2:return e.createTypeReferenceNode(\"Function\",void 0);case 3:return e.createTypeLiteralNode(n.members.map(function(n){return e.createPropertySignature(void 0,n.name,void 0,s(n),void 0)}));default:return e.Debug.assertNever(n)}}function l(n){if(\"number\"==typeof n)return{parameters:e.fill(n,function(e){return c(\"p\"+e,g())}),returnType:g()};var t=!1,r=!1;return m(n,function(n){t=t||e.isIdentifier(n)&&\"arguments\"===n.text,r=r||e.isReturnStatement(n)&&!!n.expression&&198!==n.expression.kind}),{parameters:n.parameters.map(function(e){return c(\"\"+e.name.getText(),g())}).concat(t?[e.createParameter(void 0,void 0,e.createToken(24),\"args\",void 0,e.createArrayTypeNode(g()))]:e.emptyArray),returnType:r?g():e.createKeywordTypeNode(105)}}function c(n,t){return e.createParameter(void 0,void 0,void 0,n,void 0,t)}function u(n){if(\"number\"==typeof n)return n;var t=e.tryCast(d(n),function(n){return e.isFunctionExpression(n)||e.isArrowFunction(n)||e.isClassExpression(n)});return t?e.isClassExpression(t)?e.find(t.members,e.isConstructorDeclaration):t:e.cast(e.first(e.cast(d(\"{ \"+n+\" }\"),e.isObjectLiteralExpression).properties),e.isMethodDeclaration)}function d(n){var t=\"const _ = \"+n,r=e.createSourceFile(\"test.ts\",t,6,!0);return e.first(e.cast(e.first(r.statements),e.isVariableStatement).declarationList.declarations).initializer}function m(n,t){n.body.forEachChild(function n(r){t(r),e.isFunctionLike(r)||r.forEachChild(n)})}function p(n){var t=e.stringToToken(n);return!(t&&e.isNonContextualKeyword(t))&&e.isIdentifierText(n,6)}function f(n,t){return void 0!==t&&e.addSyntheticLeadingComment(n,2,t),n}function g(){return e.createKeywordTypeNode(119)}function _(n,t,r){return e.createModuleDeclaration(void 0,n,e.createIdentifier(t),e.createModuleBlock(r),16)}e.generateTypesForModule=function(t,r,a){return n(e.inspectValue(t,r),a)},e.valueInfoToDeclarationFileText=n,function(e){e[e.ExportEquals=0]=\"ExportEquals\",e[e.NamedExport=1]=\"NamedExport\",e[e.NamespaceMember=2]=\"NamespaceMember\"}(t||(t={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"requireInTs\",r=[e.Diagnostics.require_call_may_be_converted_to_an_import.code];function a(n,t,r,a){var i=function(n,t){var r=e.getTokenAtPosition(n,t).parent;if(!e.isRequireCall(r,!0))throw e.Debug.failBadSyntaxKind(r);var a=e.cast(r.parent,e.isVariableDeclaration);return{statement:e.cast(a.parent.parent,e.isVariableStatement),name:e.cast(a.name,e.isIdentifier),required:r.arguments[0]}}(t,r),o=i.statement,s=i.name,l=i.required;n.replaceNode(t,o,e.getAllowSyntheticDefaultImports(a.getCompilerOptions())?e.createImportDeclaration(void 0,void 0,e.createImportClause(s,void 0),l):e.createImportEqualsDeclaration(void 0,void 0,s,e.createExternalModuleReference(l)))}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=e.textChanges.ChangeTracker.with(r,function(e){return a(e,r.sourceFile,r.span.start,r.program)});return[n.createCodeFixAction(t,i,e.Diagnostics.Convert_require_to_import,t,e.Diagnostics.Convert_all_require_to_import)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return a(n,t.file,t.start,e.program)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"useDefaultImport\",r=[e.Diagnostics.Import_may_be_converted_to_a_default_import.code];function a(n,t){var r=e.getTokenAtPosition(n,t);if(e.isIdentifier(r)){var a=r.parent;if(e.isImportEqualsDeclaration(a)&&e.isExternalModuleReference(a.moduleReference))return{importNode:a,name:r,moduleSpecifier:a.moduleReference.expression};if(e.isNamespaceImport(a)){var i=a.parent.parent;return{importNode:i,name:r,moduleSpecifier:i.moduleSpecifier}}}}function i(n,t,r,a){n.replaceNode(t,r.importNode,e.makeImport(r.name,void 0,r.moduleSpecifier,e.getQuotePreference(t,a)))}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span.start,l=a(o,s);if(l){var c=e.textChanges.ChangeTracker.with(r,function(e){return i(e,o,l,r.preferences)});return[n.createCodeFixAction(t,c,e.Diagnostics.Convert_to_default_import,t,e.Diagnostics.Convert_all_to_default_imports)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){var r=a(t.file,t.start);r&&i(n,t.file,r,e.preferences)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"fixAddModuleReferTypeMissingTypeof\",r=[e.Diagnostics.Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0.code];function a(n,t){var r=e.getTokenAtPosition(n,t);return e.Debug.assert(91===r.kind),e.Debug.assert(181===r.parent.kind),r.parent}function i(n,t,r){var a=e.updateImportTypeNode(r,r.argument,r.qualifier,r.typeArguments,!0);n.replaceNode(t,r,a)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,l=a(o,s.start),c=e.textChanges.ChangeTracker.with(r,function(e){return i(e,o,l)});return[n.createCodeFixAction(t,c,e.Diagnostics.Add_missing_typeof,t,e.Diagnostics.Add_missing_typeof)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return i(n,e.sourceFile,a(t.file,t.start))})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){var t=\"fixConvertToMappedObjectType\",r=[e.Diagnostics.An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead.code];function a(n,t){var r=e.getTokenAtPosition(n,t),a=e.cast(r.parent.parent,e.isIndexSignatureDeclaration);if(!e.isClassDeclaration(a.parent))return{indexSignature:a,container:e.isInterfaceDeclaration(a.parent)?a.parent:e.cast(a.parent.parent,e.isTypeAliasDeclaration)}}function i(n,t,r){var a,i,o=r.indexSignature,s=r.container,l=(e.isInterfaceDeclaration(s)?s.members:s.type.members).filter(function(n){return!e.isIndexSignatureDeclaration(n)}),c=e.first(o.parameters),u=e.createTypeParameterDeclaration(e.cast(c.name,e.isIdentifier),c.type),d=e.createMappedTypeNode(e.hasReadonlyModifier(o)?e.createModifier(132):void 0,u,o.questionToken,o.type),m=e.createIntersectionTypeNode(e.getAllSuperTypeNodes(s).concat([d],l.length?[e.createTypeLiteralNode(l)]:e.emptyArray));n.replaceNode(t,s,(a=s,i=m,e.createTypeAliasDeclaration(a.decorators,a.modifiers,a.name,a.typeParameters,i)))}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,l=a(o,s.start);if(l){var c=e.textChanges.ChangeTracker.with(r,function(e){return i(e,o,l)}),u=e.idText(l.container.name);return[n.createCodeFixAction(t,c,[e.Diagnostics.Convert_0_to_mapped_object_type,u],t,[e.Diagnostics.Convert_0_to_mapped_object_type,u])]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=a(n.file,n.start);t&&i(e,n.file,t)})}})}(e.codefix||(e.codefix={}))}(Tn||(Tn={})),function(e){!function(n){function t(n){var t=n.file,r=e.getRefactorContextSpan(n),a=e.getTokenAtPosition(t,r.start),i=e.getParentNodeInSpan(a,t,r);if(i&&(e.isSourceFile(i.parent)||e.isModuleBlock(i.parent)&&e.isAmbientModule(i.parent.parent))){var o=e.isSourceFile(i.parent)?i.parent.symbol:i.parent.parent.symbol,s=e.getModifierFlags(i),l=!!(512&s);if(1&s&&(l||!o.exports.has(\"default\")))switch(i.kind){case 237:case 238:case 239:case 241:case 240:case 242:var c=i;return c.name&&e.isIdentifier(c.name)?{exportNode:c,exportName:c.name,wasDefault:l,exportingModuleSymbol:o}:void 0;case 217:var u=i;if(!(2&u.declarationList.flags)||1!==u.declarationList.declarations.length)return;var d=e.first(u.declarationList.declarations);if(!d.initializer)return;return e.Debug.assert(!l),e.isIdentifier(d.name)?{exportNode:u,exportName:d.name,wasDefault:l,exportingModuleSymbol:o}:void 0;default:return}}}function r(n,t){return e.createImportSpecifier(n===t?void 0:e.createIdentifier(n),e.createIdentifier(t))}n.registerRefactor(\"Convert export\",{getAvailableActions:function(n){var r=t(n);if(r){var a=r.wasDefault?e.Diagnostics.Convert_default_export_to_named_export.message:e.Diagnostics.Convert_named_export_to_default_export.message;return[{name:\"Convert export\",description:a,actions:[{name:r.wasDefault?\"Convert default export to named export\":\"Convert named export to default export\",description:a}]}]}},getEditsForAction:function(n,a){return e.Debug.assert(\"Convert default export to named export\"===a||\"Convert named export to default export\"===a),{edits:e.textChanges.ChangeTracker.with(n,function(a){return i=n.file,o=n.program,s=e.Debug.assertDefined(t(n)),l=a,c=n.cancellationToken,function(n,t,r,a){var i=t.wasDefault,o=t.exportNode,s=t.exportName;if(i)r.delete(n,e.Debug.assertDefined(e.findModifier(o,79)));else{var l=e.Debug.assertDefined(e.findModifier(o,84));switch(o.kind){case 237:case 238:case 239:r.insertNodeAfter(n,l,e.createToken(79));break;case 217:if(!e.FindAllReferences.Core.isSymbolReferencedInFile(s,a,n)){r.replaceNode(n,o,e.createExportDefault(e.Debug.assertDefined(e.first(o.declarationList.declarations).initializer)));break}case 241:case 240:case 242:r.deleteModifier(n,l),r.insertNodeAfter(n,o,e.createExportDefault(e.createIdentifier(s.text)));break;default:e.Debug.assertNever(o)}}}(i,s,l,o.getTypeChecker()),void function(n,t,a,i){var o=t.wasDefault,s=t.exportName,l=t.exportingModuleSymbol,c=n.getTypeChecker(),u=e.Debug.assertDefined(c.getSymbolAtLocation(s));e.FindAllReferences.Core.eachExportReference(n.getSourceFiles(),c,i,u,l,s.text,o,function(n){var t=n.getSourceFile();o?function(n,t,a,i){var o=t.parent;switch(o.kind){case 187:a.replaceNode(n,t,e.createIdentifier(i));break;case 251:case 255:var s=o;a.replaceNode(n,s,r(i,s.name.text));break;case 248:var l=o;e.Debug.assert(l.name===t);var s=r(i,t.text),c=l.namedBindings;if(c)if(249===c.kind){a.deleteRange(n,{pos:t.getStart(n),end:c.getStart(n)});var u=e.isStringLiteral(l.parent.moduleSpecifier)?e.quotePreferenceFromString(l.parent.moduleSpecifier,n):1,d=e.makeImport(void 0,[r(i,t.text)],l.parent.moduleSpecifier,u);a.insertNodeAfter(n,l.parent,d)}else a.delete(n,t),a.insertNodeAtEndOfList(n,c.elements,s);else a.replaceNode(n,t,e.createNamedImports([s]));break;default:e.Debug.failBadSyntaxKind(o)}}(t,n,a,s.text):function(n,t,r){var a,i,o=t.parent;switch(o.kind){case 187:r.replaceNode(n,t,e.createIdentifier(\"default\"));break;case 251:case 255:var s=o;if(251===s.kind){var l=e.createIdentifier(s.name.text);1===s.parent.elements.length?r.replaceNode(n,s.parent,l):(r.delete(n,s),r.insertNodeBefore(n,s.parent,l))}else r.replaceNode(n,s,(a=\"default\",i=s.name.text,e.createExportSpecifier(a===i?void 0:e.createIdentifier(a),e.createIdentifier(i))));break;default:e.Debug.failBadSyntaxKind(o)}}(t,n,a)})}(o,s,l,c);var i,o,s,l,c}),renameFilename:void 0,renameLocation:void 0}}})}(e.refactor||(e.refactor={}))}(Tn||(Tn={})),function(e){!function(n){function t(n){var t=n.file,r=e.getRefactorContextSpan(n),a=e.getTokenAtPosition(t,r.start),i=e.getParentNodeInSpan(a,t,r);if(i&&e.isImportDeclaration(i)){var o=i.importClause;return o&&o.namedBindings}}function r(n,t,r){return e.createImportDeclaration(void 0,void 0,e.createImportClause(t,r&&r.length?e.createNamedImports(r):void 0),n.moduleSpecifier)}n.registerRefactor(\"Convert import\",{getAvailableActions:function(n){var r=t(n);if(r){var a=249===r.kind?e.Diagnostics.Convert_namespace_import_to_named_imports.message:e.Diagnostics.Convert_named_imports_to_namespace_import.message;return[{name:\"Convert import\",description:a,actions:[{name:249===r.kind?\"Convert namespace import to named imports\":\"Convert named imports to namespace import\",description:a}]}]}},getEditsForAction:function(n,a){return e.Debug.assert(\"Convert namespace import to named imports\"===a||\"Convert named imports to namespace import\"===a),{edits:e.textChanges.ChangeTracker.with(n,function(a){return i=n.file,o=n.program,s=a,l=e.Debug.assertDefined(t(n)),c=o.getTypeChecker(),void(249===l.kind?function(n,t,a,i,o){var s=!1,l=[],c=e.createMap();e.FindAllReferences.Core.eachSymbolReferenceInFile(i.name,t,n,function(n){if(e.isPropertyAccessExpression(n.parent)){var r=e.cast(n.parent,e.isPropertyAccessExpression),a=r.name.text;t.resolveName(a,n,67108863,!0)&&c.set(a,!0),e.Debug.assert(r.expression===n),l.push(r)}else s=!0});for(var u=e.createMap(),d=0,m=l;d<m.length;d++){var p=m[d],f=p.name.text,g=u.get(f);void 0===g&&u.set(f,g=c.has(f)?e.getUniqueName(f,n):f),a.replaceNode(n,p,e.createIdentifier(g))}var _=[];u.forEach(function(n,t){_.push(e.createImportSpecifier(n===t?void 0:e.createIdentifier(t),e.createIdentifier(n)))});var v=i.parent.parent;s&&!o?a.insertNodeAfter(n,v,r(v,void 0,_)):a.replaceNode(n,v,r(v,s?e.createIdentifier(i.name.text):void 0,_))}(i,c,s,l,e.getAllowSyntheticDefaultImports(o.getCompilerOptions())):function(n,t,a,i){for(var o=i.parent.parent,s=o.moduleSpecifier,l=s&&e.isStringLiteral(s)?e.codefix.moduleSpecifierToValidIdentifier(s.text,6):\"module\",c=i.elements.some(function(r){return e.FindAllReferences.Core.eachSymbolReferenceInFile(r.name,t,n,function(e){return!!t.resolveName(l,e,67108863,!0)})||!1})?e.getUniqueName(l,n):l,u=[],d=function(r){var i=(r.propertyName||r.name).text;e.FindAllReferences.Core.eachSymbolReferenceInFile(r.name,t,n,function(t){var o=e.createPropertyAccess(e.createIdentifier(c),i);e.isShorthandPropertyAssignment(t.parent)?a.replaceNode(n,t.parent,e.createPropertyAssignment(t.text,o)):e.isExportSpecifier(t.parent)&&!t.parent.propertyName?u.some(function(e){return e.name===r.name})||u.push(e.createImportSpecifier(r.propertyName&&e.createIdentifier(r.propertyName.text),e.createIdentifier(r.name.text))):a.replaceNode(n,t,o)})},m=0,p=i.elements;m<p.length;m++){var f=p[m];d(f)}a.replaceNode(n,i,e.createNamespaceImport(e.createIdentifier(c))),u.length&&a.insertNodeAfter(n,i.parent.parent,r(o,void 0,u))}(i,c,s,l));var i,o,s,l,c}),renameFilename:void 0,renameLocation:void 0}}})}(e.refactor||(e.refactor={}))}(Tn||(Tn={})),function(e){!function(n){!function(t){var r,a,i,o,s=\"Extract Symbol\";function l(n){var t=u(n.file,e.getRefactorContextSpan(n)).targetRange;if(void 0!==t){var r=function(n,t){var r=m(n,t),a=r.scopes,i=r.readsAndWrites,o=i.functionErrorsPerScope,s=i.constantErrorsPerScope;return a.map(function(n,t){var r,a,i=function(n){return e.isFunctionLikeDeclaration(n)?\"inner function\":e.isClassLike(n)?\"method\":\"function\"}(n),l=function(n){return e.isClassLike(n)?\"readonly field\":\"constant\"}(n),c=e.isFunctionLikeDeclaration(n)?function(n){switch(n.kind){case 155:return\"constructor\";case 194:case 237:return n.name?\"function '\"+n.name.text+\"'\":\"anonymous function\";case 195:return\"arrow function\";case 154:return\"method '\"+n.name.getText();case 156:return\"'get \"+n.name.getText()+\"'\";case 157:return\"'set \"+n.name.getText()+\"'\";default:throw e.Debug.assertNever(n)}}(n):e.isClassLike(n)?function(e){return 238===e.kind?e.name?\"class '\"+e.name.text+\"'\":\"anonymous class declaration\":e.name?\"class expression '\"+e.name.text+\"'\":\"anonymous class expression\"}(n):function(e){return 243===e.kind?\"namespace '\"+e.parent.name.getText()+\"'\":e.externalModuleIndicator?0:1}(n);return 1===c?(r=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[i,\"global\"]),a=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[l,\"global\"])):0===c?(r=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[i,\"module\"]),a=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[l,\"module\"])):(r=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1),[i,c]),a=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1),[l,c])),0!==t||e.isClassLike(n)||(a=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_enclosing_scope),[l])),{functionExtraction:{description:r,errors:o[t]},constantExtraction:{description:a,errors:s[t]}}})}(t,n);if(void 0!==r){for(var a=[],i=e.createMap(),o=[],l=e.createMap(),c=0,d=0,p=r;d<p.length;d++){var f=p[d],g=f.functionExtraction,_=f.constantExtraction;if(0===g.errors.length){var v=g.description;i.has(v)||(i.set(v,!0),a.push({description:v,name:\"function_scope_\"+c}))}if(0===_.errors.length){v=_.description;l.has(v)||(l.set(v,!0),o.push({description:v,name:\"constant_scope_\"+c}))}c++}var y=[];return a.length&&y.push({name:s,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_function),actions:a}),o.length&&y.push({name:s,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_constant),actions:o}),y.length?y:void 0}}}function c(n,t){var r=u(n.file,e.getRefactorContextSpan(n)).targetRange,i=/^function_scope_(\\d+)$/.exec(t);if(i){var o=+i[1];return e.Debug.assert(isFinite(o),\"Expected to parse a finite number from the function scope index\"),function(n,t,r){var i=m(n,t),o=i.scopes,s=i.readsAndWrites,l=s.target,c=s.usagesPerScope,u=s.functionErrorsPerScope,d=s.exposedVariableDeclarations;return e.Debug.assert(!u[r].length,\"The extraction went missing? How?\"),t.cancellationToken.throwIfCancellationRequested(),function(n,t,r,i,o,s){var l,c,u=r.usages,d=r.typeParameterUsages,m=r.substitutions,_=s.program.getTypeChecker(),v=t.getSourceFile(),y=e.getUniqueName(e.isClassLike(t)?\"newMethod\":\"newFunction\",v),h=e.isInJSFile(t),b=e.createIdentifier(y),E=[],T=[];u.forEach(function(n,r){var a;if(!h){var i=_.getTypeOfSymbolAtLocation(n.symbol,n.node);i=_.getBaseTypeOfLiteralType(i),a=_.typeToTypeNode(i,t,1)}var o=e.createParameter(void 0,void 0,void 0,r,void 0,a);E.push(o),2===n.usage&&(c||(c=[])).push(n),T.push(e.createIdentifier(r))});var S=e.arrayFrom(d.values()).map(function(e){return{type:e,declaration:function(e){var n,t=e.symbol;if(t&&t.declarations)for(var r=0,a=t.declarations;r<a.length;r++){var i=a[r];(void 0===n||i.pos<n.pos)&&(n=i)}return n}(e)}}).sort(p),L=0===S.length?void 0:S.map(function(e){return e.declaration}),x=void 0!==L?L.map(function(n){return e.createTypeReferenceNode(n.name,void 0)}):void 0;if(e.isExpression(n)&&!h){var A=_.getContextualType(n);l=_.typeToTypeNode(A,t,1)}var C,D=function(n,t,r,a,i){var o,s=void 0!==r||t.length>0;if(e.isBlock(n)&&!s&&0===a.size)return{body:e.createBlock(n.statements,!0),returnValueProperty:void 0};var l=!1,c=e.createNodeArray(e.isBlock(n)?n.statements.slice(0):[e.isStatement(n)?n:e.createReturn(n)]);if(s||a.size){var u=e.visitNodes(c,function n(i){if(!l&&228===i.kind&&s){var c=f(t,r);return i.expression&&(o||(o=\"__return\"),c.unshift(e.createPropertyAssignment(o,e.visitNode(i.expression,n)))),1===c.length?e.createReturn(c[0].name):e.createReturn(e.createObjectLiteral(c))}var u=l;l=l||e.isFunctionLikeDeclaration(i)||e.isClassLike(i);var d=a.get(e.getNodeId(i).toString()),m=d?e.getSynthesizedDeepClone(d):e.visitEachChild(i,n,e.nullTransformationContext);return l=u,m}).slice();if(s&&!i&&e.isStatement(n)){var d=f(t,r);1===d.length?u.push(e.createReturn(d[0].name)):u.push(e.createReturn(e.createObjectLiteral(d)))}return{body:e.createBlock(u,!0),returnValueProperty:o}}return{body:e.createBlock(c,!0),returnValueProperty:void 0}}(n,i,c,m,!!(o.facts&a.HasReturn)),k=D.body,M=D.returnValueProperty;if(e.suppressLeadingAndTrailingTrivia(k),e.isClassLike(t)){var O=h?[]:[e.createToken(112)];o.facts&a.InStaticRegion&&O.push(e.createToken(115)),o.facts&a.IsAsyncFunction&&O.push(e.createToken(120)),C=e.createMethod(void 0,O.length?O:void 0,o.facts&a.IsGenerator?e.createToken(39):void 0,b,void 0,L,E,l,k)}else C=e.createFunctionDeclaration(void 0,o.facts&a.IsAsyncFunction?[e.createToken(120)]:void 0,o.facts&a.IsGenerator?e.createToken(39):void 0,b,L,E,l,k);var I=e.textChanges.ChangeTracker.fromContext(s),R=function(n,t){return e.find(function(n){if(e.isFunctionLikeDeclaration(n)){var t=n.body;if(e.isBlock(t))return t.statements}else{if(e.isModuleBlock(n)||e.isSourceFile(n))return n.statements;if(e.isClassLike(n))return n.members;e.assertType(n)}return e.emptyArray}(t),function(t){return t.pos>=n&&e.isFunctionLikeDeclaration(t)&&!e.isConstructorDeclaration(t)})}((g(o.range)?e.last(o.range):o.range).end,t);R?I.insertNodeBefore(s.file,R,C,!0):I.insertNodeAtEndOfScope(s.file,t,C);var N=[],w=function(n,t,r){var i=e.createIdentifier(r);if(e.isClassLike(n)){var o=t.facts&a.InStaticRegion?e.createIdentifier(n.name.text):e.createThis();return e.createPropertyAccess(o,i)}return i}(t,o,y),P=e.createCall(w,x,T);if(o.facts&a.IsGenerator&&(P=e.createYield(e.createToken(39),P)),o.facts&a.IsAsyncFunction&&(P=e.createAwait(P)),i.length&&!c)if(e.Debug.assert(!M),e.Debug.assert(!(o.facts&a.HasReturn)),1===i.length){var F=i[0];N.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(e.getSynthesizedDeepClone(F.name),e.getSynthesizedDeepClone(F.type),P)],F.parent.flags)))}else{for(var G=[],V=[],K=i[0].parent.flags,B=!1,H=0,U=i;H<U.length;H++){var F=U[H];G.push(e.createBindingElement(void 0,void 0,e.getSynthesizedDeepClone(F.name)));var j=_.typeToTypeNode(_.getBaseTypeOfLiteralType(_.getTypeAtLocation(F)),t,1);V.push(e.createPropertySignature(void 0,F.symbol.name,void 0,j,void 0)),B=B||void 0!==F.type,K&=F.parent.flags}var W=B?e.createTypeLiteralNode(V):void 0;W&&e.setEmitFlags(W,1),N.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(e.createObjectBindingPattern(G),W,P)],K)))}else if(i.length||c){if(i.length)for(var q=0,z=i;q<z.length;q++){var F=z[q],J=F.parent.flags;2&J&&(J=-3&J|1),N.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(F.symbol.name,$(F.type))],J)))}M&&N.push(e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(M,$(l))],1)));var X=f(i,c);M&&X.unshift(e.createShorthandPropertyAssignment(M)),1===X.length?(e.Debug.assert(!M),N.push(e.createStatement(e.createAssignment(X[0].name,P))),o.facts&a.HasReturn&&N.push(e.createReturn())):(N.push(e.createStatement(e.createAssignment(e.createObjectLiteral(X),P))),M&&N.push(e.createReturn(e.createIdentifier(M))))}else o.facts&a.HasReturn?N.push(e.createReturn(P)):g(o.range)?N.push(e.createStatement(P)):N.push(P);g(o.range)?I.replaceNodeRangeWithNodes(s.file,e.first(o.range),e.last(o.range),N):I.replaceNodeWithNodes(s.file,o.range,N);var Y=I.getChanges(),Q=(g(o.range)?e.first(o.range):o.range).getSourceFile().fileName,Z=e.getRenameLocation(Y,Q,y,!1);return{renameFilename:Q,renameLocation:Z,edits:Y};function $(n){if(void 0!==n){for(var t=e.getSynthesizedDeepClone(n),r=t;e.isParenthesizedTypeNode(r);)r=r.type;return e.isUnionTypeNode(r)&&e.find(r.types,function(e){return 140===e.kind})?t:e.createUnionTypeNode([t,e.createKeywordTypeNode(140)])}}}(l,o[r],c[r],d,n,t)}(r,n,o)}var s=/^constant_scope_(\\d+)$/.exec(t);if(s){o=+s[1];return e.Debug.assert(isFinite(o),\"Expected to parse a finite number from the constant scope index\"),function(n,t,r){var i=m(n,t),o=i.scopes,s=i.readsAndWrites,l=s.target,c=s.usagesPerScope,u=s.constantErrorsPerScope,p=s.exposedVariableDeclarations;return e.Debug.assert(!u[r].length,\"The extraction went missing? How?\"),e.Debug.assert(0===p.length,\"Extract constant accepted a range containing a variable declaration?\"),t.cancellationToken.throwIfCancellationRequested(),function(n,t,r,i,o){var s=r.substitutions,l=o.program.getTypeChecker(),c=t.getSourceFile(),u=e.getUniqueName(e.isClassLike(t)?\"newProperty\":\"newLocal\",c),m=e.isInJSFile(t),p=m||!l.isContextSensitive(n)?void 0:l.typeToTypeNode(l.getContextualType(n),t,1),f=function(n,t){return t.size?function n(r){var a=t.get(e.getNodeId(r).toString());return a?e.getSynthesizedDeepClone(a):e.visitEachChild(r,n,e.nullTransformationContext)}(n):n}(n,s);e.suppressLeadingAndTrailingTrivia(f);var g=e.textChanges.ChangeTracker.fromContext(o);if(e.isClassLike(t)){e.Debug.assert(!m);var v=[];v.push(e.createToken(112)),i&a.InStaticRegion&&v.push(e.createToken(115)),v.push(e.createToken(132));var y=e.createProperty(void 0,v,u,void 0,p,f),h=e.createPropertyAccess(i&a.InStaticRegion?e.createIdentifier(t.name.getText()):e.createThis(),e.createIdentifier(u)),b=n.pos,E=function(n,t){var r,a=t.members;e.Debug.assert(a.length>0);for(var i=!0,o=0,s=a;o<s.length;o++){var l=s[o];if(l.pos>n)return r||a[0];if(i&&!e.isPropertyDeclaration(l)){if(void 0!==r)return l;i=!1}r=l}return void 0===r?e.Debug.fail():r}(b,t);g.insertNodeBefore(o.file,E,y,!0),g.replaceNode(o.file,n,h)}else{var T=e.createVariableDeclaration(u,p,f),S=function(n,t){for(var r;void 0!==n&&n!==t;){if(e.isVariableDeclaration(n)&&n.initializer===r&&e.isVariableDeclarationList(n.parent)&&n.parent.declarations.length>1)return n;r=n,n=n.parent}}(n,t);if(S){g.insertNodeBefore(o.file,S,T);var h=e.createIdentifier(u);g.replaceNode(o.file,n,h)}else if(219===n.parent.kind&&t===e.findAncestor(n,d)){var L=e.createVariableStatement(void 0,e.createVariableDeclarationList([T],2));g.replaceNode(o.file,n.parent,L)}else{var L=e.createVariableStatement(void 0,e.createVariableDeclarationList([T],2)),E=function(n,t){var r;e.Debug.assert(!e.isClassLike(t));for(var a=n;a!==t;a=a.parent)d(a)&&(r=a);for(var a=(r||n).parent;;a=a.parent){if(_(a)){for(var i=void 0,o=0,s=a.statements;o<s.length;o++){var l=s[o];if(l.pos>n.pos)break;i=l}return!i&&e.isCaseClause(a)?(e.Debug.assert(e.isSwitchStatement(a.parent.parent)),a.parent.parent):e.Debug.assertDefined(i)}e.Debug.assert(a!==t,\"Didn't encounter a block-like before encountering scope\")}}(n,t);if(0===E.pos?g.insertNodeAtTopOfFile(o.file,L,!1):g.insertNodeBefore(o.file,E,L,!1),219===n.parent.kind)g.delete(o.file,n.parent);else{var h=e.createIdentifier(u);g.replaceNode(o.file,n,h)}}}var x=g.getChanges(),A=n.getSourceFile().fileName,C=e.getRenameLocation(x,A,u,!0);return{renameFilename:A,renameLocation:C,edits:x}}(e.isExpression(l)?l:l.statements[0].expression,o[r],c[r],n.facts,t)}(r,n,o)}e.Debug.fail(\"Unrecognized action name\")}function u(n,t){var i=t.length;if(0===i)return{errors:[e.createFileDiagnostic(n,t.start,i,r.cannotExtractEmpty)]};var o=e.getParentNodeInSpan(e.getTokenAtPosition(n,t.start),n,t),s=e.getParentNodeInSpan(e.findTokenOnLeftOfPosition(n,e.textSpanEnd(t)),n,t),l=[],c=a.None;if(!o||!s)return{errors:[e.createFileDiagnostic(n,t.start,i,r.cannotExtractRange)]};if(o.parent!==s.parent)return{errors:[e.createFileDiagnostic(n,t.start,i,r.cannotExtractRange)]};if(o!==s){if(!_(o.parent))return{errors:[e.createFileDiagnostic(n,t.start,i,r.cannotExtractRange)]};for(var u=[],d=0,m=o.parent.statements;d<m.length;d++){var p=m[d];if(p===o||u.length){var f=y(p);if(f)return{errors:f};u.push(p)}if(p===s)break}return u.length?{targetRange:{range:u,facts:c,declarations:l}}:{errors:[e.createFileDiagnostic(n,t.start,i,r.cannotExtractRange)]}}if(e.isReturnStatement(o)&&!o.expression)return{errors:[e.createFileDiagnostic(n,t.start,i,r.cannotExtractRange)]};var g=function(n){if(e.isReturnStatement(n)){if(n.expression)return n.expression}else if(e.isVariableStatement(n)){for(var t=0,r=void 0,a=0,i=n.declarationList.declarations;a<i.length;a++){var o=i[a];o.initializer&&(t++,r=o.initializer)}if(1===t)return r}else if(e.isVariableDeclaration(n)&&n.initializer)return n.initializer;return n}(o),v=function(n){if(e.isIdentifier(e.isExpressionStatement(n)?n.expression:n))return[e.createDiagnosticForNode(n,r.cannotExtractIdentifier)];return}(g)||y(g);return v?{errors:v}:{targetRange:{range:function(n){if(e.isStatement(n))return[n];if(e.isExpressionNode(n))return e.isExpressionStatement(n.parent)?[n.parent]:n;return}(g),facts:c,declarations:l}};function y(n){var i;if(function(e){e[e.None=0]=\"None\",e[e.Break=1]=\"Break\",e[e.Continue=2]=\"Continue\",e[e.Return=4]=\"Return\"}(i||(i={})),e.Debug.assert(n.pos<=n.end,\"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809\"),e.Debug.assert(!e.positionIsSynthesized(n.pos),\"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809\"),!(e.isStatement(n)||e.isExpressionNode(n)&&function(e){var n=e.parent;switch(n.kind){case 276:return!1}switch(e.kind){case 9:return 247!==n.kind&&251!==n.kind;case 206:case 182:case 184:return!1;case 71:return 184!==n.kind&&251!==n.kind&&255!==n.kind}return!0}(n)))return[e.createDiagnosticForNode(n,r.statementOrExpressionExpected)];if(4194304&n.flags)return[e.createDiagnosticForNode(n,r.cannotExtractAmbientBlock)];var o,s=e.getContainingClass(n);s&&function(n,t){for(var r=n;r!==t;){if(152===r.kind){e.hasModifier(r,32)&&(c|=a.InStaticRegion);break}if(149===r.kind){155===e.getContainingFunction(r).kind&&(c|=a.InStaticRegion);break}154===r.kind&&e.hasModifier(r,32)&&(c|=a.InStaticRegion),r=r.parent}}(n,s);var u,d=4;return function n(i){if(o)return!0;if(e.isDeclaration(i)){var s=235===i.kind?i.parent.parent:i;if(e.hasModifier(s,1))return(o||(o=[])).push(e.createDiagnosticForNode(i,r.cannotExtractExportedEntity)),!0;l.push(i.symbol)}switch(i.kind){case 247:return(o||(o=[])).push(e.createDiagnosticForNode(i,r.cannotExtractImport)),!0;case 97:if(189===i.parent.kind){var m=e.getContainingClass(i);if(m.pos<t.start||m.end>=t.start+t.length)return(o||(o=[])).push(e.createDiagnosticForNode(i,r.cannotExtractSuper)),!0}else c|=a.UsesThis}if(e.isFunctionLikeDeclaration(i)||e.isClassLike(i)){switch(i.kind){case 237:case 238:e.isSourceFile(i.parent)&&void 0===i.parent.externalModuleIndicator&&(o||(o=[])).push(e.createDiagnosticForNode(i,r.functionWillNotBeVisibleInTheNewScope))}return!1}var p=d;switch(i.kind){case 220:case 233:d=0;break;case 216:i.parent&&233===i.parent.kind&&i.parent.finallyBlock===i&&(d=4);break;case 269:d|=1;break;default:e.isIterationStatement(i,!1)&&(d|=3)}switch(i.kind){case 176:case 99:c|=a.UsesThis;break;case 231:var f=i.label;(u||(u=[])).push(f.escapedText),e.forEachChild(i,n),u.pop();break;case 227:case 226:var f=i.label;f?e.contains(u,f.escapedText)||(o||(o=[])).push(e.createDiagnosticForNode(i,r.cannotExtractRangeContainingLabeledBreakOrContinueStatementWithTargetOutsideOfTheRange)):d&(227===i.kind?1:2)||(o||(o=[])).push(e.createDiagnosticForNode(i,r.cannotExtractRangeContainingConditionalBreakOrContinueStatements));break;case 199:c|=a.IsAsyncFunction;break;case 205:c|=a.IsGenerator;break;case 228:4&d?c|=a.HasReturn:(o||(o=[])).push(e.createDiagnosticForNode(i,r.cannotExtractRangeContainingConditionalReturnStatement));break;default:e.forEachChild(i,n)}d=p}(n),o}}function d(n){return e.isFunctionLikeDeclaration(n)||e.isSourceFile(n)||e.isModuleBlock(n)||e.isClassLike(n)}function m(n,t){var i=t.file,o=function(n){var t=g(n.range)?e.first(n.range):n.range;if(n.facts&a.UsesThis){var r=e.getContainingClass(t);if(r){var i=e.findAncestor(t,e.isFunctionLikeDeclaration);return i?[i,r]:[r]}}for(var o=[];;)if(149===(t=t.parent).kind&&(t=e.findAncestor(t,function(n){return e.isFunctionLikeDeclaration(n)}).parent),d(t)&&(o.push(t),277===t.kind))return o}(n);return{scopes:o,readsAndWrites:function(n,t,i,o,s,l){var c,u,d=e.createMap(),m=[],p=[],f=[],_=[],v=[],y=e.createMap(),h=[],b=g(n.range)?1===n.range.length&&e.isExpressionStatement(n.range[0])?n.range[0].expression:void 0:n.range;if(void 0===b){var E=n.range,T=e.first(E).getStart(),S=e.last(E).end;u=e.createFileDiagnostic(o,T,S-T,r.expressionExpected)}else 36864&s.getTypeAtLocation(b).flags&&(u=e.createDiagnosticForNode(b,r.uselessConstantType));for(var L=0,x=t;L<x.length;L++){var A=x[L];m.push({usages:e.createMap(),typeParameterUsages:e.createMap(),substitutions:e.createMap()}),p.push(e.createMap()),f.push(e.isFunctionLikeDeclaration(A)&&237!==A.kind?[e.createDiagnosticForNode(A,r.cannotExtractToOtherFunctionLike)]:[]);var C=[];u&&C.push(u),e.isClassLike(A)&&e.isInJSFile(A)&&C.push(e.createDiagnosticForNode(A,r.cannotExtractToJSClass)),e.isArrowFunction(A)&&!e.isBlock(A.body)&&C.push(e.createDiagnosticForNode(A,r.cannotExtractToExpressionArrowFunction)),_.push(C)}var D=e.createMap(),k=g(n.range)?e.createBlock(n.range):n.range,M=g(n.range)?e.first(n.range):n.range,O=(I=M,!!e.findAncestor(I,function(n){return e.isDeclarationWithTypeParameters(n)&&0!==e.getEffectiveTypeParameterDeclarations(n).length}));var I;if(function l(c,u){if(void 0===u&&(u=1),O){var d=s.getTypeAtLocation(c);j(d)}if(e.isDeclaration(c)&&c.symbol&&v.push(c),e.isAssignmentExpression(c))l(c.left,2),l(c.right);else if(e.isUnaryExpressionWithWrite(c))l(c.operand,2);else if(e.isPropertyAccessExpression(c)||e.isElementAccessExpression(c))e.forEachChild(c,l);else if(e.isIdentifier(c)){if(!c.parent)return;if(e.isQualifiedName(c.parent)&&c!==c.parent.left)return;if(e.isPropertyAccessExpression(c.parent)&&c!==c.parent.expression)return;!function(l,c,u){var d=function(l,c,u){var d=W(l);if(d){var g=e.getSymbolId(d).toString(),v=D.get(g);if(v&&v>=c)return g;if(D.set(g,c),v){for(var y=0,h=m;y<h.length;y++){var b=h[y],E=b.usages.get(l.text);E&&b.usages.set(l.text,{usage:c,symbol:d,node:l})}return g}var T=d.getDeclarations(),S=T&&e.find(T,function(e){return e.getSourceFile()===o});if(S&&!e.rangeContainsStartEnd(i,S.getStart(),S.end)){if(n.facts&a.IsGenerator&&2===c){for(var L=e.createDiagnosticForNode(l,r.cannotExtractRangeThatContainsWritesToReferencesLocatedOutsideOfTheTargetRangeInGenerators),x=0,A=f;x<A.length;x++){var C=A[x];C.push(L)}for(var k=0,M=_;k<M.length;k++){var C=M[k];C.push(L)}}for(var O=0;O<t.length;O++){var I=t[O],R=s.resolveName(d.name,I,d.flags,!1);if(R!==d&&!p[O].has(g)){var N=q(d.exportSymbol||d,I,u);if(N)p[O].set(g,N);else if(u){if(!(262144&d.flags)){var L=e.createDiagnosticForNode(l,r.typeWillNotBeVisibleInTheNewScope);f[O].push(L),_[O].push(L)}}else m[O].usages.set(l.text,{usage:c,symbol:d,node:l})}}return g}}}(l,c,u);if(d)for(var g=0;g<t.length;g++){var v=p[g].get(d);v&&m[g].substitutions.set(e.getNodeId(l).toString(),v)}}(c,u,e.isPartOfTypeNode(c))}else e.forEachChild(c,l)}(k),O&&!g(n.range)){var R=s.getContextualType(n.range);j(R)}if(d.size>0){for(var N=e.createMap(),w=0,P=M;void 0!==P&&w<t.length;P=P.parent)if(P===t[w]&&(N.forEach(function(e,n){m[w].typeParameterUsages.set(n,e)}),w++),e.isDeclarationWithTypeParameters(P))for(var F=0,G=e.getEffectiveTypeParameterDeclarations(P);F<G.length;F++){var V=G[F],K=s.getTypeAtLocation(V);d.has(K.id.toString())&&N.set(K.id.toString(),K)}e.Debug.assert(w===t.length)}if(v.length){var B=e.isBlockScope(t[0],t[0].parent)?t[0]:e.getEnclosingBlockScopeContainer(t[0]);e.forEachChild(B,function t(r){if(r===n.range||g(n.range)&&n.range.indexOf(r)>=0)return;var a=e.isIdentifier(r)?W(r):s.getSymbolAtLocation(r);if(a){var i=e.find(v,function(e){return e.symbol===a});if(i)if(e.isVariableDeclaration(i)){var o=i.symbol.id.toString();y.has(o)||(h.push(i),y.set(o,!0))}else c=c||i}e.forEachChild(r,t)})}for(var H=function(t){var a=m[t];if(t>0&&(a.usages.size>0||a.typeParameterUsages.size>0)){var i=g(n.range)?n.range[0]:n.range;_[t].push(e.createDiagnosticForNode(i,r.cannotAccessVariablesFromNestedScopes))}var o,s=!1;if(m[t].usages.forEach(function(n){2===n.usage&&(s=!0,106500&n.symbol.flags&&n.symbol.valueDeclaration&&e.hasModifier(n.symbol.valueDeclaration,64)&&(o=n.symbol.valueDeclaration))}),e.Debug.assert(g(n.range)||0===h.length),s&&!g(n.range)){var l=e.createDiagnosticForNode(n.range,r.cannotWriteInExpression);f[t].push(l),_[t].push(l)}else if(o&&t>0){var l=e.createDiagnosticForNode(o,r.cannotExtractReadonlyPropertyInitializerOutsideConstructor);f[t].push(l),_[t].push(l)}else if(c){var l=e.createDiagnosticForNode(c,r.cannotExtractExportedEntity);f[t].push(l),_[t].push(l)}},U=0;U<t.length;U++)H(U);return{target:k,usagesPerScope:m,functionErrorsPerScope:f,constantErrorsPerScope:_,exposedVariableDeclarations:h};function j(e){for(var n=s.getSymbolWalker(function(){return l.throwIfCancellationRequested(),!0}),t=n.walkType(e).visitedTypes,r=0,a=t;r<a.length;r++){var i=a[r];i.isTypeParameter()&&d.set(i.id.toString(),i)}}function W(n){return n.parent&&e.isShorthandPropertyAssignment(n.parent)&&n.parent.name===n?s.getShorthandAssignmentValueSymbol(n.parent):s.getSymbolAtLocation(n)}function q(n,t,r){if(n){var a=n.getDeclarations();if(a&&a.some(function(e){return e.parent===t}))return e.createIdentifier(n.name);var i=q(n.parent,t,r);if(void 0!==i)return r?e.createQualifiedName(i,e.createIdentifier(n.name)):e.createPropertyAccess(i,n.name)}}}(n,o,function(n,t){return g(n.range)?{pos:e.first(n.range).getStart(t),end:e.last(n.range).getEnd()}:n.range}(n,i),i,t.program.getTypeChecker(),t.cancellationToken)}}function p(n,t){var r=n.type,a=n.declaration,i=t.type,o=t.declaration;return e.compareProperties(a,o,\"pos\",e.compareValues)||e.compareStringsCaseSensitive(r.symbol?r.symbol.getName():\"\",i.symbol?i.symbol.getName():\"\")||e.compareValues(r.id,i.id)}function f(n,t){var r=e.map(n,function(n){return e.createShorthandPropertyAssignment(n.symbol.name)}),a=e.map(t,function(n){return e.createShorthandPropertyAssignment(n.symbol.name)});return void 0===r?a:void 0===a?r:r.concat(a)}function g(n){return e.isArray(n)}function _(e){switch(e.kind){case 216:case 277:case 243:case 269:return!0;default:return!1}}n.registerRefactor(s,{getAvailableActions:l,getEditsForAction:c}),t.getAvailableActions=l,t.getEditsForAction=c,function(n){function t(n){return{message:n,code:0,category:e.DiagnosticCategory.Message,key:n}}n.cannotExtractRange=t(\"Cannot extract range.\"),n.cannotExtractImport=t(\"Cannot extract import statement.\"),n.cannotExtractSuper=t(\"Cannot extract super call.\"),n.cannotExtractEmpty=t(\"Cannot extract empty range.\"),n.expressionExpected=t(\"expression expected.\"),n.uselessConstantType=t(\"No reason to extract constant of type.\"),n.statementOrExpressionExpected=t(\"Statement or expression expected.\"),n.cannotExtractRangeContainingConditionalBreakOrContinueStatements=t(\"Cannot extract range containing conditional break or continue statements.\"),n.cannotExtractRangeContainingConditionalReturnStatement=t(\"Cannot extract range containing conditional return statement.\"),n.cannotExtractRangeContainingLabeledBreakOrContinueStatementWithTargetOutsideOfTheRange=t(\"Cannot extract range containing labeled break or continue with target outside of the range.\"),n.cannotExtractRangeThatContainsWritesToReferencesLocatedOutsideOfTheTargetRangeInGenerators=t(\"Cannot extract range containing writes to references located outside of the target range in generators.\"),n.typeWillNotBeVisibleInTheNewScope=t(\"Type will not visible in the new scope.\"),n.functionWillNotBeVisibleInTheNewScope=t(\"Function will not visible in the new scope.\"),n.cannotExtractIdentifier=t(\"Select more than a single identifier.\"),n.cannotExtractExportedEntity=t(\"Cannot extract exported declaration\"),n.cannotWriteInExpression=t(\"Cannot write back side-effects when extracting an expression\"),n.cannotExtractReadonlyPropertyInitializerOutsideConstructor=t(\"Cannot move initialization of read-only class property outside of the constructor\"),n.cannotExtractAmbientBlock=t(\"Cannot extract code from ambient contexts\"),n.cannotAccessVariablesFromNestedScopes=t(\"Cannot access variables from nested scopes\"),n.cannotExtractToOtherFunctionLike=t(\"Cannot extract method to a function-like scope that is not a function\"),n.cannotExtractToJSClass=t(\"Cannot extract constant to a class scope in JS\"),n.cannotExtractToExpressionArrowFunction=t(\"Cannot extract constant to an arrow function without a block\")}(r=t.Messages||(t.Messages={})),function(e){e[e.None=0]=\"None\",e[e.HasReturn=1]=\"HasReturn\",e[e.IsGenerator=2]=\"IsGenerator\",e[e.IsAsyncFunction=4]=\"IsAsyncFunction\",e[e.UsesThis=8]=\"UsesThis\",e[e.InStaticRegion=16]=\"InStaticRegion\"}(a||(a={})),t.getRangeToExtract=u,function(e){e[e.Module=0]=\"Module\",e[e.Global=1]=\"Global\"}(i||(i={})),function(e){e[e.Read=1]=\"Read\",e[e.Write=2]=\"Write\"}(o||(o={}))}(n.extractSymbol||(n.extractSymbol={}))}(e.refactor||(e.refactor={}))}(Tn||(Tn={})),function(e){!function(n){!function(t){var r=\"Generate 'get' and 'set' accessors\",a=e.Diagnostics.Generate_get_and_set_accessors.message;function i(n){return e.isParameterPropertyDeclaration(n)||e.isPropertyDeclaration(n)||e.isPropertyAssignment(n)}function o(n,t){return e.isIdentifier(t)?e.createIdentifier(n):e.createLiteral(n)}function s(n,t,r){var a=t?r.name:e.createThis();return e.isIdentifier(n)?e.createPropertyAccess(a,n):e.createElementAccess(a,e.createLiteral(n))}function l(n,t,r){var a=e.append(n?void 0:[e.createToken(r)],t?e.createToken(115):void 0);return a&&e.createNodeArray(a)}function c(n){var t=n.file,r=n.startPosition,a=n.endPosition,s=e.getTokenAtPosition(t,r),l=e.findAncestor(s.parent,i);if(l&&e.nodeOverlapsWithStartEnd(l.name,t,r,a)&&function(n){return e.isIdentifier(n)||e.isStringLiteral(n)}(l.name)&&124==(124|e.getModifierFlags(l))){var c=l.name.text,u=function(e){return 95===e.charCodeAt(0)}(c),d=o(u?c:e.getUniqueName(\"_\"+c,t),l.name),m=o(u?e.getUniqueName(c.substring(1),t):c,l.name);return{isStatic:e.hasStaticModifier(l),isReadonly:e.hasReadonlyModifier(l),type:e.getTypeAnnotationNode(l),container:149===l.kind?l.parent.parent:l.parent,originalName:l.name,declaration:l,fieldName:d,accessorName:m,renameAccessor:u}}}function u(n,t,r,a,i){e.isParameterPropertyDeclaration(a)?n.insertNodeAtClassStart(t,i,r):e.isPropertyAssignment(a)?n.insertNodeAfterComma(t,a,r):n.insertNodeAfter(t,a,r)}n.registerRefactor(r,{getEditsForAction:function(n,t){var r=n.file,a=c(n);if(!a)return;var i=e.isSourceFileJS(r),o=e.textChanges.ChangeTracker.fromContext(n),d=a.isStatic,m=a.isReadonly,p=a.fieldName,f=a.accessorName,g=a.originalName,_=a.type,v=a.container,y=a.declaration,h=a.renameAccessor;e.suppressLeadingAndTrailingTrivia(p),e.suppressLeadingAndTrailingTrivia(y),e.suppressLeadingAndTrailingTrivia(v);var b=e.isClassLike(v),E=-65&e.getModifierFlags(y),T=b?!E||8&E?l(i,d,114):e.createNodeArray(e.createModifiersFromModifierFlags(E)):void 0,S=b?l(i,d,112):void 0;!function(n,t,r,a,i){e.isPropertyDeclaration(r)?function(n,t,r,a,i){var o=e.updateProperty(r,r.decorators,i,a,r.questionToken||r.exclamationToken,r.type,r.initializer);n.replaceNode(t,r,o)}(n,t,r,a,i):e.isPropertyAssignment(r)?function(n,t,r,a){var i=e.updatePropertyAssignment(r,a,r.initializer);n.replacePropertyAssignment(t,r,i)}(n,t,r,a):n.replaceNode(t,r,e.updateParameter(r,r.decorators,i,r.dotDotDotToken,e.cast(a,e.isIdentifier),r.questionToken,r.type,r.initializer))}(o,r,y,p,S);var L=function(n,t,r,a,i,o){return e.createGetAccessor(void 0,a,t,void 0,r,e.createBlock([e.createReturn(s(n,i,o))],!0))}(p,f,_,T,d,v);if(e.suppressLeadingAndTrailingTrivia(L),u(o,r,L,y,v),m){var x=e.getFirstConstructorWithBody(v);x&&function(n,t,r,a,i){if(!r.body)return;var o=t.file,s=t.program,l=t.cancellationToken,c=e.mapDefined(e.FindAllReferences.getReferenceEntriesForNode(i.parent.pos,i,s,[o],l),function(n){return\"node\"===n.type&&e.rangeContainsRange(r,n.node)&&e.isIdentifier(n.node)&&e.isWriteAccess(n.node)?n.node:void 0});e.forEach(c,function(t){var r=t.parent,i=e.createIdentifier(a.text),s=e.isBinaryExpression(r)?e.updateBinary(r,i,r.right,r.operatorToken):e.isPropertyAccessExpression(r)?e.updatePropertyAccess(r,r.expression,i):e.Debug.fail(\"Unexpected write access token\");n.replaceNode(o,r,s)})}(o,n,x,p,g)}else{var A=function(n,t,r,a,i,o){return e.createSetAccessor(void 0,a,t,[e.createParameter(void 0,void 0,void 0,e.createIdentifier(\"value\"),void 0,r)],e.createBlock([e.createStatement(e.createAssignment(s(n,i,o),e.createIdentifier(\"value\")))],!0))}(p,f,_,T,d,v);e.suppressLeadingAndTrailingTrivia(A),u(o,r,A,y,v)}var C=o.getChanges(),D=r.fileName,k=h?f:p,M=(e.isIdentifier(k)?0:-1)+e.getRenameLocation(C,D,k.text,e.isParameter(y));return{renameFilename:D,renameLocation:M,edits:C}},getAvailableActions:function(e){return c(e)?[{name:r,description:a,actions:[{name:r,description:a}]}]:void 0}})}(n.generateGetAccessorAndSetAccessor||(n.generateGetAccessorAndSetAccessor={}))}(e.refactor||(e.refactor={}))}(Tn||(Tn={})),function(e){!function(n){function t(n){var t=function(n){var t=n.file,r=e.createTextRangeFromSpan(e.getRefactorContextSpan(n)),a=t.statements,i=e.findIndex(a,function(e){return e.end>r.pos});if(-1!==i){var o=a[i];if(e.isNamedDeclaration(o)&&o.name&&e.rangeContainsRange(o.name,r))return{toMove:[a[i]],afterLast:a[i+1]};if(!(r.pos>o.getStart(t))){var s=e.findIndex(a,function(e){return e.end>r.end},i);if(-1===s||!(0===s||a[s].getStart(t)<r.end))return{toMove:a.slice(i,-1===s?a.length:s),afterLast:-1===s?void 0:a[s]}}}}(n);if(void 0!==t){var r=[],a=[],i=t.toMove,o=t.afterLast;return e.getRangesWhere(i,function(n){return!function(n){switch(n.kind){case 247:return!0;case 246:return!e.hasModifier(n,1);case 217:return n.declarationList.declarations.every(function(n){return!!n.initializer&&e.isRequireCall(n.initializer,!0)});default:return!1}}(n)},function(e,n){for(var t=e;t<n;t++)r.push(i[t]);a.push({first:i[e],afterLast:o})}),0===r.length?void 0:{all:r,ranges:a}}}function r(e,n,t){for(var r=0,a=n;r<a.length;r++){var i=a[r],o=i.first,s=i.afterLast;t.deleteNodeRangeExcludingEnd(e,o,s)}}function a(e){return 247===e.kind?e.moduleSpecifier:246===e.kind?e.moduleReference.expression:e.initializer.arguments[0]}function i(n,t){if(e.isImportDeclaration(n))e.isStringLiteral(n.moduleSpecifier)&&t(n);else if(e.isImportEqualsDeclaration(n))e.isExternalModuleReference(n.moduleReference)&&e.isStringLiteralLike(n.moduleReference.expression)&&t(n);else if(e.isVariableStatement(n))for(var r=0,a=n.declarationList.declarations;r<a.length;r++){var i=a[r];i.initializer&&e.isRequireCall(i.initializer,!0)&&t(i)}}function o(n,t,r,a,i){if(r=e.ensurePathIsNonModuleName(r),a){var o=t.map(function(n){return e.createImportSpecifier(void 0,e.createIdentifier(n))});return e.makeImportIfNecessary(n,o,r,i)}e.Debug.assert(!n);var c=t.map(function(n){return e.createBindingElement(void 0,void 0,n)});return c.length?s(e.createObjectBindingPattern(c),void 0,l(e.createLiteral(r))):void 0}function s(n,t,r,a){return void 0===a&&(a=2),e.createVariableStatement(void 0,e.createVariableDeclarationList([e.createVariableDeclaration(n,t,r)],a))}function l(n){return e.createCall(e.createIdentifier(\"require\"),void 0,[n])}function c(n,t,r,a){switch(t.kind){case 247:!function(e,n,t,r){if(!n.importClause)return;var a=n.importClause,i=a.name,o=a.namedBindings,s=!i||r(i),l=!o||(249===o.kind?r(o.name):0!==o.elements.length&&o.elements.every(function(e){return r(e.name)}));if(s&&l)t.delete(e,n);else if(i&&s&&t.delete(e,i),o)if(l)t.delete(e,o);else if(250===o.kind)for(var c=0,u=o.elements;c<u.length;c++){var d=u[c];r(d.name)&&t.delete(e,d)}}(n,t,r,a);break;case 246:a(t.name)&&r.delete(n,t);break;case 235:!function(n,t,r,a){var i=t.name;switch(i.kind){case 71:a(i)&&r.delete(n,i);break;case 183:break;case 182:if(i.elements.every(function(n){return e.isIdentifier(n.name)&&a(n.name)}))r.delete(n,e.isVariableDeclarationList(t.parent)&&1===t.parent.declarations.length?t.parent.parent:t);else for(var o=0,s=i.elements;o<s.length;o++){var l=s[o];e.isIdentifier(l.name)&&a(l.name)&&r.delete(n,l.name)}}}(n,t,r,a);break;default:e.Debug.assertNever(t)}}function u(n){switch(n.kind){case 246:case 251:case 248:return!0;case 235:return d(n);case 184:return e.isVariableDeclaration(n.parent.parent)&&d(n.parent.parent);default:return!1}}function d(n){return e.isSourceFile(n.parent.parent.parent)&&n.initializer&&e.isRequireCall(n.initializer,!0)}function m(n,t,r){switch(n.kind){case 247:var a=n.importClause;if(!a)return;var i=a.name&&r(a.name)?a.name:void 0,o=a.namedBindings&&function(n,t){if(249===n.kind)return t(n.name)?n:void 0;var r=n.elements.filter(function(e){return t(e.name)});return r.length?e.createNamedImports(r):void 0}(a.namedBindings,r);return i||o?e.createImportDeclaration(void 0,void 0,e.createImportClause(i,o),t):void 0;case 246:return r(n.name)?n:void 0;case 235:var c=function(n,t){switch(n.kind){case 71:return t(n)?n:void 0;case 183:return n;case 182:var r=n.elements.filter(function(n){return n.propertyName||!e.isIdentifier(n.name)||t(n.name)});return r.length?e.createObjectBindingPattern(r):void 0}}(n.name,r);return c?s(c,n.type,l(t),n.parent.flags):void 0;default:return e.Debug.assertNever(n)}}function p(n,t,r){n.forEachChild(function n(a){if(e.isIdentifier(a)&&!e.isDeclarationName(a)){var i=t.getSymbolAtLocation(a);i&&r(i)}else a.forEachChild(n)})}n.registerRefactor(\"Move to a new file\",{getAvailableActions:function(n){if(n.preferences.allowTextChangesInNewFiles&&void 0!==t(n)){var r=e.getLocaleSpecificMessage(e.Diagnostics.Move_to_a_new_file);return[{name:\"Move to a new file\",description:r,actions:[{name:\"Move to a new file\",description:r}]}]}},getEditsForAction:function(n,s){e.Debug.assert(\"Move to a new file\"===s);var d=e.Debug.assertDefined(t(n));return{edits:e.textChanges.ChangeTracker.with(n,function(t){return s=n.file,S=n.program,L=d,x=t,A=n.host,C=n.preferences,D=S.getTypeChecker(),k=function(n,t,r){for(var a=new f,i=new f,o=new f,s=0,l=t;s<l.length;s++){var c=l[s];v(c,function(n){a.add(e.Debug.assertDefined(e.isExpressionStatement(n)?r.getSymbolAtLocation(n.expression.left):n.symbol))})}for(var d=0,m=t;d<m.length;d++){var c=m[d];p(c,r,function(t){var r;if(t.declarations)for(var s=0,l=t.declarations;s<l.length;s++){var c=l[s];u(c)?i.add(t):g(c)&&(r=c,(e.isVariableDeclaration(r)?r.parent.parent.parent:r.parent)===n)&&!a.has(t)&&o.add(t)}})}for(var _=i.clone(),y=new f,h=0,b=n.statements;h<b.length;h++){var c=b[h];e.contains(t,c)||p(c,r,function(e){a.has(e)&&y.add(e),_.delete(e)})}return{movedSymbols:a,newFileImportsFromOldFile:o,oldFileImportsFromNewFile:y,oldImportsNeededByNewFile:i,unusedImportsFromOldFile:_}}(s,L.all,D),M=e.getDirectoryPath(s.fileName),O=e.extensionFromPath(s.fileName),I=function(n,t,r,a){for(var i=n,o=1;;o++){var s=e.combinePaths(r,i+t);if(!a.fileExists(s))return i;i=n+\".\"+o}}(k.movedSymbols.forEachEntry(e.symbolNameNoDefault)||\"newFile\",O,M,A),R=I+O,x.createNewFile(s,e.combinePaths(M,R),function(n,t,s,u,d,p,f){var S=d.getTypeChecker();if(!n.externalModuleIndicator&&!n.commonJsModuleIndicator)return r(n,u.ranges,s),u.all;var L=!!n.externalModuleIndicator,x=e.getQuotePreference(n,f),A=function(n,t,r,a){var i,s=[];return n.forEach(function(n){\"default\"===n.escapedName?i=e.createIdentifier(e.symbolNameNoDefault(n)):s.push(n.name)}),o(i,s,t,r,a)}(t.oldFileImportsFromNewFile,p,L,x);return A&&e.insertImport(s,n,A),function(n,t,r,a,o){for(var s=0,l=n.statements;s<l.length;s++){var u=l[s];e.contains(t,u)||i(u,function(e){return c(n,e,r,function(e){return a.has(o.getSymbolAtLocation(e))})})}}(n,u.all,s,t.unusedImportsFromOldFile,S),r(n,u.ranges,s),function(n,t,r,o,s){for(var u=t.getTypeChecker(),d=function(t){if(t===r)return\"continue\";for(var d=function(d){i(d,function(i){if(u.getSymbolAtLocation(a(i))===r.symbol){var p=function(n){var t=e.isBindingElement(n.parent)?e.getPropertySymbolFromBindingElement(u,n.parent):e.skipAlias(u.getSymbolAtLocation(n),u);return!!t&&o.has(t)};c(t,i,n,p);var f=e.combinePaths(e.getDirectoryPath(a(i).text),s),g=m(i,e.createLiteral(f),p);g&&n.insertNodeAfter(t,d,g);var _=function(n){switch(n.kind){case 247:return n.importClause&&n.importClause.namedBindings&&249===n.importClause.namedBindings.kind?n.importClause.namedBindings.name:void 0;case 246:return n.name;case 235:return e.tryCast(n.name,e.isIdentifier);default:return e.Debug.assertNever(n)}}(i);_&&function(n,t,r,a,i,o,s,c){var u=e.codefix.moduleSpecifierToValidIdentifier(i,6),d=!1,m=[];if(e.FindAllReferences.Core.eachSymbolReferenceInFile(s,r,t,function(n){e.isPropertyAccessExpression(n.parent)&&(d=d||!!r.resolveName(u,n,67108863,!0),a.has(r.getSymbolAtLocation(n.parent.name))&&m.push(n))}),m.length){for(var p=d?e.getUniqueName(u,t):u,f=0,g=m;f<g.length;f++){var _=g[f];n.replaceNode(t,_,e.createIdentifier(p))}n.insertNodeAfter(t,c,function(n,t,r){var a=e.createIdentifier(t),i=e.createLiteral(r);switch(n.kind){case 247:return e.createImportDeclaration(void 0,void 0,e.createImportClause(void 0,e.createNamespaceImport(a)),i);case 246:return e.createImportEqualsDeclaration(void 0,void 0,a,e.createExternalModuleReference(i));case 235:return e.createVariableDeclaration(a,void 0,l(i));default:return e.Debug.assertNever(n)}}(c,i,o))}}(n,t,u,o,s,f,_,i)}})},p=0,f=t.statements;p<f.length;p++){var g=f[p];d(g)}},p=0,f=t.getSourceFiles();p<f.length;p++){var g=f[p];d(g)}}(s,d,n,t.movedSymbols,p),function(n,t,r,s,l,c,u){for(var d,p=[],f=0,_=n.statements;f<_.length;f++){var v=_[f];i(v,function(n){e.append(p,m(n,a(n),function(e){return t.has(l.getSymbolAtLocation(e))}))})}var b=[],E=e.nodeSeenTracker();return r.forEach(function(t){for(var r=0,a=t.declarations;r<a.length;r++){var i=a[r];if(g(i)){var o=219===(u=i).kind?u.expression.left.name:e.tryCast(u.name,e.isIdentifier);if(o){var l=y(i);E(l)&&h(n,l,s,c),e.hasModifier(i,512)?d=o:b.push(o.text)}}}var u}),e.append(p,o(d,b,e.removeFileExtension(e.getBaseFileName(n.fileName)),c,u)),p}(n,t.oldImportsNeededByNewFile,t.newFileImportsFromOldFile,s,S,L,x).concat(function(n,t,r,a){return e.flatMap(t,function(t){if(o=t,e.Debug.assert(e.isSourceFile(o.parent)),(_(o)||e.isVariableStatement(o))&&!b(n,t,a)&&v(t,function(n){return r.has(e.Debug.assertDefined(n.symbol))})){var i=function(n,t){return t?[function(n){var t=e.concatenate([e.createModifier(84)],n.modifiers);switch(n.kind){case 237:return e.updateFunctionDeclaration(n,n.decorators,t,n.asteriskToken,n.name,n.typeParameters,n.parameters,n.type,n.body);case 238:return e.updateClassDeclaration(n,n.decorators,t,n.name,n.typeParameters,n.heritageClauses,n.members);case 217:return e.updateVariableStatement(n,t,n.declarationList);case 242:return e.updateModuleDeclaration(n,n.decorators,t,n.name,n.body);case 241:return e.updateEnumDeclaration(n,n.decorators,t,n.name,n.members);case 240:return e.updateTypeAliasDeclaration(n,n.decorators,t,n.name,n.typeParameters,n.type);case 239:return e.updateInterfaceDeclaration(n,n.decorators,t,n.name,n.typeParameters,n.heritageClauses,n.members);case 246:return e.updateImportEqualsDeclaration(n,n.decorators,t,n.name,n.moduleReference);case 219:return e.Debug.fail();default:return e.Debug.assertNever(n)}}(n)]:function(e){return[e].concat(E(e).map(T))}(n)}(t,a);if(i)return i}var o;return t})}(n,u.all,t.oldFileImportsFromNewFile,L))}(s,k,x,L,S,I,C)),void function(n,t,r,a,i){var o=n.getCompilerOptions().configFile;if(o){var s=e.normalizePath(e.combinePaths(r,\"..\",a)),l=e.getRelativePathFromFile(o.fileName,s,i),c=o.statements[0]&&e.tryCast(o.statements[0].expression,e.isObjectLiteralExpression),u=c&&e.find(c.properties,function(n){return e.isPropertyAssignment(n)&&e.isStringLiteral(n.name)&&\"files\"===n.name.text});u&&e.isArrayLiteralExpression(u.initializer)&&t.insertNodeInListAfter(o,e.last(u.initializer.elements),e.createLiteral(l),u.initializer.elements)}}(S,x,s.fileName,R,e.hostGetCanonicalFileName(A));var s,S,L,x,A,C,D,k,M,O,I,R}),renameFilename:void 0,renameLocation:void 0}}});var f=function(){function n(){this.map=e.createMap()}return n.prototype.add=function(n){this.map.set(String(e.getSymbolId(n)),n)},n.prototype.has=function(n){return this.map.has(String(e.getSymbolId(n)))},n.prototype.delete=function(n){this.map.delete(String(e.getSymbolId(n)))},n.prototype.forEach=function(e){this.map.forEach(e)},n.prototype.forEachEntry=function(n){return e.forEachEntry(this.map,n)},n.prototype.clone=function(){var t=new n;return e.copyEntries(this.map,t.map),t},n}();function g(n){return _(n)&&e.isSourceFile(n.parent)||e.isVariableDeclaration(n)&&e.isSourceFile(n.parent.parent.parent)}function _(e){switch(e.kind){case 237:case 238:case 242:case 241:case 240:case 239:case 246:return!0;default:return!1}}function v(n,t){switch(n.kind){case 237:case 238:case 242:case 241:case 240:case 239:case 246:return t(n);case 217:return e.forEach(n.declarationList.declarations,t);case 219:var r=n.expression;return e.isBinaryExpression(r)&&1===e.getAssignmentDeclarationKind(r)?t(n):void 0}}function y(n){return e.isVariableDeclaration(n)?n.parent.parent:n}function h(n,t,r,a){if(!b(n,t,a))if(a)e.isExpressionStatement(t)||r.insertExportModifier(n,t);else{var i=E(t);0!==i.length&&r.insertNodesAfter(n,t,i.map(T))}}function b(n,t,r){return r?!e.isExpressionStatement(t)&&e.hasModifier(t,1):E(t).some(function(t){return n.symbol.exports.has(e.escapeLeadingUnderscores(t))})}function E(n){switch(n.kind){case 237:case 238:return[n.name.text];case 217:return e.mapDefined(n.declarationList.declarations,function(n){return e.isIdentifier(n.name)?n.name.text:void 0});case 242:case 241:case 240:case 239:case 246:return e.emptyArray;case 219:return e.Debug.fail();default:return e.Debug.assertNever(n)}}function T(n){return e.createExpressionStatement(e.createBinary(e.createPropertyAccess(e.createIdentifier(\"exports\"),e.createIdentifier(n)),58,e.createIdentifier(n)))}}(e.refactor||(e.refactor={}))}(Tn||(Tn={})),function(e){!function(n){!function(t){var r=\"Add or remove braces in an arrow function\",a=e.Diagnostics.Add_or_remove_braces_in_an_arrow_function.message,i=\"Add braces to arrow function\",o=\"Remove braces from arrow function\",s=e.Diagnostics.Add_braces_to_arrow_function.message,l=e.Diagnostics.Remove_braces_from_arrow_function.message;function c(n,t){var r=e.getTokenAtPosition(n,t),a=e.getContainingFunction(r);if(a&&e.isArrowFunction(a)&&e.rangeContainsRange(a,r)&&!e.rangeContainsRange(a.body,r)){if(e.isExpression(a.body))return{func:a,addBraces:!0,expression:a.body};if(1===a.body.statements.length){var i=e.first(a.body.statements);if(e.isReturnStatement(i))return{func:a,addBraces:!1,expression:i.expression,returnStatement:i}}}}n.registerRefactor(r,{getEditsForAction:function(n,t){var r=n.file,a=n.startPosition,s=c(r,a);if(!s)return;var l,u=s.expression,d=s.returnStatement,m=s.func;if(t===i){var p=e.createReturn(u);l=e.createBlock([p],!0),e.suppressLeadingAndTrailingTrivia(l),e.copyComments(u,p,r,3,!0)}else if(t===o&&d){var f=u||e.createVoidZero();l=function(n){return e.isBinaryExpression(n)&&26===n.operatorToken.kind||e.isObjectLiteralExpression(n)}(f)?e.createParen(f):f,e.suppressLeadingAndTrailingTrivia(l),e.copyComments(d,l,r,3,!1)}else e.Debug.fail(\"invalid action\");var g=e.textChanges.ChangeTracker.with(n,function(e){return e.replaceNode(r,m.body,l)});return{renameFilename:void 0,renameLocation:void 0,edits:g}},getAvailableActions:function(e){var n=e.file,t=e.startPosition,u=c(n,t);return u?[{name:r,description:a,actions:[u.addBraces?{name:i,description:s}:{name:o,description:l}]}]:void 0}})}(n.addOrRemoveBracesToArrowFunction||(n.addOrRemoveBracesToArrowFunction={}))}(e.refactor||(e.refactor={}))}(Tn||(Tn={})),function(e){function n(n,r,a,i){var l=e.isNodeKind(n)?new t(n,r,a):71===n?new s(71,r,a):new o(n,r,a);return l.parent=i,l.flags=12679168&i.flags,l}e.servicesVersion=\"0.8\";var t=function(){function t(e,n,t){this.pos=n,this.end=t,this.flags=0,this.transformFlags=void 0,this.parent=void 0,this.kind=e}return t.prototype.assertHasRealPosition=function(n){e.Debug.assert(!e.positionIsSynthesized(this.pos)&&!e.positionIsSynthesized(this.end),n||\"Node must have a real position for this operation\")},t.prototype.getSourceFile=function(){return e.getSourceFileOfNode(this)},t.prototype.getStart=function(n,t){return this.assertHasRealPosition(),e.getTokenPosOfNode(this,n,t)},t.prototype.getFullStart=function(){return this.assertHasRealPosition(),this.pos},t.prototype.getEnd=function(){return this.assertHasRealPosition(),this.end},t.prototype.getWidth=function(e){return this.assertHasRealPosition(),this.getEnd()-this.getStart(e)},t.prototype.getFullWidth=function(){return this.assertHasRealPosition(),this.end-this.pos},t.prototype.getLeadingTriviaWidth=function(e){return this.assertHasRealPosition(),this.getStart(e)-this.pos},t.prototype.getFullText=function(e){return this.assertHasRealPosition(),(e||this.getSourceFile()).text.substring(this.pos,this.end)},t.prototype.getText=function(e){return this.assertHasRealPosition(),e||(e=this.getSourceFile()),e.text.substring(this.getStart(e),this.getEnd())},t.prototype.getChildCount=function(e){return this.getChildren(e).length},t.prototype.getChildAt=function(e,n){return this.getChildren(n)[e]},t.prototype.getChildren=function(t){return this.assertHasRealPosition(\"Node without a real position cannot be scanned and thus has no token nodes - use forEachChild and collect the result if that's fine\"),this._children||(this._children=function(t,a){if(!e.isNodeKind(t.kind))return e.emptyArray;var i=[];if(e.isJSDocCommentContainingNode(t))return t.forEachChild(function(e){i.push(e)}),i;e.scanner.setText((a||t.getSourceFile()).text);var o=t.pos,s=function(e){r(i,o,e.pos,t),i.push(e),o=e.end};return e.forEach(t.jsDoc,s),o=t.pos,t.forEachChild(s,function(e){r(i,o,e.pos,t),i.push(function(e,t){var a=n(304,e.pos,e.end,t);a._children=[];for(var i=e.pos,o=0,s=e;o<s.length;o++){var l=s[o];r(a._children,i,l.pos,t),a._children.push(l),i=l.end}return r(a._children,i,e.end,t),a}(e,t)),o=e.end}),r(i,o,t.end,t),e.scanner.setText(void 0),i}(this,t))},t.prototype.getFirstToken=function(n){this.assertHasRealPosition();var t=this.getChildren(n);if(t.length){var r=e.find(t,function(e){return e.kind<281||e.kind>303});return r.kind<146?r:r.getFirstToken(n)}},t.prototype.getLastToken=function(n){this.assertHasRealPosition();var t=this.getChildren(n),r=e.lastOrUndefined(t);if(r)return r.kind<146?r:r.getLastToken(n)},t.prototype.forEachChild=function(n,t){return e.forEachChild(this,n,t)},t}();function r(t,r,a,i){for(e.scanner.setTextPos(r);r<a;){var o=e.scanner.scan(),s=e.scanner.getTextPos();if(s<=a&&(71===o&&e.Debug.fail(\"Did not expect \"+e.Debug.showSyntaxKind(i)+\" to have an Identifier in its trivia\"),t.push(n(o,r,s,i))),r=s,1===o)break}}var a=function(){function n(e,n){this.pos=e,this.end=n,this.flags=0,this.parent=void 0}return n.prototype.getSourceFile=function(){return e.getSourceFileOfNode(this)},n.prototype.getStart=function(n,t){return e.getTokenPosOfNode(this,n,t)},n.prototype.getFullStart=function(){return this.pos},n.prototype.getEnd=function(){return this.end},n.prototype.getWidth=function(e){return this.getEnd()-this.getStart(e)},n.prototype.getFullWidth=function(){return this.end-this.pos},n.prototype.getLeadingTriviaWidth=function(e){return this.getStart(e)-this.pos},n.prototype.getFullText=function(e){return(e||this.getSourceFile()).text.substring(this.pos,this.end)},n.prototype.getText=function(e){return e||(e=this.getSourceFile()),e.text.substring(this.getStart(e),this.getEnd())},n.prototype.getChildCount=function(){return 0},n.prototype.getChildAt=function(){},n.prototype.getChildren=function(){return 1===this.kind&&this.jsDoc||e.emptyArray},n.prototype.getFirstToken=function(){},n.prototype.getLastToken=function(){},n.prototype.forEachChild=function(){},n}(),i=function(){function n(e,n){this.flags=e,this.escapedName=n}return n.prototype.getFlags=function(){return this.flags},Object.defineProperty(n.prototype,\"name\",{get:function(){return e.symbolName(this)},enumerable:!0,configurable:!0}),n.prototype.getEscapedName=function(){return this.escapedName},n.prototype.getName=function(){return this.name},n.prototype.getDeclarations=function(){return this.declarations},n.prototype.getDocumentationComment=function(n){return this.documentationComment||(this.documentationComment=e.emptyArray,this.documentationComment=d(this.declarations,n)),this.documentationComment},n.prototype.getJsDocTags=function(){return void 0===this.tags&&(this.tags=e.JsDoc.getJsDocTagsFromDeclarations(this.declarations)),this.tags},n}(),o=function(e){function n(n,t,r){var a=e.call(this,t,r)||this;return a.kind=n,a}return En(n,e),n}(a),s=function(n){function t(e,t,r){return n.call(this,t,r)||this}return En(t,n),Object.defineProperty(t.prototype,\"text\",{get:function(){return e.idText(this)},enumerable:!0,configurable:!0}),t}(a);s.prototype.kind=71;var l=function(){function n(e,n){this.checker=e,this.flags=n}return n.prototype.getFlags=function(){return this.flags},n.prototype.getSymbol=function(){return this.symbol},n.prototype.getProperties=function(){return this.checker.getPropertiesOfType(this)},n.prototype.getProperty=function(e){return this.checker.getPropertyOfType(this,e)},n.prototype.getApparentProperties=function(){return this.checker.getAugmentedPropertiesOfType(this)},n.prototype.getCallSignatures=function(){return this.checker.getSignaturesOfType(this,0)},n.prototype.getConstructSignatures=function(){return this.checker.getSignaturesOfType(this,1)},n.prototype.getStringIndexType=function(){return this.checker.getIndexTypeOfType(this,0)},n.prototype.getNumberIndexType=function(){return this.checker.getIndexTypeOfType(this,1)},n.prototype.getBaseTypes=function(){return this.isClassOrInterface()?this.checker.getBaseTypes(this):void 0},n.prototype.getNonNullableType=function(){return this.checker.getNonNullableType(this)},n.prototype.getConstraint=function(){return this.checker.getBaseConstraintOfType(this)},n.prototype.getDefault=function(){return this.checker.getDefaultFromTypeParameter(this)},n.prototype.isUnion=function(){return!!(262144&this.flags)},n.prototype.isIntersection=function(){return!!(524288&this.flags)},n.prototype.isUnionOrIntersection=function(){return!!(786432&this.flags)},n.prototype.isLiteral=function(){return!!(192&this.flags)},n.prototype.isStringLiteral=function(){return!!(64&this.flags)},n.prototype.isNumberLiteral=function(){return!!(128&this.flags)},n.prototype.isTypeParameter=function(){return!!(65536&this.flags)},n.prototype.isClassOrInterface=function(){return!!(3&e.getObjectFlags(this))},n.prototype.isClass=function(){return!!(1&e.getObjectFlags(this))},n}(),c=function(){function n(e){this.checker=e}return n.prototype.getDeclaration=function(){return this.declaration},n.prototype.getTypeParameters=function(){return this.typeParameters},n.prototype.getParameters=function(){return this.parameters},n.prototype.getReturnType=function(){return this.checker.getReturnTypeOfSignature(this)},n.prototype.getDocumentationComment=function(){return this.documentationComment||(this.documentationComment=d(e.singleElementArray(this.declaration),this.checker))},n.prototype.getJsDocTags=function(){return void 0===this.jsDocTags&&(this.jsDocTags=this.declaration?e.JsDoc.getJsDocTagsFromDeclarations([this.declaration]):[]),this.jsDocTags},n}();function u(n){return e.getJSDocTags(n).some(function(e){return\"inheritDoc\"===e.tagName.text})}function d(n,t){if(!n)return e.emptyArray;var r=e.JsDoc.getJsDocCommentsFromDeclarations(n);if(0===r.length||n.some(u))for(var a=0,i=n;a<i.length;a++){var o=i[a],s=m(o,o.symbol.name,t);s&&(r=0===r.length?s.slice():s.concat(e.lineBreakPart(),r))}return r}function m(n,t,r){return e.firstDefined(n.parent?e.getAllSuperTypeNodes(n.parent):e.emptyArray,function(e){var n=r.getTypeAtLocation(e),a=n&&r.getPropertyOfType(n,t),i=a&&a.getDocumentationComment(r);return i&&i.length?i:void 0})}var p=function(n){function t(e,t,r){return n.call(this,e,t,r)||this}return En(t,n),t.prototype.update=function(n,t){return e.updateSourceFile(this,n,t)},t.prototype.getLineAndCharacterOfPosition=function(n){return e.getLineAndCharacterOfPosition(this,n)},t.prototype.getLineStarts=function(){return e.getLineStarts(this)},t.prototype.getPositionOfLineAndCharacter=function(n,t){return e.getPositionOfLineAndCharacter(this,n,t)},t.prototype.getLineEndOfPosition=function(e){var n,t=this.getLineAndCharacterOfPosition(e).line,r=this.getLineStarts();t+1>=r.length&&(n=this.getEnd()),n||(n=r[t+1]-1);var a=this.getFullText();return\"\\n\"===a[n]&&\"\\r\"===a[n-1]?n-1:n},t.prototype.getNamedDeclarations=function(){return this.namedDeclarations||(this.namedDeclarations=this.computeNamedDeclarations()),this.namedDeclarations},t.prototype.computeNamedDeclarations=function(){var n=e.createMultiMap();return this.forEachChild(function a(i){switch(i.kind){case 237:case 194:case 154:case 153:var o=i,s=r(o);if(s){var l=function(e){var t=n.get(e);t||n.set(e,t=[]);return t}(s),c=e.lastOrUndefined(l);c&&o.parent===c.parent&&o.symbol===c.symbol?o.body&&!c.body&&(l[l.length-1]=o):l.push(o)}e.forEachChild(i,a);break;case 238:case 207:case 239:case 240:case 241:case 242:case 246:case 255:case 251:case 248:case 249:case 156:case 157:case 166:t(i),e.forEachChild(i,a);break;case 149:if(!e.hasModifier(i,92))break;case 235:case 184:var u=i;if(e.isBindingPattern(u.name)){e.forEachChild(u.name,a);break}u.initializer&&a(u.initializer);case 276:case 152:case 151:t(i);break;case 253:i.exportClause&&e.forEach(i.exportClause.elements,a);break;case 247:var d=i.importClause;d&&(d.name&&t(d.name),d.namedBindings&&(249===d.namedBindings.kind?t(d.namedBindings):e.forEach(d.namedBindings.elements,a)));break;case 202:0!==e.getAssignmentDeclarationKind(i)&&t(i);default:e.forEachChild(i,a)}}),n;function t(e){var t=r(e);t&&n.add(t,e)}function r(n){var t=e.getNonAssignedNameOfDeclaration(n);return t&&(e.isComputedPropertyName(t)&&e.isPropertyAccessExpression(t.expression)?t.expression.name.text:e.isPropertyName(t)?e.getNameFromPropertyName(t):void 0)}},t}(t),f=function(){function n(e,n,t){this.fileName=e,this.text=n,this.skipTrivia=t}return n.prototype.getLineAndCharacterOfPosition=function(n){return e.getLineAndCharacterOfPosition(this,n)},n}();function g(n){var t=!0;for(var r in n)if(e.hasProperty(n,r)&&!_(r)){t=!1;break}if(t)return n;var a={};for(var r in n){if(e.hasProperty(n,r))a[_(r)?r:r.charAt(0).toLowerCase()+r.substr(1)]=n[r]}return a}function _(e){return!e.length||e.charAt(0)===e.charAt(0).toLowerCase()}function v(){return{target:1,jsx:1}}e.toEditorSettings=g,e.displayPartsToString=function(n){return n?e.map(n,function(e){return e.text}).join(\"\"):\"\"},e.getDefaultCompilerOptions=v,e.getSupportedCodeFixes=function(){return e.codefix.getSupportedErrorCodes()};var y=function(){function n(n,t){this.host=n,this.currentDirectory=n.getCurrentDirectory(),this.fileNameToEntry=e.createMap();for(var r=0,a=n.getScriptFileNames();r<a.length;r++){var i=a[r];this.createEntry(i,e.toPath(i,this.currentDirectory,t))}this._compilationSettings=n.getCompilationSettings()||{target:1,jsx:1}}return n.prototype.compilationSettings=function(){return this._compilationSettings},n.prototype.getProjectReferences=function(){return this.host.getProjectReferences&&this.host.getProjectReferences()},n.prototype.createEntry=function(n,t){var r,a=this.host.getScriptSnapshot(n);return r=a?{hostFileName:n,version:this.host.getScriptVersion(n),scriptSnapshot:a,scriptKind:e.getScriptKind(n,this.host)}:n,this.fileNameToEntry.set(t,r),r},n.prototype.getEntryByPath=function(e){return this.fileNameToEntry.get(e)},n.prototype.getHostFileInformation=function(n){var t=this.fileNameToEntry.get(n);return e.isString(t)?void 0:t},n.prototype.getOrCreateEntryByPath=function(n,t){var r=this.getEntryByPath(t)||this.createEntry(n,t);return e.isString(r)?void 0:r},n.prototype.getRootFileNames=function(){var n=[];return this.fileNameToEntry.forEach(function(t){e.isString(t)?n.push(t):6!==t.scriptKind&&n.push(t.hostFileName)}),n},n.prototype.getVersion=function(e){var n=this.getHostFileInformation(e);return n&&n.version},n.prototype.getScriptSnapshot=function(e){var n=this.getHostFileInformation(e);return n&&n.scriptSnapshot},n}(),h=function(){function n(e){this.host=e}return n.prototype.getCurrentSourceFile=function(n){var t=this.host.getScriptSnapshot(n);if(!t)throw new Error(\"Could not find file: '\"+n+\"'.\");var r,a=e.getScriptKind(n,this.host),i=this.host.getScriptVersion(n);if(this.currentFileName!==n)r=E(n,t,6,i,!0,a);else if(this.currentFileVersion!==i){var o=t.getChangeRange(this.currentFileScriptSnapshot);r=T(this.currentSourceFile,t,i,o)}return r&&(this.currentFileVersion=i,this.currentFileName=n,this.currentFileScriptSnapshot=t,this.currentSourceFile=r),this.currentSourceFile},n}();function b(e,n,t){e.version=t,e.scriptSnapshot=n}function E(n,t,r,a,i,o){var s=e.createSourceFile(n,e.getSnapshotText(t),r,i,o);return b(s,t,a),s}function T(n,t,r,a,i){if(a&&r!==n.version&&!e.disableIncrementalParsing){var o=void 0,s=0!==a.span.start?n.text.substr(0,a.span.start):\"\",l=e.textSpanEnd(a.span)!==n.text.length?n.text.substr(e.textSpanEnd(a.span)):\"\";if(0===a.newLength)o=s&&l?s+l:s||l;else{var c=t.getText(a.span.start,a.span.start+a.newLength);o=s&&l?s+c+l:s?s+c:c+l}var u=e.updateSourceFile(n,o,a,i);return b(u,t,r),u.nameTable=void 0,n!==u&&n.scriptSnapshot&&(n.scriptSnapshot.dispose&&n.scriptSnapshot.dispose(),n.scriptSnapshot=void 0),u}return E(n.fileName,t,n.languageVersion,r,!0,n.scriptKind)}e.createLanguageServiceSourceFile=E,e.disableIncrementalParsing=!1,e.updateLanguageServiceSourceFile=T;var S=function(){function n(e){this.cancellationToken=e}return n.prototype.isCancellationRequested=function(){return!!this.cancellationToken&&this.cancellationToken.isCancellationRequested()},n.prototype.throwIfCancellationRequested=function(){if(this.isCancellationRequested())throw new e.OperationCanceledException},n}(),L=function(){function n(e,n){void 0===n&&(n=20),this.hostCancellationToken=e,this.throttleWaitMilliseconds=n,this.lastCancellationCheckTime=0}return n.prototype.isCancellationRequested=function(){var n=e.timestamp();return Math.abs(n-this.lastCancellationCheckTime)>=this.throttleWaitMilliseconds&&(this.lastCancellationCheckTime=n,this.hostCancellationToken.isCancellationRequested())},n.prototype.throwIfCancellationRequested=function(){if(this.isCancellationRequested())throw new e.OperationCanceledException},n}();function x(n){var t=function(n){switch(n.kind){case 9:case 8:if(147===n.parent.kind)return e.isObjectLiteralElement(n.parent.parent)?n.parent.parent:void 0;case 71:return!e.isObjectLiteralElement(n.parent)||186!==n.parent.parent.kind&&266!==n.parent.parent.kind||n.parent.name!==n?void 0:n.parent}return}(n);return t&&(e.isObjectLiteralExpression(t.parent)||e.isJsxAttributes(t.parent))?t:void 0}function A(n,t,r,a){var i=e.getNameFromPropertyName(n.name);if(!i)return e.emptyArray;if(!r.isUnion())return(o=r.getProperty(i))?[o]:e.emptyArray;var o,s=e.mapDefined(r.types,function(r){return e.isObjectLiteralExpression(n.parent)&&t.isTypeInvalidDueToUnionDiscriminant(r,n.parent)?void 0:r.getProperty(i)});if(a&&(0===s.length||s.length===r.types.length)&&(o=r.getProperty(i)))return[o];return 0===s.length?e.mapDefined(r.types,function(e){return e.getProperty(i)}):s}e.ThrottledCancellationToken=L,e.createLanguageService=function(n,t,r){var a;void 0===t&&(t=e.createDocumentRegistry(n.useCaseSensitiveFileNames&&n.useCaseSensitiveFileNames(),n.getCurrentDirectory())),void 0===r&&(r=!1);var i,o,s=new h(n),l=0,c=new S(n.getCancellationToken&&n.getCancellationToken()),u=n.getCurrentDirectory();function d(e){n.log&&n.log(e)}!e.localizedDiagnosticMessages&&n.getLocalizedDiagnosticMessages&&(e.localizedDiagnosticMessages=n.getLocalizedDiagnosticMessages());var m=e.hostUsesCaseSensitiveFileNames(n),p=e.createGetCanonicalFileName(m),f=e.getSourceMapper(p,u,d,n,function(){return i});function _(e){var n=i.getSourceFile(e);if(!n)throw new Error(\"Could not find file: '\"+e+\"'.\");return n}function v(){if(e.Debug.assert(!r),n.getProjectVersion){var a=n.getProjectVersion();if(a){if(o===a&&!n.hasChangedAutomaticTypeDirectiveNames)return;o=a}}var s=n.getTypeRootsVersion?n.getTypeRootsVersion():0;l!==s&&(d(\"TypeRoots version has changed; provide new program\"),i=void 0,l=s);var g=new y(n,p),_=g.getRootFileNames(),v=n.hasInvalidatedResolution||e.returnFalse,h=g.getProjectReferences();if(!e.isProgramUptoDate(i,_,g.compilationSettings(),function(e){return g.getVersion(e)},L,v,!!n.hasChangedAutomaticTypeDirectiveNames,h)){var b=g.compilationSettings(),E={getSourceFile:function(n,t,r,a){return x(n,e.toPath(n,u,p),0,0,a)},getSourceFileByPath:x,getCancellationToken:function(){return c},getCanonicalFileName:p,useCaseSensitiveFileNames:function(){return m},getNewLine:function(){return e.getNewLineCharacter(b,function(){return e.getNewLineOrDefaultFromHost(n)})},getDefaultLibFileName:function(e){return n.getDefaultLibFileName(e)},writeFile:e.noop,getCurrentDirectory:function(){return u},fileExists:L,readFile:function(t){var r=e.toPath(t,u,p),a=g&&g.getEntryByPath(r);return a?e.isString(a)?void 0:e.getSnapshotText(a.scriptSnapshot):n.readFile&&n.readFile(t)},realpath:n.realpath&&function(e){return n.realpath(e)},directoryExists:function(t){return e.directoryProbablyExists(t,n)},getDirectories:function(e){return n.getDirectories?n.getDirectories(e):[]},readDirectory:function(t,r,a,i,o){return e.Debug.assertDefined(n.readDirectory,\"'LanguageServiceHost.readDirectory' must be implemented to correctly process 'projectReferences'\"),n.readDirectory(t,r,a,i,o)},onReleaseOldSourceFile:function(e,n){var r=t.getKeyForCompilationSettings(n);t.releaseDocumentWithKey(e.path,r)},hasInvalidatedResolution:v,hasChangedAutomaticTypeDirectiveNames:n.hasChangedAutomaticTypeDirectiveNames};n.trace&&(E.trace=function(e){return n.trace(e)}),n.resolveModuleNames&&(E.resolveModuleNames=function(e,t,r){return n.resolveModuleNames(e,t,r)}),n.resolveTypeReferenceDirectives&&(E.resolveTypeReferenceDirectives=function(e,t){return n.resolveTypeReferenceDirectives(e,t)});var T=t.getKeyForCompilationSettings(b),S={rootNames:_,options:b,host:E,oldProgram:i,projectReferences:h};return i=e.createProgram(S),g=void 0,f.clearCache(),void i.getTypeChecker()}function L(t){var r=e.toPath(t,u,p),a=g&&g.getEntryByPath(r);return a?!e.isString(a):!!n.fileExists&&n.fileExists(t)}function x(n,r,a,o,s){e.Debug.assert(void 0!==g,\"getOrCreateSourceFileByPath called after typical CompilerHost lifetime, check the callstack something with a reference to an old host.\");var l=g&&g.getOrCreateEntryByPath(n,r);if(l){if(!s){var c=i&&i.getSourceFileByPath(r);if(c)return e.Debug.assertEqual(l.scriptKind,c.scriptKind,\"Registered script kind should match new script kind.\",r),t.updateDocumentWithKey(n,r,b,T,l.scriptSnapshot,l.version,l.scriptKind)}return t.acquireDocumentWithKey(n,r,b,T,l.scriptSnapshot,l.version,l.scriptKind)}}}function b(){if(!r)return v(),i;e.Debug.assert(void 0===i)}function E(n,t,r){var a=e.normalizePath(n);e.Debug.assert(r.some(function(n){return e.normalizePath(n)===a})),v();var o=e.map(r,function(n){return e.Debug.assertDefined(i.getSourceFile(n))}),s=_(n);return e.DocumentHighlights.getDocumentHighlights(i,c,s,t,o)}function T(n,t,r){v();var a=r&&r.isForRename?i.getSourceFiles().filter(function(e){return!i.isSourceFileDefaultLibrary(e)}):i.getSourceFiles();return e.FindAllReferences.findReferencedEntries(i,c,a,n,t,r)}function L(t){var r=e.getScriptKind(t,n);return 3===r||4===r}var C=e.createMapFromTemplate(((a={})[17]=18,a[19]=20,a[21]=22,a[29]=27,a));function D(t,r){var a=function(n){return e.toPath(n,u,p)};switch(t.type){case\"install package\":return n.installPackage?n.installPackage({fileName:a(t.file),packageName:t.packageName}):Promise.reject(\"Host does not implement `installPackage`\");case\"generate types\":var i=t.fileToGenerateTypesFor,o=t.outputFileName;return n.inspectValue?n.inspectValue({fileNameToRequire:i}).then(function(t){var i=a(o);return n.writeFile(i,e.valueInfoToDeclarationFileText(t,r||e.testFormatSettings)),{successMessage:\"Wrote types to '\"+i+\"'\"}}):Promise.reject(\"Host does not implement `installPackage`\");default:return e.Debug.assertNever(t)}}function k(t,r,a,i){var o=\"number\"==typeof r?[r,void 0]:[r.pos,r.end];return{file:t,startPosition:o[0],endPosition:o[1],program:b(),host:n,formatContext:e.formatting.getFormatContext(i),cancellationToken:c,preferences:a}}return C.forEach(function(e,n){return C.set(e.toString(),Number(n))}),{dispose:function(){i&&(e.forEach(i.getSourceFiles(),function(e){return t.releaseDocument(e.fileName,i.getCompilerOptions())}),i=void 0),n=void 0},cleanupSemanticCache:function(){i=void 0},getSyntacticDiagnostics:function(e){return v(),i.getSyntacticDiagnostics(_(e),c).slice()},getSemanticDiagnostics:function(n){v();var t=_(n),r=i.getSemanticDiagnostics(t,c);if(!e.getEmitDeclarations(i.getCompilerOptions()))return r.slice();var a=i.getDeclarationDiagnostics(t,c);return r.concat(a)},getSuggestionDiagnostics:function(n){return v(),e.computeSuggestionDiagnostics(_(n),i,c)},getCompilerOptionsDiagnostics:function(){return v(),i.getOptionsDiagnostics(c).concat(i.getGlobalDiagnostics(c))},getSyntacticClassifications:function(n,t){return e.getSyntacticClassifications(c,s.getCurrentSourceFile(n),t)},getSemanticClassifications:function(n,t){return L(n)?(v(),e.getSemanticClassifications(i.getTypeChecker(),c,_(n),i.getClassifiableNames(),t)):[]},getEncodedSyntacticClassifications:function(n,t){return e.getEncodedSyntacticClassifications(c,s.getCurrentSourceFile(n),t)},getEncodedSemanticClassifications:function(n,t){return L(n)?(v(),e.getEncodedSemanticClassifications(i.getTypeChecker(),c,_(n),i.getClassifiableNames(),t)):{spans:[],endOfLineState:0}},getCompletionsAtPosition:function(t,r,a){void 0===a&&(a=e.emptyOptions);var o=yn({},e.identity(a),{includeCompletionsForModuleExports:a.includeCompletionsForModuleExports||a.includeExternalModuleExports,includeCompletionsWithInsertText:a.includeCompletionsWithInsertText||a.includeInsertTextCompletions});return v(),e.Completions.getCompletionsAtPosition(n,i,d,_(t),r,o,a.triggerCharacter)},getCompletionEntryDetails:function(t,r,a,o,s,l){return void 0===l&&(l=e.emptyOptions),v(),e.Completions.getCompletionEntryDetails(i,d,_(t),r,{name:a,source:s},n,o&&e.formatting.getFormatContext(o),l,c)},getCompletionEntrySymbol:function(n,t,r,a){return v(),e.Completions.getCompletionEntrySymbol(i,d,_(n),t,{name:r,source:a})},getSignatureHelpItems:function(n,t,r){var a=(void 0===r?e.emptyOptions:r).triggerReason;v();var o=_(n);return e.SignatureHelp.getSignatureHelpItems(i,o,t,a,c)},getQuickInfoAtPosition:function(n,t){v();var r=_(n),a=e.getTouchingPropertyName(r,t);if(a!==r){var o=i.getTypeChecker(),s=function(n,t){var r=x(n);if(r){var a=t.getContextualType(r.parent),i=a&&A(r,t,a,!1);if(i&&1===i.length)return e.first(i)}return t.getSymbolAtLocation(n)}(a,o);if(!s||o.isUnknownSymbol(s)){var l=function(n,t,r){switch(t.kind){case 71:return!e.isLabelName(t);case 187:case 146:return!e.isInComment(n,r);case 99:case 176:case 97:return!0;default:return!1}}(r,a,t)?o.getTypeAtLocation(a):void 0;return l&&{kind:\"\",kindModifiers:\"\",textSpan:e.createTextSpanFromNode(a,r),displayParts:o.runWithCancellationToken(c,function(n){return e.typeToDisplayParts(n,l,e.getContainerNode(a))}),documentation:l.symbol?l.symbol.getDocumentationComment(o):void 0,tags:l.symbol?l.symbol.getJsDocTags():void 0}}var u=o.runWithCancellationToken(c,function(n){return e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(n,s,r,e.getContainerNode(a),a)}),d=u.symbolKind,m=u.displayParts,p=u.documentation,f=u.tags;return{kind:d,kindModifiers:e.SymbolDisplay.getSymbolModifiers(s),textSpan:e.createTextSpanFromNode(a,r),displayParts:m,documentation:p,tags:f}}},getDefinitionAtPosition:function(n,t){return v(),e.GoToDefinition.getDefinitionAtPosition(i,_(n),t)},getDefinitionAndBoundSpan:function(n,t){return v(),e.GoToDefinition.getDefinitionAndBoundSpan(i,_(n),t)},getImplementationAtPosition:function(n,t){return v(),e.FindAllReferences.getImplementationsAtPosition(i,c,i.getSourceFiles(),_(n),t)},getTypeDefinitionAtPosition:function(n,t){return v(),e.GoToDefinition.getTypeDefinitionAtPosition(i.getTypeChecker(),_(n),t)},getReferencesAtPosition:function(n,t){return v(),T(e.getTouchingPropertyName(_(n),t),t)},findReferences:function(n,t){return v(),e.FindAllReferences.findReferencedSymbols(i,c,i.getSourceFiles(),_(n),t)},getOccurrencesAtPosition:function(n,t){return e.flatMap(E(n,t,[n]),function(e){return e.highlightSpans.map(function(n){return{fileName:e.fileName,textSpan:n.textSpan,isWriteAccess:\"writtenReference\"===n.kind,isDefinition:!1,isInString:n.isInString}})})},getDocumentHighlights:E,getNameOrDottedNameSpan:function(n,t,r){var a=s.getCurrentSourceFile(n),i=e.getTouchingPropertyName(a,t);if(i!==a){switch(i.kind){case 187:case 146:case 9:case 86:case 101:case 95:case 97:case 99:case 176:case 71:break;default:return}for(var o=i;;)if(e.isRightSideOfPropertyAccess(o)||e.isRightSideOfQualifiedName(o))o=o.parent;else{if(!e.isNameOfModuleDeclaration(o))break;if(242!==o.parent.parent.kind||o.parent.parent.body!==o.parent)break;o=o.parent.parent.name}return e.createTextSpanFromBounds(o.getStart(),i.getEnd())}},getBreakpointStatementAtPosition:function(n,t){var r=s.getCurrentSourceFile(n);return e.BreakpointResolver.spanInSourceFileAtLocation(r,t)},getNavigateToItems:function(n,t,r,a){void 0===a&&(a=!1),v();var o=r?[_(r)]:i.getSourceFiles();return e.NavigateTo.getNavigateToItems(o,i.getTypeChecker(),c,n,t,a)},getRenameInfo:function(n,t){return v(),e.Rename.getRenameInfo(i,_(n),t)},findRenameLocations:function(n,t,r,a){v();var i=_(n),o=e.getTouchingPropertyName(i,t);if(e.isIdentifier(o)&&e.isJsxOpeningElement(o.parent)||e.isJsxClosingElement(o.parent)){var s=o.parent.parent;return[s.openingElement,s.closingElement].map(function(n){return{fileName:i.fileName,textSpan:e.createTextSpanFromNode(n.tagName,i)}})}var l=T(o,t,{findInStrings:r,findInComments:a,isForRename:!0});return l&&l.map(function(e){return{fileName:e.fileName,textSpan:e.textSpan}})},getNavigationBarItems:function(n){return e.NavigationBar.getNavigationBarItems(s.getCurrentSourceFile(n),c)},getNavigationTree:function(n){return e.NavigationBar.getNavigationTree(s.getCurrentSourceFile(n),c)},getOutliningSpans:function(n){var t=s.getCurrentSourceFile(n);return e.OutliningElementsCollector.collectElements(t,c)},getTodoComments:function(n,t){v();var r=_(n);c.throwIfCancellationRequested();var a,i,o=r.text,s=[];if(t.length>0&&(i=r.fileName,!e.stringContains(i,\"/node_modules/\")))for(var l=function(){var n=\"(\"+/(?:^(?:\\s|\\*)*)/.source+\"|\"+/(?:\\/\\/+\\s*)/.source+\"|\"+/(?:\\/\\*+\\s*)/.source+\")\",r=\"(?:\"+e.map(t,function(e){return\"(\"+e.text.replace(/[\\-\\[\\]\\/\\{\\}\\(\\)\\*\\+\\?\\.\\\\\\^\\$\\|]/g,\"\\\\$&\")+\")\"}).join(\"|\")+\")\";return new RegExp(n+\"(\"+r+/(?:.*?)/.source+\")\"+/(?:$|\\*\\/)/.source,\"gim\")}(),u=void 0;u=l.exec(o);){c.throwIfCancellationRequested(),e.Debug.assert(u.length===t.length+3);var d=u[1],m=u.index+d.length;if(e.isInComment(r,m)){for(var p=void 0,f=0;f<t.length;f++)u[f+3]&&(p=t[f]);if(void 0===p)return e.Debug.fail();if(!((a=o.charCodeAt(m+p.text.length))>=97&&a<=122||a>=65&&a<=90||a>=48&&a<=57)){var g=u[2];s.push({descriptor:p,message:g,position:m})}}}return s},getBraceMatchingAtPosition:function(n,t){var r=s.getCurrentSourceFile(n),a=e.getTouchingToken(r,t),i=a.getStart(r)===t?C.get(a.kind.toString()):void 0,o=i&&e.findChildOfKind(a.parent,i,r);return o?[e.createTextSpanFromNode(a,r),e.createTextSpanFromNode(o,r)].sort(function(e,n){return e.start-n.start}):e.emptyArray},getIndentationAtPosition:function(n,t,r){var a=e.timestamp(),i=g(r),o=s.getCurrentSourceFile(n);d(\"getIndentationAtPosition: getCurrentSourceFile: \"+(e.timestamp()-a)),a=e.timestamp();var l=e.formatting.SmartIndenter.getIndentation(t,o,i);return d(\"getIndentationAtPosition: computeIndentation  : \"+(e.timestamp()-a)),l},getFormattingEditsForRange:function(n,t,r,a){var i=s.getCurrentSourceFile(n);return e.formatting.formatSelection(t,r,i,e.formatting.getFormatContext(g(a)))},getFormattingEditsForDocument:function(n,t){return e.formatting.formatDocument(s.getCurrentSourceFile(n),e.formatting.getFormatContext(g(t)))},getFormattingEditsAfterKeystroke:function(n,t,r,a){var i=s.getCurrentSourceFile(n),o=e.formatting.getFormatContext(g(a));if(!e.isInComment(i,t))switch(r){case\"{\":return e.formatting.formatOnOpeningCurly(t,i,o);case\"}\":return e.formatting.formatOnClosingCurly(t,i,o);case\";\":return e.formatting.formatOnSemicolon(t,i,o);case\"\\n\":return e.formatting.formatOnEnter(t,i,o)}return[]},getDocCommentTemplateAtPosition:function(t,r){return e.JsDoc.getDocCommentTemplateAtPosition(e.getNewLineOrDefaultFromHost(n),s.getCurrentSourceFile(t),r)},isValidBraceCompletionAtPosition:function(n,t,r){if(60===r)return!1;var a=s.getCurrentSourceFile(n);if(e.isInString(a,t))return!1;if(e.isInsideJsxElementOrAttribute(a,t))return 123===r;if(e.isInTemplateString(a,t))return!1;switch(r){case 39:case 34:case 96:return!e.isInComment(a,t)}return!0},getJsxClosingTagAtPosition:function(n,t){var r=s.getCurrentSourceFile(n),a=e.findPrecedingToken(t,r);if(a){var i=29===a.kind&&e.isJsxOpeningElement(a.parent)?a.parent.parent:e.isJsxText(a)?a.parent:void 0;return i&&function n(t){var r=t.openingElement,a=t.closingElement,i=t.parent;return!e.tagNamesAreEquivalent(r.tagName,a.tagName)||e.isJsxElement(i)&&e.tagNamesAreEquivalent(r.tagName,i.openingElement.tagName)&&n(i)}(i)?{newText:\"</\"+i.openingElement.tagName.getText(r)+\">\"}:void 0}},getSpanOfEnclosingComment:function(n,t,r){var a=s.getCurrentSourceFile(n),i=e.formatting.getRangeOfEnclosingComment(a,t);return!i||r&&3!==i.kind?void 0:e.createTextSpanFromRange(i)},getCodeFixesAtPosition:function(t,r,a,o,s,l){void 0===l&&(l=e.emptyOptions),v();var u=_(t),d=e.createTextSpanFromBounds(r,a),m=e.formatting.getFormatContext(s);return e.flatMap(e.deduplicate(o,e.equateValues,e.compareValues),function(t){return c.throwIfCancellationRequested(),e.codefix.getFixes({errorCode:t,sourceFile:u,span:d,program:i,host:n,cancellationToken:c,formatContext:m,preferences:l})})},getCombinedCodeFix:function(t,r,a,o){void 0===o&&(o=e.emptyOptions),v(),e.Debug.assert(\"file\"===t.type);var s=_(t.fileName),l=e.formatting.getFormatContext(a);return e.codefix.getAllFixes({fixId:r,sourceFile:s,program:i,host:n,cancellationToken:c,formatContext:l,preferences:o})},applyCodeActionCommand:function(n,t){var r=\"string\"==typeof n?t:n,a=\"string\"!=typeof n?t:void 0;return e.isArray(r)?Promise.all(r.map(function(e){return D(e,a)})):D(r,a)},organizeImports:function(t,r,a){void 0===a&&(a=e.emptyOptions),v(),e.Debug.assert(\"file\"===t.type);var o=_(t.fileName),s=e.formatting.getFormatContext(r);return e.OrganizeImports.organizeImports(o,s,n,i,a)},getEditsForFileRename:function(t,r,a,i){return void 0===i&&(i=e.emptyOptions),e.getEditsForFileRename(b(),t,r,n,e.formatting.getFormatContext(a),i,f)},getEmitOutput:function(t,r){void 0===r&&(r=!1),v();var a=_(t),o=n.getCustomTransformers&&n.getCustomTransformers();return e.getFileEmitOutput(i,a,r,c,o)},getNonBoundSourceFile:function(e){return s.getCurrentSourceFile(e)},getProgram:b,getApplicableRefactors:function(n,t,r){void 0===r&&(r=e.emptyOptions),v();var a=_(n);return e.refactor.getApplicableRefactors(k(a,t,r))},getEditsForRefactor:function(n,t,r,a,i,o){void 0===o&&(o=e.emptyOptions),v();var s=_(n);return e.refactor.getEditsForRefactor(k(s,r,o,t),a,i)},toLineColumnOffset:f.toLineColumnOffset,getSourceMapper:function(){return f}}},e.getNameTable=function(n){return n.nameTable||function(n){var t=n.nameTable=e.createUnderscoreEscapedMap();n.forEachChild(function n(r){if(e.isIdentifier(r)&&r.escapedText||e.isStringOrNumericLiteralLike(r)&&function(n){return e.isDeclarationName(n)||257===n.parent.kind||function(e){return e&&e.parent&&188===e.parent.kind&&e.parent.argumentExpression===e}(n)||e.isLiteralComputedPropertyDeclarationName(n)}(r)){var a=e.getEscapedTextOfIdentifierOrLiteral(r);t.set(a,void 0===t.get(a)?r.pos:-1)}if(e.forEachChild(r,n),e.hasJSDocNodes(r))for(var i=0,o=r.jsDoc;i<o.length;i++){var s=o[i];e.forEachChild(s,n)}})}(n),n.nameTable},e.getContainingObjectLiteralElement=x,e.getPropertySymbolsFromContextualType=A,e.getDefaultLibFilePath=function(n){if(\"undefined\"!=typeof __dirname)return __dirname+e.directorySeparator+e.getDefaultLibFileName(n);throw new Error(\"getDefaultLibFilePath is only supported when consumed as a node module. \")},e.objectAllocator={getNodeConstructor:function(){return t},getTokenConstructor:function(){return o},getIdentifierConstructor:function(){return s},getSourceFileConstructor:function(){return p},getSymbolConstructor:function(){return i},getTypeConstructor:function(){return l},getSignatureConstructor:function(){return c},getSourceMapSourceConstructor:function(){return f}}}(Tn||(Tn={})),function(e){!function(n){n.spanInSourceFileAtLocation=function(n,t){if(!n.isDeclarationFile){var r=e.getTokenAtPosition(n,t),a=n.getLineAndCharacterOfPosition(t).line;if(n.getLineAndCharacterOfPosition(r.getStart(n)).line>a){var i=e.findPrecedingToken(r.pos,n);if(!i||n.getLineAndCharacterOfPosition(i.getEnd()).line!==a)return;r=i}if(!(4194304&r.flags))return d(r)}function o(t,r){var a=t.decorators?e.skipTrivia(n.text,t.decorators.end):t.getStart(n);return e.createTextSpanFromBounds(a,(r||t).getEnd())}function s(t,r){return o(t,e.findNextToken(r,r.parent,n))}function l(e,t){return e&&a===n.getLineAndCharacterOfPosition(e.getStart(n)).line?d(e):d(t)}function c(t){return d(e.findPrecedingToken(t.pos,n))}function u(t){return d(e.findNextToken(t,t.parent,n))}function d(t){if(t){var r=t.parent;switch(t.kind){case 217:return E(t.declarationList.declarations[0]);case 235:case 152:case 151:return E(t);case 149:return function n(t){if(e.isBindingPattern(t.name))return x(t.name);if(function(n){return!!n.initializer||void 0!==n.dotDotDotToken||e.hasModifier(n,12)}(t))return o(t);var r=t.parent,a=r.parameters.indexOf(t);return e.Debug.assert(-1!==a),0!==a?n(r.parameters[a-1]):d(r.body)}(t);case 237:case 154:case 153:case 156:case 157:case 155:case 194:case 195:return function(e){if(e.body)return T(e)?o(e):d(e.body)}(t);case 216:if(e.isFunctionBlock(t))return y=(v=t).statements.length?v.statements[0]:v.getLastToken(),T(v.parent)?l(v.parent,y):d(y);case 243:return S(t);case 272:return S(t.block);case 219:return o(t.expression);case 228:return o(t.getChildAt(0),t.expression);case 222:return s(t,t.expression);case 221:return d(t.statement);case 234:return o(t.getChildAt(0));case 220:return s(t,t.expression);case 231:return d(t.statement);case 227:case 226:return o(t.getChildAt(0),t.label);case 223:return(_=t).initializer?L(_):_.condition?o(_.condition):_.incrementor?o(_.incrementor):void 0;case 224:return s(t,t.expression);case 225:return L(t);case 230:return s(t,t.expression);case 269:case 270:return d(t.statements[0]);case 233:return S(t.tryBlock);case 232:case 252:return o(t,t.expression);case 246:return o(t,t.moduleReference);case 247:case 253:return o(t,t.moduleSpecifier);case 242:if(1!==e.getModuleInstanceState(t))return;case 238:case 241:case 276:case 184:return o(t);case 229:return d(t.statement);case 150:return h=r.decorators,e.createTextSpanFromBounds(e.skipTrivia(n.text,h.pos),h.end);case 182:case 183:return x(t);case 239:case 240:return;case 25:case 1:return l(e.findPrecedingToken(t.pos,n));case 26:return c(t);case 17:return function(t){switch(t.parent.kind){case 241:var r=t.parent;return l(e.findPrecedingToken(t.pos,n,t.parent),r.members.length?r.members[0]:r.getLastToken(n));case 238:var a=t.parent;return l(e.findPrecedingToken(t.pos,n,t.parent),a.members.length?a.members[0]:a.getLastToken(n));case 244:return l(t.parent.parent,t.parent.clauses[0])}return d(t.parent)}(t);case 18:return function(n){switch(n.parent.kind){case 243:if(1!==e.getModuleInstanceState(n.parent.parent))return;case 241:case 238:return o(n);case 216:if(e.isFunctionBlock(n.parent))return o(n);case 272:return d(e.lastOrUndefined(n.parent.statements));case 244:var t=n.parent,r=e.lastOrUndefined(t.clauses);return r?d(e.lastOrUndefined(r.statements)):void 0;case 182:var a=n.parent;return d(e.lastOrUndefined(a.elements)||a);default:if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent)){var i=n.parent;return o(e.lastOrUndefined(i.properties)||i)}return d(n.parent)}}(t);case 22:return function(n){switch(n.parent.kind){case 183:var t=n.parent;return o(e.lastOrUndefined(t.elements)||t);default:if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent)){var r=n.parent;return o(e.lastOrUndefined(r.elements)||r)}return d(n.parent)}}(t);case 19:return function(e){return 221===e.parent.kind||189===e.parent.kind||190===e.parent.kind?c(e):193===e.parent.kind?u(e):d(e.parent)}(t);case 20:return function(e){switch(e.parent.kind){case 194:case 237:case 195:case 154:case 153:case 156:case 157:case 155:case 222:case 221:case 223:case 225:case 189:case 190:case 193:return c(e);default:return d(e.parent)}}(t);case 56:return function(n){return e.isFunctionLike(n.parent)||273===n.parent.kind||149===n.parent.kind?c(n):d(n.parent)}(t);case 29:case 27:return function(e){return 192===e.parent.kind?u(e):d(e.parent)}(t);case 106:return function(e){return 221===e.parent.kind?s(e,e.parent.expression):d(e.parent)}(t);case 82:case 74:case 87:return u(t);case 145:return function(e){return 225===e.parent.kind?u(e):d(e.parent)}(t);default:if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(t))return A(t);if((71===t.kind||206===t.kind||273===t.kind||274===t.kind)&&e.isArrayLiteralOrObjectLiteralDestructuringPattern(r))return o(t);if(202===t.kind){var a=t,i=a.left,m=a.operatorToken;if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(i))return A(i);if(58===m.kind&&e.isArrayLiteralOrObjectLiteralDestructuringPattern(t.parent))return o(t);if(26===m.kind)return d(i)}if(e.isExpressionNode(t))switch(r.kind){case 221:return c(t);case 150:return d(t.parent);case 223:case 225:return o(t);case 202:if(26===t.parent.operatorToken.kind)return o(t);break;case 195:if(t.parent.body===t)return o(t)}switch(t.parent.kind){case 273:if(t.parent.name===t&&!e.isArrayLiteralOrObjectLiteralDestructuringPattern(t.parent.parent))return d(t.parent.initializer);break;case 192:if(t.parent.type===t)return u(t.parent.type);break;case 235:case 149:var p=t.parent,f=p.initializer,g=p.type;if(f===t||g===t||e.isAssignmentOperator(t.kind))return c(t);break;case 202:if(i=t.parent.left,e.isArrayLiteralOrObjectLiteralDestructuringPattern(i)&&t!==i)return c(t);break;default:if(e.isFunctionLike(t.parent)&&t.parent.type===t)return c(t)}return d(t.parent)}}var _,v,y,h;function b(t){return e.isVariableDeclarationList(t.parent)&&t.parent.declarations[0]===t?o(e.findPrecedingToken(t.pos,n,t.parent),t):o(t)}function E(t){if(224===t.parent.parent.kind)return d(t.parent.parent);var r=t.parent;return e.isBindingPattern(t.name)?x(t.name):t.initializer||e.hasModifier(t,1)||225===r.parent.kind?b(t):e.isVariableDeclarationList(t.parent)&&t.parent.declarations[0]!==t?d(e.findPrecedingToken(t.pos,n,t.parent)):void 0}function T(n){return e.hasModifier(n,1)||238===n.parent.kind&&155!==n.kind}function S(t){switch(t.parent.kind){case 242:if(1!==e.getModuleInstanceState(t.parent))return;case 222:case 220:case 224:return l(t.parent,t.statements[0]);case 223:case 225:return l(e.findPrecedingToken(t.pos,n,t.parent),t.statements[0])}return d(t.statements[0])}function L(e){if(236!==e.initializer.kind)return d(e.initializer);var n=e.initializer;return n.declarations.length>0?d(n.declarations[0]):void 0}function x(n){var t=e.forEach(n.elements,function(e){return 208!==e.kind?e:void 0});return t?d(t):184===n.parent.kind?o(n.parent):b(n.parent)}function A(n){e.Debug.assert(183!==n.kind&&182!==n.kind);var t=185===n.kind?n.elements:n.properties,r=e.forEach(t,function(e){return 208!==e.kind?e:void 0});return r?d(r):o(202===n.parent.kind?n.parent:n)}}}}(e.BreakpointResolver||(e.BreakpointResolver={}))}(Tn||(Tn={})),function(e){e.transform=function(n,t,r){var a=[];r=e.fixupCompilerOptions(r,a);var i=e.isArray(n)?n:[n],o=e.transformNodes(void 0,void 0,r,i,t,!0);return o.diagnostics=e.concatenate(o.diagnostics,a),o}}(Tn||(Tn={}));var Tn,Sn,Ln=function(){return this}();!function(e){function n(e,n){e&&e.log(\"*INTERNAL ERROR* - Exception in typescript services: \"+n.message)}var t=function(){function n(e){this.scriptSnapshotShim=e}return n.prototype.getText=function(e,n){return this.scriptSnapshotShim.getText(e,n)},n.prototype.getLength=function(){return this.scriptSnapshotShim.getLength()},n.prototype.getChangeRange=function(n){var t=n,r=this.scriptSnapshotShim.getChangeRange(t.scriptSnapshotShim);if(null===r)return null;var a=JSON.parse(r);return e.createTextChangeRange(e.createTextSpan(a.span.start,a.span.length),a.newLength)},n.prototype.dispose=function(){\"dispose\"in this.scriptSnapshotShim&&this.scriptSnapshotShim.dispose()},n}(),r=function(){function n(n){var t=this;this.shimHost=n,this.loggingEnabled=!1,this.tracingEnabled=!1,\"getModuleResolutionsForFile\"in this.shimHost&&(this.resolveModuleNames=function(n,r){var a=JSON.parse(t.shimHost.getModuleResolutionsForFile(r));return e.map(n,function(n){var t=e.getProperty(a,n);return t?{resolvedFileName:t,extension:e.extensionFromPath(t),isExternalLibraryImport:!1}:void 0})}),\"directoryExists\"in this.shimHost&&(this.directoryExists=function(e){return t.shimHost.directoryExists(e)}),\"getTypeReferenceDirectiveResolutionsForFile\"in this.shimHost&&(this.resolveTypeReferenceDirectives=function(n,r){var a=JSON.parse(t.shimHost.getTypeReferenceDirectiveResolutionsForFile(r));return e.map(n,function(n){return e.getProperty(a,n)})})}return n.prototype.log=function(e){this.loggingEnabled&&this.shimHost.log(e)},n.prototype.trace=function(e){this.tracingEnabled&&this.shimHost.trace(e)},n.prototype.error=function(e){this.shimHost.error(e)},n.prototype.getProjectVersion=function(){if(this.shimHost.getProjectVersion)return this.shimHost.getProjectVersion()},n.prototype.getTypeRootsVersion=function(){return this.shimHost.getTypeRootsVersion?this.shimHost.getTypeRootsVersion():0},n.prototype.useCaseSensitiveFileNames=function(){return!!this.shimHost.useCaseSensitiveFileNames&&this.shimHost.useCaseSensitiveFileNames()},n.prototype.getCompilationSettings=function(){var e=this.shimHost.getCompilationSettings();if(null===e||\"\"===e)throw Error(\"LanguageServiceShimHostAdapter.getCompilationSettings: empty compilationSettings\");var n=JSON.parse(e);return n.allowNonTsExtensions=!0,n},n.prototype.getScriptFileNames=function(){var e=this.shimHost.getScriptFileNames();return JSON.parse(e)},n.prototype.getScriptSnapshot=function(e){var n=this.shimHost.getScriptSnapshot(e);return n&&new t(n)},n.prototype.getScriptKind=function(e){return\"getScriptKind\"in this.shimHost?this.shimHost.getScriptKind(e):0},n.prototype.getScriptVersion=function(e){return this.shimHost.getScriptVersion(e)},n.prototype.getLocalizedDiagnosticMessages=function(){var e=this.shimHost.getLocalizedDiagnosticMessages();if(null===e||\"\"===e)return null;try{return JSON.parse(e)}catch(e){return this.log(e.description||\"diagnosticMessages.generated.json has invalid JSON format\"),null}},n.prototype.getCancellationToken=function(){var n=this.shimHost.getCancellationToken();return new e.ThrottledCancellationToken(n)},n.prototype.getCurrentDirectory=function(){return this.shimHost.getCurrentDirectory()},n.prototype.getDirectories=function(e){return JSON.parse(this.shimHost.getDirectories(e))},n.prototype.getDefaultLibFileName=function(e){return this.shimHost.getDefaultLibFileName(JSON.stringify(e))},n.prototype.readDirectory=function(n,t,r,a,i){var o=e.getFileMatcherPatterns(n,r,a,this.shimHost.useCaseSensitiveFileNames(),this.shimHost.getCurrentDirectory());return JSON.parse(this.shimHost.readDirectory(n,JSON.stringify(t),JSON.stringify(o.basePaths),o.excludePattern,o.includeFilePattern,o.includeDirectoryPattern,i))},n.prototype.readFile=function(e,n){return this.shimHost.readFile(e,n)},n.prototype.fileExists=function(e){return this.shimHost.fileExists(e)},n}();e.LanguageServiceShimHostAdapter=r;var a=function(){function n(e){var n=this;this.shimHost=e,this.useCaseSensitiveFileNames=!!this.shimHost.useCaseSensitiveFileNames&&this.shimHost.useCaseSensitiveFileNames(),\"directoryExists\"in this.shimHost&&(this.directoryExists=function(e){return n.shimHost.directoryExists(e)}),\"realpath\"in this.shimHost&&(this.realpath=function(e){return n.shimHost.realpath(e)})}return n.prototype.readDirectory=function(n,t,r,a,i){var o=e.getFileMatcherPatterns(n,r,a,this.shimHost.useCaseSensitiveFileNames(),this.shimHost.getCurrentDirectory());return JSON.parse(this.shimHost.readDirectory(n,JSON.stringify(t),JSON.stringify(o.basePaths),o.excludePattern,o.includeFilePattern,o.includeDirectoryPattern,i))},n.prototype.fileExists=function(e){return this.shimHost.fileExists(e)},n.prototype.readFile=function(e){return this.shimHost.readFile(e)},n.prototype.getDirectories=function(e){return JSON.parse(this.shimHost.getDirectories(e))},n}();function i(e,n,t,r){return o(e,n,!0,t,r)}function o(t,r,a,i,o){try{var s=function(n,t,r,a){var i;a&&(n.log(t),i=e.timestamp());var o=r();if(a){var s=e.timestamp();if(n.log(t+\" completed in \"+(s-i)+\" msec\"),e.isString(o)){var l=o;l.length>128&&(l=l.substring(0,128)+\"...\"),n.log(\"  result.length=\"+l.length+\", result='\"+JSON.stringify(l)+\"'\")}}return o}(t,r,i,o);return a?JSON.stringify({result:s}):s}catch(a){return a instanceof e.OperationCanceledException?JSON.stringify({canceled:!0}):(n(t,a),a.description=r,JSON.stringify({error:a}))}}e.CoreServicesShimHostAdapter=a;var s=function(){function e(e){this.factory=e,e.registerShim(this)}return e.prototype.dispose=function(e){this.factory.unregisterShim(this)},e}();function l(n,t){return n.map(function(n){return function(n,t){return{message:e.flattenDiagnosticMessageText(n.messageText,t),start:n.start,length:n.length,category:e.diagnosticCategoryName(n),code:n.code,reportsUnnecessary:n.reportsUnnecessary}}(n,t)})}e.realizeDiagnostics=l;var c=function(n){function t(e,t,r){var a=n.call(this,e)||this;return a.host=t,a.languageService=r,a.logPerformance=!1,a.logger=a.host,a}return En(t,n),t.prototype.forwardJSONCall=function(e,n){return i(this.logger,e,n,this.logPerformance)},t.prototype.dispose=function(e){this.logger.log(\"dispose()\"),this.languageService.dispose(),this.languageService=null,Ln&&Ln.CollectGarbage&&(Ln.CollectGarbage(),this.logger.log(\"CollectGarbage()\")),this.logger=null,n.prototype.dispose.call(this,e)},t.prototype.refresh=function(e){this.forwardJSONCall(\"refresh(\"+e+\")\",function(){return null})},t.prototype.cleanupSemanticCache=function(){var e=this;this.forwardJSONCall(\"cleanupSemanticCache()\",function(){return e.languageService.cleanupSemanticCache(),null})},t.prototype.realizeDiagnostics=function(n){return l(n,e.getNewLineOrDefaultFromHost(this.host))},t.prototype.getSyntacticClassifications=function(n,t,r){var a=this;return this.forwardJSONCall(\"getSyntacticClassifications('\"+n+\"', \"+t+\", \"+r+\")\",function(){return a.languageService.getSyntacticClassifications(n,e.createTextSpan(t,r))})},t.prototype.getSemanticClassifications=function(n,t,r){var a=this;return this.forwardJSONCall(\"getSemanticClassifications('\"+n+\"', \"+t+\", \"+r+\")\",function(){return a.languageService.getSemanticClassifications(n,e.createTextSpan(t,r))})},t.prototype.getEncodedSyntacticClassifications=function(n,t,r){var a=this;return this.forwardJSONCall(\"getEncodedSyntacticClassifications('\"+n+\"', \"+t+\", \"+r+\")\",function(){return u(a.languageService.getEncodedSyntacticClassifications(n,e.createTextSpan(t,r)))})},t.prototype.getEncodedSemanticClassifications=function(n,t,r){var a=this;return this.forwardJSONCall(\"getEncodedSemanticClassifications('\"+n+\"', \"+t+\", \"+r+\")\",function(){return u(a.languageService.getEncodedSemanticClassifications(n,e.createTextSpan(t,r)))})},t.prototype.getSyntacticDiagnostics=function(e){var n=this;return this.forwardJSONCall(\"getSyntacticDiagnostics('\"+e+\"')\",function(){var t=n.languageService.getSyntacticDiagnostics(e);return n.realizeDiagnostics(t)})},t.prototype.getSemanticDiagnostics=function(e){var n=this;return this.forwardJSONCall(\"getSemanticDiagnostics('\"+e+\"')\",function(){var t=n.languageService.getSemanticDiagnostics(e);return n.realizeDiagnostics(t)})},t.prototype.getSuggestionDiagnostics=function(e){var n=this;return this.forwardJSONCall(\"getSuggestionDiagnostics('\"+e+\"')\",function(){return n.realizeDiagnostics(n.languageService.getSuggestionDiagnostics(e))})},t.prototype.getCompilerOptionsDiagnostics=function(){var e=this;return this.forwardJSONCall(\"getCompilerOptionsDiagnostics()\",function(){var n=e.languageService.getCompilerOptionsDiagnostics();return e.realizeDiagnostics(n)})},t.prototype.getQuickInfoAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getQuickInfoAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getQuickInfoAtPosition(e,n)})},t.prototype.getNameOrDottedNameSpan=function(e,n,t){var r=this;return this.forwardJSONCall(\"getNameOrDottedNameSpan('\"+e+\"', \"+n+\", \"+t+\")\",function(){return r.languageService.getNameOrDottedNameSpan(e,n,t)})},t.prototype.getBreakpointStatementAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getBreakpointStatementAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getBreakpointStatementAtPosition(e,n)})},t.prototype.getSignatureHelpItems=function(e,n,t){var r=this;return this.forwardJSONCall(\"getSignatureHelpItems('\"+e+\"', \"+n+\")\",function(){return r.languageService.getSignatureHelpItems(e,n,t)})},t.prototype.getDefinitionAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getDefinitionAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getDefinitionAtPosition(e,n)})},t.prototype.getDefinitionAndBoundSpan=function(e,n){var t=this;return this.forwardJSONCall(\"getDefinitionAndBoundSpan('\"+e+\"', \"+n+\")\",function(){return t.languageService.getDefinitionAndBoundSpan(e,n)})},t.prototype.getTypeDefinitionAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getTypeDefinitionAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getTypeDefinitionAtPosition(e,n)})},t.prototype.getImplementationAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getImplementationAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getImplementationAtPosition(e,n)})},t.prototype.getRenameInfo=function(e,n){var t=this;return this.forwardJSONCall(\"getRenameInfo('\"+e+\"', \"+n+\")\",function(){return t.languageService.getRenameInfo(e,n)})},t.prototype.findRenameLocations=function(e,n,t,r){var a=this;return this.forwardJSONCall(\"findRenameLocations('\"+e+\"', \"+n+\", \"+t+\", \"+r+\")\",function(){return a.languageService.findRenameLocations(e,n,t,r)})},t.prototype.getBraceMatchingAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getBraceMatchingAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getBraceMatchingAtPosition(e,n)})},t.prototype.isValidBraceCompletionAtPosition=function(e,n,t){var r=this;return this.forwardJSONCall(\"isValidBraceCompletionAtPosition('\"+e+\"', \"+n+\", \"+t+\")\",function(){return r.languageService.isValidBraceCompletionAtPosition(e,n,t)})},t.prototype.getSpanOfEnclosingComment=function(e,n,t){var r=this;return this.forwardJSONCall(\"getSpanOfEnclosingComment('\"+e+\"', \"+n+\")\",function(){return r.languageService.getSpanOfEnclosingComment(e,n,t)})},t.prototype.getIndentationAtPosition=function(e,n,t){var r=this;return this.forwardJSONCall(\"getIndentationAtPosition('\"+e+\"', \"+n+\")\",function(){var a=JSON.parse(t);return r.languageService.getIndentationAtPosition(e,n,a)})},t.prototype.getReferencesAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getReferencesAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getReferencesAtPosition(e,n)})},t.prototype.findReferences=function(e,n){var t=this;return this.forwardJSONCall(\"findReferences('\"+e+\"', \"+n+\")\",function(){return t.languageService.findReferences(e,n)})},t.prototype.getOccurrencesAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getOccurrencesAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getOccurrencesAtPosition(e,n)})},t.prototype.getDocumentHighlights=function(n,t,r){var a=this;return this.forwardJSONCall(\"getDocumentHighlights('\"+n+\"', \"+t+\")\",function(){var i=a.languageService.getDocumentHighlights(n,t,JSON.parse(r)),o=e.normalizeSlashes(n).toLowerCase();return e.filter(i,function(n){return e.normalizeSlashes(n.fileName).toLowerCase()===o})})},t.prototype.getCompletionsAtPosition=function(e,n,t){var r=this;return this.forwardJSONCall(\"getCompletionsAtPosition('\"+e+\"', \"+n+\", \"+t+\")\",function(){return r.languageService.getCompletionsAtPosition(e,n,t)})},t.prototype.getCompletionEntryDetails=function(e,n,t,r,a,i){var o=this;return this.forwardJSONCall(\"getCompletionEntryDetails('\"+e+\"', \"+n+\", '\"+t+\"')\",function(){var s=void 0===r?void 0:JSON.parse(r);return o.languageService.getCompletionEntryDetails(e,n,t,s,a,i)})},t.prototype.getFormattingEditsForRange=function(e,n,t,r){var a=this;return this.forwardJSONCall(\"getFormattingEditsForRange('\"+e+\"', \"+n+\", \"+t+\")\",function(){var i=JSON.parse(r);return a.languageService.getFormattingEditsForRange(e,n,t,i)})},t.prototype.getFormattingEditsForDocument=function(e,n){var t=this;return this.forwardJSONCall(\"getFormattingEditsForDocument('\"+e+\"')\",function(){var r=JSON.parse(n);return t.languageService.getFormattingEditsForDocument(e,r)})},t.prototype.getFormattingEditsAfterKeystroke=function(e,n,t,r){var a=this;return this.forwardJSONCall(\"getFormattingEditsAfterKeystroke('\"+e+\"', \"+n+\", '\"+t+\"')\",function(){var i=JSON.parse(r);return a.languageService.getFormattingEditsAfterKeystroke(e,n,t,i)})},t.prototype.getDocCommentTemplateAtPosition=function(e,n){var t=this;return this.forwardJSONCall(\"getDocCommentTemplateAtPosition('\"+e+\"', \"+n+\")\",function(){return t.languageService.getDocCommentTemplateAtPosition(e,n)})},t.prototype.getNavigateToItems=function(e,n,t){var r=this;return this.forwardJSONCall(\"getNavigateToItems('\"+e+\"', \"+n+\", \"+t+\")\",function(){return r.languageService.getNavigateToItems(e,n,t)})},t.prototype.getNavigationBarItems=function(e){var n=this;return this.forwardJSONCall(\"getNavigationBarItems('\"+e+\"')\",function(){return n.languageService.getNavigationBarItems(e)})},t.prototype.getNavigationTree=function(e){var n=this;return this.forwardJSONCall(\"getNavigationTree('\"+e+\"')\",function(){return n.languageService.getNavigationTree(e)})},t.prototype.getOutliningSpans=function(e){var n=this;return this.forwardJSONCall(\"getOutliningSpans('\"+e+\"')\",function(){return n.languageService.getOutliningSpans(e)})},t.prototype.getTodoComments=function(e,n){var t=this;return this.forwardJSONCall(\"getTodoComments('\"+e+\"')\",function(){return t.languageService.getTodoComments(e,JSON.parse(n))})},t.prototype.getEmitOutput=function(e){var n=this;return this.forwardJSONCall(\"getEmitOutput('\"+e+\"')\",function(){return n.languageService.getEmitOutput(e)})},t.prototype.getEmitOutputObject=function(e){var n=this;return o(this.logger,\"getEmitOutput('\"+e+\"')\",!1,function(){return n.languageService.getEmitOutput(e)},this.logPerformance)},t}(s);function u(e){return{spans:e.spans.join(\",\"),endOfLineState:e.endOfLineState}}var d=function(n){function t(t,r){var a=n.call(this,t)||this;return a.logger=r,a.logPerformance=!1,a.classifier=e.createClassifier(),a}return En(t,n),t.prototype.getEncodedLexicalClassifications=function(e,n,t){var r=this;return void 0===t&&(t=!1),i(this.logger,\"getEncodedLexicalClassifications\",function(){return u(r.classifier.getEncodedLexicalClassifications(e,n,t))},this.logPerformance)},t.prototype.getClassificationsForLine=function(e,n,t){void 0===t&&(t=!1);for(var r=this.classifier.getClassificationsForLine(e,n,t),a=\"\",i=0,o=r.entries;i<o.length;i++){var s=o[i];a+=s.length+\"\\n\",a+=s.classification+\"\\n\"}return a+=r.finalLexState},t}(s),m=function(n){function t(e,t,r){var a=n.call(this,e)||this;return a.logger=t,a.host=r,a.logPerformance=!1,a}return En(t,n),t.prototype.forwardJSONCall=function(e,n){return i(this.logger,e,n,this.logPerformance)},t.prototype.resolveModuleName=function(n,t,r){var a=this;return this.forwardJSONCall(\"resolveModuleName('\"+n+\"')\",function(){var i=JSON.parse(r),o=e.resolveModuleName(t,e.normalizeSlashes(n),i,a.host),s=o.resolvedModule?o.resolvedModule.resolvedFileName:void 0;return o.resolvedModule&&\".ts\"!==o.resolvedModule.extension&&\".tsx\"!==o.resolvedModule.extension&&\".d.ts\"!==o.resolvedModule.extension&&(s=void 0),{resolvedFileName:s,failedLookupLocations:o.failedLookupLocations}})},t.prototype.resolveTypeReferenceDirective=function(n,t,r){var a=this;return this.forwardJSONCall(\"resolveTypeReferenceDirective(\"+n+\")\",function(){var i=JSON.parse(r),o=e.resolveTypeReferenceDirective(t,e.normalizeSlashes(n),i,a.host);return{resolvedFileName:o.resolvedTypeReferenceDirective?o.resolvedTypeReferenceDirective.resolvedFileName:void 0,primary:!o.resolvedTypeReferenceDirective||o.resolvedTypeReferenceDirective.primary,failedLookupLocations:o.failedLookupLocations}})},t.prototype.getPreProcessedFileInfo=function(n,t){var r=this;return this.forwardJSONCall(\"getPreProcessedFileInfo('\"+n+\"')\",function(){var n=e.preProcessFile(e.getSnapshotText(t),!0,!0);return{referencedFiles:r.convertFileReferences(n.referencedFiles),importedFiles:r.convertFileReferences(n.importedFiles),ambientExternalModules:n.ambientExternalModules,isLibFile:n.isLibFile,typeReferenceDirectives:r.convertFileReferences(n.typeReferenceDirectives),libReferenceDirectives:r.convertFileReferences(n.libReferenceDirectives)}})},t.prototype.getAutomaticTypeDirectiveNames=function(n){var t=this;return this.forwardJSONCall(\"getAutomaticTypeDirectiveNames('\"+n+\"')\",function(){var r=JSON.parse(n);return e.getAutomaticTypeDirectiveNames(r,t.host)})},t.prototype.convertFileReferences=function(n){if(n){for(var t=[],r=0,a=n;r<a.length;r++){var i=a[r];t.push({path:e.normalizeSlashes(i.fileName),position:i.pos,length:i.end-i.pos})}return t}},t.prototype.getTSConfigFileInfo=function(n,t){var r=this;return this.forwardJSONCall(\"getTSConfigFileInfo('\"+n+\"')\",function(){var a=e.parseJsonText(n,e.getSnapshotText(t)),i=e.normalizeSlashes(n),o=e.parseJsonSourceFileConfigFileContent(a,r.host,e.getDirectoryPath(i),{},i);return{options:o.options,typeAcquisition:o.typeAcquisition,files:o.fileNames,raw:o.raw,errors:l(a.parseDiagnostics.concat(o.errors),\"\\r\\n\")}})},t.prototype.getDefaultCompilationSettings=function(){return this.forwardJSONCall(\"getDefaultCompilationSettings()\",function(){return e.getDefaultCompilerOptions()})},t.prototype.discoverTypings=function(n){var t=this,r=e.createGetCanonicalFileName(!1);return this.forwardJSONCall(\"discoverTypings()\",function(){var a=JSON.parse(n);return void 0===t.safeList&&(t.safeList=e.JsTyping.loadSafeList(t.host,e.toPath(a.safeListPath,a.safeListPath,r))),e.JsTyping.discoverTypings(t.host,function(e){return t.logger.log(e)},a.fileNames,e.toPath(a.projectRootPath,a.projectRootPath,r),t.safeList,a.packageNameToTypingLocation,a.typeAcquisition,a.unresolvedImports,a.typesRegistry)})},t}(s),p=function(){function t(){this._shims=[]}return t.prototype.getServicesVersion=function(){return e.servicesVersion},t.prototype.createLanguageServiceShim=function(t){try{void 0===this.documentRegistry&&(this.documentRegistry=e.createDocumentRegistry(t.useCaseSensitiveFileNames&&t.useCaseSensitiveFileNames(),t.getCurrentDirectory()));var a=new r(t),i=e.createLanguageService(a,this.documentRegistry,!1);return new c(this,t,i)}catch(e){throw n(t,e),e}},t.prototype.createClassifierShim=function(e){try{return new d(this,e)}catch(t){throw n(e,t),t}},t.prototype.createCoreServicesShim=function(e){try{var t=new a(e);return new m(this,e,t)}catch(t){throw n(e,t),t}},t.prototype.close=function(){e.clear(this._shims),this.documentRegistry=void 0},t.prototype.registerShim=function(e){this._shims.push(e)},t.prototype.unregisterShim=function(e){for(var n=0;n<this._shims.length;n++)if(this._shims[n]===e)return void delete this._shims[n];throw new Error(\"Invalid operation\")},t}();e.TypeScriptServicesFactory=p,\"undefined\"!=typeof module&&module.exports&&(module.exports=e)}(Tn||(Tn={})),function(e){!function(e){e.TypeScriptServicesFactory=Tn.TypeScriptServicesFactory}(e.Services||(e.Services={}))}(Sn||(Sn={}));Tn.versionMajorMinor;Tn.createClassifier;const xn=Tn.createLanguageService,An=(Tn.displayPartsToString,Tn.EndOfLineState,Tn.flattenDiagnosticMessageText,Tn.IndentStyle,Tn.ScriptKind),Cn=Tn.ScriptTarget;Tn.TokenClass;var Dn=monaco.Promise,kn={NAME:\"defaultLib:lib.d.ts\",CONTENTS:'/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n/////////////////////////////\\n/// ECMAScript APIs\\n/////////////////////////////\\n\\ndeclare const NaN: number;\\ndeclare const Infinity: number;\\n\\n/**\\n  * Evaluates JavaScript code and executes it.\\n  * @param x A String value that contains valid JavaScript code.\\n  */\\ndeclare function eval(x: string): any;\\n\\n/**\\n  * Converts A string to an integer.\\n  * @param s A string to convert into a number.\\n  * @param radix A value between 2 and 36 that specifies the base of the number in numString.\\n  * If this argument is not supplied, strings with a prefix of \\'0x\\' are considered hexadecimal.\\n  * All other strings are considered decimal.\\n  */\\ndeclare function parseInt(s: string, radix?: number): number;\\n\\n/**\\n  * Converts a string to a floating-point number.\\n  * @param string A string that contains a floating-point number.\\n  */\\ndeclare function parseFloat(string: string): number;\\n\\n/**\\n  * Returns a Boolean value that indicates whether a value is the reserved value NaN (not a number).\\n  * @param number A numeric value.\\n  */\\ndeclare function isNaN(number: number): boolean;\\n\\n/**\\n  * Determines whether a supplied number is finite.\\n  * @param number Any numeric value.\\n  */\\ndeclare function isFinite(number: number): boolean;\\n\\n/**\\n  * Gets the unencoded version of an encoded Uniform Resource Identifier (URI).\\n  * @param encodedURI A value representing an encoded URI.\\n  */\\ndeclare function decodeURI(encodedURI: string): string;\\n\\n/**\\n  * Gets the unencoded version of an encoded component of a Uniform Resource Identifier (URI).\\n  * @param encodedURIComponent A value representing an encoded URI component.\\n  */\\ndeclare function decodeURIComponent(encodedURIComponent: string): string;\\n\\n/**\\n  * Encodes a text string as a valid Uniform Resource Identifier (URI)\\n  * @param uri A value representing an encoded URI.\\n  */\\ndeclare function encodeURI(uri: string): string;\\n\\n/**\\n  * Encodes a text string as a valid component of a Uniform Resource Identifier (URI).\\n  * @param uriComponent A value representing an encoded URI component.\\n  */\\ndeclare function encodeURIComponent(uriComponent: string): string;\\n\\n/**\\n  * Computes a new string in which certain characters have been replaced by a hexadecimal escape sequence.\\n  * @param string A string value\\n  */\\ndeclare function escape(string: string): string;\\n\\n/**\\n  * Computes a new string in which hexadecimal escape sequences are replaced with the character that it represents.\\n  * @param string A string value\\n  */\\ndeclare function unescape(string: string): string;\\n\\ninterface Symbol {\\n  /** Returns a string representation of an object. */\\n  toString(): string;\\n\\n  /** Returns the primitive value of the specified object. */\\n  valueOf(): symbol;\\n}\\n\\ndeclare type PropertyKey = string | number | symbol;\\n\\ninterface PropertyDescriptor {\\n    configurable?: boolean;\\n    enumerable?: boolean;\\n    value?: any;\\n    writable?: boolean;\\n    get?(): any;\\n    set?(v: any): void;\\n}\\n\\ninterface PropertyDescriptorMap {\\n    [s: string]: PropertyDescriptor;\\n}\\n\\ninterface Object {\\n    /** The initial value of Object.prototype.constructor is the standard built-in Object constructor. */\\n    constructor: Function;\\n\\n    /** Returns a string representation of an object. */\\n    toString(): string;\\n\\n    /** Returns a date converted to a string using the current locale. */\\n    toLocaleString(): string;\\n\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): Object;\\n\\n    /**\\n      * Determines whether an object has a property with the specified name.\\n      * @param v A property name.\\n      */\\n    hasOwnProperty(v: PropertyKey): boolean;\\n\\n    /**\\n      * Determines whether an object exists in another object\\'s prototype chain.\\n      * @param v Another object whose prototype chain is to be checked.\\n      */\\n    isPrototypeOf(v: Object): boolean;\\n\\n    /**\\n      * Determines whether a specified property is enumerable.\\n      * @param v A property name.\\n      */\\n    propertyIsEnumerable(v: PropertyKey): boolean;\\n}\\n\\ninterface ObjectConstructor {\\n    new(value?: any): Object;\\n    (): any;\\n    (value: any): any;\\n\\n    /** A reference to the prototype for a class of objects. */\\n    readonly prototype: Object;\\n\\n    /**\\n      * Returns the prototype of an object.\\n      * @param o The object that references the prototype.\\n      */\\n    getPrototypeOf(o: any): any;\\n\\n    /**\\n      * Gets the own property descriptor of the specified object.\\n      * An own property descriptor is one that is defined directly on the object and is not inherited from the object\\'s prototype.\\n      * @param o Object that contains the property.\\n      * @param p Name of the property.\\n    */\\n    getOwnPropertyDescriptor(o: any, p: PropertyKey): PropertyDescriptor | undefined;\\n\\n    /**\\n      * Returns the names of the own properties of an object. The own properties of an object are those that are defined directly\\n      * on that object, and are not inherited from the object\\'s prototype. The properties of an object include both fields (objects) and functions.\\n      * @param o Object that contains the own properties.\\n      */\\n    getOwnPropertyNames(o: any): string[];\\n\\n    /**\\n      * Creates an object that has the specified prototype or that has null prototype.\\n      * @param o Object to use as a prototype. May be null.\\n      */\\n    create(o: object | null): any;\\n\\n    /**\\n      * Creates an object that has the specified prototype, and that optionally contains specified properties.\\n      * @param o Object to use as a prototype. May be null\\n      * @param properties JavaScript object that contains one or more property descriptors.\\n      */\\n    create(o: object | null, properties: PropertyDescriptorMap & ThisType<any>): any;\\n\\n    /**\\n      * Adds a property to an object, or modifies attributes of an existing property.\\n      * @param o Object on which to add or modify the property. This can be a native JavaScript object (that is, a user-defined object or a built in object) or a DOM object.\\n      * @param p The property name.\\n      * @param attributes Descriptor for the property. It can be for a data property or an accessor property.\\n      */\\n    defineProperty(o: any, p: PropertyKey, attributes: PropertyDescriptor & ThisType<any>): any;\\n\\n    /**\\n      * Adds one or more properties to an object, and/or modifies attributes of existing properties.\\n      * @param o Object on which to add or modify the properties. This can be a native JavaScript object or a DOM object.\\n      * @param properties JavaScript object that contains one or more descriptor objects. Each descriptor object describes a data property or an accessor property.\\n      */\\n    defineProperties(o: any, properties: PropertyDescriptorMap & ThisType<any>): any;\\n\\n    /**\\n      * Prevents the modification of attributes of existing properties, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    seal<T>(o: T): T;\\n\\n    /**\\n      * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    freeze<T>(a: T[]): ReadonlyArray<T>;\\n\\n    /**\\n      * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    freeze<T extends Function>(f: T): T;\\n\\n    /**\\n      * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    freeze<T>(o: T): Readonly<T>;\\n\\n    /**\\n      * Prevents the addition of new properties to an object.\\n      * @param o Object to make non-extensible.\\n      */\\n    preventExtensions<T>(o: T): T;\\n\\n    /**\\n      * Returns true if existing property attributes cannot be modified in an object and new properties cannot be added to the object.\\n      * @param o Object to test.\\n      */\\n    isSealed(o: any): boolean;\\n\\n    /**\\n      * Returns true if existing property attributes and values cannot be modified in an object, and new properties cannot be added to the object.\\n      * @param o Object to test.\\n      */\\n    isFrozen(o: any): boolean;\\n\\n    /**\\n      * Returns a value that indicates whether new properties can be added to an object.\\n      * @param o Object to test.\\n      */\\n    isExtensible(o: any): boolean;\\n\\n    /**\\n      * Returns the names of the enumerable properties and methods of an object.\\n      * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\\n      */\\n    keys(o: {}): string[];\\n}\\n\\n/**\\n  * Provides functionality common to all JavaScript objects.\\n  */\\ndeclare const Object: ObjectConstructor;\\n\\n/**\\n  * Creates a new function.\\n  */\\ninterface Function {\\n    /**\\n      * Calls the function, substituting the specified object for the this value of the function, and the specified array for the arguments of the function.\\n      * @param thisArg The object to be used as the this object.\\n      * @param argArray A set of arguments to be passed to the function.\\n      */\\n    apply(this: Function, thisArg: any, argArray?: any): any;\\n\\n    /**\\n      * Calls a method of an object, substituting another object for the current object.\\n      * @param thisArg The object to be used as the current object.\\n      * @param argArray A list of arguments to be passed to the method.\\n      */\\n    call(this: Function, thisArg: any, ...argArray: any[]): any;\\n\\n    /**\\n      * For a given function, creates a bound function that has the same body as the original function.\\n      * The this object of the bound function is associated with the specified object, and has the specified initial parameters.\\n      * @param thisArg An object to which the this keyword can refer inside the new function.\\n      * @param argArray A list of arguments to be passed to the new function.\\n      */\\n    bind(this: Function, thisArg: any, ...argArray: any[]): any;\\n\\n    /** Returns a string representation of a function. */\\n    toString(): string;\\n\\n    prototype: any;\\n    readonly length: number;\\n\\n    // Non-standard extensions\\n    arguments: any;\\n    caller: Function;\\n}\\n\\ninterface FunctionConstructor {\\n    /**\\n      * Creates a new function.\\n      * @param args A list of arguments the function accepts.\\n      */\\n    new(...args: string[]): Function;\\n    (...args: string[]): Function;\\n    readonly prototype: Function;\\n}\\n\\ndeclare const Function: FunctionConstructor;\\n\\ninterface IArguments {\\n    [index: number]: any;\\n    length: number;\\n    callee: Function;\\n}\\n\\ninterface String {\\n    /** Returns a string representation of a string. */\\n    toString(): string;\\n\\n    /**\\n      * Returns the character at the specified index.\\n      * @param pos The zero-based index of the desired character.\\n      */\\n    charAt(pos: number): string;\\n\\n    /**\\n      * Returns the Unicode value of the character at the specified location.\\n      * @param index The zero-based index of the desired character. If there is no character at the specified index, NaN is returned.\\n      */\\n    charCodeAt(index: number): number;\\n\\n    /**\\n      * Returns a string that contains the concatenation of two or more strings.\\n      * @param strings The strings to append to the end of the string.\\n      */\\n    concat(...strings: string[]): string;\\n\\n    /**\\n      * Returns the position of the first occurrence of a substring.\\n      * @param searchString The substring to search for in the string\\n      * @param position The index at which to begin searching the String object. If omitted, search starts at the beginning of the string.\\n      */\\n    indexOf(searchString: string, position?: number): number;\\n\\n    /**\\n      * Returns the last occurrence of a substring in the string.\\n      * @param searchString The substring to search for.\\n      * @param position The index at which to begin searching. If omitted, the search begins at the end of the string.\\n      */\\n    lastIndexOf(searchString: string, position?: number): number;\\n\\n    /**\\n      * Determines whether two strings are equivalent in the current locale.\\n      * @param that String to compare to target string\\n      */\\n    localeCompare(that: string): number;\\n\\n    /**\\n      * Matches a string with a regular expression, and returns an array containing the results of that search.\\n      * @param regexp A variable name or string literal containing the regular expression pattern and flags.\\n      */\\n    match(regexp: string | RegExp): RegExpMatchArray | null;\\n\\n    /**\\n      * Replaces text in a string, using a regular expression or search string.\\n      * @param searchValue A string to search for.\\n      * @param replaceValue A string containing the text to replace for every successful match of searchValue in this string.\\n      */\\n    replace(searchValue: string | RegExp, replaceValue: string): string;\\n\\n    /**\\n      * Replaces text in a string, using a regular expression or search string.\\n      * @param searchValue A string to search for.\\n      * @param replacer A function that returns the replacement text.\\n      */\\n    replace(searchValue: string | RegExp, replacer: (substring: string, ...args: any[]) => string): string;\\n\\n    /**\\n      * Finds the first substring match in a regular expression search.\\n      * @param regexp The regular expression pattern and applicable flags.\\n      */\\n    search(regexp: string | RegExp): number;\\n\\n    /**\\n      * Returns a section of a string.\\n      * @param start The index to the beginning of the specified portion of stringObj.\\n      * @param end The index to the end of the specified portion of stringObj. The substring includes the characters up to, but not including, the character indicated by end.\\n      * If this value is not specified, the substring continues to the end of stringObj.\\n      */\\n    slice(start?: number, end?: number): string;\\n\\n    /**\\n      * Split a string into substrings using the specified separator and return them as an array.\\n      * @param separator A string that identifies character or characters to use in separating the string. If omitted, a single-element array containing the entire string is returned.\\n      * @param limit A value used to limit the number of elements returned in the array.\\n      */\\n    split(separator: string | RegExp, limit?: number): string[];\\n\\n    /**\\n      * Returns the substring at the specified location within a String object.\\n      * @param start The zero-based index number indicating the beginning of the substring.\\n      * @param end Zero-based index number indicating the end of the substring. The substring includes the characters up to, but not including, the character indicated by end.\\n      * If end is omitted, the characters from start through the end of the original string are returned.\\n      */\\n    substring(start: number, end?: number): string;\\n\\n    /** Converts all the alphabetic characters in a string to lowercase. */\\n    toLowerCase(): string;\\n\\n    /** Converts all alphabetic characters to lowercase, taking into account the host environment\\'s current locale. */\\n    toLocaleLowerCase(): string;\\n\\n    /** Converts all the alphabetic characters in a string to uppercase. */\\n    toUpperCase(): string;\\n\\n    /** Returns a string where all alphabetic characters have been converted to uppercase, taking into account the host environment\\'s current locale. */\\n    toLocaleUpperCase(): string;\\n\\n    /** Removes the leading and trailing white space and line terminator characters from a string. */\\n    trim(): string;\\n\\n    /** Returns the length of a String object. */\\n    readonly length: number;\\n\\n    // IE extensions\\n    /**\\n      * Gets a substring beginning at the specified location and having the specified length.\\n      * @param from The starting position of the desired substring. The index of the first character in the string is zero.\\n      * @param length The number of characters to include in the returned substring.\\n      */\\n    substr(from: number, length?: number): string;\\n\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): string;\\n\\n    readonly [index: number]: string;\\n}\\n\\ninterface StringConstructor {\\n    new(value?: any): String;\\n    (value?: any): string;\\n    readonly prototype: String;\\n    fromCharCode(...codes: number[]): string;\\n}\\n\\n/**\\n  * Allows manipulation and formatting of text strings and determination and location of substrings within strings.\\n  */\\ndeclare const String: StringConstructor;\\n\\ninterface Boolean {\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): boolean;\\n}\\n\\ninterface BooleanConstructor {\\n    new(value?: any): Boolean;\\n    (value?: any): boolean;\\n    readonly prototype: Boolean;\\n}\\n\\ndeclare const Boolean: BooleanConstructor;\\n\\ninterface Number {\\n    /**\\n      * Returns a string representation of an object.\\n      * @param radix Specifies a radix for converting numeric values to strings. This value is only used for numbers.\\n      */\\n    toString(radix?: number): string;\\n\\n    /**\\n      * Returns a string representing a number in fixed-point notation.\\n      * @param fractionDigits Number of digits after the decimal point. Must be in the range 0 - 20, inclusive.\\n      */\\n    toFixed(fractionDigits?: number): string;\\n\\n    /**\\n      * Returns a string containing a number represented in exponential notation.\\n      * @param fractionDigits Number of digits after the decimal point. Must be in the range 0 - 20, inclusive.\\n      */\\n    toExponential(fractionDigits?: number): string;\\n\\n    /**\\n      * Returns a string containing a number represented either in exponential or fixed-point notation with a specified number of digits.\\n      * @param precision Number of significant digits. Must be in the range 1 - 21, inclusive.\\n      */\\n    toPrecision(precision?: number): string;\\n\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): number;\\n}\\n\\ninterface NumberConstructor {\\n    new(value?: any): Number;\\n    (value?: any): number;\\n    readonly prototype: Number;\\n\\n    /** The largest number that can be represented in JavaScript. Equal to approximately 1.79E+308. */\\n    readonly MAX_VALUE: number;\\n\\n    /** The closest number to zero that can be represented in JavaScript. Equal to approximately 5.00E-324. */\\n    readonly MIN_VALUE: number;\\n\\n    /**\\n      * A value that is not a number.\\n      * In equality comparisons, NaN does not equal any value, including itself. To test whether a value is equivalent to NaN, use the isNaN function.\\n      */\\n    readonly NaN: number;\\n\\n    /**\\n      * A value that is less than the largest negative number that can be represented in JavaScript.\\n      * JavaScript displays NEGATIVE_INFINITY values as -infinity.\\n      */\\n    readonly NEGATIVE_INFINITY: number;\\n\\n    /**\\n      * A value greater than the largest number that can be represented in JavaScript.\\n      * JavaScript displays POSITIVE_INFINITY values as infinity.\\n      */\\n    readonly POSITIVE_INFINITY: number;\\n}\\n\\n/** An object that represents a number of any kind. All JavaScript numbers are 64-bit floating-point numbers. */\\ndeclare const Number: NumberConstructor;\\n\\ninterface TemplateStringsArray extends ReadonlyArray<string> {\\n    readonly raw: ReadonlyArray<string>;\\n}\\n\\n/**\\n * The type of `import.meta`.\\n * \\n * If you need to declare that a given property exists on `import.meta`,\\n * this type may be augmented via interface merging.\\n */\\ninterface ImportMeta {\\n}\\n\\ninterface Math {\\n    /** The mathematical constant e. This is Euler\\'s number, the base of natural logarithms. */\\n    readonly E: number;\\n    /** The natural logarithm of 10. */\\n    readonly LN10: number;\\n    /** The natural logarithm of 2. */\\n    readonly LN2: number;\\n    /** The base-2 logarithm of e. */\\n    readonly LOG2E: number;\\n    /** The base-10 logarithm of e. */\\n    readonly LOG10E: number;\\n    /** Pi. This is the ratio of the circumference of a circle to its diameter. */\\n    readonly PI: number;\\n    /** The square root of 0.5, or, equivalently, one divided by the square root of 2. */\\n    readonly SQRT1_2: number;\\n    /** The square root of 2. */\\n    readonly SQRT2: number;\\n    /**\\n      * Returns the absolute value of a number (the value without regard to whether it is positive or negative).\\n      * For example, the absolute value of -5 is the same as the absolute value of 5.\\n      * @param x A numeric expression for which the absolute value is needed.\\n      */\\n    abs(x: number): number;\\n    /**\\n      * Returns the arc cosine (or inverse cosine) of a number.\\n      * @param x A numeric expression.\\n      */\\n    acos(x: number): number;\\n    /**\\n      * Returns the arcsine of a number.\\n      * @param x A numeric expression.\\n      */\\n    asin(x: number): number;\\n    /**\\n      * Returns the arctangent of a number.\\n      * @param x A numeric expression for which the arctangent is needed.\\n      */\\n    atan(x: number): number;\\n    /**\\n      * Returns the angle (in radians) from the X axis to a point.\\n      * @param y A numeric expression representing the cartesian y-coordinate.\\n      * @param x A numeric expression representing the cartesian x-coordinate.\\n      */\\n    atan2(y: number, x: number): number;\\n    /**\\n      * Returns the smallest integer greater than or equal to its numeric argument.\\n      * @param x A numeric expression.\\n      */\\n    ceil(x: number): number;\\n    /**\\n      * Returns the cosine of a number.\\n      * @param x A numeric expression that contains an angle measured in radians.\\n      */\\n    cos(x: number): number;\\n    /**\\n      * Returns e (the base of natural logarithms) raised to a power.\\n      * @param x A numeric expression representing the power of e.\\n      */\\n    exp(x: number): number;\\n    /**\\n      * Returns the greatest integer less than or equal to its numeric argument.\\n      * @param x A numeric expression.\\n      */\\n    floor(x: number): number;\\n    /**\\n      * Returns the natural logarithm (base e) of a number.\\n      * @param x A numeric expression.\\n      */\\n    log(x: number): number;\\n    /**\\n      * Returns the larger of a set of supplied numeric expressions.\\n      * @param values Numeric expressions to be evaluated.\\n      */\\n    max(...values: number[]): number;\\n    /**\\n      * Returns the smaller of a set of supplied numeric expressions.\\n      * @param values Numeric expressions to be evaluated.\\n      */\\n    min(...values: number[]): number;\\n    /**\\n      * Returns the value of a base expression taken to a specified power.\\n      * @param x The base value of the expression.\\n      * @param y The exponent value of the expression.\\n      */\\n    pow(x: number, y: number): number;\\n    /** Returns a pseudorandom number between 0 and 1. */\\n    random(): number;\\n    /**\\n      * Returns a supplied numeric expression rounded to the nearest number.\\n      * @param x The value to be rounded to the nearest number.\\n      */\\n    round(x: number): number;\\n    /**\\n      * Returns the sine of a number.\\n      * @param x A numeric expression that contains an angle measured in radians.\\n      */\\n    sin(x: number): number;\\n    /**\\n      * Returns the square root of a number.\\n      * @param x A numeric expression.\\n      */\\n    sqrt(x: number): number;\\n    /**\\n      * Returns the tangent of a number.\\n      * @param x A numeric expression that contains an angle measured in radians.\\n      */\\n    tan(x: number): number;\\n}\\n/** An intrinsic object that provides basic mathematics functionality and constants. */\\ndeclare const Math: Math;\\n\\n/** Enables basic storage and retrieval of dates and times. */\\ninterface Date {\\n    /** Returns a string representation of a date. The format of the string depends on the locale. */\\n    toString(): string;\\n    /** Returns a date as a string value. */\\n    toDateString(): string;\\n    /** Returns a time as a string value. */\\n    toTimeString(): string;\\n    /** Returns a value as a string value appropriate to the host environment\\'s current locale. */\\n    toLocaleString(): string;\\n    /** Returns a date as a string value appropriate to the host environment\\'s current locale. */\\n    toLocaleDateString(): string;\\n    /** Returns a time as a string value appropriate to the host environment\\'s current locale. */\\n    toLocaleTimeString(): string;\\n    /** Returns the stored time value in milliseconds since midnight, January 1, 1970 UTC. */\\n    valueOf(): number;\\n    /** Gets the time value in milliseconds. */\\n    getTime(): number;\\n    /** Gets the year, using local time. */\\n    getFullYear(): number;\\n    /** Gets the year using Universal Coordinated Time (UTC). */\\n    getUTCFullYear(): number;\\n    /** Gets the month, using local time. */\\n    getMonth(): number;\\n    /** Gets the month of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCMonth(): number;\\n    /** Gets the day-of-the-month, using local time. */\\n    getDate(): number;\\n    /** Gets the day-of-the-month, using Universal Coordinated Time (UTC). */\\n    getUTCDate(): number;\\n    /** Gets the day of the week, using local time. */\\n    getDay(): number;\\n    /** Gets the day of the week using Universal Coordinated Time (UTC). */\\n    getUTCDay(): number;\\n    /** Gets the hours in a date, using local time. */\\n    getHours(): number;\\n    /** Gets the hours value in a Date object using Universal Coordinated Time (UTC). */\\n    getUTCHours(): number;\\n    /** Gets the minutes of a Date object, using local time. */\\n    getMinutes(): number;\\n    /** Gets the minutes of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCMinutes(): number;\\n    /** Gets the seconds of a Date object, using local time. */\\n    getSeconds(): number;\\n    /** Gets the seconds of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCSeconds(): number;\\n    /** Gets the milliseconds of a Date, using local time. */\\n    getMilliseconds(): number;\\n    /** Gets the milliseconds of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCMilliseconds(): number;\\n    /** Gets the difference in minutes between the time on the local computer and Universal Coordinated Time (UTC). */\\n    getTimezoneOffset(): number;\\n    /**\\n      * Sets the date and time value in the Date object.\\n      * @param time A numeric value representing the number of elapsed milliseconds since midnight, January 1, 1970 GMT.\\n      */\\n    setTime(time: number): number;\\n    /**\\n      * Sets the milliseconds value in the Date object using local time.\\n      * @param ms A numeric value equal to the millisecond value.\\n      */\\n    setMilliseconds(ms: number): number;\\n    /**\\n      * Sets the milliseconds value in the Date object using Universal Coordinated Time (UTC).\\n      * @param ms A numeric value equal to the millisecond value.\\n      */\\n    setUTCMilliseconds(ms: number): number;\\n\\n    /**\\n      * Sets the seconds value in the Date object using local time.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setSeconds(sec: number, ms?: number): number;\\n    /**\\n      * Sets the seconds value in the Date object using Universal Coordinated Time (UTC).\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setUTCSeconds(sec: number, ms?: number): number;\\n    /**\\n      * Sets the minutes value in the Date object using local time.\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setMinutes(min: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the minutes value in the Date object using Universal Coordinated Time (UTC).\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setUTCMinutes(min: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the hour value in the Date object using local time.\\n      * @param hours A numeric value equal to the hours value.\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setHours(hours: number, min?: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the hours value in the Date object using Universal Coordinated Time (UTC).\\n      * @param hours A numeric value equal to the hours value.\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setUTCHours(hours: number, min?: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the numeric day-of-the-month value of the Date object using local time.\\n      * @param date A numeric value equal to the day of the month.\\n      */\\n    setDate(date: number): number;\\n    /**\\n      * Sets the numeric day of the month in the Date object using Universal Coordinated Time (UTC).\\n      * @param date A numeric value equal to the day of the month.\\n      */\\n    setUTCDate(date: number): number;\\n    /**\\n      * Sets the month value in the Date object using local time.\\n      * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively.\\n      * @param date A numeric value representing the day of the month. If this value is not supplied, the value from a call to the getDate method is used.\\n      */\\n    setMonth(month: number, date?: number): number;\\n    /**\\n      * Sets the month value in the Date object using Universal Coordinated Time (UTC).\\n      * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively.\\n      * @param date A numeric value representing the day of the month. If it is not supplied, the value from a call to the getUTCDate method is used.\\n      */\\n    setUTCMonth(month: number, date?: number): number;\\n    /**\\n      * Sets the year of the Date object using local time.\\n      * @param year A numeric value for the year.\\n      * @param month A zero-based numeric value for the month (0 for January, 11 for December). Must be specified if numDate is specified.\\n      * @param date A numeric value equal for the day of the month.\\n      */\\n    setFullYear(year: number, month?: number, date?: number): number;\\n    /**\\n      * Sets the year value in the Date object using Universal Coordinated Time (UTC).\\n      * @param year A numeric value equal to the year.\\n      * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively. Must be supplied if numDate is supplied.\\n      * @param date A numeric value equal to the day of the month.\\n      */\\n    setUTCFullYear(year: number, month?: number, date?: number): number;\\n    /** Returns a date converted to a string using Universal Coordinated Time (UTC). */\\n    toUTCString(): string;\\n    /** Returns a date as a string value in ISO format. */\\n    toISOString(): string;\\n    /** Used by the JSON.stringify method to enable the transformation of an object\\'s data for JavaScript Object Notation (JSON) serialization. */\\n    toJSON(key?: any): string;\\n}\\n\\ninterface DateConstructor {\\n    new(): Date;\\n    new(value: number | string): Date;\\n    new(year: number, month: number, date?: number, hours?: number, minutes?: number, seconds?: number, ms?: number): Date;\\n    (): string;\\n    readonly prototype: Date;\\n    /**\\n      * Parses a string containing a date, and returns the number of milliseconds between that date and midnight, January 1, 1970.\\n      * @param s A date string\\n      */\\n    parse(s: string): number;\\n    /**\\n      * Returns the number of milliseconds between midnight, January 1, 1970 Universal Coordinated Time (UTC) (or GMT) and the specified date.\\n      * @param year The full year designation is required for cross-century date accuracy. If year is between 0 and 99 is used, then year is assumed to be 1900 + year.\\n      * @param month The month as an number between 0 and 11 (January to December).\\n      * @param date The date as an number between 1 and 31.\\n      * @param hours Must be supplied if minutes is supplied. An number from 0 to 23 (midnight to 11pm) that specifies the hour.\\n      * @param minutes Must be supplied if seconds is supplied. An number from 0 to 59 that specifies the minutes.\\n      * @param seconds Must be supplied if milliseconds is supplied. An number from 0 to 59 that specifies the seconds.\\n      * @param ms An number from 0 to 999 that specifies the milliseconds.\\n      */\\n    UTC(year: number, month: number, date?: number, hours?: number, minutes?: number, seconds?: number, ms?: number): number;\\n    now(): number;\\n}\\n\\ndeclare const Date: DateConstructor;\\n\\ninterface RegExpMatchArray extends Array<string> {\\n    index?: number;\\n    input?: string;\\n}\\n\\ninterface RegExpExecArray extends Array<string> {\\n    index: number;\\n    input: string;\\n}\\n\\ninterface RegExp {\\n    /**\\n      * Executes a search on a string using a regular expression pattern, and returns an array containing the results of that search.\\n      * @param string The String object or string literal on which to perform the search.\\n      */\\n    exec(string: string): RegExpExecArray | null;\\n\\n    /**\\n      * Returns a Boolean value that indicates whether or not a pattern exists in a searched string.\\n      * @param string String on which to perform the search.\\n      */\\n    test(string: string): boolean;\\n\\n    /** Returns a copy of the text of the regular expression pattern. Read-only. The regExp argument is a Regular expression object. It can be a variable name or a literal. */\\n    readonly source: string;\\n\\n    /** Returns a Boolean value indicating the state of the global flag (g) used with a regular expression. Default is false. Read-only. */\\n    readonly global: boolean;\\n\\n    /** Returns a Boolean value indicating the state of the ignoreCase flag (i) used with a regular expression. Default is false. Read-only. */\\n    readonly ignoreCase: boolean;\\n\\n    /** Returns a Boolean value indicating the state of the multiline flag (m) used with a regular expression. Default is false. Read-only. */\\n    readonly multiline: boolean;\\n\\n    lastIndex: number;\\n\\n    // Non-standard extensions\\n    compile(): this;\\n}\\n\\ninterface RegExpConstructor {\\n    new(pattern: RegExp | string): RegExp;\\n    new(pattern: string, flags?: string): RegExp;\\n    (pattern: RegExp | string): RegExp;\\n    (pattern: string, flags?: string): RegExp;\\n    readonly prototype: RegExp;\\n\\n    // Non-standard extensions\\n    $1: string;\\n    $2: string;\\n    $3: string;\\n    $4: string;\\n    $5: string;\\n    $6: string;\\n    $7: string;\\n    $8: string;\\n    $9: string;\\n    lastMatch: string;\\n}\\n\\ndeclare const RegExp: RegExpConstructor;\\n\\ninterface Error {\\n    name: string;\\n    message: string;\\n    stack?: string;\\n}\\n\\ninterface ErrorConstructor {\\n    new(message?: string): Error;\\n    (message?: string): Error;\\n    readonly prototype: Error;\\n}\\n\\ndeclare const Error: ErrorConstructor;\\n\\ninterface EvalError extends Error {\\n}\\n\\ninterface EvalErrorConstructor {\\n    new(message?: string): EvalError;\\n    (message?: string): EvalError;\\n    readonly prototype: EvalError;\\n}\\n\\ndeclare const EvalError: EvalErrorConstructor;\\n\\ninterface RangeError extends Error {\\n}\\n\\ninterface RangeErrorConstructor {\\n    new(message?: string): RangeError;\\n    (message?: string): RangeError;\\n    readonly prototype: RangeError;\\n}\\n\\ndeclare const RangeError: RangeErrorConstructor;\\n\\ninterface ReferenceError extends Error {\\n}\\n\\ninterface ReferenceErrorConstructor {\\n    new(message?: string): ReferenceError;\\n    (message?: string): ReferenceError;\\n    readonly prototype: ReferenceError;\\n}\\n\\ndeclare const ReferenceError: ReferenceErrorConstructor;\\n\\ninterface SyntaxError extends Error {\\n}\\n\\ninterface SyntaxErrorConstructor {\\n    new(message?: string): SyntaxError;\\n    (message?: string): SyntaxError;\\n    readonly prototype: SyntaxError;\\n}\\n\\ndeclare const SyntaxError: SyntaxErrorConstructor;\\n\\ninterface TypeError extends Error {\\n}\\n\\ninterface TypeErrorConstructor {\\n    new(message?: string): TypeError;\\n    (message?: string): TypeError;\\n    readonly prototype: TypeError;\\n}\\n\\ndeclare const TypeError: TypeErrorConstructor;\\n\\ninterface URIError extends Error {\\n}\\n\\ninterface URIErrorConstructor {\\n    new(message?: string): URIError;\\n    (message?: string): URIError;\\n    readonly prototype: URIError;\\n}\\n\\ndeclare const URIError: URIErrorConstructor;\\n\\ninterface JSON {\\n    /**\\n      * Converts a JavaScript Object Notation (JSON) string into an object.\\n      * @param text A valid JSON string.\\n      * @param reviver A function that transforms the results. This function is called for each member of the object.\\n      * If a member contains nested objects, the nested objects are transformed before the parent object is.\\n      */\\n    parse(text: string, reviver?: (key: any, value: any) => any): any;\\n    /**\\n      * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.\\n      * @param value A JavaScript value, usually an object or array, to be converted.\\n      * @param replacer A function that transforms the results.\\n      * @param space Adds indentation, white space, and line break characters to the return-value JSON text to make it easier to read.\\n      */\\n    stringify(value: any, replacer?: (key: string, value: any) => any, space?: string | number): string;\\n    /**\\n      * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.\\n      * @param value A JavaScript value, usually an object or array, to be converted.\\n      * @param replacer An array of strings and numbers that acts as a approved list for selecting the object properties that will be stringified.\\n      * @param space Adds indentation, white space, and line break characters to the return-value JSON text to make it easier to read.\\n      */\\n    stringify(value: any, replacer?: (number | string)[] | null, space?: string | number): string;\\n}\\n\\n/**\\n  * An intrinsic object that provides functions to convert JavaScript values to and from the JavaScript Object Notation (JSON) format.\\n  */\\ndeclare const JSON: JSON;\\n\\n\\n/////////////////////////////\\n/// ECMAScript Array API (specially handled by compiler)\\n/////////////////////////////\\n\\ninterface ReadonlyArray<T> {\\n    /**\\n      * Gets the length of the array. This is a number one higher than the highest element defined in an array.\\n      */\\n    readonly length: number;\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n    /**\\n      * Returns a string representation of an array. The elements are converted to string using their toLocalString methods.\\n      */\\n    toLocaleString(): string;\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: ConcatArray<T>[]): T[];\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: (T | ConcatArray<T>)[]): T[];\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): T[];\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.\\n      */\\n    indexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Returns the index of the last occurrence of a specified value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at the last index in the array.\\n      */\\n    lastIndexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls the callbackfn function for each element in array1 until the callbackfn returns false, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the callbackfn function for each element in array1 until the callbackfn returns true, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => void, thisArg?: any): void;\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map<U>(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => U, thisArg?: any): U[];\\n    /**\\n     * Returns the elements of an array that meet the condition specified in a callback function.\\n     * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    filter<S extends T>(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => value is S, thisArg?: any): S[];\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => any, thisArg?: any): T[];\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T): T;\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => U, initialValue: U): U;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T): T;\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => U, initialValue: U): U;\\n\\n    readonly [n: number]: T;\\n}\\n\\ninterface ConcatArray<T> {\\n    readonly length: number;\\n    readonly [n: number]: T;\\n    join(separator?: string): string;\\n    slice(start?: number, end?: number): T[];\\n}\\n\\ninterface Array<T> {\\n    /**\\n      * Gets or sets the length of the array. This is a number one higher than the highest element defined in an array.\\n      */\\n    length: number;\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n    /**\\n      * Returns a string representation of an array. The elements are converted to string using their toLocalString methods.\\n      */\\n    toLocaleString(): string;\\n    /**\\n      * Removes the last element from an array and returns it.\\n      */\\n    pop(): T | undefined;\\n    /**\\n      * Appends new elements to an array, and returns the new length of the array.\\n      * @param items New elements of the Array.\\n      */\\n    push(...items: T[]): number;\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: ConcatArray<T>[]): T[];\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: (T | ConcatArray<T>)[]): T[];\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): T[];\\n    /**\\n      * Removes the first element from an array and returns it.\\n      */\\n    shift(): T | undefined;\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): T[];\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: T, b: T) => number): this;\\n    /**\\n      * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.\\n      * @param start The zero-based location in the array from which to start removing elements.\\n      * @param deleteCount The number of elements to remove.\\n      */\\n    splice(start: number, deleteCount?: number): T[];\\n    /**\\n      * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.\\n      * @param start The zero-based location in the array from which to start removing elements.\\n      * @param deleteCount The number of elements to remove.\\n      * @param items Elements to insert into the array in place of the deleted elements.\\n      */\\n    splice(start: number, deleteCount: number, ...items: T[]): T[];\\n    /**\\n      * Inserts new elements at the start of an array.\\n      * @param items  Elements to insert at the start of the Array.\\n      */\\n    unshift(...items: T[]): number;\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.\\n      */\\n    indexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Returns the index of the last occurrence of a specified value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at the last index in the array.\\n      */\\n    lastIndexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls the callbackfn function for each element in array1 until the callbackfn returns false, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: T, index: number, array: T[]) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the callbackfn function for each element in array1 until the callbackfn returns true, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: T, index: number, array: T[]) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: T, index: number, array: T[]) => void, thisArg?: any): void;\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map<U>(callbackfn: (value: T, index: number, array: T[]) => U, thisArg?: any): U[];\\n    /**\\n     * Returns the elements of an array that meet the condition specified in a callback function.\\n     * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    filter<S extends T>(callbackfn: (value: T, index: number, array: T[]) => value is S, thisArg?: any): S[];\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: T, index: number, array: T[]) => any, thisArg?: any): T[];\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T): T;\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): U;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T): T;\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): U;\\n\\n    [n: number]: T;\\n}\\n\\ninterface ArrayConstructor {\\n    new(arrayLength?: number): any[];\\n    new <T>(arrayLength: number): T[];\\n    new <T>(...items: T[]): T[];\\n    (arrayLength?: number): any[];\\n    <T>(arrayLength: number): T[];\\n    <T>(...items: T[]): T[];\\n    isArray(arg: any): arg is Array<any>;\\n    readonly prototype: Array<any>;\\n}\\n\\ndeclare const Array: ArrayConstructor;\\n\\ninterface TypedPropertyDescriptor<T> {\\n    enumerable?: boolean;\\n    configurable?: boolean;\\n    writable?: boolean;\\n    value?: T;\\n    get?: () => T;\\n    set?: (value: T) => void;\\n}\\n\\ndeclare type ClassDecorator = <TFunction extends Function>(target: TFunction) => TFunction | void;\\ndeclare type PropertyDecorator = (target: Object, propertyKey: string | symbol) => void;\\ndeclare type MethodDecorator = <T>(target: Object, propertyKey: string | symbol, descriptor: TypedPropertyDescriptor<T>) => TypedPropertyDescriptor<T> | void;\\ndeclare type ParameterDecorator = (target: Object, propertyKey: string | symbol, parameterIndex: number) => void;\\n\\ndeclare type PromiseConstructorLike = new <T>(executor: (resolve: (value?: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void) => PromiseLike<T>;\\n\\ninterface PromiseLike<T> {\\n    /**\\n     * Attaches callbacks for the resolution and/or rejection of the Promise.\\n     * @param onfulfilled The callback to execute when the Promise is resolved.\\n     * @param onrejected The callback to execute when the Promise is rejected.\\n     * @returns A Promise for the completion of which ever callback is executed.\\n     */\\n    then<TResult1 = T, TResult2 = never>(onfulfilled?: ((value: T) => TResult1 | PromiseLike<TResult1>) | undefined | null, onrejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | undefined | null): PromiseLike<TResult1 | TResult2>;\\n}\\n\\n/**\\n * Represents the completion of an asynchronous operation\\n */\\ninterface Promise<T> {\\n    /**\\n     * Attaches callbacks for the resolution and/or rejection of the Promise.\\n     * @param onfulfilled The callback to execute when the Promise is resolved.\\n     * @param onrejected The callback to execute when the Promise is rejected.\\n     * @returns A Promise for the completion of which ever callback is executed.\\n     */\\n    then<TResult1 = T, TResult2 = never>(onfulfilled?: ((value: T) => TResult1 | PromiseLike<TResult1>) | undefined | null, onrejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | undefined | null): Promise<TResult1 | TResult2>;\\n\\n    /**\\n     * Attaches a callback for only the rejection of the Promise.\\n     * @param onrejected The callback to execute when the Promise is rejected.\\n     * @returns A Promise for the completion of the callback.\\n     */\\n    catch<TResult = never>(onrejected?: ((reason: any) => TResult | PromiseLike<TResult>) | undefined | null): Promise<T | TResult>;\\n}\\n\\ninterface ArrayLike<T> {\\n    readonly length: number;\\n    readonly [n: number]: T;\\n}\\n\\n/**\\n * Make all properties in T optional\\n */\\ntype Partial<T> = {\\n    [P in keyof T]?: T[P];\\n};\\n\\n/**\\n * Make all properties in T required\\n */\\ntype Required<T> = {\\n    [P in keyof T]-?: T[P];\\n};\\n\\n/**\\n * Make all properties in T readonly\\n */\\ntype Readonly<T> = {\\n    readonly [P in keyof T]: T[P];\\n};\\n\\n/**\\n * From T pick a set of properties K\\n */\\ntype Pick<T, K extends keyof T> = {\\n    [P in K]: T[P];\\n};\\n\\n/**\\n * Construct a type with a set of properties K of type T\\n */\\ntype Record<K extends keyof any, T> = {\\n    [P in K]: T;\\n};\\n\\n/**\\n * Exclude from T those types that are assignable to U\\n */\\ntype Exclude<T, U> = T extends U ? never : T;\\n\\n/**\\n * Extract from T those types that are assignable to U\\n */\\ntype Extract<T, U> = T extends U ? T : never;\\n\\n/**\\n * Exclude null and undefined from T\\n */\\ntype NonNullable<T> = T extends null | undefined ? never : T;\\n\\n/**\\n * Obtain the parameters of a function type in a tuple\\n */\\ntype Parameters<T extends (...args: any[]) => any> = T extends (...args: infer P) => any ? P : never;\\n\\n/**\\n * Obtain the parameters of a constructor function type in a tuple\\n */\\ntype ConstructorParameters<T extends new (...args: any[]) => any> = T extends new (...args: infer P) => any ? P : never;\\n\\n/**\\n * Obtain the return type of a function type\\n */\\ntype ReturnType<T extends (...args: any[]) => any> = T extends (...args: any[]) => infer R ? R : any;\\n\\n/**\\n * Obtain the return type of a constructor function type\\n */\\ntype InstanceType<T extends new (...args: any[]) => any> = T extends new (...args: any[]) => infer R ? R : any;\\n\\n/**\\n * Marker for contextual \\'this\\' type\\n */\\ninterface ThisType<T> { }\\n\\n/**\\n  * Represents a raw buffer of binary data, which is used to store data for the\\n  * different typed arrays. ArrayBuffers cannot be read from or written to directly,\\n  * but can be passed to a typed array or DataView Object to interpret the raw\\n  * buffer as needed.\\n  */\\ninterface ArrayBuffer {\\n    /**\\n      * Read-only. The length of the ArrayBuffer (in bytes).\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * Returns a section of an ArrayBuffer.\\n      */\\n    slice(begin: number, end?: number): ArrayBuffer;\\n}\\n\\n/**\\n * Allowed ArrayBuffer types for the buffer of an ArrayBufferView and related Typed Arrays.\\n */\\ninterface ArrayBufferTypes {\\n    ArrayBuffer: ArrayBuffer;\\n}\\ntype ArrayBufferLike = ArrayBufferTypes[keyof ArrayBufferTypes];\\n\\ninterface ArrayBufferConstructor {\\n    readonly prototype: ArrayBuffer;\\n    new(byteLength: number): ArrayBuffer;\\n    isView(arg: any): arg is ArrayBufferView;\\n}\\ndeclare const ArrayBuffer: ArrayBufferConstructor;\\n\\ninterface ArrayBufferView {\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    byteOffset: number;\\n}\\n\\ninterface DataView {\\n    readonly buffer: ArrayBuffer;\\n    readonly byteLength: number;\\n    readonly byteOffset: number;\\n    /**\\n      * Gets the Float32 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getFloat32(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Float64 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getFloat64(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Int8 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getInt8(byteOffset: number): number;\\n\\n    /**\\n      * Gets the Int16 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getInt16(byteOffset: number, littleEndian?: boolean): number;\\n    /**\\n      * Gets the Int32 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getInt32(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Uint8 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getUint8(byteOffset: number): number;\\n\\n    /**\\n      * Gets the Uint16 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getUint16(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Uint32 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getUint32(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Stores an Float32 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setFloat32(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Float64 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setFloat64(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Int8 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      */\\n    setInt8(byteOffset: number, value: number): void;\\n\\n    /**\\n      * Stores an Int16 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setInt16(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Int32 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setInt32(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Uint8 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      */\\n    setUint8(byteOffset: number, value: number): void;\\n\\n    /**\\n      * Stores an Uint16 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setUint16(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Uint32 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setUint32(byteOffset: number, value: number, littleEndian?: boolean): void;\\n}\\n\\ninterface DataViewConstructor {\\n    new(buffer: ArrayBufferLike, byteOffset?: number, byteLength?: number): DataView;\\n}\\ndeclare const DataView: DataViewConstructor;\\n\\n/**\\n  * A typed array of 8-bit integer values. The contents are initialized to 0. If the requested\\n  * number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Int8Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Int8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Int8Array) => any, thisArg?: any): Int8Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Int8Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Int8Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Int8Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Int8Array) => number, thisArg?: any): Int8Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Int8Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Int8Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Int8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Int8Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Int8Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\ninterface Int8ArrayConstructor {\\n    readonly prototype: Int8Array;\\n    new(length: number): Int8Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Int8Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Int8Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Int8Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int8Array;\\n\\n\\n}\\ndeclare const Int8Array: Int8ArrayConstructor;\\n\\n/**\\n  * A typed array of 8-bit unsigned integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint8Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint8Array) => any, thisArg?: any): Uint8Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint8Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint8Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint8Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint8Array) => number, thisArg?: any): Uint8Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint8Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint8Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint8Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint8Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint8ArrayConstructor {\\n    readonly prototype: Uint8Array;\\n    new(length: number): Uint8Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint8Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint8Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint8Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8Array;\\n\\n}\\ndeclare const Uint8Array: Uint8ArrayConstructor;\\n\\n/**\\n  * A typed array of 8-bit unsigned integer (clamped) values. The contents are initialized to 0.\\n  * If the requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint8ClampedArray {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => any, thisArg?: any): Uint8ClampedArray;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint8ClampedArray) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint8ClampedArray) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => number, thisArg?: any): Uint8ClampedArray;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint8ClampedArray;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint8ClampedArray;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint8ClampedArray view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint8ClampedArray;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint8ClampedArrayConstructor {\\n    readonly prototype: Uint8ClampedArray;\\n    new(length: number): Uint8ClampedArray;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint8ClampedArray;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint8ClampedArray;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint8ClampedArray;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8ClampedArray;\\n}\\ndeclare const Uint8ClampedArray: Uint8ClampedArrayConstructor;\\n\\n/**\\n  * A typed array of 16-bit signed integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Int16Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Int16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Int16Array) => any, thisArg?: any): Int16Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Int16Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Int16Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Int16Array) => void, thisArg?: any): void;\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Int16Array) => number, thisArg?: any): Int16Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Int16Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Int16Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Int16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Int16Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Int16Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Int16ArrayConstructor {\\n    readonly prototype: Int16Array;\\n    new(length: number): Int16Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Int16Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Int16Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Int16Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int16Array;\\n\\n\\n}\\ndeclare const Int16Array: Int16ArrayConstructor;\\n\\n/**\\n  * A typed array of 16-bit unsigned integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint16Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint16Array) => any, thisArg?: any): Uint16Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint16Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint16Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint16Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint16Array) => number, thisArg?: any): Uint16Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint16Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint16Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint16Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint16Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint16ArrayConstructor {\\n    readonly prototype: Uint16Array;\\n    new(length: number): Uint16Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint16Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint16Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint16Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint16Array;\\n\\n\\n}\\ndeclare const Uint16Array: Uint16ArrayConstructor;\\n/**\\n  * A typed array of 32-bit signed integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Int32Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Int32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Int32Array) => any, thisArg?: any): Int32Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Int32Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Int32Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Int32Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Int32Array) => number, thisArg?: any): Int32Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Int32Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Int32Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Int32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Int32Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Int32Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Int32ArrayConstructor {\\n    readonly prototype: Int32Array;\\n    new(length: number): Int32Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Int32Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Int32Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Int32Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int32Array;\\n\\n}\\ndeclare const Int32Array: Int32ArrayConstructor;\\n\\n/**\\n  * A typed array of 32-bit unsigned integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint32Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint32Array) => any, thisArg?: any): Uint32Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint32Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint32Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint32Array) => void, thisArg?: any): void;\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint32Array) => number, thisArg?: any): Uint32Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint32Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint32Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint32Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint32Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint32ArrayConstructor {\\n    readonly prototype: Uint32Array;\\n    new(length: number): Uint32Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint32Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint32Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint32Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint32Array;\\n\\n}\\ndeclare const Uint32Array: Uint32ArrayConstructor;\\n\\n/**\\n  * A typed array of 32-bit float values. The contents are initialized to 0. If the requested number\\n  * of bytes could not be allocated an exception is raised.\\n  */\\ninterface Float32Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Float32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Float32Array) => any, thisArg?: any): Float32Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Float32Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Float32Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Float32Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Float32Array) => number, thisArg?: any): Float32Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Float32Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Float32Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Float32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Float32Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Float32Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Float32ArrayConstructor {\\n    readonly prototype: Float32Array;\\n    new(length: number): Float32Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Float32Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Float32Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Float32Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float32Array;\\n\\n\\n}\\ndeclare const Float32Array: Float32ArrayConstructor;\\n\\n/**\\n  * A typed array of 64-bit float values. The contents are initialized to 0. If the requested\\n  * number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Float64Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Float64Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Float64Array) => any, thisArg?: any): Float64Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Float64Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Float64Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Float64Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Float64Array) => number, thisArg?: any): Float64Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float64Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float64Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Float64Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Float64Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Float64Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Float64Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Float64Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Float64ArrayConstructor {\\n    readonly prototype: Float64Array;\\n    new(length: number): Float64Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Float64Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Float64Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Float64Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float64Array;\\n\\n}\\ndeclare const Float64Array: Float64ArrayConstructor;\\n\\n/////////////////////////////\\n/// ECMAScript Internationalization API\\n/////////////////////////////\\n\\ndeclare namespace Intl {\\n    interface CollatorOptions {\\n        usage?: string;\\n        localeMatcher?: string;\\n        numeric?: boolean;\\n        caseFirst?: string;\\n        sensitivity?: string;\\n        ignorePunctuation?: boolean;\\n    }\\n\\n    interface ResolvedCollatorOptions {\\n        locale: string;\\n        usage: string;\\n        sensitivity: string;\\n        ignorePunctuation: boolean;\\n        collation: string;\\n        caseFirst: string;\\n        numeric: boolean;\\n    }\\n\\n    interface Collator {\\n        compare(x: string, y: string): number;\\n        resolvedOptions(): ResolvedCollatorOptions;\\n    }\\n    var Collator: {\\n        new(locales?: string | string[], options?: CollatorOptions): Collator;\\n        (locales?: string | string[], options?: CollatorOptions): Collator;\\n        supportedLocalesOf(locales: string | string[], options?: CollatorOptions): string[];\\n    };\\n\\n    interface NumberFormatOptions {\\n        localeMatcher?: string;\\n        style?: string;\\n        currency?: string;\\n        currencyDisplay?: string;\\n        useGrouping?: boolean;\\n        minimumIntegerDigits?: number;\\n        minimumFractionDigits?: number;\\n        maximumFractionDigits?: number;\\n        minimumSignificantDigits?: number;\\n        maximumSignificantDigits?: number;\\n    }\\n\\n    interface ResolvedNumberFormatOptions {\\n        locale: string;\\n        numberingSystem: string;\\n        style: string;\\n        currency?: string;\\n        currencyDisplay?: string;\\n        minimumIntegerDigits: number;\\n        minimumFractionDigits: number;\\n        maximumFractionDigits: number;\\n        minimumSignificantDigits?: number;\\n        maximumSignificantDigits?: number;\\n        useGrouping: boolean;\\n    }\\n\\n    interface NumberFormat {\\n        format(value: number): string;\\n        resolvedOptions(): ResolvedNumberFormatOptions;\\n    }\\n    var NumberFormat: {\\n        new(locales?: string | string[], options?: NumberFormatOptions): NumberFormat;\\n        (locales?: string | string[], options?: NumberFormatOptions): NumberFormat;\\n        supportedLocalesOf(locales: string | string[], options?: NumberFormatOptions): string[];\\n    };\\n\\n    interface DateTimeFormatOptions {\\n        localeMatcher?: string;\\n        weekday?: string;\\n        era?: string;\\n        year?: string;\\n        month?: string;\\n        day?: string;\\n        hour?: string;\\n        minute?: string;\\n        second?: string;\\n        timeZoneName?: string;\\n        formatMatcher?: string;\\n        hour12?: boolean;\\n        timeZone?: string;\\n    }\\n\\n    interface ResolvedDateTimeFormatOptions {\\n        locale: string;\\n        calendar: string;\\n        numberingSystem: string;\\n        timeZone: string;\\n        hour12?: boolean;\\n        weekday?: string;\\n        era?: string;\\n        year?: string;\\n        month?: string;\\n        day?: string;\\n        hour?: string;\\n        minute?: string;\\n        second?: string;\\n        timeZoneName?: string;\\n    }\\n\\n    interface DateTimeFormat {\\n        format(date?: Date | number): string;\\n        resolvedOptions(): ResolvedDateTimeFormatOptions;\\n    }\\n    var DateTimeFormat: {\\n        new(locales?: string | string[], options?: DateTimeFormatOptions): DateTimeFormat;\\n        (locales?: string | string[], options?: DateTimeFormatOptions): DateTimeFormat;\\n        supportedLocalesOf(locales: string | string[], options?: DateTimeFormatOptions): string[];\\n    };\\n}\\n\\ninterface String {\\n    /**\\n      * Determines whether two strings are equivalent in the current or specified locale.\\n      * @param that String to compare to target string\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used. This parameter must conform to BCP 47 standards; see the Intl.Collator object for details.\\n      * @param options An object that contains one or more properties that specify comparison options. see the Intl.Collator object for details.\\n      */\\n    localeCompare(that: string, locales?: string | string[], options?: Intl.CollatorOptions): number;\\n}\\n\\ninterface Number {\\n    /**\\n      * Converts a number to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleString(locales?: string | string[], options?: Intl.NumberFormatOptions): string;\\n}\\n\\ninterface Date {\\n    /**\\n      * Converts a date and time to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\\n    /**\\n      * Converts a date to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleDateString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\\n\\n    /**\\n      * Converts a time to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleTimeString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\\n}\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n/////////////////////////////\\n/// DOM APIs\\n/////////////////////////////\\n\\ninterface Account {\\n    displayName: string;\\n    id: string;\\n    imageURL?: string;\\n    name?: string;\\n    rpDisplayName: string;\\n}\\n\\ninterface AddEventListenerOptions extends EventListenerOptions {\\n    once?: boolean;\\n    passive?: boolean;\\n}\\n\\ninterface AesCbcParams extends Algorithm {\\n    iv: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface AesCtrParams extends Algorithm {\\n    counter: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    length: number;\\n}\\n\\ninterface AesDerivedKeyParams extends Algorithm {\\n    length: number;\\n}\\n\\ninterface AesGcmParams extends Algorithm {\\n    additionalData?: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    iv: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    tagLength?: number;\\n}\\n\\ninterface AesKeyAlgorithm extends KeyAlgorithm {\\n    length: number;\\n}\\n\\ninterface AesKeyGenParams extends Algorithm {\\n    length: number;\\n}\\n\\ninterface Algorithm {\\n    name: string;\\n}\\n\\ninterface AnalyserOptions extends AudioNodeOptions {\\n    fftSize?: number;\\n    maxDecibels?: number;\\n    minDecibels?: number;\\n    smoothingTimeConstant?: number;\\n}\\n\\ninterface AnimationEventInit extends EventInit {\\n    animationName?: string;\\n    elapsedTime?: number;\\n    pseudoElement?: string;\\n}\\n\\ninterface AnimationPlaybackEventInit extends EventInit {\\n    currentTime?: number | null;\\n    timelineTime?: number | null;\\n}\\n\\ninterface AssertionOptions {\\n    allowList?: ScopedCredentialDescriptor[];\\n    extensions?: WebAuthnExtensions;\\n    rpId?: string;\\n    timeoutSeconds?: number;\\n}\\n\\ninterface AssignedNodesOptions {\\n    flatten?: boolean;\\n}\\n\\ninterface AudioBufferOptions {\\n    length: number;\\n    numberOfChannels?: number;\\n    sampleRate: number;\\n}\\n\\ninterface AudioBufferSourceOptions {\\n    buffer?: AudioBuffer | null;\\n    detune?: number;\\n    loop?: boolean;\\n    loopEnd?: number;\\n    loopStart?: number;\\n    playbackRate?: number;\\n}\\n\\ninterface AudioContextInfo {\\n    currentTime?: number;\\n    sampleRate?: number;\\n}\\n\\ninterface AudioContextOptions {\\n    latencyHint?: AudioContextLatencyCategory | number;\\n    sampleRate?: number;\\n}\\n\\ninterface AudioNodeOptions {\\n    channelCount?: number;\\n    channelCountMode?: ChannelCountMode;\\n    channelInterpretation?: ChannelInterpretation;\\n}\\n\\ninterface AudioParamDescriptor {\\n    automationRate?: AutomationRate;\\n    defaultValue?: number;\\n    maxValue?: number;\\n    minValue?: number;\\n    name: string;\\n}\\n\\ninterface AudioProcessingEventInit extends EventInit {\\n    inputBuffer: AudioBuffer;\\n    outputBuffer: AudioBuffer;\\n    playbackTime: number;\\n}\\n\\ninterface AudioTimestamp {\\n    contextTime?: number;\\n    performanceTime?: number;\\n}\\n\\ninterface AudioWorkletNodeOptions extends AudioNodeOptions {\\n    numberOfInputs?: number;\\n    numberOfOutputs?: number;\\n    outputChannelCount?: number[];\\n    parameterData?: Record<string, number>;\\n    processorOptions?: any;\\n}\\n\\ninterface BiquadFilterOptions extends AudioNodeOptions {\\n    Q?: number;\\n    detune?: number;\\n    frequency?: number;\\n    gain?: number;\\n    type?: BiquadFilterType;\\n}\\n\\ninterface BlobPropertyBag {\\n    type?: string;\\n}\\n\\ninterface ByteLengthChunk {\\n    byteLength?: number;\\n}\\n\\ninterface CacheQueryOptions {\\n    cacheName?: string;\\n    ignoreMethod?: boolean;\\n    ignoreSearch?: boolean;\\n    ignoreVary?: boolean;\\n}\\n\\ninterface CanvasRenderingContext2DSettings {\\n    alpha?: boolean;\\n}\\n\\ninterface ChannelMergerOptions extends AudioNodeOptions {\\n    numberOfInputs?: number;\\n}\\n\\ninterface ChannelSplitterOptions extends AudioNodeOptions {\\n    numberOfOutputs?: number;\\n}\\n\\ninterface ClientData {\\n    challenge: string;\\n    extensions?: WebAuthnExtensions;\\n    hashAlg: string | Algorithm;\\n    origin: string;\\n    rpId: string;\\n    tokenBinding?: string;\\n}\\n\\ninterface ClientQueryOptions {\\n    includeUncontrolled?: boolean;\\n    type?: ClientTypes;\\n}\\n\\ninterface CloseEventInit extends EventInit {\\n    code?: number;\\n    reason?: string;\\n    wasClean?: boolean;\\n}\\n\\ninterface CompositionEventInit extends UIEventInit {\\n    data?: string;\\n}\\n\\ninterface ComputedEffectTiming extends EffectTiming {\\n    activeDuration?: number;\\n    currentIteration?: number | null;\\n    endTime?: number;\\n    localTime?: number | null;\\n    progress?: number | null;\\n}\\n\\ninterface ComputedKeyframe extends Record<keyof CSSStyleDeclaration, string> {\\n    composite?: CompositeOperation | null;\\n    computedOffset?: number;\\n    easing?: string;\\n    offset?: number | null;\\n}\\n\\ninterface ConfirmSiteSpecificExceptionsInformation extends ExceptionInformation {\\n    arrayOfDomainStrings?: string[];\\n}\\n\\ninterface ConstantSourceOptions {\\n    offset?: number;\\n}\\n\\ninterface ConstrainBooleanParameters {\\n    exact?: boolean;\\n    ideal?: boolean;\\n}\\n\\ninterface ConstrainDOMStringParameters {\\n    exact?: string | string[];\\n    ideal?: string | string[];\\n}\\n\\ninterface ConstrainDoubleRange extends DoubleRange {\\n    exact?: number;\\n    ideal?: number;\\n}\\n\\ninterface ConstrainLongRange extends LongRange {\\n    exact?: number;\\n    ideal?: number;\\n}\\n\\ninterface ConstrainVideoFacingModeParameters {\\n    exact?: VideoFacingModeEnum | VideoFacingModeEnum[];\\n    ideal?: VideoFacingModeEnum | VideoFacingModeEnum[];\\n}\\n\\ninterface ConvolverOptions extends AudioNodeOptions {\\n    buffer?: AudioBuffer | null;\\n    disableNormalization?: boolean;\\n}\\n\\ninterface CustomEventInit<T = any> extends EventInit {\\n    detail?: T;\\n}\\n\\ninterface DOMMatrix2DInit {\\n    a?: number;\\n    b?: number;\\n    c?: number;\\n    d?: number;\\n    e?: number;\\n    f?: number;\\n    m11?: number;\\n    m12?: number;\\n    m21?: number;\\n    m22?: number;\\n    m41?: number;\\n    m42?: number;\\n}\\n\\ninterface DOMMatrixInit extends DOMMatrix2DInit {\\n    is2D?: boolean;\\n    m13?: number;\\n    m14?: number;\\n    m23?: number;\\n    m24?: number;\\n    m31?: number;\\n    m32?: number;\\n    m33?: number;\\n    m34?: number;\\n    m43?: number;\\n    m44?: number;\\n}\\n\\ninterface DOMPointInit {\\n    w?: number;\\n    x?: number;\\n    y?: number;\\n    z?: number;\\n}\\n\\ninterface DOMQuadInit {\\n    p1?: DOMPointInit;\\n    p2?: DOMPointInit;\\n    p3?: DOMPointInit;\\n    p4?: DOMPointInit;\\n}\\n\\ninterface DOMRectInit {\\n    height?: number;\\n    width?: number;\\n    x?: number;\\n    y?: number;\\n}\\n\\ninterface DelayOptions extends AudioNodeOptions {\\n    delayTime?: number;\\n    maxDelayTime?: number;\\n}\\n\\ninterface DeviceAccelerationDict {\\n    x?: number | null;\\n    y?: number | null;\\n    z?: number | null;\\n}\\n\\ninterface DeviceLightEventInit extends EventInit {\\n    value?: number;\\n}\\n\\ninterface DeviceMotionEventInit extends EventInit {\\n    acceleration?: DeviceAccelerationDict | null;\\n    accelerationIncludingGravity?: DeviceAccelerationDict | null;\\n    interval?: number | null;\\n    rotationRate?: DeviceRotationRateDict | null;\\n}\\n\\ninterface DeviceOrientationEventInit extends EventInit {\\n    absolute?: boolean;\\n    alpha?: number | null;\\n    beta?: number | null;\\n    gamma?: number | null;\\n}\\n\\ninterface DeviceRotationRateDict {\\n    alpha?: number | null;\\n    beta?: number | null;\\n    gamma?: number | null;\\n}\\n\\ninterface DocumentTimelineOptions {\\n    originTime?: number;\\n}\\n\\ninterface DoubleRange {\\n    max?: number;\\n    min?: number;\\n}\\n\\ninterface DragEventInit extends MouseEventInit {\\n    dataTransfer?: DataTransfer | null;\\n}\\n\\ninterface DynamicsCompressorOptions extends AudioNodeOptions {\\n    attack?: number;\\n    knee?: number;\\n    ratio?: number;\\n    release?: number;\\n    threshold?: number;\\n}\\n\\ninterface EcKeyAlgorithm extends KeyAlgorithm {\\n    namedCurve: NamedCurve;\\n}\\n\\ninterface EcKeyGenParams extends Algorithm {\\n    namedCurve: NamedCurve;\\n}\\n\\ninterface EcKeyImportParams extends Algorithm {\\n    namedCurve: NamedCurve;\\n}\\n\\ninterface EcdhKeyDeriveParams extends Algorithm {\\n    public: CryptoKey;\\n}\\n\\ninterface EcdsaParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n}\\n\\ninterface EffectTiming {\\n    delay?: number;\\n    direction?: PlaybackDirection;\\n    duration?: number | string;\\n    easing?: string;\\n    endDelay?: number;\\n    fill?: FillMode;\\n    iterationStart?: number;\\n    iterations?: number;\\n}\\n\\ninterface ElementDefinitionOptions {\\n    extends?: string;\\n}\\n\\ninterface ErrorEventInit extends EventInit {\\n    colno?: number;\\n    error?: any;\\n    filename?: string;\\n    lineno?: number;\\n    message?: string;\\n}\\n\\ninterface EventInit {\\n    bubbles?: boolean;\\n    cancelable?: boolean;\\n    composed?: boolean;\\n}\\n\\ninterface EventListenerOptions {\\n    capture?: boolean;\\n}\\n\\ninterface EventModifierInit extends UIEventInit {\\n    altKey?: boolean;\\n    ctrlKey?: boolean;\\n    metaKey?: boolean;\\n    modifierAltGraph?: boolean;\\n    modifierCapsLock?: boolean;\\n    modifierFn?: boolean;\\n    modifierFnLock?: boolean;\\n    modifierHyper?: boolean;\\n    modifierNumLock?: boolean;\\n    modifierOS?: boolean;\\n    modifierScrollLock?: boolean;\\n    modifierSuper?: boolean;\\n    modifierSymbol?: boolean;\\n    modifierSymbolLock?: boolean;\\n    shiftKey?: boolean;\\n}\\n\\ninterface ExceptionInformation {\\n    domain?: string | null;\\n}\\n\\ninterface FilePropertyBag extends BlobPropertyBag {\\n    lastModified?: number;\\n}\\n\\ninterface FocusEventInit extends UIEventInit {\\n    relatedTarget?: EventTarget | null;\\n}\\n\\ninterface FocusNavigationEventInit extends EventInit {\\n    navigationReason?: string | null;\\n    originHeight?: number;\\n    originLeft?: number;\\n    originTop?: number;\\n    originWidth?: number;\\n}\\n\\ninterface FocusNavigationOrigin {\\n    originHeight?: number;\\n    originLeft?: number;\\n    originTop?: number;\\n    originWidth?: number;\\n}\\n\\ninterface FocusOptions {\\n    preventScroll?: boolean;\\n}\\n\\ninterface GainOptions extends AudioNodeOptions {\\n    gain?: number;\\n}\\n\\ninterface GamepadEventInit extends EventInit {\\n    gamepad?: Gamepad;\\n}\\n\\ninterface GetNotificationOptions {\\n    tag?: string;\\n}\\n\\ninterface GetRootNodeOptions {\\n    composed?: boolean;\\n}\\n\\ninterface HashChangeEventInit extends EventInit {\\n    newURL?: string;\\n    oldURL?: string;\\n}\\n\\ninterface HkdfParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    info: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    salt: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface HmacImportParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    length?: number;\\n}\\n\\ninterface HmacKeyAlgorithm extends KeyAlgorithm {\\n    hash: KeyAlgorithm;\\n    length: number;\\n}\\n\\ninterface HmacKeyGenParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    length?: number;\\n}\\n\\ninterface IDBIndexParameters {\\n    multiEntry?: boolean;\\n    unique?: boolean;\\n}\\n\\ninterface IDBObjectStoreParameters {\\n    autoIncrement?: boolean;\\n    keyPath?: string | string[] | null;\\n}\\n\\ninterface IDBVersionChangeEventInit extends EventInit {\\n    newVersion?: number | null;\\n    oldVersion?: number;\\n}\\n\\ninterface IIRFilterOptions extends AudioNodeOptions {\\n    feedback: number[];\\n    feedforward: number[];\\n}\\n\\ninterface IntersectionObserverEntryInit {\\n    boundingClientRect: DOMRectInit;\\n    intersectionRect: DOMRectInit;\\n    isIntersecting: boolean;\\n    rootBounds: DOMRectInit;\\n    target: Element;\\n    time: number;\\n}\\n\\ninterface IntersectionObserverInit {\\n    root?: Element | null;\\n    rootMargin?: string;\\n    threshold?: number | number[];\\n}\\n\\ninterface JsonWebKey {\\n    alg?: string;\\n    crv?: string;\\n    d?: string;\\n    dp?: string;\\n    dq?: string;\\n    e?: string;\\n    ext?: boolean;\\n    k?: string;\\n    key_ops?: string[];\\n    kty?: string;\\n    n?: string;\\n    oth?: RsaOtherPrimesInfo[];\\n    p?: string;\\n    q?: string;\\n    qi?: string;\\n    use?: string;\\n    x?: string;\\n    y?: string;\\n}\\n\\ninterface KeyAlgorithm {\\n    name: string;\\n}\\n\\ninterface KeyboardEventInit extends EventModifierInit {\\n    code?: string;\\n    key?: string;\\n    location?: number;\\n    repeat?: boolean;\\n}\\n\\ninterface Keyframe extends Record<keyof CSSStyleDeclaration, string> {\\n    composite?: CompositeOperation | null;\\n    easing?: string;\\n    offset?: number | null;\\n}\\n\\ninterface KeyframeAnimationOptions extends KeyframeEffectOptions {\\n    id?: string;\\n}\\n\\ninterface KeyframeEffectOptions extends EffectTiming {\\n    composite?: CompositeOperation;\\n    iterationComposite?: IterationCompositeOperation;\\n}\\n\\ninterface LongRange {\\n    max?: number;\\n    min?: number;\\n}\\n\\ninterface MediaElementAudioSourceOptions {\\n    mediaElement: HTMLMediaElement;\\n}\\n\\ninterface MediaEncryptedEventInit extends EventInit {\\n    initData?: ArrayBuffer | null;\\n    initDataType?: string;\\n}\\n\\ninterface MediaKeyMessageEventInit extends EventInit {\\n    message?: ArrayBuffer | null;\\n    messageType?: MediaKeyMessageType;\\n}\\n\\ninterface MediaKeySystemConfiguration {\\n    audioCapabilities?: MediaKeySystemMediaCapability[];\\n    distinctiveIdentifier?: MediaKeysRequirement;\\n    initDataTypes?: string[];\\n    persistentState?: MediaKeysRequirement;\\n    videoCapabilities?: MediaKeySystemMediaCapability[];\\n}\\n\\ninterface MediaKeySystemMediaCapability {\\n    contentType?: string;\\n    robustness?: string;\\n}\\n\\ninterface MediaQueryListEventInit extends EventInit {\\n    matches?: boolean;\\n    media?: string;\\n}\\n\\ninterface MediaStreamAudioSourceOptions {\\n    mediaStream: MediaStream;\\n}\\n\\ninterface MediaStreamConstraints {\\n    audio?: boolean | MediaTrackConstraints;\\n    peerIdentity?: string;\\n    video?: boolean | MediaTrackConstraints;\\n}\\n\\ninterface MediaStreamErrorEventInit extends EventInit {\\n    error?: MediaStreamError | null;\\n}\\n\\ninterface MediaStreamEventInit extends EventInit {\\n    stream?: MediaStream;\\n}\\n\\ninterface MediaStreamTrackAudioSourceOptions {\\n    mediaStreamTrack: MediaStreamTrack;\\n}\\n\\ninterface MediaStreamTrackEventInit extends EventInit {\\n    track?: MediaStreamTrack | null;\\n}\\n\\ninterface MediaTrackCapabilities {\\n    aspectRatio?: number | DoubleRange;\\n    deviceId?: string;\\n    echoCancellation?: boolean[];\\n    facingMode?: string;\\n    frameRate?: number | DoubleRange;\\n    groupId?: string;\\n    height?: number | LongRange;\\n    sampleRate?: number | LongRange;\\n    sampleSize?: number | LongRange;\\n    volume?: number | DoubleRange;\\n    width?: number | LongRange;\\n}\\n\\ninterface MediaTrackConstraintSet {\\n    aspectRatio?: number | ConstrainDoubleRange;\\n    channelCount?: number | ConstrainLongRange;\\n    deviceId?: string | string[] | ConstrainDOMStringParameters;\\n    displaySurface?: string | string[] | ConstrainDOMStringParameters;\\n    echoCancellation?: boolean | ConstrainBooleanParameters;\\n    facingMode?: string | string[] | ConstrainDOMStringParameters;\\n    frameRate?: number | ConstrainDoubleRange;\\n    groupId?: string | string[] | ConstrainDOMStringParameters;\\n    height?: number | ConstrainLongRange;\\n    latency?: number | ConstrainDoubleRange;\\n    logicalSurface?: boolean | ConstrainBooleanParameters;\\n    sampleRate?: number | ConstrainLongRange;\\n    sampleSize?: number | ConstrainLongRange;\\n    volume?: number | ConstrainDoubleRange;\\n    width?: number | ConstrainLongRange;\\n}\\n\\ninterface MediaTrackConstraints extends MediaTrackConstraintSet {\\n    advanced?: MediaTrackConstraintSet[];\\n}\\n\\ninterface MediaTrackSettings {\\n    aspectRatio?: number;\\n    deviceId?: string;\\n    echoCancellation?: boolean;\\n    facingMode?: string;\\n    frameRate?: number;\\n    groupId?: string;\\n    height?: number;\\n    sampleRate?: number;\\n    sampleSize?: number;\\n    volume?: number;\\n    width?: number;\\n}\\n\\ninterface MediaTrackSupportedConstraints {\\n    aspectRatio?: boolean;\\n    deviceId?: boolean;\\n    echoCancellation?: boolean;\\n    facingMode?: boolean;\\n    frameRate?: boolean;\\n    groupId?: boolean;\\n    height?: boolean;\\n    sampleRate?: boolean;\\n    sampleSize?: boolean;\\n    volume?: boolean;\\n    width?: boolean;\\n}\\n\\ninterface MessageEventInit extends EventInit {\\n    data?: any;\\n    lastEventId?: string;\\n    origin?: string;\\n    ports?: MessagePort[];\\n    source?: MessageEventSource | null;\\n}\\n\\ninterface MouseEventInit extends EventModifierInit {\\n    button?: number;\\n    buttons?: number;\\n    clientX?: number;\\n    clientY?: number;\\n    relatedTarget?: EventTarget | null;\\n    screenX?: number;\\n    screenY?: number;\\n}\\n\\ninterface MutationObserverInit {\\n    attributeFilter?: string[];\\n    attributeOldValue?: boolean;\\n    attributes?: boolean;\\n    characterData?: boolean;\\n    characterDataOldValue?: boolean;\\n    childList?: boolean;\\n    subtree?: boolean;\\n}\\n\\ninterface NavigationPreloadState {\\n    enabled?: boolean;\\n    headerValue?: string;\\n}\\n\\ninterface NotificationAction {\\n    action: string;\\n    icon?: string;\\n    title: string;\\n}\\n\\ninterface NotificationOptions {\\n    actions?: NotificationAction[];\\n    badge?: string;\\n    body?: string;\\n    data?: any;\\n    dir?: NotificationDirection;\\n    icon?: string;\\n    image?: string;\\n    lang?: string;\\n    renotify?: boolean;\\n    requireInteraction?: boolean;\\n    silent?: boolean;\\n    tag?: string;\\n    timestamp?: number;\\n    vibrate?: VibratePattern;\\n}\\n\\ninterface OfflineAudioCompletionEventInit extends EventInit {\\n    renderedBuffer: AudioBuffer;\\n}\\n\\ninterface OfflineAudioContextOptions {\\n    length: number;\\n    numberOfChannels?: number;\\n    sampleRate: number;\\n}\\n\\ninterface OptionalEffectTiming {\\n    delay?: number;\\n    direction?: PlaybackDirection;\\n    duration?: number | string;\\n    easing?: string;\\n    endDelay?: number;\\n    fill?: FillMode;\\n    iterationStart?: number;\\n    iterations?: number;\\n}\\n\\ninterface OscillatorOptions extends AudioNodeOptions {\\n    detune?: number;\\n    frequency?: number;\\n    periodicWave?: PeriodicWave;\\n    type?: OscillatorType;\\n}\\n\\ninterface PannerOptions extends AudioNodeOptions {\\n    coneInnerAngle?: number;\\n    coneOuterAngle?: number;\\n    coneOuterGain?: number;\\n    distanceModel?: DistanceModelType;\\n    maxDistance?: number;\\n    orientationX?: number;\\n    orientationY?: number;\\n    orientationZ?: number;\\n    panningModel?: PanningModelType;\\n    positionX?: number;\\n    positionY?: number;\\n    positionZ?: number;\\n    refDistance?: number;\\n    rolloffFactor?: number;\\n}\\n\\ninterface PaymentCurrencyAmount {\\n    currency: string;\\n    currencySystem?: string;\\n    value: string;\\n}\\n\\ninterface PaymentDetailsBase {\\n    displayItems?: PaymentItem[];\\n    modifiers?: PaymentDetailsModifier[];\\n    shippingOptions?: PaymentShippingOption[];\\n}\\n\\ninterface PaymentDetailsInit extends PaymentDetailsBase {\\n    id?: string;\\n    total: PaymentItem;\\n}\\n\\ninterface PaymentDetailsModifier {\\n    additionalDisplayItems?: PaymentItem[];\\n    data?: any;\\n    supportedMethods: string | string[];\\n    total?: PaymentItem;\\n}\\n\\ninterface PaymentDetailsUpdate extends PaymentDetailsBase {\\n    error?: string;\\n    total?: PaymentItem;\\n}\\n\\ninterface PaymentItem {\\n    amount: PaymentCurrencyAmount;\\n    label: string;\\n    pending?: boolean;\\n}\\n\\ninterface PaymentMethodData {\\n    data?: any;\\n    supportedMethods: string | string[];\\n}\\n\\ninterface PaymentOptions {\\n    requestPayerEmail?: boolean;\\n    requestPayerName?: boolean;\\n    requestPayerPhone?: boolean;\\n    requestShipping?: boolean;\\n    shippingType?: string;\\n}\\n\\ninterface PaymentRequestUpdateEventInit extends EventInit {\\n}\\n\\ninterface PaymentShippingOption {\\n    amount: PaymentCurrencyAmount;\\n    id: string;\\n    label: string;\\n    selected?: boolean;\\n}\\n\\ninterface Pbkdf2Params extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    iterations: number;\\n    salt: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface PerformanceObserverInit {\\n    buffered?: boolean;\\n    entryTypes: string[];\\n}\\n\\ninterface PeriodicWaveConstraints {\\n    disableNormalization?: boolean;\\n}\\n\\ninterface PeriodicWaveOptions extends PeriodicWaveConstraints {\\n    imag?: number[] | Float32Array;\\n    real?: number[] | Float32Array;\\n}\\n\\ninterface PointerEventInit extends MouseEventInit {\\n    height?: number;\\n    isPrimary?: boolean;\\n    pointerId?: number;\\n    pointerType?: string;\\n    pressure?: number;\\n    tangentialPressure?: number;\\n    tiltX?: number;\\n    tiltY?: number;\\n    twist?: number;\\n    width?: number;\\n}\\n\\ninterface PopStateEventInit extends EventInit {\\n    state?: any;\\n}\\n\\ninterface PositionOptions {\\n    enableHighAccuracy?: boolean;\\n    maximumAge?: number;\\n    timeout?: number;\\n}\\n\\ninterface ProgressEventInit extends EventInit {\\n    lengthComputable?: boolean;\\n    loaded?: number;\\n    total?: number;\\n}\\n\\ninterface PromiseRejectionEventInit extends EventInit {\\n    promise: Promise<any>;\\n    reason?: any;\\n}\\n\\ninterface PropertyIndexedKeyframes extends Record<keyof CSSStyleDeclaration, string | string[]> {\\n    composite?: CompositeOperation | (CompositeOperation | null)[];\\n    easing?: string | string[];\\n    offset?: number | (number | null)[];\\n}\\n\\ninterface PushSubscriptionJSON {\\n    endpoint?: string;\\n    expirationTime?: number | null;\\n    keys?: Record<string, string>;\\n}\\n\\ninterface PushSubscriptionOptionsInit {\\n    applicationServerKey?: BufferSource | string | null;\\n    userVisibleOnly?: boolean;\\n}\\n\\ninterface QueuingStrategy {\\n    highWaterMark?: number;\\n    size?: WritableStreamChunkCallback;\\n}\\n\\ninterface RTCAnswerOptions extends RTCOfferAnswerOptions {\\n}\\n\\ninterface RTCCertificateExpiration {\\n    expires?: number;\\n}\\n\\ninterface RTCConfiguration {\\n    bundlePolicy?: RTCBundlePolicy;\\n    certificates?: RTCCertificate[];\\n    iceCandidatePoolSize?: number;\\n    iceServers?: RTCIceServer[];\\n    iceTransportPolicy?: RTCIceTransportPolicy;\\n    peerIdentity?: string;\\n    rtcpMuxPolicy?: RTCRtcpMuxPolicy;\\n}\\n\\ninterface RTCDTMFToneChangeEventInit extends EventInit {\\n    tone: string;\\n}\\n\\ninterface RTCDataChannelEventInit extends EventInit {\\n    channel: RTCDataChannel;\\n}\\n\\ninterface RTCDataChannelInit {\\n    id?: number;\\n    maxPacketLifeTime?: number;\\n    maxRetransmits?: number;\\n    negotiated?: boolean;\\n    ordered?: boolean;\\n    priority?: RTCPriorityType;\\n    protocol?: string;\\n}\\n\\ninterface RTCDtlsFingerprint {\\n    algorithm?: string;\\n    value?: string;\\n}\\n\\ninterface RTCDtlsParameters {\\n    fingerprints?: RTCDtlsFingerprint[];\\n    role?: RTCDtlsRole;\\n}\\n\\ninterface RTCErrorEventInit extends EventInit {\\n    error?: RTCError | null;\\n}\\n\\ninterface RTCIceCandidateAttributes extends RTCStats {\\n    addressSourceUrl?: string;\\n    candidateType?: RTCStatsIceCandidateType;\\n    ipAddress?: string;\\n    portNumber?: number;\\n    priority?: number;\\n    transport?: string;\\n}\\n\\ninterface RTCIceCandidateComplete {\\n}\\n\\ninterface RTCIceCandidateDictionary {\\n    foundation?: string;\\n    ip?: string;\\n    msMTurnSessionId?: string;\\n    port?: number;\\n    priority?: number;\\n    protocol?: RTCIceProtocol;\\n    relatedAddress?: string;\\n    relatedPort?: number;\\n    tcpType?: RTCIceTcpCandidateType;\\n    type?: RTCIceCandidateType;\\n}\\n\\ninterface RTCIceCandidateInit {\\n    candidate?: string;\\n    sdpMLineIndex?: number | null;\\n    sdpMid?: string | null;\\n    usernameFragment?: string;\\n}\\n\\ninterface RTCIceCandidatePair {\\n    local?: RTCIceCandidate;\\n    remote?: RTCIceCandidate;\\n}\\n\\ninterface RTCIceCandidatePairStats extends RTCStats {\\n    availableIncomingBitrate?: number;\\n    availableOutgoingBitrate?: number;\\n    bytesReceived?: number;\\n    bytesSent?: number;\\n    localCandidateId?: string;\\n    nominated?: boolean;\\n    priority?: number;\\n    readable?: boolean;\\n    remoteCandidateId?: string;\\n    roundTripTime?: number;\\n    state?: RTCStatsIceCandidatePairState;\\n    transportId?: string;\\n    writable?: boolean;\\n}\\n\\ninterface RTCIceGatherOptions {\\n    gatherPolicy?: RTCIceGatherPolicy;\\n    iceservers?: RTCIceServer[];\\n}\\n\\ninterface RTCIceParameters {\\n    password?: string;\\n    usernameFragment?: string;\\n}\\n\\ninterface RTCIceServer {\\n    credential?: string | RTCOAuthCredential;\\n    credentialType?: RTCIceCredentialType;\\n    urls: string | string[];\\n    username?: string;\\n}\\n\\ninterface RTCIdentityProviderOptions {\\n    peerIdentity?: string;\\n    protocol?: string;\\n    usernameHint?: string;\\n}\\n\\ninterface RTCInboundRTPStreamStats extends RTCRTPStreamStats {\\n    bytesReceived?: number;\\n    fractionLost?: number;\\n    jitter?: number;\\n    packetsLost?: number;\\n    packetsReceived?: number;\\n}\\n\\ninterface RTCMediaStreamTrackStats extends RTCStats {\\n    audioLevel?: number;\\n    echoReturnLoss?: number;\\n    echoReturnLossEnhancement?: number;\\n    frameHeight?: number;\\n    frameWidth?: number;\\n    framesCorrupted?: number;\\n    framesDecoded?: number;\\n    framesDropped?: number;\\n    framesPerSecond?: number;\\n    framesReceived?: number;\\n    framesSent?: number;\\n    remoteSource?: boolean;\\n    ssrcIds?: string[];\\n    trackIdentifier?: string;\\n}\\n\\ninterface RTCOAuthCredential {\\n    accessToken: string;\\n    macKey: string;\\n}\\n\\ninterface RTCOfferAnswerOptions {\\n    voiceActivityDetection?: boolean;\\n}\\n\\ninterface RTCOfferOptions extends RTCOfferAnswerOptions {\\n    iceRestart?: boolean;\\n    offerToReceiveAudio?: boolean;\\n    offerToReceiveVideo?: boolean;\\n}\\n\\ninterface RTCOutboundRTPStreamStats extends RTCRTPStreamStats {\\n    bytesSent?: number;\\n    packetsSent?: number;\\n    roundTripTime?: number;\\n    targetBitrate?: number;\\n}\\n\\ninterface RTCPeerConnectionIceErrorEventInit extends EventInit {\\n    errorCode: number;\\n    hostCandidate?: string;\\n    statusText?: string;\\n    url?: string;\\n}\\n\\ninterface RTCPeerConnectionIceEventInit extends EventInit {\\n    candidate?: RTCIceCandidate | null;\\n    url?: string | null;\\n}\\n\\ninterface RTCRTPStreamStats extends RTCStats {\\n    associateStatsId?: string;\\n    codecId?: string;\\n    firCount?: number;\\n    isRemote?: boolean;\\n    mediaTrackId?: string;\\n    mediaType?: string;\\n    nackCount?: number;\\n    pliCount?: number;\\n    sliCount?: number;\\n    ssrc?: string;\\n    transportId?: string;\\n}\\n\\ninterface RTCRtcpFeedback {\\n    parameter?: string;\\n    type?: string;\\n}\\n\\ninterface RTCRtcpParameters {\\n    cname?: string;\\n    reducedSize?: boolean;\\n}\\n\\ninterface RTCRtpCapabilities {\\n    codecs: RTCRtpCodecCapability[];\\n    headerExtensions: RTCRtpHeaderExtensionCapability[];\\n}\\n\\ninterface RTCRtpCodecCapability {\\n    channels?: number;\\n    clockRate: number;\\n    mimeType: string;\\n    sdpFmtpLine?: string;\\n}\\n\\ninterface RTCRtpCodecParameters {\\n    channels?: number;\\n    clockRate: number;\\n    mimeType: string;\\n    payloadType: number;\\n    sdpFmtpLine?: string;\\n}\\n\\ninterface RTCRtpCodingParameters {\\n    rid?: string;\\n}\\n\\ninterface RTCRtpContributingSource {\\n    audioLevel?: number;\\n    source: number;\\n    timestamp: number;\\n}\\n\\ninterface RTCRtpDecodingParameters extends RTCRtpCodingParameters {\\n}\\n\\ninterface RTCRtpEncodingParameters extends RTCRtpCodingParameters {\\n    active?: boolean;\\n    codecPayloadType?: number;\\n    dtx?: RTCDtxStatus;\\n    maxBitrate?: number;\\n    maxFramerate?: number;\\n    priority?: RTCPriorityType;\\n    ptime?: number;\\n    scaleResolutionDownBy?: number;\\n}\\n\\ninterface RTCRtpFecParameters {\\n    mechanism?: string;\\n    ssrc?: number;\\n}\\n\\ninterface RTCRtpHeaderExtension {\\n    kind?: string;\\n    preferredEncrypt?: boolean;\\n    preferredId?: number;\\n    uri?: string;\\n}\\n\\ninterface RTCRtpHeaderExtensionCapability {\\n    uri?: string;\\n}\\n\\ninterface RTCRtpHeaderExtensionParameters {\\n    encrypted?: boolean;\\n    id: number;\\n    uri: string;\\n}\\n\\ninterface RTCRtpParameters {\\n    codecs: RTCRtpCodecParameters[];\\n    headerExtensions: RTCRtpHeaderExtensionParameters[];\\n    rtcp: RTCRtcpParameters;\\n}\\n\\ninterface RTCRtpReceiveParameters extends RTCRtpParameters {\\n    encodings: RTCRtpDecodingParameters[];\\n}\\n\\ninterface RTCRtpRtxParameters {\\n    ssrc?: number;\\n}\\n\\ninterface RTCRtpSendParameters extends RTCRtpParameters {\\n    degradationPreference?: RTCDegradationPreference;\\n    encodings: RTCRtpEncodingParameters[];\\n    transactionId: string;\\n}\\n\\ninterface RTCRtpSynchronizationSource extends RTCRtpContributingSource {\\n    voiceActivityFlag?: boolean;\\n}\\n\\ninterface RTCRtpTransceiverInit {\\n    direction?: RTCRtpTransceiverDirection;\\n    sendEncodings?: RTCRtpEncodingParameters[];\\n    streams?: MediaStream[];\\n}\\n\\ninterface RTCRtpUnhandled {\\n    muxId?: string;\\n    payloadType?: number;\\n    ssrc?: number;\\n}\\n\\ninterface RTCSessionDescriptionInit {\\n    sdp?: string;\\n    type: RTCSdpType;\\n}\\n\\ninterface RTCSrtpKeyParam {\\n    keyMethod?: string;\\n    keySalt?: string;\\n    lifetime?: string;\\n    mkiLength?: number;\\n    mkiValue?: number;\\n}\\n\\ninterface RTCSrtpSdesParameters {\\n    cryptoSuite?: string;\\n    keyParams?: RTCSrtpKeyParam[];\\n    sessionParams?: string[];\\n    tag?: number;\\n}\\n\\ninterface RTCSsrcRange {\\n    max?: number;\\n    min?: number;\\n}\\n\\ninterface RTCStats {\\n    id: string;\\n    timestamp: number;\\n    type: RTCStatsType;\\n}\\n\\ninterface RTCStatsEventInit extends EventInit {\\n    report: RTCStatsReport;\\n}\\n\\ninterface RTCStatsReport {\\n}\\n\\ninterface RTCTrackEventInit extends EventInit {\\n    receiver: RTCRtpReceiver;\\n    streams?: MediaStream[];\\n    track: MediaStreamTrack;\\n    transceiver: RTCRtpTransceiver;\\n}\\n\\ninterface RTCTransportStats extends RTCStats {\\n    activeConnection?: boolean;\\n    bytesReceived?: number;\\n    bytesSent?: number;\\n    localCertificateId?: string;\\n    remoteCertificateId?: string;\\n    rtcpTransportStatsId?: string;\\n    selectedCandidatePairId?: string;\\n}\\n\\ninterface RegistrationOptions {\\n    scope?: string;\\n    type?: WorkerType;\\n    updateViaCache?: ServiceWorkerUpdateViaCache;\\n}\\n\\ninterface RequestInit {\\n    body?: BodyInit | null;\\n    cache?: RequestCache;\\n    credentials?: RequestCredentials;\\n    headers?: HeadersInit;\\n    integrity?: string;\\n    keepalive?: boolean;\\n    method?: string;\\n    mode?: RequestMode;\\n    redirect?: RequestRedirect;\\n    referrer?: string;\\n    referrerPolicy?: ReferrerPolicy;\\n    signal?: AbortSignal | null;\\n    window?: any;\\n}\\n\\ninterface ResponseInit {\\n    headers?: HeadersInit;\\n    status?: number;\\n    statusText?: string;\\n}\\n\\ninterface RsaHashedImportParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n}\\n\\ninterface RsaHashedKeyAlgorithm extends RsaKeyAlgorithm {\\n    hash: KeyAlgorithm;\\n}\\n\\ninterface RsaHashedKeyGenParams extends RsaKeyGenParams {\\n    hash: HashAlgorithmIdentifier;\\n}\\n\\ninterface RsaKeyAlgorithm extends KeyAlgorithm {\\n    modulusLength: number;\\n    publicExponent: BigInteger;\\n}\\n\\ninterface RsaKeyGenParams extends Algorithm {\\n    modulusLength: number;\\n    publicExponent: BigInteger;\\n}\\n\\ninterface RsaOaepParams extends Algorithm {\\n    label?: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface RsaOtherPrimesInfo {\\n    d?: string;\\n    r?: string;\\n    t?: string;\\n}\\n\\ninterface RsaPssParams extends Algorithm {\\n    saltLength: number;\\n}\\n\\ninterface SVGBoundingBoxOptions {\\n    clipped?: boolean;\\n    fill?: boolean;\\n    markers?: boolean;\\n    stroke?: boolean;\\n}\\n\\ninterface ScopedCredentialDescriptor {\\n    id: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null;\\n    transports?: Transport[];\\n    type: ScopedCredentialType;\\n}\\n\\ninterface ScopedCredentialOptions {\\n    excludeList?: ScopedCredentialDescriptor[];\\n    extensions?: WebAuthnExtensions;\\n    rpId?: string;\\n    timeoutSeconds?: number;\\n}\\n\\ninterface ScopedCredentialParameters {\\n    algorithm: string | Algorithm;\\n    type: ScopedCredentialType;\\n}\\n\\ninterface ScrollIntoViewOptions extends ScrollOptions {\\n    block?: ScrollLogicalPosition;\\n    inline?: ScrollLogicalPosition;\\n}\\n\\ninterface ScrollOptions {\\n    behavior?: ScrollBehavior;\\n}\\n\\ninterface ScrollToOptions extends ScrollOptions {\\n    left?: number;\\n    top?: number;\\n}\\n\\ninterface SecurityPolicyViolationEventInit extends EventInit {\\n    blockedURI?: string;\\n    columnNumber?: number;\\n    documentURI?: string;\\n    effectiveDirective?: string;\\n    lineNumber?: number;\\n    originalPolicy?: string;\\n    referrer?: string;\\n    sourceFile?: string;\\n    statusCode?: number;\\n    violatedDirective?: string;\\n}\\n\\ninterface ServiceWorkerMessageEventInit extends EventInit {\\n    data?: any;\\n    lastEventId?: string;\\n    origin?: string;\\n    ports?: MessagePort[] | null;\\n    source?: ServiceWorker | MessagePort | null;\\n}\\n\\ninterface StereoPannerOptions extends AudioNodeOptions {\\n    pan?: number;\\n}\\n\\ninterface StorageEstimate {\\n    quota?: number;\\n    usage?: number;\\n}\\n\\ninterface StorageEventInit extends EventInit {\\n    key?: string | null;\\n    newValue?: string | null;\\n    oldValue?: string | null;\\n    storageArea?: Storage | null;\\n    url?: string;\\n}\\n\\ninterface StoreExceptionsInformation extends ExceptionInformation {\\n    detailURI?: string | null;\\n    explanationString?: string | null;\\n    siteName?: string | null;\\n}\\n\\ninterface StoreSiteSpecificExceptionsInformation extends StoreExceptionsInformation {\\n    arrayOfDomainStrings?: string[];\\n}\\n\\ninterface TextDecodeOptions {\\n    stream?: boolean;\\n}\\n\\ninterface TextDecoderOptions {\\n    fatal?: boolean;\\n    ignoreBOM?: boolean;\\n}\\n\\ninterface TouchEventInit extends EventModifierInit {\\n    changedTouches?: Touch[];\\n    targetTouches?: Touch[];\\n    touches?: Touch[];\\n}\\n\\ninterface TouchInit {\\n    altitudeAngle?: number;\\n    azimuthAngle?: number;\\n    clientX?: number;\\n    clientY?: number;\\n    force?: number;\\n    identifier: number;\\n    pageX?: number;\\n    pageY?: number;\\n    radiusX?: number;\\n    radiusY?: number;\\n    rotationAngle?: number;\\n    screenX?: number;\\n    screenY?: number;\\n    target: EventTarget;\\n    touchType?: TouchType;\\n}\\n\\ninterface TrackEventInit extends EventInit {\\n    track?: VideoTrack | AudioTrack | TextTrack | null;\\n}\\n\\ninterface TransitionEventInit extends EventInit {\\n    elapsedTime?: number;\\n    propertyName?: string;\\n    pseudoElement?: string;\\n}\\n\\ninterface UIEventInit extends EventInit {\\n    detail?: number;\\n    view?: Window | null;\\n}\\n\\ninterface UnderlyingSink {\\n    abort?: WritableStreamErrorCallback;\\n    close?: WritableStreamDefaultControllerCallback;\\n    start: WritableStreamDefaultControllerCallback;\\n    write?: WritableStreamChunkCallback;\\n}\\n\\ninterface VRDisplayEventInit extends EventInit {\\n    display: VRDisplay;\\n    reason?: VRDisplayEventReason;\\n}\\n\\ninterface VRLayer {\\n    leftBounds?: number[] | Float32Array | null;\\n    rightBounds?: number[] | Float32Array | null;\\n    source?: HTMLCanvasElement | null;\\n}\\n\\ninterface VRStageParameters {\\n    sittingToStandingTransform?: Float32Array;\\n    sizeX?: number;\\n    sizeY?: number;\\n}\\n\\ninterface WaveShaperOptions extends AudioNodeOptions {\\n    curve?: number[] | Float32Array;\\n    oversample?: OverSampleType;\\n}\\n\\ninterface WebAuthnExtensions {\\n}\\n\\ninterface WebGLContextAttributes {\\n    alpha?: GLboolean;\\n    antialias?: GLboolean;\\n    depth?: GLboolean;\\n    failIfMajorPerformanceCaveat?: boolean;\\n    powerPreference?: WebGLPowerPreference;\\n    premultipliedAlpha?: GLboolean;\\n    preserveDrawingBuffer?: GLboolean;\\n    stencil?: GLboolean;\\n}\\n\\ninterface WebGLContextEventInit extends EventInit {\\n    statusMessage?: string;\\n}\\n\\ninterface WheelEventInit extends MouseEventInit {\\n    deltaMode?: number;\\n    deltaX?: number;\\n    deltaY?: number;\\n    deltaZ?: number;\\n}\\n\\ninterface WorkletOptions {\\n    credentials?: RequestCredentials;\\n}\\n\\ninterface EventListener {\\n    (evt: Event): void;\\n}\\n\\ninterface ANGLE_instanced_arrays {\\n    drawArraysInstancedANGLE(mode: number, first: number, count: number, primcount: number): void;\\n    drawElementsInstancedANGLE(mode: number, count: number, type: number, offset: number, primcount: number): void;\\n    vertexAttribDivisorANGLE(index: number, divisor: number): void;\\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE: number;\\n}\\n\\ndeclare var ANGLE_instanced_arrays: {\\n    prototype: ANGLE_instanced_arrays;\\n    new(): ANGLE_instanced_arrays;\\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE: number;\\n};\\n\\ninterface AbortController {\\n    /**\\n     * Returns the AbortSignal object associated with this object.\\n     */\\n    readonly signal: AbortSignal;\\n    /**\\n     * Invoking this method will set this object\\'s AbortSignal\\'s aborted flag and\\n     * signal to any observers that the associated activity is to be aborted.\\n     */\\n    abort(): void;\\n}\\n\\ndeclare var AbortController: {\\n    prototype: AbortController;\\n    new(): AbortController;\\n};\\n\\ninterface AbortSignalEventMap {\\n    \"abort\": ProgressEvent;\\n}\\n\\ninterface AbortSignal extends EventTarget {\\n    /**\\n     * Returns true if this AbortSignal\\'s AbortController has signaled to abort, and false\\n     * otherwise.\\n     */\\n    readonly aborted: boolean;\\n    onabort: ((this: AbortSignal, ev: ProgressEvent) => any) | null;\\n    addEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AbortSignal: {\\n    prototype: AbortSignal;\\n    new(): AbortSignal;\\n};\\n\\ninterface AbstractRange {\\n    readonly collapsed: boolean;\\n    readonly endContainer: Node;\\n    readonly endOffset: number;\\n    readonly startContainer: Node;\\n    readonly startOffset: number;\\n}\\n\\ndeclare var AbstractRange: {\\n    prototype: AbstractRange;\\n    new(): AbstractRange;\\n};\\n\\ninterface AbstractWorkerEventMap {\\n    \"error\": ErrorEvent;\\n}\\n\\ninterface AbstractWorker {\\n    onerror: ((this: AbstractWorker, ev: ErrorEvent) => any) | null;\\n    addEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface AesCfbParams extends Algorithm {\\n    iv: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface AesCmacParams extends Algorithm {\\n    length: number;\\n}\\n\\ninterface AnalyserNode extends AudioNode {\\n    fftSize: number;\\n    readonly frequencyBinCount: number;\\n    maxDecibels: number;\\n    minDecibels: number;\\n    smoothingTimeConstant: number;\\n    getByteFrequencyData(array: Uint8Array): void;\\n    getByteTimeDomainData(array: Uint8Array): void;\\n    getFloatFrequencyData(array: Float32Array): void;\\n    getFloatTimeDomainData(array: Float32Array): void;\\n}\\n\\ndeclare var AnalyserNode: {\\n    prototype: AnalyserNode;\\n    new(context: BaseAudioContext, options?: AnalyserOptions): AnalyserNode;\\n};\\n\\ninterface Animatable {\\n    animate(keyframes: Keyframe[] | PropertyIndexedKeyframes | null, options?: number | KeyframeAnimationOptions): Animation;\\n    getAnimations(): Animation[];\\n}\\n\\ninterface AnimationEventMap {\\n    \"cancel\": AnimationPlaybackEvent;\\n    \"finish\": AnimationPlaybackEvent;\\n}\\n\\ninterface Animation extends EventTarget {\\n    currentTime: number | null;\\n    effect: AnimationEffect | null;\\n    readonly finished: Promise<Animation>;\\n    id: string;\\n    oncancel: ((this: Animation, ev: AnimationPlaybackEvent) => any) | null;\\n    onfinish: ((this: Animation, ev: AnimationPlaybackEvent) => any) | null;\\n    readonly pending: boolean;\\n    readonly playState: AnimationPlayState;\\n    playbackRate: number;\\n    readonly ready: Promise<Animation>;\\n    startTime: number | null;\\n    timeline: AnimationTimeline | null;\\n    cancel(): void;\\n    finish(): void;\\n    pause(): void;\\n    play(): void;\\n    reverse(): void;\\n    updatePlaybackRate(playbackRate: number): void;\\n    addEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: Animation, ev: AnimationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: Animation, ev: AnimationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Animation: {\\n    prototype: Animation;\\n    new(effect?: AnimationEffect | null, timeline?: AnimationTimeline | null): Animation;\\n};\\n\\ninterface AnimationEffect {\\n    getComputedTiming(): ComputedEffectTiming;\\n    getTiming(): EffectTiming;\\n    updateTiming(timing?: OptionalEffectTiming): void;\\n}\\n\\ndeclare var AnimationEffect: {\\n    prototype: AnimationEffect;\\n    new(): AnimationEffect;\\n};\\n\\ninterface AnimationEvent extends Event {\\n    readonly animationName: string;\\n    readonly elapsedTime: number;\\n    readonly pseudoElement: string;\\n}\\n\\ndeclare var AnimationEvent: {\\n    prototype: AnimationEvent;\\n    new(type: string, animationEventInitDict?: AnimationEventInit): AnimationEvent;\\n};\\n\\ninterface AnimationPlaybackEvent extends Event {\\n    readonly currentTime: number | null;\\n    readonly timelineTime: number | null;\\n}\\n\\ndeclare var AnimationPlaybackEvent: {\\n    prototype: AnimationPlaybackEvent;\\n    new(type: string, eventInitDict?: AnimationPlaybackEventInit): AnimationPlaybackEvent;\\n};\\n\\ninterface AnimationTimeline {\\n    readonly currentTime: number | null;\\n}\\n\\ndeclare var AnimationTimeline: {\\n    prototype: AnimationTimeline;\\n    new(): AnimationTimeline;\\n};\\n\\ninterface ApplicationCacheEventMap {\\n    \"cached\": Event;\\n    \"checking\": Event;\\n    \"downloading\": Event;\\n    \"error\": Event;\\n    \"noupdate\": Event;\\n    \"obsolete\": Event;\\n    \"progress\": ProgressEvent;\\n    \"updateready\": Event;\\n}\\n\\ninterface ApplicationCache extends EventTarget {\\n    /** @deprecated */\\n    oncached: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onchecking: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    ondownloading: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onerror: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onnoupdate: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onobsolete: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onprogress: ((this: ApplicationCache, ev: ProgressEvent) => any) | null;\\n    /** @deprecated */\\n    onupdateready: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    readonly status: number;\\n    /** @deprecated */\\n    abort(): void;\\n    /** @deprecated */\\n    swapCache(): void;\\n    /** @deprecated */\\n    update(): void;\\n    readonly CHECKING: number;\\n    readonly DOWNLOADING: number;\\n    readonly IDLE: number;\\n    readonly OBSOLETE: number;\\n    readonly UNCACHED: number;\\n    readonly UPDATEREADY: number;\\n    addEventListener<K extends keyof ApplicationCacheEventMap>(type: K, listener: (this: ApplicationCache, ev: ApplicationCacheEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ApplicationCacheEventMap>(type: K, listener: (this: ApplicationCache, ev: ApplicationCacheEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ApplicationCache: {\\n    prototype: ApplicationCache;\\n    new(): ApplicationCache;\\n    readonly CHECKING: number;\\n    readonly DOWNLOADING: number;\\n    readonly IDLE: number;\\n    readonly OBSOLETE: number;\\n    readonly UNCACHED: number;\\n    readonly UPDATEREADY: number;\\n};\\n\\ninterface Attr extends Node {\\n    readonly localName: string;\\n    readonly name: string;\\n    readonly namespaceURI: string | null;\\n    readonly ownerElement: Element | null;\\n    readonly prefix: string | null;\\n    readonly specified: boolean;\\n    value: string;\\n}\\n\\ndeclare var Attr: {\\n    prototype: Attr;\\n    new(): Attr;\\n};\\n\\ninterface AudioBuffer {\\n    readonly duration: number;\\n    readonly length: number;\\n    readonly numberOfChannels: number;\\n    readonly sampleRate: number;\\n    copyFromChannel(destination: Float32Array, channelNumber: number, startInChannel?: number): void;\\n    copyToChannel(source: Float32Array, channelNumber: number, startInChannel?: number): void;\\n    getChannelData(channel: number): Float32Array;\\n}\\n\\ndeclare var AudioBuffer: {\\n    prototype: AudioBuffer;\\n    new(options: AudioBufferOptions): AudioBuffer;\\n};\\n\\ninterface AudioBufferSourceNode extends AudioScheduledSourceNode {\\n    buffer: AudioBuffer | null;\\n    readonly detune: AudioParam;\\n    loop: boolean;\\n    loopEnd: number;\\n    loopStart: number;\\n    readonly playbackRate: AudioParam;\\n    start(when?: number, offset?: number, duration?: number): void;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioBufferSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioBufferSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioBufferSourceNode: {\\n    prototype: AudioBufferSourceNode;\\n    new(context: BaseAudioContext, options?: AudioBufferSourceOptions): AudioBufferSourceNode;\\n};\\n\\ninterface AudioContext extends BaseAudioContext {\\n    readonly baseLatency: number;\\n    readonly outputLatency: number;\\n    close(): Promise<void>;\\n    createMediaElementSource(mediaElement: HTMLMediaElement): MediaElementAudioSourceNode;\\n    createMediaStreamDestination(): MediaStreamAudioDestinationNode;\\n    createMediaStreamSource(mediaStream: MediaStream): MediaStreamAudioSourceNode;\\n    createMediaStreamTrackSource(mediaStreamTrack: MediaStreamTrack): MediaStreamTrackAudioSourceNode;\\n    getOutputTimestamp(): AudioTimestamp;\\n    suspend(): Promise<void>;\\n    addEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: AudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: AudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioContext: {\\n    prototype: AudioContext;\\n    new(contextOptions?: AudioContextOptions): AudioContext;\\n};\\n\\ninterface AudioDestinationNode extends AudioNode {\\n    readonly maxChannelCount: number;\\n}\\n\\ndeclare var AudioDestinationNode: {\\n    prototype: AudioDestinationNode;\\n    new(): AudioDestinationNode;\\n};\\n\\ninterface AudioListener {\\n    readonly forwardX: AudioParam;\\n    readonly forwardY: AudioParam;\\n    readonly forwardZ: AudioParam;\\n    readonly positionX: AudioParam;\\n    readonly positionY: AudioParam;\\n    readonly positionZ: AudioParam;\\n    readonly upX: AudioParam;\\n    readonly upY: AudioParam;\\n    readonly upZ: AudioParam;\\n    /** @deprecated */\\n    setOrientation(x: number, y: number, z: number, xUp: number, yUp: number, zUp: number): void;\\n    /** @deprecated */\\n    setPosition(x: number, y: number, z: number): void;\\n}\\n\\ndeclare var AudioListener: {\\n    prototype: AudioListener;\\n    new(): AudioListener;\\n};\\n\\ninterface AudioNode extends EventTarget {\\n    channelCount: number;\\n    channelCountMode: ChannelCountMode;\\n    channelInterpretation: ChannelInterpretation;\\n    readonly context: BaseAudioContext;\\n    readonly numberOfInputs: number;\\n    readonly numberOfOutputs: number;\\n    connect(destinationNode: AudioNode, output?: number, input?: number): AudioNode;\\n    connect(destinationParam: AudioParam, output?: number): void;\\n    disconnect(): void;\\n    disconnect(output: number): void;\\n    disconnect(destinationNode: AudioNode): void;\\n    disconnect(destinationNode: AudioNode, output: number): void;\\n    disconnect(destinationNode: AudioNode, output: number, input: number): void;\\n    disconnect(destinationParam: AudioParam): void;\\n    disconnect(destinationParam: AudioParam, output: number): void;\\n}\\n\\ndeclare var AudioNode: {\\n    prototype: AudioNode;\\n    new(): AudioNode;\\n};\\n\\ninterface AudioParam {\\n    automationRate: AutomationRate;\\n    readonly defaultValue: number;\\n    readonly maxValue: number;\\n    readonly minValue: number;\\n    value: number;\\n    cancelAndHoldAtTime(cancelTime: number): AudioParam;\\n    cancelScheduledValues(cancelTime: number): AudioParam;\\n    exponentialRampToValueAtTime(value: number, endTime: number): AudioParam;\\n    linearRampToValueAtTime(value: number, endTime: number): AudioParam;\\n    setTargetAtTime(target: number, startTime: number, timeConstant: number): AudioParam;\\n    setValueAtTime(value: number, startTime: number): AudioParam;\\n    setValueCurveAtTime(values: number[] | Float32Array, startTime: number, duration: number): AudioParam;\\n}\\n\\ndeclare var AudioParam: {\\n    prototype: AudioParam;\\n    new(): AudioParam;\\n};\\n\\ninterface AudioParamMap {\\n    forEach(callbackfn: (value: AudioParam, key: string, parent: AudioParamMap) => void, thisArg?: any): void;\\n}\\n\\ndeclare var AudioParamMap: {\\n    prototype: AudioParamMap;\\n    new(): AudioParamMap;\\n};\\n\\ninterface AudioProcessingEvent extends Event {\\n    readonly inputBuffer: AudioBuffer;\\n    readonly outputBuffer: AudioBuffer;\\n    readonly playbackTime: number;\\n}\\n\\ndeclare var AudioProcessingEvent: {\\n    prototype: AudioProcessingEvent;\\n    new(type: string, eventInitDict: AudioProcessingEventInit): AudioProcessingEvent;\\n};\\n\\ninterface AudioScheduledSourceNodeEventMap {\\n    \"ended\": Event;\\n}\\n\\ninterface AudioScheduledSourceNode extends AudioNode {\\n    onended: ((this: AudioScheduledSourceNode, ev: Event) => any) | null;\\n    start(when?: number): void;\\n    stop(when?: number): void;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioScheduledSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioScheduledSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioScheduledSourceNode: {\\n    prototype: AudioScheduledSourceNode;\\n    new(): AudioScheduledSourceNode;\\n};\\n\\ninterface AudioTrack {\\n    enabled: boolean;\\n    readonly id: string;\\n    kind: string;\\n    readonly label: string;\\n    language: string;\\n    readonly sourceBuffer: SourceBuffer;\\n}\\n\\ndeclare var AudioTrack: {\\n    prototype: AudioTrack;\\n    new(): AudioTrack;\\n};\\n\\ninterface AudioTrackListEventMap {\\n    \"addtrack\": TrackEvent;\\n    \"change\": Event;\\n    \"removetrack\": TrackEvent;\\n}\\n\\ninterface AudioTrackList extends EventTarget {\\n    readonly length: number;\\n    onaddtrack: ((this: AudioTrackList, ev: TrackEvent) => any) | null;\\n    onchange: ((this: AudioTrackList, ev: Event) => any) | null;\\n    onremovetrack: ((this: AudioTrackList, ev: TrackEvent) => any) | null;\\n    getTrackById(id: string): AudioTrack | null;\\n    item(index: number): AudioTrack;\\n    addEventListener<K extends keyof AudioTrackListEventMap>(type: K, listener: (this: AudioTrackList, ev: AudioTrackListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioTrackListEventMap>(type: K, listener: (this: AudioTrackList, ev: AudioTrackListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: AudioTrack;\\n}\\n\\ndeclare var AudioTrackList: {\\n    prototype: AudioTrackList;\\n    new(): AudioTrackList;\\n};\\n\\ninterface AudioWorklet extends Worklet {\\n}\\n\\ndeclare var AudioWorklet: {\\n    prototype: AudioWorklet;\\n    new(): AudioWorklet;\\n};\\n\\ninterface AudioWorkletNodeEventMap {\\n    \"processorerror\": Event;\\n}\\n\\ninterface AudioWorkletNode extends AudioNode {\\n    onprocessorerror: ((this: AudioWorkletNode, ev: Event) => any) | null;\\n    readonly parameters: AudioParamMap;\\n    readonly port: MessagePort;\\n    addEventListener<K extends keyof AudioWorkletNodeEventMap>(type: K, listener: (this: AudioWorkletNode, ev: AudioWorkletNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioWorkletNodeEventMap>(type: K, listener: (this: AudioWorkletNode, ev: AudioWorkletNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioWorkletNode: {\\n    prototype: AudioWorkletNode;\\n    new(context: BaseAudioContext, name: string, options?: AudioWorkletNodeOptions): AudioWorkletNode;\\n};\\n\\ninterface BarProp {\\n    readonly visible: boolean;\\n}\\n\\ndeclare var BarProp: {\\n    prototype: BarProp;\\n    new(): BarProp;\\n};\\n\\ninterface BaseAudioContextEventMap {\\n    \"statechange\": Event;\\n}\\n\\ninterface BaseAudioContext extends EventTarget {\\n    readonly audioWorklet: AudioWorklet;\\n    readonly currentTime: number;\\n    readonly destination: AudioDestinationNode;\\n    readonly listener: AudioListener;\\n    onstatechange: ((this: BaseAudioContext, ev: Event) => any) | null;\\n    readonly sampleRate: number;\\n    readonly state: AudioContextState;\\n    createAnalyser(): AnalyserNode;\\n    createBiquadFilter(): BiquadFilterNode;\\n    createBuffer(numberOfChannels: number, length: number, sampleRate: number): AudioBuffer;\\n    createBufferSource(): AudioBufferSourceNode;\\n    createChannelMerger(numberOfInputs?: number): ChannelMergerNode;\\n    createChannelSplitter(numberOfOutputs?: number): ChannelSplitterNode;\\n    createConstantSource(): ConstantSourceNode;\\n    createConvolver(): ConvolverNode;\\n    createDelay(maxDelayTime?: number): DelayNode;\\n    createDynamicsCompressor(): DynamicsCompressorNode;\\n    createGain(): GainNode;\\n    createIIRFilter(feedforward: number[], feedback: number[]): IIRFilterNode;\\n    createOscillator(): OscillatorNode;\\n    createPanner(): PannerNode;\\n    createPeriodicWave(real: number[] | Float32Array, imag: number[] | Float32Array, constraints?: PeriodicWaveConstraints): PeriodicWave;\\n    createScriptProcessor(bufferSize?: number, numberOfInputChannels?: number, numberOfOutputChannels?: number): ScriptProcessorNode;\\n    createStereoPanner(): StereoPannerNode;\\n    createWaveShaper(): WaveShaperNode;\\n    decodeAudioData(audioData: ArrayBuffer, successCallback?: DecodeSuccessCallback | null, errorCallback?: DecodeErrorCallback | null): Promise<AudioBuffer>;\\n    resume(): Promise<void>;\\n    addEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: BaseAudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: BaseAudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var BaseAudioContext: {\\n    prototype: BaseAudioContext;\\n    new(): BaseAudioContext;\\n};\\n\\ninterface BeforeUnloadEvent extends Event {\\n    returnValue: any;\\n}\\n\\ndeclare var BeforeUnloadEvent: {\\n    prototype: BeforeUnloadEvent;\\n    new(): BeforeUnloadEvent;\\n};\\n\\ninterface BhxBrowser {\\n    readonly lastError: DOMException;\\n    checkMatchesGlobExpression(pattern: string, value: string): boolean;\\n    checkMatchesUriExpression(pattern: string, value: string): boolean;\\n    clearLastError(): void;\\n    currentWindowId(): number;\\n    fireExtensionApiTelemetry(functionName: string, isSucceeded: boolean, isSupported: boolean, errorString: string): void;\\n    genericFunction(functionId: number, destination: any, parameters?: string, callbackId?: number): void;\\n    genericSynchronousFunction(functionId: number, parameters?: string): string;\\n    getExtensionId(): string;\\n    getThisAddress(): any;\\n    registerGenericFunctionCallbackHandler(callbackHandler: Function): void;\\n    registerGenericListenerHandler(eventHandler: Function): void;\\n    setLastError(parameters: string): void;\\n    webPlatformGenericFunction(destination: any, parameters?: string, callbackId?: number): void;\\n}\\n\\ndeclare var BhxBrowser: {\\n    prototype: BhxBrowser;\\n    new(): BhxBrowser;\\n};\\n\\ninterface BiquadFilterNode extends AudioNode {\\n    readonly Q: AudioParam;\\n    readonly detune: AudioParam;\\n    readonly frequency: AudioParam;\\n    readonly gain: AudioParam;\\n    type: BiquadFilterType;\\n    getFrequencyResponse(frequencyHz: Float32Array, magResponse: Float32Array, phaseResponse: Float32Array): void;\\n}\\n\\ndeclare var BiquadFilterNode: {\\n    prototype: BiquadFilterNode;\\n    new(context: BaseAudioContext, options?: BiquadFilterOptions): BiquadFilterNode;\\n};\\n\\ninterface Blob {\\n    readonly size: number;\\n    readonly type: string;\\n    slice(start?: number, end?: number, contentType?: string): Blob;\\n}\\n\\ndeclare var Blob: {\\n    prototype: Blob;\\n    new(blobParts?: BlobPart[], options?: BlobPropertyBag): Blob;\\n};\\n\\ninterface Body {\\n    readonly body: ReadableStream | null;\\n    readonly bodyUsed: boolean;\\n    arrayBuffer(): Promise<ArrayBuffer>;\\n    blob(): Promise<Blob>;\\n    formData(): Promise<FormData>;\\n    json(): Promise<any>;\\n    text(): Promise<string>;\\n}\\n\\ninterface BroadcastChannelEventMap {\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n}\\n\\ninterface BroadcastChannel extends EventTarget {\\n    /**\\n     * Returns the channel name (as passed to the constructor).\\n     */\\n    readonly name: string;\\n    onmessage: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\\n    /**\\n     * Closes the BroadcastChannel object, opening it up to garbage collection.\\n     */\\n    close(): void;\\n    /**\\n     * Sends the given message to other BroadcastChannel objects set up for this channel. Messages can be structured objects, e.g. nested objects and arrays.\\n     */\\n    postMessage(message: any): void;\\n    addEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var BroadcastChannel: {\\n    prototype: BroadcastChannel;\\n    new(name: string): BroadcastChannel;\\n};\\n\\ninterface BroadcastChannelEventMap {\\n    message: MessageEvent;\\n    messageerror: MessageEvent;\\n}\\n\\ninterface ByteLengthQueuingStrategy {\\n    highWaterMark: number;\\n    size(chunk?: any): number;\\n}\\n\\ndeclare var ByteLengthQueuingStrategy: {\\n    prototype: ByteLengthQueuingStrategy;\\n    new(strategy: QueuingStrategy): ByteLengthQueuingStrategy;\\n};\\n\\ninterface CDATASection extends Text {\\n}\\n\\ndeclare var CDATASection: {\\n    prototype: CDATASection;\\n    new(): CDATASection;\\n};\\n\\ninterface CSS {\\n    escape(value: string): string;\\n    supports(property: string, value?: string): boolean;\\n}\\ndeclare var CSS: CSS;\\n\\ninterface CSSConditionRule extends CSSGroupingRule {\\n    conditionText: string;\\n}\\n\\ndeclare var CSSConditionRule: {\\n    prototype: CSSConditionRule;\\n    new(): CSSConditionRule;\\n};\\n\\ninterface CSSFontFaceRule extends CSSRule {\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSFontFaceRule: {\\n    prototype: CSSFontFaceRule;\\n    new(): CSSFontFaceRule;\\n};\\n\\ninterface CSSGroupingRule extends CSSRule {\\n    readonly cssRules: CSSRuleList;\\n    deleteRule(index: number): void;\\n    insertRule(rule: string, index: number): number;\\n}\\n\\ndeclare var CSSGroupingRule: {\\n    prototype: CSSGroupingRule;\\n    new(): CSSGroupingRule;\\n};\\n\\ninterface CSSImportRule extends CSSRule {\\n    readonly href: string;\\n    readonly media: MediaList;\\n    readonly styleSheet: CSSStyleSheet;\\n}\\n\\ndeclare var CSSImportRule: {\\n    prototype: CSSImportRule;\\n    new(): CSSImportRule;\\n};\\n\\ninterface CSSKeyframeRule extends CSSRule {\\n    keyText: string;\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSKeyframeRule: {\\n    prototype: CSSKeyframeRule;\\n    new(): CSSKeyframeRule;\\n};\\n\\ninterface CSSKeyframesRule extends CSSRule {\\n    readonly cssRules: CSSRuleList;\\n    name: string;\\n    appendRule(rule: string): void;\\n    deleteRule(select: string): void;\\n    findRule(select: string): CSSKeyframeRule | null;\\n}\\n\\ndeclare var CSSKeyframesRule: {\\n    prototype: CSSKeyframesRule;\\n    new(): CSSKeyframesRule;\\n};\\n\\ninterface CSSMediaRule extends CSSConditionRule {\\n    readonly media: MediaList;\\n}\\n\\ndeclare var CSSMediaRule: {\\n    prototype: CSSMediaRule;\\n    new(): CSSMediaRule;\\n};\\n\\ninterface CSSNamespaceRule extends CSSRule {\\n    readonly namespaceURI: string;\\n    readonly prefix: string;\\n}\\n\\ndeclare var CSSNamespaceRule: {\\n    prototype: CSSNamespaceRule;\\n    new(): CSSNamespaceRule;\\n};\\n\\ninterface CSSPageRule extends CSSRule {\\n    readonly pseudoClass: string;\\n    readonly selector: string;\\n    selectorText: string;\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSPageRule: {\\n    prototype: CSSPageRule;\\n    new(): CSSPageRule;\\n};\\n\\ninterface CSSRule {\\n    cssText: string;\\n    readonly parentRule: CSSRule | null;\\n    readonly parentStyleSheet: CSSStyleSheet | null;\\n    readonly type: number;\\n    readonly CHARSET_RULE: number;\\n    readonly FONT_FACE_RULE: number;\\n    readonly IMPORT_RULE: number;\\n    readonly KEYFRAMES_RULE: number;\\n    readonly KEYFRAME_RULE: number;\\n    readonly MEDIA_RULE: number;\\n    readonly NAMESPACE_RULE: number;\\n    readonly PAGE_RULE: number;\\n    readonly STYLE_RULE: number;\\n    readonly SUPPORTS_RULE: number;\\n    readonly UNKNOWN_RULE: number;\\n    readonly VIEWPORT_RULE: number;\\n}\\n\\ndeclare var CSSRule: {\\n    prototype: CSSRule;\\n    new(): CSSRule;\\n    readonly CHARSET_RULE: number;\\n    readonly FONT_FACE_RULE: number;\\n    readonly IMPORT_RULE: number;\\n    readonly KEYFRAMES_RULE: number;\\n    readonly KEYFRAME_RULE: number;\\n    readonly MEDIA_RULE: number;\\n    readonly NAMESPACE_RULE: number;\\n    readonly PAGE_RULE: number;\\n    readonly STYLE_RULE: number;\\n    readonly SUPPORTS_RULE: number;\\n    readonly UNKNOWN_RULE: number;\\n    readonly VIEWPORT_RULE: number;\\n};\\n\\ninterface CSSRuleList {\\n    readonly length: number;\\n    item(index: number): CSSRule | null;\\n    [index: number]: CSSRule;\\n}\\n\\ndeclare var CSSRuleList: {\\n    prototype: CSSRuleList;\\n    new(): CSSRuleList;\\n};\\n\\ninterface CSSStyleDeclaration {\\n    alignContent: string | null;\\n    alignItems: string | null;\\n    alignSelf: string | null;\\n    alignmentBaseline: string | null;\\n    animation: string | null;\\n    animationDelay: string | null;\\n    animationDirection: string | null;\\n    animationDuration: string | null;\\n    animationFillMode: string | null;\\n    animationIterationCount: string | null;\\n    animationName: string | null;\\n    animationPlayState: string | null;\\n    animationTimingFunction: string | null;\\n    backfaceVisibility: string | null;\\n    background: string | null;\\n    backgroundAttachment: string | null;\\n    backgroundClip: string | null;\\n    backgroundColor: string | null;\\n    backgroundImage: string | null;\\n    backgroundOrigin: string | null;\\n    backgroundPosition: string | null;\\n    backgroundPositionX: string | null;\\n    backgroundPositionY: string | null;\\n    backgroundRepeat: string | null;\\n    backgroundSize: string | null;\\n    baselineShift: string | null;\\n    border: string | null;\\n    borderBottom: string | null;\\n    borderBottomColor: string | null;\\n    borderBottomLeftRadius: string | null;\\n    borderBottomRightRadius: string | null;\\n    borderBottomStyle: string | null;\\n    borderBottomWidth: string | null;\\n    borderCollapse: string | null;\\n    borderColor: string | null;\\n    borderImage: string | null;\\n    borderImageOutset: string | null;\\n    borderImageRepeat: string | null;\\n    borderImageSlice: string | null;\\n    borderImageSource: string | null;\\n    borderImageWidth: string | null;\\n    borderLeft: string | null;\\n    borderLeftColor: string | null;\\n    borderLeftStyle: string | null;\\n    borderLeftWidth: string | null;\\n    borderRadius: string | null;\\n    borderRight: string | null;\\n    borderRightColor: string | null;\\n    borderRightStyle: string | null;\\n    borderRightWidth: string | null;\\n    borderSpacing: string | null;\\n    borderStyle: string | null;\\n    borderTop: string | null;\\n    borderTopColor: string | null;\\n    borderTopLeftRadius: string | null;\\n    borderTopRightRadius: string | null;\\n    borderTopStyle: string | null;\\n    borderTopWidth: string | null;\\n    borderWidth: string | null;\\n    bottom: string | null;\\n    boxShadow: string | null;\\n    boxSizing: string | null;\\n    breakAfter: string | null;\\n    breakBefore: string | null;\\n    breakInside: string | null;\\n    captionSide: string | null;\\n    clear: string | null;\\n    clip: string | null;\\n    clipPath: string | null;\\n    clipRule: string | null;\\n    color: string | null;\\n    colorInterpolationFilters: string | null;\\n    columnCount: any;\\n    columnFill: string | null;\\n    columnGap: any;\\n    columnRule: string | null;\\n    columnRuleColor: any;\\n    columnRuleStyle: string | null;\\n    columnRuleWidth: any;\\n    columnSpan: string | null;\\n    columnWidth: any;\\n    columns: string | null;\\n    content: string | null;\\n    counterIncrement: string | null;\\n    counterReset: string | null;\\n    cssFloat: string | null;\\n    cssText: string;\\n    cursor: string | null;\\n    direction: string | null;\\n    display: string | null;\\n    dominantBaseline: string | null;\\n    emptyCells: string | null;\\n    enableBackground: string | null;\\n    fill: string | null;\\n    fillOpacity: string | null;\\n    fillRule: string | null;\\n    filter: string | null;\\n    flex: string | null;\\n    flexBasis: string | null;\\n    flexDirection: string | null;\\n    flexFlow: string | null;\\n    flexGrow: string | null;\\n    flexShrink: string | null;\\n    flexWrap: string | null;\\n    floodColor: string | null;\\n    floodOpacity: string | null;\\n    font: string | null;\\n    fontFamily: string | null;\\n    fontFeatureSettings: string | null;\\n    fontSize: string | null;\\n    fontSizeAdjust: string | null;\\n    fontStretch: string | null;\\n    fontStyle: string | null;\\n    fontVariant: string | null;\\n    fontWeight: string | null;\\n    gap: string | null;\\n    glyphOrientationHorizontal: string | null;\\n    glyphOrientationVertical: string | null;\\n    grid: string | null;\\n    gridArea: string | null;\\n    gridAutoColumns: string | null;\\n    gridAutoFlow: string | null;\\n    gridAutoRows: string | null;\\n    gridColumn: string | null;\\n    gridColumnEnd: string | null;\\n    gridColumnGap: string | null;\\n    gridColumnStart: string | null;\\n    gridGap: string | null;\\n    gridRow: string | null;\\n    gridRowEnd: string | null;\\n    gridRowGap: string | null;\\n    gridRowStart: string | null;\\n    gridTemplate: string | null;\\n    gridTemplateAreas: string | null;\\n    gridTemplateColumns: string | null;\\n    gridTemplateRows: string | null;\\n    height: string | null;\\n    imeMode: string | null;\\n    justifyContent: string | null;\\n    justifyItems: string | null;\\n    justifySelf: string | null;\\n    kerning: string | null;\\n    layoutGrid: string | null;\\n    layoutGridChar: string | null;\\n    layoutGridLine: string | null;\\n    layoutGridMode: string | null;\\n    layoutGridType: string | null;\\n    left: string | null;\\n    readonly length: number;\\n    letterSpacing: string | null;\\n    lightingColor: string | null;\\n    lineBreak: string | null;\\n    lineHeight: string | null;\\n    listStyle: string | null;\\n    listStyleImage: string | null;\\n    listStylePosition: string | null;\\n    listStyleType: string | null;\\n    margin: string | null;\\n    marginBottom: string | null;\\n    marginLeft: string | null;\\n    marginRight: string | null;\\n    marginTop: string | null;\\n    marker: string | null;\\n    markerEnd: string | null;\\n    markerMid: string | null;\\n    markerStart: string | null;\\n    mask: string | null;\\n    maskImage: string | null;\\n    maxHeight: string | null;\\n    maxWidth: string | null;\\n    minHeight: string | null;\\n    minWidth: string | null;\\n    msContentZoomChaining: string | null;\\n    msContentZoomLimit: string | null;\\n    msContentZoomLimitMax: any;\\n    msContentZoomLimitMin: any;\\n    msContentZoomSnap: string | null;\\n    msContentZoomSnapPoints: string | null;\\n    msContentZoomSnapType: string | null;\\n    msContentZooming: string | null;\\n    msFlowFrom: string | null;\\n    msFlowInto: string | null;\\n    msFontFeatureSettings: string | null;\\n    msGridColumn: any;\\n    msGridColumnAlign: string | null;\\n    msGridColumnSpan: any;\\n    msGridColumns: string | null;\\n    msGridRow: any;\\n    msGridRowAlign: string | null;\\n    msGridRowSpan: any;\\n    msGridRows: string | null;\\n    msHighContrastAdjust: string | null;\\n    msHyphenateLimitChars: string | null;\\n    msHyphenateLimitLines: any;\\n    msHyphenateLimitZone: any;\\n    msHyphens: string | null;\\n    msImeAlign: string | null;\\n    msOverflowStyle: string | null;\\n    msScrollChaining: string | null;\\n    msScrollLimit: string | null;\\n    msScrollLimitXMax: any;\\n    msScrollLimitXMin: any;\\n    msScrollLimitYMax: any;\\n    msScrollLimitYMin: any;\\n    msScrollRails: string | null;\\n    msScrollSnapPointsX: string | null;\\n    msScrollSnapPointsY: string | null;\\n    msScrollSnapType: string | null;\\n    msScrollSnapX: string | null;\\n    msScrollSnapY: string | null;\\n    msScrollTranslation: string | null;\\n    msTextCombineHorizontal: string | null;\\n    msTextSizeAdjust: any;\\n    msTouchAction: string | null;\\n    msTouchSelect: string | null;\\n    msUserSelect: string | null;\\n    msWrapFlow: string;\\n    msWrapMargin: any;\\n    msWrapThrough: string;\\n    objectFit: string | null;\\n    objectPosition: string | null;\\n    opacity: string | null;\\n    order: string | null;\\n    orphans: string | null;\\n    outline: string | null;\\n    outlineColor: string | null;\\n    outlineOffset: string | null;\\n    outlineStyle: string | null;\\n    outlineWidth: string | null;\\n    overflow: string | null;\\n    overflowX: string | null;\\n    overflowY: string | null;\\n    padding: string | null;\\n    paddingBottom: string | null;\\n    paddingLeft: string | null;\\n    paddingRight: string | null;\\n    paddingTop: string | null;\\n    pageBreakAfter: string | null;\\n    pageBreakBefore: string | null;\\n    pageBreakInside: string | null;\\n    readonly parentRule: CSSRule;\\n    penAction: string | null;\\n    perspective: string | null;\\n    perspectiveOrigin: string | null;\\n    pointerEvents: string | null;\\n    position: string | null;\\n    quotes: string | null;\\n    resize: string | null;\\n    right: string | null;\\n    rotate: string | null;\\n    rowGap: string | null;\\n    rubyAlign: string | null;\\n    rubyOverhang: string | null;\\n    rubyPosition: string | null;\\n    scale: string | null;\\n    stopColor: string | null;\\n    stopOpacity: string | null;\\n    stroke: string | null;\\n    strokeDasharray: string | null;\\n    strokeDashoffset: string | null;\\n    strokeLinecap: string | null;\\n    strokeLinejoin: string | null;\\n    strokeMiterlimit: string | null;\\n    strokeOpacity: string | null;\\n    strokeWidth: string | null;\\n    tableLayout: string | null;\\n    textAlign: string | null;\\n    textAlignLast: string | null;\\n    textAnchor: string | null;\\n    textCombineUpright: string | null;\\n    textDecoration: string | null;\\n    textIndent: string | null;\\n    textJustify: string | null;\\n    textKashida: string | null;\\n    textKashidaSpace: string | null;\\n    textOverflow: string | null;\\n    textShadow: string | null;\\n    textTransform: string | null;\\n    textUnderlinePosition: string | null;\\n    top: string | null;\\n    touchAction: string | null;\\n    transform: string | null;\\n    transformOrigin: string | null;\\n    transformStyle: string | null;\\n    transition: string | null;\\n    transitionDelay: string | null;\\n    transitionDuration: string | null;\\n    transitionProperty: string | null;\\n    transitionTimingFunction: string | null;\\n    translate: string | null;\\n    unicodeBidi: string | null;\\n    userSelect: string | null;\\n    verticalAlign: string | null;\\n    visibility: string | null;\\n    webkitAlignContent: string | null;\\n    webkitAlignItems: string | null;\\n    webkitAlignSelf: string | null;\\n    webkitAnimation: string | null;\\n    webkitAnimationDelay: string | null;\\n    webkitAnimationDirection: string | null;\\n    webkitAnimationDuration: string | null;\\n    webkitAnimationFillMode: string | null;\\n    webkitAnimationIterationCount: string | null;\\n    webkitAnimationName: string | null;\\n    webkitAnimationPlayState: string | null;\\n    webkitAnimationTimingFunction: string | null;\\n    webkitAppearance: string | null;\\n    webkitBackfaceVisibility: string | null;\\n    webkitBackgroundClip: string | null;\\n    webkitBackgroundOrigin: string | null;\\n    webkitBackgroundSize: string | null;\\n    webkitBorderBottomLeftRadius: string | null;\\n    webkitBorderBottomRightRadius: string | null;\\n    webkitBorderImage: string | null;\\n    webkitBorderRadius: string | null;\\n    webkitBorderTopLeftRadius: string | null;\\n    webkitBorderTopRightRadius: string | null;\\n    webkitBoxAlign: string | null;\\n    webkitBoxDirection: string | null;\\n    webkitBoxFlex: string | null;\\n    webkitBoxOrdinalGroup: string | null;\\n    webkitBoxOrient: string | null;\\n    webkitBoxPack: string | null;\\n    webkitBoxSizing: string | null;\\n    webkitColumnBreakAfter: string | null;\\n    webkitColumnBreakBefore: string | null;\\n    webkitColumnBreakInside: string | null;\\n    webkitColumnCount: any;\\n    webkitColumnGap: any;\\n    webkitColumnRule: string | null;\\n    webkitColumnRuleColor: any;\\n    webkitColumnRuleStyle: string | null;\\n    webkitColumnRuleWidth: any;\\n    webkitColumnSpan: string | null;\\n    webkitColumnWidth: any;\\n    webkitColumns: string | null;\\n    webkitFilter: string | null;\\n    webkitFlex: string | null;\\n    webkitFlexBasis: string | null;\\n    webkitFlexDirection: string | null;\\n    webkitFlexFlow: string | null;\\n    webkitFlexGrow: string | null;\\n    webkitFlexShrink: string | null;\\n    webkitFlexWrap: string | null;\\n    webkitJustifyContent: string | null;\\n    webkitOrder: string | null;\\n    webkitPerspective: string | null;\\n    webkitPerspectiveOrigin: string | null;\\n    webkitTapHighlightColor: string | null;\\n    webkitTextFillColor: string | null;\\n    webkitTextSizeAdjust: any;\\n    webkitTextStroke: string | null;\\n    webkitTextStrokeColor: string | null;\\n    webkitTextStrokeWidth: string | null;\\n    webkitTransform: string | null;\\n    webkitTransformOrigin: string | null;\\n    webkitTransformStyle: string | null;\\n    webkitTransition: string | null;\\n    webkitTransitionDelay: string | null;\\n    webkitTransitionDuration: string | null;\\n    webkitTransitionProperty: string | null;\\n    webkitTransitionTimingFunction: string | null;\\n    webkitUserModify: string | null;\\n    webkitUserSelect: string | null;\\n    webkitWritingMode: string | null;\\n    whiteSpace: string | null;\\n    widows: string | null;\\n    width: string | null;\\n    wordBreak: string | null;\\n    wordSpacing: string | null;\\n    wordWrap: string | null;\\n    writingMode: string | null;\\n    zIndex: string | null;\\n    zoom: string | null;\\n    getPropertyPriority(propertyName: string): string;\\n    getPropertyValue(propertyName: string): string;\\n    item(index: number): string;\\n    removeProperty(propertyName: string): string;\\n    setProperty(propertyName: string, value: string | null, priority?: string | null): void;\\n    [index: number]: string;\\n}\\n\\ndeclare var CSSStyleDeclaration: {\\n    prototype: CSSStyleDeclaration;\\n    new(): CSSStyleDeclaration;\\n};\\n\\ninterface CSSStyleRule extends CSSRule {\\n    selectorText: string;\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSStyleRule: {\\n    prototype: CSSStyleRule;\\n    new(): CSSStyleRule;\\n};\\n\\ninterface CSSStyleSheet extends StyleSheet {\\n    readonly cssRules: CSSRuleList;\\n    /** @deprecated */\\n    cssText: string;\\n    /** @deprecated */\\n    readonly id: string;\\n    /** @deprecated */\\n    readonly imports: StyleSheetList;\\n    /** @deprecated */\\n    readonly isAlternate: boolean;\\n    /** @deprecated */\\n    readonly isPrefAlternate: boolean;\\n    readonly ownerRule: CSSRule | null;\\n    /** @deprecated */\\n    readonly owningElement: Element;\\n    /** @deprecated */\\n    readonly pages: any;\\n    /** @deprecated */\\n    readonly readOnly: boolean;\\n    readonly rules: CSSRuleList;\\n    /** @deprecated */\\n    addImport(bstrURL: string, lIndex?: number): number;\\n    /** @deprecated */\\n    addPageRule(bstrSelector: string, bstrStyle: string, lIndex?: number): number;\\n    addRule(bstrSelector: string, bstrStyle?: string, lIndex?: number): number;\\n    deleteRule(index?: number): void;\\n    insertRule(rule: string, index?: number): number;\\n    /** @deprecated */\\n    removeImport(lIndex: number): void;\\n    removeRule(lIndex: number): void;\\n}\\n\\ndeclare var CSSStyleSheet: {\\n    prototype: CSSStyleSheet;\\n    new(): CSSStyleSheet;\\n};\\n\\ninterface CSSSupportsRule extends CSSConditionRule {\\n}\\n\\ndeclare var CSSSupportsRule: {\\n    prototype: CSSSupportsRule;\\n    new(): CSSSupportsRule;\\n};\\n\\ninterface Cache {\\n    add(request: RequestInfo): Promise<void>;\\n    addAll(requests: RequestInfo[]): Promise<void>;\\n    delete(request: RequestInfo, options?: CacheQueryOptions): Promise<boolean>;\\n    keys(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Request>>;\\n    match(request: RequestInfo, options?: CacheQueryOptions): Promise<Response | undefined>;\\n    matchAll(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Response>>;\\n    put(request: RequestInfo, response: Response): Promise<void>;\\n}\\n\\ndeclare var Cache: {\\n    prototype: Cache;\\n    new(): Cache;\\n};\\n\\ninterface CacheStorage {\\n    delete(cacheName: string): Promise<boolean>;\\n    has(cacheName: string): Promise<boolean>;\\n    keys(): Promise<string[]>;\\n    match(request: RequestInfo, options?: CacheQueryOptions): Promise<Response | undefined>;\\n    open(cacheName: string): Promise<Cache>;\\n}\\n\\ndeclare var CacheStorage: {\\n    prototype: CacheStorage;\\n    new(): CacheStorage;\\n};\\n\\ninterface CanvasCompositing {\\n    globalAlpha: number;\\n    globalCompositeOperation: string;\\n}\\n\\ninterface CanvasDrawImage {\\n    drawImage(image: CanvasImageSource, dx: number, dy: number): void;\\n    drawImage(image: CanvasImageSource, dx: number, dy: number, dw: number, dh: number): void;\\n    drawImage(image: CanvasImageSource, sx: number, sy: number, sw: number, sh: number, dx: number, dy: number, dw: number, dh: number): void;\\n}\\n\\ninterface CanvasDrawPath {\\n    beginPath(): void;\\n    clip(fillRule?: CanvasFillRule): void;\\n    clip(path: Path2D, fillRule?: CanvasFillRule): void;\\n    fill(fillRule?: CanvasFillRule): void;\\n    fill(path: Path2D, fillRule?: CanvasFillRule): void;\\n    isPointInPath(x: number, y: number, fillRule?: CanvasFillRule): boolean;\\n    isPointInPath(path: Path2D, x: number, y: number, fillRule?: CanvasFillRule): boolean;\\n    isPointInStroke(x: number, y: number): boolean;\\n    isPointInStroke(path: Path2D, x: number, y: number): boolean;\\n    stroke(): void;\\n    stroke(path: Path2D): void;\\n}\\n\\ninterface CanvasFillStrokeStyles {\\n    fillStyle: string | CanvasGradient | CanvasPattern;\\n    strokeStyle: string | CanvasGradient | CanvasPattern;\\n    createLinearGradient(x0: number, y0: number, x1: number, y1: number): CanvasGradient;\\n    createPattern(image: CanvasImageSource, repetition: string): CanvasPattern | null;\\n    createRadialGradient(x0: number, y0: number, r0: number, x1: number, y1: number, r1: number): CanvasGradient;\\n}\\n\\ninterface CanvasFilters {\\n    filter: string;\\n}\\n\\ninterface CanvasGradient {\\n    /**\\n     * Adds a color stop with the given color to the gradient at the given offset. 0.0 is the offset\\n     * at one end of the gradient, 1.0 is the offset at the other end.\\n     * Throws an \"IndexSizeError\" DOMException if the offset\\n     * is out of range. Throws a \"SyntaxError\" DOMException if\\n     * the color cannot be parsed.\\n     */\\n    addColorStop(offset: number, color: string): void;\\n}\\n\\ndeclare var CanvasGradient: {\\n    prototype: CanvasGradient;\\n    new(): CanvasGradient;\\n};\\n\\ninterface CanvasImageData {\\n    createImageData(sw: number, sh: number): ImageData;\\n    createImageData(imagedata: ImageData): ImageData;\\n    getImageData(sx: number, sy: number, sw: number, sh: number): ImageData;\\n    putImageData(imagedata: ImageData, dx: number, dy: number): void;\\n    putImageData(imagedata: ImageData, dx: number, dy: number, dirtyX: number, dirtyY: number, dirtyWidth: number, dirtyHeight: number): void;\\n}\\n\\ninterface CanvasImageSmoothing {\\n    imageSmoothingEnabled: boolean;\\n    imageSmoothingQuality: ImageSmoothingQuality;\\n}\\n\\ninterface CanvasPath {\\n    arc(x: number, y: number, radius: number, startAngle: number, endAngle: number, anticlockwise?: boolean): void;\\n    arcTo(x1: number, y1: number, x2: number, y2: number, radius: number): void;\\n    bezierCurveTo(cp1x: number, cp1y: number, cp2x: number, cp2y: number, x: number, y: number): void;\\n    closePath(): void;\\n    ellipse(x: number, y: number, radiusX: number, radiusY: number, rotation: number, startAngle: number, endAngle: number, anticlockwise?: boolean): void;\\n    lineTo(x: number, y: number): void;\\n    moveTo(x: number, y: number): void;\\n    quadraticCurveTo(cpx: number, cpy: number, x: number, y: number): void;\\n    rect(x: number, y: number, w: number, h: number): void;\\n}\\n\\ninterface CanvasPathDrawingStyles {\\n    lineCap: CanvasLineCap;\\n    lineDashOffset: number;\\n    lineJoin: CanvasLineJoin;\\n    lineWidth: number;\\n    miterLimit: number;\\n    getLineDash(): number[];\\n    setLineDash(segments: number[]): void;\\n}\\n\\ninterface CanvasPattern {\\n    /**\\n     * Sets the transformation matrix that will be used when rendering the pattern during a fill or\\n     * stroke painting operation.\\n     */\\n    setTransform(transform?: DOMMatrix2DInit): void;\\n}\\n\\ndeclare var CanvasPattern: {\\n    prototype: CanvasPattern;\\n    new(): CanvasPattern;\\n};\\n\\ninterface CanvasRect {\\n    clearRect(x: number, y: number, w: number, h: number): void;\\n    fillRect(x: number, y: number, w: number, h: number): void;\\n    strokeRect(x: number, y: number, w: number, h: number): void;\\n}\\n\\ninterface CanvasRenderingContext2D extends CanvasState, CanvasTransform, CanvasCompositing, CanvasImageSmoothing, CanvasFillStrokeStyles, CanvasShadowStyles, CanvasFilters, CanvasRect, CanvasDrawPath, CanvasUserInterface, CanvasText, CanvasDrawImage, CanvasImageData, CanvasPathDrawingStyles, CanvasTextDrawingStyles, CanvasPath {\\n    readonly canvas: HTMLCanvasElement;\\n}\\n\\ndeclare var CanvasRenderingContext2D: {\\n    prototype: CanvasRenderingContext2D;\\n    new(): CanvasRenderingContext2D;\\n};\\n\\ninterface CanvasShadowStyles {\\n    shadowBlur: number;\\n    shadowColor: string;\\n    shadowOffsetX: number;\\n    shadowOffsetY: number;\\n}\\n\\ninterface CanvasState {\\n    restore(): void;\\n    save(): void;\\n}\\n\\ninterface CanvasText {\\n    fillText(text: string, x: number, y: number, maxWidth?: number): void;\\n    measureText(text: string): TextMetrics;\\n    strokeText(text: string, x: number, y: number, maxWidth?: number): void;\\n}\\n\\ninterface CanvasTextDrawingStyles {\\n    direction: CanvasDirection;\\n    font: string;\\n    textAlign: CanvasTextAlign;\\n    textBaseline: CanvasTextBaseline;\\n}\\n\\ninterface CanvasTransform {\\n    getTransform(): DOMMatrix;\\n    resetTransform(): void;\\n    rotate(angle: number): void;\\n    scale(x: number, y: number): void;\\n    setTransform(a: number, b: number, c: number, d: number, e: number, f: number): void;\\n    setTransform(transform?: DOMMatrix2DInit): void;\\n    transform(a: number, b: number, c: number, d: number, e: number, f: number): void;\\n    translate(x: number, y: number): void;\\n}\\n\\ninterface CanvasUserInterface {\\n    drawFocusIfNeeded(element: Element): void;\\n    drawFocusIfNeeded(path: Path2D, element: Element): void;\\n    scrollPathIntoView(): void;\\n    scrollPathIntoView(path: Path2D): void;\\n}\\n\\ninterface CaretPosition {\\n    readonly offset: number;\\n    readonly offsetNode: Node;\\n    getClientRect(): DOMRect | null;\\n}\\n\\ndeclare var CaretPosition: {\\n    prototype: CaretPosition;\\n    new(): CaretPosition;\\n};\\n\\ninterface ChannelMergerNode extends AudioNode {\\n}\\n\\ndeclare var ChannelMergerNode: {\\n    prototype: ChannelMergerNode;\\n    new(context: BaseAudioContext, options?: ChannelMergerOptions): ChannelMergerNode;\\n};\\n\\ninterface ChannelSplitterNode extends AudioNode {\\n}\\n\\ndeclare var ChannelSplitterNode: {\\n    prototype: ChannelSplitterNode;\\n    new(context: BaseAudioContext, options?: ChannelSplitterOptions): ChannelSplitterNode;\\n};\\n\\ninterface CharacterData extends Node, NonDocumentTypeChildNode, ChildNode {\\n    data: string;\\n    readonly length: number;\\n    appendData(data: string): void;\\n    deleteData(offset: number, count: number): void;\\n    insertData(offset: number, data: string): void;\\n    replaceData(offset: number, count: number, data: string): void;\\n    substringData(offset: number, count: number): string;\\n}\\n\\ndeclare var CharacterData: {\\n    prototype: CharacterData;\\n    new(): CharacterData;\\n};\\n\\ninterface ChildNode extends Node {\\n    /**\\n     * Inserts nodes just after node, while replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    after(...nodes: (Node | string)[]): void;\\n    /**\\n     * Inserts nodes just before node, while replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    before(...nodes: (Node | string)[]): void;\\n    /**\\n     * Removes node.\\n     */\\n    remove(): void;\\n    /**\\n     * Replaces node with nodes, while replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    replaceWith(...nodes: (Node | string)[]): void;\\n}\\n\\ninterface ClientRect {\\n    bottom: number;\\n    readonly height: number;\\n    left: number;\\n    right: number;\\n    top: number;\\n    readonly width: number;\\n}\\n\\ndeclare var ClientRect: {\\n    prototype: ClientRect;\\n    new(): ClientRect;\\n};\\n\\ninterface ClientRectList {\\n    readonly length: number;\\n    item(index: number): ClientRect;\\n    [index: number]: ClientRect;\\n}\\n\\ndeclare var ClientRectList: {\\n    prototype: ClientRectList;\\n    new(): ClientRectList;\\n};\\n\\ninterface ClipboardEvent extends Event {\\n    readonly clipboardData: DataTransfer;\\n}\\n\\ndeclare var ClipboardEvent: {\\n    prototype: ClipboardEvent;\\n    new(type: string, eventInitDict?: ClipboardEventInit): ClipboardEvent;\\n};\\n\\ninterface ClipboardEventInit extends EventInit {\\n    data?: string;\\n    dataType?: string;\\n}\\n\\ninterface CloseEvent extends Event {\\n    readonly code: number;\\n    readonly reason: string;\\n    readonly wasClean: boolean;\\n    /** @deprecated */\\n    initCloseEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, wasCleanArg: boolean, codeArg: number, reasonArg: string): void;\\n}\\n\\ndeclare var CloseEvent: {\\n    prototype: CloseEvent;\\n    new(type: string, eventInitDict?: CloseEventInit): CloseEvent;\\n};\\n\\ninterface Comment extends CharacterData {\\n}\\n\\ndeclare var Comment: {\\n    prototype: Comment;\\n    new(data?: string): Comment;\\n};\\n\\ninterface CompositionEvent extends UIEvent {\\n    readonly data: string;\\n    readonly locale: string;\\n    initCompositionEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, dataArg: string, locale: string): void;\\n}\\n\\ndeclare var CompositionEvent: {\\n    prototype: CompositionEvent;\\n    new(typeArg: string, eventInitDict?: CompositionEventInit): CompositionEvent;\\n};\\n\\ninterface ConcatParams extends Algorithm {\\n    algorithmId: Uint8Array;\\n    hash?: string | Algorithm;\\n    partyUInfo: Uint8Array;\\n    partyVInfo: Uint8Array;\\n    privateInfo?: Uint8Array;\\n    publicInfo?: Uint8Array;\\n}\\n\\ninterface Console {\\n    memory: any;\\n    assert(condition?: boolean, message?: string, ...data: any[]): void;\\n    clear(): void;\\n    count(label?: string): void;\\n    debug(message?: any, ...optionalParams: any[]): void;\\n    dir(value?: any, ...optionalParams: any[]): void;\\n    dirxml(value: any): void;\\n    error(message?: any, ...optionalParams: any[]): void;\\n    exception(message?: string, ...optionalParams: any[]): void;\\n    group(groupTitle?: string, ...optionalParams: any[]): void;\\n    groupCollapsed(groupTitle?: string, ...optionalParams: any[]): void;\\n    groupEnd(): void;\\n    info(message?: any, ...optionalParams: any[]): void;\\n    log(message?: any, ...optionalParams: any[]): void;\\n    markTimeline(label?: string): void;\\n    profile(reportName?: string): void;\\n    profileEnd(reportName?: string): void;\\n    table(...tabularData: any[]): void;\\n    time(label?: string): void;\\n    timeEnd(label?: string): void;\\n    timeStamp(label?: string): void;\\n    timeline(label?: string): void;\\n    timelineEnd(label?: string): void;\\n    trace(message?: any, ...optionalParams: any[]): void;\\n    warn(message?: any, ...optionalParams: any[]): void;\\n}\\n\\ndeclare var Console: {\\n    prototype: Console;\\n    new(): Console;\\n};\\n\\ninterface ConstantSourceNode extends AudioScheduledSourceNode {\\n    readonly offset: AudioParam;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: ConstantSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: ConstantSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ConstantSourceNode: {\\n    prototype: ConstantSourceNode;\\n    new(context: BaseAudioContext, options?: ConstantSourceOptions): ConstantSourceNode;\\n};\\n\\ninterface ConvolverNode extends AudioNode {\\n    buffer: AudioBuffer | null;\\n    normalize: boolean;\\n}\\n\\ndeclare var ConvolverNode: {\\n    prototype: ConvolverNode;\\n    new(context: BaseAudioContext, options?: ConvolverOptions): ConvolverNode;\\n};\\n\\ninterface Coordinates {\\n    readonly accuracy: number;\\n    readonly altitude: number | null;\\n    readonly altitudeAccuracy: number | null;\\n    readonly heading: number | null;\\n    readonly latitude: number;\\n    readonly longitude: number;\\n    readonly speed: number | null;\\n}\\n\\ninterface CountQueuingStrategy {\\n    highWaterMark: number;\\n    size(): number;\\n}\\n\\ndeclare var CountQueuingStrategy: {\\n    prototype: CountQueuingStrategy;\\n    new(strategy: QueuingStrategy): CountQueuingStrategy;\\n};\\n\\ninterface Crypto {\\n    readonly subtle: SubtleCrypto;\\n    getRandomValues<T extends Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | null>(array: T): T;\\n}\\n\\ndeclare var Crypto: {\\n    prototype: Crypto;\\n    new(): Crypto;\\n};\\n\\ninterface CryptoKey {\\n    readonly algorithm: KeyAlgorithm;\\n    readonly extractable: boolean;\\n    readonly type: KeyType;\\n    readonly usages: KeyUsage[];\\n}\\n\\ndeclare var CryptoKey: {\\n    prototype: CryptoKey;\\n    new(): CryptoKey;\\n};\\n\\ninterface CryptoKeyPair {\\n    privateKey: CryptoKey;\\n    publicKey: CryptoKey;\\n}\\n\\ndeclare var CryptoKeyPair: {\\n    prototype: CryptoKeyPair;\\n    new(): CryptoKeyPair;\\n};\\n\\ninterface CustomElementRegistry {\\n    define(name: string, constructor: Function, options?: ElementDefinitionOptions): void;\\n    get(name: string): any;\\n    upgrade(root: Node): void;\\n    whenDefined(name: string): Promise<void>;\\n}\\n\\ndeclare var CustomElementRegistry: {\\n    prototype: CustomElementRegistry;\\n    new(): CustomElementRegistry;\\n};\\n\\ninterface CustomEvent<T = any> extends Event {\\n    /**\\n     * Returns any custom data event was created with.\\n     * Typically used for synthetic events.\\n     */\\n    readonly detail: T;\\n    initCustomEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, detailArg: T): void;\\n}\\n\\ndeclare var CustomEvent: {\\n    prototype: CustomEvent;\\n    new<T>(typeArg: string, eventInitDict?: CustomEventInit<T>): CustomEvent<T>;\\n};\\n\\ninterface DOMError {\\n    readonly name: string;\\n    toString(): string;\\n}\\n\\ndeclare var DOMError: {\\n    prototype: DOMError;\\n    new(): DOMError;\\n};\\n\\ninterface DOMException {\\n    readonly code: number;\\n    readonly message: string;\\n    readonly name: string;\\n    readonly ABORT_ERR: number;\\n    readonly DATA_CLONE_ERR: number;\\n    readonly DOMSTRING_SIZE_ERR: number;\\n    readonly HIERARCHY_REQUEST_ERR: number;\\n    readonly INDEX_SIZE_ERR: number;\\n    readonly INUSE_ATTRIBUTE_ERR: number;\\n    readonly INVALID_ACCESS_ERR: number;\\n    readonly INVALID_CHARACTER_ERR: number;\\n    readonly INVALID_MODIFICATION_ERR: number;\\n    readonly INVALID_NODE_TYPE_ERR: number;\\n    readonly INVALID_STATE_ERR: number;\\n    readonly NAMESPACE_ERR: number;\\n    readonly NETWORK_ERR: number;\\n    readonly NOT_FOUND_ERR: number;\\n    readonly NOT_SUPPORTED_ERR: number;\\n    readonly NO_DATA_ALLOWED_ERR: number;\\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\\n    readonly QUOTA_EXCEEDED_ERR: number;\\n    readonly SECURITY_ERR: number;\\n    readonly SYNTAX_ERR: number;\\n    readonly TIMEOUT_ERR: number;\\n    readonly TYPE_MISMATCH_ERR: number;\\n    readonly URL_MISMATCH_ERR: number;\\n    readonly VALIDATION_ERR: number;\\n    readonly WRONG_DOCUMENT_ERR: number;\\n}\\n\\ndeclare var DOMException: {\\n    prototype: DOMException;\\n    new(message?: string, name?: string): DOMException;\\n    readonly ABORT_ERR: number;\\n    readonly DATA_CLONE_ERR: number;\\n    readonly DOMSTRING_SIZE_ERR: number;\\n    readonly HIERARCHY_REQUEST_ERR: number;\\n    readonly INDEX_SIZE_ERR: number;\\n    readonly INUSE_ATTRIBUTE_ERR: number;\\n    readonly INVALID_ACCESS_ERR: number;\\n    readonly INVALID_CHARACTER_ERR: number;\\n    readonly INVALID_MODIFICATION_ERR: number;\\n    readonly INVALID_NODE_TYPE_ERR: number;\\n    readonly INVALID_STATE_ERR: number;\\n    readonly NAMESPACE_ERR: number;\\n    readonly NETWORK_ERR: number;\\n    readonly NOT_FOUND_ERR: number;\\n    readonly NOT_SUPPORTED_ERR: number;\\n    readonly NO_DATA_ALLOWED_ERR: number;\\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\\n    readonly QUOTA_EXCEEDED_ERR: number;\\n    readonly SECURITY_ERR: number;\\n    readonly SYNTAX_ERR: number;\\n    readonly TIMEOUT_ERR: number;\\n    readonly TYPE_MISMATCH_ERR: number;\\n    readonly URL_MISMATCH_ERR: number;\\n    readonly VALIDATION_ERR: number;\\n    readonly WRONG_DOCUMENT_ERR: number;\\n};\\n\\ninterface DOMImplementation {\\n    createDocument(namespaceURI: string | null, qualifiedName: string | null, doctype: DocumentType | null): Document;\\n    createDocumentType(qualifiedName: string, publicId: string, systemId: string): DocumentType;\\n    createHTMLDocument(title?: string): Document;\\n    /** @deprecated */\\n    hasFeature(...args: any[]): true;\\n}\\n\\ndeclare var DOMImplementation: {\\n    prototype: DOMImplementation;\\n    new(): DOMImplementation;\\n};\\n\\ninterface DOML2DeprecatedColorProperty {\\n    color: string;\\n}\\n\\ninterface DOMMatrix extends DOMMatrixReadOnly {\\n    a: number;\\n    b: number;\\n    c: number;\\n    d: number;\\n    e: number;\\n    f: number;\\n    m11: number;\\n    m12: number;\\n    m13: number;\\n    m14: number;\\n    m21: number;\\n    m22: number;\\n    m23: number;\\n    m24: number;\\n    m31: number;\\n    m32: number;\\n    m33: number;\\n    m34: number;\\n    m41: number;\\n    m42: number;\\n    m43: number;\\n    m44: number;\\n    invertSelf(): DOMMatrix;\\n    multiplySelf(other?: DOMMatrixInit): DOMMatrix;\\n    preMultiplySelf(other?: DOMMatrixInit): DOMMatrix;\\n    rotateAxisAngleSelf(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\\n    rotateFromVectorSelf(x?: number, y?: number): DOMMatrix;\\n    rotateSelf(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\\n    scale3dSelf(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    scaleSelf(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    setMatrixValue(transformList: string): DOMMatrix;\\n    skewXSelf(sx?: number): DOMMatrix;\\n    skewYSelf(sy?: number): DOMMatrix;\\n    translateSelf(tx?: number, ty?: number, tz?: number): DOMMatrix;\\n}\\n\\ndeclare var DOMMatrix: {\\n    prototype: DOMMatrix;\\n    new(init?: string | number[]): DOMMatrix;\\n    fromFloat32Array(array32: Float32Array): DOMMatrix;\\n    fromFloat64Array(array64: Float64Array): DOMMatrix;\\n    fromMatrix(other?: DOMMatrixInit): DOMMatrix;\\n};\\n\\ntype SVGMatrix = DOMMatrix;\\ndeclare var SVGMatrix: typeof DOMMatrix;\\n\\ntype WebKitCSSMatrix = DOMMatrix;\\ndeclare var WebKitCSSMatrix: typeof DOMMatrix;\\n\\ninterface DOMMatrixReadOnly {\\n    readonly a: number;\\n    readonly b: number;\\n    readonly c: number;\\n    readonly d: number;\\n    readonly e: number;\\n    readonly f: number;\\n    readonly is2D: boolean;\\n    readonly isIdentity: boolean;\\n    readonly m11: number;\\n    readonly m12: number;\\n    readonly m13: number;\\n    readonly m14: number;\\n    readonly m21: number;\\n    readonly m22: number;\\n    readonly m23: number;\\n    readonly m24: number;\\n    readonly m31: number;\\n    readonly m32: number;\\n    readonly m33: number;\\n    readonly m34: number;\\n    readonly m41: number;\\n    readonly m42: number;\\n    readonly m43: number;\\n    readonly m44: number;\\n    flipX(): DOMMatrix;\\n    flipY(): DOMMatrix;\\n    inverse(): DOMMatrix;\\n    multiply(other?: DOMMatrixInit): DOMMatrix;\\n    rotate(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\\n    rotateAxisAngle(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\\n    rotateFromVector(x?: number, y?: number): DOMMatrix;\\n    scale(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    scale3d(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    skewX(sx?: number): DOMMatrix;\\n    skewY(sy?: number): DOMMatrix;\\n    toFloat32Array(): Float32Array;\\n    toFloat64Array(): Float64Array;\\n    toJSON(): any;\\n    transformPoint(point?: DOMPointInit): DOMPoint;\\n    translate(tx?: number, ty?: number, tz?: number): DOMMatrix;\\n}\\n\\ndeclare var DOMMatrixReadOnly: {\\n    prototype: DOMMatrixReadOnly;\\n    new(init?: string | number[]): DOMMatrixReadOnly;\\n    fromFloat32Array(array32: Float32Array): DOMMatrixReadOnly;\\n    fromFloat64Array(array64: Float64Array): DOMMatrixReadOnly;\\n    fromMatrix(other?: DOMMatrixInit): DOMMatrixReadOnly;\\n};\\n\\ninterface DOMParser {\\n    parseFromString(str: string, type: SupportedType): Document;\\n}\\n\\ndeclare var DOMParser: {\\n    prototype: DOMParser;\\n    new(): DOMParser;\\n};\\n\\ninterface DOMPoint extends DOMPointReadOnly {\\n    w: number;\\n    x: number;\\n    y: number;\\n    z: number;\\n}\\n\\ndeclare var DOMPoint: {\\n    prototype: DOMPoint;\\n    new(x?: number, y?: number, z?: number, w?: number): DOMPoint;\\n    fromPoint(other?: DOMPointInit): DOMPoint;\\n};\\n\\ntype SVGPoint = DOMPoint;\\ndeclare var SVGPoint: typeof DOMPoint;\\n\\ninterface DOMPointReadOnly {\\n    readonly w: number;\\n    readonly x: number;\\n    readonly y: number;\\n    readonly z: number;\\n    matrixTransform(matrix?: DOMMatrixInit): DOMPoint;\\n    toJSON(): any;\\n}\\n\\ndeclare var DOMPointReadOnly: {\\n    prototype: DOMPointReadOnly;\\n    new(x?: number, y?: number, z?: number, w?: number): DOMPointReadOnly;\\n    fromPoint(other?: DOMPointInit): DOMPointReadOnly;\\n};\\n\\ninterface DOMQuad {\\n    readonly p1: DOMPoint;\\n    readonly p2: DOMPoint;\\n    readonly p3: DOMPoint;\\n    readonly p4: DOMPoint;\\n    getBounds(): DOMRect;\\n    toJSON(): any;\\n}\\n\\ndeclare var DOMQuad: {\\n    prototype: DOMQuad;\\n    new(p1?: DOMPointInit, p2?: DOMPointInit, p3?: DOMPointInit, p4?: DOMPointInit): DOMQuad;\\n    fromQuad(other?: DOMQuadInit): DOMQuad;\\n    fromRect(other?: DOMRectInit): DOMQuad;\\n};\\n\\ninterface DOMRect extends DOMRectReadOnly {\\n    height: number;\\n    width: number;\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var DOMRect: {\\n    prototype: DOMRect;\\n    new(x?: number, y?: number, width?: number, height?: number): DOMRect;\\n    fromRect(other?: DOMRectInit): DOMRect;\\n};\\n\\ntype SVGRect = DOMRect;\\ndeclare var SVGRect: typeof DOMRect;\\n\\ninterface DOMRectList {\\n    readonly length: number;\\n    item(index: number): DOMRect | null;\\n    [index: number]: DOMRect;\\n}\\n\\ndeclare var DOMRectList: {\\n    prototype: DOMRectList;\\n    new(): DOMRectList;\\n};\\n\\ninterface DOMRectReadOnly {\\n    readonly bottom: number;\\n    readonly height: number;\\n    readonly left: number;\\n    readonly right: number;\\n    readonly top: number;\\n    readonly width: number;\\n    readonly x: number;\\n    readonly y: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var DOMRectReadOnly: {\\n    prototype: DOMRectReadOnly;\\n    new(x?: number, y?: number, width?: number, height?: number): DOMRectReadOnly;\\n    fromRect(other?: DOMRectInit): DOMRectReadOnly;\\n};\\n\\ninterface DOMSettableTokenList extends DOMTokenList {\\n    value: string;\\n}\\n\\ndeclare var DOMSettableTokenList: {\\n    prototype: DOMSettableTokenList;\\n    new(): DOMSettableTokenList;\\n};\\n\\ninterface DOMStringList {\\n    /**\\n     * Returns the number of strings in strings.\\n     */\\n    readonly length: number;\\n    /**\\n     * Returns true if strings contains string, and false\\n     * otherwise.\\n     */\\n    contains(string: string): boolean;\\n    /**\\n     * Returns the string with index index from strings.\\n     */\\n    item(index: number): string | null;\\n    [index: number]: string;\\n}\\n\\ndeclare var DOMStringList: {\\n    prototype: DOMStringList;\\n    new(): DOMStringList;\\n};\\n\\ninterface DOMStringMap {\\n    [name: string]: string | undefined;\\n}\\n\\ndeclare var DOMStringMap: {\\n    prototype: DOMStringMap;\\n    new(): DOMStringMap;\\n};\\n\\ninterface DOMTokenList {\\n    /**\\n     * Returns the number of tokens.\\n     */\\n    readonly length: number;\\n    /**\\n     * Returns the associated set as string.\\n     * Can be set, to change the associated attribute.\\n     */\\n    value: string;\\n    /**\\n     * Adds all arguments passed, except those already present.\\n     * Throws a \"SyntaxError\" DOMException if one of the arguments is the empty\\n     * string.\\n     * Throws an \"InvalidCharacterError\" DOMException if one of the arguments\\n     * contains any ASCII whitespace.\\n     */\\n    add(...tokens: string[]): void;\\n    /**\\n     * Returns true if token is present, and false otherwise.\\n     */\\n    contains(token: string): boolean;\\n    /**\\n     * tokenlist[index]\\n     */\\n    item(index: number): string | null;\\n    /**\\n     * Removes arguments passed, if they are present.\\n     * Throws a \"SyntaxError\" DOMException if one of the arguments is the empty\\n     * string.\\n     * Throws an \"InvalidCharacterError\" DOMException if one of the arguments\\n     * contains any ASCII whitespace.\\n     */\\n    remove(...tokens: string[]): void;\\n    /**\\n     * Replaces token with newToken.\\n     * Returns true if token was replaced with newToken, and false otherwise.\\n     * Throws a \"SyntaxError\" DOMException if one of the arguments is the empty\\n     * string.\\n     * Throws an \"InvalidCharacterError\" DOMException if one of the arguments\\n     * contains any ASCII whitespace.\\n     */\\n    replace(oldToken: string, newToken: string): void;\\n    /**\\n     * Returns true if token is in the associated attribute\\'s supported tokens. Returns\\n     * false otherwise.\\n     * Throws a TypeError if the associated attribute has no supported tokens defined.\\n     */\\n    supports(token: string): boolean;\\n    toggle(token: string, force?: boolean): boolean;\\n    forEach(callbackfn: (value: string, key: number, parent: DOMTokenList) => void, thisArg?: any): void;\\n    [index: number]: string;\\n}\\n\\ndeclare var DOMTokenList: {\\n    prototype: DOMTokenList;\\n    new(): DOMTokenList;\\n};\\n\\ninterface DataCue extends TextTrackCue {\\n    data: ArrayBuffer;\\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: DataCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: DataCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var DataCue: {\\n    prototype: DataCue;\\n    new(): DataCue;\\n};\\n\\ninterface DataTransfer {\\n    dropEffect: string;\\n    effectAllowed: string;\\n    /**\\n     * Returns a FileList of the files being dragged, if any.\\n     */\\n    readonly files: FileList;\\n    /**\\n     * Returns a DataTransferItemList object, with the drag data.\\n     */\\n    readonly items: DataTransferItemList;\\n    /**\\n     * Returns a frozen array listing the formats that were set in the dragstart event. In addition, if any files are being\\n     * dragged, then one of the types will be the string \"Files\".\\n     */\\n    readonly types: ReadonlyArray<string>;\\n    /**\\n     * Removes the data of the specified formats. Removes all data if the argument is omitted.\\n     */\\n    clearData(format?: string): void;\\n    /**\\n     * Returns the specified data. If there is no such data, returns the empty string.\\n     */\\n    getData(format: string): string;\\n    /**\\n     * Adds the specified data.\\n     */\\n    setData(format: string, data: string): void;\\n    /**\\n     * Uses the given element to update the drag feedback, replacing any previously specified\\n     * feedback.\\n     */\\n    setDragImage(image: Element, x: number, y: number): void;\\n}\\n\\ndeclare var DataTransfer: {\\n    prototype: DataTransfer;\\n    new(): DataTransfer;\\n};\\n\\ninterface DataTransferItem {\\n    /**\\n     * Returns the drag data item kind, one of: \"string\",\\n     * \"file\".\\n     */\\n    readonly kind: string;\\n    /**\\n     * Returns the drag data item type string.\\n     */\\n    readonly type: string;\\n    /**\\n     * Returns a File object, if the drag data item kind is File.\\n     */\\n    getAsFile(): File | null;\\n    /**\\n     * Invokes the callback with the string data as the argument, if the drag data item\\n     * kind is Plain Unicode string.\\n     */\\n    getAsString(callback: FunctionStringCallback | null): void;\\n    webkitGetAsEntry(): any;\\n}\\n\\ndeclare var DataTransferItem: {\\n    prototype: DataTransferItem;\\n    new(): DataTransferItem;\\n};\\n\\ninterface DataTransferItemList {\\n    /**\\n     * Returns the number of items in the drag data store.\\n     */\\n    readonly length: number;\\n    /**\\n     * Adds a new entry for the given data to the drag data store. If the data is plain\\n     * text  then a type string has to be provided\\n     * also.\\n     */\\n    add(data: string, type: string): DataTransferItem | null;\\n    add(data: File): DataTransferItem | null;\\n    /**\\n     * Removes all the entries in the drag data store.\\n     */\\n    clear(): void;\\n    item(index: number): DataTransferItem;\\n    /**\\n     * Removes the indexth entry in the drag data store.\\n     */\\n    remove(index: number): void;\\n    [name: number]: DataTransferItem;\\n}\\n\\ndeclare var DataTransferItemList: {\\n    prototype: DataTransferItemList;\\n    new(): DataTransferItemList;\\n};\\n\\ninterface DeferredPermissionRequest {\\n    readonly id: number;\\n    readonly type: MSWebViewPermissionType;\\n    readonly uri: string;\\n    allow(): void;\\n    deny(): void;\\n}\\n\\ndeclare var DeferredPermissionRequest: {\\n    prototype: DeferredPermissionRequest;\\n    new(): DeferredPermissionRequest;\\n};\\n\\ninterface DelayNode extends AudioNode {\\n    readonly delayTime: AudioParam;\\n}\\n\\ndeclare var DelayNode: {\\n    prototype: DelayNode;\\n    new(context: BaseAudioContext, options?: DelayOptions): DelayNode;\\n};\\n\\ninterface DeviceAcceleration {\\n    readonly x: number | null;\\n    readonly y: number | null;\\n    readonly z: number | null;\\n}\\n\\ndeclare var DeviceAcceleration: {\\n    prototype: DeviceAcceleration;\\n    new(): DeviceAcceleration;\\n};\\n\\ninterface DeviceLightEvent extends Event {\\n    readonly value: number;\\n}\\n\\ndeclare var DeviceLightEvent: {\\n    prototype: DeviceLightEvent;\\n    new(typeArg: string, eventInitDict?: DeviceLightEventInit): DeviceLightEvent;\\n};\\n\\ninterface DeviceMotionEvent extends Event {\\n    readonly acceleration: DeviceAcceleration | null;\\n    readonly accelerationIncludingGravity: DeviceAcceleration | null;\\n    readonly interval: number | null;\\n    readonly rotationRate: DeviceRotationRate | null;\\n    initDeviceMotionEvent(type: string, bubbles: boolean, cancelable: boolean, acceleration: DeviceAccelerationDict | null, accelerationIncludingGravity: DeviceAccelerationDict | null, rotationRate: DeviceRotationRateDict | null, interval: number | null): void;\\n}\\n\\ndeclare var DeviceMotionEvent: {\\n    prototype: DeviceMotionEvent;\\n    new(typeArg: string, eventInitDict?: DeviceMotionEventInit): DeviceMotionEvent;\\n};\\n\\ninterface DeviceOrientationEvent extends Event {\\n    readonly absolute: boolean;\\n    readonly alpha: number | null;\\n    readonly beta: number | null;\\n    readonly gamma: number | null;\\n    initDeviceOrientationEvent(type: string, bubbles: boolean, cancelable: boolean, alpha: number | null, beta: number | null, gamma: number | null, absolute: boolean): void;\\n}\\n\\ndeclare var DeviceOrientationEvent: {\\n    prototype: DeviceOrientationEvent;\\n    new(typeArg: string, eventInitDict?: DeviceOrientationEventInit): DeviceOrientationEvent;\\n};\\n\\ninterface DeviceRotationRate {\\n    readonly alpha: number | null;\\n    readonly beta: number | null;\\n    readonly gamma: number | null;\\n}\\n\\ndeclare var DeviceRotationRate: {\\n    prototype: DeviceRotationRate;\\n    new(): DeviceRotationRate;\\n};\\n\\ninterface DhImportKeyParams extends Algorithm {\\n    generator: Uint8Array;\\n    prime: Uint8Array;\\n}\\n\\ninterface DhKeyAlgorithm extends KeyAlgorithm {\\n    generator: Uint8Array;\\n    prime: Uint8Array;\\n}\\n\\ninterface DhKeyDeriveParams extends Algorithm {\\n    public: CryptoKey;\\n}\\n\\ninterface DhKeyGenParams extends Algorithm {\\n    generator: Uint8Array;\\n    prime: Uint8Array;\\n}\\n\\ninterface DocumentEventMap extends GlobalEventHandlersEventMap, DocumentAndElementEventHandlersEventMap {\\n    \"fullscreenchange\": Event;\\n    \"fullscreenerror\": Event;\\n    \"readystatechange\": ProgressEvent;\\n    \"visibilitychange\": Event;\\n}\\n\\ninterface Document extends Node, NonElementParentNode, DocumentOrShadowRoot, ParentNode, GlobalEventHandlers, DocumentAndElementEventHandlers {\\n    /**\\n     * Sets or gets the URL for the current document.\\n     */\\n    readonly URL: string;\\n    /**\\n     * Gets the object that has the focus when the parent document has focus.\\n     */\\n    readonly activeElement: Element | null;\\n    /**\\n     * Sets or gets the color of all active links in the document.\\n     */\\n    /** @deprecated */\\n    alinkColor: string;\\n    /**\\n     * Returns a reference to the collection of elements contained by the object.\\n     */\\n    /** @deprecated */\\n    readonly all: HTMLAllCollection;\\n    /**\\n     * Retrieves a collection of all a objects that have a name and/or id property. Objects in this collection are in HTML source order.\\n     */\\n    /** @deprecated */\\n    readonly anchors: HTMLCollectionOf<HTMLAnchorElement>;\\n    /**\\n     * Retrieves a collection of all applet objects in the document.\\n     */\\n    /** @deprecated */\\n    readonly applets: HTMLCollectionOf<HTMLAppletElement>;\\n    /**\\n     * Deprecated. Sets or retrieves a value that indicates the background color behind the object.\\n     */\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Specifies the beginning and end of the document body.\\n     */\\n    body: HTMLElement;\\n    /**\\n     * Returns document\\'s encoding.\\n     */\\n    readonly characterSet: string;\\n    /**\\n     * Gets or sets the character set used to encode the object.\\n     */\\n    readonly charset: string;\\n    /**\\n     * Gets a value that indicates whether standards-compliant mode is switched on for the object.\\n     */\\n    readonly compatMode: string;\\n    /**\\n     * Returns document\\'s content type.\\n     */\\n    readonly contentType: string;\\n    /**\\n     * Returns the HTTP cookies that apply to the Document. If there are no cookies or\\n     * cookies can\\'t be applied to this resource, the empty string will be returned.\\n     * Can be set, to add a new cookie to the element\\'s set of HTTP cookies.\\n     * If the contents are sandboxed into a\\n     * unique origin (e.g. in an iframe with the sandbox attribute), a\\n     * \"SecurityError\" DOMException will be thrown on getting\\n     * and setting.\\n     */\\n    cookie: string;\\n    /**\\n     * Returns the script element, or the SVG script element,\\n     * that is currently executing, as long as the element represents a classic script.\\n     * In the case of reentrant script execution, returns the one that most recently started executing\\n     * amongst those that have not yet finished executing.\\n     * Returns null if the Document is not currently executing a script\\n     * or SVG script element (e.g., because the running script is an event\\n     * handler, or a timeout), or if the currently executing script or SVG\\n     * script element represents a module script.\\n     */\\n    readonly currentScript: HTMLOrSVGScriptElement | null;\\n    readonly defaultView: WindowProxy | null;\\n    /**\\n     * Sets or gets a value that indicates whether the document can be edited.\\n     */\\n    designMode: string;\\n    /**\\n     * Sets or retrieves a value that indicates the reading order of the object.\\n     */\\n    dir: string;\\n    /**\\n     * Gets an object representing the document type declaration associated with the current document.\\n     */\\n    readonly doctype: DocumentType | null;\\n    /**\\n     * Gets a reference to the root node of the document.\\n     */\\n    readonly documentElement: HTMLElement | null;\\n    /**\\n     * Returns document\\'s URL.\\n     */\\n    readonly documentURI: string;\\n    /**\\n     * Sets or gets the security domain of the document.\\n     */\\n    domain: string;\\n    /**\\n     * Retrieves a collection of all embed objects in the document.\\n     */\\n    readonly embeds: HTMLCollectionOf<HTMLEmbedElement>;\\n    /**\\n     * Sets or gets the foreground (text) color of the document.\\n     */\\n    /** @deprecated */\\n    fgColor: string;\\n    /**\\n     * Retrieves a collection, in source order, of all form objects in the document.\\n     */\\n    readonly forms: HTMLCollectionOf<HTMLFormElement>;\\n    /** @deprecated */\\n    readonly fullscreen: boolean;\\n    /**\\n     * Returns true if document has the ability to display elements fullscreen\\n     * and fullscreen is supported, or false otherwise.\\n     */\\n    readonly fullscreenEnabled: boolean;\\n    /**\\n     * Returns the head element.\\n     */\\n    readonly head: HTMLHeadElement | null;\\n    readonly hidden: boolean;\\n    /**\\n     * Retrieves a collection, in source order, of img objects in the document.\\n     */\\n    readonly images: HTMLCollectionOf<HTMLImageElement>;\\n    /**\\n     * Gets the implementation object of the current document.\\n     */\\n    readonly implementation: DOMImplementation;\\n    /**\\n     * Returns the character encoding used to create the webpage that is loaded into the document object.\\n     */\\n    readonly inputEncoding: string;\\n    /**\\n     * Gets the date that the page was last modified, if the page supplies one.\\n     */\\n    readonly lastModified: string;\\n    /**\\n     * Sets or gets the color of the document links.\\n     */\\n    /** @deprecated */\\n    linkColor: string;\\n    /**\\n     * Retrieves a collection of all a objects that specify the href property and all area objects in the document.\\n     */\\n    readonly links: HTMLCollectionOf<HTMLAnchorElement | HTMLAreaElement>;\\n    /**\\n     * Contains information about the current URL.\\n     */\\n    location: Location | null;\\n    onfullscreenchange: ((this: Document, ev: Event) => any) | null;\\n    onfullscreenerror: ((this: Document, ev: Event) => any) | null;\\n    /**\\n     * Fires when the state of the object has changed.\\n     * @param ev The event\\n     */\\n    onreadystatechange: ((this: Document, ev: ProgressEvent) => any) | null;\\n    onvisibilitychange: ((this: Document, ev: Event) => any) | null;\\n    /**\\n     * Returns document\\'s origin.\\n     */\\n    readonly origin: string;\\n    /**\\n     * Return an HTMLCollection of the embed elements in the Document.\\n     */\\n    readonly plugins: HTMLCollectionOf<HTMLEmbedElement>;\\n    /**\\n     * Retrieves a value that indicates the current state of the object.\\n     */\\n    readonly readyState: DocumentReadyState;\\n    /**\\n     * Gets the URL of the location that referred the user to the current page.\\n     */\\n    readonly referrer: string;\\n    /**\\n     * Retrieves a collection of all script objects in the document.\\n     */\\n    readonly scripts: HTMLCollectionOf<HTMLScriptElement>;\\n    readonly scrollingElement: Element | null;\\n    readonly timeline: DocumentTimeline;\\n    /**\\n     * Contains the title of the document.\\n     */\\n    title: string;\\n    readonly visibilityState: VisibilityState;\\n    /**\\n     * Sets or gets the color of the links that the user has visited.\\n     */\\n    /** @deprecated */\\n    vlinkColor: string;\\n    /**\\n     * Moves node from another document and returns it.\\n     * If node is a document, throws a \"NotSupportedError\" DOMException or, if node is a shadow root, throws a\\n     * \"HierarchyRequestError\" DOMException.\\n     */\\n    adoptNode<T extends Node>(source: T): T;\\n    /** @deprecated */\\n    captureEvents(): void;\\n    caretPositionFromPoint(x: number, y: number): CaretPosition | null;\\n    caretRangeFromPoint(x: number, y: number): Range;\\n    /** @deprecated */\\n    clear(): void;\\n    /**\\n     * Closes an output stream and forces the sent data to display.\\n     */\\n    close(): void;\\n    /**\\n     * Creates an attribute object with a specified name.\\n     * @param name String that sets the attribute object\\'s name.\\n     */\\n    createAttribute(localName: string): Attr;\\n    createAttributeNS(namespace: string | null, qualifiedName: string): Attr;\\n    /**\\n     * Returns a CDATASection node whose data is data.\\n     */\\n    createCDATASection(data: string): CDATASection;\\n    /**\\n     * Creates a comment object with the specified data.\\n     * @param data Sets the comment object\\'s data.\\n     */\\n    createComment(data: string): Comment;\\n    /**\\n     * Creates a new document.\\n     */\\n    createDocumentFragment(): DocumentFragment;\\n    /**\\n     * Creates an instance of the element for the specified tag.\\n     * @param tagName The name of an element.\\n     */\\n    createElement<K extends keyof HTMLElementTagNameMap>(tagName: K, options?: ElementCreationOptions): HTMLElementTagNameMap[K];\\n    /** @deprecated */\\n    createElement<K extends keyof HTMLElementDeprecatedTagNameMap>(tagName: K, options?: ElementCreationOptions): HTMLElementDeprecatedTagNameMap[K];\\n    createElement(tagName: string, options?: ElementCreationOptions): HTMLElement;\\n    /**\\n     * Returns an element with namespace namespace. Its namespace prefix will be everything before \":\" (U+003E) in qualifiedName or null. Its local name will be everything after\\n     * \":\" (U+003E) in qualifiedName or qualifiedName.\\n     * If localName does not match the Name production an\\n     * \"InvalidCharacterError\" DOMException will be thrown.\\n     * If one of the following conditions is true a \"NamespaceError\" DOMException will be thrown:\\n     * localName does not match the QName production.\\n     * Namespace prefix is not null and namespace is the empty string.\\n     * Namespace prefix is \"xml\" and namespace is not the XML namespace.\\n     * qualifiedName or namespace prefix is \"xmlns\" and namespace is not the XMLNS namespace.\\n     * namespace is the XMLNS namespace and\\n     * neither qualifiedName nor namespace prefix is \"xmlns\".\\n     * When supplied, options\\'s is can be used to create a customized built-in element.\\n     */\\n    createElementNS(namespaceURI: \"http://www.w3.org/1999/xhtml\", qualifiedName: string): HTMLElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"a\"): SVGAElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"circle\"): SVGCircleElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"clipPath\"): SVGClipPathElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"componentTransferFunction\"): SVGComponentTransferFunctionElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"defs\"): SVGDefsElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"desc\"): SVGDescElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"ellipse\"): SVGEllipseElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feBlend\"): SVGFEBlendElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feColorMatrix\"): SVGFEColorMatrixElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feComponentTransfer\"): SVGFEComponentTransferElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feComposite\"): SVGFECompositeElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feConvolveMatrix\"): SVGFEConvolveMatrixElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feDiffuseLighting\"): SVGFEDiffuseLightingElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feDisplacementMap\"): SVGFEDisplacementMapElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feDistantLight\"): SVGFEDistantLightElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFlood\"): SVGFEFloodElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncA\"): SVGFEFuncAElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncB\"): SVGFEFuncBElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncG\"): SVGFEFuncGElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncR\"): SVGFEFuncRElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feGaussianBlur\"): SVGFEGaussianBlurElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feImage\"): SVGFEImageElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feMerge\"): SVGFEMergeElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feMergeNode\"): SVGFEMergeNodeElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feMorphology\"): SVGFEMorphologyElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feOffset\"): SVGFEOffsetElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"fePointLight\"): SVGFEPointLightElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feSpecularLighting\"): SVGFESpecularLightingElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feSpotLight\"): SVGFESpotLightElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feTile\"): SVGFETileElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feTurbulence\"): SVGFETurbulenceElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"filter\"): SVGFilterElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"foreignObject\"): SVGForeignObjectElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"g\"): SVGGElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"image\"): SVGImageElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"gradient\"): SVGGradientElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"line\"): SVGLineElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"linearGradient\"): SVGLinearGradientElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"marker\"): SVGMarkerElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"mask\"): SVGMaskElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"path\"): SVGPathElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"metadata\"): SVGMetadataElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"pattern\"): SVGPatternElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"polygon\"): SVGPolygonElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"polyline\"): SVGPolylineElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"radialGradient\"): SVGRadialGradientElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"rect\"): SVGRectElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"svg\"): SVGSVGElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"script\"): SVGScriptElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"stop\"): SVGStopElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"style\"): SVGStyleElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"switch\"): SVGSwitchElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"symbol\"): SVGSymbolElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"tspan\"): SVGTSpanElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"textContent\"): SVGTextContentElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"text\"): SVGTextElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"textPath\"): SVGTextPathElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"textPositioning\"): SVGTextPositioningElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"title\"): SVGTitleElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"use\"): SVGUseElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"view\"): SVGViewElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: string): SVGElement;\\n    createElementNS(namespaceURI: string | null, qualifiedName: string, options?: ElementCreationOptions): Element;\\n    createElementNS(namespace: string | null, qualifiedName: string, options?: string | ElementCreationOptions): Element;\\n    createEvent(eventInterface: \"AnimationEvent\"): AnimationEvent;\\n    createEvent(eventInterface: \"AnimationPlaybackEvent\"): AnimationPlaybackEvent;\\n    createEvent(eventInterface: \"AudioProcessingEvent\"): AudioProcessingEvent;\\n    createEvent(eventInterface: \"BeforeUnloadEvent\"): BeforeUnloadEvent;\\n    createEvent(eventInterface: \"ClipboardEvent\"): ClipboardEvent;\\n    createEvent(eventInterface: \"CloseEvent\"): CloseEvent;\\n    createEvent(eventInterface: \"CompositionEvent\"): CompositionEvent;\\n    createEvent(eventInterface: \"CustomEvent\"): CustomEvent;\\n    createEvent(eventInterface: \"DeviceLightEvent\"): DeviceLightEvent;\\n    createEvent(eventInterface: \"DeviceMotionEvent\"): DeviceMotionEvent;\\n    createEvent(eventInterface: \"DeviceOrientationEvent\"): DeviceOrientationEvent;\\n    createEvent(eventInterface: \"DragEvent\"): DragEvent;\\n    createEvent(eventInterface: \"ErrorEvent\"): ErrorEvent;\\n    createEvent(eventInterface: \"Event\"): Event;\\n    createEvent(eventInterface: \"Events\"): Event;\\n    createEvent(eventInterface: \"FocusEvent\"): FocusEvent;\\n    createEvent(eventInterface: \"FocusNavigationEvent\"): FocusNavigationEvent;\\n    createEvent(eventInterface: \"GamepadEvent\"): GamepadEvent;\\n    createEvent(eventInterface: \"HashChangeEvent\"): HashChangeEvent;\\n    createEvent(eventInterface: \"IDBVersionChangeEvent\"): IDBVersionChangeEvent;\\n    createEvent(eventInterface: \"KeyboardEvent\"): KeyboardEvent;\\n    createEvent(eventInterface: \"ListeningStateChangedEvent\"): ListeningStateChangedEvent;\\n    createEvent(eventInterface: \"MSGestureEvent\"): MSGestureEvent;\\n    createEvent(eventInterface: \"MSMediaKeyMessageEvent\"): MSMediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MSMediaKeyNeededEvent\"): MSMediaKeyNeededEvent;\\n    createEvent(eventInterface: \"MSPointerEvent\"): MSPointerEvent;\\n    createEvent(eventInterface: \"MediaEncryptedEvent\"): MediaEncryptedEvent;\\n    createEvent(eventInterface: \"MediaKeyMessageEvent\"): MediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MediaQueryListEvent\"): MediaQueryListEvent;\\n    createEvent(eventInterface: \"MediaStreamErrorEvent\"): MediaStreamErrorEvent;\\n    createEvent(eventInterface: \"MediaStreamEvent\"): MediaStreamEvent;\\n    createEvent(eventInterface: \"MediaStreamTrackEvent\"): MediaStreamTrackEvent;\\n    createEvent(eventInterface: \"MessageEvent\"): MessageEvent;\\n    createEvent(eventInterface: \"MouseEvent\"): MouseEvent;\\n    createEvent(eventInterface: \"MouseEvents\"): MouseEvent;\\n    createEvent(eventInterface: \"MutationEvent\"): MutationEvent;\\n    createEvent(eventInterface: \"MutationEvents\"): MutationEvent;\\n    createEvent(eventInterface: \"OfflineAudioCompletionEvent\"): OfflineAudioCompletionEvent;\\n    createEvent(eventInterface: \"OverflowEvent\"): OverflowEvent;\\n    createEvent(eventInterface: \"PageTransitionEvent\"): PageTransitionEvent;\\n    createEvent(eventInterface: \"PaymentRequestUpdateEvent\"): PaymentRequestUpdateEvent;\\n    createEvent(eventInterface: \"PermissionRequestedEvent\"): PermissionRequestedEvent;\\n    createEvent(eventInterface: \"PointerEvent\"): PointerEvent;\\n    createEvent(eventInterface: \"PopStateEvent\"): PopStateEvent;\\n    createEvent(eventInterface: \"ProgressEvent\"): ProgressEvent;\\n    createEvent(eventInterface: \"PromiseRejectionEvent\"): PromiseRejectionEvent;\\n    createEvent(eventInterface: \"RTCDTMFToneChangeEvent\"): RTCDTMFToneChangeEvent;\\n    createEvent(eventInterface: \"RTCDataChannelEvent\"): RTCDataChannelEvent;\\n    createEvent(eventInterface: \"RTCDtlsTransportStateChangedEvent\"): RTCDtlsTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCErrorEvent\"): RTCErrorEvent;\\n    createEvent(eventInterface: \"RTCIceCandidatePairChangedEvent\"): RTCIceCandidatePairChangedEvent;\\n    createEvent(eventInterface: \"RTCIceGathererEvent\"): RTCIceGathererEvent;\\n    createEvent(eventInterface: \"RTCIceTransportStateChangedEvent\"): RTCIceTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceErrorEvent\"): RTCPeerConnectionIceErrorEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceEvent\"): RTCPeerConnectionIceEvent;\\n    createEvent(eventInterface: \"RTCSsrcConflictEvent\"): RTCSsrcConflictEvent;\\n    createEvent(eventInterface: \"RTCStatsEvent\"): RTCStatsEvent;\\n    createEvent(eventInterface: \"RTCTrackEvent\"): RTCTrackEvent;\\n    createEvent(eventInterface: \"SVGZoomEvent\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SVGZoomEvents\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SecurityPolicyViolationEvent\"): SecurityPolicyViolationEvent;\\n    createEvent(eventInterface: \"ServiceWorkerMessageEvent\"): ServiceWorkerMessageEvent;\\n    createEvent(eventInterface: \"SpeechRecognitionError\"): SpeechRecognitionError;\\n    createEvent(eventInterface: \"SpeechRecognitionEvent\"): SpeechRecognitionEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisErrorEvent\"): SpeechSynthesisErrorEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisEvent\"): SpeechSynthesisEvent;\\n    createEvent(eventInterface: \"StorageEvent\"): StorageEvent;\\n    createEvent(eventInterface: \"TextEvent\"): TextEvent;\\n    createEvent(eventInterface: \"TouchEvent\"): TouchEvent;\\n    createEvent(eventInterface: \"TrackEvent\"): TrackEvent;\\n    createEvent(eventInterface: \"TransitionEvent\"): TransitionEvent;\\n    createEvent(eventInterface: \"UIEvent\"): UIEvent;\\n    createEvent(eventInterface: \"UIEvents\"): UIEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent\"): VRDisplayEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent \"): VRDisplayEvent ;\\n    createEvent(eventInterface: \"WebGLContextEvent\"): WebGLContextEvent;\\n    createEvent(eventInterface: \"WheelEvent\"): WheelEvent;\\n    createEvent(eventInterface: string): Event;\\n    /**\\n     * Creates a NodeIterator object that you can use to traverse filtered lists of nodes or elements in a document.\\n     * @param root The root element or node to start traversing on.\\n     * @param whatToShow The type of nodes or elements to appear in the node list\\n     * @param filter A custom NodeFilter function to use. For more information, see filter. Use null for no filter.\\n     * @param entityReferenceExpansion A flag that specifies whether entity reference nodes are expanded.\\n     */\\n    createNodeIterator(root: Node, whatToShow?: number, filter?: NodeFilter | null): NodeIterator;\\n    /**\\n     * Returns a ProcessingInstruction node whose target is target and data is data.\\n     * If target does not match the Name production an\\n     * \"InvalidCharacterError\" DOMException will be thrown.\\n     * If data contains \"?>\" an\\n     * \"InvalidCharacterError\" DOMException will be thrown.\\n     */\\n    createProcessingInstruction(target: string, data: string): ProcessingInstruction;\\n    /**\\n     *  Returns an empty range object that has both of its boundary points positioned at the beginning of the document.\\n     */\\n    createRange(): Range;\\n    /**\\n     * Creates a text string from the specified value.\\n     * @param data String that specifies the nodeValue property of the text node.\\n     */\\n    createTextNode(data: string): Text;\\n    createTouch(view: WindowProxy, target: EventTarget, identifier: number, pageX: number, pageY: number, screenX: number, screenY: number): Touch;\\n    createTouchList(...touches: Touch[]): TouchList;\\n    /**\\n     * Creates a TreeWalker object that you can use to traverse filtered lists of nodes or elements in a document.\\n     * @param root The root element or node to start traversing on.\\n     * @param whatToShow The type of nodes or elements to appear in the node list. For more information, see whatToShow.\\n     * @param filter A custom NodeFilter function to use.\\n     * @param entityReferenceExpansion A flag that specifies whether entity reference nodes are expanded.\\n     */\\n    createTreeWalker(root: Node, whatToShow?: number, filter?: NodeFilter | null, entityReferenceExpansion?: boolean): TreeWalker;\\n    /**\\n     * Returns the element for the specified x coordinate and the specified y coordinate.\\n     * @param x The x-offset\\n     * @param y The y-offset\\n     */\\n    elementFromPoint(x: number, y: number): Element | null;\\n    elementsFromPoint(x: number, y: number): Element[];\\n    evaluate(expression: string, contextNode: Node, resolver: XPathNSResolver | null, type: number, result: XPathResult | null): XPathResult;\\n    /**\\n     * Executes a command on the current document, current selection, or the given range.\\n     * @param commandId String that specifies the command to execute. This command can be any of the command identifiers that can be executed in script.\\n     * @param showUI Display the user interface, defaults to false.\\n     * @param value Value to assign.\\n     */\\n    execCommand(commandId: string, showUI?: boolean, value?: string): boolean;\\n    /**\\n     * Stops document\\'s fullscreen element from being displayed fullscreen and\\n     * resolves promise when done.\\n     */\\n    exitFullscreen(): Promise<void>;\\n    getAnimations(): Animation[];\\n    /**\\n     * Returns a reference to the first object with the specified value of the ID or NAME attribute.\\n     * @param elementId String that specifies the ID value. Case-insensitive.\\n     */\\n    getElementById(elementId: string): HTMLElement | null;\\n    /**\\n     * collection = element . getElementsByClassName(classNames)\\n     */\\n    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;\\n    /**\\n     * Gets a collection of objects based on the value of the NAME or ID attribute.\\n     * @param elementName Gets a collection of objects based on the value of the NAME or ID attribute.\\n     */\\n    getElementsByName(elementName: string): NodeListOf<HTMLElement>;\\n    /**\\n     * Retrieves a collection of objects based on the specified element name.\\n     * @param name Specifies the name of an element.\\n     */\\n    getElementsByTagName<K extends keyof HTMLElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<HTMLElementTagNameMap[K]>;\\n    getElementsByTagName<K extends keyof SVGElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<SVGElementTagNameMap[K]>;\\n    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;\\n    /**\\n     * If namespace and localName are\\n     * \"*\" returns a HTMLCollection of all descendant elements.\\n     * If only namespace is \"*\" returns a HTMLCollection of all descendant elements whose local name is localName.\\n     * If only localName is \"*\" returns a HTMLCollection of all descendant elements whose namespace is namespace.\\n     * Otherwise, returns a HTMLCollection of all descendant elements whose namespace is namespace and local name is localName.\\n     */\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/1999/xhtml\", localName: string): HTMLCollectionOf<HTMLElement>;\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/2000/svg\", localName: string): HTMLCollectionOf<SVGElement>;\\n    getElementsByTagNameNS(namespaceURI: string, localName: string): HTMLCollectionOf<Element>;\\n    /**\\n     * Gets a value indicating whether the object currently has focus.\\n     */\\n    hasFocus(): boolean;\\n    importNode<T extends Node>(importedNode: T, deep: boolean): T;\\n    /**\\n     * Opens a new window and loads a document specified by a given URL. Also, opens a new window that uses the url parameter and the name parameter to collect the output of the write method and the writeln method.\\n     * @param url Specifies a MIME type for the document.\\n     * @param name Specifies the name of the window. This name is used as the value for the TARGET attribute on a form or an anchor element.\\n     * @param features Contains a list of items separated by commas. Each item consists of an option and a value, separated by an equals sign (for example, \"fullscreen=yes, toolbar=yes\"). The following values are supported.\\n     * @param replace Specifies whether the existing entry for the document is replaced in the history list.\\n     */\\n    open(url?: string, name?: string, features?: string, replace?: boolean): Document;\\n    /**\\n     * Returns a Boolean value that indicates whether a specified command can be successfully executed using execCommand, given the current state of the document.\\n     * @param commandId Specifies a command identifier.\\n     */\\n    queryCommandEnabled(commandId: string): boolean;\\n    /**\\n     * Returns a Boolean value that indicates whether the specified command is in the indeterminate state.\\n     * @param commandId String that specifies a command identifier.\\n     */\\n    queryCommandIndeterm(commandId: string): boolean;\\n    /**\\n     * Returns a Boolean value that indicates the current state of the command.\\n     * @param commandId String that specifies a command identifier.\\n     */\\n    queryCommandState(commandId: string): boolean;\\n    /**\\n     * Returns a Boolean value that indicates whether the current command is supported on the current range.\\n     * @param commandId Specifies a command identifier.\\n     */\\n    queryCommandSupported(commandId: string): boolean;\\n    /**\\n     * Returns the current value of the document, range, or current selection for the given command.\\n     * @param commandId String that specifies a command identifier.\\n     */\\n    queryCommandValue(commandId: string): string;\\n    /** @deprecated */\\n    releaseEvents(): void;\\n    /**\\n     * Writes one or more HTML expressions to a document in the specified window.\\n     * @param content Specifies the text and HTML tags to write.\\n     */\\n    write(...text: string[]): void;\\n    /**\\n     * Writes one or more HTML expressions, followed by a carriage return, to a document in the specified window.\\n     * @param content The text and HTML tags to write.\\n     */\\n    writeln(...text: string[]): void;\\n    /**\\n     * Returns an object representing the current selection of the document that is loaded into the object displaying a webpage.\\n     */\\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: Document, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: Document, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Document: {\\n    prototype: Document;\\n    new(): Document;\\n};\\n\\ninterface DocumentAndElementEventHandlersEventMap {\\n    \"copy\": ClipboardEvent;\\n    \"cut\": ClipboardEvent;\\n    \"paste\": ClipboardEvent;\\n}\\n\\ninterface DocumentAndElementEventHandlers {\\n    oncopy: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\\n    oncut: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\\n    onpaste: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\\n    addEventListener<K extends keyof DocumentAndElementEventHandlersEventMap>(type: K, listener: (this: DocumentAndElementEventHandlers, ev: DocumentAndElementEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentAndElementEventHandlersEventMap>(type: K, listener: (this: DocumentAndElementEventHandlers, ev: DocumentAndElementEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface DocumentEvent {\\n    createEvent(eventInterface: \"AnimationEvent\"): AnimationEvent;\\n    createEvent(eventInterface: \"AnimationPlaybackEvent\"): AnimationPlaybackEvent;\\n    createEvent(eventInterface: \"AudioProcessingEvent\"): AudioProcessingEvent;\\n    createEvent(eventInterface: \"BeforeUnloadEvent\"): BeforeUnloadEvent;\\n    createEvent(eventInterface: \"ClipboardEvent\"): ClipboardEvent;\\n    createEvent(eventInterface: \"CloseEvent\"): CloseEvent;\\n    createEvent(eventInterface: \"CompositionEvent\"): CompositionEvent;\\n    createEvent(eventInterface: \"CustomEvent\"): CustomEvent;\\n    createEvent(eventInterface: \"DeviceLightEvent\"): DeviceLightEvent;\\n    createEvent(eventInterface: \"DeviceMotionEvent\"): DeviceMotionEvent;\\n    createEvent(eventInterface: \"DeviceOrientationEvent\"): DeviceOrientationEvent;\\n    createEvent(eventInterface: \"DragEvent\"): DragEvent;\\n    createEvent(eventInterface: \"ErrorEvent\"): ErrorEvent;\\n    createEvent(eventInterface: \"Event\"): Event;\\n    createEvent(eventInterface: \"Events\"): Event;\\n    createEvent(eventInterface: \"FocusEvent\"): FocusEvent;\\n    createEvent(eventInterface: \"FocusNavigationEvent\"): FocusNavigationEvent;\\n    createEvent(eventInterface: \"GamepadEvent\"): GamepadEvent;\\n    createEvent(eventInterface: \"HashChangeEvent\"): HashChangeEvent;\\n    createEvent(eventInterface: \"IDBVersionChangeEvent\"): IDBVersionChangeEvent;\\n    createEvent(eventInterface: \"KeyboardEvent\"): KeyboardEvent;\\n    createEvent(eventInterface: \"ListeningStateChangedEvent\"): ListeningStateChangedEvent;\\n    createEvent(eventInterface: \"MSGestureEvent\"): MSGestureEvent;\\n    createEvent(eventInterface: \"MSMediaKeyMessageEvent\"): MSMediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MSMediaKeyNeededEvent\"): MSMediaKeyNeededEvent;\\n    createEvent(eventInterface: \"MSPointerEvent\"): MSPointerEvent;\\n    createEvent(eventInterface: \"MediaEncryptedEvent\"): MediaEncryptedEvent;\\n    createEvent(eventInterface: \"MediaKeyMessageEvent\"): MediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MediaQueryListEvent\"): MediaQueryListEvent;\\n    createEvent(eventInterface: \"MediaStreamErrorEvent\"): MediaStreamErrorEvent;\\n    createEvent(eventInterface: \"MediaStreamEvent\"): MediaStreamEvent;\\n    createEvent(eventInterface: \"MediaStreamTrackEvent\"): MediaStreamTrackEvent;\\n    createEvent(eventInterface: \"MessageEvent\"): MessageEvent;\\n    createEvent(eventInterface: \"MouseEvent\"): MouseEvent;\\n    createEvent(eventInterface: \"MouseEvents\"): MouseEvent;\\n    createEvent(eventInterface: \"MutationEvent\"): MutationEvent;\\n    createEvent(eventInterface: \"MutationEvents\"): MutationEvent;\\n    createEvent(eventInterface: \"OfflineAudioCompletionEvent\"): OfflineAudioCompletionEvent;\\n    createEvent(eventInterface: \"OverflowEvent\"): OverflowEvent;\\n    createEvent(eventInterface: \"PageTransitionEvent\"): PageTransitionEvent;\\n    createEvent(eventInterface: \"PaymentRequestUpdateEvent\"): PaymentRequestUpdateEvent;\\n    createEvent(eventInterface: \"PermissionRequestedEvent\"): PermissionRequestedEvent;\\n    createEvent(eventInterface: \"PointerEvent\"): PointerEvent;\\n    createEvent(eventInterface: \"PopStateEvent\"): PopStateEvent;\\n    createEvent(eventInterface: \"ProgressEvent\"): ProgressEvent;\\n    createEvent(eventInterface: \"PromiseRejectionEvent\"): PromiseRejectionEvent;\\n    createEvent(eventInterface: \"RTCDTMFToneChangeEvent\"): RTCDTMFToneChangeEvent;\\n    createEvent(eventInterface: \"RTCDataChannelEvent\"): RTCDataChannelEvent;\\n    createEvent(eventInterface: \"RTCDtlsTransportStateChangedEvent\"): RTCDtlsTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCErrorEvent\"): RTCErrorEvent;\\n    createEvent(eventInterface: \"RTCIceCandidatePairChangedEvent\"): RTCIceCandidatePairChangedEvent;\\n    createEvent(eventInterface: \"RTCIceGathererEvent\"): RTCIceGathererEvent;\\n    createEvent(eventInterface: \"RTCIceTransportStateChangedEvent\"): RTCIceTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceErrorEvent\"): RTCPeerConnectionIceErrorEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceEvent\"): RTCPeerConnectionIceEvent;\\n    createEvent(eventInterface: \"RTCSsrcConflictEvent\"): RTCSsrcConflictEvent;\\n    createEvent(eventInterface: \"RTCStatsEvent\"): RTCStatsEvent;\\n    createEvent(eventInterface: \"RTCTrackEvent\"): RTCTrackEvent;\\n    createEvent(eventInterface: \"SVGZoomEvent\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SVGZoomEvents\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SecurityPolicyViolationEvent\"): SecurityPolicyViolationEvent;\\n    createEvent(eventInterface: \"ServiceWorkerMessageEvent\"): ServiceWorkerMessageEvent;\\n    createEvent(eventInterface: \"SpeechRecognitionError\"): SpeechRecognitionError;\\n    createEvent(eventInterface: \"SpeechRecognitionEvent\"): SpeechRecognitionEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisErrorEvent\"): SpeechSynthesisErrorEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisEvent\"): SpeechSynthesisEvent;\\n    createEvent(eventInterface: \"StorageEvent\"): StorageEvent;\\n    createEvent(eventInterface: \"TextEvent\"): TextEvent;\\n    createEvent(eventInterface: \"TouchEvent\"): TouchEvent;\\n    createEvent(eventInterface: \"TrackEvent\"): TrackEvent;\\n    createEvent(eventInterface: \"TransitionEvent\"): TransitionEvent;\\n    createEvent(eventInterface: \"UIEvent\"): UIEvent;\\n    createEvent(eventInterface: \"UIEvents\"): UIEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent\"): VRDisplayEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent \"): VRDisplayEvent ;\\n    createEvent(eventInterface: \"WebGLContextEvent\"): WebGLContextEvent;\\n    createEvent(eventInterface: \"WheelEvent\"): WheelEvent;\\n    createEvent(eventInterface: string): Event;\\n}\\n\\ninterface DocumentFragment extends Node, NonElementParentNode, ParentNode {\\n    getElementById(elementId: string): HTMLElement | null;\\n}\\n\\ndeclare var DocumentFragment: {\\n    prototype: DocumentFragment;\\n    new(): DocumentFragment;\\n};\\n\\ninterface DocumentOrShadowRoot {\\n    readonly activeElement: Element | null;\\n    /**\\n     * Retrieves a collection of styleSheet objects representing the style sheets that correspond to each instance of a link or style object in the document.\\n     */\\n    readonly styleSheets: StyleSheetList;\\n    elementFromPoint(x: number, y: number): Element | null;\\n    elementsFromPoint(x: number, y: number): Element[];\\n    getSelection(): Selection | null;\\n}\\n\\ninterface DocumentTimeline extends AnimationTimeline {\\n}\\n\\ndeclare var DocumentTimeline: {\\n    prototype: DocumentTimeline;\\n    new(options?: DocumentTimelineOptions): DocumentTimeline;\\n};\\n\\ninterface DocumentType extends Node, ChildNode {\\n    readonly name: string;\\n    readonly publicId: string;\\n    readonly systemId: string;\\n}\\n\\ndeclare var DocumentType: {\\n    prototype: DocumentType;\\n    new(): DocumentType;\\n};\\n\\ninterface DragEvent extends MouseEvent {\\n    /**\\n     * Returns the DataTransfer object for the event.\\n     */\\n    readonly dataTransfer: DataTransfer | null;\\n}\\n\\ndeclare var DragEvent: {\\n    prototype: DragEvent;\\n    new(type: string, eventInitDict?: DragEventInit): DragEvent;\\n};\\n\\ninterface DynamicsCompressorNode extends AudioNode {\\n    readonly attack: AudioParam;\\n    readonly knee: AudioParam;\\n    readonly ratio: AudioParam;\\n    readonly reduction: number;\\n    readonly release: AudioParam;\\n    readonly threshold: AudioParam;\\n}\\n\\ndeclare var DynamicsCompressorNode: {\\n    prototype: DynamicsCompressorNode;\\n    new(context: BaseAudioContext, options?: DynamicsCompressorOptions): DynamicsCompressorNode;\\n};\\n\\ninterface EXT_blend_minmax {\\n    readonly MAX_EXT: number;\\n    readonly MIN_EXT: number;\\n}\\n\\ninterface EXT_frag_depth {\\n}\\n\\ninterface EXT_sRGB {\\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT: number;\\n    readonly SRGB8_ALPHA8_EXT: number;\\n    readonly SRGB_ALPHA_EXT: number;\\n    readonly SRGB_EXT: number;\\n}\\n\\ninterface EXT_shader_texture_lod {\\n}\\n\\ninterface EXT_texture_filter_anisotropic {\\n    readonly MAX_TEXTURE_MAX_ANISOTROPY_EXT: number;\\n    readonly TEXTURE_MAX_ANISOTROPY_EXT: number;\\n}\\n\\ndeclare var EXT_texture_filter_anisotropic: {\\n    prototype: EXT_texture_filter_anisotropic;\\n    new(): EXT_texture_filter_anisotropic;\\n    readonly MAX_TEXTURE_MAX_ANISOTROPY_EXT: number;\\n    readonly TEXTURE_MAX_ANISOTROPY_EXT: number;\\n};\\n\\ninterface ElementEventMap {\\n    \"fullscreenchange\": Event;\\n    \"fullscreenerror\": Event;\\n}\\n\\ninterface Element extends Node, ParentNode, NonDocumentTypeChildNode, ChildNode, Slotable, Animatable {\\n    readonly assignedSlot: HTMLSlotElement | null;\\n    readonly attributes: NamedNodeMap;\\n    /**\\n     * Allows for manipulation of element\\'s class content attribute as a\\n     * set of whitespace-separated tokens through a DOMTokenList object.\\n     */\\n    readonly classList: DOMTokenList;\\n    /**\\n     * Returns the value of element\\'s class content attribute. Can be set\\n     * to change it.\\n     */\\n    className: string;\\n    readonly clientHeight: number;\\n    readonly clientLeft: number;\\n    readonly clientTop: number;\\n    readonly clientWidth: number;\\n    /**\\n     * Returns the value of element\\'s id content attribute. Can be set to\\n     * change it.\\n     */\\n    id: string;\\n    innerHTML: string;\\n    /**\\n     * Returns the local name.\\n     */\\n    readonly localName: string;\\n    /**\\n     * Returns the namespace.\\n     */\\n    readonly namespaceURI: string | null;\\n    onfullscreenchange: ((this: Element, ev: Event) => any) | null;\\n    onfullscreenerror: ((this: Element, ev: Event) => any) | null;\\n    outerHTML: string;\\n    /**\\n     * Returns the namespace prefix.\\n     */\\n    readonly prefix: string | null;\\n    readonly scrollHeight: number;\\n    scrollLeft: number;\\n    scrollTop: number;\\n    readonly scrollWidth: number;\\n    /**\\n     * Returns element\\'s shadow root, if any, and if shadow root\\'s mode is \"open\", and null otherwise.\\n     */\\n    readonly shadowRoot: ShadowRoot | null;\\n    /**\\n     * Returns the value of element\\'s slot content attribute. Can be set to\\n     * change it.\\n     */\\n    slot: string;\\n    /**\\n     * Returns the HTML-uppercased qualified name.\\n     */\\n    readonly tagName: string;\\n    /**\\n     * Creates a shadow root for element and returns it.\\n     */\\n    attachShadow(shadowRootInitDict: ShadowRootInit): ShadowRoot;\\n    /**\\n     * Returns the first (starting at element) inclusive ancestor that matches selectors, and null otherwise.\\n     */\\n    closest<K extends keyof HTMLElementTagNameMap>(selector: K): HTMLElementTagNameMap[K] | null;\\n    closest<K extends keyof SVGElementTagNameMap>(selector: K): SVGElementTagNameMap[K] | null;\\n    closest(selector: string): Element | null;\\n    /**\\n     * Returns element\\'s first attribute whose qualified name is qualifiedName, and null if there is no such attribute otherwise.\\n     */\\n    getAttribute(qualifiedName: string): string | null;\\n    /**\\n     * Returns element\\'s attribute whose namespace is namespace and local name is localName, and null if there is\\n     * no such attribute otherwise.\\n     */\\n    getAttributeNS(namespace: string | null, localName: string): string | null;\\n    /**\\n     * Returns the qualified names of all element\\'s attributes.\\n     * Can contain duplicates.\\n     */\\n    getAttributeNames(): string[];\\n    getAttributeNode(name: string): Attr | null;\\n    getAttributeNodeNS(namespaceURI: string, localName: string): Attr | null;\\n    getBoundingClientRect(): ClientRect | DOMRect;\\n    getClientRects(): ClientRectList | DOMRectList;\\n    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;\\n    getElementsByTagName<K extends keyof HTMLElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<HTMLElementTagNameMap[K]>;\\n    getElementsByTagName<K extends keyof SVGElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<SVGElementTagNameMap[K]>;\\n    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/1999/xhtml\", localName: string): HTMLCollectionOf<HTMLElement>;\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/2000/svg\", localName: string): HTMLCollectionOf<SVGElement>;\\n    getElementsByTagNameNS(namespaceURI: string, localName: string): HTMLCollectionOf<Element>;\\n    /**\\n     * Returns true if element has an attribute whose qualified name is qualifiedName, and false otherwise.\\n     */\\n    hasAttribute(qualifiedName: string): boolean;\\n    /**\\n     * Returns true if element has an attribute whose namespace is namespace and local name is localName.\\n     */\\n    hasAttributeNS(namespace: string | null, localName: string): boolean;\\n    /**\\n     * Returns true if element has attributes, and false otherwise.\\n     */\\n    hasAttributes(): boolean;\\n    hasPointerCapture(pointerId: number): boolean;\\n    insertAdjacentElement(position: InsertPosition, insertedElement: Element): Element | null;\\n    insertAdjacentHTML(where: InsertPosition, html: string): void;\\n    insertAdjacentText(where: InsertPosition, text: string): void;\\n    /**\\n     * Returns true if matching selectors against element\\'s root yields element, and false otherwise.\\n     */\\n    matches(selectors: string): boolean;\\n    msGetRegionContent(): any;\\n    releasePointerCapture(pointerId: number): void;\\n    /**\\n     * Removes element\\'s first attribute whose qualified name is qualifiedName.\\n     */\\n    removeAttribute(qualifiedName: string): void;\\n    /**\\n     * Removes element\\'s attribute whose namespace is namespace and local name is localName.\\n     */\\n    removeAttributeNS(namespace: string | null, localName: string): void;\\n    removeAttributeNode(attr: Attr): Attr;\\n    /**\\n     * Displays element fullscreen and resolves promise when done.\\n     */\\n    requestFullscreen(): Promise<void>;\\n    scroll(options?: ScrollToOptions): void;\\n    scroll(x: number, y: number): void;\\n    scrollBy(options?: ScrollToOptions): void;\\n    scrollBy(x: number, y: number): void;\\n    scrollIntoView(arg?: boolean | ScrollIntoViewOptions): void;\\n    scrollTo(options?: ScrollToOptions): void;\\n    scrollTo(x: number, y: number): void;\\n    /**\\n     * Sets the value of element\\'s first attribute whose qualified name is qualifiedName to value.\\n     */\\n    setAttribute(qualifiedName: string, value: string): void;\\n    /**\\n     * Sets the value of element\\'s attribute whose namespace is namespace and local name is localName to value.\\n     */\\n    setAttributeNS(namespace: string | null, qualifiedName: string, value: string): void;\\n    setAttributeNode(attr: Attr): Attr | null;\\n    setAttributeNodeNS(attr: Attr): Attr | null;\\n    setPointerCapture(pointerId: number): void;\\n    /**\\n     * If force is not given, \"toggles\" qualifiedName, removing it if it is\\n     * present and adding it if it is not present. If force is true, adds qualifiedName. If force is false, removes qualifiedName.\\n     * Returns true if qualifiedName is now present, and false otherwise.\\n     */\\n    toggleAttribute(qualifiedName: string, force?: boolean): boolean;\\n    webkitMatchesSelector(selectors: string): boolean;\\n    addEventListener<K extends keyof ElementEventMap>(type: K, listener: (this: Element, ev: ElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ElementEventMap>(type: K, listener: (this: Element, ev: ElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Element: {\\n    prototype: Element;\\n    new(): Element;\\n};\\n\\ninterface ElementCSSInlineStyle {\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ninterface ElementContentEditable {\\n    contentEditable: string;\\n    inputMode: string;\\n    readonly isContentEditable: boolean;\\n}\\n\\ninterface ElementCreationOptions {\\n    is?: string;\\n}\\n\\ninterface ErrorEvent extends Event {\\n    readonly colno: number;\\n    readonly error: any;\\n    readonly filename: string;\\n    readonly lineno: number;\\n    readonly message: string;\\n}\\n\\ndeclare var ErrorEvent: {\\n    prototype: ErrorEvent;\\n    new(type: string, eventInitDict?: ErrorEventInit): ErrorEvent;\\n};\\n\\ninterface Event {\\n    /**\\n     * Returns true or false depending on how event was initialized. True if event goes through its target\\'s ancestors in reverse tree order, and false otherwise.\\n     */\\n    readonly bubbles: boolean;\\n    cancelBubble: boolean;\\n    readonly cancelable: boolean;\\n    /**\\n     * Returns true or false depending on how event was initialized. True if event invokes listeners past a ShadowRoot node that is the root of its target, and false otherwise.\\n     */\\n    readonly composed: boolean;\\n    /**\\n     * Returns the object whose event listener\\'s callback is currently being\\n     * invoked.\\n     */\\n    readonly currentTarget: EventTarget | null;\\n    readonly defaultPrevented: boolean;\\n    readonly eventPhase: number;\\n    /**\\n     * Returns true if event was dispatched by the user agent, and\\n     * false otherwise.\\n     */\\n    readonly isTrusted: boolean;\\n    returnValue: boolean;\\n    readonly srcElement: Element | null;\\n    /**\\n     * Returns the object to which event is dispatched (its target).\\n     */\\n    readonly target: EventTarget | null;\\n    /**\\n     * Returns the event\\'s timestamp as the number of milliseconds measured relative to\\n     * the time origin.\\n     */\\n    readonly timeStamp: number;\\n    /**\\n     * Returns the type of event, e.g.\\n     * \"click\", \"hashchange\", or\\n     * \"submit\".\\n     */\\n    readonly type: string;\\n    composedPath(): EventTarget[];\\n    initEvent(type: string, bubbles?: boolean, cancelable?: boolean): void;\\n    preventDefault(): void;\\n    /**\\n     * Invoking this method prevents event from reaching\\n     * any registered event listeners after the current one finishes running and, when dispatched in a tree, also prevents event from reaching any\\n     * other objects.\\n     */\\n    stopImmediatePropagation(): void;\\n    /**\\n     * When dispatched in a tree, invoking this method prevents event from reaching any objects other than the current object.\\n     */\\n    stopPropagation(): void;\\n    readonly AT_TARGET: number;\\n    readonly BUBBLING_PHASE: number;\\n    readonly CAPTURING_PHASE: number;\\n    readonly NONE: number;\\n}\\n\\ndeclare var Event: {\\n    prototype: Event;\\n    new(type: string, eventInitDict?: EventInit): Event;\\n    readonly AT_TARGET: number;\\n    readonly BUBBLING_PHASE: number;\\n    readonly CAPTURING_PHASE: number;\\n    readonly NONE: number;\\n};\\n\\ninterface EventListenerObject {\\n    handleEvent(evt: Event): void;\\n}\\n\\ninterface EventSource extends EventTarget {\\n    readonly CLOSED: number;\\n    readonly CONNECTING: number;\\n    readonly OPEN: number;\\n    onerror: (evt: MessageEvent) => any;\\n    onmessage: (evt: MessageEvent) => any;\\n    onopen: (evt: MessageEvent) => any;\\n    readonly readyState: number;\\n    readonly url: string;\\n    readonly withCredentials: boolean;\\n    close(): void;\\n}\\n\\ndeclare var EventSource: {\\n    prototype: EventSource;\\n    new(url: string, eventSourceInitDict?: EventSourceInit): EventSource;\\n};\\n\\ninterface EventSourceInit {\\n    readonly withCredentials: boolean;\\n}\\n\\ninterface EventTarget {\\n    /**\\n     * Appends an event listener for events whose type attribute value is type. The callback argument sets the callback that will be invoked when the event is dispatched.\\n     * The options argument sets listener-specific options. For compatibility this can be a\\n     * boolean, in which case the method behaves exactly as if the value was specified as options\\'s capture.\\n     * When set to true, options\\'s capture prevents callback from being invoked when the event\\'s eventPhase attribute value is BUBBLING_PHASE. When false (or not present), callback will not be invoked when event\\'s eventPhase attribute value is CAPTURING_PHASE. Either way, callback will be invoked if event\\'s eventPhase attribute value is AT_TARGET.\\n     * When set to true, options\\'s passive indicates that the callback will not cancel the event by invoking preventDefault(). This is used to enable performance optimizations described in §2.8 Observing event listeners.\\n     * When set to true, options\\'s once indicates that the callback will only be invoked once after which the event listener will\\n     * be removed.\\n     * The event listener is appended to target\\'s event listener list and is not appended if it has the same type, callback, and capture.\\n     */\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject | null, options?: boolean | AddEventListenerOptions): void;\\n    /**\\n     * Dispatches a synthetic event event to target and returns true\\n     * if either event\\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\\n     */\\n    dispatchEvent(event: Event): boolean;\\n    /**\\n     * Removes the event listener in target\\'s event listener list with the same type, callback, and options.\\n     */\\n    removeEventListener(type: string, callback: EventListenerOrEventListenerObject | null, options?: EventListenerOptions | boolean): void;\\n}\\n\\ndeclare var EventTarget: {\\n    prototype: EventTarget;\\n    new(): EventTarget;\\n};\\n\\ninterface ExtensionScriptApis {\\n    extensionIdToShortId(extensionId: string): number;\\n    fireExtensionApiTelemetry(functionName: string, isSucceeded: boolean, isSupported: boolean, errorString: string): void;\\n    genericFunction(routerAddress: any, parameters?: string, callbackId?: number): void;\\n    genericSynchronousFunction(functionId: number, parameters?: string): string;\\n    genericWebRuntimeCallout(to: any, from: any, payload: string): void;\\n    getExtensionId(): string;\\n    registerGenericFunctionCallbackHandler(callbackHandler: Function): void;\\n    registerGenericPersistentCallbackHandler(callbackHandler: Function): void;\\n    registerWebRuntimeCallbackHandler(handler: Function): any;\\n}\\n\\ndeclare var ExtensionScriptApis: {\\n    prototype: ExtensionScriptApis;\\n    new(): ExtensionScriptApis;\\n};\\n\\ninterface External {\\n    /** @deprecated */\\n    AddSearchProvider(): void;\\n    /** @deprecated */\\n    IsSearchProviderInstalled(): void;\\n}\\n\\ninterface File extends Blob {\\n    readonly lastModified: number;\\n    readonly name: string;\\n}\\n\\ndeclare var File: {\\n    prototype: File;\\n    new(fileBits: BlobPart[], fileName: string, options?: FilePropertyBag): File;\\n};\\n\\ninterface FileList {\\n    readonly length: number;\\n    item(index: number): File | null;\\n    [index: number]: File;\\n}\\n\\ndeclare var FileList: {\\n    prototype: FileList;\\n    new(): FileList;\\n};\\n\\ninterface FileReaderEventMap {\\n    \"abort\": ProgressEvent;\\n    \"error\": ProgressEvent;\\n    \"load\": ProgressEvent;\\n    \"loadend\": ProgressEvent;\\n    \"loadstart\": ProgressEvent;\\n    \"progress\": ProgressEvent;\\n}\\n\\ninterface FileReader extends EventTarget {\\n    readonly error: DOMException | null;\\n    onabort: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onerror: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onload: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onloadend: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onloadstart: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onprogress: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    readonly readyState: number;\\n    readonly result: string | ArrayBuffer | null;\\n    abort(): void;\\n    readAsArrayBuffer(blob: Blob): void;\\n    readAsBinaryString(blob: Blob): void;\\n    readAsDataURL(blob: Blob): void;\\n    readAsText(blob: Blob, label?: string): void;\\n    readonly DONE: number;\\n    readonly EMPTY: number;\\n    readonly LOADING: number;\\n    addEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var FileReader: {\\n    prototype: FileReader;\\n    new(): FileReader;\\n    readonly DONE: number;\\n    readonly EMPTY: number;\\n    readonly LOADING: number;\\n};\\n\\ninterface FocusEvent extends UIEvent {\\n    readonly relatedTarget: EventTarget;\\n    initFocusEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, relatedTargetArg: EventTarget): void;\\n}\\n\\ndeclare var FocusEvent: {\\n    prototype: FocusEvent;\\n    new(typeArg: string, eventInitDict?: FocusEventInit): FocusEvent;\\n};\\n\\ninterface FocusNavigationEvent extends Event {\\n    readonly navigationReason: NavigationReason;\\n    readonly originHeight: number;\\n    readonly originLeft: number;\\n    readonly originTop: number;\\n    readonly originWidth: number;\\n    requestFocus(): void;\\n}\\n\\ndeclare var FocusNavigationEvent: {\\n    prototype: FocusNavigationEvent;\\n    new(type: string, eventInitDict?: FocusNavigationEventInit): FocusNavigationEvent;\\n};\\n\\ninterface FormData {\\n    append(name: string, value: string | Blob, fileName?: string): void;\\n    delete(name: string): void;\\n    get(name: string): FormDataEntryValue | null;\\n    getAll(name: string): FormDataEntryValue[];\\n    has(name: string): boolean;\\n    set(name: string, value: string | Blob, fileName?: string): void;\\n    forEach(callbackfn: (value: FormDataEntryValue, key: string, parent: FormData) => void, thisArg?: any): void;\\n}\\n\\ndeclare var FormData: {\\n    prototype: FormData;\\n    new(form?: HTMLFormElement): FormData;\\n};\\n\\ninterface GainNode extends AudioNode {\\n    readonly gain: AudioParam;\\n}\\n\\ndeclare var GainNode: {\\n    prototype: GainNode;\\n    new(context: BaseAudioContext, options?: GainOptions): GainNode;\\n};\\n\\ninterface Gamepad {\\n    readonly axes: number[];\\n    readonly buttons: GamepadButton[];\\n    readonly connected: boolean;\\n    readonly displayId: number;\\n    readonly hand: GamepadHand;\\n    readonly hapticActuators: GamepadHapticActuator[];\\n    readonly id: string;\\n    readonly index: number;\\n    readonly mapping: GamepadMappingType;\\n    readonly pose: GamepadPose | null;\\n    readonly timestamp: number;\\n}\\n\\ndeclare var Gamepad: {\\n    prototype: Gamepad;\\n    new(): Gamepad;\\n};\\n\\ninterface GamepadButton {\\n    readonly pressed: boolean;\\n    readonly touched: boolean;\\n    readonly value: number;\\n}\\n\\ndeclare var GamepadButton: {\\n    prototype: GamepadButton;\\n    new(): GamepadButton;\\n};\\n\\ninterface GamepadEvent extends Event {\\n    readonly gamepad: Gamepad;\\n}\\n\\ndeclare var GamepadEvent: {\\n    prototype: GamepadEvent;\\n    new(typeArg: string, eventInitDict?: GamepadEventInit): GamepadEvent;\\n};\\n\\ninterface GamepadHapticActuator {\\n    readonly type: GamepadHapticActuatorType;\\n    pulse(value: number, duration: number): Promise<boolean>;\\n}\\n\\ndeclare var GamepadHapticActuator: {\\n    prototype: GamepadHapticActuator;\\n    new(): GamepadHapticActuator;\\n};\\n\\ninterface GamepadPose {\\n    readonly angularAcceleration: Float32Array | null;\\n    readonly angularVelocity: Float32Array | null;\\n    readonly hasOrientation: boolean;\\n    readonly hasPosition: boolean;\\n    readonly linearAcceleration: Float32Array | null;\\n    readonly linearVelocity: Float32Array | null;\\n    readonly orientation: Float32Array | null;\\n    readonly position: Float32Array | null;\\n}\\n\\ndeclare var GamepadPose: {\\n    prototype: GamepadPose;\\n    new(): GamepadPose;\\n};\\n\\ninterface Geolocation {\\n    clearWatch(watchId: number): void;\\n    getCurrentPosition(successCallback: PositionCallback, errorCallback?: PositionErrorCallback, options?: PositionOptions): void;\\n    watchPosition(successCallback: PositionCallback, errorCallback?: PositionErrorCallback, options?: PositionOptions): number;\\n}\\n\\ninterface GetSVGDocument {\\n    getSVGDocument(): Document;\\n}\\n\\ninterface GlobalEventHandlersEventMap {\\n    \"abort\": UIEvent;\\n    \"animationcancel\": AnimationEvent;\\n    \"animationend\": AnimationEvent;\\n    \"animationiteration\": AnimationEvent;\\n    \"animationstart\": AnimationEvent;\\n    \"auxclick\": Event;\\n    \"blur\": FocusEvent;\\n    \"cancel\": Event;\\n    \"canplay\": Event;\\n    \"canplaythrough\": Event;\\n    \"change\": Event;\\n    \"click\": MouseEvent;\\n    \"close\": Event;\\n    \"contextmenu\": MouseEvent;\\n    \"cuechange\": Event;\\n    \"dblclick\": MouseEvent;\\n    \"drag\": DragEvent;\\n    \"dragend\": DragEvent;\\n    \"dragenter\": DragEvent;\\n    \"dragexit\": Event;\\n    \"dragleave\": DragEvent;\\n    \"dragover\": DragEvent;\\n    \"dragstart\": DragEvent;\\n    \"drop\": DragEvent;\\n    \"durationchange\": Event;\\n    \"emptied\": Event;\\n    \"ended\": Event;\\n    \"error\": ErrorEvent;\\n    \"focus\": FocusEvent;\\n    \"gotpointercapture\": PointerEvent;\\n    \"input\": Event;\\n    \"invalid\": Event;\\n    \"keydown\": KeyboardEvent;\\n    \"keypress\": KeyboardEvent;\\n    \"keyup\": KeyboardEvent;\\n    \"load\": Event;\\n    \"loadeddata\": Event;\\n    \"loadedmetadata\": Event;\\n    \"loadend\": ProgressEvent;\\n    \"loadstart\": Event;\\n    \"lostpointercapture\": PointerEvent;\\n    \"mousedown\": MouseEvent;\\n    \"mouseenter\": MouseEvent;\\n    \"mouseleave\": MouseEvent;\\n    \"mousemove\": MouseEvent;\\n    \"mouseout\": MouseEvent;\\n    \"mouseover\": MouseEvent;\\n    \"mouseup\": MouseEvent;\\n    \"pause\": Event;\\n    \"play\": Event;\\n    \"playing\": Event;\\n    \"pointercancel\": PointerEvent;\\n    \"pointerdown\": PointerEvent;\\n    \"pointerenter\": PointerEvent;\\n    \"pointerleave\": PointerEvent;\\n    \"pointermove\": PointerEvent;\\n    \"pointerout\": PointerEvent;\\n    \"pointerover\": PointerEvent;\\n    \"pointerup\": PointerEvent;\\n    \"progress\": ProgressEvent;\\n    \"ratechange\": Event;\\n    \"reset\": Event;\\n    \"resize\": UIEvent;\\n    \"scroll\": UIEvent;\\n    \"securitypolicyviolation\": SecurityPolicyViolationEvent;\\n    \"seeked\": Event;\\n    \"seeking\": Event;\\n    \"select\": UIEvent;\\n    \"stalled\": Event;\\n    \"submit\": Event;\\n    \"suspend\": Event;\\n    \"timeupdate\": Event;\\n    \"toggle\": Event;\\n    \"touchcancel\": TouchEvent;\\n    \"touchend\": TouchEvent;\\n    \"touchmove\": TouchEvent;\\n    \"touchstart\": TouchEvent;\\n    \"transitioncancel\": TransitionEvent;\\n    \"transitionend\": TransitionEvent;\\n    \"transitionrun\": TransitionEvent;\\n    \"transitionstart\": TransitionEvent;\\n    \"volumechange\": Event;\\n    \"waiting\": Event;\\n    \"wheel\": WheelEvent;\\n}\\n\\ninterface GlobalEventHandlers {\\n    /**\\n     * Fires when the user aborts the download.\\n     * @param ev The event.\\n     */\\n    onabort: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    onanimationcancel: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onanimationend: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onanimationiteration: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onanimationstart: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onauxclick: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the object loses the input focus.\\n     * @param ev The focus event.\\n     */\\n    onblur: ((this: GlobalEventHandlers, ev: FocusEvent) => any) | null;\\n    oncancel: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when playback is possible, but would require further buffering.\\n     * @param ev The event.\\n     */\\n    oncanplay: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    oncanplaythrough: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the contents of the object or selection have changed.\\n     * @param ev The event.\\n     */\\n    onchange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user clicks the left mouse button on the object\\n     * @param ev The mouse event.\\n     */\\n    onclick: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    onclose: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user clicks the right mouse button in the client area, opening the context menu.\\n     * @param ev The mouse event.\\n     */\\n    oncontextmenu: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    oncuechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user double-clicks the object.\\n     * @param ev The mouse event.\\n     */\\n    ondblclick: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires on the source object continuously during a drag operation.\\n     * @param ev The event.\\n     */\\n    ondrag: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the source object when the user releases the mouse at the close of a drag operation.\\n     * @param ev The event.\\n     */\\n    ondragend: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the target element when the user drags the object to a valid drop target.\\n     * @param ev The drag event.\\n     */\\n    ondragenter: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    ondragexit: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires on the target object when the user moves the mouse out of a valid drop target during a drag operation.\\n     * @param ev The drag event.\\n     */\\n    ondragleave: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the target element continuously while the user drags the object over a valid drop target.\\n     * @param ev The event.\\n     */\\n    ondragover: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the source object when the user starts to drag a text selection or selected object.\\n     * @param ev The event.\\n     */\\n    ondragstart: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    ondrop: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Occurs when the duration attribute is updated.\\n     * @param ev The event.\\n     */\\n    ondurationchange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the media element is reset to its initial state.\\n     * @param ev The event.\\n     */\\n    onemptied: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the end of playback is reached.\\n     * @param ev The event\\n     */\\n    onended: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when an error occurs during object loading.\\n     * @param ev The event.\\n     */\\n    onerror: ErrorEventHandler;\\n    /**\\n     * Fires when the object receives focus.\\n     * @param ev The event.\\n     */\\n    onfocus: ((this: GlobalEventHandlers, ev: FocusEvent) => any) | null;\\n    ongotpointercapture: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    oninput: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    oninvalid: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user presses a key.\\n     * @param ev The keyboard event\\n     */\\n    onkeydown: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\\n    /**\\n     * Fires when the user presses an alphanumeric key.\\n     * @param ev The event.\\n     */\\n    onkeypress: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\\n    /**\\n     * Fires when the user releases a key.\\n     * @param ev The keyboard event\\n     */\\n    onkeyup: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\\n    /**\\n     * Fires immediately after the browser loads the object.\\n     * @param ev The event.\\n     */\\n    onload: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when media data is loaded at the current playback position.\\n     * @param ev The event.\\n     */\\n    onloadeddata: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the duration and dimensions of the media have been determined.\\n     * @param ev The event.\\n     */\\n    onloadedmetadata: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onloadend: ((this: GlobalEventHandlers, ev: ProgressEvent) => any) | null;\\n    /**\\n     * Occurs when Internet Explorer begins looking for media data.\\n     * @param ev The event.\\n     */\\n    onloadstart: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onlostpointercapture: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    /**\\n     * Fires when the user clicks the object with either mouse button.\\n     * @param ev The mouse event.\\n     */\\n    onmousedown: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    onmouseenter: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    onmouseleave: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user moves the mouse over the object.\\n     * @param ev The mouse event.\\n     */\\n    onmousemove: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user moves the mouse pointer outside the boundaries of the object.\\n     * @param ev The mouse event.\\n     */\\n    onmouseout: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user moves the mouse pointer into the object.\\n     * @param ev The mouse event.\\n     */\\n    onmouseover: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user releases a mouse button while the mouse is over the object.\\n     * @param ev The mouse event.\\n     */\\n    onmouseup: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Occurs when playback is paused.\\n     * @param ev The event.\\n     */\\n    onpause: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the play method is requested.\\n     * @param ev The event.\\n     */\\n    onplay: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the audio or video has started playing.\\n     * @param ev The event.\\n     */\\n    onplaying: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onpointercancel: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerdown: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerenter: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerleave: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointermove: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerout: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerover: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerup: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    /**\\n     * Occurs to indicate progress while downloading media data.\\n     * @param ev The event.\\n     */\\n    onprogress: ((this: GlobalEventHandlers, ev: ProgressEvent) => any) | null;\\n    /**\\n     * Occurs when the playback rate is increased or decreased.\\n     * @param ev The event.\\n     */\\n    onratechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user resets a form.\\n     * @param ev The event.\\n     */\\n    onreset: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onresize: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    /**\\n     * Fires when the user repositions the scroll box in the scroll bar on the object.\\n     * @param ev The event.\\n     */\\n    onscroll: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    onsecuritypolicyviolation: ((this: GlobalEventHandlers, ev: SecurityPolicyViolationEvent) => any) | null;\\n    /**\\n     * Occurs when the seek operation ends.\\n     * @param ev The event.\\n     */\\n    onseeked: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the current playback position is moved.\\n     * @param ev The event.\\n     */\\n    onseeking: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the current selection changes.\\n     * @param ev The event.\\n     */\\n    onselect: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    /**\\n     * Occurs when the download has stopped.\\n     * @param ev The event.\\n     */\\n    onstalled: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onsubmit: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs if the load operation has been intentionally halted.\\n     * @param ev The event.\\n     */\\n    onsuspend: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs to indicate the current playback position.\\n     * @param ev The event.\\n     */\\n    ontimeupdate: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    ontoggle: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    ontouchcancel: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontouchend: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontouchmove: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontouchstart: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontransitioncancel: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    ontransitionend: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    ontransitionrun: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    ontransitionstart: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    /**\\n     * Occurs when the volume is changed, or playback is muted or unmuted.\\n     * @param ev The event.\\n     */\\n    onvolumechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when playback stops because the next frame of a video resource is not available.\\n     * @param ev The event.\\n     */\\n    onwaiting: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onwheel: ((this: GlobalEventHandlers, ev: WheelEvent) => any) | null;\\n    addEventListener<K extends keyof GlobalEventHandlersEventMap>(type: K, listener: (this: GlobalEventHandlers, ev: GlobalEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof GlobalEventHandlersEventMap>(type: K, listener: (this: GlobalEventHandlers, ev: GlobalEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface GlobalFetch {\\n    fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\n}\\n\\ninterface HTMLAllCollection {\\n    /**\\n     * Returns the number of elements in the collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * element = collection(index)\\n     */\\n    item(nameOrIndex?: string): HTMLCollection | Element | null;\\n    /**\\n     * element = collection(name)\\n     */\\n    namedItem(name: string): HTMLCollection | Element | null;\\n    [index: number]: Element;\\n}\\n\\ndeclare var HTMLAllCollection: {\\n    prototype: HTMLAllCollection;\\n    new(): HTMLAllCollection;\\n};\\n\\ninterface HTMLAnchorElement extends HTMLElement, HTMLHyperlinkElementUtils {\\n    /**\\n     * Sets or retrieves the character set used to encode the object.\\n     */\\n    /** @deprecated */\\n    charset: string;\\n    /**\\n     * Sets or retrieves the coordinates of the object.\\n     */\\n    /** @deprecated */\\n    coords: string;\\n    download: string;\\n    /**\\n     * Sets or retrieves the language code of the object.\\n     */\\n    hreflang: string;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    ping: string;\\n    referrerPolicy: string;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    rel: string;\\n    readonly relList: DOMTokenList;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    /** @deprecated */\\n    rev: string;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    /** @deprecated */\\n    shape: string;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    /**\\n     * Retrieves or sets the text of the object as a string.\\n     */\\n    text: string;\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAnchorElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAnchorElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAnchorElement: {\\n    prototype: HTMLAnchorElement;\\n    new(): HTMLAnchorElement;\\n};\\n\\ninterface HTMLAppletElement extends HTMLElement {\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    /** @deprecated */\\n    alt: string;\\n    /**\\n     * Sets or retrieves a character string that can be used to implement your own archive functionality for the object.\\n     */\\n    /** @deprecated */\\n    archive: string;\\n    /** @deprecated */\\n    code: string;\\n    /**\\n     * Sets or retrieves the URL of the component.\\n     */\\n    /** @deprecated */\\n    codeBase: string;\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    /** @deprecated */\\n    height: string;\\n    /** @deprecated */\\n    hspace: number;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /** @deprecated */\\n    object: string;\\n    /** @deprecated */\\n    vspace: number;\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAppletElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAppletElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAppletElement: {\\n    prototype: HTMLAppletElement;\\n    new(): HTMLAppletElement;\\n};\\n\\ninterface HTMLAreaElement extends HTMLElement, HTMLHyperlinkElementUtils {\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    alt: string;\\n    /**\\n     * Sets or retrieves the coordinates of the object.\\n     */\\n    coords: string;\\n    download: string;\\n    /**\\n     * Sets or gets whether clicks in this region cause action.\\n     */\\n    /** @deprecated */\\n    noHref: boolean;\\n    ping: string;\\n    referrerPolicy: string;\\n    rel: string;\\n    readonly relList: DOMTokenList;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    shape: string;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAreaElement: {\\n    prototype: HTMLAreaElement;\\n    new(): HTMLAreaElement;\\n};\\n\\ninterface HTMLAudioElement extends HTMLMediaElement {\\n    addEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLAudioElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLAudioElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAudioElement: {\\n    prototype: HTMLAudioElement;\\n    new(): HTMLAudioElement;\\n};\\n\\ninterface HTMLBRElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the side on which floating objects are not to be positioned when any IHTMLBlockElement is inserted into the document.\\n     */\\n    /** @deprecated */\\n    clear: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBRElement: {\\n    prototype: HTMLBRElement;\\n    new(): HTMLBRElement;\\n};\\n\\ninterface HTMLBaseElement extends HTMLElement {\\n    /**\\n     * Gets or sets the baseline URL on which relative links are based.\\n     */\\n    href: string;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBaseElement: {\\n    prototype: HTMLBaseElement;\\n    new(): HTMLBaseElement;\\n};\\n\\ninterface HTMLBaseFontElement extends HTMLElement, DOML2DeprecatedColorProperty {\\n    /**\\n     * Sets or retrieves the current typeface family.\\n     */\\n    /** @deprecated */\\n    face: string;\\n    /**\\n     * Sets or retrieves the font size of the object.\\n     */\\n    /** @deprecated */\\n    size: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBaseFontElement: {\\n    prototype: HTMLBaseFontElement;\\n    new(): HTMLBaseFontElement;\\n};\\n\\ninterface HTMLBodyElementEventMap extends HTMLElementEventMap, WindowEventHandlersEventMap {\\n    \"orientationchange\": Event;\\n}\\n\\ninterface HTMLBodyElement extends HTMLElement, WindowEventHandlers {\\n    /** @deprecated */\\n    aLink: string;\\n    /** @deprecated */\\n    background: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    bgProperties: string;\\n    /** @deprecated */\\n    link: string;\\n    /** @deprecated */\\n    noWrap: boolean;\\n    /** @deprecated */\\n    onorientationchange: ((this: HTMLBodyElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    text: string;\\n    /** @deprecated */\\n    vLink: string;\\n    addEventListener<K extends keyof HTMLBodyElementEventMap>(type: K, listener: (this: HTMLBodyElement, ev: HTMLBodyElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLBodyElementEventMap>(type: K, listener: (this: HTMLBodyElement, ev: HTMLBodyElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBodyElement: {\\n    prototype: HTMLBodyElement;\\n    new(): HTMLBodyElement;\\n};\\n\\ninterface HTMLButtonElement extends HTMLElement {\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Overrides the action attribute (where the data on a form is sent) on the parent form element.\\n     */\\n    formAction: string;\\n    /**\\n     * Used to override the encoding (formEnctype attribute) specified on the form element.\\n     */\\n    formEnctype: string;\\n    /**\\n     * Overrides the submit method attribute previously specified on a form element.\\n     */\\n    formMethod: string;\\n    /**\\n     * Overrides any validation or required attributes on a form or form elements to allow it to be submitted without validation. This can be used to create a \"save draft\"-type submit option.\\n     */\\n    formNoValidate: boolean;\\n    /**\\n     * Overrides the target attribute on a form element.\\n     */\\n    formTarget: string;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Gets the classification and default behavior of the button.\\n     */\\n    type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Sets or retrieves the default or selected value of the control.\\n     */\\n    value: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLButtonElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLButtonElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLButtonElement: {\\n    prototype: HTMLButtonElement;\\n    new(): HTMLButtonElement;\\n};\\n\\ninterface HTMLCanvasElement extends HTMLElement {\\n    /**\\n     * Gets or sets the height of a canvas element on a document.\\n     */\\n    height: number;\\n    /**\\n     * Gets or sets the width of a canvas element on a document.\\n     */\\n    width: number;\\n    /**\\n     * Returns an object that provides methods and properties for drawing and manipulating images and graphics on a canvas element in a document. A context object includes information about colors, line widths, fonts, and other graphic parameters that can be drawn on a canvas.\\n     * @param contextId The identifier (ID) of the type of canvas to create. Internet Explorer 9 and Internet Explorer 10 support only a 2-D context using canvas.getContext(\"2d\"); IE11 Preview also supports 3-D or WebGL context using canvas.getContext(\"experimental-webgl\");\\n     */\\n    getContext(contextId: \"2d\", contextAttributes?: CanvasRenderingContext2DSettings): CanvasRenderingContext2D | null;\\n    getContext(contextId: \"webgl\" | \"experimental-webgl\", contextAttributes?: WebGLContextAttributes): WebGLRenderingContext | null;\\n    getContext(contextId: string, contextAttributes?: {}): CanvasRenderingContext2D | WebGLRenderingContext | null;\\n    toBlob(callback: BlobCallback, type?: string, quality?: any): void;\\n    /**\\n     * Returns the content of the current canvas as an image that you can use as a source for another canvas or an HTML element.\\n     * @param type The standard MIME type for the image format to return. If you do not specify this parameter, the default value is a PNG format image.\\n     */\\n    toDataURL(type?: string, quality?: any): string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLCanvasElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLCanvasElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLCanvasElement: {\\n    prototype: HTMLCanvasElement;\\n    new(): HTMLCanvasElement;\\n};\\n\\ninterface HTMLCollectionBase {\\n    /**\\n     * Sets or retrieves the number of objects in a collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * Retrieves an object from various collections.\\n     */\\n    item(index: number): Element | null;\\n    [index: number]: Element;\\n}\\n\\ninterface HTMLCollection extends HTMLCollectionBase {\\n    /**\\n     * Retrieves a select object or an object from an options collection.\\n     */\\n    namedItem(name: string): Element | null;\\n}\\n\\ndeclare var HTMLCollection: {\\n    prototype: HTMLCollection;\\n    new(): HTMLCollection;\\n};\\n\\ninterface HTMLCollectionOf<T extends Element> extends HTMLCollectionBase {\\n    item(index: number): T | null;\\n    namedItem(name: string): T | null;\\n    [index: number]: T;\\n}\\n\\ninterface HTMLDListElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDListElement: {\\n    prototype: HTMLDListElement;\\n    new(): HTMLDListElement;\\n};\\n\\ninterface HTMLDataElement extends HTMLElement {\\n    value: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDataElement: {\\n    prototype: HTMLDataElement;\\n    new(): HTMLDataElement;\\n};\\n\\ninterface HTMLDataListElement extends HTMLElement {\\n    readonly options: HTMLCollectionOf<HTMLOptionElement>;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDataListElement: {\\n    prototype: HTMLDataListElement;\\n    new(): HTMLDataListElement;\\n};\\n\\ninterface HTMLDetailsElement extends HTMLElement {\\n    open: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDetailsElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDetailsElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDetailsElement: {\\n    prototype: HTMLDetailsElement;\\n    new(): HTMLDetailsElement;\\n};\\n\\ninterface HTMLDialogElement extends HTMLElement {\\n    open: boolean;\\n    returnValue: string;\\n    close(returnValue?: string): void;\\n    show(): void;\\n    showModal(): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDialogElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDialogElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDialogElement: {\\n    prototype: HTMLDialogElement;\\n    new(): HTMLDialogElement;\\n};\\n\\ninterface HTMLDirectoryElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDirectoryElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDirectoryElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDirectoryElement: {\\n    prototype: HTMLDirectoryElement;\\n    new(): HTMLDirectoryElement;\\n};\\n\\ninterface HTMLDivElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDivElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDivElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDivElement: {\\n    prototype: HTMLDivElement;\\n    new(): HTMLDivElement;\\n};\\n\\ninterface HTMLDocument extends Document {\\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: HTMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: HTMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDocument: {\\n    prototype: HTMLDocument;\\n    new(): HTMLDocument;\\n};\\n\\ninterface HTMLElementEventMap extends ElementEventMap, GlobalEventHandlersEventMap, DocumentAndElementEventHandlersEventMap {\\n}\\n\\ninterface HTMLElement extends Element, GlobalEventHandlers, DocumentAndElementEventHandlers, ElementContentEditable, HTMLOrSVGElement, ElementCSSInlineStyle {\\n    accessKey: string;\\n    readonly accessKeyLabel: string;\\n    autocapitalize: string;\\n    dir: string;\\n    draggable: boolean;\\n    hidden: boolean;\\n    innerText: string;\\n    lang: string;\\n    readonly offsetHeight: number;\\n    readonly offsetLeft: number;\\n    readonly offsetParent: Element | null;\\n    readonly offsetTop: number;\\n    readonly offsetWidth: number;\\n    spellcheck: boolean;\\n    title: string;\\n    translate: boolean;\\n    click(): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLElement: {\\n    prototype: HTMLElement;\\n    new(): HTMLElement;\\n};\\n\\ninterface HTMLEmbedElement extends HTMLElement, GetSVGDocument {\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: string;\\n    hidden: any;\\n    /**\\n     * Gets or sets whether the DLNA PlayTo device is available.\\n     */\\n    msPlayToDisabled: boolean;\\n    /**\\n     * Gets or sets the path to the preferred media source. This enables the Play To target device to stream the media content, which can be DRM protected, from a different location, such as a cloud media server.\\n     */\\n    msPlayToPreferredSourceUri: string;\\n    /**\\n     * Gets or sets the primary DLNA PlayTo device.\\n     */\\n    msPlayToPrimary: boolean;\\n    /**\\n     * Gets the source associated with the media element for use by the PlayToManager.\\n     */\\n    readonly msPlayToSource: any;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /**\\n     * Retrieves the palette used for the embedded document.\\n     */\\n    readonly palette: string;\\n    /**\\n     * Retrieves the URL of the plug-in used to view an embedded document.\\n     */\\n    readonly pluginspage: string;\\n    readonly readyState: string;\\n    /**\\n     * Sets or retrieves a URL to be loaded by the object.\\n     */\\n    src: string;\\n    /**\\n     * Sets or retrieves the height and width units of the embed object.\\n     */\\n    units: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLEmbedElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLEmbedElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLEmbedElement: {\\n    prototype: HTMLEmbedElement;\\n    new(): HTMLEmbedElement;\\n};\\n\\ninterface HTMLFieldSetElement extends HTMLElement {\\n    disabled: boolean;\\n    readonly elements: HTMLCollection;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    name: string;\\n    readonly type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFieldSetElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFieldSetElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFieldSetElement: {\\n    prototype: HTMLFieldSetElement;\\n    new(): HTMLFieldSetElement;\\n};\\n\\ninterface HTMLFontElement extends HTMLElement {\\n    /** @deprecated */\\n    color: string;\\n    /**\\n     * Sets or retrieves the current typeface family.\\n     */\\n    /** @deprecated */\\n    face: string;\\n    /** @deprecated */\\n    size: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFontElement: {\\n    prototype: HTMLFontElement;\\n    new(): HTMLFontElement;\\n};\\n\\ninterface HTMLFormControlsCollection extends HTMLCollectionBase {\\n    /**\\n     * element = collection[name]\\n     */\\n    namedItem(name: string): RadioNodeList | Element | null;\\n}\\n\\ndeclare var HTMLFormControlsCollection: {\\n    prototype: HTMLFormControlsCollection;\\n    new(): HTMLFormControlsCollection;\\n};\\n\\ninterface HTMLFormElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a list of character encodings for input data that must be accepted by the server processing the form.\\n     */\\n    acceptCharset: string;\\n    /**\\n     * Sets or retrieves the URL to which the form content is sent for processing.\\n     */\\n    action: string;\\n    /**\\n     * Specifies whether autocomplete is applied to an editable text field.\\n     */\\n    autocomplete: string;\\n    /**\\n     * Retrieves a collection, in source order, of all controls in a given form.\\n     */\\n    readonly elements: HTMLFormControlsCollection;\\n    /**\\n     * Sets or retrieves the MIME encoding for the form.\\n     */\\n    encoding: string;\\n    /**\\n     * Sets or retrieves the encoding type for the form.\\n     */\\n    enctype: string;\\n    /**\\n     * Sets or retrieves the number of objects in a collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * Sets or retrieves how to send the form data to the server.\\n     */\\n    method: string;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Designates a form that is not validated when submitted.\\n     */\\n    noValidate: boolean;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Fires when the user resets a form.\\n     */\\n    reset(): void;\\n    /**\\n     * Fires when a FORM is about to be submitted.\\n     */\\n    submit(): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFormElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFormElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: Element;\\n    [name: string]: any;\\n}\\n\\ndeclare var HTMLFormElement: {\\n    prototype: HTMLFormElement;\\n    new(): HTMLFormElement;\\n};\\n\\ninterface HTMLFrameElement extends HTMLElement {\\n    /**\\n     * Retrieves the document object of the page or frame.\\n     */\\n    /** @deprecated */\\n    readonly contentDocument: Document | null;\\n    /**\\n     * Retrieves the object of the specified.\\n     */\\n    /** @deprecated */\\n    readonly contentWindow: WindowProxy | null;\\n    /**\\n     * Sets or retrieves whether to display a border for the frame.\\n     */\\n    /** @deprecated */\\n    frameBorder: string;\\n    /**\\n     * Sets or retrieves a URI to a long description of the object.\\n     */\\n    /** @deprecated */\\n    longDesc: string;\\n    /**\\n     * Sets or retrieves the top and bottom margin heights before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginHeight: string;\\n    /**\\n     * Sets or retrieves the left and right margin widths before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginWidth: string;\\n    /**\\n     * Sets or retrieves the frame name.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /**\\n     * Sets or retrieves whether the user can resize the frame.\\n     */\\n    /** @deprecated */\\n    noResize: boolean;\\n    /**\\n     * Sets or retrieves whether the frame can be scrolled.\\n     */\\n    /** @deprecated */\\n    scrolling: string;\\n    /**\\n     * Sets or retrieves a URL to be loaded by the object.\\n     */\\n    /** @deprecated */\\n    src: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFrameElement: {\\n    prototype: HTMLFrameElement;\\n    new(): HTMLFrameElement;\\n};\\n\\ninterface HTMLFrameSetElementEventMap extends HTMLElementEventMap, WindowEventHandlersEventMap {\\n}\\n\\ninterface HTMLFrameSetElement extends HTMLElement, WindowEventHandlers {\\n    /**\\n     * Sets or retrieves the frame widths of the object.\\n     */\\n    /** @deprecated */\\n    cols: string;\\n    /**\\n     * Sets or retrieves the frame heights of the object.\\n     */\\n    /** @deprecated */\\n    rows: string;\\n    addEventListener<K extends keyof HTMLFrameSetElementEventMap>(type: K, listener: (this: HTMLFrameSetElement, ev: HTMLFrameSetElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLFrameSetElementEventMap>(type: K, listener: (this: HTMLFrameSetElement, ev: HTMLFrameSetElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFrameSetElement: {\\n    prototype: HTMLFrameSetElement;\\n    new(): HTMLFrameSetElement;\\n};\\n\\ninterface HTMLHRElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    color: string;\\n    /**\\n     * Sets or retrieves whether the horizontal rule is drawn with 3-D shading.\\n     */\\n    /** @deprecated */\\n    noShade: boolean;\\n    /** @deprecated */\\n    size: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHRElement: {\\n    prototype: HTMLHRElement;\\n    new(): HTMLHRElement;\\n};\\n\\ninterface HTMLHeadElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHeadElement: {\\n    prototype: HTMLHeadElement;\\n    new(): HTMLHeadElement;\\n};\\n\\ninterface HTMLHeadingElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a value that indicates the table alignment.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadingElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadingElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHeadingElement: {\\n    prototype: HTMLHeadingElement;\\n    new(): HTMLHeadingElement;\\n};\\n\\ninterface HTMLHtmlElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the DTD version that governs the current document.\\n     */\\n    /** @deprecated */\\n    version: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHtmlElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHtmlElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHtmlElement: {\\n    prototype: HTMLHtmlElement;\\n    new(): HTMLHtmlElement;\\n};\\n\\ninterface HTMLHyperlinkElementUtils {\\n    hash: string;\\n    host: string;\\n    hostname: string;\\n    href: string;\\n    readonly origin: string;\\n    password: string;\\n    pathname: string;\\n    port: string;\\n    protocol: string;\\n    search: string;\\n    username: string;\\n}\\n\\ninterface HTMLIFrameElement extends HTMLElement, GetSVGDocument {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    allowFullscreen: boolean;\\n    allowPaymentRequest: boolean;\\n    /**\\n     * Retrieves the document object of the page or frame.\\n     */\\n    readonly contentDocument: Document | null;\\n    /**\\n     * Retrieves the object of the specified.\\n     */\\n    readonly contentWindow: Window | null;\\n    /**\\n     * Sets or retrieves whether to display a border for the frame.\\n     */\\n    /** @deprecated */\\n    frameBorder: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: string;\\n    /**\\n     * Sets or retrieves a URI to a long description of the object.\\n     */\\n    /** @deprecated */\\n    longDesc: string;\\n    /**\\n     * Sets or retrieves the top and bottom margin heights before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginHeight: string;\\n    /**\\n     * Sets or retrieves the left and right margin widths before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginWidth: string;\\n    /**\\n     * Sets or retrieves the frame name.\\n     */\\n    name: string;\\n    readonly sandbox: DOMTokenList;\\n    /**\\n     * Sets or retrieves whether the frame can be scrolled.\\n     */\\n    /** @deprecated */\\n    scrolling: string;\\n    /**\\n     * Sets or retrieves a URL to be loaded by the object.\\n     */\\n    src: string;\\n    /**\\n     * Sets or retrives the content of the page that is to contain.\\n     */\\n    srcdoc: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLIFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLIFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLIFrameElement: {\\n    prototype: HTMLIFrameElement;\\n    new(): HTMLIFrameElement;\\n};\\n\\ninterface HTMLImageElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    alt: string;\\n    /**\\n     * Specifies the properties of a border drawn around an object.\\n     */\\n    /** @deprecated */\\n    border: string;\\n    /**\\n     * Retrieves whether the object is fully loaded.\\n     */\\n    readonly complete: boolean;\\n    crossOrigin: string | null;\\n    readonly currentSrc: string;\\n    decoding: \"async\" | \"sync\" | \"auto\";\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: number;\\n    /**\\n     * Sets or retrieves the width of the border to draw around the object.\\n     */\\n    /** @deprecated */\\n    hspace: number;\\n    /**\\n     * Sets or retrieves whether the image is a server-side image map.\\n     */\\n    isMap: boolean;\\n    /**\\n     * Sets or retrieves a Uniform Resource Identifier (URI) to a long description of the object.\\n     */\\n    /** @deprecated */\\n    longDesc: string;\\n    /** @deprecated */\\n    lowsrc: string;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /**\\n     * The original height of the image resource before sizing.\\n     */\\n    readonly naturalHeight: number;\\n    /**\\n     * The original width of the image resource before sizing.\\n     */\\n    readonly naturalWidth: number;\\n    referrerPolicy: string;\\n    sizes: string;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    srcset: string;\\n    /**\\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\\n     */\\n    useMap: string;\\n    /**\\n     * Sets or retrieves the vertical margin for the object.\\n     */\\n    /** @deprecated */\\n    vspace: number;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: number;\\n    readonly x: number;\\n    readonly y: number;\\n    decode(): Promise<void>;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLImageElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLImageElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLImageElement: {\\n    prototype: HTMLImageElement;\\n    new(): HTMLImageElement;\\n};\\n\\ninterface HTMLInputElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a comma-separated list of content types.\\n     */\\n    accept: string;\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    alt: string;\\n    /**\\n     * Specifies whether autocomplete is applied to an editable text field.\\n     */\\n    autocomplete: string;\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    /**\\n     * Sets or retrieves the state of the check box or radio button.\\n     */\\n    checked: boolean;\\n    /**\\n     * Sets or retrieves the state of the check box or radio button.\\n     */\\n    defaultChecked: boolean;\\n    /**\\n     * Sets or retrieves the initial contents of the object.\\n     */\\n    defaultValue: string;\\n    dirName: string;\\n    disabled: boolean;\\n    /**\\n     * Returns a FileList object on a file type input object.\\n     */\\n    files: FileList | null;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Overrides the action attribute (where the data on a form is sent) on the parent form element.\\n     */\\n    formAction: string;\\n    /**\\n     * Used to override the encoding (formEnctype attribute) specified on the form element.\\n     */\\n    formEnctype: string;\\n    /**\\n     * Overrides the submit method attribute previously specified on a form element.\\n     */\\n    formMethod: string;\\n    /**\\n     * Overrides any validation or required attributes on a form or form elements to allow it to be submitted without validation. This can be used to create a \"save draft\"-type submit option.\\n     */\\n    formNoValidate: boolean;\\n    /**\\n     * Overrides the target attribute on a form element.\\n     */\\n    formTarget: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: number;\\n    indeterminate: boolean;\\n    readonly labels: NodeListOf<HTMLLabelElement> | null;\\n    /**\\n     * Specifies the ID of a pre-defined datalist of options for an input element.\\n     */\\n    readonly list: HTMLElement | null;\\n    /**\\n     * Defines the maximum acceptable value for an input element with type=\"number\".When used with the min and step attributes, lets you control the range and increment (such as only even numbers) that the user can enter into an input field.\\n     */\\n    max: string;\\n    /**\\n     * Sets or retrieves the maximum number of characters that the user can enter in a text control.\\n     */\\n    maxLength: number;\\n    /**\\n     * Defines the minimum acceptable value for an input element with type=\"number\". When used with the max and step attributes, lets you control the range and increment (such as even numbers only) that the user can enter into an input field.\\n     */\\n    min: string;\\n    minLength: number;\\n    /**\\n     * Sets or retrieves the Boolean value indicating whether multiple items can be selected from a list.\\n     */\\n    multiple: boolean;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Gets or sets a string containing a regular expression that the user\\'s input must match.\\n     */\\n    pattern: string;\\n    /**\\n     * Gets or sets a text string that is displayed in an input field as a hint or prompt to users as the format or type of information they need to enter.The text appears in an input field until the user puts focus on the field.\\n     */\\n    placeholder: string;\\n    readOnly: boolean;\\n    /**\\n     * When present, marks an element that can\\'t be submitted without a value.\\n     */\\n    required: boolean;\\n    selectionDirection: string | null;\\n    /**\\n     * Gets or sets the end position or offset of a text selection.\\n     */\\n    selectionEnd: number | null;\\n    /**\\n     * Gets or sets the starting position or offset of a text selection.\\n     */\\n    selectionStart: number | null;\\n    size: number;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    /**\\n     * Defines an increment or jump between values that you want to allow the user to enter. When used with the max and min attributes, lets you control the range and increment (for example, allow only even numbers) that the user can enter into an input field.\\n     */\\n    step: string;\\n    /**\\n     * Returns the content type of the object.\\n     */\\n    type: string;\\n    /**\\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\\n     */\\n    /** @deprecated */\\n    useMap: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Returns the value of the data at the cursor\\'s current position.\\n     */\\n    value: string;\\n    valueAsDate: any;\\n    /**\\n     * Returns the input field value as a number.\\n     */\\n    valueAsNumber: number;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: number;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Makes the selection equal to the current object.\\n     */\\n    select(): void;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    setRangeText(replacement: string): void;\\n    setRangeText(replacement: string, start: number, end: number, selectionMode?: SelectionMode): void;\\n    /**\\n     * Sets the start and end positions of a selection in a text field.\\n     * @param start The offset into the text field for the start of the selection.\\n     * @param end The offset into the text field for the end of the selection.\\n     * @param direction The direction in which the selection is performed.\\n     */\\n    setSelectionRange(start: number, end: number, direction?: \"forward\" | \"backward\" | \"none\"): void;\\n    /**\\n     * Decrements a range input control\\'s value by the value given by the Step attribute. If the optional parameter is used, it will decrement the input control\\'s step value multiplied by the parameter\\'s value.\\n     * @param n Value to decrement the value by.\\n     */\\n    stepDown(n?: number): void;\\n    /**\\n     * Increments a range input control\\'s value by the value given by the Step attribute. If the optional parameter is used, will increment the input control\\'s value by that value.\\n     * @param n Value to increment the value by.\\n     */\\n    stepUp(n?: number): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLInputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLInputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLInputElement: {\\n    prototype: HTMLInputElement;\\n    new(): HTMLInputElement;\\n};\\n\\ninterface HTMLLIElement extends HTMLElement {\\n    /** @deprecated */\\n    type: string;\\n    /**\\n     * Sets or retrieves the value of a list item.\\n     */\\n    value: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLIElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLIElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLIElement: {\\n    prototype: HTMLLIElement;\\n    new(): HTMLLIElement;\\n};\\n\\ninterface HTMLLabelElement extends HTMLElement {\\n    readonly control: HTMLElement | null;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the object to which the given label object is assigned.\\n     */\\n    htmlFor: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLabelElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLabelElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLabelElement: {\\n    prototype: HTMLLabelElement;\\n    new(): HTMLLabelElement;\\n};\\n\\ninterface HTMLLegendElement extends HTMLElement {\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLegendElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLegendElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLegendElement: {\\n    prototype: HTMLLegendElement;\\n    new(): HTMLLegendElement;\\n};\\n\\ninterface HTMLLinkElement extends HTMLElement, LinkStyle {\\n    as: string;\\n    /**\\n     * Sets or retrieves the character set used to encode the object.\\n     */\\n    /** @deprecated */\\n    charset: string;\\n    crossOrigin: string | null;\\n    disabled: boolean;\\n    /**\\n     * Sets or retrieves a destination URL or an anchor point.\\n     */\\n    href: string;\\n    /**\\n     * Sets or retrieves the language code of the object.\\n     */\\n    hreflang: string;\\n    integrity: string;\\n    /**\\n     * Sets or retrieves the media type.\\n     */\\n    media: string;\\n    referrerPolicy: string;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    rel: string;\\n    readonly relList: DOMTokenList;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    /** @deprecated */\\n    rev: string;\\n    readonly sizes: DOMTokenList;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    /** @deprecated */\\n    target: string;\\n    /**\\n     * Sets or retrieves the MIME type of the object.\\n     */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLinkElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLinkElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLinkElement: {\\n    prototype: HTMLLinkElement;\\n    new(): HTMLLinkElement;\\n};\\n\\ninterface HTMLMainElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMainElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMainElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMainElement: {\\n    prototype: HTMLMainElement;\\n    new(): HTMLMainElement;\\n};\\n\\ninterface HTMLMapElement extends HTMLElement {\\n    /**\\n     * Retrieves a collection of the area objects defined for the given map object.\\n     */\\n    readonly areas: HTMLCollection;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMapElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMapElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMapElement: {\\n    prototype: HTMLMapElement;\\n    new(): HTMLMapElement;\\n};\\n\\ninterface HTMLMarqueeElementEventMap extends HTMLElementEventMap {\\n    \"bounce\": Event;\\n    \"finish\": Event;\\n    \"start\": Event;\\n}\\n\\ninterface HTMLMarqueeElement extends HTMLElement {\\n    /** @deprecated */\\n    behavior: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /** @deprecated */\\n    direction: string;\\n    /** @deprecated */\\n    height: string;\\n    /** @deprecated */\\n    hspace: number;\\n    /** @deprecated */\\n    loop: number;\\n    /** @deprecated */\\n    onbounce: ((this: HTMLMarqueeElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onfinish: ((this: HTMLMarqueeElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onstart: ((this: HTMLMarqueeElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    scrollAmount: number;\\n    /** @deprecated */\\n    scrollDelay: number;\\n    /** @deprecated */\\n    trueSpeed: boolean;\\n    /** @deprecated */\\n    vspace: number;\\n    /** @deprecated */\\n    width: string;\\n    /** @deprecated */\\n    start(): void;\\n    /** @deprecated */\\n    stop(): void;\\n    addEventListener<K extends keyof HTMLMarqueeElementEventMap>(type: K, listener: (this: HTMLMarqueeElement, ev: HTMLMarqueeElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLMarqueeElementEventMap>(type: K, listener: (this: HTMLMarqueeElement, ev: HTMLMarqueeElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMarqueeElement: {\\n    prototype: HTMLMarqueeElement;\\n    new(): HTMLMarqueeElement;\\n};\\n\\ninterface HTMLMediaElementEventMap extends HTMLElementEventMap {\\n    \"encrypted\": MediaEncryptedEvent;\\n    \"msneedkey\": Event;\\n}\\n\\ninterface HTMLMediaElement extends HTMLElement {\\n    /**\\n     * Returns an AudioTrackList object with the audio tracks for a given video element.\\n     */\\n    readonly audioTracks: AudioTrackList;\\n    /**\\n     * Gets or sets a value that indicates whether to start playing the media automatically.\\n     */\\n    autoplay: boolean;\\n    /**\\n     * Gets a collection of buffered time ranges.\\n     */\\n    readonly buffered: TimeRanges;\\n    /**\\n     * Gets or sets a flag that indicates whether the client provides a set of controls for the media (in case the developer does not include controls for the player).\\n     */\\n    controls: boolean;\\n    crossOrigin: string | null;\\n    /**\\n     * Gets the address or URL of the current media resource that is selected by IHTMLMediaElement.\\n     */\\n    readonly currentSrc: string;\\n    /**\\n     * Gets or sets the current playback position, in seconds.\\n     */\\n    currentTime: number;\\n    defaultMuted: boolean;\\n    /**\\n     * Gets or sets the default playback rate when the user is not using fast forward or reverse for a video or audio resource.\\n     */\\n    defaultPlaybackRate: number;\\n    /**\\n     * Returns the duration in seconds of the current media resource. A NaN value is returned if duration is not available, or Infinity if the media resource is streaming.\\n     */\\n    readonly duration: number;\\n    /**\\n     * Gets information about whether the playback has ended or not.\\n     */\\n    readonly ended: boolean;\\n    /**\\n     * Returns an object representing the current error state of the audio or video element.\\n     */\\n    readonly error: MediaError | null;\\n    /**\\n     * Gets or sets a flag to specify whether playback should restart after it completes.\\n     */\\n    loop: boolean;\\n    readonly mediaKeys: MediaKeys | null;\\n    /**\\n     * Specifies the purpose of the audio or video media, such as background audio or alerts.\\n     */\\n    msAudioCategory: string;\\n    /**\\n     * Specifies the output device id that the audio will be sent to.\\n     */\\n    msAudioDeviceType: string;\\n    readonly msGraphicsTrustStatus: MSGraphicsTrust;\\n    /**\\n     * Gets the MSMediaKeys object, which is used for decrypting media data, that is associated with this media element.\\n     */\\n    /** @deprecated */\\n    readonly msKeys: MSMediaKeys;\\n    /**\\n     * Gets or sets whether the DLNA PlayTo device is available.\\n     */\\n    msPlayToDisabled: boolean;\\n    /**\\n     * Gets or sets the path to the preferred media source. This enables the Play To target device to stream the media content, which can be DRM protected, from a different location, such as a cloud media server.\\n     */\\n    msPlayToPreferredSourceUri: string;\\n    /**\\n     * Gets or sets the primary DLNA PlayTo device.\\n     */\\n    msPlayToPrimary: boolean;\\n    /**\\n     * Gets the source associated with the media element for use by the PlayToManager.\\n     */\\n    readonly msPlayToSource: any;\\n    /**\\n     * Specifies whether or not to enable low-latency playback on the media element.\\n     */\\n    msRealTime: boolean;\\n    /**\\n     * Gets or sets a flag that indicates whether the audio (either audio or the audio track on video media) is muted.\\n     */\\n    muted: boolean;\\n    /**\\n     * Gets the current network activity for the element.\\n     */\\n    readonly networkState: number;\\n    onencrypted: ((this: HTMLMediaElement, ev: MediaEncryptedEvent) => any) | null;\\n    /** @deprecated */\\n    onmsneedkey: ((this: HTMLMediaElement, ev: Event) => any) | null;\\n    /**\\n     * Gets a flag that specifies whether playback is paused.\\n     */\\n    readonly paused: boolean;\\n    /**\\n     * Gets or sets the current rate of speed for the media resource to play. This speed is expressed as a multiple of the normal speed of the media resource.\\n     */\\n    playbackRate: number;\\n    /**\\n     * Gets TimeRanges for the current media resource that has been played.\\n     */\\n    readonly played: TimeRanges;\\n    /**\\n     * Gets or sets the current playback position, in seconds.\\n     */\\n    preload: string;\\n    readonly readyState: number;\\n    /**\\n     * Returns a TimeRanges object that represents the ranges of the current media resource that can be seeked.\\n     */\\n    readonly seekable: TimeRanges;\\n    /**\\n     * Gets a flag that indicates whether the the client is currently moving to a new playback position in the media resource.\\n     */\\n    readonly seeking: boolean;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    srcObject: MediaStream | MediaSource | Blob | null;\\n    readonly textTracks: TextTrackList;\\n    readonly videoTracks: VideoTrackList;\\n    /**\\n     * Gets or sets the volume level for audio portions of the media element.\\n     */\\n    volume: number;\\n    addTextTrack(kind: TextTrackKind, label?: string, language?: string): TextTrack;\\n    /**\\n     * Returns a string that specifies whether the client can play a given media resource type.\\n     */\\n    canPlayType(type: string): CanPlayTypeResult;\\n    /**\\n     * Resets the audio or video object and loads a new media resource.\\n     */\\n    load(): void;\\n    /**\\n     * Clears all effects from the media pipeline.\\n     */\\n    msClearEffects(): void;\\n    msGetAsCastingSource(): any;\\n    /**\\n     * Inserts the specified audio effect into media pipeline.\\n     */\\n    msInsertAudioEffect(activatableClassId: string, effectRequired: boolean, config?: any): void;\\n    /** @deprecated */\\n    msSetMediaKeys(mediaKeys: MSMediaKeys): void;\\n    /**\\n     * Specifies the media protection manager for a given media pipeline.\\n     */\\n    msSetMediaProtectionManager(mediaProtectionManager?: any): void;\\n    /**\\n     * Pauses the current playback and sets paused to TRUE. This can be used to test whether the media is playing or paused. You can also use the pause or play events to tell whether the media is playing or not.\\n     */\\n    pause(): void;\\n    /**\\n     * Loads and starts playback of a media resource.\\n     */\\n    play(): Promise<void>;\\n    setMediaKeys(mediaKeys: MediaKeys | null): Promise<void>;\\n    readonly HAVE_CURRENT_DATA: number;\\n    readonly HAVE_ENOUGH_DATA: number;\\n    readonly HAVE_FUTURE_DATA: number;\\n    readonly HAVE_METADATA: number;\\n    readonly HAVE_NOTHING: number;\\n    readonly NETWORK_EMPTY: number;\\n    readonly NETWORK_IDLE: number;\\n    readonly NETWORK_LOADING: number;\\n    readonly NETWORK_NO_SOURCE: number;\\n    addEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLMediaElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLMediaElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMediaElement: {\\n    prototype: HTMLMediaElement;\\n    new(): HTMLMediaElement;\\n    readonly HAVE_CURRENT_DATA: number;\\n    readonly HAVE_ENOUGH_DATA: number;\\n    readonly HAVE_FUTURE_DATA: number;\\n    readonly HAVE_METADATA: number;\\n    readonly HAVE_NOTHING: number;\\n    readonly NETWORK_EMPTY: number;\\n    readonly NETWORK_IDLE: number;\\n    readonly NETWORK_LOADING: number;\\n    readonly NETWORK_NO_SOURCE: number;\\n};\\n\\ninterface HTMLMenuElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMenuElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMenuElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMenuElement: {\\n    prototype: HTMLMenuElement;\\n    new(): HTMLMenuElement;\\n};\\n\\ninterface HTMLMetaElement extends HTMLElement {\\n    /**\\n     * Gets or sets meta-information to associate with httpEquiv or name.\\n     */\\n    content: string;\\n    /**\\n     * Gets or sets information used to bind the value of a content attribute of a meta element to an HTTP response header.\\n     */\\n    httpEquiv: string;\\n    /**\\n     * Sets or retrieves the value specified in the content attribute of the meta object.\\n     */\\n    name: string;\\n    /**\\n     * Sets or retrieves a scheme to be used in interpreting the value of a property specified for the object.\\n     */\\n    /** @deprecated */\\n    scheme: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMetaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMetaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMetaElement: {\\n    prototype: HTMLMetaElement;\\n    new(): HTMLMetaElement;\\n};\\n\\ninterface HTMLMeterElement extends HTMLElement {\\n    high: number;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    low: number;\\n    max: number;\\n    min: number;\\n    optimum: number;\\n    value: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMeterElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMeterElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMeterElement: {\\n    prototype: HTMLMeterElement;\\n    new(): HTMLMeterElement;\\n};\\n\\ninterface HTMLModElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves reference information about the object.\\n     */\\n    cite: string;\\n    /**\\n     * Sets or retrieves the date and time of a modification to the object.\\n     */\\n    dateTime: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLModElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLModElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLModElement: {\\n    prototype: HTMLModElement;\\n    new(): HTMLModElement;\\n};\\n\\ninterface HTMLOListElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    reversed: boolean;\\n    /**\\n     * The starting number.\\n     */\\n    start: number;\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOListElement: {\\n    prototype: HTMLOListElement;\\n    new(): HTMLOListElement;\\n};\\n\\ninterface HTMLObjectElement extends HTMLElement, GetSVGDocument {\\n    /**\\n     * Retrieves a string of the URL where the object tag can be found. This is often the href of the document that the object is in, or the value set by a base element.\\n     */\\n    readonly BaseHref: string;\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a character string that can be used to implement your own archive functionality for the object.\\n     */\\n    /** @deprecated */\\n    archive: string;\\n    /** @deprecated */\\n    border: string;\\n    /**\\n     * Sets or retrieves the URL of the file containing the compiled Java class.\\n     */\\n    /** @deprecated */\\n    code: string;\\n    /**\\n     * Sets or retrieves the URL of the component.\\n     */\\n    /** @deprecated */\\n    codeBase: string;\\n    /**\\n     * Sets or retrieves the Internet media type for the code associated with the object.\\n     */\\n    /** @deprecated */\\n    codeType: string;\\n    /**\\n     * Retrieves the document object of the page or frame.\\n     */\\n    readonly contentDocument: Document | null;\\n    /**\\n     * Sets or retrieves the URL that references the data of the object.\\n     */\\n    data: string;\\n    /** @deprecated */\\n    declare: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: string;\\n    /** @deprecated */\\n    hspace: number;\\n    /**\\n     * Gets or sets whether the DLNA PlayTo device is available.\\n     */\\n    msPlayToDisabled: boolean;\\n    /**\\n     * Gets or sets the path to the preferred media source. This enables the Play To target device to stream the media content, which can be DRM protected, from a different location, such as a cloud media server.\\n     */\\n    msPlayToPreferredSourceUri: string;\\n    /**\\n     * Gets or sets the primary DLNA PlayTo device.\\n     */\\n    msPlayToPrimary: boolean;\\n    /**\\n     * Gets the source associated with the media element for use by the PlayToManager.\\n     */\\n    readonly msPlayToSource: any;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    readonly readyState: number;\\n    /**\\n     * Sets or retrieves a message to be displayed while an object is loading.\\n     */\\n    /** @deprecated */\\n    standby: string;\\n    /**\\n     * Sets or retrieves the MIME type of the object.\\n     */\\n    type: string;\\n    typemustmatch: boolean;\\n    /**\\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\\n     */\\n    useMap: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /** @deprecated */\\n    vspace: number;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLObjectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLObjectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLObjectElement: {\\n    prototype: HTMLObjectElement;\\n    new(): HTMLObjectElement;\\n};\\n\\ninterface HTMLOptGroupElement extends HTMLElement {\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves a value that you can use to implement your own label functionality for the object.\\n     */\\n    label: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptGroupElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptGroupElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOptGroupElement: {\\n    prototype: HTMLOptGroupElement;\\n    new(): HTMLOptGroupElement;\\n};\\n\\ninterface HTMLOptionElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the status of an option.\\n     */\\n    defaultSelected: boolean;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the ordinal position of an option in a list box.\\n     */\\n    readonly index: number;\\n    /**\\n     * Sets or retrieves a value that you can use to implement your own label functionality for the object.\\n     */\\n    label: string;\\n    /**\\n     * Sets or retrieves whether the option in the list box is the default item.\\n     */\\n    selected: boolean;\\n    /**\\n     * Sets or retrieves the text string specified by the option tag.\\n     */\\n    text: string;\\n    /**\\n     * Sets or retrieves the value which is returned to the server when the form control is submitted.\\n     */\\n    value: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOptionElement: {\\n    prototype: HTMLOptionElement;\\n    new(): HTMLOptionElement;\\n};\\n\\ninterface HTMLOptionsCollection extends HTMLCollectionOf<HTMLOptionElement> {\\n    /**\\n     * Returns the number of elements in the collection.\\n     * When set to a smaller number, truncates the number of option elements in the corresponding container.\\n     * When set to a greater number, adds new blank option elements to that container.\\n     */\\n    length: number;\\n    /**\\n     * Returns the index of the first selected item, if any, or −1 if there is no selected\\n     * item.\\n     * Can be set, to change the selection.\\n     */\\n    selectedIndex: number;\\n    /**\\n     * Inserts element before the node given by before.\\n     * The before argument can be a number, in which case element is inserted before the item with that number, or an element from the\\n     * collection, in which case element is inserted before that element.\\n     * If before is omitted, null, or a number out of range, then element will be added at the end of the list.\\n     * This method will throw a \"HierarchyRequestError\" DOMException if\\n     * element is an ancestor of the element into which it is to be inserted.\\n     */\\n    add(element: HTMLOptionElement | HTMLOptGroupElement, before?: HTMLElement | number | null): void;\\n    /**\\n     * Removes the item with index index from the collection.\\n     */\\n    remove(index: number): void;\\n}\\n\\ndeclare var HTMLOptionsCollection: {\\n    prototype: HTMLOptionsCollection;\\n    new(): HTMLOptionsCollection;\\n};\\n\\ninterface HTMLOrSVGElement {\\n    readonly dataset: DOMStringMap;\\n    nonce: string;\\n    tabIndex: number;\\n    blur(): void;\\n    focus(options?: FocusOptions): void;\\n}\\n\\ninterface HTMLOutputElement extends HTMLElement {\\n    defaultValue: string;\\n    readonly form: HTMLFormElement | null;\\n    readonly htmlFor: DOMTokenList;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    name: string;\\n    readonly type: string;\\n    readonly validationMessage: string;\\n    readonly validity: ValidityState;\\n    value: string;\\n    readonly willValidate: boolean;\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOutputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOutputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOutputElement: {\\n    prototype: HTMLOutputElement;\\n    new(): HTMLOutputElement;\\n};\\n\\ninterface HTMLParagraphElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParagraphElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParagraphElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLParagraphElement: {\\n    prototype: HTMLParagraphElement;\\n    new(): HTMLParagraphElement;\\n};\\n\\ninterface HTMLParamElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the name of an input parameter for an element.\\n     */\\n    name: string;\\n    /**\\n     * Sets or retrieves the content type of the resource designated by the value attribute.\\n     */\\n    /** @deprecated */\\n    type: string;\\n    /**\\n     * Sets or retrieves the value of an input parameter for an element.\\n     */\\n    value: string;\\n    /**\\n     * Sets or retrieves the data type of the value attribute.\\n     */\\n    /** @deprecated */\\n    valueType: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParamElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParamElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLParamElement: {\\n    prototype: HTMLParamElement;\\n    new(): HTMLParamElement;\\n};\\n\\ninterface HTMLPictureElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPictureElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPictureElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLPictureElement: {\\n    prototype: HTMLPictureElement;\\n    new(): HTMLPictureElement;\\n};\\n\\ninterface HTMLPreElement extends HTMLElement {\\n    /**\\n     * Sets or gets a value that you can use to implement your own width functionality for the object.\\n     */\\n    /** @deprecated */\\n    width: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPreElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPreElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLPreElement: {\\n    prototype: HTMLPreElement;\\n    new(): HTMLPreElement;\\n};\\n\\ninterface HTMLProgressElement extends HTMLElement {\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Defines the maximum, or \"done\" value for a progress element.\\n     */\\n    max: number;\\n    /**\\n     * Returns the quotient of value/max when the value attribute is set (determinate progress bar), or -1 when the value attribute is missing (indeterminate progress bar).\\n     */\\n    readonly position: number;\\n    /**\\n     * Sets or gets the current value of a progress element. The value must be a non-negative number between 0 and the max value.\\n     */\\n    value: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLProgressElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLProgressElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLProgressElement: {\\n    prototype: HTMLProgressElement;\\n    new(): HTMLProgressElement;\\n};\\n\\ninterface HTMLQuoteElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves reference information about the object.\\n     */\\n    cite: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLQuoteElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLQuoteElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLQuoteElement: {\\n    prototype: HTMLQuoteElement;\\n    new(): HTMLQuoteElement;\\n};\\n\\ninterface HTMLScriptElement extends HTMLElement {\\n    async: boolean;\\n    /**\\n     * Sets or retrieves the character set used to encode the object.\\n     */\\n    /** @deprecated */\\n    charset: string;\\n    crossOrigin: string | null;\\n    /**\\n     * Sets or retrieves the status of the script.\\n     */\\n    defer: boolean;\\n    /**\\n     * Sets or retrieves the event for which the script is written.\\n     */\\n    /** @deprecated */\\n    event: string;\\n    /**\\n     * Sets or retrieves the object that is bound to the event script.\\n     */\\n    /** @deprecated */\\n    htmlFor: string;\\n    integrity: string;\\n    noModule: boolean;\\n    referrerPolicy: string;\\n    /**\\n     * Retrieves the URL to an external file that contains the source code or data.\\n     */\\n    src: string;\\n    /**\\n     * Retrieves or sets the text of the object as a string.\\n     */\\n    text: string;\\n    /**\\n     * Sets or retrieves the MIME type for the associated scripting engine.\\n     */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLScriptElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLScriptElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLScriptElement: {\\n    prototype: HTMLScriptElement;\\n    new(): HTMLScriptElement;\\n};\\n\\ninterface HTMLSelectElement extends HTMLElement {\\n    autocomplete: string;\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Sets or retrieves the number of objects in a collection.\\n     */\\n    length: number;\\n    /**\\n     * Sets or retrieves the Boolean value indicating whether multiple items can be selected from a list.\\n     */\\n    multiple: boolean;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    readonly options: HTMLOptionsCollection;\\n    /**\\n     * When present, marks an element that can\\'t be submitted without a value.\\n     */\\n    required: boolean;\\n    /**\\n     * Sets or retrieves the index of the selected option in a select object.\\n     */\\n    selectedIndex: number;\\n    readonly selectedOptions: HTMLCollectionOf<HTMLOptionElement>;\\n    /**\\n     * Sets or retrieves the number of rows in the list box.\\n     */\\n    size: number;\\n    /**\\n     * Retrieves the type of select control based on the value of the MULTIPLE attribute.\\n     */\\n    readonly type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Sets or retrieves the value which is returned to the server when the form control is submitted.\\n     */\\n    value: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Adds an element to the areas, controlRange, or options collection.\\n     * @param element Variant of type Number that specifies the index position in the collection where the element is placed. If no value is given, the method places the element at the end of the collection.\\n     * @param before Variant of type Object that specifies an element to insert before, or null to append the object to the collection.\\n     */\\n    add(element: HTMLOptionElement | HTMLOptGroupElement, before?: HTMLElement | number | null): void;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    /**\\n     * Retrieves a select object or an object from an options collection.\\n     * @param name Variant of type Number or String that specifies the object or collection to retrieve. If this parameter is an integer, it is the zero-based index of the object. If this parameter is a string, all objects with matching name or id properties are retrieved, and a collection is returned if more than one match is made.\\n     * @param index Variant of type Number that specifies the zero-based index of the object to retrieve when a collection is returned.\\n     */\\n    item(index: number): Element | null;\\n    /**\\n     * Retrieves a select object or an object from an options collection.\\n     * @param namedItem A String that specifies the name or id property of the object to retrieve. A collection is returned if more than one match is made.\\n     */\\n    namedItem(name: string): HTMLOptionElement | null;\\n    /**\\n     * Removes an element from the collection.\\n     * @param index Number that specifies the zero-based index of the element to remove from the collection.\\n     */\\n    remove(): void;\\n    remove(index: number): void;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSelectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSelectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [name: number]: HTMLOptionElement | HTMLOptGroupElement;\\n}\\n\\ndeclare var HTMLSelectElement: {\\n    prototype: HTMLSelectElement;\\n    new(): HTMLSelectElement;\\n};\\n\\ninterface HTMLSlotElement extends HTMLElement {\\n    name: string;\\n    assignedElements(options?: AssignedNodesOptions): Element[];\\n    assignedNodes(options?: AssignedNodesOptions): Node[];\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSlotElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSlotElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLSlotElement: {\\n    prototype: HTMLSlotElement;\\n    new(): HTMLSlotElement;\\n};\\n\\ninterface HTMLSourceElement extends HTMLElement {\\n    /**\\n     * Gets or sets the intended media type of the media source.\\n     */\\n    media: string;\\n    sizes: string;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    srcset: string;\\n    /**\\n     * Gets or sets the MIME type of a media resource.\\n     */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSourceElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSourceElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLSourceElement: {\\n    prototype: HTMLSourceElement;\\n    new(): HTMLSourceElement;\\n};\\n\\ninterface HTMLSpanElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSpanElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSpanElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLSpanElement: {\\n    prototype: HTMLSpanElement;\\n    new(): HTMLSpanElement;\\n};\\n\\ninterface HTMLStyleElement extends HTMLElement, LinkStyle {\\n    /**\\n     * Sets or retrieves the media type.\\n     */\\n    media: string;\\n    /**\\n     * Retrieves the CSS language in which the style sheet is written.\\n     */\\n    /** @deprecated */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLStyleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLStyleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLStyleElement: {\\n    prototype: HTMLStyleElement;\\n    new(): HTMLStyleElement;\\n};\\n\\ninterface HTMLTableCaptionElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the alignment of the caption or legend.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCaptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCaptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableCaptionElement: {\\n    prototype: HTMLTableCaptionElement;\\n    new(): HTMLTableCaptionElement;\\n};\\n\\ninterface HTMLTableCellElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves abbreviated text for the object.\\n     */\\n    abbr: string;\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a comma-delimited list of conceptual categories associated with the object.\\n     */\\n    /** @deprecated */\\n    axis: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Retrieves the position of the object in the cells collection of a row.\\n     */\\n    readonly cellIndex: number;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Sets or retrieves the number columns in the table that the object should span.\\n     */\\n    colSpan: number;\\n    /**\\n     * Sets or retrieves a list of header cells that provide information for the object.\\n     */\\n    headers: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    /** @deprecated */\\n    height: string;\\n    /**\\n     * Sets or retrieves whether the browser automatically performs wordwrap.\\n     */\\n    /** @deprecated */\\n    noWrap: boolean;\\n    /**\\n     * Sets or retrieves how many rows in a table the cell should span.\\n     */\\n    rowSpan: number;\\n    /**\\n     * Sets or retrieves the group of cells in a table to which the object\\'s information applies.\\n     */\\n    scope: string;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableCellElement: {\\n    prototype: HTMLTableCellElement;\\n    new(): HTMLTableCellElement;\\n};\\n\\ninterface HTMLTableColElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the alignment of the object relative to the display or table.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Sets or retrieves the number of columns in the group.\\n     */\\n    span: number;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableColElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableColElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableColElement: {\\n    prototype: HTMLTableColElement;\\n    new(): HTMLTableColElement;\\n};\\n\\ninterface HTMLTableDataCellElement extends HTMLTableCellElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableDataCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableDataCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableDataCellElement: {\\n    prototype: HTMLTableDataCellElement;\\n    new(): HTMLTableDataCellElement;\\n};\\n\\ninterface HTMLTableElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a value that indicates the table alignment.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Sets or retrieves the width of the border to draw around the object.\\n     */\\n    /** @deprecated */\\n    border: string;\\n    /**\\n     * Retrieves the caption object of a table.\\n     */\\n    caption: HTMLTableCaptionElement | null;\\n    /**\\n     * Sets or retrieves the amount of space between the border of the cell and the content of the cell.\\n     */\\n    /** @deprecated */\\n    cellPadding: string;\\n    /**\\n     * Sets or retrieves the amount of space between cells in a table.\\n     */\\n    /** @deprecated */\\n    cellSpacing: string;\\n    /**\\n     * Sets or retrieves the way the border frame around the table is displayed.\\n     */\\n    /** @deprecated */\\n    frame: string;\\n    /**\\n     * Sets or retrieves the number of horizontal rows contained in the object.\\n     */\\n    readonly rows: HTMLCollectionOf<HTMLTableRowElement>;\\n    /**\\n     * Sets or retrieves which dividing lines (inner borders) are displayed.\\n     */\\n    /** @deprecated */\\n    rules: string;\\n    /**\\n     * Sets or retrieves a description and/or structure of the object.\\n     */\\n    /** @deprecated */\\n    summary: string;\\n    /**\\n     * Retrieves a collection of all tBody objects in the table. Objects in this collection are in source order.\\n     */\\n    readonly tBodies: HTMLCollectionOf<HTMLTableSectionElement>;\\n    /**\\n     * Retrieves the tFoot object of the table.\\n     */\\n    tFoot: HTMLTableSectionElement | null;\\n    /**\\n     * Retrieves the tHead object of the table.\\n     */\\n    tHead: HTMLTableSectionElement | null;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    /**\\n     * Creates an empty caption element in the table.\\n     */\\n    createCaption(): HTMLTableCaptionElement;\\n    /**\\n     * Creates an empty tBody element in the table.\\n     */\\n    createTBody(): HTMLTableSectionElement;\\n    /**\\n     * Creates an empty tFoot element in the table.\\n     */\\n    createTFoot(): HTMLTableSectionElement;\\n    /**\\n     * Returns the tHead element object if successful, or null otherwise.\\n     */\\n    createTHead(): HTMLTableSectionElement;\\n    /**\\n     * Deletes the caption element and its contents from the table.\\n     */\\n    deleteCaption(): void;\\n    /**\\n     * Removes the specified row (tr) from the element and from the rows collection.\\n     * @param index Number that specifies the zero-based position in the rows collection of the row to remove.\\n     */\\n    deleteRow(index: number): void;\\n    /**\\n     * Deletes the tFoot element and its contents from the table.\\n     */\\n    deleteTFoot(): void;\\n    /**\\n     * Deletes the tHead element and its contents from the table.\\n     */\\n    deleteTHead(): void;\\n    /**\\n     * Creates a new row (tr) in the table, and adds the row to the rows collection.\\n     * @param index Number that specifies where to insert the row in the rows collection. The default value is -1, which appends the new row to the end of the rows collection.\\n     */\\n    insertRow(index?: number): HTMLTableRowElement;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableElement: {\\n    prototype: HTMLTableElement;\\n    new(): HTMLTableElement;\\n};\\n\\ninterface HTMLTableHeaderCellElement extends HTMLTableCellElement {\\n    scope: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableHeaderCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableHeaderCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableHeaderCellElement: {\\n    prototype: HTMLTableHeaderCellElement;\\n    new(): HTMLTableHeaderCellElement;\\n};\\n\\ninterface HTMLTableRowElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Retrieves a collection of all cells in the table row.\\n     */\\n    readonly cells: HTMLCollectionOf<HTMLTableDataCellElement | HTMLTableHeaderCellElement>;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Retrieves the position of the object in the rows collection for the table.\\n     */\\n    readonly rowIndex: number;\\n    /**\\n     * Retrieves the position of the object in the collection.\\n     */\\n    readonly sectionRowIndex: number;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Removes the specified cell from the table row, as well as from the cells collection.\\n     * @param index Number that specifies the zero-based position of the cell to remove from the table row. If no value is provided, the last cell in the cells collection is deleted.\\n     */\\n    deleteCell(index: number): void;\\n    /**\\n     * Creates a new cell in the table row, and adds the cell to the cells collection.\\n     * @param index Number that specifies where to insert the cell in the tr. The default value is -1, which appends the new cell to the end of the cells collection.\\n     */\\n    insertCell(index?: number): HTMLTableDataCellElement;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableRowElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableRowElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableRowElement: {\\n    prototype: HTMLTableRowElement;\\n    new(): HTMLTableRowElement;\\n};\\n\\ninterface HTMLTableSectionElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a value that indicates the table alignment.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Sets or retrieves the number of horizontal rows contained in the object.\\n     */\\n    readonly rows: HTMLCollectionOf<HTMLTableRowElement>;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Removes the specified row (tr) from the element and from the rows collection.\\n     * @param index Number that specifies the zero-based position in the rows collection of the row to remove.\\n     */\\n    deleteRow(index: number): void;\\n    /**\\n     * Creates a new row (tr) in the table, and adds the row to the rows collection.\\n     * @param index Number that specifies where to insert the row in the rows collection. The default value is -1, which appends the new row to the end of the rows collection.\\n     */\\n    insertRow(index?: number): HTMLTableRowElement;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableSectionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableSectionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableSectionElement: {\\n    prototype: HTMLTableSectionElement;\\n    new(): HTMLTableSectionElement;\\n};\\n\\ninterface HTMLTemplateElement extends HTMLElement {\\n    readonly content: DocumentFragment;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTemplateElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTemplateElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTemplateElement: {\\n    prototype: HTMLTemplateElement;\\n    new(): HTMLTemplateElement;\\n};\\n\\ninterface HTMLTextAreaElement extends HTMLElement {\\n    autocomplete: string;\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    cols: number;\\n    /**\\n     * Sets or retrieves the initial contents of the object.\\n     */\\n    defaultValue: string;\\n    dirName: string;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Sets or retrieves the maximum number of characters that the user can enter in a text control.\\n     */\\n    maxLength: number;\\n    minLength: number;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Gets or sets a text string that is displayed in an input field as a hint or prompt to users as the format or type of information they need to enter.The text appears in an input field until the user puts focus on the field.\\n     */\\n    placeholder: string;\\n    /**\\n     * Sets or retrieves the value indicated whether the content of the object is read-only.\\n     */\\n    readOnly: boolean;\\n    /**\\n     * When present, marks an element that can\\'t be submitted without a value.\\n     */\\n    required: boolean;\\n    /**\\n     * Sets or retrieves the number of horizontal rows contained in the object.\\n     */\\n    rows: number;\\n    selectionDirection: string;\\n    /**\\n     * Gets or sets the end position or offset of a text selection.\\n     */\\n    selectionEnd: number;\\n    /**\\n     * Gets or sets the starting position or offset of a text selection.\\n     */\\n    selectionStart: number;\\n    readonly textLength: number;\\n    /**\\n     * Retrieves the type of control.\\n     */\\n    readonly type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Retrieves or sets the text in the entry field of the textArea element.\\n     */\\n    value: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Sets or retrieves how to handle wordwrapping in the object.\\n     */\\n    wrap: string;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Highlights the input area of a form element.\\n     */\\n    select(): void;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    setRangeText(replacement: string): void;\\n    setRangeText(replacement: string, start: number, end: number, selectionMode?: SelectionMode): void;\\n    /**\\n     * Sets the start and end positions of a selection in a text field.\\n     * @param start The offset into the text field for the start of the selection.\\n     * @param end The offset into the text field for the end of the selection.\\n     * @param direction The direction in which the selection is performed.\\n     */\\n    setSelectionRange(start: number, end: number, direction?: \"forward\" | \"backward\" | \"none\"): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTextAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTextAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTextAreaElement: {\\n    prototype: HTMLTextAreaElement;\\n    new(): HTMLTextAreaElement;\\n};\\n\\ninterface HTMLTimeElement extends HTMLElement {\\n    dateTime: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTimeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTimeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTimeElement: {\\n    prototype: HTMLTimeElement;\\n    new(): HTMLTimeElement;\\n};\\n\\ninterface HTMLTitleElement extends HTMLElement {\\n    /**\\n     * Retrieves or sets the text of the object as a string.\\n     */\\n    text: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTitleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTitleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTitleElement: {\\n    prototype: HTMLTitleElement;\\n    new(): HTMLTitleElement;\\n};\\n\\ninterface HTMLTrackElement extends HTMLElement {\\n    default: boolean;\\n    kind: string;\\n    label: string;\\n    readonly readyState: number;\\n    src: string;\\n    srclang: string;\\n    readonly track: TextTrack;\\n    readonly ERROR: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTrackElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTrackElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTrackElement: {\\n    prototype: HTMLTrackElement;\\n    new(): HTMLTrackElement;\\n    readonly ERROR: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n};\\n\\ninterface HTMLUListElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    /** @deprecated */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLUListElement: {\\n    prototype: HTMLUListElement;\\n    new(): HTMLUListElement;\\n};\\n\\ninterface HTMLUnknownElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUnknownElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUnknownElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLUnknownElement: {\\n    prototype: HTMLUnknownElement;\\n    new(): HTMLUnknownElement;\\n};\\n\\ninterface HTMLVideoElementEventMap extends HTMLMediaElementEventMap {\\n    \"MSVideoFormatChanged\": Event;\\n    \"MSVideoFrameStepCompleted\": Event;\\n    \"MSVideoOptimalLayoutChanged\": Event;\\n}\\n\\ninterface HTMLVideoElement extends HTMLMediaElement {\\n    /**\\n     * Gets or sets the height of the video element.\\n     */\\n    height: number;\\n    msHorizontalMirror: boolean;\\n    readonly msIsLayoutOptimalForPlayback: boolean;\\n    readonly msIsStereo3D: boolean;\\n    msStereo3DPackingMode: string;\\n    msStereo3DRenderMode: string;\\n    msZoom: boolean;\\n    onMSVideoFormatChanged: ((this: HTMLVideoElement, ev: Event) => any) | null;\\n    onMSVideoFrameStepCompleted: ((this: HTMLVideoElement, ev: Event) => any) | null;\\n    onMSVideoOptimalLayoutChanged: ((this: HTMLVideoElement, ev: Event) => any) | null;\\n    /**\\n     * Gets or sets a URL of an image to display, for example, like a movie poster. This can be a still frame from the video, or another image if no video data is available.\\n     */\\n    poster: string;\\n    /**\\n     * Gets the intrinsic height of a video in CSS pixels, or zero if the dimensions are not known.\\n     */\\n    readonly videoHeight: number;\\n    /**\\n     * Gets the intrinsic width of a video in CSS pixels, or zero if the dimensions are not known.\\n     */\\n    readonly videoWidth: number;\\n    readonly webkitDisplayingFullscreen: boolean;\\n    readonly webkitSupportsFullscreen: boolean;\\n    /**\\n     * Gets or sets the width of the video element.\\n     */\\n    width: number;\\n    getVideoPlaybackQuality(): VideoPlaybackQuality;\\n    msFrameStep(forward: boolean): void;\\n    msInsertVideoEffect(activatableClassId: string, effectRequired: boolean, config?: any): void;\\n    msSetVideoRectangle(left: number, top: number, right: number, bottom: number): void;\\n    webkitEnterFullScreen(): void;\\n    webkitEnterFullscreen(): void;\\n    webkitExitFullScreen(): void;\\n    webkitExitFullscreen(): void;\\n    addEventListener<K extends keyof HTMLVideoElementEventMap>(type: K, listener: (this: HTMLVideoElement, ev: HTMLVideoElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLVideoElementEventMap>(type: K, listener: (this: HTMLVideoElement, ev: HTMLVideoElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLVideoElement: {\\n    prototype: HTMLVideoElement;\\n    new(): HTMLVideoElement;\\n};\\n\\ninterface HashChangeEvent extends Event {\\n    readonly newURL: string;\\n    readonly oldURL: string;\\n}\\n\\ndeclare var HashChangeEvent: {\\n    prototype: HashChangeEvent;\\n    new(type: string, eventInitDict?: HashChangeEventInit): HashChangeEvent;\\n};\\n\\ninterface Headers {\\n    append(name: string, value: string): void;\\n    delete(name: string): void;\\n    get(name: string): string | null;\\n    has(name: string): boolean;\\n    set(name: string, value: string): void;\\n    forEach(callbackfn: (value: string, key: string, parent: Headers) => void, thisArg?: any): void;\\n}\\n\\ndeclare var Headers: {\\n    prototype: Headers;\\n    new(init?: HeadersInit): Headers;\\n};\\n\\ninterface History {\\n    readonly length: number;\\n    scrollRestoration: ScrollRestoration;\\n    readonly state: any;\\n    back(): void;\\n    forward(): void;\\n    go(delta?: number): void;\\n    pushState(data: any, title: string, url?: string | null): void;\\n    replaceState(data: any, title: string, url?: string | null): void;\\n}\\n\\ndeclare var History: {\\n    prototype: History;\\n    new(): History;\\n};\\n\\ninterface HkdfCtrParams extends Algorithm {\\n    context: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    hash: string | Algorithm;\\n    label: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface IDBArrayKey extends Array<IDBValidKey> {\\n}\\n\\ninterface IDBCursor {\\n    /**\\n     * Returns the direction (\"next\", \"nextunique\", \"prev\" or \"prevunique\")\\n     * of the cursor.\\n     */\\n    readonly direction: IDBCursorDirection;\\n    /**\\n     * Returns the key of the cursor.\\n     * Throws a \"InvalidStateError\" DOMException if the cursor is advancing or is finished.\\n     */\\n    readonly key: IDBValidKey | IDBKeyRange;\\n    /**\\n     * Returns the effective key of the cursor.\\n     * Throws a \"InvalidStateError\" DOMException if the cursor is advancing or is finished.\\n     */\\n    readonly primaryKey: IDBValidKey | IDBKeyRange;\\n    /**\\n     * Returns the IDBObjectStore or IDBIndex the cursor was opened from.\\n     */\\n    readonly source: IDBObjectStore | IDBIndex;\\n    /**\\n     * Advances the cursor through the next count records in\\n     * range.\\n     */\\n    advance(count: number): void;\\n    /**\\n     * Advances the cursor to the next record in range matching or\\n     * after key.\\n     */\\n    continue(key?: IDBValidKey | IDBKeyRange): void;\\n    /**\\n     * Advances the cursor to the next record in range matching\\n     * or after key and primaryKey. Throws an \"InvalidAccessError\" DOMException if the source is not an index.\\n     */\\n    continuePrimaryKey(key: IDBValidKey | IDBKeyRange, primaryKey: IDBValidKey | IDBKeyRange): void;\\n    /**\\n     * Delete the record pointed at by the cursor with a new value.\\n     * If successful, request\\'s result will be undefined.\\n     */\\n    delete(): IDBRequest<undefined>;\\n    /**\\n     * Updated the record pointed at by the cursor with a new value.\\n     * Throws a \"DataError\" DOMException if the effective object store uses in-line keys and the key would have changed.\\n     * If successful, request\\'s result will be the record\\'s key.\\n     */\\n    update(value: any): IDBRequest<IDBValidKey>;\\n}\\n\\ndeclare var IDBCursor: {\\n    prototype: IDBCursor;\\n    new(): IDBCursor;\\n};\\n\\ninterface IDBCursorWithValue extends IDBCursor {\\n    /**\\n     * Returns the cursor\\'s current value.\\n     */\\n    readonly value: any;\\n}\\n\\ndeclare var IDBCursorWithValue: {\\n    prototype: IDBCursorWithValue;\\n    new(): IDBCursorWithValue;\\n};\\n\\ninterface IDBDatabaseEventMap {\\n    \"abort\": Event;\\n    \"close\": Event;\\n    \"error\": Event;\\n    \"versionchange\": IDBVersionChangeEvent;\\n}\\n\\ninterface IDBDatabase extends EventTarget {\\n    /**\\n     * Returns the name of the database.\\n     */\\n    readonly name: string;\\n    /**\\n     * Returns a list of the names of object stores in the database.\\n     */\\n    readonly objectStoreNames: DOMStringList;\\n    onabort: ((this: IDBDatabase, ev: Event) => any) | null;\\n    onclose: ((this: IDBDatabase, ev: Event) => any) | null;\\n    onerror: ((this: IDBDatabase, ev: Event) => any) | null;\\n    onversionchange: ((this: IDBDatabase, ev: IDBVersionChangeEvent) => any) | null;\\n    /**\\n     * Returns the version of the database.\\n     */\\n    readonly version: number;\\n    /**\\n     * Closes the connection once all running transactions have finished.\\n     */\\n    close(): void;\\n    /**\\n     * Creates a new object store with the given name and options and returns a new IDBObjectStore.\\n     * Throws a \"InvalidStateError\" DOMException if not called within an upgrade transaction.\\n     */\\n    createObjectStore(name: string, optionalParameters?: IDBObjectStoreParameters): IDBObjectStore;\\n    /**\\n     * Deletes the object store with the given name.\\n     * Throws a \"InvalidStateError\" DOMException if not called within an upgrade transaction.\\n     */\\n    deleteObjectStore(name: string): void;\\n    /**\\n     * Returns a new transaction with the given mode (\"readonly\" or \"readwrite\")\\n     * and scope which can be a single object store name or an array of names.\\n     */\\n    transaction(storeNames: string | string[], mode?: IDBTransactionMode): IDBTransaction;\\n    addEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBDatabase: {\\n    prototype: IDBDatabase;\\n    new(): IDBDatabase;\\n};\\n\\ninterface IDBEnvironment {\\n    readonly indexedDB: IDBFactory;\\n}\\n\\ninterface IDBFactory {\\n    /**\\n     * Compares two values as keys. Returns -1 if key1 precedes key2, 1 if key2 precedes key1, and 0 if\\n     * the keys are equal.\\n     * Throws a \"DataError\" DOMException if either input is not a valid key.\\n     */\\n    cmp(first: any, second: any): number;\\n    /**\\n     * Attempts to delete the named database. If the\\n     * database already exists and there are open connections that don\\'t close in response to a versionchange event, the request will be blocked until all they close. If the request\\n     * is successful request\\'s result will be null.\\n     */\\n    deleteDatabase(name: string): IDBOpenDBRequest;\\n    /**\\n     * Attempts to open a connection to the named database with the specified version. If the database already exists\\n     * with a lower version and there are open connections that don\\'t close in response to a versionchange event, the request will be blocked until all they close, then an upgrade\\n     * will occur. If the database already exists with a higher\\n     * version the request will fail. If the request is\\n     * successful request\\'s result will\\n     * be the connection.\\n     */\\n    open(name: string, version?: number): IDBOpenDBRequest;\\n}\\n\\ndeclare var IDBFactory: {\\n    prototype: IDBFactory;\\n    new(): IDBFactory;\\n};\\n\\ninterface IDBIndex {\\n    readonly keyPath: string | string[];\\n    readonly multiEntry: boolean;\\n    /**\\n     * Updates the name of the store to newName.\\n     * Throws an \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    name: string;\\n    /**\\n     * Returns the IDBObjectStore the index belongs to.\\n     */\\n    readonly objectStore: IDBObjectStore;\\n    readonly unique: boolean;\\n    /**\\n     * Retrieves the number of records matching the given key or key range in query.\\n     * If successful, request\\'s result will be the\\n     * count.\\n     */\\n    count(key?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\\n    /**\\n     * Retrieves the value of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the value, or undefined if there was no matching record.\\n     */\\n    get(key: IDBValidKey | IDBKeyRange): IDBRequest<any | undefined>;\\n    /**\\n     * Retrieves the values of the records matching the given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will be an Array of the values.\\n     */\\n    getAll(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<any[]>;\\n    /**\\n     * Retrieves the keys of records matching the given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will be an Array of the keys.\\n     */\\n    getAllKeys(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<IDBValidKey[]>;\\n    /**\\n     * Retrieves the key of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the key, or undefined if there was no matching record.\\n     */\\n    getKey(key: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\\n    /**\\n     * Opens a cursor over the records matching query,\\n     * ordered by direction. If query is null, all records in index are matched.\\n     * If successful, request\\'s result will be an IDBCursorWithValue, or null if there were no matching records.\\n     */\\n    openCursor(range?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\\n    /**\\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in index are matched.\\n     * If successful, request\\'s result will be an IDBCursor, or null if there were no matching records.\\n     */\\n    openKeyCursor(range?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\\n}\\n\\ndeclare var IDBIndex: {\\n    prototype: IDBIndex;\\n    new(): IDBIndex;\\n};\\n\\ninterface IDBKeyRange {\\n    /**\\n     * Returns lower bound, or undefined if none.\\n     */\\n    readonly lower: any;\\n    /**\\n     * Returns true if the lower open flag is set, and false otherwise.\\n     */\\n    readonly lowerOpen: boolean;\\n    /**\\n     * Returns upper bound, or undefined if none.\\n     */\\n    readonly upper: any;\\n    /**\\n     * Returns true if the upper open flag is set, and false otherwise.\\n     */\\n    readonly upperOpen: boolean;\\n    /**\\n     * Returns true if key is included in the range, and false otherwise.\\n     */\\n    includes(key: any): boolean;\\n}\\n\\ndeclare var IDBKeyRange: {\\n    prototype: IDBKeyRange;\\n    new(): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange spanning from lower to upper.\\n     * If lowerOpen is true, lower is not included in the range.\\n     * If upperOpen is true, upper is not included in the range.\\n     */\\n    bound(lower: any, upper: any, lowerOpen?: boolean, upperOpen?: boolean): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange starting at key with no\\n     * upper bound. If open is true, key is not included in the\\n     * range.\\n     */\\n    lowerBound(lower: any, open?: boolean): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange spanning only key.\\n     */\\n    only(value: any): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange with no lower bound and ending at key. If open is true, key is not included in the range.\\n     */\\n    upperBound(upper: any, open?: boolean): IDBKeyRange;\\n};\\n\\ninterface IDBObjectStore {\\n    /**\\n     * Returns true if the store has a key generator, and false otherwise.\\n     */\\n    readonly autoIncrement: boolean;\\n    /**\\n     * Returns a list of the names of indexes in the store.\\n     */\\n    readonly indexNames: DOMStringList;\\n    /**\\n     * Returns the key path of the store, or null if none.\\n     */\\n    readonly keyPath: string | string[];\\n    /**\\n     * Updates the name of the store to newName.\\n     * Throws \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    name: string;\\n    /**\\n     * Returns the associated transaction.\\n     */\\n    readonly transaction: IDBTransaction;\\n    add(value: any, key?: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey>;\\n    /**\\n     * Deletes all records in store.\\n     * If successful, request\\'s result will\\n     * be undefined.\\n     */\\n    clear(): IDBRequest<undefined>;\\n    /**\\n     * Retrieves the number of records matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the count.\\n     */\\n    count(key?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\\n    /**\\n     * Creates a new index in store with the given name, keyPath and options and returns a new IDBIndex. If the keyPath and options define constraints that cannot be\\n     * satisfied with the data already in store the upgrade\\n     * transaction will abort with\\n     * a \"ConstraintError\" DOMException.\\n     * Throws an \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    createIndex(name: string, keyPath: string | string[], options?: IDBIndexParameters): IDBIndex;\\n    /**\\n     * Deletes records in store with the given key or in the given key range in query.\\n     * If successful, request\\'s result will\\n     * be undefined.\\n     */\\n    delete(key: IDBValidKey | IDBKeyRange): IDBRequest<undefined>;\\n    /**\\n     * Deletes the index in store with the given name.\\n     * Throws an \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    deleteIndex(name: string): void;\\n    /**\\n     * Retrieves the value of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the value, or undefined if there was no matching record.\\n     */\\n    get(query: IDBValidKey | IDBKeyRange): IDBRequest<any | undefined>;\\n    /**\\n     * Retrieves the values of the records matching the\\n     * given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will\\n     * be an Array of the values.\\n     */\\n    getAll(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<any[]>;\\n    /**\\n     * Retrieves the keys of records matching the\\n     * given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will\\n     * be an Array of the keys.\\n     */\\n    getAllKeys(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<IDBValidKey[]>;\\n    /**\\n     * Retrieves the key of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the key, or undefined if there was no matching record.\\n     */\\n    getKey(query: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\\n    index(name: string): IDBIndex;\\n    /**\\n     * Opens a cursor over the records matching query,\\n     * ordered by direction. If query is null, all records in store are matched.\\n     * If successful, request\\'s result will be an IDBCursorWithValue pointing at the first matching record, or null if there were no matching records.\\n     */\\n    openCursor(range?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\\n    /**\\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in store are matched.\\n     * If successful, request\\'s result will be an IDBCursor pointing at the first matching record, or\\n     * null if there were no matching records.\\n     */\\n    openKeyCursor(query?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\\n    put(value: any, key?: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey>;\\n}\\n\\ndeclare var IDBObjectStore: {\\n    prototype: IDBObjectStore;\\n    new(): IDBObjectStore;\\n};\\n\\ninterface IDBOpenDBRequestEventMap extends IDBRequestEventMap {\\n    \"blocked\": Event;\\n    \"upgradeneeded\": IDBVersionChangeEvent;\\n}\\n\\ninterface IDBOpenDBRequest extends IDBRequest<IDBDatabase> {\\n    onblocked: ((this: IDBOpenDBRequest, ev: Event) => any) | null;\\n    onupgradeneeded: ((this: IDBOpenDBRequest, ev: IDBVersionChangeEvent) => any) | null;\\n    addEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBOpenDBRequest: {\\n    prototype: IDBOpenDBRequest;\\n    new(): IDBOpenDBRequest;\\n};\\n\\ninterface IDBRequestEventMap {\\n    \"error\": Event;\\n    \"success\": Event;\\n}\\n\\ninterface IDBRequest<T = any> extends EventTarget {\\n    /**\\n     * When a request is completed, returns the error (a DOMException), or null if the request succeeded. Throws\\n     * a \"InvalidStateError\" DOMException if the request is still pending.\\n     */\\n    readonly error: DOMException | null;\\n    onerror: ((this: IDBRequest<T>, ev: Event) => any) | null;\\n    onsuccess: ((this: IDBRequest<T>, ev: Event) => any) | null;\\n    /**\\n     * Returns \"pending\" until a request is complete,\\n     * then returns \"done\".\\n     */\\n    readonly readyState: IDBRequestReadyState;\\n    /**\\n     * When a request is completed, returns the result,\\n     * or undefined if the request failed. Throws a\\n     * \"InvalidStateError\" DOMException if the request is still pending.\\n     */\\n    readonly result: T;\\n    /**\\n     * Returns the IDBObjectStore, IDBIndex, or IDBCursor the request was made against, or null if is was an open\\n     * request.\\n     */\\n    readonly source: IDBObjectStore | IDBIndex | IDBCursor;\\n    /**\\n     * Returns the IDBTransaction the request was made within.\\n     * If this as an open request, then it returns an upgrade transaction while it is running, or null otherwise.\\n     */\\n    readonly transaction: IDBTransaction | null;\\n    addEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBRequest: {\\n    prototype: IDBRequest;\\n    new(): IDBRequest;\\n};\\n\\ninterface IDBTransactionEventMap {\\n    \"abort\": Event;\\n    \"complete\": Event;\\n    \"error\": Event;\\n}\\n\\ninterface IDBTransaction extends EventTarget {\\n    /**\\n     * Returns the transaction\\'s connection.\\n     */\\n    readonly db: IDBDatabase;\\n    /**\\n     * If the transaction was aborted, returns the\\n     * error (a DOMException) providing the reason.\\n     */\\n    readonly error: DOMException;\\n    /**\\n     * Returns the mode the transaction was created with\\n     * (\"readonly\" or \"readwrite\"), or \"versionchange\" for\\n     * an upgrade transaction.\\n     */\\n    readonly mode: IDBTransactionMode;\\n    /**\\n     * Returns a list of the names of object stores in the\\n     * transaction\\'s scope. For an upgrade transaction this is all object stores in the database.\\n     */\\n    readonly objectStoreNames: DOMStringList;\\n    onabort: ((this: IDBTransaction, ev: Event) => any) | null;\\n    oncomplete: ((this: IDBTransaction, ev: Event) => any) | null;\\n    onerror: ((this: IDBTransaction, ev: Event) => any) | null;\\n    /**\\n     * Aborts the transaction. All pending requests will fail with\\n     * a \"AbortError\" DOMException and all changes made to the database will be\\n     * reverted.\\n     */\\n    abort(): void;\\n    /**\\n     * Returns an IDBObjectStore in the transaction\\'s scope.\\n     */\\n    objectStore(name: string): IDBObjectStore;\\n    addEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBTransaction: {\\n    prototype: IDBTransaction;\\n    new(): IDBTransaction;\\n};\\n\\ninterface IDBVersionChangeEvent extends Event {\\n    readonly newVersion: number | null;\\n    readonly oldVersion: number;\\n}\\n\\ndeclare var IDBVersionChangeEvent: {\\n    prototype: IDBVersionChangeEvent;\\n    new(type: string, eventInitDict?: IDBVersionChangeEventInit): IDBVersionChangeEvent;\\n};\\n\\ninterface IIRFilterNode extends AudioNode {\\n    getFrequencyResponse(frequencyHz: Float32Array, magResponse: Float32Array, phaseResponse: Float32Array): void;\\n}\\n\\ndeclare var IIRFilterNode: {\\n    prototype: IIRFilterNode;\\n    new(context: BaseAudioContext, options: IIRFilterOptions): IIRFilterNode;\\n};\\n\\ninterface ImageBitmap {\\n    /**\\n     * Returns the intrinsic height of the image, in CSS\\n     * pixels.\\n     */\\n    readonly height: number;\\n    /**\\n     * Returns the intrinsic width of the image, in CSS\\n     * pixels.\\n     */\\n    readonly width: number;\\n    /**\\n     * Releases imageBitmap\\'s underlying bitmap data.\\n     */\\n    close(): void;\\n}\\n\\ndeclare var ImageBitmap: {\\n    prototype: ImageBitmap;\\n    new(): ImageBitmap;\\n};\\n\\ninterface ImageBitmapOptions {\\n    colorSpaceConversion?: \"none\" | \"default\";\\n    imageOrientation?: \"none\" | \"flipY\";\\n    premultiplyAlpha?: \"none\" | \"premultiply\" | \"default\";\\n    resizeHeight?: number;\\n    resizeQuality?: \"pixelated\" | \"low\" | \"medium\" | \"high\";\\n    resizeWidth?: number;\\n}\\n\\ninterface ImageBitmapRenderingContext {\\n    /**\\n     * Returns the canvas element that the context is bound to.\\n     */\\n    readonly canvas: HTMLCanvasElement;\\n    /**\\n     * Replaces contents of the canvas element to which context\\n     * is bound with a transparent black bitmap whose size corresponds to the width and height\\n     * content attributes of the canvas element.\\n     */\\n    transferFromImageBitmap(bitmap: ImageBitmap | null): void;\\n}\\n\\ndeclare var ImageBitmapRenderingContext: {\\n    prototype: ImageBitmapRenderingContext;\\n    new(): ImageBitmapRenderingContext;\\n};\\n\\ninterface ImageData {\\n    /**\\n     * Returns the one-dimensional array containing the data in RGBA order, as integers in the\\n     * range 0 to 255.\\n     */\\n    readonly data: Uint8ClampedArray;\\n    /**\\n     * Returns the actual dimensions of the data in the ImageData object, in\\n     * pixels.\\n     */\\n    readonly height: number;\\n    readonly width: number;\\n}\\n\\ndeclare var ImageData: {\\n    prototype: ImageData;\\n    new(width: number, height: number): ImageData;\\n    new(array: Uint8ClampedArray, width: number, height: number): ImageData;\\n};\\n\\ninterface IntersectionObserver {\\n    readonly root: Element | null;\\n    readonly rootMargin: string;\\n    readonly thresholds: number[];\\n    disconnect(): void;\\n    observe(target: Element): void;\\n    takeRecords(): IntersectionObserverEntry[];\\n    unobserve(target: Element): void;\\n}\\n\\ndeclare var IntersectionObserver: {\\n    prototype: IntersectionObserver;\\n    new(callback: IntersectionObserverCallback, options?: IntersectionObserverInit): IntersectionObserver;\\n};\\n\\ninterface IntersectionObserverEntry {\\n    readonly boundingClientRect: ClientRect | DOMRect;\\n    readonly intersectionRatio: number;\\n    readonly intersectionRect: ClientRect | DOMRect;\\n    readonly isIntersecting: boolean;\\n    readonly rootBounds: ClientRect | DOMRect;\\n    readonly target: Element;\\n    readonly time: number;\\n}\\n\\ndeclare var IntersectionObserverEntry: {\\n    prototype: IntersectionObserverEntry;\\n    new(intersectionObserverEntryInit: IntersectionObserverEntryInit): IntersectionObserverEntry;\\n};\\n\\ninterface KeyboardEvent extends UIEvent {\\n    readonly altKey: boolean;\\n    /** @deprecated */\\n    char: string;\\n    /** @deprecated */\\n    readonly charCode: number;\\n    readonly code: string;\\n    readonly ctrlKey: boolean;\\n    readonly key: string;\\n    /** @deprecated */\\n    readonly keyCode: number;\\n    readonly location: number;\\n    readonly metaKey: boolean;\\n    readonly repeat: boolean;\\n    readonly shiftKey: boolean;\\n    /** @deprecated */\\n    readonly which: number;\\n    getModifierState(keyArg: string): boolean;\\n    /** @deprecated */\\n    initKeyboardEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, keyArg: string, locationArg: number, modifiersListArg: string, repeat: boolean, locale: string): void;\\n    readonly DOM_KEY_LOCATION_JOYSTICK: number;\\n    readonly DOM_KEY_LOCATION_LEFT: number;\\n    readonly DOM_KEY_LOCATION_MOBILE: number;\\n    readonly DOM_KEY_LOCATION_NUMPAD: number;\\n    readonly DOM_KEY_LOCATION_RIGHT: number;\\n    readonly DOM_KEY_LOCATION_STANDARD: number;\\n}\\n\\ndeclare var KeyboardEvent: {\\n    prototype: KeyboardEvent;\\n    new(typeArg: string, eventInitDict?: KeyboardEventInit): KeyboardEvent;\\n    readonly DOM_KEY_LOCATION_JOYSTICK: number;\\n    readonly DOM_KEY_LOCATION_LEFT: number;\\n    readonly DOM_KEY_LOCATION_MOBILE: number;\\n    readonly DOM_KEY_LOCATION_NUMPAD: number;\\n    readonly DOM_KEY_LOCATION_RIGHT: number;\\n    readonly DOM_KEY_LOCATION_STANDARD: number;\\n};\\n\\ninterface KeyframeEffect extends AnimationEffect {\\n    composite: CompositeOperation;\\n    iterationComposite: IterationCompositeOperation;\\n    target: Element | null;\\n    getKeyframes(): ComputedKeyframe[];\\n    setKeyframes(keyframes: Keyframe[] | PropertyIndexedKeyframes | null): void;\\n}\\n\\ndeclare var KeyframeEffect: {\\n    prototype: KeyframeEffect;\\n    new(target: Element | null, keyframes: Keyframe[] | PropertyIndexedKeyframes | null, options?: number | KeyframeEffectOptions): KeyframeEffect;\\n    new(source: KeyframeEffect): KeyframeEffect;\\n};\\n\\ninterface LinkStyle {\\n    readonly sheet: StyleSheet | null;\\n}\\n\\ninterface ListeningStateChangedEvent extends Event {\\n    readonly label: string;\\n    readonly state: ListeningState;\\n}\\n\\ndeclare var ListeningStateChangedEvent: {\\n    prototype: ListeningStateChangedEvent;\\n    new(): ListeningStateChangedEvent;\\n};\\n\\ninterface Location {\\n    /**\\n     * Returns a DOMStringList object listing the origins of the ancestor browsing contexts, from the parent browsing\\n     * context to the top-level browsing context.\\n     */\\n    readonly ancestorOrigins: DOMStringList;\\n    /**\\n     * Returns the Location object\\'s URL\\'s fragment (includes leading \"#\" if non-empty).\\n     * Can be set, to navigate to the same URL with a changed fragment (ignores leading \"#\").\\n     */\\n    hash: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s host and port (if different from the default\\n     * port for the scheme).\\n     * Can be set, to navigate to the same URL with a changed host and port.\\n     */\\n    host: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s host.\\n     * Can be set, to navigate to the same URL with a changed host.\\n     */\\n    hostname: string;\\n    /**\\n     * Returns the Location object\\'s URL.\\n     * Can be set, to navigate to the given URL.\\n     */\\n    href: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s origin.\\n     */\\n    readonly origin: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s path.\\n     * Can be set, to navigate to the same URL with a changed path.\\n     */\\n    pathname: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s port.\\n     * Can be set, to navigate to the same URL with a changed port.\\n     */\\n    port: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s scheme.\\n     * Can be set, to navigate to the same URL with a changed scheme.\\n     */\\n    protocol: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s query (includes leading \"?\" if non-empty).\\n     * Can be set, to navigate to the same URL with a changed query (ignores leading \"?\").\\n     */\\n    search: string;\\n    /**\\n     * Navigates to the given URL.\\n     */\\n    assign(url: string): void;\\n    /**\\n     * Reloads the current page.\\n     */\\n    reload(forcedReload?: boolean): void;\\n    /**\\n     * Removes the current page from the session history and navigates to the given URL.\\n     */\\n    replace(url: string): void;\\n}\\n\\ndeclare var Location: {\\n    prototype: Location;\\n    new(): Location;\\n};\\n\\ninterface MSAssertion {\\n    readonly id: string;\\n    readonly type: MSCredentialType;\\n}\\n\\ndeclare var MSAssertion: {\\n    prototype: MSAssertion;\\n    new(): MSAssertion;\\n};\\n\\ninterface MSBlobBuilder {\\n    append(data: any, endings?: string): void;\\n    getBlob(contentType?: string): Blob;\\n}\\n\\ndeclare var MSBlobBuilder: {\\n    prototype: MSBlobBuilder;\\n    new(): MSBlobBuilder;\\n};\\n\\ninterface MSFIDOCredentialAssertion extends MSAssertion {\\n    readonly algorithm: string | Algorithm;\\n    readonly attestation: any;\\n    readonly publicKey: string;\\n    readonly transportHints: MSTransportType[];\\n}\\n\\ndeclare var MSFIDOCredentialAssertion: {\\n    prototype: MSFIDOCredentialAssertion;\\n    new(): MSFIDOCredentialAssertion;\\n};\\n\\ninterface MSFIDOSignature {\\n    readonly authnrData: string;\\n    readonly clientData: string;\\n    readonly signature: string;\\n}\\n\\ndeclare var MSFIDOSignature: {\\n    prototype: MSFIDOSignature;\\n    new(): MSFIDOSignature;\\n};\\n\\ninterface MSFIDOSignatureAssertion extends MSAssertion {\\n    readonly signature: MSFIDOSignature;\\n}\\n\\ndeclare var MSFIDOSignatureAssertion: {\\n    prototype: MSFIDOSignatureAssertion;\\n    new(): MSFIDOSignatureAssertion;\\n};\\n\\ninterface MSFileSaver {\\n    msSaveBlob(blob: any, defaultName?: string): boolean;\\n    msSaveOrOpenBlob(blob: any, defaultName?: string): boolean;\\n}\\n\\ninterface MSGesture {\\n    target: Element;\\n    addPointer(pointerId: number): void;\\n    stop(): void;\\n}\\n\\ndeclare var MSGesture: {\\n    prototype: MSGesture;\\n    new(): MSGesture;\\n};\\n\\ninterface MSGestureEvent extends UIEvent {\\n    readonly clientX: number;\\n    readonly clientY: number;\\n    readonly expansion: number;\\n    readonly gestureObject: any;\\n    readonly hwTimestamp: number;\\n    readonly offsetX: number;\\n    readonly offsetY: number;\\n    readonly rotation: number;\\n    readonly scale: number;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly translationX: number;\\n    readonly translationY: number;\\n    readonly velocityAngular: number;\\n    readonly velocityExpansion: number;\\n    readonly velocityX: number;\\n    readonly velocityY: number;\\n    initGestureEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, offsetXArg: number, offsetYArg: number, translationXArg: number, translationYArg: number, scaleArg: number, expansionArg: number, rotationArg: number, velocityXArg: number, velocityYArg: number, velocityExpansionArg: number, velocityAngularArg: number, hwTimestampArg: number): void;\\n    readonly MSGESTURE_FLAG_BEGIN: number;\\n    readonly MSGESTURE_FLAG_CANCEL: number;\\n    readonly MSGESTURE_FLAG_END: number;\\n    readonly MSGESTURE_FLAG_INERTIA: number;\\n    readonly MSGESTURE_FLAG_NONE: number;\\n}\\n\\ndeclare var MSGestureEvent: {\\n    prototype: MSGestureEvent;\\n    new(): MSGestureEvent;\\n    readonly MSGESTURE_FLAG_BEGIN: number;\\n    readonly MSGESTURE_FLAG_CANCEL: number;\\n    readonly MSGESTURE_FLAG_END: number;\\n    readonly MSGESTURE_FLAG_INERTIA: number;\\n    readonly MSGESTURE_FLAG_NONE: number;\\n};\\n\\ninterface MSGraphicsTrust {\\n    readonly constrictionActive: boolean;\\n    readonly status: string;\\n}\\n\\ndeclare var MSGraphicsTrust: {\\n    prototype: MSGraphicsTrust;\\n    new(): MSGraphicsTrust;\\n};\\n\\ninterface MSInputMethodContextEventMap {\\n    \"MSCandidateWindowHide\": Event;\\n    \"MSCandidateWindowShow\": Event;\\n    \"MSCandidateWindowUpdate\": Event;\\n}\\n\\ninterface MSInputMethodContext extends EventTarget {\\n    readonly compositionEndOffset: number;\\n    readonly compositionStartOffset: number;\\n    oncandidatewindowhide: ((this: MSInputMethodContext, ev: Event) => any) | null;\\n    oncandidatewindowshow: ((this: MSInputMethodContext, ev: Event) => any) | null;\\n    oncandidatewindowupdate: ((this: MSInputMethodContext, ev: Event) => any) | null;\\n    readonly target: HTMLElement;\\n    getCandidateWindowClientRect(): ClientRect;\\n    getCompositionAlternatives(): string[];\\n    hasComposition(): boolean;\\n    isCandidateWindowVisible(): boolean;\\n    addEventListener<K extends keyof MSInputMethodContextEventMap>(type: K, listener: (this: MSInputMethodContext, ev: MSInputMethodContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MSInputMethodContextEventMap>(type: K, listener: (this: MSInputMethodContext, ev: MSInputMethodContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MSInputMethodContext: {\\n    prototype: MSInputMethodContext;\\n    new(): MSInputMethodContext;\\n};\\n\\ninterface MSMediaKeyError {\\n    readonly code: number;\\n    readonly systemCode: number;\\n    readonly MS_MEDIA_KEYERR_CLIENT: number;\\n    readonly MS_MEDIA_KEYERR_DOMAIN: number;\\n    readonly MS_MEDIA_KEYERR_HARDWARECHANGE: number;\\n    readonly MS_MEDIA_KEYERR_OUTPUT: number;\\n    readonly MS_MEDIA_KEYERR_SERVICE: number;\\n    readonly MS_MEDIA_KEYERR_UNKNOWN: number;\\n}\\n\\ndeclare var MSMediaKeyError: {\\n    prototype: MSMediaKeyError;\\n    new(): MSMediaKeyError;\\n    readonly MS_MEDIA_KEYERR_CLIENT: number;\\n    readonly MS_MEDIA_KEYERR_DOMAIN: number;\\n    readonly MS_MEDIA_KEYERR_HARDWARECHANGE: number;\\n    readonly MS_MEDIA_KEYERR_OUTPUT: number;\\n    readonly MS_MEDIA_KEYERR_SERVICE: number;\\n    readonly MS_MEDIA_KEYERR_UNKNOWN: number;\\n};\\n\\ninterface MSMediaKeyMessageEvent extends Event {\\n    readonly destinationURL: string | null;\\n    readonly message: Uint8Array;\\n}\\n\\ndeclare var MSMediaKeyMessageEvent: {\\n    prototype: MSMediaKeyMessageEvent;\\n    new(): MSMediaKeyMessageEvent;\\n};\\n\\ninterface MSMediaKeyNeededEvent extends Event {\\n    readonly initData: Uint8Array | null;\\n}\\n\\ndeclare var MSMediaKeyNeededEvent: {\\n    prototype: MSMediaKeyNeededEvent;\\n    new(): MSMediaKeyNeededEvent;\\n};\\n\\ninterface MSMediaKeySession extends EventTarget {\\n    readonly error: MSMediaKeyError | null;\\n    readonly keySystem: string;\\n    readonly sessionId: string;\\n    close(): void;\\n    update(key: Uint8Array): void;\\n}\\n\\ndeclare var MSMediaKeySession: {\\n    prototype: MSMediaKeySession;\\n    new(): MSMediaKeySession;\\n};\\n\\ninterface MSMediaKeys {\\n    readonly keySystem: string;\\n    createSession(type: string, initData: Uint8Array, cdmData?: Uint8Array | null): MSMediaKeySession;\\n}\\n\\ndeclare var MSMediaKeys: {\\n    prototype: MSMediaKeys;\\n    new(keySystem: string): MSMediaKeys;\\n    isTypeSupported(keySystem: string, type?: string | null): boolean;\\n    isTypeSupportedWithFeatures(keySystem: string, type?: string | null): string;\\n};\\n\\ninterface MSNavigatorDoNotTrack {\\n    confirmSiteSpecificTrackingException(args: ConfirmSiteSpecificExceptionsInformation): boolean;\\n    confirmWebWideTrackingException(args: ExceptionInformation): boolean;\\n    removeSiteSpecificTrackingException(args: ExceptionInformation): void;\\n    removeWebWideTrackingException(args: ExceptionInformation): void;\\n    storeSiteSpecificTrackingException(args: StoreSiteSpecificExceptionsInformation): void;\\n    storeWebWideTrackingException(args: StoreExceptionsInformation): void;\\n}\\n\\ninterface MSPointerEvent extends MouseEvent {\\n    readonly currentPoint: any;\\n    readonly height: number;\\n    readonly hwTimestamp: number;\\n    readonly intermediatePoints: any;\\n    readonly isPrimary: boolean;\\n    readonly pointerId: number;\\n    readonly pointerType: any;\\n    readonly pressure: number;\\n    readonly rotation: number;\\n    readonly tiltX: number;\\n    readonly tiltY: number;\\n    readonly width: number;\\n    getCurrentPoint(element: Element): void;\\n    getIntermediatePoints(element: Element): void;\\n    initPointerEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, ctrlKeyArg: boolean, altKeyArg: boolean, shiftKeyArg: boolean, metaKeyArg: boolean, buttonArg: number, relatedTargetArg: EventTarget, offsetXArg: number, offsetYArg: number, widthArg: number, heightArg: number, pressure: number, rotation: number, tiltX: number, tiltY: number, pointerIdArg: number, pointerType: any, hwTimestampArg: number, isPrimary: boolean): void;\\n}\\n\\ndeclare var MSPointerEvent: {\\n    prototype: MSPointerEvent;\\n    new(typeArg: string, eventInitDict?: PointerEventInit): MSPointerEvent;\\n};\\n\\ninterface MSStream {\\n    readonly type: string;\\n    msClose(): void;\\n    msDetachStream(): any;\\n}\\n\\ndeclare var MSStream: {\\n    prototype: MSStream;\\n    new(): MSStream;\\n};\\n\\ninterface MediaDeviceInfo {\\n    readonly deviceId: string;\\n    readonly groupId: string;\\n    readonly kind: MediaDeviceKind;\\n    readonly label: string;\\n}\\n\\ndeclare var MediaDeviceInfo: {\\n    prototype: MediaDeviceInfo;\\n    new(): MediaDeviceInfo;\\n};\\n\\ninterface MediaDevicesEventMap {\\n    \"devicechange\": Event;\\n}\\n\\ninterface MediaDevices extends EventTarget {\\n    ondevicechange: ((this: MediaDevices, ev: Event) => any) | null;\\n    enumerateDevices(): Promise<MediaDeviceInfo[]>;\\n    getSupportedConstraints(): MediaTrackSupportedConstraints;\\n    getUserMedia(constraints: MediaStreamConstraints): Promise<MediaStream>;\\n    addEventListener<K extends keyof MediaDevicesEventMap>(type: K, listener: (this: MediaDevices, ev: MediaDevicesEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaDevicesEventMap>(type: K, listener: (this: MediaDevices, ev: MediaDevicesEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaDevices: {\\n    prototype: MediaDevices;\\n    new(): MediaDevices;\\n};\\n\\ninterface MediaElementAudioSourceNode extends AudioNode {\\n    readonly mediaElement: HTMLMediaElement;\\n}\\n\\ndeclare var MediaElementAudioSourceNode: {\\n    prototype: MediaElementAudioSourceNode;\\n    new(context: AudioContext, options: MediaElementAudioSourceOptions): MediaElementAudioSourceNode;\\n};\\n\\ninterface MediaEncryptedEvent extends Event {\\n    readonly initData: ArrayBuffer | null;\\n    readonly initDataType: string;\\n}\\n\\ndeclare var MediaEncryptedEvent: {\\n    prototype: MediaEncryptedEvent;\\n    new(type: string, eventInitDict?: MediaEncryptedEventInit): MediaEncryptedEvent;\\n};\\n\\ninterface MediaError {\\n    readonly code: number;\\n    readonly message: string;\\n    readonly msExtendedCode: number;\\n    readonly MEDIA_ERR_ABORTED: number;\\n    readonly MEDIA_ERR_DECODE: number;\\n    readonly MEDIA_ERR_NETWORK: number;\\n    readonly MEDIA_ERR_SRC_NOT_SUPPORTED: number;\\n    readonly MS_MEDIA_ERR_ENCRYPTED: number;\\n}\\n\\ndeclare var MediaError: {\\n    prototype: MediaError;\\n    new(): MediaError;\\n    readonly MEDIA_ERR_ABORTED: number;\\n    readonly MEDIA_ERR_DECODE: number;\\n    readonly MEDIA_ERR_NETWORK: number;\\n    readonly MEDIA_ERR_SRC_NOT_SUPPORTED: number;\\n    readonly MS_MEDIA_ERR_ENCRYPTED: number;\\n};\\n\\ninterface MediaKeyMessageEvent extends Event {\\n    readonly message: ArrayBuffer;\\n    readonly messageType: MediaKeyMessageType;\\n}\\n\\ndeclare var MediaKeyMessageEvent: {\\n    prototype: MediaKeyMessageEvent;\\n    new(type: string, eventInitDict?: MediaKeyMessageEventInit): MediaKeyMessageEvent;\\n};\\n\\ninterface MediaKeySession extends EventTarget {\\n    readonly closed: Promise<void>;\\n    readonly expiration: number;\\n    readonly keyStatuses: MediaKeyStatusMap;\\n    readonly sessionId: string;\\n    close(): Promise<void>;\\n    generateRequest(initDataType: string, initData: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): Promise<void>;\\n    load(sessionId: string): Promise<boolean>;\\n    remove(): Promise<void>;\\n    update(response: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): Promise<void>;\\n}\\n\\ndeclare var MediaKeySession: {\\n    prototype: MediaKeySession;\\n    new(): MediaKeySession;\\n};\\n\\ninterface MediaKeyStatusMap {\\n    readonly size: number;\\n    forEach(callback: Function, thisArg?: any): void;\\n    get(keyId: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): MediaKeyStatus;\\n    has(keyId: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): boolean;\\n}\\n\\ndeclare var MediaKeyStatusMap: {\\n    prototype: MediaKeyStatusMap;\\n    new(): MediaKeyStatusMap;\\n};\\n\\ninterface MediaKeySystemAccess {\\n    readonly keySystem: string;\\n    createMediaKeys(): Promise<MediaKeys>;\\n    getConfiguration(): MediaKeySystemConfiguration;\\n}\\n\\ndeclare var MediaKeySystemAccess: {\\n    prototype: MediaKeySystemAccess;\\n    new(): MediaKeySystemAccess;\\n};\\n\\ninterface MediaKeys {\\n    createSession(sessionType?: MediaKeySessionType): MediaKeySession;\\n    setServerCertificate(serverCertificate: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): Promise<void>;\\n}\\n\\ndeclare var MediaKeys: {\\n    prototype: MediaKeys;\\n    new(): MediaKeys;\\n};\\n\\ninterface MediaList {\\n    readonly length: number;\\n    mediaText: string;\\n    appendMedium(medium: string): void;\\n    deleteMedium(medium: string): void;\\n    item(index: number): string | null;\\n    toString(): number;\\n    [index: number]: string;\\n}\\n\\ndeclare var MediaList: {\\n    prototype: MediaList;\\n    new(): MediaList;\\n};\\n\\ninterface MediaQueryListEventMap {\\n    \"change\": MediaQueryListEvent;\\n}\\n\\ninterface MediaQueryList extends EventTarget {\\n    readonly matches: boolean;\\n    readonly media: string;\\n    onchange: ((this: MediaQueryList, ev: MediaQueryListEvent) => any) | null;\\n    /** @deprecated */\\n    addListener(listener: ((this: MediaQueryList, ev: MediaQueryListEvent) => any) | null): void;\\n    /** @deprecated */\\n    removeListener(listener: EventListenerOrEventListenerObject | null): void;\\n    addEventListener<K extends keyof MediaQueryListEventMap>(type: K, listener: (this: MediaQueryList, ev: MediaQueryListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaQueryListEventMap>(type: K, listener: (this: MediaQueryList, ev: MediaQueryListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaQueryList: {\\n    prototype: MediaQueryList;\\n    new(): MediaQueryList;\\n};\\n\\ninterface MediaQueryListEvent extends Event {\\n    readonly matches: boolean;\\n    readonly media: string;\\n}\\n\\ndeclare var MediaQueryListEvent: {\\n    prototype: MediaQueryListEvent;\\n    new(type: string, eventInitDict?: MediaQueryListEventInit): MediaQueryListEvent;\\n};\\n\\ninterface MediaSource extends EventTarget {\\n    readonly activeSourceBuffers: SourceBufferList;\\n    duration: number;\\n    readonly readyState: ReadyState;\\n    readonly sourceBuffers: SourceBufferList;\\n    addSourceBuffer(type: string): SourceBuffer;\\n    endOfStream(error?: EndOfStreamError): void;\\n    removeSourceBuffer(sourceBuffer: SourceBuffer): void;\\n}\\n\\ndeclare var MediaSource: {\\n    prototype: MediaSource;\\n    new(): MediaSource;\\n    isTypeSupported(type: string): boolean;\\n};\\n\\ninterface MediaStreamEventMap {\\n    \"active\": Event;\\n    \"addtrack\": MediaStreamTrackEvent;\\n    \"inactive\": Event;\\n    \"removetrack\": MediaStreamTrackEvent;\\n}\\n\\ninterface MediaStream extends EventTarget {\\n    readonly active: boolean;\\n    readonly id: string;\\n    onactive: ((this: MediaStream, ev: Event) => any) | null;\\n    onaddtrack: ((this: MediaStream, ev: MediaStreamTrackEvent) => any) | null;\\n    oninactive: ((this: MediaStream, ev: Event) => any) | null;\\n    onremovetrack: ((this: MediaStream, ev: MediaStreamTrackEvent) => any) | null;\\n    addTrack(track: MediaStreamTrack): void;\\n    clone(): MediaStream;\\n    getAudioTracks(): MediaStreamTrack[];\\n    getTrackById(trackId: string): MediaStreamTrack | null;\\n    getTracks(): MediaStreamTrack[];\\n    getVideoTracks(): MediaStreamTrack[];\\n    removeTrack(track: MediaStreamTrack): void;\\n    stop(): void;\\n    addEventListener<K extends keyof MediaStreamEventMap>(type: K, listener: (this: MediaStream, ev: MediaStreamEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaStreamEventMap>(type: K, listener: (this: MediaStream, ev: MediaStreamEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaStream: {\\n    prototype: MediaStream;\\n    new(): MediaStream;\\n    new(stream: MediaStream): MediaStream;\\n    new(tracks: MediaStreamTrack[]): MediaStream;\\n};\\n\\ninterface MediaStreamAudioDestinationNode extends AudioNode {\\n    readonly stream: MediaStream;\\n}\\n\\ndeclare var MediaStreamAudioDestinationNode: {\\n    prototype: MediaStreamAudioDestinationNode;\\n    new(context: AudioContext, options?: AudioNodeOptions): MediaStreamAudioDestinationNode;\\n};\\n\\ninterface MediaStreamAudioSourceNode extends AudioNode {\\n    readonly mediaStream: MediaStream;\\n}\\n\\ndeclare var MediaStreamAudioSourceNode: {\\n    prototype: MediaStreamAudioSourceNode;\\n    new(context: AudioContext, options: MediaStreamAudioSourceOptions): MediaStreamAudioSourceNode;\\n};\\n\\ninterface MediaStreamError {\\n    readonly constraintName: string | null;\\n    readonly message: string | null;\\n    readonly name: string;\\n}\\n\\ndeclare var MediaStreamError: {\\n    prototype: MediaStreamError;\\n    new(): MediaStreamError;\\n};\\n\\ninterface MediaStreamErrorEvent extends Event {\\n    readonly error: MediaStreamError | null;\\n}\\n\\ndeclare var MediaStreamErrorEvent: {\\n    prototype: MediaStreamErrorEvent;\\n    new(typeArg: string, eventInitDict?: MediaStreamErrorEventInit): MediaStreamErrorEvent;\\n};\\n\\ninterface MediaStreamEvent extends Event {\\n    readonly stream: MediaStream | null;\\n}\\n\\ndeclare var MediaStreamEvent: {\\n    prototype: MediaStreamEvent;\\n    new(type: string, eventInitDict: MediaStreamEventInit): MediaStreamEvent;\\n};\\n\\ninterface MediaStreamTrackEventMap {\\n    \"ended\": MediaStreamErrorEvent;\\n    \"isolationchange\": Event;\\n    \"mute\": Event;\\n    \"overconstrained\": MediaStreamErrorEvent;\\n    \"unmute\": Event;\\n}\\n\\ninterface MediaStreamTrack extends EventTarget {\\n    enabled: boolean;\\n    readonly id: string;\\n    readonly isolated: boolean;\\n    readonly kind: string;\\n    readonly label: string;\\n    readonly muted: boolean;\\n    onended: ((this: MediaStreamTrack, ev: MediaStreamErrorEvent) => any) | null;\\n    onisolationchange: ((this: MediaStreamTrack, ev: Event) => any) | null;\\n    onmute: ((this: MediaStreamTrack, ev: Event) => any) | null;\\n    onoverconstrained: ((this: MediaStreamTrack, ev: MediaStreamErrorEvent) => any) | null;\\n    onunmute: ((this: MediaStreamTrack, ev: Event) => any) | null;\\n    readonly readonly: boolean;\\n    readonly readyState: MediaStreamTrackState;\\n    readonly remote: boolean;\\n    applyConstraints(constraints: MediaTrackConstraints): Promise<void>;\\n    clone(): MediaStreamTrack;\\n    getCapabilities(): MediaTrackCapabilities;\\n    getConstraints(): MediaTrackConstraints;\\n    getSettings(): MediaTrackSettings;\\n    stop(): void;\\n    addEventListener<K extends keyof MediaStreamTrackEventMap>(type: K, listener: (this: MediaStreamTrack, ev: MediaStreamTrackEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaStreamTrackEventMap>(type: K, listener: (this: MediaStreamTrack, ev: MediaStreamTrackEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaStreamTrack: {\\n    prototype: MediaStreamTrack;\\n    new(): MediaStreamTrack;\\n};\\n\\ninterface MediaStreamTrackAudioSourceNode extends AudioNode {\\n}\\n\\ndeclare var MediaStreamTrackAudioSourceNode: {\\n    prototype: MediaStreamTrackAudioSourceNode;\\n    new(context: AudioContext, options: MediaStreamTrackAudioSourceOptions): MediaStreamTrackAudioSourceNode;\\n};\\n\\ninterface MediaStreamTrackEvent extends Event {\\n    readonly track: MediaStreamTrack;\\n}\\n\\ndeclare var MediaStreamTrackEvent: {\\n    prototype: MediaStreamTrackEvent;\\n    new(typeArg: string, eventInitDict?: MediaStreamTrackEventInit): MediaStreamTrackEvent;\\n};\\n\\ninterface MessageChannel {\\n    readonly port1: MessagePort;\\n    readonly port2: MessagePort;\\n}\\n\\ndeclare var MessageChannel: {\\n    prototype: MessageChannel;\\n    new(): MessageChannel;\\n};\\n\\ninterface MessageEvent extends Event {\\n    /**\\n     * Returns the data of the message.\\n     */\\n    readonly data: any;\\n    /**\\n     * Returns the last event ID string, for\\n     * server-sent events.\\n     */\\n    readonly lastEventId: string;\\n    /**\\n     * Returns the origin of the message, for server-sent events and\\n     * cross-document messaging.\\n     */\\n    readonly origin: string;\\n    /**\\n     * Returns the MessagePort array sent with the message, for cross-document\\n     * messaging and channel messaging.\\n     */\\n    readonly ports: ReadonlyArray<MessagePort>;\\n    /**\\n     * Returns the WindowProxy of the source window, for cross-document\\n     * messaging, and the MessagePort being attached, in the connect event fired at\\n     * SharedWorkerGlobalScope objects.\\n     */\\n    readonly source: MessageEventSource | null;\\n}\\n\\ndeclare var MessageEvent: {\\n    prototype: MessageEvent;\\n    new(type: string, eventInitDict?: MessageEventInit): MessageEvent;\\n};\\n\\ninterface MessagePortEventMap {\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n}\\n\\ninterface MessagePort extends EventTarget {\\n    onmessage: ((this: MessagePort, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: MessagePort, ev: MessageEvent) => any) | null;\\n    /**\\n     * Disconnects the port, so that it is no longer active.\\n     */\\n    close(): void;\\n    /**\\n     * Posts a message through the channel. Objects listed in transfer are\\n     * transferred, not just cloned, meaning that they are no longer usable on the sending side.\\n     * Throws a \"DataCloneError\" DOMException if\\n     * transfer contains duplicate objects or port, or if message\\n     * could not be cloned.\\n     */\\n    postMessage(message: any, transfer?: Transferable[]): void;\\n    /**\\n     * Begins dispatching messages received on the port.\\n     */\\n    start(): void;\\n    addEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MessagePort: {\\n    prototype: MessagePort;\\n    new(): MessagePort;\\n};\\n\\ninterface MimeType {\\n    readonly description: string;\\n    readonly enabledPlugin: Plugin;\\n    readonly suffixes: string;\\n    readonly type: string;\\n}\\n\\ndeclare var MimeType: {\\n    prototype: MimeType;\\n    new(): MimeType;\\n};\\n\\ninterface MimeTypeArray {\\n    readonly length: number;\\n    item(index: number): Plugin;\\n    namedItem(type: string): Plugin;\\n    [index: number]: Plugin;\\n}\\n\\ndeclare var MimeTypeArray: {\\n    prototype: MimeTypeArray;\\n    new(): MimeTypeArray;\\n};\\n\\ninterface MouseEvent extends UIEvent {\\n    readonly altKey: boolean;\\n    readonly button: number;\\n    readonly buttons: number;\\n    readonly clientX: number;\\n    readonly clientY: number;\\n    readonly ctrlKey: boolean;\\n    /** @deprecated */\\n    readonly fromElement: Element;\\n    readonly layerX: number;\\n    readonly layerY: number;\\n    readonly metaKey: boolean;\\n    readonly movementX: number;\\n    readonly movementY: number;\\n    readonly offsetX: number;\\n    readonly offsetY: number;\\n    readonly pageX: number;\\n    readonly pageY: number;\\n    readonly relatedTarget: EventTarget;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly shiftKey: boolean;\\n    /** @deprecated */\\n    readonly toElement: Element;\\n    /** @deprecated */\\n    readonly which: number;\\n    readonly x: number;\\n    readonly y: number;\\n    getModifierState(keyArg: string): boolean;\\n    initMouseEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, ctrlKeyArg: boolean, altKeyArg: boolean, shiftKeyArg: boolean, metaKeyArg: boolean, buttonArg: number, relatedTargetArg: EventTarget | null): void;\\n}\\n\\ndeclare var MouseEvent: {\\n    prototype: MouseEvent;\\n    new(typeArg: string, eventInitDict?: MouseEventInit): MouseEvent;\\n};\\n\\ninterface MutationEvent extends Event {\\n    readonly attrChange: number;\\n    readonly attrName: string;\\n    readonly newValue: string;\\n    readonly prevValue: string;\\n    readonly relatedNode: Node;\\n    initMutationEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, relatedNodeArg: Node, prevValueArg: string, newValueArg: string, attrNameArg: string, attrChangeArg: number): void;\\n    readonly ADDITION: number;\\n    readonly MODIFICATION: number;\\n    readonly REMOVAL: number;\\n}\\n\\ndeclare var MutationEvent: {\\n    prototype: MutationEvent;\\n    new(): MutationEvent;\\n    readonly ADDITION: number;\\n    readonly MODIFICATION: number;\\n    readonly REMOVAL: number;\\n};\\n\\ninterface MutationObserver {\\n    disconnect(): void;\\n    /**\\n     * Instructs the user agent to observe a given target (a node) and report any mutations based on\\n     * the criteria given by options (an object).\\n     * The options argument allows for setting mutation\\n     * observation options via object members. These are the object members that\\n     * can be used:\\n     * childList\\n     * Set to true if mutations to target\\'s children are to be observed.\\n     * attributes\\n     * Set to true if mutations to target\\'s attributes are to be observed. Can be omitted if attributeOldValue or attributeFilter is\\n     * specified.\\n     * characterData\\n     * Set to true if mutations to target\\'s data are to be observed. Can be omitted if characterDataOldValue is specified.\\n     * subtree\\n     * Set to true if mutations to not just target, but\\n     * also target\\'s descendants are to be\\n     * observed.\\n     * attributeOldValue\\n     * Set to true if attributes is true or omitted\\n     * and target\\'s attribute value before the mutation\\n     * needs to be recorded.\\n     * characterDataOldValue\\n     * Set to true if characterData is set to true or omitted and target\\'s data before the mutation\\n     * needs to be recorded.\\n     * attributeFilter\\n     * Set to a list of attribute local names (without namespace) if not all attribute mutations need to be\\n     * observed and attributes is true\\n     * or omitted.\\n     */\\n    observe(target: Node, options?: MutationObserverInit): void;\\n    /**\\n     * Empties the record queue and\\n     * returns what was in there.\\n     */\\n    takeRecords(): MutationRecord[];\\n}\\n\\ndeclare var MutationObserver: {\\n    prototype: MutationObserver;\\n    new(callback: MutationCallback): MutationObserver;\\n};\\n\\ninterface MutationRecord {\\n    readonly addedNodes: NodeList;\\n    /**\\n     * Returns the local name of the\\n     * changed attribute, and null otherwise.\\n     */\\n    readonly attributeName: string | null;\\n    /**\\n     * Returns the namespace of the\\n     * changed attribute, and null otherwise.\\n     */\\n    readonly attributeNamespace: string | null;\\n    /**\\n     * Return the previous and next sibling respectively\\n     * of the added or removed nodes, and null otherwise.\\n     */\\n    readonly nextSibling: Node | null;\\n    /**\\n     * The return value depends on type. For\\n     * \"attributes\", it is the value of the\\n     * changed attribute before the change.\\n     * For \"characterData\", it is the data of the changed node before the change. For\\n     * \"childList\", it is null.\\n     */\\n    readonly oldValue: string | null;\\n    readonly previousSibling: Node | null;\\n    /**\\n     * Return the nodes added and removed\\n     * respectively.\\n     */\\n    readonly removedNodes: NodeList;\\n    readonly target: Node;\\n    /**\\n     * Returns \"attributes\" if it was an attribute mutation.\\n     * \"characterData\" if it was a mutation to a CharacterData node. And\\n     * \"childList\" if it was a mutation to the tree of nodes.\\n     */\\n    readonly type: MutationRecordType;\\n}\\n\\ndeclare var MutationRecord: {\\n    prototype: MutationRecord;\\n    new(): MutationRecord;\\n};\\n\\ninterface NamedNodeMap {\\n    readonly length: number;\\n    getNamedItem(qualifiedName: string): Attr | null;\\n    getNamedItemNS(namespace: string | null, localName: string): Attr | null;\\n    item(index: number): Attr | null;\\n    removeNamedItem(qualifiedName: string): Attr;\\n    removeNamedItemNS(namespace: string | null, localName: string): Attr;\\n    setNamedItem(attr: Attr): Attr | null;\\n    setNamedItemNS(attr: Attr): Attr | null;\\n    [index: number]: Attr;\\n}\\n\\ndeclare var NamedNodeMap: {\\n    prototype: NamedNodeMap;\\n    new(): NamedNodeMap;\\n};\\n\\ninterface NavigationPreloadManager {\\n    disable(): Promise<void>;\\n    enable(): Promise<void>;\\n    getState(): Promise<NavigationPreloadState>;\\n    setHeaderValue(value: string): Promise<void>;\\n}\\n\\ndeclare var NavigationPreloadManager: {\\n    prototype: NavigationPreloadManager;\\n    new(): NavigationPreloadManager;\\n};\\n\\ninterface Navigator extends NavigatorID, NavigatorOnLine, NavigatorContentUtils, NavigatorStorageUtils, MSNavigatorDoNotTrack, MSFileSaver, NavigatorBeacon, NavigatorConcurrentHardware, NavigatorUserMedia, NavigatorLanguage, NavigatorStorage, NavigatorAutomationInformation {\\n    readonly activeVRDisplays: ReadonlyArray<VRDisplay>;\\n    readonly authentication: WebAuthentication;\\n    readonly cookieEnabled: boolean;\\n    readonly doNotTrack: string | null;\\n    gamepadInputEmulation: GamepadInputEmulationType;\\n    readonly geolocation: Geolocation;\\n    readonly maxTouchPoints: number;\\n    readonly mimeTypes: MimeTypeArray;\\n    readonly msManipulationViewsEnabled: boolean;\\n    readonly msMaxTouchPoints: number;\\n    readonly msPointerEnabled: boolean;\\n    readonly plugins: PluginArray;\\n    readonly pointerEnabled: boolean;\\n    readonly serviceWorker: ServiceWorkerContainer;\\n    readonly webdriver: boolean;\\n    getGamepads(): (Gamepad | null)[];\\n    getVRDisplays(): Promise<VRDisplay[]>;\\n    javaEnabled(): boolean;\\n    msLaunchUri(uri: string, successCallback?: MSLaunchUriCallback, noHandlerCallback?: MSLaunchUriCallback): void;\\n    requestMediaKeySystemAccess(keySystem: string, supportedConfigurations: MediaKeySystemConfiguration[]): Promise<MediaKeySystemAccess>;\\n    vibrate(pattern: number | number[]): boolean;\\n}\\n\\ndeclare var Navigator: {\\n    prototype: Navigator;\\n    new(): Navigator;\\n};\\n\\ninterface NavigatorAutomationInformation {\\n    readonly webdriver: boolean;\\n}\\n\\ninterface NavigatorBeacon {\\n    sendBeacon(url: string, data?: Blob | Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | FormData | string | null): boolean;\\n}\\n\\ninterface NavigatorConcurrentHardware {\\n    readonly hardwareConcurrency: number;\\n}\\n\\ninterface NavigatorContentUtils {\\n}\\n\\ninterface NavigatorID {\\n    readonly appCodeName: string;\\n    readonly appName: string;\\n    readonly appVersion: string;\\n    readonly platform: string;\\n    readonly product: string;\\n    readonly productSub: string;\\n    readonly userAgent: string;\\n    readonly vendor: string;\\n    readonly vendorSub: string;\\n}\\n\\ninterface NavigatorLanguage {\\n    readonly language: string;\\n    readonly languages: ReadonlyArray<string>;\\n}\\n\\ninterface NavigatorOnLine {\\n    readonly onLine: boolean;\\n}\\n\\ninterface NavigatorStorage {\\n    readonly storage: StorageManager;\\n}\\n\\ninterface NavigatorStorageUtils {\\n}\\n\\ninterface NavigatorUserMedia {\\n    readonly mediaDevices: MediaDevices;\\n    getDisplayMedia(constraints: MediaStreamConstraints): Promise<MediaStream>;\\n    getUserMedia(constraints: MediaStreamConstraints, successCallback: NavigatorUserMediaSuccessCallback, errorCallback: NavigatorUserMediaErrorCallback): void;\\n}\\n\\ninterface Node extends EventTarget {\\n    /**\\n     * Returns node\\'s node document\\'s document base URL.\\n     */\\n    readonly baseURI: string;\\n    /**\\n     * Returns the children.\\n     */\\n    readonly childNodes: NodeListOf<ChildNode>;\\n    /**\\n     * Returns the first child.\\n     */\\n    readonly firstChild: ChildNode | null;\\n    /**\\n     * Returns true if node is connected and false otherwise.\\n     */\\n    readonly isConnected: boolean;\\n    /**\\n     * Returns the last child.\\n     */\\n    readonly lastChild: ChildNode | null;\\n    readonly namespaceURI: string | null;\\n    /**\\n     * Returns the next sibling.\\n     */\\n    readonly nextSibling: Node | null;\\n    /**\\n     * Returns a string appropriate for the type of node, as\\n     * follows:\\n     * Element\\n     * Its HTML-uppercased qualified name.\\n     * Attr\\n     * Its qualified name.\\n     * Text\\n     * \"#text\".\\n     * CDATASection\\n     * \"#cdata-section\".\\n     * ProcessingInstruction\\n     * Its target.\\n     * Comment\\n     * \"#comment\".\\n     * Document\\n     * \"#document\".\\n     * DocumentType\\n     * Its name.\\n     * DocumentFragment\\n     * \"#document-fragment\".\\n     */\\n    readonly nodeName: string;\\n    readonly nodeType: number;\\n    nodeValue: string | null;\\n    /**\\n     * Returns the node document.\\n     * Returns null for documents.\\n     */\\n    readonly ownerDocument: Document | null;\\n    /**\\n     * Returns the parent element.\\n     */\\n    readonly parentElement: HTMLElement | null;\\n    /**\\n     * Returns the parent.\\n     */\\n    readonly parentNode: Node & ParentNode | null;\\n    /**\\n     * Returns the previous sibling.\\n     */\\n    readonly previousSibling: Node | null;\\n    textContent: string | null;\\n    appendChild<T extends Node>(newChild: T): T;\\n    /**\\n     * Returns a copy of node. If deep is true, the copy also includes the node\\'s descendants.\\n     */\\n    cloneNode(deep?: boolean): Node;\\n    compareDocumentPosition(other: Node): number;\\n    /**\\n     * Returns true if other is an inclusive descendant of node, and false otherwise.\\n     */\\n    contains(other: Node | null): boolean;\\n    /**\\n     * Returns node\\'s shadow-including root.\\n     */\\n    getRootNode(options?: GetRootNodeOptions): Node;\\n    /**\\n     * Returns whether node has children.\\n     */\\n    hasChildNodes(): boolean;\\n    insertBefore<T extends Node>(newChild: T, refChild: Node | null): T;\\n    isDefaultNamespace(namespace: string | null): boolean;\\n    /**\\n     * Returns whether node and otherNode have the same properties.\\n     */\\n    isEqualNode(otherNode: Node | null): boolean;\\n    isSameNode(otherNode: Node | null): boolean;\\n    lookupNamespaceURI(prefix: string | null): string | null;\\n    lookupPrefix(namespace: string | null): string | null;\\n    /**\\n     * Removes empty exclusive Text nodes and concatenates the data of remaining contiguous exclusive Text nodes into the first of their nodes.\\n     */\\n    normalize(): void;\\n    removeChild<T extends Node>(oldChild: T): T;\\n    replaceChild<T extends Node>(newChild: Node, oldChild: T): T;\\n    readonly ATTRIBUTE_NODE: number;\\n    readonly CDATA_SECTION_NODE: number;\\n    readonly COMMENT_NODE: number;\\n    readonly DOCUMENT_FRAGMENT_NODE: number;\\n    readonly DOCUMENT_NODE: number;\\n    readonly DOCUMENT_POSITION_CONTAINED_BY: number;\\n    readonly DOCUMENT_POSITION_CONTAINS: number;\\n    readonly DOCUMENT_POSITION_DISCONNECTED: number;\\n    readonly DOCUMENT_POSITION_FOLLOWING: number;\\n    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;\\n    readonly DOCUMENT_POSITION_PRECEDING: number;\\n    readonly DOCUMENT_TYPE_NODE: number;\\n    readonly ELEMENT_NODE: number;\\n    readonly ENTITY_NODE: number;\\n    readonly ENTITY_REFERENCE_NODE: number;\\n    readonly NOTATION_NODE: number;\\n    readonly PROCESSING_INSTRUCTION_NODE: number;\\n    readonly TEXT_NODE: number;\\n}\\n\\ndeclare var Node: {\\n    prototype: Node;\\n    new(): Node;\\n    readonly ATTRIBUTE_NODE: number;\\n    readonly CDATA_SECTION_NODE: number;\\n    readonly COMMENT_NODE: number;\\n    readonly DOCUMENT_FRAGMENT_NODE: number;\\n    readonly DOCUMENT_NODE: number;\\n    readonly DOCUMENT_POSITION_CONTAINED_BY: number;\\n    readonly DOCUMENT_POSITION_CONTAINS: number;\\n    readonly DOCUMENT_POSITION_DISCONNECTED: number;\\n    readonly DOCUMENT_POSITION_FOLLOWING: number;\\n    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;\\n    readonly DOCUMENT_POSITION_PRECEDING: number;\\n    readonly DOCUMENT_TYPE_NODE: number;\\n    readonly ELEMENT_NODE: number;\\n    readonly ENTITY_NODE: number;\\n    readonly ENTITY_REFERENCE_NODE: number;\\n    readonly NOTATION_NODE: number;\\n    readonly PROCESSING_INSTRUCTION_NODE: number;\\n    readonly TEXT_NODE: number;\\n};\\n\\ninterface NodeFilter {\\n    acceptNode(node: Node): number;\\n}\\n\\ndeclare var NodeFilter: {\\n    readonly FILTER_ACCEPT: number;\\n    readonly FILTER_REJECT: number;\\n    readonly FILTER_SKIP: number;\\n    readonly SHOW_ALL: number;\\n    readonly SHOW_ATTRIBUTE: number;\\n    readonly SHOW_CDATA_SECTION: number;\\n    readonly SHOW_COMMENT: number;\\n    readonly SHOW_DOCUMENT: number;\\n    readonly SHOW_DOCUMENT_FRAGMENT: number;\\n    readonly SHOW_DOCUMENT_TYPE: number;\\n    readonly SHOW_ELEMENT: number;\\n    readonly SHOW_ENTITY: number;\\n    readonly SHOW_ENTITY_REFERENCE: number;\\n    readonly SHOW_NOTATION: number;\\n    readonly SHOW_PROCESSING_INSTRUCTION: number;\\n    readonly SHOW_TEXT: number;\\n};\\n\\ninterface NodeIterator {\\n    readonly filter: NodeFilter | null;\\n    readonly pointerBeforeReferenceNode: boolean;\\n    readonly referenceNode: Node;\\n    readonly root: Node;\\n    readonly whatToShow: number;\\n    detach(): void;\\n    nextNode(): Node | null;\\n    previousNode(): Node | null;\\n}\\n\\ndeclare var NodeIterator: {\\n    prototype: NodeIterator;\\n    new(): NodeIterator;\\n};\\n\\ninterface NodeList {\\n    /**\\n     * Returns the number of nodes in the collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * element = collection[index]\\n     */\\n    item(index: number): Node | null;\\n    /**\\n     * Performs the specified action for each node in an list.\\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the list.\\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    forEach(callbackfn: (value: Node, key: number, parent: NodeList) => void, thisArg?: any): void;\\n    [index: number]: Node;\\n}\\n\\ndeclare var NodeList: {\\n    prototype: NodeList;\\n    new(): NodeList;\\n};\\n\\ninterface NodeListOf<TNode extends Node> extends NodeList {\\n    length: number;\\n    item(index: number): TNode;\\n    /**\\n     * Performs the specified action for each node in an list.\\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the list.\\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    forEach(callbackfn: (value: TNode, key: number, parent: NodeListOf<TNode>) => void, thisArg?: any): void;\\n    [index: number]: TNode;\\n}\\n\\ninterface NodeSelector {\\n    querySelector<K extends keyof HTMLElementTagNameMap>(selectors: K): HTMLElementTagNameMap[K] | null;\\n    querySelector<K extends keyof SVGElementTagNameMap>(selectors: K): SVGElementTagNameMap[K] | null;\\n    querySelector<E extends Element = Element>(selectors: string): E | null;\\n    querySelectorAll<K extends keyof HTMLElementTagNameMap>(selectors: K): NodeListOf<HTMLElementTagNameMap[K]>;\\n    querySelectorAll<K extends keyof SVGElementTagNameMap>(selectors: K): NodeListOf<SVGElementTagNameMap[K]>;\\n    querySelectorAll<E extends Element = Element>(selectors: string): NodeListOf<E>;\\n}\\n\\ninterface NonDocumentTypeChildNode {\\n    /**\\n     * Returns the first following sibling that\\n     * is an element, and null otherwise.\\n     */\\n    readonly nextElementSibling: Element | null;\\n    /**\\n     * Returns the first preceding sibling that\\n     * is an element, and null otherwise.\\n     */\\n    readonly previousElementSibling: Element | null;\\n}\\n\\ninterface NonElementParentNode {\\n    /**\\n     * Returns the first element within node\\'s descendants whose ID is elementId.\\n     */\\n    getElementById(elementId: string): Element | null;\\n}\\n\\ninterface NotificationEventMap {\\n    \"click\": Event;\\n    \"close\": Event;\\n    \"error\": Event;\\n    \"show\": Event;\\n}\\n\\ninterface Notification extends EventTarget {\\n    readonly actions: ReadonlyArray<NotificationAction>;\\n    readonly badge: string;\\n    readonly body: string;\\n    readonly data: any;\\n    readonly dir: NotificationDirection;\\n    readonly icon: string;\\n    readonly image: string;\\n    readonly lang: string;\\n    onclick: ((this: Notification, ev: Event) => any) | null;\\n    onclose: ((this: Notification, ev: Event) => any) | null;\\n    onerror: ((this: Notification, ev: Event) => any) | null;\\n    onshow: ((this: Notification, ev: Event) => any) | null;\\n    readonly renotify: boolean;\\n    readonly requireInteraction: boolean;\\n    readonly silent: boolean;\\n    readonly tag: string;\\n    readonly timestamp: number;\\n    readonly title: string;\\n    readonly vibrate: ReadonlyArray<number>;\\n    close(): void;\\n    addEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Notification: {\\n    prototype: Notification;\\n    new(title: string, options?: NotificationOptions): Notification;\\n    readonly maxActions: number;\\n    readonly permission: NotificationPermission;\\n    requestPermission(deprecatedCallback?: NotificationPermissionCallback): Promise<NotificationPermission>;\\n};\\n\\ninterface OES_element_index_uint {\\n}\\n\\ndeclare var OES_element_index_uint: {\\n    prototype: OES_element_index_uint;\\n    new(): OES_element_index_uint;\\n};\\n\\ninterface OES_standard_derivatives {\\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT_OES: number;\\n}\\n\\ndeclare var OES_standard_derivatives: {\\n    prototype: OES_standard_derivatives;\\n    new(): OES_standard_derivatives;\\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT_OES: number;\\n};\\n\\ninterface OES_texture_float {\\n}\\n\\ndeclare var OES_texture_float: {\\n    prototype: OES_texture_float;\\n    new(): OES_texture_float;\\n};\\n\\ninterface OES_texture_float_linear {\\n}\\n\\ndeclare var OES_texture_float_linear: {\\n    prototype: OES_texture_float_linear;\\n    new(): OES_texture_float_linear;\\n};\\n\\ninterface OES_texture_half_float {\\n    readonly HALF_FLOAT_OES: number;\\n}\\n\\ndeclare var OES_texture_half_float: {\\n    prototype: OES_texture_half_float;\\n    new(): OES_texture_half_float;\\n    readonly HALF_FLOAT_OES: number;\\n};\\n\\ninterface OES_texture_half_float_linear {\\n}\\n\\ndeclare var OES_texture_half_float_linear: {\\n    prototype: OES_texture_half_float_linear;\\n    new(): OES_texture_half_float_linear;\\n};\\n\\ninterface OES_vertex_array_object {\\n    readonly VERTEX_ARRAY_BINDING_OES: number;\\n    bindVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES): void;\\n    createVertexArrayOES(): WebGLVertexArrayObjectOES;\\n    deleteVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES): void;\\n    isVertexArrayOES(value: any): value is WebGLVertexArrayObjectOES;\\n}\\n\\ninterface OfflineAudioCompletionEvent extends Event {\\n    readonly renderedBuffer: AudioBuffer;\\n}\\n\\ndeclare var OfflineAudioCompletionEvent: {\\n    prototype: OfflineAudioCompletionEvent;\\n    new(type: string, eventInitDict: OfflineAudioCompletionEventInit): OfflineAudioCompletionEvent;\\n};\\n\\ninterface OfflineAudioContextEventMap extends BaseAudioContextEventMap {\\n    \"complete\": OfflineAudioCompletionEvent;\\n}\\n\\ninterface OfflineAudioContext extends BaseAudioContext {\\n    readonly length: number;\\n    oncomplete: ((this: OfflineAudioContext, ev: OfflineAudioCompletionEvent) => any) | null;\\n    startRendering(): Promise<AudioBuffer>;\\n    suspend(suspendTime: number): Promise<void>;\\n    addEventListener<K extends keyof OfflineAudioContextEventMap>(type: K, listener: (this: OfflineAudioContext, ev: OfflineAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof OfflineAudioContextEventMap>(type: K, listener: (this: OfflineAudioContext, ev: OfflineAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var OfflineAudioContext: {\\n    prototype: OfflineAudioContext;\\n    new(contextOptions: OfflineAudioContextOptions): OfflineAudioContext;\\n    new(numberOfChannels: number, length: number, sampleRate: number): OfflineAudioContext;\\n};\\n\\ninterface OscillatorNode extends AudioScheduledSourceNode {\\n    readonly detune: AudioParam;\\n    readonly frequency: AudioParam;\\n    type: OscillatorType;\\n    setPeriodicWave(periodicWave: PeriodicWave): void;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: OscillatorNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: OscillatorNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var OscillatorNode: {\\n    prototype: OscillatorNode;\\n    new(context: BaseAudioContext, options?: OscillatorOptions): OscillatorNode;\\n};\\n\\ninterface OverflowEvent extends UIEvent {\\n    readonly horizontalOverflow: boolean;\\n    readonly orient: number;\\n    readonly verticalOverflow: boolean;\\n    readonly BOTH: number;\\n    readonly HORIZONTAL: number;\\n    readonly VERTICAL: number;\\n}\\n\\ndeclare var OverflowEvent: {\\n    prototype: OverflowEvent;\\n    new(): OverflowEvent;\\n    readonly BOTH: number;\\n    readonly HORIZONTAL: number;\\n    readonly VERTICAL: number;\\n};\\n\\ninterface PageTransitionEvent extends Event {\\n    readonly persisted: boolean;\\n}\\n\\ndeclare var PageTransitionEvent: {\\n    prototype: PageTransitionEvent;\\n    new(): PageTransitionEvent;\\n};\\n\\ninterface PannerNode extends AudioNode {\\n    coneInnerAngle: number;\\n    coneOuterAngle: number;\\n    coneOuterGain: number;\\n    distanceModel: DistanceModelType;\\n    maxDistance: number;\\n    readonly orientationX: AudioParam;\\n    readonly orientationY: AudioParam;\\n    readonly orientationZ: AudioParam;\\n    panningModel: PanningModelType;\\n    readonly positionX: AudioParam;\\n    readonly positionY: AudioParam;\\n    readonly positionZ: AudioParam;\\n    refDistance: number;\\n    rolloffFactor: number;\\n    /** @deprecated */\\n    setOrientation(x: number, y: number, z: number): void;\\n    /** @deprecated */\\n    setPosition(x: number, y: number, z: number): void;\\n}\\n\\ndeclare var PannerNode: {\\n    prototype: PannerNode;\\n    new(context: BaseAudioContext, options?: PannerOptions): PannerNode;\\n};\\n\\ninterface ParentNode {\\n    readonly childElementCount: number;\\n    /**\\n     * Returns the child elements.\\n     */\\n    readonly children: HTMLCollection;\\n    /**\\n     * Returns the first child that is an element, and null otherwise.\\n     */\\n    readonly firstElementChild: Element | null;\\n    /**\\n     * Returns the last child that is an element, and null otherwise.\\n     */\\n    readonly lastElementChild: Element | null;\\n    /**\\n     * Inserts nodes after the last child of node, while replacing\\n     * strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    append(...nodes: (Node | string)[]): void;\\n    /**\\n     * Inserts nodes before the first child of node, while\\n     * replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    prepend(...nodes: (Node | string)[]): void;\\n    /**\\n     * Returns the first element that is a descendant of node that\\n     * matches selectors.\\n     */\\n    querySelector<K extends keyof HTMLElementTagNameMap>(selectors: K): HTMLElementTagNameMap[K] | null;\\n    querySelector<K extends keyof SVGElementTagNameMap>(selectors: K): SVGElementTagNameMap[K] | null;\\n    querySelector<E extends Element = Element>(selectors: string): E | null;\\n    /**\\n     * Returns all element descendants of node that\\n     * match selectors.\\n     */\\n    querySelectorAll<K extends keyof HTMLElementTagNameMap>(selectors: K): NodeListOf<HTMLElementTagNameMap[K]>;\\n    querySelectorAll<K extends keyof SVGElementTagNameMap>(selectors: K): NodeListOf<SVGElementTagNameMap[K]>;\\n    querySelectorAll<E extends Element = Element>(selectors: string): NodeListOf<E>;\\n}\\n\\ninterface Path2D extends CanvasPath {\\n    addPath(path: Path2D, transform?: DOMMatrix2DInit): void;\\n}\\n\\ndeclare var Path2D: {\\n    prototype: Path2D;\\n    new(path?: Path2D | string): Path2D;\\n};\\n\\ninterface PaymentAddress {\\n    readonly addressLine: string[];\\n    readonly city: string;\\n    readonly country: string;\\n    readonly dependentLocality: string;\\n    readonly languageCode: string;\\n    readonly organization: string;\\n    readonly phone: string;\\n    readonly postalCode: string;\\n    readonly recipient: string;\\n    readonly region: string;\\n    readonly sortingCode: string;\\n    toJSON(): any;\\n}\\n\\ndeclare var PaymentAddress: {\\n    prototype: PaymentAddress;\\n    new(): PaymentAddress;\\n};\\n\\ninterface PaymentRequestEventMap {\\n    \"shippingaddresschange\": Event;\\n    \"shippingoptionchange\": Event;\\n}\\n\\ninterface PaymentRequest extends EventTarget {\\n    readonly id: string;\\n    onshippingaddresschange: ((this: PaymentRequest, ev: Event) => any) | null;\\n    onshippingoptionchange: ((this: PaymentRequest, ev: Event) => any) | null;\\n    readonly shippingAddress: PaymentAddress | null;\\n    readonly shippingOption: string | null;\\n    readonly shippingType: PaymentShippingType | null;\\n    abort(): Promise<void>;\\n    canMakePayment(): Promise<boolean>;\\n    show(): Promise<PaymentResponse>;\\n    addEventListener<K extends keyof PaymentRequestEventMap>(type: K, listener: (this: PaymentRequest, ev: PaymentRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof PaymentRequestEventMap>(type: K, listener: (this: PaymentRequest, ev: PaymentRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var PaymentRequest: {\\n    prototype: PaymentRequest;\\n    new(methodData: PaymentMethodData[], details: PaymentDetailsInit, options?: PaymentOptions): PaymentRequest;\\n};\\n\\ninterface PaymentRequestUpdateEvent extends Event {\\n    updateWith(detailsPromise: Promise<PaymentDetailsUpdate>): void;\\n}\\n\\ndeclare var PaymentRequestUpdateEvent: {\\n    prototype: PaymentRequestUpdateEvent;\\n    new(type: string, eventInitDict?: PaymentRequestUpdateEventInit): PaymentRequestUpdateEvent;\\n};\\n\\ninterface PaymentResponse {\\n    readonly details: any;\\n    readonly methodName: string;\\n    readonly payerEmail: string | null;\\n    readonly payerName: string | null;\\n    readonly payerPhone: string | null;\\n    readonly requestId: string;\\n    readonly shippingAddress: PaymentAddress | null;\\n    readonly shippingOption: string | null;\\n    complete(result?: PaymentComplete): Promise<void>;\\n    toJSON(): any;\\n}\\n\\ndeclare var PaymentResponse: {\\n    prototype: PaymentResponse;\\n    new(): PaymentResponse;\\n};\\n\\ninterface PerfWidgetExternal {\\n    readonly activeNetworkRequestCount: number;\\n    readonly averageFrameTime: number;\\n    readonly averagePaintTime: number;\\n    readonly extraInformationEnabled: boolean;\\n    readonly independentRenderingEnabled: boolean;\\n    readonly irDisablingContentString: string;\\n    readonly irStatusAvailable: boolean;\\n    readonly maxCpuSpeed: number;\\n    readonly paintRequestsPerSecond: number;\\n    readonly performanceCounter: number;\\n    readonly performanceCounterFrequency: number;\\n    addEventListener(eventType: string, callback: Function): void;\\n    getMemoryUsage(): number;\\n    getProcessCpuUsage(): number;\\n    getRecentCpuUsage(last: number | null): any;\\n    getRecentFrames(last: number | null): any;\\n    getRecentMemoryUsage(last: number | null): any;\\n    getRecentPaintRequests(last: number | null): any;\\n    removeEventListener(eventType: string, callback: Function): void;\\n    repositionWindow(x: number, y: number): void;\\n    resizeWindow(width: number, height: number): void;\\n}\\n\\ndeclare var PerfWidgetExternal: {\\n    prototype: PerfWidgetExternal;\\n    new(): PerfWidgetExternal;\\n};\\n\\ninterface PerformanceEventMap {\\n    \"resourcetimingbufferfull\": Event;\\n}\\n\\ninterface Performance extends EventTarget {\\n    /** @deprecated */\\n    readonly navigation: PerformanceNavigation;\\n    onresourcetimingbufferfull: ((this: Performance, ev: Event) => any) | null;\\n    readonly timeOrigin: number;\\n    /** @deprecated */\\n    readonly timing: PerformanceTiming;\\n    clearMarks(markName?: string): void;\\n    clearMeasures(measureName?: string): void;\\n    clearResourceTimings(): void;\\n    getEntries(): PerformanceEntryList;\\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\\n    getEntriesByType(type: string): PerformanceEntryList;\\n    mark(markName: string): void;\\n    measure(measureName: string, startMark?: string, endMark?: string): void;\\n    now(): number;\\n    setResourceTimingBufferSize(maxSize: number): void;\\n    toJSON(): any;\\n    addEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Performance: {\\n    prototype: Performance;\\n    new(): Performance;\\n};\\n\\ninterface PerformanceEntry {\\n    readonly duration: number;\\n    readonly entryType: string;\\n    readonly name: string;\\n    readonly startTime: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceEntry: {\\n    prototype: PerformanceEntry;\\n    new(): PerformanceEntry;\\n};\\n\\ninterface PerformanceMark extends PerformanceEntry {\\n}\\n\\ndeclare var PerformanceMark: {\\n    prototype: PerformanceMark;\\n    new(): PerformanceMark;\\n};\\n\\ninterface PerformanceMeasure extends PerformanceEntry {\\n}\\n\\ndeclare var PerformanceMeasure: {\\n    prototype: PerformanceMeasure;\\n    new(): PerformanceMeasure;\\n};\\n\\ninterface PerformanceNavigation {\\n    readonly redirectCount: number;\\n    readonly type: number;\\n    toJSON(): any;\\n    readonly TYPE_BACK_FORWARD: number;\\n    readonly TYPE_NAVIGATE: number;\\n    readonly TYPE_RELOAD: number;\\n    readonly TYPE_RESERVED: number;\\n}\\n\\ndeclare var PerformanceNavigation: {\\n    prototype: PerformanceNavigation;\\n    new(): PerformanceNavigation;\\n    readonly TYPE_BACK_FORWARD: number;\\n    readonly TYPE_NAVIGATE: number;\\n    readonly TYPE_RELOAD: number;\\n    readonly TYPE_RESERVED: number;\\n};\\n\\ninterface PerformanceNavigationTiming extends PerformanceResourceTiming {\\n    readonly domComplete: number;\\n    readonly domContentLoadedEventEnd: number;\\n    readonly domContentLoadedEventStart: number;\\n    readonly domInteractive: number;\\n    readonly loadEventEnd: number;\\n    readonly loadEventStart: number;\\n    readonly redirectCount: number;\\n    readonly type: NavigationType;\\n    readonly unloadEventEnd: number;\\n    readonly unloadEventStart: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceNavigationTiming: {\\n    prototype: PerformanceNavigationTiming;\\n    new(): PerformanceNavigationTiming;\\n};\\n\\ninterface PerformanceObserver {\\n    disconnect(): void;\\n    observe(options: PerformanceObserverInit): void;\\n    takeRecords(): PerformanceEntryList;\\n}\\n\\ndeclare var PerformanceObserver: {\\n    prototype: PerformanceObserver;\\n    new(callback: PerformanceObserverCallback): PerformanceObserver;\\n};\\n\\ninterface PerformanceObserverEntryList {\\n    getEntries(): PerformanceEntryList;\\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\\n    getEntriesByType(type: string): PerformanceEntryList;\\n}\\n\\ndeclare var PerformanceObserverEntryList: {\\n    prototype: PerformanceObserverEntryList;\\n    new(): PerformanceObserverEntryList;\\n};\\n\\ninterface PerformanceResourceTiming extends PerformanceEntry {\\n    readonly connectEnd: number;\\n    readonly connectStart: number;\\n    readonly decodedBodySize: number;\\n    readonly domainLookupEnd: number;\\n    readonly domainLookupStart: number;\\n    readonly encodedBodySize: number;\\n    readonly fetchStart: number;\\n    readonly initiatorType: string;\\n    readonly nextHopProtocol: string;\\n    readonly redirectEnd: number;\\n    readonly redirectStart: number;\\n    readonly requestStart: number;\\n    readonly responseEnd: number;\\n    readonly responseStart: number;\\n    readonly secureConnectionStart: number;\\n    readonly transferSize: number;\\n    readonly workerStart: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceResourceTiming: {\\n    prototype: PerformanceResourceTiming;\\n    new(): PerformanceResourceTiming;\\n};\\n\\ninterface PerformanceTiming {\\n    readonly connectEnd: number;\\n    readonly connectStart: number;\\n    readonly domComplete: number;\\n    readonly domContentLoadedEventEnd: number;\\n    readonly domContentLoadedEventStart: number;\\n    readonly domInteractive: number;\\n    readonly domLoading: number;\\n    readonly domainLookupEnd: number;\\n    readonly domainLookupStart: number;\\n    readonly fetchStart: number;\\n    readonly loadEventEnd: number;\\n    readonly loadEventStart: number;\\n    readonly navigationStart: number;\\n    readonly redirectEnd: number;\\n    readonly redirectStart: number;\\n    readonly requestStart: number;\\n    readonly responseEnd: number;\\n    readonly responseStart: number;\\n    readonly secureConnectionStart: number;\\n    readonly unloadEventEnd: number;\\n    readonly unloadEventStart: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceTiming: {\\n    prototype: PerformanceTiming;\\n    new(): PerformanceTiming;\\n};\\n\\ninterface PeriodicWave {\\n}\\n\\ndeclare var PeriodicWave: {\\n    prototype: PeriodicWave;\\n    new(context: BaseAudioContext, options?: PeriodicWaveOptions): PeriodicWave;\\n};\\n\\ninterface PermissionRequest extends DeferredPermissionRequest {\\n    readonly state: MSWebViewPermissionState;\\n    defer(): void;\\n}\\n\\ndeclare var PermissionRequest: {\\n    prototype: PermissionRequest;\\n    new(): PermissionRequest;\\n};\\n\\ninterface PermissionRequestedEvent extends Event {\\n    readonly permissionRequest: PermissionRequest;\\n}\\n\\ndeclare var PermissionRequestedEvent: {\\n    prototype: PermissionRequestedEvent;\\n    new(): PermissionRequestedEvent;\\n};\\n\\ninterface Plugin {\\n    readonly description: string;\\n    readonly filename: string;\\n    readonly length: number;\\n    readonly name: string;\\n    readonly version: string;\\n    item(index: number): MimeType;\\n    namedItem(type: string): MimeType;\\n    [index: number]: MimeType;\\n}\\n\\ndeclare var Plugin: {\\n    prototype: Plugin;\\n    new(): Plugin;\\n};\\n\\ninterface PluginArray {\\n    readonly length: number;\\n    item(index: number): Plugin;\\n    namedItem(name: string): Plugin;\\n    refresh(reload?: boolean): void;\\n    [index: number]: Plugin;\\n}\\n\\ndeclare var PluginArray: {\\n    prototype: PluginArray;\\n    new(): PluginArray;\\n};\\n\\ninterface PointerEvent extends MouseEvent {\\n    readonly height: number;\\n    readonly isPrimary: boolean;\\n    readonly pointerId: number;\\n    readonly pointerType: string;\\n    readonly pressure: number;\\n    readonly tangentialPressure: number;\\n    readonly tiltX: number;\\n    readonly tiltY: number;\\n    readonly twist: number;\\n    readonly width: number;\\n}\\n\\ndeclare var PointerEvent: {\\n    prototype: PointerEvent;\\n    new(type: string, eventInitDict?: PointerEventInit): PointerEvent;\\n};\\n\\ninterface PopStateEvent extends Event {\\n    readonly state: any;\\n}\\n\\ndeclare var PopStateEvent: {\\n    prototype: PopStateEvent;\\n    new(type: string, eventInitDict?: PopStateEventInit): PopStateEvent;\\n};\\n\\ninterface Position {\\n    readonly coords: Coordinates;\\n    readonly timestamp: number;\\n}\\n\\ninterface PositionError {\\n    readonly code: number;\\n    readonly message: string;\\n    readonly PERMISSION_DENIED: number;\\n    readonly POSITION_UNAVAILABLE: number;\\n    readonly TIMEOUT: number;\\n}\\n\\ninterface ProcessingInstruction extends CharacterData {\\n    readonly target: string;\\n}\\n\\ndeclare var ProcessingInstruction: {\\n    prototype: ProcessingInstruction;\\n    new(): ProcessingInstruction;\\n};\\n\\ninterface ProgressEvent extends Event {\\n    readonly lengthComputable: boolean;\\n    readonly loaded: number;\\n    readonly total: number;\\n}\\n\\ndeclare var ProgressEvent: {\\n    prototype: ProgressEvent;\\n    new(type: string, eventInitDict?: ProgressEventInit): ProgressEvent;\\n};\\n\\ninterface PromiseRejectionEvent extends Event {\\n    readonly promise: Promise<any>;\\n    readonly reason: any;\\n}\\n\\ndeclare var PromiseRejectionEvent: {\\n    prototype: PromiseRejectionEvent;\\n    new(type: string, eventInitDict: PromiseRejectionEventInit): PromiseRejectionEvent;\\n};\\n\\ninterface PushManager {\\n    getSubscription(): Promise<PushSubscription | null>;\\n    permissionState(options?: PushSubscriptionOptionsInit): Promise<PushPermissionState>;\\n    subscribe(options?: PushSubscriptionOptionsInit): Promise<PushSubscription>;\\n}\\n\\ndeclare var PushManager: {\\n    prototype: PushManager;\\n    new(): PushManager;\\n    readonly supportedContentEncodings: ReadonlyArray<string>;\\n};\\n\\ninterface PushSubscription {\\n    readonly endpoint: string;\\n    readonly expirationTime: number | null;\\n    readonly options: PushSubscriptionOptions;\\n    getKey(name: PushEncryptionKeyName): ArrayBuffer | null;\\n    toJSON(): PushSubscriptionJSON;\\n    unsubscribe(): Promise<boolean>;\\n}\\n\\ndeclare var PushSubscription: {\\n    prototype: PushSubscription;\\n    new(): PushSubscription;\\n};\\n\\ninterface PushSubscriptionOptions {\\n    readonly applicationServerKey: ArrayBuffer | null;\\n    readonly userVisibleOnly: boolean;\\n}\\n\\ndeclare var PushSubscriptionOptions: {\\n    prototype: PushSubscriptionOptions;\\n    new(): PushSubscriptionOptions;\\n};\\n\\ninterface RTCCertificate {\\n    readonly expires: number;\\n    getFingerprints(): RTCDtlsFingerprint[];\\n}\\n\\ndeclare var RTCCertificate: {\\n    prototype: RTCCertificate;\\n    new(): RTCCertificate;\\n    getSupportedAlgorithms(): AlgorithmIdentifier[];\\n};\\n\\ninterface RTCDTMFSenderEventMap {\\n    \"tonechange\": RTCDTMFToneChangeEvent;\\n}\\n\\ninterface RTCDTMFSender extends EventTarget {\\n    readonly canInsertDTMF: boolean;\\n    ontonechange: ((this: RTCDTMFSender, ev: RTCDTMFToneChangeEvent) => any) | null;\\n    readonly toneBuffer: string;\\n    insertDTMF(tones: string, duration?: number, interToneGap?: number): void;\\n    addEventListener<K extends keyof RTCDTMFSenderEventMap>(type: K, listener: (this: RTCDTMFSender, ev: RTCDTMFSenderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDTMFSenderEventMap>(type: K, listener: (this: RTCDTMFSender, ev: RTCDTMFSenderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDTMFSender: {\\n    prototype: RTCDTMFSender;\\n    new(): RTCDTMFSender;\\n};\\n\\ninterface RTCDTMFToneChangeEvent extends Event {\\n    readonly tone: string;\\n}\\n\\ndeclare var RTCDTMFToneChangeEvent: {\\n    prototype: RTCDTMFToneChangeEvent;\\n    new(type: string, eventInitDict: RTCDTMFToneChangeEventInit): RTCDTMFToneChangeEvent;\\n};\\n\\ninterface RTCDataChannelEventMap {\\n    \"bufferedamountlow\": Event;\\n    \"close\": Event;\\n    \"error\": RTCErrorEvent;\\n    \"message\": MessageEvent;\\n    \"open\": Event;\\n}\\n\\ninterface RTCDataChannel extends EventTarget {\\n    binaryType: string;\\n    readonly bufferedAmount: number;\\n    bufferedAmountLowThreshold: number;\\n    readonly id: number | null;\\n    readonly label: string;\\n    readonly maxPacketLifeTime: number | null;\\n    readonly maxRetransmits: number | null;\\n    readonly negotiated: boolean;\\n    onbufferedamountlow: ((this: RTCDataChannel, ev: Event) => any) | null;\\n    onclose: ((this: RTCDataChannel, ev: Event) => any) | null;\\n    onerror: ((this: RTCDataChannel, ev: RTCErrorEvent) => any) | null;\\n    onmessage: ((this: RTCDataChannel, ev: MessageEvent) => any) | null;\\n    onopen: ((this: RTCDataChannel, ev: Event) => any) | null;\\n    readonly ordered: boolean;\\n    readonly priority: RTCPriorityType;\\n    readonly protocol: string;\\n    readonly readyState: RTCDataChannelState;\\n    close(): void;\\n    send(data: string): void;\\n    send(data: Blob): void;\\n    send(data: ArrayBuffer): void;\\n    send(data: ArrayBufferView): void;\\n    addEventListener<K extends keyof RTCDataChannelEventMap>(type: K, listener: (this: RTCDataChannel, ev: RTCDataChannelEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDataChannelEventMap>(type: K, listener: (this: RTCDataChannel, ev: RTCDataChannelEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDataChannel: {\\n    prototype: RTCDataChannel;\\n    new(): RTCDataChannel;\\n};\\n\\ninterface RTCDataChannelEvent extends Event {\\n    readonly channel: RTCDataChannel;\\n}\\n\\ndeclare var RTCDataChannelEvent: {\\n    prototype: RTCDataChannelEvent;\\n    new(type: string, eventInitDict: RTCDataChannelEventInit): RTCDataChannelEvent;\\n};\\n\\ninterface RTCDtlsTransportEventMap {\\n    \"error\": RTCErrorEvent;\\n    \"statechange\": Event;\\n}\\n\\ninterface RTCDtlsTransport extends EventTarget {\\n    onerror: ((this: RTCDtlsTransport, ev: RTCErrorEvent) => any) | null;\\n    onstatechange: ((this: RTCDtlsTransport, ev: Event) => any) | null;\\n    readonly state: RTCDtlsTransportState;\\n    readonly transport: RTCIceTransport;\\n    getRemoteCertificates(): ArrayBuffer[];\\n    addEventListener<K extends keyof RTCDtlsTransportEventMap>(type: K, listener: (this: RTCDtlsTransport, ev: RTCDtlsTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDtlsTransportEventMap>(type: K, listener: (this: RTCDtlsTransport, ev: RTCDtlsTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDtlsTransport: {\\n    prototype: RTCDtlsTransport;\\n    new(): RTCDtlsTransport;\\n};\\n\\ninterface RTCDtlsTransportStateChangedEvent extends Event {\\n    readonly state: RTCDtlsTransportState;\\n}\\n\\ndeclare var RTCDtlsTransportStateChangedEvent: {\\n    prototype: RTCDtlsTransportStateChangedEvent;\\n    new(): RTCDtlsTransportStateChangedEvent;\\n};\\n\\ninterface RTCDtmfSenderEventMap {\\n    \"tonechange\": RTCDTMFToneChangeEvent;\\n}\\n\\ninterface RTCDtmfSender extends EventTarget {\\n    readonly canInsertDTMF: boolean;\\n    readonly duration: number;\\n    readonly interToneGap: number;\\n    ontonechange: ((this: RTCDtmfSender, ev: RTCDTMFToneChangeEvent) => any) | null;\\n    readonly sender: RTCRtpSender;\\n    readonly toneBuffer: string;\\n    insertDTMF(tones: string, duration?: number, interToneGap?: number): void;\\n    addEventListener<K extends keyof RTCDtmfSenderEventMap>(type: K, listener: (this: RTCDtmfSender, ev: RTCDtmfSenderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDtmfSenderEventMap>(type: K, listener: (this: RTCDtmfSender, ev: RTCDtmfSenderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDtmfSender: {\\n    prototype: RTCDtmfSender;\\n    new(sender: RTCRtpSender): RTCDtmfSender;\\n};\\n\\ninterface RTCError extends Error {\\n    errorDetail: string;\\n    httpRequestStatusCode: number;\\n    message: string;\\n    name: string;\\n    receivedAlert: number | null;\\n    sctpCauseCode: number;\\n    sdpLineNumber: number;\\n    sentAlert: number | null;\\n}\\n\\ndeclare var RTCError: {\\n    prototype: RTCError;\\n    new(errorDetail?: string, message?: string): RTCError;\\n};\\n\\ninterface RTCErrorEvent extends Event {\\n    readonly error: RTCError | null;\\n}\\n\\ndeclare var RTCErrorEvent: {\\n    prototype: RTCErrorEvent;\\n    new(type: string, eventInitDict: RTCErrorEventInit): RTCErrorEvent;\\n};\\n\\ninterface RTCIceCandidate {\\n    readonly candidate: string;\\n    readonly component: RTCIceComponent | null;\\n    readonly foundation: string | null;\\n    readonly ip: string | null;\\n    readonly port: number | null;\\n    readonly priority: number | null;\\n    readonly protocol: RTCIceProtocol | null;\\n    readonly relatedAddress: string | null;\\n    readonly relatedPort: number | null;\\n    readonly sdpMLineIndex: number | null;\\n    readonly sdpMid: string | null;\\n    readonly tcpType: RTCIceTcpCandidateType | null;\\n    readonly type: RTCIceCandidateType | null;\\n    readonly usernameFragment: string | null;\\n    toJSON(): RTCIceCandidateInit;\\n}\\n\\ndeclare var RTCIceCandidate: {\\n    prototype: RTCIceCandidate;\\n    new(candidateInitDict?: RTCIceCandidateInit): RTCIceCandidate;\\n};\\n\\ninterface RTCIceCandidatePairChangedEvent extends Event {\\n    readonly pair: RTCIceCandidatePair;\\n}\\n\\ndeclare var RTCIceCandidatePairChangedEvent: {\\n    prototype: RTCIceCandidatePairChangedEvent;\\n    new(): RTCIceCandidatePairChangedEvent;\\n};\\n\\ninterface RTCIceGathererEventMap {\\n    \"error\": Event;\\n    \"localcandidate\": RTCIceGathererEvent;\\n}\\n\\ninterface RTCIceGatherer extends RTCStatsProvider {\\n    readonly component: RTCIceComponent;\\n    onerror: ((this: RTCIceGatherer, ev: Event) => any) | null;\\n    onlocalcandidate: ((this: RTCIceGatherer, ev: RTCIceGathererEvent) => any) | null;\\n    createAssociatedGatherer(): RTCIceGatherer;\\n    getLocalCandidates(): RTCIceCandidateDictionary[];\\n    getLocalParameters(): RTCIceParameters;\\n    addEventListener<K extends keyof RTCIceGathererEventMap>(type: K, listener: (this: RTCIceGatherer, ev: RTCIceGathererEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCIceGathererEventMap>(type: K, listener: (this: RTCIceGatherer, ev: RTCIceGathererEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCIceGatherer: {\\n    prototype: RTCIceGatherer;\\n    new(options: RTCIceGatherOptions): RTCIceGatherer;\\n};\\n\\ninterface RTCIceGathererEvent extends Event {\\n    readonly candidate: RTCIceCandidateDictionary | RTCIceCandidateComplete;\\n}\\n\\ndeclare var RTCIceGathererEvent: {\\n    prototype: RTCIceGathererEvent;\\n    new(): RTCIceGathererEvent;\\n};\\n\\ninterface RTCIceTransportEventMap {\\n    \"gatheringstatechange\": Event;\\n    \"selectedcandidatepairchange\": Event;\\n    \"statechange\": Event;\\n}\\n\\ninterface RTCIceTransport extends EventTarget {\\n    readonly component: RTCIceComponent;\\n    readonly gatheringState: RTCIceGathererState;\\n    ongatheringstatechange: ((this: RTCIceTransport, ev: Event) => any) | null;\\n    onselectedcandidatepairchange: ((this: RTCIceTransport, ev: Event) => any) | null;\\n    onstatechange: ((this: RTCIceTransport, ev: Event) => any) | null;\\n    readonly role: RTCIceRole;\\n    readonly state: RTCIceTransportState;\\n    getLocalCandidates(): RTCIceCandidate[];\\n    getLocalParameters(): RTCIceParameters | null;\\n    getRemoteCandidates(): RTCIceCandidate[];\\n    getRemoteParameters(): RTCIceParameters | null;\\n    getSelectedCandidatePair(): RTCIceCandidatePair | null;\\n    addEventListener<K extends keyof RTCIceTransportEventMap>(type: K, listener: (this: RTCIceTransport, ev: RTCIceTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCIceTransportEventMap>(type: K, listener: (this: RTCIceTransport, ev: RTCIceTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCIceTransport: {\\n    prototype: RTCIceTransport;\\n    new(): RTCIceTransport;\\n};\\n\\ninterface RTCIceTransportStateChangedEvent extends Event {\\n    readonly state: RTCIceTransportState;\\n}\\n\\ndeclare var RTCIceTransportStateChangedEvent: {\\n    prototype: RTCIceTransportStateChangedEvent;\\n    new(): RTCIceTransportStateChangedEvent;\\n};\\n\\ninterface RTCIdentityAssertion {\\n    idp: string;\\n    name: string;\\n}\\n\\ndeclare var RTCIdentityAssertion: {\\n    prototype: RTCIdentityAssertion;\\n    new(idp: string, name: string): RTCIdentityAssertion;\\n};\\n\\ninterface RTCPeerConnectionEventMap {\\n    \"connectionstatechange\": Event;\\n    \"datachannel\": RTCDataChannelEvent;\\n    \"icecandidate\": RTCPeerConnectionIceEvent;\\n    \"icecandidateerror\": RTCPeerConnectionIceErrorEvent;\\n    \"iceconnectionstatechange\": Event;\\n    \"icegatheringstatechange\": Event;\\n    \"negotiationneeded\": Event;\\n    \"signalingstatechange\": Event;\\n    \"statsended\": RTCStatsEvent;\\n    \"track\": RTCTrackEvent;\\n}\\n\\ninterface RTCPeerConnection extends EventTarget {\\n    readonly canTrickleIceCandidates: boolean | null;\\n    readonly connectionState: RTCPeerConnectionState;\\n    readonly currentLocalDescription: RTCSessionDescription | null;\\n    readonly currentRemoteDescription: RTCSessionDescription | null;\\n    readonly iceConnectionState: RTCIceConnectionState;\\n    readonly iceGatheringState: RTCIceGatheringState;\\n    readonly idpErrorInfo: string | null;\\n    readonly idpLoginUrl: string | null;\\n    readonly localDescription: RTCSessionDescription | null;\\n    onconnectionstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    ondatachannel: ((this: RTCPeerConnection, ev: RTCDataChannelEvent) => any) | null;\\n    onicecandidate: ((this: RTCPeerConnection, ev: RTCPeerConnectionIceEvent) => any) | null;\\n    onicecandidateerror: ((this: RTCPeerConnection, ev: RTCPeerConnectionIceErrorEvent) => any) | null;\\n    oniceconnectionstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onicegatheringstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onnegotiationneeded: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onsignalingstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onstatsended: ((this: RTCPeerConnection, ev: RTCStatsEvent) => any) | null;\\n    ontrack: ((this: RTCPeerConnection, ev: RTCTrackEvent) => any) | null;\\n    readonly peerIdentity: Promise<RTCIdentityAssertion>;\\n    readonly pendingLocalDescription: RTCSessionDescription | null;\\n    readonly pendingRemoteDescription: RTCSessionDescription | null;\\n    readonly remoteDescription: RTCSessionDescription | null;\\n    readonly sctp: RTCSctpTransport | null;\\n    readonly signalingState: RTCSignalingState;\\n    addIceCandidate(candidate: RTCIceCandidateInit | RTCIceCandidate): Promise<void>;\\n    addTrack(track: MediaStreamTrack, ...streams: MediaStream[]): RTCRtpSender;\\n    addTransceiver(trackOrKind: MediaStreamTrack | string, init?: RTCRtpTransceiverInit): RTCRtpTransceiver;\\n    close(): void;\\n    createAnswer(options?: RTCOfferOptions): Promise<RTCSessionDescriptionInit>;\\n    createDataChannel(label: string, dataChannelDict?: RTCDataChannelInit): RTCDataChannel;\\n    createOffer(options?: RTCOfferOptions): Promise<RTCSessionDescriptionInit>;\\n    getConfiguration(): RTCConfiguration;\\n    getIdentityAssertion(): Promise<string>;\\n    getReceivers(): RTCRtpReceiver[];\\n    getSenders(): RTCRtpSender[];\\n    getStats(selector?: MediaStreamTrack | null): Promise<RTCStatsReport>;\\n    getTransceivers(): RTCRtpTransceiver[];\\n    removeTrack(sender: RTCRtpSender): void;\\n    setConfiguration(configuration: RTCConfiguration): void;\\n    setIdentityProvider(provider: string, options?: RTCIdentityProviderOptions): void;\\n    setLocalDescription(description: RTCSessionDescriptionInit): Promise<void>;\\n    setRemoteDescription(description: RTCSessionDescriptionInit): Promise<void>;\\n    addEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: RTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: RTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCPeerConnection: {\\n    prototype: RTCPeerConnection;\\n    new(configuration?: RTCConfiguration): RTCPeerConnection;\\n    generateCertificate(keygenAlgorithm: AlgorithmIdentifier): Promise<RTCCertificate>;\\n    getDefaultIceServers(): RTCIceServer[];\\n};\\n\\ninterface RTCPeerConnectionIceErrorEvent extends Event {\\n    readonly errorCode: number;\\n    readonly errorText: string;\\n    readonly hostCandidate: string;\\n    readonly url: string;\\n}\\n\\ndeclare var RTCPeerConnectionIceErrorEvent: {\\n    prototype: RTCPeerConnectionIceErrorEvent;\\n    new(type: string, eventInitDict: RTCPeerConnectionIceErrorEventInit): RTCPeerConnectionIceErrorEvent;\\n};\\n\\ninterface RTCPeerConnectionIceEvent extends Event {\\n    readonly candidate: RTCIceCandidate | null;\\n    readonly url: string | null;\\n}\\n\\ndeclare var RTCPeerConnectionIceEvent: {\\n    prototype: RTCPeerConnectionIceEvent;\\n    new(type: string, eventInitDict?: RTCPeerConnectionIceEventInit): RTCPeerConnectionIceEvent;\\n};\\n\\ninterface RTCRtpReceiver {\\n    readonly rtcpTransport: RTCDtlsTransport | null;\\n    readonly track: MediaStreamTrack;\\n    readonly transport: RTCDtlsTransport | null;\\n    getContributingSources(): RTCRtpContributingSource[];\\n    getParameters(): RTCRtpReceiveParameters;\\n    getStats(): Promise<RTCStatsReport>;\\n    getSynchronizationSources(): RTCRtpSynchronizationSource[];\\n}\\n\\ndeclare var RTCRtpReceiver: {\\n    prototype: RTCRtpReceiver;\\n    new(): RTCRtpReceiver;\\n    getCapabilities(kind: string): RTCRtpCapabilities | null;\\n};\\n\\ninterface RTCRtpSender {\\n    readonly dtmf: RTCDTMFSender | null;\\n    readonly rtcpTransport: RTCDtlsTransport | null;\\n    readonly track: MediaStreamTrack | null;\\n    readonly transport: RTCDtlsTransport | null;\\n    getParameters(): RTCRtpSendParameters;\\n    getStats(): Promise<RTCStatsReport>;\\n    replaceTrack(withTrack: MediaStreamTrack | null): Promise<void>;\\n    setParameters(parameters: RTCRtpSendParameters): Promise<void>;\\n    setStreams(...streams: MediaStream[]): void;\\n}\\n\\ndeclare var RTCRtpSender: {\\n    prototype: RTCRtpSender;\\n    new(): RTCRtpSender;\\n    getCapabilities(kind: string): RTCRtpCapabilities | null;\\n};\\n\\ninterface RTCRtpTransceiver {\\n    readonly currentDirection: RTCRtpTransceiverDirection | null;\\n    direction: RTCRtpTransceiverDirection;\\n    readonly mid: string | null;\\n    readonly receiver: RTCRtpReceiver;\\n    readonly sender: RTCRtpSender;\\n    readonly stopped: boolean;\\n    setCodecPreferences(codecs: RTCRtpCodecCapability[]): void;\\n    stop(): void;\\n}\\n\\ndeclare var RTCRtpTransceiver: {\\n    prototype: RTCRtpTransceiver;\\n    new(): RTCRtpTransceiver;\\n};\\n\\ninterface RTCSctpTransportEventMap {\\n    \"statechange\": Event;\\n}\\n\\ninterface RTCSctpTransport {\\n    readonly maxChannels: number | null;\\n    readonly maxMessageSize: number;\\n    onstatechange: ((this: RTCSctpTransport, ev: Event) => any) | null;\\n    readonly state: RTCSctpTransportState;\\n    readonly transport: RTCDtlsTransport;\\n    addEventListener<K extends keyof RTCSctpTransportEventMap>(type: K, listener: (this: RTCSctpTransport, ev: RTCSctpTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCSctpTransportEventMap>(type: K, listener: (this: RTCSctpTransport, ev: RTCSctpTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCSctpTransport: {\\n    prototype: RTCSctpTransport;\\n    new(): RTCSctpTransport;\\n};\\n\\ninterface RTCSessionDescription {\\n    readonly sdp: string;\\n    readonly type: RTCSdpType;\\n    toJSON(): any;\\n}\\n\\ndeclare var RTCSessionDescription: {\\n    prototype: RTCSessionDescription;\\n    new(descriptionInitDict: RTCSessionDescriptionInit): RTCSessionDescription;\\n};\\n\\ninterface RTCSrtpSdesTransportEventMap {\\n    \"error\": Event;\\n}\\n\\ninterface RTCSrtpSdesTransport extends EventTarget {\\n    onerror: ((this: RTCSrtpSdesTransport, ev: Event) => any) | null;\\n    readonly transport: RTCIceTransport;\\n    addEventListener<K extends keyof RTCSrtpSdesTransportEventMap>(type: K, listener: (this: RTCSrtpSdesTransport, ev: RTCSrtpSdesTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCSrtpSdesTransportEventMap>(type: K, listener: (this: RTCSrtpSdesTransport, ev: RTCSrtpSdesTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCSrtpSdesTransport: {\\n    prototype: RTCSrtpSdesTransport;\\n    new(transport: RTCIceTransport, encryptParameters: RTCSrtpSdesParameters, decryptParameters: RTCSrtpSdesParameters): RTCSrtpSdesTransport;\\n    getLocalParameters(): RTCSrtpSdesParameters[];\\n};\\n\\ninterface RTCSsrcConflictEvent extends Event {\\n    readonly ssrc: number;\\n}\\n\\ndeclare var RTCSsrcConflictEvent: {\\n    prototype: RTCSsrcConflictEvent;\\n    new(): RTCSsrcConflictEvent;\\n};\\n\\ninterface RTCStatsEvent extends Event {\\n    readonly report: RTCStatsReport;\\n}\\n\\ndeclare var RTCStatsEvent: {\\n    prototype: RTCStatsEvent;\\n    new(type: string, eventInitDict: RTCStatsEventInit): RTCStatsEvent;\\n};\\n\\ninterface RTCStatsProvider extends EventTarget {\\n    getStats(): Promise<RTCStatsReport>;\\n    msGetStats(): Promise<RTCStatsReport>;\\n}\\n\\ndeclare var RTCStatsProvider: {\\n    prototype: RTCStatsProvider;\\n    new(): RTCStatsProvider;\\n};\\n\\ninterface RTCStatsReport {\\n    forEach(callbackfn: (value: any, key: string, parent: RTCStatsReport) => void, thisArg?: any): void;\\n}\\n\\ndeclare var RTCStatsReport: {\\n    prototype: RTCStatsReport;\\n    new(): RTCStatsReport;\\n};\\n\\ninterface RTCTrackEvent extends Event {\\n    readonly receiver: RTCRtpReceiver;\\n    readonly streams: ReadonlyArray<MediaStream>;\\n    readonly track: MediaStreamTrack;\\n    readonly transceiver: RTCRtpTransceiver;\\n}\\n\\ndeclare var RTCTrackEvent: {\\n    prototype: RTCTrackEvent;\\n    new(type: string, eventInitDict: RTCTrackEventInit): RTCTrackEvent;\\n};\\n\\ninterface RadioNodeList extends NodeList {\\n    value: string;\\n}\\n\\ndeclare var RadioNodeList: {\\n    prototype: RadioNodeList;\\n    new(): RadioNodeList;\\n};\\n\\ninterface RandomSource {\\n    getRandomValues<T extends Int8Array | Uint8ClampedArray | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array>(array: T): T;\\n}\\n\\ndeclare var RandomSource: {\\n    prototype: RandomSource;\\n    new(): RandomSource;\\n};\\n\\ninterface Range extends AbstractRange {\\n    /**\\n     * Returns the node, furthest away from\\n     * the document, that is an ancestor of both range\\'s start node and end node.\\n     */\\n    readonly commonAncestorContainer: Node;\\n    cloneContents(): DocumentFragment;\\n    cloneRange(): Range;\\n    collapse(toStart?: boolean): void;\\n    compareBoundaryPoints(how: number, sourceRange: Range): number;\\n    /**\\n     * Returns −1 if the point is before the range, 0 if the point is\\n     * in the range, and 1 if the point is after the range.\\n     */\\n    comparePoint(node: Node, offset: number): number;\\n    createContextualFragment(fragment: string): DocumentFragment;\\n    deleteContents(): void;\\n    detach(): void;\\n    extractContents(): DocumentFragment;\\n    getBoundingClientRect(): ClientRect | DOMRect;\\n    getClientRects(): ClientRectList | DOMRectList;\\n    insertNode(node: Node): void;\\n    /**\\n     * Returns whether range intersects node.\\n     */\\n    intersectsNode(node: Node): boolean;\\n    isPointInRange(node: Node, offset: number): boolean;\\n    selectNode(node: Node): void;\\n    selectNodeContents(node: Node): void;\\n    setEnd(node: Node, offset: number): void;\\n    setEndAfter(node: Node): void;\\n    setEndBefore(node: Node): void;\\n    setStart(node: Node, offset: number): void;\\n    setStartAfter(node: Node): void;\\n    setStartBefore(node: Node): void;\\n    surroundContents(newParent: Node): void;\\n    readonly END_TO_END: number;\\n    readonly END_TO_START: number;\\n    readonly START_TO_END: number;\\n    readonly START_TO_START: number;\\n}\\n\\ndeclare var Range: {\\n    prototype: Range;\\n    new(): Range;\\n    readonly END_TO_END: number;\\n    readonly END_TO_START: number;\\n    readonly START_TO_END: number;\\n    readonly START_TO_START: number;\\n};\\n\\ninterface ReadableStream {\\n    readonly locked: boolean;\\n    cancel(): Promise<void>;\\n    getReader(): ReadableStreamReader;\\n}\\n\\ndeclare var ReadableStream: {\\n    prototype: ReadableStream;\\n    new(): ReadableStream;\\n};\\n\\ninterface ReadableStreamReader {\\n    cancel(): Promise<void>;\\n    read(): Promise<any>;\\n    releaseLock(): void;\\n}\\n\\ndeclare var ReadableStreamReader: {\\n    prototype: ReadableStreamReader;\\n    new(): ReadableStreamReader;\\n};\\n\\ninterface Request extends Body {\\n    /**\\n     * Returns the cache mode associated with request, which is a string indicating\\n     * how the the request will interact with the browser\\'s cache when fetching.\\n     */\\n    readonly cache: RequestCache;\\n    /**\\n     * Returns the credentials mode associated with request, which is a string\\n     * indicating whether credentials will be sent with the request always, never, or only when sent to a\\n     * same-origin URL.\\n     */\\n    readonly credentials: RequestCredentials;\\n    /**\\n     * Returns the kind of resource requested by request, e.g., \"document\" or\\n     * \"script\".\\n     */\\n    readonly destination: RequestDestination;\\n    /**\\n     * Returns a Headers object consisting of the headers associated with request.\\n     * Note that headers added in the network layer by the user agent will not be accounted for in this\\n     * object, e.g., the \"Host\" header.\\n     */\\n    readonly headers: Headers;\\n    /**\\n     * Returns request\\'s subresource integrity metadata, which is a cryptographic hash of\\n     * the resource being fetched. Its value consists of multiple hashes separated by whitespace. [SRI]\\n     */\\n    readonly integrity: string;\\n    /**\\n     * Returns a boolean indicating whether or not request is for a history\\n     * navigation (a.k.a. back-foward navigation).\\n     */\\n    readonly isHistoryNavigation: boolean;\\n    /**\\n     * Returns a boolean indicating whether or not request is for a reload navigation.\\n     */\\n    readonly isReloadNavigation: boolean;\\n    /**\\n     * Returns a boolean indicating whether or not request can outlive the global in which\\n     * it was created.\\n     */\\n    readonly keepalive: boolean;\\n    /**\\n     * Returns request\\'s HTTP method, which is \"GET\" by default.\\n     */\\n    readonly method: string;\\n    /**\\n     * Returns the mode associated with request, which is a string indicating\\n     * whether the request will use CORS, or will be restricted to same-origin URLs.\\n     */\\n    readonly mode: RequestMode;\\n    /**\\n     * Returns the redirect mode associated with request, which is a string\\n     * indicating how redirects for the request will be handled during fetching. A request will follow redirects by default.\\n     */\\n    readonly redirect: RequestRedirect;\\n    /**\\n     * Returns the referrer of request. Its value can be a same-origin URL if\\n     * explicitly set in init, the empty string to indicate no referrer, and\\n     * \"about:client\" when defaulting to the global\\'s default. This is used during\\n     * fetching to determine the value of the `Referer` header of the request being made.\\n     */\\n    readonly referrer: string;\\n    /**\\n     * Returns the referrer policy associated with request. This is used during\\n     * fetching to compute the value of the request\\'s referrer.\\n     */\\n    readonly referrerPolicy: ReferrerPolicy;\\n    /**\\n     * Returns the signal associated with request, which is an AbortSignal object indicating whether or not request has been aborted, and its abort\\n     * event handler.\\n     */\\n    readonly signal: AbortSignal;\\n    /**\\n     * Returns the URL of request as a string.\\n     */\\n    readonly url: string;\\n    clone(): Request;\\n}\\n\\ndeclare var Request: {\\n    prototype: Request;\\n    new(input: RequestInfo, init?: RequestInit): Request;\\n};\\n\\ninterface Response extends Body {\\n    readonly headers: Headers;\\n    readonly ok: boolean;\\n    readonly redirected: boolean;\\n    readonly status: number;\\n    readonly statusText: string;\\n    readonly trailer: Promise<Headers>;\\n    readonly type: ResponseType;\\n    readonly url: string;\\n    clone(): Response;\\n}\\n\\ndeclare var Response: {\\n    prototype: Response;\\n    new(body?: BodyInit | null, init?: ResponseInit): Response;\\n    error(): Response;\\n    redirect(url: string, status?: number): Response;\\n};\\n\\ninterface SVGAElement extends SVGGraphicsElement, SVGURIReference {\\n    readonly target: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGAElement: {\\n    prototype: SVGAElement;\\n    new(): SVGAElement;\\n};\\n\\ninterface SVGAngle {\\n    readonly unitType: number;\\n    value: number;\\n    valueAsString: string;\\n    valueInSpecifiedUnits: number;\\n    convertToSpecifiedUnits(unitType: number): void;\\n    newValueSpecifiedUnits(unitType: number, valueInSpecifiedUnits: number): void;\\n    readonly SVG_ANGLETYPE_DEG: number;\\n    readonly SVG_ANGLETYPE_GRAD: number;\\n    readonly SVG_ANGLETYPE_RAD: number;\\n    readonly SVG_ANGLETYPE_UNKNOWN: number;\\n    readonly SVG_ANGLETYPE_UNSPECIFIED: number;\\n}\\n\\ndeclare var SVGAngle: {\\n    prototype: SVGAngle;\\n    new(): SVGAngle;\\n    readonly SVG_ANGLETYPE_DEG: number;\\n    readonly SVG_ANGLETYPE_GRAD: number;\\n    readonly SVG_ANGLETYPE_RAD: number;\\n    readonly SVG_ANGLETYPE_UNKNOWN: number;\\n    readonly SVG_ANGLETYPE_UNSPECIFIED: number;\\n};\\n\\ninterface SVGAnimatedAngle {\\n    readonly animVal: SVGAngle;\\n    readonly baseVal: SVGAngle;\\n}\\n\\ndeclare var SVGAnimatedAngle: {\\n    prototype: SVGAnimatedAngle;\\n    new(): SVGAnimatedAngle;\\n};\\n\\ninterface SVGAnimatedBoolean {\\n    readonly animVal: boolean;\\n    baseVal: boolean;\\n}\\n\\ndeclare var SVGAnimatedBoolean: {\\n    prototype: SVGAnimatedBoolean;\\n    new(): SVGAnimatedBoolean;\\n};\\n\\ninterface SVGAnimatedEnumeration {\\n    readonly animVal: number;\\n    baseVal: number;\\n}\\n\\ndeclare var SVGAnimatedEnumeration: {\\n    prototype: SVGAnimatedEnumeration;\\n    new(): SVGAnimatedEnumeration;\\n};\\n\\ninterface SVGAnimatedInteger {\\n    readonly animVal: number;\\n    baseVal: number;\\n}\\n\\ndeclare var SVGAnimatedInteger: {\\n    prototype: SVGAnimatedInteger;\\n    new(): SVGAnimatedInteger;\\n};\\n\\ninterface SVGAnimatedLength {\\n    readonly animVal: SVGLength;\\n    readonly baseVal: SVGLength;\\n}\\n\\ndeclare var SVGAnimatedLength: {\\n    prototype: SVGAnimatedLength;\\n    new(): SVGAnimatedLength;\\n};\\n\\ninterface SVGAnimatedLengthList {\\n    readonly animVal: SVGLengthList;\\n    readonly baseVal: SVGLengthList;\\n}\\n\\ndeclare var SVGAnimatedLengthList: {\\n    prototype: SVGAnimatedLengthList;\\n    new(): SVGAnimatedLengthList;\\n};\\n\\ninterface SVGAnimatedNumber {\\n    readonly animVal: number;\\n    baseVal: number;\\n}\\n\\ndeclare var SVGAnimatedNumber: {\\n    prototype: SVGAnimatedNumber;\\n    new(): SVGAnimatedNumber;\\n};\\n\\ninterface SVGAnimatedNumberList {\\n    readonly animVal: SVGNumberList;\\n    readonly baseVal: SVGNumberList;\\n}\\n\\ndeclare var SVGAnimatedNumberList: {\\n    prototype: SVGAnimatedNumberList;\\n    new(): SVGAnimatedNumberList;\\n};\\n\\ninterface SVGAnimatedPoints {\\n    readonly animatedPoints: SVGPointList;\\n    readonly points: SVGPointList;\\n}\\n\\ninterface SVGAnimatedPreserveAspectRatio {\\n    readonly animVal: SVGPreserveAspectRatio;\\n    readonly baseVal: SVGPreserveAspectRatio;\\n}\\n\\ndeclare var SVGAnimatedPreserveAspectRatio: {\\n    prototype: SVGAnimatedPreserveAspectRatio;\\n    new(): SVGAnimatedPreserveAspectRatio;\\n};\\n\\ninterface SVGAnimatedRect {\\n    readonly animVal: DOMRectReadOnly;\\n    readonly baseVal: DOMRect;\\n}\\n\\ndeclare var SVGAnimatedRect: {\\n    prototype: SVGAnimatedRect;\\n    new(): SVGAnimatedRect;\\n};\\n\\ninterface SVGAnimatedString {\\n    readonly animVal: string;\\n    baseVal: string;\\n}\\n\\ndeclare var SVGAnimatedString: {\\n    prototype: SVGAnimatedString;\\n    new(): SVGAnimatedString;\\n};\\n\\ninterface SVGAnimatedTransformList {\\n    readonly animVal: SVGTransformList;\\n    readonly baseVal: SVGTransformList;\\n}\\n\\ndeclare var SVGAnimatedTransformList: {\\n    prototype: SVGAnimatedTransformList;\\n    new(): SVGAnimatedTransformList;\\n};\\n\\ninterface SVGCircleElement extends SVGGraphicsElement {\\n    readonly cx: SVGAnimatedLength;\\n    readonly cy: SVGAnimatedLength;\\n    readonly r: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGCircleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGCircleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGCircleElement: {\\n    prototype: SVGCircleElement;\\n    new(): SVGCircleElement;\\n};\\n\\ninterface SVGClipPathElement extends SVGGraphicsElement {\\n    readonly clipPathUnits: SVGAnimatedEnumeration;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGClipPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGClipPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGClipPathElement: {\\n    prototype: SVGClipPathElement;\\n    new(): SVGClipPathElement;\\n};\\n\\ninterface SVGComponentTransferFunctionElement extends SVGElement {\\n    readonly amplitude: SVGAnimatedNumber;\\n    readonly exponent: SVGAnimatedNumber;\\n    readonly intercept: SVGAnimatedNumber;\\n    readonly offset: SVGAnimatedNumber;\\n    readonly slope: SVGAnimatedNumber;\\n    readonly tableValues: SVGAnimatedNumberList;\\n    readonly type: SVGAnimatedEnumeration;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_GAMMA: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_LINEAR: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_TABLE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGComponentTransferFunctionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGComponentTransferFunctionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGComponentTransferFunctionElement: {\\n    prototype: SVGComponentTransferFunctionElement;\\n    new(): SVGComponentTransferFunctionElement;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_GAMMA: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_LINEAR: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_TABLE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGDefsElement extends SVGGraphicsElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDefsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDefsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGDefsElement: {\\n    prototype: SVGDefsElement;\\n    new(): SVGDefsElement;\\n};\\n\\ninterface SVGDescElement extends SVGElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDescElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDescElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGDescElement: {\\n    prototype: SVGDescElement;\\n    new(): SVGDescElement;\\n};\\n\\ninterface SVGElementEventMap extends ElementEventMap, GlobalEventHandlersEventMap, DocumentAndElementEventHandlersEventMap {\\n}\\n\\ninterface SVGElement extends Element, GlobalEventHandlers, DocumentAndElementEventHandlers, SVGElementInstance, HTMLOrSVGElement, ElementCSSInlineStyle {\\n    /** @deprecated */\\n    readonly className: any;\\n    readonly ownerSVGElement: SVGSVGElement | null;\\n    readonly viewportElement: SVGElement | null;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGElement: {\\n    prototype: SVGElement;\\n    new(): SVGElement;\\n};\\n\\ninterface SVGElementInstance extends EventTarget {\\n    readonly correspondingElement: SVGElement;\\n    readonly correspondingUseElement: SVGUseElement;\\n}\\n\\ndeclare var SVGElementInstance: {\\n    prototype: SVGElementInstance;\\n    new(): SVGElementInstance;\\n};\\n\\ninterface SVGElementInstanceList {\\n    /** @deprecated */\\n    readonly length: number;\\n    /** @deprecated */\\n    item(index: number): SVGElementInstance;\\n}\\n\\ndeclare var SVGElementInstanceList: {\\n    prototype: SVGElementInstanceList;\\n    new(): SVGElementInstanceList;\\n};\\n\\ninterface SVGEllipseElement extends SVGGraphicsElement {\\n    readonly cx: SVGAnimatedLength;\\n    readonly cy: SVGAnimatedLength;\\n    readonly rx: SVGAnimatedLength;\\n    readonly ry: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGEllipseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGEllipseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGEllipseElement: {\\n    prototype: SVGEllipseElement;\\n    new(): SVGEllipseElement;\\n};\\n\\ninterface SVGFEBlendElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly in2: SVGAnimatedString;\\n    readonly mode: SVGAnimatedEnumeration;\\n    readonly SVG_FEBLEND_MODE_COLOR: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_BURN: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_DODGE: number;\\n    readonly SVG_FEBLEND_MODE_DARKEN: number;\\n    readonly SVG_FEBLEND_MODE_DIFFERENCE: number;\\n    readonly SVG_FEBLEND_MODE_EXCLUSION: number;\\n    readonly SVG_FEBLEND_MODE_HARD_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_HUE: number;\\n    readonly SVG_FEBLEND_MODE_LIGHTEN: number;\\n    readonly SVG_FEBLEND_MODE_LUMINOSITY: number;\\n    readonly SVG_FEBLEND_MODE_MULTIPLY: number;\\n    readonly SVG_FEBLEND_MODE_NORMAL: number;\\n    readonly SVG_FEBLEND_MODE_OVERLAY: number;\\n    readonly SVG_FEBLEND_MODE_SATURATION: number;\\n    readonly SVG_FEBLEND_MODE_SCREEN: number;\\n    readonly SVG_FEBLEND_MODE_SOFT_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEBlendElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEBlendElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEBlendElement: {\\n    prototype: SVGFEBlendElement;\\n    new(): SVGFEBlendElement;\\n    readonly SVG_FEBLEND_MODE_COLOR: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_BURN: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_DODGE: number;\\n    readonly SVG_FEBLEND_MODE_DARKEN: number;\\n    readonly SVG_FEBLEND_MODE_DIFFERENCE: number;\\n    readonly SVG_FEBLEND_MODE_EXCLUSION: number;\\n    readonly SVG_FEBLEND_MODE_HARD_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_HUE: number;\\n    readonly SVG_FEBLEND_MODE_LIGHTEN: number;\\n    readonly SVG_FEBLEND_MODE_LUMINOSITY: number;\\n    readonly SVG_FEBLEND_MODE_MULTIPLY: number;\\n    readonly SVG_FEBLEND_MODE_NORMAL: number;\\n    readonly SVG_FEBLEND_MODE_OVERLAY: number;\\n    readonly SVG_FEBLEND_MODE_SATURATION: number;\\n    readonly SVG_FEBLEND_MODE_SCREEN: number;\\n    readonly SVG_FEBLEND_MODE_SOFT_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_UNKNOWN: number;\\n};\\n\\ninterface SVGFEColorMatrixElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly type: SVGAnimatedEnumeration;\\n    readonly values: SVGAnimatedNumberList;\\n    readonly SVG_FECOLORMATRIX_TYPE_HUEROTATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_LUMINANCETOALPHA: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_MATRIX: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_SATURATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEColorMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEColorMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEColorMatrixElement: {\\n    prototype: SVGFEColorMatrixElement;\\n    new(): SVGFEColorMatrixElement;\\n    readonly SVG_FECOLORMATRIX_TYPE_HUEROTATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_LUMINANCETOALPHA: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_MATRIX: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_SATURATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGFEComponentTransferElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEComponentTransferElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEComponentTransferElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEComponentTransferElement: {\\n    prototype: SVGFEComponentTransferElement;\\n    new(): SVGFEComponentTransferElement;\\n};\\n\\ninterface SVGFECompositeElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly in2: SVGAnimatedString;\\n    readonly k1: SVGAnimatedNumber;\\n    readonly k2: SVGAnimatedNumber;\\n    readonly k3: SVGAnimatedNumber;\\n    readonly k4: SVGAnimatedNumber;\\n    readonly operator: SVGAnimatedEnumeration;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ARITHMETIC: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ATOP: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_IN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OUT: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OVER: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_UNKNOWN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_XOR: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFECompositeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFECompositeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFECompositeElement: {\\n    prototype: SVGFECompositeElement;\\n    new(): SVGFECompositeElement;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ARITHMETIC: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ATOP: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_IN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OUT: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OVER: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_UNKNOWN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_XOR: number;\\n};\\n\\ninterface SVGFEConvolveMatrixElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly bias: SVGAnimatedNumber;\\n    readonly divisor: SVGAnimatedNumber;\\n    readonly edgeMode: SVGAnimatedEnumeration;\\n    readonly in1: SVGAnimatedString;\\n    readonly kernelMatrix: SVGAnimatedNumberList;\\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\\n    readonly orderX: SVGAnimatedInteger;\\n    readonly orderY: SVGAnimatedInteger;\\n    readonly preserveAlpha: SVGAnimatedBoolean;\\n    readonly targetX: SVGAnimatedInteger;\\n    readonly targetY: SVGAnimatedInteger;\\n    readonly SVG_EDGEMODE_DUPLICATE: number;\\n    readonly SVG_EDGEMODE_NONE: number;\\n    readonly SVG_EDGEMODE_UNKNOWN: number;\\n    readonly SVG_EDGEMODE_WRAP: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEConvolveMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEConvolveMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEConvolveMatrixElement: {\\n    prototype: SVGFEConvolveMatrixElement;\\n    new(): SVGFEConvolveMatrixElement;\\n    readonly SVG_EDGEMODE_DUPLICATE: number;\\n    readonly SVG_EDGEMODE_NONE: number;\\n    readonly SVG_EDGEMODE_UNKNOWN: number;\\n    readonly SVG_EDGEMODE_WRAP: number;\\n};\\n\\ninterface SVGFEDiffuseLightingElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly diffuseConstant: SVGAnimatedNumber;\\n    readonly in1: SVGAnimatedString;\\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\\n    readonly surfaceScale: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDiffuseLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDiffuseLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEDiffuseLightingElement: {\\n    prototype: SVGFEDiffuseLightingElement;\\n    new(): SVGFEDiffuseLightingElement;\\n};\\n\\ninterface SVGFEDisplacementMapElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly in2: SVGAnimatedString;\\n    readonly scale: SVGAnimatedNumber;\\n    readonly xChannelSelector: SVGAnimatedEnumeration;\\n    readonly yChannelSelector: SVGAnimatedEnumeration;\\n    readonly SVG_CHANNEL_A: number;\\n    readonly SVG_CHANNEL_B: number;\\n    readonly SVG_CHANNEL_G: number;\\n    readonly SVG_CHANNEL_R: number;\\n    readonly SVG_CHANNEL_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDisplacementMapElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDisplacementMapElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEDisplacementMapElement: {\\n    prototype: SVGFEDisplacementMapElement;\\n    new(): SVGFEDisplacementMapElement;\\n    readonly SVG_CHANNEL_A: number;\\n    readonly SVG_CHANNEL_B: number;\\n    readonly SVG_CHANNEL_G: number;\\n    readonly SVG_CHANNEL_R: number;\\n    readonly SVG_CHANNEL_UNKNOWN: number;\\n};\\n\\ninterface SVGFEDistantLightElement extends SVGElement {\\n    readonly azimuth: SVGAnimatedNumber;\\n    readonly elevation: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDistantLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDistantLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEDistantLightElement: {\\n    prototype: SVGFEDistantLightElement;\\n    new(): SVGFEDistantLightElement;\\n};\\n\\ninterface SVGFEFloodElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFloodElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFloodElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFloodElement: {\\n    prototype: SVGFEFloodElement;\\n    new(): SVGFEFloodElement;\\n};\\n\\ninterface SVGFEFuncAElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncAElement: {\\n    prototype: SVGFEFuncAElement;\\n    new(): SVGFEFuncAElement;\\n};\\n\\ninterface SVGFEFuncBElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncBElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncBElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncBElement: {\\n    prototype: SVGFEFuncBElement;\\n    new(): SVGFEFuncBElement;\\n};\\n\\ninterface SVGFEFuncGElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncGElement: {\\n    prototype: SVGFEFuncGElement;\\n    new(): SVGFEFuncGElement;\\n};\\n\\ninterface SVGFEFuncRElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncRElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncRElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncRElement: {\\n    prototype: SVGFEFuncRElement;\\n    new(): SVGFEFuncRElement;\\n};\\n\\ninterface SVGFEGaussianBlurElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly stdDeviationX: SVGAnimatedNumber;\\n    readonly stdDeviationY: SVGAnimatedNumber;\\n    setStdDeviation(stdDeviationX: number, stdDeviationY: number): void;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEGaussianBlurElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEGaussianBlurElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEGaussianBlurElement: {\\n    prototype: SVGFEGaussianBlurElement;\\n    new(): SVGFEGaussianBlurElement;\\n};\\n\\ninterface SVGFEImageElement extends SVGElement, SVGFilterPrimitiveStandardAttributes, SVGURIReference {\\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEImageElement: {\\n    prototype: SVGFEImageElement;\\n    new(): SVGFEImageElement;\\n};\\n\\ninterface SVGFEMergeElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEMergeElement: {\\n    prototype: SVGFEMergeElement;\\n    new(): SVGFEMergeElement;\\n};\\n\\ninterface SVGFEMergeNodeElement extends SVGElement {\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeNodeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeNodeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEMergeNodeElement: {\\n    prototype: SVGFEMergeNodeElement;\\n    new(): SVGFEMergeNodeElement;\\n};\\n\\ninterface SVGFEMorphologyElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly operator: SVGAnimatedEnumeration;\\n    readonly radiusX: SVGAnimatedNumber;\\n    readonly radiusY: SVGAnimatedNumber;\\n    readonly SVG_MORPHOLOGY_OPERATOR_DILATE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_ERODE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMorphologyElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMorphologyElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEMorphologyElement: {\\n    prototype: SVGFEMorphologyElement;\\n    new(): SVGFEMorphologyElement;\\n    readonly SVG_MORPHOLOGY_OPERATOR_DILATE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_ERODE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_UNKNOWN: number;\\n};\\n\\ninterface SVGFEOffsetElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly dx: SVGAnimatedNumber;\\n    readonly dy: SVGAnimatedNumber;\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEOffsetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEOffsetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEOffsetElement: {\\n    prototype: SVGFEOffsetElement;\\n    new(): SVGFEOffsetElement;\\n};\\n\\ninterface SVGFEPointLightElement extends SVGElement {\\n    readonly x: SVGAnimatedNumber;\\n    readonly y: SVGAnimatedNumber;\\n    readonly z: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEPointLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEPointLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEPointLightElement: {\\n    prototype: SVGFEPointLightElement;\\n    new(): SVGFEPointLightElement;\\n};\\n\\ninterface SVGFESpecularLightingElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\\n    readonly specularConstant: SVGAnimatedNumber;\\n    readonly specularExponent: SVGAnimatedNumber;\\n    readonly surfaceScale: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpecularLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpecularLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFESpecularLightingElement: {\\n    prototype: SVGFESpecularLightingElement;\\n    new(): SVGFESpecularLightingElement;\\n};\\n\\ninterface SVGFESpotLightElement extends SVGElement {\\n    readonly limitingConeAngle: SVGAnimatedNumber;\\n    readonly pointsAtX: SVGAnimatedNumber;\\n    readonly pointsAtY: SVGAnimatedNumber;\\n    readonly pointsAtZ: SVGAnimatedNumber;\\n    readonly specularExponent: SVGAnimatedNumber;\\n    readonly x: SVGAnimatedNumber;\\n    readonly y: SVGAnimatedNumber;\\n    readonly z: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpotLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpotLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFESpotLightElement: {\\n    prototype: SVGFESpotLightElement;\\n    new(): SVGFESpotLightElement;\\n};\\n\\ninterface SVGFETileElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETileElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETileElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFETileElement: {\\n    prototype: SVGFETileElement;\\n    new(): SVGFETileElement;\\n};\\n\\ninterface SVGFETurbulenceElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly baseFrequencyX: SVGAnimatedNumber;\\n    readonly baseFrequencyY: SVGAnimatedNumber;\\n    readonly numOctaves: SVGAnimatedInteger;\\n    readonly seed: SVGAnimatedNumber;\\n    readonly stitchTiles: SVGAnimatedEnumeration;\\n    readonly type: SVGAnimatedEnumeration;\\n    readonly SVG_STITCHTYPE_NOSTITCH: number;\\n    readonly SVG_STITCHTYPE_STITCH: number;\\n    readonly SVG_STITCHTYPE_UNKNOWN: number;\\n    readonly SVG_TURBULENCE_TYPE_FRACTALNOISE: number;\\n    readonly SVG_TURBULENCE_TYPE_TURBULENCE: number;\\n    readonly SVG_TURBULENCE_TYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETurbulenceElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETurbulenceElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFETurbulenceElement: {\\n    prototype: SVGFETurbulenceElement;\\n    new(): SVGFETurbulenceElement;\\n    readonly SVG_STITCHTYPE_NOSTITCH: number;\\n    readonly SVG_STITCHTYPE_STITCH: number;\\n    readonly SVG_STITCHTYPE_UNKNOWN: number;\\n    readonly SVG_TURBULENCE_TYPE_FRACTALNOISE: number;\\n    readonly SVG_TURBULENCE_TYPE_TURBULENCE: number;\\n    readonly SVG_TURBULENCE_TYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGFilterElement extends SVGElement, SVGURIReference {\\n    /** @deprecated */\\n    readonly filterResX: SVGAnimatedInteger;\\n    /** @deprecated */\\n    readonly filterResY: SVGAnimatedInteger;\\n    readonly filterUnits: SVGAnimatedEnumeration;\\n    readonly height: SVGAnimatedLength;\\n    readonly primitiveUnits: SVGAnimatedEnumeration;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    /** @deprecated */\\n    setFilterRes(filterResX: number, filterResY: number): void;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFilterElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFilterElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFilterElement: {\\n    prototype: SVGFilterElement;\\n    new(): SVGFilterElement;\\n};\\n\\ninterface SVGFilterPrimitiveStandardAttributes {\\n    readonly height: SVGAnimatedLength;\\n    readonly result: SVGAnimatedString;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n}\\n\\ninterface SVGFitToViewBox {\\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\\n    readonly viewBox: SVGAnimatedRect;\\n}\\n\\ninterface SVGForeignObjectElement extends SVGGraphicsElement {\\n    readonly height: SVGAnimatedLength;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGForeignObjectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGForeignObjectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGForeignObjectElement: {\\n    prototype: SVGForeignObjectElement;\\n    new(): SVGForeignObjectElement;\\n};\\n\\ninterface SVGGElement extends SVGGraphicsElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGElement: {\\n    prototype: SVGGElement;\\n    new(): SVGGElement;\\n};\\n\\ninterface SVGGeometryElement extends SVGGraphicsElement {\\n    readonly pathLength: SVGAnimatedNumber;\\n    getPointAtLength(distance: number): DOMPoint;\\n    getTotalLength(): number;\\n    isPointInFill(point?: DOMPointInit): boolean;\\n    isPointInStroke(point?: DOMPointInit): boolean;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGeometryElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGeometryElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGeometryElement: {\\n    prototype: SVGGeometryElement;\\n    new(): SVGGeometryElement;\\n};\\n\\ninterface SVGGradientElement extends SVGElement, SVGURIReference {\\n    readonly gradientTransform: SVGAnimatedTransformList;\\n    readonly gradientUnits: SVGAnimatedEnumeration;\\n    readonly spreadMethod: SVGAnimatedEnumeration;\\n    readonly SVG_SPREADMETHOD_PAD: number;\\n    readonly SVG_SPREADMETHOD_REFLECT: number;\\n    readonly SVG_SPREADMETHOD_REPEAT: number;\\n    readonly SVG_SPREADMETHOD_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGradientElement: {\\n    prototype: SVGGradientElement;\\n    new(): SVGGradientElement;\\n    readonly SVG_SPREADMETHOD_PAD: number;\\n    readonly SVG_SPREADMETHOD_REFLECT: number;\\n    readonly SVG_SPREADMETHOD_REPEAT: number;\\n    readonly SVG_SPREADMETHOD_UNKNOWN: number;\\n};\\n\\ninterface SVGGraphicsElement extends SVGElement, SVGTests {\\n    readonly transform: SVGAnimatedTransformList;\\n    getBBox(options?: SVGBoundingBoxOptions): DOMRect;\\n    getCTM(): DOMMatrix | null;\\n    getScreenCTM(): DOMMatrix | null;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGraphicsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGraphicsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGraphicsElement: {\\n    prototype: SVGGraphicsElement;\\n    new(): SVGGraphicsElement;\\n};\\n\\ninterface SVGImageElement extends SVGGraphicsElement, SVGURIReference {\\n    readonly height: SVGAnimatedLength;\\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGImageElement: {\\n    prototype: SVGImageElement;\\n    new(): SVGImageElement;\\n};\\n\\ninterface SVGLength {\\n    readonly unitType: number;\\n    value: number;\\n    valueAsString: string;\\n    valueInSpecifiedUnits: number;\\n    convertToSpecifiedUnits(unitType: number): void;\\n    newValueSpecifiedUnits(unitType: number, valueInSpecifiedUnits: number): void;\\n    readonly SVG_LENGTHTYPE_CM: number;\\n    readonly SVG_LENGTHTYPE_EMS: number;\\n    readonly SVG_LENGTHTYPE_EXS: number;\\n    readonly SVG_LENGTHTYPE_IN: number;\\n    readonly SVG_LENGTHTYPE_MM: number;\\n    readonly SVG_LENGTHTYPE_NUMBER: number;\\n    readonly SVG_LENGTHTYPE_PC: number;\\n    readonly SVG_LENGTHTYPE_PERCENTAGE: number;\\n    readonly SVG_LENGTHTYPE_PT: number;\\n    readonly SVG_LENGTHTYPE_PX: number;\\n    readonly SVG_LENGTHTYPE_UNKNOWN: number;\\n}\\n\\ndeclare var SVGLength: {\\n    prototype: SVGLength;\\n    new(): SVGLength;\\n    readonly SVG_LENGTHTYPE_CM: number;\\n    readonly SVG_LENGTHTYPE_EMS: number;\\n    readonly SVG_LENGTHTYPE_EXS: number;\\n    readonly SVG_LENGTHTYPE_IN: number;\\n    readonly SVG_LENGTHTYPE_MM: number;\\n    readonly SVG_LENGTHTYPE_NUMBER: number;\\n    readonly SVG_LENGTHTYPE_PC: number;\\n    readonly SVG_LENGTHTYPE_PERCENTAGE: number;\\n    readonly SVG_LENGTHTYPE_PT: number;\\n    readonly SVG_LENGTHTYPE_PX: number;\\n    readonly SVG_LENGTHTYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGLengthList {\\n    readonly length: number;\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGLength): SVGLength;\\n    clear(): void;\\n    getItem(index: number): SVGLength;\\n    initialize(newItem: SVGLength): SVGLength;\\n    insertItemBefore(newItem: SVGLength, index: number): SVGLength;\\n    removeItem(index: number): SVGLength;\\n    replaceItem(newItem: SVGLength, index: number): SVGLength;\\n    [index: number]: SVGLength;\\n}\\n\\ndeclare var SVGLengthList: {\\n    prototype: SVGLengthList;\\n    new(): SVGLengthList;\\n};\\n\\ninterface SVGLineElement extends SVGGraphicsElement {\\n    readonly x1: SVGAnimatedLength;\\n    readonly x2: SVGAnimatedLength;\\n    readonly y1: SVGAnimatedLength;\\n    readonly y2: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGLineElement: {\\n    prototype: SVGLineElement;\\n    new(): SVGLineElement;\\n};\\n\\ninterface SVGLinearGradientElement extends SVGGradientElement {\\n    readonly x1: SVGAnimatedLength;\\n    readonly x2: SVGAnimatedLength;\\n    readonly y1: SVGAnimatedLength;\\n    readonly y2: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLinearGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLinearGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGLinearGradientElement: {\\n    prototype: SVGLinearGradientElement;\\n    new(): SVGLinearGradientElement;\\n};\\n\\ninterface SVGMarkerElement extends SVGElement, SVGFitToViewBox {\\n    readonly markerHeight: SVGAnimatedLength;\\n    readonly markerUnits: SVGAnimatedEnumeration;\\n    readonly markerWidth: SVGAnimatedLength;\\n    readonly orientAngle: SVGAnimatedAngle;\\n    readonly orientType: SVGAnimatedEnumeration;\\n    readonly refX: SVGAnimatedLength;\\n    readonly refY: SVGAnimatedLength;\\n    setOrientToAngle(angle: SVGAngle): void;\\n    setOrientToAuto(): void;\\n    readonly SVG_MARKERUNITS_STROKEWIDTH: number;\\n    readonly SVG_MARKERUNITS_UNKNOWN: number;\\n    readonly SVG_MARKERUNITS_USERSPACEONUSE: number;\\n    readonly SVG_MARKER_ORIENT_ANGLE: number;\\n    readonly SVG_MARKER_ORIENT_AUTO: number;\\n    readonly SVG_MARKER_ORIENT_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMarkerElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMarkerElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGMarkerElement: {\\n    prototype: SVGMarkerElement;\\n    new(): SVGMarkerElement;\\n    readonly SVG_MARKERUNITS_STROKEWIDTH: number;\\n    readonly SVG_MARKERUNITS_UNKNOWN: number;\\n    readonly SVG_MARKERUNITS_USERSPACEONUSE: number;\\n    readonly SVG_MARKER_ORIENT_ANGLE: number;\\n    readonly SVG_MARKER_ORIENT_AUTO: number;\\n    readonly SVG_MARKER_ORIENT_UNKNOWN: number;\\n};\\n\\ninterface SVGMaskElement extends SVGElement, SVGTests {\\n    readonly height: SVGAnimatedLength;\\n    readonly maskContentUnits: SVGAnimatedEnumeration;\\n    readonly maskUnits: SVGAnimatedEnumeration;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMaskElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMaskElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGMaskElement: {\\n    prototype: SVGMaskElement;\\n    new(): SVGMaskElement;\\n};\\n\\ninterface SVGMetadataElement extends SVGElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMetadataElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMetadataElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGMetadataElement: {\\n    prototype: SVGMetadataElement;\\n    new(): SVGMetadataElement;\\n};\\n\\ninterface SVGNumber {\\n    value: number;\\n}\\n\\ndeclare var SVGNumber: {\\n    prototype: SVGNumber;\\n    new(): SVGNumber;\\n};\\n\\ninterface SVGNumberList {\\n    readonly length: number;\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGNumber): SVGNumber;\\n    clear(): void;\\n    getItem(index: number): SVGNumber;\\n    initialize(newItem: SVGNumber): SVGNumber;\\n    insertItemBefore(newItem: SVGNumber, index: number): SVGNumber;\\n    removeItem(index: number): SVGNumber;\\n    replaceItem(newItem: SVGNumber, index: number): SVGNumber;\\n    [index: number]: SVGNumber;\\n}\\n\\ndeclare var SVGNumberList: {\\n    prototype: SVGNumberList;\\n    new(): SVGNumberList;\\n};\\n\\ninterface SVGPathElement extends SVGGraphicsElement {\\n    /** @deprecated */\\n    readonly pathSegList: SVGPathSegList;\\n    /** @deprecated */\\n    createSVGPathSegArcAbs(x: number, y: number, r1: number, r2: number, angle: number, largeArcFlag: boolean, sweepFlag: boolean): SVGPathSegArcAbs;\\n    /** @deprecated */\\n    createSVGPathSegArcRel(x: number, y: number, r1: number, r2: number, angle: number, largeArcFlag: boolean, sweepFlag: boolean): SVGPathSegArcRel;\\n    /** @deprecated */\\n    createSVGPathSegClosePath(): SVGPathSegClosePath;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicAbs(x: number, y: number, x1: number, y1: number, x2: number, y2: number): SVGPathSegCurvetoCubicAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicRel(x: number, y: number, x1: number, y1: number, x2: number, y2: number): SVGPathSegCurvetoCubicRel;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicSmoothAbs(x: number, y: number, x2: number, y2: number): SVGPathSegCurvetoCubicSmoothAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicSmoothRel(x: number, y: number, x2: number, y2: number): SVGPathSegCurvetoCubicSmoothRel;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticAbs(x: number, y: number, x1: number, y1: number): SVGPathSegCurvetoQuadraticAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticRel(x: number, y: number, x1: number, y1: number): SVGPathSegCurvetoQuadraticRel;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticSmoothAbs(x: number, y: number): SVGPathSegCurvetoQuadraticSmoothAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticSmoothRel(x: number, y: number): SVGPathSegCurvetoQuadraticSmoothRel;\\n    /** @deprecated */\\n    createSVGPathSegLinetoAbs(x: number, y: number): SVGPathSegLinetoAbs;\\n    /** @deprecated */\\n    createSVGPathSegLinetoHorizontalAbs(x: number): SVGPathSegLinetoHorizontalAbs;\\n    /** @deprecated */\\n    createSVGPathSegLinetoHorizontalRel(x: number): SVGPathSegLinetoHorizontalRel;\\n    /** @deprecated */\\n    createSVGPathSegLinetoRel(x: number, y: number): SVGPathSegLinetoRel;\\n    /** @deprecated */\\n    createSVGPathSegLinetoVerticalAbs(y: number): SVGPathSegLinetoVerticalAbs;\\n    /** @deprecated */\\n    createSVGPathSegLinetoVerticalRel(y: number): SVGPathSegLinetoVerticalRel;\\n    /** @deprecated */\\n    createSVGPathSegMovetoAbs(x: number, y: number): SVGPathSegMovetoAbs;\\n    /** @deprecated */\\n    createSVGPathSegMovetoRel(x: number, y: number): SVGPathSegMovetoRel;\\n    /** @deprecated */\\n    getPathSegAtLength(distance: number): number;\\n    getPointAtLength(distance: number): SVGPoint;\\n    getTotalLength(): number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPathElement: {\\n    prototype: SVGPathElement;\\n    new(): SVGPathElement;\\n};\\n\\ninterface SVGPathSeg {\\n    readonly pathSegType: number;\\n    readonly pathSegTypeAsLetter: string;\\n    readonly PATHSEG_ARC_ABS: number;\\n    readonly PATHSEG_ARC_REL: number;\\n    readonly PATHSEG_CLOSEPATH: number;\\n    readonly PATHSEG_CURVETO_CUBIC_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_REL: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL: number;\\n    readonly PATHSEG_LINETO_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_REL: number;\\n    readonly PATHSEG_LINETO_REL: number;\\n    readonly PATHSEG_LINETO_VERTICAL_ABS: number;\\n    readonly PATHSEG_LINETO_VERTICAL_REL: number;\\n    readonly PATHSEG_MOVETO_ABS: number;\\n    readonly PATHSEG_MOVETO_REL: number;\\n    readonly PATHSEG_UNKNOWN: number;\\n}\\n\\ndeclare var SVGPathSeg: {\\n    prototype: SVGPathSeg;\\n    new(): SVGPathSeg;\\n    readonly PATHSEG_ARC_ABS: number;\\n    readonly PATHSEG_ARC_REL: number;\\n    readonly PATHSEG_CLOSEPATH: number;\\n    readonly PATHSEG_CURVETO_CUBIC_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_REL: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL: number;\\n    readonly PATHSEG_LINETO_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_REL: number;\\n    readonly PATHSEG_LINETO_REL: number;\\n    readonly PATHSEG_LINETO_VERTICAL_ABS: number;\\n    readonly PATHSEG_LINETO_VERTICAL_REL: number;\\n    readonly PATHSEG_MOVETO_ABS: number;\\n    readonly PATHSEG_MOVETO_REL: number;\\n    readonly PATHSEG_UNKNOWN: number;\\n};\\n\\ninterface SVGPathSegArcAbs extends SVGPathSeg {\\n    angle: number;\\n    largeArcFlag: boolean;\\n    r1: number;\\n    r2: number;\\n    sweepFlag: boolean;\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegArcAbs: {\\n    prototype: SVGPathSegArcAbs;\\n    new(): SVGPathSegArcAbs;\\n};\\n\\ninterface SVGPathSegArcRel extends SVGPathSeg {\\n    angle: number;\\n    largeArcFlag: boolean;\\n    r1: number;\\n    r2: number;\\n    sweepFlag: boolean;\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegArcRel: {\\n    prototype: SVGPathSegArcRel;\\n    new(): SVGPathSegArcRel;\\n};\\n\\ninterface SVGPathSegClosePath extends SVGPathSeg {\\n}\\n\\ndeclare var SVGPathSegClosePath: {\\n    prototype: SVGPathSegClosePath;\\n    new(): SVGPathSegClosePath;\\n};\\n\\ninterface SVGPathSegCurvetoCubicAbs extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    x2: number;\\n    y: number;\\n    y1: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicAbs: {\\n    prototype: SVGPathSegCurvetoCubicAbs;\\n    new(): SVGPathSegCurvetoCubicAbs;\\n};\\n\\ninterface SVGPathSegCurvetoCubicRel extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    x2: number;\\n    y: number;\\n    y1: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicRel: {\\n    prototype: SVGPathSegCurvetoCubicRel;\\n    new(): SVGPathSegCurvetoCubicRel;\\n};\\n\\ninterface SVGPathSegCurvetoCubicSmoothAbs extends SVGPathSeg {\\n    x: number;\\n    x2: number;\\n    y: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicSmoothAbs: {\\n    prototype: SVGPathSegCurvetoCubicSmoothAbs;\\n    new(): SVGPathSegCurvetoCubicSmoothAbs;\\n};\\n\\ninterface SVGPathSegCurvetoCubicSmoothRel extends SVGPathSeg {\\n    x: number;\\n    x2: number;\\n    y: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicSmoothRel: {\\n    prototype: SVGPathSegCurvetoCubicSmoothRel;\\n    new(): SVGPathSegCurvetoCubicSmoothRel;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticAbs extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    y: number;\\n    y1: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticAbs: {\\n    prototype: SVGPathSegCurvetoQuadraticAbs;\\n    new(): SVGPathSegCurvetoQuadraticAbs;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticRel extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    y: number;\\n    y1: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticRel: {\\n    prototype: SVGPathSegCurvetoQuadraticRel;\\n    new(): SVGPathSegCurvetoQuadraticRel;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticSmoothAbs extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticSmoothAbs: {\\n    prototype: SVGPathSegCurvetoQuadraticSmoothAbs;\\n    new(): SVGPathSegCurvetoQuadraticSmoothAbs;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticSmoothRel extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticSmoothRel: {\\n    prototype: SVGPathSegCurvetoQuadraticSmoothRel;\\n    new(): SVGPathSegCurvetoQuadraticSmoothRel;\\n};\\n\\ninterface SVGPathSegLinetoAbs extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoAbs: {\\n    prototype: SVGPathSegLinetoAbs;\\n    new(): SVGPathSegLinetoAbs;\\n};\\n\\ninterface SVGPathSegLinetoHorizontalAbs extends SVGPathSeg {\\n    x: number;\\n}\\n\\ndeclare var SVGPathSegLinetoHorizontalAbs: {\\n    prototype: SVGPathSegLinetoHorizontalAbs;\\n    new(): SVGPathSegLinetoHorizontalAbs;\\n};\\n\\ninterface SVGPathSegLinetoHorizontalRel extends SVGPathSeg {\\n    x: number;\\n}\\n\\ndeclare var SVGPathSegLinetoHorizontalRel: {\\n    prototype: SVGPathSegLinetoHorizontalRel;\\n    new(): SVGPathSegLinetoHorizontalRel;\\n};\\n\\ninterface SVGPathSegLinetoRel extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoRel: {\\n    prototype: SVGPathSegLinetoRel;\\n    new(): SVGPathSegLinetoRel;\\n};\\n\\ninterface SVGPathSegLinetoVerticalAbs extends SVGPathSeg {\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoVerticalAbs: {\\n    prototype: SVGPathSegLinetoVerticalAbs;\\n    new(): SVGPathSegLinetoVerticalAbs;\\n};\\n\\ninterface SVGPathSegLinetoVerticalRel extends SVGPathSeg {\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoVerticalRel: {\\n    prototype: SVGPathSegLinetoVerticalRel;\\n    new(): SVGPathSegLinetoVerticalRel;\\n};\\n\\ninterface SVGPathSegList {\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGPathSeg): SVGPathSeg;\\n    clear(): void;\\n    getItem(index: number): SVGPathSeg;\\n    initialize(newItem: SVGPathSeg): SVGPathSeg;\\n    insertItemBefore(newItem: SVGPathSeg, index: number): SVGPathSeg;\\n    removeItem(index: number): SVGPathSeg;\\n    replaceItem(newItem: SVGPathSeg, index: number): SVGPathSeg;\\n}\\n\\ndeclare var SVGPathSegList: {\\n    prototype: SVGPathSegList;\\n    new(): SVGPathSegList;\\n};\\n\\ninterface SVGPathSegMovetoAbs extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegMovetoAbs: {\\n    prototype: SVGPathSegMovetoAbs;\\n    new(): SVGPathSegMovetoAbs;\\n};\\n\\ninterface SVGPathSegMovetoRel extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegMovetoRel: {\\n    prototype: SVGPathSegMovetoRel;\\n    new(): SVGPathSegMovetoRel;\\n};\\n\\ninterface SVGPatternElement extends SVGElement, SVGTests, SVGFitToViewBox, SVGURIReference {\\n    readonly height: SVGAnimatedLength;\\n    readonly patternContentUnits: SVGAnimatedEnumeration;\\n    readonly patternTransform: SVGAnimatedTransformList;\\n    readonly patternUnits: SVGAnimatedEnumeration;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPatternElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPatternElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPatternElement: {\\n    prototype: SVGPatternElement;\\n    new(): SVGPatternElement;\\n};\\n\\ninterface SVGPointList {\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGPoint): SVGPoint;\\n    clear(): void;\\n    getItem(index: number): SVGPoint;\\n    initialize(newItem: SVGPoint): SVGPoint;\\n    insertItemBefore(newItem: SVGPoint, index: number): SVGPoint;\\n    removeItem(index: number): SVGPoint;\\n    replaceItem(newItem: SVGPoint, index: number): SVGPoint;\\n}\\n\\ndeclare var SVGPointList: {\\n    prototype: SVGPointList;\\n    new(): SVGPointList;\\n};\\n\\ninterface SVGPolygonElement extends SVGGraphicsElement, SVGAnimatedPoints {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolygonElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolygonElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPolygonElement: {\\n    prototype: SVGPolygonElement;\\n    new(): SVGPolygonElement;\\n};\\n\\ninterface SVGPolylineElement extends SVGGraphicsElement, SVGAnimatedPoints {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolylineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolylineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPolylineElement: {\\n    prototype: SVGPolylineElement;\\n    new(): SVGPolylineElement;\\n};\\n\\ninterface SVGPreserveAspectRatio {\\n    align: number;\\n    meetOrSlice: number;\\n    readonly SVG_MEETORSLICE_MEET: number;\\n    readonly SVG_MEETORSLICE_SLICE: number;\\n    readonly SVG_MEETORSLICE_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_NONE: number;\\n    readonly SVG_PRESERVEASPECTRATIO_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMIN: number;\\n}\\n\\ndeclare var SVGPreserveAspectRatio: {\\n    prototype: SVGPreserveAspectRatio;\\n    new(): SVGPreserveAspectRatio;\\n    readonly SVG_MEETORSLICE_MEET: number;\\n    readonly SVG_MEETORSLICE_SLICE: number;\\n    readonly SVG_MEETORSLICE_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_NONE: number;\\n    readonly SVG_PRESERVEASPECTRATIO_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMIN: number;\\n};\\n\\ninterface SVGRadialGradientElement extends SVGGradientElement {\\n    readonly cx: SVGAnimatedLength;\\n    readonly cy: SVGAnimatedLength;\\n    readonly fx: SVGAnimatedLength;\\n    readonly fy: SVGAnimatedLength;\\n    readonly r: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRadialGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRadialGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGRadialGradientElement: {\\n    prototype: SVGRadialGradientElement;\\n    new(): SVGRadialGradientElement;\\n};\\n\\ninterface SVGRectElement extends SVGGraphicsElement {\\n    readonly height: SVGAnimatedLength;\\n    readonly rx: SVGAnimatedLength;\\n    readonly ry: SVGAnimatedLength;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGRectElement: {\\n    prototype: SVGRectElement;\\n    new(): SVGRectElement;\\n};\\n\\ninterface SVGSVGElementEventMap extends SVGElementEventMap {\\n    \"SVGUnload\": Event;\\n    \"SVGZoom\": SVGZoomEvent;\\n}\\n\\ninterface SVGSVGElement extends SVGGraphicsElement, DocumentEvent, SVGFitToViewBox, SVGZoomAndPan {\\n    /** @deprecated */\\n    contentScriptType: string;\\n    /** @deprecated */\\n    contentStyleType: string;\\n    currentScale: number;\\n    readonly currentTranslate: SVGPoint;\\n    readonly height: SVGAnimatedLength;\\n    onunload: ((this: SVGSVGElement, ev: Event) => any) | null;\\n    onzoom: ((this: SVGSVGElement, ev: SVGZoomEvent) => any) | null;\\n    /** @deprecated */\\n    readonly pixelUnitToMillimeterX: number;\\n    /** @deprecated */\\n    readonly pixelUnitToMillimeterY: number;\\n    /** @deprecated */\\n    readonly screenPixelToMillimeterX: number;\\n    /** @deprecated */\\n    readonly screenPixelToMillimeterY: number;\\n    /** @deprecated */\\n    readonly viewport: SVGRect;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    checkEnclosure(element: SVGElement, rect: SVGRect): boolean;\\n    checkIntersection(element: SVGElement, rect: SVGRect): boolean;\\n    createSVGAngle(): SVGAngle;\\n    createSVGLength(): SVGLength;\\n    createSVGMatrix(): SVGMatrix;\\n    createSVGNumber(): SVGNumber;\\n    createSVGPoint(): SVGPoint;\\n    createSVGRect(): SVGRect;\\n    createSVGTransform(): SVGTransform;\\n    createSVGTransformFromMatrix(matrix: SVGMatrix): SVGTransform;\\n    deselectAll(): void;\\n    /** @deprecated */\\n    forceRedraw(): void;\\n    getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\\n    /** @deprecated */\\n    getCurrentTime(): number;\\n    getElementById(elementId: string): Element;\\n    getEnclosureList(rect: SVGRect, referenceElement: SVGElement): NodeListOf<SVGCircleElement | SVGEllipseElement | SVGImageElement | SVGLineElement | SVGPathElement | SVGPolygonElement | SVGPolylineElement | SVGRectElement | SVGTextElement | SVGUseElement>;\\n    getIntersectionList(rect: SVGRect, referenceElement: SVGElement): NodeListOf<SVGCircleElement | SVGEllipseElement | SVGImageElement | SVGLineElement | SVGPathElement | SVGPolygonElement | SVGPolylineElement | SVGRectElement | SVGTextElement | SVGUseElement>;\\n    /** @deprecated */\\n    pauseAnimations(): void;\\n    /** @deprecated */\\n    setCurrentTime(seconds: number): void;\\n    /** @deprecated */\\n    suspendRedraw(maxWaitMilliseconds: number): number;\\n    /** @deprecated */\\n    unpauseAnimations(): void;\\n    /** @deprecated */\\n    unsuspendRedraw(suspendHandleID: number): void;\\n    /** @deprecated */\\n    unsuspendRedrawAll(): void;\\n    addEventListener<K extends keyof SVGSVGElementEventMap>(type: K, listener: (this: SVGSVGElement, ev: SVGSVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGSVGElementEventMap>(type: K, listener: (this: SVGSVGElement, ev: SVGSVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGSVGElement: {\\n    prototype: SVGSVGElement;\\n    new(): SVGSVGElement;\\n    readonly SVG_ZOOMANDPAN_DISABLE: number;\\n    readonly SVG_ZOOMANDPAN_MAGNIFY: number;\\n    readonly SVG_ZOOMANDPAN_UNKNOWN: number;\\n};\\n\\ninterface SVGScriptElement extends SVGElement, SVGURIReference {\\n    type: string;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGScriptElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGScriptElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGScriptElement: {\\n    prototype: SVGScriptElement;\\n    new(): SVGScriptElement;\\n};\\n\\ninterface SVGStopElement extends SVGElement {\\n    readonly offset: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStopElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStopElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGStopElement: {\\n    prototype: SVGStopElement;\\n    new(): SVGStopElement;\\n};\\n\\ninterface SVGStringList {\\n    readonly length: number;\\n    readonly numberOfItems: number;\\n    appendItem(newItem: string): string;\\n    clear(): void;\\n    getItem(index: number): string;\\n    initialize(newItem: string): string;\\n    insertItemBefore(newItem: string, index: number): string;\\n    removeItem(index: number): string;\\n    replaceItem(newItem: string, index: number): string;\\n    [index: number]: string;\\n}\\n\\ndeclare var SVGStringList: {\\n    prototype: SVGStringList;\\n    new(): SVGStringList;\\n};\\n\\ninterface SVGStyleElement extends SVGElement {\\n    disabled: boolean;\\n    media: string;\\n    title: string;\\n    type: string;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStyleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStyleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGStyleElement: {\\n    prototype: SVGStyleElement;\\n    new(): SVGStyleElement;\\n};\\n\\ninterface SVGSwitchElement extends SVGGraphicsElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSwitchElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSwitchElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGSwitchElement: {\\n    prototype: SVGSwitchElement;\\n    new(): SVGSwitchElement;\\n};\\n\\ninterface SVGSymbolElement extends SVGElement, SVGFitToViewBox {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSymbolElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSymbolElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGSymbolElement: {\\n    prototype: SVGSymbolElement;\\n    new(): SVGSymbolElement;\\n};\\n\\ninterface SVGTSpanElement extends SVGTextPositioningElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTSpanElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTSpanElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTSpanElement: {\\n    prototype: SVGTSpanElement;\\n    new(): SVGTSpanElement;\\n};\\n\\ninterface SVGTests {\\n    readonly requiredExtensions: SVGStringList;\\n    readonly systemLanguage: SVGStringList;\\n}\\n\\ninterface SVGTextContentElement extends SVGGraphicsElement {\\n    readonly lengthAdjust: SVGAnimatedEnumeration;\\n    readonly textLength: SVGAnimatedLength;\\n    getCharNumAtPosition(point: SVGPoint): number;\\n    getComputedTextLength(): number;\\n    getEndPositionOfChar(charnum: number): SVGPoint;\\n    getExtentOfChar(charnum: number): SVGRect;\\n    getNumberOfChars(): number;\\n    getRotationOfChar(charnum: number): number;\\n    getStartPositionOfChar(charnum: number): SVGPoint;\\n    getSubStringLength(charnum: number, nchars: number): number;\\n    selectSubString(charnum: number, nchars: number): void;\\n    readonly LENGTHADJUST_SPACING: number;\\n    readonly LENGTHADJUST_SPACINGANDGLYPHS: number;\\n    readonly LENGTHADJUST_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextContentElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextContentElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextContentElement: {\\n    prototype: SVGTextContentElement;\\n    new(): SVGTextContentElement;\\n    readonly LENGTHADJUST_SPACING: number;\\n    readonly LENGTHADJUST_SPACINGANDGLYPHS: number;\\n    readonly LENGTHADJUST_UNKNOWN: number;\\n};\\n\\ninterface SVGTextElement extends SVGTextPositioningElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextElement: {\\n    prototype: SVGTextElement;\\n    new(): SVGTextElement;\\n};\\n\\ninterface SVGTextPathElement extends SVGTextContentElement, SVGURIReference {\\n    readonly method: SVGAnimatedEnumeration;\\n    readonly spacing: SVGAnimatedEnumeration;\\n    readonly startOffset: SVGAnimatedLength;\\n    readonly TEXTPATH_METHODTYPE_ALIGN: number;\\n    readonly TEXTPATH_METHODTYPE_STRETCH: number;\\n    readonly TEXTPATH_METHODTYPE_UNKNOWN: number;\\n    readonly TEXTPATH_SPACINGTYPE_AUTO: number;\\n    readonly TEXTPATH_SPACINGTYPE_EXACT: number;\\n    readonly TEXTPATH_SPACINGTYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextPathElement: {\\n    prototype: SVGTextPathElement;\\n    new(): SVGTextPathElement;\\n    readonly TEXTPATH_METHODTYPE_ALIGN: number;\\n    readonly TEXTPATH_METHODTYPE_STRETCH: number;\\n    readonly TEXTPATH_METHODTYPE_UNKNOWN: number;\\n    readonly TEXTPATH_SPACINGTYPE_AUTO: number;\\n    readonly TEXTPATH_SPACINGTYPE_EXACT: number;\\n    readonly TEXTPATH_SPACINGTYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGTextPositioningElement extends SVGTextContentElement {\\n    readonly dx: SVGAnimatedLengthList;\\n    readonly dy: SVGAnimatedLengthList;\\n    readonly rotate: SVGAnimatedNumberList;\\n    readonly x: SVGAnimatedLengthList;\\n    readonly y: SVGAnimatedLengthList;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPositioningElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPositioningElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextPositioningElement: {\\n    prototype: SVGTextPositioningElement;\\n    new(): SVGTextPositioningElement;\\n};\\n\\ninterface SVGTitleElement extends SVGElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTitleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTitleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTitleElement: {\\n    prototype: SVGTitleElement;\\n    new(): SVGTitleElement;\\n};\\n\\ninterface SVGTransform {\\n    readonly angle: number;\\n    readonly matrix: SVGMatrix;\\n    readonly type: number;\\n    setMatrix(matrix: SVGMatrix): void;\\n    setRotate(angle: number, cx: number, cy: number): void;\\n    setScale(sx: number, sy: number): void;\\n    setSkewX(angle: number): void;\\n    setSkewY(angle: number): void;\\n    setTranslate(tx: number, ty: number): void;\\n    readonly SVG_TRANSFORM_MATRIX: number;\\n    readonly SVG_TRANSFORM_ROTATE: number;\\n    readonly SVG_TRANSFORM_SCALE: number;\\n    readonly SVG_TRANSFORM_SKEWX: number;\\n    readonly SVG_TRANSFORM_SKEWY: number;\\n    readonly SVG_TRANSFORM_TRANSLATE: number;\\n    readonly SVG_TRANSFORM_UNKNOWN: number;\\n}\\n\\ndeclare var SVGTransform: {\\n    prototype: SVGTransform;\\n    new(): SVGTransform;\\n    readonly SVG_TRANSFORM_MATRIX: number;\\n    readonly SVG_TRANSFORM_ROTATE: number;\\n    readonly SVG_TRANSFORM_SCALE: number;\\n    readonly SVG_TRANSFORM_SKEWX: number;\\n    readonly SVG_TRANSFORM_SKEWY: number;\\n    readonly SVG_TRANSFORM_TRANSLATE: number;\\n    readonly SVG_TRANSFORM_UNKNOWN: number;\\n};\\n\\ninterface SVGTransformList {\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGTransform): SVGTransform;\\n    clear(): void;\\n    consolidate(): SVGTransform;\\n    createSVGTransformFromMatrix(matrix: SVGMatrix): SVGTransform;\\n    getItem(index: number): SVGTransform;\\n    initialize(newItem: SVGTransform): SVGTransform;\\n    insertItemBefore(newItem: SVGTransform, index: number): SVGTransform;\\n    removeItem(index: number): SVGTransform;\\n    replaceItem(newItem: SVGTransform, index: number): SVGTransform;\\n}\\n\\ndeclare var SVGTransformList: {\\n    prototype: SVGTransformList;\\n    new(): SVGTransformList;\\n};\\n\\ninterface SVGURIReference {\\n    readonly href: SVGAnimatedString;\\n}\\n\\ninterface SVGUnitTypes {\\n    readonly SVG_UNIT_TYPE_OBJECTBOUNDINGBOX: number;\\n    readonly SVG_UNIT_TYPE_UNKNOWN: number;\\n    readonly SVG_UNIT_TYPE_USERSPACEONUSE: number;\\n}\\n\\ndeclare var SVGUnitTypes: {\\n    prototype: SVGUnitTypes;\\n    new(): SVGUnitTypes;\\n    readonly SVG_UNIT_TYPE_OBJECTBOUNDINGBOX: number;\\n    readonly SVG_UNIT_TYPE_UNKNOWN: number;\\n    readonly SVG_UNIT_TYPE_USERSPACEONUSE: number;\\n};\\n\\ninterface SVGUseElement extends SVGGraphicsElement, SVGURIReference {\\n    readonly animatedInstanceRoot: SVGElementInstance | null;\\n    readonly height: SVGAnimatedLength;\\n    readonly instanceRoot: SVGElementInstance | null;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGUseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGUseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGUseElement: {\\n    prototype: SVGUseElement;\\n    new(): SVGUseElement;\\n};\\n\\ninterface SVGViewElement extends SVGElement, SVGFitToViewBox, SVGZoomAndPan {\\n    /** @deprecated */\\n    readonly viewTarget: SVGStringList;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGViewElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGViewElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGViewElement: {\\n    prototype: SVGViewElement;\\n    new(): SVGViewElement;\\n    readonly SVG_ZOOMANDPAN_DISABLE: number;\\n    readonly SVG_ZOOMANDPAN_MAGNIFY: number;\\n    readonly SVG_ZOOMANDPAN_UNKNOWN: number;\\n};\\n\\ninterface SVGZoomAndPan {\\n    readonly zoomAndPan: number;\\n}\\n\\ndeclare var SVGZoomAndPan: {\\n    readonly SVG_ZOOMANDPAN_DISABLE: number;\\n    readonly SVG_ZOOMANDPAN_MAGNIFY: number;\\n    readonly SVG_ZOOMANDPAN_UNKNOWN: number;\\n};\\n\\ninterface SVGZoomEvent extends UIEvent {\\n    readonly newScale: number;\\n    readonly newTranslate: SVGPoint;\\n    readonly previousScale: number;\\n    readonly previousTranslate: SVGPoint;\\n    readonly zoomRectScreen: SVGRect;\\n}\\n\\ndeclare var SVGZoomEvent: {\\n    prototype: SVGZoomEvent;\\n    new(): SVGZoomEvent;\\n};\\n\\ninterface ScopedCredential {\\n    readonly id: ArrayBuffer;\\n    readonly type: ScopedCredentialType;\\n}\\n\\ndeclare var ScopedCredential: {\\n    prototype: ScopedCredential;\\n    new(): ScopedCredential;\\n};\\n\\ninterface ScopedCredentialInfo {\\n    readonly credential: ScopedCredential;\\n    readonly publicKey: CryptoKey;\\n}\\n\\ndeclare var ScopedCredentialInfo: {\\n    prototype: ScopedCredentialInfo;\\n    new(): ScopedCredentialInfo;\\n};\\n\\ninterface Screen {\\n    readonly availHeight: number;\\n    readonly availWidth: number;\\n    readonly colorDepth: number;\\n    readonly height: number;\\n    readonly orientation: ScreenOrientation;\\n    readonly pixelDepth: number;\\n    readonly width: number;\\n}\\n\\ndeclare var Screen: {\\n    prototype: Screen;\\n    new(): Screen;\\n};\\n\\ninterface ScreenOrientationEventMap {\\n    \"change\": Event;\\n}\\n\\ninterface ScreenOrientation extends EventTarget {\\n    readonly angle: number;\\n    onchange: ((this: ScreenOrientation, ev: Event) => any) | null;\\n    readonly type: OrientationType;\\n    lock(orientation: OrientationLockType): Promise<void>;\\n    unlock(): void;\\n    addEventListener<K extends keyof ScreenOrientationEventMap>(type: K, listener: (this: ScreenOrientation, ev: ScreenOrientationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ScreenOrientationEventMap>(type: K, listener: (this: ScreenOrientation, ev: ScreenOrientationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ScreenOrientation: {\\n    prototype: ScreenOrientation;\\n    new(): ScreenOrientation;\\n};\\n\\ninterface ScriptProcessorNodeEventMap {\\n    \"audioprocess\": AudioProcessingEvent;\\n}\\n\\ninterface ScriptProcessorNode extends AudioNode {\\n    /** @deprecated */\\n    readonly bufferSize: number;\\n    /** @deprecated */\\n    onaudioprocess: ((this: ScriptProcessorNode, ev: AudioProcessingEvent) => any) | null;\\n    addEventListener<K extends keyof ScriptProcessorNodeEventMap>(type: K, listener: (this: ScriptProcessorNode, ev: ScriptProcessorNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ScriptProcessorNodeEventMap>(type: K, listener: (this: ScriptProcessorNode, ev: ScriptProcessorNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ScriptProcessorNode: {\\n    prototype: ScriptProcessorNode;\\n    new(): ScriptProcessorNode;\\n};\\n\\ninterface SecurityPolicyViolationEvent extends Event {\\n    readonly blockedURI: string;\\n    readonly columnNumber: number;\\n    readonly documentURI: string;\\n    readonly effectiveDirective: string;\\n    readonly lineNumber: number;\\n    readonly originalPolicy: string;\\n    readonly referrer: string;\\n    readonly sourceFile: string;\\n    readonly statusCode: number;\\n    readonly violatedDirective: string;\\n}\\n\\ndeclare var SecurityPolicyViolationEvent: {\\n    prototype: SecurityPolicyViolationEvent;\\n    new(type: string, eventInitDict?: SecurityPolicyViolationEventInit): SecurityPolicyViolationEvent;\\n};\\n\\ninterface Selection {\\n    readonly anchorNode: Node;\\n    readonly anchorOffset: number;\\n    readonly baseNode: Node;\\n    readonly baseOffset: number;\\n    readonly extentNode: Node;\\n    readonly extentOffset: number;\\n    readonly focusNode: Node;\\n    readonly focusOffset: number;\\n    readonly isCollapsed: boolean;\\n    readonly rangeCount: number;\\n    readonly type: string;\\n    addRange(range: Range): void;\\n    collapse(parentNode: Node, offset: number): void;\\n    collapseToEnd(): void;\\n    collapseToStart(): void;\\n    containsNode(node: Node, partlyContained: boolean): boolean;\\n    deleteFromDocument(): void;\\n    empty(): void;\\n    extend(newNode: Node, offset: number): void;\\n    getRangeAt(index: number): Range;\\n    removeAllRanges(): void;\\n    removeRange(range: Range): void;\\n    selectAllChildren(parentNode: Node): void;\\n    setBaseAndExtent(baseNode: Node, baseOffset: number, extentNode: Node, extentOffset: number): void;\\n    setPosition(parentNode: Node, offset: number): void;\\n    toString(): string;\\n}\\n\\ndeclare var Selection: {\\n    prototype: Selection;\\n    new(): Selection;\\n};\\n\\ninterface ServiceUIFrameContext {\\n    getCachedFrameMessage(key: string): string;\\n    postFrameMessage(key: string, data: string): void;\\n}\\ndeclare var ServiceUIFrameContext: ServiceUIFrameContext;\\n\\ninterface ServiceWorkerEventMap extends AbstractWorkerEventMap {\\n    \"statechange\": Event;\\n}\\n\\ninterface ServiceWorker extends EventTarget, AbstractWorker {\\n    onstatechange: ((this: ServiceWorker, ev: Event) => any) | null;\\n    readonly scriptURL: string;\\n    readonly state: ServiceWorkerState;\\n    postMessage(message: any, transfer?: Transferable[]): void;\\n    addEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ServiceWorker: {\\n    prototype: ServiceWorker;\\n    new(): ServiceWorker;\\n};\\n\\ninterface ServiceWorkerContainerEventMap {\\n    \"controllerchange\": Event;\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n}\\n\\ninterface ServiceWorkerContainer extends EventTarget {\\n    readonly controller: ServiceWorker | null;\\n    oncontrollerchange: ((this: ServiceWorkerContainer, ev: Event) => any) | null;\\n    onmessage: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\\n    readonly ready: Promise<ServiceWorkerRegistration>;\\n    getRegistration(clientURL?: string): Promise<ServiceWorkerRegistration | undefined>;\\n    getRegistrations(): Promise<ReadonlyArray<ServiceWorkerRegistration>>;\\n    register(scriptURL: string, options?: RegistrationOptions): Promise<ServiceWorkerRegistration>;\\n    startMessages(): void;\\n    addEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ServiceWorkerContainer: {\\n    prototype: ServiceWorkerContainer;\\n    new(): ServiceWorkerContainer;\\n};\\n\\ninterface ServiceWorkerMessageEvent extends Event {\\n    readonly data: any;\\n    readonly lastEventId: string;\\n    readonly origin: string;\\n    readonly ports: ReadonlyArray<MessagePort> | null;\\n    readonly source: ServiceWorker | MessagePort | null;\\n}\\n\\ndeclare var ServiceWorkerMessageEvent: {\\n    prototype: ServiceWorkerMessageEvent;\\n    new(type: string, eventInitDict?: ServiceWorkerMessageEventInit): ServiceWorkerMessageEvent;\\n};\\n\\ninterface ServiceWorkerRegistrationEventMap {\\n    \"updatefound\": Event;\\n}\\n\\ninterface ServiceWorkerRegistration extends EventTarget {\\n    readonly active: ServiceWorker | null;\\n    readonly installing: ServiceWorker | null;\\n    readonly navigationPreload: NavigationPreloadManager;\\n    onupdatefound: ((this: ServiceWorkerRegistration, ev: Event) => any) | null;\\n    readonly pushManager: PushManager;\\n    readonly scope: string;\\n    readonly sync: SyncManager;\\n    readonly updateViaCache: ServiceWorkerUpdateViaCache;\\n    readonly waiting: ServiceWorker | null;\\n    getNotifications(filter?: GetNotificationOptions): Promise<Notification[]>;\\n    showNotification(title: string, options?: NotificationOptions): Promise<void>;\\n    unregister(): Promise<boolean>;\\n    update(): Promise<void>;\\n    addEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ServiceWorkerRegistration: {\\n    prototype: ServiceWorkerRegistration;\\n    new(): ServiceWorkerRegistration;\\n};\\n\\ninterface ShadowRoot extends DocumentOrShadowRoot, DocumentFragment, DocumentOrShadowRoot {\\n    readonly host: Element;\\n    innerHTML: string;\\n    readonly mode: ShadowRootMode;\\n}\\n\\ninterface ShadowRootInit {\\n    delegatesFocus?: boolean;\\n    mode: \"open\" | \"closed\";\\n}\\n\\ninterface Slotable {\\n    readonly assignedSlot: HTMLSlotElement | null;\\n}\\n\\ninterface SourceBuffer extends EventTarget {\\n    appendWindowEnd: number;\\n    appendWindowStart: number;\\n    readonly audioTracks: AudioTrackList;\\n    readonly buffered: TimeRanges;\\n    mode: AppendMode;\\n    readonly textTracks: TextTrackList;\\n    timestampOffset: number;\\n    readonly updating: boolean;\\n    readonly videoTracks: VideoTrackList;\\n    abort(): void;\\n    appendBuffer(data: ArrayBuffer | Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | null): void;\\n    appendStream(stream: MSStream, maxSize?: number): void;\\n    remove(start: number, end: number): void;\\n}\\n\\ndeclare var SourceBuffer: {\\n    prototype: SourceBuffer;\\n    new(): SourceBuffer;\\n};\\n\\ninterface SourceBufferList extends EventTarget {\\n    readonly length: number;\\n    item(index: number): SourceBuffer;\\n    [index: number]: SourceBuffer;\\n}\\n\\ndeclare var SourceBufferList: {\\n    prototype: SourceBufferList;\\n    new(): SourceBufferList;\\n};\\n\\ninterface SpeechGrammar {\\n    src: string;\\n    weight: number;\\n}\\n\\ndeclare var SpeechGrammar: {\\n    prototype: SpeechGrammar;\\n    new(): SpeechGrammar;\\n};\\n\\ninterface SpeechGrammarList {\\n    readonly length: number;\\n    addFromString(string: string, weight?: number): void;\\n    addFromURI(src: string, weight?: number): void;\\n    item(index: number): SpeechGrammar;\\n    [index: number]: SpeechGrammar;\\n}\\n\\ndeclare var SpeechGrammarList: {\\n    prototype: SpeechGrammarList;\\n    new(): SpeechGrammarList;\\n};\\n\\ninterface SpeechRecognitionEventMap {\\n    \"audioend\": Event;\\n    \"audiostart\": Event;\\n    \"end\": Event;\\n    \"error\": SpeechRecognitionError;\\n    \"nomatch\": SpeechRecognitionEvent;\\n    \"result\": SpeechRecognitionEvent;\\n    \"soundend\": Event;\\n    \"soundstart\": Event;\\n    \"speechend\": Event;\\n    \"speechstart\": Event;\\n    \"start\": Event;\\n}\\n\\ninterface SpeechRecognition extends EventTarget {\\n    continuous: boolean;\\n    grammars: SpeechGrammarList;\\n    interimResults: boolean;\\n    lang: string;\\n    maxAlternatives: number;\\n    onaudioend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onaudiostart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onerror: ((this: SpeechRecognition, ev: SpeechRecognitionError) => any) | null;\\n    onnomatch: ((this: SpeechRecognition, ev: SpeechRecognitionEvent) => any) | null;\\n    onresult: ((this: SpeechRecognition, ev: SpeechRecognitionEvent) => any) | null;\\n    onsoundend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onsoundstart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onspeechend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onspeechstart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onstart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    serviceURI: string;\\n    abort(): void;\\n    start(): void;\\n    stop(): void;\\n    addEventListener<K extends keyof SpeechRecognitionEventMap>(type: K, listener: (this: SpeechRecognition, ev: SpeechRecognitionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SpeechRecognitionEventMap>(type: K, listener: (this: SpeechRecognition, ev: SpeechRecognitionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SpeechRecognition: {\\n    prototype: SpeechRecognition;\\n    new(): SpeechRecognition;\\n};\\n\\ninterface SpeechRecognitionAlternative {\\n    readonly confidence: number;\\n    readonly transcript: string;\\n}\\n\\ndeclare var SpeechRecognitionAlternative: {\\n    prototype: SpeechRecognitionAlternative;\\n    new(): SpeechRecognitionAlternative;\\n};\\n\\ninterface SpeechRecognitionError extends Event {\\n    readonly error: SpeechRecognitionErrorCode;\\n    readonly message: string;\\n}\\n\\ndeclare var SpeechRecognitionError: {\\n    prototype: SpeechRecognitionError;\\n    new(): SpeechRecognitionError;\\n};\\n\\ninterface SpeechRecognitionEvent extends Event {\\n    readonly emma: Document | null;\\n    readonly interpretation: any;\\n    readonly resultIndex: number;\\n    readonly results: SpeechRecognitionResultList;\\n}\\n\\ndeclare var SpeechRecognitionEvent: {\\n    prototype: SpeechRecognitionEvent;\\n    new(): SpeechRecognitionEvent;\\n};\\n\\ninterface SpeechRecognitionResult {\\n    readonly isFinal: boolean;\\n    readonly length: number;\\n    item(index: number): SpeechRecognitionAlternative;\\n    [index: number]: SpeechRecognitionAlternative;\\n}\\n\\ndeclare var SpeechRecognitionResult: {\\n    prototype: SpeechRecognitionResult;\\n    new(): SpeechRecognitionResult;\\n};\\n\\ninterface SpeechRecognitionResultList {\\n    readonly length: number;\\n    item(index: number): SpeechRecognitionResult;\\n    [index: number]: SpeechRecognitionResult;\\n}\\n\\ndeclare var SpeechRecognitionResultList: {\\n    prototype: SpeechRecognitionResultList;\\n    new(): SpeechRecognitionResultList;\\n};\\n\\ninterface SpeechSynthesisEventMap {\\n    \"voiceschanged\": Event;\\n}\\n\\ninterface SpeechSynthesis extends EventTarget {\\n    onvoiceschanged: ((this: SpeechSynthesis, ev: Event) => any) | null;\\n    readonly paused: boolean;\\n    readonly pending: boolean;\\n    readonly speaking: boolean;\\n    cancel(): void;\\n    getVoices(): SpeechSynthesisVoice[];\\n    pause(): void;\\n    resume(): void;\\n    speak(utterance: SpeechSynthesisUtterance): void;\\n    addEventListener<K extends keyof SpeechSynthesisEventMap>(type: K, listener: (this: SpeechSynthesis, ev: SpeechSynthesisEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SpeechSynthesisEventMap>(type: K, listener: (this: SpeechSynthesis, ev: SpeechSynthesisEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SpeechSynthesis: {\\n    prototype: SpeechSynthesis;\\n    new(): SpeechSynthesis;\\n};\\n\\ninterface SpeechSynthesisErrorEvent extends SpeechSynthesisEvent {\\n    readonly error: SpeechSynthesisErrorCode;\\n}\\n\\ndeclare var SpeechSynthesisErrorEvent: {\\n    prototype: SpeechSynthesisErrorEvent;\\n    new(): SpeechSynthesisErrorEvent;\\n};\\n\\ninterface SpeechSynthesisEvent extends Event {\\n    readonly charIndex: number;\\n    readonly elapsedTime: number;\\n    readonly name: string;\\n    readonly utterance: SpeechSynthesisUtterance;\\n}\\n\\ndeclare var SpeechSynthesisEvent: {\\n    prototype: SpeechSynthesisEvent;\\n    new(): SpeechSynthesisEvent;\\n};\\n\\ninterface SpeechSynthesisUtteranceEventMap {\\n    \"boundary\": SpeechSynthesisEvent;\\n    \"end\": SpeechSynthesisEvent;\\n    \"error\": SpeechSynthesisErrorEvent;\\n    \"mark\": SpeechSynthesisEvent;\\n    \"pause\": SpeechSynthesisEvent;\\n    \"resume\": SpeechSynthesisEvent;\\n    \"start\": SpeechSynthesisEvent;\\n}\\n\\ninterface SpeechSynthesisUtterance extends EventTarget {\\n    lang: string;\\n    onboundary: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onend: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onerror: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisErrorEvent) => any) | null;\\n    onmark: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onpause: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onresume: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onstart: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    pitch: number;\\n    rate: number;\\n    text: string;\\n    voice: SpeechSynthesisVoice;\\n    volume: number;\\n    addEventListener<K extends keyof SpeechSynthesisUtteranceEventMap>(type: K, listener: (this: SpeechSynthesisUtterance, ev: SpeechSynthesisUtteranceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SpeechSynthesisUtteranceEventMap>(type: K, listener: (this: SpeechSynthesisUtterance, ev: SpeechSynthesisUtteranceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SpeechSynthesisUtterance: {\\n    prototype: SpeechSynthesisUtterance;\\n    new(): SpeechSynthesisUtterance;\\n    new(text: string): SpeechSynthesisUtterance;\\n};\\n\\ninterface SpeechSynthesisVoice {\\n    readonly default: boolean;\\n    readonly lang: string;\\n    readonly localService: boolean;\\n    readonly name: string;\\n    readonly voiceURI: string;\\n}\\n\\ndeclare var SpeechSynthesisVoice: {\\n    prototype: SpeechSynthesisVoice;\\n    new(): SpeechSynthesisVoice;\\n};\\n\\ninterface StaticRange extends AbstractRange {\\n}\\n\\ndeclare var StaticRange: {\\n    prototype: StaticRange;\\n    new(): StaticRange;\\n};\\n\\ninterface StereoPannerNode extends AudioNode {\\n    readonly pan: AudioParam;\\n}\\n\\ndeclare var StereoPannerNode: {\\n    prototype: StereoPannerNode;\\n    new(context: BaseAudioContext, options?: StereoPannerOptions): StereoPannerNode;\\n};\\n\\ninterface Storage {\\n    /**\\n     * Returns the number of key/value pairs currently present in the list associated with the\\n     * object.\\n     */\\n    readonly length: number;\\n    /**\\n     * Empties the list associated with the object of all key/value pairs, if there are any.\\n     */\\n    clear(): void;\\n    /**\\n     * value = storage[key]\\n     */\\n    getItem(key: string): string | null;\\n    /**\\n     * Returns the name of the nth key in the list, or null if n is greater\\n     * than or equal to the number of key/value pairs in the object.\\n     */\\n    key(index: number): string | null;\\n    /**\\n     * delete storage[key]\\n     */\\n    removeItem(key: string): void;\\n    /**\\n     * storage[key] = value\\n     */\\n    setItem(key: string, value: string): void;\\n    [name: string]: any;\\n}\\n\\ndeclare var Storage: {\\n    prototype: Storage;\\n    new(): Storage;\\n};\\n\\ninterface StorageEvent extends Event {\\n    /**\\n     * Returns the key of the storage item being changed.\\n     */\\n    readonly key: string | null;\\n    /**\\n     * Returns the new value of the key of the storage item whose value is being changed.\\n     */\\n    readonly newValue: string | null;\\n    /**\\n     * Returns the old value of the key of the storage item whose value is being changed.\\n     */\\n    readonly oldValue: string | null;\\n    /**\\n     * Returns the Storage object that was affected.\\n     */\\n    readonly storageArea: Storage | null;\\n    /**\\n     * Returns the URL of the document whose storage item changed.\\n     */\\n    readonly url: string;\\n}\\n\\ndeclare var StorageEvent: {\\n    prototype: StorageEvent;\\n    new(type: string, eventInitDict?: StorageEventInit): StorageEvent;\\n};\\n\\ninterface StorageManager {\\n    estimate(): Promise<StorageEstimate>;\\n    persist(): Promise<boolean>;\\n    persisted(): Promise<boolean>;\\n}\\n\\ndeclare var StorageManager: {\\n    prototype: StorageManager;\\n    new(): StorageManager;\\n};\\n\\ninterface StyleMedia {\\n    readonly type: string;\\n    matchMedium(mediaquery: string): boolean;\\n}\\n\\ndeclare var StyleMedia: {\\n    prototype: StyleMedia;\\n    new(): StyleMedia;\\n};\\n\\ninterface StyleSheet {\\n    disabled: boolean;\\n    readonly href: string | null;\\n    readonly media: MediaList;\\n    readonly ownerNode: Node;\\n    readonly parentStyleSheet: StyleSheet | null;\\n    readonly title: string | null;\\n    readonly type: string;\\n}\\n\\ndeclare var StyleSheet: {\\n    prototype: StyleSheet;\\n    new(): StyleSheet;\\n};\\n\\ninterface StyleSheetList {\\n    readonly length: number;\\n    item(index: number): StyleSheet | null;\\n    [index: number]: StyleSheet;\\n}\\n\\ndeclare var StyleSheetList: {\\n    prototype: StyleSheetList;\\n    new(): StyleSheetList;\\n};\\n\\ninterface SubtleCrypto {\\n    decrypt(algorithm: string | RsaOaepParams | AesCtrParams | AesCbcParams | AesCmacParams | AesGcmParams | AesCfbParams, key: CryptoKey, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    deriveBits(algorithm: string | EcdhKeyDeriveParams | DhKeyDeriveParams | ConcatParams | HkdfCtrParams | Pbkdf2Params, baseKey: CryptoKey, length: number): PromiseLike<ArrayBuffer>;\\n    deriveKey(algorithm: string | EcdhKeyDeriveParams | DhKeyDeriveParams | ConcatParams | HkdfCtrParams | Pbkdf2Params, baseKey: CryptoKey, derivedKeyType: string | AesDerivedKeyParams | HmacImportParams | ConcatParams | HkdfCtrParams | Pbkdf2Params, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    digest(algorithm: string | Algorithm, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    encrypt(algorithm: string | RsaOaepParams | AesCtrParams | AesCbcParams | AesCmacParams | AesGcmParams | AesCfbParams, key: CryptoKey, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    exportKey(format: \"jwk\", key: CryptoKey): PromiseLike<JsonWebKey>;\\n    exportKey(format: \"raw\" | \"pkcs8\" | \"spki\", key: CryptoKey): PromiseLike<ArrayBuffer>;\\n    exportKey(format: string, key: CryptoKey): PromiseLike<JsonWebKey | ArrayBuffer>;\\n    generateKey(algorithm: string, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKeyPair | CryptoKey>;\\n    generateKey(algorithm: RsaHashedKeyGenParams | EcKeyGenParams | DhKeyGenParams, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKeyPair>;\\n    generateKey(algorithm: AesKeyGenParams | HmacKeyGenParams | Pbkdf2Params, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    importKey(format: \"jwk\", keyData: JsonWebKey, algorithm: string | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | DhImportKeyParams | AesKeyAlgorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    importKey(format: \"raw\" | \"pkcs8\" | \"spki\", keyData: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, algorithm: string | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | DhImportKeyParams | AesKeyAlgorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    importKey(format: string, keyData: JsonWebKey | Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, algorithm: string | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | DhImportKeyParams | AesKeyAlgorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    sign(algorithm: string | RsaPssParams | EcdsaParams | AesCmacParams, key: CryptoKey, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    unwrapKey(format: string, wrappedKey: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, unwrappingKey: CryptoKey, unwrapAlgorithm: string | Algorithm, unwrappedKeyAlgorithm: string | Algorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    verify(algorithm: string | RsaPssParams | EcdsaParams | AesCmacParams, key: CryptoKey, signature: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<boolean>;\\n    wrapKey(format: string, key: CryptoKey, wrappingKey: CryptoKey, wrapAlgorithm: string | Algorithm): PromiseLike<ArrayBuffer>;\\n}\\n\\ndeclare var SubtleCrypto: {\\n    prototype: SubtleCrypto;\\n    new(): SubtleCrypto;\\n};\\n\\ninterface SyncManager {\\n    getTags(): Promise<string[]>;\\n    register(tag: string): Promise<void>;\\n}\\n\\ndeclare var SyncManager: {\\n    prototype: SyncManager;\\n    new(): SyncManager;\\n};\\n\\ninterface Text extends CharacterData, Slotable {\\n    readonly assignedSlot: HTMLSlotElement | null;\\n    /**\\n     * Returns the combined data of all direct Text node siblings.\\n     */\\n    readonly wholeText: string;\\n    /**\\n     * Splits data at the given offset and returns the remainder as Text node.\\n     */\\n    splitText(offset: number): Text;\\n}\\n\\ndeclare var Text: {\\n    prototype: Text;\\n    new(data?: string): Text;\\n};\\n\\ninterface TextDecoder {\\n    /**\\n     * Returns encoding\\'s name, lowercased.\\n     */\\n    readonly encoding: string;\\n    /**\\n     * Returns true if error mode is \"fatal\", and false\\n     * otherwise.\\n     */\\n    readonly fatal: boolean;\\n    /**\\n     * Returns true if ignore BOM flag is set, and false otherwise.\\n     */\\n    readonly ignoreBOM: boolean;\\n    /**\\n     * Returns the result of running encoding\\'s decoder. The\\n     * method can be invoked zero or more times with options\\'s stream set to\\n     * true, and then once without options\\'s stream (or set to false), to process\\n     * a fragmented stream. If the invocation without options\\'s stream (or set to\\n     * false) has no input, it\\'s clearest to omit both arguments.\\n     * var string = \"\", decoder = new TextDecoder(encoding), buffer;\\n     * while(buffer = next_chunk()) {\\n     * string += decoder.decode(buffer, {stream:true});\\n     * }\\n     * string += decoder.decode(); // end-of-stream\\n     * If the error mode is \"fatal\" and encoding\\'s decoder returns error, throws a TypeError.\\n     */\\n    decode(input?: BufferSource, options?: TextDecodeOptions): string;\\n}\\n\\ndeclare var TextDecoder: {\\n    prototype: TextDecoder;\\n    new(label?: string, options?: TextDecoderOptions): TextDecoder;\\n};\\n\\ninterface TextEncoder {\\n    /**\\n     * Returns \"utf-8\".\\n     */\\n    readonly encoding: string;\\n    /**\\n     * Returns the result of running UTF-8\\'s encoder.\\n     */\\n    encode(input?: string): Uint8Array;\\n}\\n\\ndeclare var TextEncoder: {\\n    prototype: TextEncoder;\\n    new(): TextEncoder;\\n};\\n\\ninterface TextEvent extends UIEvent {\\n    readonly data: string;\\n    initTextEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, dataArg: string, inputMethod: number, locale: string): void;\\n    readonly DOM_INPUT_METHOD_DROP: number;\\n    readonly DOM_INPUT_METHOD_HANDWRITING: number;\\n    readonly DOM_INPUT_METHOD_IME: number;\\n    readonly DOM_INPUT_METHOD_KEYBOARD: number;\\n    readonly DOM_INPUT_METHOD_MULTIMODAL: number;\\n    readonly DOM_INPUT_METHOD_OPTION: number;\\n    readonly DOM_INPUT_METHOD_PASTE: number;\\n    readonly DOM_INPUT_METHOD_SCRIPT: number;\\n    readonly DOM_INPUT_METHOD_UNKNOWN: number;\\n    readonly DOM_INPUT_METHOD_VOICE: number;\\n}\\n\\ndeclare var TextEvent: {\\n    prototype: TextEvent;\\n    new(): TextEvent;\\n    readonly DOM_INPUT_METHOD_DROP: number;\\n    readonly DOM_INPUT_METHOD_HANDWRITING: number;\\n    readonly DOM_INPUT_METHOD_IME: number;\\n    readonly DOM_INPUT_METHOD_KEYBOARD: number;\\n    readonly DOM_INPUT_METHOD_MULTIMODAL: number;\\n    readonly DOM_INPUT_METHOD_OPTION: number;\\n    readonly DOM_INPUT_METHOD_PASTE: number;\\n    readonly DOM_INPUT_METHOD_SCRIPT: number;\\n    readonly DOM_INPUT_METHOD_UNKNOWN: number;\\n    readonly DOM_INPUT_METHOD_VOICE: number;\\n};\\n\\ninterface TextMetrics {\\n    readonly actualBoundingBoxAscent: number;\\n    readonly actualBoundingBoxDescent: number;\\n    readonly actualBoundingBoxLeft: number;\\n    readonly actualBoundingBoxRight: number;\\n    readonly alphabeticBaseline: number;\\n    readonly emHeightAscent: number;\\n    readonly emHeightDescent: number;\\n    readonly fontBoundingBoxAscent: number;\\n    readonly fontBoundingBoxDescent: number;\\n    readonly hangingBaseline: number;\\n    /**\\n     * Returns the measurement described below.\\n     */\\n    readonly ideographicBaseline: number;\\n    readonly width: number;\\n}\\n\\ndeclare var TextMetrics: {\\n    prototype: TextMetrics;\\n    new(): TextMetrics;\\n};\\n\\ninterface TextTrackEventMap {\\n    \"cuechange\": Event;\\n    \"error\": Event;\\n    \"load\": Event;\\n}\\n\\ninterface TextTrack extends EventTarget {\\n    readonly activeCues: TextTrackCueList;\\n    readonly cues: TextTrackCueList;\\n    readonly inBandMetadataTrackDispatchType: string;\\n    readonly kind: string;\\n    readonly label: string;\\n    readonly language: string;\\n    mode: TextTrackMode | number;\\n    oncuechange: ((this: TextTrack, ev: Event) => any) | null;\\n    onerror: ((this: TextTrack, ev: Event) => any) | null;\\n    onload: ((this: TextTrack, ev: Event) => any) | null;\\n    readonly readyState: number;\\n    addCue(cue: TextTrackCue): void;\\n    removeCue(cue: TextTrackCue): void;\\n    readonly DISABLED: number;\\n    readonly ERROR: number;\\n    readonly HIDDEN: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n    readonly SHOWING: number;\\n    addEventListener<K extends keyof TextTrackEventMap>(type: K, listener: (this: TextTrack, ev: TextTrackEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackEventMap>(type: K, listener: (this: TextTrack, ev: TextTrackEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var TextTrack: {\\n    prototype: TextTrack;\\n    new(): TextTrack;\\n    readonly DISABLED: number;\\n    readonly ERROR: number;\\n    readonly HIDDEN: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n    readonly SHOWING: number;\\n};\\n\\ninterface TextTrackCueEventMap {\\n    \"enter\": Event;\\n    \"exit\": Event;\\n}\\n\\ninterface TextTrackCue extends EventTarget {\\n    endTime: number;\\n    id: string;\\n    onenter: ((this: TextTrackCue, ev: Event) => any) | null;\\n    onexit: ((this: TextTrackCue, ev: Event) => any) | null;\\n    pauseOnExit: boolean;\\n    startTime: number;\\n    text: string;\\n    readonly track: TextTrack;\\n    getCueAsHTML(): DocumentFragment;\\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: TextTrackCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: TextTrackCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var TextTrackCue: {\\n    prototype: TextTrackCue;\\n    new(startTime: number, endTime: number, text: string): TextTrackCue;\\n};\\n\\ninterface TextTrackCueList {\\n    readonly length: number;\\n    getCueById(id: string): TextTrackCue;\\n    item(index: number): TextTrackCue;\\n    [index: number]: TextTrackCue;\\n}\\n\\ndeclare var TextTrackCueList: {\\n    prototype: TextTrackCueList;\\n    new(): TextTrackCueList;\\n};\\n\\ninterface TextTrackListEventMap {\\n    \"addtrack\": TrackEvent;\\n}\\n\\ninterface TextTrackList extends EventTarget {\\n    readonly length: number;\\n    onaddtrack: ((this: TextTrackList, ev: TrackEvent) => any) | null;\\n    item(index: number): TextTrack;\\n    addEventListener<K extends keyof TextTrackListEventMap>(type: K, listener: (this: TextTrackList, ev: TextTrackListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackListEventMap>(type: K, listener: (this: TextTrackList, ev: TextTrackListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: TextTrack;\\n}\\n\\ndeclare var TextTrackList: {\\n    prototype: TextTrackList;\\n    new(): TextTrackList;\\n};\\n\\ninterface TimeRanges {\\n    readonly length: number;\\n    end(index: number): number;\\n    start(index: number): number;\\n}\\n\\ndeclare var TimeRanges: {\\n    prototype: TimeRanges;\\n    new(): TimeRanges;\\n};\\n\\ninterface Touch {\\n    readonly altitudeAngle: number;\\n    readonly azimuthAngle: number;\\n    readonly clientX: number;\\n    readonly clientY: number;\\n    readonly force: number;\\n    readonly identifier: number;\\n    readonly pageX: number;\\n    readonly pageY: number;\\n    readonly radiusX: number;\\n    readonly radiusY: number;\\n    readonly rotationAngle: number;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly target: EventTarget;\\n    readonly touchType: TouchType;\\n}\\n\\ndeclare var Touch: {\\n    prototype: Touch;\\n    new(touchInitDict: TouchInit): Touch;\\n};\\n\\ninterface TouchEvent extends UIEvent {\\n    readonly altKey: boolean;\\n    readonly changedTouches: TouchList;\\n    readonly ctrlKey: boolean;\\n    readonly metaKey: boolean;\\n    readonly shiftKey: boolean;\\n    readonly targetTouches: TouchList;\\n    readonly touches: TouchList;\\n}\\n\\ndeclare var TouchEvent: {\\n    prototype: TouchEvent;\\n    new(type: string, eventInitDict?: TouchEventInit): TouchEvent;\\n};\\n\\ninterface TouchList {\\n    readonly length: number;\\n    item(index: number): Touch | null;\\n    [index: number]: Touch;\\n}\\n\\ndeclare var TouchList: {\\n    prototype: TouchList;\\n    new(): TouchList;\\n};\\n\\ninterface TrackEvent extends Event {\\n    readonly track: VideoTrack | AudioTrack | TextTrack | null;\\n}\\n\\ndeclare var TrackEvent: {\\n    prototype: TrackEvent;\\n    new(typeArg: string, eventInitDict?: TrackEventInit): TrackEvent;\\n};\\n\\ninterface TransitionEvent extends Event {\\n    readonly elapsedTime: number;\\n    readonly propertyName: string;\\n    readonly pseudoElement: string;\\n}\\n\\ndeclare var TransitionEvent: {\\n    prototype: TransitionEvent;\\n    new(type: string, transitionEventInitDict?: TransitionEventInit): TransitionEvent;\\n};\\n\\ninterface TreeWalker {\\n    currentNode: Node;\\n    readonly filter: NodeFilter | null;\\n    readonly root: Node;\\n    readonly whatToShow: number;\\n    firstChild(): Node | null;\\n    lastChild(): Node | null;\\n    nextNode(): Node | null;\\n    nextSibling(): Node | null;\\n    parentNode(): Node | null;\\n    previousNode(): Node | null;\\n    previousSibling(): Node | null;\\n}\\n\\ndeclare var TreeWalker: {\\n    prototype: TreeWalker;\\n    new(): TreeWalker;\\n};\\n\\ninterface UIEvent extends Event {\\n    readonly detail: number;\\n    readonly view: Window;\\n    initUIEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number): void;\\n}\\n\\ndeclare var UIEvent: {\\n    prototype: UIEvent;\\n    new(typeArg: string, eventInitDict?: UIEventInit): UIEvent;\\n};\\n\\ninterface URL {\\n    hash: string;\\n    host: string;\\n    hostname: string;\\n    href: string;\\n    readonly origin: string;\\n    password: string;\\n    pathname: string;\\n    port: string;\\n    protocol: string;\\n    search: string;\\n    readonly searchParams: URLSearchParams;\\n    username: string;\\n    toJSON(): string;\\n}\\n\\ndeclare var URL: {\\n    prototype: URL;\\n    new(url: string, base?: string | URL): URL;\\n    createObjectURL(object: any): string;\\n    revokeObjectURL(url: string): void;\\n};\\n\\ntype webkitURL = URL;\\ndeclare var webkitURL: typeof URL;\\n\\ninterface URLSearchParams {\\n    /**\\n     * Appends a specified key/value pair as a new search parameter.\\n     */\\n    append(name: string, value: string): void;\\n    /**\\n     * Deletes the given search parameter, and its associated value, from the list of all search parameters.\\n     */\\n    delete(name: string): void;\\n    /**\\n     * Returns the first value associated to the given search parameter.\\n     */\\n    get(name: string): string | null;\\n    /**\\n     * Returns all the values association with a given search parameter.\\n     */\\n    getAll(name: string): string[];\\n    /**\\n     * Returns a Boolean indicating if such a search parameter exists.\\n     */\\n    has(name: string): boolean;\\n    /**\\n     * Sets the value associated to a given search parameter to the given value. If there were several values, delete the others.\\n     */\\n    set(name: string, value: string): void;\\n    sort(): void;\\n    forEach(callbackfn: (value: string, key: string, parent: URLSearchParams) => void, thisArg?: any): void;\\n}\\n\\ndeclare var URLSearchParams: {\\n    prototype: URLSearchParams;\\n    new(init?: string[][] | Record<string, string> | string | URLSearchParams): URLSearchParams;\\n};\\n\\ninterface VRDisplay extends EventTarget {\\n    readonly capabilities: VRDisplayCapabilities;\\n    depthFar: number;\\n    depthNear: number;\\n    readonly displayId: number;\\n    readonly displayName: string;\\n    readonly isConnected: boolean;\\n    readonly isPresenting: boolean;\\n    readonly stageParameters: VRStageParameters | null;\\n    cancelAnimationFrame(handle: number): void;\\n    exitPresent(): Promise<void>;\\n    getEyeParameters(whichEye: string): VREyeParameters;\\n    getFrameData(frameData: VRFrameData): boolean;\\n    getLayers(): VRLayer[];\\n    /** @deprecated */\\n    getPose(): VRPose;\\n    requestAnimationFrame(callback: FrameRequestCallback): number;\\n    requestPresent(layers: VRLayer[]): Promise<void>;\\n    resetPose(): void;\\n    submitFrame(pose?: VRPose): void;\\n}\\n\\ndeclare var VRDisplay: {\\n    prototype: VRDisplay;\\n    new(): VRDisplay;\\n};\\n\\ninterface VRDisplayCapabilities {\\n    readonly canPresent: boolean;\\n    readonly hasExternalDisplay: boolean;\\n    readonly hasOrientation: boolean;\\n    readonly hasPosition: boolean;\\n    readonly maxLayers: number;\\n}\\n\\ndeclare var VRDisplayCapabilities: {\\n    prototype: VRDisplayCapabilities;\\n    new(): VRDisplayCapabilities;\\n};\\n\\ninterface VRDisplayEvent extends Event {\\n    readonly display: VRDisplay;\\n    readonly reason: VRDisplayEventReason | null;\\n}\\n\\ndeclare var VRDisplayEvent: {\\n    prototype: VRDisplayEvent;\\n    new(type: string, eventInitDict: VRDisplayEventInit): VRDisplayEvent;\\n};\\n\\ninterface VREyeParameters {\\n    /** @deprecated */\\n    readonly fieldOfView: VRFieldOfView;\\n    readonly offset: Float32Array;\\n    readonly renderHeight: number;\\n    readonly renderWidth: number;\\n}\\n\\ndeclare var VREyeParameters: {\\n    prototype: VREyeParameters;\\n    new(): VREyeParameters;\\n};\\n\\ninterface VRFieldOfView {\\n    readonly downDegrees: number;\\n    readonly leftDegrees: number;\\n    readonly rightDegrees: number;\\n    readonly upDegrees: number;\\n}\\n\\ndeclare var VRFieldOfView: {\\n    prototype: VRFieldOfView;\\n    new(): VRFieldOfView;\\n};\\n\\ninterface VRFrameData {\\n    readonly leftProjectionMatrix: Float32Array;\\n    readonly leftViewMatrix: Float32Array;\\n    readonly pose: VRPose;\\n    readonly rightProjectionMatrix: Float32Array;\\n    readonly rightViewMatrix: Float32Array;\\n    readonly timestamp: number;\\n}\\n\\ndeclare var VRFrameData: {\\n    prototype: VRFrameData;\\n    new(): VRFrameData;\\n};\\n\\ninterface VRPose {\\n    readonly angularAcceleration: Float32Array | null;\\n    readonly angularVelocity: Float32Array | null;\\n    readonly linearAcceleration: Float32Array | null;\\n    readonly linearVelocity: Float32Array | null;\\n    readonly orientation: Float32Array | null;\\n    readonly position: Float32Array | null;\\n    readonly timestamp: number;\\n}\\n\\ndeclare var VRPose: {\\n    prototype: VRPose;\\n    new(): VRPose;\\n};\\n\\ninterface VTTCue extends TextTrackCue {\\n    align: AlignSetting;\\n    line: LineAndPositionSetting;\\n    lineAlign: LineAlignSetting;\\n    position: LineAndPositionSetting;\\n    positionAlign: PositionAlignSetting;\\n    region: VTTRegion | null;\\n    size: number;\\n    snapToLines: boolean;\\n    text: string;\\n    vertical: DirectionSetting;\\n    getCueAsHTML(): DocumentFragment;\\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: VTTCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: VTTCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var VTTCue: {\\n    prototype: VTTCue;\\n    new(startTime: number, endTime: number, text: string): VTTCue;\\n};\\n\\ninterface VTTRegion {\\n    id: string;\\n    lines: number;\\n    regionAnchorX: number;\\n    regionAnchorY: number;\\n    scroll: ScrollSetting;\\n    viewportAnchorX: number;\\n    viewportAnchorY: number;\\n    width: number;\\n}\\n\\ndeclare var VTTRegion: {\\n    prototype: VTTRegion;\\n    new(): VTTRegion;\\n};\\n\\ninterface ValidityState {\\n    readonly badInput: boolean;\\n    readonly customError: boolean;\\n    readonly patternMismatch: boolean;\\n    readonly rangeOverflow: boolean;\\n    readonly rangeUnderflow: boolean;\\n    readonly stepMismatch: boolean;\\n    readonly tooLong: boolean;\\n    readonly tooShort: boolean;\\n    readonly typeMismatch: boolean;\\n    readonly valid: boolean;\\n    readonly valueMissing: boolean;\\n}\\n\\ndeclare var ValidityState: {\\n    prototype: ValidityState;\\n    new(): ValidityState;\\n};\\n\\ninterface VideoPlaybackQuality {\\n    readonly corruptedVideoFrames: number;\\n    readonly creationTime: number;\\n    readonly droppedVideoFrames: number;\\n    readonly totalFrameDelay: number;\\n    readonly totalVideoFrames: number;\\n}\\n\\ndeclare var VideoPlaybackQuality: {\\n    prototype: VideoPlaybackQuality;\\n    new(): VideoPlaybackQuality;\\n};\\n\\ninterface VideoTrack {\\n    readonly id: string;\\n    kind: string;\\n    readonly label: string;\\n    language: string;\\n    selected: boolean;\\n    readonly sourceBuffer: SourceBuffer;\\n}\\n\\ndeclare var VideoTrack: {\\n    prototype: VideoTrack;\\n    new(): VideoTrack;\\n};\\n\\ninterface VideoTrackListEventMap {\\n    \"addtrack\": TrackEvent;\\n    \"change\": Event;\\n    \"removetrack\": TrackEvent;\\n}\\n\\ninterface VideoTrackList extends EventTarget {\\n    readonly length: number;\\n    onaddtrack: ((this: VideoTrackList, ev: TrackEvent) => any) | null;\\n    onchange: ((this: VideoTrackList, ev: Event) => any) | null;\\n    onremovetrack: ((this: VideoTrackList, ev: TrackEvent) => any) | null;\\n    readonly selectedIndex: number;\\n    getTrackById(id: string): VideoTrack | null;\\n    item(index: number): VideoTrack;\\n    addEventListener<K extends keyof VideoTrackListEventMap>(type: K, listener: (this: VideoTrackList, ev: VideoTrackListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof VideoTrackListEventMap>(type: K, listener: (this: VideoTrackList, ev: VideoTrackListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: VideoTrack;\\n}\\n\\ndeclare var VideoTrackList: {\\n    prototype: VideoTrackList;\\n    new(): VideoTrackList;\\n};\\n\\ninterface WEBGL_color_buffer_float {\\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT: number;\\n    readonly RGB32F_EXT: number;\\n    readonly RGBA32F_EXT: number;\\n    readonly UNSIGNED_NORMALIZED_EXT: number;\\n}\\n\\ninterface WEBGL_compressed_texture_astc {\\n    readonly COMPRESSED_RGBA_ASTC_10x10_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_10x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_10x6_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_10x8_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_12x10_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_12x12_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_4x4_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_5x4_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_5x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_6x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_6x6_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_8x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_8x6_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_8x8_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x10_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x6_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x8_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x4_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x6_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x6_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x8_KHR: number;\\n    getSupportedProfiles(): string[];\\n}\\n\\ninterface WEBGL_compressed_texture_s3tc {\\n    readonly COMPRESSED_RGBA_S3TC_DXT1_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT3_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT5_EXT: number;\\n    readonly COMPRESSED_RGB_S3TC_DXT1_EXT: number;\\n}\\n\\ndeclare var WEBGL_compressed_texture_s3tc: {\\n    prototype: WEBGL_compressed_texture_s3tc;\\n    new(): WEBGL_compressed_texture_s3tc;\\n    readonly COMPRESSED_RGBA_S3TC_DXT1_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT3_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT5_EXT: number;\\n    readonly COMPRESSED_RGB_S3TC_DXT1_EXT: number;\\n};\\n\\ninterface WEBGL_compressed_texture_s3tc_srgb {\\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT: number;\\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT: number;\\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT: number;\\n    readonly COMPRESSED_SRGB_S3TC_DXT1_EXT: number;\\n}\\n\\ninterface WEBGL_debug_renderer_info {\\n    readonly UNMASKED_RENDERER_WEBGL: number;\\n    readonly UNMASKED_VENDOR_WEBGL: number;\\n}\\n\\ndeclare var WEBGL_debug_renderer_info: {\\n    prototype: WEBGL_debug_renderer_info;\\n    new(): WEBGL_debug_renderer_info;\\n    readonly UNMASKED_RENDERER_WEBGL: number;\\n    readonly UNMASKED_VENDOR_WEBGL: number;\\n};\\n\\ninterface WEBGL_debug_shaders {\\n    getTranslatedShaderSource(shader: WebGLShader): string;\\n}\\n\\ninterface WEBGL_depth_texture {\\n    readonly UNSIGNED_INT_24_8_WEBGL: number;\\n}\\n\\ndeclare var WEBGL_depth_texture: {\\n    prototype: WEBGL_depth_texture;\\n    new(): WEBGL_depth_texture;\\n    readonly UNSIGNED_INT_24_8_WEBGL: number;\\n};\\n\\ninterface WEBGL_draw_buffers {\\n    readonly COLOR_ATTACHMENT0_WEBGL: number;\\n    readonly COLOR_ATTACHMENT10_WEBGL: number;\\n    readonly COLOR_ATTACHMENT11_WEBGL: number;\\n    readonly COLOR_ATTACHMENT12_WEBGL: number;\\n    readonly COLOR_ATTACHMENT13_WEBGL: number;\\n    readonly COLOR_ATTACHMENT14_WEBGL: number;\\n    readonly COLOR_ATTACHMENT15_WEBGL: number;\\n    readonly COLOR_ATTACHMENT1_WEBGL: number;\\n    readonly COLOR_ATTACHMENT2_WEBGL: number;\\n    readonly COLOR_ATTACHMENT3_WEBGL: number;\\n    readonly COLOR_ATTACHMENT4_WEBGL: number;\\n    readonly COLOR_ATTACHMENT5_WEBGL: number;\\n    readonly COLOR_ATTACHMENT6_WEBGL: number;\\n    readonly COLOR_ATTACHMENT7_WEBGL: number;\\n    readonly COLOR_ATTACHMENT8_WEBGL: number;\\n    readonly COLOR_ATTACHMENT9_WEBGL: number;\\n    readonly DRAW_BUFFER0_WEBGL: number;\\n    readonly DRAW_BUFFER10_WEBGL: number;\\n    readonly DRAW_BUFFER11_WEBGL: number;\\n    readonly DRAW_BUFFER12_WEBGL: number;\\n    readonly DRAW_BUFFER13_WEBGL: number;\\n    readonly DRAW_BUFFER14_WEBGL: number;\\n    readonly DRAW_BUFFER15_WEBGL: number;\\n    readonly DRAW_BUFFER1_WEBGL: number;\\n    readonly DRAW_BUFFER2_WEBGL: number;\\n    readonly DRAW_BUFFER3_WEBGL: number;\\n    readonly DRAW_BUFFER4_WEBGL: number;\\n    readonly DRAW_BUFFER5_WEBGL: number;\\n    readonly DRAW_BUFFER6_WEBGL: number;\\n    readonly DRAW_BUFFER7_WEBGL: number;\\n    readonly DRAW_BUFFER8_WEBGL: number;\\n    readonly DRAW_BUFFER9_WEBGL: number;\\n    readonly MAX_COLOR_ATTACHMENTS_WEBGL: number;\\n    readonly MAX_DRAW_BUFFERS_WEBGL: number;\\n    drawBuffersWEBGL(buffers: number[]): void;\\n}\\n\\ninterface WEBGL_lose_context {\\n    loseContext(): void;\\n    restoreContext(): void;\\n}\\n\\ninterface WaveShaperNode extends AudioNode {\\n    curve: Float32Array | null;\\n    oversample: OverSampleType;\\n}\\n\\ndeclare var WaveShaperNode: {\\n    prototype: WaveShaperNode;\\n    new(context: BaseAudioContext, options?: WaveShaperOptions): WaveShaperNode;\\n};\\n\\ninterface WebAuthentication {\\n    getAssertion(assertionChallenge: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null, options?: AssertionOptions): Promise<WebAuthnAssertion>;\\n    makeCredential(accountInformation: Account, cryptoParameters: ScopedCredentialParameters[], attestationChallenge: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null, options?: ScopedCredentialOptions): Promise<ScopedCredentialInfo>;\\n}\\n\\ndeclare var WebAuthentication: {\\n    prototype: WebAuthentication;\\n    new(): WebAuthentication;\\n};\\n\\ninterface WebAuthnAssertion {\\n    readonly authenticatorData: ArrayBuffer;\\n    readonly clientData: ArrayBuffer;\\n    readonly credential: ScopedCredential;\\n    readonly signature: ArrayBuffer;\\n}\\n\\ndeclare var WebAuthnAssertion: {\\n    prototype: WebAuthnAssertion;\\n    new(): WebAuthnAssertion;\\n};\\n\\ninterface WebGLActiveInfo {\\n    readonly name: string;\\n    readonly size: GLint;\\n    readonly type: GLenum;\\n}\\n\\ndeclare var WebGLActiveInfo: {\\n    prototype: WebGLActiveInfo;\\n    new(): WebGLActiveInfo;\\n};\\n\\ninterface WebGLBuffer extends WebGLObject {\\n}\\n\\ndeclare var WebGLBuffer: {\\n    prototype: WebGLBuffer;\\n    new(): WebGLBuffer;\\n};\\n\\ninterface WebGLContextEvent extends Event {\\n    readonly statusMessage: string;\\n}\\n\\ndeclare var WebGLContextEvent: {\\n    prototype: WebGLContextEvent;\\n    new(type: string, eventInit?: WebGLContextEventInit): WebGLContextEvent;\\n};\\n\\ninterface WebGLFramebuffer extends WebGLObject {\\n}\\n\\ndeclare var WebGLFramebuffer: {\\n    prototype: WebGLFramebuffer;\\n    new(): WebGLFramebuffer;\\n};\\n\\ninterface WebGLObject {\\n}\\n\\ndeclare var WebGLObject: {\\n    prototype: WebGLObject;\\n    new(): WebGLObject;\\n};\\n\\ninterface WebGLProgram extends WebGLObject {\\n}\\n\\ndeclare var WebGLProgram: {\\n    prototype: WebGLProgram;\\n    new(): WebGLProgram;\\n};\\n\\ninterface WebGLRenderbuffer extends WebGLObject {\\n}\\n\\ndeclare var WebGLRenderbuffer: {\\n    prototype: WebGLRenderbuffer;\\n    new(): WebGLRenderbuffer;\\n};\\n\\ninterface WebGLRenderingContext extends WebGLRenderingContextBase {\\n}\\n\\ndeclare var WebGLRenderingContext: {\\n    prototype: WebGLRenderingContext;\\n    new(): WebGLRenderingContext;\\n    readonly ACTIVE_ATTRIBUTES: GLenum;\\n    readonly ACTIVE_TEXTURE: GLenum;\\n    readonly ACTIVE_UNIFORMS: GLenum;\\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\\n    readonly ALPHA: GLenum;\\n    readonly ALPHA_BITS: GLenum;\\n    readonly ALWAYS: GLenum;\\n    readonly ARRAY_BUFFER: GLenum;\\n    readonly ARRAY_BUFFER_BINDING: GLenum;\\n    readonly ATTACHED_SHADERS: GLenum;\\n    readonly BACK: GLenum;\\n    readonly BLEND: GLenum;\\n    readonly BLEND_COLOR: GLenum;\\n    readonly BLEND_DST_ALPHA: GLenum;\\n    readonly BLEND_DST_RGB: GLenum;\\n    readonly BLEND_EQUATION: GLenum;\\n    readonly BLEND_EQUATION_ALPHA: GLenum;\\n    readonly BLEND_EQUATION_RGB: GLenum;\\n    readonly BLEND_SRC_ALPHA: GLenum;\\n    readonly BLEND_SRC_RGB: GLenum;\\n    readonly BLUE_BITS: GLenum;\\n    readonly BOOL: GLenum;\\n    readonly BOOL_VEC2: GLenum;\\n    readonly BOOL_VEC3: GLenum;\\n    readonly BOOL_VEC4: GLenum;\\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\\n    readonly BUFFER_SIZE: GLenum;\\n    readonly BUFFER_USAGE: GLenum;\\n    readonly BYTE: GLenum;\\n    readonly CCW: GLenum;\\n    readonly CLAMP_TO_EDGE: GLenum;\\n    readonly COLOR_ATTACHMENT0: GLenum;\\n    readonly COLOR_BUFFER_BIT: GLenum;\\n    readonly COLOR_CLEAR_VALUE: GLenum;\\n    readonly COLOR_WRITEMASK: GLenum;\\n    readonly COMPILE_STATUS: GLenum;\\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\\n    readonly CONSTANT_ALPHA: GLenum;\\n    readonly CONSTANT_COLOR: GLenum;\\n    readonly CONTEXT_LOST_WEBGL: GLenum;\\n    readonly CULL_FACE: GLenum;\\n    readonly CULL_FACE_MODE: GLenum;\\n    readonly CURRENT_PROGRAM: GLenum;\\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\\n    readonly CW: GLenum;\\n    readonly DECR: GLenum;\\n    readonly DECR_WRAP: GLenum;\\n    readonly DELETE_STATUS: GLenum;\\n    readonly DEPTH_ATTACHMENT: GLenum;\\n    readonly DEPTH_BITS: GLenum;\\n    readonly DEPTH_BUFFER_BIT: GLenum;\\n    readonly DEPTH_CLEAR_VALUE: GLenum;\\n    readonly DEPTH_COMPONENT: GLenum;\\n    readonly DEPTH_COMPONENT16: GLenum;\\n    readonly DEPTH_FUNC: GLenum;\\n    readonly DEPTH_RANGE: GLenum;\\n    readonly DEPTH_STENCIL: GLenum;\\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\\n    readonly DEPTH_TEST: GLenum;\\n    readonly DEPTH_WRITEMASK: GLenum;\\n    readonly DITHER: GLenum;\\n    readonly DONT_CARE: GLenum;\\n    readonly DST_ALPHA: GLenum;\\n    readonly DST_COLOR: GLenum;\\n    readonly DYNAMIC_DRAW: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly EQUAL: GLenum;\\n    readonly FASTEST: GLenum;\\n    readonly FLOAT: GLenum;\\n    readonly FLOAT_MAT2: GLenum;\\n    readonly FLOAT_MAT3: GLenum;\\n    readonly FLOAT_MAT4: GLenum;\\n    readonly FLOAT_VEC2: GLenum;\\n    readonly FLOAT_VEC3: GLenum;\\n    readonly FLOAT_VEC4: GLenum;\\n    readonly FRAGMENT_SHADER: GLenum;\\n    readonly FRAMEBUFFER: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\\n    readonly FRAMEBUFFER_BINDING: GLenum;\\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\\n    readonly FRONT: GLenum;\\n    readonly FRONT_AND_BACK: GLenum;\\n    readonly FRONT_FACE: GLenum;\\n    readonly FUNC_ADD: GLenum;\\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\\n    readonly FUNC_SUBTRACT: GLenum;\\n    readonly GENERATE_MIPMAP_HINT: GLenum;\\n    readonly GEQUAL: GLenum;\\n    readonly GREATER: GLenum;\\n    readonly GREEN_BITS: GLenum;\\n    readonly HIGH_FLOAT: GLenum;\\n    readonly HIGH_INT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\\n    readonly INCR: GLenum;\\n    readonly INCR_WRAP: GLenum;\\n    readonly INT: GLenum;\\n    readonly INT_VEC2: GLenum;\\n    readonly INT_VEC3: GLenum;\\n    readonly INT_VEC4: GLenum;\\n    readonly INVALID_ENUM: GLenum;\\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\\n    readonly INVALID_OPERATION: GLenum;\\n    readonly INVALID_VALUE: GLenum;\\n    readonly INVERT: GLenum;\\n    readonly KEEP: GLenum;\\n    readonly LEQUAL: GLenum;\\n    readonly LESS: GLenum;\\n    readonly LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\\n    readonly LINES: GLenum;\\n    readonly LINE_LOOP: GLenum;\\n    readonly LINE_STRIP: GLenum;\\n    readonly LINE_WIDTH: GLenum;\\n    readonly LINK_STATUS: GLenum;\\n    readonly LOW_FLOAT: GLenum;\\n    readonly LOW_INT: GLenum;\\n    readonly LUMINANCE: GLenum;\\n    readonly LUMINANCE_ALPHA: GLenum;\\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_TEXTURE_SIZE: GLenum;\\n    readonly MAX_VARYING_VECTORS: GLenum;\\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_VIEWPORT_DIMS: GLenum;\\n    readonly MEDIUM_FLOAT: GLenum;\\n    readonly MEDIUM_INT: GLenum;\\n    readonly MIRRORED_REPEAT: GLenum;\\n    readonly NEAREST: GLenum;\\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\\n    readonly NEVER: GLenum;\\n    readonly NICEST: GLenum;\\n    readonly NONE: GLenum;\\n    readonly NOTEQUAL: GLenum;\\n    readonly NO_ERROR: GLenum;\\n    readonly ONE: GLenum;\\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\\n    readonly ONE_MINUS_DST_COLOR: GLenum;\\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\\n    readonly OUT_OF_MEMORY: GLenum;\\n    readonly PACK_ALIGNMENT: GLenum;\\n    readonly POINTS: GLenum;\\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\\n    readonly POLYGON_OFFSET_FILL: GLenum;\\n    readonly POLYGON_OFFSET_UNITS: GLenum;\\n    readonly RED_BITS: GLenum;\\n    readonly RENDERBUFFER: GLenum;\\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\\n    readonly RENDERBUFFER_BINDING: GLenum;\\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\\n    readonly RENDERBUFFER_HEIGHT: GLenum;\\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\\n    readonly RENDERBUFFER_WIDTH: GLenum;\\n    readonly RENDERER: GLenum;\\n    readonly REPEAT: GLenum;\\n    readonly REPLACE: GLenum;\\n    readonly RGB: GLenum;\\n    readonly RGB565: GLenum;\\n    readonly RGB5_A1: GLenum;\\n    readonly RGBA: GLenum;\\n    readonly RGBA4: GLenum;\\n    readonly SAMPLER_2D: GLenum;\\n    readonly SAMPLER_CUBE: GLenum;\\n    readonly SAMPLES: GLenum;\\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\\n    readonly SAMPLE_BUFFERS: GLenum;\\n    readonly SAMPLE_COVERAGE: GLenum;\\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\\n    readonly SCISSOR_BOX: GLenum;\\n    readonly SCISSOR_TEST: GLenum;\\n    readonly SHADER_TYPE: GLenum;\\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\\n    readonly SHORT: GLenum;\\n    readonly SRC_ALPHA: GLenum;\\n    readonly SRC_ALPHA_SATURATE: GLenum;\\n    readonly SRC_COLOR: GLenum;\\n    readonly STATIC_DRAW: GLenum;\\n    readonly STENCIL_ATTACHMENT: GLenum;\\n    readonly STENCIL_BACK_FAIL: GLenum;\\n    readonly STENCIL_BACK_FUNC: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_BACK_REF: GLenum;\\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\\n    readonly STENCIL_BITS: GLenum;\\n    readonly STENCIL_BUFFER_BIT: GLenum;\\n    readonly STENCIL_CLEAR_VALUE: GLenum;\\n    readonly STENCIL_FAIL: GLenum;\\n    readonly STENCIL_FUNC: GLenum;\\n    readonly STENCIL_INDEX8: GLenum;\\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_REF: GLenum;\\n    readonly STENCIL_TEST: GLenum;\\n    readonly STENCIL_VALUE_MASK: GLenum;\\n    readonly STENCIL_WRITEMASK: GLenum;\\n    readonly STREAM_DRAW: GLenum;\\n    readonly SUBPIXEL_BITS: GLenum;\\n    readonly TEXTURE: GLenum;\\n    readonly TEXTURE0: GLenum;\\n    readonly TEXTURE1: GLenum;\\n    readonly TEXTURE10: GLenum;\\n    readonly TEXTURE11: GLenum;\\n    readonly TEXTURE12: GLenum;\\n    readonly TEXTURE13: GLenum;\\n    readonly TEXTURE14: GLenum;\\n    readonly TEXTURE15: GLenum;\\n    readonly TEXTURE16: GLenum;\\n    readonly TEXTURE17: GLenum;\\n    readonly TEXTURE18: GLenum;\\n    readonly TEXTURE19: GLenum;\\n    readonly TEXTURE2: GLenum;\\n    readonly TEXTURE20: GLenum;\\n    readonly TEXTURE21: GLenum;\\n    readonly TEXTURE22: GLenum;\\n    readonly TEXTURE23: GLenum;\\n    readonly TEXTURE24: GLenum;\\n    readonly TEXTURE25: GLenum;\\n    readonly TEXTURE26: GLenum;\\n    readonly TEXTURE27: GLenum;\\n    readonly TEXTURE28: GLenum;\\n    readonly TEXTURE29: GLenum;\\n    readonly TEXTURE3: GLenum;\\n    readonly TEXTURE30: GLenum;\\n    readonly TEXTURE31: GLenum;\\n    readonly TEXTURE4: GLenum;\\n    readonly TEXTURE5: GLenum;\\n    readonly TEXTURE6: GLenum;\\n    readonly TEXTURE7: GLenum;\\n    readonly TEXTURE8: GLenum;\\n    readonly TEXTURE9: GLenum;\\n    readonly TEXTURE_2D: GLenum;\\n    readonly TEXTURE_BINDING_2D: GLenum;\\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\\n    readonly TEXTURE_MAG_FILTER: GLenum;\\n    readonly TEXTURE_MIN_FILTER: GLenum;\\n    readonly TEXTURE_WRAP_S: GLenum;\\n    readonly TEXTURE_WRAP_T: GLenum;\\n    readonly TRIANGLES: GLenum;\\n    readonly TRIANGLE_FAN: GLenum;\\n    readonly TRIANGLE_STRIP: GLenum;\\n    readonly UNPACK_ALIGNMENT: GLenum;\\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\\n    readonly UNSIGNED_BYTE: GLenum;\\n    readonly UNSIGNED_INT: GLenum;\\n    readonly UNSIGNED_SHORT: GLenum;\\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\\n    readonly VALIDATE_STATUS: GLenum;\\n    readonly VENDOR: GLenum;\\n    readonly VERSION: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\\n    readonly VERTEX_SHADER: GLenum;\\n    readonly VIEWPORT: GLenum;\\n    readonly ZERO: GLenum;\\n};\\n\\ninterface WebGLRenderingContextBase {\\n    readonly canvas: HTMLCanvasElement;\\n    readonly drawingBufferHeight: GLsizei;\\n    readonly drawingBufferWidth: GLsizei;\\n    activeTexture(texture: GLenum): void;\\n    attachShader(program: WebGLProgram, shader: WebGLShader): void;\\n    bindAttribLocation(program: WebGLProgram, index: GLuint, name: string): void;\\n    bindBuffer(target: GLenum, buffer: WebGLBuffer | null): void;\\n    bindFramebuffer(target: GLenum, framebuffer: WebGLFramebuffer | null): void;\\n    bindRenderbuffer(target: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\\n    bindTexture(target: GLenum, texture: WebGLTexture | null): void;\\n    blendColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\\n    blendEquation(mode: GLenum): void;\\n    blendEquationSeparate(modeRGB: GLenum, modeAlpha: GLenum): void;\\n    blendFunc(sfactor: GLenum, dfactor: GLenum): void;\\n    blendFuncSeparate(srcRGB: GLenum, dstRGB: GLenum, srcAlpha: GLenum, dstAlpha: GLenum): void;\\n    bufferData(target: GLenum, size: GLsizeiptr, usage: GLenum): void;\\n    bufferData(target: GLenum, data: BufferSource | null, usage: GLenum): void;\\n    bufferSubData(target: GLenum, offset: GLintptr, data: BufferSource): void;\\n    checkFramebufferStatus(target: GLenum): GLenum;\\n    clear(mask: GLbitfield): void;\\n    clearColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\\n    clearDepth(depth: GLclampf): void;\\n    clearStencil(s: GLint): void;\\n    colorMask(red: GLboolean, green: GLboolean, blue: GLboolean, alpha: GLboolean): void;\\n    compileShader(shader: WebGLShader): void;\\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, data: ArrayBufferView): void;\\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, data: ArrayBufferView): void;\\n    copyTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, x: GLint, y: GLint, width: GLsizei, height: GLsizei, border: GLint): void;\\n    copyTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\\n    createBuffer(): WebGLBuffer | null;\\n    createFramebuffer(): WebGLFramebuffer | null;\\n    createProgram(): WebGLProgram | null;\\n    createRenderbuffer(): WebGLRenderbuffer | null;\\n    createShader(type: GLenum): WebGLShader | null;\\n    createTexture(): WebGLTexture | null;\\n    cullFace(mode: GLenum): void;\\n    deleteBuffer(buffer: WebGLBuffer | null): void;\\n    deleteFramebuffer(framebuffer: WebGLFramebuffer | null): void;\\n    deleteProgram(program: WebGLProgram | null): void;\\n    deleteRenderbuffer(renderbuffer: WebGLRenderbuffer | null): void;\\n    deleteShader(shader: WebGLShader | null): void;\\n    deleteTexture(texture: WebGLTexture | null): void;\\n    depthFunc(func: GLenum): void;\\n    depthMask(flag: GLboolean): void;\\n    depthRange(zNear: GLclampf, zFar: GLclampf): void;\\n    detachShader(program: WebGLProgram, shader: WebGLShader): void;\\n    disable(cap: GLenum): void;\\n    disableVertexAttribArray(index: GLuint): void;\\n    drawArrays(mode: GLenum, first: GLint, count: GLsizei): void;\\n    drawElements(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr): void;\\n    enable(cap: GLenum): void;\\n    enableVertexAttribArray(index: GLuint): void;\\n    finish(): void;\\n    flush(): void;\\n    framebufferRenderbuffer(target: GLenum, attachment: GLenum, renderbuffertarget: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\\n    framebufferTexture2D(target: GLenum, attachment: GLenum, textarget: GLenum, texture: WebGLTexture | null, level: GLint): void;\\n    frontFace(mode: GLenum): void;\\n    generateMipmap(target: GLenum): void;\\n    getActiveAttrib(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\\n    getActiveUniform(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\\n    getAttachedShaders(program: WebGLProgram): WebGLShader[] | null;\\n    getAttribLocation(program: WebGLProgram, name: string): GLint;\\n    getBufferParameter(target: GLenum, pname: GLenum): any;\\n    getContextAttributes(): WebGLContextAttributes | null;\\n    getError(): GLenum;\\n    getExtension(extensionName: \"EXT_blend_minmax\"): EXT_blend_minmax | null;\\n    getExtension(extensionName: \"EXT_texture_filter_anisotropic\"): EXT_texture_filter_anisotropic | null;\\n    getExtension(extensionName: \"EXT_frag_depth\"): EXT_frag_depth | null;\\n    getExtension(extensionName: \"EXT_shader_texture_lod\"): EXT_shader_texture_lod | null;\\n    getExtension(extensionName: \"EXT_sRGB\"): EXT_sRGB | null;\\n    getExtension(extensionName: \"OES_vertex_array_object\"): OES_vertex_array_object | null;\\n    getExtension(extensionName: \"WEBGL_color_buffer_float\"): WEBGL_color_buffer_float | null;\\n    getExtension(extensionName: \"WEBGL_compressed_texture_astc\"): WEBGL_compressed_texture_astc | null;\\n    getExtension(extensionName: \"WEBGL_compressed_texture_s3tc_srgb\"): WEBGL_compressed_texture_s3tc_srgb | null;\\n    getExtension(extensionName: \"WEBGL_debug_shaders\"): WEBGL_debug_shaders | null;\\n    getExtension(extensionName: \"WEBGL_draw_buffers\"): WEBGL_draw_buffers | null;\\n    getExtension(extensionName: \"WEBGL_lose_context\"): WEBGL_lose_context | null;\\n    getExtension(extensionName: \"WEBGL_depth_texture\"): WEBGL_depth_texture | null;\\n    getExtension(extensionName: \"WEBGL_debug_renderer_info\"): WEBGL_debug_renderer_info | null;\\n    getExtension(extensionName: \"WEBGL_compressed_texture_s3tc\"): WEBGL_compressed_texture_s3tc | null;\\n    getExtension(extensionName: \"OES_texture_half_float_linear\"): OES_texture_half_float_linear | null;\\n    getExtension(extensionName: \"OES_texture_half_float\"): OES_texture_half_float | null;\\n    getExtension(extensionName: \"OES_texture_float_linear\"): OES_texture_float_linear | null;\\n    getExtension(extensionName: \"OES_texture_float\"): OES_texture_float | null;\\n    getExtension(extensionName: \"OES_standard_derivatives\"): OES_standard_derivatives | null;\\n    getExtension(extensionName: \"OES_element_index_uint\"): OES_element_index_uint | null;\\n    getExtension(extensionName: \"ANGLE_instanced_arrays\"): ANGLE_instanced_arrays | null;\\n    getExtension(extensionName: string): any;\\n    getFramebufferAttachmentParameter(target: GLenum, attachment: GLenum, pname: GLenum): any;\\n    getParameter(pname: GLenum): any;\\n    getProgramInfoLog(program: WebGLProgram): string | null;\\n    getProgramParameter(program: WebGLProgram, pname: GLenum): any;\\n    getRenderbufferParameter(target: GLenum, pname: GLenum): any;\\n    getShaderInfoLog(shader: WebGLShader): string | null;\\n    getShaderParameter(shader: WebGLShader, pname: GLenum): any;\\n    getShaderPrecisionFormat(shadertype: GLenum, precisiontype: GLenum): WebGLShaderPrecisionFormat | null;\\n    getShaderSource(shader: WebGLShader): string | null;\\n    getSupportedExtensions(): string[] | null;\\n    getTexParameter(target: GLenum, pname: GLenum): any;\\n    getUniform(program: WebGLProgram, location: WebGLUniformLocation): any;\\n    getUniformLocation(program: WebGLProgram, name: string): WebGLUniformLocation | null;\\n    getVertexAttrib(index: GLuint, pname: GLenum): any;\\n    getVertexAttribOffset(index: GLuint, pname: GLenum): GLintptr;\\n    hint(target: GLenum, mode: GLenum): void;\\n    isBuffer(buffer: WebGLBuffer | null): GLboolean;\\n    isContextLost(): boolean;\\n    isEnabled(cap: GLenum): GLboolean;\\n    isFramebuffer(framebuffer: WebGLFramebuffer | null): GLboolean;\\n    isProgram(program: WebGLProgram | null): GLboolean;\\n    isRenderbuffer(renderbuffer: WebGLRenderbuffer | null): GLboolean;\\n    isShader(shader: WebGLShader | null): GLboolean;\\n    isTexture(texture: WebGLTexture | null): GLboolean;\\n    lineWidth(width: GLfloat): void;\\n    linkProgram(program: WebGLProgram): void;\\n    pixelStorei(pname: GLenum, param: GLint): void;\\n    polygonOffset(factor: GLfloat, units: GLfloat): void;\\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\\n    renderbufferStorage(target: GLenum, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\\n    sampleCoverage(value: GLclampf, invert: GLboolean): void;\\n    scissor(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\\n    shaderSource(shader: WebGLShader, source: string): void;\\n    stencilFunc(func: GLenum, ref: GLint, mask: GLuint): void;\\n    stencilFuncSeparate(face: GLenum, func: GLenum, ref: GLint, mask: GLuint): void;\\n    stencilMask(mask: GLuint): void;\\n    stencilMaskSeparate(face: GLenum, mask: GLuint): void;\\n    stencilOp(fail: GLenum, zfail: GLenum, zpass: GLenum): void;\\n    stencilOpSeparate(face: GLenum, fail: GLenum, zfail: GLenum, zpass: GLenum): void;\\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\\n    texParameterf(target: GLenum, pname: GLenum, param: GLfloat): void;\\n    texParameteri(target: GLenum, pname: GLenum, param: GLint): void;\\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\\n    uniform1f(location: WebGLUniformLocation | null, x: GLfloat): void;\\n    uniform1fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform1i(location: WebGLUniformLocation | null, x: GLint): void;\\n    uniform1iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniform2f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat): void;\\n    uniform2fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform2i(location: WebGLUniformLocation | null, x: GLint, y: GLint): void;\\n    uniform2iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniform3f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat): void;\\n    uniform3fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform3i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint): void;\\n    uniform3iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniform4f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\\n    uniform4fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform4i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint, w: GLint): void;\\n    uniform4iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\\n    useProgram(program: WebGLProgram | null): void;\\n    validateProgram(program: WebGLProgram): void;\\n    vertexAttrib1f(index: GLuint, x: GLfloat): void;\\n    vertexAttrib1fv(index: GLuint, values: Float32List): void;\\n    vertexAttrib2f(index: GLuint, x: GLfloat, y: GLfloat): void;\\n    vertexAttrib2fv(index: GLuint, values: Float32List): void;\\n    vertexAttrib3f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat): void;\\n    vertexAttrib3fv(index: GLuint, values: Float32List): void;\\n    vertexAttrib4f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\\n    vertexAttrib4fv(index: GLuint, values: Float32List): void;\\n    vertexAttribPointer(index: GLuint, size: GLint, type: GLenum, normalized: GLboolean, stride: GLsizei, offset: GLintptr): void;\\n    viewport(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\\n    readonly ACTIVE_ATTRIBUTES: GLenum;\\n    readonly ACTIVE_TEXTURE: GLenum;\\n    readonly ACTIVE_UNIFORMS: GLenum;\\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\\n    readonly ALPHA: GLenum;\\n    readonly ALPHA_BITS: GLenum;\\n    readonly ALWAYS: GLenum;\\n    readonly ARRAY_BUFFER: GLenum;\\n    readonly ARRAY_BUFFER_BINDING: GLenum;\\n    readonly ATTACHED_SHADERS: GLenum;\\n    readonly BACK: GLenum;\\n    readonly BLEND: GLenum;\\n    readonly BLEND_COLOR: GLenum;\\n    readonly BLEND_DST_ALPHA: GLenum;\\n    readonly BLEND_DST_RGB: GLenum;\\n    readonly BLEND_EQUATION: GLenum;\\n    readonly BLEND_EQUATION_ALPHA: GLenum;\\n    readonly BLEND_EQUATION_RGB: GLenum;\\n    readonly BLEND_SRC_ALPHA: GLenum;\\n    readonly BLEND_SRC_RGB: GLenum;\\n    readonly BLUE_BITS: GLenum;\\n    readonly BOOL: GLenum;\\n    readonly BOOL_VEC2: GLenum;\\n    readonly BOOL_VEC3: GLenum;\\n    readonly BOOL_VEC4: GLenum;\\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\\n    readonly BUFFER_SIZE: GLenum;\\n    readonly BUFFER_USAGE: GLenum;\\n    readonly BYTE: GLenum;\\n    readonly CCW: GLenum;\\n    readonly CLAMP_TO_EDGE: GLenum;\\n    readonly COLOR_ATTACHMENT0: GLenum;\\n    readonly COLOR_BUFFER_BIT: GLenum;\\n    readonly COLOR_CLEAR_VALUE: GLenum;\\n    readonly COLOR_WRITEMASK: GLenum;\\n    readonly COMPILE_STATUS: GLenum;\\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\\n    readonly CONSTANT_ALPHA: GLenum;\\n    readonly CONSTANT_COLOR: GLenum;\\n    readonly CONTEXT_LOST_WEBGL: GLenum;\\n    readonly CULL_FACE: GLenum;\\n    readonly CULL_FACE_MODE: GLenum;\\n    readonly CURRENT_PROGRAM: GLenum;\\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\\n    readonly CW: GLenum;\\n    readonly DECR: GLenum;\\n    readonly DECR_WRAP: GLenum;\\n    readonly DELETE_STATUS: GLenum;\\n    readonly DEPTH_ATTACHMENT: GLenum;\\n    readonly DEPTH_BITS: GLenum;\\n    readonly DEPTH_BUFFER_BIT: GLenum;\\n    readonly DEPTH_CLEAR_VALUE: GLenum;\\n    readonly DEPTH_COMPONENT: GLenum;\\n    readonly DEPTH_COMPONENT16: GLenum;\\n    readonly DEPTH_FUNC: GLenum;\\n    readonly DEPTH_RANGE: GLenum;\\n    readonly DEPTH_STENCIL: GLenum;\\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\\n    readonly DEPTH_TEST: GLenum;\\n    readonly DEPTH_WRITEMASK: GLenum;\\n    readonly DITHER: GLenum;\\n    readonly DONT_CARE: GLenum;\\n    readonly DST_ALPHA: GLenum;\\n    readonly DST_COLOR: GLenum;\\n    readonly DYNAMIC_DRAW: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly EQUAL: GLenum;\\n    readonly FASTEST: GLenum;\\n    readonly FLOAT: GLenum;\\n    readonly FLOAT_MAT2: GLenum;\\n    readonly FLOAT_MAT3: GLenum;\\n    readonly FLOAT_MAT4: GLenum;\\n    readonly FLOAT_VEC2: GLenum;\\n    readonly FLOAT_VEC3: GLenum;\\n    readonly FLOAT_VEC4: GLenum;\\n    readonly FRAGMENT_SHADER: GLenum;\\n    readonly FRAMEBUFFER: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\\n    readonly FRAMEBUFFER_BINDING: GLenum;\\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\\n    readonly FRONT: GLenum;\\n    readonly FRONT_AND_BACK: GLenum;\\n    readonly FRONT_FACE: GLenum;\\n    readonly FUNC_ADD: GLenum;\\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\\n    readonly FUNC_SUBTRACT: GLenum;\\n    readonly GENERATE_MIPMAP_HINT: GLenum;\\n    readonly GEQUAL: GLenum;\\n    readonly GREATER: GLenum;\\n    readonly GREEN_BITS: GLenum;\\n    readonly HIGH_FLOAT: GLenum;\\n    readonly HIGH_INT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\\n    readonly INCR: GLenum;\\n    readonly INCR_WRAP: GLenum;\\n    readonly INT: GLenum;\\n    readonly INT_VEC2: GLenum;\\n    readonly INT_VEC3: GLenum;\\n    readonly INT_VEC4: GLenum;\\n    readonly INVALID_ENUM: GLenum;\\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\\n    readonly INVALID_OPERATION: GLenum;\\n    readonly INVALID_VALUE: GLenum;\\n    readonly INVERT: GLenum;\\n    readonly KEEP: GLenum;\\n    readonly LEQUAL: GLenum;\\n    readonly LESS: GLenum;\\n    readonly LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\\n    readonly LINES: GLenum;\\n    readonly LINE_LOOP: GLenum;\\n    readonly LINE_STRIP: GLenum;\\n    readonly LINE_WIDTH: GLenum;\\n    readonly LINK_STATUS: GLenum;\\n    readonly LOW_FLOAT: GLenum;\\n    readonly LOW_INT: GLenum;\\n    readonly LUMINANCE: GLenum;\\n    readonly LUMINANCE_ALPHA: GLenum;\\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_TEXTURE_SIZE: GLenum;\\n    readonly MAX_VARYING_VECTORS: GLenum;\\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_VIEWPORT_DIMS: GLenum;\\n    readonly MEDIUM_FLOAT: GLenum;\\n    readonly MEDIUM_INT: GLenum;\\n    readonly MIRRORED_REPEAT: GLenum;\\n    readonly NEAREST: GLenum;\\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\\n    readonly NEVER: GLenum;\\n    readonly NICEST: GLenum;\\n    readonly NONE: GLenum;\\n    readonly NOTEQUAL: GLenum;\\n    readonly NO_ERROR: GLenum;\\n    readonly ONE: GLenum;\\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\\n    readonly ONE_MINUS_DST_COLOR: GLenum;\\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\\n    readonly OUT_OF_MEMORY: GLenum;\\n    readonly PACK_ALIGNMENT: GLenum;\\n    readonly POINTS: GLenum;\\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\\n    readonly POLYGON_OFFSET_FILL: GLenum;\\n    readonly POLYGON_OFFSET_UNITS: GLenum;\\n    readonly RED_BITS: GLenum;\\n    readonly RENDERBUFFER: GLenum;\\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\\n    readonly RENDERBUFFER_BINDING: GLenum;\\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\\n    readonly RENDERBUFFER_HEIGHT: GLenum;\\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\\n    readonly RENDERBUFFER_WIDTH: GLenum;\\n    readonly RENDERER: GLenum;\\n    readonly REPEAT: GLenum;\\n    readonly REPLACE: GLenum;\\n    readonly RGB: GLenum;\\n    readonly RGB565: GLenum;\\n    readonly RGB5_A1: GLenum;\\n    readonly RGBA: GLenum;\\n    readonly RGBA4: GLenum;\\n    readonly SAMPLER_2D: GLenum;\\n    readonly SAMPLER_CUBE: GLenum;\\n    readonly SAMPLES: GLenum;\\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\\n    readonly SAMPLE_BUFFERS: GLenum;\\n    readonly SAMPLE_COVERAGE: GLenum;\\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\\n    readonly SCISSOR_BOX: GLenum;\\n    readonly SCISSOR_TEST: GLenum;\\n    readonly SHADER_TYPE: GLenum;\\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\\n    readonly SHORT: GLenum;\\n    readonly SRC_ALPHA: GLenum;\\n    readonly SRC_ALPHA_SATURATE: GLenum;\\n    readonly SRC_COLOR: GLenum;\\n    readonly STATIC_DRAW: GLenum;\\n    readonly STENCIL_ATTACHMENT: GLenum;\\n    readonly STENCIL_BACK_FAIL: GLenum;\\n    readonly STENCIL_BACK_FUNC: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_BACK_REF: GLenum;\\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\\n    readonly STENCIL_BITS: GLenum;\\n    readonly STENCIL_BUFFER_BIT: GLenum;\\n    readonly STENCIL_CLEAR_VALUE: GLenum;\\n    readonly STENCIL_FAIL: GLenum;\\n    readonly STENCIL_FUNC: GLenum;\\n    readonly STENCIL_INDEX8: GLenum;\\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_REF: GLenum;\\n    readonly STENCIL_TEST: GLenum;\\n    readonly STENCIL_VALUE_MASK: GLenum;\\n    readonly STENCIL_WRITEMASK: GLenum;\\n    readonly STREAM_DRAW: GLenum;\\n    readonly SUBPIXEL_BITS: GLenum;\\n    readonly TEXTURE: GLenum;\\n    readonly TEXTURE0: GLenum;\\n    readonly TEXTURE1: GLenum;\\n    readonly TEXTURE10: GLenum;\\n    readonly TEXTURE11: GLenum;\\n    readonly TEXTURE12: GLenum;\\n    readonly TEXTURE13: GLenum;\\n    readonly TEXTURE14: GLenum;\\n    readonly TEXTURE15: GLenum;\\n    readonly TEXTURE16: GLenum;\\n    readonly TEXTURE17: GLenum;\\n    readonly TEXTURE18: GLenum;\\n    readonly TEXTURE19: GLenum;\\n    readonly TEXTURE2: GLenum;\\n    readonly TEXTURE20: GLenum;\\n    readonly TEXTURE21: GLenum;\\n    readonly TEXTURE22: GLenum;\\n    readonly TEXTURE23: GLenum;\\n    readonly TEXTURE24: GLenum;\\n    readonly TEXTURE25: GLenum;\\n    readonly TEXTURE26: GLenum;\\n    readonly TEXTURE27: GLenum;\\n    readonly TEXTURE28: GLenum;\\n    readonly TEXTURE29: GLenum;\\n    readonly TEXTURE3: GLenum;\\n    readonly TEXTURE30: GLenum;\\n    readonly TEXTURE31: GLenum;\\n    readonly TEXTURE4: GLenum;\\n    readonly TEXTURE5: GLenum;\\n    readonly TEXTURE6: GLenum;\\n    readonly TEXTURE7: GLenum;\\n    readonly TEXTURE8: GLenum;\\n    readonly TEXTURE9: GLenum;\\n    readonly TEXTURE_2D: GLenum;\\n    readonly TEXTURE_BINDING_2D: GLenum;\\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\\n    readonly TEXTURE_MAG_FILTER: GLenum;\\n    readonly TEXTURE_MIN_FILTER: GLenum;\\n    readonly TEXTURE_WRAP_S: GLenum;\\n    readonly TEXTURE_WRAP_T: GLenum;\\n    readonly TRIANGLES: GLenum;\\n    readonly TRIANGLE_FAN: GLenum;\\n    readonly TRIANGLE_STRIP: GLenum;\\n    readonly UNPACK_ALIGNMENT: GLenum;\\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\\n    readonly UNSIGNED_BYTE: GLenum;\\n    readonly UNSIGNED_INT: GLenum;\\n    readonly UNSIGNED_SHORT: GLenum;\\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\\n    readonly VALIDATE_STATUS: GLenum;\\n    readonly VENDOR: GLenum;\\n    readonly VERSION: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\\n    readonly VERTEX_SHADER: GLenum;\\n    readonly VIEWPORT: GLenum;\\n    readonly ZERO: GLenum;\\n}\\n\\ninterface WebGLShader extends WebGLObject {\\n}\\n\\ndeclare var WebGLShader: {\\n    prototype: WebGLShader;\\n    new(): WebGLShader;\\n};\\n\\ninterface WebGLShaderPrecisionFormat {\\n    readonly precision: GLint;\\n    readonly rangeMax: GLint;\\n    readonly rangeMin: GLint;\\n}\\n\\ndeclare var WebGLShaderPrecisionFormat: {\\n    prototype: WebGLShaderPrecisionFormat;\\n    new(): WebGLShaderPrecisionFormat;\\n};\\n\\ninterface WebGLTexture extends WebGLObject {\\n}\\n\\ndeclare var WebGLTexture: {\\n    prototype: WebGLTexture;\\n    new(): WebGLTexture;\\n};\\n\\ninterface WebGLUniformLocation {\\n}\\n\\ndeclare var WebGLUniformLocation: {\\n    prototype: WebGLUniformLocation;\\n    new(): WebGLUniformLocation;\\n};\\n\\ninterface WebGLVertexArrayObjectOES {\\n}\\n\\ninterface WebKitPoint {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var WebKitPoint: {\\n    prototype: WebKitPoint;\\n    new(x?: number, y?: number): WebKitPoint;\\n};\\n\\ninterface WebSocketEventMap {\\n    \"close\": CloseEvent;\\n    \"error\": Event;\\n    \"message\": MessageEvent;\\n    \"open\": Event;\\n}\\n\\ninterface WebSocket extends EventTarget {\\n    binaryType: BinaryType;\\n    readonly bufferedAmount: number;\\n    readonly extensions: string;\\n    onclose: ((this: WebSocket, ev: CloseEvent) => any) | null;\\n    onerror: ((this: WebSocket, ev: Event) => any) | null;\\n    onmessage: ((this: WebSocket, ev: MessageEvent) => any) | null;\\n    onopen: ((this: WebSocket, ev: Event) => any) | null;\\n    readonly protocol: string;\\n    readonly readyState: number;\\n    readonly url: string;\\n    close(code?: number, reason?: string): void;\\n    send(data: string | ArrayBufferLike | Blob | ArrayBufferView): void;\\n    readonly CLOSED: number;\\n    readonly CLOSING: number;\\n    readonly CONNECTING: number;\\n    readonly OPEN: number;\\n    addEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var WebSocket: {\\n    prototype: WebSocket;\\n    new(url: string, protocols?: string | string[]): WebSocket;\\n    readonly CLOSED: number;\\n    readonly CLOSING: number;\\n    readonly CONNECTING: number;\\n    readonly OPEN: number;\\n};\\n\\ninterface WheelEvent extends MouseEvent {\\n    readonly deltaMode: number;\\n    readonly deltaX: number;\\n    readonly deltaY: number;\\n    readonly deltaZ: number;\\n    readonly wheelDelta: number;\\n    readonly wheelDeltaX: number;\\n    readonly wheelDeltaY: number;\\n    getCurrentPoint(element: Element): void;\\n    initWheelEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, buttonArg: number, relatedTargetArg: EventTarget, modifiersListArg: string, deltaXArg: number, deltaYArg: number, deltaZArg: number, deltaMode: number): void;\\n    readonly DOM_DELTA_LINE: number;\\n    readonly DOM_DELTA_PAGE: number;\\n    readonly DOM_DELTA_PIXEL: number;\\n}\\n\\ndeclare var WheelEvent: {\\n    prototype: WheelEvent;\\n    new(typeArg: string, eventInitDict?: WheelEventInit): WheelEvent;\\n    readonly DOM_DELTA_LINE: number;\\n    readonly DOM_DELTA_PAGE: number;\\n    readonly DOM_DELTA_PIXEL: number;\\n};\\n\\ninterface WindowEventMap extends GlobalEventHandlersEventMap, WindowEventHandlersEventMap {\\n    \"abort\": UIEvent;\\n    \"afterprint\": Event;\\n    \"beforeprint\": Event;\\n    \"beforeunload\": BeforeUnloadEvent;\\n    \"blur\": FocusEvent;\\n    \"canplay\": Event;\\n    \"canplaythrough\": Event;\\n    \"change\": Event;\\n    \"click\": MouseEvent;\\n    \"compassneedscalibration\": Event;\\n    \"contextmenu\": MouseEvent;\\n    \"dblclick\": MouseEvent;\\n    \"devicelight\": DeviceLightEvent;\\n    \"devicemotion\": DeviceMotionEvent;\\n    \"deviceorientation\": DeviceOrientationEvent;\\n    \"drag\": DragEvent;\\n    \"dragend\": DragEvent;\\n    \"dragenter\": DragEvent;\\n    \"dragleave\": DragEvent;\\n    \"dragover\": DragEvent;\\n    \"dragstart\": DragEvent;\\n    \"drop\": DragEvent;\\n    \"durationchange\": Event;\\n    \"emptied\": Event;\\n    \"ended\": Event;\\n    \"error\": ErrorEvent;\\n    \"focus\": FocusEvent;\\n    \"hashchange\": HashChangeEvent;\\n    \"input\": Event;\\n    \"invalid\": Event;\\n    \"keydown\": KeyboardEvent;\\n    \"keypress\": KeyboardEvent;\\n    \"keyup\": KeyboardEvent;\\n    \"load\": Event;\\n    \"loadeddata\": Event;\\n    \"loadedmetadata\": Event;\\n    \"loadstart\": Event;\\n    \"message\": MessageEvent;\\n    \"mousedown\": MouseEvent;\\n    \"mouseenter\": MouseEvent;\\n    \"mouseleave\": MouseEvent;\\n    \"mousemove\": MouseEvent;\\n    \"mouseout\": MouseEvent;\\n    \"mouseover\": MouseEvent;\\n    \"mouseup\": MouseEvent;\\n    \"mousewheel\": Event;\\n    \"MSGestureChange\": Event;\\n    \"MSGestureDoubleTap\": Event;\\n    \"MSGestureEnd\": Event;\\n    \"MSGestureHold\": Event;\\n    \"MSGestureStart\": Event;\\n    \"MSGestureTap\": Event;\\n    \"MSInertiaStart\": Event;\\n    \"MSPointerCancel\": Event;\\n    \"MSPointerDown\": Event;\\n    \"MSPointerEnter\": Event;\\n    \"MSPointerLeave\": Event;\\n    \"MSPointerMove\": Event;\\n    \"MSPointerOut\": Event;\\n    \"MSPointerOver\": Event;\\n    \"MSPointerUp\": Event;\\n    \"offline\": Event;\\n    \"online\": Event;\\n    \"orientationchange\": Event;\\n    \"pagehide\": PageTransitionEvent;\\n    \"pageshow\": PageTransitionEvent;\\n    \"pause\": Event;\\n    \"play\": Event;\\n    \"playing\": Event;\\n    \"popstate\": PopStateEvent;\\n    \"progress\": ProgressEvent;\\n    \"ratechange\": Event;\\n    \"readystatechange\": ProgressEvent;\\n    \"reset\": Event;\\n    \"resize\": UIEvent;\\n    \"scroll\": UIEvent;\\n    \"seeked\": Event;\\n    \"seeking\": Event;\\n    \"select\": UIEvent;\\n    \"stalled\": Event;\\n    \"storage\": StorageEvent;\\n    \"submit\": Event;\\n    \"suspend\": Event;\\n    \"timeupdate\": Event;\\n    \"unload\": Event;\\n    \"volumechange\": Event;\\n    \"vrdisplayactivate\": Event;\\n    \"vrdisplayblur\": Event;\\n    \"vrdisplayconnect\": Event;\\n    \"vrdisplaydeactivate\": Event;\\n    \"vrdisplaydisconnect\": Event;\\n    \"vrdisplayfocus\": Event;\\n    \"vrdisplaypointerrestricted\": Event;\\n    \"vrdisplaypointerunrestricted\": Event;\\n    \"vrdisplaypresentchange\": Event;\\n    \"waiting\": Event;\\n}\\n\\ninterface Window extends EventTarget, WindowTimers, WindowSessionStorage, WindowLocalStorage, WindowConsole, GlobalEventHandlers, IDBEnvironment, WindowBase64, GlobalFetch, WindowOrWorkerGlobalScope, WindowEventHandlers {\\n    Blob: typeof Blob;\\n    URL: typeof URL;\\n    URLSearchParams: typeof URLSearchParams;\\n    readonly applicationCache: ApplicationCache;\\n    readonly caches: CacheStorage;\\n    readonly clientInformation: Navigator;\\n    readonly closed: boolean;\\n    readonly crypto: Crypto;\\n    customElements: CustomElementRegistry;\\n    defaultStatus: string;\\n    readonly devicePixelRatio: number;\\n    readonly doNotTrack: string;\\n    readonly document: Document;\\n    readonly event: Event | undefined;\\n    /** @deprecated */\\n    readonly external: External;\\n    readonly frameElement: Element;\\n    readonly frames: Window;\\n    readonly history: History;\\n    readonly innerHeight: number;\\n    readonly innerWidth: number;\\n    readonly isSecureContext: boolean;\\n    readonly length: number;\\n    location: Location;\\n    readonly locationbar: BarProp;\\n    readonly menubar: BarProp;\\n    readonly msContentScript: ExtensionScriptApis;\\n    name: string;\\n    readonly navigator: Navigator;\\n    offscreenBuffering: string | boolean;\\n    oncompassneedscalibration: ((this: Window, ev: Event) => any) | null;\\n    ondevicelight: ((this: Window, ev: DeviceLightEvent) => any) | null;\\n    ondevicemotion: ((this: Window, ev: DeviceMotionEvent) => any) | null;\\n    ondeviceorientation: ((this: Window, ev: DeviceOrientationEvent) => any) | null;\\n    onmousewheel: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturechange: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturedoubletap: ((this: Window, ev: Event) => any) | null;\\n    onmsgestureend: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturehold: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturestart: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturetap: ((this: Window, ev: Event) => any) | null;\\n    onmsinertiastart: ((this: Window, ev: Event) => any) | null;\\n    onmspointercancel: ((this: Window, ev: Event) => any) | null;\\n    onmspointerdown: ((this: Window, ev: Event) => any) | null;\\n    onmspointerenter: ((this: Window, ev: Event) => any) | null;\\n    onmspointerleave: ((this: Window, ev: Event) => any) | null;\\n    onmspointermove: ((this: Window, ev: Event) => any) | null;\\n    onmspointerout: ((this: Window, ev: Event) => any) | null;\\n    onmspointerover: ((this: Window, ev: Event) => any) | null;\\n    onmspointerup: ((this: Window, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onorientationchange: ((this: Window, ev: Event) => any) | null;\\n    onreadystatechange: ((this: Window, ev: ProgressEvent) => any) | null;\\n    onvrdisplayactivate: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplayblur: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplayconnect: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaydeactivate: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaydisconnect: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplayfocus: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaypointerrestricted: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaypointerunrestricted: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaypresentchange: ((this: Window, ev: Event) => any) | null;\\n    opener: any;\\n    /** @deprecated */\\n    readonly orientation: string | number;\\n    readonly outerHeight: number;\\n    readonly outerWidth: number;\\n    readonly pageXOffset: number;\\n    readonly pageYOffset: number;\\n    readonly parent: Window;\\n    readonly performance: Performance;\\n    readonly personalbar: BarProp;\\n    readonly screen: Screen;\\n    readonly screenLeft: number;\\n    readonly screenTop: number;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly scrollX: number;\\n    readonly scrollY: number;\\n    readonly scrollbars: BarProp;\\n    readonly self: Window;\\n    readonly speechSynthesis: SpeechSynthesis;\\n    status: string;\\n    readonly statusbar: BarProp;\\n    readonly styleMedia: StyleMedia;\\n    readonly toolbar: BarProp;\\n    readonly top: Window;\\n    readonly window: Window;\\n    alert(message?: any): void;\\n    blur(): void;\\n    cancelAnimationFrame(handle: number): void;\\n    /** @deprecated */\\n    captureEvents(): void;\\n    close(): void;\\n    confirm(message?: string): boolean;\\n    departFocus(navigationReason: NavigationReason, origin: FocusNavigationOrigin): void;\\n    focus(): void;\\n    getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\\n    getMatchedCSSRules(elt: Element, pseudoElt?: string | null): CSSRuleList;\\n    getSelection(): Selection;\\n    matchMedia(query: string): MediaQueryList;\\n    moveBy(x: number, y: number): void;\\n    moveTo(x: number, y: number): void;\\n    msWriteProfilerMark(profilerMarkName: string): void;\\n    open(url?: string, target?: string, features?: string, replace?: boolean): Window | null;\\n    postMessage(message: any, targetOrigin: string, transfer?: Transferable[]): void;\\n    print(): void;\\n    prompt(message?: string, _default?: string): string | null;\\n    /** @deprecated */\\n    releaseEvents(): void;\\n    requestAnimationFrame(callback: FrameRequestCallback): number;\\n    resizeBy(x: number, y: number): void;\\n    resizeTo(x: number, y: number): void;\\n    scroll(options?: ScrollToOptions): void;\\n    scroll(x: number, y: number): void;\\n    scrollBy(options?: ScrollToOptions): void;\\n    scrollBy(x: number, y: number): void;\\n    scrollTo(options?: ScrollToOptions): void;\\n    scrollTo(x: number, y: number): void;\\n    stop(): void;\\n    webkitCancelAnimationFrame(handle: number): void;\\n    webkitConvertPointFromNodeToPage(node: Node, pt: WebKitPoint): WebKitPoint;\\n    webkitConvertPointFromPageToNode(node: Node, pt: WebKitPoint): WebKitPoint;\\n    webkitRequestAnimationFrame(callback: FrameRequestCallback): number;\\n    addEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Window: {\\n    prototype: Window;\\n    new(): Window;\\n};\\n\\ninterface WindowBase64 {\\n    atob(encodedString: string): string;\\n    btoa(rawString: string): string;\\n}\\n\\ninterface WindowConsole {\\n    readonly console: Console;\\n}\\n\\ninterface WindowEventHandlersEventMap {\\n    \"afterprint\": Event;\\n    \"beforeprint\": Event;\\n    \"beforeunload\": BeforeUnloadEvent;\\n    \"hashchange\": HashChangeEvent;\\n    \"languagechange\": Event;\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n    \"offline\": Event;\\n    \"online\": Event;\\n    \"pagehide\": PageTransitionEvent;\\n    \"pageshow\": PageTransitionEvent;\\n    \"popstate\": PopStateEvent;\\n    \"rejectionhandled\": Event;\\n    \"storage\": StorageEvent;\\n    \"unhandledrejection\": PromiseRejectionEvent;\\n    \"unload\": Event;\\n}\\n\\ninterface WindowEventHandlers {\\n    onafterprint: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onbeforeprint: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onbeforeunload: ((this: WindowEventHandlers, ev: BeforeUnloadEvent) => any) | null;\\n    onhashchange: ((this: WindowEventHandlers, ev: HashChangeEvent) => any) | null;\\n    onlanguagechange: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onmessage: ((this: WindowEventHandlers, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: WindowEventHandlers, ev: MessageEvent) => any) | null;\\n    onoffline: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    ononline: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onpagehide: ((this: WindowEventHandlers, ev: PageTransitionEvent) => any) | null;\\n    onpageshow: ((this: WindowEventHandlers, ev: PageTransitionEvent) => any) | null;\\n    onpopstate: ((this: WindowEventHandlers, ev: PopStateEvent) => any) | null;\\n    onrejectionhandled: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onstorage: ((this: WindowEventHandlers, ev: StorageEvent) => any) | null;\\n    onunhandledrejection: ((this: WindowEventHandlers, ev: PromiseRejectionEvent) => any) | null;\\n    onunload: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    addEventListener<K extends keyof WindowEventHandlersEventMap>(type: K, listener: (this: WindowEventHandlers, ev: WindowEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WindowEventHandlersEventMap>(type: K, listener: (this: WindowEventHandlers, ev: WindowEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface WindowLocalStorage {\\n    readonly localStorage: Storage;\\n}\\n\\ninterface WindowOrWorkerGlobalScope {\\n    readonly caches: CacheStorage;\\n    readonly crypto: Crypto;\\n    readonly indexedDB: IDBFactory;\\n    readonly origin: string;\\n    readonly performance: Performance;\\n    atob(data: string): string;\\n    btoa(data: string): string;\\n    clearInterval(handle?: number): void;\\n    clearTimeout(handle?: number): void;\\n    createImageBitmap(image: ImageBitmapSource): Promise<ImageBitmap>;\\n    createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number): Promise<ImageBitmap>;\\n    fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\n    queueMicrotask(callback: Function): void;\\n    setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\n    setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\n}\\n\\ninterface WindowSessionStorage {\\n    readonly sessionStorage: Storage;\\n}\\n\\ninterface WindowTimers {\\n}\\n\\ninterface WorkerEventMap extends AbstractWorkerEventMap {\\n    \"message\": MessageEvent;\\n}\\n\\ninterface Worker extends EventTarget, AbstractWorker {\\n    onmessage: ((this: Worker, ev: MessageEvent) => any) | null;\\n    postMessage(message: any, transfer?: Transferable[]): void;\\n    terminate(): void;\\n    addEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Worker: {\\n    prototype: Worker;\\n    new(stringUrl: string): Worker;\\n};\\n\\ninterface Worklet {\\n    addModule(moduleURL: string, options?: WorkletOptions): Promise<void>;\\n}\\n\\ndeclare var Worklet: {\\n    prototype: Worklet;\\n    new(): Worklet;\\n};\\n\\ninterface WritableStream {\\n    readonly locked: boolean;\\n    abort(reason?: any): Promise<void>;\\n    getWriter(): WritableStreamDefaultWriter;\\n}\\n\\ndeclare var WritableStream: {\\n    prototype: WritableStream;\\n    new(underlyingSink?: UnderlyingSink, strategy?: QueuingStrategy): WritableStream;\\n};\\n\\ninterface WritableStreamDefaultController {\\n    error(error?: any): void;\\n}\\n\\ndeclare var WritableStreamDefaultController: {\\n    prototype: WritableStreamDefaultController;\\n    new(): WritableStreamDefaultController;\\n};\\n\\ninterface WritableStreamDefaultWriter {\\n    readonly closed: Promise<void>;\\n    readonly desiredSize: number;\\n    readonly ready: Promise<void>;\\n    abort(reason?: any): Promise<void>;\\n    close(): Promise<void>;\\n    releaseLock(): void;\\n    write(chunk?: any): Promise<any>;\\n}\\n\\ndeclare var WritableStreamDefaultWriter: {\\n    prototype: WritableStreamDefaultWriter;\\n    new(): WritableStreamDefaultWriter;\\n};\\n\\ninterface XMLDocument extends Document {\\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: XMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: XMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLDocument: {\\n    prototype: XMLDocument;\\n    new(): XMLDocument;\\n};\\n\\ninterface XMLHttpRequestEventMap extends XMLHttpRequestEventTargetEventMap {\\n    \"readystatechange\": Event;\\n}\\n\\ninterface XMLHttpRequest extends XMLHttpRequestEventTarget {\\n    onreadystatechange: ((this: XMLHttpRequest, ev: Event) => any) | null;\\n    /**\\n     * Returns client\\'s state.\\n     */\\n    readonly readyState: number;\\n    /**\\n     * Returns the response\\'s body.\\n     */\\n    readonly response: any;\\n    /**\\n     * Returns the text response.\\n     * Throws an \"InvalidStateError\" DOMException if responseType is not the empty string or \"text\".\\n     */\\n    readonly responseText: string;\\n    /**\\n     * Returns the response type.\\n     * Can be set to change the response type. Values are:\\n     * the empty string (default),\\n     * \"arraybuffer\",\\n     * \"blob\",\\n     * \"document\",\\n     * \"json\", and\\n     * \"text\".\\n     * When set: setting to \"document\" is ignored if current global object is not a Window object.\\n     * When set: throws an \"InvalidStateError\" DOMException if state is loading or done.\\n     * When set: throws an \"InvalidAccessError\" DOMException if the synchronous flag is set and current global object is a Window object.\\n     */\\n    responseType: XMLHttpRequestResponseType;\\n    readonly responseURL: string;\\n    /**\\n     * Returns the document response.\\n     * Throws an \"InvalidStateError\" DOMException if responseType is not the empty string or \"document\".\\n     */\\n    readonly responseXML: Document | null;\\n    readonly status: number;\\n    readonly statusText: string;\\n    /**\\n     * Can be set to a time in milliseconds. When set to a non-zero value will cause fetching to terminate after the given time has passed. When the time has passed, the\\n     * request has not yet completed, and the synchronous flag is unset, a timeout event will then be dispatched, or a\\n     * \"TimeoutError\" DOMException will be thrown otherwise (for the send() method).\\n     * When set: throws an \"InvalidAccessError\" DOMException if the synchronous flag is set and current global object is a Window object.\\n     */\\n    timeout: number;\\n    /**\\n     * Returns the associated XMLHttpRequestUpload object. It can be used to gather transmission information when data is\\n     * transferred to a server.\\n     */\\n    readonly upload: XMLHttpRequestUpload;\\n    /**\\n     * True when credentials are to be included in a cross-origin request. False when they are\\n     * to be excluded in a cross-origin request and when cookies are to be ignored in its response.\\n     * Initially false.\\n     * When set: throws an \"InvalidStateError\" DOMException if state is not unsent or opened, or if the send() flag is set.\\n     */\\n    withCredentials: boolean;\\n    /**\\n     * Cancels any network activity.\\n     */\\n    abort(): void;\\n    getAllResponseHeaders(): string;\\n    getResponseHeader(name: string): string | null;\\n    /**\\n     * Sets the request method, request URL, and synchronous flag.\\n     * Throws a \"SyntaxError\" DOMException if either method is not a\\n     * valid HTTP method or url cannot be parsed.\\n     * Throws a \"SecurityError\" DOMException if method is a\\n     * case-insensitive match for `CONNECT`, `TRACE`, or `TRACK`.\\n     * Throws an \"InvalidAccessError\" DOMException if async is false, current global object is a Window object, and the timeout attribute is not zero or the responseType attribute is not the empty string.\\n     */\\n    open(method: string, url: string): void;\\n    open(method: string, url: string, async: boolean, username?: string | null, password?: string | null): void;\\n    /**\\n     * Acts as if the `Content-Type` header value for response is mime.\\n     * (It does not actually change the header though.)\\n     * Throws an \"InvalidStateError\" DOMException if state is loading or done.\\n     */\\n    overrideMimeType(mime: string): void;\\n    /**\\n     * Initiates the request. The optional argument provides the request body. The argument is ignored if request method is GET or HEAD.\\n     * Throws an \"InvalidStateError\" DOMException if either state is not opened or the send() flag is set.\\n     */\\n    send(body?: Document | BodyInit | null): void;\\n    /**\\n     * Combines a header in author request headers.\\n     * Throws an \"InvalidStateError\" DOMException if either state is not opened or the send() flag is set.\\n     * Throws a \"SyntaxError\" DOMException if name is not a header name\\n     * or if value is not a header value.\\n     */\\n    setRequestHeader(name: string, value: string): void;\\n    readonly DONE: number;\\n    readonly HEADERS_RECEIVED: number;\\n    readonly LOADING: number;\\n    readonly OPENED: number;\\n    readonly UNSENT: number;\\n    addEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLHttpRequest: {\\n    prototype: XMLHttpRequest;\\n    new(): XMLHttpRequest;\\n    readonly DONE: number;\\n    readonly HEADERS_RECEIVED: number;\\n    readonly LOADING: number;\\n    readonly OPENED: number;\\n    readonly UNSENT: number;\\n};\\n\\ninterface XMLHttpRequestEventTargetEventMap {\\n    \"abort\": ProgressEvent;\\n    \"error\": ProgressEvent;\\n    \"load\": ProgressEvent;\\n    \"loadend\": ProgressEvent;\\n    \"loadstart\": ProgressEvent;\\n    \"progress\": ProgressEvent;\\n    \"timeout\": ProgressEvent;\\n}\\n\\ninterface XMLHttpRequestEventTarget extends EventTarget {\\n    onabort: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onerror: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onload: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onloadend: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onloadstart: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onprogress: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    ontimeout: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLHttpRequestEventTarget: {\\n    prototype: XMLHttpRequestEventTarget;\\n    new(): XMLHttpRequestEventTarget;\\n};\\n\\ninterface XMLHttpRequestUpload extends XMLHttpRequestEventTarget {\\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLHttpRequestUpload: {\\n    prototype: XMLHttpRequestUpload;\\n    new(): XMLHttpRequestUpload;\\n};\\n\\ninterface XMLSerializer {\\n    serializeToString(root: Node): string;\\n}\\n\\ndeclare var XMLSerializer: {\\n    prototype: XMLSerializer;\\n    new(): XMLSerializer;\\n};\\n\\ninterface XPathEvaluator {\\n    createExpression(expression: string, resolver: XPathNSResolver): XPathExpression;\\n    createNSResolver(nodeResolver?: Node): XPathNSResolver;\\n    evaluate(expression: string, contextNode: Node, resolver: XPathNSResolver | null, type: number, result: XPathResult | null): XPathResult;\\n}\\n\\ndeclare var XPathEvaluator: {\\n    prototype: XPathEvaluator;\\n    new(): XPathEvaluator;\\n};\\n\\ninterface XPathExpression {\\n    evaluate(contextNode: Node, type: number, result: XPathResult | null): XPathResult;\\n}\\n\\ndeclare var XPathExpression: {\\n    prototype: XPathExpression;\\n    new(): XPathExpression;\\n};\\n\\ninterface XPathNSResolver {\\n    lookupNamespaceURI(prefix: string): string;\\n}\\n\\ndeclare var XPathNSResolver: {\\n    prototype: XPathNSResolver;\\n    new(): XPathNSResolver;\\n};\\n\\ninterface XPathResult {\\n    readonly booleanValue: boolean;\\n    readonly invalidIteratorState: boolean;\\n    readonly numberValue: number;\\n    readonly resultType: number;\\n    readonly singleNodeValue: Node;\\n    readonly snapshotLength: number;\\n    readonly stringValue: string;\\n    iterateNext(): Node;\\n    snapshotItem(index: number): Node;\\n    readonly ANY_TYPE: number;\\n    readonly ANY_UNORDERED_NODE_TYPE: number;\\n    readonly BOOLEAN_TYPE: number;\\n    readonly FIRST_ORDERED_NODE_TYPE: number;\\n    readonly NUMBER_TYPE: number;\\n    readonly ORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly ORDERED_NODE_SNAPSHOT_TYPE: number;\\n    readonly STRING_TYPE: number;\\n    readonly UNORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly UNORDERED_NODE_SNAPSHOT_TYPE: number;\\n}\\n\\ndeclare var XPathResult: {\\n    prototype: XPathResult;\\n    new(): XPathResult;\\n    readonly ANY_TYPE: number;\\n    readonly ANY_UNORDERED_NODE_TYPE: number;\\n    readonly BOOLEAN_TYPE: number;\\n    readonly FIRST_ORDERED_NODE_TYPE: number;\\n    readonly NUMBER_TYPE: number;\\n    readonly ORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly ORDERED_NODE_SNAPSHOT_TYPE: number;\\n    readonly STRING_TYPE: number;\\n    readonly UNORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly UNORDERED_NODE_SNAPSHOT_TYPE: number;\\n};\\n\\ninterface XSLTProcessor {\\n    clearParameters(): void;\\n    getParameter(namespaceURI: string, localName: string): any;\\n    importStylesheet(style: Node): void;\\n    removeParameter(namespaceURI: string, localName: string): void;\\n    reset(): void;\\n    setParameter(namespaceURI: string, localName: string, value: any): void;\\n    transformToDocument(source: Node): Document;\\n    transformToFragment(source: Node, document: Document): DocumentFragment;\\n}\\n\\ndeclare var XSLTProcessor: {\\n    prototype: XSLTProcessor;\\n    new(): XSLTProcessor;\\n};\\n\\ninterface webkitRTCPeerConnection extends RTCPeerConnection {\\n    addEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: webkitRTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: webkitRTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var webkitRTCPeerConnection: {\\n    prototype: webkitRTCPeerConnection;\\n    new(configuration: RTCConfiguration): webkitRTCPeerConnection;\\n};\\n\\ndeclare type EventListenerOrEventListenerObject = EventListener | EventListenerObject;\\n\\ninterface BlobCallback {\\n    (blob: Blob | null): void;\\n}\\n\\ninterface DecodeErrorCallback {\\n    (error: DOMException): void;\\n}\\n\\ninterface DecodeSuccessCallback {\\n    (decodedData: AudioBuffer): void;\\n}\\n\\ninterface ErrorEventHandler {\\n    (event: Event | string, source?: string, fileno?: number, columnNumber?: number, error?: Error): void;\\n}\\n\\ninterface EventHandlerNonNull {\\n    (event: Event): any;\\n}\\n\\ninterface ForEachCallback {\\n    (keyId: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null, status: MediaKeyStatus): void;\\n}\\n\\ninterface FrameRequestCallback {\\n    (time: number): void;\\n}\\n\\ninterface FunctionStringCallback {\\n    (data: string): void;\\n}\\n\\ninterface IntersectionObserverCallback {\\n    (entries: IntersectionObserverEntry[], observer: IntersectionObserver): void;\\n}\\n\\ninterface MSLaunchUriCallback {\\n    (): void;\\n}\\n\\ninterface MutationCallback {\\n    (mutations: MutationRecord[], observer: MutationObserver): void;\\n}\\n\\ninterface NavigatorUserMediaErrorCallback {\\n    (error: MediaStreamError): void;\\n}\\n\\ninterface NavigatorUserMediaSuccessCallback {\\n    (stream: MediaStream): void;\\n}\\n\\ninterface NotificationPermissionCallback {\\n    (permission: NotificationPermission): void;\\n}\\n\\ninterface OnBeforeUnloadEventHandlerNonNull {\\n    (event: Event): string | null;\\n}\\n\\ninterface OnErrorEventHandlerNonNull {\\n    (event: Event | string, source?: string, lineno?: number, colno?: number, error?: any): any;\\n}\\n\\ninterface PerformanceObserverCallback {\\n    (entries: PerformanceObserverEntryList, observer: PerformanceObserver): void;\\n}\\n\\ninterface PositionCallback {\\n    (position: Position): void;\\n}\\n\\ninterface PositionErrorCallback {\\n    (positionError: PositionError): void;\\n}\\n\\ninterface RTCPeerConnectionErrorCallback {\\n    (error: DOMException): void;\\n}\\n\\ninterface RTCSessionDescriptionCallback {\\n    (description: RTCSessionDescriptionInit): void;\\n}\\n\\ninterface RTCStatsCallback {\\n    (report: RTCStatsReport): void;\\n}\\n\\ninterface VoidFunction {\\n    (): void;\\n}\\n\\ninterface WritableStreamChunkCallback {\\n    (chunk: any, controller: WritableStreamDefaultController): void;\\n}\\n\\ninterface WritableStreamDefaultControllerCallback {\\n    (controller: WritableStreamDefaultController): void;\\n}\\n\\ninterface WritableStreamErrorCallback {\\n    (reason: string): void;\\n}\\n\\ninterface HTMLElementTagNameMap {\\n    \"a\": HTMLAnchorElement;\\n    \"abbr\": HTMLElement;\\n    \"address\": HTMLElement;\\n    \"applet\": HTMLAppletElement;\\n    \"area\": HTMLAreaElement;\\n    \"article\": HTMLElement;\\n    \"aside\": HTMLElement;\\n    \"audio\": HTMLAudioElement;\\n    \"b\": HTMLElement;\\n    \"base\": HTMLBaseElement;\\n    \"basefont\": HTMLBaseFontElement;\\n    \"bdo\": HTMLElement;\\n    \"blockquote\": HTMLQuoteElement;\\n    \"body\": HTMLBodyElement;\\n    \"br\": HTMLBRElement;\\n    \"button\": HTMLButtonElement;\\n    \"canvas\": HTMLCanvasElement;\\n    \"caption\": HTMLTableCaptionElement;\\n    \"cite\": HTMLElement;\\n    \"code\": HTMLElement;\\n    \"col\": HTMLTableColElement;\\n    \"colgroup\": HTMLTableColElement;\\n    \"data\": HTMLDataElement;\\n    \"datalist\": HTMLDataListElement;\\n    \"dd\": HTMLElement;\\n    \"del\": HTMLModElement;\\n    \"details\": HTMLDetailsElement;\\n    \"dfn\": HTMLElement;\\n    \"dialog\": HTMLDialogElement;\\n    \"dir\": HTMLDirectoryElement;\\n    \"div\": HTMLDivElement;\\n    \"dl\": HTMLDListElement;\\n    \"dt\": HTMLElement;\\n    \"em\": HTMLElement;\\n    \"embed\": HTMLEmbedElement;\\n    \"fieldset\": HTMLFieldSetElement;\\n    \"figcaption\": HTMLElement;\\n    \"figure\": HTMLElement;\\n    \"font\": HTMLFontElement;\\n    \"footer\": HTMLElement;\\n    \"form\": HTMLFormElement;\\n    \"frame\": HTMLFrameElement;\\n    \"frameset\": HTMLFrameSetElement;\\n    \"h1\": HTMLHeadingElement;\\n    \"h2\": HTMLHeadingElement;\\n    \"h3\": HTMLHeadingElement;\\n    \"h4\": HTMLHeadingElement;\\n    \"h5\": HTMLHeadingElement;\\n    \"h6\": HTMLHeadingElement;\\n    \"head\": HTMLHeadElement;\\n    \"header\": HTMLElement;\\n    \"hgroup\": HTMLElement;\\n    \"hr\": HTMLHRElement;\\n    \"html\": HTMLHtmlElement;\\n    \"i\": HTMLElement;\\n    \"iframe\": HTMLIFrameElement;\\n    \"img\": HTMLImageElement;\\n    \"input\": HTMLInputElement;\\n    \"ins\": HTMLModElement;\\n    \"kbd\": HTMLElement;\\n    \"label\": HTMLLabelElement;\\n    \"legend\": HTMLLegendElement;\\n    \"li\": HTMLLIElement;\\n    \"link\": HTMLLinkElement;\\n    \"map\": HTMLMapElement;\\n    \"mark\": HTMLElement;\\n    \"marquee\": HTMLMarqueeElement;\\n    \"menu\": HTMLMenuElement;\\n    \"meta\": HTMLMetaElement;\\n    \"meter\": HTMLMeterElement;\\n    \"nav\": HTMLElement;\\n    \"noscript\": HTMLElement;\\n    \"object\": HTMLObjectElement;\\n    \"ol\": HTMLOListElement;\\n    \"optgroup\": HTMLOptGroupElement;\\n    \"option\": HTMLOptionElement;\\n    \"output\": HTMLOutputElement;\\n    \"p\": HTMLParagraphElement;\\n    \"param\": HTMLParamElement;\\n    \"picture\": HTMLPictureElement;\\n    \"pre\": HTMLPreElement;\\n    \"progress\": HTMLProgressElement;\\n    \"q\": HTMLQuoteElement;\\n    \"rt\": HTMLElement;\\n    \"ruby\": HTMLElement;\\n    \"s\": HTMLElement;\\n    \"samp\": HTMLElement;\\n    \"script\": HTMLScriptElement;\\n    \"section\": HTMLElement;\\n    \"select\": HTMLSelectElement;\\n    \"slot\": HTMLSlotElement;\\n    \"small\": HTMLElement;\\n    \"source\": HTMLSourceElement;\\n    \"span\": HTMLSpanElement;\\n    \"strong\": HTMLElement;\\n    \"style\": HTMLStyleElement;\\n    \"sub\": HTMLElement;\\n    \"sup\": HTMLElement;\\n    \"table\": HTMLTableElement;\\n    \"tbody\": HTMLTableSectionElement;\\n    \"td\": HTMLTableDataCellElement;\\n    \"template\": HTMLTemplateElement;\\n    \"textarea\": HTMLTextAreaElement;\\n    \"tfoot\": HTMLTableSectionElement;\\n    \"th\": HTMLTableHeaderCellElement;\\n    \"thead\": HTMLTableSectionElement;\\n    \"time\": HTMLTimeElement;\\n    \"title\": HTMLTitleElement;\\n    \"tr\": HTMLTableRowElement;\\n    \"track\": HTMLTrackElement;\\n    \"u\": HTMLElement;\\n    \"ul\": HTMLUListElement;\\n    \"var\": HTMLElement;\\n    \"video\": HTMLVideoElement;\\n    \"wbr\": HTMLElement;\\n}\\n\\ninterface HTMLElementDeprecatedTagNameMap {\\n    \"listing\": HTMLPreElement;\\n    \"xmp\": HTMLPreElement;\\n}\\n\\ninterface SVGElementTagNameMap {\\n    \"circle\": SVGCircleElement;\\n    \"clipPath\": SVGClipPathElement;\\n    \"defs\": SVGDefsElement;\\n    \"desc\": SVGDescElement;\\n    \"ellipse\": SVGEllipseElement;\\n    \"feBlend\": SVGFEBlendElement;\\n    \"feColorMatrix\": SVGFEColorMatrixElement;\\n    \"feComponentTransfer\": SVGFEComponentTransferElement;\\n    \"feComposite\": SVGFECompositeElement;\\n    \"feConvolveMatrix\": SVGFEConvolveMatrixElement;\\n    \"feDiffuseLighting\": SVGFEDiffuseLightingElement;\\n    \"feDisplacementMap\": SVGFEDisplacementMapElement;\\n    \"feDistantLight\": SVGFEDistantLightElement;\\n    \"feFlood\": SVGFEFloodElement;\\n    \"feFuncA\": SVGFEFuncAElement;\\n    \"feFuncB\": SVGFEFuncBElement;\\n    \"feFuncG\": SVGFEFuncGElement;\\n    \"feFuncR\": SVGFEFuncRElement;\\n    \"feGaussianBlur\": SVGFEGaussianBlurElement;\\n    \"feImage\": SVGFEImageElement;\\n    \"feMerge\": SVGFEMergeElement;\\n    \"feMergeNode\": SVGFEMergeNodeElement;\\n    \"feMorphology\": SVGFEMorphologyElement;\\n    \"feOffset\": SVGFEOffsetElement;\\n    \"fePointLight\": SVGFEPointLightElement;\\n    \"feSpecularLighting\": SVGFESpecularLightingElement;\\n    \"feSpotLight\": SVGFESpotLightElement;\\n    \"feTile\": SVGFETileElement;\\n    \"feTurbulence\": SVGFETurbulenceElement;\\n    \"filter\": SVGFilterElement;\\n    \"foreignObject\": SVGForeignObjectElement;\\n    \"g\": SVGGElement;\\n    \"image\": SVGImageElement;\\n    \"line\": SVGLineElement;\\n    \"linearGradient\": SVGLinearGradientElement;\\n    \"marker\": SVGMarkerElement;\\n    \"mask\": SVGMaskElement;\\n    \"metadata\": SVGMetadataElement;\\n    \"path\": SVGPathElement;\\n    \"pattern\": SVGPatternElement;\\n    \"polygon\": SVGPolygonElement;\\n    \"polyline\": SVGPolylineElement;\\n    \"radialGradient\": SVGRadialGradientElement;\\n    \"rect\": SVGRectElement;\\n    \"stop\": SVGStopElement;\\n    \"svg\": SVGSVGElement;\\n    \"switch\": SVGSwitchElement;\\n    \"symbol\": SVGSymbolElement;\\n    \"text\": SVGTextElement;\\n    \"textPath\": SVGTextPathElement;\\n    \"tspan\": SVGTSpanElement;\\n    \"use\": SVGUseElement;\\n    \"view\": SVGViewElement;\\n}\\n\\n/** @deprecated Directly use HTMLElementTagNameMap or SVGElementTagNameMap as appropriate, instead. */\\ninterface ElementTagNameMap extends HTMLElementTagNameMap, SVGElementTagNameMap { }\\n\\ndeclare var Audio: {\\n    new(src?: string): HTMLAudioElement;\\n};\\ndeclare var Image: {\\n    new(width?: number, height?: number): HTMLImageElement;\\n};\\ndeclare var Option: {\\n    new(text?: string, value?: string, defaultSelected?: boolean, selected?: boolean): HTMLOptionElement;\\n};\\ndeclare var Blob: typeof Blob;\\ndeclare var URL: typeof URL;\\ndeclare var URLSearchParams: typeof URLSearchParams;\\ndeclare var applicationCache: ApplicationCache;\\ndeclare var caches: CacheStorage;\\ndeclare var clientInformation: Navigator;\\ndeclare var closed: boolean;\\ndeclare var crypto: Crypto;\\ndeclare var customElements: CustomElementRegistry;\\ndeclare var defaultStatus: string;\\ndeclare var devicePixelRatio: number;\\ndeclare var doNotTrack: string;\\ndeclare var document: Document;\\ndeclare var event: Event | undefined;\\n/** @deprecated */\\ndeclare var external: External;\\ndeclare var frameElement: Element;\\ndeclare var frames: Window;\\ndeclare var history: History;\\ndeclare var innerHeight: number;\\ndeclare var innerWidth: number;\\ndeclare var isSecureContext: boolean;\\ndeclare var length: number;\\ndeclare var location: Location;\\ndeclare var locationbar: BarProp;\\ndeclare var menubar: BarProp;\\ndeclare var msContentScript: ExtensionScriptApis;\\ndeclare const name: never;\\ndeclare var navigator: Navigator;\\ndeclare var offscreenBuffering: string | boolean;\\ndeclare var oncompassneedscalibration: ((this: Window, ev: Event) => any) | null;\\ndeclare var ondevicelight: ((this: Window, ev: DeviceLightEvent) => any) | null;\\ndeclare var ondevicemotion: ((this: Window, ev: DeviceMotionEvent) => any) | null;\\ndeclare var ondeviceorientation: ((this: Window, ev: DeviceOrientationEvent) => any) | null;\\ndeclare var onmousewheel: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturechange: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturedoubletap: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgestureend: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturehold: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturestart: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturetap: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsinertiastart: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointercancel: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerdown: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerenter: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerleave: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointermove: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerout: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerover: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerup: ((this: Window, ev: Event) => any) | null;\\n/** @deprecated */\\ndeclare var onorientationchange: ((this: Window, ev: Event) => any) | null;\\ndeclare var onreadystatechange: ((this: Window, ev: ProgressEvent) => any) | null;\\ndeclare var onvrdisplayactivate: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplayblur: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplayconnect: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaydeactivate: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaydisconnect: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplayfocus: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaypointerrestricted: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaypointerunrestricted: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaypresentchange: ((this: Window, ev: Event) => any) | null;\\ndeclare var opener: any;\\n/** @deprecated */\\ndeclare var orientation: string | number;\\ndeclare var outerHeight: number;\\ndeclare var outerWidth: number;\\ndeclare var pageXOffset: number;\\ndeclare var pageYOffset: number;\\ndeclare var parent: Window;\\ndeclare var performance: Performance;\\ndeclare var personalbar: BarProp;\\ndeclare var screen: Screen;\\ndeclare var screenLeft: number;\\ndeclare var screenTop: number;\\ndeclare var screenX: number;\\ndeclare var screenY: number;\\ndeclare var scrollX: number;\\ndeclare var scrollY: number;\\ndeclare var scrollbars: BarProp;\\ndeclare var self: Window;\\ndeclare var speechSynthesis: SpeechSynthesis;\\ndeclare var status: string;\\ndeclare var statusbar: BarProp;\\ndeclare var styleMedia: StyleMedia;\\ndeclare var toolbar: BarProp;\\ndeclare var top: Window;\\ndeclare var window: Window;\\ndeclare function alert(message?: any): void;\\ndeclare function blur(): void;\\ndeclare function cancelAnimationFrame(handle: number): void;\\n/** @deprecated */\\ndeclare function captureEvents(): void;\\ndeclare function close(): void;\\ndeclare function confirm(message?: string): boolean;\\ndeclare function departFocus(navigationReason: NavigationReason, origin: FocusNavigationOrigin): void;\\ndeclare function focus(): void;\\ndeclare function getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\\ndeclare function getMatchedCSSRules(elt: Element, pseudoElt?: string | null): CSSRuleList;\\ndeclare function getSelection(): Selection;\\ndeclare function matchMedia(query: string): MediaQueryList;\\ndeclare function moveBy(x: number, y: number): void;\\ndeclare function moveTo(x: number, y: number): void;\\ndeclare function msWriteProfilerMark(profilerMarkName: string): void;\\ndeclare function open(url?: string, target?: string, features?: string, replace?: boolean): Window | null;\\ndeclare function postMessage(message: any, targetOrigin: string, transfer?: Transferable[]): void;\\ndeclare function print(): void;\\ndeclare function prompt(message?: string, _default?: string): string | null;\\n/** @deprecated */\\ndeclare function releaseEvents(): void;\\ndeclare function requestAnimationFrame(callback: FrameRequestCallback): number;\\ndeclare function resizeBy(x: number, y: number): void;\\ndeclare function resizeTo(x: number, y: number): void;\\ndeclare function scroll(options?: ScrollToOptions): void;\\ndeclare function scroll(x: number, y: number): void;\\ndeclare function scrollBy(options?: ScrollToOptions): void;\\ndeclare function scrollBy(x: number, y: number): void;\\ndeclare function scrollTo(options?: ScrollToOptions): void;\\ndeclare function scrollTo(x: number, y: number): void;\\ndeclare function stop(): void;\\ndeclare function webkitCancelAnimationFrame(handle: number): void;\\ndeclare function webkitConvertPointFromNodeToPage(node: Node, pt: WebKitPoint): WebKitPoint;\\ndeclare function webkitConvertPointFromPageToNode(node: Node, pt: WebKitPoint): WebKitPoint;\\ndeclare function webkitRequestAnimationFrame(callback: FrameRequestCallback): number;\\ndeclare function toString(): string;\\n/**\\n * Dispatches a synthetic event event to target and returns true\\n * if either event\\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\\n */\\ndeclare function dispatchEvent(event: Event): boolean;\\ndeclare var sessionStorage: Storage;\\ndeclare var localStorage: Storage;\\ndeclare var console: Console;\\n/**\\n * Fires when the user aborts the download.\\n * @param ev The event.\\n */\\ndeclare var onabort: ((this: Window, ev: UIEvent) => any) | null;\\ndeclare var onanimationcancel: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onanimationend: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onanimationiteration: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onanimationstart: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onauxclick: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the object loses the input focus.\\n * @param ev The focus event.\\n */\\ndeclare var onblur: ((this: Window, ev: FocusEvent) => any) | null;\\ndeclare var oncancel: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when playback is possible, but would require further buffering.\\n * @param ev The event.\\n */\\ndeclare var oncanplay: ((this: Window, ev: Event) => any) | null;\\ndeclare var oncanplaythrough: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the contents of the object or selection have changed.\\n * @param ev The event.\\n */\\ndeclare var onchange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user clicks the left mouse button on the object\\n * @param ev The mouse event.\\n */\\ndeclare var onclick: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var onclose: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user clicks the right mouse button in the client area, opening the context menu.\\n * @param ev The mouse event.\\n */\\ndeclare var oncontextmenu: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var oncuechange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user double-clicks the object.\\n * @param ev The mouse event.\\n */\\ndeclare var ondblclick: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires on the source object continuously during a drag operation.\\n * @param ev The event.\\n */\\ndeclare var ondrag: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the source object when the user releases the mouse at the close of a drag operation.\\n * @param ev The event.\\n */\\ndeclare var ondragend: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the target element when the user drags the object to a valid drop target.\\n * @param ev The drag event.\\n */\\ndeclare var ondragenter: ((this: Window, ev: DragEvent) => any) | null;\\ndeclare var ondragexit: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires on the target object when the user moves the mouse out of a valid drop target during a drag operation.\\n * @param ev The drag event.\\n */\\ndeclare var ondragleave: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the target element continuously while the user drags the object over a valid drop target.\\n * @param ev The event.\\n */\\ndeclare var ondragover: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the source object when the user starts to drag a text selection or selected object.\\n * @param ev The event.\\n */\\ndeclare var ondragstart: ((this: Window, ev: DragEvent) => any) | null;\\ndeclare var ondrop: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Occurs when the duration attribute is updated.\\n * @param ev The event.\\n */\\ndeclare var ondurationchange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the media element is reset to its initial state.\\n * @param ev The event.\\n */\\ndeclare var onemptied: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the end of playback is reached.\\n * @param ev The event\\n */\\ndeclare var onended: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when an error occurs during object loading.\\n * @param ev The event.\\n */\\ndeclare var onerror: ErrorEventHandler;\\n/**\\n * Fires when the object receives focus.\\n * @param ev The event.\\n */\\ndeclare var onfocus: ((this: Window, ev: FocusEvent) => any) | null;\\ndeclare var ongotpointercapture: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var oninput: ((this: Window, ev: Event) => any) | null;\\ndeclare var oninvalid: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user presses a key.\\n * @param ev The keyboard event\\n */\\ndeclare var onkeydown: ((this: Window, ev: KeyboardEvent) => any) | null;\\n/**\\n * Fires when the user presses an alphanumeric key.\\n * @param ev The event.\\n */\\ndeclare var onkeypress: ((this: Window, ev: KeyboardEvent) => any) | null;\\n/**\\n * Fires when the user releases a key.\\n * @param ev The keyboard event\\n */\\ndeclare var onkeyup: ((this: Window, ev: KeyboardEvent) => any) | null;\\n/**\\n * Fires immediately after the browser loads the object.\\n * @param ev The event.\\n */\\ndeclare var onload: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when media data is loaded at the current playback position.\\n * @param ev The event.\\n */\\ndeclare var onloadeddata: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the duration and dimensions of the media have been determined.\\n * @param ev The event.\\n */\\ndeclare var onloadedmetadata: ((this: Window, ev: Event) => any) | null;\\ndeclare var onloadend: ((this: Window, ev: ProgressEvent) => any) | null;\\n/**\\n * Occurs when Internet Explorer begins looking for media data.\\n * @param ev The event.\\n */\\ndeclare var onloadstart: ((this: Window, ev: Event) => any) | null;\\ndeclare var onlostpointercapture: ((this: Window, ev: PointerEvent) => any) | null;\\n/**\\n * Fires when the user clicks the object with either mouse button.\\n * @param ev The mouse event.\\n */\\ndeclare var onmousedown: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var onmouseenter: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var onmouseleave: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user moves the mouse over the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmousemove: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user moves the mouse pointer outside the boundaries of the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmouseout: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user moves the mouse pointer into the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmouseover: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user releases a mouse button while the mouse is over the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmouseup: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Occurs when playback is paused.\\n * @param ev The event.\\n */\\ndeclare var onpause: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the play method is requested.\\n * @param ev The event.\\n */\\ndeclare var onplay: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the audio or video has started playing.\\n * @param ev The event.\\n */\\ndeclare var onplaying: ((this: Window, ev: Event) => any) | null;\\ndeclare var onpointercancel: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerdown: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerenter: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerleave: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointermove: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerout: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerover: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerup: ((this: Window, ev: PointerEvent) => any) | null;\\n/**\\n * Occurs to indicate progress while downloading media data.\\n * @param ev The event.\\n */\\ndeclare var onprogress: ((this: Window, ev: ProgressEvent) => any) | null;\\n/**\\n * Occurs when the playback rate is increased or decreased.\\n * @param ev The event.\\n */\\ndeclare var onratechange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user resets a form.\\n * @param ev The event.\\n */\\ndeclare var onreset: ((this: Window, ev: Event) => any) | null;\\ndeclare var onresize: ((this: Window, ev: UIEvent) => any) | null;\\n/**\\n * Fires when the user repositions the scroll box in the scroll bar on the object.\\n * @param ev The event.\\n */\\ndeclare var onscroll: ((this: Window, ev: UIEvent) => any) | null;\\ndeclare var onsecuritypolicyviolation: ((this: Window, ev: SecurityPolicyViolationEvent) => any) | null;\\n/**\\n * Occurs when the seek operation ends.\\n * @param ev The event.\\n */\\ndeclare var onseeked: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the current playback position is moved.\\n * @param ev The event.\\n */\\ndeclare var onseeking: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the current selection changes.\\n * @param ev The event.\\n */\\ndeclare var onselect: ((this: Window, ev: UIEvent) => any) | null;\\n/**\\n * Occurs when the download has stopped.\\n * @param ev The event.\\n */\\ndeclare var onstalled: ((this: Window, ev: Event) => any) | null;\\ndeclare var onsubmit: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs if the load operation has been intentionally halted.\\n * @param ev The event.\\n */\\ndeclare var onsuspend: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs to indicate the current playback position.\\n * @param ev The event.\\n */\\ndeclare var ontimeupdate: ((this: Window, ev: Event) => any) | null;\\ndeclare var ontoggle: ((this: Window, ev: Event) => any) | null;\\ndeclare var ontouchcancel: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontouchend: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontouchmove: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontouchstart: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontransitioncancel: ((this: Window, ev: TransitionEvent) => any) | null;\\ndeclare var ontransitionend: ((this: Window, ev: TransitionEvent) => any) | null;\\ndeclare var ontransitionrun: ((this: Window, ev: TransitionEvent) => any) | null;\\ndeclare var ontransitionstart: ((this: Window, ev: TransitionEvent) => any) | null;\\n/**\\n * Occurs when the volume is changed, or playback is muted or unmuted.\\n * @param ev The event.\\n */\\ndeclare var onvolumechange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when playback stops because the next frame of a video resource is not available.\\n * @param ev The event.\\n */\\ndeclare var onwaiting: ((this: Window, ev: Event) => any) | null;\\ndeclare var onwheel: ((this: Window, ev: WheelEvent) => any) | null;\\ndeclare var indexedDB: IDBFactory;\\ndeclare function atob(encodedString: string): string;\\ndeclare function btoa(rawString: string): string;\\ndeclare function fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\ndeclare var caches: CacheStorage;\\ndeclare var crypto: Crypto;\\ndeclare var indexedDB: IDBFactory;\\ndeclare var origin: string;\\ndeclare var performance: Performance;\\ndeclare function atob(data: string): string;\\ndeclare function btoa(data: string): string;\\ndeclare function clearInterval(handle?: number): void;\\ndeclare function clearTimeout(handle?: number): void;\\ndeclare function createImageBitmap(image: ImageBitmapSource): Promise<ImageBitmap>;\\ndeclare function createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number): Promise<ImageBitmap>;\\ndeclare function fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\ndeclare function queueMicrotask(callback: Function): void;\\ndeclare function setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\ndeclare function setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\ndeclare var sessionStorage: Storage;\\ndeclare var localStorage: Storage;\\ndeclare var onafterprint: ((this: Window, ev: Event) => any) | null;\\ndeclare var onbeforeprint: ((this: Window, ev: Event) => any) | null;\\ndeclare var onbeforeunload: ((this: Window, ev: BeforeUnloadEvent) => any) | null;\\ndeclare var onhashchange: ((this: Window, ev: HashChangeEvent) => any) | null;\\ndeclare var onlanguagechange: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmessage: ((this: Window, ev: MessageEvent) => any) | null;\\ndeclare var onmessageerror: ((this: Window, ev: MessageEvent) => any) | null;\\ndeclare var onoffline: ((this: Window, ev: Event) => any) | null;\\ndeclare var ononline: ((this: Window, ev: Event) => any) | null;\\ndeclare var onpagehide: ((this: Window, ev: PageTransitionEvent) => any) | null;\\ndeclare var onpageshow: ((this: Window, ev: PageTransitionEvent) => any) | null;\\ndeclare var onpopstate: ((this: Window, ev: PopStateEvent) => any) | null;\\ndeclare var onrejectionhandled: ((this: Window, ev: Event) => any) | null;\\ndeclare var onstorage: ((this: Window, ev: StorageEvent) => any) | null;\\ndeclare var onunhandledrejection: ((this: Window, ev: PromiseRejectionEvent) => any) | null;\\ndeclare var onunload: ((this: Window, ev: Event) => any) | null;\\ndeclare function addEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\ndeclare function addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\ndeclare function removeEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\ndeclare function removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\ntype BlobPart = BufferSource | Blob | string;\\ntype HeadersInit = Headers | string[][] | Record<string, string>;\\ntype BodyInit = Blob | BufferSource | FormData | URLSearchParams | ReadableStream | string;\\ntype RequestInfo = Request | string;\\ntype DOMHighResTimeStamp = number;\\ntype RenderingContext = CanvasRenderingContext2D | ImageBitmapRenderingContext | WebGLRenderingContext;\\ntype HTMLOrSVGImageElement = HTMLImageElement | SVGImageElement;\\ntype CanvasImageSource = HTMLOrSVGImageElement | HTMLVideoElement | HTMLCanvasElement | ImageBitmap;\\ntype MessageEventSource = WindowProxy | MessagePort | ServiceWorker;\\ntype HTMLOrSVGScriptElement = HTMLScriptElement | SVGScriptElement;\\ntype ImageBitmapSource = CanvasImageSource | Blob | ImageData;\\ntype OnErrorEventHandler = OnErrorEventHandlerNonNull | null;\\ntype OnBeforeUnloadEventHandler = OnBeforeUnloadEventHandlerNonNull | null;\\ntype TimerHandler = string | Function;\\ntype PerformanceEntryList = PerformanceEntry[];\\ntype VibratePattern = number | number[];\\ntype AlgorithmIdentifier = string | Algorithm;\\ntype HashAlgorithmIdentifier = AlgorithmIdentifier;\\ntype BigInteger = Uint8Array;\\ntype NamedCurve = string;\\ntype GLenum = number;\\ntype GLboolean = boolean;\\ntype GLbitfield = number;\\ntype GLint = number;\\ntype GLsizei = number;\\ntype GLintptr = number;\\ntype GLsizeiptr = number;\\ntype GLuint = number;\\ntype GLfloat = number;\\ntype GLclampf = number;\\ntype TexImageSource = ImageBitmap | ImageData | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement;\\ntype Float32List = Float32Array | GLfloat[];\\ntype Int32List = Int32Array | GLint[];\\ntype BufferSource = ArrayBufferView | ArrayBuffer;\\ntype DOMTimeStamp = number;\\ntype LineAndPositionSetting = number | AutoKeyword;\\ntype FormDataEntryValue = File | string;\\ntype InsertPosition = \"beforebegin\" | \"afterbegin\" | \"beforeend\" | \"afterend\";\\ntype IDBValidKey = number | string | Date | BufferSource | IDBArrayKey;\\ntype MutationRecordType = \"attributes\" | \"characterData\" | \"childList\";\\ntype ConstrainBoolean = boolean | ConstrainBooleanParameters;\\ntype ConstrainDOMString = string | string[] | ConstrainDOMStringParameters;\\ntype ConstrainDouble = number | ConstrainDoubleRange;\\ntype ConstrainLong = number | ConstrainLongRange;\\ntype IDBKeyPath = string;\\ntype Transferable = ArrayBuffer | MessagePort | ImageBitmap;\\ntype RTCIceGatherCandidate = RTCIceCandidateDictionary | RTCIceCandidateComplete;\\ntype RTCTransport = RTCDtlsTransport | RTCSrtpSdesTransport;\\ntype MouseWheelEvent = WheelEvent;\\ntype WindowProxy = Window;\\ntype AlignSetting = \"start\" | \"center\" | \"end\" | \"left\" | \"right\";\\ntype AnimationPlayState = \"idle\" | \"running\" | \"paused\" | \"finished\";\\ntype AppendMode = \"segments\" | \"sequence\";\\ntype AudioContextLatencyCategory = \"balanced\" | \"interactive\" | \"playback\";\\ntype AudioContextState = \"suspended\" | \"running\" | \"closed\";\\ntype AutoKeyword = \"auto\";\\ntype AutomationRate = \"a-rate\" | \"k-rate\";\\ntype BinaryType = \"blob\" | \"arraybuffer\";\\ntype BiquadFilterType = \"lowpass\" | \"highpass\" | \"bandpass\" | \"lowshelf\" | \"highshelf\" | \"peaking\" | \"notch\" | \"allpass\";\\ntype CanPlayTypeResult = \"\" | \"maybe\" | \"probably\";\\ntype CanvasDirection = \"ltr\" | \"rtl\" | \"inherit\";\\ntype CanvasFillRule = \"nonzero\" | \"evenodd\";\\ntype CanvasLineCap = \"butt\" | \"round\" | \"square\";\\ntype CanvasLineJoin = \"round\" | \"bevel\" | \"miter\";\\ntype CanvasTextAlign = \"start\" | \"end\" | \"left\" | \"right\" | \"center\";\\ntype CanvasTextBaseline = \"top\" | \"hanging\" | \"middle\" | \"alphabetic\" | \"ideographic\" | \"bottom\";\\ntype ChannelCountMode = \"max\" | \"clamped-max\" | \"explicit\";\\ntype ChannelInterpretation = \"speakers\" | \"discrete\";\\ntype ClientTypes = \"window\" | \"worker\" | \"sharedworker\" | \"all\";\\ntype CompositeOperation = \"replace\" | \"add\" | \"accumulate\";\\ntype DirectionSetting = \"\" | \"rl\" | \"lr\";\\ntype DisplayCaptureSurfaceType = \"monitor\" | \"window\" | \"application\" | \"browser\";\\ntype DistanceModelType = \"linear\" | \"inverse\" | \"exponential\";\\ntype DocumentReadyState = \"loading\" | \"interactive\" | \"complete\";\\ntype EndOfStreamError = \"network\" | \"decode\";\\ntype FillMode = \"none\" | \"forwards\" | \"backwards\" | \"both\" | \"auto\";\\ntype GamepadHand = \"\" | \"left\" | \"right\";\\ntype GamepadHapticActuatorType = \"vibration\";\\ntype GamepadInputEmulationType = \"mouse\" | \"keyboard\" | \"gamepad\";\\ntype GamepadMappingType = \"\" | \"standard\";\\ntype IDBCursorDirection = \"next\" | \"nextunique\" | \"prev\" | \"prevunique\";\\ntype IDBRequestReadyState = \"pending\" | \"done\";\\ntype IDBTransactionMode = \"readonly\" | \"readwrite\" | \"versionchange\";\\ntype ImageSmoothingQuality = \"low\" | \"medium\" | \"high\";\\ntype IterationCompositeOperation = \"replace\" | \"accumulate\";\\ntype KeyFormat = \"raw\" | \"spki\" | \"pkcs8\" | \"jwk\";\\ntype KeyType = \"public\" | \"private\" | \"secret\";\\ntype KeyUsage = \"encrypt\" | \"decrypt\" | \"sign\" | \"verify\" | \"deriveKey\" | \"deriveBits\" | \"wrapKey\" | \"unwrapKey\";\\ntype LineAlignSetting = \"start\" | \"center\" | \"end\";\\ntype ListeningState = \"inactive\" | \"active\" | \"disambiguation\";\\ntype MSCredentialType = \"FIDO_2_0\";\\ntype MSTransportType = \"Embedded\" | \"USB\" | \"NFC\" | \"BT\";\\ntype MSWebViewPermissionState = \"unknown\" | \"defer\" | \"allow\" | \"deny\";\\ntype MSWebViewPermissionType = \"geolocation\" | \"unlimitedIndexedDBQuota\" | \"media\" | \"pointerlock\" | \"webnotifications\";\\ntype MediaDeviceKind = \"audioinput\" | \"audiooutput\" | \"videoinput\";\\ntype MediaKeyMessageType = \"license-request\" | \"license-renewal\" | \"license-release\" | \"individualization-request\";\\ntype MediaKeySessionType = \"temporary\" | \"persistent-license\" | \"persistent-release-message\";\\ntype MediaKeyStatus = \"usable\" | \"expired\" | \"output-downscaled\" | \"output-not-allowed\" | \"status-pending\" | \"internal-error\";\\ntype MediaKeysRequirement = \"required\" | \"optional\" | \"not-allowed\";\\ntype MediaStreamTrackState = \"live\" | \"ended\";\\ntype NavigationReason = \"up\" | \"down\" | \"left\" | \"right\";\\ntype NavigationType = \"navigate\" | \"reload\" | \"back_forward\" | \"prerender\";\\ntype NotificationDirection = \"auto\" | \"ltr\" | \"rtl\";\\ntype NotificationPermission = \"default\" | \"denied\" | \"granted\";\\ntype OrientationLockType = \"any\" | \"natural\" | \"landscape\" | \"portrait\" | \"portrait-primary\" | \"portrait-secondary\" | \"landscape-primary\" | \"landscape-secondary\";\\ntype OrientationType = \"portrait-primary\" | \"portrait-secondary\" | \"landscape-primary\" | \"landscape-secondary\";\\ntype OscillatorType = \"sine\" | \"square\" | \"sawtooth\" | \"triangle\" | \"custom\";\\ntype OverSampleType = \"none\" | \"2x\" | \"4x\";\\ntype PanningModelType = \"equalpower\" | \"HRTF\";\\ntype PaymentComplete = \"success\" | \"fail\" | \"unknown\";\\ntype PaymentShippingType = \"shipping\" | \"delivery\" | \"pickup\";\\ntype PlaybackDirection = \"normal\" | \"reverse\" | \"alternate\" | \"alternate-reverse\";\\ntype PositionAlignSetting = \"line-left\" | \"center\" | \"line-right\" | \"auto\";\\ntype PushEncryptionKeyName = \"p256dh\" | \"auth\";\\ntype PushPermissionState = \"denied\" | \"granted\" | \"prompt\";\\ntype RTCBundlePolicy = \"balanced\" | \"max-compat\" | \"max-bundle\";\\ntype RTCDataChannelState = \"connecting\" | \"open\" | \"closing\" | \"closed\";\\ntype RTCDegradationPreference = \"maintain-framerate\" | \"maintain-resolution\" | \"balanced\";\\ntype RTCDtlsRole = \"auto\" | \"client\" | \"server\";\\ntype RTCDtlsTransportState = \"new\" | \"connecting\" | \"connected\" | \"closed\" | \"failed\";\\ntype RTCDtxStatus = \"disabled\" | \"enabled\";\\ntype RTCErrorDetailType = \"data-channel-failure\" | \"dtls-failure\" | \"fingerprint-failure\" | \"idp-bad-script-failure\" | \"idp-execution-failure\" | \"idp-load-failure\" | \"idp-need-login\" | \"idp-timeout\" | \"idp-tls-failure\" | \"idp-token-expired\" | \"idp-token-invalid\" | \"sctp-failure\" | \"sdp-syntax-error\" | \"hardware-encoder-not-available\" | \"hardware-encoder-error\";\\ntype RTCIceCandidateType = \"host\" | \"srflx\" | \"prflx\" | \"relay\";\\ntype RTCIceComponent = \"rtp\" | \"rtcp\";\\ntype RTCIceConnectionState = \"new\" | \"checking\" | \"connected\" | \"completed\" | \"disconnected\" | \"failed\" | \"closed\";\\ntype RTCIceCredentialType = \"password\" | \"oauth\";\\ntype RTCIceGatherPolicy = \"all\" | \"nohost\" | \"relay\";\\ntype RTCIceGathererState = \"new\" | \"gathering\" | \"complete\";\\ntype RTCIceGatheringState = \"new\" | \"gathering\" | \"complete\";\\ntype RTCIceProtocol = \"udp\" | \"tcp\";\\ntype RTCIceRole = \"controlling\" | \"controlled\";\\ntype RTCIceTcpCandidateType = \"active\" | \"passive\" | \"so\";\\ntype RTCIceTransportPolicy = \"relay\" | \"all\";\\ntype RTCIceTransportState = \"new\" | \"checking\" | \"connected\" | \"completed\" | \"disconnected\" | \"failed\" | \"closed\";\\ntype RTCPeerConnectionState = \"new\" | \"connecting\" | \"connected\" | \"disconnected\" | \"failed\" | \"closed\";\\ntype RTCPriorityType = \"very-low\" | \"low\" | \"medium\" | \"high\";\\ntype RTCRtcpMuxPolicy = \"negotiate\" | \"require\";\\ntype RTCRtpTransceiverDirection = \"sendrecv\" | \"sendonly\" | \"recvonly\" | \"inactive\";\\ntype RTCSctpTransportState = \"connecting\" | \"connected\" | \"closed\";\\ntype RTCSdpType = \"offer\" | \"pranswer\" | \"answer\" | \"rollback\";\\ntype RTCSignalingState = \"stable\" | \"have-local-offer\" | \"have-remote-offer\" | \"have-local-pranswer\" | \"have-remote-pranswer\" | \"closed\";\\ntype RTCStatsIceCandidatePairState = \"frozen\" | \"waiting\" | \"inprogress\" | \"failed\" | \"succeeded\" | \"cancelled\";\\ntype RTCStatsIceCandidateType = \"host\" | \"serverreflexive\" | \"peerreflexive\" | \"relayed\";\\ntype RTCStatsType = \"inboundrtp\" | \"outboundrtp\" | \"session\" | \"datachannel\" | \"track\" | \"transport\" | \"candidatepair\" | \"localcandidate\" | \"remotecandidate\";\\ntype ReadyState = \"closed\" | \"open\" | \"ended\";\\ntype ReferrerPolicy = \"\" | \"no-referrer\" | \"no-referrer-when-downgrade\" | \"origin-only\" | \"origin-when-cross-origin\" | \"unsafe-url\";\\ntype RequestCache = \"default\" | \"no-store\" | \"reload\" | \"no-cache\" | \"force-cache\" | \"only-if-cached\";\\ntype RequestCredentials = \"omit\" | \"same-origin\" | \"include\";\\ntype RequestDestination = \"\" | \"audio\" | \"audioworklet\" | \"document\" | \"embed\" | \"font\" | \"image\" | \"manifest\" | \"object\" | \"paintworklet\" | \"report\" | \"script\" | \"sharedworker\" | \"style\" | \"track\" | \"video\" | \"worker\" | \"xslt\";\\ntype RequestMode = \"navigate\" | \"same-origin\" | \"no-cors\" | \"cors\";\\ntype RequestRedirect = \"follow\" | \"error\" | \"manual\";\\ntype ResponseType = \"basic\" | \"cors\" | \"default\" | \"error\" | \"opaque\" | \"opaqueredirect\";\\ntype ScopedCredentialType = \"ScopedCred\";\\ntype ScrollBehavior = \"auto\" | \"instant\" | \"smooth\";\\ntype ScrollLogicalPosition = \"start\" | \"center\" | \"end\" | \"nearest\";\\ntype ScrollRestoration = \"auto\" | \"manual\";\\ntype ScrollSetting = \"\" | \"up\";\\ntype SelectionMode = \"select\" | \"start\" | \"end\" | \"preserve\";\\ntype ServiceWorkerState = \"installing\" | \"installed\" | \"activating\" | \"activated\" | \"redundant\";\\ntype ServiceWorkerUpdateViaCache = \"imports\" | \"all\" | \"none\";\\ntype ShadowRootMode = \"open\" | \"closed\";\\ntype SpeechRecognitionErrorCode = \"no-speech\" | \"aborted\" | \"audio-capture\" | \"network\" | \"not-allowed\" | \"service-not-allowed\" | \"bad-grammar\" | \"language-not-supported\";\\ntype SpeechSynthesisErrorCode = \"canceled\" | \"interrupted\" | \"audio-busy\" | \"audio-hardware\" | \"network\" | \"synthesis-unavailable\" | \"synthesis-failed\" | \"language-unavailable\" | \"voice-unavailable\" | \"text-too-long\" | \"invalid-argument\";\\ntype SupportedType = \"text/html\" | \"text/xml\" | \"application/xml\" | \"application/xhtml+xml\" | \"image/svg+xml\";\\ntype TextTrackKind = \"subtitles\" | \"captions\" | \"descriptions\" | \"chapters\" | \"metadata\";\\ntype TextTrackMode = \"disabled\" | \"hidden\" | \"showing\";\\ntype TouchType = \"direct\" | \"stylus\";\\ntype Transport = \"usb\" | \"nfc\" | \"ble\";\\ntype VRDisplayEventReason = \"mounted\" | \"navigation\" | \"requested\" | \"unmounted\";\\ntype VideoFacingModeEnum = \"user\" | \"environment\" | \"left\" | \"right\";\\ntype VisibilityState = \"hidden\" | \"visible\" | \"prerender\";\\ntype WebGLPowerPreference = \"default\" | \"low-power\" | \"high-performance\";\\ntype WorkerType = \"classic\" | \"module\";\\ntype XMLHttpRequestResponseType = \"\" | \"arraybuffer\" | \"blob\" | \"document\" | \"json\" | \"text\";\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n\\n/////////////////////////////\\n/// WorkerGlobalScope APIs\\n/////////////////////////////\\n// These are only available in a Web Worker\\ndeclare function importScripts(...urls: string[]): void;\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n\\n\\n/////////////////////////////\\n/// Windows Script Host APIS\\n/////////////////////////////\\n\\n\\ninterface ActiveXObject {\\n    new (s: string): any;\\n}\\ndeclare var ActiveXObject: ActiveXObject;\\n\\ninterface ITextWriter {\\n    Write(s: string): void;\\n    WriteLine(s: string): void;\\n    Close(): void;\\n}\\n\\ninterface TextStreamBase {\\n    /**\\n     * The column number of the current character position in an input stream.\\n     */\\n    Column: number;\\n\\n    /**\\n     * The current line number in an input stream.\\n     */\\n    Line: number;\\n\\n    /**\\n     * Closes a text stream.\\n     * It is not necessary to close standard streams; they close automatically when the process ends. If\\n     * you close a standard stream, be aware that any other pointers to that standard stream become invalid.\\n     */\\n    Close(): void;\\n}\\n\\ninterface TextStreamWriter extends TextStreamBase {\\n    /**\\n     * Sends a string to an output stream.\\n     */\\n    Write(s: string): void;\\n\\n    /**\\n     * Sends a specified number of blank lines (newline characters) to an output stream.\\n     */\\n    WriteBlankLines(intLines: number): void;\\n\\n    /**\\n     * Sends a string followed by a newline character to an output stream.\\n     */\\n    WriteLine(s: string): void;\\n}\\n\\ninterface TextStreamReader extends TextStreamBase {\\n    /**\\n     * Returns a specified number of characters from an input stream, starting at the current pointer position.\\n     * Does not return until the ENTER key is pressed.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     */\\n    Read(characters: number): string;\\n\\n    /**\\n     * Returns all characters from an input stream.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     */\\n    ReadAll(): string;\\n\\n    /**\\n     * Returns an entire line from an input stream.\\n     * Although this method extracts the newline character, it does not add it to the returned string.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     */\\n    ReadLine(): string;\\n\\n    /**\\n     * Skips a specified number of characters when reading from an input text stream.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     * @param characters Positive number of characters to skip forward. (Backward skipping is not supported.)\\n     */\\n    Skip(characters: number): void;\\n\\n    /**\\n     * Skips the next line when reading from an input text stream.\\n     * Can only be used on a stream in reading mode, not writing or appending mode.\\n     */\\n    SkipLine(): void;\\n\\n    /**\\n     * Indicates whether the stream pointer position is at the end of a line.\\n     */\\n    AtEndOfLine: boolean;\\n\\n    /**\\n     * Indicates whether the stream pointer position is at the end of a stream.\\n     */\\n    AtEndOfStream: boolean;\\n}\\n\\ndeclare var WScript: {\\n    /**\\n     * Outputs text to either a message box (under WScript.exe) or the command console window followed by\\n     * a newline (under CScript.exe).\\n     */\\n    Echo(s: any): void;\\n\\n    /**\\n     * Exposes the write-only error output stream for the current script.\\n     * Can be accessed only while using CScript.exe.\\n     */\\n    StdErr: TextStreamWriter;\\n\\n    /**\\n     * Exposes the write-only output stream for the current script.\\n     * Can be accessed only while using CScript.exe.\\n     */\\n    StdOut: TextStreamWriter;\\n    Arguments: { length: number; Item(n: number): string; };\\n\\n    /**\\n     *  The full path of the currently running script.\\n     */\\n    ScriptFullName: string;\\n\\n    /**\\n     * Forces the script to stop immediately, with an optional exit code.\\n     */\\n    Quit(exitCode?: number): number;\\n\\n    /**\\n     * The Windows Script Host build version number.\\n     */\\n    BuildVersion: number;\\n\\n    /**\\n     * Fully qualified path of the host executable.\\n     */\\n    FullName: string;\\n\\n    /**\\n     * Gets/sets the script mode - interactive(true) or batch(false).\\n     */\\n    Interactive: boolean;\\n\\n    /**\\n     * The name of the host executable (WScript.exe or CScript.exe).\\n     */\\n    Name: string;\\n\\n    /**\\n     * Path of the directory containing the host executable.\\n     */\\n    Path: string;\\n\\n    /**\\n     * The filename of the currently running script.\\n     */\\n    ScriptName: string;\\n\\n    /**\\n     * Exposes the read-only input stream for the current script.\\n     * Can be accessed only while using CScript.exe.\\n     */\\n    StdIn: TextStreamReader;\\n\\n    /**\\n     * Windows Script Host version\\n     */\\n    Version: string;\\n\\n    /**\\n     * Connects a COM object\\'s event sources to functions named with a given prefix, in the form prefix_event.\\n     */\\n    ConnectObject(objEventSource: any, strPrefix: string): void;\\n\\n    /**\\n     * Creates a COM object.\\n     * @param strProgiID\\n     * @param strPrefix Function names in the form prefix_event will be bound to this object\\'s COM events.\\n     */\\n    CreateObject(strProgID: string, strPrefix?: string): any;\\n\\n    /**\\n     * Disconnects a COM object from its event sources.\\n     */\\n    DisconnectObject(obj: any): void;\\n\\n    /**\\n     * Retrieves an existing object with the specified ProgID from memory, or creates a new one from a file.\\n     * @param strPathname Fully qualified path to the file containing the object persisted to disk.\\n     *                       For objects in memory, pass a zero-length string.\\n     * @param strProgID\\n     * @param strPrefix Function names in the form prefix_event will be bound to this object\\'s COM events.\\n     */\\n    GetObject(strPathname: string, strProgID?: string, strPrefix?: string): any;\\n\\n    /**\\n     * Suspends script execution for a specified length of time, then continues execution.\\n     * @param intTime Interval (in milliseconds) to suspend script execution.\\n     */\\n    Sleep(intTime: number): void;\\n};\\n\\n/**\\n * WSH is an alias for WScript under Windows Script Host\\n */\\ndeclare var WSH: typeof WScript;\\n\\n/**\\n * Represents an Automation SAFEARRAY\\n */\\ndeclare class SafeArray<T = any> {\\n    private constructor();\\n    private SafeArray_typekey: SafeArray<T>;\\n}\\n\\n/**\\n * Allows enumerating over a COM collection, which may not have indexed item access.\\n */\\ninterface Enumerator<T = any> {\\n    /**\\n     * Returns true if the current item is the last one in the collection, or the collection is empty,\\n     * or the current item is undefined.\\n     */\\n    atEnd(): boolean;\\n\\n    /**\\n     * Returns the current item in the collection\\n     */\\n    item(): T;\\n\\n    /**\\n     * Resets the current item in the collection to the first item. If there are no items in the collection,\\n     * the current item is set to undefined.\\n     */\\n    moveFirst(): void;\\n\\n    /**\\n     * Moves the current item to the next item in the collection. If the enumerator is at the end of\\n     * the collection or the collection is empty, the current item is set to undefined.\\n     */\\n    moveNext(): void;\\n}\\n\\ninterface EnumeratorConstructor {\\n    new <T = any>(safearray: SafeArray<T>): Enumerator<T>;\\n    new <T = any>(collection: { Item(index: any): T }): Enumerator<T>;\\n    new <T = any>(collection: any): Enumerator<T>;\\n}\\n\\ndeclare var Enumerator: EnumeratorConstructor;\\n\\n/**\\n * Enables reading from a COM safe array, which might have an alternate lower bound, or multiple dimensions.\\n */\\ninterface VBArray<T = any> {\\n    /**\\n     * Returns the number of dimensions (1-based).\\n     */\\n    dimensions(): number;\\n\\n    /**\\n     * Takes an index for each dimension in the array, and returns the item at the corresponding location.\\n     */\\n    getItem(dimension1Index: number, ...dimensionNIndexes: number[]): T;\\n\\n    /**\\n     * Returns the smallest available index for a given dimension.\\n     * @param dimension 1-based dimension (defaults to 1)\\n     */\\n    lbound(dimension?: number): number;\\n\\n    /**\\n     * Returns the largest available index for a given dimension.\\n     * @param dimension 1-based dimension (defaults to 1)\\n     */\\n    ubound(dimension?: number): number;\\n\\n    /**\\n     * Returns a Javascript array with all the elements in the VBArray. If there are multiple dimensions,\\n     * each successive dimension is appended to the end of the array.\\n     * Example: [[1,2,3],[4,5,6]] becomes [1,2,3,4,5,6]\\n     */\\n    toArray(): T[];\\n}\\n\\ninterface VBArrayConstructor {\\n    new <T = any>(safeArray: SafeArray<T>): VBArray<T>;\\n}\\n\\ndeclare var VBArray: VBArrayConstructor;\\n\\n/**\\n * Automation date (VT_DATE)\\n */\\ndeclare class VarDate {\\n    private constructor();\\n    private VarDate_typekey: VarDate;\\n}\\n\\ninterface DateConstructor {\\n    new (vd: VarDate): Date;\\n}\\n\\ninterface Date {\\n    getVarDate: () => VarDate;\\n}\\n'},Mn={NAME:\"defaultLib:lib.es6.d.ts\",CONTENTS:'/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n/////////////////////////////\\n/// ECMAScript APIs\\n/////////////////////////////\\n\\ndeclare const NaN: number;\\ndeclare const Infinity: number;\\n\\n/**\\n  * Evaluates JavaScript code and executes it.\\n  * @param x A String value that contains valid JavaScript code.\\n  */\\ndeclare function eval(x: string): any;\\n\\n/**\\n  * Converts A string to an integer.\\n  * @param s A string to convert into a number.\\n  * @param radix A value between 2 and 36 that specifies the base of the number in numString.\\n  * If this argument is not supplied, strings with a prefix of \\'0x\\' are considered hexadecimal.\\n  * All other strings are considered decimal.\\n  */\\ndeclare function parseInt(s: string, radix?: number): number;\\n\\n/**\\n  * Converts a string to a floating-point number.\\n  * @param string A string that contains a floating-point number.\\n  */\\ndeclare function parseFloat(string: string): number;\\n\\n/**\\n  * Returns a Boolean value that indicates whether a value is the reserved value NaN (not a number).\\n  * @param number A numeric value.\\n  */\\ndeclare function isNaN(number: number): boolean;\\n\\n/**\\n  * Determines whether a supplied number is finite.\\n  * @param number Any numeric value.\\n  */\\ndeclare function isFinite(number: number): boolean;\\n\\n/**\\n  * Gets the unencoded version of an encoded Uniform Resource Identifier (URI).\\n  * @param encodedURI A value representing an encoded URI.\\n  */\\ndeclare function decodeURI(encodedURI: string): string;\\n\\n/**\\n  * Gets the unencoded version of an encoded component of a Uniform Resource Identifier (URI).\\n  * @param encodedURIComponent A value representing an encoded URI component.\\n  */\\ndeclare function decodeURIComponent(encodedURIComponent: string): string;\\n\\n/**\\n  * Encodes a text string as a valid Uniform Resource Identifier (URI)\\n  * @param uri A value representing an encoded URI.\\n  */\\ndeclare function encodeURI(uri: string): string;\\n\\n/**\\n  * Encodes a text string as a valid component of a Uniform Resource Identifier (URI).\\n  * @param uriComponent A value representing an encoded URI component.\\n  */\\ndeclare function encodeURIComponent(uriComponent: string): string;\\n\\n/**\\n  * Computes a new string in which certain characters have been replaced by a hexadecimal escape sequence.\\n  * @param string A string value\\n  */\\ndeclare function escape(string: string): string;\\n\\n/**\\n  * Computes a new string in which hexadecimal escape sequences are replaced with the character that it represents.\\n  * @param string A string value\\n  */\\ndeclare function unescape(string: string): string;\\n\\ninterface Symbol {\\n  /** Returns a string representation of an object. */\\n  toString(): string;\\n\\n  /** Returns the primitive value of the specified object. */\\n  valueOf(): symbol;\\n}\\n\\ndeclare type PropertyKey = string | number | symbol;\\n\\ninterface PropertyDescriptor {\\n    configurable?: boolean;\\n    enumerable?: boolean;\\n    value?: any;\\n    writable?: boolean;\\n    get?(): any;\\n    set?(v: any): void;\\n}\\n\\ninterface PropertyDescriptorMap {\\n    [s: string]: PropertyDescriptor;\\n}\\n\\ninterface Object {\\n    /** The initial value of Object.prototype.constructor is the standard built-in Object constructor. */\\n    constructor: Function;\\n\\n    /** Returns a string representation of an object. */\\n    toString(): string;\\n\\n    /** Returns a date converted to a string using the current locale. */\\n    toLocaleString(): string;\\n\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): Object;\\n\\n    /**\\n      * Determines whether an object has a property with the specified name.\\n      * @param v A property name.\\n      */\\n    hasOwnProperty(v: PropertyKey): boolean;\\n\\n    /**\\n      * Determines whether an object exists in another object\\'s prototype chain.\\n      * @param v Another object whose prototype chain is to be checked.\\n      */\\n    isPrototypeOf(v: Object): boolean;\\n\\n    /**\\n      * Determines whether a specified property is enumerable.\\n      * @param v A property name.\\n      */\\n    propertyIsEnumerable(v: PropertyKey): boolean;\\n}\\n\\ninterface ObjectConstructor {\\n    new(value?: any): Object;\\n    (): any;\\n    (value: any): any;\\n\\n    /** A reference to the prototype for a class of objects. */\\n    readonly prototype: Object;\\n\\n    /**\\n      * Returns the prototype of an object.\\n      * @param o The object that references the prototype.\\n      */\\n    getPrototypeOf(o: any): any;\\n\\n    /**\\n      * Gets the own property descriptor of the specified object.\\n      * An own property descriptor is one that is defined directly on the object and is not inherited from the object\\'s prototype.\\n      * @param o Object that contains the property.\\n      * @param p Name of the property.\\n    */\\n    getOwnPropertyDescriptor(o: any, p: PropertyKey): PropertyDescriptor | undefined;\\n\\n    /**\\n      * Returns the names of the own properties of an object. The own properties of an object are those that are defined directly\\n      * on that object, and are not inherited from the object\\'s prototype. The properties of an object include both fields (objects) and functions.\\n      * @param o Object that contains the own properties.\\n      */\\n    getOwnPropertyNames(o: any): string[];\\n\\n    /**\\n      * Creates an object that has the specified prototype or that has null prototype.\\n      * @param o Object to use as a prototype. May be null.\\n      */\\n    create(o: object | null): any;\\n\\n    /**\\n      * Creates an object that has the specified prototype, and that optionally contains specified properties.\\n      * @param o Object to use as a prototype. May be null\\n      * @param properties JavaScript object that contains one or more property descriptors.\\n      */\\n    create(o: object | null, properties: PropertyDescriptorMap & ThisType<any>): any;\\n\\n    /**\\n      * Adds a property to an object, or modifies attributes of an existing property.\\n      * @param o Object on which to add or modify the property. This can be a native JavaScript object (that is, a user-defined object or a built in object) or a DOM object.\\n      * @param p The property name.\\n      * @param attributes Descriptor for the property. It can be for a data property or an accessor property.\\n      */\\n    defineProperty(o: any, p: PropertyKey, attributes: PropertyDescriptor & ThisType<any>): any;\\n\\n    /**\\n      * Adds one or more properties to an object, and/or modifies attributes of existing properties.\\n      * @param o Object on which to add or modify the properties. This can be a native JavaScript object or a DOM object.\\n      * @param properties JavaScript object that contains one or more descriptor objects. Each descriptor object describes a data property or an accessor property.\\n      */\\n    defineProperties(o: any, properties: PropertyDescriptorMap & ThisType<any>): any;\\n\\n    /**\\n      * Prevents the modification of attributes of existing properties, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    seal<T>(o: T): T;\\n\\n    /**\\n      * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    freeze<T>(a: T[]): ReadonlyArray<T>;\\n\\n    /**\\n      * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    freeze<T extends Function>(f: T): T;\\n\\n    /**\\n      * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\\n      * @param o Object on which to lock the attributes.\\n      */\\n    freeze<T>(o: T): Readonly<T>;\\n\\n    /**\\n      * Prevents the addition of new properties to an object.\\n      * @param o Object to make non-extensible.\\n      */\\n    preventExtensions<T>(o: T): T;\\n\\n    /**\\n      * Returns true if existing property attributes cannot be modified in an object and new properties cannot be added to the object.\\n      * @param o Object to test.\\n      */\\n    isSealed(o: any): boolean;\\n\\n    /**\\n      * Returns true if existing property attributes and values cannot be modified in an object, and new properties cannot be added to the object.\\n      * @param o Object to test.\\n      */\\n    isFrozen(o: any): boolean;\\n\\n    /**\\n      * Returns a value that indicates whether new properties can be added to an object.\\n      * @param o Object to test.\\n      */\\n    isExtensible(o: any): boolean;\\n\\n    /**\\n      * Returns the names of the enumerable properties and methods of an object.\\n      * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\\n      */\\n    keys(o: {}): string[];\\n}\\n\\n/**\\n  * Provides functionality common to all JavaScript objects.\\n  */\\ndeclare const Object: ObjectConstructor;\\n\\n/**\\n  * Creates a new function.\\n  */\\ninterface Function {\\n    /**\\n      * Calls the function, substituting the specified object for the this value of the function, and the specified array for the arguments of the function.\\n      * @param thisArg The object to be used as the this object.\\n      * @param argArray A set of arguments to be passed to the function.\\n      */\\n    apply(this: Function, thisArg: any, argArray?: any): any;\\n\\n    /**\\n      * Calls a method of an object, substituting another object for the current object.\\n      * @param thisArg The object to be used as the current object.\\n      * @param argArray A list of arguments to be passed to the method.\\n      */\\n    call(this: Function, thisArg: any, ...argArray: any[]): any;\\n\\n    /**\\n      * For a given function, creates a bound function that has the same body as the original function.\\n      * The this object of the bound function is associated with the specified object, and has the specified initial parameters.\\n      * @param thisArg An object to which the this keyword can refer inside the new function.\\n      * @param argArray A list of arguments to be passed to the new function.\\n      */\\n    bind(this: Function, thisArg: any, ...argArray: any[]): any;\\n\\n    /** Returns a string representation of a function. */\\n    toString(): string;\\n\\n    prototype: any;\\n    readonly length: number;\\n\\n    // Non-standard extensions\\n    arguments: any;\\n    caller: Function;\\n}\\n\\ninterface FunctionConstructor {\\n    /**\\n      * Creates a new function.\\n      * @param args A list of arguments the function accepts.\\n      */\\n    new(...args: string[]): Function;\\n    (...args: string[]): Function;\\n    readonly prototype: Function;\\n}\\n\\ndeclare const Function: FunctionConstructor;\\n\\ninterface IArguments {\\n    [index: number]: any;\\n    length: number;\\n    callee: Function;\\n}\\n\\ninterface String {\\n    /** Returns a string representation of a string. */\\n    toString(): string;\\n\\n    /**\\n      * Returns the character at the specified index.\\n      * @param pos The zero-based index of the desired character.\\n      */\\n    charAt(pos: number): string;\\n\\n    /**\\n      * Returns the Unicode value of the character at the specified location.\\n      * @param index The zero-based index of the desired character. If there is no character at the specified index, NaN is returned.\\n      */\\n    charCodeAt(index: number): number;\\n\\n    /**\\n      * Returns a string that contains the concatenation of two or more strings.\\n      * @param strings The strings to append to the end of the string.\\n      */\\n    concat(...strings: string[]): string;\\n\\n    /**\\n      * Returns the position of the first occurrence of a substring.\\n      * @param searchString The substring to search for in the string\\n      * @param position The index at which to begin searching the String object. If omitted, search starts at the beginning of the string.\\n      */\\n    indexOf(searchString: string, position?: number): number;\\n\\n    /**\\n      * Returns the last occurrence of a substring in the string.\\n      * @param searchString The substring to search for.\\n      * @param position The index at which to begin searching. If omitted, the search begins at the end of the string.\\n      */\\n    lastIndexOf(searchString: string, position?: number): number;\\n\\n    /**\\n      * Determines whether two strings are equivalent in the current locale.\\n      * @param that String to compare to target string\\n      */\\n    localeCompare(that: string): number;\\n\\n    /**\\n      * Matches a string with a regular expression, and returns an array containing the results of that search.\\n      * @param regexp A variable name or string literal containing the regular expression pattern and flags.\\n      */\\n    match(regexp: string | RegExp): RegExpMatchArray | null;\\n\\n    /**\\n      * Replaces text in a string, using a regular expression or search string.\\n      * @param searchValue A string to search for.\\n      * @param replaceValue A string containing the text to replace for every successful match of searchValue in this string.\\n      */\\n    replace(searchValue: string | RegExp, replaceValue: string): string;\\n\\n    /**\\n      * Replaces text in a string, using a regular expression or search string.\\n      * @param searchValue A string to search for.\\n      * @param replacer A function that returns the replacement text.\\n      */\\n    replace(searchValue: string | RegExp, replacer: (substring: string, ...args: any[]) => string): string;\\n\\n    /**\\n      * Finds the first substring match in a regular expression search.\\n      * @param regexp The regular expression pattern and applicable flags.\\n      */\\n    search(regexp: string | RegExp): number;\\n\\n    /**\\n      * Returns a section of a string.\\n      * @param start The index to the beginning of the specified portion of stringObj.\\n      * @param end The index to the end of the specified portion of stringObj. The substring includes the characters up to, but not including, the character indicated by end.\\n      * If this value is not specified, the substring continues to the end of stringObj.\\n      */\\n    slice(start?: number, end?: number): string;\\n\\n    /**\\n      * Split a string into substrings using the specified separator and return them as an array.\\n      * @param separator A string that identifies character or characters to use in separating the string. If omitted, a single-element array containing the entire string is returned.\\n      * @param limit A value used to limit the number of elements returned in the array.\\n      */\\n    split(separator: string | RegExp, limit?: number): string[];\\n\\n    /**\\n      * Returns the substring at the specified location within a String object.\\n      * @param start The zero-based index number indicating the beginning of the substring.\\n      * @param end Zero-based index number indicating the end of the substring. The substring includes the characters up to, but not including, the character indicated by end.\\n      * If end is omitted, the characters from start through the end of the original string are returned.\\n      */\\n    substring(start: number, end?: number): string;\\n\\n    /** Converts all the alphabetic characters in a string to lowercase. */\\n    toLowerCase(): string;\\n\\n    /** Converts all alphabetic characters to lowercase, taking into account the host environment\\'s current locale. */\\n    toLocaleLowerCase(): string;\\n\\n    /** Converts all the alphabetic characters in a string to uppercase. */\\n    toUpperCase(): string;\\n\\n    /** Returns a string where all alphabetic characters have been converted to uppercase, taking into account the host environment\\'s current locale. */\\n    toLocaleUpperCase(): string;\\n\\n    /** Removes the leading and trailing white space and line terminator characters from a string. */\\n    trim(): string;\\n\\n    /** Returns the length of a String object. */\\n    readonly length: number;\\n\\n    // IE extensions\\n    /**\\n      * Gets a substring beginning at the specified location and having the specified length.\\n      * @param from The starting position of the desired substring. The index of the first character in the string is zero.\\n      * @param length The number of characters to include in the returned substring.\\n      */\\n    substr(from: number, length?: number): string;\\n\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): string;\\n\\n    readonly [index: number]: string;\\n}\\n\\ninterface StringConstructor {\\n    new(value?: any): String;\\n    (value?: any): string;\\n    readonly prototype: String;\\n    fromCharCode(...codes: number[]): string;\\n}\\n\\n/**\\n  * Allows manipulation and formatting of text strings and determination and location of substrings within strings.\\n  */\\ndeclare const String: StringConstructor;\\n\\ninterface Boolean {\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): boolean;\\n}\\n\\ninterface BooleanConstructor {\\n    new(value?: any): Boolean;\\n    (value?: any): boolean;\\n    readonly prototype: Boolean;\\n}\\n\\ndeclare const Boolean: BooleanConstructor;\\n\\ninterface Number {\\n    /**\\n      * Returns a string representation of an object.\\n      * @param radix Specifies a radix for converting numeric values to strings. This value is only used for numbers.\\n      */\\n    toString(radix?: number): string;\\n\\n    /**\\n      * Returns a string representing a number in fixed-point notation.\\n      * @param fractionDigits Number of digits after the decimal point. Must be in the range 0 - 20, inclusive.\\n      */\\n    toFixed(fractionDigits?: number): string;\\n\\n    /**\\n      * Returns a string containing a number represented in exponential notation.\\n      * @param fractionDigits Number of digits after the decimal point. Must be in the range 0 - 20, inclusive.\\n      */\\n    toExponential(fractionDigits?: number): string;\\n\\n    /**\\n      * Returns a string containing a number represented either in exponential or fixed-point notation with a specified number of digits.\\n      * @param precision Number of significant digits. Must be in the range 1 - 21, inclusive.\\n      */\\n    toPrecision(precision?: number): string;\\n\\n    /** Returns the primitive value of the specified object. */\\n    valueOf(): number;\\n}\\n\\ninterface NumberConstructor {\\n    new(value?: any): Number;\\n    (value?: any): number;\\n    readonly prototype: Number;\\n\\n    /** The largest number that can be represented in JavaScript. Equal to approximately 1.79E+308. */\\n    readonly MAX_VALUE: number;\\n\\n    /** The closest number to zero that can be represented in JavaScript. Equal to approximately 5.00E-324. */\\n    readonly MIN_VALUE: number;\\n\\n    /**\\n      * A value that is not a number.\\n      * In equality comparisons, NaN does not equal any value, including itself. To test whether a value is equivalent to NaN, use the isNaN function.\\n      */\\n    readonly NaN: number;\\n\\n    /**\\n      * A value that is less than the largest negative number that can be represented in JavaScript.\\n      * JavaScript displays NEGATIVE_INFINITY values as -infinity.\\n      */\\n    readonly NEGATIVE_INFINITY: number;\\n\\n    /**\\n      * A value greater than the largest number that can be represented in JavaScript.\\n      * JavaScript displays POSITIVE_INFINITY values as infinity.\\n      */\\n    readonly POSITIVE_INFINITY: number;\\n}\\n\\n/** An object that represents a number of any kind. All JavaScript numbers are 64-bit floating-point numbers. */\\ndeclare const Number: NumberConstructor;\\n\\ninterface TemplateStringsArray extends ReadonlyArray<string> {\\n    readonly raw: ReadonlyArray<string>;\\n}\\n\\n/**\\n * The type of `import.meta`.\\n * \\n * If you need to declare that a given property exists on `import.meta`,\\n * this type may be augmented via interface merging.\\n */\\ninterface ImportMeta {\\n}\\n\\ninterface Math {\\n    /** The mathematical constant e. This is Euler\\'s number, the base of natural logarithms. */\\n    readonly E: number;\\n    /** The natural logarithm of 10. */\\n    readonly LN10: number;\\n    /** The natural logarithm of 2. */\\n    readonly LN2: number;\\n    /** The base-2 logarithm of e. */\\n    readonly LOG2E: number;\\n    /** The base-10 logarithm of e. */\\n    readonly LOG10E: number;\\n    /** Pi. This is the ratio of the circumference of a circle to its diameter. */\\n    readonly PI: number;\\n    /** The square root of 0.5, or, equivalently, one divided by the square root of 2. */\\n    readonly SQRT1_2: number;\\n    /** The square root of 2. */\\n    readonly SQRT2: number;\\n    /**\\n      * Returns the absolute value of a number (the value without regard to whether it is positive or negative).\\n      * For example, the absolute value of -5 is the same as the absolute value of 5.\\n      * @param x A numeric expression for which the absolute value is needed.\\n      */\\n    abs(x: number): number;\\n    /**\\n      * Returns the arc cosine (or inverse cosine) of a number.\\n      * @param x A numeric expression.\\n      */\\n    acos(x: number): number;\\n    /**\\n      * Returns the arcsine of a number.\\n      * @param x A numeric expression.\\n      */\\n    asin(x: number): number;\\n    /**\\n      * Returns the arctangent of a number.\\n      * @param x A numeric expression for which the arctangent is needed.\\n      */\\n    atan(x: number): number;\\n    /**\\n      * Returns the angle (in radians) from the X axis to a point.\\n      * @param y A numeric expression representing the cartesian y-coordinate.\\n      * @param x A numeric expression representing the cartesian x-coordinate.\\n      */\\n    atan2(y: number, x: number): number;\\n    /**\\n      * Returns the smallest integer greater than or equal to its numeric argument.\\n      * @param x A numeric expression.\\n      */\\n    ceil(x: number): number;\\n    /**\\n      * Returns the cosine of a number.\\n      * @param x A numeric expression that contains an angle measured in radians.\\n      */\\n    cos(x: number): number;\\n    /**\\n      * Returns e (the base of natural logarithms) raised to a power.\\n      * @param x A numeric expression representing the power of e.\\n      */\\n    exp(x: number): number;\\n    /**\\n      * Returns the greatest integer less than or equal to its numeric argument.\\n      * @param x A numeric expression.\\n      */\\n    floor(x: number): number;\\n    /**\\n      * Returns the natural logarithm (base e) of a number.\\n      * @param x A numeric expression.\\n      */\\n    log(x: number): number;\\n    /**\\n      * Returns the larger of a set of supplied numeric expressions.\\n      * @param values Numeric expressions to be evaluated.\\n      */\\n    max(...values: number[]): number;\\n    /**\\n      * Returns the smaller of a set of supplied numeric expressions.\\n      * @param values Numeric expressions to be evaluated.\\n      */\\n    min(...values: number[]): number;\\n    /**\\n      * Returns the value of a base expression taken to a specified power.\\n      * @param x The base value of the expression.\\n      * @param y The exponent value of the expression.\\n      */\\n    pow(x: number, y: number): number;\\n    /** Returns a pseudorandom number between 0 and 1. */\\n    random(): number;\\n    /**\\n      * Returns a supplied numeric expression rounded to the nearest number.\\n      * @param x The value to be rounded to the nearest number.\\n      */\\n    round(x: number): number;\\n    /**\\n      * Returns the sine of a number.\\n      * @param x A numeric expression that contains an angle measured in radians.\\n      */\\n    sin(x: number): number;\\n    /**\\n      * Returns the square root of a number.\\n      * @param x A numeric expression.\\n      */\\n    sqrt(x: number): number;\\n    /**\\n      * Returns the tangent of a number.\\n      * @param x A numeric expression that contains an angle measured in radians.\\n      */\\n    tan(x: number): number;\\n}\\n/** An intrinsic object that provides basic mathematics functionality and constants. */\\ndeclare const Math: Math;\\n\\n/** Enables basic storage and retrieval of dates and times. */\\ninterface Date {\\n    /** Returns a string representation of a date. The format of the string depends on the locale. */\\n    toString(): string;\\n    /** Returns a date as a string value. */\\n    toDateString(): string;\\n    /** Returns a time as a string value. */\\n    toTimeString(): string;\\n    /** Returns a value as a string value appropriate to the host environment\\'s current locale. */\\n    toLocaleString(): string;\\n    /** Returns a date as a string value appropriate to the host environment\\'s current locale. */\\n    toLocaleDateString(): string;\\n    /** Returns a time as a string value appropriate to the host environment\\'s current locale. */\\n    toLocaleTimeString(): string;\\n    /** Returns the stored time value in milliseconds since midnight, January 1, 1970 UTC. */\\n    valueOf(): number;\\n    /** Gets the time value in milliseconds. */\\n    getTime(): number;\\n    /** Gets the year, using local time. */\\n    getFullYear(): number;\\n    /** Gets the year using Universal Coordinated Time (UTC). */\\n    getUTCFullYear(): number;\\n    /** Gets the month, using local time. */\\n    getMonth(): number;\\n    /** Gets the month of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCMonth(): number;\\n    /** Gets the day-of-the-month, using local time. */\\n    getDate(): number;\\n    /** Gets the day-of-the-month, using Universal Coordinated Time (UTC). */\\n    getUTCDate(): number;\\n    /** Gets the day of the week, using local time. */\\n    getDay(): number;\\n    /** Gets the day of the week using Universal Coordinated Time (UTC). */\\n    getUTCDay(): number;\\n    /** Gets the hours in a date, using local time. */\\n    getHours(): number;\\n    /** Gets the hours value in a Date object using Universal Coordinated Time (UTC). */\\n    getUTCHours(): number;\\n    /** Gets the minutes of a Date object, using local time. */\\n    getMinutes(): number;\\n    /** Gets the minutes of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCMinutes(): number;\\n    /** Gets the seconds of a Date object, using local time. */\\n    getSeconds(): number;\\n    /** Gets the seconds of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCSeconds(): number;\\n    /** Gets the milliseconds of a Date, using local time. */\\n    getMilliseconds(): number;\\n    /** Gets the milliseconds of a Date object using Universal Coordinated Time (UTC). */\\n    getUTCMilliseconds(): number;\\n    /** Gets the difference in minutes between the time on the local computer and Universal Coordinated Time (UTC). */\\n    getTimezoneOffset(): number;\\n    /**\\n      * Sets the date and time value in the Date object.\\n      * @param time A numeric value representing the number of elapsed milliseconds since midnight, January 1, 1970 GMT.\\n      */\\n    setTime(time: number): number;\\n    /**\\n      * Sets the milliseconds value in the Date object using local time.\\n      * @param ms A numeric value equal to the millisecond value.\\n      */\\n    setMilliseconds(ms: number): number;\\n    /**\\n      * Sets the milliseconds value in the Date object using Universal Coordinated Time (UTC).\\n      * @param ms A numeric value equal to the millisecond value.\\n      */\\n    setUTCMilliseconds(ms: number): number;\\n\\n    /**\\n      * Sets the seconds value in the Date object using local time.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setSeconds(sec: number, ms?: number): number;\\n    /**\\n      * Sets the seconds value in the Date object using Universal Coordinated Time (UTC).\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setUTCSeconds(sec: number, ms?: number): number;\\n    /**\\n      * Sets the minutes value in the Date object using local time.\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setMinutes(min: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the minutes value in the Date object using Universal Coordinated Time (UTC).\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setUTCMinutes(min: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the hour value in the Date object using local time.\\n      * @param hours A numeric value equal to the hours value.\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setHours(hours: number, min?: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the hours value in the Date object using Universal Coordinated Time (UTC).\\n      * @param hours A numeric value equal to the hours value.\\n      * @param min A numeric value equal to the minutes value.\\n      * @param sec A numeric value equal to the seconds value.\\n      * @param ms A numeric value equal to the milliseconds value.\\n      */\\n    setUTCHours(hours: number, min?: number, sec?: number, ms?: number): number;\\n    /**\\n      * Sets the numeric day-of-the-month value of the Date object using local time.\\n      * @param date A numeric value equal to the day of the month.\\n      */\\n    setDate(date: number): number;\\n    /**\\n      * Sets the numeric day of the month in the Date object using Universal Coordinated Time (UTC).\\n      * @param date A numeric value equal to the day of the month.\\n      */\\n    setUTCDate(date: number): number;\\n    /**\\n      * Sets the month value in the Date object using local time.\\n      * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively.\\n      * @param date A numeric value representing the day of the month. If this value is not supplied, the value from a call to the getDate method is used.\\n      */\\n    setMonth(month: number, date?: number): number;\\n    /**\\n      * Sets the month value in the Date object using Universal Coordinated Time (UTC).\\n      * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively.\\n      * @param date A numeric value representing the day of the month. If it is not supplied, the value from a call to the getUTCDate method is used.\\n      */\\n    setUTCMonth(month: number, date?: number): number;\\n    /**\\n      * Sets the year of the Date object using local time.\\n      * @param year A numeric value for the year.\\n      * @param month A zero-based numeric value for the month (0 for January, 11 for December). Must be specified if numDate is specified.\\n      * @param date A numeric value equal for the day of the month.\\n      */\\n    setFullYear(year: number, month?: number, date?: number): number;\\n    /**\\n      * Sets the year value in the Date object using Universal Coordinated Time (UTC).\\n      * @param year A numeric value equal to the year.\\n      * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively. Must be supplied if numDate is supplied.\\n      * @param date A numeric value equal to the day of the month.\\n      */\\n    setUTCFullYear(year: number, month?: number, date?: number): number;\\n    /** Returns a date converted to a string using Universal Coordinated Time (UTC). */\\n    toUTCString(): string;\\n    /** Returns a date as a string value in ISO format. */\\n    toISOString(): string;\\n    /** Used by the JSON.stringify method to enable the transformation of an object\\'s data for JavaScript Object Notation (JSON) serialization. */\\n    toJSON(key?: any): string;\\n}\\n\\ninterface DateConstructor {\\n    new(): Date;\\n    new(value: number | string): Date;\\n    new(year: number, month: number, date?: number, hours?: number, minutes?: number, seconds?: number, ms?: number): Date;\\n    (): string;\\n    readonly prototype: Date;\\n    /**\\n      * Parses a string containing a date, and returns the number of milliseconds between that date and midnight, January 1, 1970.\\n      * @param s A date string\\n      */\\n    parse(s: string): number;\\n    /**\\n      * Returns the number of milliseconds between midnight, January 1, 1970 Universal Coordinated Time (UTC) (or GMT) and the specified date.\\n      * @param year The full year designation is required for cross-century date accuracy. If year is between 0 and 99 is used, then year is assumed to be 1900 + year.\\n      * @param month The month as an number between 0 and 11 (January to December).\\n      * @param date The date as an number between 1 and 31.\\n      * @param hours Must be supplied if minutes is supplied. An number from 0 to 23 (midnight to 11pm) that specifies the hour.\\n      * @param minutes Must be supplied if seconds is supplied. An number from 0 to 59 that specifies the minutes.\\n      * @param seconds Must be supplied if milliseconds is supplied. An number from 0 to 59 that specifies the seconds.\\n      * @param ms An number from 0 to 999 that specifies the milliseconds.\\n      */\\n    UTC(year: number, month: number, date?: number, hours?: number, minutes?: number, seconds?: number, ms?: number): number;\\n    now(): number;\\n}\\n\\ndeclare const Date: DateConstructor;\\n\\ninterface RegExpMatchArray extends Array<string> {\\n    index?: number;\\n    input?: string;\\n}\\n\\ninterface RegExpExecArray extends Array<string> {\\n    index: number;\\n    input: string;\\n}\\n\\ninterface RegExp {\\n    /**\\n      * Executes a search on a string using a regular expression pattern, and returns an array containing the results of that search.\\n      * @param string The String object or string literal on which to perform the search.\\n      */\\n    exec(string: string): RegExpExecArray | null;\\n\\n    /**\\n      * Returns a Boolean value that indicates whether or not a pattern exists in a searched string.\\n      * @param string String on which to perform the search.\\n      */\\n    test(string: string): boolean;\\n\\n    /** Returns a copy of the text of the regular expression pattern. Read-only. The regExp argument is a Regular expression object. It can be a variable name or a literal. */\\n    readonly source: string;\\n\\n    /** Returns a Boolean value indicating the state of the global flag (g) used with a regular expression. Default is false. Read-only. */\\n    readonly global: boolean;\\n\\n    /** Returns a Boolean value indicating the state of the ignoreCase flag (i) used with a regular expression. Default is false. Read-only. */\\n    readonly ignoreCase: boolean;\\n\\n    /** Returns a Boolean value indicating the state of the multiline flag (m) used with a regular expression. Default is false. Read-only. */\\n    readonly multiline: boolean;\\n\\n    lastIndex: number;\\n\\n    // Non-standard extensions\\n    compile(): this;\\n}\\n\\ninterface RegExpConstructor {\\n    new(pattern: RegExp | string): RegExp;\\n    new(pattern: string, flags?: string): RegExp;\\n    (pattern: RegExp | string): RegExp;\\n    (pattern: string, flags?: string): RegExp;\\n    readonly prototype: RegExp;\\n\\n    // Non-standard extensions\\n    $1: string;\\n    $2: string;\\n    $3: string;\\n    $4: string;\\n    $5: string;\\n    $6: string;\\n    $7: string;\\n    $8: string;\\n    $9: string;\\n    lastMatch: string;\\n}\\n\\ndeclare const RegExp: RegExpConstructor;\\n\\ninterface Error {\\n    name: string;\\n    message: string;\\n    stack?: string;\\n}\\n\\ninterface ErrorConstructor {\\n    new(message?: string): Error;\\n    (message?: string): Error;\\n    readonly prototype: Error;\\n}\\n\\ndeclare const Error: ErrorConstructor;\\n\\ninterface EvalError extends Error {\\n}\\n\\ninterface EvalErrorConstructor {\\n    new(message?: string): EvalError;\\n    (message?: string): EvalError;\\n    readonly prototype: EvalError;\\n}\\n\\ndeclare const EvalError: EvalErrorConstructor;\\n\\ninterface RangeError extends Error {\\n}\\n\\ninterface RangeErrorConstructor {\\n    new(message?: string): RangeError;\\n    (message?: string): RangeError;\\n    readonly prototype: RangeError;\\n}\\n\\ndeclare const RangeError: RangeErrorConstructor;\\n\\ninterface ReferenceError extends Error {\\n}\\n\\ninterface ReferenceErrorConstructor {\\n    new(message?: string): ReferenceError;\\n    (message?: string): ReferenceError;\\n    readonly prototype: ReferenceError;\\n}\\n\\ndeclare const ReferenceError: ReferenceErrorConstructor;\\n\\ninterface SyntaxError extends Error {\\n}\\n\\ninterface SyntaxErrorConstructor {\\n    new(message?: string): SyntaxError;\\n    (message?: string): SyntaxError;\\n    readonly prototype: SyntaxError;\\n}\\n\\ndeclare const SyntaxError: SyntaxErrorConstructor;\\n\\ninterface TypeError extends Error {\\n}\\n\\ninterface TypeErrorConstructor {\\n    new(message?: string): TypeError;\\n    (message?: string): TypeError;\\n    readonly prototype: TypeError;\\n}\\n\\ndeclare const TypeError: TypeErrorConstructor;\\n\\ninterface URIError extends Error {\\n}\\n\\ninterface URIErrorConstructor {\\n    new(message?: string): URIError;\\n    (message?: string): URIError;\\n    readonly prototype: URIError;\\n}\\n\\ndeclare const URIError: URIErrorConstructor;\\n\\ninterface JSON {\\n    /**\\n      * Converts a JavaScript Object Notation (JSON) string into an object.\\n      * @param text A valid JSON string.\\n      * @param reviver A function that transforms the results. This function is called for each member of the object.\\n      * If a member contains nested objects, the nested objects are transformed before the parent object is.\\n      */\\n    parse(text: string, reviver?: (key: any, value: any) => any): any;\\n    /**\\n      * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.\\n      * @param value A JavaScript value, usually an object or array, to be converted.\\n      * @param replacer A function that transforms the results.\\n      * @param space Adds indentation, white space, and line break characters to the return-value JSON text to make it easier to read.\\n      */\\n    stringify(value: any, replacer?: (key: string, value: any) => any, space?: string | number): string;\\n    /**\\n      * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.\\n      * @param value A JavaScript value, usually an object or array, to be converted.\\n      * @param replacer An array of strings and numbers that acts as a approved list for selecting the object properties that will be stringified.\\n      * @param space Adds indentation, white space, and line break characters to the return-value JSON text to make it easier to read.\\n      */\\n    stringify(value: any, replacer?: (number | string)[] | null, space?: string | number): string;\\n}\\n\\n/**\\n  * An intrinsic object that provides functions to convert JavaScript values to and from the JavaScript Object Notation (JSON) format.\\n  */\\ndeclare const JSON: JSON;\\n\\n\\n/////////////////////////////\\n/// ECMAScript Array API (specially handled by compiler)\\n/////////////////////////////\\n\\ninterface ReadonlyArray<T> {\\n    /**\\n      * Gets the length of the array. This is a number one higher than the highest element defined in an array.\\n      */\\n    readonly length: number;\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n    /**\\n      * Returns a string representation of an array. The elements are converted to string using their toLocalString methods.\\n      */\\n    toLocaleString(): string;\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: ConcatArray<T>[]): T[];\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: (T | ConcatArray<T>)[]): T[];\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): T[];\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.\\n      */\\n    indexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Returns the index of the last occurrence of a specified value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at the last index in the array.\\n      */\\n    lastIndexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls the callbackfn function for each element in array1 until the callbackfn returns false, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the callbackfn function for each element in array1 until the callbackfn returns true, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => void, thisArg?: any): void;\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map<U>(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => U, thisArg?: any): U[];\\n    /**\\n     * Returns the elements of an array that meet the condition specified in a callback function.\\n     * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    filter<S extends T>(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => value is S, thisArg?: any): S[];\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: T, index: number, array: ReadonlyArray<T>) => any, thisArg?: any): T[];\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T): T;\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => U, initialValue: U): U;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T): T;\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: ReadonlyArray<T>) => U, initialValue: U): U;\\n\\n    readonly [n: number]: T;\\n}\\n\\ninterface ConcatArray<T> {\\n    readonly length: number;\\n    readonly [n: number]: T;\\n    join(separator?: string): string;\\n    slice(start?: number, end?: number): T[];\\n}\\n\\ninterface Array<T> {\\n    /**\\n      * Gets or sets the length of the array. This is a number one higher than the highest element defined in an array.\\n      */\\n    length: number;\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n    /**\\n      * Returns a string representation of an array. The elements are converted to string using their toLocalString methods.\\n      */\\n    toLocaleString(): string;\\n    /**\\n      * Removes the last element from an array and returns it.\\n      */\\n    pop(): T | undefined;\\n    /**\\n      * Appends new elements to an array, and returns the new length of the array.\\n      * @param items New elements of the Array.\\n      */\\n    push(...items: T[]): number;\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: ConcatArray<T>[]): T[];\\n    /**\\n      * Combines two or more arrays.\\n      * @param items Additional items to add to the end of array1.\\n      */\\n    concat(...items: (T | ConcatArray<T>)[]): T[];\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): T[];\\n    /**\\n      * Removes the first element from an array and returns it.\\n      */\\n    shift(): T | undefined;\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): T[];\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: T, b: T) => number): this;\\n    /**\\n      * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.\\n      * @param start The zero-based location in the array from which to start removing elements.\\n      * @param deleteCount The number of elements to remove.\\n      */\\n    splice(start: number, deleteCount?: number): T[];\\n    /**\\n      * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.\\n      * @param start The zero-based location in the array from which to start removing elements.\\n      * @param deleteCount The number of elements to remove.\\n      * @param items Elements to insert into the array in place of the deleted elements.\\n      */\\n    splice(start: number, deleteCount: number, ...items: T[]): T[];\\n    /**\\n      * Inserts new elements at the start of an array.\\n      * @param items  Elements to insert at the start of the Array.\\n      */\\n    unshift(...items: T[]): number;\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.\\n      */\\n    indexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Returns the index of the last occurrence of a specified value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at the last index in the array.\\n      */\\n    lastIndexOf(searchElement: T, fromIndex?: number): number;\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls the callbackfn function for each element in array1 until the callbackfn returns false, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: T, index: number, array: T[]) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the callbackfn function for each element in array1 until the callbackfn returns true, or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: T, index: number, array: T[]) => boolean, thisArg?: any): boolean;\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: T, index: number, array: T[]) => void, thisArg?: any): void;\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map<U>(callbackfn: (value: T, index: number, array: T[]) => U, thisArg?: any): U[];\\n    /**\\n     * Returns the elements of an array that meet the condition specified in a callback function.\\n     * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    filter<S extends T>(callbackfn: (value: T, index: number, array: T[]) => value is S, thisArg?: any): S[];\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: T, index: number, array: T[]) => any, thisArg?: any): T[];\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T): T;\\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): U;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T): T;\\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue: T): T;\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): U;\\n\\n    [n: number]: T;\\n}\\n\\ninterface ArrayConstructor {\\n    new(arrayLength?: number): any[];\\n    new <T>(arrayLength: number): T[];\\n    new <T>(...items: T[]): T[];\\n    (arrayLength?: number): any[];\\n    <T>(arrayLength: number): T[];\\n    <T>(...items: T[]): T[];\\n    isArray(arg: any): arg is Array<any>;\\n    readonly prototype: Array<any>;\\n}\\n\\ndeclare const Array: ArrayConstructor;\\n\\ninterface TypedPropertyDescriptor<T> {\\n    enumerable?: boolean;\\n    configurable?: boolean;\\n    writable?: boolean;\\n    value?: T;\\n    get?: () => T;\\n    set?: (value: T) => void;\\n}\\n\\ndeclare type ClassDecorator = <TFunction extends Function>(target: TFunction) => TFunction | void;\\ndeclare type PropertyDecorator = (target: Object, propertyKey: string | symbol) => void;\\ndeclare type MethodDecorator = <T>(target: Object, propertyKey: string | symbol, descriptor: TypedPropertyDescriptor<T>) => TypedPropertyDescriptor<T> | void;\\ndeclare type ParameterDecorator = (target: Object, propertyKey: string | symbol, parameterIndex: number) => void;\\n\\ndeclare type PromiseConstructorLike = new <T>(executor: (resolve: (value?: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void) => PromiseLike<T>;\\n\\ninterface PromiseLike<T> {\\n    /**\\n     * Attaches callbacks for the resolution and/or rejection of the Promise.\\n     * @param onfulfilled The callback to execute when the Promise is resolved.\\n     * @param onrejected The callback to execute when the Promise is rejected.\\n     * @returns A Promise for the completion of which ever callback is executed.\\n     */\\n    then<TResult1 = T, TResult2 = never>(onfulfilled?: ((value: T) => TResult1 | PromiseLike<TResult1>) | undefined | null, onrejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | undefined | null): PromiseLike<TResult1 | TResult2>;\\n}\\n\\n/**\\n * Represents the completion of an asynchronous operation\\n */\\ninterface Promise<T> {\\n    /**\\n     * Attaches callbacks for the resolution and/or rejection of the Promise.\\n     * @param onfulfilled The callback to execute when the Promise is resolved.\\n     * @param onrejected The callback to execute when the Promise is rejected.\\n     * @returns A Promise for the completion of which ever callback is executed.\\n     */\\n    then<TResult1 = T, TResult2 = never>(onfulfilled?: ((value: T) => TResult1 | PromiseLike<TResult1>) | undefined | null, onrejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | undefined | null): Promise<TResult1 | TResult2>;\\n\\n    /**\\n     * Attaches a callback for only the rejection of the Promise.\\n     * @param onrejected The callback to execute when the Promise is rejected.\\n     * @returns A Promise for the completion of the callback.\\n     */\\n    catch<TResult = never>(onrejected?: ((reason: any) => TResult | PromiseLike<TResult>) | undefined | null): Promise<T | TResult>;\\n}\\n\\ninterface ArrayLike<T> {\\n    readonly length: number;\\n    readonly [n: number]: T;\\n}\\n\\n/**\\n * Make all properties in T optional\\n */\\ntype Partial<T> = {\\n    [P in keyof T]?: T[P];\\n};\\n\\n/**\\n * Make all properties in T required\\n */\\ntype Required<T> = {\\n    [P in keyof T]-?: T[P];\\n};\\n\\n/**\\n * Make all properties in T readonly\\n */\\ntype Readonly<T> = {\\n    readonly [P in keyof T]: T[P];\\n};\\n\\n/**\\n * From T pick a set of properties K\\n */\\ntype Pick<T, K extends keyof T> = {\\n    [P in K]: T[P];\\n};\\n\\n/**\\n * Construct a type with a set of properties K of type T\\n */\\ntype Record<K extends keyof any, T> = {\\n    [P in K]: T;\\n};\\n\\n/**\\n * Exclude from T those types that are assignable to U\\n */\\ntype Exclude<T, U> = T extends U ? never : T;\\n\\n/**\\n * Extract from T those types that are assignable to U\\n */\\ntype Extract<T, U> = T extends U ? T : never;\\n\\n/**\\n * Exclude null and undefined from T\\n */\\ntype NonNullable<T> = T extends null | undefined ? never : T;\\n\\n/**\\n * Obtain the parameters of a function type in a tuple\\n */\\ntype Parameters<T extends (...args: any[]) => any> = T extends (...args: infer P) => any ? P : never;\\n\\n/**\\n * Obtain the parameters of a constructor function type in a tuple\\n */\\ntype ConstructorParameters<T extends new (...args: any[]) => any> = T extends new (...args: infer P) => any ? P : never;\\n\\n/**\\n * Obtain the return type of a function type\\n */\\ntype ReturnType<T extends (...args: any[]) => any> = T extends (...args: any[]) => infer R ? R : any;\\n\\n/**\\n * Obtain the return type of a constructor function type\\n */\\ntype InstanceType<T extends new (...args: any[]) => any> = T extends new (...args: any[]) => infer R ? R : any;\\n\\n/**\\n * Marker for contextual \\'this\\' type\\n */\\ninterface ThisType<T> { }\\n\\n/**\\n  * Represents a raw buffer of binary data, which is used to store data for the\\n  * different typed arrays. ArrayBuffers cannot be read from or written to directly,\\n  * but can be passed to a typed array or DataView Object to interpret the raw\\n  * buffer as needed.\\n  */\\ninterface ArrayBuffer {\\n    /**\\n      * Read-only. The length of the ArrayBuffer (in bytes).\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * Returns a section of an ArrayBuffer.\\n      */\\n    slice(begin: number, end?: number): ArrayBuffer;\\n}\\n\\n/**\\n * Allowed ArrayBuffer types for the buffer of an ArrayBufferView and related Typed Arrays.\\n */\\ninterface ArrayBufferTypes {\\n    ArrayBuffer: ArrayBuffer;\\n}\\ntype ArrayBufferLike = ArrayBufferTypes[keyof ArrayBufferTypes];\\n\\ninterface ArrayBufferConstructor {\\n    readonly prototype: ArrayBuffer;\\n    new(byteLength: number): ArrayBuffer;\\n    isView(arg: any): arg is ArrayBufferView;\\n}\\ndeclare const ArrayBuffer: ArrayBufferConstructor;\\n\\ninterface ArrayBufferView {\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    byteOffset: number;\\n}\\n\\ninterface DataView {\\n    readonly buffer: ArrayBuffer;\\n    readonly byteLength: number;\\n    readonly byteOffset: number;\\n    /**\\n      * Gets the Float32 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getFloat32(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Float64 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getFloat64(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Int8 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getInt8(byteOffset: number): number;\\n\\n    /**\\n      * Gets the Int16 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getInt16(byteOffset: number, littleEndian?: boolean): number;\\n    /**\\n      * Gets the Int32 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getInt32(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Uint8 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getUint8(byteOffset: number): number;\\n\\n    /**\\n      * Gets the Uint16 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getUint16(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Gets the Uint32 value at the specified byte offset from the start of the view. There is\\n      * no alignment constraint; multi-byte values may be fetched from any offset.\\n      * @param byteOffset The place in the buffer at which the value should be retrieved.\\n      */\\n    getUint32(byteOffset: number, littleEndian?: boolean): number;\\n\\n    /**\\n      * Stores an Float32 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setFloat32(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Float64 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setFloat64(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Int8 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      */\\n    setInt8(byteOffset: number, value: number): void;\\n\\n    /**\\n      * Stores an Int16 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setInt16(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Int32 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setInt32(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Uint8 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      */\\n    setUint8(byteOffset: number, value: number): void;\\n\\n    /**\\n      * Stores an Uint16 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setUint16(byteOffset: number, value: number, littleEndian?: boolean): void;\\n\\n    /**\\n      * Stores an Uint32 value at the specified byte offset from the start of the view.\\n      * @param byteOffset The place in the buffer at which the value should be set.\\n      * @param value The value to set.\\n      * @param littleEndian If false or undefined, a big-endian value should be written,\\n      * otherwise a little-endian value should be written.\\n      */\\n    setUint32(byteOffset: number, value: number, littleEndian?: boolean): void;\\n}\\n\\ninterface DataViewConstructor {\\n    new(buffer: ArrayBufferLike, byteOffset?: number, byteLength?: number): DataView;\\n}\\ndeclare const DataView: DataViewConstructor;\\n\\n/**\\n  * A typed array of 8-bit integer values. The contents are initialized to 0. If the requested\\n  * number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Int8Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Int8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Int8Array) => any, thisArg?: any): Int8Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Int8Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Int8Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Int8Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Int8Array) => number, thisArg?: any): Int8Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Int8Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Int8Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Int8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Int8Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Int8Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\ninterface Int8ArrayConstructor {\\n    readonly prototype: Int8Array;\\n    new(length: number): Int8Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Int8Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Int8Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Int8Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int8Array;\\n\\n\\n}\\ndeclare const Int8Array: Int8ArrayConstructor;\\n\\n/**\\n  * A typed array of 8-bit unsigned integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint8Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint8Array) => any, thisArg?: any): Uint8Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint8Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint8Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint8Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint8Array) => number, thisArg?: any): Uint8Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint8Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint8Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint8Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint8Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint8Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint8ArrayConstructor {\\n    readonly prototype: Uint8Array;\\n    new(length: number): Uint8Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint8Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint8Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint8Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8Array;\\n\\n}\\ndeclare const Uint8Array: Uint8ArrayConstructor;\\n\\n/**\\n  * A typed array of 8-bit unsigned integer (clamped) values. The contents are initialized to 0.\\n  * If the requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint8ClampedArray {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => any, thisArg?: any): Uint8ClampedArray;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint8ClampedArray) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint8ClampedArray) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => number, thisArg?: any): Uint8ClampedArray;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint8ClampedArray;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint8ClampedArray;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint8ClampedArray view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint8ClampedArray;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint8ClampedArrayConstructor {\\n    readonly prototype: Uint8ClampedArray;\\n    new(length: number): Uint8ClampedArray;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint8ClampedArray;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint8ClampedArray;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint8ClampedArray;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8ClampedArray;\\n}\\ndeclare const Uint8ClampedArray: Uint8ClampedArrayConstructor;\\n\\n/**\\n  * A typed array of 16-bit signed integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Int16Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Int16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Int16Array) => any, thisArg?: any): Int16Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Int16Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Int16Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Int16Array) => void, thisArg?: any): void;\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Int16Array) => number, thisArg?: any): Int16Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Int16Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Int16Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Int16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Int16Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Int16Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Int16ArrayConstructor {\\n    readonly prototype: Int16Array;\\n    new(length: number): Int16Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Int16Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Int16Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Int16Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int16Array;\\n\\n\\n}\\ndeclare const Int16Array: Int16ArrayConstructor;\\n\\n/**\\n  * A typed array of 16-bit unsigned integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint16Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint16Array) => any, thisArg?: any): Uint16Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint16Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint16Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint16Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint16Array) => number, thisArg?: any): Uint16Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint16Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint16Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint16Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint16Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint16Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint16Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint16ArrayConstructor {\\n    readonly prototype: Uint16Array;\\n    new(length: number): Uint16Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint16Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint16Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint16Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint16Array;\\n\\n\\n}\\ndeclare const Uint16Array: Uint16ArrayConstructor;\\n/**\\n  * A typed array of 32-bit signed integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Int32Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Int32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Int32Array) => any, thisArg?: any): Int32Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Int32Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Int32Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Int32Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Int32Array) => number, thisArg?: any): Int32Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Int32Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Int32Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Int32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Int32Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Int32Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Int32ArrayConstructor {\\n    readonly prototype: Int32Array;\\n    new(length: number): Int32Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Int32Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Int32Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Int32Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int32Array;\\n\\n}\\ndeclare const Int32Array: Int32ArrayConstructor;\\n\\n/**\\n  * A typed array of 32-bit unsigned integer values. The contents are initialized to 0. If the\\n  * requested number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Uint32Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Uint32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Uint32Array) => any, thisArg?: any): Uint32Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Uint32Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Uint32Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Uint32Array) => void, thisArg?: any): void;\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Uint32Array) => number, thisArg?: any): Uint32Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Uint32Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Uint32Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Uint32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Uint32Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Uint32Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Uint32ArrayConstructor {\\n    readonly prototype: Uint32Array;\\n    new(length: number): Uint32Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Uint32Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Uint32Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Uint32Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint32Array;\\n\\n}\\ndeclare const Uint32Array: Uint32ArrayConstructor;\\n\\n/**\\n  * A typed array of 32-bit float values. The contents are initialized to 0. If the requested number\\n  * of bytes could not be allocated an exception is raised.\\n  */\\ninterface Float32Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Float32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Float32Array) => any, thisArg?: any): Float32Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Float32Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Float32Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Float32Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Float32Array) => number, thisArg?: any): Float32Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float32Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Float32Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Float32Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Float32Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Float32Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Float32Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Float32ArrayConstructor {\\n    readonly prototype: Float32Array;\\n    new(length: number): Float32Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Float32Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Float32Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Float32Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float32Array;\\n\\n\\n}\\ndeclare const Float32Array: Float32ArrayConstructor;\\n\\n/**\\n  * A typed array of 64-bit float values. The contents are initialized to 0. If the requested\\n  * number of bytes could not be allocated an exception is raised.\\n  */\\ninterface Float64Array {\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * The ArrayBuffer instance referenced by the array.\\n      */\\n    readonly buffer: ArrayBufferLike;\\n\\n    /**\\n      * The length in bytes of the array.\\n      */\\n    readonly byteLength: number;\\n\\n    /**\\n      * The offset in bytes of the array.\\n      */\\n    readonly byteOffset: number;\\n\\n    /**\\n      * Returns the this object after copying a section of the array identified by start and end\\n      * to the same array starting at position target\\n      * @param target If target is negative, it is treated as length+target where length is the\\n      * length of the array.\\n      * @param start If start is negative, it is treated as length+start. If end is negative, it\\n      * is treated as length+end.\\n      * @param end If not specified, length of the this object is used as its default value.\\n      */\\n    copyWithin(target: number, start: number, end?: number): this;\\n\\n    /**\\n      * Determines whether all the members of an array satisfy the specified test.\\n      * @param callbackfn A function that accepts up to three arguments. The every method calls\\n      * the callbackfn function for each element in array1 until the callbackfn returns false,\\n      * or until the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    every(callbackfn: (value: number, index: number, array: Float64Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n        * Returns the this object after filling the section identified by start and end with value\\n        * @param value value to fill array section with\\n        * @param start index to start filling the array at. If start is negative, it is treated as\\n        * length+start where length is the length of the array.\\n        * @param end index to stop filling the array at. If end is negative, it is treated as\\n        * length+end.\\n        */\\n    fill(value: number, start?: number, end?: number): this;\\n\\n    /**\\n      * Returns the elements of an array that meet the condition specified in a callback function.\\n      * @param callbackfn A function that accepts up to three arguments. The filter method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    filter(callbackfn: (value: number, index: number, array: Float64Array) => any, thisArg?: any): Float64Array;\\n\\n    /**\\n      * Returns the value of the first element in the array where predicate is true, and undefined\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found, find\\n      * immediately returns that element value. Otherwise, find returns undefined.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    find(predicate: (value: number, index: number, obj: Float64Array) => boolean, thisArg?: any): number | undefined;\\n\\n    /**\\n      * Returns the index of the first element in the array where predicate is true, and -1\\n      * otherwise.\\n      * @param predicate find calls predicate once for each element of the array, in ascending\\n      * order, until it finds one where predicate returns true. If such an element is found,\\n      * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n      * @param thisArg If provided, it will be used as the this value for each invocation of\\n      * predicate. If it is not provided, undefined is used instead.\\n      */\\n    findIndex(predicate: (value: number, index: number, obj: Float64Array) => boolean, thisArg?: any): number;\\n\\n    /**\\n      * Performs the specified action for each element in an array.\\n      * @param callbackfn  A function that accepts up to three arguments. forEach calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    forEach(callbackfn: (value: number, index: number, array: Float64Array) => void, thisArg?: any): void;\\n\\n    /**\\n      * Returns the index of the first occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      *  search starts at index 0.\\n      */\\n    indexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * Adds all the elements of an array separated by the specified separator string.\\n      * @param separator A string used to separate one element of an array from the next in the\\n      * resulting String. If omitted, the array elements are separated with a comma.\\n      */\\n    join(separator?: string): string;\\n\\n    /**\\n      * Returns the index of the last occurrence of a value in an array.\\n      * @param searchElement The value to locate in the array.\\n      * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\\n      * search starts at index 0.\\n      */\\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\\n\\n    /**\\n      * The length of the array.\\n      */\\n    readonly length: number;\\n\\n    /**\\n      * Calls a defined callback function on each element of an array, and returns an array that\\n      * contains the results.\\n      * @param callbackfn A function that accepts up to three arguments. The map method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    map(callbackfn: (value: number, index: number, array: Float64Array) => number, thisArg?: any): Float64Array;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number): number;\\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array. The return value of\\n      * the callback function is the accumulated result, and is provided as an argument in the next\\n      * call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\\n      * callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float64Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an\\n      * argument instead of an array value.\\n      */\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number): number;\\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number, initialValue: number): number;\\n\\n    /**\\n      * Calls the specified callback function for all the elements in an array, in descending order.\\n      * The return value of the callback function is the accumulated result, and is provided as an\\n      * argument in the next call to the callback function.\\n      * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\\n      * the callbackfn function one time for each element in the array.\\n      * @param initialValue If initialValue is specified, it is used as the initial value to start\\n      * the accumulation. The first call to the callbackfn function provides this value as an argument\\n      * instead of an array value.\\n      */\\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float64Array) => U, initialValue: U): U;\\n\\n    /**\\n      * Reverses the elements in an Array.\\n      */\\n    reverse(): Float64Array;\\n\\n    /**\\n      * Sets a value or an array of values.\\n      * @param array A typed or untyped array of values to set.\\n      * @param offset The index in the current array at which the values are to be written.\\n      */\\n    set(array: ArrayLike<number>, offset?: number): void;\\n\\n    /**\\n      * Returns a section of an array.\\n      * @param start The beginning of the specified portion of the array.\\n      * @param end The end of the specified portion of the array.\\n      */\\n    slice(start?: number, end?: number): Float64Array;\\n\\n    /**\\n      * Determines whether the specified callback function returns true for any element of an array.\\n      * @param callbackfn A function that accepts up to three arguments. The some method calls the\\n      * callbackfn function for each element in array1 until the callbackfn returns true, or until\\n      * the end of the array.\\n      * @param thisArg An object to which the this keyword can refer in the callbackfn function.\\n      * If thisArg is omitted, undefined is used as the this value.\\n      */\\n    some(callbackfn: (value: number, index: number, array: Float64Array) => boolean, thisArg?: any): boolean;\\n\\n    /**\\n      * Sorts an array.\\n      * @param compareFn The name of the function used to determine the order of the elements. If\\n      * omitted, the elements are sorted in ascending, ASCII character order.\\n      */\\n    sort(compareFn?: (a: number, b: number) => number): this;\\n\\n    /**\\n      * Gets a new Float64Array view of the ArrayBuffer store for this array, referencing the elements\\n      * at begin, inclusive, up to end, exclusive.\\n      * @param begin The index of the beginning of the array.\\n      * @param end The index of the end of the array.\\n      */\\n    subarray(begin: number, end?: number): Float64Array;\\n\\n    /**\\n      * Converts a number to a string by using the current locale.\\n      */\\n    toLocaleString(): string;\\n\\n    /**\\n      * Returns a string representation of an array.\\n      */\\n    toString(): string;\\n\\n    [index: number]: number;\\n}\\n\\ninterface Float64ArrayConstructor {\\n    readonly prototype: Float64Array;\\n    new(length: number): Float64Array;\\n    new(arrayOrArrayBuffer: ArrayLike<number> | ArrayBufferLike): Float64Array;\\n    new(buffer: ArrayBufferLike, byteOffset: number, length?: number): Float64Array;\\n\\n    /**\\n      * The size in bytes of each element in the array.\\n      */\\n    readonly BYTES_PER_ELEMENT: number;\\n\\n    /**\\n      * Returns a new array from a set of elements.\\n      * @param items A set of elements to include in the new array object.\\n      */\\n    of(...items: number[]): Float64Array;\\n\\n    /**\\n      * Creates an array from an array-like or iterable object.\\n      * @param arrayLike An array-like or iterable object to convert to an array.\\n      * @param mapfn A mapping function to call on every element of the array.\\n      * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n      */\\n    from(arrayLike: ArrayLike<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float64Array;\\n\\n}\\ndeclare const Float64Array: Float64ArrayConstructor;\\n\\n/////////////////////////////\\n/// ECMAScript Internationalization API\\n/////////////////////////////\\n\\ndeclare namespace Intl {\\n    interface CollatorOptions {\\n        usage?: string;\\n        localeMatcher?: string;\\n        numeric?: boolean;\\n        caseFirst?: string;\\n        sensitivity?: string;\\n        ignorePunctuation?: boolean;\\n    }\\n\\n    interface ResolvedCollatorOptions {\\n        locale: string;\\n        usage: string;\\n        sensitivity: string;\\n        ignorePunctuation: boolean;\\n        collation: string;\\n        caseFirst: string;\\n        numeric: boolean;\\n    }\\n\\n    interface Collator {\\n        compare(x: string, y: string): number;\\n        resolvedOptions(): ResolvedCollatorOptions;\\n    }\\n    var Collator: {\\n        new(locales?: string | string[], options?: CollatorOptions): Collator;\\n        (locales?: string | string[], options?: CollatorOptions): Collator;\\n        supportedLocalesOf(locales: string | string[], options?: CollatorOptions): string[];\\n    };\\n\\n    interface NumberFormatOptions {\\n        localeMatcher?: string;\\n        style?: string;\\n        currency?: string;\\n        currencyDisplay?: string;\\n        useGrouping?: boolean;\\n        minimumIntegerDigits?: number;\\n        minimumFractionDigits?: number;\\n        maximumFractionDigits?: number;\\n        minimumSignificantDigits?: number;\\n        maximumSignificantDigits?: number;\\n    }\\n\\n    interface ResolvedNumberFormatOptions {\\n        locale: string;\\n        numberingSystem: string;\\n        style: string;\\n        currency?: string;\\n        currencyDisplay?: string;\\n        minimumIntegerDigits: number;\\n        minimumFractionDigits: number;\\n        maximumFractionDigits: number;\\n        minimumSignificantDigits?: number;\\n        maximumSignificantDigits?: number;\\n        useGrouping: boolean;\\n    }\\n\\n    interface NumberFormat {\\n        format(value: number): string;\\n        resolvedOptions(): ResolvedNumberFormatOptions;\\n    }\\n    var NumberFormat: {\\n        new(locales?: string | string[], options?: NumberFormatOptions): NumberFormat;\\n        (locales?: string | string[], options?: NumberFormatOptions): NumberFormat;\\n        supportedLocalesOf(locales: string | string[], options?: NumberFormatOptions): string[];\\n    };\\n\\n    interface DateTimeFormatOptions {\\n        localeMatcher?: string;\\n        weekday?: string;\\n        era?: string;\\n        year?: string;\\n        month?: string;\\n        day?: string;\\n        hour?: string;\\n        minute?: string;\\n        second?: string;\\n        timeZoneName?: string;\\n        formatMatcher?: string;\\n        hour12?: boolean;\\n        timeZone?: string;\\n    }\\n\\n    interface ResolvedDateTimeFormatOptions {\\n        locale: string;\\n        calendar: string;\\n        numberingSystem: string;\\n        timeZone: string;\\n        hour12?: boolean;\\n        weekday?: string;\\n        era?: string;\\n        year?: string;\\n        month?: string;\\n        day?: string;\\n        hour?: string;\\n        minute?: string;\\n        second?: string;\\n        timeZoneName?: string;\\n    }\\n\\n    interface DateTimeFormat {\\n        format(date?: Date | number): string;\\n        resolvedOptions(): ResolvedDateTimeFormatOptions;\\n    }\\n    var DateTimeFormat: {\\n        new(locales?: string | string[], options?: DateTimeFormatOptions): DateTimeFormat;\\n        (locales?: string | string[], options?: DateTimeFormatOptions): DateTimeFormat;\\n        supportedLocalesOf(locales: string | string[], options?: DateTimeFormatOptions): string[];\\n    };\\n}\\n\\ninterface String {\\n    /**\\n      * Determines whether two strings are equivalent in the current or specified locale.\\n      * @param that String to compare to target string\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used. This parameter must conform to BCP 47 standards; see the Intl.Collator object for details.\\n      * @param options An object that contains one or more properties that specify comparison options. see the Intl.Collator object for details.\\n      */\\n    localeCompare(that: string, locales?: string | string[], options?: Intl.CollatorOptions): number;\\n}\\n\\ninterface Number {\\n    /**\\n      * Converts a number to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleString(locales?: string | string[], options?: Intl.NumberFormatOptions): string;\\n}\\n\\ninterface Date {\\n    /**\\n      * Converts a date and time to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\\n    /**\\n      * Converts a date to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleDateString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\\n\\n    /**\\n      * Converts a time to a string by using the current or specified locale.\\n      * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\\n      * @param options An object that contains one or more properties that specify comparison options.\\n      */\\n    toLocaleTimeString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\\n}\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface Array<T> {\\n    /**\\n     * Returns the value of the first element in the array where predicate is true, and undefined\\n     * otherwise.\\n     * @param predicate find calls predicate once for each element of the array, in ascending\\n     * order, until it finds one where predicate returns true. If such an element is found, find\\n     * immediately returns that element value. Otherwise, find returns undefined.\\n     * @param thisArg If provided, it will be used as the this value for each invocation of\\n     * predicate. If it is not provided, undefined is used instead.\\n     */\\n    find<S extends T>(predicate: (this: void, value: T, index: number, obj: T[]) => value is S, thisArg?: any): S | undefined;\\n    find(predicate: (value: T, index: number, obj: T[]) => boolean, thisArg?: any): T | undefined;\\n\\n    /**\\n     * Returns the index of the first element in the array where predicate is true, and -1\\n     * otherwise.\\n     * @param predicate find calls predicate once for each element of the array, in ascending\\n     * order, until it finds one where predicate returns true. If such an element is found,\\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n     * @param thisArg If provided, it will be used as the this value for each invocation of\\n     * predicate. If it is not provided, undefined is used instead.\\n     */\\n    findIndex(predicate: (value: T, index: number, obj: T[]) => boolean, thisArg?: any): number;\\n\\n    /**\\n     * Returns the this object after filling the section identified by start and end with value\\n     * @param value value to fill array section with\\n     * @param start index to start filling the array at. If start is negative, it is treated as\\n     * length+start where length is the length of the array.\\n     * @param end index to stop filling the array at. If end is negative, it is treated as\\n     * length+end.\\n     */\\n    fill(value: T, start?: number, end?: number): this;\\n\\n    /**\\n     * Returns the this object after copying a section of the array identified by start and end\\n     * to the same array starting at position target\\n     * @param target If target is negative, it is treated as length+target where length is the\\n     * length of the array.\\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\\n     * is treated as length+end.\\n     * @param end If not specified, length of the this object is used as its default value.\\n     */\\n    copyWithin(target: number, start: number, end?: number): this;\\n}\\n\\ninterface ArrayConstructor {\\n    /**\\n     * Creates an array from an array-like object.\\n     * @param arrayLike An array-like object to convert to an array.\\n     */\\n    from<T>(arrayLike: ArrayLike<T>): T[];\\n\\n    /**\\n     * Creates an array from an iterable object.\\n     * @param arrayLike An array-like object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from<T, U>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => U, thisArg?: any): U[];\\n\\n    /**\\n     * Returns a new array from a set of elements.\\n     * @param items A set of elements to include in the new array object.\\n     */\\n    of<T>(...items: T[]): T[];\\n}\\n\\ninterface DateConstructor {\\n    new (value: number | string | Date): Date;\\n}\\n\\ninterface Function {\\n    /**\\n     * Returns the name of the function. Function names are read-only and can not be changed.\\n     */\\n    readonly name: string;\\n}\\n\\ninterface Math {\\n    /**\\n     * Returns the number of leading zero bits in the 32-bit binary representation of a number.\\n     * @param x A numeric expression.\\n     */\\n    clz32(x: number): number;\\n\\n    /**\\n     * Returns the result of 32-bit multiplication of two numbers.\\n     * @param x First number\\n     * @param y Second number\\n     */\\n    imul(x: number, y: number): number;\\n\\n    /**\\n     * Returns the sign of the x, indicating whether x is positive, negative or zero.\\n     * @param x The numeric expression to test\\n     */\\n    sign(x: number): number;\\n\\n    /**\\n     * Returns the base 10 logarithm of a number.\\n     * @param x A numeric expression.\\n     */\\n    log10(x: number): number;\\n\\n    /**\\n     * Returns the base 2 logarithm of a number.\\n     * @param x A numeric expression.\\n     */\\n    log2(x: number): number;\\n\\n    /**\\n     * Returns the natural logarithm of 1 + x.\\n     * @param x A numeric expression.\\n     */\\n    log1p(x: number): number;\\n\\n    /**\\n     * Returns the result of (e^x - 1), which is an implementation-dependent approximation to\\n     * subtracting 1 from the exponential function of x (e raised to the power of x, where e\\n     * is the base of the natural logarithms).\\n     * @param x A numeric expression.\\n     */\\n    expm1(x: number): number;\\n\\n    /**\\n     * Returns the hyperbolic cosine of a number.\\n     * @param x A numeric expression that contains an angle measured in radians.\\n     */\\n    cosh(x: number): number;\\n\\n    /**\\n     * Returns the hyperbolic sine of a number.\\n     * @param x A numeric expression that contains an angle measured in radians.\\n     */\\n    sinh(x: number): number;\\n\\n    /**\\n     * Returns the hyperbolic tangent of a number.\\n     * @param x A numeric expression that contains an angle measured in radians.\\n     */\\n    tanh(x: number): number;\\n\\n    /**\\n     * Returns the inverse hyperbolic cosine of a number.\\n     * @param x A numeric expression that contains an angle measured in radians.\\n     */\\n    acosh(x: number): number;\\n\\n    /**\\n     * Returns the inverse hyperbolic sine of a number.\\n     * @param x A numeric expression that contains an angle measured in radians.\\n     */\\n    asinh(x: number): number;\\n\\n    /**\\n     * Returns the inverse hyperbolic tangent of a number.\\n     * @param x A numeric expression that contains an angle measured in radians.\\n     */\\n    atanh(x: number): number;\\n\\n    /**\\n     * Returns the square root of the sum of squares of its arguments.\\n     * @param values Values to compute the square root for.\\n     *     If no arguments are passed, the result is +0.\\n     *     If there is only one argument, the result is the absolute value.\\n     *     If any argument is +Infinity or -Infinity, the result is +Infinity.\\n     *     If any argument is NaN, the result is NaN.\\n     *     If all arguments are either +0 or −0, the result is +0.\\n     */\\n    hypot(...values: number[]): number;\\n\\n    /**\\n     * Returns the integral part of the a numeric expression, x, removing any fractional digits.\\n     * If x is already an integer, the result is x.\\n     * @param x A numeric expression.\\n     */\\n    trunc(x: number): number;\\n\\n    /**\\n     * Returns the nearest single precision float representation of a number.\\n     * @param x A numeric expression.\\n     */\\n    fround(x: number): number;\\n\\n    /**\\n     * Returns an implementation-dependent approximation to the cube root of number.\\n     * @param x A numeric expression.\\n     */\\n    cbrt(x: number): number;\\n}\\n\\ninterface NumberConstructor {\\n    /**\\n     * The value of Number.EPSILON is the difference between 1 and the smallest value greater than 1\\n     * that is representable as a Number value, which is approximately:\\n     * 2.2204460492503130808472633361816 x 10‍−‍16.\\n     */\\n    readonly EPSILON: number;\\n\\n    /**\\n     * Returns true if passed value is finite.\\n     * Unlike the global isFinite, Number.isFinite doesn\\'t forcibly convert the parameter to a\\n     * number. Only finite values of the type number, result in true.\\n     * @param number A numeric value.\\n     */\\n    isFinite(number: number): boolean;\\n\\n    /**\\n     * Returns true if the value passed is an integer, false otherwise.\\n     * @param number A numeric value.\\n     */\\n    isInteger(number: number): boolean;\\n\\n    /**\\n     * Returns a Boolean value that indicates whether a value is the reserved value NaN (not a\\n     * number). Unlike the global isNaN(), Number.isNaN() doesn\\'t forcefully convert the parameter\\n     * to a number. Only values of the type number, that are also NaN, result in true.\\n     * @param number A numeric value.\\n     */\\n    isNaN(number: number): boolean;\\n\\n    /**\\n     * Returns true if the value passed is a safe integer.\\n     * @param number A numeric value.\\n     */\\n    isSafeInteger(number: number): boolean;\\n\\n    /**\\n     * The value of the largest integer n such that n and n + 1 are both exactly representable as\\n     * a Number value.\\n     * The value of Number.MAX_SAFE_INTEGER is 9007199254740991 2^53 − 1.\\n     */\\n    readonly MAX_SAFE_INTEGER: number;\\n\\n    /**\\n     * The value of the smallest integer n such that n and n − 1 are both exactly representable as\\n     * a Number value.\\n     * The value of Number.MIN_SAFE_INTEGER is −9007199254740991 (−(2^53 − 1)).\\n     */\\n    readonly MIN_SAFE_INTEGER: number;\\n\\n    /**\\n     * Converts a string to a floating-point number.\\n     * @param string A string that contains a floating-point number.\\n     */\\n    parseFloat(string: string): number;\\n\\n    /**\\n     * Converts A string to an integer.\\n     * @param s A string to convert into a number.\\n     * @param radix A value between 2 and 36 that specifies the base of the number in numString.\\n     * If this argument is not supplied, strings with a prefix of \\'0x\\' are considered hexadecimal.\\n     * All other strings are considered decimal.\\n     */\\n    parseInt(string: string, radix?: number): number;\\n}\\n\\ninterface ObjectConstructor {\\n    /**\\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\\n     * target object. Returns the target object.\\n     * @param target The target object to copy to.\\n     * @param source The source object from which to copy properties.\\n     */\\n    assign<T, U>(target: T, source: U): T & U;\\n\\n    /**\\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\\n     * target object. Returns the target object.\\n     * @param target The target object to copy to.\\n     * @param source1 The first source object from which to copy properties.\\n     * @param source2 The second source object from which to copy properties.\\n     */\\n    assign<T, U, V>(target: T, source1: U, source2: V): T & U & V;\\n\\n    /**\\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\\n     * target object. Returns the target object.\\n     * @param target The target object to copy to.\\n     * @param source1 The first source object from which to copy properties.\\n     * @param source2 The second source object from which to copy properties.\\n     * @param source3 The third source object from which to copy properties.\\n     */\\n    assign<T, U, V, W>(target: T, source1: U, source2: V, source3: W): T & U & V & W;\\n\\n    /**\\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\\n     * target object. Returns the target object.\\n     * @param target The target object to copy to.\\n     * @param sources One or more source objects from which to copy properties\\n     */\\n    assign(target: object, ...sources: any[]): any;\\n\\n    /**\\n     * Returns an array of all symbol properties found directly on object o.\\n     * @param o Object to retrieve the symbols from.\\n     */\\n    getOwnPropertySymbols(o: any): symbol[];\\n\\n    /**\\n     * Returns true if the values are the same value, false otherwise.\\n     * @param value1 The first value.\\n     * @param value2 The second value.\\n     */\\n    is(value1: any, value2: any): boolean;\\n\\n    /**\\n     * Sets the prototype of a specified object o to  object proto or null. Returns the object o.\\n     * @param o The object to change its prototype.\\n     * @param proto The value of the new prototype or null.\\n     */\\n    setPrototypeOf(o: any, proto: object | null): any;\\n}\\n\\ninterface ReadonlyArray<T> {\\n    /**\\n     * Returns the value of the first element in the array where predicate is true, and undefined\\n     * otherwise.\\n     * @param predicate find calls predicate once for each element of the array, in ascending\\n     * order, until it finds one where predicate returns true. If such an element is found, find\\n     * immediately returns that element value. Otherwise, find returns undefined.\\n     * @param thisArg If provided, it will be used as the this value for each invocation of\\n     * predicate. If it is not provided, undefined is used instead.\\n     */\\n    find<S extends T>(predicate: (this: void, value: T, index: number, obj: ReadonlyArray<T>) => value is S, thisArg?: any): S | undefined;\\n    find(predicate: (value: T, index: number, obj: ReadonlyArray<T>) => boolean, thisArg?: any): T | undefined;\\n\\n    /**\\n     * Returns the index of the first element in the array where predicate is true, and -1\\n     * otherwise.\\n     * @param predicate find calls predicate once for each element of the array, in ascending\\n     * order, until it finds one where predicate returns true. If such an element is found,\\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\\n     * @param thisArg If provided, it will be used as the this value for each invocation of\\n     * predicate. If it is not provided, undefined is used instead.\\n     */\\n    findIndex(predicate: (value: T, index: number, obj: ReadonlyArray<T>) => boolean, thisArg?: any): number;\\n}\\n\\ninterface RegExp {\\n    /**\\n     * Returns a string indicating the flags of the regular expression in question. This field is read-only.\\n     * The characters in this string are sequenced and concatenated in the following order:\\n     *\\n     *    - \"g\" for global\\n     *    - \"i\" for ignoreCase\\n     *    - \"m\" for multiline\\n     *    - \"u\" for unicode\\n     *    - \"y\" for sticky\\n     *\\n     * If no flags are set, the value is the empty string.\\n     */\\n    readonly flags: string;\\n\\n    /**\\n     * Returns a Boolean value indicating the state of the sticky flag (y) used with a regular\\n     * expression. Default is false. Read-only.\\n     */\\n    readonly sticky: boolean;\\n\\n    /**\\n     * Returns a Boolean value indicating the state of the Unicode flag (u) used with a regular\\n     * expression. Default is false. Read-only.\\n     */\\n    readonly unicode: boolean;\\n}\\n\\ninterface RegExpConstructor {\\n    new (pattern: RegExp, flags?: string): RegExp;\\n    (pattern: RegExp, flags?: string): RegExp;\\n}\\n\\ninterface String {\\n    /**\\n     * Returns a nonnegative integer Number less than 1114112 (0x110000) that is the code point\\n     * value of the UTF-16 encoded code point starting at the string element at position pos in\\n     * the String resulting from converting this object to a String.\\n     * If there is no element at that position, the result is undefined.\\n     * If a valid UTF-16 surrogate pair does not begin at pos, the result is the code unit at pos.\\n     */\\n    codePointAt(pos: number): number | undefined;\\n\\n    /**\\n     * Returns true if searchString appears as a substring of the result of converting this\\n     * object to a String, at one or more positions that are\\n     * greater than or equal to position; otherwise, returns false.\\n     * @param searchString search string\\n     * @param position If position is undefined, 0 is assumed, so as to search all of the String.\\n     */\\n    includes(searchString: string, position?: number): boolean;\\n\\n    /**\\n     * Returns true if the sequence of elements of searchString converted to a String is the\\n     * same as the corresponding elements of this object (converted to a String) starting at\\n     * endPosition – length(this). Otherwise returns false.\\n     */\\n    endsWith(searchString: string, endPosition?: number): boolean;\\n\\n    /**\\n     * Returns the String value result of normalizing the string into the normalization form\\n     * named by form as specified in Unicode Standard Annex #15, Unicode Normalization Forms.\\n     * @param form Applicable values: \"NFC\", \"NFD\", \"NFKC\", or \"NFKD\", If not specified default\\n     * is \"NFC\"\\n     */\\n    normalize(form: \"NFC\" | \"NFD\" | \"NFKC\" | \"NFKD\"): string;\\n\\n    /**\\n     * Returns the String value result of normalizing the string into the normalization form\\n     * named by form as specified in Unicode Standard Annex #15, Unicode Normalization Forms.\\n     * @param form Applicable values: \"NFC\", \"NFD\", \"NFKC\", or \"NFKD\", If not specified default\\n     * is \"NFC\"\\n     */\\n    normalize(form?: string): string;\\n\\n    /**\\n     * Returns a String value that is made from count copies appended together. If count is 0,\\n     * the empty string is returned.\\n     * @param count number of copies to append\\n     */\\n    repeat(count: number): string;\\n\\n    /**\\n     * Returns true if the sequence of elements of searchString converted to a String is the\\n     * same as the corresponding elements of this object (converted to a String) starting at\\n     * position. Otherwise returns false.\\n     */\\n    startsWith(searchString: string, position?: number): boolean;\\n\\n    /**\\n     * Returns an <a> HTML anchor element and sets the name attribute to the text value\\n     * @param name\\n     */\\n    anchor(name: string): string;\\n\\n    /** Returns a <big> HTML element */\\n    big(): string;\\n\\n    /** Returns a <blink> HTML element */\\n    blink(): string;\\n\\n    /** Returns a <b> HTML element */\\n    bold(): string;\\n\\n    /** Returns a <tt> HTML element */\\n    fixed(): string;\\n\\n    /** Returns a <font> HTML element and sets the color attribute value */\\n    fontcolor(color: string): string;\\n\\n    /** Returns a <font> HTML element and sets the size attribute value */\\n    fontsize(size: number): string;\\n\\n    /** Returns a <font> HTML element and sets the size attribute value */\\n    fontsize(size: string): string;\\n\\n    /** Returns an <i> HTML element */\\n    italics(): string;\\n\\n    /** Returns an <a> HTML element and sets the href attribute value */\\n    link(url: string): string;\\n\\n    /** Returns a <small> HTML element */\\n    small(): string;\\n\\n    /** Returns a <strike> HTML element */\\n    strike(): string;\\n\\n    /** Returns a <sub> HTML element */\\n    sub(): string;\\n\\n    /** Returns a <sup> HTML element */\\n    sup(): string;\\n}\\n\\ninterface StringConstructor {\\n    /**\\n     * Return the String value whose elements are, in order, the elements in the List elements.\\n     * If length is 0, the empty string is returned.\\n     */\\n    fromCodePoint(...codePoints: number[]): string;\\n\\n    /**\\n     * String.raw is intended for use as a tag function of a Tagged Template String. When called\\n     * as such the first argument will be a well formed template call site object and the rest\\n     * parameter will contain the substitution values.\\n     * @param template A well-formed template string call site representation.\\n     * @param substitutions A set of substitution values.\\n     */\\n    raw(template: TemplateStringsArray, ...substitutions: any[]): string;\\n}\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface Map<K, V> {\\n    clear(): void;\\n    delete(key: K): boolean;\\n    forEach(callbackfn: (value: V, key: K, map: Map<K, V>) => void, thisArg?: any): void;\\n    get(key: K): V | undefined;\\n    has(key: K): boolean;\\n    set(key: K, value: V): this;\\n    readonly size: number;\\n}\\n\\ninterface MapConstructor {\\n    new <K = any, V = any>(entries?: ReadonlyArray<[K, V]> | null): Map<K, V>;\\n    readonly prototype: Map<any, any>;\\n}\\ndeclare var Map: MapConstructor;\\n\\ninterface ReadonlyMap<K, V> {\\n    forEach(callbackfn: (value: V, key: K, map: ReadonlyMap<K, V>) => void, thisArg?: any): void;\\n    get(key: K): V | undefined;\\n    has(key: K): boolean;\\n    readonly size: number;\\n}\\n\\ninterface WeakMap<K extends object, V> {\\n    delete(key: K): boolean;\\n    get(key: K): V | undefined;\\n    has(key: K): boolean;\\n    set(key: K, value: V): this;\\n}\\n\\ninterface WeakMapConstructor {\\n    new <K extends object = object, V = any>(entries?: ReadonlyArray<[K, V]> | null): WeakMap<K, V>;\\n    readonly prototype: WeakMap<object, any>;\\n}\\ndeclare var WeakMap: WeakMapConstructor;\\n\\ninterface Set<T> {\\n    add(value: T): this;\\n    clear(): void;\\n    delete(value: T): boolean;\\n    forEach(callbackfn: (value: T, value2: T, set: Set<T>) => void, thisArg?: any): void;\\n    has(value: T): boolean;\\n    readonly size: number;\\n}\\n\\ninterface SetConstructor {\\n    new <T = any>(values?: ReadonlyArray<T> | null): Set<T>;\\n    readonly prototype: Set<any>;\\n}\\ndeclare var Set: SetConstructor;\\n\\ninterface ReadonlySet<T> {\\n    forEach(callbackfn: (value: T, value2: T, set: ReadonlySet<T>) => void, thisArg?: any): void;\\n    has(value: T): boolean;\\n    readonly size: number;\\n}\\n\\ninterface WeakSet<T extends object> {\\n    add(value: T): this;\\n    delete(value: T): boolean;\\n    has(value: T): boolean;\\n}\\n\\ninterface WeakSetConstructor {\\n    new <T extends object = object>(values?: ReadonlyArray<T> | null): WeakSet<T>;\\n    readonly prototype: WeakSet<object>;\\n}\\ndeclare var WeakSet: WeakSetConstructor;\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface Generator extends Iterator<any> { }\\n\\ninterface GeneratorFunction {\\n    /**\\n     * Creates a new Generator object.\\n     * @param args A list of arguments the function accepts.\\n     */\\n    new (...args: any[]): Generator;\\n    /**\\n     * Creates a new Generator object.\\n     * @param args A list of arguments the function accepts.\\n     */\\n    (...args: any[]): Generator;\\n    /**\\n     * The length of the arguments.\\n     */\\n    readonly length: number;\\n    /**\\n     * Returns the name of the function.\\n     */\\n    readonly name: string;\\n    /**\\n     * A reference to the prototype.\\n     */\\n    readonly prototype: Generator;\\n}\\n\\ninterface GeneratorFunctionConstructor {\\n    /**\\n     * Creates a new Generator function.\\n     * @param args A list of arguments the function accepts.\\n     */\\n    new (...args: string[]): GeneratorFunction;\\n    /**\\n     * Creates a new Generator function.\\n     * @param args A list of arguments the function accepts.\\n     */\\n    (...args: string[]): GeneratorFunction;\\n    /**\\n     * The length of the arguments.\\n     */\\n    readonly length: number;\\n    /**\\n     * Returns the name of the function.\\n     */\\n    readonly name: string;\\n    /**\\n     * A reference to the prototype.\\n     */\\n    readonly prototype: GeneratorFunction;\\n}\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface PromiseConstructor {\\n    /**\\n     * A reference to the prototype.\\n     */\\n    readonly prototype: Promise<any>;\\n\\n    /**\\n     * Creates a new Promise.\\n     * @param executor A callback used to initialize the promise. This callback is passed two arguments:\\n     * a resolve callback used to resolve the promise with a value or the result of another promise,\\n     * and a reject callback used to reject the promise with a provided reason or error.\\n     */\\n    new <T>(executor: (resolve: (value?: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void): Promise<T>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike <T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>, T10 | PromiseLike<T10>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3, T4, T5, T6, T7, T8, T9>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike <T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8, T9]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3, T4, T5, T6, T7, T8>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike <T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3, T4, T5, T6, T7>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike <T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>]): Promise<[T1, T2, T3, T4, T5, T6, T7]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3, T4, T5, T6>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike <T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>]): Promise<[T1, T2, T3, T4, T5, T6]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3, T4, T5>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike <T4>, T5 | PromiseLike<T5>]): Promise<[T1, T2, T3, T4, T5]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3, T4>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike <T4>]): Promise<[T1, T2, T3, T4]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2, T3>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>]): Promise<[T1, T2, T3]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T1, T2>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>]): Promise<[T1, T2]>;\\n\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<T>(values: (T | PromiseLike<T>)[]): Promise<T[]>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>, T10 | PromiseLike<T10>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8 | T9 | T10>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3, T4, T5, T6, T7, T8, T9>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8 | T9>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3, T4, T5, T6, T7, T8>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3, T4, T5, T6, T7>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3, T4, T5, T6>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>]): Promise<T1 | T2 | T3 | T4 | T5 | T6>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3, T4, T5>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>]): Promise<T1 | T2 | T3 | T4 | T5>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3, T4>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>]): Promise<T1 | T2 | T3 | T4>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2, T3>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>]): Promise<T1 | T2 | T3>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T1, T2>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>]): Promise<T1 | T2>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T>(values: (T | PromiseLike<T>)[]): Promise<T>;\\n\\n    /**\\n     * Creates a new rejected promise for the provided reason.\\n     * @param reason The reason the promise was rejected.\\n     * @returns A new rejected Promise.\\n     */\\n    reject<T = never>(reason?: any): Promise<T>;\\n\\n    /**\\n     * Creates a new resolved promise for the provided value.\\n     * @param value A promise.\\n     * @returns A promise whose internal state matches the provided promise.\\n     */\\n    resolve<T>(value: T | PromiseLike<T>): Promise<T>;\\n\\n    /**\\n     * Creates a new resolved promise .\\n     * @returns A resolved promise.\\n     */\\n    resolve(): Promise<void>;\\n}\\n\\ndeclare var Promise: PromiseConstructor;\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface SymbolConstructor {\\n    /**\\n     * A reference to the prototype.\\n     */\\n    readonly prototype: Symbol;\\n\\n    /**\\n     * Returns a new unique Symbol value.\\n     * @param  description Description of the new Symbol object.\\n     */\\n    (description?: string | number): symbol;\\n\\n    /**\\n     * Returns a Symbol object from the global symbol registry matching the given key if found.\\n     * Otherwise, returns a new symbol with this key.\\n     * @param key key to search for.\\n     */\\n    for(key: string): symbol;\\n\\n    /**\\n     * Returns a key from the global symbol registry matching the given Symbol if found.\\n     * Otherwise, returns a undefined.\\n     * @param sym Symbol to find the key for.\\n     */\\n    keyFor(sym: symbol): string | undefined;\\n}\\n\\ndeclare var Symbol: SymbolConstructor;\\ninterface SymbolConstructor {\\n    /**\\n     * A method that returns the default iterator for an object. Called by the semantics of the\\n     * for-of statement.\\n     */\\n    readonly iterator: symbol;\\n}\\n\\ninterface IteratorResult<T> {\\n    done: boolean;\\n    value: T;\\n}\\n\\ninterface Iterator<T> {\\n    next(value?: any): IteratorResult<T>;\\n    return?(value?: any): IteratorResult<T>;\\n    throw?(e?: any): IteratorResult<T>;\\n}\\n\\ninterface Iterable<T> {\\n    [Symbol.iterator](): Iterator<T>;\\n}\\n\\ninterface IterableIterator<T> extends Iterator<T> {\\n    [Symbol.iterator](): IterableIterator<T>;\\n}\\n\\ninterface Array<T> {\\n    /** Iterator */\\n    [Symbol.iterator](): IterableIterator<T>;\\n\\n    /**\\n     * Returns an iterable of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, T]>;\\n\\n    /**\\n     * Returns an iterable of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n\\n    /**\\n     * Returns an iterable of values in the array\\n     */\\n    values(): IterableIterator<T>;\\n}\\n\\ninterface ArrayConstructor {\\n    /**\\n     * Creates an array from an iterable object.\\n     * @param iterable An iterable object to convert to an array.\\n     */\\n    from<T>(iterable: Iterable<T> | ArrayLike<T>): T[];\\n\\n    /**\\n     * Creates an array from an iterable object.\\n     * @param iterable An iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from<T, U>(iterable: Iterable<T> | ArrayLike<T>, mapfn: (v: T, k: number) => U, thisArg?: any): U[];\\n}\\n\\ninterface ReadonlyArray<T> {\\n    /** Iterator of values in the array. */\\n    [Symbol.iterator](): IterableIterator<T>;\\n\\n    /**\\n     * Returns an iterable of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, T]>;\\n\\n    /**\\n     * Returns an iterable of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n\\n    /**\\n     * Returns an iterable of values in the array\\n     */\\n    values(): IterableIterator<T>;\\n}\\n\\ninterface IArguments {\\n    /** Iterator */\\n    [Symbol.iterator](): IterableIterator<any>;\\n}\\n\\ninterface Map<K, V> {\\n    /** Returns an iterable of entries in the map. */\\n    [Symbol.iterator](): IterableIterator<[K, V]>;\\n\\n    /**\\n     * Returns an iterable of key, value pairs for every entry in the map.\\n     */\\n    entries(): IterableIterator<[K, V]>;\\n\\n    /**\\n     * Returns an iterable of keys in the map\\n     */\\n    keys(): IterableIterator<K>;\\n\\n    /**\\n     * Returns an iterable of values in the map\\n     */\\n    values(): IterableIterator<V>;\\n}\\n\\ninterface ReadonlyMap<K, V> {\\n    /** Returns an iterable of entries in the map. */\\n    [Symbol.iterator](): IterableIterator<[K, V]>;\\n\\n    /**\\n     * Returns an iterable of key, value pairs for every entry in the map.\\n     */\\n    entries(): IterableIterator<[K, V]>;\\n\\n    /**\\n     * Returns an iterable of keys in the map\\n     */\\n    keys(): IterableIterator<K>;\\n\\n    /**\\n     * Returns an iterable of values in the map\\n     */\\n    values(): IterableIterator<V>;\\n}\\n\\ninterface MapConstructor {\\n    new <K, V>(iterable: Iterable<[K, V]>): Map<K, V>;\\n}\\n\\ninterface WeakMap<K extends object, V> { }\\n\\ninterface WeakMapConstructor {\\n    new <K extends object = object, V = any>(iterable: Iterable<[K, V]>): WeakMap<K, V>;\\n}\\n\\ninterface Set<T> {\\n    /** Iterates over values in the set. */\\n    [Symbol.iterator](): IterableIterator<T>;\\n    /**\\n     * Returns an iterable of [v,v] pairs for every value `v` in the set.\\n     */\\n    entries(): IterableIterator<[T, T]>;\\n    /**\\n     * Despite its name, returns an iterable of the values in the set,\\n     */\\n    keys(): IterableIterator<T>;\\n\\n    /**\\n     * Returns an iterable of values in the set.\\n     */\\n    values(): IterableIterator<T>;\\n}\\n\\ninterface ReadonlySet<T> {\\n    /** Iterates over values in the set. */\\n    [Symbol.iterator](): IterableIterator<T>;\\n\\n    /**\\n     * Returns an iterable of [v,v] pairs for every value `v` in the set.\\n     */\\n    entries(): IterableIterator<[T, T]>;\\n\\n    /**\\n     * Despite its name, returns an iterable of the values in the set,\\n     */\\n    keys(): IterableIterator<T>;\\n\\n    /**\\n     * Returns an iterable of values in the set.\\n     */\\n    values(): IterableIterator<T>;\\n}\\n\\ninterface SetConstructor {\\n    new <T>(iterable: Iterable<T>): Set<T>;\\n}\\n\\ninterface WeakSet<T extends object> { }\\n\\ninterface WeakSetConstructor {\\n    new <T extends object = object>(iterable: Iterable<T>): WeakSet<T>;\\n}\\n\\ninterface Promise<T> { }\\n\\ninterface PromiseConstructor {\\n    /**\\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\\n     * resolve, or rejected when any Promise is rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    all<TAll>(values: Iterable<TAll | PromiseLike<TAll>>): Promise<TAll[]>;\\n\\n    /**\\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\\n     * or rejected.\\n     * @param values An array of Promises.\\n     * @returns A new Promise.\\n     */\\n    race<T>(values: Iterable<T | PromiseLike<T>>): Promise<T>;\\n}\\n\\ndeclare namespace Reflect {\\n    function enumerate(target: object): IterableIterator<any>;\\n}\\n\\ninterface String {\\n    /** Iterator */\\n    [Symbol.iterator](): IterableIterator<string>;\\n}\\n\\ninterface Int8Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Int8ArrayConstructor {\\n    new (elements: Iterable<number>): Int8Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int8Array;\\n}\\n\\ninterface Uint8Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Uint8ArrayConstructor {\\n    new (elements: Iterable<number>): Uint8Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8Array;\\n}\\n\\ninterface Uint8ClampedArray {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Uint8ClampedArrayConstructor {\\n    new (elements: Iterable<number>): Uint8ClampedArray;\\n\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8ClampedArray;\\n}\\n\\ninterface Int16Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Int16ArrayConstructor {\\n    new (elements: Iterable<number>): Int16Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int16Array;\\n}\\n\\ninterface Uint16Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Uint16ArrayConstructor {\\n    new (elements: Iterable<number>): Uint16Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint16Array;\\n}\\n\\ninterface Int32Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Int32ArrayConstructor {\\n    new (elements: Iterable<number>): Int32Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int32Array;\\n}\\n\\ninterface Uint32Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Uint32ArrayConstructor {\\n    new (elements: Iterable<number>): Uint32Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint32Array;\\n}\\n\\ninterface Float32Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Float32ArrayConstructor {\\n    new (elements: Iterable<number>): Float32Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float32Array;\\n}\\n\\ninterface Float64Array {\\n    [Symbol.iterator](): IterableIterator<number>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the array\\n     */\\n    entries(): IterableIterator<[number, number]>;\\n    /**\\n     * Returns an list of keys in the array\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the array\\n     */\\n    values(): IterableIterator<number>;\\n}\\n\\ninterface Float64ArrayConstructor {\\n    new (elements: Iterable<number>): Float64Array;\\n\\n    /**\\n     * Creates an array from an array-like or iterable object.\\n     * @param arrayLike An array-like or iterable object to convert to an array.\\n     * @param mapfn A mapping function to call on every element of the array.\\n     * @param thisArg Value of \\'this\\' used to invoke the mapfn.\\n     */\\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float64Array;\\n}\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface ProxyHandler<T extends object> {\\n    getPrototypeOf? (target: T): object | null;\\n    setPrototypeOf? (target: T, v: any): boolean;\\n    isExtensible? (target: T): boolean;\\n    preventExtensions? (target: T): boolean;\\n    getOwnPropertyDescriptor? (target: T, p: PropertyKey): PropertyDescriptor | undefined;\\n    has? (target: T, p: PropertyKey): boolean;\\n    get? (target: T, p: PropertyKey, receiver: any): any;\\n    set? (target: T, p: PropertyKey, value: any, receiver: any): boolean;\\n    deleteProperty? (target: T, p: PropertyKey): boolean;\\n    defineProperty? (target: T, p: PropertyKey, attributes: PropertyDescriptor): boolean;\\n    enumerate? (target: T): PropertyKey[];\\n    ownKeys? (target: T): PropertyKey[];\\n    apply? (target: T, thisArg: any, argArray?: any): any;\\n    construct? (target: T, argArray: any, newTarget?: any): object;\\n}\\n\\ninterface ProxyConstructor {\\n    revocable<T extends object>(target: T, handler: ProxyHandler<T>): { proxy: T; revoke: () => void; };\\n    new <T extends object>(target: T, handler: ProxyHandler<T>): T;\\n}\\ndeclare var Proxy: ProxyConstructor;\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ndeclare namespace Reflect {\\n    function apply(target: Function, thisArgument: any, argumentsList: ArrayLike<any>): any;\\n    function construct(target: Function, argumentsList: ArrayLike<any>, newTarget?: any): any;\\n    function defineProperty(target: object, propertyKey: PropertyKey, attributes: PropertyDescriptor): boolean;\\n    function deleteProperty(target: object, propertyKey: PropertyKey): boolean;\\n    function get(target: object, propertyKey: PropertyKey, receiver?: any): any;\\n    function getOwnPropertyDescriptor(target: object, propertyKey: PropertyKey): PropertyDescriptor | undefined;\\n    function getPrototypeOf(target: object): object;\\n    function has(target: object, propertyKey: PropertyKey): boolean;\\n    function isExtensible(target: object): boolean;\\n    function ownKeys(target: object): PropertyKey[];\\n    function preventExtensions(target: object): boolean;\\n    function set(target: object, propertyKey: PropertyKey, value: any, receiver?: any): boolean;\\n    function setPrototypeOf(target: object, proto: any): boolean;\\n}\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface SymbolConstructor {\\n    /**\\n     * A reference to the prototype.\\n     */\\n    readonly prototype: Symbol;\\n\\n    /**\\n     * Returns a new unique Symbol value.\\n     * @param  description Description of the new Symbol object.\\n     */\\n    (description?: string | number): symbol;\\n\\n    /**\\n     * Returns a Symbol object from the global symbol registry matching the given key if found.\\n     * Otherwise, returns a new symbol with this key.\\n     * @param key key to search for.\\n     */\\n    for(key: string): symbol;\\n\\n    /**\\n     * Returns a key from the global symbol registry matching the given Symbol if found.\\n     * Otherwise, returns a undefined.\\n     * @param sym Symbol to find the key for.\\n     */\\n    keyFor(sym: symbol): string | undefined;\\n}\\n\\ndeclare var Symbol: SymbolConstructor;/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\ninterface SymbolConstructor {\\n    /**\\n     * A reference to the prototype.\\n     */\\n    readonly prototype: Symbol;\\n\\n    /**\\n     * Returns a new unique Symbol value.\\n     * @param  description Description of the new Symbol object.\\n     */\\n    (description?: string | number): symbol;\\n\\n    /**\\n     * Returns a Symbol object from the global symbol registry matching the given key if found.\\n     * Otherwise, returns a new symbol with this key.\\n     * @param key key to search for.\\n     */\\n    for(key: string): symbol;\\n\\n    /**\\n     * Returns a key from the global symbol registry matching the given Symbol if found.\\n     * Otherwise, returns a undefined.\\n     * @param sym Symbol to find the key for.\\n     */\\n    keyFor(sym: symbol): string | undefined;\\n}\\n\\ndeclare var Symbol: SymbolConstructor;\\ninterface SymbolConstructor {\\n    /**\\n     * A method that determines if a constructor object recognizes an object as one of the\\n     * constructor’s instances. Called by the semantics of the instanceof operator.\\n     */\\n    readonly hasInstance: symbol;\\n\\n    /**\\n     * A Boolean value that if true indicates that an object should flatten to its array elements\\n     * by Array.prototype.concat.\\n     */\\n    readonly isConcatSpreadable: symbol;\\n\\n    /**\\n     * A regular expression method that matches the regular expression against a string. Called\\n     * by the String.prototype.match method.\\n     */\\n    readonly match: symbol;\\n\\n    /**\\n     * A regular expression method that replaces matched substrings of a string. Called by the\\n     * String.prototype.replace method.\\n     */\\n    readonly replace: symbol;\\n\\n    /**\\n     * A regular expression method that returns the index within a string that matches the\\n     * regular expression. Called by the String.prototype.search method.\\n     */\\n    readonly search: symbol;\\n\\n    /**\\n     * A function valued property that is the constructor function that is used to create\\n     * derived objects.\\n     */\\n    readonly species: symbol;\\n\\n    /**\\n     * A regular expression method that splits a string at the indices that match the regular\\n     * expression. Called by the String.prototype.split method.\\n     */\\n    readonly split: symbol;\\n\\n    /**\\n     * A method that converts an object to a corresponding primitive value.\\n     * Called by the ToPrimitive abstract operation.\\n     */\\n    readonly toPrimitive: symbol;\\n\\n    /**\\n     * A String value that is used in the creation of the default string description of an object.\\n     * Called by the built-in method Object.prototype.toString.\\n     */\\n    readonly toStringTag: symbol;\\n\\n    /**\\n     * An Object whose own property names are property names that are excluded from the \\'with\\'\\n     * environment bindings of the associated objects.\\n     */\\n    readonly unscopables: symbol;\\n}\\n\\ninterface Symbol {\\n    readonly [Symbol.toStringTag]: \"Symbol\";\\n}\\n\\ninterface Array<T> {\\n    /**\\n     * Returns an object whose properties have the value \\'true\\'\\n     * when they will be absent when used in a \\'with\\' statement.\\n     */\\n    [Symbol.unscopables](): {\\n        copyWithin: boolean;\\n        entries: boolean;\\n        fill: boolean;\\n        find: boolean;\\n        findIndex: boolean;\\n        keys: boolean;\\n        values: boolean;\\n    };\\n}\\n\\ninterface Date {\\n    /**\\n     * Converts a Date object to a string.\\n     */\\n    [Symbol.toPrimitive](hint: \"default\"): string;\\n    /**\\n     * Converts a Date object to a string.\\n     */\\n    [Symbol.toPrimitive](hint: \"string\"): string;\\n    /**\\n     * Converts a Date object to a number.\\n     */\\n    [Symbol.toPrimitive](hint: \"number\"): number;\\n    /**\\n     * Converts a Date object to a string or number.\\n     *\\n     * @param hint The strings \"number\", \"string\", or \"default\" to specify what primitive to return.\\n     *\\n     * @throws {TypeError} If \\'hint\\' was given something other than \"number\", \"string\", or \"default\".\\n     * @returns A number if \\'hint\\' was \"number\", a string if \\'hint\\' was \"string\" or \"default\".\\n     */\\n    [Symbol.toPrimitive](hint: string): string | number;\\n}\\n\\ninterface Map<K, V> {\\n    readonly [Symbol.toStringTag]: \"Map\";\\n}\\n\\ninterface WeakMap<K extends object, V> {\\n    readonly [Symbol.toStringTag]: \"WeakMap\";\\n}\\n\\ninterface Set<T> {\\n    readonly [Symbol.toStringTag]: \"Set\";\\n}\\n\\ninterface WeakSet<T extends object> {\\n    readonly [Symbol.toStringTag]: \"WeakSet\";\\n}\\n\\ninterface JSON {\\n    readonly [Symbol.toStringTag]: \"JSON\";\\n}\\n\\ninterface Function {\\n    /**\\n     * Determines whether the given value inherits from this function if this function was used\\n     * as a constructor function.\\n     *\\n     * A constructor function can control which objects are recognized as its instances by\\n     * \\'instanceof\\' by overriding this method.\\n     */\\n    [Symbol.hasInstance](value: any): boolean;\\n}\\n\\ninterface GeneratorFunction {\\n    readonly [Symbol.toStringTag]: \"GeneratorFunction\";\\n}\\n\\ninterface Math {\\n    readonly [Symbol.toStringTag]: \"Math\";\\n}\\n\\ninterface Promise<T> {\\n    readonly [Symbol.toStringTag]: \"Promise\";\\n}\\n\\ninterface PromiseConstructor {\\n    readonly [Symbol.species]: PromiseConstructor;\\n}\\n\\ninterface RegExp {\\n    /**\\n     * Matches a string with this regular expression, and returns an array containing the results of\\n     * that search.\\n     * @param string A string to search within.\\n     */\\n    [Symbol.match](string: string): RegExpMatchArray | null;\\n\\n    /**\\n     * Replaces text in a string, using this regular expression.\\n     * @param string A String object or string literal whose contents matching against\\n     *               this regular expression will be replaced\\n     * @param replaceValue A String object or string literal containing the text to replace for every\\n     *                     successful match of this regular expression.\\n     */\\n    [Symbol.replace](string: string, replaceValue: string): string;\\n\\n    /**\\n     * Replaces text in a string, using this regular expression.\\n     * @param string A String object or string literal whose contents matching against\\n     *               this regular expression will be replaced\\n     * @param replacer A function that returns the replacement text.\\n     */\\n    [Symbol.replace](string: string, replacer: (substring: string, ...args: any[]) => string): string;\\n\\n    /**\\n     * Finds the position beginning first substring match in a regular expression search\\n     * using this regular expression.\\n     *\\n     * @param string The string to search within.\\n     */\\n    [Symbol.search](string: string): number;\\n\\n    /**\\n     * Returns an array of substrings that were delimited by strings in the original input that\\n     * match against this regular expression.\\n     *\\n     * If the regular expression contains capturing parentheses, then each time this\\n     * regular expression matches, the results (including any undefined results) of the\\n     * capturing parentheses are spliced.\\n     *\\n     * @param string string value to split\\n     * @param limit if not undefined, the output array is truncated so that it contains no more\\n     * than \\'limit\\' elements.\\n     */\\n    [Symbol.split](string: string, limit?: number): string[];\\n}\\n\\ninterface RegExpConstructor {\\n    readonly [Symbol.species]: RegExpConstructor;\\n}\\n\\ninterface String {\\n    /**\\n     * Matches a string an object that supports being matched against, and returns an array containing the results of that search.\\n     * @param matcher An object that supports being matched against.\\n     */\\n    match(matcher: { [Symbol.match](string: string): RegExpMatchArray | null; }): RegExpMatchArray | null;\\n\\n    /**\\n     * Replaces text in a string, using an object that supports replacement within a string.\\n     * @param searchValue A object can search for and replace matches within a string.\\n     * @param replaceValue A string containing the text to replace for every successful match of searchValue in this string.\\n     */\\n    replace(searchValue: { [Symbol.replace](string: string, replaceValue: string): string; }, replaceValue: string): string;\\n\\n    /**\\n     * Replaces text in a string, using an object that supports replacement within a string.\\n     * @param searchValue A object can search for and replace matches within a string.\\n     * @param replacer A function that returns the replacement text.\\n     */\\n    replace(searchValue: { [Symbol.replace](string: string, replacer: (substring: string, ...args: any[]) => string): string; }, replacer: (substring: string, ...args: any[]) => string): string;\\n\\n    /**\\n     * Finds the first substring match in a regular expression search.\\n     * @param searcher An object which supports searching within a string.\\n     */\\n    search(searcher: { [Symbol.search](string: string): number; }): number;\\n\\n    /**\\n     * Split a string into substrings using the specified separator and return them as an array.\\n     * @param splitter An object that can split a string.\\n     * @param limit A value used to limit the number of elements returned in the array.\\n     */\\n    split(splitter: { [Symbol.split](string: string, limit?: number): string[]; }, limit?: number): string[];\\n}\\n\\ninterface ArrayBuffer {\\n    readonly [Symbol.toStringTag]: \"ArrayBuffer\";\\n}\\n\\ninterface DataView {\\n    readonly [Symbol.toStringTag]: \"DataView\";\\n}\\n\\ninterface Int8Array {\\n    readonly [Symbol.toStringTag]: \"Int8Array\";\\n}\\n\\ninterface Uint8Array {\\n    readonly [Symbol.toStringTag]: \"UInt8Array\";\\n}\\n\\ninterface Uint8ClampedArray {\\n    readonly [Symbol.toStringTag]: \"Uint8ClampedArray\";\\n}\\n\\ninterface Int16Array {\\n    readonly [Symbol.toStringTag]: \"Int16Array\";\\n}\\n\\ninterface Uint16Array {\\n    readonly [Symbol.toStringTag]: \"Uint16Array\";\\n}\\n\\ninterface Int32Array {\\n    readonly [Symbol.toStringTag]: \"Int32Array\";\\n}\\n\\ninterface Uint32Array {\\n    readonly [Symbol.toStringTag]: \"Uint32Array\";\\n}\\n\\ninterface Float32Array {\\n    readonly [Symbol.toStringTag]: \"Float32Array\";\\n}\\n\\ninterface Float64Array {\\n    readonly [Symbol.toStringTag]: \"Float64Array\";\\n}\\n\\ninterface ArrayConstructor {\\n    readonly [Symbol.species]: ArrayConstructor;\\n}\\ninterface MapConstructor {\\n    readonly [Symbol.species]: MapConstructor;\\n}\\ninterface SetConstructor {\\n    readonly [Symbol.species]: SetConstructor;\\n}\\ninterface ArrayBufferConstructor {\\n    readonly [Symbol.species]: ArrayBufferConstructor;\\n}/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n/////////////////////////////\\n/// DOM APIs\\n/////////////////////////////\\n\\ninterface Account {\\n    displayName: string;\\n    id: string;\\n    imageURL?: string;\\n    name?: string;\\n    rpDisplayName: string;\\n}\\n\\ninterface AddEventListenerOptions extends EventListenerOptions {\\n    once?: boolean;\\n    passive?: boolean;\\n}\\n\\ninterface AesCbcParams extends Algorithm {\\n    iv: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface AesCtrParams extends Algorithm {\\n    counter: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    length: number;\\n}\\n\\ninterface AesDerivedKeyParams extends Algorithm {\\n    length: number;\\n}\\n\\ninterface AesGcmParams extends Algorithm {\\n    additionalData?: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    iv: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    tagLength?: number;\\n}\\n\\ninterface AesKeyAlgorithm extends KeyAlgorithm {\\n    length: number;\\n}\\n\\ninterface AesKeyGenParams extends Algorithm {\\n    length: number;\\n}\\n\\ninterface Algorithm {\\n    name: string;\\n}\\n\\ninterface AnalyserOptions extends AudioNodeOptions {\\n    fftSize?: number;\\n    maxDecibels?: number;\\n    minDecibels?: number;\\n    smoothingTimeConstant?: number;\\n}\\n\\ninterface AnimationEventInit extends EventInit {\\n    animationName?: string;\\n    elapsedTime?: number;\\n    pseudoElement?: string;\\n}\\n\\ninterface AnimationPlaybackEventInit extends EventInit {\\n    currentTime?: number | null;\\n    timelineTime?: number | null;\\n}\\n\\ninterface AssertionOptions {\\n    allowList?: ScopedCredentialDescriptor[];\\n    extensions?: WebAuthnExtensions;\\n    rpId?: string;\\n    timeoutSeconds?: number;\\n}\\n\\ninterface AssignedNodesOptions {\\n    flatten?: boolean;\\n}\\n\\ninterface AudioBufferOptions {\\n    length: number;\\n    numberOfChannels?: number;\\n    sampleRate: number;\\n}\\n\\ninterface AudioBufferSourceOptions {\\n    buffer?: AudioBuffer | null;\\n    detune?: number;\\n    loop?: boolean;\\n    loopEnd?: number;\\n    loopStart?: number;\\n    playbackRate?: number;\\n}\\n\\ninterface AudioContextInfo {\\n    currentTime?: number;\\n    sampleRate?: number;\\n}\\n\\ninterface AudioContextOptions {\\n    latencyHint?: AudioContextLatencyCategory | number;\\n    sampleRate?: number;\\n}\\n\\ninterface AudioNodeOptions {\\n    channelCount?: number;\\n    channelCountMode?: ChannelCountMode;\\n    channelInterpretation?: ChannelInterpretation;\\n}\\n\\ninterface AudioParamDescriptor {\\n    automationRate?: AutomationRate;\\n    defaultValue?: number;\\n    maxValue?: number;\\n    minValue?: number;\\n    name: string;\\n}\\n\\ninterface AudioProcessingEventInit extends EventInit {\\n    inputBuffer: AudioBuffer;\\n    outputBuffer: AudioBuffer;\\n    playbackTime: number;\\n}\\n\\ninterface AudioTimestamp {\\n    contextTime?: number;\\n    performanceTime?: number;\\n}\\n\\ninterface AudioWorkletNodeOptions extends AudioNodeOptions {\\n    numberOfInputs?: number;\\n    numberOfOutputs?: number;\\n    outputChannelCount?: number[];\\n    parameterData?: Record<string, number>;\\n    processorOptions?: any;\\n}\\n\\ninterface BiquadFilterOptions extends AudioNodeOptions {\\n    Q?: number;\\n    detune?: number;\\n    frequency?: number;\\n    gain?: number;\\n    type?: BiquadFilterType;\\n}\\n\\ninterface BlobPropertyBag {\\n    type?: string;\\n}\\n\\ninterface ByteLengthChunk {\\n    byteLength?: number;\\n}\\n\\ninterface CacheQueryOptions {\\n    cacheName?: string;\\n    ignoreMethod?: boolean;\\n    ignoreSearch?: boolean;\\n    ignoreVary?: boolean;\\n}\\n\\ninterface CanvasRenderingContext2DSettings {\\n    alpha?: boolean;\\n}\\n\\ninterface ChannelMergerOptions extends AudioNodeOptions {\\n    numberOfInputs?: number;\\n}\\n\\ninterface ChannelSplitterOptions extends AudioNodeOptions {\\n    numberOfOutputs?: number;\\n}\\n\\ninterface ClientData {\\n    challenge: string;\\n    extensions?: WebAuthnExtensions;\\n    hashAlg: string | Algorithm;\\n    origin: string;\\n    rpId: string;\\n    tokenBinding?: string;\\n}\\n\\ninterface ClientQueryOptions {\\n    includeUncontrolled?: boolean;\\n    type?: ClientTypes;\\n}\\n\\ninterface CloseEventInit extends EventInit {\\n    code?: number;\\n    reason?: string;\\n    wasClean?: boolean;\\n}\\n\\ninterface CompositionEventInit extends UIEventInit {\\n    data?: string;\\n}\\n\\ninterface ComputedEffectTiming extends EffectTiming {\\n    activeDuration?: number;\\n    currentIteration?: number | null;\\n    endTime?: number;\\n    localTime?: number | null;\\n    progress?: number | null;\\n}\\n\\ninterface ComputedKeyframe extends Record<keyof CSSStyleDeclaration, string> {\\n    composite?: CompositeOperation | null;\\n    computedOffset?: number;\\n    easing?: string;\\n    offset?: number | null;\\n}\\n\\ninterface ConfirmSiteSpecificExceptionsInformation extends ExceptionInformation {\\n    arrayOfDomainStrings?: string[];\\n}\\n\\ninterface ConstantSourceOptions {\\n    offset?: number;\\n}\\n\\ninterface ConstrainBooleanParameters {\\n    exact?: boolean;\\n    ideal?: boolean;\\n}\\n\\ninterface ConstrainDOMStringParameters {\\n    exact?: string | string[];\\n    ideal?: string | string[];\\n}\\n\\ninterface ConstrainDoubleRange extends DoubleRange {\\n    exact?: number;\\n    ideal?: number;\\n}\\n\\ninterface ConstrainLongRange extends LongRange {\\n    exact?: number;\\n    ideal?: number;\\n}\\n\\ninterface ConstrainVideoFacingModeParameters {\\n    exact?: VideoFacingModeEnum | VideoFacingModeEnum[];\\n    ideal?: VideoFacingModeEnum | VideoFacingModeEnum[];\\n}\\n\\ninterface ConvolverOptions extends AudioNodeOptions {\\n    buffer?: AudioBuffer | null;\\n    disableNormalization?: boolean;\\n}\\n\\ninterface CustomEventInit<T = any> extends EventInit {\\n    detail?: T;\\n}\\n\\ninterface DOMMatrix2DInit {\\n    a?: number;\\n    b?: number;\\n    c?: number;\\n    d?: number;\\n    e?: number;\\n    f?: number;\\n    m11?: number;\\n    m12?: number;\\n    m21?: number;\\n    m22?: number;\\n    m41?: number;\\n    m42?: number;\\n}\\n\\ninterface DOMMatrixInit extends DOMMatrix2DInit {\\n    is2D?: boolean;\\n    m13?: number;\\n    m14?: number;\\n    m23?: number;\\n    m24?: number;\\n    m31?: number;\\n    m32?: number;\\n    m33?: number;\\n    m34?: number;\\n    m43?: number;\\n    m44?: number;\\n}\\n\\ninterface DOMPointInit {\\n    w?: number;\\n    x?: number;\\n    y?: number;\\n    z?: number;\\n}\\n\\ninterface DOMQuadInit {\\n    p1?: DOMPointInit;\\n    p2?: DOMPointInit;\\n    p3?: DOMPointInit;\\n    p4?: DOMPointInit;\\n}\\n\\ninterface DOMRectInit {\\n    height?: number;\\n    width?: number;\\n    x?: number;\\n    y?: number;\\n}\\n\\ninterface DelayOptions extends AudioNodeOptions {\\n    delayTime?: number;\\n    maxDelayTime?: number;\\n}\\n\\ninterface DeviceAccelerationDict {\\n    x?: number | null;\\n    y?: number | null;\\n    z?: number | null;\\n}\\n\\ninterface DeviceLightEventInit extends EventInit {\\n    value?: number;\\n}\\n\\ninterface DeviceMotionEventInit extends EventInit {\\n    acceleration?: DeviceAccelerationDict | null;\\n    accelerationIncludingGravity?: DeviceAccelerationDict | null;\\n    interval?: number | null;\\n    rotationRate?: DeviceRotationRateDict | null;\\n}\\n\\ninterface DeviceOrientationEventInit extends EventInit {\\n    absolute?: boolean;\\n    alpha?: number | null;\\n    beta?: number | null;\\n    gamma?: number | null;\\n}\\n\\ninterface DeviceRotationRateDict {\\n    alpha?: number | null;\\n    beta?: number | null;\\n    gamma?: number | null;\\n}\\n\\ninterface DocumentTimelineOptions {\\n    originTime?: number;\\n}\\n\\ninterface DoubleRange {\\n    max?: number;\\n    min?: number;\\n}\\n\\ninterface DragEventInit extends MouseEventInit {\\n    dataTransfer?: DataTransfer | null;\\n}\\n\\ninterface DynamicsCompressorOptions extends AudioNodeOptions {\\n    attack?: number;\\n    knee?: number;\\n    ratio?: number;\\n    release?: number;\\n    threshold?: number;\\n}\\n\\ninterface EcKeyAlgorithm extends KeyAlgorithm {\\n    namedCurve: NamedCurve;\\n}\\n\\ninterface EcKeyGenParams extends Algorithm {\\n    namedCurve: NamedCurve;\\n}\\n\\ninterface EcKeyImportParams extends Algorithm {\\n    namedCurve: NamedCurve;\\n}\\n\\ninterface EcdhKeyDeriveParams extends Algorithm {\\n    public: CryptoKey;\\n}\\n\\ninterface EcdsaParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n}\\n\\ninterface EffectTiming {\\n    delay?: number;\\n    direction?: PlaybackDirection;\\n    duration?: number | string;\\n    easing?: string;\\n    endDelay?: number;\\n    fill?: FillMode;\\n    iterationStart?: number;\\n    iterations?: number;\\n}\\n\\ninterface ElementDefinitionOptions {\\n    extends?: string;\\n}\\n\\ninterface ErrorEventInit extends EventInit {\\n    colno?: number;\\n    error?: any;\\n    filename?: string;\\n    lineno?: number;\\n    message?: string;\\n}\\n\\ninterface EventInit {\\n    bubbles?: boolean;\\n    cancelable?: boolean;\\n    composed?: boolean;\\n}\\n\\ninterface EventListenerOptions {\\n    capture?: boolean;\\n}\\n\\ninterface EventModifierInit extends UIEventInit {\\n    altKey?: boolean;\\n    ctrlKey?: boolean;\\n    metaKey?: boolean;\\n    modifierAltGraph?: boolean;\\n    modifierCapsLock?: boolean;\\n    modifierFn?: boolean;\\n    modifierFnLock?: boolean;\\n    modifierHyper?: boolean;\\n    modifierNumLock?: boolean;\\n    modifierOS?: boolean;\\n    modifierScrollLock?: boolean;\\n    modifierSuper?: boolean;\\n    modifierSymbol?: boolean;\\n    modifierSymbolLock?: boolean;\\n    shiftKey?: boolean;\\n}\\n\\ninterface ExceptionInformation {\\n    domain?: string | null;\\n}\\n\\ninterface FilePropertyBag extends BlobPropertyBag {\\n    lastModified?: number;\\n}\\n\\ninterface FocusEventInit extends UIEventInit {\\n    relatedTarget?: EventTarget | null;\\n}\\n\\ninterface FocusNavigationEventInit extends EventInit {\\n    navigationReason?: string | null;\\n    originHeight?: number;\\n    originLeft?: number;\\n    originTop?: number;\\n    originWidth?: number;\\n}\\n\\ninterface FocusNavigationOrigin {\\n    originHeight?: number;\\n    originLeft?: number;\\n    originTop?: number;\\n    originWidth?: number;\\n}\\n\\ninterface FocusOptions {\\n    preventScroll?: boolean;\\n}\\n\\ninterface GainOptions extends AudioNodeOptions {\\n    gain?: number;\\n}\\n\\ninterface GamepadEventInit extends EventInit {\\n    gamepad?: Gamepad;\\n}\\n\\ninterface GetNotificationOptions {\\n    tag?: string;\\n}\\n\\ninterface GetRootNodeOptions {\\n    composed?: boolean;\\n}\\n\\ninterface HashChangeEventInit extends EventInit {\\n    newURL?: string;\\n    oldURL?: string;\\n}\\n\\ninterface HkdfParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    info: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    salt: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface HmacImportParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    length?: number;\\n}\\n\\ninterface HmacKeyAlgorithm extends KeyAlgorithm {\\n    hash: KeyAlgorithm;\\n    length: number;\\n}\\n\\ninterface HmacKeyGenParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    length?: number;\\n}\\n\\ninterface IDBIndexParameters {\\n    multiEntry?: boolean;\\n    unique?: boolean;\\n}\\n\\ninterface IDBObjectStoreParameters {\\n    autoIncrement?: boolean;\\n    keyPath?: string | string[] | null;\\n}\\n\\ninterface IDBVersionChangeEventInit extends EventInit {\\n    newVersion?: number | null;\\n    oldVersion?: number;\\n}\\n\\ninterface IIRFilterOptions extends AudioNodeOptions {\\n    feedback: number[];\\n    feedforward: number[];\\n}\\n\\ninterface IntersectionObserverEntryInit {\\n    boundingClientRect: DOMRectInit;\\n    intersectionRect: DOMRectInit;\\n    isIntersecting: boolean;\\n    rootBounds: DOMRectInit;\\n    target: Element;\\n    time: number;\\n}\\n\\ninterface IntersectionObserverInit {\\n    root?: Element | null;\\n    rootMargin?: string;\\n    threshold?: number | number[];\\n}\\n\\ninterface JsonWebKey {\\n    alg?: string;\\n    crv?: string;\\n    d?: string;\\n    dp?: string;\\n    dq?: string;\\n    e?: string;\\n    ext?: boolean;\\n    k?: string;\\n    key_ops?: string[];\\n    kty?: string;\\n    n?: string;\\n    oth?: RsaOtherPrimesInfo[];\\n    p?: string;\\n    q?: string;\\n    qi?: string;\\n    use?: string;\\n    x?: string;\\n    y?: string;\\n}\\n\\ninterface KeyAlgorithm {\\n    name: string;\\n}\\n\\ninterface KeyboardEventInit extends EventModifierInit {\\n    code?: string;\\n    key?: string;\\n    location?: number;\\n    repeat?: boolean;\\n}\\n\\ninterface Keyframe extends Record<keyof CSSStyleDeclaration, string> {\\n    composite?: CompositeOperation | null;\\n    easing?: string;\\n    offset?: number | null;\\n}\\n\\ninterface KeyframeAnimationOptions extends KeyframeEffectOptions {\\n    id?: string;\\n}\\n\\ninterface KeyframeEffectOptions extends EffectTiming {\\n    composite?: CompositeOperation;\\n    iterationComposite?: IterationCompositeOperation;\\n}\\n\\ninterface LongRange {\\n    max?: number;\\n    min?: number;\\n}\\n\\ninterface MediaElementAudioSourceOptions {\\n    mediaElement: HTMLMediaElement;\\n}\\n\\ninterface MediaEncryptedEventInit extends EventInit {\\n    initData?: ArrayBuffer | null;\\n    initDataType?: string;\\n}\\n\\ninterface MediaKeyMessageEventInit extends EventInit {\\n    message?: ArrayBuffer | null;\\n    messageType?: MediaKeyMessageType;\\n}\\n\\ninterface MediaKeySystemConfiguration {\\n    audioCapabilities?: MediaKeySystemMediaCapability[];\\n    distinctiveIdentifier?: MediaKeysRequirement;\\n    initDataTypes?: string[];\\n    persistentState?: MediaKeysRequirement;\\n    videoCapabilities?: MediaKeySystemMediaCapability[];\\n}\\n\\ninterface MediaKeySystemMediaCapability {\\n    contentType?: string;\\n    robustness?: string;\\n}\\n\\ninterface MediaQueryListEventInit extends EventInit {\\n    matches?: boolean;\\n    media?: string;\\n}\\n\\ninterface MediaStreamAudioSourceOptions {\\n    mediaStream: MediaStream;\\n}\\n\\ninterface MediaStreamConstraints {\\n    audio?: boolean | MediaTrackConstraints;\\n    peerIdentity?: string;\\n    video?: boolean | MediaTrackConstraints;\\n}\\n\\ninterface MediaStreamErrorEventInit extends EventInit {\\n    error?: MediaStreamError | null;\\n}\\n\\ninterface MediaStreamEventInit extends EventInit {\\n    stream?: MediaStream;\\n}\\n\\ninterface MediaStreamTrackAudioSourceOptions {\\n    mediaStreamTrack: MediaStreamTrack;\\n}\\n\\ninterface MediaStreamTrackEventInit extends EventInit {\\n    track?: MediaStreamTrack | null;\\n}\\n\\ninterface MediaTrackCapabilities {\\n    aspectRatio?: number | DoubleRange;\\n    deviceId?: string;\\n    echoCancellation?: boolean[];\\n    facingMode?: string;\\n    frameRate?: number | DoubleRange;\\n    groupId?: string;\\n    height?: number | LongRange;\\n    sampleRate?: number | LongRange;\\n    sampleSize?: number | LongRange;\\n    volume?: number | DoubleRange;\\n    width?: number | LongRange;\\n}\\n\\ninterface MediaTrackConstraintSet {\\n    aspectRatio?: number | ConstrainDoubleRange;\\n    channelCount?: number | ConstrainLongRange;\\n    deviceId?: string | string[] | ConstrainDOMStringParameters;\\n    displaySurface?: string | string[] | ConstrainDOMStringParameters;\\n    echoCancellation?: boolean | ConstrainBooleanParameters;\\n    facingMode?: string | string[] | ConstrainDOMStringParameters;\\n    frameRate?: number | ConstrainDoubleRange;\\n    groupId?: string | string[] | ConstrainDOMStringParameters;\\n    height?: number | ConstrainLongRange;\\n    latency?: number | ConstrainDoubleRange;\\n    logicalSurface?: boolean | ConstrainBooleanParameters;\\n    sampleRate?: number | ConstrainLongRange;\\n    sampleSize?: number | ConstrainLongRange;\\n    volume?: number | ConstrainDoubleRange;\\n    width?: number | ConstrainLongRange;\\n}\\n\\ninterface MediaTrackConstraints extends MediaTrackConstraintSet {\\n    advanced?: MediaTrackConstraintSet[];\\n}\\n\\ninterface MediaTrackSettings {\\n    aspectRatio?: number;\\n    deviceId?: string;\\n    echoCancellation?: boolean;\\n    facingMode?: string;\\n    frameRate?: number;\\n    groupId?: string;\\n    height?: number;\\n    sampleRate?: number;\\n    sampleSize?: number;\\n    volume?: number;\\n    width?: number;\\n}\\n\\ninterface MediaTrackSupportedConstraints {\\n    aspectRatio?: boolean;\\n    deviceId?: boolean;\\n    echoCancellation?: boolean;\\n    facingMode?: boolean;\\n    frameRate?: boolean;\\n    groupId?: boolean;\\n    height?: boolean;\\n    sampleRate?: boolean;\\n    sampleSize?: boolean;\\n    volume?: boolean;\\n    width?: boolean;\\n}\\n\\ninterface MessageEventInit extends EventInit {\\n    data?: any;\\n    lastEventId?: string;\\n    origin?: string;\\n    ports?: MessagePort[];\\n    source?: MessageEventSource | null;\\n}\\n\\ninterface MouseEventInit extends EventModifierInit {\\n    button?: number;\\n    buttons?: number;\\n    clientX?: number;\\n    clientY?: number;\\n    relatedTarget?: EventTarget | null;\\n    screenX?: number;\\n    screenY?: number;\\n}\\n\\ninterface MutationObserverInit {\\n    attributeFilter?: string[];\\n    attributeOldValue?: boolean;\\n    attributes?: boolean;\\n    characterData?: boolean;\\n    characterDataOldValue?: boolean;\\n    childList?: boolean;\\n    subtree?: boolean;\\n}\\n\\ninterface NavigationPreloadState {\\n    enabled?: boolean;\\n    headerValue?: string;\\n}\\n\\ninterface NotificationAction {\\n    action: string;\\n    icon?: string;\\n    title: string;\\n}\\n\\ninterface NotificationOptions {\\n    actions?: NotificationAction[];\\n    badge?: string;\\n    body?: string;\\n    data?: any;\\n    dir?: NotificationDirection;\\n    icon?: string;\\n    image?: string;\\n    lang?: string;\\n    renotify?: boolean;\\n    requireInteraction?: boolean;\\n    silent?: boolean;\\n    tag?: string;\\n    timestamp?: number;\\n    vibrate?: VibratePattern;\\n}\\n\\ninterface OfflineAudioCompletionEventInit extends EventInit {\\n    renderedBuffer: AudioBuffer;\\n}\\n\\ninterface OfflineAudioContextOptions {\\n    length: number;\\n    numberOfChannels?: number;\\n    sampleRate: number;\\n}\\n\\ninterface OptionalEffectTiming {\\n    delay?: number;\\n    direction?: PlaybackDirection;\\n    duration?: number | string;\\n    easing?: string;\\n    endDelay?: number;\\n    fill?: FillMode;\\n    iterationStart?: number;\\n    iterations?: number;\\n}\\n\\ninterface OscillatorOptions extends AudioNodeOptions {\\n    detune?: number;\\n    frequency?: number;\\n    periodicWave?: PeriodicWave;\\n    type?: OscillatorType;\\n}\\n\\ninterface PannerOptions extends AudioNodeOptions {\\n    coneInnerAngle?: number;\\n    coneOuterAngle?: number;\\n    coneOuterGain?: number;\\n    distanceModel?: DistanceModelType;\\n    maxDistance?: number;\\n    orientationX?: number;\\n    orientationY?: number;\\n    orientationZ?: number;\\n    panningModel?: PanningModelType;\\n    positionX?: number;\\n    positionY?: number;\\n    positionZ?: number;\\n    refDistance?: number;\\n    rolloffFactor?: number;\\n}\\n\\ninterface PaymentCurrencyAmount {\\n    currency: string;\\n    currencySystem?: string;\\n    value: string;\\n}\\n\\ninterface PaymentDetailsBase {\\n    displayItems?: PaymentItem[];\\n    modifiers?: PaymentDetailsModifier[];\\n    shippingOptions?: PaymentShippingOption[];\\n}\\n\\ninterface PaymentDetailsInit extends PaymentDetailsBase {\\n    id?: string;\\n    total: PaymentItem;\\n}\\n\\ninterface PaymentDetailsModifier {\\n    additionalDisplayItems?: PaymentItem[];\\n    data?: any;\\n    supportedMethods: string | string[];\\n    total?: PaymentItem;\\n}\\n\\ninterface PaymentDetailsUpdate extends PaymentDetailsBase {\\n    error?: string;\\n    total?: PaymentItem;\\n}\\n\\ninterface PaymentItem {\\n    amount: PaymentCurrencyAmount;\\n    label: string;\\n    pending?: boolean;\\n}\\n\\ninterface PaymentMethodData {\\n    data?: any;\\n    supportedMethods: string | string[];\\n}\\n\\ninterface PaymentOptions {\\n    requestPayerEmail?: boolean;\\n    requestPayerName?: boolean;\\n    requestPayerPhone?: boolean;\\n    requestShipping?: boolean;\\n    shippingType?: string;\\n}\\n\\ninterface PaymentRequestUpdateEventInit extends EventInit {\\n}\\n\\ninterface PaymentShippingOption {\\n    amount: PaymentCurrencyAmount;\\n    id: string;\\n    label: string;\\n    selected?: boolean;\\n}\\n\\ninterface Pbkdf2Params extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n    iterations: number;\\n    salt: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface PerformanceObserverInit {\\n    buffered?: boolean;\\n    entryTypes: string[];\\n}\\n\\ninterface PeriodicWaveConstraints {\\n    disableNormalization?: boolean;\\n}\\n\\ninterface PeriodicWaveOptions extends PeriodicWaveConstraints {\\n    imag?: number[] | Float32Array;\\n    real?: number[] | Float32Array;\\n}\\n\\ninterface PointerEventInit extends MouseEventInit {\\n    height?: number;\\n    isPrimary?: boolean;\\n    pointerId?: number;\\n    pointerType?: string;\\n    pressure?: number;\\n    tangentialPressure?: number;\\n    tiltX?: number;\\n    tiltY?: number;\\n    twist?: number;\\n    width?: number;\\n}\\n\\ninterface PopStateEventInit extends EventInit {\\n    state?: any;\\n}\\n\\ninterface PositionOptions {\\n    enableHighAccuracy?: boolean;\\n    maximumAge?: number;\\n    timeout?: number;\\n}\\n\\ninterface ProgressEventInit extends EventInit {\\n    lengthComputable?: boolean;\\n    loaded?: number;\\n    total?: number;\\n}\\n\\ninterface PromiseRejectionEventInit extends EventInit {\\n    promise: Promise<any>;\\n    reason?: any;\\n}\\n\\ninterface PropertyIndexedKeyframes extends Record<keyof CSSStyleDeclaration, string | string[]> {\\n    composite?: CompositeOperation | (CompositeOperation | null)[];\\n    easing?: string | string[];\\n    offset?: number | (number | null)[];\\n}\\n\\ninterface PushSubscriptionJSON {\\n    endpoint?: string;\\n    expirationTime?: number | null;\\n    keys?: Record<string, string>;\\n}\\n\\ninterface PushSubscriptionOptionsInit {\\n    applicationServerKey?: BufferSource | string | null;\\n    userVisibleOnly?: boolean;\\n}\\n\\ninterface QueuingStrategy {\\n    highWaterMark?: number;\\n    size?: WritableStreamChunkCallback;\\n}\\n\\ninterface RTCAnswerOptions extends RTCOfferAnswerOptions {\\n}\\n\\ninterface RTCCertificateExpiration {\\n    expires?: number;\\n}\\n\\ninterface RTCConfiguration {\\n    bundlePolicy?: RTCBundlePolicy;\\n    certificates?: RTCCertificate[];\\n    iceCandidatePoolSize?: number;\\n    iceServers?: RTCIceServer[];\\n    iceTransportPolicy?: RTCIceTransportPolicy;\\n    peerIdentity?: string;\\n    rtcpMuxPolicy?: RTCRtcpMuxPolicy;\\n}\\n\\ninterface RTCDTMFToneChangeEventInit extends EventInit {\\n    tone: string;\\n}\\n\\ninterface RTCDataChannelEventInit extends EventInit {\\n    channel: RTCDataChannel;\\n}\\n\\ninterface RTCDataChannelInit {\\n    id?: number;\\n    maxPacketLifeTime?: number;\\n    maxRetransmits?: number;\\n    negotiated?: boolean;\\n    ordered?: boolean;\\n    priority?: RTCPriorityType;\\n    protocol?: string;\\n}\\n\\ninterface RTCDtlsFingerprint {\\n    algorithm?: string;\\n    value?: string;\\n}\\n\\ninterface RTCDtlsParameters {\\n    fingerprints?: RTCDtlsFingerprint[];\\n    role?: RTCDtlsRole;\\n}\\n\\ninterface RTCErrorEventInit extends EventInit {\\n    error?: RTCError | null;\\n}\\n\\ninterface RTCIceCandidateAttributes extends RTCStats {\\n    addressSourceUrl?: string;\\n    candidateType?: RTCStatsIceCandidateType;\\n    ipAddress?: string;\\n    portNumber?: number;\\n    priority?: number;\\n    transport?: string;\\n}\\n\\ninterface RTCIceCandidateComplete {\\n}\\n\\ninterface RTCIceCandidateDictionary {\\n    foundation?: string;\\n    ip?: string;\\n    msMTurnSessionId?: string;\\n    port?: number;\\n    priority?: number;\\n    protocol?: RTCIceProtocol;\\n    relatedAddress?: string;\\n    relatedPort?: number;\\n    tcpType?: RTCIceTcpCandidateType;\\n    type?: RTCIceCandidateType;\\n}\\n\\ninterface RTCIceCandidateInit {\\n    candidate?: string;\\n    sdpMLineIndex?: number | null;\\n    sdpMid?: string | null;\\n    usernameFragment?: string;\\n}\\n\\ninterface RTCIceCandidatePair {\\n    local?: RTCIceCandidate;\\n    remote?: RTCIceCandidate;\\n}\\n\\ninterface RTCIceCandidatePairStats extends RTCStats {\\n    availableIncomingBitrate?: number;\\n    availableOutgoingBitrate?: number;\\n    bytesReceived?: number;\\n    bytesSent?: number;\\n    localCandidateId?: string;\\n    nominated?: boolean;\\n    priority?: number;\\n    readable?: boolean;\\n    remoteCandidateId?: string;\\n    roundTripTime?: number;\\n    state?: RTCStatsIceCandidatePairState;\\n    transportId?: string;\\n    writable?: boolean;\\n}\\n\\ninterface RTCIceGatherOptions {\\n    gatherPolicy?: RTCIceGatherPolicy;\\n    iceservers?: RTCIceServer[];\\n}\\n\\ninterface RTCIceParameters {\\n    password?: string;\\n    usernameFragment?: string;\\n}\\n\\ninterface RTCIceServer {\\n    credential?: string | RTCOAuthCredential;\\n    credentialType?: RTCIceCredentialType;\\n    urls: string | string[];\\n    username?: string;\\n}\\n\\ninterface RTCIdentityProviderOptions {\\n    peerIdentity?: string;\\n    protocol?: string;\\n    usernameHint?: string;\\n}\\n\\ninterface RTCInboundRTPStreamStats extends RTCRTPStreamStats {\\n    bytesReceived?: number;\\n    fractionLost?: number;\\n    jitter?: number;\\n    packetsLost?: number;\\n    packetsReceived?: number;\\n}\\n\\ninterface RTCMediaStreamTrackStats extends RTCStats {\\n    audioLevel?: number;\\n    echoReturnLoss?: number;\\n    echoReturnLossEnhancement?: number;\\n    frameHeight?: number;\\n    frameWidth?: number;\\n    framesCorrupted?: number;\\n    framesDecoded?: number;\\n    framesDropped?: number;\\n    framesPerSecond?: number;\\n    framesReceived?: number;\\n    framesSent?: number;\\n    remoteSource?: boolean;\\n    ssrcIds?: string[];\\n    trackIdentifier?: string;\\n}\\n\\ninterface RTCOAuthCredential {\\n    accessToken: string;\\n    macKey: string;\\n}\\n\\ninterface RTCOfferAnswerOptions {\\n    voiceActivityDetection?: boolean;\\n}\\n\\ninterface RTCOfferOptions extends RTCOfferAnswerOptions {\\n    iceRestart?: boolean;\\n    offerToReceiveAudio?: boolean;\\n    offerToReceiveVideo?: boolean;\\n}\\n\\ninterface RTCOutboundRTPStreamStats extends RTCRTPStreamStats {\\n    bytesSent?: number;\\n    packetsSent?: number;\\n    roundTripTime?: number;\\n    targetBitrate?: number;\\n}\\n\\ninterface RTCPeerConnectionIceErrorEventInit extends EventInit {\\n    errorCode: number;\\n    hostCandidate?: string;\\n    statusText?: string;\\n    url?: string;\\n}\\n\\ninterface RTCPeerConnectionIceEventInit extends EventInit {\\n    candidate?: RTCIceCandidate | null;\\n    url?: string | null;\\n}\\n\\ninterface RTCRTPStreamStats extends RTCStats {\\n    associateStatsId?: string;\\n    codecId?: string;\\n    firCount?: number;\\n    isRemote?: boolean;\\n    mediaTrackId?: string;\\n    mediaType?: string;\\n    nackCount?: number;\\n    pliCount?: number;\\n    sliCount?: number;\\n    ssrc?: string;\\n    transportId?: string;\\n}\\n\\ninterface RTCRtcpFeedback {\\n    parameter?: string;\\n    type?: string;\\n}\\n\\ninterface RTCRtcpParameters {\\n    cname?: string;\\n    reducedSize?: boolean;\\n}\\n\\ninterface RTCRtpCapabilities {\\n    codecs: RTCRtpCodecCapability[];\\n    headerExtensions: RTCRtpHeaderExtensionCapability[];\\n}\\n\\ninterface RTCRtpCodecCapability {\\n    channels?: number;\\n    clockRate: number;\\n    mimeType: string;\\n    sdpFmtpLine?: string;\\n}\\n\\ninterface RTCRtpCodecParameters {\\n    channels?: number;\\n    clockRate: number;\\n    mimeType: string;\\n    payloadType: number;\\n    sdpFmtpLine?: string;\\n}\\n\\ninterface RTCRtpCodingParameters {\\n    rid?: string;\\n}\\n\\ninterface RTCRtpContributingSource {\\n    audioLevel?: number;\\n    source: number;\\n    timestamp: number;\\n}\\n\\ninterface RTCRtpDecodingParameters extends RTCRtpCodingParameters {\\n}\\n\\ninterface RTCRtpEncodingParameters extends RTCRtpCodingParameters {\\n    active?: boolean;\\n    codecPayloadType?: number;\\n    dtx?: RTCDtxStatus;\\n    maxBitrate?: number;\\n    maxFramerate?: number;\\n    priority?: RTCPriorityType;\\n    ptime?: number;\\n    scaleResolutionDownBy?: number;\\n}\\n\\ninterface RTCRtpFecParameters {\\n    mechanism?: string;\\n    ssrc?: number;\\n}\\n\\ninterface RTCRtpHeaderExtension {\\n    kind?: string;\\n    preferredEncrypt?: boolean;\\n    preferredId?: number;\\n    uri?: string;\\n}\\n\\ninterface RTCRtpHeaderExtensionCapability {\\n    uri?: string;\\n}\\n\\ninterface RTCRtpHeaderExtensionParameters {\\n    encrypted?: boolean;\\n    id: number;\\n    uri: string;\\n}\\n\\ninterface RTCRtpParameters {\\n    codecs: RTCRtpCodecParameters[];\\n    headerExtensions: RTCRtpHeaderExtensionParameters[];\\n    rtcp: RTCRtcpParameters;\\n}\\n\\ninterface RTCRtpReceiveParameters extends RTCRtpParameters {\\n    encodings: RTCRtpDecodingParameters[];\\n}\\n\\ninterface RTCRtpRtxParameters {\\n    ssrc?: number;\\n}\\n\\ninterface RTCRtpSendParameters extends RTCRtpParameters {\\n    degradationPreference?: RTCDegradationPreference;\\n    encodings: RTCRtpEncodingParameters[];\\n    transactionId: string;\\n}\\n\\ninterface RTCRtpSynchronizationSource extends RTCRtpContributingSource {\\n    voiceActivityFlag?: boolean;\\n}\\n\\ninterface RTCRtpTransceiverInit {\\n    direction?: RTCRtpTransceiverDirection;\\n    sendEncodings?: RTCRtpEncodingParameters[];\\n    streams?: MediaStream[];\\n}\\n\\ninterface RTCRtpUnhandled {\\n    muxId?: string;\\n    payloadType?: number;\\n    ssrc?: number;\\n}\\n\\ninterface RTCSessionDescriptionInit {\\n    sdp?: string;\\n    type: RTCSdpType;\\n}\\n\\ninterface RTCSrtpKeyParam {\\n    keyMethod?: string;\\n    keySalt?: string;\\n    lifetime?: string;\\n    mkiLength?: number;\\n    mkiValue?: number;\\n}\\n\\ninterface RTCSrtpSdesParameters {\\n    cryptoSuite?: string;\\n    keyParams?: RTCSrtpKeyParam[];\\n    sessionParams?: string[];\\n    tag?: number;\\n}\\n\\ninterface RTCSsrcRange {\\n    max?: number;\\n    min?: number;\\n}\\n\\ninterface RTCStats {\\n    id: string;\\n    timestamp: number;\\n    type: RTCStatsType;\\n}\\n\\ninterface RTCStatsEventInit extends EventInit {\\n    report: RTCStatsReport;\\n}\\n\\ninterface RTCStatsReport {\\n}\\n\\ninterface RTCTrackEventInit extends EventInit {\\n    receiver: RTCRtpReceiver;\\n    streams?: MediaStream[];\\n    track: MediaStreamTrack;\\n    transceiver: RTCRtpTransceiver;\\n}\\n\\ninterface RTCTransportStats extends RTCStats {\\n    activeConnection?: boolean;\\n    bytesReceived?: number;\\n    bytesSent?: number;\\n    localCertificateId?: string;\\n    remoteCertificateId?: string;\\n    rtcpTransportStatsId?: string;\\n    selectedCandidatePairId?: string;\\n}\\n\\ninterface RegistrationOptions {\\n    scope?: string;\\n    type?: WorkerType;\\n    updateViaCache?: ServiceWorkerUpdateViaCache;\\n}\\n\\ninterface RequestInit {\\n    body?: BodyInit | null;\\n    cache?: RequestCache;\\n    credentials?: RequestCredentials;\\n    headers?: HeadersInit;\\n    integrity?: string;\\n    keepalive?: boolean;\\n    method?: string;\\n    mode?: RequestMode;\\n    redirect?: RequestRedirect;\\n    referrer?: string;\\n    referrerPolicy?: ReferrerPolicy;\\n    signal?: AbortSignal | null;\\n    window?: any;\\n}\\n\\ninterface ResponseInit {\\n    headers?: HeadersInit;\\n    status?: number;\\n    statusText?: string;\\n}\\n\\ninterface RsaHashedImportParams extends Algorithm {\\n    hash: HashAlgorithmIdentifier;\\n}\\n\\ninterface RsaHashedKeyAlgorithm extends RsaKeyAlgorithm {\\n    hash: KeyAlgorithm;\\n}\\n\\ninterface RsaHashedKeyGenParams extends RsaKeyGenParams {\\n    hash: HashAlgorithmIdentifier;\\n}\\n\\ninterface RsaKeyAlgorithm extends KeyAlgorithm {\\n    modulusLength: number;\\n    publicExponent: BigInteger;\\n}\\n\\ninterface RsaKeyGenParams extends Algorithm {\\n    modulusLength: number;\\n    publicExponent: BigInteger;\\n}\\n\\ninterface RsaOaepParams extends Algorithm {\\n    label?: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface RsaOtherPrimesInfo {\\n    d?: string;\\n    r?: string;\\n    t?: string;\\n}\\n\\ninterface RsaPssParams extends Algorithm {\\n    saltLength: number;\\n}\\n\\ninterface SVGBoundingBoxOptions {\\n    clipped?: boolean;\\n    fill?: boolean;\\n    markers?: boolean;\\n    stroke?: boolean;\\n}\\n\\ninterface ScopedCredentialDescriptor {\\n    id: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null;\\n    transports?: Transport[];\\n    type: ScopedCredentialType;\\n}\\n\\ninterface ScopedCredentialOptions {\\n    excludeList?: ScopedCredentialDescriptor[];\\n    extensions?: WebAuthnExtensions;\\n    rpId?: string;\\n    timeoutSeconds?: number;\\n}\\n\\ninterface ScopedCredentialParameters {\\n    algorithm: string | Algorithm;\\n    type: ScopedCredentialType;\\n}\\n\\ninterface ScrollIntoViewOptions extends ScrollOptions {\\n    block?: ScrollLogicalPosition;\\n    inline?: ScrollLogicalPosition;\\n}\\n\\ninterface ScrollOptions {\\n    behavior?: ScrollBehavior;\\n}\\n\\ninterface ScrollToOptions extends ScrollOptions {\\n    left?: number;\\n    top?: number;\\n}\\n\\ninterface SecurityPolicyViolationEventInit extends EventInit {\\n    blockedURI?: string;\\n    columnNumber?: number;\\n    documentURI?: string;\\n    effectiveDirective?: string;\\n    lineNumber?: number;\\n    originalPolicy?: string;\\n    referrer?: string;\\n    sourceFile?: string;\\n    statusCode?: number;\\n    violatedDirective?: string;\\n}\\n\\ninterface ServiceWorkerMessageEventInit extends EventInit {\\n    data?: any;\\n    lastEventId?: string;\\n    origin?: string;\\n    ports?: MessagePort[] | null;\\n    source?: ServiceWorker | MessagePort | null;\\n}\\n\\ninterface StereoPannerOptions extends AudioNodeOptions {\\n    pan?: number;\\n}\\n\\ninterface StorageEstimate {\\n    quota?: number;\\n    usage?: number;\\n}\\n\\ninterface StorageEventInit extends EventInit {\\n    key?: string | null;\\n    newValue?: string | null;\\n    oldValue?: string | null;\\n    storageArea?: Storage | null;\\n    url?: string;\\n}\\n\\ninterface StoreExceptionsInformation extends ExceptionInformation {\\n    detailURI?: string | null;\\n    explanationString?: string | null;\\n    siteName?: string | null;\\n}\\n\\ninterface StoreSiteSpecificExceptionsInformation extends StoreExceptionsInformation {\\n    arrayOfDomainStrings?: string[];\\n}\\n\\ninterface TextDecodeOptions {\\n    stream?: boolean;\\n}\\n\\ninterface TextDecoderOptions {\\n    fatal?: boolean;\\n    ignoreBOM?: boolean;\\n}\\n\\ninterface TouchEventInit extends EventModifierInit {\\n    changedTouches?: Touch[];\\n    targetTouches?: Touch[];\\n    touches?: Touch[];\\n}\\n\\ninterface TouchInit {\\n    altitudeAngle?: number;\\n    azimuthAngle?: number;\\n    clientX?: number;\\n    clientY?: number;\\n    force?: number;\\n    identifier: number;\\n    pageX?: number;\\n    pageY?: number;\\n    radiusX?: number;\\n    radiusY?: number;\\n    rotationAngle?: number;\\n    screenX?: number;\\n    screenY?: number;\\n    target: EventTarget;\\n    touchType?: TouchType;\\n}\\n\\ninterface TrackEventInit extends EventInit {\\n    track?: VideoTrack | AudioTrack | TextTrack | null;\\n}\\n\\ninterface TransitionEventInit extends EventInit {\\n    elapsedTime?: number;\\n    propertyName?: string;\\n    pseudoElement?: string;\\n}\\n\\ninterface UIEventInit extends EventInit {\\n    detail?: number;\\n    view?: Window | null;\\n}\\n\\ninterface UnderlyingSink {\\n    abort?: WritableStreamErrorCallback;\\n    close?: WritableStreamDefaultControllerCallback;\\n    start: WritableStreamDefaultControllerCallback;\\n    write?: WritableStreamChunkCallback;\\n}\\n\\ninterface VRDisplayEventInit extends EventInit {\\n    display: VRDisplay;\\n    reason?: VRDisplayEventReason;\\n}\\n\\ninterface VRLayer {\\n    leftBounds?: number[] | Float32Array | null;\\n    rightBounds?: number[] | Float32Array | null;\\n    source?: HTMLCanvasElement | null;\\n}\\n\\ninterface VRStageParameters {\\n    sittingToStandingTransform?: Float32Array;\\n    sizeX?: number;\\n    sizeY?: number;\\n}\\n\\ninterface WaveShaperOptions extends AudioNodeOptions {\\n    curve?: number[] | Float32Array;\\n    oversample?: OverSampleType;\\n}\\n\\ninterface WebAuthnExtensions {\\n}\\n\\ninterface WebGLContextAttributes {\\n    alpha?: GLboolean;\\n    antialias?: GLboolean;\\n    depth?: GLboolean;\\n    failIfMajorPerformanceCaveat?: boolean;\\n    powerPreference?: WebGLPowerPreference;\\n    premultipliedAlpha?: GLboolean;\\n    preserveDrawingBuffer?: GLboolean;\\n    stencil?: GLboolean;\\n}\\n\\ninterface WebGLContextEventInit extends EventInit {\\n    statusMessage?: string;\\n}\\n\\ninterface WheelEventInit extends MouseEventInit {\\n    deltaMode?: number;\\n    deltaX?: number;\\n    deltaY?: number;\\n    deltaZ?: number;\\n}\\n\\ninterface WorkletOptions {\\n    credentials?: RequestCredentials;\\n}\\n\\ninterface EventListener {\\n    (evt: Event): void;\\n}\\n\\ninterface ANGLE_instanced_arrays {\\n    drawArraysInstancedANGLE(mode: number, first: number, count: number, primcount: number): void;\\n    drawElementsInstancedANGLE(mode: number, count: number, type: number, offset: number, primcount: number): void;\\n    vertexAttribDivisorANGLE(index: number, divisor: number): void;\\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE: number;\\n}\\n\\ndeclare var ANGLE_instanced_arrays: {\\n    prototype: ANGLE_instanced_arrays;\\n    new(): ANGLE_instanced_arrays;\\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE: number;\\n};\\n\\ninterface AbortController {\\n    /**\\n     * Returns the AbortSignal object associated with this object.\\n     */\\n    readonly signal: AbortSignal;\\n    /**\\n     * Invoking this method will set this object\\'s AbortSignal\\'s aborted flag and\\n     * signal to any observers that the associated activity is to be aborted.\\n     */\\n    abort(): void;\\n}\\n\\ndeclare var AbortController: {\\n    prototype: AbortController;\\n    new(): AbortController;\\n};\\n\\ninterface AbortSignalEventMap {\\n    \"abort\": ProgressEvent;\\n}\\n\\ninterface AbortSignal extends EventTarget {\\n    /**\\n     * Returns true if this AbortSignal\\'s AbortController has signaled to abort, and false\\n     * otherwise.\\n     */\\n    readonly aborted: boolean;\\n    onabort: ((this: AbortSignal, ev: ProgressEvent) => any) | null;\\n    addEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AbortSignal: {\\n    prototype: AbortSignal;\\n    new(): AbortSignal;\\n};\\n\\ninterface AbstractRange {\\n    readonly collapsed: boolean;\\n    readonly endContainer: Node;\\n    readonly endOffset: number;\\n    readonly startContainer: Node;\\n    readonly startOffset: number;\\n}\\n\\ndeclare var AbstractRange: {\\n    prototype: AbstractRange;\\n    new(): AbstractRange;\\n};\\n\\ninterface AbstractWorkerEventMap {\\n    \"error\": ErrorEvent;\\n}\\n\\ninterface AbstractWorker {\\n    onerror: ((this: AbstractWorker, ev: ErrorEvent) => any) | null;\\n    addEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface AesCfbParams extends Algorithm {\\n    iv: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface AesCmacParams extends Algorithm {\\n    length: number;\\n}\\n\\ninterface AnalyserNode extends AudioNode {\\n    fftSize: number;\\n    readonly frequencyBinCount: number;\\n    maxDecibels: number;\\n    minDecibels: number;\\n    smoothingTimeConstant: number;\\n    getByteFrequencyData(array: Uint8Array): void;\\n    getByteTimeDomainData(array: Uint8Array): void;\\n    getFloatFrequencyData(array: Float32Array): void;\\n    getFloatTimeDomainData(array: Float32Array): void;\\n}\\n\\ndeclare var AnalyserNode: {\\n    prototype: AnalyserNode;\\n    new(context: BaseAudioContext, options?: AnalyserOptions): AnalyserNode;\\n};\\n\\ninterface Animatable {\\n    animate(keyframes: Keyframe[] | PropertyIndexedKeyframes | null, options?: number | KeyframeAnimationOptions): Animation;\\n    getAnimations(): Animation[];\\n}\\n\\ninterface AnimationEventMap {\\n    \"cancel\": AnimationPlaybackEvent;\\n    \"finish\": AnimationPlaybackEvent;\\n}\\n\\ninterface Animation extends EventTarget {\\n    currentTime: number | null;\\n    effect: AnimationEffect | null;\\n    readonly finished: Promise<Animation>;\\n    id: string;\\n    oncancel: ((this: Animation, ev: AnimationPlaybackEvent) => any) | null;\\n    onfinish: ((this: Animation, ev: AnimationPlaybackEvent) => any) | null;\\n    readonly pending: boolean;\\n    readonly playState: AnimationPlayState;\\n    playbackRate: number;\\n    readonly ready: Promise<Animation>;\\n    startTime: number | null;\\n    timeline: AnimationTimeline | null;\\n    cancel(): void;\\n    finish(): void;\\n    pause(): void;\\n    play(): void;\\n    reverse(): void;\\n    updatePlaybackRate(playbackRate: number): void;\\n    addEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: Animation, ev: AnimationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: Animation, ev: AnimationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Animation: {\\n    prototype: Animation;\\n    new(effect?: AnimationEffect | null, timeline?: AnimationTimeline | null): Animation;\\n};\\n\\ninterface AnimationEffect {\\n    getComputedTiming(): ComputedEffectTiming;\\n    getTiming(): EffectTiming;\\n    updateTiming(timing?: OptionalEffectTiming): void;\\n}\\n\\ndeclare var AnimationEffect: {\\n    prototype: AnimationEffect;\\n    new(): AnimationEffect;\\n};\\n\\ninterface AnimationEvent extends Event {\\n    readonly animationName: string;\\n    readonly elapsedTime: number;\\n    readonly pseudoElement: string;\\n}\\n\\ndeclare var AnimationEvent: {\\n    prototype: AnimationEvent;\\n    new(type: string, animationEventInitDict?: AnimationEventInit): AnimationEvent;\\n};\\n\\ninterface AnimationPlaybackEvent extends Event {\\n    readonly currentTime: number | null;\\n    readonly timelineTime: number | null;\\n}\\n\\ndeclare var AnimationPlaybackEvent: {\\n    prototype: AnimationPlaybackEvent;\\n    new(type: string, eventInitDict?: AnimationPlaybackEventInit): AnimationPlaybackEvent;\\n};\\n\\ninterface AnimationTimeline {\\n    readonly currentTime: number | null;\\n}\\n\\ndeclare var AnimationTimeline: {\\n    prototype: AnimationTimeline;\\n    new(): AnimationTimeline;\\n};\\n\\ninterface ApplicationCacheEventMap {\\n    \"cached\": Event;\\n    \"checking\": Event;\\n    \"downloading\": Event;\\n    \"error\": Event;\\n    \"noupdate\": Event;\\n    \"obsolete\": Event;\\n    \"progress\": ProgressEvent;\\n    \"updateready\": Event;\\n}\\n\\ninterface ApplicationCache extends EventTarget {\\n    /** @deprecated */\\n    oncached: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onchecking: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    ondownloading: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onerror: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onnoupdate: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onobsolete: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onprogress: ((this: ApplicationCache, ev: ProgressEvent) => any) | null;\\n    /** @deprecated */\\n    onupdateready: ((this: ApplicationCache, ev: Event) => any) | null;\\n    /** @deprecated */\\n    readonly status: number;\\n    /** @deprecated */\\n    abort(): void;\\n    /** @deprecated */\\n    swapCache(): void;\\n    /** @deprecated */\\n    update(): void;\\n    readonly CHECKING: number;\\n    readonly DOWNLOADING: number;\\n    readonly IDLE: number;\\n    readonly OBSOLETE: number;\\n    readonly UNCACHED: number;\\n    readonly UPDATEREADY: number;\\n    addEventListener<K extends keyof ApplicationCacheEventMap>(type: K, listener: (this: ApplicationCache, ev: ApplicationCacheEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ApplicationCacheEventMap>(type: K, listener: (this: ApplicationCache, ev: ApplicationCacheEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ApplicationCache: {\\n    prototype: ApplicationCache;\\n    new(): ApplicationCache;\\n    readonly CHECKING: number;\\n    readonly DOWNLOADING: number;\\n    readonly IDLE: number;\\n    readonly OBSOLETE: number;\\n    readonly UNCACHED: number;\\n    readonly UPDATEREADY: number;\\n};\\n\\ninterface Attr extends Node {\\n    readonly localName: string;\\n    readonly name: string;\\n    readonly namespaceURI: string | null;\\n    readonly ownerElement: Element | null;\\n    readonly prefix: string | null;\\n    readonly specified: boolean;\\n    value: string;\\n}\\n\\ndeclare var Attr: {\\n    prototype: Attr;\\n    new(): Attr;\\n};\\n\\ninterface AudioBuffer {\\n    readonly duration: number;\\n    readonly length: number;\\n    readonly numberOfChannels: number;\\n    readonly sampleRate: number;\\n    copyFromChannel(destination: Float32Array, channelNumber: number, startInChannel?: number): void;\\n    copyToChannel(source: Float32Array, channelNumber: number, startInChannel?: number): void;\\n    getChannelData(channel: number): Float32Array;\\n}\\n\\ndeclare var AudioBuffer: {\\n    prototype: AudioBuffer;\\n    new(options: AudioBufferOptions): AudioBuffer;\\n};\\n\\ninterface AudioBufferSourceNode extends AudioScheduledSourceNode {\\n    buffer: AudioBuffer | null;\\n    readonly detune: AudioParam;\\n    loop: boolean;\\n    loopEnd: number;\\n    loopStart: number;\\n    readonly playbackRate: AudioParam;\\n    start(when?: number, offset?: number, duration?: number): void;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioBufferSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioBufferSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioBufferSourceNode: {\\n    prototype: AudioBufferSourceNode;\\n    new(context: BaseAudioContext, options?: AudioBufferSourceOptions): AudioBufferSourceNode;\\n};\\n\\ninterface AudioContext extends BaseAudioContext {\\n    readonly baseLatency: number;\\n    readonly outputLatency: number;\\n    close(): Promise<void>;\\n    createMediaElementSource(mediaElement: HTMLMediaElement): MediaElementAudioSourceNode;\\n    createMediaStreamDestination(): MediaStreamAudioDestinationNode;\\n    createMediaStreamSource(mediaStream: MediaStream): MediaStreamAudioSourceNode;\\n    createMediaStreamTrackSource(mediaStreamTrack: MediaStreamTrack): MediaStreamTrackAudioSourceNode;\\n    getOutputTimestamp(): AudioTimestamp;\\n    suspend(): Promise<void>;\\n    addEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: AudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: AudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioContext: {\\n    prototype: AudioContext;\\n    new(contextOptions?: AudioContextOptions): AudioContext;\\n};\\n\\ninterface AudioDestinationNode extends AudioNode {\\n    readonly maxChannelCount: number;\\n}\\n\\ndeclare var AudioDestinationNode: {\\n    prototype: AudioDestinationNode;\\n    new(): AudioDestinationNode;\\n};\\n\\ninterface AudioListener {\\n    readonly forwardX: AudioParam;\\n    readonly forwardY: AudioParam;\\n    readonly forwardZ: AudioParam;\\n    readonly positionX: AudioParam;\\n    readonly positionY: AudioParam;\\n    readonly positionZ: AudioParam;\\n    readonly upX: AudioParam;\\n    readonly upY: AudioParam;\\n    readonly upZ: AudioParam;\\n    /** @deprecated */\\n    setOrientation(x: number, y: number, z: number, xUp: number, yUp: number, zUp: number): void;\\n    /** @deprecated */\\n    setPosition(x: number, y: number, z: number): void;\\n}\\n\\ndeclare var AudioListener: {\\n    prototype: AudioListener;\\n    new(): AudioListener;\\n};\\n\\ninterface AudioNode extends EventTarget {\\n    channelCount: number;\\n    channelCountMode: ChannelCountMode;\\n    channelInterpretation: ChannelInterpretation;\\n    readonly context: BaseAudioContext;\\n    readonly numberOfInputs: number;\\n    readonly numberOfOutputs: number;\\n    connect(destinationNode: AudioNode, output?: number, input?: number): AudioNode;\\n    connect(destinationParam: AudioParam, output?: number): void;\\n    disconnect(): void;\\n    disconnect(output: number): void;\\n    disconnect(destinationNode: AudioNode): void;\\n    disconnect(destinationNode: AudioNode, output: number): void;\\n    disconnect(destinationNode: AudioNode, output: number, input: number): void;\\n    disconnect(destinationParam: AudioParam): void;\\n    disconnect(destinationParam: AudioParam, output: number): void;\\n}\\n\\ndeclare var AudioNode: {\\n    prototype: AudioNode;\\n    new(): AudioNode;\\n};\\n\\ninterface AudioParam {\\n    automationRate: AutomationRate;\\n    readonly defaultValue: number;\\n    readonly maxValue: number;\\n    readonly minValue: number;\\n    value: number;\\n    cancelAndHoldAtTime(cancelTime: number): AudioParam;\\n    cancelScheduledValues(cancelTime: number): AudioParam;\\n    exponentialRampToValueAtTime(value: number, endTime: number): AudioParam;\\n    linearRampToValueAtTime(value: number, endTime: number): AudioParam;\\n    setTargetAtTime(target: number, startTime: number, timeConstant: number): AudioParam;\\n    setValueAtTime(value: number, startTime: number): AudioParam;\\n    setValueCurveAtTime(values: number[] | Float32Array, startTime: number, duration: number): AudioParam;\\n}\\n\\ndeclare var AudioParam: {\\n    prototype: AudioParam;\\n    new(): AudioParam;\\n};\\n\\ninterface AudioParamMap {\\n    forEach(callbackfn: (value: AudioParam, key: string, parent: AudioParamMap) => void, thisArg?: any): void;\\n}\\n\\ndeclare var AudioParamMap: {\\n    prototype: AudioParamMap;\\n    new(): AudioParamMap;\\n};\\n\\ninterface AudioProcessingEvent extends Event {\\n    readonly inputBuffer: AudioBuffer;\\n    readonly outputBuffer: AudioBuffer;\\n    readonly playbackTime: number;\\n}\\n\\ndeclare var AudioProcessingEvent: {\\n    prototype: AudioProcessingEvent;\\n    new(type: string, eventInitDict: AudioProcessingEventInit): AudioProcessingEvent;\\n};\\n\\ninterface AudioScheduledSourceNodeEventMap {\\n    \"ended\": Event;\\n}\\n\\ninterface AudioScheduledSourceNode extends AudioNode {\\n    onended: ((this: AudioScheduledSourceNode, ev: Event) => any) | null;\\n    start(when?: number): void;\\n    stop(when?: number): void;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioScheduledSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioScheduledSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioScheduledSourceNode: {\\n    prototype: AudioScheduledSourceNode;\\n    new(): AudioScheduledSourceNode;\\n};\\n\\ninterface AudioTrack {\\n    enabled: boolean;\\n    readonly id: string;\\n    kind: string;\\n    readonly label: string;\\n    language: string;\\n    readonly sourceBuffer: SourceBuffer;\\n}\\n\\ndeclare var AudioTrack: {\\n    prototype: AudioTrack;\\n    new(): AudioTrack;\\n};\\n\\ninterface AudioTrackListEventMap {\\n    \"addtrack\": TrackEvent;\\n    \"change\": Event;\\n    \"removetrack\": TrackEvent;\\n}\\n\\ninterface AudioTrackList extends EventTarget {\\n    readonly length: number;\\n    onaddtrack: ((this: AudioTrackList, ev: TrackEvent) => any) | null;\\n    onchange: ((this: AudioTrackList, ev: Event) => any) | null;\\n    onremovetrack: ((this: AudioTrackList, ev: TrackEvent) => any) | null;\\n    getTrackById(id: string): AudioTrack | null;\\n    item(index: number): AudioTrack;\\n    addEventListener<K extends keyof AudioTrackListEventMap>(type: K, listener: (this: AudioTrackList, ev: AudioTrackListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioTrackListEventMap>(type: K, listener: (this: AudioTrackList, ev: AudioTrackListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: AudioTrack;\\n}\\n\\ndeclare var AudioTrackList: {\\n    prototype: AudioTrackList;\\n    new(): AudioTrackList;\\n};\\n\\ninterface AudioWorklet extends Worklet {\\n}\\n\\ndeclare var AudioWorklet: {\\n    prototype: AudioWorklet;\\n    new(): AudioWorklet;\\n};\\n\\ninterface AudioWorkletNodeEventMap {\\n    \"processorerror\": Event;\\n}\\n\\ninterface AudioWorkletNode extends AudioNode {\\n    onprocessorerror: ((this: AudioWorkletNode, ev: Event) => any) | null;\\n    readonly parameters: AudioParamMap;\\n    readonly port: MessagePort;\\n    addEventListener<K extends keyof AudioWorkletNodeEventMap>(type: K, listener: (this: AudioWorkletNode, ev: AudioWorkletNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioWorkletNodeEventMap>(type: K, listener: (this: AudioWorkletNode, ev: AudioWorkletNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var AudioWorkletNode: {\\n    prototype: AudioWorkletNode;\\n    new(context: BaseAudioContext, name: string, options?: AudioWorkletNodeOptions): AudioWorkletNode;\\n};\\n\\ninterface BarProp {\\n    readonly visible: boolean;\\n}\\n\\ndeclare var BarProp: {\\n    prototype: BarProp;\\n    new(): BarProp;\\n};\\n\\ninterface BaseAudioContextEventMap {\\n    \"statechange\": Event;\\n}\\n\\ninterface BaseAudioContext extends EventTarget {\\n    readonly audioWorklet: AudioWorklet;\\n    readonly currentTime: number;\\n    readonly destination: AudioDestinationNode;\\n    readonly listener: AudioListener;\\n    onstatechange: ((this: BaseAudioContext, ev: Event) => any) | null;\\n    readonly sampleRate: number;\\n    readonly state: AudioContextState;\\n    createAnalyser(): AnalyserNode;\\n    createBiquadFilter(): BiquadFilterNode;\\n    createBuffer(numberOfChannels: number, length: number, sampleRate: number): AudioBuffer;\\n    createBufferSource(): AudioBufferSourceNode;\\n    createChannelMerger(numberOfInputs?: number): ChannelMergerNode;\\n    createChannelSplitter(numberOfOutputs?: number): ChannelSplitterNode;\\n    createConstantSource(): ConstantSourceNode;\\n    createConvolver(): ConvolverNode;\\n    createDelay(maxDelayTime?: number): DelayNode;\\n    createDynamicsCompressor(): DynamicsCompressorNode;\\n    createGain(): GainNode;\\n    createIIRFilter(feedforward: number[], feedback: number[]): IIRFilterNode;\\n    createOscillator(): OscillatorNode;\\n    createPanner(): PannerNode;\\n    createPeriodicWave(real: number[] | Float32Array, imag: number[] | Float32Array, constraints?: PeriodicWaveConstraints): PeriodicWave;\\n    createScriptProcessor(bufferSize?: number, numberOfInputChannels?: number, numberOfOutputChannels?: number): ScriptProcessorNode;\\n    createStereoPanner(): StereoPannerNode;\\n    createWaveShaper(): WaveShaperNode;\\n    decodeAudioData(audioData: ArrayBuffer, successCallback?: DecodeSuccessCallback | null, errorCallback?: DecodeErrorCallback | null): Promise<AudioBuffer>;\\n    resume(): Promise<void>;\\n    addEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: BaseAudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: BaseAudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var BaseAudioContext: {\\n    prototype: BaseAudioContext;\\n    new(): BaseAudioContext;\\n};\\n\\ninterface BeforeUnloadEvent extends Event {\\n    returnValue: any;\\n}\\n\\ndeclare var BeforeUnloadEvent: {\\n    prototype: BeforeUnloadEvent;\\n    new(): BeforeUnloadEvent;\\n};\\n\\ninterface BhxBrowser {\\n    readonly lastError: DOMException;\\n    checkMatchesGlobExpression(pattern: string, value: string): boolean;\\n    checkMatchesUriExpression(pattern: string, value: string): boolean;\\n    clearLastError(): void;\\n    currentWindowId(): number;\\n    fireExtensionApiTelemetry(functionName: string, isSucceeded: boolean, isSupported: boolean, errorString: string): void;\\n    genericFunction(functionId: number, destination: any, parameters?: string, callbackId?: number): void;\\n    genericSynchronousFunction(functionId: number, parameters?: string): string;\\n    getExtensionId(): string;\\n    getThisAddress(): any;\\n    registerGenericFunctionCallbackHandler(callbackHandler: Function): void;\\n    registerGenericListenerHandler(eventHandler: Function): void;\\n    setLastError(parameters: string): void;\\n    webPlatformGenericFunction(destination: any, parameters?: string, callbackId?: number): void;\\n}\\n\\ndeclare var BhxBrowser: {\\n    prototype: BhxBrowser;\\n    new(): BhxBrowser;\\n};\\n\\ninterface BiquadFilterNode extends AudioNode {\\n    readonly Q: AudioParam;\\n    readonly detune: AudioParam;\\n    readonly frequency: AudioParam;\\n    readonly gain: AudioParam;\\n    type: BiquadFilterType;\\n    getFrequencyResponse(frequencyHz: Float32Array, magResponse: Float32Array, phaseResponse: Float32Array): void;\\n}\\n\\ndeclare var BiquadFilterNode: {\\n    prototype: BiquadFilterNode;\\n    new(context: BaseAudioContext, options?: BiquadFilterOptions): BiquadFilterNode;\\n};\\n\\ninterface Blob {\\n    readonly size: number;\\n    readonly type: string;\\n    slice(start?: number, end?: number, contentType?: string): Blob;\\n}\\n\\ndeclare var Blob: {\\n    prototype: Blob;\\n    new(blobParts?: BlobPart[], options?: BlobPropertyBag): Blob;\\n};\\n\\ninterface Body {\\n    readonly body: ReadableStream | null;\\n    readonly bodyUsed: boolean;\\n    arrayBuffer(): Promise<ArrayBuffer>;\\n    blob(): Promise<Blob>;\\n    formData(): Promise<FormData>;\\n    json(): Promise<any>;\\n    text(): Promise<string>;\\n}\\n\\ninterface BroadcastChannelEventMap {\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n}\\n\\ninterface BroadcastChannel extends EventTarget {\\n    /**\\n     * Returns the channel name (as passed to the constructor).\\n     */\\n    readonly name: string;\\n    onmessage: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\\n    /**\\n     * Closes the BroadcastChannel object, opening it up to garbage collection.\\n     */\\n    close(): void;\\n    /**\\n     * Sends the given message to other BroadcastChannel objects set up for this channel. Messages can be structured objects, e.g. nested objects and arrays.\\n     */\\n    postMessage(message: any): void;\\n    addEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var BroadcastChannel: {\\n    prototype: BroadcastChannel;\\n    new(name: string): BroadcastChannel;\\n};\\n\\ninterface BroadcastChannelEventMap {\\n    message: MessageEvent;\\n    messageerror: MessageEvent;\\n}\\n\\ninterface ByteLengthQueuingStrategy {\\n    highWaterMark: number;\\n    size(chunk?: any): number;\\n}\\n\\ndeclare var ByteLengthQueuingStrategy: {\\n    prototype: ByteLengthQueuingStrategy;\\n    new(strategy: QueuingStrategy): ByteLengthQueuingStrategy;\\n};\\n\\ninterface CDATASection extends Text {\\n}\\n\\ndeclare var CDATASection: {\\n    prototype: CDATASection;\\n    new(): CDATASection;\\n};\\n\\ninterface CSS {\\n    escape(value: string): string;\\n    supports(property: string, value?: string): boolean;\\n}\\ndeclare var CSS: CSS;\\n\\ninterface CSSConditionRule extends CSSGroupingRule {\\n    conditionText: string;\\n}\\n\\ndeclare var CSSConditionRule: {\\n    prototype: CSSConditionRule;\\n    new(): CSSConditionRule;\\n};\\n\\ninterface CSSFontFaceRule extends CSSRule {\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSFontFaceRule: {\\n    prototype: CSSFontFaceRule;\\n    new(): CSSFontFaceRule;\\n};\\n\\ninterface CSSGroupingRule extends CSSRule {\\n    readonly cssRules: CSSRuleList;\\n    deleteRule(index: number): void;\\n    insertRule(rule: string, index: number): number;\\n}\\n\\ndeclare var CSSGroupingRule: {\\n    prototype: CSSGroupingRule;\\n    new(): CSSGroupingRule;\\n};\\n\\ninterface CSSImportRule extends CSSRule {\\n    readonly href: string;\\n    readonly media: MediaList;\\n    readonly styleSheet: CSSStyleSheet;\\n}\\n\\ndeclare var CSSImportRule: {\\n    prototype: CSSImportRule;\\n    new(): CSSImportRule;\\n};\\n\\ninterface CSSKeyframeRule extends CSSRule {\\n    keyText: string;\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSKeyframeRule: {\\n    prototype: CSSKeyframeRule;\\n    new(): CSSKeyframeRule;\\n};\\n\\ninterface CSSKeyframesRule extends CSSRule {\\n    readonly cssRules: CSSRuleList;\\n    name: string;\\n    appendRule(rule: string): void;\\n    deleteRule(select: string): void;\\n    findRule(select: string): CSSKeyframeRule | null;\\n}\\n\\ndeclare var CSSKeyframesRule: {\\n    prototype: CSSKeyframesRule;\\n    new(): CSSKeyframesRule;\\n};\\n\\ninterface CSSMediaRule extends CSSConditionRule {\\n    readonly media: MediaList;\\n}\\n\\ndeclare var CSSMediaRule: {\\n    prototype: CSSMediaRule;\\n    new(): CSSMediaRule;\\n};\\n\\ninterface CSSNamespaceRule extends CSSRule {\\n    readonly namespaceURI: string;\\n    readonly prefix: string;\\n}\\n\\ndeclare var CSSNamespaceRule: {\\n    prototype: CSSNamespaceRule;\\n    new(): CSSNamespaceRule;\\n};\\n\\ninterface CSSPageRule extends CSSRule {\\n    readonly pseudoClass: string;\\n    readonly selector: string;\\n    selectorText: string;\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSPageRule: {\\n    prototype: CSSPageRule;\\n    new(): CSSPageRule;\\n};\\n\\ninterface CSSRule {\\n    cssText: string;\\n    readonly parentRule: CSSRule | null;\\n    readonly parentStyleSheet: CSSStyleSheet | null;\\n    readonly type: number;\\n    readonly CHARSET_RULE: number;\\n    readonly FONT_FACE_RULE: number;\\n    readonly IMPORT_RULE: number;\\n    readonly KEYFRAMES_RULE: number;\\n    readonly KEYFRAME_RULE: number;\\n    readonly MEDIA_RULE: number;\\n    readonly NAMESPACE_RULE: number;\\n    readonly PAGE_RULE: number;\\n    readonly STYLE_RULE: number;\\n    readonly SUPPORTS_RULE: number;\\n    readonly UNKNOWN_RULE: number;\\n    readonly VIEWPORT_RULE: number;\\n}\\n\\ndeclare var CSSRule: {\\n    prototype: CSSRule;\\n    new(): CSSRule;\\n    readonly CHARSET_RULE: number;\\n    readonly FONT_FACE_RULE: number;\\n    readonly IMPORT_RULE: number;\\n    readonly KEYFRAMES_RULE: number;\\n    readonly KEYFRAME_RULE: number;\\n    readonly MEDIA_RULE: number;\\n    readonly NAMESPACE_RULE: number;\\n    readonly PAGE_RULE: number;\\n    readonly STYLE_RULE: number;\\n    readonly SUPPORTS_RULE: number;\\n    readonly UNKNOWN_RULE: number;\\n    readonly VIEWPORT_RULE: number;\\n};\\n\\ninterface CSSRuleList {\\n    readonly length: number;\\n    item(index: number): CSSRule | null;\\n    [index: number]: CSSRule;\\n}\\n\\ndeclare var CSSRuleList: {\\n    prototype: CSSRuleList;\\n    new(): CSSRuleList;\\n};\\n\\ninterface CSSStyleDeclaration {\\n    alignContent: string | null;\\n    alignItems: string | null;\\n    alignSelf: string | null;\\n    alignmentBaseline: string | null;\\n    animation: string | null;\\n    animationDelay: string | null;\\n    animationDirection: string | null;\\n    animationDuration: string | null;\\n    animationFillMode: string | null;\\n    animationIterationCount: string | null;\\n    animationName: string | null;\\n    animationPlayState: string | null;\\n    animationTimingFunction: string | null;\\n    backfaceVisibility: string | null;\\n    background: string | null;\\n    backgroundAttachment: string | null;\\n    backgroundClip: string | null;\\n    backgroundColor: string | null;\\n    backgroundImage: string | null;\\n    backgroundOrigin: string | null;\\n    backgroundPosition: string | null;\\n    backgroundPositionX: string | null;\\n    backgroundPositionY: string | null;\\n    backgroundRepeat: string | null;\\n    backgroundSize: string | null;\\n    baselineShift: string | null;\\n    border: string | null;\\n    borderBottom: string | null;\\n    borderBottomColor: string | null;\\n    borderBottomLeftRadius: string | null;\\n    borderBottomRightRadius: string | null;\\n    borderBottomStyle: string | null;\\n    borderBottomWidth: string | null;\\n    borderCollapse: string | null;\\n    borderColor: string | null;\\n    borderImage: string | null;\\n    borderImageOutset: string | null;\\n    borderImageRepeat: string | null;\\n    borderImageSlice: string | null;\\n    borderImageSource: string | null;\\n    borderImageWidth: string | null;\\n    borderLeft: string | null;\\n    borderLeftColor: string | null;\\n    borderLeftStyle: string | null;\\n    borderLeftWidth: string | null;\\n    borderRadius: string | null;\\n    borderRight: string | null;\\n    borderRightColor: string | null;\\n    borderRightStyle: string | null;\\n    borderRightWidth: string | null;\\n    borderSpacing: string | null;\\n    borderStyle: string | null;\\n    borderTop: string | null;\\n    borderTopColor: string | null;\\n    borderTopLeftRadius: string | null;\\n    borderTopRightRadius: string | null;\\n    borderTopStyle: string | null;\\n    borderTopWidth: string | null;\\n    borderWidth: string | null;\\n    bottom: string | null;\\n    boxShadow: string | null;\\n    boxSizing: string | null;\\n    breakAfter: string | null;\\n    breakBefore: string | null;\\n    breakInside: string | null;\\n    captionSide: string | null;\\n    clear: string | null;\\n    clip: string | null;\\n    clipPath: string | null;\\n    clipRule: string | null;\\n    color: string | null;\\n    colorInterpolationFilters: string | null;\\n    columnCount: any;\\n    columnFill: string | null;\\n    columnGap: any;\\n    columnRule: string | null;\\n    columnRuleColor: any;\\n    columnRuleStyle: string | null;\\n    columnRuleWidth: any;\\n    columnSpan: string | null;\\n    columnWidth: any;\\n    columns: string | null;\\n    content: string | null;\\n    counterIncrement: string | null;\\n    counterReset: string | null;\\n    cssFloat: string | null;\\n    cssText: string;\\n    cursor: string | null;\\n    direction: string | null;\\n    display: string | null;\\n    dominantBaseline: string | null;\\n    emptyCells: string | null;\\n    enableBackground: string | null;\\n    fill: string | null;\\n    fillOpacity: string | null;\\n    fillRule: string | null;\\n    filter: string | null;\\n    flex: string | null;\\n    flexBasis: string | null;\\n    flexDirection: string | null;\\n    flexFlow: string | null;\\n    flexGrow: string | null;\\n    flexShrink: string | null;\\n    flexWrap: string | null;\\n    floodColor: string | null;\\n    floodOpacity: string | null;\\n    font: string | null;\\n    fontFamily: string | null;\\n    fontFeatureSettings: string | null;\\n    fontSize: string | null;\\n    fontSizeAdjust: string | null;\\n    fontStretch: string | null;\\n    fontStyle: string | null;\\n    fontVariant: string | null;\\n    fontWeight: string | null;\\n    gap: string | null;\\n    glyphOrientationHorizontal: string | null;\\n    glyphOrientationVertical: string | null;\\n    grid: string | null;\\n    gridArea: string | null;\\n    gridAutoColumns: string | null;\\n    gridAutoFlow: string | null;\\n    gridAutoRows: string | null;\\n    gridColumn: string | null;\\n    gridColumnEnd: string | null;\\n    gridColumnGap: string | null;\\n    gridColumnStart: string | null;\\n    gridGap: string | null;\\n    gridRow: string | null;\\n    gridRowEnd: string | null;\\n    gridRowGap: string | null;\\n    gridRowStart: string | null;\\n    gridTemplate: string | null;\\n    gridTemplateAreas: string | null;\\n    gridTemplateColumns: string | null;\\n    gridTemplateRows: string | null;\\n    height: string | null;\\n    imeMode: string | null;\\n    justifyContent: string | null;\\n    justifyItems: string | null;\\n    justifySelf: string | null;\\n    kerning: string | null;\\n    layoutGrid: string | null;\\n    layoutGridChar: string | null;\\n    layoutGridLine: string | null;\\n    layoutGridMode: string | null;\\n    layoutGridType: string | null;\\n    left: string | null;\\n    readonly length: number;\\n    letterSpacing: string | null;\\n    lightingColor: string | null;\\n    lineBreak: string | null;\\n    lineHeight: string | null;\\n    listStyle: string | null;\\n    listStyleImage: string | null;\\n    listStylePosition: string | null;\\n    listStyleType: string | null;\\n    margin: string | null;\\n    marginBottom: string | null;\\n    marginLeft: string | null;\\n    marginRight: string | null;\\n    marginTop: string | null;\\n    marker: string | null;\\n    markerEnd: string | null;\\n    markerMid: string | null;\\n    markerStart: string | null;\\n    mask: string | null;\\n    maskImage: string | null;\\n    maxHeight: string | null;\\n    maxWidth: string | null;\\n    minHeight: string | null;\\n    minWidth: string | null;\\n    msContentZoomChaining: string | null;\\n    msContentZoomLimit: string | null;\\n    msContentZoomLimitMax: any;\\n    msContentZoomLimitMin: any;\\n    msContentZoomSnap: string | null;\\n    msContentZoomSnapPoints: string | null;\\n    msContentZoomSnapType: string | null;\\n    msContentZooming: string | null;\\n    msFlowFrom: string | null;\\n    msFlowInto: string | null;\\n    msFontFeatureSettings: string | null;\\n    msGridColumn: any;\\n    msGridColumnAlign: string | null;\\n    msGridColumnSpan: any;\\n    msGridColumns: string | null;\\n    msGridRow: any;\\n    msGridRowAlign: string | null;\\n    msGridRowSpan: any;\\n    msGridRows: string | null;\\n    msHighContrastAdjust: string | null;\\n    msHyphenateLimitChars: string | null;\\n    msHyphenateLimitLines: any;\\n    msHyphenateLimitZone: any;\\n    msHyphens: string | null;\\n    msImeAlign: string | null;\\n    msOverflowStyle: string | null;\\n    msScrollChaining: string | null;\\n    msScrollLimit: string | null;\\n    msScrollLimitXMax: any;\\n    msScrollLimitXMin: any;\\n    msScrollLimitYMax: any;\\n    msScrollLimitYMin: any;\\n    msScrollRails: string | null;\\n    msScrollSnapPointsX: string | null;\\n    msScrollSnapPointsY: string | null;\\n    msScrollSnapType: string | null;\\n    msScrollSnapX: string | null;\\n    msScrollSnapY: string | null;\\n    msScrollTranslation: string | null;\\n    msTextCombineHorizontal: string | null;\\n    msTextSizeAdjust: any;\\n    msTouchAction: string | null;\\n    msTouchSelect: string | null;\\n    msUserSelect: string | null;\\n    msWrapFlow: string;\\n    msWrapMargin: any;\\n    msWrapThrough: string;\\n    objectFit: string | null;\\n    objectPosition: string | null;\\n    opacity: string | null;\\n    order: string | null;\\n    orphans: string | null;\\n    outline: string | null;\\n    outlineColor: string | null;\\n    outlineOffset: string | null;\\n    outlineStyle: string | null;\\n    outlineWidth: string | null;\\n    overflow: string | null;\\n    overflowX: string | null;\\n    overflowY: string | null;\\n    padding: string | null;\\n    paddingBottom: string | null;\\n    paddingLeft: string | null;\\n    paddingRight: string | null;\\n    paddingTop: string | null;\\n    pageBreakAfter: string | null;\\n    pageBreakBefore: string | null;\\n    pageBreakInside: string | null;\\n    readonly parentRule: CSSRule;\\n    penAction: string | null;\\n    perspective: string | null;\\n    perspectiveOrigin: string | null;\\n    pointerEvents: string | null;\\n    position: string | null;\\n    quotes: string | null;\\n    resize: string | null;\\n    right: string | null;\\n    rotate: string | null;\\n    rowGap: string | null;\\n    rubyAlign: string | null;\\n    rubyOverhang: string | null;\\n    rubyPosition: string | null;\\n    scale: string | null;\\n    stopColor: string | null;\\n    stopOpacity: string | null;\\n    stroke: string | null;\\n    strokeDasharray: string | null;\\n    strokeDashoffset: string | null;\\n    strokeLinecap: string | null;\\n    strokeLinejoin: string | null;\\n    strokeMiterlimit: string | null;\\n    strokeOpacity: string | null;\\n    strokeWidth: string | null;\\n    tableLayout: string | null;\\n    textAlign: string | null;\\n    textAlignLast: string | null;\\n    textAnchor: string | null;\\n    textCombineUpright: string | null;\\n    textDecoration: string | null;\\n    textIndent: string | null;\\n    textJustify: string | null;\\n    textKashida: string | null;\\n    textKashidaSpace: string | null;\\n    textOverflow: string | null;\\n    textShadow: string | null;\\n    textTransform: string | null;\\n    textUnderlinePosition: string | null;\\n    top: string | null;\\n    touchAction: string | null;\\n    transform: string | null;\\n    transformOrigin: string | null;\\n    transformStyle: string | null;\\n    transition: string | null;\\n    transitionDelay: string | null;\\n    transitionDuration: string | null;\\n    transitionProperty: string | null;\\n    transitionTimingFunction: string | null;\\n    translate: string | null;\\n    unicodeBidi: string | null;\\n    userSelect: string | null;\\n    verticalAlign: string | null;\\n    visibility: string | null;\\n    webkitAlignContent: string | null;\\n    webkitAlignItems: string | null;\\n    webkitAlignSelf: string | null;\\n    webkitAnimation: string | null;\\n    webkitAnimationDelay: string | null;\\n    webkitAnimationDirection: string | null;\\n    webkitAnimationDuration: string | null;\\n    webkitAnimationFillMode: string | null;\\n    webkitAnimationIterationCount: string | null;\\n    webkitAnimationName: string | null;\\n    webkitAnimationPlayState: string | null;\\n    webkitAnimationTimingFunction: string | null;\\n    webkitAppearance: string | null;\\n    webkitBackfaceVisibility: string | null;\\n    webkitBackgroundClip: string | null;\\n    webkitBackgroundOrigin: string | null;\\n    webkitBackgroundSize: string | null;\\n    webkitBorderBottomLeftRadius: string | null;\\n    webkitBorderBottomRightRadius: string | null;\\n    webkitBorderImage: string | null;\\n    webkitBorderRadius: string | null;\\n    webkitBorderTopLeftRadius: string | null;\\n    webkitBorderTopRightRadius: string | null;\\n    webkitBoxAlign: string | null;\\n    webkitBoxDirection: string | null;\\n    webkitBoxFlex: string | null;\\n    webkitBoxOrdinalGroup: string | null;\\n    webkitBoxOrient: string | null;\\n    webkitBoxPack: string | null;\\n    webkitBoxSizing: string | null;\\n    webkitColumnBreakAfter: string | null;\\n    webkitColumnBreakBefore: string | null;\\n    webkitColumnBreakInside: string | null;\\n    webkitColumnCount: any;\\n    webkitColumnGap: any;\\n    webkitColumnRule: string | null;\\n    webkitColumnRuleColor: any;\\n    webkitColumnRuleStyle: string | null;\\n    webkitColumnRuleWidth: any;\\n    webkitColumnSpan: string | null;\\n    webkitColumnWidth: any;\\n    webkitColumns: string | null;\\n    webkitFilter: string | null;\\n    webkitFlex: string | null;\\n    webkitFlexBasis: string | null;\\n    webkitFlexDirection: string | null;\\n    webkitFlexFlow: string | null;\\n    webkitFlexGrow: string | null;\\n    webkitFlexShrink: string | null;\\n    webkitFlexWrap: string | null;\\n    webkitJustifyContent: string | null;\\n    webkitOrder: string | null;\\n    webkitPerspective: string | null;\\n    webkitPerspectiveOrigin: string | null;\\n    webkitTapHighlightColor: string | null;\\n    webkitTextFillColor: string | null;\\n    webkitTextSizeAdjust: any;\\n    webkitTextStroke: string | null;\\n    webkitTextStrokeColor: string | null;\\n    webkitTextStrokeWidth: string | null;\\n    webkitTransform: string | null;\\n    webkitTransformOrigin: string | null;\\n    webkitTransformStyle: string | null;\\n    webkitTransition: string | null;\\n    webkitTransitionDelay: string | null;\\n    webkitTransitionDuration: string | null;\\n    webkitTransitionProperty: string | null;\\n    webkitTransitionTimingFunction: string | null;\\n    webkitUserModify: string | null;\\n    webkitUserSelect: string | null;\\n    webkitWritingMode: string | null;\\n    whiteSpace: string | null;\\n    widows: string | null;\\n    width: string | null;\\n    wordBreak: string | null;\\n    wordSpacing: string | null;\\n    wordWrap: string | null;\\n    writingMode: string | null;\\n    zIndex: string | null;\\n    zoom: string | null;\\n    getPropertyPriority(propertyName: string): string;\\n    getPropertyValue(propertyName: string): string;\\n    item(index: number): string;\\n    removeProperty(propertyName: string): string;\\n    setProperty(propertyName: string, value: string | null, priority?: string | null): void;\\n    [index: number]: string;\\n}\\n\\ndeclare var CSSStyleDeclaration: {\\n    prototype: CSSStyleDeclaration;\\n    new(): CSSStyleDeclaration;\\n};\\n\\ninterface CSSStyleRule extends CSSRule {\\n    selectorText: string;\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ndeclare var CSSStyleRule: {\\n    prototype: CSSStyleRule;\\n    new(): CSSStyleRule;\\n};\\n\\ninterface CSSStyleSheet extends StyleSheet {\\n    readonly cssRules: CSSRuleList;\\n    /** @deprecated */\\n    cssText: string;\\n    /** @deprecated */\\n    readonly id: string;\\n    /** @deprecated */\\n    readonly imports: StyleSheetList;\\n    /** @deprecated */\\n    readonly isAlternate: boolean;\\n    /** @deprecated */\\n    readonly isPrefAlternate: boolean;\\n    readonly ownerRule: CSSRule | null;\\n    /** @deprecated */\\n    readonly owningElement: Element;\\n    /** @deprecated */\\n    readonly pages: any;\\n    /** @deprecated */\\n    readonly readOnly: boolean;\\n    readonly rules: CSSRuleList;\\n    /** @deprecated */\\n    addImport(bstrURL: string, lIndex?: number): number;\\n    /** @deprecated */\\n    addPageRule(bstrSelector: string, bstrStyle: string, lIndex?: number): number;\\n    addRule(bstrSelector: string, bstrStyle?: string, lIndex?: number): number;\\n    deleteRule(index?: number): void;\\n    insertRule(rule: string, index?: number): number;\\n    /** @deprecated */\\n    removeImport(lIndex: number): void;\\n    removeRule(lIndex: number): void;\\n}\\n\\ndeclare var CSSStyleSheet: {\\n    prototype: CSSStyleSheet;\\n    new(): CSSStyleSheet;\\n};\\n\\ninterface CSSSupportsRule extends CSSConditionRule {\\n}\\n\\ndeclare var CSSSupportsRule: {\\n    prototype: CSSSupportsRule;\\n    new(): CSSSupportsRule;\\n};\\n\\ninterface Cache {\\n    add(request: RequestInfo): Promise<void>;\\n    addAll(requests: RequestInfo[]): Promise<void>;\\n    delete(request: RequestInfo, options?: CacheQueryOptions): Promise<boolean>;\\n    keys(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Request>>;\\n    match(request: RequestInfo, options?: CacheQueryOptions): Promise<Response | undefined>;\\n    matchAll(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Response>>;\\n    put(request: RequestInfo, response: Response): Promise<void>;\\n}\\n\\ndeclare var Cache: {\\n    prototype: Cache;\\n    new(): Cache;\\n};\\n\\ninterface CacheStorage {\\n    delete(cacheName: string): Promise<boolean>;\\n    has(cacheName: string): Promise<boolean>;\\n    keys(): Promise<string[]>;\\n    match(request: RequestInfo, options?: CacheQueryOptions): Promise<Response | undefined>;\\n    open(cacheName: string): Promise<Cache>;\\n}\\n\\ndeclare var CacheStorage: {\\n    prototype: CacheStorage;\\n    new(): CacheStorage;\\n};\\n\\ninterface CanvasCompositing {\\n    globalAlpha: number;\\n    globalCompositeOperation: string;\\n}\\n\\ninterface CanvasDrawImage {\\n    drawImage(image: CanvasImageSource, dx: number, dy: number): void;\\n    drawImage(image: CanvasImageSource, dx: number, dy: number, dw: number, dh: number): void;\\n    drawImage(image: CanvasImageSource, sx: number, sy: number, sw: number, sh: number, dx: number, dy: number, dw: number, dh: number): void;\\n}\\n\\ninterface CanvasDrawPath {\\n    beginPath(): void;\\n    clip(fillRule?: CanvasFillRule): void;\\n    clip(path: Path2D, fillRule?: CanvasFillRule): void;\\n    fill(fillRule?: CanvasFillRule): void;\\n    fill(path: Path2D, fillRule?: CanvasFillRule): void;\\n    isPointInPath(x: number, y: number, fillRule?: CanvasFillRule): boolean;\\n    isPointInPath(path: Path2D, x: number, y: number, fillRule?: CanvasFillRule): boolean;\\n    isPointInStroke(x: number, y: number): boolean;\\n    isPointInStroke(path: Path2D, x: number, y: number): boolean;\\n    stroke(): void;\\n    stroke(path: Path2D): void;\\n}\\n\\ninterface CanvasFillStrokeStyles {\\n    fillStyle: string | CanvasGradient | CanvasPattern;\\n    strokeStyle: string | CanvasGradient | CanvasPattern;\\n    createLinearGradient(x0: number, y0: number, x1: number, y1: number): CanvasGradient;\\n    createPattern(image: CanvasImageSource, repetition: string): CanvasPattern | null;\\n    createRadialGradient(x0: number, y0: number, r0: number, x1: number, y1: number, r1: number): CanvasGradient;\\n}\\n\\ninterface CanvasFilters {\\n    filter: string;\\n}\\n\\ninterface CanvasGradient {\\n    /**\\n     * Adds a color stop with the given color to the gradient at the given offset. 0.0 is the offset\\n     * at one end of the gradient, 1.0 is the offset at the other end.\\n     * Throws an \"IndexSizeError\" DOMException if the offset\\n     * is out of range. Throws a \"SyntaxError\" DOMException if\\n     * the color cannot be parsed.\\n     */\\n    addColorStop(offset: number, color: string): void;\\n}\\n\\ndeclare var CanvasGradient: {\\n    prototype: CanvasGradient;\\n    new(): CanvasGradient;\\n};\\n\\ninterface CanvasImageData {\\n    createImageData(sw: number, sh: number): ImageData;\\n    createImageData(imagedata: ImageData): ImageData;\\n    getImageData(sx: number, sy: number, sw: number, sh: number): ImageData;\\n    putImageData(imagedata: ImageData, dx: number, dy: number): void;\\n    putImageData(imagedata: ImageData, dx: number, dy: number, dirtyX: number, dirtyY: number, dirtyWidth: number, dirtyHeight: number): void;\\n}\\n\\ninterface CanvasImageSmoothing {\\n    imageSmoothingEnabled: boolean;\\n    imageSmoothingQuality: ImageSmoothingQuality;\\n}\\n\\ninterface CanvasPath {\\n    arc(x: number, y: number, radius: number, startAngle: number, endAngle: number, anticlockwise?: boolean): void;\\n    arcTo(x1: number, y1: number, x2: number, y2: number, radius: number): void;\\n    bezierCurveTo(cp1x: number, cp1y: number, cp2x: number, cp2y: number, x: number, y: number): void;\\n    closePath(): void;\\n    ellipse(x: number, y: number, radiusX: number, radiusY: number, rotation: number, startAngle: number, endAngle: number, anticlockwise?: boolean): void;\\n    lineTo(x: number, y: number): void;\\n    moveTo(x: number, y: number): void;\\n    quadraticCurveTo(cpx: number, cpy: number, x: number, y: number): void;\\n    rect(x: number, y: number, w: number, h: number): void;\\n}\\n\\ninterface CanvasPathDrawingStyles {\\n    lineCap: CanvasLineCap;\\n    lineDashOffset: number;\\n    lineJoin: CanvasLineJoin;\\n    lineWidth: number;\\n    miterLimit: number;\\n    getLineDash(): number[];\\n    setLineDash(segments: number[]): void;\\n}\\n\\ninterface CanvasPattern {\\n    /**\\n     * Sets the transformation matrix that will be used when rendering the pattern during a fill or\\n     * stroke painting operation.\\n     */\\n    setTransform(transform?: DOMMatrix2DInit): void;\\n}\\n\\ndeclare var CanvasPattern: {\\n    prototype: CanvasPattern;\\n    new(): CanvasPattern;\\n};\\n\\ninterface CanvasRect {\\n    clearRect(x: number, y: number, w: number, h: number): void;\\n    fillRect(x: number, y: number, w: number, h: number): void;\\n    strokeRect(x: number, y: number, w: number, h: number): void;\\n}\\n\\ninterface CanvasRenderingContext2D extends CanvasState, CanvasTransform, CanvasCompositing, CanvasImageSmoothing, CanvasFillStrokeStyles, CanvasShadowStyles, CanvasFilters, CanvasRect, CanvasDrawPath, CanvasUserInterface, CanvasText, CanvasDrawImage, CanvasImageData, CanvasPathDrawingStyles, CanvasTextDrawingStyles, CanvasPath {\\n    readonly canvas: HTMLCanvasElement;\\n}\\n\\ndeclare var CanvasRenderingContext2D: {\\n    prototype: CanvasRenderingContext2D;\\n    new(): CanvasRenderingContext2D;\\n};\\n\\ninterface CanvasShadowStyles {\\n    shadowBlur: number;\\n    shadowColor: string;\\n    shadowOffsetX: number;\\n    shadowOffsetY: number;\\n}\\n\\ninterface CanvasState {\\n    restore(): void;\\n    save(): void;\\n}\\n\\ninterface CanvasText {\\n    fillText(text: string, x: number, y: number, maxWidth?: number): void;\\n    measureText(text: string): TextMetrics;\\n    strokeText(text: string, x: number, y: number, maxWidth?: number): void;\\n}\\n\\ninterface CanvasTextDrawingStyles {\\n    direction: CanvasDirection;\\n    font: string;\\n    textAlign: CanvasTextAlign;\\n    textBaseline: CanvasTextBaseline;\\n}\\n\\ninterface CanvasTransform {\\n    getTransform(): DOMMatrix;\\n    resetTransform(): void;\\n    rotate(angle: number): void;\\n    scale(x: number, y: number): void;\\n    setTransform(a: number, b: number, c: number, d: number, e: number, f: number): void;\\n    setTransform(transform?: DOMMatrix2DInit): void;\\n    transform(a: number, b: number, c: number, d: number, e: number, f: number): void;\\n    translate(x: number, y: number): void;\\n}\\n\\ninterface CanvasUserInterface {\\n    drawFocusIfNeeded(element: Element): void;\\n    drawFocusIfNeeded(path: Path2D, element: Element): void;\\n    scrollPathIntoView(): void;\\n    scrollPathIntoView(path: Path2D): void;\\n}\\n\\ninterface CaretPosition {\\n    readonly offset: number;\\n    readonly offsetNode: Node;\\n    getClientRect(): DOMRect | null;\\n}\\n\\ndeclare var CaretPosition: {\\n    prototype: CaretPosition;\\n    new(): CaretPosition;\\n};\\n\\ninterface ChannelMergerNode extends AudioNode {\\n}\\n\\ndeclare var ChannelMergerNode: {\\n    prototype: ChannelMergerNode;\\n    new(context: BaseAudioContext, options?: ChannelMergerOptions): ChannelMergerNode;\\n};\\n\\ninterface ChannelSplitterNode extends AudioNode {\\n}\\n\\ndeclare var ChannelSplitterNode: {\\n    prototype: ChannelSplitterNode;\\n    new(context: BaseAudioContext, options?: ChannelSplitterOptions): ChannelSplitterNode;\\n};\\n\\ninterface CharacterData extends Node, NonDocumentTypeChildNode, ChildNode {\\n    data: string;\\n    readonly length: number;\\n    appendData(data: string): void;\\n    deleteData(offset: number, count: number): void;\\n    insertData(offset: number, data: string): void;\\n    replaceData(offset: number, count: number, data: string): void;\\n    substringData(offset: number, count: number): string;\\n}\\n\\ndeclare var CharacterData: {\\n    prototype: CharacterData;\\n    new(): CharacterData;\\n};\\n\\ninterface ChildNode extends Node {\\n    /**\\n     * Inserts nodes just after node, while replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    after(...nodes: (Node | string)[]): void;\\n    /**\\n     * Inserts nodes just before node, while replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    before(...nodes: (Node | string)[]): void;\\n    /**\\n     * Removes node.\\n     */\\n    remove(): void;\\n    /**\\n     * Replaces node with nodes, while replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    replaceWith(...nodes: (Node | string)[]): void;\\n}\\n\\ninterface ClientRect {\\n    bottom: number;\\n    readonly height: number;\\n    left: number;\\n    right: number;\\n    top: number;\\n    readonly width: number;\\n}\\n\\ndeclare var ClientRect: {\\n    prototype: ClientRect;\\n    new(): ClientRect;\\n};\\n\\ninterface ClientRectList {\\n    readonly length: number;\\n    item(index: number): ClientRect;\\n    [index: number]: ClientRect;\\n}\\n\\ndeclare var ClientRectList: {\\n    prototype: ClientRectList;\\n    new(): ClientRectList;\\n};\\n\\ninterface ClipboardEvent extends Event {\\n    readonly clipboardData: DataTransfer;\\n}\\n\\ndeclare var ClipboardEvent: {\\n    prototype: ClipboardEvent;\\n    new(type: string, eventInitDict?: ClipboardEventInit): ClipboardEvent;\\n};\\n\\ninterface ClipboardEventInit extends EventInit {\\n    data?: string;\\n    dataType?: string;\\n}\\n\\ninterface CloseEvent extends Event {\\n    readonly code: number;\\n    readonly reason: string;\\n    readonly wasClean: boolean;\\n    /** @deprecated */\\n    initCloseEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, wasCleanArg: boolean, codeArg: number, reasonArg: string): void;\\n}\\n\\ndeclare var CloseEvent: {\\n    prototype: CloseEvent;\\n    new(type: string, eventInitDict?: CloseEventInit): CloseEvent;\\n};\\n\\ninterface Comment extends CharacterData {\\n}\\n\\ndeclare var Comment: {\\n    prototype: Comment;\\n    new(data?: string): Comment;\\n};\\n\\ninterface CompositionEvent extends UIEvent {\\n    readonly data: string;\\n    readonly locale: string;\\n    initCompositionEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, dataArg: string, locale: string): void;\\n}\\n\\ndeclare var CompositionEvent: {\\n    prototype: CompositionEvent;\\n    new(typeArg: string, eventInitDict?: CompositionEventInit): CompositionEvent;\\n};\\n\\ninterface ConcatParams extends Algorithm {\\n    algorithmId: Uint8Array;\\n    hash?: string | Algorithm;\\n    partyUInfo: Uint8Array;\\n    partyVInfo: Uint8Array;\\n    privateInfo?: Uint8Array;\\n    publicInfo?: Uint8Array;\\n}\\n\\ninterface Console {\\n    memory: any;\\n    assert(condition?: boolean, message?: string, ...data: any[]): void;\\n    clear(): void;\\n    count(label?: string): void;\\n    debug(message?: any, ...optionalParams: any[]): void;\\n    dir(value?: any, ...optionalParams: any[]): void;\\n    dirxml(value: any): void;\\n    error(message?: any, ...optionalParams: any[]): void;\\n    exception(message?: string, ...optionalParams: any[]): void;\\n    group(groupTitle?: string, ...optionalParams: any[]): void;\\n    groupCollapsed(groupTitle?: string, ...optionalParams: any[]): void;\\n    groupEnd(): void;\\n    info(message?: any, ...optionalParams: any[]): void;\\n    log(message?: any, ...optionalParams: any[]): void;\\n    markTimeline(label?: string): void;\\n    profile(reportName?: string): void;\\n    profileEnd(reportName?: string): void;\\n    table(...tabularData: any[]): void;\\n    time(label?: string): void;\\n    timeEnd(label?: string): void;\\n    timeStamp(label?: string): void;\\n    timeline(label?: string): void;\\n    timelineEnd(label?: string): void;\\n    trace(message?: any, ...optionalParams: any[]): void;\\n    warn(message?: any, ...optionalParams: any[]): void;\\n}\\n\\ndeclare var Console: {\\n    prototype: Console;\\n    new(): Console;\\n};\\n\\ninterface ConstantSourceNode extends AudioScheduledSourceNode {\\n    readonly offset: AudioParam;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: ConstantSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: ConstantSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ConstantSourceNode: {\\n    prototype: ConstantSourceNode;\\n    new(context: BaseAudioContext, options?: ConstantSourceOptions): ConstantSourceNode;\\n};\\n\\ninterface ConvolverNode extends AudioNode {\\n    buffer: AudioBuffer | null;\\n    normalize: boolean;\\n}\\n\\ndeclare var ConvolverNode: {\\n    prototype: ConvolverNode;\\n    new(context: BaseAudioContext, options?: ConvolverOptions): ConvolverNode;\\n};\\n\\ninterface Coordinates {\\n    readonly accuracy: number;\\n    readonly altitude: number | null;\\n    readonly altitudeAccuracy: number | null;\\n    readonly heading: number | null;\\n    readonly latitude: number;\\n    readonly longitude: number;\\n    readonly speed: number | null;\\n}\\n\\ninterface CountQueuingStrategy {\\n    highWaterMark: number;\\n    size(): number;\\n}\\n\\ndeclare var CountQueuingStrategy: {\\n    prototype: CountQueuingStrategy;\\n    new(strategy: QueuingStrategy): CountQueuingStrategy;\\n};\\n\\ninterface Crypto {\\n    readonly subtle: SubtleCrypto;\\n    getRandomValues<T extends Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | null>(array: T): T;\\n}\\n\\ndeclare var Crypto: {\\n    prototype: Crypto;\\n    new(): Crypto;\\n};\\n\\ninterface CryptoKey {\\n    readonly algorithm: KeyAlgorithm;\\n    readonly extractable: boolean;\\n    readonly type: KeyType;\\n    readonly usages: KeyUsage[];\\n}\\n\\ndeclare var CryptoKey: {\\n    prototype: CryptoKey;\\n    new(): CryptoKey;\\n};\\n\\ninterface CryptoKeyPair {\\n    privateKey: CryptoKey;\\n    publicKey: CryptoKey;\\n}\\n\\ndeclare var CryptoKeyPair: {\\n    prototype: CryptoKeyPair;\\n    new(): CryptoKeyPair;\\n};\\n\\ninterface CustomElementRegistry {\\n    define(name: string, constructor: Function, options?: ElementDefinitionOptions): void;\\n    get(name: string): any;\\n    upgrade(root: Node): void;\\n    whenDefined(name: string): Promise<void>;\\n}\\n\\ndeclare var CustomElementRegistry: {\\n    prototype: CustomElementRegistry;\\n    new(): CustomElementRegistry;\\n};\\n\\ninterface CustomEvent<T = any> extends Event {\\n    /**\\n     * Returns any custom data event was created with.\\n     * Typically used for synthetic events.\\n     */\\n    readonly detail: T;\\n    initCustomEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, detailArg: T): void;\\n}\\n\\ndeclare var CustomEvent: {\\n    prototype: CustomEvent;\\n    new<T>(typeArg: string, eventInitDict?: CustomEventInit<T>): CustomEvent<T>;\\n};\\n\\ninterface DOMError {\\n    readonly name: string;\\n    toString(): string;\\n}\\n\\ndeclare var DOMError: {\\n    prototype: DOMError;\\n    new(): DOMError;\\n};\\n\\ninterface DOMException {\\n    readonly code: number;\\n    readonly message: string;\\n    readonly name: string;\\n    readonly ABORT_ERR: number;\\n    readonly DATA_CLONE_ERR: number;\\n    readonly DOMSTRING_SIZE_ERR: number;\\n    readonly HIERARCHY_REQUEST_ERR: number;\\n    readonly INDEX_SIZE_ERR: number;\\n    readonly INUSE_ATTRIBUTE_ERR: number;\\n    readonly INVALID_ACCESS_ERR: number;\\n    readonly INVALID_CHARACTER_ERR: number;\\n    readonly INVALID_MODIFICATION_ERR: number;\\n    readonly INVALID_NODE_TYPE_ERR: number;\\n    readonly INVALID_STATE_ERR: number;\\n    readonly NAMESPACE_ERR: number;\\n    readonly NETWORK_ERR: number;\\n    readonly NOT_FOUND_ERR: number;\\n    readonly NOT_SUPPORTED_ERR: number;\\n    readonly NO_DATA_ALLOWED_ERR: number;\\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\\n    readonly QUOTA_EXCEEDED_ERR: number;\\n    readonly SECURITY_ERR: number;\\n    readonly SYNTAX_ERR: number;\\n    readonly TIMEOUT_ERR: number;\\n    readonly TYPE_MISMATCH_ERR: number;\\n    readonly URL_MISMATCH_ERR: number;\\n    readonly VALIDATION_ERR: number;\\n    readonly WRONG_DOCUMENT_ERR: number;\\n}\\n\\ndeclare var DOMException: {\\n    prototype: DOMException;\\n    new(message?: string, name?: string): DOMException;\\n    readonly ABORT_ERR: number;\\n    readonly DATA_CLONE_ERR: number;\\n    readonly DOMSTRING_SIZE_ERR: number;\\n    readonly HIERARCHY_REQUEST_ERR: number;\\n    readonly INDEX_SIZE_ERR: number;\\n    readonly INUSE_ATTRIBUTE_ERR: number;\\n    readonly INVALID_ACCESS_ERR: number;\\n    readonly INVALID_CHARACTER_ERR: number;\\n    readonly INVALID_MODIFICATION_ERR: number;\\n    readonly INVALID_NODE_TYPE_ERR: number;\\n    readonly INVALID_STATE_ERR: number;\\n    readonly NAMESPACE_ERR: number;\\n    readonly NETWORK_ERR: number;\\n    readonly NOT_FOUND_ERR: number;\\n    readonly NOT_SUPPORTED_ERR: number;\\n    readonly NO_DATA_ALLOWED_ERR: number;\\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\\n    readonly QUOTA_EXCEEDED_ERR: number;\\n    readonly SECURITY_ERR: number;\\n    readonly SYNTAX_ERR: number;\\n    readonly TIMEOUT_ERR: number;\\n    readonly TYPE_MISMATCH_ERR: number;\\n    readonly URL_MISMATCH_ERR: number;\\n    readonly VALIDATION_ERR: number;\\n    readonly WRONG_DOCUMENT_ERR: number;\\n};\\n\\ninterface DOMImplementation {\\n    createDocument(namespaceURI: string | null, qualifiedName: string | null, doctype: DocumentType | null): Document;\\n    createDocumentType(qualifiedName: string, publicId: string, systemId: string): DocumentType;\\n    createHTMLDocument(title?: string): Document;\\n    /** @deprecated */\\n    hasFeature(...args: any[]): true;\\n}\\n\\ndeclare var DOMImplementation: {\\n    prototype: DOMImplementation;\\n    new(): DOMImplementation;\\n};\\n\\ninterface DOML2DeprecatedColorProperty {\\n    color: string;\\n}\\n\\ninterface DOMMatrix extends DOMMatrixReadOnly {\\n    a: number;\\n    b: number;\\n    c: number;\\n    d: number;\\n    e: number;\\n    f: number;\\n    m11: number;\\n    m12: number;\\n    m13: number;\\n    m14: number;\\n    m21: number;\\n    m22: number;\\n    m23: number;\\n    m24: number;\\n    m31: number;\\n    m32: number;\\n    m33: number;\\n    m34: number;\\n    m41: number;\\n    m42: number;\\n    m43: number;\\n    m44: number;\\n    invertSelf(): DOMMatrix;\\n    multiplySelf(other?: DOMMatrixInit): DOMMatrix;\\n    preMultiplySelf(other?: DOMMatrixInit): DOMMatrix;\\n    rotateAxisAngleSelf(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\\n    rotateFromVectorSelf(x?: number, y?: number): DOMMatrix;\\n    rotateSelf(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\\n    scale3dSelf(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    scaleSelf(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    setMatrixValue(transformList: string): DOMMatrix;\\n    skewXSelf(sx?: number): DOMMatrix;\\n    skewYSelf(sy?: number): DOMMatrix;\\n    translateSelf(tx?: number, ty?: number, tz?: number): DOMMatrix;\\n}\\n\\ndeclare var DOMMatrix: {\\n    prototype: DOMMatrix;\\n    new(init?: string | number[]): DOMMatrix;\\n    fromFloat32Array(array32: Float32Array): DOMMatrix;\\n    fromFloat64Array(array64: Float64Array): DOMMatrix;\\n    fromMatrix(other?: DOMMatrixInit): DOMMatrix;\\n};\\n\\ntype SVGMatrix = DOMMatrix;\\ndeclare var SVGMatrix: typeof DOMMatrix;\\n\\ntype WebKitCSSMatrix = DOMMatrix;\\ndeclare var WebKitCSSMatrix: typeof DOMMatrix;\\n\\ninterface DOMMatrixReadOnly {\\n    readonly a: number;\\n    readonly b: number;\\n    readonly c: number;\\n    readonly d: number;\\n    readonly e: number;\\n    readonly f: number;\\n    readonly is2D: boolean;\\n    readonly isIdentity: boolean;\\n    readonly m11: number;\\n    readonly m12: number;\\n    readonly m13: number;\\n    readonly m14: number;\\n    readonly m21: number;\\n    readonly m22: number;\\n    readonly m23: number;\\n    readonly m24: number;\\n    readonly m31: number;\\n    readonly m32: number;\\n    readonly m33: number;\\n    readonly m34: number;\\n    readonly m41: number;\\n    readonly m42: number;\\n    readonly m43: number;\\n    readonly m44: number;\\n    flipX(): DOMMatrix;\\n    flipY(): DOMMatrix;\\n    inverse(): DOMMatrix;\\n    multiply(other?: DOMMatrixInit): DOMMatrix;\\n    rotate(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\\n    rotateAxisAngle(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\\n    rotateFromVector(x?: number, y?: number): DOMMatrix;\\n    scale(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    scale3d(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\\n    skewX(sx?: number): DOMMatrix;\\n    skewY(sy?: number): DOMMatrix;\\n    toFloat32Array(): Float32Array;\\n    toFloat64Array(): Float64Array;\\n    toJSON(): any;\\n    transformPoint(point?: DOMPointInit): DOMPoint;\\n    translate(tx?: number, ty?: number, tz?: number): DOMMatrix;\\n}\\n\\ndeclare var DOMMatrixReadOnly: {\\n    prototype: DOMMatrixReadOnly;\\n    new(init?: string | number[]): DOMMatrixReadOnly;\\n    fromFloat32Array(array32: Float32Array): DOMMatrixReadOnly;\\n    fromFloat64Array(array64: Float64Array): DOMMatrixReadOnly;\\n    fromMatrix(other?: DOMMatrixInit): DOMMatrixReadOnly;\\n};\\n\\ninterface DOMParser {\\n    parseFromString(str: string, type: SupportedType): Document;\\n}\\n\\ndeclare var DOMParser: {\\n    prototype: DOMParser;\\n    new(): DOMParser;\\n};\\n\\ninterface DOMPoint extends DOMPointReadOnly {\\n    w: number;\\n    x: number;\\n    y: number;\\n    z: number;\\n}\\n\\ndeclare var DOMPoint: {\\n    prototype: DOMPoint;\\n    new(x?: number, y?: number, z?: number, w?: number): DOMPoint;\\n    fromPoint(other?: DOMPointInit): DOMPoint;\\n};\\n\\ntype SVGPoint = DOMPoint;\\ndeclare var SVGPoint: typeof DOMPoint;\\n\\ninterface DOMPointReadOnly {\\n    readonly w: number;\\n    readonly x: number;\\n    readonly y: number;\\n    readonly z: number;\\n    matrixTransform(matrix?: DOMMatrixInit): DOMPoint;\\n    toJSON(): any;\\n}\\n\\ndeclare var DOMPointReadOnly: {\\n    prototype: DOMPointReadOnly;\\n    new(x?: number, y?: number, z?: number, w?: number): DOMPointReadOnly;\\n    fromPoint(other?: DOMPointInit): DOMPointReadOnly;\\n};\\n\\ninterface DOMQuad {\\n    readonly p1: DOMPoint;\\n    readonly p2: DOMPoint;\\n    readonly p3: DOMPoint;\\n    readonly p4: DOMPoint;\\n    getBounds(): DOMRect;\\n    toJSON(): any;\\n}\\n\\ndeclare var DOMQuad: {\\n    prototype: DOMQuad;\\n    new(p1?: DOMPointInit, p2?: DOMPointInit, p3?: DOMPointInit, p4?: DOMPointInit): DOMQuad;\\n    fromQuad(other?: DOMQuadInit): DOMQuad;\\n    fromRect(other?: DOMRectInit): DOMQuad;\\n};\\n\\ninterface DOMRect extends DOMRectReadOnly {\\n    height: number;\\n    width: number;\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var DOMRect: {\\n    prototype: DOMRect;\\n    new(x?: number, y?: number, width?: number, height?: number): DOMRect;\\n    fromRect(other?: DOMRectInit): DOMRect;\\n};\\n\\ntype SVGRect = DOMRect;\\ndeclare var SVGRect: typeof DOMRect;\\n\\ninterface DOMRectList {\\n    readonly length: number;\\n    item(index: number): DOMRect | null;\\n    [index: number]: DOMRect;\\n}\\n\\ndeclare var DOMRectList: {\\n    prototype: DOMRectList;\\n    new(): DOMRectList;\\n};\\n\\ninterface DOMRectReadOnly {\\n    readonly bottom: number;\\n    readonly height: number;\\n    readonly left: number;\\n    readonly right: number;\\n    readonly top: number;\\n    readonly width: number;\\n    readonly x: number;\\n    readonly y: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var DOMRectReadOnly: {\\n    prototype: DOMRectReadOnly;\\n    new(x?: number, y?: number, width?: number, height?: number): DOMRectReadOnly;\\n    fromRect(other?: DOMRectInit): DOMRectReadOnly;\\n};\\n\\ninterface DOMSettableTokenList extends DOMTokenList {\\n    value: string;\\n}\\n\\ndeclare var DOMSettableTokenList: {\\n    prototype: DOMSettableTokenList;\\n    new(): DOMSettableTokenList;\\n};\\n\\ninterface DOMStringList {\\n    /**\\n     * Returns the number of strings in strings.\\n     */\\n    readonly length: number;\\n    /**\\n     * Returns true if strings contains string, and false\\n     * otherwise.\\n     */\\n    contains(string: string): boolean;\\n    /**\\n     * Returns the string with index index from strings.\\n     */\\n    item(index: number): string | null;\\n    [index: number]: string;\\n}\\n\\ndeclare var DOMStringList: {\\n    prototype: DOMStringList;\\n    new(): DOMStringList;\\n};\\n\\ninterface DOMStringMap {\\n    [name: string]: string | undefined;\\n}\\n\\ndeclare var DOMStringMap: {\\n    prototype: DOMStringMap;\\n    new(): DOMStringMap;\\n};\\n\\ninterface DOMTokenList {\\n    /**\\n     * Returns the number of tokens.\\n     */\\n    readonly length: number;\\n    /**\\n     * Returns the associated set as string.\\n     * Can be set, to change the associated attribute.\\n     */\\n    value: string;\\n    /**\\n     * Adds all arguments passed, except those already present.\\n     * Throws a \"SyntaxError\" DOMException if one of the arguments is the empty\\n     * string.\\n     * Throws an \"InvalidCharacterError\" DOMException if one of the arguments\\n     * contains any ASCII whitespace.\\n     */\\n    add(...tokens: string[]): void;\\n    /**\\n     * Returns true if token is present, and false otherwise.\\n     */\\n    contains(token: string): boolean;\\n    /**\\n     * tokenlist[index]\\n     */\\n    item(index: number): string | null;\\n    /**\\n     * Removes arguments passed, if they are present.\\n     * Throws a \"SyntaxError\" DOMException if one of the arguments is the empty\\n     * string.\\n     * Throws an \"InvalidCharacterError\" DOMException if one of the arguments\\n     * contains any ASCII whitespace.\\n     */\\n    remove(...tokens: string[]): void;\\n    /**\\n     * Replaces token with newToken.\\n     * Returns true if token was replaced with newToken, and false otherwise.\\n     * Throws a \"SyntaxError\" DOMException if one of the arguments is the empty\\n     * string.\\n     * Throws an \"InvalidCharacterError\" DOMException if one of the arguments\\n     * contains any ASCII whitespace.\\n     */\\n    replace(oldToken: string, newToken: string): void;\\n    /**\\n     * Returns true if token is in the associated attribute\\'s supported tokens. Returns\\n     * false otherwise.\\n     * Throws a TypeError if the associated attribute has no supported tokens defined.\\n     */\\n    supports(token: string): boolean;\\n    toggle(token: string, force?: boolean): boolean;\\n    forEach(callbackfn: (value: string, key: number, parent: DOMTokenList) => void, thisArg?: any): void;\\n    [index: number]: string;\\n}\\n\\ndeclare var DOMTokenList: {\\n    prototype: DOMTokenList;\\n    new(): DOMTokenList;\\n};\\n\\ninterface DataCue extends TextTrackCue {\\n    data: ArrayBuffer;\\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: DataCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: DataCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var DataCue: {\\n    prototype: DataCue;\\n    new(): DataCue;\\n};\\n\\ninterface DataTransfer {\\n    dropEffect: string;\\n    effectAllowed: string;\\n    /**\\n     * Returns a FileList of the files being dragged, if any.\\n     */\\n    readonly files: FileList;\\n    /**\\n     * Returns a DataTransferItemList object, with the drag data.\\n     */\\n    readonly items: DataTransferItemList;\\n    /**\\n     * Returns a frozen array listing the formats that were set in the dragstart event. In addition, if any files are being\\n     * dragged, then one of the types will be the string \"Files\".\\n     */\\n    readonly types: ReadonlyArray<string>;\\n    /**\\n     * Removes the data of the specified formats. Removes all data if the argument is omitted.\\n     */\\n    clearData(format?: string): void;\\n    /**\\n     * Returns the specified data. If there is no such data, returns the empty string.\\n     */\\n    getData(format: string): string;\\n    /**\\n     * Adds the specified data.\\n     */\\n    setData(format: string, data: string): void;\\n    /**\\n     * Uses the given element to update the drag feedback, replacing any previously specified\\n     * feedback.\\n     */\\n    setDragImage(image: Element, x: number, y: number): void;\\n}\\n\\ndeclare var DataTransfer: {\\n    prototype: DataTransfer;\\n    new(): DataTransfer;\\n};\\n\\ninterface DataTransferItem {\\n    /**\\n     * Returns the drag data item kind, one of: \"string\",\\n     * \"file\".\\n     */\\n    readonly kind: string;\\n    /**\\n     * Returns the drag data item type string.\\n     */\\n    readonly type: string;\\n    /**\\n     * Returns a File object, if the drag data item kind is File.\\n     */\\n    getAsFile(): File | null;\\n    /**\\n     * Invokes the callback with the string data as the argument, if the drag data item\\n     * kind is Plain Unicode string.\\n     */\\n    getAsString(callback: FunctionStringCallback | null): void;\\n    webkitGetAsEntry(): any;\\n}\\n\\ndeclare var DataTransferItem: {\\n    prototype: DataTransferItem;\\n    new(): DataTransferItem;\\n};\\n\\ninterface DataTransferItemList {\\n    /**\\n     * Returns the number of items in the drag data store.\\n     */\\n    readonly length: number;\\n    /**\\n     * Adds a new entry for the given data to the drag data store. If the data is plain\\n     * text  then a type string has to be provided\\n     * also.\\n     */\\n    add(data: string, type: string): DataTransferItem | null;\\n    add(data: File): DataTransferItem | null;\\n    /**\\n     * Removes all the entries in the drag data store.\\n     */\\n    clear(): void;\\n    item(index: number): DataTransferItem;\\n    /**\\n     * Removes the indexth entry in the drag data store.\\n     */\\n    remove(index: number): void;\\n    [name: number]: DataTransferItem;\\n}\\n\\ndeclare var DataTransferItemList: {\\n    prototype: DataTransferItemList;\\n    new(): DataTransferItemList;\\n};\\n\\ninterface DeferredPermissionRequest {\\n    readonly id: number;\\n    readonly type: MSWebViewPermissionType;\\n    readonly uri: string;\\n    allow(): void;\\n    deny(): void;\\n}\\n\\ndeclare var DeferredPermissionRequest: {\\n    prototype: DeferredPermissionRequest;\\n    new(): DeferredPermissionRequest;\\n};\\n\\ninterface DelayNode extends AudioNode {\\n    readonly delayTime: AudioParam;\\n}\\n\\ndeclare var DelayNode: {\\n    prototype: DelayNode;\\n    new(context: BaseAudioContext, options?: DelayOptions): DelayNode;\\n};\\n\\ninterface DeviceAcceleration {\\n    readonly x: number | null;\\n    readonly y: number | null;\\n    readonly z: number | null;\\n}\\n\\ndeclare var DeviceAcceleration: {\\n    prototype: DeviceAcceleration;\\n    new(): DeviceAcceleration;\\n};\\n\\ninterface DeviceLightEvent extends Event {\\n    readonly value: number;\\n}\\n\\ndeclare var DeviceLightEvent: {\\n    prototype: DeviceLightEvent;\\n    new(typeArg: string, eventInitDict?: DeviceLightEventInit): DeviceLightEvent;\\n};\\n\\ninterface DeviceMotionEvent extends Event {\\n    readonly acceleration: DeviceAcceleration | null;\\n    readonly accelerationIncludingGravity: DeviceAcceleration | null;\\n    readonly interval: number | null;\\n    readonly rotationRate: DeviceRotationRate | null;\\n    initDeviceMotionEvent(type: string, bubbles: boolean, cancelable: boolean, acceleration: DeviceAccelerationDict | null, accelerationIncludingGravity: DeviceAccelerationDict | null, rotationRate: DeviceRotationRateDict | null, interval: number | null): void;\\n}\\n\\ndeclare var DeviceMotionEvent: {\\n    prototype: DeviceMotionEvent;\\n    new(typeArg: string, eventInitDict?: DeviceMotionEventInit): DeviceMotionEvent;\\n};\\n\\ninterface DeviceOrientationEvent extends Event {\\n    readonly absolute: boolean;\\n    readonly alpha: number | null;\\n    readonly beta: number | null;\\n    readonly gamma: number | null;\\n    initDeviceOrientationEvent(type: string, bubbles: boolean, cancelable: boolean, alpha: number | null, beta: number | null, gamma: number | null, absolute: boolean): void;\\n}\\n\\ndeclare var DeviceOrientationEvent: {\\n    prototype: DeviceOrientationEvent;\\n    new(typeArg: string, eventInitDict?: DeviceOrientationEventInit): DeviceOrientationEvent;\\n};\\n\\ninterface DeviceRotationRate {\\n    readonly alpha: number | null;\\n    readonly beta: number | null;\\n    readonly gamma: number | null;\\n}\\n\\ndeclare var DeviceRotationRate: {\\n    prototype: DeviceRotationRate;\\n    new(): DeviceRotationRate;\\n};\\n\\ninterface DhImportKeyParams extends Algorithm {\\n    generator: Uint8Array;\\n    prime: Uint8Array;\\n}\\n\\ninterface DhKeyAlgorithm extends KeyAlgorithm {\\n    generator: Uint8Array;\\n    prime: Uint8Array;\\n}\\n\\ninterface DhKeyDeriveParams extends Algorithm {\\n    public: CryptoKey;\\n}\\n\\ninterface DhKeyGenParams extends Algorithm {\\n    generator: Uint8Array;\\n    prime: Uint8Array;\\n}\\n\\ninterface DocumentEventMap extends GlobalEventHandlersEventMap, DocumentAndElementEventHandlersEventMap {\\n    \"fullscreenchange\": Event;\\n    \"fullscreenerror\": Event;\\n    \"readystatechange\": ProgressEvent;\\n    \"visibilitychange\": Event;\\n}\\n\\ninterface Document extends Node, NonElementParentNode, DocumentOrShadowRoot, ParentNode, GlobalEventHandlers, DocumentAndElementEventHandlers {\\n    /**\\n     * Sets or gets the URL for the current document.\\n     */\\n    readonly URL: string;\\n    /**\\n     * Gets the object that has the focus when the parent document has focus.\\n     */\\n    readonly activeElement: Element | null;\\n    /**\\n     * Sets or gets the color of all active links in the document.\\n     */\\n    /** @deprecated */\\n    alinkColor: string;\\n    /**\\n     * Returns a reference to the collection of elements contained by the object.\\n     */\\n    /** @deprecated */\\n    readonly all: HTMLAllCollection;\\n    /**\\n     * Retrieves a collection of all a objects that have a name and/or id property. Objects in this collection are in HTML source order.\\n     */\\n    /** @deprecated */\\n    readonly anchors: HTMLCollectionOf<HTMLAnchorElement>;\\n    /**\\n     * Retrieves a collection of all applet objects in the document.\\n     */\\n    /** @deprecated */\\n    readonly applets: HTMLCollectionOf<HTMLAppletElement>;\\n    /**\\n     * Deprecated. Sets or retrieves a value that indicates the background color behind the object.\\n     */\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Specifies the beginning and end of the document body.\\n     */\\n    body: HTMLElement;\\n    /**\\n     * Returns document\\'s encoding.\\n     */\\n    readonly characterSet: string;\\n    /**\\n     * Gets or sets the character set used to encode the object.\\n     */\\n    readonly charset: string;\\n    /**\\n     * Gets a value that indicates whether standards-compliant mode is switched on for the object.\\n     */\\n    readonly compatMode: string;\\n    /**\\n     * Returns document\\'s content type.\\n     */\\n    readonly contentType: string;\\n    /**\\n     * Returns the HTTP cookies that apply to the Document. If there are no cookies or\\n     * cookies can\\'t be applied to this resource, the empty string will be returned.\\n     * Can be set, to add a new cookie to the element\\'s set of HTTP cookies.\\n     * If the contents are sandboxed into a\\n     * unique origin (e.g. in an iframe with the sandbox attribute), a\\n     * \"SecurityError\" DOMException will be thrown on getting\\n     * and setting.\\n     */\\n    cookie: string;\\n    /**\\n     * Returns the script element, or the SVG script element,\\n     * that is currently executing, as long as the element represents a classic script.\\n     * In the case of reentrant script execution, returns the one that most recently started executing\\n     * amongst those that have not yet finished executing.\\n     * Returns null if the Document is not currently executing a script\\n     * or SVG script element (e.g., because the running script is an event\\n     * handler, or a timeout), or if the currently executing script or SVG\\n     * script element represents a module script.\\n     */\\n    readonly currentScript: HTMLOrSVGScriptElement | null;\\n    readonly defaultView: WindowProxy | null;\\n    /**\\n     * Sets or gets a value that indicates whether the document can be edited.\\n     */\\n    designMode: string;\\n    /**\\n     * Sets or retrieves a value that indicates the reading order of the object.\\n     */\\n    dir: string;\\n    /**\\n     * Gets an object representing the document type declaration associated with the current document.\\n     */\\n    readonly doctype: DocumentType | null;\\n    /**\\n     * Gets a reference to the root node of the document.\\n     */\\n    readonly documentElement: HTMLElement | null;\\n    /**\\n     * Returns document\\'s URL.\\n     */\\n    readonly documentURI: string;\\n    /**\\n     * Sets or gets the security domain of the document.\\n     */\\n    domain: string;\\n    /**\\n     * Retrieves a collection of all embed objects in the document.\\n     */\\n    readonly embeds: HTMLCollectionOf<HTMLEmbedElement>;\\n    /**\\n     * Sets or gets the foreground (text) color of the document.\\n     */\\n    /** @deprecated */\\n    fgColor: string;\\n    /**\\n     * Retrieves a collection, in source order, of all form objects in the document.\\n     */\\n    readonly forms: HTMLCollectionOf<HTMLFormElement>;\\n    /** @deprecated */\\n    readonly fullscreen: boolean;\\n    /**\\n     * Returns true if document has the ability to display elements fullscreen\\n     * and fullscreen is supported, or false otherwise.\\n     */\\n    readonly fullscreenEnabled: boolean;\\n    /**\\n     * Returns the head element.\\n     */\\n    readonly head: HTMLHeadElement | null;\\n    readonly hidden: boolean;\\n    /**\\n     * Retrieves a collection, in source order, of img objects in the document.\\n     */\\n    readonly images: HTMLCollectionOf<HTMLImageElement>;\\n    /**\\n     * Gets the implementation object of the current document.\\n     */\\n    readonly implementation: DOMImplementation;\\n    /**\\n     * Returns the character encoding used to create the webpage that is loaded into the document object.\\n     */\\n    readonly inputEncoding: string;\\n    /**\\n     * Gets the date that the page was last modified, if the page supplies one.\\n     */\\n    readonly lastModified: string;\\n    /**\\n     * Sets or gets the color of the document links.\\n     */\\n    /** @deprecated */\\n    linkColor: string;\\n    /**\\n     * Retrieves a collection of all a objects that specify the href property and all area objects in the document.\\n     */\\n    readonly links: HTMLCollectionOf<HTMLAnchorElement | HTMLAreaElement>;\\n    /**\\n     * Contains information about the current URL.\\n     */\\n    location: Location | null;\\n    onfullscreenchange: ((this: Document, ev: Event) => any) | null;\\n    onfullscreenerror: ((this: Document, ev: Event) => any) | null;\\n    /**\\n     * Fires when the state of the object has changed.\\n     * @param ev The event\\n     */\\n    onreadystatechange: ((this: Document, ev: ProgressEvent) => any) | null;\\n    onvisibilitychange: ((this: Document, ev: Event) => any) | null;\\n    /**\\n     * Returns document\\'s origin.\\n     */\\n    readonly origin: string;\\n    /**\\n     * Return an HTMLCollection of the embed elements in the Document.\\n     */\\n    readonly plugins: HTMLCollectionOf<HTMLEmbedElement>;\\n    /**\\n     * Retrieves a value that indicates the current state of the object.\\n     */\\n    readonly readyState: DocumentReadyState;\\n    /**\\n     * Gets the URL of the location that referred the user to the current page.\\n     */\\n    readonly referrer: string;\\n    /**\\n     * Retrieves a collection of all script objects in the document.\\n     */\\n    readonly scripts: HTMLCollectionOf<HTMLScriptElement>;\\n    readonly scrollingElement: Element | null;\\n    readonly timeline: DocumentTimeline;\\n    /**\\n     * Contains the title of the document.\\n     */\\n    title: string;\\n    readonly visibilityState: VisibilityState;\\n    /**\\n     * Sets or gets the color of the links that the user has visited.\\n     */\\n    /** @deprecated */\\n    vlinkColor: string;\\n    /**\\n     * Moves node from another document and returns it.\\n     * If node is a document, throws a \"NotSupportedError\" DOMException or, if node is a shadow root, throws a\\n     * \"HierarchyRequestError\" DOMException.\\n     */\\n    adoptNode<T extends Node>(source: T): T;\\n    /** @deprecated */\\n    captureEvents(): void;\\n    caretPositionFromPoint(x: number, y: number): CaretPosition | null;\\n    caretRangeFromPoint(x: number, y: number): Range;\\n    /** @deprecated */\\n    clear(): void;\\n    /**\\n     * Closes an output stream and forces the sent data to display.\\n     */\\n    close(): void;\\n    /**\\n     * Creates an attribute object with a specified name.\\n     * @param name String that sets the attribute object\\'s name.\\n     */\\n    createAttribute(localName: string): Attr;\\n    createAttributeNS(namespace: string | null, qualifiedName: string): Attr;\\n    /**\\n     * Returns a CDATASection node whose data is data.\\n     */\\n    createCDATASection(data: string): CDATASection;\\n    /**\\n     * Creates a comment object with the specified data.\\n     * @param data Sets the comment object\\'s data.\\n     */\\n    createComment(data: string): Comment;\\n    /**\\n     * Creates a new document.\\n     */\\n    createDocumentFragment(): DocumentFragment;\\n    /**\\n     * Creates an instance of the element for the specified tag.\\n     * @param tagName The name of an element.\\n     */\\n    createElement<K extends keyof HTMLElementTagNameMap>(tagName: K, options?: ElementCreationOptions): HTMLElementTagNameMap[K];\\n    /** @deprecated */\\n    createElement<K extends keyof HTMLElementDeprecatedTagNameMap>(tagName: K, options?: ElementCreationOptions): HTMLElementDeprecatedTagNameMap[K];\\n    createElement(tagName: string, options?: ElementCreationOptions): HTMLElement;\\n    /**\\n     * Returns an element with namespace namespace. Its namespace prefix will be everything before \":\" (U+003E) in qualifiedName or null. Its local name will be everything after\\n     * \":\" (U+003E) in qualifiedName or qualifiedName.\\n     * If localName does not match the Name production an\\n     * \"InvalidCharacterError\" DOMException will be thrown.\\n     * If one of the following conditions is true a \"NamespaceError\" DOMException will be thrown:\\n     * localName does not match the QName production.\\n     * Namespace prefix is not null and namespace is the empty string.\\n     * Namespace prefix is \"xml\" and namespace is not the XML namespace.\\n     * qualifiedName or namespace prefix is \"xmlns\" and namespace is not the XMLNS namespace.\\n     * namespace is the XMLNS namespace and\\n     * neither qualifiedName nor namespace prefix is \"xmlns\".\\n     * When supplied, options\\'s is can be used to create a customized built-in element.\\n     */\\n    createElementNS(namespaceURI: \"http://www.w3.org/1999/xhtml\", qualifiedName: string): HTMLElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"a\"): SVGAElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"circle\"): SVGCircleElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"clipPath\"): SVGClipPathElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"componentTransferFunction\"): SVGComponentTransferFunctionElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"defs\"): SVGDefsElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"desc\"): SVGDescElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"ellipse\"): SVGEllipseElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feBlend\"): SVGFEBlendElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feColorMatrix\"): SVGFEColorMatrixElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feComponentTransfer\"): SVGFEComponentTransferElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feComposite\"): SVGFECompositeElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feConvolveMatrix\"): SVGFEConvolveMatrixElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feDiffuseLighting\"): SVGFEDiffuseLightingElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feDisplacementMap\"): SVGFEDisplacementMapElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feDistantLight\"): SVGFEDistantLightElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFlood\"): SVGFEFloodElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncA\"): SVGFEFuncAElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncB\"): SVGFEFuncBElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncG\"): SVGFEFuncGElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feFuncR\"): SVGFEFuncRElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feGaussianBlur\"): SVGFEGaussianBlurElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feImage\"): SVGFEImageElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feMerge\"): SVGFEMergeElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feMergeNode\"): SVGFEMergeNodeElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feMorphology\"): SVGFEMorphologyElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feOffset\"): SVGFEOffsetElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"fePointLight\"): SVGFEPointLightElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feSpecularLighting\"): SVGFESpecularLightingElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feSpotLight\"): SVGFESpotLightElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feTile\"): SVGFETileElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"feTurbulence\"): SVGFETurbulenceElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"filter\"): SVGFilterElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"foreignObject\"): SVGForeignObjectElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"g\"): SVGGElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"image\"): SVGImageElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"gradient\"): SVGGradientElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"line\"): SVGLineElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"linearGradient\"): SVGLinearGradientElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"marker\"): SVGMarkerElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"mask\"): SVGMaskElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"path\"): SVGPathElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"metadata\"): SVGMetadataElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"pattern\"): SVGPatternElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"polygon\"): SVGPolygonElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"polyline\"): SVGPolylineElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"radialGradient\"): SVGRadialGradientElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"rect\"): SVGRectElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"svg\"): SVGSVGElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"script\"): SVGScriptElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"stop\"): SVGStopElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"style\"): SVGStyleElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"switch\"): SVGSwitchElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"symbol\"): SVGSymbolElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"tspan\"): SVGTSpanElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"textContent\"): SVGTextContentElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"text\"): SVGTextElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"textPath\"): SVGTextPathElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"textPositioning\"): SVGTextPositioningElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"title\"): SVGTitleElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"use\"): SVGUseElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: \"view\"): SVGViewElement;\\n    createElementNS(namespaceURI: \"http://www.w3.org/2000/svg\", qualifiedName: string): SVGElement;\\n    createElementNS(namespaceURI: string | null, qualifiedName: string, options?: ElementCreationOptions): Element;\\n    createElementNS(namespace: string | null, qualifiedName: string, options?: string | ElementCreationOptions): Element;\\n    createEvent(eventInterface: \"AnimationEvent\"): AnimationEvent;\\n    createEvent(eventInterface: \"AnimationPlaybackEvent\"): AnimationPlaybackEvent;\\n    createEvent(eventInterface: \"AudioProcessingEvent\"): AudioProcessingEvent;\\n    createEvent(eventInterface: \"BeforeUnloadEvent\"): BeforeUnloadEvent;\\n    createEvent(eventInterface: \"ClipboardEvent\"): ClipboardEvent;\\n    createEvent(eventInterface: \"CloseEvent\"): CloseEvent;\\n    createEvent(eventInterface: \"CompositionEvent\"): CompositionEvent;\\n    createEvent(eventInterface: \"CustomEvent\"): CustomEvent;\\n    createEvent(eventInterface: \"DeviceLightEvent\"): DeviceLightEvent;\\n    createEvent(eventInterface: \"DeviceMotionEvent\"): DeviceMotionEvent;\\n    createEvent(eventInterface: \"DeviceOrientationEvent\"): DeviceOrientationEvent;\\n    createEvent(eventInterface: \"DragEvent\"): DragEvent;\\n    createEvent(eventInterface: \"ErrorEvent\"): ErrorEvent;\\n    createEvent(eventInterface: \"Event\"): Event;\\n    createEvent(eventInterface: \"Events\"): Event;\\n    createEvent(eventInterface: \"FocusEvent\"): FocusEvent;\\n    createEvent(eventInterface: \"FocusNavigationEvent\"): FocusNavigationEvent;\\n    createEvent(eventInterface: \"GamepadEvent\"): GamepadEvent;\\n    createEvent(eventInterface: \"HashChangeEvent\"): HashChangeEvent;\\n    createEvent(eventInterface: \"IDBVersionChangeEvent\"): IDBVersionChangeEvent;\\n    createEvent(eventInterface: \"KeyboardEvent\"): KeyboardEvent;\\n    createEvent(eventInterface: \"ListeningStateChangedEvent\"): ListeningStateChangedEvent;\\n    createEvent(eventInterface: \"MSGestureEvent\"): MSGestureEvent;\\n    createEvent(eventInterface: \"MSMediaKeyMessageEvent\"): MSMediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MSMediaKeyNeededEvent\"): MSMediaKeyNeededEvent;\\n    createEvent(eventInterface: \"MSPointerEvent\"): MSPointerEvent;\\n    createEvent(eventInterface: \"MediaEncryptedEvent\"): MediaEncryptedEvent;\\n    createEvent(eventInterface: \"MediaKeyMessageEvent\"): MediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MediaQueryListEvent\"): MediaQueryListEvent;\\n    createEvent(eventInterface: \"MediaStreamErrorEvent\"): MediaStreamErrorEvent;\\n    createEvent(eventInterface: \"MediaStreamEvent\"): MediaStreamEvent;\\n    createEvent(eventInterface: \"MediaStreamTrackEvent\"): MediaStreamTrackEvent;\\n    createEvent(eventInterface: \"MessageEvent\"): MessageEvent;\\n    createEvent(eventInterface: \"MouseEvent\"): MouseEvent;\\n    createEvent(eventInterface: \"MouseEvents\"): MouseEvent;\\n    createEvent(eventInterface: \"MutationEvent\"): MutationEvent;\\n    createEvent(eventInterface: \"MutationEvents\"): MutationEvent;\\n    createEvent(eventInterface: \"OfflineAudioCompletionEvent\"): OfflineAudioCompletionEvent;\\n    createEvent(eventInterface: \"OverflowEvent\"): OverflowEvent;\\n    createEvent(eventInterface: \"PageTransitionEvent\"): PageTransitionEvent;\\n    createEvent(eventInterface: \"PaymentRequestUpdateEvent\"): PaymentRequestUpdateEvent;\\n    createEvent(eventInterface: \"PermissionRequestedEvent\"): PermissionRequestedEvent;\\n    createEvent(eventInterface: \"PointerEvent\"): PointerEvent;\\n    createEvent(eventInterface: \"PopStateEvent\"): PopStateEvent;\\n    createEvent(eventInterface: \"ProgressEvent\"): ProgressEvent;\\n    createEvent(eventInterface: \"PromiseRejectionEvent\"): PromiseRejectionEvent;\\n    createEvent(eventInterface: \"RTCDTMFToneChangeEvent\"): RTCDTMFToneChangeEvent;\\n    createEvent(eventInterface: \"RTCDataChannelEvent\"): RTCDataChannelEvent;\\n    createEvent(eventInterface: \"RTCDtlsTransportStateChangedEvent\"): RTCDtlsTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCErrorEvent\"): RTCErrorEvent;\\n    createEvent(eventInterface: \"RTCIceCandidatePairChangedEvent\"): RTCIceCandidatePairChangedEvent;\\n    createEvent(eventInterface: \"RTCIceGathererEvent\"): RTCIceGathererEvent;\\n    createEvent(eventInterface: \"RTCIceTransportStateChangedEvent\"): RTCIceTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceErrorEvent\"): RTCPeerConnectionIceErrorEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceEvent\"): RTCPeerConnectionIceEvent;\\n    createEvent(eventInterface: \"RTCSsrcConflictEvent\"): RTCSsrcConflictEvent;\\n    createEvent(eventInterface: \"RTCStatsEvent\"): RTCStatsEvent;\\n    createEvent(eventInterface: \"RTCTrackEvent\"): RTCTrackEvent;\\n    createEvent(eventInterface: \"SVGZoomEvent\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SVGZoomEvents\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SecurityPolicyViolationEvent\"): SecurityPolicyViolationEvent;\\n    createEvent(eventInterface: \"ServiceWorkerMessageEvent\"): ServiceWorkerMessageEvent;\\n    createEvent(eventInterface: \"SpeechRecognitionError\"): SpeechRecognitionError;\\n    createEvent(eventInterface: \"SpeechRecognitionEvent\"): SpeechRecognitionEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisErrorEvent\"): SpeechSynthesisErrorEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisEvent\"): SpeechSynthesisEvent;\\n    createEvent(eventInterface: \"StorageEvent\"): StorageEvent;\\n    createEvent(eventInterface: \"TextEvent\"): TextEvent;\\n    createEvent(eventInterface: \"TouchEvent\"): TouchEvent;\\n    createEvent(eventInterface: \"TrackEvent\"): TrackEvent;\\n    createEvent(eventInterface: \"TransitionEvent\"): TransitionEvent;\\n    createEvent(eventInterface: \"UIEvent\"): UIEvent;\\n    createEvent(eventInterface: \"UIEvents\"): UIEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent\"): VRDisplayEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent \"): VRDisplayEvent ;\\n    createEvent(eventInterface: \"WebGLContextEvent\"): WebGLContextEvent;\\n    createEvent(eventInterface: \"WheelEvent\"): WheelEvent;\\n    createEvent(eventInterface: string): Event;\\n    /**\\n     * Creates a NodeIterator object that you can use to traverse filtered lists of nodes or elements in a document.\\n     * @param root The root element or node to start traversing on.\\n     * @param whatToShow The type of nodes or elements to appear in the node list\\n     * @param filter A custom NodeFilter function to use. For more information, see filter. Use null for no filter.\\n     * @param entityReferenceExpansion A flag that specifies whether entity reference nodes are expanded.\\n     */\\n    createNodeIterator(root: Node, whatToShow?: number, filter?: NodeFilter | null): NodeIterator;\\n    /**\\n     * Returns a ProcessingInstruction node whose target is target and data is data.\\n     * If target does not match the Name production an\\n     * \"InvalidCharacterError\" DOMException will be thrown.\\n     * If data contains \"?>\" an\\n     * \"InvalidCharacterError\" DOMException will be thrown.\\n     */\\n    createProcessingInstruction(target: string, data: string): ProcessingInstruction;\\n    /**\\n     *  Returns an empty range object that has both of its boundary points positioned at the beginning of the document.\\n     */\\n    createRange(): Range;\\n    /**\\n     * Creates a text string from the specified value.\\n     * @param data String that specifies the nodeValue property of the text node.\\n     */\\n    createTextNode(data: string): Text;\\n    createTouch(view: WindowProxy, target: EventTarget, identifier: number, pageX: number, pageY: number, screenX: number, screenY: number): Touch;\\n    createTouchList(...touches: Touch[]): TouchList;\\n    /**\\n     * Creates a TreeWalker object that you can use to traverse filtered lists of nodes or elements in a document.\\n     * @param root The root element or node to start traversing on.\\n     * @param whatToShow The type of nodes or elements to appear in the node list. For more information, see whatToShow.\\n     * @param filter A custom NodeFilter function to use.\\n     * @param entityReferenceExpansion A flag that specifies whether entity reference nodes are expanded.\\n     */\\n    createTreeWalker(root: Node, whatToShow?: number, filter?: NodeFilter | null, entityReferenceExpansion?: boolean): TreeWalker;\\n    /**\\n     * Returns the element for the specified x coordinate and the specified y coordinate.\\n     * @param x The x-offset\\n     * @param y The y-offset\\n     */\\n    elementFromPoint(x: number, y: number): Element | null;\\n    elementsFromPoint(x: number, y: number): Element[];\\n    evaluate(expression: string, contextNode: Node, resolver: XPathNSResolver | null, type: number, result: XPathResult | null): XPathResult;\\n    /**\\n     * Executes a command on the current document, current selection, or the given range.\\n     * @param commandId String that specifies the command to execute. This command can be any of the command identifiers that can be executed in script.\\n     * @param showUI Display the user interface, defaults to false.\\n     * @param value Value to assign.\\n     */\\n    execCommand(commandId: string, showUI?: boolean, value?: string): boolean;\\n    /**\\n     * Stops document\\'s fullscreen element from being displayed fullscreen and\\n     * resolves promise when done.\\n     */\\n    exitFullscreen(): Promise<void>;\\n    getAnimations(): Animation[];\\n    /**\\n     * Returns a reference to the first object with the specified value of the ID or NAME attribute.\\n     * @param elementId String that specifies the ID value. Case-insensitive.\\n     */\\n    getElementById(elementId: string): HTMLElement | null;\\n    /**\\n     * collection = element . getElementsByClassName(classNames)\\n     */\\n    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;\\n    /**\\n     * Gets a collection of objects based on the value of the NAME or ID attribute.\\n     * @param elementName Gets a collection of objects based on the value of the NAME or ID attribute.\\n     */\\n    getElementsByName(elementName: string): NodeListOf<HTMLElement>;\\n    /**\\n     * Retrieves a collection of objects based on the specified element name.\\n     * @param name Specifies the name of an element.\\n     */\\n    getElementsByTagName<K extends keyof HTMLElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<HTMLElementTagNameMap[K]>;\\n    getElementsByTagName<K extends keyof SVGElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<SVGElementTagNameMap[K]>;\\n    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;\\n    /**\\n     * If namespace and localName are\\n     * \"*\" returns a HTMLCollection of all descendant elements.\\n     * If only namespace is \"*\" returns a HTMLCollection of all descendant elements whose local name is localName.\\n     * If only localName is \"*\" returns a HTMLCollection of all descendant elements whose namespace is namespace.\\n     * Otherwise, returns a HTMLCollection of all descendant elements whose namespace is namespace and local name is localName.\\n     */\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/1999/xhtml\", localName: string): HTMLCollectionOf<HTMLElement>;\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/2000/svg\", localName: string): HTMLCollectionOf<SVGElement>;\\n    getElementsByTagNameNS(namespaceURI: string, localName: string): HTMLCollectionOf<Element>;\\n    /**\\n     * Gets a value indicating whether the object currently has focus.\\n     */\\n    hasFocus(): boolean;\\n    importNode<T extends Node>(importedNode: T, deep: boolean): T;\\n    /**\\n     * Opens a new window and loads a document specified by a given URL. Also, opens a new window that uses the url parameter and the name parameter to collect the output of the write method and the writeln method.\\n     * @param url Specifies a MIME type for the document.\\n     * @param name Specifies the name of the window. This name is used as the value for the TARGET attribute on a form or an anchor element.\\n     * @param features Contains a list of items separated by commas. Each item consists of an option and a value, separated by an equals sign (for example, \"fullscreen=yes, toolbar=yes\"). The following values are supported.\\n     * @param replace Specifies whether the existing entry for the document is replaced in the history list.\\n     */\\n    open(url?: string, name?: string, features?: string, replace?: boolean): Document;\\n    /**\\n     * Returns a Boolean value that indicates whether a specified command can be successfully executed using execCommand, given the current state of the document.\\n     * @param commandId Specifies a command identifier.\\n     */\\n    queryCommandEnabled(commandId: string): boolean;\\n    /**\\n     * Returns a Boolean value that indicates whether the specified command is in the indeterminate state.\\n     * @param commandId String that specifies a command identifier.\\n     */\\n    queryCommandIndeterm(commandId: string): boolean;\\n    /**\\n     * Returns a Boolean value that indicates the current state of the command.\\n     * @param commandId String that specifies a command identifier.\\n     */\\n    queryCommandState(commandId: string): boolean;\\n    /**\\n     * Returns a Boolean value that indicates whether the current command is supported on the current range.\\n     * @param commandId Specifies a command identifier.\\n     */\\n    queryCommandSupported(commandId: string): boolean;\\n    /**\\n     * Returns the current value of the document, range, or current selection for the given command.\\n     * @param commandId String that specifies a command identifier.\\n     */\\n    queryCommandValue(commandId: string): string;\\n    /** @deprecated */\\n    releaseEvents(): void;\\n    /**\\n     * Writes one or more HTML expressions to a document in the specified window.\\n     * @param content Specifies the text and HTML tags to write.\\n     */\\n    write(...text: string[]): void;\\n    /**\\n     * Writes one or more HTML expressions, followed by a carriage return, to a document in the specified window.\\n     * @param content The text and HTML tags to write.\\n     */\\n    writeln(...text: string[]): void;\\n    /**\\n     * Returns an object representing the current selection of the document that is loaded into the object displaying a webpage.\\n     */\\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: Document, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: Document, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Document: {\\n    prototype: Document;\\n    new(): Document;\\n};\\n\\ninterface DocumentAndElementEventHandlersEventMap {\\n    \"copy\": ClipboardEvent;\\n    \"cut\": ClipboardEvent;\\n    \"paste\": ClipboardEvent;\\n}\\n\\ninterface DocumentAndElementEventHandlers {\\n    oncopy: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\\n    oncut: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\\n    onpaste: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\\n    addEventListener<K extends keyof DocumentAndElementEventHandlersEventMap>(type: K, listener: (this: DocumentAndElementEventHandlers, ev: DocumentAndElementEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentAndElementEventHandlersEventMap>(type: K, listener: (this: DocumentAndElementEventHandlers, ev: DocumentAndElementEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface DocumentEvent {\\n    createEvent(eventInterface: \"AnimationEvent\"): AnimationEvent;\\n    createEvent(eventInterface: \"AnimationPlaybackEvent\"): AnimationPlaybackEvent;\\n    createEvent(eventInterface: \"AudioProcessingEvent\"): AudioProcessingEvent;\\n    createEvent(eventInterface: \"BeforeUnloadEvent\"): BeforeUnloadEvent;\\n    createEvent(eventInterface: \"ClipboardEvent\"): ClipboardEvent;\\n    createEvent(eventInterface: \"CloseEvent\"): CloseEvent;\\n    createEvent(eventInterface: \"CompositionEvent\"): CompositionEvent;\\n    createEvent(eventInterface: \"CustomEvent\"): CustomEvent;\\n    createEvent(eventInterface: \"DeviceLightEvent\"): DeviceLightEvent;\\n    createEvent(eventInterface: \"DeviceMotionEvent\"): DeviceMotionEvent;\\n    createEvent(eventInterface: \"DeviceOrientationEvent\"): DeviceOrientationEvent;\\n    createEvent(eventInterface: \"DragEvent\"): DragEvent;\\n    createEvent(eventInterface: \"ErrorEvent\"): ErrorEvent;\\n    createEvent(eventInterface: \"Event\"): Event;\\n    createEvent(eventInterface: \"Events\"): Event;\\n    createEvent(eventInterface: \"FocusEvent\"): FocusEvent;\\n    createEvent(eventInterface: \"FocusNavigationEvent\"): FocusNavigationEvent;\\n    createEvent(eventInterface: \"GamepadEvent\"): GamepadEvent;\\n    createEvent(eventInterface: \"HashChangeEvent\"): HashChangeEvent;\\n    createEvent(eventInterface: \"IDBVersionChangeEvent\"): IDBVersionChangeEvent;\\n    createEvent(eventInterface: \"KeyboardEvent\"): KeyboardEvent;\\n    createEvent(eventInterface: \"ListeningStateChangedEvent\"): ListeningStateChangedEvent;\\n    createEvent(eventInterface: \"MSGestureEvent\"): MSGestureEvent;\\n    createEvent(eventInterface: \"MSMediaKeyMessageEvent\"): MSMediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MSMediaKeyNeededEvent\"): MSMediaKeyNeededEvent;\\n    createEvent(eventInterface: \"MSPointerEvent\"): MSPointerEvent;\\n    createEvent(eventInterface: \"MediaEncryptedEvent\"): MediaEncryptedEvent;\\n    createEvent(eventInterface: \"MediaKeyMessageEvent\"): MediaKeyMessageEvent;\\n    createEvent(eventInterface: \"MediaQueryListEvent\"): MediaQueryListEvent;\\n    createEvent(eventInterface: \"MediaStreamErrorEvent\"): MediaStreamErrorEvent;\\n    createEvent(eventInterface: \"MediaStreamEvent\"): MediaStreamEvent;\\n    createEvent(eventInterface: \"MediaStreamTrackEvent\"): MediaStreamTrackEvent;\\n    createEvent(eventInterface: \"MessageEvent\"): MessageEvent;\\n    createEvent(eventInterface: \"MouseEvent\"): MouseEvent;\\n    createEvent(eventInterface: \"MouseEvents\"): MouseEvent;\\n    createEvent(eventInterface: \"MutationEvent\"): MutationEvent;\\n    createEvent(eventInterface: \"MutationEvents\"): MutationEvent;\\n    createEvent(eventInterface: \"OfflineAudioCompletionEvent\"): OfflineAudioCompletionEvent;\\n    createEvent(eventInterface: \"OverflowEvent\"): OverflowEvent;\\n    createEvent(eventInterface: \"PageTransitionEvent\"): PageTransitionEvent;\\n    createEvent(eventInterface: \"PaymentRequestUpdateEvent\"): PaymentRequestUpdateEvent;\\n    createEvent(eventInterface: \"PermissionRequestedEvent\"): PermissionRequestedEvent;\\n    createEvent(eventInterface: \"PointerEvent\"): PointerEvent;\\n    createEvent(eventInterface: \"PopStateEvent\"): PopStateEvent;\\n    createEvent(eventInterface: \"ProgressEvent\"): ProgressEvent;\\n    createEvent(eventInterface: \"PromiseRejectionEvent\"): PromiseRejectionEvent;\\n    createEvent(eventInterface: \"RTCDTMFToneChangeEvent\"): RTCDTMFToneChangeEvent;\\n    createEvent(eventInterface: \"RTCDataChannelEvent\"): RTCDataChannelEvent;\\n    createEvent(eventInterface: \"RTCDtlsTransportStateChangedEvent\"): RTCDtlsTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCErrorEvent\"): RTCErrorEvent;\\n    createEvent(eventInterface: \"RTCIceCandidatePairChangedEvent\"): RTCIceCandidatePairChangedEvent;\\n    createEvent(eventInterface: \"RTCIceGathererEvent\"): RTCIceGathererEvent;\\n    createEvent(eventInterface: \"RTCIceTransportStateChangedEvent\"): RTCIceTransportStateChangedEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceErrorEvent\"): RTCPeerConnectionIceErrorEvent;\\n    createEvent(eventInterface: \"RTCPeerConnectionIceEvent\"): RTCPeerConnectionIceEvent;\\n    createEvent(eventInterface: \"RTCSsrcConflictEvent\"): RTCSsrcConflictEvent;\\n    createEvent(eventInterface: \"RTCStatsEvent\"): RTCStatsEvent;\\n    createEvent(eventInterface: \"RTCTrackEvent\"): RTCTrackEvent;\\n    createEvent(eventInterface: \"SVGZoomEvent\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SVGZoomEvents\"): SVGZoomEvent;\\n    createEvent(eventInterface: \"SecurityPolicyViolationEvent\"): SecurityPolicyViolationEvent;\\n    createEvent(eventInterface: \"ServiceWorkerMessageEvent\"): ServiceWorkerMessageEvent;\\n    createEvent(eventInterface: \"SpeechRecognitionError\"): SpeechRecognitionError;\\n    createEvent(eventInterface: \"SpeechRecognitionEvent\"): SpeechRecognitionEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisErrorEvent\"): SpeechSynthesisErrorEvent;\\n    createEvent(eventInterface: \"SpeechSynthesisEvent\"): SpeechSynthesisEvent;\\n    createEvent(eventInterface: \"StorageEvent\"): StorageEvent;\\n    createEvent(eventInterface: \"TextEvent\"): TextEvent;\\n    createEvent(eventInterface: \"TouchEvent\"): TouchEvent;\\n    createEvent(eventInterface: \"TrackEvent\"): TrackEvent;\\n    createEvent(eventInterface: \"TransitionEvent\"): TransitionEvent;\\n    createEvent(eventInterface: \"UIEvent\"): UIEvent;\\n    createEvent(eventInterface: \"UIEvents\"): UIEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent\"): VRDisplayEvent;\\n    createEvent(eventInterface: \"VRDisplayEvent \"): VRDisplayEvent ;\\n    createEvent(eventInterface: \"WebGLContextEvent\"): WebGLContextEvent;\\n    createEvent(eventInterface: \"WheelEvent\"): WheelEvent;\\n    createEvent(eventInterface: string): Event;\\n}\\n\\ninterface DocumentFragment extends Node, NonElementParentNode, ParentNode {\\n    getElementById(elementId: string): HTMLElement | null;\\n}\\n\\ndeclare var DocumentFragment: {\\n    prototype: DocumentFragment;\\n    new(): DocumentFragment;\\n};\\n\\ninterface DocumentOrShadowRoot {\\n    readonly activeElement: Element | null;\\n    /**\\n     * Retrieves a collection of styleSheet objects representing the style sheets that correspond to each instance of a link or style object in the document.\\n     */\\n    readonly styleSheets: StyleSheetList;\\n    elementFromPoint(x: number, y: number): Element | null;\\n    elementsFromPoint(x: number, y: number): Element[];\\n    getSelection(): Selection | null;\\n}\\n\\ninterface DocumentTimeline extends AnimationTimeline {\\n}\\n\\ndeclare var DocumentTimeline: {\\n    prototype: DocumentTimeline;\\n    new(options?: DocumentTimelineOptions): DocumentTimeline;\\n};\\n\\ninterface DocumentType extends Node, ChildNode {\\n    readonly name: string;\\n    readonly publicId: string;\\n    readonly systemId: string;\\n}\\n\\ndeclare var DocumentType: {\\n    prototype: DocumentType;\\n    new(): DocumentType;\\n};\\n\\ninterface DragEvent extends MouseEvent {\\n    /**\\n     * Returns the DataTransfer object for the event.\\n     */\\n    readonly dataTransfer: DataTransfer | null;\\n}\\n\\ndeclare var DragEvent: {\\n    prototype: DragEvent;\\n    new(type: string, eventInitDict?: DragEventInit): DragEvent;\\n};\\n\\ninterface DynamicsCompressorNode extends AudioNode {\\n    readonly attack: AudioParam;\\n    readonly knee: AudioParam;\\n    readonly ratio: AudioParam;\\n    readonly reduction: number;\\n    readonly release: AudioParam;\\n    readonly threshold: AudioParam;\\n}\\n\\ndeclare var DynamicsCompressorNode: {\\n    prototype: DynamicsCompressorNode;\\n    new(context: BaseAudioContext, options?: DynamicsCompressorOptions): DynamicsCompressorNode;\\n};\\n\\ninterface EXT_blend_minmax {\\n    readonly MAX_EXT: number;\\n    readonly MIN_EXT: number;\\n}\\n\\ninterface EXT_frag_depth {\\n}\\n\\ninterface EXT_sRGB {\\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT: number;\\n    readonly SRGB8_ALPHA8_EXT: number;\\n    readonly SRGB_ALPHA_EXT: number;\\n    readonly SRGB_EXT: number;\\n}\\n\\ninterface EXT_shader_texture_lod {\\n}\\n\\ninterface EXT_texture_filter_anisotropic {\\n    readonly MAX_TEXTURE_MAX_ANISOTROPY_EXT: number;\\n    readonly TEXTURE_MAX_ANISOTROPY_EXT: number;\\n}\\n\\ndeclare var EXT_texture_filter_anisotropic: {\\n    prototype: EXT_texture_filter_anisotropic;\\n    new(): EXT_texture_filter_anisotropic;\\n    readonly MAX_TEXTURE_MAX_ANISOTROPY_EXT: number;\\n    readonly TEXTURE_MAX_ANISOTROPY_EXT: number;\\n};\\n\\ninterface ElementEventMap {\\n    \"fullscreenchange\": Event;\\n    \"fullscreenerror\": Event;\\n}\\n\\ninterface Element extends Node, ParentNode, NonDocumentTypeChildNode, ChildNode, Slotable, Animatable {\\n    readonly assignedSlot: HTMLSlotElement | null;\\n    readonly attributes: NamedNodeMap;\\n    /**\\n     * Allows for manipulation of element\\'s class content attribute as a\\n     * set of whitespace-separated tokens through a DOMTokenList object.\\n     */\\n    readonly classList: DOMTokenList;\\n    /**\\n     * Returns the value of element\\'s class content attribute. Can be set\\n     * to change it.\\n     */\\n    className: string;\\n    readonly clientHeight: number;\\n    readonly clientLeft: number;\\n    readonly clientTop: number;\\n    readonly clientWidth: number;\\n    /**\\n     * Returns the value of element\\'s id content attribute. Can be set to\\n     * change it.\\n     */\\n    id: string;\\n    innerHTML: string;\\n    /**\\n     * Returns the local name.\\n     */\\n    readonly localName: string;\\n    /**\\n     * Returns the namespace.\\n     */\\n    readonly namespaceURI: string | null;\\n    onfullscreenchange: ((this: Element, ev: Event) => any) | null;\\n    onfullscreenerror: ((this: Element, ev: Event) => any) | null;\\n    outerHTML: string;\\n    /**\\n     * Returns the namespace prefix.\\n     */\\n    readonly prefix: string | null;\\n    readonly scrollHeight: number;\\n    scrollLeft: number;\\n    scrollTop: number;\\n    readonly scrollWidth: number;\\n    /**\\n     * Returns element\\'s shadow root, if any, and if shadow root\\'s mode is \"open\", and null otherwise.\\n     */\\n    readonly shadowRoot: ShadowRoot | null;\\n    /**\\n     * Returns the value of element\\'s slot content attribute. Can be set to\\n     * change it.\\n     */\\n    slot: string;\\n    /**\\n     * Returns the HTML-uppercased qualified name.\\n     */\\n    readonly tagName: string;\\n    /**\\n     * Creates a shadow root for element and returns it.\\n     */\\n    attachShadow(shadowRootInitDict: ShadowRootInit): ShadowRoot;\\n    /**\\n     * Returns the first (starting at element) inclusive ancestor that matches selectors, and null otherwise.\\n     */\\n    closest<K extends keyof HTMLElementTagNameMap>(selector: K): HTMLElementTagNameMap[K] | null;\\n    closest<K extends keyof SVGElementTagNameMap>(selector: K): SVGElementTagNameMap[K] | null;\\n    closest(selector: string): Element | null;\\n    /**\\n     * Returns element\\'s first attribute whose qualified name is qualifiedName, and null if there is no such attribute otherwise.\\n     */\\n    getAttribute(qualifiedName: string): string | null;\\n    /**\\n     * Returns element\\'s attribute whose namespace is namespace and local name is localName, and null if there is\\n     * no such attribute otherwise.\\n     */\\n    getAttributeNS(namespace: string | null, localName: string): string | null;\\n    /**\\n     * Returns the qualified names of all element\\'s attributes.\\n     * Can contain duplicates.\\n     */\\n    getAttributeNames(): string[];\\n    getAttributeNode(name: string): Attr | null;\\n    getAttributeNodeNS(namespaceURI: string, localName: string): Attr | null;\\n    getBoundingClientRect(): ClientRect | DOMRect;\\n    getClientRects(): ClientRectList | DOMRectList;\\n    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;\\n    getElementsByTagName<K extends keyof HTMLElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<HTMLElementTagNameMap[K]>;\\n    getElementsByTagName<K extends keyof SVGElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<SVGElementTagNameMap[K]>;\\n    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/1999/xhtml\", localName: string): HTMLCollectionOf<HTMLElement>;\\n    getElementsByTagNameNS(namespaceURI: \"http://www.w3.org/2000/svg\", localName: string): HTMLCollectionOf<SVGElement>;\\n    getElementsByTagNameNS(namespaceURI: string, localName: string): HTMLCollectionOf<Element>;\\n    /**\\n     * Returns true if element has an attribute whose qualified name is qualifiedName, and false otherwise.\\n     */\\n    hasAttribute(qualifiedName: string): boolean;\\n    /**\\n     * Returns true if element has an attribute whose namespace is namespace and local name is localName.\\n     */\\n    hasAttributeNS(namespace: string | null, localName: string): boolean;\\n    /**\\n     * Returns true if element has attributes, and false otherwise.\\n     */\\n    hasAttributes(): boolean;\\n    hasPointerCapture(pointerId: number): boolean;\\n    insertAdjacentElement(position: InsertPosition, insertedElement: Element): Element | null;\\n    insertAdjacentHTML(where: InsertPosition, html: string): void;\\n    insertAdjacentText(where: InsertPosition, text: string): void;\\n    /**\\n     * Returns true if matching selectors against element\\'s root yields element, and false otherwise.\\n     */\\n    matches(selectors: string): boolean;\\n    msGetRegionContent(): any;\\n    releasePointerCapture(pointerId: number): void;\\n    /**\\n     * Removes element\\'s first attribute whose qualified name is qualifiedName.\\n     */\\n    removeAttribute(qualifiedName: string): void;\\n    /**\\n     * Removes element\\'s attribute whose namespace is namespace and local name is localName.\\n     */\\n    removeAttributeNS(namespace: string | null, localName: string): void;\\n    removeAttributeNode(attr: Attr): Attr;\\n    /**\\n     * Displays element fullscreen and resolves promise when done.\\n     */\\n    requestFullscreen(): Promise<void>;\\n    scroll(options?: ScrollToOptions): void;\\n    scroll(x: number, y: number): void;\\n    scrollBy(options?: ScrollToOptions): void;\\n    scrollBy(x: number, y: number): void;\\n    scrollIntoView(arg?: boolean | ScrollIntoViewOptions): void;\\n    scrollTo(options?: ScrollToOptions): void;\\n    scrollTo(x: number, y: number): void;\\n    /**\\n     * Sets the value of element\\'s first attribute whose qualified name is qualifiedName to value.\\n     */\\n    setAttribute(qualifiedName: string, value: string): void;\\n    /**\\n     * Sets the value of element\\'s attribute whose namespace is namespace and local name is localName to value.\\n     */\\n    setAttributeNS(namespace: string | null, qualifiedName: string, value: string): void;\\n    setAttributeNode(attr: Attr): Attr | null;\\n    setAttributeNodeNS(attr: Attr): Attr | null;\\n    setPointerCapture(pointerId: number): void;\\n    /**\\n     * If force is not given, \"toggles\" qualifiedName, removing it if it is\\n     * present and adding it if it is not present. If force is true, adds qualifiedName. If force is false, removes qualifiedName.\\n     * Returns true if qualifiedName is now present, and false otherwise.\\n     */\\n    toggleAttribute(qualifiedName: string, force?: boolean): boolean;\\n    webkitMatchesSelector(selectors: string): boolean;\\n    addEventListener<K extends keyof ElementEventMap>(type: K, listener: (this: Element, ev: ElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ElementEventMap>(type: K, listener: (this: Element, ev: ElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Element: {\\n    prototype: Element;\\n    new(): Element;\\n};\\n\\ninterface ElementCSSInlineStyle {\\n    readonly style: CSSStyleDeclaration;\\n}\\n\\ninterface ElementContentEditable {\\n    contentEditable: string;\\n    inputMode: string;\\n    readonly isContentEditable: boolean;\\n}\\n\\ninterface ElementCreationOptions {\\n    is?: string;\\n}\\n\\ninterface ErrorEvent extends Event {\\n    readonly colno: number;\\n    readonly error: any;\\n    readonly filename: string;\\n    readonly lineno: number;\\n    readonly message: string;\\n}\\n\\ndeclare var ErrorEvent: {\\n    prototype: ErrorEvent;\\n    new(type: string, eventInitDict?: ErrorEventInit): ErrorEvent;\\n};\\n\\ninterface Event {\\n    /**\\n     * Returns true or false depending on how event was initialized. True if event goes through its target\\'s ancestors in reverse tree order, and false otherwise.\\n     */\\n    readonly bubbles: boolean;\\n    cancelBubble: boolean;\\n    readonly cancelable: boolean;\\n    /**\\n     * Returns true or false depending on how event was initialized. True if event invokes listeners past a ShadowRoot node that is the root of its target, and false otherwise.\\n     */\\n    readonly composed: boolean;\\n    /**\\n     * Returns the object whose event listener\\'s callback is currently being\\n     * invoked.\\n     */\\n    readonly currentTarget: EventTarget | null;\\n    readonly defaultPrevented: boolean;\\n    readonly eventPhase: number;\\n    /**\\n     * Returns true if event was dispatched by the user agent, and\\n     * false otherwise.\\n     */\\n    readonly isTrusted: boolean;\\n    returnValue: boolean;\\n    readonly srcElement: Element | null;\\n    /**\\n     * Returns the object to which event is dispatched (its target).\\n     */\\n    readonly target: EventTarget | null;\\n    /**\\n     * Returns the event\\'s timestamp as the number of milliseconds measured relative to\\n     * the time origin.\\n     */\\n    readonly timeStamp: number;\\n    /**\\n     * Returns the type of event, e.g.\\n     * \"click\", \"hashchange\", or\\n     * \"submit\".\\n     */\\n    readonly type: string;\\n    composedPath(): EventTarget[];\\n    initEvent(type: string, bubbles?: boolean, cancelable?: boolean): void;\\n    preventDefault(): void;\\n    /**\\n     * Invoking this method prevents event from reaching\\n     * any registered event listeners after the current one finishes running and, when dispatched in a tree, also prevents event from reaching any\\n     * other objects.\\n     */\\n    stopImmediatePropagation(): void;\\n    /**\\n     * When dispatched in a tree, invoking this method prevents event from reaching any objects other than the current object.\\n     */\\n    stopPropagation(): void;\\n    readonly AT_TARGET: number;\\n    readonly BUBBLING_PHASE: number;\\n    readonly CAPTURING_PHASE: number;\\n    readonly NONE: number;\\n}\\n\\ndeclare var Event: {\\n    prototype: Event;\\n    new(type: string, eventInitDict?: EventInit): Event;\\n    readonly AT_TARGET: number;\\n    readonly BUBBLING_PHASE: number;\\n    readonly CAPTURING_PHASE: number;\\n    readonly NONE: number;\\n};\\n\\ninterface EventListenerObject {\\n    handleEvent(evt: Event): void;\\n}\\n\\ninterface EventSource extends EventTarget {\\n    readonly CLOSED: number;\\n    readonly CONNECTING: number;\\n    readonly OPEN: number;\\n    onerror: (evt: MessageEvent) => any;\\n    onmessage: (evt: MessageEvent) => any;\\n    onopen: (evt: MessageEvent) => any;\\n    readonly readyState: number;\\n    readonly url: string;\\n    readonly withCredentials: boolean;\\n    close(): void;\\n}\\n\\ndeclare var EventSource: {\\n    prototype: EventSource;\\n    new(url: string, eventSourceInitDict?: EventSourceInit): EventSource;\\n};\\n\\ninterface EventSourceInit {\\n    readonly withCredentials: boolean;\\n}\\n\\ninterface EventTarget {\\n    /**\\n     * Appends an event listener for events whose type attribute value is type. The callback argument sets the callback that will be invoked when the event is dispatched.\\n     * The options argument sets listener-specific options. For compatibility this can be a\\n     * boolean, in which case the method behaves exactly as if the value was specified as options\\'s capture.\\n     * When set to true, options\\'s capture prevents callback from being invoked when the event\\'s eventPhase attribute value is BUBBLING_PHASE. When false (or not present), callback will not be invoked when event\\'s eventPhase attribute value is CAPTURING_PHASE. Either way, callback will be invoked if event\\'s eventPhase attribute value is AT_TARGET.\\n     * When set to true, options\\'s passive indicates that the callback will not cancel the event by invoking preventDefault(). This is used to enable performance optimizations described in §2.8 Observing event listeners.\\n     * When set to true, options\\'s once indicates that the callback will only be invoked once after which the event listener will\\n     * be removed.\\n     * The event listener is appended to target\\'s event listener list and is not appended if it has the same type, callback, and capture.\\n     */\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject | null, options?: boolean | AddEventListenerOptions): void;\\n    /**\\n     * Dispatches a synthetic event event to target and returns true\\n     * if either event\\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\\n     */\\n    dispatchEvent(event: Event): boolean;\\n    /**\\n     * Removes the event listener in target\\'s event listener list with the same type, callback, and options.\\n     */\\n    removeEventListener(type: string, callback: EventListenerOrEventListenerObject | null, options?: EventListenerOptions | boolean): void;\\n}\\n\\ndeclare var EventTarget: {\\n    prototype: EventTarget;\\n    new(): EventTarget;\\n};\\n\\ninterface ExtensionScriptApis {\\n    extensionIdToShortId(extensionId: string): number;\\n    fireExtensionApiTelemetry(functionName: string, isSucceeded: boolean, isSupported: boolean, errorString: string): void;\\n    genericFunction(routerAddress: any, parameters?: string, callbackId?: number): void;\\n    genericSynchronousFunction(functionId: number, parameters?: string): string;\\n    genericWebRuntimeCallout(to: any, from: any, payload: string): void;\\n    getExtensionId(): string;\\n    registerGenericFunctionCallbackHandler(callbackHandler: Function): void;\\n    registerGenericPersistentCallbackHandler(callbackHandler: Function): void;\\n    registerWebRuntimeCallbackHandler(handler: Function): any;\\n}\\n\\ndeclare var ExtensionScriptApis: {\\n    prototype: ExtensionScriptApis;\\n    new(): ExtensionScriptApis;\\n};\\n\\ninterface External {\\n    /** @deprecated */\\n    AddSearchProvider(): void;\\n    /** @deprecated */\\n    IsSearchProviderInstalled(): void;\\n}\\n\\ninterface File extends Blob {\\n    readonly lastModified: number;\\n    readonly name: string;\\n}\\n\\ndeclare var File: {\\n    prototype: File;\\n    new(fileBits: BlobPart[], fileName: string, options?: FilePropertyBag): File;\\n};\\n\\ninterface FileList {\\n    readonly length: number;\\n    item(index: number): File | null;\\n    [index: number]: File;\\n}\\n\\ndeclare var FileList: {\\n    prototype: FileList;\\n    new(): FileList;\\n};\\n\\ninterface FileReaderEventMap {\\n    \"abort\": ProgressEvent;\\n    \"error\": ProgressEvent;\\n    \"load\": ProgressEvent;\\n    \"loadend\": ProgressEvent;\\n    \"loadstart\": ProgressEvent;\\n    \"progress\": ProgressEvent;\\n}\\n\\ninterface FileReader extends EventTarget {\\n    readonly error: DOMException | null;\\n    onabort: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onerror: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onload: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onloadend: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onloadstart: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    onprogress: ((this: FileReader, ev: ProgressEvent) => any) | null;\\n    readonly readyState: number;\\n    readonly result: string | ArrayBuffer | null;\\n    abort(): void;\\n    readAsArrayBuffer(blob: Blob): void;\\n    readAsBinaryString(blob: Blob): void;\\n    readAsDataURL(blob: Blob): void;\\n    readAsText(blob: Blob, label?: string): void;\\n    readonly DONE: number;\\n    readonly EMPTY: number;\\n    readonly LOADING: number;\\n    addEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var FileReader: {\\n    prototype: FileReader;\\n    new(): FileReader;\\n    readonly DONE: number;\\n    readonly EMPTY: number;\\n    readonly LOADING: number;\\n};\\n\\ninterface FocusEvent extends UIEvent {\\n    readonly relatedTarget: EventTarget;\\n    initFocusEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, relatedTargetArg: EventTarget): void;\\n}\\n\\ndeclare var FocusEvent: {\\n    prototype: FocusEvent;\\n    new(typeArg: string, eventInitDict?: FocusEventInit): FocusEvent;\\n};\\n\\ninterface FocusNavigationEvent extends Event {\\n    readonly navigationReason: NavigationReason;\\n    readonly originHeight: number;\\n    readonly originLeft: number;\\n    readonly originTop: number;\\n    readonly originWidth: number;\\n    requestFocus(): void;\\n}\\n\\ndeclare var FocusNavigationEvent: {\\n    prototype: FocusNavigationEvent;\\n    new(type: string, eventInitDict?: FocusNavigationEventInit): FocusNavigationEvent;\\n};\\n\\ninterface FormData {\\n    append(name: string, value: string | Blob, fileName?: string): void;\\n    delete(name: string): void;\\n    get(name: string): FormDataEntryValue | null;\\n    getAll(name: string): FormDataEntryValue[];\\n    has(name: string): boolean;\\n    set(name: string, value: string | Blob, fileName?: string): void;\\n    forEach(callbackfn: (value: FormDataEntryValue, key: string, parent: FormData) => void, thisArg?: any): void;\\n}\\n\\ndeclare var FormData: {\\n    prototype: FormData;\\n    new(form?: HTMLFormElement): FormData;\\n};\\n\\ninterface GainNode extends AudioNode {\\n    readonly gain: AudioParam;\\n}\\n\\ndeclare var GainNode: {\\n    prototype: GainNode;\\n    new(context: BaseAudioContext, options?: GainOptions): GainNode;\\n};\\n\\ninterface Gamepad {\\n    readonly axes: number[];\\n    readonly buttons: GamepadButton[];\\n    readonly connected: boolean;\\n    readonly displayId: number;\\n    readonly hand: GamepadHand;\\n    readonly hapticActuators: GamepadHapticActuator[];\\n    readonly id: string;\\n    readonly index: number;\\n    readonly mapping: GamepadMappingType;\\n    readonly pose: GamepadPose | null;\\n    readonly timestamp: number;\\n}\\n\\ndeclare var Gamepad: {\\n    prototype: Gamepad;\\n    new(): Gamepad;\\n};\\n\\ninterface GamepadButton {\\n    readonly pressed: boolean;\\n    readonly touched: boolean;\\n    readonly value: number;\\n}\\n\\ndeclare var GamepadButton: {\\n    prototype: GamepadButton;\\n    new(): GamepadButton;\\n};\\n\\ninterface GamepadEvent extends Event {\\n    readonly gamepad: Gamepad;\\n}\\n\\ndeclare var GamepadEvent: {\\n    prototype: GamepadEvent;\\n    new(typeArg: string, eventInitDict?: GamepadEventInit): GamepadEvent;\\n};\\n\\ninterface GamepadHapticActuator {\\n    readonly type: GamepadHapticActuatorType;\\n    pulse(value: number, duration: number): Promise<boolean>;\\n}\\n\\ndeclare var GamepadHapticActuator: {\\n    prototype: GamepadHapticActuator;\\n    new(): GamepadHapticActuator;\\n};\\n\\ninterface GamepadPose {\\n    readonly angularAcceleration: Float32Array | null;\\n    readonly angularVelocity: Float32Array | null;\\n    readonly hasOrientation: boolean;\\n    readonly hasPosition: boolean;\\n    readonly linearAcceleration: Float32Array | null;\\n    readonly linearVelocity: Float32Array | null;\\n    readonly orientation: Float32Array | null;\\n    readonly position: Float32Array | null;\\n}\\n\\ndeclare var GamepadPose: {\\n    prototype: GamepadPose;\\n    new(): GamepadPose;\\n};\\n\\ninterface Geolocation {\\n    clearWatch(watchId: number): void;\\n    getCurrentPosition(successCallback: PositionCallback, errorCallback?: PositionErrorCallback, options?: PositionOptions): void;\\n    watchPosition(successCallback: PositionCallback, errorCallback?: PositionErrorCallback, options?: PositionOptions): number;\\n}\\n\\ninterface GetSVGDocument {\\n    getSVGDocument(): Document;\\n}\\n\\ninterface GlobalEventHandlersEventMap {\\n    \"abort\": UIEvent;\\n    \"animationcancel\": AnimationEvent;\\n    \"animationend\": AnimationEvent;\\n    \"animationiteration\": AnimationEvent;\\n    \"animationstart\": AnimationEvent;\\n    \"auxclick\": Event;\\n    \"blur\": FocusEvent;\\n    \"cancel\": Event;\\n    \"canplay\": Event;\\n    \"canplaythrough\": Event;\\n    \"change\": Event;\\n    \"click\": MouseEvent;\\n    \"close\": Event;\\n    \"contextmenu\": MouseEvent;\\n    \"cuechange\": Event;\\n    \"dblclick\": MouseEvent;\\n    \"drag\": DragEvent;\\n    \"dragend\": DragEvent;\\n    \"dragenter\": DragEvent;\\n    \"dragexit\": Event;\\n    \"dragleave\": DragEvent;\\n    \"dragover\": DragEvent;\\n    \"dragstart\": DragEvent;\\n    \"drop\": DragEvent;\\n    \"durationchange\": Event;\\n    \"emptied\": Event;\\n    \"ended\": Event;\\n    \"error\": ErrorEvent;\\n    \"focus\": FocusEvent;\\n    \"gotpointercapture\": PointerEvent;\\n    \"input\": Event;\\n    \"invalid\": Event;\\n    \"keydown\": KeyboardEvent;\\n    \"keypress\": KeyboardEvent;\\n    \"keyup\": KeyboardEvent;\\n    \"load\": Event;\\n    \"loadeddata\": Event;\\n    \"loadedmetadata\": Event;\\n    \"loadend\": ProgressEvent;\\n    \"loadstart\": Event;\\n    \"lostpointercapture\": PointerEvent;\\n    \"mousedown\": MouseEvent;\\n    \"mouseenter\": MouseEvent;\\n    \"mouseleave\": MouseEvent;\\n    \"mousemove\": MouseEvent;\\n    \"mouseout\": MouseEvent;\\n    \"mouseover\": MouseEvent;\\n    \"mouseup\": MouseEvent;\\n    \"pause\": Event;\\n    \"play\": Event;\\n    \"playing\": Event;\\n    \"pointercancel\": PointerEvent;\\n    \"pointerdown\": PointerEvent;\\n    \"pointerenter\": PointerEvent;\\n    \"pointerleave\": PointerEvent;\\n    \"pointermove\": PointerEvent;\\n    \"pointerout\": PointerEvent;\\n    \"pointerover\": PointerEvent;\\n    \"pointerup\": PointerEvent;\\n    \"progress\": ProgressEvent;\\n    \"ratechange\": Event;\\n    \"reset\": Event;\\n    \"resize\": UIEvent;\\n    \"scroll\": UIEvent;\\n    \"securitypolicyviolation\": SecurityPolicyViolationEvent;\\n    \"seeked\": Event;\\n    \"seeking\": Event;\\n    \"select\": UIEvent;\\n    \"stalled\": Event;\\n    \"submit\": Event;\\n    \"suspend\": Event;\\n    \"timeupdate\": Event;\\n    \"toggle\": Event;\\n    \"touchcancel\": TouchEvent;\\n    \"touchend\": TouchEvent;\\n    \"touchmove\": TouchEvent;\\n    \"touchstart\": TouchEvent;\\n    \"transitioncancel\": TransitionEvent;\\n    \"transitionend\": TransitionEvent;\\n    \"transitionrun\": TransitionEvent;\\n    \"transitionstart\": TransitionEvent;\\n    \"volumechange\": Event;\\n    \"waiting\": Event;\\n    \"wheel\": WheelEvent;\\n}\\n\\ninterface GlobalEventHandlers {\\n    /**\\n     * Fires when the user aborts the download.\\n     * @param ev The event.\\n     */\\n    onabort: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    onanimationcancel: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onanimationend: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onanimationiteration: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onanimationstart: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\\n    onauxclick: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the object loses the input focus.\\n     * @param ev The focus event.\\n     */\\n    onblur: ((this: GlobalEventHandlers, ev: FocusEvent) => any) | null;\\n    oncancel: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when playback is possible, but would require further buffering.\\n     * @param ev The event.\\n     */\\n    oncanplay: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    oncanplaythrough: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the contents of the object or selection have changed.\\n     * @param ev The event.\\n     */\\n    onchange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user clicks the left mouse button on the object\\n     * @param ev The mouse event.\\n     */\\n    onclick: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    onclose: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user clicks the right mouse button in the client area, opening the context menu.\\n     * @param ev The mouse event.\\n     */\\n    oncontextmenu: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    oncuechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user double-clicks the object.\\n     * @param ev The mouse event.\\n     */\\n    ondblclick: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires on the source object continuously during a drag operation.\\n     * @param ev The event.\\n     */\\n    ondrag: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the source object when the user releases the mouse at the close of a drag operation.\\n     * @param ev The event.\\n     */\\n    ondragend: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the target element when the user drags the object to a valid drop target.\\n     * @param ev The drag event.\\n     */\\n    ondragenter: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    ondragexit: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires on the target object when the user moves the mouse out of a valid drop target during a drag operation.\\n     * @param ev The drag event.\\n     */\\n    ondragleave: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the target element continuously while the user drags the object over a valid drop target.\\n     * @param ev The event.\\n     */\\n    ondragover: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Fires on the source object when the user starts to drag a text selection or selected object.\\n     * @param ev The event.\\n     */\\n    ondragstart: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    ondrop: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\\n    /**\\n     * Occurs when the duration attribute is updated.\\n     * @param ev The event.\\n     */\\n    ondurationchange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the media element is reset to its initial state.\\n     * @param ev The event.\\n     */\\n    onemptied: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the end of playback is reached.\\n     * @param ev The event\\n     */\\n    onended: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when an error occurs during object loading.\\n     * @param ev The event.\\n     */\\n    onerror: ErrorEventHandler;\\n    /**\\n     * Fires when the object receives focus.\\n     * @param ev The event.\\n     */\\n    onfocus: ((this: GlobalEventHandlers, ev: FocusEvent) => any) | null;\\n    ongotpointercapture: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    oninput: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    oninvalid: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user presses a key.\\n     * @param ev The keyboard event\\n     */\\n    onkeydown: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\\n    /**\\n     * Fires when the user presses an alphanumeric key.\\n     * @param ev The event.\\n     */\\n    onkeypress: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\\n    /**\\n     * Fires when the user releases a key.\\n     * @param ev The keyboard event\\n     */\\n    onkeyup: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\\n    /**\\n     * Fires immediately after the browser loads the object.\\n     * @param ev The event.\\n     */\\n    onload: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when media data is loaded at the current playback position.\\n     * @param ev The event.\\n     */\\n    onloadeddata: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the duration and dimensions of the media have been determined.\\n     * @param ev The event.\\n     */\\n    onloadedmetadata: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onloadend: ((this: GlobalEventHandlers, ev: ProgressEvent) => any) | null;\\n    /**\\n     * Occurs when Internet Explorer begins looking for media data.\\n     * @param ev The event.\\n     */\\n    onloadstart: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onlostpointercapture: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    /**\\n     * Fires when the user clicks the object with either mouse button.\\n     * @param ev The mouse event.\\n     */\\n    onmousedown: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    onmouseenter: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    onmouseleave: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user moves the mouse over the object.\\n     * @param ev The mouse event.\\n     */\\n    onmousemove: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user moves the mouse pointer outside the boundaries of the object.\\n     * @param ev The mouse event.\\n     */\\n    onmouseout: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user moves the mouse pointer into the object.\\n     * @param ev The mouse event.\\n     */\\n    onmouseover: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Fires when the user releases a mouse button while the mouse is over the object.\\n     * @param ev The mouse event.\\n     */\\n    onmouseup: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\\n    /**\\n     * Occurs when playback is paused.\\n     * @param ev The event.\\n     */\\n    onpause: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the play method is requested.\\n     * @param ev The event.\\n     */\\n    onplay: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the audio or video has started playing.\\n     * @param ev The event.\\n     */\\n    onplaying: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onpointercancel: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerdown: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerenter: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerleave: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointermove: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerout: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerover: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    onpointerup: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\\n    /**\\n     * Occurs to indicate progress while downloading media data.\\n     * @param ev The event.\\n     */\\n    onprogress: ((this: GlobalEventHandlers, ev: ProgressEvent) => any) | null;\\n    /**\\n     * Occurs when the playback rate is increased or decreased.\\n     * @param ev The event.\\n     */\\n    onratechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the user resets a form.\\n     * @param ev The event.\\n     */\\n    onreset: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onresize: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    /**\\n     * Fires when the user repositions the scroll box in the scroll bar on the object.\\n     * @param ev The event.\\n     */\\n    onscroll: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    onsecuritypolicyviolation: ((this: GlobalEventHandlers, ev: SecurityPolicyViolationEvent) => any) | null;\\n    /**\\n     * Occurs when the seek operation ends.\\n     * @param ev The event.\\n     */\\n    onseeked: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when the current playback position is moved.\\n     * @param ev The event.\\n     */\\n    onseeking: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Fires when the current selection changes.\\n     * @param ev The event.\\n     */\\n    onselect: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\\n    /**\\n     * Occurs when the download has stopped.\\n     * @param ev The event.\\n     */\\n    onstalled: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onsubmit: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs if the load operation has been intentionally halted.\\n     * @param ev The event.\\n     */\\n    onsuspend: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs to indicate the current playback position.\\n     * @param ev The event.\\n     */\\n    ontimeupdate: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    ontoggle: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    ontouchcancel: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontouchend: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontouchmove: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontouchstart: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null;\\n    ontransitioncancel: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    ontransitionend: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    ontransitionrun: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    ontransitionstart: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\\n    /**\\n     * Occurs when the volume is changed, or playback is muted or unmuted.\\n     * @param ev The event.\\n     */\\n    onvolumechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    /**\\n     * Occurs when playback stops because the next frame of a video resource is not available.\\n     * @param ev The event.\\n     */\\n    onwaiting: ((this: GlobalEventHandlers, ev: Event) => any) | null;\\n    onwheel: ((this: GlobalEventHandlers, ev: WheelEvent) => any) | null;\\n    addEventListener<K extends keyof GlobalEventHandlersEventMap>(type: K, listener: (this: GlobalEventHandlers, ev: GlobalEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof GlobalEventHandlersEventMap>(type: K, listener: (this: GlobalEventHandlers, ev: GlobalEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface GlobalFetch {\\n    fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\n}\\n\\ninterface HTMLAllCollection {\\n    /**\\n     * Returns the number of elements in the collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * element = collection(index)\\n     */\\n    item(nameOrIndex?: string): HTMLCollection | Element | null;\\n    /**\\n     * element = collection(name)\\n     */\\n    namedItem(name: string): HTMLCollection | Element | null;\\n    [index: number]: Element;\\n}\\n\\ndeclare var HTMLAllCollection: {\\n    prototype: HTMLAllCollection;\\n    new(): HTMLAllCollection;\\n};\\n\\ninterface HTMLAnchorElement extends HTMLElement, HTMLHyperlinkElementUtils {\\n    /**\\n     * Sets or retrieves the character set used to encode the object.\\n     */\\n    /** @deprecated */\\n    charset: string;\\n    /**\\n     * Sets or retrieves the coordinates of the object.\\n     */\\n    /** @deprecated */\\n    coords: string;\\n    download: string;\\n    /**\\n     * Sets or retrieves the language code of the object.\\n     */\\n    hreflang: string;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    ping: string;\\n    referrerPolicy: string;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    rel: string;\\n    readonly relList: DOMTokenList;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    /** @deprecated */\\n    rev: string;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    /** @deprecated */\\n    shape: string;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    /**\\n     * Retrieves or sets the text of the object as a string.\\n     */\\n    text: string;\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAnchorElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAnchorElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAnchorElement: {\\n    prototype: HTMLAnchorElement;\\n    new(): HTMLAnchorElement;\\n};\\n\\ninterface HTMLAppletElement extends HTMLElement {\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    /** @deprecated */\\n    alt: string;\\n    /**\\n     * Sets or retrieves a character string that can be used to implement your own archive functionality for the object.\\n     */\\n    /** @deprecated */\\n    archive: string;\\n    /** @deprecated */\\n    code: string;\\n    /**\\n     * Sets or retrieves the URL of the component.\\n     */\\n    /** @deprecated */\\n    codeBase: string;\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    /** @deprecated */\\n    height: string;\\n    /** @deprecated */\\n    hspace: number;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /** @deprecated */\\n    object: string;\\n    /** @deprecated */\\n    vspace: number;\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAppletElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAppletElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAppletElement: {\\n    prototype: HTMLAppletElement;\\n    new(): HTMLAppletElement;\\n};\\n\\ninterface HTMLAreaElement extends HTMLElement, HTMLHyperlinkElementUtils {\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    alt: string;\\n    /**\\n     * Sets or retrieves the coordinates of the object.\\n     */\\n    coords: string;\\n    download: string;\\n    /**\\n     * Sets or gets whether clicks in this region cause action.\\n     */\\n    /** @deprecated */\\n    noHref: boolean;\\n    ping: string;\\n    referrerPolicy: string;\\n    rel: string;\\n    readonly relList: DOMTokenList;\\n    /**\\n     * Sets or retrieves the shape of the object.\\n     */\\n    shape: string;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAreaElement: {\\n    prototype: HTMLAreaElement;\\n    new(): HTMLAreaElement;\\n};\\n\\ninterface HTMLAudioElement extends HTMLMediaElement {\\n    addEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLAudioElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLAudioElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLAudioElement: {\\n    prototype: HTMLAudioElement;\\n    new(): HTMLAudioElement;\\n};\\n\\ninterface HTMLBRElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the side on which floating objects are not to be positioned when any IHTMLBlockElement is inserted into the document.\\n     */\\n    /** @deprecated */\\n    clear: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBRElement: {\\n    prototype: HTMLBRElement;\\n    new(): HTMLBRElement;\\n};\\n\\ninterface HTMLBaseElement extends HTMLElement {\\n    /**\\n     * Gets or sets the baseline URL on which relative links are based.\\n     */\\n    href: string;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBaseElement: {\\n    prototype: HTMLBaseElement;\\n    new(): HTMLBaseElement;\\n};\\n\\ninterface HTMLBaseFontElement extends HTMLElement, DOML2DeprecatedColorProperty {\\n    /**\\n     * Sets or retrieves the current typeface family.\\n     */\\n    /** @deprecated */\\n    face: string;\\n    /**\\n     * Sets or retrieves the font size of the object.\\n     */\\n    /** @deprecated */\\n    size: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBaseFontElement: {\\n    prototype: HTMLBaseFontElement;\\n    new(): HTMLBaseFontElement;\\n};\\n\\ninterface HTMLBodyElementEventMap extends HTMLElementEventMap, WindowEventHandlersEventMap {\\n    \"orientationchange\": Event;\\n}\\n\\ninterface HTMLBodyElement extends HTMLElement, WindowEventHandlers {\\n    /** @deprecated */\\n    aLink: string;\\n    /** @deprecated */\\n    background: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    bgProperties: string;\\n    /** @deprecated */\\n    link: string;\\n    /** @deprecated */\\n    noWrap: boolean;\\n    /** @deprecated */\\n    onorientationchange: ((this: HTMLBodyElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    text: string;\\n    /** @deprecated */\\n    vLink: string;\\n    addEventListener<K extends keyof HTMLBodyElementEventMap>(type: K, listener: (this: HTMLBodyElement, ev: HTMLBodyElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLBodyElementEventMap>(type: K, listener: (this: HTMLBodyElement, ev: HTMLBodyElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLBodyElement: {\\n    prototype: HTMLBodyElement;\\n    new(): HTMLBodyElement;\\n};\\n\\ninterface HTMLButtonElement extends HTMLElement {\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Overrides the action attribute (where the data on a form is sent) on the parent form element.\\n     */\\n    formAction: string;\\n    /**\\n     * Used to override the encoding (formEnctype attribute) specified on the form element.\\n     */\\n    formEnctype: string;\\n    /**\\n     * Overrides the submit method attribute previously specified on a form element.\\n     */\\n    formMethod: string;\\n    /**\\n     * Overrides any validation or required attributes on a form or form elements to allow it to be submitted without validation. This can be used to create a \"save draft\"-type submit option.\\n     */\\n    formNoValidate: boolean;\\n    /**\\n     * Overrides the target attribute on a form element.\\n     */\\n    formTarget: string;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Gets the classification and default behavior of the button.\\n     */\\n    type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Sets or retrieves the default or selected value of the control.\\n     */\\n    value: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLButtonElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLButtonElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLButtonElement: {\\n    prototype: HTMLButtonElement;\\n    new(): HTMLButtonElement;\\n};\\n\\ninterface HTMLCanvasElement extends HTMLElement {\\n    /**\\n     * Gets or sets the height of a canvas element on a document.\\n     */\\n    height: number;\\n    /**\\n     * Gets or sets the width of a canvas element on a document.\\n     */\\n    width: number;\\n    /**\\n     * Returns an object that provides methods and properties for drawing and manipulating images and graphics on a canvas element in a document. A context object includes information about colors, line widths, fonts, and other graphic parameters that can be drawn on a canvas.\\n     * @param contextId The identifier (ID) of the type of canvas to create. Internet Explorer 9 and Internet Explorer 10 support only a 2-D context using canvas.getContext(\"2d\"); IE11 Preview also supports 3-D or WebGL context using canvas.getContext(\"experimental-webgl\");\\n     */\\n    getContext(contextId: \"2d\", contextAttributes?: CanvasRenderingContext2DSettings): CanvasRenderingContext2D | null;\\n    getContext(contextId: \"webgl\" | \"experimental-webgl\", contextAttributes?: WebGLContextAttributes): WebGLRenderingContext | null;\\n    getContext(contextId: string, contextAttributes?: {}): CanvasRenderingContext2D | WebGLRenderingContext | null;\\n    toBlob(callback: BlobCallback, type?: string, quality?: any): void;\\n    /**\\n     * Returns the content of the current canvas as an image that you can use as a source for another canvas or an HTML element.\\n     * @param type The standard MIME type for the image format to return. If you do not specify this parameter, the default value is a PNG format image.\\n     */\\n    toDataURL(type?: string, quality?: any): string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLCanvasElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLCanvasElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLCanvasElement: {\\n    prototype: HTMLCanvasElement;\\n    new(): HTMLCanvasElement;\\n};\\n\\ninterface HTMLCollectionBase {\\n    /**\\n     * Sets or retrieves the number of objects in a collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * Retrieves an object from various collections.\\n     */\\n    item(index: number): Element | null;\\n    [index: number]: Element;\\n}\\n\\ninterface HTMLCollection extends HTMLCollectionBase {\\n    /**\\n     * Retrieves a select object or an object from an options collection.\\n     */\\n    namedItem(name: string): Element | null;\\n}\\n\\ndeclare var HTMLCollection: {\\n    prototype: HTMLCollection;\\n    new(): HTMLCollection;\\n};\\n\\ninterface HTMLCollectionOf<T extends Element> extends HTMLCollectionBase {\\n    item(index: number): T | null;\\n    namedItem(name: string): T | null;\\n    [index: number]: T;\\n}\\n\\ninterface HTMLDListElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDListElement: {\\n    prototype: HTMLDListElement;\\n    new(): HTMLDListElement;\\n};\\n\\ninterface HTMLDataElement extends HTMLElement {\\n    value: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDataElement: {\\n    prototype: HTMLDataElement;\\n    new(): HTMLDataElement;\\n};\\n\\ninterface HTMLDataListElement extends HTMLElement {\\n    readonly options: HTMLCollectionOf<HTMLOptionElement>;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDataListElement: {\\n    prototype: HTMLDataListElement;\\n    new(): HTMLDataListElement;\\n};\\n\\ninterface HTMLDetailsElement extends HTMLElement {\\n    open: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDetailsElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDetailsElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDetailsElement: {\\n    prototype: HTMLDetailsElement;\\n    new(): HTMLDetailsElement;\\n};\\n\\ninterface HTMLDialogElement extends HTMLElement {\\n    open: boolean;\\n    returnValue: string;\\n    close(returnValue?: string): void;\\n    show(): void;\\n    showModal(): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDialogElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDialogElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDialogElement: {\\n    prototype: HTMLDialogElement;\\n    new(): HTMLDialogElement;\\n};\\n\\ninterface HTMLDirectoryElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDirectoryElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDirectoryElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDirectoryElement: {\\n    prototype: HTMLDirectoryElement;\\n    new(): HTMLDirectoryElement;\\n};\\n\\ninterface HTMLDivElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDivElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDivElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDivElement: {\\n    prototype: HTMLDivElement;\\n    new(): HTMLDivElement;\\n};\\n\\ninterface HTMLDocument extends Document {\\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: HTMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: HTMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLDocument: {\\n    prototype: HTMLDocument;\\n    new(): HTMLDocument;\\n};\\n\\ninterface HTMLElementEventMap extends ElementEventMap, GlobalEventHandlersEventMap, DocumentAndElementEventHandlersEventMap {\\n}\\n\\ninterface HTMLElement extends Element, GlobalEventHandlers, DocumentAndElementEventHandlers, ElementContentEditable, HTMLOrSVGElement, ElementCSSInlineStyle {\\n    accessKey: string;\\n    readonly accessKeyLabel: string;\\n    autocapitalize: string;\\n    dir: string;\\n    draggable: boolean;\\n    hidden: boolean;\\n    innerText: string;\\n    lang: string;\\n    readonly offsetHeight: number;\\n    readonly offsetLeft: number;\\n    readonly offsetParent: Element | null;\\n    readonly offsetTop: number;\\n    readonly offsetWidth: number;\\n    spellcheck: boolean;\\n    title: string;\\n    translate: boolean;\\n    click(): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLElement: {\\n    prototype: HTMLElement;\\n    new(): HTMLElement;\\n};\\n\\ninterface HTMLEmbedElement extends HTMLElement, GetSVGDocument {\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: string;\\n    hidden: any;\\n    /**\\n     * Gets or sets whether the DLNA PlayTo device is available.\\n     */\\n    msPlayToDisabled: boolean;\\n    /**\\n     * Gets or sets the path to the preferred media source. This enables the Play To target device to stream the media content, which can be DRM protected, from a different location, such as a cloud media server.\\n     */\\n    msPlayToPreferredSourceUri: string;\\n    /**\\n     * Gets or sets the primary DLNA PlayTo device.\\n     */\\n    msPlayToPrimary: boolean;\\n    /**\\n     * Gets the source associated with the media element for use by the PlayToManager.\\n     */\\n    readonly msPlayToSource: any;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /**\\n     * Retrieves the palette used for the embedded document.\\n     */\\n    readonly palette: string;\\n    /**\\n     * Retrieves the URL of the plug-in used to view an embedded document.\\n     */\\n    readonly pluginspage: string;\\n    readonly readyState: string;\\n    /**\\n     * Sets or retrieves a URL to be loaded by the object.\\n     */\\n    src: string;\\n    /**\\n     * Sets or retrieves the height and width units of the embed object.\\n     */\\n    units: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLEmbedElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLEmbedElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLEmbedElement: {\\n    prototype: HTMLEmbedElement;\\n    new(): HTMLEmbedElement;\\n};\\n\\ninterface HTMLFieldSetElement extends HTMLElement {\\n    disabled: boolean;\\n    readonly elements: HTMLCollection;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    name: string;\\n    readonly type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFieldSetElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFieldSetElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFieldSetElement: {\\n    prototype: HTMLFieldSetElement;\\n    new(): HTMLFieldSetElement;\\n};\\n\\ninterface HTMLFontElement extends HTMLElement {\\n    /** @deprecated */\\n    color: string;\\n    /**\\n     * Sets or retrieves the current typeface family.\\n     */\\n    /** @deprecated */\\n    face: string;\\n    /** @deprecated */\\n    size: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFontElement: {\\n    prototype: HTMLFontElement;\\n    new(): HTMLFontElement;\\n};\\n\\ninterface HTMLFormControlsCollection extends HTMLCollectionBase {\\n    /**\\n     * element = collection[name]\\n     */\\n    namedItem(name: string): RadioNodeList | Element | null;\\n}\\n\\ndeclare var HTMLFormControlsCollection: {\\n    prototype: HTMLFormControlsCollection;\\n    new(): HTMLFormControlsCollection;\\n};\\n\\ninterface HTMLFormElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a list of character encodings for input data that must be accepted by the server processing the form.\\n     */\\n    acceptCharset: string;\\n    /**\\n     * Sets or retrieves the URL to which the form content is sent for processing.\\n     */\\n    action: string;\\n    /**\\n     * Specifies whether autocomplete is applied to an editable text field.\\n     */\\n    autocomplete: string;\\n    /**\\n     * Retrieves a collection, in source order, of all controls in a given form.\\n     */\\n    readonly elements: HTMLFormControlsCollection;\\n    /**\\n     * Sets or retrieves the MIME encoding for the form.\\n     */\\n    encoding: string;\\n    /**\\n     * Sets or retrieves the encoding type for the form.\\n     */\\n    enctype: string;\\n    /**\\n     * Sets or retrieves the number of objects in a collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * Sets or retrieves how to send the form data to the server.\\n     */\\n    method: string;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Designates a form that is not validated when submitted.\\n     */\\n    noValidate: boolean;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    target: string;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Fires when the user resets a form.\\n     */\\n    reset(): void;\\n    /**\\n     * Fires when a FORM is about to be submitted.\\n     */\\n    submit(): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFormElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFormElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: Element;\\n    [name: string]: any;\\n}\\n\\ndeclare var HTMLFormElement: {\\n    prototype: HTMLFormElement;\\n    new(): HTMLFormElement;\\n};\\n\\ninterface HTMLFrameElement extends HTMLElement {\\n    /**\\n     * Retrieves the document object of the page or frame.\\n     */\\n    /** @deprecated */\\n    readonly contentDocument: Document | null;\\n    /**\\n     * Retrieves the object of the specified.\\n     */\\n    /** @deprecated */\\n    readonly contentWindow: WindowProxy | null;\\n    /**\\n     * Sets or retrieves whether to display a border for the frame.\\n     */\\n    /** @deprecated */\\n    frameBorder: string;\\n    /**\\n     * Sets or retrieves a URI to a long description of the object.\\n     */\\n    /** @deprecated */\\n    longDesc: string;\\n    /**\\n     * Sets or retrieves the top and bottom margin heights before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginHeight: string;\\n    /**\\n     * Sets or retrieves the left and right margin widths before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginWidth: string;\\n    /**\\n     * Sets or retrieves the frame name.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /**\\n     * Sets or retrieves whether the user can resize the frame.\\n     */\\n    /** @deprecated */\\n    noResize: boolean;\\n    /**\\n     * Sets or retrieves whether the frame can be scrolled.\\n     */\\n    /** @deprecated */\\n    scrolling: string;\\n    /**\\n     * Sets or retrieves a URL to be loaded by the object.\\n     */\\n    /** @deprecated */\\n    src: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFrameElement: {\\n    prototype: HTMLFrameElement;\\n    new(): HTMLFrameElement;\\n};\\n\\ninterface HTMLFrameSetElementEventMap extends HTMLElementEventMap, WindowEventHandlersEventMap {\\n}\\n\\ninterface HTMLFrameSetElement extends HTMLElement, WindowEventHandlers {\\n    /**\\n     * Sets or retrieves the frame widths of the object.\\n     */\\n    /** @deprecated */\\n    cols: string;\\n    /**\\n     * Sets or retrieves the frame heights of the object.\\n     */\\n    /** @deprecated */\\n    rows: string;\\n    addEventListener<K extends keyof HTMLFrameSetElementEventMap>(type: K, listener: (this: HTMLFrameSetElement, ev: HTMLFrameSetElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLFrameSetElementEventMap>(type: K, listener: (this: HTMLFrameSetElement, ev: HTMLFrameSetElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLFrameSetElement: {\\n    prototype: HTMLFrameSetElement;\\n    new(): HTMLFrameSetElement;\\n};\\n\\ninterface HTMLHRElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    color: string;\\n    /**\\n     * Sets or retrieves whether the horizontal rule is drawn with 3-D shading.\\n     */\\n    /** @deprecated */\\n    noShade: boolean;\\n    /** @deprecated */\\n    size: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHRElement: {\\n    prototype: HTMLHRElement;\\n    new(): HTMLHRElement;\\n};\\n\\ninterface HTMLHeadElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHeadElement: {\\n    prototype: HTMLHeadElement;\\n    new(): HTMLHeadElement;\\n};\\n\\ninterface HTMLHeadingElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a value that indicates the table alignment.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadingElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadingElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHeadingElement: {\\n    prototype: HTMLHeadingElement;\\n    new(): HTMLHeadingElement;\\n};\\n\\ninterface HTMLHtmlElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the DTD version that governs the current document.\\n     */\\n    /** @deprecated */\\n    version: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHtmlElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHtmlElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLHtmlElement: {\\n    prototype: HTMLHtmlElement;\\n    new(): HTMLHtmlElement;\\n};\\n\\ninterface HTMLHyperlinkElementUtils {\\n    hash: string;\\n    host: string;\\n    hostname: string;\\n    href: string;\\n    readonly origin: string;\\n    password: string;\\n    pathname: string;\\n    port: string;\\n    protocol: string;\\n    search: string;\\n    username: string;\\n}\\n\\ninterface HTMLIFrameElement extends HTMLElement, GetSVGDocument {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    allowFullscreen: boolean;\\n    allowPaymentRequest: boolean;\\n    /**\\n     * Retrieves the document object of the page or frame.\\n     */\\n    readonly contentDocument: Document | null;\\n    /**\\n     * Retrieves the object of the specified.\\n     */\\n    readonly contentWindow: Window | null;\\n    /**\\n     * Sets or retrieves whether to display a border for the frame.\\n     */\\n    /** @deprecated */\\n    frameBorder: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: string;\\n    /**\\n     * Sets or retrieves a URI to a long description of the object.\\n     */\\n    /** @deprecated */\\n    longDesc: string;\\n    /**\\n     * Sets or retrieves the top and bottom margin heights before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginHeight: string;\\n    /**\\n     * Sets or retrieves the left and right margin widths before displaying the text in a frame.\\n     */\\n    /** @deprecated */\\n    marginWidth: string;\\n    /**\\n     * Sets or retrieves the frame name.\\n     */\\n    name: string;\\n    readonly sandbox: DOMTokenList;\\n    /**\\n     * Sets or retrieves whether the frame can be scrolled.\\n     */\\n    /** @deprecated */\\n    scrolling: string;\\n    /**\\n     * Sets or retrieves a URL to be loaded by the object.\\n     */\\n    src: string;\\n    /**\\n     * Sets or retrives the content of the page that is to contain.\\n     */\\n    srcdoc: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLIFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLIFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLIFrameElement: {\\n    prototype: HTMLIFrameElement;\\n    new(): HTMLIFrameElement;\\n};\\n\\ninterface HTMLImageElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    alt: string;\\n    /**\\n     * Specifies the properties of a border drawn around an object.\\n     */\\n    /** @deprecated */\\n    border: string;\\n    /**\\n     * Retrieves whether the object is fully loaded.\\n     */\\n    readonly complete: boolean;\\n    crossOrigin: string | null;\\n    readonly currentSrc: string;\\n    decoding: \"async\" | \"sync\" | \"auto\";\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: number;\\n    /**\\n     * Sets or retrieves the width of the border to draw around the object.\\n     */\\n    /** @deprecated */\\n    hspace: number;\\n    /**\\n     * Sets or retrieves whether the image is a server-side image map.\\n     */\\n    isMap: boolean;\\n    /**\\n     * Sets or retrieves a Uniform Resource Identifier (URI) to a long description of the object.\\n     */\\n    /** @deprecated */\\n    longDesc: string;\\n    /** @deprecated */\\n    lowsrc: string;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    /** @deprecated */\\n    name: string;\\n    /**\\n     * The original height of the image resource before sizing.\\n     */\\n    readonly naturalHeight: number;\\n    /**\\n     * The original width of the image resource before sizing.\\n     */\\n    readonly naturalWidth: number;\\n    referrerPolicy: string;\\n    sizes: string;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    srcset: string;\\n    /**\\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\\n     */\\n    useMap: string;\\n    /**\\n     * Sets or retrieves the vertical margin for the object.\\n     */\\n    /** @deprecated */\\n    vspace: number;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: number;\\n    readonly x: number;\\n    readonly y: number;\\n    decode(): Promise<void>;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLImageElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLImageElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLImageElement: {\\n    prototype: HTMLImageElement;\\n    new(): HTMLImageElement;\\n};\\n\\ninterface HTMLInputElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a comma-separated list of content types.\\n     */\\n    accept: string;\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a text alternative to the graphic.\\n     */\\n    alt: string;\\n    /**\\n     * Specifies whether autocomplete is applied to an editable text field.\\n     */\\n    autocomplete: string;\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    /**\\n     * Sets or retrieves the state of the check box or radio button.\\n     */\\n    checked: boolean;\\n    /**\\n     * Sets or retrieves the state of the check box or radio button.\\n     */\\n    defaultChecked: boolean;\\n    /**\\n     * Sets or retrieves the initial contents of the object.\\n     */\\n    defaultValue: string;\\n    dirName: string;\\n    disabled: boolean;\\n    /**\\n     * Returns a FileList object on a file type input object.\\n     */\\n    files: FileList | null;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Overrides the action attribute (where the data on a form is sent) on the parent form element.\\n     */\\n    formAction: string;\\n    /**\\n     * Used to override the encoding (formEnctype attribute) specified on the form element.\\n     */\\n    formEnctype: string;\\n    /**\\n     * Overrides the submit method attribute previously specified on a form element.\\n     */\\n    formMethod: string;\\n    /**\\n     * Overrides any validation or required attributes on a form or form elements to allow it to be submitted without validation. This can be used to create a \"save draft\"-type submit option.\\n     */\\n    formNoValidate: boolean;\\n    /**\\n     * Overrides the target attribute on a form element.\\n     */\\n    formTarget: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: number;\\n    indeterminate: boolean;\\n    readonly labels: NodeListOf<HTMLLabelElement> | null;\\n    /**\\n     * Specifies the ID of a pre-defined datalist of options for an input element.\\n     */\\n    readonly list: HTMLElement | null;\\n    /**\\n     * Defines the maximum acceptable value for an input element with type=\"number\".When used with the min and step attributes, lets you control the range and increment (such as only even numbers) that the user can enter into an input field.\\n     */\\n    max: string;\\n    /**\\n     * Sets or retrieves the maximum number of characters that the user can enter in a text control.\\n     */\\n    maxLength: number;\\n    /**\\n     * Defines the minimum acceptable value for an input element with type=\"number\". When used with the max and step attributes, lets you control the range and increment (such as even numbers only) that the user can enter into an input field.\\n     */\\n    min: string;\\n    minLength: number;\\n    /**\\n     * Sets or retrieves the Boolean value indicating whether multiple items can be selected from a list.\\n     */\\n    multiple: boolean;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Gets or sets a string containing a regular expression that the user\\'s input must match.\\n     */\\n    pattern: string;\\n    /**\\n     * Gets or sets a text string that is displayed in an input field as a hint or prompt to users as the format or type of information they need to enter.The text appears in an input field until the user puts focus on the field.\\n     */\\n    placeholder: string;\\n    readOnly: boolean;\\n    /**\\n     * When present, marks an element that can\\'t be submitted without a value.\\n     */\\n    required: boolean;\\n    selectionDirection: string | null;\\n    /**\\n     * Gets or sets the end position or offset of a text selection.\\n     */\\n    selectionEnd: number | null;\\n    /**\\n     * Gets or sets the starting position or offset of a text selection.\\n     */\\n    selectionStart: number | null;\\n    size: number;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    /**\\n     * Defines an increment or jump between values that you want to allow the user to enter. When used with the max and min attributes, lets you control the range and increment (for example, allow only even numbers) that the user can enter into an input field.\\n     */\\n    step: string;\\n    /**\\n     * Returns the content type of the object.\\n     */\\n    type: string;\\n    /**\\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\\n     */\\n    /** @deprecated */\\n    useMap: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Returns the value of the data at the cursor\\'s current position.\\n     */\\n    value: string;\\n    valueAsDate: any;\\n    /**\\n     * Returns the input field value as a number.\\n     */\\n    valueAsNumber: number;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: number;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Makes the selection equal to the current object.\\n     */\\n    select(): void;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    setRangeText(replacement: string): void;\\n    setRangeText(replacement: string, start: number, end: number, selectionMode?: SelectionMode): void;\\n    /**\\n     * Sets the start and end positions of a selection in a text field.\\n     * @param start The offset into the text field for the start of the selection.\\n     * @param end The offset into the text field for the end of the selection.\\n     * @param direction The direction in which the selection is performed.\\n     */\\n    setSelectionRange(start: number, end: number, direction?: \"forward\" | \"backward\" | \"none\"): void;\\n    /**\\n     * Decrements a range input control\\'s value by the value given by the Step attribute. If the optional parameter is used, it will decrement the input control\\'s step value multiplied by the parameter\\'s value.\\n     * @param n Value to decrement the value by.\\n     */\\n    stepDown(n?: number): void;\\n    /**\\n     * Increments a range input control\\'s value by the value given by the Step attribute. If the optional parameter is used, will increment the input control\\'s value by that value.\\n     * @param n Value to increment the value by.\\n     */\\n    stepUp(n?: number): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLInputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLInputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLInputElement: {\\n    prototype: HTMLInputElement;\\n    new(): HTMLInputElement;\\n};\\n\\ninterface HTMLLIElement extends HTMLElement {\\n    /** @deprecated */\\n    type: string;\\n    /**\\n     * Sets or retrieves the value of a list item.\\n     */\\n    value: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLIElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLIElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLIElement: {\\n    prototype: HTMLLIElement;\\n    new(): HTMLLIElement;\\n};\\n\\ninterface HTMLLabelElement extends HTMLElement {\\n    readonly control: HTMLElement | null;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the object to which the given label object is assigned.\\n     */\\n    htmlFor: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLabelElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLabelElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLabelElement: {\\n    prototype: HTMLLabelElement;\\n    new(): HTMLLabelElement;\\n};\\n\\ninterface HTMLLegendElement extends HTMLElement {\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLegendElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLegendElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLegendElement: {\\n    prototype: HTMLLegendElement;\\n    new(): HTMLLegendElement;\\n};\\n\\ninterface HTMLLinkElement extends HTMLElement, LinkStyle {\\n    as: string;\\n    /**\\n     * Sets or retrieves the character set used to encode the object.\\n     */\\n    /** @deprecated */\\n    charset: string;\\n    crossOrigin: string | null;\\n    disabled: boolean;\\n    /**\\n     * Sets or retrieves a destination URL or an anchor point.\\n     */\\n    href: string;\\n    /**\\n     * Sets or retrieves the language code of the object.\\n     */\\n    hreflang: string;\\n    integrity: string;\\n    /**\\n     * Sets or retrieves the media type.\\n     */\\n    media: string;\\n    referrerPolicy: string;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    rel: string;\\n    readonly relList: DOMTokenList;\\n    /**\\n     * Sets or retrieves the relationship between the object and the destination of the link.\\n     */\\n    /** @deprecated */\\n    rev: string;\\n    readonly sizes: DOMTokenList;\\n    /**\\n     * Sets or retrieves the window or frame at which to target content.\\n     */\\n    /** @deprecated */\\n    target: string;\\n    /**\\n     * Sets or retrieves the MIME type of the object.\\n     */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLinkElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLinkElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLLinkElement: {\\n    prototype: HTMLLinkElement;\\n    new(): HTMLLinkElement;\\n};\\n\\ninterface HTMLMainElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMainElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMainElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMainElement: {\\n    prototype: HTMLMainElement;\\n    new(): HTMLMainElement;\\n};\\n\\ninterface HTMLMapElement extends HTMLElement {\\n    /**\\n     * Retrieves a collection of the area objects defined for the given map object.\\n     */\\n    readonly areas: HTMLCollection;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMapElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMapElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMapElement: {\\n    prototype: HTMLMapElement;\\n    new(): HTMLMapElement;\\n};\\n\\ninterface HTMLMarqueeElementEventMap extends HTMLElementEventMap {\\n    \"bounce\": Event;\\n    \"finish\": Event;\\n    \"start\": Event;\\n}\\n\\ninterface HTMLMarqueeElement extends HTMLElement {\\n    /** @deprecated */\\n    behavior: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /** @deprecated */\\n    direction: string;\\n    /** @deprecated */\\n    height: string;\\n    /** @deprecated */\\n    hspace: number;\\n    /** @deprecated */\\n    loop: number;\\n    /** @deprecated */\\n    onbounce: ((this: HTMLMarqueeElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onfinish: ((this: HTMLMarqueeElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onstart: ((this: HTMLMarqueeElement, ev: Event) => any) | null;\\n    /** @deprecated */\\n    scrollAmount: number;\\n    /** @deprecated */\\n    scrollDelay: number;\\n    /** @deprecated */\\n    trueSpeed: boolean;\\n    /** @deprecated */\\n    vspace: number;\\n    /** @deprecated */\\n    width: string;\\n    /** @deprecated */\\n    start(): void;\\n    /** @deprecated */\\n    stop(): void;\\n    addEventListener<K extends keyof HTMLMarqueeElementEventMap>(type: K, listener: (this: HTMLMarqueeElement, ev: HTMLMarqueeElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLMarqueeElementEventMap>(type: K, listener: (this: HTMLMarqueeElement, ev: HTMLMarqueeElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMarqueeElement: {\\n    prototype: HTMLMarqueeElement;\\n    new(): HTMLMarqueeElement;\\n};\\n\\ninterface HTMLMediaElementEventMap extends HTMLElementEventMap {\\n    \"encrypted\": MediaEncryptedEvent;\\n    \"msneedkey\": Event;\\n}\\n\\ninterface HTMLMediaElement extends HTMLElement {\\n    /**\\n     * Returns an AudioTrackList object with the audio tracks for a given video element.\\n     */\\n    readonly audioTracks: AudioTrackList;\\n    /**\\n     * Gets or sets a value that indicates whether to start playing the media automatically.\\n     */\\n    autoplay: boolean;\\n    /**\\n     * Gets a collection of buffered time ranges.\\n     */\\n    readonly buffered: TimeRanges;\\n    /**\\n     * Gets or sets a flag that indicates whether the client provides a set of controls for the media (in case the developer does not include controls for the player).\\n     */\\n    controls: boolean;\\n    crossOrigin: string | null;\\n    /**\\n     * Gets the address or URL of the current media resource that is selected by IHTMLMediaElement.\\n     */\\n    readonly currentSrc: string;\\n    /**\\n     * Gets or sets the current playback position, in seconds.\\n     */\\n    currentTime: number;\\n    defaultMuted: boolean;\\n    /**\\n     * Gets or sets the default playback rate when the user is not using fast forward or reverse for a video or audio resource.\\n     */\\n    defaultPlaybackRate: number;\\n    /**\\n     * Returns the duration in seconds of the current media resource. A NaN value is returned if duration is not available, or Infinity if the media resource is streaming.\\n     */\\n    readonly duration: number;\\n    /**\\n     * Gets information about whether the playback has ended or not.\\n     */\\n    readonly ended: boolean;\\n    /**\\n     * Returns an object representing the current error state of the audio or video element.\\n     */\\n    readonly error: MediaError | null;\\n    /**\\n     * Gets or sets a flag to specify whether playback should restart after it completes.\\n     */\\n    loop: boolean;\\n    readonly mediaKeys: MediaKeys | null;\\n    /**\\n     * Specifies the purpose of the audio or video media, such as background audio or alerts.\\n     */\\n    msAudioCategory: string;\\n    /**\\n     * Specifies the output device id that the audio will be sent to.\\n     */\\n    msAudioDeviceType: string;\\n    readonly msGraphicsTrustStatus: MSGraphicsTrust;\\n    /**\\n     * Gets the MSMediaKeys object, which is used for decrypting media data, that is associated with this media element.\\n     */\\n    /** @deprecated */\\n    readonly msKeys: MSMediaKeys;\\n    /**\\n     * Gets or sets whether the DLNA PlayTo device is available.\\n     */\\n    msPlayToDisabled: boolean;\\n    /**\\n     * Gets or sets the path to the preferred media source. This enables the Play To target device to stream the media content, which can be DRM protected, from a different location, such as a cloud media server.\\n     */\\n    msPlayToPreferredSourceUri: string;\\n    /**\\n     * Gets or sets the primary DLNA PlayTo device.\\n     */\\n    msPlayToPrimary: boolean;\\n    /**\\n     * Gets the source associated with the media element for use by the PlayToManager.\\n     */\\n    readonly msPlayToSource: any;\\n    /**\\n     * Specifies whether or not to enable low-latency playback on the media element.\\n     */\\n    msRealTime: boolean;\\n    /**\\n     * Gets or sets a flag that indicates whether the audio (either audio or the audio track on video media) is muted.\\n     */\\n    muted: boolean;\\n    /**\\n     * Gets the current network activity for the element.\\n     */\\n    readonly networkState: number;\\n    onencrypted: ((this: HTMLMediaElement, ev: MediaEncryptedEvent) => any) | null;\\n    /** @deprecated */\\n    onmsneedkey: ((this: HTMLMediaElement, ev: Event) => any) | null;\\n    /**\\n     * Gets a flag that specifies whether playback is paused.\\n     */\\n    readonly paused: boolean;\\n    /**\\n     * Gets or sets the current rate of speed for the media resource to play. This speed is expressed as a multiple of the normal speed of the media resource.\\n     */\\n    playbackRate: number;\\n    /**\\n     * Gets TimeRanges for the current media resource that has been played.\\n     */\\n    readonly played: TimeRanges;\\n    /**\\n     * Gets or sets the current playback position, in seconds.\\n     */\\n    preload: string;\\n    readonly readyState: number;\\n    /**\\n     * Returns a TimeRanges object that represents the ranges of the current media resource that can be seeked.\\n     */\\n    readonly seekable: TimeRanges;\\n    /**\\n     * Gets a flag that indicates whether the the client is currently moving to a new playback position in the media resource.\\n     */\\n    readonly seeking: boolean;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    srcObject: MediaStream | MediaSource | Blob | null;\\n    readonly textTracks: TextTrackList;\\n    readonly videoTracks: VideoTrackList;\\n    /**\\n     * Gets or sets the volume level for audio portions of the media element.\\n     */\\n    volume: number;\\n    addTextTrack(kind: TextTrackKind, label?: string, language?: string): TextTrack;\\n    /**\\n     * Returns a string that specifies whether the client can play a given media resource type.\\n     */\\n    canPlayType(type: string): CanPlayTypeResult;\\n    /**\\n     * Resets the audio or video object and loads a new media resource.\\n     */\\n    load(): void;\\n    /**\\n     * Clears all effects from the media pipeline.\\n     */\\n    msClearEffects(): void;\\n    msGetAsCastingSource(): any;\\n    /**\\n     * Inserts the specified audio effect into media pipeline.\\n     */\\n    msInsertAudioEffect(activatableClassId: string, effectRequired: boolean, config?: any): void;\\n    /** @deprecated */\\n    msSetMediaKeys(mediaKeys: MSMediaKeys): void;\\n    /**\\n     * Specifies the media protection manager for a given media pipeline.\\n     */\\n    msSetMediaProtectionManager(mediaProtectionManager?: any): void;\\n    /**\\n     * Pauses the current playback and sets paused to TRUE. This can be used to test whether the media is playing or paused. You can also use the pause or play events to tell whether the media is playing or not.\\n     */\\n    pause(): void;\\n    /**\\n     * Loads and starts playback of a media resource.\\n     */\\n    play(): Promise<void>;\\n    setMediaKeys(mediaKeys: MediaKeys | null): Promise<void>;\\n    readonly HAVE_CURRENT_DATA: number;\\n    readonly HAVE_ENOUGH_DATA: number;\\n    readonly HAVE_FUTURE_DATA: number;\\n    readonly HAVE_METADATA: number;\\n    readonly HAVE_NOTHING: number;\\n    readonly NETWORK_EMPTY: number;\\n    readonly NETWORK_IDLE: number;\\n    readonly NETWORK_LOADING: number;\\n    readonly NETWORK_NO_SOURCE: number;\\n    addEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLMediaElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLMediaElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMediaElement: {\\n    prototype: HTMLMediaElement;\\n    new(): HTMLMediaElement;\\n    readonly HAVE_CURRENT_DATA: number;\\n    readonly HAVE_ENOUGH_DATA: number;\\n    readonly HAVE_FUTURE_DATA: number;\\n    readonly HAVE_METADATA: number;\\n    readonly HAVE_NOTHING: number;\\n    readonly NETWORK_EMPTY: number;\\n    readonly NETWORK_IDLE: number;\\n    readonly NETWORK_LOADING: number;\\n    readonly NETWORK_NO_SOURCE: number;\\n};\\n\\ninterface HTMLMenuElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMenuElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMenuElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMenuElement: {\\n    prototype: HTMLMenuElement;\\n    new(): HTMLMenuElement;\\n};\\n\\ninterface HTMLMetaElement extends HTMLElement {\\n    /**\\n     * Gets or sets meta-information to associate with httpEquiv or name.\\n     */\\n    content: string;\\n    /**\\n     * Gets or sets information used to bind the value of a content attribute of a meta element to an HTTP response header.\\n     */\\n    httpEquiv: string;\\n    /**\\n     * Sets or retrieves the value specified in the content attribute of the meta object.\\n     */\\n    name: string;\\n    /**\\n     * Sets or retrieves a scheme to be used in interpreting the value of a property specified for the object.\\n     */\\n    /** @deprecated */\\n    scheme: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMetaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMetaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMetaElement: {\\n    prototype: HTMLMetaElement;\\n    new(): HTMLMetaElement;\\n};\\n\\ninterface HTMLMeterElement extends HTMLElement {\\n    high: number;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    low: number;\\n    max: number;\\n    min: number;\\n    optimum: number;\\n    value: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMeterElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMeterElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLMeterElement: {\\n    prototype: HTMLMeterElement;\\n    new(): HTMLMeterElement;\\n};\\n\\ninterface HTMLModElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves reference information about the object.\\n     */\\n    cite: string;\\n    /**\\n     * Sets or retrieves the date and time of a modification to the object.\\n     */\\n    dateTime: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLModElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLModElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLModElement: {\\n    prototype: HTMLModElement;\\n    new(): HTMLModElement;\\n};\\n\\ninterface HTMLOListElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    reversed: boolean;\\n    /**\\n     * The starting number.\\n     */\\n    start: number;\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOListElement: {\\n    prototype: HTMLOListElement;\\n    new(): HTMLOListElement;\\n};\\n\\ninterface HTMLObjectElement extends HTMLElement, GetSVGDocument {\\n    /**\\n     * Retrieves a string of the URL where the object tag can be found. This is often the href of the document that the object is in, or the value set by a base element.\\n     */\\n    readonly BaseHref: string;\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a character string that can be used to implement your own archive functionality for the object.\\n     */\\n    /** @deprecated */\\n    archive: string;\\n    /** @deprecated */\\n    border: string;\\n    /**\\n     * Sets or retrieves the URL of the file containing the compiled Java class.\\n     */\\n    /** @deprecated */\\n    code: string;\\n    /**\\n     * Sets or retrieves the URL of the component.\\n     */\\n    /** @deprecated */\\n    codeBase: string;\\n    /**\\n     * Sets or retrieves the Internet media type for the code associated with the object.\\n     */\\n    /** @deprecated */\\n    codeType: string;\\n    /**\\n     * Retrieves the document object of the page or frame.\\n     */\\n    readonly contentDocument: Document | null;\\n    /**\\n     * Sets or retrieves the URL that references the data of the object.\\n     */\\n    data: string;\\n    /** @deprecated */\\n    declare: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    height: string;\\n    /** @deprecated */\\n    hspace: number;\\n    /**\\n     * Gets or sets whether the DLNA PlayTo device is available.\\n     */\\n    msPlayToDisabled: boolean;\\n    /**\\n     * Gets or sets the path to the preferred media source. This enables the Play To target device to stream the media content, which can be DRM protected, from a different location, such as a cloud media server.\\n     */\\n    msPlayToPreferredSourceUri: string;\\n    /**\\n     * Gets or sets the primary DLNA PlayTo device.\\n     */\\n    msPlayToPrimary: boolean;\\n    /**\\n     * Gets the source associated with the media element for use by the PlayToManager.\\n     */\\n    readonly msPlayToSource: any;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    readonly readyState: number;\\n    /**\\n     * Sets or retrieves a message to be displayed while an object is loading.\\n     */\\n    /** @deprecated */\\n    standby: string;\\n    /**\\n     * Sets or retrieves the MIME type of the object.\\n     */\\n    type: string;\\n    typemustmatch: boolean;\\n    /**\\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\\n     */\\n    useMap: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /** @deprecated */\\n    vspace: number;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    width: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLObjectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLObjectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLObjectElement: {\\n    prototype: HTMLObjectElement;\\n    new(): HTMLObjectElement;\\n};\\n\\ninterface HTMLOptGroupElement extends HTMLElement {\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves a value that you can use to implement your own label functionality for the object.\\n     */\\n    label: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptGroupElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptGroupElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOptGroupElement: {\\n    prototype: HTMLOptGroupElement;\\n    new(): HTMLOptGroupElement;\\n};\\n\\ninterface HTMLOptionElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the status of an option.\\n     */\\n    defaultSelected: boolean;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    /**\\n     * Sets or retrieves the ordinal position of an option in a list box.\\n     */\\n    readonly index: number;\\n    /**\\n     * Sets or retrieves a value that you can use to implement your own label functionality for the object.\\n     */\\n    label: string;\\n    /**\\n     * Sets or retrieves whether the option in the list box is the default item.\\n     */\\n    selected: boolean;\\n    /**\\n     * Sets or retrieves the text string specified by the option tag.\\n     */\\n    text: string;\\n    /**\\n     * Sets or retrieves the value which is returned to the server when the form control is submitted.\\n     */\\n    value: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOptionElement: {\\n    prototype: HTMLOptionElement;\\n    new(): HTMLOptionElement;\\n};\\n\\ninterface HTMLOptionsCollection extends HTMLCollectionOf<HTMLOptionElement> {\\n    /**\\n     * Returns the number of elements in the collection.\\n     * When set to a smaller number, truncates the number of option elements in the corresponding container.\\n     * When set to a greater number, adds new blank option elements to that container.\\n     */\\n    length: number;\\n    /**\\n     * Returns the index of the first selected item, if any, or −1 if there is no selected\\n     * item.\\n     * Can be set, to change the selection.\\n     */\\n    selectedIndex: number;\\n    /**\\n     * Inserts element before the node given by before.\\n     * The before argument can be a number, in which case element is inserted before the item with that number, or an element from the\\n     * collection, in which case element is inserted before that element.\\n     * If before is omitted, null, or a number out of range, then element will be added at the end of the list.\\n     * This method will throw a \"HierarchyRequestError\" DOMException if\\n     * element is an ancestor of the element into which it is to be inserted.\\n     */\\n    add(element: HTMLOptionElement | HTMLOptGroupElement, before?: HTMLElement | number | null): void;\\n    /**\\n     * Removes the item with index index from the collection.\\n     */\\n    remove(index: number): void;\\n}\\n\\ndeclare var HTMLOptionsCollection: {\\n    prototype: HTMLOptionsCollection;\\n    new(): HTMLOptionsCollection;\\n};\\n\\ninterface HTMLOrSVGElement {\\n    readonly dataset: DOMStringMap;\\n    nonce: string;\\n    tabIndex: number;\\n    blur(): void;\\n    focus(options?: FocusOptions): void;\\n}\\n\\ninterface HTMLOutputElement extends HTMLElement {\\n    defaultValue: string;\\n    readonly form: HTMLFormElement | null;\\n    readonly htmlFor: DOMTokenList;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    name: string;\\n    readonly type: string;\\n    readonly validationMessage: string;\\n    readonly validity: ValidityState;\\n    value: string;\\n    readonly willValidate: boolean;\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOutputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOutputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLOutputElement: {\\n    prototype: HTMLOutputElement;\\n    new(): HTMLOutputElement;\\n};\\n\\ninterface HTMLParagraphElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParagraphElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParagraphElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLParagraphElement: {\\n    prototype: HTMLParagraphElement;\\n    new(): HTMLParagraphElement;\\n};\\n\\ninterface HTMLParamElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the name of an input parameter for an element.\\n     */\\n    name: string;\\n    /**\\n     * Sets or retrieves the content type of the resource designated by the value attribute.\\n     */\\n    /** @deprecated */\\n    type: string;\\n    /**\\n     * Sets or retrieves the value of an input parameter for an element.\\n     */\\n    value: string;\\n    /**\\n     * Sets or retrieves the data type of the value attribute.\\n     */\\n    /** @deprecated */\\n    valueType: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParamElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParamElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLParamElement: {\\n    prototype: HTMLParamElement;\\n    new(): HTMLParamElement;\\n};\\n\\ninterface HTMLPictureElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPictureElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPictureElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLPictureElement: {\\n    prototype: HTMLPictureElement;\\n    new(): HTMLPictureElement;\\n};\\n\\ninterface HTMLPreElement extends HTMLElement {\\n    /**\\n     * Sets or gets a value that you can use to implement your own width functionality for the object.\\n     */\\n    /** @deprecated */\\n    width: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPreElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPreElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLPreElement: {\\n    prototype: HTMLPreElement;\\n    new(): HTMLPreElement;\\n};\\n\\ninterface HTMLProgressElement extends HTMLElement {\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Defines the maximum, or \"done\" value for a progress element.\\n     */\\n    max: number;\\n    /**\\n     * Returns the quotient of value/max when the value attribute is set (determinate progress bar), or -1 when the value attribute is missing (indeterminate progress bar).\\n     */\\n    readonly position: number;\\n    /**\\n     * Sets or gets the current value of a progress element. The value must be a non-negative number between 0 and the max value.\\n     */\\n    value: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLProgressElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLProgressElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLProgressElement: {\\n    prototype: HTMLProgressElement;\\n    new(): HTMLProgressElement;\\n};\\n\\ninterface HTMLQuoteElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves reference information about the object.\\n     */\\n    cite: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLQuoteElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLQuoteElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLQuoteElement: {\\n    prototype: HTMLQuoteElement;\\n    new(): HTMLQuoteElement;\\n};\\n\\ninterface HTMLScriptElement extends HTMLElement {\\n    async: boolean;\\n    /**\\n     * Sets or retrieves the character set used to encode the object.\\n     */\\n    /** @deprecated */\\n    charset: string;\\n    crossOrigin: string | null;\\n    /**\\n     * Sets or retrieves the status of the script.\\n     */\\n    defer: boolean;\\n    /**\\n     * Sets or retrieves the event for which the script is written.\\n     */\\n    /** @deprecated */\\n    event: string;\\n    /**\\n     * Sets or retrieves the object that is bound to the event script.\\n     */\\n    /** @deprecated */\\n    htmlFor: string;\\n    integrity: string;\\n    noModule: boolean;\\n    referrerPolicy: string;\\n    /**\\n     * Retrieves the URL to an external file that contains the source code or data.\\n     */\\n    src: string;\\n    /**\\n     * Retrieves or sets the text of the object as a string.\\n     */\\n    text: string;\\n    /**\\n     * Sets or retrieves the MIME type for the associated scripting engine.\\n     */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLScriptElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLScriptElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLScriptElement: {\\n    prototype: HTMLScriptElement;\\n    new(): HTMLScriptElement;\\n};\\n\\ninterface HTMLSelectElement extends HTMLElement {\\n    autocomplete: string;\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Sets or retrieves the number of objects in a collection.\\n     */\\n    length: number;\\n    /**\\n     * Sets or retrieves the Boolean value indicating whether multiple items can be selected from a list.\\n     */\\n    multiple: boolean;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    readonly options: HTMLOptionsCollection;\\n    /**\\n     * When present, marks an element that can\\'t be submitted without a value.\\n     */\\n    required: boolean;\\n    /**\\n     * Sets or retrieves the index of the selected option in a select object.\\n     */\\n    selectedIndex: number;\\n    readonly selectedOptions: HTMLCollectionOf<HTMLOptionElement>;\\n    /**\\n     * Sets or retrieves the number of rows in the list box.\\n     */\\n    size: number;\\n    /**\\n     * Retrieves the type of select control based on the value of the MULTIPLE attribute.\\n     */\\n    readonly type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Sets or retrieves the value which is returned to the server when the form control is submitted.\\n     */\\n    value: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Adds an element to the areas, controlRange, or options collection.\\n     * @param element Variant of type Number that specifies the index position in the collection where the element is placed. If no value is given, the method places the element at the end of the collection.\\n     * @param before Variant of type Object that specifies an element to insert before, or null to append the object to the collection.\\n     */\\n    add(element: HTMLOptionElement | HTMLOptGroupElement, before?: HTMLElement | number | null): void;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    /**\\n     * Retrieves a select object or an object from an options collection.\\n     * @param name Variant of type Number or String that specifies the object or collection to retrieve. If this parameter is an integer, it is the zero-based index of the object. If this parameter is a string, all objects with matching name or id properties are retrieved, and a collection is returned if more than one match is made.\\n     * @param index Variant of type Number that specifies the zero-based index of the object to retrieve when a collection is returned.\\n     */\\n    item(index: number): Element | null;\\n    /**\\n     * Retrieves a select object or an object from an options collection.\\n     * @param namedItem A String that specifies the name or id property of the object to retrieve. A collection is returned if more than one match is made.\\n     */\\n    namedItem(name: string): HTMLOptionElement | null;\\n    /**\\n     * Removes an element from the collection.\\n     * @param index Number that specifies the zero-based index of the element to remove from the collection.\\n     */\\n    remove(): void;\\n    remove(index: number): void;\\n    reportValidity(): boolean;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSelectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSelectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [name: number]: HTMLOptionElement | HTMLOptGroupElement;\\n}\\n\\ndeclare var HTMLSelectElement: {\\n    prototype: HTMLSelectElement;\\n    new(): HTMLSelectElement;\\n};\\n\\ninterface HTMLSlotElement extends HTMLElement {\\n    name: string;\\n    assignedElements(options?: AssignedNodesOptions): Element[];\\n    assignedNodes(options?: AssignedNodesOptions): Node[];\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSlotElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSlotElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLSlotElement: {\\n    prototype: HTMLSlotElement;\\n    new(): HTMLSlotElement;\\n};\\n\\ninterface HTMLSourceElement extends HTMLElement {\\n    /**\\n     * Gets or sets the intended media type of the media source.\\n     */\\n    media: string;\\n    sizes: string;\\n    /**\\n     * The address or URL of the a media resource that is to be considered.\\n     */\\n    src: string;\\n    srcset: string;\\n    /**\\n     * Gets or sets the MIME type of a media resource.\\n     */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSourceElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSourceElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLSourceElement: {\\n    prototype: HTMLSourceElement;\\n    new(): HTMLSourceElement;\\n};\\n\\ninterface HTMLSpanElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSpanElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSpanElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLSpanElement: {\\n    prototype: HTMLSpanElement;\\n    new(): HTMLSpanElement;\\n};\\n\\ninterface HTMLStyleElement extends HTMLElement, LinkStyle {\\n    /**\\n     * Sets or retrieves the media type.\\n     */\\n    media: string;\\n    /**\\n     * Retrieves the CSS language in which the style sheet is written.\\n     */\\n    /** @deprecated */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLStyleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLStyleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLStyleElement: {\\n    prototype: HTMLStyleElement;\\n    new(): HTMLStyleElement;\\n};\\n\\ninterface HTMLTableCaptionElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the alignment of the caption or legend.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCaptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCaptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableCaptionElement: {\\n    prototype: HTMLTableCaptionElement;\\n    new(): HTMLTableCaptionElement;\\n};\\n\\ninterface HTMLTableCellElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves abbreviated text for the object.\\n     */\\n    abbr: string;\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /**\\n     * Sets or retrieves a comma-delimited list of conceptual categories associated with the object.\\n     */\\n    /** @deprecated */\\n    axis: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Retrieves the position of the object in the cells collection of a row.\\n     */\\n    readonly cellIndex: number;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Sets or retrieves the number columns in the table that the object should span.\\n     */\\n    colSpan: number;\\n    /**\\n     * Sets or retrieves a list of header cells that provide information for the object.\\n     */\\n    headers: string;\\n    /**\\n     * Sets or retrieves the height of the object.\\n     */\\n    /** @deprecated */\\n    height: string;\\n    /**\\n     * Sets or retrieves whether the browser automatically performs wordwrap.\\n     */\\n    /** @deprecated */\\n    noWrap: boolean;\\n    /**\\n     * Sets or retrieves how many rows in a table the cell should span.\\n     */\\n    rowSpan: number;\\n    /**\\n     * Sets or retrieves the group of cells in a table to which the object\\'s information applies.\\n     */\\n    scope: string;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableCellElement: {\\n    prototype: HTMLTableCellElement;\\n    new(): HTMLTableCellElement;\\n};\\n\\ninterface HTMLTableColElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves the alignment of the object relative to the display or table.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Sets or retrieves the number of columns in the group.\\n     */\\n    span: number;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableColElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableColElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableColElement: {\\n    prototype: HTMLTableColElement;\\n    new(): HTMLTableColElement;\\n};\\n\\ninterface HTMLTableDataCellElement extends HTMLTableCellElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableDataCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableDataCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableDataCellElement: {\\n    prototype: HTMLTableDataCellElement;\\n    new(): HTMLTableDataCellElement;\\n};\\n\\ninterface HTMLTableElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a value that indicates the table alignment.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Sets or retrieves the width of the border to draw around the object.\\n     */\\n    /** @deprecated */\\n    border: string;\\n    /**\\n     * Retrieves the caption object of a table.\\n     */\\n    caption: HTMLTableCaptionElement | null;\\n    /**\\n     * Sets or retrieves the amount of space between the border of the cell and the content of the cell.\\n     */\\n    /** @deprecated */\\n    cellPadding: string;\\n    /**\\n     * Sets or retrieves the amount of space between cells in a table.\\n     */\\n    /** @deprecated */\\n    cellSpacing: string;\\n    /**\\n     * Sets or retrieves the way the border frame around the table is displayed.\\n     */\\n    /** @deprecated */\\n    frame: string;\\n    /**\\n     * Sets or retrieves the number of horizontal rows contained in the object.\\n     */\\n    readonly rows: HTMLCollectionOf<HTMLTableRowElement>;\\n    /**\\n     * Sets or retrieves which dividing lines (inner borders) are displayed.\\n     */\\n    /** @deprecated */\\n    rules: string;\\n    /**\\n     * Sets or retrieves a description and/or structure of the object.\\n     */\\n    /** @deprecated */\\n    summary: string;\\n    /**\\n     * Retrieves a collection of all tBody objects in the table. Objects in this collection are in source order.\\n     */\\n    readonly tBodies: HTMLCollectionOf<HTMLTableSectionElement>;\\n    /**\\n     * Retrieves the tFoot object of the table.\\n     */\\n    tFoot: HTMLTableSectionElement | null;\\n    /**\\n     * Retrieves the tHead object of the table.\\n     */\\n    tHead: HTMLTableSectionElement | null;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    /** @deprecated */\\n    width: string;\\n    /**\\n     * Creates an empty caption element in the table.\\n     */\\n    createCaption(): HTMLTableCaptionElement;\\n    /**\\n     * Creates an empty tBody element in the table.\\n     */\\n    createTBody(): HTMLTableSectionElement;\\n    /**\\n     * Creates an empty tFoot element in the table.\\n     */\\n    createTFoot(): HTMLTableSectionElement;\\n    /**\\n     * Returns the tHead element object if successful, or null otherwise.\\n     */\\n    createTHead(): HTMLTableSectionElement;\\n    /**\\n     * Deletes the caption element and its contents from the table.\\n     */\\n    deleteCaption(): void;\\n    /**\\n     * Removes the specified row (tr) from the element and from the rows collection.\\n     * @param index Number that specifies the zero-based position in the rows collection of the row to remove.\\n     */\\n    deleteRow(index: number): void;\\n    /**\\n     * Deletes the tFoot element and its contents from the table.\\n     */\\n    deleteTFoot(): void;\\n    /**\\n     * Deletes the tHead element and its contents from the table.\\n     */\\n    deleteTHead(): void;\\n    /**\\n     * Creates a new row (tr) in the table, and adds the row to the rows collection.\\n     * @param index Number that specifies where to insert the row in the rows collection. The default value is -1, which appends the new row to the end of the rows collection.\\n     */\\n    insertRow(index?: number): HTMLTableRowElement;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableElement: {\\n    prototype: HTMLTableElement;\\n    new(): HTMLTableElement;\\n};\\n\\ninterface HTMLTableHeaderCellElement extends HTMLTableCellElement {\\n    scope: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableHeaderCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableHeaderCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableHeaderCellElement: {\\n    prototype: HTMLTableHeaderCellElement;\\n    new(): HTMLTableHeaderCellElement;\\n};\\n\\ninterface HTMLTableRowElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves how the object is aligned with adjacent text.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    bgColor: string;\\n    /**\\n     * Retrieves a collection of all cells in the table row.\\n     */\\n    readonly cells: HTMLCollectionOf<HTMLTableDataCellElement | HTMLTableHeaderCellElement>;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Retrieves the position of the object in the rows collection for the table.\\n     */\\n    readonly rowIndex: number;\\n    /**\\n     * Retrieves the position of the object in the collection.\\n     */\\n    readonly sectionRowIndex: number;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Removes the specified cell from the table row, as well as from the cells collection.\\n     * @param index Number that specifies the zero-based position of the cell to remove from the table row. If no value is provided, the last cell in the cells collection is deleted.\\n     */\\n    deleteCell(index: number): void;\\n    /**\\n     * Creates a new cell in the table row, and adds the cell to the cells collection.\\n     * @param index Number that specifies where to insert the cell in the tr. The default value is -1, which appends the new cell to the end of the cells collection.\\n     */\\n    insertCell(index?: number): HTMLTableDataCellElement;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableRowElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableRowElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableRowElement: {\\n    prototype: HTMLTableRowElement;\\n    new(): HTMLTableRowElement;\\n};\\n\\ninterface HTMLTableSectionElement extends HTMLElement {\\n    /**\\n     * Sets or retrieves a value that indicates the table alignment.\\n     */\\n    /** @deprecated */\\n    align: string;\\n    /** @deprecated */\\n    ch: string;\\n    /** @deprecated */\\n    chOff: string;\\n    /**\\n     * Sets or retrieves the number of horizontal rows contained in the object.\\n     */\\n    readonly rows: HTMLCollectionOf<HTMLTableRowElement>;\\n    /** @deprecated */\\n    vAlign: string;\\n    /**\\n     * Removes the specified row (tr) from the element and from the rows collection.\\n     * @param index Number that specifies the zero-based position in the rows collection of the row to remove.\\n     */\\n    deleteRow(index: number): void;\\n    /**\\n     * Creates a new row (tr) in the table, and adds the row to the rows collection.\\n     * @param index Number that specifies where to insert the row in the rows collection. The default value is -1, which appends the new row to the end of the rows collection.\\n     */\\n    insertRow(index?: number): HTMLTableRowElement;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableSectionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableSectionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTableSectionElement: {\\n    prototype: HTMLTableSectionElement;\\n    new(): HTMLTableSectionElement;\\n};\\n\\ninterface HTMLTemplateElement extends HTMLElement {\\n    readonly content: DocumentFragment;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTemplateElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTemplateElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTemplateElement: {\\n    prototype: HTMLTemplateElement;\\n    new(): HTMLTemplateElement;\\n};\\n\\ninterface HTMLTextAreaElement extends HTMLElement {\\n    autocomplete: string;\\n    /**\\n     * Provides a way to direct a user to a specific field when a document loads. This can provide both direction and convenience for a user, reducing the need to click or tab to a field when a page opens. This attribute is true when present on an element, and false when missing.\\n     */\\n    autofocus: boolean;\\n    /**\\n     * Sets or retrieves the width of the object.\\n     */\\n    cols: number;\\n    /**\\n     * Sets or retrieves the initial contents of the object.\\n     */\\n    defaultValue: string;\\n    dirName: string;\\n    disabled: boolean;\\n    /**\\n     * Retrieves a reference to the form that the object is embedded in.\\n     */\\n    readonly form: HTMLFormElement | null;\\n    readonly labels: NodeListOf<HTMLLabelElement>;\\n    /**\\n     * Sets or retrieves the maximum number of characters that the user can enter in a text control.\\n     */\\n    maxLength: number;\\n    minLength: number;\\n    /**\\n     * Sets or retrieves the name of the object.\\n     */\\n    name: string;\\n    /**\\n     * Gets or sets a text string that is displayed in an input field as a hint or prompt to users as the format or type of information they need to enter.The text appears in an input field until the user puts focus on the field.\\n     */\\n    placeholder: string;\\n    /**\\n     * Sets or retrieves the value indicated whether the content of the object is read-only.\\n     */\\n    readOnly: boolean;\\n    /**\\n     * When present, marks an element that can\\'t be submitted without a value.\\n     */\\n    required: boolean;\\n    /**\\n     * Sets or retrieves the number of horizontal rows contained in the object.\\n     */\\n    rows: number;\\n    selectionDirection: string;\\n    /**\\n     * Gets or sets the end position or offset of a text selection.\\n     */\\n    selectionEnd: number;\\n    /**\\n     * Gets or sets the starting position or offset of a text selection.\\n     */\\n    selectionStart: number;\\n    readonly textLength: number;\\n    /**\\n     * Retrieves the type of control.\\n     */\\n    readonly type: string;\\n    /**\\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as \"this is a required field\". The result is that the user sees validation messages without actually submitting.\\n     */\\n    readonly validationMessage: string;\\n    /**\\n     * Returns a  ValidityState object that represents the validity states of an element.\\n     */\\n    readonly validity: ValidityState;\\n    /**\\n     * Retrieves or sets the text in the entry field of the textArea element.\\n     */\\n    value: string;\\n    /**\\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\\n     */\\n    readonly willValidate: boolean;\\n    /**\\n     * Sets or retrieves how to handle wordwrapping in the object.\\n     */\\n    wrap: string;\\n    /**\\n     * Returns whether a form will validate when it is submitted, without having to submit it.\\n     */\\n    checkValidity(): boolean;\\n    reportValidity(): boolean;\\n    /**\\n     * Highlights the input area of a form element.\\n     */\\n    select(): void;\\n    /**\\n     * Sets a custom error message that is displayed when a form is submitted.\\n     * @param error Sets a custom error message that is displayed when a form is submitted.\\n     */\\n    setCustomValidity(error: string): void;\\n    setRangeText(replacement: string): void;\\n    setRangeText(replacement: string, start: number, end: number, selectionMode?: SelectionMode): void;\\n    /**\\n     * Sets the start and end positions of a selection in a text field.\\n     * @param start The offset into the text field for the start of the selection.\\n     * @param end The offset into the text field for the end of the selection.\\n     * @param direction The direction in which the selection is performed.\\n     */\\n    setSelectionRange(start: number, end: number, direction?: \"forward\" | \"backward\" | \"none\"): void;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTextAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTextAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTextAreaElement: {\\n    prototype: HTMLTextAreaElement;\\n    new(): HTMLTextAreaElement;\\n};\\n\\ninterface HTMLTimeElement extends HTMLElement {\\n    dateTime: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTimeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTimeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTimeElement: {\\n    prototype: HTMLTimeElement;\\n    new(): HTMLTimeElement;\\n};\\n\\ninterface HTMLTitleElement extends HTMLElement {\\n    /**\\n     * Retrieves or sets the text of the object as a string.\\n     */\\n    text: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTitleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTitleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTitleElement: {\\n    prototype: HTMLTitleElement;\\n    new(): HTMLTitleElement;\\n};\\n\\ninterface HTMLTrackElement extends HTMLElement {\\n    default: boolean;\\n    kind: string;\\n    label: string;\\n    readonly readyState: number;\\n    src: string;\\n    srclang: string;\\n    readonly track: TextTrack;\\n    readonly ERROR: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTrackElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTrackElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLTrackElement: {\\n    prototype: HTMLTrackElement;\\n    new(): HTMLTrackElement;\\n    readonly ERROR: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n};\\n\\ninterface HTMLUListElement extends HTMLElement {\\n    /** @deprecated */\\n    compact: boolean;\\n    /** @deprecated */\\n    type: string;\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLUListElement: {\\n    prototype: HTMLUListElement;\\n    new(): HTMLUListElement;\\n};\\n\\ninterface HTMLUnknownElement extends HTMLElement {\\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUnknownElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUnknownElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLUnknownElement: {\\n    prototype: HTMLUnknownElement;\\n    new(): HTMLUnknownElement;\\n};\\n\\ninterface HTMLVideoElementEventMap extends HTMLMediaElementEventMap {\\n    \"MSVideoFormatChanged\": Event;\\n    \"MSVideoFrameStepCompleted\": Event;\\n    \"MSVideoOptimalLayoutChanged\": Event;\\n}\\n\\ninterface HTMLVideoElement extends HTMLMediaElement {\\n    /**\\n     * Gets or sets the height of the video element.\\n     */\\n    height: number;\\n    msHorizontalMirror: boolean;\\n    readonly msIsLayoutOptimalForPlayback: boolean;\\n    readonly msIsStereo3D: boolean;\\n    msStereo3DPackingMode: string;\\n    msStereo3DRenderMode: string;\\n    msZoom: boolean;\\n    onMSVideoFormatChanged: ((this: HTMLVideoElement, ev: Event) => any) | null;\\n    onMSVideoFrameStepCompleted: ((this: HTMLVideoElement, ev: Event) => any) | null;\\n    onMSVideoOptimalLayoutChanged: ((this: HTMLVideoElement, ev: Event) => any) | null;\\n    /**\\n     * Gets or sets a URL of an image to display, for example, like a movie poster. This can be a still frame from the video, or another image if no video data is available.\\n     */\\n    poster: string;\\n    /**\\n     * Gets the intrinsic height of a video in CSS pixels, or zero if the dimensions are not known.\\n     */\\n    readonly videoHeight: number;\\n    /**\\n     * Gets the intrinsic width of a video in CSS pixels, or zero if the dimensions are not known.\\n     */\\n    readonly videoWidth: number;\\n    readonly webkitDisplayingFullscreen: boolean;\\n    readonly webkitSupportsFullscreen: boolean;\\n    /**\\n     * Gets or sets the width of the video element.\\n     */\\n    width: number;\\n    getVideoPlaybackQuality(): VideoPlaybackQuality;\\n    msFrameStep(forward: boolean): void;\\n    msInsertVideoEffect(activatableClassId: string, effectRequired: boolean, config?: any): void;\\n    msSetVideoRectangle(left: number, top: number, right: number, bottom: number): void;\\n    webkitEnterFullScreen(): void;\\n    webkitEnterFullscreen(): void;\\n    webkitExitFullScreen(): void;\\n    webkitExitFullscreen(): void;\\n    addEventListener<K extends keyof HTMLVideoElementEventMap>(type: K, listener: (this: HTMLVideoElement, ev: HTMLVideoElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof HTMLVideoElementEventMap>(type: K, listener: (this: HTMLVideoElement, ev: HTMLVideoElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var HTMLVideoElement: {\\n    prototype: HTMLVideoElement;\\n    new(): HTMLVideoElement;\\n};\\n\\ninterface HashChangeEvent extends Event {\\n    readonly newURL: string;\\n    readonly oldURL: string;\\n}\\n\\ndeclare var HashChangeEvent: {\\n    prototype: HashChangeEvent;\\n    new(type: string, eventInitDict?: HashChangeEventInit): HashChangeEvent;\\n};\\n\\ninterface Headers {\\n    append(name: string, value: string): void;\\n    delete(name: string): void;\\n    get(name: string): string | null;\\n    has(name: string): boolean;\\n    set(name: string, value: string): void;\\n    forEach(callbackfn: (value: string, key: string, parent: Headers) => void, thisArg?: any): void;\\n}\\n\\ndeclare var Headers: {\\n    prototype: Headers;\\n    new(init?: HeadersInit): Headers;\\n};\\n\\ninterface History {\\n    readonly length: number;\\n    scrollRestoration: ScrollRestoration;\\n    readonly state: any;\\n    back(): void;\\n    forward(): void;\\n    go(delta?: number): void;\\n    pushState(data: any, title: string, url?: string | null): void;\\n    replaceState(data: any, title: string, url?: string | null): void;\\n}\\n\\ndeclare var History: {\\n    prototype: History;\\n    new(): History;\\n};\\n\\ninterface HkdfCtrParams extends Algorithm {\\n    context: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n    hash: string | Algorithm;\\n    label: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer;\\n}\\n\\ninterface IDBArrayKey extends Array<IDBValidKey> {\\n}\\n\\ninterface IDBCursor {\\n    /**\\n     * Returns the direction (\"next\", \"nextunique\", \"prev\" or \"prevunique\")\\n     * of the cursor.\\n     */\\n    readonly direction: IDBCursorDirection;\\n    /**\\n     * Returns the key of the cursor.\\n     * Throws a \"InvalidStateError\" DOMException if the cursor is advancing or is finished.\\n     */\\n    readonly key: IDBValidKey | IDBKeyRange;\\n    /**\\n     * Returns the effective key of the cursor.\\n     * Throws a \"InvalidStateError\" DOMException if the cursor is advancing or is finished.\\n     */\\n    readonly primaryKey: IDBValidKey | IDBKeyRange;\\n    /**\\n     * Returns the IDBObjectStore or IDBIndex the cursor was opened from.\\n     */\\n    readonly source: IDBObjectStore | IDBIndex;\\n    /**\\n     * Advances the cursor through the next count records in\\n     * range.\\n     */\\n    advance(count: number): void;\\n    /**\\n     * Advances the cursor to the next record in range matching or\\n     * after key.\\n     */\\n    continue(key?: IDBValidKey | IDBKeyRange): void;\\n    /**\\n     * Advances the cursor to the next record in range matching\\n     * or after key and primaryKey. Throws an \"InvalidAccessError\" DOMException if the source is not an index.\\n     */\\n    continuePrimaryKey(key: IDBValidKey | IDBKeyRange, primaryKey: IDBValidKey | IDBKeyRange): void;\\n    /**\\n     * Delete the record pointed at by the cursor with a new value.\\n     * If successful, request\\'s result will be undefined.\\n     */\\n    delete(): IDBRequest<undefined>;\\n    /**\\n     * Updated the record pointed at by the cursor with a new value.\\n     * Throws a \"DataError\" DOMException if the effective object store uses in-line keys and the key would have changed.\\n     * If successful, request\\'s result will be the record\\'s key.\\n     */\\n    update(value: any): IDBRequest<IDBValidKey>;\\n}\\n\\ndeclare var IDBCursor: {\\n    prototype: IDBCursor;\\n    new(): IDBCursor;\\n};\\n\\ninterface IDBCursorWithValue extends IDBCursor {\\n    /**\\n     * Returns the cursor\\'s current value.\\n     */\\n    readonly value: any;\\n}\\n\\ndeclare var IDBCursorWithValue: {\\n    prototype: IDBCursorWithValue;\\n    new(): IDBCursorWithValue;\\n};\\n\\ninterface IDBDatabaseEventMap {\\n    \"abort\": Event;\\n    \"close\": Event;\\n    \"error\": Event;\\n    \"versionchange\": IDBVersionChangeEvent;\\n}\\n\\ninterface IDBDatabase extends EventTarget {\\n    /**\\n     * Returns the name of the database.\\n     */\\n    readonly name: string;\\n    /**\\n     * Returns a list of the names of object stores in the database.\\n     */\\n    readonly objectStoreNames: DOMStringList;\\n    onabort: ((this: IDBDatabase, ev: Event) => any) | null;\\n    onclose: ((this: IDBDatabase, ev: Event) => any) | null;\\n    onerror: ((this: IDBDatabase, ev: Event) => any) | null;\\n    onversionchange: ((this: IDBDatabase, ev: IDBVersionChangeEvent) => any) | null;\\n    /**\\n     * Returns the version of the database.\\n     */\\n    readonly version: number;\\n    /**\\n     * Closes the connection once all running transactions have finished.\\n     */\\n    close(): void;\\n    /**\\n     * Creates a new object store with the given name and options and returns a new IDBObjectStore.\\n     * Throws a \"InvalidStateError\" DOMException if not called within an upgrade transaction.\\n     */\\n    createObjectStore(name: string, optionalParameters?: IDBObjectStoreParameters): IDBObjectStore;\\n    /**\\n     * Deletes the object store with the given name.\\n     * Throws a \"InvalidStateError\" DOMException if not called within an upgrade transaction.\\n     */\\n    deleteObjectStore(name: string): void;\\n    /**\\n     * Returns a new transaction with the given mode (\"readonly\" or \"readwrite\")\\n     * and scope which can be a single object store name or an array of names.\\n     */\\n    transaction(storeNames: string | string[], mode?: IDBTransactionMode): IDBTransaction;\\n    addEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBDatabase: {\\n    prototype: IDBDatabase;\\n    new(): IDBDatabase;\\n};\\n\\ninterface IDBEnvironment {\\n    readonly indexedDB: IDBFactory;\\n}\\n\\ninterface IDBFactory {\\n    /**\\n     * Compares two values as keys. Returns -1 if key1 precedes key2, 1 if key2 precedes key1, and 0 if\\n     * the keys are equal.\\n     * Throws a \"DataError\" DOMException if either input is not a valid key.\\n     */\\n    cmp(first: any, second: any): number;\\n    /**\\n     * Attempts to delete the named database. If the\\n     * database already exists and there are open connections that don\\'t close in response to a versionchange event, the request will be blocked until all they close. If the request\\n     * is successful request\\'s result will be null.\\n     */\\n    deleteDatabase(name: string): IDBOpenDBRequest;\\n    /**\\n     * Attempts to open a connection to the named database with the specified version. If the database already exists\\n     * with a lower version and there are open connections that don\\'t close in response to a versionchange event, the request will be blocked until all they close, then an upgrade\\n     * will occur. If the database already exists with a higher\\n     * version the request will fail. If the request is\\n     * successful request\\'s result will\\n     * be the connection.\\n     */\\n    open(name: string, version?: number): IDBOpenDBRequest;\\n}\\n\\ndeclare var IDBFactory: {\\n    prototype: IDBFactory;\\n    new(): IDBFactory;\\n};\\n\\ninterface IDBIndex {\\n    readonly keyPath: string | string[];\\n    readonly multiEntry: boolean;\\n    /**\\n     * Updates the name of the store to newName.\\n     * Throws an \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    name: string;\\n    /**\\n     * Returns the IDBObjectStore the index belongs to.\\n     */\\n    readonly objectStore: IDBObjectStore;\\n    readonly unique: boolean;\\n    /**\\n     * Retrieves the number of records matching the given key or key range in query.\\n     * If successful, request\\'s result will be the\\n     * count.\\n     */\\n    count(key?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\\n    /**\\n     * Retrieves the value of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the value, or undefined if there was no matching record.\\n     */\\n    get(key: IDBValidKey | IDBKeyRange): IDBRequest<any | undefined>;\\n    /**\\n     * Retrieves the values of the records matching the given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will be an Array of the values.\\n     */\\n    getAll(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<any[]>;\\n    /**\\n     * Retrieves the keys of records matching the given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will be an Array of the keys.\\n     */\\n    getAllKeys(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<IDBValidKey[]>;\\n    /**\\n     * Retrieves the key of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the key, or undefined if there was no matching record.\\n     */\\n    getKey(key: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\\n    /**\\n     * Opens a cursor over the records matching query,\\n     * ordered by direction. If query is null, all records in index are matched.\\n     * If successful, request\\'s result will be an IDBCursorWithValue, or null if there were no matching records.\\n     */\\n    openCursor(range?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\\n    /**\\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in index are matched.\\n     * If successful, request\\'s result will be an IDBCursor, or null if there were no matching records.\\n     */\\n    openKeyCursor(range?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\\n}\\n\\ndeclare var IDBIndex: {\\n    prototype: IDBIndex;\\n    new(): IDBIndex;\\n};\\n\\ninterface IDBKeyRange {\\n    /**\\n     * Returns lower bound, or undefined if none.\\n     */\\n    readonly lower: any;\\n    /**\\n     * Returns true if the lower open flag is set, and false otherwise.\\n     */\\n    readonly lowerOpen: boolean;\\n    /**\\n     * Returns upper bound, or undefined if none.\\n     */\\n    readonly upper: any;\\n    /**\\n     * Returns true if the upper open flag is set, and false otherwise.\\n     */\\n    readonly upperOpen: boolean;\\n    /**\\n     * Returns true if key is included in the range, and false otherwise.\\n     */\\n    includes(key: any): boolean;\\n}\\n\\ndeclare var IDBKeyRange: {\\n    prototype: IDBKeyRange;\\n    new(): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange spanning from lower to upper.\\n     * If lowerOpen is true, lower is not included in the range.\\n     * If upperOpen is true, upper is not included in the range.\\n     */\\n    bound(lower: any, upper: any, lowerOpen?: boolean, upperOpen?: boolean): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange starting at key with no\\n     * upper bound. If open is true, key is not included in the\\n     * range.\\n     */\\n    lowerBound(lower: any, open?: boolean): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange spanning only key.\\n     */\\n    only(value: any): IDBKeyRange;\\n    /**\\n     * Returns a new IDBKeyRange with no lower bound and ending at key. If open is true, key is not included in the range.\\n     */\\n    upperBound(upper: any, open?: boolean): IDBKeyRange;\\n};\\n\\ninterface IDBObjectStore {\\n    /**\\n     * Returns true if the store has a key generator, and false otherwise.\\n     */\\n    readonly autoIncrement: boolean;\\n    /**\\n     * Returns a list of the names of indexes in the store.\\n     */\\n    readonly indexNames: DOMStringList;\\n    /**\\n     * Returns the key path of the store, or null if none.\\n     */\\n    readonly keyPath: string | string[];\\n    /**\\n     * Updates the name of the store to newName.\\n     * Throws \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    name: string;\\n    /**\\n     * Returns the associated transaction.\\n     */\\n    readonly transaction: IDBTransaction;\\n    add(value: any, key?: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey>;\\n    /**\\n     * Deletes all records in store.\\n     * If successful, request\\'s result will\\n     * be undefined.\\n     */\\n    clear(): IDBRequest<undefined>;\\n    /**\\n     * Retrieves the number of records matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the count.\\n     */\\n    count(key?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\\n    /**\\n     * Creates a new index in store with the given name, keyPath and options and returns a new IDBIndex. If the keyPath and options define constraints that cannot be\\n     * satisfied with the data already in store the upgrade\\n     * transaction will abort with\\n     * a \"ConstraintError\" DOMException.\\n     * Throws an \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    createIndex(name: string, keyPath: string | string[], options?: IDBIndexParameters): IDBIndex;\\n    /**\\n     * Deletes records in store with the given key or in the given key range in query.\\n     * If successful, request\\'s result will\\n     * be undefined.\\n     */\\n    delete(key: IDBValidKey | IDBKeyRange): IDBRequest<undefined>;\\n    /**\\n     * Deletes the index in store with the given name.\\n     * Throws an \"InvalidStateError\" DOMException if not called within an upgrade\\n     * transaction.\\n     */\\n    deleteIndex(name: string): void;\\n    /**\\n     * Retrieves the value of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the value, or undefined if there was no matching record.\\n     */\\n    get(query: IDBValidKey | IDBKeyRange): IDBRequest<any | undefined>;\\n    /**\\n     * Retrieves the values of the records matching the\\n     * given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will\\n     * be an Array of the values.\\n     */\\n    getAll(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<any[]>;\\n    /**\\n     * Retrieves the keys of records matching the\\n     * given key or key range in query (up to count if given).\\n     * If successful, request\\'s result will\\n     * be an Array of the keys.\\n     */\\n    getAllKeys(query?: IDBValidKey | IDBKeyRange, count?: number): IDBRequest<IDBValidKey[]>;\\n    /**\\n     * Retrieves the key of the first record matching the\\n     * given key or key range in query.\\n     * If successful, request\\'s result will be the key, or undefined if there was no matching record.\\n     */\\n    getKey(query: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\\n    index(name: string): IDBIndex;\\n    /**\\n     * Opens a cursor over the records matching query,\\n     * ordered by direction. If query is null, all records in store are matched.\\n     * If successful, request\\'s result will be an IDBCursorWithValue pointing at the first matching record, or null if there were no matching records.\\n     */\\n    openCursor(range?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\\n    /**\\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in store are matched.\\n     * If successful, request\\'s result will be an IDBCursor pointing at the first matching record, or\\n     * null if there were no matching records.\\n     */\\n    openKeyCursor(query?: IDBValidKey | IDBKeyRange, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\\n    put(value: any, key?: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey>;\\n}\\n\\ndeclare var IDBObjectStore: {\\n    prototype: IDBObjectStore;\\n    new(): IDBObjectStore;\\n};\\n\\ninterface IDBOpenDBRequestEventMap extends IDBRequestEventMap {\\n    \"blocked\": Event;\\n    \"upgradeneeded\": IDBVersionChangeEvent;\\n}\\n\\ninterface IDBOpenDBRequest extends IDBRequest<IDBDatabase> {\\n    onblocked: ((this: IDBOpenDBRequest, ev: Event) => any) | null;\\n    onupgradeneeded: ((this: IDBOpenDBRequest, ev: IDBVersionChangeEvent) => any) | null;\\n    addEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBOpenDBRequest: {\\n    prototype: IDBOpenDBRequest;\\n    new(): IDBOpenDBRequest;\\n};\\n\\ninterface IDBRequestEventMap {\\n    \"error\": Event;\\n    \"success\": Event;\\n}\\n\\ninterface IDBRequest<T = any> extends EventTarget {\\n    /**\\n     * When a request is completed, returns the error (a DOMException), or null if the request succeeded. Throws\\n     * a \"InvalidStateError\" DOMException if the request is still pending.\\n     */\\n    readonly error: DOMException | null;\\n    onerror: ((this: IDBRequest<T>, ev: Event) => any) | null;\\n    onsuccess: ((this: IDBRequest<T>, ev: Event) => any) | null;\\n    /**\\n     * Returns \"pending\" until a request is complete,\\n     * then returns \"done\".\\n     */\\n    readonly readyState: IDBRequestReadyState;\\n    /**\\n     * When a request is completed, returns the result,\\n     * or undefined if the request failed. Throws a\\n     * \"InvalidStateError\" DOMException if the request is still pending.\\n     */\\n    readonly result: T;\\n    /**\\n     * Returns the IDBObjectStore, IDBIndex, or IDBCursor the request was made against, or null if is was an open\\n     * request.\\n     */\\n    readonly source: IDBObjectStore | IDBIndex | IDBCursor;\\n    /**\\n     * Returns the IDBTransaction the request was made within.\\n     * If this as an open request, then it returns an upgrade transaction while it is running, or null otherwise.\\n     */\\n    readonly transaction: IDBTransaction | null;\\n    addEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBRequest: {\\n    prototype: IDBRequest;\\n    new(): IDBRequest;\\n};\\n\\ninterface IDBTransactionEventMap {\\n    \"abort\": Event;\\n    \"complete\": Event;\\n    \"error\": Event;\\n}\\n\\ninterface IDBTransaction extends EventTarget {\\n    /**\\n     * Returns the transaction\\'s connection.\\n     */\\n    readonly db: IDBDatabase;\\n    /**\\n     * If the transaction was aborted, returns the\\n     * error (a DOMException) providing the reason.\\n     */\\n    readonly error: DOMException;\\n    /**\\n     * Returns the mode the transaction was created with\\n     * (\"readonly\" or \"readwrite\"), or \"versionchange\" for\\n     * an upgrade transaction.\\n     */\\n    readonly mode: IDBTransactionMode;\\n    /**\\n     * Returns a list of the names of object stores in the\\n     * transaction\\'s scope. For an upgrade transaction this is all object stores in the database.\\n     */\\n    readonly objectStoreNames: DOMStringList;\\n    onabort: ((this: IDBTransaction, ev: Event) => any) | null;\\n    oncomplete: ((this: IDBTransaction, ev: Event) => any) | null;\\n    onerror: ((this: IDBTransaction, ev: Event) => any) | null;\\n    /**\\n     * Aborts the transaction. All pending requests will fail with\\n     * a \"AbortError\" DOMException and all changes made to the database will be\\n     * reverted.\\n     */\\n    abort(): void;\\n    /**\\n     * Returns an IDBObjectStore in the transaction\\'s scope.\\n     */\\n    objectStore(name: string): IDBObjectStore;\\n    addEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var IDBTransaction: {\\n    prototype: IDBTransaction;\\n    new(): IDBTransaction;\\n};\\n\\ninterface IDBVersionChangeEvent extends Event {\\n    readonly newVersion: number | null;\\n    readonly oldVersion: number;\\n}\\n\\ndeclare var IDBVersionChangeEvent: {\\n    prototype: IDBVersionChangeEvent;\\n    new(type: string, eventInitDict?: IDBVersionChangeEventInit): IDBVersionChangeEvent;\\n};\\n\\ninterface IIRFilterNode extends AudioNode {\\n    getFrequencyResponse(frequencyHz: Float32Array, magResponse: Float32Array, phaseResponse: Float32Array): void;\\n}\\n\\ndeclare var IIRFilterNode: {\\n    prototype: IIRFilterNode;\\n    new(context: BaseAudioContext, options: IIRFilterOptions): IIRFilterNode;\\n};\\n\\ninterface ImageBitmap {\\n    /**\\n     * Returns the intrinsic height of the image, in CSS\\n     * pixels.\\n     */\\n    readonly height: number;\\n    /**\\n     * Returns the intrinsic width of the image, in CSS\\n     * pixels.\\n     */\\n    readonly width: number;\\n    /**\\n     * Releases imageBitmap\\'s underlying bitmap data.\\n     */\\n    close(): void;\\n}\\n\\ndeclare var ImageBitmap: {\\n    prototype: ImageBitmap;\\n    new(): ImageBitmap;\\n};\\n\\ninterface ImageBitmapOptions {\\n    colorSpaceConversion?: \"none\" | \"default\";\\n    imageOrientation?: \"none\" | \"flipY\";\\n    premultiplyAlpha?: \"none\" | \"premultiply\" | \"default\";\\n    resizeHeight?: number;\\n    resizeQuality?: \"pixelated\" | \"low\" | \"medium\" | \"high\";\\n    resizeWidth?: number;\\n}\\n\\ninterface ImageBitmapRenderingContext {\\n    /**\\n     * Returns the canvas element that the context is bound to.\\n     */\\n    readonly canvas: HTMLCanvasElement;\\n    /**\\n     * Replaces contents of the canvas element to which context\\n     * is bound with a transparent black bitmap whose size corresponds to the width and height\\n     * content attributes of the canvas element.\\n     */\\n    transferFromImageBitmap(bitmap: ImageBitmap | null): void;\\n}\\n\\ndeclare var ImageBitmapRenderingContext: {\\n    prototype: ImageBitmapRenderingContext;\\n    new(): ImageBitmapRenderingContext;\\n};\\n\\ninterface ImageData {\\n    /**\\n     * Returns the one-dimensional array containing the data in RGBA order, as integers in the\\n     * range 0 to 255.\\n     */\\n    readonly data: Uint8ClampedArray;\\n    /**\\n     * Returns the actual dimensions of the data in the ImageData object, in\\n     * pixels.\\n     */\\n    readonly height: number;\\n    readonly width: number;\\n}\\n\\ndeclare var ImageData: {\\n    prototype: ImageData;\\n    new(width: number, height: number): ImageData;\\n    new(array: Uint8ClampedArray, width: number, height: number): ImageData;\\n};\\n\\ninterface IntersectionObserver {\\n    readonly root: Element | null;\\n    readonly rootMargin: string;\\n    readonly thresholds: number[];\\n    disconnect(): void;\\n    observe(target: Element): void;\\n    takeRecords(): IntersectionObserverEntry[];\\n    unobserve(target: Element): void;\\n}\\n\\ndeclare var IntersectionObserver: {\\n    prototype: IntersectionObserver;\\n    new(callback: IntersectionObserverCallback, options?: IntersectionObserverInit): IntersectionObserver;\\n};\\n\\ninterface IntersectionObserverEntry {\\n    readonly boundingClientRect: ClientRect | DOMRect;\\n    readonly intersectionRatio: number;\\n    readonly intersectionRect: ClientRect | DOMRect;\\n    readonly isIntersecting: boolean;\\n    readonly rootBounds: ClientRect | DOMRect;\\n    readonly target: Element;\\n    readonly time: number;\\n}\\n\\ndeclare var IntersectionObserverEntry: {\\n    prototype: IntersectionObserverEntry;\\n    new(intersectionObserverEntryInit: IntersectionObserverEntryInit): IntersectionObserverEntry;\\n};\\n\\ninterface KeyboardEvent extends UIEvent {\\n    readonly altKey: boolean;\\n    /** @deprecated */\\n    char: string;\\n    /** @deprecated */\\n    readonly charCode: number;\\n    readonly code: string;\\n    readonly ctrlKey: boolean;\\n    readonly key: string;\\n    /** @deprecated */\\n    readonly keyCode: number;\\n    readonly location: number;\\n    readonly metaKey: boolean;\\n    readonly repeat: boolean;\\n    readonly shiftKey: boolean;\\n    /** @deprecated */\\n    readonly which: number;\\n    getModifierState(keyArg: string): boolean;\\n    /** @deprecated */\\n    initKeyboardEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, keyArg: string, locationArg: number, modifiersListArg: string, repeat: boolean, locale: string): void;\\n    readonly DOM_KEY_LOCATION_JOYSTICK: number;\\n    readonly DOM_KEY_LOCATION_LEFT: number;\\n    readonly DOM_KEY_LOCATION_MOBILE: number;\\n    readonly DOM_KEY_LOCATION_NUMPAD: number;\\n    readonly DOM_KEY_LOCATION_RIGHT: number;\\n    readonly DOM_KEY_LOCATION_STANDARD: number;\\n}\\n\\ndeclare var KeyboardEvent: {\\n    prototype: KeyboardEvent;\\n    new(typeArg: string, eventInitDict?: KeyboardEventInit): KeyboardEvent;\\n    readonly DOM_KEY_LOCATION_JOYSTICK: number;\\n    readonly DOM_KEY_LOCATION_LEFT: number;\\n    readonly DOM_KEY_LOCATION_MOBILE: number;\\n    readonly DOM_KEY_LOCATION_NUMPAD: number;\\n    readonly DOM_KEY_LOCATION_RIGHT: number;\\n    readonly DOM_KEY_LOCATION_STANDARD: number;\\n};\\n\\ninterface KeyframeEffect extends AnimationEffect {\\n    composite: CompositeOperation;\\n    iterationComposite: IterationCompositeOperation;\\n    target: Element | null;\\n    getKeyframes(): ComputedKeyframe[];\\n    setKeyframes(keyframes: Keyframe[] | PropertyIndexedKeyframes | null): void;\\n}\\n\\ndeclare var KeyframeEffect: {\\n    prototype: KeyframeEffect;\\n    new(target: Element | null, keyframes: Keyframe[] | PropertyIndexedKeyframes | null, options?: number | KeyframeEffectOptions): KeyframeEffect;\\n    new(source: KeyframeEffect): KeyframeEffect;\\n};\\n\\ninterface LinkStyle {\\n    readonly sheet: StyleSheet | null;\\n}\\n\\ninterface ListeningStateChangedEvent extends Event {\\n    readonly label: string;\\n    readonly state: ListeningState;\\n}\\n\\ndeclare var ListeningStateChangedEvent: {\\n    prototype: ListeningStateChangedEvent;\\n    new(): ListeningStateChangedEvent;\\n};\\n\\ninterface Location {\\n    /**\\n     * Returns a DOMStringList object listing the origins of the ancestor browsing contexts, from the parent browsing\\n     * context to the top-level browsing context.\\n     */\\n    readonly ancestorOrigins: DOMStringList;\\n    /**\\n     * Returns the Location object\\'s URL\\'s fragment (includes leading \"#\" if non-empty).\\n     * Can be set, to navigate to the same URL with a changed fragment (ignores leading \"#\").\\n     */\\n    hash: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s host and port (if different from the default\\n     * port for the scheme).\\n     * Can be set, to navigate to the same URL with a changed host and port.\\n     */\\n    host: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s host.\\n     * Can be set, to navigate to the same URL with a changed host.\\n     */\\n    hostname: string;\\n    /**\\n     * Returns the Location object\\'s URL.\\n     * Can be set, to navigate to the given URL.\\n     */\\n    href: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s origin.\\n     */\\n    readonly origin: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s path.\\n     * Can be set, to navigate to the same URL with a changed path.\\n     */\\n    pathname: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s port.\\n     * Can be set, to navigate to the same URL with a changed port.\\n     */\\n    port: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s scheme.\\n     * Can be set, to navigate to the same URL with a changed scheme.\\n     */\\n    protocol: string;\\n    /**\\n     * Returns the Location object\\'s URL\\'s query (includes leading \"?\" if non-empty).\\n     * Can be set, to navigate to the same URL with a changed query (ignores leading \"?\").\\n     */\\n    search: string;\\n    /**\\n     * Navigates to the given URL.\\n     */\\n    assign(url: string): void;\\n    /**\\n     * Reloads the current page.\\n     */\\n    reload(forcedReload?: boolean): void;\\n    /**\\n     * Removes the current page from the session history and navigates to the given URL.\\n     */\\n    replace(url: string): void;\\n}\\n\\ndeclare var Location: {\\n    prototype: Location;\\n    new(): Location;\\n};\\n\\ninterface MSAssertion {\\n    readonly id: string;\\n    readonly type: MSCredentialType;\\n}\\n\\ndeclare var MSAssertion: {\\n    prototype: MSAssertion;\\n    new(): MSAssertion;\\n};\\n\\ninterface MSBlobBuilder {\\n    append(data: any, endings?: string): void;\\n    getBlob(contentType?: string): Blob;\\n}\\n\\ndeclare var MSBlobBuilder: {\\n    prototype: MSBlobBuilder;\\n    new(): MSBlobBuilder;\\n};\\n\\ninterface MSFIDOCredentialAssertion extends MSAssertion {\\n    readonly algorithm: string | Algorithm;\\n    readonly attestation: any;\\n    readonly publicKey: string;\\n    readonly transportHints: MSTransportType[];\\n}\\n\\ndeclare var MSFIDOCredentialAssertion: {\\n    prototype: MSFIDOCredentialAssertion;\\n    new(): MSFIDOCredentialAssertion;\\n};\\n\\ninterface MSFIDOSignature {\\n    readonly authnrData: string;\\n    readonly clientData: string;\\n    readonly signature: string;\\n}\\n\\ndeclare var MSFIDOSignature: {\\n    prototype: MSFIDOSignature;\\n    new(): MSFIDOSignature;\\n};\\n\\ninterface MSFIDOSignatureAssertion extends MSAssertion {\\n    readonly signature: MSFIDOSignature;\\n}\\n\\ndeclare var MSFIDOSignatureAssertion: {\\n    prototype: MSFIDOSignatureAssertion;\\n    new(): MSFIDOSignatureAssertion;\\n};\\n\\ninterface MSFileSaver {\\n    msSaveBlob(blob: any, defaultName?: string): boolean;\\n    msSaveOrOpenBlob(blob: any, defaultName?: string): boolean;\\n}\\n\\ninterface MSGesture {\\n    target: Element;\\n    addPointer(pointerId: number): void;\\n    stop(): void;\\n}\\n\\ndeclare var MSGesture: {\\n    prototype: MSGesture;\\n    new(): MSGesture;\\n};\\n\\ninterface MSGestureEvent extends UIEvent {\\n    readonly clientX: number;\\n    readonly clientY: number;\\n    readonly expansion: number;\\n    readonly gestureObject: any;\\n    readonly hwTimestamp: number;\\n    readonly offsetX: number;\\n    readonly offsetY: number;\\n    readonly rotation: number;\\n    readonly scale: number;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly translationX: number;\\n    readonly translationY: number;\\n    readonly velocityAngular: number;\\n    readonly velocityExpansion: number;\\n    readonly velocityX: number;\\n    readonly velocityY: number;\\n    initGestureEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, offsetXArg: number, offsetYArg: number, translationXArg: number, translationYArg: number, scaleArg: number, expansionArg: number, rotationArg: number, velocityXArg: number, velocityYArg: number, velocityExpansionArg: number, velocityAngularArg: number, hwTimestampArg: number): void;\\n    readonly MSGESTURE_FLAG_BEGIN: number;\\n    readonly MSGESTURE_FLAG_CANCEL: number;\\n    readonly MSGESTURE_FLAG_END: number;\\n    readonly MSGESTURE_FLAG_INERTIA: number;\\n    readonly MSGESTURE_FLAG_NONE: number;\\n}\\n\\ndeclare var MSGestureEvent: {\\n    prototype: MSGestureEvent;\\n    new(): MSGestureEvent;\\n    readonly MSGESTURE_FLAG_BEGIN: number;\\n    readonly MSGESTURE_FLAG_CANCEL: number;\\n    readonly MSGESTURE_FLAG_END: number;\\n    readonly MSGESTURE_FLAG_INERTIA: number;\\n    readonly MSGESTURE_FLAG_NONE: number;\\n};\\n\\ninterface MSGraphicsTrust {\\n    readonly constrictionActive: boolean;\\n    readonly status: string;\\n}\\n\\ndeclare var MSGraphicsTrust: {\\n    prototype: MSGraphicsTrust;\\n    new(): MSGraphicsTrust;\\n};\\n\\ninterface MSInputMethodContextEventMap {\\n    \"MSCandidateWindowHide\": Event;\\n    \"MSCandidateWindowShow\": Event;\\n    \"MSCandidateWindowUpdate\": Event;\\n}\\n\\ninterface MSInputMethodContext extends EventTarget {\\n    readonly compositionEndOffset: number;\\n    readonly compositionStartOffset: number;\\n    oncandidatewindowhide: ((this: MSInputMethodContext, ev: Event) => any) | null;\\n    oncandidatewindowshow: ((this: MSInputMethodContext, ev: Event) => any) | null;\\n    oncandidatewindowupdate: ((this: MSInputMethodContext, ev: Event) => any) | null;\\n    readonly target: HTMLElement;\\n    getCandidateWindowClientRect(): ClientRect;\\n    getCompositionAlternatives(): string[];\\n    hasComposition(): boolean;\\n    isCandidateWindowVisible(): boolean;\\n    addEventListener<K extends keyof MSInputMethodContextEventMap>(type: K, listener: (this: MSInputMethodContext, ev: MSInputMethodContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MSInputMethodContextEventMap>(type: K, listener: (this: MSInputMethodContext, ev: MSInputMethodContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MSInputMethodContext: {\\n    prototype: MSInputMethodContext;\\n    new(): MSInputMethodContext;\\n};\\n\\ninterface MSMediaKeyError {\\n    readonly code: number;\\n    readonly systemCode: number;\\n    readonly MS_MEDIA_KEYERR_CLIENT: number;\\n    readonly MS_MEDIA_KEYERR_DOMAIN: number;\\n    readonly MS_MEDIA_KEYERR_HARDWARECHANGE: number;\\n    readonly MS_MEDIA_KEYERR_OUTPUT: number;\\n    readonly MS_MEDIA_KEYERR_SERVICE: number;\\n    readonly MS_MEDIA_KEYERR_UNKNOWN: number;\\n}\\n\\ndeclare var MSMediaKeyError: {\\n    prototype: MSMediaKeyError;\\n    new(): MSMediaKeyError;\\n    readonly MS_MEDIA_KEYERR_CLIENT: number;\\n    readonly MS_MEDIA_KEYERR_DOMAIN: number;\\n    readonly MS_MEDIA_KEYERR_HARDWARECHANGE: number;\\n    readonly MS_MEDIA_KEYERR_OUTPUT: number;\\n    readonly MS_MEDIA_KEYERR_SERVICE: number;\\n    readonly MS_MEDIA_KEYERR_UNKNOWN: number;\\n};\\n\\ninterface MSMediaKeyMessageEvent extends Event {\\n    readonly destinationURL: string | null;\\n    readonly message: Uint8Array;\\n}\\n\\ndeclare var MSMediaKeyMessageEvent: {\\n    prototype: MSMediaKeyMessageEvent;\\n    new(): MSMediaKeyMessageEvent;\\n};\\n\\ninterface MSMediaKeyNeededEvent extends Event {\\n    readonly initData: Uint8Array | null;\\n}\\n\\ndeclare var MSMediaKeyNeededEvent: {\\n    prototype: MSMediaKeyNeededEvent;\\n    new(): MSMediaKeyNeededEvent;\\n};\\n\\ninterface MSMediaKeySession extends EventTarget {\\n    readonly error: MSMediaKeyError | null;\\n    readonly keySystem: string;\\n    readonly sessionId: string;\\n    close(): void;\\n    update(key: Uint8Array): void;\\n}\\n\\ndeclare var MSMediaKeySession: {\\n    prototype: MSMediaKeySession;\\n    new(): MSMediaKeySession;\\n};\\n\\ninterface MSMediaKeys {\\n    readonly keySystem: string;\\n    createSession(type: string, initData: Uint8Array, cdmData?: Uint8Array | null): MSMediaKeySession;\\n}\\n\\ndeclare var MSMediaKeys: {\\n    prototype: MSMediaKeys;\\n    new(keySystem: string): MSMediaKeys;\\n    isTypeSupported(keySystem: string, type?: string | null): boolean;\\n    isTypeSupportedWithFeatures(keySystem: string, type?: string | null): string;\\n};\\n\\ninterface MSNavigatorDoNotTrack {\\n    confirmSiteSpecificTrackingException(args: ConfirmSiteSpecificExceptionsInformation): boolean;\\n    confirmWebWideTrackingException(args: ExceptionInformation): boolean;\\n    removeSiteSpecificTrackingException(args: ExceptionInformation): void;\\n    removeWebWideTrackingException(args: ExceptionInformation): void;\\n    storeSiteSpecificTrackingException(args: StoreSiteSpecificExceptionsInformation): void;\\n    storeWebWideTrackingException(args: StoreExceptionsInformation): void;\\n}\\n\\ninterface MSPointerEvent extends MouseEvent {\\n    readonly currentPoint: any;\\n    readonly height: number;\\n    readonly hwTimestamp: number;\\n    readonly intermediatePoints: any;\\n    readonly isPrimary: boolean;\\n    readonly pointerId: number;\\n    readonly pointerType: any;\\n    readonly pressure: number;\\n    readonly rotation: number;\\n    readonly tiltX: number;\\n    readonly tiltY: number;\\n    readonly width: number;\\n    getCurrentPoint(element: Element): void;\\n    getIntermediatePoints(element: Element): void;\\n    initPointerEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, ctrlKeyArg: boolean, altKeyArg: boolean, shiftKeyArg: boolean, metaKeyArg: boolean, buttonArg: number, relatedTargetArg: EventTarget, offsetXArg: number, offsetYArg: number, widthArg: number, heightArg: number, pressure: number, rotation: number, tiltX: number, tiltY: number, pointerIdArg: number, pointerType: any, hwTimestampArg: number, isPrimary: boolean): void;\\n}\\n\\ndeclare var MSPointerEvent: {\\n    prototype: MSPointerEvent;\\n    new(typeArg: string, eventInitDict?: PointerEventInit): MSPointerEvent;\\n};\\n\\ninterface MSStream {\\n    readonly type: string;\\n    msClose(): void;\\n    msDetachStream(): any;\\n}\\n\\ndeclare var MSStream: {\\n    prototype: MSStream;\\n    new(): MSStream;\\n};\\n\\ninterface MediaDeviceInfo {\\n    readonly deviceId: string;\\n    readonly groupId: string;\\n    readonly kind: MediaDeviceKind;\\n    readonly label: string;\\n}\\n\\ndeclare var MediaDeviceInfo: {\\n    prototype: MediaDeviceInfo;\\n    new(): MediaDeviceInfo;\\n};\\n\\ninterface MediaDevicesEventMap {\\n    \"devicechange\": Event;\\n}\\n\\ninterface MediaDevices extends EventTarget {\\n    ondevicechange: ((this: MediaDevices, ev: Event) => any) | null;\\n    enumerateDevices(): Promise<MediaDeviceInfo[]>;\\n    getSupportedConstraints(): MediaTrackSupportedConstraints;\\n    getUserMedia(constraints: MediaStreamConstraints): Promise<MediaStream>;\\n    addEventListener<K extends keyof MediaDevicesEventMap>(type: K, listener: (this: MediaDevices, ev: MediaDevicesEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaDevicesEventMap>(type: K, listener: (this: MediaDevices, ev: MediaDevicesEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaDevices: {\\n    prototype: MediaDevices;\\n    new(): MediaDevices;\\n};\\n\\ninterface MediaElementAudioSourceNode extends AudioNode {\\n    readonly mediaElement: HTMLMediaElement;\\n}\\n\\ndeclare var MediaElementAudioSourceNode: {\\n    prototype: MediaElementAudioSourceNode;\\n    new(context: AudioContext, options: MediaElementAudioSourceOptions): MediaElementAudioSourceNode;\\n};\\n\\ninterface MediaEncryptedEvent extends Event {\\n    readonly initData: ArrayBuffer | null;\\n    readonly initDataType: string;\\n}\\n\\ndeclare var MediaEncryptedEvent: {\\n    prototype: MediaEncryptedEvent;\\n    new(type: string, eventInitDict?: MediaEncryptedEventInit): MediaEncryptedEvent;\\n};\\n\\ninterface MediaError {\\n    readonly code: number;\\n    readonly message: string;\\n    readonly msExtendedCode: number;\\n    readonly MEDIA_ERR_ABORTED: number;\\n    readonly MEDIA_ERR_DECODE: number;\\n    readonly MEDIA_ERR_NETWORK: number;\\n    readonly MEDIA_ERR_SRC_NOT_SUPPORTED: number;\\n    readonly MS_MEDIA_ERR_ENCRYPTED: number;\\n}\\n\\ndeclare var MediaError: {\\n    prototype: MediaError;\\n    new(): MediaError;\\n    readonly MEDIA_ERR_ABORTED: number;\\n    readonly MEDIA_ERR_DECODE: number;\\n    readonly MEDIA_ERR_NETWORK: number;\\n    readonly MEDIA_ERR_SRC_NOT_SUPPORTED: number;\\n    readonly MS_MEDIA_ERR_ENCRYPTED: number;\\n};\\n\\ninterface MediaKeyMessageEvent extends Event {\\n    readonly message: ArrayBuffer;\\n    readonly messageType: MediaKeyMessageType;\\n}\\n\\ndeclare var MediaKeyMessageEvent: {\\n    prototype: MediaKeyMessageEvent;\\n    new(type: string, eventInitDict?: MediaKeyMessageEventInit): MediaKeyMessageEvent;\\n};\\n\\ninterface MediaKeySession extends EventTarget {\\n    readonly closed: Promise<void>;\\n    readonly expiration: number;\\n    readonly keyStatuses: MediaKeyStatusMap;\\n    readonly sessionId: string;\\n    close(): Promise<void>;\\n    generateRequest(initDataType: string, initData: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): Promise<void>;\\n    load(sessionId: string): Promise<boolean>;\\n    remove(): Promise<void>;\\n    update(response: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): Promise<void>;\\n}\\n\\ndeclare var MediaKeySession: {\\n    prototype: MediaKeySession;\\n    new(): MediaKeySession;\\n};\\n\\ninterface MediaKeyStatusMap {\\n    readonly size: number;\\n    forEach(callback: Function, thisArg?: any): void;\\n    get(keyId: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): MediaKeyStatus;\\n    has(keyId: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): boolean;\\n}\\n\\ndeclare var MediaKeyStatusMap: {\\n    prototype: MediaKeyStatusMap;\\n    new(): MediaKeyStatusMap;\\n};\\n\\ninterface MediaKeySystemAccess {\\n    readonly keySystem: string;\\n    createMediaKeys(): Promise<MediaKeys>;\\n    getConfiguration(): MediaKeySystemConfiguration;\\n}\\n\\ndeclare var MediaKeySystemAccess: {\\n    prototype: MediaKeySystemAccess;\\n    new(): MediaKeySystemAccess;\\n};\\n\\ninterface MediaKeys {\\n    createSession(sessionType?: MediaKeySessionType): MediaKeySession;\\n    setServerCertificate(serverCertificate: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null): Promise<void>;\\n}\\n\\ndeclare var MediaKeys: {\\n    prototype: MediaKeys;\\n    new(): MediaKeys;\\n};\\n\\ninterface MediaList {\\n    readonly length: number;\\n    mediaText: string;\\n    appendMedium(medium: string): void;\\n    deleteMedium(medium: string): void;\\n    item(index: number): string | null;\\n    toString(): number;\\n    [index: number]: string;\\n}\\n\\ndeclare var MediaList: {\\n    prototype: MediaList;\\n    new(): MediaList;\\n};\\n\\ninterface MediaQueryListEventMap {\\n    \"change\": MediaQueryListEvent;\\n}\\n\\ninterface MediaQueryList extends EventTarget {\\n    readonly matches: boolean;\\n    readonly media: string;\\n    onchange: ((this: MediaQueryList, ev: MediaQueryListEvent) => any) | null;\\n    /** @deprecated */\\n    addListener(listener: ((this: MediaQueryList, ev: MediaQueryListEvent) => any) | null): void;\\n    /** @deprecated */\\n    removeListener(listener: EventListenerOrEventListenerObject | null): void;\\n    addEventListener<K extends keyof MediaQueryListEventMap>(type: K, listener: (this: MediaQueryList, ev: MediaQueryListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaQueryListEventMap>(type: K, listener: (this: MediaQueryList, ev: MediaQueryListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaQueryList: {\\n    prototype: MediaQueryList;\\n    new(): MediaQueryList;\\n};\\n\\ninterface MediaQueryListEvent extends Event {\\n    readonly matches: boolean;\\n    readonly media: string;\\n}\\n\\ndeclare var MediaQueryListEvent: {\\n    prototype: MediaQueryListEvent;\\n    new(type: string, eventInitDict?: MediaQueryListEventInit): MediaQueryListEvent;\\n};\\n\\ninterface MediaSource extends EventTarget {\\n    readonly activeSourceBuffers: SourceBufferList;\\n    duration: number;\\n    readonly readyState: ReadyState;\\n    readonly sourceBuffers: SourceBufferList;\\n    addSourceBuffer(type: string): SourceBuffer;\\n    endOfStream(error?: EndOfStreamError): void;\\n    removeSourceBuffer(sourceBuffer: SourceBuffer): void;\\n}\\n\\ndeclare var MediaSource: {\\n    prototype: MediaSource;\\n    new(): MediaSource;\\n    isTypeSupported(type: string): boolean;\\n};\\n\\ninterface MediaStreamEventMap {\\n    \"active\": Event;\\n    \"addtrack\": MediaStreamTrackEvent;\\n    \"inactive\": Event;\\n    \"removetrack\": MediaStreamTrackEvent;\\n}\\n\\ninterface MediaStream extends EventTarget {\\n    readonly active: boolean;\\n    readonly id: string;\\n    onactive: ((this: MediaStream, ev: Event) => any) | null;\\n    onaddtrack: ((this: MediaStream, ev: MediaStreamTrackEvent) => any) | null;\\n    oninactive: ((this: MediaStream, ev: Event) => any) | null;\\n    onremovetrack: ((this: MediaStream, ev: MediaStreamTrackEvent) => any) | null;\\n    addTrack(track: MediaStreamTrack): void;\\n    clone(): MediaStream;\\n    getAudioTracks(): MediaStreamTrack[];\\n    getTrackById(trackId: string): MediaStreamTrack | null;\\n    getTracks(): MediaStreamTrack[];\\n    getVideoTracks(): MediaStreamTrack[];\\n    removeTrack(track: MediaStreamTrack): void;\\n    stop(): void;\\n    addEventListener<K extends keyof MediaStreamEventMap>(type: K, listener: (this: MediaStream, ev: MediaStreamEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaStreamEventMap>(type: K, listener: (this: MediaStream, ev: MediaStreamEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaStream: {\\n    prototype: MediaStream;\\n    new(): MediaStream;\\n    new(stream: MediaStream): MediaStream;\\n    new(tracks: MediaStreamTrack[]): MediaStream;\\n};\\n\\ninterface MediaStreamAudioDestinationNode extends AudioNode {\\n    readonly stream: MediaStream;\\n}\\n\\ndeclare var MediaStreamAudioDestinationNode: {\\n    prototype: MediaStreamAudioDestinationNode;\\n    new(context: AudioContext, options?: AudioNodeOptions): MediaStreamAudioDestinationNode;\\n};\\n\\ninterface MediaStreamAudioSourceNode extends AudioNode {\\n    readonly mediaStream: MediaStream;\\n}\\n\\ndeclare var MediaStreamAudioSourceNode: {\\n    prototype: MediaStreamAudioSourceNode;\\n    new(context: AudioContext, options: MediaStreamAudioSourceOptions): MediaStreamAudioSourceNode;\\n};\\n\\ninterface MediaStreamError {\\n    readonly constraintName: string | null;\\n    readonly message: string | null;\\n    readonly name: string;\\n}\\n\\ndeclare var MediaStreamError: {\\n    prototype: MediaStreamError;\\n    new(): MediaStreamError;\\n};\\n\\ninterface MediaStreamErrorEvent extends Event {\\n    readonly error: MediaStreamError | null;\\n}\\n\\ndeclare var MediaStreamErrorEvent: {\\n    prototype: MediaStreamErrorEvent;\\n    new(typeArg: string, eventInitDict?: MediaStreamErrorEventInit): MediaStreamErrorEvent;\\n};\\n\\ninterface MediaStreamEvent extends Event {\\n    readonly stream: MediaStream | null;\\n}\\n\\ndeclare var MediaStreamEvent: {\\n    prototype: MediaStreamEvent;\\n    new(type: string, eventInitDict: MediaStreamEventInit): MediaStreamEvent;\\n};\\n\\ninterface MediaStreamTrackEventMap {\\n    \"ended\": MediaStreamErrorEvent;\\n    \"isolationchange\": Event;\\n    \"mute\": Event;\\n    \"overconstrained\": MediaStreamErrorEvent;\\n    \"unmute\": Event;\\n}\\n\\ninterface MediaStreamTrack extends EventTarget {\\n    enabled: boolean;\\n    readonly id: string;\\n    readonly isolated: boolean;\\n    readonly kind: string;\\n    readonly label: string;\\n    readonly muted: boolean;\\n    onended: ((this: MediaStreamTrack, ev: MediaStreamErrorEvent) => any) | null;\\n    onisolationchange: ((this: MediaStreamTrack, ev: Event) => any) | null;\\n    onmute: ((this: MediaStreamTrack, ev: Event) => any) | null;\\n    onoverconstrained: ((this: MediaStreamTrack, ev: MediaStreamErrorEvent) => any) | null;\\n    onunmute: ((this: MediaStreamTrack, ev: Event) => any) | null;\\n    readonly readonly: boolean;\\n    readonly readyState: MediaStreamTrackState;\\n    readonly remote: boolean;\\n    applyConstraints(constraints: MediaTrackConstraints): Promise<void>;\\n    clone(): MediaStreamTrack;\\n    getCapabilities(): MediaTrackCapabilities;\\n    getConstraints(): MediaTrackConstraints;\\n    getSettings(): MediaTrackSettings;\\n    stop(): void;\\n    addEventListener<K extends keyof MediaStreamTrackEventMap>(type: K, listener: (this: MediaStreamTrack, ev: MediaStreamTrackEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MediaStreamTrackEventMap>(type: K, listener: (this: MediaStreamTrack, ev: MediaStreamTrackEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MediaStreamTrack: {\\n    prototype: MediaStreamTrack;\\n    new(): MediaStreamTrack;\\n};\\n\\ninterface MediaStreamTrackAudioSourceNode extends AudioNode {\\n}\\n\\ndeclare var MediaStreamTrackAudioSourceNode: {\\n    prototype: MediaStreamTrackAudioSourceNode;\\n    new(context: AudioContext, options: MediaStreamTrackAudioSourceOptions): MediaStreamTrackAudioSourceNode;\\n};\\n\\ninterface MediaStreamTrackEvent extends Event {\\n    readonly track: MediaStreamTrack;\\n}\\n\\ndeclare var MediaStreamTrackEvent: {\\n    prototype: MediaStreamTrackEvent;\\n    new(typeArg: string, eventInitDict?: MediaStreamTrackEventInit): MediaStreamTrackEvent;\\n};\\n\\ninterface MessageChannel {\\n    readonly port1: MessagePort;\\n    readonly port2: MessagePort;\\n}\\n\\ndeclare var MessageChannel: {\\n    prototype: MessageChannel;\\n    new(): MessageChannel;\\n};\\n\\ninterface MessageEvent extends Event {\\n    /**\\n     * Returns the data of the message.\\n     */\\n    readonly data: any;\\n    /**\\n     * Returns the last event ID string, for\\n     * server-sent events.\\n     */\\n    readonly lastEventId: string;\\n    /**\\n     * Returns the origin of the message, for server-sent events and\\n     * cross-document messaging.\\n     */\\n    readonly origin: string;\\n    /**\\n     * Returns the MessagePort array sent with the message, for cross-document\\n     * messaging and channel messaging.\\n     */\\n    readonly ports: ReadonlyArray<MessagePort>;\\n    /**\\n     * Returns the WindowProxy of the source window, for cross-document\\n     * messaging, and the MessagePort being attached, in the connect event fired at\\n     * SharedWorkerGlobalScope objects.\\n     */\\n    readonly source: MessageEventSource | null;\\n}\\n\\ndeclare var MessageEvent: {\\n    prototype: MessageEvent;\\n    new(type: string, eventInitDict?: MessageEventInit): MessageEvent;\\n};\\n\\ninterface MessagePortEventMap {\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n}\\n\\ninterface MessagePort extends EventTarget {\\n    onmessage: ((this: MessagePort, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: MessagePort, ev: MessageEvent) => any) | null;\\n    /**\\n     * Disconnects the port, so that it is no longer active.\\n     */\\n    close(): void;\\n    /**\\n     * Posts a message through the channel. Objects listed in transfer are\\n     * transferred, not just cloned, meaning that they are no longer usable on the sending side.\\n     * Throws a \"DataCloneError\" DOMException if\\n     * transfer contains duplicate objects or port, or if message\\n     * could not be cloned.\\n     */\\n    postMessage(message: any, transfer?: Transferable[]): void;\\n    /**\\n     * Begins dispatching messages received on the port.\\n     */\\n    start(): void;\\n    addEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var MessagePort: {\\n    prototype: MessagePort;\\n    new(): MessagePort;\\n};\\n\\ninterface MimeType {\\n    readonly description: string;\\n    readonly enabledPlugin: Plugin;\\n    readonly suffixes: string;\\n    readonly type: string;\\n}\\n\\ndeclare var MimeType: {\\n    prototype: MimeType;\\n    new(): MimeType;\\n};\\n\\ninterface MimeTypeArray {\\n    readonly length: number;\\n    item(index: number): Plugin;\\n    namedItem(type: string): Plugin;\\n    [index: number]: Plugin;\\n}\\n\\ndeclare var MimeTypeArray: {\\n    prototype: MimeTypeArray;\\n    new(): MimeTypeArray;\\n};\\n\\ninterface MouseEvent extends UIEvent {\\n    readonly altKey: boolean;\\n    readonly button: number;\\n    readonly buttons: number;\\n    readonly clientX: number;\\n    readonly clientY: number;\\n    readonly ctrlKey: boolean;\\n    /** @deprecated */\\n    readonly fromElement: Element;\\n    readonly layerX: number;\\n    readonly layerY: number;\\n    readonly metaKey: boolean;\\n    readonly movementX: number;\\n    readonly movementY: number;\\n    readonly offsetX: number;\\n    readonly offsetY: number;\\n    readonly pageX: number;\\n    readonly pageY: number;\\n    readonly relatedTarget: EventTarget;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly shiftKey: boolean;\\n    /** @deprecated */\\n    readonly toElement: Element;\\n    /** @deprecated */\\n    readonly which: number;\\n    readonly x: number;\\n    readonly y: number;\\n    getModifierState(keyArg: string): boolean;\\n    initMouseEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, ctrlKeyArg: boolean, altKeyArg: boolean, shiftKeyArg: boolean, metaKeyArg: boolean, buttonArg: number, relatedTargetArg: EventTarget | null): void;\\n}\\n\\ndeclare var MouseEvent: {\\n    prototype: MouseEvent;\\n    new(typeArg: string, eventInitDict?: MouseEventInit): MouseEvent;\\n};\\n\\ninterface MutationEvent extends Event {\\n    readonly attrChange: number;\\n    readonly attrName: string;\\n    readonly newValue: string;\\n    readonly prevValue: string;\\n    readonly relatedNode: Node;\\n    initMutationEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, relatedNodeArg: Node, prevValueArg: string, newValueArg: string, attrNameArg: string, attrChangeArg: number): void;\\n    readonly ADDITION: number;\\n    readonly MODIFICATION: number;\\n    readonly REMOVAL: number;\\n}\\n\\ndeclare var MutationEvent: {\\n    prototype: MutationEvent;\\n    new(): MutationEvent;\\n    readonly ADDITION: number;\\n    readonly MODIFICATION: number;\\n    readonly REMOVAL: number;\\n};\\n\\ninterface MutationObserver {\\n    disconnect(): void;\\n    /**\\n     * Instructs the user agent to observe a given target (a node) and report any mutations based on\\n     * the criteria given by options (an object).\\n     * The options argument allows for setting mutation\\n     * observation options via object members. These are the object members that\\n     * can be used:\\n     * childList\\n     * Set to true if mutations to target\\'s children are to be observed.\\n     * attributes\\n     * Set to true if mutations to target\\'s attributes are to be observed. Can be omitted if attributeOldValue or attributeFilter is\\n     * specified.\\n     * characterData\\n     * Set to true if mutations to target\\'s data are to be observed. Can be omitted if characterDataOldValue is specified.\\n     * subtree\\n     * Set to true if mutations to not just target, but\\n     * also target\\'s descendants are to be\\n     * observed.\\n     * attributeOldValue\\n     * Set to true if attributes is true or omitted\\n     * and target\\'s attribute value before the mutation\\n     * needs to be recorded.\\n     * characterDataOldValue\\n     * Set to true if characterData is set to true or omitted and target\\'s data before the mutation\\n     * needs to be recorded.\\n     * attributeFilter\\n     * Set to a list of attribute local names (without namespace) if not all attribute mutations need to be\\n     * observed and attributes is true\\n     * or omitted.\\n     */\\n    observe(target: Node, options?: MutationObserverInit): void;\\n    /**\\n     * Empties the record queue and\\n     * returns what was in there.\\n     */\\n    takeRecords(): MutationRecord[];\\n}\\n\\ndeclare var MutationObserver: {\\n    prototype: MutationObserver;\\n    new(callback: MutationCallback): MutationObserver;\\n};\\n\\ninterface MutationRecord {\\n    readonly addedNodes: NodeList;\\n    /**\\n     * Returns the local name of the\\n     * changed attribute, and null otherwise.\\n     */\\n    readonly attributeName: string | null;\\n    /**\\n     * Returns the namespace of the\\n     * changed attribute, and null otherwise.\\n     */\\n    readonly attributeNamespace: string | null;\\n    /**\\n     * Return the previous and next sibling respectively\\n     * of the added or removed nodes, and null otherwise.\\n     */\\n    readonly nextSibling: Node | null;\\n    /**\\n     * The return value depends on type. For\\n     * \"attributes\", it is the value of the\\n     * changed attribute before the change.\\n     * For \"characterData\", it is the data of the changed node before the change. For\\n     * \"childList\", it is null.\\n     */\\n    readonly oldValue: string | null;\\n    readonly previousSibling: Node | null;\\n    /**\\n     * Return the nodes added and removed\\n     * respectively.\\n     */\\n    readonly removedNodes: NodeList;\\n    readonly target: Node;\\n    /**\\n     * Returns \"attributes\" if it was an attribute mutation.\\n     * \"characterData\" if it was a mutation to a CharacterData node. And\\n     * \"childList\" if it was a mutation to the tree of nodes.\\n     */\\n    readonly type: MutationRecordType;\\n}\\n\\ndeclare var MutationRecord: {\\n    prototype: MutationRecord;\\n    new(): MutationRecord;\\n};\\n\\ninterface NamedNodeMap {\\n    readonly length: number;\\n    getNamedItem(qualifiedName: string): Attr | null;\\n    getNamedItemNS(namespace: string | null, localName: string): Attr | null;\\n    item(index: number): Attr | null;\\n    removeNamedItem(qualifiedName: string): Attr;\\n    removeNamedItemNS(namespace: string | null, localName: string): Attr;\\n    setNamedItem(attr: Attr): Attr | null;\\n    setNamedItemNS(attr: Attr): Attr | null;\\n    [index: number]: Attr;\\n}\\n\\ndeclare var NamedNodeMap: {\\n    prototype: NamedNodeMap;\\n    new(): NamedNodeMap;\\n};\\n\\ninterface NavigationPreloadManager {\\n    disable(): Promise<void>;\\n    enable(): Promise<void>;\\n    getState(): Promise<NavigationPreloadState>;\\n    setHeaderValue(value: string): Promise<void>;\\n}\\n\\ndeclare var NavigationPreloadManager: {\\n    prototype: NavigationPreloadManager;\\n    new(): NavigationPreloadManager;\\n};\\n\\ninterface Navigator extends NavigatorID, NavigatorOnLine, NavigatorContentUtils, NavigatorStorageUtils, MSNavigatorDoNotTrack, MSFileSaver, NavigatorBeacon, NavigatorConcurrentHardware, NavigatorUserMedia, NavigatorLanguage, NavigatorStorage, NavigatorAutomationInformation {\\n    readonly activeVRDisplays: ReadonlyArray<VRDisplay>;\\n    readonly authentication: WebAuthentication;\\n    readonly cookieEnabled: boolean;\\n    readonly doNotTrack: string | null;\\n    gamepadInputEmulation: GamepadInputEmulationType;\\n    readonly geolocation: Geolocation;\\n    readonly maxTouchPoints: number;\\n    readonly mimeTypes: MimeTypeArray;\\n    readonly msManipulationViewsEnabled: boolean;\\n    readonly msMaxTouchPoints: number;\\n    readonly msPointerEnabled: boolean;\\n    readonly plugins: PluginArray;\\n    readonly pointerEnabled: boolean;\\n    readonly serviceWorker: ServiceWorkerContainer;\\n    readonly webdriver: boolean;\\n    getGamepads(): (Gamepad | null)[];\\n    getVRDisplays(): Promise<VRDisplay[]>;\\n    javaEnabled(): boolean;\\n    msLaunchUri(uri: string, successCallback?: MSLaunchUriCallback, noHandlerCallback?: MSLaunchUriCallback): void;\\n    requestMediaKeySystemAccess(keySystem: string, supportedConfigurations: MediaKeySystemConfiguration[]): Promise<MediaKeySystemAccess>;\\n    vibrate(pattern: number | number[]): boolean;\\n}\\n\\ndeclare var Navigator: {\\n    prototype: Navigator;\\n    new(): Navigator;\\n};\\n\\ninterface NavigatorAutomationInformation {\\n    readonly webdriver: boolean;\\n}\\n\\ninterface NavigatorBeacon {\\n    sendBeacon(url: string, data?: Blob | Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | FormData | string | null): boolean;\\n}\\n\\ninterface NavigatorConcurrentHardware {\\n    readonly hardwareConcurrency: number;\\n}\\n\\ninterface NavigatorContentUtils {\\n}\\n\\ninterface NavigatorID {\\n    readonly appCodeName: string;\\n    readonly appName: string;\\n    readonly appVersion: string;\\n    readonly platform: string;\\n    readonly product: string;\\n    readonly productSub: string;\\n    readonly userAgent: string;\\n    readonly vendor: string;\\n    readonly vendorSub: string;\\n}\\n\\ninterface NavigatorLanguage {\\n    readonly language: string;\\n    readonly languages: ReadonlyArray<string>;\\n}\\n\\ninterface NavigatorOnLine {\\n    readonly onLine: boolean;\\n}\\n\\ninterface NavigatorStorage {\\n    readonly storage: StorageManager;\\n}\\n\\ninterface NavigatorStorageUtils {\\n}\\n\\ninterface NavigatorUserMedia {\\n    readonly mediaDevices: MediaDevices;\\n    getDisplayMedia(constraints: MediaStreamConstraints): Promise<MediaStream>;\\n    getUserMedia(constraints: MediaStreamConstraints, successCallback: NavigatorUserMediaSuccessCallback, errorCallback: NavigatorUserMediaErrorCallback): void;\\n}\\n\\ninterface Node extends EventTarget {\\n    /**\\n     * Returns node\\'s node document\\'s document base URL.\\n     */\\n    readonly baseURI: string;\\n    /**\\n     * Returns the children.\\n     */\\n    readonly childNodes: NodeListOf<ChildNode>;\\n    /**\\n     * Returns the first child.\\n     */\\n    readonly firstChild: ChildNode | null;\\n    /**\\n     * Returns true if node is connected and false otherwise.\\n     */\\n    readonly isConnected: boolean;\\n    /**\\n     * Returns the last child.\\n     */\\n    readonly lastChild: ChildNode | null;\\n    readonly namespaceURI: string | null;\\n    /**\\n     * Returns the next sibling.\\n     */\\n    readonly nextSibling: Node | null;\\n    /**\\n     * Returns a string appropriate for the type of node, as\\n     * follows:\\n     * Element\\n     * Its HTML-uppercased qualified name.\\n     * Attr\\n     * Its qualified name.\\n     * Text\\n     * \"#text\".\\n     * CDATASection\\n     * \"#cdata-section\".\\n     * ProcessingInstruction\\n     * Its target.\\n     * Comment\\n     * \"#comment\".\\n     * Document\\n     * \"#document\".\\n     * DocumentType\\n     * Its name.\\n     * DocumentFragment\\n     * \"#document-fragment\".\\n     */\\n    readonly nodeName: string;\\n    readonly nodeType: number;\\n    nodeValue: string | null;\\n    /**\\n     * Returns the node document.\\n     * Returns null for documents.\\n     */\\n    readonly ownerDocument: Document | null;\\n    /**\\n     * Returns the parent element.\\n     */\\n    readonly parentElement: HTMLElement | null;\\n    /**\\n     * Returns the parent.\\n     */\\n    readonly parentNode: Node & ParentNode | null;\\n    /**\\n     * Returns the previous sibling.\\n     */\\n    readonly previousSibling: Node | null;\\n    textContent: string | null;\\n    appendChild<T extends Node>(newChild: T): T;\\n    /**\\n     * Returns a copy of node. If deep is true, the copy also includes the node\\'s descendants.\\n     */\\n    cloneNode(deep?: boolean): Node;\\n    compareDocumentPosition(other: Node): number;\\n    /**\\n     * Returns true if other is an inclusive descendant of node, and false otherwise.\\n     */\\n    contains(other: Node | null): boolean;\\n    /**\\n     * Returns node\\'s shadow-including root.\\n     */\\n    getRootNode(options?: GetRootNodeOptions): Node;\\n    /**\\n     * Returns whether node has children.\\n     */\\n    hasChildNodes(): boolean;\\n    insertBefore<T extends Node>(newChild: T, refChild: Node | null): T;\\n    isDefaultNamespace(namespace: string | null): boolean;\\n    /**\\n     * Returns whether node and otherNode have the same properties.\\n     */\\n    isEqualNode(otherNode: Node | null): boolean;\\n    isSameNode(otherNode: Node | null): boolean;\\n    lookupNamespaceURI(prefix: string | null): string | null;\\n    lookupPrefix(namespace: string | null): string | null;\\n    /**\\n     * Removes empty exclusive Text nodes and concatenates the data of remaining contiguous exclusive Text nodes into the first of their nodes.\\n     */\\n    normalize(): void;\\n    removeChild<T extends Node>(oldChild: T): T;\\n    replaceChild<T extends Node>(newChild: Node, oldChild: T): T;\\n    readonly ATTRIBUTE_NODE: number;\\n    readonly CDATA_SECTION_NODE: number;\\n    readonly COMMENT_NODE: number;\\n    readonly DOCUMENT_FRAGMENT_NODE: number;\\n    readonly DOCUMENT_NODE: number;\\n    readonly DOCUMENT_POSITION_CONTAINED_BY: number;\\n    readonly DOCUMENT_POSITION_CONTAINS: number;\\n    readonly DOCUMENT_POSITION_DISCONNECTED: number;\\n    readonly DOCUMENT_POSITION_FOLLOWING: number;\\n    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;\\n    readonly DOCUMENT_POSITION_PRECEDING: number;\\n    readonly DOCUMENT_TYPE_NODE: number;\\n    readonly ELEMENT_NODE: number;\\n    readonly ENTITY_NODE: number;\\n    readonly ENTITY_REFERENCE_NODE: number;\\n    readonly NOTATION_NODE: number;\\n    readonly PROCESSING_INSTRUCTION_NODE: number;\\n    readonly TEXT_NODE: number;\\n}\\n\\ndeclare var Node: {\\n    prototype: Node;\\n    new(): Node;\\n    readonly ATTRIBUTE_NODE: number;\\n    readonly CDATA_SECTION_NODE: number;\\n    readonly COMMENT_NODE: number;\\n    readonly DOCUMENT_FRAGMENT_NODE: number;\\n    readonly DOCUMENT_NODE: number;\\n    readonly DOCUMENT_POSITION_CONTAINED_BY: number;\\n    readonly DOCUMENT_POSITION_CONTAINS: number;\\n    readonly DOCUMENT_POSITION_DISCONNECTED: number;\\n    readonly DOCUMENT_POSITION_FOLLOWING: number;\\n    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;\\n    readonly DOCUMENT_POSITION_PRECEDING: number;\\n    readonly DOCUMENT_TYPE_NODE: number;\\n    readonly ELEMENT_NODE: number;\\n    readonly ENTITY_NODE: number;\\n    readonly ENTITY_REFERENCE_NODE: number;\\n    readonly NOTATION_NODE: number;\\n    readonly PROCESSING_INSTRUCTION_NODE: number;\\n    readonly TEXT_NODE: number;\\n};\\n\\ninterface NodeFilter {\\n    acceptNode(node: Node): number;\\n}\\n\\ndeclare var NodeFilter: {\\n    readonly FILTER_ACCEPT: number;\\n    readonly FILTER_REJECT: number;\\n    readonly FILTER_SKIP: number;\\n    readonly SHOW_ALL: number;\\n    readonly SHOW_ATTRIBUTE: number;\\n    readonly SHOW_CDATA_SECTION: number;\\n    readonly SHOW_COMMENT: number;\\n    readonly SHOW_DOCUMENT: number;\\n    readonly SHOW_DOCUMENT_FRAGMENT: number;\\n    readonly SHOW_DOCUMENT_TYPE: number;\\n    readonly SHOW_ELEMENT: number;\\n    readonly SHOW_ENTITY: number;\\n    readonly SHOW_ENTITY_REFERENCE: number;\\n    readonly SHOW_NOTATION: number;\\n    readonly SHOW_PROCESSING_INSTRUCTION: number;\\n    readonly SHOW_TEXT: number;\\n};\\n\\ninterface NodeIterator {\\n    readonly filter: NodeFilter | null;\\n    readonly pointerBeforeReferenceNode: boolean;\\n    readonly referenceNode: Node;\\n    readonly root: Node;\\n    readonly whatToShow: number;\\n    detach(): void;\\n    nextNode(): Node | null;\\n    previousNode(): Node | null;\\n}\\n\\ndeclare var NodeIterator: {\\n    prototype: NodeIterator;\\n    new(): NodeIterator;\\n};\\n\\ninterface NodeList {\\n    /**\\n     * Returns the number of nodes in the collection.\\n     */\\n    readonly length: number;\\n    /**\\n     * element = collection[index]\\n     */\\n    item(index: number): Node | null;\\n    /**\\n     * Performs the specified action for each node in an list.\\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the list.\\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    forEach(callbackfn: (value: Node, key: number, parent: NodeList) => void, thisArg?: any): void;\\n    [index: number]: Node;\\n}\\n\\ndeclare var NodeList: {\\n    prototype: NodeList;\\n    new(): NodeList;\\n};\\n\\ninterface NodeListOf<TNode extends Node> extends NodeList {\\n    length: number;\\n    item(index: number): TNode;\\n    /**\\n     * Performs the specified action for each node in an list.\\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the list.\\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\\n     */\\n    forEach(callbackfn: (value: TNode, key: number, parent: NodeListOf<TNode>) => void, thisArg?: any): void;\\n    [index: number]: TNode;\\n}\\n\\ninterface NodeSelector {\\n    querySelector<K extends keyof HTMLElementTagNameMap>(selectors: K): HTMLElementTagNameMap[K] | null;\\n    querySelector<K extends keyof SVGElementTagNameMap>(selectors: K): SVGElementTagNameMap[K] | null;\\n    querySelector<E extends Element = Element>(selectors: string): E | null;\\n    querySelectorAll<K extends keyof HTMLElementTagNameMap>(selectors: K): NodeListOf<HTMLElementTagNameMap[K]>;\\n    querySelectorAll<K extends keyof SVGElementTagNameMap>(selectors: K): NodeListOf<SVGElementTagNameMap[K]>;\\n    querySelectorAll<E extends Element = Element>(selectors: string): NodeListOf<E>;\\n}\\n\\ninterface NonDocumentTypeChildNode {\\n    /**\\n     * Returns the first following sibling that\\n     * is an element, and null otherwise.\\n     */\\n    readonly nextElementSibling: Element | null;\\n    /**\\n     * Returns the first preceding sibling that\\n     * is an element, and null otherwise.\\n     */\\n    readonly previousElementSibling: Element | null;\\n}\\n\\ninterface NonElementParentNode {\\n    /**\\n     * Returns the first element within node\\'s descendants whose ID is elementId.\\n     */\\n    getElementById(elementId: string): Element | null;\\n}\\n\\ninterface NotificationEventMap {\\n    \"click\": Event;\\n    \"close\": Event;\\n    \"error\": Event;\\n    \"show\": Event;\\n}\\n\\ninterface Notification extends EventTarget {\\n    readonly actions: ReadonlyArray<NotificationAction>;\\n    readonly badge: string;\\n    readonly body: string;\\n    readonly data: any;\\n    readonly dir: NotificationDirection;\\n    readonly icon: string;\\n    readonly image: string;\\n    readonly lang: string;\\n    onclick: ((this: Notification, ev: Event) => any) | null;\\n    onclose: ((this: Notification, ev: Event) => any) | null;\\n    onerror: ((this: Notification, ev: Event) => any) | null;\\n    onshow: ((this: Notification, ev: Event) => any) | null;\\n    readonly renotify: boolean;\\n    readonly requireInteraction: boolean;\\n    readonly silent: boolean;\\n    readonly tag: string;\\n    readonly timestamp: number;\\n    readonly title: string;\\n    readonly vibrate: ReadonlyArray<number>;\\n    close(): void;\\n    addEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Notification: {\\n    prototype: Notification;\\n    new(title: string, options?: NotificationOptions): Notification;\\n    readonly maxActions: number;\\n    readonly permission: NotificationPermission;\\n    requestPermission(deprecatedCallback?: NotificationPermissionCallback): Promise<NotificationPermission>;\\n};\\n\\ninterface OES_element_index_uint {\\n}\\n\\ndeclare var OES_element_index_uint: {\\n    prototype: OES_element_index_uint;\\n    new(): OES_element_index_uint;\\n};\\n\\ninterface OES_standard_derivatives {\\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT_OES: number;\\n}\\n\\ndeclare var OES_standard_derivatives: {\\n    prototype: OES_standard_derivatives;\\n    new(): OES_standard_derivatives;\\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT_OES: number;\\n};\\n\\ninterface OES_texture_float {\\n}\\n\\ndeclare var OES_texture_float: {\\n    prototype: OES_texture_float;\\n    new(): OES_texture_float;\\n};\\n\\ninterface OES_texture_float_linear {\\n}\\n\\ndeclare var OES_texture_float_linear: {\\n    prototype: OES_texture_float_linear;\\n    new(): OES_texture_float_linear;\\n};\\n\\ninterface OES_texture_half_float {\\n    readonly HALF_FLOAT_OES: number;\\n}\\n\\ndeclare var OES_texture_half_float: {\\n    prototype: OES_texture_half_float;\\n    new(): OES_texture_half_float;\\n    readonly HALF_FLOAT_OES: number;\\n};\\n\\ninterface OES_texture_half_float_linear {\\n}\\n\\ndeclare var OES_texture_half_float_linear: {\\n    prototype: OES_texture_half_float_linear;\\n    new(): OES_texture_half_float_linear;\\n};\\n\\ninterface OES_vertex_array_object {\\n    readonly VERTEX_ARRAY_BINDING_OES: number;\\n    bindVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES): void;\\n    createVertexArrayOES(): WebGLVertexArrayObjectOES;\\n    deleteVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES): void;\\n    isVertexArrayOES(value: any): value is WebGLVertexArrayObjectOES;\\n}\\n\\ninterface OfflineAudioCompletionEvent extends Event {\\n    readonly renderedBuffer: AudioBuffer;\\n}\\n\\ndeclare var OfflineAudioCompletionEvent: {\\n    prototype: OfflineAudioCompletionEvent;\\n    new(type: string, eventInitDict: OfflineAudioCompletionEventInit): OfflineAudioCompletionEvent;\\n};\\n\\ninterface OfflineAudioContextEventMap extends BaseAudioContextEventMap {\\n    \"complete\": OfflineAudioCompletionEvent;\\n}\\n\\ninterface OfflineAudioContext extends BaseAudioContext {\\n    readonly length: number;\\n    oncomplete: ((this: OfflineAudioContext, ev: OfflineAudioCompletionEvent) => any) | null;\\n    startRendering(): Promise<AudioBuffer>;\\n    suspend(suspendTime: number): Promise<void>;\\n    addEventListener<K extends keyof OfflineAudioContextEventMap>(type: K, listener: (this: OfflineAudioContext, ev: OfflineAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof OfflineAudioContextEventMap>(type: K, listener: (this: OfflineAudioContext, ev: OfflineAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var OfflineAudioContext: {\\n    prototype: OfflineAudioContext;\\n    new(contextOptions: OfflineAudioContextOptions): OfflineAudioContext;\\n    new(numberOfChannels: number, length: number, sampleRate: number): OfflineAudioContext;\\n};\\n\\ninterface OscillatorNode extends AudioScheduledSourceNode {\\n    readonly detune: AudioParam;\\n    readonly frequency: AudioParam;\\n    type: OscillatorType;\\n    setPeriodicWave(periodicWave: PeriodicWave): void;\\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: OscillatorNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: OscillatorNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var OscillatorNode: {\\n    prototype: OscillatorNode;\\n    new(context: BaseAudioContext, options?: OscillatorOptions): OscillatorNode;\\n};\\n\\ninterface OverflowEvent extends UIEvent {\\n    readonly horizontalOverflow: boolean;\\n    readonly orient: number;\\n    readonly verticalOverflow: boolean;\\n    readonly BOTH: number;\\n    readonly HORIZONTAL: number;\\n    readonly VERTICAL: number;\\n}\\n\\ndeclare var OverflowEvent: {\\n    prototype: OverflowEvent;\\n    new(): OverflowEvent;\\n    readonly BOTH: number;\\n    readonly HORIZONTAL: number;\\n    readonly VERTICAL: number;\\n};\\n\\ninterface PageTransitionEvent extends Event {\\n    readonly persisted: boolean;\\n}\\n\\ndeclare var PageTransitionEvent: {\\n    prototype: PageTransitionEvent;\\n    new(): PageTransitionEvent;\\n};\\n\\ninterface PannerNode extends AudioNode {\\n    coneInnerAngle: number;\\n    coneOuterAngle: number;\\n    coneOuterGain: number;\\n    distanceModel: DistanceModelType;\\n    maxDistance: number;\\n    readonly orientationX: AudioParam;\\n    readonly orientationY: AudioParam;\\n    readonly orientationZ: AudioParam;\\n    panningModel: PanningModelType;\\n    readonly positionX: AudioParam;\\n    readonly positionY: AudioParam;\\n    readonly positionZ: AudioParam;\\n    refDistance: number;\\n    rolloffFactor: number;\\n    /** @deprecated */\\n    setOrientation(x: number, y: number, z: number): void;\\n    /** @deprecated */\\n    setPosition(x: number, y: number, z: number): void;\\n}\\n\\ndeclare var PannerNode: {\\n    prototype: PannerNode;\\n    new(context: BaseAudioContext, options?: PannerOptions): PannerNode;\\n};\\n\\ninterface ParentNode {\\n    readonly childElementCount: number;\\n    /**\\n     * Returns the child elements.\\n     */\\n    readonly children: HTMLCollection;\\n    /**\\n     * Returns the first child that is an element, and null otherwise.\\n     */\\n    readonly firstElementChild: Element | null;\\n    /**\\n     * Returns the last child that is an element, and null otherwise.\\n     */\\n    readonly lastElementChild: Element | null;\\n    /**\\n     * Inserts nodes after the last child of node, while replacing\\n     * strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    append(...nodes: (Node | string)[]): void;\\n    /**\\n     * Inserts nodes before the first child of node, while\\n     * replacing strings in nodes with equivalent Text nodes.\\n     * Throws a \"HierarchyRequestError\" DOMException if the constraints of\\n     * the node tree are violated.\\n     */\\n    prepend(...nodes: (Node | string)[]): void;\\n    /**\\n     * Returns the first element that is a descendant of node that\\n     * matches selectors.\\n     */\\n    querySelector<K extends keyof HTMLElementTagNameMap>(selectors: K): HTMLElementTagNameMap[K] | null;\\n    querySelector<K extends keyof SVGElementTagNameMap>(selectors: K): SVGElementTagNameMap[K] | null;\\n    querySelector<E extends Element = Element>(selectors: string): E | null;\\n    /**\\n     * Returns all element descendants of node that\\n     * match selectors.\\n     */\\n    querySelectorAll<K extends keyof HTMLElementTagNameMap>(selectors: K): NodeListOf<HTMLElementTagNameMap[K]>;\\n    querySelectorAll<K extends keyof SVGElementTagNameMap>(selectors: K): NodeListOf<SVGElementTagNameMap[K]>;\\n    querySelectorAll<E extends Element = Element>(selectors: string): NodeListOf<E>;\\n}\\n\\ninterface Path2D extends CanvasPath {\\n    addPath(path: Path2D, transform?: DOMMatrix2DInit): void;\\n}\\n\\ndeclare var Path2D: {\\n    prototype: Path2D;\\n    new(path?: Path2D | string): Path2D;\\n};\\n\\ninterface PaymentAddress {\\n    readonly addressLine: string[];\\n    readonly city: string;\\n    readonly country: string;\\n    readonly dependentLocality: string;\\n    readonly languageCode: string;\\n    readonly organization: string;\\n    readonly phone: string;\\n    readonly postalCode: string;\\n    readonly recipient: string;\\n    readonly region: string;\\n    readonly sortingCode: string;\\n    toJSON(): any;\\n}\\n\\ndeclare var PaymentAddress: {\\n    prototype: PaymentAddress;\\n    new(): PaymentAddress;\\n};\\n\\ninterface PaymentRequestEventMap {\\n    \"shippingaddresschange\": Event;\\n    \"shippingoptionchange\": Event;\\n}\\n\\ninterface PaymentRequest extends EventTarget {\\n    readonly id: string;\\n    onshippingaddresschange: ((this: PaymentRequest, ev: Event) => any) | null;\\n    onshippingoptionchange: ((this: PaymentRequest, ev: Event) => any) | null;\\n    readonly shippingAddress: PaymentAddress | null;\\n    readonly shippingOption: string | null;\\n    readonly shippingType: PaymentShippingType | null;\\n    abort(): Promise<void>;\\n    canMakePayment(): Promise<boolean>;\\n    show(): Promise<PaymentResponse>;\\n    addEventListener<K extends keyof PaymentRequestEventMap>(type: K, listener: (this: PaymentRequest, ev: PaymentRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof PaymentRequestEventMap>(type: K, listener: (this: PaymentRequest, ev: PaymentRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var PaymentRequest: {\\n    prototype: PaymentRequest;\\n    new(methodData: PaymentMethodData[], details: PaymentDetailsInit, options?: PaymentOptions): PaymentRequest;\\n};\\n\\ninterface PaymentRequestUpdateEvent extends Event {\\n    updateWith(detailsPromise: Promise<PaymentDetailsUpdate>): void;\\n}\\n\\ndeclare var PaymentRequestUpdateEvent: {\\n    prototype: PaymentRequestUpdateEvent;\\n    new(type: string, eventInitDict?: PaymentRequestUpdateEventInit): PaymentRequestUpdateEvent;\\n};\\n\\ninterface PaymentResponse {\\n    readonly details: any;\\n    readonly methodName: string;\\n    readonly payerEmail: string | null;\\n    readonly payerName: string | null;\\n    readonly payerPhone: string | null;\\n    readonly requestId: string;\\n    readonly shippingAddress: PaymentAddress | null;\\n    readonly shippingOption: string | null;\\n    complete(result?: PaymentComplete): Promise<void>;\\n    toJSON(): any;\\n}\\n\\ndeclare var PaymentResponse: {\\n    prototype: PaymentResponse;\\n    new(): PaymentResponse;\\n};\\n\\ninterface PerfWidgetExternal {\\n    readonly activeNetworkRequestCount: number;\\n    readonly averageFrameTime: number;\\n    readonly averagePaintTime: number;\\n    readonly extraInformationEnabled: boolean;\\n    readonly independentRenderingEnabled: boolean;\\n    readonly irDisablingContentString: string;\\n    readonly irStatusAvailable: boolean;\\n    readonly maxCpuSpeed: number;\\n    readonly paintRequestsPerSecond: number;\\n    readonly performanceCounter: number;\\n    readonly performanceCounterFrequency: number;\\n    addEventListener(eventType: string, callback: Function): void;\\n    getMemoryUsage(): number;\\n    getProcessCpuUsage(): number;\\n    getRecentCpuUsage(last: number | null): any;\\n    getRecentFrames(last: number | null): any;\\n    getRecentMemoryUsage(last: number | null): any;\\n    getRecentPaintRequests(last: number | null): any;\\n    removeEventListener(eventType: string, callback: Function): void;\\n    repositionWindow(x: number, y: number): void;\\n    resizeWindow(width: number, height: number): void;\\n}\\n\\ndeclare var PerfWidgetExternal: {\\n    prototype: PerfWidgetExternal;\\n    new(): PerfWidgetExternal;\\n};\\n\\ninterface PerformanceEventMap {\\n    \"resourcetimingbufferfull\": Event;\\n}\\n\\ninterface Performance extends EventTarget {\\n    /** @deprecated */\\n    readonly navigation: PerformanceNavigation;\\n    onresourcetimingbufferfull: ((this: Performance, ev: Event) => any) | null;\\n    readonly timeOrigin: number;\\n    /** @deprecated */\\n    readonly timing: PerformanceTiming;\\n    clearMarks(markName?: string): void;\\n    clearMeasures(measureName?: string): void;\\n    clearResourceTimings(): void;\\n    getEntries(): PerformanceEntryList;\\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\\n    getEntriesByType(type: string): PerformanceEntryList;\\n    mark(markName: string): void;\\n    measure(measureName: string, startMark?: string, endMark?: string): void;\\n    now(): number;\\n    setResourceTimingBufferSize(maxSize: number): void;\\n    toJSON(): any;\\n    addEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Performance: {\\n    prototype: Performance;\\n    new(): Performance;\\n};\\n\\ninterface PerformanceEntry {\\n    readonly duration: number;\\n    readonly entryType: string;\\n    readonly name: string;\\n    readonly startTime: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceEntry: {\\n    prototype: PerformanceEntry;\\n    new(): PerformanceEntry;\\n};\\n\\ninterface PerformanceMark extends PerformanceEntry {\\n}\\n\\ndeclare var PerformanceMark: {\\n    prototype: PerformanceMark;\\n    new(): PerformanceMark;\\n};\\n\\ninterface PerformanceMeasure extends PerformanceEntry {\\n}\\n\\ndeclare var PerformanceMeasure: {\\n    prototype: PerformanceMeasure;\\n    new(): PerformanceMeasure;\\n};\\n\\ninterface PerformanceNavigation {\\n    readonly redirectCount: number;\\n    readonly type: number;\\n    toJSON(): any;\\n    readonly TYPE_BACK_FORWARD: number;\\n    readonly TYPE_NAVIGATE: number;\\n    readonly TYPE_RELOAD: number;\\n    readonly TYPE_RESERVED: number;\\n}\\n\\ndeclare var PerformanceNavigation: {\\n    prototype: PerformanceNavigation;\\n    new(): PerformanceNavigation;\\n    readonly TYPE_BACK_FORWARD: number;\\n    readonly TYPE_NAVIGATE: number;\\n    readonly TYPE_RELOAD: number;\\n    readonly TYPE_RESERVED: number;\\n};\\n\\ninterface PerformanceNavigationTiming extends PerformanceResourceTiming {\\n    readonly domComplete: number;\\n    readonly domContentLoadedEventEnd: number;\\n    readonly domContentLoadedEventStart: number;\\n    readonly domInteractive: number;\\n    readonly loadEventEnd: number;\\n    readonly loadEventStart: number;\\n    readonly redirectCount: number;\\n    readonly type: NavigationType;\\n    readonly unloadEventEnd: number;\\n    readonly unloadEventStart: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceNavigationTiming: {\\n    prototype: PerformanceNavigationTiming;\\n    new(): PerformanceNavigationTiming;\\n};\\n\\ninterface PerformanceObserver {\\n    disconnect(): void;\\n    observe(options: PerformanceObserverInit): void;\\n    takeRecords(): PerformanceEntryList;\\n}\\n\\ndeclare var PerformanceObserver: {\\n    prototype: PerformanceObserver;\\n    new(callback: PerformanceObserverCallback): PerformanceObserver;\\n};\\n\\ninterface PerformanceObserverEntryList {\\n    getEntries(): PerformanceEntryList;\\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\\n    getEntriesByType(type: string): PerformanceEntryList;\\n}\\n\\ndeclare var PerformanceObserverEntryList: {\\n    prototype: PerformanceObserverEntryList;\\n    new(): PerformanceObserverEntryList;\\n};\\n\\ninterface PerformanceResourceTiming extends PerformanceEntry {\\n    readonly connectEnd: number;\\n    readonly connectStart: number;\\n    readonly decodedBodySize: number;\\n    readonly domainLookupEnd: number;\\n    readonly domainLookupStart: number;\\n    readonly encodedBodySize: number;\\n    readonly fetchStart: number;\\n    readonly initiatorType: string;\\n    readonly nextHopProtocol: string;\\n    readonly redirectEnd: number;\\n    readonly redirectStart: number;\\n    readonly requestStart: number;\\n    readonly responseEnd: number;\\n    readonly responseStart: number;\\n    readonly secureConnectionStart: number;\\n    readonly transferSize: number;\\n    readonly workerStart: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceResourceTiming: {\\n    prototype: PerformanceResourceTiming;\\n    new(): PerformanceResourceTiming;\\n};\\n\\ninterface PerformanceTiming {\\n    readonly connectEnd: number;\\n    readonly connectStart: number;\\n    readonly domComplete: number;\\n    readonly domContentLoadedEventEnd: number;\\n    readonly domContentLoadedEventStart: number;\\n    readonly domInteractive: number;\\n    readonly domLoading: number;\\n    readonly domainLookupEnd: number;\\n    readonly domainLookupStart: number;\\n    readonly fetchStart: number;\\n    readonly loadEventEnd: number;\\n    readonly loadEventStart: number;\\n    readonly navigationStart: number;\\n    readonly redirectEnd: number;\\n    readonly redirectStart: number;\\n    readonly requestStart: number;\\n    readonly responseEnd: number;\\n    readonly responseStart: number;\\n    readonly secureConnectionStart: number;\\n    readonly unloadEventEnd: number;\\n    readonly unloadEventStart: number;\\n    toJSON(): any;\\n}\\n\\ndeclare var PerformanceTiming: {\\n    prototype: PerformanceTiming;\\n    new(): PerformanceTiming;\\n};\\n\\ninterface PeriodicWave {\\n}\\n\\ndeclare var PeriodicWave: {\\n    prototype: PeriodicWave;\\n    new(context: BaseAudioContext, options?: PeriodicWaveOptions): PeriodicWave;\\n};\\n\\ninterface PermissionRequest extends DeferredPermissionRequest {\\n    readonly state: MSWebViewPermissionState;\\n    defer(): void;\\n}\\n\\ndeclare var PermissionRequest: {\\n    prototype: PermissionRequest;\\n    new(): PermissionRequest;\\n};\\n\\ninterface PermissionRequestedEvent extends Event {\\n    readonly permissionRequest: PermissionRequest;\\n}\\n\\ndeclare var PermissionRequestedEvent: {\\n    prototype: PermissionRequestedEvent;\\n    new(): PermissionRequestedEvent;\\n};\\n\\ninterface Plugin {\\n    readonly description: string;\\n    readonly filename: string;\\n    readonly length: number;\\n    readonly name: string;\\n    readonly version: string;\\n    item(index: number): MimeType;\\n    namedItem(type: string): MimeType;\\n    [index: number]: MimeType;\\n}\\n\\ndeclare var Plugin: {\\n    prototype: Plugin;\\n    new(): Plugin;\\n};\\n\\ninterface PluginArray {\\n    readonly length: number;\\n    item(index: number): Plugin;\\n    namedItem(name: string): Plugin;\\n    refresh(reload?: boolean): void;\\n    [index: number]: Plugin;\\n}\\n\\ndeclare var PluginArray: {\\n    prototype: PluginArray;\\n    new(): PluginArray;\\n};\\n\\ninterface PointerEvent extends MouseEvent {\\n    readonly height: number;\\n    readonly isPrimary: boolean;\\n    readonly pointerId: number;\\n    readonly pointerType: string;\\n    readonly pressure: number;\\n    readonly tangentialPressure: number;\\n    readonly tiltX: number;\\n    readonly tiltY: number;\\n    readonly twist: number;\\n    readonly width: number;\\n}\\n\\ndeclare var PointerEvent: {\\n    prototype: PointerEvent;\\n    new(type: string, eventInitDict?: PointerEventInit): PointerEvent;\\n};\\n\\ninterface PopStateEvent extends Event {\\n    readonly state: any;\\n}\\n\\ndeclare var PopStateEvent: {\\n    prototype: PopStateEvent;\\n    new(type: string, eventInitDict?: PopStateEventInit): PopStateEvent;\\n};\\n\\ninterface Position {\\n    readonly coords: Coordinates;\\n    readonly timestamp: number;\\n}\\n\\ninterface PositionError {\\n    readonly code: number;\\n    readonly message: string;\\n    readonly PERMISSION_DENIED: number;\\n    readonly POSITION_UNAVAILABLE: number;\\n    readonly TIMEOUT: number;\\n}\\n\\ninterface ProcessingInstruction extends CharacterData {\\n    readonly target: string;\\n}\\n\\ndeclare var ProcessingInstruction: {\\n    prototype: ProcessingInstruction;\\n    new(): ProcessingInstruction;\\n};\\n\\ninterface ProgressEvent extends Event {\\n    readonly lengthComputable: boolean;\\n    readonly loaded: number;\\n    readonly total: number;\\n}\\n\\ndeclare var ProgressEvent: {\\n    prototype: ProgressEvent;\\n    new(type: string, eventInitDict?: ProgressEventInit): ProgressEvent;\\n};\\n\\ninterface PromiseRejectionEvent extends Event {\\n    readonly promise: Promise<any>;\\n    readonly reason: any;\\n}\\n\\ndeclare var PromiseRejectionEvent: {\\n    prototype: PromiseRejectionEvent;\\n    new(type: string, eventInitDict: PromiseRejectionEventInit): PromiseRejectionEvent;\\n};\\n\\ninterface PushManager {\\n    getSubscription(): Promise<PushSubscription | null>;\\n    permissionState(options?: PushSubscriptionOptionsInit): Promise<PushPermissionState>;\\n    subscribe(options?: PushSubscriptionOptionsInit): Promise<PushSubscription>;\\n}\\n\\ndeclare var PushManager: {\\n    prototype: PushManager;\\n    new(): PushManager;\\n    readonly supportedContentEncodings: ReadonlyArray<string>;\\n};\\n\\ninterface PushSubscription {\\n    readonly endpoint: string;\\n    readonly expirationTime: number | null;\\n    readonly options: PushSubscriptionOptions;\\n    getKey(name: PushEncryptionKeyName): ArrayBuffer | null;\\n    toJSON(): PushSubscriptionJSON;\\n    unsubscribe(): Promise<boolean>;\\n}\\n\\ndeclare var PushSubscription: {\\n    prototype: PushSubscription;\\n    new(): PushSubscription;\\n};\\n\\ninterface PushSubscriptionOptions {\\n    readonly applicationServerKey: ArrayBuffer | null;\\n    readonly userVisibleOnly: boolean;\\n}\\n\\ndeclare var PushSubscriptionOptions: {\\n    prototype: PushSubscriptionOptions;\\n    new(): PushSubscriptionOptions;\\n};\\n\\ninterface RTCCertificate {\\n    readonly expires: number;\\n    getFingerprints(): RTCDtlsFingerprint[];\\n}\\n\\ndeclare var RTCCertificate: {\\n    prototype: RTCCertificate;\\n    new(): RTCCertificate;\\n    getSupportedAlgorithms(): AlgorithmIdentifier[];\\n};\\n\\ninterface RTCDTMFSenderEventMap {\\n    \"tonechange\": RTCDTMFToneChangeEvent;\\n}\\n\\ninterface RTCDTMFSender extends EventTarget {\\n    readonly canInsertDTMF: boolean;\\n    ontonechange: ((this: RTCDTMFSender, ev: RTCDTMFToneChangeEvent) => any) | null;\\n    readonly toneBuffer: string;\\n    insertDTMF(tones: string, duration?: number, interToneGap?: number): void;\\n    addEventListener<K extends keyof RTCDTMFSenderEventMap>(type: K, listener: (this: RTCDTMFSender, ev: RTCDTMFSenderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDTMFSenderEventMap>(type: K, listener: (this: RTCDTMFSender, ev: RTCDTMFSenderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDTMFSender: {\\n    prototype: RTCDTMFSender;\\n    new(): RTCDTMFSender;\\n};\\n\\ninterface RTCDTMFToneChangeEvent extends Event {\\n    readonly tone: string;\\n}\\n\\ndeclare var RTCDTMFToneChangeEvent: {\\n    prototype: RTCDTMFToneChangeEvent;\\n    new(type: string, eventInitDict: RTCDTMFToneChangeEventInit): RTCDTMFToneChangeEvent;\\n};\\n\\ninterface RTCDataChannelEventMap {\\n    \"bufferedamountlow\": Event;\\n    \"close\": Event;\\n    \"error\": RTCErrorEvent;\\n    \"message\": MessageEvent;\\n    \"open\": Event;\\n}\\n\\ninterface RTCDataChannel extends EventTarget {\\n    binaryType: string;\\n    readonly bufferedAmount: number;\\n    bufferedAmountLowThreshold: number;\\n    readonly id: number | null;\\n    readonly label: string;\\n    readonly maxPacketLifeTime: number | null;\\n    readonly maxRetransmits: number | null;\\n    readonly negotiated: boolean;\\n    onbufferedamountlow: ((this: RTCDataChannel, ev: Event) => any) | null;\\n    onclose: ((this: RTCDataChannel, ev: Event) => any) | null;\\n    onerror: ((this: RTCDataChannel, ev: RTCErrorEvent) => any) | null;\\n    onmessage: ((this: RTCDataChannel, ev: MessageEvent) => any) | null;\\n    onopen: ((this: RTCDataChannel, ev: Event) => any) | null;\\n    readonly ordered: boolean;\\n    readonly priority: RTCPriorityType;\\n    readonly protocol: string;\\n    readonly readyState: RTCDataChannelState;\\n    close(): void;\\n    send(data: string): void;\\n    send(data: Blob): void;\\n    send(data: ArrayBuffer): void;\\n    send(data: ArrayBufferView): void;\\n    addEventListener<K extends keyof RTCDataChannelEventMap>(type: K, listener: (this: RTCDataChannel, ev: RTCDataChannelEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDataChannelEventMap>(type: K, listener: (this: RTCDataChannel, ev: RTCDataChannelEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDataChannel: {\\n    prototype: RTCDataChannel;\\n    new(): RTCDataChannel;\\n};\\n\\ninterface RTCDataChannelEvent extends Event {\\n    readonly channel: RTCDataChannel;\\n}\\n\\ndeclare var RTCDataChannelEvent: {\\n    prototype: RTCDataChannelEvent;\\n    new(type: string, eventInitDict: RTCDataChannelEventInit): RTCDataChannelEvent;\\n};\\n\\ninterface RTCDtlsTransportEventMap {\\n    \"error\": RTCErrorEvent;\\n    \"statechange\": Event;\\n}\\n\\ninterface RTCDtlsTransport extends EventTarget {\\n    onerror: ((this: RTCDtlsTransport, ev: RTCErrorEvent) => any) | null;\\n    onstatechange: ((this: RTCDtlsTransport, ev: Event) => any) | null;\\n    readonly state: RTCDtlsTransportState;\\n    readonly transport: RTCIceTransport;\\n    getRemoteCertificates(): ArrayBuffer[];\\n    addEventListener<K extends keyof RTCDtlsTransportEventMap>(type: K, listener: (this: RTCDtlsTransport, ev: RTCDtlsTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDtlsTransportEventMap>(type: K, listener: (this: RTCDtlsTransport, ev: RTCDtlsTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDtlsTransport: {\\n    prototype: RTCDtlsTransport;\\n    new(): RTCDtlsTransport;\\n};\\n\\ninterface RTCDtlsTransportStateChangedEvent extends Event {\\n    readonly state: RTCDtlsTransportState;\\n}\\n\\ndeclare var RTCDtlsTransportStateChangedEvent: {\\n    prototype: RTCDtlsTransportStateChangedEvent;\\n    new(): RTCDtlsTransportStateChangedEvent;\\n};\\n\\ninterface RTCDtmfSenderEventMap {\\n    \"tonechange\": RTCDTMFToneChangeEvent;\\n}\\n\\ninterface RTCDtmfSender extends EventTarget {\\n    readonly canInsertDTMF: boolean;\\n    readonly duration: number;\\n    readonly interToneGap: number;\\n    ontonechange: ((this: RTCDtmfSender, ev: RTCDTMFToneChangeEvent) => any) | null;\\n    readonly sender: RTCRtpSender;\\n    readonly toneBuffer: string;\\n    insertDTMF(tones: string, duration?: number, interToneGap?: number): void;\\n    addEventListener<K extends keyof RTCDtmfSenderEventMap>(type: K, listener: (this: RTCDtmfSender, ev: RTCDtmfSenderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCDtmfSenderEventMap>(type: K, listener: (this: RTCDtmfSender, ev: RTCDtmfSenderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCDtmfSender: {\\n    prototype: RTCDtmfSender;\\n    new(sender: RTCRtpSender): RTCDtmfSender;\\n};\\n\\ninterface RTCError extends Error {\\n    errorDetail: string;\\n    httpRequestStatusCode: number;\\n    message: string;\\n    name: string;\\n    receivedAlert: number | null;\\n    sctpCauseCode: number;\\n    sdpLineNumber: number;\\n    sentAlert: number | null;\\n}\\n\\ndeclare var RTCError: {\\n    prototype: RTCError;\\n    new(errorDetail?: string, message?: string): RTCError;\\n};\\n\\ninterface RTCErrorEvent extends Event {\\n    readonly error: RTCError | null;\\n}\\n\\ndeclare var RTCErrorEvent: {\\n    prototype: RTCErrorEvent;\\n    new(type: string, eventInitDict: RTCErrorEventInit): RTCErrorEvent;\\n};\\n\\ninterface RTCIceCandidate {\\n    readonly candidate: string;\\n    readonly component: RTCIceComponent | null;\\n    readonly foundation: string | null;\\n    readonly ip: string | null;\\n    readonly port: number | null;\\n    readonly priority: number | null;\\n    readonly protocol: RTCIceProtocol | null;\\n    readonly relatedAddress: string | null;\\n    readonly relatedPort: number | null;\\n    readonly sdpMLineIndex: number | null;\\n    readonly sdpMid: string | null;\\n    readonly tcpType: RTCIceTcpCandidateType | null;\\n    readonly type: RTCIceCandidateType | null;\\n    readonly usernameFragment: string | null;\\n    toJSON(): RTCIceCandidateInit;\\n}\\n\\ndeclare var RTCIceCandidate: {\\n    prototype: RTCIceCandidate;\\n    new(candidateInitDict?: RTCIceCandidateInit): RTCIceCandidate;\\n};\\n\\ninterface RTCIceCandidatePairChangedEvent extends Event {\\n    readonly pair: RTCIceCandidatePair;\\n}\\n\\ndeclare var RTCIceCandidatePairChangedEvent: {\\n    prototype: RTCIceCandidatePairChangedEvent;\\n    new(): RTCIceCandidatePairChangedEvent;\\n};\\n\\ninterface RTCIceGathererEventMap {\\n    \"error\": Event;\\n    \"localcandidate\": RTCIceGathererEvent;\\n}\\n\\ninterface RTCIceGatherer extends RTCStatsProvider {\\n    readonly component: RTCIceComponent;\\n    onerror: ((this: RTCIceGatherer, ev: Event) => any) | null;\\n    onlocalcandidate: ((this: RTCIceGatherer, ev: RTCIceGathererEvent) => any) | null;\\n    createAssociatedGatherer(): RTCIceGatherer;\\n    getLocalCandidates(): RTCIceCandidateDictionary[];\\n    getLocalParameters(): RTCIceParameters;\\n    addEventListener<K extends keyof RTCIceGathererEventMap>(type: K, listener: (this: RTCIceGatherer, ev: RTCIceGathererEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCIceGathererEventMap>(type: K, listener: (this: RTCIceGatherer, ev: RTCIceGathererEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCIceGatherer: {\\n    prototype: RTCIceGatherer;\\n    new(options: RTCIceGatherOptions): RTCIceGatherer;\\n};\\n\\ninterface RTCIceGathererEvent extends Event {\\n    readonly candidate: RTCIceCandidateDictionary | RTCIceCandidateComplete;\\n}\\n\\ndeclare var RTCIceGathererEvent: {\\n    prototype: RTCIceGathererEvent;\\n    new(): RTCIceGathererEvent;\\n};\\n\\ninterface RTCIceTransportEventMap {\\n    \"gatheringstatechange\": Event;\\n    \"selectedcandidatepairchange\": Event;\\n    \"statechange\": Event;\\n}\\n\\ninterface RTCIceTransport extends EventTarget {\\n    readonly component: RTCIceComponent;\\n    readonly gatheringState: RTCIceGathererState;\\n    ongatheringstatechange: ((this: RTCIceTransport, ev: Event) => any) | null;\\n    onselectedcandidatepairchange: ((this: RTCIceTransport, ev: Event) => any) | null;\\n    onstatechange: ((this: RTCIceTransport, ev: Event) => any) | null;\\n    readonly role: RTCIceRole;\\n    readonly state: RTCIceTransportState;\\n    getLocalCandidates(): RTCIceCandidate[];\\n    getLocalParameters(): RTCIceParameters | null;\\n    getRemoteCandidates(): RTCIceCandidate[];\\n    getRemoteParameters(): RTCIceParameters | null;\\n    getSelectedCandidatePair(): RTCIceCandidatePair | null;\\n    addEventListener<K extends keyof RTCIceTransportEventMap>(type: K, listener: (this: RTCIceTransport, ev: RTCIceTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCIceTransportEventMap>(type: K, listener: (this: RTCIceTransport, ev: RTCIceTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCIceTransport: {\\n    prototype: RTCIceTransport;\\n    new(): RTCIceTransport;\\n};\\n\\ninterface RTCIceTransportStateChangedEvent extends Event {\\n    readonly state: RTCIceTransportState;\\n}\\n\\ndeclare var RTCIceTransportStateChangedEvent: {\\n    prototype: RTCIceTransportStateChangedEvent;\\n    new(): RTCIceTransportStateChangedEvent;\\n};\\n\\ninterface RTCIdentityAssertion {\\n    idp: string;\\n    name: string;\\n}\\n\\ndeclare var RTCIdentityAssertion: {\\n    prototype: RTCIdentityAssertion;\\n    new(idp: string, name: string): RTCIdentityAssertion;\\n};\\n\\ninterface RTCPeerConnectionEventMap {\\n    \"connectionstatechange\": Event;\\n    \"datachannel\": RTCDataChannelEvent;\\n    \"icecandidate\": RTCPeerConnectionIceEvent;\\n    \"icecandidateerror\": RTCPeerConnectionIceErrorEvent;\\n    \"iceconnectionstatechange\": Event;\\n    \"icegatheringstatechange\": Event;\\n    \"negotiationneeded\": Event;\\n    \"signalingstatechange\": Event;\\n    \"statsended\": RTCStatsEvent;\\n    \"track\": RTCTrackEvent;\\n}\\n\\ninterface RTCPeerConnection extends EventTarget {\\n    readonly canTrickleIceCandidates: boolean | null;\\n    readonly connectionState: RTCPeerConnectionState;\\n    readonly currentLocalDescription: RTCSessionDescription | null;\\n    readonly currentRemoteDescription: RTCSessionDescription | null;\\n    readonly iceConnectionState: RTCIceConnectionState;\\n    readonly iceGatheringState: RTCIceGatheringState;\\n    readonly idpErrorInfo: string | null;\\n    readonly idpLoginUrl: string | null;\\n    readonly localDescription: RTCSessionDescription | null;\\n    onconnectionstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    ondatachannel: ((this: RTCPeerConnection, ev: RTCDataChannelEvent) => any) | null;\\n    onicecandidate: ((this: RTCPeerConnection, ev: RTCPeerConnectionIceEvent) => any) | null;\\n    onicecandidateerror: ((this: RTCPeerConnection, ev: RTCPeerConnectionIceErrorEvent) => any) | null;\\n    oniceconnectionstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onicegatheringstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onnegotiationneeded: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onsignalingstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\\n    onstatsended: ((this: RTCPeerConnection, ev: RTCStatsEvent) => any) | null;\\n    ontrack: ((this: RTCPeerConnection, ev: RTCTrackEvent) => any) | null;\\n    readonly peerIdentity: Promise<RTCIdentityAssertion>;\\n    readonly pendingLocalDescription: RTCSessionDescription | null;\\n    readonly pendingRemoteDescription: RTCSessionDescription | null;\\n    readonly remoteDescription: RTCSessionDescription | null;\\n    readonly sctp: RTCSctpTransport | null;\\n    readonly signalingState: RTCSignalingState;\\n    addIceCandidate(candidate: RTCIceCandidateInit | RTCIceCandidate): Promise<void>;\\n    addTrack(track: MediaStreamTrack, ...streams: MediaStream[]): RTCRtpSender;\\n    addTransceiver(trackOrKind: MediaStreamTrack | string, init?: RTCRtpTransceiverInit): RTCRtpTransceiver;\\n    close(): void;\\n    createAnswer(options?: RTCOfferOptions): Promise<RTCSessionDescriptionInit>;\\n    createDataChannel(label: string, dataChannelDict?: RTCDataChannelInit): RTCDataChannel;\\n    createOffer(options?: RTCOfferOptions): Promise<RTCSessionDescriptionInit>;\\n    getConfiguration(): RTCConfiguration;\\n    getIdentityAssertion(): Promise<string>;\\n    getReceivers(): RTCRtpReceiver[];\\n    getSenders(): RTCRtpSender[];\\n    getStats(selector?: MediaStreamTrack | null): Promise<RTCStatsReport>;\\n    getTransceivers(): RTCRtpTransceiver[];\\n    removeTrack(sender: RTCRtpSender): void;\\n    setConfiguration(configuration: RTCConfiguration): void;\\n    setIdentityProvider(provider: string, options?: RTCIdentityProviderOptions): void;\\n    setLocalDescription(description: RTCSessionDescriptionInit): Promise<void>;\\n    setRemoteDescription(description: RTCSessionDescriptionInit): Promise<void>;\\n    addEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: RTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: RTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCPeerConnection: {\\n    prototype: RTCPeerConnection;\\n    new(configuration?: RTCConfiguration): RTCPeerConnection;\\n    generateCertificate(keygenAlgorithm: AlgorithmIdentifier): Promise<RTCCertificate>;\\n    getDefaultIceServers(): RTCIceServer[];\\n};\\n\\ninterface RTCPeerConnectionIceErrorEvent extends Event {\\n    readonly errorCode: number;\\n    readonly errorText: string;\\n    readonly hostCandidate: string;\\n    readonly url: string;\\n}\\n\\ndeclare var RTCPeerConnectionIceErrorEvent: {\\n    prototype: RTCPeerConnectionIceErrorEvent;\\n    new(type: string, eventInitDict: RTCPeerConnectionIceErrorEventInit): RTCPeerConnectionIceErrorEvent;\\n};\\n\\ninterface RTCPeerConnectionIceEvent extends Event {\\n    readonly candidate: RTCIceCandidate | null;\\n    readonly url: string | null;\\n}\\n\\ndeclare var RTCPeerConnectionIceEvent: {\\n    prototype: RTCPeerConnectionIceEvent;\\n    new(type: string, eventInitDict?: RTCPeerConnectionIceEventInit): RTCPeerConnectionIceEvent;\\n};\\n\\ninterface RTCRtpReceiver {\\n    readonly rtcpTransport: RTCDtlsTransport | null;\\n    readonly track: MediaStreamTrack;\\n    readonly transport: RTCDtlsTransport | null;\\n    getContributingSources(): RTCRtpContributingSource[];\\n    getParameters(): RTCRtpReceiveParameters;\\n    getStats(): Promise<RTCStatsReport>;\\n    getSynchronizationSources(): RTCRtpSynchronizationSource[];\\n}\\n\\ndeclare var RTCRtpReceiver: {\\n    prototype: RTCRtpReceiver;\\n    new(): RTCRtpReceiver;\\n    getCapabilities(kind: string): RTCRtpCapabilities | null;\\n};\\n\\ninterface RTCRtpSender {\\n    readonly dtmf: RTCDTMFSender | null;\\n    readonly rtcpTransport: RTCDtlsTransport | null;\\n    readonly track: MediaStreamTrack | null;\\n    readonly transport: RTCDtlsTransport | null;\\n    getParameters(): RTCRtpSendParameters;\\n    getStats(): Promise<RTCStatsReport>;\\n    replaceTrack(withTrack: MediaStreamTrack | null): Promise<void>;\\n    setParameters(parameters: RTCRtpSendParameters): Promise<void>;\\n    setStreams(...streams: MediaStream[]): void;\\n}\\n\\ndeclare var RTCRtpSender: {\\n    prototype: RTCRtpSender;\\n    new(): RTCRtpSender;\\n    getCapabilities(kind: string): RTCRtpCapabilities | null;\\n};\\n\\ninterface RTCRtpTransceiver {\\n    readonly currentDirection: RTCRtpTransceiverDirection | null;\\n    direction: RTCRtpTransceiverDirection;\\n    readonly mid: string | null;\\n    readonly receiver: RTCRtpReceiver;\\n    readonly sender: RTCRtpSender;\\n    readonly stopped: boolean;\\n    setCodecPreferences(codecs: RTCRtpCodecCapability[]): void;\\n    stop(): void;\\n}\\n\\ndeclare var RTCRtpTransceiver: {\\n    prototype: RTCRtpTransceiver;\\n    new(): RTCRtpTransceiver;\\n};\\n\\ninterface RTCSctpTransportEventMap {\\n    \"statechange\": Event;\\n}\\n\\ninterface RTCSctpTransport {\\n    readonly maxChannels: number | null;\\n    readonly maxMessageSize: number;\\n    onstatechange: ((this: RTCSctpTransport, ev: Event) => any) | null;\\n    readonly state: RTCSctpTransportState;\\n    readonly transport: RTCDtlsTransport;\\n    addEventListener<K extends keyof RTCSctpTransportEventMap>(type: K, listener: (this: RTCSctpTransport, ev: RTCSctpTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCSctpTransportEventMap>(type: K, listener: (this: RTCSctpTransport, ev: RTCSctpTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCSctpTransport: {\\n    prototype: RTCSctpTransport;\\n    new(): RTCSctpTransport;\\n};\\n\\ninterface RTCSessionDescription {\\n    readonly sdp: string;\\n    readonly type: RTCSdpType;\\n    toJSON(): any;\\n}\\n\\ndeclare var RTCSessionDescription: {\\n    prototype: RTCSessionDescription;\\n    new(descriptionInitDict: RTCSessionDescriptionInit): RTCSessionDescription;\\n};\\n\\ninterface RTCSrtpSdesTransportEventMap {\\n    \"error\": Event;\\n}\\n\\ninterface RTCSrtpSdesTransport extends EventTarget {\\n    onerror: ((this: RTCSrtpSdesTransport, ev: Event) => any) | null;\\n    readonly transport: RTCIceTransport;\\n    addEventListener<K extends keyof RTCSrtpSdesTransportEventMap>(type: K, listener: (this: RTCSrtpSdesTransport, ev: RTCSrtpSdesTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCSrtpSdesTransportEventMap>(type: K, listener: (this: RTCSrtpSdesTransport, ev: RTCSrtpSdesTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var RTCSrtpSdesTransport: {\\n    prototype: RTCSrtpSdesTransport;\\n    new(transport: RTCIceTransport, encryptParameters: RTCSrtpSdesParameters, decryptParameters: RTCSrtpSdesParameters): RTCSrtpSdesTransport;\\n    getLocalParameters(): RTCSrtpSdesParameters[];\\n};\\n\\ninterface RTCSsrcConflictEvent extends Event {\\n    readonly ssrc: number;\\n}\\n\\ndeclare var RTCSsrcConflictEvent: {\\n    prototype: RTCSsrcConflictEvent;\\n    new(): RTCSsrcConflictEvent;\\n};\\n\\ninterface RTCStatsEvent extends Event {\\n    readonly report: RTCStatsReport;\\n}\\n\\ndeclare var RTCStatsEvent: {\\n    prototype: RTCStatsEvent;\\n    new(type: string, eventInitDict: RTCStatsEventInit): RTCStatsEvent;\\n};\\n\\ninterface RTCStatsProvider extends EventTarget {\\n    getStats(): Promise<RTCStatsReport>;\\n    msGetStats(): Promise<RTCStatsReport>;\\n}\\n\\ndeclare var RTCStatsProvider: {\\n    prototype: RTCStatsProvider;\\n    new(): RTCStatsProvider;\\n};\\n\\ninterface RTCStatsReport {\\n    forEach(callbackfn: (value: any, key: string, parent: RTCStatsReport) => void, thisArg?: any): void;\\n}\\n\\ndeclare var RTCStatsReport: {\\n    prototype: RTCStatsReport;\\n    new(): RTCStatsReport;\\n};\\n\\ninterface RTCTrackEvent extends Event {\\n    readonly receiver: RTCRtpReceiver;\\n    readonly streams: ReadonlyArray<MediaStream>;\\n    readonly track: MediaStreamTrack;\\n    readonly transceiver: RTCRtpTransceiver;\\n}\\n\\ndeclare var RTCTrackEvent: {\\n    prototype: RTCTrackEvent;\\n    new(type: string, eventInitDict: RTCTrackEventInit): RTCTrackEvent;\\n};\\n\\ninterface RadioNodeList extends NodeList {\\n    value: string;\\n}\\n\\ndeclare var RadioNodeList: {\\n    prototype: RadioNodeList;\\n    new(): RadioNodeList;\\n};\\n\\ninterface RandomSource {\\n    getRandomValues<T extends Int8Array | Uint8ClampedArray | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array>(array: T): T;\\n}\\n\\ndeclare var RandomSource: {\\n    prototype: RandomSource;\\n    new(): RandomSource;\\n};\\n\\ninterface Range extends AbstractRange {\\n    /**\\n     * Returns the node, furthest away from\\n     * the document, that is an ancestor of both range\\'s start node and end node.\\n     */\\n    readonly commonAncestorContainer: Node;\\n    cloneContents(): DocumentFragment;\\n    cloneRange(): Range;\\n    collapse(toStart?: boolean): void;\\n    compareBoundaryPoints(how: number, sourceRange: Range): number;\\n    /**\\n     * Returns −1 if the point is before the range, 0 if the point is\\n     * in the range, and 1 if the point is after the range.\\n     */\\n    comparePoint(node: Node, offset: number): number;\\n    createContextualFragment(fragment: string): DocumentFragment;\\n    deleteContents(): void;\\n    detach(): void;\\n    extractContents(): DocumentFragment;\\n    getBoundingClientRect(): ClientRect | DOMRect;\\n    getClientRects(): ClientRectList | DOMRectList;\\n    insertNode(node: Node): void;\\n    /**\\n     * Returns whether range intersects node.\\n     */\\n    intersectsNode(node: Node): boolean;\\n    isPointInRange(node: Node, offset: number): boolean;\\n    selectNode(node: Node): void;\\n    selectNodeContents(node: Node): void;\\n    setEnd(node: Node, offset: number): void;\\n    setEndAfter(node: Node): void;\\n    setEndBefore(node: Node): void;\\n    setStart(node: Node, offset: number): void;\\n    setStartAfter(node: Node): void;\\n    setStartBefore(node: Node): void;\\n    surroundContents(newParent: Node): void;\\n    readonly END_TO_END: number;\\n    readonly END_TO_START: number;\\n    readonly START_TO_END: number;\\n    readonly START_TO_START: number;\\n}\\n\\ndeclare var Range: {\\n    prototype: Range;\\n    new(): Range;\\n    readonly END_TO_END: number;\\n    readonly END_TO_START: number;\\n    readonly START_TO_END: number;\\n    readonly START_TO_START: number;\\n};\\n\\ninterface ReadableStream {\\n    readonly locked: boolean;\\n    cancel(): Promise<void>;\\n    getReader(): ReadableStreamReader;\\n}\\n\\ndeclare var ReadableStream: {\\n    prototype: ReadableStream;\\n    new(): ReadableStream;\\n};\\n\\ninterface ReadableStreamReader {\\n    cancel(): Promise<void>;\\n    read(): Promise<any>;\\n    releaseLock(): void;\\n}\\n\\ndeclare var ReadableStreamReader: {\\n    prototype: ReadableStreamReader;\\n    new(): ReadableStreamReader;\\n};\\n\\ninterface Request extends Body {\\n    /**\\n     * Returns the cache mode associated with request, which is a string indicating\\n     * how the the request will interact with the browser\\'s cache when fetching.\\n     */\\n    readonly cache: RequestCache;\\n    /**\\n     * Returns the credentials mode associated with request, which is a string\\n     * indicating whether credentials will be sent with the request always, never, or only when sent to a\\n     * same-origin URL.\\n     */\\n    readonly credentials: RequestCredentials;\\n    /**\\n     * Returns the kind of resource requested by request, e.g., \"document\" or\\n     * \"script\".\\n     */\\n    readonly destination: RequestDestination;\\n    /**\\n     * Returns a Headers object consisting of the headers associated with request.\\n     * Note that headers added in the network layer by the user agent will not be accounted for in this\\n     * object, e.g., the \"Host\" header.\\n     */\\n    readonly headers: Headers;\\n    /**\\n     * Returns request\\'s subresource integrity metadata, which is a cryptographic hash of\\n     * the resource being fetched. Its value consists of multiple hashes separated by whitespace. [SRI]\\n     */\\n    readonly integrity: string;\\n    /**\\n     * Returns a boolean indicating whether or not request is for a history\\n     * navigation (a.k.a. back-foward navigation).\\n     */\\n    readonly isHistoryNavigation: boolean;\\n    /**\\n     * Returns a boolean indicating whether or not request is for a reload navigation.\\n     */\\n    readonly isReloadNavigation: boolean;\\n    /**\\n     * Returns a boolean indicating whether or not request can outlive the global in which\\n     * it was created.\\n     */\\n    readonly keepalive: boolean;\\n    /**\\n     * Returns request\\'s HTTP method, which is \"GET\" by default.\\n     */\\n    readonly method: string;\\n    /**\\n     * Returns the mode associated with request, which is a string indicating\\n     * whether the request will use CORS, or will be restricted to same-origin URLs.\\n     */\\n    readonly mode: RequestMode;\\n    /**\\n     * Returns the redirect mode associated with request, which is a string\\n     * indicating how redirects for the request will be handled during fetching. A request will follow redirects by default.\\n     */\\n    readonly redirect: RequestRedirect;\\n    /**\\n     * Returns the referrer of request. Its value can be a same-origin URL if\\n     * explicitly set in init, the empty string to indicate no referrer, and\\n     * \"about:client\" when defaulting to the global\\'s default. This is used during\\n     * fetching to determine the value of the `Referer` header of the request being made.\\n     */\\n    readonly referrer: string;\\n    /**\\n     * Returns the referrer policy associated with request. This is used during\\n     * fetching to compute the value of the request\\'s referrer.\\n     */\\n    readonly referrerPolicy: ReferrerPolicy;\\n    /**\\n     * Returns the signal associated with request, which is an AbortSignal object indicating whether or not request has been aborted, and its abort\\n     * event handler.\\n     */\\n    readonly signal: AbortSignal;\\n    /**\\n     * Returns the URL of request as a string.\\n     */\\n    readonly url: string;\\n    clone(): Request;\\n}\\n\\ndeclare var Request: {\\n    prototype: Request;\\n    new(input: RequestInfo, init?: RequestInit): Request;\\n};\\n\\ninterface Response extends Body {\\n    readonly headers: Headers;\\n    readonly ok: boolean;\\n    readonly redirected: boolean;\\n    readonly status: number;\\n    readonly statusText: string;\\n    readonly trailer: Promise<Headers>;\\n    readonly type: ResponseType;\\n    readonly url: string;\\n    clone(): Response;\\n}\\n\\ndeclare var Response: {\\n    prototype: Response;\\n    new(body?: BodyInit | null, init?: ResponseInit): Response;\\n    error(): Response;\\n    redirect(url: string, status?: number): Response;\\n};\\n\\ninterface SVGAElement extends SVGGraphicsElement, SVGURIReference {\\n    readonly target: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGAElement: {\\n    prototype: SVGAElement;\\n    new(): SVGAElement;\\n};\\n\\ninterface SVGAngle {\\n    readonly unitType: number;\\n    value: number;\\n    valueAsString: string;\\n    valueInSpecifiedUnits: number;\\n    convertToSpecifiedUnits(unitType: number): void;\\n    newValueSpecifiedUnits(unitType: number, valueInSpecifiedUnits: number): void;\\n    readonly SVG_ANGLETYPE_DEG: number;\\n    readonly SVG_ANGLETYPE_GRAD: number;\\n    readonly SVG_ANGLETYPE_RAD: number;\\n    readonly SVG_ANGLETYPE_UNKNOWN: number;\\n    readonly SVG_ANGLETYPE_UNSPECIFIED: number;\\n}\\n\\ndeclare var SVGAngle: {\\n    prototype: SVGAngle;\\n    new(): SVGAngle;\\n    readonly SVG_ANGLETYPE_DEG: number;\\n    readonly SVG_ANGLETYPE_GRAD: number;\\n    readonly SVG_ANGLETYPE_RAD: number;\\n    readonly SVG_ANGLETYPE_UNKNOWN: number;\\n    readonly SVG_ANGLETYPE_UNSPECIFIED: number;\\n};\\n\\ninterface SVGAnimatedAngle {\\n    readonly animVal: SVGAngle;\\n    readonly baseVal: SVGAngle;\\n}\\n\\ndeclare var SVGAnimatedAngle: {\\n    prototype: SVGAnimatedAngle;\\n    new(): SVGAnimatedAngle;\\n};\\n\\ninterface SVGAnimatedBoolean {\\n    readonly animVal: boolean;\\n    baseVal: boolean;\\n}\\n\\ndeclare var SVGAnimatedBoolean: {\\n    prototype: SVGAnimatedBoolean;\\n    new(): SVGAnimatedBoolean;\\n};\\n\\ninterface SVGAnimatedEnumeration {\\n    readonly animVal: number;\\n    baseVal: number;\\n}\\n\\ndeclare var SVGAnimatedEnumeration: {\\n    prototype: SVGAnimatedEnumeration;\\n    new(): SVGAnimatedEnumeration;\\n};\\n\\ninterface SVGAnimatedInteger {\\n    readonly animVal: number;\\n    baseVal: number;\\n}\\n\\ndeclare var SVGAnimatedInteger: {\\n    prototype: SVGAnimatedInteger;\\n    new(): SVGAnimatedInteger;\\n};\\n\\ninterface SVGAnimatedLength {\\n    readonly animVal: SVGLength;\\n    readonly baseVal: SVGLength;\\n}\\n\\ndeclare var SVGAnimatedLength: {\\n    prototype: SVGAnimatedLength;\\n    new(): SVGAnimatedLength;\\n};\\n\\ninterface SVGAnimatedLengthList {\\n    readonly animVal: SVGLengthList;\\n    readonly baseVal: SVGLengthList;\\n}\\n\\ndeclare var SVGAnimatedLengthList: {\\n    prototype: SVGAnimatedLengthList;\\n    new(): SVGAnimatedLengthList;\\n};\\n\\ninterface SVGAnimatedNumber {\\n    readonly animVal: number;\\n    baseVal: number;\\n}\\n\\ndeclare var SVGAnimatedNumber: {\\n    prototype: SVGAnimatedNumber;\\n    new(): SVGAnimatedNumber;\\n};\\n\\ninterface SVGAnimatedNumberList {\\n    readonly animVal: SVGNumberList;\\n    readonly baseVal: SVGNumberList;\\n}\\n\\ndeclare var SVGAnimatedNumberList: {\\n    prototype: SVGAnimatedNumberList;\\n    new(): SVGAnimatedNumberList;\\n};\\n\\ninterface SVGAnimatedPoints {\\n    readonly animatedPoints: SVGPointList;\\n    readonly points: SVGPointList;\\n}\\n\\ninterface SVGAnimatedPreserveAspectRatio {\\n    readonly animVal: SVGPreserveAspectRatio;\\n    readonly baseVal: SVGPreserveAspectRatio;\\n}\\n\\ndeclare var SVGAnimatedPreserveAspectRatio: {\\n    prototype: SVGAnimatedPreserveAspectRatio;\\n    new(): SVGAnimatedPreserveAspectRatio;\\n};\\n\\ninterface SVGAnimatedRect {\\n    readonly animVal: DOMRectReadOnly;\\n    readonly baseVal: DOMRect;\\n}\\n\\ndeclare var SVGAnimatedRect: {\\n    prototype: SVGAnimatedRect;\\n    new(): SVGAnimatedRect;\\n};\\n\\ninterface SVGAnimatedString {\\n    readonly animVal: string;\\n    baseVal: string;\\n}\\n\\ndeclare var SVGAnimatedString: {\\n    prototype: SVGAnimatedString;\\n    new(): SVGAnimatedString;\\n};\\n\\ninterface SVGAnimatedTransformList {\\n    readonly animVal: SVGTransformList;\\n    readonly baseVal: SVGTransformList;\\n}\\n\\ndeclare var SVGAnimatedTransformList: {\\n    prototype: SVGAnimatedTransformList;\\n    new(): SVGAnimatedTransformList;\\n};\\n\\ninterface SVGCircleElement extends SVGGraphicsElement {\\n    readonly cx: SVGAnimatedLength;\\n    readonly cy: SVGAnimatedLength;\\n    readonly r: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGCircleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGCircleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGCircleElement: {\\n    prototype: SVGCircleElement;\\n    new(): SVGCircleElement;\\n};\\n\\ninterface SVGClipPathElement extends SVGGraphicsElement {\\n    readonly clipPathUnits: SVGAnimatedEnumeration;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGClipPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGClipPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGClipPathElement: {\\n    prototype: SVGClipPathElement;\\n    new(): SVGClipPathElement;\\n};\\n\\ninterface SVGComponentTransferFunctionElement extends SVGElement {\\n    readonly amplitude: SVGAnimatedNumber;\\n    readonly exponent: SVGAnimatedNumber;\\n    readonly intercept: SVGAnimatedNumber;\\n    readonly offset: SVGAnimatedNumber;\\n    readonly slope: SVGAnimatedNumber;\\n    readonly tableValues: SVGAnimatedNumberList;\\n    readonly type: SVGAnimatedEnumeration;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_GAMMA: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_LINEAR: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_TABLE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGComponentTransferFunctionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGComponentTransferFunctionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGComponentTransferFunctionElement: {\\n    prototype: SVGComponentTransferFunctionElement;\\n    new(): SVGComponentTransferFunctionElement;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_GAMMA: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_LINEAR: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_TABLE: number;\\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGDefsElement extends SVGGraphicsElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDefsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDefsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGDefsElement: {\\n    prototype: SVGDefsElement;\\n    new(): SVGDefsElement;\\n};\\n\\ninterface SVGDescElement extends SVGElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDescElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDescElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGDescElement: {\\n    prototype: SVGDescElement;\\n    new(): SVGDescElement;\\n};\\n\\ninterface SVGElementEventMap extends ElementEventMap, GlobalEventHandlersEventMap, DocumentAndElementEventHandlersEventMap {\\n}\\n\\ninterface SVGElement extends Element, GlobalEventHandlers, DocumentAndElementEventHandlers, SVGElementInstance, HTMLOrSVGElement, ElementCSSInlineStyle {\\n    /** @deprecated */\\n    readonly className: any;\\n    readonly ownerSVGElement: SVGSVGElement | null;\\n    readonly viewportElement: SVGElement | null;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGElement: {\\n    prototype: SVGElement;\\n    new(): SVGElement;\\n};\\n\\ninterface SVGElementInstance extends EventTarget {\\n    readonly correspondingElement: SVGElement;\\n    readonly correspondingUseElement: SVGUseElement;\\n}\\n\\ndeclare var SVGElementInstance: {\\n    prototype: SVGElementInstance;\\n    new(): SVGElementInstance;\\n};\\n\\ninterface SVGElementInstanceList {\\n    /** @deprecated */\\n    readonly length: number;\\n    /** @deprecated */\\n    item(index: number): SVGElementInstance;\\n}\\n\\ndeclare var SVGElementInstanceList: {\\n    prototype: SVGElementInstanceList;\\n    new(): SVGElementInstanceList;\\n};\\n\\ninterface SVGEllipseElement extends SVGGraphicsElement {\\n    readonly cx: SVGAnimatedLength;\\n    readonly cy: SVGAnimatedLength;\\n    readonly rx: SVGAnimatedLength;\\n    readonly ry: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGEllipseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGEllipseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGEllipseElement: {\\n    prototype: SVGEllipseElement;\\n    new(): SVGEllipseElement;\\n};\\n\\ninterface SVGFEBlendElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly in2: SVGAnimatedString;\\n    readonly mode: SVGAnimatedEnumeration;\\n    readonly SVG_FEBLEND_MODE_COLOR: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_BURN: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_DODGE: number;\\n    readonly SVG_FEBLEND_MODE_DARKEN: number;\\n    readonly SVG_FEBLEND_MODE_DIFFERENCE: number;\\n    readonly SVG_FEBLEND_MODE_EXCLUSION: number;\\n    readonly SVG_FEBLEND_MODE_HARD_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_HUE: number;\\n    readonly SVG_FEBLEND_MODE_LIGHTEN: number;\\n    readonly SVG_FEBLEND_MODE_LUMINOSITY: number;\\n    readonly SVG_FEBLEND_MODE_MULTIPLY: number;\\n    readonly SVG_FEBLEND_MODE_NORMAL: number;\\n    readonly SVG_FEBLEND_MODE_OVERLAY: number;\\n    readonly SVG_FEBLEND_MODE_SATURATION: number;\\n    readonly SVG_FEBLEND_MODE_SCREEN: number;\\n    readonly SVG_FEBLEND_MODE_SOFT_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEBlendElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEBlendElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEBlendElement: {\\n    prototype: SVGFEBlendElement;\\n    new(): SVGFEBlendElement;\\n    readonly SVG_FEBLEND_MODE_COLOR: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_BURN: number;\\n    readonly SVG_FEBLEND_MODE_COLOR_DODGE: number;\\n    readonly SVG_FEBLEND_MODE_DARKEN: number;\\n    readonly SVG_FEBLEND_MODE_DIFFERENCE: number;\\n    readonly SVG_FEBLEND_MODE_EXCLUSION: number;\\n    readonly SVG_FEBLEND_MODE_HARD_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_HUE: number;\\n    readonly SVG_FEBLEND_MODE_LIGHTEN: number;\\n    readonly SVG_FEBLEND_MODE_LUMINOSITY: number;\\n    readonly SVG_FEBLEND_MODE_MULTIPLY: number;\\n    readonly SVG_FEBLEND_MODE_NORMAL: number;\\n    readonly SVG_FEBLEND_MODE_OVERLAY: number;\\n    readonly SVG_FEBLEND_MODE_SATURATION: number;\\n    readonly SVG_FEBLEND_MODE_SCREEN: number;\\n    readonly SVG_FEBLEND_MODE_SOFT_LIGHT: number;\\n    readonly SVG_FEBLEND_MODE_UNKNOWN: number;\\n};\\n\\ninterface SVGFEColorMatrixElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly type: SVGAnimatedEnumeration;\\n    readonly values: SVGAnimatedNumberList;\\n    readonly SVG_FECOLORMATRIX_TYPE_HUEROTATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_LUMINANCETOALPHA: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_MATRIX: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_SATURATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEColorMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEColorMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEColorMatrixElement: {\\n    prototype: SVGFEColorMatrixElement;\\n    new(): SVGFEColorMatrixElement;\\n    readonly SVG_FECOLORMATRIX_TYPE_HUEROTATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_LUMINANCETOALPHA: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_MATRIX: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_SATURATE: number;\\n    readonly SVG_FECOLORMATRIX_TYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGFEComponentTransferElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEComponentTransferElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEComponentTransferElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEComponentTransferElement: {\\n    prototype: SVGFEComponentTransferElement;\\n    new(): SVGFEComponentTransferElement;\\n};\\n\\ninterface SVGFECompositeElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly in2: SVGAnimatedString;\\n    readonly k1: SVGAnimatedNumber;\\n    readonly k2: SVGAnimatedNumber;\\n    readonly k3: SVGAnimatedNumber;\\n    readonly k4: SVGAnimatedNumber;\\n    readonly operator: SVGAnimatedEnumeration;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ARITHMETIC: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ATOP: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_IN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OUT: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OVER: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_UNKNOWN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_XOR: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFECompositeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFECompositeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFECompositeElement: {\\n    prototype: SVGFECompositeElement;\\n    new(): SVGFECompositeElement;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ARITHMETIC: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_ATOP: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_IN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OUT: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_OVER: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_UNKNOWN: number;\\n    readonly SVG_FECOMPOSITE_OPERATOR_XOR: number;\\n};\\n\\ninterface SVGFEConvolveMatrixElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly bias: SVGAnimatedNumber;\\n    readonly divisor: SVGAnimatedNumber;\\n    readonly edgeMode: SVGAnimatedEnumeration;\\n    readonly in1: SVGAnimatedString;\\n    readonly kernelMatrix: SVGAnimatedNumberList;\\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\\n    readonly orderX: SVGAnimatedInteger;\\n    readonly orderY: SVGAnimatedInteger;\\n    readonly preserveAlpha: SVGAnimatedBoolean;\\n    readonly targetX: SVGAnimatedInteger;\\n    readonly targetY: SVGAnimatedInteger;\\n    readonly SVG_EDGEMODE_DUPLICATE: number;\\n    readonly SVG_EDGEMODE_NONE: number;\\n    readonly SVG_EDGEMODE_UNKNOWN: number;\\n    readonly SVG_EDGEMODE_WRAP: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEConvolveMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEConvolveMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEConvolveMatrixElement: {\\n    prototype: SVGFEConvolveMatrixElement;\\n    new(): SVGFEConvolveMatrixElement;\\n    readonly SVG_EDGEMODE_DUPLICATE: number;\\n    readonly SVG_EDGEMODE_NONE: number;\\n    readonly SVG_EDGEMODE_UNKNOWN: number;\\n    readonly SVG_EDGEMODE_WRAP: number;\\n};\\n\\ninterface SVGFEDiffuseLightingElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly diffuseConstant: SVGAnimatedNumber;\\n    readonly in1: SVGAnimatedString;\\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\\n    readonly surfaceScale: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDiffuseLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDiffuseLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEDiffuseLightingElement: {\\n    prototype: SVGFEDiffuseLightingElement;\\n    new(): SVGFEDiffuseLightingElement;\\n};\\n\\ninterface SVGFEDisplacementMapElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly in2: SVGAnimatedString;\\n    readonly scale: SVGAnimatedNumber;\\n    readonly xChannelSelector: SVGAnimatedEnumeration;\\n    readonly yChannelSelector: SVGAnimatedEnumeration;\\n    readonly SVG_CHANNEL_A: number;\\n    readonly SVG_CHANNEL_B: number;\\n    readonly SVG_CHANNEL_G: number;\\n    readonly SVG_CHANNEL_R: number;\\n    readonly SVG_CHANNEL_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDisplacementMapElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDisplacementMapElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEDisplacementMapElement: {\\n    prototype: SVGFEDisplacementMapElement;\\n    new(): SVGFEDisplacementMapElement;\\n    readonly SVG_CHANNEL_A: number;\\n    readonly SVG_CHANNEL_B: number;\\n    readonly SVG_CHANNEL_G: number;\\n    readonly SVG_CHANNEL_R: number;\\n    readonly SVG_CHANNEL_UNKNOWN: number;\\n};\\n\\ninterface SVGFEDistantLightElement extends SVGElement {\\n    readonly azimuth: SVGAnimatedNumber;\\n    readonly elevation: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDistantLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDistantLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEDistantLightElement: {\\n    prototype: SVGFEDistantLightElement;\\n    new(): SVGFEDistantLightElement;\\n};\\n\\ninterface SVGFEFloodElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFloodElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFloodElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFloodElement: {\\n    prototype: SVGFEFloodElement;\\n    new(): SVGFEFloodElement;\\n};\\n\\ninterface SVGFEFuncAElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncAElement: {\\n    prototype: SVGFEFuncAElement;\\n    new(): SVGFEFuncAElement;\\n};\\n\\ninterface SVGFEFuncBElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncBElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncBElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncBElement: {\\n    prototype: SVGFEFuncBElement;\\n    new(): SVGFEFuncBElement;\\n};\\n\\ninterface SVGFEFuncGElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncGElement: {\\n    prototype: SVGFEFuncGElement;\\n    new(): SVGFEFuncGElement;\\n};\\n\\ninterface SVGFEFuncRElement extends SVGComponentTransferFunctionElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncRElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncRElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEFuncRElement: {\\n    prototype: SVGFEFuncRElement;\\n    new(): SVGFEFuncRElement;\\n};\\n\\ninterface SVGFEGaussianBlurElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly stdDeviationX: SVGAnimatedNumber;\\n    readonly stdDeviationY: SVGAnimatedNumber;\\n    setStdDeviation(stdDeviationX: number, stdDeviationY: number): void;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEGaussianBlurElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEGaussianBlurElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEGaussianBlurElement: {\\n    prototype: SVGFEGaussianBlurElement;\\n    new(): SVGFEGaussianBlurElement;\\n};\\n\\ninterface SVGFEImageElement extends SVGElement, SVGFilterPrimitiveStandardAttributes, SVGURIReference {\\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEImageElement: {\\n    prototype: SVGFEImageElement;\\n    new(): SVGFEImageElement;\\n};\\n\\ninterface SVGFEMergeElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEMergeElement: {\\n    prototype: SVGFEMergeElement;\\n    new(): SVGFEMergeElement;\\n};\\n\\ninterface SVGFEMergeNodeElement extends SVGElement {\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeNodeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeNodeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEMergeNodeElement: {\\n    prototype: SVGFEMergeNodeElement;\\n    new(): SVGFEMergeNodeElement;\\n};\\n\\ninterface SVGFEMorphologyElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly operator: SVGAnimatedEnumeration;\\n    readonly radiusX: SVGAnimatedNumber;\\n    readonly radiusY: SVGAnimatedNumber;\\n    readonly SVG_MORPHOLOGY_OPERATOR_DILATE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_ERODE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMorphologyElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMorphologyElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEMorphologyElement: {\\n    prototype: SVGFEMorphologyElement;\\n    new(): SVGFEMorphologyElement;\\n    readonly SVG_MORPHOLOGY_OPERATOR_DILATE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_ERODE: number;\\n    readonly SVG_MORPHOLOGY_OPERATOR_UNKNOWN: number;\\n};\\n\\ninterface SVGFEOffsetElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly dx: SVGAnimatedNumber;\\n    readonly dy: SVGAnimatedNumber;\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEOffsetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEOffsetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEOffsetElement: {\\n    prototype: SVGFEOffsetElement;\\n    new(): SVGFEOffsetElement;\\n};\\n\\ninterface SVGFEPointLightElement extends SVGElement {\\n    readonly x: SVGAnimatedNumber;\\n    readonly y: SVGAnimatedNumber;\\n    readonly z: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEPointLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEPointLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFEPointLightElement: {\\n    prototype: SVGFEPointLightElement;\\n    new(): SVGFEPointLightElement;\\n};\\n\\ninterface SVGFESpecularLightingElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\\n    readonly specularConstant: SVGAnimatedNumber;\\n    readonly specularExponent: SVGAnimatedNumber;\\n    readonly surfaceScale: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpecularLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpecularLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFESpecularLightingElement: {\\n    prototype: SVGFESpecularLightingElement;\\n    new(): SVGFESpecularLightingElement;\\n};\\n\\ninterface SVGFESpotLightElement extends SVGElement {\\n    readonly limitingConeAngle: SVGAnimatedNumber;\\n    readonly pointsAtX: SVGAnimatedNumber;\\n    readonly pointsAtY: SVGAnimatedNumber;\\n    readonly pointsAtZ: SVGAnimatedNumber;\\n    readonly specularExponent: SVGAnimatedNumber;\\n    readonly x: SVGAnimatedNumber;\\n    readonly y: SVGAnimatedNumber;\\n    readonly z: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpotLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpotLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFESpotLightElement: {\\n    prototype: SVGFESpotLightElement;\\n    new(): SVGFESpotLightElement;\\n};\\n\\ninterface SVGFETileElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly in1: SVGAnimatedString;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETileElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETileElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFETileElement: {\\n    prototype: SVGFETileElement;\\n    new(): SVGFETileElement;\\n};\\n\\ninterface SVGFETurbulenceElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\\n    readonly baseFrequencyX: SVGAnimatedNumber;\\n    readonly baseFrequencyY: SVGAnimatedNumber;\\n    readonly numOctaves: SVGAnimatedInteger;\\n    readonly seed: SVGAnimatedNumber;\\n    readonly stitchTiles: SVGAnimatedEnumeration;\\n    readonly type: SVGAnimatedEnumeration;\\n    readonly SVG_STITCHTYPE_NOSTITCH: number;\\n    readonly SVG_STITCHTYPE_STITCH: number;\\n    readonly SVG_STITCHTYPE_UNKNOWN: number;\\n    readonly SVG_TURBULENCE_TYPE_FRACTALNOISE: number;\\n    readonly SVG_TURBULENCE_TYPE_TURBULENCE: number;\\n    readonly SVG_TURBULENCE_TYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETurbulenceElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETurbulenceElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFETurbulenceElement: {\\n    prototype: SVGFETurbulenceElement;\\n    new(): SVGFETurbulenceElement;\\n    readonly SVG_STITCHTYPE_NOSTITCH: number;\\n    readonly SVG_STITCHTYPE_STITCH: number;\\n    readonly SVG_STITCHTYPE_UNKNOWN: number;\\n    readonly SVG_TURBULENCE_TYPE_FRACTALNOISE: number;\\n    readonly SVG_TURBULENCE_TYPE_TURBULENCE: number;\\n    readonly SVG_TURBULENCE_TYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGFilterElement extends SVGElement, SVGURIReference {\\n    /** @deprecated */\\n    readonly filterResX: SVGAnimatedInteger;\\n    /** @deprecated */\\n    readonly filterResY: SVGAnimatedInteger;\\n    readonly filterUnits: SVGAnimatedEnumeration;\\n    readonly height: SVGAnimatedLength;\\n    readonly primitiveUnits: SVGAnimatedEnumeration;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    /** @deprecated */\\n    setFilterRes(filterResX: number, filterResY: number): void;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFilterElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFilterElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGFilterElement: {\\n    prototype: SVGFilterElement;\\n    new(): SVGFilterElement;\\n};\\n\\ninterface SVGFilterPrimitiveStandardAttributes {\\n    readonly height: SVGAnimatedLength;\\n    readonly result: SVGAnimatedString;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n}\\n\\ninterface SVGFitToViewBox {\\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\\n    readonly viewBox: SVGAnimatedRect;\\n}\\n\\ninterface SVGForeignObjectElement extends SVGGraphicsElement {\\n    readonly height: SVGAnimatedLength;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGForeignObjectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGForeignObjectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGForeignObjectElement: {\\n    prototype: SVGForeignObjectElement;\\n    new(): SVGForeignObjectElement;\\n};\\n\\ninterface SVGGElement extends SVGGraphicsElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGElement: {\\n    prototype: SVGGElement;\\n    new(): SVGGElement;\\n};\\n\\ninterface SVGGeometryElement extends SVGGraphicsElement {\\n    readonly pathLength: SVGAnimatedNumber;\\n    getPointAtLength(distance: number): DOMPoint;\\n    getTotalLength(): number;\\n    isPointInFill(point?: DOMPointInit): boolean;\\n    isPointInStroke(point?: DOMPointInit): boolean;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGeometryElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGeometryElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGeometryElement: {\\n    prototype: SVGGeometryElement;\\n    new(): SVGGeometryElement;\\n};\\n\\ninterface SVGGradientElement extends SVGElement, SVGURIReference {\\n    readonly gradientTransform: SVGAnimatedTransformList;\\n    readonly gradientUnits: SVGAnimatedEnumeration;\\n    readonly spreadMethod: SVGAnimatedEnumeration;\\n    readonly SVG_SPREADMETHOD_PAD: number;\\n    readonly SVG_SPREADMETHOD_REFLECT: number;\\n    readonly SVG_SPREADMETHOD_REPEAT: number;\\n    readonly SVG_SPREADMETHOD_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGradientElement: {\\n    prototype: SVGGradientElement;\\n    new(): SVGGradientElement;\\n    readonly SVG_SPREADMETHOD_PAD: number;\\n    readonly SVG_SPREADMETHOD_REFLECT: number;\\n    readonly SVG_SPREADMETHOD_REPEAT: number;\\n    readonly SVG_SPREADMETHOD_UNKNOWN: number;\\n};\\n\\ninterface SVGGraphicsElement extends SVGElement, SVGTests {\\n    readonly transform: SVGAnimatedTransformList;\\n    getBBox(options?: SVGBoundingBoxOptions): DOMRect;\\n    getCTM(): DOMMatrix | null;\\n    getScreenCTM(): DOMMatrix | null;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGraphicsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGraphicsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGGraphicsElement: {\\n    prototype: SVGGraphicsElement;\\n    new(): SVGGraphicsElement;\\n};\\n\\ninterface SVGImageElement extends SVGGraphicsElement, SVGURIReference {\\n    readonly height: SVGAnimatedLength;\\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGImageElement: {\\n    prototype: SVGImageElement;\\n    new(): SVGImageElement;\\n};\\n\\ninterface SVGLength {\\n    readonly unitType: number;\\n    value: number;\\n    valueAsString: string;\\n    valueInSpecifiedUnits: number;\\n    convertToSpecifiedUnits(unitType: number): void;\\n    newValueSpecifiedUnits(unitType: number, valueInSpecifiedUnits: number): void;\\n    readonly SVG_LENGTHTYPE_CM: number;\\n    readonly SVG_LENGTHTYPE_EMS: number;\\n    readonly SVG_LENGTHTYPE_EXS: number;\\n    readonly SVG_LENGTHTYPE_IN: number;\\n    readonly SVG_LENGTHTYPE_MM: number;\\n    readonly SVG_LENGTHTYPE_NUMBER: number;\\n    readonly SVG_LENGTHTYPE_PC: number;\\n    readonly SVG_LENGTHTYPE_PERCENTAGE: number;\\n    readonly SVG_LENGTHTYPE_PT: number;\\n    readonly SVG_LENGTHTYPE_PX: number;\\n    readonly SVG_LENGTHTYPE_UNKNOWN: number;\\n}\\n\\ndeclare var SVGLength: {\\n    prototype: SVGLength;\\n    new(): SVGLength;\\n    readonly SVG_LENGTHTYPE_CM: number;\\n    readonly SVG_LENGTHTYPE_EMS: number;\\n    readonly SVG_LENGTHTYPE_EXS: number;\\n    readonly SVG_LENGTHTYPE_IN: number;\\n    readonly SVG_LENGTHTYPE_MM: number;\\n    readonly SVG_LENGTHTYPE_NUMBER: number;\\n    readonly SVG_LENGTHTYPE_PC: number;\\n    readonly SVG_LENGTHTYPE_PERCENTAGE: number;\\n    readonly SVG_LENGTHTYPE_PT: number;\\n    readonly SVG_LENGTHTYPE_PX: number;\\n    readonly SVG_LENGTHTYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGLengthList {\\n    readonly length: number;\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGLength): SVGLength;\\n    clear(): void;\\n    getItem(index: number): SVGLength;\\n    initialize(newItem: SVGLength): SVGLength;\\n    insertItemBefore(newItem: SVGLength, index: number): SVGLength;\\n    removeItem(index: number): SVGLength;\\n    replaceItem(newItem: SVGLength, index: number): SVGLength;\\n    [index: number]: SVGLength;\\n}\\n\\ndeclare var SVGLengthList: {\\n    prototype: SVGLengthList;\\n    new(): SVGLengthList;\\n};\\n\\ninterface SVGLineElement extends SVGGraphicsElement {\\n    readonly x1: SVGAnimatedLength;\\n    readonly x2: SVGAnimatedLength;\\n    readonly y1: SVGAnimatedLength;\\n    readonly y2: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGLineElement: {\\n    prototype: SVGLineElement;\\n    new(): SVGLineElement;\\n};\\n\\ninterface SVGLinearGradientElement extends SVGGradientElement {\\n    readonly x1: SVGAnimatedLength;\\n    readonly x2: SVGAnimatedLength;\\n    readonly y1: SVGAnimatedLength;\\n    readonly y2: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLinearGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLinearGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGLinearGradientElement: {\\n    prototype: SVGLinearGradientElement;\\n    new(): SVGLinearGradientElement;\\n};\\n\\ninterface SVGMarkerElement extends SVGElement, SVGFitToViewBox {\\n    readonly markerHeight: SVGAnimatedLength;\\n    readonly markerUnits: SVGAnimatedEnumeration;\\n    readonly markerWidth: SVGAnimatedLength;\\n    readonly orientAngle: SVGAnimatedAngle;\\n    readonly orientType: SVGAnimatedEnumeration;\\n    readonly refX: SVGAnimatedLength;\\n    readonly refY: SVGAnimatedLength;\\n    setOrientToAngle(angle: SVGAngle): void;\\n    setOrientToAuto(): void;\\n    readonly SVG_MARKERUNITS_STROKEWIDTH: number;\\n    readonly SVG_MARKERUNITS_UNKNOWN: number;\\n    readonly SVG_MARKERUNITS_USERSPACEONUSE: number;\\n    readonly SVG_MARKER_ORIENT_ANGLE: number;\\n    readonly SVG_MARKER_ORIENT_AUTO: number;\\n    readonly SVG_MARKER_ORIENT_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMarkerElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMarkerElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGMarkerElement: {\\n    prototype: SVGMarkerElement;\\n    new(): SVGMarkerElement;\\n    readonly SVG_MARKERUNITS_STROKEWIDTH: number;\\n    readonly SVG_MARKERUNITS_UNKNOWN: number;\\n    readonly SVG_MARKERUNITS_USERSPACEONUSE: number;\\n    readonly SVG_MARKER_ORIENT_ANGLE: number;\\n    readonly SVG_MARKER_ORIENT_AUTO: number;\\n    readonly SVG_MARKER_ORIENT_UNKNOWN: number;\\n};\\n\\ninterface SVGMaskElement extends SVGElement, SVGTests {\\n    readonly height: SVGAnimatedLength;\\n    readonly maskContentUnits: SVGAnimatedEnumeration;\\n    readonly maskUnits: SVGAnimatedEnumeration;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMaskElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMaskElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGMaskElement: {\\n    prototype: SVGMaskElement;\\n    new(): SVGMaskElement;\\n};\\n\\ninterface SVGMetadataElement extends SVGElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMetadataElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMetadataElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGMetadataElement: {\\n    prototype: SVGMetadataElement;\\n    new(): SVGMetadataElement;\\n};\\n\\ninterface SVGNumber {\\n    value: number;\\n}\\n\\ndeclare var SVGNumber: {\\n    prototype: SVGNumber;\\n    new(): SVGNumber;\\n};\\n\\ninterface SVGNumberList {\\n    readonly length: number;\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGNumber): SVGNumber;\\n    clear(): void;\\n    getItem(index: number): SVGNumber;\\n    initialize(newItem: SVGNumber): SVGNumber;\\n    insertItemBefore(newItem: SVGNumber, index: number): SVGNumber;\\n    removeItem(index: number): SVGNumber;\\n    replaceItem(newItem: SVGNumber, index: number): SVGNumber;\\n    [index: number]: SVGNumber;\\n}\\n\\ndeclare var SVGNumberList: {\\n    prototype: SVGNumberList;\\n    new(): SVGNumberList;\\n};\\n\\ninterface SVGPathElement extends SVGGraphicsElement {\\n    /** @deprecated */\\n    readonly pathSegList: SVGPathSegList;\\n    /** @deprecated */\\n    createSVGPathSegArcAbs(x: number, y: number, r1: number, r2: number, angle: number, largeArcFlag: boolean, sweepFlag: boolean): SVGPathSegArcAbs;\\n    /** @deprecated */\\n    createSVGPathSegArcRel(x: number, y: number, r1: number, r2: number, angle: number, largeArcFlag: boolean, sweepFlag: boolean): SVGPathSegArcRel;\\n    /** @deprecated */\\n    createSVGPathSegClosePath(): SVGPathSegClosePath;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicAbs(x: number, y: number, x1: number, y1: number, x2: number, y2: number): SVGPathSegCurvetoCubicAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicRel(x: number, y: number, x1: number, y1: number, x2: number, y2: number): SVGPathSegCurvetoCubicRel;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicSmoothAbs(x: number, y: number, x2: number, y2: number): SVGPathSegCurvetoCubicSmoothAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoCubicSmoothRel(x: number, y: number, x2: number, y2: number): SVGPathSegCurvetoCubicSmoothRel;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticAbs(x: number, y: number, x1: number, y1: number): SVGPathSegCurvetoQuadraticAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticRel(x: number, y: number, x1: number, y1: number): SVGPathSegCurvetoQuadraticRel;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticSmoothAbs(x: number, y: number): SVGPathSegCurvetoQuadraticSmoothAbs;\\n    /** @deprecated */\\n    createSVGPathSegCurvetoQuadraticSmoothRel(x: number, y: number): SVGPathSegCurvetoQuadraticSmoothRel;\\n    /** @deprecated */\\n    createSVGPathSegLinetoAbs(x: number, y: number): SVGPathSegLinetoAbs;\\n    /** @deprecated */\\n    createSVGPathSegLinetoHorizontalAbs(x: number): SVGPathSegLinetoHorizontalAbs;\\n    /** @deprecated */\\n    createSVGPathSegLinetoHorizontalRel(x: number): SVGPathSegLinetoHorizontalRel;\\n    /** @deprecated */\\n    createSVGPathSegLinetoRel(x: number, y: number): SVGPathSegLinetoRel;\\n    /** @deprecated */\\n    createSVGPathSegLinetoVerticalAbs(y: number): SVGPathSegLinetoVerticalAbs;\\n    /** @deprecated */\\n    createSVGPathSegLinetoVerticalRel(y: number): SVGPathSegLinetoVerticalRel;\\n    /** @deprecated */\\n    createSVGPathSegMovetoAbs(x: number, y: number): SVGPathSegMovetoAbs;\\n    /** @deprecated */\\n    createSVGPathSegMovetoRel(x: number, y: number): SVGPathSegMovetoRel;\\n    /** @deprecated */\\n    getPathSegAtLength(distance: number): number;\\n    getPointAtLength(distance: number): SVGPoint;\\n    getTotalLength(): number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPathElement: {\\n    prototype: SVGPathElement;\\n    new(): SVGPathElement;\\n};\\n\\ninterface SVGPathSeg {\\n    readonly pathSegType: number;\\n    readonly pathSegTypeAsLetter: string;\\n    readonly PATHSEG_ARC_ABS: number;\\n    readonly PATHSEG_ARC_REL: number;\\n    readonly PATHSEG_CLOSEPATH: number;\\n    readonly PATHSEG_CURVETO_CUBIC_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_REL: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL: number;\\n    readonly PATHSEG_LINETO_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_REL: number;\\n    readonly PATHSEG_LINETO_REL: number;\\n    readonly PATHSEG_LINETO_VERTICAL_ABS: number;\\n    readonly PATHSEG_LINETO_VERTICAL_REL: number;\\n    readonly PATHSEG_MOVETO_ABS: number;\\n    readonly PATHSEG_MOVETO_REL: number;\\n    readonly PATHSEG_UNKNOWN: number;\\n}\\n\\ndeclare var SVGPathSeg: {\\n    prototype: SVGPathSeg;\\n    new(): SVGPathSeg;\\n    readonly PATHSEG_ARC_ABS: number;\\n    readonly PATHSEG_ARC_REL: number;\\n    readonly PATHSEG_CLOSEPATH: number;\\n    readonly PATHSEG_CURVETO_CUBIC_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_REL: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_CUBIC_SMOOTH_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_REL: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS: number;\\n    readonly PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL: number;\\n    readonly PATHSEG_LINETO_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_ABS: number;\\n    readonly PATHSEG_LINETO_HORIZONTAL_REL: number;\\n    readonly PATHSEG_LINETO_REL: number;\\n    readonly PATHSEG_LINETO_VERTICAL_ABS: number;\\n    readonly PATHSEG_LINETO_VERTICAL_REL: number;\\n    readonly PATHSEG_MOVETO_ABS: number;\\n    readonly PATHSEG_MOVETO_REL: number;\\n    readonly PATHSEG_UNKNOWN: number;\\n};\\n\\ninterface SVGPathSegArcAbs extends SVGPathSeg {\\n    angle: number;\\n    largeArcFlag: boolean;\\n    r1: number;\\n    r2: number;\\n    sweepFlag: boolean;\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegArcAbs: {\\n    prototype: SVGPathSegArcAbs;\\n    new(): SVGPathSegArcAbs;\\n};\\n\\ninterface SVGPathSegArcRel extends SVGPathSeg {\\n    angle: number;\\n    largeArcFlag: boolean;\\n    r1: number;\\n    r2: number;\\n    sweepFlag: boolean;\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegArcRel: {\\n    prototype: SVGPathSegArcRel;\\n    new(): SVGPathSegArcRel;\\n};\\n\\ninterface SVGPathSegClosePath extends SVGPathSeg {\\n}\\n\\ndeclare var SVGPathSegClosePath: {\\n    prototype: SVGPathSegClosePath;\\n    new(): SVGPathSegClosePath;\\n};\\n\\ninterface SVGPathSegCurvetoCubicAbs extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    x2: number;\\n    y: number;\\n    y1: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicAbs: {\\n    prototype: SVGPathSegCurvetoCubicAbs;\\n    new(): SVGPathSegCurvetoCubicAbs;\\n};\\n\\ninterface SVGPathSegCurvetoCubicRel extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    x2: number;\\n    y: number;\\n    y1: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicRel: {\\n    prototype: SVGPathSegCurvetoCubicRel;\\n    new(): SVGPathSegCurvetoCubicRel;\\n};\\n\\ninterface SVGPathSegCurvetoCubicSmoothAbs extends SVGPathSeg {\\n    x: number;\\n    x2: number;\\n    y: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicSmoothAbs: {\\n    prototype: SVGPathSegCurvetoCubicSmoothAbs;\\n    new(): SVGPathSegCurvetoCubicSmoothAbs;\\n};\\n\\ninterface SVGPathSegCurvetoCubicSmoothRel extends SVGPathSeg {\\n    x: number;\\n    x2: number;\\n    y: number;\\n    y2: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoCubicSmoothRel: {\\n    prototype: SVGPathSegCurvetoCubicSmoothRel;\\n    new(): SVGPathSegCurvetoCubicSmoothRel;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticAbs extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    y: number;\\n    y1: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticAbs: {\\n    prototype: SVGPathSegCurvetoQuadraticAbs;\\n    new(): SVGPathSegCurvetoQuadraticAbs;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticRel extends SVGPathSeg {\\n    x: number;\\n    x1: number;\\n    y: number;\\n    y1: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticRel: {\\n    prototype: SVGPathSegCurvetoQuadraticRel;\\n    new(): SVGPathSegCurvetoQuadraticRel;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticSmoothAbs extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticSmoothAbs: {\\n    prototype: SVGPathSegCurvetoQuadraticSmoothAbs;\\n    new(): SVGPathSegCurvetoQuadraticSmoothAbs;\\n};\\n\\ninterface SVGPathSegCurvetoQuadraticSmoothRel extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegCurvetoQuadraticSmoothRel: {\\n    prototype: SVGPathSegCurvetoQuadraticSmoothRel;\\n    new(): SVGPathSegCurvetoQuadraticSmoothRel;\\n};\\n\\ninterface SVGPathSegLinetoAbs extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoAbs: {\\n    prototype: SVGPathSegLinetoAbs;\\n    new(): SVGPathSegLinetoAbs;\\n};\\n\\ninterface SVGPathSegLinetoHorizontalAbs extends SVGPathSeg {\\n    x: number;\\n}\\n\\ndeclare var SVGPathSegLinetoHorizontalAbs: {\\n    prototype: SVGPathSegLinetoHorizontalAbs;\\n    new(): SVGPathSegLinetoHorizontalAbs;\\n};\\n\\ninterface SVGPathSegLinetoHorizontalRel extends SVGPathSeg {\\n    x: number;\\n}\\n\\ndeclare var SVGPathSegLinetoHorizontalRel: {\\n    prototype: SVGPathSegLinetoHorizontalRel;\\n    new(): SVGPathSegLinetoHorizontalRel;\\n};\\n\\ninterface SVGPathSegLinetoRel extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoRel: {\\n    prototype: SVGPathSegLinetoRel;\\n    new(): SVGPathSegLinetoRel;\\n};\\n\\ninterface SVGPathSegLinetoVerticalAbs extends SVGPathSeg {\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoVerticalAbs: {\\n    prototype: SVGPathSegLinetoVerticalAbs;\\n    new(): SVGPathSegLinetoVerticalAbs;\\n};\\n\\ninterface SVGPathSegLinetoVerticalRel extends SVGPathSeg {\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegLinetoVerticalRel: {\\n    prototype: SVGPathSegLinetoVerticalRel;\\n    new(): SVGPathSegLinetoVerticalRel;\\n};\\n\\ninterface SVGPathSegList {\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGPathSeg): SVGPathSeg;\\n    clear(): void;\\n    getItem(index: number): SVGPathSeg;\\n    initialize(newItem: SVGPathSeg): SVGPathSeg;\\n    insertItemBefore(newItem: SVGPathSeg, index: number): SVGPathSeg;\\n    removeItem(index: number): SVGPathSeg;\\n    replaceItem(newItem: SVGPathSeg, index: number): SVGPathSeg;\\n}\\n\\ndeclare var SVGPathSegList: {\\n    prototype: SVGPathSegList;\\n    new(): SVGPathSegList;\\n};\\n\\ninterface SVGPathSegMovetoAbs extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegMovetoAbs: {\\n    prototype: SVGPathSegMovetoAbs;\\n    new(): SVGPathSegMovetoAbs;\\n};\\n\\ninterface SVGPathSegMovetoRel extends SVGPathSeg {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var SVGPathSegMovetoRel: {\\n    prototype: SVGPathSegMovetoRel;\\n    new(): SVGPathSegMovetoRel;\\n};\\n\\ninterface SVGPatternElement extends SVGElement, SVGTests, SVGFitToViewBox, SVGURIReference {\\n    readonly height: SVGAnimatedLength;\\n    readonly patternContentUnits: SVGAnimatedEnumeration;\\n    readonly patternTransform: SVGAnimatedTransformList;\\n    readonly patternUnits: SVGAnimatedEnumeration;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPatternElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPatternElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPatternElement: {\\n    prototype: SVGPatternElement;\\n    new(): SVGPatternElement;\\n};\\n\\ninterface SVGPointList {\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGPoint): SVGPoint;\\n    clear(): void;\\n    getItem(index: number): SVGPoint;\\n    initialize(newItem: SVGPoint): SVGPoint;\\n    insertItemBefore(newItem: SVGPoint, index: number): SVGPoint;\\n    removeItem(index: number): SVGPoint;\\n    replaceItem(newItem: SVGPoint, index: number): SVGPoint;\\n}\\n\\ndeclare var SVGPointList: {\\n    prototype: SVGPointList;\\n    new(): SVGPointList;\\n};\\n\\ninterface SVGPolygonElement extends SVGGraphicsElement, SVGAnimatedPoints {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolygonElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolygonElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPolygonElement: {\\n    prototype: SVGPolygonElement;\\n    new(): SVGPolygonElement;\\n};\\n\\ninterface SVGPolylineElement extends SVGGraphicsElement, SVGAnimatedPoints {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolylineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolylineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGPolylineElement: {\\n    prototype: SVGPolylineElement;\\n    new(): SVGPolylineElement;\\n};\\n\\ninterface SVGPreserveAspectRatio {\\n    align: number;\\n    meetOrSlice: number;\\n    readonly SVG_MEETORSLICE_MEET: number;\\n    readonly SVG_MEETORSLICE_SLICE: number;\\n    readonly SVG_MEETORSLICE_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_NONE: number;\\n    readonly SVG_PRESERVEASPECTRATIO_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMIN: number;\\n}\\n\\ndeclare var SVGPreserveAspectRatio: {\\n    prototype: SVGPreserveAspectRatio;\\n    new(): SVGPreserveAspectRatio;\\n    readonly SVG_MEETORSLICE_MEET: number;\\n    readonly SVG_MEETORSLICE_SLICE: number;\\n    readonly SVG_MEETORSLICE_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_NONE: number;\\n    readonly SVG_PRESERVEASPECTRATIO_UNKNOWN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMIN: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMAX: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMID: number;\\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMIN: number;\\n};\\n\\ninterface SVGRadialGradientElement extends SVGGradientElement {\\n    readonly cx: SVGAnimatedLength;\\n    readonly cy: SVGAnimatedLength;\\n    readonly fx: SVGAnimatedLength;\\n    readonly fy: SVGAnimatedLength;\\n    readonly r: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRadialGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRadialGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGRadialGradientElement: {\\n    prototype: SVGRadialGradientElement;\\n    new(): SVGRadialGradientElement;\\n};\\n\\ninterface SVGRectElement extends SVGGraphicsElement {\\n    readonly height: SVGAnimatedLength;\\n    readonly rx: SVGAnimatedLength;\\n    readonly ry: SVGAnimatedLength;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGRectElement: {\\n    prototype: SVGRectElement;\\n    new(): SVGRectElement;\\n};\\n\\ninterface SVGSVGElementEventMap extends SVGElementEventMap {\\n    \"SVGUnload\": Event;\\n    \"SVGZoom\": SVGZoomEvent;\\n}\\n\\ninterface SVGSVGElement extends SVGGraphicsElement, DocumentEvent, SVGFitToViewBox, SVGZoomAndPan {\\n    /** @deprecated */\\n    contentScriptType: string;\\n    /** @deprecated */\\n    contentStyleType: string;\\n    currentScale: number;\\n    readonly currentTranslate: SVGPoint;\\n    readonly height: SVGAnimatedLength;\\n    onunload: ((this: SVGSVGElement, ev: Event) => any) | null;\\n    onzoom: ((this: SVGSVGElement, ev: SVGZoomEvent) => any) | null;\\n    /** @deprecated */\\n    readonly pixelUnitToMillimeterX: number;\\n    /** @deprecated */\\n    readonly pixelUnitToMillimeterY: number;\\n    /** @deprecated */\\n    readonly screenPixelToMillimeterX: number;\\n    /** @deprecated */\\n    readonly screenPixelToMillimeterY: number;\\n    /** @deprecated */\\n    readonly viewport: SVGRect;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    checkEnclosure(element: SVGElement, rect: SVGRect): boolean;\\n    checkIntersection(element: SVGElement, rect: SVGRect): boolean;\\n    createSVGAngle(): SVGAngle;\\n    createSVGLength(): SVGLength;\\n    createSVGMatrix(): SVGMatrix;\\n    createSVGNumber(): SVGNumber;\\n    createSVGPoint(): SVGPoint;\\n    createSVGRect(): SVGRect;\\n    createSVGTransform(): SVGTransform;\\n    createSVGTransformFromMatrix(matrix: SVGMatrix): SVGTransform;\\n    deselectAll(): void;\\n    /** @deprecated */\\n    forceRedraw(): void;\\n    getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\\n    /** @deprecated */\\n    getCurrentTime(): number;\\n    getElementById(elementId: string): Element;\\n    getEnclosureList(rect: SVGRect, referenceElement: SVGElement): NodeListOf<SVGCircleElement | SVGEllipseElement | SVGImageElement | SVGLineElement | SVGPathElement | SVGPolygonElement | SVGPolylineElement | SVGRectElement | SVGTextElement | SVGUseElement>;\\n    getIntersectionList(rect: SVGRect, referenceElement: SVGElement): NodeListOf<SVGCircleElement | SVGEllipseElement | SVGImageElement | SVGLineElement | SVGPathElement | SVGPolygonElement | SVGPolylineElement | SVGRectElement | SVGTextElement | SVGUseElement>;\\n    /** @deprecated */\\n    pauseAnimations(): void;\\n    /** @deprecated */\\n    setCurrentTime(seconds: number): void;\\n    /** @deprecated */\\n    suspendRedraw(maxWaitMilliseconds: number): number;\\n    /** @deprecated */\\n    unpauseAnimations(): void;\\n    /** @deprecated */\\n    unsuspendRedraw(suspendHandleID: number): void;\\n    /** @deprecated */\\n    unsuspendRedrawAll(): void;\\n    addEventListener<K extends keyof SVGSVGElementEventMap>(type: K, listener: (this: SVGSVGElement, ev: SVGSVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGSVGElementEventMap>(type: K, listener: (this: SVGSVGElement, ev: SVGSVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGSVGElement: {\\n    prototype: SVGSVGElement;\\n    new(): SVGSVGElement;\\n    readonly SVG_ZOOMANDPAN_DISABLE: number;\\n    readonly SVG_ZOOMANDPAN_MAGNIFY: number;\\n    readonly SVG_ZOOMANDPAN_UNKNOWN: number;\\n};\\n\\ninterface SVGScriptElement extends SVGElement, SVGURIReference {\\n    type: string;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGScriptElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGScriptElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGScriptElement: {\\n    prototype: SVGScriptElement;\\n    new(): SVGScriptElement;\\n};\\n\\ninterface SVGStopElement extends SVGElement {\\n    readonly offset: SVGAnimatedNumber;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStopElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStopElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGStopElement: {\\n    prototype: SVGStopElement;\\n    new(): SVGStopElement;\\n};\\n\\ninterface SVGStringList {\\n    readonly length: number;\\n    readonly numberOfItems: number;\\n    appendItem(newItem: string): string;\\n    clear(): void;\\n    getItem(index: number): string;\\n    initialize(newItem: string): string;\\n    insertItemBefore(newItem: string, index: number): string;\\n    removeItem(index: number): string;\\n    replaceItem(newItem: string, index: number): string;\\n    [index: number]: string;\\n}\\n\\ndeclare var SVGStringList: {\\n    prototype: SVGStringList;\\n    new(): SVGStringList;\\n};\\n\\ninterface SVGStyleElement extends SVGElement {\\n    disabled: boolean;\\n    media: string;\\n    title: string;\\n    type: string;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStyleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStyleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGStyleElement: {\\n    prototype: SVGStyleElement;\\n    new(): SVGStyleElement;\\n};\\n\\ninterface SVGSwitchElement extends SVGGraphicsElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSwitchElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSwitchElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGSwitchElement: {\\n    prototype: SVGSwitchElement;\\n    new(): SVGSwitchElement;\\n};\\n\\ninterface SVGSymbolElement extends SVGElement, SVGFitToViewBox {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSymbolElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSymbolElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGSymbolElement: {\\n    prototype: SVGSymbolElement;\\n    new(): SVGSymbolElement;\\n};\\n\\ninterface SVGTSpanElement extends SVGTextPositioningElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTSpanElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTSpanElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTSpanElement: {\\n    prototype: SVGTSpanElement;\\n    new(): SVGTSpanElement;\\n};\\n\\ninterface SVGTests {\\n    readonly requiredExtensions: SVGStringList;\\n    readonly systemLanguage: SVGStringList;\\n}\\n\\ninterface SVGTextContentElement extends SVGGraphicsElement {\\n    readonly lengthAdjust: SVGAnimatedEnumeration;\\n    readonly textLength: SVGAnimatedLength;\\n    getCharNumAtPosition(point: SVGPoint): number;\\n    getComputedTextLength(): number;\\n    getEndPositionOfChar(charnum: number): SVGPoint;\\n    getExtentOfChar(charnum: number): SVGRect;\\n    getNumberOfChars(): number;\\n    getRotationOfChar(charnum: number): number;\\n    getStartPositionOfChar(charnum: number): SVGPoint;\\n    getSubStringLength(charnum: number, nchars: number): number;\\n    selectSubString(charnum: number, nchars: number): void;\\n    readonly LENGTHADJUST_SPACING: number;\\n    readonly LENGTHADJUST_SPACINGANDGLYPHS: number;\\n    readonly LENGTHADJUST_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextContentElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextContentElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextContentElement: {\\n    prototype: SVGTextContentElement;\\n    new(): SVGTextContentElement;\\n    readonly LENGTHADJUST_SPACING: number;\\n    readonly LENGTHADJUST_SPACINGANDGLYPHS: number;\\n    readonly LENGTHADJUST_UNKNOWN: number;\\n};\\n\\ninterface SVGTextElement extends SVGTextPositioningElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextElement: {\\n    prototype: SVGTextElement;\\n    new(): SVGTextElement;\\n};\\n\\ninterface SVGTextPathElement extends SVGTextContentElement, SVGURIReference {\\n    readonly method: SVGAnimatedEnumeration;\\n    readonly spacing: SVGAnimatedEnumeration;\\n    readonly startOffset: SVGAnimatedLength;\\n    readonly TEXTPATH_METHODTYPE_ALIGN: number;\\n    readonly TEXTPATH_METHODTYPE_STRETCH: number;\\n    readonly TEXTPATH_METHODTYPE_UNKNOWN: number;\\n    readonly TEXTPATH_SPACINGTYPE_AUTO: number;\\n    readonly TEXTPATH_SPACINGTYPE_EXACT: number;\\n    readonly TEXTPATH_SPACINGTYPE_UNKNOWN: number;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextPathElement: {\\n    prototype: SVGTextPathElement;\\n    new(): SVGTextPathElement;\\n    readonly TEXTPATH_METHODTYPE_ALIGN: number;\\n    readonly TEXTPATH_METHODTYPE_STRETCH: number;\\n    readonly TEXTPATH_METHODTYPE_UNKNOWN: number;\\n    readonly TEXTPATH_SPACINGTYPE_AUTO: number;\\n    readonly TEXTPATH_SPACINGTYPE_EXACT: number;\\n    readonly TEXTPATH_SPACINGTYPE_UNKNOWN: number;\\n};\\n\\ninterface SVGTextPositioningElement extends SVGTextContentElement {\\n    readonly dx: SVGAnimatedLengthList;\\n    readonly dy: SVGAnimatedLengthList;\\n    readonly rotate: SVGAnimatedNumberList;\\n    readonly x: SVGAnimatedLengthList;\\n    readonly y: SVGAnimatedLengthList;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPositioningElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPositioningElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTextPositioningElement: {\\n    prototype: SVGTextPositioningElement;\\n    new(): SVGTextPositioningElement;\\n};\\n\\ninterface SVGTitleElement extends SVGElement {\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTitleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTitleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGTitleElement: {\\n    prototype: SVGTitleElement;\\n    new(): SVGTitleElement;\\n};\\n\\ninterface SVGTransform {\\n    readonly angle: number;\\n    readonly matrix: SVGMatrix;\\n    readonly type: number;\\n    setMatrix(matrix: SVGMatrix): void;\\n    setRotate(angle: number, cx: number, cy: number): void;\\n    setScale(sx: number, sy: number): void;\\n    setSkewX(angle: number): void;\\n    setSkewY(angle: number): void;\\n    setTranslate(tx: number, ty: number): void;\\n    readonly SVG_TRANSFORM_MATRIX: number;\\n    readonly SVG_TRANSFORM_ROTATE: number;\\n    readonly SVG_TRANSFORM_SCALE: number;\\n    readonly SVG_TRANSFORM_SKEWX: number;\\n    readonly SVG_TRANSFORM_SKEWY: number;\\n    readonly SVG_TRANSFORM_TRANSLATE: number;\\n    readonly SVG_TRANSFORM_UNKNOWN: number;\\n}\\n\\ndeclare var SVGTransform: {\\n    prototype: SVGTransform;\\n    new(): SVGTransform;\\n    readonly SVG_TRANSFORM_MATRIX: number;\\n    readonly SVG_TRANSFORM_ROTATE: number;\\n    readonly SVG_TRANSFORM_SCALE: number;\\n    readonly SVG_TRANSFORM_SKEWX: number;\\n    readonly SVG_TRANSFORM_SKEWY: number;\\n    readonly SVG_TRANSFORM_TRANSLATE: number;\\n    readonly SVG_TRANSFORM_UNKNOWN: number;\\n};\\n\\ninterface SVGTransformList {\\n    readonly numberOfItems: number;\\n    appendItem(newItem: SVGTransform): SVGTransform;\\n    clear(): void;\\n    consolidate(): SVGTransform;\\n    createSVGTransformFromMatrix(matrix: SVGMatrix): SVGTransform;\\n    getItem(index: number): SVGTransform;\\n    initialize(newItem: SVGTransform): SVGTransform;\\n    insertItemBefore(newItem: SVGTransform, index: number): SVGTransform;\\n    removeItem(index: number): SVGTransform;\\n    replaceItem(newItem: SVGTransform, index: number): SVGTransform;\\n}\\n\\ndeclare var SVGTransformList: {\\n    prototype: SVGTransformList;\\n    new(): SVGTransformList;\\n};\\n\\ninterface SVGURIReference {\\n    readonly href: SVGAnimatedString;\\n}\\n\\ninterface SVGUnitTypes {\\n    readonly SVG_UNIT_TYPE_OBJECTBOUNDINGBOX: number;\\n    readonly SVG_UNIT_TYPE_UNKNOWN: number;\\n    readonly SVG_UNIT_TYPE_USERSPACEONUSE: number;\\n}\\n\\ndeclare var SVGUnitTypes: {\\n    prototype: SVGUnitTypes;\\n    new(): SVGUnitTypes;\\n    readonly SVG_UNIT_TYPE_OBJECTBOUNDINGBOX: number;\\n    readonly SVG_UNIT_TYPE_UNKNOWN: number;\\n    readonly SVG_UNIT_TYPE_USERSPACEONUSE: number;\\n};\\n\\ninterface SVGUseElement extends SVGGraphicsElement, SVGURIReference {\\n    readonly animatedInstanceRoot: SVGElementInstance | null;\\n    readonly height: SVGAnimatedLength;\\n    readonly instanceRoot: SVGElementInstance | null;\\n    readonly width: SVGAnimatedLength;\\n    readonly x: SVGAnimatedLength;\\n    readonly y: SVGAnimatedLength;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGUseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGUseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGUseElement: {\\n    prototype: SVGUseElement;\\n    new(): SVGUseElement;\\n};\\n\\ninterface SVGViewElement extends SVGElement, SVGFitToViewBox, SVGZoomAndPan {\\n    /** @deprecated */\\n    readonly viewTarget: SVGStringList;\\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGViewElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGViewElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SVGViewElement: {\\n    prototype: SVGViewElement;\\n    new(): SVGViewElement;\\n    readonly SVG_ZOOMANDPAN_DISABLE: number;\\n    readonly SVG_ZOOMANDPAN_MAGNIFY: number;\\n    readonly SVG_ZOOMANDPAN_UNKNOWN: number;\\n};\\n\\ninterface SVGZoomAndPan {\\n    readonly zoomAndPan: number;\\n}\\n\\ndeclare var SVGZoomAndPan: {\\n    readonly SVG_ZOOMANDPAN_DISABLE: number;\\n    readonly SVG_ZOOMANDPAN_MAGNIFY: number;\\n    readonly SVG_ZOOMANDPAN_UNKNOWN: number;\\n};\\n\\ninterface SVGZoomEvent extends UIEvent {\\n    readonly newScale: number;\\n    readonly newTranslate: SVGPoint;\\n    readonly previousScale: number;\\n    readonly previousTranslate: SVGPoint;\\n    readonly zoomRectScreen: SVGRect;\\n}\\n\\ndeclare var SVGZoomEvent: {\\n    prototype: SVGZoomEvent;\\n    new(): SVGZoomEvent;\\n};\\n\\ninterface ScopedCredential {\\n    readonly id: ArrayBuffer;\\n    readonly type: ScopedCredentialType;\\n}\\n\\ndeclare var ScopedCredential: {\\n    prototype: ScopedCredential;\\n    new(): ScopedCredential;\\n};\\n\\ninterface ScopedCredentialInfo {\\n    readonly credential: ScopedCredential;\\n    readonly publicKey: CryptoKey;\\n}\\n\\ndeclare var ScopedCredentialInfo: {\\n    prototype: ScopedCredentialInfo;\\n    new(): ScopedCredentialInfo;\\n};\\n\\ninterface Screen {\\n    readonly availHeight: number;\\n    readonly availWidth: number;\\n    readonly colorDepth: number;\\n    readonly height: number;\\n    readonly orientation: ScreenOrientation;\\n    readonly pixelDepth: number;\\n    readonly width: number;\\n}\\n\\ndeclare var Screen: {\\n    prototype: Screen;\\n    new(): Screen;\\n};\\n\\ninterface ScreenOrientationEventMap {\\n    \"change\": Event;\\n}\\n\\ninterface ScreenOrientation extends EventTarget {\\n    readonly angle: number;\\n    onchange: ((this: ScreenOrientation, ev: Event) => any) | null;\\n    readonly type: OrientationType;\\n    lock(orientation: OrientationLockType): Promise<void>;\\n    unlock(): void;\\n    addEventListener<K extends keyof ScreenOrientationEventMap>(type: K, listener: (this: ScreenOrientation, ev: ScreenOrientationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ScreenOrientationEventMap>(type: K, listener: (this: ScreenOrientation, ev: ScreenOrientationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ScreenOrientation: {\\n    prototype: ScreenOrientation;\\n    new(): ScreenOrientation;\\n};\\n\\ninterface ScriptProcessorNodeEventMap {\\n    \"audioprocess\": AudioProcessingEvent;\\n}\\n\\ninterface ScriptProcessorNode extends AudioNode {\\n    /** @deprecated */\\n    readonly bufferSize: number;\\n    /** @deprecated */\\n    onaudioprocess: ((this: ScriptProcessorNode, ev: AudioProcessingEvent) => any) | null;\\n    addEventListener<K extends keyof ScriptProcessorNodeEventMap>(type: K, listener: (this: ScriptProcessorNode, ev: ScriptProcessorNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ScriptProcessorNodeEventMap>(type: K, listener: (this: ScriptProcessorNode, ev: ScriptProcessorNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ScriptProcessorNode: {\\n    prototype: ScriptProcessorNode;\\n    new(): ScriptProcessorNode;\\n};\\n\\ninterface SecurityPolicyViolationEvent extends Event {\\n    readonly blockedURI: string;\\n    readonly columnNumber: number;\\n    readonly documentURI: string;\\n    readonly effectiveDirective: string;\\n    readonly lineNumber: number;\\n    readonly originalPolicy: string;\\n    readonly referrer: string;\\n    readonly sourceFile: string;\\n    readonly statusCode: number;\\n    readonly violatedDirective: string;\\n}\\n\\ndeclare var SecurityPolicyViolationEvent: {\\n    prototype: SecurityPolicyViolationEvent;\\n    new(type: string, eventInitDict?: SecurityPolicyViolationEventInit): SecurityPolicyViolationEvent;\\n};\\n\\ninterface Selection {\\n    readonly anchorNode: Node;\\n    readonly anchorOffset: number;\\n    readonly baseNode: Node;\\n    readonly baseOffset: number;\\n    readonly extentNode: Node;\\n    readonly extentOffset: number;\\n    readonly focusNode: Node;\\n    readonly focusOffset: number;\\n    readonly isCollapsed: boolean;\\n    readonly rangeCount: number;\\n    readonly type: string;\\n    addRange(range: Range): void;\\n    collapse(parentNode: Node, offset: number): void;\\n    collapseToEnd(): void;\\n    collapseToStart(): void;\\n    containsNode(node: Node, partlyContained: boolean): boolean;\\n    deleteFromDocument(): void;\\n    empty(): void;\\n    extend(newNode: Node, offset: number): void;\\n    getRangeAt(index: number): Range;\\n    removeAllRanges(): void;\\n    removeRange(range: Range): void;\\n    selectAllChildren(parentNode: Node): void;\\n    setBaseAndExtent(baseNode: Node, baseOffset: number, extentNode: Node, extentOffset: number): void;\\n    setPosition(parentNode: Node, offset: number): void;\\n    toString(): string;\\n}\\n\\ndeclare var Selection: {\\n    prototype: Selection;\\n    new(): Selection;\\n};\\n\\ninterface ServiceUIFrameContext {\\n    getCachedFrameMessage(key: string): string;\\n    postFrameMessage(key: string, data: string): void;\\n}\\ndeclare var ServiceUIFrameContext: ServiceUIFrameContext;\\n\\ninterface ServiceWorkerEventMap extends AbstractWorkerEventMap {\\n    \"statechange\": Event;\\n}\\n\\ninterface ServiceWorker extends EventTarget, AbstractWorker {\\n    onstatechange: ((this: ServiceWorker, ev: Event) => any) | null;\\n    readonly scriptURL: string;\\n    readonly state: ServiceWorkerState;\\n    postMessage(message: any, transfer?: Transferable[]): void;\\n    addEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ServiceWorker: {\\n    prototype: ServiceWorker;\\n    new(): ServiceWorker;\\n};\\n\\ninterface ServiceWorkerContainerEventMap {\\n    \"controllerchange\": Event;\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n}\\n\\ninterface ServiceWorkerContainer extends EventTarget {\\n    readonly controller: ServiceWorker | null;\\n    oncontrollerchange: ((this: ServiceWorkerContainer, ev: Event) => any) | null;\\n    onmessage: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\\n    readonly ready: Promise<ServiceWorkerRegistration>;\\n    getRegistration(clientURL?: string): Promise<ServiceWorkerRegistration | undefined>;\\n    getRegistrations(): Promise<ReadonlyArray<ServiceWorkerRegistration>>;\\n    register(scriptURL: string, options?: RegistrationOptions): Promise<ServiceWorkerRegistration>;\\n    startMessages(): void;\\n    addEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ServiceWorkerContainer: {\\n    prototype: ServiceWorkerContainer;\\n    new(): ServiceWorkerContainer;\\n};\\n\\ninterface ServiceWorkerMessageEvent extends Event {\\n    readonly data: any;\\n    readonly lastEventId: string;\\n    readonly origin: string;\\n    readonly ports: ReadonlyArray<MessagePort> | null;\\n    readonly source: ServiceWorker | MessagePort | null;\\n}\\n\\ndeclare var ServiceWorkerMessageEvent: {\\n    prototype: ServiceWorkerMessageEvent;\\n    new(type: string, eventInitDict?: ServiceWorkerMessageEventInit): ServiceWorkerMessageEvent;\\n};\\n\\ninterface ServiceWorkerRegistrationEventMap {\\n    \"updatefound\": Event;\\n}\\n\\ninterface ServiceWorkerRegistration extends EventTarget {\\n    readonly active: ServiceWorker | null;\\n    readonly installing: ServiceWorker | null;\\n    readonly navigationPreload: NavigationPreloadManager;\\n    onupdatefound: ((this: ServiceWorkerRegistration, ev: Event) => any) | null;\\n    readonly pushManager: PushManager;\\n    readonly scope: string;\\n    readonly sync: SyncManager;\\n    readonly updateViaCache: ServiceWorkerUpdateViaCache;\\n    readonly waiting: ServiceWorker | null;\\n    getNotifications(filter?: GetNotificationOptions): Promise<Notification[]>;\\n    showNotification(title: string, options?: NotificationOptions): Promise<void>;\\n    unregister(): Promise<boolean>;\\n    update(): Promise<void>;\\n    addEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var ServiceWorkerRegistration: {\\n    prototype: ServiceWorkerRegistration;\\n    new(): ServiceWorkerRegistration;\\n};\\n\\ninterface ShadowRoot extends DocumentOrShadowRoot, DocumentFragment, DocumentOrShadowRoot {\\n    readonly host: Element;\\n    innerHTML: string;\\n    readonly mode: ShadowRootMode;\\n}\\n\\ninterface ShadowRootInit {\\n    delegatesFocus?: boolean;\\n    mode: \"open\" | \"closed\";\\n}\\n\\ninterface Slotable {\\n    readonly assignedSlot: HTMLSlotElement | null;\\n}\\n\\ninterface SourceBuffer extends EventTarget {\\n    appendWindowEnd: number;\\n    appendWindowStart: number;\\n    readonly audioTracks: AudioTrackList;\\n    readonly buffered: TimeRanges;\\n    mode: AppendMode;\\n    readonly textTracks: TextTrackList;\\n    timestampOffset: number;\\n    readonly updating: boolean;\\n    readonly videoTracks: VideoTrackList;\\n    abort(): void;\\n    appendBuffer(data: ArrayBuffer | Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | null): void;\\n    appendStream(stream: MSStream, maxSize?: number): void;\\n    remove(start: number, end: number): void;\\n}\\n\\ndeclare var SourceBuffer: {\\n    prototype: SourceBuffer;\\n    new(): SourceBuffer;\\n};\\n\\ninterface SourceBufferList extends EventTarget {\\n    readonly length: number;\\n    item(index: number): SourceBuffer;\\n    [index: number]: SourceBuffer;\\n}\\n\\ndeclare var SourceBufferList: {\\n    prototype: SourceBufferList;\\n    new(): SourceBufferList;\\n};\\n\\ninterface SpeechGrammar {\\n    src: string;\\n    weight: number;\\n}\\n\\ndeclare var SpeechGrammar: {\\n    prototype: SpeechGrammar;\\n    new(): SpeechGrammar;\\n};\\n\\ninterface SpeechGrammarList {\\n    readonly length: number;\\n    addFromString(string: string, weight?: number): void;\\n    addFromURI(src: string, weight?: number): void;\\n    item(index: number): SpeechGrammar;\\n    [index: number]: SpeechGrammar;\\n}\\n\\ndeclare var SpeechGrammarList: {\\n    prototype: SpeechGrammarList;\\n    new(): SpeechGrammarList;\\n};\\n\\ninterface SpeechRecognitionEventMap {\\n    \"audioend\": Event;\\n    \"audiostart\": Event;\\n    \"end\": Event;\\n    \"error\": SpeechRecognitionError;\\n    \"nomatch\": SpeechRecognitionEvent;\\n    \"result\": SpeechRecognitionEvent;\\n    \"soundend\": Event;\\n    \"soundstart\": Event;\\n    \"speechend\": Event;\\n    \"speechstart\": Event;\\n    \"start\": Event;\\n}\\n\\ninterface SpeechRecognition extends EventTarget {\\n    continuous: boolean;\\n    grammars: SpeechGrammarList;\\n    interimResults: boolean;\\n    lang: string;\\n    maxAlternatives: number;\\n    onaudioend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onaudiostart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onerror: ((this: SpeechRecognition, ev: SpeechRecognitionError) => any) | null;\\n    onnomatch: ((this: SpeechRecognition, ev: SpeechRecognitionEvent) => any) | null;\\n    onresult: ((this: SpeechRecognition, ev: SpeechRecognitionEvent) => any) | null;\\n    onsoundend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onsoundstart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onspeechend: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onspeechstart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    onstart: ((this: SpeechRecognition, ev: Event) => any) | null;\\n    serviceURI: string;\\n    abort(): void;\\n    start(): void;\\n    stop(): void;\\n    addEventListener<K extends keyof SpeechRecognitionEventMap>(type: K, listener: (this: SpeechRecognition, ev: SpeechRecognitionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SpeechRecognitionEventMap>(type: K, listener: (this: SpeechRecognition, ev: SpeechRecognitionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SpeechRecognition: {\\n    prototype: SpeechRecognition;\\n    new(): SpeechRecognition;\\n};\\n\\ninterface SpeechRecognitionAlternative {\\n    readonly confidence: number;\\n    readonly transcript: string;\\n}\\n\\ndeclare var SpeechRecognitionAlternative: {\\n    prototype: SpeechRecognitionAlternative;\\n    new(): SpeechRecognitionAlternative;\\n};\\n\\ninterface SpeechRecognitionError extends Event {\\n    readonly error: SpeechRecognitionErrorCode;\\n    readonly message: string;\\n}\\n\\ndeclare var SpeechRecognitionError: {\\n    prototype: SpeechRecognitionError;\\n    new(): SpeechRecognitionError;\\n};\\n\\ninterface SpeechRecognitionEvent extends Event {\\n    readonly emma: Document | null;\\n    readonly interpretation: any;\\n    readonly resultIndex: number;\\n    readonly results: SpeechRecognitionResultList;\\n}\\n\\ndeclare var SpeechRecognitionEvent: {\\n    prototype: SpeechRecognitionEvent;\\n    new(): SpeechRecognitionEvent;\\n};\\n\\ninterface SpeechRecognitionResult {\\n    readonly isFinal: boolean;\\n    readonly length: number;\\n    item(index: number): SpeechRecognitionAlternative;\\n    [index: number]: SpeechRecognitionAlternative;\\n}\\n\\ndeclare var SpeechRecognitionResult: {\\n    prototype: SpeechRecognitionResult;\\n    new(): SpeechRecognitionResult;\\n};\\n\\ninterface SpeechRecognitionResultList {\\n    readonly length: number;\\n    item(index: number): SpeechRecognitionResult;\\n    [index: number]: SpeechRecognitionResult;\\n}\\n\\ndeclare var SpeechRecognitionResultList: {\\n    prototype: SpeechRecognitionResultList;\\n    new(): SpeechRecognitionResultList;\\n};\\n\\ninterface SpeechSynthesisEventMap {\\n    \"voiceschanged\": Event;\\n}\\n\\ninterface SpeechSynthesis extends EventTarget {\\n    onvoiceschanged: ((this: SpeechSynthesis, ev: Event) => any) | null;\\n    readonly paused: boolean;\\n    readonly pending: boolean;\\n    readonly speaking: boolean;\\n    cancel(): void;\\n    getVoices(): SpeechSynthesisVoice[];\\n    pause(): void;\\n    resume(): void;\\n    speak(utterance: SpeechSynthesisUtterance): void;\\n    addEventListener<K extends keyof SpeechSynthesisEventMap>(type: K, listener: (this: SpeechSynthesis, ev: SpeechSynthesisEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SpeechSynthesisEventMap>(type: K, listener: (this: SpeechSynthesis, ev: SpeechSynthesisEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SpeechSynthesis: {\\n    prototype: SpeechSynthesis;\\n    new(): SpeechSynthesis;\\n};\\n\\ninterface SpeechSynthesisErrorEvent extends SpeechSynthesisEvent {\\n    readonly error: SpeechSynthesisErrorCode;\\n}\\n\\ndeclare var SpeechSynthesisErrorEvent: {\\n    prototype: SpeechSynthesisErrorEvent;\\n    new(): SpeechSynthesisErrorEvent;\\n};\\n\\ninterface SpeechSynthesisEvent extends Event {\\n    readonly charIndex: number;\\n    readonly elapsedTime: number;\\n    readonly name: string;\\n    readonly utterance: SpeechSynthesisUtterance;\\n}\\n\\ndeclare var SpeechSynthesisEvent: {\\n    prototype: SpeechSynthesisEvent;\\n    new(): SpeechSynthesisEvent;\\n};\\n\\ninterface SpeechSynthesisUtteranceEventMap {\\n    \"boundary\": SpeechSynthesisEvent;\\n    \"end\": SpeechSynthesisEvent;\\n    \"error\": SpeechSynthesisErrorEvent;\\n    \"mark\": SpeechSynthesisEvent;\\n    \"pause\": SpeechSynthesisEvent;\\n    \"resume\": SpeechSynthesisEvent;\\n    \"start\": SpeechSynthesisEvent;\\n}\\n\\ninterface SpeechSynthesisUtterance extends EventTarget {\\n    lang: string;\\n    onboundary: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onend: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onerror: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisErrorEvent) => any) | null;\\n    onmark: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onpause: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onresume: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    onstart: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\\n    pitch: number;\\n    rate: number;\\n    text: string;\\n    voice: SpeechSynthesisVoice;\\n    volume: number;\\n    addEventListener<K extends keyof SpeechSynthesisUtteranceEventMap>(type: K, listener: (this: SpeechSynthesisUtterance, ev: SpeechSynthesisUtteranceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof SpeechSynthesisUtteranceEventMap>(type: K, listener: (this: SpeechSynthesisUtterance, ev: SpeechSynthesisUtteranceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var SpeechSynthesisUtterance: {\\n    prototype: SpeechSynthesisUtterance;\\n    new(): SpeechSynthesisUtterance;\\n    new(text: string): SpeechSynthesisUtterance;\\n};\\n\\ninterface SpeechSynthesisVoice {\\n    readonly default: boolean;\\n    readonly lang: string;\\n    readonly localService: boolean;\\n    readonly name: string;\\n    readonly voiceURI: string;\\n}\\n\\ndeclare var SpeechSynthesisVoice: {\\n    prototype: SpeechSynthesisVoice;\\n    new(): SpeechSynthesisVoice;\\n};\\n\\ninterface StaticRange extends AbstractRange {\\n}\\n\\ndeclare var StaticRange: {\\n    prototype: StaticRange;\\n    new(): StaticRange;\\n};\\n\\ninterface StereoPannerNode extends AudioNode {\\n    readonly pan: AudioParam;\\n}\\n\\ndeclare var StereoPannerNode: {\\n    prototype: StereoPannerNode;\\n    new(context: BaseAudioContext, options?: StereoPannerOptions): StereoPannerNode;\\n};\\n\\ninterface Storage {\\n    /**\\n     * Returns the number of key/value pairs currently present in the list associated with the\\n     * object.\\n     */\\n    readonly length: number;\\n    /**\\n     * Empties the list associated with the object of all key/value pairs, if there are any.\\n     */\\n    clear(): void;\\n    /**\\n     * value = storage[key]\\n     */\\n    getItem(key: string): string | null;\\n    /**\\n     * Returns the name of the nth key in the list, or null if n is greater\\n     * than or equal to the number of key/value pairs in the object.\\n     */\\n    key(index: number): string | null;\\n    /**\\n     * delete storage[key]\\n     */\\n    removeItem(key: string): void;\\n    /**\\n     * storage[key] = value\\n     */\\n    setItem(key: string, value: string): void;\\n    [name: string]: any;\\n}\\n\\ndeclare var Storage: {\\n    prototype: Storage;\\n    new(): Storage;\\n};\\n\\ninterface StorageEvent extends Event {\\n    /**\\n     * Returns the key of the storage item being changed.\\n     */\\n    readonly key: string | null;\\n    /**\\n     * Returns the new value of the key of the storage item whose value is being changed.\\n     */\\n    readonly newValue: string | null;\\n    /**\\n     * Returns the old value of the key of the storage item whose value is being changed.\\n     */\\n    readonly oldValue: string | null;\\n    /**\\n     * Returns the Storage object that was affected.\\n     */\\n    readonly storageArea: Storage | null;\\n    /**\\n     * Returns the URL of the document whose storage item changed.\\n     */\\n    readonly url: string;\\n}\\n\\ndeclare var StorageEvent: {\\n    prototype: StorageEvent;\\n    new(type: string, eventInitDict?: StorageEventInit): StorageEvent;\\n};\\n\\ninterface StorageManager {\\n    estimate(): Promise<StorageEstimate>;\\n    persist(): Promise<boolean>;\\n    persisted(): Promise<boolean>;\\n}\\n\\ndeclare var StorageManager: {\\n    prototype: StorageManager;\\n    new(): StorageManager;\\n};\\n\\ninterface StyleMedia {\\n    readonly type: string;\\n    matchMedium(mediaquery: string): boolean;\\n}\\n\\ndeclare var StyleMedia: {\\n    prototype: StyleMedia;\\n    new(): StyleMedia;\\n};\\n\\ninterface StyleSheet {\\n    disabled: boolean;\\n    readonly href: string | null;\\n    readonly media: MediaList;\\n    readonly ownerNode: Node;\\n    readonly parentStyleSheet: StyleSheet | null;\\n    readonly title: string | null;\\n    readonly type: string;\\n}\\n\\ndeclare var StyleSheet: {\\n    prototype: StyleSheet;\\n    new(): StyleSheet;\\n};\\n\\ninterface StyleSheetList {\\n    readonly length: number;\\n    item(index: number): StyleSheet | null;\\n    [index: number]: StyleSheet;\\n}\\n\\ndeclare var StyleSheetList: {\\n    prototype: StyleSheetList;\\n    new(): StyleSheetList;\\n};\\n\\ninterface SubtleCrypto {\\n    decrypt(algorithm: string | RsaOaepParams | AesCtrParams | AesCbcParams | AesCmacParams | AesGcmParams | AesCfbParams, key: CryptoKey, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    deriveBits(algorithm: string | EcdhKeyDeriveParams | DhKeyDeriveParams | ConcatParams | HkdfCtrParams | Pbkdf2Params, baseKey: CryptoKey, length: number): PromiseLike<ArrayBuffer>;\\n    deriveKey(algorithm: string | EcdhKeyDeriveParams | DhKeyDeriveParams | ConcatParams | HkdfCtrParams | Pbkdf2Params, baseKey: CryptoKey, derivedKeyType: string | AesDerivedKeyParams | HmacImportParams | ConcatParams | HkdfCtrParams | Pbkdf2Params, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    digest(algorithm: string | Algorithm, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    encrypt(algorithm: string | RsaOaepParams | AesCtrParams | AesCbcParams | AesCmacParams | AesGcmParams | AesCfbParams, key: CryptoKey, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    exportKey(format: \"jwk\", key: CryptoKey): PromiseLike<JsonWebKey>;\\n    exportKey(format: \"raw\" | \"pkcs8\" | \"spki\", key: CryptoKey): PromiseLike<ArrayBuffer>;\\n    exportKey(format: string, key: CryptoKey): PromiseLike<JsonWebKey | ArrayBuffer>;\\n    generateKey(algorithm: string, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKeyPair | CryptoKey>;\\n    generateKey(algorithm: RsaHashedKeyGenParams | EcKeyGenParams | DhKeyGenParams, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKeyPair>;\\n    generateKey(algorithm: AesKeyGenParams | HmacKeyGenParams | Pbkdf2Params, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    importKey(format: \"jwk\", keyData: JsonWebKey, algorithm: string | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | DhImportKeyParams | AesKeyAlgorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    importKey(format: \"raw\" | \"pkcs8\" | \"spki\", keyData: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, algorithm: string | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | DhImportKeyParams | AesKeyAlgorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    importKey(format: string, keyData: JsonWebKey | Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, algorithm: string | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | DhImportKeyParams | AesKeyAlgorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    sign(algorithm: string | RsaPssParams | EcdsaParams | AesCmacParams, key: CryptoKey, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<ArrayBuffer>;\\n    unwrapKey(format: string, wrappedKey: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, unwrappingKey: CryptoKey, unwrapAlgorithm: string | Algorithm, unwrappedKeyAlgorithm: string | Algorithm, extractable: boolean, keyUsages: string[]): PromiseLike<CryptoKey>;\\n    verify(algorithm: string | RsaPssParams | EcdsaParams | AesCmacParams, key: CryptoKey, signature: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer, data: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer): PromiseLike<boolean>;\\n    wrapKey(format: string, key: CryptoKey, wrappingKey: CryptoKey, wrapAlgorithm: string | Algorithm): PromiseLike<ArrayBuffer>;\\n}\\n\\ndeclare var SubtleCrypto: {\\n    prototype: SubtleCrypto;\\n    new(): SubtleCrypto;\\n};\\n\\ninterface SyncManager {\\n    getTags(): Promise<string[]>;\\n    register(tag: string): Promise<void>;\\n}\\n\\ndeclare var SyncManager: {\\n    prototype: SyncManager;\\n    new(): SyncManager;\\n};\\n\\ninterface Text extends CharacterData, Slotable {\\n    readonly assignedSlot: HTMLSlotElement | null;\\n    /**\\n     * Returns the combined data of all direct Text node siblings.\\n     */\\n    readonly wholeText: string;\\n    /**\\n     * Splits data at the given offset and returns the remainder as Text node.\\n     */\\n    splitText(offset: number): Text;\\n}\\n\\ndeclare var Text: {\\n    prototype: Text;\\n    new(data?: string): Text;\\n};\\n\\ninterface TextDecoder {\\n    /**\\n     * Returns encoding\\'s name, lowercased.\\n     */\\n    readonly encoding: string;\\n    /**\\n     * Returns true if error mode is \"fatal\", and false\\n     * otherwise.\\n     */\\n    readonly fatal: boolean;\\n    /**\\n     * Returns true if ignore BOM flag is set, and false otherwise.\\n     */\\n    readonly ignoreBOM: boolean;\\n    /**\\n     * Returns the result of running encoding\\'s decoder. The\\n     * method can be invoked zero or more times with options\\'s stream set to\\n     * true, and then once without options\\'s stream (or set to false), to process\\n     * a fragmented stream. If the invocation without options\\'s stream (or set to\\n     * false) has no input, it\\'s clearest to omit both arguments.\\n     * var string = \"\", decoder = new TextDecoder(encoding), buffer;\\n     * while(buffer = next_chunk()) {\\n     * string += decoder.decode(buffer, {stream:true});\\n     * }\\n     * string += decoder.decode(); // end-of-stream\\n     * If the error mode is \"fatal\" and encoding\\'s decoder returns error, throws a TypeError.\\n     */\\n    decode(input?: BufferSource, options?: TextDecodeOptions): string;\\n}\\n\\ndeclare var TextDecoder: {\\n    prototype: TextDecoder;\\n    new(label?: string, options?: TextDecoderOptions): TextDecoder;\\n};\\n\\ninterface TextEncoder {\\n    /**\\n     * Returns \"utf-8\".\\n     */\\n    readonly encoding: string;\\n    /**\\n     * Returns the result of running UTF-8\\'s encoder.\\n     */\\n    encode(input?: string): Uint8Array;\\n}\\n\\ndeclare var TextEncoder: {\\n    prototype: TextEncoder;\\n    new(): TextEncoder;\\n};\\n\\ninterface TextEvent extends UIEvent {\\n    readonly data: string;\\n    initTextEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, dataArg: string, inputMethod: number, locale: string): void;\\n    readonly DOM_INPUT_METHOD_DROP: number;\\n    readonly DOM_INPUT_METHOD_HANDWRITING: number;\\n    readonly DOM_INPUT_METHOD_IME: number;\\n    readonly DOM_INPUT_METHOD_KEYBOARD: number;\\n    readonly DOM_INPUT_METHOD_MULTIMODAL: number;\\n    readonly DOM_INPUT_METHOD_OPTION: number;\\n    readonly DOM_INPUT_METHOD_PASTE: number;\\n    readonly DOM_INPUT_METHOD_SCRIPT: number;\\n    readonly DOM_INPUT_METHOD_UNKNOWN: number;\\n    readonly DOM_INPUT_METHOD_VOICE: number;\\n}\\n\\ndeclare var TextEvent: {\\n    prototype: TextEvent;\\n    new(): TextEvent;\\n    readonly DOM_INPUT_METHOD_DROP: number;\\n    readonly DOM_INPUT_METHOD_HANDWRITING: number;\\n    readonly DOM_INPUT_METHOD_IME: number;\\n    readonly DOM_INPUT_METHOD_KEYBOARD: number;\\n    readonly DOM_INPUT_METHOD_MULTIMODAL: number;\\n    readonly DOM_INPUT_METHOD_OPTION: number;\\n    readonly DOM_INPUT_METHOD_PASTE: number;\\n    readonly DOM_INPUT_METHOD_SCRIPT: number;\\n    readonly DOM_INPUT_METHOD_UNKNOWN: number;\\n    readonly DOM_INPUT_METHOD_VOICE: number;\\n};\\n\\ninterface TextMetrics {\\n    readonly actualBoundingBoxAscent: number;\\n    readonly actualBoundingBoxDescent: number;\\n    readonly actualBoundingBoxLeft: number;\\n    readonly actualBoundingBoxRight: number;\\n    readonly alphabeticBaseline: number;\\n    readonly emHeightAscent: number;\\n    readonly emHeightDescent: number;\\n    readonly fontBoundingBoxAscent: number;\\n    readonly fontBoundingBoxDescent: number;\\n    readonly hangingBaseline: number;\\n    /**\\n     * Returns the measurement described below.\\n     */\\n    readonly ideographicBaseline: number;\\n    readonly width: number;\\n}\\n\\ndeclare var TextMetrics: {\\n    prototype: TextMetrics;\\n    new(): TextMetrics;\\n};\\n\\ninterface TextTrackEventMap {\\n    \"cuechange\": Event;\\n    \"error\": Event;\\n    \"load\": Event;\\n}\\n\\ninterface TextTrack extends EventTarget {\\n    readonly activeCues: TextTrackCueList;\\n    readonly cues: TextTrackCueList;\\n    readonly inBandMetadataTrackDispatchType: string;\\n    readonly kind: string;\\n    readonly label: string;\\n    readonly language: string;\\n    mode: TextTrackMode | number;\\n    oncuechange: ((this: TextTrack, ev: Event) => any) | null;\\n    onerror: ((this: TextTrack, ev: Event) => any) | null;\\n    onload: ((this: TextTrack, ev: Event) => any) | null;\\n    readonly readyState: number;\\n    addCue(cue: TextTrackCue): void;\\n    removeCue(cue: TextTrackCue): void;\\n    readonly DISABLED: number;\\n    readonly ERROR: number;\\n    readonly HIDDEN: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n    readonly SHOWING: number;\\n    addEventListener<K extends keyof TextTrackEventMap>(type: K, listener: (this: TextTrack, ev: TextTrackEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackEventMap>(type: K, listener: (this: TextTrack, ev: TextTrackEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var TextTrack: {\\n    prototype: TextTrack;\\n    new(): TextTrack;\\n    readonly DISABLED: number;\\n    readonly ERROR: number;\\n    readonly HIDDEN: number;\\n    readonly LOADED: number;\\n    readonly LOADING: number;\\n    readonly NONE: number;\\n    readonly SHOWING: number;\\n};\\n\\ninterface TextTrackCueEventMap {\\n    \"enter\": Event;\\n    \"exit\": Event;\\n}\\n\\ninterface TextTrackCue extends EventTarget {\\n    endTime: number;\\n    id: string;\\n    onenter: ((this: TextTrackCue, ev: Event) => any) | null;\\n    onexit: ((this: TextTrackCue, ev: Event) => any) | null;\\n    pauseOnExit: boolean;\\n    startTime: number;\\n    text: string;\\n    readonly track: TextTrack;\\n    getCueAsHTML(): DocumentFragment;\\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: TextTrackCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: TextTrackCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var TextTrackCue: {\\n    prototype: TextTrackCue;\\n    new(startTime: number, endTime: number, text: string): TextTrackCue;\\n};\\n\\ninterface TextTrackCueList {\\n    readonly length: number;\\n    getCueById(id: string): TextTrackCue;\\n    item(index: number): TextTrackCue;\\n    [index: number]: TextTrackCue;\\n}\\n\\ndeclare var TextTrackCueList: {\\n    prototype: TextTrackCueList;\\n    new(): TextTrackCueList;\\n};\\n\\ninterface TextTrackListEventMap {\\n    \"addtrack\": TrackEvent;\\n}\\n\\ninterface TextTrackList extends EventTarget {\\n    readonly length: number;\\n    onaddtrack: ((this: TextTrackList, ev: TrackEvent) => any) | null;\\n    item(index: number): TextTrack;\\n    addEventListener<K extends keyof TextTrackListEventMap>(type: K, listener: (this: TextTrackList, ev: TextTrackListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackListEventMap>(type: K, listener: (this: TextTrackList, ev: TextTrackListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: TextTrack;\\n}\\n\\ndeclare var TextTrackList: {\\n    prototype: TextTrackList;\\n    new(): TextTrackList;\\n};\\n\\ninterface TimeRanges {\\n    readonly length: number;\\n    end(index: number): number;\\n    start(index: number): number;\\n}\\n\\ndeclare var TimeRanges: {\\n    prototype: TimeRanges;\\n    new(): TimeRanges;\\n};\\n\\ninterface Touch {\\n    readonly altitudeAngle: number;\\n    readonly azimuthAngle: number;\\n    readonly clientX: number;\\n    readonly clientY: number;\\n    readonly force: number;\\n    readonly identifier: number;\\n    readonly pageX: number;\\n    readonly pageY: number;\\n    readonly radiusX: number;\\n    readonly radiusY: number;\\n    readonly rotationAngle: number;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly target: EventTarget;\\n    readonly touchType: TouchType;\\n}\\n\\ndeclare var Touch: {\\n    prototype: Touch;\\n    new(touchInitDict: TouchInit): Touch;\\n};\\n\\ninterface TouchEvent extends UIEvent {\\n    readonly altKey: boolean;\\n    readonly changedTouches: TouchList;\\n    readonly ctrlKey: boolean;\\n    readonly metaKey: boolean;\\n    readonly shiftKey: boolean;\\n    readonly targetTouches: TouchList;\\n    readonly touches: TouchList;\\n}\\n\\ndeclare var TouchEvent: {\\n    prototype: TouchEvent;\\n    new(type: string, eventInitDict?: TouchEventInit): TouchEvent;\\n};\\n\\ninterface TouchList {\\n    readonly length: number;\\n    item(index: number): Touch | null;\\n    [index: number]: Touch;\\n}\\n\\ndeclare var TouchList: {\\n    prototype: TouchList;\\n    new(): TouchList;\\n};\\n\\ninterface TrackEvent extends Event {\\n    readonly track: VideoTrack | AudioTrack | TextTrack | null;\\n}\\n\\ndeclare var TrackEvent: {\\n    prototype: TrackEvent;\\n    new(typeArg: string, eventInitDict?: TrackEventInit): TrackEvent;\\n};\\n\\ninterface TransitionEvent extends Event {\\n    readonly elapsedTime: number;\\n    readonly propertyName: string;\\n    readonly pseudoElement: string;\\n}\\n\\ndeclare var TransitionEvent: {\\n    prototype: TransitionEvent;\\n    new(type: string, transitionEventInitDict?: TransitionEventInit): TransitionEvent;\\n};\\n\\ninterface TreeWalker {\\n    currentNode: Node;\\n    readonly filter: NodeFilter | null;\\n    readonly root: Node;\\n    readonly whatToShow: number;\\n    firstChild(): Node | null;\\n    lastChild(): Node | null;\\n    nextNode(): Node | null;\\n    nextSibling(): Node | null;\\n    parentNode(): Node | null;\\n    previousNode(): Node | null;\\n    previousSibling(): Node | null;\\n}\\n\\ndeclare var TreeWalker: {\\n    prototype: TreeWalker;\\n    new(): TreeWalker;\\n};\\n\\ninterface UIEvent extends Event {\\n    readonly detail: number;\\n    readonly view: Window;\\n    initUIEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number): void;\\n}\\n\\ndeclare var UIEvent: {\\n    prototype: UIEvent;\\n    new(typeArg: string, eventInitDict?: UIEventInit): UIEvent;\\n};\\n\\ninterface URL {\\n    hash: string;\\n    host: string;\\n    hostname: string;\\n    href: string;\\n    readonly origin: string;\\n    password: string;\\n    pathname: string;\\n    port: string;\\n    protocol: string;\\n    search: string;\\n    readonly searchParams: URLSearchParams;\\n    username: string;\\n    toJSON(): string;\\n}\\n\\ndeclare var URL: {\\n    prototype: URL;\\n    new(url: string, base?: string | URL): URL;\\n    createObjectURL(object: any): string;\\n    revokeObjectURL(url: string): void;\\n};\\n\\ntype webkitURL = URL;\\ndeclare var webkitURL: typeof URL;\\n\\ninterface URLSearchParams {\\n    /**\\n     * Appends a specified key/value pair as a new search parameter.\\n     */\\n    append(name: string, value: string): void;\\n    /**\\n     * Deletes the given search parameter, and its associated value, from the list of all search parameters.\\n     */\\n    delete(name: string): void;\\n    /**\\n     * Returns the first value associated to the given search parameter.\\n     */\\n    get(name: string): string | null;\\n    /**\\n     * Returns all the values association with a given search parameter.\\n     */\\n    getAll(name: string): string[];\\n    /**\\n     * Returns a Boolean indicating if such a search parameter exists.\\n     */\\n    has(name: string): boolean;\\n    /**\\n     * Sets the value associated to a given search parameter to the given value. If there were several values, delete the others.\\n     */\\n    set(name: string, value: string): void;\\n    sort(): void;\\n    forEach(callbackfn: (value: string, key: string, parent: URLSearchParams) => void, thisArg?: any): void;\\n}\\n\\ndeclare var URLSearchParams: {\\n    prototype: URLSearchParams;\\n    new(init?: string[][] | Record<string, string> | string | URLSearchParams): URLSearchParams;\\n};\\n\\ninterface VRDisplay extends EventTarget {\\n    readonly capabilities: VRDisplayCapabilities;\\n    depthFar: number;\\n    depthNear: number;\\n    readonly displayId: number;\\n    readonly displayName: string;\\n    readonly isConnected: boolean;\\n    readonly isPresenting: boolean;\\n    readonly stageParameters: VRStageParameters | null;\\n    cancelAnimationFrame(handle: number): void;\\n    exitPresent(): Promise<void>;\\n    getEyeParameters(whichEye: string): VREyeParameters;\\n    getFrameData(frameData: VRFrameData): boolean;\\n    getLayers(): VRLayer[];\\n    /** @deprecated */\\n    getPose(): VRPose;\\n    requestAnimationFrame(callback: FrameRequestCallback): number;\\n    requestPresent(layers: VRLayer[]): Promise<void>;\\n    resetPose(): void;\\n    submitFrame(pose?: VRPose): void;\\n}\\n\\ndeclare var VRDisplay: {\\n    prototype: VRDisplay;\\n    new(): VRDisplay;\\n};\\n\\ninterface VRDisplayCapabilities {\\n    readonly canPresent: boolean;\\n    readonly hasExternalDisplay: boolean;\\n    readonly hasOrientation: boolean;\\n    readonly hasPosition: boolean;\\n    readonly maxLayers: number;\\n}\\n\\ndeclare var VRDisplayCapabilities: {\\n    prototype: VRDisplayCapabilities;\\n    new(): VRDisplayCapabilities;\\n};\\n\\ninterface VRDisplayEvent extends Event {\\n    readonly display: VRDisplay;\\n    readonly reason: VRDisplayEventReason | null;\\n}\\n\\ndeclare var VRDisplayEvent: {\\n    prototype: VRDisplayEvent;\\n    new(type: string, eventInitDict: VRDisplayEventInit): VRDisplayEvent;\\n};\\n\\ninterface VREyeParameters {\\n    /** @deprecated */\\n    readonly fieldOfView: VRFieldOfView;\\n    readonly offset: Float32Array;\\n    readonly renderHeight: number;\\n    readonly renderWidth: number;\\n}\\n\\ndeclare var VREyeParameters: {\\n    prototype: VREyeParameters;\\n    new(): VREyeParameters;\\n};\\n\\ninterface VRFieldOfView {\\n    readonly downDegrees: number;\\n    readonly leftDegrees: number;\\n    readonly rightDegrees: number;\\n    readonly upDegrees: number;\\n}\\n\\ndeclare var VRFieldOfView: {\\n    prototype: VRFieldOfView;\\n    new(): VRFieldOfView;\\n};\\n\\ninterface VRFrameData {\\n    readonly leftProjectionMatrix: Float32Array;\\n    readonly leftViewMatrix: Float32Array;\\n    readonly pose: VRPose;\\n    readonly rightProjectionMatrix: Float32Array;\\n    readonly rightViewMatrix: Float32Array;\\n    readonly timestamp: number;\\n}\\n\\ndeclare var VRFrameData: {\\n    prototype: VRFrameData;\\n    new(): VRFrameData;\\n};\\n\\ninterface VRPose {\\n    readonly angularAcceleration: Float32Array | null;\\n    readonly angularVelocity: Float32Array | null;\\n    readonly linearAcceleration: Float32Array | null;\\n    readonly linearVelocity: Float32Array | null;\\n    readonly orientation: Float32Array | null;\\n    readonly position: Float32Array | null;\\n    readonly timestamp: number;\\n}\\n\\ndeclare var VRPose: {\\n    prototype: VRPose;\\n    new(): VRPose;\\n};\\n\\ninterface VTTCue extends TextTrackCue {\\n    align: AlignSetting;\\n    line: LineAndPositionSetting;\\n    lineAlign: LineAlignSetting;\\n    position: LineAndPositionSetting;\\n    positionAlign: PositionAlignSetting;\\n    region: VTTRegion | null;\\n    size: number;\\n    snapToLines: boolean;\\n    text: string;\\n    vertical: DirectionSetting;\\n    getCueAsHTML(): DocumentFragment;\\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: VTTCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: VTTCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var VTTCue: {\\n    prototype: VTTCue;\\n    new(startTime: number, endTime: number, text: string): VTTCue;\\n};\\n\\ninterface VTTRegion {\\n    id: string;\\n    lines: number;\\n    regionAnchorX: number;\\n    regionAnchorY: number;\\n    scroll: ScrollSetting;\\n    viewportAnchorX: number;\\n    viewportAnchorY: number;\\n    width: number;\\n}\\n\\ndeclare var VTTRegion: {\\n    prototype: VTTRegion;\\n    new(): VTTRegion;\\n};\\n\\ninterface ValidityState {\\n    readonly badInput: boolean;\\n    readonly customError: boolean;\\n    readonly patternMismatch: boolean;\\n    readonly rangeOverflow: boolean;\\n    readonly rangeUnderflow: boolean;\\n    readonly stepMismatch: boolean;\\n    readonly tooLong: boolean;\\n    readonly tooShort: boolean;\\n    readonly typeMismatch: boolean;\\n    readonly valid: boolean;\\n    readonly valueMissing: boolean;\\n}\\n\\ndeclare var ValidityState: {\\n    prototype: ValidityState;\\n    new(): ValidityState;\\n};\\n\\ninterface VideoPlaybackQuality {\\n    readonly corruptedVideoFrames: number;\\n    readonly creationTime: number;\\n    readonly droppedVideoFrames: number;\\n    readonly totalFrameDelay: number;\\n    readonly totalVideoFrames: number;\\n}\\n\\ndeclare var VideoPlaybackQuality: {\\n    prototype: VideoPlaybackQuality;\\n    new(): VideoPlaybackQuality;\\n};\\n\\ninterface VideoTrack {\\n    readonly id: string;\\n    kind: string;\\n    readonly label: string;\\n    language: string;\\n    selected: boolean;\\n    readonly sourceBuffer: SourceBuffer;\\n}\\n\\ndeclare var VideoTrack: {\\n    prototype: VideoTrack;\\n    new(): VideoTrack;\\n};\\n\\ninterface VideoTrackListEventMap {\\n    \"addtrack\": TrackEvent;\\n    \"change\": Event;\\n    \"removetrack\": TrackEvent;\\n}\\n\\ninterface VideoTrackList extends EventTarget {\\n    readonly length: number;\\n    onaddtrack: ((this: VideoTrackList, ev: TrackEvent) => any) | null;\\n    onchange: ((this: VideoTrackList, ev: Event) => any) | null;\\n    onremovetrack: ((this: VideoTrackList, ev: TrackEvent) => any) | null;\\n    readonly selectedIndex: number;\\n    getTrackById(id: string): VideoTrack | null;\\n    item(index: number): VideoTrack;\\n    addEventListener<K extends keyof VideoTrackListEventMap>(type: K, listener: (this: VideoTrackList, ev: VideoTrackListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof VideoTrackListEventMap>(type: K, listener: (this: VideoTrackList, ev: VideoTrackListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n    [index: number]: VideoTrack;\\n}\\n\\ndeclare var VideoTrackList: {\\n    prototype: VideoTrackList;\\n    new(): VideoTrackList;\\n};\\n\\ninterface WEBGL_color_buffer_float {\\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT: number;\\n    readonly RGB32F_EXT: number;\\n    readonly RGBA32F_EXT: number;\\n    readonly UNSIGNED_NORMALIZED_EXT: number;\\n}\\n\\ninterface WEBGL_compressed_texture_astc {\\n    readonly COMPRESSED_RGBA_ASTC_10x10_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_10x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_10x6_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_10x8_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_12x10_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_12x12_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_4x4_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_5x4_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_5x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_6x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_6x6_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_8x5_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_8x6_KHR: number;\\n    readonly COMPRESSED_RGBA_ASTC_8x8_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x10_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x6_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x8_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x4_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x6_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x5_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x6_KHR: number;\\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x8_KHR: number;\\n    getSupportedProfiles(): string[];\\n}\\n\\ninterface WEBGL_compressed_texture_s3tc {\\n    readonly COMPRESSED_RGBA_S3TC_DXT1_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT3_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT5_EXT: number;\\n    readonly COMPRESSED_RGB_S3TC_DXT1_EXT: number;\\n}\\n\\ndeclare var WEBGL_compressed_texture_s3tc: {\\n    prototype: WEBGL_compressed_texture_s3tc;\\n    new(): WEBGL_compressed_texture_s3tc;\\n    readonly COMPRESSED_RGBA_S3TC_DXT1_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT3_EXT: number;\\n    readonly COMPRESSED_RGBA_S3TC_DXT5_EXT: number;\\n    readonly COMPRESSED_RGB_S3TC_DXT1_EXT: number;\\n};\\n\\ninterface WEBGL_compressed_texture_s3tc_srgb {\\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT: number;\\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT: number;\\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT: number;\\n    readonly COMPRESSED_SRGB_S3TC_DXT1_EXT: number;\\n}\\n\\ninterface WEBGL_debug_renderer_info {\\n    readonly UNMASKED_RENDERER_WEBGL: number;\\n    readonly UNMASKED_VENDOR_WEBGL: number;\\n}\\n\\ndeclare var WEBGL_debug_renderer_info: {\\n    prototype: WEBGL_debug_renderer_info;\\n    new(): WEBGL_debug_renderer_info;\\n    readonly UNMASKED_RENDERER_WEBGL: number;\\n    readonly UNMASKED_VENDOR_WEBGL: number;\\n};\\n\\ninterface WEBGL_debug_shaders {\\n    getTranslatedShaderSource(shader: WebGLShader): string;\\n}\\n\\ninterface WEBGL_depth_texture {\\n    readonly UNSIGNED_INT_24_8_WEBGL: number;\\n}\\n\\ndeclare var WEBGL_depth_texture: {\\n    prototype: WEBGL_depth_texture;\\n    new(): WEBGL_depth_texture;\\n    readonly UNSIGNED_INT_24_8_WEBGL: number;\\n};\\n\\ninterface WEBGL_draw_buffers {\\n    readonly COLOR_ATTACHMENT0_WEBGL: number;\\n    readonly COLOR_ATTACHMENT10_WEBGL: number;\\n    readonly COLOR_ATTACHMENT11_WEBGL: number;\\n    readonly COLOR_ATTACHMENT12_WEBGL: number;\\n    readonly COLOR_ATTACHMENT13_WEBGL: number;\\n    readonly COLOR_ATTACHMENT14_WEBGL: number;\\n    readonly COLOR_ATTACHMENT15_WEBGL: number;\\n    readonly COLOR_ATTACHMENT1_WEBGL: number;\\n    readonly COLOR_ATTACHMENT2_WEBGL: number;\\n    readonly COLOR_ATTACHMENT3_WEBGL: number;\\n    readonly COLOR_ATTACHMENT4_WEBGL: number;\\n    readonly COLOR_ATTACHMENT5_WEBGL: number;\\n    readonly COLOR_ATTACHMENT6_WEBGL: number;\\n    readonly COLOR_ATTACHMENT7_WEBGL: number;\\n    readonly COLOR_ATTACHMENT8_WEBGL: number;\\n    readonly COLOR_ATTACHMENT9_WEBGL: number;\\n    readonly DRAW_BUFFER0_WEBGL: number;\\n    readonly DRAW_BUFFER10_WEBGL: number;\\n    readonly DRAW_BUFFER11_WEBGL: number;\\n    readonly DRAW_BUFFER12_WEBGL: number;\\n    readonly DRAW_BUFFER13_WEBGL: number;\\n    readonly DRAW_BUFFER14_WEBGL: number;\\n    readonly DRAW_BUFFER15_WEBGL: number;\\n    readonly DRAW_BUFFER1_WEBGL: number;\\n    readonly DRAW_BUFFER2_WEBGL: number;\\n    readonly DRAW_BUFFER3_WEBGL: number;\\n    readonly DRAW_BUFFER4_WEBGL: number;\\n    readonly DRAW_BUFFER5_WEBGL: number;\\n    readonly DRAW_BUFFER6_WEBGL: number;\\n    readonly DRAW_BUFFER7_WEBGL: number;\\n    readonly DRAW_BUFFER8_WEBGL: number;\\n    readonly DRAW_BUFFER9_WEBGL: number;\\n    readonly MAX_COLOR_ATTACHMENTS_WEBGL: number;\\n    readonly MAX_DRAW_BUFFERS_WEBGL: number;\\n    drawBuffersWEBGL(buffers: number[]): void;\\n}\\n\\ninterface WEBGL_lose_context {\\n    loseContext(): void;\\n    restoreContext(): void;\\n}\\n\\ninterface WaveShaperNode extends AudioNode {\\n    curve: Float32Array | null;\\n    oversample: OverSampleType;\\n}\\n\\ndeclare var WaveShaperNode: {\\n    prototype: WaveShaperNode;\\n    new(context: BaseAudioContext, options?: WaveShaperOptions): WaveShaperNode;\\n};\\n\\ninterface WebAuthentication {\\n    getAssertion(assertionChallenge: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null, options?: AssertionOptions): Promise<WebAuthnAssertion>;\\n    makeCredential(accountInformation: Account, cryptoParameters: ScopedCredentialParameters[], attestationChallenge: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null, options?: ScopedCredentialOptions): Promise<ScopedCredentialInfo>;\\n}\\n\\ndeclare var WebAuthentication: {\\n    prototype: WebAuthentication;\\n    new(): WebAuthentication;\\n};\\n\\ninterface WebAuthnAssertion {\\n    readonly authenticatorData: ArrayBuffer;\\n    readonly clientData: ArrayBuffer;\\n    readonly credential: ScopedCredential;\\n    readonly signature: ArrayBuffer;\\n}\\n\\ndeclare var WebAuthnAssertion: {\\n    prototype: WebAuthnAssertion;\\n    new(): WebAuthnAssertion;\\n};\\n\\ninterface WebGLActiveInfo {\\n    readonly name: string;\\n    readonly size: GLint;\\n    readonly type: GLenum;\\n}\\n\\ndeclare var WebGLActiveInfo: {\\n    prototype: WebGLActiveInfo;\\n    new(): WebGLActiveInfo;\\n};\\n\\ninterface WebGLBuffer extends WebGLObject {\\n}\\n\\ndeclare var WebGLBuffer: {\\n    prototype: WebGLBuffer;\\n    new(): WebGLBuffer;\\n};\\n\\ninterface WebGLContextEvent extends Event {\\n    readonly statusMessage: string;\\n}\\n\\ndeclare var WebGLContextEvent: {\\n    prototype: WebGLContextEvent;\\n    new(type: string, eventInit?: WebGLContextEventInit): WebGLContextEvent;\\n};\\n\\ninterface WebGLFramebuffer extends WebGLObject {\\n}\\n\\ndeclare var WebGLFramebuffer: {\\n    prototype: WebGLFramebuffer;\\n    new(): WebGLFramebuffer;\\n};\\n\\ninterface WebGLObject {\\n}\\n\\ndeclare var WebGLObject: {\\n    prototype: WebGLObject;\\n    new(): WebGLObject;\\n};\\n\\ninterface WebGLProgram extends WebGLObject {\\n}\\n\\ndeclare var WebGLProgram: {\\n    prototype: WebGLProgram;\\n    new(): WebGLProgram;\\n};\\n\\ninterface WebGLRenderbuffer extends WebGLObject {\\n}\\n\\ndeclare var WebGLRenderbuffer: {\\n    prototype: WebGLRenderbuffer;\\n    new(): WebGLRenderbuffer;\\n};\\n\\ninterface WebGLRenderingContext extends WebGLRenderingContextBase {\\n}\\n\\ndeclare var WebGLRenderingContext: {\\n    prototype: WebGLRenderingContext;\\n    new(): WebGLRenderingContext;\\n    readonly ACTIVE_ATTRIBUTES: GLenum;\\n    readonly ACTIVE_TEXTURE: GLenum;\\n    readonly ACTIVE_UNIFORMS: GLenum;\\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\\n    readonly ALPHA: GLenum;\\n    readonly ALPHA_BITS: GLenum;\\n    readonly ALWAYS: GLenum;\\n    readonly ARRAY_BUFFER: GLenum;\\n    readonly ARRAY_BUFFER_BINDING: GLenum;\\n    readonly ATTACHED_SHADERS: GLenum;\\n    readonly BACK: GLenum;\\n    readonly BLEND: GLenum;\\n    readonly BLEND_COLOR: GLenum;\\n    readonly BLEND_DST_ALPHA: GLenum;\\n    readonly BLEND_DST_RGB: GLenum;\\n    readonly BLEND_EQUATION: GLenum;\\n    readonly BLEND_EQUATION_ALPHA: GLenum;\\n    readonly BLEND_EQUATION_RGB: GLenum;\\n    readonly BLEND_SRC_ALPHA: GLenum;\\n    readonly BLEND_SRC_RGB: GLenum;\\n    readonly BLUE_BITS: GLenum;\\n    readonly BOOL: GLenum;\\n    readonly BOOL_VEC2: GLenum;\\n    readonly BOOL_VEC3: GLenum;\\n    readonly BOOL_VEC4: GLenum;\\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\\n    readonly BUFFER_SIZE: GLenum;\\n    readonly BUFFER_USAGE: GLenum;\\n    readonly BYTE: GLenum;\\n    readonly CCW: GLenum;\\n    readonly CLAMP_TO_EDGE: GLenum;\\n    readonly COLOR_ATTACHMENT0: GLenum;\\n    readonly COLOR_BUFFER_BIT: GLenum;\\n    readonly COLOR_CLEAR_VALUE: GLenum;\\n    readonly COLOR_WRITEMASK: GLenum;\\n    readonly COMPILE_STATUS: GLenum;\\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\\n    readonly CONSTANT_ALPHA: GLenum;\\n    readonly CONSTANT_COLOR: GLenum;\\n    readonly CONTEXT_LOST_WEBGL: GLenum;\\n    readonly CULL_FACE: GLenum;\\n    readonly CULL_FACE_MODE: GLenum;\\n    readonly CURRENT_PROGRAM: GLenum;\\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\\n    readonly CW: GLenum;\\n    readonly DECR: GLenum;\\n    readonly DECR_WRAP: GLenum;\\n    readonly DELETE_STATUS: GLenum;\\n    readonly DEPTH_ATTACHMENT: GLenum;\\n    readonly DEPTH_BITS: GLenum;\\n    readonly DEPTH_BUFFER_BIT: GLenum;\\n    readonly DEPTH_CLEAR_VALUE: GLenum;\\n    readonly DEPTH_COMPONENT: GLenum;\\n    readonly DEPTH_COMPONENT16: GLenum;\\n    readonly DEPTH_FUNC: GLenum;\\n    readonly DEPTH_RANGE: GLenum;\\n    readonly DEPTH_STENCIL: GLenum;\\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\\n    readonly DEPTH_TEST: GLenum;\\n    readonly DEPTH_WRITEMASK: GLenum;\\n    readonly DITHER: GLenum;\\n    readonly DONT_CARE: GLenum;\\n    readonly DST_ALPHA: GLenum;\\n    readonly DST_COLOR: GLenum;\\n    readonly DYNAMIC_DRAW: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly EQUAL: GLenum;\\n    readonly FASTEST: GLenum;\\n    readonly FLOAT: GLenum;\\n    readonly FLOAT_MAT2: GLenum;\\n    readonly FLOAT_MAT3: GLenum;\\n    readonly FLOAT_MAT4: GLenum;\\n    readonly FLOAT_VEC2: GLenum;\\n    readonly FLOAT_VEC3: GLenum;\\n    readonly FLOAT_VEC4: GLenum;\\n    readonly FRAGMENT_SHADER: GLenum;\\n    readonly FRAMEBUFFER: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\\n    readonly FRAMEBUFFER_BINDING: GLenum;\\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\\n    readonly FRONT: GLenum;\\n    readonly FRONT_AND_BACK: GLenum;\\n    readonly FRONT_FACE: GLenum;\\n    readonly FUNC_ADD: GLenum;\\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\\n    readonly FUNC_SUBTRACT: GLenum;\\n    readonly GENERATE_MIPMAP_HINT: GLenum;\\n    readonly GEQUAL: GLenum;\\n    readonly GREATER: GLenum;\\n    readonly GREEN_BITS: GLenum;\\n    readonly HIGH_FLOAT: GLenum;\\n    readonly HIGH_INT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\\n    readonly INCR: GLenum;\\n    readonly INCR_WRAP: GLenum;\\n    readonly INT: GLenum;\\n    readonly INT_VEC2: GLenum;\\n    readonly INT_VEC3: GLenum;\\n    readonly INT_VEC4: GLenum;\\n    readonly INVALID_ENUM: GLenum;\\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\\n    readonly INVALID_OPERATION: GLenum;\\n    readonly INVALID_VALUE: GLenum;\\n    readonly INVERT: GLenum;\\n    readonly KEEP: GLenum;\\n    readonly LEQUAL: GLenum;\\n    readonly LESS: GLenum;\\n    readonly LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\\n    readonly LINES: GLenum;\\n    readonly LINE_LOOP: GLenum;\\n    readonly LINE_STRIP: GLenum;\\n    readonly LINE_WIDTH: GLenum;\\n    readonly LINK_STATUS: GLenum;\\n    readonly LOW_FLOAT: GLenum;\\n    readonly LOW_INT: GLenum;\\n    readonly LUMINANCE: GLenum;\\n    readonly LUMINANCE_ALPHA: GLenum;\\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_TEXTURE_SIZE: GLenum;\\n    readonly MAX_VARYING_VECTORS: GLenum;\\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_VIEWPORT_DIMS: GLenum;\\n    readonly MEDIUM_FLOAT: GLenum;\\n    readonly MEDIUM_INT: GLenum;\\n    readonly MIRRORED_REPEAT: GLenum;\\n    readonly NEAREST: GLenum;\\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\\n    readonly NEVER: GLenum;\\n    readonly NICEST: GLenum;\\n    readonly NONE: GLenum;\\n    readonly NOTEQUAL: GLenum;\\n    readonly NO_ERROR: GLenum;\\n    readonly ONE: GLenum;\\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\\n    readonly ONE_MINUS_DST_COLOR: GLenum;\\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\\n    readonly OUT_OF_MEMORY: GLenum;\\n    readonly PACK_ALIGNMENT: GLenum;\\n    readonly POINTS: GLenum;\\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\\n    readonly POLYGON_OFFSET_FILL: GLenum;\\n    readonly POLYGON_OFFSET_UNITS: GLenum;\\n    readonly RED_BITS: GLenum;\\n    readonly RENDERBUFFER: GLenum;\\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\\n    readonly RENDERBUFFER_BINDING: GLenum;\\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\\n    readonly RENDERBUFFER_HEIGHT: GLenum;\\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\\n    readonly RENDERBUFFER_WIDTH: GLenum;\\n    readonly RENDERER: GLenum;\\n    readonly REPEAT: GLenum;\\n    readonly REPLACE: GLenum;\\n    readonly RGB: GLenum;\\n    readonly RGB565: GLenum;\\n    readonly RGB5_A1: GLenum;\\n    readonly RGBA: GLenum;\\n    readonly RGBA4: GLenum;\\n    readonly SAMPLER_2D: GLenum;\\n    readonly SAMPLER_CUBE: GLenum;\\n    readonly SAMPLES: GLenum;\\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\\n    readonly SAMPLE_BUFFERS: GLenum;\\n    readonly SAMPLE_COVERAGE: GLenum;\\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\\n    readonly SCISSOR_BOX: GLenum;\\n    readonly SCISSOR_TEST: GLenum;\\n    readonly SHADER_TYPE: GLenum;\\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\\n    readonly SHORT: GLenum;\\n    readonly SRC_ALPHA: GLenum;\\n    readonly SRC_ALPHA_SATURATE: GLenum;\\n    readonly SRC_COLOR: GLenum;\\n    readonly STATIC_DRAW: GLenum;\\n    readonly STENCIL_ATTACHMENT: GLenum;\\n    readonly STENCIL_BACK_FAIL: GLenum;\\n    readonly STENCIL_BACK_FUNC: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_BACK_REF: GLenum;\\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\\n    readonly STENCIL_BITS: GLenum;\\n    readonly STENCIL_BUFFER_BIT: GLenum;\\n    readonly STENCIL_CLEAR_VALUE: GLenum;\\n    readonly STENCIL_FAIL: GLenum;\\n    readonly STENCIL_FUNC: GLenum;\\n    readonly STENCIL_INDEX8: GLenum;\\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_REF: GLenum;\\n    readonly STENCIL_TEST: GLenum;\\n    readonly STENCIL_VALUE_MASK: GLenum;\\n    readonly STENCIL_WRITEMASK: GLenum;\\n    readonly STREAM_DRAW: GLenum;\\n    readonly SUBPIXEL_BITS: GLenum;\\n    readonly TEXTURE: GLenum;\\n    readonly TEXTURE0: GLenum;\\n    readonly TEXTURE1: GLenum;\\n    readonly TEXTURE10: GLenum;\\n    readonly TEXTURE11: GLenum;\\n    readonly TEXTURE12: GLenum;\\n    readonly TEXTURE13: GLenum;\\n    readonly TEXTURE14: GLenum;\\n    readonly TEXTURE15: GLenum;\\n    readonly TEXTURE16: GLenum;\\n    readonly TEXTURE17: GLenum;\\n    readonly TEXTURE18: GLenum;\\n    readonly TEXTURE19: GLenum;\\n    readonly TEXTURE2: GLenum;\\n    readonly TEXTURE20: GLenum;\\n    readonly TEXTURE21: GLenum;\\n    readonly TEXTURE22: GLenum;\\n    readonly TEXTURE23: GLenum;\\n    readonly TEXTURE24: GLenum;\\n    readonly TEXTURE25: GLenum;\\n    readonly TEXTURE26: GLenum;\\n    readonly TEXTURE27: GLenum;\\n    readonly TEXTURE28: GLenum;\\n    readonly TEXTURE29: GLenum;\\n    readonly TEXTURE3: GLenum;\\n    readonly TEXTURE30: GLenum;\\n    readonly TEXTURE31: GLenum;\\n    readonly TEXTURE4: GLenum;\\n    readonly TEXTURE5: GLenum;\\n    readonly TEXTURE6: GLenum;\\n    readonly TEXTURE7: GLenum;\\n    readonly TEXTURE8: GLenum;\\n    readonly TEXTURE9: GLenum;\\n    readonly TEXTURE_2D: GLenum;\\n    readonly TEXTURE_BINDING_2D: GLenum;\\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\\n    readonly TEXTURE_MAG_FILTER: GLenum;\\n    readonly TEXTURE_MIN_FILTER: GLenum;\\n    readonly TEXTURE_WRAP_S: GLenum;\\n    readonly TEXTURE_WRAP_T: GLenum;\\n    readonly TRIANGLES: GLenum;\\n    readonly TRIANGLE_FAN: GLenum;\\n    readonly TRIANGLE_STRIP: GLenum;\\n    readonly UNPACK_ALIGNMENT: GLenum;\\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\\n    readonly UNSIGNED_BYTE: GLenum;\\n    readonly UNSIGNED_INT: GLenum;\\n    readonly UNSIGNED_SHORT: GLenum;\\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\\n    readonly VALIDATE_STATUS: GLenum;\\n    readonly VENDOR: GLenum;\\n    readonly VERSION: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\\n    readonly VERTEX_SHADER: GLenum;\\n    readonly VIEWPORT: GLenum;\\n    readonly ZERO: GLenum;\\n};\\n\\ninterface WebGLRenderingContextBase {\\n    readonly canvas: HTMLCanvasElement;\\n    readonly drawingBufferHeight: GLsizei;\\n    readonly drawingBufferWidth: GLsizei;\\n    activeTexture(texture: GLenum): void;\\n    attachShader(program: WebGLProgram, shader: WebGLShader): void;\\n    bindAttribLocation(program: WebGLProgram, index: GLuint, name: string): void;\\n    bindBuffer(target: GLenum, buffer: WebGLBuffer | null): void;\\n    bindFramebuffer(target: GLenum, framebuffer: WebGLFramebuffer | null): void;\\n    bindRenderbuffer(target: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\\n    bindTexture(target: GLenum, texture: WebGLTexture | null): void;\\n    blendColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\\n    blendEquation(mode: GLenum): void;\\n    blendEquationSeparate(modeRGB: GLenum, modeAlpha: GLenum): void;\\n    blendFunc(sfactor: GLenum, dfactor: GLenum): void;\\n    blendFuncSeparate(srcRGB: GLenum, dstRGB: GLenum, srcAlpha: GLenum, dstAlpha: GLenum): void;\\n    bufferData(target: GLenum, size: GLsizeiptr, usage: GLenum): void;\\n    bufferData(target: GLenum, data: BufferSource | null, usage: GLenum): void;\\n    bufferSubData(target: GLenum, offset: GLintptr, data: BufferSource): void;\\n    checkFramebufferStatus(target: GLenum): GLenum;\\n    clear(mask: GLbitfield): void;\\n    clearColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\\n    clearDepth(depth: GLclampf): void;\\n    clearStencil(s: GLint): void;\\n    colorMask(red: GLboolean, green: GLboolean, blue: GLboolean, alpha: GLboolean): void;\\n    compileShader(shader: WebGLShader): void;\\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, data: ArrayBufferView): void;\\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, data: ArrayBufferView): void;\\n    copyTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, x: GLint, y: GLint, width: GLsizei, height: GLsizei, border: GLint): void;\\n    copyTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\\n    createBuffer(): WebGLBuffer | null;\\n    createFramebuffer(): WebGLFramebuffer | null;\\n    createProgram(): WebGLProgram | null;\\n    createRenderbuffer(): WebGLRenderbuffer | null;\\n    createShader(type: GLenum): WebGLShader | null;\\n    createTexture(): WebGLTexture | null;\\n    cullFace(mode: GLenum): void;\\n    deleteBuffer(buffer: WebGLBuffer | null): void;\\n    deleteFramebuffer(framebuffer: WebGLFramebuffer | null): void;\\n    deleteProgram(program: WebGLProgram | null): void;\\n    deleteRenderbuffer(renderbuffer: WebGLRenderbuffer | null): void;\\n    deleteShader(shader: WebGLShader | null): void;\\n    deleteTexture(texture: WebGLTexture | null): void;\\n    depthFunc(func: GLenum): void;\\n    depthMask(flag: GLboolean): void;\\n    depthRange(zNear: GLclampf, zFar: GLclampf): void;\\n    detachShader(program: WebGLProgram, shader: WebGLShader): void;\\n    disable(cap: GLenum): void;\\n    disableVertexAttribArray(index: GLuint): void;\\n    drawArrays(mode: GLenum, first: GLint, count: GLsizei): void;\\n    drawElements(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr): void;\\n    enable(cap: GLenum): void;\\n    enableVertexAttribArray(index: GLuint): void;\\n    finish(): void;\\n    flush(): void;\\n    framebufferRenderbuffer(target: GLenum, attachment: GLenum, renderbuffertarget: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\\n    framebufferTexture2D(target: GLenum, attachment: GLenum, textarget: GLenum, texture: WebGLTexture | null, level: GLint): void;\\n    frontFace(mode: GLenum): void;\\n    generateMipmap(target: GLenum): void;\\n    getActiveAttrib(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\\n    getActiveUniform(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\\n    getAttachedShaders(program: WebGLProgram): WebGLShader[] | null;\\n    getAttribLocation(program: WebGLProgram, name: string): GLint;\\n    getBufferParameter(target: GLenum, pname: GLenum): any;\\n    getContextAttributes(): WebGLContextAttributes | null;\\n    getError(): GLenum;\\n    getExtension(extensionName: \"EXT_blend_minmax\"): EXT_blend_minmax | null;\\n    getExtension(extensionName: \"EXT_texture_filter_anisotropic\"): EXT_texture_filter_anisotropic | null;\\n    getExtension(extensionName: \"EXT_frag_depth\"): EXT_frag_depth | null;\\n    getExtension(extensionName: \"EXT_shader_texture_lod\"): EXT_shader_texture_lod | null;\\n    getExtension(extensionName: \"EXT_sRGB\"): EXT_sRGB | null;\\n    getExtension(extensionName: \"OES_vertex_array_object\"): OES_vertex_array_object | null;\\n    getExtension(extensionName: \"WEBGL_color_buffer_float\"): WEBGL_color_buffer_float | null;\\n    getExtension(extensionName: \"WEBGL_compressed_texture_astc\"): WEBGL_compressed_texture_astc | null;\\n    getExtension(extensionName: \"WEBGL_compressed_texture_s3tc_srgb\"): WEBGL_compressed_texture_s3tc_srgb | null;\\n    getExtension(extensionName: \"WEBGL_debug_shaders\"): WEBGL_debug_shaders | null;\\n    getExtension(extensionName: \"WEBGL_draw_buffers\"): WEBGL_draw_buffers | null;\\n    getExtension(extensionName: \"WEBGL_lose_context\"): WEBGL_lose_context | null;\\n    getExtension(extensionName: \"WEBGL_depth_texture\"): WEBGL_depth_texture | null;\\n    getExtension(extensionName: \"WEBGL_debug_renderer_info\"): WEBGL_debug_renderer_info | null;\\n    getExtension(extensionName: \"WEBGL_compressed_texture_s3tc\"): WEBGL_compressed_texture_s3tc | null;\\n    getExtension(extensionName: \"OES_texture_half_float_linear\"): OES_texture_half_float_linear | null;\\n    getExtension(extensionName: \"OES_texture_half_float\"): OES_texture_half_float | null;\\n    getExtension(extensionName: \"OES_texture_float_linear\"): OES_texture_float_linear | null;\\n    getExtension(extensionName: \"OES_texture_float\"): OES_texture_float | null;\\n    getExtension(extensionName: \"OES_standard_derivatives\"): OES_standard_derivatives | null;\\n    getExtension(extensionName: \"OES_element_index_uint\"): OES_element_index_uint | null;\\n    getExtension(extensionName: \"ANGLE_instanced_arrays\"): ANGLE_instanced_arrays | null;\\n    getExtension(extensionName: string): any;\\n    getFramebufferAttachmentParameter(target: GLenum, attachment: GLenum, pname: GLenum): any;\\n    getParameter(pname: GLenum): any;\\n    getProgramInfoLog(program: WebGLProgram): string | null;\\n    getProgramParameter(program: WebGLProgram, pname: GLenum): any;\\n    getRenderbufferParameter(target: GLenum, pname: GLenum): any;\\n    getShaderInfoLog(shader: WebGLShader): string | null;\\n    getShaderParameter(shader: WebGLShader, pname: GLenum): any;\\n    getShaderPrecisionFormat(shadertype: GLenum, precisiontype: GLenum): WebGLShaderPrecisionFormat | null;\\n    getShaderSource(shader: WebGLShader): string | null;\\n    getSupportedExtensions(): string[] | null;\\n    getTexParameter(target: GLenum, pname: GLenum): any;\\n    getUniform(program: WebGLProgram, location: WebGLUniformLocation): any;\\n    getUniformLocation(program: WebGLProgram, name: string): WebGLUniformLocation | null;\\n    getVertexAttrib(index: GLuint, pname: GLenum): any;\\n    getVertexAttribOffset(index: GLuint, pname: GLenum): GLintptr;\\n    hint(target: GLenum, mode: GLenum): void;\\n    isBuffer(buffer: WebGLBuffer | null): GLboolean;\\n    isContextLost(): boolean;\\n    isEnabled(cap: GLenum): GLboolean;\\n    isFramebuffer(framebuffer: WebGLFramebuffer | null): GLboolean;\\n    isProgram(program: WebGLProgram | null): GLboolean;\\n    isRenderbuffer(renderbuffer: WebGLRenderbuffer | null): GLboolean;\\n    isShader(shader: WebGLShader | null): GLboolean;\\n    isTexture(texture: WebGLTexture | null): GLboolean;\\n    lineWidth(width: GLfloat): void;\\n    linkProgram(program: WebGLProgram): void;\\n    pixelStorei(pname: GLenum, param: GLint): void;\\n    polygonOffset(factor: GLfloat, units: GLfloat): void;\\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\\n    renderbufferStorage(target: GLenum, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\\n    sampleCoverage(value: GLclampf, invert: GLboolean): void;\\n    scissor(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\\n    shaderSource(shader: WebGLShader, source: string): void;\\n    stencilFunc(func: GLenum, ref: GLint, mask: GLuint): void;\\n    stencilFuncSeparate(face: GLenum, func: GLenum, ref: GLint, mask: GLuint): void;\\n    stencilMask(mask: GLuint): void;\\n    stencilMaskSeparate(face: GLenum, mask: GLuint): void;\\n    stencilOp(fail: GLenum, zfail: GLenum, zpass: GLenum): void;\\n    stencilOpSeparate(face: GLenum, fail: GLenum, zfail: GLenum, zpass: GLenum): void;\\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\\n    texParameterf(target: GLenum, pname: GLenum, param: GLfloat): void;\\n    texParameteri(target: GLenum, pname: GLenum, param: GLint): void;\\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\\n    uniform1f(location: WebGLUniformLocation | null, x: GLfloat): void;\\n    uniform1fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform1i(location: WebGLUniformLocation | null, x: GLint): void;\\n    uniform1iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniform2f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat): void;\\n    uniform2fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform2i(location: WebGLUniformLocation | null, x: GLint, y: GLint): void;\\n    uniform2iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniform3f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat): void;\\n    uniform3fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform3i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint): void;\\n    uniform3iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniform4f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\\n    uniform4fv(location: WebGLUniformLocation | null, v: Float32List): void;\\n    uniform4i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint, w: GLint): void;\\n    uniform4iv(location: WebGLUniformLocation | null, v: Int32List): void;\\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\\n    useProgram(program: WebGLProgram | null): void;\\n    validateProgram(program: WebGLProgram): void;\\n    vertexAttrib1f(index: GLuint, x: GLfloat): void;\\n    vertexAttrib1fv(index: GLuint, values: Float32List): void;\\n    vertexAttrib2f(index: GLuint, x: GLfloat, y: GLfloat): void;\\n    vertexAttrib2fv(index: GLuint, values: Float32List): void;\\n    vertexAttrib3f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat): void;\\n    vertexAttrib3fv(index: GLuint, values: Float32List): void;\\n    vertexAttrib4f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\\n    vertexAttrib4fv(index: GLuint, values: Float32List): void;\\n    vertexAttribPointer(index: GLuint, size: GLint, type: GLenum, normalized: GLboolean, stride: GLsizei, offset: GLintptr): void;\\n    viewport(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\\n    readonly ACTIVE_ATTRIBUTES: GLenum;\\n    readonly ACTIVE_TEXTURE: GLenum;\\n    readonly ACTIVE_UNIFORMS: GLenum;\\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\\n    readonly ALPHA: GLenum;\\n    readonly ALPHA_BITS: GLenum;\\n    readonly ALWAYS: GLenum;\\n    readonly ARRAY_BUFFER: GLenum;\\n    readonly ARRAY_BUFFER_BINDING: GLenum;\\n    readonly ATTACHED_SHADERS: GLenum;\\n    readonly BACK: GLenum;\\n    readonly BLEND: GLenum;\\n    readonly BLEND_COLOR: GLenum;\\n    readonly BLEND_DST_ALPHA: GLenum;\\n    readonly BLEND_DST_RGB: GLenum;\\n    readonly BLEND_EQUATION: GLenum;\\n    readonly BLEND_EQUATION_ALPHA: GLenum;\\n    readonly BLEND_EQUATION_RGB: GLenum;\\n    readonly BLEND_SRC_ALPHA: GLenum;\\n    readonly BLEND_SRC_RGB: GLenum;\\n    readonly BLUE_BITS: GLenum;\\n    readonly BOOL: GLenum;\\n    readonly BOOL_VEC2: GLenum;\\n    readonly BOOL_VEC3: GLenum;\\n    readonly BOOL_VEC4: GLenum;\\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\\n    readonly BUFFER_SIZE: GLenum;\\n    readonly BUFFER_USAGE: GLenum;\\n    readonly BYTE: GLenum;\\n    readonly CCW: GLenum;\\n    readonly CLAMP_TO_EDGE: GLenum;\\n    readonly COLOR_ATTACHMENT0: GLenum;\\n    readonly COLOR_BUFFER_BIT: GLenum;\\n    readonly COLOR_CLEAR_VALUE: GLenum;\\n    readonly COLOR_WRITEMASK: GLenum;\\n    readonly COMPILE_STATUS: GLenum;\\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\\n    readonly CONSTANT_ALPHA: GLenum;\\n    readonly CONSTANT_COLOR: GLenum;\\n    readonly CONTEXT_LOST_WEBGL: GLenum;\\n    readonly CULL_FACE: GLenum;\\n    readonly CULL_FACE_MODE: GLenum;\\n    readonly CURRENT_PROGRAM: GLenum;\\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\\n    readonly CW: GLenum;\\n    readonly DECR: GLenum;\\n    readonly DECR_WRAP: GLenum;\\n    readonly DELETE_STATUS: GLenum;\\n    readonly DEPTH_ATTACHMENT: GLenum;\\n    readonly DEPTH_BITS: GLenum;\\n    readonly DEPTH_BUFFER_BIT: GLenum;\\n    readonly DEPTH_CLEAR_VALUE: GLenum;\\n    readonly DEPTH_COMPONENT: GLenum;\\n    readonly DEPTH_COMPONENT16: GLenum;\\n    readonly DEPTH_FUNC: GLenum;\\n    readonly DEPTH_RANGE: GLenum;\\n    readonly DEPTH_STENCIL: GLenum;\\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\\n    readonly DEPTH_TEST: GLenum;\\n    readonly DEPTH_WRITEMASK: GLenum;\\n    readonly DITHER: GLenum;\\n    readonly DONT_CARE: GLenum;\\n    readonly DST_ALPHA: GLenum;\\n    readonly DST_COLOR: GLenum;\\n    readonly DYNAMIC_DRAW: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly EQUAL: GLenum;\\n    readonly FASTEST: GLenum;\\n    readonly FLOAT: GLenum;\\n    readonly FLOAT_MAT2: GLenum;\\n    readonly FLOAT_MAT3: GLenum;\\n    readonly FLOAT_MAT4: GLenum;\\n    readonly FLOAT_VEC2: GLenum;\\n    readonly FLOAT_VEC3: GLenum;\\n    readonly FLOAT_VEC4: GLenum;\\n    readonly FRAGMENT_SHADER: GLenum;\\n    readonly FRAMEBUFFER: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\\n    readonly FRAMEBUFFER_BINDING: GLenum;\\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\\n    readonly FRONT: GLenum;\\n    readonly FRONT_AND_BACK: GLenum;\\n    readonly FRONT_FACE: GLenum;\\n    readonly FUNC_ADD: GLenum;\\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\\n    readonly FUNC_SUBTRACT: GLenum;\\n    readonly GENERATE_MIPMAP_HINT: GLenum;\\n    readonly GEQUAL: GLenum;\\n    readonly GREATER: GLenum;\\n    readonly GREEN_BITS: GLenum;\\n    readonly HIGH_FLOAT: GLenum;\\n    readonly HIGH_INT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\\n    readonly INCR: GLenum;\\n    readonly INCR_WRAP: GLenum;\\n    readonly INT: GLenum;\\n    readonly INT_VEC2: GLenum;\\n    readonly INT_VEC3: GLenum;\\n    readonly INT_VEC4: GLenum;\\n    readonly INVALID_ENUM: GLenum;\\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\\n    readonly INVALID_OPERATION: GLenum;\\n    readonly INVALID_VALUE: GLenum;\\n    readonly INVERT: GLenum;\\n    readonly KEEP: GLenum;\\n    readonly LEQUAL: GLenum;\\n    readonly LESS: GLenum;\\n    readonly LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\\n    readonly LINES: GLenum;\\n    readonly LINE_LOOP: GLenum;\\n    readonly LINE_STRIP: GLenum;\\n    readonly LINE_WIDTH: GLenum;\\n    readonly LINK_STATUS: GLenum;\\n    readonly LOW_FLOAT: GLenum;\\n    readonly LOW_INT: GLenum;\\n    readonly LUMINANCE: GLenum;\\n    readonly LUMINANCE_ALPHA: GLenum;\\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_TEXTURE_SIZE: GLenum;\\n    readonly MAX_VARYING_VECTORS: GLenum;\\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\\n    readonly MAX_VIEWPORT_DIMS: GLenum;\\n    readonly MEDIUM_FLOAT: GLenum;\\n    readonly MEDIUM_INT: GLenum;\\n    readonly MIRRORED_REPEAT: GLenum;\\n    readonly NEAREST: GLenum;\\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\\n    readonly NEVER: GLenum;\\n    readonly NICEST: GLenum;\\n    readonly NONE: GLenum;\\n    readonly NOTEQUAL: GLenum;\\n    readonly NO_ERROR: GLenum;\\n    readonly ONE: GLenum;\\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\\n    readonly ONE_MINUS_DST_COLOR: GLenum;\\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\\n    readonly OUT_OF_MEMORY: GLenum;\\n    readonly PACK_ALIGNMENT: GLenum;\\n    readonly POINTS: GLenum;\\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\\n    readonly POLYGON_OFFSET_FILL: GLenum;\\n    readonly POLYGON_OFFSET_UNITS: GLenum;\\n    readonly RED_BITS: GLenum;\\n    readonly RENDERBUFFER: GLenum;\\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\\n    readonly RENDERBUFFER_BINDING: GLenum;\\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\\n    readonly RENDERBUFFER_HEIGHT: GLenum;\\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\\n    readonly RENDERBUFFER_WIDTH: GLenum;\\n    readonly RENDERER: GLenum;\\n    readonly REPEAT: GLenum;\\n    readonly REPLACE: GLenum;\\n    readonly RGB: GLenum;\\n    readonly RGB565: GLenum;\\n    readonly RGB5_A1: GLenum;\\n    readonly RGBA: GLenum;\\n    readonly RGBA4: GLenum;\\n    readonly SAMPLER_2D: GLenum;\\n    readonly SAMPLER_CUBE: GLenum;\\n    readonly SAMPLES: GLenum;\\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\\n    readonly SAMPLE_BUFFERS: GLenum;\\n    readonly SAMPLE_COVERAGE: GLenum;\\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\\n    readonly SCISSOR_BOX: GLenum;\\n    readonly SCISSOR_TEST: GLenum;\\n    readonly SHADER_TYPE: GLenum;\\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\\n    readonly SHORT: GLenum;\\n    readonly SRC_ALPHA: GLenum;\\n    readonly SRC_ALPHA_SATURATE: GLenum;\\n    readonly SRC_COLOR: GLenum;\\n    readonly STATIC_DRAW: GLenum;\\n    readonly STENCIL_ATTACHMENT: GLenum;\\n    readonly STENCIL_BACK_FAIL: GLenum;\\n    readonly STENCIL_BACK_FUNC: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_BACK_REF: GLenum;\\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\\n    readonly STENCIL_BITS: GLenum;\\n    readonly STENCIL_BUFFER_BIT: GLenum;\\n    readonly STENCIL_CLEAR_VALUE: GLenum;\\n    readonly STENCIL_FAIL: GLenum;\\n    readonly STENCIL_FUNC: GLenum;\\n    readonly STENCIL_INDEX8: GLenum;\\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\\n    readonly STENCIL_REF: GLenum;\\n    readonly STENCIL_TEST: GLenum;\\n    readonly STENCIL_VALUE_MASK: GLenum;\\n    readonly STENCIL_WRITEMASK: GLenum;\\n    readonly STREAM_DRAW: GLenum;\\n    readonly SUBPIXEL_BITS: GLenum;\\n    readonly TEXTURE: GLenum;\\n    readonly TEXTURE0: GLenum;\\n    readonly TEXTURE1: GLenum;\\n    readonly TEXTURE10: GLenum;\\n    readonly TEXTURE11: GLenum;\\n    readonly TEXTURE12: GLenum;\\n    readonly TEXTURE13: GLenum;\\n    readonly TEXTURE14: GLenum;\\n    readonly TEXTURE15: GLenum;\\n    readonly TEXTURE16: GLenum;\\n    readonly TEXTURE17: GLenum;\\n    readonly TEXTURE18: GLenum;\\n    readonly TEXTURE19: GLenum;\\n    readonly TEXTURE2: GLenum;\\n    readonly TEXTURE20: GLenum;\\n    readonly TEXTURE21: GLenum;\\n    readonly TEXTURE22: GLenum;\\n    readonly TEXTURE23: GLenum;\\n    readonly TEXTURE24: GLenum;\\n    readonly TEXTURE25: GLenum;\\n    readonly TEXTURE26: GLenum;\\n    readonly TEXTURE27: GLenum;\\n    readonly TEXTURE28: GLenum;\\n    readonly TEXTURE29: GLenum;\\n    readonly TEXTURE3: GLenum;\\n    readonly TEXTURE30: GLenum;\\n    readonly TEXTURE31: GLenum;\\n    readonly TEXTURE4: GLenum;\\n    readonly TEXTURE5: GLenum;\\n    readonly TEXTURE6: GLenum;\\n    readonly TEXTURE7: GLenum;\\n    readonly TEXTURE8: GLenum;\\n    readonly TEXTURE9: GLenum;\\n    readonly TEXTURE_2D: GLenum;\\n    readonly TEXTURE_BINDING_2D: GLenum;\\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\\n    readonly TEXTURE_MAG_FILTER: GLenum;\\n    readonly TEXTURE_MIN_FILTER: GLenum;\\n    readonly TEXTURE_WRAP_S: GLenum;\\n    readonly TEXTURE_WRAP_T: GLenum;\\n    readonly TRIANGLES: GLenum;\\n    readonly TRIANGLE_FAN: GLenum;\\n    readonly TRIANGLE_STRIP: GLenum;\\n    readonly UNPACK_ALIGNMENT: GLenum;\\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\\n    readonly UNSIGNED_BYTE: GLenum;\\n    readonly UNSIGNED_INT: GLenum;\\n    readonly UNSIGNED_SHORT: GLenum;\\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\\n    readonly VALIDATE_STATUS: GLenum;\\n    readonly VENDOR: GLenum;\\n    readonly VERSION: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\\n    readonly VERTEX_SHADER: GLenum;\\n    readonly VIEWPORT: GLenum;\\n    readonly ZERO: GLenum;\\n}\\n\\ninterface WebGLShader extends WebGLObject {\\n}\\n\\ndeclare var WebGLShader: {\\n    prototype: WebGLShader;\\n    new(): WebGLShader;\\n};\\n\\ninterface WebGLShaderPrecisionFormat {\\n    readonly precision: GLint;\\n    readonly rangeMax: GLint;\\n    readonly rangeMin: GLint;\\n}\\n\\ndeclare var WebGLShaderPrecisionFormat: {\\n    prototype: WebGLShaderPrecisionFormat;\\n    new(): WebGLShaderPrecisionFormat;\\n};\\n\\ninterface WebGLTexture extends WebGLObject {\\n}\\n\\ndeclare var WebGLTexture: {\\n    prototype: WebGLTexture;\\n    new(): WebGLTexture;\\n};\\n\\ninterface WebGLUniformLocation {\\n}\\n\\ndeclare var WebGLUniformLocation: {\\n    prototype: WebGLUniformLocation;\\n    new(): WebGLUniformLocation;\\n};\\n\\ninterface WebGLVertexArrayObjectOES {\\n}\\n\\ninterface WebKitPoint {\\n    x: number;\\n    y: number;\\n}\\n\\ndeclare var WebKitPoint: {\\n    prototype: WebKitPoint;\\n    new(x?: number, y?: number): WebKitPoint;\\n};\\n\\ninterface WebSocketEventMap {\\n    \"close\": CloseEvent;\\n    \"error\": Event;\\n    \"message\": MessageEvent;\\n    \"open\": Event;\\n}\\n\\ninterface WebSocket extends EventTarget {\\n    binaryType: BinaryType;\\n    readonly bufferedAmount: number;\\n    readonly extensions: string;\\n    onclose: ((this: WebSocket, ev: CloseEvent) => any) | null;\\n    onerror: ((this: WebSocket, ev: Event) => any) | null;\\n    onmessage: ((this: WebSocket, ev: MessageEvent) => any) | null;\\n    onopen: ((this: WebSocket, ev: Event) => any) | null;\\n    readonly protocol: string;\\n    readonly readyState: number;\\n    readonly url: string;\\n    close(code?: number, reason?: string): void;\\n    send(data: string | ArrayBufferLike | Blob | ArrayBufferView): void;\\n    readonly CLOSED: number;\\n    readonly CLOSING: number;\\n    readonly CONNECTING: number;\\n    readonly OPEN: number;\\n    addEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var WebSocket: {\\n    prototype: WebSocket;\\n    new(url: string, protocols?: string | string[]): WebSocket;\\n    readonly CLOSED: number;\\n    readonly CLOSING: number;\\n    readonly CONNECTING: number;\\n    readonly OPEN: number;\\n};\\n\\ninterface WheelEvent extends MouseEvent {\\n    readonly deltaMode: number;\\n    readonly deltaX: number;\\n    readonly deltaY: number;\\n    readonly deltaZ: number;\\n    readonly wheelDelta: number;\\n    readonly wheelDeltaX: number;\\n    readonly wheelDeltaY: number;\\n    getCurrentPoint(element: Element): void;\\n    initWheelEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, buttonArg: number, relatedTargetArg: EventTarget, modifiersListArg: string, deltaXArg: number, deltaYArg: number, deltaZArg: number, deltaMode: number): void;\\n    readonly DOM_DELTA_LINE: number;\\n    readonly DOM_DELTA_PAGE: number;\\n    readonly DOM_DELTA_PIXEL: number;\\n}\\n\\ndeclare var WheelEvent: {\\n    prototype: WheelEvent;\\n    new(typeArg: string, eventInitDict?: WheelEventInit): WheelEvent;\\n    readonly DOM_DELTA_LINE: number;\\n    readonly DOM_DELTA_PAGE: number;\\n    readonly DOM_DELTA_PIXEL: number;\\n};\\n\\ninterface WindowEventMap extends GlobalEventHandlersEventMap, WindowEventHandlersEventMap {\\n    \"abort\": UIEvent;\\n    \"afterprint\": Event;\\n    \"beforeprint\": Event;\\n    \"beforeunload\": BeforeUnloadEvent;\\n    \"blur\": FocusEvent;\\n    \"canplay\": Event;\\n    \"canplaythrough\": Event;\\n    \"change\": Event;\\n    \"click\": MouseEvent;\\n    \"compassneedscalibration\": Event;\\n    \"contextmenu\": MouseEvent;\\n    \"dblclick\": MouseEvent;\\n    \"devicelight\": DeviceLightEvent;\\n    \"devicemotion\": DeviceMotionEvent;\\n    \"deviceorientation\": DeviceOrientationEvent;\\n    \"drag\": DragEvent;\\n    \"dragend\": DragEvent;\\n    \"dragenter\": DragEvent;\\n    \"dragleave\": DragEvent;\\n    \"dragover\": DragEvent;\\n    \"dragstart\": DragEvent;\\n    \"drop\": DragEvent;\\n    \"durationchange\": Event;\\n    \"emptied\": Event;\\n    \"ended\": Event;\\n    \"error\": ErrorEvent;\\n    \"focus\": FocusEvent;\\n    \"hashchange\": HashChangeEvent;\\n    \"input\": Event;\\n    \"invalid\": Event;\\n    \"keydown\": KeyboardEvent;\\n    \"keypress\": KeyboardEvent;\\n    \"keyup\": KeyboardEvent;\\n    \"load\": Event;\\n    \"loadeddata\": Event;\\n    \"loadedmetadata\": Event;\\n    \"loadstart\": Event;\\n    \"message\": MessageEvent;\\n    \"mousedown\": MouseEvent;\\n    \"mouseenter\": MouseEvent;\\n    \"mouseleave\": MouseEvent;\\n    \"mousemove\": MouseEvent;\\n    \"mouseout\": MouseEvent;\\n    \"mouseover\": MouseEvent;\\n    \"mouseup\": MouseEvent;\\n    \"mousewheel\": Event;\\n    \"MSGestureChange\": Event;\\n    \"MSGestureDoubleTap\": Event;\\n    \"MSGestureEnd\": Event;\\n    \"MSGestureHold\": Event;\\n    \"MSGestureStart\": Event;\\n    \"MSGestureTap\": Event;\\n    \"MSInertiaStart\": Event;\\n    \"MSPointerCancel\": Event;\\n    \"MSPointerDown\": Event;\\n    \"MSPointerEnter\": Event;\\n    \"MSPointerLeave\": Event;\\n    \"MSPointerMove\": Event;\\n    \"MSPointerOut\": Event;\\n    \"MSPointerOver\": Event;\\n    \"MSPointerUp\": Event;\\n    \"offline\": Event;\\n    \"online\": Event;\\n    \"orientationchange\": Event;\\n    \"pagehide\": PageTransitionEvent;\\n    \"pageshow\": PageTransitionEvent;\\n    \"pause\": Event;\\n    \"play\": Event;\\n    \"playing\": Event;\\n    \"popstate\": PopStateEvent;\\n    \"progress\": ProgressEvent;\\n    \"ratechange\": Event;\\n    \"readystatechange\": ProgressEvent;\\n    \"reset\": Event;\\n    \"resize\": UIEvent;\\n    \"scroll\": UIEvent;\\n    \"seeked\": Event;\\n    \"seeking\": Event;\\n    \"select\": UIEvent;\\n    \"stalled\": Event;\\n    \"storage\": StorageEvent;\\n    \"submit\": Event;\\n    \"suspend\": Event;\\n    \"timeupdate\": Event;\\n    \"unload\": Event;\\n    \"volumechange\": Event;\\n    \"vrdisplayactivate\": Event;\\n    \"vrdisplayblur\": Event;\\n    \"vrdisplayconnect\": Event;\\n    \"vrdisplaydeactivate\": Event;\\n    \"vrdisplaydisconnect\": Event;\\n    \"vrdisplayfocus\": Event;\\n    \"vrdisplaypointerrestricted\": Event;\\n    \"vrdisplaypointerunrestricted\": Event;\\n    \"vrdisplaypresentchange\": Event;\\n    \"waiting\": Event;\\n}\\n\\ninterface Window extends EventTarget, WindowTimers, WindowSessionStorage, WindowLocalStorage, WindowConsole, GlobalEventHandlers, IDBEnvironment, WindowBase64, GlobalFetch, WindowOrWorkerGlobalScope, WindowEventHandlers {\\n    Blob: typeof Blob;\\n    URL: typeof URL;\\n    URLSearchParams: typeof URLSearchParams;\\n    readonly applicationCache: ApplicationCache;\\n    readonly caches: CacheStorage;\\n    readonly clientInformation: Navigator;\\n    readonly closed: boolean;\\n    readonly crypto: Crypto;\\n    customElements: CustomElementRegistry;\\n    defaultStatus: string;\\n    readonly devicePixelRatio: number;\\n    readonly doNotTrack: string;\\n    readonly document: Document;\\n    readonly event: Event | undefined;\\n    /** @deprecated */\\n    readonly external: External;\\n    readonly frameElement: Element;\\n    readonly frames: Window;\\n    readonly history: History;\\n    readonly innerHeight: number;\\n    readonly innerWidth: number;\\n    readonly isSecureContext: boolean;\\n    readonly length: number;\\n    location: Location;\\n    readonly locationbar: BarProp;\\n    readonly menubar: BarProp;\\n    readonly msContentScript: ExtensionScriptApis;\\n    name: string;\\n    readonly navigator: Navigator;\\n    offscreenBuffering: string | boolean;\\n    oncompassneedscalibration: ((this: Window, ev: Event) => any) | null;\\n    ondevicelight: ((this: Window, ev: DeviceLightEvent) => any) | null;\\n    ondevicemotion: ((this: Window, ev: DeviceMotionEvent) => any) | null;\\n    ondeviceorientation: ((this: Window, ev: DeviceOrientationEvent) => any) | null;\\n    onmousewheel: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturechange: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturedoubletap: ((this: Window, ev: Event) => any) | null;\\n    onmsgestureend: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturehold: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturestart: ((this: Window, ev: Event) => any) | null;\\n    onmsgesturetap: ((this: Window, ev: Event) => any) | null;\\n    onmsinertiastart: ((this: Window, ev: Event) => any) | null;\\n    onmspointercancel: ((this: Window, ev: Event) => any) | null;\\n    onmspointerdown: ((this: Window, ev: Event) => any) | null;\\n    onmspointerenter: ((this: Window, ev: Event) => any) | null;\\n    onmspointerleave: ((this: Window, ev: Event) => any) | null;\\n    onmspointermove: ((this: Window, ev: Event) => any) | null;\\n    onmspointerout: ((this: Window, ev: Event) => any) | null;\\n    onmspointerover: ((this: Window, ev: Event) => any) | null;\\n    onmspointerup: ((this: Window, ev: Event) => any) | null;\\n    /** @deprecated */\\n    onorientationchange: ((this: Window, ev: Event) => any) | null;\\n    onreadystatechange: ((this: Window, ev: ProgressEvent) => any) | null;\\n    onvrdisplayactivate: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplayblur: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplayconnect: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaydeactivate: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaydisconnect: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplayfocus: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaypointerrestricted: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaypointerunrestricted: ((this: Window, ev: Event) => any) | null;\\n    onvrdisplaypresentchange: ((this: Window, ev: Event) => any) | null;\\n    opener: any;\\n    /** @deprecated */\\n    readonly orientation: string | number;\\n    readonly outerHeight: number;\\n    readonly outerWidth: number;\\n    readonly pageXOffset: number;\\n    readonly pageYOffset: number;\\n    readonly parent: Window;\\n    readonly performance: Performance;\\n    readonly personalbar: BarProp;\\n    readonly screen: Screen;\\n    readonly screenLeft: number;\\n    readonly screenTop: number;\\n    readonly screenX: number;\\n    readonly screenY: number;\\n    readonly scrollX: number;\\n    readonly scrollY: number;\\n    readonly scrollbars: BarProp;\\n    readonly self: Window;\\n    readonly speechSynthesis: SpeechSynthesis;\\n    status: string;\\n    readonly statusbar: BarProp;\\n    readonly styleMedia: StyleMedia;\\n    readonly toolbar: BarProp;\\n    readonly top: Window;\\n    readonly window: Window;\\n    alert(message?: any): void;\\n    blur(): void;\\n    cancelAnimationFrame(handle: number): void;\\n    /** @deprecated */\\n    captureEvents(): void;\\n    close(): void;\\n    confirm(message?: string): boolean;\\n    departFocus(navigationReason: NavigationReason, origin: FocusNavigationOrigin): void;\\n    focus(): void;\\n    getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\\n    getMatchedCSSRules(elt: Element, pseudoElt?: string | null): CSSRuleList;\\n    getSelection(): Selection;\\n    matchMedia(query: string): MediaQueryList;\\n    moveBy(x: number, y: number): void;\\n    moveTo(x: number, y: number): void;\\n    msWriteProfilerMark(profilerMarkName: string): void;\\n    open(url?: string, target?: string, features?: string, replace?: boolean): Window | null;\\n    postMessage(message: any, targetOrigin: string, transfer?: Transferable[]): void;\\n    print(): void;\\n    prompt(message?: string, _default?: string): string | null;\\n    /** @deprecated */\\n    releaseEvents(): void;\\n    requestAnimationFrame(callback: FrameRequestCallback): number;\\n    resizeBy(x: number, y: number): void;\\n    resizeTo(x: number, y: number): void;\\n    scroll(options?: ScrollToOptions): void;\\n    scroll(x: number, y: number): void;\\n    scrollBy(options?: ScrollToOptions): void;\\n    scrollBy(x: number, y: number): void;\\n    scrollTo(options?: ScrollToOptions): void;\\n    scrollTo(x: number, y: number): void;\\n    stop(): void;\\n    webkitCancelAnimationFrame(handle: number): void;\\n    webkitConvertPointFromNodeToPage(node: Node, pt: WebKitPoint): WebKitPoint;\\n    webkitConvertPointFromPageToNode(node: Node, pt: WebKitPoint): WebKitPoint;\\n    webkitRequestAnimationFrame(callback: FrameRequestCallback): number;\\n    addEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Window: {\\n    prototype: Window;\\n    new(): Window;\\n};\\n\\ninterface WindowBase64 {\\n    atob(encodedString: string): string;\\n    btoa(rawString: string): string;\\n}\\n\\ninterface WindowConsole {\\n    readonly console: Console;\\n}\\n\\ninterface WindowEventHandlersEventMap {\\n    \"afterprint\": Event;\\n    \"beforeprint\": Event;\\n    \"beforeunload\": BeforeUnloadEvent;\\n    \"hashchange\": HashChangeEvent;\\n    \"languagechange\": Event;\\n    \"message\": MessageEvent;\\n    \"messageerror\": MessageEvent;\\n    \"offline\": Event;\\n    \"online\": Event;\\n    \"pagehide\": PageTransitionEvent;\\n    \"pageshow\": PageTransitionEvent;\\n    \"popstate\": PopStateEvent;\\n    \"rejectionhandled\": Event;\\n    \"storage\": StorageEvent;\\n    \"unhandledrejection\": PromiseRejectionEvent;\\n    \"unload\": Event;\\n}\\n\\ninterface WindowEventHandlers {\\n    onafterprint: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onbeforeprint: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onbeforeunload: ((this: WindowEventHandlers, ev: BeforeUnloadEvent) => any) | null;\\n    onhashchange: ((this: WindowEventHandlers, ev: HashChangeEvent) => any) | null;\\n    onlanguagechange: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onmessage: ((this: WindowEventHandlers, ev: MessageEvent) => any) | null;\\n    onmessageerror: ((this: WindowEventHandlers, ev: MessageEvent) => any) | null;\\n    onoffline: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    ononline: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onpagehide: ((this: WindowEventHandlers, ev: PageTransitionEvent) => any) | null;\\n    onpageshow: ((this: WindowEventHandlers, ev: PageTransitionEvent) => any) | null;\\n    onpopstate: ((this: WindowEventHandlers, ev: PopStateEvent) => any) | null;\\n    onrejectionhandled: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    onstorage: ((this: WindowEventHandlers, ev: StorageEvent) => any) | null;\\n    onunhandledrejection: ((this: WindowEventHandlers, ev: PromiseRejectionEvent) => any) | null;\\n    onunload: ((this: WindowEventHandlers, ev: Event) => any) | null;\\n    addEventListener<K extends keyof WindowEventHandlersEventMap>(type: K, listener: (this: WindowEventHandlers, ev: WindowEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WindowEventHandlersEventMap>(type: K, listener: (this: WindowEventHandlers, ev: WindowEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ninterface WindowLocalStorage {\\n    readonly localStorage: Storage;\\n}\\n\\ninterface WindowOrWorkerGlobalScope {\\n    readonly caches: CacheStorage;\\n    readonly crypto: Crypto;\\n    readonly indexedDB: IDBFactory;\\n    readonly origin: string;\\n    readonly performance: Performance;\\n    atob(data: string): string;\\n    btoa(data: string): string;\\n    clearInterval(handle?: number): void;\\n    clearTimeout(handle?: number): void;\\n    createImageBitmap(image: ImageBitmapSource): Promise<ImageBitmap>;\\n    createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number): Promise<ImageBitmap>;\\n    fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\n    queueMicrotask(callback: Function): void;\\n    setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\n    setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\n}\\n\\ninterface WindowSessionStorage {\\n    readonly sessionStorage: Storage;\\n}\\n\\ninterface WindowTimers {\\n}\\n\\ninterface WorkerEventMap extends AbstractWorkerEventMap {\\n    \"message\": MessageEvent;\\n}\\n\\ninterface Worker extends EventTarget, AbstractWorker {\\n    onmessage: ((this: Worker, ev: MessageEvent) => any) | null;\\n    postMessage(message: any, transfer?: Transferable[]): void;\\n    terminate(): void;\\n    addEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var Worker: {\\n    prototype: Worker;\\n    new(stringUrl: string): Worker;\\n};\\n\\ninterface Worklet {\\n    addModule(moduleURL: string, options?: WorkletOptions): Promise<void>;\\n}\\n\\ndeclare var Worklet: {\\n    prototype: Worklet;\\n    new(): Worklet;\\n};\\n\\ninterface WritableStream {\\n    readonly locked: boolean;\\n    abort(reason?: any): Promise<void>;\\n    getWriter(): WritableStreamDefaultWriter;\\n}\\n\\ndeclare var WritableStream: {\\n    prototype: WritableStream;\\n    new(underlyingSink?: UnderlyingSink, strategy?: QueuingStrategy): WritableStream;\\n};\\n\\ninterface WritableStreamDefaultController {\\n    error(error?: any): void;\\n}\\n\\ndeclare var WritableStreamDefaultController: {\\n    prototype: WritableStreamDefaultController;\\n    new(): WritableStreamDefaultController;\\n};\\n\\ninterface WritableStreamDefaultWriter {\\n    readonly closed: Promise<void>;\\n    readonly desiredSize: number;\\n    readonly ready: Promise<void>;\\n    abort(reason?: any): Promise<void>;\\n    close(): Promise<void>;\\n    releaseLock(): void;\\n    write(chunk?: any): Promise<any>;\\n}\\n\\ndeclare var WritableStreamDefaultWriter: {\\n    prototype: WritableStreamDefaultWriter;\\n    new(): WritableStreamDefaultWriter;\\n};\\n\\ninterface XMLDocument extends Document {\\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: XMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: XMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLDocument: {\\n    prototype: XMLDocument;\\n    new(): XMLDocument;\\n};\\n\\ninterface XMLHttpRequestEventMap extends XMLHttpRequestEventTargetEventMap {\\n    \"readystatechange\": Event;\\n}\\n\\ninterface XMLHttpRequest extends XMLHttpRequestEventTarget {\\n    onreadystatechange: ((this: XMLHttpRequest, ev: Event) => any) | null;\\n    /**\\n     * Returns client\\'s state.\\n     */\\n    readonly readyState: number;\\n    /**\\n     * Returns the response\\'s body.\\n     */\\n    readonly response: any;\\n    /**\\n     * Returns the text response.\\n     * Throws an \"InvalidStateError\" DOMException if responseType is not the empty string or \"text\".\\n     */\\n    readonly responseText: string;\\n    /**\\n     * Returns the response type.\\n     * Can be set to change the response type. Values are:\\n     * the empty string (default),\\n     * \"arraybuffer\",\\n     * \"blob\",\\n     * \"document\",\\n     * \"json\", and\\n     * \"text\".\\n     * When set: setting to \"document\" is ignored if current global object is not a Window object.\\n     * When set: throws an \"InvalidStateError\" DOMException if state is loading or done.\\n     * When set: throws an \"InvalidAccessError\" DOMException if the synchronous flag is set and current global object is a Window object.\\n     */\\n    responseType: XMLHttpRequestResponseType;\\n    readonly responseURL: string;\\n    /**\\n     * Returns the document response.\\n     * Throws an \"InvalidStateError\" DOMException if responseType is not the empty string or \"document\".\\n     */\\n    readonly responseXML: Document | null;\\n    readonly status: number;\\n    readonly statusText: string;\\n    /**\\n     * Can be set to a time in milliseconds. When set to a non-zero value will cause fetching to terminate after the given time has passed. When the time has passed, the\\n     * request has not yet completed, and the synchronous flag is unset, a timeout event will then be dispatched, or a\\n     * \"TimeoutError\" DOMException will be thrown otherwise (for the send() method).\\n     * When set: throws an \"InvalidAccessError\" DOMException if the synchronous flag is set and current global object is a Window object.\\n     */\\n    timeout: number;\\n    /**\\n     * Returns the associated XMLHttpRequestUpload object. It can be used to gather transmission information when data is\\n     * transferred to a server.\\n     */\\n    readonly upload: XMLHttpRequestUpload;\\n    /**\\n     * True when credentials are to be included in a cross-origin request. False when they are\\n     * to be excluded in a cross-origin request and when cookies are to be ignored in its response.\\n     * Initially false.\\n     * When set: throws an \"InvalidStateError\" DOMException if state is not unsent or opened, or if the send() flag is set.\\n     */\\n    withCredentials: boolean;\\n    /**\\n     * Cancels any network activity.\\n     */\\n    abort(): void;\\n    getAllResponseHeaders(): string;\\n    getResponseHeader(name: string): string | null;\\n    /**\\n     * Sets the request method, request URL, and synchronous flag.\\n     * Throws a \"SyntaxError\" DOMException if either method is not a\\n     * valid HTTP method or url cannot be parsed.\\n     * Throws a \"SecurityError\" DOMException if method is a\\n     * case-insensitive match for `CONNECT`, `TRACE`, or `TRACK`.\\n     * Throws an \"InvalidAccessError\" DOMException if async is false, current global object is a Window object, and the timeout attribute is not zero or the responseType attribute is not the empty string.\\n     */\\n    open(method: string, url: string): void;\\n    open(method: string, url: string, async: boolean, username?: string | null, password?: string | null): void;\\n    /**\\n     * Acts as if the `Content-Type` header value for response is mime.\\n     * (It does not actually change the header though.)\\n     * Throws an \"InvalidStateError\" DOMException if state is loading or done.\\n     */\\n    overrideMimeType(mime: string): void;\\n    /**\\n     * Initiates the request. The optional argument provides the request body. The argument is ignored if request method is GET or HEAD.\\n     * Throws an \"InvalidStateError\" DOMException if either state is not opened or the send() flag is set.\\n     */\\n    send(body?: Document | BodyInit | null): void;\\n    /**\\n     * Combines a header in author request headers.\\n     * Throws an \"InvalidStateError\" DOMException if either state is not opened or the send() flag is set.\\n     * Throws a \"SyntaxError\" DOMException if name is not a header name\\n     * or if value is not a header value.\\n     */\\n    setRequestHeader(name: string, value: string): void;\\n    readonly DONE: number;\\n    readonly HEADERS_RECEIVED: number;\\n    readonly LOADING: number;\\n    readonly OPENED: number;\\n    readonly UNSENT: number;\\n    addEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLHttpRequest: {\\n    prototype: XMLHttpRequest;\\n    new(): XMLHttpRequest;\\n    readonly DONE: number;\\n    readonly HEADERS_RECEIVED: number;\\n    readonly LOADING: number;\\n    readonly OPENED: number;\\n    readonly UNSENT: number;\\n};\\n\\ninterface XMLHttpRequestEventTargetEventMap {\\n    \"abort\": ProgressEvent;\\n    \"error\": ProgressEvent;\\n    \"load\": ProgressEvent;\\n    \"loadend\": ProgressEvent;\\n    \"loadstart\": ProgressEvent;\\n    \"progress\": ProgressEvent;\\n    \"timeout\": ProgressEvent;\\n}\\n\\ninterface XMLHttpRequestEventTarget extends EventTarget {\\n    onabort: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onerror: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onload: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onloadend: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onloadstart: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    onprogress: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    ontimeout: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLHttpRequestEventTarget: {\\n    prototype: XMLHttpRequestEventTarget;\\n    new(): XMLHttpRequestEventTarget;\\n};\\n\\ninterface XMLHttpRequestUpload extends XMLHttpRequestEventTarget {\\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var XMLHttpRequestUpload: {\\n    prototype: XMLHttpRequestUpload;\\n    new(): XMLHttpRequestUpload;\\n};\\n\\ninterface XMLSerializer {\\n    serializeToString(root: Node): string;\\n}\\n\\ndeclare var XMLSerializer: {\\n    prototype: XMLSerializer;\\n    new(): XMLSerializer;\\n};\\n\\ninterface XPathEvaluator {\\n    createExpression(expression: string, resolver: XPathNSResolver): XPathExpression;\\n    createNSResolver(nodeResolver?: Node): XPathNSResolver;\\n    evaluate(expression: string, contextNode: Node, resolver: XPathNSResolver | null, type: number, result: XPathResult | null): XPathResult;\\n}\\n\\ndeclare var XPathEvaluator: {\\n    prototype: XPathEvaluator;\\n    new(): XPathEvaluator;\\n};\\n\\ninterface XPathExpression {\\n    evaluate(contextNode: Node, type: number, result: XPathResult | null): XPathResult;\\n}\\n\\ndeclare var XPathExpression: {\\n    prototype: XPathExpression;\\n    new(): XPathExpression;\\n};\\n\\ninterface XPathNSResolver {\\n    lookupNamespaceURI(prefix: string): string;\\n}\\n\\ndeclare var XPathNSResolver: {\\n    prototype: XPathNSResolver;\\n    new(): XPathNSResolver;\\n};\\n\\ninterface XPathResult {\\n    readonly booleanValue: boolean;\\n    readonly invalidIteratorState: boolean;\\n    readonly numberValue: number;\\n    readonly resultType: number;\\n    readonly singleNodeValue: Node;\\n    readonly snapshotLength: number;\\n    readonly stringValue: string;\\n    iterateNext(): Node;\\n    snapshotItem(index: number): Node;\\n    readonly ANY_TYPE: number;\\n    readonly ANY_UNORDERED_NODE_TYPE: number;\\n    readonly BOOLEAN_TYPE: number;\\n    readonly FIRST_ORDERED_NODE_TYPE: number;\\n    readonly NUMBER_TYPE: number;\\n    readonly ORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly ORDERED_NODE_SNAPSHOT_TYPE: number;\\n    readonly STRING_TYPE: number;\\n    readonly UNORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly UNORDERED_NODE_SNAPSHOT_TYPE: number;\\n}\\n\\ndeclare var XPathResult: {\\n    prototype: XPathResult;\\n    new(): XPathResult;\\n    readonly ANY_TYPE: number;\\n    readonly ANY_UNORDERED_NODE_TYPE: number;\\n    readonly BOOLEAN_TYPE: number;\\n    readonly FIRST_ORDERED_NODE_TYPE: number;\\n    readonly NUMBER_TYPE: number;\\n    readonly ORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly ORDERED_NODE_SNAPSHOT_TYPE: number;\\n    readonly STRING_TYPE: number;\\n    readonly UNORDERED_NODE_ITERATOR_TYPE: number;\\n    readonly UNORDERED_NODE_SNAPSHOT_TYPE: number;\\n};\\n\\ninterface XSLTProcessor {\\n    clearParameters(): void;\\n    getParameter(namespaceURI: string, localName: string): any;\\n    importStylesheet(style: Node): void;\\n    removeParameter(namespaceURI: string, localName: string): void;\\n    reset(): void;\\n    setParameter(namespaceURI: string, localName: string, value: any): void;\\n    transformToDocument(source: Node): Document;\\n    transformToFragment(source: Node, document: Document): DocumentFragment;\\n}\\n\\ndeclare var XSLTProcessor: {\\n    prototype: XSLTProcessor;\\n    new(): XSLTProcessor;\\n};\\n\\ninterface webkitRTCPeerConnection extends RTCPeerConnection {\\n    addEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: webkitRTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\n    removeEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: webkitRTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\n}\\n\\ndeclare var webkitRTCPeerConnection: {\\n    prototype: webkitRTCPeerConnection;\\n    new(configuration: RTCConfiguration): webkitRTCPeerConnection;\\n};\\n\\ndeclare type EventListenerOrEventListenerObject = EventListener | EventListenerObject;\\n\\ninterface BlobCallback {\\n    (blob: Blob | null): void;\\n}\\n\\ninterface DecodeErrorCallback {\\n    (error: DOMException): void;\\n}\\n\\ninterface DecodeSuccessCallback {\\n    (decodedData: AudioBuffer): void;\\n}\\n\\ninterface ErrorEventHandler {\\n    (event: Event | string, source?: string, fileno?: number, columnNumber?: number, error?: Error): void;\\n}\\n\\ninterface EventHandlerNonNull {\\n    (event: Event): any;\\n}\\n\\ninterface ForEachCallback {\\n    (keyId: Int8Array | Int16Array | Int32Array | Uint8Array | Uint16Array | Uint32Array | Uint8ClampedArray | Float32Array | Float64Array | DataView | ArrayBuffer | null, status: MediaKeyStatus): void;\\n}\\n\\ninterface FrameRequestCallback {\\n    (time: number): void;\\n}\\n\\ninterface FunctionStringCallback {\\n    (data: string): void;\\n}\\n\\ninterface IntersectionObserverCallback {\\n    (entries: IntersectionObserverEntry[], observer: IntersectionObserver): void;\\n}\\n\\ninterface MSLaunchUriCallback {\\n    (): void;\\n}\\n\\ninterface MutationCallback {\\n    (mutations: MutationRecord[], observer: MutationObserver): void;\\n}\\n\\ninterface NavigatorUserMediaErrorCallback {\\n    (error: MediaStreamError): void;\\n}\\n\\ninterface NavigatorUserMediaSuccessCallback {\\n    (stream: MediaStream): void;\\n}\\n\\ninterface NotificationPermissionCallback {\\n    (permission: NotificationPermission): void;\\n}\\n\\ninterface OnBeforeUnloadEventHandlerNonNull {\\n    (event: Event): string | null;\\n}\\n\\ninterface OnErrorEventHandlerNonNull {\\n    (event: Event | string, source?: string, lineno?: number, colno?: number, error?: any): any;\\n}\\n\\ninterface PerformanceObserverCallback {\\n    (entries: PerformanceObserverEntryList, observer: PerformanceObserver): void;\\n}\\n\\ninterface PositionCallback {\\n    (position: Position): void;\\n}\\n\\ninterface PositionErrorCallback {\\n    (positionError: PositionError): void;\\n}\\n\\ninterface RTCPeerConnectionErrorCallback {\\n    (error: DOMException): void;\\n}\\n\\ninterface RTCSessionDescriptionCallback {\\n    (description: RTCSessionDescriptionInit): void;\\n}\\n\\ninterface RTCStatsCallback {\\n    (report: RTCStatsReport): void;\\n}\\n\\ninterface VoidFunction {\\n    (): void;\\n}\\n\\ninterface WritableStreamChunkCallback {\\n    (chunk: any, controller: WritableStreamDefaultController): void;\\n}\\n\\ninterface WritableStreamDefaultControllerCallback {\\n    (controller: WritableStreamDefaultController): void;\\n}\\n\\ninterface WritableStreamErrorCallback {\\n    (reason: string): void;\\n}\\n\\ninterface HTMLElementTagNameMap {\\n    \"a\": HTMLAnchorElement;\\n    \"abbr\": HTMLElement;\\n    \"address\": HTMLElement;\\n    \"applet\": HTMLAppletElement;\\n    \"area\": HTMLAreaElement;\\n    \"article\": HTMLElement;\\n    \"aside\": HTMLElement;\\n    \"audio\": HTMLAudioElement;\\n    \"b\": HTMLElement;\\n    \"base\": HTMLBaseElement;\\n    \"basefont\": HTMLBaseFontElement;\\n    \"bdo\": HTMLElement;\\n    \"blockquote\": HTMLQuoteElement;\\n    \"body\": HTMLBodyElement;\\n    \"br\": HTMLBRElement;\\n    \"button\": HTMLButtonElement;\\n    \"canvas\": HTMLCanvasElement;\\n    \"caption\": HTMLTableCaptionElement;\\n    \"cite\": HTMLElement;\\n    \"code\": HTMLElement;\\n    \"col\": HTMLTableColElement;\\n    \"colgroup\": HTMLTableColElement;\\n    \"data\": HTMLDataElement;\\n    \"datalist\": HTMLDataListElement;\\n    \"dd\": HTMLElement;\\n    \"del\": HTMLModElement;\\n    \"details\": HTMLDetailsElement;\\n    \"dfn\": HTMLElement;\\n    \"dialog\": HTMLDialogElement;\\n    \"dir\": HTMLDirectoryElement;\\n    \"div\": HTMLDivElement;\\n    \"dl\": HTMLDListElement;\\n    \"dt\": HTMLElement;\\n    \"em\": HTMLElement;\\n    \"embed\": HTMLEmbedElement;\\n    \"fieldset\": HTMLFieldSetElement;\\n    \"figcaption\": HTMLElement;\\n    \"figure\": HTMLElement;\\n    \"font\": HTMLFontElement;\\n    \"footer\": HTMLElement;\\n    \"form\": HTMLFormElement;\\n    \"frame\": HTMLFrameElement;\\n    \"frameset\": HTMLFrameSetElement;\\n    \"h1\": HTMLHeadingElement;\\n    \"h2\": HTMLHeadingElement;\\n    \"h3\": HTMLHeadingElement;\\n    \"h4\": HTMLHeadingElement;\\n    \"h5\": HTMLHeadingElement;\\n    \"h6\": HTMLHeadingElement;\\n    \"head\": HTMLHeadElement;\\n    \"header\": HTMLElement;\\n    \"hgroup\": HTMLElement;\\n    \"hr\": HTMLHRElement;\\n    \"html\": HTMLHtmlElement;\\n    \"i\": HTMLElement;\\n    \"iframe\": HTMLIFrameElement;\\n    \"img\": HTMLImageElement;\\n    \"input\": HTMLInputElement;\\n    \"ins\": HTMLModElement;\\n    \"kbd\": HTMLElement;\\n    \"label\": HTMLLabelElement;\\n    \"legend\": HTMLLegendElement;\\n    \"li\": HTMLLIElement;\\n    \"link\": HTMLLinkElement;\\n    \"map\": HTMLMapElement;\\n    \"mark\": HTMLElement;\\n    \"marquee\": HTMLMarqueeElement;\\n    \"menu\": HTMLMenuElement;\\n    \"meta\": HTMLMetaElement;\\n    \"meter\": HTMLMeterElement;\\n    \"nav\": HTMLElement;\\n    \"noscript\": HTMLElement;\\n    \"object\": HTMLObjectElement;\\n    \"ol\": HTMLOListElement;\\n    \"optgroup\": HTMLOptGroupElement;\\n    \"option\": HTMLOptionElement;\\n    \"output\": HTMLOutputElement;\\n    \"p\": HTMLParagraphElement;\\n    \"param\": HTMLParamElement;\\n    \"picture\": HTMLPictureElement;\\n    \"pre\": HTMLPreElement;\\n    \"progress\": HTMLProgressElement;\\n    \"q\": HTMLQuoteElement;\\n    \"rt\": HTMLElement;\\n    \"ruby\": HTMLElement;\\n    \"s\": HTMLElement;\\n    \"samp\": HTMLElement;\\n    \"script\": HTMLScriptElement;\\n    \"section\": HTMLElement;\\n    \"select\": HTMLSelectElement;\\n    \"slot\": HTMLSlotElement;\\n    \"small\": HTMLElement;\\n    \"source\": HTMLSourceElement;\\n    \"span\": HTMLSpanElement;\\n    \"strong\": HTMLElement;\\n    \"style\": HTMLStyleElement;\\n    \"sub\": HTMLElement;\\n    \"sup\": HTMLElement;\\n    \"table\": HTMLTableElement;\\n    \"tbody\": HTMLTableSectionElement;\\n    \"td\": HTMLTableDataCellElement;\\n    \"template\": HTMLTemplateElement;\\n    \"textarea\": HTMLTextAreaElement;\\n    \"tfoot\": HTMLTableSectionElement;\\n    \"th\": HTMLTableHeaderCellElement;\\n    \"thead\": HTMLTableSectionElement;\\n    \"time\": HTMLTimeElement;\\n    \"title\": HTMLTitleElement;\\n    \"tr\": HTMLTableRowElement;\\n    \"track\": HTMLTrackElement;\\n    \"u\": HTMLElement;\\n    \"ul\": HTMLUListElement;\\n    \"var\": HTMLElement;\\n    \"video\": HTMLVideoElement;\\n    \"wbr\": HTMLElement;\\n}\\n\\ninterface HTMLElementDeprecatedTagNameMap {\\n    \"listing\": HTMLPreElement;\\n    \"xmp\": HTMLPreElement;\\n}\\n\\ninterface SVGElementTagNameMap {\\n    \"circle\": SVGCircleElement;\\n    \"clipPath\": SVGClipPathElement;\\n    \"defs\": SVGDefsElement;\\n    \"desc\": SVGDescElement;\\n    \"ellipse\": SVGEllipseElement;\\n    \"feBlend\": SVGFEBlendElement;\\n    \"feColorMatrix\": SVGFEColorMatrixElement;\\n    \"feComponentTransfer\": SVGFEComponentTransferElement;\\n    \"feComposite\": SVGFECompositeElement;\\n    \"feConvolveMatrix\": SVGFEConvolveMatrixElement;\\n    \"feDiffuseLighting\": SVGFEDiffuseLightingElement;\\n    \"feDisplacementMap\": SVGFEDisplacementMapElement;\\n    \"feDistantLight\": SVGFEDistantLightElement;\\n    \"feFlood\": SVGFEFloodElement;\\n    \"feFuncA\": SVGFEFuncAElement;\\n    \"feFuncB\": SVGFEFuncBElement;\\n    \"feFuncG\": SVGFEFuncGElement;\\n    \"feFuncR\": SVGFEFuncRElement;\\n    \"feGaussianBlur\": SVGFEGaussianBlurElement;\\n    \"feImage\": SVGFEImageElement;\\n    \"feMerge\": SVGFEMergeElement;\\n    \"feMergeNode\": SVGFEMergeNodeElement;\\n    \"feMorphology\": SVGFEMorphologyElement;\\n    \"feOffset\": SVGFEOffsetElement;\\n    \"fePointLight\": SVGFEPointLightElement;\\n    \"feSpecularLighting\": SVGFESpecularLightingElement;\\n    \"feSpotLight\": SVGFESpotLightElement;\\n    \"feTile\": SVGFETileElement;\\n    \"feTurbulence\": SVGFETurbulenceElement;\\n    \"filter\": SVGFilterElement;\\n    \"foreignObject\": SVGForeignObjectElement;\\n    \"g\": SVGGElement;\\n    \"image\": SVGImageElement;\\n    \"line\": SVGLineElement;\\n    \"linearGradient\": SVGLinearGradientElement;\\n    \"marker\": SVGMarkerElement;\\n    \"mask\": SVGMaskElement;\\n    \"metadata\": SVGMetadataElement;\\n    \"path\": SVGPathElement;\\n    \"pattern\": SVGPatternElement;\\n    \"polygon\": SVGPolygonElement;\\n    \"polyline\": SVGPolylineElement;\\n    \"radialGradient\": SVGRadialGradientElement;\\n    \"rect\": SVGRectElement;\\n    \"stop\": SVGStopElement;\\n    \"svg\": SVGSVGElement;\\n    \"switch\": SVGSwitchElement;\\n    \"symbol\": SVGSymbolElement;\\n    \"text\": SVGTextElement;\\n    \"textPath\": SVGTextPathElement;\\n    \"tspan\": SVGTSpanElement;\\n    \"use\": SVGUseElement;\\n    \"view\": SVGViewElement;\\n}\\n\\n/** @deprecated Directly use HTMLElementTagNameMap or SVGElementTagNameMap as appropriate, instead. */\\ninterface ElementTagNameMap extends HTMLElementTagNameMap, SVGElementTagNameMap { }\\n\\ndeclare var Audio: {\\n    new(src?: string): HTMLAudioElement;\\n};\\ndeclare var Image: {\\n    new(width?: number, height?: number): HTMLImageElement;\\n};\\ndeclare var Option: {\\n    new(text?: string, value?: string, defaultSelected?: boolean, selected?: boolean): HTMLOptionElement;\\n};\\ndeclare var Blob: typeof Blob;\\ndeclare var URL: typeof URL;\\ndeclare var URLSearchParams: typeof URLSearchParams;\\ndeclare var applicationCache: ApplicationCache;\\ndeclare var caches: CacheStorage;\\ndeclare var clientInformation: Navigator;\\ndeclare var closed: boolean;\\ndeclare var crypto: Crypto;\\ndeclare var customElements: CustomElementRegistry;\\ndeclare var defaultStatus: string;\\ndeclare var devicePixelRatio: number;\\ndeclare var doNotTrack: string;\\ndeclare var document: Document;\\ndeclare var event: Event | undefined;\\n/** @deprecated */\\ndeclare var external: External;\\ndeclare var frameElement: Element;\\ndeclare var frames: Window;\\ndeclare var history: History;\\ndeclare var innerHeight: number;\\ndeclare var innerWidth: number;\\ndeclare var isSecureContext: boolean;\\ndeclare var length: number;\\ndeclare var location: Location;\\ndeclare var locationbar: BarProp;\\ndeclare var menubar: BarProp;\\ndeclare var msContentScript: ExtensionScriptApis;\\ndeclare const name: never;\\ndeclare var navigator: Navigator;\\ndeclare var offscreenBuffering: string | boolean;\\ndeclare var oncompassneedscalibration: ((this: Window, ev: Event) => any) | null;\\ndeclare var ondevicelight: ((this: Window, ev: DeviceLightEvent) => any) | null;\\ndeclare var ondevicemotion: ((this: Window, ev: DeviceMotionEvent) => any) | null;\\ndeclare var ondeviceorientation: ((this: Window, ev: DeviceOrientationEvent) => any) | null;\\ndeclare var onmousewheel: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturechange: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturedoubletap: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgestureend: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturehold: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturestart: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsgesturetap: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmsinertiastart: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointercancel: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerdown: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerenter: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerleave: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointermove: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerout: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerover: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmspointerup: ((this: Window, ev: Event) => any) | null;\\n/** @deprecated */\\ndeclare var onorientationchange: ((this: Window, ev: Event) => any) | null;\\ndeclare var onreadystatechange: ((this: Window, ev: ProgressEvent) => any) | null;\\ndeclare var onvrdisplayactivate: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplayblur: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplayconnect: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaydeactivate: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaydisconnect: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplayfocus: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaypointerrestricted: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaypointerunrestricted: ((this: Window, ev: Event) => any) | null;\\ndeclare var onvrdisplaypresentchange: ((this: Window, ev: Event) => any) | null;\\ndeclare var opener: any;\\n/** @deprecated */\\ndeclare var orientation: string | number;\\ndeclare var outerHeight: number;\\ndeclare var outerWidth: number;\\ndeclare var pageXOffset: number;\\ndeclare var pageYOffset: number;\\ndeclare var parent: Window;\\ndeclare var performance: Performance;\\ndeclare var personalbar: BarProp;\\ndeclare var screen: Screen;\\ndeclare var screenLeft: number;\\ndeclare var screenTop: number;\\ndeclare var screenX: number;\\ndeclare var screenY: number;\\ndeclare var scrollX: number;\\ndeclare var scrollY: number;\\ndeclare var scrollbars: BarProp;\\ndeclare var self: Window;\\ndeclare var speechSynthesis: SpeechSynthesis;\\ndeclare var status: string;\\ndeclare var statusbar: BarProp;\\ndeclare var styleMedia: StyleMedia;\\ndeclare var toolbar: BarProp;\\ndeclare var top: Window;\\ndeclare var window: Window;\\ndeclare function alert(message?: any): void;\\ndeclare function blur(): void;\\ndeclare function cancelAnimationFrame(handle: number): void;\\n/** @deprecated */\\ndeclare function captureEvents(): void;\\ndeclare function close(): void;\\ndeclare function confirm(message?: string): boolean;\\ndeclare function departFocus(navigationReason: NavigationReason, origin: FocusNavigationOrigin): void;\\ndeclare function focus(): void;\\ndeclare function getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\\ndeclare function getMatchedCSSRules(elt: Element, pseudoElt?: string | null): CSSRuleList;\\ndeclare function getSelection(): Selection;\\ndeclare function matchMedia(query: string): MediaQueryList;\\ndeclare function moveBy(x: number, y: number): void;\\ndeclare function moveTo(x: number, y: number): void;\\ndeclare function msWriteProfilerMark(profilerMarkName: string): void;\\ndeclare function open(url?: string, target?: string, features?: string, replace?: boolean): Window | null;\\ndeclare function postMessage(message: any, targetOrigin: string, transfer?: Transferable[]): void;\\ndeclare function print(): void;\\ndeclare function prompt(message?: string, _default?: string): string | null;\\n/** @deprecated */\\ndeclare function releaseEvents(): void;\\ndeclare function requestAnimationFrame(callback: FrameRequestCallback): number;\\ndeclare function resizeBy(x: number, y: number): void;\\ndeclare function resizeTo(x: number, y: number): void;\\ndeclare function scroll(options?: ScrollToOptions): void;\\ndeclare function scroll(x: number, y: number): void;\\ndeclare function scrollBy(options?: ScrollToOptions): void;\\ndeclare function scrollBy(x: number, y: number): void;\\ndeclare function scrollTo(options?: ScrollToOptions): void;\\ndeclare function scrollTo(x: number, y: number): void;\\ndeclare function stop(): void;\\ndeclare function webkitCancelAnimationFrame(handle: number): void;\\ndeclare function webkitConvertPointFromNodeToPage(node: Node, pt: WebKitPoint): WebKitPoint;\\ndeclare function webkitConvertPointFromPageToNode(node: Node, pt: WebKitPoint): WebKitPoint;\\ndeclare function webkitRequestAnimationFrame(callback: FrameRequestCallback): number;\\ndeclare function toString(): string;\\n/**\\n * Dispatches a synthetic event event to target and returns true\\n * if either event\\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\\n */\\ndeclare function dispatchEvent(event: Event): boolean;\\ndeclare var sessionStorage: Storage;\\ndeclare var localStorage: Storage;\\ndeclare var console: Console;\\n/**\\n * Fires when the user aborts the download.\\n * @param ev The event.\\n */\\ndeclare var onabort: ((this: Window, ev: UIEvent) => any) | null;\\ndeclare var onanimationcancel: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onanimationend: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onanimationiteration: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onanimationstart: ((this: Window, ev: AnimationEvent) => any) | null;\\ndeclare var onauxclick: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the object loses the input focus.\\n * @param ev The focus event.\\n */\\ndeclare var onblur: ((this: Window, ev: FocusEvent) => any) | null;\\ndeclare var oncancel: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when playback is possible, but would require further buffering.\\n * @param ev The event.\\n */\\ndeclare var oncanplay: ((this: Window, ev: Event) => any) | null;\\ndeclare var oncanplaythrough: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the contents of the object or selection have changed.\\n * @param ev The event.\\n */\\ndeclare var onchange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user clicks the left mouse button on the object\\n * @param ev The mouse event.\\n */\\ndeclare var onclick: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var onclose: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user clicks the right mouse button in the client area, opening the context menu.\\n * @param ev The mouse event.\\n */\\ndeclare var oncontextmenu: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var oncuechange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user double-clicks the object.\\n * @param ev The mouse event.\\n */\\ndeclare var ondblclick: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires on the source object continuously during a drag operation.\\n * @param ev The event.\\n */\\ndeclare var ondrag: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the source object when the user releases the mouse at the close of a drag operation.\\n * @param ev The event.\\n */\\ndeclare var ondragend: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the target element when the user drags the object to a valid drop target.\\n * @param ev The drag event.\\n */\\ndeclare var ondragenter: ((this: Window, ev: DragEvent) => any) | null;\\ndeclare var ondragexit: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires on the target object when the user moves the mouse out of a valid drop target during a drag operation.\\n * @param ev The drag event.\\n */\\ndeclare var ondragleave: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the target element continuously while the user drags the object over a valid drop target.\\n * @param ev The event.\\n */\\ndeclare var ondragover: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Fires on the source object when the user starts to drag a text selection or selected object.\\n * @param ev The event.\\n */\\ndeclare var ondragstart: ((this: Window, ev: DragEvent) => any) | null;\\ndeclare var ondrop: ((this: Window, ev: DragEvent) => any) | null;\\n/**\\n * Occurs when the duration attribute is updated.\\n * @param ev The event.\\n */\\ndeclare var ondurationchange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the media element is reset to its initial state.\\n * @param ev The event.\\n */\\ndeclare var onemptied: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the end of playback is reached.\\n * @param ev The event\\n */\\ndeclare var onended: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when an error occurs during object loading.\\n * @param ev The event.\\n */\\ndeclare var onerror: ErrorEventHandler;\\n/**\\n * Fires when the object receives focus.\\n * @param ev The event.\\n */\\ndeclare var onfocus: ((this: Window, ev: FocusEvent) => any) | null;\\ndeclare var ongotpointercapture: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var oninput: ((this: Window, ev: Event) => any) | null;\\ndeclare var oninvalid: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user presses a key.\\n * @param ev The keyboard event\\n */\\ndeclare var onkeydown: ((this: Window, ev: KeyboardEvent) => any) | null;\\n/**\\n * Fires when the user presses an alphanumeric key.\\n * @param ev The event.\\n */\\ndeclare var onkeypress: ((this: Window, ev: KeyboardEvent) => any) | null;\\n/**\\n * Fires when the user releases a key.\\n * @param ev The keyboard event\\n */\\ndeclare var onkeyup: ((this: Window, ev: KeyboardEvent) => any) | null;\\n/**\\n * Fires immediately after the browser loads the object.\\n * @param ev The event.\\n */\\ndeclare var onload: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when media data is loaded at the current playback position.\\n * @param ev The event.\\n */\\ndeclare var onloadeddata: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the duration and dimensions of the media have been determined.\\n * @param ev The event.\\n */\\ndeclare var onloadedmetadata: ((this: Window, ev: Event) => any) | null;\\ndeclare var onloadend: ((this: Window, ev: ProgressEvent) => any) | null;\\n/**\\n * Occurs when Internet Explorer begins looking for media data.\\n * @param ev The event.\\n */\\ndeclare var onloadstart: ((this: Window, ev: Event) => any) | null;\\ndeclare var onlostpointercapture: ((this: Window, ev: PointerEvent) => any) | null;\\n/**\\n * Fires when the user clicks the object with either mouse button.\\n * @param ev The mouse event.\\n */\\ndeclare var onmousedown: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var onmouseenter: ((this: Window, ev: MouseEvent) => any) | null;\\ndeclare var onmouseleave: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user moves the mouse over the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmousemove: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user moves the mouse pointer outside the boundaries of the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmouseout: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user moves the mouse pointer into the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmouseover: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Fires when the user releases a mouse button while the mouse is over the object.\\n * @param ev The mouse event.\\n */\\ndeclare var onmouseup: ((this: Window, ev: MouseEvent) => any) | null;\\n/**\\n * Occurs when playback is paused.\\n * @param ev The event.\\n */\\ndeclare var onpause: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the play method is requested.\\n * @param ev The event.\\n */\\ndeclare var onplay: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the audio or video has started playing.\\n * @param ev The event.\\n */\\ndeclare var onplaying: ((this: Window, ev: Event) => any) | null;\\ndeclare var onpointercancel: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerdown: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerenter: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerleave: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointermove: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerout: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerover: ((this: Window, ev: PointerEvent) => any) | null;\\ndeclare var onpointerup: ((this: Window, ev: PointerEvent) => any) | null;\\n/**\\n * Occurs to indicate progress while downloading media data.\\n * @param ev The event.\\n */\\ndeclare var onprogress: ((this: Window, ev: ProgressEvent) => any) | null;\\n/**\\n * Occurs when the playback rate is increased or decreased.\\n * @param ev The event.\\n */\\ndeclare var onratechange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the user resets a form.\\n * @param ev The event.\\n */\\ndeclare var onreset: ((this: Window, ev: Event) => any) | null;\\ndeclare var onresize: ((this: Window, ev: UIEvent) => any) | null;\\n/**\\n * Fires when the user repositions the scroll box in the scroll bar on the object.\\n * @param ev The event.\\n */\\ndeclare var onscroll: ((this: Window, ev: UIEvent) => any) | null;\\ndeclare var onsecuritypolicyviolation: ((this: Window, ev: SecurityPolicyViolationEvent) => any) | null;\\n/**\\n * Occurs when the seek operation ends.\\n * @param ev The event.\\n */\\ndeclare var onseeked: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when the current playback position is moved.\\n * @param ev The event.\\n */\\ndeclare var onseeking: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Fires when the current selection changes.\\n * @param ev The event.\\n */\\ndeclare var onselect: ((this: Window, ev: UIEvent) => any) | null;\\n/**\\n * Occurs when the download has stopped.\\n * @param ev The event.\\n */\\ndeclare var onstalled: ((this: Window, ev: Event) => any) | null;\\ndeclare var onsubmit: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs if the load operation has been intentionally halted.\\n * @param ev The event.\\n */\\ndeclare var onsuspend: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs to indicate the current playback position.\\n * @param ev The event.\\n */\\ndeclare var ontimeupdate: ((this: Window, ev: Event) => any) | null;\\ndeclare var ontoggle: ((this: Window, ev: Event) => any) | null;\\ndeclare var ontouchcancel: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontouchend: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontouchmove: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontouchstart: ((this: Window, ev: TouchEvent) => any) | null;\\ndeclare var ontransitioncancel: ((this: Window, ev: TransitionEvent) => any) | null;\\ndeclare var ontransitionend: ((this: Window, ev: TransitionEvent) => any) | null;\\ndeclare var ontransitionrun: ((this: Window, ev: TransitionEvent) => any) | null;\\ndeclare var ontransitionstart: ((this: Window, ev: TransitionEvent) => any) | null;\\n/**\\n * Occurs when the volume is changed, or playback is muted or unmuted.\\n * @param ev The event.\\n */\\ndeclare var onvolumechange: ((this: Window, ev: Event) => any) | null;\\n/**\\n * Occurs when playback stops because the next frame of a video resource is not available.\\n * @param ev The event.\\n */\\ndeclare var onwaiting: ((this: Window, ev: Event) => any) | null;\\ndeclare var onwheel: ((this: Window, ev: WheelEvent) => any) | null;\\ndeclare var indexedDB: IDBFactory;\\ndeclare function atob(encodedString: string): string;\\ndeclare function btoa(rawString: string): string;\\ndeclare function fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\ndeclare var caches: CacheStorage;\\ndeclare var crypto: Crypto;\\ndeclare var indexedDB: IDBFactory;\\ndeclare var origin: string;\\ndeclare var performance: Performance;\\ndeclare function atob(data: string): string;\\ndeclare function btoa(data: string): string;\\ndeclare function clearInterval(handle?: number): void;\\ndeclare function clearTimeout(handle?: number): void;\\ndeclare function createImageBitmap(image: ImageBitmapSource): Promise<ImageBitmap>;\\ndeclare function createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number): Promise<ImageBitmap>;\\ndeclare function fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\\ndeclare function queueMicrotask(callback: Function): void;\\ndeclare function setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\ndeclare function setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\\ndeclare var sessionStorage: Storage;\\ndeclare var localStorage: Storage;\\ndeclare var onafterprint: ((this: Window, ev: Event) => any) | null;\\ndeclare var onbeforeprint: ((this: Window, ev: Event) => any) | null;\\ndeclare var onbeforeunload: ((this: Window, ev: BeforeUnloadEvent) => any) | null;\\ndeclare var onhashchange: ((this: Window, ev: HashChangeEvent) => any) | null;\\ndeclare var onlanguagechange: ((this: Window, ev: Event) => any) | null;\\ndeclare var onmessage: ((this: Window, ev: MessageEvent) => any) | null;\\ndeclare var onmessageerror: ((this: Window, ev: MessageEvent) => any) | null;\\ndeclare var onoffline: ((this: Window, ev: Event) => any) | null;\\ndeclare var ononline: ((this: Window, ev: Event) => any) | null;\\ndeclare var onpagehide: ((this: Window, ev: PageTransitionEvent) => any) | null;\\ndeclare var onpageshow: ((this: Window, ev: PageTransitionEvent) => any) | null;\\ndeclare var onpopstate: ((this: Window, ev: PopStateEvent) => any) | null;\\ndeclare var onrejectionhandled: ((this: Window, ev: Event) => any) | null;\\ndeclare var onstorage: ((this: Window, ev: StorageEvent) => any) | null;\\ndeclare var onunhandledrejection: ((this: Window, ev: PromiseRejectionEvent) => any) | null;\\ndeclare var onunload: ((this: Window, ev: Event) => any) | null;\\ndeclare function addEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\\ndeclare function addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\\ndeclare function removeEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\\ndeclare function removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\\ntype BlobPart = BufferSource | Blob | string;\\ntype HeadersInit = Headers | string[][] | Record<string, string>;\\ntype BodyInit = Blob | BufferSource | FormData | URLSearchParams | ReadableStream | string;\\ntype RequestInfo = Request | string;\\ntype DOMHighResTimeStamp = number;\\ntype RenderingContext = CanvasRenderingContext2D | ImageBitmapRenderingContext | WebGLRenderingContext;\\ntype HTMLOrSVGImageElement = HTMLImageElement | SVGImageElement;\\ntype CanvasImageSource = HTMLOrSVGImageElement | HTMLVideoElement | HTMLCanvasElement | ImageBitmap;\\ntype MessageEventSource = WindowProxy | MessagePort | ServiceWorker;\\ntype HTMLOrSVGScriptElement = HTMLScriptElement | SVGScriptElement;\\ntype ImageBitmapSource = CanvasImageSource | Blob | ImageData;\\ntype OnErrorEventHandler = OnErrorEventHandlerNonNull | null;\\ntype OnBeforeUnloadEventHandler = OnBeforeUnloadEventHandlerNonNull | null;\\ntype TimerHandler = string | Function;\\ntype PerformanceEntryList = PerformanceEntry[];\\ntype VibratePattern = number | number[];\\ntype AlgorithmIdentifier = string | Algorithm;\\ntype HashAlgorithmIdentifier = AlgorithmIdentifier;\\ntype BigInteger = Uint8Array;\\ntype NamedCurve = string;\\ntype GLenum = number;\\ntype GLboolean = boolean;\\ntype GLbitfield = number;\\ntype GLint = number;\\ntype GLsizei = number;\\ntype GLintptr = number;\\ntype GLsizeiptr = number;\\ntype GLuint = number;\\ntype GLfloat = number;\\ntype GLclampf = number;\\ntype TexImageSource = ImageBitmap | ImageData | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement;\\ntype Float32List = Float32Array | GLfloat[];\\ntype Int32List = Int32Array | GLint[];\\ntype BufferSource = ArrayBufferView | ArrayBuffer;\\ntype DOMTimeStamp = number;\\ntype LineAndPositionSetting = number | AutoKeyword;\\ntype FormDataEntryValue = File | string;\\ntype InsertPosition = \"beforebegin\" | \"afterbegin\" | \"beforeend\" | \"afterend\";\\ntype IDBValidKey = number | string | Date | BufferSource | IDBArrayKey;\\ntype MutationRecordType = \"attributes\" | \"characterData\" | \"childList\";\\ntype ConstrainBoolean = boolean | ConstrainBooleanParameters;\\ntype ConstrainDOMString = string | string[] | ConstrainDOMStringParameters;\\ntype ConstrainDouble = number | ConstrainDoubleRange;\\ntype ConstrainLong = number | ConstrainLongRange;\\ntype IDBKeyPath = string;\\ntype Transferable = ArrayBuffer | MessagePort | ImageBitmap;\\ntype RTCIceGatherCandidate = RTCIceCandidateDictionary | RTCIceCandidateComplete;\\ntype RTCTransport = RTCDtlsTransport | RTCSrtpSdesTransport;\\ntype MouseWheelEvent = WheelEvent;\\ntype WindowProxy = Window;\\ntype AlignSetting = \"start\" | \"center\" | \"end\" | \"left\" | \"right\";\\ntype AnimationPlayState = \"idle\" | \"running\" | \"paused\" | \"finished\";\\ntype AppendMode = \"segments\" | \"sequence\";\\ntype AudioContextLatencyCategory = \"balanced\" | \"interactive\" | \"playback\";\\ntype AudioContextState = \"suspended\" | \"running\" | \"closed\";\\ntype AutoKeyword = \"auto\";\\ntype AutomationRate = \"a-rate\" | \"k-rate\";\\ntype BinaryType = \"blob\" | \"arraybuffer\";\\ntype BiquadFilterType = \"lowpass\" | \"highpass\" | \"bandpass\" | \"lowshelf\" | \"highshelf\" | \"peaking\" | \"notch\" | \"allpass\";\\ntype CanPlayTypeResult = \"\" | \"maybe\" | \"probably\";\\ntype CanvasDirection = \"ltr\" | \"rtl\" | \"inherit\";\\ntype CanvasFillRule = \"nonzero\" | \"evenodd\";\\ntype CanvasLineCap = \"butt\" | \"round\" | \"square\";\\ntype CanvasLineJoin = \"round\" | \"bevel\" | \"miter\";\\ntype CanvasTextAlign = \"start\" | \"end\" | \"left\" | \"right\" | \"center\";\\ntype CanvasTextBaseline = \"top\" | \"hanging\" | \"middle\" | \"alphabetic\" | \"ideographic\" | \"bottom\";\\ntype ChannelCountMode = \"max\" | \"clamped-max\" | \"explicit\";\\ntype ChannelInterpretation = \"speakers\" | \"discrete\";\\ntype ClientTypes = \"window\" | \"worker\" | \"sharedworker\" | \"all\";\\ntype CompositeOperation = \"replace\" | \"add\" | \"accumulate\";\\ntype DirectionSetting = \"\" | \"rl\" | \"lr\";\\ntype DisplayCaptureSurfaceType = \"monitor\" | \"window\" | \"application\" | \"browser\";\\ntype DistanceModelType = \"linear\" | \"inverse\" | \"exponential\";\\ntype DocumentReadyState = \"loading\" | \"interactive\" | \"complete\";\\ntype EndOfStreamError = \"network\" | \"decode\";\\ntype FillMode = \"none\" | \"forwards\" | \"backwards\" | \"both\" | \"auto\";\\ntype GamepadHand = \"\" | \"left\" | \"right\";\\ntype GamepadHapticActuatorType = \"vibration\";\\ntype GamepadInputEmulationType = \"mouse\" | \"keyboard\" | \"gamepad\";\\ntype GamepadMappingType = \"\" | \"standard\";\\ntype IDBCursorDirection = \"next\" | \"nextunique\" | \"prev\" | \"prevunique\";\\ntype IDBRequestReadyState = \"pending\" | \"done\";\\ntype IDBTransactionMode = \"readonly\" | \"readwrite\" | \"versionchange\";\\ntype ImageSmoothingQuality = \"low\" | \"medium\" | \"high\";\\ntype IterationCompositeOperation = \"replace\" | \"accumulate\";\\ntype KeyFormat = \"raw\" | \"spki\" | \"pkcs8\" | \"jwk\";\\ntype KeyType = \"public\" | \"private\" | \"secret\";\\ntype KeyUsage = \"encrypt\" | \"decrypt\" | \"sign\" | \"verify\" | \"deriveKey\" | \"deriveBits\" | \"wrapKey\" | \"unwrapKey\";\\ntype LineAlignSetting = \"start\" | \"center\" | \"end\";\\ntype ListeningState = \"inactive\" | \"active\" | \"disambiguation\";\\ntype MSCredentialType = \"FIDO_2_0\";\\ntype MSTransportType = \"Embedded\" | \"USB\" | \"NFC\" | \"BT\";\\ntype MSWebViewPermissionState = \"unknown\" | \"defer\" | \"allow\" | \"deny\";\\ntype MSWebViewPermissionType = \"geolocation\" | \"unlimitedIndexedDBQuota\" | \"media\" | \"pointerlock\" | \"webnotifications\";\\ntype MediaDeviceKind = \"audioinput\" | \"audiooutput\" | \"videoinput\";\\ntype MediaKeyMessageType = \"license-request\" | \"license-renewal\" | \"license-release\" | \"individualization-request\";\\ntype MediaKeySessionType = \"temporary\" | \"persistent-license\" | \"persistent-release-message\";\\ntype MediaKeyStatus = \"usable\" | \"expired\" | \"output-downscaled\" | \"output-not-allowed\" | \"status-pending\" | \"internal-error\";\\ntype MediaKeysRequirement = \"required\" | \"optional\" | \"not-allowed\";\\ntype MediaStreamTrackState = \"live\" | \"ended\";\\ntype NavigationReason = \"up\" | \"down\" | \"left\" | \"right\";\\ntype NavigationType = \"navigate\" | \"reload\" | \"back_forward\" | \"prerender\";\\ntype NotificationDirection = \"auto\" | \"ltr\" | \"rtl\";\\ntype NotificationPermission = \"default\" | \"denied\" | \"granted\";\\ntype OrientationLockType = \"any\" | \"natural\" | \"landscape\" | \"portrait\" | \"portrait-primary\" | \"portrait-secondary\" | \"landscape-primary\" | \"landscape-secondary\";\\ntype OrientationType = \"portrait-primary\" | \"portrait-secondary\" | \"landscape-primary\" | \"landscape-secondary\";\\ntype OscillatorType = \"sine\" | \"square\" | \"sawtooth\" | \"triangle\" | \"custom\";\\ntype OverSampleType = \"none\" | \"2x\" | \"4x\";\\ntype PanningModelType = \"equalpower\" | \"HRTF\";\\ntype PaymentComplete = \"success\" | \"fail\" | \"unknown\";\\ntype PaymentShippingType = \"shipping\" | \"delivery\" | \"pickup\";\\ntype PlaybackDirection = \"normal\" | \"reverse\" | \"alternate\" | \"alternate-reverse\";\\ntype PositionAlignSetting = \"line-left\" | \"center\" | \"line-right\" | \"auto\";\\ntype PushEncryptionKeyName = \"p256dh\" | \"auth\";\\ntype PushPermissionState = \"denied\" | \"granted\" | \"prompt\";\\ntype RTCBundlePolicy = \"balanced\" | \"max-compat\" | \"max-bundle\";\\ntype RTCDataChannelState = \"connecting\" | \"open\" | \"closing\" | \"closed\";\\ntype RTCDegradationPreference = \"maintain-framerate\" | \"maintain-resolution\" | \"balanced\";\\ntype RTCDtlsRole = \"auto\" | \"client\" | \"server\";\\ntype RTCDtlsTransportState = \"new\" | \"connecting\" | \"connected\" | \"closed\" | \"failed\";\\ntype RTCDtxStatus = \"disabled\" | \"enabled\";\\ntype RTCErrorDetailType = \"data-channel-failure\" | \"dtls-failure\" | \"fingerprint-failure\" | \"idp-bad-script-failure\" | \"idp-execution-failure\" | \"idp-load-failure\" | \"idp-need-login\" | \"idp-timeout\" | \"idp-tls-failure\" | \"idp-token-expired\" | \"idp-token-invalid\" | \"sctp-failure\" | \"sdp-syntax-error\" | \"hardware-encoder-not-available\" | \"hardware-encoder-error\";\\ntype RTCIceCandidateType = \"host\" | \"srflx\" | \"prflx\" | \"relay\";\\ntype RTCIceComponent = \"rtp\" | \"rtcp\";\\ntype RTCIceConnectionState = \"new\" | \"checking\" | \"connected\" | \"completed\" | \"disconnected\" | \"failed\" | \"closed\";\\ntype RTCIceCredentialType = \"password\" | \"oauth\";\\ntype RTCIceGatherPolicy = \"all\" | \"nohost\" | \"relay\";\\ntype RTCIceGathererState = \"new\" | \"gathering\" | \"complete\";\\ntype RTCIceGatheringState = \"new\" | \"gathering\" | \"complete\";\\ntype RTCIceProtocol = \"udp\" | \"tcp\";\\ntype RTCIceRole = \"controlling\" | \"controlled\";\\ntype RTCIceTcpCandidateType = \"active\" | \"passive\" | \"so\";\\ntype RTCIceTransportPolicy = \"relay\" | \"all\";\\ntype RTCIceTransportState = \"new\" | \"checking\" | \"connected\" | \"completed\" | \"disconnected\" | \"failed\" | \"closed\";\\ntype RTCPeerConnectionState = \"new\" | \"connecting\" | \"connected\" | \"disconnected\" | \"failed\" | \"closed\";\\ntype RTCPriorityType = \"very-low\" | \"low\" | \"medium\" | \"high\";\\ntype RTCRtcpMuxPolicy = \"negotiate\" | \"require\";\\ntype RTCRtpTransceiverDirection = \"sendrecv\" | \"sendonly\" | \"recvonly\" | \"inactive\";\\ntype RTCSctpTransportState = \"connecting\" | \"connected\" | \"closed\";\\ntype RTCSdpType = \"offer\" | \"pranswer\" | \"answer\" | \"rollback\";\\ntype RTCSignalingState = \"stable\" | \"have-local-offer\" | \"have-remote-offer\" | \"have-local-pranswer\" | \"have-remote-pranswer\" | \"closed\";\\ntype RTCStatsIceCandidatePairState = \"frozen\" | \"waiting\" | \"inprogress\" | \"failed\" | \"succeeded\" | \"cancelled\";\\ntype RTCStatsIceCandidateType = \"host\" | \"serverreflexive\" | \"peerreflexive\" | \"relayed\";\\ntype RTCStatsType = \"inboundrtp\" | \"outboundrtp\" | \"session\" | \"datachannel\" | \"track\" | \"transport\" | \"candidatepair\" | \"localcandidate\" | \"remotecandidate\";\\ntype ReadyState = \"closed\" | \"open\" | \"ended\";\\ntype ReferrerPolicy = \"\" | \"no-referrer\" | \"no-referrer-when-downgrade\" | \"origin-only\" | \"origin-when-cross-origin\" | \"unsafe-url\";\\ntype RequestCache = \"default\" | \"no-store\" | \"reload\" | \"no-cache\" | \"force-cache\" | \"only-if-cached\";\\ntype RequestCredentials = \"omit\" | \"same-origin\" | \"include\";\\ntype RequestDestination = \"\" | \"audio\" | \"audioworklet\" | \"document\" | \"embed\" | \"font\" | \"image\" | \"manifest\" | \"object\" | \"paintworklet\" | \"report\" | \"script\" | \"sharedworker\" | \"style\" | \"track\" | \"video\" | \"worker\" | \"xslt\";\\ntype RequestMode = \"navigate\" | \"same-origin\" | \"no-cors\" | \"cors\";\\ntype RequestRedirect = \"follow\" | \"error\" | \"manual\";\\ntype ResponseType = \"basic\" | \"cors\" | \"default\" | \"error\" | \"opaque\" | \"opaqueredirect\";\\ntype ScopedCredentialType = \"ScopedCred\";\\ntype ScrollBehavior = \"auto\" | \"instant\" | \"smooth\";\\ntype ScrollLogicalPosition = \"start\" | \"center\" | \"end\" | \"nearest\";\\ntype ScrollRestoration = \"auto\" | \"manual\";\\ntype ScrollSetting = \"\" | \"up\";\\ntype SelectionMode = \"select\" | \"start\" | \"end\" | \"preserve\";\\ntype ServiceWorkerState = \"installing\" | \"installed\" | \"activating\" | \"activated\" | \"redundant\";\\ntype ServiceWorkerUpdateViaCache = \"imports\" | \"all\" | \"none\";\\ntype ShadowRootMode = \"open\" | \"closed\";\\ntype SpeechRecognitionErrorCode = \"no-speech\" | \"aborted\" | \"audio-capture\" | \"network\" | \"not-allowed\" | \"service-not-allowed\" | \"bad-grammar\" | \"language-not-supported\";\\ntype SpeechSynthesisErrorCode = \"canceled\" | \"interrupted\" | \"audio-busy\" | \"audio-hardware\" | \"network\" | \"synthesis-unavailable\" | \"synthesis-failed\" | \"language-unavailable\" | \"voice-unavailable\" | \"text-too-long\" | \"invalid-argument\";\\ntype SupportedType = \"text/html\" | \"text/xml\" | \"application/xml\" | \"application/xhtml+xml\" | \"image/svg+xml\";\\ntype TextTrackKind = \"subtitles\" | \"captions\" | \"descriptions\" | \"chapters\" | \"metadata\";\\ntype TextTrackMode = \"disabled\" | \"hidden\" | \"showing\";\\ntype TouchType = \"direct\" | \"stylus\";\\ntype Transport = \"usb\" | \"nfc\" | \"ble\";\\ntype VRDisplayEventReason = \"mounted\" | \"navigation\" | \"requested\" | \"unmounted\";\\ntype VideoFacingModeEnum = \"user\" | \"environment\" | \"left\" | \"right\";\\ntype VisibilityState = \"hidden\" | \"visible\" | \"prerender\";\\ntype WebGLPowerPreference = \"default\" | \"low-power\" | \"high-performance\";\\ntype WorkerType = \"classic\" | \"module\";\\ntype XMLHttpRequestResponseType = \"\" | \"arraybuffer\" | \"blob\" | \"document\" | \"json\" | \"text\";\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n/////////////////////////////\\n/// DOM Iterable APIs\\n/////////////////////////////\\n\\ninterface AudioParamMap extends ReadonlyMap<string, AudioParam> {\\n}\\n\\ninterface AudioTrackList {\\n    [Symbol.iterator](): IterableIterator<AudioTrack>;\\n}\\n\\ninterface CSSRuleList {\\n    [Symbol.iterator](): IterableIterator<CSSRule>;\\n}\\n\\ninterface CSSStyleDeclaration {\\n    [Symbol.iterator](): IterableIterator<string>;\\n}\\n\\ninterface ClientRectList {\\n    [Symbol.iterator](): IterableIterator<ClientRect>;\\n}\\n\\ninterface DOMRectList {\\n    [Symbol.iterator](): IterableIterator<DOMRect>;\\n}\\n\\ninterface DOMStringList {\\n    [Symbol.iterator](): IterableIterator<string>;\\n}\\n\\ninterface DOMTokenList {\\n    [Symbol.iterator](): IterableIterator<string>;\\n    entries(): IterableIterator<[number, string]>;\\n    keys(): IterableIterator<number>;\\n    values(): IterableIterator<string>;\\n}\\n\\ninterface DataTransferItemList {\\n    [Symbol.iterator](): IterableIterator<DataTransferItem>;\\n}\\n\\ninterface FileList {\\n    [Symbol.iterator](): IterableIterator<File>;\\n}\\n\\ninterface FormData {\\n    [Symbol.iterator](): IterableIterator<[string, FormDataEntryValue]>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the list.\\n     */\\n    entries(): IterableIterator<[string, FormDataEntryValue]>;\\n    /**\\n     * Returns a list of keys in the list.\\n     */\\n    keys(): IterableIterator<string>;\\n    /**\\n     * Returns a list of values in the list.\\n     */\\n    values(): IterableIterator<FormDataEntryValue>;\\n}\\n\\ninterface HTMLAllCollection {\\n    [Symbol.iterator](): IterableIterator<Element>;\\n}\\n\\ninterface HTMLCollectionBase {\\n    [Symbol.iterator](): IterableIterator<Element>;\\n}\\n\\ninterface HTMLCollectionOf<T extends Element> {\\n    [Symbol.iterator](): IterableIterator<T>;\\n}\\n\\ninterface HTMLFormElement {\\n    [Symbol.iterator](): IterableIterator<Element>;\\n}\\n\\ninterface HTMLSelectElement {\\n    [Symbol.iterator](): IterableIterator<Element>;\\n}\\n\\ninterface Headers {\\n    [Symbol.iterator](): IterableIterator<[string, string]>;\\n    /**\\n     * Returns an iterator allowing to go through all key/value pairs contained in this object.\\n     */\\n    entries(): IterableIterator<[string, string]>;\\n    /**\\n     * Returns an iterator allowing to go through all keys of the key/value pairs contained in this object.\\n     */\\n    keys(): IterableIterator<string>;\\n    /**\\n     * Returns an iterator allowing to go through all values of the key/value pairs contained in this object.\\n     */\\n    values(): IterableIterator<string>;\\n}\\n\\ninterface MediaList {\\n    [Symbol.iterator](): IterableIterator<string>;\\n}\\n\\ninterface MimeTypeArray {\\n    [Symbol.iterator](): IterableIterator<Plugin>;\\n}\\n\\ninterface NamedNodeMap {\\n    [Symbol.iterator](): IterableIterator<Attr>;\\n}\\n\\ninterface NodeList {\\n    [Symbol.iterator](): IterableIterator<Node>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the list.\\n     */\\n    entries(): IterableIterator<[number, Node]>;\\n    /**\\n     * Returns an list of keys in the list.\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the list.\\n     */\\n    values(): IterableIterator<Node>;\\n}\\n\\ninterface NodeListOf<TNode extends Node> {\\n    [Symbol.iterator](): IterableIterator<TNode>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the list.\\n     */\\n    entries(): IterableIterator<[number, TNode]>;\\n    /**\\n     * Returns an list of keys in the list.\\n     */\\n    keys(): IterableIterator<number>;\\n    /**\\n     * Returns an list of values in the list.\\n     */\\n    values(): IterableIterator<TNode>;\\n}\\n\\ninterface Plugin {\\n    [Symbol.iterator](): IterableIterator<MimeType>;\\n}\\n\\ninterface PluginArray {\\n    [Symbol.iterator](): IterableIterator<Plugin>;\\n}\\n\\ninterface RTCStatsReport extends ReadonlyMap<string, any> {\\n}\\n\\ninterface SVGLengthList {\\n    [Symbol.iterator](): IterableIterator<SVGLength>;\\n}\\n\\ninterface SVGNumberList {\\n    [Symbol.iterator](): IterableIterator<SVGNumber>;\\n}\\n\\ninterface SVGStringList {\\n    [Symbol.iterator](): IterableIterator<string>;\\n}\\n\\ninterface SourceBufferList {\\n    [Symbol.iterator](): IterableIterator<SourceBuffer>;\\n}\\n\\ninterface SpeechGrammarList {\\n    [Symbol.iterator](): IterableIterator<SpeechGrammar>;\\n}\\n\\ninterface SpeechRecognitionResult {\\n    [Symbol.iterator](): IterableIterator<SpeechRecognitionAlternative>;\\n}\\n\\ninterface SpeechRecognitionResultList {\\n    [Symbol.iterator](): IterableIterator<SpeechRecognitionResult>;\\n}\\n\\ninterface StyleSheetList {\\n    [Symbol.iterator](): IterableIterator<StyleSheet>;\\n}\\n\\ninterface TextTrackCueList {\\n    [Symbol.iterator](): IterableIterator<TextTrackCue>;\\n}\\n\\ninterface TextTrackList {\\n    [Symbol.iterator](): IterableIterator<TextTrack>;\\n}\\n\\ninterface TouchList {\\n    [Symbol.iterator](): IterableIterator<Touch>;\\n}\\n\\ninterface URLSearchParams {\\n    [Symbol.iterator](): IterableIterator<[string, string]>;\\n    /**\\n     * Returns an array of key, value pairs for every entry in the search params.\\n     */\\n    entries(): IterableIterator<[string, string]>;\\n    /**\\n     * Returns a list of keys in the search params.\\n     */\\n    keys(): IterableIterator<string>;\\n    /**\\n     * Returns a list of values in the search params.\\n     */\\n    values(): IterableIterator<string>;\\n}\\n\\ninterface VideoTrackList {\\n    [Symbol.iterator](): IterableIterator<VideoTrack>;\\n}\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n\\n/////////////////////////////\\n/// WorkerGlobalScope APIs\\n/////////////////////////////\\n// These are only available in a Web Worker\\ndeclare function importScripts(...urls: string[]): void;\\n/*! *****************************************************************************\\nCopyright (c) Microsoft Corporation. All rights reserved. \\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\\nthis file except in compliance with the License. You may obtain a copy of the\\nLicense at http://www.apache.org/licenses/LICENSE-2.0  \\n \\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, \\nMERCHANTABLITY OR NON-INFRINGEMENT. \\n \\nSee the Apache Version 2.0 License for specific language governing permissions\\nand limitations under the License.\\n***************************************************************************** */\\n\\n\\n\\n/// <reference no-default-lib=\"true\"/>\\n\\n\\n\\n\\n/////////////////////////////\\n/// Windows Script Host APIS\\n/////////////////////////////\\n\\n\\ninterface ActiveXObject {\\n    new (s: string): any;\\n}\\ndeclare var ActiveXObject: ActiveXObject;\\n\\ninterface ITextWriter {\\n    Write(s: string): void;\\n    WriteLine(s: string): void;\\n    Close(): void;\\n}\\n\\ninterface TextStreamBase {\\n    /**\\n     * The column number of the current character position in an input stream.\\n     */\\n    Column: number;\\n\\n    /**\\n     * The current line number in an input stream.\\n     */\\n    Line: number;\\n\\n    /**\\n     * Closes a text stream.\\n     * It is not necessary to close standard streams; they close automatically when the process ends. If\\n     * you close a standard stream, be aware that any other pointers to that standard stream become invalid.\\n     */\\n    Close(): void;\\n}\\n\\ninterface TextStreamWriter extends TextStreamBase {\\n    /**\\n     * Sends a string to an output stream.\\n     */\\n    Write(s: string): void;\\n\\n    /**\\n     * Sends a specified number of blank lines (newline characters) to an output stream.\\n     */\\n    WriteBlankLines(intLines: number): void;\\n\\n    /**\\n     * Sends a string followed by a newline character to an output stream.\\n     */\\n    WriteLine(s: string): void;\\n}\\n\\ninterface TextStreamReader extends TextStreamBase {\\n    /**\\n     * Returns a specified number of characters from an input stream, starting at the current pointer position.\\n     * Does not return until the ENTER key is pressed.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     */\\n    Read(characters: number): string;\\n\\n    /**\\n     * Returns all characters from an input stream.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     */\\n    ReadAll(): string;\\n\\n    /**\\n     * Returns an entire line from an input stream.\\n     * Although this method extracts the newline character, it does not add it to the returned string.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     */\\n    ReadLine(): string;\\n\\n    /**\\n     * Skips a specified number of characters when reading from an input text stream.\\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\\n     * @param characters Positive number of characters to skip forward. (Backward skipping is not supported.)\\n     */\\n    Skip(characters: number): void;\\n\\n    /**\\n     * Skips the next line when reading from an input text stream.\\n     * Can only be used on a stream in reading mode, not writing or appending mode.\\n     */\\n    SkipLine(): void;\\n\\n    /**\\n     * Indicates whether the stream pointer position is at the end of a line.\\n     */\\n    AtEndOfLine: boolean;\\n\\n    /**\\n     * Indicates whether the stream pointer position is at the end of a stream.\\n     */\\n    AtEndOfStream: boolean;\\n}\\n\\ndeclare var WScript: {\\n    /**\\n     * Outputs text to either a message box (under WScript.exe) or the command console window followed by\\n     * a newline (under CScript.exe).\\n     */\\n    Echo(s: any): void;\\n\\n    /**\\n     * Exposes the write-only error output stream for the current script.\\n     * Can be accessed only while using CScript.exe.\\n     */\\n    StdErr: TextStreamWriter;\\n\\n    /**\\n     * Exposes the write-only output stream for the current script.\\n     * Can be accessed only while using CScript.exe.\\n     */\\n    StdOut: TextStreamWriter;\\n    Arguments: { length: number; Item(n: number): string; };\\n\\n    /**\\n     *  The full path of the currently running script.\\n     */\\n    ScriptFullName: string;\\n\\n    /**\\n     * Forces the script to stop immediately, with an optional exit code.\\n     */\\n    Quit(exitCode?: number): number;\\n\\n    /**\\n     * The Windows Script Host build version number.\\n     */\\n    BuildVersion: number;\\n\\n    /**\\n     * Fully qualified path of the host executable.\\n     */\\n    FullName: string;\\n\\n    /**\\n     * Gets/sets the script mode - interactive(true) or batch(false).\\n     */\\n    Interactive: boolean;\\n\\n    /**\\n     * The name of the host executable (WScript.exe or CScript.exe).\\n     */\\n    Name: string;\\n\\n    /**\\n     * Path of the directory containing the host executable.\\n     */\\n    Path: string;\\n\\n    /**\\n     * The filename of the currently running script.\\n     */\\n    ScriptName: string;\\n\\n    /**\\n     * Exposes the read-only input stream for the current script.\\n     * Can be accessed only while using CScript.exe.\\n     */\\n    StdIn: TextStreamReader;\\n\\n    /**\\n     * Windows Script Host version\\n     */\\n    Version: string;\\n\\n    /**\\n     * Connects a COM object\\'s event sources to functions named with a given prefix, in the form prefix_event.\\n     */\\n    ConnectObject(objEventSource: any, strPrefix: string): void;\\n\\n    /**\\n     * Creates a COM object.\\n     * @param strProgiID\\n     * @param strPrefix Function names in the form prefix_event will be bound to this object\\'s COM events.\\n     */\\n    CreateObject(strProgID: string, strPrefix?: string): any;\\n\\n    /**\\n     * Disconnects a COM object from its event sources.\\n     */\\n    DisconnectObject(obj: any): void;\\n\\n    /**\\n     * Retrieves an existing object with the specified ProgID from memory, or creates a new one from a file.\\n     * @param strPathname Fully qualified path to the file containing the object persisted to disk.\\n     *                       For objects in memory, pass a zero-length string.\\n     * @param strProgID\\n     * @param strPrefix Function names in the form prefix_event will be bound to this object\\'s COM events.\\n     */\\n    GetObject(strPathname: string, strProgID?: string, strPrefix?: string): any;\\n\\n    /**\\n     * Suspends script execution for a specified length of time, then continues execution.\\n     * @param intTime Interval (in milliseconds) to suspend script execution.\\n     */\\n    Sleep(intTime: number): void;\\n};\\n\\n/**\\n * WSH is an alias for WScript under Windows Script Host\\n */\\ndeclare var WSH: typeof WScript;\\n\\n/**\\n * Represents an Automation SAFEARRAY\\n */\\ndeclare class SafeArray<T = any> {\\n    private constructor();\\n    private SafeArray_typekey: SafeArray<T>;\\n}\\n\\n/**\\n * Allows enumerating over a COM collection, which may not have indexed item access.\\n */\\ninterface Enumerator<T = any> {\\n    /**\\n     * Returns true if the current item is the last one in the collection, or the collection is empty,\\n     * or the current item is undefined.\\n     */\\n    atEnd(): boolean;\\n\\n    /**\\n     * Returns the current item in the collection\\n     */\\n    item(): T;\\n\\n    /**\\n     * Resets the current item in the collection to the first item. If there are no items in the collection,\\n     * the current item is set to undefined.\\n     */\\n    moveFirst(): void;\\n\\n    /**\\n     * Moves the current item to the next item in the collection. If the enumerator is at the end of\\n     * the collection or the collection is empty, the current item is set to undefined.\\n     */\\n    moveNext(): void;\\n}\\n\\ninterface EnumeratorConstructor {\\n    new <T = any>(safearray: SafeArray<T>): Enumerator<T>;\\n    new <T = any>(collection: { Item(index: any): T }): Enumerator<T>;\\n    new <T = any>(collection: any): Enumerator<T>;\\n}\\n\\ndeclare var Enumerator: EnumeratorConstructor;\\n\\n/**\\n * Enables reading from a COM safe array, which might have an alternate lower bound, or multiple dimensions.\\n */\\ninterface VBArray<T = any> {\\n    /**\\n     * Returns the number of dimensions (1-based).\\n     */\\n    dimensions(): number;\\n\\n    /**\\n     * Takes an index for each dimension in the array, and returns the item at the corresponding location.\\n     */\\n    getItem(dimension1Index: number, ...dimensionNIndexes: number[]): T;\\n\\n    /**\\n     * Returns the smallest available index for a given dimension.\\n     * @param dimension 1-based dimension (defaults to 1)\\n     */\\n    lbound(dimension?: number): number;\\n\\n    /**\\n     * Returns the largest available index for a given dimension.\\n     * @param dimension 1-based dimension (defaults to 1)\\n     */\\n    ubound(dimension?: number): number;\\n\\n    /**\\n     * Returns a Javascript array with all the elements in the VBArray. If there are multiple dimensions,\\n     * each successive dimension is appended to the end of the array.\\n     * Example: [[1,2,3],[4,5,6]] becomes [1,2,3,4,5,6]\\n     */\\n    toArray(): T[];\\n}\\n\\ninterface VBArrayConstructor {\\n    new <T = any>(safeArray: SafeArray<T>): VBArray<T>;\\n}\\n\\ndeclare var VBArray: VBArrayConstructor;\\n\\n/**\\n * Automation date (VT_DATE)\\n */\\ndeclare class VarDate {\\n    private constructor();\\n    private VarDate_typekey: VarDate;\\n}\\n\\ninterface DateConstructor {\\n    new (vd: VarDate): Date;\\n}\\n\\ninterface Date {\\n    getVarDate: () => VarDate;\\n}\\n'},On=function(){function e(e,n){this._extraLibs=Object.create(null),this._languageService=xn(this),this._ctx=e,this._compilerOptions=n.compilerOptions,this._extraLibs=n.extraLibs}return e.prototype.getCompilationSettings=function(){return this._compilerOptions},e.prototype.getScriptFileNames=function(){return this._ctx.getMirrorModels().map(function(e){return e.uri.toString()}).concat(Object.keys(this._extraLibs))},e.prototype._getModel=function(e){for(var n=this._ctx.getMirrorModels(),t=0;t<n.length;t++)if(n[t].uri.toString()===e)return n[t];return null},e.prototype.getScriptVersion=function(e){var n=this._getModel(e);return n?n.version.toString():this.isDefaultLibFileName(e)||e in this._extraLibs?\"1\":void 0},e.prototype.getScriptSnapshot=function(e){var n,t=this._getModel(e);if(t)n=t.getValue();else if(e in this._extraLibs)n=this._extraLibs[e];else if(e===kn.NAME)n=kn.CONTENTS;else{if(e!==Mn.NAME)return;n=Mn.CONTENTS}return{getText:function(e,t){return n.substring(e,t)},getLength:function(){return n.length},getChangeRange:function(){}}},e.prototype.getScriptKind=function(e){switch(e.substr(e.lastIndexOf(\".\")+1)){case\"ts\":return An.TS;case\"tsx\":return An.TSX;case\"js\":return An.JS;case\"jsx\":return An.JSX;default:return this.getCompilationSettings().allowJs?An.JS:An.TS}},e.prototype.getCurrentDirectory=function(){return\"\"},e.prototype.getDefaultLibFileName=function(e){return e.target<=Cn.ES5?kn.NAME:Mn.NAME},e.prototype.isDefaultLibFileName=function(e){return e===this.getDefaultLibFileName(this._compilerOptions)},e.clearFiles=function(e){e.forEach(function(e){e.file=void 0;var n=e.relatedInformation;n&&n.forEach(function(e){return e.file=void 0})})},e.prototype.getSyntacticDiagnostics=function(n){var t=this._languageService.getSyntacticDiagnostics(n);return e.clearFiles(t),Dn.as(t)},e.prototype.getSemanticDiagnostics=function(n){var t=this._languageService.getSemanticDiagnostics(n);return e.clearFiles(t),Dn.as(t)},e.prototype.getCompilerOptionsDiagnostics=function(n){var t=this._languageService.getCompilerOptionsDiagnostics();return e.clearFiles(t),Dn.as(t)},e.prototype.getCompletionsAtPosition=function(e,n){return Dn.as(this._languageService.getCompletionsAtPosition(e,n,void 0))},e.prototype.getCompletionEntryDetails=function(e,n,t){return Dn.as(this._languageService.getCompletionEntryDetails(e,n,t,void 0,void 0,void 0))},e.prototype.getSignatureHelpItems=function(e,n){return Dn.as(this._languageService.getSignatureHelpItems(e,n,void 0))},e.prototype.getQuickInfoAtPosition=function(e,n){return Dn.as(this._languageService.getQuickInfoAtPosition(e,n))},e.prototype.getOccurrencesAtPosition=function(e,n){return Dn.as(this._languageService.getOccurrencesAtPosition(e,n))},e.prototype.getDefinitionAtPosition=function(e,n){return Dn.as(this._languageService.getDefinitionAtPosition(e,n))},e.prototype.getReferencesAtPosition=function(e,n){return Dn.as(this._languageService.getReferencesAtPosition(e,n))},e.prototype.getNavigationBarItems=function(e){return Dn.as(this._languageService.getNavigationBarItems(e))},e.prototype.getFormattingEditsForDocument=function(e,n){return Dn.as(this._languageService.getFormattingEditsForDocument(e,n))},e.prototype.getFormattingEditsForRange=function(e,n,t,r){return Dn.as(this._languageService.getFormattingEditsForRange(e,n,t,r))},e.prototype.getFormattingEditsAfterKeystroke=function(e,n,t,r){return Dn.as(this._languageService.getFormattingEditsAfterKeystroke(e,n,t,r))},e.prototype.getEmitOutput=function(e){return Dn.as(this._languageService.getEmitOutput(e))},e}();self.onmessage=function(){vn(function(e,n){return new On(e,n)})}}();\n"], { type: "text/javascript" }));

const workerURL = {
    css: cssWorker,
    html: htmlWorker,
    javascript: typescriptWorker,
    get typescript() {
        return this.javascript
    },
};
Object.setPrototypeOf(workerURL, null);

/*globals window */
window.MonacoEnvironment = {
    getWorkerUrl: (_, label) => workerURL[label] || editorWorker,
};

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco$5 = (typeof monaco === 'undefined' ? self.monaco : monaco);
var conf = {
    wordPattern: /(-?\d*\.\d\w*)|([^\`\~\!\@\#\%\^\&\*\(\)\-\=\+\[\{\]\}\\\|\;\:\'\"\,\.\<\>\/\?\s]+)/g,
    comments: {
        lineComment: '//',
        blockComment: ['/*', '*/']
    },
    brackets: [
        ['{', '}'],
        ['[', ']'],
        ['(', ')']
    ],
    onEnterRules: [
        {
            // e.g. /** | */
            beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
            afterText: /^\s*\*\/$/,
            action: { indentAction: _monaco$5.languages.IndentAction.IndentOutdent, appendText: ' * ' }
        },
        {
            // e.g. /** ...|
            beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
            action: { indentAction: _monaco$5.languages.IndentAction.None, appendText: ' * ' }
        },
        {
            // e.g.  * ...|
            beforeText: /^(\t|(\ \ ))*\ \*(\ ([^\*]|\*(?!\/))*)?$/,
            action: { indentAction: _monaco$5.languages.IndentAction.None, appendText: '* ' }
        },
        {
            // e.g.  */|
            beforeText: /^(\t|(\ \ ))*\ \*\/\s*$/,
            action: { indentAction: _monaco$5.languages.IndentAction.None, removeText: 1 }
        }
    ],
    autoClosingPairs: [
        { open: '{', close: '}' },
        { open: '[', close: ']' },
        { open: '(', close: ')' },
        { open: '"', close: '"', notIn: ['string'] },
        { open: '\'', close: '\'', notIn: ['string', 'comment'] },
        { open: '`', close: '`', notIn: ['string', 'comment'] },
        { open: "/**", close: " */", notIn: ["string"] }
    ],
    folding: {
        markers: {
            start: new RegExp("^\\s*//\\s*#?region\\b"),
            end: new RegExp("^\\s*//\\s*#?endregion\\b")
        }
    }
};
var language = {
    // Set defaultToken to invalid to see what you do not tokenize yet
    defaultToken: 'invalid',
    tokenPostfix: '.ts',
    keywords: [
        'abstract', 'as', 'break', 'case', 'catch', 'class', 'continue', 'const',
        'constructor', 'debugger', 'declare', 'default', 'delete', 'do', 'else',
        'enum', 'export', 'extends', 'false', 'finally', 'for', 'from', 'function',
        'get', 'if', 'implements', 'import', 'in', 'infer', 'instanceof', 'interface',
        'is', 'keyof', 'let', 'module', 'namespace', 'never', 'new', 'null', 'package',
        'private', 'protected', 'public', 'readonly', 'require', 'global', 'return',
        'set', 'static', 'super', 'switch', 'symbol', 'this', 'throw', 'true', 'try',
        'type', 'typeof', 'unique', 'var', 'void', 'while', 'with', 'yield', 'async',
        'await', 'of'
    ],
    typeKeywords: [
        'any', 'boolean', 'number', 'object', 'string', 'undefined'
    ],
    operators: [
        '<=', '>=', '==', '!=', '===', '!==', '=>', '+', '-', '**',
        '*', '/', '%', '++', '--', '<<', '</', '>>', '>>>', '&',
        '|', '^', '!', '~', '&&', '||', '?', ':', '=', '+=', '-=',
        '*=', '**=', '/=', '%=', '<<=', '>>=', '>>>=', '&=', '|=',
        '^=', '@',
    ],
    // we include these common regular expressions
    symbols: /[=><!~?:&|+\-*\/\^%]+/,
    escapes: /\\(?:[abfnrtv\\"']|x[0-9A-Fa-f]{1,4}|u[0-9A-Fa-f]{4}|U[0-9A-Fa-f]{8})/,
    digits: /\d+(_+\d+)*/,
    octaldigits: /[0-7]+(_+[0-7]+)*/,
    binarydigits: /[0-1]+(_+[0-1]+)*/,
    hexdigits: /[[0-9a-fA-F]+(_+[0-9a-fA-F]+)*/,
    regexpctl: /[(){}\[\]\$\^|\-*+?\.]/,
    regexpesc: /\\(?:[bBdDfnrstvwWn0\\\/]|@regexpctl|c[A-Z]|x[0-9a-fA-F]{2}|u[0-9a-fA-F]{4})/,
    // The main tokenizer for our languages
    tokenizer: {
        root: [
            [/[{}]/, 'delimiter.bracket'],
            { include: 'common' }
        ],
        common: [
            // identifiers and keywords
            [/[a-z_$][\w$]*/, {
                    cases: {
                        '@typeKeywords': 'keyword',
                        '@keywords': 'keyword',
                        '@default': 'identifier'
                    }
                }],
            [/[A-Z][\w\$]*/, 'type.identifier'],
            // [/[A-Z][\w\$]*/, 'identifier'],
            // whitespace
            { include: '@whitespace' },
            // regular expression: ensure it is terminated before beginning (otherwise it is an opeator)
            [/\/(?=([^\\\/]|\\.)+\/([gimsuy]*)(\s*)(\.|;|\/|,|\)|\]|\}|$))/, { token: 'regexp', bracket: '@open', next: '@regexp' }],
            // delimiters and operators
            [/[()\[\]]/, '@brackets'],
            [/[<>](?!@symbols)/, '@brackets'],
            [/@symbols/, {
                    cases: {
                        '@operators': 'delimiter',
                        '@default': ''
                    }
                }],
            // numbers
            [/(@digits)[eE]([\-+]?(@digits))?/, 'number.float'],
            [/(@digits)\.(@digits)([eE][\-+]?(@digits))?/, 'number.float'],
            [/0[xX](@hexdigits)/, 'number.hex'],
            [/0[oO]?(@octaldigits)/, 'number.octal'],
            [/0[bB](@binarydigits)/, 'number.binary'],
            [/(@digits)/, 'number'],
            // delimiter: after number because of .\d floats
            [/[;,.]/, 'delimiter'],
            // strings
            [/"([^"\\]|\\.)*$/, 'string.invalid'],
            [/'([^'\\]|\\.)*$/, 'string.invalid'],
            [/"/, 'string', '@string_double'],
            [/'/, 'string', '@string_single'],
            [/`/, 'string', '@string_backtick'],
        ],
        whitespace: [
            [/[ \t\r\n]+/, ''],
            [/\/\*\*(?!\/)/, 'comment.doc', '@jsdoc'],
            [/\/\*/, 'comment', '@comment'],
            [/\/\/.*$/, 'comment'],
        ],
        comment: [
            [/[^\/*]+/, 'comment'],
            [/\*\//, 'comment', '@pop'],
            [/[\/*]/, 'comment']
        ],
        jsdoc: [
            [/[^\/*]+/, 'comment.doc'],
            [/\*\//, 'comment.doc', '@pop'],
            [/[\/*]/, 'comment.doc']
        ],
        // We match regular expression quite precisely
        regexp: [
            [/(\{)(\d+(?:,\d*)?)(\})/, ['regexp.escape.control', 'regexp.escape.control', 'regexp.escape.control']],
            [/(\[)(\^?)(?=(?:[^\]\\\/]|\\.)+)/, ['regexp.escape.control', { token: 'regexp.escape.control', next: '@regexrange' }]],
            [/(\()(\?:|\?=|\?!)/, ['regexp.escape.control', 'regexp.escape.control']],
            [/[()]/, 'regexp.escape.control'],
            [/@regexpctl/, 'regexp.escape.control'],
            [/[^\\\/]/, 'regexp'],
            [/@regexpesc/, 'regexp.escape'],
            [/\\\./, 'regexp.invalid'],
            [/(\/)([gimsuy]*)/, [{ token: 'regexp', bracket: '@close', next: '@pop' }, 'keyword.other']],
        ],
        regexrange: [
            [/-/, 'regexp.escape.control'],
            [/\^/, 'regexp.invalid'],
            [/@regexpesc/, 'regexp.escape'],
            [/[^\]]/, 'regexp'],
            [/\]/, '@brackets.regexp.escape.control', '@pop'],
        ],
        string_double: [
            [/[^\\"]+/, 'string'],
            [/@escapes/, 'string.escape'],
            [/\\./, 'string.escape.invalid'],
            [/"/, 'string', '@pop']
        ],
        string_single: [
            [/[^\\']+/, 'string'],
            [/@escapes/, 'string.escape'],
            [/\\./, 'string.escape.invalid'],
            [/'/, 'string', '@pop']
        ],
        string_backtick: [
            [/\$\{/, { token: 'delimiter.bracket', next: '@bracketCounting' }],
            [/[^\\`$]+/, 'string'],
            [/@escapes/, 'string.escape'],
            [/\\./, 'string.escape.invalid'],
            [/`/, 'string', '@pop']
        ],
        bracketCounting: [
            [/\{/, 'delimiter.bracket', '@bracketCounting'],
            [/\}/, 'delimiter.bracket', '@pop'],
            { include: 'common' }
        ],
    },
};

var typescript = /*#__PURE__*/Object.freeze({
    conf: conf,
    language: language
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco$6 = (typeof monaco === 'undefined' ? self.monaco : monaco);
var conf$1 = conf;
var language$1 = {
    // Set defaultToken to invalid to see what you do not tokenize yet
    defaultToken: 'invalid',
    tokenPostfix: '.js',
    keywords: [
        'break', 'case', 'catch', 'class', 'continue', 'const',
        'constructor', 'debugger', 'default', 'delete', 'do', 'else',
        'export', 'extends', 'false', 'finally', 'for', 'from', 'function',
        'get', 'if', 'import', 'in', 'instanceof', 'let', 'new', 'null',
        'return', 'set', 'super', 'switch', 'symbol', 'this', 'throw', 'true',
        'try', 'typeof', 'undefined', 'var', 'void', 'while', 'with', 'yield',
        'async', 'await', 'of'
    ],
    typeKeywords: [],
    operators: language.operators,
    symbols: language.symbols,
    escapes: language.escapes,
    digits: language.digits,
    octaldigits: language.octaldigits,
    binarydigits: language.binarydigits,
    hexdigits: language.hexdigits,
    regexpctl: language.regexpctl,
    regexpesc: language.regexpesc,
    tokenizer: language.tokenizer,
};

var javascript = /*#__PURE__*/Object.freeze({
    conf: conf$1,
    language: language$1
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var conf$2 = {
    wordPattern: /(#?-?\d*\.\d\w*%?)|((::|[@#.!:])?[\w-?]+%?)|::|[@#.!:]/g,
    comments: {
        blockComment: ['/*', '*/']
    },
    brackets: [
        ['{', '}'],
        ['[', ']'],
        ['(', ')']
    ],
    autoClosingPairs: [
        { open: '{', close: '}', notIn: ['string', 'comment'] },
        { open: '[', close: ']', notIn: ['string', 'comment'] },
        { open: '(', close: ')', notIn: ['string', 'comment'] },
        { open: '"', close: '"', notIn: ['string', 'comment'] },
        { open: '\'', close: '\'', notIn: ['string', 'comment'] }
    ],
    surroundingPairs: [
        { open: '{', close: '}' },
        { open: '[', close: ']' },
        { open: '(', close: ')' },
        { open: '"', close: '"' },
        { open: '\'', close: '\'' }
    ],
    folding: {
        markers: {
            start: new RegExp("^\\s*\\/\\*\\s*#region\\b\\s*(.*?)\\s*\\*\\/"),
            end: new RegExp("^\\s*\\/\\*\\s*#endregion\\b.*\\*\\/")
        }
    }
};
var language$2 = {
    defaultToken: '',
    tokenPostfix: '.css',
    ws: '[ \t\n\r\f]*',
    identifier: '-?-?([a-zA-Z]|(\\\\(([0-9a-fA-F]{1,6}\\s?)|[^[0-9a-fA-F])))([\\w\\-]|(\\\\(([0-9a-fA-F]{1,6}\\s?)|[^[0-9a-fA-F])))*',
    brackets: [
        { open: '{', close: '}', token: 'delimiter.bracket' },
        { open: '[', close: ']', token: 'delimiter.bracket' },
        { open: '(', close: ')', token: 'delimiter.parenthesis' },
        { open: '<', close: '>', token: 'delimiter.angle' }
    ],
    tokenizer: {
        root: [
            { include: '@selector' },
        ],
        selector: [
            { include: '@comments' },
            { include: '@import' },
            { include: '@strings' },
            ['[@](keyframes|-webkit-keyframes|-moz-keyframes|-o-keyframes)', { token: 'keyword', next: '@keyframedeclaration' }],
            ['[@](page|content|font-face|-moz-document)', { token: 'keyword' }],
            ['[@](charset|namespace)', { token: 'keyword', next: '@declarationbody' }],
            ['(url-prefix)(\\()', ['attribute.value', { token: 'delimiter.parenthesis', next: '@urldeclaration' }]],
            ['(url)(\\()', ['attribute.value', { token: 'delimiter.parenthesis', next: '@urldeclaration' }]],
            { include: '@selectorname' },
            ['[\\*]', 'tag'],
            ['[>\\+,]', 'delimiter'],
            ['\\[', { token: 'delimiter.bracket', next: '@selectorattribute' }],
            ['{', { token: 'delimiter.bracket', next: '@selectorbody' }]
        ],
        selectorbody: [
            { include: '@comments' },
            ['[*_]?@identifier@ws:(?=(\\s|\\d|[^{;}]*[;}]))', 'attribute.name', '@rulevalue'],
            ['}', { token: 'delimiter.bracket', next: '@pop' }]
        ],
        selectorname: [
            ['(\\.|#(?=[^{])|%|(@identifier)|:)+', 'tag'],
        ],
        selectorattribute: [
            { include: '@term' },
            [']', { token: 'delimiter.bracket', next: '@pop' }],
        ],
        term: [
            { include: '@comments' },
            ['(url-prefix)(\\()', ['attribute.value', { token: 'delimiter.parenthesis', next: '@urldeclaration' }]],
            ['(url)(\\()', ['attribute.value', { token: 'delimiter.parenthesis', next: '@urldeclaration' }]],
            { include: '@functioninvocation' },
            { include: '@numbers' },
            { include: '@name' },
            ['([<>=\\+\\-\\*\\/\\^\\|\\~,])', 'delimiter'],
            [',', 'delimiter']
        ],
        rulevalue: [
            { include: '@comments' },
            { include: '@strings' },
            { include: '@term' },
            ['!important', 'keyword'],
            [';', 'delimiter', '@pop'],
            ['(?=})', { token: '', next: '@pop' }] // missing semicolon
        ],
        warndebug: [
            ['[@](warn|debug)', { token: 'keyword', next: '@declarationbody' }]
        ],
        import: [
            ['[@](import)', { token: 'keyword', next: '@declarationbody' }]
        ],
        urldeclaration: [
            { include: '@strings' },
            ['[^)\r\n]+', 'string'],
            ['\\)', { token: 'delimiter.parenthesis', next: '@pop' }]
        ],
        parenthizedterm: [
            { include: '@term' },
            ['\\)', { token: 'delimiter.parenthesis', next: '@pop' }]
        ],
        declarationbody: [
            { include: '@term' },
            [';', 'delimiter', '@pop'],
            ['(?=})', { token: '', next: '@pop' }] // missing semicolon
        ],
        comments: [
            ['\\/\\*', 'comment', '@comment'],
            ['\\/\\/+.*', 'comment']
        ],
        comment: [
            ['\\*\\/', 'comment', '@pop'],
            [/[^*/]+/, 'comment'],
            [/./, 'comment'],
        ],
        name: [
            ['@identifier', 'attribute.value']
        ],
        numbers: [
            ['-?(\\d*\\.)?\\d+([eE][\\-+]?\\d+)?', { token: 'attribute.value.number', next: '@units' }],
            ['#[0-9a-fA-F_]+(?!\\w)', 'attribute.value.hex']
        ],
        units: [
            ['(em|ex|ch|rem|vmin|vmax|vw|vh|vm|cm|mm|in|px|pt|pc|deg|grad|rad|turn|s|ms|Hz|kHz|%)?', 'attribute.value.unit', '@pop']
        ],
        keyframedeclaration: [
            ['@identifier', 'attribute.value'],
            ['{', { token: 'delimiter.bracket', switchTo: '@keyframebody' }],
        ],
        keyframebody: [
            { include: '@term' },
            ['{', { token: 'delimiter.bracket', next: '@selectorbody' }],
            ['}', { token: 'delimiter.bracket', next: '@pop' }],
        ],
        functioninvocation: [
            ['@identifier\\(', { token: 'attribute.value', next: '@functionarguments' }],
        ],
        functionarguments: [
            ['\\$@identifier@ws:', 'attribute.name'],
            ['[,]', 'delimiter'],
            { include: '@term' },
            ['\\)', { token: 'attribute.value', next: '@pop' }],
        ],
        strings: [
            ['~?"', { token: 'string', next: '@stringenddoublequote' }],
            ['~?\'', { token: 'string', next: '@stringendquote' }]
        ],
        stringenddoublequote: [
            ['\\\\.', 'string'],
            ['"', { token: 'string', next: '@pop' }],
            [/[^\\"]+/, 'string'],
            ['.', 'string']
        ],
        stringendquote: [
            ['\\\\.', 'string'],
            ['\'', { token: 'string', next: '@pop' }],
            [/[^\\']+/, 'string'],
            ['.', 'string']
        ]
    }
};

var css = /*#__PURE__*/Object.freeze({
    conf: conf$2,
    language: language$2
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// Allow for running under nodejs/requirejs in tests
var _monaco$7 = (typeof monaco === 'undefined' ? self.monaco : monaco);
var EMPTY_ELEMENTS = ['area', 'base', 'br', 'col', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'menuitem', 'meta', 'param', 'source', 'track', 'wbr'];
var conf$3 = {
    wordPattern: /(-?\d*\.\d\w*)|([^\`\~\!\@\$\^\&\*\(\)\=\+\[\{\]\}\\\|\;\:\'\"\,\.\<\>\/\s]+)/g,
    comments: {
        blockComment: ['<!--', '-->']
    },
    brackets: [
        ['<!--', '-->'],
        ['<', '>'],
        ['{', '}'],
        ['(', ')']
    ],
    autoClosingPairs: [
        { open: '{', close: '}' },
        { open: '[', close: ']' },
        { open: '(', close: ')' },
        { open: '"', close: '"' },
        { open: '\'', close: '\'' }
    ],
    surroundingPairs: [
        { open: '"', close: '"' },
        { open: '\'', close: '\'' },
        { open: '{', close: '}' },
        { open: '[', close: ']' },
        { open: '(', close: ')' },
        { open: '<', close: '>' },
    ],
    onEnterRules: [
        {
            beforeText: new RegExp("<(?!(?:" + EMPTY_ELEMENTS.join('|') + "))([_:\\w][_:\\w-.\\d]*)([^/>]*(?!/)>)[^<]*$", 'i'),
            afterText: /^<\/([_:\w][_:\w-.\d]*)\s*>$/i,
            action: { indentAction: _monaco$7.languages.IndentAction.IndentOutdent }
        },
        {
            beforeText: new RegExp("<(?!(?:" + EMPTY_ELEMENTS.join('|') + "))(\\w[\\w\\d]*)([^/>]*(?!/)>)[^<]*$", 'i'),
            action: { indentAction: _monaco$7.languages.IndentAction.Indent }
        }
    ],
    folding: {
        markers: {
            start: new RegExp("^\\s*<!--\\s*#region\\b.*-->"),
            end: new RegExp("^\\s*<!--\\s*#endregion\\b.*-->")
        }
    }
};
var language$3 = {
    defaultToken: '',
    tokenPostfix: '.html',
    ignoreCase: true,
    // The main tokenizer for our languages
    tokenizer: {
        root: [
            [/<!DOCTYPE/, 'metatag', '@doctype'],
            [/<!--/, 'comment', '@comment'],
            [/(<)((?:[\w\-]+:)?[\w\-]+)(\s*)(\/>)/, ['delimiter', 'tag', '', 'delimiter']],
            [/(<)(script)/, ['delimiter', { token: 'tag', next: '@script' }]],
            [/(<)(style)/, ['delimiter', { token: 'tag', next: '@style' }]],
            [/(<)((?:[\w\-]+:)?[\w\-]+)/, ['delimiter', { token: 'tag', next: '@otherTag' }]],
            [/(<\/)((?:[\w\-]+:)?[\w\-]+)/, ['delimiter', { token: 'tag', next: '@otherTag' }]],
            [/</, 'delimiter'],
            [/[^<]+/],
        ],
        doctype: [
            [/[^>]+/, 'metatag.content'],
            [/>/, 'metatag', '@pop'],
        ],
        comment: [
            [/-->/, 'comment', '@pop'],
            [/[^-]+/, 'comment.content'],
            [/./, 'comment.content']
        ],
        otherTag: [
            [/\/?>/, 'delimiter', '@pop'],
            [/"([^"]*)"/, 'attribute.value'],
            [/'([^']*)'/, 'attribute.value'],
            [/[\w\-]+/, 'attribute.name'],
            [/=/, 'delimiter'],
            [/[ \t\r\n]+/],
        ],
        // -- BEGIN <script> tags handling
        // After <script
        script: [
            [/type/, 'attribute.name', '@scriptAfterType'],
            [/"([^"]*)"/, 'attribute.value'],
            [/'([^']*)'/, 'attribute.value'],
            [/[\w\-]+/, 'attribute.name'],
            [/=/, 'delimiter'],
            [/>/, { token: 'delimiter', next: '@scriptEmbedded', nextEmbedded: 'text/javascript' }],
            [/[ \t\r\n]+/],
            [/(<\/)(script\s*)(>)/, ['delimiter', 'tag', { token: 'delimiter', next: '@pop' }]]
        ],
        // After <script ... type
        scriptAfterType: [
            [/=/, 'delimiter', '@scriptAfterTypeEquals'],
            [/>/, { token: 'delimiter', next: '@scriptEmbedded', nextEmbedded: 'text/javascript' }],
            [/[ \t\r\n]+/],
            [/<\/script\s*>/, { token: '@rematch', next: '@pop' }]
        ],
        // After <script ... type =
        scriptAfterTypeEquals: [
            [/"([^"]*)"/, { token: 'attribute.value', switchTo: '@scriptWithCustomType.$1' }],
            [/'([^']*)'/, { token: 'attribute.value', switchTo: '@scriptWithCustomType.$1' }],
            [/>/, { token: 'delimiter', next: '@scriptEmbedded', nextEmbedded: 'text/javascript' }],
            [/[ \t\r\n]+/],
            [/<\/script\s*>/, { token: '@rematch', next: '@pop' }]
        ],
        // After <script ... type = $S2
        scriptWithCustomType: [
            [/>/, { token: 'delimiter', next: '@scriptEmbedded.$S2', nextEmbedded: '$S2' }],
            [/"([^"]*)"/, 'attribute.value'],
            [/'([^']*)'/, 'attribute.value'],
            [/[\w\-]+/, 'attribute.name'],
            [/=/, 'delimiter'],
            [/[ \t\r\n]+/],
            [/<\/script\s*>/, { token: '@rematch', next: '@pop' }]
        ],
        scriptEmbedded: [
            [/<\/script/, { token: '@rematch', next: '@pop', nextEmbedded: '@pop' }],
            [/[^<]+/, '']
        ],
        // -- END <script> tags handling
        // -- BEGIN <style> tags handling
        // After <style
        style: [
            [/type/, 'attribute.name', '@styleAfterType'],
            [/"([^"]*)"/, 'attribute.value'],
            [/'([^']*)'/, 'attribute.value'],
            [/[\w\-]+/, 'attribute.name'],
            [/=/, 'delimiter'],
            [/>/, { token: 'delimiter', next: '@styleEmbedded', nextEmbedded: 'text/css' }],
            [/[ \t\r\n]+/],
            [/(<\/)(style\s*)(>)/, ['delimiter', 'tag', { token: 'delimiter', next: '@pop' }]]
        ],
        // After <style ... type
        styleAfterType: [
            [/=/, 'delimiter', '@styleAfterTypeEquals'],
            [/>/, { token: 'delimiter', next: '@styleEmbedded', nextEmbedded: 'text/css' }],
            [/[ \t\r\n]+/],
            [/<\/style\s*>/, { token: '@rematch', next: '@pop' }]
        ],
        // After <style ... type =
        styleAfterTypeEquals: [
            [/"([^"]*)"/, { token: 'attribute.value', switchTo: '@styleWithCustomType.$1' }],
            [/'([^']*)'/, { token: 'attribute.value', switchTo: '@styleWithCustomType.$1' }],
            [/>/, { token: 'delimiter', next: '@styleEmbedded', nextEmbedded: 'text/css' }],
            [/[ \t\r\n]+/],
            [/<\/style\s*>/, { token: '@rematch', next: '@pop' }]
        ],
        // After <style ... type = $S2
        styleWithCustomType: [
            [/>/, { token: 'delimiter', next: '@styleEmbedded.$S2', nextEmbedded: '$S2' }],
            [/"([^"]*)"/, 'attribute.value'],
            [/'([^']*)'/, 'attribute.value'],
            [/[\w\-]+/, 'attribute.name'],
            [/=/, 'delimiter'],
            [/[ \t\r\n]+/],
            [/<\/style\s*>/, { token: '@rematch', next: '@pop' }]
        ],
        styleEmbedded: [
            [/<\/style/, { token: '@rematch', next: '@pop', nextEmbedded: '@pop' }],
            [/[^<]+/, '']
        ],
    },
};

var html = /*#__PURE__*/Object.freeze({
    conf: conf$3,
    language: language$3
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var WorkerManager$1 = /** @class */ (function () {
    function WorkerManager(modeId, defaults) {
        var _this = this;
        this._modeId = modeId;
        this._defaults = defaults;
        this._worker = null;
        this._idleCheckInterval = setInterval(function () { return _this._checkIfIdle(); }, 30 * 1000);
        this._lastUsedTime = 0;
        this._configChangeListener = this._defaults.onDidChange(function () { return _this._stopWorker(); });
    }
    WorkerManager.prototype._stopWorker = function () {
        if (this._worker) {
            this._worker.dispose();
            this._worker = null;
        }
        this._client = null;
    };
    WorkerManager.prototype.dispose = function () {
        clearInterval(this._idleCheckInterval);
        this._configChangeListener.dispose();
        this._stopWorker();
    };
    WorkerManager.prototype._checkIfIdle = function () {
        if (!this._worker) {
            return;
        }
        var maxIdleTime = this._defaults.getWorkerMaxIdleTime();
        var timePassedSinceLastUsed = Date.now() - this._lastUsedTime;
        if (maxIdleTime > 0 && timePassedSinceLastUsed > maxIdleTime) {
            this._stopWorker();
        }
    };
    WorkerManager.prototype._getClient = function () {
        var _this = this;
        this._lastUsedTime = Date.now();
        if (!this._client) {
            this._worker = monaco.editor.createWebWorker({
                // module that exports the create() method and returns a `TypeScriptWorker` instance
                moduleId: 'vs/language/typescript/tsWorker',
                label: this._modeId,
                // passed in to the create() method
                createData: {
                    compilerOptions: this._defaults.getCompilerOptions(),
                    extraLibs: this._defaults.getExtraLibs()
                }
            });
            var p = this._worker.getProxy();
            if (this._defaults.getEagerModelSync()) {
                p = p.then(function (worker) {
                    return _this._worker.withSyncedResources(monaco.editor.getModels()
                        .filter(function (model) { return model.getModeId() === _this._modeId; })
                        .map(function (model) { return model.uri; }));
                });
            }
            this._client = p;
        }
        return this._client;
    };
    WorkerManager.prototype.getLanguageServiceWorker = function () {
        var _this = this;
        var resources = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            resources[_i] = arguments[_i];
        }
        var _client;
        return this._getClient().then(function (client) {
            _client = client;
        }).then(function (_) {
            return _this._worker.withSyncedResources(resources);
        }).then(function (_) { return _client; });
    };
    return WorkerManager;
}());

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends$2k = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Uri$1 = monaco.Uri;
var Promise$3 = monaco.Promise;
//#region utils copied from typescript to prevent loading the entire typescriptServices ---
var IndentStyle;
(function (IndentStyle) {
    IndentStyle[IndentStyle["None"] = 0] = "None";
    IndentStyle[IndentStyle["Block"] = 1] = "Block";
    IndentStyle[IndentStyle["Smart"] = 2] = "Smart";
})(IndentStyle || (IndentStyle = {}));
function flattenDiagnosticMessageText(messageText, newLine) {
    if (typeof messageText === "string") {
        return messageText;
    }
    else {
        var diagnosticChain = messageText;
        var result = "";
        var indent = 0;
        while (diagnosticChain) {
            if (indent) {
                result += newLine;
                for (var i = 0; i < indent; i++) {
                    result += "  ";
                }
            }
            result += diagnosticChain.messageText;
            indent++;
            diagnosticChain = diagnosticChain.next;
        }
        return result;
    }
}
function displayPartsToString(displayParts) {
    if (displayParts) {
        return displayParts.map(function (displayPart) { return displayPart.text; }).join("");
    }
    return "";
}
//#endregion
var Adapter = /** @class */ (function () {
    function Adapter(_worker) {
        this._worker = _worker;
    }
    Adapter.prototype._positionToOffset = function (uri, position) {
        var model = monaco.editor.getModel(uri);
        return model.getOffsetAt(position);
    };
    Adapter.prototype._offsetToPosition = function (uri, offset) {
        var model = monaco.editor.getModel(uri);
        return model.getPositionAt(offset);
    };
    Adapter.prototype._textSpanToRange = function (uri, span) {
        var p1 = this._offsetToPosition(uri, span.start);
        var p2 = this._offsetToPosition(uri, span.start + span.length);
        var startLineNumber = p1.lineNumber, startColumn = p1.column;
        var endLineNumber = p2.lineNumber, endColumn = p2.column;
        return { startLineNumber: startLineNumber, startColumn: startColumn, endLineNumber: endLineNumber, endColumn: endColumn };
    };
    return Adapter;
}());
// --- diagnostics --- ---
var DiagnostcsAdapter = /** @class */ (function (_super) {
    __extends$2k(DiagnostcsAdapter, _super);
    function DiagnostcsAdapter(_defaults, _selector, worker) {
        var _this = _super.call(this, worker) || this;
        _this._defaults = _defaults;
        _this._selector = _selector;
        _this._disposables = [];
        _this._listener = Object.create(null);
        var onModelAdd = function (model) {
            if (model.getModeId() !== _selector) {
                return;
            }
            var handle;
            var changeSubscription = model.onDidChangeContent(function () {
                clearTimeout(handle);
                handle = setTimeout(function () { return _this._doValidate(model.uri); }, 500);
            });
            _this._listener[model.uri.toString()] = {
                dispose: function () {
                    changeSubscription.dispose();
                    clearTimeout(handle);
                }
            };
            _this._doValidate(model.uri);
        };
        var onModelRemoved = function (model) {
            monaco.editor.setModelMarkers(model, _this._selector, []);
            var key = model.uri.toString();
            if (_this._listener[key]) {
                _this._listener[key].dispose();
                delete _this._listener[key];
            }
        };
        _this._disposables.push(monaco.editor.onDidCreateModel(onModelAdd));
        _this._disposables.push(monaco.editor.onWillDisposeModel(onModelRemoved));
        _this._disposables.push(monaco.editor.onDidChangeModelLanguage(function (event) {
            onModelRemoved(event.model);
            onModelAdd(event.model);
        }));
        _this._disposables.push({
            dispose: function () {
                for (var _i = 0, _a = monaco.editor.getModels(); _i < _a.length; _i++) {
                    var model = _a[_i];
                    onModelRemoved(model);
                }
            }
        });
        _this._disposables.push(_this._defaults.onDidChange(function () {
            // redo diagnostics when options change
            for (var _i = 0, _a = monaco.editor.getModels(); _i < _a.length; _i++) {
                var model = _a[_i];
                onModelRemoved(model);
                onModelAdd(model);
            }
        }));
        monaco.editor.getModels().forEach(onModelAdd);
        return _this;
    }
    DiagnostcsAdapter.prototype.dispose = function () {
        this._disposables.forEach(function (d) { return d && d.dispose(); });
        this._disposables = [];
    };
    DiagnostcsAdapter.prototype._doValidate = function (resource) {
        var _this = this;
        this._worker(resource).then(function (worker) {
            if (!monaco.editor.getModel(resource)) {
                // model was disposed in the meantime
                return null;
            }
            var promises = [];
            var _a = _this._defaults.getDiagnosticsOptions(), noSyntaxValidation = _a.noSyntaxValidation, noSemanticValidation = _a.noSemanticValidation;
            if (!noSyntaxValidation) {
                promises.push(worker.getSyntacticDiagnostics(resource.toString()));
            }
            if (!noSemanticValidation) {
                promises.push(worker.getSemanticDiagnostics(resource.toString()));
            }
            return Promise$3.join(promises);
        }).then(function (diagnostics) {
            if (!diagnostics || !monaco.editor.getModel(resource)) {
                // model was disposed in the meantime
                return null;
            }
            var markers = diagnostics
                .reduce(function (p, c) { return c.concat(p); }, [])
                .map(function (d) { return _this._convertDiagnostics(resource, d); });
            monaco.editor.setModelMarkers(monaco.editor.getModel(resource), _this._selector, markers);
        }).then(undefined, function (err) {
            console.error(err);
        });
    };
    DiagnostcsAdapter.prototype._convertDiagnostics = function (resource, diag) {
        var _a = this._offsetToPosition(resource, diag.start), startLineNumber = _a.lineNumber, startColumn = _a.column;
        var _b = this._offsetToPosition(resource, diag.start + diag.length), endLineNumber = _b.lineNumber, endColumn = _b.column;
        return {
            severity: monaco.MarkerSeverity.Error,
            startLineNumber: startLineNumber,
            startColumn: startColumn,
            endLineNumber: endLineNumber,
            endColumn: endColumn,
            message: flattenDiagnosticMessageText(diag.messageText, '\n')
        };
    };
    return DiagnostcsAdapter;
}(Adapter));
var SuggestAdapter = /** @class */ (function (_super) {
    __extends$2k(SuggestAdapter, _super);
    function SuggestAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SuggestAdapter.prototype, "triggerCharacters", {
        get: function () {
            return ['.'];
        },
        enumerable: true,
        configurable: true
    });
    SuggestAdapter.prototype.provideCompletionItems = function (model, position, _context, token) {
        var wordInfo = model.getWordUntilPosition(position);
        var resource = model.uri;
        var offset = this._positionToOffset(resource, position);
        return this._worker(resource).then(function (worker) {
            return worker.getCompletionsAtPosition(resource.toString(), offset);
        }).then(function (info) {
            if (!info) {
                return;
            }
            var suggestions = info.entries.map(function (entry) {
                return {
                    uri: resource,
                    position: position,
                    label: entry.name,
                    insertText: entry.name,
                    sortText: entry.sortText,
                    kind: SuggestAdapter.convertKind(entry.kind)
                };
            });
            return {
                suggestions: suggestions
            };
        });
    };
    SuggestAdapter.prototype.resolveCompletionItem = function (_model, _position, item, token) {
        var _this = this;
        var myItem = item;
        var resource = myItem.uri;
        var position = myItem.position;
        return this._worker(resource).then(function (worker) {
            return worker.getCompletionEntryDetails(resource.toString(), _this._positionToOffset(resource, position), myItem.label);
        }).then(function (details) {
            if (!details) {
                return myItem;
            }
            return {
                uri: resource,
                position: position,
                label: details.name,
                kind: SuggestAdapter.convertKind(details.kind),
                detail: displayPartsToString(details.displayParts),
                documentation: {
                    value: displayPartsToString(details.documentation)
                }
            };
        });
    };
    SuggestAdapter.convertKind = function (kind) {
        switch (kind) {
            case Kind.primitiveType:
            case Kind.keyword:
                return monaco.languages.CompletionItemKind.Keyword;
            case Kind.variable:
            case Kind.localVariable:
                return monaco.languages.CompletionItemKind.Variable;
            case Kind.memberVariable:
            case Kind.memberGetAccessor:
            case Kind.memberSetAccessor:
                return monaco.languages.CompletionItemKind.Field;
            case Kind.function:
            case Kind.memberFunction:
            case Kind.constructSignature:
            case Kind.callSignature:
            case Kind.indexSignature:
                return monaco.languages.CompletionItemKind.Function;
            case Kind.enum:
                return monaco.languages.CompletionItemKind.Enum;
            case Kind.module:
                return monaco.languages.CompletionItemKind.Module;
            case Kind.class:
                return monaco.languages.CompletionItemKind.Class;
            case Kind.interface:
                return monaco.languages.CompletionItemKind.Interface;
            case Kind.warning:
                return monaco.languages.CompletionItemKind.File;
        }
        return monaco.languages.CompletionItemKind.Property;
    };
    return SuggestAdapter;
}(Adapter));
var SignatureHelpAdapter = /** @class */ (function (_super) {
    __extends$2k(SignatureHelpAdapter, _super);
    function SignatureHelpAdapter() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.signatureHelpTriggerCharacters = ['(', ','];
        return _this;
    }
    SignatureHelpAdapter.prototype.provideSignatureHelp = function (model, position, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.getSignatureHelpItems(resource.toString(), _this._positionToOffset(resource, position)); }).then(function (info) {
            if (!info) {
                return;
            }
            var ret = {
                activeSignature: info.selectedItemIndex,
                activeParameter: info.argumentIndex,
                signatures: []
            };
            info.items.forEach(function (item) {
                var signature = {
                    label: '',
                    documentation: null,
                    parameters: []
                };
                signature.label += displayPartsToString(item.prefixDisplayParts);
                item.parameters.forEach(function (p, i, a) {
                    var label = displayPartsToString(p.displayParts);
                    var parameter = {
                        label: label,
                        documentation: displayPartsToString(p.documentation)
                    };
                    signature.label += label;
                    signature.parameters.push(parameter);
                    if (i < a.length - 1) {
                        signature.label += displayPartsToString(item.separatorDisplayParts);
                    }
                });
                signature.label += displayPartsToString(item.suffixDisplayParts);
                ret.signatures.push(signature);
            });
            return ret;
        });
    };
    return SignatureHelpAdapter;
}(Adapter));
// --- hover ------
var QuickInfoAdapter = /** @class */ (function (_super) {
    __extends$2k(QuickInfoAdapter, _super);
    function QuickInfoAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    QuickInfoAdapter.prototype.provideHover = function (model, position, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.getQuickInfoAtPosition(resource.toString(), _this._positionToOffset(resource, position));
        }).then(function (info) {
            if (!info) {
                return;
            }
            var documentation = displayPartsToString(info.documentation);
            var tags = info.tags ? info.tags.map(function (tag) {
                var label = "*@" + tag.name + "*";
                if (!tag.text) {
                    return label;
                }
                return label + (tag.text.match(/\r\n|\n/g) ? ' \n' + tag.text : " - " + tag.text);
            })
                .join('  \n\n') : '';
            var contents = displayPartsToString(info.displayParts);
            return {
                range: _this._textSpanToRange(resource, info.textSpan),
                contents: [{
                        value: '```js\n' + contents + '\n```\n'
                    }, {
                        value: documentation + (tags ? '\n\n' + tags : '')
                    }]
            };
        });
    };
    return QuickInfoAdapter;
}(Adapter));
// --- occurrences ------
var OccurrencesAdapter = /** @class */ (function (_super) {
    __extends$2k(OccurrencesAdapter, _super);
    function OccurrencesAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    OccurrencesAdapter.prototype.provideDocumentHighlights = function (model, position, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.getOccurrencesAtPosition(resource.toString(), _this._positionToOffset(resource, position));
        }).then(function (entries) {
            if (!entries) {
                return;
            }
            return entries.map(function (entry) {
                return {
                    range: _this._textSpanToRange(resource, entry.textSpan),
                    kind: entry.isWriteAccess ? monaco.languages.DocumentHighlightKind.Write : monaco.languages.DocumentHighlightKind.Text
                };
            });
        });
    };
    return OccurrencesAdapter;
}(Adapter));
// --- definition ------
var DefinitionAdapter = /** @class */ (function (_super) {
    __extends$2k(DefinitionAdapter, _super);
    function DefinitionAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DefinitionAdapter.prototype.provideDefinition = function (model, position, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.getDefinitionAtPosition(resource.toString(), _this._positionToOffset(resource, position));
        }).then(function (entries) {
            if (!entries) {
                return;
            }
            var result = [];
            for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                var entry = entries_1[_i];
                var uri = Uri$1.parse(entry.fileName);
                if (monaco.editor.getModel(uri)) {
                    result.push({
                        uri: uri,
                        range: _this._textSpanToRange(uri, entry.textSpan)
                    });
                }
            }
            return result;
        });
    };
    return DefinitionAdapter;
}(Adapter));
// --- references ------
var ReferenceAdapter = /** @class */ (function (_super) {
    __extends$2k(ReferenceAdapter, _super);
    function ReferenceAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ReferenceAdapter.prototype.provideReferences = function (model, position, context, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.getReferencesAtPosition(resource.toString(), _this._positionToOffset(resource, position));
        }).then(function (entries) {
            if (!entries) {
                return;
            }
            var result = [];
            for (var _i = 0, entries_2 = entries; _i < entries_2.length; _i++) {
                var entry = entries_2[_i];
                var uri = Uri$1.parse(entry.fileName);
                if (monaco.editor.getModel(uri)) {
                    result.push({
                        uri: uri,
                        range: _this._textSpanToRange(uri, entry.textSpan)
                    });
                }
            }
            return result;
        });
    };
    return ReferenceAdapter;
}(Adapter));
// --- outline ------
var OutlineAdapter = /** @class */ (function (_super) {
    __extends$2k(OutlineAdapter, _super);
    function OutlineAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    OutlineAdapter.prototype.provideDocumentSymbols = function (model, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.getNavigationBarItems(resource.toString()); }).then(function (items) {
            if (!items) {
                return;
            }
            var convert = function (bucket, item, containerLabel) {
                var result = {
                    name: item.text,
                    detail: '',
                    kind: (outlineTypeTable[item.kind] || monaco.languages.SymbolKind.Variable),
                    range: _this._textSpanToRange(resource, item.spans[0]),
                    selectionRange: _this._textSpanToRange(resource, item.spans[0]),
                    containerName: containerLabel
                };
                if (item.childItems && item.childItems.length > 0) {
                    for (var _i = 0, _a = item.childItems; _i < _a.length; _i++) {
                        var child = _a[_i];
                        convert(bucket, child, result.name);
                    }
                }
                bucket.push(result);
            };
            var result = [];
            items.forEach(function (item) { return convert(result, item); });
            return result;
        });
    };
    return OutlineAdapter;
}(Adapter));
var Kind = /** @class */ (function () {
    function Kind() {
    }
    Kind.unknown = '';
    Kind.keyword = 'keyword';
    Kind.script = 'script';
    Kind.module = 'module';
    Kind.class = 'class';
    Kind.interface = 'interface';
    Kind.type = 'type';
    Kind.enum = 'enum';
    Kind.variable = 'var';
    Kind.localVariable = 'local var';
    Kind.function = 'function';
    Kind.localFunction = 'local function';
    Kind.memberFunction = 'method';
    Kind.memberGetAccessor = 'getter';
    Kind.memberSetAccessor = 'setter';
    Kind.memberVariable = 'property';
    Kind.constructorImplementation = 'constructor';
    Kind.callSignature = 'call';
    Kind.indexSignature = 'index';
    Kind.constructSignature = 'construct';
    Kind.parameter = 'parameter';
    Kind.typeParameter = 'type parameter';
    Kind.primitiveType = 'primitive type';
    Kind.label = 'label';
    Kind.alias = 'alias';
    Kind.const = 'const';
    Kind.let = 'let';
    Kind.warning = 'warning';
    return Kind;
}());
var outlineTypeTable = Object.create(null);
outlineTypeTable[Kind.module] = monaco.languages.SymbolKind.Module;
outlineTypeTable[Kind.class] = monaco.languages.SymbolKind.Class;
outlineTypeTable[Kind.enum] = monaco.languages.SymbolKind.Enum;
outlineTypeTable[Kind.interface] = monaco.languages.SymbolKind.Interface;
outlineTypeTable[Kind.memberFunction] = monaco.languages.SymbolKind.Method;
outlineTypeTable[Kind.memberVariable] = monaco.languages.SymbolKind.Property;
outlineTypeTable[Kind.memberGetAccessor] = monaco.languages.SymbolKind.Property;
outlineTypeTable[Kind.memberSetAccessor] = monaco.languages.SymbolKind.Property;
outlineTypeTable[Kind.variable] = monaco.languages.SymbolKind.Variable;
outlineTypeTable[Kind.const] = monaco.languages.SymbolKind.Variable;
outlineTypeTable[Kind.localVariable] = monaco.languages.SymbolKind.Variable;
outlineTypeTable[Kind.variable] = monaco.languages.SymbolKind.Variable;
outlineTypeTable[Kind.function] = monaco.languages.SymbolKind.Function;
outlineTypeTable[Kind.localFunction] = monaco.languages.SymbolKind.Function;
// --- formatting ----
var FormatHelper = /** @class */ (function (_super) {
    __extends$2k(FormatHelper, _super);
    function FormatHelper() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FormatHelper._convertOptions = function (options) {
        return {
            ConvertTabsToSpaces: options.insertSpaces,
            TabSize: options.tabSize,
            IndentSize: options.tabSize,
            IndentStyle: IndentStyle.Smart,
            NewLineCharacter: '\n',
            InsertSpaceAfterCommaDelimiter: true,
            InsertSpaceAfterSemicolonInForStatements: true,
            InsertSpaceBeforeAndAfterBinaryOperators: true,
            InsertSpaceAfterKeywordsInControlFlowStatements: true,
            InsertSpaceAfterFunctionKeywordForAnonymousFunctions: true,
            InsertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis: false,
            InsertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets: false,
            InsertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces: false,
            PlaceOpenBraceOnNewLineForControlBlocks: false,
            PlaceOpenBraceOnNewLineForFunctions: false
        };
    };
    FormatHelper.prototype._convertTextChanges = function (uri, change) {
        return {
            text: change.newText,
            range: this._textSpanToRange(uri, change.span)
        };
    };
    return FormatHelper;
}(Adapter));
var FormatAdapter = /** @class */ (function (_super) {
    __extends$2k(FormatAdapter, _super);
    function FormatAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FormatAdapter.prototype.provideDocumentRangeFormattingEdits = function (model, range, options, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.getFormattingEditsForRange(resource.toString(), _this._positionToOffset(resource, { lineNumber: range.startLineNumber, column: range.startColumn }), _this._positionToOffset(resource, { lineNumber: range.endLineNumber, column: range.endColumn }), FormatHelper._convertOptions(options));
        }).then(function (edits) {
            if (edits) {
                return edits.map(function (edit) { return _this._convertTextChanges(resource, edit); });
            }
        });
    };
    return FormatAdapter;
}(FormatHelper));
var FormatOnTypeAdapter = /** @class */ (function (_super) {
    __extends$2k(FormatOnTypeAdapter, _super);
    function FormatOnTypeAdapter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(FormatOnTypeAdapter.prototype, "autoFormatTriggerCharacters", {
        get: function () {
            return [';', '}', '\n'];
        },
        enumerable: true,
        configurable: true
    });
    FormatOnTypeAdapter.prototype.provideOnTypeFormattingEdits = function (model, position, ch, options, token) {
        var _this = this;
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.getFormattingEditsAfterKeystroke(resource.toString(), _this._positionToOffset(resource, position), ch, FormatHelper._convertOptions(options));
        }).then(function (edits) {
            if (edits) {
                return edits.map(function (edit) { return _this._convertTextChanges(resource, edit); });
            }
        });
    };
    return FormatOnTypeAdapter;
}(FormatHelper));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var javaScriptWorker;
var typeScriptWorker;
function setupTypeScript(defaults) {
    typeScriptWorker = setupMode(defaults, 'typescript');
}
function setupJavaScript(defaults) {
    javaScriptWorker = setupMode(defaults, 'javascript');
}
function getJavaScriptWorker$1() {
    return new monaco.Promise(function (resolve, reject) {
        if (!javaScriptWorker) {
            return reject("JavaScript not registered!");
        }
        resolve(javaScriptWorker);
    });
}
function getTypeScriptWorker$1() {
    return new monaco.Promise(function (resolve, reject) {
        if (!typeScriptWorker) {
            return reject("TypeScript not registered!");
        }
        resolve(typeScriptWorker);
    });
}
function setupMode(defaults, modeId) {
    var client = new WorkerManager$1(modeId, defaults);
    var worker = function (first) {
        var more = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            more[_i - 1] = arguments[_i];
        }
        return client.getLanguageServiceWorker.apply(client, [first].concat(more));
    };
    monaco.languages.registerCompletionItemProvider(modeId, new SuggestAdapter(worker));
    monaco.languages.registerSignatureHelpProvider(modeId, new SignatureHelpAdapter(worker));
    monaco.languages.registerHoverProvider(modeId, new QuickInfoAdapter(worker));
    monaco.languages.registerDocumentHighlightProvider(modeId, new OccurrencesAdapter(worker));
    monaco.languages.registerDefinitionProvider(modeId, new DefinitionAdapter(worker));
    monaco.languages.registerReferenceProvider(modeId, new ReferenceAdapter(worker));
    monaco.languages.registerDocumentSymbolProvider(modeId, new OutlineAdapter(worker));
    monaco.languages.registerDocumentRangeFormattingEditProvider(modeId, new FormatAdapter(worker));
    monaco.languages.registerOnTypeFormattingEditProvider(modeId, new FormatOnTypeAdapter(worker));
    new DiagnostcsAdapter(defaults, modeId, worker);
    return worker;
}

var tsMode = /*#__PURE__*/Object.freeze({
    setupTypeScript: setupTypeScript,
    setupJavaScript: setupJavaScript,
    getJavaScriptWorker: getJavaScriptWorker$1,
    getTypeScriptWorker: getTypeScriptWorker$1
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var STOP_WHEN_IDLE_FOR = 2 * 60 * 1000; // 2min
var WorkerManager$2 = /** @class */ (function () {
    function WorkerManager(defaults) {
        var _this = this;
        this._defaults = defaults;
        this._worker = null;
        this._idleCheckInterval = setInterval(function () { return _this._checkIfIdle(); }, 30 * 1000);
        this._lastUsedTime = 0;
        this._configChangeListener = this._defaults.onDidChange(function () { return _this._stopWorker(); });
    }
    WorkerManager.prototype._stopWorker = function () {
        if (this._worker) {
            this._worker.dispose();
            this._worker = null;
        }
        this._client = null;
    };
    WorkerManager.prototype.dispose = function () {
        clearInterval(this._idleCheckInterval);
        this._configChangeListener.dispose();
        this._stopWorker();
    };
    WorkerManager.prototype._checkIfIdle = function () {
        if (!this._worker) {
            return;
        }
        var timePassedSinceLastUsed = Date.now() - this._lastUsedTime;
        if (timePassedSinceLastUsed > STOP_WHEN_IDLE_FOR) {
            this._stopWorker();
        }
    };
    WorkerManager.prototype._getClient = function () {
        this._lastUsedTime = Date.now();
        if (!this._client) {
            this._worker = monaco.editor.createWebWorker({
                // module that exports the create() method and returns a `CSSWorker` instance
                moduleId: 'vs/language/css/cssWorker',
                label: this._defaults.languageId,
                // passed in to the create() method
                createData: {
                    languageSettings: this._defaults.diagnosticsOptions,
                    languageId: this._defaults.languageId
                }
            });
            this._client = this._worker.getProxy();
        }
        return this._client;
    };
    WorkerManager.prototype.getLanguageServiceWorker = function () {
        var _this = this;
        var resources = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            resources[_i] = arguments[_i];
        }
        var _client;
        return this._getClient().then(function (client) {
            _client = client;
        }).then(function (_) {
            return _this._worker.withSyncedResources(resources);
        }).then(function (_) { return _client; });
    };
    return WorkerManager;
}());

/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
/**
 * The Position namespace provides helper functions to work with
 * [Position](#Position) literals.
 */
var Position$2;
(function (Position) {
    /**
     * Creates a new Position literal from the given line and character.
     * @param line The position's line.
     * @param character The position's character.
     */
    function create(line, character) {
        return { line: line, character: character };
    }
    Position.create = create;
    /**
     * Checks whether the given liternal conforms to the [Position](#Position) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.objectLiteral(candidate) && Is.number(candidate.line) && Is.number(candidate.character);
    }
    Position.is = is;
})(Position$2 || (Position$2 = {}));
/**
 * The Range namespace provides helper functions to work with
 * [Range](#Range) literals.
 */
var Range$3;
(function (Range) {
    function create(one, two, three, four) {
        if (Is.number(one) && Is.number(two) && Is.number(three) && Is.number(four)) {
            return { start: Position$2.create(one, two), end: Position$2.create(three, four) };
        }
        else if (Position$2.is(one) && Position$2.is(two)) {
            return { start: one, end: two };
        }
        else {
            throw new Error("Range#create called with invalid arguments[" + one + ", " + two + ", " + three + ", " + four + "]");
        }
    }
    Range.create = create;
    /**
     * Checks whether the given literal conforms to the [Range](#Range) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.objectLiteral(candidate) && Position$2.is(candidate.start) && Position$2.is(candidate.end);
    }
    Range.is = is;
})(Range$3 || (Range$3 = {}));
/**
 * The Location namespace provides helper functions to work with
 * [Location](#Location) literals.
 */
var Location;
(function (Location) {
    /**
     * Creates a Location literal.
     * @param uri The location's uri.
     * @param range The location's range.
     */
    function create(uri, range) {
        return { uri: uri, range: range };
    }
    Location.create = create;
    /**
     * Checks whether the given literal conforms to the [Location](#Location) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Range$3.is(candidate.range) && (Is.string(candidate.uri) || Is.undefined(candidate.uri));
    }
    Location.is = is;
})(Location || (Location = {}));
/**
 * The Color namespace provides helper functions to work with
 * [Color](#Color) literals.
 */
var Color$1;
(function (Color) {
    /**
     * Creates a new Color literal.
     */
    function create(red, green, blue, alpha) {
        return {
            red: red,
            green: green,
            blue: blue,
            alpha: alpha,
        };
    }
    Color.create = create;
    /**
     * Checks whether the given literal conforms to the [Color](#Color) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.number(candidate.red)
            && Is.number(candidate.green)
            && Is.number(candidate.blue)
            && Is.number(candidate.alpha);
    }
    Color.is = is;
})(Color$1 || (Color$1 = {}));
/**
 * The ColorInformation namespace provides helper functions to work with
 * [ColorInformation](#ColorInformation) literals.
 */
var ColorInformation;
(function (ColorInformation) {
    /**
     * Creates a new ColorInformation literal.
     */
    function create(range, color) {
        return {
            range: range,
            color: color,
        };
    }
    ColorInformation.create = create;
    /**
     * Checks whether the given literal conforms to the [ColorInformation](#ColorInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Range$3.is(candidate.range) && Color$1.is(candidate.color);
    }
    ColorInformation.is = is;
})(ColorInformation || (ColorInformation = {}));
/**
 * The Color namespace provides helper functions to work with
 * [ColorPresentation](#ColorPresentation) literals.
 */
var ColorPresentation;
(function (ColorPresentation) {
    /**
     * Creates a new ColorInformation literal.
     */
    function create(label, textEdit, additionalTextEdits) {
        return {
            label: label,
            textEdit: textEdit,
            additionalTextEdits: additionalTextEdits,
        };
    }
    ColorPresentation.create = create;
    /**
     * Checks whether the given literal conforms to the [ColorInformation](#ColorInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.string(candidate.label)
            && (Is.undefined(candidate.textEdit) || TextEdit.is(candidate))
            && (Is.undefined(candidate.additionalTextEdits) || Is.typedArray(candidate.additionalTextEdits, TextEdit.is));
    }
    ColorPresentation.is = is;
})(ColorPresentation || (ColorPresentation = {}));
/**
 * Enum of known range kinds
 */
var FoldingRangeKind$1;
(function (FoldingRangeKind) {
    /**
     * Folding range for a comment
     */
    FoldingRangeKind["Comment"] = "comment";
    /**
     * Folding range for a imports or includes
     */
    FoldingRangeKind["Imports"] = "imports";
    /**
     * Folding range for a region (e.g. `#region`)
     */
    FoldingRangeKind["Region"] = "region";
})(FoldingRangeKind$1 || (FoldingRangeKind$1 = {}));
/**
 * The folding range namespace provides helper functions to work with
 * [FoldingRange](#FoldingRange) literals.
 */
var FoldingRange;
(function (FoldingRange) {
    /**
     * Creates a new FoldingRange literal.
     */
    function create(startLine, endLine, startCharacter, endCharacter, kind) {
        var result = {
            startLine: startLine,
            endLine: endLine
        };
        if (Is.defined(startCharacter)) {
            result.startCharacter = startCharacter;
        }
        if (Is.defined(endCharacter)) {
            result.endCharacter = endCharacter;
        }
        if (Is.defined(kind)) {
            result.kind = kind;
        }
        return result;
    }
    FoldingRange.create = create;
    /**
     * Checks whether the given literal conforms to the [FoldingRange](#FoldingRange) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.number(candidate.startLine) && Is.number(candidate.startLine)
            && (Is.undefined(candidate.startCharacter) || Is.number(candidate.startCharacter))
            && (Is.undefined(candidate.endCharacter) || Is.number(candidate.endCharacter))
            && (Is.undefined(candidate.kind) || Is.string(candidate.kind));
    }
    FoldingRange.is = is;
})(FoldingRange || (FoldingRange = {}));
/**
 * The DiagnosticRelatedInformation namespace provides helper functions to work with
 * [DiagnosticRelatedInformation](#DiagnosticRelatedInformation) literals.
 */
var DiagnosticRelatedInformation;
(function (DiagnosticRelatedInformation) {
    /**
     * Creates a new DiagnosticRelatedInformation literal.
     */
    function create(location, message) {
        return {
            location: location,
            message: message
        };
    }
    DiagnosticRelatedInformation.create = create;
    /**
     * Checks whether the given literal conforms to the [DiagnosticRelatedInformation](#DiagnosticRelatedInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Location.is(candidate.location) && Is.string(candidate.message);
    }
    DiagnosticRelatedInformation.is = is;
})(DiagnosticRelatedInformation || (DiagnosticRelatedInformation = {}));
/**
 * The diagnostic's severity.
 */
var DiagnosticSeverity;
(function (DiagnosticSeverity) {
    /**
     * Reports an error.
     */
    DiagnosticSeverity.Error = 1;
    /**
     * Reports a warning.
     */
    DiagnosticSeverity.Warning = 2;
    /**
     * Reports an information.
     */
    DiagnosticSeverity.Information = 3;
    /**
     * Reports a hint.
     */
    DiagnosticSeverity.Hint = 4;
})(DiagnosticSeverity || (DiagnosticSeverity = {}));
/**
 * The Diagnostic namespace provides helper functions to work with
 * [Diagnostic](#Diagnostic) literals.
 */
var Diagnostic;
(function (Diagnostic) {
    /**
     * Creates a new Diagnostic literal.
     */
    function create(range, message, severity, code, source, relatedInformation) {
        var result = { range: range, message: message };
        if (Is.defined(severity)) {
            result.severity = severity;
        }
        if (Is.defined(code)) {
            result.code = code;
        }
        if (Is.defined(source)) {
            result.source = source;
        }
        if (Is.defined(relatedInformation)) {
            result.relatedInformation = relatedInformation;
        }
        return result;
    }
    Diagnostic.create = create;
    /**
     * Checks whether the given literal conforms to the [Diagnostic](#Diagnostic) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate)
            && Range$3.is(candidate.range)
            && Is.string(candidate.message)
            && (Is.number(candidate.severity) || Is.undefined(candidate.severity))
            && (Is.number(candidate.code) || Is.string(candidate.code) || Is.undefined(candidate.code))
            && (Is.string(candidate.source) || Is.undefined(candidate.source))
            && (Is.undefined(candidate.relatedInformation) || Is.typedArray(candidate.relatedInformation, DiagnosticRelatedInformation.is));
    }
    Diagnostic.is = is;
})(Diagnostic || (Diagnostic = {}));
/**
 * The Command namespace provides helper functions to work with
 * [Command](#Command) literals.
 */
var Command$1;
(function (Command) {
    /**
     * Creates a new Command literal.
     */
    function create(title, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var result = { title: title, command: command };
        if (Is.defined(args) && args.length > 0) {
            result.arguments = args;
        }
        return result;
    }
    Command.create = create;
    /**
     * Checks whether the given literal conforms to the [Command](#Command) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Is.string(candidate.title) && Is.string(candidate.command);
    }
    Command.is = is;
})(Command$1 || (Command$1 = {}));
/**
 * The TextEdit namespace provides helper function to create replace,
 * insert and delete edits more easily.
 */
var TextEdit;
(function (TextEdit) {
    /**
     * Creates a replace text edit.
     * @param range The range of text to be replaced.
     * @param newText The new text.
     */
    function replace(range, newText) {
        return { range: range, newText: newText };
    }
    TextEdit.replace = replace;
    /**
     * Creates a insert text edit.
     * @param position The position to insert the text at.
     * @param newText The text to be inserted.
     */
    function insert(position, newText) {
        return { range: { start: position, end: position }, newText: newText };
    }
    TextEdit.insert = insert;
    /**
     * Creates a delete text edit.
     * @param range The range of text to be deleted.
     */
    function del(range) {
        return { range: range, newText: '' };
    }
    TextEdit.del = del;
    function is(value) {
        var candidate = value;
        return Is.objectLiteral(candidate)
            && Is.string(candidate.newText)
            && Range$3.is(candidate.range);
    }
    TextEdit.is = is;
})(TextEdit || (TextEdit = {}));
/**
 * The TextDocumentEdit namespace provides helper function to create
 * an edit that manipulates a text document.
 */
var TextDocumentEdit;
(function (TextDocumentEdit) {
    /**
     * Creates a new `TextDocumentEdit`
     */
    function create(textDocument, edits) {
        return { textDocument: textDocument, edits: edits };
    }
    TextDocumentEdit.create = create;
    function is(value) {
        var candidate = value;
        return Is.defined(candidate)
            && VersionedTextDocumentIdentifier.is(candidate.textDocument)
            && Array.isArray(candidate.edits);
    }
    TextDocumentEdit.is = is;
})(TextDocumentEdit || (TextDocumentEdit = {}));
var WorkspaceEdit;
(function (WorkspaceEdit) {
    function is(value) {
        var candidate = value;
        return candidate &&
            (candidate.changes !== void 0 || candidate.documentChanges !== void 0) &&
            (candidate.documentChanges === void 0 || Is.typedArray(candidate.documentChanges, TextDocumentEdit.is));
    }
    WorkspaceEdit.is = is;
})(WorkspaceEdit || (WorkspaceEdit = {}));
var TextEditChangeImpl = /** @class */ (function () {
    function TextEditChangeImpl(edits) {
        this.edits = edits;
    }
    TextEditChangeImpl.prototype.insert = function (position, newText) {
        this.edits.push(TextEdit.insert(position, newText));
    };
    TextEditChangeImpl.prototype.replace = function (range, newText) {
        this.edits.push(TextEdit.replace(range, newText));
    };
    TextEditChangeImpl.prototype.delete = function (range) {
        this.edits.push(TextEdit.del(range));
    };
    TextEditChangeImpl.prototype.add = function (edit) {
        this.edits.push(edit);
    };
    TextEditChangeImpl.prototype.all = function () {
        return this.edits;
    };
    TextEditChangeImpl.prototype.clear = function () {
        this.edits.splice(0, this.edits.length);
    };
    return TextEditChangeImpl;
}());
/**
 * A workspace change helps constructing changes to a workspace.
 */
var WorkspaceChange = /** @class */ (function () {
    function WorkspaceChange(workspaceEdit) {
        var _this = this;
        this._textEditChanges = Object.create(null);
        if (workspaceEdit) {
            this._workspaceEdit = workspaceEdit;
            if (workspaceEdit.documentChanges) {
                workspaceEdit.documentChanges.forEach(function (textDocumentEdit) {
                    var textEditChange = new TextEditChangeImpl(textDocumentEdit.edits);
                    _this._textEditChanges[textDocumentEdit.textDocument.uri] = textEditChange;
                });
            }
            else if (workspaceEdit.changes) {
                Object.keys(workspaceEdit.changes).forEach(function (key) {
                    var textEditChange = new TextEditChangeImpl(workspaceEdit.changes[key]);
                    _this._textEditChanges[key] = textEditChange;
                });
            }
        }
    }
    Object.defineProperty(WorkspaceChange.prototype, "edit", {
        /**
         * Returns the underlying [WorkspaceEdit](#WorkspaceEdit) literal
         * use to be returned from a workspace edit operation like rename.
         */
        get: function () {
            return this._workspaceEdit;
        },
        enumerable: true,
        configurable: true
    });
    WorkspaceChange.prototype.getTextEditChange = function (key) {
        if (VersionedTextDocumentIdentifier.is(key)) {
            if (!this._workspaceEdit) {
                this._workspaceEdit = {
                    documentChanges: []
                };
            }
            if (!this._workspaceEdit.documentChanges) {
                throw new Error('Workspace edit is not configured for versioned document changes.');
            }
            var textDocument = key;
            var result = this._textEditChanges[textDocument.uri];
            if (!result) {
                var edits = [];
                var textDocumentEdit = {
                    textDocument: textDocument,
                    edits: edits
                };
                this._workspaceEdit.documentChanges.push(textDocumentEdit);
                result = new TextEditChangeImpl(edits);
                this._textEditChanges[textDocument.uri] = result;
            }
            return result;
        }
        else {
            if (!this._workspaceEdit) {
                this._workspaceEdit = {
                    changes: Object.create(null)
                };
            }
            if (!this._workspaceEdit.changes) {
                throw new Error('Workspace edit is not configured for normal text edit changes.');
            }
            var result = this._textEditChanges[key];
            if (!result) {
                var edits = [];
                this._workspaceEdit.changes[key] = edits;
                result = new TextEditChangeImpl(edits);
                this._textEditChanges[key] = result;
            }
            return result;
        }
    };
    return WorkspaceChange;
}());
/**
 * The TextDocumentIdentifier namespace provides helper functions to work with
 * [TextDocumentIdentifier](#TextDocumentIdentifier) literals.
 */
var TextDocumentIdentifier;
(function (TextDocumentIdentifier) {
    /**
     * Creates a new TextDocumentIdentifier literal.
     * @param uri The document's uri.
     */
    function create(uri) {
        return { uri: uri };
    }
    TextDocumentIdentifier.create = create;
    /**
     * Checks whether the given literal conforms to the [TextDocumentIdentifier](#TextDocumentIdentifier) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Is.string(candidate.uri);
    }
    TextDocumentIdentifier.is = is;
})(TextDocumentIdentifier || (TextDocumentIdentifier = {}));
/**
 * The VersionedTextDocumentIdentifier namespace provides helper functions to work with
 * [VersionedTextDocumentIdentifier](#VersionedTextDocumentIdentifier) literals.
 */
var VersionedTextDocumentIdentifier;
(function (VersionedTextDocumentIdentifier) {
    /**
     * Creates a new VersionedTextDocumentIdentifier literal.
     * @param uri The document's uri.
     * @param uri The document's text.
     */
    function create(uri, version) {
        return { uri: uri, version: version };
    }
    VersionedTextDocumentIdentifier.create = create;
    /**
     * Checks whether the given literal conforms to the [VersionedTextDocumentIdentifier](#VersionedTextDocumentIdentifier) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Is.string(candidate.uri) && Is.number(candidate.version);
    }
    VersionedTextDocumentIdentifier.is = is;
})(VersionedTextDocumentIdentifier || (VersionedTextDocumentIdentifier = {}));
/**
 * The TextDocumentItem namespace provides helper functions to work with
 * [TextDocumentItem](#TextDocumentItem) literals.
 */
var TextDocumentItem;
(function (TextDocumentItem) {
    /**
     * Creates a new TextDocumentItem literal.
     * @param uri The document's uri.
     * @param languageId The document's language identifier.
     * @param version The document's version number.
     * @param text The document's text.
     */
    function create(uri, languageId, version, text) {
        return { uri: uri, languageId: languageId, version: version, text: text };
    }
    TextDocumentItem.create = create;
    /**
     * Checks whether the given literal conforms to the [TextDocumentItem](#TextDocumentItem) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Is.string(candidate.uri) && Is.string(candidate.languageId) && Is.number(candidate.version) && Is.string(candidate.text);
    }
    TextDocumentItem.is = is;
})(TextDocumentItem || (TextDocumentItem = {}));
/**
 * Describes the content type that a client supports in various
 * result literals like `Hover`, `ParameterInfo` or `CompletionItem`.
 *
 * Please note that `MarkupKinds` must not start with a `$`. This kinds
 * are reserved for internal usage.
 */
var MarkupKind;
(function (MarkupKind) {
    /**
     * Plain text is supported as a content format
     */
    MarkupKind.PlainText = 'plaintext';
    /**
     * Markdown is supported as a content format
     */
    MarkupKind.Markdown = 'markdown';
})(MarkupKind || (MarkupKind = {}));
(function (MarkupKind) {
    /**
     * Checks whether the given value is a value of the [MarkupKind](#MarkupKind) type.
     */
    function is(value) {
        var candidate = value;
        return candidate === MarkupKind.PlainText || candidate === MarkupKind.Markdown;
    }
    MarkupKind.is = is;
})(MarkupKind || (MarkupKind = {}));
var MarkupContent;
(function (MarkupContent) {
    /**
     * Checks whether the given value conforms to the [MarkupContent](#MarkupContent) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.objectLiteral(value) && MarkupKind.is(candidate.kind) && Is.string(candidate.value);
    }
    MarkupContent.is = is;
})(MarkupContent || (MarkupContent = {}));
/**
 * The kind of a completion entry.
 */
var CompletionItemKind$1;
(function (CompletionItemKind) {
    CompletionItemKind.Text = 1;
    CompletionItemKind.Method = 2;
    CompletionItemKind.Function = 3;
    CompletionItemKind.Constructor = 4;
    CompletionItemKind.Field = 5;
    CompletionItemKind.Variable = 6;
    CompletionItemKind.Class = 7;
    CompletionItemKind.Interface = 8;
    CompletionItemKind.Module = 9;
    CompletionItemKind.Property = 10;
    CompletionItemKind.Unit = 11;
    CompletionItemKind.Value = 12;
    CompletionItemKind.Enum = 13;
    CompletionItemKind.Keyword = 14;
    CompletionItemKind.Snippet = 15;
    CompletionItemKind.Color = 16;
    CompletionItemKind.File = 17;
    CompletionItemKind.Reference = 18;
    CompletionItemKind.Folder = 19;
    CompletionItemKind.EnumMember = 20;
    CompletionItemKind.Constant = 21;
    CompletionItemKind.Struct = 22;
    CompletionItemKind.Event = 23;
    CompletionItemKind.Operator = 24;
    CompletionItemKind.TypeParameter = 25;
})(CompletionItemKind$1 || (CompletionItemKind$1 = {}));
/**
 * Defines whether the insert text in a completion item should be interpreted as
 * plain text or a snippet.
 */
var InsertTextFormat;
(function (InsertTextFormat) {
    /**
     * The primary text to be inserted is treated as a plain string.
     */
    InsertTextFormat.PlainText = 1;
    /**
     * The primary text to be inserted is treated as a snippet.
     *
     * A snippet can define tab stops and placeholders with `$1`, `$2`
     * and `${3:foo}`. `$0` defines the final tab stop, it defaults to
     * the end of the snippet. Placeholders with equal identifiers are linked,
     * that is typing in one will update others too.
     *
     * See also: https://github.com/Microsoft/vscode/blob/master/src/vs/editor/contrib/snippet/common/snippet.md
     */
    InsertTextFormat.Snippet = 2;
})(InsertTextFormat || (InsertTextFormat = {}));
/**
 * The CompletionItem namespace provides functions to deal with
 * completion items.
 */
var CompletionItem;
(function (CompletionItem) {
    /**
     * Create a completion item and seed it with a label.
     * @param label The completion item's label
     */
    function create(label) {
        return { label: label };
    }
    CompletionItem.create = create;
})(CompletionItem || (CompletionItem = {}));
/**
 * The CompletionList namespace provides functions to deal with
 * completion lists.
 */
var CompletionList;
(function (CompletionList) {
    /**
     * Creates a new completion list.
     *
     * @param items The completion items.
     * @param isIncomplete The list is not complete.
     */
    function create(items, isIncomplete) {
        return { items: items ? items : [], isIncomplete: !!isIncomplete };
    }
    CompletionList.create = create;
})(CompletionList || (CompletionList = {}));
var MarkedString;
(function (MarkedString) {
    /**
     * Creates a marked string from plain text.
     *
     * @param plainText The plain text.
     */
    function fromPlainText(plainText) {
        return plainText.replace(/[\\`*_{}[\]()#+\-.!]/g, "\\$&"); // escape markdown syntax tokens: http://daringfireball.net/projects/markdown/syntax#backslash
    }
    MarkedString.fromPlainText = fromPlainText;
    /**
     * Checks whether the given value conforms to the [MarkedString](#MarkedString) type.
     */
    function is(value) {
        var candidate = value;
        return Is.string(candidate) || (Is.objectLiteral(candidate) && Is.string(candidate.language) && Is.string(candidate.value));
    }
    MarkedString.is = is;
})(MarkedString || (MarkedString = {}));
var Hover;
(function (Hover) {
    /**
     * Checks whether the given value conforms to the [Hover](#Hover) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.objectLiteral(candidate) && (MarkupContent.is(candidate.contents) ||
            MarkedString.is(candidate.contents) ||
            Is.typedArray(candidate.contents, MarkedString.is)) && (value.range === void 0 || Range$3.is(value.range));
    }
    Hover.is = is;
})(Hover || (Hover = {}));
/**
 * The ParameterInformation namespace provides helper functions to work with
 * [ParameterInformation](#ParameterInformation) literals.
 */
var ParameterInformation;
(function (ParameterInformation) {
    /**
     * Creates a new parameter information literal.
     *
     * @param label A label string.
     * @param documentation A doc string.
     */
    function create(label, documentation) {
        return documentation ? { label: label, documentation: documentation } : { label: label };
    }
    ParameterInformation.create = create;
})(ParameterInformation || (ParameterInformation = {}));
/**
 * The SignatureInformation namespace provides helper functions to work with
 * [SignatureInformation](#SignatureInformation) literals.
 */
var SignatureInformation;
(function (SignatureInformation) {
    function create(label, documentation) {
        var parameters = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            parameters[_i - 2] = arguments[_i];
        }
        var result = { label: label };
        if (Is.defined(documentation)) {
            result.documentation = documentation;
        }
        if (Is.defined(parameters)) {
            result.parameters = parameters;
        }
        else {
            result.parameters = [];
        }
        return result;
    }
    SignatureInformation.create = create;
})(SignatureInformation || (SignatureInformation = {}));
/**
 * A document highlight kind.
 */
var DocumentHighlightKind$2;
(function (DocumentHighlightKind) {
    /**
     * A textual occurrence.
     */
    DocumentHighlightKind.Text = 1;
    /**
     * Read-access of a symbol, like reading a variable.
     */
    DocumentHighlightKind.Read = 2;
    /**
     * Write-access of a symbol, like writing to a variable.
     */
    DocumentHighlightKind.Write = 3;
})(DocumentHighlightKind$2 || (DocumentHighlightKind$2 = {}));
/**
 * DocumentHighlight namespace to provide helper functions to work with
 * [DocumentHighlight](#DocumentHighlight) literals.
 */
var DocumentHighlight;
(function (DocumentHighlight) {
    /**
     * Create a DocumentHighlight object.
     * @param range The range the highlight applies to.
     */
    function create(range, kind) {
        var result = { range: range };
        if (Is.number(kind)) {
            result.kind = kind;
        }
        return result;
    }
    DocumentHighlight.create = create;
})(DocumentHighlight || (DocumentHighlight = {}));
/**
 * A symbol kind.
 */
var SymbolKind$1;
(function (SymbolKind) {
    SymbolKind.File = 1;
    SymbolKind.Module = 2;
    SymbolKind.Namespace = 3;
    SymbolKind.Package = 4;
    SymbolKind.Class = 5;
    SymbolKind.Method = 6;
    SymbolKind.Property = 7;
    SymbolKind.Field = 8;
    SymbolKind.Constructor = 9;
    SymbolKind.Enum = 10;
    SymbolKind.Interface = 11;
    SymbolKind.Function = 12;
    SymbolKind.Variable = 13;
    SymbolKind.Constant = 14;
    SymbolKind.String = 15;
    SymbolKind.Number = 16;
    SymbolKind.Boolean = 17;
    SymbolKind.Array = 18;
    SymbolKind.Object = 19;
    SymbolKind.Key = 20;
    SymbolKind.Null = 21;
    SymbolKind.EnumMember = 22;
    SymbolKind.Struct = 23;
    SymbolKind.Event = 24;
    SymbolKind.Operator = 25;
    SymbolKind.TypeParameter = 26;
})(SymbolKind$1 || (SymbolKind$1 = {}));
var SymbolInformation;
(function (SymbolInformation) {
    /**
     * Creates a new symbol information literal.
     *
     * @param name The name of the symbol.
     * @param kind The kind of the symbol.
     * @param range The range of the location of the symbol.
     * @param uri The resource of the location of symbol, defaults to the current document.
     * @param containerName The name of the symbol containing the symbol.
     */
    function create(name, kind, range, uri, containerName) {
        var result = {
            name: name,
            kind: kind,
            location: { uri: uri, range: range }
        };
        if (containerName) {
            result.containerName = containerName;
        }
        return result;
    }
    SymbolInformation.create = create;
})(SymbolInformation || (SymbolInformation = {}));
/**
 * Represents programming constructs like variables, classes, interfaces etc.
 * that appear in a document. Document symbols can be hierarchical and they
 * have two ranges: one that encloses its definition and one that points to
 * its most interesting range, e.g. the range of an identifier.
 */
var DocumentSymbol = /** @class */ (function () {
    function DocumentSymbol() {
    }
    return DocumentSymbol;
}());
(function (DocumentSymbol) {
    /**
     * Creates a new symbol information literal.
     *
     * @param name The name of the symbol.
     * @param detail The detail of the symbol.
     * @param kind The kind of the symbol.
     * @param range The range of the symbol.
     * @param selectionRange The selectionRange of the symbol.
     * @param children Children of the symbol.
     */
    function create(name, detail, kind, range, selectionRange, children) {
        var result = {
            name: name,
            detail: detail,
            kind: kind,
            range: range,
            selectionRange: selectionRange
        };
        if (children !== void 0) {
            result.children = children;
        }
        return result;
    }
    DocumentSymbol.create = create;
    /**
     * Checks whether the given literal conforms to the [DocumentSymbol](#DocumentSymbol) interface.
     */
    function is(value) {
        var candidate = value;
        return candidate &&
            Is.string(candidate.name) && Is.string(candidate.detail) && Is.number(candidate.kind) &&
            Range$3.is(candidate.range) && Range$3.is(candidate.selectionRange) &&
            (candidate.deprecated === void 0 || Is.boolean(candidate.deprecated)) &&
            (candidate.children === void 0 || Array.isArray(candidate.children));
    }
    DocumentSymbol.is = is;
})(DocumentSymbol || (DocumentSymbol = {}));
/**
 * A set of predefined code action kinds
 */
var CodeActionKind;
(function (CodeActionKind) {
    /**
     * Base kind for quickfix actions: 'quickfix'
     */
    CodeActionKind.QuickFix = 'quickfix';
    /**
     * Base kind for refactoring actions: 'refactor'
     */
    CodeActionKind.Refactor = 'refactor';
    /**
     * Base kind for refactoring extraction actions: 'refactor.extract'
     *
     * Example extract actions:
     *
     * - Extract method
     * - Extract function
     * - Extract variable
     * - Extract interface from class
     * - ...
     */
    CodeActionKind.RefactorExtract = 'refactor.extract';
    /**
     * Base kind for refactoring inline actions: 'refactor.inline'
     *
     * Example inline actions:
     *
     * - Inline function
     * - Inline variable
     * - Inline constant
     * - ...
     */
    CodeActionKind.RefactorInline = 'refactor.inline';
    /**
     * Base kind for refactoring rewrite actions: 'refactor.rewrite'
     *
     * Example rewrite actions:
     *
     * - Convert JavaScript function to class
     * - Add or remove parameter
     * - Encapsulate field
     * - Make method static
     * - Move method to base class
     * - ...
     */
    CodeActionKind.RefactorRewrite = 'refactor.rewrite';
    /**
     * Base kind for source actions: `source`
     *
     * Source code actions apply to the entire file.
     */
    CodeActionKind.Source = 'source';
    /**
     * Base kind for an organize imports source action: `source.organizeImports`
     */
    CodeActionKind.SourceOrganizeImports = 'source.organizeImports';
})(CodeActionKind || (CodeActionKind = {}));
/**
 * The CodeActionContext namespace provides helper functions to work with
 * [CodeActionContext](#CodeActionContext) literals.
 */
var CodeActionContext;
(function (CodeActionContext) {
    /**
     * Creates a new CodeActionContext literal.
     */
    function create(diagnostics, only) {
        var result = { diagnostics: diagnostics };
        if (only !== void 0 && only !== null) {
            result.only = only;
        }
        return result;
    }
    CodeActionContext.create = create;
    /**
     * Checks whether the given literal conforms to the [CodeActionContext](#CodeActionContext) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Is.typedArray(candidate.diagnostics, Diagnostic.is) && (candidate.only === void 0 || Is.typedArray(candidate.only, Is.string));
    }
    CodeActionContext.is = is;
})(CodeActionContext || (CodeActionContext = {}));
var CodeAction;
(function (CodeAction) {
    function create(title, commandOrEdit, kind) {
        var result = { title: title };
        if (Command$1.is(commandOrEdit)) {
            result.command = commandOrEdit;
        }
        else {
            result.edit = commandOrEdit;
        }
        if (kind !== void null) {
            result.kind = kind;
        }
        return result;
    }
    CodeAction.create = create;
    function is(value) {
        var candidate = value;
        return candidate && Is.string(candidate.title) &&
            (candidate.diagnostics === void 0 || Is.typedArray(candidate.diagnostics, Diagnostic.is)) &&
            (candidate.kind === void 0 || Is.string(candidate.kind)) &&
            (candidate.edit !== void 0 || candidate.command !== void 0) &&
            (candidate.command === void 0 || Command$1.is(candidate.command)) &&
            (candidate.edit === void 0 || WorkspaceEdit.is(candidate.edit));
    }
    CodeAction.is = is;
})(CodeAction || (CodeAction = {}));
/**
 * The CodeLens namespace provides helper functions to work with
 * [CodeLens](#CodeLens) literals.
 */
var CodeLens;
(function (CodeLens) {
    /**
     * Creates a new CodeLens literal.
     */
    function create(range, data) {
        var result = { range: range };
        if (Is.defined(data))
            result.data = data;
        return result;
    }
    CodeLens.create = create;
    /**
     * Checks whether the given literal conforms to the [CodeLens](#CodeLens) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Range$3.is(candidate.range) && (Is.undefined(candidate.command) || Command$1.is(candidate.command));
    }
    CodeLens.is = is;
})(CodeLens || (CodeLens = {}));
/**
 * The FormattingOptions namespace provides helper functions to work with
 * [FormattingOptions](#FormattingOptions) literals.
 */
var FormattingOptions;
(function (FormattingOptions) {
    /**
     * Creates a new FormattingOptions literal.
     */
    function create(tabSize, insertSpaces) {
        return { tabSize: tabSize, insertSpaces: insertSpaces };
    }
    FormattingOptions.create = create;
    /**
     * Checks whether the given literal conforms to the [FormattingOptions](#FormattingOptions) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Is.number(candidate.tabSize) && Is.boolean(candidate.insertSpaces);
    }
    FormattingOptions.is = is;
})(FormattingOptions || (FormattingOptions = {}));
/**
 * A document link is a range in a text document that links to an internal or external resource, like another
 * text document or a web site.
 */
var DocumentLink = /** @class */ (function () {
    function DocumentLink() {
    }
    return DocumentLink;
}());
/**
 * The DocumentLink namespace provides helper functions to work with
 * [DocumentLink](#DocumentLink) literals.
 */
(function (DocumentLink) {
    /**
     * Creates a new DocumentLink literal.
     */
    function create(range, target, data) {
        return { range: range, target: target, data: data };
    }
    DocumentLink.create = create;
    /**
     * Checks whether the given literal conforms to the [DocumentLink](#DocumentLink) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Range$3.is(candidate.range) && (Is.undefined(candidate.target) || Is.string(candidate.target));
    }
    DocumentLink.is = is;
})(DocumentLink || (DocumentLink = {}));
var TextDocument;
(function (TextDocument) {
    /**
     * Creates a new ITextDocument literal from the given uri and content.
     * @param uri The document's uri.
     * @param languageId  The document's language Id.
     * @param content The document's content.
     */
    function create(uri, languageId, version, content) {
        return new FullTextDocument(uri, languageId, version, content);
    }
    TextDocument.create = create;
    /**
     * Checks whether the given literal conforms to the [ITextDocument](#ITextDocument) interface.
     */
    function is(value) {
        var candidate = value;
        return Is.defined(candidate) && Is.string(candidate.uri) && (Is.undefined(candidate.languageId) || Is.string(candidate.languageId)) && Is.number(candidate.lineCount)
            && Is.func(candidate.getText) && Is.func(candidate.positionAt) && Is.func(candidate.offsetAt) ? true : false;
    }
    TextDocument.is = is;
    function applyEdits(document, edits) {
        var text = document.getText();
        var sortedEdits = mergeSort(edits, function (a, b) {
            var diff = a.range.start.line - b.range.start.line;
            if (diff === 0) {
                return a.range.start.character - b.range.start.character;
            }
            return diff;
        });
        var lastModifiedOffset = text.length;
        for (var i = sortedEdits.length - 1; i >= 0; i--) {
            var e = sortedEdits[i];
            var startOffset = document.offsetAt(e.range.start);
            var endOffset = document.offsetAt(e.range.end);
            if (endOffset <= lastModifiedOffset) {
                text = text.substring(0, startOffset) + e.newText + text.substring(endOffset, text.length);
            }
            else {
                throw new Error('Ovelapping edit');
            }
            lastModifiedOffset = startOffset;
        }
        return text;
    }
    TextDocument.applyEdits = applyEdits;
    function mergeSort(data, compare) {
        if (data.length <= 1) {
            // sorted
            return data;
        }
        var p = (data.length / 2) | 0;
        var left = data.slice(0, p);
        var right = data.slice(p);
        mergeSort(left, compare);
        mergeSort(right, compare);
        var leftIdx = 0;
        var rightIdx = 0;
        var i = 0;
        while (leftIdx < left.length && rightIdx < right.length) {
            var ret = compare(left[leftIdx], right[rightIdx]);
            if (ret <= 0) {
                // smaller_equal -> take left to preserve order
                data[i++] = left[leftIdx++];
            }
            else {
                // greater -> take right
                data[i++] = right[rightIdx++];
            }
        }
        while (leftIdx < left.length) {
            data[i++] = left[leftIdx++];
        }
        while (rightIdx < right.length) {
            data[i++] = right[rightIdx++];
        }
        return data;
    }
})(TextDocument || (TextDocument = {}));
/**
 * Represents reasons why a text document is saved.
 */
var TextDocumentSaveReason;
(function (TextDocumentSaveReason) {
    /**
     * Manually triggered, e.g. by the user pressing save, by starting debugging,
     * or by an API call.
     */
    TextDocumentSaveReason.Manual = 1;
    /**
     * Automatic after a delay.
     */
    TextDocumentSaveReason.AfterDelay = 2;
    /**
     * When the editor lost focus.
     */
    TextDocumentSaveReason.FocusOut = 3;
})(TextDocumentSaveReason || (TextDocumentSaveReason = {}));
var FullTextDocument = /** @class */ (function () {
    function FullTextDocument(uri, languageId, version, content) {
        this._uri = uri;
        this._languageId = languageId;
        this._version = version;
        this._content = content;
        this._lineOffsets = null;
    }
    Object.defineProperty(FullTextDocument.prototype, "uri", {
        get: function () {
            return this._uri;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FullTextDocument.prototype, "languageId", {
        get: function () {
            return this._languageId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FullTextDocument.prototype, "version", {
        get: function () {
            return this._version;
        },
        enumerable: true,
        configurable: true
    });
    FullTextDocument.prototype.getText = function (range) {
        if (range) {
            var start = this.offsetAt(range.start);
            var end = this.offsetAt(range.end);
            return this._content.substring(start, end);
        }
        return this._content;
    };
    FullTextDocument.prototype.update = function (event, version) {
        this._content = event.text;
        this._version = version;
        this._lineOffsets = null;
    };
    FullTextDocument.prototype.getLineOffsets = function () {
        if (this._lineOffsets === null) {
            var lineOffsets = [];
            var text = this._content;
            var isLineStart = true;
            for (var i = 0; i < text.length; i++) {
                if (isLineStart) {
                    lineOffsets.push(i);
                    isLineStart = false;
                }
                var ch = text.charAt(i);
                isLineStart = (ch === '\r' || ch === '\n');
                if (ch === '\r' && i + 1 < text.length && text.charAt(i + 1) === '\n') {
                    i++;
                }
            }
            if (isLineStart && text.length > 0) {
                lineOffsets.push(text.length);
            }
            this._lineOffsets = lineOffsets;
        }
        return this._lineOffsets;
    };
    FullTextDocument.prototype.positionAt = function (offset) {
        offset = Math.max(Math.min(offset, this._content.length), 0);
        var lineOffsets = this.getLineOffsets();
        var low = 0, high = lineOffsets.length;
        if (high === 0) {
            return Position$2.create(0, offset);
        }
        while (low < high) {
            var mid = Math.floor((low + high) / 2);
            if (lineOffsets[mid] > offset) {
                high = mid;
            }
            else {
                low = mid + 1;
            }
        }
        // low is the least x for which the line offset is larger than the current offset
        // or array.length if no line offset is larger than the current offset
        var line = low - 1;
        return Position$2.create(line, offset - lineOffsets[line]);
    };
    FullTextDocument.prototype.offsetAt = function (position) {
        var lineOffsets = this.getLineOffsets();
        if (position.line >= lineOffsets.length) {
            return this._content.length;
        }
        else if (position.line < 0) {
            return 0;
        }
        var lineOffset = lineOffsets[position.line];
        var nextLineOffset = (position.line + 1 < lineOffsets.length) ? lineOffsets[position.line + 1] : this._content.length;
        return Math.max(Math.min(lineOffset + position.character, nextLineOffset), lineOffset);
    };
    Object.defineProperty(FullTextDocument.prototype, "lineCount", {
        get: function () {
            return this.getLineOffsets().length;
        },
        enumerable: true,
        configurable: true
    });
    return FullTextDocument;
}());
var Is;
(function (Is) {
    var toString = Object.prototype.toString;
    function defined(value) {
        return typeof value !== 'undefined';
    }
    Is.defined = defined;
    function undefined(value) {
        return typeof value === 'undefined';
    }
    Is.undefined = undefined;
    function boolean(value) {
        return value === true || value === false;
    }
    Is.boolean = boolean;
    function string(value) {
        return toString.call(value) === '[object String]';
    }
    Is.string = string;
    function number(value) {
        return toString.call(value) === '[object Number]';
    }
    Is.number = number;
    function func(value) {
        return toString.call(value) === '[object Function]';
    }
    Is.func = func;
    function objectLiteral(value) {
        // Strictly speaking class instances pass this check as well. Since the LSP
        // doesn't use classes we ignore this for now. If we do we need to add something
        // like this: `Object.getPrototypeOf(Object.getPrototypeOf(x)) === null`
        return value !== null && typeof value === 'object';
    }
    Is.objectLiteral = objectLiteral;
    function typedArray(value, check) {
        return Array.isArray(value) && value.every(check);
    }
    Is.typedArray = typedArray;
})(Is || (Is = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Uri$2 = monaco.Uri;
// --- diagnostics --- ---
var DiagnosticsAdapter = /** @class */ (function () {
    function DiagnosticsAdapter(_languageId, _worker, defaults) {
        var _this = this;
        this._languageId = _languageId;
        this._worker = _worker;
        this._disposables = [];
        this._listener = Object.create(null);
        var onModelAdd = function (model) {
            var modeId = model.getModeId();
            if (modeId !== _this._languageId) {
                return;
            }
            var handle;
            _this._listener[model.uri.toString()] = model.onDidChangeContent(function () {
                clearTimeout(handle);
                handle = setTimeout(function () { return _this._doValidate(model.uri, modeId); }, 500);
            });
            _this._doValidate(model.uri, modeId);
        };
        var onModelRemoved = function (model) {
            monaco.editor.setModelMarkers(model, _this._languageId, []);
            var uriStr = model.uri.toString();
            var listener = _this._listener[uriStr];
            if (listener) {
                listener.dispose();
                delete _this._listener[uriStr];
            }
        };
        this._disposables.push(monaco.editor.onDidCreateModel(onModelAdd));
        this._disposables.push(monaco.editor.onWillDisposeModel(onModelRemoved));
        this._disposables.push(monaco.editor.onDidChangeModelLanguage(function (event) {
            onModelRemoved(event.model);
            onModelAdd(event.model);
        }));
        defaults.onDidChange(function (_) {
            monaco.editor.getModels().forEach(function (model) {
                if (model.getModeId() === _this._languageId) {
                    onModelRemoved(model);
                    onModelAdd(model);
                }
            });
        });
        this._disposables.push({
            dispose: function () {
                for (var key in _this._listener) {
                    _this._listener[key].dispose();
                }
            }
        });
        monaco.editor.getModels().forEach(onModelAdd);
    }
    DiagnosticsAdapter.prototype.dispose = function () {
        this._disposables.forEach(function (d) { return d && d.dispose(); });
        this._disposables = [];
    };
    DiagnosticsAdapter.prototype._doValidate = function (resource, languageId) {
        this._worker(resource).then(function (worker) {
            return worker.doValidation(resource.toString());
        }).then(function (diagnostics) {
            var markers = diagnostics.map(function (d) { return toDiagnostics(resource, d); });
            var model = monaco.editor.getModel(resource);
            if (model.getModeId() === languageId) {
                monaco.editor.setModelMarkers(model, languageId, markers);
            }
        }).then(undefined, function (err) {
            console.error(err);
        });
    };
    return DiagnosticsAdapter;
}());
function toSeverity(lsSeverity) {
    switch (lsSeverity) {
        case DiagnosticSeverity.Error: return monaco.MarkerSeverity.Error;
        case DiagnosticSeverity.Warning: return monaco.MarkerSeverity.Warning;
        case DiagnosticSeverity.Information: return monaco.MarkerSeverity.Info;
        case DiagnosticSeverity.Hint: return monaco.MarkerSeverity.Hint;
        default:
            return monaco.MarkerSeverity.Info;
    }
}
function toDiagnostics(resource, diag) {
    var code = typeof diag.code === 'number' ? String(diag.code) : diag.code;
    return {
        severity: toSeverity(diag.severity),
        startLineNumber: diag.range.start.line + 1,
        startColumn: diag.range.start.character + 1,
        endLineNumber: diag.range.end.line + 1,
        endColumn: diag.range.end.character + 1,
        message: diag.message,
        code: code,
        source: diag.source
    };
}
// --- completion ------
function fromPosition(position) {
    if (!position) {
        return void 0;
    }
    return { character: position.column - 1, line: position.lineNumber - 1 };
}
function fromRange(range) {
    if (!range) {
        return void 0;
    }
    return { start: { line: range.startLineNumber - 1, character: range.startColumn - 1 }, end: { line: range.endLineNumber - 1, character: range.endColumn - 1 } };
}
function toRange(range) {
    if (!range) {
        return void 0;
    }
    return new monaco.Range(range.start.line + 1, range.start.character + 1, range.end.line + 1, range.end.character + 1);
}
function toCompletionItemKind(kind) {
    var mItemKind = monaco.languages.CompletionItemKind;
    switch (kind) {
        case CompletionItemKind$1.Text: return mItemKind.Text;
        case CompletionItemKind$1.Method: return mItemKind.Method;
        case CompletionItemKind$1.Function: return mItemKind.Function;
        case CompletionItemKind$1.Constructor: return mItemKind.Constructor;
        case CompletionItemKind$1.Field: return mItemKind.Field;
        case CompletionItemKind$1.Variable: return mItemKind.Variable;
        case CompletionItemKind$1.Class: return mItemKind.Class;
        case CompletionItemKind$1.Interface: return mItemKind.Interface;
        case CompletionItemKind$1.Module: return mItemKind.Module;
        case CompletionItemKind$1.Property: return mItemKind.Property;
        case CompletionItemKind$1.Unit: return mItemKind.Unit;
        case CompletionItemKind$1.Value: return mItemKind.Value;
        case CompletionItemKind$1.Enum: return mItemKind.Enum;
        case CompletionItemKind$1.Keyword: return mItemKind.Keyword;
        case CompletionItemKind$1.Snippet: return mItemKind.Snippet;
        case CompletionItemKind$1.Color: return mItemKind.Color;
        case CompletionItemKind$1.File: return mItemKind.File;
        case CompletionItemKind$1.Reference: return mItemKind.Reference;
    }
    return mItemKind.Property;
}
function toTextEdit(textEdit) {
    if (!textEdit) {
        return void 0;
    }
    return {
        range: toRange(textEdit.range),
        text: textEdit.newText
    };
}
var CompletionAdapter = /** @class */ (function () {
    function CompletionAdapter(_worker) {
        this._worker = _worker;
    }
    Object.defineProperty(CompletionAdapter.prototype, "triggerCharacters", {
        get: function () {
            return [' ', ':'];
        },
        enumerable: true,
        configurable: true
    });
    CompletionAdapter.prototype.provideCompletionItems = function (model, position, context, token) {
        var wordInfo = model.getWordUntilPosition(position);
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.doComplete(resource.toString(), fromPosition(position));
        }).then(function (info) {
            if (!info) {
                return;
            }
            var items = info.items.map(function (entry) {
                var item = {
                    label: entry.label,
                    insertText: entry.insertText || entry.label,
                    sortText: entry.sortText,
                    filterText: entry.filterText,
                    documentation: entry.documentation,
                    detail: entry.detail,
                    kind: toCompletionItemKind(entry.kind),
                };
                if (entry.textEdit) {
                    item.range = toRange(entry.textEdit.range);
                    item.insertText = entry.textEdit.newText;
                }
                if (entry.additionalTextEdits) {
                    item.additionalTextEdits = entry.additionalTextEdits.map(toTextEdit);
                }
                if (entry.insertTextFormat === InsertTextFormat.Snippet) {
                    item.insertTextRules = monaco.languages.CompletionItemInsertTextRule.InsertAsSnippet;
                }
                return item;
            });
            return {
                isIncomplete: info.isIncomplete,
                suggestions: items
            };
        });
    };
    return CompletionAdapter;
}());
function isMarkupContent(thing) {
    return thing && typeof thing === 'object' && typeof thing.kind === 'string';
}
function toMarkdownString(entry) {
    if (typeof entry === 'string') {
        return {
            value: entry
        };
    }
    if (isMarkupContent(entry)) {
        if (entry.kind === 'plaintext') {
            return {
                value: entry.value.replace(/[\\`*_{}[\]()#+\-.!]/g, '\\$&')
            };
        }
        return {
            value: entry.value
        };
    }
    return { value: '```' + entry.language + '\n' + entry.value + '\n```\n' };
}
function toMarkedStringArray(contents) {
    if (!contents) {
        return void 0;
    }
    if (Array.isArray(contents)) {
        return contents.map(toMarkdownString);
    }
    return [toMarkdownString(contents)];
}
// --- hover ------
var HoverAdapter = /** @class */ (function () {
    function HoverAdapter(_worker) {
        this._worker = _worker;
    }
    HoverAdapter.prototype.provideHover = function (model, position, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.doHover(resource.toString(), fromPosition(position));
        }).then(function (info) {
            if (!info) {
                return;
            }
            return {
                range: toRange(info.range),
                contents: toMarkedStringArray(info.contents)
            };
        });
    };
    return HoverAdapter;
}());
// --- document highlights ------
function toDocumentHighlightKind(kind) {
    switch (kind) {
        case DocumentHighlightKind$2.Read: return monaco.languages.DocumentHighlightKind.Read;
        case DocumentHighlightKind$2.Write: return monaco.languages.DocumentHighlightKind.Write;
        case DocumentHighlightKind$2.Text: return monaco.languages.DocumentHighlightKind.Text;
    }
    return monaco.languages.DocumentHighlightKind.Text;
}
var DocumentHighlightAdapter = /** @class */ (function () {
    function DocumentHighlightAdapter(_worker) {
        this._worker = _worker;
    }
    DocumentHighlightAdapter.prototype.provideDocumentHighlights = function (model, position, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.findDocumentHighlights(resource.toString(), fromPosition(position));
        }).then(function (entries) {
            if (!entries) {
                return;
            }
            return entries.map(function (entry) {
                return {
                    range: toRange(entry.range),
                    kind: toDocumentHighlightKind(entry.kind)
                };
            });
        });
    };
    return DocumentHighlightAdapter;
}());
// --- definition ------
function toLocation(location) {
    return {
        uri: Uri$2.parse(location.uri),
        range: toRange(location.range)
    };
}
var DefinitionAdapter$1 = /** @class */ (function () {
    function DefinitionAdapter(_worker) {
        this._worker = _worker;
    }
    DefinitionAdapter.prototype.provideDefinition = function (model, position, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.findDefinition(resource.toString(), fromPosition(position));
        }).then(function (definition) {
            if (!definition) {
                return;
            }
            return [toLocation(definition)];
        });
    };
    return DefinitionAdapter;
}());
// --- references ------
var ReferenceAdapter$1 = /** @class */ (function () {
    function ReferenceAdapter(_worker) {
        this._worker = _worker;
    }
    ReferenceAdapter.prototype.provideReferences = function (model, position, context, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.findReferences(resource.toString(), fromPosition(position));
        }).then(function (entries) {
            if (!entries) {
                return;
            }
            return entries.map(toLocation);
        });
    };
    return ReferenceAdapter;
}());
// --- rename ------
function toWorkspaceEdit(edit) {
    if (!edit || !edit.changes) {
        return void 0;
    }
    var resourceEdits = [];
    for (var uri in edit.changes) {
        var edits = [];
        for (var _i = 0, _a = edit.changes[uri]; _i < _a.length; _i++) {
            var e = _a[_i];
            edits.push({
                range: toRange(e.range),
                text: e.newText
            });
        }
        resourceEdits.push({ resource: Uri$2.parse(uri), edits: edits });
    }
    return {
        edits: resourceEdits
    };
}
var RenameAdapter = /** @class */ (function () {
    function RenameAdapter(_worker) {
        this._worker = _worker;
    }
    RenameAdapter.prototype.provideRenameEdits = function (model, position, newName, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.doRename(resource.toString(), fromPosition(position), newName);
        }).then(function (edit) {
            return toWorkspaceEdit(edit);
        });
    };
    return RenameAdapter;
}());
// --- document symbols ------
function toSymbolKind(kind) {
    var mKind = monaco.languages.SymbolKind;
    switch (kind) {
        case SymbolKind$1.File: return mKind.Array;
        case SymbolKind$1.Module: return mKind.Module;
        case SymbolKind$1.Namespace: return mKind.Namespace;
        case SymbolKind$1.Package: return mKind.Package;
        case SymbolKind$1.Class: return mKind.Class;
        case SymbolKind$1.Method: return mKind.Method;
        case SymbolKind$1.Property: return mKind.Property;
        case SymbolKind$1.Field: return mKind.Field;
        case SymbolKind$1.Constructor: return mKind.Constructor;
        case SymbolKind$1.Enum: return mKind.Enum;
        case SymbolKind$1.Interface: return mKind.Interface;
        case SymbolKind$1.Function: return mKind.Function;
        case SymbolKind$1.Variable: return mKind.Variable;
        case SymbolKind$1.Constant: return mKind.Constant;
        case SymbolKind$1.String: return mKind.String;
        case SymbolKind$1.Number: return mKind.Number;
        case SymbolKind$1.Boolean: return mKind.Boolean;
        case SymbolKind$1.Array: return mKind.Array;
    }
    return mKind.Function;
}
var DocumentSymbolAdapter = /** @class */ (function () {
    function DocumentSymbolAdapter(_worker) {
        this._worker = _worker;
    }
    DocumentSymbolAdapter.prototype.provideDocumentSymbols = function (model, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.findDocumentSymbols(resource.toString()); }).then(function (items) {
            if (!items) {
                return;
            }
            return items.map(function (item) { return ({
                name: item.name,
                detail: '',
                containerName: item.containerName,
                kind: toSymbolKind(item.kind),
                range: toRange(item.location.range),
                selectionRange: toRange(item.location.range)
            }); });
        });
    };
    return DocumentSymbolAdapter;
}());
var DocumentColorAdapter = /** @class */ (function () {
    function DocumentColorAdapter(_worker) {
        this._worker = _worker;
    }
    DocumentColorAdapter.prototype.provideDocumentColors = function (model, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.findDocumentColors(resource.toString()); }).then(function (infos) {
            if (!infos) {
                return;
            }
            return infos.map(function (item) { return ({
                color: item.color,
                range: toRange(item.range)
            }); });
        });
    };
    DocumentColorAdapter.prototype.provideColorPresentations = function (model, info, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.getColorPresentations(resource.toString(), info.color, fromRange(info.range)); }).then(function (presentations) {
            if (!presentations) {
                return;
            }
            return presentations.map(function (presentation) {
                var item = {
                    label: presentation.label,
                };
                if (presentation.textEdit) {
                    item.textEdit = toTextEdit(presentation.textEdit);
                }
                if (presentation.additionalTextEdits) {
                    item.additionalTextEdits = presentation.additionalTextEdits.map(toTextEdit);
                }
                return item;
            });
        });
    };
    return DocumentColorAdapter;
}());
var FoldingRangeAdapter = /** @class */ (function () {
    function FoldingRangeAdapter(_worker) {
        this._worker = _worker;
    }
    FoldingRangeAdapter.prototype.provideFoldingRanges = function (model, context, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.provideFoldingRanges(resource.toString(), context); }).then(function (ranges) {
            if (!ranges) {
                return;
            }
            return ranges.map(function (range) {
                var result = {
                    start: range.startLine + 1,
                    end: range.endLine + 1
                };
                if (typeof range.kind !== 'undefined') {
                    result.kind = toFoldingRangeKind(range.kind);
                }
                return result;
            });
        });
    };
    return FoldingRangeAdapter;
}());
function toFoldingRangeKind(kind) {
    switch (kind) {
        case FoldingRangeKind$1.Comment: return monaco.languages.FoldingRangeKind.Comment;
        case FoldingRangeKind$1.Imports: return monaco.languages.FoldingRangeKind.Imports;
        case FoldingRangeKind$1.Region: return monaco.languages.FoldingRangeKind.Region;
    }
    return void 0;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function setupMode$1(defaults) {
    var client = new WorkerManager$2(defaults);
    var worker = function (first) {
        var more = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            more[_i - 1] = arguments[_i];
        }
        return client.getLanguageServiceWorker.apply(client, [first].concat(more));
    };
    var languageId = defaults.languageId;
    monaco.languages.registerCompletionItemProvider(languageId, new CompletionAdapter(worker));
    monaco.languages.registerHoverProvider(languageId, new HoverAdapter(worker));
    monaco.languages.registerDocumentHighlightProvider(languageId, new DocumentHighlightAdapter(worker));
    monaco.languages.registerDefinitionProvider(languageId, new DefinitionAdapter$1(worker));
    monaco.languages.registerReferenceProvider(languageId, new ReferenceAdapter$1(worker));
    monaco.languages.registerDocumentSymbolProvider(languageId, new DocumentSymbolAdapter(worker));
    monaco.languages.registerRenameProvider(languageId, new RenameAdapter(worker));
    monaco.languages.registerColorProvider(languageId, new DocumentColorAdapter(worker));
    monaco.languages.registerFoldingRangeProvider(languageId, new FoldingRangeAdapter(worker));
    new DiagnosticsAdapter(languageId, worker, defaults);
}

var cssMode = /*#__PURE__*/Object.freeze({
    setupMode: setupMode$1
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var STOP_WHEN_IDLE_FOR$1 = 2 * 60 * 1000; // 2min
var WorkerManager$3 = /** @class */ (function () {
    function WorkerManager(defaults) {
        var _this = this;
        this._defaults = defaults;
        this._worker = null;
        this._idleCheckInterval = setInterval(function () { return _this._checkIfIdle(); }, 30 * 1000);
        this._lastUsedTime = 0;
        this._configChangeListener = this._defaults.onDidChange(function () { return _this._stopWorker(); });
    }
    WorkerManager.prototype._stopWorker = function () {
        if (this._worker) {
            this._worker.dispose();
            this._worker = null;
        }
        this._client = null;
    };
    WorkerManager.prototype.dispose = function () {
        clearInterval(this._idleCheckInterval);
        this._configChangeListener.dispose();
        this._stopWorker();
    };
    WorkerManager.prototype._checkIfIdle = function () {
        if (!this._worker) {
            return;
        }
        var timePassedSinceLastUsed = Date.now() - this._lastUsedTime;
        if (timePassedSinceLastUsed > STOP_WHEN_IDLE_FOR$1) {
            this._stopWorker();
        }
    };
    WorkerManager.prototype._getClient = function () {
        this._lastUsedTime = Date.now();
        if (!this._client) {
            this._worker = monaco.editor.createWebWorker({
                // module that exports the create() method and returns a `JSONWorker` instance
                moduleId: 'vs/language/json/jsonWorker',
                label: this._defaults.languageId,
                // passed in to the create() method
                createData: {
                    languageSettings: this._defaults.diagnosticsOptions,
                    languageId: this._defaults.languageId,
                    enableSchemaRequest: this._defaults.diagnosticsOptions.enableSchemaRequest
                }
            });
            this._client = this._worker.getProxy();
        }
        return this._client;
    };
    WorkerManager.prototype.getLanguageServiceWorker = function () {
        var _this = this;
        var resources = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            resources[_i] = arguments[_i];
        }
        var _client;
        return this._getClient().then(function (client) {
            _client = client;
        }).then(function (_) {
            return _this._worker.withSyncedResources(resources);
        }).then(function (_) { return _client; });
    };
    return WorkerManager;
}());

/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
/**
 * The Position namespace provides helper functions to work with
 * [Position](#Position) literals.
 */
var Position$3;
(function (Position) {
    /**
     * Creates a new Position literal from the given line and character.
     * @param line The position's line.
     * @param character The position's character.
     */
    function create(line, character) {
        return { line: line, character: character };
    }
    Position.create = create;
    /**
     * Checks whether the given liternal conforms to the [Position](#Position) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.objectLiteral(candidate) && Is$1.number(candidate.line) && Is$1.number(candidate.character);
    }
    Position.is = is;
})(Position$3 || (Position$3 = {}));
/**
 * The Range namespace provides helper functions to work with
 * [Range](#Range) literals.
 */
var Range$4;
(function (Range) {
    function create(one, two, three, four) {
        if (Is$1.number(one) && Is$1.number(two) && Is$1.number(three) && Is$1.number(four)) {
            return { start: Position$3.create(one, two), end: Position$3.create(three, four) };
        }
        else if (Position$3.is(one) && Position$3.is(two)) {
            return { start: one, end: two };
        }
        else {
            throw new Error("Range#create called with invalid arguments[" + one + ", " + two + ", " + three + ", " + four + "]");
        }
    }
    Range.create = create;
    /**
     * Checks whether the given literal conforms to the [Range](#Range) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.objectLiteral(candidate) && Position$3.is(candidate.start) && Position$3.is(candidate.end);
    }
    Range.is = is;
})(Range$4 || (Range$4 = {}));
/**
 * The Location namespace provides helper functions to work with
 * [Location](#Location) literals.
 */
var Location$1;
(function (Location) {
    /**
     * Creates a Location literal.
     * @param uri The location's uri.
     * @param range The location's range.
     */
    function create(uri, range) {
        return { uri: uri, range: range };
    }
    Location.create = create;
    /**
     * Checks whether the given literal conforms to the [Location](#Location) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Range$4.is(candidate.range) && (Is$1.string(candidate.uri) || Is$1.undefined(candidate.uri));
    }
    Location.is = is;
})(Location$1 || (Location$1 = {}));
/**
 * The Color namespace provides helper functions to work with
 * [Color](#Color) literals.
 */
var Color$2;
(function (Color) {
    /**
     * Creates a new Color literal.
     */
    function create(red, green, blue, alpha) {
        return {
            red: red,
            green: green,
            blue: blue,
            alpha: alpha,
        };
    }
    Color.create = create;
    /**
     * Checks whether the given literal conforms to the [Color](#Color) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.number(candidate.red)
            && Is$1.number(candidate.green)
            && Is$1.number(candidate.blue)
            && Is$1.number(candidate.alpha);
    }
    Color.is = is;
})(Color$2 || (Color$2 = {}));
/**
 * The ColorInformation namespace provides helper functions to work with
 * [ColorInformation](#ColorInformation) literals.
 */
var ColorInformation$1;
(function (ColorInformation) {
    /**
     * Creates a new ColorInformation literal.
     */
    function create(range, color) {
        return {
            range: range,
            color: color,
        };
    }
    ColorInformation.create = create;
    /**
     * Checks whether the given literal conforms to the [ColorInformation](#ColorInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Range$4.is(candidate.range) && Color$2.is(candidate.color);
    }
    ColorInformation.is = is;
})(ColorInformation$1 || (ColorInformation$1 = {}));
/**
 * The Color namespace provides helper functions to work with
 * [ColorPresentation](#ColorPresentation) literals.
 */
var ColorPresentation$1;
(function (ColorPresentation) {
    /**
     * Creates a new ColorInformation literal.
     */
    function create(label, textEdit, additionalTextEdits) {
        return {
            label: label,
            textEdit: textEdit,
            additionalTextEdits: additionalTextEdits,
        };
    }
    ColorPresentation.create = create;
    /**
     * Checks whether the given literal conforms to the [ColorInformation](#ColorInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.string(candidate.label)
            && (Is$1.undefined(candidate.textEdit) || TextEdit$1.is(candidate))
            && (Is$1.undefined(candidate.additionalTextEdits) || Is$1.typedArray(candidate.additionalTextEdits, TextEdit$1.is));
    }
    ColorPresentation.is = is;
})(ColorPresentation$1 || (ColorPresentation$1 = {}));
/**
 * Enum of known range kinds
 */
var FoldingRangeKind$2;
(function (FoldingRangeKind) {
    /**
     * Folding range for a comment
     */
    FoldingRangeKind["Comment"] = "comment";
    /**
     * Folding range for a imports or includes
     */
    FoldingRangeKind["Imports"] = "imports";
    /**
     * Folding range for a region (e.g. `#region`)
     */
    FoldingRangeKind["Region"] = "region";
})(FoldingRangeKind$2 || (FoldingRangeKind$2 = {}));
/**
 * The folding range namespace provides helper functions to work with
 * [FoldingRange](#FoldingRange) literals.
 */
var FoldingRange$1;
(function (FoldingRange) {
    /**
     * Creates a new FoldingRange literal.
     */
    function create(startLine, endLine, startCharacter, endCharacter, kind) {
        var result = {
            startLine: startLine,
            endLine: endLine
        };
        if (Is$1.defined(startCharacter)) {
            result.startCharacter = startCharacter;
        }
        if (Is$1.defined(endCharacter)) {
            result.endCharacter = endCharacter;
        }
        if (Is$1.defined(kind)) {
            result.kind = kind;
        }
        return result;
    }
    FoldingRange.create = create;
    /**
     * Checks whether the given literal conforms to the [FoldingRange](#FoldingRange) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.number(candidate.startLine) && Is$1.number(candidate.startLine)
            && (Is$1.undefined(candidate.startCharacter) || Is$1.number(candidate.startCharacter))
            && (Is$1.undefined(candidate.endCharacter) || Is$1.number(candidate.endCharacter))
            && (Is$1.undefined(candidate.kind) || Is$1.string(candidate.kind));
    }
    FoldingRange.is = is;
})(FoldingRange$1 || (FoldingRange$1 = {}));
/**
 * The DiagnosticRelatedInformation namespace provides helper functions to work with
 * [DiagnosticRelatedInformation](#DiagnosticRelatedInformation) literals.
 */
var DiagnosticRelatedInformation$1;
(function (DiagnosticRelatedInformation) {
    /**
     * Creates a new DiagnosticRelatedInformation literal.
     */
    function create(location, message) {
        return {
            location: location,
            message: message
        };
    }
    DiagnosticRelatedInformation.create = create;
    /**
     * Checks whether the given literal conforms to the [DiagnosticRelatedInformation](#DiagnosticRelatedInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Location$1.is(candidate.location) && Is$1.string(candidate.message);
    }
    DiagnosticRelatedInformation.is = is;
})(DiagnosticRelatedInformation$1 || (DiagnosticRelatedInformation$1 = {}));
/**
 * The diagnostic's severity.
 */
var DiagnosticSeverity$1;
(function (DiagnosticSeverity) {
    /**
     * Reports an error.
     */
    DiagnosticSeverity.Error = 1;
    /**
     * Reports a warning.
     */
    DiagnosticSeverity.Warning = 2;
    /**
     * Reports an information.
     */
    DiagnosticSeverity.Information = 3;
    /**
     * Reports a hint.
     */
    DiagnosticSeverity.Hint = 4;
})(DiagnosticSeverity$1 || (DiagnosticSeverity$1 = {}));
/**
 * The Diagnostic namespace provides helper functions to work with
 * [Diagnostic](#Diagnostic) literals.
 */
var Diagnostic$1;
(function (Diagnostic) {
    /**
     * Creates a new Diagnostic literal.
     */
    function create(range, message, severity, code, source, relatedInformation) {
        var result = { range: range, message: message };
        if (Is$1.defined(severity)) {
            result.severity = severity;
        }
        if (Is$1.defined(code)) {
            result.code = code;
        }
        if (Is$1.defined(source)) {
            result.source = source;
        }
        if (Is$1.defined(relatedInformation)) {
            result.relatedInformation = relatedInformation;
        }
        return result;
    }
    Diagnostic.create = create;
    /**
     * Checks whether the given literal conforms to the [Diagnostic](#Diagnostic) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate)
            && Range$4.is(candidate.range)
            && Is$1.string(candidate.message)
            && (Is$1.number(candidate.severity) || Is$1.undefined(candidate.severity))
            && (Is$1.number(candidate.code) || Is$1.string(candidate.code) || Is$1.undefined(candidate.code))
            && (Is$1.string(candidate.source) || Is$1.undefined(candidate.source))
            && (Is$1.undefined(candidate.relatedInformation) || Is$1.typedArray(candidate.relatedInformation, DiagnosticRelatedInformation$1.is));
    }
    Diagnostic.is = is;
})(Diagnostic$1 || (Diagnostic$1 = {}));
/**
 * The Command namespace provides helper functions to work with
 * [Command](#Command) literals.
 */
var Command$2;
(function (Command) {
    /**
     * Creates a new Command literal.
     */
    function create(title, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var result = { title: title, command: command };
        if (Is$1.defined(args) && args.length > 0) {
            result.arguments = args;
        }
        return result;
    }
    Command.create = create;
    /**
     * Checks whether the given literal conforms to the [Command](#Command) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Is$1.string(candidate.title) && Is$1.string(candidate.command);
    }
    Command.is = is;
})(Command$2 || (Command$2 = {}));
/**
 * The TextEdit namespace provides helper function to create replace,
 * insert and delete edits more easily.
 */
var TextEdit$1;
(function (TextEdit) {
    /**
     * Creates a replace text edit.
     * @param range The range of text to be replaced.
     * @param newText The new text.
     */
    function replace(range, newText) {
        return { range: range, newText: newText };
    }
    TextEdit.replace = replace;
    /**
     * Creates a insert text edit.
     * @param position The position to insert the text at.
     * @param newText The text to be inserted.
     */
    function insert(position, newText) {
        return { range: { start: position, end: position }, newText: newText };
    }
    TextEdit.insert = insert;
    /**
     * Creates a delete text edit.
     * @param range The range of text to be deleted.
     */
    function del(range) {
        return { range: range, newText: '' };
    }
    TextEdit.del = del;
    function is(value) {
        var candidate = value;
        return Is$1.objectLiteral(candidate)
            && Is$1.string(candidate.newText)
            && Range$4.is(candidate.range);
    }
    TextEdit.is = is;
})(TextEdit$1 || (TextEdit$1 = {}));
/**
 * The TextDocumentEdit namespace provides helper function to create
 * an edit that manipulates a text document.
 */
var TextDocumentEdit$1;
(function (TextDocumentEdit) {
    /**
     * Creates a new `TextDocumentEdit`
     */
    function create(textDocument, edits) {
        return { textDocument: textDocument, edits: edits };
    }
    TextDocumentEdit.create = create;
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate)
            && VersionedTextDocumentIdentifier$1.is(candidate.textDocument)
            && Array.isArray(candidate.edits);
    }
    TextDocumentEdit.is = is;
})(TextDocumentEdit$1 || (TextDocumentEdit$1 = {}));
var WorkspaceEdit$1;
(function (WorkspaceEdit) {
    function is(value) {
        var candidate = value;
        return candidate &&
            (candidate.changes !== void 0 || candidate.documentChanges !== void 0) &&
            (candidate.documentChanges === void 0 || Is$1.typedArray(candidate.documentChanges, TextDocumentEdit$1.is));
    }
    WorkspaceEdit.is = is;
})(WorkspaceEdit$1 || (WorkspaceEdit$1 = {}));
var TextEditChangeImpl$1 = /** @class */ (function () {
    function TextEditChangeImpl(edits) {
        this.edits = edits;
    }
    TextEditChangeImpl.prototype.insert = function (position, newText) {
        this.edits.push(TextEdit$1.insert(position, newText));
    };
    TextEditChangeImpl.prototype.replace = function (range, newText) {
        this.edits.push(TextEdit$1.replace(range, newText));
    };
    TextEditChangeImpl.prototype.delete = function (range) {
        this.edits.push(TextEdit$1.del(range));
    };
    TextEditChangeImpl.prototype.add = function (edit) {
        this.edits.push(edit);
    };
    TextEditChangeImpl.prototype.all = function () {
        return this.edits;
    };
    TextEditChangeImpl.prototype.clear = function () {
        this.edits.splice(0, this.edits.length);
    };
    return TextEditChangeImpl;
}());
/**
 * A workspace change helps constructing changes to a workspace.
 */
var WorkspaceChange$1 = /** @class */ (function () {
    function WorkspaceChange(workspaceEdit) {
        var _this = this;
        this._textEditChanges = Object.create(null);
        if (workspaceEdit) {
            this._workspaceEdit = workspaceEdit;
            if (workspaceEdit.documentChanges) {
                workspaceEdit.documentChanges.forEach(function (textDocumentEdit) {
                    var textEditChange = new TextEditChangeImpl$1(textDocumentEdit.edits);
                    _this._textEditChanges[textDocumentEdit.textDocument.uri] = textEditChange;
                });
            }
            else if (workspaceEdit.changes) {
                Object.keys(workspaceEdit.changes).forEach(function (key) {
                    var textEditChange = new TextEditChangeImpl$1(workspaceEdit.changes[key]);
                    _this._textEditChanges[key] = textEditChange;
                });
            }
        }
    }
    Object.defineProperty(WorkspaceChange.prototype, "edit", {
        /**
         * Returns the underlying [WorkspaceEdit](#WorkspaceEdit) literal
         * use to be returned from a workspace edit operation like rename.
         */
        get: function () {
            return this._workspaceEdit;
        },
        enumerable: true,
        configurable: true
    });
    WorkspaceChange.prototype.getTextEditChange = function (key) {
        if (VersionedTextDocumentIdentifier$1.is(key)) {
            if (!this._workspaceEdit) {
                this._workspaceEdit = {
                    documentChanges: []
                };
            }
            if (!this._workspaceEdit.documentChanges) {
                throw new Error('Workspace edit is not configured for versioned document changes.');
            }
            var textDocument = key;
            var result = this._textEditChanges[textDocument.uri];
            if (!result) {
                var edits = [];
                var textDocumentEdit = {
                    textDocument: textDocument,
                    edits: edits
                };
                this._workspaceEdit.documentChanges.push(textDocumentEdit);
                result = new TextEditChangeImpl$1(edits);
                this._textEditChanges[textDocument.uri] = result;
            }
            return result;
        }
        else {
            if (!this._workspaceEdit) {
                this._workspaceEdit = {
                    changes: Object.create(null)
                };
            }
            if (!this._workspaceEdit.changes) {
                throw new Error('Workspace edit is not configured for normal text edit changes.');
            }
            var result = this._textEditChanges[key];
            if (!result) {
                var edits = [];
                this._workspaceEdit.changes[key] = edits;
                result = new TextEditChangeImpl$1(edits);
                this._textEditChanges[key] = result;
            }
            return result;
        }
    };
    return WorkspaceChange;
}());
/**
 * The TextDocumentIdentifier namespace provides helper functions to work with
 * [TextDocumentIdentifier](#TextDocumentIdentifier) literals.
 */
var TextDocumentIdentifier$1;
(function (TextDocumentIdentifier) {
    /**
     * Creates a new TextDocumentIdentifier literal.
     * @param uri The document's uri.
     */
    function create(uri) {
        return { uri: uri };
    }
    TextDocumentIdentifier.create = create;
    /**
     * Checks whether the given literal conforms to the [TextDocumentIdentifier](#TextDocumentIdentifier) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Is$1.string(candidate.uri);
    }
    TextDocumentIdentifier.is = is;
})(TextDocumentIdentifier$1 || (TextDocumentIdentifier$1 = {}));
/**
 * The VersionedTextDocumentIdentifier namespace provides helper functions to work with
 * [VersionedTextDocumentIdentifier](#VersionedTextDocumentIdentifier) literals.
 */
var VersionedTextDocumentIdentifier$1;
(function (VersionedTextDocumentIdentifier) {
    /**
     * Creates a new VersionedTextDocumentIdentifier literal.
     * @param uri The document's uri.
     * @param uri The document's text.
     */
    function create(uri, version) {
        return { uri: uri, version: version };
    }
    VersionedTextDocumentIdentifier.create = create;
    /**
     * Checks whether the given literal conforms to the [VersionedTextDocumentIdentifier](#VersionedTextDocumentIdentifier) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Is$1.string(candidate.uri) && Is$1.number(candidate.version);
    }
    VersionedTextDocumentIdentifier.is = is;
})(VersionedTextDocumentIdentifier$1 || (VersionedTextDocumentIdentifier$1 = {}));
/**
 * The TextDocumentItem namespace provides helper functions to work with
 * [TextDocumentItem](#TextDocumentItem) literals.
 */
var TextDocumentItem$1;
(function (TextDocumentItem) {
    /**
     * Creates a new TextDocumentItem literal.
     * @param uri The document's uri.
     * @param languageId The document's language identifier.
     * @param version The document's version number.
     * @param text The document's text.
     */
    function create(uri, languageId, version, text) {
        return { uri: uri, languageId: languageId, version: version, text: text };
    }
    TextDocumentItem.create = create;
    /**
     * Checks whether the given literal conforms to the [TextDocumentItem](#TextDocumentItem) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Is$1.string(candidate.uri) && Is$1.string(candidate.languageId) && Is$1.number(candidate.version) && Is$1.string(candidate.text);
    }
    TextDocumentItem.is = is;
})(TextDocumentItem$1 || (TextDocumentItem$1 = {}));
/**
 * Describes the content type that a client supports in various
 * result literals like `Hover`, `ParameterInfo` or `CompletionItem`.
 *
 * Please note that `MarkupKinds` must not start with a `$`. This kinds
 * are reserved for internal usage.
 */
var MarkupKind$1;
(function (MarkupKind) {
    /**
     * Plain text is supported as a content format
     */
    MarkupKind.PlainText = 'plaintext';
    /**
     * Markdown is supported as a content format
     */
    MarkupKind.Markdown = 'markdown';
})(MarkupKind$1 || (MarkupKind$1 = {}));
(function (MarkupKind) {
    /**
     * Checks whether the given value is a value of the [MarkupKind](#MarkupKind) type.
     */
    function is(value) {
        var candidate = value;
        return candidate === MarkupKind.PlainText || candidate === MarkupKind.Markdown;
    }
    MarkupKind.is = is;
})(MarkupKind$1 || (MarkupKind$1 = {}));
var MarkupContent$1;
(function (MarkupContent) {
    /**
     * Checks whether the given value conforms to the [MarkupContent](#MarkupContent) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.objectLiteral(value) && MarkupKind$1.is(candidate.kind) && Is$1.string(candidate.value);
    }
    MarkupContent.is = is;
})(MarkupContent$1 || (MarkupContent$1 = {}));
/**
 * The kind of a completion entry.
 */
var CompletionItemKind$2;
(function (CompletionItemKind) {
    CompletionItemKind.Text = 1;
    CompletionItemKind.Method = 2;
    CompletionItemKind.Function = 3;
    CompletionItemKind.Constructor = 4;
    CompletionItemKind.Field = 5;
    CompletionItemKind.Variable = 6;
    CompletionItemKind.Class = 7;
    CompletionItemKind.Interface = 8;
    CompletionItemKind.Module = 9;
    CompletionItemKind.Property = 10;
    CompletionItemKind.Unit = 11;
    CompletionItemKind.Value = 12;
    CompletionItemKind.Enum = 13;
    CompletionItemKind.Keyword = 14;
    CompletionItemKind.Snippet = 15;
    CompletionItemKind.Color = 16;
    CompletionItemKind.File = 17;
    CompletionItemKind.Reference = 18;
    CompletionItemKind.Folder = 19;
    CompletionItemKind.EnumMember = 20;
    CompletionItemKind.Constant = 21;
    CompletionItemKind.Struct = 22;
    CompletionItemKind.Event = 23;
    CompletionItemKind.Operator = 24;
    CompletionItemKind.TypeParameter = 25;
})(CompletionItemKind$2 || (CompletionItemKind$2 = {}));
/**
 * Defines whether the insert text in a completion item should be interpreted as
 * plain text or a snippet.
 */
var InsertTextFormat$1;
(function (InsertTextFormat) {
    /**
     * The primary text to be inserted is treated as a plain string.
     */
    InsertTextFormat.PlainText = 1;
    /**
     * The primary text to be inserted is treated as a snippet.
     *
     * A snippet can define tab stops and placeholders with `$1`, `$2`
     * and `${3:foo}`. `$0` defines the final tab stop, it defaults to
     * the end of the snippet. Placeholders with equal identifiers are linked,
     * that is typing in one will update others too.
     *
     * See also: https://github.com/Microsoft/vscode/blob/master/src/vs/editor/contrib/snippet/common/snippet.md
     */
    InsertTextFormat.Snippet = 2;
})(InsertTextFormat$1 || (InsertTextFormat$1 = {}));
/**
 * The CompletionItem namespace provides functions to deal with
 * completion items.
 */
var CompletionItem$1;
(function (CompletionItem) {
    /**
     * Create a completion item and seed it with a label.
     * @param label The completion item's label
     */
    function create(label) {
        return { label: label };
    }
    CompletionItem.create = create;
})(CompletionItem$1 || (CompletionItem$1 = {}));
/**
 * The CompletionList namespace provides functions to deal with
 * completion lists.
 */
var CompletionList$1;
(function (CompletionList) {
    /**
     * Creates a new completion list.
     *
     * @param items The completion items.
     * @param isIncomplete The list is not complete.
     */
    function create(items, isIncomplete) {
        return { items: items ? items : [], isIncomplete: !!isIncomplete };
    }
    CompletionList.create = create;
})(CompletionList$1 || (CompletionList$1 = {}));
var MarkedString$1;
(function (MarkedString) {
    /**
     * Creates a marked string from plain text.
     *
     * @param plainText The plain text.
     */
    function fromPlainText(plainText) {
        return plainText.replace(/[\\`*_{}[\]()#+\-.!]/g, "\\$&"); // escape markdown syntax tokens: http://daringfireball.net/projects/markdown/syntax#backslash
    }
    MarkedString.fromPlainText = fromPlainText;
    /**
     * Checks whether the given value conforms to the [MarkedString](#MarkedString) type.
     */
    function is(value) {
        var candidate = value;
        return Is$1.string(candidate) || (Is$1.objectLiteral(candidate) && Is$1.string(candidate.language) && Is$1.string(candidate.value));
    }
    MarkedString.is = is;
})(MarkedString$1 || (MarkedString$1 = {}));
var Hover$1;
(function (Hover) {
    /**
     * Checks whether the given value conforms to the [Hover](#Hover) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.objectLiteral(candidate) && (MarkupContent$1.is(candidate.contents) ||
            MarkedString$1.is(candidate.contents) ||
            Is$1.typedArray(candidate.contents, MarkedString$1.is)) && (value.range === void 0 || Range$4.is(value.range));
    }
    Hover.is = is;
})(Hover$1 || (Hover$1 = {}));
/**
 * The ParameterInformation namespace provides helper functions to work with
 * [ParameterInformation](#ParameterInformation) literals.
 */
var ParameterInformation$1;
(function (ParameterInformation) {
    /**
     * Creates a new parameter information literal.
     *
     * @param label A label string.
     * @param documentation A doc string.
     */
    function create(label, documentation) {
        return documentation ? { label: label, documentation: documentation } : { label: label };
    }
    ParameterInformation.create = create;
})(ParameterInformation$1 || (ParameterInformation$1 = {}));
/**
 * The SignatureInformation namespace provides helper functions to work with
 * [SignatureInformation](#SignatureInformation) literals.
 */
var SignatureInformation$1;
(function (SignatureInformation) {
    function create(label, documentation) {
        var parameters = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            parameters[_i - 2] = arguments[_i];
        }
        var result = { label: label };
        if (Is$1.defined(documentation)) {
            result.documentation = documentation;
        }
        if (Is$1.defined(parameters)) {
            result.parameters = parameters;
        }
        else {
            result.parameters = [];
        }
        return result;
    }
    SignatureInformation.create = create;
})(SignatureInformation$1 || (SignatureInformation$1 = {}));
/**
 * A document highlight kind.
 */
var DocumentHighlightKind$3;
(function (DocumentHighlightKind) {
    /**
     * A textual occurrence.
     */
    DocumentHighlightKind.Text = 1;
    /**
     * Read-access of a symbol, like reading a variable.
     */
    DocumentHighlightKind.Read = 2;
    /**
     * Write-access of a symbol, like writing to a variable.
     */
    DocumentHighlightKind.Write = 3;
})(DocumentHighlightKind$3 || (DocumentHighlightKind$3 = {}));
/**
 * DocumentHighlight namespace to provide helper functions to work with
 * [DocumentHighlight](#DocumentHighlight) literals.
 */
var DocumentHighlight$1;
(function (DocumentHighlight) {
    /**
     * Create a DocumentHighlight object.
     * @param range The range the highlight applies to.
     */
    function create(range, kind) {
        var result = { range: range };
        if (Is$1.number(kind)) {
            result.kind = kind;
        }
        return result;
    }
    DocumentHighlight.create = create;
})(DocumentHighlight$1 || (DocumentHighlight$1 = {}));
/**
 * A symbol kind.
 */
var SymbolKind$2;
(function (SymbolKind) {
    SymbolKind.File = 1;
    SymbolKind.Module = 2;
    SymbolKind.Namespace = 3;
    SymbolKind.Package = 4;
    SymbolKind.Class = 5;
    SymbolKind.Method = 6;
    SymbolKind.Property = 7;
    SymbolKind.Field = 8;
    SymbolKind.Constructor = 9;
    SymbolKind.Enum = 10;
    SymbolKind.Interface = 11;
    SymbolKind.Function = 12;
    SymbolKind.Variable = 13;
    SymbolKind.Constant = 14;
    SymbolKind.String = 15;
    SymbolKind.Number = 16;
    SymbolKind.Boolean = 17;
    SymbolKind.Array = 18;
    SymbolKind.Object = 19;
    SymbolKind.Key = 20;
    SymbolKind.Null = 21;
    SymbolKind.EnumMember = 22;
    SymbolKind.Struct = 23;
    SymbolKind.Event = 24;
    SymbolKind.Operator = 25;
    SymbolKind.TypeParameter = 26;
})(SymbolKind$2 || (SymbolKind$2 = {}));
var SymbolInformation$1;
(function (SymbolInformation) {
    /**
     * Creates a new symbol information literal.
     *
     * @param name The name of the symbol.
     * @param kind The kind of the symbol.
     * @param range The range of the location of the symbol.
     * @param uri The resource of the location of symbol, defaults to the current document.
     * @param containerName The name of the symbol containing the symbol.
     */
    function create(name, kind, range, uri, containerName) {
        var result = {
            name: name,
            kind: kind,
            location: { uri: uri, range: range }
        };
        if (containerName) {
            result.containerName = containerName;
        }
        return result;
    }
    SymbolInformation.create = create;
})(SymbolInformation$1 || (SymbolInformation$1 = {}));
/**
 * Represents programming constructs like variables, classes, interfaces etc.
 * that appear in a document. Document symbols can be hierarchical and they
 * have two ranges: one that encloses its definition and one that points to
 * its most interesting range, e.g. the range of an identifier.
 */
var DocumentSymbol$1 = /** @class */ (function () {
    function DocumentSymbol() {
    }
    return DocumentSymbol;
}());
(function (DocumentSymbol) {
    /**
     * Creates a new symbol information literal.
     *
     * @param name The name of the symbol.
     * @param detail The detail of the symbol.
     * @param kind The kind of the symbol.
     * @param range The range of the symbol.
     * @param selectionRange The selectionRange of the symbol.
     * @param children Children of the symbol.
     */
    function create(name, detail, kind, range, selectionRange, children) {
        var result = {
            name: name,
            detail: detail,
            kind: kind,
            range: range,
            selectionRange: selectionRange
        };
        if (children !== void 0) {
            result.children = children;
        }
        return result;
    }
    DocumentSymbol.create = create;
    /**
     * Checks whether the given literal conforms to the [DocumentSymbol](#DocumentSymbol) interface.
     */
    function is(value) {
        var candidate = value;
        return candidate &&
            Is$1.string(candidate.name) && Is$1.number(candidate.kind) &&
            Range$4.is(candidate.range) && Range$4.is(candidate.selectionRange) &&
            (candidate.detail === void 0 || Is$1.string(candidate.detail)) &&
            (candidate.deprecated === void 0 || Is$1.boolean(candidate.deprecated)) &&
            (candidate.children === void 0 || Array.isArray(candidate.children));
    }
    DocumentSymbol.is = is;
})(DocumentSymbol$1 || (DocumentSymbol$1 = {}));
/**
 * A set of predefined code action kinds
 */
var CodeActionKind$1;
(function (CodeActionKind) {
    /**
     * Base kind for quickfix actions: 'quickfix'
     */
    CodeActionKind.QuickFix = 'quickfix';
    /**
     * Base kind for refactoring actions: 'refactor'
     */
    CodeActionKind.Refactor = 'refactor';
    /**
     * Base kind for refactoring extraction actions: 'refactor.extract'
     *
     * Example extract actions:
     *
     * - Extract method
     * - Extract function
     * - Extract variable
     * - Extract interface from class
     * - ...
     */
    CodeActionKind.RefactorExtract = 'refactor.extract';
    /**
     * Base kind for refactoring inline actions: 'refactor.inline'
     *
     * Example inline actions:
     *
     * - Inline function
     * - Inline variable
     * - Inline constant
     * - ...
     */
    CodeActionKind.RefactorInline = 'refactor.inline';
    /**
     * Base kind for refactoring rewrite actions: 'refactor.rewrite'
     *
     * Example rewrite actions:
     *
     * - Convert JavaScript function to class
     * - Add or remove parameter
     * - Encapsulate field
     * - Make method static
     * - Move method to base class
     * - ...
     */
    CodeActionKind.RefactorRewrite = 'refactor.rewrite';
    /**
     * Base kind for source actions: `source`
     *
     * Source code actions apply to the entire file.
     */
    CodeActionKind.Source = 'source';
    /**
     * Base kind for an organize imports source action: `source.organizeImports`
     */
    CodeActionKind.SourceOrganizeImports = 'source.organizeImports';
})(CodeActionKind$1 || (CodeActionKind$1 = {}));
/**
 * The CodeActionContext namespace provides helper functions to work with
 * [CodeActionContext](#CodeActionContext) literals.
 */
var CodeActionContext$1;
(function (CodeActionContext) {
    /**
     * Creates a new CodeActionContext literal.
     */
    function create(diagnostics, only) {
        var result = { diagnostics: diagnostics };
        if (only !== void 0 && only !== null) {
            result.only = only;
        }
        return result;
    }
    CodeActionContext.create = create;
    /**
     * Checks whether the given literal conforms to the [CodeActionContext](#CodeActionContext) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Is$1.typedArray(candidate.diagnostics, Diagnostic$1.is) && (candidate.only === void 0 || Is$1.typedArray(candidate.only, Is$1.string));
    }
    CodeActionContext.is = is;
})(CodeActionContext$1 || (CodeActionContext$1 = {}));
var CodeAction$1;
(function (CodeAction) {
    function create(title, commandOrEdit, kind) {
        var result = { title: title };
        if (Command$2.is(commandOrEdit)) {
            result.command = commandOrEdit;
        }
        else {
            result.edit = commandOrEdit;
        }
        if (kind !== void null) {
            result.kind = kind;
        }
        return result;
    }
    CodeAction.create = create;
    function is(value) {
        var candidate = value;
        return candidate && Is$1.string(candidate.title) &&
            (candidate.diagnostics === void 0 || Is$1.typedArray(candidate.diagnostics, Diagnostic$1.is)) &&
            (candidate.kind === void 0 || Is$1.string(candidate.kind)) &&
            (candidate.edit !== void 0 || candidate.command !== void 0) &&
            (candidate.command === void 0 || Command$2.is(candidate.command)) &&
            (candidate.edit === void 0 || WorkspaceEdit$1.is(candidate.edit));
    }
    CodeAction.is = is;
})(CodeAction$1 || (CodeAction$1 = {}));
/**
 * The CodeLens namespace provides helper functions to work with
 * [CodeLens](#CodeLens) literals.
 */
var CodeLens$1;
(function (CodeLens) {
    /**
     * Creates a new CodeLens literal.
     */
    function create(range, data) {
        var result = { range: range };
        if (Is$1.defined(data))
            result.data = data;
        return result;
    }
    CodeLens.create = create;
    /**
     * Checks whether the given literal conforms to the [CodeLens](#CodeLens) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Range$4.is(candidate.range) && (Is$1.undefined(candidate.command) || Command$2.is(candidate.command));
    }
    CodeLens.is = is;
})(CodeLens$1 || (CodeLens$1 = {}));
/**
 * The FormattingOptions namespace provides helper functions to work with
 * [FormattingOptions](#FormattingOptions) literals.
 */
var FormattingOptions$1;
(function (FormattingOptions) {
    /**
     * Creates a new FormattingOptions literal.
     */
    function create(tabSize, insertSpaces) {
        return { tabSize: tabSize, insertSpaces: insertSpaces };
    }
    FormattingOptions.create = create;
    /**
     * Checks whether the given literal conforms to the [FormattingOptions](#FormattingOptions) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Is$1.number(candidate.tabSize) && Is$1.boolean(candidate.insertSpaces);
    }
    FormattingOptions.is = is;
})(FormattingOptions$1 || (FormattingOptions$1 = {}));
/**
 * A document link is a range in a text document that links to an internal or external resource, like another
 * text document or a web site.
 */
var DocumentLink$1 = /** @class */ (function () {
    function DocumentLink() {
    }
    return DocumentLink;
}());
/**
 * The DocumentLink namespace provides helper functions to work with
 * [DocumentLink](#DocumentLink) literals.
 */
(function (DocumentLink) {
    /**
     * Creates a new DocumentLink literal.
     */
    function create(range, target, data) {
        return { range: range, target: target, data: data };
    }
    DocumentLink.create = create;
    /**
     * Checks whether the given literal conforms to the [DocumentLink](#DocumentLink) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Range$4.is(candidate.range) && (Is$1.undefined(candidate.target) || Is$1.string(candidate.target));
    }
    DocumentLink.is = is;
})(DocumentLink$1 || (DocumentLink$1 = {}));
var TextDocument$1;
(function (TextDocument) {
    /**
     * Creates a new ITextDocument literal from the given uri and content.
     * @param uri The document's uri.
     * @param languageId  The document's language Id.
     * @param content The document's content.
     */
    function create(uri, languageId, version, content) {
        return new FullTextDocument$1(uri, languageId, version, content);
    }
    TextDocument.create = create;
    /**
     * Checks whether the given literal conforms to the [ITextDocument](#ITextDocument) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$1.defined(candidate) && Is$1.string(candidate.uri) && (Is$1.undefined(candidate.languageId) || Is$1.string(candidate.languageId)) && Is$1.number(candidate.lineCount)
            && Is$1.func(candidate.getText) && Is$1.func(candidate.positionAt) && Is$1.func(candidate.offsetAt) ? true : false;
    }
    TextDocument.is = is;
    function applyEdits(document, edits) {
        var text = document.getText();
        var sortedEdits = mergeSort(edits, function (a, b) {
            var diff = a.range.start.line - b.range.start.line;
            if (diff === 0) {
                return a.range.start.character - b.range.start.character;
            }
            return diff;
        });
        var lastModifiedOffset = text.length;
        for (var i = sortedEdits.length - 1; i >= 0; i--) {
            var e = sortedEdits[i];
            var startOffset = document.offsetAt(e.range.start);
            var endOffset = document.offsetAt(e.range.end);
            if (endOffset <= lastModifiedOffset) {
                text = text.substring(0, startOffset) + e.newText + text.substring(endOffset, text.length);
            }
            else {
                throw new Error('Ovelapping edit');
            }
            lastModifiedOffset = startOffset;
        }
        return text;
    }
    TextDocument.applyEdits = applyEdits;
    function mergeSort(data, compare) {
        if (data.length <= 1) {
            // sorted
            return data;
        }
        var p = (data.length / 2) | 0;
        var left = data.slice(0, p);
        var right = data.slice(p);
        mergeSort(left, compare);
        mergeSort(right, compare);
        var leftIdx = 0;
        var rightIdx = 0;
        var i = 0;
        while (leftIdx < left.length && rightIdx < right.length) {
            var ret = compare(left[leftIdx], right[rightIdx]);
            if (ret <= 0) {
                // smaller_equal -> take left to preserve order
                data[i++] = left[leftIdx++];
            }
            else {
                // greater -> take right
                data[i++] = right[rightIdx++];
            }
        }
        while (leftIdx < left.length) {
            data[i++] = left[leftIdx++];
        }
        while (rightIdx < right.length) {
            data[i++] = right[rightIdx++];
        }
        return data;
    }
})(TextDocument$1 || (TextDocument$1 = {}));
/**
 * Represents reasons why a text document is saved.
 */
var TextDocumentSaveReason$1;
(function (TextDocumentSaveReason) {
    /**
     * Manually triggered, e.g. by the user pressing save, by starting debugging,
     * or by an API call.
     */
    TextDocumentSaveReason.Manual = 1;
    /**
     * Automatic after a delay.
     */
    TextDocumentSaveReason.AfterDelay = 2;
    /**
     * When the editor lost focus.
     */
    TextDocumentSaveReason.FocusOut = 3;
})(TextDocumentSaveReason$1 || (TextDocumentSaveReason$1 = {}));
var FullTextDocument$1 = /** @class */ (function () {
    function FullTextDocument(uri, languageId, version, content) {
        this._uri = uri;
        this._languageId = languageId;
        this._version = version;
        this._content = content;
        this._lineOffsets = null;
    }
    Object.defineProperty(FullTextDocument.prototype, "uri", {
        get: function () {
            return this._uri;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FullTextDocument.prototype, "languageId", {
        get: function () {
            return this._languageId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FullTextDocument.prototype, "version", {
        get: function () {
            return this._version;
        },
        enumerable: true,
        configurable: true
    });
    FullTextDocument.prototype.getText = function (range) {
        if (range) {
            var start = this.offsetAt(range.start);
            var end = this.offsetAt(range.end);
            return this._content.substring(start, end);
        }
        return this._content;
    };
    FullTextDocument.prototype.update = function (event, version) {
        this._content = event.text;
        this._version = version;
        this._lineOffsets = null;
    };
    FullTextDocument.prototype.getLineOffsets = function () {
        if (this._lineOffsets === null) {
            var lineOffsets = [];
            var text = this._content;
            var isLineStart = true;
            for (var i = 0; i < text.length; i++) {
                if (isLineStart) {
                    lineOffsets.push(i);
                    isLineStart = false;
                }
                var ch = text.charAt(i);
                isLineStart = (ch === '\r' || ch === '\n');
                if (ch === '\r' && i + 1 < text.length && text.charAt(i + 1) === '\n') {
                    i++;
                }
            }
            if (isLineStart && text.length > 0) {
                lineOffsets.push(text.length);
            }
            this._lineOffsets = lineOffsets;
        }
        return this._lineOffsets;
    };
    FullTextDocument.prototype.positionAt = function (offset) {
        offset = Math.max(Math.min(offset, this._content.length), 0);
        var lineOffsets = this.getLineOffsets();
        var low = 0, high = lineOffsets.length;
        if (high === 0) {
            return Position$3.create(0, offset);
        }
        while (low < high) {
            var mid = Math.floor((low + high) / 2);
            if (lineOffsets[mid] > offset) {
                high = mid;
            }
            else {
                low = mid + 1;
            }
        }
        // low is the least x for which the line offset is larger than the current offset
        // or array.length if no line offset is larger than the current offset
        var line = low - 1;
        return Position$3.create(line, offset - lineOffsets[line]);
    };
    FullTextDocument.prototype.offsetAt = function (position) {
        var lineOffsets = this.getLineOffsets();
        if (position.line >= lineOffsets.length) {
            return this._content.length;
        }
        else if (position.line < 0) {
            return 0;
        }
        var lineOffset = lineOffsets[position.line];
        var nextLineOffset = (position.line + 1 < lineOffsets.length) ? lineOffsets[position.line + 1] : this._content.length;
        return Math.max(Math.min(lineOffset + position.character, nextLineOffset), lineOffset);
    };
    Object.defineProperty(FullTextDocument.prototype, "lineCount", {
        get: function () {
            return this.getLineOffsets().length;
        },
        enumerable: true,
        configurable: true
    });
    return FullTextDocument;
}());
var Is$1;
(function (Is) {
    var toString = Object.prototype.toString;
    function defined(value) {
        return typeof value !== 'undefined';
    }
    Is.defined = defined;
    function undefined(value) {
        return typeof value === 'undefined';
    }
    Is.undefined = undefined;
    function boolean(value) {
        return value === true || value === false;
    }
    Is.boolean = boolean;
    function string(value) {
        return toString.call(value) === '[object String]';
    }
    Is.string = string;
    function number(value) {
        return toString.call(value) === '[object Number]';
    }
    Is.number = number;
    function func(value) {
        return toString.call(value) === '[object Function]';
    }
    Is.func = func;
    function objectLiteral(value) {
        // Strictly speaking class instances pass this check as well. Since the LSP
        // doesn't use classes we ignore this for now. If we do we need to add something
        // like this: `Object.getPrototypeOf(Object.getPrototypeOf(x)) === null`
        return value !== null && typeof value === 'object';
    }
    Is.objectLiteral = objectLiteral;
    function typedArray(value, check) {
        return Array.isArray(value) && value.every(check);
    }
    Is.typedArray = typedArray;
})(Is$1 || (Is$1 = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Uri$3 = monaco.Uri;
var Range$5 = monaco.Range;
// --- diagnostics --- ---
var DiagnosticsAdapter$1 = /** @class */ (function () {
    function DiagnosticsAdapter(_languageId, _worker, defaults) {
        var _this = this;
        this._languageId = _languageId;
        this._worker = _worker;
        this._disposables = [];
        this._listener = Object.create(null);
        var onModelAdd = function (model) {
            var modeId = model.getModeId();
            if (modeId !== _this._languageId) {
                return;
            }
            var handle;
            _this._listener[model.uri.toString()] = model.onDidChangeContent(function () {
                clearTimeout(handle);
                handle = setTimeout(function () { return _this._doValidate(model.uri, modeId); }, 500);
            });
            _this._doValidate(model.uri, modeId);
        };
        var onModelRemoved = function (model) {
            monaco.editor.setModelMarkers(model, _this._languageId, []);
            var uriStr = model.uri.toString();
            var listener = _this._listener[uriStr];
            if (listener) {
                listener.dispose();
                delete _this._listener[uriStr];
            }
        };
        this._disposables.push(monaco.editor.onDidCreateModel(onModelAdd));
        this._disposables.push(monaco.editor.onWillDisposeModel(function (model) {
            onModelRemoved(model);
            _this._resetSchema(model.uri);
        }));
        this._disposables.push(monaco.editor.onDidChangeModelLanguage(function (event) {
            onModelRemoved(event.model);
            onModelAdd(event.model);
            _this._resetSchema(event.model.uri);
        }));
        this._disposables.push(defaults.onDidChange(function (_) {
            monaco.editor.getModels().forEach(function (model) {
                if (model.getModeId() === _this._languageId) {
                    onModelRemoved(model);
                    onModelAdd(model);
                }
            });
        }));
        this._disposables.push({
            dispose: function () {
                monaco.editor.getModels().forEach(onModelRemoved);
                for (var key in _this._listener) {
                    _this._listener[key].dispose();
                }
            }
        });
        monaco.editor.getModels().forEach(onModelAdd);
    }
    DiagnosticsAdapter.prototype.dispose = function () {
        this._disposables.forEach(function (d) { return d && d.dispose(); });
        this._disposables = [];
    };
    DiagnosticsAdapter.prototype._resetSchema = function (resource) {
        this._worker().then(function (worker) {
            worker.resetSchema(resource.toString());
        });
    };
    DiagnosticsAdapter.prototype._doValidate = function (resource, languageId) {
        this._worker(resource).then(function (worker) {
            return worker.doValidation(resource.toString()).then(function (diagnostics) {
                var markers = diagnostics.map(function (d) { return toDiagnostics$1(resource, d); });
                var model = monaco.editor.getModel(resource);
                if (model.getModeId() === languageId) {
                    monaco.editor.setModelMarkers(model, languageId, markers);
                }
            });
        }).then(undefined, function (err) {
            console.error(err);
        });
    };
    return DiagnosticsAdapter;
}());
function toSeverity$1(lsSeverity) {
    switch (lsSeverity) {
        case DiagnosticSeverity$1.Error: return monaco.MarkerSeverity.Error;
        case DiagnosticSeverity$1.Warning: return monaco.MarkerSeverity.Warning;
        case DiagnosticSeverity$1.Information: return monaco.MarkerSeverity.Info;
        case DiagnosticSeverity$1.Hint: return monaco.MarkerSeverity.Hint;
        default:
            return monaco.MarkerSeverity.Info;
    }
}
function toDiagnostics$1(resource, diag) {
    var code = typeof diag.code === 'number' ? String(diag.code) : diag.code;
    return {
        severity: toSeverity$1(diag.severity),
        startLineNumber: diag.range.start.line + 1,
        startColumn: diag.range.start.character + 1,
        endLineNumber: diag.range.end.line + 1,
        endColumn: diag.range.end.character + 1,
        message: diag.message,
        code: code,
        source: diag.source
    };
}
// --- completion ------
function fromPosition$1(position) {
    if (!position) {
        return void 0;
    }
    return { character: position.column - 1, line: position.lineNumber - 1 };
}
function fromRange$1(range) {
    if (!range) {
        return void 0;
    }
    return { start: { line: range.startLineNumber - 1, character: range.startColumn - 1 }, end: { line: range.endLineNumber - 1, character: range.endColumn - 1 } };
}
function toRange$1(range) {
    if (!range) {
        return void 0;
    }
    return new Range$5(range.start.line + 1, range.start.character + 1, range.end.line + 1, range.end.character + 1);
}
function toCompletionItemKind$1(kind) {
    var mItemKind = monaco.languages.CompletionItemKind;
    switch (kind) {
        case CompletionItemKind$2.Text: return mItemKind.Text;
        case CompletionItemKind$2.Method: return mItemKind.Method;
        case CompletionItemKind$2.Function: return mItemKind.Function;
        case CompletionItemKind$2.Constructor: return mItemKind.Constructor;
        case CompletionItemKind$2.Field: return mItemKind.Field;
        case CompletionItemKind$2.Variable: return mItemKind.Variable;
        case CompletionItemKind$2.Class: return mItemKind.Class;
        case CompletionItemKind$2.Interface: return mItemKind.Interface;
        case CompletionItemKind$2.Module: return mItemKind.Module;
        case CompletionItemKind$2.Property: return mItemKind.Property;
        case CompletionItemKind$2.Unit: return mItemKind.Unit;
        case CompletionItemKind$2.Value: return mItemKind.Value;
        case CompletionItemKind$2.Enum: return mItemKind.Enum;
        case CompletionItemKind$2.Keyword: return mItemKind.Keyword;
        case CompletionItemKind$2.Snippet: return mItemKind.Snippet;
        case CompletionItemKind$2.Color: return mItemKind.Color;
        case CompletionItemKind$2.File: return mItemKind.File;
        case CompletionItemKind$2.Reference: return mItemKind.Reference;
    }
    return mItemKind.Property;
}
function toTextEdit$1(textEdit) {
    if (!textEdit) {
        return void 0;
    }
    return {
        range: toRange$1(textEdit.range),
        text: textEdit.newText
    };
}
var CompletionAdapter$1 = /** @class */ (function () {
    function CompletionAdapter(_worker) {
        this._worker = _worker;
    }
    Object.defineProperty(CompletionAdapter.prototype, "triggerCharacters", {
        get: function () {
            return [' ', ':'];
        },
        enumerable: true,
        configurable: true
    });
    CompletionAdapter.prototype.provideCompletionItems = function (model, position, context, token) {
        var wordInfo = model.getWordUntilPosition(position);
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.doComplete(resource.toString(), fromPosition$1(position));
        }).then(function (info) {
            if (!info) {
                return;
            }
            var items = info.items.map(function (entry) {
                var item = {
                    label: entry.label,
                    insertText: entry.insertText || entry.label,
                    sortText: entry.sortText,
                    filterText: entry.filterText,
                    documentation: entry.documentation,
                    detail: entry.detail,
                    kind: toCompletionItemKind$1(entry.kind),
                };
                if (entry.textEdit) {
                    item.range = toRange$1(entry.textEdit.range);
                    item.insertText = entry.textEdit.newText;
                }
                if (entry.additionalTextEdits) {
                    item.additionalTextEdits = entry.additionalTextEdits.map(toTextEdit$1);
                }
                if (entry.insertTextFormat === InsertTextFormat$1.Snippet) {
                    item.insertTextRules = monaco.languages.CompletionItemInsertTextRule.InsertAsSnippet;
                }
                return item;
            });
            return {
                isIncomplete: info.isIncomplete,
                suggestions: items
            };
        });
    };
    return CompletionAdapter;
}());
function isMarkupContent$1(thing) {
    return thing && typeof thing === 'object' && typeof thing.kind === 'string';
}
function toMarkdownString$1(entry) {
    if (typeof entry === 'string') {
        return {
            value: entry
        };
    }
    if (isMarkupContent$1(entry)) {
        if (entry.kind === 'plaintext') {
            return {
                value: entry.value.replace(/[\\`*_{}[\]()#+\-.!]/g, '\\$&')
            };
        }
        return {
            value: entry.value
        };
    }
    return { value: '```' + entry.language + '\n' + entry.value + '\n```\n' };
}
function toMarkedStringArray$1(contents) {
    if (!contents) {
        return void 0;
    }
    if (Array.isArray(contents)) {
        return contents.map(toMarkdownString$1);
    }
    return [toMarkdownString$1(contents)];
}
// --- hover ------
var HoverAdapter$1 = /** @class */ (function () {
    function HoverAdapter(_worker) {
        this._worker = _worker;
    }
    HoverAdapter.prototype.provideHover = function (model, position, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.doHover(resource.toString(), fromPosition$1(position));
        }).then(function (info) {
            if (!info) {
                return;
            }
            return {
                range: toRange$1(info.range),
                contents: toMarkedStringArray$1(info.contents)
            };
        });
    };
    return HoverAdapter;
}());
// --- document symbols ------
function toSymbolKind$1(kind) {
    var mKind = monaco.languages.SymbolKind;
    switch (kind) {
        case SymbolKind$2.File: return mKind.Array;
        case SymbolKind$2.Module: return mKind.Module;
        case SymbolKind$2.Namespace: return mKind.Namespace;
        case SymbolKind$2.Package: return mKind.Package;
        case SymbolKind$2.Class: return mKind.Class;
        case SymbolKind$2.Method: return mKind.Method;
        case SymbolKind$2.Property: return mKind.Property;
        case SymbolKind$2.Field: return mKind.Field;
        case SymbolKind$2.Constructor: return mKind.Constructor;
        case SymbolKind$2.Enum: return mKind.Enum;
        case SymbolKind$2.Interface: return mKind.Interface;
        case SymbolKind$2.Function: return mKind.Function;
        case SymbolKind$2.Variable: return mKind.Variable;
        case SymbolKind$2.Constant: return mKind.Constant;
        case SymbolKind$2.String: return mKind.String;
        case SymbolKind$2.Number: return mKind.Number;
        case SymbolKind$2.Boolean: return mKind.Boolean;
        case SymbolKind$2.Array: return mKind.Array;
    }
    return mKind.Function;
}
var DocumentSymbolAdapter$1 = /** @class */ (function () {
    function DocumentSymbolAdapter(_worker) {
        this._worker = _worker;
    }
    DocumentSymbolAdapter.prototype.provideDocumentSymbols = function (model, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.findDocumentSymbols(resource.toString()); }).then(function (items) {
            if (!items) {
                return;
            }
            return items.map(function (item) { return ({
                name: item.name,
                detail: '',
                containerName: item.containerName,
                kind: toSymbolKind$1(item.kind),
                range: toRange$1(item.location.range),
                selectionRange: toRange$1(item.location.range)
            }); });
        });
    };
    return DocumentSymbolAdapter;
}());
function fromFormattingOptions(options) {
    return {
        tabSize: options.tabSize,
        insertSpaces: options.insertSpaces
    };
}
var DocumentFormattingEditProvider = /** @class */ (function () {
    function DocumentFormattingEditProvider(_worker) {
        this._worker = _worker;
    }
    DocumentFormattingEditProvider.prototype.provideDocumentFormattingEdits = function (model, options, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.format(resource.toString(), null, fromFormattingOptions(options)).then(function (edits) {
                if (!edits || edits.length === 0) {
                    return;
                }
                return edits.map(toTextEdit$1);
            });
        });
    };
    return DocumentFormattingEditProvider;
}());
var DocumentRangeFormattingEditProvider = /** @class */ (function () {
    function DocumentRangeFormattingEditProvider(_worker) {
        this._worker = _worker;
    }
    DocumentRangeFormattingEditProvider.prototype.provideDocumentRangeFormattingEdits = function (model, range, options, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.format(resource.toString(), fromRange$1(range), fromFormattingOptions(options)).then(function (edits) {
                if (!edits || edits.length === 0) {
                    return;
                }
                return edits.map(toTextEdit$1);
            });
        });
    };
    return DocumentRangeFormattingEditProvider;
}());
var DocumentColorAdapter$1 = /** @class */ (function () {
    function DocumentColorAdapter(_worker) {
        this._worker = _worker;
    }
    DocumentColorAdapter.prototype.provideDocumentColors = function (model, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.findDocumentColors(resource.toString()); }).then(function (infos) {
            if (!infos) {
                return;
            }
            return infos.map(function (item) { return ({
                color: item.color,
                range: toRange$1(item.range)
            }); });
        });
    };
    DocumentColorAdapter.prototype.provideColorPresentations = function (model, info, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.getColorPresentations(resource.toString(), info.color, fromRange$1(info.range)); }).then(function (presentations) {
            if (!presentations) {
                return;
            }
            return presentations.map(function (presentation) {
                var item = {
                    label: presentation.label,
                };
                if (presentation.textEdit) {
                    item.textEdit = toTextEdit$1(presentation.textEdit);
                }
                if (presentation.additionalTextEdits) {
                    item.additionalTextEdits = presentation.additionalTextEdits.map(toTextEdit$1);
                }
                return item;
            });
        });
    };
    return DocumentColorAdapter;
}());
var FoldingRangeAdapter$1 = /** @class */ (function () {
    function FoldingRangeAdapter(_worker) {
        this._worker = _worker;
    }
    FoldingRangeAdapter.prototype.provideFoldingRanges = function (model, context, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.provideFoldingRanges(resource.toString(), context); }).then(function (ranges) {
            if (!ranges) {
                return;
            }
            return ranges.map(function (range) {
                var result = {
                    start: range.startLine + 1,
                    end: range.endLine + 1
                };
                if (typeof range.kind !== 'undefined') {
                    result.kind = toFoldingRangeKind$1(range.kind);
                }
                return result;
            });
        });
    };
    return FoldingRangeAdapter;
}());
function toFoldingRangeKind$1(kind) {
    switch (kind) {
        case FoldingRangeKind$2.Comment: return monaco.languages.FoldingRangeKind.Comment;
        case FoldingRangeKind$2.Imports: return monaco.languages.FoldingRangeKind.Imports;
        case FoldingRangeKind$2.Region: return monaco.languages.FoldingRangeKind.Region;
    }
    return void 0;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Creates a JSON scanner on the given text.
 * If ignoreTrivia is set, whitespaces or comments are ignored.
 */
function createScanner(text, ignoreTrivia) {
    if (ignoreTrivia === void 0) { ignoreTrivia = false; }
    var pos = 0, len = text.length, value = '', tokenOffset = 0, token = 16 /* Unknown */, scanError = 0 /* None */;
    function scanHexDigits(count, exact) {
        var digits = 0;
        var value = 0;
        while (digits < count || !exact) {
            var ch = text.charCodeAt(pos);
            if (ch >= 48 /* _0 */ && ch <= 57 /* _9 */) {
                value = value * 16 + ch - 48 /* _0 */;
            }
            else if (ch >= 65 /* A */ && ch <= 70 /* F */) {
                value = value * 16 + ch - 65 /* A */ + 10;
            }
            else if (ch >= 97 /* a */ && ch <= 102 /* f */) {
                value = value * 16 + ch - 97 /* a */ + 10;
            }
            else {
                break;
            }
            pos++;
            digits++;
        }
        if (digits < count) {
            value = -1;
        }
        return value;
    }
    function setPosition(newPosition) {
        pos = newPosition;
        value = '';
        tokenOffset = 0;
        token = 16 /* Unknown */;
        scanError = 0 /* None */;
    }
    function scanNumber() {
        var start = pos;
        if (text.charCodeAt(pos) === 48 /* _0 */) {
            pos++;
        }
        else {
            pos++;
            while (pos < text.length && isDigit(text.charCodeAt(pos))) {
                pos++;
            }
        }
        if (pos < text.length && text.charCodeAt(pos) === 46 /* dot */) {
            pos++;
            if (pos < text.length && isDigit(text.charCodeAt(pos))) {
                pos++;
                while (pos < text.length && isDigit(text.charCodeAt(pos))) {
                    pos++;
                }
            }
            else {
                scanError = 3 /* UnexpectedEndOfNumber */;
                return text.substring(start, pos);
            }
        }
        var end = pos;
        if (pos < text.length && (text.charCodeAt(pos) === 69 /* E */ || text.charCodeAt(pos) === 101 /* e */)) {
            pos++;
            if (pos < text.length && text.charCodeAt(pos) === 43 /* plus */ || text.charCodeAt(pos) === 45 /* minus */) {
                pos++;
            }
            if (pos < text.length && isDigit(text.charCodeAt(pos))) {
                pos++;
                while (pos < text.length && isDigit(text.charCodeAt(pos))) {
                    pos++;
                }
                end = pos;
            }
            else {
                scanError = 3 /* UnexpectedEndOfNumber */;
            }
        }
        return text.substring(start, end);
    }
    function scanString() {
        var result = '', start = pos;
        while (true) {
            if (pos >= len) {
                result += text.substring(start, pos);
                scanError = 2 /* UnexpectedEndOfString */;
                break;
            }
            var ch = text.charCodeAt(pos);
            if (ch === 34 /* doubleQuote */) {
                result += text.substring(start, pos);
                pos++;
                break;
            }
            if (ch === 92 /* backslash */) {
                result += text.substring(start, pos);
                pos++;
                if (pos >= len) {
                    scanError = 2 /* UnexpectedEndOfString */;
                    break;
                }
                ch = text.charCodeAt(pos++);
                switch (ch) {
                    case 34 /* doubleQuote */:
                        result += '\"';
                        break;
                    case 92 /* backslash */:
                        result += '\\';
                        break;
                    case 47 /* slash */:
                        result += '/';
                        break;
                    case 98 /* b */:
                        result += '\b';
                        break;
                    case 102 /* f */:
                        result += '\f';
                        break;
                    case 110 /* n */:
                        result += '\n';
                        break;
                    case 114 /* r */:
                        result += '\r';
                        break;
                    case 116 /* t */:
                        result += '\t';
                        break;
                    case 117 /* u */:
                        var ch_1 = scanHexDigits(4, true);
                        if (ch_1 >= 0) {
                            result += String.fromCharCode(ch_1);
                        }
                        else {
                            scanError = 4 /* InvalidUnicode */;
                        }
                        break;
                    default:
                        scanError = 5 /* InvalidEscapeCharacter */;
                }
                start = pos;
                continue;
            }
            if (ch >= 0 && ch <= 0x1f) {
                if (isLineBreak(ch)) {
                    result += text.substring(start, pos);
                    scanError = 2 /* UnexpectedEndOfString */;
                    break;
                }
                else {
                    scanError = 6 /* InvalidCharacter */;
                    // mark as error but continue with string
                }
            }
            pos++;
        }
        return result;
    }
    function scanNext() {
        value = '';
        scanError = 0 /* None */;
        tokenOffset = pos;
        if (pos >= len) {
            // at the end
            tokenOffset = len;
            return token = 17 /* EOF */;
        }
        var code = text.charCodeAt(pos);
        // trivia: whitespace
        if (isWhiteSpace(code)) {
            do {
                pos++;
                value += String.fromCharCode(code);
                code = text.charCodeAt(pos);
            } while (isWhiteSpace(code));
            return token = 15 /* Trivia */;
        }
        // trivia: newlines
        if (isLineBreak(code)) {
            pos++;
            value += String.fromCharCode(code);
            if (code === 13 /* carriageReturn */ && text.charCodeAt(pos) === 10 /* lineFeed */) {
                pos++;
                value += '\n';
            }
            return token = 14 /* LineBreakTrivia */;
        }
        switch (code) {
            // tokens: []{}:,
            case 123 /* openBrace */:
                pos++;
                return token = 1 /* OpenBraceToken */;
            case 125 /* closeBrace */:
                pos++;
                return token = 2 /* CloseBraceToken */;
            case 91 /* openBracket */:
                pos++;
                return token = 3 /* OpenBracketToken */;
            case 93 /* closeBracket */:
                pos++;
                return token = 4 /* CloseBracketToken */;
            case 58 /* colon */:
                pos++;
                return token = 6 /* ColonToken */;
            case 44 /* comma */:
                pos++;
                return token = 5 /* CommaToken */;
            // strings
            case 34 /* doubleQuote */:
                pos++;
                value = scanString();
                return token = 10 /* StringLiteral */;
            // comments
            case 47 /* slash */:
                var start = pos - 1;
                // Single-line comment
                if (text.charCodeAt(pos + 1) === 47 /* slash */) {
                    pos += 2;
                    while (pos < len) {
                        if (isLineBreak(text.charCodeAt(pos))) {
                            break;
                        }
                        pos++;
                    }
                    value = text.substring(start, pos);
                    return token = 12 /* LineCommentTrivia */;
                }
                // Multi-line comment
                if (text.charCodeAt(pos + 1) === 42 /* asterisk */) {
                    pos += 2;
                    var commentClosed = false;
                    while (pos < len) {
                        var ch = text.charCodeAt(pos);
                        if (ch === 42 /* asterisk */ && (pos + 1 < len) && text.charCodeAt(pos + 1) === 47 /* slash */) {
                            pos += 2;
                            commentClosed = true;
                            break;
                        }
                        pos++;
                    }
                    if (!commentClosed) {
                        pos++;
                        scanError = 1 /* UnexpectedEndOfComment */;
                    }
                    value = text.substring(start, pos);
                    return token = 13 /* BlockCommentTrivia */;
                }
                // just a single slash
                value += String.fromCharCode(code);
                pos++;
                return token = 16 /* Unknown */;
            // numbers
            case 45 /* minus */:
                value += String.fromCharCode(code);
                pos++;
                if (pos === len || !isDigit(text.charCodeAt(pos))) {
                    return token = 16 /* Unknown */;
                }
            // found a minus, followed by a number so
            // we fall through to proceed with scanning
            // numbers
            case 48 /* _0 */:
            case 49 /* _1 */:
            case 50 /* _2 */:
            case 51 /* _3 */:
            case 52 /* _4 */:
            case 53 /* _5 */:
            case 54 /* _6 */:
            case 55 /* _7 */:
            case 56 /* _8 */:
            case 57 /* _9 */:
                value += scanNumber();
                return token = 11 /* NumericLiteral */;
            // literals and unknown symbols
            default:
                // is a literal? Read the full word.
                while (pos < len && isUnknownContentCharacter(code)) {
                    pos++;
                    code = text.charCodeAt(pos);
                }
                if (tokenOffset !== pos) {
                    value = text.substring(tokenOffset, pos);
                    // keywords: true, false, null
                    switch (value) {
                        case 'true': return token = 8 /* TrueKeyword */;
                        case 'false': return token = 9 /* FalseKeyword */;
                        case 'null': return token = 7 /* NullKeyword */;
                    }
                    return token = 16 /* Unknown */;
                }
                // some
                value += String.fromCharCode(code);
                pos++;
                return token = 16 /* Unknown */;
        }
    }
    function isUnknownContentCharacter(code) {
        if (isWhiteSpace(code) || isLineBreak(code)) {
            return false;
        }
        switch (code) {
            case 125 /* closeBrace */:
            case 93 /* closeBracket */:
            case 123 /* openBrace */:
            case 91 /* openBracket */:
            case 34 /* doubleQuote */:
            case 58 /* colon */:
            case 44 /* comma */:
            case 47 /* slash */:
                return false;
        }
        return true;
    }
    function scanNextNonTrivia() {
        var result;
        do {
            result = scanNext();
        } while (result >= 12 /* LineCommentTrivia */ && result <= 15 /* Trivia */);
        return result;
    }
    return {
        setPosition: setPosition,
        getPosition: function () { return pos; },
        scan: ignoreTrivia ? scanNextNonTrivia : scanNext,
        getToken: function () { return token; },
        getTokenValue: function () { return value; },
        getTokenOffset: function () { return tokenOffset; },
        getTokenLength: function () { return pos - tokenOffset; },
        getTokenError: function () { return scanError; }
    };
}
function isWhiteSpace(ch) {
    return ch === 32 /* space */ || ch === 9 /* tab */ || ch === 11 /* verticalTab */ || ch === 12 /* formFeed */ ||
        ch === 160 /* nonBreakingSpace */ || ch === 5760 /* ogham */ || ch >= 8192 /* enQuad */ && ch <= 8203 /* zeroWidthSpace */ ||
        ch === 8239 /* narrowNoBreakSpace */ || ch === 8287 /* mathematicalSpace */ || ch === 12288 /* ideographicSpace */ || ch === 65279 /* byteOrderMark */;
}
function isLineBreak(ch) {
    return ch === 10 /* lineFeed */ || ch === 13 /* carriageReturn */ || ch === 8232 /* lineSeparator */ || ch === 8233 /* paragraphSeparator */;
}
function isDigit(ch) {
    return ch >= 48 /* _0 */ && ch <= 57 /* _9 */;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/**
 * Creates a JSON scanner on the given text.
 * If ignoreTrivia is set, whitespaces or comments are ignored.
 */
var createScanner$1 = createScanner;

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function createTokenizationSupport$1(supportComments) {
    return {
        getInitialState: function () { return new JSONState(null, null, false); },
        tokenize: function (line, state, offsetDelta, stopAtOffset) { return tokenize$1(supportComments, line, state, offsetDelta, stopAtOffset); }
    };
}
var TOKEN_DELIM_OBJECT = 'delimiter.bracket.json';
var TOKEN_DELIM_ARRAY = 'delimiter.array.json';
var TOKEN_DELIM_COLON = 'delimiter.colon.json';
var TOKEN_DELIM_COMMA = 'delimiter.comma.json';
var TOKEN_VALUE_BOOLEAN = 'keyword.json';
var TOKEN_VALUE_NULL = 'keyword.json';
var TOKEN_VALUE_STRING = 'string.value.json';
var TOKEN_VALUE_NUMBER = 'number.json';
var TOKEN_PROPERTY_NAME = 'string.key.json';
var TOKEN_COMMENT_BLOCK = 'comment.block.json';
var TOKEN_COMMENT_LINE = 'comment.line.json';
var JSONState = /** @class */ (function () {
    function JSONState(state, scanError, lastWasColon) {
        this._state = state;
        this.scanError = scanError;
        this.lastWasColon = lastWasColon;
    }
    JSONState.prototype.clone = function () {
        return new JSONState(this._state, this.scanError, this.lastWasColon);
    };
    JSONState.prototype.equals = function (other) {
        if (other === this) {
            return true;
        }
        if (!other || !(other instanceof JSONState)) {
            return false;
        }
        return this.scanError === other.scanError &&
            this.lastWasColon === other.lastWasColon;
    };
    JSONState.prototype.getStateData = function () {
        return this._state;
    };
    JSONState.prototype.setStateData = function (state) {
        this._state = state;
    };
    return JSONState;
}());
function tokenize$1(comments, line, state, offsetDelta, stopAtOffset) {
    if (offsetDelta === void 0) { offsetDelta = 0; }
    // handle multiline strings and block comments
    var numberOfInsertedCharacters = 0, adjustOffset = false;
    switch (state.scanError) {
        case 2 /* UnexpectedEndOfString */:
            line = '"' + line;
            numberOfInsertedCharacters = 1;
            break;
        case 1 /* UnexpectedEndOfComment */:
            line = '/*' + line;
            numberOfInsertedCharacters = 2;
            break;
    }
    var scanner = createScanner$1(line), kind, ret, lastWasColon = state.lastWasColon;
    ret = {
        tokens: [],
        endState: state.clone()
    };
    while (true) {
        var offset = offsetDelta + scanner.getPosition(), type = '';
        kind = scanner.scan();
        if (kind === 17 /* EOF */) {
            break;
        }
        // Check that the scanner has advanced
        if (offset === offsetDelta + scanner.getPosition()) {
            throw new Error('Scanner did not advance, next 3 characters are: ' + line.substr(scanner.getPosition(), 3));
        }
        // In case we inserted /* or " character, we need to
        // adjust the offset of all tokens (except the first)
        if (adjustOffset) {
            offset -= numberOfInsertedCharacters;
        }
        adjustOffset = numberOfInsertedCharacters > 0;
        // brackets and type
        switch (kind) {
            case 1 /* OpenBraceToken */:
                type = TOKEN_DELIM_OBJECT;
                lastWasColon = false;
                break;
            case 2 /* CloseBraceToken */:
                type = TOKEN_DELIM_OBJECT;
                lastWasColon = false;
                break;
            case 3 /* OpenBracketToken */:
                type = TOKEN_DELIM_ARRAY;
                lastWasColon = false;
                break;
            case 4 /* CloseBracketToken */:
                type = TOKEN_DELIM_ARRAY;
                lastWasColon = false;
                break;
            case 6 /* ColonToken */:
                type = TOKEN_DELIM_COLON;
                lastWasColon = true;
                break;
            case 5 /* CommaToken */:
                type = TOKEN_DELIM_COMMA;
                lastWasColon = false;
                break;
            case 8 /* TrueKeyword */:
            case 9 /* FalseKeyword */:
                type = TOKEN_VALUE_BOOLEAN;
                lastWasColon = false;
                break;
            case 7 /* NullKeyword */:
                type = TOKEN_VALUE_NULL;
                lastWasColon = false;
                break;
            case 10 /* StringLiteral */:
                type = lastWasColon ? TOKEN_VALUE_STRING : TOKEN_PROPERTY_NAME;
                lastWasColon = false;
                break;
            case 11 /* NumericLiteral */:
                type = TOKEN_VALUE_NUMBER;
                lastWasColon = false;
                break;
        }
        // comments, iff enabled
        if (comments) {
            switch (kind) {
                case 12 /* LineCommentTrivia */:
                    type = TOKEN_COMMENT_LINE;
                    break;
                case 13 /* BlockCommentTrivia */:
                    type = TOKEN_COMMENT_BLOCK;
                    break;
            }
        }
        ret.endState = new JSONState(state.getStateData(), scanner.getTokenError(), lastWasColon);
        ret.tokens.push({
            startIndex: offset,
            scopes: type
        });
    }
    return ret;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function setupMode$2(defaults) {
    var disposables = [];
    var client = new WorkerManager$3(defaults);
    disposables.push(client);
    var worker = function () {
        var uris = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            uris[_i] = arguments[_i];
        }
        return client.getLanguageServiceWorker.apply(client, uris);
    };
    var languageId = defaults.languageId;
    disposables.push(monaco.languages.registerCompletionItemProvider(languageId, new CompletionAdapter$1(worker)));
    disposables.push(monaco.languages.registerHoverProvider(languageId, new HoverAdapter$1(worker)));
    disposables.push(monaco.languages.registerDocumentSymbolProvider(languageId, new DocumentSymbolAdapter$1(worker)));
    disposables.push(monaco.languages.registerDocumentFormattingEditProvider(languageId, new DocumentFormattingEditProvider(worker)));
    disposables.push(monaco.languages.registerDocumentRangeFormattingEditProvider(languageId, new DocumentRangeFormattingEditProvider(worker)));
    disposables.push(new DiagnosticsAdapter$1(languageId, worker, defaults));
    disposables.push(monaco.languages.setTokensProvider(languageId, createTokenizationSupport$1(true)));
    disposables.push(monaco.languages.setLanguageConfiguration(languageId, richEditConfiguration));
    disposables.push(monaco.languages.registerColorProvider(languageId, new DocumentColorAdapter$1(worker)));
    disposables.push(monaco.languages.registerFoldingRangeProvider(languageId, new FoldingRangeAdapter$1(worker)));
}
var richEditConfiguration = {
    wordPattern: /(-?\d*\.\d\w*)|([^\[\{\]\}\:\"\,\s]+)/g,
    comments: {
        lineComment: '//',
        blockComment: ['/*', '*/']
    },
    brackets: [
        ['{', '}'],
        ['[', ']']
    ],
    autoClosingPairs: [
        { open: '{', close: '}', notIn: ['string'] },
        { open: '[', close: ']', notIn: ['string'] },
        { open: '"', close: '"', notIn: ['string'] }
    ]
};

var jsonMode = /*#__PURE__*/Object.freeze({
    setupMode: setupMode$2
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var STOP_WHEN_IDLE_FOR$2 = 2 * 60 * 1000; // 2min
var WorkerManager$4 = /** @class */ (function () {
    function WorkerManager(defaults) {
        var _this = this;
        this._defaults = defaults;
        this._worker = null;
        this._idleCheckInterval = setInterval(function () { return _this._checkIfIdle(); }, 30 * 1000);
        this._lastUsedTime = 0;
        this._configChangeListener = this._defaults.onDidChange(function () { return _this._stopWorker(); });
    }
    WorkerManager.prototype._stopWorker = function () {
        if (this._worker) {
            this._worker.dispose();
            this._worker = null;
        }
        this._client = null;
    };
    WorkerManager.prototype.dispose = function () {
        clearInterval(this._idleCheckInterval);
        this._configChangeListener.dispose();
        this._stopWorker();
    };
    WorkerManager.prototype._checkIfIdle = function () {
        if (!this._worker) {
            return;
        }
        var timePassedSinceLastUsed = Date.now() - this._lastUsedTime;
        if (timePassedSinceLastUsed > STOP_WHEN_IDLE_FOR$2) {
            this._stopWorker();
        }
    };
    WorkerManager.prototype._getClient = function () {
        this._lastUsedTime = Date.now();
        if (!this._client) {
            this._worker = monaco.editor.createWebWorker({
                // module that exports the create() method and returns a `HTMLWorker` instance
                moduleId: 'vs/language/html/htmlWorker',
                // passed in to the create() method
                createData: {
                    languageSettings: this._defaults.options,
                    languageId: this._defaults.languageId
                },
                label: this._defaults.languageId
            });
            this._client = this._worker.getProxy();
        }
        return this._client;
    };
    WorkerManager.prototype.getLanguageServiceWorker = function () {
        var _this = this;
        var resources = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            resources[_i] = arguments[_i];
        }
        var _client;
        return this._getClient().then(function (client) {
            _client = client;
        }).then(function (_) {
            return _this._worker.withSyncedResources(resources);
        }).then(function (_) { return _client; });
    };
    return WorkerManager;
}());

/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
/**
 * The Position namespace provides helper functions to work with
 * [Position](#Position) literals.
 */
var Position$4;
(function (Position) {
    /**
     * Creates a new Position literal from the given line and character.
     * @param line The position's line.
     * @param character The position's character.
     */
    function create(line, character) {
        return { line: line, character: character };
    }
    Position.create = create;
    /**
     * Checks whether the given liternal conforms to the [Position](#Position) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.objectLiteral(candidate) && Is$2.number(candidate.line) && Is$2.number(candidate.character);
    }
    Position.is = is;
})(Position$4 || (Position$4 = {}));
/**
 * The Range namespace provides helper functions to work with
 * [Range](#Range) literals.
 */
var Range$6;
(function (Range) {
    function create(one, two, three, four) {
        if (Is$2.number(one) && Is$2.number(two) && Is$2.number(three) && Is$2.number(four)) {
            return { start: Position$4.create(one, two), end: Position$4.create(three, four) };
        }
        else if (Position$4.is(one) && Position$4.is(two)) {
            return { start: one, end: two };
        }
        else {
            throw new Error("Range#create called with invalid arguments[" + one + ", " + two + ", " + three + ", " + four + "]");
        }
    }
    Range.create = create;
    /**
     * Checks whether the given literal conforms to the [Range](#Range) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.objectLiteral(candidate) && Position$4.is(candidate.start) && Position$4.is(candidate.end);
    }
    Range.is = is;
})(Range$6 || (Range$6 = {}));
/**
 * The Location namespace provides helper functions to work with
 * [Location](#Location) literals.
 */
var Location$2;
(function (Location) {
    /**
     * Creates a Location literal.
     * @param uri The location's uri.
     * @param range The location's range.
     */
    function create(uri, range) {
        return { uri: uri, range: range };
    }
    Location.create = create;
    /**
     * Checks whether the given literal conforms to the [Location](#Location) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Range$6.is(candidate.range) && (Is$2.string(candidate.uri) || Is$2.undefined(candidate.uri));
    }
    Location.is = is;
})(Location$2 || (Location$2 = {}));
/**
 * The Color namespace provides helper functions to work with
 * [Color](#Color) literals.
 */
var Color$3;
(function (Color) {
    /**
     * Creates a new Color literal.
     */
    function create(red, green, blue, alpha) {
        return {
            red: red,
            green: green,
            blue: blue,
            alpha: alpha,
        };
    }
    Color.create = create;
    /**
     * Checks whether the given literal conforms to the [Color](#Color) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.number(candidate.red)
            && Is$2.number(candidate.green)
            && Is$2.number(candidate.blue)
            && Is$2.number(candidate.alpha);
    }
    Color.is = is;
})(Color$3 || (Color$3 = {}));
/**
 * The ColorInformation namespace provides helper functions to work with
 * [ColorInformation](#ColorInformation) literals.
 */
var ColorInformation$2;
(function (ColorInformation) {
    /**
     * Creates a new ColorInformation literal.
     */
    function create(range, color) {
        return {
            range: range,
            color: color,
        };
    }
    ColorInformation.create = create;
    /**
     * Checks whether the given literal conforms to the [ColorInformation](#ColorInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Range$6.is(candidate.range) && Color$3.is(candidate.color);
    }
    ColorInformation.is = is;
})(ColorInformation$2 || (ColorInformation$2 = {}));
/**
 * The Color namespace provides helper functions to work with
 * [ColorPresentation](#ColorPresentation) literals.
 */
var ColorPresentation$2;
(function (ColorPresentation) {
    /**
     * Creates a new ColorInformation literal.
     */
    function create(label, textEdit, additionalTextEdits) {
        return {
            label: label,
            textEdit: textEdit,
            additionalTextEdits: additionalTextEdits,
        };
    }
    ColorPresentation.create = create;
    /**
     * Checks whether the given literal conforms to the [ColorInformation](#ColorInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.string(candidate.label)
            && (Is$2.undefined(candidate.textEdit) || TextEdit$2.is(candidate))
            && (Is$2.undefined(candidate.additionalTextEdits) || Is$2.typedArray(candidate.additionalTextEdits, TextEdit$2.is));
    }
    ColorPresentation.is = is;
})(ColorPresentation$2 || (ColorPresentation$2 = {}));
/**
 * Enum of known range kinds
 */
var FoldingRangeKind$3;
(function (FoldingRangeKind) {
    /**
     * Folding range for a comment
     */
    FoldingRangeKind["Comment"] = "comment";
    /**
     * Folding range for a imports or includes
     */
    FoldingRangeKind["Imports"] = "imports";
    /**
     * Folding range for a region (e.g. `#region`)
     */
    FoldingRangeKind["Region"] = "region";
})(FoldingRangeKind$3 || (FoldingRangeKind$3 = {}));
/**
 * The folding range namespace provides helper functions to work with
 * [FoldingRange](#FoldingRange) literals.
 */
var FoldingRange$2;
(function (FoldingRange) {
    /**
     * Creates a new FoldingRange literal.
     */
    function create(startLine, endLine, startCharacter, endCharacter, kind) {
        var result = {
            startLine: startLine,
            endLine: endLine
        };
        if (Is$2.defined(startCharacter)) {
            result.startCharacter = startCharacter;
        }
        if (Is$2.defined(endCharacter)) {
            result.endCharacter = endCharacter;
        }
        if (Is$2.defined(kind)) {
            result.kind = kind;
        }
        return result;
    }
    FoldingRange.create = create;
    /**
     * Checks whether the given literal conforms to the [FoldingRange](#FoldingRange) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.number(candidate.startLine) && Is$2.number(candidate.startLine)
            && (Is$2.undefined(candidate.startCharacter) || Is$2.number(candidate.startCharacter))
            && (Is$2.undefined(candidate.endCharacter) || Is$2.number(candidate.endCharacter))
            && (Is$2.undefined(candidate.kind) || Is$2.string(candidate.kind));
    }
    FoldingRange.is = is;
})(FoldingRange$2 || (FoldingRange$2 = {}));
/**
 * The DiagnosticRelatedInformation namespace provides helper functions to work with
 * [DiagnosticRelatedInformation](#DiagnosticRelatedInformation) literals.
 */
var DiagnosticRelatedInformation$2;
(function (DiagnosticRelatedInformation) {
    /**
     * Creates a new DiagnosticRelatedInformation literal.
     */
    function create(location, message) {
        return {
            location: location,
            message: message
        };
    }
    DiagnosticRelatedInformation.create = create;
    /**
     * Checks whether the given literal conforms to the [DiagnosticRelatedInformation](#DiagnosticRelatedInformation) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Location$2.is(candidate.location) && Is$2.string(candidate.message);
    }
    DiagnosticRelatedInformation.is = is;
})(DiagnosticRelatedInformation$2 || (DiagnosticRelatedInformation$2 = {}));
/**
 * The diagnostic's severity.
 */
var DiagnosticSeverity$2;
(function (DiagnosticSeverity) {
    /**
     * Reports an error.
     */
    DiagnosticSeverity.Error = 1;
    /**
     * Reports a warning.
     */
    DiagnosticSeverity.Warning = 2;
    /**
     * Reports an information.
     */
    DiagnosticSeverity.Information = 3;
    /**
     * Reports a hint.
     */
    DiagnosticSeverity.Hint = 4;
})(DiagnosticSeverity$2 || (DiagnosticSeverity$2 = {}));
/**
 * The Diagnostic namespace provides helper functions to work with
 * [Diagnostic](#Diagnostic) literals.
 */
var Diagnostic$2;
(function (Diagnostic) {
    /**
     * Creates a new Diagnostic literal.
     */
    function create(range, message, severity, code, source, relatedInformation) {
        var result = { range: range, message: message };
        if (Is$2.defined(severity)) {
            result.severity = severity;
        }
        if (Is$2.defined(code)) {
            result.code = code;
        }
        if (Is$2.defined(source)) {
            result.source = source;
        }
        if (Is$2.defined(relatedInformation)) {
            result.relatedInformation = relatedInformation;
        }
        return result;
    }
    Diagnostic.create = create;
    /**
     * Checks whether the given literal conforms to the [Diagnostic](#Diagnostic) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate)
            && Range$6.is(candidate.range)
            && Is$2.string(candidate.message)
            && (Is$2.number(candidate.severity) || Is$2.undefined(candidate.severity))
            && (Is$2.number(candidate.code) || Is$2.string(candidate.code) || Is$2.undefined(candidate.code))
            && (Is$2.string(candidate.source) || Is$2.undefined(candidate.source))
            && (Is$2.undefined(candidate.relatedInformation) || Is$2.typedArray(candidate.relatedInformation, DiagnosticRelatedInformation$2.is));
    }
    Diagnostic.is = is;
})(Diagnostic$2 || (Diagnostic$2 = {}));
/**
 * The Command namespace provides helper functions to work with
 * [Command](#Command) literals.
 */
var Command$3;
(function (Command) {
    /**
     * Creates a new Command literal.
     */
    function create(title, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var result = { title: title, command: command };
        if (Is$2.defined(args) && args.length > 0) {
            result.arguments = args;
        }
        return result;
    }
    Command.create = create;
    /**
     * Checks whether the given literal conforms to the [Command](#Command) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Is$2.string(candidate.title) && Is$2.string(candidate.command);
    }
    Command.is = is;
})(Command$3 || (Command$3 = {}));
/**
 * The TextEdit namespace provides helper function to create replace,
 * insert and delete edits more easily.
 */
var TextEdit$2;
(function (TextEdit) {
    /**
     * Creates a replace text edit.
     * @param range The range of text to be replaced.
     * @param newText The new text.
     */
    function replace(range, newText) {
        return { range: range, newText: newText };
    }
    TextEdit.replace = replace;
    /**
     * Creates a insert text edit.
     * @param position The position to insert the text at.
     * @param newText The text to be inserted.
     */
    function insert(position, newText) {
        return { range: { start: position, end: position }, newText: newText };
    }
    TextEdit.insert = insert;
    /**
     * Creates a delete text edit.
     * @param range The range of text to be deleted.
     */
    function del(range) {
        return { range: range, newText: '' };
    }
    TextEdit.del = del;
    function is(value) {
        var candidate = value;
        return Is$2.objectLiteral(candidate)
            && Is$2.string(candidate.newText)
            && Range$6.is(candidate.range);
    }
    TextEdit.is = is;
})(TextEdit$2 || (TextEdit$2 = {}));
/**
 * The TextDocumentEdit namespace provides helper function to create
 * an edit that manipulates a text document.
 */
var TextDocumentEdit$2;
(function (TextDocumentEdit) {
    /**
     * Creates a new `TextDocumentEdit`
     */
    function create(textDocument, edits) {
        return { textDocument: textDocument, edits: edits };
    }
    TextDocumentEdit.create = create;
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate)
            && VersionedTextDocumentIdentifier$2.is(candidate.textDocument)
            && Array.isArray(candidate.edits);
    }
    TextDocumentEdit.is = is;
})(TextDocumentEdit$2 || (TextDocumentEdit$2 = {}));
var WorkspaceEdit$2;
(function (WorkspaceEdit) {
    function is(value) {
        var candidate = value;
        return candidate &&
            (candidate.changes !== void 0 || candidate.documentChanges !== void 0) &&
            (candidate.documentChanges === void 0 || Is$2.typedArray(candidate.documentChanges, TextDocumentEdit$2.is));
    }
    WorkspaceEdit.is = is;
})(WorkspaceEdit$2 || (WorkspaceEdit$2 = {}));
var TextEditChangeImpl$2 = /** @class */ (function () {
    function TextEditChangeImpl(edits) {
        this.edits = edits;
    }
    TextEditChangeImpl.prototype.insert = function (position, newText) {
        this.edits.push(TextEdit$2.insert(position, newText));
    };
    TextEditChangeImpl.prototype.replace = function (range, newText) {
        this.edits.push(TextEdit$2.replace(range, newText));
    };
    TextEditChangeImpl.prototype.delete = function (range) {
        this.edits.push(TextEdit$2.del(range));
    };
    TextEditChangeImpl.prototype.add = function (edit) {
        this.edits.push(edit);
    };
    TextEditChangeImpl.prototype.all = function () {
        return this.edits;
    };
    TextEditChangeImpl.prototype.clear = function () {
        this.edits.splice(0, this.edits.length);
    };
    return TextEditChangeImpl;
}());
/**
 * A workspace change helps constructing changes to a workspace.
 */
var WorkspaceChange$2 = /** @class */ (function () {
    function WorkspaceChange(workspaceEdit) {
        var _this = this;
        this._textEditChanges = Object.create(null);
        if (workspaceEdit) {
            this._workspaceEdit = workspaceEdit;
            if (workspaceEdit.documentChanges) {
                workspaceEdit.documentChanges.forEach(function (textDocumentEdit) {
                    var textEditChange = new TextEditChangeImpl$2(textDocumentEdit.edits);
                    _this._textEditChanges[textDocumentEdit.textDocument.uri] = textEditChange;
                });
            }
            else if (workspaceEdit.changes) {
                Object.keys(workspaceEdit.changes).forEach(function (key) {
                    var textEditChange = new TextEditChangeImpl$2(workspaceEdit.changes[key]);
                    _this._textEditChanges[key] = textEditChange;
                });
            }
        }
    }
    Object.defineProperty(WorkspaceChange.prototype, "edit", {
        /**
         * Returns the underlying [WorkspaceEdit](#WorkspaceEdit) literal
         * use to be returned from a workspace edit operation like rename.
         */
        get: function () {
            return this._workspaceEdit;
        },
        enumerable: true,
        configurable: true
    });
    WorkspaceChange.prototype.getTextEditChange = function (key) {
        if (VersionedTextDocumentIdentifier$2.is(key)) {
            if (!this._workspaceEdit) {
                this._workspaceEdit = {
                    documentChanges: []
                };
            }
            if (!this._workspaceEdit.documentChanges) {
                throw new Error('Workspace edit is not configured for versioned document changes.');
            }
            var textDocument = key;
            var result = this._textEditChanges[textDocument.uri];
            if (!result) {
                var edits = [];
                var textDocumentEdit = {
                    textDocument: textDocument,
                    edits: edits
                };
                this._workspaceEdit.documentChanges.push(textDocumentEdit);
                result = new TextEditChangeImpl$2(edits);
                this._textEditChanges[textDocument.uri] = result;
            }
            return result;
        }
        else {
            if (!this._workspaceEdit) {
                this._workspaceEdit = {
                    changes: Object.create(null)
                };
            }
            if (!this._workspaceEdit.changes) {
                throw new Error('Workspace edit is not configured for normal text edit changes.');
            }
            var result = this._textEditChanges[key];
            if (!result) {
                var edits = [];
                this._workspaceEdit.changes[key] = edits;
                result = new TextEditChangeImpl$2(edits);
                this._textEditChanges[key] = result;
            }
            return result;
        }
    };
    return WorkspaceChange;
}());
/**
 * The TextDocumentIdentifier namespace provides helper functions to work with
 * [TextDocumentIdentifier](#TextDocumentIdentifier) literals.
 */
var TextDocumentIdentifier$2;
(function (TextDocumentIdentifier) {
    /**
     * Creates a new TextDocumentIdentifier literal.
     * @param uri The document's uri.
     */
    function create(uri) {
        return { uri: uri };
    }
    TextDocumentIdentifier.create = create;
    /**
     * Checks whether the given literal conforms to the [TextDocumentIdentifier](#TextDocumentIdentifier) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Is$2.string(candidate.uri);
    }
    TextDocumentIdentifier.is = is;
})(TextDocumentIdentifier$2 || (TextDocumentIdentifier$2 = {}));
/**
 * The VersionedTextDocumentIdentifier namespace provides helper functions to work with
 * [VersionedTextDocumentIdentifier](#VersionedTextDocumentIdentifier) literals.
 */
var VersionedTextDocumentIdentifier$2;
(function (VersionedTextDocumentIdentifier) {
    /**
     * Creates a new VersionedTextDocumentIdentifier literal.
     * @param uri The document's uri.
     * @param uri The document's text.
     */
    function create(uri, version) {
        return { uri: uri, version: version };
    }
    VersionedTextDocumentIdentifier.create = create;
    /**
     * Checks whether the given literal conforms to the [VersionedTextDocumentIdentifier](#VersionedTextDocumentIdentifier) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Is$2.string(candidate.uri) && Is$2.number(candidate.version);
    }
    VersionedTextDocumentIdentifier.is = is;
})(VersionedTextDocumentIdentifier$2 || (VersionedTextDocumentIdentifier$2 = {}));
/**
 * The TextDocumentItem namespace provides helper functions to work with
 * [TextDocumentItem](#TextDocumentItem) literals.
 */
var TextDocumentItem$2;
(function (TextDocumentItem) {
    /**
     * Creates a new TextDocumentItem literal.
     * @param uri The document's uri.
     * @param languageId The document's language identifier.
     * @param version The document's version number.
     * @param text The document's text.
     */
    function create(uri, languageId, version, text) {
        return { uri: uri, languageId: languageId, version: version, text: text };
    }
    TextDocumentItem.create = create;
    /**
     * Checks whether the given literal conforms to the [TextDocumentItem](#TextDocumentItem) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Is$2.string(candidate.uri) && Is$2.string(candidate.languageId) && Is$2.number(candidate.version) && Is$2.string(candidate.text);
    }
    TextDocumentItem.is = is;
})(TextDocumentItem$2 || (TextDocumentItem$2 = {}));
/**
 * Describes the content type that a client supports in various
 * result literals like `Hover`, `ParameterInfo` or `CompletionItem`.
 *
 * Please note that `MarkupKinds` must not start with a `$`. This kinds
 * are reserved for internal usage.
 */
var MarkupKind$2;
(function (MarkupKind) {
    /**
     * Plain text is supported as a content format
     */
    MarkupKind.PlainText = 'plaintext';
    /**
     * Markdown is supported as a content format
     */
    MarkupKind.Markdown = 'markdown';
})(MarkupKind$2 || (MarkupKind$2 = {}));
(function (MarkupKind) {
    /**
     * Checks whether the given value is a value of the [MarkupKind](#MarkupKind) type.
     */
    function is(value) {
        var candidate = value;
        return candidate === MarkupKind.PlainText || candidate === MarkupKind.Markdown;
    }
    MarkupKind.is = is;
})(MarkupKind$2 || (MarkupKind$2 = {}));
var MarkupContent$2;
(function (MarkupContent) {
    /**
     * Checks whether the given value conforms to the [MarkupContent](#MarkupContent) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.objectLiteral(value) && MarkupKind$2.is(candidate.kind) && Is$2.string(candidate.value);
    }
    MarkupContent.is = is;
})(MarkupContent$2 || (MarkupContent$2 = {}));
/**
 * The kind of a completion entry.
 */
var CompletionItemKind$3;
(function (CompletionItemKind) {
    CompletionItemKind.Text = 1;
    CompletionItemKind.Method = 2;
    CompletionItemKind.Function = 3;
    CompletionItemKind.Constructor = 4;
    CompletionItemKind.Field = 5;
    CompletionItemKind.Variable = 6;
    CompletionItemKind.Class = 7;
    CompletionItemKind.Interface = 8;
    CompletionItemKind.Module = 9;
    CompletionItemKind.Property = 10;
    CompletionItemKind.Unit = 11;
    CompletionItemKind.Value = 12;
    CompletionItemKind.Enum = 13;
    CompletionItemKind.Keyword = 14;
    CompletionItemKind.Snippet = 15;
    CompletionItemKind.Color = 16;
    CompletionItemKind.File = 17;
    CompletionItemKind.Reference = 18;
    CompletionItemKind.Folder = 19;
    CompletionItemKind.EnumMember = 20;
    CompletionItemKind.Constant = 21;
    CompletionItemKind.Struct = 22;
    CompletionItemKind.Event = 23;
    CompletionItemKind.Operator = 24;
    CompletionItemKind.TypeParameter = 25;
})(CompletionItemKind$3 || (CompletionItemKind$3 = {}));
/**
 * Defines whether the insert text in a completion item should be interpreted as
 * plain text or a snippet.
 */
var InsertTextFormat$2;
(function (InsertTextFormat) {
    /**
     * The primary text to be inserted is treated as a plain string.
     */
    InsertTextFormat.PlainText = 1;
    /**
     * The primary text to be inserted is treated as a snippet.
     *
     * A snippet can define tab stops and placeholders with `$1`, `$2`
     * and `${3:foo}`. `$0` defines the final tab stop, it defaults to
     * the end of the snippet. Placeholders with equal identifiers are linked,
     * that is typing in one will update others too.
     *
     * See also: https://github.com/Microsoft/vscode/blob/master/src/vs/editor/contrib/snippet/common/snippet.md
     */
    InsertTextFormat.Snippet = 2;
})(InsertTextFormat$2 || (InsertTextFormat$2 = {}));
/**
 * The CompletionItem namespace provides functions to deal with
 * completion items.
 */
var CompletionItem$2;
(function (CompletionItem) {
    /**
     * Create a completion item and seed it with a label.
     * @param label The completion item's label
     */
    function create(label) {
        return { label: label };
    }
    CompletionItem.create = create;
})(CompletionItem$2 || (CompletionItem$2 = {}));
/**
 * The CompletionList namespace provides functions to deal with
 * completion lists.
 */
var CompletionList$2;
(function (CompletionList) {
    /**
     * Creates a new completion list.
     *
     * @param items The completion items.
     * @param isIncomplete The list is not complete.
     */
    function create(items, isIncomplete) {
        return { items: items ? items : [], isIncomplete: !!isIncomplete };
    }
    CompletionList.create = create;
})(CompletionList$2 || (CompletionList$2 = {}));
var MarkedString$2;
(function (MarkedString) {
    /**
     * Creates a marked string from plain text.
     *
     * @param plainText The plain text.
     */
    function fromPlainText(plainText) {
        return plainText.replace(/[\\`*_{}[\]()#+\-.!]/g, "\\$&"); // escape markdown syntax tokens: http://daringfireball.net/projects/markdown/syntax#backslash
    }
    MarkedString.fromPlainText = fromPlainText;
    /**
     * Checks whether the given value conforms to the [MarkedString](#MarkedString) type.
     */
    function is(value) {
        var candidate = value;
        return Is$2.string(candidate) || (Is$2.objectLiteral(candidate) && Is$2.string(candidate.language) && Is$2.string(candidate.value));
    }
    MarkedString.is = is;
})(MarkedString$2 || (MarkedString$2 = {}));
var Hover$2;
(function (Hover) {
    /**
     * Checks whether the given value conforms to the [Hover](#Hover) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.objectLiteral(candidate) && (MarkupContent$2.is(candidate.contents) ||
            MarkedString$2.is(candidate.contents) ||
            Is$2.typedArray(candidate.contents, MarkedString$2.is)) && (value.range === void 0 || Range$6.is(value.range));
    }
    Hover.is = is;
})(Hover$2 || (Hover$2 = {}));
/**
 * The ParameterInformation namespace provides helper functions to work with
 * [ParameterInformation](#ParameterInformation) literals.
 */
var ParameterInformation$2;
(function (ParameterInformation) {
    /**
     * Creates a new parameter information literal.
     *
     * @param label A label string.
     * @param documentation A doc string.
     */
    function create(label, documentation) {
        return documentation ? { label: label, documentation: documentation } : { label: label };
    }
    ParameterInformation.create = create;
})(ParameterInformation$2 || (ParameterInformation$2 = {}));
/**
 * The SignatureInformation namespace provides helper functions to work with
 * [SignatureInformation](#SignatureInformation) literals.
 */
var SignatureInformation$2;
(function (SignatureInformation) {
    function create(label, documentation) {
        var parameters = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            parameters[_i - 2] = arguments[_i];
        }
        var result = { label: label };
        if (Is$2.defined(documentation)) {
            result.documentation = documentation;
        }
        if (Is$2.defined(parameters)) {
            result.parameters = parameters;
        }
        else {
            result.parameters = [];
        }
        return result;
    }
    SignatureInformation.create = create;
})(SignatureInformation$2 || (SignatureInformation$2 = {}));
/**
 * A document highlight kind.
 */
var DocumentHighlightKind$4;
(function (DocumentHighlightKind) {
    /**
     * A textual occurrence.
     */
    DocumentHighlightKind.Text = 1;
    /**
     * Read-access of a symbol, like reading a variable.
     */
    DocumentHighlightKind.Read = 2;
    /**
     * Write-access of a symbol, like writing to a variable.
     */
    DocumentHighlightKind.Write = 3;
})(DocumentHighlightKind$4 || (DocumentHighlightKind$4 = {}));
/**
 * DocumentHighlight namespace to provide helper functions to work with
 * [DocumentHighlight](#DocumentHighlight) literals.
 */
var DocumentHighlight$2;
(function (DocumentHighlight) {
    /**
     * Create a DocumentHighlight object.
     * @param range The range the highlight applies to.
     */
    function create(range, kind) {
        var result = { range: range };
        if (Is$2.number(kind)) {
            result.kind = kind;
        }
        return result;
    }
    DocumentHighlight.create = create;
})(DocumentHighlight$2 || (DocumentHighlight$2 = {}));
/**
 * A symbol kind.
 */
var SymbolKind$3;
(function (SymbolKind) {
    SymbolKind.File = 1;
    SymbolKind.Module = 2;
    SymbolKind.Namespace = 3;
    SymbolKind.Package = 4;
    SymbolKind.Class = 5;
    SymbolKind.Method = 6;
    SymbolKind.Property = 7;
    SymbolKind.Field = 8;
    SymbolKind.Constructor = 9;
    SymbolKind.Enum = 10;
    SymbolKind.Interface = 11;
    SymbolKind.Function = 12;
    SymbolKind.Variable = 13;
    SymbolKind.Constant = 14;
    SymbolKind.String = 15;
    SymbolKind.Number = 16;
    SymbolKind.Boolean = 17;
    SymbolKind.Array = 18;
    SymbolKind.Object = 19;
    SymbolKind.Key = 20;
    SymbolKind.Null = 21;
    SymbolKind.EnumMember = 22;
    SymbolKind.Struct = 23;
    SymbolKind.Event = 24;
    SymbolKind.Operator = 25;
    SymbolKind.TypeParameter = 26;
})(SymbolKind$3 || (SymbolKind$3 = {}));
var SymbolInformation$2;
(function (SymbolInformation) {
    /**
     * Creates a new symbol information literal.
     *
     * @param name The name of the symbol.
     * @param kind The kind of the symbol.
     * @param range The range of the location of the symbol.
     * @param uri The resource of the location of symbol, defaults to the current document.
     * @param containerName The name of the symbol containing the symbol.
     */
    function create(name, kind, range, uri, containerName) {
        var result = {
            name: name,
            kind: kind,
            location: { uri: uri, range: range }
        };
        if (containerName) {
            result.containerName = containerName;
        }
        return result;
    }
    SymbolInformation.create = create;
})(SymbolInformation$2 || (SymbolInformation$2 = {}));
/**
 * Represents programming constructs like variables, classes, interfaces etc.
 * that appear in a document. Document symbols can be hierarchical and they
 * have two ranges: one that encloses its definition and one that points to
 * its most interesting range, e.g. the range of an identifier.
 */
var DocumentSymbol$2 = /** @class */ (function () {
    function DocumentSymbol() {
    }
    return DocumentSymbol;
}());
(function (DocumentSymbol) {
    /**
     * Creates a new symbol information literal.
     *
     * @param name The name of the symbol.
     * @param detail The detail of the symbol.
     * @param kind The kind of the symbol.
     * @param range The range of the symbol.
     * @param selectionRange The selectionRange of the symbol.
     * @param children Children of the symbol.
     */
    function create(name, detail, kind, range, selectionRange, children) {
        var result = {
            name: name,
            detail: detail,
            kind: kind,
            range: range,
            selectionRange: selectionRange
        };
        if (children !== void 0) {
            result.children = children;
        }
        return result;
    }
    DocumentSymbol.create = create;
    /**
     * Checks whether the given literal conforms to the [DocumentSymbol](#DocumentSymbol) interface.
     */
    function is(value) {
        var candidate = value;
        return candidate &&
            Is$2.string(candidate.name) && Is$2.string(candidate.detail) && Is$2.number(candidate.kind) &&
            Range$6.is(candidate.range) && Range$6.is(candidate.selectionRange) &&
            (candidate.deprecated === void 0 || Is$2.boolean(candidate.deprecated)) &&
            (candidate.children === void 0 || Array.isArray(candidate.children));
    }
    DocumentSymbol.is = is;
})(DocumentSymbol$2 || (DocumentSymbol$2 = {}));
/**
 * A set of predefined code action kinds
 */
var CodeActionKind$2;
(function (CodeActionKind) {
    /**
     * Base kind for quickfix actions: 'quickfix'
     */
    CodeActionKind.QuickFix = 'quickfix';
    /**
     * Base kind for refactoring actions: 'refactor'
     */
    CodeActionKind.Refactor = 'refactor';
    /**
     * Base kind for refactoring extraction actions: 'refactor.extract'
     *
     * Example extract actions:
     *
     * - Extract method
     * - Extract function
     * - Extract variable
     * - Extract interface from class
     * - ...
     */
    CodeActionKind.RefactorExtract = 'refactor.extract';
    /**
     * Base kind for refactoring inline actions: 'refactor.inline'
     *
     * Example inline actions:
     *
     * - Inline function
     * - Inline variable
     * - Inline constant
     * - ...
     */
    CodeActionKind.RefactorInline = 'refactor.inline';
    /**
     * Base kind for refactoring rewrite actions: 'refactor.rewrite'
     *
     * Example rewrite actions:
     *
     * - Convert JavaScript function to class
     * - Add or remove parameter
     * - Encapsulate field
     * - Make method static
     * - Move method to base class
     * - ...
     */
    CodeActionKind.RefactorRewrite = 'refactor.rewrite';
    /**
     * Base kind for source actions: `source`
     *
     * Source code actions apply to the entire file.
     */
    CodeActionKind.Source = 'source';
    /**
     * Base kind for an organize imports source action: `source.organizeImports`
     */
    CodeActionKind.SourceOrganizeImports = 'source.organizeImports';
})(CodeActionKind$2 || (CodeActionKind$2 = {}));
/**
 * The CodeActionContext namespace provides helper functions to work with
 * [CodeActionContext](#CodeActionContext) literals.
 */
var CodeActionContext$2;
(function (CodeActionContext) {
    /**
     * Creates a new CodeActionContext literal.
     */
    function create(diagnostics, only) {
        var result = { diagnostics: diagnostics };
        if (only !== void 0 && only !== null) {
            result.only = only;
        }
        return result;
    }
    CodeActionContext.create = create;
    /**
     * Checks whether the given literal conforms to the [CodeActionContext](#CodeActionContext) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Is$2.typedArray(candidate.diagnostics, Diagnostic$2.is) && (candidate.only === void 0 || Is$2.typedArray(candidate.only, Is$2.string));
    }
    CodeActionContext.is = is;
})(CodeActionContext$2 || (CodeActionContext$2 = {}));
var CodeAction$2;
(function (CodeAction) {
    function create(title, commandOrEdit, kind) {
        var result = { title: title };
        if (Command$3.is(commandOrEdit)) {
            result.command = commandOrEdit;
        }
        else {
            result.edit = commandOrEdit;
        }
        if (kind !== void null) {
            result.kind = kind;
        }
        return result;
    }
    CodeAction.create = create;
    function is(value) {
        var candidate = value;
        return candidate && Is$2.string(candidate.title) &&
            (candidate.diagnostics === void 0 || Is$2.typedArray(candidate.diagnostics, Diagnostic$2.is)) &&
            (candidate.kind === void 0 || Is$2.string(candidate.kind)) &&
            (candidate.edit !== void 0 || candidate.command !== void 0) &&
            (candidate.command === void 0 || Command$3.is(candidate.command)) &&
            (candidate.edit === void 0 || WorkspaceEdit$2.is(candidate.edit));
    }
    CodeAction.is = is;
})(CodeAction$2 || (CodeAction$2 = {}));
/**
 * The CodeLens namespace provides helper functions to work with
 * [CodeLens](#CodeLens) literals.
 */
var CodeLens$2;
(function (CodeLens) {
    /**
     * Creates a new CodeLens literal.
     */
    function create(range, data) {
        var result = { range: range };
        if (Is$2.defined(data))
            result.data = data;
        return result;
    }
    CodeLens.create = create;
    /**
     * Checks whether the given literal conforms to the [CodeLens](#CodeLens) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Range$6.is(candidate.range) && (Is$2.undefined(candidate.command) || Command$3.is(candidate.command));
    }
    CodeLens.is = is;
})(CodeLens$2 || (CodeLens$2 = {}));
/**
 * The FormattingOptions namespace provides helper functions to work with
 * [FormattingOptions](#FormattingOptions) literals.
 */
var FormattingOptions$2;
(function (FormattingOptions) {
    /**
     * Creates a new FormattingOptions literal.
     */
    function create(tabSize, insertSpaces) {
        return { tabSize: tabSize, insertSpaces: insertSpaces };
    }
    FormattingOptions.create = create;
    /**
     * Checks whether the given literal conforms to the [FormattingOptions](#FormattingOptions) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Is$2.number(candidate.tabSize) && Is$2.boolean(candidate.insertSpaces);
    }
    FormattingOptions.is = is;
})(FormattingOptions$2 || (FormattingOptions$2 = {}));
/**
 * A document link is a range in a text document that links to an internal or external resource, like another
 * text document or a web site.
 */
var DocumentLink$2 = /** @class */ (function () {
    function DocumentLink() {
    }
    return DocumentLink;
}());
/**
 * The DocumentLink namespace provides helper functions to work with
 * [DocumentLink](#DocumentLink) literals.
 */
(function (DocumentLink) {
    /**
     * Creates a new DocumentLink literal.
     */
    function create(range, target, data) {
        return { range: range, target: target, data: data };
    }
    DocumentLink.create = create;
    /**
     * Checks whether the given literal conforms to the [DocumentLink](#DocumentLink) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Range$6.is(candidate.range) && (Is$2.undefined(candidate.target) || Is$2.string(candidate.target));
    }
    DocumentLink.is = is;
})(DocumentLink$2 || (DocumentLink$2 = {}));
var TextDocument$2;
(function (TextDocument) {
    /**
     * Creates a new ITextDocument literal from the given uri and content.
     * @param uri The document's uri.
     * @param languageId  The document's language Id.
     * @param content The document's content.
     */
    function create(uri, languageId, version, content) {
        return new FullTextDocument$2(uri, languageId, version, content);
    }
    TextDocument.create = create;
    /**
     * Checks whether the given literal conforms to the [ITextDocument](#ITextDocument) interface.
     */
    function is(value) {
        var candidate = value;
        return Is$2.defined(candidate) && Is$2.string(candidate.uri) && (Is$2.undefined(candidate.languageId) || Is$2.string(candidate.languageId)) && Is$2.number(candidate.lineCount)
            && Is$2.func(candidate.getText) && Is$2.func(candidate.positionAt) && Is$2.func(candidate.offsetAt) ? true : false;
    }
    TextDocument.is = is;
    function applyEdits(document, edits) {
        var text = document.getText();
        var sortedEdits = mergeSort(edits, function (a, b) {
            var diff = a.range.start.line - b.range.start.line;
            if (diff === 0) {
                return a.range.start.character - b.range.start.character;
            }
            return diff;
        });
        var lastModifiedOffset = text.length;
        for (var i = sortedEdits.length - 1; i >= 0; i--) {
            var e = sortedEdits[i];
            var startOffset = document.offsetAt(e.range.start);
            var endOffset = document.offsetAt(e.range.end);
            if (endOffset <= lastModifiedOffset) {
                text = text.substring(0, startOffset) + e.newText + text.substring(endOffset, text.length);
            }
            else {
                throw new Error('Ovelapping edit');
            }
            lastModifiedOffset = startOffset;
        }
        return text;
    }
    TextDocument.applyEdits = applyEdits;
    function mergeSort(data, compare) {
        if (data.length <= 1) {
            // sorted
            return data;
        }
        var p = (data.length / 2) | 0;
        var left = data.slice(0, p);
        var right = data.slice(p);
        mergeSort(left, compare);
        mergeSort(right, compare);
        var leftIdx = 0;
        var rightIdx = 0;
        var i = 0;
        while (leftIdx < left.length && rightIdx < right.length) {
            var ret = compare(left[leftIdx], right[rightIdx]);
            if (ret <= 0) {
                // smaller_equal -> take left to preserve order
                data[i++] = left[leftIdx++];
            }
            else {
                // greater -> take right
                data[i++] = right[rightIdx++];
            }
        }
        while (leftIdx < left.length) {
            data[i++] = left[leftIdx++];
        }
        while (rightIdx < right.length) {
            data[i++] = right[rightIdx++];
        }
        return data;
    }
})(TextDocument$2 || (TextDocument$2 = {}));
/**
 * Represents reasons why a text document is saved.
 */
var TextDocumentSaveReason$2;
(function (TextDocumentSaveReason) {
    /**
     * Manually triggered, e.g. by the user pressing save, by starting debugging,
     * or by an API call.
     */
    TextDocumentSaveReason.Manual = 1;
    /**
     * Automatic after a delay.
     */
    TextDocumentSaveReason.AfterDelay = 2;
    /**
     * When the editor lost focus.
     */
    TextDocumentSaveReason.FocusOut = 3;
})(TextDocumentSaveReason$2 || (TextDocumentSaveReason$2 = {}));
var FullTextDocument$2 = /** @class */ (function () {
    function FullTextDocument(uri, languageId, version, content) {
        this._uri = uri;
        this._languageId = languageId;
        this._version = version;
        this._content = content;
        this._lineOffsets = null;
    }
    Object.defineProperty(FullTextDocument.prototype, "uri", {
        get: function () {
            return this._uri;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FullTextDocument.prototype, "languageId", {
        get: function () {
            return this._languageId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FullTextDocument.prototype, "version", {
        get: function () {
            return this._version;
        },
        enumerable: true,
        configurable: true
    });
    FullTextDocument.prototype.getText = function (range) {
        if (range) {
            var start = this.offsetAt(range.start);
            var end = this.offsetAt(range.end);
            return this._content.substring(start, end);
        }
        return this._content;
    };
    FullTextDocument.prototype.update = function (event, version) {
        this._content = event.text;
        this._version = version;
        this._lineOffsets = null;
    };
    FullTextDocument.prototype.getLineOffsets = function () {
        if (this._lineOffsets === null) {
            var lineOffsets = [];
            var text = this._content;
            var isLineStart = true;
            for (var i = 0; i < text.length; i++) {
                if (isLineStart) {
                    lineOffsets.push(i);
                    isLineStart = false;
                }
                var ch = text.charAt(i);
                isLineStart = (ch === '\r' || ch === '\n');
                if (ch === '\r' && i + 1 < text.length && text.charAt(i + 1) === '\n') {
                    i++;
                }
            }
            if (isLineStart && text.length > 0) {
                lineOffsets.push(text.length);
            }
            this._lineOffsets = lineOffsets;
        }
        return this._lineOffsets;
    };
    FullTextDocument.prototype.positionAt = function (offset) {
        offset = Math.max(Math.min(offset, this._content.length), 0);
        var lineOffsets = this.getLineOffsets();
        var low = 0, high = lineOffsets.length;
        if (high === 0) {
            return Position$4.create(0, offset);
        }
        while (low < high) {
            var mid = Math.floor((low + high) / 2);
            if (lineOffsets[mid] > offset) {
                high = mid;
            }
            else {
                low = mid + 1;
            }
        }
        // low is the least x for which the line offset is larger than the current offset
        // or array.length if no line offset is larger than the current offset
        var line = low - 1;
        return Position$4.create(line, offset - lineOffsets[line]);
    };
    FullTextDocument.prototype.offsetAt = function (position) {
        var lineOffsets = this.getLineOffsets();
        if (position.line >= lineOffsets.length) {
            return this._content.length;
        }
        else if (position.line < 0) {
            return 0;
        }
        var lineOffset = lineOffsets[position.line];
        var nextLineOffset = (position.line + 1 < lineOffsets.length) ? lineOffsets[position.line + 1] : this._content.length;
        return Math.max(Math.min(lineOffset + position.character, nextLineOffset), lineOffset);
    };
    Object.defineProperty(FullTextDocument.prototype, "lineCount", {
        get: function () {
            return this.getLineOffsets().length;
        },
        enumerable: true,
        configurable: true
    });
    return FullTextDocument;
}());
var Is$2;
(function (Is) {
    var toString = Object.prototype.toString;
    function defined(value) {
        return typeof value !== 'undefined';
    }
    Is.defined = defined;
    function undefined(value) {
        return typeof value === 'undefined';
    }
    Is.undefined = undefined;
    function boolean(value) {
        return value === true || value === false;
    }
    Is.boolean = boolean;
    function string(value) {
        return toString.call(value) === '[object String]';
    }
    Is.string = string;
    function number(value) {
        return toString.call(value) === '[object Number]';
    }
    Is.number = number;
    function func(value) {
        return toString.call(value) === '[object Function]';
    }
    Is.func = func;
    function objectLiteral(value) {
        // Strictly speaking class instances pass this check as well. Since the LSP
        // doesn't use classes we ignore this for now. If we do we need to add something
        // like this: `Object.getPrototypeOf(Object.getPrototypeOf(x)) === null`
        return value !== null && typeof value === 'object';
    }
    Is.objectLiteral = objectLiteral;
    function typedArray(value, check) {
        return Array.isArray(value) && value.every(check);
    }
    Is.typedArray = typedArray;
})(Is$2 || (Is$2 = {}));

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var Range$7 = monaco.Range;
// --- diagnostics --- ---
var DiagnosticsAdapter$2 = /** @class */ (function () {
    function DiagnosticsAdapter(_languageId, _worker, defaults) {
        var _this = this;
        this._languageId = _languageId;
        this._worker = _worker;
        this._disposables = [];
        this._listener = Object.create(null);
        var onModelAdd = function (model) {
            var modeId = model.getModeId();
            if (modeId !== _this._languageId) {
                return;
            }
            var handle;
            _this._listener[model.uri.toString()] = model.onDidChangeContent(function () {
                clearTimeout(handle);
                handle = setTimeout(function () { return _this._doValidate(model.uri, modeId); }, 500);
            });
            _this._doValidate(model.uri, modeId);
        };
        var onModelRemoved = function (model) {
            monaco.editor.setModelMarkers(model, _this._languageId, []);
            var uriStr = model.uri.toString();
            var listener = _this._listener[uriStr];
            if (listener) {
                listener.dispose();
                delete _this._listener[uriStr];
            }
        };
        this._disposables.push(monaco.editor.onDidCreateModel(onModelAdd));
        this._disposables.push(monaco.editor.onWillDisposeModel(function (model) {
            onModelRemoved(model);
        }));
        this._disposables.push(monaco.editor.onDidChangeModelLanguage(function (event) {
            onModelRemoved(event.model);
            onModelAdd(event.model);
        }));
        this._disposables.push(defaults.onDidChange(function (_) {
            monaco.editor.getModels().forEach(function (model) {
                if (model.getModeId() === _this._languageId) {
                    onModelRemoved(model);
                    onModelAdd(model);
                }
            });
        }));
        this._disposables.push({
            dispose: function () {
                for (var key in _this._listener) {
                    _this._listener[key].dispose();
                }
            }
        });
        monaco.editor.getModels().forEach(onModelAdd);
    }
    DiagnosticsAdapter.prototype.dispose = function () {
        this._disposables.forEach(function (d) { return d && d.dispose(); });
        this._disposables = [];
    };
    DiagnosticsAdapter.prototype._doValidate = function (resource, languageId) {
        this._worker(resource).then(function (worker) {
            return worker.doValidation(resource.toString()).then(function (diagnostics) {
                var markers = diagnostics.map(function (d) { return toDiagnostics$2(resource, d); });
                monaco.editor.setModelMarkers(monaco.editor.getModel(resource), languageId, markers);
            });
        }).then(undefined, function (err) {
            console.error(err);
        });
    };
    return DiagnosticsAdapter;
}());
function toSeverity$2(lsSeverity) {
    switch (lsSeverity) {
        case DiagnosticSeverity$2.Error: return monaco.MarkerSeverity.Error;
        case DiagnosticSeverity$2.Warning: return monaco.MarkerSeverity.Warning;
        case DiagnosticSeverity$2.Information: return monaco.MarkerSeverity.Info;
        case DiagnosticSeverity$2.Hint: return monaco.MarkerSeverity.Hint;
        default:
            return monaco.MarkerSeverity.Info;
    }
}
function toDiagnostics$2(resource, diag) {
    var code = typeof diag.code === 'number' ? String(diag.code) : diag.code;
    return {
        severity: toSeverity$2(diag.severity),
        startLineNumber: diag.range.start.line + 1,
        startColumn: diag.range.start.character + 1,
        endLineNumber: diag.range.end.line + 1,
        endColumn: diag.range.end.character + 1,
        message: diag.message,
        code: code,
        source: diag.source
    };
}
// --- completion ------
function fromPosition$2(position) {
    if (!position) {
        return void 0;
    }
    return { character: position.column - 1, line: position.lineNumber - 1 };
}
function fromRange$2(range) {
    if (!range) {
        return void 0;
    }
    return { start: fromPosition$2(range.getStartPosition()), end: fromPosition$2(range.getEndPosition()) };
}
function toRange$2(range) {
    if (!range) {
        return void 0;
    }
    return new Range$7(range.start.line + 1, range.start.character + 1, range.end.line + 1, range.end.character + 1);
}
function toCompletionItemKind$2(kind) {
    var mItemKind = monaco.languages.CompletionItemKind;
    switch (kind) {
        case CompletionItemKind$3.Text: return mItemKind.Text;
        case CompletionItemKind$3.Method: return mItemKind.Method;
        case CompletionItemKind$3.Function: return mItemKind.Function;
        case CompletionItemKind$3.Constructor: return mItemKind.Constructor;
        case CompletionItemKind$3.Field: return mItemKind.Field;
        case CompletionItemKind$3.Variable: return mItemKind.Variable;
        case CompletionItemKind$3.Class: return mItemKind.Class;
        case CompletionItemKind$3.Interface: return mItemKind.Interface;
        case CompletionItemKind$3.Module: return mItemKind.Module;
        case CompletionItemKind$3.Property: return mItemKind.Property;
        case CompletionItemKind$3.Unit: return mItemKind.Unit;
        case CompletionItemKind$3.Value: return mItemKind.Value;
        case CompletionItemKind$3.Enum: return mItemKind.Enum;
        case CompletionItemKind$3.Keyword: return mItemKind.Keyword;
        case CompletionItemKind$3.Snippet: return mItemKind.Snippet;
        case CompletionItemKind$3.Color: return mItemKind.Color;
        case CompletionItemKind$3.File: return mItemKind.File;
        case CompletionItemKind$3.Reference: return mItemKind.Reference;
    }
    return mItemKind.Property;
}
function toTextEdit$2(textEdit) {
    if (!textEdit) {
        return void 0;
    }
    return {
        range: toRange$2(textEdit.range),
        text: textEdit.newText
    };
}
var CompletionAdapter$2 = /** @class */ (function () {
    function CompletionAdapter(_worker) {
        this._worker = _worker;
    }
    Object.defineProperty(CompletionAdapter.prototype, "triggerCharacters", {
        get: function () {
            return ['.', ':', '<', '"', '=', '/'];
        },
        enumerable: true,
        configurable: true
    });
    CompletionAdapter.prototype.provideCompletionItems = function (model, position, context, token) {
        var wordInfo = model.getWordUntilPosition(position);
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.doComplete(resource.toString(), fromPosition$2(position));
        }).then(function (info) {
            if (!info) {
                return;
            }
            var items = info.items.map(function (entry) {
                var item = {
                    label: entry.label,
                    insertText: entry.insertText || entry.label,
                    sortText: entry.sortText,
                    filterText: entry.filterText,
                    documentation: entry.documentation,
                    detail: entry.detail,
                    kind: toCompletionItemKind$2(entry.kind),
                };
                if (entry.textEdit) {
                    item.range = toRange$2(entry.textEdit.range);
                    item.insertText = entry.textEdit.newText;
                }
                if (entry.additionalTextEdits) {
                    item.additionalTextEdits = entry.additionalTextEdits.map(toTextEdit$2);
                }
                if (entry.insertTextFormat === InsertTextFormat$2.Snippet) {
                    item.insertTextRules = monaco.languages.CompletionItemInsertTextRule.InsertAsSnippet;
                }
                return item;
            });
            return {
                isIncomplete: info.isIncomplete,
                suggestions: items
            };
        });
    };
    return CompletionAdapter;
}());
function toHighlighKind(kind) {
    var mKind = monaco.languages.DocumentHighlightKind;
    switch (kind) {
        case DocumentHighlightKind$4.Read: return mKind.Read;
        case DocumentHighlightKind$4.Write: return mKind.Write;
        case DocumentHighlightKind$4.Text: return mKind.Text;
    }
    return mKind.Text;
}
var DocumentHighlightAdapter$1 = /** @class */ (function () {
    function DocumentHighlightAdapter(_worker) {
        this._worker = _worker;
    }
    DocumentHighlightAdapter.prototype.provideDocumentHighlights = function (model, position, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.findDocumentHighlights(resource.toString(), fromPosition$2(position)); }).then(function (items) {
            if (!items) {
                return;
            }
            return items.map(function (item) { return ({
                range: toRange$2(item.range),
                kind: toHighlighKind(item.kind)
            }); });
        });
    };
    return DocumentHighlightAdapter;
}());
var DocumentLinkAdapter = /** @class */ (function () {
    function DocumentLinkAdapter(_worker) {
        this._worker = _worker;
    }
    DocumentLinkAdapter.prototype.provideLinks = function (model, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.findDocumentLinks(resource.toString()); }).then(function (items) {
            if (!items) {
                return;
            }
            return items.map(function (item) { return ({
                range: toRange$2(item.range),
                url: item.target
            }); });
        });
    };
    return DocumentLinkAdapter;
}());
function fromFormattingOptions$1(options) {
    return {
        tabSize: options.tabSize,
        insertSpaces: options.insertSpaces
    };
}
var DocumentFormattingEditProvider$1 = /** @class */ (function () {
    function DocumentFormattingEditProvider(_worker) {
        this._worker = _worker;
    }
    DocumentFormattingEditProvider.prototype.provideDocumentFormattingEdits = function (model, options, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.format(resource.toString(), null, fromFormattingOptions$1(options)).then(function (edits) {
                if (!edits || edits.length === 0) {
                    return;
                }
                return edits.map(toTextEdit$2);
            });
        });
    };
    return DocumentFormattingEditProvider;
}());
var DocumentRangeFormattingEditProvider$1 = /** @class */ (function () {
    function DocumentRangeFormattingEditProvider(_worker) {
        this._worker = _worker;
    }
    DocumentRangeFormattingEditProvider.prototype.provideDocumentRangeFormattingEdits = function (model, range, options, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) {
            return worker.format(resource.toString(), fromRange$2(range), fromFormattingOptions$1(options)).then(function (edits) {
                if (!edits || edits.length === 0) {
                    return;
                }
                return edits.map(toTextEdit$2);
            });
        });
    };
    return DocumentRangeFormattingEditProvider;
}());
var FoldingRangeAdapter$2 = /** @class */ (function () {
    function FoldingRangeAdapter(_worker) {
        this._worker = _worker;
    }
    FoldingRangeAdapter.prototype.provideFoldingRanges = function (model, context, token) {
        var resource = model.uri;
        return this._worker(resource).then(function (worker) { return worker.provideFoldingRanges(resource.toString(), context); }).then(function (ranges) {
            if (!ranges) {
                return;
            }
            return ranges.map(function (range) {
                var result = {
                    start: range.startLine + 1,
                    end: range.endLine + 1
                };
                if (typeof range.kind !== 'undefined') {
                    result.kind = toFoldingRangeKind$2(range.kind);
                }
                return result;
            });
        });
    };
    return FoldingRangeAdapter;
}());
function toFoldingRangeKind$2(kind) {
    switch (kind) {
        case FoldingRangeKind$3.Comment: return monaco.languages.FoldingRangeKind.Comment;
        case FoldingRangeKind$3.Imports: return monaco.languages.FoldingRangeKind.Imports;
        case FoldingRangeKind$3.Region: return monaco.languages.FoldingRangeKind.Region;
    }
    return void 0;
}

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
function setupMode$3(defaults) {
    var client = new WorkerManager$4(defaults);
    var worker = function () {
        var uris = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            uris[_i] = arguments[_i];
        }
        return client.getLanguageServiceWorker.apply(client, uris);
    };
    var languageId = defaults.languageId;
    // all modes
    monaco.languages.registerCompletionItemProvider(languageId, new CompletionAdapter$2(worker));
    monaco.languages.registerDocumentHighlightProvider(languageId, new DocumentHighlightAdapter$1(worker));
    monaco.languages.registerLinkProvider(languageId, new DocumentLinkAdapter(worker));
    monaco.languages.registerFoldingRangeProvider(languageId, new FoldingRangeAdapter$2(worker));
    // only html
    if (languageId === 'html') {
        monaco.languages.registerDocumentFormattingEditProvider(languageId, new DocumentFormattingEditProvider$1(worker));
        monaco.languages.registerDocumentRangeFormattingEditProvider(languageId, new DocumentRangeFormattingEditProvider$1(worker));
        new DiagnosticsAdapter$2(languageId, worker, defaults);
    }
}

var htmlMode = /*#__PURE__*/Object.freeze({
    setupMode: setupMode$3
});

export default monaco$1;
